--
-- PostgreSQL database dump
--

-- Dumped from database version 13.0 (Debian 13.0-1.pgdg100+1)
-- Dumped by pg_dump version 13.0 (Debian 13.0-1.pgdg100+1)

-- Started on 2020-10-12 20:01:18 UTC

SET statement_timeout = 0;
SET lock_timeout = 0;
SET idle_in_transaction_session_timeout = 0;
SET client_encoding = 'UTF8';
SET standard_conforming_strings = on;
SELECT pg_catalog.set_config('search_path', '', false);
SET check_function_bodies = false;
SET xmloption = content;
SET client_min_messages = warning;
SET row_security = off;

--
-- TOC entry 637 (class 1247 OID 16392)
-- Name: trade_env; Type: TYPE; Schema: public; Owner: liu
--

CREATE TYPE public.trade_env AS ENUM (
    'PAPER',
    'PROD'
);


ALTER TYPE public.trade_env OWNER TO liu;

--
-- TOC entry 634 (class 1247 OID 16386)
-- Name: trade_operation; Type: TYPE; Schema: public; Owner: liu
--

CREATE TYPE public.trade_operation AS ENUM (
    'buy',
    'sell',
    'sell_short',
    'buy_short'
);


ALTER TYPE public.trade_operation OWNER TO liu;

SET default_tablespace = '';

SET default_table_access_method = heap;

--
-- TOC entry 202 (class 1259 OID 16401)
-- Name: algo_run; Type: TABLE; Schema: public; Owner: liu
--

CREATE TABLE public.algo_run (
    algo_run_id integer NOT NULL,
    algo_name text NOT NULL,
    algo_env text NOT NULL,
    build_number text NOT NULL,
    parameters jsonb,
    start_time timestamp without time zone DEFAULT CURRENT_TIMESTAMP,
    end_time timestamp without time zone,
    end_reason text,
    batch_id text DEFAULT ''::text NOT NULL,
    ref_algo_run integer
);


ALTER TABLE public.algo_run OWNER TO liu;

--
-- TOC entry 201 (class 1259 OID 16399)
-- Name: algo_run_algo_run_id_seq; Type: SEQUENCE; Schema: public; Owner: liu
--

CREATE SEQUENCE public.algo_run_algo_run_id_seq
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.algo_run_algo_run_id_seq OWNER TO liu;

--
-- TOC entry 3036 (class 0 OID 0)
-- Dependencies: 201
-- Name: algo_run_algo_run_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: liu
--

ALTER SEQUENCE public.algo_run_algo_run_id_seq OWNED BY public.algo_run.algo_run_id;


--
-- TOC entry 206 (class 1259 OID 16434)
-- Name: new_trades; Type: TABLE; Schema: public; Owner: liu
--

CREATE TABLE public.new_trades (
    trade_id integer NOT NULL,
    algo_run_id integer,
    symbol text NOT NULL,
    operation public.trade_operation NOT NULL,
    qty integer NOT NULL,
    price numeric(8,2) NOT NULL,
    indicators jsonb NOT NULL,
    client_time text,
    tstamp timestamp without time zone DEFAULT CURRENT_TIMESTAMP,
    stop_price numeric(8,2),
    target_price numeric(8,2),
    expire_tstamp timestamp without time zone,
    CONSTRAINT new_trades_qty_check CHECK ((qty <> 0))
);


ALTER TABLE public.new_trades OWNER TO liu;

--
-- TOC entry 205 (class 1259 OID 16432)
-- Name: new_trades_trade_id_seq; Type: SEQUENCE; Schema: public; Owner: liu
--

CREATE SEQUENCE public.new_trades_trade_id_seq
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.new_trades_trade_id_seq OWNER TO liu;

--
-- TOC entry 3037 (class 0 OID 0)
-- Dependencies: 205
-- Name: new_trades_trade_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: liu
--

ALTER SEQUENCE public.new_trades_trade_id_seq OWNED BY public.new_trades.trade_id;


--
-- TOC entry 211 (class 1259 OID 16491)
-- Name: stock_ohlc; Type: TABLE; Schema: public; Owner: liu
--

CREATE TABLE public.stock_ohlc (
    symbol_id integer NOT NULL,
    symbol text NOT NULL,
    symbol_date date NOT NULL,
    open double precision NOT NULL,
    high double precision NOT NULL,
    low double precision NOT NULL,
    close double precision NOT NULL,
    volume integer NOT NULL,
    indicators jsonb,
    modify_tstamp timestamp without time zone,
    create_tstamp timestamp without time zone DEFAULT CURRENT_TIMESTAMP
);


ALTER TABLE public.stock_ohlc OWNER TO liu;

--
-- TOC entry 210 (class 1259 OID 16489)
-- Name: stock_ohlc_symbol_id_seq; Type: SEQUENCE; Schema: public; Owner: liu
--

CREATE SEQUENCE public.stock_ohlc_symbol_id_seq
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.stock_ohlc_symbol_id_seq OWNER TO liu;

--
-- TOC entry 3038 (class 0 OID 0)
-- Dependencies: 210
-- Name: stock_ohlc_symbol_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: liu
--

ALTER SEQUENCE public.stock_ohlc_symbol_id_seq OWNED BY public.stock_ohlc.symbol_id;


--
-- TOC entry 207 (class 1259 OID 16455)
-- Name: ticker_data; Type: TABLE; Schema: public; Owner: liu
--

CREATE TABLE public.ticker_data (
    symbol text NOT NULL,
    name text NOT NULL,
    description text NOT NULL,
    tags text[],
    similar_tickers text[],
    industry text,
    sector text,
    exchange text,
    short_ratio double precision,
    create_tstamp timestamp without time zone DEFAULT CURRENT_TIMESTAMP,
    modify_tstamp timestamp without time zone
);


ALTER TABLE public.ticker_data OWNER TO liu;

--
-- TOC entry 204 (class 1259 OID 16413)
-- Name: trades; Type: TABLE; Schema: public; Owner: liu
--

CREATE TABLE public.trades (
    trade_id integer NOT NULL,
    algo_run_id integer,
    is_win boolean,
    symbol text NOT NULL,
    qty integer NOT NULL,
    buy_price numeric(8,2) NOT NULL,
    buy_indicators jsonb NOT NULL,
    buy_time timestamp without time zone DEFAULT CURRENT_TIMESTAMP,
    sell_price numeric(8,2),
    sell_indicators jsonb,
    sell_time timestamp without time zone,
    client_sell_time text,
    client_buy_time text,
    CONSTRAINT trades_qty_check CHECK ((qty > 0))
);


ALTER TABLE public.trades OWNER TO liu;

--
-- TOC entry 203 (class 1259 OID 16411)
-- Name: trades_trade_id_seq; Type: SEQUENCE; Schema: public; Owner: liu
--

CREATE SEQUENCE public.trades_trade_id_seq
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.trades_trade_id_seq OWNER TO liu;

--
-- TOC entry 3039 (class 0 OID 0)
-- Dependencies: 203
-- Name: trades_trade_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: liu
--

ALTER SEQUENCE public.trades_trade_id_seq OWNED BY public.trades.trade_id;


--
-- TOC entry 200 (class 1259 OID 16397)
-- Name: transaction_id_seq; Type: SEQUENCE; Schema: public; Owner: liu
--

CREATE SEQUENCE public.transaction_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.transaction_id_seq OWNER TO liu;

--
-- TOC entry 209 (class 1259 OID 16477)
-- Name: trending_tickers; Type: TABLE; Schema: public; Owner: liu
--

CREATE TABLE public.trending_tickers (
    trending_id integer NOT NULL,
    batch_id text NOT NULL,
    symbol text NOT NULL,
    create_tstamp timestamp without time zone DEFAULT CURRENT_TIMESTAMP
);


ALTER TABLE public.trending_tickers OWNER TO liu;

--
-- TOC entry 208 (class 1259 OID 16475)
-- Name: trending_tickers_trending_id_seq; Type: SEQUENCE; Schema: public; Owner: liu
--

CREATE SEQUENCE public.trending_tickers_trending_id_seq
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.trending_tickers_trending_id_seq OWNER TO liu;

--
-- TOC entry 3040 (class 0 OID 0)
-- Dependencies: 208
-- Name: trending_tickers_trending_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: liu
--

ALTER SEQUENCE public.trending_tickers_trending_id_seq OWNED BY public.trending_tickers.trending_id;


--
-- TOC entry 2845 (class 2604 OID 16404)
-- Name: algo_run algo_run_id; Type: DEFAULT; Schema: public; Owner: liu
--

ALTER TABLE ONLY public.algo_run ALTER COLUMN algo_run_id SET DEFAULT nextval('public.algo_run_algo_run_id_seq'::regclass);


--
-- TOC entry 2851 (class 2604 OID 16437)
-- Name: new_trades trade_id; Type: DEFAULT; Schema: public; Owner: liu
--

ALTER TABLE ONLY public.new_trades ALTER COLUMN trade_id SET DEFAULT nextval('public.new_trades_trade_id_seq'::regclass);


--
-- TOC entry 2857 (class 2604 OID 16494)
-- Name: stock_ohlc symbol_id; Type: DEFAULT; Schema: public; Owner: liu
--

ALTER TABLE ONLY public.stock_ohlc ALTER COLUMN symbol_id SET DEFAULT nextval('public.stock_ohlc_symbol_id_seq'::regclass);


--
-- TOC entry 2848 (class 2604 OID 16416)
-- Name: trades trade_id; Type: DEFAULT; Schema: public; Owner: liu
--

ALTER TABLE ONLY public.trades ALTER COLUMN trade_id SET DEFAULT nextval('public.trades_trade_id_seq'::regclass);


--
-- TOC entry 2855 (class 2604 OID 16480)
-- Name: trending_tickers trending_id; Type: DEFAULT; Schema: public; Owner: liu
--

ALTER TABLE ONLY public.trending_tickers ALTER COLUMN trending_id SET DEFAULT nextval('public.trending_tickers_trending_id_seq'::regclass);


--
-- TOC entry 3021 (class 0 OID 16401)
-- Dependencies: 202
-- Data for Name: algo_run; Type: TABLE DATA; Schema: public; Owner: liu
--

COPY public.algo_run (algo_run_id, algo_name, algo_env, build_number, parameters, start_time, end_time, end_reason, batch_id, ref_algo_run) FROM stdin;
1	momentum_long	BACKTEST	v0.0.55-39-g38dbedc	{"DSN": "postgresql://liu:liu@localhost:5400/liu"}	2020-10-11 16:49:00.671719	\N	\N	dc476def-113d-49b7-a8c4-e9a25416371b	\N
2	vwap_short	BACKTEST	v0.0.55-39-g38dbedc	{"DSN": "postgresql://liu:liu@localhost:5400/liu"}	2020-10-11 16:49:01.069344	\N	\N	dc476def-113d-49b7-a8c4-e9a25416371b	\N
3	momentum_long	BACKTEST	v0.0.55-40-g40f81dd	{"DSN": "postgresql://liu:liu@localhost:5400/liu"}	2020-10-11 17:15:04.221782	\N	\N	964e2134-47c3-4fe3-9f58-6e8fdeec094f	\N
4	vwap_short	BACKTEST	v0.0.55-40-g40f81dd	{"DSN": "postgresql://liu:liu@localhost:5400/liu"}	2020-10-11 17:15:04.515981	\N	\N	964e2134-47c3-4fe3-9f58-6e8fdeec094f	\N
5	momentum_long	BACKTEST	v0.0.55-40-g40f81dd	{"DSN": "postgresql://liu:liu@localhost:5400/liu"}	2020-10-11 17:43:45.258109	\N	\N	8d6a9d2e-c9da-44ba-852c-c221fb6f854e	\N
6	vwap_short	BACKTEST	v0.0.55-40-g40f81dd	{"DSN": "postgresql://liu:liu@localhost:5400/liu"}	2020-10-11 17:43:45.528071	\N	\N	8d6a9d2e-c9da-44ba-852c-c221fb6f854e	\N
7	momentum_long	BACKTEST	v0.0.55-40-g40f81dd	{"DSN": "postgresql://liu:liu@localhost:5400/liu"}	2020-10-11 22:47:25.6939	\N	\N	a5f36d9c-7a84-48c7-aa2f-df092dcaea88	\N
8	vwap_short	BACKTEST	v0.0.55-40-g40f81dd	{"DSN": "postgresql://liu:liu@localhost:5400/liu"}	2020-10-11 22:47:26.023623	\N	\N	a5f36d9c-7a84-48c7-aa2f-df092dcaea88	\N
9	momentum_long	BACKTEST	v0.0.55-45-g4de0413	{"DSN": "postgresql://liu:liu@localhost:5400/liu"}	2020-10-12 18:12:20.041936	\N	\N	9111c17f-6200-4aeb-a5eb-779999956721	\N
10	vwap_short	BACKTEST	v0.0.55-45-g4de0413	{"DSN": "postgresql://liu:liu@localhost:5400/liu"}	2020-10-12 18:12:20.365301	\N	\N	9111c17f-6200-4aeb-a5eb-779999956721	\N
11	momentum_long	BACKTEST	v0.0.55-46-ge40bf14	{"DSN": "postgresql://liu:liu@localhost:5400/liu"}	2020-10-12 19:44:33.240149	\N	\N	2398380c-5146-4b58-843a-a50c458c8071	\N
12	vwap_short	BACKTEST	v0.0.55-46-ge40bf14	{"DSN": "postgresql://liu:liu@localhost:5400/liu"}	2020-10-12 19:44:33.515653	\N	\N	2398380c-5146-4b58-843a-a50c458c8071	\N
\.


--
-- TOC entry 3025 (class 0 OID 16434)
-- Dependencies: 206
-- Data for Name: new_trades; Type: TABLE DATA; Schema: public; Owner: liu
--

COPY public.new_trades (trade_id, algo_run_id, symbol, operation, qty, price, indicators, client_time, tstamp, stop_price, target_price, expire_tstamp) FROM stdin;
1	7	DDD	buy	425	5.69	{"avg": 5.615625766336536, "macd": [0.017562152156591182, 0.01869661730334382, 0.020058416295205106, 0.02155089794073639, 0.02284157845571677], "vwap": 5.692166666666668, "macd_signal": [0.01935767517624882, 0.019225463601667818, 0.019392054140375276, 0.0198238229004475, 0.020427374011501354]}	2020-10-07 10:13:00-04:00	2020-10-11 22:48:04.940012	5.46	6.40	\N
2	7	FBP	buy	455	6.23	{"avg": 6.126580476361969, "macd": [0.008650413842197224, 0.008449122523763286, 0.00939171055035981, 0.011219758599113838, 0.01451843147378895], "vwap": 6.226666666666667, "macd_signal": [0.012498542232139724, 0.011688658290464437, 0.011229268742443512, 0.011227366713777578, 0.011885579665779852]}	2020-10-07 10:19:00-04:00	2020-10-11 22:48:08.532155	6.01	6.89	\N
3	7	ASB	buy	243	14.05	{"avg": 13.88698810673029, "macd": [0.02215774152331207, 0.0220592481313453, 0.02292727469739475, 0.024941514603018078, 0.029825132596277726], "vwap": 14.043333333333331, "macd_signal": [0.02375835807470555, 0.0234185360860335, 0.02332028380830575, 0.023644529967248216, 0.02488065049305412]}	2020-10-07 10:24:00-04:00	2020-10-11 22:48:10.870144	13.64	15.28	\N
4	7	ACBI	buy	215	13.18	{"avg": 13.026450379720853, "macd": [0.060625441208900455, 0.062063059458862924, 0.06276930669790559, 0.06819136015644034, 0.07325773981590089], "vwap": 13.176666666666668, "macd_signal": [0.05487995732194116, 0.05631657774932551, 0.05760712353904153, 0.05972397086252129, 0.06243072465319721]}	2020-10-07 10:25:00-04:00	2020-10-11 22:48:11.583239	12.72	14.57	\N
5	7	AGLE	buy	265	7.82	{"avg": 7.66207930052387, "macd": [0.032000117529672245, 0.03468928591624021, 0.038794027120346186, 0.04236561607375755, 0.05033691093955017], "vwap": 7.808633333333333, "macd_signal": [0.03727346101365987, 0.03675662599417594, 0.037164106219409986, 0.038204408190279496, 0.04063090874013363]}	2020-10-07 10:26:00-04:00	2020-10-11 22:48:12.290303	7.44	8.95	\N
6	7	TGNA	buy	264	12.17	{"avg": 12.07185853064421, "macd": [0.018006928861892035, 0.01855185655103675, 0.021559399822578484, 0.023271185578655817, 0.025144850366709548], "vwap": 12.168333333333331, "macd_signal": [0.01617461032101155, 0.016650059567016588, 0.017631927618128967, 0.018759779210234338, 0.02003679344152938]}	2020-10-07 10:26:00-04:00	2020-10-11 22:48:12.698968	11.79	13.31	\N
7	7	FLY	buy	269	7.46	{"avg": 7.387833203668563, "macd": [0.003120733069985171, 0.005382460612210593, 0.008289711984470038, 0.011270724176588587, 0.017618004678372934], "vwap": 7.4546, "macd_signal": [0.022081536947424967, 0.018741721680382094, 0.016651319741199683, 0.015575200628277464, 0.015983761438296558]}	2020-10-07 10:29:00-04:00	2020-10-11 22:48:14.159559	7.09	8.57	\N
8	7	DDD	sell	425	5.70	{"avg": 5.612834991724703, "rsi": [61.182699916722264, 68.32323403287627, 86.1532416118605], "vwap": 5.663333333333334, "reasons": "rsi max, cool-down for 5 minutes", "movement": 0.0015110517623081114, "sell_macd": [-0.0031351503477550224, -0.0027230143020195996, -0.0010678042105114471, 0.0012747985128784833, 0.007932132256045499], "sell_macd_signal": [-0.0031165735755675434, -0.0030378617208579547, -0.002643850218788653, -0.0018601204724552258, 0.00009833007324491902]}	2020-10-07 11:31:00-04:00	2020-10-11 22:48:20.958755	5.46	6.40	\N
9	7	FBP	sell	455	6.23	{"avg": 6.152575242343232, "rsi": [73.74376595419874, 73.74376595419874, 79.13610424913038], "vwap": 6.2250000000000005, "reasons": "rsi max, cool-down for 5 minutes", "movement": 0.0, "sell_macd": [0.006451527750680697, 0.006514109051350303, 0.00647827488237418, 0.0063662187132713655, 0.006455964312499951], "sell_macd_signal": [0.005460660230834196, 0.005671349994937418, 0.00583273497242477, 0.005939431720594089, 0.006042738238975261]}	2020-10-07 15:04:00-04:00	2020-10-11 22:48:42.75749	6.01	6.89	\N
10	7	ASB	sell	243	13.93	{"liquidate": 1}	2020-10-07 16:00:00-04:00	2020-10-11 22:48:48.393143	\N	\N	\N
11	7	ACBI	sell	215	13.00	{"liquidate": 1}	2020-10-07 16:00:00-04:00	2020-10-11 22:48:48.524192	\N	\N	\N
12	7	AGLE	sell	265	7.84	{"liquidate": 1}	2020-10-07 16:00:00-04:00	2020-10-11 22:48:48.642911	\N	\N	\N
13	7	FLY	sell	269	7.28	{"liquidate": 1}	2020-10-07 16:00:00-04:00	2020-10-11 22:48:48.82891	\N	\N	\N
14	7	TGNA	sell	264	12.13	{"liquidate": 1}	2020-10-07 16:00:00-04:00	2020-10-11 22:48:48.98169	\N	\N	\N
15	9	MBWM	buy	109	18.28	{"avg": 18.227523949889463, "macd": [-0.01900254431929227, -0.007074915486352751, 0.013518809844139668, 0.02870171362337004, 0.04425866999030603], "vwap": 18.276666666666667, "macd_signal": [-0.030452457820819012, -0.02577694935392576, -0.017917797514312675, -0.008593895286776131, 0.0019766177686403024]}	2020-10-01 09:46:00-04:00	2020-10-12 18:18:06.451289	17.36	21.03	\N
16	9	AGLE	buy	274	7.21	{"avg": 7.133804952073494, "macd": [0.012057366240095568, 0.01517833776819888, 0.01585512927576449, 0.02099101858481678, 0.026371091193507112], "vwap": 7.206666666666667, "macd_signal": [0.008339616552454198, 0.009707360795603134, 0.010936914491635405, 0.01294773531027168, 0.015632406486918767]}	2020-10-01 09:47:00-04:00	2020-10-12 18:18:10.345636	6.85	8.30	\N
17	9	VECO	buy	258	11.88	{"avg": 11.781087517389818, "macd": [0.023337244035138482, 0.024040509707985436, 0.02431753540941095, 0.026052327222101823, 0.031701496565696985], "vwap": 11.873333333333335, "macd_signal": [0.02326799284881777, 0.023422496220651304, 0.023601504058403232, 0.024091668691142952, 0.02561363426605376]}	2020-10-01 09:47:00-04:00	2020-10-12 18:18:15.668564	11.49	13.04	\N
18	9	AGTC	buy	357	4.98	{"avg": 4.855714638604812, "macd": [0.0007414437299662069, 0.007755607942219633, 0.012364931949629288, 0.015835314782219, 0.022362420251829995], "vwap": 4.976666666666667, "macd_signal": [-0.0012790498127005201, 0.0005278817382835107, 0.0028952917805526663, 0.005483296380885933, 0.008859121155074746]}	2020-10-01 09:48:00-04:00	2020-10-12 18:18:15.938758	4.70	5.82	\N
19	9	GNK	buy	473	7.07	{"avg": 7.0124081951982795, "macd": [0.011462879071444831, 0.014166574235443541, 0.016921132278628903, 0.018886428383361853, 0.025795005919738934], "vwap": 7.0566666666666675, "macd_signal": [0.005987185024744446, 0.007623062866884266, 0.009482676749233193, 0.011363427076058925, 0.014249742844794928]}	2020-10-01 09:48:00-04:00	2020-10-12 18:18:17.930093	6.86	7.70	\N
20	9	ETH	buy	135	14.04	{"avg": 13.744914727553015, "macd": [0.06681055742693687, 0.060437587333943554, 0.06512614253363402, 0.07124821692047156, 0.08879402201647935], "vwap": 14.003333333333332, "macd_signal": [0.047665176565224635, 0.05021965871896842, 0.05320095548190154, 0.056810407769615545, 0.0632071306189883]}	2020-10-01 09:49:00-04:00	2020-10-12 18:18:23.464427	13.30	16.25	\N
21	9	ARDX	buy	420	5.41	{"avg": 5.344627072199096, "macd": [0.01598339852210895, 0.020629613714255868, 0.024034715489137604, 0.02563090958069214, 0.028982561252642647], "vwap": 5.403333333333333, "macd_signal": [0.020619218112396948, 0.02062129723276873, 0.021303980884042506, 0.022169366623372433, 0.023532005549226475]}	2020-10-01 09:53:00-04:00	2020-10-12 18:18:44.90623	5.17	6.12	\N
22	9	BLDP	buy	148	15.62	{"avg": 15.445073180487048, "macd": [0.056915686918374675, 0.06421146390667687, 0.07597640070484246, 0.08392925545320828, 0.09051192791471152], "vwap": 15.620466666666667, "macd_signal": [0.06176426989360948, 0.06225370869622296, 0.06499824709794685, 0.06878444876899914, 0.07312994459814162]}	2020-10-01 09:54:00-04:00	2020-10-12 18:18:51.113354	14.95	17.64	\N
23	9	CCCL	buy	406	2.42	{"avg": 2.330856770746662, "macd": [0.04269764769673268, 0.04089746660171256, 0.04340048749191938, 0.04526581169374033, 0.05068661200350055], "vwap": 2.3987, "macd_signal": [0.03801005291162056, 0.03858753564963896, 0.039550126018095044, 0.0406932631532241, 0.04269193292327939]}	2020-10-01 09:55:00-04:00	2020-10-12 18:18:57.046556	2.17	3.15	\N
24	9	EVGN	buy	409	4.22	{"avg": 4.131378077591527, "macd": [0.024364228891769812, 0.024436367318810603, 0.02461326930787333, 0.025412685176136485, 0.029594419144991058], "vwap": 4.206466666666667, "macd_signal": [0.026493529126306502, 0.02608209676480732, 0.025788331273420523, 0.025713202053963716, 0.026489445472169186]}	2020-10-01 09:55:00-04:00	2020-10-12 18:18:57.83096	3.98	4.95	\N
25	9	INO	buy	159	11.98	{"avg": 11.735804572106847, "macd": [0.036118360602792166, 0.03581959305856941, 0.036948255603025615, 0.043171016791953676, 0.05694359587966602], "vwap": 11.9433, "macd_signal": [0.028844541761627842, 0.030239552021016157, 0.03158129273741805, 0.033899237548325176, 0.03850810921459334]}	2020-10-01 09:55:00-04:00	2020-10-12 18:18:58.537031	11.35	13.87	\N
26	9	CGEN	buy	155	16.71	{"avg": 16.512347670503736, "macd": [0.04442684909117389, 0.04312129277732879, 0.05197738064793711, 0.06151445939957512, 0.0698809517616894], "vwap": 16.713333333333335, "macd_signal": [0.04246130482453998, 0.04259330241509775, 0.04447011806166562, 0.047878986329247517, 0.05227937941573589]}	2020-10-01 09:56:00-04:00	2020-10-12 18:19:02.562436	16.07	18.64	\N
27	9	MGI	buy	565	2.85	{"avg": 2.7904824429505832, "macd": [0.006537022141604343, 0.008575729639916041, 0.010067301273751106, 0.011918905245858369, 0.014031484936302796], "vwap": 2.8467000000000002, "macd_signal": [0.0010685550084106907, 0.0025699899347117606, 0.00406945220251963, 0.005639342811187378, 0.007317771236210462]}	2020-10-01 09:56:00-04:00	2020-10-12 18:19:04.265551	2.67	3.38	\N
28	9	ANAB	buy	151	15.16	{"avg": 14.962721743938074, "macd": [0.05057742002812127, 0.05108961814029023, 0.053301857480134274, 0.05682082724804438, 0.06411550923432152], "vwap": 15.159999999999998, "macd_signal": [0.050059071302750946, 0.0502651806702588, 0.050872516032233896, 0.05206217827539599, 0.0544728444671811]}	2020-10-01 09:57:00-04:00	2020-10-12 18:19:07.462845	14.51	17.14	\N
29	9	ASMB	buy	133	16.97	{"avg": 16.495381653484, "macd": [0.05240410928978534, 0.0580797395520527, 0.06585318503329418, 0.07830869719058242, 0.09521590817203673], "vwap": 16.936666666666667, "macd_signal": [0.04610434725330307, 0.048499425713053, 0.051970177577101234, 0.05723788149979747, 0.06483348683424532]}	2020-10-01 09:57:00-04:00	2020-10-12 18:19:07.690196	16.22	19.22	\N
30	9	KIRK	buy	179	8.88	{"avg": 8.672310715840416, "macd": [0.06633660866702229, 0.06539339093507301, 0.06710006204610863, 0.07963833642602403, 0.0925428153683967], "vwap": 8.858333333333334, "macd_signal": [0.0816231583051551, 0.07837720483113868, 0.07612177627413266, 0.07682508830451093, 0.07996863371728809]}	2020-10-01 09:58:00-04:00	2020-10-12 18:19:16.236734	8.32	10.55	\N
31	9	OPK	buy	678	3.79	{"avg": 3.750332123100643, "macd": [0.007622983498203784, 0.009861357125588999, 0.01230040931655374, 0.013672311639200796, 0.015516710192394534], "vwap": 3.790533333333333, "macd_signal": [0.00993141799735238, 0.009917405822999704, 0.01039400652171051, 0.011049667545208568, 0.011943076074645762]}	2020-10-01 09:59:00-04:00	2020-10-12 18:19:22.025279	3.64	4.23	\N
32	9	QTNT	buy	305	5.39	{"avg": 5.253003163126377, "macd": [0.022352248627350413, 0.021652315625478202, 0.023250347425999962, 0.026630567811867678, 0.03376173115906411], "vwap": 5.37, "macd_signal": [0.0251764311585102, 0.0244716080519038, 0.02422735592672303, 0.02470799830375196, 0.02651874487481439]}	2020-10-01 09:59:00-04:00	2020-10-12 18:19:22.52116	5.06	6.37	\N
33	9	AGLE	sell	274	7.34	{"avg": 7.219157758706618, "rsi": [74.53281857258753, 76.80628260035778, 79.2052623386686], "vwap": 7.323333333333333, "reasons": "rsi max, cool-down for 5 minutes", "movement": 0.018723994452149763, "sell_macd": [0.02886157081025953, 0.031263081925577474, 0.03298811549692182, 0.03520258171489221, 0.03690386385381039], "sell_macd_signal": [0.018806417732235927, 0.021297750570904238, 0.023635823556107754, 0.025949175187864645, 0.028140112921053794]}	2020-10-01 10:01:00-04:00	2020-10-12 18:19:29.172308	6.85	8.30	\N
34	9	TBPH	buy	150	15.31	{"avg": 14.977423058608597, "macd": [0.07787958261303807, 0.07680583203842062, 0.0822687815080112, 0.08640905494990747, 0.09425218593771945], "vwap": 15.301666666666668, "macd_signal": [0.06501865471211402, 0.06737609017737534, 0.07035462844350251, 0.07356551374478351, 0.0777028481833707]}	2020-10-01 10:02:00-04:00	2020-10-12 18:19:38.043526	14.64	17.31	\N
35	9	ANAB	sell	151	15.36	{"avg": 14.97753223992502, "rsi": [75.5695526757776, 77.95315304749421, 79.3349031595894], "vwap": 15.333333333333334, "reasons": "rsi max, cool-down for 5 minutes", "movement": 0.012858555885262137, "sell_macd": [0.038533656549761375, 0.042433739529712966, 0.049597002377341326, 0.057391791578384144, 0.06315320578380579], "sell_macd_signal": [0.03272980001447593, 0.03467058791752334, 0.03765587080948694, 0.04160305496326638, 0.045913085127374256]}	2020-10-01 10:03:00-04:00	2020-10-12 18:19:40.387025	14.51	17.14	\N
36	9	CTMX	buy	375	6.82	{"avg": 6.73431499276458, "macd": [0.026314794539278985, 0.026513948901426332, 0.025570107459788716, 0.026134674906256095, 0.028672331168801968], "vwap": 6.815, "macd_signal": [0.022890278142309824, 0.023615012294133125, 0.024006031327264244, 0.024431760043062613, 0.025279874268210482]}	2020-10-01 10:03:00-04:00	2020-10-12 18:19:41.513766	6.55	7.62	\N
37	9	VVPR	buy	130	10.06	{"avg": 9.825896675550432, "macd": [0.08894648926850124, 0.091780991776087, 0.09122476448970929, 0.09788612756175219, 0.11060501740381845], "vwap": 10.061666666666667, "macd_signal": [0.09043850790918484, 0.09070700468256528, 0.09081055664399408, 0.09222567082754571, 0.09590154014280025]}	2020-10-01 10:03:00-04:00	2020-10-12 18:19:44.394964	9.30	12.36	\N
38	9	VVPR	sell	130	10.24	{"avg": 9.87138565486752, "rsi": [69.95735082175429, 74.42149320055688, 79.69544475533445], "vwap": 10.1783, "reasons": "rsi max, cool-down for 5 minutes", "movement": 0.017416790859413812, "sell_macd": [0.05216037608961521, 0.05182174260248473, 0.05618356400728608, 0.06446311881612843, 0.07596325089034117], "sell_macd_signal": [0.05137947175568494, 0.0514679259250449, 0.052411053541493134, 0.05482146659642019, 0.05904982345520439]}	2020-10-01 10:04:00-04:00	2020-10-12 18:19:48.7788	9.30	12.36	\N
39	9	CBAY	buy	271	7.41	{"avg": 7.244957561308057, "macd": [0.019334535153858745, 0.02409297559610213, 0.028344255422975806, 0.033346320752600356, 0.04365795232706304], "vwap": 7.388433333333334, "macd_signal": [0.0064480329854720245, 0.009977021507598046, 0.013650468290673599, 0.01758963878305895, 0.022803301491859766]}	2020-10-01 10:06:00-04:00	2020-10-12 18:19:54.106731	7.04	8.51	\N
40	9	CCCL	sell	406	2.42	{"avg": 2.3703361691407827, "rsi": [56.87659839575572, 55.290827335934466, 60.28700854530699], "vwap": 2.4041333333333332, "reasons": "bail post whipsawed", "movement": 0.002400562890608752, "sell_macd": [0.0345203125604554, 0.032420682144757684, 0.029324597637139238, 0.02601174480474633, 0.024346165825331223], "sell_macd_signal": [0.03340799785727032, 0.03321053471476779, 0.032433347299242084, 0.031149026800342934, 0.029788454605340593]}	2020-10-01 10:06:00-04:00	2020-10-12 18:19:54.338716	2.17	3.15	\N
41	9	SYRS	buy	323	9.07	{"avg": 8.948097851337565, "macd": [0.026841082850022957, 0.028016823956113157, 0.02756571714229672, 0.02878076773383853, 0.0333295308157151], "vwap": 9.052133333333332, "macd_signal": [0.025488997864112697, 0.02599456308251279, 0.026308793894469575, 0.026803188662343366, 0.028108457093017715]}	2020-10-01 10:07:00-04:00	2020-10-12 18:20:01.116261	8.76	10.00	\N
42	9	BLDP	sell	148	15.87	{"avg": 15.563174341468862, "rsi": [74.26896367241903, 74.26896367241902, 79.71899676528392], "vwap": 15.8266, "reasons": "rsi max, cool-down for 5 minutes", "movement": 0.015907665126045037, "sell_macd": [0.06575011875989567, 0.06472619083477937, 0.06412668607709371, 0.06282672540776524, 0.06393809728351307], "sell_macd_signal": [0.059524906065063124, 0.06056516301900637, 0.06127746763062384, 0.06158731918605212, 0.06205747480554431]}	2020-10-01 10:08:00-04:00	2020-10-12 18:20:01.886089	14.95	17.64	\N
43	9	VCEL	buy	166	18.71	{"avg": 18.55873378123425, "macd": [0.009564904138880337, 0.01563815542910163, 0.02221249013877369, 0.027907909183230828, 0.034189984968111986], "vwap": 18.7012, "macd_signal": [0.0029504006934698854, 0.0054879516405962345, 0.008832859340231727, 0.012647869308831546, 0.016956292440687633]}	2020-10-01 10:08:00-04:00	2020-10-12 18:20:04.710679	18.11	20.52	\N
44	9	ARAY	buy	1176	2.47	{"avg": 2.4151319172910033, "macd": [0.006943683954747648, 0.007166140473674254, 0.008056485148347114, 0.008422920604021389, 0.009651063247594571], "vwap": 2.466666666666667, "macd_signal": [0.007485847655672173, 0.0074219062192725895, 0.007548822005087495, 0.007723641724874273, 0.008109126029418334]}	2020-10-01 10:12:00-04:00	2020-10-12 18:20:17.641309	2.39	2.73	\N
45	9	CCCL	buy	331	2.61	{"avg": 2.395773126211695, "macd": [0.04604491590965187, 0.04313019389274775, 0.043944811185569144, 0.04559791805389857, 0.056424734910237806], "vwap": 2.5852, "macd_signal": [0.049017783846918773, 0.04784026585608457, 0.04706117492198149, 0.0467685235483649, 0.04869976582073948]}	2020-10-01 10:12:00-04:00	2020-10-12 18:20:18.143001	2.31	3.52	\N
46	9	IVC	buy	229	7.78	{"avg": 7.650839201497038, "macd": [0.021372928370743338, 0.0222926309064837, 0.02834319407942587, 0.03196294364136687, 0.04081644607717383], "vwap": 7.773333333333333, "macd_signal": [0.015086756696199768, 0.016527931538256553, 0.018890984046490415, 0.021505375965465708, 0.025367589987807332]}	2020-10-01 10:12:00-04:00	2020-10-12 18:20:19.231846	7.34	9.09	\N
47	9	ADES	buy	125	6.11	{"avg": 5.76587695821305, "macd": [0.15391715043184195, 0.15931642182807693, 0.1696444387701055, 0.18020633878143766, 0.1868744218241467], "vwap": 6.118333333333333, "macd_signal": [0.15879015935052093, 0.15889541184603212, 0.1610452172308468, 0.16487744154096498, 0.16927683759760132]}	2020-10-01 10:13:00-04:00	2020-10-12 18:20:21.219715	5.31	8.50	\N
48	9	NEXA	buy	308	5.48	{"avg": 5.298357117971114, "macd": [0.0030603524818735295, 0.0015528890997549283, -0.004432193482815094, -0.0034868058368759947, 0.012450313325462758], "vwap": 5.426666666666667, "macd_signal": [0.005160114557061174, 0.004438669465599924, 0.0026644968759169203, 0.0014342363333583372, 0.0036374517317792213]}	2020-10-01 10:14:00-04:00	2020-10-12 18:20:26.932377	5.16	6.45	\N
49	9	VRA	buy	348	6.36	{"avg": 6.265106301494539, "macd": [-0.0012208403057707429, -0.0003368857790775337, 0.0027526727336191215, 0.005532780710990615, 0.012442178348977428], "vwap": 6.345, "macd_signal": [0.012835875570807861, 0.010201323300830782, 0.00871159318738845, 0.008075830692108883, 0.008949100223482592]}	2020-10-01 10:14:00-04:00	2020-10-12 18:20:28.089301	6.07	7.22	\N
50	9	KSS	buy	148	19.12	{"avg": 18.8835409578788, "macd": [0.009597798468174545, 0.012349559125869547, 0.015146527628328244, 0.022757297507709495, 0.032449412968674096], "vwap": 19.11, "macd_signal": [0.022581936046936858, 0.020535460662723395, 0.019457674055844365, 0.02011759874621739, 0.02258396159070873]}	2020-10-01 10:15:00-04:00	2020-10-12 18:20:30.807725	18.45	21.14	\N
51	9	SCWX	buy	219	11.90	{"avg": 11.684464888312435, "macd": [0.07691427880516599, 0.0721641214142359, 0.06722124458186407, 0.06936319299431482, 0.08022240393860791], "vwap": 11.87, "macd_signal": [0.058184532803723626, 0.06098045052582608, 0.06222860933703368, 0.06365552606848991, 0.06696890164251351]}	2020-10-01 10:15:00-04:00	2020-10-12 18:20:31.805017	11.45	13.27	\N
52	9	JWN	buy	185	12.44	{"avg": 12.200148222971492, "macd": [0.005152556872983993, 0.00944479794254427, 0.01389661304897949, 0.01960334439391964, 0.029443053061392277], "vwap": 12.416633333333332, "macd_signal": [0.01767996830507853, 0.016032934232571676, 0.01560566999585324, 0.01640520487546652, 0.01901277451265167]}	2020-10-01 10:15:00-04:00	2020-10-12 18:20:32.81291	11.90	14.05	\N
53	9	FLY	buy	357	7.48	{"avg": 7.364073619178961, "macd": [0.014190495545344461, 0.018179916482290892, 0.020699492242207462, 0.02243762718525133, 0.02633573715846005], "vwap": 7.476666666666667, "macd_signal": [0.021817670673908503, 0.02109011983558498, 0.021011994316909477, 0.021297120890577847, 0.022304844144154286]}	2020-10-01 10:17:00-04:00	2020-10-12 18:20:38.760769	7.20	8.32	\N
54	9	CHEF	buy	168	15.17	{"avg": 14.964009906864737, "macd": [0.018974286909061888, 0.022327430180840935, 0.027492115876041723, 0.03561268596913614, 0.0463554381962723], "vwap": 15.17, "macd_signal": [0.02341436613836013, 0.02319697894685629, 0.024056006332693378, 0.02636734225998193, 0.030364961447240005]}	2020-10-01 10:17:00-04:00	2020-10-12 18:20:41.055389	14.58	16.95	\N
55	9	CUE	buy	158	15.51	{"avg": 15.284494416613143, "macd": [0.04364171264767869, 0.04083534933008082, 0.04056443391794495, 0.043080788692847705, 0.05493171293309551], "vwap": 15.520000000000001, "macd_signal": [0.05163171958588373, 0.049472445534723145, 0.047690843211367503, 0.046768832307663544, 0.04840140843274994]}	2020-10-01 10:18:00-04:00	2020-10-12 18:20:42.579116	14.88	17.40	\N
56	9	LAC	buy	153	12.63	{"avg": 12.254879699289749, "macd": [0.08780807543423208, 0.09490795280211728, 0.09461061509544955, 0.10446756572304139, 0.11817922144867055], "vwap": 12.609966666666665, "macd_signal": [0.08996778754496801, 0.09095582059639787, 0.09168677949620821, 0.09424293674157484, 0.09903019368299398]}	2020-10-01 10:18:00-04:00	2020-10-12 18:20:43.83655	11.98	14.59	\N
57	9	ECOM	buy	193	14.91	{"avg": 14.800629617648676, "macd": [0.028056592996451357, 0.02513360271893461, 0.022557087798157127, 0.02905631869728431, 0.03860350736529128], "vwap": 14.906666666666666, "macd_signal": [0.041856798316256515, 0.038512159196792135, 0.03532114491706513, 0.03406817967310897, 0.03497524521154543]}	2020-10-01 10:19:00-04:00	2020-10-12 18:20:47.700214	14.39	16.46	\N
58	9	TVTY	buy	217	14.47	{"avg": 14.301838043078316, "macd": [0.0321606731016999, 0.028365624786557575, 0.025866762148920586, 0.027602789858477195, 0.035827851000057365], "vwap": 14.456666666666669, "macd_signal": [0.03475281105024529, 0.03347537379750774, 0.03195365146779031, 0.03108347914592769, 0.032032353516753626]}	2020-10-01 10:20:00-04:00	2020-10-12 18:20:54.846613	14.01	15.85	\N
59	9	ANGI	buy	249	11.82	{"avg": 11.671258006245717, "macd": [0.016699243812412945, 0.02009222020538637, 0.023319284670474616, 0.027975027089896187, 0.032101605599814675], "vwap": 11.816666666666668, "macd_signal": [0.02187925642493586, 0.02152184918102596, 0.02188133627891569, 0.02310007444111179, 0.024900380672852367]}	2020-10-01 10:21:00-04:00	2020-10-12 18:20:56.082863	11.42	13.02	\N
60	9	PACB	buy	243	10.50	{"avg": 10.284754247906474, "macd": [0.04210549755194393, 0.047003782616519274, 0.05070467510168264, 0.05262754328690633, 0.05473090105838452], "vwap": 10.496033333333333, "macd_signal": [0.0418690140611966, 0.04289596777226114, 0.04445770923814544, 0.046091676047897615, 0.047819521049995]}	2020-10-01 10:21:00-04:00	2020-10-12 18:21:00.595956	10.09	11.73	\N
61	9	FLDM	buy	327	7.78	{"avg": 7.668683391727555, "macd": [0.007827524095096017, 0.008285560005821146, 0.010943160106750227, 0.011384947535036716, 0.018701049394405977], "vwap": 7.756666666666667, "macd_signal": [0.010013222040576478, 0.009667689633625413, 0.009922783728250376, 0.010215216489607644, 0.011912383070567311]}	2020-10-01 10:22:00-04:00	2020-10-12 18:21:02.647159	7.47	8.70	\N
62	9	DESP	buy	307	6.75	{"avg": 6.589233789364616, "macd": [0.0021657216561559167, 0.012395619562190596, 0.01787607382404488, 0.02356160267508045, 0.03014073094668923], "vwap": 6.746666666666667, "macd_signal": [0.011715992161412676, 0.01185191764156826, 0.013056748878063584, 0.015157719637466958, 0.01815432189931141]}	2020-10-01 10:23:00-04:00	2020-10-12 18:21:07.364016	6.43	7.72	\N
63	9	MOMO	buy	196	14.14	{"avg": 14.000408787884544, "macd": [0.007897400210294947, 0.007792885096714386, 0.01161079621745209, 0.018857184745465716, 0.025915099578007528], "vwap": 14.1395, "macd_signal": [0.0058698390035728755, 0.006254448222201178, 0.00732571782125136, 0.009632011206094231, 0.01288862888047689]}	2020-10-01 10:24:00-04:00	2020-10-12 18:21:13.234001	13.63	15.66	\N
64	9	OCUL	buy	321	8.05	{"avg": 7.888001197897813, "macd": [0.030273372761842055, 0.03175630158954057, 0.03056194324554884, 0.03188645516127586, 0.03513743794241453], "vwap": 8.042433333333333, "macd_signal": [0.027453489675486227, 0.028314052058297096, 0.028763630295747446, 0.02938819526885313, 0.03053804380356541]}	2020-10-01 10:24:00-04:00	2020-10-12 18:21:13.597988	7.74	8.98	\N
65	9	CCCL	sell	331	2.30	{"avg": 2.451177530134921, "rsi": [36.88913124904659, 37.59974437390433, 34.33437228321034], "vwap": 2.3000333333333334, "reasons": "stopped", "movement": -0.11877394636015329, "sell_macd": [0.013789153144388777, 0.006973946777176554, -0.0012895120385363334, -0.007446888881448732, -0.014224322496513508], "sell_macd_signal": [0.025774320150111725, 0.02201424547552469, 0.017353493972712485, 0.012393417401880241, 0.007069869422201491]}	2020-10-01 10:27:00-04:00	2020-10-12 18:21:28.056537	2.31	3.52	\N
66	9	OUT	buy	177	15.22	{"avg": 15.044552165683909, "macd": [0.02474417407969831, 0.030876928932906722, 0.03493105961353393, 0.037709298071421316, 0.04145054702764206], "vwap": 15.219999999999999, "macd_signal": [0.020825679998113255, 0.02283592978507195, 0.025254955750764346, 0.02774582421489574, 0.030486768777445]}	2020-10-01 10:27:00-04:00	2020-10-12 18:21:29.764022	14.66	16.91	\N
67	9	MTEM	buy	266	11.13	{"avg": 11.043978307694609, "macd": [0.00862435169555198, 0.012097800983985607, 0.01547901850644351, 0.01795172148265678, 0.021279887957096122], "vwap": 11.133333333333335, "macd_signal": [0.0093356253409364, 0.009888060469546243, 0.011006252076925696, 0.012395345958071912, 0.014172254357876755]}	2020-10-01 10:28:00-04:00	2020-10-12 18:21:33.676946	10.75	12.26	\N
68	9	IVC	sell	229	7.95	{"avg": 7.72763212771254, "rsi": [67.70872970710835, 67.70872970710835, 69.97384996023153], "vwap": 7.941666666666666, "reasons": "bail", "movement": 0.02185089974293058, "sell_macd": [0.042186108793154986, 0.04034763784146911, 0.04020924959546157, 0.03957921655187757, 0.039440066779147465], "sell_macd_signal": [0.04624056737553771, 0.04506198146872399, 0.044091435094071506, 0.043188991385632716, 0.04243920646433567]}	2020-10-01 10:34:00-04:00	2020-10-12 18:21:45.288448	7.34	9.09	\N
69	9	FLDM	sell	327	8.03	{"avg": 7.756141924145821, "rsi": [76.014934721351, 78.8933783441244, 80.38242403445165], "vwap": 8.029666666666666, "reasons": "rsi max, cool-down for 5 minutes", "movement": 0.032133676092544874, "sell_macd": [0.040546595330929414, 0.04166644250651608, 0.04251740858481057, 0.04459196126160059, 0.04676531017085317], "sell_macd_signal": [0.03269330462965025, 0.034487932205023414, 0.03609382748098085, 0.0377934542371048, 0.03958782542385447]}	2020-10-01 10:40:00-04:00	2020-10-12 18:21:52.705609	7.47	8.70	\N
70	9	NEXA	sell	308	5.80	{"avg": 5.314467710681301, "rsi": [73.53591987435942, 75.98819476584188, 80.3108180233873], "vwap": 5.786666666666666, "reasons": "rsi max, cool-down for 5 minutes", "movement": 0.05839416058394149, "sell_macd": [0.04226879429263697, 0.04574499744916771, 0.050976519422450295, 0.05698839992861249, 0.06595451807286956], "sell_macd_signal": [0.024237844214652207, 0.02853927486155531, 0.03302672377373431, 0.03781905900470994, 0.04344615081834187]}	2020-10-01 10:40:00-04:00	2020-10-12 18:21:53.202603	5.16	6.45	\N
71	9	KSS	sell	148	19.17	{"avg": 18.93598868579368, "rsi": [63.12964665835696, 58.524556910656855, 59.06057556676776], "vwap": 19.16, "reasons": "bail post whipsawed", "movement": 0.0026150627615063133, "sell_macd": [0.030909182772486332, 0.03099167866507102, 0.031172952364940443, 0.02935091950295643, 0.027290232603970566], "sell_macd_signal": [0.025064592832881274, 0.026250009999319222, 0.027234598472443465, 0.027657862678546058, 0.02758433666363096]}	2020-10-01 10:43:00-04:00	2020-10-12 18:21:56.581842	18.45	21.14	\N
72	9	VCEL	sell	166	19.10	{"avg": 18.65389561391351, "rsi": [76.56651425039928, 77.09395055749482, 79.09568000014579], "vwap": 19.1, "reasons": "rsi max, cool-down for 5 minutes", "movement": 0.020746267061426543, "sell_macd": [0.057158617411218415, 0.060587617266815386, 0.06325193381316652, 0.06503124626729573, 0.06764588913717162], "sell_macd_signal": [0.046616871943493586, 0.049411021008157946, 0.05217920356915966, 0.05474961210878687, 0.057328867514463824]}	2020-10-01 10:46:00-04:00	2020-10-12 18:22:00.39347	18.11	20.52	\N
73	9	ADES	sell	125	5.50	{"avg": 5.81283356994255, "rsi": [42.68242967107159, 39.03033745368091, 35.73728366498145], "vwap": 5.5000333333333336, "reasons": "below cost & macd negative & RSI trending down and too far from VWAP", "movement": -0.09909909909909916, "sell_macd": [-0.02596743876939911, -0.024367602871947724, -0.02384347616131155, -0.02520333228079874, -0.02801183489839776], "sell_macd_signal": [-0.027741402057408635, -0.027066642220316454, -0.026422009008515473, -0.026178273662972127, -0.026544985910057252]}	2020-10-01 10:48:00-04:00	2020-10-12 18:22:01.781186	5.31	8.50	\N
74	9	AGTC	sell	357	5.00	{"avg": 4.9586487277297895, "rsi": [53.14444714194673, 55.58531951253066, 55.58531951253066], "vwap": 5.0, "reasons": "bail post whipsawed", "movement": 0.004016064257028026, "sell_macd": [0.005471070107359033, 0.005693205432164561, 0.004992360160167131, 0.0046411513747113275, 0.0043072074980612385], "sell_macd_signal": [0.005450396372482123, 0.0054989581844186106, 0.005397638579568315, 0.005246341138596918, 0.005058514410489782]}	2020-10-01 11:06:00-04:00	2020-10-12 18:22:22.133587	4.70	5.82	\N
75	9	LAC	sell	153	12.68	{"avg": 12.352589597073738, "rsi": [61.32119257150981, 58.66679763253387, 53.72811704156203], "vwap": 12.698266666666669, "reasons": "bail post whipsawed", "movement": 0.0035629453681710155, "sell_macd": [0.0657374197153775, 0.0672213076929502, 0.06545782679634549, 0.06138942834709127, 0.054994276806366926], "sell_macd_signal": [0.05753941293185064, 0.05947579188407055, 0.06067219886652554, 0.06081564476263869, 0.05965137117138434]}	2020-10-01 11:08:00-04:00	2020-10-12 18:22:25.090763	11.98	14.59	\N
76	9	ANGI	sell	249	12.09	{"avg": 11.831103810346855, "rsi": [68.20142991660116, 61.16299856454839, 63.59088545391969], "vwap": 12.086666666666666, "reasons": "bail", "movement": 0.022842639593908594, "sell_macd": [0.028596526528502864, 0.028235537057756943, 0.02758755127460688, 0.025425646102352317, 0.024016902079354807], "sell_macd_signal": [0.02540235697517105, 0.025968992991688228, 0.026292704648271957, 0.02611929293908803, 0.025698814767141386]}	2020-10-01 11:12:00-04:00	2020-10-12 18:22:29.346292	11.42	13.02	\N
77	9	FLY	sell	357	7.68	{"avg": 7.501070490931919, "rsi": [64.09843516501078, 59.32101807167517, 56.293653967265925], "vwap": 7.675699999999999, "reasons": "bail", "movement": 0.026283422459892944, "sell_macd": [0.026819383235638483, 0.026866986832316364, 0.0260356781176192, 0.02401081836438035, 0.02137831961686487], "sell_macd_signal": [0.021231545286778505, 0.022358633595886076, 0.0230940425002327, 0.02327739767306223, 0.02289758206182276]}	2020-10-01 11:31:00-04:00	2020-10-12 18:22:50.870601	7.20	8.32	\N
78	9	MOMO	sell	196	14.59	{"avg": 14.154357440748534, "rsi": [76.64372364373034, 78.12460891055747, 81.21401026652266], "vwap": 14.57, "reasons": "rsi max, cool-down for 5 minutes", "movement": 0.03182461103253177, "sell_macd": [0.020807354742203188, 0.02198955884688658, 0.024526476865512237, 0.026922432764889592, 0.029177431741484128], "sell_macd_signal": [0.011970821907545675, 0.013974569295413856, 0.01608495080943353, 0.018252447200524744, 0.02043744410871662]}	2020-10-01 11:59:00-04:00	2020-10-12 18:23:22.539692	13.63	15.66	\N
79	9	VRA	sell	348	6.37	{"avg": 6.303763906646326, "rsi": [53.18435313634032, 59.80141214592001, 51.90124249983038], "vwap": 6.378333333333334, "reasons": "bail post whipsawed", "movement": 0.0015723270440251237, "sell_macd": [0.008707002677565967, 0.008781145166166127, 0.007685916648632052, 0.00777163576940243, 0.007131438948754898], "sell_macd_signal": [0.00854183364551526, 0.008589695949645433, 0.008408940089442757, 0.008281479225434691, 0.008051471170098733]}	2020-10-01 11:59:00-04:00	2020-10-12 18:23:23.089169	6.07	7.22	\N
80	9	CHEF	sell	168	15.28	{"avg": 15.069180399008548, "rsi": [65.61252685216387, 65.61252685216387, 62.80760171370895], "vwap": 15.281666666666666, "reasons": "bail post whipsawed", "movement": 0.00725115359261697, "sell_macd": [0.0306666977685488, 0.029872617877710894, 0.028865898250074906, 0.02770655960191526, 0.02601045295483928], "sell_macd_signal": [0.026246266541189375, 0.02697153680849368, 0.027350409096809924, 0.027421639197830992, 0.02713940194923265]}	2020-10-01 12:11:00-04:00	2020-10-12 18:23:37.816145	14.58	16.95	\N
81	9	KIRK	sell	179	8.84	{"avg": 8.632546476335934, "rsi": [72.32080454735332, 73.41452739062369, 79.51618018066475], "vwap": 8.8397, "reasons": "rsi max, cool-down for 5 minutes", "movement": -0.0045045045045046085, "sell_macd": [0.018690439442730522, 0.019841133571389946, 0.022558023365522928, 0.0249015995250339, 0.030025144590563713], "sell_macd_signal": [0.013293145494634149, 0.014602743109985309, 0.016193799161092832, 0.017935359233881044, 0.020353316305217577]}	2020-10-01 12:21:00-04:00	2020-10-12 18:23:50.79666	8.32	10.55	\N
82	9	QTNT	sell	305	5.61	{"avg": 5.376183127040583, "rsi": [76.80783743487117, 77.27149475652722, 80.17872052439773], "vwap": 5.601666666666667, "reasons": "rsi max, cool-down for 5 minutes", "movement": 0.04081632653061237, "sell_macd": [0.01693183027843581, 0.01888721459640408, 0.020425342446815264, 0.02152485320756714, 0.022801638390738788], "sell_macd_signal": [0.012119794844515752, 0.013473278794893417, 0.014863691525277787, 0.016195923861735656, 0.017517066767536282]}	2020-10-01 12:23:00-04:00	2020-10-12 18:23:53.311675	5.06	6.37	\N
83	9	TBPH	sell	150	14.64	{"avg": 15.04718703883953, "rsi": [32.53777123197021, 30.921758562885373, 29.341842497023546], "vwap": 14.64, "reasons": "stopped", "movement": -0.04376224689745264, "sell_macd": [-0.029169326602223933, -0.029639075385460956, -0.02962538514661439, -0.030003998140413657, -0.030688579718173514], "sell_macd_signal": [-0.03465163731310654, -0.03364912492757742, -0.032844376971384814, -0.03227630120519058, -0.03195875690778717]}	2020-10-01 12:28:00-04:00	2020-10-12 18:23:58.805731	14.64	17.31	\N
84	9	GNK	sell	473	7.12	{"avg": 7.0137715390496345, "rsi": [74.41035859404104, 77.78186557592736, 79.25368812829248], "vwap": 7.12, "reasons": "rsi max, cool-down for 5 minutes", "movement": 0.007072135785007047, "sell_macd": [0.015982355280192095, 0.01720021226931312, 0.01792581236123514, 0.019297373311287558, 0.020634987385208525], "sell_macd_signal": [0.010047572708857328, 0.011478100620948487, 0.012767642969005817, 0.014073589037462166, 0.015385868707011438]}	2020-10-01 12:37:00-04:00	2020-10-12 18:24:07.535952	6.86	7.70	\N
85	9	JWN	sell	185	12.44	{"avg": 12.277491867154422, "rsi": [64.89955258928096, 56.549868719384634, 57.72142550771794], "vwap": 12.441666666666668, "reasons": "bail post whipsawed", "movement": 0.000008038649828356091, "sell_macd": [0.011431628598169041, 0.012380283967786454, 0.012958713035439828, 0.011942440711074198, 0.011341763912289693], "sell_macd_signal": [0.011320013397113318, 0.011532067511247945, 0.011817396616086322, 0.011842405435083896, 0.011742277130525055]}	2020-10-01 12:47:00-04:00	2020-10-12 18:24:19.554297	11.90	14.05	\N
86	9	MTEM	sell	266	11.19	{"avg": 11.061192005335803, "rsi": [68.62888127326801, 64.46924897176329, 64.4692489717633], "vwap": 11.19, "reasons": "bail post whipsawed", "movement": 0.00539083557951471, "sell_macd": [0.021417163956114038, 0.020687202073181865, 0.020369216163453885, 0.019337419140763146, 0.018282286147227822], "sell_macd_signal": [0.017222158921232064, 0.017915167551622025, 0.0184059772739884, 0.01859226564734335, 0.018530269747320242]}	2020-10-01 12:50:00-04:00	2020-10-12 18:24:22.613284	10.75	12.26	\N
87	9	SYRS	sell	323	8.75	{"avg": 8.956669273171702, "rsi": [35.75355678286559, 30.233276812106347, 12.11039340624242], "vwap": 8.78, "reasons": "stopped", "movement": -0.03508965395558095, "sell_macd": [-0.004694654033643886, -0.004745718082576289, -0.004723875548204859, -0.005164986877939981, -0.008558515357240282], "sell_macd_signal": [-0.005385275165352506, -0.005257363748797263, -0.005150666108678782, -0.005153530262531022, -0.005834527281472874]}	2020-10-01 12:51:00-04:00	2020-10-12 18:24:24.374509	8.76	10.00	\N
88	9	ASMB	sell	133	17.03	{"avg": 16.666682374472284, "rsi": [63.08590060973223, 57.57554229887318, 60.08372237002308], "vwap": 17.0334, "reasons": "bail post whipsawed", "movement": 0.0037360047142016182, "sell_macd": [0.03934157093718582, 0.04060027676934652, 0.041053810187012374, 0.03827664388666108, 0.03735058386188328], "sell_macd_signal": [0.03769968293574916, 0.03827980170246863, 0.038834603399377374, 0.03872301149683412, 0.03844852596984395]}	2020-10-01 12:52:00-04:00	2020-10-12 18:24:24.676747	16.22	19.22	\N
89	9	PACB	sell	243	10.08	{"avg": 10.33097755560022, "rsi": [41.34308587332204, 34.420660427145684, 32.26782190308487], "vwap": 10.089266666666667, "reasons": "stopped", "movement": -0.039628571428571444, "sell_macd": [-0.0045126666224479806, -0.004576272469968501, -0.0050858838775180715, -0.006975374794080835, -0.00865495159321128], "sell_macd_signal": [-0.006613688692448799, -0.006206205447952739, -0.0059821411338658054, -0.006180787865908812, -0.006675620611369305]}	2020-10-01 12:52:00-04:00	2020-10-12 18:24:25.769463	10.09	11.73	\N
90	9	CBAY	sell	271	7.03	{"avg": 7.3172914257040444, "rsi": [40.104871665296955, 40.065716420194576, 30.45833701494332], "vwap": 7.040666666666667, "reasons": "stopped", "movement": -0.05126924789808231, "sell_macd": [-0.005379396957505556, -0.00551956969841072, -0.005692177422512934, -0.005757947370871541, -0.0067387471483257855], "sell_macd_signal": [-0.004013961503109665, -0.004315083142169876, -0.004590501998238487, -0.0048239910727650975, -0.005206942287877235]}	2020-10-01 13:07:00-04:00	2020-10-12 18:24:42.071386	7.04	8.51	\N
91	9	ARAY	sell	588	2.57	{"avg": 2.4728447051997167, "rsi": [69.67468555831083, 74.12592148604276, 77.6576606981675], "vwap": 2.565, "reasons": "scale-out", "movement": 0.040485829959514025, "sell_macd": [0.0031944315314911265, 0.003467386121323912, 0.004154527540397357, 0.005155488028760757, 0.006128243110228748], "sell_macd_signal": [0.0022029777037704903, 0.0024558593872811745, 0.002795593017904411, 0.00326757202007568, 0.003839706238106294]}	2020-10-01 13:13:00-04:00	2020-10-12 18:24:48.832327	2.39	2.73	\N
92	9	ARAY	sell	588	2.58	{"avg": 2.4746769995245947, "rsi": [77.6576606981675, 77.65766069816749, 80.7111684249709], "vwap": 2.5772666666666666, "reasons": "rsi max, cool-down for 5 minutes", "movement": 0.003891050583657678, "sell_macd": [0.004154527540397357, 0.005155488028760757, 0.0063879833699695965, 0.007265408527145745, 0.008232249664419378], "sell_macd_signal": [0.002795593017904411, 0.00326757202007568, 0.0038916542900544636, 0.00456640513747272, 0.005299574042862051]}	2020-10-01 13:15:00-04:00	2020-10-12 18:24:51.26738	2.39	2.73	\N
93	9	MBWM	sell	109	18.36	{"avg": 18.166894132900072, "rsi": [66.17242951164516, 60.014452757247675, 57.60885159625311], "vwap": 18.365, "reasons": "bail post whipsawed", "movement": 0.00464989059080948, "sell_macd": [0.05652514494806482, 0.06045290512023627, 0.060650472720364945, 0.05689986990591578, 0.05193966161016661], "sell_macd_signal": [0.04878699178883285, 0.051120174455113535, 0.05302623410816382, 0.053800961267714206, 0.053428701336204684]}	2020-10-01 13:49:00-04:00	2020-10-12 18:25:28.743826	17.36	21.03	\N
94	9	ECOM	sell	193	15.21	{"avg": 14.949603530300616, "rsi": [69.55658058903495, 65.75964419561056, 50.8214835844111], "vwap": 15.223333333333334, "reasons": "bail", "movement": 0.020120724346076507, "sell_macd": [0.016746954491068777, 0.01724101810537526, 0.01792157393333582, 0.017699648943001023, 0.015392092903654842], "sell_macd_signal": [0.017422360722613637, 0.01738609219916596, 0.017493188545999933, 0.01753448062540015, 0.017106003081051088]}	2020-10-01 13:59:00-04:00	2020-10-12 18:25:40.092632	14.39	16.46	\N
95	9	MGI	sell	282	2.97	{"avg": 2.8746440587892215, "rsi": [64.92266423118085, 69.19742291066353, 71.79828228672012], "vwap": 2.966666666666667, "reasons": "scale-out", "movement": 0.04210526315789477, "sell_macd": [0.009893969167867223, 0.01013511532644129, 0.009729058887324449, 0.00986292323493032, 0.010071177836074252], "sell_macd_signal": [0.008457028207630174, 0.008792645631392397, 0.008979928282578807, 0.00915652727304911, 0.00933945738565414]}	2020-10-01 14:53:00-04:00	2020-10-12 18:26:39.88663	2.67	3.38	\N
96	9	MGI	sell	283	2.95	{"avg": 2.8752064950424328, "rsi": [64.30202399894316, 67.91015868444144, 56.272321790483815], "vwap": 2.955666666666667, "reasons": "bail", "movement": -0.00639730639730644, "sell_macd": [0.00986292323493032, 0.010244338009234522, 0.009889020456624653, 0.010003201318790289, 0.009217672969151547], "sell_macd_signal": [0.00915652727304911, 0.009374089420286193, 0.009477075627553885, 0.009582300765801166, 0.009509375206471242]}	2020-10-01 14:57:00-04:00	2020-10-12 18:26:44.547089	2.67	3.38	\N
97	9	ARDX	sell	420	5.45	{"avg": 5.404138888008098, "rsi": [65.59269508361957, 65.59269508361957, 65.59269508361959], "vwap": 5.45, "reasons": "bail post whipsawed", "movement": 0.0073937153419593405, "sell_macd": [0.009817262794307169, 0.009542649858128804, 0.009204735234377814, 0.008821883121917118, 0.008408986889844705], "sell_macd_signal": [0.009564019673064455, 0.009559745710077325, 0.009488743614937423, 0.009355371516333363, 0.009166094591035632]}	2020-10-01 14:59:00-04:00	2020-10-12 18:26:46.417117	5.17	6.12	\N
98	9	VECO	sell	258	12.17	{"avg": 12.005730388819085, "rsi": [69.96629367850102, 57.66299193798255, 51.199173074818674], "vwap": 12.17, "reasons": "bail", "movement": 0.024410774410774338, "sell_macd": [0.01624661024587226, 0.016292790827389325, 0.016636661529515706, 0.015130110628192028, 0.012720718878741266], "sell_macd_signal": [0.012034039371930965, 0.012885789663022637, 0.01363596403632125, 0.013934793354695406, 0.013691978459504577]}	2020-10-01 15:00:00-04:00	2020-10-12 18:26:48.568068	11.49	13.04	\N
99	9	CGEN	sell	155	17.06	{"avg": 16.798220798578654, "rsi": [57.79819459858435, 50.72124420419757, 50.72124420419757], "vwap": 17.06, "reasons": "bail", "movement": 0.020945541591861034, "sell_macd": [0.007292926425733981, 0.00754802477760208, 0.008687737457268696, 0.007385075422497778, 0.006274104026282146], "sell_macd_signal": [0.0057075371072173985, 0.006075634641294335, 0.006598055204489207, 0.006755459248090921, 0.006659188203729166]}	2020-10-01 15:08:00-04:00	2020-10-12 18:26:56.271137	16.07	18.64	\N
100	9	CUE	sell	158	15.69	{"avg": 15.418669182892296, "rsi": [64.37124145272932, 58.639546122514716, 59.032386895684084], "vwap": 15.685, "reasons": "bail post whipsawed", "movement": 0.011283043197936861, "sell_macd": [0.038629328238652505, 0.03825705653085443, 0.03980779212827379, 0.037901770618102404, 0.03618406852422673], "sell_macd_signal": [0.035141814934801376, 0.035764863254011986, 0.03657344902886435, 0.03683911334671196, 0.03670810438221491]}	2020-10-01 15:08:00-04:00	2020-10-12 18:26:56.451416	14.88	17.40	\N
101	9	TVTY	sell	217	14.54	{"avg": 14.325678849266344, "rsi": [68.25096131922818, 78.24986156368736, 79.03974041599461], "vwap": 14.54, "reasons": "rsi max, cool-down for 5 minutes", "movement": 0.004837595024187871, "sell_macd": [0.021837002626217128, 0.021699287141958123, 0.02182964797056286, 0.02632305059063711, 0.03000202600120616], "sell_macd_signal": [0.02373152817017542, 0.02332507996453196, 0.02302599356573814, 0.023685404970717933, 0.02494872917681558]}	2020-10-01 15:14:00-04:00	2020-10-12 18:27:03.559854	14.01	15.85	\N
102	9	DESP	sell	307	6.76	{"avg": 6.643861714077856, "rsi": [58.1181416794968, 52.68330373655654, 54.951645550802866], "vwap": 6.756666666666667, "reasons": "bail post whipsawed", "movement": 0.00148148148148145, "sell_macd": [0.011739927357139912, 0.009893168092063753, 0.009884093754445544, 0.008709696949100731, 0.008027789954255127], "sell_macd_signal": [0.013225239047923765, 0.012558824856751763, 0.012023878636290519, 0.01136104229885256, 0.010694391829933075]}	2020-10-01 15:22:00-04:00	2020-10-12 18:27:11.46888	6.43	7.72	\N
103	9	CTMX	sell	375	6.85	{"avg": 6.779683645725906, "rsi": [52.23840743278964, 52.23840743278964, 52.23840743278964], "vwap": 6.849999999999999, "reasons": "bail post whipsawed", "movement": 0.004398826979472047, "sell_macd": [0.006924614122134187, 0.007452270190522192, 0.006727686994583593, 0.00607570415366343, 0.005488760327573949], "sell_macd_signal": [0.00583599697789411, 0.006159251620419726, 0.006272938695252499, 0.006233491786934686, 0.006084545495062538]}	2020-10-01 15:24:00-04:00	2020-10-12 18:27:13.657365	6.55	7.62	\N
104	9	ETH	sell	135	13.77	{"liquidate": 1}	2020-10-01 16:00:00-04:00	2020-10-12 18:27:51.264965	\N	\N	\N
105	9	EVGN	sell	409	4.06	{"liquidate": 1}	2020-10-01 16:00:00-04:00	2020-10-12 18:27:51.486366	\N	\N	\N
106	9	INO	sell	159	11.74	{"liquidate": 1}	2020-10-01 16:00:00-04:00	2020-10-12 18:27:51.790998	\N	\N	\N
107	9	OCUL	sell	321	7.91	{"liquidate": 1}	2020-10-01 16:00:00-04:00	2020-10-12 18:27:52.497354	\N	\N	\N
108	9	OPK	sell	678	3.89	{"liquidate": 1}	2020-10-01 16:00:00-04:00	2020-10-12 18:27:53.145019	\N	\N	\N
109	9	OUT	sell	177	15.22	{"liquidate": 1}	2020-10-01 16:00:00-04:00	2020-10-12 18:27:54.876413	\N	\N	\N
110	9	SCWX	sell	219	11.76	{"liquidate": 1}	2020-10-01 16:00:00-04:00	2020-10-12 18:27:55.232622	\N	\N	\N
111	11	MBWM	buy	109	18.28	{"avg": 18.227523949889463, "macd": [-0.01900254431929227, -0.007074915486352751, 0.013518809844139668, 0.02870171362337004, 0.04425866999030603], "vwap": 18.276666666666667, "macd_signal": [-0.030452457820819012, -0.02577694935392576, -0.017917797514312675, -0.008593895286776131, 0.0019766177686403024]}	2020-10-01 09:46:00-04:00	2020-10-12 19:49:51.703883	17.36	21.03	\N
112	11	AGLE	buy	274	7.21	{"avg": 7.133804952073494, "macd": [0.012057366240095568, 0.01517833776819888, 0.01585512927576449, 0.02099101858481678, 0.026371091193507112], "vwap": 7.206666666666667, "macd_signal": [0.008339616552454198, 0.009707360795603134, 0.010936914491635405, 0.01294773531027168, 0.015632406486918767]}	2020-10-01 09:47:00-04:00	2020-10-12 19:49:55.543082	6.85	8.30	\N
113	11	VECO	buy	258	11.88	{"avg": 11.781087517389818, "macd": [0.023337244035138482, 0.024040509707985436, 0.02431753540941095, 0.026052327222101823, 0.031701496565696985], "vwap": 11.873333333333335, "macd_signal": [0.02326799284881777, 0.023422496220651304, 0.023601504058403232, 0.024091668691142952, 0.02561363426605376]}	2020-10-01 09:47:00-04:00	2020-10-12 19:50:01.018877	11.49	13.04	\N
114	11	AGTC	buy	357	4.98	{"avg": 4.855714638604812, "macd": [0.0007414437299662069, 0.007755607942219633, 0.012364931949629288, 0.015835314782219, 0.022362420251829995], "vwap": 4.976666666666667, "macd_signal": [-0.0012790498127005201, 0.0005278817382835107, 0.0028952917805526663, 0.005483296380885933, 0.008859121155074746]}	2020-10-01 09:48:00-04:00	2020-10-12 19:50:01.287484	4.70	5.82	\N
115	11	GNK	buy	473	7.07	{"avg": 7.0124081951982795, "macd": [0.011462879071444831, 0.014166574235443541, 0.016921132278628903, 0.018886428383361853, 0.025795005919738934], "vwap": 7.0566666666666675, "macd_signal": [0.005987185024744446, 0.007623062866884266, 0.009482676749233193, 0.011363427076058925, 0.014249742844794928]}	2020-10-01 09:48:00-04:00	2020-10-12 19:50:03.319563	6.86	7.70	\N
116	11	ETH	buy	135	14.04	{"avg": 13.744914727553015, "macd": [0.06681055742693687, 0.060437587333943554, 0.06512614253363402, 0.07124821692047156, 0.08879402201647935], "vwap": 14.003333333333332, "macd_signal": [0.047665176565224635, 0.05021965871896842, 0.05320095548190154, 0.056810407769615545, 0.0632071306189883]}	2020-10-01 09:49:00-04:00	2020-10-12 19:50:08.803419	13.30	16.25	\N
117	11	ARDX	buy	420	5.41	{"avg": 5.344627072199096, "macd": [0.01598339852210895, 0.020629613714255868, 0.024034715489137604, 0.02563090958069214, 0.028982561252642647], "vwap": 5.403333333333333, "macd_signal": [0.020619218112396948, 0.02062129723276873, 0.021303980884042506, 0.022169366623372433, 0.023532005549226475]}	2020-10-01 09:53:00-04:00	2020-10-12 19:50:30.644202	5.17	6.12	\N
118	11	BLDP	buy	148	15.62	{"avg": 15.445073180487048, "macd": [0.056915686918374675, 0.06421146390667687, 0.07597640070484246, 0.08392925545320828, 0.09051192791471152], "vwap": 15.620466666666667, "macd_signal": [0.06176426989360948, 0.06225370869622296, 0.06499824709794685, 0.06878444876899914, 0.07312994459814162]}	2020-10-01 09:54:00-04:00	2020-10-12 19:50:36.611314	14.95	17.64	\N
119	11	CCCL	buy	406	2.42	{"avg": 2.330856770746662, "macd": [0.04269764769673268, 0.04089746660171256, 0.04340048749191938, 0.04526581169374033, 0.05068661200350055], "vwap": 2.3987, "macd_signal": [0.03801005291162056, 0.03858753564963896, 0.039550126018095044, 0.0406932631532241, 0.04269193292327939]}	2020-10-01 09:55:00-04:00	2020-10-12 19:50:42.551531	2.17	3.15	\N
120	11	EVGN	buy	409	4.22	{"avg": 4.131378077591527, "macd": [0.024364228891769812, 0.024436367318810603, 0.02461326930787333, 0.025412685176136485, 0.029594419144991058], "vwap": 4.206466666666667, "macd_signal": [0.026493529126306502, 0.02608209676480732, 0.025788331273420523, 0.025713202053963716, 0.026489445472169186]}	2020-10-01 09:55:00-04:00	2020-10-12 19:50:43.403633	3.98	4.95	\N
121	11	INO	buy	159	11.98	{"avg": 11.735804572106847, "macd": [0.036118360602792166, 0.03581959305856941, 0.036948255603025615, 0.043171016791953676, 0.05694359587966602], "vwap": 11.9433, "macd_signal": [0.028844541761627842, 0.030239552021016157, 0.03158129273741805, 0.033899237548325176, 0.03850810921459334]}	2020-10-01 09:55:00-04:00	2020-10-12 19:50:44.110273	11.35	13.87	\N
122	11	CGEN	buy	155	16.71	{"avg": 16.512347670503736, "macd": [0.04442684909117389, 0.04312129277732879, 0.05197738064793711, 0.06151445939957512, 0.0698809517616894], "vwap": 16.713333333333335, "macd_signal": [0.04246130482453998, 0.04259330241509775, 0.04447011806166562, 0.047878986329247517, 0.05227937941573589]}	2020-10-01 09:56:00-04:00	2020-10-12 19:50:48.13841	16.07	18.64	\N
123	11	MGI	buy	565	2.85	{"avg": 2.7904824429505832, "macd": [0.006537022141604343, 0.008575729639916041, 0.010067301273751106, 0.011918905245858369, 0.014031484936302796], "vwap": 2.8467000000000002, "macd_signal": [0.0010685550084106907, 0.0025699899347117606, 0.00406945220251963, 0.005639342811187378, 0.007317771236210462]}	2020-10-01 09:56:00-04:00	2020-10-12 19:50:49.886944	2.67	3.38	\N
124	11	ANAB	buy	151	15.16	{"avg": 14.962721743938074, "macd": [0.05057742002812127, 0.05108961814029023, 0.053301857480134274, 0.05682082724804438, 0.06411550923432152], "vwap": 15.159999999999998, "macd_signal": [0.050059071302750946, 0.0502651806702588, 0.050872516032233896, 0.05206217827539599, 0.0544728444671811]}	2020-10-01 09:57:00-04:00	2020-10-12 19:50:52.94202	14.51	17.14	\N
125	11	ASMB	buy	133	16.97	{"avg": 16.495381653484, "macd": [0.05240410928978534, 0.0580797395520527, 0.06585318503329418, 0.07830869719058242, 0.09521590817203673], "vwap": 16.936666666666667, "macd_signal": [0.04610434725330307, 0.048499425713053, 0.051970177577101234, 0.05723788149979747, 0.06483348683424532]}	2020-10-01 09:57:00-04:00	2020-10-12 19:50:53.156891	16.22	19.22	\N
126	11	KIRK	buy	179	8.88	{"avg": 8.672310715840416, "macd": [0.06633660866702229, 0.06539339093507301, 0.06710006204610863, 0.07963833642602403, 0.0925428153683967], "vwap": 8.858333333333334, "macd_signal": [0.0816231583051551, 0.07837720483113868, 0.07612177627413266, 0.07682508830451093, 0.07996863371728809]}	2020-10-01 09:58:00-04:00	2020-10-12 19:51:01.051256	8.32	10.55	\N
127	11	OPK	buy	678	3.79	{"avg": 3.750332123100643, "macd": [0.007622983498203784, 0.009861357125588999, 0.01230040931655374, 0.013672311639200796, 0.015516710192394534], "vwap": 3.790533333333333, "macd_signal": [0.00993141799735238, 0.009917405822999704, 0.01039400652171051, 0.011049667545208568, 0.011943076074645762]}	2020-10-01 09:59:00-04:00	2020-10-12 19:51:08.20369	3.64	4.23	\N
128	11	QTNT	buy	305	5.39	{"avg": 5.253003163126377, "macd": [0.022352248627350413, 0.021652315625478202, 0.023250347425999962, 0.026630567811867678, 0.03376173115906411], "vwap": 5.37, "macd_signal": [0.0251764311585102, 0.0244716080519038, 0.02422735592672303, 0.02470799830375196, 0.02651874487481439]}	2020-10-01 09:59:00-04:00	2020-10-12 19:51:08.77043	5.06	6.37	\N
129	11	AGLE	sell	274	7.34	{"avg": 7.219157758706618, "rsi": [74.53281857258753, 76.80628260035778, 79.2052623386686], "vwap": 7.323333333333333, "reasons": "rsi max, cool-down for 5 minutes", "movement": 0.018723994452149763, "sell_macd": [0.02886157081025953, 0.031263081925577474, 0.03298811549692182, 0.03520258171489221, 0.03690386385381039], "sell_macd_signal": [0.018806417732235927, 0.021297750570904238, 0.023635823556107754, 0.025949175187864645, 0.028140112921053794]}	2020-10-01 10:01:00-04:00	2020-10-12 19:51:15.593674	6.85	8.30	\N
130	11	TBPH	buy	150	15.31	{"avg": 14.977423058608597, "macd": [0.07787958261303807, 0.07680583203842062, 0.0822687815080112, 0.08640905494990747, 0.09425218593771945], "vwap": 15.301666666666668, "macd_signal": [0.06501865471211402, 0.06737609017737534, 0.07035462844350251, 0.07356551374478351, 0.0777028481833707]}	2020-10-01 10:02:00-04:00	2020-10-12 19:51:23.531936	14.64	17.31	\N
131	11	ANAB	sell	151	15.36	{"avg": 14.97753223992502, "rsi": [75.5695526757776, 77.95315304749421, 79.3349031595894], "vwap": 15.333333333333334, "reasons": "rsi max, cool-down for 5 minutes", "movement": 0.012858555885262137, "sell_macd": [0.038533656549761375, 0.042433739529712966, 0.049597002377341326, 0.057391791578384144, 0.06315320578380579], "sell_macd_signal": [0.03272980001447593, 0.03467058791752334, 0.03765587080948694, 0.04160305496326638, 0.045913085127374256]}	2020-10-01 10:03:00-04:00	2020-10-12 19:51:25.172032	14.51	17.14	\N
132	11	CTMX	buy	375	6.82	{"avg": 6.73431499276458, "macd": [0.026314794539278985, 0.026513948901426332, 0.025570107459788716, 0.026134674906256095, 0.028672331168801968], "vwap": 6.815, "macd_signal": [0.022890278142309824, 0.023615012294133125, 0.024006031327264244, 0.024431760043062613, 0.025279874268210482]}	2020-10-01 10:03:00-04:00	2020-10-12 19:51:26.320647	6.55	7.62	\N
133	11	VVPR	buy	130	10.06	{"avg": 9.825896675550432, "macd": [0.08894648926850124, 0.091780991776087, 0.09122476448970929, 0.09788612756175219, 0.11060501740381845], "vwap": 10.061666666666667, "macd_signal": [0.09043850790918484, 0.09070700468256528, 0.09081055664399408, 0.09222567082754571, 0.09590154014280025]}	2020-10-01 10:03:00-04:00	2020-10-12 19:51:29.402302	9.30	12.36	\N
134	11	VVPR	sell	130	10.24	{"avg": 9.87138565486752, "rsi": [69.95735082175429, 74.42149320055688, 79.69544475533445], "vwap": 10.1783, "reasons": "rsi max, cool-down for 5 minutes", "movement": 0.017416790859413812, "sell_macd": [0.05216037608961521, 0.05182174260248473, 0.05618356400728608, 0.06446311881612843, 0.07596325089034117], "sell_macd_signal": [0.05137947175568494, 0.0514679259250449, 0.052411053541493134, 0.05482146659642019, 0.05904982345520439]}	2020-10-01 10:04:00-04:00	2020-10-12 19:51:34.276568	9.30	12.36	\N
135	11	CBAY	buy	271	7.41	{"avg": 7.244957561308057, "macd": [0.019334535153858745, 0.02409297559610213, 0.028344255422975806, 0.033346320752600356, 0.04365795232706304], "vwap": 7.388433333333334, "macd_signal": [0.0064480329854720245, 0.009977021507598046, 0.013650468290673599, 0.01758963878305895, 0.022803301491859766]}	2020-10-01 10:06:00-04:00	2020-10-12 19:51:40.340351	7.04	8.51	\N
136	11	CCCL	sell	406	2.42	{"avg": 2.3703361691407827, "rsi": [56.87659839575572, 55.290827335934466, 60.28700854530699], "vwap": 2.4041333333333332, "reasons": "bail post whipsawed", "movement": 0.002400562890608752, "sell_macd": [0.0345203125604554, 0.032420682144757684, 0.029324597637139238, 0.02601174480474633, 0.024346165825331223], "sell_macd_signal": [0.03340799785727032, 0.03321053471476779, 0.032433347299242084, 0.031149026800342934, 0.029788454605340593]}	2020-10-01 10:06:00-04:00	2020-10-12 19:51:40.615771	2.17	3.15	\N
137	11	SYRS	buy	323	9.07	{"avg": 8.948097851337565, "macd": [0.026841082850022957, 0.028016823956113157, 0.02756571714229672, 0.02878076773383853, 0.0333295308157151], "vwap": 9.052133333333332, "macd_signal": [0.025488997864112697, 0.02599456308251279, 0.026308793894469575, 0.026803188662343366, 0.028108457093017715]}	2020-10-01 10:07:00-04:00	2020-10-12 19:51:49.686404	8.76	10.00	\N
138	11	BLDP	sell	148	15.87	{"avg": 15.563174341468862, "rsi": [74.26896367241903, 74.26896367241902, 79.71899676528392], "vwap": 15.8266, "reasons": "rsi max, cool-down for 5 minutes", "movement": 0.015907665126045037, "sell_macd": [0.06575011875989567, 0.06472619083477937, 0.06412668607709371, 0.06282672540776524, 0.06393809728351307], "sell_macd_signal": [0.059524906065063124, 0.06056516301900637, 0.06127746763062384, 0.06158731918605212, 0.06205747480554431]}	2020-10-01 10:08:00-04:00	2020-10-12 19:51:51.097507	14.95	17.64	\N
139	11	VCEL	buy	166	18.71	{"avg": 18.55873378123425, "macd": [0.009564904138880337, 0.01563815542910163, 0.02221249013877369, 0.027907909183230828, 0.034189984968111986], "vwap": 18.7012, "macd_signal": [0.0029504006934698854, 0.0054879516405962345, 0.008832859340231727, 0.012647869308831546, 0.016956292440687633]}	2020-10-01 10:08:00-04:00	2020-10-12 19:51:54.069195	18.11	20.52	\N
140	11	ARAY	buy	1176	2.47	{"avg": 2.4151319172910033, "macd": [0.006943683954747648, 0.007166140473674254, 0.008056485148347114, 0.008422920604021389, 0.009651063247594571], "vwap": 2.466666666666667, "macd_signal": [0.007485847655672173, 0.0074219062192725895, 0.007548822005087495, 0.007723641724874273, 0.008109126029418334]}	2020-10-01 10:12:00-04:00	2020-10-12 19:52:11.049339	2.39	2.73	\N
141	11	CCCL	buy	331	2.61	{"avg": 2.395773126211695, "macd": [0.04604491590965187, 0.04313019389274775, 0.043944811185569144, 0.04559791805389857, 0.056424734910237806], "vwap": 2.5852, "macd_signal": [0.049017783846918773, 0.04784026585608457, 0.04706117492198149, 0.0467685235483649, 0.04869976582073948]}	2020-10-01 10:12:00-04:00	2020-10-12 19:52:11.649639	2.31	3.52	\N
142	11	IVC	buy	229	7.78	{"avg": 7.650839201497038, "macd": [0.021372928370743338, 0.0222926309064837, 0.02834319407942587, 0.03196294364136687, 0.04081644607717383], "vwap": 7.773333333333333, "macd_signal": [0.015086756696199768, 0.016527931538256553, 0.018890984046490415, 0.021505375965465708, 0.025367589987807332]}	2020-10-01 10:12:00-04:00	2020-10-12 19:52:12.781464	7.34	9.09	\N
143	11	ADES	buy	125	6.11	{"avg": 5.76587695821305, "macd": [0.15391715043184195, 0.15931642182807693, 0.1696444387701055, 0.18020633878143766, 0.1868744218241467], "vwap": 6.118333333333333, "macd_signal": [0.15879015935052093, 0.15889541184603212, 0.1610452172308468, 0.16487744154096498, 0.16927683759760132]}	2020-10-01 10:13:00-04:00	2020-10-12 19:52:14.896209	5.31	8.50	\N
144	11	NEXA	buy	308	5.48	{"avg": 5.298357117971114, "macd": [0.0030603524818735295, 0.0015528890997549283, -0.004432193482815094, -0.0034868058368759947, 0.012450313325462758], "vwap": 5.426666666666667, "macd_signal": [0.005160114557061174, 0.004438669465599924, 0.0026644968759169203, 0.0014342363333583372, 0.0036374517317792213]}	2020-10-01 10:14:00-04:00	2020-10-12 19:52:21.366557	5.16	6.45	\N
145	11	VRA	buy	348	6.36	{"avg": 6.265106301494539, "macd": [-0.0012208403057707429, -0.0003368857790775337, 0.0027526727336191215, 0.005532780710990615, 0.012442178348977428], "vwap": 6.345, "macd_signal": [0.012835875570807861, 0.010201323300830782, 0.00871159318738845, 0.008075830692108883, 0.008949100223482592]}	2020-10-01 10:14:00-04:00	2020-10-12 19:52:22.552489	6.07	7.22	\N
146	11	KSS	buy	148	19.12	{"avg": 18.8835409578788, "macd": [0.009597798468174545, 0.012349559125869547, 0.015146527628328244, 0.022757297507709495, 0.032449412968674096], "vwap": 19.11, "macd_signal": [0.022581936046936858, 0.020535460662723395, 0.019457674055844365, 0.02011759874621739, 0.02258396159070873]}	2020-10-01 10:15:00-04:00	2020-10-12 19:52:25.296602	18.45	21.14	\N
147	11	SCWX	buy	219	11.90	{"avg": 11.684464888312435, "macd": [0.07691427880516599, 0.0721641214142359, 0.06722124458186407, 0.06936319299431482, 0.08022240393860791], "vwap": 11.87, "macd_signal": [0.058184532803723626, 0.06098045052582608, 0.06222860933703368, 0.06365552606848991, 0.06696890164251351]}	2020-10-01 10:15:00-04:00	2020-10-12 19:52:26.297764	11.45	13.27	\N
148	11	JWN	buy	185	12.44	{"avg": 12.200148222971492, "macd": [0.005152556872983993, 0.00944479794254427, 0.01389661304897949, 0.01960334439391964, 0.029443053061392277], "vwap": 12.416633333333332, "macd_signal": [0.01767996830507853, 0.016032934232571676, 0.01560566999585324, 0.01640520487546652, 0.01901277451265167]}	2020-10-01 10:15:00-04:00	2020-10-12 19:52:27.306383	11.90	14.05	\N
149	11	FLY	buy	357	7.48	{"avg": 7.364073619178961, "macd": [0.014190495545344461, 0.018179916482290892, 0.020699492242207462, 0.02243762718525133, 0.02633573715846005], "vwap": 7.476666666666667, "macd_signal": [0.021817670673908503, 0.02109011983558498, 0.021011994316909477, 0.021297120890577847, 0.022304844144154286]}	2020-10-01 10:17:00-04:00	2020-10-12 19:52:33.569957	7.20	8.32	\N
150	11	CHEF	buy	168	15.17	{"avg": 14.964009906864737, "macd": [0.018974286909061888, 0.022327430180840935, 0.027492115876041723, 0.03561268596913614, 0.0463554381962723], "vwap": 15.17, "macd_signal": [0.02341436613836013, 0.02319697894685629, 0.024056006332693378, 0.02636734225998193, 0.030364961447240005]}	2020-10-01 10:17:00-04:00	2020-10-12 19:52:36.01342	14.58	16.95	\N
151	11	CUE	buy	158	15.51	{"avg": 15.284494416613143, "macd": [0.04364171264767869, 0.04083534933008082, 0.04056443391794495, 0.043080788692847705, 0.05493171293309551], "vwap": 15.520000000000001, "macd_signal": [0.05163171958588373, 0.049472445534723145, 0.047690843211367503, 0.046768832307663544, 0.04840140843274994]}	2020-10-01 10:18:00-04:00	2020-10-12 19:52:37.617916	14.88	17.40	\N
152	11	LAC	buy	153	12.63	{"avg": 12.254879699289749, "macd": [0.08780807543423208, 0.09490795280211728, 0.09461061509544955, 0.10446756572304139, 0.11817922144867055], "vwap": 12.609966666666665, "macd_signal": [0.08996778754496801, 0.09095582059639787, 0.09168677949620821, 0.09424293674157484, 0.09903019368299398]}	2020-10-01 10:18:00-04:00	2020-10-12 19:52:38.916481	11.98	14.59	\N
153	11	ECOM	buy	193	14.91	{"avg": 14.800629617648676, "macd": [0.028056592996451357, 0.02513360271893461, 0.022557087798157127, 0.02905631869728431, 0.03860350736529128], "vwap": 14.906666666666666, "macd_signal": [0.041856798316256515, 0.038512159196792135, 0.03532114491706513, 0.03406817967310897, 0.03497524521154543]}	2020-10-01 10:19:00-04:00	2020-10-12 19:52:42.621911	14.39	16.46	\N
154	11	TVTY	buy	217	14.47	{"avg": 14.301838043078316, "macd": [0.0321606731016999, 0.028365624786557575, 0.025866762148920586, 0.027602789858477195, 0.035827851000057365], "vwap": 14.456666666666669, "macd_signal": [0.03475281105024529, 0.03347537379750774, 0.03195365146779031, 0.03108347914592769, 0.032032353516753626]}	2020-10-01 10:20:00-04:00	2020-10-12 19:52:49.785151	14.01	15.85	\N
155	11	ANGI	buy	249	11.82	{"avg": 11.671258006245717, "macd": [0.016699243812412945, 0.02009222020538637, 0.023319284670474616, 0.027975027089896187, 0.032101605599814675], "vwap": 11.816666666666668, "macd_signal": [0.02187925642493586, 0.02152184918102596, 0.02188133627891569, 0.02310007444111179, 0.024900380672852367]}	2020-10-01 10:21:00-04:00	2020-10-12 19:52:50.987391	11.42	13.02	\N
156	11	PACB	buy	243	10.50	{"avg": 10.284754247906474, "macd": [0.04210549755194393, 0.047003782616519274, 0.05070467510168264, 0.05262754328690633, 0.05473090105838452], "vwap": 10.496033333333333, "macd_signal": [0.0418690140611966, 0.04289596777226114, 0.04445770923814544, 0.046091676047897615, 0.047819521049995]}	2020-10-01 10:21:00-04:00	2020-10-12 19:52:55.104936	10.09	11.73	\N
157	11	FLDM	buy	327	7.78	{"avg": 7.668683391727555, "macd": [0.007827524095096017, 0.008285560005821146, 0.010943160106750227, 0.011384947535036716, 0.018701049394405977], "vwap": 7.756666666666667, "macd_signal": [0.010013222040576478, 0.009667689633625413, 0.009922783728250376, 0.010215216489607644, 0.011912383070567311]}	2020-10-01 10:22:00-04:00	2020-10-12 19:52:57.031094	7.47	8.70	\N
158	11	DESP	buy	307	6.75	{"avg": 6.589233789364616, "macd": [0.0021657216561559167, 0.012395619562190596, 0.01787607382404488, 0.02356160267508045, 0.03014073094668923], "vwap": 6.746666666666667, "macd_signal": [0.011715992161412676, 0.01185191764156826, 0.013056748878063584, 0.015157719637466958, 0.01815432189931141]}	2020-10-01 10:23:00-04:00	2020-10-12 19:53:01.542329	6.43	7.72	\N
159	11	MOMO	buy	196	14.14	{"avg": 14.000408787884544, "macd": [0.007897400210294947, 0.007792885096714386, 0.01161079621745209, 0.018857184745465716, 0.025915099578007528], "vwap": 14.1395, "macd_signal": [0.0058698390035728755, 0.006254448222201178, 0.00732571782125136, 0.009632011206094231, 0.01288862888047689]}	2020-10-01 10:24:00-04:00	2020-10-12 19:53:07.397052	13.63	15.66	\N
160	11	OCUL	buy	321	8.05	{"avg": 7.888001197897813, "macd": [0.030273372761842055, 0.03175630158954057, 0.03056194324554884, 0.03188645516127586, 0.03513743794241453], "vwap": 8.042433333333333, "macd_signal": [0.027453489675486227, 0.028314052058297096, 0.028763630295747446, 0.02938819526885313, 0.03053804380356541]}	2020-10-01 10:24:00-04:00	2020-10-12 19:53:07.782543	7.74	8.98	\N
161	11	CCCL	sell	331	2.30	{"avg": 2.451177530134921, "rsi": [36.88913124904659, 37.59974437390433, 34.33437228321034], "vwap": 2.3000333333333334, "reasons": "stopped", "movement": -0.11877394636015329, "sell_macd": [0.013789153144388777, 0.006973946777176554, -0.0012895120385363334, -0.007446888881448732, -0.014224322496513508], "sell_macd_signal": [0.025774320150111725, 0.02201424547552469, 0.017353493972712485, 0.012393417401880241, 0.007069869422201491]}	2020-10-01 10:27:00-04:00	2020-10-12 19:53:21.335386	2.31	3.52	\N
162	11	OUT	buy	177	15.22	{"avg": 15.044552165683909, "macd": [0.02474417407969831, 0.030876928932906722, 0.03493105961353393, 0.037709298071421316, 0.04145054702764206], "vwap": 15.219999999999999, "macd_signal": [0.020825679998113255, 0.02283592978507195, 0.025254955750764346, 0.02774582421489574, 0.030486768777445]}	2020-10-01 10:27:00-04:00	2020-10-12 19:53:23.557774	14.66	16.91	\N
163	11	MTEM	buy	266	11.13	{"avg": 11.043978307694609, "macd": [0.00862435169555198, 0.012097800983985607, 0.01547901850644351, 0.01795172148265678, 0.021279887957096122], "vwap": 11.133333333333335, "macd_signal": [0.0093356253409364, 0.009888060469546243, 0.011006252076925696, 0.012395345958071912, 0.014172254357876755]}	2020-10-01 10:28:00-04:00	2020-10-12 19:53:27.626694	10.75	12.26	\N
164	11	IVC	sell	229	7.95	{"avg": 7.72763212771254, "rsi": [67.70872970710835, 67.70872970710835, 69.97384996023153], "vwap": 7.941666666666666, "reasons": "bail", "movement": 0.02185089974293058, "sell_macd": [0.042186108793154986, 0.04034763784146911, 0.04020924959546157, 0.03957921655187757, 0.039440066779147465], "sell_macd_signal": [0.04624056737553771, 0.04506198146872399, 0.044091435094071506, 0.043188991385632716, 0.04243920646433567]}	2020-10-01 10:34:00-04:00	2020-10-12 19:53:39.05798	7.34	9.09	\N
165	11	FLDM	sell	327	8.03	{"avg": 7.756141924145821, "rsi": [76.014934721351, 78.8933783441244, 80.38242403445165], "vwap": 8.029666666666666, "reasons": "rsi max, cool-down for 5 minutes", "movement": 0.032133676092544874, "sell_macd": [0.040546595330929414, 0.04166644250651608, 0.04251740858481057, 0.04459196126160059, 0.04676531017085317], "sell_macd_signal": [0.03269330462965025, 0.034487932205023414, 0.03609382748098085, 0.0377934542371048, 0.03958782542385447]}	2020-10-01 10:40:00-04:00	2020-10-12 19:53:45.802837	7.47	8.70	\N
166	11	NEXA	sell	308	5.80	{"avg": 5.314467710681301, "rsi": [73.53591987435942, 75.98819476584188, 80.3108180233873], "vwap": 5.786666666666666, "reasons": "rsi max, cool-down for 5 minutes", "movement": 0.05839416058394149, "sell_macd": [0.04226879429263697, 0.04574499744916771, 0.050976519422450295, 0.05698839992861249, 0.06595451807286956], "sell_macd_signal": [0.024237844214652207, 0.02853927486155531, 0.03302672377373431, 0.03781905900470994, 0.04344615081834187]}	2020-10-01 10:40:00-04:00	2020-10-12 19:53:46.246718	5.16	6.45	\N
167	11	KSS	sell	148	19.17	{"avg": 18.93598868579368, "rsi": [63.12964665835696, 58.524556910656855, 59.06057556676776], "vwap": 19.16, "reasons": "bail post whipsawed", "movement": 0.0026150627615063133, "sell_macd": [0.030909182772486332, 0.03099167866507102, 0.031172952364940443, 0.02935091950295643, 0.027290232603970566], "sell_macd_signal": [0.025064592832881274, 0.026250009999319222, 0.027234598472443465, 0.027657862678546058, 0.02758433666363096]}	2020-10-01 10:43:00-04:00	2020-10-12 19:53:49.635051	18.45	21.14	\N
168	11	VCEL	sell	166	19.10	{"avg": 18.65389561391351, "rsi": [76.56651425039928, 77.09395055749482, 79.09568000014579], "vwap": 19.1, "reasons": "rsi max, cool-down for 5 minutes", "movement": 0.020746267061426543, "sell_macd": [0.057158617411218415, 0.060587617266815386, 0.06325193381316652, 0.06503124626729573, 0.06764588913717162], "sell_macd_signal": [0.046616871943493586, 0.049411021008157946, 0.05217920356915966, 0.05474961210878687, 0.057328867514463824]}	2020-10-01 10:46:00-04:00	2020-10-12 19:53:53.522435	18.11	20.52	\N
169	11	ADES	sell	125	5.50	{"avg": 5.81283356994255, "rsi": [42.68242967107159, 39.03033745368091, 35.73728366498145], "vwap": 5.5000333333333336, "reasons": "below cost & macd negative & RSI trending down and too far from VWAP", "movement": -0.09909909909909916, "sell_macd": [-0.02596743876939911, -0.024367602871947724, -0.02384347616131155, -0.02520333228079874, -0.02801183489839776], "sell_macd_signal": [-0.027741402057408635, -0.027066642220316454, -0.026422009008515473, -0.026178273662972127, -0.026544985910057252]}	2020-10-01 10:48:00-04:00	2020-10-12 19:53:55.004003	5.31	8.50	\N
170	11	AGTC	sell	357	5.00	{"avg": 4.9586487277297895, "rsi": [53.14444714194673, 55.58531951253066, 55.58531951253066], "vwap": 5.0, "reasons": "bail post whipsawed", "movement": 0.004016064257028026, "sell_macd": [0.005471070107359033, 0.005693205432164561, 0.004992360160167131, 0.0046411513747113275, 0.0043072074980612385], "sell_macd_signal": [0.005450396372482123, 0.0054989581844186106, 0.005397638579568315, 0.005246341138596918, 0.005058514410489782]}	2020-10-01 11:06:00-04:00	2020-10-12 19:54:15.170677	4.70	5.82	\N
171	11	LAC	sell	153	12.68	{"avg": 12.352589597073738, "rsi": [61.32119257150981, 58.66679763253387, 53.72811704156203], "vwap": 12.698266666666669, "reasons": "bail post whipsawed", "movement": 0.0035629453681710155, "sell_macd": [0.0657374197153775, 0.0672213076929502, 0.06545782679634549, 0.06138942834709127, 0.054994276806366926], "sell_macd_signal": [0.05753941293185064, 0.05947579188407055, 0.06067219886652554, 0.06081564476263869, 0.05965137117138434]}	2020-10-01 11:08:00-04:00	2020-10-12 19:54:18.004772	11.98	14.59	\N
172	11	ANGI	sell	249	12.09	{"avg": 11.831103810346855, "rsi": [68.20142991660116, 61.16299856454839, 63.59088545391969], "vwap": 12.086666666666666, "reasons": "bail", "movement": 0.022842639593908594, "sell_macd": [0.028596526528502864, 0.028235537057756943, 0.02758755127460688, 0.025425646102352317, 0.024016902079354807], "sell_macd_signal": [0.02540235697517105, 0.025968992991688228, 0.026292704648271957, 0.02611929293908803, 0.025698814767141386]}	2020-10-01 11:12:00-04:00	2020-10-12 19:54:22.124918	11.42	13.02	\N
173	11	FLY	sell	357	7.68	{"avg": 7.501070490931919, "rsi": [64.09843516501078, 59.32101807167517, 56.293653967265925], "vwap": 7.675699999999999, "reasons": "bail", "movement": 0.026283422459892944, "sell_macd": [0.026819383235638483, 0.026866986832316364, 0.0260356781176192, 0.02401081836438035, 0.02137831961686487], "sell_macd_signal": [0.021231545286778505, 0.022358633595886076, 0.0230940425002327, 0.02327739767306223, 0.02289758206182276]}	2020-10-01 11:31:00-04:00	2020-10-12 19:54:43.522949	7.20	8.32	\N
174	11	MOMO	sell	196	14.59	{"avg": 14.154357440748534, "rsi": [76.64372364373034, 78.12460891055747, 81.21401026652266], "vwap": 14.57, "reasons": "rsi max, cool-down for 5 minutes", "movement": 0.03182461103253177, "sell_macd": [0.020807354742203188, 0.02198955884688658, 0.024526476865512237, 0.026922432764889592, 0.029177431741484128], "sell_macd_signal": [0.011970821907545675, 0.013974569295413856, 0.01608495080943353, 0.018252447200524744, 0.02043744410871662]}	2020-10-01 11:59:00-04:00	2020-10-12 19:55:14.378715	13.63	15.66	\N
175	11	VRA	sell	348	6.37	{"avg": 6.303763906646326, "rsi": [53.18435313634032, 59.80141214592001, 51.90124249983038], "vwap": 6.378333333333334, "reasons": "bail post whipsawed", "movement": 0.0015723270440251237, "sell_macd": [0.008707002677565967, 0.008781145166166127, 0.007685916648632052, 0.00777163576940243, 0.007131438948754898], "sell_macd_signal": [0.00854183364551526, 0.008589695949645433, 0.008408940089442757, 0.008281479225434691, 0.008051471170098733]}	2020-10-01 11:59:00-04:00	2020-10-12 19:55:14.897395	6.07	7.22	\N
176	11	CHEF	sell	168	15.28	{"avg": 15.069180399008548, "rsi": [65.61252685216387, 65.61252685216387, 62.80760171370895], "vwap": 15.281666666666666, "reasons": "bail post whipsawed", "movement": 0.00725115359261697, "sell_macd": [0.0306666977685488, 0.029872617877710894, 0.028865898250074906, 0.02770655960191526, 0.02601045295483928], "sell_macd_signal": [0.026246266541189375, 0.02697153680849368, 0.027350409096809924, 0.027421639197830992, 0.02713940194923265]}	2020-10-01 12:11:00-04:00	2020-10-12 19:55:28.090637	14.58	16.95	\N
177	11	KIRK	sell	179	8.84	{"avg": 8.632546476335934, "rsi": [72.32080454735332, 73.41452739062369, 79.51618018066475], "vwap": 8.8397, "reasons": "rsi max, cool-down for 5 minutes", "movement": -0.0045045045045046085, "sell_macd": [0.018690439442730522, 0.019841133571389946, 0.022558023365522928, 0.0249015995250339, 0.030025144590563713], "sell_macd_signal": [0.013293145494634149, 0.014602743109985309, 0.016193799161092832, 0.017935359233881044, 0.020353316305217577]}	2020-10-01 12:21:00-04:00	2020-10-12 19:55:38.63032	8.32	10.55	\N
178	11	QTNT	sell	305	5.61	{"avg": 5.376183127040583, "rsi": [76.80783743487117, 77.27149475652722, 80.17872052439773], "vwap": 5.601666666666667, "reasons": "rsi max, cool-down for 5 minutes", "movement": 0.04081632653061237, "sell_macd": [0.01693183027843581, 0.01888721459640408, 0.020425342446815264, 0.02152485320756714, 0.022801638390738788], "sell_macd_signal": [0.012119794844515752, 0.013473278794893417, 0.014863691525277787, 0.016195923861735656, 0.017517066767536282]}	2020-10-01 12:23:00-04:00	2020-10-12 19:55:41.173733	5.06	6.37	\N
179	11	TBPH	sell	150	14.64	{"avg": 15.04718703883953, "rsi": [32.53777123197021, 30.921758562885373, 29.341842497023546], "vwap": 14.64, "reasons": "stopped", "movement": -0.04376224689745264, "sell_macd": [-0.029169326602223933, -0.029639075385460956, -0.02962538514661439, -0.030003998140413657, -0.030688579718173514], "sell_macd_signal": [-0.03465163731310654, -0.03364912492757742, -0.032844376971384814, -0.03227630120519058, -0.03195875690778717]}	2020-10-01 12:28:00-04:00	2020-10-12 19:55:46.833378	14.64	17.31	\N
180	11	GNK	sell	473	7.12	{"avg": 7.0137715390496345, "rsi": [74.41035859404104, 77.78186557592736, 79.25368812829248], "vwap": 7.12, "reasons": "rsi max, cool-down for 5 minutes", "movement": 0.007072135785007047, "sell_macd": [0.015982355280192095, 0.01720021226931312, 0.01792581236123514, 0.019297373311287558, 0.020634987385208525], "sell_macd_signal": [0.010047572708857328, 0.011478100620948487, 0.012767642969005817, 0.014073589037462166, 0.015385868707011438]}	2020-10-01 12:37:00-04:00	2020-10-12 19:55:56.130647	6.86	7.70	\N
181	11	JWN	sell	185	12.44	{"avg": 12.277491867154422, "rsi": [64.89955258928096, 56.549868719384634, 57.72142550771794], "vwap": 12.441666666666668, "reasons": "bail post whipsawed", "movement": 0.000008038649828356091, "sell_macd": [0.011431628598169041, 0.012380283967786454, 0.012958713035439828, 0.011942440711074198, 0.011341763912289693], "sell_macd_signal": [0.011320013397113318, 0.011532067511247945, 0.011817396616086322, 0.011842405435083896, 0.011742277130525055]}	2020-10-01 12:47:00-04:00	2020-10-12 19:56:07.626046	11.90	14.05	\N
182	11	MTEM	sell	266	11.19	{"avg": 11.061192005335803, "rsi": [68.62888127326801, 64.46924897176329, 64.4692489717633], "vwap": 11.19, "reasons": "bail post whipsawed", "movement": 0.00539083557951471, "sell_macd": [0.021417163956114038, 0.020687202073181865, 0.020369216163453885, 0.019337419140763146, 0.018282286147227822], "sell_macd_signal": [0.017222158921232064, 0.017915167551622025, 0.0184059772739884, 0.01859226564734335, 0.018530269747320242]}	2020-10-01 12:50:00-04:00	2020-10-12 19:56:10.635646	10.75	12.26	\N
183	11	SYRS	sell	323	8.75	{"avg": 8.956669273171702, "rsi": [35.75355678286559, 30.233276812106347, 12.11039340624242], "vwap": 8.78, "reasons": "stopped", "movement": -0.03508965395558095, "sell_macd": [-0.004694654033643886, -0.004745718082576289, -0.004723875548204859, -0.005164986877939981, -0.008558515357240282], "sell_macd_signal": [-0.005385275165352506, -0.005257363748797263, -0.005150666108678782, -0.005153530262531022, -0.005834527281472874]}	2020-10-01 12:51:00-04:00	2020-10-12 19:56:12.337584	8.76	10.00	\N
184	11	ASMB	sell	133	17.03	{"avg": 16.666682374472284, "rsi": [63.08590060973223, 57.57554229887318, 60.08372237002308], "vwap": 17.0334, "reasons": "bail post whipsawed", "movement": 0.0037360047142016182, "sell_macd": [0.03934157093718582, 0.04060027676934652, 0.041053810187012374, 0.03827664388666108, 0.03735058386188328], "sell_macd_signal": [0.03769968293574916, 0.03827980170246863, 0.038834603399377374, 0.03872301149683412, 0.03844852596984395]}	2020-10-01 12:52:00-04:00	2020-10-12 19:56:12.655127	16.22	19.22	\N
185	11	PACB	sell	243	10.08	{"avg": 10.33097755560022, "rsi": [41.34308587332204, 34.420660427145684, 32.26782190308487], "vwap": 10.089266666666667, "reasons": "stopped", "movement": -0.039628571428571444, "sell_macd": [-0.0045126666224479806, -0.004576272469968501, -0.0050858838775180715, -0.006975374794080835, -0.00865495159321128], "sell_macd_signal": [-0.006613688692448799, -0.006206205447952739, -0.0059821411338658054, -0.006180787865908812, -0.006675620611369305]}	2020-10-01 12:52:00-04:00	2020-10-12 19:56:13.722411	10.09	11.73	\N
186	11	CBAY	sell	271	7.03	{"avg": 7.3172914257040444, "rsi": [40.104871665296955, 40.065716420194576, 30.45833701494332], "vwap": 7.040666666666667, "reasons": "stopped", "movement": -0.05126924789808231, "sell_macd": [-0.005379396957505556, -0.00551956969841072, -0.005692177422512934, -0.005757947370871541, -0.0067387471483257855], "sell_macd_signal": [-0.004013961503109665, -0.004315083142169876, -0.004590501998238487, -0.0048239910727650975, -0.005206942287877235]}	2020-10-01 13:07:00-04:00	2020-10-12 19:56:28.858453	7.04	8.51	\N
187	11	ARAY	sell	588	2.57	{"avg": 2.4728447051997167, "rsi": [69.67468555831083, 74.12592148604276, 77.6576606981675], "vwap": 2.565, "reasons": "scale-out", "movement": 0.040485829959514025, "sell_macd": [0.0031944315314911265, 0.003467386121323912, 0.004154527540397357, 0.005155488028760757, 0.006128243110228748], "sell_macd_signal": [0.0022029777037704903, 0.0024558593872811745, 0.002795593017904411, 0.00326757202007568, 0.003839706238106294]}	2020-10-01 13:13:00-04:00	2020-10-12 19:56:35.260345	2.39	2.73	\N
188	11	ARAY	sell	588	2.58	{"avg": 2.4746769995245947, "rsi": [77.6576606981675, 77.65766069816749, 80.7111684249709], "vwap": 2.5772666666666666, "reasons": "rsi max, cool-down for 5 minutes", "movement": 0.003891050583657678, "sell_macd": [0.004154527540397357, 0.005155488028760757, 0.0063879833699695965, 0.007265408527145745, 0.008232249664419378], "sell_macd_signal": [0.002795593017904411, 0.00326757202007568, 0.0038916542900544636, 0.00456640513747272, 0.005299574042862051]}	2020-10-01 13:15:00-04:00	2020-10-12 19:56:37.509078	2.39	2.73	\N
189	11	MBWM	sell	109	18.36	{"avg": 18.166894132900072, "rsi": [66.17242951164516, 60.014452757247675, 57.60885159625311], "vwap": 18.365, "reasons": "bail post whipsawed", "movement": 0.00464989059080948, "sell_macd": [0.05652514494806482, 0.06045290512023627, 0.060650472720364945, 0.05689986990591578, 0.05193966161016661], "sell_macd_signal": [0.04878699178883285, 0.051120174455113535, 0.05302623410816382, 0.053800961267714206, 0.053428701336204684]}	2020-10-01 13:49:00-04:00	2020-10-12 19:57:15.834289	17.36	21.03	\N
190	11	ECOM	sell	193	15.21	{"avg": 14.949603530300616, "rsi": [69.55658058903495, 65.75964419561056, 50.8214835844111], "vwap": 15.223333333333334, "reasons": "bail", "movement": 0.020120724346076507, "sell_macd": [0.016746954491068777, 0.01724101810537526, 0.01792157393333582, 0.017699648943001023, 0.015392092903654842], "sell_macd_signal": [0.017422360722613637, 0.01738609219916596, 0.017493188545999933, 0.01753448062540015, 0.017106003081051088]}	2020-10-01 13:59:00-04:00	2020-10-12 19:57:27.212529	14.39	16.46	\N
191	11	MGI	sell	282	2.97	{"avg": 2.8746440587892215, "rsi": [64.92266423118085, 69.19742291066353, 71.79828228672012], "vwap": 2.966666666666667, "reasons": "scale-out", "movement": 0.04210526315789477, "sell_macd": [0.009893969167867223, 0.01013511532644129, 0.009729058887324449, 0.00986292323493032, 0.010071177836074252], "sell_macd_signal": [0.008457028207630174, 0.008792645631392397, 0.008979928282578807, 0.00915652727304911, 0.00933945738565414]}	2020-10-01 14:53:00-04:00	2020-10-12 19:58:27.729758	2.67	3.38	\N
192	11	MGI	sell	283	2.95	{"avg": 2.8752064950424328, "rsi": [64.30202399894316, 67.91015868444144, 56.272321790483815], "vwap": 2.955666666666667, "reasons": "bail", "movement": -0.00639730639730644, "sell_macd": [0.00986292323493032, 0.010244338009234522, 0.009889020456624653, 0.010003201318790289, 0.009217672969151547], "sell_macd_signal": [0.00915652727304911, 0.009374089420286193, 0.009477075627553885, 0.009582300765801166, 0.009509375206471242]}	2020-10-01 14:57:00-04:00	2020-10-12 19:58:32.129959	2.67	3.38	\N
193	11	ARDX	sell	420	5.45	{"avg": 5.404138888008098, "rsi": [65.59269508361957, 65.59269508361957, 65.59269508361959], "vwap": 5.45, "reasons": "bail post whipsawed", "movement": 0.0073937153419593405, "sell_macd": [0.009817262794307169, 0.009542649858128804, 0.009204735234377814, 0.008821883121917118, 0.008408986889844705], "sell_macd_signal": [0.009564019673064455, 0.009559745710077325, 0.009488743614937423, 0.009355371516333363, 0.009166094591035632]}	2020-10-01 14:59:00-04:00	2020-10-12 19:58:33.985811	5.17	6.12	\N
194	11	VECO	sell	258	12.17	{"avg": 12.005730388819085, "rsi": [69.96629367850102, 57.66299193798255, 51.199173074818674], "vwap": 12.17, "reasons": "bail", "movement": 0.024410774410774338, "sell_macd": [0.01624661024587226, 0.016292790827389325, 0.016636661529515706, 0.015130110628192028, 0.012720718878741266], "sell_macd_signal": [0.012034039371930965, 0.012885789663022637, 0.01363596403632125, 0.013934793354695406, 0.013691978459504577]}	2020-10-01 15:00:00-04:00	2020-10-12 19:58:36.285445	11.49	13.04	\N
195	11	CGEN	sell	155	17.06	{"avg": 16.798220798578654, "rsi": [57.79819459858435, 50.72124420419757, 50.72124420419757], "vwap": 17.06, "reasons": "bail", "movement": 0.020945541591861034, "sell_macd": [0.007292926425733981, 0.00754802477760208, 0.008687737457268696, 0.007385075422497778, 0.006274104026282146], "sell_macd_signal": [0.0057075371072173985, 0.006075634641294335, 0.006598055204489207, 0.006755459248090921, 0.006659188203729166]}	2020-10-01 15:08:00-04:00	2020-10-12 19:58:44.042558	16.07	18.64	\N
196	11	CUE	sell	158	15.69	{"avg": 15.418669182892296, "rsi": [64.37124145272932, 58.639546122514716, 59.032386895684084], "vwap": 15.685, "reasons": "bail post whipsawed", "movement": 0.011283043197936861, "sell_macd": [0.038629328238652505, 0.03825705653085443, 0.03980779212827379, 0.037901770618102404, 0.03618406852422673], "sell_macd_signal": [0.035141814934801376, 0.035764863254011986, 0.03657344902886435, 0.03683911334671196, 0.03670810438221491]}	2020-10-01 15:08:00-04:00	2020-10-12 19:58:44.259372	14.88	17.40	\N
197	11	TVTY	sell	217	14.54	{"avg": 14.325678849266344, "rsi": [68.25096131922818, 78.24986156368736, 79.03974041599461], "vwap": 14.54, "reasons": "rsi max, cool-down for 5 minutes", "movement": 0.004837595024187871, "sell_macd": [0.021837002626217128, 0.021699287141958123, 0.02182964797056286, 0.02632305059063711, 0.03000202600120616], "sell_macd_signal": [0.02373152817017542, 0.02332507996453196, 0.02302599356573814, 0.023685404970717933, 0.02494872917681558]}	2020-10-01 15:14:00-04:00	2020-10-12 19:58:51.819036	14.01	15.85	\N
198	11	DESP	sell	307	6.76	{"avg": 6.643861714077856, "rsi": [58.1181416794968, 52.68330373655654, 54.951645550802866], "vwap": 6.756666666666667, "reasons": "bail post whipsawed", "movement": 0.00148148148148145, "sell_macd": [0.011739927357139912, 0.009893168092063753, 0.009884093754445544, 0.008709696949100731, 0.008027789954255127], "sell_macd_signal": [0.013225239047923765, 0.012558824856751763, 0.012023878636290519, 0.01136104229885256, 0.010694391829933075]}	2020-10-01 15:22:00-04:00	2020-10-12 19:59:00.035086	6.43	7.72	\N
199	11	CTMX	sell	375	6.85	{"avg": 6.779683645725906, "rsi": [52.23840743278964, 52.23840743278964, 52.23840743278964], "vwap": 6.849999999999999, "reasons": "bail post whipsawed", "movement": 0.004398826979472047, "sell_macd": [0.006924614122134187, 0.007452270190522192, 0.006727686994583593, 0.00607570415366343, 0.005488760327573949], "sell_macd_signal": [0.00583599697789411, 0.006159251620419726, 0.006272938695252499, 0.006233491786934686, 0.006084545495062538]}	2020-10-01 15:24:00-04:00	2020-10-12 19:59:02.625769	6.55	7.62	\N
200	11	ETH	sell	135	13.77	{"liquidate": 1}	2020-10-01 16:00:00-04:00	2020-10-12 19:59:40.866052	\N	\N	\N
201	11	EVGN	sell	409	4.06	{"liquidate": 1}	2020-10-01 16:00:00-04:00	2020-10-12 19:59:41.055284	\N	\N	\N
202	11	INO	sell	159	11.74	{"liquidate": 1}	2020-10-01 16:00:00-04:00	2020-10-12 19:59:41.245887	\N	\N	\N
203	11	OCUL	sell	321	7.91	{"liquidate": 1}	2020-10-01 16:00:00-04:00	2020-10-12 19:59:41.404036	\N	\N	\N
204	11	OPK	sell	678	3.89	{"liquidate": 1}	2020-10-01 16:00:00-04:00	2020-10-12 19:59:41.564407	\N	\N	\N
205	11	OUT	sell	177	15.22	{"liquidate": 1}	2020-10-01 16:00:00-04:00	2020-10-12 19:59:41.704408	\N	\N	\N
206	11	SCWX	sell	219	11.76	{"liquidate": 1}	2020-10-01 16:00:00-04:00	2020-10-12 19:59:41.851094	\N	\N	\N
\.


--
-- TOC entry 3030 (class 0 OID 16491)
-- Dependencies: 211
-- Data for Name: stock_ohlc; Type: TABLE DATA; Schema: public; Owner: liu
--

COPY public.stock_ohlc (symbol_id, symbol, symbol_date, open, high, low, close, volume, indicators, modify_tstamp, create_tstamp) FROM stdin;
24345	ACM	2020-09-28	40	41.17	39.925	40.91	1448366	{}	\N	2020-10-12 14:08:42.325464
24346	ACM	2020-09-29	40.87	41.645	40.87	41.37	2545727	{}	\N	2020-10-12 14:08:42.387809
24347	ACM	2020-09-30	41.54	42.67	41.4201	41.84	3608563	{}	\N	2020-10-12 14:08:42.396355
24348	ACM	2020-10-01	41.91	42.47	41.68	42.24	2419144	{}	\N	2020-10-12 14:08:42.403332
24349	ACM	2020-10-02	41.41	44.59	41.32	44.05	5489265	{}	\N	2020-10-12 14:08:42.409917
24350	ACM	2020-10-05	44.46	45.139	44.28	44.34	4789247	{}	\N	2020-10-12 14:08:42.417247
24351	ACM	2020-10-06	44.71	45.5	43.8	44.11	3657627	{}	\N	2020-10-12 14:08:42.423518
24352	ACM	2020-10-07	44.75	45.58	44.63	45.26	3081767	{}	\N	2020-10-12 14:08:42.428406
24353	ACM	2020-10-08	45.48	46.18	45.29	46.08	3250851	{}	\N	2020-10-12 14:08:42.434381
24354	ACM	2020-10-09	46.35	46.55	45.89	46.16	2961598	{}	\N	2020-10-12 14:08:42.440591
24355	ACM	2020-10-12	46.05	46.79	45.85	46.65	163205	{}	\N	2020-10-12 14:08:42.447281
24356	ACN	2020-09-28	219.09	223.89	218.3005	222.78	2653489	{}	\N	2020-10-12 14:08:43.929286
24357	ACN	2020-09-29	223.37	225.17	222.31	224.08	4751074	{}	\N	2020-10-12 14:08:43.941017
24358	ACN	2020-09-30	225.27	228.295	224.635	225.99	4090862	{}	\N	2020-10-12 14:08:43.9481
24359	ACN	2020-10-01	227.36	229.24	224.03	225.18	3400990	{}	\N	2020-10-12 14:08:43.954273
24360	ACN	2020-10-02	222.05	224.92	221.35	222.4	3497274	{}	\N	2020-10-12 14:08:43.959464
24361	ACN	2020-10-05	224.37	224.93	222.04	223.49	2888414	{}	\N	2020-10-12 14:08:43.964059
24362	ACN	2020-10-06	223.59	225.27	220.09	220.35	3942119	{}	\N	2020-10-12 14:08:43.968525
24363	ACN	2020-10-07	221.9	224.63	220.05	223.45	3621530	{}	\N	2020-10-12 14:08:43.973032
24364	ACN	2020-10-08	226.04	226.79	223.97	225.61	2742663	{}	\N	2020-10-12 14:08:43.978403
24365	ACN	2020-10-09	225.86	229.09	225.8	228.69	2437010	{}	\N	2020-10-12 14:08:43.985889
24366	ACN	2020-10-12	229.69	230.39	227.95	227.98	424657	{}	\N	2020-10-12 14:08:43.992453
24367	ACTG	2020-09-28	3.45	3.63	3.45	3.51	413826	{}	\N	2020-10-12 14:08:45.974838
24368	ACTG	2020-09-29	3.5	3.6	3.43	3.45	71857	{}	\N	2020-10-12 14:08:45.994491
24369	ACTG	2020-09-30	3.44	3.54	3.44	3.47	41658	{}	\N	2020-10-12 14:08:46.00254
24370	ACTG	2020-10-01	3.49	3.58	3.49	3.52	140490	{}	\N	2020-10-12 14:08:46.008763
24371	ACTG	2020-10-02	3.42	3.4899	3.39	3.41	117782	{}	\N	2020-10-12 14:08:46.014741
24372	ACTG	2020-10-05	3.44	3.53	3.36	3.37	120199	{}	\N	2020-10-12 14:08:46.019877
24373	ACTG	2020-10-06	3.37	3.4	3.33	3.34	130268	{}	\N	2020-10-12 14:08:46.025029
24374	ACTG	2020-10-07	3.37	3.37	3.3	3.325	188449	{}	\N	2020-10-12 14:08:46.030217
24375	ACTG	2020-10-08	3.35	3.36	3.25	3.26	279849	{}	\N	2020-10-12 14:08:46.035025
24376	ACTG	2020-10-09	3.36	3.415	3.35	3.38	162354	{}	\N	2020-10-12 14:08:46.039628
24377	ACTG	2020-10-12	3.4	3.445	3.38	3.38	9599	{}	\N	2020-10-12 14:08:46.044502
24378	ACST	2020-09-28	0.2198	0.221	0.2062	0.208	3102173	{}	\N	2020-10-12 14:08:47.210718
24379	ACST	2020-09-29	0.2188	0.2241	0.2091	0.215	5261797	{}	\N	2020-10-12 14:08:47.220463
24380	ACST	2020-09-30	0.215	0.2151	0.2	0.2002	4044784	{}	\N	2020-10-12 14:08:47.227451
24381	ACST	2020-10-01	0.2049	0.208	0.197	0.19993	3105791	{}	\N	2020-10-12 14:08:47.233421
24382	ACST	2020-10-02	0.196	0.2039	0.196	0.203	3235793	{}	\N	2020-10-12 14:08:47.238574
24383	ACST	2020-10-05	0.209	0.2174	0.198	0.21	4534243	{}	\N	2020-10-12 14:08:47.243399
24384	ACST	2020-10-06	0.2189	0.2189	0.2045	0.2069	2644975	{}	\N	2020-10-12 14:08:47.248016
24385	ACST	2020-10-07	0.2033	0.2069	0.198	0.2017	2933987	{}	\N	2020-10-12 14:08:47.252602
24386	ACST	2020-10-08	0.208	0.208	0.2006	0.20745	1550366	{}	\N	2020-10-12 14:08:47.257111
24387	ACST	2020-10-09	0.2075	0.214	0.2044	0.2045	2407954	{}	\N	2020-10-12 14:08:47.261545
24388	ACST	2020-10-12	0.2112	0.2112	0.2066	0.2067	288189	{}	\N	2020-10-12 14:08:47.266025
24389	ADAP	2020-09-28	8.04	8.125	7.42	7.84	479004	{}	\N	2020-10-12 14:08:51.001965
24390	ADAP	2020-09-29	7.79	8.01	7.5	7.65	322673	{}	\N	2020-10-12 14:08:51.012268
24391	ADAP	2020-09-30	7.63	8.32	7.58	7.98	562059	{}	\N	2020-10-12 14:08:51.019784
24392	ADAP	2020-10-01	8	8.37	7.84	8.36	456526	{}	\N	2020-10-12 14:08:51.026835
24393	ADAP	2020-10-02	8.1	8.68	7.945	8.01	313404	{}	\N	2020-10-12 14:08:51.032751
24394	ADAP	2020-10-05	8.06	8.4899	7.99	8.46	311749	{}	\N	2020-10-12 14:08:51.037786
24395	ADAP	2020-10-06	8.49	8.61	8.25	8.31	291026	{}	\N	2020-10-12 14:08:51.043351
24396	ADAP	2020-10-07	8.39	8.71	8.3	8.66	407066	{}	\N	2020-10-12 14:08:51.048459
24397	ADAP	2020-10-08	8.71	8.8594	8.05	8.14	446951	{}	\N	2020-10-12 14:08:51.053337
24398	ADAP	2020-10-09	8.19	8.42	7.97	8.21	352381	{}	\N	2020-10-12 14:08:51.059388
24399	ADAP	2020-10-12	8.36	8.435	8.1627	8.25	70090	{}	\N	2020-10-12 14:08:51.064901
24400	ADES	2020-09-28	4.02	4.22	4.01	4.15	87059	{}	\N	2020-10-12 14:08:52.396015
24401	ADES	2020-09-29	4.12	4.167	3.96	4	208909	{}	\N	2020-10-12 14:08:52.406394
24402	ADES	2020-09-30	4	4.15	4	4.07	2925717	{}	\N	2020-10-12 14:08:52.413623
24403	ADES	2020-10-01	5.66	6.25	4.81	5.03	13715835	{}	\N	2020-10-12 14:08:52.419242
24404	ADES	2020-10-02	4.9	4.98	4.71	4.81	507674	{}	\N	2020-10-12 14:08:52.424263
24405	ADES	2020-10-05	4.76	4.9	4.5482	4.57	272674	{}	\N	2020-10-12 14:08:52.42928
24406	ADES	2020-10-06	4.5899	4.83	4.45	4.57	192673	{}	\N	2020-10-12 14:08:52.433564
24407	ADES	2020-10-07	4.64	4.85	4.62	4.82	146135	{}	\N	2020-10-12 14:08:52.437765
24408	ADES	2020-10-08	4.79	4.8756	4.69	4.74	100426	{}	\N	2020-10-12 14:08:52.442031
24409	ADES	2020-10-09	4.74	4.79	4.62	4.69	94512	{}	\N	2020-10-12 14:08:52.446393
24410	ADES	2020-10-12	4.72	4.72	4.6544	4.7	6007	{}	\N	2020-10-12 14:08:52.450514
24411	ADRE	2020-09-28	45.81	46.12	45.743	45.9282	3860	{}	\N	2020-10-12 14:08:54.592777
24412	ADRE	2020-09-29	45.68	45.93	45.68	45.855	1405	{}	\N	2020-10-12 14:08:54.60493
24413	ADRE	2020-09-30	46.32	46.99	46.32	46.725	26380	{}	\N	2020-10-12 14:08:54.612369
24414	ADRE	2020-10-01	47.014	47.105	46.9672	47.065	2208	{}	\N	2020-10-12 14:08:54.618782
24415	ADRE	2020-10-02	46.42	46.9879	46.42	46.63	5781	{}	\N	2020-10-12 14:08:54.623599
24416	ADRE	2020-10-05	46.97	47.4605	46.97	47.455	19335	{}	\N	2020-10-12 14:08:54.628441
24417	ADRE	2020-10-06	47.768	47.9311	47.364	47.364	3008	{}	\N	2020-10-12 14:08:54.633209
24418	ADRE	2020-10-07	48.04	48.5	48.04	48.5	17782	{}	\N	2020-10-12 14:08:54.636996
24419	ADRE	2020-10-08	48.66	48.9755	48.655	48.9755	7156	{}	\N	2020-10-12 14:08:54.641248
24420	ADRE	2020-10-09	49.24	49.485	49.1877	49.38	4881	{}	\N	2020-10-12 14:08:54.646437
24421	ADRE	2020-10-12	49.94	49.94	49.825	49.9	1994	{}	\N	2020-10-12 14:08:54.650513
24422	ADTN	2020-09-28	10.07	10.33	10.07	10.21	224085	{}	\N	2020-10-12 14:08:57.853135
24423	ADTN	2020-09-29	10.23	10.49	10.23	10.39	146286	{}	\N	2020-10-12 14:08:57.861896
24424	ADTN	2020-09-30	10.37	10.51	10.17	10.255	187570	{}	\N	2020-10-12 14:08:57.867867
24425	ADTN	2020-10-01	10.33	10.3899	10.13	10.31	125245	{}	\N	2020-10-12 14:08:57.873738
24426	ADTN	2020-10-02	10.09	10.42	10.09	10.35	179829	{}	\N	2020-10-12 14:08:57.879877
24427	ADTN	2020-10-05	10.51	10.8	10.43	10.79	179233	{}	\N	2020-10-12 14:08:57.887784
24428	ADTN	2020-10-06	10.84	10.97	10.51	10.52	205374	{}	\N	2020-10-12 14:08:57.894424
24429	ADTN	2020-10-07	10.61	11.12	10.52	11.02	288705	{}	\N	2020-10-12 14:08:57.901176
24430	ADTN	2020-10-08	11.063	11.2249	10.77	10.97	130986	{}	\N	2020-10-12 14:08:57.906946
24431	ADTN	2020-10-09	11.11	11.27	10.86	10.89	168175	{}	\N	2020-10-12 14:08:57.912517
24432	ADTN	2020-10-12	11.04	11.2545	10.9153	11.06	38524	{}	\N	2020-10-12 14:08:57.918371
24433	ADVM	2020-09-28	10.98	10.98	10.19	10.34	757850	{}	\N	2020-10-12 14:09:00.175066
24434	ADVM	2020-09-29	10.58	10.62	10.16	10.45	684290	{}	\N	2020-10-12 14:09:00.185351
24435	ADVM	2020-09-30	10.485	10.52	10.1	10.3	1253217	{}	\N	2020-10-12 14:09:00.192865
24436	ADVM	2020-10-01	10.6	10.89	10.215	10.52	1196258	{}	\N	2020-10-12 14:09:00.199158
24437	ADVM	2020-10-02	10.25	10.42	9.83	9.84	796456	{}	\N	2020-10-12 14:09:00.204841
24438	ADVM	2020-10-05	9.92	10.93	9.92	10.73	1030800	{}	\N	2020-10-12 14:09:00.209918
24439	ADVM	2020-10-06	10.73	10.86	10.2901	10.36	441134	{}	\N	2020-10-12 14:09:00.21449
24440	ADVM	2020-10-07	10.38	11.17	10.38	10.99	694959	{}	\N	2020-10-12 14:09:00.218985
24441	ADVM	2020-10-08	11.25	11.25	10.78	11.06	569597	{}	\N	2020-10-12 14:09:00.223246
24442	ADVM	2020-10-09	11.22	11.37	11	11.14	546624	{}	\N	2020-10-12 14:09:00.227899
24443	ADVM	2020-10-12	11.28	11.32	10.9	11.04	32619	{}	\N	2020-10-12 14:09:00.232347
24444	AEE	2020-09-28	78.61	79.37	78.08	78.42	661873	{}	\N	2020-10-12 14:09:01.486233
24445	AEE	2020-09-29	78.83	79.36	78.5	79.02	1667198	{}	\N	2020-10-12 14:09:01.49326
24446	AEE	2020-09-30	79.3	79.91	78.53	79.08	2375620	{}	\N	2020-10-12 14:09:01.498064
24447	AEE	2020-10-01	79.17	80.21	79.06	79.86	1968979	{}	\N	2020-10-12 14:09:01.502534
24448	AEE	2020-10-02	79.44	80.715	78.96	79.81	1643443	{}	\N	2020-10-12 14:09:01.508884
24449	AEE	2020-10-05	79.85	81.06	78.82	80.72	2130181	{}	\N	2020-10-12 14:09:01.514297
24450	AEE	2020-10-06	80.71	82.03	79.83	81.3	2234071	{}	\N	2020-10-12 14:09:01.519329
24451	AEE	2020-10-07	81.51	81.8649	80.05	80.69	1828542	{}	\N	2020-10-12 14:09:01.524625
24452	AEE	2020-10-08	81.18	81.75	80.72	81.73	2387147	{}	\N	2020-10-12 14:09:01.53015
24453	AEE	2020-10-09	81.83	82.53	81.32	82.05	1812738	{}	\N	2020-10-12 14:09:01.534709
24454	AEE	2020-10-12	82.11	82.64	82.05	82.54	126573	{}	\N	2020-10-12 14:09:01.539533
24455	MRUS	2020-09-28	12.25	12.29	11.87	12.05	43266	{}	\N	2020-10-12 14:09:04.539625
24456	MRUS	2020-09-29	12.15	12.18	11.715	12.11	27850	{}	\N	2020-10-12 14:09:04.55016
24457	MRUS	2020-09-30	12.1	12.13	11.95	12	23523	{}	\N	2020-10-12 14:09:04.557401
24458	MRUS	2020-10-01	12.36	12.36	11.96	12.31	41544	{}	\N	2020-10-12 14:09:04.563325
24459	MRUS	2020-10-02	12.145	12.16	11.83	12.03	40327	{}	\N	2020-10-12 14:09:04.569152
24460	MRUS	2020-10-05	12.195	12.745	12.015	12.56	25939	{}	\N	2020-10-12 14:09:04.57421
24461	MRUS	2020-10-06	12.54	12.84	12.45	12.57	20899	{}	\N	2020-10-12 14:09:04.579268
24462	MRUS	2020-10-07	12.9	12.94	12.645	12.71	24001	{}	\N	2020-10-12 14:09:04.584184
24463	MRUS	2020-10-08	12.5	12.75	12.44	12.67	9705	{}	\N	2020-10-12 14:09:04.589307
24464	MRUS	2020-10-09	12.89	12.89	12.54	12.7	18404	{}	\N	2020-10-12 14:09:04.593972
24465	MRUS	2020-10-12	12.71	12.88	12.7	12.88	4377	{}	\N	2020-10-12 14:09:04.598865
24466	ACGLO	2020-09-28	26.2	26.2	26.01	26.03	28854	{}	\N	2020-10-12 14:09:06.223734
24467	ACGLO	2020-09-29	26.16	26.2673	25.995	26.14	10684	{}	\N	2020-10-12 14:09:06.234859
24468	ACGLO	2020-09-30	26.13	26.21	25.85	25.86	34621	{}	\N	2020-10-12 14:09:06.240814
24469	ACGLO	2020-10-01	26	26.16	25.946	26.16	16867	{}	\N	2020-10-12 14:09:06.246516
24470	ACGLO	2020-10-02	25.97	26.26	25.97	26.13	14797	{}	\N	2020-10-12 14:09:06.251598
24471	ACGLO	2020-10-05	26.11	26.28	26.0579	26.28	11729	{}	\N	2020-10-12 14:09:06.256321
24472	ACGLO	2020-10-06	26.27	26.28	26.15	26.26	10052	{}	\N	2020-10-12 14:09:06.260545
24473	ACGLO	2020-10-07	26.14	26.41	26.14	26.41	19905	{}	\N	2020-10-12 14:09:06.264324
24474	ACGLO	2020-10-08	26.39	26.44	26.277	26.44	15741	{}	\N	2020-10-12 14:09:06.268583
24475	ACGLO	2020-10-09	26.26	26.47	26.2481	26.47	9553	{}	\N	2020-10-12 14:09:06.273075
24476	ACGLO	2020-10-12	26.37	26.37	26.305	26.305	517	{}	\N	2020-10-12 14:09:06.277125
24477	ACIW	2020-09-28	25.61	25.88	25.52	25.7	616089	{}	\N	2020-10-12 14:09:07.700209
24478	ACIW	2020-09-29	25.63	26.02	25.55	25.66	565519	{}	\N	2020-10-12 14:09:07.712332
24479	ACIW	2020-09-30	25.73	26.18	25.6341	26.13	828958	{}	\N	2020-10-12 14:09:07.719119
24480	ACIW	2020-10-01	26.19	26.885	26.045	26.85	906596	{}	\N	2020-10-12 14:09:07.724336
24481	ACIW	2020-10-02	26.33	27.26	26.105	27.255	821733	{}	\N	2020-10-12 14:09:07.729311
24482	ACIW	2020-10-05	27.46	28.08	27.44	27.95	1578541	{}	\N	2020-10-12 14:09:07.733841
24483	ACIW	2020-10-06	28.05	28.5	27.5	27.62	1486025	{}	\N	2020-10-12 14:09:07.738207
24484	ACIW	2020-10-07	27.94	29.26	27.82	29.22	1414706	{}	\N	2020-10-12 14:09:07.742167
24485	ACIW	2020-10-08	29.2	30.02	28.6975	30.01	1257925	{}	\N	2020-10-12 14:09:07.746242
24486	ACIW	2020-10-09	30.34	30.41	29.79	30.08	1396606	{}	\N	2020-10-12 14:09:07.750251
24487	ACIW	2020-10-12	30.15	30.21	29.93	29.93	27466	{}	\N	2020-10-12 14:09:07.75481
24488	ACLS	2020-09-28	21.87	22.38	21.63	22.28	333298	{}	\N	2020-10-12 14:09:08.969064
24489	ACLS	2020-09-29	22.25	22.465	21.95	22.08	161724	{}	\N	2020-10-12 14:09:08.977329
24490	ACLS	2020-09-30	22.07	22.32	21.84	22	471088	{}	\N	2020-10-12 14:09:08.982709
24491	ACLS	2020-10-01	22.28	23.26	22.28	23.03	289389	{}	\N	2020-10-12 14:09:08.988491
24492	ACLS	2020-10-02	22.26	22.75	22.1	22.25	247259	{}	\N	2020-10-12 14:09:08.993166
24493	ACLS	2020-10-05	22.57	23.29	22.57	23.26	149226	{}	\N	2020-10-12 14:09:08.997673
24494	ACLS	2020-10-06	23.3	23.715	22.86	22.94	286932	{}	\N	2020-10-12 14:09:09.001454
24495	ACLS	2020-10-07	23.16	23.63	23.03	23.13	287690	{}	\N	2020-10-12 14:09:09.005233
24496	ACLS	2020-10-08	23.22	23.595	23.085	23.49	321659	{}	\N	2020-10-12 14:09:09.009327
24497	ACLS	2020-10-09	23.95	23.99	23.55	23.78	232748	{}	\N	2020-10-12 14:09:09.013111
24498	ACLS	2020-10-12	23.94	24.1	23.79	23.9	43724	{}	\N	2020-10-12 14:09:09.017152
24499	ACRS	2020-09-28	2.48	2.63	2.43	2.57	205503	{}	\N	2020-10-12 14:09:10.206887
24500	ACRS	2020-09-29	2.59	2.63	2.44	2.56	260505	{}	\N	2020-10-12 14:09:10.216728
24501	ACRS	2020-09-30	2.56	2.6	2.52	2.57	153024	{}	\N	2020-10-12 14:09:10.222948
24502	ACRS	2020-10-01	2.61	2.66	2.54	2.65	207627	{}	\N	2020-10-12 14:09:10.228184
24503	ACRS	2020-10-02	2.6	3.1399	2.58	2.96	1268552	{}	\N	2020-10-12 14:09:10.233186
24504	ACRS	2020-10-05	2.96	3.4	2.95	3.35	908813	{}	\N	2020-10-12 14:09:10.237927
24505	ACRS	2020-10-06	3.4	3.49	3.02	3.32	637782	{}	\N	2020-10-12 14:09:10.242058
24506	ACRS	2020-10-07	3.39	3.8	3.31	3.75	796104	{}	\N	2020-10-12 14:09:10.258878
24507	ACRS	2020-10-08	3.86	4.28	3.83	4.15	1288166	{}	\N	2020-10-12 14:09:10.266897
24508	ACRS	2020-10-09	4.35	5.04	3.78	4.75	4166463	{}	\N	2020-10-12 14:09:10.273846
24509	ACRS	2020-10-12	4.53	4.59	4.3	4.3991	293420	{}	\N	2020-10-12 14:09:10.281358
24510	ACSI	2020-09-28	37.3533	37.3533	37.3533	37.3533	100	{}	\N	2020-10-12 14:09:12.194702
24511	ACSI	2020-09-29	37.1567	37.1567	37.1567	37.1567	119	{}	\N	2020-10-12 14:09:12.203994
24512	ACSI	2020-09-30	37.4786	37.4786	37.4786	37.4786	100	{}	\N	2020-10-12 14:09:12.210636
24513	ACSI	2020-10-01	37.8999	37.9674	37.8999	37.9674	114	{}	\N	2020-10-12 14:09:12.218509
24514	ACSI	2020-10-02	37.7025	37.7025	37.7025	37.7025	121	{}	\N	2020-10-12 14:09:12.223407
24515	ACSI	2020-10-05	38.2111	38.2111	38.2111	38.2111	298	{}	\N	2020-10-12 14:09:12.228362
24516	ACSI	2020-10-06	37.731	37.731	37.731	37.731	106	{}	\N	2020-10-12 14:09:12.233264
24517	ACSI	2020-10-07	38.5084	38.5084	38.5084	38.5084	110	{}	\N	2020-10-12 14:09:12.237831
24518	ACSI	2020-10-08	38.6492	38.6492	38.6492	38.6492	101	{}	\N	2020-10-12 14:09:12.243471
24519	ACSI	2020-10-09	39.0595	39.0704	39.0595	39.0704	384	{}	\N	2020-10-12 14:09:12.248588
24520	ACWV	2020-09-28	91.34	91.95	91.14	91.76	304244	{}	\N	2020-10-12 14:09:13.557414
24521	ACWV	2020-09-29	91.68	92.04	91.42	91.63	532877	{}	\N	2020-10-12 14:09:13.566611
24522	ACWV	2020-09-30	91.71	92.36	91.5	91.92	433623	{}	\N	2020-10-12 14:09:13.574456
24523	ACWV	2020-10-01	92.29	92.511	91.91	92.41	322927	{}	\N	2020-10-12 14:09:13.580145
24524	ACWV	2020-10-02	91.26	92.3199	91.26	92.08	193631	{}	\N	2020-10-12 14:09:13.585272
24525	ACWV	2020-10-05	92.53	93.12	92.53	93.06	361558	{}	\N	2020-10-12 14:09:13.59073
24526	ACWV	2020-10-06	93.15	93.43	92.25	92.41	214419	{}	\N	2020-10-12 14:09:13.595642
24527	ACWV	2020-10-07	92.77	93.1286	92.65	92.93	137161	{}	\N	2020-10-12 14:09:13.599859
24528	ACWV	2020-10-08	93.22	93.58	93.22	93.5	138038	{}	\N	2020-10-12 14:09:13.604203
24529	ACWV	2020-10-09	93.76	94.28	93.74	94.08	298272	{}	\N	2020-10-12 14:09:13.608767
24530	ACWV	2020-10-12	94.54	94.54	94.17	94.41	204166	{}	\N	2020-10-12 14:09:13.613286
24531	ACWX	2020-09-28	45.962	46.05	45.86	46.04	615471	{}	\N	2020-10-12 14:09:15.331356
24532	ACWX	2020-09-29	45.96	46.11	45.81	45.93	441802	{}	\N	2020-10-12 14:09:15.342482
24533	ACWX	2020-09-30	45.94	46.28	45.9001	46	1184221	{}	\N	2020-10-12 14:09:15.349873
24534	ACWX	2020-10-01	46.33	46.37	46.07	46.37	863516	{}	\N	2020-10-12 14:09:15.355916
24535	ACWX	2020-10-02	45.75	46.3	45.73	46.14	635106	{}	\N	2020-10-12 14:09:15.361316
24536	ACWX	2020-10-05	46.5	46.83	46.5	46.83	703214	{}	\N	2020-10-12 14:09:15.366189
24537	ACWX	2020-10-06	46.89	46.99	46.38	46.46	1208343	{}	\N	2020-10-12 14:09:15.371139
24538	ACWX	2020-10-07	46.83	46.9901	46.771	46.9	803160	{}	\N	2020-10-12 14:09:15.375884
24539	ACWX	2020-10-08	47.11	47.28	47.07	47.28	511488	{}	\N	2020-10-12 14:09:15.380294
24540	ACWX	2020-10-09	47.47	47.63	47.37	47.59	747345	{}	\N	2020-10-12 14:09:15.384965
24541	ACWX	2020-10-12	47.76	47.76	47.68	47.74	71531	{}	\N	2020-10-12 14:09:15.38948
24542	ADMS	2020-09-28	4.24	4.33	4.04	4.04	157619	{}	\N	2020-10-12 14:09:17.770502
24543	ADMS	2020-09-29	4.05	4.26	4.04	4.135	342349	{}	\N	2020-10-12 14:09:17.78152
24544	ADMS	2020-09-30	4.22	4.25	4.09	4.12	239285	{}	\N	2020-10-12 14:09:17.787901
24545	ADMS	2020-10-01	4.14	4.38	4.05	4.28	200548	{}	\N	2020-10-12 14:09:17.793695
24546	ADMS	2020-10-02	4.23	4.34	4.11	4.19	87181	{}	\N	2020-10-12 14:09:17.79826
24547	ADMS	2020-10-05	4.24	4.47	4.06	4.18	347101	{}	\N	2020-10-12 14:09:17.802593
24548	ADMS	2020-10-06	4.22	4.47	4.22	4.28	132574	{}	\N	2020-10-12 14:09:17.806888
24549	ADMS	2020-10-07	4.31	4.48	4.22	4.45	94637	{}	\N	2020-10-12 14:09:17.810994
24550	ADMS	2020-10-08	4.49	4.536	4.31	4.47	254206	{}	\N	2020-10-12 14:09:17.814652
24551	ADMS	2020-10-09	4.49	4.83	4.35	4.82	163245	{}	\N	2020-10-12 14:09:17.818323
24552	ADMS	2020-10-12	4.84	4.92	4.77	4.77	8238	{}	\N	2020-10-12 14:09:17.822034
24553	ADSW	2020-09-28	30.18	30.27	30.18	30.21	479871	{}	\N	2020-10-12 14:09:21.435556
24554	ADSW	2020-09-29	30.25	30.27	30.21	30.21	1154044	{}	\N	2020-10-12 14:09:21.443397
24555	ADSW	2020-09-30	30.25	30.25	30.23	30.23	1643766	{}	\N	2020-10-12 14:09:21.449237
24556	ADSW	2020-10-01	30.25	30.29	30.23	30.29	1483723	{}	\N	2020-10-12 14:09:21.454937
24557	ADSW	2020-10-02	30.26	30.28	30.22	30.24	1488069	{}	\N	2020-10-12 14:09:21.459558
24558	ADSW	2020-10-05	30.27	30.29	30.23	30.29	1521752	{}	\N	2020-10-12 14:09:21.463546
24559	ADSW	2020-10-06	30.27	30.27	30.23	30.25	1674454	{}	\N	2020-10-12 14:09:21.467444
24560	ADSW	2020-10-07	30.26	30.28	30.24	30.27	1265517	{}	\N	2020-10-12 14:09:21.471106
24561	ADSW	2020-10-08	30.28	30.29	30.27	30.27	840051	{}	\N	2020-10-12 14:09:21.47503
24562	ADSW	2020-10-09	30.27	30.29	30.26	30.27	746301	{}	\N	2020-10-12 14:09:21.478948
24563	ADSW	2020-10-12	30.29	30.34	30.26	30.275	31202	{}	\N	2020-10-12 14:09:21.482595
24564	ACHC	2020-09-28	28.65	29.245	27.92	28.04	369602	{}	\N	2020-10-12 14:09:23.30646
24565	ACHC	2020-09-29	28.52	29.24	27.965	28.22	545473	{}	\N	2020-10-12 14:09:23.32036
24566	ACHC	2020-09-30	30.18	30.64	29.21	29.48	603538	{}	\N	2020-10-12 14:09:23.327434
24567	ACHC	2020-10-01	29.48	30.17	29.44	29.965	638821	{}	\N	2020-10-12 14:09:23.332573
24568	ACHC	2020-10-02	29.75	31.29	29.5	30.945	556586	{}	\N	2020-10-12 14:09:23.338407
24569	ACHC	2020-10-05	31.23	32.05	31.05	31.88	476181	{}	\N	2020-10-12 14:09:23.343174
24570	ACHC	2020-10-06	32.01	32.46	31.3	31.37	396333	{}	\N	2020-10-12 14:09:23.347708
24571	ACHC	2020-10-07	31.82	32.2999	31.18	31.22	245291	{}	\N	2020-10-12 14:09:23.352314
24572	ACHC	2020-10-08	31.5	32.12	31.13	32.12	264845	{}	\N	2020-10-12 14:09:23.357152
24573	ACHC	2020-10-09	32.69	32.98	31.795	32.41	451437	{}	\N	2020-10-12 14:09:23.361622
24574	ACHC	2020-10-12	32.33	32.49	32	32.49	59101	{}	\N	2020-10-12 14:09:23.366296
24575	ACOR	2020-09-28	0.6314	0.73	0.5525	0.5997	21332183	{}	\N	2020-10-12 14:09:24.970759
24576	ACOR	2020-09-29	0.569	0.62	0.55	0.5575	5069824	{}	\N	2020-10-12 14:09:24.98198
24577	ACOR	2020-09-30	0.54	0.5595	0.503	0.515	3817445	{}	\N	2020-10-12 14:09:24.990694
24578	ACOR	2020-10-01	0.5014	0.5275	0.481	0.52	3035582	{}	\N	2020-10-12 14:09:24.996123
24579	ACOR	2020-10-02	0.4945	0.57	0.4945	0.5603	2663146	{}	\N	2020-10-12 14:09:25.00128
24580	ACOR	2020-10-05	0.56	0.5975	0.5569	0.5781	2291918	{}	\N	2020-10-12 14:09:25.006005
24581	ACOR	2020-10-06	0.5775	0.6675	0.5723	0.6466	4756135	{}	\N	2020-10-12 14:09:25.010423
24582	ACOR	2020-10-07	0.6774	0.6774	0.6155	0.6505	2278157	{}	\N	2020-10-12 14:09:25.014937
24583	ACOR	2020-10-08	0.6524	0.6592	0.6302	0.6425	1664850	{}	\N	2020-10-12 14:09:25.019037
24584	ACOR	2020-10-09	0.65	0.785	0.64	0.7356	7764430	{}	\N	2020-10-12 14:09:25.023017
24585	ACOR	2020-10-12	0.731	0.74	0.6825	0.6987	773586	{}	\N	2020-10-12 14:09:25.027879
24586	ACV	2020-09-28	25.29	25.43	25.15	25.31	42435	{}	\N	2020-10-12 14:09:26.27859
24587	ACV	2020-09-29	25.31	25.37	25.07	25.28	118849	{}	\N	2020-10-12 14:09:26.287609
24588	ACV	2020-09-30	25.33	25.51	25.21	25.34	90652	{}	\N	2020-10-12 14:09:26.293726
24589	ACV	2020-10-01	25.52	25.75	25.33	25.75	136787	{}	\N	2020-10-12 14:09:26.298695
24590	ACV	2020-10-02	25.6	25.63	25.29	25.6	139711	{}	\N	2020-10-12 14:09:26.304128
24591	ACV	2020-10-05	25.6	25.9	25.55	25.88	159088	{}	\N	2020-10-12 14:09:26.312315
24592	ACV	2020-10-06	26	26.54	25.8793	26.23	187430	{}	\N	2020-10-12 14:09:26.317337
24593	ACV	2020-10-07	26.57	27.14	26.4733	27.05	138919	{}	\N	2020-10-12 14:09:26.321748
24594	ACV	2020-10-08	27.23	27.4	26.9223	27.27	111306	{}	\N	2020-10-12 14:09:26.326407
24595	ACV	2020-10-09	27.32	27.57	27.28	27.44	119683	{}	\N	2020-10-12 14:09:26.331609
24596	ACV	2020-10-12	27.4398	27.44	27.3525	27.43	18420	{}	\N	2020-10-12 14:09:26.336208
24597	ADP	2020-09-28	136.72	138.04	136.08	137.31	2447189	{}	\N	2020-10-12 14:09:27.947187
24598	ADP	2020-09-29	137.55	138.505	136.4989	137.54	1838734	{}	\N	2020-10-12 14:09:27.956954
24599	ADP	2020-09-30	137.95	140.97	137.77	139.49	2594380	{}	\N	2020-10-12 14:09:27.9627
24600	ADP	2020-10-01	141.62	141.67	136.69	137.71	1639067	{}	\N	2020-10-12 14:09:27.968122
24601	ADP	2020-10-02	135.18	139.32	135.11	138.67	2051207	{}	\N	2020-10-12 14:09:27.97351
24602	ADP	2020-10-05	139.29	141.87	138.925	140.96	1928098	{}	\N	2020-10-12 14:09:27.978181
24603	ADP	2020-10-06	141.22	145.58	140.01	142.26	2069775	{}	\N	2020-10-12 14:09:27.982338
24604	ADP	2020-10-07	143.37	144.42	142	143.73	2374584	{}	\N	2020-10-12 14:09:27.987975
24605	ADP	2020-10-08	145.21	147.32	144.905	147.05	1690409	{}	\N	2020-10-12 14:09:27.993037
24606	ADP	2020-10-09	148	150.08	146.87	149.85	2561750	{}	\N	2020-10-12 14:09:27.99828
24607	ADP	2020-10-12	151.5	151.88	150.33	151.19	339069	{}	\N	2020-10-12 14:09:28.002928
24608	AEP	2020-09-28	81.06	81.53	80.46	80.65	1647422	{}	\N	2020-10-12 14:09:30.553838
24609	AEP	2020-09-29	81.01	81.555	80.48	80.97	3200104	{}	\N	2020-10-12 14:09:30.564758
24610	AEP	2020-09-30	81.54	82.095	81.245	81.73	4868695	{}	\N	2020-10-12 14:09:30.570991
24611	AEP	2020-10-01	81.69	83.09	81.3	83.06	2269696	{}	\N	2020-10-12 14:09:30.576663
24612	AEP	2020-10-02	82.73	85.61	82.22	85.3	3811988	{}	\N	2020-10-12 14:09:30.581767
24613	AEP	2020-10-05	85.25	87.33	84.25	87.05	3367970	{}	\N	2020-10-12 14:09:30.586562
24614	AEP	2020-10-06	86.63	89.68	86.36	88.43	4125352	{}	\N	2020-10-12 14:09:30.591007
24615	AEP	2020-10-07	88.8	89.65	88.09	88.61	2231365	{}	\N	2020-10-12 14:09:30.595523
24616	AEP	2020-10-08	88.72	89.82	88.371	89.38	1584156	{}	\N	2020-10-12 14:09:30.599615
24617	AEP	2020-10-09	89.95	89.95	88.56	89.82	2686113	{}	\N	2020-10-12 14:09:30.603751
24618	AEP	2020-10-12	89.7	90	88.91	89.69	478617	{}	\N	2020-10-12 14:09:30.608473
24619	ADS	2020-09-28	43.61	43.81	42.64	43.26	740926	{}	\N	2020-10-12 14:09:31.820731
24620	ADS	2020-09-29	42.97	43.05	41.67	42.82	1307275	{}	\N	2020-10-12 14:09:31.833109
24621	ADS	2020-09-30	42.91	43.5	41.61	41.98	2091590	{}	\N	2020-10-12 14:09:31.839486
24622	ADS	2020-10-01	42.4	43.45	42.23	43.44	2616085	{}	\N	2020-10-12 14:09:31.845896
24623	ADS	2020-10-02	42.36	45.905	42.1134	45.32	2146642	{}	\N	2020-10-12 14:09:31.850626
24624	ADS	2020-10-05	45.97	47.4	45.89	46.92	2129653	{}	\N	2020-10-12 14:09:31.855561
24625	ADS	2020-10-06	47.5	48.95	45.77	46	2046243	{}	\N	2020-10-12 14:09:31.859854
24626	ADS	2020-10-07	46.8	47.56	46.19	46.77	995808	{}	\N	2020-10-12 14:09:31.864769
24627	ADS	2020-10-08	47.48	48.525	46.875	48.41	1394552	{}	\N	2020-10-12 14:09:31.869093
24628	ADS	2020-10-09	49	49.46	48.01	48.62	1450101	{}	\N	2020-10-12 14:09:31.873783
24629	ADS	2020-10-12	48.99	48.99	48.21	48.915	125124	{}	\N	2020-10-12 14:09:31.878463
24630	AEZS	2020-09-28	0.3601	0.3601	0.337	0.3431	1192490	{}	\N	2020-10-12 14:09:33.095175
24631	AEZS	2020-09-29	0.3438	0.3624	0.342	0.36	847869	{}	\N	2020-10-12 14:09:33.10698
24632	AEZS	2020-09-30	0.36	0.36	0.3485	0.351	839315	{}	\N	2020-10-12 14:09:33.114124
24633	AEZS	2020-10-01	0.35	0.3645	0.3485	0.3604	842470	{}	\N	2020-10-12 14:09:33.119826
24634	AEZS	2020-10-02	0.355	0.377036	0.34	0.36	1109637	{}	\N	2020-10-12 14:09:33.125055
24635	AEZS	2020-10-05	0.3605	0.375	0.3549	0.3692	981820	{}	\N	2020-10-12 14:09:33.130066
24636	AEZS	2020-10-06	0.3689	0.369	0.36	0.36	1080539	{}	\N	2020-10-12 14:09:33.135254
24637	AEZS	2020-10-07	0.3613	0.38	0.3524	0.3749	1687624	{}	\N	2020-10-12 14:09:33.140152
24638	AEZS	2020-10-08	0.39	0.3901	0.372	0.3763	1079542	{}	\N	2020-10-12 14:09:33.145017
24639	AEZS	2020-10-09	0.397	0.397	0.3802	0.385	524981	{}	\N	2020-10-12 14:09:33.14946
24640	AEZS	2020-10-12	0.39	0.39	0.3754	0.3756	131572	{}	\N	2020-10-12 14:09:33.154099
24641	AGFS	2020-09-28	2.39	2.58	2.3712	2.56	41378	{}	\N	2020-10-12 14:09:36.320651
24642	AGFS	2020-09-29	2.56	2.56	2.38	2.43	32098	{}	\N	2020-10-12 14:09:36.336621
24643	AGFS	2020-09-30	2.45	2.51	2.4	2.43	21052	{}	\N	2020-10-12 14:09:36.343065
24644	AGFS	2020-10-01	2.44	2.4426	2.35	2.39	71687	{}	\N	2020-10-12 14:09:36.348739
24645	AGFS	2020-10-02	2.28	2.42	2.28	2.39	38276	{}	\N	2020-10-12 14:09:36.353545
24646	AGFS	2020-10-05	2.39	2.47	2.32	2.45	24099	{}	\N	2020-10-12 14:09:36.358461
24647	AGFS	2020-10-06	2.47	2.55	2.41	2.42	29679	{}	\N	2020-10-12 14:09:36.362627
24648	AGFS	2020-10-07	2.43	2.57	2.43	2.56	34191	{}	\N	2020-10-12 14:09:36.366831
24649	AGFS	2020-10-08	2.6	2.684	2.6	2.65	40496	{}	\N	2020-10-12 14:09:36.370849
24650	AGFS	2020-10-09	2.67	2.69	2.4	2.6	39897	{}	\N	2020-10-12 14:09:36.374807
24651	AGFS	2020-10-12	2.57	2.57	2.51	2.52	7085	{}	\N	2020-10-12 14:09:36.379056
24652	AGS	2020-09-28	3.61	4.11	3.61	3.95	967749	{}	\N	2020-10-12 14:09:39.060832
24653	AGS	2020-09-29	3.92	3.9591	3.57	3.63	1193499	{}	\N	2020-10-12 14:09:39.067598
24654	AGS	2020-09-30	3.66	3.75	3.46	3.54	1159165	{}	\N	2020-10-12 14:09:39.075025
24655	AGS	2020-10-01	3.61	3.7	3.5072	3.62	427969	{}	\N	2020-10-12 14:09:39.08051
24656	AGS	2020-10-02	3.47	3.725	3.38	3.62	489501	{}	\N	2020-10-12 14:09:39.087652
24657	AGS	2020-10-05	3.67	3.7813	3.55	3.57	580277	{}	\N	2020-10-12 14:09:39.093629
24658	AGS	2020-10-06	3.68	3.799	3.49	3.5	885953	{}	\N	2020-10-12 14:09:39.098901
24659	AGS	2020-10-07	3.59	3.64	3.46	3.53	762517	{}	\N	2020-10-12 14:09:39.103698
24660	AGS	2020-10-08	3.58	3.6	3.45	3.49	8418012	{}	\N	2020-10-12 14:09:39.109189
24661	AGS	2020-10-09	3.56	3.5608	3.42	3.43	413863	{}	\N	2020-10-12 14:09:39.11527
24662	AGS	2020-10-12	3.41	3.4403	3.31	3.4403	100183	{}	\N	2020-10-12 14:09:39.120512
25740	AMCA	2020-09-28	27.48	27.7689	27.48	27.7689	2516	{}	\N	2020-10-12 14:13:43.7147
25741	AMCA	2020-09-29	27.6	27.6475	27.6	27.6475	195	{}	\N	2020-10-12 14:13:43.729019
25742	AMCA	2020-10-06	28.89	28.89	28.4674	28.4674	399	{}	\N	2020-10-12 14:13:43.733643
25743	AMCA	2020-10-08	29.074	29.08	29.074	29.08	2553	{}	\N	2020-10-12 14:13:43.738297
25744	AMPH	2020-09-28	18.13	18.54	18.06	18.51	170304	{}	\N	2020-10-12 14:13:45.556003
25745	AMPH	2020-09-29	18.54	18.68	18.29	18.63	241816	{}	\N	2020-10-12 14:13:45.565538
25746	AMPH	2020-09-30	18.72	19	18.57	18.76	144473	{}	\N	2020-10-12 14:13:45.572941
25747	AMPH	2020-10-01	18.89	19.09	18.73	19.04	169434	{}	\N	2020-10-12 14:13:45.578625
25748	AMPH	2020-10-02	18.78	19.22	18.55	19	192781	{}	\N	2020-10-12 14:13:45.583589
25749	AMPH	2020-10-05	20.3	20.7115	19.75	20.14	234696	{}	\N	2020-10-12 14:13:45.589029
25750	AMPH	2020-10-06	20.33	20.57	19.86	20.21	178099	{}	\N	2020-10-12 14:13:45.593961
25751	AMPH	2020-10-07	20.33	20.59	20.09	20.44	187615	{}	\N	2020-10-12 14:13:45.598555
25752	AMPH	2020-10-08	20.58	21.11	20.58	20.92	140564	{}	\N	2020-10-12 14:13:45.602667
25753	AMPH	2020-10-09	21	21.34	20.76	21.24	102002	{}	\N	2020-10-12 14:13:45.606918
25754	AMPH	2020-10-12	21.24	21.27	20.855	21.2	6622	{}	\N	2020-10-12 14:13:45.611188
25755	AMSF	2020-09-28	58.16	59.16	57.81	58.06	64897	{}	\N	2020-10-12 14:13:46.557167
25756	AMSF	2020-09-29	58.07	58.07	56.57	57.22	67930	{}	\N	2020-10-12 14:13:46.566598
25757	AMSF	2020-09-30	57.66	58.02	57.11	57.36	99744	{}	\N	2020-10-12 14:13:46.573489
25758	AMSF	2020-10-01	57.52	57.6	56.41	57.3	100865	{}	\N	2020-10-12 14:13:46.579557
25759	AMSF	2020-10-02	56.64	58.37	56.64	57.81	66651	{}	\N	2020-10-12 14:13:46.585057
25760	AMSF	2020-10-05	58.23	58.75	57.96	58.38	42609	{}	\N	2020-10-12 14:13:46.59022
25761	AMSF	2020-10-06	58.8	59.23	57.91	58.05	99635	{}	\N	2020-10-12 14:13:46.594761
25762	AMSF	2020-10-07	58.3	59.35	57.43	57.68	86184	{}	\N	2020-10-12 14:13:46.599088
25763	AMSF	2020-10-08	57.86	58.6	56.94	57.59	63052	{}	\N	2020-10-12 14:13:46.603409
25764	AMSF	2020-10-09	57.72	58.29	57.43	57.95	69748	{}	\N	2020-10-12 14:13:46.608102
25765	AMSF	2020-10-12	57.94	58.57	57.66	58.4	2095	{}	\N	2020-10-12 14:13:46.61273
25766	AN	2020-09-28	52.68	53.82	52.34	53.77	443972	{}	\N	2020-10-12 14:13:47.63575
25767	AN	2020-09-29	53.6	53.765	51.32	52.04	993070	{}	\N	2020-10-12 14:13:47.645634
25768	AN	2020-09-30	52.45	53.23	52.025	52.93	1333052	{}	\N	2020-10-12 14:13:47.653516
25769	AN	2020-10-01	53.28	54.645	52.5	54.55	1033326	{}	\N	2020-10-12 14:13:47.658738
25770	AN	2020-10-02	53.63	57.96	53.63	57.44	1653565	{}	\N	2020-10-12 14:13:47.66421
25771	AN	2020-10-05	57.7	59.93	57.7	59.47	1590863	{}	\N	2020-10-12 14:13:47.669623
25772	AN	2020-10-06	60	61.96	58.92	59.03	2132460	{}	\N	2020-10-12 14:13:47.674531
25773	AN	2020-10-07	60.09	61.15	59.9	60.16	1708494	{}	\N	2020-10-12 14:13:47.678977
25774	AN	2020-10-08	60.39	61.8699	59.5928	61.61	1378406	{}	\N	2020-10-12 14:13:47.68362
25775	AN	2020-10-09	62.27	62.27	60.35	60.56	1329340	{}	\N	2020-10-12 14:13:47.689
25776	AN	2020-10-12	60.5	60.955	60.14	60.955	63461	{}	\N	2020-10-12 14:13:47.69425
25777	ANH	2020-09-28	1.74	1.77	1.71	1.74	628997	{}	\N	2020-10-12 14:13:48.850715
25778	ANH	2020-09-29	1.71	1.71	1.63	1.67	1048320	{}	\N	2020-10-12 14:13:48.857112
25779	ANH	2020-09-30	1.66	1.69	1.62	1.64	1018330	{}	\N	2020-10-12 14:13:48.862369
25780	ANH	2020-10-01	1.65	1.66	1.62	1.64	667333	{}	\N	2020-10-12 14:13:48.867664
25781	ANH	2020-10-02	1.62	1.67	1.62	1.66	580447	{}	\N	2020-10-12 14:13:48.873091
25782	ANH	2020-10-05	1.64	1.72	1.64	1.69	768891	{}	\N	2020-10-12 14:13:48.877402
25783	ANH	2020-10-06	1.7	1.77	1.7	1.7	1293789	{}	\N	2020-10-12 14:13:48.88171
25784	ANH	2020-10-07	1.72	1.73	1.68	1.71	804738	{}	\N	2020-10-12 14:13:48.886494
25785	ANH	2020-10-08	1.72	1.78	1.7	1.75	1523654	{}	\N	2020-10-12 14:13:48.891397
25786	ANH	2020-10-09	1.77	1.7774	1.73	1.73	535515	{}	\N	2020-10-12 14:13:48.895699
25787	ANH	2020-10-12	1.72	1.74	1.72	1.7397	156660	{}	\N	2020-10-12 14:13:48.899774
25788	AOM	2020-09-28	40.99	41.097	40.94	41.06	101676	{}	\N	2020-10-12 14:13:49.850667
25789	AOM	2020-09-29	41.07	41.33	40.96	41	87039	{}	\N	2020-10-12 14:13:49.861859
25790	AOM	2020-09-30	40.94	41.18	40.94	41.05	88742	{}	\N	2020-10-12 14:13:49.868802
25791	AOM	2020-10-01	41.12	41.2199	41.03	41.17	152477	{}	\N	2020-10-12 14:13:49.874995
25792	AOM	2020-10-02	40.68	40.999	40.63	40.95	105931	{}	\N	2020-10-12 14:13:49.879977
25793	AOM	2020-10-05	41.03	41.15	41.0005	41.1	142177	{}	\N	2020-10-12 14:13:49.885013
25794	AOM	2020-10-06	41.19	41.28	40.92	40.92	186545	{}	\N	2020-10-12 14:13:49.889581
25795	AOM	2020-10-07	41.13	41.268	40.95	41.19	79802	{}	\N	2020-10-12 14:13:49.894046
25796	AOM	2020-10-08	41.35	41.45	41.28	41.37	86895	{}	\N	2020-10-12 14:13:49.898406
24663	AIMT	2020-09-28	34.3	34.35	34.3	34.325	902569	{}	\N	2020-10-12 14:09:41.234938
24664	AIMT	2020-09-29	34.33	34.41	34.32	34.35	805682	{}	\N	2020-10-12 14:09:41.251234
24665	AIMT	2020-09-30	34.33	34.47	34.33	34.45	1763265	{}	\N	2020-10-12 14:09:41.256669
24666	AIMT	2020-10-01	34.4	34.45	34.37	34.44	983870	{}	\N	2020-10-12 14:09:41.261432
24667	AIMT	2020-10-02	34.4	34.42	34.34	34.35	1622114	{}	\N	2020-10-12 14:09:41.266132
24668	AIMT	2020-10-05	34.3	34.44	34.3	34.44	1529627	{}	\N	2020-10-12 14:09:41.270217
24669	AIMT	2020-10-06	34.42	34.45	34.42	34.42	986201	{}	\N	2020-10-12 14:09:41.274592
24670	AIMT	2020-10-07	34.45	34.45	34.42	34.43	1168122	{}	\N	2020-10-12 14:09:41.279151
24671	AIMT	2020-10-08	34.44	34.44	34.43	34.43	769970	{}	\N	2020-10-12 14:09:41.283247
24672	AIMT	2020-10-09	34.43	34.59	34.43	34.59	4252970	{}	\N	2020-10-12 14:09:41.287345
24673	AIMT	2020-10-12	34.56	34.56	34.5	34.52	123290	{}	\N	2020-10-12 14:09:41.291499
24674	ACHV	2020-09-28	9.28	9.32	8.66	8.84	139917	{}	\N	2020-10-12 14:09:42.427669
24675	ACHV	2020-09-29	8.73	8.83	8.47	8.71	94999	{}	\N	2020-10-12 14:09:42.440847
24676	ACHV	2020-09-30	8.62	8.76	8.21	8.37	84396	{}	\N	2020-10-12 14:09:42.447588
24677	ACHV	2020-10-01	8.3	8.75	8.27	8.57	169814	{}	\N	2020-10-12 14:09:42.455749
24678	ACHV	2020-10-02	8.74	8.74	8.25	8.41	133571	{}	\N	2020-10-12 14:09:42.462396
24679	ACHV	2020-10-05	8.52	8.97	8.45	8.66	114907	{}	\N	2020-10-12 14:09:42.469084
24680	ACHV	2020-10-06	8.72	8.85	8.51	8.58	90491	{}	\N	2020-10-12 14:09:42.476273
24681	ACHV	2020-10-07	8.82	9.76	8.82	9.21	200338	{}	\N	2020-10-12 14:09:42.486858
24682	ACHV	2020-10-08	9.27	10.1	9.1101	9.87	150660	{}	\N	2020-10-12 14:09:42.494184
24683	ACHV	2020-10-09	9.82	9.87	9.46	9.62	60473	{}	\N	2020-10-12 14:09:42.500717
24684	ACHV	2020-10-12	9.61	9.91	9.61	9.835	24558	{}	\N	2020-10-12 14:09:42.507415
24685	ACRE	2020-09-28	9.73	9.98	9.73	9.79	397836	{}	\N	2020-10-12 14:09:43.749421
24686	ACRE	2020-09-29	9.31	9.37	9.01	9.11	868350	{}	\N	2020-10-12 14:09:43.759315
24687	ACRE	2020-09-30	9.05	9.29	9.02	9.14	807526	{}	\N	2020-10-12 14:09:43.765273
24688	ACRE	2020-10-01	9.25	9.34	9.06	9.33	297744	{}	\N	2020-10-12 14:09:43.770384
24689	ACRE	2020-10-02	9.19	9.55	9.05	9.49	432905	{}	\N	2020-10-12 14:09:43.775204
24690	ACRE	2020-10-05	9.55	9.69	9.4878	9.61	388968	{}	\N	2020-10-12 14:09:43.779743
24691	ACRE	2020-10-06	9.7	9.85	9.46	9.46	427913	{}	\N	2020-10-12 14:09:43.783607
24692	ACRE	2020-10-07	9.58	9.61	9.21	9.37	403913	{}	\N	2020-10-12 14:09:43.787439
24693	ACRE	2020-10-08	9.49	9.66	9.4381	9.64	364939	{}	\N	2020-10-12 14:09:43.791275
24694	ACRE	2020-10-09	9.72	9.78	9.415	9.52	319513	{}	\N	2020-10-12 14:09:43.795054
24695	ACRE	2020-10-12	9.54	9.54	9.42	9.54	32065	{}	\N	2020-10-12 14:09:43.798732
24696	ADMA	2020-09-28	2.18	2.22	2.11	2.17	1507575	{}	\N	2020-10-12 14:09:45.203821
24697	ADMA	2020-09-29	2.175	2.235	2.1	2.2	1267466	{}	\N	2020-10-12 14:09:45.210836
24698	ADMA	2020-09-30	2.19	2.63	2.1698	2.39	13020628	{}	\N	2020-10-12 14:09:45.216172
24699	ADMA	2020-10-01	2.41	2.41	2.15	2.23	5595680	{}	\N	2020-10-12 14:09:45.220761
24700	ADMA	2020-10-02	2.21	2.56	2.21	2.49	7941186	{}	\N	2020-10-12 14:09:45.225152
24701	ADMA	2020-10-05	2.56	2.7	2.43	2.49	6999179	{}	\N	2020-10-12 14:09:45.229433
24702	ADMA	2020-10-06	2.53	2.55	2.39	2.42	3866177	{}	\N	2020-10-12 14:09:45.233554
24703	ADMA	2020-10-07	2.5	2.67	2.47	2.56	5188359	{}	\N	2020-10-12 14:09:45.237441
24704	ADMA	2020-10-08	2.615	2.6168	2.47	2.5	3284813	{}	\N	2020-10-12 14:09:45.241231
24705	ADMA	2020-10-09	2.52	2.55	2.47	2.49	2074343	{}	\N	2020-10-12 14:09:45.245165
24706	ADMA	2020-10-12	2.53	2.54	2.44	2.5	655121	{}	\N	2020-10-12 14:09:45.249045
25797	AOM	2020-10-09	41.42	41.6	41.41	41.6	181509	{}	\N	2020-10-12 14:13:49.9026
25798	AOM	2020-10-12	41.71	41.71	41.6	41.675	40289	{}	\N	2020-10-12 14:13:49.907179
25799	AJXA	2020-09-28	24.22	24.22	22.5	23.215	274156	{}	\N	2020-10-12 14:13:51.798083
25800	AJXA	2020-09-29	23.0513	23.41	23.03	23.24	322607	{}	\N	2020-10-12 14:13:51.805142
25801	AJXA	2020-09-30	22.85	23.83	22.85	23.83	83727	{}	\N	2020-10-12 14:13:51.810767
25802	AJXA	2020-10-01	23.5	24.02	23.5	24.02	27873	{}	\N	2020-10-12 14:13:51.815691
25803	AJXA	2020-10-02	23.85	24	23.72	23.9225	5482	{}	\N	2020-10-12 14:13:51.820416
25804	AJXA	2020-10-05	23.97	24	23.83	23.83	4040	{}	\N	2020-10-12 14:13:51.824692
25805	AJXA	2020-10-06	24.0084	24.15	23.88	23.88	14454	{}	\N	2020-10-12 14:13:51.828864
25806	AJXA	2020-10-07	23.89	24.26	23.89	24	6888	{}	\N	2020-10-12 14:13:51.832947
25807	AJXA	2020-10-08	23.9	24.026	23.9	24	14348	{}	\N	2020-10-12 14:13:51.837568
25808	AJXA	2020-10-09	23.99	23.99	23.91	23.91	1802	{}	\N	2020-10-12 14:13:51.841779
25809	AKTS	2020-09-28	8.37	8.6	8.27	8.34	311549	{}	\N	2020-10-12 14:13:53.276446
25810	AKTS	2020-09-29	8.33	8.88	8.27	8.51	468173	{}	\N	2020-10-12 14:13:53.285574
25811	AKTS	2020-09-30	8.52	8.62	8.09	8.17	394948	{}	\N	2020-10-12 14:13:53.291786
25812	AKTS	2020-10-01	8.28	8.38	7.95	8.05	332259	{}	\N	2020-10-12 14:13:53.297541
25813	AKTS	2020-10-02	7.91	8.16	7.88	8.03	201394	{}	\N	2020-10-12 14:13:53.302696
25814	AKTS	2020-10-05	8	8.29	7.96	8.08	236988	{}	\N	2020-10-12 14:13:53.307545
25815	AKTS	2020-10-06	8.08	8.415	8.02	8.19	267926	{}	\N	2020-10-12 14:13:53.312333
25816	AKTS	2020-10-07	8.31	8.44	8.18	8.38	197060	{}	\N	2020-10-12 14:13:53.316656
25817	AKTS	2020-10-08	8.49	8.59	8.38	8.45	215578	{}	\N	2020-10-12 14:13:53.324294
25818	AKTS	2020-10-09	8.5	8.58	8.4247	8.54	202817	{}	\N	2020-10-12 14:13:53.328987
25819	AKTS	2020-10-12	8.59	8.66	8.4658	8.57	47314	{}	\N	2020-10-12 14:13:53.333465
25820	ALEX	2020-09-28	11.02	11.36	11.02	11.19	282851	{}	\N	2020-10-12 14:13:56.150365
25821	ALEX	2020-09-29	11.12	11.16	10.84	10.94	781444	{}	\N	2020-10-12 14:13:56.160479
25822	ALEX	2020-09-30	11.09	11.41	10.94	11.21	888356	{}	\N	2020-10-12 14:13:56.167616
25823	ALEX	2020-10-01	11.28	11.62	11.201	11.59	463266	{}	\N	2020-10-12 14:13:56.173439
25824	ALEX	2020-10-02	11.3	11.885	11.2101	11.85	441867	{}	\N	2020-10-12 14:13:56.178702
25825	ALEX	2020-10-05	11.91	11.91	11.61	11.79	419896	{}	\N	2020-10-12 14:13:56.183773
25826	ALEX	2020-10-06	11.89	12.31	11.84	11.93	731281	{}	\N	2020-10-12 14:13:56.18866
25827	ALEX	2020-10-07	12	12.22	11.67	11.96	557607	{}	\N	2020-10-12 14:13:56.193761
24707	AEIS	2020-09-28	60.93	62.86	60.43	61.98	259027	{}	\N	2020-10-12 14:10:58.703803
24708	AEIS	2020-09-29	62	63.85	62	62.91	349367	{}	\N	2020-10-12 14:10:58.7164
24709	AEIS	2020-09-30	62.89	64.12	62.11	62.94	322694	{}	\N	2020-10-12 14:10:58.721908
24710	AEIS	2020-10-01	63.68	64.44	63.16	64.06	251395	{}	\N	2020-10-12 14:10:58.726946
24711	AEIS	2020-10-02	62.4	62.99	61.68	61.72	225384	{}	\N	2020-10-12 14:10:58.732144
24712	AEIS	2020-10-05	62.39	64.74	62.39	64.69	210040	{}	\N	2020-10-12 14:10:58.736476
24713	AEIS	2020-10-06	66.62	68.1	64.72	65.76	511062	{}	\N	2020-10-12 14:10:58.740475
24714	AEIS	2020-10-07	66.73	67.79	66.09	67.07	369423	{}	\N	2020-10-12 14:10:58.744388
24715	AEIS	2020-10-08	67.79	68.78	67.38	68	166855	{}	\N	2020-10-12 14:10:58.748286
24716	AEIS	2020-10-09	69.155	70.51	68.47	70.4	212526	{}	\N	2020-10-12 14:10:58.752385
24717	AEIS	2020-10-12	71.23	71.28	70.11	71.18	26247	{}	\N	2020-10-12 14:10:58.756905
24718	AER	2020-09-28	24.02	25.99	23.895	25.51	1806679	{}	\N	2020-10-12 14:11:00.007603
24719	AER	2020-09-29	25.38	25.38	24.49	25.06	2439534	{}	\N	2020-10-12 14:11:00.018861
24720	AER	2020-09-30	25.05	26.04	24.945	25.19	2382602	{}	\N	2020-10-12 14:11:00.025034
24721	AER	2020-10-01	25.36	26.47	25.25	26.07	4457692	{}	\N	2020-10-12 14:11:00.030951
24722	AER	2020-10-02	25.28	26.75	24.94	26.6	4380760	{}	\N	2020-10-12 14:11:00.03646
24723	AER	2020-10-05	27.11	27.49	26.67	27.01	3088371	{}	\N	2020-10-12 14:11:00.04124
24724	AER	2020-10-06	27.54	27.65	25.62	25.73	2671921	{}	\N	2020-10-12 14:11:00.045359
24725	AER	2020-10-07	26.14	27.36	26.14	26.91	2348271	{}	\N	2020-10-12 14:11:00.049739
24726	AER	2020-10-08	27.4	27.96	27.04	27.67	3743074	{}	\N	2020-10-12 14:11:00.054227
24727	AER	2020-10-09	27.8	28.06	27.02	27.77	2410355	{}	\N	2020-10-12 14:11:00.05852
24728	AER	2020-10-12	27.76	28.135	27.41	28.01	315945	{}	\N	2020-10-12 14:11:00.062639
24729	AFTY	2020-09-28	19.2301	19.3116	19.23	19.3116	3418	{}	\N	2020-10-12 14:11:02.399905
24730	AFTY	2020-09-29	19.22	19.31	19.21	19.31	5413	{}	\N	2020-10-12 14:11:02.409148
24731	AFTY	2020-09-30	19.21	19.3514	19.21	19.3514	4048	{}	\N	2020-10-12 14:11:02.41561
24732	AFTY	2020-10-01	19.55	19.65	19.55	19.65	1137	{}	\N	2020-10-12 14:11:02.42129
24733	AFTY	2020-10-02	19.3946	19.3946	19.3946	19.3946	134	{}	\N	2020-10-12 14:11:02.425764
24734	AFTY	2020-10-05	19.64	19.6553	19.6	19.6553	1729	{}	\N	2020-10-12 14:11:02.430477
24735	AFTY	2020-10-06	19.7	19.72	19.6227	19.6227	2100	{}	\N	2020-10-12 14:11:02.45326
24736	AFTY	2020-10-07	19.77	19.8319	19.77	19.8319	806	{}	\N	2020-10-12 14:11:02.462442
24737	AFTY	2020-10-08	19.79	19.8754	19.79	19.8754	739	{}	\N	2020-10-12 14:11:02.468895
24738	AFTY	2020-10-09	19.95	20	19.94	19.98	10220	{}	\N	2020-10-12 14:11:02.474464
24739	AFTY	2020-10-12	20.33	20.3337	20.32	20.32	1742	{}	\N	2020-10-12 14:11:02.47934
24740	AGI	2020-09-28	8.95	9.02	8.73	8.85	1924306	{}	\N	2020-10-12 14:11:03.548603
24741	AGI	2020-09-29	8.91	9	8.81	8.82	5185245	{}	\N	2020-10-12 14:11:03.556306
24742	AGI	2020-09-30	8.76	8.955	8.7	8.81	3555022	{}	\N	2020-10-12 14:11:03.561481
24743	AGI	2020-10-01	8.89	9.05	8.825	8.89	2683334	{}	\N	2020-10-12 14:11:03.566395
24744	AGI	2020-10-02	8.84	8.9125	8.71	8.76	2252291	{}	\N	2020-10-12 14:11:03.571267
24745	AGI	2020-10-05	8.85	9.04	8.81	8.95	5236886	{}	\N	2020-10-12 14:11:03.57565
24746	AGI	2020-10-06	8.99	9.09	8.61	8.63	4940378	{}	\N	2020-10-12 14:11:03.579983
24747	AGI	2020-10-07	8.8	8.83	8.635	8.74	4570533	{}	\N	2020-10-12 14:11:03.58405
24748	AGI	2020-10-08	8.81	8.88	8.6748	8.8	3003391	{}	\N	2020-10-12 14:11:03.587579
24749	AGI	2020-10-09	9.05	9.27	8.93	9.27	3729977	{}	\N	2020-10-12 14:11:03.591169
24750	AGI	2020-10-12	9.22	9.35	9.13	9.28	658561	{}	\N	2020-10-12 14:11:03.594692
24751	AGQ	2020-09-28	42.26	43.68	41.455	43.45	2039259	{}	\N	2020-10-12 14:11:04.517299
24752	AGQ	2020-09-29	44.26	46.189	44.05	45.45	2420168	{}	\N	2020-10-12 14:11:04.528198
24753	AGQ	2020-09-30	44.3237	44.8177	41.33	41.77	2727690	{}	\N	2020-10-12 14:11:04.535708
24754	AGQ	2020-10-01	43.26	45.19	42.3	43.57	2344999	{}	\N	2020-10-12 14:11:04.54143
24755	AGQ	2020-10-02	44.12	45.36	43.35	43.67	1551835	{}	\N	2020-10-12 14:11:04.546357
24756	AGQ	2020-10-05	44.45	46.5	44.3913	45.81	1338803	{}	\N	2020-10-12 14:11:04.551451
24757	AGQ	2020-10-06	45.68	46.06	41.56	42	2315870	{}	\N	2020-10-12 14:11:04.555835
24758	AGQ	2020-10-07	43	44.39	42.622	43.55	1320385	{}	\N	2020-10-12 14:11:04.559934
24759	AGQ	2020-10-08	44.67	45.26	43.16	43.85	1034041	{}	\N	2020-10-12 14:11:04.564124
24760	AGQ	2020-10-09	46.4	48.72	46.295	48.7	2422904	{}	\N	2020-10-12 14:11:04.568576
24761	AGQ	2020-10-12	48.3168	48.6	47.95	48.41	1036340	{}	\N	2020-10-12 14:11:04.572829
24762	AHH	2020-09-28	9.245	9.55	9.245	9.41	210171	{}	\N	2020-10-12 14:11:06.385566
24763	AHH	2020-09-29	9.24	9.3	8.93	9.17	443486	{}	\N	2020-10-12 14:11:06.391919
24764	AHH	2020-09-30	9.18	9.56	9.11	9.26	876724	{}	\N	2020-10-12 14:11:06.396649
24765	AHH	2020-10-01	9.29	9.54	9.29	9.52	503969	{}	\N	2020-10-12 14:11:06.401429
24766	AHH	2020-10-02	9.23	9.62	9.23	9.57	356781	{}	\N	2020-10-12 14:11:06.406834
24767	AHH	2020-10-05	9.7	9.73	9.27	9.49	443824	{}	\N	2020-10-12 14:11:06.411868
24768	AHH	2020-10-06	9.61	9.76	9.4	9.44	614422	{}	\N	2020-10-12 14:11:06.417274
24769	AHH	2020-10-07	9.53	9.56	9.16	9.26	660639	{}	\N	2020-10-12 14:11:06.42261
24770	AHH	2020-10-08	9.4	9.46	9.19	9.4	498958	{}	\N	2020-10-12 14:11:06.427809
24771	AHH	2020-10-09	9.39	9.51	9.21	9.26	422581	{}	\N	2020-10-12 14:11:06.433375
24772	AHH	2020-10-12	9.25	9.31	9.16	9.26	51722	{}	\N	2020-10-12 14:11:06.438593
24773	AIMC	2020-09-28	37.29	38.13	37.12	37.53	353816	{}	\N	2020-10-12 14:11:07.638329
24774	AIMC	2020-09-29	37.38	37.98	36.15	36.87	202877	{}	\N	2020-10-12 14:11:07.648771
24775	AIMC	2020-09-30	37.03	38.08	36.52	36.97	253390	{}	\N	2020-10-12 14:11:07.656039
24776	AIMC	2020-10-01	36.94	37.81	36.7216	37.74	436677	{}	\N	2020-10-12 14:11:07.662883
24777	AIMC	2020-10-02	36.73	38.85	36.21	38.32	172956	{}	\N	2020-10-12 14:11:07.668807
24778	AIMC	2020-10-05	38.74	39.96	38.74	39.61	135639	{}	\N	2020-10-12 14:11:07.673878
24779	AIMC	2020-10-06	40.03	41.78	39.32	40.43	253334	{}	\N	2020-10-12 14:11:07.678711
24780	AIMC	2020-10-07	40.8	41.71	40.5	40.9	318387	{}	\N	2020-10-12 14:11:07.683691
24781	AIMC	2020-10-08	41.71	41.71	40.34	40.72	205644	{}	\N	2020-10-12 14:11:07.68844
24782	AIMC	2020-10-09	41.37	41.61	40.08	40.21	307185	{}	\N	2020-10-12 14:11:07.69388
24783	AIMC	2020-10-12	40.19	40.9	40.13	40.9	11922	{}	\N	2020-10-12 14:11:07.700381
24784	GLBS	2020-09-28	0.138	0.156	0.1279	0.1463	63049030	{}	\N	2020-10-12 14:11:09.51396
24785	GLBS	2020-09-29	0.121	0.1419	0.12	0.1242	24600627	{}	\N	2020-10-12 14:11:09.524345
24786	GLBS	2020-09-30	0.13	0.1323	0.121	0.1267	7459578	{}	\N	2020-10-12 14:11:09.531275
24787	GLBS	2020-10-01	0.1287	0.1287	0.1168	0.125	11717061	{}	\N	2020-10-12 14:11:09.537215
24788	GLBS	2020-10-02	0.12	0.123	0.1168	0.1206	5641671	{}	\N	2020-10-12 14:11:09.542499
24789	GLBS	2020-10-05	0.1213	0.122	0.116	0.1191	7086185	{}	\N	2020-10-12 14:11:09.547165
24790	GLBS	2020-10-06	0.119	0.1277	0.117	0.12	15314263	{}	\N	2020-10-12 14:11:09.552124
24791	GLBS	2020-10-07	0.1217	0.129	0.121	0.125	14065046	{}	\N	2020-10-12 14:11:09.556845
24792	GLBS	2020-10-08	0.126	0.129	0.1205	0.1215	5977840	{}	\N	2020-10-12 14:11:09.561506
24793	GLBS	2020-10-09	0.121	0.1261	0.1201	0.1245	9429905	{}	\N	2020-10-12 14:11:09.56629
24794	GLBS	2020-10-12	0.14	0.156	0.132	0.138435	29615788	{}	\N	2020-10-12 14:11:09.571003
24795	ACU	2020-09-28	22.58	22.8693	22.05	22.05	11754	{}	\N	2020-10-12 14:11:12.948401
24796	ACU	2020-09-29	22.47	22.98	22.47	22.85	5604	{}	\N	2020-10-12 14:11:12.958783
24797	ACU	2020-09-30	22.75	23.11	22.2939	23.0281	4509	{}	\N	2020-10-12 14:11:12.964718
24798	ACU	2020-10-01	23.5	24.07	23.3	23.76	14874	{}	\N	2020-10-12 14:11:12.969888
24799	ACU	2020-10-02	23.75	23.75	23.3015	23.4499	3980	{}	\N	2020-10-12 14:11:12.974455
24800	ACU	2020-10-05	24.44	24.88	23.955	24.16	19866	{}	\N	2020-10-12 14:11:12.979234
24801	ACU	2020-10-06	24.16	24.88	24.02	24.02	11594	{}	\N	2020-10-12 14:11:12.985026
24802	ACU	2020-10-07	24.39	25.2	24.39	25.01	13846	{}	\N	2020-10-12 14:11:12.989471
24803	ACU	2020-10-08	24.7219	25.15	24.1	25.15	11921	{}	\N	2020-10-12 14:11:12.993526
24804	ACU	2020-10-09	25.16	25.16	24.25	24.25	4440	{}	\N	2020-10-12 14:11:12.99737
24805	ACU	2020-10-12	24.7	24.7	23.75	23.75	2218	{}	\N	2020-10-12 14:11:13.001096
24806	ADI	2020-09-28	115.31	117.07	114.765	117.06	1937393	{}	\N	2020-10-12 14:11:15.802804
24807	ADI	2020-09-29	115.99	117.94	115.84	116.65	2155353	{}	\N	2020-10-12 14:11:15.811018
24808	ADI	2020-09-30	116.63	118.25	116.21	116.74	2829378	{}	\N	2020-10-12 14:11:15.816146
24809	ADI	2020-10-01	118.98	119.64	117.37	118.99	2927981	{}	\N	2020-10-12 14:11:15.82079
24810	ADI	2020-10-02	116.5	117.95	114	114.01	2975054	{}	\N	2020-10-12 14:11:15.825471
24811	ADI	2020-10-05	115.36	116.69	114.965	116.6	4338618	{}	\N	2020-10-12 14:11:15.829824
24812	ADI	2020-10-06	115.96	118.93	115.025	115.54	3203686	{}	\N	2020-10-12 14:11:15.834117
24813	ADI	2020-10-07	117.1	118.98	115.79	118.67	4211842	{}	\N	2020-10-12 14:11:15.837761
24814	ADI	2020-10-08	119.93	121.1	119.29	120.34	3901434	{}	\N	2020-10-12 14:11:15.841664
24815	ADI	2020-10-09	122.01	124.41	121.76	123.33	3839089	{}	\N	2020-10-12 14:11:15.845699
24816	ADI	2020-10-12	124.72	124.9436	124.01	124.71	326241	{}	\N	2020-10-12 14:11:15.849309
24817	ADT	2020-09-28	8.28	8.495	8.2	8.42	4109870	{}	\N	2020-10-12 14:11:17.851872
24818	ADT	2020-09-29	8.39	8.4899	8.25	8.36	5323691	{}	\N	2020-10-12 14:11:17.860135
24819	ADT	2020-09-30	8.35	8.53	7.99	8.17	13690650	{}	\N	2020-10-12 14:11:17.866319
24820	ADT	2020-10-01	8.22	8.22	7.905	8.16	9204373	{}	\N	2020-10-12 14:11:17.872263
24821	ADT	2020-10-02	8	8.26	7.96	8.19	4758440	{}	\N	2020-10-12 14:11:17.877165
24822	ADT	2020-10-05	8.23	8.45	8.08	8.2	6980859	{}	\N	2020-10-12 14:11:17.882099
24823	ADT	2020-10-06	8.21	8.47	8.135	8.36	9153862	{}	\N	2020-10-12 14:11:17.886433
24824	ADT	2020-10-07	8.42	8.605	8.3	8.42	8379651	{}	\N	2020-10-12 14:11:17.891196
24825	ADT	2020-10-08	8.47	8.575	8.25	8.41	7096047	{}	\N	2020-10-12 14:11:17.895309
24826	ADT	2020-10-09	8.47	8.5	8.33	8.34	4330561	{}	\N	2020-10-12 14:11:17.899452
24827	ADT	2020-10-12	8.36	8.405	8.32	8.4	592960	{}	\N	2020-10-12 14:11:17.903692
24828	AEGN	2020-09-28	14.11	14.5	14.11	14.26	111735	{}	\N	2020-10-12 14:11:19.247232
24829	AEGN	2020-09-29	14.12	14.43	14.04	14.28	125560	{}	\N	2020-10-12 14:11:19.257779
24830	AEGN	2020-09-30	14.4	14.64	14	14.13	169038	{}	\N	2020-10-12 14:11:19.264629
24831	AEGN	2020-10-01	14.035	14.31	13.895	14.245	127720	{}	\N	2020-10-12 14:11:19.270358
24832	AEGN	2020-10-02	14.3	14.68	13.91	14.55	53485	{}	\N	2020-10-12 14:11:19.276013
24833	AEGN	2020-10-05	14.67	15.47	14.5	14.83	100889	{}	\N	2020-10-12 14:11:19.280781
24834	AEGN	2020-10-06	15.04	15.66	14.5824	14.61	129071	{}	\N	2020-10-12 14:11:19.285433
24835	AEGN	2020-10-07	14.75	15.07	14.74	14.92	120665	{}	\N	2020-10-12 14:11:19.290028
24836	AEGN	2020-10-08	14.79	15.17	14.49	15.15	93057	{}	\N	2020-10-12 14:11:19.294797
24837	AEGN	2020-10-09	15.33	15.61	15.1	15.49	103788	{}	\N	2020-10-12 14:11:19.299608
24838	AEGN	2020-10-12	15.5	15.66	14.68	15.55	29891	{}	\N	2020-10-12 14:11:19.304282
24839	AES	2020-09-28	18.28	18.45	17.85	17.87	4965726	{}	\N	2020-10-12 14:11:20.179887
24840	AES	2020-09-29	18.06	18.23	17.82	18	8186893	{}	\N	2020-10-12 14:11:20.190706
24841	AES	2020-09-30	18.04	18.27	17.96	18.11	12099926	{}	\N	2020-10-12 14:11:20.200569
24842	AES	2020-10-01	18.24	18.31	17.68	17.92	9134804	{}	\N	2020-10-12 14:11:20.205383
24843	AES	2020-10-02	17.53	18.545	17.46	18.54	11997655	{}	\N	2020-10-12 14:11:20.209943
24844	AES	2020-10-05	18.65	18.78	18.545	18.58	6904291	{}	\N	2020-10-12 14:11:20.214663
24845	AES	2020-10-06	18.59	18.84	18.42	18.66	6345002	{}	\N	2020-10-12 14:11:20.218807
24846	AES	2020-10-07	18.76	18.93	18.585	18.75	6648244	{}	\N	2020-10-12 14:11:20.222259
24847	AES	2020-10-08	18.87	19.48	18.78	19.44	17541832	{}	\N	2020-10-12 14:11:20.225758
24848	AES	2020-10-09	19.5	19.57	18.98	19.32	15292170	{}	\N	2020-10-12 14:11:20.229597
24849	AES	2020-10-12	19.32	19.4	19.21	19.39	1383179	{}	\N	2020-10-12 14:11:20.233469
24850	AGEN	2020-09-28	4.05	4.14	3.965	4.05	1228397	{}	\N	2020-10-12 14:11:23.091201
24851	AGEN	2020-09-29	4.05	4.13	3.98	4.07	1222305	{}	\N	2020-10-12 14:11:23.099323
24852	AGEN	2020-09-30	4.07	4.14	3.95	4	1239359	{}	\N	2020-10-12 14:11:23.104926
24853	AGEN	2020-10-01	4	4.1	3.96	4.05	1012012	{}	\N	2020-10-12 14:11:23.110625
24854	AGEN	2020-10-02	3.96	4.0599	3.88	3.88	1458099	{}	\N	2020-10-12 14:11:23.116085
24855	AGEN	2020-10-05	3.89	4.1	3.88	4.09	1318840	{}	\N	2020-10-12 14:11:23.120449
24856	AGEN	2020-10-06	4.09	4.29	4.08	4.19	1470046	{}	\N	2020-10-12 14:11:23.124491
24857	AGEN	2020-10-07	4.25	4.51	4.14	4.45	1374525	{}	\N	2020-10-12 14:11:23.12854
24858	AGEN	2020-10-08	4.49	4.54	4.34	4.4	1132906	{}	\N	2020-10-12 14:11:23.132922
24859	AGEN	2020-10-09	4.43	4.53	4.28	4.35	1337423	{}	\N	2020-10-12 14:11:23.136708
24860	AGEN	2020-10-12	4.4	4.41	4.295	4.32	243973	{}	\N	2020-10-12 14:11:23.140625
24861	AGGY	2020-09-28	53.8	53.83	53.76	53.78	108914	{}	\N	2020-10-12 14:11:25.017782
24862	AGGY	2020-09-29	53.81	53.8904	53.81	53.87	59918	{}	\N	2020-10-12 14:11:25.027041
24863	AGGY	2020-09-30	53.86	53.87	53.72	53.82	240658	{}	\N	2020-10-12 14:11:25.033495
24864	AGGY	2020-10-01	53.75	53.87	53.696	53.77	170723	{}	\N	2020-10-12 14:11:25.039333
24865	AGGY	2020-10-02	53.82	53.88	53.78	53.87	129385	{}	\N	2020-10-12 14:11:25.044281
24866	AGGY	2020-10-05	53.72	53.72	53.6216	53.65	155102	{}	\N	2020-10-12 14:11:25.049377
24867	AGGY	2020-10-06	53.61	53.73	53.55	53.71	212276	{}	\N	2020-10-12 14:11:25.053993
24868	AGGY	2020-10-07	53.59	53.69	53.55	53.615	180740	{}	\N	2020-10-12 14:11:25.05812
24869	AGGY	2020-10-08	53.55	53.71	53.55	53.65	142181	{}	\N	2020-10-12 14:11:25.062676
24870	AGGY	2020-10-09	53.62	53.67	53.56	53.66	68420	{}	\N	2020-10-12 14:11:25.067541
24871	AGGY	2020-10-12	53.61	53.6857	53.61	53.6857	8739	{}	\N	2020-10-12 14:11:25.072187
24872	AGTC	2020-09-28	4.91	4.99	4.77	4.91	99332	{}	\N	2020-10-12 14:11:27.815615
24873	AGTC	2020-09-29	4.88	5.0821	4.85	4.97	321346	{}	\N	2020-10-12 14:11:27.825279
24874	AGTC	2020-09-30	4.96	5.0243	4.8	4.86	139883	{}	\N	2020-10-12 14:11:27.832834
24875	AGTC	2020-10-01	4.91	5.15	4.8	5	480391	{}	\N	2020-10-12 14:11:27.838118
24876	AGTC	2020-10-02	4.88	5	4.66	4.92	174482	{}	\N	2020-10-12 14:11:27.84295
24877	AGTC	2020-10-05	4.98	5.6	4.94	5.47	383179	{}	\N	2020-10-12 14:11:27.848186
24878	AGTC	2020-10-06	5.54	5.67	5.3	5.39	368115	{}	\N	2020-10-12 14:11:27.852371
24879	AGTC	2020-10-07	5.45	5.7	5.4	5.47	164103	{}	\N	2020-10-12 14:11:27.85618
24880	AGTC	2020-10-08	5.59	5.65	5.45	5.5	133119	{}	\N	2020-10-12 14:11:27.85979
24881	AGTC	2020-10-09	5.51	5.7	5.43	5.55	127793	{}	\N	2020-10-12 14:11:27.863739
24882	AGTC	2020-10-12	5.54	5.69	5.5	5.51	26612	{}	\N	2020-10-12 14:11:27.867541
24883	AI	2020-09-28	2.77	2.89	2.77	2.83	362161	{}	\N	2020-10-12 14:11:29.989701
24884	AI	2020-09-29	2.84	2.86	2.7745	2.85	470027	{}	\N	2020-10-12 14:11:29.999615
24885	AI	2020-09-30	2.84	2.9	2.8	2.84	318934	{}	\N	2020-10-12 14:11:30.006071
24886	AI	2020-10-01	2.81	2.85	2.8	2.85	332177	{}	\N	2020-10-12 14:11:30.011628
24887	AI	2020-10-02	2.79	2.85	2.79	2.85	274399	{}	\N	2020-10-12 14:11:30.017238
24888	AI	2020-10-05	2.85	2.89	2.8174	2.86	286995	{}	\N	2020-10-12 14:11:30.021609
24889	AI	2020-10-06	2.82	2.92	2.81	2.81	359368	{}	\N	2020-10-12 14:11:30.02589
24890	AI	2020-10-07	2.84	2.87	2.82	2.82	369307	{}	\N	2020-10-12 14:11:30.030695
24891	AI	2020-10-08	2.86	2.98	2.86	2.95	414686	{}	\N	2020-10-12 14:11:30.035078
24892	AI	2020-10-09	2.95	2.98	2.82	2.83	375859	{}	\N	2020-10-12 14:11:30.039088
24893	AI	2020-10-12	2.81	2.86	2.81	2.85	72999	{}	\N	2020-10-12 14:11:30.043343
24894	ACIA	2020-09-28	67.94	67.94	67.52	67.55	534856	{}	\N	2020-10-12 14:11:32.80317
24895	ACIA	2020-09-29	67.61	67.97	67.45	67.49	266423	{}	\N	2020-10-12 14:11:32.809762
24896	ACIA	2020-09-30	67.58	67.75	67.33	67.4	315697	{}	\N	2020-10-12 14:11:32.815496
24897	ACIA	2020-10-01	67.35	67.67	66.895	67.37	543775	{}	\N	2020-10-12 14:11:32.820383
24898	ACIA	2020-10-02	66.97	67.195	66.76	67.04	916111	{}	\N	2020-10-12 14:11:32.825302
24899	ACIA	2020-10-05	67.38	67.51	67.11	67.41	335125	{}	\N	2020-10-12 14:11:32.829717
24900	ACIA	2020-10-06	67.2972	67.4	67.15	67.28	287142	{}	\N	2020-10-12 14:11:32.834037
24901	ACIA	2020-10-07	67.27	67.5	67.11	67.19	342035	{}	\N	2020-10-12 14:11:32.838014
24902	ACIA	2020-10-08	67.39	67.6	67.21	67.57	173962	{}	\N	2020-10-12 14:11:32.841861
24903	ACIA	2020-10-09	67.65	67.915	67.5	67.74	402125	{}	\N	2020-10-12 14:11:32.846113
24904	ACIA	2020-10-12	67.74	67.95	67.54	67.89	10409	{}	\N	2020-10-12 14:11:32.850384
24905	ACP	2020-09-28	9.635	9.76	9.635	9.76	102306	{}	\N	2020-10-12 14:11:34.627231
24906	ACP	2020-09-29	9.71	9.9198	9.71	9.87	147843	{}	\N	2020-10-12 14:11:34.635466
24907	ACP	2020-09-30	9.9	10.08	9.8701	9.92	238618	{}	\N	2020-10-12 14:11:34.640761
24908	ACP	2020-10-01	9.88	9.93	9.8	9.8	154279	{}	\N	2020-10-12 14:11:34.646594
24909	ACP	2020-10-02	9.79	9.8369	9.68	9.75	157981	{}	\N	2020-10-12 14:11:34.650987
24910	ACP	2020-10-05	9.79	9.905	9.76	9.89	115988	{}	\N	2020-10-12 14:11:34.65555
24911	ACP	2020-10-06	9.92	10.08	9.918	10.01	142769	{}	\N	2020-10-12 14:11:34.659338
24912	ACP	2020-10-07	10	10.13	10	10.063	173726	{}	\N	2020-10-12 14:11:34.663177
24913	ACP	2020-10-08	10.05	10.0755	9.94	9.99	167510	{}	\N	2020-10-12 14:11:34.667467
24914	ACP	2020-10-09	10.01	10.07	9.96	9.98	119430	{}	\N	2020-10-12 14:11:34.671412
24915	ACP	2020-10-12	10.03	10.07	10.0249	10.042	6549	{}	\N	2020-10-12 14:11:34.675395
24916	ACWI	2020-09-28	79.69	79.93	79.45	79.84	2699391	{}	\N	2020-10-12 14:11:36.304407
24917	ACWI	2020-09-29	79.73	79.94	79.35	79.55	2066777	{}	\N	2020-10-12 14:11:36.319653
24918	ACWI	2020-09-30	79.63	80.52	79.53	79.92	10153998	{}	\N	2020-10-12 14:11:36.326939
24919	ACWI	2020-10-01	80.53	80.66	80.07	80.52	3056625	{}	\N	2020-10-12 14:11:36.333116
24920	ACWI	2020-10-02	79.36	80.31	79.26	79.91	2204753	{}	\N	2020-10-12 14:11:36.338259
24921	ACWI	2020-10-05	80.39	81.25	80.39	81.23	2928907	{}	\N	2020-10-12 14:11:36.342741
24922	ACWI	2020-10-06	81.33	81.68	80.17	80.36	3238134	{}	\N	2020-10-12 14:11:36.347555
24923	ACWI	2020-10-07	81.05	81.65	81.02	81.46	3603900	{}	\N	2020-10-12 14:11:36.351992
24924	ACWI	2020-10-08	81.95	82.14	81.76	82.12	4074621	{}	\N	2020-10-12 14:11:36.356461
24925	ACWI	2020-10-09	82.54	82.885	82.41	82.79	1060349	{}	\N	2020-10-12 14:11:36.360691
24926	ACWI	2020-10-12	83.26	83.33	83.19	83.29	117946	{}	\N	2020-10-12 14:11:36.365164
24927	ADM	2020-09-28	46.665	47.1	46.435	46.5	2325753	{}	\N	2020-10-12 14:11:37.782168
24928	ADM	2020-09-29	46.58	46.58	45.74	45.98	3157643	{}	\N	2020-10-12 14:11:37.791004
24929	ADM	2020-09-30	46.35	46.805	46.06	46.49	4550005	{}	\N	2020-10-12 14:11:37.797253
24930	ADM	2020-10-01	46.66	46.75	46.19	46.6	3497300	{}	\N	2020-10-12 14:11:37.80308
24931	ADM	2020-10-02	45.95	47.05	45.925	46.74	3800145	{}	\N	2020-10-12 14:11:37.8082
24932	ADM	2020-10-05	47.04	47.815	46.94	47.78	6017273	{}	\N	2020-10-12 14:11:37.813558
24933	ADM	2020-10-06	47.85	48.59	47.44	47.6	4313452	{}	\N	2020-10-12 14:11:37.818209
24934	ADM	2020-10-07	48.04	48.45	47.91	48.31	3927878	{}	\N	2020-10-12 14:11:37.822825
24935	ADM	2020-10-08	48.52	48.86	48.3	48.85	4893466	{}	\N	2020-10-12 14:11:37.827316
24936	ADM	2020-10-09	49	49.51	48.895	48.97	5548957	{}	\N	2020-10-12 14:11:37.832025
24937	ADM	2020-10-12	49.22	49.56	49.19	49.56	400889	{}	\N	2020-10-12 14:11:37.836425
24938	ADRO	2020-09-28	2.37	2.4301	2.28	2.37	258235	{}	\N	2020-10-12 14:11:39.974056
24939	ADRO	2020-09-29	2.36	2.38	2.3	2.31	201257	{}	\N	2020-10-12 14:11:39.981082
24940	ADRO	2020-09-30	2.34	2.51	2.32	2.43	439544	{}	\N	2020-10-12 14:11:39.985437
24941	ADRO	2020-10-01	2.42	3	2.4	2.86	2728496	{}	\N	2020-10-12 14:11:39.990196
24942	ADRO	2020-10-02	12.0705	15.7283	12.0705	14.01	949173	{}	\N	2020-10-12 14:11:39.995444
24943	ADRO	2020-10-05	14.05	14.8599	13.92	14.595	170658	{}	\N	2020-10-12 14:11:40.00057
25828	ALEX	2020-10-08	12.06	12.27	11.96	12.21	328870	{}	\N	2020-10-12 14:13:56.198455
25829	ALEX	2020-10-09	12.32	12.42	12.15	12.38	305204	{}	\N	2020-10-12 14:13:56.203009
25830	ALEX	2020-10-12	12.28	12.45	12.13	12.315	115536	{}	\N	2020-10-12 14:13:56.207416
25831	ALRN	2020-09-28	1.27	1.3	1.25	1.26	108120	{}	\N	2020-10-12 14:13:57.138513
25832	ALRN	2020-09-29	1.26	1.2667	1.2	1.22	117064	{}	\N	2020-10-12 14:13:57.148722
25833	ALRN	2020-09-30	1.25	1.29	1.22	1.29	303847	{}	\N	2020-10-12 14:13:57.155386
25834	ALRN	2020-10-01	1.33	1.33	1.23	1.23	157482	{}	\N	2020-10-12 14:13:57.161349
25835	ALRN	2020-10-02	1.2	1.225	1.15	1.16	270695	{}	\N	2020-10-12 14:13:57.167406
25836	ALRN	2020-10-05	1.18	1.25	1.174	1.24	222218	{}	\N	2020-10-12 14:13:57.172594
25837	ALRN	2020-10-06	1.25	1.29	1.21	1.24	153336	{}	\N	2020-10-12 14:13:57.177267
25838	ALRN	2020-10-07	1.24	1.35	1.22	1.33	593367	{}	\N	2020-10-12 14:13:57.18148
25839	ALRN	2020-10-08	1.35	1.68	1.33	1.62	2464763	{}	\N	2020-10-12 14:13:57.186278
25840	ALRN	2020-10-09	1.68	1.69	1.46	1.55	832495	{}	\N	2020-10-12 14:13:57.190743
25841	ALRN	2020-10-12	1.6	1.63	1.56	1.58	91325	{}	\N	2020-10-12 14:13:57.196049
25842	ALV	2020-09-28	71.71	72.62	71.61	72.06	262802	{}	\N	2020-10-12 14:14:00.497046
25843	ALV	2020-09-29	73.54	73.9	72.45	72.87	793968	{}	\N	2020-10-12 14:14:00.506097
25844	ALV	2020-09-30	73.44	74.29	72.4	72.88	1069697	{}	\N	2020-10-12 14:14:00.512948
25845	ALV	2020-10-01	76.27	77.17	75.53	76.67	1361893	{}	\N	2020-10-12 14:14:00.519319
25846	ALV	2020-10-02	75.97	78.11	75.87	77.58	1365001	{}	\N	2020-10-12 14:14:00.524699
25847	ALV	2020-10-05	81.34	81.93	80.24	81.31	963406	{}	\N	2020-10-12 14:14:00.529487
25848	ALV	2020-10-06	82.27	82.71	80.375	80.38	757262	{}	\N	2020-10-12 14:14:00.534182
25849	ALV	2020-10-07	81.28	83.63	81.155	83.37	613240	{}	\N	2020-10-12 14:14:00.538517
25850	ALV	2020-10-08	82.44	83.16	81.68	82.48	789424	{}	\N	2020-10-12 14:14:00.542739
25851	ALV	2020-10-09	82.27	83.1	81.87	82.71	664691	{}	\N	2020-10-12 14:14:00.547381
25852	ALV	2020-10-12	82.11	82.86	81.99	82.4	215720	{}	\N	2020-10-12 14:14:00.551637
25853	AMH	2020-09-28	28.105	28.83	28.01	28.68	1388334	{}	\N	2020-10-12 14:14:01.999119
25854	AMH	2020-09-29	28.38	28.77	28.02	28.4	2037572	{}	\N	2020-10-12 14:14:02.006928
25855	AMH	2020-09-30	28.68	28.72	28.13	28.48	4870830	{}	\N	2020-10-12 14:14:02.013154
25856	AMH	2020-10-01	28.73	29.34	28.575	29.33	4048572	{}	\N	2020-10-12 14:14:02.018442
25857	AMH	2020-10-02	29	29.7	28.7	29.5	3746131	{}	\N	2020-10-12 14:14:02.023497
25858	AMH	2020-10-05	29.77	29.94	29.42	29.83	2676607	{}	\N	2020-10-12 14:14:02.027832
25859	AMH	2020-10-06	29.95	29.96	29.49	29.75	1754319	{}	\N	2020-10-12 14:14:02.032495
25860	AMH	2020-10-07	29.99	30.03	29.38	29.47	2012072	{}	\N	2020-10-12 14:14:02.03724
25861	AMH	2020-10-08	29.59	30.075	29.59	30.04	2526005	{}	\N	2020-10-12 14:14:02.041851
25862	AMH	2020-10-09	30.31	30.31	29.835	29.9	3515910	{}	\N	2020-10-12 14:14:02.046531
25863	AMH	2020-10-12	29.96	30.015	29.67	29.83	107759	{}	\N	2020-10-12 14:14:02.051298
25864	AMPE	2020-09-28	0.96	0.96	0.8927	0.92	2263177	{}	\N	2020-10-12 14:14:02.938826
25865	AMPE	2020-09-29	0.914	0.97	0.8955	0.96	1500732	{}	\N	2020-10-12 14:14:02.951891
25866	AMPE	2020-09-30	0.95	0.9601	0.905	0.957	1530174	{}	\N	2020-10-12 14:14:02.959616
25867	AMPE	2020-10-01	0.95	0.955	0.9	0.9166	1444229	{}	\N	2020-10-12 14:14:02.966248
25868	AMPE	2020-10-02	0.9	0.9194	0.86	0.8742	1523906	{}	\N	2020-10-12 14:14:02.972087
25869	AMPE	2020-10-05	0.88	0.9	0.8674	0.8728	1096941	{}	\N	2020-10-12 14:14:02.977309
25870	AMPE	2020-10-06	0.88	0.88	0.8186	0.8186	1954265	{}	\N	2020-10-12 14:14:02.981993
25871	AMPE	2020-10-07	0.83	0.8699	0.83	0.85	1013016	{}	\N	2020-10-12 14:14:02.986743
25872	AMPE	2020-10-08	0.85	0.91	0.8301	0.901	2281200	{}	\N	2020-10-12 14:14:02.991339
25873	AMPE	2020-10-09	0.8917	0.899	0.8602	0.8748	815959	{}	\N	2020-10-12 14:14:02.997468
25874	AMPE	2020-10-12	0.87	0.8748	0.862	0.8709	231958	{}	\N	2020-10-12 14:14:03.002543
25875	AMSC	2020-09-28	13.56	15	13.56	14.97	832165	{}	\N	2020-10-12 14:14:04.567804
25876	AMSC	2020-09-29	15	16.47	15	16.03	2040387	{}	\N	2020-10-12 14:14:04.575077
25877	AMSC	2020-09-30	15.76	16.3151	14.21	14.48	709535	{}	\N	2020-10-12 14:14:04.580423
25878	AMSC	2020-10-01	14.37	14.81	13.91	14.23	270754	{}	\N	2020-10-12 14:14:04.585564
25879	AMSC	2020-10-02	13.93	14.96	13.92	14.35	191936	{}	\N	2020-10-12 14:14:04.590436
25880	AMSC	2020-10-05	14.59	15.3976	14.51	14.85	313827	{}	\N	2020-10-12 14:14:04.59523
25881	AMSC	2020-10-06	15.66	16.11	14.67	15.79	408549	{}	\N	2020-10-12 14:14:04.599832
25882	AMSC	2020-10-07	16.24	17.84	16.18	17.54	1162991	{}	\N	2020-10-12 14:14:04.604698
25883	AMSC	2020-10-08	17.69	17.815	15.86	16.32	535713	{}	\N	2020-10-12 14:14:04.60928
25884	AMSC	2020-10-09	16.76	17.25	16.38	17.01	280366	{}	\N	2020-10-12 14:14:04.614052
25885	AMSC	2020-10-12	17.32	17.45	16.85	17.45	27522	{}	\N	2020-10-12 14:14:04.618667
25886	AMZA	2020-09-28	13.14	13.5541	12.97	13.43	100995	{}	\N	2020-10-12 14:14:05.617729
25887	AMZA	2020-09-29	13.46	13.8937	13.26	13.84	57692	{}	\N	2020-10-12 14:14:05.627304
25888	AMZA	2020-09-30	13.86	14.18	13.35	13.52	108041	{}	\N	2020-10-12 14:14:05.634151
25889	AMZA	2020-10-01	13.45	13.53	12.945	13.2	94454	{}	\N	2020-10-12 14:14:05.640138
25890	AMZA	2020-10-02	12.89	13.535	12.81	13.52	104980	{}	\N	2020-10-12 14:14:05.645852
24944	ADX	2020-09-28	16.1	16.15	16.0411	16.09	213280	{}	\N	2020-10-12 14:11:41.509703
24945	ADX	2020-09-29	16.13	16.13	16	16.06	236229	{}	\N	2020-10-12 14:11:41.526528
24946	ADX	2020-09-30	16.06	16.34	16.06	16.21	296493	{}	\N	2020-10-12 14:11:41.532639
24947	ADX	2020-10-01	16.37	16.37	16.2	16.27	729639	{}	\N	2020-10-12 14:11:41.538005
24948	ADX	2020-10-02	15.97	16.21	15.97	16.15	306080	{}	\N	2020-10-12 14:11:41.543435
24949	ADX	2020-10-05	16.21	16.38	16.13	16.37	589578	{}	\N	2020-10-12 14:11:41.548077
24950	ADX	2020-10-06	16.4	16.51	16.18	16.22	494371	{}	\N	2020-10-12 14:11:41.552582
24951	ADX	2020-10-07	16.34	16.52	16.34	16.5	682348	{}	\N	2020-10-12 14:11:41.557291
24952	ADX	2020-10-08	16.56	16.63	16.51	16.57	401309	{}	\N	2020-10-12 14:11:41.561686
24953	ADX	2020-10-09	16.69	16.71	16.61	16.63	406440	{}	\N	2020-10-12 14:11:41.566075
24954	ADX	2020-10-12	16.75	16.81	16.74	16.81	42528	{}	\N	2020-10-12 14:11:41.570686
24955	AEG	2020-09-28	2.5	2.56	2.5	2.53	1967238	{}	\N	2020-10-12 14:11:43.872365
24956	AEG	2020-09-29	2.53	2.53	2.48	2.51	5233696	{}	\N	2020-10-12 14:11:43.882235
24957	AEG	2020-09-30	2.54	2.6	2.54	2.56	3014538	{}	\N	2020-10-12 14:11:43.888481
24958	AEG	2020-10-01	2.7	2.725	2.65	2.68	3559292	{}	\N	2020-10-12 14:11:43.894167
24959	AEG	2020-10-02	2.68	2.75	2.67	2.74	4519303	{}	\N	2020-10-12 14:11:43.898952
24960	AEG	2020-10-05	2.78	2.79	2.75	2.77	3698894	{}	\N	2020-10-12 14:11:43.903616
24961	AEG	2020-10-06	2.92	2.96	2.87	2.87	7270166	{}	\N	2020-10-12 14:11:43.907995
24962	AEG	2020-10-07	2.94	2.97	2.93	2.96	3262527	{}	\N	2020-10-12 14:11:43.912499
24963	AEG	2020-10-08	3.02	3.05	3	3.04	4739395	{}	\N	2020-10-12 14:11:43.916727
24964	AEG	2020-10-09	3.06	3.06	3.02	3.02	4939167	{}	\N	2020-10-12 14:11:43.920771
24965	AEG	2020-10-12	3.1	3.11	3.08	3.09	1374856	{}	\N	2020-10-12 14:11:43.926302
24966	AEY	2020-09-28	1.9	1.93	1.88	1.89	60778	{}	\N	2020-10-12 14:11:45.755658
24967	AEY	2020-09-29	1.88	1.92	1.8728	1.9	46086	{}	\N	2020-10-12 14:11:45.768594
24968	AEY	2020-09-30	1.89	1.98	1.89	1.92	159361	{}	\N	2020-10-12 14:11:45.775467
24969	AEY	2020-10-01	1.92	1.9799	1.91	1.9299	55850	{}	\N	2020-10-12 14:11:45.781855
24970	AEY	2020-10-02	1.9	2.01	1.9	2	108074	{}	\N	2020-10-12 14:11:45.786896
24971	AEY	2020-10-05	2	2.07	2	2.02	35300	{}	\N	2020-10-12 14:11:45.792027
24972	AEY	2020-10-06	2.04	2.09	1.98	2.01	54426	{}	\N	2020-10-12 14:11:45.796502
24973	AEY	2020-10-07	2.01	2.09	2	2.01	48911	{}	\N	2020-10-12 14:11:45.800472
24974	AEY	2020-10-08	2.06	2.08	2.01	2.05	56568	{}	\N	2020-10-12 14:11:45.804358
24975	AEY	2020-10-09	2.09	2.1	2.03	2.07	94795	{}	\N	2020-10-12 14:11:45.808446
24976	AEY	2020-10-12	2.07	2.08	2.04	2.065	35268	{}	\N	2020-10-12 14:11:45.812478
24977	AFMD	2020-09-28	3.35	3.4	3.29	3.32	415239	{}	\N	2020-10-12 14:11:48.286366
24978	AFMD	2020-09-29	3.32	3.45	3.29	3.42	1523444	{}	\N	2020-10-12 14:11:48.298082
24979	AFMD	2020-09-30	3.41	3.48	3.31	3.39	373424	{}	\N	2020-10-12 14:11:48.305731
24980	AFMD	2020-10-01	3.39	3.42	3.31	3.39	267602	{}	\N	2020-10-12 14:11:48.3111
24981	AFMD	2020-10-02	3.29	3.37	3.2	3.2	513371	{}	\N	2020-10-12 14:11:48.316516
24982	AFMD	2020-10-05	3.25	3.55	3.21	3.54	543268	{}	\N	2020-10-12 14:11:48.323206
24983	AFMD	2020-10-06	3.66	3.71	3.58	3.63	797292	{}	\N	2020-10-12 14:11:48.327302
24984	AFMD	2020-10-07	3.67	3.75	3.62	3.65	574556	{}	\N	2020-10-12 14:11:48.331324
24985	AFMD	2020-10-08	3.71	3.79	3.66	3.72	486723	{}	\N	2020-10-12 14:11:48.335887
24986	AFMD	2020-10-09	3.76	3.78	3.63	3.7	396376	{}	\N	2020-10-12 14:11:48.339984
24987	AFMD	2020-10-12	3.7	3.74	3.64	3.705	65757	{}	\N	2020-10-12 14:11:48.344914
24988	AGG	2020-09-28	118.18	118.23	118.11	118.23	5938427	{}	\N	2020-10-12 14:11:49.957749
24989	AGG	2020-09-29	118.28	118.33	118.21	118.25	5366360	{}	\N	2020-10-12 14:11:49.966993
24990	AGG	2020-09-30	118.26	118.26	117.9516	118.06	6134147	{}	\N	2020-10-12 14:11:49.973413
24991	AGG	2020-10-01	117.81	118.04	117.75	118.02	9760640	{}	\N	2020-10-12 14:11:49.97915
24992	AGG	2020-10-02	118.03	118.04	117.81	117.9	5697095	{}	\N	2020-10-12 14:11:49.98359
24993	AGG	2020-10-05	117.74	117.74	117.52	117.52	5052533	{}	\N	2020-10-12 14:11:49.987948
24994	AGG	2020-10-06	117.6	117.84	117.49	117.62	4386963	{}	\N	2020-10-12 14:11:49.992835
24995	AGG	2020-10-07	117.53	117.66	117.4515	117.54	4866102	{}	\N	2020-10-12 14:11:49.996577
24996	AGG	2020-10-08	117.67	117.71	117.59	117.69	4313757	{}	\N	2020-10-12 14:11:50.000076
24997	AGG	2020-10-09	117.65	117.74	117.5	117.72	2974554	{}	\N	2020-10-12 14:11:50.003434
24998	AGG	2020-10-12	117.85	117.85	117.77	117.785	388661	{}	\N	2020-10-12 14:11:50.007147
24999	AGRO	2020-09-28	4.62	4.7334	4.62	4.66	185574	{}	\N	2020-10-12 14:11:51.377979
25000	AGRO	2020-09-29	4.62	4.68	4.55	4.57	857024	{}	\N	2020-10-12 14:11:51.385206
25001	AGRO	2020-09-30	4.57	4.69	4.57	4.67	293358	{}	\N	2020-10-12 14:11:51.391593
25002	AGRO	2020-10-01	4.73	4.835	4.67	4.79	476357	{}	\N	2020-10-12 14:11:51.397561
25003	AGRO	2020-10-02	4.71	4.79	4.69	4.77	321108	{}	\N	2020-10-12 14:11:51.403333
25004	AGRO	2020-10-05	4.77	4.85	4.69	4.73	713290	{}	\N	2020-10-12 14:11:51.407958
25005	AGRO	2020-10-06	4.75	4.8	4.67	4.75	521802	{}	\N	2020-10-12 14:11:51.413322
25006	AGRO	2020-10-07	4.79	4.83	4.71	4.75	383774	{}	\N	2020-10-12 14:11:51.418976
25007	AGRO	2020-10-08	4.79	4.94	4.75	4.87	603968	{}	\N	2020-10-12 14:11:51.423292
25008	AGRO	2020-10-09	4.9	4.93	4.81	4.91	197736	{}	\N	2020-10-12 14:11:51.431057
25009	AGRO	2020-10-12	4.9	4.9	4.85	4.875	29864	{}	\N	2020-10-12 14:11:51.435731
25010	AHC	2020-09-28	1.37	1.39	1.37	1.39	3309	{}	\N	2020-10-12 14:11:53.439274
25011	AHC	2020-09-29	1.44	1.44	1.39	1.43	17912	{}	\N	2020-10-12 14:11:53.450264
25012	AHC	2020-09-30	1.4	1.42	1.4	1.41	13052	{}	\N	2020-10-12 14:11:53.458551
25013	AHC	2020-10-01	1.4102	1.44	1.39	1.39	27155	{}	\N	2020-10-12 14:11:53.464223
25014	AHC	2020-10-02	1.4	1.4	1.35	1.39	82650	{}	\N	2020-10-12 14:11:53.469174
25015	AHC	2020-10-05	1.4	1.4	1.36	1.37	40750	{}	\N	2020-10-12 14:11:53.473444
25016	AHC	2020-10-06	1.38	1.4	1.36	1.36	38459	{}	\N	2020-10-12 14:11:53.477681
25017	AHC	2020-10-07	1.41	1.41	1.37	1.38	23002	{}	\N	2020-10-12 14:11:53.481713
25018	AHC	2020-10-08	1.35	1.4	1.35	1.4	30037	{}	\N	2020-10-12 14:11:53.485954
25019	AHC	2020-10-09	1.43	1.48	1.41	1.45	93568	{}	\N	2020-10-12 14:11:53.490056
25020	AHC	2020-10-12	1.47	1.47	1.4534	1.4669	9618	{}	\N	2020-10-12 14:11:53.494443
25021	AIG	2020-09-28	27.66	28.035	27.42	27.63	7213788	{}	\N	2020-10-12 14:11:55.217158
25022	AIG	2020-09-29	27.54	27.645	26.9601	27.29	7868947	{}	\N	2020-10-12 14:11:55.228967
25023	AIG	2020-09-30	27.42	27.94	27.25	27.53	9566792	{}	\N	2020-10-12 14:11:55.236766
25024	AIG	2020-10-01	27.66	28.01	27.305	27.72	9110297	{}	\N	2020-10-12 14:11:55.242571
25025	AIG	2020-10-02	27.19	28.48	27.07	28.22	7830051	{}	\N	2020-10-12 14:11:55.249018
25026	AIG	2020-10-05	28.77	29.42	28.58	29.05	10650987	{}	\N	2020-10-12 14:11:55.254093
25027	AIG	2020-10-06	29.4	29.98	28.925	28.98	13617594	{}	\N	2020-10-12 14:11:55.258225
25028	AIG	2020-10-07	29.28	29.68	28.955	29.56	8948546	{}	\N	2020-10-12 14:11:55.262542
25029	AIG	2020-10-08	29.81	30.48	29.35	30.26	11054479	{}	\N	2020-10-12 14:11:55.266716
25030	AIG	2020-10-09	30.43	30.57	29.75	29.95	14914174	{}	\N	2020-10-12 14:11:55.271058
25031	AIG	2020-10-12	29.87	30.185	29.76	30.125	931927	{}	\N	2020-10-12 14:11:55.275288
25032	AIT	2020-09-28	54.63	55.8	54.63	54.9	149586	{}	\N	2020-10-12 14:11:56.280667
25033	AIT	2020-09-29	54.79	55.2	53.87	54.72	318181	{}	\N	2020-10-12 14:11:56.289356
25034	AIT	2020-09-30	54.89	56.09	54.54	55.1	586614	{}	\N	2020-10-12 14:11:56.295462
25035	AIT	2020-10-01	55.11	56.04	54.83	55.52	489415	{}	\N	2020-10-12 14:11:56.300791
25036	AIT	2020-10-02	54.45	56.82	54.45	55.52	819215	{}	\N	2020-10-12 14:11:56.305374
25037	AIT	2020-10-05	56.32	57.43	56.205	57.33	572710	{}	\N	2020-10-12 14:11:56.310942
25038	AIT	2020-10-06	58.23	59.42	57.34	57.57	586784	{}	\N	2020-10-12 14:11:56.317626
25039	AIT	2020-10-07	58.27	59.29	58.27	58.87	407009	{}	\N	2020-10-12 14:11:56.321868
25040	AIT	2020-10-08	59.66	60.295	59.02	59.7	401452	{}	\N	2020-10-12 14:11:56.326058
25041	AIT	2020-10-09	60.6	60.85	59.85	60	299269	{}	\N	2020-10-12 14:11:56.330397
25042	AIT	2020-10-12	60.07	60.43	60.07	60.43	5808	{}	\N	2020-10-12 14:11:56.33485
25043	ACIU	2020-09-28	4.8	4.85	4.585	4.78	422149	{}	\N	2020-10-12 14:11:57.887659
25044	ACIU	2020-09-29	4.69	4.925	4.6202	4.81	498793	{}	\N	2020-10-12 14:11:57.898856
25045	ACIU	2020-09-30	4.81	4.9562	4.81	4.88	375289	{}	\N	2020-10-12 14:11:57.906087
25046	ACIU	2020-10-01	4.8883	4.96	4.79	4.85	232396	{}	\N	2020-10-12 14:11:57.912134
25047	ACIU	2020-10-02	4.73	4.885	4.65	4.85	269147	{}	\N	2020-10-12 14:11:57.916779
25048	ACIU	2020-10-05	4.83	5.1	4.77	5.02	385966	{}	\N	2020-10-12 14:11:57.921116
25049	ACIU	2020-10-06	5.02	5.11	4.89	4.94	307674	{}	\N	2020-10-12 14:11:57.924974
25050	ACIU	2020-10-07	4.94	5.04	4.82	4.98	234601	{}	\N	2020-10-12 14:11:57.930442
25051	ACIU	2020-10-08	5.01	5.09	4.97	5.03	125576	{}	\N	2020-10-12 14:11:57.934653
25052	ACIU	2020-10-09	5.1	5.17	4.95	5.13	250937	{}	\N	2020-10-12 14:11:57.938545
25053	ACIU	2020-10-12	5.2	5.205	5.1	5.2	62300	{}	\N	2020-10-12 14:11:57.942715
25054	ACRX	2020-09-28	1.49	1.69	1.47	1.53	1863136	{}	\N	2020-10-12 14:11:59.397077
25055	ACRX	2020-09-29	1.56	1.5695	1.42	1.43	1012937	{}	\N	2020-10-12 14:11:59.409473
25056	ACRX	2020-09-30	1.45	1.47	1.39	1.42	1087708	{}	\N	2020-10-12 14:11:59.417186
25057	ACRX	2020-10-01	1.44	1.44	1.34	1.36	1124110	{}	\N	2020-10-12 14:11:59.423266
25058	ACRX	2020-10-02	1.32	1.47	1.25	1.35	1714381	{}	\N	2020-10-12 14:11:59.429397
25059	ACRX	2020-10-05	1.37	1.41	1.32	1.34	1271590	{}	\N	2020-10-12 14:11:59.434605
25060	ACRX	2020-10-06	1.35	1.425	1.33	1.33	1195794	{}	\N	2020-10-12 14:11:59.439379
25061	ACRX	2020-10-07	1.37	1.47	1.35	1.44	1458122	{}	\N	2020-10-12 14:11:59.443664
25062	ACRX	2020-10-08	1.44	1.6756	1.4333	1.59	2657290	{}	\N	2020-10-12 14:11:59.448246
25063	ACRX	2020-10-09	1.695	1.98	1.625	1.88	4992697	{}	\N	2020-10-12 14:11:59.452707
25064	ACRX	2020-10-12	2	2.025	1.872	1.91	1181815	{}	\N	2020-10-12 14:11:59.456886
25065	ACY	2020-09-28	1.77	1.77	1.64	1.7301	27996	{}	\N	2020-10-12 14:12:00.475533
25066	ACY	2020-09-29	2.1	2.1	1.88	1.98	18542	{}	\N	2020-10-12 14:12:00.485626
25067	ACY	2020-09-30	1.9845	2.02	1.9	1.99	10235	{}	\N	2020-10-12 14:12:00.492359
25068	ACY	2020-10-01	1.9703	1.9703	1.9	1.96	2954	{}	\N	2020-10-12 14:12:00.498114
25069	ACY	2020-10-02	1.9	1.9	1.5373	1.67	58626	{}	\N	2020-10-12 14:12:00.503033
25070	ACY	2020-10-05	1.67	1.7575	1.67	1.75	7201	{}	\N	2020-10-12 14:12:00.50808
25071	ACY	2020-10-06	1.87	2.33	1.7	1.99	115877	{}	\N	2020-10-12 14:12:00.513342
25072	ACY	2020-10-07	2.07	2.14	1.9	1.956	11984	{}	\N	2020-10-12 14:12:00.517771
25073	ACY	2020-10-08	1.8657	1.98	1.8657	1.97	8185	{}	\N	2020-10-12 14:12:00.522137
25074	ACY	2020-10-09	1.9585	1.97	1.92	1.97	2598	{}	\N	2020-10-12 14:12:00.52703
25075	ACY	2020-10-12	2	2	1.9973	2	1129	{}	\N	2020-10-12 14:12:00.532288
25076	AE	2020-09-28	19.935	20.295	19.87	20.01	23373	{}	\N	2020-10-12 14:12:02.282412
25077	AE	2020-09-29	20	20.1	20	20	3690	{}	\N	2020-10-12 14:12:02.292963
25078	AE	2020-09-30	20	20.06	19.9	19.9	7445	{}	\N	2020-10-12 14:12:02.30028
25079	AE	2020-10-01	19.91	19.95	19.77	19.9	5251	{}	\N	2020-10-12 14:12:02.30535
25080	AE	2020-10-02	19.77	20.07	19.77	20	4610	{}	\N	2020-10-12 14:12:02.310853
25081	AE	2020-10-05	19.975	20.135	19.88	19.88	3630	{}	\N	2020-10-12 14:12:02.317657
25082	AE	2020-10-06	19.9	19.9	19.75	19.75	5343	{}	\N	2020-10-12 14:12:02.321212
25083	AE	2020-10-07	19.8937	19.96	19.8291	19.96	2998	{}	\N	2020-10-12 14:12:02.324631
25084	AE	2020-10-08	20	20.09	19.77	19.82	7279	{}	\N	2020-10-12 14:12:02.328347
25085	AE	2020-10-09	19.815	22.5	19.64	21.75	24183	{}	\N	2020-10-12 14:12:02.331854
25086	AE	2020-10-12	21.51	21.53	18.36	20.3201	24930	{}	\N	2020-10-12 14:12:02.33528
25087	AEM	2020-09-28	78.99	79.26	77.74	78.28	1081076	{}	\N	2020-10-12 14:12:03.860531
25088	AEM	2020-09-29	79	80.43	78.68	79.8	2007829	{}	\N	2020-10-12 14:12:03.869308
25089	AEM	2020-09-30	79.54	80.49	78.37	79.61	2414805	{}	\N	2020-10-12 14:12:03.874925
25090	AEM	2020-10-01	80.7	81.9473	80.19	81.08	2138701	{}	\N	2020-10-12 14:12:03.880644
25091	AEM	2020-10-02	80.49	81.09	79.41	79.58	1412028	{}	\N	2020-10-12 14:12:03.885133
25092	AEM	2020-10-05	79.6	81.66	79.6	80.9	1740961	{}	\N	2020-10-12 14:12:03.889863
25093	AEM	2020-10-06	81.46	82.085	77.89	77.92	2222675	{}	\N	2020-10-12 14:12:03.894003
25094	AEM	2020-10-07	78.93	79.25	76.97	77.6	2312620	{}	\N	2020-10-12 14:12:03.898027
25095	AEM	2020-10-08	78.15	79.38	77.86	78.55	2156362	{}	\N	2020-10-12 14:12:03.901761
25096	AEM	2020-10-09	79.69	82.73	79.69	82.69	2440688	{}	\N	2020-10-12 14:12:03.905592
25097	AEM	2020-10-12	82.67	83.46	82.13	83.14	273159	{}	\N	2020-10-12 14:12:03.910054
25098	ADSK	2020-09-28	232.5	234.25	229.1	232.15	914434	{}	\N	2020-10-12 14:12:05.69731
25099	ADSK	2020-09-29	232.51	234.38	230.57	232.92	912386	{}	\N	2020-10-12 14:12:05.708796
25100	ADSK	2020-09-30	232.29	236.71	228.945	231.01	1345160	{}	\N	2020-10-12 14:12:05.717612
25101	ADSK	2020-10-01	232.76	238.61	232.75	233.96	1353455	{}	\N	2020-10-12 14:12:05.723964
25102	ADSK	2020-10-02	227.9	231.14	223.51	225.47	994859	{}	\N	2020-10-12 14:12:05.729776
25103	ADSK	2020-10-05	226.86	230.13	225.04	228.93	895069	{}	\N	2020-10-12 14:12:05.735337
25104	ADSK	2020-10-06	228.32	231.57	226.53	228.2	752411	{}	\N	2020-10-12 14:12:05.740294
25105	ADSK	2020-10-07	229.31	234.21	229.31	233.01	997297	{}	\N	2020-10-12 14:12:05.745044
25106	ADSK	2020-10-08	237.23	238	232.26	234.31	1322113	{}	\N	2020-10-12 14:12:05.749665
25107	ADSK	2020-10-09	234.38	238.14	233.32	237.5	1721979	{}	\N	2020-10-12 14:12:05.754649
25108	ADSK	2020-10-12	243.46	245.78	239.92	240.955	168178	{}	\N	2020-10-12 14:12:05.759566
25109	AFB	2020-09-28	13.78	13.94	13.7785	13.9	55380	{}	\N	2020-10-12 14:12:07.672733
25110	AFB	2020-09-29	13.92	13.92	13.87	13.91	60468	{}	\N	2020-10-12 14:12:07.68282
25111	AFB	2020-09-30	13.94	13.95	13.85	13.94	69295	{}	\N	2020-10-12 14:12:07.689898
25112	AFB	2020-10-01	13.89	13.89	13.72	13.72	145370	{}	\N	2020-10-12 14:12:07.697236
25113	AFB	2020-10-02	13.7	13.78	13.7	13.75	64684	{}	\N	2020-10-12 14:12:07.703571
25114	AFB	2020-10-05	13.77	13.82	13.54	13.63	85590	{}	\N	2020-10-12 14:12:07.708665
25115	AFB	2020-10-06	13.6	13.65	13.54	13.54	102072	{}	\N	2020-10-12 14:12:07.713975
25116	AFB	2020-10-07	13.57	13.64	13.52	13.52	92084	{}	\N	2020-10-12 14:12:07.719043
25117	AFB	2020-10-08	13.54	13.5899	13.53	13.58	41226	{}	\N	2020-10-12 14:12:07.72443
25118	AFB	2020-10-09	13.6	13.6129	13.54	13.55	47562	{}	\N	2020-10-12 14:12:07.729852
25119	AFB	2020-10-12	13.55	13.595	13.55	13.5885	15078	{}	\N	2020-10-12 14:12:07.734672
25120	AG	2020-09-28	9.82	9.9	9.33	9.57	7398361	{}	\N	2020-10-12 14:12:10.408037
25121	AG	2020-09-29	9.63	9.79	9.49	9.55	14362761	{}	\N	2020-10-12 14:12:10.415661
25122	AG	2020-09-30	9.5	9.66	9.3311	9.52	8702671	{}	\N	2020-10-12 14:12:10.422032
25123	AG	2020-10-01	9.6	9.89	9.535	9.76	8440137	{}	\N	2020-10-12 14:12:10.428337
25124	AG	2020-10-02	9.74	9.86	9.57	9.65	9413346	{}	\N	2020-10-12 14:12:10.434177
25125	AG	2020-10-05	9.71	10.06	9.69	9.94	8303294	{}	\N	2020-10-12 14:12:10.440473
25126	AG	2020-10-06	9.97	10.04	9.41	9.43	10302294	{}	\N	2020-10-12 14:12:10.445431
25127	AG	2020-10-07	9.62	9.67	9.46	9.59	9937265	{}	\N	2020-10-12 14:12:10.450206
25128	AG	2020-10-08	9.7	9.87	9.63	9.83	7746688	{}	\N	2020-10-12 14:12:10.456001
25129	AG	2020-10-09	10.13	10.6	10.09	10.56	13431144	{}	\N	2020-10-12 14:12:10.470347
25130	AG	2020-10-12	10.58	10.5817	10.31	10.53	1768196	{}	\N	2020-10-12 14:12:10.478653
25131	AGGP	2020-09-28	19.7477	19.7654	19.7477	19.7654	613	{}	\N	2020-10-12 14:12:12.922813
25132	AGGP	2020-09-29	19.7736	19.7736	19.76	19.76	994	{}	\N	2020-10-12 14:12:12.933145
25133	AGGP	2020-09-30	19.73	19.755	19.72	19.72	2884	{}	\N	2020-10-12 14:12:12.94009
25134	AGGP	2020-10-01	19.7481	19.7945	19.71	19.7945	2244	{}	\N	2020-10-12 14:12:12.946039
25135	AGGP	2020-10-02	19.82	19.82	19.74	19.7686	4329	{}	\N	2020-10-12 14:12:12.952352
25136	AGGP	2020-10-05	19.83	19.83	19.7743	19.8	2324	{}	\N	2020-10-12 14:12:12.957101
25137	AGGP	2020-10-06	19.7664	19.7873	19.76	19.7745	1958	{}	\N	2020-10-12 14:12:12.961363
25138	AGGP	2020-10-07	19.76	19.7624	19.71	19.73	3675	{}	\N	2020-10-12 14:12:12.965534
25139	AGGP	2020-10-08	19.72	19.7282	19.71	19.7282	2187	{}	\N	2020-10-12 14:12:12.970379
25140	AGGP	2020-10-09	19.74	19.7656	19.74	19.7656	4414	{}	\N	2020-10-12 14:12:12.974789
25141	AGGP	2020-10-12	19.8026	19.8026	19.8026	19.8026	1050	{}	\N	2020-10-12 14:12:12.979004
25142	AGNCN	2020-09-28	24.62	24.72	24.5	24.56	39670	{}	\N	2020-10-12 14:12:15.740615
25143	AGNCN	2020-09-29	24.56	24.66	24.33	24.45	37866	{}	\N	2020-10-12 14:12:15.750763
25144	AGNCN	2020-09-30	24.29	24.29	23.6601	23.83	68914	{}	\N	2020-10-12 14:12:15.757782
25145	AGNCN	2020-10-01	23.99	23.99	23.65	23.79	29644	{}	\N	2020-10-12 14:12:15.763526
25146	AGNCN	2020-10-02	23.65	23.69	23.4	23.68	13801	{}	\N	2020-10-12 14:12:15.769535
25147	AGNCN	2020-10-05	23.73	23.79	23.59	23.62	38851	{}	\N	2020-10-12 14:12:15.774624
25148	AGNCN	2020-10-06	23.67	23.8787	23.51	23.62	39286	{}	\N	2020-10-12 14:12:15.779502
25149	AGNCN	2020-10-07	23.71	23.72	23.53	23.6	30883	{}	\N	2020-10-12 14:12:15.784329
25150	AGNCN	2020-10-08	23.65	23.85	23.5869	23.78	25880	{}	\N	2020-10-12 14:12:15.788813
25151	AGNCN	2020-10-09	23.84	23.84	23.62	23.73	20166	{}	\N	2020-10-12 14:12:15.7933
25152	AGNCN	2020-10-12	23.82	23.82	23.66	23.66	4010	{}	\N	2020-10-12 14:12:15.798528
25153	AGX	2020-09-28	42.09	43.02	41.74	42.5	67382	{}	\N	2020-10-12 14:12:17.168573
25154	AGX	2020-09-29	42.38	42.38	41.25	41.79	105494	{}	\N	2020-10-12 14:12:17.177549
25155	AGX	2020-09-30	41.84	42.615	41.56	41.91	146061	{}	\N	2020-10-12 14:12:17.183219
25156	AGX	2020-10-01	42.135	42.135	41.605	41.82	129519	{}	\N	2020-10-12 14:12:17.189224
25157	AGX	2020-10-02	41.31	42.3801	41.28	42.1	228371	{}	\N	2020-10-12 14:12:17.195454
25158	AGX	2020-10-05	42.69	44.325	42.65	44.2	184264	{}	\N	2020-10-12 14:12:17.200144
25159	AGX	2020-10-06	44.4	44.94	43.53	43.84	143596	{}	\N	2020-10-12 14:12:17.204531
25160	AGX	2020-10-07	43.88	45.28	43.88	44.17	271845	{}	\N	2020-10-12 14:12:17.209311
25161	AGX	2020-10-08	44.29	44.73	43.91	44.16	161807	{}	\N	2020-10-12 14:12:17.213894
25162	AGX	2020-10-09	44.19	44.77	43.58	43.86	103226	{}	\N	2020-10-12 14:12:17.218111
25163	AGX	2020-10-12	43.85	44.04	43.75	43.9	13229	{}	\N	2020-10-12 14:12:17.222928
25164	AHT	2020-09-28	1.79	2.88	1.68	1.94	12842378	{}	\N	2020-10-12 14:12:18.34668
25165	AHT	2020-09-29	1.83	1.83	1.59	1.71	3046668	{}	\N	2020-10-12 14:12:18.356153
25166	AHT	2020-09-30	1.66	1.71	1.62	1.65	1214401	{}	\N	2020-10-12 14:12:18.362717
25167	AHT	2020-10-01	1.7	1.77	1.5959	1.65	1138790	{}	\N	2020-10-12 14:12:18.368861
25168	AHT	2020-10-02	1.87	1.87	1.58	1.77	5992179	{}	\N	2020-10-12 14:12:18.374001
25169	AHT	2020-10-05	1.74	1.78	1.59	1.66	1789124	{}	\N	2020-10-12 14:12:18.378731
25170	AHT	2020-10-06	1.64	1.64	1.59	1.6	1163574	{}	\N	2020-10-12 14:12:18.382616
25171	AHT	2020-10-07	1.39	1.5192	1.32	1.47	4516529	{}	\N	2020-10-12 14:12:18.386942
25172	AHT	2020-10-08	1.5	1.5266	1.46	1.48	1179110	{}	\N	2020-10-12 14:12:18.391014
25173	AHT	2020-10-09	1.48	1.57	1.4511	1.56	1731186	{}	\N	2020-10-12 14:12:18.395067
25174	AHT	2020-10-12	1.65	1.65	1.55	1.5653	176669	{}	\N	2020-10-12 14:12:18.399037
25175	AIR	2020-09-28	20	20.9	19.85	19.86	479467	{}	\N	2020-10-12 14:12:19.897703
25176	AIR	2020-09-29	19.7	19.72	18.82	19.11	517562	{}	\N	2020-10-12 14:12:19.904972
25177	AIR	2020-09-30	19.24	19.715	18.66	18.8	870247	{}	\N	2020-10-12 14:12:19.910403
25178	AIR	2020-10-01	18.95	19.5	18.8474	19.09	555597	{}	\N	2020-10-12 14:12:19.915649
25179	AIR	2020-10-02	18.48	19.59	18.37	19.48	427455	{}	\N	2020-10-12 14:12:19.920207
25180	AIR	2020-10-05	19.62	19.86	19.38	19.4	466967	{}	\N	2020-10-12 14:12:19.924222
25181	AIR	2020-10-06	19.68	19.94	19.01	19.03	425344	{}	\N	2020-10-12 14:12:19.928354
25182	AIR	2020-10-07	19.42	19.83	19.07	19.33	564271	{}	\N	2020-10-12 14:12:19.932346
25183	AIR	2020-10-08	19.71	20.55	19.39	20.41	518794	{}	\N	2020-10-12 14:12:19.935971
25184	AIR	2020-10-09	20.66	20.75	20.1384	20.48	533941	{}	\N	2020-10-12 14:12:19.939657
25185	AIR	2020-10-12	20.31	20.68	20.2	20.64	19122	{}	\N	2020-10-12 14:12:19.943559
25186	AJG	2020-09-28	104.03	105.415	103.26	104.96	617413	{}	\N	2020-10-12 14:12:20.946981
25187	AJG	2020-09-29	104.68	105.44	104.35	104.77	855272	{}	\N	2020-10-12 14:12:20.956721
25188	AJG	2020-09-30	105.26	106.41	104.55	105.58	2240394	{}	\N	2020-10-12 14:12:20.964581
25189	AJG	2020-10-01	105.78	106.46	104.545	105.61	1333536	{}	\N	2020-10-12 14:12:20.969999
25190	AJG	2020-10-02	104.53	106.94	103.95	106.21	945987	{}	\N	2020-10-12 14:12:20.974832
25191	AJG	2020-10-05	106.72	107.65	106.4	107.06	1100561	{}	\N	2020-10-12 14:12:20.98029
25192	AJG	2020-10-06	107.73	108.055	105.39	105.76	1053528	{}	\N	2020-10-12 14:12:20.984355
25193	AJG	2020-10-07	106.54	107.22	105.67	106.42	988881	{}	\N	2020-10-12 14:12:20.988637
25194	AJG	2020-10-08	106.66	107.79	106.3693	107.11	919609	{}	\N	2020-10-12 14:12:20.993153
25195	AJG	2020-10-09	107.52	108.84	107.51	108.55	848049	{}	\N	2020-10-12 14:12:20.99726
25196	AJG	2020-10-12	108.55	109.07	108.4	109.07	44304	{}	\N	2020-10-12 14:12:21.001317
25197	AFG	2020-09-28	65.85	67.54	65.85	66.76	517246	{}	\N	2020-10-12 14:12:23.191769
25198	AFG	2020-09-29	66.7	66.7	64.75	66.15	1039914	{}	\N	2020-10-12 14:12:23.200888
25199	AFG	2020-09-30	67	67.93	66.25	66.98	805491	{}	\N	2020-10-12 14:12:23.207186
25200	AFG	2020-10-01	67.5	69.62	67.5	68.95	1110960	{}	\N	2020-10-12 14:12:23.213006
25201	AFG	2020-10-02	67.15	70.18	67.15	69.73	761399	{}	\N	2020-10-12 14:12:23.217679
25202	AFG	2020-10-05	70.88	71.62	70.27	70.6	878705	{}	\N	2020-10-12 14:12:23.222373
25203	AFG	2020-10-06	71.59	72.75	70.12	70.6	936304	{}	\N	2020-10-12 14:12:23.226853
25204	AFG	2020-10-07	71.45	72.1599	70.36	71.6	559451	{}	\N	2020-10-12 14:12:23.230768
25205	AFG	2020-10-08	72.16	73.11	71.8	72.94	546955	{}	\N	2020-10-12 14:12:23.234576
25206	AFG	2020-10-09	73.8	74.29	72.8	73.4	971216	{}	\N	2020-10-12 14:12:23.23837
25207	AFG	2020-10-12	73.15	74.17	73.15	74.01	80008	{}	\N	2020-10-12 14:12:23.242498
25208	AFL	2020-09-28	36.36	36.95	36.25	36.33	3602499	{}	\N	2020-10-12 14:12:25.004718
25209	AFL	2020-09-29	36.1	36.34	35.6	36.02	5220888	{}	\N	2020-10-12 14:12:25.015118
25210	AFL	2020-09-30	36.25	36.815	36.04	36.35	6804552	{}	\N	2020-10-12 14:12:25.0222
25211	AFL	2020-10-01	36.38	36.7	35.9442	36.39	4118909	{}	\N	2020-10-12 14:12:25.028454
25212	AFL	2020-10-02	35.99	37.15	35.69	36.88	4837562	{}	\N	2020-10-12 14:12:25.033988
25213	AFL	2020-10-05	37.33	37.74	37.14	37.43	6578768	{}	\N	2020-10-12 14:12:25.038511
25214	AFL	2020-10-06	37.83	38.465	37.215	37.37	6850007	{}	\N	2020-10-12 14:12:25.042703
25215	AFL	2020-10-07	37.73	38.395	37.59	37.91	5991897	{}	\N	2020-10-12 14:12:25.046842
25216	AFL	2020-10-08	38.21	38.6	37.93	38.24	4581230	{}	\N	2020-10-12 14:12:25.051051
25217	AFL	2020-10-09	38.445	38.5496	37.935	38.1	4392501	{}	\N	2020-10-12 14:12:25.055014
25218	AFL	2020-10-12	38.14	38.505	38.06	38.48	484581	{}	\N	2020-10-12 14:12:25.058839
25219	AGIO	2020-09-28	35	35.63	34.03	35.25	482692	{}	\N	2020-10-12 14:12:26.326943
25220	AGIO	2020-09-29	35.26	35.79	34.39	34.4	652882	{}	\N	2020-10-12 14:12:26.33376
25221	AGIO	2020-09-30	34.43	35.53	34.43	35	857132	{}	\N	2020-10-12 14:12:26.338804
25222	AGIO	2020-10-01	35.14	35.87	34.675	34.95	785433	{}	\N	2020-10-12 14:12:26.344076
25223	AGIO	2020-10-02	34.18	34.95	33.0207	33.42	744758	{}	\N	2020-10-12 14:12:26.34884
25224	AGIO	2020-10-05	33.79	36.88	33.55	36.35	1540660	{}	\N	2020-10-12 14:12:26.352911
25225	AGIO	2020-10-06	36	36.705	35.42	35.56	771446	{}	\N	2020-10-12 14:12:26.356701
25226	AGIO	2020-10-07	35.84	36.365	35.1	36.12	524205	{}	\N	2020-10-12 14:12:26.362006
25227	AGIO	2020-10-08	36.78	37.22	36.43	36.61	506897	{}	\N	2020-10-12 14:12:26.365829
25228	AGIO	2020-10-09	36.79	36.99	35.5	35.61	337731	{}	\N	2020-10-12 14:12:26.369635
25229	AGIO	2020-10-12	35.85	36.12	35.19	35.77	35427	{}	\N	2020-10-12 14:12:26.373587
25230	AGO	2020-09-28	20.6	21.41	20.53	21.02	932105	{}	\N	2020-10-12 14:12:27.283023
25231	AGO	2020-09-29	20.82	20.83	20.2	20.67	2287389	{}	\N	2020-10-12 14:12:27.292648
25232	AGO	2020-09-30	20.75	21.585	20.745	21.48	2085520	{}	\N	2020-10-12 14:12:27.298895
25233	AGO	2020-10-01	21.63	21.97	21.4	21.94	1597895	{}	\N	2020-10-12 14:12:27.304641
25234	AGO	2020-10-02	24.13	25.77	23.985	25.54	9649563	{}	\N	2020-10-12 14:12:27.309616
25235	AGO	2020-10-05	25.53	27.86	25.53	27.62	6776339	{}	\N	2020-10-12 14:12:27.316015
25236	AGO	2020-10-06	27.83	29.52	27.35	28.14	36359421	{}	\N	2020-10-12 14:12:27.322244
25237	AGO	2020-10-07	28.29	29.31	28.04	28.85	6454593	{}	\N	2020-10-12 14:12:27.326511
25238	AGO	2020-10-08	29.2	29.33	27.99	28.49	3591793	{}	\N	2020-10-12 14:12:27.3305
25239	AGO	2020-10-09	28.45	28.75	27.815	28.05	3759373	{}	\N	2020-10-12 14:12:27.334553
25240	AGO	2020-10-12	27.89	28.15	27.64	27.95	216661	{}	\N	2020-10-12 14:12:27.338526
25241	AGZ	2020-09-28	121.14	121.29	121.13	121.17	32004	{}	\N	2020-10-12 14:12:29.016067
25242	AGZ	2020-09-29	121.17	122	121.17	121.32	34075	{}	\N	2020-10-12 14:12:29.026206
25243	AGZ	2020-09-30	121.26	121.26	121.05	121.08	47305	{}	\N	2020-10-12 14:12:29.032753
25244	AGZ	2020-10-01	121.07	121.465	120.9499	121.14	102721	{}	\N	2020-10-12 14:12:29.038199
25245	AGZ	2020-10-02	121.19	121.19	120.995	121.09	58056	{}	\N	2020-10-12 14:12:29.043086
25246	AGZ	2020-10-05	120.9	120.909	120.79	120.79	27022	{}	\N	2020-10-12 14:12:29.048511
25247	AGZ	2020-10-06	120.81	120.9	120.72	120.86	17390	{}	\N	2020-10-12 14:12:29.052835
25248	AGZ	2020-10-07	120.8	120.84	120.73	120.76	90807	{}	\N	2020-10-12 14:12:29.057036
25249	AGZ	2020-10-08	120.75	120.88	120.75	120.87	68147	{}	\N	2020-10-12 14:12:29.06138
25250	AGZ	2020-10-09	120.88	120.89	120.7727	120.87	45146	{}	\N	2020-10-12 14:12:29.065575
25251	AGZ	2020-10-12	120.92	120.92	120.83	120.83	3713	{}	\N	2020-10-12 14:12:29.06946
25252	AIA	2020-09-28	70.89	70.97	70.71	70.94	44853	{}	\N	2020-10-12 14:12:30.22077
25253	AIA	2020-09-29	70.75	71.07	70.61	70.9	99753	{}	\N	2020-10-12 14:12:30.23034
25254	AIA	2020-09-30	71.2	71.97	71.19	71.82	102395	{}	\N	2020-10-12 14:12:30.237349
25255	AIA	2020-10-01	72.51	72.75	72.1	72.68	131685	{}	\N	2020-10-12 14:12:30.243171
25256	AIA	2020-10-02	71.54	72.45	71.51	71.82	82291	{}	\N	2020-10-12 14:12:30.248019
25257	AIA	2020-10-05	72.1	72.74	72.1	72.74	45109	{}	\N	2020-10-12 14:12:30.252952
25258	AIA	2020-10-06	72.79	73.155	72.55	72.62	122200	{}	\N	2020-10-12 14:12:30.257086
25259	AIA	2020-10-07	73.78	74.09	73.59	74	83725	{}	\N	2020-10-12 14:12:30.261009
25260	AIA	2020-10-08	74.07	74.29	73.87	74.22	57404	{}	\N	2020-10-12 14:12:30.264736
25261	AIA	2020-10-09	74.44	74.88	74.23	74.84	159406	{}	\N	2020-10-12 14:12:30.268735
25262	AIA	2020-10-12	75.51	75.53	75.35	75.49	21211	{}	\N	2020-10-12 14:12:30.272868
25263	AIRG	2020-09-28	13.2	13.9	13.2	13.62	85042	{}	\N	2020-10-12 14:12:31.230584
25264	AIRG	2020-09-29	13.58	13.66	13.33	13.49	75453	{}	\N	2020-10-12 14:12:31.239223
25265	AIRG	2020-09-30	13.41	13.64	13.2001	13.34	65867	{}	\N	2020-10-12 14:12:31.245954
25266	AIRG	2020-10-01	13.39	13.6512	13.22	13.53	64534	{}	\N	2020-10-12 14:12:31.2522
25267	AIRG	2020-10-02	13.34	13.635	13.18	13.38	45683	{}	\N	2020-10-12 14:12:31.257297
25268	AIRG	2020-10-05	13.42	13.85	13.41	13.63	71584	{}	\N	2020-10-12 14:12:31.262022
25269	AIRG	2020-10-06	13.61	14.3	13.41	13.54	115509	{}	\N	2020-10-12 14:12:31.266091
25270	AIRG	2020-10-07	13.62	14	13.61	13.86	34758	{}	\N	2020-10-12 14:12:31.270073
25271	AIRG	2020-10-08	13.96	14	13.85	13.94	40323	{}	\N	2020-10-12 14:12:31.274308
25272	AIRG	2020-10-09	13.98	14.54	13.98	14.19	56112	{}	\N	2020-10-12 14:12:31.278238
25273	AIRG	2020-10-12	14.25	14.25	14.0434	14.0585	2147	{}	\N	2020-10-12 14:12:31.282125
25274	AJRD	2020-09-28	41.89	42.17	41.07	41.15	309274	{}	\N	2020-10-12 14:12:32.470099
25275	AJRD	2020-09-29	41.01	41.02	40.33	40.39	813555	{}	\N	2020-10-12 14:12:32.481213
25276	AJRD	2020-09-30	40.43	41.04	39.68	39.89	1304892	{}	\N	2020-10-12 14:12:32.488563
25277	AJRD	2020-10-01	39.94	40.72	39.55	40.1	2667688	{}	\N	2020-10-12 14:12:32.494602
25278	AJRD	2020-10-02	39.26	40.31	38.61	40.06	3868387	{}	\N	2020-10-12 14:12:32.500292
25279	AJRD	2020-10-05	40.49	41.0799	40.145	40.39	1132498	{}	\N	2020-10-12 14:12:32.505335
25280	AJRD	2020-10-06	40.79	41.31	40.57	40.68	1071591	{}	\N	2020-10-12 14:12:32.509827
25281	AJRD	2020-10-07	40.98	41.14	40.35	40.46	1090904	{}	\N	2020-10-12 14:12:32.51411
25282	AJRD	2020-10-08	41.01	41.24	40.62	41.1	996569	{}	\N	2020-10-12 14:12:32.518887
25283	AJRD	2020-10-09	41.43	41.63	40.63	40.82	1211449	{}	\N	2020-10-12 14:12:32.52345
25284	AJRD	2020-10-12	41.02	41.5	40.9	41.32	84317	{}	\N	2020-10-12 14:12:32.527866
25285	ALKS	2020-09-28	16.87	16.9	16.56	16.89	888421	{}	\N	2020-10-12 14:12:35.025027
25286	ALKS	2020-09-29	17.02	17.29	16.495	16.53	710477	{}	\N	2020-10-12 14:12:35.035081
25287	ALKS	2020-09-30	16.41	16.62	16.21	16.57	877850	{}	\N	2020-10-12 14:12:35.041239
25288	ALKS	2020-10-01	16.61	16.87	15.965	16.24	972265	{}	\N	2020-10-12 14:12:35.046676
25289	ALKS	2020-10-02	15.99	16.22	15.74	15.79	849284	{}	\N	2020-10-12 14:12:35.051011
25290	ALKS	2020-10-05	15.8	16.55	15.8	16.26	1825978	{}	\N	2020-10-12 14:12:35.055238
25291	ALKS	2020-10-06	16.17	16.535	15.6902	15.98	1244999	{}	\N	2020-10-12 14:12:35.059036
25292	ALKS	2020-10-07	16.08	17.09	15.78	16.94	1923876	{}	\N	2020-10-12 14:12:35.062677
25293	ALKS	2020-10-08	16.99	17.18	16.645	16.86	863201	{}	\N	2020-10-12 14:12:35.066308
25294	ALKS	2020-10-12	19	19.11	16.96	17.07	891484	{}	\N	2020-10-12 14:12:35.069687
25295	ALSN	2020-09-28	34.4	35.41	34.4	34.95	701118	{}	\N	2020-10-12 14:12:36.610575
25296	ALSN	2020-09-29	34.94	35.42	34.8	34.86	2905594	{}	\N	2020-10-12 14:12:36.621714
25297	ALSN	2020-09-30	34.91	35.88	34.89	35.14	2192715	{}	\N	2020-10-12 14:12:36.629407
25298	ALSN	2020-10-01	35.51	35.79	34.3	34.64	2275740	{}	\N	2020-10-12 14:12:36.635608
25299	ALSN	2020-10-02	33.92	35.525	33.91	34.92	2445491	{}	\N	2020-10-12 14:12:36.641335
25300	ALSN	2020-10-05	35.29	36.48	35.09	36.16	1548612	{}	\N	2020-10-12 14:12:36.647629
25301	ALSN	2020-10-06	36.6	37.655	36.51	36.78	1931926	{}	\N	2020-10-12 14:12:36.65245
25302	ALSN	2020-10-07	37.32	39.955	37.18	39.79	3488018	{}	\N	2020-10-12 14:12:36.657348
25303	ALSN	2020-10-08	39.9	40.975	39.75	40.8	2903778	{}	\N	2020-10-12 14:12:36.662518
25304	ALSN	2020-10-09	41	41.51	40.73	40.77	1848305	{}	\N	2020-10-12 14:12:36.667312
25305	ALSN	2020-10-12	40.7	41.0098	40.59	40.92	150986	{}	\N	2020-10-12 14:12:36.672023
25306	GLO	2020-09-28	9.1	9.23	9.06	9.19	128527	{}	\N	2020-10-12 14:12:37.79727
25307	GLO	2020-09-29	9.24	9.28	9.12	9.19	78244	{}	\N	2020-10-12 14:12:37.805472
25308	GLO	2020-09-30	9.25	9.361	9.1991	9.32	184127	{}	\N	2020-10-12 14:12:37.811886
25309	GLO	2020-10-01	9.4	9.47	9.31	9.42	188301	{}	\N	2020-10-12 14:12:37.817279
25310	GLO	2020-10-02	9.4	9.43	9.285	9.43	154155	{}	\N	2020-10-12 14:12:37.822258
25311	GLO	2020-10-05	9.45	9.57	9.44	9.56	153539	{}	\N	2020-10-12 14:12:37.827057
25312	GLO	2020-10-06	9.57	9.6978	9.5	9.5	135029	{}	\N	2020-10-12 14:12:37.831418
25313	GLO	2020-10-07	9.64	9.7	9.61	9.69	118375	{}	\N	2020-10-12 14:12:37.835477
25314	GLO	2020-10-08	9.75	9.75	9.7	9.75	116890	{}	\N	2020-10-12 14:12:37.839542
25315	GLO	2020-10-09	9.79	9.81	9.74	9.81	91873	{}	\N	2020-10-12 14:12:37.843587
25316	GLO	2020-10-12	9.9	9.9	9.85	9.89	43854	{}	\N	2020-10-12 14:12:37.847774
25317	AEMD	2020-09-28	1.35	1.375	1.34	1.36	53825	{}	\N	2020-10-12 14:12:38.853308
25318	AEMD	2020-09-29	1.36	1.45	1.36	1.43	221911	{}	\N	2020-10-12 14:12:38.862143
25319	AEMD	2020-09-30	1.42	1.4399	1.35	1.35	287911	{}	\N	2020-10-12 14:12:38.868486
25320	AEMD	2020-10-01	1.37	1.398	1.31	1.32	233027	{}	\N	2020-10-12 14:12:38.874451
25321	AEMD	2020-10-02	1.33	1.36	1.32	1.35	182816	{}	\N	2020-10-12 14:12:38.879372
25322	AEMD	2020-10-05	1.35	1.36	1.29	1.32	186279	{}	\N	2020-10-12 14:12:38.884073
25323	AEMD	2020-10-06	1.36	1.38	1.31	1.31	98543	{}	\N	2020-10-12 14:12:38.888324
25324	AEMD	2020-10-07	1.34	1.4183	1.34	1.39	397581	{}	\N	2020-10-12 14:12:38.892849
25325	AEMD	2020-10-08	1.4	1.47	1.38	1.46	231205	{}	\N	2020-10-12 14:12:38.896919
25326	AEMD	2020-10-09	1.46	1.54	1.45	1.52	193855	{}	\N	2020-10-12 14:12:38.901277
25327	AEMD	2020-10-12	1.52	1.52	1.46	1.46	12509	{}	\N	2020-10-12 14:12:38.905164
25328	AERI	2020-09-28	11.3	11.38	10.84	11.03	464707	{}	\N	2020-10-12 14:12:40.117227
25329	AERI	2020-09-29	11.13	11.46	10.97	11.43	497757	{}	\N	2020-10-12 14:12:40.126517
25330	AERI	2020-09-30	11.47	12.05	11.45	11.77	399119	{}	\N	2020-10-12 14:12:40.133394
25331	AERI	2020-10-01	11.77	12.09	11.6	11.84	343459	{}	\N	2020-10-12 14:12:40.138649
25332	AERI	2020-10-02	11.54	12.05	11.3701	11.97	359304	{}	\N	2020-10-12 14:12:40.143705
25333	AERI	2020-10-05	12.13	12.55	12.03	12.25	606009	{}	\N	2020-10-12 14:12:40.148427
25334	AERI	2020-10-06	12.34	12.58	11.9	12.04	639664	{}	\N	2020-10-12 14:12:40.153174
25335	AERI	2020-10-07	12.13	12.435	11.41	11.88	642302	{}	\N	2020-10-12 14:12:40.157491
25336	AERI	2020-10-08	11.8976	12.38	11.88	11.895	727482	{}	\N	2020-10-12 14:12:40.161486
25337	AERI	2020-10-09	11.92	12.04	11.58	11.83	685345	{}	\N	2020-10-12 14:12:40.165222
25338	AERI	2020-10-12	11.83	11.93	11.48	11.695	39159	{}	\N	2020-10-12 14:12:40.169914
25891	AMZA	2020-10-05	13.79	14.2	13.5979	14.18	117104	{}	\N	2020-10-12 14:14:05.651127
25892	AMZA	2020-10-06	14.45	14.5699	13.63	13.86	82714	{}	\N	2020-10-12 14:14:05.656073
25893	AMZA	2020-10-07	14.04	14.04	13.8	13.97	69860	{}	\N	2020-10-12 14:14:05.661316
25894	AMZA	2020-10-08	14.14	15.09	14.078	15.07	173807	{}	\N	2020-10-12 14:14:05.666412
25895	AMZA	2020-10-09	15.22	15.265	14.8	15.04	78316	{}	\N	2020-10-12 14:14:05.671346
25896	AMZA	2020-10-12	15.02	15.02	14.8487	14.85	24277	{}	\N	2020-10-12 14:14:05.676387
25897	ANF	2020-09-28	14.81	15.13	14.65	14.76	1514926	{}	\N	2020-10-12 14:14:06.643464
25898	ANF	2020-09-29	14.89	14.89	13.7	13.96	4038696	{}	\N	2020-10-12 14:14:06.653224
25899	ANF	2020-09-30	14.08	14.48	13.77	13.93	3733483	{}	\N	2020-10-12 14:14:06.660401
25900	ANF	2020-10-01	13.9	14.68	13.9	14.65	6095040	{}	\N	2020-10-12 14:14:06.666182
25901	ANF	2020-10-02	14.28	15.25	14.1	15.2	4092252	{}	\N	2020-10-12 14:14:06.672001
25902	ANF	2020-10-05	15.49	15.9	15.22	15.73	5726053	{}	\N	2020-10-12 14:14:06.676629
25903	ANF	2020-10-06	15.76	16.12	15.44	15.54	4011581	{}	\N	2020-10-12 14:14:06.680792
25904	ANF	2020-10-07	15.92	16.52	15.82	16.51	4245388	{}	\N	2020-10-12 14:14:06.684885
25905	ANF	2020-10-08	16.52	16.9484	16.41	16.88	4238268	{}	\N	2020-10-12 14:14:06.68975
25906	ANF	2020-10-09	16.98	17.2	16.35	16.37	3130122	{}	\N	2020-10-12 14:14:06.694215
25907	ANF	2020-10-12	16.46	16.4999	16.02	16.45	757661	{}	\N	2020-10-12 14:14:06.698473
25908	ANY	2020-09-28	2.62	2.68	2.5	2.63	100054	{}	\N	2020-10-12 14:14:07.636269
25909	ANY	2020-09-29	2.63	2.66	2.45	2.59	55407	{}	\N	2020-10-12 14:14:07.645915
25910	ANY	2020-09-30	2.62	2.62	2.47	2.47	113838	{}	\N	2020-10-12 14:14:07.653224
25911	ANY	2020-10-01	2.45	2.54	2.45	2.46	93594	{}	\N	2020-10-12 14:14:07.659509
25912	ANY	2020-10-02	2.45	2.48	2.36	2.4	19625	{}	\N	2020-10-12 14:14:07.665104
25913	ANY	2020-10-05	2.38	2.43	2.23	2.26	93822	{}	\N	2020-10-12 14:14:07.670158
25914	ANY	2020-10-06	2.3	2.39	2.23	2.25	141758	{}	\N	2020-10-12 14:14:07.674606
25915	ANY	2020-10-07	2.23	2.25	2.1194	2.14	269290	{}	\N	2020-10-12 14:14:07.678815
25916	ANY	2020-10-08	2.17	2.32	2.05	2.14	457535	{}	\N	2020-10-12 14:14:07.683228
25917	ANY	2020-10-09	2.17	2.2	1.95	2.06	283962	{}	\N	2020-10-12 14:14:07.687665
25918	ANY	2020-10-12	2.07	2.09	2.05	2.056	19667	{}	\N	2020-10-12 14:14:07.692025
25919	AOSL	2020-09-28	12.42	13.14	12.24	13.08	104199	{}	\N	2020-10-12 14:14:08.99087
25920	AOSL	2020-09-29	12.95	13.38	12.81	13.29	82361	{}	\N	2020-10-12 14:14:09.001178
25921	AOSL	2020-09-30	13.35	13.4	12.71	12.82	82409	{}	\N	2020-10-12 14:14:09.007742
25922	AOSL	2020-10-01	12.99	13.27	12.89	13.14	47113	{}	\N	2020-10-12 14:14:09.013601
25923	AOSL	2020-10-02	12.76	13.4	12.74	13.07	100498	{}	\N	2020-10-12 14:14:09.019328
25924	AOSL	2020-10-05	14.28	15.9792	14.1722	15.38	823938	{}	\N	2020-10-12 14:14:09.024358
25925	AOSL	2020-10-06	15.62	16.5	15.41	15.54	383791	{}	\N	2020-10-12 14:14:09.028764
25926	AOSL	2020-10-07	15.9	17.2	15.81	16.88	242689	{}	\N	2020-10-12 14:14:09.033318
25927	AOSL	2020-10-08	17.11	17.3	16.46	16.65	117659	{}	\N	2020-10-12 14:14:09.038132
25928	AOSL	2020-10-09	16.76	16.9555	16.22	16.36	117048	{}	\N	2020-10-12 14:14:09.043034
25929	AOSL	2020-10-12	16.52	16.78	16.2362	16.38	24373	{}	\N	2020-10-12 14:14:09.047999
25930	APH	2020-09-28	107.18	108.94	107	108.18	1120763	{}	\N	2020-10-12 14:14:10.414675
25931	APH	2020-09-29	108.71	109.28	107.61	108.63	2979768	{}	\N	2020-10-12 14:14:10.424556
25932	APH	2020-09-30	109.04	109.79	107.5906	108.27	3526377	{}	\N	2020-10-12 14:14:10.431164
25933	APH	2020-10-01	109.92	110	108.945	109.79	1884740	{}	\N	2020-10-12 14:14:10.437544
25934	APH	2020-10-02	107.63	110.71	106.98	109.36	2260851	{}	\N	2020-10-12 14:14:10.442306
25935	APH	2020-10-05	110.26	112.63	110.26	111.53	2882053	{}	\N	2020-10-12 14:14:10.447044
25936	APH	2020-10-06	111.29	112.76	110.04	110.14	2321696	{}	\N	2020-10-12 14:14:10.451287
25937	APH	2020-10-07	111.23	113.56	111.23	113.03	1660380	{}	\N	2020-10-12 14:14:10.455218
25938	APH	2020-10-08	113.53	114.55	113.17	113.5	1363751	{}	\N	2020-10-12 14:14:10.459274
25939	APH	2020-10-09	114.53	115.255	113.12	113.49	2384337	{}	\N	2020-10-12 14:14:10.463384
25940	APH	2020-10-12	114.2	116.08	113.74	115.91	725323	{}	\N	2020-10-12 14:14:10.467511
25941	APT	2020-09-28	14.3	14.3846	13.71	14.12	731936	{}	\N	2020-10-12 14:14:11.436112
25942	APT	2020-09-29	13.78	15.03	13.78	14.78	1138792	{}	\N	2020-10-12 14:14:11.445481
25943	APT	2020-09-30	14.52	15.25	14.52	14.78	854990	{}	\N	2020-10-12 14:14:11.451929
25944	APT	2020-10-01	14.82	15.35	14.78	15.27	668752	{}	\N	2020-10-12 14:14:11.457558
25945	APT	2020-10-02	16.5	16.73	15.62	16.03	2607698	{}	\N	2020-10-12 14:14:11.461852
25339	AFI	2020-09-28	3.88	3.92	3.72	3.72	167576	{}	\N	2020-10-12 14:12:41.138548
25340	AFI	2020-09-29	3.67	3.77	3.58	3.62	349817	{}	\N	2020-10-12 14:12:41.153402
25341	AFI	2020-09-30	3.69	3.7276	3.42	3.45	457246	{}	\N	2020-10-12 14:12:41.15848
25342	AFI	2020-10-01	3.48	3.58	3.22	3.4	844713	{}	\N	2020-10-12 14:12:41.162653
25343	AFI	2020-10-02	3.28	3.66	3.2	3.64	436438	{}	\N	2020-10-12 14:12:41.167097
25344	AFI	2020-10-05	3.68	3.88	3.68	3.75	369026	{}	\N	2020-10-12 14:12:41.170898
25345	AFI	2020-10-06	3.81	3.979	3.76	3.77	323429	{}	\N	2020-10-12 14:12:41.174893
25346	AFI	2020-10-07	3.88	3.98	3.83	3.92	237341	{}	\N	2020-10-12 14:12:41.178705
25347	AFI	2020-10-08	3.96	4.14	3.93	4.01	214218	{}	\N	2020-10-12 14:12:41.182642
25348	AFI	2020-10-09	4.06	4.12	3.87	3.87	166712	{}	\N	2020-10-12 14:12:41.186934
25349	AFI	2020-10-12	3.91	3.99	3.9	3.98	17360	{}	\N	2020-10-12 14:12:41.190676
25350	AGCO	2020-09-28	72.69	73.56	72.22	72.9	322406	{}	\N	2020-10-12 14:12:42.158519
25351	AGCO	2020-09-29	72.75	73.29	71.79	72.66	529621	{}	\N	2020-10-12 14:12:42.16441
25352	AGCO	2020-09-30	72.8	75.5	72.8	74.27	1353263	{}	\N	2020-10-12 14:12:42.17014
25353	AGCO	2020-10-01	74.91	75.2	72.65	73.31	1226590	{}	\N	2020-10-12 14:12:42.175349
25354	AGCO	2020-10-02	72.175	75.575	71.91	75.19	649408	{}	\N	2020-10-12 14:12:42.179831
25355	AGCO	2020-10-05	76.11	77.48	75.97	76.67	765714	{}	\N	2020-10-12 14:12:42.184045
25356	AGCO	2020-10-06	77.32	79.12	76.67	77.01	893763	{}	\N	2020-10-12 14:12:42.18835
25357	AGCO	2020-10-07	77.98	79.36	77.58	78.95	729555	{}	\N	2020-10-12 14:12:42.193363
25358	AGCO	2020-10-08	79.29	79.79	78.33	79.66	575591	{}	\N	2020-10-12 14:12:42.197893
25359	AGCO	2020-10-09	80.54	81.1099	79.74	80.57	806373	{}	\N	2020-10-12 14:12:42.202604
25360	AGCO	2020-10-12	81.18	81.18	80.41	80.54	29400	{}	\N	2020-10-12 14:12:42.207136
25361	AGLE	2020-09-28	7.16	7.365	7.1	7.17	155330	{}	\N	2020-10-12 14:12:43.67791
25362	AGLE	2020-09-29	7.15	7.25	6.98	7.07	118862	{}	\N	2020-10-12 14:12:43.687254
25363	AGLE	2020-09-30	7.11	7.27	7	7.09	232694	{}	\N	2020-10-12 14:12:43.693694
25364	AGLE	2020-10-01	7.12	7.84	7.1	7.84	1843260	{}	\N	2020-10-12 14:12:43.700088
25365	AGLE	2020-10-02	7.82	7.82	6.95	7.02	437452	{}	\N	2020-10-12 14:12:43.705418
25366	AGLE	2020-10-05	7.04	7.36	7.03	7.32	235974	{}	\N	2020-10-12 14:12:43.710483
25367	AGLE	2020-10-06	7.33	7.74	7.2	7.46	158550	{}	\N	2020-10-12 14:12:43.714604
25368	AGLE	2020-10-07	7.51	7.87	7.43	7.84	269436	{}	\N	2020-10-12 14:12:43.718693
25369	AGLE	2020-10-08	7.99	8.73	7.89	7.95	296962	{}	\N	2020-10-12 14:12:43.72262
25370	AGLE	2020-10-09	8.02	8.1664	7.76	7.9	92830	{}	\N	2020-10-12 14:12:43.727143
25371	AGLE	2020-10-12	7.91	7.91	7.59	7.88	8379	{}	\N	2020-10-12 14:12:43.731533
25372	AGYS	2020-09-28	24.69	25.22	24.44	24.59	74628	{}	\N	2020-10-12 14:12:47.28375
25373	AGYS	2020-09-29	24.61	24.84	24.39	24.59	86201	{}	\N	2020-10-12 14:12:47.293996
25374	AGYS	2020-09-30	24.63	25.07	24.06	24.16	180566	{}	\N	2020-10-12 14:12:47.303269
25375	AGYS	2020-10-01	24.29	24.94	24.045	24.75	100694	{}	\N	2020-10-12 14:12:47.3086
25376	AGYS	2020-10-02	24.13	24.88	24.13	24.52	59210	{}	\N	2020-10-12 14:12:47.313814
25377	AGYS	2020-10-05	24.75	25.045	24.2	24.91	60547	{}	\N	2020-10-12 14:12:47.321431
25378	AGYS	2020-10-06	25.2	25.765	24.94	24.97	62001	{}	\N	2020-10-12 14:12:47.326087
25379	AGYS	2020-10-07	25.19	25.41	24.86	25.18	82900	{}	\N	2020-10-12 14:12:47.330426
25380	AGYS	2020-10-08	25.39	25.5	25.15	25.36	67006	{}	\N	2020-10-12 14:12:47.335109
25381	AGYS	2020-10-09	25.58	27.72	25.28	27.48	305448	{}	\N	2020-10-12 14:12:47.339404
25382	AGYS	2020-10-12	27.73	27.73	26.5	26.71	22919	{}	\N	2020-10-12 14:12:47.343807
25383	AIEQ	2020-09-28	30.43	30.61	30.35	30.61	44798	{}	\N	2020-10-12 14:12:48.601865
25384	AIEQ	2020-09-29	30.61	30.72	30.4509	30.63	8304	{}	\N	2020-10-12 14:12:48.614266
25385	AIEQ	2020-09-30	30.51	30.97	30.5013	30.64	9486	{}	\N	2020-10-12 14:12:48.623575
25386	AIEQ	2020-10-01	30.75	31.155	30.7	31.11	26216	{}	\N	2020-10-12 14:12:48.629557
25387	AIEQ	2020-10-02	30.49	31.11	30.49	30.9779	14680	{}	\N	2020-10-12 14:12:48.634711
25388	AIEQ	2020-10-05	31.0101	31.51	31.0101	31.485	10636	{}	\N	2020-10-12 14:12:48.639395
25389	AIEQ	2020-10-06	31.43	31.68	31.077	31.0907	9703	{}	\N	2020-10-12 14:12:48.64369
25390	AIEQ	2020-10-07	31.67	31.8166	31.4447	31.765	8157	{}	\N	2020-10-12 14:12:48.647946
25391	AIEQ	2020-10-08	31.97	31.98	31.845	31.85	6656	{}	\N	2020-10-12 14:12:48.653408
25392	AIEQ	2020-10-09	32	32.24	32	32.1796	11584	{}	\N	2020-10-12 14:12:48.657694
25393	AIEQ	2020-10-12	32.38	32.53	32.35	32.38	7579	{}	\N	2020-10-12 14:12:48.661644
25394	AIRI	2020-09-28	1.1	1.19	1.1	1.15	22901	{}	\N	2020-10-12 14:12:50.686991
25395	AIRI	2020-09-29	1.16	1.27	1.16	1.26	94934	{}	\N	2020-10-12 14:12:50.695999
25396	AIRI	2020-09-30	1.27	1.49	1.21	1.22	375300	{}	\N	2020-10-12 14:12:50.701525
25397	AIRI	2020-10-01	1.2102	1.24	1.1	1.13	112033	{}	\N	2020-10-12 14:12:50.707198
25398	AIRI	2020-10-02	1.1	1.13	1.1	1.13	35210	{}	\N	2020-10-12 14:12:50.711976
25399	AIRI	2020-10-05	1.1299	1.19	1.12	1.14	36625	{}	\N	2020-10-12 14:12:50.716545
25400	AIRI	2020-10-06	1.14	1.19	1.105	1.13	60371	{}	\N	2020-10-12 14:12:50.720299
25401	AIRI	2020-10-07	1.15	1.15	1.1	1.15	21554	{}	\N	2020-10-12 14:12:50.724203
25402	AIRI	2020-10-08	1.15	1.15	1.12	1.14	18043	{}	\N	2020-10-12 14:12:50.728195
25403	AIRI	2020-10-09	1.12	1.15	1.09	1.09	102424	{}	\N	2020-10-12 14:12:50.731942
25404	AIRI	2020-10-12	1.09	1.13	1.09	1.1123	5778	{}	\N	2020-10-12 14:12:50.735864
25405	AJX	2020-09-28	9.5	9.815	7.66	8.46	1044073	{}	\N	2020-10-12 14:12:51.78747
25406	AJX	2020-09-29	8.06	8.39	7.92	8.31	2673569	{}	\N	2020-10-12 14:12:51.79851
25407	AJX	2020-09-30	8.2	8.63	8.185	8.29	1770603	{}	\N	2020-10-12 14:12:51.805106
25408	AJX	2020-10-01	8.4	8.7	8.3	8.69	287735	{}	\N	2020-10-12 14:12:51.811493
25409	AJX	2020-10-02	8.41	8.85	8.3686	8.72	241040	{}	\N	2020-10-12 14:12:51.816542
25410	AJX	2020-10-05	8.63	8.84	8.28	8.4	577606	{}	\N	2020-10-12 14:12:51.821391
25411	AJX	2020-10-06	8.44	8.78	8.4	8.42	214854	{}	\N	2020-10-12 14:12:51.825523
25412	AJX	2020-10-07	8.48	8.69	8.37	8.55	415694	{}	\N	2020-10-12 14:12:51.829441
25413	AJX	2020-10-08	8.55	8.7565	8.5081	8.69	187042	{}	\N	2020-10-12 14:12:51.833456
25414	AJX	2020-10-09	8.8	8.8	8.61	8.63	230223	{}	\N	2020-10-12 14:12:51.837536
25415	AJX	2020-10-12	8.65	8.65	8.54	8.54	14752	{}	\N	2020-10-12 14:12:51.841635
25416	ALBO	2020-09-28	33.6	33.82	32.4788	32.9	290334	{}	\N	2020-10-12 14:12:53.734652
25417	ALBO	2020-09-29	32.72	33.075	31.8	32.64	364262	{}	\N	2020-10-12 14:12:53.742948
25418	ALBO	2020-09-30	32.64	34.54	32.64	33.37	205533	{}	\N	2020-10-12 14:12:53.749457
25419	ALBO	2020-10-01	33.31	34.39	32.93	34.24	222740	{}	\N	2020-10-12 14:12:53.755295
25420	ALBO	2020-10-02	33.63	34.79	33	33.59	209661	{}	\N	2020-10-12 14:12:53.760355
25421	ALBO	2020-10-05	33.81	35.52	33.69	35.3	164981	{}	\N	2020-10-12 14:12:53.764602
25422	ALBO	2020-10-06	35.22	36.82	34.81	35.64	158330	{}	\N	2020-10-12 14:12:53.768665
25423	ALBO	2020-10-07	35.87	37.03	35.7	36.88	209693	{}	\N	2020-10-12 14:12:53.772782
25424	ALBO	2020-10-08	37.01	38.1999	37.01	37.3	164859	{}	\N	2020-10-12 14:12:53.776478
25425	ALBO	2020-10-09	37.42	37.93	36.82	37.66	140262	{}	\N	2020-10-12 14:12:53.780507
25426	ALBO	2020-10-12	38.34	38.34	37.14	37.93	15850	{}	\N	2020-10-12 14:12:53.784261
25427	ALL	2020-09-28	92.09	94.13	91.89	93.22	1257384	{}	\N	2020-10-12 14:12:55.060416
25428	ALL	2020-09-29	92.84	93.79	92.3	93.44	4489980	{}	\N	2020-10-12 14:12:55.069004
25429	ALL	2020-09-30	93.55	94.68	93.32	94.14	3816637	{}	\N	2020-10-12 14:12:55.074945
25430	ALL	2020-10-01	95.1	95.15	91.66	92.26	4453698	{}	\N	2020-10-12 14:12:55.080287
25431	ALL	2020-10-02	91.34	94.36	91.18	93.02	2282712	{}	\N	2020-10-12 14:12:55.085186
25432	ALL	2020-10-05	93.81	94.95	93.14	93.68	2947656	{}	\N	2020-10-12 14:12:55.090169
25433	ALL	2020-10-06	93.99	94.83	92.27	92.65	3840497	{}	\N	2020-10-12 14:12:55.094439
25434	ALL	2020-10-07	93.4	94.84	93.03	93.29	2836885	{}	\N	2020-10-12 14:12:55.098526
25435	ALL	2020-10-08	93.6	94.9	93.26	93.58	2796042	{}	\N	2020-10-12 14:12:55.102494
25436	ALL	2020-10-09	93.78	93.97	92.77	92.86	2748347	{}	\N	2020-10-12 14:12:55.106797
25437	ALL	2020-10-12	93.11	93.94	92.9	93.91	319971	{}	\N	2020-10-12 14:12:55.110873
25438	ALSK	2020-09-28	1.93	2	1.93	1.96	130485	{}	\N	2020-10-12 14:12:56.527082
25439	ALSK	2020-09-29	1.95	2.05	1.939	2.05	145037	{}	\N	2020-10-12 14:12:56.535859
25440	ALSK	2020-09-30	2.05	2.055	1.98	2	212198	{}	\N	2020-10-12 14:12:56.541704
25441	ALSK	2020-10-01	2	2.06	1.9911	2.03	124552	{}	\N	2020-10-12 14:12:56.547462
25442	ALSK	2020-10-02	2.02	2.03	1.98	2.02	109667	{}	\N	2020-10-12 14:12:56.552453
25443	ALSK	2020-10-05	2.03	2.07	2.02	2.07	101338	{}	\N	2020-10-12 14:12:56.557104
25444	ALSK	2020-10-06	2.03	2.08	1.96	1.96	178057	{}	\N	2020-10-12 14:12:56.561347
25445	ALSK	2020-10-07	1.99	2.01	1.96	1.99	129899	{}	\N	2020-10-12 14:12:56.565382
25446	ALSK	2020-10-08	2	2.04	1.9701	2.03	103603	{}	\N	2020-10-12 14:12:56.569408
25447	ALSK	2020-10-09	2.05	2.05	2	2.04	174358	{}	\N	2020-10-12 14:12:56.573525
25448	ALSK	2020-10-12	2.0365	2.05	2.01	2.03	10773	{}	\N	2020-10-12 14:12:56.577432
25449	AFK	2020-09-28	17.2	17.525	17.2	17.29	8725	{}	\N	2020-10-12 14:12:58.482296
25450	AFK	2020-09-29	17.46	17.4903	17.3408	17.35	2531	{}	\N	2020-10-12 14:12:58.495638
25451	AFK	2020-09-30	17.58	17.635	17.39	17.5	5076	{}	\N	2020-10-12 14:12:58.50192
25452	AFK	2020-10-01	17.7	17.7	17.465	17.465	1768	{}	\N	2020-10-12 14:12:58.508364
25453	AFK	2020-10-02	17.25	17.5927	17.16	17.29	10171	{}	\N	2020-10-12 14:12:58.515199
25454	AFK	2020-10-05	17.38	17.93	17.38	17.52	7966	{}	\N	2020-10-12 14:12:58.526048
25455	AFK	2020-10-06	17.59	17.83	17.5585	17.62	12471	{}	\N	2020-10-12 14:12:58.532254
25456	AFK	2020-10-07	17.62	17.83	17.62	17.8	3995	{}	\N	2020-10-12 14:12:58.541701
25457	AFK	2020-10-08	17.85	17.85	17.669	17.7902	5406	{}	\N	2020-10-12 14:12:58.54845
25458	AFK	2020-10-09	17.85	18.01	17.85	17.92	4110	{}	\N	2020-10-12 14:12:58.55619
25459	AFK	2020-10-12	17.99	17.9935	17.96	17.9935	1236	{}	\N	2020-10-12 14:12:58.565385
25460	AGNC	2020-09-28	14.14	14.3982	14.13	14.27	6657258	{}	\N	2020-10-12 14:13:00.175124
25461	AGNC	2020-09-29	14.18	14.1866	13.865	13.925	6014984	{}	\N	2020-10-12 14:13:00.183723
25462	AGNC	2020-09-30	13.98	14.0577	13.88	13.91	7074956	{}	\N	2020-10-12 14:13:00.191521
25463	AGNC	2020-10-01	13.9518	13.9518	13.72	13.9	6144472	{}	\N	2020-10-12 14:13:00.196588
25464	AGNC	2020-10-02	13.7081	14.01	13.68	13.94	6097674	{}	\N	2020-10-12 14:13:00.201162
25465	AGNC	2020-10-05	14.01	14.1	13.95	13.995	4589925	{}	\N	2020-10-12 14:13:00.205241
25466	AGNC	2020-10-06	14.02	14.17	13.91	13.94	6010619	{}	\N	2020-10-12 14:13:00.209615
25467	AGNC	2020-10-07	14.06	14.07	13.85	13.99	6122875	{}	\N	2020-10-12 14:13:00.213864
25468	AGNC	2020-10-08	14.07	14.29	14.04	14.22	11490147	{}	\N	2020-10-12 14:13:00.218545
25469	AGNC	2020-10-09	14.27	14.33	14.04	14.07	3872317	{}	\N	2020-10-12 14:13:00.223116
25470	AGNC	2020-10-12	14.1	14.14	14.0522	14.1048	974341	{}	\N	2020-10-12 14:13:00.228169
25471	AGT	2020-09-28	20.29	20.29	20.084	20.084	405	{}	\N	2020-10-12 14:13:02.428685
25472	AGT	2020-09-29	20.03	20.03	19.8075	19.8075	882	{}	\N	2020-10-12 14:13:02.436978
25473	AGT	2020-09-30	19.9792	19.9792	19.9792	19.9792	124	{}	\N	2020-10-12 14:13:02.443254
25474	AGT	2020-10-01	20.3301	20.3686	20.3301	20.3686	834	{}	\N	2020-10-12 14:13:02.448832
25475	AGT	2020-10-02	20.1126	20.1126	20.1126	20.1126	103	{}	\N	2020-10-12 14:13:02.454505
25476	AGT	2020-10-05	20.3807	20.3807	20.3807	20.3807	110	{}	\N	2020-10-12 14:13:02.458901
25477	AGT	2020-10-06	20.51	20.51	20.1848	20.1848	527	{}	\N	2020-10-12 14:13:02.4635
25478	AGT	2020-10-07	20.6413	20.6413	20.6413	20.6413	123	{}	\N	2020-10-12 14:13:02.468098
25479	AGT	2020-10-08	20.73	20.7408	20.7	20.7408	1255	{}	\N	2020-10-12 14:13:02.472861
25480	AGT	2020-10-09	21.1441	21.1441	21.1441	21.1441	171	{}	\N	2020-10-12 14:13:02.477409
25481	AHPI	2020-09-28	5.63	5.842	5.6	5.66	54581	{}	\N	2020-10-12 14:13:03.429811
25482	AHPI	2020-09-29	5.66	6.23	5.51	5.53	243111	{}	\N	2020-10-12 14:13:03.440263
25483	AHPI	2020-09-30	5.6	5.77	5.42	5.52	110065	{}	\N	2020-10-12 14:13:03.447037
25484	AHPI	2020-10-01	5.5539	5.75	5.46	5.63	80361	{}	\N	2020-10-12 14:13:03.453311
25485	AHPI	2020-10-02	5.84	6.32	5.67	6.03	785804	{}	\N	2020-10-12 14:13:03.458953
25486	AHPI	2020-10-05	5.97	6.2099	5.7	5.83	339349	{}	\N	2020-10-12 14:13:03.464019
25487	AHPI	2020-10-06	5.91	5.91	5.5	5.6	124393	{}	\N	2020-10-12 14:13:03.468897
25488	AHPI	2020-10-07	5.7	5.85	5.52	5.58	140326	{}	\N	2020-10-12 14:13:03.474233
25489	AHPI	2020-10-08	5.58	5.65	5.33	5.45	94528	{}	\N	2020-10-12 14:13:03.479807
25490	AHPI	2020-10-09	5.5	5.5	5.16	5.24	83051	{}	\N	2020-10-12 14:13:03.493222
25491	AHPI	2020-10-12	5.2	5.33	5.2	5.33	8665	{}	\N	2020-10-12 14:13:03.499739
25492	AIN	2020-09-28	50.3	51.45	50.3	50.83	125778	{}	\N	2020-10-12 14:13:04.634711
25493	AIN	2020-09-29	50.54	50.72	49.47	50.03	240971	{}	\N	2020-10-12 14:13:04.647004
25494	AIN	2020-09-30	50.16	50.91	49.3	49.51	332712	{}	\N	2020-10-12 14:13:04.655336
25495	AIN	2020-10-01	49.7	50.62	49.17	50.55	370938	{}	\N	2020-10-12 14:13:04.661096
25496	AIN	2020-10-02	49.62	51.67	49.4	51.11	204605	{}	\N	2020-10-12 14:13:04.665676
25497	AIN	2020-10-05	51.52	52.13	51.34	51.81	234521	{}	\N	2020-10-12 14:13:04.671116
25498	AIN	2020-10-06	52.43	54.35	52.04	52.33	384775	{}	\N	2020-10-12 14:13:04.675496
25499	AIN	2020-10-07	52.99	53.66	51.98	53.46	409688	{}	\N	2020-10-12 14:13:04.679321
25500	AIN	2020-10-08	53.98	54.21	53	53.45	224338	{}	\N	2020-10-12 14:13:04.683193
25501	AIN	2020-10-09	53.84	54.75	53.575	54.74	333851	{}	\N	2020-10-12 14:13:04.687079
25502	AIN	2020-10-12	54.96	55.545	54.34	55.48	21038	{}	\N	2020-10-12 14:13:04.690978
25503	AIW	2020-09-28	24.2732	24.2732	24.1601	24.1601	834	{}	\N	2020-10-12 14:13:05.822931
25504	AIW	2020-09-29	24.4	24.4	24.3534	24.3534	1240	{}	\N	2020-10-12 14:13:05.830236
25505	AIW	2020-10-01	24.205	24.205	24.14	24.1698	2160	{}	\N	2020-10-12 14:13:05.835904
25506	AIW	2020-10-02	24.34	24.34	24.265	24.265	2800	{}	\N	2020-10-12 14:13:05.840779
25507	AIW	2020-10-06	24.19	24.1947	24.12	24.12	10838	{}	\N	2020-10-12 14:13:05.845521
25508	AIW	2020-10-07	24.37	24.39	24.37	24.39	1200	{}	\N	2020-10-12 14:13:05.84981
25509	AIW	2020-10-08	24.4	24.4	24.25	24.38	4240	{}	\N	2020-10-12 14:13:05.854212
25510	AIW	2020-10-09	24.311	24.35	24.311	24.35	5600	{}	\N	2020-10-12 14:13:05.858225
25511	AKER	2020-09-28	2.24	2.25	2.11	2.23	279221	{}	\N	2020-10-12 14:13:07.298115
25512	AKER	2020-09-29	2.24	2.32	2.16	2.16	324166	{}	\N	2020-10-12 14:13:07.308382
25513	AKER	2020-09-30	2.21	2.27	2.156	2.25	239243	{}	\N	2020-10-12 14:13:07.315736
25514	AKER	2020-10-01	2.205	2.28	2.205	2.22	164219	{}	\N	2020-10-12 14:13:07.3263
25515	AKER	2020-10-02	2.19	2.34	2.19	2.27	429767	{}	\N	2020-10-12 14:13:07.331709
25516	AKER	2020-10-05	2.3	2.39	2.25	2.27	400866	{}	\N	2020-10-12 14:13:07.336452
25517	AKER	2020-10-06	2.3086	2.38	2.27	2.32	154994	{}	\N	2020-10-12 14:13:07.341221
25518	AKER	2020-10-07	2.335	2.37	2.2801	2.3	187716	{}	\N	2020-10-12 14:13:07.346147
25519	AKER	2020-10-08	2.3	2.3392	2.29	2.31	93703	{}	\N	2020-10-12 14:13:07.350903
25520	AKER	2020-10-09	2.35	2.35	2.28	2.34	133876	{}	\N	2020-10-12 14:13:07.355782
25521	AKER	2020-10-12	2.33	2.33	2.27	2.3001	21283	{}	\N	2020-10-12 14:13:07.360639
25522	ALE	2020-09-28	51.71	52.6	51.58	52	244616	{}	\N	2020-10-12 14:13:08.491595
25523	ALE	2020-09-29	51.92	52.28	51.09	51.48	450768	{}	\N	2020-10-12 14:13:08.501344
25524	ALE	2020-09-30	51.8	52.42	51.3	51.74	524017	{}	\N	2020-10-12 14:13:08.508117
25525	ALE	2020-10-01	51.67	52.27	51.38	52.1	375176	{}	\N	2020-10-12 14:13:08.513853
25526	ALE	2020-10-02	51.67	53.29	51.67	53.12	322446	{}	\N	2020-10-12 14:13:08.51874
25527	ALE	2020-10-05	53.12	53.92	52.91	53.82	483887	{}	\N	2020-10-12 14:13:08.523854
25528	ALE	2020-10-06	54.15	55.2199	53.68	53.9	579644	{}	\N	2020-10-12 14:13:08.528336
25529	ALE	2020-10-07	53.98	54.435	53.6733	54.13	402116	{}	\N	2020-10-12 14:13:08.532596
25530	ALE	2020-10-08	54.46	55.07	54.05	54.51	391403	{}	\N	2020-10-12 14:13:08.53677
25531	ALE	2020-10-09	54.85	54.85	53.655	53.84	310091	{}	\N	2020-10-12 14:13:08.541462
25532	ALE	2020-10-12	53.62	54.34	53.45	54.06	63938	{}	\N	2020-10-12 14:13:08.545533
25533	ALIM	2020-09-28	5.1156	5.1156	5.07	5.11	2915	{}	\N	2020-10-12 14:13:09.675856
25534	ALIM	2020-09-29	5.26	5.45	5.18	5.18	5290	{}	\N	2020-10-12 14:13:09.687041
25535	ALIM	2020-09-30	5	5.065	4.85	4.86	23101	{}	\N	2020-10-12 14:13:09.694592
25536	ALIM	2020-10-01	4.87	4.9	4.8	4.85	11829	{}	\N	2020-10-12 14:13:09.700194
25537	ALIM	2020-10-02	4.83	4.9096	4.75	4.8	14410	{}	\N	2020-10-12 14:13:09.704979
25538	ALIM	2020-10-05	5.07	5.07	4.75	4.75	15257	{}	\N	2020-10-12 14:13:09.710532
25539	ALIM	2020-10-06	4.75	4.9	4.74	4.75	13627	{}	\N	2020-10-12 14:13:09.714926
25540	ALIM	2020-10-07	4.77	4.885	4.77	4.78	4975	{}	\N	2020-10-12 14:13:09.719359
25541	ALIM	2020-10-08	4.91	4.91	4.81	4.81	4327	{}	\N	2020-10-12 14:13:09.723862
25542	ALIM	2020-10-09	4.85	4.89	4.85	4.86	4284	{}	\N	2020-10-12 14:13:09.72831
25543	ALIM	2020-10-12	4.9	4.9	4.89	4.89	1361	{}	\N	2020-10-12 14:13:09.732726
25544	ALNA	2020-09-28	1.27	1.27	1.1945	1.22	295360	{}	\N	2020-10-12 14:13:10.872008
25545	ALNA	2020-09-29	1.2005	1.24	1.18	1.21	426683	{}	\N	2020-10-12 14:13:10.881153
25546	ALNA	2020-09-30	1.49	1.78	1.33	1.48	31974932	{}	\N	2020-10-12 14:13:10.887753
25547	ALNA	2020-10-01	1.4	1.75	1.3612	1.61	3537974	{}	\N	2020-10-12 14:13:10.893249
25548	ALNA	2020-10-02	1.47	1.64	1.45	1.51	1231814	{}	\N	2020-10-12 14:13:10.897973
25549	ALNA	2020-10-05	1.52	1.75	1.52	1.69	1927450	{}	\N	2020-10-12 14:13:10.90263
25550	ALNA	2020-10-06	1.702	1.82	1.57	1.655	2163214	{}	\N	2020-10-12 14:13:10.907368
25551	ALNA	2020-10-07	1.71	1.7157	1.631	1.69	696709	{}	\N	2020-10-12 14:13:10.911967
25552	ALNA	2020-10-08	1.69	1.73	1.58	1.63	1197455	{}	\N	2020-10-12 14:13:10.916891
25553	ALNA	2020-10-09	1.65	1.67	1.57	1.57	406188	{}	\N	2020-10-12 14:13:10.921519
25554	ALNA	2020-10-12	1.57	1.61	1.55	1.5801	107507	{}	\N	2020-10-12 14:13:10.925803
25555	ALTR	2020-09-28	42	42.62	41.79	41.88	189970	{}	\N	2020-10-12 14:13:12.102778
25556	ALTR	2020-09-29	41.83	42.38	41.39	41.7	122720	{}	\N	2020-10-12 14:13:12.112119
25557	ALTR	2020-09-30	41.83	42.53	41.315	41.98	193558	{}	\N	2020-10-12 14:13:12.118187
25558	ALTR	2020-10-01	42.46	43.77	42.23	43.53	271216	{}	\N	2020-10-12 14:13:12.123603
25559	ALTR	2020-10-02	42.2	43.525	41.95	43	303514	{}	\N	2020-10-12 14:13:12.128461
25560	ALTR	2020-10-05	43.33	44.98	42.76	44.92	293161	{}	\N	2020-10-12 14:13:12.133028
25561	ALTR	2020-10-06	45	45.99	44.4801	45.39	585676	{}	\N	2020-10-12 14:13:12.13709
25562	ALTR	2020-10-07	45.59	46.15	44.93	44.99	471531	{}	\N	2020-10-12 14:13:12.141176
25563	ALTR	2020-10-08	46.03	46.22	44.54	44.68	166268	{}	\N	2020-10-12 14:13:12.144958
25564	ALTR	2020-10-09	45.075	45.82	44.84	45.1	231583	{}	\N	2020-10-12 14:13:12.148791
25565	ALTR	2020-10-12	45.15	45.62	44.97	45.46	69475	{}	\N	2020-10-12 14:13:12.153051
25566	AMBC	2020-09-28	12.63	12.9	12.62	12.67	279477	{}	\N	2020-10-12 14:13:14.54348
25567	AMBC	2020-09-29	12.55	12.57	12.21	12.41	569131	{}	\N	2020-10-12 14:13:14.552824
25568	AMBC	2020-09-30	12.5705	12.92	12.5705	12.77	587152	{}	\N	2020-10-12 14:13:14.558468
25569	AMBC	2020-10-01	12.89	13.03	12.67	12.88	666297	{}	\N	2020-10-12 14:13:14.564178
25570	AMBC	2020-10-02	12.63	13.71	12.63	13.58	959829	{}	\N	2020-10-12 14:13:14.569838
25571	AMBC	2020-10-05	13.79	14.3	13.65	13.97	1295451	{}	\N	2020-10-12 14:13:14.574891
25572	AMBC	2020-10-06	14.05	14.32	13.61	13.65	834846	{}	\N	2020-10-12 14:13:14.578759
25573	AMBC	2020-10-07	13.79	14.12	13.5	13.65	873963	{}	\N	2020-10-12 14:13:14.582548
25574	AMBC	2020-10-08	13.76	13.87	13.47	13.79	378231	{}	\N	2020-10-12 14:13:14.58657
25575	AMBC	2020-10-09	13.82	13.95	13.39	13.47	270712	{}	\N	2020-10-12 14:13:14.590903
25576	AMBC	2020-10-12	13.37	13.53	13.27	13.53	91710	{}	\N	2020-10-12 14:13:14.595268
25577	AMGN	2020-09-28	247.68	249.04	244.77	247.03	2079132	{}	\N	2020-10-12 14:13:15.574033
25578	AMGN	2020-09-29	247.55	249.05	245.48	248.3	1881458	{}	\N	2020-10-12 14:13:15.58173
25579	AMGN	2020-09-30	249.81	256.115	248.1	254.16	3155015	{}	\N	2020-10-12 14:13:15.587678
25580	AMGN	2020-10-01	255.07	258.505	254.05	255.39	2290615	{}	\N	2020-10-12 14:13:15.592885
25581	AMGN	2020-10-02	252.46	254.66	244.01	245.41	2312084	{}	\N	2020-10-12 14:13:15.59749
25582	AMGN	2020-10-05	246.52	256.6499	246.2405	256.01	2796924	{}	\N	2020-10-12 14:13:15.602553
25583	AMGN	2020-10-06	257.35	257.35	252.16	252.69	2212679	{}	\N	2020-10-12 14:13:15.606721
25584	AMGN	2020-10-07	253.96	258.405	253.1	257.67	1781097	{}	\N	2020-10-12 14:13:15.610535
25585	AMGN	2020-10-08	249.27	250.06	239	240.09	5540053	{}	\N	2020-10-12 14:13:15.614501
25586	AMGN	2020-10-09	238.5	239.58	234.02	236.7	4085665	{}	\N	2020-10-12 14:13:15.618725
25587	AMGN	2020-10-12	237.73	238.97	236.2	236.815	351037	{}	\N	2020-10-12 14:13:15.622708
25588	MVO	2020-09-28	2.36	2.42	2.36	2.36	15144	{}	\N	2020-10-12 14:13:16.533864
25589	MVO	2020-09-29	2.36	2.3997	2.36	2.36	33504	{}	\N	2020-10-12 14:13:16.542581
25590	MVO	2020-09-30	2.3501	2.351	2.26	2.28	20420	{}	\N	2020-10-12 14:13:16.549117
25591	MVO	2020-10-01	2.26	2.3	2.26	2.27	56189	{}	\N	2020-10-12 14:13:16.554991
25592	MVO	2020-10-02	2.26	2.35	2.26	2.31	19694	{}	\N	2020-10-12 14:13:16.56056
25593	MVO	2020-10-05	2.3	2.3299	2.25	2.28	57757	{}	\N	2020-10-12 14:13:16.565476
25594	MVO	2020-10-06	2.4	2.69	2.4	2.68	305980	{}	\N	2020-10-12 14:13:16.569774
25595	MVO	2020-10-07	2.57	2.68	2.45	2.6	173304	{}	\N	2020-10-12 14:13:16.574238
25596	MVO	2020-10-08	2.69	2.8	2.5298	2.73	230986	{}	\N	2020-10-12 14:13:16.578491
25597	MVO	2020-10-09	2.84	2.99	2.7768	2.86	269408	{}	\N	2020-10-12 14:13:16.582665
25598	MVO	2020-10-12	2.99	2.99	2.86	2.958	18322	{}	\N	2020-10-12 14:13:16.586947
25599	AFC	2020-09-28	26.16	26.1704	26.085	26.09	10655	{}	\N	2020-10-12 14:13:17.960211
25600	AFC	2020-09-29	25.97	26.32	25.97	26.32	12832	{}	\N	2020-10-12 14:13:17.969338
25601	AFC	2020-09-30	26	26	25.44	25.44	144614	{}	\N	2020-10-12 14:13:17.975283
25602	AFC	2020-10-01	25.62	25.7	25.34	25.45	101721	{}	\N	2020-10-12 14:13:17.98064
25603	AFC	2020-10-02	25.34	25.54	25.34	25.52	55438	{}	\N	2020-10-12 14:13:17.985532
25604	AFC	2020-10-05	25.59	25.65	25.491	25.5	14274	{}	\N	2020-10-12 14:13:17.990424
25605	AFC	2020-10-06	25.56	25.63	25.5	25.55	26252	{}	\N	2020-10-12 14:13:17.995797
25606	AFC	2020-10-07	25.51	25.69	25.51	25.68	34031	{}	\N	2020-10-12 14:13:17.99966
25607	AFC	2020-10-08	25.74	25.84	25.7333	25.79	30850	{}	\N	2020-10-12 14:13:18.003548
25608	AFC	2020-10-09	25.79	25.79	25.61	25.75	17056	{}	\N	2020-10-12 14:13:18.007184
25609	AFC	2020-10-12	25.715	25.715	25.715	25.715	482	{}	\N	2020-10-12 14:13:18.010595
25610	AGM	2020-09-28	65.4	66.925	65.12	65.19	60966	{}	\N	2020-10-12 14:13:21.617131
25611	AGM	2020-09-29	65.23	65.23	62.805	63.78	88736	{}	\N	2020-10-12 14:13:21.627244
25612	AGM	2020-09-30	64.09	64.98	63.19	63.66	90372	{}	\N	2020-10-12 14:13:21.634001
25613	AGM	2020-10-01	63.67	64.81	62.75	64.7	95702	{}	\N	2020-10-12 14:13:21.640228
25614	AGM	2020-10-02	63.69	65.82	63.505	65.69	71030	{}	\N	2020-10-12 14:13:21.645123
25615	AGM	2020-10-05	66.13	66.92	65.7136	66.66	69402	{}	\N	2020-10-12 14:13:21.649957
25616	AGM	2020-10-06	67.34	68.81	67.02	67.32	118672	{}	\N	2020-10-12 14:13:21.654204
25617	AGM	2020-10-07	68.32	68.5	67.14	67.24	78437	{}	\N	2020-10-12 14:13:21.658199
25618	AGM	2020-10-08	67.9	68.44	67.27	67.74	54163	{}	\N	2020-10-12 14:13:21.662226
25619	AGM	2020-10-09	68.36	68.46	66.61	67.53	98511	{}	\N	2020-10-12 14:13:21.6664
25620	AGM	2020-10-12	67.4	67.5	67.17	67.5	1970	{}	\N	2020-10-12 14:13:21.671303
25621	AGRX	2020-09-28	2.91	2.96	2.72	2.84	1811415	{}	\N	2020-10-12 14:13:22.956579
25622	AGRX	2020-09-29	2.89	3.11	2.84	3.11	1913576	{}	\N	2020-10-12 14:13:22.964089
25623	AGRX	2020-09-30	3.11	3.1199	2.95	3.04	944544	{}	\N	2020-10-12 14:13:22.970264
25624	AGRX	2020-10-01	3.04	3.06	2.96	3.05	699893	{}	\N	2020-10-12 14:13:22.975981
25625	AGRX	2020-10-02	2.96	3.06	2.9409	3.06	608076	{}	\N	2020-10-12 14:13:22.980968
25626	AGRX	2020-10-05	3.05	3.27	3.012	3.19	1013618	{}	\N	2020-10-12 14:13:22.985666
25627	AGRX	2020-10-06	3.3	3.3	3.055	3.1	768020	{}	\N	2020-10-12 14:13:22.992459
25628	AGRX	2020-10-07	3.1	3.25	3.08	3.24	706756	{}	\N	2020-10-12 14:13:22.998117
25629	AGRX	2020-10-08	3.25	3.29	3.12	3.15	771506	{}	\N	2020-10-12 14:13:23.002673
25630	AGRX	2020-10-09	3.18	3.25	3.1601	3.25	683746	{}	\N	2020-10-12 14:13:23.007383
25631	AGRX	2020-10-12	3.26	3.28	3.18	3.21	115492	{}	\N	2020-10-12 14:13:23.011616
25632	AINC	2020-09-28	6.02	6.2	5.775	5.85	7104	{}	\N	2020-10-12 14:13:25.005488
25633	AINC	2020-09-29	5.8501	6.24	5.69	5.91	6556	{}	\N	2020-10-12 14:13:25.013566
25634	AINC	2020-09-30	6.01	6.2399	5.72	5.8	7163	{}	\N	2020-10-12 14:13:25.018853
25635	AINC	2020-10-01	5.85	6.07	5.7	5.76	7790	{}	\N	2020-10-12 14:13:25.024263
25636	AINC	2020-10-02	5.62	5.9199	5.55	5.7394	19662	{}	\N	2020-10-12 14:13:25.028721
25637	AINC	2020-10-05	5.65	5.96	5.6	5.85	4284	{}	\N	2020-10-12 14:13:25.032856
25638	AINC	2020-10-06	5.92	6.04	5.86	5.88	5916	{}	\N	2020-10-12 14:13:25.037198
25639	AINC	2020-10-07	5.56	5.866	5.56	5.81	7297	{}	\N	2020-10-12 14:13:25.041946
25640	AINC	2020-10-08	5.92	5.92	5.83	5.875	1962	{}	\N	2020-10-12 14:13:25.046941
25641	AINC	2020-10-09	5.8035	5.9072	5.8035	5.9	687	{}	\N	2020-10-12 14:13:25.051374
25642	ALCO	2020-09-28	28.15	28.9988	27.9	28.52	7178	{}	\N	2020-10-12 14:13:28.314595
25643	ALCO	2020-09-29	28.91	28.91	28.19	28.66	11441	{}	\N	2020-10-12 14:13:28.325941
25644	ALCO	2020-09-30	28.96	29.1688	28.27	28.62	5289	{}	\N	2020-10-12 14:13:28.332459
25645	ALCO	2020-10-01	28.2766	29.26	28.2766	28.65	5286	{}	\N	2020-10-12 14:13:28.337699
25646	ALCO	2020-10-02	28.8333	29.38	28.8333	29.38	4808	{}	\N	2020-10-12 14:13:28.341948
25647	ALCO	2020-10-05	29.53	30.08	29.4901	30.06	6612	{}	\N	2020-10-12 14:13:28.346624
25648	ALCO	2020-10-06	30	30.33	29.8	29.8	8187	{}	\N	2020-10-12 14:13:28.351406
25649	ALCO	2020-10-07	30.26	31.57	29.62	31.4	17911	{}	\N	2020-10-12 14:13:28.356438
25650	ALCO	2020-10-08	31.4	31.74	30.84	31.1	12515	{}	\N	2020-10-12 14:13:28.360917
25651	ALCO	2020-10-09	31.36	31.585	31.07	31.17	2528	{}	\N	2020-10-12 14:13:28.365322
25652	ALJJ	2020-09-28	0.7472	0.771	0.7053	0.713	27482	{}	\N	2020-10-12 14:13:29.71367
25653	ALJJ	2020-09-29	0.7197	0.7561	0.7197	0.723	16516	{}	\N	2020-10-12 14:13:29.723414
25654	ALJJ	2020-09-30	0.73	0.775	0.7201	0.7251	81415	{}	\N	2020-10-12 14:13:29.730372
25655	ALJJ	2020-10-01	0.7652	0.8283	0.737	0.791	59719	{}	\N	2020-10-12 14:13:29.736119
25656	ALJJ	2020-10-02	0.791	0.881	0.761	0.827	159398	{}	\N	2020-10-12 14:13:29.740828
25657	ALJJ	2020-10-05	0.85	0.888	0.8	0.8075	32314	{}	\N	2020-10-12 14:13:29.745403
25658	ALJJ	2020-10-06	0.8174	0.867	0.8174	0.8574	94684	{}	\N	2020-10-12 14:13:29.749712
25659	ALJJ	2020-10-07	0.8565	0.867	0.812	0.83	1071557	{}	\N	2020-10-12 14:13:29.754058
25660	ALJJ	2020-10-08	1.06	1.12	0.93	1.04	2475096	{}	\N	2020-10-12 14:13:29.758001
25661	ALJJ	2020-10-09	0.931	1.04	0.931	0.99	246259	{}	\N	2020-10-12 14:13:29.762121
25662	ALJJ	2020-10-12	0.96	1.01	0.96	0.9651	22384	{}	\N	2020-10-12 14:13:29.766113
25663	AMAG	2020-09-28	9.03	9.3	9	9.23	197574	{}	\N	2020-10-12 14:13:31.421539
25664	AMAG	2020-09-29	9.17	9.33	9.07	9.27	202238	{}	\N	2020-10-12 14:13:31.432184
25665	AMAG	2020-09-30	9.28	9.6	9.28	9.4	927689	{}	\N	2020-10-12 14:13:31.442304
25666	AMAG	2020-10-01	12.98	13.8	12.55	13.65	29816764	{}	\N	2020-10-12 14:13:31.448031
25667	AMAG	2020-10-02	13.6	13.76	13.6	13.74	8531750	{}	\N	2020-10-12 14:13:31.453692
25668	AMAG	2020-10-05	13.77	13.77	12.8	13.08	13745640	{}	\N	2020-10-12 14:13:31.458406
25669	AMAG	2020-10-06	13.05	13.23	12.95	13.04	3170576	{}	\N	2020-10-12 14:13:31.462755
25670	AMAG	2020-10-07	13.12	13.49	13.1	13.34	2697463	{}	\N	2020-10-12 14:13:31.467491
25671	AMAG	2020-10-08	13.35	13.48	13.32	13.36	1058263	{}	\N	2020-10-12 14:13:31.471999
25672	AMAG	2020-10-09	13.4	13.45	13.3	13.33	860976	{}	\N	2020-10-12 14:13:31.477818
25673	AMAG	2020-10-12	13.35	13.4	13.32	13.355	91565	{}	\N	2020-10-12 14:13:31.481704
25674	AMD	2020-09-28	79.1165	79.57	77.52	79.48	47223189	{}	\N	2020-10-12 14:13:32.725594
25675	AMD	2020-09-29	79.3	82.55	79.3	81.77	60991674	{}	\N	2020-10-12 14:13:32.735881
25676	AMD	2020-09-30	81.75	82.94	80.66	81.99	49808178	{}	\N	2020-10-12 14:13:32.742522
25677	AMD	2020-10-01	83.06	85.25	82.86	84.89	50369245	{}	\N	2020-10-12 14:13:32.748417
25678	AMD	2020-10-02	82.62	84.65	81.66	81.8	50916113	{}	\N	2020-10-12 14:13:32.754044
25679	AMD	2020-10-05	82.55	86.2819	82.55	86.15	47200276	{}	\N	2020-10-12 14:13:32.758967
25680	AMD	2020-10-06	86.21	87.25	83.548	84.489	49697928	{}	\N	2020-10-12 14:13:32.763421
25681	AMD	2020-10-07	86.1	87.79	85.65	86.69	42975625	{}	\N	2020-10-12 14:13:32.768015
25682	AMD	2020-10-08	88.12	88.72	85.95	86.51	54136304	{}	\N	2020-10-12 14:13:32.772605
25683	AMD	2020-10-09	84.75	85.75	82.3465	83.1	80531685	{}	\N	2020-10-12 14:13:32.776781
25684	AMD	2020-10-12	83.65	84.89	83.12	84.425	14753090	{}	\N	2020-10-12 14:13:32.781218
25685	AMLP	2020-09-28	19.8	20.22	19.515	20.11	2667673	{}	\N	2020-10-12 14:13:34.074985
25686	AMLP	2020-09-29	20.04	20.505	19.79	20.39	2084638	{}	\N	2020-10-12 14:13:34.083673
25687	AMLP	2020-09-30	20.42	20.77	19.795	19.98	2864522	{}	\N	2020-10-12 14:13:34.088671
25688	AMLP	2020-10-01	19.97	20.11	19.465	19.83	2628768	{}	\N	2020-10-12 14:13:34.094495
25689	AMLP	2020-10-02	19.29	20.205	19.29	20.07	3627938	{}	\N	2020-10-12 14:13:34.098974
25690	AMLP	2020-10-05	20.3	20.95	20.21	20.9	2754198	{}	\N	2020-10-12 14:13:34.104199
25691	AMLP	2020-10-06	21.05	21.4	20.225	20.48	3403735	{}	\N	2020-10-12 14:13:34.108594
25692	AMLP	2020-10-07	20.54	20.7	20.34	20.48	2893584	{}	\N	2020-10-12 14:13:34.113314
25693	AMLP	2020-10-08	20.7	21.86	20.6	21.86	2910095	{}	\N	2020-10-12 14:13:34.118152
25694	AMLP	2020-10-09	21.97	22.17	21.53	21.81	1709916	{}	\N	2020-10-12 14:13:34.122368
25695	AMLP	2020-10-12	21.78	21.81	21.555	21.63	294204	{}	\N	2020-10-12 14:13:34.126546
25696	AIZ	2020-09-28	119.39	121.365	119.07	120.1	210429	{}	\N	2020-10-12 14:13:35.65804
25697	AIZ	2020-09-29	120.02	120.975	118.41	120.59	474478	{}	\N	2020-10-12 14:13:35.668024
25698	AIZ	2020-09-30	121.2	123.46	120.76	121.31	860103	{}	\N	2020-10-12 14:13:35.674857
25699	AIZ	2020-10-01	121.35	122.71	120.1	122.47	563086	{}	\N	2020-10-12 14:13:35.680472
25700	AIZ	2020-10-02	121.2	124.42	120.7949	123.43	484021	{}	\N	2020-10-12 14:13:35.685409
25701	AIZ	2020-10-05	124.98	125.9	124.125	125.36	558513	{}	\N	2020-10-12 14:13:35.690938
25702	AIZ	2020-10-06	125.78	126	122.28	122.9	652355	{}	\N	2020-10-12 14:13:35.695768
25703	AIZ	2020-10-07	123.74	124.77	122.325	123.81	725802	{}	\N	2020-10-12 14:13:35.701991
25704	AIZ	2020-10-08	124.77	125.75	123.98	125.63	1036982	{}	\N	2020-10-12 14:13:35.706605
25705	AIZ	2020-10-09	126.51	127.47	125.42	125.57	803197	{}	\N	2020-10-12 14:13:35.710636
25706	AIZ	2020-10-12	125.3	126.875	125.02	126.875	39599	{}	\N	2020-10-12 14:13:35.714965
25707	AKR	2020-09-28	10.43	10.735	10.43	10.53	1055541	{}	\N	2020-10-12 14:13:36.696815
25708	AKR	2020-09-29	10.51	10.51	9.965	10.34	1708247	{}	\N	2020-10-12 14:13:36.707002
25709	AKR	2020-09-30	10.455	10.7	10.31	10.5	2383336	{}	\N	2020-10-12 14:13:36.713598
25710	AKR	2020-10-01	10.47	10.84	10.38	10.66	2654093	{}	\N	2020-10-12 14:13:36.719467
25711	AKR	2020-10-02	10.36	11.16	10.36	11.13	2362363	{}	\N	2020-10-12 14:13:36.724552
25712	AKR	2020-10-05	11.11	11.2	10.7	10.85	1274002	{}	\N	2020-10-12 14:13:36.729454
25713	AKR	2020-10-06	10.98	11.09	10.72	10.86	2283337	{}	\N	2020-10-12 14:13:36.734357
25714	AKR	2020-10-07	10.94	10.99	10.49	10.52	1941665	{}	\N	2020-10-12 14:13:36.738746
25715	AKR	2020-10-08	10.63	10.785	10.51	10.71	2467154	{}	\N	2020-10-12 14:13:36.743053
25716	AKR	2020-10-09	10.82	10.855	10.41	10.55	1335293	{}	\N	2020-10-12 14:13:36.747382
25717	AKR	2020-10-12	10.52	10.6	10.3134	10.53	97753	{}	\N	2020-10-12 14:13:36.75163
25718	ALK	2020-09-28	37.59	38.26	36.74	37.54	2892218	{}	\N	2020-10-12 14:13:38.451088
25719	ALK	2020-09-29	37.4	37.42	36.285	36.67	3371028	{}	\N	2020-10-12 14:13:38.460162
25720	ALK	2020-09-30	37.47	38.27	36.475	36.63	3515126	{}	\N	2020-10-12 14:13:38.466329
25721	ALK	2020-10-01	37.09	37.21	36.23	36.91	3478242	{}	\N	2020-10-12 14:13:38.472277
25722	ALK	2020-10-02	35.6	38.15	35.31	37.52	7169601	{}	\N	2020-10-12 14:13:38.477183
25723	ALK	2020-10-05	37.92	38	36.87	37.57	4006148	{}	\N	2020-10-12 14:13:38.481871
25724	ALK	2020-10-06	38.22	38.44	36.06	36.09	3800404	{}	\N	2020-10-12 14:13:38.486238
25725	ALK	2020-10-07	37.21	37.64	36.8	37.02	4019392	{}	\N	2020-10-12 14:13:38.490305
25726	ALK	2020-10-08	37.73	39.46	37.62	38.63	7267906	{}	\N	2020-10-12 14:13:38.494281
25727	ALK	2020-10-09	38.23	39.1	37.92	38.69	5102331	{}	\N	2020-10-12 14:13:38.498305
25728	ALK	2020-10-12	38.32	38.49	37.82	38.38	566797	{}	\N	2020-10-12 14:13:38.502313
25729	ALPN	2020-09-28	8.18	8.74	7.5	8.48	26570	{}	\N	2020-10-12 14:13:39.386875
25730	ALPN	2020-09-29	8.45	8.64	8.3055	8.62	25968	{}	\N	2020-10-12 14:13:39.3964
25731	ALPN	2020-09-30	8.61	8.8	8.32	8.69	27172	{}	\N	2020-10-12 14:13:39.40287
25732	ALPN	2020-10-01	8.52	9.1	8.51	9.1	48797	{}	\N	2020-10-12 14:13:39.40853
25733	ALPN	2020-10-02	8.79	9.46	8.15	8.98	39912	{}	\N	2020-10-12 14:13:39.413176
25734	ALPN	2020-10-05	8.96	9.34	8.717	9.1	37105	{}	\N	2020-10-12 14:13:39.417626
25735	ALPN	2020-10-06	9.325	9.325	9.06	9.09	26222	{}	\N	2020-10-12 14:13:39.421797
25736	ALPN	2020-10-07	9.19	9.29	8.81	8.92	13512	{}	\N	2020-10-12 14:13:39.425612
25737	ALPN	2020-10-08	9.06	9.06	8.58	8.58	24277	{}	\N	2020-10-12 14:13:39.429594
25738	ALPN	2020-10-09	8.53	8.72	8.43	8.49	20516	{}	\N	2020-10-12 14:13:39.433361
25739	ALPN	2020-10-12	8.6	8.6818	8.5	8.5	1456	{}	\N	2020-10-12 14:13:39.437047
25946	APT	2020-10-05	16.03	17.08	16.03	16.39	1433098	{}	\N	2020-10-12 14:14:11.466432
25947	APT	2020-10-06	16.85	16.9	15.4	15.49	914264	{}	\N	2020-10-12 14:14:11.47069
25948	APT	2020-10-07	15.85	15.99	15.4604	15.82	717124	{}	\N	2020-10-12 14:14:11.474541
25949	APT	2020-10-08	15.95	15.95	15.31	15.47	657056	{}	\N	2020-10-12 14:14:11.47816
25950	APT	2020-10-09	15.77	15.77	15.08	15.09	716359	{}	\N	2020-10-12 14:14:11.481811
25951	APT	2020-10-12	15.4	15.48	15.2003	15.3307	132633	{}	\N	2020-10-12 14:14:11.48601
25952	AKBA	2020-09-28	2.53	2.53	2.4	2.44	4205278	{}	\N	2020-10-12 14:14:12.438234
25953	AKBA	2020-09-29	2.44	2.48	2.37	2.39	3443091	{}	\N	2020-10-12 14:14:12.448709
25954	AKBA	2020-09-30	2.39	2.5338	2.36	2.5199	4979774	{}	\N	2020-10-12 14:14:12.454604
25955	AKBA	2020-10-01	2.53	2.59	2.43	2.52	4473346	{}	\N	2020-10-12 14:14:12.45974
25956	AKBA	2020-10-02	2.42	2.52	2.41	2.47	3023832	{}	\N	2020-10-12 14:14:12.464883
25957	AKBA	2020-10-05	2.51	2.7	2.51	2.69	6580510	{}	\N	2020-10-12 14:14:12.470247
25958	AKBA	2020-10-06	2.7317	2.75	2.62	2.7	3774578	{}	\N	2020-10-12 14:14:12.47411
25959	AKBA	2020-10-07	2.69	2.83	2.67	2.82	3710956	{}	\N	2020-10-12 14:14:12.477943
25960	AKBA	2020-10-08	2.85	2.975	2.79	2.95	6099112	{}	\N	2020-10-12 14:14:12.481641
25961	AKBA	2020-10-09	2.98	3.12	2.85	3.11	7078187	{}	\N	2020-10-12 14:14:12.485186
25962	AKBA	2020-10-12	3.26	3.53	3.15	3.5283	5093174	{}	\N	2020-10-12 14:14:12.489149
25963	ALDX	2020-09-28	6.93	7.24	6.845	7.13	627661	{}	\N	2020-10-12 14:14:13.443275
25964	ALDX	2020-09-29	7.1	7.63	6.99	7.53	944023	{}	\N	2020-10-12 14:14:13.449996
25965	ALDX	2020-09-30	7.55	7.79	7.26	7.41	1379242	{}	\N	2020-10-12 14:14:13.455672
25966	ALDX	2020-10-01	7.36	7.51	7.09	7.12	608101	{}	\N	2020-10-12 14:14:13.460978
25967	ALDX	2020-10-02	7	7.24	6.91	6.97	291278	{}	\N	2020-10-12 14:14:13.465458
25968	ALDX	2020-10-05	7.02	7.32	7.02	7.16	329829	{}	\N	2020-10-12 14:14:13.470139
25969	ALDX	2020-10-06	7.24	7.45	7.14	7.16	302918	{}	\N	2020-10-12 14:14:13.474558
25970	ALDX	2020-10-07	7.11	7.37	7.08	7.23	280307	{}	\N	2020-10-12 14:14:13.478638
25971	ALDX	2020-10-08	7.3	7.33	7.01	7.03	225582	{}	\N	2020-10-12 14:14:13.482885
25972	ALDX	2020-10-09	7.06	7.15	6.95	7.05	181814	{}	\N	2020-10-12 14:14:13.487241
25973	ALDX	2020-10-12	7.06	7.09	6.91	6.98	58271	{}	\N	2020-10-12 14:14:13.491644
25974	ALGN	2020-09-28	324.93	328.78	320.99	322.86	499760	{}	\N	2020-10-12 14:14:14.799765
25975	ALGN	2020-09-29	323.9	328.92	320	320.17	437916	{}	\N	2020-10-12 14:14:14.808872
25976	ALGN	2020-09-30	321.32	332.88	320.29	327.36	635906	{}	\N	2020-10-12 14:14:14.815408
25977	ALGN	2020-10-01	332.345	332.345	313.86	317.61	819324	{}	\N	2020-10-12 14:14:14.820853
25978	ALGN	2020-10-02	312.01	319.13	310.03	314.89	507867	{}	\N	2020-10-12 14:14:14.82578
25979	ALGN	2020-10-05	317.63	320.31	312.72	319.51	639890	{}	\N	2020-10-12 14:14:14.830375
25980	ALGN	2020-10-06	323.285	327.9127	315.24	316.08	455131	{}	\N	2020-10-12 14:14:14.835138
25981	ALGN	2020-10-07	320.4	327.0947	320.4	323.62	480123	{}	\N	2020-10-12 14:14:14.839455
25982	ALGN	2020-10-08	325.59	328.56	321.86	327.57	336487	{}	\N	2020-10-12 14:14:14.843731
25983	ALGN	2020-10-09	331.39	331.855	325.18	328.64	365196	{}	\N	2020-10-12 14:14:14.848381
25984	ALGN	2020-10-12	334.85	335.32	331.34	332.415	42345	{}	\N	2020-10-12 14:14:14.853387
25985	ALRM	2020-09-28	55.6447	56.92	55.56	56.2	296640	{}	\N	2020-10-12 14:14:16.26862
25986	ALRM	2020-09-29	56.58	57.27	55.65	55.89	249095	{}	\N	2020-10-12 14:14:16.28109
25987	ALRM	2020-09-30	57.79	58.19	54.785	55.24	479319	{}	\N	2020-10-12 14:14:16.288777
25988	ALRM	2020-10-01	55.8	57.92	55.8	57.49	260983	{}	\N	2020-10-12 14:14:16.29493
25989	ALRM	2020-10-02	55.7	57.6	55.6	57.12	160217	{}	\N	2020-10-12 14:14:16.300344
25990	ALRM	2020-10-05	57.71	58.69	57.677	58.5	172365	{}	\N	2020-10-12 14:14:16.305371
25991	ALRM	2020-10-06	58.78	60.05	58.64	58.97	248151	{}	\N	2020-10-12 14:14:16.309839
25992	ALRM	2020-10-07	59.45	60.055	58.785	59.4	288082	{}	\N	2020-10-12 14:14:16.314795
25993	ALRM	2020-10-08	60.34	60.54	59.42	59.68	178975	{}	\N	2020-10-12 14:14:16.321479
25994	ALRM	2020-10-09	60.25	61.14	60.24	60.54	176448	{}	\N	2020-10-12 14:14:16.330893
25995	ALRM	2020-10-12	61.22	61.31	60.57	61.1117	11806	{}	\N	2020-10-12 14:14:16.336425
25996	ALX	2020-09-28	247.87	253.19	247.87	251.2	6355	{}	\N	2020-10-12 14:14:17.670462
25997	ALX	2020-09-29	253.28	253.28	243.53	244.58	28976	{}	\N	2020-10-12 14:14:17.680908
25998	ALX	2020-09-30	249.12	249.12	242.99	245.22	14691	{}	\N	2020-10-12 14:14:17.687325
25999	ALX	2020-10-01	247.51	249.67	244.67	245.08	23860	{}	\N	2020-10-12 14:14:17.693618
26000	ALX	2020-10-02	247.17	254.64	245.97	254.64	17505	{}	\N	2020-10-12 14:14:17.699767
26001	ALX	2020-10-05	254.7434	254.995	250.18	250.19	29616	{}	\N	2020-10-12 14:14:17.705603
26002	ALX	2020-10-06	251	254.6657	245.47	248.82	18170	{}	\N	2020-10-12 14:14:17.710427
26003	ALX	2020-10-07	252.23	254.47	250.1646	253.7	16538	{}	\N	2020-10-12 14:14:17.716509
26004	ALX	2020-10-08	254.19	255.0486	252.45	252.98	17896	{}	\N	2020-10-12 14:14:17.721742
26005	ALX	2020-10-09	255.975	258.9949	250.23	251.49	23007	{}	\N	2020-10-12 14:14:17.727641
26006	ALX	2020-10-12	249.5	250.52	249.5	250.52	2002	{}	\N	2020-10-12 14:14:17.732592
26007	AMC	2020-09-28	5.04	5.04	4.61	4.91	4732525	{}	\N	2020-10-12 14:14:18.970232
26008	AMC	2020-09-29	4.88	4.94	4.75	4.86	4997849	{}	\N	2020-10-12 14:14:18.981338
26009	AMC	2020-09-30	4.8	4.95	4.7	4.71	5052453	{}	\N	2020-10-12 14:14:18.988835
26010	AMC	2020-10-01	4.78	4.8	4.63	4.65	6374968	{}	\N	2020-10-12 14:14:18.994857
26011	AMC	2020-10-02	4.48	4.66	4.42	4.65	7295003	{}	\N	2020-10-12 14:14:19.000365
26012	AMC	2020-10-05	4.3	4.36	4.05	4.13	19198545	{}	\N	2020-10-12 14:14:19.00537
26013	AMC	2020-10-06	4.26	4.27	4.05	4.06	16543044	{}	\N	2020-10-12 14:14:19.010179
26014	AMC	2020-10-07	4.08	4.11	3.94	4.04	13705463	{}	\N	2020-10-12 14:14:19.01495
26015	AMC	2020-10-08	4.13	4.185	4.02	4.14	16010743	{}	\N	2020-10-12 14:14:19.019513
26016	AMC	2020-10-09	4.17	4.18	4.04	4.05	10156787	{}	\N	2020-10-12 14:14:19.024028
26017	AMC	2020-10-12	4.03	4.08	4	4.0493	2113856	{}	\N	2020-10-12 14:14:19.028433
26018	AMKR	2020-09-28	11.14	11.315	11.04	11.28	874284	{}	\N	2020-10-12 14:14:20.074584
26019	AMKR	2020-09-29	11.28	11.44	11.17	11.26	673253	{}	\N	2020-10-12 14:14:20.085761
26020	AMKR	2020-09-30	11.26	11.475	11.16	11.2	645714	{}	\N	2020-10-12 14:14:20.093293
26021	AMKR	2020-10-01	11.33	11.455	11.25	11.435	627073	{}	\N	2020-10-12 14:14:20.099335
26022	AMKR	2020-10-02	11.09	11.35	11	11.15	634169	{}	\N	2020-10-12 14:14:20.104988
26023	AMKR	2020-10-05	11.28	11.535	11.28	11.49	741249	{}	\N	2020-10-12 14:14:20.109904
26024	AMKR	2020-10-06	11.57	11.79	11.4	11.425	785439	{}	\N	2020-10-12 14:14:20.114154
26025	AMKR	2020-10-07	11.64	11.88	11.605	11.86	1042176	{}	\N	2020-10-12 14:14:20.11826
26026	AMKR	2020-10-08	11.97	12.29	11.88	12.27	1200720	{}	\N	2020-10-12 14:14:20.122222
26027	AMKR	2020-10-09	12.5	12.73	12.4	12.69	925019	{}	\N	2020-10-12 14:14:20.126206
26028	AMKR	2020-10-12	12.91	12.93	12.79	12.845	114033	{}	\N	2020-10-12 14:14:20.130304
26029	AMS	2020-09-28	1.93	2.4701	1.831	2.065	3485901	{}	\N	2020-10-12 14:14:21.754767
26030	AMS	2020-09-29	1.96	2.13	1.82	1.96	842491	{}	\N	2020-10-12 14:14:21.76336
26031	AMS	2020-09-30	1.84	1.946	1.81	1.81	155707	{}	\N	2020-10-12 14:14:21.769125
26032	AMS	2020-10-01	1.82	1.87	1.77	1.79	178435	{}	\N	2020-10-12 14:14:21.774954
26033	AMS	2020-10-02	1.7756	1.85	1.77	1.8	49399	{}	\N	2020-10-12 14:14:21.779264
26034	AMS	2020-10-05	1.82	1.9	1.75	1.85	210871	{}	\N	2020-10-12 14:14:21.783463
26035	AMS	2020-10-06	1.82	2.1028	1.8101	1.91	311460	{}	\N	2020-10-12 14:14:21.787674
26036	AMS	2020-10-07	1.91	2.06	1.85	1.95	239584	{}	\N	2020-10-12 14:14:21.792726
26037	AMS	2020-10-08	2.01	2.03	1.87	1.93	131481	{}	\N	2020-10-12 14:14:21.796759
26038	AMS	2020-10-09	1.98	1.98	1.86	1.92	37421	{}	\N	2020-10-12 14:14:21.800507
26039	AMS	2020-10-12	1.87	1.9055	1.87	1.89	2761	{}	\N	2020-10-12 14:14:21.804431
26040	AMX	2020-09-28	12.27	12.4124	12.035	12.26	1915143	{}	\N	2020-10-12 14:14:22.798797
26041	AMX	2020-09-29	12.19	12.34	12.17	12.28	4546450	{}	\N	2020-10-12 14:14:22.809554
26042	AMX	2020-09-30	12.4	12.73	12.36	12.49	8088422	{}	\N	2020-10-12 14:14:22.817383
26043	AMX	2020-10-01	12.61	12.61	12.205	12.24	3513161	{}	\N	2020-10-12 14:14:22.82366
26044	AMX	2020-10-02	12.04	12.4475	11.95	12.34	3533687	{}	\N	2020-10-12 14:14:22.829292
26045	AMX	2020-10-05	12.51	12.67	12.46	12.48	6010305	{}	\N	2020-10-12 14:14:22.834456
26046	AMX	2020-10-06	12.52	12.61	12.28	12.38	3706260	{}	\N	2020-10-12 14:14:22.839062
26047	AMX	2020-10-07	12.55	12.605	12.43	12.54	2473925	{}	\N	2020-10-12 14:14:22.843932
26048	AMX	2020-10-08	12.58	12.95	12.52	12.89	3161316	{}	\N	2020-10-12 14:14:22.848628
26049	AMX	2020-10-09	12.95	13.21	12.8	12.97	4596580	{}	\N	2020-10-12 14:14:22.853475
26050	AMX	2020-10-12	13	13.085	12.92	13.015	291216	{}	\N	2020-10-12 14:14:22.857695
26051	ANET	2020-09-28	201.31	207.385	201.31	205.05	750725	{}	\N	2020-10-12 14:14:23.888705
26052	ANET	2020-09-29	205.08	212	204.95	210.46	1312281	{}	\N	2020-10-12 14:14:23.898534
26053	ANET	2020-09-30	210.46	211.55	205.115	206.93	1661360	{}	\N	2020-10-12 14:14:23.904878
26054	ANET	2020-10-01	208.5	209.25	205.08	207.05	1246001	{}	\N	2020-10-12 14:14:23.910641
26055	ANET	2020-10-02	203.5	211.19	202.81	203.9	897876	{}	\N	2020-10-12 14:14:23.915532
26056	ANET	2020-10-05	205.75	207.76	204.2	206	961922	{}	\N	2020-10-12 14:14:23.92008
26057	ANET	2020-10-06	205.41	210.8112	205.41	206.8	950048	{}	\N	2020-10-12 14:14:23.924015
26058	ANET	2020-10-07	209.23	213.45	207.905	211.99	710929	{}	\N	2020-10-12 14:14:23.927756
26059	ANET	2020-10-08	221	228.74	219.48	222.49	2349056	{}	\N	2020-10-12 14:14:23.931685
26060	ANET	2020-10-09	224.63	228	222.5	227.61	1056022	{}	\N	2020-10-12 14:14:23.935948
26061	ANET	2020-10-12	225.38	225.83	220.58	224.39	482956	{}	\N	2020-10-12 14:14:23.939918
26062	ANTM	2020-09-28	258.8	261.8	255.78	257.06	866048	{}	\N	2020-10-12 14:14:24.88466
26063	ANTM	2020-09-29	258.35	261.75	256.56	259.79	1599250	{}	\N	2020-10-12 14:14:24.894308
26064	ANTM	2020-09-30	259.52	271.155	258.75	268.59	2435232	{}	\N	2020-10-12 14:14:24.901173
26065	ANTM	2020-10-01	268.59	270.765	266.11	270.17	1662289	{}	\N	2020-10-12 14:14:24.90691
26066	ANTM	2020-10-02	264.44	280.48	264.44	275.01	3008425	{}	\N	2020-10-12 14:14:24.911687
26067	ANTM	2020-10-05	276.98	285	276.68	284.79	1906968	{}	\N	2020-10-12 14:14:24.916945
26068	ANTM	2020-10-06	286.15	290.64	282.5	283.64	1956190	{}	\N	2020-10-12 14:14:24.921061
26069	ANTM	2020-10-07	285.23	293.935	285.23	293.12	1773841	{}	\N	2020-10-12 14:14:24.925143
26070	ANTM	2020-10-08	294.5	294.5	288.74	293.29	1962263	{}	\N	2020-10-12 14:14:24.929296
26071	ANTM	2020-10-09	295	298.45	293.83	294.41	1485555	{}	\N	2020-10-12 14:14:24.933571
26072	ANTM	2020-10-12	295.43	300.04	294.43	299.46	599673	{}	\N	2020-10-12 14:14:24.937705
26073	AOS	2020-09-28	52.71	53.64	52.71	53.29	1712438	{}	\N	2020-10-12 14:14:26.399515
26074	AOS	2020-09-29	53.32	54.46	53.22	53.73	1799684	{}	\N	2020-10-12 14:14:26.409059
26075	AOS	2020-09-30	53.71	54.015	52.72	52.8	2617948	{}	\N	2020-10-12 14:14:26.415629
26076	AOS	2020-10-01	53.32	54.065	52.96	53.59	1762994	{}	\N	2020-10-12 14:14:26.421486
26077	AOS	2020-10-02	52.81	54.19	52.59	53.53	1305716	{}	\N	2020-10-12 14:14:26.426282
26078	AOS	2020-10-05	54.05	55.22	53.955	55.21	1608422	{}	\N	2020-10-12 14:14:26.431674
26079	AOS	2020-10-06	55	55.21	53.38	53.6	2462725	{}	\N	2020-10-12 14:14:26.436172
26080	AOS	2020-10-07	54.25	54.47	53.82	54.08	2592120	{}	\N	2020-10-12 14:14:26.440208
26081	AOS	2020-10-08	54.46	57.01	54.155	56.75	4430168	{}	\N	2020-10-12 14:14:26.444594
26082	AOS	2020-10-09	56.94	57.63	56.51	57.19	3324690	{}	\N	2020-10-12 14:14:26.448926
26083	AOS	2020-10-12	57.2	57.95	57.12	57.95	338135	{}	\N	2020-10-12 14:14:26.453254
26084	APOPW	2020-09-29	0.078	0.079	0.078	0.0789	1911	{}	\N	2020-10-12 14:14:28.337308
26085	APOPW	2020-09-30	0.0445	0.0709	0.0445	0.0709	3590	{}	\N	2020-10-12 14:14:28.343952
26086	APOPW	2020-10-01	0.0502	0.07	0.0502	0.07	1227	{}	\N	2020-10-12 14:14:28.350167
26087	APOPW	2020-10-05	0.07	0.0763	0.0697	0.0763	2145	{}	\N	2020-10-12 14:14:28.355569
26088	APOPW	2020-10-06	0.0447	0.07	0.0447	0.07	1351	{}	\N	2020-10-12 14:14:28.360295
26089	APOPW	2020-10-08	0.0745	0.0745	0.07	0.07	2424	{}	\N	2020-10-12 14:14:28.364226
26090	APOPW	2020-10-09	0.07	0.07	0.07	0.07	500	{}	\N	2020-10-12 14:14:28.368599
26091	APOPW	2020-10-12	0.07	0.07	0.07	0.07	200	{}	\N	2020-10-12 14:14:28.373069
26092	ALG	2020-09-28	103.26	106.27	102.295	105.15	42870	{}	\N	2020-10-12 14:14:30.093789
26093	ALG	2020-09-29	105.56	107	105.01	106.4	117445	{}	\N	2020-10-12 14:14:30.103212
26094	ALG	2020-09-30	107.25	110.06	107.25	108.03	105575	{}	\N	2020-10-12 14:14:30.109948
26095	ALG	2020-10-01	107.6	109.54	105.89	108.07	56856	{}	\N	2020-10-12 14:14:30.116465
26096	ALG	2020-10-02	106.23	110.71	105.99	110.04	65099	{}	\N	2020-10-12 14:14:30.122001
26097	ALG	2020-10-05	111.36	113.145	110.86	112.1	52962	{}	\N	2020-10-12 14:14:30.126488
26098	ALG	2020-10-06	113.47	115.03	111.26	112.25	78860	{}	\N	2020-10-12 14:14:30.130848
26099	ALG	2020-10-07	113.87	114.29	111.48	113.97	120579	{}	\N	2020-10-12 14:14:30.135714
26100	ALG	2020-10-08	115.29	117.72	113.59	117.29	102116	{}	\N	2020-10-12 14:14:30.140228
26101	ALG	2020-10-09	117.91	118.94	116.6	117	74796	{}	\N	2020-10-12 14:14:30.144221
26102	ALG	2020-10-12	116.77	116.77	116.375	116.375	3506	{}	\N	2020-10-12 14:14:30.148919
26103	ALLY	2020-09-28	25.24	25.39	24.83	24.94	5106514	{}	\N	2020-10-12 14:14:31.088885
26104	ALLY	2020-09-29	24.75	25.01	24.41	24.45	7196807	{}	\N	2020-10-12 14:14:31.100909
26105	ALLY	2020-09-30	24.55	25.55	24.55	25.07	9198894	{}	\N	2020-10-12 14:14:31.108872
26106	ALLY	2020-10-01	25.47	25.575	24.93	25.5	5244204	{}	\N	2020-10-12 14:14:31.117766
26107	ALLY	2020-10-02	24.98	26.6	24.65	26.49	8579135	{}	\N	2020-10-12 14:14:31.122903
26108	ALLY	2020-10-05	26.9	27.43	26.74	27.13	11526616	{}	\N	2020-10-12 14:14:31.128511
26109	ALLY	2020-10-06	27.51	27.83	26.43	26.66	9171854	{}	\N	2020-10-12 14:14:31.133096
26110	ALLY	2020-10-07	27.52	27.88	27.02	27.6	10507118	{}	\N	2020-10-12 14:14:31.137399
26111	ALLY	2020-10-08	27.75	27.99	27.23	27.91	6338332	{}	\N	2020-10-12 14:14:31.141735
26112	ALLY	2020-10-09	28.15	28.35	27.69	27.85	7824629	{}	\N	2020-10-12 14:14:31.146078
26113	ALLY	2020-10-12	28.38	28.38	27.84	27.98	1375443	{}	\N	2020-10-12 14:14:31.150417
26114	ALT	2020-09-28	13.13	13.13	12	12.3	1217564	{}	\N	2020-10-12 14:14:32.434075
26115	ALT	2020-09-29	12.17	13.01	11.89	13.01	1377353	{}	\N	2020-10-12 14:14:32.441579
26116	ALT	2020-09-30	12.9	13.45	12.7006	13.2	961278	{}	\N	2020-10-12 14:14:32.448225
26117	ALT	2020-10-01	13.11	13.44	12.62	12.93	835929	{}	\N	2020-10-12 14:14:32.454937
26118	ALT	2020-10-02	12.7	13.35	12.52	12.97	732834	{}	\N	2020-10-12 14:14:32.460217
26119	ALT	2020-10-05	13.01	13.91	12.75	13.87	969039	{}	\N	2020-10-12 14:14:32.464653
26120	ALT	2020-10-06	13.93	14.4	13.6	13.64	828905	{}	\N	2020-10-12 14:14:32.469091
26121	ALT	2020-10-07	13.78	14.46	13.51	13.78	910629	{}	\N	2020-10-12 14:14:32.473392
26122	ALT	2020-10-08	13.98	14.065	13.21	13.3	801196	{}	\N	2020-10-12 14:14:32.477551
26123	ALT	2020-10-09	13.34	13.35	12.21	12.37	1213131	{}	\N	2020-10-12 14:14:32.482884
26124	ALT	2020-10-12	12.8	13.1	12.31	12.86	735299	{}	\N	2020-10-12 14:14:32.488028
26125	AMAT	2020-09-28	58.34	59.85	58.25	59.36	8095288	{}	\N	2020-10-12 14:14:33.40285
26126	AMAT	2020-09-29	59.44	60.25	59.15	59.67	4764016	{}	\N	2020-10-12 14:14:33.41273
26127	AMAT	2020-09-30	59.4069	60.183	58.99	59.45	8373586	{}	\N	2020-10-12 14:14:33.419892
26128	AMAT	2020-10-01	60.37	61.01	59.51	60.65	7133170	{}	\N	2020-10-12 14:14:33.425778
26129	AMAT	2020-10-02	59.09	59.66	58.19	58.21	6562914	{}	\N	2020-10-12 14:14:33.43108
26130	AMAT	2020-10-05	58.93	60.63	58.76	60.6	6582327	{}	\N	2020-10-12 14:14:33.436564
26131	AMAT	2020-10-06	60.33	62.16	60.17	60.4	6267490	{}	\N	2020-10-12 14:14:33.440977
26132	AMAT	2020-10-07	61.5	61.79	60.915	61.12	5133511	{}	\N	2020-10-12 14:14:33.445481
26133	AMAT	2020-10-08	61.7	62.697	61.37	62.6	5290875	{}	\N	2020-10-12 14:14:33.45012
26134	AMAT	2020-10-09	63.29	63.81	63.06	63.29	6015667	{}	\N	2020-10-12 14:14:33.454911
26135	AMAT	2020-10-12	64.06	64.71	63.98	64.09	1033696	{}	\N	2020-10-12 14:14:33.45924
26136	AMEH	2020-09-28	17.165	17.66	17.12	17.53	64630	{}	\N	2020-10-12 14:14:35.015536
26137	AMEH	2020-09-29	17.57	17.62	17.2	17.36	49415	{}	\N	2020-10-12 14:14:35.024203
26138	AMEH	2020-09-30	17.43	17.98	17.32	17.94	68171	{}	\N	2020-10-12 14:14:35.030536
26139	AMEH	2020-10-01	17.99	17.99	17.44	17.78	54217	{}	\N	2020-10-12 14:14:35.036642
26140	AMEH	2020-10-02	17.5	18.21	17.5	18.05	90952	{}	\N	2020-10-12 14:14:35.042201
26141	AMEH	2020-10-05	18.1	18.33	17.71	17.88	33256	{}	\N	2020-10-12 14:14:35.046923
26142	AMEH	2020-10-06	17.85	18.5	17.85	17.87	53792	{}	\N	2020-10-12 14:14:35.051865
26143	AMEH	2020-10-07	17.92	18.49	17.92	18.36	52432	{}	\N	2020-10-12 14:14:35.056671
26144	AMEH	2020-10-08	18.18	18.39	17.87	18.25	37114	{}	\N	2020-10-12 14:14:35.061386
26145	AMEH	2020-10-09	18.44	18.45	17.65	17.65	43664	{}	\N	2020-10-12 14:14:35.065846
26146	AMEH	2020-10-12	17.65	17.7	17.576	17.67	5031	{}	\N	2020-10-12 14:14:35.070566
26147	AMRS	2020-09-28	2.8	2.845	2.64	2.77	1415404	{}	\N	2020-10-12 14:14:36.978349
26148	AMRS	2020-09-29	2.77	2.94	2.73	2.92	1657497	{}	\N	2020-10-12 14:14:36.985156
26149	AMRS	2020-09-30	2.95	3.08	2.86	2.92	1844012	{}	\N	2020-10-12 14:14:36.990923
26150	AMRS	2020-10-01	3.02	3.2	3	3.11	2555963	{}	\N	2020-10-12 14:14:36.997526
26151	AMRS	2020-10-02	3.0481	3.12	3.01	3.04	1771106	{}	\N	2020-10-12 14:14:37.003745
26152	AMRS	2020-10-05	3.1	3.19	3.08	3.18	1387439	{}	\N	2020-10-12 14:14:37.008539
26153	AMRS	2020-10-06	3.18	3.22	3.03	3.12	2384174	{}	\N	2020-10-12 14:14:37.01364
26154	AMRS	2020-10-07	3.15	3.255	3.05	3.21	1913152	{}	\N	2020-10-12 14:14:37.018677
26155	AMRS	2020-10-08	3.27	3.3	3.055	3.07	1620145	{}	\N	2020-10-12 14:14:37.023002
26156	AMRS	2020-10-09	3.08	3.16	3.04	3.13	1529830	{}	\N	2020-10-12 14:14:37.027309
26157	AMRS	2020-10-12	3.16	3.16	3.055	3.12	160360	{}	\N	2020-10-12 14:14:37.031837
26158	AMUB	2020-09-28	6.69	6.8393	6.69	6.8393	5274	{}	\N	2020-10-12 14:14:39.127228
26159	AMUB	2020-09-29	6.845	6.96	6.845	6.93	3253	{}	\N	2020-10-12 14:14:39.135339
26160	AMUB	2020-09-30	6.89	6.94	6.75	6.79	14118	{}	\N	2020-10-12 14:14:39.141244
26161	AMUB	2020-10-01	6.7	6.76	6.68	6.73	9351	{}	\N	2020-10-12 14:14:39.146513
26162	AMUB	2020-10-02	6.6204	6.845	6.6204	6.83	17447	{}	\N	2020-10-12 14:14:39.151772
26163	AMUB	2020-10-05	6.96	7.0976	6.96	7.0976	2264	{}	\N	2020-10-12 14:14:39.156349
26164	AMUB	2020-10-06	7.1	7.1	6.9	6.9	38104	{}	\N	2020-10-12 14:14:39.160595
26165	AMUB	2020-10-07	6.91	6.9725	6.9	6.9725	345777	{}	\N	2020-10-12 14:14:39.16499
26166	AMUB	2020-10-08	7.1444	7.4011	7.1444	7.4011	5823	{}	\N	2020-10-12 14:14:39.169535
26167	AMUB	2020-10-09	7.49	7.5	7.32	7.44	27926	{}	\N	2020-10-12 14:14:39.17418
27506	ARKW	2020-09-28	108.74	108.74	106.82	107.84	614619	{}	\N	2020-10-12 14:17:41.271929
27507	ARKW	2020-09-29	108.06	108.8899	107.4155	108.19	429811	{}	\N	2020-10-12 14:17:41.287773
27508	ARKW	2020-09-30	108.34	109.82	107.8201	108.6	459379	{}	\N	2020-10-12 14:17:41.29348
27509	ARKW	2020-10-01	110.16	112.1	109.8715	112.06	732858	{}	\N	2020-10-12 14:17:41.298863
27510	ARKW	2020-10-02	108.84	111.99	108.72	110.06	965309	{}	\N	2020-10-12 14:17:41.303642
27511	ARKW	2020-10-05	111.55	112.7	111.0117	112.57	769324	{}	\N	2020-10-12 14:17:41.308012
27512	ARKW	2020-10-06	112.9	114.04	110.8621	111.83	960771	{}	\N	2020-10-12 14:17:41.313282
27513	ARKW	2020-10-07	113.42	114.86	113.02	114.37	542671	{}	\N	2020-10-12 14:17:41.317978
27514	ARKW	2020-10-08	116.53	116.63	115.54	115.92	600346	{}	\N	2020-10-12 14:17:41.322874
27515	ARKW	2020-10-09	117.23	117.99	116.8545	117.84	756604	{}	\N	2020-10-12 14:17:41.333449
27516	ARKW	2020-10-12	120.22	120.3	118.72	119.3788	373513	{}	\N	2020-10-12 14:17:41.339894
27517	AROW	2020-09-28	25.35	25.69	25.27	25.42	22035	{}	\N	2020-10-12 14:17:42.36824
27518	AROW	2020-09-29	25.34	25.37	24.74	25.05	16260	{}	\N	2020-10-12 14:17:42.376236
27519	AROW	2020-09-30	25.19	25.37	24.92	25.09	13949	{}	\N	2020-10-12 14:17:42.382417
27520	AROW	2020-10-01	25.09	25.13	24.8	25.1	18443	{}	\N	2020-10-12 14:17:42.38842
27521	AROW	2020-10-02	24.51	26.0599	24.51	25.65	9500	{}	\N	2020-10-12 14:17:42.394215
27522	AROW	2020-10-05	26.03	26.54	25.91	26.48	21975	{}	\N	2020-10-12 14:17:42.398938
27523	AROW	2020-10-06	26.79	27.46	26.18	26.41	26606	{}	\N	2020-10-12 14:17:42.404217
27524	AROW	2020-10-07	26.91	27.22	26.65	27.03	23822	{}	\N	2020-10-12 14:17:42.409371
27525	AROW	2020-10-08	27.27	27.4	26.81	27.16	21082	{}	\N	2020-10-12 14:17:42.414224
27526	AROW	2020-10-09	27.4	27.6735	26.85	27.19	14219	{}	\N	2020-10-12 14:17:42.418816
27527	AROW	2020-10-12	27.05	27.07	27.05	27.07	765	{}	\N	2020-10-12 14:17:42.42321
27528	ASA	2020-09-28	21.24	21.34	20.77	21.05	55292	{}	\N	2020-10-12 14:17:43.783444
27529	ASA	2020-09-29	21.3	21.54	21.09	21.41	129724	{}	\N	2020-10-12 14:17:43.794395
27530	ASA	2020-09-30	21.37	21.52	20.96	21.28	114048	{}	\N	2020-10-12 14:17:43.801669
27531	ASA	2020-10-01	21.49	22.29	21.42	21.88	173600	{}	\N	2020-10-12 14:17:43.807832
27532	ASA	2020-10-02	21.56	21.86	21.11	21.49	107128	{}	\N	2020-10-12 14:17:43.81291
27533	ASA	2020-10-05	21.53	22	21.35	21.52	96694	{}	\N	2020-10-12 14:17:43.818029
27534	ASA	2020-10-06	21.66	21.85	20.92	20.92	139159	{}	\N	2020-10-12 14:17:43.822422
27535	ASA	2020-10-07	20.92	21.28	20.92	21.03	72594	{}	\N	2020-10-12 14:17:43.826943
27536	ASA	2020-10-08	21.22	21.33	21.0612	21.21	43646	{}	\N	2020-10-12 14:17:43.831504
27537	ASA	2020-10-09	21.64	22.15	21.61	22.08	178308	{}	\N	2020-10-12 14:17:43.836162
27538	ASA	2020-10-12	22.23	22.24	21.81	21.945	51276	{}	\N	2020-10-12 14:17:43.84028
27539	ASHR	2020-09-28	34.11	34.22	34.03	34.18	1908975	{}	\N	2020-10-12 14:17:45.157914
27540	ASHR	2020-09-29	34.16	34.465	34.16	34.39	1903340	{}	\N	2020-10-12 14:17:45.167377
27541	ASHR	2020-09-30	34.24	34.51	34.21	34.48	3726959	{}	\N	2020-10-12 14:17:45.174241
27542	ASHR	2020-10-01	34.98	35.03	34.88	35.03	1718921	{}	\N	2020-10-12 14:17:45.180196
27543	ASHR	2020-10-02	34.41	34.775	34.32	34.58	2782504	{}	\N	2020-10-12 14:17:45.185175
27544	ASHR	2020-10-05	34.81	35.06	34.79	35.06	1779112	{}	\N	2020-10-12 14:17:45.190134
27545	ASHR	2020-10-06	35.19	35.21	34.965	35.09	3431806	{}	\N	2020-10-12 14:17:45.194343
27546	ASHR	2020-10-07	35.34	35.45	35.23	35.39	2584796	{}	\N	2020-10-12 14:17:45.198815
27547	ASHR	2020-10-08	35.43	35.48	35.35	35.45	2891445	{}	\N	2020-10-12 14:17:45.203136
27548	ASHR	2020-10-09	35.69	35.7781	35.63	35.73	1737480	{}	\N	2020-10-12 14:17:45.207278
27549	ASHR	2020-10-12	36.38	36.4	36.31	36.36	412425	{}	\N	2020-10-12 14:17:45.211521
27550	ASYS	2020-09-28	5.13	5.19	5	5.03	38637	{}	\N	2020-10-12 14:17:46.651302
27551	ASYS	2020-09-29	5.0214	5.09	4.99	5.01	17370	{}	\N	2020-10-12 14:17:46.661004
27552	ASYS	2020-09-30	4.97	5.05	4.85	4.89	40126	{}	\N	2020-10-12 14:17:46.667854
27553	ASYS	2020-10-01	4.9	5.1415	4.9	5.1	61962	{}	\N	2020-10-12 14:17:46.673574
27554	ASYS	2020-10-02	5.01	5.37	5.01	5.35	38261	{}	\N	2020-10-12 14:17:46.678683
27555	ASYS	2020-10-05	5.4	5.5	5.3902	5.45	26349	{}	\N	2020-10-12 14:17:46.68311
27556	ASYS	2020-10-06	5.45	5.648	5.41	5.43	33312	{}	\N	2020-10-12 14:17:46.687215
27557	ASYS	2020-10-07	5.49	5.65	5.47	5.65	35966	{}	\N	2020-10-12 14:17:46.691471
26168	ANIP	2020-09-28	27.55	28.24	27.55	27.72	52540	{}	\N	2020-10-12 14:14:40.844995
26169	ANIP	2020-09-29	28.055	28.52	27.65	28.24	67513	{}	\N	2020-10-12 14:14:40.855227
26170	ANIP	2020-09-30	28.34	29.02	28.15	28.21	60114	{}	\N	2020-10-12 14:14:40.859217
26171	ANIP	2020-10-01	28.45	28.7	28.21	28.45	51459	{}	\N	2020-10-12 14:14:40.863693
26172	ANIP	2020-10-02	27.91	28.96	27.6	28.59	67190	{}	\N	2020-10-12 14:14:40.868234
26173	ANIP	2020-10-05	28.88	29.4	28.77	29.13	47308	{}	\N	2020-10-12 14:14:40.872682
26174	ANIP	2020-10-06	29.49	29.64	28.66	29.03	60572	{}	\N	2020-10-12 14:14:40.8773
26175	ANIP	2020-10-07	29.11	29.33	28.02	29.14	85544	{}	\N	2020-10-12 14:14:40.881679
26176	ANIP	2020-10-08	29.5	30.33	29.5	30.19	68081	{}	\N	2020-10-12 14:14:40.88604
26177	ANIP	2020-10-09	30.5	31.2299	30.35	30.78	34975	{}	\N	2020-10-12 14:14:40.89064
26178	ANIP	2020-10-12	30.72	30.72	30.35	30.36	2553	{}	\N	2020-10-12 14:14:40.895248
26179	AON	2020-09-28	198.76	204.9	198.52	203.43	1509999	{}	\N	2020-10-12 14:14:41.828431
26180	AON	2020-09-29	202.74	205.43	202.2	205.09	2604817	{}	\N	2020-10-12 14:14:41.839359
26181	AON	2020-09-30	205.96	208.235	203.66	206.3	2996626	{}	\N	2020-10-12 14:14:41.846934
26182	AON	2020-10-01	207.21	207.71	204.29	206.18	2620916	{}	\N	2020-10-12 14:14:41.852923
26183	AON	2020-10-02	204.86	207.92	204.38	205.52	1536765	{}	\N	2020-10-12 14:14:41.858018
26184	AON	2020-10-05	207.06	207.9	204.74	206.51	1728583	{}	\N	2020-10-12 14:14:41.862824
26185	AON	2020-10-06	207.87	209.38	204.7719	205.65	1961277	{}	\N	2020-10-12 14:14:41.867318
26186	AON	2020-10-07	206.67	209.115	206.39	208.37	2088327	{}	\N	2020-10-12 14:14:41.871543
26187	AON	2020-10-08	210	212.95	207.94	208.07	1777244	{}	\N	2020-10-12 14:14:41.875798
26188	AON	2020-10-09	209.39	210.51	208.32	209.19	1960229	{}	\N	2020-10-12 14:14:41.880103
26189	AON	2020-10-12	209.81	212.02	208.8	211.815	504532	{}	\N	2020-10-12 14:14:41.884096
26190	APDN	2020-09-28	7.75	8.38	7.6	8.34	212382	{}	\N	2020-10-12 14:14:42.930903
26191	APDN	2020-09-29	8.38	8.94	7.715	7.8	620422	{}	\N	2020-10-12 14:14:42.941141
26192	APDN	2020-09-30	7.8	8.06	7.67	7.73	97542	{}	\N	2020-10-12 14:14:42.950374
26193	APDN	2020-10-01	7.845	7.845	7.55	7.6	80823	{}	\N	2020-10-12 14:14:42.955859
26194	APDN	2020-10-02	7.57	7.67	7.37	7.4	116976	{}	\N	2020-10-12 14:14:42.961033
26195	APDN	2020-10-05	7.47	7.62	7.42	7.5	43375	{}	\N	2020-10-12 14:14:42.966124
26196	APDN	2020-10-06	7.56	7.74	7.5	7.71	91687	{}	\N	2020-10-12 14:14:42.970508
26197	APDN	2020-10-07	7.7	7.8212	7.5293	7.54	71753	{}	\N	2020-10-12 14:14:42.975172
26198	APDN	2020-10-08	7.65	7.7156	7.54	7.66	52522	{}	\N	2020-10-12 14:14:42.979559
26199	APDN	2020-10-09	7.62	7.71	7.57	7.65	58442	{}	\N	2020-10-12 14:14:42.983729
26200	APDN	2020-10-12	7.85	7.85	7.66	7.7107	57220	{}	\N	2020-10-12 14:14:42.988126
26201	APOP	2020-09-28	2.22	2.25	2.15	2.2	69211	{}	\N	2020-10-12 14:14:43.950378
26202	APOP	2020-09-29	2.18	2.25	2.1	2.15	56349	{}	\N	2020-10-12 14:14:43.960693
26203	APOP	2020-09-30	2.11	2.25	2.1	2.1	77244	{}	\N	2020-10-12 14:14:43.968112
26204	APOP	2020-10-01	2.1393	2.14	2.07	2.08	56078	{}	\N	2020-10-12 14:14:43.973511
26205	APOP	2020-10-02	2.08	2.66	1.99	2.04	1517161	{}	\N	2020-10-12 14:14:43.978503
26206	APOP	2020-10-05	2.07	2.14	2.0201	2.14	109126	{}	\N	2020-10-12 14:14:43.983451
26207	APOP	2020-10-06	2.14	2.15	2.08	2.0989	55255	{}	\N	2020-10-12 14:14:43.987505
26208	APOP	2020-10-07	2.09	2.15	2.09	2.1376	25950	{}	\N	2020-10-12 14:14:43.991224
26209	APOP	2020-10-08	2.1	2.229	2.1	2.18	65709	{}	\N	2020-10-12 14:14:43.995511
26210	APOP	2020-10-09	2.18	2.21	2.16	2.21	27064	{}	\N	2020-10-12 14:14:44.000029
26211	APOP	2020-10-12	2.24	2.3	2.2108	2.25	15670	{}	\N	2020-10-12 14:14:44.004408
26212	ALB	2020-09-28	86.465	87.4	83.88	84.84	1339044	{}	\N	2020-10-12 14:14:45.019975
26213	ALB	2020-09-29	84.95	87.63	84.78	86.86	2348138	{}	\N	2020-10-12 14:14:45.027712
26214	ALB	2020-09-30	87.41	90.13	87.22	89.28	2525695	{}	\N	2020-10-12 14:14:45.033405
26215	ALB	2020-10-01	90	90.9699	86.47	86.73	2525030	{}	\N	2020-10-12 14:14:45.038774
26216	ALB	2020-10-02	85.44	91.0789	84.57	89.29	2227679	{}	\N	2020-10-12 14:14:45.043417
26217	ALB	2020-10-05	91.26	95.89	91.14	94.76	3214166	{}	\N	2020-10-12 14:14:45.048119
26218	ALB	2020-10-06	96.28	96.9691	93.52	93.74	2607433	{}	\N	2020-10-12 14:14:45.052495
26219	ALB	2020-10-07	96	97.9869	95.6	96.69	2152525	{}	\N	2020-10-12 14:14:45.056354
26220	ALB	2020-10-08	98.12	98.365	95.96	96.95	2280648	{}	\N	2020-10-12 14:14:45.060023
26221	ALB	2020-10-09	98.1	99.13	95.66	96.03	1451853	{}	\N	2020-10-12 14:14:45.063752
26222	ALB	2020-10-12	96.54	96.73	95.1155	95.66	137989	{}	\N	2020-10-12 14:14:45.067419
26223	ALGT	2020-09-28	126.08	131.14	125.6	127.11	187725	{}	\N	2020-10-12 14:14:46.057532
26224	ALGT	2020-09-29	125.51	126.0672	120.22	121.09	185250	{}	\N	2020-10-12 14:14:46.071379
26225	ALGT	2020-09-30	124.53	126.13	119.7	119.8	250828	{}	\N	2020-10-12 14:14:46.078319
26226	ALGT	2020-10-01	122.0208	122.0208	117.88	118.18	270832	{}	\N	2020-10-12 14:14:46.083657
26227	ALGT	2020-10-02	114.33	124.71	113.67	119.65	434592	{}	\N	2020-10-12 14:14:46.088367
26228	ALGT	2020-10-05	121.47	122.95	117.24	117.66	234531	{}	\N	2020-10-12 14:14:46.092915
26229	ALGT	2020-10-06	119.62	120.17	112.71	112.73	328726	{}	\N	2020-10-12 14:14:46.097082
26230	ALGT	2020-10-07	115.8	116.75	113.995	116.33	283390	{}	\N	2020-10-12 14:14:46.10047
26231	ALGT	2020-10-08	118.0097	122.39	117.51	121.78	275068	{}	\N	2020-10-12 14:14:46.104209
26232	ALGT	2020-10-09	121.71	124.63	118.58	124.41	198983	{}	\N	2020-10-12 14:14:46.108083
26233	ALGT	2020-10-12	128.01	128.73	126.59	127.185	26759	{}	\N	2020-10-12 14:14:46.112032
26234	ALNY	2020-09-28	139.16	140	137.17	139.81	340407	{}	\N	2020-10-12 14:14:47.797251
26235	ALNY	2020-09-29	140	145.25	138.66	145.01	488865	{}	\N	2020-10-12 14:14:47.806289
26236	ALNY	2020-09-30	144.17	147	142.37	145.6	521300	{}	\N	2020-10-12 14:14:47.812886
26237	ALNY	2020-10-01	146.7	148.75	143.3	143.74	542299	{}	\N	2020-10-12 14:14:47.818642
26238	ALNY	2020-10-02	141.84	143.72	135.24	135.76	542641	{}	\N	2020-10-12 14:14:47.823432
26239	ALNY	2020-10-05	136.29	142.27	136.29	142	493598	{}	\N	2020-10-12 14:14:47.827553
26240	ALNY	2020-10-06	142.01	146.765	142.01	143.26	356214	{}	\N	2020-10-12 14:14:47.831976
26241	ALNY	2020-10-07	144.03	146.5	142.58	145.4	385422	{}	\N	2020-10-12 14:14:47.836152
26242	ALNY	2020-10-08	146.8	147.76	144.805	146.04	248211	{}	\N	2020-10-12 14:14:47.840072
26243	ALNY	2020-10-09	145.25	147.975	145.19	147	271100	{}	\N	2020-10-12 14:14:47.843987
26244	ALNY	2020-10-12	148	148.05	145.09	146.58	31548	{}	\N	2020-10-12 14:14:47.848155
26245	AMG	2020-09-28	65.715	68.5643	65.715	68.05	333672	{}	\N	2020-10-12 14:14:49.865597
26246	AMG	2020-09-29	68.08	68.24	66.95	68.01	520554	{}	\N	2020-10-12 14:14:49.875545
26247	AMG	2020-09-30	68.44	69.56	67.53	68.38	767338	{}	\N	2020-10-12 14:14:49.882093
26248	AMG	2020-10-01	68.885	69.11	67.02	67.47	586121	{}	\N	2020-10-12 14:14:49.887685
26249	AMG	2020-10-02	66.43	69.94	66.43	69.1	1406107	{}	\N	2020-10-12 14:14:49.892352
26250	AMG	2020-10-05	70.16	71.325	69.8434	71.14	423834	{}	\N	2020-10-12 14:14:49.897239
26251	AMG	2020-10-06	71.98	73.56	70.73	71.61	665681	{}	\N	2020-10-12 14:14:49.901273
26252	AMG	2020-10-07	72.41	74.29	72.41	73.02	647855	{}	\N	2020-10-12 14:14:49.904999
26253	AMG	2020-10-08	74.14	76.27	73.78	76.25	938637	{}	\N	2020-10-12 14:14:49.908981
26254	AMG	2020-10-09	76.5	77.2799	75.55	76.41	556914	{}	\N	2020-10-12 14:14:49.913383
26255	AMG	2020-10-12	76.45	77.56	76.45	77.24	17959	{}	\N	2020-10-12 14:14:49.917202
26256	AMN	2020-09-28	57.68	58.46	57.52	58.28	373373	{}	\N	2020-10-12 14:14:51.756629
26257	AMN	2020-09-29	58.51	58.64	57.46	57.98	712228	{}	\N	2020-10-12 14:14:51.764382
26258	AMN	2020-09-30	58.4	59.26	58.07	58.46	691297	{}	\N	2020-10-12 14:14:51.770173
26259	AMN	2020-10-01	59.005	59.0316	56.45	57.28	460378	{}	\N	2020-10-12 14:14:51.77649
26260	AMN	2020-10-02	56.22	58.68	56.0501	58.16	453733	{}	\N	2020-10-12 14:14:51.781948
26261	AMN	2020-10-05	58.66	59.19	58.09	58.94	461099	{}	\N	2020-10-12 14:14:51.786608
26262	AMN	2020-10-06	59.4	60	58.34	58.74	579699	{}	\N	2020-10-12 14:14:51.790629
26263	AMN	2020-10-07	58.98	60.74	58.94	60.43	648811	{}	\N	2020-10-12 14:14:51.794855
26264	AMN	2020-10-08	61.77	63.01	61.28	63.01	604347	{}	\N	2020-10-12 14:14:51.79921
26265	AMN	2020-10-09	63.58	64.43	63.43	64.23	589226	{}	\N	2020-10-12 14:14:51.803296
26266	AMN	2020-10-12	64.58	65.34	64.24	65.155	45907	{}	\N	2020-10-12 14:14:51.807466
26267	AMRHW	2020-09-28	0.0533	0.0533	0.0532	0.0532	3605	{}	\N	2020-10-12 14:14:53.856362
26268	AMRHW	2020-09-29	0.06	0.06	0.05998	0.05998	4373	{}	\N	2020-10-12 14:14:53.865837
26269	AMRHW	2020-09-30	0.06	0.06	0.059	0.06	7818	{}	\N	2020-10-12 14:14:53.872312
26270	AMRHW	2020-10-01	0.0658	0.0658	0.0465	0.0465	4666	{}	\N	2020-10-12 14:14:53.878048
26271	AMRHW	2020-10-02	0.0596	0.0596	0.0593	0.0596	700	{}	\N	2020-10-12 14:14:53.883268
26272	AMRHW	2020-10-05	0.0619	0.0619	0.05	0.0503	1151	{}	\N	2020-10-12 14:14:53.888043
26273	AMRHW	2020-10-06	0.05	0.05	0.0499	0.0499	800	{}	\N	2020-10-12 14:14:53.891786
26274	AMRHW	2020-10-07	0.05	0.05	0.0499	0.0499	600	{}	\N	2020-10-12 14:14:53.89583
26275	AMRHW	2020-10-09	0.06	0.0655	0.05	0.0502	11732	{}	\N	2020-10-12 14:14:53.899735
26276	AMRHW	2020-10-12	0.06	0.0847	0.06	0.0847	5748	{}	\N	2020-10-12 14:14:53.903655
26277	AMTD	2020-09-28	38.49	39.29	38.44	39.17	2048805	{}	\N	2020-10-12 14:14:54.781363
26278	AMTD	2020-09-29	39.05	39.2	38.36	38.74	2941346	{}	\N	2020-10-12 14:14:54.789887
26279	AMTD	2020-09-30	38.93	39.64	38.86	39.15	2199711	{}	\N	2020-10-12 14:14:54.797036
26280	AMTD	2020-10-01	40.01	40.92	39.62	40.92	5315088	{}	\N	2020-10-12 14:14:54.80314
26281	AMTD	2020-10-02	40.18	41.67	40.14	40.82	7003174	{}	\N	2020-10-12 14:14:54.807723
26282	AMTD	2020-10-05	41.42	41.94	39.82	39.97	40681259	{}	\N	2020-10-12 14:14:54.812331
26283	ANIK	2020-09-28	34.55	35.7417	34.5	35.34	138719	{}	\N	2020-10-12 14:14:57.550477
26284	ANIK	2020-09-29	35.14	36.1234	33.78	34.18	87214	{}	\N	2020-10-12 14:14:57.559107
26285	ANIK	2020-09-30	34.2	35.59	33.665	35.39	159242	{}	\N	2020-10-12 14:14:57.564449
26286	ANIK	2020-10-01	35.56	37.73	35.21	37.25	167989	{}	\N	2020-10-12 14:14:57.569936
26287	ANIK	2020-10-02	36.41	37.46	35.45	36.09	159150	{}	\N	2020-10-12 14:14:57.574492
26288	ANIK	2020-10-05	36.62	37.375	35.78	35.99	82781	{}	\N	2020-10-12 14:14:57.579159
26289	ANIK	2020-10-06	36.06	36.635	35.35	35.75	150665	{}	\N	2020-10-12 14:14:57.583516
26290	ANIK	2020-10-07	35.91	37.77	35.89	37.53	114592	{}	\N	2020-10-12 14:14:57.587315
26291	ANIK	2020-10-08	37.99	38.35	37.06	37.93	74973	{}	\N	2020-10-12 14:14:57.591657
26292	ANIK	2020-10-09	38.18	38.42	36.5	38.32	54530	{}	\N	2020-10-12 14:14:57.595465
26293	ANIK	2020-10-12	38.43	38.58	38.39	38.52	3419	{}	\N	2020-10-12 14:14:57.599175
26294	AOR	2020-09-28	48.27	48.4	48.2138	48.4	81767	{}	\N	2020-10-12 14:14:59.521397
26295	AOR	2020-09-29	48.4	48.4283	48.1946	48.32	108623	{}	\N	2020-10-12 14:14:59.529839
26296	AOR	2020-09-30	48.22	48.6	48.22	48.44	229383	{}	\N	2020-10-12 14:14:59.536003
26297	AOR	2020-10-01	48.51	48.655	48.46	48.64	87497	{}	\N	2020-10-12 14:14:59.541608
26298	AOR	2020-10-02	47.98	48.37	47.98	48.2	183999	{}	\N	2020-10-12 14:14:59.547221
26299	AOR	2020-10-05	48.43	48.82	48.43	48.66	103916	{}	\N	2020-10-12 14:14:59.5518
26300	AOR	2020-10-06	48.73	48.9172	48.348	48.38	111817	{}	\N	2020-10-12 14:14:59.556171
26301	AOR	2020-10-07	48.7	48.8755	48.66	48.78	68871	{}	\N	2020-10-12 14:14:59.560277
26302	AOR	2020-10-08	48.93	49.13	48.93	49.13	93216	{}	\N	2020-10-12 14:14:59.564661
26303	AOR	2020-10-09	49.21	49.37	49.2	49.37	159336	{}	\N	2020-10-12 14:14:59.569968
26304	AOR	2020-10-12	49.52	49.58	49.49	49.58	21188	{}	\N	2020-10-12 14:14:59.574722
26305	APEI	2020-09-28	28.05	28.75	28.05	28.47	75150	{}	\N	2020-10-12 14:15:00.505801
26306	APEI	2020-09-29	28.33	28.79	27.72	28.64	70678	{}	\N	2020-10-12 14:15:00.514271
26307	APEI	2020-09-30	28.63	29.04	28.07	28.19	110653	{}	\N	2020-10-12 14:15:00.519969
26308	APEI	2020-10-01	28.38	28.76	27.91	28.6	62892	{}	\N	2020-10-12 14:15:00.525517
26309	APEI	2020-10-02	28.295	29.36	28.22	29.22	65011	{}	\N	2020-10-12 14:15:00.531258
26310	APEI	2020-10-05	29.21	30.6	29.18	30.41	78128	{}	\N	2020-10-12 14:15:00.536801
26311	APEI	2020-10-06	30.73	31.87	29.8066	30.63	77768	{}	\N	2020-10-12 14:15:00.541318
26312	APEI	2020-10-07	30.975	30.98	29.31	29.39	112309	{}	\N	2020-10-12 14:15:00.545632
26313	APEI	2020-10-08	29.68	29.975	29.32	29.62	51235	{}	\N	2020-10-12 14:15:00.550253
26314	APEI	2020-10-09	29.72	30.69	29.14	30.15	62677	{}	\N	2020-10-12 14:15:00.554753
26315	APEI	2020-10-12	30.33	31.24	30.245	31.08	19675	{}	\N	2020-10-12 14:15:00.558905
26316	APPF	2020-09-28	145.72	146.8	141.885	144.07	156738	{}	\N	2020-10-12 14:15:01.516016
26317	APPF	2020-09-29	143.95	145.62	139.53	140.11	127772	{}	\N	2020-10-12 14:15:01.525211
26318	APPF	2020-09-30	139.86	143.15	138.6	141.81	254131	{}	\N	2020-10-12 14:15:01.532334
26319	APPF	2020-10-01	142.99	149.3	141.44	149.26	165206	{}	\N	2020-10-12 14:15:01.537982
26320	APPF	2020-10-02	146.51	149.19	145.5	147.56	130476	{}	\N	2020-10-12 14:15:01.542509
26321	APPF	2020-10-05	148.27	154.805	148.27	154.34	240121	{}	\N	2020-10-12 14:15:01.547639
26322	APPF	2020-10-06	154.69	160.51	154.69	157.05	120538	{}	\N	2020-10-12 14:15:01.55148
26323	APPF	2020-10-07	157.83	162.14	157.83	159.9	109601	{}	\N	2020-10-12 14:15:01.555499
26324	APPF	2020-10-08	160.759	164.63	160.759	161.84	78413	{}	\N	2020-10-12 14:15:01.55922
26325	APPF	2020-10-09	163.31	164.5902	162.48	164.31	72370	{}	\N	2020-10-12 14:15:01.563013
26326	APPF	2020-10-12	165.29	167.39	164.55	166.43	6400	{}	\N	2020-10-12 14:15:01.566912
26327	ALFA	2020-09-28	63.3168	63.8621	63.24	63.8621	702	{}	\N	2020-10-12 14:15:05.340497
26328	ALFA	2020-09-29	64.59	64.59	63.95	63.95	649	{}	\N	2020-10-12 14:15:05.347686
26329	ALFA	2020-09-30	64.77	64.77	64.62	64.62	4220	{}	\N	2020-10-12 14:15:05.3533
26330	ALFA	2020-10-01	65.35	66.0751	65.35	66.0751	780	{}	\N	2020-10-12 14:15:05.358062
26331	ALFA	2020-10-02	65.19	65.19	65.0302	65.0302	644	{}	\N	2020-10-12 14:15:05.362527
26332	ALFA	2020-10-05	65.5	66.655	65.5	66.655	9904	{}	\N	2020-10-12 14:15:05.367066
26333	ALFA	2020-10-06	66.63	66.63	65.9769	65.9769	8279	{}	\N	2020-10-12 14:15:05.371596
26334	ALFA	2020-10-07	67.6801	67.7821	67.6801	67.7821	426	{}	\N	2020-10-12 14:15:05.376702
26335	ALFA	2020-10-08	68.09	68.09	67.8474	67.8681	1319	{}	\N	2020-10-12 14:15:05.381266
26336	ALFA	2020-10-09	69.261	69.261	69.261	69.261	329	{}	\N	2020-10-12 14:15:05.385898
26337	ALLE	2020-09-28	98.82	100.72	98.455	99.21	556925	{}	\N	2020-10-12 14:15:06.446036
26338	ALLE	2020-09-29	99.02	100.49	98.575	99.17	896956	{}	\N	2020-10-12 14:15:06.456516
26339	ALLE	2020-09-30	99.74	101.72	97.9	98.91	2609973	{}	\N	2020-10-12 14:15:06.463138
26340	ALLE	2020-10-01	99.61	101.68	98.17	98.5	1610077	{}	\N	2020-10-12 14:15:06.468921
26341	ALLE	2020-10-02	96.53	99.69	96.32	98.82	3023103	{}	\N	2020-10-12 14:15:06.473958
26342	ALLE	2020-10-05	99.89	101.22	99.54	100.8	1924339	{}	\N	2020-10-12 14:15:06.478763
26343	ALLE	2020-10-06	101.01	102.76	100.44	100.86	1163518	{}	\N	2020-10-12 14:15:06.483013
26344	ALLE	2020-10-07	101.72	102.4	100.72	101.56	1276357	{}	\N	2020-10-12 14:15:06.487164
26345	ALLE	2020-10-08	102.7	103.77	102.05	103.69	1014850	{}	\N	2020-10-12 14:15:06.491661
26346	ALLE	2020-10-09	104.67	105.375	103.92	103.99	900847	{}	\N	2020-10-12 14:15:06.496163
26347	ALLE	2020-10-12	104.68	106.6	104.68	106.57	109340	{}	\N	2020-10-12 14:15:06.501153
26348	ALOT	2020-09-28	7.72	7.72	7.46	7.46	27477	{}	\N	2020-10-12 14:15:07.476068
26349	ALOT	2020-09-29	7.47	7.62	7.4636	7.53	3572	{}	\N	2020-10-12 14:15:07.484878
26350	ALOT	2020-09-30	7.58	8.19	7.58	8.01	17335	{}	\N	2020-10-12 14:15:07.490689
26351	ALOT	2020-10-01	8.06	8.58	8	8	29259	{}	\N	2020-10-12 14:15:07.4962
26352	ALOT	2020-10-02	8	8.2	7.36	8.08	12441	{}	\N	2020-10-12 14:15:07.50156
26353	ALOT	2020-10-05	8.07	8.3	7.56	7.67	14287	{}	\N	2020-10-12 14:15:07.506503
26354	ALOT	2020-10-06	7.82	7.9985	7.555	7.8	10499	{}	\N	2020-10-12 14:15:07.510563
26355	ALOT	2020-10-07	8.05	8.05	7.55	7.55	12381	{}	\N	2020-10-12 14:15:07.515117
26356	ALOT	2020-10-08	7.56	8.1699	7.56	7.99	7446	{}	\N	2020-10-12 14:15:07.519279
26357	ALOT	2020-10-09	7.94	7.99	7.78	7.9	7129	{}	\N	2020-10-12 14:15:07.523319
26358	ALOT	2020-10-12	7.91	7.91	7.91	7.91	476	{}	\N	2020-10-12 14:15:07.527398
26359	ALXN	2020-09-28	115.7	115.76	112.73	113.3	1245118	{}	\N	2020-10-12 14:15:08.496239
26360	ALXN	2020-09-29	113.57	114.12	112.035	112.46	987023	{}	\N	2020-10-12 14:15:08.504912
26361	ALXN	2020-09-30	113.13	115	112.85	114.43	1581785	{}	\N	2020-10-12 14:15:08.510697
26362	ALXN	2020-10-01	114.62	116.53	114.11	115.63	1413715	{}	\N	2020-10-12 14:15:08.517269
26363	ALXN	2020-10-02	113.83	115.25	110.42	111.55	1918875	{}	\N	2020-10-12 14:15:08.522142
26364	ALXN	2020-10-05	113.43	116.165	113.07	115.25	2825262	{}	\N	2020-10-12 14:15:08.526514
26365	ALXN	2020-10-06	119.17	119.9	114.69	116.08	4111600	{}	\N	2020-10-12 14:15:08.531156
26366	ALXN	2020-10-07	117.5	126.65	117.37	125.33	6478001	{}	\N	2020-10-12 14:15:08.535472
26367	ALXN	2020-10-08	125.64	126.73	121.64	123.14	2764640	{}	\N	2020-10-12 14:15:08.539834
26368	ALXN	2020-10-09	122.82	128.57	120.78	124.52	4548493	{}	\N	2020-10-12 14:15:08.544113
26369	ALXN	2020-10-12	124.83	125.185	122.31	122.9159	409958	{}	\N	2020-10-12 14:15:08.548393
26370	AME	2020-09-28	98.82	100.44	98.82	99.49	614369	{}	\N	2020-10-12 14:15:12.526912
26371	AME	2020-09-29	99.95	100.91	99.3	99.89	1916534	{}	\N	2020-10-12 14:15:12.535366
26372	AME	2020-09-30	100	101.08	98.76	99.4	1835389	{}	\N	2020-10-12 14:15:12.541127
26373	AME	2020-10-01	100.7	101.38	99.01	99.85	1900380	{}	\N	2020-10-12 14:15:12.546576
26374	AME	2020-10-02	98.15	101.68	98.04	100.78	1283316	{}	\N	2020-10-12 14:15:12.551352
26375	AME	2020-10-05	101.94	103.97	101.76	103.5	1717406	{}	\N	2020-10-12 14:15:12.5565
26376	AME	2020-10-06	103.5	105.87	103.22	104.33	2062500	{}	\N	2020-10-12 14:15:12.560514
26377	AME	2020-10-07	105.63	107.28	105.32	106.54	2301513	{}	\N	2020-10-12 14:15:12.564686
26378	AME	2020-10-08	107.01	107.01	105.73	106.5	2566392	{}	\N	2020-10-12 14:15:12.568822
26379	AME	2020-10-09	107.5	108.63	106.78	106.94	1583958	{}	\N	2020-10-12 14:15:12.57293
26380	AME	2020-10-12	107.81	107.92	107.11	107.42	422910	{}	\N	2020-10-12 14:15:12.576866
26381	AMOV	2020-09-28	11.88	12.1	11.88	12.04	6190	{}	\N	2020-10-12 14:15:17.928743
26382	AMOV	2020-09-29	12.01	12.1376	11.98	12.1376	8172	{}	\N	2020-10-12 14:15:17.935901
26383	AMOV	2020-09-30	12.425	12.585	12.425	12.48	4100	{}	\N	2020-10-12 14:15:17.941804
26384	AMOV	2020-10-01	12.31	12.31	12.2	12.2015	7536	{}	\N	2020-10-12 14:15:17.946557
26385	AMOV	2020-10-02	12.3	12.35	12.05	12.19	8270	{}	\N	2020-10-12 14:15:17.951741
26386	AMOV	2020-10-05	12.5	12.51	12.45	12.451	5568	{}	\N	2020-10-12 14:15:17.956522
26387	AMOV	2020-10-06	12.5	12.5	12.44	12.44	12347	{}	\N	2020-10-12 14:15:17.960707
26388	AMOV	2020-10-07	12.43	12.56	12.43	12.43	4008	{}	\N	2020-10-12 14:15:17.965075
26389	AMOV	2020-10-08	12.43	12.8	12.43	12.79	6264	{}	\N	2020-10-12 14:15:17.969371
26390	AMOV	2020-10-09	13.03	13.03	12.83	12.83	2837	{}	\N	2020-10-12 14:15:17.973789
26391	AMRK	2020-09-28	33.37	34.95	33.0972	34.51	137153	{}	\N	2020-10-12 14:15:18.936916
26392	AMRK	2020-09-29	35.05	35.05	33.02	33.5	179296	{}	\N	2020-10-12 14:15:18.945897
26393	AMRK	2020-09-30	33.65	35.14	33.33	33.72	147296	{}	\N	2020-10-12 14:15:18.952194
26394	AMRK	2020-10-01	34.5	34.5	32.58	33.4	66383	{}	\N	2020-10-12 14:15:18.957599
26395	AMRK	2020-10-02	33.05	34.4348	32.8662	34.15	37649	{}	\N	2020-10-12 14:15:18.962073
26396	AMRK	2020-10-05	34.23	35.6725	34.15	35.15	127597	{}	\N	2020-10-12 14:15:18.966875
26397	AMRK	2020-10-06	35.39	35.64	34.57	34.82	50973	{}	\N	2020-10-12 14:15:18.971489
26398	AMRK	2020-10-07	35.01	35.9	33.1165	35.75	92707	{}	\N	2020-10-12 14:15:18.975763
26399	AMRK	2020-10-08	35.99	36.1	35.1857	36.05	53758	{}	\N	2020-10-12 14:15:18.97978
26400	AMRK	2020-10-09	36.25	37.45	36.0101	37	63349	{}	\N	2020-10-12 14:15:18.983731
26401	AMRK	2020-10-12	37	37.45	36.5717	37	22987	{}	\N	2020-10-12 14:15:18.987648
26402	AMTX	2020-09-28	2.72	2.88	2.61	2.875	638851	{}	\N	2020-10-12 14:15:20.97051
26403	AMTX	2020-09-29	2.7816	3.29	2.78	3.17	1796857	{}	\N	2020-10-12 14:15:20.980595
26404	AMTX	2020-09-30	3.17	3.28	2.98	3.2	1009404	{}	\N	2020-10-12 14:15:20.987187
26405	AMTX	2020-10-01	3.14	3.9	3.14	3.7	3575511	{}	\N	2020-10-12 14:15:20.993306
26406	AMTX	2020-10-02	3.48	3.88	3.38	3.48	2915272	{}	\N	2020-10-12 14:15:20.999222
26407	AMTX	2020-10-05	3.4	3.65	3.33	3.47	2680953	{}	\N	2020-10-12 14:15:21.003244
26408	AMTX	2020-10-06	3.56	3.7	3.37	3.47	1490872	{}	\N	2020-10-12 14:15:21.00799
26409	AMTX	2020-10-07	3.55	3.85	3.48	3.67	1571801	{}	\N	2020-10-12 14:15:21.012596
26410	AMTX	2020-10-08	3.67	3.715	3.45	3.49	1202994	{}	\N	2020-10-12 14:15:21.016955
26411	AMTX	2020-10-09	3.47	3.54	3.34	3.39	547792	{}	\N	2020-10-12 14:15:21.020922
26412	AMTX	2020-10-12	3.415	3.415	3.2	3.31	247364	{}	\N	2020-10-12 14:15:21.024908
26413	ANGO	2020-09-28	10.19	10.92	10.11	10.67	547416	{}	\N	2020-10-12 14:15:22.797683
26414	ANGO	2020-09-29	11.08	12.86	10.79	12.74	2013008	{}	\N	2020-10-12 14:15:22.807002
26415	ANGO	2020-09-30	12.5	12.74	11.78	12.06	772871	{}	\N	2020-10-12 14:15:22.813756
26416	ANGO	2020-10-01	12.04	12.14	11.4	11.47	430157	{}	\N	2020-10-12 14:15:22.819799
26417	ANGO	2020-10-02	11.32	11.65	11.09	11.495	466084	{}	\N	2020-10-12 14:15:22.825087
26418	ANGO	2020-10-05	11.58	11.95	11.24	11.855	260622	{}	\N	2020-10-12 14:15:22.83011
26419	ANGO	2020-10-06	11.92	12.08	11.59	11.65	242741	{}	\N	2020-10-12 14:15:22.834636
26420	ANGO	2020-10-07	11.66	11.8	11.35	11.645	300256	{}	\N	2020-10-12 14:15:22.838892
26421	ANGO	2020-10-08	11.63	11.97	11.31	11.83	260748	{}	\N	2020-10-12 14:15:22.843236
26422	ANGO	2020-10-09	11.8	11.99	11.43	11.62	208096	{}	\N	2020-10-12 14:15:22.847495
26423	ANGO	2020-10-12	11.67	11.67	11.36	11.46	66515	{}	\N	2020-10-12 14:15:22.851689
26424	FHB	2020-09-28	14.77	15.155	14.75	14.91	767919	{}	\N	2020-10-12 14:15:24.83583
26425	FHB	2020-09-29	14.83	14.85	14.405	14.52	1532838	{}	\N	2020-10-12 14:15:24.844646
26426	FHB	2020-09-30	14.51	14.96	14.325	14.47	1390450	{}	\N	2020-10-12 14:15:24.850837
26427	FHB	2020-10-01	14.41	14.7	14.155	14.42	1333741	{}	\N	2020-10-12 14:15:24.856312
26428	FHB	2020-10-02	14.18	15.03	14.18	14.96	1238089	{}	\N	2020-10-12 14:15:24.861725
26429	FHB	2020-10-05	15.14	15.5	15.06	15.38	1418231	{}	\N	2020-10-12 14:15:24.866877
26430	FHB	2020-10-06	15.52	16.195	15.39	15.55	1498405	{}	\N	2020-10-12 14:15:24.871248
26431	FHB	2020-10-07	15.71	16.03	15.555	15.96	1084933	{}	\N	2020-10-12 14:15:24.875505
26432	FHB	2020-10-08	16.04	16.25	15.91	16.22	1046207	{}	\N	2020-10-12 14:15:24.880055
26433	FHB	2020-10-09	16.25	16.38	16.05	16.11	935694	{}	\N	2020-10-12 14:15:24.884246
26434	FHB	2020-10-12	16.03	16.24	15.97	16.17	52299	{}	\N	2020-10-12 14:15:24.888442
26435	AMCX	2020-09-28	24.52	24.92	24.44	24.72	829305	{}	\N	2020-10-12 14:15:26.432126
26436	AMCX	2020-09-29	24.67	24.96	24.02	24.6	1007703	{}	\N	2020-10-12 14:15:26.441733
26437	AMCX	2020-09-30	24.76	25.03	24.6	24.71	766002	{}	\N	2020-10-12 14:15:26.448751
26438	AMCX	2020-10-01	24.63	25.04	24.02	24.26	1590457	{}	\N	2020-10-12 14:15:26.455016
26439	AMCX	2020-10-02	23.88	24.28	23.68	24.05	1940346	{}	\N	2020-10-12 14:15:26.460878
26440	AMCX	2020-10-05	24.31	24.63	23.37	23.69	1231741	{}	\N	2020-10-12 14:15:26.466043
26441	AMCX	2020-10-06	23.85	24.2	23.61	23.63	1256951	{}	\N	2020-10-12 14:15:26.470603
26442	AMCX	2020-10-07	23.89	24.25	23.83	23.89	1056197	{}	\N	2020-10-12 14:15:26.47514
26443	AMCX	2020-10-08	24.11	24.36	23.85	24	2279033	{}	\N	2020-10-12 14:15:26.479748
26444	AMCX	2020-10-09	24.21	24.25	23.7101	23.75	2172654	{}	\N	2020-10-12 14:15:26.484452
26445	AMCX	2020-10-12	23.79	24.38	23.79	24.275	130817	{}	\N	2020-10-12 14:15:26.489113
26446	AMJ	2020-09-28	10.71	10.94	10.57	10.85	1097326	{}	\N	2020-10-12 14:15:27.444464
26447	AMJ	2020-09-29	10.95	11.09	10.72	11.01	665714	{}	\N	2020-10-12 14:15:27.455996
26448	AMJ	2020-09-30	11.02	11.21	10.7	10.78	1017391	{}	\N	2020-10-12 14:15:27.462978
26449	AMJ	2020-10-01	10.86	10.87	10.53	10.68	825810	{}	\N	2020-10-12 14:15:27.468766
26450	AMJ	2020-10-02	10.48	10.91	10.45	10.85	1362798	{}	\N	2020-10-12 14:15:27.474321
26451	AMJ	2020-10-05	10.96	11.31	10.9299	11.3	1054271	{}	\N	2020-10-12 14:15:27.479436
26452	AMJ	2020-10-06	11.34	11.55	10.925	11.01	1381703	{}	\N	2020-10-12 14:15:27.483586
26453	AMJ	2020-10-07	11.11	11.17	10.975	11.07	771181	{}	\N	2020-10-12 14:15:27.48781
26454	AMJ	2020-10-08	11.17	11.78	11.12	11.77	781160	{}	\N	2020-10-12 14:15:27.49236
26455	AMJ	2020-10-09	11.84	11.95	11.62	11.65	845836	{}	\N	2020-10-12 14:15:27.496513
26456	AMJ	2020-10-12	11.73	11.765	11.625	11.65	101955	{}	\N	2020-10-12 14:15:27.5007
26457	AMRB	2020-09-28	9.3	9.31	9.01	9.1	67224	{}	\N	2020-10-12 14:15:28.517101
26458	AMRB	2020-09-29	9.3	10.16	9.3	9.96	1595	{}	\N	2020-10-12 14:15:28.525821
26459	AMRB	2020-09-30	10.44	10.64	9.96	9.96	1896	{}	\N	2020-10-12 14:15:28.53218
26460	AMRB	2020-10-02	9.7	9.7	9.7	9.7	498	{}	\N	2020-10-12 14:15:28.537974
26461	AMRB	2020-10-05	10.47	10.47	10.03	10.03	1273	{}	\N	2020-10-12 14:15:28.542296
26462	AMRB	2020-10-06	10.01	10.11	10.01	10.11	1145	{}	\N	2020-10-12 14:15:28.546855
26463	AMRB	2020-10-07	10.5	10.6	10.46	10.5	3323	{}	\N	2020-10-12 14:15:28.551576
26464	AMRB	2020-10-08	10.5265	10.6805	10.5265	10.6	782	{}	\N	2020-10-12 14:15:28.555604
26465	AMRB	2020-10-09	10.6396	10.6396	10.6396	10.6396	651	{}	\N	2020-10-12 14:15:28.559779
26466	AMRB	2020-10-12	10.725	10.725	10.725	10.725	101	{}	\N	2020-10-12 14:15:28.564095
26467	AMSWA	2020-09-28	14.15	14.41	14.02	14.34	108384	{}	\N	2020-10-12 14:15:29.767417
26468	AMSWA	2020-09-29	14.33	14.365	14.05	14.32	114234	{}	\N	2020-10-12 14:15:29.777708
26469	AMSWA	2020-09-30	14.26	14.38	14	14.04	79933	{}	\N	2020-10-12 14:15:29.784763
26470	AMSWA	2020-10-01	14.135	14.38	14.07	14.36	133796	{}	\N	2020-10-12 14:15:29.790698
26471	AMSWA	2020-10-02	14.06	14.405	13.65	14.32	78444	{}	\N	2020-10-12 14:15:29.796698
26472	AMSWA	2020-10-05	14.37	14.65	14.37	14.49	88636	{}	\N	2020-10-12 14:15:29.80286
26473	AMSWA	2020-10-06	14.61	14.92	14.45	14.6	80718	{}	\N	2020-10-12 14:15:29.807814
26474	AMSWA	2020-10-07	14.65	14.85	14.55	14.74	115070	{}	\N	2020-10-12 14:15:29.812502
26475	AMSWA	2020-10-08	14.9	14.99	14.76	14.9	59096	{}	\N	2020-10-12 14:15:29.816791
26476	AMSWA	2020-10-09	14.98	15.51	14.98	15.49	90394	{}	\N	2020-10-12 14:15:29.821154
26477	AMSWA	2020-10-12	15.6	15.73	15.47	15.52	7649	{}	\N	2020-10-12 14:15:29.825348
26478	AMZN	2020-09-28	3148.85	3175.04	3117.1684	3174.05	4081481	{}	\N	2020-10-12 14:15:31.068317
26479	AMZN	2020-09-29	3175.39	3188.26	3132.54	3144.88	3390304	{}	\N	2020-10-12 14:15:31.07929
26480	AMZN	2020-09-30	3141.14	3212.88	3133.99	3148.73	4622061	{}	\N	2020-10-12 14:15:31.087167
26481	AMZN	2020-10-01	3208	3224	3172	3221.26	4828038	{}	\N	2020-10-12 14:15:31.092937
26482	AMZN	2020-10-02	3153.63	3195.7965	3123	3125	5363653	{}	\N	2020-10-12 14:15:31.098056
26483	AMZN	2020-10-05	3145.84	3202.53	3140.85	3199.2	3743523	{}	\N	2020-10-12 14:15:31.103594
26484	AMZN	2020-10-06	3165	3182	3090	3099.96	4758946	{}	\N	2020-10-12 14:15:31.108412
26485	AMZN	2020-10-07	3135	3200	3132.385	3195.69	4280689	{}	\N	2020-10-12 14:15:31.113084
26486	AMZN	2020-10-08	3225	3233.2944	3174.99	3190.55	3133653	{}	\N	2020-10-12 14:15:31.117019
26487	AMZN	2020-10-09	3210	3288.99	3197.83	3286.65	4794403	{}	\N	2020-10-12 14:15:31.121348
26488	AMZN	2020-10-12	3349.94	3395	3339.55	3392.67	1489597	{}	\N	2020-10-12 14:15:31.125334
26489	ANGI	2020-09-28	10.48	10.81	10.273	10.55	2917022	{}	\N	2020-10-12 14:15:32.529714
26490	ANGI	2020-09-29	10.53	10.97	10.53	10.84	2921875	{}	\N	2020-10-12 14:15:32.540045
26491	ANGI	2020-09-30	10.9	11.46	10.78	11.095	6385491	{}	\N	2020-10-12 14:15:32.547769
26492	ANGI	2020-10-01	11.3	12.4	11.24	12.29	5698296	{}	\N	2020-10-12 14:15:32.553945
26493	ANGI	2020-10-02	11.72	12.18	11.71	12.02	2775661	{}	\N	2020-10-12 14:15:32.559116
26494	ANGI	2020-10-05	12.12	12.89	12.05	12.75	2363621	{}	\N	2020-10-12 14:15:32.564386
26495	ANGI	2020-10-06	12.7	12.795	12.225	12.28	3616308	{}	\N	2020-10-12 14:15:32.568906
26496	ANGI	2020-10-07	12.53	12.785	12.36	12.74	2873345	{}	\N	2020-10-12 14:15:32.574029
26497	ANGI	2020-10-08	12.34	12.35	11.02	11.04	15055423	{}	\N	2020-10-12 14:15:32.578647
26498	ANGI	2020-10-09	11.45	11.47	10.9	11.31	4121404	{}	\N	2020-10-12 14:15:32.583096
26499	ANGI	2020-10-12	11.45	11.46	11.16	11.22	311789	{}	\N	2020-10-12 14:15:32.588279
26500	APAM	2020-09-28	37.89	38.69	37.83	38.6	434930	{}	\N	2020-10-12 14:15:33.752953
26501	APAM	2020-09-29	38.6	38.99	38.38	38.7	641844	{}	\N	2020-10-12 14:15:33.763049
26502	APAM	2020-09-30	38.92	39.7	38.72	38.99	763422	{}	\N	2020-10-12 14:15:33.769783
26503	APAM	2020-10-01	39.24	39.35	37.94	38.3	842674	{}	\N	2020-10-12 14:15:33.775488
26504	APAM	2020-10-02	37.66	39.15	37.5	38.65	783099	{}	\N	2020-10-12 14:15:33.779939
26505	APAM	2020-10-05	38.99	39.4615	38.89	39.21	548701	{}	\N	2020-10-12 14:15:33.784316
26506	APAM	2020-10-06	39.79	40.3157	39.11	39.16	634211	{}	\N	2020-10-12 14:15:33.789075
26507	APAM	2020-10-07	39.8	40.8	39.5401	39.95	1043207	{}	\N	2020-10-12 14:15:33.792795
26508	APAM	2020-10-08	40.98	42.92	40.51	42.66	1550316	{}	\N	2020-10-12 14:15:33.796811
26509	APAM	2020-10-09	43.06	44.2	42.77	44.15	1197258	{}	\N	2020-10-12 14:15:33.800648
26510	APAM	2020-10-12	44.1	44.99	43.02	44.83	162544	{}	\N	2020-10-12 14:15:33.804204
26511	APLE	2020-09-28	9.72	9.99	9.68	9.89	1035546	{}	\N	2020-10-12 14:15:34.73513
26512	APLE	2020-09-29	9.88	9.88	9.61	9.81	3016754	{}	\N	2020-10-12 14:15:34.743096
26513	APLE	2020-09-30	9.87	10.15	9.445	9.61	8837203	{}	\N	2020-10-12 14:15:34.749748
26514	APLE	2020-10-01	9.66	10.21	9.66	10.16	4200214	{}	\N	2020-10-12 14:15:34.756894
26515	APLE	2020-10-02	9.75	10.37	9.72	10.35	2584836	{}	\N	2020-10-12 14:15:34.762288
26516	APLE	2020-10-05	10.65	10.68	10.28	10.43	2798599	{}	\N	2020-10-12 14:15:34.766406
26517	APLE	2020-10-06	10.51	10.705	10.375	10.42	3260097	{}	\N	2020-10-12 14:15:34.770802
26518	APLE	2020-10-07	10.5	10.83	10.4411	10.77	3854344	{}	\N	2020-10-12 14:15:34.77518
26519	APLE	2020-10-08	10.88	10.99	10.725	10.98	3041789	{}	\N	2020-10-12 14:15:34.778947
26520	APLE	2020-10-09	11.02	11.06	10.74	10.79	2395840	{}	\N	2020-10-12 14:15:34.782569
26521	APLE	2020-10-12	10.75	10.81	10.6185	10.7	288425	{}	\N	2020-10-12 14:15:34.786074
26522	APRN	2020-09-28	7.02	7.45	6.92	7.4	882526	{}	\N	2020-10-12 14:15:36.100702
26523	APRN	2020-09-29	7.41	7.45	7.05	7.42	1165864	{}	\N	2020-10-12 14:15:36.112393
26524	APRN	2020-09-30	7.37	7.5	7.02	7.16	962023	{}	\N	2020-10-12 14:15:36.119851
26525	APRN	2020-10-01	7.19	7.34	6.72	6.89	1654372	{}	\N	2020-10-12 14:15:36.125713
26526	APRN	2020-10-02	6.77	7.2	6.73	6.77	974758	{}	\N	2020-10-12 14:15:36.130908
26527	APRN	2020-10-05	6.84	7.06	6.77	6.88	780971	{}	\N	2020-10-12 14:15:36.135974
26528	APRN	2020-10-06	6.88	7.14	6.8	6.85	623019	{}	\N	2020-10-12 14:15:36.140158
26529	APRN	2020-10-07	6.85	7.0199	6.67	6.73	1371460	{}	\N	2020-10-12 14:15:36.144239
26530	APRN	2020-10-08	6.8	6.94	6.64	6.85	1034984	{}	\N	2020-10-12 14:15:36.14829
26531	APRN	2020-10-09	6.85	7.18	6.79	6.96	1032742	{}	\N	2020-10-12 14:15:36.152366
26532	APRN	2020-10-12	7	7.21	6.9	7.13	207129	{}	\N	2020-10-12 14:15:36.156524
26533	ARCH	2020-09-28	47.2717	47.64	46.35	46.65	343510	{}	\N	2020-10-12 14:15:37.865741
26534	ARCH	2020-09-29	46.62	46.835	42.7	43.12	1408845	{}	\N	2020-10-12 14:15:37.875904
26535	ARCH	2020-09-30	42.68	44.09	42.41	42.48	500001	{}	\N	2020-10-12 14:15:37.882942
26536	ARCH	2020-10-01	42.44	42.94	41.04	42.56	478221	{}	\N	2020-10-12 14:15:37.888964
26537	ARCH	2020-10-02	41.32	44.99	41.32	44.79	890191	{}	\N	2020-10-12 14:15:37.894478
26538	ARCH	2020-10-05	45.63	47.52	45.63	46.23	602085	{}	\N	2020-10-12 14:15:37.899511
26539	ARCH	2020-10-06	47.51	47.53	44.12	44.26	440122	{}	\N	2020-10-12 14:15:37.90403
26540	ARCH	2020-10-07	44.63	44.63	41.59	42	1068142	{}	\N	2020-10-12 14:15:37.908422
26541	ARCH	2020-10-08	42.26	43.31	41.25	41.89	431644	{}	\N	2020-10-12 14:15:37.913132
26542	ARCH	2020-10-09	42.21	42.21	39.1	40.27	730433	{}	\N	2020-10-12 14:15:37.917471
26543	ARCH	2020-10-12	40.16	40.25	39.8101	40.005	12815	{}	\N	2020-10-12 14:15:37.922227
27558	ASYS	2020-10-08	5.68	5.75	5.6	5.7	31300	{}	\N	2020-10-12 14:17:46.695652
27559	ASYS	2020-10-09	5.65	5.94	5.54	5.805	78638	{}	\N	2020-10-12 14:17:46.699834
27560	ASYS	2020-10-12	5.94	6	5.94	5.9795	11763	{}	\N	2020-10-12 14:17:46.703699
27561	ATGE	2020-09-28	24.94	25.53	24.7976	25.23	831591	{}	\N	2020-10-12 14:17:47.643921
27562	ATGE	2020-09-29	25.14	25.14	24.015	24.81	592791	{}	\N	2020-10-12 14:17:47.653313
27563	ATGE	2020-09-30	24.82	25.305	24.325	24.54	689345	{}	\N	2020-10-12 14:17:47.660323
27564	ATGE	2020-10-01	24.43	24.77	24.05	24.57	734089	{}	\N	2020-10-12 14:17:47.666957
27565	ATGE	2020-10-02	24.2	25.04	23.75	24.58	730321	{}	\N	2020-10-12 14:17:47.67265
27566	ATGE	2020-10-05	24.76	25.32	24.76	25.29	584910	{}	\N	2020-10-12 14:17:47.677479
27567	ATGE	2020-10-06	25.55	26.88	25.5	26.09	725553	{}	\N	2020-10-12 14:17:47.681889
27568	ATGE	2020-10-07	26.29	26.74	26.13	26.19	869122	{}	\N	2020-10-12 14:17:47.686368
27569	ATGE	2020-10-08	26.84	27.21	26.37	26.92	1020987	{}	\N	2020-10-12 14:17:47.690703
27570	ATGE	2020-10-09	27	27.09	26.51	26.63	683371	{}	\N	2020-10-12 14:17:47.696507
27571	ATGE	2020-10-12	26.6	26.655	26.38	26.38	23378	{}	\N	2020-10-12 14:17:47.704789
27572	ATNI	2020-09-28	51.99	52.03	49.75	50.79	70571	{}	\N	2020-10-12 14:17:49.152574
27573	ATNI	2020-09-29	50.55	52.11	50.195	51.5	72974	{}	\N	2020-10-12 14:17:49.163281
27574	ATNI	2020-09-30	51.85	51.87	49.58	50.14	55479	{}	\N	2020-10-12 14:17:49.170984
27575	ATNI	2020-10-01	50.03	51.03	48.67	49.34	55089	{}	\N	2020-10-12 14:17:49.176257
27576	ATNI	2020-10-02	48.83	49.21	48.39	48.7	37171	{}	\N	2020-10-12 14:17:49.180794
27577	ATNI	2020-10-05	49.04	50.85	49.04	50.12	37841	{}	\N	2020-10-12 14:17:49.185154
27578	ATNI	2020-10-06	50.39	51.17	49.45	49.88	50509	{}	\N	2020-10-12 14:17:49.189056
27579	ATNI	2020-10-07	49.98	49.98	48.94	49.8	49471	{}	\N	2020-10-12 14:17:49.192744
27580	ATNI	2020-10-08	50.11	50.78	49.3	50.38	29691	{}	\N	2020-10-12 14:17:49.196533
27581	ATNI	2020-10-09	50.89	51.01	49.78	50.37	30420	{}	\N	2020-10-12 14:17:49.200285
27582	ATNI	2020-10-12	50.4	50.95	50	50.66	5998	{}	\N	2020-10-12 14:17:49.204047
27583	ATRO	2020-09-28	8	8.29	7.96	7.99	118705	{}	\N	2020-10-12 14:17:50.696898
27584	ATRO	2020-09-29	7.93	7.93	7.67	7.7	133141	{}	\N	2020-10-12 14:17:50.708427
27585	ATRO	2020-09-30	7.68	7.94	7.62	7.72	275317	{}	\N	2020-10-12 14:17:50.716891
27586	ATRO	2020-10-01	7.74	8.01	7.74	7.91	164930	{}	\N	2020-10-12 14:17:50.722686
27587	ATRO	2020-10-02	7.65	8.1	7.65	8.05	111768	{}	\N	2020-10-12 14:17:50.727286
27588	ATRO	2020-10-05	8.12	8.265	8.09	8.2	123937	{}	\N	2020-10-12 14:17:50.732332
27589	ATRO	2020-10-06	8.34	8.45	7.72	7.74	146182	{}	\N	2020-10-12 14:17:50.736878
27590	ATRO	2020-10-07	8.16	8.16	7.81	8.07	101633	{}	\N	2020-10-12 14:17:50.740709
27591	ATRO	2020-10-08	8.14	8.2113	7.79	8.09	256335	{}	\N	2020-10-12 14:17:50.744704
27592	ATRO	2020-10-09	8.29	8.4471	8.07	8.24	126581	{}	\N	2020-10-12 14:17:50.748894
27593	ATRO	2020-10-12	8.29	8.43	8.1392	8.2703	7426	{}	\N	2020-10-12 14:17:50.753109
27594	ATVI	2020-09-28	81.63	82.02	80.505	81.95	3592176	{}	\N	2020-10-12 14:17:52.366738
27595	ATVI	2020-09-29	81.83	82	80.54	80.78	4663160	{}	\N	2020-10-12 14:17:52.375206
27596	ATVI	2020-09-30	80.61	82.015	80.4	80.95	6952802	{}	\N	2020-10-12 14:17:52.381688
27597	ATVI	2020-10-01	81.72	83.54	81.38	82.68	5728124	{}	\N	2020-10-12 14:17:52.387427
27598	ATVI	2020-10-02	81.99	81.99	78.13	78.3	8241293	{}	\N	2020-10-12 14:17:52.392943
27599	ATVI	2020-10-05	79.36	80.17	78.4	79.99	5445152	{}	\N	2020-10-12 14:17:52.397851
27600	ATVI	2020-10-06	79.66	80.644	78.12	78.3	4368285	{}	\N	2020-10-12 14:17:52.402432
27601	ATVI	2020-10-07	78.68	78.925	77.66	78.47	5345904	{}	\N	2020-10-12 14:17:52.406951
27602	ATVI	2020-10-08	78.9618	78.9618	77.35	77.8	5302563	{}	\N	2020-10-12 14:17:52.411228
27603	ATVI	2020-10-09	78.19	78.75	77.72	78.21	5190027	{}	\N	2020-10-12 14:17:52.415753
27604	ATVI	2020-10-12	79.28	79.58	78.95	79.44	756443	{}	\N	2020-10-12 14:17:52.420057
27605	AVAL	2020-09-28	4.657	4.78	4.6498	4.67	90909	{}	\N	2020-10-12 14:17:54.399656
27606	AVAL	2020-09-29	4.65	4.67	4.61	4.61	175896	{}	\N	2020-10-12 14:17:54.413249
27607	AVAL	2020-09-30	4.52	4.65	4.52	4.61	926043	{}	\N	2020-10-12 14:17:54.420766
27608	AVAL	2020-10-01	4.66	4.66	4.61	4.63	201054	{}	\N	2020-10-12 14:17:54.426557
27609	AVAL	2020-10-02	4.61	4.64	4.6	4.6	137258	{}	\N	2020-10-12 14:17:54.432705
27610	AVAL	2020-10-05	4.6	4.64	4.5889	4.61	1429593	{}	\N	2020-10-12 14:17:54.437895
27611	AVAL	2020-10-06	4.67	4.67	4.58	4.58	477719	{}	\N	2020-10-12 14:17:54.442225
27612	AVAL	2020-10-07	4.62	4.64	4.6	4.61	133455	{}	\N	2020-10-12 14:17:54.446922
27613	AVAL	2020-10-08	4.65	4.67	4.62	4.64	243574	{}	\N	2020-10-12 14:17:54.451776
27614	AVAL	2020-10-09	4.68	4.69	4.61	4.64	835860	{}	\N	2020-10-12 14:17:54.45608
27615	AVAL	2020-10-12	4.65	4.6599	4.64	4.64	4405	{}	\N	2020-10-12 14:17:54.460266
27616	KALU	2020-09-28	51.6	54.16	51.3685	53.47	127079	{}	\N	2020-10-12 14:17:58.420498
27617	KALU	2020-09-29	53.04	53.4	51.71	53	148799	{}	\N	2020-10-12 14:17:58.429744
27618	KALU	2020-09-30	53.34	55.08	53.335	53.59	180551	{}	\N	2020-10-12 14:17:58.436203
27619	KALU	2020-10-01	53.63	54.18	52.68	54.18	96826	{}	\N	2020-10-12 14:17:58.44171
27620	KALU	2020-10-02	52.87	55.9	52.87	55.07	120080	{}	\N	2020-10-12 14:17:58.445961
27621	KALU	2020-10-05	55.65	56.84	55.65	56.44	117516	{}	\N	2020-10-12 14:17:58.450851
27622	KALU	2020-10-06	57.57	57.57	54.72	54.81	90337	{}	\N	2020-10-12 14:17:58.455172
27623	KALU	2020-10-07	56.04	58.17	55.59	57.96	163029	{}	\N	2020-10-12 14:17:58.458883
27624	KALU	2020-10-08	58.83	60.12	58.35	59.94	113782	{}	\N	2020-10-12 14:17:58.46279
27625	KALU	2020-10-09	60.72	62.97	60.72	62.77	175503	{}	\N	2020-10-12 14:17:58.466701
27626	KALU	2020-10-12	62.35	62.95	61.855	62.9	6210	{}	\N	2020-10-12 14:17:58.470456
27627	ARCT	2020-09-28	40.47	41.15	38.54	41.05	421720	{}	\N	2020-10-12 14:18:01.456213
27628	ARCT	2020-09-29	41.06	42.865	40.28	42.73	569933	{}	\N	2020-10-12 14:18:01.465317
27629	ARCT	2020-09-30	42.59	44.1499	41.85	42.97	280732	{}	\N	2020-10-12 14:18:01.471928
27630	ARCT	2020-10-01	43.99	44.04	41.205	42.73	471222	{}	\N	2020-10-12 14:18:01.478009
27631	ARCT	2020-10-02	41.25	44.08	39.52	42.79	301531	{}	\N	2020-10-12 14:18:01.483252
26544	ARGX	2020-09-28	250.99	255.75	250.99	255.37	92959	{}	\N	2020-10-12 14:15:40.918543
26545	ARGX	2020-09-29	256.16	261.96	256.14	260.2	113704	{}	\N	2020-10-12 14:15:40.934875
26546	ARGX	2020-09-30	262.84	266.8	262.01	262.52	134724	{}	\N	2020-10-12 14:15:40.940004
26547	ARGX	2020-10-01	268.22	268.66	265.5	265.8	138428	{}	\N	2020-10-12 14:15:40.944389
26548	ARGX	2020-10-02	266.96	268.51	262.18	262.3	150235	{}	\N	2020-10-12 14:15:40.948572
26549	ARGX	2020-10-05	268.28	272.88	267.7	270.22	283758	{}	\N	2020-10-12 14:15:40.952831
26550	ARGX	2020-10-06	269.09	272.56	266.3582	268.21	169396	{}	\N	2020-10-12 14:15:40.956928
26551	ARGX	2020-10-07	270.025	275.5791	268.54	274.09	132449	{}	\N	2020-10-12 14:15:40.960991
26552	ARGX	2020-10-08	275.88	275.88	270.36	272	64625	{}	\N	2020-10-12 14:15:40.965107
26553	ARGX	2020-10-09	273.75	277.38	273.6	276.27	116691	{}	\N	2020-10-12 14:15:40.969322
26554	ARGX	2020-10-12	283.06	284	281.49	283.64	26480	{}	\N	2020-10-12 14:15:40.973432
26555	ARMK	2020-09-28	27.16	27.6	26.72	27.14	1832724	{}	\N	2020-10-12 14:15:41.942972
26556	ARMK	2020-09-29	27.06	27.1	26.19	26.42	4761255	{}	\N	2020-10-12 14:15:41.953113
26557	ARMK	2020-09-30	26.62	27.45	26.26	26.45	5069567	{}	\N	2020-10-12 14:15:41.959604
26558	ARMK	2020-10-01	26.69	27.2	26.33	27.06	3327900	{}	\N	2020-10-12 14:15:41.965567
26559	ARMK	2020-10-02	26.12	27.605	25.84	27.55	3158470	{}	\N	2020-10-12 14:15:41.970994
26560	ARMK	2020-10-05	28.08	28.19	27.35	27.81	3746066	{}	\N	2020-10-12 14:15:41.975467
26561	ARMK	2020-10-06	28.24	28.825	27.34	27.37	4778751	{}	\N	2020-10-12 14:15:41.979724
26562	ARMK	2020-10-07	27.83	28.23	27.64	27.93	2616788	{}	\N	2020-10-12 14:15:41.983815
26563	ARMK	2020-10-08	28.31	28.37	27.93	28.28	2938969	{}	\N	2020-10-12 14:15:41.987943
26564	ARMK	2020-10-09	28.4	28.91	28.3	28.49	1797923	{}	\N	2020-10-12 14:15:41.991953
26565	ARMK	2020-10-12	28.41	28.77	28.18	28.71	137785	{}	\N	2020-10-12 14:15:41.996322
26566	ARTW	2020-09-28	2.25	2.28	2.24	2.25	6890	{}	\N	2020-10-12 14:15:43.270309
26567	ARTW	2020-09-29	2.25	2.36	2.22	2.349	35428	{}	\N	2020-10-12 14:15:43.281015
26568	ARTW	2020-09-30	2.3439	2.3996	2.3	2.3001	9365	{}	\N	2020-10-12 14:15:43.287524
26569	ARTW	2020-10-01	2.3	2.345	2.25	2.2965	9331	{}	\N	2020-10-12 14:15:43.293393
26570	ARTW	2020-10-02	2.25	2.4	2.25	2.35	14017	{}	\N	2020-10-12 14:15:43.299722
26571	ARTW	2020-10-05	2.375	2.4	2.31	2.31	6050	{}	\N	2020-10-12 14:15:43.304247
26572	ARTW	2020-10-06	2.31	2.55	2.31	2.36	96095	{}	\N	2020-10-12 14:15:43.30901
26573	ARTW	2020-10-07	2.38	2.4044	2.34	2.38	8738	{}	\N	2020-10-12 14:15:43.313133
26574	ARTW	2020-10-08	2.41	2.9799	2.31	2.39	572651	{}	\N	2020-10-12 14:15:43.317268
26575	ARTW	2020-10-09	2.35	2.419	2.3	2.34	30595	{}	\N	2020-10-12 14:15:43.325001
26576	ARTW	2020-10-12	2.34	2.35	2.302	2.35	704	{}	\N	2020-10-12 14:15:43.331355
26577	AMRN	2020-09-28	3.9977	4.0889	3.88	4	7776147	{}	\N	2020-10-12 14:15:44.766004
26578	AMRN	2020-09-29	4	4.09	3.89	4.04	4969822	{}	\N	2020-10-12 14:15:44.778993
26579	AMRN	2020-09-30	3.98	4.275	3.92	4.2	9038499	{}	\N	2020-10-12 14:15:44.784699
26580	AMRN	2020-10-01	4.22	4.26	4.1	4.26	4821116	{}	\N	2020-10-12 14:15:44.789632
26581	AMRN	2020-10-02	4.15	4.475	4.1458	4.47	7901421	{}	\N	2020-10-12 14:15:44.794471
26582	AMRN	2020-10-05	4.85	5.15	4.74	5.06	18165540	{}	\N	2020-10-12 14:15:44.798745
26583	AMRN	2020-10-06	5.09	5.18	4.94	4.98	10077299	{}	\N	2020-10-12 14:15:44.802607
26584	AMRN	2020-10-07	5.045	5.57	5	5.31	10214546	{}	\N	2020-10-12 14:15:44.806202
26585	AMRN	2020-10-08	5.4	5.4794	5.225	5.28	5246123	{}	\N	2020-10-12 14:15:44.809856
26586	AMRN	2020-10-09	5.33	5.39	5.23	5.25	3703853	{}	\N	2020-10-12 14:15:44.814073
26587	AMRN	2020-10-12	5.28	5.345	5.15	5.1801	1068709	{}	\N	2020-10-12 14:15:44.818092
26588	AMU	2020-09-28	6.74	6.85	6.69	6.82	6168	{}	\N	2020-10-12 14:15:46.785201
26589	AMU	2020-09-29	6.82	6.9383	6.8	6.9383	1175	{}	\N	2020-10-12 14:15:46.791603
26590	AMU	2020-09-30	7	7	6.8	6.81	1652	{}	\N	2020-10-12 14:15:46.796659
26591	AMU	2020-10-01	6.83	6.83	6.68	6.75	2879	{}	\N	2020-10-12 14:15:46.800978
26592	AMU	2020-10-02	6.82	6.83	6.8	6.81	10862	{}	\N	2020-10-12 14:15:46.805327
26593	AMU	2020-10-05	6.82	7.13	6.82	7.13	2536	{}	\N	2020-10-12 14:15:46.809292
26594	AMU	2020-10-06	7.13	7.13	6.95	6.97	35348	{}	\N	2020-10-12 14:15:46.812904
26595	AMU	2020-10-07	6.95	6.95	6.95	6.95	194	{}	\N	2020-10-12 14:15:46.816462
26596	AMU	2020-10-08	7.025	7.27	7.025	7.27	2716	{}	\N	2020-10-12 14:15:46.820072
26597	AMU	2020-10-09	7.43	7.43	7.33	7.42	4381	{}	\N	2020-10-12 14:15:46.823699
26598	ANDE	2020-09-28	19.79	20.39	19.664	20.01	441578	{}	\N	2020-10-12 14:15:47.829461
26599	ANDE	2020-09-29	20.04	20.2	19.48	19.64	128247	{}	\N	2020-10-12 14:15:47.840631
26600	ANDE	2020-09-30	19.47	19.85	19.07	19.17	201950	{}	\N	2020-10-12 14:15:47.847904
26601	ANDE	2020-10-01	19.23	19.64	18.8992	19.09	140367	{}	\N	2020-10-12 14:15:47.853655
26602	ANDE	2020-10-02	18.56	19.52	18.55	19.44	171213	{}	\N	2020-10-12 14:15:47.858723
26603	ANDE	2020-10-05	19.63	19.93	19.55	19.79	147035	{}	\N	2020-10-12 14:15:47.863687
26604	ANDE	2020-10-06	20	20.23	19.625	19.64	246649	{}	\N	2020-10-12 14:15:47.867858
26605	ANDE	2020-10-07	19.4	20.04	19.4	19.72	269883	{}	\N	2020-10-12 14:15:47.871873
26606	ANDE	2020-10-08	19.95	20.16	19.42	19.75	141665	{}	\N	2020-10-12 14:15:47.876109
26607	ANDE	2020-10-09	19.85	20.12	19.28	19.61	145330	{}	\N	2020-10-12 14:15:47.880095
26608	ANDE	2020-10-12	19.66	19.965	19.55	19.85	20283	{}	\N	2020-10-12 14:15:47.883982
26609	ANSS	2020-09-28	318.36	320.24	315.54	318.8	293437	{}	\N	2020-10-12 14:15:48.842719
26610	ANSS	2020-09-29	318.22	321.87	317.36	319.51	454272	{}	\N	2020-10-12 14:15:48.850202
26611	ANSS	2020-09-30	318.51	328.79	317.35	327.23	630021	{}	\N	2020-10-12 14:15:48.855956
26612	ANSS	2020-10-01	332.07	336.35	329.285	334.8	459139	{}	\N	2020-10-12 14:15:48.861273
26613	ANSS	2020-10-02	330.19	331.46	318.94	321.58	365388	{}	\N	2020-10-12 14:15:48.865954
26614	ANSS	2020-10-05	322.46	326.195	321.58	323.87	375373	{}	\N	2020-10-12 14:15:48.870076
26615	ANSS	2020-10-06	322.7	324.86	314.9	316.47	667533	{}	\N	2020-10-12 14:15:48.874653
26616	ANSS	2020-10-07	319.54	330.96	319.54	330.4	382347	{}	\N	2020-10-12 14:15:48.878911
26617	ANSS	2020-10-08	331.97	334.38	328.82	333.54	317070	{}	\N	2020-10-12 14:15:48.882511
26618	ANSS	2020-10-09	336	339.17	334.09	336.82	320258	{}	\N	2020-10-12 14:15:48.886282
26619	ANSS	2020-10-12	340.56	346.97	340.56	342.03	57515	{}	\N	2020-10-12 14:15:48.889803
26620	AP	2020-09-28	3.56	3.685	3.38	3.48	159526	{}	\N	2020-10-12 14:15:49.940959
26621	AP	2020-09-29	3.42	3.69	3.41	3.49	303623	{}	\N	2020-10-12 14:15:49.950546
26622	AP	2020-09-30	3.48	3.6437	3.29	3.29	251968	{}	\N	2020-10-12 14:15:49.95693
26623	AP	2020-10-01	3.286	3.5	3.27	3.36	109339	{}	\N	2020-10-12 14:15:49.962865
26624	AP	2020-10-02	3.45	3.75	3.44	3.63	276378	{}	\N	2020-10-12 14:15:49.968009
26625	AP	2020-10-05	3.69	3.95	3.61	3.7	174231	{}	\N	2020-10-12 14:15:49.973086
26626	AP	2020-10-06	3.68	3.76	3.49	3.52	121698	{}	\N	2020-10-12 14:15:49.977488
26627	AP	2020-10-07	3.55	3.95	3.53	3.84	503706	{}	\N	2020-10-12 14:15:49.981487
26628	AP	2020-10-08	3.8	3.8585	3.607	3.76	241954	{}	\N	2020-10-12 14:15:49.985459
26629	AP	2020-10-09	3.75	3.875	3.62	3.62	149426	{}	\N	2020-10-12 14:15:49.989695
26630	AP	2020-10-12	3.63	3.7497	3.62	3.7317	19956	{}	\N	2020-10-12 14:15:49.994201
26631	APEN	2020-09-28	1.41	1.5	1.38	1.38	22232	{}	\N	2020-10-12 14:15:51.035075
26632	APEN	2020-09-29	1.41	1.68	1.4	1.67	393264	{}	\N	2020-10-12 14:15:51.043835
26633	APEN	2020-09-30	1.71	1.72	1.5607	1.69	180083	{}	\N	2020-10-12 14:15:51.050842
26634	APEN	2020-10-01	1.74	1.74	1.65	1.74	21044	{}	\N	2020-10-12 14:15:51.056939
26635	APEN	2020-10-02	1.61	1.73	1.61	1.72	15949	{}	\N	2020-10-12 14:15:51.062266
26636	APEN	2020-10-05	1.74	1.74	1.62	1.7	49984	{}	\N	2020-10-12 14:15:51.067006
26637	APEN	2020-10-06	1.6803	1.74	1.67	1.72	25813	{}	\N	2020-10-12 14:15:51.071252
26638	APEN	2020-10-07	1.71	1.83	1.68	1.69	28777	{}	\N	2020-10-12 14:15:51.075634
26639	APEN	2020-10-08	1.78	1.79	1.65	1.74	116295	{}	\N	2020-10-12 14:15:51.079874
26640	APEN	2020-10-09	1.71	1.75	1.65	1.66	8098	{}	\N	2020-10-12 14:15:51.083985
26641	APEN	2020-10-12	1.68	1.71	1.68	1.7	4744	{}	\N	2020-10-12 14:15:51.088181
26642	APPS	2020-09-28	31.66	32.17	30.56	31.05	2449080	{}	\N	2020-10-12 14:15:52.108834
26643	APPS	2020-09-29	31.2	33.77	31.11	33.03	4527714	{}	\N	2020-10-12 14:15:52.118551
26644	APPS	2020-09-30	33.0532	34.17	32.0301	32.74	4958013	{}	\N	2020-10-12 14:15:52.1249
26645	APPS	2020-10-01	33.6162	35.35	33.19	35.2	4148573	{}	\N	2020-10-12 14:15:52.130498
26646	APPS	2020-10-02	33.52	36.1998	33.4	34.4	4287874	{}	\N	2020-10-12 14:15:52.135169
26647	APPS	2020-10-05	34.835	35.97	34.75	35.58	2682255	{}	\N	2020-10-12 14:15:52.139064
26648	APPS	2020-10-06	35.97	36.9299	35.14	35.57	3222466	{}	\N	2020-10-12 14:15:52.143282
26649	APPS	2020-10-07	36.16	36.75	34.83	36.53	3377488	{}	\N	2020-10-12 14:15:52.147292
26650	APPS	2020-10-08	36.84	37.314	35.22	35.81	2125367	{}	\N	2020-10-12 14:15:52.150997
26651	APPS	2020-10-09	35.24	37.2617	35.2	36.43	2106265	{}	\N	2020-10-12 14:15:52.154912
26652	APPS	2020-10-12	37.66	38.4	37.11	38.23	747495	{}	\N	2020-10-12 14:15:52.15877
26653	AQB	2020-09-28	4.7	4.81	4.435	4.76	1292924	{}	\N	2020-10-12 14:15:53.538481
26654	AQB	2020-09-29	4.7	4.7	4.44	4.55	759286	{}	\N	2020-10-12 14:15:53.548973
26655	AQB	2020-09-30	4.55	4.595	4.35	4.48	551493	{}	\N	2020-10-12 14:15:53.555474
26656	AQB	2020-10-01	4.45	4.45	4.04	4.14	1289309	{}	\N	2020-10-12 14:15:53.561726
26657	AQB	2020-10-02	4	4.43	3.96	4.25	758116	{}	\N	2020-10-12 14:15:53.568053
26658	AQB	2020-10-05	4.37	4.53	4.35	4.45	540733	{}	\N	2020-10-12 14:15:53.572852
26659	AQB	2020-10-06	4.44	4.52	4.2	4.28	489357	{}	\N	2020-10-12 14:15:53.577074
26660	AQB	2020-10-07	4.3	4.51	4.22	4.44	717465	{}	\N	2020-10-12 14:15:53.580673
26661	AQB	2020-10-08	4.54	5.08	4.5	5	1639446	{}	\N	2020-10-12 14:15:53.584116
26662	AQB	2020-10-09	4.99	4.99	4.58	4.63	1184565	{}	\N	2020-10-12 14:15:53.587841
26663	AQB	2020-10-12	4.6448	4.68	4.39	4.51	267221	{}	\N	2020-10-12 14:15:53.591641
26664	ARCC	2020-09-28	13.82	14.2	13.76	14.11	1983523	{}	\N	2020-10-12 14:15:54.762674
26665	ARCC	2020-09-29	14.11	14.17	13.77	13.94	1572397	{}	\N	2020-10-12 14:15:54.773334
26666	ARCC	2020-09-30	14.22	14.22	13.92	13.95	2316104	{}	\N	2020-10-12 14:15:54.780934
26667	ARCC	2020-10-01	13.96	14.37	13.96	14.36	2632785	{}	\N	2020-10-12 14:15:54.787823
26668	ARCC	2020-10-02	14.05	14.41	14	14.34	1606002	{}	\N	2020-10-12 14:15:54.794354
26669	ARCC	2020-10-05	14.41	14.485	14.25	14.28	1110831	{}	\N	2020-10-12 14:15:54.800025
26670	ARCC	2020-10-06	14.35	14.39	14.08	14.11	1033180	{}	\N	2020-10-12 14:15:54.804307
26671	ARCC	2020-10-07	14.19	14.29	14.1	14.18	947091	{}	\N	2020-10-12 14:15:54.808851
26672	ARCC	2020-10-08	14.22	14.42	14.22	14.38	1140083	{}	\N	2020-10-12 14:15:54.813684
26673	ARCC	2020-10-09	14.5	14.6	14.32	14.36	1253528	{}	\N	2020-10-12 14:15:54.818187
26674	ARCC	2020-10-12	14.38	14.48	14.25	14.25	205771	{}	\N	2020-10-12 14:15:54.822568
26675	ARDX	2020-09-28	5.41	5.495	5.2936	5.34	307811	{}	\N	2020-10-12 14:15:55.826432
26676	ARDX	2020-09-29	5.36	5.395	5.2045	5.28	430232	{}	\N	2020-10-12 14:15:55.836565
26677	ARDX	2020-09-30	5.32	5.42	5.22	5.255	335464	{}	\N	2020-10-12 14:15:55.843749
26678	ARDX	2020-10-01	5.25	5.5	5.25	5.43	368804	{}	\N	2020-10-12 14:15:55.84984
26679	ARDX	2020-10-02	5.28	5.42	5.2	5.255	314881	{}	\N	2020-10-12 14:15:55.855149
26680	ARDX	2020-10-05	5.3	5.64	5.3	5.59	499146	{}	\N	2020-10-12 14:15:55.860437
26681	ARDX	2020-10-06	5.62	5.76	5.5536	5.565	333607	{}	\N	2020-10-12 14:15:55.86564
26682	ARDX	2020-10-07	5.6	5.78	5.58	5.74	263737	{}	\N	2020-10-12 14:15:55.870122
26683	ARDX	2020-10-08	5.83	5.88	5.65	5.79	235438	{}	\N	2020-10-12 14:15:55.874673
26684	ARDX	2020-10-09	5.79	5.82	5.52	5.79	183507	{}	\N	2020-10-12 14:15:55.878874
26685	ARDX	2020-10-12	5.755	5.86	5.7	5.79	43726	{}	\N	2020-10-12 14:15:55.883036
26686	ARKQ	2020-09-28	56.74	57.18	56.572	57.05	173298	{}	\N	2020-10-12 14:15:56.890514
26687	ARKQ	2020-09-29	57.25	57.52	56.79	57.29	257367	{}	\N	2020-10-12 14:15:56.897334
26688	ARKQ	2020-09-30	57.72	58.07	57.0618	57.39	196506	{}	\N	2020-10-12 14:15:56.90207
26689	ARKQ	2020-10-01	58.26	58.4124	57.7881	58.34	139895	{}	\N	2020-10-12 14:15:56.906672
26690	ARKQ	2020-10-02	56.88	58.0199	56.24	57.36	209977	{}	\N	2020-10-12 14:15:56.911659
26691	ARKQ	2020-10-05	58	58.76	58	58.756	163605	{}	\N	2020-10-12 14:15:56.915525
26692	ARKQ	2020-10-06	58.95	59.96	58.31	58.54	270934	{}	\N	2020-10-12 14:15:56.919104
26693	ARKQ	2020-10-07	59.4	60.2708	59.25	60.15	198717	{}	\N	2020-10-12 14:15:56.922812
26694	ARKQ	2020-10-08	61.28	61.28	60.3701	60.62	197335	{}	\N	2020-10-12 14:15:56.926925
26695	ARKQ	2020-10-09	61.65	62.54	61.5777	62.46	363630	{}	\N	2020-10-12 14:15:56.93062
26696	ARKQ	2020-10-12	63.47	63.6597	63.1	63.49	126200	{}	\N	2020-10-12 14:15:56.935157
26697	ARR	2020-09-28	9.45	9.775	9.45	9.67	857557	{}	\N	2020-10-12 14:15:58.410007
26698	ARR	2020-09-29	9.68	9.68	9.39	9.47	1334164	{}	\N	2020-10-12 14:15:58.420339
26699	ARR	2020-09-30	9.48	9.68	9.44	9.51	1333992	{}	\N	2020-10-12 14:15:58.427581
26700	ARR	2020-10-01	9.54	9.56	9.39	9.55	1120125	{}	\N	2020-10-12 14:15:58.433729
26701	ARR	2020-10-02	9.4	9.81	9.37	9.78	1998617	{}	\N	2020-10-12 14:15:58.438657
26702	ARR	2020-10-05	9.83	9.8727	9.725	9.83	1389458	{}	\N	2020-10-12 14:15:58.443737
26703	ARR	2020-10-06	9.88	10.04	9.72	9.75	1918937	{}	\N	2020-10-12 14:15:58.448253
26704	ARR	2020-10-07	9.79	9.86	9.665	9.82	1608165	{}	\N	2020-10-12 14:15:58.452455
26705	ARR	2020-10-08	9.87	10.08	9.8643	10	2445742	{}	\N	2020-10-12 14:15:58.45651
26706	ARR	2020-10-09	10.08	10.09	9.8819	9.9	1881805	{}	\N	2020-10-12 14:15:58.460839
26707	ARR	2020-10-12	9.93	10	9.9	9.96	195033	{}	\N	2020-10-12 14:15:58.464761
26708	ASB	2020-09-28	12.71	12.91	12.625	12.76	833281	{}	\N	2020-10-12 14:15:59.791962
26709	ASB	2020-09-29	12.63	12.66	12.36	12.51	1620716	{}	\N	2020-10-12 14:15:59.805816
26710	ASB	2020-09-30	12.51	12.81	12.44	12.62	2938591	{}	\N	2020-10-12 14:15:59.812888
26711	ASB	2020-10-01	12.64	12.81	12.4427	12.68	2167936	{}	\N	2020-10-12 14:15:59.817985
26712	ASB	2020-10-02	12.45	12.97	12.39	12.9	2907314	{}	\N	2020-10-12 14:15:59.822989
26713	ASB	2020-10-05	13.09	13.53	13	13.47	2669297	{}	\N	2020-10-12 14:15:59.828026
26714	ASB	2020-10-06	13.7	14.04	13.41	13.46	3074313	{}	\N	2020-10-12 14:15:59.832636
26715	ASB	2020-10-07	13.65	14.07	13.65	13.93	2766856	{}	\N	2020-10-12 14:15:59.83696
26716	ASB	2020-10-08	14.48	14.5	14.025	14.3	2045623	{}	\N	2020-10-12 14:15:59.841089
26717	ASB	2020-10-09	14.38	14.49	14	14.07	1707216	{}	\N	2020-10-12 14:15:59.845106
26718	ASB	2020-10-12	13.98	14.13	13.94	14.06	152222	{}	\N	2020-10-12 14:15:59.849929
26719	FEMB	2020-09-28	34.12	34.35	34	34.24	16362	{}	\N	2020-10-12 14:16:01.485008
26720	FEMB	2020-09-29	34.18	34.2738	34.0748	34.15	162336	{}	\N	2020-10-12 14:16:01.494316
26721	FEMB	2020-09-30	34.21	34.27	34	34.21	284755	{}	\N	2020-10-12 14:16:01.499963
26722	FEMB	2020-10-01	34.49	34.49	34.21	34.38	81478	{}	\N	2020-10-12 14:16:01.504975
26723	FEMB	2020-10-02	34.33	34.47	34.19	34.45	62239	{}	\N	2020-10-12 14:16:01.509517
26724	FEMB	2020-10-05	34.37	34.44	34.28	34.42	33018	{}	\N	2020-10-12 14:16:01.514033
26725	FEMB	2020-10-06	34.43	34.6	34.24	34.36	17762	{}	\N	2020-10-12 14:16:01.517834
26726	FEMB	2020-10-07	34.31	34.31	34.19	34.22	27142	{}	\N	2020-10-12 14:16:01.521397
26727	FEMB	2020-10-08	34.43	34.48	34.34	34.46	16803	{}	\N	2020-10-12 14:16:01.525048
26728	FEMB	2020-10-09	34.63	34.89	34.63	34.88	25341	{}	\N	2020-10-12 14:16:01.529222
26729	FEMB	2020-10-12	34.89	34.92	34.89	34.91	12805	{}	\N	2020-10-12 14:16:01.533375
26730	AMT	2020-09-28	243.07	244.42	238.41	240	2480184	{}	\N	2020-10-12 14:16:03.110951
26731	AMT	2020-09-29	241.21	242	239.12	240.12	5959413	{}	\N	2020-10-12 14:16:03.121888
26732	AMT	2020-09-30	241.93	243.58	239.69	241.73	6306300	{}	\N	2020-10-12 14:16:03.128787
26733	AMT	2020-10-01	242.19	245	239.96	242.15	4583307	{}	\N	2020-10-12 14:16:03.134764
26734	AMT	2020-10-02	240.07	243.91	238.99	243.3	3892702	{}	\N	2020-10-12 14:16:03.139495
26735	AMT	2020-10-05	243.25	243.64	237.89	243.33	4284452	{}	\N	2020-10-12 14:16:03.144559
26736	AMT	2020-10-06	243.91	244.41	240.16	241.01	3518668	{}	\N	2020-10-12 14:16:03.148587
26737	AMT	2020-10-07	241.19	243.51	241.19	241.77	3871868	{}	\N	2020-10-12 14:16:03.15293
26738	AMT	2020-10-08	243.76	246.5404	242.86	243.56	4989795	{}	\N	2020-10-12 14:16:03.157069
26739	AMT	2020-10-09	244.27	246.3936	243.15	244.68	2576625	{}	\N	2020-10-12 14:16:03.16096
26740	AMT	2020-10-12	245.25	246.275	244.33	246.23	327671	{}	\N	2020-10-12 14:16:03.164816
26741	ANAB	2020-09-28	14.68	14.87	14.13	14.17	112790	{}	\N	2020-10-12 14:16:04.441005
26742	ANAB	2020-09-29	14.16	14.8	13.96	14.71	208926	{}	\N	2020-10-12 14:16:04.449541
26743	ANAB	2020-09-30	14.71	15.29	14.66	14.75	189939	{}	\N	2020-10-12 14:16:04.455627
26744	ANAB	2020-10-01	14.55	15.98	14.55	15.94	282423	{}	\N	2020-10-12 14:16:04.460788
26745	ANAB	2020-10-02	15.72	16.08	15.29	15.61	187192	{}	\N	2020-10-12 14:16:04.465708
26746	ANAB	2020-10-05	15.77	16.19	15.72	16.06	171745	{}	\N	2020-10-12 14:16:04.470549
26747	ANAB	2020-10-06	16.1	16.52	15.91	15.96	126812	{}	\N	2020-10-12 14:16:04.47509
26748	ANAB	2020-10-07	16.02	16.53	15.77	16.32	121530	{}	\N	2020-10-12 14:16:04.479565
26749	ANAB	2020-10-08	16.43	16.84	16.41	16.69	162663	{}	\N	2020-10-12 14:16:04.483574
26750	ANAB	2020-10-09	16.71	16.96	16.49	16.84	118584	{}	\N	2020-10-12 14:16:04.487419
26751	ANAB	2020-10-12	16.6	16.99	16.19	16.99	9743	{}	\N	2020-10-12 14:16:04.492033
26752	ANGL	2020-09-28	29.5	29.61	29.46	29.57	394666	{}	\N	2020-10-12 14:16:05.59794
26753	ANGL	2020-09-29	29.57	29.59	29.48	29.58	773456	{}	\N	2020-10-12 14:16:05.608496
26754	ANGL	2020-09-30	29.6	29.77	29.6	29.74	625037	{}	\N	2020-10-12 14:16:05.61482
26755	ANGL	2020-10-01	29.72	29.74	29.63	29.69	2044780	{}	\N	2020-10-12 14:16:05.620106
26756	ANGL	2020-10-02	29.6	29.72	29.53	29.66	2890265	{}	\N	2020-10-12 14:16:05.625699
26757	ANGL	2020-10-05	29.69	29.88	29.69	29.87	2425478	{}	\N	2020-10-12 14:16:05.630167
26758	ANGL	2020-10-06	29.91	30.04	29.83	29.9	3289217	{}	\N	2020-10-12 14:16:05.634787
26759	ANGL	2020-10-07	29.909	30.02	29.909	29.95	850168	{}	\N	2020-10-12 14:16:05.639054
26760	ANGL	2020-10-08	30.08	30.08	30	30.07	602350	{}	\N	2020-10-12 14:16:05.643213
26761	ANGL	2020-10-09	30.14	30.19	30.08	30.15	1114041	{}	\N	2020-10-12 14:16:05.647536
26762	ANGL	2020-10-12	30.2	30.255	30.195	30.2232	47849	{}	\N	2020-10-12 14:16:05.652302
26763	APLS	2020-09-28	30.93	31.01	29.7	29.86	293571	{}	\N	2020-10-12 14:16:06.997181
26764	APLS	2020-09-29	29.63	30.21	28.74	28.89	416523	{}	\N	2020-10-12 14:16:07.006771
26765	APLS	2020-09-30	29.13	30.34	28.89	30.14	304341	{}	\N	2020-10-12 14:16:07.01396
26766	APLS	2020-10-01	30.22	31.82	29.8	31.82	637497	{}	\N	2020-10-12 14:16:07.019407
26767	APLS	2020-10-02	31.12	32.02	30.5998	30.79	442937	{}	\N	2020-10-12 14:16:07.024856
26768	APLS	2020-10-05	31.17	33.085	31.17	33.01	565158	{}	\N	2020-10-12 14:16:07.029843
26769	APLS	2020-10-06	33.21	33.54	32.36	32.54	466956	{}	\N	2020-10-12 14:16:07.034482
26770	APLS	2020-10-07	32.82	36.56	32.6995	35.52	902563	{}	\N	2020-10-12 14:16:07.038831
26771	APLS	2020-10-08	35.85	36.04	34.75	35.26	381935	{}	\N	2020-10-12 14:16:07.043221
26772	APLS	2020-10-09	35.57	36.57	33.89	36.29	612556	{}	\N	2020-10-12 14:16:07.048376
26773	APLS	2020-10-12	36.12	36.49	35.57	36.11	30945	{}	\N	2020-10-12 14:16:07.053996
26774	APTS	2020-09-28	5.53	5.88	5.53	5.71	554195	{}	\N	2020-10-12 14:16:08.046036
26775	APTS	2020-09-29	5.69	5.69	5.38	5.43	719503	{}	\N	2020-10-12 14:16:08.058546
26776	APTS	2020-09-30	5.45	5.53	5.3	5.4	823204	{}	\N	2020-10-12 14:16:08.065169
26777	APTS	2020-10-01	5.45	5.68	5.33	5.65	863076	{}	\N	2020-10-12 14:16:08.070792
26778	APTS	2020-10-02	5.4842	5.805	5.48	5.79	650838	{}	\N	2020-10-12 14:16:08.07632
26779	APTS	2020-10-05	5.76	5.905	5.605	5.67	584325	{}	\N	2020-10-12 14:16:08.081083
26780	APTS	2020-10-06	5.65	5.87	5.62	5.66	697647	{}	\N	2020-10-12 14:16:08.085154
26781	APTS	2020-10-07	5.68	5.73	5.5801	5.67	815399	{}	\N	2020-10-12 14:16:08.089076
26782	APTS	2020-10-08	5.66	5.95	5.641	5.95	586706	{}	\N	2020-10-12 14:16:08.093436
26783	APTS	2020-10-09	6	6.04	5.77	5.85	670286	{}	\N	2020-10-12 14:16:08.098077
26784	APTS	2020-10-12	5.81	5.81	5.75	5.75	89425	{}	\N	2020-10-12 14:16:08.101791
26785	AQUA	2020-09-28	20.015	20.634	19.87	20.43	401433	{}	\N	2020-10-12 14:16:09.027091
26786	AQUA	2020-09-29	20.48	21.07	20.28	21.01	845168	{}	\N	2020-10-12 14:16:09.038858
26787	AQUA	2020-09-30	20.97	21.68	20.68	21.22	1120310	{}	\N	2020-10-12 14:16:09.046115
26788	AQUA	2020-10-01	21.31	21.68	20.93	21.65	1786447	{}	\N	2020-10-12 14:16:09.052937
26789	AQUA	2020-10-02	21.09	22	21.01	21.74	809619	{}	\N	2020-10-12 14:16:09.05835
26790	AQUA	2020-10-05	21.94	22.86	21.85	22.42	2456697	{}	\N	2020-10-12 14:16:09.063665
26791	AQUA	2020-10-06	22.61	23.25	22.43	22.8	2286333	{}	\N	2020-10-12 14:16:09.067859
26792	AQUA	2020-10-07	23.12	23.5	23.02	23.31	1630707	{}	\N	2020-10-12 14:16:09.072223
26793	AQUA	2020-10-08	23.66	23.77	23.25	23.5	960704	{}	\N	2020-10-12 14:16:09.077027
26794	AQUA	2020-10-09	23.72	24.22	23.48	24	1620735	{}	\N	2020-10-12 14:16:09.08245
26795	AQUA	2020-10-12	24.2	25.19	24.015	24.51	294715	{}	\N	2020-10-12 14:16:09.087982
26796	ARCO	2020-09-28	4.28	4.3	4.13	4.15	885616	{}	\N	2020-10-12 14:16:10.090001
26797	ARCO	2020-09-29	4.13	4.18	4.03	4.11	5197009	{}	\N	2020-10-12 14:16:10.099517
26798	ARCO	2020-09-30	4.12	4.25	4.07	4.1	1848331	{}	\N	2020-10-12 14:16:10.104893
26799	ARCO	2020-10-01	4.13	4.17	4.06	4.13	708701	{}	\N	2020-10-12 14:16:10.110816
26800	ARCO	2020-10-02	4.06	4.19	4.06	4.14	576118	{}	\N	2020-10-12 14:16:10.115299
26801	ARCO	2020-10-05	4.17	4.21	4.105	4.14	870250	{}	\N	2020-10-12 14:16:10.119025
26802	ARCO	2020-10-06	4.14	4.38	4.14	4.25	2872212	{}	\N	2020-10-12 14:16:10.122902
26803	ARCO	2020-10-07	4.34	4.35	4.23	4.24	2450315	{}	\N	2020-10-12 14:16:10.126923
26804	ARCO	2020-10-08	4.31	4.38	4.27	4.32	1604647	{}	\N	2020-10-12 14:16:10.130627
26805	ARCO	2020-10-09	4.33	4.37	4.26	4.32	1589234	{}	\N	2020-10-12 14:16:10.13517
26806	ARCO	2020-10-12	4.34	4.42	4.34	4.4	189366	{}	\N	2020-10-12 14:16:10.139019
26807	ARGT	2020-09-28	23.67	23.67	23.28	23.4	9808	{}	\N	2020-10-12 14:16:11.532793
26808	ARGT	2020-09-29	23.4	23.4	23	23.08	9765	{}	\N	2020-10-12 14:16:11.541406
26809	ARGT	2020-09-30	23.13	23.57	23.13	23.29	25244	{}	\N	2020-10-12 14:16:11.548527
26810	ARGT	2020-10-01	23.42	23.75	23.42	23.75	14572	{}	\N	2020-10-12 14:16:11.554676
26811	ARGT	2020-10-02	23.4	23.579	23.3001	23.405	2411	{}	\N	2020-10-12 14:16:11.55974
26812	ARGT	2020-10-05	23.51	23.79	23.51	23.76	18045	{}	\N	2020-10-12 14:16:11.564688
26813	ARGT	2020-10-06	23.72	24.074	23.48	23.51	10151	{}	\N	2020-10-12 14:16:11.568816
26814	ARGT	2020-10-07	23.68	24.06	23.46	24.06	4286	{}	\N	2020-10-12 14:16:11.573172
26815	ARGT	2020-10-08	24.12	24.36	24.11	24.25	18395	{}	\N	2020-10-12 14:16:11.577329
26816	ARGT	2020-10-09	24.5	24.63	24.41	24.63	42195	{}	\N	2020-10-12 14:16:11.581283
26817	ARGT	2020-10-12	24.84	24.8831	24.63	24.63	2446	{}	\N	2020-10-12 14:16:11.585096
26818	ARL	2020-09-28	8.11	8.25	7.98	8.12	3830	{}	\N	2020-10-12 14:16:12.556024
26819	ARL	2020-09-29	8.1	8.5165	8.1	8.49	15998	{}	\N	2020-10-12 14:16:12.565524
26820	ARL	2020-09-30	8.28	9.68	8.28	9.07	3928	{}	\N	2020-10-12 14:16:12.572509
26821	ARL	2020-10-01	9.5	10.25	9.5	9.76	14540	{}	\N	2020-10-12 14:16:12.57791
26822	ARL	2020-10-02	9.98	10.31	9.85	9.85	6996	{}	\N	2020-10-12 14:16:12.582349
26823	ARL	2020-10-05	9.66	9.66	9.24	9.24	11276	{}	\N	2020-10-12 14:16:12.586943
26824	ARL	2020-10-06	9.72	9.72	9.34	9.34	3691	{}	\N	2020-10-12 14:16:12.591094
26825	ARL	2020-10-07	9.41	10.21	9.41	10.21	2348	{}	\N	2020-10-12 14:16:12.595102
26826	ARL	2020-10-08	10.01	10.01	9.27	9.27	2650	{}	\N	2020-10-12 14:16:12.600197
26827	ARL	2020-10-09	9.4	9.4	9.4	9.4	1176	{}	\N	2020-10-12 14:16:12.603699
26828	ARL	2020-10-12	9.31	9.31	9.31	9.31	413	{}	\N	2020-10-12 14:16:12.607236
26829	FET	2020-09-28	0.508	0.575	0.508	0.5704	726758	{}	\N	2020-10-12 14:16:14.316886
26830	FET	2020-09-29	0.55	0.5749	0.5291	0.5749	1739850	{}	\N	2020-10-12 14:16:14.333047
26831	FET	2020-09-30	0.541	0.5689	0.541	0.5495	1462243	{}	\N	2020-10-12 14:16:14.339851
26832	FET	2020-10-01	0.5312	0.56	0.5312	0.5565	1099635	{}	\N	2020-10-12 14:16:14.345155
26833	FET	2020-10-02	0.6	0.62	0.551	0.5701	3384762	{}	\N	2020-10-12 14:16:14.350749
26834	FET	2020-10-05	0.57	0.5999	0.55	0.5617	1695028	{}	\N	2020-10-12 14:16:14.355629
26835	FET	2020-10-06	0.578	0.5863	0.5505	0.5505	1603154	{}	\N	2020-10-12 14:16:14.360108
26836	FET	2020-10-07	0.563	0.5739	0.5511	0.5631	1143050	{}	\N	2020-10-12 14:16:14.364573
26837	FET	2020-10-08	0.58	0.6166	0.56	0.5938	4430418	{}	\N	2020-10-12 14:16:14.369621
26838	FET	2020-10-09	0.5933	0.6199	0.587449	0.6026	3008370	{}	\N	2020-10-12 14:16:14.375982
26839	FET	2020-10-12	0.6599	0.6599	0.605	0.6176	879782	{}	\N	2020-10-12 14:16:14.380559
26840	AMWD	2020-09-28	76.75	78.91	76.41	78.3	115032	{}	\N	2020-10-12 14:16:15.782149
26841	AMWD	2020-09-29	78.32	78.65	76.95	77.47	126847	{}	\N	2020-10-12 14:16:15.7957
26842	AMWD	2020-09-30	78.92	80.31	77.86	78.54	116393	{}	\N	2020-10-12 14:16:15.803002
26843	AMWD	2020-10-01	79.03	82.39	78.48	82.13	166181	{}	\N	2020-10-12 14:16:15.809006
26844	AMWD	2020-10-02	80.78	84.47	80.63	83.91	114331	{}	\N	2020-10-12 14:16:15.81378
26845	AMWD	2020-10-05	84.85	86.84	84.765	86.66	136343	{}	\N	2020-10-12 14:16:15.8183
26846	AMWD	2020-10-06	87.57	90.77	86.275	88.24	206005	{}	\N	2020-10-12 14:16:15.822708
26847	AMWD	2020-10-07	89.47	92.3	89.22	90.54	191828	{}	\N	2020-10-12 14:16:15.826866
26848	AMWD	2020-10-08	91.65	92.78	89.39	90.81	134381	{}	\N	2020-10-12 14:16:15.831102
26849	AMWD	2020-10-09	92.35	92.74	91.0601	91.84	103718	{}	\N	2020-10-12 14:16:15.835267
26850	AMWD	2020-10-12	91.9	91.9	91.4116	91.88	4643	{}	\N	2020-10-12 14:16:15.839125
26851	APA	2020-09-28	10.15	10.65	10.1006	10.27	15464967	{}	\N	2020-10-12 14:16:17.431279
26852	APA	2020-09-29	10.16	10.17	9.43	9.54	13175350	{}	\N	2020-10-12 14:16:17.437322
26853	APA	2020-09-30	9.61	9.79	9.385	9.47	12026365	{}	\N	2020-10-12 14:16:17.442649
26854	APA	2020-10-01	9.38	9.51	9.01	9.31	15770710	{}	\N	2020-10-12 14:16:17.448073
26855	APA	2020-10-02	8.83	9.895	8.75	9.74	14756365	{}	\N	2020-10-12 14:16:17.452846
26856	APA	2020-10-05	9.97	10.13	9.62	9.84	11148264	{}	\N	2020-10-12 14:16:17.457171
26857	APA	2020-10-06	10.03	10.27	9.33	9.41	12315338	{}	\N	2020-10-12 14:16:17.461021
26858	APA	2020-10-07	9.6752	9.6752	9.37	9.57	8890238	{}	\N	2020-10-12 14:16:17.464965
26859	APA	2020-10-08	9.73	10.15	9.51	10.14	10249016	{}	\N	2020-10-12 14:16:17.469099
26860	APA	2020-10-09	10.27	10.42	9.61	9.665	11605124	{}	\N	2020-10-12 14:16:17.473208
26861	APA	2020-10-12	9.56	9.6	9.19	9.295	3164266	{}	\N	2020-10-12 14:16:17.477519
26862	APPN	2020-09-28	61.23	62.78	61.02	62.53	497429	{}	\N	2020-10-12 14:16:19.025711
26863	APPN	2020-09-29	62.3	66.68	62.26	66.24	1041473	{}	\N	2020-10-12 14:16:19.03709
26864	APPN	2020-09-30	66.31	67.42	64.32	64.75	631231	{}	\N	2020-10-12 14:16:19.04372
26865	APPN	2020-10-01	66.595	67.3	64.76	66.72	497825	{}	\N	2020-10-12 14:16:19.049197
26866	APPN	2020-10-02	64.57	67.2839	64.13	65.79	413595	{}	\N	2020-10-12 14:16:19.054206
26867	APPN	2020-10-05	66.19	67.8412	65.02	67.71	437603	{}	\N	2020-10-12 14:16:19.059455
26868	APPN	2020-10-06	68.2466	71.29	68.01	70.15	672759	{}	\N	2020-10-12 14:16:19.06362
26869	APPN	2020-10-07	71.23	74.99	71.23	73.81	665440	{}	\N	2020-10-12 14:16:19.067696
26870	APPN	2020-10-08	75.01	75.25	72.85	73.17	371721	{}	\N	2020-10-12 14:16:19.071692
26871	APPN	2020-10-09	73.72	78.22	73.35	77.19	653689	{}	\N	2020-10-12 14:16:19.076323
26872	APPN	2020-10-12	78.92	79.77	76.14	76.9	94322	{}	\N	2020-10-12 14:16:19.080377
26873	APWC	2020-09-28	1.63	1.68	1.32	1.51	636946	{}	\N	2020-10-12 14:16:20.16255
26874	APWC	2020-09-29	1.37	1.45	1.21	1.27	347537	{}	\N	2020-10-12 14:16:20.174688
26875	APWC	2020-09-30	1.43	1.43	1.23	1.235	144364	{}	\N	2020-10-12 14:16:20.18124
26876	APWC	2020-10-01	1.22	1.26	1.15	1.2261	117978	{}	\N	2020-10-12 14:16:20.187038
26877	APWC	2020-10-02	1.23	1.28	1.19	1.26	65678	{}	\N	2020-10-12 14:16:20.192228
26878	APWC	2020-10-05	1.26	1.52	1.2421	1.34	1061530	{}	\N	2020-10-12 14:16:20.197184
26879	APWC	2020-10-06	1.37	2.69	1.37	2.63	8494691	{}	\N	2020-10-12 14:16:20.201588
26880	APWC	2020-10-07	2.03	2.26	1.62	1.79	2537884	{}	\N	2020-10-12 14:16:20.206102
26881	APWC	2020-10-08	1.69	1.81	1.55	1.68	600467	{}	\N	2020-10-12 14:16:20.210233
26882	APWC	2020-10-09	1.58	1.67	1.56	1.65	225349	{}	\N	2020-10-12 14:16:20.214797
26883	APWC	2020-10-12	1.6	1.6184	1.5	1.5402	68959	{}	\N	2020-10-12 14:16:20.21886
26884	ARC	2020-09-28	0.9458	0.968	0.857	0.9	67935	{}	\N	2020-10-12 14:16:21.695683
26885	ARC	2020-09-29	0.93	0.9699	0.87	0.93	306718	{}	\N	2020-10-12 14:16:21.706429
26886	ARC	2020-09-30	0.95	0.9595	0.9	0.9	107734	{}	\N	2020-10-12 14:16:21.713282
26887	ARC	2020-10-01	0.9	0.9452	0.9	0.93	259423	{}	\N	2020-10-12 14:16:21.718758
26888	ARC	2020-10-02	0.89	0.9998	0.89	0.9795	166842	{}	\N	2020-10-12 14:16:21.72427
26889	ARC	2020-10-05	0.9367	0.99	0.9366	0.97	313618	{}	\N	2020-10-12 14:16:21.72936
26890	ARC	2020-10-06	0.945	1.08	0.945	1	281768	{}	\N	2020-10-12 14:16:21.735046
26891	ARC	2020-10-07	1.08	1.12	1	1.12	434340	{}	\N	2020-10-12 14:16:21.7407
26892	ARC	2020-10-08	1.13	1.13	1.025	1.1	222358	{}	\N	2020-10-12 14:16:21.747261
26893	ARC	2020-10-09	1.07	1.11	1.03	1.09	340508	{}	\N	2020-10-12 14:16:21.754803
26894	ARC	2020-10-12	1.09	1.09	1.0519	1.06	28676	{}	\N	2020-10-12 14:16:21.75989
26895	ARKR	2020-09-28	12	12.8	11.5901	11.91	16582	{}	\N	2020-10-12 14:16:23.43181
26896	ARKR	2020-09-29	12.25	12.25	11.8	11.995	5332	{}	\N	2020-10-12 14:16:23.441802
26897	ARKR	2020-09-30	12.15	12.47	10.76	10.86	49999	{}	\N	2020-10-12 14:16:23.449297
26898	ARKR	2020-10-01	11.2	11.57	10.855	11.38	4711	{}	\N	2020-10-12 14:16:23.455653
26899	ARKR	2020-10-02	11.27	11.6071	11.27	11.505	5540	{}	\N	2020-10-12 14:16:23.460837
26900	ARKR	2020-10-05	11.7	11.79	11.2001	11.6	4960	{}	\N	2020-10-12 14:16:23.465659
26901	ARKR	2020-10-06	11.81	12.0066	11.55	11.67	15796	{}	\N	2020-10-12 14:16:23.472274
26902	ARKR	2020-10-07	11.6174	11.96	11.3074	11.96	7068	{}	\N	2020-10-12 14:16:23.478526
26903	ARKR	2020-10-08	12	12	11.3101	12	4804	{}	\N	2020-10-12 14:16:23.485148
26904	ARKR	2020-10-09	12.25	12.3862	12	12.175	18631	{}	\N	2020-10-12 14:16:23.492127
26905	ARKR	2020-10-12	12.36	12.36	12.1	12.1699	7804	{}	\N	2020-10-12 14:16:23.498814
26906	ARTNA	2020-09-28	33.95	34.42	33.8947	34.27	18931	{}	\N	2020-10-12 14:16:24.427831
26907	ARTNA	2020-09-29	34.3	34.62	34.1	34.3	13627	{}	\N	2020-10-12 14:16:24.438145
26908	ARTNA	2020-09-30	34.51	34.9116	34.4	34.47	28193	{}	\N	2020-10-12 14:16:24.445285
26909	ARTNA	2020-10-01	34.69	34.78	34.37	34.72	18413	{}	\N	2020-10-12 14:16:24.45202
26910	ARTNA	2020-10-02	34.63	35.23	34.5	35.08	14448	{}	\N	2020-10-12 14:16:24.457304
26911	ARTNA	2020-10-05	35.06	35.25	35	35.08	15745	{}	\N	2020-10-12 14:16:24.462306
26912	ARTNA	2020-10-06	35.3	35.3	34.53	34.54	18870	{}	\N	2020-10-12 14:16:24.466707
26913	ARTNA	2020-10-07	34.82	35.025	34.79	34.9	21708	{}	\N	2020-10-12 14:16:24.471693
26914	ARTNA	2020-10-08	35.12	35.29	34.91	35.1	12828	{}	\N	2020-10-12 14:16:24.476441
26915	ARTNA	2020-10-09	35.1	35.1	34.7	34.81	31693	{}	\N	2020-10-12 14:16:24.481
26916	ARTNA	2020-10-12	34.8	35.82	34.65	35.775	29075	{}	\N	2020-10-12 14:16:24.48533
26917	ASFI	2020-09-28	13.08	13.09	13.08	13.08	1331	{}	\N	2020-10-12 14:16:25.613452
26918	ASML	2020-09-28	365.9	371.49	364.35	371.06	764699	{}	\N	2020-10-12 14:16:26.790678
26919	ASML	2020-09-29	370.07	375.88	369.58	374	569223	{}	\N	2020-10-12 14:16:26.802075
26920	ASML	2020-09-30	368.78	373.2499	367.5	369.27	804759	{}	\N	2020-10-12 14:16:26.809457
26921	ASML	2020-10-01	375.66	382.65	374.42	382.01	845606	{}	\N	2020-10-12 14:16:26.815622
26922	ASML	2020-10-02	372.4	375.72	370.02	370.2	589894	{}	\N	2020-10-12 14:16:26.820482
26923	ASML	2020-10-05	375.1	382.4	374.94	382.11	620103	{}	\N	2020-10-12 14:16:26.825663
26924	ASML	2020-10-06	374.2	377.38	368.56	370.11	628622	{}	\N	2020-10-12 14:16:26.82977
26925	ASML	2020-10-07	374.43	376.48	371.63	375.8	532969	{}	\N	2020-10-12 14:16:26.833736
26926	ASML	2020-10-08	376.77	380.95	376.44	380.89	663803	{}	\N	2020-10-12 14:16:26.837667
26927	ASML	2020-10-09	385.8	389.975	385.77	388.99	508612	{}	\N	2020-10-12 14:16:26.841589
26928	ASML	2020-10-12	395.43	398.76	395.43	397.33	186238	{}	\N	2020-10-12 14:16:26.845234
26929	AT	2020-09-28	1.98	2.01	1.98	2	180691	{}	\N	2020-10-12 14:16:28.23498
26930	AT	2020-09-29	1.99	2	1.935	1.94	485301	{}	\N	2020-10-12 14:16:28.244654
26931	AT	2020-09-30	1.93	2.01	1.925	1.96	1229560	{}	\N	2020-10-12 14:16:28.252723
26932	AT	2020-10-01	1.97	2.025	1.95	2.01	596468	{}	\N	2020-10-12 14:16:28.259039
26933	AT	2020-10-02	1.96	2.01	1.96	2	291448	{}	\N	2020-10-12 14:16:28.264259
26934	AT	2020-10-05	2	2.02	1.98	2.01	355098	{}	\N	2020-10-12 14:16:28.269161
26935	AT	2020-10-06	2.01	2.02	1.98	2.01	478310	{}	\N	2020-10-12 14:16:28.273613
26936	AT	2020-10-07	2.01	2.05	2	2.04	383721	{}	\N	2020-10-12 14:16:28.277629
26937	AT	2020-10-08	2.03	2.07	2.03	2.05	557061	{}	\N	2020-10-12 14:16:28.281496
26938	AT	2020-10-09	2.07	2.08	2.055	2.08	314578	{}	\N	2020-10-12 14:16:28.285603
26939	AT	2020-10-12	2.08	2.0822	2.07	2.07	35195	{}	\N	2020-10-12 14:16:28.289952
26940	ATI	2020-09-28	9.61	9.77	9.29	9.29	1938174	{}	\N	2020-10-12 14:16:29.547363
26941	ATI	2020-09-29	9.21	9.3	8.65	8.74	7225434	{}	\N	2020-10-12 14:16:29.558041
26942	ATI	2020-09-30	8.74	9.325	8.69	8.72	5002058	{}	\N	2020-10-12 14:16:29.566134
26943	ATI	2020-10-01	8.74	9.23	8.74	9.22	3620435	{}	\N	2020-10-12 14:16:29.571752
26944	ATI	2020-10-02	8.9	9.655	8.85	9.59	4495389	{}	\N	2020-10-12 14:16:29.577094
26945	ATI	2020-10-05	9.77	9.98	9.63	9.75	2901323	{}	\N	2020-10-12 14:16:29.581656
26946	ATI	2020-10-06	9.94	9.98	9.11	9.14	5091101	{}	\N	2020-10-12 14:16:29.585551
26947	ATI	2020-10-07	9.32	9.71	9.2501	9.59	3694676	{}	\N	2020-10-12 14:16:29.589928
26948	ATI	2020-10-08	9.69	9.89	9.52	9.78	4663774	{}	\N	2020-10-12 14:16:29.594168
26949	ATI	2020-10-09	9.91	9.92	9.4	9.4	2393630	{}	\N	2020-10-12 14:16:29.598422
26950	ATI	2020-10-12	9.32	9.38	9.155	9.28	427835	{}	\N	2020-10-12 14:16:29.60285
26951	APO	2020-09-28	45.317	46.6	45.317	46.3	1090300	{}	\N	2020-10-12 14:16:31.320831
26952	APO	2020-09-29	46.43	46.85	45.34	45.35	1679158	{}	\N	2020-10-12 14:16:31.336279
26953	APO	2020-09-30	45.31	45.6	44.22	44.75	2394562	{}	\N	2020-10-12 14:16:31.342178
26954	APO	2020-10-01	45.28	47.3399	45.2	47.03	2788293	{}	\N	2020-10-12 14:16:31.346879
26955	APO	2020-10-02	45.65	47.89	45.65	47.34	1405848	{}	\N	2020-10-12 14:16:31.351665
26956	APO	2020-10-05	47.8	48.25	47.47	48.09	1785779	{}	\N	2020-10-12 14:16:31.356051
26957	APO	2020-10-06	47.94	48.11	46.38	46.52	1753179	{}	\N	2020-10-12 14:16:31.359763
26958	APO	2020-10-07	46.34	47.15	45.56	46	2174943	{}	\N	2020-10-12 14:16:31.363722
26959	APO	2020-10-08	46.49	47.19	46.345	46.95	1298373	{}	\N	2020-10-12 14:16:31.367569
26960	APO	2020-10-09	47.41	47.88	46.54	46.55	1600539	{}	\N	2020-10-12 14:16:31.371383
26961	APO	2020-10-12	46.55	46.6307	45.25	45.57	495337	{}	\N	2020-10-12 14:16:31.375499
26962	AQN	2020-09-28	14.29	14.37	14.19	14.36	600037	{}	\N	2020-10-12 14:16:32.773231
26963	AQN	2020-09-29	14.26	14.365	14.135	14.29	1391460	{}	\N	2020-10-12 14:16:32.784064
26964	AQN	2020-09-30	14.36	14.75	14.3	14.54	2842439	{}	\N	2020-10-12 14:16:32.791063
26965	AQN	2020-10-01	14.58	14.66	14.46	14.66	1502947	{}	\N	2020-10-12 14:16:32.797232
26966	AQN	2020-10-02	14.48	14.81	14.48	14.76	1464919	{}	\N	2020-10-12 14:16:32.802837
26967	AQN	2020-10-05	14.9	15.4482	14.83	15.31	2349451	{}	\N	2020-10-12 14:16:32.808513
26968	AQN	2020-10-06	15.35	15.4379	15.12	15.3	1422475	{}	\N	2020-10-12 14:16:32.81413
26969	AQN	2020-10-07	15.45	15.5	15.28	15.41	1459159	{}	\N	2020-10-12 14:16:32.819432
26970	AQN	2020-10-08	15.51	15.635	15.4675	15.6	1718674	{}	\N	2020-10-12 14:16:32.82472
26971	AQN	2020-10-09	15.6	15.8	15.57	15.65	2063658	{}	\N	2020-10-12 14:16:32.830332
26972	AQN	2020-10-12	15.78	15.78	15.65	15.71	154218	{}	\N	2020-10-12 14:16:32.836292
26973	ARES	2020-09-28	40.08	40.85	40.08	40.51	735636	{}	\N	2020-10-12 14:16:33.775823
26974	ARES	2020-09-29	40.6	40.67	39.92	40.2	2837981	{}	\N	2020-10-12 14:16:33.787964
26975	ARES	2020-09-30	40.21	40.88	40.14	40.42	1273489	{}	\N	2020-10-12 14:16:33.795135
26976	ARES	2020-10-01	40.87	40.94	39.985	40.53	1373407	{}	\N	2020-10-12 14:16:33.801001
26977	ARES	2020-10-02	39.95	41.26	39.8	41	753132	{}	\N	2020-10-12 14:16:33.805562
26978	ARES	2020-10-05	41.36	42.35	41.22	42.25	871045	{}	\N	2020-10-12 14:16:33.810398
26979	ARES	2020-10-06	42.36	42.71	41.74	42.08	1135623	{}	\N	2020-10-12 14:16:33.814831
26980	ARES	2020-10-07	42.6	42.6	41.85	42.06	957530	{}	\N	2020-10-12 14:16:33.818668
26981	ARES	2020-10-08	42.49	43.24	42.17	42.97	1202124	{}	\N	2020-10-12 14:16:33.822694
26982	ARES	2020-10-09	43.3	43.49	42.92	43.15	993885	{}	\N	2020-10-12 14:16:33.826536
26983	ARES	2020-10-12	43.24	43.69	42.92	43.69	92674	{}	\N	2020-10-12 14:16:33.830522
26984	ARLP	2020-09-28	2.9	2.97	2.78	2.85	849065	{}	\N	2020-10-12 14:16:35.436594
26985	ARLP	2020-09-29	2.91	2.95	2.78	2.79	568515	{}	\N	2020-10-12 14:16:35.445639
26986	ARLP	2020-09-30	2.81	2.8751	2.76	2.77	536841	{}	\N	2020-10-12 14:16:35.452177
26987	ARLP	2020-10-01	2.77	2.815	2.63	2.77	555793	{}	\N	2020-10-12 14:16:35.458954
26988	ARLP	2020-10-02	2.66	2.87	2.66	2.86	340652	{}	\N	2020-10-12 14:16:35.463728
26989	ARLP	2020-10-05	2.86	2.97	2.86	2.94	236062	{}	\N	2020-10-12 14:16:35.46838
26990	ARLP	2020-10-06	2.94	3.02	2.82	2.83	518982	{}	\N	2020-10-12 14:16:35.472667
26991	ARLP	2020-10-07	2.8	2.85	2.76	2.76	396794	{}	\N	2020-10-12 14:16:35.476826
26992	ARLP	2020-10-08	2.85	2.86	2.72	2.83	517089	{}	\N	2020-10-12 14:16:35.480552
26993	ARLP	2020-10-09	2.87	2.89	2.72	2.79	463705	{}	\N	2020-10-12 14:16:35.48439
26994	ARLP	2020-10-12	2.75	2.8026	2.75	2.76	31231	{}	\N	2020-10-12 14:16:35.488166
26995	ASC	2020-09-28	3.05	3.0683	2.9399	3.01	406039	{}	\N	2020-10-12 14:16:37.282297
26996	ASC	2020-09-29	3.19	3.3854	3.13	3.2	1341869	{}	\N	2020-10-12 14:16:37.29197
26997	ASC	2020-09-30	3.2	3.64	3.18	3.56	1464985	{}	\N	2020-10-12 14:16:37.298556
26998	ASC	2020-10-01	3.54	3.55	3.4	3.47	472606	{}	\N	2020-10-12 14:16:37.30393
26999	ASC	2020-10-02	3.41	3.575	3.35	3.55	399127	{}	\N	2020-10-12 14:16:37.309061
27000	ASC	2020-10-05	3.57	3.71	3.54	3.66	514838	{}	\N	2020-10-12 14:16:37.313722
27001	ASC	2020-10-06	3.69	3.77	3.39	3.45	774220	{}	\N	2020-10-12 14:16:37.317673
27002	ASC	2020-10-07	3.55	3.55	3.36	3.45	642930	{}	\N	2020-10-12 14:16:37.321494
27003	ASC	2020-10-08	3.5	3.63	3.45	3.6	342613	{}	\N	2020-10-12 14:16:37.328773
27004	ASC	2020-10-09	3.65	3.6777	3.46	3.49	227476	{}	\N	2020-10-12 14:16:37.33258
27005	ASC	2020-10-12	3.46	3.46	3.39	3.41	52763	{}	\N	2020-10-12 14:16:37.336464
27006	ASHX	2020-09-28	25.1	25.1	25.1	25.1	7	{}	\N	2020-10-12 14:16:38.806211
27007	ASHX	2020-09-29	25.3138	25.3138	25.3138	25.3138	67	{}	\N	2020-10-12 14:16:38.813802
27008	ASHX	2020-09-30	25.5099	25.5099	25.3751	25.3751	400	{}	\N	2020-10-12 14:16:38.819385
27009	ASHX	2020-10-01	25.7907	25.7907	25.7907	25.7907	0	{}	\N	2020-10-12 14:16:38.824072
27010	ASHX	2020-10-02	25.73	25.75	25.6241	25.6241	1000	{}	\N	2020-10-12 14:16:38.828756
27011	ASHX	2020-10-05	25.77	25.77	25.77	25.77	300	{}	\N	2020-10-12 14:16:38.833352
27012	ASHX	2020-10-06	25.94	25.94	25.9	25.9	469	{}	\N	2020-10-12 14:16:38.837274
27013	ASHX	2020-10-07	26.09	26.09	26.09	26.09	110	{}	\N	2020-10-12 14:16:38.84126
27014	ASHX	2020-10-08	26.2253	26.2253	26.2253	26.2253	40	{}	\N	2020-10-12 14:16:38.844981
27015	ASHX	2020-10-09	26.3	26.3	26.3	26.3	0	{}	\N	2020-10-12 14:16:38.848928
27016	ASPU	2020-09-28	10.95	11.41	10.85	11.24	209980	{}	\N	2020-10-12 14:16:39.865849
27017	ASPU	2020-09-29	11.2	11.66	11.0607	11.51	165811	{}	\N	2020-10-12 14:16:39.874778
27018	ASPU	2020-09-30	11.51	11.6	10.97	11.17	112830	{}	\N	2020-10-12 14:16:39.882224
27019	ASPU	2020-10-01	11.11	11.23	10.77	11.09	87475	{}	\N	2020-10-12 14:16:39.887426
27020	ASPU	2020-10-02	10.87	11.37	10.7401	10.92	65621	{}	\N	2020-10-12 14:16:39.892375
27021	ASPU	2020-10-05	11.1	11.56	10.85	11.4	120841	{}	\N	2020-10-12 14:16:39.896603
27022	ASPU	2020-10-06	11.6	11.85	11.4	11.45	74661	{}	\N	2020-10-12 14:16:39.900748
27023	ASPU	2020-10-07	11.56	11.87	11.56	11.71	76828	{}	\N	2020-10-12 14:16:39.904955
27024	ASPU	2020-10-08	11.81	11.9	11.52	11.67	51243	{}	\N	2020-10-12 14:16:39.909305
27025	ASPU	2020-10-09	11.75	12.08	11.65	12.07	70091	{}	\N	2020-10-12 14:16:39.913629
27026	ASPU	2020-10-12	12.08	12.57	11.99	12.06	31715	{}	\N	2020-10-12 14:16:39.91764
27027	ATAX	2020-09-28	4.02	4.2	4.02	4.11	112322	{}	\N	2020-10-12 14:16:40.862504
27028	ATAX	2020-09-29	4.13	4.22	4.095	4.22	203698	{}	\N	2020-10-12 14:16:40.867397
27029	ATAX	2020-09-30	4.2	4.23	4.04	4.05	111932	{}	\N	2020-10-12 14:16:40.871752
27030	ATAX	2020-10-01	4.15	4.22	4.12	4.16	126317	{}	\N	2020-10-12 14:16:40.876149
27031	ATAX	2020-10-02	4.12	4.22	4.08	4.16	59329	{}	\N	2020-10-12 14:16:40.881161
27032	ATAX	2020-10-05	4.2	4.23	4.14	4.2	111122	{}	\N	2020-10-12 14:16:40.885175
27033	ATAX	2020-10-06	4.21	4.42	4.12	4.28	175819	{}	\N	2020-10-12 14:16:40.889623
27034	ATAX	2020-10-07	4.3	4.335	4.15	4.22	89779	{}	\N	2020-10-12 14:16:40.893524
27035	ATAX	2020-10-08	4.22	4.2599	4.14	4.14	88892	{}	\N	2020-10-12 14:16:40.898298
27036	ATAX	2020-10-09	4.14	4.14	4.08	4.09	86927	{}	\N	2020-10-12 14:16:40.903006
27632	ARCT	2020-10-05	43.22	50.775	43.22	49.97	598846	{}	\N	2020-10-12 14:18:01.4882
27633	ARCT	2020-10-06	51.5	51.55	46.72	47.15	752850	{}	\N	2020-10-12 14:18:01.49238
27634	ARCT	2020-10-07	47.5651	51.25	46.44	50.92	562265	{}	\N	2020-10-12 14:18:01.496441
27635	ARCT	2020-10-08	52.15	55.13	50.51	51.62	604237	{}	\N	2020-10-12 14:18:01.500618
27636	ARCT	2020-10-09	51.88	52.35	50.0101	51.12	394967	{}	\N	2020-10-12 14:18:01.504726
27637	ARCT	2020-10-12	51.65	52.33	50.95	51.68	144723	{}	\N	2020-10-12 14:18:01.508757
27638	ARI	2020-09-28	9.18	9.57	9.18	9.4	1437521	{}	\N	2020-10-12 14:18:02.870989
27639	ARI	2020-09-29	8.97	9.06	8.78	8.99	3481963	{}	\N	2020-10-12 14:18:02.88212
27640	ARI	2020-09-30	8.99	9.12	8.95	9.01	3029978	{}	\N	2020-10-12 14:18:02.888322
27641	ARI	2020-10-01	9.02	9.16	8.95	9.16	2088956	{}	\N	2020-10-12 14:18:02.89378
27642	ARI	2020-10-02	8.95	9.48	8.91	9.44	2652123	{}	\N	2020-10-12 14:18:02.899106
27643	ARI	2020-10-05	9.52	9.62	9.425	9.46	1661020	{}	\N	2020-10-12 14:18:02.903904
27644	ARI	2020-10-06	9.55	9.695	9.26	9.26	2922846	{}	\N	2020-10-12 14:18:02.908511
27645	ARI	2020-10-07	9.32	9.45	9.075	9.14	2320931	{}	\N	2020-10-12 14:18:02.912598
27646	ARI	2020-10-08	9.21	9.56	9.21	9.52	2193201	{}	\N	2020-10-12 14:18:02.916838
27647	ARI	2020-10-09	9.56	9.62	9.215	9.24	1952674	{}	\N	2020-10-12 14:18:02.920924
27648	ARI	2020-10-12	9.21	9.26	9.18	9.2	279257	{}	\N	2020-10-12 14:18:02.92479
27649	ARNA	2020-09-28	74.17	76.11	73.405	75.83	567480	{}	\N	2020-10-12 14:18:04.287744
27650	ARNA	2020-09-29	76.09	76.49	74.5308	75.6	391749	{}	\N	2020-10-12 14:18:04.297688
27651	ARNA	2020-09-30	75.8	77.175	73.75	74.79	542813	{}	\N	2020-10-12 14:18:04.303797
27652	ARNA	2020-10-01	74.16	75.76	73.59	75.73	612269	{}	\N	2020-10-12 14:18:04.308971
27653	ARNA	2020-10-02	74.4859	75.99	74.05	74.98	462064	{}	\N	2020-10-12 14:18:04.313449
27654	ARNA	2020-10-05	75.47	76.8	75.21	76.46	782052	{}	\N	2020-10-12 14:18:04.318403
27655	ARNA	2020-10-06	76.25	78.49	75.67	77.19	823432	{}	\N	2020-10-12 14:18:04.322712
27656	ARNA	2020-10-07	77.22	79.86	76.31	79.48	725147	{}	\N	2020-10-12 14:18:04.330222
27657	ARNA	2020-10-08	79.97	81.4	79.28	80	673387	{}	\N	2020-10-12 14:18:04.334441
27658	ARNA	2020-10-09	80.46	83.85	79.6	82.57	743787	{}	\N	2020-10-12 14:18:04.338344
27659	ARNA	2020-10-12	83.8	86	83.22	83.67	142951	{}	\N	2020-10-12 14:18:04.342489
27660	ASMB	2020-09-28	16.39	16.53	16.02	16.16	134035	{}	\N	2020-10-12 14:18:07.74618
27661	ASMB	2020-09-29	16.05	16.63	16.01	16.26	122476	{}	\N	2020-10-12 14:18:07.751406
27662	ASMB	2020-09-30	16.28	16.94	16.28	16.44	220692	{}	\N	2020-10-12 14:18:07.756373
27663	ASMB	2020-10-01	16.48	17.29	16.22	17.29	259589	{}	\N	2020-10-12 14:18:07.760577
27037	ATAX	2020-10-12	4.09	4.1	4.05	4.1	32058	{}	\N	2020-10-12 14:16:40.909909
27038	ATNX	2020-09-28	12.45	12.64	12.24	12.27	472643	{}	\N	2020-10-12 14:16:42.305311
27039	ATNX	2020-09-29	12.25	12.29	11.81	12.145	388812	{}	\N	2020-10-12 14:16:42.3159
27040	ATNX	2020-09-30	12.22	12.65	12.02	12.1	580880	{}	\N	2020-10-12 14:16:42.322468
27041	ATNX	2020-10-01	12.15	12.42	12.06	12.27	396223	{}	\N	2020-10-12 14:16:42.33412
27042	ATNX	2020-10-02	11.92	12.4	11.9	12.23	363906	{}	\N	2020-10-12 14:16:42.339281
27043	ATNX	2020-10-05	12.4	12.71	12.27	12.58	318739	{}	\N	2020-10-12 14:16:42.343242
27044	ATNX	2020-10-06	12.82	12.83	12.31	12.31	463700	{}	\N	2020-10-12 14:16:42.346675
27045	ATNX	2020-10-07	12.35	12.87	12.35	12.8	394948	{}	\N	2020-10-12 14:16:42.350161
27046	ATNX	2020-10-08	12.9	13.16	12.72	13.06	414386	{}	\N	2020-10-12 14:16:42.354005
27047	ATNX	2020-10-09	13.03	13.1	12.22	12.24	545730	{}	\N	2020-10-12 14:16:42.357666
27048	ATNX	2020-10-12	12.23	12.347	11.9	12.14	153325	{}	\N	2020-10-12 14:16:42.361267
27049	ATRC	2020-09-28	40.68	40.73	39.8	40.36	333145	{}	\N	2020-10-12 14:16:43.49339
27050	ATRC	2020-09-29	40.54	40.64	39.535	40.02	201468	{}	\N	2020-10-12 14:16:43.503917
27051	ATRC	2020-09-30	37.11	40.23	34.12	39.9	1472972	{}	\N	2020-10-12 14:16:43.511082
27052	ATRC	2020-10-01	40.25	40.33	37.86	38.26	921272	{}	\N	2020-10-12 14:16:43.516806
27053	ATRC	2020-10-02	37.51	38.725	37.42	37.86	440574	{}	\N	2020-10-12 14:16:43.521554
27054	ATRC	2020-10-05	38.24	39.17	37.895	39.16	313166	{}	\N	2020-10-12 14:16:43.52649
27055	ATRC	2020-10-06	39.15	40.15	39	39.6	335482	{}	\N	2020-10-12 14:16:43.530783
27056	ATRC	2020-10-07	39.88	40.92	39.77	40.18	321363	{}	\N	2020-10-12 14:16:43.534914
27057	ATRC	2020-10-08	40.52	40.88	39.76	39.94	404931	{}	\N	2020-10-12 14:16:43.538906
27058	ATRC	2020-10-09	40	41.36	39.6348	41.35	315804	{}	\N	2020-10-12 14:16:43.542988
27059	ATRC	2020-10-12	41.61	41.8	41.08	41.39	11318	{}	\N	2020-10-12 14:16:43.546942
27060	APOG	2020-09-28	21.19	21.77	21.03	21.51	208962	{}	\N	2020-10-12 14:16:44.516128
27061	APOG	2020-09-29	21.38	21.5	20.72	21.14	269782	{}	\N	2020-10-12 14:16:44.525317
27062	APOG	2020-09-30	21.445	21.97	21.301	21.37	447469	{}	\N	2020-10-12 14:16:44.53263
27063	APOG	2020-10-01	21.35	21.63	20.68	20.92	246965	{}	\N	2020-10-12 14:16:44.538564
27064	APOG	2020-10-02	20.22	21.46	20.2	21.34	295059	{}	\N	2020-10-12 14:16:44.543391
27065	APOG	2020-10-05	21.88	23.2999	21.8141	22.63	974295	{}	\N	2020-10-12 14:16:44.548319
27066	APOG	2020-10-06	22.96	23.92	22.7184	23.19	271852	{}	\N	2020-10-12 14:16:44.55313
27067	APOG	2020-10-07	23.58	23.94	22.97	23.355	263037	{}	\N	2020-10-12 14:16:44.557367
27068	APOG	2020-10-08	23.99	24.42	23.58	24.27	268730	{}	\N	2020-10-12 14:16:44.561462
27069	APOG	2020-10-09	24.36	25.58	24.1875	25.45	461256	{}	\N	2020-10-12 14:16:44.565379
27070	APOG	2020-10-12	25.4	26.22	25.31	26.115	61474	{}	\N	2020-10-12 14:16:44.569582
27071	APTO	2020-09-28	5.76	5.92	5.68	5.75	398123	{}	\N	2020-10-12 14:16:46.247805
27072	APTO	2020-09-29	5.76	5.945	5.73	5.9	496741	{}	\N	2020-10-12 14:16:46.257962
27073	APTO	2020-09-30	5.96	6.185	5.9	6	435959	{}	\N	2020-10-12 14:16:46.26459
27074	APTO	2020-10-01	6	6.24	5.97	6.24	676301	{}	\N	2020-10-12 14:16:46.270949
27075	APTO	2020-10-02	6.09	6.2	5.85	5.85	610155	{}	\N	2020-10-12 14:16:46.276114
27076	APTO	2020-10-05	5.9	6.02	5.85	5.92	700892	{}	\N	2020-10-12 14:16:46.280669
27077	APTO	2020-10-06	5.96	6.02	5.88	5.9	319593	{}	\N	2020-10-12 14:16:46.284429
27078	APTO	2020-10-07	5.93	6	5.83	5.89	574820	{}	\N	2020-10-12 14:16:46.288704
27079	APTO	2020-10-08	5.94	6.1716	5.87	5.95	384876	{}	\N	2020-10-12 14:16:46.292933
27080	APTO	2020-10-09	5.98	6.19	5.88	6.16	550375	{}	\N	2020-10-12 14:16:46.297116
27081	APTO	2020-10-12	6.18	6.2	6.05	6.15	35202	{}	\N	2020-10-12 14:16:46.300888
27082	ARA	2020-09-28	6.35	6.65	6.35	6.54	42756	{}	\N	2020-10-12 14:16:47.74088
27083	ARA	2020-09-29	6.54	6.73	6.46	6.68	31736	{}	\N	2020-10-12 14:16:47.750634
27084	ARA	2020-09-30	6.75	6.99	6.75	6.9	199069	{}	\N	2020-10-12 14:16:47.757619
27085	ARA	2020-10-01	6.99	6.99	6.755	6.92	86325	{}	\N	2020-10-12 14:16:47.762702
27086	ARA	2020-10-02	11.35	11.83	11.31	11.52	4534810	{}	\N	2020-10-12 14:16:47.767124
27087	ARA	2020-10-05	11.57	11.71	11.48	11.59	2037198	{}	\N	2020-10-12 14:16:47.772072
27088	ARA	2020-10-06	11.59	11.9799	11.5647	11.79	1212696	{}	\N	2020-10-12 14:16:47.776132
27089	ARA	2020-10-07	11.7	11.75	11.57	11.63	1040614	{}	\N	2020-10-12 14:16:47.780029
27090	ARA	2020-10-08	11.6	11.69	11.55	11.56	238069	{}	\N	2020-10-12 14:16:47.783884
27091	ARA	2020-10-09	11.57	11.6	11.39	11.44	350989	{}	\N	2020-10-12 14:16:47.788045
27092	ARA	2020-10-12	11.44	11.52	11.3815	11.52	79442	{}	\N	2020-10-12 14:16:47.791799
27093	ARNC	2020-09-28	18.5894	20.21	18.5894	20.06	808630	{}	\N	2020-10-12 14:16:49.265979
27094	ARNC	2020-09-29	19.9	20.14	19.17	19.64	901324	{}	\N	2020-10-12 14:16:49.277378
27095	ARNC	2020-09-30	19.6	20.25	18.69	19.05	1826352	{}	\N	2020-10-12 14:16:49.283491
27096	ARNC	2020-10-01	19.18	19.81	18.57	18.83	1303850	{}	\N	2020-10-12 14:16:49.289527
27097	ARNC	2020-10-02	18.34	19.8	18.15	19.52	1089222	{}	\N	2020-10-12 14:16:49.294595
27098	ARNC	2020-10-05	19.73	20.84	19.59	20.68	1224605	{}	\N	2020-10-12 14:16:49.29958
27099	ARNC	2020-10-06	20.98	21.47	20.35	20.37	1499244	{}	\N	2020-10-12 14:16:49.303558
27100	ARNC	2020-10-07	20.66	21.75	20.66	21.39	1830381	{}	\N	2020-10-12 14:16:49.307747
27101	ARNC	2020-10-08	21.56	22.075	21.23	21.94	1173975	{}	\N	2020-10-12 14:16:49.311653
27102	ARNC	2020-10-09	22.25	22.73	21.9	22.67	1387027	{}	\N	2020-10-12 14:16:49.316176
27103	ARNC	2020-10-12	22.7	22.8999	22.66	22.76	54336	{}	\N	2020-10-12 14:16:49.319852
27104	ARW	2020-09-28	79.14	80.31	79.11	79.6	672253	{}	\N	2020-10-12 14:16:50.29085
27105	ARW	2020-09-29	79.51	79.625	78.175	78.54	991159	{}	\N	2020-10-12 14:16:50.301025
27106	ARW	2020-09-30	78.98	79.71	78.03	78.66	1013085	{}	\N	2020-10-12 14:16:50.30829
27107	ARW	2020-10-01	79.47	79.7493	78.155	79.02	877052	{}	\N	2020-10-12 14:16:50.313666
27108	ARW	2020-10-02	77.45	79.78	77.14	79.34	849133	{}	\N	2020-10-12 14:16:50.318018
27109	ARW	2020-10-05	80	82.39	79.77	81.69	2261915	{}	\N	2020-10-12 14:16:50.326997
27110	ARW	2020-10-06	81.93	83.93	81.77	82.01	1218271	{}	\N	2020-10-12 14:16:50.333043
27111	ARW	2020-10-07	82.94	84.77	82.9	84.43	735154	{}	\N	2020-10-12 14:16:50.338124
27112	ARW	2020-10-08	84.99	84.99	83.455	83.65	913398	{}	\N	2020-10-12 14:16:50.342077
27113	ARW	2020-10-09	84.4	84.5	83.06	83.45	824395	{}	\N	2020-10-12 14:16:50.345692
27114	ARW	2020-10-12	84.59	84.59	83.56	83.805	107662	{}	\N	2020-10-12 14:16:50.349651
27115	ASGN	2020-09-28	64.86	65.93	64.86	65.27	186263	{}	\N	2020-10-12 14:16:51.786155
27116	ASGN	2020-09-29	65.27	65.36	63.4	64.06	415710	{}	\N	2020-10-12 14:16:51.796785
27117	ASGN	2020-09-30	64.51	65.42	63.04	63.56	417789	{}	\N	2020-10-12 14:16:51.803674
27118	ASGN	2020-10-01	63.89	64.62	63.04	63.89	542993	{}	\N	2020-10-12 14:16:51.809282
27119	ASGN	2020-10-02	62.88	66.95	62.21	66.4	427930	{}	\N	2020-10-12 14:16:51.814068
27120	ASGN	2020-10-05	66.94	68.29	66.45	67.66	359487	{}	\N	2020-10-12 14:16:51.818739
27121	ASGN	2020-10-06	68.28	69.89	67.71	68.48	510618	{}	\N	2020-10-12 14:16:51.823159
27122	ASGN	2020-10-07	69.34	70.88	69.34	70.53	418206	{}	\N	2020-10-12 14:16:51.827557
27123	ASGN	2020-10-08	71.1	71.91	70.33	71.11	310431	{}	\N	2020-10-12 14:16:51.831599
27124	ASGN	2020-10-09	72	72	69.16	69.27	274688	{}	\N	2020-10-12 14:16:51.835778
27125	ASGN	2020-10-12	69.19	69.69	69.19	69.62	18014	{}	\N	2020-10-12 14:16:51.839673
27126	ASRVP	2020-09-28	27.8728	28.65	27.8728	28.6	1409	{}	\N	2020-10-12 14:16:53.430795
27127	ASRVP	2020-09-29	28.39	28.4	28.225	28.2597	1731	{}	\N	2020-10-12 14:16:53.438662
27128	ASRVP	2020-10-01	28.3	28.4	28.3	28.4	858	{}	\N	2020-10-12 14:16:53.444221
27129	ASRVP	2020-10-02	28.1732	28.1732	28.13	28.1397	1088	{}	\N	2020-10-12 14:16:53.449224
27130	ASRVP	2020-10-05	28.15	28.15	27.8525	27.865	1783	{}	\N	2020-10-12 14:16:53.454247
27131	ASRVP	2020-10-06	27.9999	28	27.8718	27.91	934	{}	\N	2020-10-12 14:16:53.458269
27132	ASRVP	2020-10-08	28	28	28	28	996	{}	\N	2020-10-12 14:16:53.46226
27133	ASUR	2020-09-28	7.05	7.38	7.03	7.35	139935	{}	\N	2020-10-12 14:16:54.987726
27134	ASUR	2020-09-29	7.31	7.6	7.31	7.49	148950	{}	\N	2020-10-12 14:16:54.997722
27135	ASUR	2020-09-30	7.5	7.65	7.5	7.55	54695	{}	\N	2020-10-12 14:16:55.00488
27136	ASUR	2020-10-01	7.625	7.7	7.48	7.61	54016	{}	\N	2020-10-12 14:16:55.010534
27137	ASUR	2020-10-02	7.49	7.63	7.34	7.39	93666	{}	\N	2020-10-12 14:16:55.015909
27138	ASUR	2020-10-05	7.47	7.61	7.38	7.41	95207	{}	\N	2020-10-12 14:16:55.020942
27139	ASUR	2020-10-06	7.41	7.55	7.32	7.45	55937	{}	\N	2020-10-12 14:16:55.025117
27140	ASUR	2020-10-07	7.53	8.02	7.53	7.99	101532	{}	\N	2020-10-12 14:16:55.029531
27141	ASUR	2020-10-08	8.03	8.24	7.95	8.22	62054	{}	\N	2020-10-12 14:16:55.033771
27142	ASUR	2020-10-09	8.28	8.4	8.17	8.37	89091	{}	\N	2020-10-12 14:16:55.03833
27143	ASUR	2020-10-12	8.29	8.38	8.2	8.26	37996	{}	\N	2020-10-12 14:16:55.042495
27144	ATHX	2020-09-28	2.04	2.1	1.94	2.08	1512964	{}	\N	2020-10-12 14:16:57.011678
27145	ATHX	2020-09-29	2.06	2.075	2	2.03	963887	{}	\N	2020-10-12 14:16:57.022955
27146	ATHX	2020-09-30	2.04	2.08	1.925	1.95	1661797	{}	\N	2020-10-12 14:16:57.030119
27147	ATHX	2020-10-01	1.95	1.98	1.88	1.93	1475706	{}	\N	2020-10-12 14:16:57.036072
27148	ATHX	2020-10-02	1.9	1.985	1.85	1.86	1231555	{}	\N	2020-10-12 14:16:57.041626
27149	ATHX	2020-10-05	1.9	1.95	1.894	1.93	1283883	{}	\N	2020-10-12 14:16:57.046596
27150	ATHX	2020-10-06	1.9417	2	1.925	1.95	1218804	{}	\N	2020-10-12 14:16:57.051134
27151	ATHX	2020-10-07	1.94	1.99	1.93	1.94	1226219	{}	\N	2020-10-12 14:16:57.055896
27152	ATHX	2020-10-08	1.98	1.98	1.88	1.93	1543628	{}	\N	2020-10-12 14:16:57.060513
27153	ATHX	2020-10-09	1.95	1.96	1.89	1.93	1250198	{}	\N	2020-10-12 14:16:57.064851
27154	ATHX	2020-10-12	1.93	1.96	1.913	1.945	151006	{}	\N	2020-10-12 14:16:57.069356
27155	ATRS	2020-09-28	2.7	2.79	2.7	2.75	454616	{}	\N	2020-10-12 14:16:58.732483
27156	ATRS	2020-09-29	2.73	2.75	2.68	2.71	441737	{}	\N	2020-10-12 14:16:58.742803
27157	ATRS	2020-09-30	2.73	2.775	2.68	2.71	441180	{}	\N	2020-10-12 14:16:58.749926
27158	ATRS	2020-10-01	2.71	2.9295	2.71	2.84	1171913	{}	\N	2020-10-12 14:16:58.756578
27159	ATRS	2020-10-02	2.75	2.9	2.74	2.88	711781	{}	\N	2020-10-12 14:16:58.761054
27160	ATRS	2020-10-05	2.89	3.03	2.89	2.92	1188763	{}	\N	2020-10-12 14:16:58.765739
27161	ATRS	2020-10-06	2.97	3.03	2.93	3.01	1027703	{}	\N	2020-10-12 14:16:58.769976
27162	ATRS	2020-10-07	3.02	3.22	3.02	3.21	1143214	{}	\N	2020-10-12 14:16:58.774182
27163	ATRS	2020-10-08	3.24	3.29	3.2	3.28	942554	{}	\N	2020-10-12 14:16:58.778133
27164	ATRS	2020-10-09	3.28	3.32	3.26	3.3	934127	{}	\N	2020-10-12 14:16:58.78195
27165	ATRS	2020-10-12	3.32	3.33	3.26	3.31	138213	{}	\N	2020-10-12 14:16:58.785687
27166	AUG	2020-09-28	2	2.025	1.935	1.98	311709	{}	\N	2020-10-12 14:16:59.982306
27167	AUG	2020-09-29	2	2.09	1.975	2.09	386425	{}	\N	2020-10-12 14:16:59.991424
27168	AUG	2020-09-30	2.08	2.08	1.94	1.97	350558	{}	\N	2020-10-12 14:16:59.997253
27169	AUG	2020-10-01	1.99	2.08	1.95	2.07	304316	{}	\N	2020-10-12 14:17:00.003223
27170	AUG	2020-10-02	2.12	2.3	2.06	2.12	559233	{}	\N	2020-10-12 14:17:00.008343
27171	AUG	2020-10-05	2.15	2.19	2.03	2.09	397212	{}	\N	2020-10-12 14:17:00.013247
27172	AUG	2020-10-06	2.21	2.22	2.11	2.16	857946	{}	\N	2020-10-12 14:17:00.019824
27173	AUG	2020-10-07	2.25	2.25	1.98	2.05	1321416	{}	\N	2020-10-12 14:17:00.023859
27174	AUG	2020-10-08	2.15	2.24	2.05	2.24	847506	{}	\N	2020-10-12 14:17:00.027571
27175	AUG	2020-10-09	2.25	2.52	2.19	2.45	929493	{}	\N	2020-10-12 14:17:00.03169
27176	AVA	2020-09-28	33.9	34.53	33.9	34.12	420044	{}	\N	2020-10-12 14:17:01.631602
27177	AVA	2020-09-29	34.32	34.49	33.54	33.72	783977	{}	\N	2020-10-12 14:17:01.642457
27178	AVA	2020-09-30	33.99	34.2	33.69	34.12	1012897	{}	\N	2020-10-12 14:17:01.649775
27179	AVA	2020-10-01	34.04	34.52	34.04	34.41	496828	{}	\N	2020-10-12 14:17:01.65677
27180	AVA	2020-10-02	34.25	34.89	34.05	34.61	764517	{}	\N	2020-10-12 14:17:01.6619
27181	AVA	2020-10-05	34.69	34.93	34.23	34.26	686188	{}	\N	2020-10-12 14:17:01.667204
27182	AVA	2020-10-06	34.51	35.16	34.38	34.62	954529	{}	\N	2020-10-12 14:17:01.671601
27183	AVA	2020-10-07	34.68	35.15	34.36	34.94	1199511	{}	\N	2020-10-12 14:17:01.676101
27184	AVA	2020-10-08	35.22	35.59	34.9216	35.03	790332	{}	\N	2020-10-12 14:17:01.680421
27185	AVA	2020-10-09	35.17	35.27	34.73	35.08	841845	{}	\N	2020-10-12 14:17:01.684525
27186	AVA	2020-10-12	34.87	35.43	34.87	35.42	43859	{}	\N	2020-10-12 14:17:01.689061
27187	AVGR	2020-09-28	0.313	0.3301	0.310501	0.3277	3028922	{}	\N	2020-10-12 14:17:03.135656
27188	AVGR	2020-09-29	0.329	0.3331	0.3176	0.328	1988363	{}	\N	2020-10-12 14:17:03.145598
27189	AVGR	2020-09-30	0.3302	0.3331	0.3205	0.3221	2120507	{}	\N	2020-10-12 14:17:03.152295
27190	AVGR	2020-10-01	0.3231	0.3299	0.3222	0.3242	1460036	{}	\N	2020-10-12 14:17:03.157425
27191	AVGR	2020-10-02	0.3019	0.3228	0.301	0.3173	1734541	{}	\N	2020-10-12 14:17:03.161978
27192	AVGR	2020-10-05	0.3226	0.335	0.317	0.3215	1656951	{}	\N	2020-10-12 14:17:03.166622
27193	AVGR	2020-10-06	0.32	0.3286	0.3156	0.316	2612391	{}	\N	2020-10-12 14:17:03.170903
27194	AVGR	2020-10-07	0.3134	0.3233	0.3051	0.3101	3875855	{}	\N	2020-10-12 14:17:03.174861
27195	AVGR	2020-10-08	0.3098	0.3148	0.305	0.3054	2422715	{}	\N	2020-10-12 14:17:03.178646
27196	AVGR	2020-10-09	0.3083	0.309	0.3	0.3026	2317803	{}	\N	2020-10-12 14:17:03.18222
27197	AVGR	2020-10-12	0.3026	0.3081	0.2983	0.3001	527903	{}	\N	2020-10-12 14:17:03.186128
27198	AWF	2020-09-28	10.53	10.55	10.48	10.48	243880	{}	\N	2020-10-12 14:17:04.297965
27199	AWF	2020-09-29	10.52	10.525	10.43	10.5	539420	{}	\N	2020-10-12 14:17:04.308034
27200	AWF	2020-09-30	10.49	10.57	10.48	10.51	920895	{}	\N	2020-10-12 14:17:04.315474
27201	AWF	2020-10-01	10.47	10.52	10.45	10.49	577911	{}	\N	2020-10-12 14:17:04.321449
27202	AWF	2020-10-02	10.42	10.47	10.42	10.45	371483	{}	\N	2020-10-12 14:17:04.329422
27203	AWF	2020-10-05	10.47	10.52	10.45	10.49	520070	{}	\N	2020-10-12 14:17:04.334355
27204	AWF	2020-10-06	10.5	10.574	10.48	10.51	526733	{}	\N	2020-10-12 14:17:04.339819
27205	AWF	2020-10-07	10.53	10.6	10.53	10.57	559893	{}	\N	2020-10-12 14:17:04.343652
27206	AWF	2020-10-08	10.59	10.65	10.59	10.63	239294	{}	\N	2020-10-12 14:17:04.347645
27207	AWF	2020-10-09	10.65	10.65	10.59	10.6	308436	{}	\N	2020-10-12 14:17:04.35238
27208	AWF	2020-10-12	10.62	10.6433	10.61	10.6145	45890	{}	\N	2020-10-12 14:17:04.356265
27209	AR	2020-09-28	3.02	3.05	2.835	2.96	9708916	{}	\N	2020-10-12 14:17:05.352734
27210	AR	2020-09-29	2.93	2.9858	2.765	2.87	16412425	{}	\N	2020-10-12 14:17:05.361694
27211	AR	2020-09-30	2.82	2.94	2.73	2.75	18047152	{}	\N	2020-10-12 14:17:05.367467
27212	AR	2020-10-01	2.79	2.85	2.68	2.7	17133153	{}	\N	2020-10-12 14:17:05.373159
27213	AR	2020-10-02	2.61	2.825	2.555	2.79	15067007	{}	\N	2020-10-12 14:17:05.378861
27214	AR	2020-10-05	2.82	3.05	2.82	3.04	19854503	{}	\N	2020-10-12 14:17:05.383508
27215	AR	2020-10-06	3.08	3.235	3.03	3.11	18975085	{}	\N	2020-10-12 14:17:05.38857
27216	AR	2020-10-07	3.16	3.455	3.16	3.41	21433944	{}	\N	2020-10-12 14:17:05.392951
27217	AR	2020-10-08	3.47	3.67	3.41	3.65	16266223	{}	\N	2020-10-12 14:17:05.397517
27218	AR	2020-10-09	3.74	3.89	3.425	3.71	23596119	{}	\N	2020-10-12 14:17:05.402061
27219	AR	2020-10-12	3.72	3.76	3.635	3.675	2316040	{}	\N	2020-10-12 14:17:05.406163
27220	ARD	2020-09-28	14.01	14.27	13.86	13.95	50007	{}	\N	2020-10-12 14:17:06.487254
27221	ARD	2020-09-29	13.9	14.53	13.89	14.33	114868	{}	\N	2020-10-12 14:17:06.497288
27222	ARD	2020-09-30	14.36	14.38	13.945	14.05	49985	{}	\N	2020-10-12 14:17:06.504432
27223	ARD	2020-10-01	14.11	14.42	14.06	14.17	76824	{}	\N	2020-10-12 14:17:06.50998
27224	ARD	2020-10-02	14.17	14.5492	14	14.05	88429	{}	\N	2020-10-12 14:17:06.514682
27225	ARD	2020-10-05	14.34	14.61	14.1	14.19	112141	{}	\N	2020-10-12 14:17:06.51958
27226	ARD	2020-10-06	14.12	14.295	13.5	13.61	235266	{}	\N	2020-10-12 14:17:06.524034
27227	ARD	2020-10-07	13.62	13.85	13.365	13.48	108870	{}	\N	2020-10-12 14:17:06.527994
27228	ARD	2020-10-08	13.48	13.62	13.4	13.57	127804	{}	\N	2020-10-12 14:17:06.531923
27229	ARD	2020-10-09	13.66	13.97	13.66	13.77	102402	{}	\N	2020-10-12 14:17:06.536102
27230	ARD	2020-10-12	13.93	13.9929	13.8	13.9	16822	{}	\N	2020-10-12 14:17:06.540006
27231	AROC	2020-09-28	5.53	5.735	5.51	5.67	1467939	{}	\N	2020-10-12 14:17:08.089693
27232	AROC	2020-09-29	5.64	5.66	5.425	5.61	2136216	{}	\N	2020-10-12 14:17:08.099469
27233	AROC	2020-09-30	5.63	5.765	5.32	5.38	2308297	{}	\N	2020-10-12 14:17:08.105825
27234	AROC	2020-10-01	5.28	5.46	5.265	5.44	1764508	{}	\N	2020-10-12 14:17:08.111389
27235	AROC	2020-10-02	5.28	5.62	5.23	5.56	1649458	{}	\N	2020-10-12 14:17:08.115648
27236	AROC	2020-10-05	5.67	5.78	5.585	5.67	951537	{}	\N	2020-10-12 14:17:08.120294
27237	AROC	2020-10-06	5.8	5.84	5.56	5.57	2070482	{}	\N	2020-10-12 14:17:08.124594
27238	AROC	2020-10-07	5.62	5.72	5.47	5.67	1790677	{}	\N	2020-10-12 14:17:08.128545
27239	AROC	2020-10-08	5.77	5.93	5.71	5.9	1146746	{}	\N	2020-10-12 14:17:08.132358
27240	AROC	2020-10-09	5.98	5.99	5.79	5.82	1184284	{}	\N	2020-10-12 14:17:08.136422
27241	AROC	2020-10-12	5.82	5.835	5.725	5.725	184808	{}	\N	2020-10-12 14:17:08.140121
27242	ARWR	2020-09-28	45.17	45.17	42.4	43.68	1068686	{}	\N	2020-10-12 14:17:09.106638
27243	ARWR	2020-09-29	43.61	44.29	43.22	43.42	801885	{}	\N	2020-10-12 14:17:09.115344
27244	ARWR	2020-09-30	43.745	44.36	42.65	43.06	865530	{}	\N	2020-10-12 14:17:09.121137
27245	ARWR	2020-10-01	43.12	45.44	42.97	44.76	960039	{}	\N	2020-10-12 14:17:09.126327
27246	ARWR	2020-10-02	43.75	44.7	42.44	43.82	957846	{}	\N	2020-10-12 14:17:09.130982
27247	ARWR	2020-10-05	44.19	47.27	44.19	47.11	1121633	{}	\N	2020-10-12 14:17:09.136504
27248	ARWR	2020-10-06	46.97	48.4599	46.08	46.405	845550	{}	\N	2020-10-12 14:17:09.140283
27249	ARWR	2020-10-07	46.42	48.34	46.2079	48.1	510922	{}	\N	2020-10-12 14:17:09.143754
27250	ARWR	2020-10-08	51.3425	51.3425	47.24	48.01	1879609	{}	\N	2020-10-12 14:17:09.14745
27251	ARWR	2020-10-09	48.195	48.59	45.79	46.05	974744	{}	\N	2020-10-12 14:17:09.151324
27252	ARWR	2020-10-12	46.67	47.01	45.14	46.59	129160	{}	\N	2020-10-12 14:17:09.15529
27253	ASH	2020-09-28	70.55	72.34	70.43	71.85	442784	{}	\N	2020-10-12 14:17:10.669621
27254	ASH	2020-09-29	72.29	72.575	70.48	70.53	844059	{}	\N	2020-10-12 14:17:10.678931
27255	ASH	2020-09-30	70.83	71.35	69.995	70.92	1872860	{}	\N	2020-10-12 14:17:10.684509
27256	ASH	2020-10-01	71.03	72.03	70.55	71.09	1326016	{}	\N	2020-10-12 14:17:10.68973
27257	ASH	2020-10-02	70.03	72.01	69.95	70.92	610728	{}	\N	2020-10-12 14:17:10.69469
27258	ASH	2020-10-05	71.85	73.15	71.85	72.44	527119	{}	\N	2020-10-12 14:17:10.699152
27259	ASH	2020-10-06	72.82	73.92	71.68	71.87	537714	{}	\N	2020-10-12 14:17:10.703053
27260	ASH	2020-10-07	72.79	74.32	72.79	73.14	485707	{}	\N	2020-10-12 14:17:10.707244
27261	ASH	2020-10-08	73.86	74.82	73.0825	73.6	409865	{}	\N	2020-10-12 14:17:10.710996
27262	ASH	2020-10-09	74.54	75.49	73.69	74.39	712546	{}	\N	2020-10-12 14:17:10.714734
27263	ASH	2020-10-12	74.64	74.64	73.66	74.24	27343	{}	\N	2020-10-12 14:17:10.718435
27264	ASRV	2020-09-28	2.71	2.765	2.7	2.74	6531	{}	\N	2020-10-12 14:17:11.832479
27265	ASRV	2020-09-29	2.77	2.82	2.715	2.82	13646	{}	\N	2020-10-12 14:17:11.8413
27266	ASRV	2020-09-30	2.8199	2.83	2.775	2.81	9831	{}	\N	2020-10-12 14:17:11.846917
27267	ASRV	2020-10-01	2.732	2.7918	2.7	2.7918	2711	{}	\N	2020-10-12 14:17:11.853008
27268	ASRV	2020-10-02	2.68	2.726	2.68	2.69	7409	{}	\N	2020-10-12 14:17:11.857872
27269	ASRV	2020-10-05	2.69	2.8	2.666	2.77	14396	{}	\N	2020-10-12 14:17:11.862201
27270	ASRV	2020-10-06	2.79	2.92	2.7787	2.8	10580	{}	\N	2020-10-12 14:17:11.865896
27271	ASRV	2020-10-07	2.6901	2.895	2.6901	2.88	3019	{}	\N	2020-10-12 14:17:11.869772
27272	ASRV	2020-10-08	2.88	2.88	2.85	2.88	3484	{}	\N	2020-10-12 14:17:11.873524
27273	ASRV	2020-10-09	2.8673	2.95	2.86	2.88	3727	{}	\N	2020-10-12 14:17:11.87709
27274	ASRV	2020-10-12	2.9081	2.915	2.9081	2.915	273	{}	\N	2020-10-12 14:17:11.880556
27275	ASX	2020-09-28	4.096	4.16	4.096	4.16	401359	{}	\N	2020-10-12 14:17:12.880388
27276	ASX	2020-09-29	4.04	4.05	4.02	4.02	1191391	{}	\N	2020-10-12 14:17:12.888177
27277	ASX	2020-09-30	4.07	4.13	4.05	4.08	878801	{}	\N	2020-10-12 14:17:12.894086
27278	ASX	2020-10-01	4.15	4.165	4.115	4.15	1070246	{}	\N	2020-10-12 14:17:12.900126
27279	ASX	2020-10-02	4.12	4.12	4.02	4.03	824565	{}	\N	2020-10-12 14:17:12.904996
27280	ASX	2020-10-05	4.12	4.16	4.12	4.14	1479343	{}	\N	2020-10-12 14:17:12.909369
27281	ASX	2020-10-06	4.16	4.18	4.09	4.09	865695	{}	\N	2020-10-12 14:17:12.913954
27282	ASX	2020-10-07	4.18	4.2	4.15	4.19	527567	{}	\N	2020-10-12 14:17:12.918009
27283	ASX	2020-10-08	4.27	4.36	4.26	4.33	1110937	{}	\N	2020-10-12 14:17:12.922067
27284	ASX	2020-10-09	4.36	4.36	4.31	4.33	759374	{}	\N	2020-10-12 14:17:12.92556
27285	ASX	2020-10-12	4.32	4.33	4.3	4.31	109008	{}	\N	2020-10-12 14:17:12.929565
27286	ATH	2020-09-28	34.39	35.18	34.22	34.87	1197418	{}	\N	2020-10-12 14:17:14.039019
27287	ATH	2020-09-29	34.6	34.82	33.66	34	1895949	{}	\N	2020-10-12 14:17:14.046738
27288	ATH	2020-09-30	34.15	34.53	33.56	34.08	2504551	{}	\N	2020-10-12 14:17:14.053421
27289	ATH	2020-10-01	35.11	35.8	34.55	35.64	3614218	{}	\N	2020-10-12 14:17:14.058737
27290	ATH	2020-10-02	34.86	36.45	34.65	36.22	6924333	{}	\N	2020-10-12 14:17:14.063655
27291	ATH	2020-10-05	36.78	37.59	36.71	37	2826914	{}	\N	2020-10-12 14:17:14.068256
27292	ATH	2020-10-06	37.38	37.38	35.97	36.29	2734841	{}	\N	2020-10-12 14:17:14.072471
27293	ATH	2020-10-07	36.88	37.28	36.43	36.86	2534793	{}	\N	2020-10-12 14:17:14.076619
27294	ATH	2020-10-08	37.08	37.6	36.735	37.39	1765448	{}	\N	2020-10-12 14:17:14.080764
27295	ATH	2020-10-09	37.77	38	36.82	37.01	1262258	{}	\N	2020-10-12 14:17:14.08494
27296	ATH	2020-10-12	37.01	37.22	36.82	37.115	67561	{}	\N	2020-10-12 14:17:14.089007
27297	ATNM	2020-09-28	9.35	9.35	8.56	8.77	206203	{}	\N	2020-10-12 14:17:14.982526
27298	ATNM	2020-09-29	8.88	9.43	8.88	9.34	247573	{}	\N	2020-10-12 14:17:14.990388
27299	ATNM	2020-09-30	9.38	9.99	9.3	9.69	360356	{}	\N	2020-10-12 14:17:14.996055
27300	ATNM	2020-10-01	9.7	9.7	9.18	9.34	223717	{}	\N	2020-10-12 14:17:15.002782
27301	ATNM	2020-10-02	9.01	9.42	9	9.18	158016	{}	\N	2020-10-12 14:17:15.007387
27302	ATNM	2020-10-05	9.21	9.78	9.15	9.6	173956	{}	\N	2020-10-12 14:17:15.01195
27303	ATNM	2020-10-06	9.59	10	9.5713	9.89	263532	{}	\N	2020-10-12 14:17:15.01577
27304	ATNM	2020-10-07	9.91	10.1	9.8	10.03	141702	{}	\N	2020-10-12 14:17:15.020077
27305	ATNM	2020-10-08	10.27	10.4602	10	10.15	186695	{}	\N	2020-10-12 14:17:15.02406
27306	ATNM	2020-10-09	10.09	10.59	10	10.54	205323	{}	\N	2020-10-12 14:17:15.028005
27307	ATNM	2020-10-12	10.61	10.64	10.26	10.64	42011	{}	\N	2020-10-12 14:17:15.031952
27308	ATSG	2020-09-28	24.93	25.7299	24.93	25.51	308096	{}	\N	2020-10-12 14:17:16.144115
27309	ATSG	2020-09-29	25.46	25.54	25.095	25.34	201046	{}	\N	2020-10-12 14:17:16.153311
27310	ATSG	2020-09-30	25.57	25.6	25	25.06	271666	{}	\N	2020-10-12 14:17:16.159537
27311	ATSG	2020-10-01	25.1	25.48	24.97	25.33	220559	{}	\N	2020-10-12 14:17:16.165218
27312	ATSG	2020-10-02	24.84	25.59	24.62	25.5	232365	{}	\N	2020-10-12 14:17:16.171487
27313	ATSG	2020-10-05	25.71	26.3	25.55	26.27	483336	{}	\N	2020-10-12 14:17:16.1766
27314	ATSG	2020-10-06	26.45	26.7799	25.73	25.82	475612	{}	\N	2020-10-12 14:17:16.181114
27315	ATSG	2020-10-07	26.1	26.49	25.92	26.35	390632	{}	\N	2020-10-12 14:17:16.185516
27316	ATSG	2020-10-08	26.59	26.725	26.23	26.39	194456	{}	\N	2020-10-12 14:17:16.18995
27317	ATSG	2020-10-09	26.49	26.6	26.235	26.55	215103	{}	\N	2020-10-12 14:17:16.195138
27318	ATSG	2020-10-12	27.23	27.68	27	27.42	87608	{}	\N	2020-10-12 14:17:16.19961
27319	ATXI	2020-09-28	11.05	11.21	10.55	10.91	147156	{}	\N	2020-10-12 14:17:17.851654
27320	ATXI	2020-09-29	10.81	11.1499	10.81	10.9	42462	{}	\N	2020-10-12 14:17:17.860672
27321	ATXI	2020-09-30	10.83	10.91	10.7	10.83	97379	{}	\N	2020-10-12 14:17:17.866487
27322	ATXI	2020-10-01	10.93	11.34	10.65	11.01	128857	{}	\N	2020-10-12 14:17:17.872504
27323	ATXI	2020-10-02	11	11.0896	10.865	10.96	75089	{}	\N	2020-10-12 14:17:17.877627
27324	ATXI	2020-10-05	11.09	11.27	10.7	10.91	106302	{}	\N	2020-10-12 14:17:17.882033
27325	ATXI	2020-10-06	10.955	10.96	10.5	10.79	79437	{}	\N	2020-10-12 14:17:17.88624
27326	ATXI	2020-10-07	10.66	11.1	10.6064	10.96	117237	{}	\N	2020-10-12 14:17:17.890482
27327	ATXI	2020-10-08	10.92	11.105	10.36	10.58	148455	{}	\N	2020-10-12 14:17:17.89443
27328	ATXI	2020-10-09	10.79	11.08	10.58	11.04	163635	{}	\N	2020-10-12 14:17:17.898497
27329	ATXI	2020-10-12	5.19	5.5	4.78	4.8501	2863956	{}	\N	2020-10-12 14:17:17.902581
27330	AVAV	2020-09-28	62.27	63.31	61.38	61.52	112199	{}	\N	2020-10-12 14:17:19.244996
27331	AVAV	2020-09-29	61.19	62.33	61.19	61.91	103522	{}	\N	2020-10-12 14:17:19.256921
27332	AVAV	2020-09-30	62.1	62.39	59.58	60.01	167548	{}	\N	2020-10-12 14:17:19.263565
27333	AVAV	2020-10-01	60.21	62.04	60.21	61.54	175436	{}	\N	2020-10-12 14:17:19.268924
27334	AVAV	2020-10-02	60.2	63.17	60.08	62.97	171547	{}	\N	2020-10-12 14:17:19.27307
27335	AVAV	2020-10-05	63.7	63.98	62.83	63.55	152234	{}	\N	2020-10-12 14:17:19.27737
27336	AVAV	2020-10-06	64.11	66.02	64.01	64.09	162719	{}	\N	2020-10-12 14:17:19.281103
27337	AVAV	2020-10-07	64.62	65.83	64.62	65.24	156379	{}	\N	2020-10-12 14:17:19.28458
27338	AVAV	2020-10-08	69.99	74.56	68.21	69.5	560705	{}	\N	2020-10-12 14:17:19.288337
27339	AVAV	2020-10-09	70.455	71.775	69.76	71.14	237245	{}	\N	2020-10-12 14:17:19.2919
27340	AVAV	2020-10-12	71.22	72.2901	71.1	72.18	23205	{}	\N	2020-10-12 14:17:19.295833
27341	AVID	2020-09-28	8.3	8.5375	8.28	8.45	612912	{}	\N	2020-10-12 14:17:20.273498
27342	AVID	2020-09-29	8.45	8.94	8.425	8.82	640302	{}	\N	2020-10-12 14:17:20.282418
27343	AVID	2020-09-30	8.81	8.925	8.495	8.56	559156	{}	\N	2020-10-12 14:17:20.288603
27344	AVID	2020-10-01	8.61	8.66	8.35	8.36	422482	{}	\N	2020-10-12 14:17:20.294535
27345	AVID	2020-10-02	8.2	8.44	8.0492	8.37	333013	{}	\N	2020-10-12 14:17:20.299781
27346	AVID	2020-10-05	8.46	8.7	8.4501	8.6	463151	{}	\N	2020-10-12 14:17:20.304815
27347	AVID	2020-10-06	8.72	8.92	8.53	8.53	418785	{}	\N	2020-10-12 14:17:20.309346
27348	AVID	2020-10-07	8.7	8.87	8.6201	8.76	207779	{}	\N	2020-10-12 14:17:20.313417
27349	AVID	2020-10-08	8.9	9.15	8.87	9.04	413059	{}	\N	2020-10-12 14:17:20.317841
27350	AVID	2020-10-09	9.05	9.17	8.85	8.91	322031	{}	\N	2020-10-12 14:17:20.322043
27351	AVID	2020-10-12	8.87	9.02	8.76	9.02	66389	{}	\N	2020-10-12 14:17:20.329492
27352	AVY	2020-09-28	120.76	123.86	120.525	123.01	557980	{}	\N	2020-10-12 14:17:21.311437
27353	AVY	2020-09-29	123.64	125.34	123.24	124.59	1276390	{}	\N	2020-10-12 14:17:21.320101
27354	AVY	2020-09-30	125.36	129.4	125.05	127.84	1702799	{}	\N	2020-10-12 14:17:21.325817
27355	AVY	2020-10-01	128.71	129.5	124.31	124.65	1279471	{}	\N	2020-10-12 14:17:21.338499
27356	AVY	2020-10-02	122.79	125.7	122.56	124.33	1366454	{}	\N	2020-10-12 14:17:21.343095
27357	AVY	2020-10-05	125.96	128.72	125.8	128.22	1494650	{}	\N	2020-10-12 14:17:21.34762
27358	AVY	2020-10-06	129.28	130.6	125.75	126.34	1496421	{}	\N	2020-10-12 14:17:21.352915
27359	AVY	2020-10-07	128.09	131.46	127.96	129.56	778755	{}	\N	2020-10-12 14:17:21.358957
27360	AVY	2020-10-08	130.03	132.18	130.03	132.06	819786	{}	\N	2020-10-12 14:17:21.363825
27361	AVY	2020-10-09	133.04	133.22	130.18	130.5	929770	{}	\N	2020-10-12 14:17:21.36806
27362	AVY	2020-10-12	131.87	131.87	129.5	130.73	234625	{}	\N	2020-10-12 14:17:21.372489
27363	ARAY	2020-09-28	2.38	2.5	2.34	2.45	465048	{}	\N	2020-10-12 14:17:23.179124
27364	ARAY	2020-09-29	2.58	2.6	2.35	2.4	488873	{}	\N	2020-10-12 14:17:23.189082
27365	ARAY	2020-09-30	2.43	2.48	2.37	2.4	2128667	{}	\N	2020-10-12 14:17:23.196189
27366	ARAY	2020-10-01	2.37	2.59	2.37	2.53	558141	{}	\N	2020-10-12 14:17:23.20275
27367	ARAY	2020-10-02	2.43	2.59	2.43	2.55	1688826	{}	\N	2020-10-12 14:17:23.207176
27368	ARAY	2020-10-05	2.58	2.73	2.57	2.65	648733	{}	\N	2020-10-12 14:17:23.211827
27369	ARAY	2020-10-06	2.67	2.88	2.67	2.68	1533904	{}	\N	2020-10-12 14:17:23.216058
27370	ARAY	2020-10-07	2.81	2.89	2.72	2.76	564791	{}	\N	2020-10-12 14:17:23.220312
27371	ARAY	2020-10-08	2.8	2.9	2.78	2.86	800794	{}	\N	2020-10-12 14:17:23.224351
27372	ARAY	2020-10-09	2.88	2.9	2.69	2.74	907261	{}	\N	2020-10-12 14:17:23.228362
27373	ARAY	2020-10-12	2.78	2.7955	2.7402	2.745	114747	{}	\N	2020-10-12 14:17:23.232294
27374	ARKK	2020-09-28	92.92	93.05	90.96	91.72	1986630	{}	\N	2020-10-12 14:17:25.010841
27375	ARKK	2020-09-29	91.76	93.38	91.74	92.77	1558720	{}	\N	2020-10-12 14:17:25.019897
27376	ARKK	2020-09-30	92.85	93.5481	91.32	92	2883236	{}	\N	2020-10-12 14:17:25.025059
27377	ARKK	2020-10-01	93.39	94.54	92.9201	94.54	2382305	{}	\N	2020-10-12 14:17:25.031086
27378	ARKK	2020-10-02	91.56	94.06	91.55	92.285	2759652	{}	\N	2020-10-12 14:17:25.036122
27379	ARKK	2020-10-05	93.56	95.96	93.56	95.89	2015842	{}	\N	2020-10-12 14:17:25.040462
27380	ARKK	2020-10-06	95.94	97.91	94.88	95.67	2567542	{}	\N	2020-10-12 14:17:25.044392
27381	ARKK	2020-10-07	97.1	100.02	97.1	99.21	3168970	{}	\N	2020-10-12 14:17:25.048204
27382	ARKK	2020-10-08	101.8599	101.8599	100.19	100.57	2574787	{}	\N	2020-10-12 14:17:25.052352
27383	ARKK	2020-10-09	101.66	103.03	101.32	102.76	2773657	{}	\N	2020-10-12 14:17:25.056066
27384	ARKK	2020-10-12	104.77	105	103.01	103.52	1214900	{}	\N	2020-10-12 14:17:25.059803
27385	ASIX	2020-09-28	12.37	13.05	12.37	12.63	172696	{}	\N	2020-10-12 14:17:27.597357
27386	ASIX	2020-09-29	12.58	12.75	12.3453	12.63	305216	{}	\N	2020-10-12 14:17:27.607175
27387	ASIX	2020-09-30	12.75	13.37	12.75	12.88	345158	{}	\N	2020-10-12 14:17:27.613915
27388	ASIX	2020-10-01	12.87	13.19	12.75	13.14	323782	{}	\N	2020-10-12 14:17:27.619156
27389	ASIX	2020-10-02	12.84	13.77	12.84	13.68	233858	{}	\N	2020-10-12 14:17:27.62361
27390	ASIX	2020-10-05	13.88	14.23	13.86	13.97	175966	{}	\N	2020-10-12 14:17:27.628369
27391	ASIX	2020-10-06	14.2	14.29	13.93	13.95	348863	{}	\N	2020-10-12 14:17:27.632428
27392	ASIX	2020-10-07	14.1	14.41	14.01	14.22	258854	{}	\N	2020-10-12 14:17:27.636391
27393	ASIX	2020-10-08	14.43	14.61	14.25	14.53	172268	{}	\N	2020-10-12 14:17:27.640697
27394	ASIX	2020-10-09	14.69	14.69	14.31	14.33	217556	{}	\N	2020-10-12 14:17:27.645091
27395	ASIX	2020-10-12	14.23	14.26	14.1	14.26	30338	{}	\N	2020-10-12 14:17:27.650245
27396	ASPS	2020-09-28	12.18	13.57	12.18	12.51	121825	{}	\N	2020-10-12 14:17:29.041688
27397	ASPS	2020-09-29	12.47	13.445	12.29	13.1	95909	{}	\N	2020-10-12 14:17:29.050658
27398	ASPS	2020-09-30	13.1	13.32	12.52	12.67	44353	{}	\N	2020-10-12 14:17:29.057313
27399	ASPS	2020-10-01	12.69	12.98	12.19	12.92	45414	{}	\N	2020-10-12 14:17:29.062411
27400	ASPS	2020-10-02	12.58	13.2	12.12	13.14	42909	{}	\N	2020-10-12 14:17:29.066881
27401	ASPS	2020-10-05	13.39	13.395	12.5	12.91	76063	{}	\N	2020-10-12 14:17:29.071634
27402	ASPS	2020-10-06	12.99	13.55	12.81	13.11	76011	{}	\N	2020-10-12 14:17:29.075631
27403	ASPS	2020-10-07	13.22	13.37	12.92	13.13	54592	{}	\N	2020-10-12 14:17:29.079185
27404	ASPS	2020-10-08	13.1588	13.48	12.95	13.11	59587	{}	\N	2020-10-12 14:17:29.08274
27405	ASPS	2020-10-09	13.12	13.62	12.97	13.38	47941	{}	\N	2020-10-12 14:17:29.086425
27406	ASPS	2020-10-12	13.27	13.54	13.16	13.54	7152	{}	\N	2020-10-12 14:17:29.090068
27407	ASTE	2020-09-28	51.68	53.76	51.68	53.28	218488	{}	\N	2020-10-12 14:17:30.126798
27408	ASTE	2020-09-29	53.14	53.8	52.4	53.67	110482	{}	\N	2020-10-12 14:17:30.136223
27409	ASTE	2020-09-30	54.05	55.01	53.16	54.25	145473	{}	\N	2020-10-12 14:17:30.142422
27410	ASTE	2020-10-01	53.35	54.53	52.25	53.78	187932	{}	\N	2020-10-12 14:17:30.148306
27411	ASTE	2020-10-02	52.78	54.97	52.78	54.83	134944	{}	\N	2020-10-12 14:17:30.153458
27412	ASTE	2020-10-05	55.03	56.91	54.71	56.8	237017	{}	\N	2020-10-12 14:17:30.158079
27413	ASTE	2020-10-06	56.405	59.96	56.405	57.62	198386	{}	\N	2020-10-12 14:17:30.162171
27414	ASTE	2020-10-07	58.47	59.55	57.17	58.21	179830	{}	\N	2020-10-12 14:17:30.166412
27415	ASTE	2020-10-08	59.35	59.35	57.435	59.06	105827	{}	\N	2020-10-12 14:17:30.171865
27416	ASTE	2020-10-09	58.99	59.3	57.96	58.48	112516	{}	\N	2020-10-12 14:17:30.175842
27417	ASTE	2020-10-12	58.97	59.25	58.28	59.25	8234	{}	\N	2020-10-12 14:17:30.179848
27418	ATMP	2020-09-28	9.1	9.24	9.06	9.18	313326	{}	\N	2020-10-12 14:17:31.367635
27419	ATMP	2020-09-29	9.18	9.29	9.11	9.25	123251	{}	\N	2020-10-12 14:17:31.37501
27420	ATMP	2020-09-30	9.26	9.39	9.04	9.05	231260	{}	\N	2020-10-12 14:17:31.380629
27421	ATMP	2020-10-01	9.04	9.04	8.89	8.95	75372	{}	\N	2020-10-12 14:17:31.38711
27422	ATMP	2020-10-02	8.93	9.1123	8.93	9.08	59941	{}	\N	2020-10-12 14:17:31.391821
27423	ATMP	2020-10-05	9.38	9.45	9.21	9.45	63441	{}	\N	2020-10-12 14:17:31.396535
27424	ATMP	2020-10-06	9.55	9.5707	9.2	9.23	44251	{}	\N	2020-10-12 14:17:31.401356
27425	ATMP	2020-10-07	9.35	9.35	9.22	9.29	14126	{}	\N	2020-10-12 14:17:31.406058
27426	ATMP	2020-10-08	9.44	9.86	9.41	9.85	106860	{}	\N	2020-10-12 14:17:31.410766
27427	ATMP	2020-10-09	9.87	9.94	9.76	9.86	114124	{}	\N	2020-10-12 14:17:31.415454
27428	ATMP	2020-10-12	10.01	10.01	9.81	9.81	1182	{}	\N	2020-10-12 14:17:31.420028
27429	ATRI	2020-09-28	632.2	672.98	632.2	634.99	8860	{}	\N	2020-10-12 14:17:32.385325
27430	ATRI	2020-09-29	630.2	632.98	621.1	622.1	10157	{}	\N	2020-10-12 14:17:32.396557
27431	ATRI	2020-09-30	630	633.35	626	626	10482	{}	\N	2020-10-12 14:17:32.403714
27432	ATRI	2020-10-01	631	631	608.005	611.1	11910	{}	\N	2020-10-12 14:17:32.409405
27433	ATRI	2020-10-02	609	610.51	581.0001	605	13383	{}	\N	2020-10-12 14:17:32.414728
27434	ATRI	2020-10-05	613	625.5	608	625.5	9457	{}	\N	2020-10-12 14:17:32.419897
27435	ATRI	2020-10-06	628	638.24	618.7155	621	6696	{}	\N	2020-10-12 14:17:32.424008
27436	ATRI	2020-10-07	630	632	623	629.11	8611	{}	\N	2020-10-12 14:17:32.428402
27437	ATRI	2020-10-08	630.69	630.69	623.01	624.21	9381	{}	\N	2020-10-12 14:17:32.433162
27438	ATRI	2020-10-09	629.9	645.91	626.1	640.5	13109	{}	\N	2020-10-12 14:17:32.437281
27439	ATRI	2020-10-12	638.35	649.91	638.35	649.61	1921	{}	\N	2020-10-12 14:17:32.441439
27440	AU	2020-09-28	26.31	26.33	25.555	25.82	2248893	{}	\N	2020-10-12 14:17:34.102645
27441	AU	2020-09-29	26.08	26.65	25.84	26.46	4202944	{}	\N	2020-10-12 14:17:34.111075
27442	AU	2020-09-30	26.38	26.58	25.91	26.38	4074450	{}	\N	2020-10-12 14:17:34.117796
27443	AU	2020-10-01	26.55	26.72	26.13	26.44	3960489	{}	\N	2020-10-12 14:17:34.122774
27444	AU	2020-10-02	25.99	26.54	25.75	26.01	2998546	{}	\N	2020-10-12 14:17:34.127267
27445	AU	2020-10-05	26.18	26.74	25.97	26.06	2895286	{}	\N	2020-10-12 14:17:34.131866
27446	AU	2020-10-06	26.45	26.62	25.27	25.32	4933090	{}	\N	2020-10-12 14:17:34.136516
27447	AU	2020-10-07	25.66	25.84	25.18	25.49	3167914	{}	\N	2020-10-12 14:17:34.140595
27448	AU	2020-10-08	26.21	26.65	26.01	26.53	4129839	{}	\N	2020-10-12 14:17:34.144188
27449	AU	2020-10-09	27.49	28.02	27.33	27.94	6093336	{}	\N	2020-10-12 14:17:34.148271
27450	AU	2020-10-12	28.15	28.21	27.63	28.09	971123	{}	\N	2020-10-12 14:17:34.153033
27451	AVD	2020-09-28	13.05	13.86	13.05	13.35	111912	{}	\N	2020-10-12 14:17:36.299025
27452	AVD	2020-09-29	13.39	13.49	13.19	13.44	116414	{}	\N	2020-10-12 14:17:36.308115
27453	AVD	2020-09-30	13.37	13.64	13.015	13.14	247292	{}	\N	2020-10-12 14:17:36.315911
27454	AVD	2020-10-01	13.15	13.3	13.06	13.13	254863	{}	\N	2020-10-12 14:17:36.321252
27455	AVD	2020-10-02	12.91	13.41	12.91	13.22	186003	{}	\N	2020-10-12 14:17:36.328638
27456	AVD	2020-10-05	13.44	14.16	13.44	14.02	201993	{}	\N	2020-10-12 14:17:36.334391
27457	AVD	2020-10-06	14.17	14.49	13.81	13.81	218678	{}	\N	2020-10-12 14:17:36.33871
27458	AVD	2020-10-07	14.03	14.105	13.74	14	178635	{}	\N	2020-10-12 14:17:36.342656
27459	AVD	2020-10-08	14.29	14.73	14.29	14.62	122289	{}	\N	2020-10-12 14:17:36.346513
27460	AVD	2020-10-09	14.84	15.1	14.73	14.84	101452	{}	\N	2020-10-12 14:17:36.350758
27461	AVD	2020-10-12	14.82	14.92	14.82	14.92	5980	{}	\N	2020-10-12 14:17:36.355005
27462	AVNW	2020-09-28	20.83	21.25	20.55	20.81	17502	{}	\N	2020-10-12 14:17:37.372111
27463	AVNW	2020-09-29	20.6157	23.5	20.6157	22.78	89531	{}	\N	2020-10-12 14:17:37.381477
27464	AVNW	2020-09-30	22.78	23.3	21.9	21.96	23096	{}	\N	2020-10-12 14:17:37.387457
27465	AVNW	2020-10-01	22.15	22.5	21.54	21.86	29287	{}	\N	2020-10-12 14:17:37.392321
27466	AVNW	2020-10-02	21.64	21.95	18.5	21.11	17943	{}	\N	2020-10-12 14:17:37.396557
27467	AVNW	2020-10-05	21.27	22.44	21.14	21.7	13142	{}	\N	2020-10-12 14:17:37.401392
27468	AVNW	2020-10-06	21.6	22.04	21.01	21.61	30418	{}	\N	2020-10-12 14:17:37.405314
27469	AVNW	2020-10-07	21.52	22.105	20.9001	21.51	11388	{}	\N	2020-10-12 14:17:37.408766
27470	AVNW	2020-10-08	21.93	22.465	21.6001	22.17	24218	{}	\N	2020-10-12 14:17:37.412152
27471	AVNW	2020-10-09	22.4	22.655	21.69	22.18	20058	{}	\N	2020-10-12 14:17:37.415616
27472	AVNW	2020-10-12	23.87	23.87	22.23	23.115	19739	{}	\N	2020-10-12 14:17:37.419196
27473	AWK	2020-09-28	144.47	146.06	143.65	144.55	593349	{}	\N	2020-10-12 14:17:38.302334
27474	AWK	2020-09-29	144.75	145.1	142.88	143.7	1114391	{}	\N	2020-10-12 14:17:38.310033
27475	AWK	2020-09-30	144.63	146.28	144.14	144.88	2116802	{}	\N	2020-10-12 14:17:38.315765
27476	AWK	2020-10-01	145.56	148.86	145.44	148.57	2854446	{}	\N	2020-10-12 14:17:38.320758
27477	AWK	2020-10-02	147.52	149.97	146.87	148.96	1422912	{}	\N	2020-10-12 14:17:38.32967
27478	AWK	2020-10-05	149.25	152.155	148.59	151.89	1378220	{}	\N	2020-10-12 14:17:38.335449
27479	AWK	2020-10-06	151.85	153.74	150.87	152.49	1435838	{}	\N	2020-10-12 14:17:38.339846
27480	AWK	2020-10-07	153.94	155.46	152.91	154.51	1827202	{}	\N	2020-10-12 14:17:38.343823
27481	AWK	2020-10-08	155	156.39	154.08	155.85	1109238	{}	\N	2020-10-12 14:17:38.347831
27482	AWK	2020-10-09	156.21	157.27	154.6067	155.75	1304719	{}	\N	2020-10-12 14:17:38.352626
27483	AWK	2020-10-12	156.65	158.41	156.29	158.02	128163	{}	\N	2020-10-12 14:17:38.357004
27484	FEYE	2020-09-28	12.84	12.9701	12.74	12.855	2165356	{}	\N	2020-10-12 14:17:39.289989
27485	FEYE	2020-09-29	12.87	12.9	12.53	12.615	2032356	{}	\N	2020-10-12 14:17:39.301887
27486	FEYE	2020-09-30	12.5785	12.735	12.31	12.345	2723186	{}	\N	2020-10-12 14:17:39.309152
27487	FEYE	2020-10-01	12.42	12.69	12.37	12.63	2834414	{}	\N	2020-10-12 14:17:39.314644
27488	FEYE	2020-10-02	12.31	12.82	12.3	12.38	2591822	{}	\N	2020-10-12 14:17:39.319698
27489	FEYE	2020-10-05	12.48	12.77	12.455	12.655	1932144	{}	\N	2020-10-12 14:17:39.324604
27490	FEYE	2020-10-06	12.7	13.06	12.64	12.68	1754272	{}	\N	2020-10-12 14:17:39.333415
27491	FEYE	2020-10-07	12.75	13.175	12.71	13.14	4463634	{}	\N	2020-10-12 14:17:39.337823
27492	FEYE	2020-10-08	13.23	13.33	13.1001	13.21	2604411	{}	\N	2020-10-12 14:17:39.341921
27493	FEYE	2020-10-09	13.33	13.6	13.3	13.595	2360944	{}	\N	2020-10-12 14:17:39.345882
27494	FEYE	2020-10-12	13.67	13.79	13.58	13.765	674733	{}	\N	2020-10-12 14:17:39.350129
27495	ARE	2020-09-28	160.54	162.45	159.85	161.35	1309110	{}	\N	2020-10-12 14:17:40.305874
27496	ARE	2020-09-29	159.52	161.96	159	159.93	1397400	{}	\N	2020-10-12 14:17:40.316935
27497	ARE	2020-09-30	160.94	163.14	158.3	160	3170018	{}	\N	2020-10-12 14:17:40.323552
27498	ARE	2020-10-01	160.6	163.22	160.045	163.19	1205037	{}	\N	2020-10-12 14:17:40.335358
27499	ARE	2020-10-02	161.67	165.3	160.565	164.71	1432765	{}	\N	2020-10-12 14:17:40.341857
27500	ARE	2020-10-05	166.06	167.14	163.4	167	1243007	{}	\N	2020-10-12 14:17:40.347822
27501	ARE	2020-10-06	167.37	167.88	164.32	165.1	1237044	{}	\N	2020-10-12 14:17:40.35309
27502	ARE	2020-10-07	165.29	165.85	163.09	163.12	1281405	{}	\N	2020-10-12 14:17:40.35784
27503	ARE	2020-10-08	163.59	166.99	163.29	166.09	1169751	{}	\N	2020-10-12 14:17:40.362068
27504	ARE	2020-10-09	167.11	167.11	164.56	165.15	1031141	{}	\N	2020-10-12 14:17:40.366258
27505	ARE	2020-10-12	165.29	165.62	164.49	165.3126	186436	{}	\N	2020-10-12 14:17:40.370686
27664	ASMB	2020-10-02	16.92	17.1516	16.35	16.53	175491	{}	\N	2020-10-12 14:18:07.765323
27665	ASMB	2020-10-05	16.66	17.5	16.57	16.99	259951	{}	\N	2020-10-12 14:18:07.769951
27666	ASMB	2020-10-06	17.05	17.205	15.98	16.25	201398	{}	\N	2020-10-12 14:18:07.774483
27667	ASMB	2020-10-07	16.26	16.38	15.48	15.7	235137	{}	\N	2020-10-12 14:18:07.778611
27668	ASMB	2020-10-08	15.96	15.96	15.42	15.67	172750	{}	\N	2020-10-12 14:18:07.783159
27669	ASMB	2020-10-09	15.74	15.84	15.21	15.42	144476	{}	\N	2020-10-12 14:18:07.787102
27670	ASMB	2020-10-12	15.5	15.51	15.0822	15.31	10635	{}	\N	2020-10-12 14:18:07.791387
27671	ASPN	2020-09-28	10.5	11.2	10.45	10.73	109283	{}	\N	2020-10-12 14:18:09.01266
27672	ASPN	2020-09-29	10.74	11.03	10.6	10.99	203066	{}	\N	2020-10-12 14:18:09.022708
27673	ASPN	2020-09-30	11	11.1	10.4	10.95	380628	{}	\N	2020-10-12 14:18:09.029357
27674	ASPN	2020-10-01	10.99	14.63	10.62	11.57	7422265	{}	\N	2020-10-12 14:18:09.03485
27675	ASPN	2020-10-02	10.75	11.4999	10.56	11.03	688909	{}	\N	2020-10-12 14:18:09.039783
27676	ASPN	2020-10-05	11.14	11.48	10.97	11.41	477477	{}	\N	2020-10-12 14:18:09.044436
27677	ASPN	2020-10-06	11.5	11.5	10.61	10.84	298759	{}	\N	2020-10-12 14:18:09.048727
27678	ASPN	2020-10-07	10.87	11.28	10.6589	11.17	238535	{}	\N	2020-10-12 14:18:09.052973
27679	ASPN	2020-10-08	11.25	11.36	10.8001	11.05	173399	{}	\N	2020-10-12 14:18:09.05714
27680	ASPN	2020-10-09	11.35	11.35	11.11	11.21	240580	{}	\N	2020-10-12 14:18:09.061357
27681	ASPN	2020-10-12	11.31	11.39	11.01	11.01	29094	{}	\N	2020-10-12 14:18:09.065529
27682	ATHM	2020-09-28	91.7	92.32	90.16	90.82	176545	{}	\N	2020-10-12 14:18:11.066716
27683	ATHM	2020-09-29	91.03	91.03	89.66	90.7	572126	{}	\N	2020-10-12 14:18:11.077846
27684	ATHM	2020-09-30	92.83	96.69	91.01	96	1075816	{}	\N	2020-10-12 14:18:11.084937
27685	ATHM	2020-10-01	97.16	98.5	96.13	97.48	704441	{}	\N	2020-10-12 14:18:11.090684
27686	ATHM	2020-10-02	96.03	97.07	94.56	96.6	232400	{}	\N	2020-10-12 14:18:11.095794
27687	ATHM	2020-10-05	97.14	99.2	96.81	98.63	397941	{}	\N	2020-10-12 14:18:11.10116
27688	ATHM	2020-10-06	98.63	99.25	96.255	97.26	351500	{}	\N	2020-10-12 14:18:11.10539
27689	ATHM	2020-10-07	98.51	100	97.5277	99.03	332969	{}	\N	2020-10-12 14:18:11.109799
27690	ATHM	2020-10-08	99.78	100.25	98.8	99.79	331714	{}	\N	2020-10-12 14:18:11.114453
27691	ATHM	2020-10-09	99.95	100.89	99.2	99.51	463443	{}	\N	2020-10-12 14:18:11.118615
27692	ATHM	2020-10-12	100.13	102	100.13	101.72	77542	{}	\N	2020-10-12 14:18:11.122709
27693	ATO	2020-09-28	95.3	95.76	94.65	95.04	507171	{}	\N	2020-10-12 14:18:12.039675
27694	ATO	2020-09-29	95.17	95.93	94.89	95.1	1838476	{}	\N	2020-10-12 14:18:12.05332
27695	ATO	2020-09-30	95.82	95.95	94.69	95.59	1330357	{}	\N	2020-10-12 14:18:12.061093
27696	ATO	2020-10-01	95.61	96.34	94.82	95.47	2178355	{}	\N	2020-10-12 14:18:12.067323
27697	ATO	2020-10-02	94.95	97	94.59	96.02	2101967	{}	\N	2020-10-12 14:18:12.072488
27698	ATO	2020-10-05	96.59	96.59	94.945	95.22	2049881	{}	\N	2020-10-12 14:18:12.077806
27699	ATO	2020-10-06	95.36	97.874	95.36	96.98	1714822	{}	\N	2020-10-12 14:18:12.082607
27700	ATO	2020-10-07	97.07	97.42	95.62	96.38	1872387	{}	\N	2020-10-12 14:18:12.086937
27701	ATO	2020-10-08	96.7	97.28	96.2	96.79	1957293	{}	\N	2020-10-12 14:18:12.091221
27702	ATO	2020-10-09	97.4	97.8	95.99	96.51	2507428	{}	\N	2020-10-12 14:18:12.095439
27703	ATO	2020-10-12	96.29	97.01	96.1079	96.86	281693	{}	\N	2020-10-12 14:18:12.099582
27704	ATTO	2020-09-28	9.71	9.71	9.265	9.32	4862	{}	\N	2020-10-12 14:18:13.100604
27705	ATTO	2020-09-29	9.62	9.62	9.4	9.475	5932	{}	\N	2020-10-12 14:18:13.111671
27706	ATTO	2020-09-30	9.395	9.5	9.21	9.21	3058	{}	\N	2020-10-12 14:18:13.11885
27707	ATTO	2020-10-01	9.47	9.47	9.21	9.37	5570	{}	\N	2020-10-12 14:18:13.124044
27708	ATTO	2020-10-02	9.345	9.45	9.325	9.325	4062	{}	\N	2020-10-12 14:18:13.128585
27709	ATTO	2020-10-05	9.495	9.495	9.2	9.2	10388	{}	\N	2020-10-12 14:18:13.133216
27710	ATTO	2020-10-06	9.22	9.54	9.2	9.22	4490	{}	\N	2020-10-12 14:18:13.137293
27711	ATTO	2020-10-07	9.13	9.29	9.1118	9.13	7208	{}	\N	2020-10-12 14:18:13.140966
27712	ATTO	2020-10-08	9.18	9.29	9.02	9.02	17849	{}	\N	2020-10-12 14:18:13.144509
27713	ATTO	2020-10-09	9.13	9.21	8.9	8.93	45430	{}	\N	2020-10-12 14:18:13.148407
27714	ATTO	2020-10-12	8.89	8.89	8.89	8.89	578	{}	\N	2020-10-12 14:18:13.152092
27715	AUDC	2020-09-28	31.94	32.4262	31.39	31.62	212146	{}	\N	2020-10-12 14:18:14.112663
27716	AUDC	2020-09-29	31.57	32.31	31.54	32	131599	{}	\N	2020-10-12 14:18:14.120766
27717	AUDC	2020-09-30	31.79	31.99	30.98	31.46	280917	{}	\N	2020-10-12 14:18:14.126247
27718	AUDC	2020-10-01	32.01	32.13	31.61	31.82	157531	{}	\N	2020-10-12 14:18:14.132105
27719	AUDC	2020-10-02	31.35	31.67	30.8693	31.17	120410	{}	\N	2020-10-12 14:18:14.137094
27720	AUDC	2020-10-05	31.62	32.28	31.2811	32.25	152454	{}	\N	2020-10-12 14:18:14.141363
27721	AUDC	2020-10-06	32.25	33.04	31.7	32	176732	{}	\N	2020-10-12 14:18:14.145303
27722	AUDC	2020-10-07	32.1	32.88	31.87	32.7	151976	{}	\N	2020-10-12 14:18:14.149719
27723	AUDC	2020-10-08	32.96	33.0267	32.12	32.18	118125	{}	\N	2020-10-12 14:18:14.153741
27724	AUDC	2020-10-09	32.44	34.11	32.39	34.03	217343	{}	\N	2020-10-12 14:18:14.157624
27725	AUDC	2020-10-12	34.72	35.33	34.14	34.6	73674	{}	\N	2020-10-12 14:18:14.161717
27726	AVB	2020-09-28	150.37	152.82	149.58	151.45	1115905	{}	\N	2020-10-12 14:18:15.265253
27727	AVB	2020-09-29	149.34	150.78	146.44	147.52	1643966	{}	\N	2020-10-12 14:18:15.27574
27728	AVB	2020-09-30	148.65	150.48	147.31	149.34	2297711	{}	\N	2020-10-12 14:18:15.282096
27729	AVB	2020-10-01	149.72	155.63	149.61	155.6	2119017	{}	\N	2020-10-12 14:18:15.287879
27730	AVB	2020-10-02	153.32	159.02	151.59	157.94	2724081	{}	\N	2020-10-12 14:18:15.292535
27731	AVB	2020-10-05	159.04	160.84	156.71	160.43	2108905	{}	\N	2020-10-12 14:18:15.297467
27732	AVB	2020-10-06	161.49	162.33	157.79	158.6	2173296	{}	\N	2020-10-12 14:18:15.301918
27733	AVB	2020-10-07	159.12	160.13	155.75	156.85	1971563	{}	\N	2020-10-12 14:18:15.305711
27734	AVB	2020-10-08	157.96	160.82	157.35	160.1	1393908	{}	\N	2020-10-12 14:18:15.309388
27735	AVB	2020-10-09	161.01	161.82	157.29	158.34	1752541	{}	\N	2020-10-12 14:18:15.313231
27736	AVB	2020-10-12	158.99	159	157.57	158.02	152609	{}	\N	2020-10-12 14:18:15.317225
27737	AVT	2020-09-28	26.05	26.4	25.82	26.29	437216	{}	\N	2020-10-12 14:18:17.01086
27738	AVT	2020-09-29	26.16	26.45	25.68	25.87	459821	{}	\N	2020-10-12 14:18:17.020865
27739	AVT	2020-09-30	26.05	26.46	25.65	25.84	636339	{}	\N	2020-10-12 14:18:17.027901
27740	AVT	2020-10-01	26.11	26.385	25.77	26.31	492387	{}	\N	2020-10-12 14:18:17.034201
27741	AVT	2020-10-02	25.79	26.3	25.75	25.97	702330	{}	\N	2020-10-12 14:18:17.039674
27742	AVT	2020-10-05	26.16	26.91	26.16	26.47	1156290	{}	\N	2020-10-12 14:18:17.043979
27743	AVT	2020-10-06	26.51	27.01	26.2	26.27	568356	{}	\N	2020-10-12 14:18:17.049051
27744	AVT	2020-10-07	26.44	27.92	26.33	27.78	942870	{}	\N	2020-10-12 14:18:17.053473
27745	AVT	2020-10-08	28.05	28.05	27.34	27.65	456181	{}	\N	2020-10-12 14:18:17.057485
27746	AVT	2020-10-09	28.02	28.09	27.5527	28.07	469367	{}	\N	2020-10-12 14:18:17.061385
27747	AVT	2020-10-12	28.09	28.2	27.98	28.04	22939	{}	\N	2020-10-12 14:18:17.065123
27748	AWX	2020-09-28	1.47	1.58	1.44	1.5484	35647	{}	\N	2020-10-12 14:18:18.058249
27749	AWX	2020-09-29	1.55	2.33	1.55	2.07	704472	{}	\N	2020-10-12 14:18:18.066702
27750	AWX	2020-09-30	1.7	1.935	1.6	1.75	548234	{}	\N	2020-10-12 14:18:18.072689
27751	AWX	2020-10-01	1.75	1.85	1.61	1.68	357135	{}	\N	2020-10-12 14:18:18.078731
27752	AWX	2020-10-02	1.62	1.75	1.57	1.6693	98281	{}	\N	2020-10-12 14:18:18.083717
27753	AWX	2020-10-05	1.63	1.74	1.62	1.62	30225	{}	\N	2020-10-12 14:18:18.089195
27754	AWX	2020-10-06	1.6528	1.75	1.6399	1.65	55391	{}	\N	2020-10-12 14:18:18.093212
27755	AWX	2020-10-07	1.66	1.7	1.63	1.7	14422	{}	\N	2020-10-12 14:18:18.097244
27756	AWX	2020-10-08	1.66	1.8505	1.66	1.8	50972	{}	\N	2020-10-12 14:18:18.101184
27757	AWX	2020-10-09	1.78	2.11	1.73	1.79	451950	{}	\N	2020-10-12 14:18:18.104932
27758	AWX	2020-10-12	1.72	1.825	1.72	1.82	22646	{}	\N	2020-10-12 14:18:18.108788
27759	ASM	2020-09-28	1.04	1.05	1	1.04	709991	{}	\N	2020-10-12 14:18:19.018622
27760	ASM	2020-09-29	1.05	1.08	1.02	1.04	607819	{}	\N	2020-10-12 14:18:19.028116
27761	ASM	2020-09-30	1.02	1.03	0.9925	0.9926	525074	{}	\N	2020-10-12 14:18:19.035327
27762	ASM	2020-10-01	1.03	1.03	0.991	1.02	686982	{}	\N	2020-10-12 14:18:19.04128
27763	ASM	2020-10-02	0.98	1	0.955	0.96685	884150	{}	\N	2020-10-12 14:18:19.046402
27764	ASM	2020-10-05	0.98	1.01	0.9701	0.999	799458	{}	\N	2020-10-12 14:18:19.051653
27765	ASM	2020-10-06	1	1.02	0.9294	0.9395	923872	{}	\N	2020-10-12 14:18:19.056255
27766	ASM	2020-10-07	0.95	0.96	0.9311	0.9502	466729	{}	\N	2020-10-12 14:18:19.060551
27767	ASM	2020-10-08	0.98	1.01	0.9311	0.9653	921932	{}	\N	2020-10-12 14:18:19.06481
27768	ASM	2020-10-09	1.01	1.07	0.995	1.07	1363156	{}	\N	2020-10-12 14:18:19.069293
27769	ASM	2020-10-12	1.08	1.09	1.07	1.0827	254836	{}	\N	2020-10-12 14:18:19.073697
27770	ATEC	2020-09-28	6.83	7.28	6.8	6.94	980902	{}	\N	2020-10-12 14:18:20.455655
27771	ATEC	2020-09-29	6.95	7.24	6.75	6.87	576216	{}	\N	2020-10-12 14:18:20.464602
27772	ATEC	2020-09-30	6.88	7.05	6.51	6.64	496402	{}	\N	2020-10-12 14:18:20.471684
27773	ATEC	2020-10-01	6.76	6.98	6.26	6.37	442780	{}	\N	2020-10-12 14:18:20.477618
27774	ATEC	2020-10-02	6.2	6.59	6.11	6.51	306534	{}	\N	2020-10-12 14:18:20.483892
27775	ATEC	2020-10-05	6.53	6.81	6.53	6.7	289603	{}	\N	2020-10-12 14:18:20.489221
27776	ATEC	2020-10-06	6.77	6.96	6.72	6.75	402313	{}	\N	2020-10-12 14:18:20.493361
27777	ATEC	2020-10-07	6.85	6.97	6.76	6.88	853318	{}	\N	2020-10-12 14:18:20.497716
27778	ATEC	2020-10-08	8.0635	9.55	8.05	9.42	11288517	{}	\N	2020-10-12 14:18:20.502006
27779	ATEC	2020-10-09	9.2	10.1	9.17	9.68	3541939	{}	\N	2020-10-12 14:18:20.506414
27780	ATEC	2020-10-12	9.6207	9.6999	9.27	9.54	302587	{}	\N	2020-10-12 14:18:20.510483
27781	ATKR	2020-09-28	22.84	23.72	22.735	23.25	519563	{}	\N	2020-10-12 14:18:21.949008
27782	ATKR	2020-09-29	23.22	23.33	22.48	22.79	532712	{}	\N	2020-10-12 14:18:21.957497
27783	ATKR	2020-09-30	22.96	23.52	22.55	22.73	878625	{}	\N	2020-10-12 14:18:21.963728
27784	ATKR	2020-10-01	22.83	23.4	22.71	23.01	475584	{}	\N	2020-10-12 14:18:21.970472
27785	ATKR	2020-10-02	22.5	23.5	22.5	23.19	486170	{}	\N	2020-10-12 14:18:21.976216
27786	ATKR	2020-10-05	23.55	23.735	23.1	23.5	740926	{}	\N	2020-10-12 14:18:21.980845
27787	ATKR	2020-10-06	23.83	24.46	23.45	23.59	789525	{}	\N	2020-10-12 14:18:21.985285
27788	ATKR	2020-10-07	23.87	24.26	23.69	23.74	963374	{}	\N	2020-10-12 14:18:21.989617
27789	ATKR	2020-10-08	24.15	24.32	23.77	24.23	748226	{}	\N	2020-10-12 14:18:21.994024
27790	ATKR	2020-10-09	24.58	25.07	23.9	24.52	982494	{}	\N	2020-10-12 14:18:21.998391
27791	ATKR	2020-10-12	24.63	24.63	24.36	24.55	33169	{}	\N	2020-10-12 14:18:22.003725
27792	ATOM	2020-09-28	10.28	10.5	10.02	10.42	307579	{}	\N	2020-10-12 14:18:23.103642
27793	ATOM	2020-09-29	10.39	10.83	10.28	10.55	423714	{}	\N	2020-10-12 14:18:23.113383
27794	ATOM	2020-09-30	10.51	10.92	10.24	10.45	495518	{}	\N	2020-10-12 14:18:23.119546
27795	ATOM	2020-10-01	10.55	10.96	10.45	10.75	465160	{}	\N	2020-10-12 14:18:23.124745
27796	ATOM	2020-10-02	10.3	10.66	10.17	10.44	321724	{}	\N	2020-10-12 14:18:23.129889
27797	ATOM	2020-10-05	10.43	10.71	10.39	10.65	230635	{}	\N	2020-10-12 14:18:23.135384
27798	ATOM	2020-10-06	10.64	10.68	10.11	10.11	220156	{}	\N	2020-10-12 14:18:23.13937
27799	ATOM	2020-10-07	10.16	10.55	10.11	10.46	215213	{}	\N	2020-10-12 14:18:23.143384
27800	ATOM	2020-10-08	10.5	10.73	10.45	10.64	254043	{}	\N	2020-10-12 14:18:23.147695
27801	ATOM	2020-10-09	10.7	10.97	10.66	10.85	359947	{}	\N	2020-10-12 14:18:23.151784
27802	ATOM	2020-10-12	10.94	10.95	10.7661	10.8	44232	{}	\N	2020-10-12 14:18:23.156036
27803	AVDL	2020-09-28	5.37	5.4112	5.15	5.26	438858	{}	\N	2020-10-12 14:18:25.109348
27804	AVDL	2020-09-29	5.2	5.3	5.12	5.14	418093	{}	\N	2020-10-12 14:18:25.119536
27805	AVDL	2020-09-30	5.14	5.1835	4.93	5.04	651980	{}	\N	2020-10-12 14:18:25.125143
27806	AVDL	2020-10-01	5.1	5.17	5.05	5.11	464552	{}	\N	2020-10-12 14:18:25.130336
27807	AVDL	2020-10-02	5.01	5.22	4.9	5.12	390363	{}	\N	2020-10-12 14:18:25.134888
27808	AVDL	2020-10-05	5.16	5.28	5.05	5.25	354932	{}	\N	2020-10-12 14:18:25.139257
27809	AVDL	2020-10-06	5.34	5.5	5.1182	5.2	589636	{}	\N	2020-10-12 14:18:25.142906
27810	AVDL	2020-10-07	5.25	5.61	5.2294	5.55	385840	{}	\N	2020-10-12 14:18:25.147048
27811	AVDL	2020-10-08	5.55	5.96	5.55	5.89	610443	{}	\N	2020-10-12 14:18:25.151273
27812	AVDL	2020-10-09	5.85	5.91	5.56	5.62	383849	{}	\N	2020-10-12 14:18:25.155433
27813	AVDL	2020-10-12	5.6	5.6744	5.44	5.6	83876	{}	\N	2020-10-12 14:18:25.159394
27814	AWRE	2020-09-28	2.8118	2.8118	2.72	2.76	2610	{}	\N	2020-10-12 14:18:26.890073
27815	AWRE	2020-09-29	2.78	2.79	2.7	2.75	9562	{}	\N	2020-10-12 14:18:26.90134
27816	AWRE	2020-09-30	2.73	2.76	2.7	2.7	5397	{}	\N	2020-10-12 14:18:26.908576
27817	AWRE	2020-10-01	2.75	2.75	2.65	2.65	3955	{}	\N	2020-10-12 14:18:26.914677
27818	AWRE	2020-10-02	2.595	2.77	2.595	2.73	6675	{}	\N	2020-10-12 14:18:26.919738
27819	AWRE	2020-10-05	2.74	2.79	2.67	2.69	6749	{}	\N	2020-10-12 14:18:26.924731
27820	AWRE	2020-10-06	2.74	2.8	2.67	2.67	12329	{}	\N	2020-10-12 14:18:26.929118
27821	AWRE	2020-10-07	2.68	2.76	2.6791	2.737	1554	{}	\N	2020-10-12 14:18:26.933515
27822	AWRE	2020-10-08	2.7	2.98	2.65	2.73	263549	{}	\N	2020-10-12 14:18:26.937651
27823	AWRE	2020-10-09	2.75	2.82	2.66	2.69	65660	{}	\N	2020-10-12 14:18:26.941661
27824	AWRE	2020-10-12	2.67	2.7378	2.67	2.7	631	{}	\N	2020-10-12 14:18:26.947337
27825	AXR	2020-09-28	4.9778	5.15	4.9778	5.12	10879	{}	\N	2020-10-12 14:18:29.050781
27826	AXR	2020-09-29	5.07	5.43	5.07	5.43	14420	{}	\N	2020-10-12 14:18:29.059789
27827	AXR	2020-09-30	5.24	5.63	5.24	5.6	14740	{}	\N	2020-10-12 14:18:29.065836
27828	AXR	2020-10-01	5.21	5.59	5.21	5.44	57104	{}	\N	2020-10-12 14:18:29.071579
27829	AXR	2020-10-02	5.4	5.705	5.38	5.705	1680	{}	\N	2020-10-12 14:18:29.077005
27830	AXR	2020-10-05	5.74	5.74	5.74	5.74	884	{}	\N	2020-10-12 14:18:29.081543
27831	AXR	2020-10-06	5.74	6.17	5.71	5.71	4646	{}	\N	2020-10-12 14:18:29.085994
27832	AXR	2020-10-07	5.77	5.96	5.77	5.96	923	{}	\N	2020-10-12 14:18:29.097391
27833	AXR	2020-10-08	6.06	6.06	6.06	6.06	726	{}	\N	2020-10-12 14:18:29.102022
27834	AYX	2020-09-28	113	114.77	110.9	113.69	1023495	{}	\N	2020-10-12 14:18:30.047931
27835	AYX	2020-09-29	113.6	114.5	112.52	113.8	1876657	{}	\N	2020-10-12 14:18:30.059093
27836	AYX	2020-09-30	112.8	116	112.34	113.55	2739893	{}	\N	2020-10-12 14:18:30.066601
27837	AYX	2020-10-01	115.28	118.35	113.14	117.59	2527352	{}	\N	2020-10-12 14:18:30.072659
27838	AYX	2020-10-02	114.03	117.86	112.5	113.09	2235680	{}	\N	2020-10-12 14:18:30.078115
27839	AYX	2020-10-05	113.65	115.7812	111.7373	113.88	6174103	{}	\N	2020-10-12 14:18:30.083411
27840	AYX	2020-10-06	139	148	137.4	145.96	23144995	{}	\N	2020-10-12 14:18:30.087966
27841	AYX	2020-10-07	145.06	147.5	142.3	143.68	4717344	{}	\N	2020-10-12 14:18:30.09252
27842	AYX	2020-10-08	146.84	154.0124	146.1201	146.92	4879130	{}	\N	2020-10-12 14:18:30.097347
27843	AYX	2020-10-09	148.2	153.86	147.06	151.97	2685785	{}	\N	2020-10-12 14:18:30.102173
27844	AYX	2020-10-12	153.9	154.83	148.05	148.72	792105	{}	\N	2020-10-12 14:18:30.106776
27845	BAB	2020-09-28	33.49	33.51	33.41	33.44	1097342	{}	\N	2020-10-12 14:18:31.09862
27846	BAB	2020-09-29	33.45	33.49	33.41	33.43	918672	{}	\N	2020-10-12 14:18:31.10842
27847	BAB	2020-09-30	33.41	33.42	33.25	33.35	529788	{}	\N	2020-10-12 14:18:31.115508
27848	BAB	2020-10-01	33.29	33.34	33.2208	33.3	330356	{}	\N	2020-10-12 14:18:31.121171
27849	BAB	2020-10-02	33.36	33.4	33.23	33.24	277496	{}	\N	2020-10-12 14:18:31.126457
27850	BAB	2020-10-05	33.19	33.19	32.92	33.02	199279	{}	\N	2020-10-12 14:18:31.131333
27851	BAB	2020-10-06	32.97	33.135	32.9031	33.11	533832	{}	\N	2020-10-12 14:18:31.135815
27852	BAB	2020-10-07	33.02	33.02	32.865	32.91	375536	{}	\N	2020-10-12 14:18:31.140217
27853	BAB	2020-10-08	32.98	33	32.9	33	315351	{}	\N	2020-10-12 14:18:31.144226
27854	BAB	2020-10-09	32.95	33	32.86	33	167574	{}	\N	2020-10-12 14:18:31.148445
27855	BAB	2020-10-12	33.01	33.01	32.943	32.955	23719	{}	\N	2020-10-12 14:18:31.153197
27856	ASND	2020-09-28	154.2	154.72	149.02	151.94	205997	{}	\N	2020-10-12 14:18:32.710483
27857	ASND	2020-09-29	152.13	154.05	148.245	151.89	268046	{}	\N	2020-10-12 14:18:32.720455
27858	ASND	2020-09-30	153.13	155.8	150.03	154.32	465421	{}	\N	2020-10-12 14:18:32.727491
27859	ASND	2020-10-01	156.54	158.0738	153.37	155.98	142234	{}	\N	2020-10-12 14:18:32.734239
27860	ASND	2020-10-02	154.25	159.57	153.44	158.63	159782	{}	\N	2020-10-12 14:18:32.739483
27861	ASND	2020-10-05	159	166.99	158.81	161.77	266935	{}	\N	2020-10-12 14:18:32.744592
27862	ASND	2020-10-06	162.1	164.98	161.09	162.04	118588	{}	\N	2020-10-12 14:18:32.749912
27863	ASND	2020-10-07	163.57	164.8	160.135	163.75	125107	{}	\N	2020-10-12 14:18:32.754855
27864	ASND	2020-10-08	164.37	164.37	160.62	160.97	79493	{}	\N	2020-10-12 14:18:32.75955
27865	ASND	2020-10-09	160.83	162.08	159.89	160.73	143813	{}	\N	2020-10-12 14:18:32.764318
27866	ASND	2020-10-12	161.39	161.39	158	160.65	12658	{}	\N	2020-10-12 14:18:32.769363
27867	ATEN	2020-09-28	6.57	6.78	6.45	6.69	1055927	{}	\N	2020-10-12 14:18:34.129892
27868	ATEN	2020-09-29	6.68	6.69	6.45	6.55	1985378	{}	\N	2020-10-12 14:18:34.140239
27869	ATEN	2020-09-30	6.51	6.595	6.355	6.37	1500158	{}	\N	2020-10-12 14:18:34.147065
27870	ATEN	2020-10-01	6.4	6.46	6.27	6.32	1035317	{}	\N	2020-10-12 14:18:34.152989
27871	ATEN	2020-10-02	6.17	6.335	6.1301	6.27	1089186	{}	\N	2020-10-12 14:18:34.158215
27872	ATEN	2020-10-05	6.54	6.57	6.335	6.4	1823563	{}	\N	2020-10-12 14:18:34.163847
27873	ATEN	2020-10-06	6.44	6.83	6.43	6.61	2416386	{}	\N	2020-10-12 14:18:34.16827
27874	ATEN	2020-10-07	6.63	6.875	6.62	6.81	1811116	{}	\N	2020-10-12 14:18:34.172736
27875	ATEN	2020-10-08	6.87	7.02	6.83	6.97	1269490	{}	\N	2020-10-12 14:18:34.177335
27876	ATEN	2020-10-09	7.02	7.165	6.965	7.14	1490065	{}	\N	2020-10-12 14:18:34.181596
27877	ATEN	2020-10-12	7.2	7.3919	7.175	7.25	386125	{}	\N	2020-10-12 14:18:34.185705
27878	ATLO	2020-09-28	17.45	18	17.085	17.24	45214	{}	\N	2020-10-12 14:18:36.621175
27879	ATLO	2020-09-29	16.87	17.1093	16.87	16.91	14657	{}	\N	2020-10-12 14:18:36.634114
27880	ATLO	2020-09-30	16.95	17.155	16.89	16.89	28354	{}	\N	2020-10-12 14:18:36.64175
27881	ATLO	2020-10-01	16.9523	17	16.7	16.81	17182	{}	\N	2020-10-12 14:18:36.647827
27882	ATLO	2020-10-02	16.75	16.99	16.62	16.79	43687	{}	\N	2020-10-12 14:18:36.653229
27883	ATLO	2020-10-05	16.8	17.4	16.775	17.26	96249	{}	\N	2020-10-12 14:18:36.658331
27884	ATLO	2020-10-06	17.26	18.4	16.92	17.85	35064	{}	\N	2020-10-12 14:18:36.66397
27885	ATLO	2020-10-07	18.13	18.49	17.81	18.34	16853	{}	\N	2020-10-12 14:18:36.667897
27886	ATLO	2020-10-08	18.34	18.99	17.81	18.72	13499	{}	\N	2020-10-12 14:18:36.672347
27887	ATLO	2020-10-09	18.0101	18.91	18.0101	18.78	14325	{}	\N	2020-10-12 14:18:36.676442
27888	ATLO	2020-10-12	18.77	19.41	18.64	18.64	308	{}	\N	2020-10-12 14:18:36.680227
27889	ATRA	2020-09-28	12.98	13.18	12.6	12.94	748843	{}	\N	2020-10-12 14:18:37.719375
27890	ATRA	2020-09-29	12.85	13.26	12.82	13.08	496203	{}	\N	2020-10-12 14:18:37.726602
27891	ATRA	2020-09-30	13.12	13.38	11.97	12.96	1357026	{}	\N	2020-10-12 14:18:37.733033
27892	ATRA	2020-10-01	13.06	13.69	12.96	13.23	583722	{}	\N	2020-10-12 14:18:37.739292
27893	ATRA	2020-10-02	12.95	13.365	12.66	13.07	359020	{}	\N	2020-10-12 14:18:37.744207
27894	ATRA	2020-10-05	13.3	13.84	13.24	13.68	608055	{}	\N	2020-10-12 14:18:37.748991
27895	ATRA	2020-10-06	13.79	14.48	13.7201	13.94	372210	{}	\N	2020-10-12 14:18:37.754204
27896	ATRA	2020-10-07	14.07	14.76	14.01	14.5	556318	{}	\N	2020-10-12 14:18:37.758496
27897	ATRA	2020-10-08	14.67	14.8487	14.28	14.45	344922	{}	\N	2020-10-12 14:18:37.762839
27898	ATRA	2020-10-09	14.49	14.7	14.25	14.47	404224	{}	\N	2020-10-12 14:18:37.767206
27899	ATRA	2020-10-12	14.47	14.74	14.24	14.47	45646	{}	\N	2020-10-12 14:18:37.77123
27900	ATUS	2020-09-28	26.48	26.805	26.44	26.54	3795548	{}	\N	2020-10-12 14:18:38.97927
27901	ATUS	2020-09-29	26.5	26.745	26.29	26.3	9771850	{}	\N	2020-10-12 14:18:38.988596
27902	ATUS	2020-09-30	26.34	26.51	25.89	26	7262530	{}	\N	2020-10-12 14:18:38.995573
27903	ATUS	2020-10-01	26.09	26.27	25.68	26.12	15469926	{}	\N	2020-10-12 14:18:39.001542
27904	ATUS	2020-10-02	25.69	26.315	25.69	26.05	9099851	{}	\N	2020-10-12 14:18:39.006945
27905	ATUS	2020-10-05	27.02	27.265	26.47	26.81	11421492	{}	\N	2020-10-12 14:18:39.011614
27906	ATUS	2020-10-06	26.84	27.1	26.33	26.39	9874491	{}	\N	2020-10-12 14:18:39.017004
27907	ATUS	2020-10-07	26.63	26.84	26.38	26.53	7252615	{}	\N	2020-10-12 14:18:39.021543
27908	ATUS	2020-10-08	26.7	27.4	26.55	27.31	10317023	{}	\N	2020-10-12 14:18:39.026168
27909	ATUS	2020-10-09	27.51	27.51	26.99	27.2	8135491	{}	\N	2020-10-12 14:18:39.030915
27910	ATUS	2020-10-12	27.31	27.69	27.3	27.635	1569685	{}	\N	2020-10-12 14:18:39.035525
27911	AUMN	2020-09-28	0.422	0.4335	0.4112	0.43	707236	{}	\N	2020-10-12 14:18:40.383001
27912	AUMN	2020-09-29	0.4304	0.4367	0.4299	0.4345	791078	{}	\N	2020-10-12 14:18:40.393734
27913	AUMN	2020-09-30	0.4295	0.432	0.4126	0.42	317716	{}	\N	2020-10-12 14:18:40.400551
27914	AUMN	2020-10-01	0.425	0.4396	0.42	0.434	570315	{}	\N	2020-10-12 14:18:40.405896
27915	AUMN	2020-10-02	0.4226	0.4249	0.405	0.414	2489426	{}	\N	2020-10-12 14:18:40.411066
27916	AUMN	2020-10-05	0.4199	0.425	0.4142	0.4218	585051	{}	\N	2020-10-12 14:18:40.416335
27917	AUMN	2020-10-06	0.4223	0.4236	0.401	0.401	1216962	{}	\N	2020-10-12 14:18:40.420207
27918	AUMN	2020-10-07	0.41	0.5	0.407	0.426	14736951	{}	\N	2020-10-12 14:18:40.423766
27919	AUMN	2020-10-08	0.43	0.43	0.41	0.4149	1684212	{}	\N	2020-10-12 14:18:40.428264
27920	AUMN	2020-10-09	0.4299	0.4355	0.4149	0.43	1083683	{}	\N	2020-10-12 14:18:40.432415
27921	AUMN	2020-10-12	0.4298	0.432	0.420201	0.425	447587	{}	\N	2020-10-12 14:18:40.436267
29707	BMLP	2020-09-28	26.19	26.19	26.19	26.19	101	{}	\N	2020-10-12 14:22:42.277225
29708	BMLP	2020-09-29	26.42	26.42	26.42	26.42	109	{}	\N	2020-10-12 14:22:42.287857
29709	BMLP	2020-09-30	26.96	27.39	26.96	27.39	807	{}	\N	2020-10-12 14:22:42.292566
29710	BMLP	2020-10-01	26.51	26.68	26.51	26.68	872	{}	\N	2020-10-12 14:22:42.297175
29711	BMLP	2020-10-02	25.82	25.82	25.82	25.82	529	{}	\N	2020-10-12 14:22:42.301893
29712	BMLP	2020-10-05	27.25	27.28	27.21	27.25	1303	{}	\N	2020-10-12 14:22:42.306082
29713	BMLP	2020-10-06	27.78	27.9	27.39	27.51	2029	{}	\N	2020-10-12 14:22:42.310538
29714	BMLP	2020-10-07	27.67	27.76	27.42	27.42	8240	{}	\N	2020-10-12 14:22:42.314896
29715	BMLP	2020-10-08	28.18	28.74	28.04	28.74	1007	{}	\N	2020-10-12 14:22:42.319281
29716	BMLP	2020-10-09	28.87	28.92	28.87	28.92	802	{}	\N	2020-10-12 14:22:42.328454
29717	BND	2020-09-28	88.27	88.31	88.23	88.3	4234548	{}	\N	2020-10-12 14:22:43.343144
29718	BND	2020-09-29	88.39	88.44	88.33	88.36	8017435	{}	\N	2020-10-12 14:22:43.357021
29719	BND	2020-09-30	88.28	88.3	88.13	88.22	6180322	{}	\N	2020-10-12 14:22:43.365384
29720	BND	2020-10-01	87.9917	88.19	87.97	88.16	4507152	{}	\N	2020-10-12 14:22:43.371402
29721	BND	2020-10-02	88.21	88.21	87.98	88.1	5593616	{}	\N	2020-10-12 14:22:43.37663
29722	BND	2020-10-05	87.98	87.98	87.77	87.77	6335482	{}	\N	2020-10-12 14:22:43.381788
29723	BND	2020-10-06	87.86	88.04	87.74	87.85	5669560	{}	\N	2020-10-12 14:22:43.386643
29724	BND	2020-10-07	87.82	87.9	87.72	87.77	3507021	{}	\N	2020-10-12 14:22:43.391493
29725	BND	2020-10-08	87.89	87.95	87.85	87.92	5753733	{}	\N	2020-10-12 14:22:43.398151
29726	BND	2020-10-09	87.8265	87.94	87.76	87.93	4067785	{}	\N	2020-10-12 14:22:43.405923
29727	BND	2020-10-12	87.99	88	87.96	87.995	418089	{}	\N	2020-10-12 14:22:43.412885
29728	BNTC	2020-09-28	6.145	6.145	5.29	5.69	15791	{}	\N	2020-10-12 14:22:44.374246
29729	BNTC	2020-09-29	5.62	6.14	5.62	6	14470	{}	\N	2020-10-12 14:22:44.384286
27922	AWP	2020-09-28	4.6	4.69	4.59	4.64	413544	{}	\N	2020-10-12 14:18:41.897276
27923	AWP	2020-09-29	4.65	4.6501	4.57	4.58	562189	{}	\N	2020-10-12 14:18:41.910033
27924	AWP	2020-09-30	4.62	4.64	4.57	4.61	829846	{}	\N	2020-10-12 14:18:41.91553
27925	AWP	2020-10-01	4.6463	4.73	4.63	4.71	997759	{}	\N	2020-10-12 14:18:41.920402
27926	AWP	2020-10-02	4.64	4.75	4.6268	4.72	641353	{}	\N	2020-10-12 14:18:41.924439
27927	AWP	2020-10-05	4.75	4.79	4.73	4.78	1134353	{}	\N	2020-10-12 14:18:41.929039
27928	AWP	2020-10-06	4.81	4.87	4.78	4.82	1183464	{}	\N	2020-10-12 14:18:41.932888
27929	AWP	2020-10-07	4.87	4.87	4.82	4.85	747596	{}	\N	2020-10-12 14:18:41.936752
27930	AWP	2020-10-08	4.89	4.97	4.84	4.97	1192732	{}	\N	2020-10-12 14:18:41.940391
27931	AWP	2020-10-09	4.97	4.99	4.91	4.97	1214373	{}	\N	2020-10-12 14:18:41.944128
27932	AWP	2020-10-12	4.95	4.97	4.95	4.95	152912	{}	\N	2020-10-12 14:18:41.948343
27933	AXU	2020-09-28	2.64	2.68	2.5613	2.67	733098	{}	\N	2020-10-12 14:18:44.148337
27934	AXU	2020-09-29	2.71	2.74	2.62	2.68	814012	{}	\N	2020-10-12 14:18:44.15852
27935	AXU	2020-09-30	2.67	2.72	2.59	2.64	1106802	{}	\N	2020-10-12 14:18:44.164739
27936	AXU	2020-10-01	2.73	2.77	2.68	2.68	852691	{}	\N	2020-10-12 14:18:44.172697
27937	AXU	2020-10-02	2.65	2.73	2.59	2.61	1192872	{}	\N	2020-10-12 14:18:44.177195
27938	AXU	2020-10-05	2.64	2.69	2.58	2.64	1012575	{}	\N	2020-10-12 14:18:44.181611
27939	AXU	2020-10-06	2.65	2.68	2.46	2.49	1313455	{}	\N	2020-10-12 14:18:44.185213
27940	AXU	2020-10-07	2.5	2.55	2.46	2.5	1084584	{}	\N	2020-10-12 14:18:44.189117
27941	AXU	2020-10-08	2.51	2.55	2.45	2.45	718507	{}	\N	2020-10-12 14:18:44.192674
27942	AXU	2020-10-09	2.52	2.78	2.52	2.75	1814788	{}	\N	2020-10-12 14:18:44.19662
27943	AXU	2020-10-12	2.8	2.87	2.77	2.86	304608	{}	\N	2020-10-12 14:18:44.200153
27944	AZRX	2020-09-28	0.75	0.75	0.7202	0.74	88725	{}	\N	2020-10-12 14:18:45.66323
27945	AZRX	2020-09-29	0.815	0.815	0.72	0.7238	172823	{}	\N	2020-10-12 14:18:45.675245
27946	AZRX	2020-09-30	0.72	0.756	0.7	0.711	125525	{}	\N	2020-10-12 14:18:45.682524
27947	AZRX	2020-10-01	0.73	0.73	0.6956	0.7	149493	{}	\N	2020-10-12 14:18:45.688514
27948	AZRX	2020-10-02	0.6975	0.7398	0.6975	0.7156	64977	{}	\N	2020-10-12 14:18:45.693779
27949	AZRX	2020-10-05	0.7099	0.7425	0.705	0.7301	88707	{}	\N	2020-10-12 14:18:45.699106
27950	AZRX	2020-10-06	0.7325	0.76	0.7325	0.745	84773	{}	\N	2020-10-12 14:18:45.702921
27951	AZRX	2020-10-07	0.7514	0.7514	0.73	0.745	62778	{}	\N	2020-10-12 14:18:45.706608
27952	AZRX	2020-10-08	0.75	0.8	0.75	0.7844	192611	{}	\N	2020-10-12 14:18:45.710614
27953	AZRX	2020-10-09	0.79	0.813	0.775	0.7753	128281	{}	\N	2020-10-12 14:18:45.714757
27954	AZRX	2020-10-12	0.78	0.79	0.775	0.775	12906	{}	\N	2020-10-12 14:18:45.71854
27955	BAND	2020-09-28	175	178.625	169.1126	171.08	474352	{}	\N	2020-10-12 14:18:47.59478
27956	BAND	2020-09-29	171.83	179.11	171.24	175.16	337845	{}	\N	2020-10-12 14:18:47.604646
27957	BAND	2020-09-30	175.9	178.35	173	174.57	309535	{}	\N	2020-10-12 14:18:47.611451
27958	BAND	2020-10-01	177.14	181.68	173.62	179.95	333535	{}	\N	2020-10-12 14:18:47.617323
27959	BAND	2020-10-02	176.52	183.962	173.67	181.25	299538	{}	\N	2020-10-12 14:18:47.621895
27960	BAND	2020-10-05	181.51	184.5	179.1	182.21	672113	{}	\N	2020-10-12 14:18:47.626626
27961	BAND	2020-10-06	183.01	188.19	181.12	184.63	381781	{}	\N	2020-10-12 14:18:47.631391
27962	BAND	2020-10-07	177.16	178.95	169.04	171.57	1129745	{}	\N	2020-10-12 14:18:47.63565
27963	BAND	2020-10-08	172.04	177.38	171.17	173.82	569451	{}	\N	2020-10-12 14:18:47.640138
27964	BAND	2020-10-09	174.65	178.91	174.65	178.12	285484	{}	\N	2020-10-12 14:18:47.644999
27965	BAND	2020-10-12	183.41	184.74	180.25	183.6507	94777	{}	\N	2020-10-12 14:18:47.649774
27966	BASI	2020-09-28	4.99	5	4.8	4.84	5222	{}	\N	2020-10-12 14:18:49.114134
27967	BASI	2020-09-29	4.8007	4.97	4.8007	4.92	1798	{}	\N	2020-10-12 14:18:49.123008
27968	BASI	2020-09-30	4.94	4.94	4.78	4.78	1056	{}	\N	2020-10-12 14:18:49.129501
27969	BASI	2020-10-01	4.77	4.8002	4.6648	4.7	3693	{}	\N	2020-10-12 14:18:49.13513
27970	BASI	2020-10-02	4.72	5	4.72	4.985	20959	{}	\N	2020-10-12 14:18:49.139624
27971	BASI	2020-10-05	5.11	5.25	5	5.1	19179	{}	\N	2020-10-12 14:18:49.143985
27972	BASI	2020-10-06	5.13	5.15	5	5.09	7393	{}	\N	2020-10-12 14:18:49.148259
27973	BASI	2020-10-07	5.25	5.25	5.095	5.1499	10387	{}	\N	2020-10-12 14:18:49.152114
27974	BASI	2020-10-08	5.11	5.22	5.11	5.17	7295	{}	\N	2020-10-12 14:18:49.156112
27975	BASI	2020-10-09	5.14	5.24	5.05	5.2	7688	{}	\N	2020-10-12 14:18:49.160117
27976	BASI	2020-10-12	5.24	5.24	5.2	5.2084	455	{}	\N	2020-10-12 14:18:49.166129
27977	BBGI	2020-09-28	1.37	1.39	1.21	1.26	221092	{}	\N	2020-10-12 14:18:50.672822
27978	BBGI	2020-09-29	1.26	1.37	1.2	1.32	440114	{}	\N	2020-10-12 14:18:50.684047
27979	BBGI	2020-09-30	1.3	1.3	1.21	1.24	80871	{}	\N	2020-10-12 14:18:50.692797
27980	BBGI	2020-10-01	1.24	1.25	1.22	1.23	20313	{}	\N	2020-10-12 14:18:50.700413
27981	BBGI	2020-10-02	1.22	1.24	1.22	1.23	42718	{}	\N	2020-10-12 14:18:50.705804
27982	BBGI	2020-10-05	1.24	1.2815	1.21	1.24	189377	{}	\N	2020-10-12 14:18:50.711633
27983	BBGI	2020-10-06	1.23	1.33	1.23	1.25	71337	{}	\N	2020-10-12 14:18:50.716728
27984	BBGI	2020-10-07	1.2501	1.38	1.25	1.31	417321	{}	\N	2020-10-12 14:18:50.721837
27985	BBGI	2020-10-08	1.32	1.36	1.28	1.32	93651	{}	\N	2020-10-12 14:18:50.726582
27986	BBGI	2020-10-09	1.33	1.3301	1.27	1.3	90377	{}	\N	2020-10-12 14:18:50.73104
27987	BBGI	2020-10-12	1.3	1.34	1.28	1.31	4000	{}	\N	2020-10-12 14:18:50.735574
27988	ASR	2020-09-28	112.1	114.84	110.86	114.49	37444	{}	\N	2020-10-12 14:18:52.499886
27989	ASR	2020-09-29	113.91	115.68	113.22	114.43	129644	{}	\N	2020-10-12 14:18:52.509886
27990	ASR	2020-09-30	114.69	118.9	114.69	116.03	99086	{}	\N	2020-10-12 14:18:52.516359
27991	ASR	2020-10-01	112.09	115.55	110.24	110.82	139385	{}	\N	2020-10-12 14:18:52.523033
27992	ASR	2020-10-02	109.02	112.87	108.3	112.32	74607	{}	\N	2020-10-12 14:18:52.52918
27993	ASR	2020-10-05	113.13	115.66	113.13	114.98	58679	{}	\N	2020-10-12 14:18:52.53379
27994	ASR	2020-10-06	115.52	115.52	111.75	113.18	175023	{}	\N	2020-10-12 14:18:52.537757
27995	ASR	2020-10-07	112.63	116.25	111.08	115.04	141946	{}	\N	2020-10-12 14:18:52.541768
27996	ASR	2020-10-08	115.22	116.64	114.22	115.71	90389	{}	\N	2020-10-12 14:18:52.545887
27997	ASR	2020-10-09	115.73	117.68	115.26	117.06	114192	{}	\N	2020-10-12 14:18:52.550096
27998	ASR	2020-10-12	116.74	118	116.74	117.95	12870	{}	\N	2020-10-12 14:18:52.554527
27999	ATLC	2020-09-28	11.56	12.84	11.52	12.09	7923	{}	\N	2020-10-12 14:18:54.877218
28000	ATLC	2020-09-29	12.225	12.225	11.81	12.04	4018	{}	\N	2020-10-12 14:18:54.886738
28001	ATLC	2020-09-30	11.9	11.9	11.9	11.9	843	{}	\N	2020-10-12 14:18:54.893967
28002	ATLC	2020-10-01	11.615	12.47	11.615	12.47	4294	{}	\N	2020-10-12 14:18:54.899703
28003	ATLC	2020-10-02	12.6	13.8	12.6	12.73	9024	{}	\N	2020-10-12 14:18:54.904724
28004	ATLC	2020-10-05	13.1289	13.47	13.1289	13.3	4788	{}	\N	2020-10-12 14:18:54.909543
28005	ATLC	2020-10-06	13.3	13.7298	13.0065	13.12	5559	{}	\N	2020-10-12 14:18:54.913842
28006	ATLC	2020-10-07	12.22	13.58	12.22	13.58	4911	{}	\N	2020-10-12 14:18:54.917947
28007	ATLC	2020-10-08	13.53	13.9	13.32	13.71	6361	{}	\N	2020-10-12 14:18:54.921858
28008	ATLC	2020-10-09	14	14	13.44	13.67	11289	{}	\N	2020-10-12 14:18:54.925673
28009	ATLC	2020-10-12	13.7	13.7	13.31	13.485	2275	{}	\N	2020-10-12 14:18:54.929609
28010	ATOS	2020-09-28	2.35	2.44	2.33	2.36	135745	{}	\N	2020-10-12 14:18:56.046773
28011	ATOS	2020-09-29	2.35	2.64	2.3487	2.52	309384	{}	\N	2020-10-12 14:18:56.057549
28012	ATOS	2020-09-30	2.4785	2.6	2.41	2.42	133905	{}	\N	2020-10-12 14:18:56.064155
28013	ATOS	2020-10-01	2.42	2.55	2.42	2.43	141358	{}	\N	2020-10-12 14:18:56.069891
28014	ATOS	2020-10-02	2.4	2.4905	2.35	2.4	110938	{}	\N	2020-10-12 14:18:56.074804
28015	ATOS	2020-10-05	2.44	2.44	2.26	2.3	155076	{}	\N	2020-10-12 14:18:56.080141
28016	ATOS	2020-10-06	2.31	2.36	2.19	2.26	156528	{}	\N	2020-10-12 14:18:56.085766
28017	ATOS	2020-10-07	2.25	2.4	2.2412	2.34	111973	{}	\N	2020-10-12 14:18:56.089927
28018	ATOS	2020-10-08	2.373	2.47	2.33	2.36	137768	{}	\N	2020-10-12 14:18:56.094061
28019	ATOS	2020-10-09	2.41	2.41	2.31	2.33	63022	{}	\N	2020-10-12 14:18:56.097997
28020	ATOS	2020-10-12	2.35	2.37	2.3	2.32	25621	{}	\N	2020-10-12 14:18:56.102235
28021	AUPH	2020-09-28	14.36	14.52	14.17	14.36	558976	{}	\N	2020-10-12 14:18:58.1118
28022	AUPH	2020-09-29	14.36	14.66	14.3	14.56	651740	{}	\N	2020-10-12 14:18:58.121782
28023	AUPH	2020-09-30	14.5599	14.89	14.49	14.73	801859	{}	\N	2020-10-12 14:18:58.128709
28024	AUPH	2020-10-01	14.79	14.9	14.46	14.63	992191	{}	\N	2020-10-12 14:18:58.134839
28025	AUPH	2020-10-02	14.4	14.58	14.02	14.04	961062	{}	\N	2020-10-12 14:18:58.139858
28026	AUPH	2020-10-05	14.16	15.05	14.11	15	1330060	{}	\N	2020-10-12 14:18:58.14485
28027	AUPH	2020-10-06	14.99	15.97	14.99	15.22	1831212	{}	\N	2020-10-12 14:18:58.149309
28028	AUPH	2020-10-07	15.78	15.78	15.15	15.48	1295424	{}	\N	2020-10-12 14:18:58.154087
28029	AUPH	2020-10-08	15.5	15.6999	15.43	15.5	761154	{}	\N	2020-10-12 14:18:58.158298
28030	AUPH	2020-10-09	15.71	15.87	15.47	15.68	552575	{}	\N	2020-10-12 14:18:58.162728
28031	AUPH	2020-10-12	15.84	15.86	15.54	15.63	102880	{}	\N	2020-10-12 14:18:58.166822
28032	AVEO	2020-09-28	5.91	5.98	5.52	5.6	318845	{}	\N	2020-10-12 14:18:59.046362
28033	AVEO	2020-09-29	5.6	5.83	5.42	5.79	390533	{}	\N	2020-10-12 14:18:59.056674
28034	AVEO	2020-09-30	5.79	6.24	5.76	5.94	433747	{}	\N	2020-10-12 14:18:59.063254
28035	AVEO	2020-10-01	5.94	6	5.75	5.93	237281	{}	\N	2020-10-12 14:18:59.06909
28036	AVEO	2020-10-02	5.83	6.16	5.83	6.02	360826	{}	\N	2020-10-12 14:18:59.074274
28037	AVEO	2020-10-05	6.04	6.49	6.04	6.48	548921	{}	\N	2020-10-12 14:18:59.079363
28038	AVEO	2020-10-06	6.34	6.7	6.2174	6.41	349396	{}	\N	2020-10-12 14:18:59.083641
28039	AVEO	2020-10-07	6.48	6.72	6.41	6.68	328073	{}	\N	2020-10-12 14:18:59.087813
28040	AVEO	2020-10-08	6.75	6.78	6.32	6.61	342431	{}	\N	2020-10-12 14:18:59.092181
28041	AVEO	2020-10-09	6.66	6.69	6.5	6.54	140965	{}	\N	2020-10-12 14:18:59.0965
28042	AVEO	2020-10-12	6.5	6.52	6.36	6.47	29800	{}	\N	2020-10-12 14:18:59.100372
28043	AVXL	2020-09-28	4.09	4.715	3.98	4.66	1758658	{}	\N	2020-10-12 14:19:00.396263
28044	AVXL	2020-09-29	4.72	4.73	4.32	4.68	1177849	{}	\N	2020-10-12 14:19:00.40411
28045	AVXL	2020-09-30	4.45	4.8186	4.42	4.55	1242395	{}	\N	2020-10-12 14:19:00.409887
28046	AVXL	2020-10-01	4.55	4.56	4.22	4.23	737815	{}	\N	2020-10-12 14:19:00.416155
28047	AVXL	2020-10-02	4.1	4.17	3.95	4.01	857870	{}	\N	2020-10-12 14:19:00.421564
28048	AVXL	2020-10-05	4.07	4.39	4.07	4.37	568407	{}	\N	2020-10-12 14:19:00.42679
28049	AVXL	2020-10-06	4.49	4.68	4.23	4.315	717769	{}	\N	2020-10-12 14:19:00.431268
28050	AVXL	2020-10-07	4.4	4.58	4.33	4.415	566961	{}	\N	2020-10-12 14:19:00.435794
28051	AVXL	2020-10-08	4.47	4.47	4.33	4.33	385681	{}	\N	2020-10-12 14:19:00.440067
28052	AVXL	2020-10-09	4.34	4.44	4.27	4.3	392379	{}	\N	2020-10-12 14:19:00.444386
28053	AVXL	2020-10-12	4.27	4.34	4.25	4.34	80601	{}	\N	2020-10-12 14:19:00.448791
28054	AXAS	2020-09-28	0.152	0.1527	0.1439	0.1462	3638041	{}	\N	2020-10-12 14:19:01.995463
28055	AXAS	2020-09-29	0.155	0.1588	0.1462	0.1563	8535349	{}	\N	2020-10-12 14:19:02.002964
28056	AXAS	2020-09-30	0.15	0.1535	0.145	0.146	3807993	{}	\N	2020-10-12 14:19:02.010069
28057	AXAS	2020-10-01	0.1463	0.1499	0.1401	0.1422	3816765	{}	\N	2020-10-12 14:19:02.015804
28058	AXAS	2020-10-02	0.14	0.1479	0.1327	0.144	3435930	{}	\N	2020-10-12 14:19:02.020676
28059	AXAS	2020-10-05	0.1489	0.1489	0.1412	0.1453	2917715	{}	\N	2020-10-12 14:19:02.025009
28060	AXAS	2020-10-06	0.1471	0.1488	0.1421	0.1434	3128727	{}	\N	2020-10-12 14:19:02.029155
28061	AXAS	2020-10-07	0.143	0.147	0.14	0.1447	3150469	{}	\N	2020-10-12 14:19:02.033358
28062	AXAS	2020-10-08	0.1446	0.1649	0.1415	0.1627	11622174	{}	\N	2020-10-12 14:19:02.037314
28063	AXAS	2020-10-09	0.1775	0.2792	0.171	0.1779	127076126	{}	\N	2020-10-12 14:19:02.041255
28064	AXAS	2020-10-12	0.1631	0.175	0.156	0.1587	5253561	{}	\N	2020-10-12 14:19:02.045004
28065	AXTA	2020-09-28	22.38	22.85	22.29	22.54	2253187	{}	\N	2020-10-12 14:19:03.576501
28066	AXTA	2020-09-29	22.68	22.68	22.04	22.09	5305608	{}	\N	2020-10-12 14:19:03.585308
28067	AXTA	2020-09-30	22.2	22.485	22.03	22.17	4645823	{}	\N	2020-10-12 14:19:03.591217
28068	AXTA	2020-10-01	22.31	23.6704	22.31	23.5	15396000	{}	\N	2020-10-12 14:19:03.596968
28069	AXTA	2020-10-02	23.06	24.19	23.0001	24	6531174	{}	\N	2020-10-12 14:19:03.601468
28070	AXTA	2020-10-05	24.31	24.775	24.24	24.62	5166071	{}	\N	2020-10-12 14:19:03.605867
28071	AXTA	2020-10-06	24.87	25.42	24.74	24.77	6076992	{}	\N	2020-10-12 14:19:03.61099
28072	AXTA	2020-10-07	25.17	25.56	25.135	25.29	5422150	{}	\N	2020-10-12 14:19:03.615129
28073	AXTA	2020-10-08	25.78	26.21	25.415	25.45	5736948	{}	\N	2020-10-12 14:19:03.61911
28074	AXTA	2020-10-09	25.69	25.97	25.375	25.69	3992591	{}	\N	2020-10-12 14:19:03.623255
28075	AXTA	2020-10-12	25.9	25.9	25.605	25.8	357507	{}	\N	2020-10-12 14:19:03.627018
28076	AZN	2020-09-28	55.07	55.34	54.6	54.75	2693247	{}	\N	2020-10-12 14:19:05.120653
28077	AZN	2020-09-29	54.71	55.2	54.705	55.11	4031272	{}	\N	2020-10-12 14:19:05.131693
28078	AZN	2020-09-30	55.02	55.45	54.61	54.8	2882184	{}	\N	2020-10-12 14:19:05.138281
28079	AZN	2020-10-01	55.2	55.31	54.22	54.3	4604480	{}	\N	2020-10-12 14:19:05.144284
28080	AZN	2020-10-02	54.42	54.76	54.13	54.22	5542877	{}	\N	2020-10-12 14:19:05.149066
28081	AZN	2020-10-05	54.47	54.85	54.18	54.49	6681490	{}	\N	2020-10-12 14:19:05.153962
28082	AZN	2020-10-06	54.15	54.53	53.675	53.71	3470973	{}	\N	2020-10-12 14:19:05.158083
28083	AZN	2020-10-07	53.81	53.93	53.33	53.53	3493793	{}	\N	2020-10-12 14:19:05.162171
28084	AZN	2020-10-08	53.68	54.29	53.58	54.09	2317117	{}	\N	2020-10-12 14:19:05.165867
28085	AZN	2020-10-09	54.39	54.85	54.32	54.72	2029743	{}	\N	2020-10-12 14:19:05.169774
28086	AZN	2020-10-12	55.19	55.3	54.97	55.24	845468	{}	\N	2020-10-12 14:19:05.173898
28087	BABA	2020-09-28	275.16	278.84	274.1989	276.01	8812078	{}	\N	2020-10-12 14:19:06.052768
28088	BABA	2020-09-29	275.43	279.3	274.9	276.93	15346776	{}	\N	2020-10-12 14:19:06.061596
28089	BABA	2020-09-30	284.01	295	283.71	293.98	49342786	{}	\N	2020-10-12 14:19:06.066818
28090	BABA	2020-10-01	295.48	295.59	288.25	290.05	31886518	{}	\N	2020-10-12 14:19:06.072457
28091	BABA	2020-10-02	286.62	291.9794	286.51	288.17	21961867	{}	\N	2020-10-12 14:19:06.077869
28092	BABA	2020-10-05	289.06	290	284.88	288.27	35128883	{}	\N	2020-10-12 14:19:06.082517
28093	BABA	2020-10-06	287.41	296.84	286.91	292.39	32128996	{}	\N	2020-10-12 14:19:06.086503
28094	BABA	2020-10-07	297.9	299.51	292.87	296.5	33081714	{}	\N	2020-10-12 14:19:06.091052
28095	BABA	2020-10-08	298.11	302.3	295.441	300.54	30859960	{}	\N	2020-10-12 14:19:06.095638
28096	BABA	2020-10-09	298.55	302.61	297.74	299.74	23035093	{}	\N	2020-10-12 14:19:06.099772
28097	BABA	2020-10-12	303.95	305.1	301.2848	304.85	6595866	{}	\N	2020-10-12 14:19:06.104288
28098	BANFP	2020-09-28	28.7	28.7	28.4	28.7	3270	{}	\N	2020-10-12 14:19:07.246251
28099	BANFP	2020-10-01	28.7	28.7	28.7	28.7	201	{}	\N	2020-10-12 14:19:07.254022
28100	BANFP	2020-10-02	27.97	28.335	27.97	28.335	344	{}	\N	2020-10-12 14:19:07.25947
28101	BANFP	2020-10-05	28.01	28.4415	28.01	28.3	533	{}	\N	2020-10-12 14:19:07.264157
28102	BANFP	2020-10-06	28.3	28.3	28.3	28.3	200	{}	\N	2020-10-12 14:19:07.268871
28103	BANFP	2020-10-08	28.3	28.32	28	28	1222	{}	\N	2020-10-12 14:19:07.273694
28104	BBBY	2020-09-28	14.76	15.33	14.17	14.88	9399257	{}	\N	2020-10-12 14:19:08.234794
28105	BBBY	2020-09-29	14.85	14.899	14.25	14.51	6822387	{}	\N	2020-10-12 14:19:08.245463
28106	BBBY	2020-09-30	14.36	15.38	14.31	14.98	11818236	{}	\N	2020-10-12 14:19:08.253131
28107	BBBY	2020-10-01	18.29	20.37	17.95	18.745	85104583	{}	\N	2020-10-12 14:19:08.259097
28108	BBBY	2020-10-02	18.22	20.9	18.21	20.6	33751319	{}	\N	2020-10-12 14:19:08.264181
28109	BBBY	2020-10-05	21.21	22.12	20.38	20.49	20927438	{}	\N	2020-10-12 14:19:08.268986
28110	BBBY	2020-10-06	20.55	21.28	19.7	19.97	16697362	{}	\N	2020-10-12 14:19:08.273254
28111	BBBY	2020-10-07	20.372	21.45	20.34	20.85	12590497	{}	\N	2020-10-12 14:19:08.277657
28112	BBBY	2020-10-08	21	21.865	20.69	21.76	8049038	{}	\N	2020-10-12 14:19:08.281823
28113	BBBY	2020-10-09	21.96	22.09	21.365	21.46	8044146	{}	\N	2020-10-12 14:19:08.286013
28114	BBBY	2020-10-12	21.46	21.85	21.4	21.7182	1078844	{}	\N	2020-10-12 14:19:08.290272
28115	AUTO	2020-09-28	3.065	3.25	3.03	3.22	224728	{}	\N	2020-10-12 14:19:09.261196
28116	AUTO	2020-09-29	3.161	3.25	3.03	3.19	146560	{}	\N	2020-10-12 14:19:09.270995
28117	AUTO	2020-09-30	3.25	3.3	3.09	3.14	147357	{}	\N	2020-10-12 14:19:09.277786
28118	AUTO	2020-10-01	3.13	3.36	3.05	3.27	588026	{}	\N	2020-10-12 14:19:09.283592
28119	AUTO	2020-10-02	3.17	3.3883	3.13	3.15	242553	{}	\N	2020-10-12 14:19:09.288914
28120	AUTO	2020-10-05	3.2	3.6	3.18	3.45	470377	{}	\N	2020-10-12 14:19:09.294159
28121	AUTO	2020-10-06	3.5	3.52	3.26	3.33	187951	{}	\N	2020-10-12 14:19:09.298958
28122	AUTO	2020-10-07	3.31	3.45	3.26	3.32	170362	{}	\N	2020-10-12 14:19:09.303236
28123	AUTO	2020-10-08	3.32	3.5499	3.3	3.5	133546	{}	\N	2020-10-12 14:19:09.307779
28124	AUTO	2020-10-09	3.52	3.88	3.43	3.74	543720	{}	\N	2020-10-12 14:19:09.312282
28125	AUTO	2020-10-12	3.95	4.4999	3.9	4.3747	240953	{}	\N	2020-10-12 14:19:09.31646
28126	AVGO	2020-09-28	361.68	368.43	359.03	367.3	2089832	{}	\N	2020-10-12 14:19:10.280544
28127	AVGO	2020-09-29	367.96	370.8	363.985	364.26	1293526	{}	\N	2020-10-12 14:19:10.286257
28128	AVGO	2020-09-30	363.99	371	361.75	364.32	2051177	{}	\N	2020-10-12 14:19:10.292842
28129	AVGO	2020-10-01	368.98	370.71	364.82	368.6	1483119	{}	\N	2020-10-12 14:19:10.299824
28130	AVGO	2020-10-02	360	364.92	356.15	356.83	1561515	{}	\N	2020-10-12 14:19:10.307277
28131	AVGO	2020-10-05	360.84	366.85	359.9	366.1	1854363	{}	\N	2020-10-12 14:19:10.3153
28132	AVGO	2020-10-06	366.11	372	362.02	363.98	1528646	{}	\N	2020-10-12 14:19:10.321889
28133	AVGO	2020-10-07	368.35	369.86	366.06	366.74	1218904	{}	\N	2020-10-12 14:19:10.328382
28134	AVGO	2020-10-08	369.89	374.29	368.31	373.33	1651143	{}	\N	2020-10-12 14:19:10.334517
28135	AVGO	2020-10-09	375	380	374	376.7	1603091	{}	\N	2020-10-12 14:19:10.339438
28136	AVGO	2020-10-12	381.79	382.5	379.79	381.36	276121	{}	\N	2020-10-12 14:19:10.344163
28137	AVYA	2020-09-28	14.07	14.27	14.01	14.12	846231	{}	\N	2020-10-12 14:19:11.976534
28138	AVYA	2020-09-29	14.19	15.2	14.17	15.06	2224085	{}	\N	2020-10-12 14:19:11.986615
28139	AVYA	2020-09-30	15.06	15.46	14.79	15.2	1977730	{}	\N	2020-10-12 14:19:11.994514
28140	AVYA	2020-10-01	15.35	16.44	15.17	16.22	5817775	{}	\N	2020-10-12 14:19:12.002971
28141	AVYA	2020-10-02	15.8	17.055	15.77	16.89	7170551	{}	\N	2020-10-12 14:19:12.018372
28142	AVYA	2020-10-05	17.11	18.14	17.1	17.93	5244456	{}	\N	2020-10-12 14:19:12.033853
28143	AVYA	2020-10-06	18.01	18.36	17.25	17.44	5349126	{}	\N	2020-10-12 14:19:12.049745
28144	AVYA	2020-10-07	17.98	18.4	17.68	18.3	6431388	{}	\N	2020-10-12 14:19:12.071538
28145	AVYA	2020-10-08	18.64	18.96	18.45	18.95	10448100	{}	\N	2020-10-12 14:19:12.097186
28146	AVYA	2020-10-09	19.24	19.7	18.55	18.55	5920817	{}	\N	2020-10-12 14:19:12.113589
28147	AVYA	2020-10-12	18.71	19.03	18.61	18.825	470563	{}	\N	2020-10-12 14:19:12.12248
28148	AXSM	2020-09-28	77.37	77.99	75.79	76.46	221389	{}	\N	2020-10-12 14:19:13.691902
28149	AXSM	2020-09-29	72.07	73.9999	70	72.36	806148	{}	\N	2020-10-12 14:19:13.70109
28150	AXSM	2020-09-30	71.7626	73.64	69.41	71.25	486591	{}	\N	2020-10-12 14:19:13.706553
28151	AXSM	2020-10-01	71.65	73.06	70.25	72.21	297101	{}	\N	2020-10-12 14:19:13.713169
28152	AXSM	2020-10-02	70.55	72.43	70.18	70.5	229717	{}	\N	2020-10-12 14:19:13.71877
28153	AXSM	2020-10-05	71.24	74.415	71.24	74.12	243367	{}	\N	2020-10-12 14:19:13.723208
28154	AXSM	2020-10-06	74.35	75.57	73.08	73.29	416182	{}	\N	2020-10-12 14:19:13.727661
28155	AXSM	2020-10-07	73.5001	74.88	72.42	72.92	280851	{}	\N	2020-10-12 14:19:13.732707
28156	AXSM	2020-10-08	73.81	76.1	72.85	74.78	383092	{}	\N	2020-10-12 14:19:13.737304
28157	AXSM	2020-10-09	74.18	75.5583	73.22	74.6	178965	{}	\N	2020-10-12 14:19:13.741521
28158	AXSM	2020-10-12	75.5	75.5	73.12	74.2796	38536	{}	\N	2020-10-12 14:19:13.746007
28159	AZO	2020-09-28	1150.36	1162.5	1140.35	1155.56	201504	{}	\N	2020-10-12 14:19:14.726799
28160	AZO	2020-09-29	1150.78	1167.06	1146.0001	1154.92	347914	{}	\N	2020-10-12 14:19:14.738402
28161	AZO	2020-09-30	1153.64	1183.575	1153.64	1177.64	495615	{}	\N	2020-10-12 14:19:14.745388
28162	AZO	2020-10-01	1181.06	1192.95	1168.19	1181.72	424994	{}	\N	2020-10-12 14:19:14.751347
28163	AZO	2020-10-02	1169.81	1183.76	1158.86	1176.99	263916	{}	\N	2020-10-12 14:19:14.757567
28164	AZO	2020-10-05	1174.17	1191.76	1174.17	1178.27	317605	{}	\N	2020-10-12 14:19:14.763606
28165	AZO	2020-10-06	1180.75	1182.84	1132.12	1138.53	499573	{}	\N	2020-10-12 14:19:14.769828
28166	AZO	2020-10-07	1146.46	1155.3415	1136	1139.99	372154	{}	\N	2020-10-12 14:19:14.774527
28167	AZO	2020-10-08	1146.11	1157.48	1145.92	1150	298387	{}	\N	2020-10-12 14:19:14.779377
28168	AZO	2020-10-09	1156.65	1156.65	1137.08	1137.74	319802	{}	\N	2020-10-12 14:19:14.784698
28169	AZO	2020-10-12	1142.62	1146.71	1131.86	1133.945	34890	{}	\N	2020-10-12 14:19:14.78931
28170	BAF	2020-09-28	14.06	14.27	14.06	14.1196	5009	{}	\N	2020-10-12 14:19:15.825146
28171	BAF	2020-09-29	14.05	14.14	14.05	14.125	15816	{}	\N	2020-10-12 14:19:15.842174
28172	BAF	2020-09-30	14.09	14.13	14.09	14.13	14920	{}	\N	2020-10-12 14:19:15.850094
28173	BAF	2020-10-01	14.23	14.23	14.1177	14.18	14799	{}	\N	2020-10-12 14:19:15.855672
28174	BAF	2020-10-02	14.16	14.27	14.16	14.27	36366	{}	\N	2020-10-12 14:19:15.861097
28175	BAF	2020-10-05	14.22	14.26	14.19	14.25	27570	{}	\N	2020-10-12 14:19:15.865692
28176	BAF	2020-10-06	14.19	14.29	14.16	14.28	28036	{}	\N	2020-10-12 14:19:15.870338
28177	BAF	2020-10-07	14.22	14.31	14.22	14.31	20632	{}	\N	2020-10-12 14:19:15.875174
28178	BAF	2020-10-08	14.33	14.34	14.255	14.34	31898	{}	\N	2020-10-12 14:19:15.879895
28179	BAF	2020-10-09	14.32	14.4	14.29	14.4	21026	{}	\N	2020-10-12 14:19:15.885118
28180	BAF	2020-10-12	14.35	14.4	14.346	14.35	13568	{}	\N	2020-10-12 14:19:15.889424
28181	BANX	2020-09-28	18.53	19.18	18.31	18.6009	7087	{}	\N	2020-10-12 14:19:17.186418
28182	BANX	2020-09-29	18.5675	19.15	18.49	19.09	11403	{}	\N	2020-10-12 14:19:17.198244
28183	BANX	2020-09-30	19.58	19.6	18.6101	19.41	15514	{}	\N	2020-10-12 14:19:17.205606
28184	BANX	2020-10-01	19.3784	19.3784	18.81	19	8047	{}	\N	2020-10-12 14:19:17.211926
28185	BANX	2020-10-02	18.8	18.9	18.6961	18.82	3782	{}	\N	2020-10-12 14:19:17.217538
28186	BANX	2020-10-05	18.82	19.165	18.67	18.9	7648	{}	\N	2020-10-12 14:19:17.222499
28187	BANX	2020-10-06	18.7995	19.45	18.62	18.8	16131	{}	\N	2020-10-12 14:19:17.227272
28188	BANX	2020-10-07	18.75	19.45	18.6	18.63	19129	{}	\N	2020-10-12 14:19:17.253203
28189	BANX	2020-10-08	18.74	19.03	18.49	18.7	21158	{}	\N	2020-10-12 14:19:17.266046
28190	BANX	2020-10-09	18.675	19	18.56	18.58	12899	{}	\N	2020-10-12 14:19:17.273171
28191	BANX	2020-10-12	18.5504	18.5504	18.5504	18.5504	213	{}	\N	2020-10-12 14:19:17.279412
28192	BBDO	2020-09-28	3.405	3.405	3.26	3.26	1220	{}	\N	2020-10-12 14:19:18.813619
28193	BBDO	2020-09-29	3.24	3.24	3.11	3.11	31992	{}	\N	2020-10-12 14:19:18.82199
28194	BBDO	2020-09-30	3.15	3.2599	3.15	3.195	7593	{}	\N	2020-10-12 14:19:18.827434
28195	BBDO	2020-10-01	3.15	3.234	3.1296	3.21	38132	{}	\N	2020-10-12 14:19:18.832156
28196	BBDO	2020-10-02	3.22	3.26	3.2	3.22	43801	{}	\N	2020-10-12 14:19:18.837223
28197	BBDO	2020-10-05	3.22	3.331	3.19	3.3	12503	{}	\N	2020-10-12 14:19:18.841389
28198	BBDO	2020-10-06	3.48	3.48	3.24	3.306	12442	{}	\N	2020-10-12 14:19:18.845303
28199	BBDO	2020-10-07	3.28	3.28	3.259	3.26	7157	{}	\N	2020-10-12 14:19:18.849557
28200	BBDO	2020-10-08	3.3	3.41	3.28	3.39	39200	{}	\N	2020-10-12 14:19:18.854166
28201	BBDO	2020-10-09	3.4296	3.5	3.41	3.43	17598	{}	\N	2020-10-12 14:19:18.858339
28202	BCM	2020-09-28	26.74	26.9011	26.74	26.84	945	{}	\N	2020-10-12 14:19:21.397209
28203	BCM	2020-09-29	26.625	26.695	26.57	26.695	420	{}	\N	2020-10-12 14:19:21.403838
28204	BCM	2020-09-30	26.85	26.92	26.85	26.92	1135	{}	\N	2020-10-12 14:19:21.409263
28205	BCM	2020-10-01	26.55	26.6149	26.55	26.6149	681	{}	\N	2020-10-12 14:19:21.414083
28206	BCM	2020-10-02	26.28	26.53	26.28	26.45	8237	{}	\N	2020-10-12 14:19:21.41888
28207	BCM	2020-10-05	26.7862	26.99	26.7862	26.98	758	{}	\N	2020-10-12 14:19:21.424475
28208	BCM	2020-10-06	27.1799	27.1799	26.905	26.905	916	{}	\N	2020-10-12 14:19:21.430029
28209	BCM	2020-10-07	27.045	27.045	27.045	27.045	177	{}	\N	2020-10-12 14:19:21.434697
28210	BCM	2020-10-08	27.235	27.235	27.235	27.235	349	{}	\N	2020-10-12 14:19:21.439697
28211	BCM	2020-10-09	27.575	27.575	27.575	27.575	2	{}	\N	2020-10-12 14:19:21.445558
28212	BDL	2020-09-28	16.15	16.3961	16.15	16.3961	1783	{}	\N	2020-10-12 14:19:23.156153
28213	BDL	2020-09-30	17.84	17.84	17.84	17.84	154	{}	\N	2020-10-12 14:19:23.17356
28214	BDL	2020-10-01	17.84	17.84	17.84	17.84	836	{}	\N	2020-10-12 14:19:23.181267
28215	BDL	2020-10-06	17.6444	17.6444	17.39	17.39	1438	{}	\N	2020-10-12 14:19:23.187336
28216	BDL	2020-10-08	16.55	17.2301	16.55	17.2301	424	{}	\N	2020-10-12 14:19:23.192488
28217	AUY	2020-09-28	5.6899	5.75	5.57	5.68	11941088	{}	\N	2020-10-12 14:19:24.787474
28218	AUY	2020-09-29	5.72	5.77	5.635	5.7	22019856	{}	\N	2020-10-12 14:19:24.795961
28219	AUY	2020-09-30	5.66	5.72	5.555	5.68	24485866	{}	\N	2020-10-12 14:19:24.802619
28220	AUY	2020-10-01	5.75	5.84	5.69	5.73	23683521	{}	\N	2020-10-12 14:19:24.808972
28221	AUY	2020-10-02	5.7	5.825	5.67	5.77	23315602	{}	\N	2020-10-12 14:19:24.814414
28222	AUY	2020-10-05	5.79	5.98	5.78	5.92	32245770	{}	\N	2020-10-12 14:19:24.818606
28223	AUY	2020-10-06	5.96	6.05	5.59	5.61	35689821	{}	\N	2020-10-12 14:19:24.822484
28224	AUY	2020-10-07	5.69	5.72	5.5	5.54	39521522	{}	\N	2020-10-12 14:19:24.826317
28225	AUY	2020-10-08	5.68	5.9	5.67	5.9	38530463	{}	\N	2020-10-12 14:19:24.829849
28226	AUY	2020-10-09	6.04	6.18	5.94	6.17	40839423	{}	\N	2020-10-12 14:19:24.833364
28227	AUY	2020-10-12	6.15	6.19	6.09	6.1725	3466668	{}	\N	2020-10-12 14:19:24.836965
28228	AVK	2020-09-28	13.69	13.8	13.64	13.73	123964	{}	\N	2020-10-12 14:19:26.048926
28229	AVK	2020-09-29	13.76	13.78	13.6	13.76	223654	{}	\N	2020-10-12 14:19:26.058171
28230	AVK	2020-09-30	13.85	13.89	13.69	13.81	260429	{}	\N	2020-10-12 14:19:26.064597
28231	AVK	2020-10-01	13.8	13.96	13.8	13.96	198562	{}	\N	2020-10-12 14:19:26.070699
28232	AVK	2020-10-02	13.79	13.91	13.78	13.88	199488	{}	\N	2020-10-12 14:19:26.07644
28233	AVK	2020-10-05	13.94	14.12	13.94	14.1	183382	{}	\N	2020-10-12 14:19:26.081287
28234	AVK	2020-10-06	14.12	14.26	14.01	14.1	232154	{}	\N	2020-10-12 14:19:26.085272
28235	AVK	2020-10-07	14.25	14.55	14.16	14.42	299729	{}	\N	2020-10-12 14:19:26.089136
28236	AVK	2020-10-08	14.51	14.55	14.45	14.53	160635	{}	\N	2020-10-12 14:19:26.093108
28237	AVK	2020-10-09	14.53	14.6	14.5017	14.53	194082	{}	\N	2020-10-12 14:19:26.096634
28238	AVK	2020-10-12	14.56	14.6	14.56	14.6	47086	{}	\N	2020-10-12 14:19:26.100616
28239	AWR	2020-09-28	74.01	74.95	73.91	74.33	235930	{}	\N	2020-10-12 14:19:27.104123
28240	AWR	2020-09-29	74.46	74.89	73.75	74.45	220300	{}	\N	2020-10-12 14:19:27.113596
28241	AWR	2020-09-30	74.96	75.7	74.27	74.95	398698	{}	\N	2020-10-12 14:19:27.121007
28242	AWR	2020-10-01	74.93	76.2	74.63	76.1	317509	{}	\N	2020-10-12 14:19:27.127582
28243	AWR	2020-10-02	75.36	77.07	75.3	76.72	304412	{}	\N	2020-10-12 14:19:27.133808
28244	AWR	2020-10-05	76.88	76.99	75.89	76.4	258260	{}	\N	2020-10-12 14:19:27.138907
28245	AWR	2020-10-06	77.1	77.59	76.13	76.78	467119	{}	\N	2020-10-12 14:19:27.143815
28246	AWR	2020-10-07	76.86	77.075	74.86	75.92	387131	{}	\N	2020-10-12 14:19:27.149297
28247	AWR	2020-10-08	76.42	77.24	76.0484	76.54	197414	{}	\N	2020-10-12 14:19:27.15401
28248	AWR	2020-10-09	76.92	77.25	76	76.67	263470	{}	\N	2020-10-12 14:19:27.158734
28249	AWR	2020-10-12	76.58	77.72	76.58	77.55	30699	{}	\N	2020-10-12 14:19:27.164012
28250	AXGN	2020-09-28	11.91	12.18	11.81	11.94	265614	{}	\N	2020-10-12 14:19:28.770155
28251	AXGN	2020-09-29	11.94	12.03	11.43	11.66	278728	{}	\N	2020-10-12 14:19:28.779677
28252	AXGN	2020-09-30	11.76	11.9375	11.48	11.63	205239	{}	\N	2020-10-12 14:19:28.786141
28253	AXGN	2020-10-01	11.63	11.74	11.04	11.5	361417	{}	\N	2020-10-12 14:19:28.791836
28254	AXGN	2020-10-02	11.13	11.61	11.09	11.51	221337	{}	\N	2020-10-12 14:19:28.797169
28255	AXGN	2020-10-05	11.63	12.33	11.56	12.02	218106	{}	\N	2020-10-12 14:19:28.802307
28256	AXGN	2020-10-06	12.12	12.4	11.75	11.78	136664	{}	\N	2020-10-12 14:19:28.806835
28257	AXGN	2020-10-07	11.82	12.4737	11.82	12.19	205534	{}	\N	2020-10-12 14:19:28.811467
28258	AXGN	2020-10-08	12.19	12.41	12.01	12.17	155376	{}	\N	2020-10-12 14:19:28.816062
28259	AXGN	2020-10-09	12.255	13.04	12.25	13	214667	{}	\N	2020-10-12 14:19:28.820464
28260	AXGN	2020-10-12	13.25	13.51	13.16	13.4689	25388	{}	\N	2020-10-12 14:19:28.824817
28261	AY	2020-09-28	27.68	27.805	27.41	27.61	275925	{}	\N	2020-10-12 14:19:29.819965
28262	AY	2020-09-29	27.77	28.32	27.68	28.105	633709	{}	\N	2020-10-12 14:19:29.831617
28263	AY	2020-09-30	28.16	28.67	28.16	28.61	528315	{}	\N	2020-10-12 14:19:29.838923
28264	AY	2020-10-01	28.81	28.85	28.125	28.47	565419	{}	\N	2020-10-12 14:19:29.845139
28265	AY	2020-10-02	27.96	28.4	27.96	28.26	292308	{}	\N	2020-10-12 14:19:29.850731
28266	AY	2020-10-05	28.31	29.87	28.31	29.72	772821	{}	\N	2020-10-12 14:19:29.85615
28267	AY	2020-10-06	29.72	30.24	29.525	29.83	703714	{}	\N	2020-10-12 14:19:29.860688
28268	AY	2020-10-07	30.25	30.89	30.225	30.37	860159	{}	\N	2020-10-12 14:19:29.865136
28269	AY	2020-10-08	30.71	31.45	30.71	31.28	784191	{}	\N	2020-10-12 14:19:29.86971
28270	AY	2020-10-09	30.84	31.12	29.45	30.04	1125543	{}	\N	2020-10-12 14:19:29.874428
28271	AY	2020-10-12	29.99	29.99	28.85	29.08	174781	{}	\N	2020-10-12 14:19:29.878998
28272	AZUL	2020-09-28	14.9	15.19	13.745	13.79	2970580	{}	\N	2020-10-12 14:19:30.782198
28273	AZUL	2020-09-29	13.46	13.58	12.69	12.7	5552280	{}	\N	2020-10-12 14:19:30.792134
28274	AZUL	2020-09-30	12.81	13.37	12.79	13.16	4973018	{}	\N	2020-10-12 14:19:30.798946
28275	AZUL	2020-10-01	13.48	13.89	13.26	13.71	6447503	{}	\N	2020-10-12 14:19:30.804655
28276	AZUL	2020-10-02	13.43	13.6	12.93	12.96	6669030	{}	\N	2020-10-12 14:19:30.809726
28277	AZUL	2020-10-05	13.16	13.2773	12.82	13.11	3936919	{}	\N	2020-10-12 14:19:30.814689
28278	AZUL	2020-10-06	13.76	14.4	13.72	13.97	6010814	{}	\N	2020-10-12 14:19:30.81895
28279	AZUL	2020-10-07	14.12	14.31	13.26	13.26	10951915	{}	\N	2020-10-12 14:19:30.823032
28280	AZUL	2020-10-08	13.46	13.5794	13.2401	13.54	3060760	{}	\N	2020-10-12 14:19:30.827482
28281	AZUL	2020-10-09	13.85	14.125	13.69	13.8	3656992	{}	\N	2020-10-12 14:19:30.831449
28282	AZUL	2020-10-12	13.82	13.86	13.4425	13.675	505187	{}	\N	2020-10-12 14:19:30.835739
28283	BAH	2020-09-28	85.64	86.285	84.88	84.95	452102	{}	\N	2020-10-12 14:19:31.789506
28284	BAH	2020-09-29	85.02	85.33	84.14	84.24	1160264	{}	\N	2020-10-12 14:19:31.801029
28285	BAH	2020-09-30	84.33	84.38	82.42	82.98	1801048	{}	\N	2020-10-12 14:19:31.808538
28286	BAH	2020-10-01	83.31	83.79	82.48	83.05	1045731	{}	\N	2020-10-12 14:19:31.815206
28287	BAH	2020-10-02	82.39	83.41	81.54	82.85	1181862	{}	\N	2020-10-12 14:19:31.82006
28288	BAH	2020-10-05	82.91	83.3	81.66	82.44	1676130	{}	\N	2020-10-12 14:19:31.824985
28289	BAH	2020-10-06	82.46	83.38	81.68	82.26	2268306	{}	\N	2020-10-12 14:19:31.830236
28290	BAH	2020-10-07	82.88	82.975	81.2	81.59	1854124	{}	\N	2020-10-12 14:19:31.834493
28291	BAH	2020-10-08	82.23	82.805	81.23	81.36	1748516	{}	\N	2020-10-12 14:19:31.8385
28292	BAH	2020-10-09	81.99	82.53	80.8	81.16	2760936	{}	\N	2020-10-12 14:19:31.842543
28293	BAH	2020-10-12	81.73	81.83	80.87	81.31	314866	{}	\N	2020-10-12 14:19:31.846865
28294	BAR	2020-09-28	18.63	18.72	18.5483	18.705	354617	{}	\N	2020-10-12 14:19:33.03961
28295	BAR	2020-09-29	18.78	18.88	18.74	18.87	248399	{}	\N	2020-10-12 14:19:33.046354
28296	BAR	2020-09-30	18.83	18.9199	18.7123	18.75	384421	{}	\N	2020-10-12 14:19:33.0527
28297	BAR	2020-10-01	18.94	19.01	18.86	18.94	395110	{}	\N	2020-10-12 14:19:33.057926
28298	BAR	2020-10-02	18.94	18.985	18.87	18.9	348395	{}	\N	2020-10-12 14:19:33.063121
28299	BAR	2020-10-05	18.97	19.08	18.957	19.01	305693	{}	\N	2020-10-12 14:19:33.067542
28300	BAR	2020-10-06	19.07	19.1	18.76	18.77	267066	{}	\N	2020-10-12 14:19:33.071606
28301	BAR	2020-10-07	18.75	18.785	18.69	18.76	376032	{}	\N	2020-10-12 14:19:33.076147
28302	BAR	2020-10-08	18.86	18.9	18.7119	18.83	268914	{}	\N	2020-10-12 14:19:33.081149
28303	BAR	2020-10-09	19.07	19.19	19.06	19.185	679970	{}	\N	2020-10-12 14:19:33.085833
28304	BAR	2020-10-12	19.14	19.15	19.1	19.14	78493	{}	\N	2020-10-12 14:19:33.09033
28305	BBP	2020-09-28	47.8	47.8	47.25	47.3367	3238	{}	\N	2020-10-12 14:19:34.270006
28306	BBP	2020-09-29	47.42	47.5921	47.42	47.5921	184	{}	\N	2020-10-12 14:19:34.279557
28307	BBP	2020-09-30	47.58	48.0799	47.58	47.6724	3949	{}	\N	2020-10-12 14:19:34.287149
28308	BBP	2020-10-01	47.968	48	47.968	48	1138	{}	\N	2020-10-12 14:19:34.293902
28309	BBP	2020-10-02	47.92	47.9932	47.0046	47.0046	3152	{}	\N	2020-10-12 14:19:34.299453
28310	BBP	2020-10-05	48.26	48.88	48.26	48.645	2940	{}	\N	2020-10-12 14:19:34.304161
28311	BBP	2020-10-06	48.7	49.01	48.5163	48.5163	1874	{}	\N	2020-10-12 14:19:34.308471
28312	BBP	2020-10-07	49.08	49.8485	49.08	49.8485	901	{}	\N	2020-10-12 14:19:34.313077
28313	BBP	2020-10-08	49.69	50.0854	49.69	50.0854	14079	{}	\N	2020-10-12 14:19:34.317433
28314	BBP	2020-10-09	50.1953	50.1953	50.13	50.16	1731	{}	\N	2020-10-12 14:19:34.321649
28315	BBP	2020-10-12	50.22	50.22	50.16	50.16	822	{}	\N	2020-10-12 14:19:34.325879
28316	BCI	2020-09-28	19.89	20.01	19.79	19.93	22549	{}	\N	2020-10-12 14:19:35.828352
28317	BCI	2020-09-29	19.81	19.81	19.63	19.71	56097	{}	\N	2020-10-12 14:19:35.838167
28318	BCI	2020-09-30	19.66	19.935	19.66	19.91	22899	{}	\N	2020-10-12 14:19:35.84454
28319	BCI	2020-10-01	19.869	19.869	19.625	19.74	42898	{}	\N	2020-10-12 14:19:35.850261
28320	BCI	2020-10-02	19.57	19.76	19.5435	19.56	78812	{}	\N	2020-10-12 14:19:35.855112
28321	BCI	2020-10-05	19.8	20.0599	19.8	19.99	41129	{}	\N	2020-10-12 14:19:35.860659
28322	BCI	2020-10-06	20.05	20.22	19.9	19.92	64159	{}	\N	2020-10-12 14:19:35.86475
28323	BCI	2020-10-07	20.17	20.28	20.0788	20.22	128949	{}	\N	2020-10-12 14:19:35.868701
28324	BCI	2020-10-08	20.3	20.3926	20.2293	20.31	20475	{}	\N	2020-10-12 14:19:35.872862
28325	BCI	2020-10-09	20.48	20.665	20.48	20.65	25611	{}	\N	2020-10-12 14:19:35.877245
28326	BCI	2020-10-12	20.71	20.71	20.5316	20.56	2851	{}	\N	2020-10-12 14:19:35.881325
28327	BCPC	2020-09-28	96.41	97.79	96.41	96.7	159322	{}	\N	2020-10-12 14:19:36.815873
28328	BCPC	2020-09-29	96.83	97.87	96.61	97.24	71581	{}	\N	2020-10-12 14:19:36.82534
28329	BCPC	2020-09-30	96.2	99.275	96.2	97.66	56350	{}	\N	2020-10-12 14:19:36.832857
28330	BCPC	2020-10-01	97.57	98.9	97.535	98.75	89569	{}	\N	2020-10-12 14:19:36.838765
28331	BCPC	2020-10-02	97.06	99.42	97.06	98.58	92626	{}	\N	2020-10-12 14:19:36.843931
28332	BCPC	2020-10-05	99.08	100.41	98.21	99.26	107956	{}	\N	2020-10-12 14:19:36.848826
28333	BCPC	2020-10-06	99.58	101.88	99.31	100.89	189344	{}	\N	2020-10-12 14:19:36.853219
28334	BCPC	2020-10-07	101.85	103.5	101.14	101.14	128851	{}	\N	2020-10-12 14:19:36.858095
28335	BCPC	2020-10-08	100.31	102.84	100.31	102.08	67836	{}	\N	2020-10-12 14:19:36.862047
28336	BCPC	2020-10-09	102.4	104.87	102.4	103.42	55583	{}	\N	2020-10-12 14:19:36.865933
28337	BCPC	2020-10-12	103.31	104.68	103.0368	104.345	15422	{}	\N	2020-10-12 14:19:36.870048
28338	BDCS	2020-09-28	15.21	15.21	14.2702	14.47	3518	{}	\N	2020-10-12 14:19:38.27
28339	BDCS	2020-09-29	14.28	14.35	14.14	14.27	10031	{}	\N	2020-10-12 14:19:38.280396
28340	BDCS	2020-09-30	14.41	14.67	14.225	14.67	1175	{}	\N	2020-10-12 14:19:38.288063
28341	BDCS	2020-10-01	14.18	14.55	14.18	14.55	3934	{}	\N	2020-10-12 14:19:38.294041
28342	BDCS	2020-10-02	14.29	14.7	14.29	14.645	6955	{}	\N	2020-10-12 14:19:38.299068
28343	BDCS	2020-10-05	14.65	14.65	14.5226	14.57	3369	{}	\N	2020-10-12 14:19:38.303687
28344	BDCS	2020-10-06	14.55	14.65	14.5	14.65	32317	{}	\N	2020-10-12 14:19:38.307949
28345	BDCS	2020-10-07	14.8	14.81	14.48	14.81	20012	{}	\N	2020-10-12 14:19:38.312295
28346	BDCS	2020-10-08	14.85	14.85	14.6103	14.66	2104	{}	\N	2020-10-12 14:19:38.31635
28347	BDCS	2020-10-09	14.295	14.34	14.27	14.3	2801	{}	\N	2020-10-12 14:19:38.320494
28348	BDCS	2020-10-12	14.19	14.2025	14.16	14.17	4763	{}	\N	2020-10-12 14:19:38.324612
28349	AXL	2020-09-28	5.17	5.69	5.15	5.6	2821220	{}	\N	2020-10-12 14:19:39.3321
28350	AXL	2020-09-29	5.59	5.62	5.37	5.58	4058664	{}	\N	2020-10-12 14:19:39.346025
28351	AXL	2020-09-30	5.58	5.91	5.55	5.77	3864895	{}	\N	2020-10-12 14:19:39.352026
28352	AXL	2020-10-01	5.85	6.09	5.66	6.07	4521894	{}	\N	2020-10-12 14:19:39.356985
28353	AXL	2020-10-02	5.75	6.295	5.7299	6.28	3687597	{}	\N	2020-10-12 14:19:39.361342
28354	AXL	2020-10-05	6.42	6.5	6.23	6.38	4424479	{}	\N	2020-10-12 14:19:39.365542
28355	AXL	2020-10-06	6.48	6.55	6.15	6.18	4388530	{}	\N	2020-10-12 14:19:39.369429
28356	AXL	2020-10-07	6.32	6.63	6.25	6.59	3898457	{}	\N	2020-10-12 14:19:39.373971
28357	AXL	2020-10-08	6.68	6.93	6.62	6.8	4604545	{}	\N	2020-10-12 14:19:39.378015
28358	AXL	2020-10-09	6.9	6.95	6.755	6.85	2835792	{}	\N	2020-10-12 14:19:39.381784
28359	AXL	2020-10-12	6.83	6.9901	6.83	6.9402	265541	{}	\N	2020-10-12 14:19:39.385414
28360	AYI	2020-09-28	99.31	102.64	99.24	101.55	705021	{}	\N	2020-10-12 14:19:40.84189
28361	AYI	2020-09-29	101.4	102.48	100.47	100.99	1008261	{}	\N	2020-10-12 14:19:40.85013
28362	AYI	2020-09-30	101.65	103.86	101.44	102.35	942117	{}	\N	2020-10-12 14:19:40.856445
28363	AYI	2020-10-01	103.42	104.65	102.03	102.64	1228930	{}	\N	2020-10-12 14:19:40.862156
28364	AYI	2020-10-02	100.26	106.05	100.26	105.61	1094694	{}	\N	2020-10-12 14:19:40.866965
28365	AYI	2020-10-05	106.99	110.69	106.6	109.99	1031370	{}	\N	2020-10-12 14:19:40.872119
28366	AYI	2020-10-06	110.75	111.46	108.9206	109.69	1099877	{}	\N	2020-10-12 14:19:40.876867
28367	AYI	2020-10-07	111.38	114.76	111.32	113.26	1221100	{}	\N	2020-10-12 14:19:40.881732
28368	AYI	2020-10-08	113.63	113.64	101.18	101.54	4739782	{}	\N	2020-10-12 14:19:40.886602
28369	AYI	2020-10-09	102.8	103.97	99.18	99.62	2158726	{}	\N	2020-10-12 14:19:40.892024
28370	AYI	2020-10-12	100.21	100.21	98.1	98.88	230631	{}	\N	2020-10-12 14:19:40.898375
29730	BNTC	2020-09-30	6.109	7	5.9326	6.28	23144	{}	\N	2020-10-12 14:22:44.39086
28371	AZRE	2020-09-28	29.76	30.06	28.82	30.04	164163	{}	\N	2020-10-12 14:19:45.011177
28372	AZRE	2020-09-29	29.8	32.68	29.37	31.59	2518327	{}	\N	2020-10-12 14:19:45.024797
28373	AZRE	2020-09-30	32	32	29.66	29.8	353234	{}	\N	2020-10-12 14:19:45.031343
28374	AZRE	2020-10-01	30.17	32.5	29.8901	32.11	762324	{}	\N	2020-10-12 14:19:45.036994
28375	AZRE	2020-10-02	31.4	32.35	30.77	31.21	247992	{}	\N	2020-10-12 14:19:45.041769
28376	AZRE	2020-10-05	31.47	32.5	31.47	32.02	169997	{}	\N	2020-10-12 14:19:45.046603
28377	AZRE	2020-10-06	32.28	33.4604	31.7668	32.1	419006	{}	\N	2020-10-12 14:19:45.051276
28378	AZRE	2020-10-07	32.31	32.9799	31.71	32.37	532307	{}	\N	2020-10-12 14:19:45.055989
28379	AZRE	2020-10-08	32.35	32.96	31.8	31.93	205755	{}	\N	2020-10-12 14:19:45.060364
28380	AZRE	2020-10-09	31.94	32.24	30.24	31.03	224365	{}	\N	2020-10-12 14:19:45.06436
28381	AZRE	2020-10-12	30.93	31.28	30.41	30.85	32676	{}	\N	2020-10-12 14:19:45.069243
28382	BANR	2020-09-28	32.13	32.905	32.1	32.62	139517	{}	\N	2020-10-12 14:19:49.349542
28383	BANR	2020-09-29	32.46	32.525	31.69	32.33	91112	{}	\N	2020-10-12 14:19:49.357456
28384	BANR	2020-09-30	32.59	32.93	32.01	32.26	143780	{}	\N	2020-10-12 14:19:49.362762
28385	BANR	2020-10-01	32.15	33.38	31.52	32.43	130600	{}	\N	2020-10-12 14:19:49.36824
28386	BANR	2020-10-02	31.74	33.635	31.49	33.36	153386	{}	\N	2020-10-12 14:19:49.373367
28387	BANR	2020-10-05	33.92	34.94	33.49	34.87	148152	{}	\N	2020-10-12 14:19:49.377441
28388	BANR	2020-10-06	35.42	36.46	35.09	35.3	308357	{}	\N	2020-10-12 14:19:49.381255
28389	BANR	2020-10-07	35.81	36.64	35.52	36.12	232310	{}	\N	2020-10-12 14:19:49.38499
28390	BANR	2020-10-08	36.63	36.76	34.0001	36.65	185168	{}	\N	2020-10-12 14:19:49.389012
28391	BANR	2020-10-09	36.5	36.95	35.79	35.83	123320	{}	\N	2020-10-12 14:19:49.392675
28392	BANR	2020-10-12	35.83	36.67	35.64	36	16713	{}	\N	2020-10-12 14:19:49.396327
28393	BB	2020-09-28	4.86	4.86	4.6	4.61	6934725	{}	\N	2020-10-12 14:19:50.622503
28394	BB	2020-09-29	4.63	4.6878	4.49	4.63	11655906	{}	\N	2020-10-12 14:19:50.633385
28395	BB	2020-09-30	4.65	4.76	4.58	4.59	7390488	{}	\N	2020-10-12 14:19:50.640286
28396	BB	2020-10-01	4.59	4.64	4.52	4.58	9669238	{}	\N	2020-10-12 14:19:50.645789
28397	BB	2020-10-02	4.49	4.5	4.37	4.44	12474999	{}	\N	2020-10-12 14:19:50.651139
28398	BB	2020-10-05	4.46	4.51	4.43	4.5	6223552	{}	\N	2020-10-12 14:19:50.656751
28399	BB	2020-10-06	4.53	4.68	4.51	4.55	7952648	{}	\N	2020-10-12 14:19:50.661462
28400	BB	2020-10-07	4.6	4.64	4.52	4.57	4258351	{}	\N	2020-10-12 14:19:50.666273
28401	BB	2020-10-08	4.6	4.6399	4.54	4.6	4320972	{}	\N	2020-10-12 14:19:50.671211
28402	BB	2020-10-09	4.62	4.74	4.61	4.73	5196132	{}	\N	2020-10-12 14:19:50.676128
28403	BB	2020-10-12	4.75	4.83	4.75	4.8025	1231800	{}	\N	2020-10-12 14:19:50.681924
28404	BBW	2020-09-28	2.64	2.7818	2.55	2.57	138166	{}	\N	2020-10-12 14:19:52.078109
28405	BBW	2020-09-29	2.54	2.64	2.33	2.62	292326	{}	\N	2020-10-12 14:19:52.087676
28406	BBW	2020-09-30	2.55	2.62	2.3562	2.5	292508	{}	\N	2020-10-12 14:19:52.094518
28407	BBW	2020-10-01	2.47	2.79	2.4	2.76	305426	{}	\N	2020-10-12 14:19:52.100986
28408	BBW	2020-10-02	2.65	2.82	2.5151	2.75	262605	{}	\N	2020-10-12 14:19:52.106701
28409	BBW	2020-10-05	2.76	2.85	2.72	2.81	155937	{}	\N	2020-10-12 14:19:52.113117
28410	BBW	2020-10-06	2.82	2.87	2.7317	2.78	585613	{}	\N	2020-10-12 14:19:52.118283
28411	BBW	2020-10-07	2.85	3.04	2.78	3.02	195739	{}	\N	2020-10-12 14:19:52.123269
28412	BBW	2020-10-08	3.05	3.05	2.86	2.91	205704	{}	\N	2020-10-12 14:19:52.127941
28413	BBW	2020-10-09	2.91	3.11	2.75	2.98	342215	{}	\N	2020-10-12 14:19:52.13289
28414	BBW	2020-10-12	2.96	2.9814	2.8746	2.94	18230	{}	\N	2020-10-12 14:19:52.137404
28415	BCOV	2020-09-28	10.05	10.17	9.94	10.12	96362	{}	\N	2020-10-12 14:19:53.380376
28416	BCOV	2020-09-29	10.49	10.62	10.23	10.39	115688	{}	\N	2020-10-12 14:19:53.390174
28417	BCOV	2020-09-30	10.37	10.46	10.13	10.24	123296	{}	\N	2020-10-12 14:19:53.397384
28418	BCOV	2020-10-01	10.37	10.425	10.16	10.34	114289	{}	\N	2020-10-12 14:19:53.402945
28419	BCOV	2020-10-02	10.16	10.29	10	10.05	105511	{}	\N	2020-10-12 14:19:53.408999
28420	BCOV	2020-10-05	10.09	10.31	9.97	10.29	90449	{}	\N	2020-10-12 14:19:53.414786
28421	BCOV	2020-10-06	10.32	10.76	10.27	10.53	112102	{}	\N	2020-10-12 14:19:53.419613
28422	BCOV	2020-10-07	10.6	10.83	10.555	10.77	89349	{}	\N	2020-10-12 14:19:53.424018
28423	BCOV	2020-10-08	10.95	10.96	10.63	10.725	68554	{}	\N	2020-10-12 14:19:53.428328
28424	BCOV	2020-10-09	10.84	10.93	10.69	10.72	69803	{}	\N	2020-10-12 14:19:53.432975
28425	BCOV	2020-10-12	10.83	10.99	10.41	10.9	4890	{}	\N	2020-10-12 14:19:53.437097
28426	BDCZ	2020-09-28	14.25	14.5	14.25	14.4999	1772	{}	\N	2020-10-12 14:19:54.84931
28427	BDCZ	2020-09-29	14.35	14.35	14.1901	14.2299	7288	{}	\N	2020-10-12 14:19:54.856991
28428	BDCZ	2020-09-30	14.6	14.6	14.2165	14.2165	3201	{}	\N	2020-10-12 14:19:54.862515
28429	BDCZ	2020-10-01	14.44	14.5768	14.435	14.5768	883	{}	\N	2020-10-12 14:19:54.8678
28430	BDCZ	2020-10-02	13.95	14.62	13.95	14.22	8458	{}	\N	2020-10-12 14:19:54.873115
28431	BDCZ	2020-10-05	14.16	14.645	14.16	14.5709	3632	{}	\N	2020-10-12 14:19:54.877446
28432	BDCZ	2020-10-06	14.645	14.649	14.4693	14.49	8570	{}	\N	2020-10-12 14:19:54.881932
28433	BDCZ	2020-10-07	14.52	14.55	14.47	14.55	7018	{}	\N	2020-10-12 14:19:54.886263
28434	BDCZ	2020-10-08	14.6	14.6511	14.6	14.6511	553	{}	\N	2020-10-12 14:19:54.890977
28435	BDCZ	2020-10-09	14.28	14.28	14.2621	14.2621	326	{}	\N	2020-10-12 14:19:54.895427
28436	BDSI	2020-09-28	3.83	3.85	3.74	3.77	604350	{}	\N	2020-10-12 14:19:56.576386
28437	BDSI	2020-09-29	3.76	3.7924	3.68	3.765	591103	{}	\N	2020-10-12 14:19:56.584379
28438	BDSI	2020-09-30	3.71	3.85	3.7	3.73	1329140	{}	\N	2020-10-12 14:19:56.589899
28439	BDSI	2020-10-01	3.78	3.8175	3.69	3.81	736296	{}	\N	2020-10-12 14:19:56.595545
28440	BDSI	2020-10-02	3.74	3.8338	3.7	3.83	340238	{}	\N	2020-10-12 14:19:56.600263
28441	BDSI	2020-10-05	3.85	3.9047	3.825	3.84	381790	{}	\N	2020-10-12 14:19:56.605074
28442	BDSI	2020-10-06	3.87	3.8891	3.72	3.73	1054897	{}	\N	2020-10-12 14:19:56.609495
28443	BDSI	2020-10-07	3.76	3.85	3.7259	3.83	1790422	{}	\N	2020-10-12 14:19:56.613678
28444	BDSI	2020-10-08	3.89	3.9684	3.88	3.92	1131265	{}	\N	2020-10-12 14:19:56.617898
28445	BDSI	2020-10-09	3.94	4.16	3.94	4.16	657865	{}	\N	2020-10-12 14:19:56.622174
28446	BDSI	2020-10-12	4.14	4.19	4.02	4.05	119495	{}	\N	2020-10-12 14:19:56.626328
28447	BELFB	2020-09-28	10.49	10.8	10.43	10.7	49301	{}	\N	2020-10-12 14:19:57.662936
28448	BELFB	2020-09-29	10.96	10.97	10.4538	10.53	22353	{}	\N	2020-10-12 14:19:57.672504
28449	BELFB	2020-09-30	10.78	10.94	10.56	10.69	29429	{}	\N	2020-10-12 14:19:57.680094
28450	BELFB	2020-10-01	10.7501	10.885	10.55	10.64	33048	{}	\N	2020-10-12 14:19:57.687519
28451	BELFB	2020-10-02	10.4	10.84	10.4	10.7	28814	{}	\N	2020-10-12 14:19:57.693831
28452	BELFB	2020-10-05	10.93	11.28	10.71	11.01	37088	{}	\N	2020-10-12 14:19:57.698822
28453	BELFB	2020-10-06	11.21	11.49	10.27	10.27	38598	{}	\N	2020-10-12 14:19:57.7036
28454	BELFB	2020-10-07	10.54	11.0152	10.32	10.85	43495	{}	\N	2020-10-12 14:19:57.709243
28455	BELFB	2020-10-08	10.87	11.04	10.675	10.89	22817	{}	\N	2020-10-12 14:19:57.714133
28456	BELFB	2020-10-09	10.94	11.09	10.32	10.5	20710	{}	\N	2020-10-12 14:19:57.718733
28457	BELFB	2020-10-12	10.59	10.84	10.5	10.58	3282	{}	\N	2020-10-12 14:19:57.726359
28458	BFO	2020-09-28	14.74	14.74	14.709	14.709	3677	{}	\N	2020-10-12 14:19:59.500992
28459	BFO	2020-09-29	14.73	14.73	14.7116	14.7116	2938	{}	\N	2020-10-12 14:19:59.51014
28460	BFO	2020-09-30	14.74	14.74	14.7001	14.74	13662	{}	\N	2020-10-12 14:19:59.516409
28461	BFO	2020-10-01	14.7101	14.7101	14.7101	14.7101	2408	{}	\N	2020-10-12 14:19:59.522515
28462	BFO	2020-10-02	14.74	14.74	14.7	14.7	18900	{}	\N	2020-10-12 14:19:59.527795
28463	BFO	2020-10-05	14.7167	14.7207	14.7	14.71	34900	{}	\N	2020-10-12 14:19:59.532363
28464	BFO	2020-10-06	14.74	14.74	14.72	14.72	2006	{}	\N	2020-10-12 14:19:59.536937
28465	BFO	2020-10-07	14.74	14.74	14.72	14.72	15604	{}	\N	2020-10-12 14:19:59.54168
28466	BFO	2020-10-08	14.76	14.76	14.71	14.72	47788	{}	\N	2020-10-12 14:19:59.545833
28467	BFO	2020-10-09	14.73	14.73	14.71	14.71	16176	{}	\N	2020-10-12 14:19:59.550226
28468	AXS	2020-09-28	43.52	44.64	43.5	44.22	545138	{}	\N	2020-10-12 14:20:00.905873
28469	AXS	2020-09-29	43.95	44.1	43.38	43.88	544045	{}	\N	2020-10-12 14:20:00.916564
28470	AXS	2020-09-30	44.13	44.47	43.58	44.04	1019270	{}	\N	2020-10-12 14:20:00.922971
28471	AXS	2020-10-01	43.84	44.1938	43.32	43.82	908465	{}	\N	2020-10-12 14:20:00.928556
28472	AXS	2020-10-02	43.14	44.5	42.97	44.23	723048	{}	\N	2020-10-12 14:20:00.933094
28473	AXS	2020-10-05	44.87	45.82	44.475	45.66	894505	{}	\N	2020-10-12 14:20:00.937593
28474	AXS	2020-10-06	46.03	46.14	44.59	45.01	792604	{}	\N	2020-10-12 14:20:00.94188
28475	AXS	2020-10-07	45.29	45.66	43.92	44.53	794893	{}	\N	2020-10-12 14:20:00.945414
28476	AXS	2020-10-08	44.93	45.26	44.34	45	917099	{}	\N	2020-10-12 14:20:00.949105
28477	AXS	2020-10-09	45.37	45.83	44.97	45.27	846453	{}	\N	2020-10-12 14:20:00.95289
28478	AXS	2020-10-12	45.06	45.51	44.94	45.25	42003	{}	\N	2020-10-12 14:20:00.957304
28479	AZPN	2020-09-28	129.17	130.27	127.58	128.27	354016	{}	\N	2020-10-12 14:20:01.962208
28480	AZPN	2020-09-29	128.76	130.123	126.89	127.21	423990	{}	\N	2020-10-12 14:20:01.973341
28481	AZPN	2020-09-30	127	128.66	125.4	126.59	483030	{}	\N	2020-10-12 14:20:01.98182
28482	AZPN	2020-10-01	127.95	129.13	126.82	127.76	215106	{}	\N	2020-10-12 14:20:01.987479
28483	AZPN	2020-10-02	125.53	127.565	125.33	126.54	279361	{}	\N	2020-10-12 14:20:01.992392
28484	AZPN	2020-10-05	127.16	128.16	123.83	125.74	429941	{}	\N	2020-10-12 14:20:01.996516
28485	AZPN	2020-10-06	125.22	127.89	123.93	124.18	251423	{}	\N	2020-10-12 14:20:02.000399
28486	AZPN	2020-10-07	125.4	128.2175	125.21	127.75	295665	{}	\N	2020-10-12 14:20:02.004202
28487	AZPN	2020-10-08	128.25	129.37	127.54	128.9	199471	{}	\N	2020-10-12 14:20:02.009274
28488	AZPN	2020-10-09	130.31	133.16	129	132.4	226869	{}	\N	2020-10-12 14:20:02.013475
28489	AZPN	2020-10-12	133.99	134.41	131.83	132.78	21943	{}	\N	2020-10-12 14:20:02.017217
28490	BAC	2020-09-28	23.88	24.31	23.79	24.09	44346633	{}	\N	2020-10-12 14:20:02.998325
28491	BAC	2020-09-29	23.98	23.99	23.525	23.77	88132258	{}	\N	2020-10-12 14:20:03.009118
28492	BAC	2020-09-30	23.78	24.32	23.77	24.09	104819841	{}	\N	2020-10-12 14:20:03.015718
28493	BAC	2020-10-01	24.27	24.41	23.863	24.1	79440695	{}	\N	2020-10-12 14:20:03.022148
28494	BAC	2020-10-02	23.58	24.431	23.57	24.21	91348763	{}	\N	2020-10-12 14:20:03.02715
28495	BAC	2020-10-05	24.52	24.71	24.38	24.62	83898913	{}	\N	2020-10-12 14:20:03.031938
28496	BAC	2020-10-06	24.92	25.19	24.28	24.37	130741875	{}	\N	2020-10-12 14:20:03.035936
28497	BAC	2020-10-07	24.63	25.11	24.6	24.88	90410223	{}	\N	2020-10-12 14:20:03.041116
28498	BAC	2020-10-08	24.95	25.3	24.81	25.24	100829544	{}	\N	2020-10-12 14:20:03.045368
28499	BAC	2020-10-09	25.34	25.49	25.04	25.36	87007824	{}	\N	2020-10-12 14:20:03.049588
28500	BAC	2020-10-12	25.3	25.59	25.24	25.48	17512642	{}	\N	2020-10-12 14:20:03.053602
28501	BAP	2020-09-28	124.97	126.67	123.28	123.53	207223	{}	\N	2020-10-12 14:20:03.968061
28502	BAP	2020-09-29	123.15	123.7	121.64	123.25	655474	{}	\N	2020-10-12 14:20:03.979719
28503	BAP	2020-09-30	124.16	125.69	123.41	123.99	388336	{}	\N	2020-10-12 14:20:03.987551
28504	BAP	2020-10-01	123.84	125.94	122.16	123.46	547997	{}	\N	2020-10-12 14:20:03.993601
28505	BAP	2020-10-02	122.98	124.545	122.42	124.03	409559	{}	\N	2020-10-12 14:20:04.000389
28506	BAP	2020-10-05	125.23	125.25	123.64	124.15	425951	{}	\N	2020-10-12 14:20:04.006771
28507	BAP	2020-10-06	125	130.355	124.62	128.67	950966	{}	\N	2020-10-12 14:20:04.010988
28508	BAP	2020-10-07	129	129.815	125.13	125.43	704981	{}	\N	2020-10-12 14:20:04.015475
28509	BAP	2020-10-08	125.99	127.89	125.4	127.71	2092015	{}	\N	2020-10-12 14:20:04.020195
28510	BAP	2020-10-09	128.15	129.71	126.4	126.51	989900	{}	\N	2020-10-12 14:20:04.024649
28511	BAP	2020-10-12	126.05	126.285	123.725	123.92	60830	{}	\N	2020-10-12 14:20:04.029045
28512	BBF	2020-09-28	12.87	12.97	12.87	12.9326	8323	{}	\N	2020-10-12 14:20:05.797035
28513	BBF	2020-09-29	12.92	12.98	12.92	12.95	14168	{}	\N	2020-10-12 14:20:05.806824
28514	BBF	2020-09-30	12.93	12.98	12.93	12.96	8664	{}	\N	2020-10-12 14:20:05.812756
28515	BBF	2020-10-01	13	13.02	12.95	13	18044	{}	\N	2020-10-12 14:20:05.818283
28516	BBF	2020-10-02	13	13.01	12.93	12.97	20983	{}	\N	2020-10-12 14:20:05.823705
28517	BBF	2020-10-05	13.03	13.04	13	13.01	58540	{}	\N	2020-10-12 14:20:05.828227
28518	BBF	2020-10-06	12.99	13.05	12.99	12.99	38194	{}	\N	2020-10-12 14:20:05.833162
28519	BBF	2020-10-07	13	13.104	12.96	13.04	16414	{}	\N	2020-10-12 14:20:05.836899
28520	BBF	2020-10-08	13.06	13.06	12.995	13.01	13662	{}	\N	2020-10-12 14:20:05.840699
28521	BBF	2020-10-09	13.02	13.12	13.02	13.12	14220	{}	\N	2020-10-12 14:20:05.845324
28522	BBF	2020-10-12	13.23	13.23	13.08	13.17	9120	{}	\N	2020-10-12 14:20:05.849729
28523	BBSI	2020-09-28	50.07	51.23	50.07	50.62	49017	{}	\N	2020-10-12 14:20:07.46782
28524	BBSI	2020-09-29	50.48	52	49.02	49.81	51591	{}	\N	2020-10-12 14:20:07.47812
28525	BBSI	2020-09-30	50.81	53.76	50.8	52.44	90055	{}	\N	2020-10-12 14:20:07.489906
28526	BBSI	2020-10-01	52.41	53.88	51.36	52.83	87132	{}	\N	2020-10-12 14:20:07.500075
28527	BBSI	2020-10-02	51.67	54.9	51.06	54.4	74646	{}	\N	2020-10-12 14:20:07.505193
28528	BBSI	2020-10-05	54.79	55.22	53.55	55.01	51170	{}	\N	2020-10-12 14:20:07.509279
28529	BBSI	2020-10-06	55.57	56.1	54.41	55.46	54471	{}	\N	2020-10-12 14:20:07.513658
28530	BBSI	2020-10-07	55.61	56.0701	54.92	55.14	43704	{}	\N	2020-10-12 14:20:07.518171
28531	BBSI	2020-10-08	55.77	56.1999	55.14	55.68	40635	{}	\N	2020-10-12 14:20:07.522583
28532	BBSI	2020-10-09	55.03	56.47	55	55.55	38404	{}	\N	2020-10-12 14:20:07.527207
28533	BBSI	2020-10-12	55.48	56.11	55.19	55.9806	4406	{}	\N	2020-10-12 14:20:07.531633
28534	BCC	2020-09-28	39.35	40.4	39.225	40.11	218779	{}	\N	2020-10-12 14:20:08.516697
28535	BCC	2020-09-29	40.52	40.99	39.75	39.91	450058	{}	\N	2020-10-12 14:20:08.526673
28536	BCC	2020-09-30	39.94	40.4827	39.6	39.92	695028	{}	\N	2020-10-12 14:20:08.534476
28537	BCC	2020-10-01	40.09	40.77	39.65	40.58	326947	{}	\N	2020-10-12 14:20:08.541233
28538	BCC	2020-10-02	39.71	41.28	39.5	40.95	348571	{}	\N	2020-10-12 14:20:08.547261
28539	BCC	2020-10-05	41.72	42.08	41.17	41.87	413092	{}	\N	2020-10-12 14:20:08.552245
28540	BCC	2020-10-06	42.41	43.05	41.85	42.1	661152	{}	\N	2020-10-12 14:20:08.556941
28541	BCC	2020-10-07	42.84	43.98	42.7483	43.8	670458	{}	\N	2020-10-12 14:20:08.561454
28542	BCC	2020-10-08	44.23	44.49	42.8787	43.55	497554	{}	\N	2020-10-12 14:20:08.56612
28543	BCC	2020-10-09	43.85	44.23	43.16	43.61	414796	{}	\N	2020-10-12 14:20:08.570595
28544	BCC	2020-10-12	44	44.85	43.78	44.69	91719	{}	\N	2020-10-12 14:20:08.575074
28545	BCO	2020-09-28	41.05	41.5093	40.42	40.71	456192	{}	\N	2020-10-12 14:20:09.624431
28546	BCO	2020-09-29	40.7	40.7	39.12	39.8	719719	{}	\N	2020-10-12 14:20:09.631926
28547	BCO	2020-09-30	39.93	41.39	39.93	41.09	1022446	{}	\N	2020-10-12 14:20:09.637144
28548	BCO	2020-10-01	41.28	42.21	41.01	42.18	898516	{}	\N	2020-10-12 14:20:09.642597
28549	BCO	2020-10-02	40.95	45.66	40.95	45.32	902896	{}	\N	2020-10-12 14:20:09.647752
28550	BCO	2020-10-05	46	46.465	45.175	46.19	756711	{}	\N	2020-10-12 14:20:09.651935
28551	BCO	2020-10-06	46.92	48.23	46.36	47.26	882018	{}	\N	2020-10-12 14:20:09.656214
28552	BCO	2020-10-07	47.93	48.44	46.56	47.39	652135	{}	\N	2020-10-12 14:20:09.662748
28553	BCO	2020-10-08	48.11	48.59	47.42	48.55	799876	{}	\N	2020-10-12 14:20:09.66758
28554	BCO	2020-10-09	49	49.75	48.3	48.88	531166	{}	\N	2020-10-12 14:20:09.671647
28555	BCO	2020-10-12	48.9	48.99	48.31	48.62	29639	{}	\N	2020-10-12 14:20:09.675589
28556	BCX	2020-09-28	6.04	6.0699	6	6.01	238535	{}	\N	2020-10-12 14:20:10.806525
28557	BCX	2020-09-29	6.01	6.01	5.92	5.96	541960	{}	\N	2020-10-12 14:20:10.816403
28558	BCX	2020-09-30	5.96	6.05	5.95	5.95	876262	{}	\N	2020-10-12 14:20:10.822451
28559	BCX	2020-10-01	5.95	5.98	5.9	5.91	474177	{}	\N	2020-10-12 14:20:10.827754
28560	BCX	2020-10-02	5.85	5.96	5.85	5.96	390256	{}	\N	2020-10-12 14:20:10.832537
28561	BCX	2020-10-05	6	6.04	5.99	6.03	389111	{}	\N	2020-10-12 14:20:10.837072
28562	BCX	2020-10-06	6.07	6.11	5.96	5.97	528245	{}	\N	2020-10-12 14:20:10.841115
28563	BCX	2020-10-07	5.98	6.09	5.98	6.08	589495	{}	\N	2020-10-12 14:20:10.844929
28564	BCX	2020-10-08	6.1	6.17	6.06	6.16	366159	{}	\N	2020-10-12 14:20:10.849637
28565	BCX	2020-10-09	6.17	6.215	6.16	6.18	295483	{}	\N	2020-10-12 14:20:10.853596
28566	BCX	2020-10-12	6.18	6.18	6.15	6.16	106638	{}	\N	2020-10-12 14:20:10.857741
28567	BDR	2020-09-28	0.83	0.89	0.82	0.863441	168497	{}	\N	2020-10-12 14:20:11.987435
28568	BDR	2020-09-29	0.8771	0.8771	0.807	0.8203	108741	{}	\N	2020-10-12 14:20:11.993154
28569	BDR	2020-09-30	0.82	0.8599	0.82	0.8201	68569	{}	\N	2020-10-12 14:20:11.999555
28570	BDR	2020-10-01	0.83	0.85	0.812	0.8243	62323	{}	\N	2020-10-12 14:20:12.005384
28571	BDR	2020-10-02	0.8243	1.15	0.8	1.015	2173820	{}	\N	2020-10-12 14:20:12.011144
28572	BDR	2020-10-05	0.9825	1.049	0.9284	0.9429	371413	{}	\N	2020-10-12 14:20:12.016651
28573	BDR	2020-10-06	0.952	0.9799	0.89	0.9228	123567	{}	\N	2020-10-12 14:20:12.021603
28574	BDR	2020-10-07	0.9076	0.96	0.8923	0.9317	185547	{}	\N	2020-10-12 14:20:12.026206
28575	BDR	2020-10-08	0.93	1.08	0.9272	1.0654	544366	{}	\N	2020-10-12 14:20:12.031022
28576	BDR	2020-10-09	1.08	1.2	1	1.18	476299	{}	\N	2020-10-12 14:20:12.036171
28577	BDR	2020-10-12	1.17	1.21	1.15	1.185	104221	{}	\N	2020-10-12 14:20:12.042232
28578	BEP	2020-09-28	48.73	49.87	48.415	49.67	319304	{}	\N	2020-10-12 14:20:13.053616
28579	BEP	2020-09-29	49.92	50.82	49.65	50.78	444697	{}	\N	2020-10-12 14:20:13.061825
28580	BEP	2020-09-30	51.44	52.715	51.01	52.55	821468	{}	\N	2020-10-12 14:20:13.068545
28581	BEP	2020-10-01	52.7	54.06	52.63	53.83	1098088	{}	\N	2020-10-12 14:20:13.074412
28582	BEP	2020-10-02	52.7	53.93	52.5	53.62	672883	{}	\N	2020-10-12 14:20:13.07959
28583	BEP	2020-10-05	54.235	56.07	53.95	55.68	1216296	{}	\N	2020-10-12 14:20:13.083963
28584	BEP	2020-10-06	55.42	55.42	52.21	52.77	1673228	{}	\N	2020-10-12 14:20:13.08867
28585	BEP	2020-10-07	53.15	53.61	52.74	53.11	1189139	{}	\N	2020-10-12 14:20:13.092925
28586	BEP	2020-10-08	52.78	53.45	50.94	51.27	1304182	{}	\N	2020-10-12 14:20:13.097306
28587	BEP	2020-10-09	51.66	52.36	51.355	52.24	1063448	{}	\N	2020-10-12 14:20:13.102203
28588	BEP	2020-10-12	52.7	53.515	52.56	53.425	133199	{}	\N	2020-10-12 14:20:13.107015
28589	BFS	2020-09-28	26	27.16	25.9	26.84	83797	{}	\N	2020-10-12 14:20:14.076778
28590	BFS	2020-09-29	26.84	26.87	25.8	26.83	185306	{}	\N	2020-10-12 14:20:14.085492
28591	BFS	2020-09-30	27.09	27.583	26.37	26.58	263185	{}	\N	2020-10-12 14:20:14.091281
28592	BFS	2020-10-01	26.7	27.33	26.43	27.12	153617	{}	\N	2020-10-12 14:20:14.096634
28593	BFS	2020-10-02	26.58	27.77	26.3008	27.6	101865	{}	\N	2020-10-12 14:20:14.102166
28594	BFS	2020-10-05	27.9	28.06	27.39	27.68	129110	{}	\N	2020-10-12 14:20:14.10702
28595	BFS	2020-10-06	27.86	28.68	27.55	27.64	160940	{}	\N	2020-10-12 14:20:14.112044
28596	BFS	2020-10-07	27.86	28.07	27.04	27.3	184146	{}	\N	2020-10-12 14:20:14.116897
28597	BFS	2020-10-08	27.61	28.06	27.32	27.92	97816	{}	\N	2020-10-12 14:20:14.12211
28598	BFS	2020-10-09	28.12	28.12	27.49	27.73	77044	{}	\N	2020-10-12 14:20:14.12658
28599	BFS	2020-10-12	27.83	27.9633	27.41	27.82	20323	{}	\N	2020-10-12 14:20:14.131065
28600	AXTI	2020-09-28	4.94	5.08	4.94	4.97	270710	{}	\N	2020-10-12 14:20:16.032109
28601	AXTI	2020-09-29	4.93	6.36	4.93	5.94	9810570	{}	\N	2020-10-12 14:20:16.03911
28602	AXTI	2020-09-30	6.03	6.4199	5.79	6.12	2423418	{}	\N	2020-10-12 14:20:16.053
28603	AXTI	2020-10-01	6.34	6.44	6.13	6.42	918329	{}	\N	2020-10-12 14:20:16.063126
28604	AXTI	2020-10-02	6.24	6.54	6.16	6.21	634986	{}	\N	2020-10-12 14:20:16.071699
28605	AXTI	2020-10-05	6.39	6.7399	6.36	6.55	610210	{}	\N	2020-10-12 14:20:16.078931
28606	AXTI	2020-10-06	6.18	6.24	5.8777	5.97	1036351	{}	\N	2020-10-12 14:20:16.086802
28607	AXTI	2020-10-07	6.15	6.25	6.02	6.05	356463	{}	\N	2020-10-12 14:20:16.094773
28608	AXTI	2020-10-08	6.15	6.22	6.0783	6.19	284682	{}	\N	2020-10-12 14:20:16.10454
28609	AXTI	2020-10-09	6.38	6.44	6.19	6.31	404826	{}	\N	2020-10-12 14:20:16.111512
28610	AXTI	2020-10-12	6.41	6.435	6.32	6.33	101391	{}	\N	2020-10-12 14:20:16.116599
28611	AZZ	2020-09-28	33.97	34.69	33.74	34.33	99573	{}	\N	2020-10-12 14:20:17.088755
28612	AZZ	2020-09-29	34.28	34.38	33.53	34.06	258445	{}	\N	2020-10-12 14:20:17.099008
28613	AZZ	2020-09-30	34.25	35	34.11	34.12	323144	{}	\N	2020-10-12 14:20:17.107526
28614	AZZ	2020-10-01	34.06	34.43	33.58	34.11	234088	{}	\N	2020-10-12 14:20:17.113835
28615	AZZ	2020-10-02	33.54	35.35	33.54	35.06	218184	{}	\N	2020-10-12 14:20:17.119187
28616	AZZ	2020-10-05	35.36	35.86	34.44	35.74	220179	{}	\N	2020-10-12 14:20:17.124506
28617	AZZ	2020-10-06	36.15	36.74	35.75	35.77	350873	{}	\N	2020-10-12 14:20:17.129127
28618	AZZ	2020-10-07	36.09	36.39	35.45	35.88	351766	{}	\N	2020-10-12 14:20:17.133277
28619	AZZ	2020-10-08	36.3	36.76	35.74	36.62	260761	{}	\N	2020-10-12 14:20:17.138065
28620	AZZ	2020-10-09	36.72	37.33	36.39	36.54	224149	{}	\N	2020-10-12 14:20:17.142753
28621	AZZ	2020-10-12	36.55	36.95	36.34	36.895	19903	{}	\N	2020-10-12 14:20:17.147653
28622	BAK	2020-09-28	7.5	7.61	7.04	7.04	79093	{}	\N	2020-10-12 14:20:18.985432
28623	BAK	2020-09-29	7	7.27	7	7.1	42134	{}	\N	2020-10-12 14:20:18.990928
28624	BAK	2020-09-30	7.14	7.48	7.14	7.47	50786	{}	\N	2020-10-12 14:20:18.995205
28625	BAK	2020-10-01	7.41	7.585	7.32	7.46	44124	{}	\N	2020-10-12 14:20:18.999806
28626	BAK	2020-10-02	7.4	7.63	7.22	7.3	68340	{}	\N	2020-10-12 14:20:19.004601
28627	BAK	2020-10-05	7.39	7.66	7.355	7.66	33207	{}	\N	2020-10-12 14:20:19.009789
28628	BAK	2020-10-06	8.23	8.26	7.78	7.86	228886	{}	\N	2020-10-12 14:20:19.014216
28629	BAK	2020-10-07	8.03	8.09	7.73	8	53495	{}	\N	2020-10-12 14:20:19.018604
28630	BAK	2020-10-08	7.86	8.04	7.77	7.8	552656	{}	\N	2020-10-12 14:20:19.023925
28631	BAK	2020-10-09	7.79	7.86	7.64	7.7	70890	{}	\N	2020-10-12 14:20:19.028519
28632	BAK	2020-10-12	7.76	7.76	7.49	7.49	1460	{}	\N	2020-10-12 14:20:19.032858
28633	BBD	2020-09-28	3.66	3.7	3.44	3.48	30888901	{}	\N	2020-10-12 14:20:20.566069
28634	BBD	2020-09-29	3.42	3.4885	3.36	3.37	41951169	{}	\N	2020-10-12 14:20:20.574729
28635	BBD	2020-09-30	3.41	3.465	3.38	3.43	30123780	{}	\N	2020-10-12 14:20:20.580709
28636	BBD	2020-10-01	3.41	3.445	3.38	3.44	21140473	{}	\N	2020-10-12 14:20:20.586478
28637	BBD	2020-10-02	3.41	3.5984	3.41	3.44	47500811	{}	\N	2020-10-12 14:20:20.591157
28638	BBD	2020-10-05	3.44	3.58	3.41	3.57	38512639	{}	\N	2020-10-12 14:20:20.595715
28639	BBD	2020-10-06	3.59	3.69	3.49	3.54	40041124	{}	\N	2020-10-12 14:20:20.600263
28640	BBD	2020-10-07	3.56	3.5679	3.44	3.44	59907529	{}	\N	2020-10-12 14:20:20.604683
28641	BBD	2020-10-08	3.48	3.69	3.465	3.68	48439457	{}	\N	2020-10-12 14:20:20.609124
28642	BBD	2020-10-09	3.66	3.81	3.61	3.75	59288517	{}	\N	2020-10-12 14:20:20.613019
28643	BBD	2020-10-12	3.73	3.765	3.72	3.74	2286321	{}	\N	2020-10-12 14:20:20.617261
28644	BCLI	2020-09-28	15	16.5	14.96	15.95	716808	{}	\N	2020-10-12 14:20:22.793109
28645	BCLI	2020-09-29	15.94	16.26	15.715	16.19	436050	{}	\N	2020-10-12 14:20:22.80276
28646	BCLI	2020-09-30	16.21	17.08	16.21	16.92	1003066	{}	\N	2020-10-12 14:20:22.810502
28647	BCLI	2020-10-01	16.99	17.39	15.86	15.97	656587	{}	\N	2020-10-12 14:20:22.818176
28648	BCLI	2020-10-02	15.26	16.26	15.02	15.29	451307	{}	\N	2020-10-12 14:20:22.824069
28649	BCLI	2020-10-05	15.38	16.67	15.19	16.6	463454	{}	\N	2020-10-12 14:20:22.828968
28650	BCLI	2020-10-06	16.76	16.78	15.9648	16.37	228661	{}	\N	2020-10-12 14:20:22.832886
28651	BCLI	2020-10-07	16.39	17.17	16.08	17.01	418949	{}	\N	2020-10-12 14:20:22.836795
28652	BCLI	2020-10-08	17.08	17.85	16.9	17.7	773194	{}	\N	2020-10-12 14:20:22.840933
28653	BCLI	2020-10-09	17.7605	17.95	17.06	17.55	360425	{}	\N	2020-10-12 14:20:22.84495
28654	BCLI	2020-10-12	17.73	17.73	17.1	17.26	67832	{}	\N	2020-10-12 14:20:22.849162
28655	BCRX	2020-09-28	3.7	3.7001	3.54	3.67	3318568	{}	\N	2020-10-12 14:20:24.603334
28656	BCRX	2020-09-29	3.73	3.86	3.68	3.835	3984076	{}	\N	2020-10-12 14:20:24.614829
28657	BCRX	2020-09-30	3.79	3.8199	3.36	3.435	12415589	{}	\N	2020-10-12 14:20:24.621683
28658	BCRX	2020-10-01	3.46	3.53	3.443	3.53	3704581	{}	\N	2020-10-12 14:20:24.628619
28659	BCRX	2020-10-02	3.43	3.54	3.36	3.37	4499414	{}	\N	2020-10-12 14:20:24.633575
28660	BCRX	2020-10-05	3.4329	3.65	3.415	3.645	5844093	{}	\N	2020-10-12 14:20:24.638077
28661	BCRX	2020-10-06	3.64	3.685	3.44	3.48	3965008	{}	\N	2020-10-12 14:20:24.642204
28662	BCRX	2020-10-07	3.53	3.77	3.51	3.75	4161699	{}	\N	2020-10-12 14:20:24.646264
28663	BCRX	2020-10-08	3.778	3.915	3.73	3.895	4216083	{}	\N	2020-10-12 14:20:24.650335
28664	BCRX	2020-10-09	3.9	3.935	3.84	3.91	3378874	{}	\N	2020-10-12 14:20:24.655737
28665	BCRX	2020-10-12	3.95	3.95	3.85	3.9	871054	{}	\N	2020-10-12 14:20:24.659707
28666	BDJ	2020-09-28	7.3272	7.34	7.28	7.28	626214	{}	\N	2020-10-12 14:20:25.59049
28667	BDJ	2020-09-29	7.28	7.31	7.23	7.31	1036715	{}	\N	2020-10-12 14:20:25.595559
28668	BDJ	2020-09-30	7.37	7.3899	7.31	7.31	1679620	{}	\N	2020-10-12 14:20:25.603563
28669	BDJ	2020-10-01	7.33	7.37	7.27	7.28	1890575	{}	\N	2020-10-12 14:20:25.608198
28670	BDJ	2020-10-02	7.18	7.28	7.18	7.28	1058455	{}	\N	2020-10-12 14:20:25.612746
28671	BDJ	2020-10-05	7.29	7.36	7.29	7.36	1362639	{}	\N	2020-10-12 14:20:25.618188
28672	BDJ	2020-10-06	7.37	7.405	7.29	7.31	1804201	{}	\N	2020-10-12 14:20:25.62231
28673	BDJ	2020-10-07	7.38	7.43	7.35	7.41	1292604	{}	\N	2020-10-12 14:20:25.626231
28674	BDJ	2020-10-08	7.45	7.46	7.42	7.46	926466	{}	\N	2020-10-12 14:20:25.631295
28675	BDJ	2020-10-09	7.5	7.51	7.445	7.47	1078216	{}	\N	2020-10-12 14:20:25.635759
28676	BDJ	2020-10-12	7.49	7.505	7.48	7.5	166175	{}	\N	2020-10-12 14:20:25.639813
28677	BFIN	2020-09-28	7.4	7.42	7.33	7.33	13157	{}	\N	2020-10-12 14:20:28.134929
28678	BFIN	2020-09-29	7.38	7.46	7.21	7.3	11070	{}	\N	2020-10-12 14:20:28.14396
28679	BFIN	2020-09-30	7.345	7.37	7.2	7.22	9736	{}	\N	2020-10-12 14:20:28.14996
28680	BFIN	2020-10-01	7.14	7.31	7.14	7.21	38357	{}	\N	2020-10-12 14:20:28.155348
28681	BFIN	2020-10-02	7.16	7.36	7.16	7.28	28107	{}	\N	2020-10-12 14:20:28.160072
28682	BFIN	2020-10-05	7.3092	7.36	7.3	7.3	19599	{}	\N	2020-10-12 14:20:28.164916
28683	BFIN	2020-10-06	7.34	7.41	7.16	7.16	33343	{}	\N	2020-10-12 14:20:28.169043
28684	BFIN	2020-10-07	7.28	7.44	7.2	7.37	46702	{}	\N	2020-10-12 14:20:28.172845
28685	BFIN	2020-10-08	7.38	7.39	7.295	7.34	30006	{}	\N	2020-10-12 14:20:28.17643
28686	BFIN	2020-10-09	7.37	7.45	7.25	7.25	17801	{}	\N	2020-10-12 14:20:28.180298
28687	BFIN	2020-10-12	7.26	7.3	7.26	7.3	3414	{}	\N	2020-10-12 14:20:28.18433
28688	BGCP	2020-09-28	2.35	2.43	2.2985	2.4	3147584	{}	\N	2020-10-12 14:20:29.154815
28689	BGCP	2020-09-29	2.4	2.44	2.36	2.39	1472555	{}	\N	2020-10-12 14:20:29.164208
28690	BGCP	2020-09-30	2.41	2.47	2.36	2.4	2265193	{}	\N	2020-10-12 14:20:29.172198
28691	BGCP	2020-10-01	2.39	2.52	2.39	2.51	2210923	{}	\N	2020-10-12 14:20:29.178291
28692	BGCP	2020-10-02	2.46	2.61	2.4317	2.59	2042871	{}	\N	2020-10-12 14:20:29.182731
28693	BGCP	2020-10-05	2.58	2.73	2.58	2.71	2748342	{}	\N	2020-10-12 14:20:29.18762
28694	BGCP	2020-10-06	2.73	2.76	2.64	2.65	1570969	{}	\N	2020-10-12 14:20:29.191331
28695	BGCP	2020-10-07	2.68	2.735	2.64	2.67	1519475	{}	\N	2020-10-12 14:20:29.196099
28696	BGCP	2020-10-08	2.72	2.76	2.68	2.75	1550767	{}	\N	2020-10-12 14:20:29.20087
28697	BGCP	2020-10-09	2.77	2.78	2.67	2.68	1595074	{}	\N	2020-10-12 14:20:29.206524
28698	BGCP	2020-10-12	2.67	2.735	2.64	2.73	183524	{}	\N	2020-10-12 14:20:29.21201
28699	BGT	2020-09-28	11.31	11.31	11.21	11.27	61787	{}	\N	2020-10-12 14:20:31.200039
28700	BGT	2020-09-29	11.29	11.29	11.2	11.24	109376	{}	\N	2020-10-12 14:20:31.210711
28701	BGT	2020-09-30	11.31	11.32	11.23	11.23	216510	{}	\N	2020-10-12 14:20:31.218069
28702	BGT	2020-10-01	11.25	11.34	11.25	11.29	79610	{}	\N	2020-10-12 14:20:31.223985
28703	BGT	2020-10-02	11.28	11.3977	11.28	11.39	117874	{}	\N	2020-10-12 14:20:31.229719
28704	BGT	2020-10-05	11.45	11.54	11.39	11.53	127728	{}	\N	2020-10-12 14:20:31.234573
28705	BGT	2020-10-06	11.55	11.59	11.47	11.52	76513	{}	\N	2020-10-12 14:20:31.23898
28706	BGT	2020-10-07	11.59	11.62	11.54	11.55	284570	{}	\N	2020-10-12 14:20:31.243317
28707	BGT	2020-10-08	11.58	11.65	11	11	82642	{}	\N	2020-10-12 14:20:31.248143
28708	BGT	2020-10-09	11.51	11.63	11.5	11.51	192498	{}	\N	2020-10-12 14:20:31.252802
28709	BGT	2020-10-12	11.51	11.59	11.51	11.55	37578	{}	\N	2020-10-12 14:20:31.257262
28710	B	2020-09-28	36.11	37.31	35.92	35.98	223603	{}	\N	2020-10-12 14:20:33.232781
28711	B	2020-09-29	35.76	35.97	34.93	35.48	274429	{}	\N	2020-10-12 14:20:33.242781
28712	B	2020-09-30	35.74	36.79	35.41	35.74	429694	{}	\N	2020-10-12 14:20:33.249829
28713	B	2020-10-01	35.88	36.32	35.26	35.84	339822	{}	\N	2020-10-12 14:20:33.255299
28714	B	2020-10-02	34.98	36.75	34.98	36.45	263778	{}	\N	2020-10-12 14:20:33.260474
28715	B	2020-10-05	36.88	37.85	36.7	37.8	392353	{}	\N	2020-10-12 14:20:33.265879
28716	B	2020-10-06	38.47	39.47	37.85	38.11	607930	{}	\N	2020-10-12 14:20:33.270285
28717	B	2020-10-07	38.65	39.52	37.905	38.39	530225	{}	\N	2020-10-12 14:20:33.274532
28718	B	2020-10-08	38.99	39.36	38.51	39.04	219185	{}	\N	2020-10-12 14:20:33.278766
28719	B	2020-10-09	39.6	39.7	38.775	39.17	192272	{}	\N	2020-10-12 14:20:33.283663
28720	B	2020-10-12	39.19	39.37	38.98	39.26	15267	{}	\N	2020-10-12 14:20:33.288177
28721	BAM	2020-09-28	33.18	33.88	33.03	33.46	1834004	{}	\N	2020-10-12 14:20:34.253212
28722	BAM	2020-09-29	33.27	33.45	32.66	32.91	4745050	{}	\N	2020-10-12 14:20:34.266149
28723	BAM	2020-09-30	33.02	33.4	32.91	33.06	4132548	{}	\N	2020-10-12 14:20:34.27446
28724	BAM	2020-10-01	33.35	33.87	33.01	33.73	4630813	{}	\N	2020-10-12 14:20:34.281084
28725	BAM	2020-10-02	33.25	33.9	33.14	33.84	2547682	{}	\N	2020-10-12 14:20:34.285753
28726	BAM	2020-10-05	34.18	34.39	33.79	34.18	3537793	{}	\N	2020-10-12 14:20:34.290576
28727	BAM	2020-10-06	34.13	34.785	33.8199	33.88	4237449	{}	\N	2020-10-12 14:20:34.295323
28728	BAM	2020-10-07	34.32	35.19	34.23	35.07	4606394	{}	\N	2020-10-12 14:20:34.299703
28729	BAM	2020-10-08	35.44	35.83	35.17	35.54	4922734	{}	\N	2020-10-12 14:20:34.3041
28730	BAM	2020-10-09	35.75	35.81	35.135	35.18	3079826	{}	\N	2020-10-12 14:20:34.308232
28731	BAM	2020-10-12	35.13	35.25	35.05	35.25	236842	{}	\N	2020-10-12 14:20:34.312362
28732	BATRA	2020-09-28	20.43	21.0479	20.42	20.65	60593	{}	\N	2020-10-12 14:20:35.283304
28733	BATRA	2020-09-29	20.54	20.54	19.96	20.33	45663	{}	\N	2020-10-12 14:20:35.291943
28734	BATRA	2020-09-30	20.44	20.92	20.44	20.92	59310	{}	\N	2020-10-12 14:20:35.298417
28735	BATRA	2020-10-01	20.93	21.1	20.476	21.03	54114	{}	\N	2020-10-12 14:20:35.30409
28736	BATRA	2020-10-02	20.58	20.967	20.4	20.65	48594	{}	\N	2020-10-12 14:20:35.308773
28737	BATRA	2020-10-05	20.595	21.22	20.55	21.05	59140	{}	\N	2020-10-12 14:20:35.313148
28738	BATRA	2020-10-06	21.24	22.2	21.13	21.5	119297	{}	\N	2020-10-12 14:20:35.321399
28739	BATRA	2020-10-07	21.78	22.07	21.49	21.67	92520	{}	\N	2020-10-12 14:20:35.327299
28740	BATRA	2020-10-08	21.98	22.61	21.7697	22.48	79459	{}	\N	2020-10-12 14:20:35.333529
28741	BATRA	2020-10-09	22.62	22.7	22.32	22.58	85084	{}	\N	2020-10-12 14:20:35.338556
28742	BATRA	2020-10-12	22.55	22.85	22.55	22.8	7729	{}	\N	2020-10-12 14:20:35.342689
28743	BBN	2020-09-28	25.99	25.99	25.82	25.91	92227	{}	\N	2020-10-12 14:20:36.359209
28744	BBN	2020-09-29	25.86	26	25.86	25.98	173951	{}	\N	2020-10-12 14:20:36.368611
28745	BBN	2020-09-30	26.12	26.1271	25.9	25.92	258289	{}	\N	2020-10-12 14:20:36.374672
28746	BBN	2020-10-01	26	26.0655	25.9	25.9	145065	{}	\N	2020-10-12 14:20:36.380041
28747	BBN	2020-10-02	25.99	26.03	25.745	25.85	169684	{}	\N	2020-10-12 14:20:36.385106
28748	BBN	2020-10-05	25.92	25.92	25.65	25.69	252446	{}	\N	2020-10-12 14:20:36.389261
28749	BBN	2020-10-06	25.64	25.72	25.58	25.72	266766	{}	\N	2020-10-12 14:20:36.393812
28750	BBN	2020-10-07	25.77	25.92	25.72	25.85	253244	{}	\N	2020-10-12 14:20:36.397761
28751	BBN	2020-10-08	25.83	25.9	25.51	25.56	290130	{}	\N	2020-10-12 14:20:36.401474
28752	BBN	2020-10-09	25.66	25.7	25.56	25.63	208442	{}	\N	2020-10-12 14:20:36.404839
28753	BBN	2020-10-12	25.64	25.688	25.63	25.688	27116	{}	\N	2020-10-12 14:20:36.408327
28754	BBX	2020-09-28	13.99	14.48	13.93	14.36	16938	{}	\N	2020-10-12 14:20:37.246897
28755	BBX	2020-09-29	14.15	14.23	13.7	14.04	53776	{}	\N	2020-10-12 14:20:37.253286
28756	BBX	2020-09-30	13.95	14.12	13.05	13.39	51312	{}	\N	2020-10-12 14:20:37.258582
28757	BCE	2020-09-28	41.7	41.97	41.63	41.89	757120	{}	\N	2020-10-12 14:20:38.765941
28758	BCE	2020-09-29	41.8	41.92	41.385	41.65	1874942	{}	\N	2020-10-12 14:20:38.777496
28759	BCE	2020-09-30	41.69	41.79	41.38	41.47	1620052	{}	\N	2020-10-12 14:20:38.784676
28760	BCE	2020-10-01	41.41	41.66	41.27	41.55	1188144	{}	\N	2020-10-12 14:20:38.790477
28761	BCE	2020-10-02	41.19	41.7	41.11	41.63	1330790	{}	\N	2020-10-12 14:20:38.796427
28762	BCE	2020-10-05	41.9	41.99	41.5	41.84	1290086	{}	\N	2020-10-12 14:20:38.801916
28763	BCE	2020-10-06	41.85	41.89	41.37	41.51	1518717	{}	\N	2020-10-12 14:20:38.805965
28764	BCE	2020-10-07	41.63	41.89	41.5363	41.79	1114090	{}	\N	2020-10-12 14:20:38.810059
28765	BCE	2020-10-08	41.85	42.3	41.85	42.26	1220545	{}	\N	2020-10-12 14:20:38.814041
28766	BCE	2020-10-09	42.38	42.68	42.205	42.43	1541595	{}	\N	2020-10-12 14:20:38.818292
28767	BCE	2020-10-12	42.4858	42.79	42.4858	42.78	101840	{}	\N	2020-10-12 14:20:38.822497
28768	BCV	2020-09-28	24.5	24.895	24.49	24.74	29062	{}	\N	2020-10-12 14:20:39.962325
28769	BCV	2020-09-29	24.74	24.8074	24.54	24.74	15819	{}	\N	2020-10-12 14:20:39.972081
28770	BCV	2020-09-30	24.94	24.94	24.67	24.85	20918	{}	\N	2020-10-12 14:20:39.981537
28771	BCV	2020-10-01	24.8	25.22	24.66	25.165	27401	{}	\N	2020-10-12 14:20:39.988546
28772	BCV	2020-10-02	24.9	24.9728	24.78	24.86	14766	{}	\N	2020-10-12 14:20:39.993955
28773	BCV	2020-10-05	24.9	25.23	24.9	25.23	14872	{}	\N	2020-10-12 14:20:39.998603
28774	BCV	2020-10-06	24.88	25.551	24.88	25.13	25428	{}	\N	2020-10-12 14:20:40.003839
28775	BCV	2020-10-07	25.49	25.9027	25.44	25.82	16753	{}	\N	2020-10-12 14:20:40.008414
28776	BCV	2020-10-08	26.14	26.33	26.04	26.21	35454	{}	\N	2020-10-12 14:20:40.012742
28777	BCV	2020-10-09	26.07	26.36	26.0637	26.265	12797	{}	\N	2020-10-12 14:20:40.016955
28778	BCV	2020-10-12	26.75	26.75	26.4343	26.54	1022	{}	\N	2020-10-12 14:20:40.021256
28779	BDN	2020-09-28	10.19	10.53	10.1601	10.37	1244948	{}	\N	2020-10-12 14:20:41.851718
28780	BDN	2020-09-29	10.32	10.4	10.125	10.26	3453454	{}	\N	2020-10-12 14:20:41.863154
28781	BDN	2020-09-30	10.28	10.59	10.205	10.34	4517888	{}	\N	2020-10-12 14:20:41.870105
28782	BDN	2020-10-01	10.37	10.53	10.3	10.52	3208232	{}	\N	2020-10-12 14:20:41.87565
28783	BDN	2020-10-02	10.28	10.9	10.21	10.82	3105843	{}	\N	2020-10-12 14:20:41.880065
28784	BDN	2020-10-05	10.92	11.02	10.73	10.93	3547775	{}	\N	2020-10-12 14:20:41.884265
28785	BDN	2020-10-06	10.8	10.91	10.61	10.7	4182729	{}	\N	2020-10-12 14:20:41.888369
28786	BDN	2020-10-07	10.76	10.88	10.545	10.66	3116251	{}	\N	2020-10-12 14:20:41.892296
28787	BDN	2020-10-08	10.73	10.935	10.65	10.92	2561030	{}	\N	2020-10-12 14:20:41.896084
28788	BDN	2020-10-09	10.98	11.01	10.63	10.67	3808980	{}	\N	2020-10-12 14:20:41.899854
28789	BDN	2020-10-12	10.67	10.695	10.52	10.63	541503	{}	\N	2020-10-12 14:20:41.903611
28790	BELFA	2020-09-28	10.72	10.8245	10.7	10.75	6635	{}	\N	2020-10-12 14:20:42.795586
28791	BELFA	2020-09-29	11.76	11.76	11.227	11.227	867	{}	\N	2020-10-12 14:20:42.805416
28792	BELFA	2020-09-30	11.68	11.73	11.68	11.71	894	{}	\N	2020-10-12 14:20:42.812112
28793	BELFA	2020-10-01	11.76	11.76	11.76	11.76	894	{}	\N	2020-10-12 14:20:42.818156
28794	BELFA	2020-10-02	11.38	11.38	10.88	10.88	1335	{}	\N	2020-10-12 14:20:42.823774
28795	BELFA	2020-10-05	11.29	11.35	11.29	11.35	364	{}	\N	2020-10-12 14:20:42.828257
28796	BELFA	2020-10-06	11.71	11.71	10.1	10.1	2230	{}	\N	2020-10-12 14:20:42.832065
28797	BELFA	2020-10-07	10.43	10.47	10.22	10.22	434	{}	\N	2020-10-12 14:20:42.836118
28798	BELFA	2020-10-08	10.93	10.93	10.22	10.612	734	{}	\N	2020-10-12 14:20:42.840287
28799	BELFA	2020-10-09	10.6	10.75	10.6	10.75	1257	{}	\N	2020-10-12 14:20:42.844496
28800	BELFA	2020-10-12	10.82	10.82	10.8	10.8	1336	{}	\N	2020-10-12 14:20:42.848425
28801	BFY	2020-09-28	13.6525	13.659	13.62	13.62	4355	{}	\N	2020-10-12 14:20:43.789395
28802	BFY	2020-09-29	13.67	13.67	13.54	13.55	26226	{}	\N	2020-10-12 14:20:43.800096
28803	BFY	2020-09-30	13.58	13.6099	13.58	13.6	6732	{}	\N	2020-10-12 14:20:43.806964
28804	BFY	2020-10-01	13.66	13.7	13.62	13.68	12304	{}	\N	2020-10-12 14:20:43.812296
28805	BFY	2020-10-02	13.61	13.81	13.61	13.81	18210	{}	\N	2020-10-12 14:20:43.817158
28806	BFY	2020-10-05	13.74	13.82	13.715	13.82	31260	{}	\N	2020-10-12 14:20:43.821996
28807	BFY	2020-10-06	13.84	13.84	13.8	13.83	5798	{}	\N	2020-10-12 14:20:43.825923
28808	BFY	2020-10-07	13.78	13.85	13.7737	13.85	25484	{}	\N	2020-10-12 14:20:43.829892
28809	BFY	2020-10-08	13.85	13.87	13.8118	13.87	16880	{}	\N	2020-10-12 14:20:43.833901
28810	BFY	2020-10-09	13.821	13.87	13.81	13.87	5040	{}	\N	2020-10-12 14:20:43.838099
28811	BFY	2020-10-12	13.87	13.87	13.87	13.87	300	{}	\N	2020-10-12 14:20:43.842204
28812	BGIO	2020-09-28	8.6464	8.66	8.545	8.58	17829	{}	\N	2020-10-12 14:20:44.756494
28813	BGIO	2020-09-29	8.5883	8.72	8.5883	8.71	24392	{}	\N	2020-10-12 14:20:44.764148
28814	BGIO	2020-09-30	8.79	8.79	8.6	8.65	82900	{}	\N	2020-10-12 14:20:44.769894
28815	BGIO	2020-10-01	8.61	8.74	8.5986	8.72	35419	{}	\N	2020-10-12 14:20:44.774968
28816	BGIO	2020-10-02	8.7216	8.84	8.57	8.76	93289	{}	\N	2020-10-12 14:20:44.780034
28817	BGIO	2020-10-05	8.81	8.81	8.69	8.81	23214	{}	\N	2020-10-12 14:20:44.784411
28818	BGIO	2020-10-06	8.71	8.79	8.67	8.79	34456	{}	\N	2020-10-12 14:20:44.788401
28819	BGIO	2020-10-07	8.71	8.72	8.61	8.67	112814	{}	\N	2020-10-12 14:20:44.793177
28820	BGIO	2020-10-08	8.6568	8.6699	8.56	8.61	72182	{}	\N	2020-10-12 14:20:44.797551
28821	BGIO	2020-10-09	8.58	8.6054	8.46	8.56	100284	{}	\N	2020-10-12 14:20:44.802459
28822	BGIO	2020-10-12	8.6	8.61	8.5926	8.605	10724	{}	\N	2020-10-12 14:20:44.806482
28823	BHK	2020-09-28	15.51	15.54	15.45	15.52	122591	{}	\N	2020-10-12 14:20:48.323312
28824	BHK	2020-09-29	15.55	15.61	15.49	15.57	244006	{}	\N	2020-10-12 14:20:48.331664
28825	BHK	2020-09-30	15.67	15.68	15.6	15.66	740745	{}	\N	2020-10-12 14:20:48.340749
28826	BHK	2020-10-01	15.7	15.76	15.6	15.65	516745	{}	\N	2020-10-12 14:20:48.34775
28827	BHK	2020-10-02	15.64	15.75	15.635	15.67	245679	{}	\N	2020-10-12 14:20:48.353839
28828	BHK	2020-10-05	15.64	15.75	15.61	15.75	267640	{}	\N	2020-10-12 14:20:48.359301
28829	BHK	2020-10-06	15.77	15.8	15.62	15.75	298341	{}	\N	2020-10-12 14:20:48.364021
28830	BHK	2020-10-07	15.75	15.75	15.66	15.73	297338	{}	\N	2020-10-12 14:20:48.368279
28831	BHK	2020-10-08	15.75	15.87	15.7122	15.79	294910	{}	\N	2020-10-12 14:20:48.372741
28832	BHK	2020-10-09	15.8	15.8	15.68	15.68	172048	{}	\N	2020-10-12 14:20:48.377309
28833	BHK	2020-10-12	15.56	15.665	15.52	15.57	121760	{}	\N	2020-10-12 14:20:48.382815
28834	BIIB	2020-09-28	282	287.95	279.18	282.035	1703605	{}	\N	2020-10-12 14:20:49.402753
28835	BIIB	2020-09-29	280.84	285.45	280.23	282.35	805601	{}	\N	2020-10-12 14:20:49.411517
28836	BIIB	2020-09-30	283.87	284.85	280.79	283.68	1078855	{}	\N	2020-10-12 14:20:49.417939
28837	BIIB	2020-10-01	284.96	286.17	280.59	285.57	908825	{}	\N	2020-10-12 14:20:49.423839
28838	BIIB	2020-10-02	281.65	285.47	276.0601	277.64	883791	{}	\N	2020-10-12 14:20:49.428873
28839	BIIB	2020-10-05	278.67	285.92	277.355	284.18	880285	{}	\N	2020-10-12 14:20:49.433317
28840	BIIB	2020-10-06	284.17	285.8999	279.39	281.15	700938	{}	\N	2020-10-12 14:20:49.437843
28841	BIIB	2020-10-07	282.24	287.0776	280.905	285.75	622937	{}	\N	2020-10-12 14:20:49.442263
28842	BIIB	2020-10-08	287.4	288.44	285.35	286.92	486444	{}	\N	2020-10-12 14:20:49.446174
28843	BIIB	2020-10-09	287.16	289	285.17	286.54	902230	{}	\N	2020-10-12 14:20:49.450298
28844	BIIB	2020-10-12	289.28	289.29	285.78	288.64	193941	{}	\N	2020-10-12 14:20:49.454335
28845	BIV	2020-09-28	93.43	93.525	93.425	93.51	1280578	{}	\N	2020-10-12 14:20:50.774406
28846	BIV	2020-09-29	93.62	93.65	93.5632	93.6	678198	{}	\N	2020-10-12 14:20:50.78541
28847	BIV	2020-09-30	93.58	93.581	93.38	93.48	2064706	{}	\N	2020-10-12 14:20:50.792954
28848	BIV	2020-10-01	93.24	93.42	93.19	93.38	1267613	{}	\N	2020-10-12 14:20:50.799302
28849	BIV	2020-10-02	93.47	93.4999	93.28	93.34	1023382	{}	\N	2020-10-12 14:20:50.805843
28850	BIV	2020-10-05	93.27	93.27	93.02	93.02	1073461	{}	\N	2020-10-12 14:20:50.810785
28851	BIV	2020-10-06	93.15	93.26	93.03	93.09	1049392	{}	\N	2020-10-12 14:20:50.815513
28852	BIV	2020-10-07	93	93.1	92.92	93.04	1964325	{}	\N	2020-10-12 14:20:50.82036
28853	BIV	2020-10-08	93.14	93.19	93.07	93.15	738820	{}	\N	2020-10-12 14:20:50.824879
28854	BIV	2020-10-09	93.17	93.23	93.07	93.22	1565762	{}	\N	2020-10-12 14:20:50.829512
28855	BIV	2020-10-12	93.3	93.34	93.22	93.235	105949	{}	\N	2020-10-12 14:20:50.834107
28856	AYTU	2020-09-28	1.18	1.24	1.16	1.19	2345182	{}	\N	2020-10-12 14:20:52.824907
28857	AYTU	2020-09-29	1.15	1.2	1.12	1.2	2218855	{}	\N	2020-10-12 14:20:52.835589
28858	AYTU	2020-09-30	1.14	1.1963	1.14	1.19	1776393	{}	\N	2020-10-12 14:20:52.844158
28859	AYTU	2020-10-01	1.22	1.25	1.17	1.25	3129985	{}	\N	2020-10-12 14:20:52.850932
28860	AYTU	2020-10-02	1.2	1.24	1.18	1.21	1403232	{}	\N	2020-10-12 14:20:52.857136
28861	AYTU	2020-10-05	1.23	1.35	1.22	1.35	4800894	{}	\N	2020-10-12 14:20:52.862755
28862	AYTU	2020-10-06	1.4	1.4	1.29	1.33	12004692	{}	\N	2020-10-12 14:20:52.868389
28863	AYTU	2020-10-07	1.2	1.24	1.14	1.16	9779616	{}	\N	2020-10-12 14:20:52.873529
28864	AYTU	2020-10-08	1.1564	1.18	1.15	1.18	2372043	{}	\N	2020-10-12 14:20:52.878479
28865	AYTU	2020-10-09	1.17	1.17	1.12	1.13	4097612	{}	\N	2020-10-12 14:20:52.883305
28866	AYTU	2020-10-12	1.13	1.14	1.08	1.101	1499485	{}	\N	2020-10-12 14:20:52.888067
28867	BA	2020-09-28	160.375	168	159.2	166.08	38295216	{}	\N	2020-10-12 14:20:54.114391
28868	BA	2020-09-29	164.77	165.5	162.01	163.6	39506255	{}	\N	2020-10-12 14:20:54.125225
28869	BA	2020-09-30	164.5	170.98	164.11	165.26	75782846	{}	\N	2020-10-12 14:20:54.131331
28870	BA	2020-10-01	170.15	172.76	165.57	167.86	59167531	{}	\N	2020-10-12 14:20:54.137099
28871	BA	2020-10-02	161.85	169.78	161.8	168.08	61796959	{}	\N	2020-10-12 14:20:54.14234
28872	BA	2020-10-05	169.49	172.08	167.51	171.2	37728965	{}	\N	2020-10-12 14:20:54.147414
28873	BA	2020-10-06	173.04	175.56	159.33	159.54	93153133	{}	\N	2020-10-12 14:20:54.151863
28874	BA	2020-10-07	163.71	166.3447	162.38	164.61	52606247	{}	\N	2020-10-12 14:20:54.15658
28875	BA	2020-10-08	166.22	168.09	163.5401	168	40959951	{}	\N	2020-10-12 14:20:54.161474
28876	BA	2020-10-09	169.93	170	166.82	167.33	34818996	{}	\N	2020-10-12 14:20:54.166073
28877	BA	2020-10-12	167.75	168.68	165.95	167.97	7174144	{}	\N	2020-10-12 14:20:54.170562
28878	BANF	2020-09-28	40.94	42	40.94	41.55	69356	{}	\N	2020-10-12 14:20:55.153398
28879	BANF	2020-09-29	41.05	41.0875	39.88	40.82	42484	{}	\N	2020-10-12 14:20:55.161024
28880	BANF	2020-09-30	41.03	41.58	40.57	40.84	76630	{}	\N	2020-10-12 14:20:55.166476
28881	BANF	2020-10-01	40.74	41.23	40.07	40.94	56194	{}	\N	2020-10-12 14:20:55.17105
28882	BANF	2020-10-02	40.53	42.45	40.53	42.27	95483	{}	\N	2020-10-12 14:20:55.175803
28883	BANF	2020-10-05	42.77	43.7	42.77	43.51	73953	{}	\N	2020-10-12 14:20:55.180776
28884	BANF	2020-10-06	44.15	45.31	43.46	43.59	70591	{}	\N	2020-10-12 14:20:55.185214
28885	BANF	2020-10-07	44.16	44.68	43.63	44.42	67303	{}	\N	2020-10-12 14:20:55.189064
28886	BANF	2020-10-08	44.81	45.28	44.26	45.07	31597	{}	\N	2020-10-12 14:20:55.193324
28887	BANF	2020-10-09	45.28	45.28	44.42	44.55	39118	{}	\N	2020-10-12 14:20:55.197517
28888	BANF	2020-10-12	44.46	44.46	43.88	44.21	8337	{}	\N	2020-10-12 14:20:55.202341
28889	BAX	2020-09-28	80.31	81.1	79.395	79.54	1710931	{}	\N	2020-10-12 14:20:56.159087
28890	BAX	2020-09-29	79.39	80.41	79.02	79.09	7064979	{}	\N	2020-10-12 14:20:56.169648
28891	BAX	2020-09-30	79.49	81.2	79.03	80.42	5912320	{}	\N	2020-10-12 14:20:56.175825
28892	BAX	2020-10-01	82.02	82.3854	79.44	79.72	3956778	{}	\N	2020-10-12 14:20:56.181806
28893	BAX	2020-10-02	79.34	79.93	78.64	78.83	2833730	{}	\N	2020-10-12 14:20:56.186626
28894	BAX	2020-10-05	79.17	80.685	79.07	80.57	4672212	{}	\N	2020-10-12 14:20:56.191401
28895	BAX	2020-10-06	80.48	81.97	80.275	80.46	5035625	{}	\N	2020-10-12 14:20:56.195712
28896	BAX	2020-10-07	80.91	82.02	80.27	81.65	2904486	{}	\N	2020-10-12 14:20:56.199367
28897	BAX	2020-10-08	81	81.3	78.56	78.85	8568834	{}	\N	2020-10-12 14:20:56.203285
28898	BAX	2020-10-09	78.85	79.04	77.73	78.5	8039442	{}	\N	2020-10-12 14:20:56.207274
28899	BAX	2020-10-12	78.83	79.59	78.54	79.4	1645737	{}	\N	2020-10-12 14:20:56.211389
28900	BBK	2020-09-28	15.18	15.25	15.145	15.25	41849	{}	\N	2020-10-12 14:20:59.044413
28901	BBK	2020-09-29	15.22	15.25	15.1801	15.24	17224	{}	\N	2020-10-12 14:20:59.051716
28902	BBK	2020-09-30	15.19	15.23	15.19	15.2	27994	{}	\N	2020-10-12 14:20:59.057084
28903	BBK	2020-10-01	15.3	15.3	15.21	15.27	29448	{}	\N	2020-10-12 14:20:59.062349
28904	BBK	2020-10-02	15.28	15.42	15.28	15.42	44202	{}	\N	2020-10-12 14:20:59.066795
28905	BBK	2020-10-05	15.42	15.48	15.4	15.4	71795	{}	\N	2020-10-12 14:20:59.070904
28906	BBK	2020-10-06	15.4	15.435	15.2	15.4	31645	{}	\N	2020-10-12 14:20:59.074721
28907	BBK	2020-10-07	15.49	15.49	15.34	15.49	19662	{}	\N	2020-10-12 14:20:59.078614
28908	BBK	2020-10-08	15.41	15.5	15.41	15.5	37750	{}	\N	2020-10-12 14:20:59.082417
28909	BBK	2020-10-09	15.45	15.5	15.45	15.5	12138	{}	\N	2020-10-12 14:20:59.086174
28910	BBY	2020-09-28	107.25	108.57	106.65	108.44	1458819	{}	\N	2020-10-12 14:21:00.910483
28911	BBY	2020-09-29	109.94	111.22	108.84	110.08	4710452	{}	\N	2020-10-12 14:21:00.924808
28912	BBY	2020-09-30	109.5	112.41	109.5	111.29	4218752	{}	\N	2020-10-12 14:21:00.930824
28913	BBY	2020-10-01	112.69	114.72	111.86	114.09	6891063	{}	\N	2020-10-12 14:21:00.936685
28914	BBY	2020-10-02	111.04	113.46	111.01	112.42	3500343	{}	\N	2020-10-12 14:21:00.942459
28915	BBY	2020-10-05	112.57	116.74	112.57	116.07	3915002	{}	\N	2020-10-12 14:21:00.947617
28916	BBY	2020-10-06	115.98	116.43	112.25	112.45	4784898	{}	\N	2020-10-12 14:21:00.951954
28917	BBY	2020-10-07	114	115.44	113.76	114.49	3014586	{}	\N	2020-10-12 14:21:00.956845
28918	BBY	2020-10-08	115.13	115.65	113.91	114.92	3000670	{}	\N	2020-10-12 14:21:00.961421
28919	BBY	2020-10-09	115.37	118.25	115.11	116.59	5638427	{}	\N	2020-10-12 14:21:00.966005
28920	BBY	2020-10-12	117.87	118.7	116.86	117.51	332596	{}	\N	2020-10-12 14:21:00.970562
28921	BCBP	2020-09-28	7.94	8.28	7.93	8.22	31178	{}	\N	2020-10-12 14:21:02.52304
28922	BCBP	2020-09-29	8.23	8.24	7.88	8.02	25021	{}	\N	2020-10-12 14:21:02.532289
28923	BCBP	2020-09-30	8.1	8.15	7.88	8	17016	{}	\N	2020-10-12 14:21:02.539862
28924	BCBP	2020-10-01	8	8.01	7.815	7.94	26692	{}	\N	2020-10-12 14:21:02.54595
28925	BCBP	2020-10-02	7.85	8.13	7.8301	8.05	15908	{}	\N	2020-10-12 14:21:02.550804
28926	BCBP	2020-10-05	8.17	8.6	8.102	8.55	38152	{}	\N	2020-10-12 14:21:02.555671
28927	BCBP	2020-10-06	8.6	8.78	8.16	8.19	44103	{}	\N	2020-10-12 14:21:02.560501
28928	BCBP	2020-10-07	8.32	8.46	8.23	8.44	29290	{}	\N	2020-10-12 14:21:02.565558
28929	BCBP	2020-10-08	8.54	8.54	8.31	8.46	14144	{}	\N	2020-10-12 14:21:02.569726
28930	BCBP	2020-10-09	8.56	8.56	8.21	8.37	14605	{}	\N	2020-10-12 14:21:02.574463
28931	BCBP	2020-10-12	8.36	8.36	8.23	8.23	721	{}	\N	2020-10-12 14:21:02.578937
28932	BCOR	2020-09-28	8.97	9.27	8.97	9.08	328552	{}	\N	2020-10-12 14:21:03.576659
28933	BCOR	2020-09-29	9.2435	9.4	9.02	9.33	442029	{}	\N	2020-10-12 14:21:03.586713
28934	BCOR	2020-09-30	9.4	9.71	9.321	9.42	456138	{}	\N	2020-10-12 14:21:03.593109
28935	BCOR	2020-10-01	9.42	9.51	9.17	9.49	247768	{}	\N	2020-10-12 14:21:03.598741
28936	BCOR	2020-10-02	9.15	9.76	9.15	9.69	337064	{}	\N	2020-10-12 14:21:03.603936
28937	BCOR	2020-10-05	9.7	10.09	9.7	10.07	354566	{}	\N	2020-10-12 14:21:03.608646
28938	BCOR	2020-10-06	10.19	10.39	9.91	10.135	636378	{}	\N	2020-10-12 14:21:03.612855
28939	BCOR	2020-10-07	10.12	10.46	10.12	10.41	281053	{}	\N	2020-10-12 14:21:03.617476
28940	BCOR	2020-10-08	10.55	11	10.55	10.91	271646	{}	\N	2020-10-12 14:21:03.621959
28941	BCOR	2020-10-09	11.03	11.15	10.77	10.83	199253	{}	\N	2020-10-12 14:21:03.626168
28942	BCOR	2020-10-12	10.87	10.87	10.62	10.69	18568	{}	\N	2020-10-12 14:21:03.630469
28943	BDC	2020-09-28	30.98	32.25	30.98	31.3	409043	{}	\N	2020-10-12 14:21:04.77708
28944	BDC	2020-09-29	31.32	31.55	30.54	31.17	511598	{}	\N	2020-10-12 14:21:04.78687
28945	BDC	2020-09-30	31.47	32.2	30.92	31.12	710575	{}	\N	2020-10-12 14:21:04.792721
28946	BDC	2020-10-01	31.36	31.99	30.73	31.24	1094987	{}	\N	2020-10-12 14:21:04.798154
28947	BDC	2020-10-02	30.5	32.06	30.4001	31.41	775462	{}	\N	2020-10-12 14:21:04.80335
28948	BDC	2020-10-05	31.88	32.46	31.78	32.26	1001690	{}	\N	2020-10-12 14:21:04.809049
28949	BDC	2020-10-06	32.65	33.96	32.63	32.89	863520	{}	\N	2020-10-12 14:21:04.813619
28950	BDC	2020-10-07	33.25	34.1	33.25	33.61	647729	{}	\N	2020-10-12 14:21:04.818156
28951	BDC	2020-10-08	34.09	34.16	33.625	34.12	323345	{}	\N	2020-10-12 14:21:04.82261
28952	BDC	2020-10-09	34.64	34.78	33.91	34.06	440437	{}	\N	2020-10-12 14:21:04.826891
28953	BDC	2020-10-12	34.17	34.33	33.98	34.09	21180	{}	\N	2020-10-12 14:21:04.831287
28954	BECN	2020-09-28	30.01	30.94	30.01	30.74	503833	{}	\N	2020-10-12 14:21:05.881112
28955	BECN	2020-09-29	30.82	31.35	30.49	30.9	351757	{}	\N	2020-10-12 14:21:05.889111
28956	BECN	2020-09-30	31.03	31.82	30.75	31.07	489019	{}	\N	2020-10-12 14:21:05.895853
28957	BECN	2020-10-01	31.48	31.84	30.67	31.59	361957	{}	\N	2020-10-12 14:21:05.901946
28958	BECN	2020-10-02	30.81	32.34	30.21	32.16	315403	{}	\N	2020-10-12 14:21:05.906976
28959	BECN	2020-10-05	32.62	32.9	32.32	32.66	444844	{}	\N	2020-10-12 14:21:05.91215
28960	BECN	2020-10-06	34.49	35.92	34.05	34.4	1112767	{}	\N	2020-10-12 14:21:05.916264
28961	BECN	2020-10-07	34.7	36.07	34.7	35.51	671064	{}	\N	2020-10-12 14:21:05.920664
28962	BECN	2020-10-08	35.81	36.21	35.14	35.79	392407	{}	\N	2020-10-12 14:21:05.924746
28963	BECN	2020-10-09	36.1	36.4	35.22	35.85	384488	{}	\N	2020-10-12 14:21:05.928974
28964	BECN	2020-10-12	36.83	36.88	36.1	36.325	114377	{}	\N	2020-10-12 14:21:05.932821
28965	BERY	2020-09-28	47.8	48.72	47.6233	48.32	1179202	{}	\N	2020-10-12 14:21:08.389072
28966	BERY	2020-09-29	48.35	48.615	47.595	47.67	1246007	{}	\N	2020-10-12 14:21:08.399357
28967	BERY	2020-09-30	47.9	49.065	47.855	48.32	2178423	{}	\N	2020-10-12 14:21:08.406683
28968	BERY	2020-10-01	48.62	49.57	48.43	49.35	2348651	{}	\N	2020-10-12 14:21:08.412735
28969	BERY	2020-10-02	48.39	50.47	48.39	49.97	1227817	{}	\N	2020-10-12 14:21:08.419707
28970	BERY	2020-10-05	50.31	51.07	49.88	50.19	1944448	{}	\N	2020-10-12 14:21:08.423899
28971	BERY	2020-10-06	50.91	51.67	50.03	50.13	1568229	{}	\N	2020-10-12 14:21:08.428071
28972	BERY	2020-10-07	50.56	51.25	49.57	49.68	1646299	{}	\N	2020-10-12 14:21:08.432252
28973	BERY	2020-10-08	50.2	50.51	49.41	50.08	983102	{}	\N	2020-10-12 14:21:08.436385
28974	BERY	2020-10-09	50.38	50.68	49.18	49.39	1958981	{}	\N	2020-10-12 14:21:08.440395
28975	BERY	2020-10-12	49.68	49.93	49.39	49.675	70037	{}	\N	2020-10-12 14:21:08.444484
28976	BG	2020-09-28	46.97	47.5637	46.49	46.505	849670	{}	\N	2020-10-12 14:21:09.403659
28977	BG	2020-09-29	46.28	46.35	44.965	45.18	1230015	{}	\N	2020-10-12 14:21:09.411586
28978	BG	2020-09-30	45.22	46.1	45.22	45.7	1719892	{}	\N	2020-10-12 14:21:09.417586
28979	BG	2020-10-01	45.84	46.08	45.27	45.82	1041681	{}	\N	2020-10-12 14:21:09.422966
28980	BG	2020-10-02	45.04	46.48	44.82	46.39	1028021	{}	\N	2020-10-12 14:21:09.427921
28981	BG	2020-10-05	46.78	47.52	46.56	47.24	1471269	{}	\N	2020-10-12 14:21:09.432553
28982	BG	2020-10-06	47.89	48.905	47.51	47.6	2119018	{}	\N	2020-10-12 14:21:09.436869
28983	BG	2020-10-07	48.08	49.215	48	49.06	2675933	{}	\N	2020-10-12 14:21:09.441123
28984	BG	2020-10-08	49.42	49.43	48.66	49.02	3018070	{}	\N	2020-10-12 14:21:09.445464
28985	BG	2020-10-09	49.5	51.21	49.385	50.47	3164362	{}	\N	2020-10-12 14:21:09.449776
28986	BG	2020-10-12	50.75	51	50.34	50.6026	217312	{}	\N	2020-10-12 14:21:09.454193
28987	BGSF	2020-09-28	8.6467	8.77	8.51	8.74	48861	{}	\N	2020-10-12 14:21:12.679387
28988	BGSF	2020-09-29	8.68	8.79	8.52	8.68	79882	{}	\N	2020-10-12 14:21:12.688264
28989	BGSF	2020-09-30	8.79	8.95	8.325	8.47	124111	{}	\N	2020-10-12 14:21:12.694823
28990	BGSF	2020-10-01	8.55	8.84	8.46	8.83	88205	{}	\N	2020-10-12 14:21:12.700777
28991	BGSF	2020-10-02	8.64	9.0337	8.47	9	135917	{}	\N	2020-10-12 14:21:12.706573
28992	BGSF	2020-10-05	9.14	9.29	9.0011	9.22	52215	{}	\N	2020-10-12 14:21:12.711896
28993	BGSF	2020-10-06	9.25	9.28	8.78	8.97	104672	{}	\N	2020-10-12 14:21:12.717045
28994	BGSF	2020-10-07	9.18	9.229	8.87	9.07	92239	{}	\N	2020-10-12 14:21:12.721685
28995	BGSF	2020-10-08	9.14	9.33	9.05	9.28	89066	{}	\N	2020-10-12 14:21:12.726375
28996	BGSF	2020-10-09	9.44	9.87	9.34	9.64	91563	{}	\N	2020-10-12 14:21:12.730536
28997	BGSF	2020-10-12	9.61	9.61	9.5973	9.5973	3452	{}	\N	2020-10-12 14:21:12.735366
28998	BHF	2020-09-28	26.6	27.625	26.6	27.28	568118	{}	\N	2020-10-12 14:21:14.022123
28999	BHF	2020-09-29	27.09	27.33	26.26	26.51	479798	{}	\N	2020-10-12 14:21:14.030774
29000	BHF	2020-09-30	26.8	27.46	26.7	26.91	728120	{}	\N	2020-10-12 14:21:14.037257
29001	BHF	2020-10-01	27.55	28.35	27.32	27.76	819828	{}	\N	2020-10-12 14:21:14.04521
29002	BHF	2020-10-02	27.02	28.79	27.02	28.69	543619	{}	\N	2020-10-12 14:21:14.050351
29003	BHF	2020-10-05	29.31	30.6	29.22	29.72	752548	{}	\N	2020-10-12 14:21:14.054845
29004	BHF	2020-10-06	30.33	30.94	29.485	29.79	761416	{}	\N	2020-10-12 14:21:14.059799
29005	BHF	2020-10-07	30.4	31.425	30.25	30.81	724598	{}	\N	2020-10-12 14:21:14.064775
29006	BHF	2020-10-08	31.15	31.56	30.78	31	729971	{}	\N	2020-10-12 14:21:14.069796
29007	BHF	2020-10-09	31.47	31.71	30.56	30.61	487653	{}	\N	2020-10-12 14:21:14.074618
29008	BHF	2020-10-12	30.61	31.155	30.47	30.94	49942	{}	\N	2020-10-12 14:21:14.07939
29009	BATRK	2020-09-28	20.55	21.16	20.55	20.77	97949	{}	\N	2020-10-12 14:21:15.086831
29010	BATRK	2020-09-29	20.72	20.79	20.04	20.38	100403	{}	\N	2020-10-12 14:21:15.097592
29011	BATRK	2020-09-30	20.55	21.09	20.53	21.01	204659	{}	\N	2020-10-12 14:21:15.104586
29012	BATRK	2020-10-01	21.12	21.23	20.61	21.2	124693	{}	\N	2020-10-12 14:21:15.110567
29013	BATRK	2020-10-02	21.115	21.115	20.53	20.71	110560	{}	\N	2020-10-12 14:21:15.116635
29014	BATRK	2020-10-05	20.875	21.33	20.61	21.15	106588	{}	\N	2020-10-12 14:21:15.121654
29015	BATRK	2020-10-06	21.4	22.13	21.16	21.48	169036	{}	\N	2020-10-12 14:21:15.126136
29016	BATRK	2020-10-07	21.73	22.09	21.53	21.66	156191	{}	\N	2020-10-12 14:21:15.130524
29017	BATRK	2020-10-08	21.8	22.51	21.65	22.34	140410	{}	\N	2020-10-12 14:21:15.135155
29018	BATRK	2020-10-09	22.61	22.69	22.19	22.23	92091	{}	\N	2020-10-12 14:21:15.139737
29019	BATRK	2020-10-12	22.43	22.5	22.25	22.32	15089	{}	\N	2020-10-12 14:21:15.144406
29020	BBVA	2020-09-28	2.7	2.77	2.695	2.76	3774781	{}	\N	2020-10-12 14:21:16.611257
29021	BBVA	2020-09-29	2.73	2.74	2.68	2.7	8091804	{}	\N	2020-10-12 14:21:16.622297
29022	BBVA	2020-09-30	2.75	2.8	2.73	2.74	6261236	{}	\N	2020-10-12 14:21:16.628612
29023	BBVA	2020-10-01	2.78	2.8	2.74	2.76	4282565	{}	\N	2020-10-12 14:21:16.634135
29024	BBVA	2020-10-02	2.69	2.785	2.69	2.78	3200772	{}	\N	2020-10-12 14:21:16.639404
29025	BBVA	2020-10-05	2.77	2.82	2.765	2.81	5363171	{}	\N	2020-10-12 14:21:16.644321
29026	BBVA	2020-10-06	2.91	2.92	2.82	2.83	10290290	{}	\N	2020-10-12 14:21:16.649541
29027	BBVA	2020-10-07	2.84	2.86	2.79	2.81	3524177	{}	\N	2020-10-12 14:21:16.653643
29028	BBVA	2020-10-08	2.86	2.86	2.815	2.84	3550520	{}	\N	2020-10-12 14:21:16.657583
29029	BBVA	2020-10-09	2.84	2.85	2.78	2.8	4359753	{}	\N	2020-10-12 14:21:16.661784
29030	BBVA	2020-10-12	2.76	2.76	2.74	2.743	613261	{}	\N	2020-10-12 14:21:16.666035
29031	BCH	2020-09-28	15.485	15.4852	15.12	15.13	76270	{}	\N	2020-10-12 14:21:17.61069
29032	BCH	2020-09-29	15.16	15.17	14.88	14.91	182318	{}	\N	2020-10-12 14:21:17.622185
29033	BCH	2020-09-30	15.03	15.2999	15	15.12	201154	{}	\N	2020-10-12 14:21:17.62884
29034	BCH	2020-10-01	15.3	15.54	15.27	15.36	238625	{}	\N	2020-10-12 14:21:17.635343
29035	BCH	2020-10-02	15.21	15.9	15.21	15.78	204393	{}	\N	2020-10-12 14:21:17.64135
29036	BCH	2020-10-05	15.7	15.89	15.5051	15.7	206430	{}	\N	2020-10-12 14:21:17.646591
29037	BCH	2020-10-06	15.52	15.6	15.02	15.03	250553	{}	\N	2020-10-12 14:21:17.651857
29038	BCH	2020-10-07	15.13	15.14	14.835	14.95	141770	{}	\N	2020-10-12 14:21:17.657313
29039	BCH	2020-10-08	14.94	15.18	14.89	15.16	263211	{}	\N	2020-10-12 14:21:17.66241
29040	BCH	2020-10-09	15.2	15.45	15.17	15.36	205309	{}	\N	2020-10-12 14:21:17.667028
29041	BCH	2020-10-12	15.37	15.47	15.33	15.36	10222	{}	\N	2020-10-12 14:21:17.67196
29042	BEN	2020-09-28	20	20.575	19.99	20.47	2706499	{}	\N	2020-10-12 14:21:20.165521
29043	BEN	2020-09-29	20.09	20.18	19.785	20.04	4316572	{}	\N	2020-10-12 14:21:20.176116
29044	BEN	2020-09-30	20.13	20.546	20.07	20.35	4578194	{}	\N	2020-10-12 14:21:20.182382
29045	BEN	2020-10-01	20.31	20.39	19.83	19.99	3981164	{}	\N	2020-10-12 14:21:20.187662
29046	BEN	2020-10-02	19.74	20.585	19.71	20.52	4468815	{}	\N	2020-10-12 14:21:20.191964
29047	BEN	2020-10-05	20.78	21.1	20.72	20.8	5519326	{}	\N	2020-10-12 14:21:20.196547
29048	BEN	2020-10-06	20.99	21.59	20.82	20.91	8285538	{}	\N	2020-10-12 14:21:20.200835
29049	BEN	2020-10-07	20.86	21.85	20.86	21.74	7127298	{}	\N	2020-10-12 14:21:20.205097
29050	BEN	2020-10-08	22.22	22.7	22.17	22.32	13173358	{}	\N	2020-10-12 14:21:20.209476
29051	BEN	2020-10-09	23.17	23.3	22.475	22.59	12937364	{}	\N	2020-10-12 14:21:20.213776
29052	BEN	2020-10-12	22.59	22.9127	22.53	22.82	925378	{}	\N	2020-10-12 14:21:20.217991
29053	BFK	2020-09-28	13.65	13.738	13.61	13.73	44513	{}	\N	2020-10-12 14:21:21.212248
29054	BFK	2020-09-29	13.74	13.78	13.68	13.75	81868	{}	\N	2020-10-12 14:21:21.22407
29055	BFK	2020-09-30	13.75	13.81	13.71	13.71	130386	{}	\N	2020-10-12 14:21:21.231014
29056	BFK	2020-10-01	13.75	13.98	13.73	13.84	217118	{}	\N	2020-10-12 14:21:21.236906
29057	BFK	2020-10-02	13.76	13.89	13.76	13.84	95898	{}	\N	2020-10-12 14:21:21.242859
29058	BFK	2020-10-05	13.82	13.89	13.72	13.75	123142	{}	\N	2020-10-12 14:21:21.247839
29059	BFK	2020-10-06	13.74	13.83	13.74	13.74	94828	{}	\N	2020-10-12 14:21:21.253311
29060	BFK	2020-10-07	13.81	13.86	13.7601	13.82	68036	{}	\N	2020-10-12 14:21:21.257657
29061	BFK	2020-10-08	13.88	13.93	13.8403	13.86	104062	{}	\N	2020-10-12 14:21:21.261846
29062	BFK	2020-10-09	13.85	13.932	13.81	13.9	98978	{}	\N	2020-10-12 14:21:21.265805
29063	BFK	2020-10-12	13.87	13.957	13.86	13.895	13202	{}	\N	2020-10-12 14:21:21.26979
29064	BGFV	2020-09-28	6.02	6.4	5.89	6.38	999706	{}	\N	2020-10-12 14:21:22.519968
29065	BGFV	2020-09-29	6.4	6.71	6.16	6.44	973576	{}	\N	2020-10-12 14:21:22.527533
29066	BGFV	2020-09-30	6.44	7.6	6.43	7.48	3935146	{}	\N	2020-10-12 14:21:22.53321
29067	BGFV	2020-10-01	7.6	7.95	7.27	7.85	2262357	{}	\N	2020-10-12 14:21:22.538399
29068	BGFV	2020-10-02	7.46	8.18	7.4	7.86	1608070	{}	\N	2020-10-12 14:21:22.543256
29069	BGFV	2020-10-05	8.02	8.235	7.69	7.99	1293512	{}	\N	2020-10-12 14:21:22.547465
29070	BGFV	2020-10-06	8	8.64	7.41	7.75	2281428	{}	\N	2020-10-12 14:21:22.5515
29071	BGFV	2020-10-07	7.97	8.6	7.87	8.52	1229319	{}	\N	2020-10-12 14:21:22.555626
29072	BGFV	2020-10-08	8.68	8.86	8.31	8.56	1077869	{}	\N	2020-10-12 14:21:22.55962
29073	BGFV	2020-10-09	8.5125	8.63	8.06	8.1	652017	{}	\N	2020-10-12 14:21:22.56351
29074	BGFV	2020-10-12	8.22	8.22	7.79	7.9401	214510	{}	\N	2020-10-12 14:21:22.567443
29075	BGY	2020-09-28	5.26	5.3	5.24	5.29	221792	{}	\N	2020-10-12 14:21:23.537252
29076	BGY	2020-09-29	5.29	5.32	5.26	5.28	309450	{}	\N	2020-10-12 14:21:23.546203
29077	BGY	2020-09-30	5.32	5.35	5.27	5.27	916742	{}	\N	2020-10-12 14:21:23.552401
29078	BGY	2020-10-01	5.2664	5.33	5.2664	5.28	466760	{}	\N	2020-10-12 14:21:23.55815
29079	BGY	2020-10-02	5.25	5.31	5.25	5.29	307519	{}	\N	2020-10-12 14:21:23.563357
29080	BGY	2020-10-05	5.3	5.39	5.3	5.35	449219	{}	\N	2020-10-12 14:21:23.567834
29081	BGY	2020-10-06	5.37	5.38	5.32	5.33	506350	{}	\N	2020-10-12 14:21:23.572209
29082	BGY	2020-10-07	5.35	5.39	5.34	5.35	459200	{}	\N	2020-10-12 14:21:23.576245
29083	BGY	2020-10-08	5.37	5.4	5.36	5.39	181631	{}	\N	2020-10-12 14:21:23.580213
29084	BGY	2020-10-09	5.42	5.45	5.38	5.39	238228	{}	\N	2020-10-12 14:21:23.584355
29085	BGY	2020-10-12	5.42	5.46	5.42	5.455	53774	{}	\N	2020-10-12 14:21:23.588651
29086	BHV	2020-09-28	16.23	16.23	16.0637	16.0637	940	{}	\N	2020-10-12 14:21:24.820835
29087	BHV	2020-09-29	16.12	16.23	16.12	16.23	5768	{}	\N	2020-10-12 14:21:24.829408
29088	BHV	2020-09-30	16.292	16.36	16.257	16.36	10694	{}	\N	2020-10-12 14:21:24.836606
29089	BHV	2020-10-01	16.37	17	16.37	16.8835	5259	{}	\N	2020-10-12 14:21:24.842166
29090	BHV	2020-10-02	16.7934	16.7934	16.5	16.5	540	{}	\N	2020-10-12 14:21:24.847051
29091	BHV	2020-10-05	15.9	16.3	15.9	16.27	4322	{}	\N	2020-10-12 14:21:24.851517
29092	BHV	2020-10-08	16.4055	16.4055	16.4055	16.4055	6088	{}	\N	2020-10-12 14:21:24.855886
29093	BIO	2020-09-28	509.18	519.465	509.18	513.92	108883	{}	\N	2020-10-12 14:21:25.8178
29094	BIO	2020-09-29	517.06	521.06	513.54	515.1	190336	{}	\N	2020-10-12 14:21:25.828077
29095	BIO	2020-09-30	518.9	525.61	510.74	515.46	381365	{}	\N	2020-10-12 14:21:25.834518
29096	BIO	2020-10-01	519.19	521.23	512.665	520.81	280922	{}	\N	2020-10-12 14:21:25.840246
29097	BIO	2020-10-02	513.26	518.52	507.82	510.68	148634	{}	\N	2020-10-12 14:21:25.845073
29098	BIO	2020-10-05	511.76	534.555	511.76	530.66	443511	{}	\N	2020-10-12 14:21:25.851116
29099	BIO	2020-10-06	529.51	537.695	520.36	533.33	268104	{}	\N	2020-10-12 14:21:25.856718
29100	BIO	2020-10-07	539	549.56	536.795	545.39	336797	{}	\N	2020-10-12 14:21:25.861029
29101	BIO	2020-10-08	549.95	551.14	540.76	551.1	188812	{}	\N	2020-10-12 14:21:25.865065
29102	BIO	2020-10-09	552.9	567.33	551.94	566.14	188315	{}	\N	2020-10-12 14:21:25.869208
29103	BIO	2020-10-12	572.71	572.71	560.34	565.03	20538	{}	\N	2020-10-12 14:21:25.873324
29104	BIZD	2020-09-28	12.15	12.45	12.15	12.335	83532	{}	\N	2020-10-12 14:21:26.753769
29105	BIZD	2020-09-29	12.39	12.39	12.13	12.13	80008	{}	\N	2020-10-12 14:21:26.762648
29106	BIZD	2020-09-30	12.18	12.34	12.15	12.15	159716	{}	\N	2020-10-12 14:21:26.767896
29107	BIZD	2020-10-01	12	12.07	11.8244	12.03	109130	{}	\N	2020-10-12 14:21:26.772775
29108	BIZD	2020-10-02	11.81	12.1499	11.81	12.11	92818	{}	\N	2020-10-12 14:21:26.77718
29109	BIZD	2020-10-05	12.11	12.24	12.09	12.09	119643	{}	\N	2020-10-12 14:21:26.782416
29110	BIZD	2020-10-06	12.09	12.17	12	12	68562	{}	\N	2020-10-12 14:21:26.786774
29111	BIZD	2020-10-07	12.19	12.19	12.01	12.08	193895	{}	\N	2020-10-12 14:21:26.790751
29112	BIZD	2020-10-08	12.08	12.2	12.08	12.18	162445	{}	\N	2020-10-12 14:21:26.795075
29113	BIZD	2020-10-09	12.2229	12.24	12.11	12.14	78451	{}	\N	2020-10-12 14:21:26.799392
29114	BIZD	2020-10-12	12.18	12.18	12.081	12.081	9307	{}	\N	2020-10-12 14:21:26.804117
29115	GPMT	2020-09-28	6.09	7.01	6.05	6.86	1874442	{}	\N	2020-10-12 14:21:27.991746
29116	GPMT	2020-09-29	7.58	8.02	7.02	7.15	3965605	{}	\N	2020-10-12 14:21:28.000602
29117	GPMT	2020-09-30	7.08	7.3	6.99	7.09	1820474	{}	\N	2020-10-12 14:21:28.005842
29118	GPMT	2020-10-01	7.1	7.32	6.98	7.27	1210497	{}	\N	2020-10-12 14:21:28.011195
29119	GPMT	2020-10-02	7.01	7.5578	7	7.49	1108037	{}	\N	2020-10-12 14:21:28.015154
29120	GPMT	2020-10-05	7.55	7.66	7.37	7.48	1452290	{}	\N	2020-10-12 14:21:28.01969
29121	GPMT	2020-10-06	7.58	7.8	7.36	7.4	1683099	{}	\N	2020-10-12 14:21:28.023996
29122	GPMT	2020-10-07	7.28	7.29	6.99	7.18	1947213	{}	\N	2020-10-12 14:21:28.027254
29123	GPMT	2020-10-08	7.25	7.51	7.15	7.4	1609549	{}	\N	2020-10-12 14:21:28.030423
29124	GPMT	2020-10-09	7.48	7.4902	7.07	7.1	891150	{}	\N	2020-10-12 14:21:28.033666
29125	GPMT	2020-10-12	7.01	7.12	6.91	7.06	447452	{}	\N	2020-10-12 14:21:28.036928
29126	BDX	2020-09-28	226.41	227.91	224.71	225	1045808	{}	\N	2020-10-12 14:21:29.804605
29127	BDX	2020-09-29	226.1	226.98	224.93	225.43	1680788	{}	\N	2020-10-12 14:21:29.814905
29128	BDX	2020-09-30	227.31	235.57	226.8	232.68	4550403	{}	\N	2020-10-12 14:21:29.821696
29129	BDX	2020-10-01	234.41	236.82	228.65	230.85	2276924	{}	\N	2020-10-12 14:21:29.827115
29130	BDX	2020-10-02	231.9	234.999	231	232.66	1901641	{}	\N	2020-10-12 14:21:29.831682
29131	BDX	2020-10-05	234.17	238.51	234.1301	238.14	1809979	{}	\N	2020-10-12 14:21:29.835871
29132	BDX	2020-10-06	238	240.32	235.03	235.45	1985090	{}	\N	2020-10-12 14:21:29.840528
29133	BDX	2020-10-07	236.27	237.73	234.05	235.7	2029608	{}	\N	2020-10-12 14:21:29.84429
29134	BDX	2020-10-08	235.77	237.38	234.44	236.21	2005481	{}	\N	2020-10-12 14:21:29.847741
29135	BDX	2020-10-09	237.52	238.125	234.42	237.31	3142783	{}	\N	2020-10-12 14:21:29.851704
29136	BDX	2020-10-12	238.08	238.9	236.71	238.7	221651	{}	\N	2020-10-12 14:21:29.856043
29137	BFOR	2020-09-28	41.845	41.94	41.845	41.94	572	{}	\N	2020-10-12 14:21:31.446989
29138	BFOR	2020-09-29	41.7	41.84	41.7	41.84	294	{}	\N	2020-10-12 14:21:31.454288
29139	BFOR	2020-09-30	42.301	42.32	41.95	41.99	1879	{}	\N	2020-10-12 14:21:31.459987
29140	BFOR	2020-10-01	42.261	42.2973	42.17	42.26	3612	{}	\N	2020-10-12 14:21:31.464754
29141	BFOR	2020-10-02	42.1559	42.555	42.1559	42.48	504	{}	\N	2020-10-12 14:21:31.469368
29142	BFOR	2020-10-05	43.391	43.391	43.37	43.37	1796	{}	\N	2020-10-12 14:21:31.473732
29143	BFOR	2020-10-06	43.26	44	43.14	43.14	10778	{}	\N	2020-10-12 14:21:31.4776
29144	BFOR	2020-10-07	43.6523	43.919	43.6523	43.919	5800	{}	\N	2020-10-12 14:21:31.481967
29145	BFOR	2020-10-08	44.32	44.3673	44.32	44.3673	1301	{}	\N	2020-10-12 14:21:31.485704
29146	BFOR	2020-10-09	44.57	44.79	44.57	44.6927	990	{}	\N	2020-10-12 14:21:31.48966
29147	BFOR	2020-10-12	44.7	44.8299	44.7	44.8299	866	{}	\N	2020-10-12 14:21:31.493989
29148	BGR	2020-09-28	6.0602	6.13	6.04	6.06	114161	{}	\N	2020-10-12 14:21:32.795809
29149	BGR	2020-09-29	6.08	6.08	5.885	5.94	374632	{}	\N	2020-10-12 14:21:32.805335
29150	BGR	2020-09-30	5.96	6.027	5.94	5.97	262892	{}	\N	2020-10-12 14:21:32.812
29151	BGR	2020-10-01	5.92	5.92	5.78	5.8	573309	{}	\N	2020-10-12 14:21:32.81768
29152	BGR	2020-10-02	5.62	5.8836	5.62	5.87	360585	{}	\N	2020-10-12 14:21:32.822662
29153	BGR	2020-10-05	5.91	6.02	5.9	6	267844	{}	\N	2020-10-12 14:21:32.827594
29154	BGR	2020-10-06	6.06	6.13	5.98	5.98	341057	{}	\N	2020-10-12 14:21:32.831963
29155	BGR	2020-10-07	6.02	6.08	5.98	6.08	280854	{}	\N	2020-10-12 14:21:32.836402
29156	BGR	2020-10-08	6.12	6.36	6.1	6.28	389650	{}	\N	2020-10-12 14:21:32.840967
29157	BGR	2020-10-09	6.33	6.38	6.11	6.17	468524	{}	\N	2020-10-12 14:21:32.845173
29158	BGR	2020-10-12	6.14	6.17	6.124	6.16	50878	{}	\N	2020-10-12 14:21:32.849563
29159	BHE	2020-09-28	20.03	20.68	20.03	20.55	201915	{}	\N	2020-10-12 14:21:34.308263
29160	BHE	2020-09-29	20.35	20.6	20.17	20.36	241037	{}	\N	2020-10-12 14:21:34.315302
29161	BHE	2020-09-30	20.37	20.78	20.11	20.15	618654	{}	\N	2020-10-12 14:21:34.325546
29162	BHE	2020-10-01	20.17	20.43	20.01	20.26	279275	{}	\N	2020-10-12 14:21:34.332097
29163	BHE	2020-10-02	19.77	20.3	19.73	20.18	504143	{}	\N	2020-10-12 14:21:34.341746
29164	BHE	2020-10-05	20.31	20.82	20.31	20.65	283140	{}	\N	2020-10-12 14:21:34.348012
29165	BHE	2020-10-06	20.74	21.155	20.57	20.87	630134	{}	\N	2020-10-12 14:21:34.354355
29166	BHE	2020-10-07	21.07	21.57	21.07	21.44	584796	{}	\N	2020-10-12 14:21:34.360224
29167	BHE	2020-10-08	21.69	22.01	21.6	21.96	226425	{}	\N	2020-10-12 14:21:34.365891
29168	BHE	2020-10-09	22.19	22.22	21.84	21.87	282666	{}	\N	2020-10-12 14:21:34.372491
29169	BHE	2020-10-12	21.88	22.34	21.85	22.1	121989	{}	\N	2020-10-12 14:21:34.378564
29170	BIF	2020-09-28	10	10.0899	9.9601	10.02	97698	{}	\N	2020-10-12 14:21:35.330307
29171	BIF	2020-09-29	10.01	10.01	9.91	9.91	217086	{}	\N	2020-10-12 14:21:35.336998
29172	BIF	2020-09-30	9.91	10.05	9.91	10	325987	{}	\N	2020-10-12 14:21:35.342376
29173	BIF	2020-10-01	10.04	10.1	9.99	10.03	377492	{}	\N	2020-10-12 14:21:35.346701
29174	BIF	2020-10-02	9.9	10.09	9.9	9.99	229756	{}	\N	2020-10-12 14:21:35.351718
29175	BIF	2020-10-05	10.07	10.11	10.04	10.07	191547	{}	\N	2020-10-12 14:21:35.357705
29176	BIF	2020-10-06	10.07	10.14	10.002	10.03	181229	{}	\N	2020-10-12 14:21:35.362839
29177	BIF	2020-10-07	10.07	10.146	10.05	10.11	280682	{}	\N	2020-10-12 14:21:35.368715
29178	BIF	2020-10-08	10.16	10.24	10.1301	10.24	132492	{}	\N	2020-10-12 14:21:35.375324
29179	BIF	2020-10-09	10.27	10.3041	10.17	10.17	253872	{}	\N	2020-10-12 14:21:35.381
29180	BIF	2020-10-12	10.21	10.24	10.21	10.2242	16360	{}	\N	2020-10-12 14:21:35.386932
29181	BIP	2020-09-28	47.56	48.33	47.56	47.77	223633	{}	\N	2020-10-12 14:21:36.392772
29182	BIP	2020-09-29	47.8	47.81	47.15	47.41	267180	{}	\N	2020-10-12 14:21:36.398094
29183	BIP	2020-09-30	47.78	48.0093	47.22	47.62	742395	{}	\N	2020-10-12 14:21:36.404762
29184	BIP	2020-10-01	48.04	48.19	47.61	47.8	463299	{}	\N	2020-10-12 14:21:36.411033
29185	BIP	2020-10-02	46.66	48.18	46.66	47.8	664191	{}	\N	2020-10-12 14:21:36.417368
29186	BIP	2020-10-05	48	48.55	47.745	48.32	394860	{}	\N	2020-10-12 14:21:36.422836
29187	BIP	2020-10-06	48.64	48.64	47.66	48.03	331126	{}	\N	2020-10-12 14:21:36.427447
29188	BIP	2020-10-07	48.18	48.52	47.74	47.75	645111	{}	\N	2020-10-12 14:21:36.432149
29189	BIP	2020-10-08	47.85	48.15	46.98	47.34	524824	{}	\N	2020-10-12 14:21:36.438437
29190	BIP	2020-10-09	47.72	47.72	46.9025	47.25	590082	{}	\N	2020-10-12 14:21:36.443284
29191	BIP	2020-10-12	47.25	47.5	47.25	47.46	56388	{}	\N	2020-10-12 14:21:36.44865
29192	BK	2020-09-28	33.97	34.61	33.86	34.29	3046364	{}	\N	2020-10-12 14:21:37.415199
29193	BK	2020-09-29	34.08	34.16	33.63	33.87	8219575	{}	\N	2020-10-12 14:21:37.429331
29194	BK	2020-09-30	33.87	34.52	33.87	34.34	16094932	{}	\N	2020-10-12 14:21:37.445897
29195	BK	2020-10-01	34.46	34.705	33.91	34.17	14762113	{}	\N	2020-10-12 14:21:37.453959
29196	BK	2020-10-02	33.88	34.95	33.81	34.74	10532863	{}	\N	2020-10-12 14:21:37.460908
29197	BK	2020-10-05	35.04	35.635	35.01	35.38	6831735	{}	\N	2020-10-12 14:21:37.465986
29198	BK	2020-10-06	35.8	36.2	35.16	35.23	10019720	{}	\N	2020-10-12 14:21:37.470857
29199	BK	2020-10-07	35.42	36.67	35.42	36.46	11371297	{}	\N	2020-10-12 14:21:37.475423
29200	BK	2020-10-08	36.61	37.21	36.47	37.18	7106263	{}	\N	2020-10-12 14:21:37.479785
29201	BK	2020-10-09	37.3	37.98	37.15	37.7	11515825	{}	\N	2020-10-12 14:21:37.484544
29202	BK	2020-10-12	37.62	37.97	37.545	37.75	1979012	{}	\N	2020-10-12 14:21:37.489487
29203	BL	2020-09-28	90.13	90.95	88.2801	90.84	444027	{}	\N	2020-10-12 14:21:39.027588
29204	BL	2020-09-29	90.2	90.73	89.24	90.33	444200	{}	\N	2020-10-12 14:21:39.034975
29205	BL	2020-09-30	90.67	91.55	89.3153	89.63	550091	{}	\N	2020-10-12 14:21:39.04139
29206	BL	2020-10-01	90.9999	91.475	90.3111	90.94	299141	{}	\N	2020-10-12 14:21:39.046777
29207	BL	2020-10-02	88.9	91.75	87.33	88.17	455804	{}	\N	2020-10-12 14:21:39.052078
29208	BL	2020-10-05	88.49	88.99	87.34	88.26	340862	{}	\N	2020-10-12 14:21:39.056855
29209	BL	2020-10-06	89.91	95.36	88.45	92.7	503814	{}	\N	2020-10-12 14:21:39.06205
29210	BL	2020-10-07	93.85	98	93.6247	96.82	534754	{}	\N	2020-10-12 14:21:39.066811
29211	BL	2020-10-08	98.01	98.4499	95.5	96.26	320306	{}	\N	2020-10-12 14:21:39.072384
29212	BL	2020-10-09	96.52	98.56	96.34	98.43	308440	{}	\N	2020-10-12 14:21:39.077364
29213	BL	2020-10-12	100	100.58	98.31	98.98	41116	{}	\N	2020-10-12 14:21:39.083016
29214	BLHY	2020-09-28	23.145	23.145	23.145	23.145	1	{}	\N	2020-10-12 14:21:40.903444
29215	BLHY	2020-09-29	23.1591	23.1591	23.1591	23.1591	6	{}	\N	2020-10-12 14:21:40.914333
29216	BLHY	2020-09-30	23.2	23.2	23.2	23.2	106	{}	\N	2020-10-12 14:21:40.92504
29217	BLHY	2020-10-01	23.23	23.23	23.23	23.23	3	{}	\N	2020-10-12 14:21:40.93064
29218	BLHY	2020-10-02	23.23	23.23	23.2	23.2	300	{}	\N	2020-10-12 14:21:40.937506
29219	BLHY	2020-10-05	23.2	23.23	23.2	23.23	966	{}	\N	2020-10-12 14:21:40.943574
29220	BLHY	2020-10-06	23.38	23.38	23.38	23.38	79	{}	\N	2020-10-12 14:21:40.950581
29221	BLHY	2020-10-07	23.34	23.41	23.34	23.41	308	{}	\N	2020-10-12 14:21:40.959614
29222	BLHY	2020-10-08	23.47	23.47	23.47	23.47	0	{}	\N	2020-10-12 14:21:40.968579
29223	BLHY	2020-10-09	23.51	23.51	23.475	23.475	177	{}	\N	2020-10-12 14:21:40.975219
29731	BNTC	2020-10-01	6.09	6.21	4.51	4.7899	131328	{}	\N	2020-10-12 14:22:44.396813
29732	BNTC	2020-10-02	2.9	3.2	2.75	2.98	1675787	{}	\N	2020-10-12 14:22:44.402043
29733	BNTC	2020-10-05	3.15	3.25	3.05	3.2	430289	{}	\N	2020-10-12 14:22:44.406841
29734	BNTC	2020-10-06	3.16	3.18	3.01	3.0552	257384	{}	\N	2020-10-12 14:22:44.410973
29735	BNTC	2020-10-07	3.12	3.1885	3.05	3.11	349627	{}	\N	2020-10-12 14:22:44.41538
29736	BNTC	2020-10-08	3.1	3.15	2.91	3.05	331038	{}	\N	2020-10-12 14:22:44.419648
29737	BNTC	2020-10-09	3.12	3.12	3	3.1	160457	{}	\N	2020-10-12 14:22:44.423716
29738	BNTC	2020-10-12	3.08	3.08	3	3.05	49427	{}	\N	2020-10-12 14:22:44.427583
29739	BOKFL	2020-09-28	25.85	25.85	25.48	25.66	23849	{}	\N	2020-10-12 14:22:46.243684
29740	BOKFL	2020-09-29	25.705	25.74	25.63	25.65	7826	{}	\N	2020-10-12 14:22:46.252853
29741	BOKFL	2020-09-30	26.14	26.14	25.51	25.8	49498	{}	\N	2020-10-12 14:22:46.258704
29742	BOKFL	2020-10-01	25.61	25.61	25.34	25.34	39859	{}	\N	2020-10-12 14:22:46.26421
29743	BOKFL	2020-10-02	25.33	25.48	25.26	25.45	41332	{}	\N	2020-10-12 14:22:46.269032
29744	BOKFL	2020-10-05	25.31	25.54	25.3	25.47	10976	{}	\N	2020-10-12 14:22:46.273531
29745	BOKFL	2020-10-06	25.57	25.6104	25.39	25.48	8904	{}	\N	2020-10-12 14:22:46.277609
29746	BOKFL	2020-10-07	25.605	25.63	25.45	25.485	27734	{}	\N	2020-10-12 14:22:46.282046
29747	BOKFL	2020-10-08	25.52	25.72	25.52	25.68	11866	{}	\N	2020-10-12 14:22:46.286302
29748	BOKFL	2020-10-09	25.57	25.6	25.42	25.549	26237	{}	\N	2020-10-12 14:22:46.29044
29749	BOKFL	2020-10-12	25.57	25.57	25.5	25.57	1260	{}	\N	2020-10-12 14:22:46.294981
29750	BPFH	2020-09-28	5.38	5.695	5.37	5.62	1105688	{}	\N	2020-10-12 14:22:48.305079
29751	BPFH	2020-09-29	5.59	5.6135	5.47	5.52	826990	{}	\N	2020-10-12 14:22:48.316344
29752	BPFH	2020-09-30	5.55	5.67	5.47	5.52	1113784	{}	\N	2020-10-12 14:22:48.325968
29753	BPFH	2020-10-01	5.53	5.61	5.455	5.57	1111016	{}	\N	2020-10-12 14:22:48.33211
29754	BPFH	2020-10-02	5.51	5.79	5.42	5.7	1087313	{}	\N	2020-10-12 14:22:48.337361
29755	BPFH	2020-10-05	5.88	5.96	5.68	5.95	1456480	{}	\N	2020-10-12 14:22:48.342053
29756	BPFH	2020-10-06	5.87	6.26	5.87	6.055	2142331	{}	\N	2020-10-12 14:22:48.346427
29757	BPFH	2020-10-07	6.12	6.3	6.11	6.27	1219484	{}	\N	2020-10-12 14:22:48.351131
29758	BPFH	2020-10-08	6.32	6.34	6.26	6.27	430381	{}	\N	2020-10-12 14:22:48.355549
29759	BPFH	2020-10-09	6.3	6.355	6.25	6.31	669490	{}	\N	2020-10-12 14:22:48.361508
29760	BPFH	2020-10-12	6.29	6.3	6.21	6.22	113897	{}	\N	2020-10-12 14:22:48.365997
29761	BPRN	2020-09-28	18.2574	18.81	18.2574	18.46	5796	{}	\N	2020-10-12 14:22:49.542574
29762	BPRN	2020-09-29	17.7634	18.38	17.7634	18.38	3515	{}	\N	2020-10-12 14:22:49.551626
29763	BPRN	2020-09-30	18.77	18.77	18.17	18.17	2813	{}	\N	2020-10-12 14:22:49.557824
29764	BPRN	2020-10-02	18.18	18.18	18.18	18.18	2055	{}	\N	2020-10-12 14:22:49.563708
29765	BPRN	2020-10-05	18.77	19.25	18.46	19.25	5036	{}	\N	2020-10-12 14:22:49.568595
29766	BPRN	2020-10-06	19.38	20.105	19.25	19.7	7518	{}	\N	2020-10-12 14:22:49.573656
29767	BPRN	2020-10-07	19.885	20.25	19.885	20.25	5565	{}	\N	2020-10-12 14:22:49.577632
29768	BPRN	2020-10-08	19.88	20.7	19.88	20.18	7163	{}	\N	2020-10-12 14:22:49.58167
29769	BPRN	2020-10-09	20.4	20.4	20.3	20.3	1283	{}	\N	2020-10-12 14:22:49.585482
29770	GRA	2020-09-28	42.19	43.11	41.78	41.83	431448	{}	\N	2020-10-12 14:22:51.015327
29771	GRA	2020-09-29	41.9	42.04	40.6	40.63	438222	{}	\N	2020-10-12 14:22:51.021437
29772	GRA	2020-09-30	40.58	41.28	40.07	40.29	582987	{}	\N	2020-10-12 14:22:51.025831
29773	GRA	2020-10-01	40.12	40.71	39.42	39.55	1020225	{}	\N	2020-10-12 14:22:51.031181
29774	GRA	2020-10-02	38.7	40.68	38.7	40.1	942699	{}	\N	2020-10-12 14:22:51.035773
29775	GRA	2020-10-05	40.55	41.18	40.25	40.32	740832	{}	\N	2020-10-12 14:22:51.040287
29776	GRA	2020-10-06	40.89	41.05	39.79	39.8	1015723	{}	\N	2020-10-12 14:22:51.044569
29777	GRA	2020-10-07	40.08	40.43	39.6	40.24	1217270	{}	\N	2020-10-12 14:22:51.048779
29224	BCEI	2020-09-28	19.02	19.82	18.46	19.6	109322	{}	\N	2020-10-12 14:21:42.276262
29225	BCEI	2020-09-29	19.42	19.71	18.87	19.57	208199	{}	\N	2020-10-12 14:21:42.283744
29226	BCEI	2020-09-30	19.58	19.71	18.76	18.8	306218	{}	\N	2020-10-12 14:21:42.288005
29227	BCEI	2020-10-01	18.56	18.5943	17.46	18.26	445630	{}	\N	2020-10-12 14:21:42.293721
29228	BCEI	2020-10-02	17.71	18.82	17.6974	18.62	257941	{}	\N	2020-10-12 14:21:42.299339
29229	BCEI	2020-10-05	19	19.75	18.88	19.67	288902	{}	\N	2020-10-12 14:21:42.304512
29230	BCEI	2020-10-06	20	20.45	19.18	19.18	437203	{}	\N	2020-10-12 14:21:42.309538
29231	BCEI	2020-10-07	19.31	20.07	19.03	19.69	493919	{}	\N	2020-10-12 14:21:42.314217
29232	BCEI	2020-10-08	19.88	20.77	19.83	20.65	453266	{}	\N	2020-10-12 14:21:42.318903
29233	BCEI	2020-10-09	20.77	20.77	20.32	20.5	355462	{}	\N	2020-10-12 14:21:42.323538
29234	BCEI	2020-10-12	20.43	20.43	20.05	20.11	17405	{}	\N	2020-10-12 14:21:42.333966
29235	BCS	2020-09-28	4.93	5.03	4.925	4.99	3469883	{}	\N	2020-10-12 14:21:43.592411
29236	BCS	2020-09-29	4.99	5.02	4.92	4.96	7385290	{}	\N	2020-10-12 14:21:43.603571
29237	BCS	2020-09-30	5.06	5.1	4.99	5.01	5886790	{}	\N	2020-10-12 14:21:43.612157
29238	BCS	2020-10-01	5.02	5.0501	4.95	5	3811530	{}	\N	2020-10-12 14:21:43.619359
29239	BCS	2020-10-02	4.94	5.11	4.94	5.08	6075113	{}	\N	2020-10-12 14:21:43.626711
29240	BCS	2020-10-05	5.07	5.145	5.065	5.13	5154478	{}	\N	2020-10-12 14:21:43.632801
29241	BCS	2020-10-06	5.34	5.39	5.252	5.28	12142559	{}	\N	2020-10-12 14:21:43.639137
29242	BCS	2020-10-07	5.46	5.505	5.35	5.39	5334899	{}	\N	2020-10-12 14:21:43.645625
29243	BCS	2020-10-08	5.51	5.55	5.465	5.54	3693514	{}	\N	2020-10-12 14:21:43.652405
29244	BCS	2020-10-09	5.6	5.6064	5.52	5.55	4451191	{}	\N	2020-10-12 14:21:43.658065
29245	BCS	2020-10-12	5.48	5.495	5.46	5.469	890207	{}	\N	2020-10-12 14:21:43.663702
29246	BDGE	2020-09-28	17.48	17.8099	17.35	17.54	61640	{}	\N	2020-10-12 14:21:44.701451
29247	BDGE	2020-09-29	17.45	17.45	16.92	17.34	43515	{}	\N	2020-10-12 14:21:44.710847
29248	BDGE	2020-09-30	17.27	17.81	17.27	17.43	49857	{}	\N	2020-10-12 14:21:44.718743
29249	BDGE	2020-10-01	17.37	17.56	17.06	17.56	77616	{}	\N	2020-10-12 14:21:44.723757
29250	BDGE	2020-10-02	17.47	18.16	17.41	17.91	114777	{}	\N	2020-10-12 14:21:44.728303
29251	BDGE	2020-10-05	18.12	18.58	18.06	18.54	79244	{}	\N	2020-10-12 14:21:44.733142
29252	BDGE	2020-10-06	18.81	19.39	18.64	18.75	119413	{}	\N	2020-10-12 14:21:44.737808
29253	BDGE	2020-10-07	18.97	19.2699	18.755	19.04	57629	{}	\N	2020-10-12 14:21:44.74324
29254	BDGE	2020-10-08	19.09	19.51	18.637	19.38	77895	{}	\N	2020-10-12 14:21:44.748471
29255	BDGE	2020-10-09	19.4	19.57	19.17	19.32	61577	{}	\N	2020-10-12 14:21:44.752785
29256	BDGE	2020-10-12	19.29	19.34	19.18	19.34	2991	{}	\N	2020-10-12 14:21:44.757965
29257	BEAT	2020-09-28	44.72	45.57	44.62	44.85	261066	{}	\N	2020-10-12 14:21:46.108149
29258	BEAT	2020-09-29	45.36	45.59	44.46	44.88	250979	{}	\N	2020-10-12 14:21:46.116544
29259	BEAT	2020-09-30	45.19	46.06	44.93	45.58	319699	{}	\N	2020-10-12 14:21:46.123489
29260	BEAT	2020-10-01	45.97	46.03	44.78	44.96	390567	{}	\N	2020-10-12 14:21:46.128151
29261	BEAT	2020-10-02	44.22	45	43.64	44.74	318247	{}	\N	2020-10-12 14:21:46.134228
29262	BEAT	2020-10-05	45.09	45.84	44.65	44.77	211226	{}	\N	2020-10-12 14:21:46.139156
29263	BEAT	2020-10-06	45.26	45.82	43.8917	44.79	187571	{}	\N	2020-10-12 14:21:46.143392
29264	BEAT	2020-10-07	44.97	45.81	44.9071	45.46	255485	{}	\N	2020-10-12 14:21:46.148254
29265	BEAT	2020-10-08	45.99	45.99	45.05	45.1	229880	{}	\N	2020-10-12 14:21:46.154858
29266	BEAT	2020-10-09	45.65	45.69	44.765	45.13	207531	{}	\N	2020-10-12 14:21:46.15915
29267	BEAT	2020-10-12	45.23	45.23	44.65	44.85	28133	{}	\N	2020-10-12 14:21:46.163561
29268	BFAM	2020-09-28	146.45	149.46	146.19	148.53	360959	{}	\N	2020-10-12 14:21:47.085907
29269	BFAM	2020-09-29	148.55	149.88	145.54	149.75	1067009	{}	\N	2020-10-12 14:21:47.092272
29270	BFAM	2020-09-30	149.2	152.56	149.2	152.04	1283933	{}	\N	2020-10-12 14:21:47.097287
29271	BFAM	2020-10-01	153.51	155	149.93	155	875805	{}	\N	2020-10-12 14:21:47.102024
29272	BFAM	2020-10-02	152.64	156.81	152.215	154.94	704085	{}	\N	2020-10-12 14:21:47.106539
29273	BFAM	2020-10-05	156.07	157.99	155.14	157.99	862982	{}	\N	2020-10-12 14:21:47.110818
29274	BFAM	2020-10-06	157.31	157.46	149.415	149.92	1324891	{}	\N	2020-10-12 14:21:47.115438
29275	BFAM	2020-10-07	151.49	153.47	151.08	152.59	631828	{}	\N	2020-10-12 14:21:47.120328
29276	BFAM	2020-10-08	153.46	155.67	153.225	153.79	504970	{}	\N	2020-10-12 14:21:47.12475
29277	BFAM	2020-10-09	154.03	155.9084	153.545	154.6	518198	{}	\N	2020-10-12 14:21:47.128884
29278	BFAM	2020-10-12	155.46	155.69	154.95	155.07	53672	{}	\N	2020-10-12 14:21:47.132859
29279	BGB	2020-09-28	12	12.15	11.99	12.13	319655	{}	\N	2020-10-12 14:21:48.091756
29280	BGB	2020-09-29	12.13	12.18	12.06	12.13	562395	{}	\N	2020-10-12 14:21:48.099042
29281	BGB	2020-09-30	12.16	12.41	12.07	12.22	2170461	{}	\N	2020-10-12 14:21:48.104926
29282	BGB	2020-10-01	12.25	12.27	12.1784	12.19	382788	{}	\N	2020-10-12 14:21:48.110596
29283	BGB	2020-10-02	12.1	12.225	12.1	12.22	273111	{}	\N	2020-10-12 14:21:48.115138
29284	BGB	2020-10-05	12.23	12.31	12.12	12.29	341197	{}	\N	2020-10-12 14:21:48.120096
29285	BGB	2020-10-06	12.31	12.33	12.24	12.26	186760	{}	\N	2020-10-12 14:21:48.12462
29286	BGB	2020-10-07	12.28	12.33	12.18	12.22	517385	{}	\N	2020-10-12 14:21:48.128758
29287	BGB	2020-10-08	12.28	12.34	12.25	12.29	230618	{}	\N	2020-10-12 14:21:48.133438
29288	BGB	2020-10-09	12.29	12.33	12.29	12.31	109225	{}	\N	2020-10-12 14:21:48.138374
29289	BGB	2020-10-12	12.34	12.36	12.3	12.316	51781	{}	\N	2020-10-12 14:21:48.142961
29290	BGS	2020-09-28	28.17	28.6	28.02	28.44	765399	{}	\N	2020-10-12 14:21:49.282112
29291	BGS	2020-09-29	28.06	28.08	27.42	27.81	1248755	{}	\N	2020-10-12 14:21:49.291958
29292	BGS	2020-09-30	27.57	27.96	27.4	27.77	1447301	{}	\N	2020-10-12 14:21:49.299285
29293	BGS	2020-10-01	27.8	27.95	26.74	27.81	2061147	{}	\N	2020-10-12 14:21:49.305064
29294	BGS	2020-10-02	27.52	28.33	27.5	28.19	1151060	{}	\N	2020-10-12 14:21:49.309837
29295	BGS	2020-10-05	28.22	28.66	27.82	28.33	1134640	{}	\N	2020-10-12 14:21:49.314225
29296	BGS	2020-10-06	28.33	28.7034	28.1	28.2	1280875	{}	\N	2020-10-12 14:21:49.320474
29297	BGS	2020-10-07	28.32	28.59	28.03	28.48	1347397	{}	\N	2020-10-12 14:21:49.330726
29298	BGS	2020-10-08	28.53	28.97	28.36	28.51	893743	{}	\N	2020-10-12 14:21:49.336231
29299	BGS	2020-10-09	28.75	29.12	28.7	28.73	1077663	{}	\N	2020-10-12 14:21:49.341168
29300	BGS	2020-10-12	28.73	29.0438	28.37	28.973	302171	{}	\N	2020-10-12 14:21:49.345176
29301	BHP	2020-09-28	53.12	53.175	52.575	52.74	1929039	{}	\N	2020-10-12 14:21:50.583185
29302	BHP	2020-09-29	52.08	52.35	51.43	51.6	4751537	{}	\N	2020-10-12 14:21:50.591602
29303	BHP	2020-09-30	52.06	52.06	51.43	51.71	3847345	{}	\N	2020-10-12 14:21:50.598144
29304	BHP	2020-10-01	51.86	51.97	51.03	51.34	3428814	{}	\N	2020-10-12 14:21:50.603737
29305	BHP	2020-10-02	50.63	51.82	50.5	51.46	3902618	{}	\N	2020-10-12 14:21:50.609523
29306	BHP	2020-10-05	51.76	52.265	51.75	52.17	2890701	{}	\N	2020-10-12 14:21:50.614173
29307	BHP	2020-10-06	51.86	51.93	50.34	50.5	9129013	{}	\N	2020-10-12 14:21:50.619521
29308	BHP	2020-10-07	51.46	52.29	51.32	51.94	4834577	{}	\N	2020-10-12 14:21:50.624795
29309	BHP	2020-10-08	52.06	52.53	51.87	52.52	4434071	{}	\N	2020-10-12 14:21:50.631198
29310	BHP	2020-10-09	53.12	53.18	52.675	52.87	3231857	{}	\N	2020-10-12 14:21:50.637669
29311	BHP	2020-10-12	52.84	52.84	52.42	52.51	418662	{}	\N	2020-10-12 14:21:50.643885
29312	BIG	2020-09-28	47.22	49.07	47.11	47.76	1011579	{}	\N	2020-10-12 14:21:51.615146
29313	BIG	2020-09-29	51.04	51.9	46.76	47.07	13539322	{}	\N	2020-10-12 14:21:51.621192
29314	BIG	2020-09-30	47.3	47.5599	44.26	44.6	6282231	{}	\N	2020-10-12 14:21:51.626261
29315	BIG	2020-10-01	44.9	45.95	44.11	44.99	5152450	{}	\N	2020-10-12 14:21:51.630882
29316	BIG	2020-10-02	43.75	46.33	43.02	45.56	4686638	{}	\N	2020-10-12 14:21:51.635682
29317	BIG	2020-10-05	45.94	46.98	45.46	46.82	2370929	{}	\N	2020-10-12 14:21:51.640366
29318	BIG	2020-10-06	46.94	49.44	46.2	46.43	3730165	{}	\N	2020-10-12 14:21:51.645353
29319	BIG	2020-10-07	47.28	49.32	47.28	49.3	2973158	{}	\N	2020-10-12 14:21:51.650961
29320	BIG	2020-10-08	49.81	50.98	49.64	50.79	2593169	{}	\N	2020-10-12 14:21:51.655987
29321	BIG	2020-10-09	52	52.84	50.4	50.42	3631317	{}	\N	2020-10-12 14:21:51.661064
29322	BIG	2020-10-12	51.04	52	50.8	51.91	531403	{}	\N	2020-10-12 14:21:51.665919
29323	BKD	2020-09-28	2.51	2.575	2.48	2.52	1571760	{}	\N	2020-10-12 14:21:52.574649
29324	BKD	2020-09-29	2.53	2.58	2.42	2.55	3460014	{}	\N	2020-10-12 14:21:52.584165
29325	BKD	2020-09-30	2.52	2.65	2.52	2.54	2933504	{}	\N	2020-10-12 14:21:52.589792
29326	BKD	2020-10-01	2.57	2.63	2.51	2.55	3470082	{}	\N	2020-10-12 14:21:52.595499
29327	BKD	2020-10-02	2.48	2.69	2.45	2.6	4977356	{}	\N	2020-10-12 14:21:52.600791
29328	BKD	2020-10-05	2.62	2.715	2.62	2.69	2242937	{}	\N	2020-10-12 14:21:52.604798
29329	BKD	2020-10-06	2.7	2.83	2.685	2.69	3405857	{}	\N	2020-10-12 14:21:52.609849
29330	BKD	2020-10-07	2.71	2.82	2.6799	2.78	2682767	{}	\N	2020-10-12 14:21:52.614226
29331	BKD	2020-10-08	2.8	2.89	2.77	2.86	3249580	{}	\N	2020-10-12 14:21:52.618616
29332	BKD	2020-10-09	2.88	2.92	2.8	2.8	1950502	{}	\N	2020-10-12 14:21:52.622846
29333	BKD	2020-10-12	2.79	2.81	2.745	2.795	227217	{}	\N	2020-10-12 14:21:52.627735
29334	BKN	2020-09-28	15.63	15.77	15.6268	15.76	19235	{}	\N	2020-10-12 14:21:53.648744
29335	BKN	2020-09-29	15.84	15.84	15.68	15.708	49820	{}	\N	2020-10-12 14:21:53.654925
29336	BKN	2020-09-30	15.65	15.71	15.62	15.63	70118	{}	\N	2020-10-12 14:21:53.662123
29337	BKN	2020-10-01	15.75	15.77	15.61	15.69	73246	{}	\N	2020-10-12 14:21:53.66774
29338	BKN	2020-10-02	15.58	15.8	15.58	15.8	91882	{}	\N	2020-10-12 14:21:53.672146
29339	BKN	2020-10-05	15.8	15.93	15.8	15.93	99404	{}	\N	2020-10-12 14:21:53.677286
29340	BKN	2020-10-06	15.88	16.09	15.85	16.03	64857	{}	\N	2020-10-12 14:21:53.682888
29341	BKN	2020-10-07	16.18	16.3	16.1	16.16	79124	{}	\N	2020-10-12 14:21:53.687565
29342	BKN	2020-10-08	16.26	16.35	16.12	16.29	35492	{}	\N	2020-10-12 14:21:53.693349
29343	BKN	2020-10-09	16.27	16.65	16.27	16.46	149058	{}	\N	2020-10-12 14:21:53.699738
29344	BKN	2020-10-12	16.51	16.51	16.445	16.445	4596	{}	\N	2020-10-12 14:21:53.704576
29345	BLDP	2020-09-28	14.84	15.68	14.7	14.915	5559724	{}	\N	2020-10-12 14:21:54.678036
29346	BLDP	2020-09-29	15.2099	15.49	14.635	15.1	5996788	{}	\N	2020-10-12 14:21:54.687551
29347	BLDP	2020-09-30	15.19	15.35	14.735	15.1	3605205	{}	\N	2020-10-12 14:21:54.694091
29348	BLDP	2020-10-01	15.3	16.47	15.26	16.46	5591247	{}	\N	2020-10-12 14:21:54.699957
29349	BLDP	2020-10-02	15.59	16.675	15.58	16.03	3703374	{}	\N	2020-10-12 14:21:54.704789
29350	BLDP	2020-10-05	16.52	17.58	16.35	17.54	5069630	{}	\N	2020-10-12 14:21:54.708883
29351	BLDP	2020-10-06	17.63	17.9	16.96	17.39	4308755	{}	\N	2020-10-12 14:21:54.713502
29352	BLDP	2020-10-07	17.76	18.695	17.71	18.36	4832479	{}	\N	2020-10-12 14:21:54.717729
29353	BLDP	2020-10-08	18.9	19.178	17.28	17.65	4525460	{}	\N	2020-10-12 14:21:54.721643
29354	BLDP	2020-10-09	17.75	19.15	17.75	18.82	4318251	{}	\N	2020-10-12 14:21:54.726
29355	BLDP	2020-10-12	19.2147	19.2147	18.07	18.7106	1431738	{}	\N	2020-10-12 14:21:54.730421
29356	BLMN	2020-09-28	15.57	15.7936	15.27	15.66	2599319	{}	\N	2020-10-12 14:21:55.662377
29357	BLMN	2020-09-29	15.54	15.6	14.56	15.125	3303982	{}	\N	2020-10-12 14:21:55.67242
29358	BLMN	2020-09-30	15.27	15.65	15.16	15.27	2073186	{}	\N	2020-10-12 14:21:55.678906
29359	BLMN	2020-10-01	15.4	15.82	15.24	15.79	1497841	{}	\N	2020-10-12 14:21:55.684335
29360	BLMN	2020-10-02	15.13	16.25	15	16.14	1846595	{}	\N	2020-10-12 14:21:55.689327
29361	BLMN	2020-10-05	16.3	16.72	15.99	16.4	1598953	{}	\N	2020-10-12 14:21:55.694268
29362	BLMN	2020-10-06	16.53	17.0289	15.795	16.02	2875833	{}	\N	2020-10-12 14:21:55.698465
29363	BLMN	2020-10-07	16.3	16.37	15.96	16.22	1598507	{}	\N	2020-10-12 14:21:55.702583
29364	BLMN	2020-10-08	16.45	16.9	16.29	16.875	2231564	{}	\N	2020-10-12 14:21:55.706414
29365	BLMN	2020-10-09	16.99	17.66	16.98	17.33	2504492	{}	\N	2020-10-12 14:21:55.710451
29366	BLMN	2020-10-12	17.64	17.855	17.42	17.64	612613	{}	\N	2020-10-12 14:21:55.714731
29367	BMRN	2020-09-28	78.51	79.21	77.72	77.99	1409919	{}	\N	2020-10-12 14:21:57.251408
29368	BMRN	2020-09-29	78.12	78.37	75.94	76	1310081	{}	\N	2020-10-12 14:21:57.265506
29369	BMRN	2020-09-30	76.26	77.555	75.4301	76.08	1509538	{}	\N	2020-10-12 14:21:57.272205
29370	BMRN	2020-10-01	77.02	77.24	75.33	75.82	1071085	{}	\N	2020-10-12 14:21:57.27741
29371	BMRN	2020-10-02	75.33	76.04	73.8	74.98	2422938	{}	\N	2020-10-12 14:21:57.282302
29372	BMRN	2020-10-05	75.5457	77.62	75.17	77	1488770	{}	\N	2020-10-12 14:21:57.287036
29373	BMRN	2020-10-06	76.68	77.57	75.5	75.55	1062524	{}	\N	2020-10-12 14:21:57.291804
29374	BMRN	2020-10-07	75.96	77.99	75.48	77.71	1091248	{}	\N	2020-10-12 14:21:57.296532
29375	BMRN	2020-10-08	78	79.86	77.72	79.61	1219032	{}	\N	2020-10-12 14:21:57.302881
29376	BMRN	2020-10-09	79.79	80.19	78.78	78.85	1133082	{}	\N	2020-10-12 14:21:57.307523
29377	BMRN	2020-10-12	80.26	80.26	77.94	78.975	137102	{}	\N	2020-10-12 14:21:57.312198
29378	BFZ	2020-09-28	13.18	13.24	13.17	13.17	60164	{}	\N	2020-10-12 14:21:58.663288
29379	BFZ	2020-09-29	13.2	13.26	13.2	13.22	44700	{}	\N	2020-10-12 14:21:58.671089
29380	BFZ	2020-09-30	13.21	13.24	13.1801	13.19	90892	{}	\N	2020-10-12 14:21:58.677428
29381	BFZ	2020-10-01	13.23	13.25	13.1815	13.24	91192	{}	\N	2020-10-12 14:21:58.682325
29382	BFZ	2020-10-02	13.17	13.23	13.04	13.12	225186	{}	\N	2020-10-12 14:21:58.687845
29383	BFZ	2020-10-05	13.03	13.15	13.03	13.06	207878	{}	\N	2020-10-12 14:21:58.692353
29384	BFZ	2020-10-06	13.06	13.09	13.04	13.045	81507	{}	\N	2020-10-12 14:21:58.696818
29385	BFZ	2020-10-07	13.11	13.13	13.06	13.07	101258	{}	\N	2020-10-12 14:21:58.701206
29386	BFZ	2020-10-08	13.11	13.12	13.08	13.09	41756	{}	\N	2020-10-12 14:21:58.705608
29387	BFZ	2020-10-09	13.12	13.16	13.1	13.1	104778	{}	\N	2020-10-12 14:21:58.710255
29388	BFZ	2020-10-12	13.16	13.16	13.16	13.16	5492	{}	\N	2020-10-12 14:21:58.714673
29389	BGNE	2020-09-28	282.55	283	277.51	280.65	227264	{}	\N	2020-10-12 14:21:59.844639
29390	BGNE	2020-09-29	276.57	284.455	272.3	281.46	231797	{}	\N	2020-10-12 14:21:59.852951
29391	BGNE	2020-09-30	280.4	287.74	280.4	286.44	176096	{}	\N	2020-10-12 14:21:59.858937
29392	BGNE	2020-10-01	287.52	287.6	282.1207	285.2	247117	{}	\N	2020-10-12 14:21:59.865024
29393	BGNE	2020-10-02	282.69	286.61	281.47	283.28	130118	{}	\N	2020-10-12 14:21:59.869979
29394	BGNE	2020-10-05	288	313.125	287.1124	306.04	462941	{}	\N	2020-10-12 14:21:59.874486
29395	BGNE	2020-10-06	306	307.54	295.48	295.87	211533	{}	\N	2020-10-12 14:21:59.878571
29396	BGNE	2020-10-07	298.74	299.25	286.29	297.55	488630	{}	\N	2020-10-12 14:21:59.882667
29397	BGNE	2020-10-08	300.38	305.59	295.22	301.49	127813	{}	\N	2020-10-12 14:21:59.886663
29398	BGNE	2020-10-09	304.95	313.69	302.62	312.66	198439	{}	\N	2020-10-12 14:21:59.890762
29399	BGNE	2020-10-12	313.13	318.29	311.13	317.35	28028	{}	\N	2020-10-12 14:21:59.895037
29400	BHB	2020-09-28	19.2151	20.16	19.1	19.9	37792	{}	\N	2020-10-12 14:22:00.854352
29401	BHB	2020-09-29	19.78	20.58	19.57	20.48	48881	{}	\N	2020-10-12 14:22:00.863171
29402	BHB	2020-09-30	20.38	20.67	20.38	20.55	41694	{}	\N	2020-10-12 14:22:00.86937
29403	BHB	2020-10-01	20.7663	20.94	20.365	20.67	35843	{}	\N	2020-10-12 14:22:00.874993
29404	BHB	2020-10-02	20.31	21.3197	20.31	20.97	22940	{}	\N	2020-10-12 14:22:00.880319
29405	BHB	2020-10-05	21	21.57	21	21.47	27593	{}	\N	2020-10-12 14:22:00.885551
29406	BHB	2020-10-06	21.78	22.43	21	21.88	55126	{}	\N	2020-10-12 14:22:00.889326
29407	BHB	2020-10-07	22.205	22.92	21.99	22.72	50466	{}	\N	2020-10-12 14:22:00.893022
29408	BHB	2020-10-08	22.99	23.22	22.0101	23.09	39221	{}	\N	2020-10-12 14:22:00.896931
29409	BHB	2020-10-09	22.97	23.54	22.5194	22.77	35198	{}	\N	2020-10-12 14:22:00.900842
29410	BHB	2020-10-12	22.65	22.8	22.375	22.6596	5074	{}	\N	2020-10-12 14:22:00.904837
29411	BIDU	2020-09-28	124.49	126.12	124.23	126.04	3894504	{}	\N	2020-10-12 14:22:02.082239
29412	BIDU	2020-09-29	125.01	125.8	122.47	122.58	3776053	{}	\N	2020-10-12 14:22:02.093676
29413	BIDU	2020-09-30	123.835	129	123.07	126.59	3648189	{}	\N	2020-10-12 14:22:02.100008
29414	BIDU	2020-10-01	126.45	127.285	125.52	126.45	1785847	{}	\N	2020-10-12 14:22:02.105507
29415	BIDU	2020-10-02	124.66	126.09	123.5	124.62	1884192	{}	\N	2020-10-12 14:22:02.109989
29416	BIDU	2020-10-05	124.83	126.12	124.1	124.7	1917762	{}	\N	2020-10-12 14:22:02.114769
29417	BIDU	2020-10-06	124.98	127.075	124.18	124.83	1495652	{}	\N	2020-10-12 14:22:02.119452
29418	BIDU	2020-10-07	125.8	126.99	125.32	126.35	1698005	{}	\N	2020-10-12 14:22:02.123629
29419	BIDU	2020-10-08	128.7	128.7	126.3901	126.75	1308196	{}	\N	2020-10-12 14:22:02.127911
29420	BIDU	2020-10-09	126.79	127.64	126.11	126.48	1182468	{}	\N	2020-10-12 14:22:02.131947
29421	BIDU	2020-10-12	129.03	129.1	127.13	128	846724	{}	\N	2020-10-12 14:22:02.135979
29422	BIT	2020-09-28	15.64	15.72	15.64	15.69	103922	{}	\N	2020-10-12 14:22:03.328986
29423	BIT	2020-09-29	15.7	15.77	15.6921	15.75	340592	{}	\N	2020-10-12 14:22:03.336434
29424	BIT	2020-09-30	15.77	15.84	15.745	15.78	508183	{}	\N	2020-10-12 14:22:03.341978
29425	BIT	2020-10-01	15.8	15.87	15.8	15.81	266489	{}	\N	2020-10-12 14:22:03.349206
29426	BIT	2020-10-02	15.8	15.85	15.74	15.75	399311	{}	\N	2020-10-12 14:22:03.35387
29427	BIT	2020-10-05	15.77	15.9	15.77	15.86	244642	{}	\N	2020-10-12 14:22:03.357985
29428	BIT	2020-10-06	15.9	16.08	15.875	16.02	166384	{}	\N	2020-10-12 14:22:03.361869
29429	BIT	2020-10-07	16.1	16.23	16.04	16.04	252539	{}	\N	2020-10-12 14:22:03.365717
29430	BIT	2020-10-08	16.13	16.21	16.08	16.11	150052	{}	\N	2020-10-12 14:22:03.369716
29431	BIT	2020-10-09	16.15	16.22	16.1	16.15	212620	{}	\N	2020-10-12 14:22:03.373698
29432	BIT	2020-10-12	16.13	16.18	16.13	16.16	40173	{}	\N	2020-10-12 14:22:03.377694
29433	BKEPP	2020-09-28	5.9001	5.92	5.81	5.84	16087	{}	\N	2020-10-12 14:22:04.767428
29434	BKEPP	2020-09-29	5.8908	5.92	5.85	5.92	4449	{}	\N	2020-10-12 14:22:04.776096
29435	BKEPP	2020-09-30	5.92	5.92	5.8702	5.8812	6414	{}	\N	2020-10-12 14:22:04.781673
29436	BKEPP	2020-10-01	5.85	5.85	5.75	5.84	4454	{}	\N	2020-10-12 14:22:04.786846
29437	BKEPP	2020-10-02	5.9199	5.9199	5.8	5.8404	8432	{}	\N	2020-10-12 14:22:04.792154
29438	BKEPP	2020-10-05	5.87	5.89	5.87	5.89	8924	{}	\N	2020-10-12 14:22:04.79692
29439	BKEPP	2020-10-06	5.89	5.9	5.77	5.84	12751	{}	\N	2020-10-12 14:22:04.801125
29440	BKEPP	2020-10-07	5.88	5.9	5.84	5.8535	13795	{}	\N	2020-10-12 14:22:04.805209
29441	BKEPP	2020-10-08	5.81	5.91	5.785	5.88	42644	{}	\N	2020-10-12 14:22:04.809215
29442	BKEPP	2020-10-09	5.88	5.886	5.85	5.886	7471	{}	\N	2020-10-12 14:22:04.813141
29443	BKEPP	2020-10-12	5.9	5.9002	5.9	5.9002	2161	{}	\N	2020-10-12 14:22:04.817125
29444	BKU	2020-09-28	21.87	22.21	21.8	21.87	942310	{}	\N	2020-10-12 14:22:05.93071
29445	BKU	2020-09-29	21.72	21.74	21.08	21.63	1090882	{}	\N	2020-10-12 14:22:05.942246
29446	BKU	2020-09-30	21.71	22.28	21.56	21.91	1261742	{}	\N	2020-10-12 14:22:05.949682
29447	BKU	2020-10-01	22.06	22.45	21.66	22.16	1431555	{}	\N	2020-10-12 14:22:05.956162
29448	BKU	2020-10-02	21.64	23.1	21.4857	22.98	1151501	{}	\N	2020-10-12 14:22:05.961509
29449	BKU	2020-10-05	23.34	24.06	23.14	23.95	1858701	{}	\N	2020-10-12 14:22:05.966557
29450	BKU	2020-10-06	24.33	25.07	23.72	23.75	1831597	{}	\N	2020-10-12 14:22:05.970713
29451	BKU	2020-10-07	24.15	24.81	24.07	24.34	1615907	{}	\N	2020-10-12 14:22:05.974767
29452	BKU	2020-10-08	24.78	24.93	24.36	24.77	868958	{}	\N	2020-10-12 14:22:05.978717
29453	BKU	2020-10-09	25.11	25.11	24.225	24.35	1011621	{}	\N	2020-10-12 14:22:05.982793
29454	BKU	2020-10-12	24.22	24.52	24.175	24.32	159666	{}	\N	2020-10-12 14:22:05.987103
29455	BLOK	2020-09-28	24.49	24.555	24.37	24.5	64909	{}	\N	2020-10-12 14:22:06.95005
29456	BLOK	2020-09-29	24.48	24.53	24.3601	24.39	28912	{}	\N	2020-10-12 14:22:06.963624
29457	BLOK	2020-09-30	24.43	24.68	24.43	24.51	30211	{}	\N	2020-10-12 14:22:06.972144
29458	BLOK	2020-10-01	24.68	24.76	24.63	24.75	29995	{}	\N	2020-10-12 14:22:06.978749
29459	BLOK	2020-10-02	24.41	24.66	24.39	24.45	30500	{}	\N	2020-10-12 14:22:06.983917
29460	BLOK	2020-10-05	24.66	24.934	24.66	24.934	47627	{}	\N	2020-10-12 14:22:06.988643
29461	BLOK	2020-10-06	24.95	25.05	24.6074	24.65	25250	{}	\N	2020-10-12 14:22:06.994203
29462	BLOK	2020-10-07	24.94	25.06	24.85	25.0393	44180	{}	\N	2020-10-12 14:22:06.999756
29463	BLOK	2020-10-08	25.26	25.439	25.225	25.43	25544	{}	\N	2020-10-12 14:22:07.004576
29464	BLOK	2020-10-09	25.57	25.645	25.51	25.6	65378	{}	\N	2020-10-12 14:22:07.009261
29465	BLOK	2020-10-12	25.79	25.87	25.74	25.87	17772	{}	\N	2020-10-12 14:22:07.013805
29466	BME	2020-09-28	42.72	42.96	42.535	42.78	42308	{}	\N	2020-10-12 14:22:08.052174
29467	BME	2020-09-29	42.95	42.96	42.57	42.88	67960	{}	\N	2020-10-12 14:22:08.064033
29468	BME	2020-09-30	43.4	43.4	42.7801	42.86	93590	{}	\N	2020-10-12 14:22:08.071247
29469	BME	2020-10-01	43.39	43.39	42.69	42.94	53617	{}	\N	2020-10-12 14:22:08.077327
29470	BME	2020-10-02	42.27	42.79	42.09	42.42	79604	{}	\N	2020-10-12 14:22:08.08299
29471	BME	2020-10-05	42.43	43.65	42.43	43.52	64746	{}	\N	2020-10-12 14:22:08.087861
29472	BME	2020-10-06	43.68	43.9	43.4	43.51	66828	{}	\N	2020-10-12 14:22:08.092733
29473	BME	2020-10-07	43.67	44.3	43.5	44.14	123874	{}	\N	2020-10-12 14:22:08.097417
29474	BME	2020-10-08	44.34	44.34	44.0012	44.07	44147	{}	\N	2020-10-12 14:22:08.101988
29475	BME	2020-10-09	44.25	44.66	44.2032	44.26	61804	{}	\N	2020-10-12 14:22:08.106341
29476	BME	2020-10-12	44.44	44.49	44.3367	44.35	12084	{}	\N	2020-10-12 14:22:08.110705
29477	BMY	2020-09-28	59.37	60.21	58.96	59.71	9337052	{}	\N	2020-10-12 14:22:09.018057
29478	BMY	2020-09-29	59.84	60.94	59.68	59.9	16129082	{}	\N	2020-10-12 14:22:09.028892
29479	BMY	2020-09-30	60.14	60.64	59.79	60.29	20737957	{}	\N	2020-10-12 14:22:09.035808
29480	BMY	2020-10-01	60.38	60.57	59.625	59.82	16183536	{}	\N	2020-10-12 14:22:09.041821
29481	BMY	2020-10-02	59.39	60.01	58.68	58.72	23329725	{}	\N	2020-10-12 14:22:09.04715
29482	BMY	2020-10-05	58.55	59.28	57.95	59.2	21448172	{}	\N	2020-10-12 14:22:09.052289
29483	BMY	2020-10-06	59.34	59.46	57.61	57.74	25914820	{}	\N	2020-10-12 14:22:09.056719
29484	BMY	2020-10-07	58.15	60.98	57.96	60.42	27086613	{}	\N	2020-10-12 14:22:09.061249
29485	BMY	2020-10-08	60.85	62.09	60.46	61.62	23328571	{}	\N	2020-10-12 14:22:09.065452
29486	BMY	2020-10-09	61.8	62.295	61.605	61.82	18672191	{}	\N	2020-10-12 14:22:09.069707
29487	BMY	2020-10-12	61.98	62.25	61.48	61.84	4928148	{}	\N	2020-10-12 14:22:09.073922
29488	BNY	2020-09-28	13.56	13.5818	13.51	13.5818	1926	{}	\N	2020-10-12 14:22:10.029156
29489	BNY	2020-09-29	13.65	13.65	13.52	13.561	21540	{}	\N	2020-10-12 14:22:10.038787
29490	BNY	2020-09-30	13.56	13.5661	13.48	13.56	57868	{}	\N	2020-10-12 14:22:10.045279
29491	BNY	2020-10-01	13.6	13.75	13.5445	13.7059	31394	{}	\N	2020-10-12 14:22:10.050452
29492	BNY	2020-10-02	13.6	13.71	13.57	13.68	86936	{}	\N	2020-10-12 14:22:10.055524
29493	BNY	2020-10-05	13.75	13.75	13.59	13.65	21136	{}	\N	2020-10-12 14:22:10.060746
29494	BNY	2020-10-06	13.7	13.7259	13.59	13.64	40006	{}	\N	2020-10-12 14:22:10.065209
29495	BNY	2020-10-07	13.6	13.73	13.59	13.63	64538	{}	\N	2020-10-12 14:22:10.069328
29496	BNY	2020-10-08	13.65	13.65	13.48	13.65	50134	{}	\N	2020-10-12 14:22:10.073411
29497	BNY	2020-10-09	13.65	13.67	13.58	13.615	64782	{}	\N	2020-10-12 14:22:10.077663
29498	BNY	2020-10-12	13.7	13.7	13.62	13.7	2686	{}	\N	2020-10-12 14:22:10.08196
29499	BGH	2020-09-28	13.25	13.4	13.25	13.4	81377	{}	\N	2020-10-12 14:22:12.802546
29500	BGH	2020-09-29	13.35	13.39	13.3246	13.38	187322	{}	\N	2020-10-12 14:22:12.811341
29501	BGH	2020-09-30	13.35	13.41	13.32	13.32	198706	{}	\N	2020-10-12 14:22:12.817725
29502	BGH	2020-10-01	13.44	13.44	13.315	13.39	117749	{}	\N	2020-10-12 14:22:12.822975
29503	BGH	2020-10-02	13.32	13.42	13.2901	13.41	88452	{}	\N	2020-10-12 14:22:12.827589
29504	BGH	2020-10-05	13.41	13.4324	13.34	13.43	113538	{}	\N	2020-10-12 14:22:12.831727
29505	BGH	2020-10-06	13.43	13.49	13.4	13.45	134337	{}	\N	2020-10-12 14:22:12.835746
29506	BGH	2020-10-07	13.55	13.57	13.47	13.5	138865	{}	\N	2020-10-12 14:22:12.839542
29507	BGH	2020-10-08	13.57	13.612	13.53	13.53	117679	{}	\N	2020-10-12 14:22:12.843116
29508	BGH	2020-10-09	13.55	13.6458	13.55	13.59	80092	{}	\N	2020-10-12 14:22:12.847294
29509	BGH	2020-10-12	13.62	13.64	13.59	13.64	23060	{}	\N	2020-10-12 14:22:12.85136
29510	BGX	2020-09-28	12.8	12.86	12.77	12.84	102741	{}	\N	2020-10-12 14:22:14.175152
29511	BGX	2020-09-29	12.91	12.91	12.82	12.84	444302	{}	\N	2020-10-12 14:22:14.184751
29512	BGX	2020-09-30	12.87	12.92	12.81	12.86	537845	{}	\N	2020-10-12 14:22:14.191053
29513	BGX	2020-10-01	12.92	12.96	12.87	12.92	215300	{}	\N	2020-10-12 14:22:14.196573
29514	BGX	2020-10-02	12.9	12.95	12.835	12.95	167272	{}	\N	2020-10-12 14:22:14.201373
29515	BGX	2020-10-05	12.95	13.08	12.9115	13.08	173409	{}	\N	2020-10-12 14:22:14.205844
29516	BGX	2020-10-06	13.12	13.12	13.03	13.09	85599	{}	\N	2020-10-12 14:22:14.210155
29517	BGX	2020-10-07	13.13	13.13	13.01	13.05	97458	{}	\N	2020-10-12 14:22:14.214287
29518	BGX	2020-10-08	13.1	13.12	13.05	13.12	45574	{}	\N	2020-10-12 14:22:14.218388
29519	BGX	2020-10-09	13.16	13.16	13.09	13.15	86858	{}	\N	2020-10-12 14:22:14.222437
29520	BGX	2020-10-12	13.15	13.18	13.08	13.18	26858	{}	\N	2020-10-12 14:22:14.226569
29521	BICK	2020-09-28	28.95	28.95	28.77	28.86	2419	{}	\N	2020-10-12 14:22:16.870322
29522	BICK	2020-09-29	28.8	28.8	28.7735	28.7843	797	{}	\N	2020-10-12 14:22:16.876723
29523	BICK	2020-09-30	29.19	29.19	29.09	29.09	1899	{}	\N	2020-10-12 14:22:16.882825
29524	BICK	2020-10-01	29.22	29.4739	29.21	29.4739	852	{}	\N	2020-10-12 14:22:16.888242
29525	BICK	2020-10-02	29.448	29.448	29.12	29.12	1740	{}	\N	2020-10-12 14:22:16.892771
29526	BICK	2020-10-05	29.53	29.65	29.53	29.65	799	{}	\N	2020-10-12 14:22:16.898168
29527	BICK	2020-10-06	30.02	30.05	29.92	29.92	6186	{}	\N	2020-10-12 14:22:16.906062
29528	BICK	2020-10-07	29.82	30.02	29.82	30.02	628	{}	\N	2020-10-12 14:22:16.911281
29529	BICK	2020-10-08	30.1	30.3775	30.1	30.3775	1397	{}	\N	2020-10-12 14:22:16.916898
29530	BICK	2020-10-09	30.57	30.6813	30.56	30.56	5836	{}	\N	2020-10-12 14:22:16.922091
29531	BIS	2020-09-28	31.36	31.97	31.35	31.47	23109	{}	\N	2020-10-12 14:22:18.412272
29532	BIS	2020-09-29	31.395	31.73	31.05	31.39	16670	{}	\N	2020-10-12 14:22:18.421651
29533	BIS	2020-09-30	31.14	31.43	30.48	30.9742	37336	{}	\N	2020-10-12 14:22:18.427121
29534	BIS	2020-10-01	30.66	31.03	30.41	30.48	38942	{}	\N	2020-10-12 14:22:18.432277
29535	BIS	2020-10-02	31.3	32.0066	30.54	31.8	29812	{}	\N	2020-10-12 14:22:18.437039
29536	BIS	2020-10-05	30.62	30.67	28.96	29.03	86286	{}	\N	2020-10-12 14:22:18.441824
29537	BIS	2020-10-06	28.93	29.75	28.68	29.67	40470	{}	\N	2020-10-12 14:22:18.446061
29538	BIS	2020-10-07	29.35	29.35	28.2	28.4	22339	{}	\N	2020-10-12 14:22:18.449965
29539	BIS	2020-10-08	28	28.49	28	28.235	23180	{}	\N	2020-10-12 14:22:18.454163
29540	BIS	2020-10-09	27.86	28.27	27.73	27.78	24996	{}	\N	2020-10-12 14:22:18.458321
29541	BIS	2020-10-12	27.28	27.95	27.27	27.55	10469	{}	\N	2020-10-12 14:22:18.46248
29542	BKE	2020-09-28	20.7465	21.33	20.57	20.79	488495	{}	\N	2020-10-12 14:22:19.737853
29543	BKE	2020-09-29	20.7	20.7	19.76	20.28	697781	{}	\N	2020-10-12 14:22:19.748808
29544	BKE	2020-09-30	20.53	20.99	20.205	20.39	1202903	{}	\N	2020-10-12 14:22:19.755311
29545	BKE	2020-10-01	20.3	21.32	20.3	21.22	1379916	{}	\N	2020-10-12 14:22:19.761187
29546	BKE	2020-10-02	20.25	21.66	20.2	21.47	914528	{}	\N	2020-10-12 14:22:19.765932
29547	BKE	2020-10-05	21.75	23.57	21.69	23.2	1969348	{}	\N	2020-10-12 14:22:19.77058
29548	BKE	2020-10-06	23.24	23.9	22.75	22.87	1820994	{}	\N	2020-10-12 14:22:19.774986
29549	BKE	2020-10-07	23.41	24.56	23.41	23.8	1540565	{}	\N	2020-10-12 14:22:19.779399
29550	BKE	2020-10-08	24.28	24.38	23.45	23.81	821979	{}	\N	2020-10-12 14:22:19.783511
29551	BKE	2020-10-09	23.82	24.16	23.48	23.53	585296	{}	\N	2020-10-12 14:22:19.787628
29552	BKE	2020-10-12	23.5	23.849	23.4	23.65	107385	{}	\N	2020-10-12 14:22:19.791679
29553	BLDR	2020-09-28	31.44	32.38	31.32	32.295	1875907	{}	\N	2020-10-12 14:22:21.222591
29554	BLDR	2020-09-29	32.26	32.52	31.95	32	1313154	{}	\N	2020-10-12 14:22:21.2325
29555	BLDR	2020-09-30	32.23	33.21	32.15	32.6	1703510	{}	\N	2020-10-12 14:22:21.239143
29556	BLDR	2020-10-01	32.96	33.275	32.48	33.165	1922182	{}	\N	2020-10-12 14:22:21.244802
29557	BLDR	2020-10-02	32.2	33.88	32	33.69	2517953	{}	\N	2020-10-12 14:22:21.249527
29558	BLDR	2020-10-05	34.15	34.42	32.83	33.39	2345107	{}	\N	2020-10-12 14:22:21.254525
29559	BLDR	2020-10-06	33.4114	34.36	33.26	33.42	2544778	{}	\N	2020-10-12 14:22:21.258472
29560	BLDR	2020-10-07	34.19	34.38	33.46	33.84	1291620	{}	\N	2020-10-12 14:22:21.262534
29561	BLDR	2020-10-08	34.2	34.4	33.36	33.87	1090057	{}	\N	2020-10-12 14:22:21.267056
29562	BLDR	2020-10-09	34.33	34.49	33.48	34	1133166	{}	\N	2020-10-12 14:22:21.271003
29563	BLDR	2020-10-12	34.4	34.61	34.035	34.36	120982	{}	\N	2020-10-12 14:22:21.275341
29564	BLKB	2020-09-28	58.1	58.18	57.22	57.89	218886	{}	\N	2020-10-12 14:22:22.817121
29565	BLKB	2020-09-29	57.77	57.93	56.85	57.3	101626	{}	\N	2020-10-12 14:22:22.826338
29566	BLKB	2020-09-30	56.61	57.375	55.83	55.83	237367	{}	\N	2020-10-12 14:22:22.831801
29567	BLKB	2020-10-01	55.66	56.38	55.13	55.45	281274	{}	\N	2020-10-12 14:22:22.837248
29568	BLKB	2020-10-02	54.27	55.81	53.985	54.23	306838	{}	\N	2020-10-12 14:22:22.842227
29569	BLKB	2020-10-05	54.82	55.99	54.82	55.71	290689	{}	\N	2020-10-12 14:22:22.847209
29570	BLKB	2020-10-06	55.96	58.14	55.96	56.75	342836	{}	\N	2020-10-12 14:22:22.851845
29571	BLKB	2020-10-07	56.96	57.97	56.58	57.68	255181	{}	\N	2020-10-12 14:22:22.856087
29572	BLKB	2020-10-08	58.54	59.7389	57.69	59	204964	{}	\N	2020-10-12 14:22:22.860657
29573	BLKB	2020-10-09	59.57	60.11	59.17	59.93	170759	{}	\N	2020-10-12 14:22:22.864913
29574	BLKB	2020-10-12	60.42	60.42	59.485	59.485	22520	{}	\N	2020-10-12 14:22:22.869344
29575	BMA	2020-09-28	14.76	15.0779	14.28	14.33	287363	{}	\N	2020-10-12 14:22:24.518228
29576	BMA	2020-09-29	14.04	14.45	13.9101	14.1	592793	{}	\N	2020-10-12 14:22:24.528847
29577	BMA	2020-09-30	14.11	14.53	13.99	14.3	662294	{}	\N	2020-10-12 14:22:24.535133
29578	BMA	2020-10-01	14.21	14.67	14.21	14.62	544413	{}	\N	2020-10-12 14:22:24.540575
29579	BMA	2020-10-02	14.07	14.41	14	14.25	554046	{}	\N	2020-10-12 14:22:24.546355
29580	BMA	2020-10-05	14.35	14.4723	13.92	14.05	420447	{}	\N	2020-10-12 14:22:24.552203
29581	BMA	2020-10-06	14.14	14.7	13.6	13.74	663163	{}	\N	2020-10-12 14:22:24.556508
29582	BMA	2020-10-07	13.74	14.25	13.3827	13.88	466451	{}	\N	2020-10-12 14:22:24.561107
29583	BMA	2020-10-08	13.97	14.36	13.75	14.27	561764	{}	\N	2020-10-12 14:22:24.565429
29584	BMA	2020-10-09	14.35	14.47	13.8	14.1	400679	{}	\N	2020-10-12 14:22:24.569329
29585	BMA	2020-10-12	14.03	14.03	13.64	13.79	96913	{}	\N	2020-10-12 14:22:24.573458
29586	BMTC	2020-09-28	24.56	25.4999	24.56	25.3	61712	{}	\N	2020-10-12 14:22:27.185543
29587	BMTC	2020-09-29	25.21	25.25	24.58	24.98	31892	{}	\N	2020-10-12 14:22:27.196393
29588	BMTC	2020-09-30	25.39	25.52	24.74	24.87	53074	{}	\N	2020-10-12 14:22:27.203313
29589	BMTC	2020-10-01	24.9	25.04	24.44	24.95	60255	{}	\N	2020-10-12 14:22:27.208903
29590	BMTC	2020-10-02	24.54	25.88	24.54	25.79	42528	{}	\N	2020-10-12 14:22:27.214409
29591	BMTC	2020-10-05	25.96	26.91	25.96	26.84	65674	{}	\N	2020-10-12 14:22:27.218908
29592	BMTC	2020-10-06	27.25	27.8	25.7639	26.79	123137	{}	\N	2020-10-12 14:22:27.222911
29593	BMTC	2020-10-07	27.22	27.67	26.58	27.48	62586	{}	\N	2020-10-12 14:22:27.227011
29594	BMTC	2020-10-08	27.5	28.07	27.5	27.89	93406	{}	\N	2020-10-12 14:22:27.231152
29595	BMTC	2020-10-09	28.23	28.23	27.38	27.53	29975	{}	\N	2020-10-12 14:22:27.235359
29596	BMTC	2020-10-12	27.53	27.675	27.41	27.56	5078	{}	\N	2020-10-12 14:22:27.239335
29597	BNED	2020-09-28	2.41	2.56	2.41	2.55	232623	{}	\N	2020-10-12 14:22:28.121365
29598	BNED	2020-09-29	2.55	2.56	2.43	2.44	441400	{}	\N	2020-10-12 14:22:28.130915
29599	BNED	2020-09-30	2.37	2.63	2.37	2.58	545334	{}	\N	2020-10-12 14:22:28.137955
29600	BNED	2020-10-01	2.58	2.77	2.58	2.69	571892	{}	\N	2020-10-12 14:22:28.14437
29601	BNED	2020-10-02	2.59	2.8	2.55	2.78	622784	{}	\N	2020-10-12 14:22:28.149783
29602	BNED	2020-10-05	2.8	2.89	2.63	2.65	806538	{}	\N	2020-10-12 14:22:28.154331
29603	BNED	2020-10-06	2.74	2.74	2.53	2.56	566256	{}	\N	2020-10-12 14:22:28.158524
29604	BNED	2020-10-07	2.55	2.65	2.5139	2.6	574342	{}	\N	2020-10-12 14:22:28.16292
29605	BNED	2020-10-08	2.58	2.68	2.58	2.65	368468	{}	\N	2020-10-12 14:22:28.167204
29606	BNED	2020-10-09	2.69	2.73	2.64	2.64	314913	{}	\N	2020-10-12 14:22:28.171346
29607	BNED	2020-10-12	2.67	2.7	2.62	2.62	54088	{}	\N	2020-10-12 14:22:28.175616
29608	BOH	2020-09-28	50.48	51.5395	50.48	50.94	244256	{}	\N	2020-10-12 14:22:29.549345
29609	BOH	2020-09-29	50.55	50.6265	49.58	49.9	465773	{}	\N	2020-10-12 14:22:29.556266
29610	BOH	2020-09-30	50.09	50.725	49.92	50.52	561936	{}	\N	2020-10-12 14:22:29.561359
29611	BOH	2020-10-01	50.12	50.84	49.61	50.03	388206	{}	\N	2020-10-12 14:22:29.566459
29612	BOH	2020-10-02	49.25	51.3	49.25	50.96	422816	{}	\N	2020-10-12 14:22:29.572585
29613	BOH	2020-10-05	51.56	52.88	51.41	52.87	566761	{}	\N	2020-10-12 14:22:29.579332
29614	BOH	2020-10-06	53.24	55.11	53.08	53.18	585726	{}	\N	2020-10-12 14:22:29.585111
29615	BOH	2020-10-07	53.61	55.06	53.61	54.75	591141	{}	\N	2020-10-12 14:22:29.590715
29616	BOH	2020-10-08	54.89	56	54.61	55.67	483820	{}	\N	2020-10-12 14:22:29.595648
29617	BOH	2020-10-09	55.77	56.1221	55.07	55.3	539176	{}	\N	2020-10-12 14:22:29.601044
29618	BOH	2020-10-12	55.9	55.9	55.14	55.78	97885	{}	\N	2020-10-12 14:22:29.605643
29619	FFC	2020-09-28	20.6	20.84	20.545	20.74	121313	{}	\N	2020-10-12 14:22:30.708003
29620	FFC	2020-09-29	20.8	20.97	20.65	20.97	152938	{}	\N	2020-10-12 14:22:30.716671
29621	FFC	2020-09-30	21.13	21.2	20.96	20.96	249312	{}	\N	2020-10-12 14:22:30.722224
29622	FFC	2020-10-01	21.16	21.2	21.015	21.18	229932	{}	\N	2020-10-12 14:22:30.72734
29623	FFC	2020-10-02	20.99	21.1	20.93	21.06	138756	{}	\N	2020-10-12 14:22:30.732433
29624	FFC	2020-10-05	21.06	21.185	21.06	21.15	137336	{}	\N	2020-10-12 14:22:30.737272
29625	FFC	2020-10-06	21.1	21.1999	21.02	21.08	174487	{}	\N	2020-10-12 14:22:30.742677
29626	FFC	2020-10-07	21.1	21.28	21.078	21.265	101476	{}	\N	2020-10-12 14:22:30.747018
29627	FFC	2020-10-08	21.27	21.3306	21.1	21.21	225676	{}	\N	2020-10-12 14:22:30.75136
29628	FFC	2020-10-09	21.23	21.2974	21.12	21.175	130754	{}	\N	2020-10-12 14:22:30.755551
29629	FFC	2020-10-12	21.34	21.34	21.2	21.28	63096	{}	\N	2020-10-12 14:22:30.759619
29630	BHVN	2020-09-28	63.505	63.98	61.95	62.29	360668	{}	\N	2020-10-12 14:22:31.80623
29631	BHVN	2020-09-29	62.07	63.94	61.5	63.16	687169	{}	\N	2020-10-12 14:22:31.81477
29632	BHVN	2020-09-30	63.73	65.19	62.86	65.01	1064494	{}	\N	2020-10-12 14:22:31.821372
29633	BHVN	2020-10-01	65.53	66.5	64.495	65.88	855142	{}	\N	2020-10-12 14:22:31.826241
29634	BHVN	2020-10-02	64.2	66.135	64.05	65.07	704467	{}	\N	2020-10-12 14:22:31.830777
29635	BHVN	2020-10-05	65.73	69.23	65.73	68.84	793941	{}	\N	2020-10-12 14:22:31.835429
29636	BHVN	2020-10-06	69.31	71.24	68.87	69.34	993890	{}	\N	2020-10-12 14:22:31.840319
29637	BHVN	2020-10-07	70.18	74.47	70.18	73.95	1213487	{}	\N	2020-10-12 14:22:31.844175
29638	BHVN	2020-10-08	74.99	78	74.24	77.31	1386026	{}	\N	2020-10-12 14:22:31.848129
29639	BHVN	2020-10-09	77.18	78.45	76.25	77.98	1052743	{}	\N	2020-10-12 14:22:31.852158
29640	BHVN	2020-10-12	78.76	78.76	77.27	77.68	141209	{}	\N	2020-10-12 14:22:31.856293
29641	BIOC	2020-09-28	4.26	4.45	4.16	4.36	521584	{}	\N	2020-10-12 14:22:33.913127
29642	BIOC	2020-09-29	4.59	4.8799	4.42	4.53	1006037	{}	\N	2020-10-12 14:22:33.923307
29643	BIOC	2020-09-30	4.46	4.52	4.35	4.41	251484	{}	\N	2020-10-12 14:22:33.9289
29644	BIOC	2020-10-01	4.42	4.52	4.36	4.5	269042	{}	\N	2020-10-12 14:22:33.934638
29645	BIOC	2020-10-02	4.5	4.75	4.37	4.55	431547	{}	\N	2020-10-12 14:22:33.939379
29646	BIOC	2020-10-05	4.75	5.04	4.57	4.78	1170040	{}	\N	2020-10-12 14:22:33.943636
29647	BIOC	2020-10-06	4.88	5.13	4.78	4.99	685759	{}	\N	2020-10-12 14:22:33.948059
29648	BIOC	2020-10-07	5.0451	5.1	4.8	4.94	434521	{}	\N	2020-10-12 14:22:33.9522
29649	BIOC	2020-10-08	4.94	5.09	4.84	5.02	382709	{}	\N	2020-10-12 14:22:33.95614
29650	BIOC	2020-10-09	5	5.035	4.66	4.77	400096	{}	\N	2020-10-12 14:22:33.95988
29651	BIOC	2020-10-12	4.82	4.83	4.72	4.81	52707	{}	\N	2020-10-12 14:22:33.964031
29652	BJRI	2020-09-28	30.59	31.3	30.41	30.96	385445	{}	\N	2020-10-12 14:22:35.104715
29653	BJRI	2020-09-29	30.89	31.08	28.8495	29.49	260774	{}	\N	2020-10-12 14:22:35.114421
29654	BJRI	2020-09-30	29.77	30.53	29.2	29.44	257328	{}	\N	2020-10-12 14:22:35.120941
29655	BJRI	2020-10-01	29.48	30.26	29.4	30.25	171835	{}	\N	2020-10-12 14:22:35.126761
29656	BJRI	2020-10-02	28.66	30.63	28.6	30.52	276778	{}	\N	2020-10-12 14:22:35.131506
29657	BJRI	2020-10-05	31.02	31.68	29.53	30.02	305462	{}	\N	2020-10-12 14:22:35.136106
29658	BJRI	2020-10-06	30.75	31.63	30.05	30.1	336723	{}	\N	2020-10-12 14:22:35.140083
29659	BJRI	2020-10-07	30.46	31.12	30.04	30.94	294300	{}	\N	2020-10-12 14:22:35.143786
29660	BJRI	2020-10-08	31.34	32.51	30.48	32.35	306091	{}	\N	2020-10-12 14:22:35.147914
29661	BJRI	2020-10-09	32.76	33.21	32.22	33.04	246733	{}	\N	2020-10-12 14:22:35.152673
29662	BJRI	2020-10-12	33.12	33.69	32.71	33.41	39174	{}	\N	2020-10-12 14:22:35.156513
29663	BKI	2020-09-28	86.04	86.96	85.55	85.8	542721	{}	\N	2020-10-12 14:22:36.284524
29664	BKI	2020-09-29	86.92	87.46	86.565	86.96	891888	{}	\N	2020-10-12 14:22:36.293059
29665	BKI	2020-09-30	87.17	87.75	86.62	87.05	1324737	{}	\N	2020-10-12 14:22:36.299226
29666	BKI	2020-10-01	87.75	89.22	87.75	88.79	1044942	{}	\N	2020-10-12 14:22:36.304824
29667	BKI	2020-10-02	87.4	88.7599	87.06	87.64	914418	{}	\N	2020-10-12 14:22:36.309807
29668	BKI	2020-10-05	88.22	88.795	87.6	88.67	1761791	{}	\N	2020-10-12 14:22:36.314761
29669	BKI	2020-10-06	88.67	90.26	88.67	89.03	1141945	{}	\N	2020-10-12 14:22:36.318595
29670	BKI	2020-10-07	89.9	90.67	89.27	89.92	1257703	{}	\N	2020-10-12 14:22:36.325749
29671	BKI	2020-10-08	90.38	90.47	89.27	89.36	1000591	{}	\N	2020-10-12 14:22:36.329906
29672	BKI	2020-10-09	89.43	90.54	89.05	90.49	623182	{}	\N	2020-10-12 14:22:36.333592
29673	BKI	2020-10-12	91.64	91.64	90.85	91.2	62832	{}	\N	2020-10-12 14:22:36.338153
29674	BLCM	2020-09-28	6.58	6.69	6.34	6.44	40953	{}	\N	2020-10-12 14:22:37.534634
29675	BLCM	2020-09-29	6.44	6.8	6.4	6.75	38995	{}	\N	2020-10-12 14:22:37.543293
29676	BLCM	2020-09-30	6.84	6.84	6.43	6.57	35611	{}	\N	2020-10-12 14:22:37.549567
29677	BLCM	2020-10-01	6.65	6.65	6.3994	6.52	23194	{}	\N	2020-10-12 14:22:37.555247
29678	BLCM	2020-10-02	6.16	6.55	6.16	6.33	26769	{}	\N	2020-10-12 14:22:37.560143
29679	BLCM	2020-10-05	6.33	6.67	6.32	6.5	26466	{}	\N	2020-10-12 14:22:37.565029
29680	BLCM	2020-10-06	6.48	6.7	6.36	6.37	19132	{}	\N	2020-10-12 14:22:37.5692
29681	BLCM	2020-10-07	6.49	6.58	6.34	6.49	18741	{}	\N	2020-10-12 14:22:37.573142
29682	BLCM	2020-10-08	6.5	6.99	6.5	6.95	46093	{}	\N	2020-10-12 14:22:37.577113
29683	BLCM	2020-10-09	6.97	7.04	6.74	7.04	28925	{}	\N	2020-10-12 14:22:37.581356
29684	BLCM	2020-10-12	7.12	7.12	6.57	6.6299	11188	{}	\N	2020-10-12 14:22:37.58542
29685	BLIN	2020-09-28	1.85	1.9586	1.85	1.92	33195	{}	\N	2020-10-12 14:22:39.218637
29686	BLIN	2020-09-29	1.92	1.9814	1.9	1.9	21475	{}	\N	2020-10-12 14:22:39.22975
29687	BLIN	2020-09-30	1.9	1.93	1.86	1.86	38534	{}	\N	2020-10-12 14:22:39.236747
29688	BLIN	2020-10-01	1.88	2	1.8794	1.99	65262	{}	\N	2020-10-12 14:22:39.242954
29689	BLIN	2020-10-02	1.87	2.1598	1.87	2.11	167139	{}	\N	2020-10-12 14:22:39.248054
29690	BLIN	2020-10-05	2.13	2.65	2.1	2.23	520371	{}	\N	2020-10-12 14:22:39.253103
29691	BLIN	2020-10-06	2.24	2.4199	2.17	2.26	198510	{}	\N	2020-10-12 14:22:39.257309
29692	BLIN	2020-10-07	2.345	2.345	2.1853	2.23	53056	{}	\N	2020-10-12 14:22:39.261849
29693	BLIN	2020-10-08	2.25	2.29	2.14	2.24	78456	{}	\N	2020-10-12 14:22:39.266149
29694	BLIN	2020-10-09	2.2	2.37	2.2	2.34	104181	{}	\N	2020-10-12 14:22:39.27032
29695	BLIN	2020-10-12	2.31	2.35	2.2916	2.345	7641	{}	\N	2020-10-12 14:22:39.274446
29696	BLRX	2020-09-28	1.6	1.65	1.56	1.65	82250	{}	\N	2020-10-12 14:22:40.403741
29697	BLRX	2020-09-29	1.66	1.67	1.59	1.65	121526	{}	\N	2020-10-12 14:22:40.413666
29698	BLRX	2020-09-30	1.68	1.685	1.61	1.64	160841	{}	\N	2020-10-12 14:22:40.420345
29699	BLRX	2020-10-01	1.68	1.68	1.58	1.6	228523	{}	\N	2020-10-12 14:22:40.426189
29700	BLRX	2020-10-02	1.6	1.61	1.55	1.57	125150	{}	\N	2020-10-12 14:22:40.431279
29701	BLRX	2020-10-05	1.59	1.62	1.56	1.6	154960	{}	\N	2020-10-12 14:22:40.436232
29702	BLRX	2020-10-06	1.6	1.6	1.56	1.57	118659	{}	\N	2020-10-12 14:22:40.440311
29703	BLRX	2020-10-07	1.58	1.5933	1.55	1.59	148378	{}	\N	2020-10-12 14:22:40.444586
29704	BLRX	2020-10-08	1.59	1.6	1.57	1.6	80724	{}	\N	2020-10-12 14:22:40.448837
29705	BLRX	2020-10-09	1.6	1.62	1.57	1.59	112716	{}	\N	2020-10-12 14:22:40.452974
29706	BLRX	2020-10-12	1.65	1.65	1.61	1.6299	35711	{}	\N	2020-10-12 14:22:40.457204
29778	GRA	2020-10-08	40.71	41.455	40.46	41.36	962692	{}	\N	2020-10-12 14:22:51.052856
29779	GRA	2020-10-09	41.89	42.29	41.06	41.15	437167	{}	\N	2020-10-12 14:22:51.05715
29780	GRA	2020-10-12	41.16	41.31	40.65	41.31	37888	{}	\N	2020-10-12 14:22:51.061383
29781	BIOL	2020-09-28	0.2852	0.29	0.276	0.2795	1108025	{}	\N	2020-10-12 14:22:53.03828
29782	BIOL	2020-09-29	0.2792	0.2792	0.273	0.273	721057	{}	\N	2020-10-12 14:22:53.048285
29783	BIOL	2020-09-30	0.2701	0.286	0.2701	0.2751	1701213	{}	\N	2020-10-12 14:22:53.054163
29784	BIOL	2020-10-01	0.288	0.29	0.2701	0.2783	795566	{}	\N	2020-10-12 14:22:53.059368
29785	BIOL	2020-10-02	0.275	0.285	0.2701	0.2787	1382768	{}	\N	2020-10-12 14:22:53.064119
29786	BIOL	2020-10-05	0.28	0.295	0.278	0.2906	1817414	{}	\N	2020-10-12 14:22:53.068727
29787	BIOL	2020-10-06	0.29	0.2935	0.2806	0.2809	1000441	{}	\N	2020-10-12 14:22:53.072738
29788	BIOL	2020-10-07	0.281	0.289	0.281	0.285	841181	{}	\N	2020-10-12 14:22:53.076388
29789	BIOL	2020-10-08	0.282	0.2995	0.2811	0.2978	1515420	{}	\N	2020-10-12 14:22:53.080363
29790	BIOL	2020-10-09	0.301	0.304	0.2917	0.2919	1285715	{}	\N	2020-10-12 14:22:53.084279
29791	BIOL	2020-10-12	0.297	0.297	0.291	0.2955	129240	{}	\N	2020-10-12 14:22:53.088052
29792	BKK	2020-09-28	15	15.01	15	15.01	6676	{}	\N	2020-10-12 14:22:55.313726
29793	BKK	2020-09-29	15	15.02	15	15.01	45824	{}	\N	2020-10-12 14:22:55.33226
29794	BKK	2020-09-30	15	15.03	15	15.03	89986	{}	\N	2020-10-12 14:22:55.341515
29795	BKK	2020-10-01	15	15.0355	14.99	15.02	238949	{}	\N	2020-10-12 14:22:55.347179
29796	BKK	2020-10-02	15.01	15.05	15	15.0101	76436	{}	\N	2020-10-12 14:22:55.351881
29797	BKK	2020-10-05	15.0295	15.0295	15.01	15.02	5098	{}	\N	2020-10-12 14:22:55.356217
29798	BKK	2020-10-06	15.03	15.04	15	15.01	53898	{}	\N	2020-10-12 14:22:55.36104
29799	BKK	2020-10-07	15.0373	15.0373	15.02	15.02	16530	{}	\N	2020-10-12 14:22:55.365312
29800	BKK	2020-10-08	15.01	15.02	15.01	15.01	25360	{}	\N	2020-10-12 14:22:55.369441
29801	BKK	2020-10-09	15.015	15.0159	15.01	15.01	5038	{}	\N	2020-10-12 14:22:55.373753
29802	BKK	2020-10-12	15.0185	15.0185	15.0101	15.0161	6290	{}	\N	2020-10-12 14:22:55.377977
29803	BLFS	2020-09-28	26.1	27.29	26.06	26.97	230858	{}	\N	2020-10-12 14:22:56.682641
29804	BLFS	2020-09-29	27.23	29.43	27.0601	28.66	342289	{}	\N	2020-10-12 14:22:56.691618
29805	BLFS	2020-09-30	29.06	29.49	28.61	28.94	420046	{}	\N	2020-10-12 14:22:56.698073
29806	BLFS	2020-10-01	29.29	29.63	28.695	29.42	241192	{}	\N	2020-10-12 14:22:56.703551
29807	BLFS	2020-10-02	28.825	29.39	28.53	28.6	189206	{}	\N	2020-10-12 14:22:56.708474
29808	BLFS	2020-10-05	28.98	29.83	28.67	29.04	184181	{}	\N	2020-10-12 14:22:56.713773
29809	BLFS	2020-10-06	29.19	30.27	28.861	29.5	215791	{}	\N	2020-10-12 14:22:56.717742
29810	BLFS	2020-10-07	29.85	30.26	28.92	29.41	274914	{}	\N	2020-10-12 14:22:56.721658
29811	BLFS	2020-10-08	29.5	29.7	28.28	28.58	321948	{}	\N	2020-10-12 14:22:56.725626
29812	BLFS	2020-10-09	28.7	29.15	28.5	29.02	180970	{}	\N	2020-10-12 14:22:56.729382
29813	BLFS	2020-10-12	29.1	29.18	28.66	29.18	7652	{}	\N	2020-10-12 14:22:56.733537
29814	BLPH	2020-09-28	9.77	9.92	9.39	9.77	19347	{}	\N	2020-10-12 14:22:57.68581
29815	BLPH	2020-09-29	9.77	9.87	9.54	9.73	11352	{}	\N	2020-10-12 14:22:57.696106
29816	BLPH	2020-09-30	9.88	10.31	9.76	10.18	23108	{}	\N	2020-10-12 14:22:57.703934
29817	BLPH	2020-10-01	10.19	10.4065	9.62	9.9	43671	{}	\N	2020-10-12 14:22:57.710641
29818	BLPH	2020-10-02	9.7	10.25	9.62	9.93	19512	{}	\N	2020-10-12 14:22:57.716281
29819	BLPH	2020-10-05	10	10.4294	10	10.23	16972	{}	\N	2020-10-12 14:22:57.721535
29820	BLPH	2020-10-06	10.19	10.4	10.07	10.16	18765	{}	\N	2020-10-12 14:22:57.72602
29821	BLPH	2020-10-07	10.33	10.33	10	10.27	23320	{}	\N	2020-10-12 14:22:57.730286
29822	BLPH	2020-10-08	10.06	10.27	9.72	10.14	21751	{}	\N	2020-10-12 14:22:57.734718
29823	BLPH	2020-10-09	10.07	10.42	10	10.37	32829	{}	\N	2020-10-12 14:22:57.73876
29824	BLPH	2020-10-12	10.37	10.37	9.85	9.96	1699	{}	\N	2020-10-12 14:22:57.743327
29825	BMRC	2020-09-28	28.24	29.17	28.24	28.79	30276	{}	\N	2020-10-12 14:22:58.723639
29826	BMRC	2020-09-29	30.75	30.75	28.77	29.29	35584	{}	\N	2020-10-12 14:22:58.733908
29827	BMRC	2020-09-30	29.55	29.55	28.75	28.96	32285	{}	\N	2020-10-12 14:22:58.740462
29828	BMRC	2020-10-01	28.86	28.86	28.4	28.58	44646	{}	\N	2020-10-12 14:22:58.746106
29829	BMRC	2020-10-02	28.486	29.89	28.486	29.62	16854	{}	\N	2020-10-12 14:22:58.750859
29830	BMRC	2020-10-05	29.61	31.62	29.61	30.58	21862	{}	\N	2020-10-12 14:22:58.755596
29831	BMRC	2020-10-06	30.6873	32.145	30.6873	30.8	20405	{}	\N	2020-10-12 14:22:58.760395
29832	BMRC	2020-10-07	30.98	31.92	30.385	31.7	24945	{}	\N	2020-10-12 14:22:58.764408
29833	BMRC	2020-10-08	31.26	32.53	30.76	32.37	37891	{}	\N	2020-10-12 14:22:58.768571
29834	BMRC	2020-10-09	32.61	32.61	31.77	31.93	18606	{}	\N	2020-10-12 14:22:58.772456
29835	BMRC	2020-10-12	31.82	32.14	31.335	32	908	{}	\N	2020-10-12 14:22:58.776648
29836	BNS	2020-09-28	41.37	42	41.24	41.83	1089830	{}	\N	2020-10-12 14:23:01.077739
29837	BNS	2020-09-29	41.72	41.83	41.08	41.39	2050924	{}	\N	2020-10-12 14:23:01.087099
29838	BNS	2020-09-30	41.54	41.92	41.41	41.51	2081467	{}	\N	2020-10-12 14:23:01.092506
29839	BNS	2020-10-01	41.84	41.84	41.31	41.73	6909505	{}	\N	2020-10-12 14:23:01.098591
29840	BNS	2020-10-02	41.4	42.2199	41.16	42.06	2467355	{}	\N	2020-10-12 14:23:01.103529
29841	BNS	2020-10-05	41.77	42.16	41.4845	42.02	7704747	{}	\N	2020-10-12 14:23:01.108063
29842	BNS	2020-10-06	42.36	42.6531	41.87	42	3470038	{}	\N	2020-10-12 14:23:01.112472
29843	BNS	2020-10-07	42.6	42.8	42.42	42.68	2555418	{}	\N	2020-10-12 14:23:01.116594
29844	BNS	2020-10-08	42.95	43.13	42.76	42.99	1830741	{}	\N	2020-10-12 14:23:01.120874
29845	BNS	2020-10-09	43.4	43.4	42.75	42.98	2313518	{}	\N	2020-10-12 14:23:01.124729
29846	BNS	2020-10-12	43.15	43.51	43.0545	43.335	498183	{}	\N	2020-10-12 14:23:01.128625
29847	BOSC	2020-09-28	2.33	2.46	2.29	2.45	4579	{}	\N	2020-10-12 14:23:02.445546
29848	BOSC	2020-09-29	2.5	2.5081	2.415	2.4902	3708	{}	\N	2020-10-12 14:23:02.454175
29849	BOSC	2020-09-30	2.5	2.5	2.4182	2.43	922	{}	\N	2020-10-12 14:23:02.460149
29850	BOSC	2020-10-01	2.49	2.63	2.49	2.49	8036	{}	\N	2020-10-12 14:23:02.465375
29851	BOSC	2020-10-02	2.41	2.4699	2.41	2.4699	1192	{}	\N	2020-10-12 14:23:02.469825
29852	BOSC	2020-10-05	2.53	2.53	2.5	2.5	4191	{}	\N	2020-10-12 14:23:02.474287
29853	BOSC	2020-10-06	2.571	2.571	2.51	2.51	3376	{}	\N	2020-10-12 14:23:02.478341
29854	BOSC	2020-10-07	2.5611	2.64	2.5	2.6399	3615	{}	\N	2020-10-12 14:23:02.482208
29855	BOSC	2020-10-08	2.61	2.6285	2.462	2.47	14654	{}	\N	2020-10-12 14:23:02.485748
29856	BOSC	2020-10-09	2.5635	2.5635	2.46	2.49	7041	{}	\N	2020-10-12 14:23:02.48911
29857	BOSC	2020-10-12	2.53	2.535	2.52	2.52	2487	{}	\N	2020-10-12 14:23:02.492534
29858	BRPA	2020-10-01	10.76	10.81	10.76	10.81	3933	{}	\N	2020-10-12 14:23:07.811153
29859	BRPA	2020-10-02	10.73	10.78	10.725	10.78	2774	{}	\N	2020-10-12 14:23:07.819273
29860	BRPA	2020-10-05	10.7883	10.7883	10.7661	10.7661	879	{}	\N	2020-10-12 14:23:07.825096
29861	BRPA	2020-10-07	10.7301	10.7875	10.73	10.7837	3777	{}	\N	2020-10-12 14:23:07.830558
29862	BSCL	2020-09-28	21.3	21.34	21.2909	21.34	263663	{}	\N	2020-10-12 14:23:08.811407
29863	BSCL	2020-09-29	21.3	21.33	21.3	21.33	297676	{}	\N	2020-10-12 14:23:08.822653
29864	BSCL	2020-09-30	21.33	21.33	21.31	21.32	247079	{}	\N	2020-10-12 14:23:08.829329
29865	BSCL	2020-10-01	21.32	21.33	21.31	21.31	185656	{}	\N	2020-10-12 14:23:08.835165
29866	BSCL	2020-10-02	21.29	21.33	21.29	21.31	268387	{}	\N	2020-10-12 14:23:08.839866
29867	BSCL	2020-10-05	21.31	21.34	21.305	21.32	261195	{}	\N	2020-10-12 14:23:08.844527
29868	BSCL	2020-10-06	21.33	21.33	21.31	21.33	377131	{}	\N	2020-10-12 14:23:08.848408
29869	BSCL	2020-10-07	21.32	21.33	21.31	21.3207	326892	{}	\N	2020-10-12 14:23:08.852437
29870	BSCL	2020-10-08	21.32	21.33	21.31	21.31	232832	{}	\N	2020-10-12 14:23:08.856608
29871	BSCL	2020-10-09	21.32	21.33	21.31	21.33	220565	{}	\N	2020-10-12 14:23:08.860605
29872	BSCL	2020-10-12	21.33	21.3365	21.33	21.3316	5112	{}	\N	2020-10-12 14:23:08.864626
29873	OBLN	2020-09-28	0.8695	0.928	0.8241	0.872	146563	{}	\N	2020-10-12 14:23:10.011222
29874	OBLN	2020-09-29	0.9	0.95	0.895101	0.915	88672	{}	\N	2020-10-12 14:23:10.018114
29875	OBLN	2020-09-30	0.94	0.95	0.9	0.9104	60626	{}	\N	2020-10-12 14:23:10.023328
29876	OBLN	2020-10-01	0.94	0.943	0.9	0.9	27109	{}	\N	2020-10-12 14:23:10.027583
29877	OBLN	2020-10-02	0.85	0.8868	0.85	0.8712	60840	{}	\N	2020-10-12 14:23:10.032257
29878	OBLN	2020-10-05	0.8634	0.8979	0.8634	0.894	21647	{}	\N	2020-10-12 14:23:10.036786
29879	OBLN	2020-10-06	0.8837	0.92	0.855	0.8804	38848	{}	\N	2020-10-12 14:23:10.040881
29880	OBLN	2020-10-07	0.8659	0.9202	0.8659	0.887	43232	{}	\N	2020-10-12 14:23:10.045523
29881	OBLN	2020-10-08	0.8713	0.9091	0.811	0.85	43772	{}	\N	2020-10-12 14:23:10.049702
29882	OBLN	2020-10-09	0.86	0.88	0.85	0.85	58185	{}	\N	2020-10-12 14:23:10.05398
29883	OBLN	2020-10-12	0.848	0.91	0.84	0.91	6711	{}	\N	2020-10-12 14:23:10.058026
29884	BJK	2020-09-28	39.39	39.65	39.1216	39.4	19012	{}	\N	2020-10-12 14:23:11.007833
29885	BJK	2020-09-29	39.49	39.5	38.86	38.95	163153	{}	\N	2020-10-12 14:23:11.015129
29886	BJK	2020-09-30	39.04	39.4718	39.0135	39.13	13549	{}	\N	2020-10-12 14:23:11.020775
29887	BJK	2020-10-01	39.37	39.7699	39.311	39.65	11086	{}	\N	2020-10-12 14:23:11.026267
29888	BJK	2020-10-02	38.9	39.7999	38.9	39.64	7930	{}	\N	2020-10-12 14:23:11.03088
29889	BJK	2020-10-05	40	40	39.43	39.67	32755	{}	\N	2020-10-12 14:23:11.036173
29890	BJK	2020-10-06	39.99	40.11	39.29	39.29	9479	{}	\N	2020-10-12 14:23:11.040668
29891	BJK	2020-10-07	39.72	39.72	39.29	39.64	6019	{}	\N	2020-10-12 14:23:11.044864
29892	BJK	2020-10-08	39.8	39.83	39.39	39.83	8011	{}	\N	2020-10-12 14:23:11.049029
29893	BJK	2020-10-09	39.73	39.8528	39.465	39.49	11826	{}	\N	2020-10-12 14:23:11.053099
29894	BJK	2020-10-12	39.85	39.92	39.4707	39.735	4200	{}	\N	2020-10-12 14:23:11.057315
29895	BKF	2020-09-28	45.78	45.84	45.5	45.68	25212	{}	\N	2020-10-12 14:23:12.049667
29896	BKF	2020-09-29	45.49	45.7	45.48	45.58	36437	{}	\N	2020-10-12 14:23:12.057433
29897	BKF	2020-09-30	45.96	46.58	45.96	46.48	14142	{}	\N	2020-10-12 14:23:12.063528
29898	BKF	2020-10-01	46.83	47.014	46.605	46.83	18359	{}	\N	2020-10-12 14:23:12.068789
29899	BKF	2020-10-02	46.19	46.842	46.19	46.48	13397	{}	\N	2020-10-12 14:23:12.074201
29900	BKF	2020-10-05	46.5	46.85	46.5	46.84	31261	{}	\N	2020-10-12 14:23:12.079395
29901	BKF	2020-10-06	47.22	47.47	47.03	47.09	24854	{}	\N	2020-10-12 14:23:12.083564
29902	BKF	2020-10-07	47.53	47.58	47.33	47.45	71377	{}	\N	2020-10-12 14:23:12.087493
29903	BKF	2020-10-08	47.65	47.86	47.61	47.74	18340	{}	\N	2020-10-12 14:23:12.091522
29904	BKF	2020-10-09	47.85	48.21	47.85	48.13	16237	{}	\N	2020-10-12 14:23:12.095942
29905	BKF	2020-10-12	48.62	48.63	48.58	48.63	1929	{}	\N	2020-10-12 14:23:12.100167
29906	BKT	2020-09-28	6.0637	6.0887	6.05	6.07	307036	{}	\N	2020-10-12 14:23:14.212629
29907	BKT	2020-09-29	6.06	6.09	6.05	6.07	719947	{}	\N	2020-10-12 14:23:14.217871
29908	BKT	2020-09-30	6.09	6.14	6.07	6.1	747658	{}	\N	2020-10-12 14:23:14.22216
29909	BKT	2020-10-01	6.09	6.1199	6.05	6.05	712755	{}	\N	2020-10-12 14:23:14.226203
29910	BKT	2020-10-02	6.08	6.08	6.01	6.01	606197	{}	\N	2020-10-12 14:23:14.230585
29911	BKT	2020-10-05	6.01	6.11	6.01	6.11	601889	{}	\N	2020-10-12 14:23:14.234763
29912	BKT	2020-10-06	6.1	6.19	6.06	6.18	523868	{}	\N	2020-10-12 14:23:14.238575
29913	BKT	2020-10-07	6.14	6.14	6.11	6.13	546376	{}	\N	2020-10-12 14:23:14.242602
29914	BKT	2020-10-08	6.14	6.16	6.11	6.13	337416	{}	\N	2020-10-12 14:23:14.246627
29915	BKT	2020-10-09	6.12	6.13	6.1	6.11	332982	{}	\N	2020-10-12 14:23:14.250643
29916	BKT	2020-10-12	6.12	6.137	6.1	6.11	62555	{}	\N	2020-10-12 14:23:14.254909
29917	BLV	2020-09-28	112.17	112.48	112.1528	112.48	278589	{}	\N	2020-10-12 14:23:17.478302
29918	BLV	2020-09-29	112.66	112.8058	112.271	112.39	275858	{}	\N	2020-10-12 14:23:17.486259
29919	BLV	2020-09-30	112.01	112.2999	111.4298	111.84	477805	{}	\N	2020-10-12 14:23:17.491845
29920	BLV	2020-10-01	111.31	112.03	111.23	111.86	405021	{}	\N	2020-10-12 14:23:17.498251
29921	BLV	2020-10-02	111.86	111.86	111.24	111.75	369106	{}	\N	2020-10-12 14:23:17.503961
29922	BLV	2020-10-05	110.98	111	110.34	110.49	679843	{}	\N	2020-10-12 14:23:17.509086
29923	BLV	2020-10-06	110.52	111.45	110.12	110.94	718180	{}	\N	2020-10-12 14:23:17.513766
29924	BLV	2020-10-07	110.5	110.87	110	110.16	405689	{}	\N	2020-10-12 14:23:17.518781
29925	BLV	2020-10-08	110.7	110.9	110.52	110.74	410418	{}	\N	2020-10-12 14:23:17.524229
29926	BLV	2020-10-09	110.53	111.0369	110.27	110.98	359729	{}	\N	2020-10-12 14:23:17.529097
29927	BLV	2020-10-12	111.19	111.3315	111.09	111.235	48102	{}	\N	2020-10-12 14:23:17.533981
29928	BMI	2020-09-28	65.75	66.57	65.73	66.25	126618	{}	\N	2020-10-12 14:23:18.673143
29929	BMI	2020-09-29	66.39	66.84	65.245	65.94	222728	{}	\N	2020-10-12 14:23:18.679954
29930	BMI	2020-09-30	65.97	66.63	65	65.37	293986	{}	\N	2020-10-12 14:23:18.685707
29931	BMI	2020-10-01	65.58	66.41	64.96	66.4	264486	{}	\N	2020-10-12 14:23:18.690743
29932	BMI	2020-10-02	65.39	67.235	65.39	66.72	187002	{}	\N	2020-10-12 14:23:18.699525
29933	BMI	2020-10-05	67.04	68	67.04	67.66	252874	{}	\N	2020-10-12 14:23:18.706758
29934	BMI	2020-10-06	67.9	69.87	67.49	68.19	452050	{}	\N	2020-10-12 14:23:18.713557
29935	BMI	2020-10-07	68.69	68.96	67.71	68.52	290310	{}	\N	2020-10-12 14:23:18.721211
29936	BMI	2020-10-08	68.99	69.99	68.8034	69.96	348103	{}	\N	2020-10-12 14:23:18.729636
29937	BMI	2020-10-09	70	71.14	69.88	70.49	228858	{}	\N	2020-10-12 14:23:18.741089
29938	BMI	2020-10-12	71.16	72.16	70.49	71.98	26945	{}	\N	2020-10-12 14:23:18.749068
29939	BNDC	2020-09-28	26.85	26.96	26.85	26.9	15717	{}	\N	2020-10-12 14:23:20.823302
29940	BNDC	2020-09-29	26.99	26.99	26.9	26.9	3912	{}	\N	2020-10-12 14:23:20.829333
29941	BNDC	2020-09-30	26.9148	26.95	26.89	26.89	1296	{}	\N	2020-10-12 14:23:20.834544
29942	BNDC	2020-10-01	26.94	26.97	26.9082	26.93	9597	{}	\N	2020-10-12 14:23:20.838896
29943	BNDC	2020-10-02	27.12	27.12	26.92	26.96	74152	{}	\N	2020-10-12 14:23:20.843155
29944	BNDC	2020-10-05	26.94	26.94	26.88	26.91	1882	{}	\N	2020-10-12 14:23:20.84748
29945	BNDC	2020-10-06	26.58	26.93	26.58	26.84	40426	{}	\N	2020-10-12 14:23:20.851827
29946	BNDC	2020-10-07	26.84	26.9	26.77	26.89	5308	{}	\N	2020-10-12 14:23:20.856099
29947	BNDC	2020-10-08	26.8488	26.8782	26.84	26.865	6614	{}	\N	2020-10-12 14:23:20.860513
29948	BNDC	2020-10-09	26.844	26.91	26.8391	26.885	6855	{}	\N	2020-10-12 14:23:20.865048
29949	BOE	2020-09-28	9.55	9.61	9.54	9.56	150835	{}	\N	2020-10-12 14:23:22.503639
29950	BOE	2020-09-29	9.59	9.6048	9.55	9.58	182840	{}	\N	2020-10-12 14:23:22.513241
29951	BOE	2020-09-30	9.64	9.7	9.57	9.57	507192	{}	\N	2020-10-12 14:23:22.520734
29952	BOE	2020-10-01	9.68	9.7019	9.62	9.65	389075	{}	\N	2020-10-12 14:23:22.526646
29953	BOE	2020-10-02	9.58	9.68	9.55	9.68	251830	{}	\N	2020-10-12 14:23:22.531979
29954	BOE	2020-10-05	9.69	9.8	9.69	9.8	366046	{}	\N	2020-10-12 14:23:22.537029
29955	BOE	2020-10-06	9.85	9.85	9.69	9.73	197817	{}	\N	2020-10-12 14:23:22.542318
29956	BOE	2020-10-07	9.77	9.9	9.77	9.9	445342	{}	\N	2020-10-12 14:23:22.546758
29957	BOE	2020-10-08	9.95	9.95	9.87	9.9	290488	{}	\N	2020-10-12 14:23:22.551329
29958	BOE	2020-10-09	9.95	10	9.92	10	229106	{}	\N	2020-10-12 14:23:22.555856
29959	BOE	2020-10-12	10	10.015	9.97	10.01	30068	{}	\N	2020-10-12 14:23:22.560589
29960	BOOM	2020-09-28	31.46	32.89	31.46	32.14	70518	{}	\N	2020-10-12 14:23:23.682373
29961	BOOM	2020-09-29	32	32.96	31.57	32.87	92371	{}	\N	2020-10-12 14:23:23.692646
29962	BOOM	2020-09-30	33.18	34.29	31.9256	32.95	106669	{}	\N	2020-10-12 14:23:23.699872
29963	BOOM	2020-10-01	33.15	33.15	30.63	31.79	547966	{}	\N	2020-10-12 14:23:23.70595
29964	BOOM	2020-10-02	30.96	33.45	30.5	32.95	105509	{}	\N	2020-10-12 14:23:23.7114
29965	BOOM	2020-10-05	33.38	34.32	32.31	34.13	53760	{}	\N	2020-10-12 14:23:23.716323
29966	BOOM	2020-10-06	34.9	36.1442	33.86	34.08	72887	{}	\N	2020-10-12 14:23:23.721056
29967	BOOM	2020-10-07	34.53	35.45	33.03	33.77	83265	{}	\N	2020-10-12 14:23:23.725334
29968	BOOM	2020-10-08	34.25	34.66	33.92	34.25	84394	{}	\N	2020-10-12 14:23:23.729818
29969	BOOM	2020-10-09	34.7	34.7	33.775	34.05	65293	{}	\N	2020-10-12 14:23:23.734148
29970	BOOM	2020-10-12	33.92	33.92	33.06	33.06	4318	{}	\N	2020-10-12 14:23:23.738489
29971	BPT	2020-09-28	1.57	1.5893	1.52	1.53	71801	{}	\N	2020-10-12 14:23:27.583872
29972	BPT	2020-09-29	1.51	1.53	1.44	1.45	285780	{}	\N	2020-10-12 14:23:27.595868
29973	BPT	2020-09-30	1.5	1.5199	1.45	1.48	137834	{}	\N	2020-10-12 14:23:27.603097
29974	BPT	2020-10-01	1.49	1.5148	1.44	1.47	115815	{}	\N	2020-10-12 14:23:27.608864
29975	BPT	2020-10-02	1.43	1.47	1.43	1.46	139740	{}	\N	2020-10-12 14:23:27.613481
29976	BPT	2020-10-05	1.5	1.5	1.45	1.45	133927	{}	\N	2020-10-12 14:23:27.618291
29977	BPT	2020-10-06	1.43	1.5	1.43	1.49	161045	{}	\N	2020-10-12 14:23:27.623627
29978	BPT	2020-10-07	1.49	1.53	1.47	1.53	122108	{}	\N	2020-10-12 14:23:27.627572
29979	BPT	2020-10-08	1.53	1.64	1.51	1.59	302628	{}	\N	2020-10-12 14:23:27.631475
29980	BPT	2020-10-09	1.6	1.6426	1.5387	1.58	280410	{}	\N	2020-10-12 14:23:27.635594
29981	BPT	2020-10-12	1.51	1.5699	1.51	1.5559	39266	{}	\N	2020-10-12 14:23:27.639443
29982	BRFS	2020-09-28	3.55	3.565	3.285	3.35	4502044	{}	\N	2020-10-12 14:23:29.017172
29983	BRFS	2020-09-29	3.3	3.41	3.22	3.23	4824510	{}	\N	2020-10-12 14:23:29.023871
29984	BRFS	2020-09-30	3.26	3.3	3.21	3.28	5751155	{}	\N	2020-10-12 14:23:29.028553
29985	BRFS	2020-10-01	3.23	3.29	3.18	3.27	5785842	{}	\N	2020-10-12 14:23:29.032598
29986	BRFS	2020-10-02	3.26	3.325	3.19	3.21	3802509	{}	\N	2020-10-12 14:23:29.037366
29987	BRFS	2020-10-05	3.2	3.26	3.18	3.25	3140178	{}	\N	2020-10-12 14:23:29.041899
29988	BRFS	2020-10-06	3.3	3.325	3.185	3.22	6891502	{}	\N	2020-10-12 14:23:29.045337
29989	BRFS	2020-10-07	3.2	3.23	3.13	3.16	3456778	{}	\N	2020-10-12 14:23:29.049118
29990	BRFS	2020-10-08	3.14	3.25	3.125	3.25	4246850	{}	\N	2020-10-12 14:23:29.052837
29991	BRFS	2020-10-09	3.2	3.2452	3.155	3.17	5775815	{}	\N	2020-10-12 14:23:29.056525
29992	BRFS	2020-10-12	3.21	3.2341	3.18	3.1878	200264	{}	\N	2020-10-12 14:23:29.060236
29993	BRO	2020-09-28	44.32	44.99	44.05	44.87	879357	{}	\N	2020-10-12 14:23:32.085092
29994	BRO	2020-09-29	44.83	45.42	44.73	45.17	2094443	{}	\N	2020-10-12 14:23:32.093882
29995	BRO	2020-09-30	45.33	45.645	44.94	45.27	1751240	{}	\N	2020-10-12 14:23:32.101291
29996	BRO	2020-10-01	45.51	45.71	44.75	44.94	1851109	{}	\N	2020-10-12 14:23:32.10748
29997	BRO	2020-10-02	44.54	45.9	44.445	45.36	1817931	{}	\N	2020-10-12 14:23:32.113012
29998	BRO	2020-10-05	45.81	46	45.37	45.73	1932532	{}	\N	2020-10-12 14:23:32.117763
29999	BRO	2020-10-06	45.93	46.47	45.6	45.73	2037952	{}	\N	2020-10-12 14:23:32.123179
30000	BRO	2020-10-07	45.96	46.41	45.86	46.27	1441668	{}	\N	2020-10-12 14:23:32.127387
30001	BRO	2020-10-08	46.57	46.85	46.32	46.71	1192807	{}	\N	2020-10-12 14:23:32.131499
30002	BRO	2020-10-09	47	47.5	46.84	47.22	1428195	{}	\N	2020-10-12 14:23:32.136884
30003	BRO	2020-10-12	47.51	47.85	47.31	47.79	113406	{}	\N	2020-10-12 14:23:32.142962
30004	BKCC	2020-09-28	2.52	2.6	2.52	2.55	310118	{}	\N	2020-10-12 14:23:33.063076
30005	BKCC	2020-09-29	2.53	2.55	2.485	2.54	273208	{}	\N	2020-10-12 14:23:33.06886
30006	BKCC	2020-09-30	2.51	2.56	2.44	2.44	303932	{}	\N	2020-10-12 14:23:33.073709
30007	BKCC	2020-10-01	2.5	2.5	2.46	2.46	124276	{}	\N	2020-10-12 14:23:33.078177
30008	BKCC	2020-10-02	2.39	2.5	2.39	2.49	188767	{}	\N	2020-10-12 14:23:33.082884
30009	BKCC	2020-10-05	2.49	2.565	2.49	2.55	295758	{}	\N	2020-10-12 14:23:33.087312
30010	BKCC	2020-10-06	2.54	2.57	2.5	2.53	261240	{}	\N	2020-10-12 14:23:33.091386
30011	BKCC	2020-10-07	2.55	2.56	2.47	2.47	171770	{}	\N	2020-10-12 14:23:33.096109
30012	BKCC	2020-10-08	2.49	2.62	2.49	2.62	442595	{}	\N	2020-10-12 14:23:33.100912
30013	BKCC	2020-10-09	2.62	2.67	2.59	2.66	265874	{}	\N	2020-10-12 14:23:33.10476
30014	BKCC	2020-10-12	2.65	2.66	2.61	2.6171	13642	{}	\N	2020-10-12 14:23:33.109877
30015	BLCN	2020-09-28	33.93	34.33	33.93	34.19	33098	{}	\N	2020-10-12 14:23:34.144251
30016	BLCN	2020-09-29	34.13	34.19	33.94	34.08	29862	{}	\N	2020-10-12 14:23:34.152036
30017	BLCN	2020-09-30	34	34.4	34	34.1826	42350	{}	\N	2020-10-12 14:23:34.157881
30018	BLCN	2020-10-01	34.4999	34.5201	34.36	34.46	24081	{}	\N	2020-10-12 14:23:34.163535
30019	BLCN	2020-10-02	33.95	34.3378	33.92	34.14	26207	{}	\N	2020-10-12 14:23:34.168472
30020	BLCN	2020-10-05	34.5	34.7499	34.47	34.715	44643	{}	\N	2020-10-12 14:23:34.17299
30021	BLCN	2020-10-06	34.7	35	34.4	34.4	54206	{}	\N	2020-10-12 14:23:34.177102
30022	BLCN	2020-10-07	34.73	34.95	34.7	34.94	22294	{}	\N	2020-10-12 14:23:34.18123
30023	BLCN	2020-10-08	35.18	35.31	35.05	35.31	36687	{}	\N	2020-10-12 14:23:34.185152
30024	BLCN	2020-10-09	35.55	35.69	35.4	35.53	46422	{}	\N	2020-10-12 14:23:34.189061
30025	BLCN	2020-10-12	35.84	35.84	35.795	35.8213	8639	{}	\N	2020-10-12 14:23:34.193412
30026	BLK	2020-09-28	554	563.94	554	558.65	523259	{}	\N	2020-10-12 14:23:35.114811
30027	BLK	2020-09-29	560.16	561.855	552.37	554.8	803407	{}	\N	2020-10-12 14:23:35.125326
30028	BLK	2020-09-30	557.2	570.8825	555.11	563.55	1407710	{}	\N	2020-10-12 14:23:35.142338
30029	BLK	2020-10-01	576.84	578.24	566.17	567.91	1029450	{}	\N	2020-10-12 14:23:35.147432
30030	BLK	2020-10-02	560.54	574.61	559.18	570.12	949795	{}	\N	2020-10-12 14:23:35.151302
30031	BLK	2020-10-05	574.7	581.52	573	580.98	809630	{}	\N	2020-10-12 14:23:35.154953
30032	BLK	2020-10-06	580.98	590.6044	578.13	580.23	1379464	{}	\N	2020-10-12 14:23:35.159173
30033	BLK	2020-10-07	584.32	592.79	584.32	592.05	1083613	{}	\N	2020-10-12 14:23:35.162973
30034	BLK	2020-10-08	595.36	606.21	594.34	604.44	1056253	{}	\N	2020-10-12 14:23:35.166769
30035	BLK	2020-10-09	610.6	614.3	606.58	611.57	1022537	{}	\N	2020-10-12 14:23:35.170296
30036	BLK	2020-10-12	615.37	620.66	613.66	619.41	213077	{}	\N	2020-10-12 14:23:35.174226
30037	BLW	2020-09-28	14.69	14.775	14.63	14.71	82027	{}	\N	2020-10-12 14:23:36.53618
30038	BLW	2020-09-29	14.65	14.72	14.63	14.68	305848	{}	\N	2020-10-12 14:23:36.543195
30039	BLW	2020-09-30	14.76	14.79	14.67	14.71	524878	{}	\N	2020-10-12 14:23:36.548378
30040	BLW	2020-10-01	14.73	14.86	14.73	14.83	174554	{}	\N	2020-10-12 14:23:36.553531
30041	BLW	2020-10-02	14.7	14.84	14.7	14.76	181066	{}	\N	2020-10-12 14:23:36.559047
30042	BLW	2020-10-05	14.8	14.91	14.79	14.89	264892	{}	\N	2020-10-12 14:23:36.564348
30043	BLW	2020-10-06	14.87	14.98	14.87	14.93	131826	{}	\N	2020-10-12 14:23:36.572109
30044	BLW	2020-10-07	15.04	15.15	15.015	15.04	186563	{}	\N	2020-10-12 14:23:36.578032
30045	BLW	2020-10-08	15.13	15.13	14.96	14.98	164638	{}	\N	2020-10-12 14:23:36.583676
30046	BLW	2020-10-09	15	15.07	15	15.06	82248	{}	\N	2020-10-12 14:23:36.588673
30047	BLW	2020-10-12	15.12	15.12	15.08	15.0825	26166	{}	\N	2020-10-12 14:23:36.592888
30048	BMRA	2020-09-28	7.25	7.25	6.8867	7.03	89269	{}	\N	2020-10-12 14:23:37.576436
30049	BMRA	2020-09-29	6.98	7.25	6.83	7.17	81815	{}	\N	2020-10-12 14:23:37.586519
30050	BMRA	2020-09-30	7.16	7.16	6.93	6.93	81365	{}	\N	2020-10-12 14:23:37.593674
30051	BMRA	2020-10-01	6.93	7	6.76	6.97	107894	{}	\N	2020-10-12 14:23:37.600927
30052	BMRA	2020-10-02	7.01	7.36	6.86	7.35	137620	{}	\N	2020-10-12 14:23:37.607121
30053	BMRA	2020-10-05	7.35	7.4	7	7.13	107771	{}	\N	2020-10-12 14:23:37.61221
30054	BMRA	2020-10-06	7.12	7.39	7.12	7.35	165393	{}	\N	2020-10-12 14:23:37.617831
30055	BMRA	2020-10-07	7.42	7.5513	7.38	7.49	101659	{}	\N	2020-10-12 14:23:37.623429
30056	BMRA	2020-10-08	7.61	7.84	7.55	7.84	164344	{}	\N	2020-10-12 14:23:37.629574
30057	BMRA	2020-10-09	7.84	7.88	7.65	7.85	99428	{}	\N	2020-10-12 14:23:37.634094
30058	BMRA	2020-10-12	7.88	7.88	7.62	7.7765	21326	{}	\N	2020-10-12 14:23:37.638207
30059	BNDX	2020-09-28	58.19	58.22	58.15	58.215	4416487	{}	\N	2020-10-12 14:23:38.584112
30060	BNDX	2020-09-29	58.22	58.27	58.21	58.255	3215992	{}	\N	2020-10-12 14:23:38.590092
30061	BNDX	2020-09-30	58.24	58.24	58.15	58.185	2901427	{}	\N	2020-10-12 14:23:38.59473
30062	BNDX	2020-10-01	58.08	58.17	58.07	58.17	2403393	{}	\N	2020-10-12 14:23:38.600293
30063	BNDX	2020-10-02	58.24	58.24	58.16	58.205	1846329	{}	\N	2020-10-12 14:23:38.605337
30064	BNDX	2020-10-05	58.14	58.15	58.06	58.08	2057344	{}	\N	2020-10-12 14:23:38.610827
30065	BNDX	2020-10-06	58.08	58.12	58.07	58.115	2384416	{}	\N	2020-10-12 14:23:38.615967
30066	BNDX	2020-10-07	58.1	58.11	58.07	58.075	2041527	{}	\N	2020-10-12 14:23:38.620807
30067	BNDX	2020-10-08	58.13	58.18	58.1059	58.18	1504506	{}	\N	2020-10-12 14:23:38.625685
30068	BNDX	2020-10-09	58.22	58.26	58.19	58.24	1388354	{}	\N	2020-10-12 14:23:38.629873
30069	BNDX	2020-10-12	58.2762	58.29	58.26	58.265	138534	{}	\N	2020-10-12 14:23:38.635589
30070	BOMN	2020-09-28	15.79	15.79	15.36	15.53	191240	{}	\N	2020-10-12 14:23:39.568694
30071	BOMN	2020-09-29	15.605	15.76	15.41	15.68	49823	{}	\N	2020-10-12 14:23:39.574443
30072	BOMN	2020-09-30	15.75	16.05	15.71	16	89186	{}	\N	2020-10-12 14:23:39.578737
30073	BOMN	2020-10-01	16.07	16.41	16.07	16.41	80934	{}	\N	2020-10-12 14:23:39.583408
30074	BOMN	2020-10-02	16.34	16.5	16.01	16.41	157454	{}	\N	2020-10-12 14:23:39.58876
30075	BOMN	2020-10-05	16.51	17	16.401	16.5	124932	{}	\N	2020-10-12 14:23:39.595563
30076	BOMN	2020-10-06	16.59	16.955	16.39	16.58	139975	{}	\N	2020-10-12 14:23:39.601007
30077	BOMN	2020-10-07	16.81	16.96	16.67	16.75	78224	{}	\N	2020-10-12 14:23:39.606682
30078	BOMN	2020-10-08	16.99	17.33	16.88	17.24	128497	{}	\N	2020-10-12 14:23:39.614178
30079	BOMN	2020-10-09	17.32	17.87	17.28	17.53	86027	{}	\N	2020-10-12 14:23:39.621268
30080	BOMN	2020-10-12	17.67	17.91	17.5	17.77	26406	{}	\N	2020-10-12 14:23:39.628989
30081	BOX	2020-09-28	17.59	17.86	17.5167	17.67	1115804	{}	\N	2020-10-12 14:23:40.82059
30082	BOX	2020-09-29	17.7	17.82	17.4	17.45	2225915	{}	\N	2020-10-12 14:23:40.827048
30083	BOX	2020-09-30	17.4	17.58	17.22	17.36	3027305	{}	\N	2020-10-12 14:23:40.832138
30084	BOX	2020-10-01	17.47	17.82	17.415	17.52	2772073	{}	\N	2020-10-12 14:23:40.83829
30085	BOX	2020-10-02	17.18	17.47	16.96	17.1	2936637	{}	\N	2020-10-12 14:23:40.847186
30086	BOX	2020-10-05	17.15	17.25	16.92	17.22	3831455	{}	\N	2020-10-12 14:23:40.853958
30087	BOX	2020-10-06	17.3	17.7288	17.08	17.16	3465130	{}	\N	2020-10-12 14:23:40.860222
30088	BOX	2020-10-07	17.24	17.5	17.08	17.38	2443952	{}	\N	2020-10-12 14:23:40.865819
30089	BOX	2020-10-08	17.5	17.76	17.35	17.73	3179074	{}	\N	2020-10-12 14:23:40.872035
30090	BOX	2020-10-09	17.8	18.12	17.77	17.91	2340928	{}	\N	2020-10-12 14:23:40.878433
30091	BOX	2020-10-12	18.1	18.16	17.81	17.94	490596	{}	\N	2020-10-12 14:23:40.884663
30092	BPOPM	2020-09-28	25.7	26.45	25.68	26.43	13284	{}	\N	2020-10-12 14:23:42.289664
30093	BPOPM	2020-09-29	26.29	26.29	26	26.25	2318	{}	\N	2020-10-12 14:23:42.295624
30094	BPOPM	2020-09-30	26.25	26.39	26.066	26.32	1964	{}	\N	2020-10-12 14:23:42.300314
30095	BPOPM	2020-10-01	26	26.4097	26	26.4097	2071	{}	\N	2020-10-12 14:23:42.304413
30096	BPOPM	2020-10-02	26.38	26.39	26.38	26.39	561	{}	\N	2020-10-12 14:23:42.309144
30097	BPOPM	2020-10-05	26.44	26.5	26.215	26.5	2381	{}	\N	2020-10-12 14:23:42.314353
30098	BPOPM	2020-10-06	27	27	26.9	26.9	3183	{}	\N	2020-10-12 14:23:42.318709
30099	BPOPM	2020-10-07	26.95	27	26.57	26.8	8487	{}	\N	2020-10-12 14:23:42.323874
30100	BPOPM	2020-10-08	27	27	26.86	26.935	5695	{}	\N	2020-10-12 14:23:42.328472
30101	BPOPM	2020-10-09	26.62	26.625	26.46	26.625	3056	{}	\N	2020-10-12 14:23:42.332941
30102	BRF	2020-09-28	17.17	17.2198	16.52	16.52	3934	{}	\N	2020-10-12 14:23:43.260252
30103	BRF	2020-09-29	16.51	16.61	16.33	16.33	3051	{}	\N	2020-10-12 14:23:43.266819
30104	BRF	2020-09-30	16.52	16.688	16.49	16.688	1147	{}	\N	2020-10-12 14:23:43.272467
30105	BRF	2020-10-01	16.57	16.9265	16.46	16.92	91123	{}	\N	2020-10-12 14:23:43.277647
30106	BRF	2020-10-02	16.72	16.8	16.6076	16.6076	2279	{}	\N	2020-10-12 14:23:43.282467
30107	BRF	2020-10-05	16.75	17.0315	16.62	17.02	6604	{}	\N	2020-10-12 14:23:43.286954
30108	BRF	2020-10-06	17.43	17.43	16.88	16.91	4253	{}	\N	2020-10-12 14:23:43.29151
30109	BRF	2020-10-07	17.05	17.1	16.81	16.83	4307	{}	\N	2020-10-12 14:23:43.296272
30110	BRF	2020-10-08	17.01	17.2445	17.01	17.2	1346	{}	\N	2020-10-12 14:23:43.300592
30111	BRF	2020-10-09	17.45	17.6999	17.43	17.43	17022	{}	\N	2020-10-12 14:23:43.304808
30112	BRF	2020-10-12	17.54	17.59	17.54	17.59	2102	{}	\N	2020-10-12 14:23:43.309281
30113	BRPAR	2020-09-28	0.2474	0.2474	0.2049	0.2049	9163	{}	\N	2020-10-12 14:23:45.570354
30114	BRPAR	2020-10-01	0.2049	0.206	0.2049	0.206	1200	{}	\N	2020-10-12 14:23:45.576178
30115	BRPAR	2020-10-02	0.206	0.206	0.206	0.206	2420	{}	\N	2020-10-12 14:23:45.581296
30116	BRPAR	2020-10-05	0.2081	0.208181	0.2	0.2	27132	{}	\N	2020-10-12 14:23:45.585742
30117	BRPAR	2020-10-06	0.1999	0.2	0.177	0.2	33522	{}	\N	2020-10-12 14:23:45.590138
30118	BRPAR	2020-10-07	0.205	0.205	0.205	0.205	1223	{}	\N	2020-10-12 14:23:45.595085
30119	BRPAR	2020-10-08	0.15	0.208	0.1369	0.208	9542	{}	\N	2020-10-12 14:23:45.599635
30120	BRPAR	2020-10-09	0.231	0.3499	0.231	0.231	4766	{}	\N	2020-10-12 14:23:45.604003
30121	BSCM	2020-09-28	21.81	21.83	21.78	21.83	270456	{}	\N	2020-10-12 14:23:46.514753
30122	BSCM	2020-09-29	21.82	21.83	21.81	21.81	301559	{}	\N	2020-10-12 14:23:46.523557
30123	BSCM	2020-09-30	21.79	21.84	21.79	21.84	394898	{}	\N	2020-10-12 14:23:46.529167
30124	BSCM	2020-10-01	21.83	21.84	21.82	21.82	216312	{}	\N	2020-10-12 14:23:46.535586
30125	BSCM	2020-10-02	21.81	21.84	21.81	21.82	259294	{}	\N	2020-10-12 14:23:46.540309
30126	BSCM	2020-10-05	21.83	21.85	21.815	21.85	171200	{}	\N	2020-10-12 14:23:46.544758
30127	BSCM	2020-10-06	21.85	21.86	21.83	21.84	539977	{}	\N	2020-10-12 14:23:46.549294
30128	BSCM	2020-10-07	21.85	21.85	21.82	21.84	373798	{}	\N	2020-10-12 14:23:46.553494
30129	BSCM	2020-10-08	21.84	21.86	21.83	21.83	209535	{}	\N	2020-10-12 14:23:46.55783
30130	BSCM	2020-10-09	21.84	21.87	21.83	21.87	187483	{}	\N	2020-10-12 14:23:46.562044
30131	BSCM	2020-10-12	21.85	21.8699	21.85	21.8546	14008	{}	\N	2020-10-12 14:23:46.566181
30132	BSJK	2020-09-28	23.37	23.3799	23.34	23.37	81028	{}	\N	2020-10-12 14:23:47.572184
30133	BSJK	2020-09-29	23.36	23.37	23.3582	23.36	87283	{}	\N	2020-10-12 14:23:47.58336
30134	BSJK	2020-09-30	23.3572	23.37	23.35	23.36	68824	{}	\N	2020-10-12 14:23:47.589939
30135	BSJK	2020-10-01	23.36	23.37	23.35	23.355	85754	{}	\N	2020-10-12 14:23:47.59578
30136	BSJK	2020-10-02	23.34	23.37	23.34	23.35	144513	{}	\N	2020-10-12 14:23:47.600952
30137	BSJK	2020-10-05	23.35	23.3756	23.34	23.37	129645	{}	\N	2020-10-12 14:23:47.605305
30138	BSJK	2020-10-06	23.38	23.38	23.3547	23.36	264255	{}	\N	2020-10-12 14:23:47.610191
30139	BSJK	2020-10-07	23.36	23.37	23.35	23.36	164655	{}	\N	2020-10-12 14:23:47.615732
30140	BSJK	2020-10-08	23.35	23.37	23.35	23.36	90068	{}	\N	2020-10-12 14:23:47.620086
30141	BSJK	2020-10-09	23.35	23.37	23.35	23.36	70420	{}	\N	2020-10-12 14:23:47.624579
30142	BSJK	2020-10-12	23.37	23.3722	23.36	23.3722	22542	{}	\N	2020-10-12 14:23:47.628897
30143	OBSV	2020-09-28	2.5	2.54	2.42	2.46	285074	{}	\N	2020-10-12 14:23:49.34578
30144	OBSV	2020-09-29	2.47	2.5411	2.43	2.48	343049	{}	\N	2020-10-12 14:23:49.354681
30145	OBSV	2020-09-30	2.49	2.53	2.4589	2.48	235082	{}	\N	2020-10-12 14:23:49.361015
30146	OBSV	2020-10-01	2.51	2.52	2.345	2.4	683171	{}	\N	2020-10-12 14:23:49.366404
30147	OBSV	2020-10-02	2.35	2.49	2.2801	2.42	369204	{}	\N	2020-10-12 14:23:49.37158
30148	OBSV	2020-10-05	2.41	2.57	2.41	2.52	353232	{}	\N	2020-10-12 14:23:49.375864
30149	OBSV	2020-10-06	2.5558	2.71	2.45	2.555	952696	{}	\N	2020-10-12 14:23:49.380385
30150	OBSV	2020-10-07	2.6188	2.65	2.42	2.47	739291	{}	\N	2020-10-12 14:23:49.384289
30151	OBSV	2020-10-08	2.5	2.54	2.45	2.45	592363	{}	\N	2020-10-12 14:23:49.388084
30152	OBSV	2020-10-09	2.49	2.49	2.43	2.47	311372	{}	\N	2020-10-12 14:23:49.392221
30153	OBSV	2020-10-12	2.5	2.51	2.45	2.5	70809	{}	\N	2020-10-12 14:23:49.396279
30154	BKSC	2020-09-28	16.52	17	16.075	16.27	7858	{}	\N	2020-10-12 14:23:50.32547
30155	BKSC	2020-09-29	16.48	16.5175	16.35	16.375	1812	{}	\N	2020-10-12 14:23:50.337242
30156	BKSC	2020-09-30	15.83	16.73	15.83	16.03	5067	{}	\N	2020-10-12 14:23:50.345207
30157	BKSC	2020-10-01	16.3	16.34	16.04	16.27	6246	{}	\N	2020-10-12 14:23:50.352557
30158	BKSC	2020-10-02	15.77	16.2768	15.68	16.05	3828	{}	\N	2020-10-12 14:23:50.356901
30159	BKSC	2020-10-05	16.17	17	16.17	16.17	7283	{}	\N	2020-10-12 14:23:50.361197
30160	BKSC	2020-10-06	16.73	16.95	16.3372	16.52	6832	{}	\N	2020-10-12 14:23:50.364613
30161	BKSC	2020-10-07	16.7259	16.9899	16.51	16.755	1370	{}	\N	2020-10-12 14:23:50.368265
30162	BKSC	2020-10-08	16.8	17	16.412	16.55	5285	{}	\N	2020-10-12 14:23:50.371888
30163	BKSC	2020-10-09	16.5904	16.7999	16.585	16.67	1182	{}	\N	2020-10-12 14:23:50.375461
30164	BKSC	2020-10-12	16.2988	16.2988	16.2988	16.2988	201	{}	\N	2020-10-12 14:23:50.379206
30165	BLD	2020-09-28	162.62	168.65	161.3516	167.73	373044	{}	\N	2020-10-12 14:23:51.563993
30166	BLD	2020-09-29	169.65	171.82	167.28	167.35	458478	{}	\N	2020-10-12 14:23:51.568055
30167	BLD	2020-09-30	168.35	172.4	168.35	170.69	708412	{}	\N	2020-10-12 14:23:51.571923
30168	BLD	2020-10-01	172.1	177.27	170.41	176.89	1022962	{}	\N	2020-10-12 14:23:51.575999
30169	BLD	2020-10-02	170.35	180.94	170.35	179.75	457406	{}	\N	2020-10-12 14:23:51.580082
30170	BLD	2020-10-05	177.88	183.55	173.31	177.75	862934	{}	\N	2020-10-12 14:23:51.584252
30171	BLD	2020-10-06	180.77	186.1	175.56	177.6	872703	{}	\N	2020-10-12 14:23:51.588867
30172	BLD	2020-10-07	180.94	183.79	178.99	180.48	454113	{}	\N	2020-10-12 14:23:51.592676
30173	BLD	2020-10-08	183.25	186.54	182.381	183.53	471868	{}	\N	2020-10-12 14:23:51.596832
30174	BLD	2020-10-09	185.85	187.965	183.27	186.91	366262	{}	\N	2020-10-12 14:23:51.601308
30175	BLD	2020-10-12	188.43	190.35	186.57	189.26	30208	{}	\N	2020-10-12 14:23:51.605104
30176	BLL	2020-09-28	82.22	82.795	81.295	82.24	876892	{}	\N	2020-10-12 14:23:52.503857
30177	BLL	2020-09-29	82.22	82.915	81.75	82.36	1794259	{}	\N	2020-10-12 14:23:52.510726
30178	BLL	2020-09-30	82.77	84	82.2	83.12	2980061	{}	\N	2020-10-12 14:23:52.515506
30179	BLL	2020-10-01	83.63	84.29	82.48	82.6	3254658	{}	\N	2020-10-12 14:23:52.520168
30180	BLL	2020-10-02	81.67	83.25	81.43	82.67	2494323	{}	\N	2020-10-12 14:23:52.524949
30181	BLL	2020-10-05	83.82	86.4	83.58	85.37	4300335	{}	\N	2020-10-12 14:23:52.529436
30182	BLL	2020-10-06	85.57	86.19	83.99	84.31	3576634	{}	\N	2020-10-12 14:23:52.533564
30183	BLL	2020-10-07	85.75	86.87	85.515	86.55	2693186	{}	\N	2020-10-12 14:23:52.538162
30184	BLL	2020-10-08	88.13	89.4	87.68	89	4630054	{}	\N	2020-10-12 14:23:52.542833
30185	BLL	2020-10-09	89.45	90.89	89.2	90.26	3105587	{}	\N	2020-10-12 14:23:52.547043
30186	BLL	2020-10-12	90.93	91	90.36	90.46	635314	{}	\N	2020-10-12 14:23:52.551038
30187	BMCH	2020-09-28	40.87	42.33	40.42	41.91	1071380	{}	\N	2020-10-12 14:23:54.016209
30188	BMCH	2020-09-29	41.92	42.26	41.49	41.58	618286	{}	\N	2020-10-12 14:23:54.024163
30189	BMCH	2020-09-30	42.1	43.54	41.84	42.83	1002798	{}	\N	2020-10-12 14:23:54.030407
30190	BMCH	2020-10-01	42.83	43.45	42.56	43.25	444929	{}	\N	2020-10-12 14:23:54.035857
30191	BMCH	2020-10-02	42.29	44.02	42.2	43.82	991550	{}	\N	2020-10-12 14:23:54.04103
30192	BMCH	2020-10-05	44.34	44.75	42.67	43.43	1051300	{}	\N	2020-10-12 14:23:54.045493
30193	BMCH	2020-10-06	43.78	44.68	43.175	43.54	878955	{}	\N	2020-10-12 14:23:54.049478
30194	BMCH	2020-10-07	44.24	44.41	43.45	43.97	520615	{}	\N	2020-10-12 14:23:54.053716
30195	BMCH	2020-10-08	44.69	44.69	43.36	44.03	295319	{}	\N	2020-10-12 14:23:54.058838
30196	BMCH	2020-10-09	44.52	44.76	43.57	44.39	240778	{}	\N	2020-10-12 14:23:54.063215
30197	BMCH	2020-10-12	44.8	45.12	44.45	44.72	19998	{}	\N	2020-10-12 14:23:54.067793
30198	BOCH	2020-09-28	6.73	7.16	6.73	7.1	68428	{}	\N	2020-10-12 14:23:55.946394
30199	BOCH	2020-09-29	7.14	7.14	6.82	7.11	16528	{}	\N	2020-10-12 14:23:55.952412
30200	BOCH	2020-09-30	7.08	7.12	6.92	6.97	10167	{}	\N	2020-10-12 14:23:55.957622
30201	BOCH	2020-10-01	6.98	6.98	6.8	6.86	20309	{}	\N	2020-10-12 14:23:55.965254
30202	BOCH	2020-10-02	6.75	7.5	6.75	7.4	56900	{}	\N	2020-10-12 14:23:55.971524
30203	BOCH	2020-10-05	7.3901	7.475	7	7.19	30959	{}	\N	2020-10-12 14:23:55.976436
30204	BOCH	2020-10-06	7.26	7.29	7.04	7.12	57897	{}	\N	2020-10-12 14:23:55.983433
30205	BOCH	2020-10-07	7.305	7.47	7.18	7.46	19366	{}	\N	2020-10-12 14:23:55.989888
30206	BOCH	2020-10-08	7.46	7.645	7.3	7.59	20994	{}	\N	2020-10-12 14:23:56.000905
30207	BOCH	2020-10-09	7.69	7.74	7.35	7.4	15513	{}	\N	2020-10-12 14:23:56.028571
30208	BOCH	2020-10-12	7.45	7.5	7.4101	7.4101	1482	{}	\N	2020-10-12 14:23:56.03859
30209	BOND	2020-09-28	112.21	112.42	112.15	112.41	251810	{}	\N	2020-10-12 14:23:57.504845
30210	BOND	2020-09-29	112.25	112.59	112.25	112.51	423553	{}	\N	2020-10-12 14:23:57.512433
30211	BOND	2020-09-30	112.51	112.55	112.31	112.43	340267	{}	\N	2020-10-12 14:23:57.518555
30212	BOND	2020-10-01	112.16	112.2799	112.03	112.2	328418	{}	\N	2020-10-12 14:23:57.524245
30213	BOND	2020-10-02	112.16	112.23	112.01	112.1	329467	{}	\N	2020-10-12 14:23:57.528958
30214	BOND	2020-10-05	111.99	112.04	111.86	111.93	157857	{}	\N	2020-10-12 14:23:57.533628
30215	BOND	2020-10-06	111.85	111.9981	111.791	111.81	183585	{}	\N	2020-10-12 14:23:57.538001
30216	BOND	2020-10-07	111.88	111.97	111.74	111.91	190443	{}	\N	2020-10-12 14:23:57.542618
30217	BOND	2020-10-08	111.77	111.99	111.72	111.75	241717	{}	\N	2020-10-12 14:23:57.547738
30218	BOND	2020-10-09	111.92	111.92	111.6901	111.71	95811	{}	\N	2020-10-12 14:23:57.552417
30219	BOND	2020-10-12	111.63	112.04	111.63	111.7762	25759	{}	\N	2020-10-12 14:23:57.557842
30220	BOXL	2020-09-28	1.68	1.71	1.55	1.62	4158199	{}	\N	2020-10-12 14:23:59.294604
30221	BOXL	2020-09-29	1.6	1.62	1.51	1.62	2661408	{}	\N	2020-10-12 14:23:59.304319
30222	BOXL	2020-09-30	1.59	1.65	1.53	1.61	2651142	{}	\N	2020-10-12 14:23:59.311199
30223	BOXL	2020-10-01	1.61	1.85	1.565	1.85	8060143	{}	\N	2020-10-12 14:23:59.317358
30224	BOXL	2020-10-02	1.71	1.85	1.69	1.79	6190836	{}	\N	2020-10-12 14:23:59.32427
30225	BOXL	2020-10-05	1.9429	2.12	1.87	2.055	17361691	{}	\N	2020-10-12 14:23:59.328617
30226	BOXL	2020-10-06	2.03	2.12	1.8	1.9	6811313	{}	\N	2020-10-12 14:23:59.333047
30227	BOXL	2020-10-07	1.9	1.95	1.81	1.87	3145056	{}	\N	2020-10-12 14:23:59.337364
30228	BOXL	2020-10-08	1.94	1.94	1.81	1.86	2719511	{}	\N	2020-10-12 14:23:59.341689
30229	BOXL	2020-10-09	1.87	1.88	1.75	1.75	2977361	{}	\N	2020-10-12 14:23:59.345995
30230	BOXL	2020-10-12	1.79	1.81	1.76	1.775	474280	{}	\N	2020-10-12 14:23:59.350292
30231	BPOP	2020-09-28	36.07	36.94	35.945	36.71	340312	{}	\N	2020-10-12 14:24:00.440317
30232	BPOP	2020-09-29	36.51	36.51	35.505	36.15	221891	{}	\N	2020-10-12 14:24:00.449008
30233	BPOP	2020-09-30	36.13	36.87	35.97	36.27	484148	{}	\N	2020-10-12 14:24:00.454706
30234	BPOP	2020-10-01	36.49	36.985	36.07	36.33	341471	{}	\N	2020-10-12 14:24:00.460369
30235	BPOP	2020-10-02	35.82	37.87	35.82	37.8	361302	{}	\N	2020-10-12 14:24:00.465442
30236	BPOP	2020-10-05	38.26	39.47	38.235	39.45	494235	{}	\N	2020-10-12 14:24:00.470159
30237	BPOP	2020-10-06	39.93	40.7	38.69	38.84	381183	{}	\N	2020-10-12 14:24:00.474902
30238	BPOP	2020-10-07	39.39	40.4	39.3	39.83	623652	{}	\N	2020-10-12 14:24:00.479
30239	BPOP	2020-10-08	39.85	40.64	39.73	40.54	498635	{}	\N	2020-10-12 14:24:00.483088
30240	BPOP	2020-10-09	41.01	41.09	40.27	40.45	678385	{}	\N	2020-10-12 14:24:00.486941
30241	BPOP	2020-10-12	40.63	40.82	40.1101	40.2	23839	{}	\N	2020-10-12 14:24:00.490865
30242	BRN	2020-09-28	0.91	0.91	0.7632	0.80997	51073	{}	\N	2020-10-12 14:24:02.716558
30243	BRN	2020-09-29	1	1.64	0.867251	0.9265	12769847	{}	\N	2020-10-12 14:24:02.724669
30244	BRN	2020-09-30	0.805	0.89	0.76	0.8526	806726	{}	\N	2020-10-12 14:24:02.730079
30245	BRN	2020-10-01	0.8201	0.84	0.78	0.8309	134802	{}	\N	2020-10-12 14:24:02.735166
30246	BRN	2020-10-02	0.81	0.8307	0.7611	0.8299	127305	{}	\N	2020-10-12 14:24:02.742284
30247	BRN	2020-10-05	0.7948	0.8212	0.770101	0.8	64761	{}	\N	2020-10-12 14:24:02.746891
30248	BRN	2020-10-06	0.7999	0.9	0.78	0.85	176018	{}	\N	2020-10-12 14:24:02.750946
30249	BRN	2020-10-07	0.8384	0.86	0.80041	0.8502	74893	{}	\N	2020-10-12 14:24:02.756396
30250	BRN	2020-10-08	0.8502	0.999	0.8442	0.92	658361	{}	\N	2020-10-12 14:24:02.761243
30251	BRN	2020-10-09	0.9298	1.17	0.87	1.02	1247145	{}	\N	2020-10-12 14:24:02.765927
30252	BRN	2020-10-12	1.05	1.06	0.8716	0.890101	143931	{}	\N	2020-10-12 14:24:02.770093
30253	BSCQ	2020-09-28	21.62	21.68	21.57	21.6525	89349	{}	\N	2020-10-12 14:24:03.925552
30254	BSCQ	2020-09-29	21.67	21.68	21.6144	21.68	65064	{}	\N	2020-10-12 14:24:03.935416
30255	BSCQ	2020-09-30	21.64	21.66	21.625	21.64	44140	{}	\N	2020-10-12 14:24:03.942813
30256	BSCQ	2020-10-01	21.655	21.69	21.6368	21.68	42898	{}	\N	2020-10-12 14:24:03.948648
30257	BSCQ	2020-10-02	21.63	21.69	21.63	21.6755	47119	{}	\N	2020-10-12 14:24:03.953342
30258	BSCQ	2020-10-05	21.75	21.75	21.6582	21.69	25644	{}	\N	2020-10-12 14:24:03.958871
30259	BSCQ	2020-10-06	21.7	21.7299	21.65	21.69	289861	{}	\N	2020-10-12 14:24:03.963455
30260	BSCQ	2020-10-07	21.67	21.67	21.63	21.6439	47789	{}	\N	2020-10-12 14:24:03.967581
30261	BSCQ	2020-10-08	21.64	21.68	21.64	21.67	31218	{}	\N	2020-10-12 14:24:03.971697
30262	BSCQ	2020-10-09	21.6625	21.7	21.652	21.67	32527	{}	\N	2020-10-12 14:24:03.976368
30263	BSCQ	2020-10-12	21.7014	21.7014	21.7014	21.7014	1532	{}	\N	2020-10-12 14:24:03.980784
30264	BSJO	2020-09-28	24	24.11	23.965	24.04	46860	{}	\N	2020-10-12 14:24:04.921211
30265	BSJO	2020-09-29	24.05	24.0799	23.97	24.05	25374	{}	\N	2020-10-12 14:24:04.929904
30266	BSJO	2020-09-30	24.04	24.13	24.04	24.11	42514	{}	\N	2020-10-12 14:24:04.935885
30267	BSJO	2020-10-01	24.14	24.219	24.14	24.2	32600	{}	\N	2020-10-12 14:24:04.942764
30268	BSJO	2020-10-02	24.1	24.17	24.07	24.16	72973	{}	\N	2020-10-12 14:24:04.948432
30269	BSJO	2020-10-05	24.19	24.27	24.15	24.26	105082	{}	\N	2020-10-12 14:24:04.952809
30270	BSJO	2020-10-06	24.25	24.35	24.221	24.2674	151691	{}	\N	2020-10-12 14:24:04.957476
30271	BSJO	2020-10-07	24.31	24.3284	24.27	24.3	48632	{}	\N	2020-10-12 14:24:04.962177
30272	BSJO	2020-10-08	24.29	24.37	24.285	24.37	52146	{}	\N	2020-10-12 14:24:04.966537
30273	BSJO	2020-10-09	24.4	24.4045	24.32	24.39	59214	{}	\N	2020-10-12 14:24:04.970846
30274	BSJO	2020-10-12	24.4414	24.46	24.3818	24.45	36087	{}	\N	2020-10-12 14:24:04.975959
30275	BKYI	2020-09-28	0.44	0.4629	0.43	0.447	868850	{}	\N	2020-10-12 14:24:06.081461
30276	BKYI	2020-09-29	0.4595	0.4596	0.4295	0.43	915077	{}	\N	2020-10-12 14:24:06.090657
30277	BKYI	2020-09-30	0.4295	0.4475	0.425	0.4397	1083254	{}	\N	2020-10-12 14:24:06.097291
30278	BKYI	2020-10-01	0.4389	0.45	0.416	0.44	932243	{}	\N	2020-10-12 14:24:06.103124
30279	BKYI	2020-10-02	0.42	0.445	0.4176	0.432	928511	{}	\N	2020-10-12 14:24:06.108127
30280	BKYI	2020-10-05	0.4214	0.44	0.421	0.425	778110	{}	\N	2020-10-12 14:24:06.11276
30281	BKYI	2020-10-06	0.426	0.4388	0.4225	0.4265	1033179	{}	\N	2020-10-12 14:24:06.116738
30282	BKYI	2020-10-07	0.425	0.438	0.41	0.4201	1135701	{}	\N	2020-10-12 14:24:06.120595
30283	BKYI	2020-10-08	0.683	0.72	0.553	0.553	124580099	{}	\N	2020-10-12 14:24:06.124443
30284	BKYI	2020-10-09	0.5176	0.55	0.5	0.538	11742803	{}	\N	2020-10-12 14:24:06.129256
30285	BKYI	2020-10-12	0.5198	0.52	0.492	0.4922	2182901	{}	\N	2020-10-12 14:24:06.133109
30286	BLE	2020-09-28	15.08	15.23	15.0362	15.08	19677	{}	\N	2020-10-12 14:24:07.124081
30287	BLE	2020-09-29	15.15	15.16	14.97	15.04	70158	{}	\N	2020-10-12 14:24:07.132874
30288	BLE	2020-09-30	15.04	15.1	14.9	14.9	68884	{}	\N	2020-10-12 14:24:07.138999
30289	BLE	2020-10-01	15.07	15.1	14.94	15.01	85099	{}	\N	2020-10-12 14:24:07.144938
30290	BLE	2020-10-02	15	15.09	14.96	15.08	39664	{}	\N	2020-10-12 14:24:07.149967
30291	BLE	2020-10-05	15.1	15.15	14.97	15	60320	{}	\N	2020-10-12 14:24:07.154702
30292	BLE	2020-10-06	15.08	15.14	14.9834	15.06	74680	{}	\N	2020-10-12 14:24:07.158784
30293	BLE	2020-10-07	15.08	15.13	14.9904	15.08	78820	{}	\N	2020-10-12 14:24:07.162888
30294	BLE	2020-10-08	15.12	15.35	15.12	15.153	37216	{}	\N	2020-10-12 14:24:07.1665
30295	BLE	2020-10-09	15.11	15.33	15.1	15.16	70236	{}	\N	2020-10-12 14:24:07.170239
30296	BLE	2020-10-12	15.06	15.06	15.03	15.035	55790	{}	\N	2020-10-12 14:24:07.174239
30297	BLX	2020-09-28	12.14	12.36	12.05	12.08	116436	{}	\N	2020-10-12 14:24:08.784386
30298	BLX	2020-09-29	12.02	12.1	11.8	11.89	253499	{}	\N	2020-10-12 14:24:08.793338
30299	BLX	2020-09-30	12.12	12.29	11.9724	12.15	153578	{}	\N	2020-10-12 14:24:08.799718
30300	BLX	2020-10-01	12.09	12.32	11.6	12.17	345340	{}	\N	2020-10-12 14:24:08.80545
30301	BLX	2020-10-02	12	12.28	12	12.25	168984	{}	\N	2020-10-12 14:24:08.810642
30302	BLX	2020-10-05	12.26	12.55	12.18	12.49	232188	{}	\N	2020-10-12 14:24:08.815626
30303	BLX	2020-10-06	12.49	13.1	12.49	12.71	210483	{}	\N	2020-10-12 14:24:08.819519
30304	BLX	2020-10-07	12.87	13.19	12.8	13.17	154994	{}	\N	2020-10-12 14:24:08.823594
30305	BLX	2020-10-08	13.26	13.69	13.2589	13.58	186695	{}	\N	2020-10-12 14:24:08.827525
30306	BLX	2020-10-09	13.5	13.591	13.1	13.15	122984	{}	\N	2020-10-12 14:24:08.83228
30307	BLX	2020-10-12	13.09	13.25	13.05	13.23	27240	{}	\N	2020-10-12 14:24:08.83648
30308	BOOT	2020-09-28	28.14	28.6	27.46	27.48	626232	{}	\N	2020-10-12 14:24:10.908293
30309	BOOT	2020-09-29	27.66	27.76	26.08	26.83	1305540	{}	\N	2020-10-12 14:24:10.919343
30310	BOOT	2020-09-30	26.81	28.62	26.81	28.14	1226704	{}	\N	2020-10-12 14:24:10.926098
30311	BOOT	2020-10-01	28.12	29.44	28.06	29.38	1049417	{}	\N	2020-10-12 14:24:10.932344
30312	BOOT	2020-10-02	28.43	31.46	28.25	30.93	1734409	{}	\N	2020-10-12 14:24:10.937669
30313	BOOT	2020-10-05	31.38	31.97	29.79	30.63	1460311	{}	\N	2020-10-12 14:24:10.942683
30314	BOOT	2020-10-06	30.97	31.62	29.5	30.61	1768128	{}	\N	2020-10-12 14:24:10.946724
30315	BOOT	2020-10-07	31.27	32.72	30.86	32.61	2135556	{}	\N	2020-10-12 14:24:10.950656
30316	BOOT	2020-10-08	33.06	33.32	31.89	32.29	1508659	{}	\N	2020-10-12 14:24:10.954786
30317	BOOT	2020-10-09	32.65	33.36	32.34	32.54	1266531	{}	\N	2020-10-12 14:24:10.959094
30318	BOOT	2020-10-12	33.32	33.77	32.84	33.23	249037	{}	\N	2020-10-12 14:24:10.963408
30319	BP	2020-09-28	18.23	18.345	18.11	18.18	13952502	{}	\N	2020-10-12 14:24:11.881457
30320	BP	2020-09-29	17.95	17.96	17.57	17.67	30172998	{}	\N	2020-10-12 14:24:11.891192
30321	BP	2020-09-30	17.72	17.8	17.35	17.46	31182305	{}	\N	2020-10-12 14:24:11.897789
30322	BP	2020-10-01	17.04	17.145	16.7575	16.83	34398489	{}	\N	2020-10-12 14:24:11.904259
30323	BP	2020-10-02	16.42	16.9174	16.38	16.85	36711892	{}	\N	2020-10-12 14:24:11.909044
30324	BP	2020-10-05	16.97	17.23	16.865	17.19	24638679	{}	\N	2020-10-12 14:24:11.913537
30325	BP	2020-10-06	17.41	17.56	16.87	16.93	31405390	{}	\N	2020-10-12 14:24:11.917788
30326	BP	2020-10-07	16.9	17.085	16.74	17.03	26621144	{}	\N	2020-10-12 14:24:11.922018
30327	BP	2020-10-08	17.16	17.51	17.16	17.49	28561776	{}	\N	2020-10-12 14:24:11.926317
30328	BP	2020-10-09	17.68	17.74	17.31	17.36	23003803	{}	\N	2020-10-12 14:24:11.930397
30329	BP	2020-10-12	17.2	17.22	17.06	17.1	6164790	{}	\N	2020-10-12 14:24:11.934983
30330	BPOPN	2020-09-28	26.54	27	26.43	26.98	24054	{}	\N	2020-10-12 14:24:13.566291
30331	BPOPN	2020-09-29	26.75	27	26.69	27	2973	{}	\N	2020-10-12 14:24:13.573673
30332	BPOPN	2020-09-30	26.81	27.26	26.76	27.26	58017	{}	\N	2020-10-12 14:24:13.579405
30333	BPOPN	2020-10-01	26.95	27.08	26.95	27.0291	11007	{}	\N	2020-10-12 14:24:13.583988
30334	BPOPN	2020-10-02	27	27	26.913	27	1920	{}	\N	2020-10-12 14:24:13.588904
30335	BPOPN	2020-10-05	27	27.05	26.82	27.01	5026	{}	\N	2020-10-12 14:24:13.592859
30336	BPOPN	2020-10-06	27.46	27.5	27.28	27.28	7978	{}	\N	2020-10-12 14:24:13.59693
30337	BPOPN	2020-10-07	27.37	27.62	27.35	27.51	8738	{}	\N	2020-10-12 14:24:13.601335
30338	BPOPN	2020-10-08	27.77	27.77	27.37	27.76	9195	{}	\N	2020-10-12 14:24:13.605687
30339	BPOPN	2020-10-09	27.48	27.49	26.1501	26.95	15195	{}	\N	2020-10-12 14:24:13.609648
30340	BR	2020-09-28	131.8	133.29	131.375	132	563718	{}	\N	2020-10-12 14:24:14.975112
30341	BR	2020-09-29	132.1	133.14	131.36	131.72	441637	{}	\N	2020-10-12 14:24:14.983623
30342	BR	2020-09-30	131.92	132.85	130.85	132	1526561	{}	\N	2020-10-12 14:24:14.989191
30343	BR	2020-10-01	133.41	135.4	132.6	133.94	1379949	{}	\N	2020-10-12 14:24:14.993915
30344	BR	2020-10-02	131.7	133.88	131.23	133.41	923461	{}	\N	2020-10-12 14:24:15.000388
30345	BR	2020-10-05	134.22	136.42	133.46	136.31	881617	{}	\N	2020-10-12 14:24:15.00514
30346	BR	2020-10-06	136.2	138.675	135.56	135.75	1065895	{}	\N	2020-10-12 14:24:15.009098
30347	BR	2020-10-07	136.59	137.82	135.3208	137.51	794672	{}	\N	2020-10-12 14:24:15.012745
30348	BR	2020-10-08	138.72	139.91	137.81	139.05	595715	{}	\N	2020-10-12 14:24:15.01657
30349	BR	2020-10-09	139.73	141.42	139.19	140.84	538210	{}	\N	2020-10-12 14:24:15.020464
30350	BR	2020-10-12	142.15	142.15	140.98	141.47	74082	{}	\N	2020-10-12 14:24:15.024738
30351	BSCP	2020-09-28	22.26	22.3599	22.26	22.355	157153	{}	\N	2020-10-12 14:24:17.139451
30352	BSCP	2020-09-29	22.36	22.36	22.33	22.35	91228	{}	\N	2020-10-12 14:24:17.150275
30353	BSCP	2020-09-30	22.35	22.36	22.34	22.36	94777	{}	\N	2020-10-12 14:24:17.157127
30354	BSCP	2020-10-01	22.35	22.38	22.34	22.38	58391	{}	\N	2020-10-12 14:24:17.162902
30355	BSCP	2020-10-02	22.39	22.39	22.3401	22.39	105553	{}	\N	2020-10-12 14:24:17.167674
30356	BSCP	2020-10-05	22.38	22.39	22.3601	22.38	129145	{}	\N	2020-10-12 14:24:17.172581
30357	BSCP	2020-10-06	22.38	22.41	22.37	22.4	294298	{}	\N	2020-10-12 14:24:17.176839
30358	BSCP	2020-10-07	22.37	22.38	22.36	22.36	106531	{}	\N	2020-10-12 14:24:17.180998
30359	BSCP	2020-10-08	22.39	22.39	22.37	22.38	91363	{}	\N	2020-10-12 14:24:17.184778
30360	BSCP	2020-10-09	22.4	22.42	22.38	22.41	70204	{}	\N	2020-10-12 14:24:17.188619
30361	BSCP	2020-10-12	22.42	22.42	22.402	22.4038	5263	{}	\N	2020-10-12 14:24:17.192741
30362	BMO	2020-09-28	59.24	59.6994	59.15	59.51	473777	{}	\N	2020-10-12 14:24:18.144619
30363	BMO	2020-09-29	59.26	59.34	58.13	58.38	873456	{}	\N	2020-10-12 14:24:18.150636
30364	BMO	2020-09-30	58.5	59.07	58.24	58.41	1058716	{}	\N	2020-10-12 14:24:18.156582
30365	BMO	2020-10-01	58.66	58.78	57.88	58	747600	{}	\N	2020-10-12 14:24:18.162869
30366	BMO	2020-10-02	57.21	58.35	56.96	58.13	1252955	{}	\N	2020-10-12 14:24:18.169243
30367	BMO	2020-10-05	58.64	59.36	58.52	59.12	1446312	{}	\N	2020-10-12 14:24:18.176477
30368	BMO	2020-10-06	59.82	61.14	59.48	60.41	1736699	{}	\N	2020-10-12 14:24:18.183444
30369	BMO	2020-10-07	61.16	61.71	60.95	61.55	798596	{}	\N	2020-10-12 14:24:18.190286
30370	BMO	2020-10-08	61.87	62.66	61.87	62.3	948088	{}	\N	2020-10-12 14:24:18.197099
30371	BMO	2020-10-09	62.7	62.84	61.96	62.31	895097	{}	\N	2020-10-12 14:24:18.20328
30372	BMO	2020-10-12	62.405	62.86	62.34	62.53	51357	{}	\N	2020-10-12 14:24:18.208589
30373	BNFT	2020-09-28	11.61	12.25	11.61	11.89	316896	{}	\N	2020-10-12 14:24:21.116853
30374	BNFT	2020-09-29	11.82	12.04	11.44	11.67	278082	{}	\N	2020-10-12 14:24:21.127807
30375	BNFT	2020-09-30	11.67	11.89	11.11	11.2	247507	{}	\N	2020-10-12 14:24:21.134953
30376	BNFT	2020-10-01	11.22	11.52	11.09	11.51	170912	{}	\N	2020-10-12 14:24:21.140987
30377	BNFT	2020-10-02	11.21	11.62	11.21	11.46	180653	{}	\N	2020-10-12 14:24:21.146295
30378	BNFT	2020-10-05	11.58	11.8	11.29	11.33	167158	{}	\N	2020-10-12 14:24:21.150755
30379	BNFT	2020-10-06	11.3305	11.61	10.68	10.7	212623	{}	\N	2020-10-12 14:24:21.155358
30380	BNFT	2020-10-07	10.75	11.14	10.56	10.6	211660	{}	\N	2020-10-12 14:24:21.159704
30381	BNFT	2020-10-08	10.96	11.0397	10.605	11	183564	{}	\N	2020-10-12 14:24:21.164197
30382	BNFT	2020-10-09	11.17	11.23	10.76	10.84	215617	{}	\N	2020-10-12 14:24:21.168569
30383	BNFT	2020-10-12	10.54	10.92	10.54	10.86	22381	{}	\N	2020-10-12 14:24:21.172658
30384	BOTJ	2020-09-28	9.8355	9.87	9.835	9.87	2160	{}	\N	2020-10-12 14:24:30.704439
30385	BOTJ	2020-09-29	9.9182	9.9182	9.9182	9.9182	192	{}	\N	2020-10-12 14:24:30.710542
30386	BOTJ	2020-09-30	9.82	9.82	9.75	9.75	668	{}	\N	2020-10-12 14:24:30.715357
30387	BOTJ	2020-10-01	10	10	9.98	10	1188	{}	\N	2020-10-12 14:24:30.720533
30388	BOTJ	2020-10-02	9.8	9.99	9.46	9.99	9398	{}	\N	2020-10-12 14:24:30.725315
30389	BOTJ	2020-10-05	10.25	10.25	9.83	9.83	2118	{}	\N	2020-10-12 14:24:30.730126
30390	BOTJ	2020-10-06	10.118	10.172	9.82	9.82	1310	{}	\N	2020-10-12 14:24:30.735892
30391	BOTJ	2020-10-07	10.0101	10.25	10.01	10.25	5687	{}	\N	2020-10-12 14:24:30.743074
30392	BOTJ	2020-10-08	10.1429	10.15	10.01	10.01	1263	{}	\N	2020-10-12 14:24:30.750409
30393	BOTJ	2020-10-09	10.235	10.235	10.23	10.23	417	{}	\N	2020-10-12 14:24:30.758222
30394	BPMC	2020-09-28	88.3	89.54	87.39	88.88	399992	{}	\N	2020-10-12 14:24:32.039241
30395	BPMC	2020-09-29	89	92.61	88.63	91.57	534657	{}	\N	2020-10-12 14:24:32.045393
30396	BPMC	2020-09-30	91.55	93.57	91.39	92.7	382970	{}	\N	2020-10-12 14:24:32.050529
30397	BPMC	2020-10-01	93.02	94.69	91.79	93.93	413925	{}	\N	2020-10-12 14:24:32.05471
30398	BPMC	2020-10-02	91.54	94.02	91.54	92.08	338616	{}	\N	2020-10-12 14:24:32.05883
30399	BPMC	2020-10-05	92.94	95.69	92.73	95.24	341255	{}	\N	2020-10-12 14:24:32.063243
30400	BPMC	2020-10-06	94.99	99.03	94.99	96.54	343935	{}	\N	2020-10-12 14:24:32.067449
30401	BPMC	2020-10-07	97.66	100.52	97.13	99.32	529111	{}	\N	2020-10-12 14:24:32.07131
30402	BPMC	2020-10-08	99.5	100.3	98.4	99.96	273428	{}	\N	2020-10-12 14:24:32.075312
30403	BPMC	2020-10-09	99.75	99.95	98.17	99.91	254028	{}	\N	2020-10-12 14:24:32.079444
30404	BPMC	2020-10-12	100.49	100.97	99.4	100.5	32250	{}	\N	2020-10-12 14:24:32.083363
30405	BQH	2020-09-28	15.9	15.9173	15.9	15.9	2327	{}	\N	2020-10-12 14:24:33.825174
30406	BQH	2020-09-29	15.73	15.9162	15.73	15.83	7932	{}	\N	2020-10-12 14:24:33.834677
30407	BQH	2020-10-01	15.9	15.9	15.9	15.9	1042	{}	\N	2020-10-12 14:24:33.841336
30408	BQH	2020-10-02	15.8	15.8799	15.73	15.73	4412	{}	\N	2020-10-12 14:24:33.846165
30409	BQH	2020-10-06	15.71	15.8044	15.71	15.8041	836	{}	\N	2020-10-12 14:24:33.85221
30410	BQH	2020-10-07	15.745	15.7637	15.7	15.7	16022	{}	\N	2020-10-12 14:24:33.857406
30411	BQH	2020-10-08	15.7	15.76	15.69	15.76	12424	{}	\N	2020-10-12 14:24:33.861825
30412	BQH	2020-10-09	15.63	15.63	15.56	15.56	6606	{}	\N	2020-10-12 14:24:33.865778
30413	BRG	2020-09-28	7.44	7.68	7.27	7.48	213961	{}	\N	2020-10-12 14:24:35.316905
30414	BRG	2020-09-29	7.53	7.555	7.231	7.44	156479	{}	\N	2020-10-12 14:24:35.335577
30415	BRG	2020-09-30	7.45	7.67	7.44	7.58	151179	{}	\N	2020-10-12 14:24:35.344124
30416	BRG	2020-10-01	7.635	7.92	7.635	7.85	220037	{}	\N	2020-10-12 14:24:35.350048
30417	BRG	2020-10-02	7.73	8.17	7.67	8.16	191750	{}	\N	2020-10-12 14:24:35.354495
30418	BRG	2020-10-05	8.19	8.2	7.89	8.1	207813	{}	\N	2020-10-12 14:24:35.358667
30419	BRG	2020-10-06	8.18	8.34	8.04	8.1	224258	{}	\N	2020-10-12 14:24:35.36286
30420	BRG	2020-10-07	8.2	8.45	8.15	8.38	153958	{}	\N	2020-10-12 14:24:35.36694
30421	BRG	2020-10-08	8.43	8.67	8.36	8.61	157617	{}	\N	2020-10-12 14:24:35.371014
30422	BRG	2020-10-09	8.71	8.9	8.47	8.48	140151	{}	\N	2020-10-12 14:24:35.375094
30423	BRG	2020-10-12	8.58	8.58	8.45	8.53	22375	{}	\N	2020-10-12 14:24:35.379124
30424	BSBR	2020-09-28	5.21	5.25	4.855	4.92	1484906	{}	\N	2020-10-12 14:24:37.287663
30425	BSBR	2020-09-29	4.89	4.97	4.77	4.77	1555064	{}	\N	2020-10-12 14:24:37.298054
30426	BSBR	2020-09-30	4.77	4.95	4.77	4.94	2183149	{}	\N	2020-10-12 14:24:37.305161
30427	BSBR	2020-10-01	4.91	4.92	4.795	4.91	1254236	{}	\N	2020-10-12 14:24:37.312082
30428	BSBR	2020-10-02	4.89	5.12	4.89	5.01	2127204	{}	\N	2020-10-12 14:24:37.316917
30429	BSBR	2020-10-05	4.99	5.11	4.93	5.07	1470413	{}	\N	2020-10-12 14:24:37.321339
30430	BSBR	2020-10-06	5.15	5.255	5.025	5.08	1625507	{}	\N	2020-10-12 14:24:37.329204
30431	BSBR	2020-10-07	5.15	5.15	5.02	5.04	1203250	{}	\N	2020-10-12 14:24:37.334794
30432	BSBR	2020-10-08	5.05	5.495	5.05	5.47	2190219	{}	\N	2020-10-12 14:24:37.340467
30433	BSBR	2020-10-09	5.48	5.79	5.41	5.64	2930302	{}	\N	2020-10-12 14:24:37.344855
30434	BSBR	2020-10-12	5.63	5.66	5.57	5.63	167207	{}	\N	2020-10-12 14:24:37.348956
30435	BSQR	2020-09-28	1.37	1.379	1.3101	1.36	12385	{}	\N	2020-10-12 14:24:38.932
30436	BSQR	2020-09-29	1.36	1.41	1.334	1.3424	4308	{}	\N	2020-10-12 14:24:38.942381
30437	BSQR	2020-09-30	1.38	1.4	1.35	1.35	15920	{}	\N	2020-10-12 14:24:38.948499
30438	BSQR	2020-10-01	1.395	1.4	1.35	1.36	7268	{}	\N	2020-10-12 14:24:38.954296
30439	BSQR	2020-10-02	1.35	1.41	1.35	1.36	5191	{}	\N	2020-10-12 14:24:38.960313
30440	BSQR	2020-10-05	1.4	1.42	1.37	1.39	11732	{}	\N	2020-10-12 14:24:38.965026
30441	BSQR	2020-10-06	1.37	1.42	1.37	1.37	8843	{}	\N	2020-10-12 14:24:38.970542
30442	BSQR	2020-10-07	1.37	1.42	1.32	1.385	70511	{}	\N	2020-10-12 14:24:38.975409
30443	BSQR	2020-10-08	1.395	1.42	1.33	1.42	18034	{}	\N	2020-10-12 14:24:38.980322
30444	BSQR	2020-10-09	1.4199	1.42	1.3727	1.38	36433	{}	\N	2020-10-12 14:24:38.986243
30445	BSQR	2020-10-12	1.37	1.4	1.37	1.4	485	{}	\N	2020-10-12 14:24:38.992734
30446	BTI	2020-09-28	36.41	36.765	36.393	36.44	2110409	{}	\N	2020-10-12 14:24:39.921664
30447	BTI	2020-09-29	35.96	36.1	35.63	35.64	3797180	{}	\N	2020-10-12 14:24:39.927995
30448	BTI	2020-09-30	35.87	36.26	35.82	36.15	3849455	{}	\N	2020-10-12 14:24:39.93367
30449	BTI	2020-10-01	35.21	35.315	34.82	35.27	3278315	{}	\N	2020-10-12 14:24:39.939347
30450	BTI	2020-10-02	35.1	35.69	35.07	35.45	2376566	{}	\N	2020-10-12 14:24:39.945555
30451	BTI	2020-10-05	36.27	36.41	36.0617	36.33	3155420	{}	\N	2020-10-12 14:24:39.95215
30452	BTI	2020-10-06	35.79	36.05	35.415	35.56	3020331	{}	\N	2020-10-12 14:24:39.95905
30453	BTI	2020-10-07	35.52	35.65	35.35	35.48	3303100	{}	\N	2020-10-12 14:24:39.965421
30454	BTI	2020-10-08	35.3	35.49	35.2	35.31	2555227	{}	\N	2020-10-12 14:24:39.971401
30455	BTI	2020-10-09	35.58	35.6	35.25	35.25	2614655	{}	\N	2020-10-12 14:24:39.977385
30456	BTI	2020-10-12	35.3	35.45	35.22	35.4463	526576	{}	\N	2020-10-12 14:24:39.984054
32527	CHIQ	2020-09-28	26.75	26.7599	26.495	26.74	100750	{}	\N	2020-10-12 14:29:42.154814
32528	CHIQ	2020-09-29	26.69	26.88	26.62	26.83	229852	{}	\N	2020-10-12 14:29:42.170108
32529	CHIQ	2020-09-30	27.28	27.66	27.255	27.49	425814	{}	\N	2020-10-12 14:29:42.175728
32530	CHIQ	2020-10-01	27.83	27.87	27.578	27.84	149996	{}	\N	2020-10-12 14:29:42.181139
32531	CHIQ	2020-10-02	27.3	27.7699	27.1	27.47	99459	{}	\N	2020-10-12 14:29:42.186018
32532	CHIQ	2020-10-05	27.6	27.7	27.485	27.7	201746	{}	\N	2020-10-12 14:29:42.19036
32533	CHIQ	2020-10-06	27.97	28.26	27.86	27.94	487665	{}	\N	2020-10-12 14:29:42.194946
32534	CHIQ	2020-10-07	28.32	28.4599	28.2092	28.39	224865	{}	\N	2020-10-12 14:29:42.199334
32535	CHIQ	2020-10-08	28.64	28.64	28.4619	28.61	490057	{}	\N	2020-10-12 14:29:42.203729
32536	CHIQ	2020-10-09	28.55	28.91	28.45	28.825	132905	{}	\N	2020-10-12 14:29:42.208068
32537	CHIQ	2020-10-12	29.24	29.3343	29.185	29.3343	352233	{}	\N	2020-10-12 14:29:42.212261
32538	CDXS	2020-09-28	11.94	12.25	11.83	12.09	486617	{}	\N	2020-10-12 14:29:44.098239
32539	CDXS	2020-09-29	12.1	12.15	11.81	12	273214	{}	\N	2020-10-12 14:29:44.106653
32540	CDXS	2020-09-30	12.05	12.17	11.73	11.74	300395	{}	\N	2020-10-12 14:29:44.112718
32541	CDXS	2020-10-01	11.75	12.29	11.555	12.14	295740	{}	\N	2020-10-12 14:29:44.118324
32542	CDXS	2020-10-02	11.83	12.06	11.6	11.71	268957	{}	\N	2020-10-12 14:29:44.123839
32543	CDXS	2020-10-05	11.8	12.3	11.8	12.28	145547	{}	\N	2020-10-12 14:29:44.12851
32544	CDXS	2020-10-06	12.405	13.1	12.405	12.64	592644	{}	\N	2020-10-12 14:29:44.132866
32545	CDXS	2020-10-07	12.82	13.02	12.51	12.69	327603	{}	\N	2020-10-12 14:29:44.136856
32546	CDXS	2020-10-08	13	13.65	12.84	13.55	378078	{}	\N	2020-10-12 14:29:44.140699
32547	CDXS	2020-10-09	13.64	13.8779	13.31	13.82	384809	{}	\N	2020-10-12 14:29:44.144959
32548	CDXS	2020-10-12	13.665	14.1599	13.665	14.1	30223	{}	\N	2020-10-12 14:29:44.148655
32549	CETX	2020-09-28	1	1.05	0.978	1.04	1528723	{}	\N	2020-10-12 14:29:45.556442
32550	CETX	2020-09-29	1.0323	1.09	1.02	1.06	853019	{}	\N	2020-10-12 14:29:45.567641
32551	CETX	2020-09-30	1.06	1.16	1.04	1.11	1718433	{}	\N	2020-10-12 14:29:45.574622
32552	CETX	2020-10-01	1.11	1.11	1.02	1.05	600272	{}	\N	2020-10-12 14:29:45.580862
32553	CETX	2020-10-02	0.99	1.15	0.9801	1.1	1599224	{}	\N	2020-10-12 14:29:45.586304
32554	CETX	2020-10-05	1.08	1.19	1.06	1.15	1763918	{}	\N	2020-10-12 14:29:45.591523
32555	CETX	2020-10-06	1.1714	1.2	1.01	1.05	1947573	{}	\N	2020-10-12 14:29:45.595936
32556	CETX	2020-10-07	1.07	1.08	1.04	1.05	981541	{}	\N	2020-10-12 14:29:45.600298
32557	CETX	2020-10-08	1.13	1.14	1.05	1.1	1515350	{}	\N	2020-10-12 14:29:45.60487
32558	CETX	2020-10-09	1.08	1.13	1.06	1.07	662961	{}	\N	2020-10-12 14:29:45.609192
32559	CETX	2020-10-12	1.07	1.1	1.06	1.0839	90933	{}	\N	2020-10-12 14:29:45.613782
32560	CFO	2020-09-28	53.88	54.3797	53.88	54.15	26905	{}	\N	2020-10-12 14:29:46.746146
32561	CFO	2020-09-29	54.13	54.43	54.0101	54.03	19490	{}	\N	2020-10-12 14:29:46.756386
32562	CFO	2020-09-30	54.16	54.81	54.0549	54.4587	71896	{}	\N	2020-10-12 14:29:46.763135
32563	CFO	2020-10-01	54.84	54.84	54.38	54.59	22384	{}	\N	2020-10-12 14:29:46.768874
32564	CFO	2020-10-02	53.96	54.805	53.96	54.58	26590	{}	\N	2020-10-12 14:29:46.773404
30457	BWX	2020-09-28	29.7	29.75	29.63	29.75	183135	{}	\N	2020-10-12 14:24:42.128362
30458	BWX	2020-09-29	29.78	29.84	29.73	29.8	258912	{}	\N	2020-10-12 14:24:42.141629
30459	BWX	2020-09-30	29.79	29.91	29.67	29.67	1235669	{}	\N	2020-10-12 14:24:42.148273
30460	BWX	2020-10-01	29.82	29.93	29.72	29.73	702662	{}	\N	2020-10-12 14:24:42.153948
30461	BWX	2020-10-02	29.79	29.85	29.76	29.78	441076	{}	\N	2020-10-12 14:24:42.158759
30462	BWX	2020-10-05	29.8	29.91	29.78	29.8	698682	{}	\N	2020-10-12 14:24:42.164474
30463	BWX	2020-10-06	29.79	29.893	29.74	29.81	192364	{}	\N	2020-10-12 14:24:42.169208
30464	BWX	2020-10-07	29.76	29.8478	29.69	29.69	283265	{}	\N	2020-10-12 14:24:42.173709
30465	BWX	2020-10-08	29.7	29.87	29.7	29.87	225549	{}	\N	2020-10-12 14:24:42.178722
30466	BWX	2020-10-09	29.9	30.02	29.9	30.01	120923	{}	\N	2020-10-12 14:24:42.184063
30467	BWX	2020-10-12	30.1	30.1	29.9989	29.9989	18609	{}	\N	2020-10-12 14:24:42.189212
30468	BOKF	2020-09-28	50.99	52.385	50.99	51.6	170760	{}	\N	2020-10-12 14:24:43.090216
30469	BOKF	2020-09-29	51.37	51.37	49.81	50.97	147886	{}	\N	2020-10-12 14:24:43.101579
30470	BOKF	2020-09-30	51.5	52.62	50.93	51.51	249060	{}	\N	2020-10-12 14:24:43.108055
30471	BOKF	2020-10-01	51.33	52.36	50.735	51.64	130903	{}	\N	2020-10-12 14:24:43.114208
30472	BOKF	2020-10-02	50.48	54.74	50.48	53.96	215876	{}	\N	2020-10-12 14:24:43.120128
30473	BOKF	2020-10-05	54.57	56.27	54.28	55.57	184714	{}	\N	2020-10-12 14:24:43.124671
30474	BOKF	2020-10-06	56.06	58.36	56.04	56.29	167716	{}	\N	2020-10-12 14:24:43.129047
30475	BOKF	2020-10-07	56.86	58.55	56.78	57.23	145784	{}	\N	2020-10-12 14:24:43.133716
30476	BOKF	2020-10-08	58.08	58.37	57.18	57.9	142596	{}	\N	2020-10-12 14:24:43.13823
30477	BOKF	2020-10-09	58.31	58.31	56.66	57.38	126190	{}	\N	2020-10-12 14:24:43.142874
30478	BOKF	2020-10-12	57.3	58.04	55.81	57.52	5307	{}	\N	2020-10-12 14:24:43.147419
30479	BPY	2020-09-28	11.32	12	11.32	11.73	2610885	{}	\N	2020-10-12 14:24:44.516656
30480	BPY	2020-09-29	11.7	11.95	11.7	11.73	2299590	{}	\N	2020-10-12 14:24:44.522565
30481	BPY	2020-09-30	11.79	12.21	11.78	12.03	3319890	{}	\N	2020-10-12 14:24:44.527408
30482	BPY	2020-10-01	12.13	12.65	12.07	12.65	3164211	{}	\N	2020-10-12 14:24:44.531899
30483	BPY	2020-10-02	12.245	13.015	12.15	12.91	2478295	{}	\N	2020-10-12 14:24:44.53789
30484	BPY	2020-10-05	13.11	13.35	12.83	13.05	2784735	{}	\N	2020-10-12 14:24:44.544839
30485	BPY	2020-10-06	13.1574	13.36	12.82	12.86	3059696	{}	\N	2020-10-12 14:24:44.55001
30486	BPY	2020-10-07	12.9	13.56	12.9	13.53	3075286	{}	\N	2020-10-12 14:24:44.555766
30487	BPY	2020-10-08	13.61	13.98	13.61	13.88	2433502	{}	\N	2020-10-12 14:24:44.560917
30488	BPY	2020-10-09	14	14.058	13.72	13.79	2362871	{}	\N	2020-10-12 14:24:44.565716
30489	BPY	2020-10-12	13.77	13.96	13.75	13.93	334065	{}	\N	2020-10-12 14:24:44.570805
30490	BREW	2020-09-28	16.48	16.5	16.45	16.49	209636	{}	\N	2020-10-12 14:24:45.613114
30491	BREW	2020-09-29	16.49	16.51	16.49	16.51	480561	{}	\N	2020-10-12 14:24:45.620541
30492	BRKS	2020-09-28	46	46.42	45.485	46.26	349060	{}	\N	2020-10-12 14:24:47.029453
30493	BRKS	2020-09-29	46.37	47.78	46.37	47.05	334312	{}	\N	2020-10-12 14:24:47.037948
30494	BRKS	2020-09-30	46.97	47.29	46	46.26	427659	{}	\N	2020-10-12 14:24:47.044332
30495	BRKS	2020-10-01	46.96	48.47	46.6	48.4	385707	{}	\N	2020-10-12 14:24:47.050198
30496	BRKS	2020-10-02	46.84	47.885	46.84	47.14	295531	{}	\N	2020-10-12 14:24:47.055874
30497	BRKS	2020-10-05	47.59	49.229	47.59	49.22	256943	{}	\N	2020-10-12 14:24:47.061241
30498	BRKS	2020-10-06	49.22	51.595	49.12	49.89	458115	{}	\N	2020-10-12 14:24:47.066556
30499	BRKS	2020-10-07	50.43	51.05	50.02	50.74	371166	{}	\N	2020-10-12 14:24:47.070854
30500	BRKS	2020-10-08	51.4	51.62	50.78	51.3	402295	{}	\N	2020-10-12 14:24:47.075696
30501	BRKS	2020-10-09	51.99	52.78	51.85	52.55	307831	{}	\N	2020-10-12 14:24:47.080069
30502	BRKS	2020-10-12	53.34	53.34	52.89	52.91	21139	{}	\N	2020-10-12 14:24:47.08403
30503	BRT	2020-09-28	11.82	12.22	11.82	12.07	26798	{}	\N	2020-10-12 14:24:49.06334
30504	BRT	2020-09-29	12.07	12.07	11.71	11.89	37977	{}	\N	2020-10-12 14:24:49.071241
30505	BRT	2020-09-30	11.89	12.105	11.58	11.78	49895	{}	\N	2020-10-12 14:24:49.078037
30506	BRT	2020-10-01	11.87	12.29	11.755	12.26	64995	{}	\N	2020-10-12 14:24:49.08328
30507	BRT	2020-10-02	12.08	12.43	12.08	12.3	44877	{}	\N	2020-10-12 14:24:49.088487
30508	BRT	2020-10-05	12.39	12.629	12.07	12.23	98507	{}	\N	2020-10-12 14:24:49.092858
30509	BRT	2020-10-06	12.31	12.99	12.17	12.4	70463	{}	\N	2020-10-12 14:24:49.096973
30510	BRT	2020-10-07	12.49	12.72	12.28	12.64	50197	{}	\N	2020-10-12 14:24:49.101188
30511	BRT	2020-10-08	12.8	13.1782	12.8	12.84	93710	{}	\N	2020-10-12 14:24:49.105059
30512	BRT	2020-10-09	12.89	12.96	12.795	12.82	43700	{}	\N	2020-10-12 14:24:49.109201
30513	BRT	2020-10-12	13.04	13.04	12.81	12.81	11584	{}	\N	2020-10-12 14:24:49.113316
30514	BSCO	2020-09-28	22.15	22.19	22.14	22.14	185032	{}	\N	2020-10-12 14:24:50.077109
30515	BSCO	2020-09-29	22.17	22.19	22.152	22.17	144759	{}	\N	2020-10-12 14:24:50.081285
30516	BSCO	2020-09-30	22.16	22.19	22.1421	22.185	171431	{}	\N	2020-10-12 14:24:50.085536
30517	BSCO	2020-10-01	22.1638	22.2	22.1638	22.18	101508	{}	\N	2020-10-12 14:24:50.089795
30518	BSCO	2020-10-02	22.205	22.21	22.18	22.21	138780	{}	\N	2020-10-12 14:24:50.098009
30519	BSCO	2020-10-05	22.19	22.21	22.17	22.2	108131	{}	\N	2020-10-12 14:24:50.106032
30520	BSCO	2020-10-06	22.21	22.22	22.17	22.2	356210	{}	\N	2020-10-12 14:24:50.114419
30521	BSCO	2020-10-07	22.19	22.22	22.1703	22.21	258362	{}	\N	2020-10-12 14:24:50.122583
30522	BSCO	2020-10-08	22.21	22.24	22.2	22.23	121630	{}	\N	2020-10-12 14:24:50.128806
30523	BSCO	2020-10-09	22.21	22.24	22.21	22.23	120207	{}	\N	2020-10-12 14:24:50.134336
30524	BSCO	2020-10-12	22.24	22.26	22.2301	22.2547	10152	{}	\N	2020-10-12 14:24:50.138134
30525	BSJP	2020-09-28	23.29	23.4	23.29	23.37	74825	{}	\N	2020-10-12 14:24:51.43003
30526	BSJP	2020-09-29	23.33	23.39	23.3	23.37	32758	{}	\N	2020-10-12 14:24:51.434802
30527	BSJP	2020-09-30	23.37	23.51	23.37	23.4999	66282	{}	\N	2020-10-12 14:24:51.438305
30528	BSJP	2020-10-01	23.51	23.57	23.495	23.56	68134	{}	\N	2020-10-12 14:24:51.442778
30529	BSJP	2020-10-02	23.5	23.58	23.46	23.57	97450	{}	\N	2020-10-12 14:24:51.447243
30530	BSJP	2020-10-05	23.58	23.689	23.58	23.68	112914	{}	\N	2020-10-12 14:24:51.451918
30531	BSJP	2020-10-06	23.68	23.84	23.61	23.68	177008	{}	\N	2020-10-12 14:24:51.456017
30532	BSJP	2020-10-07	23.73	23.78	23.68	23.76	97670	{}	\N	2020-10-12 14:24:51.460842
30533	BSJP	2020-10-08	23.73	23.79	23.72	23.79	57342	{}	\N	2020-10-12 14:24:51.465059
30534	BSJP	2020-10-09	23.8	23.85	23.78	23.81	120183	{}	\N	2020-10-12 14:24:51.469644
30535	BSJP	2020-10-12	23.84	23.895	23.805	23.895	47374	{}	\N	2020-10-12 14:24:51.4734
30536	BSRR	2020-09-28	16.52	17.035	16.52	16.83	27171	{}	\N	2020-10-12 14:24:52.968817
30537	BSRR	2020-09-29	16.565	16.87	16.27	16.7	12252	{}	\N	2020-10-12 14:24:52.978587
30538	BSRR	2020-09-30	16.64	16.93	16.625	16.79	35490	{}	\N	2020-10-12 14:24:52.986007
30539	BSRR	2020-10-01	16.9	17.02	16.57	16.8	19874	{}	\N	2020-10-12 14:24:52.992322
30540	BSRR	2020-10-02	16.47	17.57	16.47	17.51	16697	{}	\N	2020-10-12 14:24:53.000203
30541	BSRR	2020-10-05	17.59	18.16	17.502	18.11	25416	{}	\N	2020-10-12 14:24:53.005759
30542	BSRR	2020-10-06	18.27	18.48	17.56	17.57	41004	{}	\N	2020-10-12 14:24:53.013588
30543	BSRR	2020-10-07	17.74	18.185	17.6	18.13	29596	{}	\N	2020-10-12 14:24:53.020449
30544	BSRR	2020-10-08	18.22	18.22	17.225	17.93	42281	{}	\N	2020-10-12 14:24:53.027039
30545	BSRR	2020-10-09	17.96	17.96	17.6	17.68	24999	{}	\N	2020-10-12 14:24:53.032165
30546	BSRR	2020-10-12	17.405	17.53	17.405	17.52	668	{}	\N	2020-10-12 14:24:53.038795
30547	BTG	2020-09-28	6.44	6.53	6.315	6.48	5918829	{}	\N	2020-10-12 14:24:54.012839
30548	BTG	2020-09-29	6.59	6.68	6.525	6.64	5517322	{}	\N	2020-10-12 14:24:54.018746
30549	BTG	2020-09-30	6.54	6.605	6.43	6.52	5269322	{}	\N	2020-10-12 14:24:54.024
30550	BTG	2020-10-01	6.58	6.7564	6.57	6.66	5622040	{}	\N	2020-10-12 14:24:54.028801
30551	BTG	2020-10-02	6.61	6.64	6.47	6.54	5030330	{}	\N	2020-10-12 14:24:54.035353
30552	BTG	2020-10-05	6.6	6.72	6.53	6.65	4612444	{}	\N	2020-10-12 14:24:54.040907
30553	BTG	2020-10-06	6.67	6.73	6.38	6.42	5661678	{}	\N	2020-10-12 14:24:54.045905
30554	BTG	2020-10-07	6.5	6.56	6.36	6.38	4584048	{}	\N	2020-10-12 14:24:54.051253
30555	BTG	2020-10-08	6.44	6.58	6.42	6.55	6349821	{}	\N	2020-10-12 14:24:54.055886
30556	BTG	2020-10-09	6.69	6.94	6.6702	6.94	7363301	{}	\N	2020-10-12 14:24:54.060331
30557	BTG	2020-10-12	7	7	6.86	6.94	981284	{}	\N	2020-10-12 14:24:54.064451
30558	BW	2020-09-28	2.37	2.47	2.345	2.44	32599	{}	\N	2020-10-12 14:24:55.594563
30559	BW	2020-09-29	2.46	2.46	2.24	2.34	140325	{}	\N	2020-10-12 14:24:55.601785
30560	BW	2020-09-30	2.34	2.45	2.26	2.32	127664	{}	\N	2020-10-12 14:24:55.608078
30561	BW	2020-10-01	2.31	2.39	2.28	2.39	106133	{}	\N	2020-10-12 14:24:55.614154
30562	BW	2020-10-02	2.33	2.43	2.32	2.4	206198	{}	\N	2020-10-12 14:24:55.621048
30563	BW	2020-10-05	2.43	2.47	2.36	2.42	220612	{}	\N	2020-10-12 14:24:55.627595
30564	BW	2020-10-06	2.45	2.49	2.32	2.36	257091	{}	\N	2020-10-12 14:24:55.632118
30565	BW	2020-10-07	2.4	2.5979	2.3973	2.48	393743	{}	\N	2020-10-12 14:24:55.637605
30566	BW	2020-10-08	2.52	2.64	2.4801	2.54	380856	{}	\N	2020-10-12 14:24:55.642194
30567	BW	2020-10-09	2.55	2.59	2.3893	2.47	361428	{}	\N	2020-10-12 14:24:55.646623
30568	BW	2020-10-12	2.48	2.48	2.4201	2.43	11188	{}	\N	2020-10-12 14:24:55.651282
30569	FLGT	2020-09-28	40	41.24	37.62	38.94	661640	{}	\N	2020-10-12 14:24:56.743997
30570	FLGT	2020-09-29	39	42.58	39	41.81	1007002	{}	\N	2020-10-12 14:24:56.751803
30571	FLGT	2020-09-30	42.06	42.87	39.7	40.03	845119	{}	\N	2020-10-12 14:24:56.757888
30572	FLGT	2020-10-01	40.35	40.9194	38.8438	40.62	545341	{}	\N	2020-10-12 14:24:56.76457
30573	FLGT	2020-10-02	39.41	40.81	39.25	39.97	550685	{}	\N	2020-10-12 14:24:56.76982
30574	FLGT	2020-10-05	40.59	47.16	40.55	46.98	1029628	{}	\N	2020-10-12 14:24:56.774418
30575	FLGT	2020-10-06	47	47.13	43.51	44.52	1054250	{}	\N	2020-10-12 14:24:56.779325
30576	FLGT	2020-10-07	45	46	43.85	44.98	586698	{}	\N	2020-10-12 14:24:56.784241
30577	FLGT	2020-10-08	45.9	46.4	44.19	44.97	585623	{}	\N	2020-10-12 14:24:56.788846
30578	FLGT	2020-10-09	44.68	44.68	43.62	44	652685	{}	\N	2020-10-12 14:24:56.793496
30579	FLGT	2020-10-12	45.3793	45.45	44.12	44.595	155248	{}	\N	2020-10-12 14:24:56.797935
30580	BPTH	2020-09-28	4.1	4.3	3.97	4.19	44551	{}	\N	2020-10-12 14:24:58.198975
30581	BPTH	2020-09-29	4.22	4.33	4.2	4.32	19876	{}	\N	2020-10-12 14:24:58.206761
30582	BPTH	2020-09-30	4.34	4.5	4.24	4.24	66445	{}	\N	2020-10-12 14:24:58.213092
30583	BPTH	2020-10-01	4.26	4.4734	4.21	4.34	17907	{}	\N	2020-10-12 14:24:58.220005
30584	BPTH	2020-10-02	4.19	4.34	4.09	4.14	22464	{}	\N	2020-10-12 14:24:58.226734
30585	BPTH	2020-10-05	4.3555	4.3555	4.1	4.23	41734	{}	\N	2020-10-12 14:24:58.232518
30586	BPTH	2020-10-06	4.3131	4.58	4.2512	4.3919	46741	{}	\N	2020-10-12 14:24:58.238317
30587	BPTH	2020-10-07	4.41	4.5	4.3	4.35	32165	{}	\N	2020-10-12 14:24:58.243711
30588	BPTH	2020-10-08	4.4	4.47	4.3065	4.43	17166	{}	\N	2020-10-12 14:24:58.24835
30589	BPTH	2020-10-09	4.4	4.5199	4.31	4.34	12639	{}	\N	2020-10-12 14:24:58.253084
30590	BPTH	2020-10-12	4.31	4.3168	4.17	4.2672	3022	{}	\N	2020-10-12 14:24:58.258657
30591	BRC	2020-09-28	39.49	39.98	39.44	39.59	312763	{}	\N	2020-10-12 14:24:59.318235
30592	BRC	2020-09-29	39.74	39.86	38.92	39.58	529721	{}	\N	2020-10-12 14:24:59.329785
30593	BRC	2020-09-30	39.57	40.32	39.52	40.02	723676	{}	\N	2020-10-12 14:24:59.337473
30594	BRC	2020-10-01	40.13	40.8	40.13	40.5	618644	{}	\N	2020-10-12 14:24:59.343647
30595	BRC	2020-10-02	39.9	41.8	39.73	41.63	622421	{}	\N	2020-10-12 14:24:59.348804
30596	BRC	2020-10-05	41.81	42.56	41.81	42.14	599228	{}	\N	2020-10-12 14:24:59.353173
30597	BRC	2020-10-06	42.59	43.49	41.92	42.35	603386	{}	\N	2020-10-12 14:24:59.358113
30598	BRC	2020-10-07	42.67	43.01	41.95	42.17	471715	{}	\N	2020-10-12 14:24:59.362467
30599	BRC	2020-10-08	42.37	42.785	41.92	42.43	349179	{}	\N	2020-10-12 14:24:59.367437
30600	BRC	2020-10-09	42.9	42.97	41.87	41.92	276738	{}	\N	2020-10-12 14:24:59.371788
30601	BRC	2020-10-12	41.85	41.92	41.48	41.58	18957	{}	\N	2020-10-12 14:24:59.376339
30602	BRKL	2020-09-28	8.68	8.81	8.54	8.74	282953	{}	\N	2020-10-12 14:25:00.683483
30603	BRKL	2020-09-29	8.69	8.7	8.46	8.64	207639	{}	\N	2020-10-12 14:25:00.69179
30604	BRKL	2020-09-30	8.77	8.82	8.56	8.65	244706	{}	\N	2020-10-12 14:25:00.697537
30605	BRKL	2020-10-01	8.58	8.72	8.465	8.675	334914	{}	\N	2020-10-12 14:25:00.7031
30606	BRKL	2020-10-02	8.54	8.88	8.54	8.79	519428	{}	\N	2020-10-12 14:25:00.708371
30607	BRKL	2020-10-05	8.9	9.105	8.86	9.07	403033	{}	\N	2020-10-12 14:25:00.712873
30608	BRKL	2020-10-06	9.14	9.46	9.09	9.12	339275	{}	\N	2020-10-12 14:25:00.717566
30609	BRKL	2020-10-07	9.19	9.39	9.091	9.22	427551	{}	\N	2020-10-12 14:25:00.722096
30610	BRKL	2020-10-08	9.28	9.48	9.25	9.4	222015	{}	\N	2020-10-12 14:25:00.72636
30611	BRKL	2020-10-09	9.44	9.49	9.3	9.3	299538	{}	\N	2020-10-12 14:25:00.730598
30612	BRKL	2020-10-12	9.26	9.39	9.25	9.37	54431	{}	\N	2020-10-12 14:25:00.734834
30613	BRPAW	2020-09-28	0.26	0.29	0.247	0.2889	4775	{}	\N	2020-10-12 14:25:02.696773
30614	BRPAW	2020-09-29	0.29	0.36	0.29	0.3401	17525	{}	\N	2020-10-12 14:25:02.705117
30615	BRPAW	2020-09-30	0.34	0.34	0.260101	0.260101	2000	{}	\N	2020-10-12 14:25:02.710846
30616	BRPAW	2020-10-01	0.25	0.25	0.25	0.25	228	{}	\N	2020-10-12 14:25:02.716269
30617	BRPAW	2020-10-02	0.25	0.25	0.25	0.25	104	{}	\N	2020-10-12 14:25:02.721225
30618	BRPAW	2020-10-05	0.24	0.27	0.24	0.27	1716	{}	\N	2020-10-12 14:25:02.725402
30619	BRPAW	2020-10-06	0.22	0.23	0.22	0.2201	11934	{}	\N	2020-10-12 14:25:02.729641
30620	BRPAW	2020-10-07	0.22	0.22	0.22	0.22	1442	{}	\N	2020-10-12 14:25:02.733794
30621	BRPAW	2020-10-08	0.2497	0.3	0.190078	0.2267	121884	{}	\N	2020-10-12 14:25:02.737505
30622	BRPAW	2020-10-09	0.19	0.19	0.1301	0.188066	196977	{}	\N	2020-10-12 14:25:02.74155
30623	BRPAW	2020-10-12	0.179999	0.18	0.173251	0.1799	9900	{}	\N	2020-10-12 14:25:02.745548
30624	BSAC	2020-09-28	13.97	14.05	13.64	13.64	283585	{}	\N	2020-10-12 14:25:03.969815
30625	BSAC	2020-09-29	13.66	13.68	13.335	13.47	938925	{}	\N	2020-10-12 14:25:03.980146
30626	BSAC	2020-09-30	13.59	13.93	13.59	13.86	1189799	{}	\N	2020-10-12 14:25:03.989191
30627	BSAC	2020-10-01	13.9	14.14	13.67	13.77	710180	{}	\N	2020-10-12 14:25:03.996153
30628	BSAC	2020-10-02	13.58	14.32	13.55	14.23	539569	{}	\N	2020-10-12 14:25:04.002854
30629	BSAC	2020-10-05	14.35	14.55	14.25	14.51	833081	{}	\N	2020-10-12 14:25:04.007818
30630	BSAC	2020-10-06	14.43	14.43	13.82	13.87	752513	{}	\N	2020-10-12 14:25:04.013656
30631	BSAC	2020-10-07	13.89	13.91	13.51	13.61	456149	{}	\N	2020-10-12 14:25:04.01877
30632	BSAC	2020-10-08	13.66	13.74	13.46	13.62	748509	{}	\N	2020-10-12 14:25:04.023438
30633	BSAC	2020-10-09	13.67	13.94	13.66	13.79	503887	{}	\N	2020-10-12 14:25:04.028697
30634	BSAC	2020-10-12	13.75	13.78	13.62	13.63	26296	{}	\N	2020-10-12 14:25:04.033308
30635	BSD	2020-09-28	13.37	13.42	13.3694	13.42	1911	{}	\N	2020-10-12 14:25:05.00272
30636	BSD	2020-09-29	13.44	13.44	13.4	13.4	7522	{}	\N	2020-10-12 14:25:05.011376
30637	BSD	2020-09-30	13.35	13.4064	13.32	13.33	50494	{}	\N	2020-10-12 14:25:05.018505
30638	BSD	2020-10-01	13.43	13.43	13.3774	13.39	26308	{}	\N	2020-10-12 14:25:05.024908
30639	BSD	2020-10-02	13.39	13.5	13.39	13.48	89097	{}	\N	2020-10-12 14:25:05.029719
30640	BSD	2020-10-05	13.48	13.532	13.41	13.45	49010	{}	\N	2020-10-12 14:25:05.034335
30641	BSD	2020-10-06	13.4	13.491	13.39	13.43	38500	{}	\N	2020-10-12 14:25:05.039114
30642	BSD	2020-10-07	13.47	13.48	13.4	13.45	35373	{}	\N	2020-10-12 14:25:05.043666
30643	BSD	2020-10-08	13.42	13.53	13.4	13.44	36144	{}	\N	2020-10-12 14:25:05.048135
30644	BSD	2020-10-09	13.45	13.53	13.42	13.48	34362	{}	\N	2020-10-12 14:25:05.053079
30645	BSD	2020-10-12	13.47	13.48	13.45	13.45	8152	{}	\N	2020-10-12 14:25:05.057709
30646	BSL	2020-09-28	13.74	13.77	13.68	13.7	31531	{}	\N	2020-10-12 14:25:06.181978
30647	BSL	2020-09-29	13.71	13.71	13.62	13.69	65414	{}	\N	2020-10-12 14:25:06.191876
30648	BSL	2020-09-30	13.75	13.77	13.69	13.76	76434	{}	\N	2020-10-12 14:25:06.198504
30649	BSL	2020-10-01	13.8	13.8	13.66	13.75	68001	{}	\N	2020-10-12 14:25:06.20465
30650	BSL	2020-10-02	13.7	13.81	13.67	13.81	66659	{}	\N	2020-10-12 14:25:06.209428
30651	BSL	2020-10-05	13.82	13.86	13.65	13.86	62524	{}	\N	2020-10-12 14:25:06.214064
30652	BSL	2020-10-06	13.82	13.8852	13.71	13.87	82182	{}	\N	2020-10-12 14:25:06.218593
30653	BSL	2020-10-07	13.76	13.83	13.68	13.72	195352	{}	\N	2020-10-12 14:25:06.223129
30654	BSL	2020-10-08	13.7	13.79	13.7	13.71	65752	{}	\N	2020-10-12 14:25:06.22716
30655	BSL	2020-10-09	13.75	13.78	13.71	13.71	70670	{}	\N	2020-10-12 14:25:06.23121
30656	BSL	2020-10-12	13.79	13.7901	13.78	13.79	6068	{}	\N	2020-10-12 14:25:06.235467
30657	BTE	2020-09-28	0.3329	0.3666	0.3326	0.3471	1450649	{}	\N	2020-10-12 14:25:07.141123
30658	BTE	2020-09-29	0.3552	0.3552	0.3275	0.3513	17021768	{}	\N	2020-10-12 14:25:07.148595
30659	BTE	2020-09-30	0.3528	0.3587	0.34105	0.3526	24166999	{}	\N	2020-10-12 14:25:07.154378
30660	BTE	2020-10-01	0.35	0.3523	0.3295	0.3306	3268671	{}	\N	2020-10-12 14:25:07.15905
30661	BTE	2020-10-02	0.314	0.351	0.3029	0.346	2321173	{}	\N	2020-10-12 14:25:07.163454
30662	BTE	2020-10-05	0.3459	0.3643	0.3459	0.3592	1792722	{}	\N	2020-10-12 14:25:07.168021
30663	BTE	2020-10-06	0.363	0.3719	0.3404	0.3466	2430940	{}	\N	2020-10-12 14:25:07.172345
30664	BTE	2020-10-07	0.35	0.35	0.3359	0.346	2907881	{}	\N	2020-10-12 14:25:07.176156
30665	BTE	2020-10-08	0.3351	0.379	0.3351	0.3777	3173174	{}	\N	2020-10-12 14:25:07.1818
30666	BTE	2020-10-09	0.394	0.3941	0.3655	0.3748	7700896	{}	\N	2020-10-12 14:25:07.185634
30667	BTE	2020-10-12	0.3748	0.3748	0.3561	0.3561	298462	{}	\N	2020-10-12 14:25:07.189351
30668	BUY	2020-09-28	16.11	16.25	15.68	15.9059	9880	{}	\N	2020-10-12 14:25:09.300779
30669	BUY	2020-09-29	16.19	16.19	15.56	15.6471	1896	{}	\N	2020-10-12 14:25:09.308156
30670	BUY	2020-09-30	16.12	16.2715	15.995	15.995	1253	{}	\N	2020-10-12 14:25:09.31304
30671	BUY	2020-10-01	16.54	16.54	16.1756	16.1756	593	{}	\N	2020-10-12 14:25:09.319756
30672	BUY	2020-10-02	16.04	16.3101	16.03	16.3101	2636	{}	\N	2020-10-12 14:25:09.328113
30673	BUY	2020-10-05	17.065	17.065	17.065	17.065	53	{}	\N	2020-10-12 14:25:09.34403
30674	BUY	2020-10-06	16.925	16.925	16.925	16.925	214	{}	\N	2020-10-12 14:25:09.350265
30675	BUY	2020-10-07	17.37	17.37	17.37	17.37	75	{}	\N	2020-10-12 14:25:09.355922
30676	BUY	2020-10-09	17.6516	17.6516	17.6516	17.6516	0	{}	\N	2020-10-12 14:25:09.360429
30677	BUY	2020-10-12	17.45	17.45	17.45	17.45	221	{}	\N	2020-10-12 14:25:09.364965
30678	BWFG	2020-09-28	14.98	15.36	14.84	14.85	6462	{}	\N	2020-10-12 14:25:10.309402
30679	BWFG	2020-09-29	15.04	15.05	14.25	14.42	11269	{}	\N	2020-10-12 14:25:10.317689
30680	BWFG	2020-09-30	14.3678	14.56	14.08	14.15	12385	{}	\N	2020-10-12 14:25:10.330129
30681	BWFG	2020-10-01	14.43	14.99	14.25	14.35	15673	{}	\N	2020-10-12 14:25:10.337601
30682	BWFG	2020-10-02	14.2	15	14.2	14.96	4438	{}	\N	2020-10-12 14:25:10.344505
30683	BWFG	2020-10-05	15.34	15.34	15.02	15.03	8679	{}	\N	2020-10-12 14:25:10.349916
30684	BWFG	2020-10-06	15.04	15.25	15	15.03	14960	{}	\N	2020-10-12 14:25:10.354229
30685	BWFG	2020-10-07	15.175	15.3	15.15	15.15	20317	{}	\N	2020-10-12 14:25:10.359441
30686	BWFG	2020-10-08	15.33	15.33	15.06	15.06	13183	{}	\N	2020-10-12 14:25:10.364329
30687	BWFG	2020-10-09	15.2	15.3	15.2	15.3	4004	{}	\N	2020-10-12 14:25:10.370132
30688	BWFG	2020-10-12	15.11	15.4999	15.11	15.4999	3370	{}	\N	2020-10-12 14:25:10.375533
30689	BXMX	2020-09-28	11.18	11.21	11.14	11.16	228312	{}	\N	2020-10-12 14:25:11.327165
30690	BXMX	2020-09-29	11.16	11.2	11.12	11.18	530920	{}	\N	2020-10-12 14:25:11.337282
30691	BXMX	2020-09-30	11.18	11.32	11.161	11.28	734698	{}	\N	2020-10-12 14:25:11.343139
30692	BXMX	2020-10-01	11.39	11.42	11.31	11.37	869800	{}	\N	2020-10-12 14:25:11.347829
30693	BXMX	2020-10-02	11.26	11.32	11.25	11.32	378322	{}	\N	2020-10-12 14:25:11.352637
30694	BXMX	2020-10-05	11.37	11.42	11.33	11.42	661188	{}	\N	2020-10-12 14:25:11.356627
30695	BXMX	2020-10-06	11.42	11.48	11.315	11.33	703870	{}	\N	2020-10-12 14:25:11.360796
30696	BXMX	2020-10-07	11.38	11.48	11.38	11.48	545473	{}	\N	2020-10-12 14:25:11.364442
30697	BXMX	2020-10-08	11.5	11.62	11.4941	11.62	645283	{}	\N	2020-10-12 14:25:11.368192
30698	BXMX	2020-10-09	11.65	11.7	11.593	11.67	603802	{}	\N	2020-10-12 14:25:11.371793
30699	BXMX	2020-10-12	11.68	11.7399	11.68	11.7199	97046	{}	\N	2020-10-12 14:25:11.37611
30700	BYM	2020-09-28	14.02	14.1	14.01	14.1	37703	{}	\N	2020-10-12 14:25:12.771389
30701	BYM	2020-09-29	14.14	14.23	14.0914	14.2	86049	{}	\N	2020-10-12 14:25:12.780243
30702	BYM	2020-09-30	14.23	14.24	14.16	14.24	77468	{}	\N	2020-10-12 14:25:12.786604
30703	BYM	2020-10-01	14.28	14.3499	14.172	14.29	73554	{}	\N	2020-10-12 14:25:12.791538
30704	BYM	2020-10-02	14.29	14.42	14.224	14.32	66458	{}	\N	2020-10-12 14:25:12.796092
30705	BYM	2020-10-05	14.33	14.33	14.28	14.31	66830	{}	\N	2020-10-12 14:25:12.800826
30706	BYM	2020-10-06	14.33	14.43	14.27	14.32	108138	{}	\N	2020-10-12 14:25:12.804729
30707	BYM	2020-10-07	14.32	14.345	14.25	14.25	111752	{}	\N	2020-10-12 14:25:12.808848
30708	BYM	2020-10-08	14.27	14.3	14.2	14.24	57398	{}	\N	2020-10-12 14:25:12.812942
30709	BYM	2020-10-09	14.27	14.3155	14.12	14.12	140810	{}	\N	2020-10-12 14:25:12.816978
30710	BYM	2020-10-12	14.17	14.21	14.17	14.1834	9126	{}	\N	2020-10-12 14:25:12.821656
30711	BPMP	2020-09-28	9.63	9.75	9.32	9.67	222066	{}	\N	2020-10-12 14:25:14.187524
30712	BPMP	2020-09-29	9.58	10.19	9.555	10.03	413600	{}	\N	2020-10-12 14:25:14.196535
30713	BPMP	2020-09-30	10.09	10.26	9.91	9.91	560472	{}	\N	2020-10-12 14:25:14.203118
30714	BPMP	2020-10-01	9.91	10.1	9.59	9.9	336661	{}	\N	2020-10-12 14:25:14.208491
30715	BPMP	2020-10-02	9.73	9.87	9.62	9.84	355884	{}	\N	2020-10-12 14:25:14.213168
30716	BPMP	2020-10-05	9.98	10.18	9.77	10.09	224938	{}	\N	2020-10-12 14:25:14.218218
30717	BPMP	2020-10-06	10.2	10.2	9.56	9.67	320657	{}	\N	2020-10-12 14:25:14.222322
30718	BPMP	2020-10-07	9.78	9.85	9.55	9.61	263022	{}	\N	2020-10-12 14:25:14.226099
30719	BPMP	2020-10-08	9.66	9.9684	9.6	9.88	386446	{}	\N	2020-10-12 14:25:14.229895
30720	BPMP	2020-10-09	9.99	10.25	9.86	10.06	549232	{}	\N	2020-10-12 14:25:14.233631
30721	BPMP	2020-10-12	10	10.14	9.927	9.95	45512	{}	\N	2020-10-12 14:25:14.237496
30722	BRID	2020-09-28	17.605	18.37	17.605	18.16	5437	{}	\N	2020-10-12 14:25:15.823346
30723	BRID	2020-09-29	18.46	18.83	18.13	18.36	26045	{}	\N	2020-10-12 14:25:15.830855
30724	BRID	2020-09-30	18.5	18.975	18.31	18.31	2597	{}	\N	2020-10-12 14:25:15.836393
30725	BRID	2020-10-01	18.48	19.045	18.4501	18.52	9248	{}	\N	2020-10-12 14:25:15.841698
30726	BRID	2020-10-02	18.5	18.5	18.25	18.2999	814	{}	\N	2020-10-12 14:25:15.846482
30727	BRID	2020-10-05	19.1456	19.1456	18.33	18.33	3925	{}	\N	2020-10-12 14:25:15.85085
30728	BRID	2020-10-06	18.06	18.06	18.06	18.06	889	{}	\N	2020-10-12 14:25:15.854916
30729	BRID	2020-10-07	17.955	18.05	17.955	18.05	1097	{}	\N	2020-10-12 14:25:15.858751
30730	BRID	2020-10-08	18.16	18.53	18.02	18.02	2829	{}	\N	2020-10-12 14:25:15.862695
30731	BRID	2020-10-09	18.35	18.7	18.35	18.7	736	{}	\N	2020-10-12 14:25:15.866609
30732	BRQS	2020-09-28	1.07	1.07	0.9901	1.02	615909	{}	\N	2020-10-12 14:25:17.682499
30733	BRQS	2020-09-29	1.05	1.05	1.01	1.03	278029	{}	\N	2020-10-12 14:25:17.690622
30734	BRQS	2020-09-30	1	1.03	1	1.01	290842	{}	\N	2020-10-12 14:25:17.696986
30735	BRQS	2020-10-01	1.03	1.03	0.9501	0.9805	271098	{}	\N	2020-10-12 14:25:17.702842
30736	BRQS	2020-10-02	0.96	1.01	0.95	0.9701	224140	{}	\N	2020-10-12 14:25:17.707949
30737	BRQS	2020-10-05	1.01	1.13	0.9799	1.03	1895391	{}	\N	2020-10-12 14:25:17.712537
30738	BRQS	2020-10-06	1.07	1.17	1.02	1.05	1588258	{}	\N	2020-10-12 14:25:17.717206
30739	BRQS	2020-10-07	1.01	1.08	1.01	1.04	268417	{}	\N	2020-10-12 14:25:17.72188
30740	BRQS	2020-10-08	1.08	1.085	1.02	1.05	230997	{}	\N	2020-10-12 14:25:17.726409
30741	BRQS	2020-10-09	1.07	1.25	1.06	1.18	2510039	{}	\N	2020-10-12 14:25:17.730924
30742	BRQS	2020-10-12	1.25	1.38	1.19	1.23	1491811	{}	\N	2020-10-12 14:25:17.735456
30743	BSCK	2020-09-28	21.22	21.22	21.21	21.22	298724	{}	\N	2020-10-12 14:25:18.771996
30744	BSCK	2020-09-29	21.22	21.23	21.21	21.22	327887	{}	\N	2020-10-12 14:25:18.781923
30745	BSCK	2020-09-30	21.22	21.23	21.2	21.22	489732	{}	\N	2020-10-12 14:25:18.788619
30746	BSCK	2020-10-01	21.22	21.2299	21.2	21.2	238556	{}	\N	2020-10-12 14:25:18.794397
30747	BSCK	2020-10-02	21.2	21.21	21.195	21.21	327652	{}	\N	2020-10-12 14:25:18.799614
30748	BSCK	2020-10-05	21.22	21.22	21.21	21.22	358501	{}	\N	2020-10-12 14:25:18.8043
30749	BSCK	2020-10-06	21.21	21.22	21.2	21.2122	1976128	{}	\N	2020-10-12 14:25:18.808254
30750	BSCK	2020-10-07	21.21	21.23	21.21	21.215	315760	{}	\N	2020-10-12 14:25:18.812538
30751	BSCK	2020-10-08	21.22	21.23	21.21	21.21	378636	{}	\N	2020-10-12 14:25:18.816958
30752	BSCK	2020-10-09	21.22	21.23	21.21	21.23	208960	{}	\N	2020-10-12 14:25:18.821117
30753	BSCK	2020-10-12	21.23	21.2398	21.22	21.225	24538	{}	\N	2020-10-12 14:25:18.82503
30754	BSET	2020-09-28	13.37	13.95	13.235	13.34	61690	{}	\N	2020-10-12 14:25:22.468773
30755	BSET	2020-09-29	13.45	13.765	13.4	13.7	55073	{}	\N	2020-10-12 14:25:22.476399
30756	BSET	2020-09-30	13.76	14.28	13.26	13.68	119062	{}	\N	2020-10-12 14:25:22.483863
30757	BSET	2020-10-01	14.41	15.9	13.91	15.66	194545	{}	\N	2020-10-12 14:25:22.491255
30758	BSET	2020-10-02	15.02	16.22	14.83	15.15	149775	{}	\N	2020-10-12 14:25:22.498275
30759	BSET	2020-10-05	15.38	16.14	14.81	15.89	132039	{}	\N	2020-10-12 14:25:22.504321
30760	BSET	2020-10-06	16.1	16.42	15.48	15.52	74303	{}	\N	2020-10-12 14:25:22.510163
30761	BSET	2020-10-07	15.85	16.79	15.7	16.45	76604	{}	\N	2020-10-12 14:25:22.515444
30762	BSET	2020-10-08	16.45	16.46	15.515	15.77	77440	{}	\N	2020-10-12 14:25:22.519661
30763	BSET	2020-10-09	15.94	16.42	15.44	15.6	97602	{}	\N	2020-10-12 14:25:22.523768
30764	BSET	2020-10-12	15.75	16.3	15.41	15.88	8741	{}	\N	2020-10-12 14:25:22.528062
30765	BTAL	2020-09-28	23.54	23.6257	23.4	23.46	27963	{}	\N	2020-10-12 14:25:25.275782
30766	BTAL	2020-09-29	23.6	23.7849	23.465	23.7	394677	{}	\N	2020-10-12 14:25:25.285862
30767	BTAL	2020-09-30	23.47	23.61	23.3885	23.59	37637	{}	\N	2020-10-12 14:25:25.292368
30768	BTAL	2020-10-01	23.75	23.75	23.41	23.49	74362	{}	\N	2020-10-12 14:25:25.29866
30769	BTAL	2020-10-02	23.58	23.58	23.0911	23.1	108710	{}	\N	2020-10-12 14:25:25.30412
30770	BTAL	2020-10-05	22.98	23.07	22.84	23.03	42122	{}	\N	2020-10-12 14:25:25.308806
30771	BTAL	2020-10-06	23.23	23.23	22.71	23.07	109663	{}	\N	2020-10-12 14:25:25.31305
30772	BTAL	2020-10-07	22.94	23.0602	22.81	22.83	40915	{}	\N	2020-10-12 14:25:25.3178
30773	BTAL	2020-10-08	22.75	22.7939	22.6201	22.665	79714	{}	\N	2020-10-12 14:25:25.321919
30774	BTAL	2020-10-09	22.67	22.78	22.65	22.7	64213	{}	\N	2020-10-12 14:25:25.331299
30775	BTAL	2020-10-12	22.56	22.925	22.56	22.875	15067	{}	\N	2020-10-12 14:25:25.33874
30776	BUI	2020-09-28	21.2	21.47	21.19	21.31	51894	{}	\N	2020-10-12 14:25:27.001816
30777	BUI	2020-09-29	21.32	21.54	21.28	21.5	52350	{}	\N	2020-10-12 14:25:27.01141
30778	BUI	2020-09-30	21.66	22.1065	21.515	21.77	130989	{}	\N	2020-10-12 14:25:27.017586
30779	BUI	2020-10-01	21.71	22.1	21.71	21.92	95806	{}	\N	2020-10-12 14:25:27.023883
30780	BUI	2020-10-02	21.82	21.89	21.67	21.75	86312	{}	\N	2020-10-12 14:25:27.02887
30781	BUI	2020-10-05	21.91	22.4499	21.855	22.3234	135050	{}	\N	2020-10-12 14:25:27.033461
30782	BUI	2020-10-06	22.3	22.45	22.25	22.39	116883	{}	\N	2020-10-12 14:25:27.037599
30783	BUI	2020-10-07	22.45	22.47	22.3	22.42	84615	{}	\N	2020-10-12 14:25:27.041715
30784	BUI	2020-10-08	22.61	22.74	22.44	22.54	171134	{}	\N	2020-10-12 14:25:27.045419
30785	BUI	2020-10-09	22.72	22.9	22.18	22.28	209870	{}	\N	2020-10-12 14:25:27.049534
30786	BUI	2020-10-12	22.35	22.54	22.35	22.4545	17558	{}	\N	2020-10-12 14:25:27.053729
30787	BWXT	2020-09-28	57.74	58.615	57.74	57.86	184405	{}	\N	2020-10-12 14:25:28.551589
30788	BWXT	2020-09-29	57.62	57.85	57.05	57.27	423096	{}	\N	2020-10-12 14:25:28.561591
30789	BWXT	2020-09-30	57.47	57.9	56.02	56.31	691015	{}	\N	2020-10-12 14:25:28.567886
30790	BWXT	2020-10-01	56.38	56.97	55.65	56.65	839768	{}	\N	2020-10-12 14:25:28.573362
30791	BWXT	2020-10-02	56.105	57.37	55.92	56.42	940358	{}	\N	2020-10-12 14:25:28.57881
30792	BWXT	2020-10-05	56.89	57.38	56.02	56.18	727404	{}	\N	2020-10-12 14:25:28.583548
30793	BWXT	2020-10-06	56.37	56.67	54.88	55	1286860	{}	\N	2020-10-12 14:25:28.587534
30794	BWXT	2020-10-07	55.3	55.76	54.69	55.44	1365368	{}	\N	2020-10-12 14:25:28.591455
30795	BWXT	2020-10-08	55.64	58.02	55.64	57.97	838776	{}	\N	2020-10-12 14:25:28.595155
30796	BWXT	2020-10-09	57.84	58.56	57.42	57.85	897884	{}	\N	2020-10-12 14:25:28.598894
30797	BWXT	2020-10-12	58.16	58.61	57.9	58.32	46523	{}	\N	2020-10-12 14:25:28.60323
30798	BXS	2020-09-28	18.96	19.39	18.92	19.31	329630	{}	\N	2020-10-12 14:25:30.597364
30799	BXS	2020-09-29	19.43	19.43	18.87	19.19	922902	{}	\N	2020-10-12 14:25:30.607794
30800	BXS	2020-09-30	19.39	19.66	19.18	19.38	712494	{}	\N	2020-10-12 14:25:30.614825
30801	BXS	2020-10-01	19.43	19.84	19.14	19.83	1253939	{}	\N	2020-10-12 14:25:30.621625
30802	BXS	2020-10-02	19.4	20.85	18.77	20.66	1244112	{}	\N	2020-10-12 14:25:30.6267
30803	BXS	2020-10-05	20.94	21.57	20.81	21.55	900730	{}	\N	2020-10-12 14:25:30.631475
30804	BXS	2020-10-06	21.93	22.7	21.6	21.64	1488156	{}	\N	2020-10-12 14:25:30.636047
30805	BXS	2020-10-07	21.86	22.4	21.86	22.24	1057330	{}	\N	2020-10-12 14:25:30.640519
30806	BXS	2020-10-08	22.53	22.85	22.32	22.62	841251	{}	\N	2020-10-12 14:25:30.644607
30807	BXS	2020-10-09	22.76	22.94	22.18	22.22	703807	{}	\N	2020-10-12 14:25:30.649187
30808	BXS	2020-10-12	22.15	22.38	22	22.34	70708	{}	\N	2020-10-12 14:25:30.653821
30809	CACC	2020-09-28	309.58	322.58	307.74	321	242191	{}	\N	2020-10-12 14:25:31.625939
30810	CACC	2020-09-29	319.76	334	319.76	333	286591	{}	\N	2020-10-12 14:25:31.633821
30811	CACC	2020-09-30	330.8	342.235	329.56	338.64	327053	{}	\N	2020-10-12 14:25:31.63967
30812	CACC	2020-10-01	346.32	348.6	338.03	347.13	188566	{}	\N	2020-10-12 14:25:31.645154
30813	CACC	2020-10-02	340.19	360.83	336.01	353.93	222869	{}	\N	2020-10-12 14:25:31.650302
30814	CACC	2020-10-05	359.23	362.8	348.3	350	120356	{}	\N	2020-10-12 14:25:31.654965
30815	CACC	2020-10-06	353.79	358.83	333.77	334.91	116239	{}	\N	2020-10-12 14:25:31.6596
30816	CACC	2020-10-07	338	345.43	337.49	342.85	94057	{}	\N	2020-10-12 14:25:31.664602
30817	CACC	2020-10-08	343.02	359.88	343.02	356.44	108252	{}	\N	2020-10-12 14:25:31.669024
30818	CACC	2020-10-09	363.39	364.34	351.3	353.82	71757	{}	\N	2020-10-12 14:25:31.673182
30819	CACC	2020-10-12	353.81	358.84	348.035	349.09	10836	{}	\N	2020-10-12 14:25:31.677606
30820	CAL	2020-09-28	9.64	10.02	9.61	9.66	530979	{}	\N	2020-10-12 14:25:32.732556
30821	CAL	2020-09-29	9.66	9.66	9.0999	9.42	1012152	{}	\N	2020-10-12 14:25:32.742028
30822	CAL	2020-09-30	9.52	9.93	9.46	9.56	802655	{}	\N	2020-10-12 14:25:32.749117
30823	CAL	2020-10-01	9.6	9.89	9.52	9.8	732538	{}	\N	2020-10-12 14:25:32.755938
30824	CAL	2020-10-02	9.32	10.355	9.2	10.26	925990	{}	\N	2020-10-12 14:25:32.761992
30825	CAL	2020-10-05	10.4	10.52	9.87	10.17	1271221	{}	\N	2020-10-12 14:25:32.766263
30826	CAL	2020-10-06	10.28	10.48	9.62	9.68	1113011	{}	\N	2020-10-12 14:25:32.770943
30827	CAL	2020-10-07	9.95	10.09	9.74	9.93	728070	{}	\N	2020-10-12 14:25:32.775248
30828	CAL	2020-10-08	10	10.23	9.75	10.17	643614	{}	\N	2020-10-12 14:25:32.779671
30829	CAL	2020-10-09	10.2	10.41	9.86	10.07	659487	{}	\N	2020-10-12 14:25:32.783819
30830	CAL	2020-10-12	9.94	10.37	9.85	10.12	125656	{}	\N	2020-10-12 14:25:32.787714
30831	BSCN	2020-09-28	21.78	21.8	21.77	21.8	200834	{}	\N	2020-10-12 14:25:34.984368
30832	BSCN	2020-09-29	21.79	21.81	21.79	21.81	167521	{}	\N	2020-10-12 14:25:34.989847
30833	BSCN	2020-09-30	21.78	21.819	21.78	21.79	168375	{}	\N	2020-10-12 14:25:34.995421
30834	BSCN	2020-10-01	21.79	21.95	21.79	21.81	194959	{}	\N	2020-10-12 14:25:35.001556
30835	BSCN	2020-10-02	21.82	21.83	21.79	21.83	182975	{}	\N	2020-10-12 14:25:35.009943
30836	BSCN	2020-10-05	21.83	21.84	21.81	21.83	145247	{}	\N	2020-10-12 14:25:35.017148
30837	BSCN	2020-10-06	21.83	21.84	21.81	21.83	317317	{}	\N	2020-10-12 14:25:35.025328
30838	BSCN	2020-10-07	21.85	21.85	21.82	21.84	262059	{}	\N	2020-10-12 14:25:35.035082
30839	BSCN	2020-10-08	21.83	21.85	21.82	21.83	163318	{}	\N	2020-10-12 14:25:35.042515
30840	BSCN	2020-10-09	21.84	21.86	21.83	21.85	162836	{}	\N	2020-10-12 14:25:35.051168
30841	BSCN	2020-10-12	21.85	21.8593	21.85	21.85	14130	{}	\N	2020-10-12 14:25:35.058082
30842	BSE	2020-09-28	13.17	13.17	13.07	13.08	8019	{}	\N	2020-10-12 14:25:36.463729
30843	BSE	2020-09-29	13.11	13.16	13.1	13.1	11340	{}	\N	2020-10-12 14:25:36.474755
30844	BSE	2020-09-30	13.06	13.105	13.06	13.07	6276	{}	\N	2020-10-12 14:25:36.481928
30845	BSE	2020-10-01	13.1	13.2	13.0787	13.08	15340	{}	\N	2020-10-12 14:25:36.488199
30846	BSE	2020-10-02	13.03	13.15	13.03	13.11	62238	{}	\N	2020-10-12 14:25:36.493234
30847	BSE	2020-10-05	13.08	13.11	13.07	13.1	16938	{}	\N	2020-10-12 14:25:36.497664
30848	BSE	2020-10-06	13.15	13.1667	13.06	13.1	20386	{}	\N	2020-10-12 14:25:36.502593
30849	BSE	2020-10-07	13.05	13.1	13.05	13.1	14006	{}	\N	2020-10-12 14:25:36.50641
30850	BSE	2020-10-08	13.105	13.12	13.0701	13.12	12410	{}	\N	2020-10-12 14:25:36.510225
30851	BSE	2020-10-09	13.11	13.11	13.01	13.0972	28438	{}	\N	2020-10-12 14:25:36.514203
30852	BSE	2020-10-12	13.2	13.2	13.1	13.12	6690	{}	\N	2020-10-12 14:25:36.51884
30853	BSM	2020-09-28	5.96	6.29	5.96	6.19	490315	{}	\N	2020-10-12 14:25:37.796802
30854	BSM	2020-09-29	6.2	6.25	6.01	6.15	934165	{}	\N	2020-10-12 14:25:37.805833
30855	BSM	2020-09-30	6.15	6.21	6.05	6.14	539602	{}	\N	2020-10-12 14:25:37.812049
30856	BSM	2020-10-01	6.09	6.1399	5.92	5.98	502139	{}	\N	2020-10-12 14:25:37.818841
30857	BSM	2020-10-02	5.83	6.11	5.83	5.95	429227	{}	\N	2020-10-12 14:25:37.824164
30858	BSM	2020-10-05	6.01	6.26	5.96	6.25	518777	{}	\N	2020-10-12 14:25:37.828379
30859	BSM	2020-10-06	6.26	6.3499	6.1	6.14	522267	{}	\N	2020-10-12 14:25:37.832978
30860	BSM	2020-10-07	6.17	6.2	6.04	6.11	340061	{}	\N	2020-10-12 14:25:37.837916
30861	BSM	2020-10-08	6.22	6.38	6.12	6.38	592159	{}	\N	2020-10-12 14:25:37.842335
30862	BSM	2020-10-09	6.38	6.3864	6.2	6.3	1070946	{}	\N	2020-10-12 14:25:37.846825
30863	BSM	2020-10-12	6.3	6.42	6.22	6.335	91074	{}	\N	2020-10-12 14:25:37.851893
30864	BSV	2020-09-28	83.02	83.04	82.965	83.04	1447232	{}	\N	2020-10-12 14:25:43.901582
30865	BSV	2020-09-29	83.06	83.06	83.03	83.05	1081704	{}	\N	2020-10-12 14:25:43.912024
30866	BSV	2020-09-30	83.05	83.06	83.02	83.05	1766674	{}	\N	2020-10-12 14:25:43.918229
30867	BSV	2020-10-01	82.9	82.95	82.89	82.95	1399175	{}	\N	2020-10-12 14:25:43.924397
30868	BSV	2020-10-02	82.96	82.96	82.91	82.92	1560319	{}	\N	2020-10-12 14:25:43.929514
30869	BSV	2020-10-05	82.93	82.94	82.89	82.9	1386728	{}	\N	2020-10-12 14:25:43.933884
30870	BSV	2020-10-06	82.91	82.91	82.86	82.86	1812659	{}	\N	2020-10-12 14:25:43.937872
30871	BSV	2020-10-07	82.87	82.8752	82.84	82.85	3964578	{}	\N	2020-10-12 14:25:43.941948
30872	BSV	2020-10-08	82.9	82.94	82.86	82.87	1643282	{}	\N	2020-10-12 14:25:43.945732
30873	BSV	2020-10-09	82.87	82.907	82.86	82.9	1747332	{}	\N	2020-10-12 14:25:43.949909
30874	BSV	2020-10-12	82.91	82.92	82.9	82.91	157920	{}	\N	2020-10-12 14:25:43.95384
30875	BUD	2020-09-28	54.67	54.92	54.22	54.38	1429678	{}	\N	2020-10-12 14:25:44.926166
30876	BUD	2020-09-29	54.43	54.67	54.11	54.28	2601048	{}	\N	2020-10-12 14:25:44.935875
30877	BUD	2020-09-30	54.33	55.11	53.51	53.88	3215717	{}	\N	2020-10-12 14:25:44.943098
30878	BUD	2020-10-01	53.82	53.95	53.24	53.6	2116886	{}	\N	2020-10-12 14:25:44.949067
30879	BUD	2020-10-02	53.01	54.32	52.93	54	2248717	{}	\N	2020-10-12 14:25:44.953981
30880	BUD	2020-10-05	55.23	55.6399	54.84	55.29	2036288	{}	\N	2020-10-12 14:25:44.959326
30881	BUD	2020-10-06	57.41	57.96	56.5401	56.82	4462985	{}	\N	2020-10-12 14:25:44.96361
30882	BUD	2020-10-07	58.5	58.82	58.155	58.44	3722221	{}	\N	2020-10-12 14:25:44.967647
30883	BUD	2020-10-08	59	59.685	58.661	59.63	3484072	{}	\N	2020-10-12 14:25:44.971634
30884	BUD	2020-10-09	59.6	59.85	59.14	59.5	2315290	{}	\N	2020-10-12 14:25:44.976409
30885	BUD	2020-10-12	59.62	60.07	59.6	60.035	458737	{}	\N	2020-10-12 14:25:44.980419
30886	BVXV	2020-09-28	40	41.7306	39.12	39.95	30184	{}	\N	2020-10-12 14:25:46.186581
30887	BVXV	2020-09-29	39.95	40.5	38.42	38.79	43450	{}	\N	2020-10-12 14:25:46.195418
30888	BVXV	2020-09-30	38.78	39.97	38.45	39.09	36196	{}	\N	2020-10-12 14:25:46.201799
30889	BVXV	2020-10-01	40	40.92	38.56	38.57	56470	{}	\N	2020-10-12 14:25:46.20755
30890	BVXV	2020-10-02	38.2	39.5	37.5	39.45	35566	{}	\N	2020-10-12 14:25:46.213196
30891	BVXV	2020-10-05	40.49	40.5	37.81	38.38	40607	{}	\N	2020-10-12 14:25:46.217718
30892	BVXV	2020-10-06	39.46	39.46	37.56	38.58	28088	{}	\N	2020-10-12 14:25:46.222147
30893	BVXV	2020-10-07	38.14	39.9457	38	39.77	33541	{}	\N	2020-10-12 14:25:46.226745
30894	BVXV	2020-10-08	39.8	39.8	37.51	38.07	56782	{}	\N	2020-10-12 14:25:46.230755
30895	BVXV	2020-10-09	38.07	38.49	37.0219	37.55	30671	{}	\N	2020-10-12 14:25:46.235477
30896	BVXV	2020-10-12	37.55	37.8	35.73	36.06	29832	{}	\N	2020-10-12 14:25:46.240032
30897	BY	2020-09-28	11.18	11.61	11.18	11.41	67095	{}	\N	2020-10-12 14:25:49.378444
30898	BY	2020-09-29	11.41	11.41	11.08	11.35	70353	{}	\N	2020-10-12 14:25:49.38848
30899	BY	2020-09-30	11.36	11.575	11.13	11.28	112363	{}	\N	2020-10-12 14:25:49.395659
30900	BY	2020-10-01	11.32	11.49	11.07	11.48	74947	{}	\N	2020-10-12 14:25:49.401584
30901	BY	2020-10-02	11.08	11.71	11.07	11.66	64836	{}	\N	2020-10-12 14:25:49.406413
30902	BY	2020-10-05	11.8	12.34	11.75	12.22	111402	{}	\N	2020-10-12 14:25:49.411013
30903	BY	2020-10-06	12.44	12.94	12.27	12.34	81798	{}	\N	2020-10-12 14:25:49.415608
30904	BY	2020-10-07	12.56	12.88	12.37	12.7	91803	{}	\N	2020-10-12 14:25:49.419743
30905	BY	2020-10-08	12.79	13.07	12.57	12.93	89801	{}	\N	2020-10-12 14:25:49.423714
30906	BY	2020-10-09	12.98	12.98	12.64	12.69	46410	{}	\N	2020-10-12 14:25:49.42764
30907	BY	2020-10-12	12.65	12.65	12.65	12.65	968	{}	\N	2020-10-12 14:25:49.431858
30908	BZM	2020-09-28	14.1	14.1	13.94	14.01	2462	{}	\N	2020-10-12 14:25:51.035192
30909	BZM	2020-09-29	14.11	14.11	13.995	14.0723	12560	{}	\N	2020-10-12 14:25:51.045761
30910	BZM	2020-09-30	14.07	14.07	13.89	13.9866	5874	{}	\N	2020-10-12 14:25:51.05249
30911	BZM	2020-10-01	14.36	14.36	14.36	14.36	580	{}	\N	2020-10-12 14:25:51.058197
30912	BZM	2020-10-02	14.29	14.29	13.85	14.05	6406	{}	\N	2020-10-12 14:25:51.062863
30913	BZM	2020-10-05	14.1	14.1	14.0428	14.1	2184	{}	\N	2020-10-12 14:25:51.06751
30914	BZM	2020-10-06	14.0423	14.0423	13.834	13.85	88341	{}	\N	2020-10-12 14:25:51.071264
30915	BZM	2020-10-07	13.85	13.9861	13.85	13.85	13522	{}	\N	2020-10-12 14:25:51.075254
30916	BZM	2020-10-08	13.9038	13.9175	13.88	13.88	8038	{}	\N	2020-10-12 14:25:51.079627
30917	BZM	2020-10-09	13.9	14.02	13.88	13.88	31174	{}	\N	2020-10-12 14:25:51.083672
30918	BZM	2020-10-12	13.91	13.91	13.91	13.91	400	{}	\N	2020-10-12 14:25:51.087614
30919	CAG	2020-09-28	35.31	35.75	35.16	35.46	2170604	{}	\N	2020-10-12 14:25:52.518903
30920	CAG	2020-09-29	35.43	35.745	35.105	35.48	5028333	{}	\N	2020-10-12 14:25:52.528818
30921	CAG	2020-09-30	35.5	35.99	35.34	35.71	7884413	{}	\N	2020-10-12 14:25:52.535842
30922	CAG	2020-10-01	36.16	36.51	34.53	35.87	12101378	{}	\N	2020-10-12 14:25:52.542036
30923	CAG	2020-10-02	36.12	37.175	36.055	36.88	10301283	{}	\N	2020-10-12 14:25:52.547337
30924	CAG	2020-10-05	36.75	37.165	36.63	37.1	4614624	{}	\N	2020-10-12 14:25:52.552071
30925	CAG	2020-10-06	37.05	37.19	36.63	36.85	3873201	{}	\N	2020-10-12 14:25:52.556568
30926	CAG	2020-10-07	37	37.56	36.855	37.38	5753727	{}	\N	2020-10-12 14:25:52.560743
30927	CAG	2020-10-08	37.43	37.73	37.085	37.25	3574758	{}	\N	2020-10-12 14:25:52.565036
30928	CAG	2020-10-09	37.43	37.745	37.2	37.32	4626621	{}	\N	2020-10-12 14:25:52.569058
30929	CAG	2020-10-12	37.52	37.81	37.25	37.77	1182555	{}	\N	2020-10-12 14:25:52.573102
30930	CAMP	2020-09-28	7.31	7.815	7.31	7.62	298649	{}	\N	2020-10-12 14:25:53.767038
30931	CAMP	2020-09-29	7.645	7.76	7.36	7.41	201167	{}	\N	2020-10-12 14:25:53.774656
30932	CAMP	2020-09-30	7.5899	7.5899	7.16	7.19	276963	{}	\N	2020-10-12 14:25:53.78039
30933	CAMP	2020-10-01	7.24	7.42	7.04	7.37	196802	{}	\N	2020-10-12 14:25:53.787223
30934	CAMP	2020-10-02	7.19	7.57	7.01	7.46	139364	{}	\N	2020-10-12 14:25:53.792729
30935	CAMP	2020-10-05	7.57	7.75	7.5	7.61	131496	{}	\N	2020-10-12 14:25:53.798258
30936	CAMP	2020-10-06	7.69	7.89	7.6	7.615	334589	{}	\N	2020-10-12 14:25:53.804406
30937	CAMP	2020-10-07	7.69	7.725	7.47	7.565	170132	{}	\N	2020-10-12 14:25:53.809098
30938	CAMP	2020-10-08	7.7	7.81	7.4906	7.67	165934	{}	\N	2020-10-12 14:25:53.815268
30939	CAMP	2020-10-09	7.74	7.98	7.71	7.93	173818	{}	\N	2020-10-12 14:25:53.821639
30940	CAMP	2020-10-12	7.98	8	7.9	7.93	11211	{}	\N	2020-10-12 14:25:53.827083
30941	BSCR	2020-09-28	21.92	22	21.88	21.92	59413	{}	\N	2020-10-12 14:25:55.702979
30942	BSCR	2020-09-29	21.99	22	21.955	22	22775	{}	\N	2020-10-12 14:25:55.712479
30943	BSCR	2020-09-30	21.98	21.99	21.935	21.99	17698	{}	\N	2020-10-12 14:25:55.719996
30944	BSCR	2020-10-01	21.9567	22.03	21.9469	22.03	24029	{}	\N	2020-10-12 14:25:55.725883
30945	BSCR	2020-10-02	21.94	22.03	21.94	22.03	42111	{}	\N	2020-10-12 14:25:55.731278
30946	BSCR	2020-10-05	21.99	22.01	21.96	21.98	53916	{}	\N	2020-10-12 14:25:55.736869
30947	BSCR	2020-10-06	21.97	22.02	21.95	21.99	110793	{}	\N	2020-10-12 14:25:55.741321
30948	BSCR	2020-10-07	21.97	22.01	21.94	21.98	34026	{}	\N	2020-10-12 14:25:55.745796
30949	BSCR	2020-10-08	21.95	22.03	21.95	22.03	15813	{}	\N	2020-10-12 14:25:55.750531
30950	BSCR	2020-10-09	22.02	22.04	21.995	22.04	22556	{}	\N	2020-10-12 14:25:55.754953
30951	BSCR	2020-10-12	22.05	22.07	22.0364	22.07	1689	{}	\N	2020-10-12 14:25:55.759279
30952	BSMX	2020-09-28	3.06	3.19	3.06	3.16	339063	{}	\N	2020-10-12 14:25:57.814863
30953	BSMX	2020-09-29	3.2	3.2	3.1	3.1	437076	{}	\N	2020-10-12 14:25:57.82404
30954	BSMX	2020-09-30	3.12	3.21	3.12	3.2	540806	{}	\N	2020-10-12 14:25:57.830462
30955	BSMX	2020-10-01	3.23	3.27	3.14	3.18	793418	{}	\N	2020-10-12 14:25:57.836339
30956	BSMX	2020-10-02	3.14	3.24	3.12	3.24	952629	{}	\N	2020-10-12 14:25:57.841575
30957	BSMX	2020-10-05	3.26	3.3	3.205	3.27	2335668	{}	\N	2020-10-12 14:25:57.847216
30958	BSMX	2020-10-06	3.29	3.36	3.21	3.22	705777	{}	\N	2020-10-12 14:25:57.852218
30959	BSMX	2020-10-07	3.22	3.2899	3.17	3.21	515766	{}	\N	2020-10-12 14:25:57.857549
30960	BSMX	2020-10-08	3.26	3.445	3.21	3.44	812092	{}	\N	2020-10-12 14:25:57.861957
30961	BSMX	2020-10-09	3.45	3.755	3.4	3.63	1897851	{}	\N	2020-10-12 14:25:57.866187
30962	BSMX	2020-10-12	3.6	3.71	3.57	3.69	333521	{}	\N	2020-10-12 14:25:57.870862
30963	BTO	2020-09-28	20.76	21.3253	20.5528	21.15	58099	{}	\N	2020-10-12 14:25:59.401635
30964	BTO	2020-09-29	21.17	21.19	20.4272	21.08	165620	{}	\N	2020-10-12 14:25:59.411871
30965	BTO	2020-09-30	21.39	21.69	21.19	21.43	221903	{}	\N	2020-10-12 14:25:59.418748
30966	BTO	2020-10-01	21.54	21.65	20.93	21.23	250583	{}	\N	2020-10-12 14:25:59.426487
30967	BTO	2020-10-02	20.84	21.92	20.84	21.75	137832	{}	\N	2020-10-12 14:25:59.431932
30968	BTO	2020-10-05	21.94	22.52	21.7439	22.44	144600	{}	\N	2020-10-12 14:25:59.436505
30969	BTO	2020-10-06	22.6	23.485	22.52	22.62	143860	{}	\N	2020-10-12 14:25:59.441749
30970	BTO	2020-10-07	22.85	23.56	22.85	23.23	113543	{}	\N	2020-10-12 14:25:59.446446
30971	BTO	2020-10-08	23.27	23.722	23.27	23.68	166200	{}	\N	2020-10-12 14:25:59.450866
30972	BTO	2020-10-09	23.8	23.8	23.23	23.34	103066	{}	\N	2020-10-12 14:25:59.456034
30973	BTO	2020-10-12	23.36	23.56	23.34	23.4645	12682	{}	\N	2020-10-12 14:25:59.461165
30974	BUSE	2020-09-28	16.05	16.43	15.97	16.17	176195	{}	\N	2020-10-12 14:26:04.785274
30975	BUSE	2020-09-29	16.08	16.12	15.6	16.08	117598	{}	\N	2020-10-12 14:26:04.795913
30976	BUSE	2020-09-30	16.25	16.27	15.81	15.89	213385	{}	\N	2020-10-12 14:26:04.802513
30977	BUSE	2020-10-01	15.89	16.24	15.73	16.23	148341	{}	\N	2020-10-12 14:26:04.809358
30978	BUSE	2020-10-02	16	16.55	15.95	16.47	157782	{}	\N	2020-10-12 14:26:04.815507
30979	BUSE	2020-10-05	16.72	16.97	16.52	16.91	136119	{}	\N	2020-10-12 14:26:04.8216
30980	BUSE	2020-10-06	17.1	17.62	16.93	16.96	113061	{}	\N	2020-10-12 14:26:04.826622
30981	BUSE	2020-10-07	17.17	17.44	17.06	17.26	192406	{}	\N	2020-10-12 14:26:04.831428
30982	BUSE	2020-10-08	17.42	17.6	17.17	17.5	115439	{}	\N	2020-10-12 14:26:04.836336
30983	BUSE	2020-10-09	17.72	17.72	17.26	17.34	112612	{}	\N	2020-10-12 14:26:04.840954
30984	BUSE	2020-10-12	17.31	17.395	17.19	17.34	23256	{}	\N	2020-10-12 14:26:04.845654
30985	BWA	2020-09-28	37.67	38.295	37.52	38.01	3175071	{}	\N	2020-10-12 14:26:07.057888
30986	BWA	2020-09-29	38.07	38.47	37.94	38.03	3217704	{}	\N	2020-10-12 14:26:07.065156
30987	BWA	2020-09-30	38.33	39.06	38.09	38.74	7932287	{}	\N	2020-10-12 14:26:07.071112
30988	BWA	2020-10-01	39.27	39.95	39.085	39.54	12865390	{}	\N	2020-10-12 14:26:07.076558
30989	BWA	2020-10-02	38.59	39.49	37.42	39.2	8259355	{}	\N	2020-10-12 14:26:07.082037
30990	BWA	2020-10-05	39.67	41.34	39.34	40.88	9056304	{}	\N	2020-10-12 14:26:07.087241
30991	BWA	2020-10-06	41.39	41.76	39.91	39.94	24892402	{}	\N	2020-10-12 14:26:07.09329
30992	BWA	2020-10-07	40.59	41.94	40.455	41.54	6583546	{}	\N	2020-10-12 14:26:07.098113
30993	BWA	2020-10-08	42	42.24	39.86	40.09	8155209	{}	\N	2020-10-12 14:26:07.104425
30994	BWA	2020-10-09	40.25	40.92	39.57	39.62	6460764	{}	\N	2020-10-12 14:26:07.136112
30995	BWA	2020-10-12	40	40.64	39.915	40.04	740795	{}	\N	2020-10-12 14:26:07.145741
30996	BXP	2020-09-28	82.01	84.04	81.89	82.73	1227757	{}	\N	2020-10-12 14:26:09.015886
30997	BXP	2020-09-29	81.31	81.765	79	80.28	1776356	{}	\N	2020-10-12 14:26:09.023817
30998	BXP	2020-09-30	80.82	82.11	79.28	80.3	2545785	{}	\N	2020-10-12 14:26:09.028959
30999	BXP	2020-10-01	80.6	82.9	80.37	82.83	1665831	{}	\N	2020-10-12 14:26:09.033545
31000	BXP	2020-10-02	81.24	85.54	80.715	85.27	2013647	{}	\N	2020-10-12 14:26:09.038674
31001	BXP	2020-10-05	85.92	86.34	83.32	84.5	1784290	{}	\N	2020-10-12 14:26:09.043003
31002	BXP	2020-10-06	85.09	85.71	82.61	82.94	2555553	{}	\N	2020-10-12 14:26:09.047223
31003	BXP	2020-10-07	83.19	83.99	82.02	82.38	2383131	{}	\N	2020-10-12 14:26:09.052146
31004	BXP	2020-10-08	83.07	84.75	82.74	84.17	3087292	{}	\N	2020-10-12 14:26:09.056852
31005	BXP	2020-10-09	84.5	84.5	82	82.78	1921827	{}	\N	2020-10-12 14:26:09.061155
31006	BXP	2020-10-12	82.78	82.95	81.855	82.75	115751	{}	\N	2020-10-12 14:26:09.06541
31007	BYSI	2020-09-28	13.21	13.85	13.0501	13.67	91797	{}	\N	2020-10-12 14:26:10.072234
31008	BYSI	2020-09-29	13.53	13.88	13.43	13.56	83437	{}	\N	2020-10-12 14:26:10.080545
31009	BYSI	2020-09-30	13.57	13.73	13.26	13.31	47905	{}	\N	2020-10-12 14:26:10.086413
31010	BYSI	2020-10-01	13.38	13.99	13.2085	13.8	57087	{}	\N	2020-10-12 14:26:10.09153
31011	BYSI	2020-10-02	13.7	13.705	13.07	13.47	62025	{}	\N	2020-10-12 14:26:10.096388
31012	BYSI	2020-10-05	13.57	14.45	13.57	14.37	83121	{}	\N	2020-10-12 14:26:10.101057
31013	BYSI	2020-10-06	14.44	14.93	14.25	14.6	58038	{}	\N	2020-10-12 14:26:10.105399
31014	BYSI	2020-10-07	14.7	15.15	14.7	14.97	64720	{}	\N	2020-10-12 14:26:10.109852
31015	BYSI	2020-10-08	15.15	15.25	14.685	15.05	43113	{}	\N	2020-10-12 14:26:10.114367
31016	BYSI	2020-10-09	15.02	15.4	14.9481	15.36	62853	{}	\N	2020-10-12 14:26:10.119778
31017	BYSI	2020-10-12	15.31	15.31	15.1	15.29	2085	{}	\N	2020-10-12 14:26:10.124229
31018	C	2020-09-28	42.92	43.88	42.89	43.34	20819956	{}	\N	2020-10-12 14:26:11.468959
31019	C	2020-09-29	43.1	43.29	42.02	42.42	41065916	{}	\N	2020-10-12 14:26:11.477729
31020	C	2020-09-30	42.58	43.59	42.5	43.11	44511032	{}	\N	2020-10-12 14:26:11.484443
31021	C	2020-10-01	43.33	43.65	42.6	43.44	42323006	{}	\N	2020-10-12 14:26:11.490787
31022	C	2020-10-02	42.54	43.87	42.52	43.66	46705360	{}	\N	2020-10-12 14:26:11.496862
31023	C	2020-10-05	44.26	45.23	44.21	44.91	48545854	{}	\N	2020-10-12 14:26:11.50266
31024	C	2020-10-06	45.475	46.06	44.16	44.41	57403343	{}	\N	2020-10-12 14:26:11.508124
31025	C	2020-10-07	45.02	45.63	44.61	44.84	43716518	{}	\N	2020-10-12 14:26:11.513965
31026	C	2020-10-08	44.91	45.17	44.28	44.72	57323681	{}	\N	2020-10-12 14:26:11.52821
31027	C	2020-10-09	45.01	45.59	44.69	44.93	50505015	{}	\N	2020-10-12 14:26:11.543875
31028	C	2020-10-12	45.09	45.67	44.95	45.5801	13119294	{}	\N	2020-10-12 14:26:11.551935
31029	CACI	2020-09-28	219.03	221.55	217.375	217.68	105068	{}	\N	2020-10-12 14:26:12.488453
31030	CACI	2020-09-29	218.29	218.29	213.44	214.8	286228	{}	\N	2020-10-12 14:26:12.494432
31031	CACI	2020-09-30	214.37	216.23	211.94	213.16	257734	{}	\N	2020-10-12 14:26:12.499685
31032	CACI	2020-10-01	213.81	215.72	210.7	215.23	301227	{}	\N	2020-10-12 14:26:12.504943
31033	CACI	2020-10-02	211.01	217.47	211.01	215.58	262140	{}	\N	2020-10-12 14:26:12.510391
31034	CACI	2020-10-05	217.33	218.43	213.18	215.09	627990	{}	\N	2020-10-12 14:26:12.516262
31035	CACI	2020-10-06	215.99	218.3	212.3	212.5	258926	{}	\N	2020-10-12 14:26:12.521083
31036	CACI	2020-10-07	213.8	215.34	209.63	211.59	467470	{}	\N	2020-10-12 14:26:12.526045
31037	CACI	2020-10-08	213.63	215.535	210.42	211.08	283306	{}	\N	2020-10-12 14:26:12.530853
31038	CACI	2020-10-09	212.57	216.59	211.86	213.44	300178	{}	\N	2020-10-12 14:26:12.535704
31039	CACI	2020-10-12	215.56	215.67	213.8	214.47	22760	{}	\N	2020-10-12 14:26:12.540733
31040	CAH	2020-09-28	48	48.54	47.46	47.55	1768345	{}	\N	2020-10-12 14:26:13.522324
31041	CAH	2020-09-29	47.63	47.68	46.74	46.89	3075027	{}	\N	2020-10-12 14:26:13.532808
31042	CAH	2020-09-30	46.59	47.41	46.342	46.95	6374840	{}	\N	2020-10-12 14:26:13.54012
31043	CAH	2020-10-01	47.19	47.2	45.81	46.08	4113232	{}	\N	2020-10-12 14:26:13.546203
31044	CAH	2020-10-02	45.52	46.765	45.29	46.3	3406498	{}	\N	2020-10-12 14:26:13.551376
31045	CAH	2020-10-05	46.5	47.43	46.07	47.15	5658460	{}	\N	2020-10-12 14:26:13.556376
31046	CAH	2020-10-06	47.59	47.8	46.32	46.47	3628820	{}	\N	2020-10-12 14:26:13.560791
31047	CAH	2020-10-07	46.55	47.43	46.54	47.15	3459616	{}	\N	2020-10-12 14:26:13.564954
31048	CAH	2020-10-08	47.29	48.73	46.8	48.68	3465306	{}	\N	2020-10-12 14:26:13.569196
31049	CAH	2020-10-09	48.92	48.93	47.81	48.03	3942340	{}	\N	2020-10-12 14:26:13.573383
31050	CAH	2020-10-12	48.23	48.43	47.88	48.256	305370	{}	\N	2020-10-12 14:26:13.577476
31051	CALM	2020-09-28	40.59	41.485	38.66	39.23	710602	{}	\N	2020-10-12 14:26:16.548529
31052	CALM	2020-09-29	39.61	39.95	38.3	38.57	516649	{}	\N	2020-10-12 14:26:16.558564
31053	CALM	2020-09-30	38.5	39.25	38.18	38.37	507030	{}	\N	2020-10-12 14:26:16.565229
31054	CALM	2020-10-01	38.18	38.68	37.77	38.4	367502	{}	\N	2020-10-12 14:26:16.571418
31055	CALM	2020-10-02	38.2	38.6	37.83	38.4	266370	{}	\N	2020-10-12 14:26:16.576287
31056	CALM	2020-10-05	38.835	38.835	37.86	38.05	267572	{}	\N	2020-10-12 14:26:16.581191
31057	CALM	2020-10-06	38.28	38.5499	37.91	38.35	353164	{}	\N	2020-10-12 14:26:16.585375
31058	CALM	2020-10-07	38.61	38.63	37.85	38.44	283658	{}	\N	2020-10-12 14:26:16.589741
31059	CALM	2020-10-08	38.72	38.84	37.86	38.3	216189	{}	\N	2020-10-12 14:26:16.593777
31060	CALM	2020-10-09	38.35	38.548	37.84	38.01	157209	{}	\N	2020-10-12 14:26:16.598473
31061	CALM	2020-10-12	38.06	38.1001	37.75	37.87	18922	{}	\N	2020-10-12 14:26:16.603381
31062	CAPL	2020-09-28	15.23	15.4	14.81	14.99	30950	{}	\N	2020-10-12 14:26:17.605859
31063	CAPL	2020-09-29	14.86	15.17	14.75	15.1699	97296	{}	\N	2020-10-12 14:26:17.616258
31064	CAPL	2020-09-30	14.945	15.25	14.71	14.87	81022	{}	\N	2020-10-12 14:26:17.622781
31065	CAPL	2020-10-01	14.83	14.9	14.55	14.9	37354	{}	\N	2020-10-12 14:26:17.628704
31066	CAPL	2020-10-02	14.5	15.2353	14.5	15.2	33926	{}	\N	2020-10-12 14:26:17.634294
31067	CAPL	2020-10-05	15.25	15.6	15.25	15.58	30976	{}	\N	2020-10-12 14:26:17.639202
31068	CAPL	2020-10-06	15.56	15.6773	15.0001	15.18	42812	{}	\N	2020-10-12 14:26:17.643619
31069	CAPL	2020-10-07	15.18	15.3	14.63	14.95	97110	{}	\N	2020-10-12 14:26:17.647958
31070	CAPL	2020-10-08	15.2632	15.2632	14.94	15.05	31572	{}	\N	2020-10-12 14:26:17.652428
31071	CAPL	2020-10-09	15.05	15.223	14.7	14.86	49542	{}	\N	2020-10-12 14:26:17.656902
31072	CAPL	2020-10-12	14.78	14.99	14.78	14.88	6636	{}	\N	2020-10-12 14:26:17.661356
31073	CATB	2020-09-28	6.45	6.49	6.06	6.24	216618	{}	\N	2020-10-12 14:26:18.942166
31074	CATB	2020-09-29	6.2	6.34	6.045	6.11	191531	{}	\N	2020-10-12 14:26:18.95201
31075	CATB	2020-09-30	6.13	6.29	6.06	6.19	124364	{}	\N	2020-10-12 14:26:18.958825
31076	CATB	2020-10-01	6.7	6.87	6.2	6.3	725643	{}	\N	2020-10-12 14:26:18.96454
31077	CATB	2020-10-02	6.24	6.3	6.02	6.13	319507	{}	\N	2020-10-12 14:26:18.969676
31078	CATB	2020-10-05	6.14	6.55	6.1101	6.44	222679	{}	\N	2020-10-12 14:26:18.974706
31079	CATB	2020-10-06	6.58	6.747	6.44	6.51	271234	{}	\N	2020-10-12 14:26:18.978942
31080	CATB	2020-10-07	6.62	6.73	6.482	6.63	195760	{}	\N	2020-10-12 14:26:18.983693
31081	CATB	2020-10-08	6.72	6.85	6.37	6.54	164894	{}	\N	2020-10-12 14:26:18.988091
31082	CATB	2020-10-09	6.54	6.59	6.23	6.28	156590	{}	\N	2020-10-12 14:26:18.992521
31083	CATB	2020-10-12	6.25	6.3385	6.12	6.32	63474	{}	\N	2020-10-12 14:26:18.99769
31084	BST	2020-09-28	41.91	42.5	41.77	42.32	83989	{}	\N	2020-10-12 14:26:20.456205
31085	BST	2020-09-29	42.31	42.72	42.245	42.72	130634	{}	\N	2020-10-12 14:26:20.466687
31086	BST	2020-09-30	42.91	43.449	42.7056	43.05	134018	{}	\N	2020-10-12 14:26:20.473498
31087	BST	2020-10-01	43.42	44.05	43.28	43.97	166855	{}	\N	2020-10-12 14:26:20.479715
31088	BST	2020-10-02	43.2	43.827	43.2	43.6	183223	{}	\N	2020-10-12 14:26:20.484835
31089	BST	2020-10-05	43.7	44.21	43.61	44.02	202224	{}	\N	2020-10-12 14:26:20.489495
31090	BST	2020-10-06	44.06	44.74	43.946	44.24	153295	{}	\N	2020-10-12 14:26:20.493647
31091	BST	2020-10-07	44.5	45.18	44.365	44.81	214756	{}	\N	2020-10-12 14:26:20.498207
31092	BST	2020-10-08	45.15	45.3	44.89	44.99	112500	{}	\N	2020-10-12 14:26:20.502626
31093	BST	2020-10-09	45.28	45.47	44.98	45.4	190689	{}	\N	2020-10-12 14:26:20.506597
31094	BST	2020-10-12	45.6	45.9493	45.5	45.55	128098	{}	\N	2020-10-12 14:26:20.510749
31095	BTN	2020-09-28	1.56	1.5814	1.5	1.54	8037	{}	\N	2020-10-12 14:26:24.404671
31096	BTN	2020-09-29	1.56	1.66	1.56	1.6262	6504	{}	\N	2020-10-12 14:26:24.413896
31097	BTN	2020-09-30	1.5401	1.6	1.5401	1.56	4592	{}	\N	2020-10-12 14:26:24.419827
31098	BTN	2020-10-01	1.573	1.6	1.57	1.57	4895	{}	\N	2020-10-12 14:26:24.425091
31099	BTN	2020-10-02	1.49	1.57	1.49	1.55	2799	{}	\N	2020-10-12 14:26:24.429717
31100	BTN	2020-10-05	1.5	1.57	1.5	1.52	6850	{}	\N	2020-10-12 14:26:24.43387
31101	BTN	2020-10-06	1.53	1.5919	1.53	1.53	8991	{}	\N	2020-10-12 14:26:24.437859
31102	BTN	2020-10-07	1.53	1.5899	1.5	1.55	10093	{}	\N	2020-10-12 14:26:24.441696
31103	BTN	2020-10-08	1.6	1.6	1.51	1.6	19142	{}	\N	2020-10-12 14:26:24.44565
31104	BTN	2020-10-09	1.65	1.7044	1.57	1.6	72071	{}	\N	2020-10-12 14:26:24.449639
31105	BTN	2020-10-12	1.65	1.65	1.61	1.61	6879	{}	\N	2020-10-12 14:26:24.453414
31106	BURL	2020-09-28	203.06	207.44	202.8	206.95	1025745	{}	\N	2020-10-12 14:26:25.678483
31107	BURL	2020-09-29	207.27	208.61	201.47	204.75	1266270	{}	\N	2020-10-12 14:26:25.688379
31108	BURL	2020-09-30	204.53	210.55	203.45	206.09	1497420	{}	\N	2020-10-12 14:26:25.694947
31109	BURL	2020-10-01	209.28	212.55	206.93	208.54	1259297	{}	\N	2020-10-12 14:26:25.701121
31110	BURL	2020-10-02	204.31	218.6	204.16	216.95	1660167	{}	\N	2020-10-12 14:26:25.706324
31111	BURL	2020-10-05	218.86	220.78	214.91	217.65	970412	{}	\N	2020-10-12 14:26:25.710902
31112	BURL	2020-10-06	217.79	219.15	211.01	211.45	998661	{}	\N	2020-10-12 14:26:25.715231
31113	BURL	2020-10-07	215.44	221.13	214.69	219.02	793358	{}	\N	2020-10-12 14:26:25.720323
31114	BURL	2020-10-08	222.37	223.5	217.04	218.45	775219	{}	\N	2020-10-12 14:26:25.724275
31115	BURL	2020-10-09	220.25	221.64	217.9	218.95	824678	{}	\N	2020-10-12 14:26:25.728446
31116	BURL	2020-10-12	219.14	220.555	217.35	217.35	77798	{}	\N	2020-10-12 14:26:25.73263
31117	BVN	2020-09-28	12.24	12.4	12.015	12.11	983758	{}	\N	2020-10-12 14:26:27.391562
31118	BVN	2020-09-29	12.16	12.41	12.08	12.14	2541280	{}	\N	2020-10-12 14:26:27.404162
31119	BVN	2020-09-30	12.08	12.31	11.865	12.22	2238214	{}	\N	2020-10-12 14:26:27.41127
31120	BVN	2020-10-01	12.37	12.39	12.07	12.14	1991721	{}	\N	2020-10-12 14:26:27.417764
31121	BVN	2020-10-02	12	12.23	11.88	12	1113970	{}	\N	2020-10-12 14:26:27.42319
31122	BVN	2020-10-05	12.08	12.405	12	12.31	1191425	{}	\N	2020-10-12 14:26:27.428014
31123	BVN	2020-10-06	12.45	12.45	11.92	12.03	1513505	{}	\N	2020-10-12 14:26:27.432218
31124	BVN	2020-10-07	12.09	12.41	12.03	12.25	1447305	{}	\N	2020-10-12 14:26:27.436431
31125	BVN	2020-10-08	12.44	12.53	12.25	12.37	1308378	{}	\N	2020-10-12 14:26:27.440633
31126	BVN	2020-10-09	12.5	12.66	12.44	12.6	2753225	{}	\N	2020-10-12 14:26:27.44489
31127	BVN	2020-10-12	12.63	12.66	12.41	12.56	140805	{}	\N	2020-10-12 14:26:27.449236
31128	BXC	2020-09-28	19.24	20.26	19.24	19.6	98033	{}	\N	2020-10-12 14:26:28.734343
31129	BXC	2020-09-29	19.55	21.16	19.16	20.68	298734	{}	\N	2020-10-12 14:26:28.744992
31130	BXC	2020-09-30	20.67	21.79	20.67	21.53	266097	{}	\N	2020-10-12 14:26:28.751896
31131	BXC	2020-10-01	21.402	21.85	20.81	21.55	216998	{}	\N	2020-10-12 14:26:28.758339
31132	BXC	2020-10-02	20.92	22.86	20.57	22.71	503974	{}	\N	2020-10-12 14:26:28.764002
31133	BXC	2020-10-05	23	24.81	22.75	24.6	489301	{}	\N	2020-10-12 14:26:28.76939
31134	BXC	2020-10-06	24.98	26.735	24.8	25.44	467170	{}	\N	2020-10-12 14:26:28.773701
31135	BXC	2020-10-07	25.83	28.26	25.83	27.05	461901	{}	\N	2020-10-12 14:26:28.778341
31136	BXC	2020-10-08	27.43	27.86	26.3649	27.61	510260	{}	\N	2020-10-12 14:26:28.782592
31137	BXC	2020-10-09	27.7	27.87	26.13	26.46	386919	{}	\N	2020-10-12 14:26:28.786601
31138	BXC	2020-10-12	26.51	26.72	26.065	26.26	115453	{}	\N	2020-10-12 14:26:28.790556
31139	CAE	2020-09-28	14.39	14.78	14.39	14.67	390044	{}	\N	2020-10-12 14:26:30.032226
31140	CAE	2020-09-29	14.7	14.7	14.35	14.42	1494236	{}	\N	2020-10-12 14:26:30.042896
31141	CAE	2020-09-30	14.4	14.8	14.4	14.62	1190664	{}	\N	2020-10-12 14:26:30.049825
31142	CAE	2020-10-01	15.24	16.03	15.08	15.29	1629234	{}	\N	2020-10-12 14:26:30.055852
31143	CAE	2020-10-02	14.9	15.345	14.9	15.32	787105	{}	\N	2020-10-12 14:26:30.061821
31144	CAE	2020-10-05	15.54	15.69	15.39	15.53	709026	{}	\N	2020-10-12 14:26:30.067047
31145	CAE	2020-10-06	15.62	15.76	15.32	15.44	884071	{}	\N	2020-10-12 14:26:30.071412
31146	CAE	2020-10-07	15.64	16	15.64	15.81	731344	{}	\N	2020-10-12 14:26:30.075743
31147	CAE	2020-10-08	16	16.1	15.85	16.04	457501	{}	\N	2020-10-12 14:26:30.080194
31148	CAE	2020-10-09	16.12	16.31	15.92	16.12	483336	{}	\N	2020-10-12 14:26:30.084826
31149	CAE	2020-10-12	16.11	16.13	15.95	15.95	17646	{}	\N	2020-10-12 14:26:30.089391
31150	CANF	2020-09-28	1.71	1.7599	1.71	1.72	114836	{}	\N	2020-10-12 14:26:31.540895
31151	CANF	2020-09-29	1.72	1.78	1.71	1.78	135581	{}	\N	2020-10-12 14:26:31.551955
31152	CANF	2020-09-30	1.77	1.8	1.74	1.77	165286	{}	\N	2020-10-12 14:26:31.56019
31153	CANF	2020-10-01	1.79	1.79	1.7055	1.73	106062	{}	\N	2020-10-12 14:26:31.566218
31154	CANF	2020-10-02	1.7	1.74	1.68	1.72	181923	{}	\N	2020-10-12 14:26:31.57252
31155	CANF	2020-10-05	1.72	1.77	1.72	1.75	157256	{}	\N	2020-10-12 14:26:31.577511
31156	CANF	2020-10-06	1.95	1.95	1.81	1.84	846703	{}	\N	2020-10-12 14:26:31.581823
31157	CANF	2020-10-07	1.83	1.84	1.76	1.7899	324677	{}	\N	2020-10-12 14:26:31.586425
31158	CANF	2020-10-08	1.77	1.785	1.7588	1.77	98722	{}	\N	2020-10-12 14:26:31.591832
31159	CANF	2020-10-09	1.78	1.83	1.76	1.8	143919	{}	\N	2020-10-12 14:26:31.596318
31160	CANF	2020-10-12	1.85	1.85	1.8	1.8027	49664	{}	\N	2020-10-12 14:26:31.600256
31161	CASH	2020-09-28	18.85	19.76	18.85	19.61	230641	{}	\N	2020-10-12 14:26:32.876304
31162	CASH	2020-09-29	19.53	19.53	18.92	19.29	141520	{}	\N	2020-10-12 14:26:32.885178
31163	CASH	2020-09-30	19.39	19.76	19.11	19.22	188036	{}	\N	2020-10-12 14:26:32.891236
31164	CASH	2020-10-01	19.19	19.81	19.09	19.78	216201	{}	\N	2020-10-12 14:26:32.897193
31165	CASH	2020-10-02	19.29	20.59	19.29	20.45	223431	{}	\N	2020-10-12 14:26:32.902762
31166	CASH	2020-10-05	20.7	21.19	19.85	21.05	187650	{}	\N	2020-10-12 14:26:32.906918
31167	CASH	2020-10-06	21.27	22.02	21.13	21.3	215613	{}	\N	2020-10-12 14:26:32.911599
31168	CASH	2020-10-07	21.65	22.02	21.42	21.6	240409	{}	\N	2020-10-12 14:26:32.915977
31169	CASH	2020-10-08	21.85	22.545	21.75	22.51	207421	{}	\N	2020-10-12 14:26:32.920228
31170	CASH	2020-10-09	22.63	22.96	22	22.05	159598	{}	\N	2020-10-12 14:26:32.924571
31171	CASH	2020-10-12	22.05	22.25	21.99	22.03	19272	{}	\N	2020-10-12 14:26:32.928918
31172	CBLI	2020-09-28	2.05	2.1046	1.97	2.06	35600	{}	\N	2020-10-12 14:26:35.160717
31173	CBLI	2020-09-29	2.05	2.05	1.95	1.95	65254	{}	\N	2020-10-12 14:26:35.168235
31174	CBLI	2020-09-30	1.92	2.0199	1.9059	1.97	35368	{}	\N	2020-10-12 14:26:35.173772
31175	CBLI	2020-10-01	1.95	2.0245	1.95	2	56996	{}	\N	2020-10-12 14:26:35.179873
31176	CBLI	2020-10-02	1.95	2.02	1.86	2	39412	{}	\N	2020-10-12 14:26:35.185777
31177	CBLI	2020-10-05	2.04	2.1	2	2.06	101602	{}	\N	2020-10-12 14:26:35.192911
31178	CBLI	2020-10-06	2.1	2.2	2.06	2.06	61848	{}	\N	2020-10-12 14:26:35.202723
31179	CBLI	2020-10-07	2.06	2.15	2.06	2.09	31747	{}	\N	2020-10-12 14:26:35.211624
31180	CBLI	2020-10-08	2.09	2.2	2.07	2.12	42694	{}	\N	2020-10-12 14:26:35.22101
31181	CBLI	2020-10-09	2.12	2.13	2.0777	2.0977	40534	{}	\N	2020-10-12 14:26:35.231312
31182	CBLI	2020-10-12	2.13	2.15	2.0901	2.149	8162	{}	\N	2020-10-12 14:26:35.242701
31183	CBRL	2020-09-28	118.38	120.27	116.94	119.6	346682	{}	\N	2020-10-12 14:26:36.302503
31184	CBRL	2020-09-29	119.11	119.11	113.53	113.9	359729	{}	\N	2020-10-12 14:26:36.309812
31185	CBRL	2020-09-30	114.03	117.342	114.0101	114.66	405496	{}	\N	2020-10-12 14:26:36.31527
31186	CBRL	2020-10-01	115.1	115.772	112.11	114.32	432573	{}	\N	2020-10-12 14:26:36.320629
31187	CBRL	2020-10-02	111.8	118.95	111.26	118.3	410828	{}	\N	2020-10-12 14:26:36.325579
31188	CBRL	2020-10-05	119.15	119.91	115.47	117.02	379989	{}	\N	2020-10-12 14:26:36.330353
31189	CBRL	2020-10-06	118.22	121.98	115.8	116.44	363570	{}	\N	2020-10-12 14:26:36.334963
31190	CBRL	2020-10-07	118.11	119.0276	116.04	116.69	217339	{}	\N	2020-10-12 14:26:36.338987
31191	CBRL	2020-10-08	117.39	118.1132	115.815	117.62	250577	{}	\N	2020-10-12 14:26:36.343301
31192	CBRL	2020-10-09	118.82	119.38	115.11	118.26	318836	{}	\N	2020-10-12 14:26:36.347568
31193	CBRL	2020-10-12	120.1	120.94	118.52	118.71	94992	{}	\N	2020-10-12 14:26:36.351772
31194	CC	2020-09-28	19.98	20.55	19.86	20.43	1166954	{}	\N	2020-10-12 14:26:39.901082
31195	CC	2020-09-29	20.6	20.83	20.05	20.3	1471229	{}	\N	2020-10-12 14:26:39.910599
31196	CC	2020-09-30	20.33	21.5178	20.33	20.91	5173684	{}	\N	2020-10-12 14:26:39.917652
31197	CC	2020-10-01	21.27	21.27	20.3	20.54	1701595	{}	\N	2020-10-12 14:26:39.924864
31198	CC	2020-10-02	19.89	21.51	19.81	21.41	2771957	{}	\N	2020-10-12 14:26:39.930161
31199	CC	2020-10-05	21.69	22.76	21.69	22.28	2824598	{}	\N	2020-10-12 14:26:39.934793
31200	CC	2020-10-06	22.36	23.03	22.07	22.16	2298196	{}	\N	2020-10-12 14:26:39.938933
31201	CC	2020-10-07	22.76	23	22.4635	22.88	2007745	{}	\N	2020-10-12 14:26:39.943319
31202	CC	2020-10-08	23.1	23.69	23.01	23.58	1848369	{}	\N	2020-10-12 14:26:39.947292
31203	CC	2020-10-09	23.79	23.92	22.98	22.99	1535395	{}	\N	2020-10-12 14:26:39.951603
31204	CC	2020-10-12	23.27	23.46	23.0201	23.42	279164	{}	\N	2020-10-12 14:26:39.956064
32565	CFO	2020-10-05	54.93	55.64	54.93	55.57	49041	{}	\N	2020-10-12 14:29:46.778116
32566	CFO	2020-10-06	55.78	56.1583	55.09	55.23	14427	{}	\N	2020-10-12 14:29:46.782636
31205	BTT	2020-09-28	24.9	25.05	24.9	24.98	53503	{}	\N	2020-10-12 14:26:42.118923
31206	BTT	2020-09-29	24.99	25	24.88	24.9	117040	{}	\N	2020-10-12 14:26:42.133405
31207	BTT	2020-09-30	24.84	25.06	24.84	24.99	150600	{}	\N	2020-10-12 14:26:42.140105
31208	BTT	2020-10-01	25.04	25.1	25.01	25.06	165182	{}	\N	2020-10-12 14:26:42.145368
31209	BTT	2020-10-02	25.04	25.1	24.91	25	223616	{}	\N	2020-10-12 14:26:42.151357
31210	BTT	2020-10-05	25.06	25.06	24.86	24.86	127642	{}	\N	2020-10-12 14:26:42.156338
31211	BTT	2020-10-06	24.91	25.05	24.91	25	177445	{}	\N	2020-10-12 14:26:42.161725
31212	BTT	2020-10-07	24.95	25.04	24.9	24.9	196208	{}	\N	2020-10-12 14:26:42.166927
31213	BTT	2020-10-08	24.9	25.1	24.9	24.92	221916	{}	\N	2020-10-12 14:26:42.171953
31214	BTT	2020-10-09	24.9	25.02	24.87	24.9	178712	{}	\N	2020-10-12 14:26:42.17708
31215	BTT	2020-10-12	24.9373	24.95	24.9	24.95	7492	{}	\N	2020-10-12 14:26:42.182154
31216	BWEN	2020-09-28	2.92	3.04	2.92	3.01	112677	{}	\N	2020-10-12 14:26:43.4783
31217	BWEN	2020-09-29	3	3.01	2.93	2.95	82842	{}	\N	2020-10-12 14:26:43.486232
31218	BWEN	2020-09-30	2.95	3.0058	2.84	2.87	221783	{}	\N	2020-10-12 14:26:43.491995
31219	BWEN	2020-10-01	2.87	2.93	2.83	2.83	99408	{}	\N	2020-10-12 14:26:43.498076
31220	BWEN	2020-10-02	2.79	2.87	2.68	2.73	138842	{}	\N	2020-10-12 14:26:43.503387
31221	BWEN	2020-10-05	2.79	3.15	2.74	3.08	455156	{}	\N	2020-10-12 14:26:43.507629
31222	BWEN	2020-10-06	3.279	3.7	3.08	3.52	648071	{}	\N	2020-10-12 14:26:43.511941
31223	BWEN	2020-10-07	3.76	4.5	3.63	4.1	988584	{}	\N	2020-10-12 14:26:43.516083
31224	BWEN	2020-10-08	4.4	4.42	3.65	3.93	555034	{}	\N	2020-10-12 14:26:43.520525
31225	BWEN	2020-10-09	3.89	3.93	3.72	3.78	129527	{}	\N	2020-10-12 14:26:43.525388
31226	BWEN	2020-10-12	3.76	3.88	3.7	3.74	61854	{}	\N	2020-10-12 14:26:43.529793
31227	BXG	2020-09-28	5.22	5.25	5.16	5.18	23707	{}	\N	2020-10-12 14:26:44.464896
31228	BXG	2020-09-29	5.15	5.175	4.98	5.02	63442	{}	\N	2020-10-12 14:26:44.472193
31229	BXG	2020-09-30	5.04	5.0701	4.85	4.9	103595	{}	\N	2020-10-12 14:26:44.478521
31230	BXG	2020-10-01	4.9804	5.04	4.83	5.04	92396	{}	\N	2020-10-12 14:26:44.485675
31231	BXG	2020-10-02	4.97	5.3	4.96	5.27	75602	{}	\N	2020-10-12 14:26:44.496872
31232	BXG	2020-10-05	5.3	5.3	4.93	5.15	83474	{}	\N	2020-10-12 14:26:44.502511
31233	BXG	2020-10-06	5.21	5.21	5.06	5.1	198533	{}	\N	2020-10-12 14:26:44.506978
31234	BXG	2020-10-07	5.11	5.25	5.11	5.15	45058	{}	\N	2020-10-12 14:26:44.511948
31235	BXG	2020-10-08	5.17	5.3	5.17	5.24	68393	{}	\N	2020-10-12 14:26:44.516587
31236	BXG	2020-10-09	5.29	5.4	5.21	5.31	45740	{}	\N	2020-10-12 14:26:44.521467
31237	BXG	2020-10-12	5.2708	5.295	5.26	5.28	2704	{}	\N	2020-10-12 14:26:44.526888
31238	BYFC	2020-09-28	1.73	2.33	1.6	1.79	12173388	{}	\N	2020-10-12 14:26:45.447904
31239	BYFC	2020-09-29	1.73	1.97	1.66	1.83	3380777	{}	\N	2020-10-12 14:26:45.453993
31240	BYFC	2020-09-30	1.68	1.7265	1.65	1.68	826006	{}	\N	2020-10-12 14:26:45.461237
31241	BYFC	2020-10-01	1.66	1.71	1.65	1.71	356307	{}	\N	2020-10-12 14:26:45.466454
31242	BYFC	2020-10-02	1.65	1.77	1.65	1.68	496252	{}	\N	2020-10-12 14:26:45.471647
31243	BYFC	2020-10-05	1.68	1.73	1.66	1.69	319632	{}	\N	2020-10-12 14:26:45.476969
31244	BYFC	2020-10-06	1.7	1.84	1.66	1.68	648547	{}	\N	2020-10-12 14:26:45.482387
31245	BYFC	2020-10-07	1.7	1.88	1.7	1.81	1777024	{}	\N	2020-10-12 14:26:45.487195
31246	BYFC	2020-10-08	1.92	2.1	1.78	1.88	5298403	{}	\N	2020-10-12 14:26:45.492344
31247	BYFC	2020-10-09	1.81	1.98	1.8	1.89	1942569	{}	\N	2020-10-12 14:26:45.4983
31248	BYFC	2020-10-12	1.88	1.8835	1.81	1.8191	190646	{}	\N	2020-10-12 14:26:45.503453
31249	CAAS	2020-09-28	2.9	3.23	2.8701	3	16603	{}	\N	2020-10-12 14:26:46.446235
31250	CAAS	2020-09-29	2.9437	3.09	2.94	2.95	31162	{}	\N	2020-10-12 14:26:46.455957
31251	CAAS	2020-09-30	3.01	3.07	2.95	2.96	22787	{}	\N	2020-10-12 14:26:46.463103
31252	CAAS	2020-10-01	3.01	3.01	2.87	2.95	28485	{}	\N	2020-10-12 14:26:46.46872
31253	CAAS	2020-10-02	2.94	3.05	2.935	2.98	19874	{}	\N	2020-10-12 14:26:46.474313
31254	CAAS	2020-10-05	2.975	3.04	2.9201	3	9336	{}	\N	2020-10-12 14:26:46.479358
31255	CAAS	2020-10-06	2.97	3.0965	2.9601	2.9877	27165	{}	\N	2020-10-12 14:26:46.483776
31256	CAAS	2020-10-07	2.97	3.1	2.95	3	24161	{}	\N	2020-10-12 14:26:46.488194
31257	CAAS	2020-10-08	2.97	3.08	2.94	2.95	11563	{}	\N	2020-10-12 14:26:46.492717
31258	CAAS	2020-10-09	2.94	3	2.9	2.96	17390	{}	\N	2020-10-12 14:26:46.497599
31259	CAAS	2020-10-12	2.92	3	2.92	3	2938	{}	\N	2020-10-12 14:26:46.502398
31260	CADE	2020-09-28	8.32	8.74	8.31	8.66	1002247	{}	\N	2020-10-12 14:26:47.478749
31261	CADE	2020-09-29	8.6	8.75	8.37	8.45	2001306	{}	\N	2020-10-12 14:26:47.489127
31262	CADE	2020-09-30	8.55	8.755	8.49	8.59	2200589	{}	\N	2020-10-12 14:26:47.496471
31263	CADE	2020-10-01	8.63	8.92	8.48	8.91	2771567	{}	\N	2020-10-12 14:26:47.502413
31264	CADE	2020-10-02	8.67	9.4	8.67	9.35	3392219	{}	\N	2020-10-12 14:26:47.507438
31265	CADE	2020-10-05	9.5	10	9.46	9.97	1677818	{}	\N	2020-10-12 14:26:47.511824
31266	CADE	2020-10-06	10.18	10.485	9.89	9.97	3425621	{}	\N	2020-10-12 14:26:47.516513
31267	CADE	2020-10-07	10.12	10.57	10.07	10.43	3562607	{}	\N	2020-10-12 14:26:47.520774
31268	CADE	2020-10-08	10.5	10.93	10.3501	10.91	3221928	{}	\N	2020-10-12 14:26:47.525175
31269	CADE	2020-10-09	10.97	11.2	10.58	10.58	2571932	{}	\N	2020-10-12 14:26:47.529499
31270	CADE	2020-10-12	10.58	10.66	10.455	10.55	228580	{}	\N	2020-10-12 14:26:47.534136
31271	CALF	2020-09-28	25.295	25.6	25.295	25.48	15865	{}	\N	2020-10-12 14:26:48.501634
31272	CALF	2020-09-29	25.32	25.3337	24.9794	25.23	23476	{}	\N	2020-10-12 14:26:48.509087
31273	CALF	2020-09-30	25.5699	25.58	25.3415	25.3415	1587	{}	\N	2020-10-12 14:26:48.515427
31274	CALF	2020-10-01	25.25	25.6189	25.25	25.6189	8944	{}	\N	2020-10-12 14:26:48.522208
31275	CALF	2020-10-02	25.13	26.04	25.13	26.04	5655	{}	\N	2020-10-12 14:26:48.527067
31276	CALF	2020-10-05	26.07	26.71	26.07	26.6401	12088	{}	\N	2020-10-12 14:26:48.533886
31277	CALF	2020-10-06	27	27.3799	26.66	26.66	6250	{}	\N	2020-10-12 14:26:48.540122
31278	CALF	2020-10-07	27.005	27.22	27.005	27.22	1897	{}	\N	2020-10-12 14:26:48.545885
31279	CALF	2020-10-08	27.61	27.61	27.3114	27.61	3844	{}	\N	2020-10-12 14:26:48.550647
31280	CALF	2020-10-09	27.87	27.87	27.7399	27.75	7808	{}	\N	2020-10-12 14:26:48.554557
31281	CALF	2020-10-12	27.75	27.75	27.75	27.75	940	{}	\N	2020-10-12 14:26:48.558843
31282	CAR	2020-09-28	28.11	28.41	27.33	27.52	1751507	{}	\N	2020-10-12 14:26:49.701784
31283	CAR	2020-09-29	27.3	27.32	25.69	25.82	2003618	{}	\N	2020-10-12 14:26:49.72021
31284	CAR	2020-09-30	26.095	26.81	25.54	26.32	2639861	{}	\N	2020-10-12 14:26:49.736707
31285	CAR	2020-10-01	26.65	28.84	26.5228	28.73	3085868	{}	\N	2020-10-12 14:26:49.744008
31286	CAR	2020-10-02	27.31	29.14	27.25	28.34	1865759	{}	\N	2020-10-12 14:26:49.753855
31287	CAR	2020-10-05	28.81	29.75	28.56	29.07	1610000	{}	\N	2020-10-12 14:26:49.762853
31288	CAR	2020-10-06	29.56	30.75	28.75	29.05	2099221	{}	\N	2020-10-12 14:26:49.770461
31289	CAR	2020-10-07	29.82	30.97	29.63	30.61	2890981	{}	\N	2020-10-12 14:26:49.777453
31290	CAR	2020-10-08	31.24	31.76	30.66	31.17	1691482	{}	\N	2020-10-12 14:26:49.785439
31291	CAR	2020-10-09	31.8	32.58	31.17	31.57	1787523	{}	\N	2020-10-12 14:26:49.793936
31292	CAR	2020-10-12	31.72	31.9318	30.44	31.215	267351	{}	\N	2020-10-12 14:26:49.802
31293	CAT	2020-09-28	147.85	150.31	147.41	147.68	2106644	{}	\N	2020-10-12 14:26:52.568139
31294	CAT	2020-09-29	148.37	148.97	146.21	147.41	3242404	{}	\N	2020-10-12 14:26:52.576295
31295	CAT	2020-09-30	148	150.95	147.735	149.15	8222701	{}	\N	2020-10-12 14:26:52.58267
31296	CAT	2020-10-01	149.92	150.31	145.65	146.71	4190731	{}	\N	2020-10-12 14:26:52.58947
31297	CAT	2020-10-02	144.27	151.25	143.75	149.94	6158331	{}	\N	2020-10-12 14:26:52.595041
31298	CAT	2020-10-05	152.17	154.45	151.62	153.49	5253272	{}	\N	2020-10-12 14:26:52.599546
31299	CAT	2020-10-06	154	155.45	151.28	151.53	4936293	{}	\N	2020-10-12 14:26:52.605803
31300	CAT	2020-10-07	153.28	156.06	153.2	154.77	4542563	{}	\N	2020-10-12 14:26:52.612568
31301	CAT	2020-10-08	155.75	156.73	154.24	156.66	3595565	{}	\N	2020-10-12 14:26:52.618584
31302	CAT	2020-10-09	157.87	159.39	157.4	158.94	7993298	{}	\N	2020-10-12 14:26:52.624353
31303	CAT	2020-10-12	159.97	161.71	159.52	160.6398	1626521	{}	\N	2020-10-12 14:26:52.629448
31304	CBH	2020-09-28	8.918	8.9512	8.82	8.88	109186	{}	\N	2020-10-12 14:26:55.020441
31305	CBH	2020-09-29	8.93	8.93	8.8	8.81	76900	{}	\N	2020-10-12 14:26:55.028479
31306	CBH	2020-09-30	8.79	8.87	8.78	8.8	517574	{}	\N	2020-10-12 14:26:55.034507
31307	CBH	2020-10-01	8.93	8.96	8.86	8.94	62252	{}	\N	2020-10-12 14:26:55.040023
31308	CBH	2020-10-02	9.0676	9.07	8.88	9.03	45142	{}	\N	2020-10-12 14:26:55.04556
31309	CBH	2020-10-05	8.94	9.23	8.94	9.04	30584	{}	\N	2020-10-12 14:26:55.050111
31310	CBH	2020-10-06	9.2	9.24	8.92	9.06	136705	{}	\N	2020-10-12 14:26:55.054967
31311	CBH	2020-10-07	9.1	9.1	8.97	9.05	126913	{}	\N	2020-10-12 14:26:55.059638
31312	CBH	2020-10-08	9.06	9.24	9.05	9.08	35628	{}	\N	2020-10-12 14:26:55.065071
31313	CBH	2020-10-09	9.06	9.0796	8.95	8.98	104766	{}	\N	2020-10-12 14:26:55.07013
31314	CBH	2020-10-12	9.02	9.09	9.02	9.09	2210	{}	\N	2020-10-12 14:26:55.074747
31315	BTU	2020-09-28	3.0888	3.31	2.99	3.12	2045972	{}	\N	2020-10-12 14:26:56.507344
31316	BTU	2020-09-29	3.06	3.06	2.4001	2.44	11301698	{}	\N	2020-10-12 14:26:56.516335
31317	BTU	2020-09-30	2.46	2.49	2.3	2.3	6169578	{}	\N	2020-10-12 14:26:56.522319
31318	BTU	2020-10-01	2.34	2.34	2.15	2.2	6906728	{}	\N	2020-10-12 14:26:56.527639
31319	BTU	2020-10-02	2.15	2.405	2.1	2.39	5570386	{}	\N	2020-10-12 14:26:56.532638
31320	BTU	2020-10-05	2.5	2.52	2.17	2.18	6199706	{}	\N	2020-10-12 14:26:56.537528
31321	BTU	2020-10-06	2.29	2.31	2.12	2.12	4775719	{}	\N	2020-10-12 14:26:56.541746
31322	BTU	2020-10-07	2.14	2.15	2.05	2.07	4504647	{}	\N	2020-10-12 14:26:56.545869
31323	BTU	2020-10-08	2.11	2.15	2.01	2.06	5046553	{}	\N	2020-10-12 14:26:56.550137
31324	BTU	2020-10-09	2.08	2.11	1.9	2.04	9311706	{}	\N	2020-10-12 14:26:56.55445
31325	BTU	2020-10-12	2.05	2.05	1.9	1.915	1271209	{}	\N	2020-10-12 14:26:56.558786
31326	BWG	2020-09-28	11.69	11.69	11.58	11.61	39853	{}	\N	2020-10-12 14:26:57.592639
31327	BWG	2020-09-29	11.6	11.6	11.38	11.42	702250	{}	\N	2020-10-12 14:26:57.60462
31328	BWG	2020-09-30	11.38	11.54	11.38	11.51	625204	{}	\N	2020-10-12 14:26:57.611301
31329	BWG	2020-10-01	11.55	11.5666	11.51	11.52	358318	{}	\N	2020-10-12 14:26:57.617433
31330	BWG	2020-10-02	11.48	11.48	11.41	11.48	96451	{}	\N	2020-10-12 14:26:57.622609
31331	BWG	2020-10-05	11.44	11.58	11.44	11.53	124092	{}	\N	2020-10-12 14:26:57.627357
31332	BWG	2020-10-06	11.55	11.59	11.48	11.51	177092	{}	\N	2020-10-12 14:26:57.631736
31333	BWG	2020-10-07	11.55	11.58	11.54	11.56	63134	{}	\N	2020-10-12 14:26:57.636227
31334	BWG	2020-10-08	11.63	11.63	11.57	11.6	138537	{}	\N	2020-10-12 14:26:57.640553
31335	BWG	2020-10-09	11.68	11.68	11.64	11.66	105140	{}	\N	2020-10-12 14:26:57.644574
31336	BWG	2020-10-12	11.65	11.65	11.6239	11.63	12280	{}	\N	2020-10-12 14:26:57.648817
31337	BXMT	2020-09-28	22.79	23.47	22.705	23.17	1062893	{}	\N	2020-10-12 14:26:59.826635
31338	BXMT	2020-09-29	22.45	22.57	21.665	22.18	2293009	{}	\N	2020-10-12 14:26:59.835092
31339	BXMT	2020-09-30	22.2	22.535	21.86	21.97	1817484	{}	\N	2020-10-12 14:26:59.841658
31340	BXMT	2020-10-01	22.18	22.39	21.74	22.37	1655998	{}	\N	2020-10-12 14:26:59.847774
31341	BXMT	2020-10-02	21.84	22.74	21.82	22.56	1965330	{}	\N	2020-10-12 14:26:59.853113
31342	BXMT	2020-10-05	22.85	23	22.59	22.94	1459398	{}	\N	2020-10-12 14:26:59.857848
31343	BXMT	2020-10-06	23	23.48	22.78	22.83	2172541	{}	\N	2020-10-12 14:26:59.862185
31344	BXMT	2020-10-07	22.95	23.15	22.49	22.72	1757551	{}	\N	2020-10-12 14:26:59.866609
31345	BXMT	2020-10-08	22.92	24.23	22.85	24.21	3713703	{}	\N	2020-10-12 14:26:59.870887
31346	BXMT	2020-10-09	24.2	24.38	23	23.36	2558885	{}	\N	2020-10-12 14:26:59.875259
31347	BXMT	2020-10-12	23.34	23.435	23.24	23.32	126505	{}	\N	2020-10-12 14:26:59.879539
31348	BZUN	2020-09-28	32.96	34.07	32.55	33.98	1281918	{}	\N	2020-10-12 14:27:00.876882
31349	BZUN	2020-09-29	32.7	32.89	31.64	31.87	3763254	{}	\N	2020-10-12 14:27:00.888585
31350	BZUN	2020-09-30	31.87	32.59	31.76	32.49	1615420	{}	\N	2020-10-12 14:27:00.897021
31351	BZUN	2020-10-01	32.89	33.97	32.655	33.65	1131192	{}	\N	2020-10-12 14:27:00.903389
31352	BZUN	2020-10-02	32.87	33.38	32.5583	32.6	603185	{}	\N	2020-10-12 14:27:00.908522
31353	BZUN	2020-10-05	33.21	33.21	32.3	32.92	558876	{}	\N	2020-10-12 14:27:00.913014
31354	BZUN	2020-10-06	32.501	32.9	31.91	32.48	788358	{}	\N	2020-10-12 14:27:00.917848
31355	BZUN	2020-10-07	32.78	32.945	32.05	32.15	1118742	{}	\N	2020-10-12 14:27:00.922671
31356	BZUN	2020-10-08	32.4	32.99	31.93	32.72	1533580	{}	\N	2020-10-12 14:27:00.927244
31357	BZUN	2020-10-09	32.72	34.5	32.43	33.78	776089	{}	\N	2020-10-12 14:27:00.931778
31358	BZUN	2020-10-12	34.43	35.25	34	35.04	344295	{}	\N	2020-10-12 14:27:00.936214
31359	CAF	2020-09-28	20.98	21.21	20.98	21.21	36909	{}	\N	2020-10-12 14:27:02.174447
31360	CAF	2020-09-29	21.03	21.21	20.81	21.16	84090	{}	\N	2020-10-12 14:27:02.181508
31361	CAF	2020-09-30	21.13	21.255	21.01	21.12	78656	{}	\N	2020-10-12 14:27:02.186858
31362	CAF	2020-10-01	21.21	21.35	21.12	21.2	84952	{}	\N	2020-10-12 14:27:02.191315
31363	CAF	2020-10-02	21.07	21.16	21	21.08	27080	{}	\N	2020-10-12 14:27:02.20074
31364	CAF	2020-10-05	20.98	21.207	20.98	21.1	48854	{}	\N	2020-10-12 14:27:02.208375
31365	CAF	2020-10-06	21.01	21.17	20.88	20.88	81090	{}	\N	2020-10-12 14:27:02.213746
31366	CAF	2020-10-07	21.16	21.26	21.0955	21.14	44674	{}	\N	2020-10-12 14:27:02.219111
31367	CAF	2020-10-08	21.08	21.36	21.07	21.36	25826	{}	\N	2020-10-12 14:27:02.22519
31368	CAF	2020-10-09	21.52	21.56	21.375	21.54	39874	{}	\N	2020-10-12 14:27:02.229769
31369	CAF	2020-10-12	21.75	21.9402	21.75	21.86	25462	{}	\N	2020-10-12 14:27:02.234207
31370	CANE	2020-09-28	5.96	6.1625	5.9	5.91	57894	{}	\N	2020-10-12 14:27:04.186539
31371	CANE	2020-09-29	5.98	6.08	5.98	6.06	31486	{}	\N	2020-10-12 14:27:04.19582
31372	CANE	2020-09-30	6.05	6.0999	5.952	6.09	15960	{}	\N	2020-10-12 14:27:04.202615
31373	CANE	2020-10-01	6.13	6.15	6.0601	6.12	21657	{}	\N	2020-10-12 14:27:04.208583
31374	CANE	2020-10-02	6.02	6.16	6.02	6.1148	11524	{}	\N	2020-10-12 14:27:04.213622
31375	CANE	2020-10-05	6.14	6.19	6.14	6.16	33159	{}	\N	2020-10-12 14:27:04.218127
31376	CANE	2020-10-06	6.17	6.26	6.17	6.24	34243	{}	\N	2020-10-12 14:27:04.222887
31377	CANE	2020-10-07	6.27	6.34	6.27	6.3	204709	{}	\N	2020-10-12 14:27:04.227441
31378	CANE	2020-10-08	6.32	6.33	6.3	6.3	30501	{}	\N	2020-10-12 14:27:04.23205
31379	CANE	2020-10-09	6.26	6.34	6.25	6.33	59836	{}	\N	2020-10-12 14:27:04.236693
31380	CANE	2020-10-12	6.38	6.38	6.2885	6.3235	9967	{}	\N	2020-10-12 14:27:04.2414
31381	CARV	2020-09-28	6.04	7.5	5.85	6.79	2740216	{}	\N	2020-10-12 14:27:05.303326
31382	CARV	2020-09-29	6.31	7.75	6.3	7.27	1645032	{}	\N	2020-10-12 14:27:05.31644
31383	CARV	2020-09-30	6.65	6.98	6.3445	6.45	701394	{}	\N	2020-10-12 14:27:05.322586
31384	CARV	2020-10-01	6.42	6.57	6.21	6.27	166377	{}	\N	2020-10-12 14:27:05.328309
31385	CARV	2020-10-02	6.17	6.44	6.02	6.39	170669	{}	\N	2020-10-12 14:27:05.333484
31386	CARV	2020-10-05	6.35	6.4126	6.08	6.23	95697	{}	\N	2020-10-12 14:27:05.338446
31387	CARV	2020-10-06	6.26	6.42	6.03	6.22	304943	{}	\N	2020-10-12 14:27:05.343318
31388	CARV	2020-10-07	6.18	6.76	6.1719	6.59	962418	{}	\N	2020-10-12 14:27:05.347507
31389	CARV	2020-10-08	6.95	9.25	6.8	7.53	10738665	{}	\N	2020-10-12 14:27:05.351929
31390	CARV	2020-10-09	6.95	7.35	6.74	7.06	879513	{}	\N	2020-10-12 14:27:05.356775
31391	CARV	2020-10-12	7.02	7.2799	7.02	7.1	55993	{}	\N	2020-10-12 14:27:05.36164
31392	CATC	2020-09-28	50	51.25	49.88	50.03	12350	{}	\N	2020-10-12 14:27:06.717713
31393	CATC	2020-09-29	50.53	52	50.5199	51.95	7322	{}	\N	2020-10-12 14:27:06.728363
31394	CATC	2020-09-30	51.67	53.3185	51.3642	53.16	11652	{}	\N	2020-10-12 14:27:06.734972
31395	CATC	2020-10-01	53.345	54.01	52.285	53.9	18283	{}	\N	2020-10-12 14:27:06.741115
31396	CATC	2020-10-02	55.5	55.5	53.07	55	16659	{}	\N	2020-10-12 14:27:06.746148
31397	CATC	2020-10-05	55.63	57.5	54.6096	56.73	13126	{}	\N	2020-10-12 14:27:06.751071
31398	CATC	2020-10-06	56.3	60.44	56.3	59.56	36403	{}	\N	2020-10-12 14:27:06.757337
31399	CATC	2020-10-07	60	60.89	59.8	60.5	16911	{}	\N	2020-10-12 14:27:06.761472
31400	CATC	2020-10-08	61	62.25	60.2	62.1	18745	{}	\N	2020-10-12 14:27:06.765167
31401	CATC	2020-10-09	62.26	64.885	61.77	63.48	16780	{}	\N	2020-10-12 14:27:06.768901
31402	CATC	2020-10-12	63.04	64.66	63.04	64.66	974	{}	\N	2020-10-12 14:27:06.772743
31403	CBAY	2020-09-28	6.57	6.615	6.13	6.52	1421252	{}	\N	2020-10-12 14:27:08.19558
31404	CBAY	2020-09-29	6.5	7.38	6.39	7.08	4354297	{}	\N	2020-10-12 14:27:08.209275
31405	CBAY	2020-09-30	7.1439	7.35	6.8	7.23	2435986	{}	\N	2020-10-12 14:27:08.21666
31406	CBAY	2020-10-01	7.3237	7.64	6.96	7.04	2721873	{}	\N	2020-10-12 14:27:08.222415
31407	CBAY	2020-10-02	6.82	7.1953	6.6	7.04	1355954	{}	\N	2020-10-12 14:27:08.227113
31408	CBAY	2020-10-05	7.05	7.64	7.05	7.6	1658059	{}	\N	2020-10-12 14:27:08.231549
31409	CBAY	2020-10-06	7.62	7.69	7.22	7.25	1313130	{}	\N	2020-10-12 14:27:08.235636
31410	CBAY	2020-10-07	7.38	7.82	7.33	7.62	1399577	{}	\N	2020-10-12 14:27:08.239705
31411	CBAY	2020-10-08	7.69	7.81	7.54	7.8	1067965	{}	\N	2020-10-12 14:27:08.243935
31412	CBAY	2020-10-09	7.9	8.39	7.88	7.91	1491882	{}	\N	2020-10-12 14:27:08.24787
31413	CBAY	2020-10-12	7.92	8.01	7.63	7.85	267041	{}	\N	2020-10-12 14:27:08.252039
31414	CBU	2020-09-28	53.65	54.76	53.55	54.43	213194	{}	\N	2020-10-12 14:27:11.17868
31415	CBU	2020-09-29	54.32	54.32	53.07	53.87	286285	{}	\N	2020-10-12 14:27:11.18865
31416	CBU	2020-09-30	53.76	55.1	53.76	54.46	531617	{}	\N	2020-10-12 14:27:11.194908
31417	CBU	2020-10-01	54.34	54.83	53.6	54.35	260606	{}	\N	2020-10-12 14:27:11.200583
31418	CBU	2020-10-02	53.64	55.29	53.64	55.1	298047	{}	\N	2020-10-12 14:27:11.20539
31419	CBU	2020-10-05	55.63	56.81	55.6	56.37	265642	{}	\N	2020-10-12 14:27:11.210408
31420	CBU	2020-10-06	57.23	58.81	56.58	57.06	525003	{}	\N	2020-10-12 14:27:11.214619
31421	CBU	2020-10-07	57.66	58.7	56.94	57.36	421003	{}	\N	2020-10-12 14:27:11.218733
31422	CBU	2020-10-08	58.01	59	57.32	58.63	322187	{}	\N	2020-10-12 14:27:11.222824
31423	CBU	2020-10-09	59.13	59.24	58.075	58.38	307730	{}	\N	2020-10-12 14:27:11.226836
31424	CBU	2020-10-12	58.15	58.84	58.15	58.755	12872	{}	\N	2020-10-12 14:27:11.230703
31425	CCI	2020-09-28	163.86	164.75	161.83	163.59	1753077	{}	\N	2020-10-12 14:27:12.390996
31426	CCI	2020-09-29	163.21	164.5	162.3845	164.18	3092592	{}	\N	2020-10-12 14:27:12.402659
31427	CCI	2020-09-30	164.8	167	164.35	166.5	4605163	{}	\N	2020-10-12 14:27:12.409596
31428	CCI	2020-10-01	166.75	168.3	165	168.01	3342430	{}	\N	2020-10-12 14:27:12.415501
31429	CCI	2020-10-02	166.91	169.3	166.1	168.62	4444103	{}	\N	2020-10-12 14:27:12.42053
31430	CCI	2020-10-05	168.62	169	165.28	168.66	2907397	{}	\N	2020-10-12 14:27:12.425246
31431	CCI	2020-10-06	168.75	168.75	164.44	165.09	3923748	{}	\N	2020-10-12 14:27:12.429799
31432	CCI	2020-10-07	166.2	166.5	164.18	164.92	3002389	{}	\N	2020-10-12 14:27:12.433994
31433	CCI	2020-10-08	165.75	169.31	165.34	165.78	2618608	{}	\N	2020-10-12 14:27:12.438141
31434	CCI	2020-10-09	166.6	167.63	164.88	167.36	1988335	{}	\N	2020-10-12 14:27:12.442339
31435	CCI	2020-10-12	168.23	168.23	166.52	167.7	254943	{}	\N	2020-10-12 14:27:12.446407
31436	CCO	2020-09-28	0.95	1	0.95	0.9954	2016573	{}	\N	2020-10-12 14:27:14.307628
31437	CCO	2020-09-29	0.99	1.01	0.93	0.9756	8335401	{}	\N	2020-10-12 14:27:14.316375
31438	CCO	2020-09-30	0.97	1.02	0.97	1	3842260	{}	\N	2020-10-12 14:27:14.322259
31439	CCO	2020-10-01	0.99	1.03	0.97	1.02	2425521	{}	\N	2020-10-12 14:27:14.327268
31440	CCO	2020-10-02	0.9701	1.04	0.95	1.02	3777117	{}	\N	2020-10-12 14:27:14.332576
31441	CCO	2020-10-05	1.03	1.05	1	1.04	2253576	{}	\N	2020-10-12 14:27:14.337517
31442	CCO	2020-10-06	1.04	1.15	1.04	1.07	3943937	{}	\N	2020-10-12 14:27:14.341848
31443	CCO	2020-10-07	1.11	1.15	1.07	1.1	2957308	{}	\N	2020-10-12 14:27:14.345772
31444	CCO	2020-10-08	1.15	1.17	1.11	1.15	1796206	{}	\N	2020-10-12 14:27:14.349638
31445	CCO	2020-10-09	1.15	1.2	1.14	1.14	1110646	{}	\N	2020-10-12 14:27:14.353609
31446	CCO	2020-10-12	1.14	1.15	1.08	1.0814	265008	{}	\N	2020-10-12 14:27:14.357599
31447	CCZ	2020-09-28	61.35	61.35	61.35	61.35	100	{}	\N	2020-10-12 14:27:15.97215
31448	CCZ	2020-09-30	57.76	57.76	57.76	57.76	472	{}	\N	2020-10-12 14:27:15.982649
31449	CCZ	2020-10-06	57.88	57.88	57.88	57.88	404	{}	\N	2020-10-12 14:27:15.989286
31450	CCZ	2020-10-08	59.074	59.074	59.074	59.074	438	{}	\N	2020-10-12 14:27:15.994672
31451	CCZ	2020-10-12	57.77	57.77	57.77	57.77	400	{}	\N	2020-10-12 14:27:16.000329
31452	BTZ	2020-09-28	14.2	14.21	14.02	14.06	276229	{}	\N	2020-10-12 14:27:19.531938
31453	BTZ	2020-09-29	14.04	14.11	14.0311	14.11	402350	{}	\N	2020-10-12 14:27:19.541822
31454	BTZ	2020-09-30	14.19	14.27	14.13	14.22	855160	{}	\N	2020-10-12 14:27:19.548553
31455	BTZ	2020-10-01	14.19	14.26	14.17	14.19	306374	{}	\N	2020-10-12 14:27:19.554118
31456	BTZ	2020-10-02	14.19	14.19	13.985	14	372721	{}	\N	2020-10-12 14:27:19.558996
31457	BTZ	2020-10-05	14.01	14.05	13.9	13.96	425699	{}	\N	2020-10-12 14:27:19.563927
31458	BTZ	2020-10-06	13.95	14.09	13.95	14.01	274915	{}	\N	2020-10-12 14:27:19.567986
31459	BTZ	2020-10-07	14.08	14.125	13.87	13.94	586503	{}	\N	2020-10-12 14:27:19.571896
31460	BTZ	2020-10-08	13.97	13.98	13.9	13.98	453521	{}	\N	2020-10-12 14:27:19.575691
31461	BTZ	2020-10-09	14.04	14.04	13.9	13.91	497200	{}	\N	2020-10-12 14:27:19.579462
31462	BTZ	2020-10-12	13.98	13.98	13.91	13.92	66563	{}	\N	2020-10-12 14:27:19.583285
31463	BX	2020-09-28	51.88	52.8411	51.79	52.66	2277979	{}	\N	2020-10-12 14:27:21.302612
31464	BX	2020-09-29	52.59	53.22	52.01	52.71	5006251	{}	\N	2020-10-12 14:27:21.312675
31465	BX	2020-09-30	52.71	53.35	52.02	52.2	6156504	{}	\N	2020-10-12 14:27:21.319909
31466	BX	2020-10-01	52.49	53.65	52.43	53.51	4591003	{}	\N	2020-10-12 14:27:21.325298
31467	BX	2020-10-02	52.51	54.49	52.35	53.68	3588475	{}	\N	2020-10-12 14:27:21.333017
31468	BX	2020-10-05	54.06	54.61	53.7	54.29	4533841	{}	\N	2020-10-12 14:27:21.337495
31469	BX	2020-10-06	54.62	55.22	53.8	54.04	4894397	{}	\N	2020-10-12 14:27:21.341383
31470	BX	2020-10-07	54.35	55.09	53.98	54.46	5107394	{}	\N	2020-10-12 14:27:21.345177
31471	BX	2020-10-08	54.91	55.43	54.77	55.09	3718307	{}	\N	2020-10-12 14:27:21.348969
31472	BX	2020-10-09	55.42	55.63	54.61	54.72	3289196	{}	\N	2020-10-12 14:27:21.352818
31473	BX	2020-10-12	54.35	54.87	54.06	54.8371	1354395	{}	\N	2020-10-12 14:27:21.356847
31474	BZH	2020-09-28	12.56	13.56	12.45	13.51	794488	{}	\N	2020-10-12 14:27:22.50967
31475	BZH	2020-09-29	13.55	13.6	12.93	13.06	723240	{}	\N	2020-10-12 14:27:22.522844
31476	BZH	2020-09-30	13.11	13.36	13.06	13.2	754204	{}	\N	2020-10-12 14:27:22.530051
31477	BZH	2020-10-01	13.4	13.6083	13.15	13.58	716276	{}	\N	2020-10-12 14:27:22.536025
31478	BZH	2020-10-02	13.31	13.85	13.1	13.79	693171	{}	\N	2020-10-12 14:27:22.541293
31479	BZH	2020-10-05	14	14.07	13.27	13.6	644748	{}	\N	2020-10-12 14:27:22.54647
31480	BZH	2020-10-06	13.73	13.73	13.05	13.06	547270	{}	\N	2020-10-12 14:27:22.5515
31481	BZH	2020-10-07	13.26	13.74	13.15	13.72	707638	{}	\N	2020-10-12 14:27:22.555923
31482	BZH	2020-10-08	13.89	14.28	13.7	14.24	614923	{}	\N	2020-10-12 14:27:22.560446
31483	BZH	2020-10-09	14.57	14.63	14.21	14.32	943765	{}	\N	2020-10-12 14:27:22.564935
31484	BZH	2020-10-12	14.39	14.5471	14.02	14.16	149926	{}	\N	2020-10-12 14:27:22.569712
31485	CACG	2020-09-28	39.44	39.44	39.12	39.36	12162	{}	\N	2020-10-12 14:27:23.581809
31486	CACG	2020-09-29	39.44	39.57	39.2	39.39	18678	{}	\N	2020-10-12 14:27:23.591011
31487	CACG	2020-09-30	39.52	39.83	39.3133	39.68	14878	{}	\N	2020-10-12 14:27:23.596252
31488	CACG	2020-10-01	40.033	40.035	39.711	39.9	14420	{}	\N	2020-10-12 14:27:23.601383
31489	CACG	2020-10-02	39.61	39.61	39.01	39.195	58326	{}	\N	2020-10-12 14:27:23.605679
31490	CACG	2020-10-05	39.38	39.99	39.38	39.98	20685	{}	\N	2020-10-12 14:27:23.60963
31491	CACG	2020-10-06	39.9567	40.132	39.36	39.66	7016	{}	\N	2020-10-12 14:27:23.613397
31492	CACG	2020-10-07	39.83	40.23	39.79	40.15	5686	{}	\N	2020-10-12 14:27:23.616915
31493	CACG	2020-10-08	40.35	40.46	40.29	40.317	4422	{}	\N	2020-10-12 14:27:23.620251
31494	CACG	2020-10-09	40.67	40.841	40.4701	40.708	11910	{}	\N	2020-10-12 14:27:23.623672
31495	CACG	2020-10-12	41.02	41.19	41.02	41.19	384	{}	\N	2020-10-12 14:27:23.627062
31496	CAJ	2020-09-28	16.765	17.045	16.765	17.03	732690	{}	\N	2020-10-12 14:27:24.86595
31497	CAJ	2020-09-29	16.57	16.69	16.54	16.57	1246254	{}	\N	2020-10-12 14:27:24.876168
31498	CAJ	2020-09-30	16.63	16.71	16.5	16.59	1289436	{}	\N	2020-10-12 14:27:24.882187
31499	CAJ	2020-10-01	16.59	16.61	16.475	16.61	657171	{}	\N	2020-10-12 14:27:24.887481
31500	CAJ	2020-10-02	16.14	16.41	16.13	16.37	1129981	{}	\N	2020-10-12 14:27:24.892409
31501	CAJ	2020-10-05	16.42	16.59	16.42	16.57	751679	{}	\N	2020-10-12 14:27:24.897243
31502	CAJ	2020-10-06	16.8	16.94	16.71	16.73	1204291	{}	\N	2020-10-12 14:27:24.901511
31503	CAJ	2020-10-07	16.61	16.65	16.55	16.6	813979	{}	\N	2020-10-12 14:27:24.90555
31504	CAJ	2020-10-08	16.47	16.525	16.4111	16.45	782688	{}	\N	2020-10-12 14:27:24.909527
31505	CAJ	2020-10-09	16.19	16.19	16.05	16.08	1027090	{}	\N	2020-10-12 14:27:24.913518
31506	CAJ	2020-10-12	15.98	15.98	15.9	15.94	218417	{}	\N	2020-10-12 14:27:24.917818
31507	CAMT	2020-09-28	15.11	15.44	15.11	15.4	28091	{}	\N	2020-10-12 14:27:26.182752
31508	CAMT	2020-09-29	15.4	15.57	15.2	15.3	52383	{}	\N	2020-10-12 14:27:26.193581
31509	CAMT	2020-09-30	15.33	15.52	15.25	15.4	51341	{}	\N	2020-10-12 14:27:26.201109
31510	CAMT	2020-10-01	16.51	18.1	16.45	18.08	577379	{}	\N	2020-10-12 14:27:26.207185
31511	CAMT	2020-10-02	17.92	18.13	17.06	17.44	219531	{}	\N	2020-10-12 14:27:26.212108
31512	CAMT	2020-10-05	17.79	18	17.55	17.87	94835	{}	\N	2020-10-12 14:27:26.217247
31513	CAMT	2020-10-06	17.93	18.42	17.91	18.42	90860	{}	\N	2020-10-12 14:27:26.221493
31514	CAMT	2020-10-07	17.53	18.55	17.51	18.44	181500	{}	\N	2020-10-12 14:27:26.225813
31515	CAMT	2020-10-08	18.86	19.04	18.65	18.82	103047	{}	\N	2020-10-12 14:27:26.231394
31516	CAMT	2020-10-09	19.06	19.48	18.95	19.43	75977	{}	\N	2020-10-12 14:27:26.235828
31517	CAMT	2020-10-12	19.52	19.52	19.23	19.25	28608	{}	\N	2020-10-12 14:27:26.240328
31518	CARG	2020-09-28	22.36	22.57	21.21	21.93	1446254	{}	\N	2020-10-12 14:27:27.759737
31519	CARG	2020-09-29	21.93	22.0258	21.42	21.79	1224295	{}	\N	2020-10-12 14:27:27.76942
31520	CARG	2020-09-30	21.73	22.06	21.41	21.63	1091680	{}	\N	2020-10-12 14:27:27.776464
31521	CARG	2020-10-01	21.77	22.51	21.61	22.31	1038334	{}	\N	2020-10-12 14:27:27.78496
31522	CARG	2020-10-02	21.8	22.39	21.585	21.97	1057426	{}	\N	2020-10-12 14:27:27.789952
31523	CARG	2020-10-05	22.1	22.73	21.81	22.61	1963459	{}	\N	2020-10-12 14:27:27.794296
31524	CARG	2020-10-06	22.64	23.155	22.21	22.51	755460	{}	\N	2020-10-12 14:27:27.798716
31525	CARG	2020-10-07	22.67	23.15	22.36	22.5	624744	{}	\N	2020-10-12 14:27:27.803104
31526	CARG	2020-10-08	22.89	22.9	22.26	22.36	718266	{}	\N	2020-10-12 14:27:27.807448
31527	CARG	2020-10-09	22.72	22.84	22.22	22.63	708585	{}	\N	2020-10-12 14:27:27.811942
31528	CARG	2020-10-12	22.78	22.86	22.36	22.57	65960	{}	\N	2020-10-12 14:27:27.816709
31529	CB	2020-09-28	116.01	118.38	115.34	116.97	1612256	{}	\N	2020-10-12 14:27:30.088429
31530	CB	2020-09-29	116.32	116.42	113.16	114.34	4925072	{}	\N	2020-10-12 14:27:30.099389
31531	CB	2020-09-30	114.88	117.13	114.54	116.12	3531126	{}	\N	2020-10-12 14:27:30.107474
31532	CB	2020-10-01	116.5	117.1665	114.56	115.88	2796149	{}	\N	2020-10-12 14:27:30.113555
31533	CB	2020-10-02	114.76	117.17	114.21	116.56	2401880	{}	\N	2020-10-12 14:27:30.118916
31534	CB	2020-10-05	117.91	119.5995	117.44	119.1	2602005	{}	\N	2020-10-12 14:27:30.124347
31535	CB	2020-10-06	120	120.16	116.835	117.26	2959361	{}	\N	2020-10-12 14:27:30.129175
31536	CB	2020-10-07	117.4	119.38	117.2801	119.11	3762920	{}	\N	2020-10-12 14:27:30.133549
31537	CB	2020-10-08	119.6	122.06	119.13	121.28	2405437	{}	\N	2020-10-12 14:27:30.138068
31538	CB	2020-10-09	121.87	121.94	119.97	120.45	2396084	{}	\N	2020-10-12 14:27:30.143255
31539	CB	2020-10-12	120.65	122	119.93	121.91	405263	{}	\N	2020-10-12 14:27:30.147525
31540	CBIO	2020-09-28	4.46	4.54	4.33	4.38	105101	{}	\N	2020-10-12 14:27:31.190512
31541	CBIO	2020-09-29	4.35	4.38	4.05	4.35	267107	{}	\N	2020-10-12 14:27:31.199777
31542	CBIO	2020-09-30	4.32	4.7	4.21	4.3	193500	{}	\N	2020-10-12 14:27:31.206841
31543	CBIO	2020-10-01	4.31	4.66	4.31	4.61	184743	{}	\N	2020-10-12 14:27:31.213681
31544	CBIO	2020-10-02	4.5	4.65	4.45	4.55	166380	{}	\N	2020-10-12 14:27:31.219171
31545	CBIO	2020-10-05	4.63	4.69	4.5391	4.65	148451	{}	\N	2020-10-12 14:27:31.223636
31546	CBIO	2020-10-06	4.43	4.73	4.43	4.6	69340	{}	\N	2020-10-12 14:27:31.22805
31547	CBIO	2020-10-07	4.51	4.7	4.51	4.66	93285	{}	\N	2020-10-12 14:27:31.232337
31548	CBIO	2020-10-08	4.69	4.69	4.59	4.61	85571	{}	\N	2020-10-12 14:27:31.236632
31549	CBIO	2020-10-09	4.65	4.68	4.6	4.65	75577	{}	\N	2020-10-12 14:27:31.240977
31550	CBIO	2020-10-12	4.7	4.7	4.6378	4.6384	4216	{}	\N	2020-10-12 14:27:31.245178
31551	CBZ	2020-09-28	22.8	23.14	22.65	23.02	271847	{}	\N	2020-10-12 14:27:33.147617
31552	CBZ	2020-09-29	23.08	23.08	22.69	23	692228	{}	\N	2020-10-12 14:27:33.156639
31553	CBZ	2020-09-30	23.13	23.18	22.63	22.87	741622	{}	\N	2020-10-12 14:27:33.162932
31554	CBZ	2020-10-01	22.89	23.25	22.72	23.24	728350	{}	\N	2020-10-12 14:27:33.168759
31555	CBZ	2020-10-02	22.95	23.62	22.84	23.42	619156	{}	\N	2020-10-12 14:27:33.174269
31556	CBZ	2020-10-05	23.5	23.77	23.32	23.53	272780	{}	\N	2020-10-12 14:27:33.178836
31557	CBZ	2020-10-06	23.78	23.84	23.34	23.36	418472	{}	\N	2020-10-12 14:27:33.182925
31558	CBZ	2020-10-07	23.52	23.87	23.37	23.76	418130	{}	\N	2020-10-12 14:27:33.187297
31559	CBZ	2020-10-08	23.93	24.18	23.83	24.01	381325	{}	\N	2020-10-12 14:27:33.191491
31560	CBZ	2020-10-09	24.2	24.63	24.04	24.5	364231	{}	\N	2020-10-12 14:27:33.195707
31561	CBZ	2020-10-12	24.47	24.72	24.46	24.72	17358	{}	\N	2020-10-12 14:27:33.199845
31562	CCL	2020-09-28	15.54	15.55	14.78	15.31	37037075	{}	\N	2020-10-12 14:27:34.829911
31563	CCL	2020-09-29	15.25	15.43	14.815	15.07	46955310	{}	\N	2020-10-12 14:27:34.840182
31564	CCL	2020-09-30	15.42	15.96	15.11	15.18	88295639	{}	\N	2020-10-12 14:27:34.846188
31565	CCL	2020-10-01	15.43	15.43	14.7	15.03	56021325	{}	\N	2020-10-12 14:27:34.851947
31566	CCL	2020-10-02	14.26	15.215	14.23	15.16	61602311	{}	\N	2020-10-12 14:27:34.857142
31567	CCL	2020-10-05	15.11	15.23	14.8	14.96	44856588	{}	\N	2020-10-12 14:27:34.862141
31568	CCL	2020-10-06	15.26	15.97	15.1218	15.19	85986893	{}	\N	2020-10-12 14:27:34.866356
31569	CCL	2020-10-07	15.48	16.045	15.29	15.99	60182441	{}	\N	2020-10-12 14:27:34.870406
31570	CCL	2020-10-08	16.16	16.17	15.37	15.61	81933536	{}	\N	2020-10-12 14:27:34.874612
31571	CCL	2020-10-09	15.72	16.065	15.5409	15.69	67542908	{}	\N	2020-10-12 14:27:34.878601
31572	CCL	2020-10-12	15.45	15.51	15.31	15.39	19327975	{}	\N	2020-10-12 14:27:34.882638
31573	BYD	2020-09-28	30.25	31.31	29.45	30.92	2504081	{}	\N	2020-10-12 14:27:36.385182
31574	BYD	2020-09-29	30.83	30.83	29.64	30.07	2791489	{}	\N	2020-10-12 14:27:36.391757
31575	BYD	2020-09-30	30.3	31.52	30.2	30.69	3235729	{}	\N	2020-10-12 14:27:36.397255
31576	BYD	2020-10-01	31.06	31.61	30.46	31.44	3317500	{}	\N	2020-10-12 14:27:36.402617
31577	BYD	2020-10-02	30.17	32.29	30.01	31.97	3282156	{}	\N	2020-10-12 14:27:36.407773
31578	BYD	2020-10-05	32.24	32.56	31.29	32.23	3683232	{}	\N	2020-10-12 14:27:36.411841
31579	BYD	2020-10-06	32.5	32.92	30.86	30.91	3251598	{}	\N	2020-10-12 14:27:36.416295
31580	BYD	2020-10-07	31.45	31.79	30.85	31.29	3737023	{}	\N	2020-10-12 14:27:36.420471
31581	BYD	2020-10-08	31.66	32.47	31.66	32.4	2701715	{}	\N	2020-10-12 14:27:36.4245
31582	BYD	2020-10-09	32.58	32.75	31.91	32.28	2303716	{}	\N	2020-10-12 14:27:36.428462
31583	BYD	2020-10-12	32.75	33.01	32.165	32.29	533498	{}	\N	2020-10-12 14:27:36.432623
31584	BZQ	2020-09-28	15.69	17.67	15.6	17.44	1027039	{}	\N	2020-10-12 14:27:38.439752
31585	BZQ	2020-09-29	17.66	17.87	16.89	17.87	977328	{}	\N	2020-10-12 14:27:38.451152
31586	BZQ	2020-09-30	17.58	17.58	16.87	17.13	834174	{}	\N	2020-10-12 14:27:38.458339
31587	BZQ	2020-10-01	17.49	17.86	17.085	17.15	687634	{}	\N	2020-10-12 14:27:38.464204
31588	BZQ	2020-10-02	17.39	17.81	16.71	17.79	979565	{}	\N	2020-10-12 14:27:38.469322
31589	BZQ	2020-10-05	17.4	17.55	16.12	16.24	630583	{}	\N	2020-10-12 14:27:38.47428
31590	BZQ	2020-10-06	15.8	16.85	15.4	16.62	888738	{}	\N	2020-10-12 14:27:38.478332
31591	BZQ	2020-10-07	16.35	17.055	16.22	16.75	724665	{}	\N	2020-10-12 14:27:38.482446
31592	BZQ	2020-10-08	16.68	16.78	15.65	15.75	825798	{}	\N	2020-10-12 14:27:38.486357
31593	BZQ	2020-10-09	15.6	15.74	15.06	15.5	758908	{}	\N	2020-10-12 14:27:38.490301
31594	BZQ	2020-10-12	15.34	15.45	15.25	15.31	61026	{}	\N	2020-10-12 14:27:38.494235
31595	CAC	2020-09-28	29.72	30.57	29.72	30.21	31133	{}	\N	2020-10-12 14:27:39.569539
31596	CAC	2020-09-29	29.99	30.37	29.47	30.11	31369	{}	\N	2020-10-12 14:27:39.579023
31597	CAC	2020-09-30	30.21	30.56	29.96	30.225	42943	{}	\N	2020-10-12 14:27:39.585126
31598	CAC	2020-10-01	30.14	30.3	29.76	30.3	38306	{}	\N	2020-10-12 14:27:39.590708
31599	CAC	2020-10-02	29.94	31.1871	29.88	31.05	27093	{}	\N	2020-10-12 14:27:39.595551
31600	CAC	2020-10-05	31.36	32.05	31.36	32.02	27585	{}	\N	2020-10-12 14:27:39.60119
31601	CAC	2020-10-06	32.49	33.02	31.03	31.89	35441	{}	\N	2020-10-12 14:27:39.605327
31602	CAC	2020-10-07	32.35	32.79	32.24	32.63	34712	{}	\N	2020-10-12 14:27:39.60943
31603	CAC	2020-10-08	33.11	33.23	32.6	33.04	31862	{}	\N	2020-10-12 14:27:39.61335
31604	CAC	2020-10-09	33.33	33.33	32.5	32.51	33437	{}	\N	2020-10-12 14:27:39.617526
31605	CAC	2020-10-12	32.52	32.5807	32.3	32.42	4231	{}	\N	2020-10-12 14:27:39.621665
31606	CAI	2020-09-28	26.1	28.3638	26.1	27.57	115013	{}	\N	2020-10-12 14:27:40.83722
31607	CAI	2020-09-29	27.91	27.91	26.5238	27.43	198264	{}	\N	2020-10-12 14:27:40.846094
31608	CAI	2020-09-30	27.43	28.11	27.24	27.53	135657	{}	\N	2020-10-12 14:27:40.852288
31609	CAI	2020-10-01	27.89	27.93	26.22	27.07	220787	{}	\N	2020-10-12 14:27:40.857828
31610	CAI	2020-10-02	26.45	27.94	26.36	27.59	158243	{}	\N	2020-10-12 14:27:40.863292
31611	CAI	2020-10-05	27.61	28.74	27.35	27.92	155130	{}	\N	2020-10-12 14:27:40.867854
31612	CAI	2020-10-06	28.31	28.89	27.34	27.84	400969	{}	\N	2020-10-12 14:27:40.872222
31613	CAI	2020-10-07	28.17	28.98	27.76	28.12	137445	{}	\N	2020-10-12 14:27:40.876491
31614	CAI	2020-10-08	28.6	28.85	28.035	28.36	149327	{}	\N	2020-10-12 14:27:40.880878
31615	CAI	2020-10-09	28.55	28.82	27.65	27.99	92367	{}	\N	2020-10-12 14:27:40.885042
31616	CAI	2020-10-12	28.19	28.255	27.72	28.255	5582	{}	\N	2020-10-12 14:27:40.889395
31617	CALX	2020-09-28	17.79	18.15	17.45	18.03	451291	{}	\N	2020-10-12 14:27:42.663472
31618	CALX	2020-09-29	18.16	18.53	17.865	18.27	825202	{}	\N	2020-10-12 14:27:42.672485
31619	CALX	2020-09-30	18.26	18.66	17.75	17.78	1027887	{}	\N	2020-10-12 14:27:42.679541
31620	CALX	2020-10-01	18	18.21	17.2	18.18	1716476	{}	\N	2020-10-12 14:27:42.685794
31621	CALX	2020-10-02	17.64	19.19	17.64	19.11	2380713	{}	\N	2020-10-12 14:27:42.691868
31622	CALX	2020-10-05	19.25	19.7924	18.78	19.68	1586279	{}	\N	2020-10-12 14:27:42.696831
31623	CALX	2020-10-06	19.65	20.13	19.06	19.07	2116077	{}	\N	2020-10-12 14:27:42.701459
31624	CALX	2020-10-07	19.37	19.97	19.2982	19.91	1376572	{}	\N	2020-10-12 14:27:42.706025
31625	CALX	2020-10-08	19.97	20.14	19.56	19.79	654510	{}	\N	2020-10-12 14:27:42.710631
31626	CALX	2020-10-09	20	20.54	19.89	20.36	957178	{}	\N	2020-10-12 14:27:42.715353
31627	CALX	2020-10-12	20.5	20.54	19.61	19.75	255988	{}	\N	2020-10-12 14:27:42.720036
31628	CARA	2020-09-28	12.98	13.065	12.38	12.4	637875	{}	\N	2020-10-12 14:27:43.750869
31629	CARA	2020-09-29	12.4	12.7	12.3	12.34	494658	{}	\N	2020-10-12 14:27:43.759189
31630	CARA	2020-09-30	12.37	12.94	12.37	12.725	497043	{}	\N	2020-10-12 14:27:43.764825
31631	CARA	2020-10-01	12.69	13	12.65	12.74	541104	{}	\N	2020-10-12 14:27:43.77001
31632	CARA	2020-10-02	12.47	13.27	12.47	13.18	461358	{}	\N	2020-10-12 14:27:43.77469
31633	CARA	2020-10-05	13.27	13.86	13.27	13.77	470140	{}	\N	2020-10-12 14:27:43.778872
31634	CARA	2020-10-06	13.8	14	13.56	13.56	423037	{}	\N	2020-10-12 14:27:43.782989
31635	CARA	2020-10-07	13.63	14	13.63	13.93	359808	{}	\N	2020-10-12 14:27:43.786473
31636	CARA	2020-10-08	14.08	14.47	13.89	14.11	448980	{}	\N	2020-10-12 14:27:43.789945
31637	CARA	2020-10-09	14.15	14.34	13.9399	13.99	364523	{}	\N	2020-10-12 14:27:43.79362
31638	CARA	2020-10-12	13.93	14.125	13.6	13.9	107309	{}	\N	2020-10-12 14:27:43.79769
31639	CARZ	2020-09-28	38.34	38.445	38.2316	38.445	1138	{}	\N	2020-10-12 14:27:44.779769
31640	CARZ	2020-09-29	38.6048	38.8759	38.5653	38.8759	3750	{}	\N	2020-10-12 14:27:44.791115
31641	CARZ	2020-09-30	38.9772	39.44	38.84	39.33	2865	{}	\N	2020-10-12 14:27:44.798509
31642	CARZ	2020-10-01	39.33	39.49	39.13	39.4223	6678	{}	\N	2020-10-12 14:27:44.804881
31643	CARZ	2020-10-02	38.55	39.362	38.55	39.29	1742	{}	\N	2020-10-12 14:27:44.80972
31644	CARZ	2020-10-05	39.8561	40.1	39.69	40.1	25822	{}	\N	2020-10-12 14:27:44.814653
31645	CARZ	2020-10-06	40.255	40.5	39.8709	39.8709	3827	{}	\N	2020-10-12 14:27:44.818939
31646	CARZ	2020-10-07	39.93	40.705	39.93	40.705	3211	{}	\N	2020-10-12 14:27:44.823716
31647	CARZ	2020-10-08	40.8235	40.8235	40.6601	40.82	3046	{}	\N	2020-10-12 14:27:44.828042
31648	CARZ	2020-10-09	40.6101	40.755	40.61	40.755	5561	{}	\N	2020-10-12 14:27:44.831933
31649	CARZ	2020-10-12	40.76	40.76	40.66	40.662	853	{}	\N	2020-10-12 14:27:44.836037
31650	CATO	2020-09-28	8.07	8.25	8.04	8.08	124150	{}	\N	2020-10-12 14:27:46.18925
31651	CATO	2020-09-29	8.08	8.08	7.79	7.8	295909	{}	\N	2020-10-12 14:27:46.19765
31652	CATO	2020-09-30	7.84	8.08	7.8	7.82	493103	{}	\N	2020-10-12 14:27:46.203794
31653	CATO	2020-10-01	7.85	7.99	7.77	7.83	358353	{}	\N	2020-10-12 14:27:46.209846
31654	CATO	2020-10-02	7.66	8.04	7.66	8.01	514922	{}	\N	2020-10-12 14:27:46.215203
31655	CATO	2020-10-05	8.1	8.24	7.91	7.94	289477	{}	\N	2020-10-12 14:27:46.220166
31656	CATO	2020-10-06	7.99	8.11	7.68	7.69	321495	{}	\N	2020-10-12 14:27:46.224815
31657	CATO	2020-10-07	7.78	8.015	7.7	7.73	450536	{}	\N	2020-10-12 14:27:46.229294
31658	CATO	2020-10-08	7.74	7.93	7.74	7.77	408121	{}	\N	2020-10-12 14:27:46.233679
31659	CATO	2020-10-09	7.84	8.04	7.73	7.74	399853	{}	\N	2020-10-12 14:27:46.238231
31660	CATO	2020-10-12	7.74	7.82	7.71	7.715	25110	{}	\N	2020-10-12 14:27:46.243112
31661	CBB	2020-09-28	15.02	15.08	15.01	15.04	478624	{}	\N	2020-10-12 14:27:47.373575
31662	CBB	2020-09-29	15.05	15.06	15	15.05	388141	{}	\N	2020-10-12 14:27:47.380067
31663	CBB	2020-09-30	15.02	15.065	14.97	15	1270911	{}	\N	2020-10-12 14:27:47.384744
31664	CBB	2020-10-01	15.11	15.11	15.03	15.05	646493	{}	\N	2020-10-12 14:27:47.389334
31665	CBB	2020-10-02	15	15.06	15	15.02	1164882	{}	\N	2020-10-12 14:27:47.394136
31666	CBB	2020-10-05	15.02	15.06	15.01	15.02	658040	{}	\N	2020-10-12 14:27:47.398729
31667	CBB	2020-10-06	15.02	15.04	15	15	776368	{}	\N	2020-10-12 14:27:47.403738
31668	CBB	2020-10-07	15.01	15.06	15.01	15.04	701412	{}	\N	2020-10-12 14:27:47.408166
31669	CBB	2020-10-08	15.04	15.07	15.03	15.05	491264	{}	\N	2020-10-12 14:27:47.412417
31670	CBB	2020-10-09	15.06	15.09	15.04	15.07	634138	{}	\N	2020-10-12 14:27:47.417201
31671	CBB	2020-10-12	15.09	15.12	15.06	15.095	190100	{}	\N	2020-10-12 14:27:47.421774
31672	CCC	2020-09-28	29.05	29.88	28.48	29.7	1299791	{}	\N	2020-10-12 14:27:48.498452
31673	CCC	2020-09-29	29.61	30.295	29.28	29.54	2151088	{}	\N	2020-10-12 14:27:48.509153
31674	CCC	2020-09-30	29.53	32.23	29.455	30.99	5993667	{}	\N	2020-10-12 14:27:48.516564
31675	CCC	2020-10-01	31.63	32.47	31.3	32.09	5680073	{}	\N	2020-10-12 14:27:48.523159
31676	CCC	2020-10-02	32.3	32.68	31.63	32.5	1884004	{}	\N	2020-10-12 14:27:48.528896
31677	CCC	2020-10-05	32.53	33.55	32.52	33.39	1585380	{}	\N	2020-10-12 14:27:48.533419
31678	CCC	2020-10-06	33.29	33.41	32.57	32.72	1661368	{}	\N	2020-10-12 14:27:48.538113
31679	CCC	2020-10-07	32.96	33.24	32.3083	32.87	2731172	{}	\N	2020-10-12 14:27:48.542895
31680	CCC	2020-10-08	33.1	33.265	32.04	32.17	1650302	{}	\N	2020-10-12 14:27:48.547222
31681	CCC	2020-10-09	32.41	32.87	31.975	32.8	1787325	{}	\N	2020-10-12 14:27:48.551637
31682	CCC	2020-10-12	33.18	33.29	32.7	32.9	851839	{}	\N	2020-10-12 14:27:48.556085
31683	CCJ	2020-09-28	10.07	10.17	9.99	10.11	1733347	{}	\N	2020-10-12 14:27:50.308805
31684	CCJ	2020-09-29	10.09	10.235	10.04	10.18	2833423	{}	\N	2020-10-12 14:27:50.320549
31685	CCJ	2020-09-30	10.19	10.3	10.06	10.1	2535314	{}	\N	2020-10-12 14:27:50.327084
31686	CCJ	2020-10-01	10.1	10.12	9.81	9.85	4055154	{}	\N	2020-10-12 14:27:50.338466
31687	CCJ	2020-10-02	9.74	9.82	9.54	9.75	7566275	{}	\N	2020-10-12 14:27:50.345097
31688	CCJ	2020-10-05	9.8	9.96	9.78	9.89	2393957	{}	\N	2020-10-12 14:27:50.349793
31689	CCJ	2020-10-06	9.9	9.925	9.61	9.61	4288764	{}	\N	2020-10-12 14:27:50.353628
31690	CCJ	2020-10-07	9.72	9.8	9.53	9.76	3811294	{}	\N	2020-10-12 14:27:50.357321
31691	CCJ	2020-10-08	9.82	9.83	9.66	9.69	2798533	{}	\N	2020-10-12 14:27:50.361252
31692	CCJ	2020-10-09	9.7	9.77	9.59	9.73	3852889	{}	\N	2020-10-12 14:27:50.364931
31693	CCJ	2020-10-12	9.76	9.78	9.56	9.5683	391396	{}	\N	2020-10-12 14:27:50.368498
31694	CCRN	2020-09-28	6.21	6.46	6.21	6.34	181250	{}	\N	2020-10-12 14:27:51.582209
31695	CCRN	2020-09-29	6.1892	6.54	6.1892	6.45	120790	{}	\N	2020-10-12 14:27:51.591458
31696	CCRN	2020-09-30	6.52	6.63	6.43	6.49	173949	{}	\N	2020-10-12 14:27:51.597916
31697	CCRN	2020-10-01	6.54	6.6	6.23	6.29	88133	{}	\N	2020-10-12 14:27:51.604289
31698	CCRN	2020-10-02	6.13	6.365	6.07	6.18	138469	{}	\N	2020-10-12 14:27:51.609438
31699	CCRN	2020-10-05	6.25	6.72	6.22	6.65	154484	{}	\N	2020-10-12 14:27:51.613772
31700	CCRN	2020-10-06	6.71	6.88	6.49	6.49	162253	{}	\N	2020-10-12 14:27:51.618108
31701	CCRN	2020-10-07	6.53	6.75	6.45	6.7	143066	{}	\N	2020-10-12 14:27:51.622466
31702	CCRN	2020-10-08	6.78	6.88	6.67	6.82	196971	{}	\N	2020-10-12 14:27:51.626688
31703	CCRN	2020-10-09	6.87	7.39	6.87	7.27	172046	{}	\N	2020-10-12 14:27:51.630991
31704	CCRN	2020-10-12	7.33	7.38	7.21	7.31	14929	{}	\N	2020-10-12 14:27:51.635315
31705	GRAM	2020-09-28	2.32	2.35	2.32	2.34	5954	{}	\N	2020-10-12 14:27:52.62642
31706	GRAM	2020-09-29	2.33	2.35	2.32	2.34	6842	{}	\N	2020-10-12 14:27:52.634206
31707	GRAM	2020-09-30	2.33	2.35	2.27	2.27	20852	{}	\N	2020-10-12 14:27:52.639781
31708	GRAM	2020-10-01	2.28	2.28	2.28	2.28	210	{}	\N	2020-10-12 14:27:52.644824
31709	GRAM	2020-10-02	2.29	2.32	2.211	2.32	5606	{}	\N	2020-10-12 14:27:52.649668
31710	GRAM	2020-10-05	2.32	2.32	2.2	2.2	45342	{}	\N	2020-10-12 14:27:52.654776
31711	GRAM	2020-10-06	2.22	2.34	2.21	2.34	6116	{}	\N	2020-10-12 14:27:52.660424
31712	GRAM	2020-10-07	2.28	2.28	2.21	2.27	1422	{}	\N	2020-10-12 14:27:52.66518
31713	GRAM	2020-10-08	2.23	2.31	2.22	2.31	53166	{}	\N	2020-10-12 14:27:52.669564
31714	GRAM	2020-10-09	2.26	2.3	2.25	2.25	5406	{}	\N	2020-10-12 14:27:52.674422
31715	GRAM	2020-10-12	2.24	2.29	2.24	2.2645	4546	{}	\N	2020-10-12 14:27:52.678843
31716	CABO	2020-09-28	1770.3	1780.99	1748.32	1762.68	34548	{}	\N	2020-10-12 14:27:53.743596
31717	CABO	2020-09-29	1800	1850.22	1800	1805	108325	{}	\N	2020-10-12 14:27:53.752303
31718	CABO	2020-09-30	1798.32	1889.03	1798.32	1885.43	158726	{}	\N	2020-10-12 14:27:53.75836
31719	CABO	2020-10-01	1891	1892.05	1840.01	1851.23	104899	{}	\N	2020-10-12 14:27:53.764512
31720	CABO	2020-10-02	1843	1851.9999	1818.9	1841.29	76020	{}	\N	2020-10-12 14:27:53.76907
31721	CABO	2020-10-05	1865.49	1880	1832.515	1837.66	96716	{}	\N	2020-10-12 14:27:53.773499
31722	CABO	2020-10-06	1849.5	1856.48	1826.1	1845.04	68036	{}	\N	2020-10-12 14:27:53.777576
31723	CABO	2020-10-07	1857.065	1878.75	1836	1849.83	54382	{}	\N	2020-10-12 14:27:53.781369
31724	CABO	2020-10-08	1862.7	1865.16	1817.915	1830.13	73058	{}	\N	2020-10-12 14:27:53.785016
31725	CABO	2020-10-09	1841.98	1841.98	1820	1835.86	43659	{}	\N	2020-10-12 14:27:53.788562
31726	CABO	2020-10-12	1848	1848	1838.995	1838.995	1816	{}	\N	2020-10-12 14:27:53.792145
31727	CAKE	2020-09-28	27.24	28.31	26.6	28.11	2111565	{}	\N	2020-10-12 14:27:54.768671
31728	CAKE	2020-09-29	28.11	28.14	26.52	27.05	1439910	{}	\N	2020-10-12 14:27:54.775273
31729	CAKE	2020-09-30	27.31	28.25	27.31	27.74	1583103	{}	\N	2020-10-12 14:27:54.78083
31730	CAKE	2020-10-01	28	28.64	27.52	28.6	1158351	{}	\N	2020-10-12 14:27:54.785756
31731	CAKE	2020-10-02	27.63	29.43	27.33	29.29	1270445	{}	\N	2020-10-12 14:27:54.791168
31732	CAKE	2020-10-05	29.6	29.93	29.03	29.52	1144610	{}	\N	2020-10-12 14:27:54.797157
31733	CAKE	2020-10-06	29.5	30.73	29.19	29.24	1389091	{}	\N	2020-10-12 14:27:54.802352
31734	CAKE	2020-10-07	29.98	30.1	29.02	29.79	1149178	{}	\N	2020-10-12 14:27:54.807329
31735	CAKE	2020-10-08	30.24	31.25	29.58	31.12	1013081	{}	\N	2020-10-12 14:27:54.812163
31736	CAKE	2020-10-09	31.5	32.1	31.45	31.67	1510657	{}	\N	2020-10-12 14:27:54.816401
31737	CAKE	2020-10-12	31.85	31.85	31.02	31.09	228008	{}	\N	2020-10-12 14:27:54.820294
31738	CAPR	2020-09-28	5.34	5.6	5.07	5.24	732495	{}	\N	2020-10-12 14:27:55.827243
31739	CAPR	2020-09-29	5.23	5.45	5.13	5.41	517331	{}	\N	2020-10-12 14:27:55.837451
31740	CAPR	2020-09-30	5.53	5.59	5.13	5.25	645932	{}	\N	2020-10-12 14:27:55.844643
31741	CAPR	2020-10-01	5.26	5.3621	5.0901	5.23	634589	{}	\N	2020-10-12 14:27:55.850278
31742	CAPR	2020-10-02	4.87	5.18	4.77	4.92	806276	{}	\N	2020-10-12 14:27:55.855257
31743	CAPR	2020-10-05	4.88	5.2	4.88	5.18	452845	{}	\N	2020-10-12 14:27:55.859925
31744	CAPR	2020-10-06	5.19	5.24	5.01	5.04	478865	{}	\N	2020-10-12 14:27:55.864171
31745	CAPR	2020-10-07	5.08	5.15	4.9	5	467625	{}	\N	2020-10-12 14:27:55.867748
31746	CAPR	2020-10-08	5.01	5.0615	4.81	4.89	435975	{}	\N	2020-10-12 14:27:55.87126
31747	CAPR	2020-10-09	4.91	4.98	4.83	4.93	437343	{}	\N	2020-10-12 14:27:55.874903
31748	CAPR	2020-10-12	5	5.09	4.9002	4.9765	140246	{}	\N	2020-10-12 14:27:55.878427
31749	CASI	2020-09-28	1.54	1.56	1.5152	1.52	105069	{}	\N	2020-10-12 14:27:56.854113
31750	CASI	2020-09-29	1.53	1.6	1.52	1.55	175451	{}	\N	2020-10-12 14:27:56.863878
31751	CASI	2020-09-30	1.56	1.575	1.53	1.53	70247	{}	\N	2020-10-12 14:27:56.870255
31752	CASI	2020-10-01	1.51	1.58	1.49	1.56	182918	{}	\N	2020-10-12 14:27:56.876185
31753	CASI	2020-10-02	1.48	1.55	1.48	1.53	116890	{}	\N	2020-10-12 14:27:56.880955
31754	CASI	2020-10-05	1.53	1.63	1.52	1.59	217468	{}	\N	2020-10-12 14:27:56.885871
31755	CASI	2020-10-06	1.62	1.69	1.59	1.61	142312	{}	\N	2020-10-12 14:27:56.889923
31756	CASI	2020-10-07	1.63	1.75	1.58	1.72	271749	{}	\N	2020-10-12 14:27:56.893703
31757	CASI	2020-10-08	1.76	1.763	1.71	1.73	110220	{}	\N	2020-10-12 14:27:56.897559
31758	CASI	2020-10-09	1.78	1.82	1.73	1.8	136819	{}	\N	2020-10-12 14:27:56.901283
31759	CASI	2020-10-12	1.8	1.8	1.76	1.78	16424	{}	\N	2020-10-12 14:27:56.905281
31760	CBAN	2020-09-28	10.1	10.64	10.1	10.43	13911	{}	\N	2020-10-12 14:27:58.078577
31761	CBAN	2020-09-29	10.69	10.89	10.36	10.8	12098	{}	\N	2020-10-12 14:27:58.087448
31762	CBAN	2020-09-30	11.03	11.04	10.75	10.75	2315	{}	\N	2020-10-12 14:27:58.092844
31763	CBAN	2020-10-01	10.61	11	10.57	10.57	4781	{}	\N	2020-10-12 14:27:58.098221
31764	CBAN	2020-10-02	10.35	10.5	10.35	10.39	3900	{}	\N	2020-10-12 14:27:58.103625
31765	CBAN	2020-10-05	10.5	11.01	10.5	10.73	7624	{}	\N	2020-10-12 14:27:58.107693
31766	CBAN	2020-10-06	10.9	11.025	10.4	10.45	6397	{}	\N	2020-10-12 14:27:58.111542
31767	CBAN	2020-10-07	10.85	10.85	10.21	10.8	12837	{}	\N	2020-10-12 14:27:58.11542
31768	CBAN	2020-10-08	10.94	11.3999	10.57	11	35642	{}	\N	2020-10-12 14:27:58.11957
31769	CBAN	2020-10-09	11.2	11.4	11.2	11.4	22818	{}	\N	2020-10-12 14:27:58.123573
31770	CBAN	2020-10-12	11.45	12.5	11.45	11.8	6232	{}	\N	2020-10-12 14:27:58.127867
31771	CBMG	2020-09-28	18.47	18.5	18.17	18.3	35322	{}	\N	2020-10-12 14:27:59.274079
31772	CBMG	2020-09-29	18.31	18.45	18.24	18.3	22771	{}	\N	2020-10-12 14:27:59.284042
31773	CBMG	2020-09-30	18.38	18.49	18.25	18.34	83316	{}	\N	2020-10-12 14:27:59.289812
31774	CBMG	2020-10-01	18.29	18.35	18.25	18.31	57606	{}	\N	2020-10-12 14:27:59.29652
31775	CBMG	2020-10-02	18.23	18.45	18.23	18.38	71436	{}	\N	2020-10-12 14:27:59.301504
31776	CBMG	2020-10-05	18.45	18.5	18.31	18.44	38810	{}	\N	2020-10-12 14:27:59.306531
31777	CBMG	2020-10-06	18.47	18.53	18.28	18.35	76357	{}	\N	2020-10-12 14:27:59.310324
31778	CBMG	2020-10-07	18.44	18.44	18.2404	18.34	40724	{}	\N	2020-10-12 14:27:59.314055
31779	CBMG	2020-10-08	18.35	18.35	18.3	18.34	27938	{}	\N	2020-10-12 14:27:59.318191
31780	CBMG	2020-10-09	18.39	18.46	18.32	18.46	45760	{}	\N	2020-10-12 14:27:59.322256
31781	CBMG	2020-10-12	18.45	18.45	18.4	18.45	3643	{}	\N	2020-10-12 14:27:59.326162
31782	CCD	2020-09-28	22.69	22.84	22.62	22.77	50173	{}	\N	2020-10-12 14:28:02.306053
31783	CCD	2020-09-29	22.65	22.77	22.61	22.72	63393	{}	\N	2020-10-12 14:28:02.313914
31784	CCD	2020-09-30	22.84	22.9694	22.7	22.88	75592	{}	\N	2020-10-12 14:28:02.320037
31785	CCD	2020-10-01	22.8996	23.1696	22.89	23.1	78709	{}	\N	2020-10-12 14:28:02.325057
31786	CCD	2020-10-02	22.91	23.275	22.81	23.27	137969	{}	\N	2020-10-12 14:28:02.331042
31787	CCD	2020-10-05	23.26	23.48	23.245	23.44	101721	{}	\N	2020-10-12 14:28:02.335737
31788	CCD	2020-10-06	23.51	24.08	23.5	23.65	91346	{}	\N	2020-10-12 14:28:02.34112
31789	CCD	2020-10-07	23.89	24.52	23.71	24.26	119769	{}	\N	2020-10-12 14:28:02.345082
31790	CCD	2020-10-08	24.4	24.4	24.2309	24.28	75665	{}	\N	2020-10-12 14:28:02.349146
31791	CCD	2020-10-09	24.43	24.7	24.35	24.58	86524	{}	\N	2020-10-12 14:28:02.353652
31792	CCD	2020-10-12	24.92	24.92	24.73	24.805	32411	{}	\N	2020-10-12 14:28:02.358452
31793	CCMP	2020-09-28	140.22	142.69	139.01	142.44	115566	{}	\N	2020-10-12 14:28:03.489046
31794	CCMP	2020-09-29	142.15	144.31	141.14	142.49	107263	{}	\N	2020-10-12 14:28:03.49946
31795	CCMP	2020-09-30	144.16	144.895	142.35	142.81	164380	{}	\N	2020-10-12 14:28:03.506497
31796	CCMP	2020-10-01	143.49	146.44	142.53	145.99	117693	{}	\N	2020-10-12 14:28:03.512311
31797	CCMP	2020-10-02	143.32	146.1	142.42	142.43	113131	{}	\N	2020-10-12 14:28:03.517236
31798	CCMP	2020-10-05	143.27	148.195	143.27	147.91	129444	{}	\N	2020-10-12 14:28:03.522395
31799	CCMP	2020-10-06	148.7	152.22	146.785	147.64	130093	{}	\N	2020-10-12 14:28:03.526523
31800	CCMP	2020-10-07	149.13	153.52	149.13	152.71	138106	{}	\N	2020-10-12 14:28:03.530457
31801	CCMP	2020-10-08	154.56	155.585	152.49	155.36	66327	{}	\N	2020-10-12 14:28:03.534364
31802	CCMP	2020-10-09	157.05	159.93	153.22	156.55	117571	{}	\N	2020-10-12 14:28:03.538913
31803	CCMP	2020-10-12	158.9	159.9	156.455	159.1	72327	{}	\N	2020-10-12 14:28:03.543209
31804	CCS	2020-09-28	38.6	40.275	38.4	40.06	208726	{}	\N	2020-10-12 14:28:06.068766
31805	CCS	2020-09-29	39.98	40.51	39.55	40.2	475416	{}	\N	2020-10-12 14:28:06.078969
31806	CCS	2020-09-30	40.6	43.335	40.6	42.33	1489425	{}	\N	2020-10-12 14:28:06.086494
31807	CCS	2020-10-01	42.7	43.97	42.11	43.88	799366	{}	\N	2020-10-12 14:28:06.09301
31808	CCS	2020-10-02	43.1	44.9	43	44.59	520300	{}	\N	2020-10-12 14:28:06.097597
31809	CCS	2020-10-05	45.19	45.6	43.25	43.89	601428	{}	\N	2020-10-12 14:28:06.102704
31810	CCS	2020-10-06	44.27	44.46	41.93	42.46	795424	{}	\N	2020-10-12 14:28:06.106465
31811	CCS	2020-10-07	43.34	44.37	41.99	43	624409	{}	\N	2020-10-12 14:28:06.110385
31812	CCS	2020-10-08	45.62	46.46	44.685	45.69	867761	{}	\N	2020-10-12 14:28:06.114163
31813	CCS	2020-10-09	46.3	47.35	46.125	46.69	628178	{}	\N	2020-10-12 14:28:06.118084
31814	CCS	2020-10-12	47.4	47.58	46.005	46.375	215915	{}	\N	2020-10-12 14:28:06.122082
31815	CDL	2020-09-28	40.17	40.6531	40.17	40.5062	17970	{}	\N	2020-10-12 14:28:07.629849
31816	CDL	2020-09-29	40.31	40.32	39.86	40.07	22543	{}	\N	2020-10-12 14:28:07.63777
31817	CDL	2020-09-30	40.11	40.6691	40.11	40.4401	50012	{}	\N	2020-10-12 14:28:07.643979
31818	CDL	2020-10-01	40.46	40.537	40.0313	40.218	16543	{}	\N	2020-10-12 14:28:07.650038
31819	CDL	2020-10-02	39.58	40.8948	39.58	40.74	9431	{}	\N	2020-10-12 14:28:07.656271
31820	CDL	2020-10-05	41	41.56	41	41.5375	37131	{}	\N	2020-10-12 14:28:07.66142
31821	CDL	2020-10-06	41.64	42.25	41.26	41.4269	31599	{}	\N	2020-10-12 14:28:07.666207
31822	CDL	2020-10-07	41.66	41.93	41.58	41.9204	29776	{}	\N	2020-10-12 14:28:07.671106
31823	CDL	2020-10-08	42.1773	42.61	42.1073	42.61	10955	{}	\N	2020-10-12 14:28:07.675701
31824	CDL	2020-10-09	42.92	42.92	42.44	42.494	8897	{}	\N	2020-10-12 14:28:07.680482
31825	CDL	2020-10-12	42.71	42.71	42.5075	42.63	4087	{}	\N	2020-10-12 14:28:07.685001
31826	CDW	2020-09-28	114.44	116.72	113.11	116.5	1161785	{}	\N	2020-10-12 14:28:08.68622
31827	CDW	2020-09-29	116.99	118.965	116.51	118.11	745870	{}	\N	2020-10-12 14:28:08.696342
31828	CDW	2020-09-30	118.53	121.89	118.1	119.56	688742	{}	\N	2020-10-12 14:28:08.703639
31829	CDW	2020-10-01	120.445	124.53	120.445	123.5	941349	{}	\N	2020-10-12 14:28:08.709532
31830	CDW	2020-10-02	120.64	124.62	120.42	123.49	1051325	{}	\N	2020-10-12 14:28:08.714172
31831	CDW	2020-10-05	124.35	126.43	124.06	125.88	639033	{}	\N	2020-10-12 14:28:08.719083
31832	CDW	2020-10-06	126.13	127.24	122.52	123.07	914036	{}	\N	2020-10-12 14:28:08.723505
31833	CDW	2020-10-07	124.68	126.24	124.3	125.93	478374	{}	\N	2020-10-12 14:28:08.727232
31834	CDW	2020-10-08	126.73	127.16	125.57	126.2	386050	{}	\N	2020-10-12 14:28:08.730932
31835	CDW	2020-10-09	127.25	127.83	125.62	125.99	474720	{}	\N	2020-10-12 14:28:08.734739
31836	CDW	2020-10-12	127.86	128.15	126.48	128.14	82007	{}	\N	2020-10-12 14:28:08.738456
31837	CEL	2020-09-28	3.88	3.88	3.815	3.815	22285	{}	\N	2020-10-12 14:28:10.239122
31838	CEL	2020-09-29	3.76	3.76	3.69	3.69	6746	{}	\N	2020-10-12 14:28:10.24859
31839	CEL	2020-09-30	4.1	4.1332	4.01	4.09	114780	{}	\N	2020-10-12 14:28:10.255002
31840	CEL	2020-10-01	4.07	4.07	3.91	3.91	29432	{}	\N	2020-10-12 14:28:10.2605
31841	CEL	2020-10-02	3.94	3.98	3.85	3.85	41622	{}	\N	2020-10-12 14:28:10.265129
31842	CEL	2020-10-05	3.92	4	3.8601	3.93	15772	{}	\N	2020-10-12 14:28:10.26946
31843	CEL	2020-10-06	3.88	3.98	3.88	3.91	9284	{}	\N	2020-10-12 14:28:10.273322
31844	CEL	2020-10-07	3.79	3.93	3.79	3.86	13388	{}	\N	2020-10-12 14:28:10.276995
31845	CEL	2020-10-08	3.8453	3.93	3.8453	3.93	2318	{}	\N	2020-10-12 14:28:10.280551
31846	CEL	2020-10-09	4	4.01	3.91	3.99	22118	{}	\N	2020-10-12 14:28:10.284161
31847	CEL	2020-10-12	3.98	3.98	3.88	3.88	13674	{}	\N	2020-10-12 14:28:10.287746
31848	CEN	2020-09-28	7.5	7.54	7.33	7.5	47580	{}	\N	2020-10-12 14:28:15.356286
31849	CEN	2020-09-29	7.45	7.79	7.41	7.71	73437	{}	\N	2020-10-12 14:28:15.363189
31850	CEN	2020-09-30	7.66	7.77	7.39	7.45	74764	{}	\N	2020-10-12 14:28:15.368219
31851	CEN	2020-10-01	7.48	7.485	7.34	7.45	89586	{}	\N	2020-10-12 14:28:15.373388
31852	CEN	2020-10-02	7.27	7.48	7.27	7.48	55176	{}	\N	2020-10-12 14:28:15.378418
31853	CEN	2020-10-05	7.53	7.808	7.53	7.79	129112	{}	\N	2020-10-12 14:28:15.382852
31854	CEN	2020-10-06	7.87	7.93	7.69	7.73	122738	{}	\N	2020-10-12 14:28:15.387272
31855	CEN	2020-10-07	7.76	7.865	7.75	7.8	70950	{}	\N	2020-10-12 14:28:15.391528
31856	CEN	2020-10-08	7.84	8.23	7.84	8.22	49986	{}	\N	2020-10-12 14:28:15.396233
31857	CEN	2020-10-09	8.26	8.26	8.14	8.18	33778	{}	\N	2020-10-12 14:28:15.40129
31858	CEN	2020-10-12	8.18	8.18	8.14	8.14	9386	{}	\N	2020-10-12 14:28:15.405657
31859	CALA	2020-09-28	3.43	3.5	3.39	3.45	399508	{}	\N	2020-10-12 14:28:17.266501
31860	CALA	2020-09-29	3.45	3.5	3.38	3.44	674675	{}	\N	2020-10-12 14:28:17.276633
31861	CALA	2020-09-30	3.45	3.58	3.44	3.45	458423	{}	\N	2020-10-12 14:28:17.284254
31862	CALA	2020-10-01	3.48	3.66	3.42	3.61	387364	{}	\N	2020-10-12 14:28:17.290165
31863	CALA	2020-10-02	3.55	3.59	3.48	3.55	321026	{}	\N	2020-10-12 14:28:17.2957
31864	CALA	2020-10-05	3.61	3.74	3.58	3.61	518643	{}	\N	2020-10-12 14:28:17.300859
31865	CALA	2020-10-06	3.64	3.71	3.59	3.615	288072	{}	\N	2020-10-12 14:28:17.305157
31866	CALA	2020-10-07	3.63	3.79	3.6	3.76	270247	{}	\N	2020-10-12 14:28:17.309269
31867	CALA	2020-10-08	3.82	3.86	3.75	3.81	351670	{}	\N	2020-10-12 14:28:17.313507
31868	CALA	2020-10-09	3.87	3.88	3.71	3.77	526530	{}	\N	2020-10-12 14:28:17.317846
31869	CALA	2020-10-12	3.79	3.8	3.74	3.78	13862	{}	\N	2020-10-12 14:28:17.322194
31870	CASA	2020-09-28	3.89	4.0187	3.85	3.91	152797	{}	\N	2020-10-12 14:28:19.280947
31871	CASA	2020-09-29	3.92	4.13	3.92	4.11	156523	{}	\N	2020-10-12 14:28:19.291411
31872	CASA	2020-09-30	4.09	4.17	3.96	4.03	187504	{}	\N	2020-10-12 14:28:19.298374
31873	CASA	2020-10-01	4.09	4.12	3.99	4.09	100750	{}	\N	2020-10-12 14:28:19.304442
31874	CASA	2020-10-02	4	4.11	3.95	4.1	142561	{}	\N	2020-10-12 14:28:19.310414
31875	CASA	2020-10-05	4.14	4.31	4.13	4.26	209477	{}	\N	2020-10-12 14:28:19.315255
31876	CASA	2020-10-06	4.3	4.46	4.11	4.13	181548	{}	\N	2020-10-12 14:28:19.319518
31877	CASA	2020-10-07	4.2	4.28	4.18	4.27	192706	{}	\N	2020-10-12 14:28:19.323787
31878	CASA	2020-10-08	4.25	4.3798	4.12	4.35	193405	{}	\N	2020-10-12 14:28:19.327657
31879	CASA	2020-10-09	4.38	4.5864	4.38	4.53	213547	{}	\N	2020-10-12 14:28:19.337719
31880	CASA	2020-10-12	4.51	4.575	4.4	4.415	24053	{}	\N	2020-10-12 14:28:19.342347
31881	CATM	2020-09-28	19.82	20.9	19.82	20.27	345017	{}	\N	2020-10-12 14:28:20.409227
31882	CATM	2020-09-29	20.24	20.5675	19.86	20.28	386799	{}	\N	2020-10-12 14:28:20.415357
31883	CATM	2020-09-30	20.41	20.8329	19.44	19.8	294858	{}	\N	2020-10-12 14:28:20.419612
31884	CATM	2020-10-01	19.78	20.41	19.34	20.41	362787	{}	\N	2020-10-12 14:28:20.424212
31885	CATM	2020-10-02	19.83	21.22	19.4	21.09	357160	{}	\N	2020-10-12 14:28:20.428885
31886	CATM	2020-10-05	21.41	21.43	20.58	20.97	355263	{}	\N	2020-10-12 14:28:20.434108
31887	CATM	2020-10-06	21.33	22.03	21.05	21.33	370315	{}	\N	2020-10-12 14:28:20.439352
31888	CATM	2020-10-07	21.43	21.89	20.61	20.88	393623	{}	\N	2020-10-12 14:28:20.444361
31889	CATM	2020-10-08	21.17	21.75	20.9401	21.19	294978	{}	\N	2020-10-12 14:28:20.449253
31890	CATM	2020-10-09	21.43	21.43	20.71	21.03	204138	{}	\N	2020-10-12 14:28:20.45503
31891	CATM	2020-10-12	21.14	21.14	20.7	20.97	40465	{}	\N	2020-10-12 14:28:20.460068
31892	CBD	2020-09-28	13.25	13.31	12.44	12.61	648682	{}	\N	2020-10-12 14:28:22.418641
31893	CBD	2020-09-29	12.42	12.775	12.38	12.39	745283	{}	\N	2020-10-12 14:28:22.429476
31894	CBD	2020-09-30	12.39	12.63	12.28	12.34	946837	{}	\N	2020-10-12 14:28:22.435789
31895	CBD	2020-10-01	12.25	12.44	12.02	12.42	598960	{}	\N	2020-10-12 14:28:22.44168
31896	CBD	2020-10-02	12.24	12.49	12.005	12.05	776740	{}	\N	2020-10-12 14:28:22.446355
31897	CBD	2020-10-05	12.01	12.3	11.94	12.22	480611	{}	\N	2020-10-12 14:28:22.451063
31898	CBD	2020-10-06	12.27	12.43	12.02	12.18	518085	{}	\N	2020-10-12 14:28:22.455324
31899	CBD	2020-10-07	12.3	12.51	12.1	12.29	496209	{}	\N	2020-10-12 14:28:22.459409
31900	CBD	2020-10-08	12.05	12.3	11.97	12.29	693778	{}	\N	2020-10-12 14:28:22.463861
31901	CBD	2020-10-09	12.3	12.31	12.095	12.2	842000	{}	\N	2020-10-12 14:28:22.468094
31902	CBD	2020-10-12	12.29	12.38	12.19	12.38	33643	{}	\N	2020-10-12 14:28:22.471786
31903	CBT	2020-09-28	35.77	36.75	35.77	36.4	195984	{}	\N	2020-10-12 14:28:25.996685
31904	CBT	2020-09-29	36.35	36.72	35.62	36.04	408751	{}	\N	2020-10-12 14:28:26.006606
31905	CBT	2020-09-30	36.2	36.82	35.75	36.03	495157	{}	\N	2020-10-12 14:28:26.013787
31906	CBT	2020-10-01	36.24	36.79	35.89	36.29	520036	{}	\N	2020-10-12 14:28:26.020297
31907	CBT	2020-10-02	35.59	38.09	35.59	37.92	537064	{}	\N	2020-10-12 14:28:26.025269
31908	CBT	2020-10-05	38.44	39.74	38.44	39.69	473681	{}	\N	2020-10-12 14:28:26.029944
31909	CBT	2020-10-06	40.22	41	39.53	39.57	490819	{}	\N	2020-10-12 14:28:26.03418
31910	CBT	2020-10-07	40.2	41.18	40.2	40.96	430856	{}	\N	2020-10-12 14:28:26.038758
31911	CBT	2020-10-08	41.25	41.84	40.69	40.8	455477	{}	\N	2020-10-12 14:28:26.042979
31912	CBT	2020-10-09	41.26	41.48	40.34	40.68	352216	{}	\N	2020-10-12 14:28:26.047202
31913	CBT	2020-10-12	41.1	41.19	40.77	41	48405	{}	\N	2020-10-12 14:28:26.051233
31914	CCF	2020-09-28	97.15	99.39	97.15	97.52	12835	{}	\N	2020-10-12 14:28:27.039299
31915	CCF	2020-09-29	97.55	98	95.16	96.33	19160	{}	\N	2020-10-12 14:28:27.050042
31916	CCF	2020-09-30	97.8	98.4	95.22	95.4	21769	{}	\N	2020-10-12 14:28:27.056882
31917	CCF	2020-10-01	96.55	96.625	93.865	96.11	21798	{}	\N	2020-10-12 14:28:27.062373
31918	CCF	2020-10-02	93.84	97.17	93.84	96.47	18448	{}	\N	2020-10-12 14:28:27.067438
31919	CCF	2020-10-05	96.21	98.77	96.21	97.92	22460	{}	\N	2020-10-12 14:28:27.072681
31920	CCF	2020-10-06	99.2	100	96.81	97.27	21108	{}	\N	2020-10-12 14:28:27.076884
31921	CCF	2020-10-07	99.25	100	97.95	99.29	19958	{}	\N	2020-10-12 14:28:27.081023
31922	CCF	2020-10-08	100.12	100.5	98.97	99.26	17860	{}	\N	2020-10-12 14:28:27.085533
31923	CCF	2020-10-09	100.86	100.86	97.88	97.88	9459	{}	\N	2020-10-12 14:28:27.090191
31924	CCF	2020-10-12	98.5137	99.14	98.5137	98.97	2161	{}	\N	2020-10-12 14:28:27.094401
31925	CCU	2020-09-28	13.21	13.3	12.8801	12.92	153921	{}	\N	2020-10-12 14:28:28.495279
31926	CCU	2020-09-29	12.87	12.92	12.66	12.72	429707	{}	\N	2020-10-12 14:28:28.504962
31927	CCU	2020-09-30	12.8	12.97	12.7	12.97	261342	{}	\N	2020-10-12 14:28:28.511648
31928	CCU	2020-10-01	12.98	13.01	12.69	12.77	200021	{}	\N	2020-10-12 14:28:28.517904
31929	CCU	2020-10-02	12.64	12.73	12.62	12.66	191787	{}	\N	2020-10-12 14:28:28.522833
31930	CCU	2020-10-05	12.44	12.65	12.34	12.51	561292	{}	\N	2020-10-12 14:28:28.527398
31931	CCU	2020-10-06	12.4	12.55	12.24	12.27	384802	{}	\N	2020-10-12 14:28:28.531465
31932	CCU	2020-10-07	12.21	12.43	12.2	12.37	281325	{}	\N	2020-10-12 14:28:28.535669
31933	CCU	2020-10-08	12.38	12.47	12.2	12.34	329222	{}	\N	2020-10-12 14:28:28.53934
31934	CCU	2020-10-09	12.43	12.69	12.3	12.52	280188	{}	\N	2020-10-12 14:28:28.543302
31935	CCU	2020-10-12	12.55	12.6	12.41	12.495	34666	{}	\N	2020-10-12 14:28:28.547018
31936	CDNA	2020-09-28	33.04	34.75	32.6325	34.64	445313	{}	\N	2020-10-12 14:28:30.3377
31937	CDNA	2020-09-29	34.79	36.97	33.98	36.88	820612	{}	\N	2020-10-12 14:28:30.347214
31938	CDNA	2020-09-30	36.98	38.67	36.85	37.94	779740	{}	\N	2020-10-12 14:28:30.353409
31939	CDNA	2020-10-01	38.89	42.79	38.59	41.43	1798817	{}	\N	2020-10-12 14:28:30.358441
31940	CDNA	2020-10-02	41.31	43.85	40.44	41.78	1087752	{}	\N	2020-10-12 14:28:30.363034
31941	CDNA	2020-10-05	41.75	43.3	41.285	42.41	474902	{}	\N	2020-10-12 14:28:30.366707
31942	CDNA	2020-10-06	43.15	44.58	42.46	43.07	478209	{}	\N	2020-10-12 14:28:30.370201
31943	CDNA	2020-10-07	43.37	44.25	42.76	43.86	517392	{}	\N	2020-10-12 14:28:30.374011
31944	CDNA	2020-10-08	50	52.4999	49.25	50.53	2985801	{}	\N	2020-10-12 14:28:30.378544
31945	CDNA	2020-10-09	50.55	54.2199	50.55	53.955	1017043	{}	\N	2020-10-12 14:28:30.382305
31946	CDNA	2020-10-12	54.38	54.75	53.42	54.06	128058	{}	\N	2020-10-12 14:28:30.386034
31947	CEE	2020-09-28	19.98	20.07	19.7711	19.81	8943	{}	\N	2020-10-12 14:28:31.633017
31948	CEE	2020-09-29	19.77	19.78	19.559	19.56	19860	{}	\N	2020-10-12 14:28:31.644254
31949	CEE	2020-09-30	19.9567	20.11	19.64	19.64	7116	{}	\N	2020-10-12 14:28:31.650575
31950	CEE	2020-10-01	19.94	20.32	19.832	19.97	10330	{}	\N	2020-10-12 14:28:31.65638
31951	CEE	2020-10-02	19.665	19.73	19.665	19.73	1158	{}	\N	2020-10-12 14:28:31.661561
31952	CEE	2020-10-05	19.77	19.9599	19.69	19.7	12196	{}	\N	2020-10-12 14:28:31.666481
31953	CEE	2020-10-06	19.8	19.82	19.75	19.75	6742	{}	\N	2020-10-12 14:28:31.670498
31954	CEE	2020-10-07	19.72	19.72	19.56	19.7	13974	{}	\N	2020-10-12 14:28:31.674587
31955	CEE	2020-10-08	19.8	19.805	19.71	19.71	14518	{}	\N	2020-10-12 14:28:31.678782
31956	CEE	2020-10-09	19.81	19.9301	19.79	19.79	8598	{}	\N	2020-10-12 14:28:31.682985
31957	CEE	2020-10-12	19.99	19.995	19.99	19.995	1102	{}	\N	2020-10-12 14:28:31.687174
31958	CEIX	2020-09-28	4.53	4.74	4.29	4.68	360635	{}	\N	2020-10-12 14:28:32.788925
31959	CEIX	2020-09-29	4.65	4.71	4.35	4.52	1042078	{}	\N	2020-10-12 14:28:32.797151
31960	CEIX	2020-09-30	4.49	4.74	4.38	4.43	800323	{}	\N	2020-10-12 14:28:32.802704
31961	CEIX	2020-10-01	4.42	4.45	4.05	4.12	676684	{}	\N	2020-10-12 14:28:32.808276
31962	CEIX	2020-10-02	4.06	4.43	4.05	4.38	638287	{}	\N	2020-10-12 14:28:32.813483
31963	CEIX	2020-10-05	4.49	4.5824	4.33	4.37	279286	{}	\N	2020-10-12 14:28:32.818408
31964	CEIX	2020-10-06	4.43	4.58	4.17	4.19	446940	{}	\N	2020-10-12 14:28:32.82266
31965	CEIX	2020-10-07	4.22	4.26	4.02	4.12	594171	{}	\N	2020-10-12 14:28:32.826779
31966	CEIX	2020-10-08	4.16	4.33	4.07	4.32	344272	{}	\N	2020-10-12 14:28:32.83092
31967	CEIX	2020-10-09	4.44	4.5	4.19	4.3	387279	{}	\N	2020-10-12 14:28:32.835056
31968	CEIX	2020-10-12	4.38	4.55	4.3	4.42	172224	{}	\N	2020-10-12 14:28:32.839249
31969	CENX	2020-09-28	7.35	7.53	7.24	7.35	1312700	{}	\N	2020-10-12 14:28:34.7833
31970	CENX	2020-09-29	7.31	7.43	7.04	7.13	955261	{}	\N	2020-10-12 14:28:34.791866
31971	CENX	2020-09-30	7.12	7.31	6.98	7.12	1001288	{}	\N	2020-10-12 14:28:34.798406
31972	CENX	2020-10-01	7.17	7.19	6.6601	6.94	1099656	{}	\N	2020-10-12 14:28:34.805052
31973	CENX	2020-10-02	6.71	7.235	6.71	7.13	667279	{}	\N	2020-10-12 14:28:34.810202
31974	CENX	2020-10-05	7.23	7.3599	7.01	7.15	895823	{}	\N	2020-10-12 14:28:34.814582
31975	CENX	2020-10-06	7.3	7.46	6.87	6.88	1134017	{}	\N	2020-10-12 14:28:34.818914
31976	CENX	2020-10-07	7.07	7.36	7.0205	7.21	738478	{}	\N	2020-10-12 14:28:34.822861
31977	CENX	2020-10-08	7.35	7.61	7.27	7.59	891247	{}	\N	2020-10-12 14:28:34.825981
31978	CENX	2020-10-09	7.84	8.14	7.65	7.97	1427174	{}	\N	2020-10-12 14:28:34.829562
31979	CENX	2020-10-12	7.98	8.0382	7.69	7.76	168939	{}	\N	2020-10-12 14:28:34.833018
31980	CETXW	2020-09-28	0.05	0.0542	0.05	0.0524	6330	{}	\N	2020-10-12 14:28:35.806447
31981	CETXW	2020-09-29	0.0524	0.056801	0.0524	0.0562	7184	{}	\N	2020-10-12 14:28:35.815981
31982	CETXW	2020-09-30	0.059	0.059	0.0523	0.0525	19180	{}	\N	2020-10-12 14:28:35.822228
31983	CETXW	2020-10-01	0.0527	0.0553	0.0526	0.0526	5060	{}	\N	2020-10-12 14:28:35.827429
31984	CETXW	2020-10-02	0.0525	0.0525	0.05	0.0501	32250	{}	\N	2020-10-12 14:28:35.832309
31985	CETXW	2020-10-05	0.0525	0.0639	0.0501	0.0639	35827	{}	\N	2020-10-12 14:28:35.837394
31986	CETXW	2020-10-06	0.0639	0.064	0.042	0.047249	22411	{}	\N	2020-10-12 14:28:35.842265
31987	CETXW	2020-10-07	0.05	0.05	0.047	0.0475	12320	{}	\N	2020-10-12 14:28:35.847329
31988	CETXW	2020-10-08	0.048749	0.050001	0.048	0.048	31090	{}	\N	2020-10-12 14:28:35.851337
31989	CETXW	2020-10-09	0.055	0.06	0.055	0.055	10746	{}	\N	2020-10-12 14:28:35.855553
31990	CETXW	2020-10-12	0.055	0.055	0.055	0.055	2545	{}	\N	2020-10-12 14:28:35.859978
31991	CFFI	2020-09-28	29.57	29.58	28.79	29.11	6026	{}	\N	2020-10-12 14:28:37.387256
31992	CFFI	2020-09-29	29.17	29.49	28.7501	29.2	8006	{}	\N	2020-10-12 14:28:37.395053
31993	CFFI	2020-09-30	29.5158	30.105	29.3	29.7	4281	{}	\N	2020-10-12 14:28:37.40082
31994	CFFI	2020-10-01	29.98	29.98	29.3525	29.85	4380	{}	\N	2020-10-12 14:28:37.405653
31995	CFFI	2020-10-02	29.35	29.725	29.35	29.71	3087	{}	\N	2020-10-12 14:28:37.410398
31996	CFFI	2020-10-05	30	30.45	29.83	29.83	7255	{}	\N	2020-10-12 14:28:37.414689
31997	CFFI	2020-10-06	29.77	29.82	29.45	29.45	3745	{}	\N	2020-10-12 14:28:37.418863
31998	CFFI	2020-10-07	29.5	30.15	29.5	29.99	4827	{}	\N	2020-10-12 14:28:37.423084
31999	CFFI	2020-10-08	29.5148	31.57	29.5101	30.86	14714	{}	\N	2020-10-12 14:28:37.427265
32000	CFFI	2020-10-09	30.8127	31.31	30.6	31.05	6028	{}	\N	2020-10-12 14:28:37.431545
32001	CASS	2020-09-28	37.98	39.18	37.96	38.9	25564	{}	\N	2020-10-12 14:28:38.398841
32002	CASS	2020-09-29	38.75	39.58	38.545	39.5	39581	{}	\N	2020-10-12 14:28:38.411031
32003	CASS	2020-09-30	39.67	40.68	39.67	40.24	37681	{}	\N	2020-10-12 14:28:38.41866
32004	CASS	2020-10-01	40.15	40.59	39.84	40.59	35494	{}	\N	2020-10-12 14:28:38.424964
32005	CASS	2020-10-02	40.06	40.96	39.77	40.21	36735	{}	\N	2020-10-12 14:28:38.429985
32006	CASS	2020-10-05	40.79	41.18	40.3	41.11	35299	{}	\N	2020-10-12 14:28:38.435073
32007	CASS	2020-10-06	41.56	42.575	40.75	41.08	24553	{}	\N	2020-10-12 14:28:38.439322
32008	CASS	2020-10-07	41.3	42.52	41.25	42.19	31281	{}	\N	2020-10-12 14:28:38.44338
32009	CASS	2020-10-08	42.81	43.7	42.75	43.5	42211	{}	\N	2020-10-12 14:28:38.44766
32010	CASS	2020-10-09	43.66	44.045	43.49	43.5	23722	{}	\N	2020-10-12 14:28:38.451781
32011	CASS	2020-10-12	43.46	43.65	43.36	43.5	2279	{}	\N	2020-10-12 14:28:38.456332
32012	CBL	2020-09-28	0.154	0.16575	0.154	0.1617	5562535	{}	\N	2020-10-12 14:28:40.058613
32013	CBL	2020-09-29	0.1617	0.1725	0.161	0.169	8647956	{}	\N	2020-10-12 14:28:40.06786
32014	CBL	2020-09-30	0.17	0.17	0.16	0.1611	10530707	{}	\N	2020-10-12 14:28:40.074273
32015	CBL	2020-10-01	0.16	0.17591	0.16	0.1721	8367823	{}	\N	2020-10-12 14:28:40.079637
32016	CBL	2020-10-02	0.1656	0.1693	0.162	0.1654	7294289	{}	\N	2020-10-12 14:28:40.084163
32017	CBL	2020-10-05	0.1633	0.17	0.1633	0.17	5924490	{}	\N	2020-10-12 14:28:40.088243
32018	CBL	2020-10-06	0.1655	0.1734	0.163	0.163	7620780	{}	\N	2020-10-12 14:28:40.092337
32019	CBL	2020-10-07	0.163	0.1675	0.163	0.1648	3793370	{}	\N	2020-10-12 14:28:40.096329
32020	CBL	2020-10-08	0.1698	0.17	0.1633	0.1656	5082546	{}	\N	2020-10-12 14:28:40.100219
32021	CBL	2020-10-09	0.169	0.169	0.1611	0.1637	6865939	{}	\N	2020-10-12 14:28:40.104058
32022	CBL	2020-10-12	0.165	0.17	0.165	0.167699	3079022	{}	\N	2020-10-12 14:28:40.107798
32023	CBSH	2020-09-28	55.02	55.8451	54.47	55.61	336309	{}	\N	2020-10-12 14:28:41.185543
32024	CBSH	2020-09-29	54.84	55.32	54.14	55.17	310631	{}	\N	2020-10-12 14:28:41.190432
32025	CBSH	2020-09-30	55.51	56.53	55.08	56.29	513335	{}	\N	2020-10-12 14:28:41.198957
32026	CBSH	2020-10-01	56.09	56.92	55.28	55.86	420926	{}	\N	2020-10-12 14:28:41.207907
32027	CBSH	2020-10-02	55.04	57.23	54.7	56.9	343062	{}	\N	2020-10-12 14:28:41.213202
32028	CBSH	2020-10-05	57.5	58.93	57.5	58.75	498911	{}	\N	2020-10-12 14:28:41.219302
32029	CBSH	2020-10-06	58.8	61	58.73	59.31	371727	{}	\N	2020-10-12 14:28:41.224202
32030	CBSH	2020-10-07	60.05	61	59.8	60.37	408594	{}	\N	2020-10-12 14:28:41.228742
32031	CBSH	2020-10-08	60.82	61.13	59.59	61.05	277949	{}	\N	2020-10-12 14:28:41.238416
32032	CBSH	2020-10-09	61.35	61.3745	60.14	60.52	169719	{}	\N	2020-10-12 14:28:41.242315
32033	CBSH	2020-10-12	60.52	60.92	60.42	60.8565	13923	{}	\N	2020-10-12 14:28:41.246025
32034	CCCL	2020-09-28	2.25	2.34	2.09	2.14	320544	{}	\N	2020-10-12 14:28:42.258173
32035	CCCL	2020-09-29	2.096	2.1399	1.92	2.06	555871	{}	\N	2020-10-12 14:28:42.266303
32036	CCCL	2020-09-30	2.0601	2.23	2.01	2.18	340140	{}	\N	2020-10-12 14:28:42.272177
32037	CCCL	2020-10-01	2.1918	2.74	2.11	2.39	3683110	{}	\N	2020-10-12 14:28:42.277607
32038	CCCL	2020-10-02	2.17	2.4	2.1	2.33	324456	{}	\N	2020-10-12 14:28:42.283076
32039	CCCL	2020-10-05	2.25	2.3006	2.08	2.13	360463	{}	\N	2020-10-12 14:28:42.288094
32040	CCCL	2020-10-06	2.12	2.3399	2.08	2.1708	342090	{}	\N	2020-10-12 14:28:42.292359
32041	CCCL	2020-10-07	2.17	2.24	2.12	2.165	80743	{}	\N	2020-10-12 14:28:42.296645
32042	CCCL	2020-10-08	2.12	2.21	2.08	2.14	396746	{}	\N	2020-10-12 14:28:42.300821
32043	CCCL	2020-10-09	2.21	2.2579	2.12	2.2	144967	{}	\N	2020-10-12 14:28:42.304924
32044	CCCL	2020-10-12	2.41	2.48	2.33	2.335	400415	{}	\N	2020-10-12 14:28:42.309089
32045	CDE	2020-09-28	7.33	7.47	7.13	7.26	3438314	{}	\N	2020-10-12 14:28:43.419452
32046	CDE	2020-09-29	7.32	7.495	7.2118	7.43	6927785	{}	\N	2020-10-12 14:28:43.424131
32047	CDE	2020-09-30	7.38	7.51	7.25	7.38	5068215	{}	\N	2020-10-12 14:28:43.428511
32048	CDE	2020-10-01	7.44	7.67	7.335	7.64	6652240	{}	\N	2020-10-12 14:28:43.432906
32049	CDE	2020-10-02	7.47	7.565	7.335	7.44	5416187	{}	\N	2020-10-12 14:28:43.437445
32050	CDE	2020-10-05	7.5	7.76	7.5	7.64	5444283	{}	\N	2020-10-12 14:28:43.442209
32051	CDE	2020-10-06	7.69	7.765	7.19	7.2	6294993	{}	\N	2020-10-12 14:28:43.446253
32052	CDE	2020-10-07	7.33	7.5	7.23	7.42	5234575	{}	\N	2020-10-12 14:28:43.450418
32053	CDE	2020-10-08	7.56	7.72	7.51	7.64	7632017	{}	\N	2020-10-12 14:28:43.454671
32054	CDE	2020-10-09	7.93	8.2	7.81	8.2	6973326	{}	\N	2020-10-12 14:28:43.458852
32055	CDE	2020-10-12	8.17	8.315	8.02	8.1801	1764173	{}	\N	2020-10-12 14:28:43.462999
32056	CDNS	2020-09-28	105.05	105.27	103.83	105.25	1320600	{}	\N	2020-10-12 14:28:44.899617
32057	CDNS	2020-09-29	105.82	106.94	105.65	105.78	1120469	{}	\N	2020-10-12 14:28:44.91105
32058	CDNS	2020-09-30	105.35	107.9501	105.3	106.63	1657726	{}	\N	2020-10-12 14:28:44.918079
32059	CDNS	2020-10-01	107.85	109.31	107.59	108.63	1715805	{}	\N	2020-10-12 14:28:44.924103
32060	CDNS	2020-10-02	106.45	107.47	104.48	105.32	1187675	{}	\N	2020-10-12 14:28:44.928847
32061	CDNS	2020-10-05	106.11	107.65	106.04	107.46	931589	{}	\N	2020-10-12 14:28:44.934099
32062	CDNS	2020-10-06	107	108	105.19	105.52	1019743	{}	\N	2020-10-12 14:28:44.938111
32063	CDNS	2020-10-07	106.53	108.99	105.79	108.44	1105098	{}	\N	2020-10-12 14:28:44.942256
32064	CDNS	2020-10-08	109.41	109.55	107.9359	109.32	630564	{}	\N	2020-10-12 14:28:44.946098
32065	CDNS	2020-10-09	110.29	112.3	109.85	111.93	960627	{}	\N	2020-10-12 14:28:44.950052
32066	CDNS	2020-10-12	114.26	115.0384	113.0568	113.38	271804	{}	\N	2020-10-12 14:28:44.953898
32067	CE	2020-09-28	108.04	109.94	107.58	108.6	703271	{}	\N	2020-10-12 14:28:46.322725
32068	CE	2020-09-29	108.49	109.22	106.48	106.88	940972	{}	\N	2020-10-12 14:28:46.337392
32069	CE	2020-09-30	107.61	109.3214	107.15	107.45	1405681	{}	\N	2020-10-12 14:28:46.345128
32070	CE	2020-10-01	108.19	109.535	105.59	106.04	1419886	{}	\N	2020-10-12 14:28:46.350638
32071	CE	2020-10-02	104.31	109.31	103.3	108.25	1508310	{}	\N	2020-10-12 14:28:46.356004
32072	CE	2020-10-05	109.2	111.82	109.11	110.7	1715880	{}	\N	2020-10-12 14:28:46.360508
32073	CE	2020-10-06	111.7	114.86	111.09	111.17	1689087	{}	\N	2020-10-12 14:28:46.365085
32074	CE	2020-10-07	114.77	117.13	113.945	116.46	1527794	{}	\N	2020-10-12 14:28:46.369451
32075	CE	2020-10-08	117.21	118.12	116.03	117.95	1009104	{}	\N	2020-10-12 14:28:46.373548
32076	CE	2020-10-09	119.19	120	117.18	117.84	1555690	{}	\N	2020-10-12 14:28:46.378051
32077	CE	2020-10-12	118.55	118.55	116.79	117.52	198364	{}	\N	2020-10-12 14:28:46.382569
32078	CENTA	2020-09-28	34.43	35.3	34.28	35.05	145098	{}	\N	2020-10-12 14:28:47.886574
32079	CENTA	2020-09-29	35.31	35.52	34.94	35.45	128976	{}	\N	2020-10-12 14:28:47.897655
32080	CENTA	2020-09-30	35.71	36.22	35.53	36.14	247458	{}	\N	2020-10-12 14:28:47.904822
32081	CENTA	2020-10-01	35.49	36.44	35.055	36.26	214623	{}	\N	2020-10-12 14:28:47.910794
32082	CENTA	2020-10-02	35.94	36.67	35.74	36.46	134935	{}	\N	2020-10-12 14:28:47.915854
32083	CENTA	2020-10-05	36.74	37.02	36.44	36.67	131444	{}	\N	2020-10-12 14:28:47.920819
32084	CENTA	2020-10-06	36.77	37.52	36.5	37.02	140027	{}	\N	2020-10-12 14:28:47.925027
32085	CENTA	2020-10-07	37.08	37.57	36.9	37.14	204472	{}	\N	2020-10-12 14:28:47.929115
32086	CENTA	2020-10-08	37.32	37.9	37.16	37.81	126354	{}	\N	2020-10-12 14:28:47.933207
32087	CENTA	2020-10-09	38.23	38.67	38.005	38.6	136999	{}	\N	2020-10-12 14:28:47.937289
32088	CENTA	2020-10-12	38.84	39.06	38.71	38.9	21779	{}	\N	2020-10-12 14:28:47.941317
32089	CETXP	2020-09-28	1.9902	2.8	1.99	2.25	10931	{}	\N	2020-10-12 14:28:49.061836
32090	CETXP	2020-09-30	2.8	2.82	2.55	2.55	1055	{}	\N	2020-10-12 14:28:49.06994
32091	CETXP	2020-10-01	2.555	2.555	2.53	2.55	501	{}	\N	2020-10-12 14:28:49.076177
32092	CETXP	2020-10-02	2.81	2.85	2.52	2.52	1910	{}	\N	2020-10-12 14:28:49.081519
32093	CETXP	2020-10-05	2.07	2.07	2.07	2.07	187	{}	\N	2020-10-12 14:28:49.086598
32094	CETXP	2020-10-06	2.27	2.27	2.2401	2.2401	588	{}	\N	2020-10-12 14:28:49.091035
32095	CETXP	2020-10-07	2.08	2.08	1.935	2.08	2170	{}	\N	2020-10-12 14:28:49.095388
32096	CETXP	2020-10-08	2.08	2.08	1.66	1.66	541	{}	\N	2020-10-12 14:28:49.099624
32097	CETXP	2020-10-09	2	2	1.9999	1.9999	202	{}	\N	2020-10-12 14:28:49.104157
32098	CFBI	2020-09-28	7.45	7.47	7.45	7.47	723	{}	\N	2020-10-12 14:28:50.495944
32099	CFBI	2020-09-29	7.11	7.1855	7.1	7.18	3559	{}	\N	2020-10-12 14:28:50.507721
32100	CFBI	2020-09-30	7.1524	7.1524	7.15	7.15	1120	{}	\N	2020-10-12 14:28:50.514721
32101	CFBI	2020-10-01	7.19	7.19	7.19	7.19	211	{}	\N	2020-10-12 14:28:50.520521
32102	CFBI	2020-10-05	7.225	7.225	7.16	7.16	2317	{}	\N	2020-10-12 14:28:50.525075
32103	CFBI	2020-10-06	7.2001	7.3	6.58	6.68	9802	{}	\N	2020-10-12 14:28:50.529483
32104	CFBI	2020-10-07	6.91	7.2399	6.91	7.2399	2445	{}	\N	2020-10-12 14:28:50.533991
32105	CFBI	2020-10-08	7	7.1472	7	7.1439	3007	{}	\N	2020-10-12 14:28:50.538272
32106	CFBI	2020-10-12	7.07	7.2072	7.07	7.2072	500	{}	\N	2020-10-12 14:28:50.542383
32107	CGBD	2020-09-28	9.31	9.89	9.31	9.54	455788	{}	\N	2020-10-12 14:28:51.696521
32108	CGBD	2020-09-29	9.26	9.26	8.92	9.11	313324	{}	\N	2020-10-12 14:28:51.706507
32109	CGBD	2020-09-30	9.15	9.18	8.83	8.91	294378	{}	\N	2020-10-12 14:28:51.713289
32110	CGBD	2020-10-01	8.93	8.96	8.8	8.9	213789	{}	\N	2020-10-12 14:28:51.719521
32111	CGBD	2020-10-02	8.8	9.12	8.8	9.05	126117	{}	\N	2020-10-12 14:28:51.724988
32112	CGBD	2020-10-05	9.09	9.2099	8.95	9.03	177445	{}	\N	2020-10-12 14:28:51.729945
32113	CGBD	2020-10-06	9.08	9.08	8.9	8.95	149194	{}	\N	2020-10-12 14:28:51.73462
32114	CGBD	2020-10-07	9	9.02	8.8	8.95	196177	{}	\N	2020-10-12 14:28:51.739296
32115	CGBD	2020-10-08	8.92	9.1	8.92	9.04	79303	{}	\N	2020-10-12 14:28:51.744322
32116	CGBD	2020-10-09	9.06	9.19	8.97	8.98	148491	{}	\N	2020-10-12 14:28:51.748641
32117	CGBD	2020-10-12	8.97	9.12	8.83	8.9	59761	{}	\N	2020-10-12 14:28:51.753173
32118	FFHG	2020-09-28	24.6	24.74	24.43	24.59	16676	{}	\N	2020-10-12 14:28:52.699012
32119	FFHG	2020-09-29	24.66	24.69	24.386	24.6229	17193	{}	\N	2020-10-12 14:28:52.709892
32120	FFHG	2020-09-30	24.74	25.0109	24.254	24.62	27854	{}	\N	2020-10-12 14:28:52.718514
32121	FFHG	2020-10-01	24.6	24.93	24.58	24.74	10463	{}	\N	2020-10-12 14:28:52.724909
32122	FFHG	2020-10-02	24.7	24.9728	24.65	24.9025	58171	{}	\N	2020-10-12 14:28:52.729767
32123	FFHG	2020-10-05	25.06	25.4828	25.06	25.4828	13182	{}	\N	2020-10-12 14:28:52.73452
32124	FFHG	2020-10-06	25.49	25.92	25.14	25.23	17529	{}	\N	2020-10-12 14:28:52.741407
32125	FFHG	2020-10-07	25.77	25.9239	25.75	25.83	9047	{}	\N	2020-10-12 14:28:52.746736
32126	FFHG	2020-10-08	26.08	26.2688	25.8207	26.2688	19459	{}	\N	2020-10-12 14:28:52.751864
32127	FFHG	2020-10-09	26.18	26.4671	26.18	26.26	8052	{}	\N	2020-10-12 14:28:52.756351
32128	FFHG	2020-10-12	26.23	26.6	26.23	26.6	356	{}	\N	2020-10-12 14:28:52.760782
32129	CATY	2020-09-28	21.73	22.18	21.66	22.005	320611	{}	\N	2020-10-12 14:28:53.684971
32130	CATY	2020-09-29	21.88	21.9	21.43	21.71	357781	{}	\N	2020-10-12 14:28:53.69525
32131	CATY	2020-09-30	21.9	22.18	21.48	21.67	246893	{}	\N	2020-10-12 14:28:53.703283
32132	CATY	2020-10-01	21.58	21.88	21.34	21.7	316966	{}	\N	2020-10-12 14:28:53.709123
32133	CATY	2020-10-02	21.27	22.34	21.27	22.205	311237	{}	\N	2020-10-12 14:28:53.714161
32134	CATY	2020-10-05	22.51	23.4	22.48	23.39	343143	{}	\N	2020-10-12 14:28:53.718925
32135	CATY	2020-10-06	23.79	24.05	23.1	23.16	441293	{}	\N	2020-10-12 14:28:53.723208
32136	CATY	2020-10-07	23.53	24.04	23.42	23.94	519462	{}	\N	2020-10-12 14:28:53.727452
32137	CATY	2020-10-08	24.02	24.21	23.76	23.9	371405	{}	\N	2020-10-12 14:28:53.731234
32138	CATY	2020-10-09	24.11	24.23	23.7	23.83	189900	{}	\N	2020-10-12 14:28:53.735244
32139	CATY	2020-10-12	23.75	23.9	23.73	23.875	30969	{}	\N	2020-10-12 14:28:53.739189
32140	CBOE	2020-09-28	90.48	91.57	88.84	89	519912	{}	\N	2020-10-12 14:28:55.13507
32141	CBOE	2020-09-29	89.26	90.57	83.79	86.89	1323920	{}	\N	2020-10-12 14:28:55.145541
32142	CBOE	2020-09-30	87.68	88.92	86.66	87.74	871664	{}	\N	2020-10-12 14:28:55.152315
32143	CBOE	2020-10-01	88.5	88.9535	87.16	88.59	583547	{}	\N	2020-10-12 14:28:55.158498
32144	CBOE	2020-10-02	87.29	88.24	86.78	87.49	532225	{}	\N	2020-10-12 14:28:55.164043
32145	CBOE	2020-10-05	89.23	89.365	87.73	87.83	750975	{}	\N	2020-10-12 14:28:55.169044
32146	CBOE	2020-10-06	88.15	88.6983	86.32	86.84	658003	{}	\N	2020-10-12 14:28:55.172997
32147	CBOE	2020-10-07	87.76	88	84.35	84.89	798220	{}	\N	2020-10-12 14:28:55.177184
32148	CBOE	2020-10-08	85.88	86.38	84.85	85.03	1006736	{}	\N	2020-10-12 14:28:55.181468
32149	CBOE	2020-10-09	85.89	85.89	84.78	85.36	769823	{}	\N	2020-10-12 14:28:55.185985
32150	CBOE	2020-10-12	85.4	85.61	84.7041	84.97	144092	{}	\N	2020-10-12 14:28:55.190138
32151	CBTX	2020-09-28	16.21	16.6	16.2	16.57	36271	{}	\N	2020-10-12 14:28:56.065103
32152	CBTX	2020-09-29	16.731	16.87	16.16	16.3	38970	{}	\N	2020-10-12 14:28:56.077646
32153	CBTX	2020-09-30	16.32	16.54	16.21	16.34	40286	{}	\N	2020-10-12 14:28:56.085348
32154	CBTX	2020-10-01	16.38	16.49	16.06	16.26	60597	{}	\N	2020-10-12 14:28:56.091812
32155	CBTX	2020-10-02	16.12	16.96	16.12	16.84	28063	{}	\N	2020-10-12 14:28:56.097238
32156	CBTX	2020-10-05	16.97	17.5	16.86	17.31	29511	{}	\N	2020-10-12 14:28:56.102741
32157	CBTX	2020-10-06	17.58	18.08	17.15	17.48	36194	{}	\N	2020-10-12 14:28:56.107426
32158	CBTX	2020-10-07	17.6	18.36	17.305	18.17	51872	{}	\N	2020-10-12 14:28:56.111867
32159	CBTX	2020-10-08	18.4	18.57	18.17	18.43	44270	{}	\N	2020-10-12 14:28:56.116385
32160	CBTX	2020-10-09	18.67	18.6761	18.17	18.21	16700	{}	\N	2020-10-12 14:28:56.120975
32161	CBTX	2020-10-12	18.1	18.45	18	18.45	21616	{}	\N	2020-10-12 14:28:56.125644
32162	CCK	2020-09-28	75.81	77.115	75.53	76.36	753946	{}	\N	2020-10-12 14:28:57.716514
32163	CCK	2020-09-29	76.61	76.85	75.66	76.36	1329006	{}	\N	2020-10-12 14:28:57.726542
32164	CCK	2020-09-30	76.72	77.53	76.28	76.86	1446905	{}	\N	2020-10-12 14:28:57.733596
32165	CCK	2020-10-01	77.22	78.01	75.79	75.98	1628146	{}	\N	2020-10-12 14:28:57.740996
32166	CCK	2020-10-02	74.63	77.32	74.24	76.71	1019655	{}	\N	2020-10-12 14:28:57.746737
32167	CCK	2020-10-05	77.39	79.33	77.39	78.64	1895642	{}	\N	2020-10-12 14:28:57.752192
32168	CCK	2020-10-06	78.64	79.195	77.96	78.08	1868773	{}	\N	2020-10-12 14:28:57.757879
32169	CCK	2020-10-07	79.33	80.35	78.935	80.04	1344343	{}	\N	2020-10-12 14:28:57.762706
32170	CCK	2020-10-08	80.61	82.29	80.25	82.2	1661209	{}	\N	2020-10-12 14:28:57.767741
32171	CCK	2020-10-09	82.5	83.12	81.55	81.65	1351339	{}	\N	2020-10-12 14:28:57.772404
32172	CCK	2020-10-12	82.28	82.39	81.611	82.215	163850	{}	\N	2020-10-12 14:28:57.77739
32173	CCRC	2020-09-28	3.97	4.2	3.96	4.16	7375	{}	\N	2020-10-12 14:28:59.064317
32174	CCRC	2020-09-29	4.3	4.3	4.06	4.175	943	{}	\N	2020-10-12 14:28:59.071223
32175	CCRC	2020-09-30	4.3	4.37	4.18	4.21	3030	{}	\N	2020-10-12 14:28:59.077106
32176	CCRC	2020-10-01	4.27	4.27	4.12	4.24	2855	{}	\N	2020-10-12 14:28:59.082252
32177	CCRC	2020-10-02	4.26	4.28	4.25	4.27	2692	{}	\N	2020-10-12 14:28:59.087179
32178	CCRC	2020-10-05	4.28	4.65	4.25	4.59	3608	{}	\N	2020-10-12 14:28:59.091488
32179	CCRC	2020-10-06	4.65	4.65	4.58	4.65	1120	{}	\N	2020-10-12 14:28:59.09574
32180	CCRC	2020-10-07	4.2	4.52	4.18	4.52	1687	{}	\N	2020-10-12 14:28:59.100108
32181	CCRC	2020-10-08	4.43	4.69	4.43	4.69	1354	{}	\N	2020-10-12 14:28:59.104402
32182	CCRC	2020-10-09	5.0162	5.2	4.955	5.06	3159	{}	\N	2020-10-12 14:28:59.108284
32183	CDEV	2020-09-28	0.5982	0.6799	0.5978	0.6549	7000582	{}	\N	2020-10-12 14:29:00.104344
32184	CDEV	2020-09-29	0.659799	0.6598	0.6013	0.6199	8035151	{}	\N	2020-10-12 14:29:00.115454
32185	CDEV	2020-09-30	0.62	0.639699	0.6	0.6024	6980411	{}	\N	2020-10-12 14:29:00.122091
32186	CDEV	2020-10-01	0.6	0.605	0.57	0.5768	5514733	{}	\N	2020-10-12 14:29:00.127312
32187	CDEV	2020-10-02	0.56	0.65	0.5512	0.6044	7605587	{}	\N	2020-10-12 14:29:00.131843
32188	CDEV	2020-10-05	0.6353	0.645	0.61	0.6409	8236456	{}	\N	2020-10-12 14:29:00.136436
32189	CDEV	2020-10-06	0.6577	0.673	0.6129	0.6201	4730461	{}	\N	2020-10-12 14:29:00.14063
32190	CDEV	2020-10-07	0.6196	0.6414	0.6051	0.6287	4474082	{}	\N	2020-10-12 14:29:00.144916
32191	CDEV	2020-10-08	0.66	0.712	0.629	0.7088	7408185	{}	\N	2020-10-12 14:29:00.148944
32192	CDEV	2020-10-09	0.72	0.75	0.6621	0.6838	5696420	{}	\N	2020-10-12 14:29:00.152857
32193	CDEV	2020-10-12	0.67	0.68	0.635	0.6463	1961133	{}	\N	2020-10-12 14:29:00.156993
32194	CDTX	2020-09-28	3.06	3.06	2.88	2.89	153913	{}	\N	2020-10-12 14:29:01.100075
32195	CDTX	2020-09-29	2.89	2.9583	2.88	2.93	44942	{}	\N	2020-10-12 14:29:01.109537
32196	CDTX	2020-09-30	2.93	2.95	2.82	2.85	142830	{}	\N	2020-10-12 14:29:01.115689
32197	CDTX	2020-10-01	2.85	2.92	2.81	2.87	137946	{}	\N	2020-10-12 14:29:01.121161
32198	CDTX	2020-10-02	2.8	2.88	2.8	2.81	89846	{}	\N	2020-10-12 14:29:01.125596
32199	CDTX	2020-10-05	2.86	3.08	2.82	3.04	158532	{}	\N	2020-10-12 14:29:01.129626
32200	CDTX	2020-10-06	3.0621	3.073	2.97	2.97	51486	{}	\N	2020-10-12 14:29:01.133446
32201	CDTX	2020-10-07	3	3.11	2.95	3.11	89946	{}	\N	2020-10-12 14:29:01.137014
32202	CDTX	2020-10-08	3.14	3.25	3.1	3.23	66784	{}	\N	2020-10-12 14:29:01.140553
32203	CDTX	2020-10-09	3.1201	3.279	3.1201	3.25	83734	{}	\N	2020-10-12 14:29:01.144125
32204	CDTX	2020-10-12	3.26	3.26	3.13	3.13	22180	{}	\N	2020-10-12 14:29:01.147713
32205	CEF	2020-09-28	18.3	18.49	18.175	18.42	527180	{}	\N	2020-10-12 14:29:02.365055
32206	CEF	2020-09-29	18.6	18.825	18.55	18.775	567495	{}	\N	2020-10-12 14:29:02.373679
32207	CEF	2020-09-30	18.69	18.76	18.315	18.34	829682	{}	\N	2020-10-12 14:29:02.379596
32208	CEF	2020-10-01	18.6	18.81	18.46	18.67	506232	{}	\N	2020-10-12 14:29:02.385225
32209	CEF	2020-10-02	18.68	18.8058	18.56	18.6	355179	{}	\N	2020-10-12 14:29:02.389854
32210	CEF	2020-10-05	18.65	18.94	18.65	18.87	527171	{}	\N	2020-10-12 14:29:02.394137
32211	CEF	2020-10-06	18.96	18.99	18.435	18.45	586532	{}	\N	2020-10-12 14:29:02.398262
32212	CEF	2020-10-07	18.45	18.608	18.44	18.58	438323	{}	\N	2020-10-12 14:29:02.402034
32213	CEF	2020-10-08	18.68	18.73	18.5257	18.68	600605	{}	\N	2020-10-12 14:29:02.405709
32214	CEF	2020-10-09	18.97	19.22	18.92	19.22	701254	{}	\N	2020-10-12 14:29:02.409465
32215	CEF	2020-10-12	19.22	19.22	19.0875	19.0901	116129	{}	\N	2020-10-12 14:29:02.413235
32216	CELC	2020-09-28	5.98	6.145	5.98	6	1541	{}	\N	2020-10-12 14:29:03.880882
32217	CELC	2020-09-30	6.0021	6.0021	5.8	5.8	962	{}	\N	2020-10-12 14:29:03.8876
32218	CELC	2020-10-01	5.8527	6.645	5.85	6.3	9773	{}	\N	2020-10-12 14:29:03.893108
32219	CELC	2020-10-02	5.908	6.295	5.43	5.67	12772	{}	\N	2020-10-12 14:29:03.898013
32220	CELC	2020-10-05	5.87	5.87	5.87	5.87	828	{}	\N	2020-10-12 14:29:03.902774
32221	CELC	2020-10-06	5.78	6.2	5.78	5.79	2447	{}	\N	2020-10-12 14:29:03.906929
32222	CELC	2020-10-07	5.69	5.76	5.53	5.75	8695	{}	\N	2020-10-12 14:29:03.910714
32223	CELC	2020-10-08	5.89	5.89	5.62	5.73	6685	{}	\N	2020-10-12 14:29:03.914603
32224	CELC	2020-10-09	5.7801	5.7801	5.78	5.78	2100	{}	\N	2020-10-12 14:29:03.919012
32225	CEMI	2020-09-28	4.65	4.84	4.5889	4.72	1102290	{}	\N	2020-10-12 14:29:05.539653
32226	CEMI	2020-09-29	4.65	4.84	4.62	4.76	1192637	{}	\N	2020-10-12 14:29:05.549961
32227	CEMI	2020-09-30	4.75	4.935	4.67	4.86	1789560	{}	\N	2020-10-12 14:29:05.556247
32228	CEMI	2020-10-01	4.83	5.02	4.74	4.85	1628672	{}	\N	2020-10-12 14:29:05.562187
32229	CEMI	2020-10-02	4.96	5.19	4.7	4.8	7594409	{}	\N	2020-10-12 14:29:05.567407
32230	CEMI	2020-10-05	4.81	5.7	4.81	5.44	10760230	{}	\N	2020-10-12 14:29:05.572543
32231	CEMI	2020-10-06	5.4083	5.45	4.88	4.92	4962545	{}	\N	2020-10-12 14:29:05.577439
32232	CEMI	2020-10-07	5.01	5.2	4.9313	5.02	2406129	{}	\N	2020-10-12 14:29:05.582574
32233	CEMI	2020-10-08	5.0825	5.09	4.965	4.99	1271481	{}	\N	2020-10-12 14:29:05.586888
32234	CEMI	2020-10-09	4.98	5.08	4.93	5.04	1339298	{}	\N	2020-10-12 14:29:05.591116
32235	CEMI	2020-10-12	5.01	5.1	4.97	4.97	304910	{}	\N	2020-10-12 14:29:05.595425
32236	CERN	2020-09-28	70.05	71.63	69.81	71.6	1748901	{}	\N	2020-10-12 14:29:06.741572
32237	CERN	2020-09-29	71.38	72.5	71.1	71.19	1187836	{}	\N	2020-10-12 14:29:06.749855
32238	CERN	2020-09-30	71.58	73.16	70.639	72.29	1527807	{}	\N	2020-10-12 14:29:06.756053
32239	CERN	2020-10-01	72.85	73.3	71.75	72.31	1270553	{}	\N	2020-10-12 14:29:06.761852
32240	CERN	2020-10-02	71.51	71.98	70.48	71.18	939197	{}	\N	2020-10-12 14:29:06.766407
32241	CERN	2020-10-05	70.71	71.96	70.71	71.76	1279781	{}	\N	2020-10-12 14:29:06.770823
32242	CERN	2020-10-06	72.17	73.14	71.5	71.98	1229562	{}	\N	2020-10-12 14:29:06.774602
32243	CERN	2020-10-07	72.82	72.93	71.4	71.48	995306	{}	\N	2020-10-12 14:29:06.779012
32244	CERN	2020-10-08	71.84	72.1839	71.44	71.84	785585	{}	\N	2020-10-12 14:29:06.783338
32245	CERN	2020-10-09	73.18	73.18	71.87	72.42	795667	{}	\N	2020-10-12 14:29:06.787187
32246	CERN	2020-10-12	72.85	74.15	72.7	74.12	333534	{}	\N	2020-10-12 14:29:06.79112
32247	CF	2020-09-28	31.15	31.5	30.89	30.95	1414312	{}	\N	2020-10-12 14:29:07.740929
32248	CF	2020-09-29	30.9	30.9	29.77	29.86	3186359	{}	\N	2020-10-12 14:29:07.746657
32249	CF	2020-09-30	30.04	31.08	29.99	30.71	4581004	{}	\N	2020-10-12 14:29:07.751256
32250	CF	2020-10-01	30.62	30.89	29.77	29.88	3889483	{}	\N	2020-10-12 14:29:07.755478
32251	CF	2020-10-02	29.49	30.85	29.3	30.57	2971309	{}	\N	2020-10-12 14:29:07.760173
32252	CF	2020-10-05	30.75	31.77	30.72	31.53	3197599	{}	\N	2020-10-12 14:29:07.764935
32253	CF	2020-10-06	31.92	32.62	31.25	31.29	3066793	{}	\N	2020-10-12 14:29:07.769062
32254	CF	2020-10-07	31.64	31.87	30.93	30.95	4195480	{}	\N	2020-10-12 14:29:07.773502
32255	CF	2020-10-08	31.34	31.86	31.11	31.86	2470380	{}	\N	2020-10-12 14:29:07.777991
32256	CF	2020-10-09	31.96	32.28	30.97	31.33	3671098	{}	\N	2020-10-12 14:29:07.782497
32257	CF	2020-10-12	31.12	31.17	30.36	30.56	847948	{}	\N	2020-10-12 14:29:07.787264
32258	CFR	2020-09-28	63.91	65.38	63.75	64.79	323362	{}	\N	2020-10-12 14:29:09.097855
32259	CFR	2020-09-29	64.75	64.75	61.84	63.25	1177620	{}	\N	2020-10-12 14:29:09.106282
32260	CFR	2020-09-30	63.74	64.8	63.23	63.95	758520	{}	\N	2020-10-12 14:29:09.11171
32261	CFR	2020-10-01	63.42	64.61	63	63.75	627536	{}	\N	2020-10-12 14:29:09.117271
32262	CFR	2020-10-02	62.37	65.44	62.26	64.99	766233	{}	\N	2020-10-12 14:29:09.122258
32263	CFR	2020-10-05	65.7	67.32	65.27	67.02	997288	{}	\N	2020-10-12 14:29:09.126363
32264	CFR	2020-10-06	68.18	70.69	67.4	67.87	694106	{}	\N	2020-10-12 14:29:09.130677
32265	CFR	2020-10-07	68.63	70.37	68.52	69.48	638673	{}	\N	2020-10-12 14:29:09.134695
32266	CFR	2020-10-08	69.88	70.45	69.215	70.02	479650	{}	\N	2020-10-12 14:29:09.138861
32267	CFR	2020-10-09	70.5	70.9014	69.44	69.59	441419	{}	\N	2020-10-12 14:29:09.142936
32268	CFR	2020-10-12	69.5	69.82	69.11	69.7	28968	{}	\N	2020-10-12 14:29:09.147115
32269	CHCO	2020-09-28	57.18	58.29	57.18	57.88	89179	{}	\N	2020-10-12 14:29:11.243246
32270	CHCO	2020-09-29	57.71	57.82	56.59	57.39	64051	{}	\N	2020-10-12 14:29:11.253124
32271	CHCO	2020-09-30	57.08	58.4	57.08	57.61	106884	{}	\N	2020-10-12 14:29:11.260257
32272	CHCO	2020-10-01	57.51	57.94	56.28	56.98	59195	{}	\N	2020-10-12 14:29:11.266176
32273	CHCO	2020-10-02	56.25	57.85	55.74	57.59	79046	{}	\N	2020-10-12 14:29:11.271703
32274	CHCO	2020-10-05	58.16	59.68	57.605	58.94	68844	{}	\N	2020-10-12 14:29:11.277065
32275	CHCO	2020-10-06	59.39	61.03	58.76	59.06	76258	{}	\N	2020-10-12 14:29:11.281949
32276	CHCO	2020-10-07	59.35	60.79	59.12	59.66	63505	{}	\N	2020-10-12 14:29:11.28653
32277	CHCO	2020-10-08	60.57	61.24	59.88	60.99	46788	{}	\N	2020-10-12 14:29:11.291312
32278	CHCO	2020-10-09	60.64	61.43	60.59	60.65	58480	{}	\N	2020-10-12 14:29:11.296052
32279	CHCO	2020-10-12	60.61	61.14	60.435	60.91	6283	{}	\N	2020-10-12 14:29:11.300722
32280	CBFV	2020-09-28	18.395	19.42	18.395	18.91	5979	{}	\N	2020-10-12 14:29:12.425521
32281	CBFV	2020-09-29	19.06	19.44	18.3	18.91	2653	{}	\N	2020-10-12 14:29:12.43498
32282	CBFV	2020-09-30	18.91	19.08	18.91	19.08	1548	{}	\N	2020-10-12 14:29:12.441917
32283	CBFV	2020-10-01	19.11	19.11	19.08	19.09	3435	{}	\N	2020-10-12 14:29:12.447742
32284	CBFV	2020-10-02	18.75	18.75	18.75	18.75	1913	{}	\N	2020-10-12 14:29:12.453126
32285	CBFV	2020-10-05	19.49	19.68	19.46	19.67	5876	{}	\N	2020-10-12 14:29:12.458357
32286	CBFV	2020-10-06	19.67	20	19.61	19.74	5993	{}	\N	2020-10-12 14:29:12.462886
32287	CBFV	2020-10-07	19.99	20	19.5576	19.84	8721	{}	\N	2020-10-12 14:29:12.467146
32288	CBFV	2020-10-08	19.68	20.02	19.68	19.79	15773	{}	\N	2020-10-12 14:29:12.471633
32289	CBFV	2020-10-09	20.65	20.85	20.425	20.46	5939	{}	\N	2020-10-12 14:29:12.476255
32290	CBFV	2020-10-12	20.44	21.3942	20.44	20.6	782	{}	\N	2020-10-12 14:29:12.48075
32291	CBON	2020-09-28	22.78	22.78	22.73	22.76	15224	{}	\N	2020-10-12 14:29:13.847237
32292	CBON	2020-09-29	22.76	22.8423	22.735	22.83	27888	{}	\N	2020-10-12 14:29:13.854709
32293	CBON	2020-09-30	22.87	23.01	22.835	23	43080	{}	\N	2020-10-12 14:29:13.860759
32294	CBON	2020-10-01	23.04	23.055	23.0003	23.03	34703	{}	\N	2020-10-12 14:29:13.866409
32295	CBON	2020-10-02	23.23	23.27	23.1	23.27	24143	{}	\N	2020-10-12 14:29:13.871923
32296	CBON	2020-10-05	23.29	23.29	23.1513	23.22	37955	{}	\N	2020-10-12 14:29:13.876546
32297	CBON	2020-10-06	23.32	23.36	23.2901	23.3	38597	{}	\N	2020-10-12 14:29:13.880998
32298	CBON	2020-10-07	23.38	23.38	23.23	23.26	42122	{}	\N	2020-10-12 14:29:13.885342
32299	CBON	2020-10-08	23.26	23.32	23.2146	23.275	43246	{}	\N	2020-10-12 14:29:13.889373
32300	CBON	2020-10-09	23.28	23.29	23.27	23.27	47238	{}	\N	2020-10-12 14:29:13.893458
32301	CBON	2020-10-12	23.32	23.32	23.07	23.1311	39189	{}	\N	2020-10-12 14:29:13.897995
32302	CCM	2020-09-28	2.02	2.02	1.93	1.975	785	{}	\N	2020-10-12 14:29:16.084501
32303	CCM	2020-09-29	1.966	1.9922	1.88	1.9814	7200	{}	\N	2020-10-12 14:29:16.091304
32304	CCM	2020-09-30	1.95	2	1.95	1.975	4364	{}	\N	2020-10-12 14:29:16.096347
32305	CCM	2020-10-01	1.94	1.94	1.88	1.88	3392	{}	\N	2020-10-12 14:29:16.101108
32306	CCM	2020-10-02	1.81	1.87	1.805	1.805	3836	{}	\N	2020-10-12 14:29:16.105321
32307	CCM	2020-10-05	1.82	2.0371	1.82	2.0371	12660	{}	\N	2020-10-12 14:29:16.110898
32308	CCM	2020-10-06	1.9	1.9	1.78	1.81	29710	{}	\N	2020-10-12 14:29:16.11469
32309	CCM	2020-10-07	1.81	1.8173	1.795	1.8173	3106	{}	\N	2020-10-12 14:29:16.1183
32310	CCM	2020-10-09	1.74	1.78	1.74	1.75	2550	{}	\N	2020-10-12 14:29:16.122155
32311	CCXI	2020-09-28	53.15	54.03	51.92	53.5	574039	{}	\N	2020-10-12 14:29:17.536358
32312	CCXI	2020-09-29	53.25	54.33	53.01	53.53	590601	{}	\N	2020-10-12 14:29:17.545534
32313	CCXI	2020-09-30	53.15	55.49	53.15	54.8	680027	{}	\N	2020-10-12 14:29:17.552378
32314	CCXI	2020-10-01	55.87	56.485	54.46	54.89	493428	{}	\N	2020-10-12 14:29:17.55806
32315	CCXI	2020-10-02	53.63	54.9712	52.825	53.72	238401	{}	\N	2020-10-12 14:29:17.562799
32316	CCXI	2020-10-05	54.53	59.64	54.31	59.15	1019371	{}	\N	2020-10-12 14:29:17.567144
32317	CCXI	2020-10-06	59.12	60.14	57.93	58.9	552299	{}	\N	2020-10-12 14:29:17.570892
32318	CCXI	2020-10-07	59.5	60.36	58.48	59.28	429986	{}	\N	2020-10-12 14:29:17.574251
32319	CCXI	2020-10-08	59.49	59.82	57.98	59.06	442224	{}	\N	2020-10-12 14:29:17.578307
32320	CCXI	2020-10-09	59.82	60.64	58.55	59.27	583042	{}	\N	2020-10-12 14:29:17.582307
32321	CCXI	2020-10-12	59.88	60.56	58.05	59.18	26256	{}	\N	2020-10-12 14:29:17.585765
32322	CDOR	2020-09-28	2.8	2.88	2.72	2.8	52810	{}	\N	2020-10-12 14:29:18.485389
32323	CDOR	2020-09-29	2.761	2.785	2.69	2.715	22964	{}	\N	2020-10-12 14:29:18.49404
32324	CDOR	2020-09-30	2.74	2.75	2.62	2.62	21307	{}	\N	2020-10-12 14:29:18.500885
32325	CDOR	2020-10-01	2.61	2.62	2.4501	2.485	47583	{}	\N	2020-10-12 14:29:18.506837
32326	CDOR	2020-10-02	2.4	2.51	2.4	2.48	21086	{}	\N	2020-10-12 14:29:18.512104
32327	CDOR	2020-10-05	2.53	2.53	2.25	2.34	88779	{}	\N	2020-10-12 14:29:18.517154
32328	CDOR	2020-10-06	2.31	2.46	2.26	2.26	36913	{}	\N	2020-10-12 14:29:18.522787
32329	CDOR	2020-10-07	2.37	2.49	2.33	2.49	37472	{}	\N	2020-10-12 14:29:18.526788
32330	CDOR	2020-10-08	2.55	2.6	2.26	2.32	125156	{}	\N	2020-10-12 14:29:18.530974
32331	CDOR	2020-10-09	2.32	2.4195	2.32	2.3705	8050	{}	\N	2020-10-12 14:29:18.535559
32332	CDOR	2020-10-12	2.45	2.48	2.4017	2.47	23323	{}	\N	2020-10-12 14:29:18.540101
32333	CECE	2020-09-28	7.26	7.5	7.26	7.36	43347	{}	\N	2020-10-12 14:29:19.460536
32334	CECE	2020-09-29	7.31	7.37	7.15	7.31	26633	{}	\N	2020-10-12 14:29:19.47087
32335	CECE	2020-09-30	7.37	7.7	7.236	7.3	28921	{}	\N	2020-10-12 14:29:19.478182
32336	CECE	2020-10-01	7.29	7.32	7.04	7.15	42832	{}	\N	2020-10-12 14:29:19.484741
32337	CECE	2020-10-02	7	7.54	7	7.45	93950	{}	\N	2020-10-12 14:29:19.489685
32338	CECE	2020-10-05	7.47	7.88	7.47	7.76	75261	{}	\N	2020-10-12 14:29:19.494208
32339	CECE	2020-10-06	7.85	8.08	7.62	7.9	63994	{}	\N	2020-10-12 14:29:19.498308
32340	CECE	2020-10-07	7.985	8.095	7.81	7.93	80441	{}	\N	2020-10-12 14:29:19.502575
32341	CECE	2020-10-08	7.98	8.05	7.85	8	46293	{}	\N	2020-10-12 14:29:19.506464
32342	CECE	2020-10-09	8.1	8.13	7.94	7.99	88655	{}	\N	2020-10-12 14:29:19.510491
32343	CECE	2020-10-12	7.98	8.04	7.94	7.99	15932	{}	\N	2020-10-12 14:29:19.51443
32344	CEI	2020-09-28	0.576	0.5799	0.5011	0.52	3098866	{}	\N	2020-10-12 14:29:20.759454
32345	CEI	2020-09-29	0.59	0.8348	0.56001	0.8	24604672	{}	\N	2020-10-12 14:29:20.769705
32346	CEI	2020-09-30	0.663	0.69	0.63	0.64	6103187	{}	\N	2020-10-12 14:29:20.777002
32347	CEI	2020-10-01	0.6212	0.64	0.55	0.6284	3597530	{}	\N	2020-10-12 14:29:20.783327
32348	CEI	2020-10-02	0.57	0.6248	0.5519	0.58	2161340	{}	\N	2020-10-12 14:29:20.788764
32349	CEI	2020-10-05	0.5911	0.8	0.58	0.7851	22246585	{}	\N	2020-10-12 14:29:20.793384
32350	CEI	2020-10-06	0.875	0.875	0.6803	0.7056	18553067	{}	\N	2020-10-12 14:29:20.79774
32351	CEI	2020-10-07	0.67	0.75	0.6655	0.701	6380450	{}	\N	2020-10-12 14:29:20.8024
32352	CEI	2020-10-08	0.73	1.43	0.7	1.25	120544795	{}	\N	2020-10-12 14:29:20.806766
32353	CEI	2020-10-09	1.09	1.16	0.86	0.945	49168314	{}	\N	2020-10-12 14:29:20.810842
32354	CEI	2020-10-12	0.8715	1	0.87	0.983	5424587	{}	\N	2020-10-12 14:29:20.815263
32355	CENT	2020-09-28	38.14	38.68	38.05	38.54	61357	{}	\N	2020-10-12 14:29:22.157058
32356	CENT	2020-09-29	38.8	39.2	38.565	39.2	66821	{}	\N	2020-10-12 14:29:22.16491
32357	CENT	2020-09-30	39.34	40.04	39.015	39.93	116721	{}	\N	2020-10-12 14:29:22.170787
32358	CENT	2020-10-01	40.17	40.33	38.825	40.23	148848	{}	\N	2020-10-12 14:29:22.17808
32359	CENT	2020-10-02	39.3	40.325	39.27	40.01	64192	{}	\N	2020-10-12 14:29:22.184427
32360	CENT	2020-10-05	40.27	40.55	39.15	40.11	62217	{}	\N	2020-10-12 14:29:22.18958
32361	CENT	2020-10-06	40.51	41.19	39.79	40.44	57843	{}	\N	2020-10-12 14:29:22.194537
32362	CENT	2020-10-07	40.68	41.055	40.34	40.73	46070	{}	\N	2020-10-12 14:29:22.199432
32363	CENT	2020-10-08	40.89	41.77	40.44	41.62	59719	{}	\N	2020-10-12 14:29:22.204379
32364	CENT	2020-10-09	41.97	42.71	41.78	42.7	40021	{}	\N	2020-10-12 14:29:22.209275
32365	CENT	2020-10-12	42.88	43.1	42.7	42.94	8537	{}	\N	2020-10-12 14:29:22.213924
32366	CETV	2020-09-28	4.25	4.36	4.24	4.31	78317	{}	\N	2020-10-12 14:29:23.10824
32367	CETV	2020-09-29	4.26	4.37	4.26	4.37	117594	{}	\N	2020-10-12 14:29:23.119704
32368	CETV	2020-09-30	4.37	4.37	4.19	4.19	291905	{}	\N	2020-10-12 14:29:23.126978
32369	CETV	2020-10-01	4.2	4.24	4.18	4.21	492534	{}	\N	2020-10-12 14:29:23.132675
32370	CETV	2020-10-02	4.22	4.25	4.2	4.21	193203	{}	\N	2020-10-12 14:29:23.138108
32371	CETV	2020-10-05	4.24	4.34	4.22	4.335	165377	{}	\N	2020-10-12 14:29:23.142917
32372	CETV	2020-10-06	4.34	4.5601	4.28	4.555	5700697	{}	\N	2020-10-12 14:29:23.147206
32373	CETV	2020-10-07	4.55	4.58	4.55	4.58	3017358	{}	\N	2020-10-12 14:29:23.151892
32374	CETV	2020-10-08	4.57	4.58	4.56	4.57	1725298	{}	\N	2020-10-12 14:29:23.156424
32375	CETV	2020-10-09	4.58	4.58	4.57	4.58	170262	{}	\N	2020-10-12 14:29:23.16105
32376	CETV	2020-10-12	4.58	4.58	4.57	4.57	36319	{}	\N	2020-10-12 14:29:23.165763
32377	CEY	2020-09-28	18.64	18.64	18.57	18.6	5045	{}	\N	2020-10-12 14:29:24.098492
32378	CEY	2020-09-29	18.52	18.6045	18.52	18.6045	838	{}	\N	2020-10-12 14:29:24.108836
32379	CEY	2020-09-30	18.58	18.6927	18.58	18.6383	28107	{}	\N	2020-10-12 14:29:24.115138
32380	CEY	2020-10-01	18.7	18.79	18.675	18.79	19463	{}	\N	2020-10-12 14:29:24.121702
32381	CEY	2020-10-02	18.6463	18.736	18.6463	18.66	6494	{}	\N	2020-10-12 14:29:24.126937
32382	CEY	2020-10-05	18.75	18.75	18.7	18.7101	871	{}	\N	2020-10-12 14:29:24.131237
32383	CEY	2020-10-06	18.73	18.73	18.73	18.73	109	{}	\N	2020-10-12 14:29:24.135982
32384	CEY	2020-10-07	18.7666	18.797	18.7666	18.797	4290	{}	\N	2020-10-12 14:29:24.140172
32385	CEY	2020-10-08	18.82	18.9401	18.82	18.9401	2411	{}	\N	2020-10-12 14:29:24.144338
32386	CEY	2020-10-09	19.039	19.039	19.0002	19.0002	250	{}	\N	2020-10-12 14:29:24.148368
32387	CEY	2020-10-12	18.98	18.9967	18.97	18.9967	1001	{}	\N	2020-10-12 14:29:24.152443
32388	CFRX	2020-09-28	5.21	5.3	5.07	5.24	159893	{}	\N	2020-10-12 14:29:25.130972
32389	CFRX	2020-09-29	5.22	5.44	5.22	5.29	122296	{}	\N	2020-10-12 14:29:25.141887
32390	CFRX	2020-09-30	5.29	5.4	5.11	5.28	95812	{}	\N	2020-10-12 14:29:25.14811
32391	CFRX	2020-10-01	5.32	5.32	5.1201	5.17	65036	{}	\N	2020-10-12 14:29:25.153596
32392	CFRX	2020-10-02	5.16	5.27	5.085	5.16	71195	{}	\N	2020-10-12 14:29:25.158587
32393	CFRX	2020-10-05	5.18	5.44	5.13	5.28	138194	{}	\N	2020-10-12 14:29:25.163564
32394	CFRX	2020-10-06	5.3	5.46	5.2	5.3	105779	{}	\N	2020-10-12 14:29:25.167728
32395	CFRX	2020-10-07	5.33	5.5	5.17	5.45	153462	{}	\N	2020-10-12 14:29:25.172157
32396	CFRX	2020-10-08	5.47	5.49	5.36	5.42	72541	{}	\N	2020-10-12 14:29:25.176247
32397	CFRX	2020-10-09	5.44	5.45	5.285	5.43	71539	{}	\N	2020-10-12 14:29:25.180406
32398	CFRX	2020-10-12	5.43	5.44	5.4	5.4212	4543	{}	\N	2020-10-12 14:29:25.184166
32399	CHA	2020-09-28	30.23	30.36	30.07	30.27	61935	{}	\N	2020-10-12 14:29:26.341466
32400	CHA	2020-09-29	30.15	30.21	30.035	30.16	86062	{}	\N	2020-10-12 14:29:26.349334
32401	CHA	2020-09-30	30.08	30.22	29.93	30.02	101409	{}	\N	2020-10-12 14:29:26.354927
32402	CHA	2020-10-01	30.35	30.48	30.12	30.45	69685	{}	\N	2020-10-12 14:29:26.360747
32403	CHA	2020-10-02	30.08	30.45	30.08	30.26	80599	{}	\N	2020-10-12 14:29:26.365783
32404	CHA	2020-10-05	30.75	31.03	30.57	30.88	103982	{}	\N	2020-10-12 14:29:26.370681
32405	CHA	2020-10-06	30.3	30.47	30.14	30.19	179191	{}	\N	2020-10-12 14:29:26.374949
32406	CHA	2020-10-07	30.06	30.26	30.02	30.2	161224	{}	\N	2020-10-12 14:29:26.379198
32407	CHA	2020-10-08	30.61	30.61	30.27	30.41	116383	{}	\N	2020-10-12 14:29:26.383831
32408	CHA	2020-10-09	30.34	30.45	30.251	30.3	131692	{}	\N	2020-10-12 14:29:26.388147
32409	CHA	2020-10-12	31.47	31.72	31.42	31.66	28316	{}	\N	2020-10-12 14:29:26.392681
32410	CCBG	2020-09-28	18.2	18.97	18.2	18.85	37027	{}	\N	2020-10-12 14:29:27.605492
32411	CCBG	2020-09-29	18.6881	18.74	18.26	18.74	12370	{}	\N	2020-10-12 14:29:27.614337
32412	CCBG	2020-09-30	18.74	19.05	18.59	18.81	22510	{}	\N	2020-10-12 14:29:27.620577
32413	CCBG	2020-10-01	18.86	18.97	18.56	18.94	21576	{}	\N	2020-10-12 14:29:27.626323
32414	CCBG	2020-10-02	18.145	19.84	18.145	19.76	27631	{}	\N	2020-10-12 14:29:27.631037
32415	CCBG	2020-10-05	19.3666	20.5	19.3666	20.5	26048	{}	\N	2020-10-12 14:29:27.636487
32416	CCBG	2020-10-06	20.15	21.6	20.15	20.71	26105	{}	\N	2020-10-12 14:29:27.641151
32417	CCBG	2020-10-07	21.385	21.86	21.21	21.57	23435	{}	\N	2020-10-12 14:29:27.645322
32418	CCBG	2020-10-08	21.8	22.09	21.49	21.83	19330	{}	\N	2020-10-12 14:29:27.6495
32419	CCBG	2020-10-09	21.9	22.07	21.47	21.54	12377	{}	\N	2020-10-12 14:29:27.654002
32420	CCBG	2020-10-12	21.4	21.53	21.21	21.42	1588	{}	\N	2020-10-12 14:29:27.658484
32421	CCLP	2020-09-28	0.84	0.88	0.83	0.83	65437	{}	\N	2020-10-12 14:29:28.76715
32422	CCLP	2020-09-29	0.8151	0.88	0.8	0.84	82361	{}	\N	2020-10-12 14:29:28.775777
32423	CCLP	2020-09-30	0.8261	0.91	0.825	0.89005	25755	{}	\N	2020-10-12 14:29:28.781889
32424	CCLP	2020-10-01	0.85	0.89	0.85	0.86	63394	{}	\N	2020-10-12 14:29:28.787932
32425	CCLP	2020-10-02	0.84	0.88	0.815101	0.88	66856	{}	\N	2020-10-12 14:29:28.793334
32426	CCLP	2020-10-05	0.85	0.9098	0.85	0.866	81792	{}	\N	2020-10-12 14:29:28.798118
32427	CCLP	2020-10-06	0.88	0.8998	0.84	0.84	60321	{}	\N	2020-10-12 14:29:28.802513
32428	CCLP	2020-10-07	0.8587	0.86	0.83	0.84	46984	{}	\N	2020-10-12 14:29:28.806829
32429	CCLP	2020-10-08	0.84	0.91	0.8	0.89505	130511	{}	\N	2020-10-12 14:29:28.811408
32430	CCLP	2020-10-09	0.860101	0.91	0.860101	0.889201	77794	{}	\N	2020-10-12 14:29:28.815378
32431	CCLP	2020-10-12	0.894049	0.895	0.872	0.885	6942	{}	\N	2020-10-12 14:29:28.819818
32432	CDMO	2020-09-28	7.6	7.68	7.385	7.49	201975	{}	\N	2020-10-12 14:29:30.547122
32433	CDMO	2020-09-29	7.5	7.75	7.425	7.69	195888	{}	\N	2020-10-12 14:29:30.555003
32434	CDMO	2020-09-30	7.73	7.8	7.51	7.615	216367	{}	\N	2020-10-12 14:29:30.561491
32435	CDMO	2020-10-01	7.67	7.78	7.51	7.75	156527	{}	\N	2020-10-12 14:29:30.567
32436	CDMO	2020-10-02	7.56	7.8	7.5	7.51	223818	{}	\N	2020-10-12 14:29:30.572749
32437	CDMO	2020-10-05	7.56	7.795	7.3392	7.765	272186	{}	\N	2020-10-12 14:29:30.578121
32438	CDMO	2020-10-06	7.76	7.88	7.52	7.71	384877	{}	\N	2020-10-12 14:29:30.582593
32439	CDMO	2020-10-07	7.69	7.91	7.585	7.89	319697	{}	\N	2020-10-12 14:29:30.586995
32440	CDMO	2020-10-08	8.03	8.14	7.89	7.91	200094	{}	\N	2020-10-12 14:29:30.591334
32441	CDMO	2020-10-09	7.88	8.06	7.76	8.04	160558	{}	\N	2020-10-12 14:29:30.595693
32442	CDMO	2020-10-12	7.96	8.19	7.95	8.085	35648	{}	\N	2020-10-12 14:29:30.600213
32443	CDXC	2020-09-28	4.0681	4.11	4.01	4.08	86654	{}	\N	2020-10-12 14:29:31.618744
32444	CDXC	2020-09-29	4.09	4.12	3.93	4.02	157664	{}	\N	2020-10-12 14:29:31.629805
32445	CDXC	2020-09-30	4.02	4.08	3.95	4.01	127999	{}	\N	2020-10-12 14:29:31.637436
32446	CDXC	2020-10-01	4.01	4.02	3.92	3.95	175888	{}	\N	2020-10-12 14:29:31.643775
32447	CDXC	2020-10-02	3.86	4	3.82	3.93	207701	{}	\N	2020-10-12 14:29:31.648717
32448	CDXC	2020-10-05	3.97	4.09	3.97	4.07	182870	{}	\N	2020-10-12 14:29:31.653013
32449	CDXC	2020-10-06	4.3	5.49	4.19	5.19	3172950	{}	\N	2020-10-12 14:29:31.658118
32450	CDXC	2020-10-07	5.52	6	4.9	5.04	2792023	{}	\N	2020-10-12 14:29:31.662197
32451	CDXC	2020-10-08	4.9	4.95	4.66	4.73	936804	{}	\N	2020-10-12 14:29:31.666108
32452	CDXC	2020-10-09	4.72	4.98	4.6789	4.97	271587	{}	\N	2020-10-12 14:29:31.669859
32453	CDXC	2020-10-12	5.02	5.08	4.93	4.98	78704	{}	\N	2020-10-12 14:29:31.674024
32454	CEFS	2020-09-28	18.2798	18.49	18.16	18.16	9122	{}	\N	2020-10-12 14:29:32.703966
32455	CEFS	2020-09-29	18.5	18.5	18.19	18.2499	6386	{}	\N	2020-10-12 14:29:32.714605
32456	CEFS	2020-09-30	18.41	18.41	18.17	18.2999	8369	{}	\N	2020-10-12 14:29:32.722394
32457	CEFS	2020-10-01	18.41	18.41	18.2733	18.3	6853	{}	\N	2020-10-12 14:29:32.72796
32458	CEFS	2020-10-02	18.29	18.44	18.28	18.44	14087	{}	\N	2020-10-12 14:29:32.733461
32459	CEFS	2020-10-05	18.5	18.5	18.34	18.37	25208	{}	\N	2020-10-12 14:29:32.738402
32460	CEFS	2020-10-06	18.29	18.43	18.23	18.375	13864	{}	\N	2020-10-12 14:29:32.743068
32461	CEFS	2020-10-07	18.29	18.5	18.27	18.5	8789	{}	\N	2020-10-12 14:29:32.747589
32462	CEFS	2020-10-08	18.47	18.71	18.47	18.614	12427	{}	\N	2020-10-12 14:29:32.753389
32463	CEFS	2020-10-09	18.6	18.6	18.56	18.5828	5594	{}	\N	2020-10-12 14:29:32.759115
32464	CEFS	2020-10-12	18.55	18.55	18.5458	18.5458	582	{}	\N	2020-10-12 14:29:32.764891
32465	CEM	2020-09-28	11.09	11.4	11.01	11.18	64566	{}	\N	2020-10-12 14:29:34.55582
32466	CEM	2020-09-29	11.22	11.4398	11.1	11.3594	133353	{}	\N	2020-10-12 14:29:34.566017
32467	CEM	2020-09-30	11.44	11.5699	10.9401	11.05	155796	{}	\N	2020-10-12 14:29:34.572945
32468	CEM	2020-10-01	11.02	11.096	10.7	10.79	163025	{}	\N	2020-10-12 14:29:34.579525
32469	CEM	2020-10-02	10.65	10.99	10.61	10.98	222015	{}	\N	2020-10-12 14:29:34.584728
32470	CEM	2020-10-05	11.17	11.795	11.17	11.78	556326	{}	\N	2020-10-12 14:29:34.589813
32471	CEM	2020-10-06	11.88	12.195	11.7173	11.8	269791	{}	\N	2020-10-12 14:29:34.594137
32472	CEM	2020-10-07	12	12.1522	11.7	11.98	189916	{}	\N	2020-10-12 14:29:34.598329
32473	CEM	2020-10-08	12.16	12.89	12.09	12.82	311350	{}	\N	2020-10-12 14:29:34.602726
32474	CEM	2020-10-09	12.96	13.1604	12.75	13.04	251973	{}	\N	2020-10-12 14:29:34.60717
32475	CEM	2020-10-12	13.16	13.16	12.995	13.0094	58906	{}	\N	2020-10-12 14:29:34.611297
32476	CET	2020-09-28	28.84	29.06	28.784	28.95	19412	{}	\N	2020-10-12 14:29:35.900891
32477	CET	2020-09-29	28.825	28.988	28.82	28.94	27759	{}	\N	2020-10-12 14:29:35.909904
32478	CET	2020-09-30	29.0402	29.2099	28.99	28.99	21278	{}	\N	2020-10-12 14:29:35.91644
32479	CET	2020-10-01	29.1933	29.25	29	29.25	15278	{}	\N	2020-10-12 14:29:35.922832
32480	CET	2020-10-02	28.94	29.05	28.75	28.75	18055	{}	\N	2020-10-12 14:29:35.927975
32481	CET	2020-10-05	29	29.17	29	29.14	12919	{}	\N	2020-10-12 14:29:35.932697
32482	CET	2020-10-06	29.135	29.32	28.91	28.91	23077	{}	\N	2020-10-12 14:29:35.936775
32483	CET	2020-10-07	29.03	29.3	29.0101	29.16	18490	{}	\N	2020-10-12 14:29:35.941015
32484	CET	2020-10-08	29.33	29.3432	29.18	29.21	34462	{}	\N	2020-10-12 14:29:35.945016
32485	CET	2020-10-09	29.36	29.46	29.34	29.39	24895	{}	\N	2020-10-12 14:29:35.949056
32486	CET	2020-10-12	29.39	29.5208	29.37	29.5	19617	{}	\N	2020-10-12 14:29:35.953006
32487	CEZ	2020-09-28	23.16	23.16	23.08	23.08	642	{}	\N	2020-10-12 14:29:37.091322
32488	CEZ	2020-09-30	23.35	23.35	23.3087	23.3087	204	{}	\N	2020-10-12 14:29:37.098659
32489	CEZ	2020-10-01	23.2954	23.4478	23.2954	23.4478	1387	{}	\N	2020-10-12 14:29:37.104264
32490	CEZ	2020-10-02	23.3107	23.3107	23.2845	23.2845	1258	{}	\N	2020-10-12 14:29:37.109685
32491	CEZ	2020-10-05	23.41	23.4391	23.41	23.4391	131	{}	\N	2020-10-12 14:29:37.115841
32492	CEZ	2020-10-06	23.521	23.56	23.521	23.56	1012	{}	\N	2020-10-12 14:29:37.119888
32493	CEZ	2020-10-07	23.58	23.58	23.5611	23.5611	101	{}	\N	2020-10-12 14:29:37.124179
32494	CFMS	2020-09-28	0.8889	0.9297	0.861	0.8755	681859	{}	\N	2020-10-12 14:29:38.106357
32495	CFMS	2020-09-29	0.89	1.12	0.8513	0.862	2420459	{}	\N	2020-10-12 14:29:38.117943
32496	CFMS	2020-09-30	0.8506	0.87	0.81144	0.8375	409812	{}	\N	2020-10-12 14:29:38.124892
32497	CFMS	2020-10-01	0.84	0.8518	0.7629	0.7978	622759	{}	\N	2020-10-12 14:29:38.13101
32498	CFMS	2020-10-02	0.797	0.797	0.75	0.7518	659376	{}	\N	2020-10-12 14:29:38.136338
32499	CFMS	2020-10-05	0.75	0.79	0.7298	0.749	616296	{}	\N	2020-10-12 14:29:38.14149
32500	CFMS	2020-10-06	0.77	0.8	0.75	0.75	580406	{}	\N	2020-10-12 14:29:38.146288
32501	CFMS	2020-10-07	0.787	0.8	0.7318	0.7423	570167	{}	\N	2020-10-12 14:29:38.150768
32502	CFMS	2020-10-08	0.7719	0.7754	0.75	0.756	289519	{}	\N	2020-10-12 14:29:38.155797
32503	CFMS	2020-10-09	0.77	0.7899	0.75	0.7642	376570	{}	\N	2020-10-12 14:29:38.160782
32504	CFMS	2020-10-12	0.7743	0.788	0.76	0.761212	35460	{}	\N	2020-10-12 14:29:38.16723
32505	CGO	2020-09-28	11.82	11.95	11.75	11.82	37956	{}	\N	2020-10-12 14:29:39.296262
32506	CGO	2020-09-29	11.79	11.8562	11.74	11.76	33111	{}	\N	2020-10-12 14:29:39.306361
32507	CGO	2020-09-30	11.77	11.91	11.76	11.86	34960	{}	\N	2020-10-12 14:29:39.312966
32508	CGO	2020-10-01	11.86	12	11.84	11.95	50544	{}	\N	2020-10-12 14:29:39.319081
32509	CGO	2020-10-02	11.83	11.94	11.79	11.92	33624	{}	\N	2020-10-12 14:29:39.323887
32510	CGO	2020-10-05	11.92	12.2	11.92	12.13	48806	{}	\N	2020-10-12 14:29:39.328516
32511	CGO	2020-10-06	12.11	12.3599	12.11	12.13	50479	{}	\N	2020-10-12 14:29:39.334411
32512	CGO	2020-10-07	12.3	12.49	12.2901	12.47	46329	{}	\N	2020-10-12 14:29:39.341235
32513	CGO	2020-10-08	12.54	12.665	12.4973	12.665	26000	{}	\N	2020-10-12 14:29:39.345261
32514	CGO	2020-10-09	12.72	12.85	12.5963	12.7201	20705	{}	\N	2020-10-12 14:29:39.349108
32515	CGO	2020-10-12	12.83	12.9765	12.7764	12.86	10029	{}	\N	2020-10-12 14:29:39.35335
32516	CHDN	2020-09-28	172.03	177	169.91	174.25	336042	{}	\N	2020-10-12 14:29:40.493484
32517	CHDN	2020-09-29	174.29	176	165.96	165.96	308355	{}	\N	2020-10-12 14:29:40.503839
32518	CHDN	2020-09-30	168.12	170.2	162.01	163.82	303367	{}	\N	2020-10-12 14:29:40.511087
32519	CHDN	2020-10-01	164.77	170.81	164.045	169.93	274778	{}	\N	2020-10-12 14:29:40.517117
32520	CHDN	2020-10-02	163.93	172.55	162.805	171.2	159621	{}	\N	2020-10-12 14:29:40.52223
32521	CHDN	2020-10-05	173.32	173.32	168.59	171.34	212167	{}	\N	2020-10-12 14:29:40.527286
32522	CHDN	2020-10-06	173.83	177.55	167.78	168.58	468747	{}	\N	2020-10-12 14:29:40.531719
32523	CHDN	2020-10-07	170.96	175.85	168.685	173.34	804072	{}	\N	2020-10-12 14:29:40.535955
32524	CHDN	2020-10-08	176.34	177.57	170.85	175.96	228539	{}	\N	2020-10-12 14:29:40.540325
32525	CHDN	2020-10-09	177.2335	177.74	169.35	170.68	380090	{}	\N	2020-10-12 14:29:40.544894
32526	CHDN	2020-10-12	170.88	173.5775	170.83	172.75	18076	{}	\N	2020-10-12 14:29:40.548957
32567	CFO	2020-10-07	55.5	56.14	55.5	56.06	14515	{}	\N	2020-10-12 14:29:46.786712
32568	CFO	2020-10-08	56.27	56.66	56.27	56.6301	24315	{}	\N	2020-10-12 14:29:46.790993
32569	CFO	2020-10-09	57.0626	57.1644	56.8579	56.96	21972	{}	\N	2020-10-12 14:29:46.795306
32570	CFO	2020-10-12	57.18	57.26	57.1451	57.26	1489	{}	\N	2020-10-12 14:29:46.799301
32571	CGW	2020-09-28	40.98	41.2	40.98	41.08	39714	{}	\N	2020-10-12 14:29:47.708527
32572	CGW	2020-09-29	41.16	41.3419	41.07	41.21	37549	{}	\N	2020-10-12 14:29:47.719386
32573	CGW	2020-09-30	41.24	41.6224	41.14	41.35	63398	{}	\N	2020-10-12 14:29:47.725793
32574	CGW	2020-10-01	41.69	41.78	41.4285	41.7	54850	{}	\N	2020-10-12 14:29:47.731465
32575	CGW	2020-10-02	41.27	42	41.27	41.866	40616	{}	\N	2020-10-12 14:29:47.736695
32576	CGW	2020-10-05	42	42.43	42	42.395	83612	{}	\N	2020-10-12 14:29:47.741572
32577	CGW	2020-10-06	42.51	42.8493	42.241	42.2683	32784	{}	\N	2020-10-12 14:29:47.74591
32578	CGW	2020-10-07	42.62	42.85	42.58	42.76	41485	{}	\N	2020-10-12 14:29:47.750771
32579	CGW	2020-10-08	42.97	43.21	42.94	43.2	36517	{}	\N	2020-10-12 14:29:47.75497
32580	CGW	2020-10-09	43.28	43.525	43.2087	43.44	33522	{}	\N	2020-10-12 14:29:47.759229
32581	CGW	2020-10-12	43.66	43.89	43.65	43.875	17106	{}	\N	2020-10-12 14:29:47.763601
32582	CHE	2020-09-28	493.59	493.68	479.68	480.36	46039	{}	\N	2020-10-12 14:29:48.712491
32583	CHE	2020-09-29	479.27	484.58	474.93	477.67	109077	{}	\N	2020-10-12 14:29:48.7248
32584	CHE	2020-09-30	480.16	484.33	474.41	480.35	175102	{}	\N	2020-10-12 14:29:48.731872
32585	CHE	2020-10-01	478.2	483.65	476.22	480.64	191124	{}	\N	2020-10-12 14:29:48.737935
32586	CHE	2020-10-02	474.96	477.42	466.45	470.82	142859	{}	\N	2020-10-12 14:29:48.743068
32587	CHE	2020-10-05	476.33	485.84	474.91	482.89	230695	{}	\N	2020-10-12 14:29:48.748142
32588	CHE	2020-10-06	486.14	486.14	471.6	478.68	125909	{}	\N	2020-10-12 14:29:48.752678
32589	CHE	2020-10-07	481.62	484.13	477.39	477.7	116783	{}	\N	2020-10-12 14:29:48.757312
32590	CHE	2020-10-08	480.72	482.5	472.55	479.46	122243	{}	\N	2020-10-12 14:29:48.761818
32591	CHE	2020-10-09	482.07	483.64	476.96	478.97	126947	{}	\N	2020-10-12 14:29:48.766022
32592	CHE	2020-10-12	482.3	484.99	482.3	484.84	6134	{}	\N	2020-10-12 14:29:48.770312
32593	CHGX	2020-09-28	25.34	25.42	25.2739	25.36	7597	{}	\N	2020-10-12 14:29:50.023756
32594	CHGX	2020-09-29	25.41	25.41	25.3282	25.3282	1390	{}	\N	2020-10-12 14:29:50.029447
32595	CHGX	2020-09-30	25.5992	25.6698	25.41	25.41	5451	{}	\N	2020-10-12 14:29:50.033852
32596	CHGX	2020-10-01	25.61	25.77	25.61	25.76	8251	{}	\N	2020-10-12 14:29:50.038418
32597	CHGX	2020-10-02	25.62	25.665	25.5307	25.63	1457	{}	\N	2020-10-12 14:29:50.04288
32598	CHGX	2020-10-05	25.92	26.0693	25.9019	26.0693	843	{}	\N	2020-10-12 14:29:50.047035
32599	CHGX	2020-10-06	25.97	26.0487	25.6499	25.6499	3742	{}	\N	2020-10-12 14:29:50.051284
32600	CHGX	2020-10-07	26.04	26.18	25.96	25.96	3090	{}	\N	2020-10-12 14:29:50.055639
32601	CHGX	2020-10-08	26.6	26.6	26.2777	26.31	1633	{}	\N	2020-10-12 14:29:50.060308
32602	CHGX	2020-10-09	26.77	26.77	26.52	26.57	5488	{}	\N	2020-10-12 14:29:50.064982
32603	CHGX	2020-10-12	27.01	27.01	27	27	885	{}	\N	2020-10-12 14:29:50.069982
32604	CHL	2020-09-28	32.57	32.65	32.42	32.54	1015000	{}	\N	2020-10-12 14:29:51.039343
32605	CHL	2020-09-29	31.88	31.92	31.71	31.83	3016159	{}	\N	2020-10-12 14:29:51.050234
32606	CHL	2020-09-30	32.12	32.275	32.02	32.16	2244743	{}	\N	2020-10-12 14:29:51.056859
32607	CHL	2020-10-01	32.27	32.57	32.24	32.5	2003439	{}	\N	2020-10-12 14:29:51.062205
32608	CHL	2020-10-02	32.17	32.605	32.17	32.32	1698654	{}	\N	2020-10-12 14:29:51.066901
32609	CHL	2020-10-05	32.14	32.3	32.05	32.28	2052072	{}	\N	2020-10-12 14:29:51.071397
32610	CHL	2020-10-06	32.14	32.15	31.84	31.96	1993926	{}	\N	2020-10-12 14:29:51.0758
32611	CHL	2020-10-07	32.42	32.73	32.355	32.69	2443252	{}	\N	2020-10-12 14:29:51.079659
32612	CHL	2020-10-08	32.65	32.65	32.46	32.61	2049864	{}	\N	2020-10-12 14:29:51.08368
32613	CHL	2020-10-09	32.46	32.46	32.155	32.39	2024635	{}	\N	2020-10-12 14:29:51.087665
32614	CHL	2020-10-12	32.83	32.94	32.75	32.895	489169	{}	\N	2020-10-12 14:29:51.091584
32615	CHN	2020-09-28	25.025	25.0999	24.92	25.035	14561	{}	\N	2020-10-12 14:29:52.314148
32616	CHN	2020-09-29	24.89	24.96	24	24.91	23066	{}	\N	2020-10-12 14:29:52.325312
32617	CHN	2020-09-30	24.95	25.2	24.95	25.18	44138	{}	\N	2020-10-12 14:29:52.332183
32618	CHN	2020-10-01	25.48	25.56	24.7631	25.25	16722	{}	\N	2020-10-12 14:29:52.339039
32619	CHN	2020-10-02	25.09	25.84	25.09	25.68	2830	{}	\N	2020-10-12 14:29:52.344351
32620	CHN	2020-10-05	25.6	25.6	25.2723	25.545	22464	{}	\N	2020-10-12 14:29:52.349062
32621	CHN	2020-10-06	25.66	25.88	25.47	25.768	16334	{}	\N	2020-10-12 14:29:52.353129
32622	CHN	2020-10-07	25.88	25.99	25.855	25.98	13132	{}	\N	2020-10-12 14:29:52.357522
32623	CHN	2020-10-08	26	26.3	25.95	26.27	26309	{}	\N	2020-10-12 14:29:52.362124
32624	CHN	2020-10-09	26.38	26.39	26.3	26.37	18420	{}	\N	2020-10-12 14:29:52.366348
32625	CHN	2020-10-12	26.68	26.99	26.6666	26.75	6842	{}	\N	2020-10-12 14:29:52.370643
32626	CHSCO	2020-09-28	27.73	28.03	27.72	28.01	10451	{}	\N	2020-10-12 14:29:53.306161
32627	CHSCO	2020-09-29	27.95	28.2	27.79	28.04	34560	{}	\N	2020-10-12 14:29:53.314088
32628	CHSCO	2020-09-30	28.19	28.19	27.79	27.79	70376	{}	\N	2020-10-12 14:29:53.319801
32629	CHSCO	2020-10-01	27.91	28.05	27.56	28.05	11037	{}	\N	2020-10-12 14:29:53.324892
32630	CHSCO	2020-10-02	27.9877	28.19	27.97	28.13	17047	{}	\N	2020-10-12 14:29:53.329431
32631	CHSCO	2020-10-05	28.15	28.24	27.99	28.225	14677	{}	\N	2020-10-12 14:29:53.334225
32632	CHSCO	2020-10-06	28.25	28.4	28.25	28.4	25052	{}	\N	2020-10-12 14:29:53.338111
32633	CHSCO	2020-10-07	28.28	28.48	28.226	28.43	11584	{}	\N	2020-10-12 14:29:53.34199
32634	CHSCO	2020-10-08	28.43	28.47	28.4	28.47	8513	{}	\N	2020-10-12 14:29:53.346114
32635	CHSCO	2020-10-09	28.47	28.48	28.31	28.48	13206	{}	\N	2020-10-12 14:29:53.350394
32636	CHSCO	2020-10-12	28.4	28.4	28.2	28.2	2273	{}	\N	2020-10-12 14:29:53.354248
32637	CHW	2020-09-28	7.96	8.01	7.92	7.98	186710	{}	\N	2020-10-12 14:29:54.320473
32638	CHW	2020-09-29	7.98	8.06	7.98	8.04	119666	{}	\N	2020-10-12 14:29:54.330925
32639	CHW	2020-09-30	8.06	8.17	8.03	8.13	89617	{}	\N	2020-10-12 14:29:54.337858
32640	CHW	2020-10-01	8.14	8.17	8.06	8.08	179384	{}	\N	2020-10-12 14:29:54.344482
32641	CHW	2020-10-02	8.05	8.07	8	8.06	149075	{}	\N	2020-10-12 14:29:54.349565
32642	CHW	2020-10-05	8.09	8.22	8.05	8.22	329328	{}	\N	2020-10-12 14:29:54.3544
32643	CHW	2020-10-06	8.25	8.44	8.2364	8.39	688166	{}	\N	2020-10-12 14:29:54.358921
32644	CHW	2020-10-07	8.45	8.65	8.4301	8.65	438040	{}	\N	2020-10-12 14:29:54.363253
32645	CHW	2020-10-08	8.7	8.75	8.6601	8.75	270854	{}	\N	2020-10-12 14:29:54.367275
32646	CHW	2020-10-09	8.78	8.85	8.75	8.79	251709	{}	\N	2020-10-12 14:29:54.371339
32647	CHW	2020-10-12	8.85	8.8799	8.85	8.875	81766	{}	\N	2020-10-12 14:29:54.375361
32648	CDC	2020-09-28	47.41	47.58	47.23	47.32	26842	{}	\N	2020-10-12 14:29:59.086502
32649	CDC	2020-09-29	47.3092	47.35	46.901	47.13	7161	{}	\N	2020-10-12 14:29:59.091029
32650	CDC	2020-09-30	47.13	47.6327	47.13	47.43	26487	{}	\N	2020-10-12 14:29:59.095187
32651	CDC	2020-10-01	47.42	47.5303	47.0599	47.3	14958	{}	\N	2020-10-12 14:29:59.099105
32652	CDC	2020-10-02	46.68	47.86	46.68	47.76	19339	{}	\N	2020-10-12 14:29:59.103209
32653	CDC	2020-10-05	48	48.4499	48	48.4	16345	{}	\N	2020-10-12 14:29:59.107884
32654	CDC	2020-10-06	48.64	49.11	48.1733	48.32	15742	{}	\N	2020-10-12 14:29:59.112569
32655	CDC	2020-10-07	48.65	48.8299	48.5	48.75	16816	{}	\N	2020-10-12 14:29:59.11664
32656	CDC	2020-10-08	48.87	49.4	48.87	49.4	28545	{}	\N	2020-10-12 14:29:59.120687
32657	CDC	2020-10-09	49.61	49.7	49.23	49.31	26453	{}	\N	2020-10-12 14:29:59.124867
32658	CDC	2020-10-12	49.4	49.445	49.375	49.445	1712	{}	\N	2020-10-12 14:29:59.128982
32659	CDR	2020-09-28	0.73	0.8022	0.722	0.7966	413699	{}	\N	2020-10-12 14:30:00.347309
32660	CDR	2020-09-29	0.7985	0.8326	0.7781	0.8026	2077274	{}	\N	2020-10-12 14:30:00.355301
32661	CDR	2020-09-30	0.8299	0.84	0.8	0.81	1041112	{}	\N	2020-10-12 14:30:00.36046
32662	CDR	2020-10-01	0.81	0.8398	0.8	0.8347	911072	{}	\N	2020-10-12 14:30:00.365215
32663	CDR	2020-10-02	0.82	0.8386	0.81	0.8224	1285113	{}	\N	2020-10-12 14:30:00.369856
32664	CDR	2020-10-05	0.811	0.85	0.811	0.84	714910	{}	\N	2020-10-12 14:30:00.374415
32665	CDR	2020-10-06	0.84	0.88	0.82	0.85	1244776	{}	\N	2020-10-12 14:30:00.378694
32666	CDR	2020-10-07	0.8574	0.88	0.8395	0.8727	452224	{}	\N	2020-10-12 14:30:00.382835
32667	CDR	2020-10-08	0.8731	0.9116	0.8619	0.9054	614578	{}	\N	2020-10-12 14:30:00.386702
32668	CDR	2020-10-09	0.93	0.93	0.8702	0.894	1192458	{}	\N	2020-10-12 14:30:00.391066
32669	CDR	2020-10-12	0.9023	0.9045	0.8844	0.8901	76788	{}	\N	2020-10-12 14:30:00.395112
32670	CELP	2020-09-28	2.21	2.31	2.21	2.24	31683	{}	\N	2020-10-12 14:30:03.038296
32671	CELP	2020-09-29	2.197	2.23	2.1801	2.2	54750	{}	\N	2020-10-12 14:30:03.047598
32672	CELP	2020-09-30	2.18	2.24	2.11	2.12	121476	{}	\N	2020-10-12 14:30:03.053469
32673	CELP	2020-10-01	2.13	2.18	2.06	2.14	47684	{}	\N	2020-10-12 14:30:03.058482
32674	CELP	2020-10-02	2.06	2.14	2.06	2.1331	24561	{}	\N	2020-10-12 14:30:03.063133
32675	CELP	2020-10-05	2.14	2.2	2.09	2.105	62362	{}	\N	2020-10-12 14:30:03.067283
32676	CELP	2020-10-06	2.1801	2.22	2.1335	2.15	17860	{}	\N	2020-10-12 14:30:03.070778
32677	CELP	2020-10-07	2.11	2.165	2.08	2.1	39630	{}	\N	2020-10-12 14:30:03.074341
32678	CELP	2020-10-08	2.12	2.19	2.1031	2.16	31566	{}	\N	2020-10-12 14:30:03.077943
32679	CELP	2020-10-09	2.22	2.49	2.193	2.3	333616	{}	\N	2020-10-12 14:30:03.081897
32680	CELP	2020-10-12	2.25	2.285	2.23	2.2505	30230	{}	\N	2020-10-12 14:30:03.085388
32681	CEO	2020-09-28	97.44	97.8	96.73	97.15	107159	{}	\N	2020-10-12 14:30:04.302957
32682	CEO	2020-09-29	97.16	97.16	95.02	95.82	218372	{}	\N	2020-10-12 14:30:04.312274
32683	CEO	2020-09-30	96.79	97.35	95.74	96.05	215567	{}	\N	2020-10-12 14:30:04.319813
32684	CEO	2020-10-01	96.45	96.45	93.71	94.71	291335	{}	\N	2020-10-12 14:30:04.325828
32685	CEO	2020-10-02	93	94.68	93	94.13	188539	{}	\N	2020-10-12 14:30:04.331048
32686	CEO	2020-10-05	95.08	96.6	94.72	96.56	182789	{}	\N	2020-10-12 14:30:04.338739
32687	CEO	2020-10-06	98.06	98.68	96.39	97.13	288899	{}	\N	2020-10-12 14:30:04.344881
32688	CEO	2020-10-07	99.09	99.09	97.94	98.68	490582	{}	\N	2020-10-12 14:30:04.349397
32689	CEO	2020-10-08	99.05	99.94	98.38	99.87	586074	{}	\N	2020-10-12 14:30:04.353765
32690	CEO	2020-10-09	99.25	99.26	97.72	97.95	441625	{}	\N	2020-10-12 14:30:04.358175
32691	CEO	2020-10-12	98.5	98.5	97.84	97.995	38520	{}	\N	2020-10-12 14:30:04.3623
32692	CEW	2020-09-28	17.2713	17.279	17.17	17.171	6696	{}	\N	2020-10-12 14:30:05.798238
32693	CEW	2020-09-29	17.28	17.28	17.21	17.2393	5303	{}	\N	2020-10-12 14:30:05.806718
32694	CEW	2020-09-30	17.3	17.3599	17.3	17.3	1926	{}	\N	2020-10-12 14:30:05.814076
32695	CEW	2020-10-01	17.3767	17.3767	17.3767	17.3767	27	{}	\N	2020-10-12 14:30:05.819184
32696	CEW	2020-10-02	17.349	17.35	17.3343	17.3343	1406	{}	\N	2020-10-12 14:30:05.824102
32697	CEW	2020-10-05	17.44	17.45	17.4342	17.45	1026	{}	\N	2020-10-12 14:30:05.828479
32698	CEW	2020-10-06	17.41	17.4351	17.3684	17.3684	1326	{}	\N	2020-10-12 14:30:05.832725
32699	CEW	2020-10-07	17.4	17.4	17.3847	17.3847	445	{}	\N	2020-10-12 14:30:05.83724
32700	CEW	2020-10-08	17.3901	17.41	17.3901	17.41	163	{}	\N	2020-10-12 14:30:05.841453
32701	CEW	2020-10-09	17.535	17.535	17.535	17.535	282	{}	\N	2020-10-12 14:30:05.845714
32702	CFG	2020-09-28	24.96	25.54	24.91	25.34	3973690	{}	\N	2020-10-12 14:30:06.842793
32703	CFG	2020-09-29	25.22	25.36	24.49	24.78	5659579	{}	\N	2020-10-12 14:30:06.851975
32704	CFG	2020-09-30	24.92	25.72	24.9	25.28	9582639	{}	\N	2020-10-12 14:30:06.858579
32705	CFG	2020-10-01	25.46	25.71	25.14	25.43	6604861	{}	\N	2020-10-12 14:30:06.86428
32706	CFG	2020-10-02	24.9	26.36	24.8	26.21	6249496	{}	\N	2020-10-12 14:30:06.869208
32707	CFG	2020-10-05	26.52	27.5	26.45	27.48	9213361	{}	\N	2020-10-12 14:30:06.874324
32708	CFG	2020-10-06	28	28.48	27.02	27.13	9161789	{}	\N	2020-10-12 14:30:06.878843
32709	CFG	2020-10-07	27.63	28.46	27.59	28.12	5713891	{}	\N	2020-10-12 14:30:06.882993
32710	CFG	2020-10-08	28.41	28.63	27.96	28.56	4858400	{}	\N	2020-10-12 14:30:06.887024
32711	CFG	2020-10-09	28.75	28.87	27.87	27.96	8080277	{}	\N	2020-10-12 14:30:06.891163
32712	CFG	2020-10-12	28	28.32	27.82	28.29	1542317	{}	\N	2020-10-12 14:30:06.895536
32713	CGNX	2020-09-28	62.75	63.91	62.75	63.47	533200	{}	\N	2020-10-12 14:30:07.873709
32714	CGNX	2020-09-29	63.6	65.36	63.52	64.96	695519	{}	\N	2020-10-12 14:30:07.885089
32715	CGNX	2020-09-30	65.05	65.96	64.51	65.06	476660	{}	\N	2020-10-12 14:30:07.891871
32716	CGNX	2020-10-01	65.83	66.23	65.08	65.56	605953	{}	\N	2020-10-12 14:30:07.89775
32717	CGNX	2020-10-02	63.82	65.44	63.1	64.41	597441	{}	\N	2020-10-12 14:30:07.903431
32718	CGNX	2020-10-05	65.13	67.25	65.13	66.87	637646	{}	\N	2020-10-12 14:30:07.908838
32719	CGNX	2020-10-06	66.8	67.3612	65.7875	66.14	700540	{}	\N	2020-10-12 14:30:07.913138
32720	CGNX	2020-10-07	67.18	68.88	67.0177	68.16	750172	{}	\N	2020-10-12 14:30:07.917134
32721	CGNX	2020-10-08	68.35	68.88	67.88	68.38	494669	{}	\N	2020-10-12 14:30:07.921374
32722	CGNX	2020-10-09	69	69.74	68.52	69.31	578740	{}	\N	2020-10-12 14:30:07.925331
32723	CGNX	2020-10-12	69.63	70.55	69.63	70.12	164587	{}	\N	2020-10-12 14:30:07.930033
32724	CHCT	2020-09-28	45.88	47.47	45.78	46.8	105406	{}	\N	2020-10-12 14:30:08.899849
32725	CHCT	2020-09-29	46.9	47.08	45.98	46.61	157037	{}	\N	2020-10-12 14:30:08.907381
32726	CHCT	2020-09-30	46.98	47.86	46.03	46.76	299497	{}	\N	2020-10-12 14:30:08.913266
32727	CHCT	2020-10-01	47.25	49.13	47.25	49.13	256248	{}	\N	2020-10-12 14:30:08.919765
32728	CHCT	2020-10-02	48.37	50.98	48.37	50.79	302748	{}	\N	2020-10-12 14:30:08.926406
32729	CHCT	2020-10-05	51.02	51.3535	49.51	50.47	149145	{}	\N	2020-10-12 14:30:08.931692
32730	CHCT	2020-10-06	50.96	51.23	49.9	50.41	210819	{}	\N	2020-10-12 14:30:08.936724
32731	CHCT	2020-10-07	50.65	50.72	48.91	49	198539	{}	\N	2020-10-12 14:30:08.941807
32732	CHCT	2020-10-08	48.98	49.95	48.98	49.92	157436	{}	\N	2020-10-12 14:30:08.946751
32733	CHCT	2020-10-09	50.68	51.06	50.28	50.73	149536	{}	\N	2020-10-12 14:30:08.951209
32734	CHCT	2020-10-12	50.8	51.04	50.47	51.04	6304	{}	\N	2020-10-12 14:30:08.955508
32735	CHII	2020-09-28	13.31	13.31	13.31	13.31	113	{}	\N	2020-10-12 14:30:11.239335
32736	CHII	2020-09-29	13.31	13.31	13.31	13.31	55	{}	\N	2020-10-12 14:30:11.246515
32737	CHII	2020-09-30	13.305	13.305	13.305	13.305	177	{}	\N	2020-10-12 14:30:11.25224
32738	CHII	2020-10-01	13.5355	13.5355	13.5355	13.5355	49	{}	\N	2020-10-12 14:30:11.25738
32739	CHII	2020-10-02	13.4251	13.4251	13.4251	13.4251	209	{}	\N	2020-10-12 14:30:11.262315
32740	CHII	2020-10-05	13.386	13.386	13.386	13.386	21	{}	\N	2020-10-12 14:30:11.26643
32741	CHII	2020-10-06	13.54	13.54	13.54	13.54	244	{}	\N	2020-10-12 14:30:11.270511
32742	CHII	2020-10-07	13.5401	13.5401	13.5401	13.5401	122	{}	\N	2020-10-12 14:30:11.274423
32743	CHII	2020-10-08	13.62	13.62	13.59	13.59	592	{}	\N	2020-10-12 14:30:11.278605
32744	CHMA	2020-09-28	4.5	4.54	4.36	4.4	283112	{}	\N	2020-10-12 14:30:12.518783
32745	CHMA	2020-09-29	4.39	4.52	4.33	4.35	375279	{}	\N	2020-10-12 14:30:12.528771
32746	CHMA	2020-09-30	4.37	4.4039	4.24	4.3	1003782	{}	\N	2020-10-12 14:30:12.534985
32747	CHMA	2020-10-01	4.33	4.33	4.12	4.17	503690	{}	\N	2020-10-12 14:30:12.542572
32748	CHMA	2020-10-02	4.12	4.23	4.06	4.09	292766	{}	\N	2020-10-12 14:30:12.548369
32749	CHMA	2020-10-05	4.13	4.2	4.04	4.14	676646	{}	\N	2020-10-12 14:30:12.552895
32750	CHMA	2020-10-06	4.21	4.3	4.01	4.14	1132785	{}	\N	2020-10-12 14:30:12.557265
32751	CHMA	2020-10-07	4.18	4.21	4.033	4.14	1556738	{}	\N	2020-10-12 14:30:12.562581
32752	CHMA	2020-10-08	4.18	4.2685	4.01	4.06	861919	{}	\N	2020-10-12 14:30:12.567983
32753	CHMA	2020-10-09	4.13	4.1525	3.9	3.91	1296346	{}	\N	2020-10-12 14:30:12.573508
32754	CHMA	2020-10-12	3.97	4.01	3.88	3.93	74691	{}	\N	2020-10-12 14:30:12.578486
32755	CEA	2020-09-28	20.95	21.68	20.95	21.59	29090	{}	\N	2020-10-12 14:30:13.529229
32756	CEA	2020-09-29	21.35	21.3789	21.14	21.28	27976	{}	\N	2020-10-12 14:30:13.536719
32757	CEA	2020-09-30	21.5278	21.86	21.51	21.6	38894	{}	\N	2020-10-12 14:30:13.542242
32758	CEA	2020-10-01	21.94	22.01	21.71	21.91	20850	{}	\N	2020-10-12 14:30:13.547417
32759	CEA	2020-10-02	21.65	22	21.51	21.64	16867	{}	\N	2020-10-12 14:30:13.552275
32760	CEA	2020-10-05	21.68	21.68	21.4401	21.57	18036	{}	\N	2020-10-12 14:30:13.556591
32761	CEA	2020-10-06	21.9	22.06	21.6102	21.69	13810	{}	\N	2020-10-12 14:30:13.560519
32762	CEA	2020-10-07	21.96	22.1173	21.88	21.98	10832	{}	\N	2020-10-12 14:30:13.564647
32763	CEA	2020-10-08	22.12	22.71	21.94	22.71	44161	{}	\N	2020-10-12 14:30:13.568388
32764	CEA	2020-10-09	21.98	22.28	21.98	22.07	22526	{}	\N	2020-10-12 14:30:13.571804
32765	CEA	2020-10-12	21.92	21.94	21.75	21.94	7560	{}	\N	2020-10-12 14:30:13.575415
32766	CELH	2020-09-28	20.15	20.8799	19.94	20.25	553223	{}	\N	2020-10-12 14:30:14.481423
32767	CELH	2020-09-29	20.39	20.91	19.855	20.83	624120	{}	\N	2020-10-12 14:30:14.49198
32768	CELH	2020-09-30	20.83	22.8	20.77	22.7	956057	{}	\N	2020-10-12 14:30:14.498298
32769	CELH	2020-10-01	23.02	24.34	19.8	21.93	4535892	{}	\N	2020-10-12 14:30:14.504537
32770	CELH	2020-10-02	21.2	23.73	20.81	23.03	1203169	{}	\N	2020-10-12 14:30:14.510733
32771	CELH	2020-10-05	23.03	23.3621	21.95	22.62	822938	{}	\N	2020-10-12 14:30:14.516115
32772	CELH	2020-10-06	22.46	22.6387	20.6	21.21	1162674	{}	\N	2020-10-12 14:30:14.520164
32773	CELH	2020-10-07	21.32	22.27	20.84	22.02	954011	{}	\N	2020-10-12 14:30:14.524435
32774	CELH	2020-10-08	22.32	22.86	20.19	20.48	1167830	{}	\N	2020-10-12 14:30:14.528652
32775	CELH	2020-10-09	20.655	22.5499	19.51	21.97	2258853	{}	\N	2020-10-12 14:30:14.532544
32776	CELH	2020-10-12	22.14	22.55	20.74	21.1399	220110	{}	\N	2020-10-12 14:30:14.536586
32777	CEQP	2020-09-28	12.02	12.48	12.02	12.44	446127	{}	\N	2020-10-12 14:30:15.832785
32778	CEQP	2020-09-29	12.3	12.56	12.13	12.39	783767	{}	\N	2020-10-12 14:30:15.841738
32779	CEQP	2020-09-30	12.5	12.99	12.27	12.46	2090640	{}	\N	2020-10-12 14:30:15.847801
32780	CEQP	2020-10-01	12.27	12.54	11.83	11.99	1752086	{}	\N	2020-10-12 14:30:15.8533
32781	CEQP	2020-10-02	11.5	12.37	11.43	12	1242322	{}	\N	2020-10-12 14:30:15.858601
32782	CEQP	2020-10-05	12.35	12.62	12.11	12.59	832740	{}	\N	2020-10-12 14:30:15.863184
32783	CEQP	2020-10-06	12.85	13.55	12.79	12.89	1231261	{}	\N	2020-10-12 14:30:15.867523
32784	CEQP	2020-10-07	13.04	13.32	12.874	13.21	819277	{}	\N	2020-10-12 14:30:15.871684
32785	CEQP	2020-10-08	13.34	14.6	13.21	14.43	2070228	{}	\N	2020-10-12 14:30:15.876703
32786	CEQP	2020-10-09	14.57	14.9	14.23	14.6	1419400	{}	\N	2020-10-12 14:30:15.880624
32787	CEQP	2020-10-12	14.65	14.65	14.36	14.48	93362	{}	\N	2020-10-12 14:30:15.88516
32788	CEVA	2020-09-28	38.33	39.145	37.97	39.07	76234	{}	\N	2020-10-12 14:30:17.081535
32789	CEVA	2020-09-29	39.1	39.81	39.03	39.65	86574	{}	\N	2020-10-12 14:30:17.090349
32790	CEVA	2020-09-30	39.25	39.84	39.15	39.37	100742	{}	\N	2020-10-12 14:30:17.096617
32791	CEVA	2020-10-01	40.06	40.8	39.56	40.41	118953	{}	\N	2020-10-12 14:30:17.102731
32792	CEVA	2020-10-02	39.49	39.92	38.73	39.05	101868	{}	\N	2020-10-12 14:30:17.108014
32793	CEVA	2020-10-05	39.3	40.185	39.22	40.03	82354	{}	\N	2020-10-12 14:30:17.113206
32794	CEVA	2020-10-06	40.19	41.325	39.81	40.26	129761	{}	\N	2020-10-12 14:30:17.117906
32795	CEVA	2020-10-07	40.64	41.295	40.21	40.79	165478	{}	\N	2020-10-12 14:30:17.122188
32796	CEVA	2020-10-08	41.12	41.77	41	41.77	95391	{}	\N	2020-10-12 14:30:17.126299
32797	CEVA	2020-10-09	42.35	44.83	42.255	44.81	214950	{}	\N	2020-10-12 14:30:17.130432
32798	CEVA	2020-10-12	45	45.79	44.8201	45.58	18446	{}	\N	2020-10-12 14:30:17.134756
32799	CFFN	2020-09-28	9.13	9.34	9.13	9.29	769970	{}	\N	2020-10-12 14:30:18.47551
32800	CFFN	2020-09-29	9.23	9.26	9	9.16	622497	{}	\N	2020-10-12 14:30:18.488045
32801	CFFN	2020-09-30	9.21	9.38	9.2	9.265	569033	{}	\N	2020-10-12 14:30:18.495981
32802	CFFN	2020-10-01	9.295	9.36	9.2	9.33	458522	{}	\N	2020-10-12 14:30:18.501798
32803	CFFN	2020-10-02	10.19	11.06	10.11	10.59	8016453	{}	\N	2020-10-12 14:30:18.506213
32804	CFFN	2020-10-05	10.4	10.92	10.181	10.85	3719567	{}	\N	2020-10-12 14:30:18.510777
32805	CFFN	2020-10-06	10.88	10.9	10.41	10.63	26159642	{}	\N	2020-10-12 14:30:18.514719
32806	CFFN	2020-10-07	10.68	10.95	10.61	10.93	1753479	{}	\N	2020-10-12 14:30:18.518814
32807	CFFN	2020-10-08	10.99	11.32	10.93	11.245	2240671	{}	\N	2020-10-12 14:30:18.522615
32808	CFFN	2020-10-09	11.25	11.39	11.13	11.245	1076706	{}	\N	2020-10-12 14:30:18.52705
32809	CFFN	2020-10-12	11.19	11.41	11.15	11.39	112548	{}	\N	2020-10-12 14:30:18.530822
32810	CGEN	2020-09-28	16.34	16.5867	15.84	16.1	853176	{}	\N	2020-10-12 14:30:19.45626
32811	CGEN	2020-09-29	16.07	16.57	16.07	16.32	832615	{}	\N	2020-10-12 14:30:19.466894
32812	CGEN	2020-09-30	16.38	16.58	16.13	16.25	657629	{}	\N	2020-10-12 14:30:19.474516
32813	CGEN	2020-10-01	16.35	17.13	16.35	17.11	617565	{}	\N	2020-10-12 14:30:19.480962
32814	CGEN	2020-10-02	15.17	16.295	15.04	15.185	1426758	{}	\N	2020-10-12 14:30:19.485821
32815	CGEN	2020-10-05	15.35	16.14	15.31	15.97	878361	{}	\N	2020-10-12 14:30:19.493412
32816	CGEN	2020-10-06	15.79	16.62	15.79	15.96	777200	{}	\N	2020-10-12 14:30:19.499342
32817	CGEN	2020-10-07	15.99	16.98	15.99	16.665	876190	{}	\N	2020-10-12 14:30:19.504167
32818	CGEN	2020-10-08	17.14	17.86	16.78	16.93	843175	{}	\N	2020-10-12 14:30:19.508432
32819	CGEN	2020-10-09	17.15	17.72	17.03	17.53	629027	{}	\N	2020-10-12 14:30:19.514443
32820	CGEN	2020-10-12	17.59	17.79	17.27	17.69	207225	{}	\N	2020-10-12 14:30:19.519496
32821	CHEK	2020-09-28	0.343	0.35	0.331	0.35	393686	{}	\N	2020-10-12 14:30:20.449396
32822	CHEK	2020-09-29	0.3599	0.3599	0.3307	0.3499	387745	{}	\N	2020-10-12 14:30:20.462775
32823	CHEK	2020-09-30	0.351	0.3675	0.35	0.3605	577884	{}	\N	2020-10-12 14:30:20.470703
32824	CHEK	2020-10-01	0.369	0.369	0.3401	0.3596	563939	{}	\N	2020-10-12 14:30:20.477893
32825	CHEK	2020-10-02	0.3528	0.361	0.34	0.3599	457198	{}	\N	2020-10-12 14:30:20.483097
32826	CHEK	2020-10-05	0.3516	0.364	0.342	0.364	366532	{}	\N	2020-10-12 14:30:20.487477
32827	CHEK	2020-10-06	0.356201	0.365	0.352	0.359	507473	{}	\N	2020-10-12 14:30:20.491501
32828	CHEK	2020-10-07	0.36	0.3699	0.351	0.3699	406475	{}	\N	2020-10-12 14:30:20.496215
32829	CHEK	2020-10-08	0.3642	0.39	0.3589	0.3886	870574	{}	\N	2020-10-12 14:30:20.500702
32830	CHEK	2020-10-09	0.3886	0.39	0.3651	0.375	624307	{}	\N	2020-10-12 14:30:20.504909
32831	CHEK	2020-10-12	0.38	0.3838	0.375	0.378987	73656	{}	\N	2020-10-12 14:30:20.509175
32832	CHRS	2020-09-28	18.57	18.88	18.2	18.83	479292	{}	\N	2020-10-12 14:30:21.520891
32833	CHRS	2020-09-29	18.83	19.02	18.58	18.81	436638	{}	\N	2020-10-12 14:30:21.529494
32834	CHRS	2020-09-30	18.52	18.86	18.15	18.32	948375	{}	\N	2020-10-12 14:30:21.535419
32835	CHRS	2020-10-01	18.41	18.59	17.88	18.07	828659	{}	\N	2020-10-12 14:30:21.542026
32836	CHRS	2020-10-02	17.75	18.19	16.94	16.96	1153592	{}	\N	2020-10-12 14:30:21.547133
32837	CHRS	2020-10-05	16.925	17.46	16.74	17.29	881959	{}	\N	2020-10-12 14:30:21.551507
32838	CHRS	2020-10-06	17.39	17.73	17.01	17.49	1014554	{}	\N	2020-10-12 14:30:21.555492
32839	CHRS	2020-10-07	17.64	18.02	17.47	17.83	970792	{}	\N	2020-10-12 14:30:21.559636
32840	CHRS	2020-10-08	18.15	18.77	17.99	18.66	1203214	{}	\N	2020-10-12 14:30:21.563573
32841	CHRS	2020-10-09	18.56	18.9	18.56	18.61	842959	{}	\N	2020-10-12 14:30:21.567177
32842	CHRS	2020-10-12	18.65	18.84	18.46	18.6	101676	{}	\N	2020-10-12 14:30:21.571184
32843	CHSCP	2020-09-28	28.9	29.3158	28.8584	29.27	20186	{}	\N	2020-10-12 14:30:23.100254
32844	CHSCP	2020-09-29	29.17	29.29	29.083	29.25	7607	{}	\N	2020-10-12 14:30:23.11048
32845	CHSCP	2020-09-30	29.23	29.4	28.67	28.67	53396	{}	\N	2020-10-12 14:30:23.117439
32846	CHSCP	2020-10-01	28.89	28.9	28.53	28.88	20268	{}	\N	2020-10-12 14:30:23.123886
32847	CHSCP	2020-10-02	28.695	28.845	28.64	28.84	7550	{}	\N	2020-10-12 14:30:23.129189
32848	CHSCP	2020-10-05	28.71	29.14	28.15	28.875	20212	{}	\N	2020-10-12 14:30:23.134215
32849	CHSCP	2020-10-06	29	29.07	28.74	28.9	12586	{}	\N	2020-10-12 14:30:23.138843
32850	CHSCP	2020-10-07	28.73	29.25	28.73	28.89	9802	{}	\N	2020-10-12 14:30:23.143209
32851	CHSCP	2020-10-08	28.97	29.25	28.95	29.21	7673	{}	\N	2020-10-12 14:30:23.147541
32852	CHSCP	2020-10-09	29.3	29.44	29.13	29.26	35730	{}	\N	2020-10-12 14:30:23.151803
32853	CHSCP	2020-10-12	29.22	29.22	29.1633	29.1633	605	{}	\N	2020-10-12 14:30:23.156298
32854	CIA	2020-09-28	5.33	5.59	5.32	5.57	61503	{}	\N	2020-10-12 14:30:24.134155
32855	CIA	2020-09-29	5.53	5.63	5.49	5.63	54079	{}	\N	2020-10-12 14:30:24.143783
32856	CIA	2020-09-30	5.63	5.7	5.51	5.54	68874	{}	\N	2020-10-12 14:30:24.149623
32857	CIA	2020-10-01	5.53	5.66	5.47	5.61	124490	{}	\N	2020-10-12 14:30:24.155564
32858	CIA	2020-10-02	5.41	5.69	5.41	5.65	110295	{}	\N	2020-10-12 14:30:24.160576
32859	CIA	2020-10-05	5.67	5.9	5.67	5.85	148032	{}	\N	2020-10-12 14:30:24.165374
32860	CIA	2020-10-06	5.9	5.94	5.69	5.69	156417	{}	\N	2020-10-12 14:30:24.169227
32861	CIA	2020-10-07	5.71	5.94	5.71	5.89	136946	{}	\N	2020-10-12 14:30:24.172998
32862	CIA	2020-10-08	5.9	5.95	5.78	5.85	135715	{}	\N	2020-10-12 14:30:24.17697
32863	CIA	2020-10-09	5.92	5.92	5.8	5.86	44985	{}	\N	2020-10-12 14:30:24.180862
32864	CIA	2020-10-12	5.85	5.87	5.82	5.87	2476	{}	\N	2020-10-12 14:30:24.184558
32865	CIO	2020-09-28	7.51	7.83	7.5	7.7	297217	{}	\N	2020-10-12 14:30:25.107552
32866	CIO	2020-09-29	7.7	7.7	7.39	7.54	726604	{}	\N	2020-10-12 14:30:25.118631
32867	CIO	2020-09-30	7.52	7.695	7.4	7.52	645182	{}	\N	2020-10-12 14:30:25.124448
32868	CIO	2020-10-01	7.5	7.66	7.42	7.66	474726	{}	\N	2020-10-12 14:30:25.129741
32869	CIO	2020-10-02	7.51	7.83	7.45	7.77	364493	{}	\N	2020-10-12 14:30:25.134371
32870	CIO	2020-10-05	7.86	7.8998	7.66	7.69	515802	{}	\N	2020-10-12 14:30:25.139108
32871	CIO	2020-10-06	7.79	7.88	7.64	7.68	962304	{}	\N	2020-10-12 14:30:25.14334
32872	CIO	2020-10-07	7.57	7.61	7.3713	7.54	667768	{}	\N	2020-10-12 14:30:25.147211
32873	CIO	2020-10-08	7.57	7.76	7.4649	7.63	421414	{}	\N	2020-10-12 14:30:25.151144
32874	CIO	2020-10-09	7.61	7.6483	7.4	7.44	424755	{}	\N	2020-10-12 14:30:25.155213
32875	CIO	2020-10-12	7.41	7.44	7.27	7.35	86721	{}	\N	2020-10-12 14:30:25.159618
32876	CIZ	2020-09-28	27.85	27.943	27.85	27.943	1089	{}	\N	2020-10-12 14:30:26.114967
32877	CIZ	2020-09-29	27.96	27.96	27.92	27.93	5295	{}	\N	2020-10-12 14:30:26.120341
32878	CIZ	2020-09-30	27.85	27.98	27.825	27.825	2978	{}	\N	2020-10-12 14:30:26.125098
32879	CIZ	2020-10-01	28.01	28.0422	27.93	28.04	8079	{}	\N	2020-10-12 14:30:26.129111
32880	CIZ	2020-10-02	27.77	28.0255	27.77	27.98	4511	{}	\N	2020-10-12 14:30:26.133768
32881	CIZ	2020-10-05	28.21	28.3875	28.21	28.38	3121	{}	\N	2020-10-12 14:30:26.138046
32882	CIZ	2020-10-06	28.338	28.338	28.075	28.08	2810	{}	\N	2020-10-12 14:30:26.142323
32883	CIZ	2020-10-07	28.19	28.24	28.19	28.2399	694	{}	\N	2020-10-12 14:30:26.146476
32884	CIZ	2020-10-08	28.33	28.3897	28.33	28.3897	9910	{}	\N	2020-10-12 14:30:26.150688
32885	CIZ	2020-10-09	28.49	28.62	28.49	28.55	28034	{}	\N	2020-10-12 14:30:26.154732
32886	CIZ	2020-10-12	28.64	28.64	28.6217	28.6217	419	{}	\N	2020-10-12 14:30:26.158927
32887	CLAR	2020-09-28	14	14.55	14	14.23	132668	{}	\N	2020-10-12 14:30:27.036557
32888	CLAR	2020-09-29	14.33	14.4	14.06	14.175	60881	{}	\N	2020-10-12 14:30:27.046531
32889	CLAR	2020-09-30	14.11	14.36	14.02	14.14	58598	{}	\N	2020-10-12 14:30:27.054056
32890	CLAR	2020-10-01	14.14	14.55	13.99	14.47	85891	{}	\N	2020-10-12 14:30:27.060188
32891	CLAR	2020-10-02	14.17	14.75	14.0883	14.56	87193	{}	\N	2020-10-12 14:30:27.065729
32892	CLAR	2020-10-05	14.81	14.965	14.78	14.89	71820	{}	\N	2020-10-12 14:30:27.070658
32893	CLAR	2020-10-06	14.9	15.075	14.66	14.7	122639	{}	\N	2020-10-12 14:30:27.075157
32894	CLAR	2020-10-07	15.065	15.33	14.9412	15.17	103871	{}	\N	2020-10-12 14:30:27.07955
32895	CLAR	2020-10-08	15.35	15.73	14.97	15.16	66408	{}	\N	2020-10-12 14:30:27.08432
32896	CLAR	2020-10-09	15.3	15.4299	15	15.24	62286	{}	\N	2020-10-12 14:30:27.088932
32897	CLAR	2020-10-12	15.35	15.45	15.3035	15.4365	5422	{}	\N	2020-10-12 14:30:27.093473
32898	CLGX	2020-09-28	67.42	68.13	67.285	67.74	402683	{}	\N	2020-10-12 14:30:28.146602
32899	CLGX	2020-09-29	67.76	68.01	67.38	67.43	755394	{}	\N	2020-10-12 14:30:28.157545
32900	CLGX	2020-09-30	67.4	67.87	67.15	67.67	1036403	{}	\N	2020-10-12 14:30:28.165353
32901	CLGX	2020-10-01	67.51	67.89	67.46	67.71	942028	{}	\N	2020-10-12 14:30:28.171726
32902	CLGX	2020-10-02	67.24	67.82	67.12	67.56	579586	{}	\N	2020-10-12 14:30:28.177498
32903	CLGX	2020-10-05	67.6	68.005	67.6	67.98	1330655	{}	\N	2020-10-12 14:30:28.182624
32904	CLGX	2020-10-06	67.94	68.54	67.61	67.61	1485447	{}	\N	2020-10-12 14:30:28.187059
32905	CLGX	2020-10-07	67.95	68.57	67.94	67.96	1016566	{}	\N	2020-10-12 14:30:28.191662
32906	CLGX	2020-10-08	68.3	68.75	68.19	68.39	1164956	{}	\N	2020-10-12 14:30:28.196432
32907	CLGX	2020-10-09	68.55	69.02	68.43	68.98	1130405	{}	\N	2020-10-12 14:30:28.201227
32908	CLGX	2020-10-12	69.14	69.435	69.14	69.3	97726	{}	\N	2020-10-12 14:30:28.205838
32909	CDZI	2020-09-28	9.55	9.895	9.55	9.805	77523	{}	\N	2020-10-12 14:30:29.821198
32910	CDZI	2020-09-29	9.73	9.93	9.73	9.92	55746	{}	\N	2020-10-12 14:30:29.829821
32911	CDZI	2020-09-30	9.95	10.03	9.87	9.93	83235	{}	\N	2020-10-12 14:30:29.835771
32912	CDZI	2020-10-01	9.89	10.05	9.89	10.02	88419	{}	\N	2020-10-12 14:30:29.841583
32913	CDZI	2020-10-02	10	10.16	9.94	10.03	221275	{}	\N	2020-10-12 14:30:29.846616
32914	CDZI	2020-10-05	10.09	10.24	10.02	10.065	148386	{}	\N	2020-10-12 14:30:29.851486
32915	CDZI	2020-10-06	10.11	10.2	10.03	10.13	147493	{}	\N	2020-10-12 14:30:29.855542
32916	CDZI	2020-10-07	10.17	10.205	10.09	10.14	100558	{}	\N	2020-10-12 14:30:29.859424
32917	CDZI	2020-10-08	10.19	10.2	10.08	10.13	117568	{}	\N	2020-10-12 14:30:29.863268
32918	CDZI	2020-10-09	10.16	10.205	10.1	10.17	128462	{}	\N	2020-10-12 14:30:29.86696
32919	CDZI	2020-10-12	10.16	10.3	10.1492	10.24	14579	{}	\N	2020-10-12 14:30:29.870702
32920	CERC	2020-09-28	2.45	2.45	2.26	2.29	196820	{}	\N	2020-10-12 14:30:30.957876
32921	CERC	2020-09-29	2.27	2.36	2.26	2.27	343406	{}	\N	2020-10-12 14:30:30.966301
32922	CERC	2020-09-30	2.29	2.32	2.2	2.275	293388	{}	\N	2020-10-12 14:30:30.971755
32923	CERC	2020-10-01	2.2976	2.2976	2.18	2.24	249656	{}	\N	2020-10-12 14:30:30.977278
32924	CERC	2020-10-02	2.25	2.29	2.18	2.24	207573	{}	\N	2020-10-12 14:30:30.982703
32925	CERC	2020-10-05	2.26	2.35	2.24	2.28	216615	{}	\N	2020-10-12 14:30:30.987095
32926	CERC	2020-10-06	2.3309	2.43	2.285	2.33	166040	{}	\N	2020-10-12 14:30:30.99104
32927	CERC	2020-10-07	2.4	2.48	2.35	2.45	202545	{}	\N	2020-10-12 14:30:30.995041
32928	CERC	2020-10-08	2.47	2.517	2.4422	2.45	130940	{}	\N	2020-10-12 14:30:30.999139
32929	CERC	2020-10-09	2.48	2.5	2.43	2.45	100895	{}	\N	2020-10-12 14:30:31.003441
32930	CERC	2020-10-12	2.46	2.56	2.44	2.53	48186	{}	\N	2020-10-12 14:30:31.007359
32931	CFA	2020-09-28	54.04	54.3487	53.9239	54.21	29813	{}	\N	2020-10-12 14:30:31.891767
32932	CFA	2020-09-29	54.2	54.3	53.911	54.01	9144	{}	\N	2020-10-12 14:30:31.90141
32933	CFA	2020-09-30	54.04	54.75	54.0182	54.37	32353	{}	\N	2020-10-12 14:30:31.908471
32934	CFA	2020-10-01	54.52	54.81	54.36	54.54	15210	{}	\N	2020-10-12 14:30:31.914222
32935	CFA	2020-10-02	53.65	54.75	53.65	54.55	11103	{}	\N	2020-10-12 14:30:31.919396
32936	CFA	2020-10-05	54.91	55.58	54.824	55.54	106016	{}	\N	2020-10-12 14:30:31.924512
32937	CFA	2020-10-06	55.67	56.1099	55.09	55.15	11303	{}	\N	2020-10-12 14:30:31.928645
32938	CFA	2020-10-07	55.54	56.1	55.54	56.02	18093	{}	\N	2020-10-12 14:30:31.932801
32939	CFA	2020-10-08	56.33	56.6	56.25	56.6	11634	{}	\N	2020-10-12 14:30:31.937024
32940	CFA	2020-10-09	56.85	57.15	56.801	56.93	21201	{}	\N	2020-10-12 14:30:31.941325
32941	CFA	2020-10-12	57.185	57.215	57.185	57.215	388	{}	\N	2020-10-12 14:30:31.945572
32942	CFX	2020-09-28	31.3	31.96	31.22	31.39	596234	{}	\N	2020-10-12 14:30:32.903221
32943	CFX	2020-09-29	31.15	31.645	30.94	31.21	825375	{}	\N	2020-10-12 14:30:32.91203
32944	CFX	2020-09-30	31.52	32.01	31.025	31.36	1407667	{}	\N	2020-10-12 14:30:32.918528
32945	CFX	2020-10-01	31.57	32.09	31.11	31.5	968676	{}	\N	2020-10-12 14:30:32.923545
32946	CFX	2020-10-02	30.5	31.73	30.5	31.25	1824516	{}	\N	2020-10-12 14:30:32.927722
32947	CFX	2020-10-05	31.84	32.57	31.59	32.32	2158716	{}	\N	2020-10-12 14:30:32.932081
32948	CFX	2020-10-06	32.64	33.29	31.64	31.7	3664040	{}	\N	2020-10-12 14:30:32.936068
32949	CFX	2020-10-07	32.47	33.18	32.06	32.37	1686382	{}	\N	2020-10-12 14:30:32.939629
32950	CFX	2020-10-08	32.91	32.91	31.84	32.22	2248871	{}	\N	2020-10-12 14:30:32.943297
32951	CFX	2020-10-09	32.68	32.82	31.87	32.19	1548277	{}	\N	2020-10-12 14:30:32.946688
32952	CFX	2020-10-12	32.58	33	32.22	32.85	146982	{}	\N	2020-10-12 14:30:32.950059
32953	CGIX	2020-09-28	3.82	4.14	3.8	4.12	65621	{}	\N	2020-10-12 14:30:34.386986
32954	CGIX	2020-09-29	4.1	4.22	3.905	4.22	36104	{}	\N	2020-10-12 14:30:34.395227
32955	CGIX	2020-09-30	4.18	4.3	4.12	4.14	64084	{}	\N	2020-10-12 14:30:34.400643
32956	CGIX	2020-10-01	4.1	4.13	3.86	3.9	20365	{}	\N	2020-10-12 14:30:34.406063
32957	CGIX	2020-10-02	3.85	4.0299	3.85	3.97	24388	{}	\N	2020-10-12 14:30:34.410788
32958	CGIX	2020-10-05	3.97	4.09	3.919	3.98	21004	{}	\N	2020-10-12 14:30:34.415327
32959	CGIX	2020-10-06	3.98	4.17	3.93	3.99	67108	{}	\N	2020-10-12 14:30:34.419451
32960	CGIX	2020-10-07	3.99	3.99	3.8	3.88	37464	{}	\N	2020-10-12 14:30:34.423411
32961	CGIX	2020-10-08	3.9	4.2	3.81	4.15	215507	{}	\N	2020-10-12 14:30:34.426973
32962	CGIX	2020-10-09	4.01	4.6565	3.9	4.26	261621	{}	\N	2020-10-12 14:30:34.430533
32963	CGIX	2020-10-12	4.35	4.9	4.28	4.77	104353	{}	\N	2020-10-12 14:30:34.434318
32964	CHD	2020-09-28	93.62	94.4	93.21	93.46	1036384	{}	\N	2020-10-12 14:30:35.617906
32965	CHD	2020-09-29	93.74	94.1	92.265	92.85	2238903	{}	\N	2020-10-12 14:30:35.626453
32966	CHD	2020-09-30	93.32	94.16	92.74	93.71	2534531	{}	\N	2020-10-12 14:30:35.63286
32967	CHD	2020-10-01	94.01	94.03	92.59	93.1	1806627	{}	\N	2020-10-12 14:30:35.638774
32968	CHD	2020-10-02	93.16	93.31	91.06	91.17	2898395	{}	\N	2020-10-12 14:30:35.644167
32969	CHD	2020-10-05	91.02	91.92	90.23	90.73	3691515	{}	\N	2020-10-12 14:30:35.648609
32970	CHD	2020-10-06	90.76	91.225	89.39	89.85	2922759	{}	\N	2020-10-12 14:30:35.652765
32971	CHD	2020-10-07	90.49	91.6	90.363	90.93	4043671	{}	\N	2020-10-12 14:30:35.656908
32972	CHD	2020-10-08	90.82	91.98	90.51	91.14	1950286	{}	\N	2020-10-12 14:30:35.660917
32973	CHD	2020-10-09	91.46	93.46	91.34	92.76	2143494	{}	\N	2020-10-12 14:30:35.664941
32974	CHD	2020-10-12	92.69	93	92.14	92.48	468450	{}	\N	2020-10-12 14:30:35.669272
32975	CHI	2020-09-28	10.98	11.05	10.94	11.02	175243	{}	\N	2020-10-12 14:30:37.09341
32976	CHI	2020-09-29	10.99	11.06	10.96	11.01	145924	{}	\N	2020-10-12 14:30:37.105228
32977	CHI	2020-09-30	11.02	11.115	10.97	11	233829	{}	\N	2020-10-12 14:30:37.112162
32978	CHI	2020-10-01	11	11.205	11	11.16	204309	{}	\N	2020-10-12 14:30:37.118682
32979	CHI	2020-10-02	11.03	11.18	10.9701	11.15	163892	{}	\N	2020-10-12 14:30:37.123489
32980	CHI	2020-10-05	11.14	11.33	11.14	11.28	305059	{}	\N	2020-10-12 14:30:37.127907
32981	CHI	2020-10-06	11.28	11.47	11.25	11.34	258373	{}	\N	2020-10-12 14:30:37.131907
32982	CHI	2020-10-07	11.39	11.68	11.39	11.61	261401	{}	\N	2020-10-12 14:30:37.135496
32983	CHI	2020-10-08	11.65	11.7027	11.63	11.66	175330	{}	\N	2020-10-12 14:30:37.138956
32984	CHI	2020-10-09	11.69	11.88	11.69	11.85	210862	{}	\N	2020-10-12 14:30:37.142762
32985	CHI	2020-10-12	11.83	11.89	11.83	11.89	33842	{}	\N	2020-10-12 14:30:37.147813
32986	CHMI	2020-09-28	9.56	9.75	9.47	9.54	297970	{}	\N	2020-10-12 14:30:38.072586
32987	CHMI	2020-09-29	9.2	9.35	8.97	9.05	279041	{}	\N	2020-10-12 14:30:38.082494
32988	CHMI	2020-09-30	9	9.14	8.92	8.98	202214	{}	\N	2020-10-12 14:30:38.088294
32989	CHMI	2020-10-01	8.99	9.22	8.85	9.21	268180	{}	\N	2020-10-12 14:30:38.092697
32990	CHMI	2020-10-02	9.09	9.39	9.01	9.32	250065	{}	\N	2020-10-12 14:30:38.097365
32991	CHMI	2020-10-05	9.45	9.45	9.27	9.39	271645	{}	\N	2020-10-12 14:30:38.101706
32992	CHMI	2020-10-06	9.41	9.6501	9.35	9.36	317712	{}	\N	2020-10-12 14:30:38.105986
32993	CHMI	2020-10-07	9.47	9.47	9.23	9.41	201594	{}	\N	2020-10-12 14:30:38.109671
32994	CHMI	2020-10-08	9.46	9.7846	9.44	9.68	224983	{}	\N	2020-10-12 14:30:38.113194
32995	CHMI	2020-10-09	9.8	9.8	9.55	9.57	139599	{}	\N	2020-10-12 14:30:38.116759
32996	CHMI	2020-10-12	9.57	9.6	9.5	9.6	32342	{}	\N	2020-10-12 14:30:38.120197
32997	CID	2020-09-29	25.7901	25.7901	25.7901	25.7901	642	{}	\N	2020-10-12 14:30:39.492057
32998	CID	2020-09-30	25.815	25.885	25.815	25.85	2410	{}	\N	2020-10-12 14:30:39.500827
32999	CID	2020-10-01	25.9099	25.9099	25.9	25.9	209	{}	\N	2020-10-12 14:30:39.50678
33000	CID	2020-10-02	25.8057	25.9033	25.8057	25.9033	281	{}	\N	2020-10-12 14:30:39.512009
33001	CID	2020-10-06	26.435	26.437	26.245	26.245	1316	{}	\N	2020-10-12 14:30:39.517058
33002	CID	2020-10-07	26.2799	26.28	26.2799	26.28	1608	{}	\N	2020-10-12 14:30:39.521286
33003	CID	2020-10-08	26.4629	26.4629	26.44	26.44	193	{}	\N	2020-10-12 14:30:39.52556
33004	CID	2020-10-09	26.5299	26.616	26.49	26.51	1625	{}	\N	2020-10-12 14:30:39.529211
33005	CID	2020-10-12	26.6216	26.6216	26.6216	26.6216	163	{}	\N	2020-10-12 14:30:39.53272
33006	CIR	2020-09-28	27.16	28.89	27.08	27.64	97826	{}	\N	2020-10-12 14:30:40.725747
33007	CIR	2020-09-29	27.75	28	27.24	27.71	124573	{}	\N	2020-10-12 14:30:40.736206
33008	CIR	2020-09-30	28	28.68	27.22	27.35	246951	{}	\N	2020-10-12 14:30:40.743102
33009	CIR	2020-10-01	27.26	27.93	26.95	27.6	171465	{}	\N	2020-10-12 14:30:40.748698
33010	CIR	2020-10-02	26.8	28.55	26.8	28.12	366563	{}	\N	2020-10-12 14:30:40.753914
33011	CIR	2020-10-05	28.59	29.1936	28.02	28.19	106813	{}	\N	2020-10-12 14:30:40.758783
33012	CIR	2020-10-06	28.65	28.98	27.81	27.86	205185	{}	\N	2020-10-12 14:30:40.763104
33013	CIR	2020-10-07	28.41	29.505	28.23	29.18	213661	{}	\N	2020-10-12 14:30:40.767606
33014	CIR	2020-10-08	29.69	29.76	28.84	29.46	284467	{}	\N	2020-10-12 14:30:40.771874
33015	CIR	2020-10-09	29.95	30.17	29.1605	29.64	172652	{}	\N	2020-10-12 14:30:40.776366
33016	CIR	2020-10-12	29.64	29.97	29.23	29.97	8000	{}	\N	2020-10-12 14:30:40.780983
33017	CIZN	2020-09-28	20.22	20.7192	20.1	20.32	4675	{}	\N	2020-10-12 14:30:41.881244
33018	CIZN	2020-09-29	20.37	22.07	20.32	21.07	3338	{}	\N	2020-10-12 14:30:41.890213
33019	CIZN	2020-09-30	21.76	22.46	21.52	22.42	8335	{}	\N	2020-10-12 14:30:41.895708
33020	CIZN	2020-10-01	22.1	22.98	22.0001	22.6	8365	{}	\N	2020-10-12 14:30:41.901249
33021	CIZN	2020-10-02	22.4	22.61	21.77	21.77	2543	{}	\N	2020-10-12 14:30:41.906077
33022	CIZN	2020-10-05	21.27	22.99	20.54	21.7	9320	{}	\N	2020-10-12 14:30:41.910319
33023	CIZN	2020-10-06	21.02	22.24	20.62	20.86	9500	{}	\N	2020-10-12 14:30:41.914148
33024	CIZN	2020-10-07	21.25	21.54	21.23	21.54	5152	{}	\N	2020-10-12 14:30:41.918053
33025	CIZN	2020-10-08	21.6	21.67	21.19	21.19	2300	{}	\N	2020-10-12 14:30:41.922067
33026	CIZN	2020-10-09	22	22.23	22	22.23	2063	{}	\N	2020-10-12 14:30:41.925818
33027	CIZN	2020-10-12	22.23	22.23	22.23	22.23	238	{}	\N	2020-10-12 14:30:41.929563
33028	CLBS	2020-09-28	1.77	1.8	1.74	1.78	84848	{}	\N	2020-10-12 14:30:42.852232
33029	CLBS	2020-09-29	1.79	1.92	1.7628	1.88	316493	{}	\N	2020-10-12 14:30:42.861178
33030	CLBS	2020-09-30	1.88	1.9	1.78	1.8	132961	{}	\N	2020-10-12 14:30:42.867196
33031	CLBS	2020-10-01	1.82	1.86	1.77	1.84	74102	{}	\N	2020-10-12 14:30:42.872803
33032	CLBS	2020-10-02	1.76	1.85	1.76	1.84	126210	{}	\N	2020-10-12 14:30:42.878446
33033	CLBS	2020-10-05	1.83	1.91	1.8	1.87	125571	{}	\N	2020-10-12 14:30:42.883376
33034	CLBS	2020-10-06	1.89	1.95	1.835	1.85	146269	{}	\N	2020-10-12 14:30:42.887718
33035	CLBS	2020-10-07	1.86	1.94	1.86	1.86	149454	{}	\N	2020-10-12 14:30:42.892043
33036	CLBS	2020-10-08	1.87	1.9	1.8	1.8	283319	{}	\N	2020-10-12 14:30:42.896539
33037	CLBS	2020-10-09	1.78	1.84	1.78	1.8	128733	{}	\N	2020-10-12 14:30:42.901061
33038	CLBS	2020-10-12	1.82	1.83	1.76	1.79	84473	{}	\N	2020-10-12 14:30:42.905495
33039	CFBK	2020-09-28	11.92	12.071	11.92	11.99	5911	{}	\N	2020-10-12 14:30:44.329999
33040	CFBK	2020-09-29	12	12.03	11.96	11.96	2159	{}	\N	2020-10-12 14:30:44.34219
33041	CFBK	2020-09-30	12.08	12.1	12.08	12.08	1535	{}	\N	2020-10-12 14:30:44.347288
33042	CFBK	2020-10-01	12.1	12.1	11.69	12.0799	1041	{}	\N	2020-10-12 14:30:44.352078
33043	CFBK	2020-10-02	11.925	12.05	11.8	12.03	951	{}	\N	2020-10-12 14:30:44.390689
33044	CFBK	2020-10-05	12.0998	12.1	12.0998	12.1	1663	{}	\N	2020-10-12 14:30:44.398827
33045	CFBK	2020-10-06	12.26	12.5019	11.9264	12.26	38270	{}	\N	2020-10-12 14:30:44.405772
33046	CFBK	2020-10-07	12.23	12.45	12.22	12.36	3263	{}	\N	2020-10-12 14:30:44.41206
33047	CFBK	2020-10-08	12.48	12.555	12.25	12.5	5090	{}	\N	2020-10-12 14:30:44.416765
33048	CFBK	2020-10-09	12.3887	12.3887	12.3887	12.3887	351	{}	\N	2020-10-12 14:30:44.420946
33049	CG	2020-09-28	24.04	24.87	24.04	24.74	532659	{}	\N	2020-10-12 14:30:45.377131
33050	CG	2020-09-29	24.65	24.87	24.56	24.61	580181	{}	\N	2020-10-12 14:30:45.386806
33051	CG	2020-09-30	24.79	24.91	24.47	24.67	468589	{}	\N	2020-10-12 14:30:45.393094
33052	CG	2020-10-01	24.95	25.11	24.565	24.88	836347	{}	\N	2020-10-12 14:30:45.399157
33053	CG	2020-10-02	24.21	25.58	24.21	25.3	591492	{}	\N	2020-10-12 14:30:45.404233
33054	CG	2020-10-05	25.61	26.05	25.495	26.02	680035	{}	\N	2020-10-12 14:30:45.408884
33055	CG	2020-10-06	25.83	26.4	25.67	25.77	895143	{}	\N	2020-10-12 14:30:45.412826
33056	CG	2020-10-07	25.99	26.21	25.65	25.92	1131776	{}	\N	2020-10-12 14:30:45.416857
33057	CG	2020-10-08	26.33	26.71	26.09	26.7	879054	{}	\N	2020-10-12 14:30:45.42115
33058	CG	2020-10-09	26.78	26.89	26.45	26.8	746591	{}	\N	2020-10-12 14:30:45.425342
33059	CG	2020-10-12	27.4	27.479	26.7867	27.1	112279	{}	\N	2020-10-12 14:30:45.429236
33060	CHCI	2020-09-28	2.214	2.347	2.214	2.31	44867	{}	\N	2020-10-12 14:30:46.573758
33061	CHCI	2020-09-29	2.31	2.6424	2.31	2.47	95437	{}	\N	2020-10-12 14:30:46.583783
33062	CHCI	2020-09-30	2.41	2.52	2.41	2.46	13487	{}	\N	2020-10-12 14:30:46.589766
33063	CHCI	2020-10-01	2.41	2.537	2.32	2.48	20847	{}	\N	2020-10-12 14:30:46.595559
33064	CHCI	2020-10-02	2.33	2.505	2.33	2.43	13672	{}	\N	2020-10-12 14:30:46.600014
33065	CHCI	2020-10-05	2.48	2.74	2.38	2.55	67205	{}	\N	2020-10-12 14:30:46.604814
33066	CHCI	2020-10-06	2.58	3.8267	2.58	2.87	4601035	{}	\N	2020-10-12 14:30:46.609107
33067	CHCI	2020-10-07	2.84	2.87	2.56	2.67	299215	{}	\N	2020-10-12 14:30:46.613204
33068	CHCI	2020-10-08	2.5	2.6999	2.43	2.44	143827	{}	\N	2020-10-12 14:30:46.617423
33069	CHCI	2020-10-09	2.45	2.6	2.42	2.47	75166	{}	\N	2020-10-12 14:30:46.622231
33070	CHCI	2020-10-12	2.43	2.4724	2.43	2.4724	14704	{}	\N	2020-10-12 14:30:46.626844
33071	CHH	2020-09-28	86.88	88.92	86.88	88.32	266656	{}	\N	2020-10-12 14:30:47.891719
33072	CHH	2020-09-29	87.49	88.21	86.48	86.48	531280	{}	\N	2020-10-12 14:30:47.901075
33073	CHH	2020-09-30	86.94	88.2	85.595	85.96	465619	{}	\N	2020-10-12 14:30:47.908165
33074	CHH	2020-10-01	86.73	87.44	85.9	87.42	553721	{}	\N	2020-10-12 14:30:47.914059
33075	CHH	2020-10-02	85.13	87.94	85.13	87.71	422336	{}	\N	2020-10-12 14:30:47.919036
33076	CHH	2020-10-05	88.69	89.31	87.88	88.52	751032	{}	\N	2020-10-12 14:30:47.92401
33077	CHH	2020-10-06	89.53	91.66	88.99	89.17	942571	{}	\N	2020-10-12 14:30:47.927954
33078	CHH	2020-10-07	90.02	92.075	90.02	91.16	620696	{}	\N	2020-10-12 14:30:47.931961
33079	CHH	2020-10-08	91.77	91.9325	90.2	91.67	564571	{}	\N	2020-10-12 14:30:47.936035
33080	CHH	2020-10-09	92.18	93.2	91.15	91.27	490799	{}	\N	2020-10-12 14:30:47.940152
33081	CHH	2020-10-12	91.08	91.08	90.18	90.52	52323	{}	\N	2020-10-12 14:30:47.944463
33082	CHSCL	2020-09-28	27.87	28.25	27.87	28.25	25867	{}	\N	2020-10-12 14:30:49.764529
33083	CHSCL	2020-09-29	28.07	28.33	28.06	28.25	21374	{}	\N	2020-10-12 14:30:49.772282
33084	CHSCL	2020-09-30	28.1	28.32	27.88	27.88	77901	{}	\N	2020-10-12 14:30:49.777987
33085	CHSCL	2020-10-01	28.1	28.37	27.8334	28.37	18234	{}	\N	2020-10-12 14:30:49.783348
33086	CHSCL	2020-10-02	28.25	28.46	28.055	28.45	12121	{}	\N	2020-10-12 14:30:49.788142
33087	CHSCL	2020-10-05	28.48	28.7	28.1101	28.7	22452	{}	\N	2020-10-12 14:30:49.792621
33088	CHSCL	2020-10-06	28.7	28.8	28.48	28.8	26389	{}	\N	2020-10-12 14:30:49.796664
33089	CHSCL	2020-10-07	28.8	28.99	28.74	28.99	19989	{}	\N	2020-10-12 14:30:49.800557
33090	CHSCL	2020-10-08	28.77	28.9	28.63	28.87	36214	{}	\N	2020-10-12 14:30:49.804696
33091	CHSCL	2020-10-09	28.7281	28.8909	28.53	28.87	20418	{}	\N	2020-10-12 14:30:49.80874
33092	CHSCL	2020-10-12	28.66	28.88	28.47	28.52	10152	{}	\N	2020-10-12 14:30:49.812576
33093	CII	2020-09-28	15.25	15.36	15.22	15.31	114603	{}	\N	2020-10-12 14:30:51.343853
33094	CII	2020-09-29	15.26	15.33	15.23	15.27	288383	{}	\N	2020-10-12 14:30:51.351908
33095	CII	2020-09-30	15.36	15.49	15.27	15.46	305091	{}	\N	2020-10-12 14:30:51.358391
33096	CII	2020-10-01	15.56	15.57	15.45	15.51	228282	{}	\N	2020-10-12 14:30:51.366598
33097	CII	2020-10-02	15.43	15.5	15.35	15.45	274373	{}	\N	2020-10-12 14:30:51.373622
33098	CII	2020-10-05	15.58	15.68	15.523	15.68	159514	{}	\N	2020-10-12 14:30:51.385879
33099	CII	2020-10-06	15.71	15.72	15.52	15.57	263591	{}	\N	2020-10-12 14:30:51.395201
33100	CII	2020-10-07	15.59	15.7	15.59	15.67	381037	{}	\N	2020-10-12 14:30:51.40436
33101	CII	2020-10-08	15.7005	15.7393	15.64	15.68	280212	{}	\N	2020-10-12 14:30:51.411931
33102	CII	2020-10-09	15.76	15.99	15.76	15.86	264078	{}	\N	2020-10-12 14:30:51.41988
33103	CII	2020-10-12	16	16.03	15.98	16.02	66924	{}	\N	2020-10-12 14:30:51.427206
33104	CKH	2020-09-28	28.06	29.33	28.06	29.04	48473	{}	\N	2020-10-12 14:30:53.377017
33105	CKH	2020-09-29	29.19	29.26	28.43	28.68	120228	{}	\N	2020-10-12 14:30:53.387176
33106	CKH	2020-09-30	28.79	29.46	28.64	29.08	235615	{}	\N	2020-10-12 14:30:53.393315
33107	CKH	2020-10-01	29.09	29.44	28.56	29.29	131762	{}	\N	2020-10-12 14:30:53.398348
33108	CKH	2020-10-02	28.77	30.66	28.77	30.58	132416	{}	\N	2020-10-12 14:30:53.402726
33109	CKH	2020-10-05	30.88	31.31	30.26	30.54	150006	{}	\N	2020-10-12 14:30:53.406965
33110	CKH	2020-10-06	31.04	31.85	30.11	30.95	191782	{}	\N	2020-10-12 14:30:53.410645
33111	CKH	2020-10-07	31.09	31.09	29.61	30.38	226567	{}	\N	2020-10-12 14:30:53.413898
33112	CKH	2020-10-08	30.56	31.51	30.2	31.32	167072	{}	\N	2020-10-12 14:30:53.417329
33113	CKH	2020-10-09	31.6	32.31	31.32	32.11	86486	{}	\N	2020-10-12 14:30:53.420938
33114	CKH	2020-10-12	31.99	32.09	31.38	31.64	6051	{}	\N	2020-10-12 14:30:53.4242
33115	CLDR	2020-09-28	10.76	11.03	10.76	10.85	2893055	{}	\N	2020-10-12 14:30:54.323854
33116	CLDR	2020-09-29	10.92	11.08	10.85	10.95	4670595	{}	\N	2020-10-12 14:30:54.332362
33117	CLDR	2020-09-30	10.94	11.1516	10.78	10.89	5694256	{}	\N	2020-10-12 14:30:54.342109
33118	CLDR	2020-10-01	10.96	11.06	10.81	10.89	5688870	{}	\N	2020-10-12 14:30:54.347017
33119	CLDR	2020-10-02	10.64	10.8588	10.53	10.61	5176757	{}	\N	2020-10-12 14:30:54.351063
33120	CLDR	2020-10-05	10.79	10.9	10.59	10.72	7368819	{}	\N	2020-10-12 14:30:54.355664
33121	CLDR	2020-10-06	10.79	11.07	10.71	10.74	9730924	{}	\N	2020-10-12 14:30:54.359852
33122	CLDR	2020-10-07	10.82	10.98	10.71	10.93	7637922	{}	\N	2020-10-12 14:30:54.36405
33123	CLDR	2020-10-08	11.04	11.56	11.02	11.23	10952622	{}	\N	2020-10-12 14:30:54.367762
33124	CLDR	2020-10-09	11.28	11.57	11.23	11.43	6508094	{}	\N	2020-10-12 14:30:54.371627
33125	CLDR	2020-10-12	11.66	11.75	11.34	11.4685	2031273	{}	\N	2020-10-12 14:30:54.375669
33126	CLR	2020-09-28	13.08	13.12	12.61	12.82	3978926	{}	\N	2020-10-12 14:30:56.004
33127	CLR	2020-09-29	12.72	12.85	11.925	12.32	6920791	{}	\N	2020-10-12 14:30:56.013808
33128	CLR	2020-09-30	12.54	12.8148	12.19	12.28	7509620	{}	\N	2020-10-12 14:30:56.020214
33129	CLR	2020-10-01	12.08	12.17	11.465	11.76	7918991	{}	\N	2020-10-12 14:30:56.025284
33130	CLR	2020-10-02	11.12	12.38	11.09	12.3	7331762	{}	\N	2020-10-12 14:30:56.029872
33131	CLR	2020-10-05	12.62	13.29	12.59	13.24	9779764	{}	\N	2020-10-12 14:30:56.034539
33132	CLR	2020-10-06	13.55	13.7499	12.635	12.69	6965906	{}	\N	2020-10-12 14:30:56.038798
33133	CLR	2020-10-07	12.86	13.01	12.71	12.98	5162061	{}	\N	2020-10-12 14:30:56.043114
33134	CLR	2020-10-08	13.08	13.98	12.9356	13.93	6506947	{}	\N	2020-10-12 14:30:56.047089
33135	CLR	2020-10-09	14.09	14.09	13.305	13.45	5253676	{}	\N	2020-10-12 14:30:56.050854
33136	CLR	2020-10-12	13.3	13.34	12.9358	13.19	1091955	{}	\N	2020-10-12 14:30:56.05458
33137	CMCL	2020-09-28	17.07	17.94	17.07	17.61	65752	{}	\N	2020-10-12 14:30:56.926035
33138	CMCL	2020-09-29	17.74	17.96	17.15	17.81	30879	{}	\N	2020-10-12 14:30:56.936688
33139	CMCL	2020-09-30	17.42	17.75	16.45	16.99	77475	{}	\N	2020-10-12 14:30:56.944044
33140	CMCL	2020-10-01	17.2872	17.485	16.4	16.41	90764	{}	\N	2020-10-12 14:30:56.949526
33141	CMCL	2020-10-02	16.33	16.62	15.75	16.22	42004	{}	\N	2020-10-12 14:30:56.954271
33142	CMCL	2020-10-05	16.21	18.29	16.21	17.98	116869	{}	\N	2020-10-12 14:30:56.958985
33143	CMCL	2020-10-06	18.4	18.4	17.01	17.24	65991	{}	\N	2020-10-12 14:30:56.963342
33144	CMCL	2020-10-07	17.21	17.78	16.9745	17.78	55096	{}	\N	2020-10-12 14:30:56.967178
33145	CMCL	2020-10-08	17.78	17.91	17.2	17.62	30430	{}	\N	2020-10-12 14:30:56.970894
33146	CMCL	2020-10-09	18	19.25	17.8228	18.63	94241	{}	\N	2020-10-12 14:30:56.974803
33147	CMCL	2020-10-12	18.5	18.6899	18.3101	18.6	7240	{}	\N	2020-10-12 14:30:56.979276
33148	CNI	2020-09-28	107.27	107.45	106.47	106.58	614889	{}	\N	2020-10-12 14:30:59.365379
33149	CNI	2020-09-29	106.65	107.03	105.35	106.36	1432287	{}	\N	2020-10-12 14:30:59.37303
33150	CNI	2020-09-30	106.97	107.45	106.15	106.46	1432123	{}	\N	2020-10-12 14:30:59.379033
33151	CNI	2020-10-01	106.95	107.01	105.92	106.43	1271140	{}	\N	2020-10-12 14:30:59.384624
33152	CNI	2020-10-02	104.96	107.5	104.85	106.62	1561009	{}	\N	2020-10-12 14:30:59.38992
33153	CNI	2020-10-05	107.29	108.38	106.745	108.25	1006980	{}	\N	2020-10-12 14:30:59.394535
33154	CNI	2020-10-06	108.62	109.115	107.27	107.62	1398581	{}	\N	2020-10-12 14:30:59.398798
33155	CNI	2020-10-07	108.44	109.93	108.44	109.72	939165	{}	\N	2020-10-12 14:30:59.402949
33156	CNI	2020-10-08	110.32	110.38	109.21	109.41	843279	{}	\N	2020-10-12 14:30:59.406941
33157	CNI	2020-10-09	110	110.68	109.71	109.99	764558	{}	\N	2020-10-12 14:30:59.410934
33158	CNI	2020-10-12	110.45	111.6	110.44	110.565	145458	{}	\N	2020-10-12 14:30:59.414734
33159	CHAU	2020-09-28	29.67	29.8	29.49	29.75	130505	{}	\N	2020-10-12 14:31:00.891207
33160	CHAU	2020-09-29	29.76	30.23	29.7097	30.11	84946	{}	\N	2020-10-12 14:31:00.901139
33161	CHAU	2020-09-30	29.88	30.3	29.8	30.2	142061	{}	\N	2020-10-12 14:31:00.907865
33162	CHAU	2020-10-01	30.98	31.22	30.9601	31.19	113741	{}	\N	2020-10-12 14:31:00.913283
33163	CHAU	2020-10-02	30.12	30.7578	29.95	30.48	68491	{}	\N	2020-10-12 14:31:00.918475
33164	CHAU	2020-10-05	30.7	31.26	30.7	31.2	84671	{}	\N	2020-10-12 14:31:00.92338
33165	CHAU	2020-10-06	31.44	31.54	31.12	31.36	72599	{}	\N	2020-10-12 14:31:00.92725
33166	CHAU	2020-10-07	31.79	31.9573	31.58	31.81	78113	{}	\N	2020-10-12 14:31:00.930969
33167	CHAU	2020-10-08	31.9	32.02	31.78	31.94	75609	{}	\N	2020-10-12 14:31:00.93467
33168	CHAU	2020-10-09	32.3	32.55	32.3	32.47	175754	{}	\N	2020-10-12 14:31:00.938782
33169	CHAU	2020-10-12	33.67	33.69	33.55	33.64	44059	{}	\N	2020-10-12 14:31:00.942822
33170	CHFS	2020-09-28	0.2914	0.2935	0.2812	0.2885	2334581	{}	\N	2020-10-12 14:31:01.919514
33171	CHFS	2020-09-29	0.2882	0.3	0.2807	0.2999	3673125	{}	\N	2020-10-12 14:31:01.929862
33172	CHFS	2020-09-30	0.301	0.3028	0.2851	0.2915	4732953	{}	\N	2020-10-12 14:31:01.937012
33173	CHFS	2020-10-01	0.295	0.3876	0.2907	0.375	40687765	{}	\N	2020-10-12 14:31:01.943847
33174	CHFS	2020-10-02	0.34	0.3889	0.3297	0.3608	16268092	{}	\N	2020-10-12 14:31:01.948957
33175	CHFS	2020-10-05	0.385	0.419	0.3661	0.375	19079047	{}	\N	2020-10-12 14:31:01.953948
33176	CHFS	2020-10-06	0.3844	0.385	0.3356	0.3401	10943102	{}	\N	2020-10-12 14:31:01.958915
33177	CHFS	2020-10-07	0.3521	0.3537	0.34	0.3403	4842115	{}	\N	2020-10-12 14:31:01.963523
33178	CHFS	2020-10-08	0.3395	0.345	0.331	0.333	3233678	{}	\N	2020-10-12 14:31:01.968294
33179	CHFS	2020-10-09	0.34	0.3449	0.3301	0.339	3520849	{}	\N	2020-10-12 14:31:01.973072
33180	CHFS	2020-10-12	0.33	0.33	0.3121	0.3248	1256250	{}	\N	2020-10-12 14:31:01.978687
33181	CHS	2020-09-28	1.02	1.07	0.99	1.01	1257181	{}	\N	2020-10-12 14:31:03.506754
33182	CHS	2020-09-29	1.01	1.02	0.9661	0.98	3925366	{}	\N	2020-10-12 14:31:03.513754
33183	CHS	2020-09-30	1.05	1.05	0.96	0.9725	2971954	{}	\N	2020-10-12 14:31:03.519504
33184	CHS	2020-10-01	0.9732	0.9998	0.94	0.9691	2226994	{}	\N	2020-10-12 14:31:03.524488
33185	CHS	2020-10-02	0.931	1.08	0.9091	0.9965	6499511	{}	\N	2020-10-12 14:31:03.52913
33186	CHS	2020-10-05	1	1.075	0.9811	0.9901	2634559	{}	\N	2020-10-12 14:31:03.533863
33187	CHS	2020-10-06	1	1.04	0.92	0.92	6542656	{}	\N	2020-10-12 14:31:03.538216
33188	CHS	2020-10-07	0.9605	1	0.94	0.9751	3340500	{}	\N	2020-10-12 14:31:03.542497
33189	CHS	2020-10-08	0.99	1.04	0.97	1.04	2841858	{}	\N	2020-10-12 14:31:03.54637
33190	CHS	2020-10-09	1.04	1.17	1.04	1.17	5724106	{}	\N	2020-10-12 14:31:03.550165
33191	CHS	2020-10-12	1.13	1.26	1.1	1.2131	1824066	{}	\N	2020-10-12 14:31:03.554059
33192	CHU	2020-09-28	6.6	6.69	6.5905	6.65	290831	{}	\N	2020-10-12 14:31:04.593892
33193	CHU	2020-09-29	6.57	6.58	6.5	6.56	516550	{}	\N	2020-10-12 14:31:04.604
33194	CHU	2020-09-30	6.61	6.61	6.53	6.54	770014	{}	\N	2020-10-12 14:31:04.610998
33195	CHU	2020-10-01	6.63	6.66	6.6	6.63	272653	{}	\N	2020-10-12 14:31:04.616552
33196	CHU	2020-10-02	6.55	6.64	6.52	6.59	491995	{}	\N	2020-10-12 14:31:04.621253
33197	CHU	2020-10-05	6.59	6.65	6.58	6.65	447962	{}	\N	2020-10-12 14:31:04.625989
33198	CHU	2020-10-06	6.54	6.59	6.52	6.55	850859	{}	\N	2020-10-12 14:31:04.630079
33199	CHU	2020-10-07	6.64	6.74	6.63	6.73	683597	{}	\N	2020-10-12 14:31:04.63388
33200	CHU	2020-10-08	6.67	6.7	6.62	6.69	707527	{}	\N	2020-10-12 14:31:04.637882
33201	CHU	2020-10-09	6.54	6.6	6.5	6.57	742499	{}	\N	2020-10-12 14:31:04.641602
33202	CHU	2020-10-12	6.98	7.035	6.95	7.02	268534	{}	\N	2020-10-12 14:31:04.645267
33203	CIM	2020-09-28	8.59	8.675	8.53	8.63	4787283	{}	\N	2020-10-12 14:31:05.640882
33204	CIM	2020-09-29	8.63	8.665	8.285	8.31	5584889	{}	\N	2020-10-12 14:31:05.651773
33205	CIM	2020-09-30	8.3	8.47	8.17	8.2	5292575	{}	\N	2020-10-12 14:31:05.657389
33206	CIM	2020-10-01	8.2	8.25	8.09	8.23	5244531	{}	\N	2020-10-12 14:31:05.663107
33207	CIM	2020-10-02	8.07	8.49	8.05	8.48	5577954	{}	\N	2020-10-12 14:31:05.668376
33208	CIM	2020-10-05	8.5	8.67	8.475	8.66	5984368	{}	\N	2020-10-12 14:31:05.673049
33209	CIM	2020-10-06	8.7	8.89	8.6	8.61	7919262	{}	\N	2020-10-12 14:31:05.677102
33210	CIM	2020-10-07	8.64	8.685	8.5318	8.63	3855313	{}	\N	2020-10-12 14:31:05.681172
33211	CIM	2020-10-08	8.69	8.93	8.655	8.91	5167847	{}	\N	2020-10-12 14:31:05.685196
33212	CIM	2020-10-09	8.93	8.945	8.73	8.73	3971811	{}	\N	2020-10-12 14:31:05.689176
33213	CIM	2020-10-12	8.76	8.78	8.715	8.76	565696	{}	\N	2020-10-12 14:31:05.693617
33214	CIT	2020-09-28	16.77	17.97	16.76	17.78	2017932	{}	\N	2020-10-12 14:31:06.791158
33215	CIT	2020-09-29	17.54	17.59	16.905	17.24	2866792	{}	\N	2020-10-12 14:31:06.803543
33216	CIT	2020-09-30	17.37	17.96	17.2303	17.71	4872831	{}	\N	2020-10-12 14:31:06.810802
33217	CIT	2020-10-01	17.78	18.78	17.67	18.72	5101110	{}	\N	2020-10-12 14:31:06.817086
33218	CIT	2020-10-02	18.06	19.385	17.94	19.2	4399065	{}	\N	2020-10-12 14:31:06.822124
33219	CIT	2020-10-05	19.49	20.07	19.32	19.74	3630109	{}	\N	2020-10-12 14:31:06.827043
33220	CIT	2020-10-06	20.09	20.26	18.91	19.1	5609777	{}	\N	2020-10-12 14:31:06.83095
33221	CIT	2020-10-07	19.55	20.26	19.55	20.05	2794204	{}	\N	2020-10-12 14:31:06.834848
33222	CIT	2020-10-08	20.26	20.53	19.94	20.48	2500187	{}	\N	2020-10-12 14:31:06.839046
33223	CIT	2020-10-09	20.73	20.85	20.13	20.21	2209338	{}	\N	2020-10-12 14:31:06.843605
33224	CIT	2020-10-12	20.04	20.5	19.96	20.46	356438	{}	\N	2020-10-12 14:31:06.847841
33225	CLI	2020-09-28	12.98	13.405	12.75	13.02	885542	{}	\N	2020-10-12 14:31:08.980027
33226	CLI	2020-09-29	13.02	13.02	12.45	12.75	1179259	{}	\N	2020-10-12 14:31:08.989206
33227	CLI	2020-09-30	12.72	13.09	12.48	12.62	1446469	{}	\N	2020-10-12 14:31:08.994884
33228	CLI	2020-10-01	12.17	12.88	12.17	12.86	2351768	{}	\N	2020-10-12 14:31:09.000998
33229	CLI	2020-10-02	12.57	13.24	12.57	13.13	1664844	{}	\N	2020-10-12 14:31:09.005955
33230	CLI	2020-10-05	13.23	13.33	12.745	12.78	1375940	{}	\N	2020-10-12 14:31:09.010492
33231	CLI	2020-10-06	12.82	13.24	12.75	12.8	2162430	{}	\N	2020-10-12 14:31:09.014749
33232	CLI	2020-10-07	12.89	12.9	12.49	12.63	1596493	{}	\N	2020-10-12 14:31:09.0189
33233	CLI	2020-10-08	12.7	13.16	12.695	13.14	1376258	{}	\N	2020-10-12 14:31:09.022884
33234	CLI	2020-10-09	13.23	13.24	12.78	12.81	926358	{}	\N	2020-10-12 14:31:09.026651
33235	CLI	2020-10-12	12.73	12.87	12.62	12.86	129222	{}	\N	2020-10-12 14:31:09.030407
33236	CLRB	2020-09-28	1.23	1.2499	1.2	1.22	118512	{}	\N	2020-10-12 14:31:09.969187
33237	CLRB	2020-09-29	1.23	1.23	1.17	1.17	195461	{}	\N	2020-10-12 14:31:09.978022
33238	CLRB	2020-09-30	1.19	1.28	1.19	1.24	504598	{}	\N	2020-10-12 14:31:09.983826
33239	CLRB	2020-10-01	1.26	1.27	1.22	1.25	462384	{}	\N	2020-10-12 14:31:09.989485
33240	CLRB	2020-10-02	1.2	1.24	1.18	1.21	149531	{}	\N	2020-10-12 14:31:09.994392
33241	CLRB	2020-10-05	1.2	1.28	1.2	1.26	257427	{}	\N	2020-10-12 14:31:09.999279
33242	CLRB	2020-10-06	1.26	1.35	1.25	1.29	423716	{}	\N	2020-10-12 14:31:10.0036
33243	CLRB	2020-10-07	1.28	1.36	1.27	1.32	305874	{}	\N	2020-10-12 14:31:10.00748
33244	CLRB	2020-10-08	1.32	1.39	1.32	1.37	227138	{}	\N	2020-10-12 14:31:10.011491
33245	CLRB	2020-10-09	1.35	1.37	1.3	1.3	286744	{}	\N	2020-10-12 14:31:10.015459
33246	CLRB	2020-10-12	1.32	1.32	1.27	1.3	50091	{}	\N	2020-10-12 14:31:10.019623
33247	CM	2020-09-28	75.55	76.49	75.34	75.64	488723	{}	\N	2020-10-12 14:31:11.496077
33248	CM	2020-09-29	75.44	75.51	74.05	74.55	862410	{}	\N	2020-10-12 14:31:11.504739
33249	CM	2020-09-30	74.76	75.61	74.58	74.72	699862	{}	\N	2020-10-12 14:31:11.509964
33250	CM	2020-10-01	74.97	75.11	74.39	74.75	472865	{}	\N	2020-10-12 14:31:11.515391
33251	CM	2020-10-02	73.63	75.15	73.44	74.8	761823	{}	\N	2020-10-12 14:31:11.520258
33252	CM	2020-10-05	75.63	76.54	75.1	76.25	456440	{}	\N	2020-10-12 14:31:11.524827
33253	CM	2020-10-06	76.84	77.36	76.09	76.36	852404	{}	\N	2020-10-12 14:31:11.528747
33254	CM	2020-10-07	77	77.47	76.895	76.94	475615	{}	\N	2020-10-12 14:31:11.532881
33255	CM	2020-10-08	77.18	77.83	77.18	77.67	605302	{}	\N	2020-10-12 14:31:11.537128
33256	CM	2020-10-09	77.8	78.09	77.25	77.37	757983	{}	\N	2020-10-12 14:31:11.54119
33257	CM	2020-10-12	77.37	77.78	77.26	77.41	28283	{}	\N	2020-10-12 14:31:11.545213
33258	CMCSA	2020-09-28	46.89	47.07	46.24	46.76	15287493	{}	\N	2020-10-12 14:31:12.435692
33259	CMCSA	2020-09-29	46.85	46.87	46.17	46.2	11073750	{}	\N	2020-10-12 14:31:12.447045
33260	CMCSA	2020-09-30	46.5271	46.72	45.75	46.26	25538404	{}	\N	2020-10-12 14:31:12.453277
33261	CMCSA	2020-10-01	46.566	46.77	46.17	46.44	14771366	{}	\N	2020-10-12 14:31:12.459677
33262	CMCSA	2020-10-02	45.8	45.99	45.11	45.15	16849037	{}	\N	2020-10-12 14:31:12.464634
33263	CMCSA	2020-10-05	45.3	45.93	45.02	45.81	12336140	{}	\N	2020-10-12 14:31:12.468929
33264	CMCSA	2020-10-06	45.28	45.52	44.37	44.51	16574589	{}	\N	2020-10-12 14:31:12.472962
33265	CMCSA	2020-10-07	45.03	45.105	44.26	44.96	17500821	{}	\N	2020-10-12 14:31:12.476662
33266	CMCSA	2020-10-08	45.0895	45.39	44.83	45.3	15023477	{}	\N	2020-10-12 14:31:12.480883
33267	CMCSA	2020-10-09	45.3801	45.63	44.68	44.82	20004359	{}	\N	2020-10-12 14:31:12.484657
33268	CMCSA	2020-10-12	45.3	45.46	44.922	45.13	2511199	{}	\N	2020-10-12 14:31:12.488209
33269	CHIE	2020-09-28	8.0911	8.0917	8.05	8.0917	237	{}	\N	2020-10-12 14:31:13.566269
33270	CHIE	2020-09-29	8.11	8.1304	8.11	8.1304	281	{}	\N	2020-10-12 14:31:13.572911
33271	CHIE	2020-09-30	8.15	8.15	8.0664	8.0664	126	{}	\N	2020-10-12 14:31:13.578254
33272	CHIE	2020-10-01	8.175	8.175	8.175	8.175	93	{}	\N	2020-10-12 14:31:13.583021
33273	CHIE	2020-10-02	8.12	8.12	8.12	8.12	32	{}	\N	2020-10-12 14:31:13.587132
33274	CHIE	2020-10-05	8.0658	8.0658	8.0658	8.0658	26	{}	\N	2020-10-12 14:31:13.59127
33275	CHIE	2020-10-06	8.14	8.14	8.14	8.14	8	{}	\N	2020-10-12 14:31:13.594985
33276	CHIE	2020-10-07	8.09	8.09	8.09	8.09	43	{}	\N	2020-10-12 14:31:13.598574
33277	CHIE	2020-10-08	8.1	8.1	8.09	8.09	1663	{}	\N	2020-10-12 14:31:13.602359
33278	CHIE	2020-10-09	8.33	8.33	8.33	8.33	77	{}	\N	2020-10-12 14:31:13.60577
33279	CHKP	2020-09-28	121.8	122.29	120.55	121.6	771637	{}	\N	2020-10-12 14:31:14.940087
33280	CHKP	2020-09-29	121.88	122.0561	120.16	120.73	686254	{}	\N	2020-10-12 14:31:14.950592
33281	CHKP	2020-09-30	120.43	121.89	119.64	120.33	858189	{}	\N	2020-10-12 14:31:14.957342
33282	CHKP	2020-10-01	121.26	122.06	118.11	118.85	1368519	{}	\N	2020-10-12 14:31:14.962661
33283	CHKP	2020-10-02	116.97	119.47	116.97	118.31	772049	{}	\N	2020-10-12 14:31:14.967789
33284	CHKP	2020-10-05	119	121.67	118.41	121.26	756772	{}	\N	2020-10-12 14:31:14.972766
33285	CHKP	2020-10-06	120.95	122.99	120.57	121.04	848799	{}	\N	2020-10-12 14:31:14.977364
33286	CHKP	2020-10-07	121.75	123.125	121.75	122.62	575252	{}	\N	2020-10-12 14:31:14.981684
33287	CHKP	2020-10-08	123.59	124.82	123.07	123.33	659772	{}	\N	2020-10-12 14:31:14.985884
33288	CHKP	2020-10-09	123.53	125.24	123.35	125.24	420765	{}	\N	2020-10-12 14:31:14.989844
33289	CHKP	2020-10-12	126.28	126.6945	124.84	125.62	108459	{}	\N	2020-10-12 14:31:14.993881
33290	CHSCM	2020-09-28	26.31	26.3784	26.19	26.34	19987	{}	\N	2020-10-12 14:31:15.95642
33291	CHSCM	2020-09-29	26.35	26.51	26.2258	26.5	17584	{}	\N	2020-10-12 14:31:15.966466
33292	CHSCM	2020-09-30	26.62	26.62	26.16	26.25	66810	{}	\N	2020-10-12 14:31:15.972712
33293	CHSCM	2020-10-01	26.42	26.61	26.17	26.58	18841	{}	\N	2020-10-12 14:31:15.978753
33294	CHSCM	2020-10-02	26.42	26.72	26.17	26.72	16127	{}	\N	2020-10-12 14:31:15.984096
33295	CHSCM	2020-10-05	26.65	26.735	26.22	26.48	13260	{}	\N	2020-10-12 14:31:15.988847
33296	CHSCM	2020-10-06	26.7	26.92	26.5121	26.85	30724	{}	\N	2020-10-12 14:31:15.992784
33297	CHSCM	2020-10-07	26.92	27.18	26.92	27.12	34916	{}	\N	2020-10-12 14:31:15.996929
33298	CHSCM	2020-10-08	27.12	27.32	27.12	27.17	12093	{}	\N	2020-10-12 14:31:16.001322
33299	CHSCM	2020-10-09	27.0816	27.33	26.8659	27.1944	5840	{}	\N	2020-10-12 14:31:16.005599
33300	CHSCM	2020-10-12	27	27	26.51	26.51	12263	{}	\N	2020-10-12 14:31:16.009697
33301	CHUY	2020-09-28	19.42	20.19	19.285	19.97	174797	{}	\N	2020-10-12 14:31:17.08441
33302	CHUY	2020-09-29	19.94	19.94	18.935	19.62	134555	{}	\N	2020-10-12 14:31:17.09273
33303	CHUY	2020-09-30	19.9	20.335	19.475	19.58	167188	{}	\N	2020-10-12 14:31:17.098485
33304	CHUY	2020-10-01	19.89	19.915	19.31	19.82	88953	{}	\N	2020-10-12 14:31:17.104268
33305	CHUY	2020-10-02	19.2	20.41	19.2	20.31	120335	{}	\N	2020-10-12 14:31:17.109423
33306	CHUY	2020-10-05	20.58	21.02	20.17	20.32	120346	{}	\N	2020-10-12 14:31:17.114232
33307	CHUY	2020-10-06	20.49	21.42	20.4	20.46	128653	{}	\N	2020-10-12 14:31:17.119184
33308	CHUY	2020-10-07	21.1	21.1	20.62	20.91	98963	{}	\N	2020-10-12 14:31:17.123761
33309	CHUY	2020-10-08	21.3	21.43	20.81	21.34	151127	{}	\N	2020-10-12 14:31:17.127894
33310	CHUY	2020-10-09	21.59	22.42	21.59	22.36	167296	{}	\N	2020-10-12 14:31:17.132142
33311	CHUY	2020-10-12	22.23	22.49	21.76	22.49	13158	{}	\N	2020-10-12 14:31:17.136618
33312	CIBR	2020-09-28	34.8638	35.07	34.66	34.89	298366	{}	\N	2020-10-12 14:31:18.141477
33313	CIBR	2020-09-29	34.87	35.04	34.779	34.84	312749	{}	\N	2020-10-12 14:31:18.151725
33314	CIBR	2020-09-30	34.83	35.145	34.63	34.8	854184	{}	\N	2020-10-12 14:31:18.158483
33315	CIBR	2020-10-01	35.17	35.51	35.13	35.37	622341	{}	\N	2020-10-12 14:31:18.163997
33316	CIBR	2020-10-02	34.79	35.4	34.63	34.965	297104	{}	\N	2020-10-12 14:31:18.168391
33317	CIBR	2020-10-05	35.18	35.58	35.17	35.55	231627	{}	\N	2020-10-12 14:31:18.172648
33318	CIBR	2020-10-06	35.58	36.24	35.42	35.54	499031	{}	\N	2020-10-12 14:31:18.176604
33319	CIBR	2020-10-07	35.76	36.17	35.71	36.07	693769	{}	\N	2020-10-12 14:31:18.180426
33320	CIBR	2020-10-08	36.34	36.44	36.1053	36.17	289668	{}	\N	2020-10-12 14:31:18.184196
33321	CIBR	2020-10-09	36.39	36.72	36.39	36.67	343262	{}	\N	2020-10-12 14:31:18.187842
33322	CIBR	2020-10-12	36.9	37.21	36.9	37.1541	75604	{}	\N	2020-10-12 14:31:18.191382
33323	CIL	2020-09-28	36.794	36.88	36.782	36.88	1307	{}	\N	2020-10-12 14:31:19.30379
33324	CIL	2020-09-30	36.86	36.86	36.86	36.86	387	{}	\N	2020-10-12 14:31:19.311619
33325	CIL	2020-10-02	36.86	36.9388	36.86	36.9388	750	{}	\N	2020-10-12 14:31:19.317977
33326	CIL	2020-10-06	37.081	37.081	37.081	37.081	227	{}	\N	2020-10-12 14:31:19.323951
33327	CIL	2020-10-07	37.1902	37.2869	37.19	37.2869	1373	{}	\N	2020-10-12 14:31:19.329272
33328	CIL	2020-10-08	37.5098	37.51	37.465	37.465	1307	{}	\N	2020-10-12 14:31:19.333584
33329	CIL	2020-10-09	37.6665	37.6971	37.65	37.6971	783	{}	\N	2020-10-12 14:31:19.342024
33330	CL	2020-09-28	76.73	77.51	76.42	76.61	2546766	{}	\N	2020-10-12 14:31:20.648637
33331	CL	2020-09-29	76.67	76.78	75.85	76.22	4153062	{}	\N	2020-10-12 14:31:20.659724
33332	CL	2020-09-30	76.58	77.45	76.41	77.15	7788739	{}	\N	2020-10-12 14:31:20.666489
33333	CL	2020-10-01	77.53	77.73	76.7351	77.06	7864784	{}	\N	2020-10-12 14:31:20.671523
33334	CL	2020-10-02	76.59	77.96	76.46	77.4	6765600	{}	\N	2020-10-12 14:31:20.676207
33335	CL	2020-10-05	77.5	78.49	77.43	78.42	5585374	{}	\N	2020-10-12 14:31:20.68049
33336	CL	2020-10-06	78.55	78.63	77.2	77.54	5201024	{}	\N	2020-10-12 14:31:20.684674
33337	CL	2020-10-07	77.83	78.39	77.69	78.13	3777599	{}	\N	2020-10-12 14:31:20.688117
33338	CL	2020-10-08	78.23	78.84	78.08	78.74	4291356	{}	\N	2020-10-12 14:31:20.691566
33339	CL	2020-10-09	79.09	80.1262	78.89	79.77	5702067	{}	\N	2020-10-12 14:31:20.695086
33340	CL	2020-10-12	79.69	80.38	79.62	80.27	1775860	{}	\N	2020-10-12 14:31:20.698637
33341	CLDX	2020-09-28	12.96	15.65	12.82	14.98	3596413	{}	\N	2020-10-12 14:31:21.63183
33342	CLDX	2020-09-29	15.3865	16.19	14.22	15.19	2178892	{}	\N	2020-10-12 14:31:21.64115
33343	CLDX	2020-09-30	15.12	15.63	14.33	14.82	929616	{}	\N	2020-10-12 14:31:21.647377
33344	CLDX	2020-10-01	14.85	15.48	14.6	14.99	820146	{}	\N	2020-10-12 14:31:21.652472
33345	CLDX	2020-10-02	14.56	15.43	14.4	14.99	864318	{}	\N	2020-10-12 14:31:21.657101
33346	CLDX	2020-10-05	15.46	16.265	14.52	15.2	1176775	{}	\N	2020-10-12 14:31:21.661355
33347	CLDX	2020-10-06	15.04	18.79	14.55	18.16	2473254	{}	\N	2020-10-12 14:31:21.666252
33348	CLDX	2020-10-07	18.16	19.32	17.62	18.22	974029	{}	\N	2020-10-12 14:31:21.669809
33349	CLDX	2020-10-08	18.46	18.72	16.79	17.01	547455	{}	\N	2020-10-12 14:31:21.673335
33350	CLDX	2020-10-09	17.25	17.505	16.87	17.14	592729	{}	\N	2020-10-12 14:31:21.676971
33351	CLDX	2020-10-12	17.27	17.72	16.81	17.04	98140	{}	\N	2020-10-12 14:31:21.680413
33352	CLIX	2020-09-28	84.5	84.73	83.49	84.73	18513	{}	\N	2020-10-12 14:31:23.053667
33353	CLIX	2020-09-29	85.11	85.3	84.5524	84.72	16985	{}	\N	2020-10-12 14:31:23.061636
33354	CLIX	2020-09-30	85.64	85.64	84.1902	84.22	16412	{}	\N	2020-10-12 14:31:23.067001
33355	CLIX	2020-10-01	85.35	85.89	84.57	85.89	92150	{}	\N	2020-10-12 14:31:23.072088
33356	CLIX	2020-10-02	83.89	85.59	83.89	84.17	23765	{}	\N	2020-10-12 14:31:23.076818
33357	CLIX	2020-10-05	84.23	85.1985	84.21	85.11	12719	{}	\N	2020-10-12 14:31:23.081369
33358	CLIX	2020-10-06	85.11	85.4184	84.435	84.5357	23291	{}	\N	2020-10-12 14:31:23.085534
33359	CLIX	2020-10-07	84.95	86.04	84.95	86.04	25730	{}	\N	2020-10-12 14:31:23.089121
33360	CLIX	2020-10-08	86.24	86.24	85.4801	85.5	65241	{}	\N	2020-10-12 14:31:23.092191
33361	CLIX	2020-10-09	85.69	87.49	85.69	87.49	28043	{}	\N	2020-10-12 14:31:23.095617
33362	CLIX	2020-10-12	88.32	89.16	88.29	89.075	10926	{}	\N	2020-10-12 14:31:23.099091
33363	CLTL	2020-09-28	105.75	105.77	105.74	105.74	39739	{}	\N	2020-10-12 14:31:24.096051
33364	CLTL	2020-09-29	105.7425	105.78	105.74	105.78	73861	{}	\N	2020-10-12 14:31:24.104704
33365	CLTL	2020-09-30	105.78	105.78	105.73	105.73	48917	{}	\N	2020-10-12 14:31:24.109647
33366	CLTL	2020-10-01	105.7644	105.77	105.74	105.74	17511	{}	\N	2020-10-12 14:31:24.115019
33367	CLTL	2020-10-02	105.73	105.77	105.73	105.75	22259	{}	\N	2020-10-12 14:31:24.119813
33368	CLTL	2020-10-05	105.75	105.76	105.74	105.74	49330	{}	\N	2020-10-12 14:31:24.124208
33369	CLTL	2020-10-06	105.74	105.76	105.74	105.7599	298980	{}	\N	2020-10-12 14:31:24.127884
33370	CLTL	2020-10-07	105.75	105.763	105.74	105.75	25768	{}	\N	2020-10-12 14:31:24.13144
33371	CLTL	2020-10-08	105.76	105.77	105.75	105.75	52928	{}	\N	2020-10-12 14:31:24.135335
33372	CLTL	2020-10-09	105.77	105.77	105.75	105.76	65488	{}	\N	2020-10-12 14:31:24.13892
33373	CLTL	2020-10-12	105.75	105.77	105.75	105.77	531	{}	\N	2020-10-12 14:31:24.142597
33374	CHRW	2020-09-28	103.15	104.535	102.68	103.25	999636	{}	\N	2020-10-12 14:31:25.482744
33375	CHRW	2020-09-29	103.51	104.95	102.95	103.7	1153494	{}	\N	2020-10-12 14:31:25.491101
33376	CHRW	2020-09-30	104.11	105.6	101.73	102.19	1420551	{}	\N	2020-10-12 14:31:25.497326
33377	CHRW	2020-10-01	103.09	103.71	100.84	101.22	878335	{}	\N	2020-10-12 14:31:25.50283
33378	CHRW	2020-10-02	100.13	101.69	99	100.87	1181061	{}	\N	2020-10-12 14:31:25.507691
33379	CHRW	2020-10-05	102.21	104.34	101.88	104	1243948	{}	\N	2020-10-12 14:31:25.511998
33380	CHRW	2020-10-06	103.4	104.27	102.25	103.59	1029249	{}	\N	2020-10-12 14:31:25.515864
33381	CHRW	2020-10-07	104.66	106.47	104.205	106.16	1226338	{}	\N	2020-10-12 14:31:25.519888
33382	CHRW	2020-10-08	106.48	106.75	105.61	106.14	824125	{}	\N	2020-10-12 14:31:25.523838
33383	CHRW	2020-10-09	106.6	106.61	104.82	105.45	981264	{}	\N	2020-10-12 14:31:25.527575
33384	CHRW	2020-10-12	106.02	106.29	105.28	105.345	176584	{}	\N	2020-10-12 14:31:25.531404
33385	CHTR	2020-09-28	625.16	633.64	619.64	628.23	950833	{}	\N	2020-10-12 14:31:26.705373
33386	CHTR	2020-09-29	626.85	635.8262	625.02	626.5	768978	{}	\N	2020-10-12 14:31:26.715218
33387	CHTR	2020-09-30	629.81	632.06	620.71	624.34	1039402	{}	\N	2020-10-12 14:31:26.721721
33388	CHTR	2020-10-01	627.36	640	627.36	635.11	1319887	{}	\N	2020-10-12 14:31:26.728046
33389	CHTR	2020-10-02	629.36	633.17	617.285	620.23	926851	{}	\N	2020-10-12 14:31:26.733504
33390	CHTR	2020-10-05	623.35	639.27	621.63	633.98	790180	{}	\N	2020-10-12 14:31:26.738316
33391	CHTR	2020-10-06	631.31	638.06	619.1	620.89	968310	{}	\N	2020-10-12 14:31:26.742616
33392	CHTR	2020-10-07	623.23	629.73	622.99	623.73	771792	{}	\N	2020-10-12 14:31:26.746533
33393	CHTR	2020-10-08	626.81	633.689	622.235	629.08	566945	{}	\N	2020-10-12 14:31:26.750288
33394	CHTR	2020-10-09	631.88	639.34	631.64	635.94	507834	{}	\N	2020-10-12 14:31:26.754484
33395	CHTR	2020-10-12	643.57	650.49	639.24	648.99	239393	{}	\N	2020-10-12 14:31:26.759897
33396	CIB	2020-09-28	25.96	26.31	25.48	25.7	243446	{}	\N	2020-10-12 14:31:30.25004
33397	CIB	2020-09-29	25.64	25.72	24.92	25.04	605675	{}	\N	2020-10-12 14:31:30.261971
33398	CIB	2020-09-30	25.25	25.63	25.08	25.55	418036	{}	\N	2020-10-12 14:31:30.268887
33399	CIB	2020-10-01	25.66	25.93	25.51	25.66	172997	{}	\N	2020-10-12 14:31:30.274844
33400	CIB	2020-10-02	25.15	25.94	25.11	25.85	111769	{}	\N	2020-10-12 14:31:30.279756
33401	CIB	2020-10-05	26.17	26.31	25.89	26.08	165117	{}	\N	2020-10-12 14:31:30.284676
33402	CIB	2020-10-06	26.41	26.534	25.87	25.88	283360	{}	\N	2020-10-12 14:31:30.288944
33403	CIB	2020-10-07	26.1	26.176	25.52	25.67	357958	{}	\N	2020-10-12 14:31:30.292906
33404	CIB	2020-10-08	25.86	26.16	25.77	26.03	327302	{}	\N	2020-10-12 14:31:30.297406
33405	CIB	2020-10-09	26.22	26.415	26.21	26.33	245020	{}	\N	2020-10-12 14:31:30.301723
33406	CIB	2020-10-12	26.36	26.36	25.694	25.694	14914	{}	\N	2020-10-12 14:31:30.305829
33407	CIGI	2020-09-28	66.47	67.31	66.46	67.15	65862	{}	\N	2020-10-12 14:31:31.437385
33408	CIGI	2020-09-29	66.73	66.7825	65.23	66.25	109249	{}	\N	2020-10-12 14:31:31.446213
33409	CIGI	2020-09-30	66.135	67.42	66.02	66.595	207762	{}	\N	2020-10-12 14:31:31.452701
33410	CIGI	2020-10-01	67.265	67.5	65.77	66.74	280115	{}	\N	2020-10-12 14:31:31.458224
33411	CIGI	2020-10-02	65.79	66.47	65.455	65.81	42020	{}	\N	2020-10-12 14:31:31.464676
33412	CIGI	2020-10-05	66.39	66.76	64.63	65.64	44979	{}	\N	2020-10-12 14:31:31.469574
33413	CIGI	2020-10-06	65.62	66.71	65.46	65.97	79147	{}	\N	2020-10-12 14:31:31.474737
33414	CIGI	2020-10-07	67.12	67.69	66.29	66.32	27659	{}	\N	2020-10-12 14:31:31.479597
33415	CIGI	2020-10-08	66.83	67.695	66.7101	66.86	73912	{}	\N	2020-10-12 14:31:31.484269
33416	CIGI	2020-10-09	67.06	67.155	66.325	66.94	19975	{}	\N	2020-10-12 14:31:31.489723
33417	CIGI	2020-10-12	67.11	67.84	66.54	66.9824	3389	{}	\N	2020-10-12 14:31:31.49438
33418	CIVB	2020-09-28	12.55	12.909	12.55	12.63	49642	{}	\N	2020-10-12 14:31:33.864427
33419	CIVB	2020-09-29	12.4497	12.59	12.25	12.58	63623	{}	\N	2020-10-12 14:31:33.873924
33420	CIVB	2020-09-30	12.705	12.82	12.5	12.52	37738	{}	\N	2020-10-12 14:31:33.879934
33421	CIVB	2020-10-01	12.54	12.94	12.32	12.84	42344	{}	\N	2020-10-12 14:31:33.886162
33422	CIVB	2020-10-02	12.73	13.41	12.64	13.3	25504	{}	\N	2020-10-12 14:31:33.891757
33423	CIVB	2020-10-05	13.38	13.76	13.29	13.6	34691	{}	\N	2020-10-12 14:31:33.896379
33424	CIVB	2020-10-06	13.81	14.2	13.47	13.52	25850	{}	\N	2020-10-12 14:31:33.900574
33425	CIVB	2020-10-07	13.63	14.06	13.59	14.05	41381	{}	\N	2020-10-12 14:31:33.904933
33426	CIVB	2020-10-08	14.23	14.23	13.88	14.02	60214	{}	\N	2020-10-12 14:31:33.909134
33427	CIVB	2020-10-09	14.25	14.25	13.56	14.02	20941	{}	\N	2020-10-12 14:31:33.913337
33428	CIVB	2020-10-12	14.2	14.2	14.2	14.2	1719	{}	\N	2020-10-12 14:31:33.917497
33429	CKPT	2020-09-28	2.8001	2.8177	2.5532	2.62	1376250	{}	\N	2020-10-12 14:31:37.772712
33430	CKPT	2020-09-29	2.7	3.06	2.67	2.79	2524562	{}	\N	2020-10-12 14:31:37.782815
33431	CKPT	2020-09-30	2.81	2.88	2.51	2.68	1340203	{}	\N	2020-10-12 14:31:37.788677
33432	CKPT	2020-10-01	2.67	2.74	2.5901	2.68	662490	{}	\N	2020-10-12 14:31:37.793351
33433	CKPT	2020-10-02	2.56	2.69	2.53	2.64	725884	{}	\N	2020-10-12 14:31:37.79849
33434	CKPT	2020-10-05	2.59	2.74	2.51	2.6	621042	{}	\N	2020-10-12 14:31:37.80345
33435	CKPT	2020-10-06	2.635	2.7	2.585	2.63	528500	{}	\N	2020-10-12 14:31:37.807592
33436	CKPT	2020-10-07	2.64	2.83	2.62	2.79	897337	{}	\N	2020-10-12 14:31:37.811437
33437	CKPT	2020-10-08	2.87	3.08	2.84	2.96	1470050	{}	\N	2020-10-12 14:31:37.815468
33438	CKPT	2020-10-09	2.89	2.94	2.72	2.83	706552	{}	\N	2020-10-12 14:31:37.819625
33439	CKPT	2020-10-12	2.88	2.8801	2.8	2.81	122935	{}	\N	2020-10-12 14:31:37.823996
33440	CLH	2020-09-28	55.88	56.49	55.24	56.13	406694	{}	\N	2020-10-12 14:31:39.589582
33441	CLH	2020-09-29	56.17	56.61	55.0099	56.01	542724	{}	\N	2020-10-12 14:31:39.598281
33442	CLH	2020-09-30	56.57	57.17	55.62	56.03	406705	{}	\N	2020-10-12 14:31:39.604704
33443	CLH	2020-10-01	56.25	56.98	55.82	56.37	489103	{}	\N	2020-10-12 14:31:39.610825
33444	CLH	2020-10-02	55.3	57.89	55.29	57.88	407163	{}	\N	2020-10-12 14:31:39.615832
33445	CLH	2020-10-05	58.31	58.61	57.71	58.29	482212	{}	\N	2020-10-12 14:31:39.620585
33446	CLH	2020-10-06	58.94	59.22	57.62	57.79	503942	{}	\N	2020-10-12 14:31:39.625394
33447	CLH	2020-10-07	58.27	59.63	57.82	58.85	488123	{}	\N	2020-10-12 14:31:39.629894
33448	CLH	2020-10-08	59.4	59.81	58.35	59.07	286536	{}	\N	2020-10-12 14:31:39.634669
33449	CLH	2020-10-09	59.49	59.55	58.59	58.87	257608	{}	\N	2020-10-12 14:31:39.639362
33450	CLH	2020-10-12	59.29	59.54	58.99	59.44	27272	{}	\N	2020-10-12 14:31:39.643978
33451	CLMT	2020-09-28	2.74	2.84	2.65	2.67	123203	{}	\N	2020-10-12 14:31:41.079117
33452	CLMT	2020-09-29	2.64	2.69	2.56	2.62	35772	{}	\N	2020-10-12 14:31:41.090922
33453	CLMT	2020-09-30	2.61	2.68	2.54	2.55	80476	{}	\N	2020-10-12 14:31:41.098356
33454	CLMT	2020-10-01	2.56	3.07	2.51	2.59	444292	{}	\N	2020-10-12 14:31:41.104546
33455	CLMT	2020-10-02	2.55	2.89	2.55	2.86	339265	{}	\N	2020-10-12 14:31:41.109576
33456	CLMT	2020-10-05	2.98	3.06	2.89	2.94	459474	{}	\N	2020-10-12 14:31:41.114809
33457	CLMT	2020-10-06	2.91	3.19	2.86	3.1	283137	{}	\N	2020-10-12 14:31:41.119519
33458	CLMT	2020-10-07	3.09	3.2	3.04	3.14	152566	{}	\N	2020-10-12 14:31:41.123994
33459	CLMT	2020-10-08	3.15	3.47	3.15	3.44	212189	{}	\N	2020-10-12 14:31:41.128901
33460	CLMT	2020-10-09	3.41	3.4994	3.24	3.45	151093	{}	\N	2020-10-12 14:31:41.133497
33461	CLMT	2020-10-12	3.38	3.46	3.31	3.4	27146	{}	\N	2020-10-12 14:31:41.138316
35951	DBAW	2020-09-28	27.34	27.4389	27.34	27.366	3461	{}	\N	2020-10-12 14:37:43.022957
35952	DBAW	2020-09-29	27.4	27.4	27.3367	27.3367	3674	{}	\N	2020-10-12 14:37:43.036641
35953	DBAW	2020-09-30	27.3512	27.4001	27.2	27.26	343911	{}	\N	2020-10-12 14:37:43.042373
35954	DBAW	2020-10-01	27.46	27.46	27.31	27.38	31643	{}	\N	2020-10-12 14:37:43.047913
35955	DBAW	2020-10-02	27.13	27.36	27.13	27.3253	8720	{}	\N	2020-10-12 14:37:43.052424
35956	DBAW	2020-10-05	27.49	27.58	27.47	27.58	8970	{}	\N	2020-10-12 14:37:43.057144
35957	DBAW	2020-10-06	27.7	27.71	27.52	27.52	4862	{}	\N	2020-10-12 14:37:43.061575
35958	DBAW	2020-10-07	27.71	27.78	27.68	27.7707	13256	{}	\N	2020-10-12 14:37:43.065824
35959	DBAW	2020-10-08	27.89	27.9301	27.89	27.9207	7713	{}	\N	2020-10-12 14:37:43.070198
35960	DBAW	2020-10-09	27.9	28.1225	27.9	28.12	164639	{}	\N	2020-10-12 14:37:43.074484
35961	DBAW	2020-10-12	28.213	28.213	28.19	28.19	615	{}	\N	2020-10-12 14:37:43.078378
35962	CTEK	2020-09-28	1.1	1.16	1.1	1.14	51178	{}	\N	2020-10-12 14:37:44.565285
35963	CTEK	2020-09-29	1.141	1.16	1.06	1.11	25531	{}	\N	2020-10-12 14:37:44.577248
35964	CTEK	2020-09-30	1.96	2.5	1.34	1.57	53223347	{}	\N	2020-10-12 14:37:44.583962
35965	CTEK	2020-10-01	1.33	1.6	1.3	1.55	2066038	{}	\N	2020-10-12 14:37:44.589767
35966	CTEK	2020-10-02	1.36	1.4321	1.35	1.392	546119	{}	\N	2020-10-12 14:37:44.594635
35967	CTEK	2020-10-05	1.35	1.45	1.31	1.35	418590	{}	\N	2020-10-12 14:37:44.599638
35968	CTEK	2020-10-06	1.32	1.45	1.32	1.4	277238	{}	\N	2020-10-12 14:37:44.603936
35969	CTEK	2020-10-07	1.35	1.41	1.34	1.4	242179	{}	\N	2020-10-12 14:37:44.608151
35970	CTEK	2020-10-08	1.42	1.47	1.405	1.43	193690	{}	\N	2020-10-12 14:37:44.61214
35971	CTEK	2020-10-09	1.39	1.49	1.39	1.49	115811	{}	\N	2020-10-12 14:37:44.616309
35972	CTEK	2020-10-12	1.42	1.5	1.41	1.5	6493	{}	\N	2020-10-12 14:37:44.620698
35973	CTXS	2020-09-28	137.7	139	136.69	138.32	714075	{}	\N	2020-10-12 14:37:46.801399
35974	CTXS	2020-09-29	138.19	139.05	136.89	136.95	746195	{}	\N	2020-10-12 14:37:46.809828
35975	CTXS	2020-09-30	137.22	139.14	136.06	137.71	869914	{}	\N	2020-10-12 14:37:46.815785
35976	CTXS	2020-10-01	139.03	139.93	137.75	139.21	678318	{}	\N	2020-10-12 14:37:46.820975
35977	CTXS	2020-10-02	137.42	138.49	134.6	134.89	708919	{}	\N	2020-10-12 14:37:46.825508
35978	CTXS	2020-10-05	135.49	136.17	134.2	135.63	951797	{}	\N	2020-10-12 14:37:46.82977
35979	CTXS	2020-10-06	134.9813	137.43	132.86	133.34	1003888	{}	\N	2020-10-12 14:37:46.834119
35980	CTXS	2020-10-07	134.05	134.84	132.21	133.51	1182614	{}	\N	2020-10-12 14:37:46.837926
35981	CTXS	2020-10-08	134.79	135.77	133.65	135.51	704665	{}	\N	2020-10-12 14:37:46.841584
35982	CTXS	2020-10-09	137.02	139.4758	136.1	139.43	735052	{}	\N	2020-10-12 14:37:46.845188
35983	CTXS	2020-10-12	141	142.21	140.7102	141.69	204619	{}	\N	2020-10-12 14:37:46.848882
33462	CLSD	2020-09-28	1.53	1.58	1.5	1.5	61645	{}	\N	2020-10-12 14:31:42.41816
33463	CLSD	2020-09-29	1.48	1.5103	1.4522	1.48	100837	{}	\N	2020-10-12 14:31:42.43382
33464	CLSD	2020-09-30	1.48	1.54	1.4601	1.54	129268	{}	\N	2020-10-12 14:31:42.439993
33465	CLSD	2020-10-01	1.53	1.58	1.523	1.58	57375	{}	\N	2020-10-12 14:31:42.445408
33466	CLSD	2020-10-02	1.53	1.56	1.49	1.55	74761	{}	\N	2020-10-12 14:31:42.449925
33467	CLSD	2020-10-05	1.55	1.6	1.55	1.58	94252	{}	\N	2020-10-12 14:31:42.454155
33468	CLSD	2020-10-06	1.59	1.6	1.5052	1.52	63438	{}	\N	2020-10-12 14:31:42.458501
33469	CLSD	2020-10-07	1.55	1.55	1.49	1.52	50738	{}	\N	2020-10-12 14:31:42.464712
33470	CLSD	2020-10-08	1.53	1.57	1.51	1.57	219489	{}	\N	2020-10-12 14:31:42.468778
33471	CLSD	2020-10-09	1.5649	1.6	1.47	1.54	444545	{}	\N	2020-10-12 14:31:42.472757
33472	CLSD	2020-10-12	1.56	1.5845	1.5177	1.555	14266	{}	\N	2020-10-12 14:31:42.476971
33473	CMC	2020-09-28	20.69	20.98	20.4	20.62	748213	{}	\N	2020-10-12 14:31:43.70343
33474	CMC	2020-09-29	20.6	20.72	19.96	20.03	1281808	{}	\N	2020-10-12 14:31:43.712484
33475	CMC	2020-09-30	20.13	20.54	19.88	19.98	2220033	{}	\N	2020-10-12 14:31:43.718435
33476	CMC	2020-10-01	20.02	20.36	19.86	20.1	1797421	{}	\N	2020-10-12 14:31:43.723806
33477	CMC	2020-10-02	19.67	20.89	19.61	20.56	2017536	{}	\N	2020-10-12 14:31:43.728685
33478	CMC	2020-10-05	20.88	21.45	20.83	21.33	1735532	{}	\N	2020-10-12 14:31:43.733568
33479	CMC	2020-10-06	21.5	21.87	21.035	21.06	3195128	{}	\N	2020-10-12 14:31:43.738048
33480	CMC	2020-10-07	21.49	22.24	21.49	22.12	2235212	{}	\N	2020-10-12 14:31:43.742396
33481	CMC	2020-10-08	22.41	22.85	22.24	22.79	1709496	{}	\N	2020-10-12 14:31:43.7466
33482	CMC	2020-10-09	23	23.17	22.5	22.64	3168117	{}	\N	2020-10-12 14:31:43.750958
33483	CMC	2020-10-12	22.6	23.245	22.53	23.085	528089	{}	\N	2020-10-12 14:31:43.755476
33484	CMPR	2020-09-28	73.39	75.545	72.5	75.13	128326	{}	\N	2020-10-12 14:31:44.718081
33485	CMPR	2020-09-29	75.14	75.37	73.335	74.12	69966	{}	\N	2020-10-12 14:31:44.729595
33486	CMPR	2020-09-30	74.94	76.74	74.51	75.16	96192	{}	\N	2020-10-12 14:31:44.736622
33487	CMPR	2020-10-01	74.945	77.035	73.1	75.31	115225	{}	\N	2020-10-12 14:31:44.742849
33488	CMPR	2020-10-02	73.45	78.32	72.86	78.22	115160	{}	\N	2020-10-12 14:31:44.748121
33489	CMPR	2020-10-05	79.05	80.84	78.01	80.68	145453	{}	\N	2020-10-12 14:31:44.752872
33490	CMPR	2020-10-06	81.48	84.17	80.26	81.06	148594	{}	\N	2020-10-12 14:31:44.758201
33491	CMPR	2020-10-07	82.78	85.35	82.28	84.72	136764	{}	\N	2020-10-12 14:31:44.766513
33492	CMPR	2020-10-08	86.2	87.93	84.98	85.94	89119	{}	\N	2020-10-12 14:31:44.771478
33493	CMPR	2020-10-09	86.82	89.69	85.535	85.72	113168	{}	\N	2020-10-12 14:31:44.775815
33494	CMPR	2020-10-12	85.77	86.97	85.31	86.78	8218	{}	\N	2020-10-12 14:31:44.780461
33495	CNK	2020-09-28	10.64	10.64	10.2	10.31	5767710	{}	\N	2020-10-12 14:31:48.894585
33496	CNK	2020-09-29	10.21	10.25	9.76	9.85	12607315	{}	\N	2020-10-12 14:31:48.904684
33497	CNK	2020-09-30	10.4	10.7	9.86	10	17197015	{}	\N	2020-10-12 14:31:48.911332
33498	CNK	2020-10-01	9.98	10.1601	9.62	10.06	12314392	{}	\N	2020-10-12 14:31:48.917516
33499	CNK	2020-10-02	9.41	10.22	9.4	10.09	11865497	{}	\N	2020-10-12 14:31:48.922265
33500	CNK	2020-10-05	9.46	9.65	8.22	8.33	58978178	{}	\N	2020-10-12 14:31:48.926833
33501	CNK	2020-10-06	8.5	9.14	8.44	8.6	40323546	{}	\N	2020-10-12 14:31:48.931665
33502	CNK	2020-10-07	8.65	8.73	8.1181	8.43	28263919	{}	\N	2020-10-12 14:31:48.936008
33503	CNK	2020-10-08	8.5	8.84	8.26	8.77	28597800	{}	\N	2020-10-12 14:31:48.940087
33504	CNK	2020-10-09	8.88	9.02	8.36	8.43	23365710	{}	\N	2020-10-12 14:31:48.94409
33505	CNK	2020-10-12	8.41	8.51	8.17	8.365	4750671	{}	\N	2020-10-12 14:31:48.948438
33506	CHNR	2020-09-28	1.11	1.1385	1.05	1.08	108032	{}	\N	2020-10-12 14:31:50.219784
33507	CHNR	2020-09-29	1.1	1.15	1.08	1.14	187190	{}	\N	2020-10-12 14:31:50.229828
33508	CHNR	2020-09-30	1.12	1.16	1.1	1.11	185475	{}	\N	2020-10-12 14:31:50.236492
33509	CHNR	2020-10-01	1.1	1.2	1.1	1.16	316935	{}	\N	2020-10-12 14:31:50.242032
33510	CHNR	2020-10-02	1.12	1.21	1.09	1.16	539189	{}	\N	2020-10-12 14:31:50.247057
33511	CHNR	2020-10-05	1.1	1.35	1.09	1.25	1370174	{}	\N	2020-10-12 14:31:50.251882
33512	CHNR	2020-10-06	1.3737	1.98	1.18	1.57	7575580	{}	\N	2020-10-12 14:31:50.255893
33513	CHNR	2020-10-07	1.46	1.62	1.3	1.32	1229847	{}	\N	2020-10-12 14:31:50.26012
33514	CHNR	2020-10-08	1.3	1.3747	1.17	1.21	530343	{}	\N	2020-10-12 14:31:50.264116
33515	CHNR	2020-10-09	1.22	1.33	1.18	1.29	566020	{}	\N	2020-10-12 14:31:50.268301
33516	CHNR	2020-10-12	1.29	1.29	1.2301	1.28	43785	{}	\N	2020-10-12 14:31:50.272529
33517	CI	2020-09-28	164.91	166.71	163.89	164.29	1150008	{}	\N	2020-10-12 14:31:51.275947
33518	CI	2020-09-29	164.75	165.46	162.42	164.31	2892189	{}	\N	2020-10-12 14:31:51.28669
33519	CI	2020-09-30	164.08	170.81	163.66	169.41	4571580	{}	\N	2020-10-12 14:31:51.294716
33520	CI	2020-10-01	169.35	169.87	166.07	167.38	2585102	{}	\N	2020-10-12 14:31:51.300769
33521	CI	2020-10-02	164.46	170.15	164.04	167.23	3516731	{}	\N	2020-10-12 14:31:51.30574
33522	CI	2020-10-05	169.2	173.33	168.71	172.22	3789664	{}	\N	2020-10-12 14:31:51.310699
33523	CI	2020-10-06	173.29	175.6	170.76	170.88	2868719	{}	\N	2020-10-12 14:31:51.315133
33524	CI	2020-10-07	172.3	177.04	172.3	176.3	3086621	{}	\N	2020-10-12 14:31:51.319856
33525	CI	2020-10-08	176.4	178.145	175.13	177.87	2239728	{}	\N	2020-10-12 14:31:51.32406
33526	CI	2020-10-09	178.5	178.64	175.78	176.4	2262821	{}	\N	2020-10-12 14:31:51.328259
33527	CI	2020-10-12	176.13	178.75	175.4875	178.36	399690	{}	\N	2020-10-12 14:31:51.332511
33528	CIG	2020-09-28	1.96	1.9788	1.83	1.86	4833643	{}	\N	2020-10-12 14:31:52.305693
33529	CIG	2020-09-29	1.82	1.87	1.8119	1.85	7606658	{}	\N	2020-10-12 14:31:52.316594
33530	CIG	2020-09-30	1.85	1.875	1.83	1.85	10955628	{}	\N	2020-10-12 14:31:52.323424
33531	CIG	2020-10-01	1.84	1.89	1.815	1.89	4187764	{}	\N	2020-10-12 14:31:52.328852
33532	CIG	2020-10-02	1.85	1.9	1.83	1.85	7272646	{}	\N	2020-10-12 14:31:52.333993
33533	CIG	2020-10-05	1.85	1.95	1.82	1.94	5291782	{}	\N	2020-10-12 14:31:52.338647
33534	CIG	2020-10-06	1.96	1.98	1.895	1.92	8783629	{}	\N	2020-10-12 14:31:52.342896
33535	CIG	2020-10-07	1.91	1.935	1.87	1.91	12827084	{}	\N	2020-10-12 14:31:52.346652
33536	CIG	2020-10-08	1.9	1.96	1.88	1.96	16427085	{}	\N	2020-10-12 14:31:52.350249
33537	CIG	2020-10-09	1.98	2.02	1.94	2.01	9286255	{}	\N	2020-10-12 14:31:52.353692
33538	CIG	2020-10-12	2.01	2.02	1.94	1.955	632478	{}	\N	2020-10-12 14:31:52.357373
33539	CIX	2020-09-28	13.41	14.895	13.41	14.5	31624	{}	\N	2020-10-12 14:31:53.376948
33540	CIX	2020-09-29	14.5	14.665	14.27	14.51	2467	{}	\N	2020-10-12 14:31:53.388547
33541	CIX	2020-09-30	14.49	14.95	14.49	14.95	10413	{}	\N	2020-10-12 14:31:53.394535
33542	CIX	2020-10-01	14.58	14.7	14.31	14.31	4725	{}	\N	2020-10-12 14:31:53.399887
33543	CIX	2020-10-02	14.11	14.11	13.82	13.82	839	{}	\N	2020-10-12 14:31:53.40454
33544	CIX	2020-10-05	13.71	14.38	13.71	14.38	2361	{}	\N	2020-10-12 14:31:53.408532
33545	CIX	2020-10-06	13.7	14.05	13.7	13.85	4590	{}	\N	2020-10-12 14:31:53.412746
33546	CIX	2020-10-07	13.76	14.17	13.66	13.9	4620	{}	\N	2020-10-12 14:31:53.416752
33547	CIX	2020-10-08	13.79	13.99	13.78	13.85	8376	{}	\N	2020-10-12 14:31:53.420512
33548	CIX	2020-10-09	14	14.04	13.58	14.04	16801	{}	\N	2020-10-12 14:31:53.424366
33549	CIX	2020-10-12	14.25	14.27	14.25	14.27	549	{}	\N	2020-10-12 14:31:53.428168
33550	CLDT	2020-09-28	7.35	7.83	7.305	7.7	378956	{}	\N	2020-10-12 14:31:54.579546
33551	CLDT	2020-09-29	7.68	7.68	7.33	7.49	666476	{}	\N	2020-10-12 14:31:54.589935
33552	CLDT	2020-09-30	7.54	7.96	7.54	7.62	837510	{}	\N	2020-10-12 14:31:54.596616
33553	CLDT	2020-10-01	7.65	7.83	7.54	7.81	709075	{}	\N	2020-10-12 14:31:54.602493
33554	CLDT	2020-10-02	7.55	7.85	7.44	7.83	657784	{}	\N	2020-10-12 14:31:54.607513
33555	CLDT	2020-10-05	7.79	7.91	7.34	7.42	572122	{}	\N	2020-10-12 14:31:54.611985
33556	CLDT	2020-10-06	7.52	7.8	7.3801	7.41	561198	{}	\N	2020-10-12 14:31:54.616333
33557	CLDT	2020-10-07	7.49	7.65	7.34	7.43	552262	{}	\N	2020-10-12 14:31:54.620005
33558	CLDT	2020-10-08	7.5	7.7	7.41	7.69	415968	{}	\N	2020-10-12 14:31:54.623939
33559	CLDT	2020-10-09	7.8	7.8	7.4901	7.6	530639	{}	\N	2020-10-12 14:31:54.627743
33560	CLDT	2020-10-12	7.56	7.63	7.3416	7.63	114960	{}	\N	2020-10-12 14:31:54.631455
33561	CLLS	2020-09-28	18.56	18.8	17.88	18.5	124762	{}	\N	2020-10-12 14:31:55.558164
33562	CLLS	2020-09-29	18.5	18.66	18.045	18.46	121046	{}	\N	2020-10-12 14:31:55.569403
33563	CLLS	2020-09-30	18.38	19.07	18.23	18.5	137796	{}	\N	2020-10-12 14:31:55.577368
33564	CLLS	2020-10-01	18.85	18.93	18.2677	18.43	124678	{}	\N	2020-10-12 14:31:55.583594
33565	CLLS	2020-10-02	18.29	18.4199	17	17.27	210390	{}	\N	2020-10-12 14:31:55.588965
33566	CLLS	2020-10-05	17.19	18.14	16.95	18.06	168516	{}	\N	2020-10-12 14:31:55.593589
33567	CLLS	2020-10-06	18.24	18.79	18.05	18.11	142462	{}	\N	2020-10-12 14:31:55.59719
33568	CLLS	2020-10-07	18.11	18.6324	18.04	18.43	138498	{}	\N	2020-10-12 14:31:55.601006
33569	CLLS	2020-10-08	18.77	19.48	18.77	19.38	222533	{}	\N	2020-10-12 14:31:55.604806
33570	CLLS	2020-10-09	19.41	20.38	19.41	20.06	179214	{}	\N	2020-10-12 14:31:55.608477
33571	CLLS	2020-10-12	20.15	20.43	19.9212	19.9212	30153	{}	\N	2020-10-12 14:31:55.612146
33572	CLRBZ	2020-09-28	0.124	0.15	0.1211	0.1488	24902	{}	\N	2020-10-12 14:31:56.866231
33573	CLRBZ	2020-09-29	0.1248	0.1331	0.1201	0.1326	14737	{}	\N	2020-10-12 14:31:56.876005
33574	CLRBZ	2020-09-30	0.15	0.15	0.13	0.13	3267	{}	\N	2020-10-12 14:31:56.882494
33575	CLRBZ	2020-10-01	0.13	0.136	0.12	0.12	7065	{}	\N	2020-10-12 14:31:56.88791
33576	CLRBZ	2020-10-02	0.120481	0.1278	0.115	0.115	8632	{}	\N	2020-10-12 14:31:56.892213
33577	CLRBZ	2020-10-05	0.0891	0.125	0.0891	0.1103	24290	{}	\N	2020-10-12 14:31:56.896548
33578	CLRBZ	2020-10-06	0.1103	0.18	0.094999	0.1231	74903	{}	\N	2020-10-12 14:31:56.900778
33579	CLRBZ	2020-10-07	0.1235	0.1235	0.121	0.121	5387	{}	\N	2020-10-12 14:31:56.90473
33580	CLRBZ	2020-10-08	0.1221	0.14	0.122	0.1399	6137	{}	\N	2020-10-12 14:31:56.908792
33581	CLRBZ	2020-10-09	0.1369	0.137	0.13	0.137	3770	{}	\N	2020-10-12 14:31:56.912728
33582	CLRBZ	2020-10-12	0.137	0.137	0.099	0.101	17612	{}	\N	2020-10-12 14:31:56.916471
33583	CLVS	2020-09-28	5.89	6.22	5.86	6.025	2956110	{}	\N	2020-10-12 14:31:58.181085
33584	CLVS	2020-09-29	5.99	6.05	5.8	5.94	2906839	{}	\N	2020-10-12 14:31:58.189044
33585	CLVS	2020-09-30	5.95	6.27	5.78	5.8218	3517675	{}	\N	2020-10-12 14:31:58.194624
33586	CLVS	2020-10-01	5.83	5.94	5.67	5.93	2747225	{}	\N	2020-10-12 14:31:58.200222
33587	CLVS	2020-10-02	5.73	5.9299	5.63	5.75	2497901	{}	\N	2020-10-12 14:31:58.20505
33588	CLVS	2020-10-05	5.84	5.97	5.73	5.75	2666068	{}	\N	2020-10-12 14:31:58.209889
33589	CLVS	2020-10-06	5.81	5.9	5.57	5.61	3407339	{}	\N	2020-10-12 14:31:58.21376
33590	CLVS	2020-10-07	5.64	5.87	5.5801	5.85	3214614	{}	\N	2020-10-12 14:31:58.217942
33591	CLVS	2020-10-08	5.86	5.96	5.76	5.845	1970157	{}	\N	2020-10-12 14:31:58.221747
33592	CLVS	2020-10-09	5.8011	5.85	5.63	5.68	2383896	{}	\N	2020-10-12 14:31:58.225482
33593	CLVS	2020-10-12	5.69	5.75	5.62	5.655	424633	{}	\N	2020-10-12 14:31:58.229198
33594	CMBS	2020-09-28	55.49	55.5999	55.43	55.56	63171	{}	\N	2020-10-12 14:31:59.138512
33595	CMBS	2020-09-29	55.53	55.56	55.43	55.5	38239	{}	\N	2020-10-12 14:31:59.149962
33596	CMBS	2020-09-30	55.35	55.56	55.28	55.52	74388	{}	\N	2020-10-12 14:31:59.156546
33597	CMBS	2020-10-01	55.09	55.29	55.0806	55.19	62180	{}	\N	2020-10-12 14:31:59.162509
33598	CMBS	2020-10-02	55.3	55.6	55.15	55.59	174022	{}	\N	2020-10-12 14:31:59.168112
33599	CMBS	2020-10-05	55.58	55.59	55.1122	55.25	27663	{}	\N	2020-10-12 14:31:59.172817
33600	CMBS	2020-10-06	55.21	55.42	55.05	55.37	110856	{}	\N	2020-10-12 14:31:59.177
33601	CMBS	2020-10-07	55.29	55.6	55.15	55.18	38297	{}	\N	2020-10-12 14:31:59.181405
33602	CMBS	2020-10-08	55.36	55.43	55.15	55.17	20225	{}	\N	2020-10-12 14:31:59.185716
33603	CMBS	2020-10-09	55.3	55.36	55.09	55.09	20010	{}	\N	2020-10-12 14:31:59.189676
33604	CMBS	2020-10-12	54.99	55.2821	54.99	55.23	3644	{}	\N	2020-10-12 14:31:59.193608
33605	CMCTP	2020-09-29	23.6753	23.6753	23.6753	23.6753	103	{}	\N	2020-10-12 14:32:00.215689
33606	CMP	2020-09-28	58.64	59.37	58.15	58.96	145334	{}	\N	2020-10-12 14:32:01.504914
33607	CMP	2020-09-29	58.94	59.49	58.22	58.92	308362	{}	\N	2020-10-12 14:32:01.516123
33608	CMP	2020-09-30	59.27	60.505	58.54	59.35	476800	{}	\N	2020-10-12 14:32:01.522984
33609	CMP	2020-10-01	59.19	59.77	58.0919	59.72	505555	{}	\N	2020-10-12 14:32:01.528343
33610	CMP	2020-10-02	58.83	61.12	58.25	60.75	385033	{}	\N	2020-10-12 14:32:01.533156
33611	CMP	2020-10-05	61.93	63.1	61.93	62.42	581998	{}	\N	2020-10-12 14:32:01.537804
33612	CMP	2020-10-06	62.94	64.49	62.58	63.16	509295	{}	\N	2020-10-12 14:32:01.542075
33613	CMP	2020-10-07	64.11	65.26	62.34	63.18	667288	{}	\N	2020-10-12 14:32:01.545897
33614	CMP	2020-10-08	63.73	65.14	63.73	64.26	265253	{}	\N	2020-10-12 14:32:01.549644
33615	CMP	2020-10-09	64.5	64.89	63.45	63.84	308759	{}	\N	2020-10-12 14:32:01.553675
33616	CMP	2020-10-12	63.84	64.38	63.31	64.28	25437	{}	\N	2020-10-12 14:32:01.557492
33617	CNA	2020-09-28	29.98	30.9	29.98	30.47	287758	{}	\N	2020-10-12 14:32:02.952855
33618	CNA	2020-09-29	30.29	30.3	29.39	29.83	467698	{}	\N	2020-10-12 14:32:02.961034
33619	CNA	2020-09-30	29.9	30.3394	29.77	29.99	445474	{}	\N	2020-10-12 14:32:02.966979
33620	CNA	2020-10-01	30.23	30.27	29.68	30.06	290719	{}	\N	2020-10-12 14:32:02.972516
33621	CNA	2020-10-02	29.67	30.5	29.67	30.38	272149	{}	\N	2020-10-12 14:32:02.977657
33622	CNA	2020-10-05	30.75	31.06	30.57	30.84	488646	{}	\N	2020-10-12 14:32:02.982781
33623	CNA	2020-10-06	31.19	31.28	30.38	30.48	429847	{}	\N	2020-10-12 14:32:02.987478
33624	CNA	2020-10-07	30.65	30.91	30.4	30.72	407564	{}	\N	2020-10-12 14:32:02.991549
33625	CNA	2020-10-08	31.05	31.44	30.87	31.26	342845	{}	\N	2020-10-12 14:32:02.995883
33626	CNA	2020-10-09	31.41	31.66	30.94	30.98	416009	{}	\N	2020-10-12 14:32:03.000639
33627	CNA	2020-10-12	30.83	31.27	30.58	31.27	27562	{}	\N	2020-10-12 14:32:03.005367
33628	CNHI	2020-09-28	7.77	7.82	7.7117	7.77	1404443	{}	\N	2020-10-12 14:32:04.128394
33629	CNHI	2020-09-29	7.8	7.885	7.7852	7.85	2017393	{}	\N	2020-10-12 14:32:04.137507
33630	CNHI	2020-09-30	7.75	7.91	7.745	7.82	3659265	{}	\N	2020-10-12 14:32:04.143181
33631	CNHI	2020-10-01	7.83	7.845	7.745	7.8	3236246	{}	\N	2020-10-12 14:32:04.148456
33632	CNHI	2020-10-02	7.5	7.86	7.49	7.8	4408375	{}	\N	2020-10-12 14:32:04.153434
33633	CNHI	2020-10-05	8.02	8.1	7.99	8.07	3300527	{}	\N	2020-10-12 14:32:04.1582
33634	CNHI	2020-10-06	8.07	8.185	7.97	7.99	3072846	{}	\N	2020-10-12 14:32:04.162163
33635	CNHI	2020-10-07	8.13	8.21	8.09	8.16	3477755	{}	\N	2020-10-12 14:32:04.166459
33636	CNHI	2020-10-08	8.21	8.25	8.155	8.25	1960174	{}	\N	2020-10-12 14:32:04.170481
33637	CNHI	2020-10-09	8.45	8.485	8.38	8.41	2692628	{}	\N	2020-10-12 14:32:04.174477
33638	CNHI	2020-10-12	8.39	8.405	8.345	8.385	308177	{}	\N	2020-10-12 14:32:04.178302
33639	CNP	2020-09-28	19.17	19.295	18.99	19.02	3005379	{}	\N	2020-10-12 14:32:05.10135
33640	CNP	2020-09-29	19.11	19.26	18.78	19	5683077	{}	\N	2020-10-12 14:32:05.112935
33641	CNP	2020-09-30	19.14	19.49	19.06	19.35	10271930	{}	\N	2020-10-12 14:32:05.12026
33642	CNP	2020-10-01	19.44	19.48	19.19	19.4	6318694	{}	\N	2020-10-12 14:32:05.126329
33643	CNP	2020-10-02	19.2	19.98	19.1	19.93	11281082	{}	\N	2020-10-12 14:32:05.131156
33644	CNP	2020-10-05	20.02	20.235	19.71	20.19	6776479	{}	\N	2020-10-12 14:32:05.136239
33645	CNP	2020-10-06	20.29	20.81	20.09	20.57	10118522	{}	\N	2020-10-12 14:32:05.140719
33646	CNP	2020-10-07	20.7	20.92	20.47	20.65	7783869	{}	\N	2020-10-12 14:32:05.144853
33647	CNP	2020-10-08	20.77	21.36	20.68	21.32	9177041	{}	\N	2020-10-12 14:32:05.148943
33648	CNP	2020-10-09	21.39	21.535	21.21	21.37	10802901	{}	\N	2020-10-12 14:32:05.153049
33649	CNP	2020-10-12	21.4	21.77	21.22	21.71	2553380	{}	\N	2020-10-12 14:32:05.157654
33650	CHSCN	2020-09-28	27.14	27.24	26.86	27.2228	15950	{}	\N	2020-10-12 14:32:06.113628
33651	CHSCN	2020-09-29	27.26	27.4	27	27.32	12472	{}	\N	2020-10-12 14:32:06.126966
33652	CHSCN	2020-09-30	27.39	27.39	26.65	26.65	45519	{}	\N	2020-10-12 14:32:06.134628
33653	CHSCN	2020-10-01	26.84	27.1	26.6546	27.04	21105	{}	\N	2020-10-12 14:32:06.14073
33654	CHSCN	2020-10-02	26.97	27.25	26.715	27.25	13334	{}	\N	2020-10-12 14:32:06.145473
33655	CHSCN	2020-10-05	27.13	27.369	26.84	27.1	13867	{}	\N	2020-10-12 14:32:06.150386
33656	CHSCN	2020-10-06	27.34	27.64	27.25	27.63	27609	{}	\N	2020-10-12 14:32:06.154572
33657	CHSCN	2020-10-07	27.6	27.98	27.6	27.98	21251	{}	\N	2020-10-12 14:32:06.158601
33658	CHSCN	2020-10-08	27.69	28	27.69	28	14365	{}	\N	2020-10-12 14:32:06.162684
33659	CHSCN	2020-10-09	27.5	28.19	27.5	28.16	10620	{}	\N	2020-10-12 14:32:06.16669
33660	CHSCN	2020-10-12	28.15	28.15	27.05	27.05	10879	{}	\N	2020-10-12 14:32:06.170414
33661	CHY	2020-09-28	11.61	11.73	11.56	11.66	253998	{}	\N	2020-10-12 14:32:07.335833
33662	CHY	2020-09-29	11.64	11.73	11.62	11.71	187450	{}	\N	2020-10-12 14:32:07.348472
33663	CHY	2020-09-30	11.71	11.75	11.63	11.68	305745	{}	\N	2020-10-12 14:32:07.355342
33664	CHY	2020-10-01	11.75	11.83	11.69	11.79	202120	{}	\N	2020-10-12 14:32:07.360492
33665	CHY	2020-10-02	11.59	11.83	11.59	11.79	194049	{}	\N	2020-10-12 14:32:07.365465
33666	CHY	2020-10-05	11.79	11.95	11.79	11.91	204225	{}	\N	2020-10-12 14:32:07.369956
33667	CHY	2020-10-06	11.94	12.12	11.94	12.05	244859	{}	\N	2020-10-12 14:32:07.374404
33668	CHY	2020-10-07	12.0776	12.3399	12.0776	12.29	287633	{}	\N	2020-10-12 14:32:07.378732
33669	CHY	2020-10-08	12.33	12.3942	12.28	12.33	116227	{}	\N	2020-10-12 14:32:07.382943
33670	CHY	2020-10-09	12.38	12.5	12.36	12.5	152261	{}	\N	2020-10-12 14:32:07.386888
33671	CHY	2020-10-12	12.5	12.54	12.48	12.54	42311	{}	\N	2020-10-12 14:32:07.391117
33672	CIEN	2020-09-28	39.91	40.15	39.5	39.78	1387067	{}	\N	2020-10-12 14:32:08.295846
33673	CIEN	2020-09-29	39.78	40.47	39.64	39.99	2976426	{}	\N	2020-10-12 14:32:08.304341
33674	CIEN	2020-09-30	39.85	40.42	39.37	39.69	2817611	{}	\N	2020-10-12 14:32:08.310205
33675	CIEN	2020-10-01	39.85	40.16	39.07	39.74	2910821	{}	\N	2020-10-12 14:32:08.315383
33676	CIEN	2020-10-02	39.03	40.38	39.01	40.12	3206339	{}	\N	2020-10-12 14:32:08.320543
33677	CIEN	2020-10-05	40.56	41.41	40.45	41.36	3830989	{}	\N	2020-10-12 14:32:08.324934
33678	CIEN	2020-10-06	41.3	41.9359	40.92	41.14	5826547	{}	\N	2020-10-12 14:32:08.328975
33679	CIEN	2020-10-07	41.5	41.95	41.12	41.88	3989269	{}	\N	2020-10-12 14:32:08.332838
33680	CIEN	2020-10-08	42.27	42.86	41.8271	42.38	4496309	{}	\N	2020-10-12 14:32:08.340002
33681	CIEN	2020-10-09	42.85	43.25	42.25	43.03	4562934	{}	\N	2020-10-12 14:32:08.344628
33682	CIEN	2020-10-12	43.22	43.4	42.35	42.38	843476	{}	\N	2020-10-12 14:32:08.348534
33683	CLB	2020-09-28	16.01	16.48	15.93	16	540677	{}	\N	2020-10-12 14:32:09.877542
33684	CLB	2020-09-29	16.7	16.98	15.4938	15.95	1087164	{}	\N	2020-10-12 14:32:09.889531
33685	CLB	2020-09-30	15.94	16.26	15.19	15.26	1160444	{}	\N	2020-10-12 14:32:09.898692
33686	CLB	2020-10-01	15.05	15.23	14.2	14.29	1932885	{}	\N	2020-10-12 14:32:09.905305
33687	CLB	2020-10-02	13.83	15.49	13.82	14.88	2430588	{}	\N	2020-10-12 14:32:09.911191
33688	CLB	2020-10-05	15.21	15.79	15.21	15.47	947603	{}	\N	2020-10-12 14:32:09.916302
33689	CLB	2020-10-06	15.91	16.1499	15.55	15.84	1337174	{}	\N	2020-10-12 14:32:09.920486
33690	CLB	2020-10-07	15.95	16.4	15.72	16.2	987469	{}	\N	2020-10-12 14:32:09.925096
33691	CLB	2020-10-08	16.39	16.86	16.32	16.75	452557	{}	\N	2020-10-12 14:32:09.930085
33692	CLB	2020-10-09	16.87	16.87	16.11	16.31	996555	{}	\N	2020-10-12 14:32:09.934581
33693	CLB	2020-10-12	16.14	16.34	15.9	16.03	80101	{}	\N	2020-10-12 14:32:09.939917
33694	CLFD	2020-09-28	19.26	20.1632	19.06	19.74	83551	{}	\N	2020-10-12 14:32:10.84663
33695	CLFD	2020-09-29	19.88	20.44	19.77	20.19	74847	{}	\N	2020-10-12 14:32:10.855642
33696	CLFD	2020-09-30	20.14	20.5199	19.81	20.17	60742	{}	\N	2020-10-12 14:32:10.861819
33697	CLFD	2020-10-01	20.62	21.35	20.18	21.21	99530	{}	\N	2020-10-12 14:32:10.867601
33698	CLFD	2020-10-02	20.6	21.2699	20.2574	20.68	87843	{}	\N	2020-10-12 14:32:10.872758
33699	CLFD	2020-10-05	21.41	22.76	21.23	22.34	168996	{}	\N	2020-10-12 14:32:10.877399
33700	CLFD	2020-10-06	22.64	23.4299	22.33	22.36	123159	{}	\N	2020-10-12 14:32:10.88172
33701	CLFD	2020-10-07	22.63	23.4	22.63	23.33	78226	{}	\N	2020-10-12 14:32:10.885815
33702	CLFD	2020-10-08	23.5	23.89	23	23.48	81376	{}	\N	2020-10-12 14:32:10.889985
33703	CLFD	2020-10-09	23.48	23.96	23.06	23.33	70170	{}	\N	2020-10-12 14:32:10.89412
33704	CLFD	2020-10-12	23.33	23.4946	23.2572	23.3908	12767	{}	\N	2020-10-12 14:32:10.898349
33705	CLM	2020-09-28	10.88	10.9299	10.8	10.87	520709	{}	\N	2020-10-12 14:32:12.352277
33706	CLM	2020-09-29	10.94	10.97	10.9	10.93	436641	{}	\N	2020-10-12 14:32:12.364435
33707	CLM	2020-09-30	10.97	11.13	10.93	11.09	599546	{}	\N	2020-10-12 14:32:12.37189
33708	CLM	2020-10-01	11.1655	11.2127	11.05	11.15	574143	{}	\N	2020-10-12 14:32:12.377883
33709	CLM	2020-10-02	10.82	11.12	10.82	11.02	814209	{}	\N	2020-10-12 14:32:12.382971
33710	CLM	2020-10-05	11.15	11.2	11.082	11.2	482427	{}	\N	2020-10-12 14:32:12.387444
33711	CLM	2020-10-06	11.17	11.26	11.05	11.08	780599	{}	\N	2020-10-12 14:32:12.391797
33712	CLM	2020-10-07	11.21	11.21	11.1	11.17	549991	{}	\N	2020-10-12 14:32:12.396161
33713	CLM	2020-10-08	11.2	11.22	11.16	11.2	441868	{}	\N	2020-10-12 14:32:12.400283
33714	CLM	2020-10-09	11.24	11.25	11.17	11.19	710876	{}	\N	2020-10-12 14:32:12.404337
33715	CLM	2020-10-12	11.26	11.27	11.22	11.24	246726	{}	\N	2020-10-12 14:32:12.408436
33716	CLRG	2020-09-29	24.91	24.91	24.91	24.91	522	{}	\N	2020-10-12 14:32:13.83929
33717	CLRG	2020-10-05	25.23	25.51	25.23	25.51	772	{}	\N	2020-10-12 14:32:13.847167
33718	CLRG	2020-10-06	25.62	25.6282	25.2163	25.2163	411	{}	\N	2020-10-12 14:32:13.852988
33719	CLRG	2020-10-08	25.86	25.9	25.86	25.9	101611	{}	\N	2020-10-12 14:32:13.858227
33720	CLRG	2020-10-09	25.98	26.04	25.96	26.02	1425	{}	\N	2020-10-12 14:32:13.863535
33721	CLW	2020-09-28	36.13	37.1	35.73	36.58	95649	{}	\N	2020-10-12 14:32:14.814705
33722	CLW	2020-09-29	36.58	36.58	35.6	36.35	179382	{}	\N	2020-10-12 14:32:14.825383
33723	CLW	2020-09-30	36.49	38.08	36.24	37.94	462667	{}	\N	2020-10-12 14:32:14.832077
33724	CLW	2020-10-01	38.24	38.38	37.07	37.83	233470	{}	\N	2020-10-12 14:32:14.837921
33725	CLW	2020-10-02	37.19	39.08	36.73	38.05	284563	{}	\N	2020-10-12 14:32:14.842612
33726	CLW	2020-10-05	38.47	39.9404	38	39.12	544292	{}	\N	2020-10-12 14:32:14.847296
33727	CLW	2020-10-06	39.58	40	38.5	39.19	290049	{}	\N	2020-10-12 14:32:14.851661
33728	CLW	2020-10-07	39.94	41.545	39.75	40.68	618382	{}	\N	2020-10-12 14:32:14.855727
33729	CLW	2020-10-08	41.14	41.34	40.55	40.95	345053	{}	\N	2020-10-12 14:32:14.859487
33730	CLW	2020-10-09	41.03	42.45	40.15	40.48	526925	{}	\N	2020-10-12 14:32:14.863196
33731	CLW	2020-10-12	40.33	40.91	40.24	40.8	44704	{}	\N	2020-10-12 14:32:14.866912
33732	CMA	2020-09-28	38.07	38.88	37.82	38.55	1210043	{}	\N	2020-10-12 14:32:15.793529
33733	CMA	2020-09-29	38.26	38.29	36.85	37.73	1798983	{}	\N	2020-10-12 14:32:15.803233
33734	CMA	2020-09-30	38.07	38.86	37.83	38.25	2539040	{}	\N	2020-10-12 14:32:15.81049
33735	CMA	2020-10-01	38.2	38.87	37.75	38.08	2407961	{}	\N	2020-10-12 14:32:15.81663
33736	CMA	2020-10-02	37.28	39.495	37.12	39.33	2837639	{}	\N	2020-10-12 14:32:15.822138
33737	CMA	2020-10-05	40.01	41.89	39.92	41.83	3300605	{}	\N	2020-10-12 14:32:15.826972
33738	CMA	2020-10-06	42.95	43.475	41.07	41.3	4218742	{}	\N	2020-10-12 14:32:15.831441
33739	CMA	2020-10-07	41.93	43.54	41.93	43.1	2786434	{}	\N	2020-10-12 14:32:15.83589
33740	CMA	2020-10-08	43.75	43.82	42.65	43.1	2622994	{}	\N	2020-10-12 14:32:15.840422
33741	CMA	2020-10-09	43.44	43.7	42.17	42.34	2431014	{}	\N	2020-10-12 14:32:15.84464
33742	CMA	2020-10-12	42.25	42.5	42.095	42.39	240631	{}	\N	2020-10-12 14:32:15.848882
33743	CMF	2020-09-28	62.41	62.455	62.4	62.42	45785	{}	\N	2020-10-12 14:32:16.804967
33744	CMF	2020-09-29	62.5	62.5	62.42	62.47	68146	{}	\N	2020-10-12 14:32:16.814303
33745	CMF	2020-09-30	62.4	62.47	62.37	62.3917	99884	{}	\N	2020-10-12 14:32:16.820648
33746	CMF	2020-10-01	62.34	62.34	62.25	62.28	69656	{}	\N	2020-10-12 14:32:16.82657
33747	CMF	2020-10-02	62.32	62.35	62.18	62.24	99874	{}	\N	2020-10-12 14:32:16.832139
33748	CMF	2020-10-05	62.29	62.29	62.12	62.12	135088	{}	\N	2020-10-12 14:32:16.837064
33749	CMF	2020-10-06	62.19	62.2	61.99	62.19	115025	{}	\N	2020-10-12 14:32:16.841478
33750	CMF	2020-10-07	62.15	62.15	61.95	61.98	109672	{}	\N	2020-10-12 14:32:16.845699
33751	CMF	2020-10-08	62.05	62.05	61.93	61.95	79697	{}	\N	2020-10-12 14:32:16.850074
33752	CMF	2020-10-09	62.03	62.059	61.79	61.87	192682	{}	\N	2020-10-12 14:32:16.854553
33753	CMF	2020-10-12	61.86	61.9177	61.84	61.9177	22172	{}	\N	2020-10-12 14:32:16.858715
33754	CMO	2020-09-28	5.85	6.06	5.8385	6	764242	{}	\N	2020-10-12 14:32:18.292235
33755	CMO	2020-09-29	5.88	5.88	5.67	5.74	918331	{}	\N	2020-10-12 14:32:18.304729
33756	CMO	2020-09-30	5.74	5.81	5.605	5.62	1246708	{}	\N	2020-10-12 14:32:18.31291
33757	CMO	2020-10-01	5.62	5.73	5.52	5.57	1268047	{}	\N	2020-10-12 14:32:18.319248
33758	CMO	2020-10-02	5.44	5.725	5.41	5.69	1076121	{}	\N	2020-10-12 14:32:18.325443
33759	CMO	2020-10-05	5.74	5.77	5.68	5.75	760844	{}	\N	2020-10-12 14:32:18.332738
33760	CMO	2020-10-06	5.81	5.92	5.74	5.74	1029290	{}	\N	2020-10-12 14:32:18.337381
33761	CMO	2020-10-07	5.77	5.78	5.63	5.74	1057564	{}	\N	2020-10-12 14:32:18.34269
33762	CMO	2020-10-08	5.77	6	5.71	5.97	1095445	{}	\N	2020-10-12 14:32:18.347232
33763	CMO	2020-10-09	6	6.03	5.85	5.89	673471	{}	\N	2020-10-12 14:32:18.351463
33764	CMO	2020-10-12	5.9	5.91	5.86	5.9	85620	{}	\N	2020-10-12 14:32:18.356065
33765	CINR	2020-09-28	12.28	12.78	12.2274	12.4	10658	{}	\N	2020-10-12 14:32:19.57751
33766	CINR	2020-09-29	12.5565	12.72	12.4802	12.51	35478	{}	\N	2020-10-12 14:32:19.584658
33767	CINR	2020-09-30	12.57	12.66	12.385	12.51	23048	{}	\N	2020-10-12 14:32:19.590305
33768	CINR	2020-10-01	12.38	12.51	12.1	12.32	50462	{}	\N	2020-10-12 14:32:19.595752
33769	CINR	2020-10-02	12.11	12.64	11.8842	12.64	20040	{}	\N	2020-10-12 14:32:19.601331
33770	CINR	2020-10-05	12.65	13.0325	12.64	12.73	27394	{}	\N	2020-10-12 14:32:19.606079
33771	CINR	2020-10-06	12.86	12.92	12.43	12.74	19630	{}	\N	2020-10-12 14:32:19.610329
33772	CINR	2020-10-07	12.9899	12.9899	12.2693	12.54	18220	{}	\N	2020-10-12 14:32:19.615208
33773	CINR	2020-10-08	12.49	12.91	12.49	12.66	38160	{}	\N	2020-10-12 14:32:19.619505
33774	CINR	2020-10-09	12.68	12.68	12.3431	12.57	30788	{}	\N	2020-10-12 14:32:19.623557
33775	CINR	2020-10-12	12.5	12.5	12.42	12.45	39312	{}	\N	2020-10-12 14:32:19.627625
33776	CJJD	2020-09-28	0.9357	0.96	0.923	0.96	94995	{}	\N	2020-10-12 14:32:21.372661
33777	CJJD	2020-09-29	0.94	0.98	0.94	0.9621	181011	{}	\N	2020-10-12 14:32:21.385177
33778	CJJD	2020-09-30	0.9501	0.99	0.95	0.9698	268322	{}	\N	2020-10-12 14:32:21.392082
33779	CJJD	2020-10-01	0.9698	0.97	0.945	0.95	125785	{}	\N	2020-10-12 14:32:21.398227
33780	CJJD	2020-10-02	0.95	0.95	0.93	0.9499	95754	{}	\N	2020-10-12 14:32:21.403346
33781	CJJD	2020-10-05	0.94	0.9599	0.9351	0.9411	126769	{}	\N	2020-10-12 14:32:21.407789
33782	CJJD	2020-10-06	0.954	0.958	0.9432	0.958	226328	{}	\N	2020-10-12 14:32:21.412297
33783	CJJD	2020-10-07	0.9404	0.96	0.937	0.94	231910	{}	\N	2020-10-12 14:32:21.416593
33784	CJJD	2020-10-08	0.9381	0.9499	0.93	0.94	285723	{}	\N	2020-10-12 14:32:21.420601
33785	CJJD	2020-10-09	0.9431	0.95	0.939	0.9459	121974	{}	\N	2020-10-12 14:32:21.424369
33786	CJJD	2020-10-12	0.95	0.95	0.9401	0.947	58708	{}	\N	2020-10-12 14:32:21.428606
33787	CLCT	2020-09-28	51.36	52.72	50.96	52.58	81311	{}	\N	2020-10-12 14:32:22.651294
33788	CLCT	2020-09-29	52.12	52.57	50.9	50.97	86393	{}	\N	2020-10-12 14:32:22.66158
33789	CLCT	2020-09-30	50.97	51.24	49.1	49.49	132729	{}	\N	2020-10-12 14:32:22.669259
33790	CLCT	2020-10-01	49.65	51.9399	49.52	51.04	120188	{}	\N	2020-10-12 14:32:22.676021
33791	CLCT	2020-10-02	49.75	53.61	48.95	52.47	127997	{}	\N	2020-10-12 14:32:22.681572
33792	CLCT	2020-10-05	52.87	59.3	52.87	58.31	196201	{}	\N	2020-10-12 14:32:22.688085
33793	CLCT	2020-10-06	58.31	62.06	56.9	58.84	169394	{}	\N	2020-10-12 14:32:22.69243
33794	CLCT	2020-10-07	59.71	60.84	58.86	59.44	91145	{}	\N	2020-10-12 14:32:22.696814
33795	CLCT	2020-10-08	60.03	60.54	57.5	58.47	122380	{}	\N	2020-10-12 14:32:22.701452
33796	CLCT	2020-10-09	58.69	60.77	58.5	60.05	77118	{}	\N	2020-10-12 14:32:22.70617
33797	CLCT	2020-10-12	60.39	64.04	60.15	62.9423	34098	{}	\N	2020-10-12 14:32:22.711343
33798	CLGN	2020-09-28	8.5498	8.5498	7.67	8.2906	18102	{}	\N	2020-10-12 14:32:24.156672
33799	CLGN	2020-09-29	8.11	8.5	7.84	8.5	15585	{}	\N	2020-10-12 14:32:24.164386
33800	CLGN	2020-09-30	8.03	8.39	7.78	8.34	25067	{}	\N	2020-10-12 14:32:24.170382
33801	CLGN	2020-10-01	8.17	8.17	7.91	7.9549	4813	{}	\N	2020-10-12 14:32:24.175215
33802	CLGN	2020-10-02	8	8.03	7.75	7.93	9615	{}	\N	2020-10-12 14:32:24.180397
33803	CLGN	2020-10-05	8	8.12	7.94	7.94	5263	{}	\N	2020-10-12 14:32:24.211253
33804	CLGN	2020-10-06	8	8.6017	7.71	7.955	17464	{}	\N	2020-10-12 14:32:24.222839
33805	CLGN	2020-10-07	8.1	8.1	7.905	8	17125	{}	\N	2020-10-12 14:32:24.229601
33806	CLGN	2020-10-08	7.965	7.965	7.9	7.95	1910	{}	\N	2020-10-12 14:32:24.235272
33807	CLGN	2020-10-09	7.965	7.965	7.965	7.965	435	{}	\N	2020-10-12 14:32:24.240443
33808	CLNC	2020-09-28	4.956	5.13	4.9	5.08	282137	{}	\N	2020-10-12 14:32:25.913874
33809	CLNC	2020-09-29	5.03	5.1	4.88	4.96	619103	{}	\N	2020-10-12 14:32:25.925475
33810	CLNC	2020-09-30	4.95	5.11	4.85	4.91	439929	{}	\N	2020-10-12 14:32:25.93244
33811	CLNC	2020-10-01	4.95	5.2	4.9	5.19	1020170	{}	\N	2020-10-12 14:32:25.938512
33812	CLNC	2020-10-02	5.09	5.31	4.87	5.29	844956	{}	\N	2020-10-12 14:32:25.943424
33813	CLNC	2020-10-05	5.25	5.48	5.24	5.41	845314	{}	\N	2020-10-12 14:32:25.94942
33814	CLNC	2020-10-06	5.54	5.76	5.36	5.46	742889	{}	\N	2020-10-12 14:32:25.9543
33815	CLNC	2020-10-07	5.56	5.73	5.27	5.39	1035132	{}	\N	2020-10-12 14:32:25.958471
33816	CLNC	2020-10-08	5.49	5.86	5.46	5.8	680604	{}	\N	2020-10-12 14:32:25.962801
33817	CLNC	2020-10-09	5.77	5.88	5.49	5.6	555013	{}	\N	2020-10-12 14:32:25.967425
33818	CLNC	2020-10-12	5.56	5.595	5.41	5.53	197061	{}	\N	2020-10-12 14:32:25.972184
33819	CLRO	2020-09-28	2.32	2.4	2.3	2.32	69020	{}	\N	2020-10-12 14:32:28.082532
33820	CLRO	2020-09-29	2.35	2.35	2.21	2.28	51820	{}	\N	2020-10-12 14:32:28.09272
33821	CLRO	2020-09-30	2.28	2.38	2.28	2.29	58899	{}	\N	2020-10-12 14:32:28.100511
33822	CLRO	2020-10-01	2.3	2.345	2.27	2.29	49836	{}	\N	2020-10-12 14:32:28.106514
33823	CLRO	2020-10-02	2.21	2.3	2.21	2.25	84293	{}	\N	2020-10-12 14:32:28.111448
33824	CLRO	2020-10-05	2.22	2.36	2.22	2.22	76660	{}	\N	2020-10-12 14:32:28.116489
33825	CLRO	2020-10-06	2.255	2.29	2.11	2.16	155007	{}	\N	2020-10-12 14:32:28.121213
33826	CLRO	2020-10-07	2.17	2.25	2.09	2.13	89998	{}	\N	2020-10-12 14:32:28.125159
33827	CLRO	2020-10-08	2.14	2.23	2.12	2.15	51882	{}	\N	2020-10-12 14:32:28.129236
33828	CLRO	2020-10-09	2.23	2.27	2.165	2.19	90455	{}	\N	2020-10-12 14:32:28.133578
33829	CLRO	2020-10-12	2.2	2.21	2.175	2.21	10705	{}	\N	2020-10-12 14:32:28.137761
33830	CLWT	2020-09-28	2.54	2.5761	2.45	2.57	13002	{}	\N	2020-10-12 14:32:30.10059
33831	CLWT	2020-09-29	2.48	2.7115	2.43	2.7115	25846	{}	\N	2020-10-12 14:32:30.109467
33832	CLWT	2020-09-30	2.7	2.7283	2.62	2.67	11918	{}	\N	2020-10-12 14:32:30.115169
33833	CLWT	2020-10-01	2.63	2.85	2.63	2.705	63654	{}	\N	2020-10-12 14:32:30.120554
33834	CLWT	2020-10-02	2.69	2.7899	2.6001	2.76	7231	{}	\N	2020-10-12 14:32:30.125376
33835	CLWT	2020-10-05	2.76	2.77	2.76	2.77	623	{}	\N	2020-10-12 14:32:30.129954
33836	CLWT	2020-10-06	2.695	2.9545	2.695	2.87	56365	{}	\N	2020-10-12 14:32:30.134562
33837	CLWT	2020-10-07	2.8862	2.92	2.7	2.71	21010	{}	\N	2020-10-12 14:32:30.138609
33838	CLWT	2020-10-08	2.71	2.82	2.65	2.78	4988	{}	\N	2020-10-12 14:32:30.142396
33839	CLWT	2020-10-09	3	3.0499	2.8	2.92	57303	{}	\N	2020-10-12 14:32:30.146305
33840	CLWT	2020-10-12	2.98	2.98	2.86	2.98	3580	{}	\N	2020-10-12 14:32:30.150313
33841	CME	2020-09-28	167.83	170.125	167.58	168.93	1329394	{}	\N	2020-10-12 14:32:31.18336
33842	CME	2020-09-29	169	169.8	164.13	166.12	1655989	{}	\N	2020-10-12 14:32:31.193032
33843	CME	2020-09-30	166.6	168.91	165.94	167.31	1201188	{}	\N	2020-10-12 14:32:31.199613
33844	CME	2020-10-01	168.49	170.97	167.63	170.26	1099303	{}	\N	2020-10-12 14:32:31.205234
33845	CME	2020-10-02	168.63	170.185	168.02	168.48	936006	{}	\N	2020-10-12 14:32:31.210819
33846	CME	2020-10-05	168.29	169.87	166.33	166.94	1867577	{}	\N	2020-10-12 14:32:31.215835
33847	CME	2020-10-06	168.36	170.39	166.99	168.33	1613503	{}	\N	2020-10-12 14:32:31.21988
33848	CME	2020-10-07	168.48	170.04	167.58	169.5	1978566	{}	\N	2020-10-12 14:32:31.223733
33849	CME	2020-10-08	169.56	172.4	169.3943	170.9	1368890	{}	\N	2020-10-12 14:32:31.2277
33850	CME	2020-10-09	171.95	172.06	168.7	169.41	1049108	{}	\N	2020-10-12 14:32:31.231606
33851	CME	2020-10-12	169.18	170.52	169.18	170.4681	154009	{}	\N	2020-10-12 14:32:31.235736
33852	CMU	2020-09-28	4.2	4.21	4.185	4.205	54588	{}	\N	2020-10-12 14:32:32.302318
33853	CMU	2020-09-29	4.2	4.23	4.2	4.23	42524	{}	\N	2020-10-12 14:32:32.312577
33854	CMU	2020-09-30	4.25	4.25	4.2	4.2	149802	{}	\N	2020-10-12 14:32:32.318766
33855	CMU	2020-10-01	4.22	4.24	4.22	4.24	45406	{}	\N	2020-10-12 14:32:32.324599
33856	CMU	2020-10-02	4.23	4.25	4.215	4.25	81444	{}	\N	2020-10-12 14:32:32.329715
33857	CMU	2020-10-05	4.25	4.26	4.23	4.26	57590	{}	\N	2020-10-12 14:32:32.334426
33858	CMU	2020-10-06	4.24	4.26	4.2	4.2405	94728	{}	\N	2020-10-12 14:32:32.339617
33859	CMU	2020-10-07	4.22	4.26	4.22	4.24	129904	{}	\N	2020-10-12 14:32:32.346431
33860	CMU	2020-10-08	4.25	4.26	4.24	4.24	20110	{}	\N	2020-10-12 14:32:32.350612
33861	CMU	2020-10-09	4.23	4.236	4.22	4.23	86418	{}	\N	2020-10-12 14:32:32.354556
33862	CMU	2020-10-12	4.23	4.24	4.22	4.24	96712	{}	\N	2020-10-12 14:32:32.35841
33863	CNCR	2020-09-28	27.1	27.1	26.73	26.99	53979	{}	\N	2020-10-12 14:32:34.655633
33864	CNCR	2020-09-29	27.256	27.256	27.11	27.1921	1268	{}	\N	2020-10-12 14:32:34.665838
33865	CNCR	2020-09-30	27.36	27.51	27.36	27.46	2037	{}	\N	2020-10-12 14:32:34.673165
33866	CNCR	2020-10-01	27.37	27.6356	27.37	27.6356	2120	{}	\N	2020-10-12 14:32:34.679234
33867	CNCR	2020-10-02	27.15	27.15	26.66	26.7	14781	{}	\N	2020-10-12 14:32:34.684033
33868	CNCR	2020-10-05	26.88	28.0492	26.88	28.0492	5502	{}	\N	2020-10-12 14:32:34.688616
33869	CNCR	2020-10-06	28	28.2	27.7795	27.7795	2785	{}	\N	2020-10-12 14:32:34.692649
33870	CNCR	2020-10-07	28	28.57	28	28.57	5078	{}	\N	2020-10-12 14:32:34.696683
33871	CNCR	2020-10-08	28.675	28.75	28.4601	28.6	5584	{}	\N	2020-10-12 14:32:34.701165
33872	CNCR	2020-10-09	28.72	29.07	28.72	29.05	2774	{}	\N	2020-10-12 14:32:34.705881
33873	CNCR	2020-10-12	29.2	29.26	29.19	29.23	1018	{}	\N	2020-10-12 14:32:34.710415
33874	CNNE	2020-09-28	35.63	36.04	35.345	35.9	309129	{}	\N	2020-10-12 14:32:35.91611
33875	CNNE	2020-09-29	35.97	36.14	35.29	36.09	765555	{}	\N	2020-10-12 14:32:35.929207
33876	CNNE	2020-09-30	36.54	37.68	36.49	37.26	1270533	{}	\N	2020-10-12 14:32:35.936189
33877	CNNE	2020-10-01	37.55	38.19	37.16	38.04	1007191	{}	\N	2020-10-12 14:32:35.941935
33878	CNNE	2020-10-02	37.27	37.82	36.74	37.64	574998	{}	\N	2020-10-12 14:32:35.947231
33879	CNNE	2020-10-05	37.89	38.88	37.66	38.82	623057	{}	\N	2020-10-12 14:32:35.952257
33880	CNNE	2020-10-06	39.05	39.9	38.55	38.61	675992	{}	\N	2020-10-12 14:32:35.956489
33881	CNNE	2020-10-07	38.85	39.19	38.23	38.51	663522	{}	\N	2020-10-12 14:32:35.960716
33882	CNNE	2020-10-08	38.93	39.61	38.59	39.59	591419	{}	\N	2020-10-12 14:32:35.965405
33883	CNNE	2020-10-09	39.79	40.48	39.5888	40.43	518300	{}	\N	2020-10-12 14:32:35.969571
33884	CNNE	2020-10-12	40.45	40.86	40.01	40.805	78392	{}	\N	2020-10-12 14:32:35.974293
33885	CNXN	2020-09-28	40.83	41.76	40.83	41.33	63452	{}	\N	2020-10-12 14:32:36.936791
33886	CNXN	2020-09-29	41.22	41.66	41.22	41.44	41597	{}	\N	2020-10-12 14:32:36.946842
33887	CNXN	2020-09-30	41.6	41.96	40.96	41.06	89462	{}	\N	2020-10-12 14:32:36.953793
33888	CNXN	2020-10-01	41.38	41.7296	41.02	41.4	64475	{}	\N	2020-10-12 14:32:36.959712
33889	CNXN	2020-10-02	40.89	41.58	40.89	41.23	42666	{}	\N	2020-10-12 14:32:36.965023
33890	CNXN	2020-10-05	41.69	42.65	41.455	42	65814	{}	\N	2020-10-12 14:32:36.969817
33891	CNXN	2020-10-06	42.31	43.43	42.11	42.35	35820	{}	\N	2020-10-12 14:32:36.974668
33892	CNXN	2020-10-07	42.92	43.73	42.5907	43.46	54308	{}	\N	2020-10-12 14:32:36.979751
33893	CNXN	2020-10-08	44.01	44.145	42.88	43.15	85841	{}	\N	2020-10-12 14:32:36.984437
33894	CNXN	2020-10-09	43.41	44.66	43.28	43.81	59180	{}	\N	2020-10-12 14:32:36.98942
33895	CNXN	2020-10-12	44.07	44.07	44.07	44.07	557	{}	\N	2020-10-12 14:32:36.993869
33896	COF	2020-09-28	71.15	72.38	69.97	71.33	1682071	{}	\N	2020-10-12 14:32:38.027944
33897	COF	2020-09-29	71.19	71.64	69.75	70.45	4267452	{}	\N	2020-10-12 14:32:38.037264
33898	COF	2020-09-30	70.99	73.54	70.73	71.86	5364119	{}	\N	2020-10-12 14:32:38.043178
33899	COF	2020-10-01	72.98	75.31	71.31	72.93	5492669	{}	\N	2020-10-12 14:32:38.04875
33900	COF	2020-10-02	71.53	75.68	71.03	74.94	5181879	{}	\N	2020-10-12 14:32:38.0535
33901	COF	2020-10-05	76.26	78.21	75.69	78.04	6135811	{}	\N	2020-10-12 14:32:38.058189
33902	COF	2020-10-06	79	79.98	75.23	75.63	5515600	{}	\N	2020-10-12 14:32:38.062707
33903	COF	2020-10-07	76.87	78.78	76.87	78.24	5364922	{}	\N	2020-10-12 14:32:38.066726
33904	COF	2020-10-08	78.59	80.39	77.56	80.28	4007456	{}	\N	2020-10-12 14:32:38.070484
33905	COF	2020-10-09	81.25	81.67	79.21	79.88	3424521	{}	\N	2020-10-12 14:32:38.07456
33906	COF	2020-10-12	79.81	80.25	79.31	80.21	634883	{}	\N	2020-10-12 14:32:38.07845
33907	COM	2020-09-28	22.7893	22.86	22.669	22.86	3126	{}	\N	2020-10-12 14:32:38.992734
33908	COM	2020-09-29	22.8	22.8	22.64	22.6402	2096	{}	\N	2020-10-12 14:32:39.002311
33909	COM	2020-09-30	22.63	23.19	22.63	23	6996	{}	\N	2020-10-12 14:32:39.0085
33910	COM	2020-10-01	22.865	22.9647	22.865	22.9647	438	{}	\N	2020-10-12 14:32:39.014449
33911	COM	2020-10-02	22.64	22.865	22.64	22.865	2629	{}	\N	2020-10-12 14:32:39.019924
33912	COM	2020-10-05	23.075	23.2	23.07	23.07	4034	{}	\N	2020-10-12 14:32:39.024928
33913	COM	2020-10-06	23.22	23.43	23.22	23.33	13208	{}	\N	2020-10-12 14:32:39.02915
33914	COM	2020-10-07	23.43	23.44	23.36	23.44	10408	{}	\N	2020-10-12 14:32:39.033142
33915	COM	2020-10-08	23.6	23.6	23.4015	23.4015	1283	{}	\N	2020-10-12 14:32:39.037116
33916	COM	2020-10-09	23.5272	23.67	23.5272	23.645	3401	{}	\N	2020-10-12 14:32:39.041146
33917	COM	2020-10-12	23.66	23.68	23.5101	23.5101	4166	{}	\N	2020-10-12 14:32:39.044929
33918	CIK	2020-09-28	2.9	2.96	2.9	2.94	302369	{}	\N	2020-10-12 14:32:40.639688
33919	CIK	2020-09-29	2.95	2.97	2.94	2.96	162485	{}	\N	2020-10-12 14:32:40.649254
33920	CIK	2020-09-30	2.95	2.96	2.94	2.94	239458	{}	\N	2020-10-12 14:32:40.655958
33921	CIK	2020-10-01	2.94	2.98	2.94	2.97	209358	{}	\N	2020-10-12 14:32:40.661617
33922	CIK	2020-10-02	2.95	2.96	2.91	2.91	473067	{}	\N	2020-10-12 14:32:40.666497
33923	CIK	2020-10-05	2.92	2.97	2.91	2.96	171415	{}	\N	2020-10-12 14:32:40.671155
33924	CIK	2020-10-06	2.96	3	2.95	3	440157	{}	\N	2020-10-12 14:32:40.675208
33925	CIK	2020-10-07	3	3.01	2.98	2.985	103343	{}	\N	2020-10-12 14:32:40.679157
33926	CIK	2020-10-08	2.98	3.01	2.97	2.99	137589	{}	\N	2020-10-12 14:32:40.683138
33927	CIK	2020-10-09	3	3.03	2.995	3.03	89045	{}	\N	2020-10-12 14:32:40.687211
33928	CIK	2020-10-12	3.03	3.04	3	3.03	15642	{}	\N	2020-10-12 14:32:40.691687
33929	CKX	2020-10-02	9.31	9.31	9.31	9.31	528	{}	\N	2020-10-12 14:32:42.591915
33930	CKX	2020-10-05	9.31	9.786	9.31	9.55	2844	{}	\N	2020-10-12 14:32:42.598801
33931	CKX	2020-10-06	9.5	9.9999	9.5	9.5	2034	{}	\N	2020-10-12 14:32:42.603263
33932	CKX	2020-10-07	9.245	9.245	9.245	9.245	547	{}	\N	2020-10-12 14:32:42.607672
33933	CKX	2020-10-08	9.3973	9.4	9.3973	9.4	902	{}	\N	2020-10-12 14:32:42.611413
33934	CLF	2020-09-28	6.34	6.75	6.155	6.56	28212289	{}	\N	2020-10-12 14:32:44.34749
33935	CLF	2020-09-29	6.56	6.79	6.31	6.57	32259127	{}	\N	2020-10-12 14:32:44.357438
33936	CLF	2020-09-30	6.56	6.72	6.34	6.42	26757505	{}	\N	2020-10-12 14:32:44.365214
33937	CLF	2020-10-01	6.48	6.73	6.39	6.7	24610486	{}	\N	2020-10-12 14:32:44.370952
33938	CLF	2020-10-02	6.48	7.11	6.48	7.03	28688451	{}	\N	2020-10-12 14:32:44.376023
33939	CLF	2020-10-05	7.13	7.525	7.13	7.44	24156250	{}	\N	2020-10-12 14:32:44.380921
33940	CLF	2020-10-06	7.48	7.675	7.175	7.22	29700249	{}	\N	2020-10-12 14:32:44.385461
33941	CLF	2020-10-07	7.43	7.82	7.36	7.7	34060656	{}	\N	2020-10-12 14:32:44.389802
33942	CLF	2020-10-08	7.8	7.92	7.69	7.72	23310668	{}	\N	2020-10-12 14:32:44.394292
33943	CLF	2020-10-09	7.82	7.91	7.45	7.55	24744111	{}	\N	2020-10-12 14:32:44.398587
33944	CLF	2020-10-12	7.54	7.62	7.46	7.55	2374790	{}	\N	2020-10-12 14:32:44.403095
33945	CLPR	2020-09-28	6.14	6.52	6.14	6.35	76485	{}	\N	2020-10-12 14:32:45.432882
33946	CLPR	2020-09-29	6.35	6.35	6.075	6.12	58958	{}	\N	2020-10-12 14:32:45.441652
33947	CLPR	2020-09-30	6.17	6.23	6	6.05	146869	{}	\N	2020-10-12 14:32:45.4477
33948	CLPR	2020-10-01	6.05	6.1699	5.98	6.08	76746	{}	\N	2020-10-12 14:32:45.452839
33949	CLPR	2020-10-02	5.97	6.14	5.95	6.1	95906	{}	\N	2020-10-12 14:32:45.457668
33950	CLPR	2020-10-05	6.21	6.4	6.14	6.31	181490	{}	\N	2020-10-12 14:32:45.462321
33951	CLPR	2020-10-06	6.46	6.73	6.4201	6.52	220280	{}	\N	2020-10-12 14:32:45.466379
33952	CLPR	2020-10-07	6.57	6.64	6.39	6.54	108867	{}	\N	2020-10-12 14:32:45.470228
33953	CLPR	2020-10-08	6.62	6.75	6.58	6.69	88769	{}	\N	2020-10-12 14:32:45.474046
33954	CLPR	2020-10-09	6.75	6.75	6.56	6.59	63992	{}	\N	2020-10-12 14:32:45.478292
33955	CLPR	2020-10-12	6.54	6.54	6.451	6.505	12444	{}	\N	2020-10-12 14:32:45.482434
33956	CMCM	2020-09-28	1.85	1.94	1.85	1.92	59246	{}	\N	2020-10-12 14:32:46.882876
33957	CMCM	2020-09-29	1.92	1.93	1.89	1.89	248196	{}	\N	2020-10-12 14:32:46.895566
33958	CMCM	2020-09-30	1.91	2.11	1.91	2.06	979190	{}	\N	2020-10-12 14:32:46.902208
33959	CMCM	2020-10-01	2.08	2.13	2	2.05	359808	{}	\N	2020-10-12 14:32:46.907595
33960	CMCM	2020-10-02	2	2.0717	1.97	2.02	388940	{}	\N	2020-10-12 14:32:46.912514
33961	CMCM	2020-10-05	2.02	2.0719	1.95	1.95	293690	{}	\N	2020-10-12 14:32:46.917204
33962	CMCM	2020-10-06	1.97	2.05	1.94	1.97	196968	{}	\N	2020-10-12 14:32:46.921573
33963	CMCM	2020-10-07	1.97	2.02	1.96	1.98	93339	{}	\N	2020-10-12 14:32:46.925502
33964	CMCM	2020-10-08	2	2.03	1.96	2	136534	{}	\N	2020-10-12 14:32:46.929296
33965	CMCM	2020-10-09	1.99	2.04	1.99	2.02	171974	{}	\N	2020-10-12 14:32:46.933124
33966	CMCM	2020-10-12	2.04	2.05	2	2.01	99599	{}	\N	2020-10-12 14:32:46.936962
33967	CMG	2020-09-28	1263.16	1274.94	1243.5	1263.35	196197	{}	\N	2020-10-12 14:32:48.105391
33968	CMG	2020-09-29	1263.26	1266.7	1245.13	1261.61	354955	{}	\N	2020-10-12 14:32:48.115766
33969	CMG	2020-09-30	1261.8	1272	1234.37	1243.71	562000	{}	\N	2020-10-12 14:32:48.12195
33970	CMG	2020-10-01	1255.57	1265	1231	1238.02	682549	{}	\N	2020-10-12 14:32:48.126893
33971	CMG	2020-10-02	1214.3	1265.715	1214	1254.8	551348	{}	\N	2020-10-12 14:32:48.131275
33972	CMG	2020-10-05	1258.07	1264	1242	1253.01	405092	{}	\N	2020-10-12 14:32:48.135373
33973	CMG	2020-10-06	1252.48	1291.983	1248.03	1256.41	916800	{}	\N	2020-10-12 14:32:48.139563
33974	CMG	2020-10-07	1272.6	1295.345	1266.06	1278.68	475248	{}	\N	2020-10-12 14:32:48.143083
33975	CMG	2020-10-08	1287.92	1288	1260	1281.75	477608	{}	\N	2020-10-12 14:32:48.146545
33976	CMG	2020-10-09	1287.48	1291.9962	1276.895	1281.99	482971	{}	\N	2020-10-12 14:32:48.14993
33977	CMG	2020-10-12	1291.19	1292.12	1279.82	1279.82	90358	{}	\N	2020-10-12 14:32:48.153328
33978	CMRE	2020-09-28	5.5684	5.84	5.47	5.79	1078626	{}	\N	2020-10-12 14:32:49.12812
33979	CMRE	2020-09-29	5.84	6.425	5.83	6.16	3338805	{}	\N	2020-10-12 14:32:49.139834
33980	CMRE	2020-09-30	6.16	6.26	6.06	6.07	1446573	{}	\N	2020-10-12 14:32:49.146702
33981	CMRE	2020-10-01	6.1	6.44	6.1	6.38	2088899	{}	\N	2020-10-12 14:32:49.152144
33982	CMRE	2020-10-02	6.17	6.59	6.11	6.52	1057964	{}	\N	2020-10-12 14:32:49.156868
33983	CMRE	2020-10-05	6.57	6.74	6.36	6.38	914260	{}	\N	2020-10-12 14:32:49.161466
33984	CMRE	2020-10-06	6.42	6.43	6.18	6.18	1504682	{}	\N	2020-10-12 14:32:49.16582
33985	CMRE	2020-10-07	6.3	6.38	6.16	6.34	1405363	{}	\N	2020-10-12 14:32:49.170087
33986	CMRE	2020-10-08	6.34	6.56	6.34	6.53	899943	{}	\N	2020-10-12 14:32:49.173992
33987	CMRE	2020-10-09	6.6	6.67	6.45	6.49	748935	{}	\N	2020-10-12 14:32:49.177859
33988	CMRE	2020-10-12	6.48	6.56	6.38	6.54	146472	{}	\N	2020-10-12 14:32:49.181529
33989	CMTL	2020-09-28	13.63	14.31	13.63	14.03	258340	{}	\N	2020-10-12 14:32:50.517782
33990	CMTL	2020-09-29	13.96	14.16	13.82	14.05	548532	{}	\N	2020-10-12 14:32:50.530215
33991	CMTL	2020-09-30	14.7	14.8893	13.59	14	898882	{}	\N	2020-10-12 14:32:50.538127
33992	CMTL	2020-10-01	13.94	14.15	13.4	13.87	343471	{}	\N	2020-10-12 14:32:50.543131
33993	CMTL	2020-10-02	13.36	14.18	13.36	13.97	549009	{}	\N	2020-10-12 14:32:50.548104
33994	CMTL	2020-10-05	14.22	15.71	14.22	15.54	1102143	{}	\N	2020-10-12 14:32:50.552362
33995	CMTL	2020-10-06	15.48	16.5	15.05	16	653115	{}	\N	2020-10-12 14:32:50.556474
33996	CMTL	2020-10-07	16.3	16.57	15.3	15.36	625163	{}	\N	2020-10-12 14:32:50.560332
33997	CMTL	2020-10-08	15.75	15.75	15.195	15.51	313916	{}	\N	2020-10-12 14:32:50.564163
33998	CMTL	2020-10-09	15.74	16.59	15.6271	16.43	297636	{}	\N	2020-10-12 14:32:50.567953
33999	CMTL	2020-10-12	16.34	16.41	16.03	16.16	39692	{}	\N	2020-10-12 14:32:50.571629
34000	CNCE	2020-09-28	9.02	9.46	9	9.32	112646	{}	\N	2020-10-12 14:32:51.899669
34001	CNCE	2020-09-29	9.29	9.89	9.19	9.65	75674	{}	\N	2020-10-12 14:32:51.910357
34002	CNCE	2020-09-30	9.63	9.93	9.46	9.82	200244	{}	\N	2020-10-12 14:32:51.916852
34003	CNCE	2020-10-01	9.72	10.37	9.72	10.27	109900	{}	\N	2020-10-12 14:32:51.922216
34004	CNCE	2020-10-02	10.12	10.75	10.1	10.68	95435	{}	\N	2020-10-12 14:32:51.926887
34005	CNCE	2020-10-05	10.92	11.27	10.89	11.21	89705	{}	\N	2020-10-12 14:32:51.93131
34006	CNCE	2020-10-06	11.2	11.72	11.1041	11.36	80262	{}	\N	2020-10-12 14:32:51.936031
34007	CNCE	2020-10-07	11.355	11.69	10.98	11.45	586243	{}	\N	2020-10-12 14:32:51.939808
34008	CNCE	2020-10-08	11.64	11.71	11.2	11.3	106155	{}	\N	2020-10-12 14:32:51.943458
34009	CNCE	2020-10-09	11.44	11.556	10.859	11.14	117477	{}	\N	2020-10-12 14:32:51.947299
34010	CNCE	2020-10-12	11.17	11.27	11	11.27	21963	{}	\N	2020-10-12 14:32:51.951007
34011	CNMD	2020-09-28	78.27	79.9	77.98	78.12	166811	{}	\N	2020-10-12 14:32:52.950396
34012	CNMD	2020-09-29	78.13	78.49	75.77	77.12	465456	{}	\N	2020-10-12 14:32:52.960921
34013	CNMD	2020-09-30	77.64	79.69	77.64	78.67	532441	{}	\N	2020-10-12 14:32:52.96753
34014	CNMD	2020-10-01	79.04	80.08	77.05	78.3	433381	{}	\N	2020-10-12 14:32:52.972846
34015	CNMD	2020-10-02	77	79.55	76.5	79.07	548868	{}	\N	2020-10-12 14:32:52.977605
34016	CNMD	2020-10-05	80.05	81.065	79.09	80.81	368833	{}	\N	2020-10-12 14:32:52.982492
34017	CNMD	2020-10-06	82.31	84.38	80.83	82.45	581772	{}	\N	2020-10-12 14:32:52.987234
34018	CNMD	2020-10-07	82.75	84.57	82.59	83.99	472203	{}	\N	2020-10-12 14:32:52.991059
34019	CNMD	2020-10-08	85.21	85.27	83.49	84.04	217730	{}	\N	2020-10-12 14:32:52.994914
34020	CNMD	2020-10-09	84.75	86.06	83.96	85.95	301681	{}	\N	2020-10-12 14:32:52.999245
34021	CNMD	2020-10-12	86.22	87.14	85.92	87.14	32419	{}	\N	2020-10-12 14:32:53.003054
34022	CO	2020-09-28	3.37	3.47	3.37	3.39	58643	{}	\N	2020-10-12 14:32:54.501032
34023	CO	2020-09-29	3.42	3.45	3.3987	3.43	65823	{}	\N	2020-10-12 14:32:54.510866
34024	CO	2020-09-30	3.45	3.65	3.43	3.54	423447	{}	\N	2020-10-12 14:32:54.517402
34025	CO	2020-10-01	3.59	3.59	3.46	3.49	159598	{}	\N	2020-10-12 14:32:54.52282
34026	CO	2020-10-02	3.45	3.55	3.422	3.52	76836	{}	\N	2020-10-12 14:32:54.527368
34027	CO	2020-10-05	3.52	3.73	3.51	3.68	259712	{}	\N	2020-10-12 14:32:54.531865
34028	CO	2020-10-06	3.65	3.78	3.6467	3.69	221648	{}	\N	2020-10-12 14:32:54.536302
34029	CO	2020-10-07	3.7	3.715	3.62	3.62	52752	{}	\N	2020-10-12 14:32:54.540221
34030	CO	2020-10-08	3.66	3.68	3.62	3.63	188196	{}	\N	2020-10-12 14:32:54.54395
34031	CO	2020-10-09	3.62	3.63	3.45	3.45	224396	{}	\N	2020-10-12 14:32:54.547647
34032	CO	2020-10-12	3.5	3.6	3.49	3.5	41026	{}	\N	2020-10-12 14:32:54.55124
34033	CLX	2020-09-28	212.59	214.25	210.47	211.97	914891	{}	\N	2020-10-12 14:32:56.983794
34034	CLX	2020-09-29	212.72	214.2	210.11	210.41	2120567	{}	\N	2020-10-12 14:32:56.994571
34035	CLX	2020-09-30	210.16	211.36	208.03	210.17	2748934	{}	\N	2020-10-12 14:32:57.001688
34036	CLX	2020-10-01	209.53	212.35	207.52	211.65	2270762	{}	\N	2020-10-12 14:32:57.007519
34037	CLX	2020-10-02	211.68	216	208.29	209.22	2599442	{}	\N	2020-10-12 14:32:57.012909
34038	CLX	2020-10-05	209.02	210.05	207.825	209.11	2146452	{}	\N	2020-10-12 14:32:57.017682
34039	CLX	2020-10-06	209.19	212.095	208.65	211.15	1977539	{}	\N	2020-10-12 14:32:57.022357
34040	CLX	2020-10-07	211.06	217.5	211	215.13	4044756	{}	\N	2020-10-12 14:32:57.026735
34041	CLX	2020-10-08	215.07	215.795	211.96	212.86	1710277	{}	\N	2020-10-12 14:32:57.031268
34042	CLX	2020-10-09	213.05	216.32	213.05	214.79	2057137	{}	\N	2020-10-12 14:32:57.035397
34043	CLX	2020-10-12	215.63	216.65	214.8	215.46	279438	{}	\N	2020-10-12 14:32:57.039223
34044	CMD	2020-09-28	46	47.08	44.96	45.06	567884	{}	\N	2020-10-12 14:32:58.21827
34045	CMD	2020-09-29	45.29	46	44.42	44.67	377365	{}	\N	2020-10-12 14:32:58.227706
34046	CMD	2020-09-30	44.78	45.27	43.86	43.94	566894	{}	\N	2020-10-12 14:32:58.233808
34047	CMD	2020-10-01	44.58	45.58	43.65	44.04	518020	{}	\N	2020-10-12 14:32:58.239559
34048	CMD	2020-10-02	42.8	43.59	42.66	42.85	818167	{}	\N	2020-10-12 14:32:58.244689
34049	CMD	2020-10-05	43.49	44.36	42.06	43.1	618701	{}	\N	2020-10-12 14:32:58.249182
34050	CMD	2020-10-06	43.73	45.82	43.73	44.26	535064	{}	\N	2020-10-12 14:32:58.253383
34051	CMD	2020-10-07	44.65	45.83	44.423	45.76	499181	{}	\N	2020-10-12 14:32:58.257112
34052	CMD	2020-10-08	46.45	48.04	45.69	47.4	782261	{}	\N	2020-10-12 14:32:58.261446
34053	CMD	2020-10-09	47.83	47.98	46.28	47.36	600793	{}	\N	2020-10-12 14:32:58.265319
34054	CMD	2020-10-12	47.38	48.13	47.12	48.13	19621	{}	\N	2020-10-12 14:32:58.269181
34055	CMT	2020-09-28	9.35	9.35	8.72	8.9362	41588	{}	\N	2020-10-12 14:33:00.599384
34056	CMT	2020-09-29	8.21	9	7.97	8.8	61400	{}	\N	2020-10-12 14:33:00.607407
34057	CMT	2020-09-30	8.9559	9	8.8	8.86	22552	{}	\N	2020-10-12 14:33:00.6132
34058	CMT	2020-10-01	8.85	9.01	8.7274	8.91	9296	{}	\N	2020-10-12 14:33:00.618458
34059	CMT	2020-10-02	8.91	9.27	8.7301	9.21	18101	{}	\N	2020-10-12 14:33:00.623001
34060	CMT	2020-10-05	9.085	9.3199	8.71	8.73	31571	{}	\N	2020-10-12 14:33:00.627658
34061	CMT	2020-10-06	8.73	9.11	8.73	8.75	18142	{}	\N	2020-10-12 14:33:00.632101
34062	CMT	2020-10-07	9.1035	9.15	8.65	9	21857	{}	\N	2020-10-12 14:33:00.636127
34063	CMT	2020-10-08	8.895	9.28	8.88	9.28	11174	{}	\N	2020-10-12 14:33:00.640109
34064	CMT	2020-10-09	9.22	9.26	8.8	9.09	51929	{}	\N	2020-10-12 14:33:00.64414
34065	CMT	2020-10-12	9.54	9.54	9.09	9.1999	1577	{}	\N	2020-10-12 14:33:00.648259
34066	CNSL	2020-09-28	5.68	5.85	5.68	5.84	365318	{}	\N	2020-10-12 14:33:02.412003
34067	CNSL	2020-09-29	5.81	5.86	5.69	5.72	327974	{}	\N	2020-10-12 14:33:02.419433
34068	CNSL	2020-09-30	5.7244	5.84	5.555	5.69	525557	{}	\N	2020-10-12 14:33:02.424712
34069	CNSL	2020-10-01	5.71	5.795	5.615	5.73	501445	{}	\N	2020-10-12 14:33:02.429598
34070	CNSL	2020-10-02	5.65	5.95	5.55	5.82	300882	{}	\N	2020-10-12 14:33:02.433855
34071	CNSL	2020-10-05	5.86	5.99	5.77	5.79	280257	{}	\N	2020-10-12 14:33:02.438235
34072	CNSL	2020-10-06	5.9253	5.99	5.74	5.785	385726	{}	\N	2020-10-12 14:33:02.442287
34073	CNSL	2020-10-07	5.79	5.84	5.63	5.75	457619	{}	\N	2020-10-12 14:33:02.445992
34074	CNSL	2020-10-08	5.81	5.88	5.75	5.8	274055	{}	\N	2020-10-12 14:33:02.44962
34075	CNSL	2020-10-09	5.85	5.95	5.82	5.88	278255	{}	\N	2020-10-12 14:33:02.453933
34076	CNSL	2020-10-12	5.85	5.97	5.85	5.94	14957	{}	\N	2020-10-12 14:33:02.45755
34077	CODA	2020-09-28	5.5	5.55	5.35	5.49	11385	{}	\N	2020-10-12 14:33:05.445481
34078	CODA	2020-09-29	5.59	5.62	5.37	5.48	17369	{}	\N	2020-10-12 14:33:05.457113
34079	CODA	2020-09-30	5.44	5.6	5.3995	5.53	18121	{}	\N	2020-10-12 14:33:05.463039
34080	CODA	2020-10-01	5.51	5.671	5.5	5.54	11869	{}	\N	2020-10-12 14:33:05.467598
34081	CODA	2020-10-02	5.34	5.66	5.34	5.58	21589	{}	\N	2020-10-12 14:33:05.472303
34082	CODA	2020-10-05	5.67	6.04	5.6001	5.91	32003	{}	\N	2020-10-12 14:33:05.476324
34083	CODA	2020-10-06	6	6.09	5.8901	6.03	5705	{}	\N	2020-10-12 14:33:05.480211
34084	CODA	2020-10-07	5.95	6.05	5.8	6.05	14187	{}	\N	2020-10-12 14:33:05.483838
34085	CODA	2020-10-08	6	6.1	5.95	6.08	12892	{}	\N	2020-10-12 14:33:05.487252
34086	CODA	2020-10-09	6.1	6.1	5.8424	6.08	17115	{}	\N	2020-10-12 14:33:05.490647
34087	CODA	2020-10-12	6.05	6.055	6.02	6.055	1801	{}	\N	2020-10-12 14:33:05.494021
34088	COLB	2020-09-28	23.5	24.01	23.07	23.76	246063	{}	\N	2020-10-12 14:33:07.237354
34089	COLB	2020-09-29	23.7	23.71	23.17	23.6	189730	{}	\N	2020-10-12 14:33:07.249279
34090	COLB	2020-09-30	23.87	24.255	23.66	23.85	311889	{}	\N	2020-10-12 14:33:07.255452
34091	COLB	2020-10-01	23.74	24.09	23.38	24.09	280087	{}	\N	2020-10-12 14:33:07.261269
34092	COLB	2020-10-02	23.69	24.98	23.47	24.75	206580	{}	\N	2020-10-12 14:33:07.266665
34093	COLB	2020-10-05	24.99	25.43	24.765	25.35	240022	{}	\N	2020-10-12 14:33:07.271703
34094	COLB	2020-10-06	25.65	26.46	25.47	25.52	301716	{}	\N	2020-10-12 14:33:07.276305
34095	COLB	2020-10-07	25.77	26.44	25.62	26.04	417470	{}	\N	2020-10-12 14:33:07.280646
34096	COLB	2020-10-08	26.56	26.65	26.07	26.49	197081	{}	\N	2020-10-12 14:33:07.285115
34097	COLB	2020-10-09	26.59	26.89	26.04	26.09	266783	{}	\N	2020-10-12 14:33:07.289443
34098	COLB	2020-10-12	26.04	26.46	26.01	26.46	21301	{}	\N	2020-10-12 14:33:07.293629
34099	COR	2020-09-28	119.02	119.56	118.41	119.56	330053	{}	\N	2020-10-12 14:33:08.459854
34100	COR	2020-09-29	118.85	121.36	118.85	120.22	620505	{}	\N	2020-10-12 14:33:08.470016
34101	COR	2020-09-30	120.85	121.03	118.28	118.88	871940	{}	\N	2020-10-12 14:33:08.475805
34102	COR	2020-10-01	119.04	120.26	118.44	119.73	697204	{}	\N	2020-10-12 14:33:08.480593
34103	COR	2020-10-02	118.95	123.19	118.02	122.76	711683	{}	\N	2020-10-12 14:33:08.48462
34104	COR	2020-10-05	122.81	124.92	120.81	124.47	536778	{}	\N	2020-10-12 14:33:08.488934
34105	COR	2020-10-06	125.09	126.58	123.41	125.27	597337	{}	\N	2020-10-12 14:33:08.49285
34106	COR	2020-10-07	125.44	126.08	123.7	124.9	760873	{}	\N	2020-10-12 14:33:08.496589
34107	COR	2020-10-08	125.83	127.85	124.98	127.8	336320	{}	\N	2020-10-12 14:33:08.500051
34108	COR	2020-10-09	128.41	128.41	126.61	127.19	664594	{}	\N	2020-10-12 14:33:08.503574
34109	COR	2020-10-12	127.25	127.96	126.86	127.59	65120	{}	\N	2020-10-12 14:33:08.507049
34110	CPE	2020-09-28	4.94	5.325	4.83	5.16	3433018	{}	\N	2020-10-12 14:33:10.208528
34111	CPE	2020-09-29	5.15	5.19	4.9	4.93	5210377	{}	\N	2020-10-12 14:33:10.218784
34112	CPE	2020-09-30	4.9	5.049	4.78	4.82	4014376	{}	\N	2020-10-12 14:33:10.225401
34113	CPE	2020-10-01	5.31	5.49	4.77	4.81	11119196	{}	\N	2020-10-12 14:33:10.23046
34114	CPE	2020-10-02	4.58	5.02	4.55	4.97	6406515	{}	\N	2020-10-12 14:33:10.235328
34115	CPE	2020-10-05	5.08	5.1215	4.815	5.05	5372335	{}	\N	2020-10-12 14:33:10.239604
34116	CPE	2020-10-06	5.17	5.275	4.823	4.86	5755410	{}	\N	2020-10-12 14:33:10.243838
34117	CPE	2020-10-07	4.91	5.15	4.85	5.11	4238781	{}	\N	2020-10-12 14:33:10.247793
34118	CPE	2020-10-08	5.18	5.55	5.08	5.55	6228005	{}	\N	2020-10-12 14:33:10.251455
34119	CPE	2020-10-09	5.6	5.75	5.24	5.25	5042246	{}	\N	2020-10-12 14:33:10.255053
34120	CPE	2020-10-12	5.11	5.13	4.96	5.04	1488794	{}	\N	2020-10-12 14:33:10.259017
34121	CPRT	2020-09-28	105.61	106.5	105.4	106.16	982066	{}	\N	2020-10-12 14:33:11.396192
34122	CPRT	2020-09-29	106.24	106.82	105.26	105.74	549943	{}	\N	2020-10-12 14:33:11.404187
34123	CPRT	2020-09-30	105.86	107.01	104.63	105.17	1035859	{}	\N	2020-10-12 14:33:11.412778
34124	CPRT	2020-10-01	106.18	107.48	105.86	107.22	1044130	{}	\N	2020-10-12 14:33:11.421408
34125	CPRT	2020-10-02	105.66	107.66	105.66	107.34	857543	{}	\N	2020-10-12 14:33:11.428341
34126	CPRT	2020-10-05	108.95	109.96	108.185	109.69	1193008	{}	\N	2020-10-12 14:33:11.433115
34127	CPRT	2020-10-06	109.22	112.02	109.08	109.68	962896	{}	\N	2020-10-12 14:33:11.436977
34128	CPRT	2020-10-07	110.78	111.74	110.57	111.32	1086128	{}	\N	2020-10-12 14:33:11.44064
34129	CPRT	2020-10-08	111.56	114.01	111.43	114	1180227	{}	\N	2020-10-12 14:33:11.445518
34130	CPRT	2020-10-09	114.415	116.45	114.28	115.21	900401	{}	\N	2020-10-12 14:33:11.44955
34131	CPRT	2020-10-12	116.42	116.63	114.76	115.75	154646	{}	\N	2020-10-12 14:33:11.453425
34132	CPT	2020-09-28	91.79	92.495	90.9	90.97	593228	{}	\N	2020-10-12 14:33:12.436989
34133	CPT	2020-09-29	89.5	90.29	88.39	89.42	1232255	{}	\N	2020-10-12 14:33:12.44873
34134	CPT	2020-09-30	90.12	90.43	88.1	88.98	1534457	{}	\N	2020-10-12 14:33:12.454812
34135	CPT	2020-10-01	89.15	92.02	88.87	91.99	1155962	{}	\N	2020-10-12 14:33:12.459694
34136	CPT	2020-10-02	90.74	95.23	89.77	94.84	1592434	{}	\N	2020-10-12 14:33:12.464293
34137	CPT	2020-10-05	95.59	96.61	94.465	96.56	1628980	{}	\N	2020-10-12 14:33:12.468396
34138	CPT	2020-10-06	97.08	98.01	95.66	96.87	1958852	{}	\N	2020-10-12 14:33:12.472939
34139	CPT	2020-10-07	97.36	97.6408	95.79	96.5	1153090	{}	\N	2020-10-12 14:33:12.47684
34140	CPT	2020-10-08	97.02	98.14	96.36	98.06	815767	{}	\N	2020-10-12 14:33:12.480573
34141	CPT	2020-10-09	98.56	98.9899	96.12	96.32	1079671	{}	\N	2020-10-12 14:33:12.484671
34142	CPT	2020-10-12	96.57	96.89	96.03	96.84	154618	{}	\N	2020-10-12 14:33:12.488775
34143	CRBP	2020-09-28	1.66	1.67	1.52	1.64	3526972	{}	\N	2020-10-12 14:33:13.507841
34144	CRBP	2020-09-29	1.67	2.05	1.6586	1.97	8094635	{}	\N	2020-10-12 14:33:13.515358
34145	CRBP	2020-09-30	1.99	2.0736	1.77	1.8	4955493	{}	\N	2020-10-12 14:33:13.522293
34146	CRBP	2020-10-01	1.81	1.86	1.71	1.85	2595085	{}	\N	2020-10-12 14:33:13.52968
34147	CRBP	2020-10-02	1.78	1.915	1.72	1.8	2678132	{}	\N	2020-10-12 14:33:13.53618
34148	CRBP	2020-10-05	1.77	1.89	1.76	1.89	2713662	{}	\N	2020-10-12 14:33:13.541758
34149	CRBP	2020-10-06	1.11	1.28	1.04	1.21	25605479	{}	\N	2020-10-12 14:33:13.548433
34150	CRBP	2020-10-07	1.21	1.27	1.16	1.21	7454374	{}	\N	2020-10-12 14:33:13.555046
34151	CRBP	2020-10-08	1.18	1.26	1.16	1.23	4747568	{}	\N	2020-10-12 14:33:13.56142
34152	CRBP	2020-10-09	1.25	1.26	1.19	1.2	3539576	{}	\N	2020-10-12 14:33:13.567243
34153	CRBP	2020-10-12	1.22	1.22	1.15	1.175	1106528	{}	\N	2020-10-12 14:33:13.573883
34154	FFIN	2020-09-28	27.73	28.25	27.425	28.23	450372	{}	\N	2020-10-12 14:33:19.137114
34155	FFIN	2020-09-29	28.12	28.18	27.48	27.82	292513	{}	\N	2020-10-12 14:33:19.14617
34156	FFIN	2020-09-30	28.04	28.41	27.72	27.91	351441	{}	\N	2020-10-12 14:33:19.15198
34157	FFIN	2020-10-01	27.81	28.2854	27.49	28.09	412277	{}	\N	2020-10-12 14:33:19.156826
34158	FFIN	2020-10-02	27.61	28.925	27.61	28.66	282849	{}	\N	2020-10-12 14:33:19.161554
34159	FFIN	2020-10-05	28.99	29.92	28.88	29.85	363525	{}	\N	2020-10-12 14:33:19.165383
34160	FFIN	2020-10-06	30	31.08	29.81	29.86	467400	{}	\N	2020-10-12 14:33:19.16933
34161	FFIN	2020-10-07	30.13	30.92	30.07	30.61	386669	{}	\N	2020-10-12 14:33:19.173226
34162	FFIN	2020-10-08	30.91	31.75	30.61	31.13	883049	{}	\N	2020-10-12 14:33:19.177048
34163	FFIN	2020-10-09	31.14	31.405	30.89	30.93	234177	{}	\N	2020-10-12 14:33:19.181042
34164	FFIN	2020-10-12	30.8	31.27	30.74	31.2414	29790	{}	\N	2020-10-12 14:33:19.184874
34165	CLSN	2020-09-28	0.765	0.77	0.7225	0.7318	324424	{}	\N	2020-10-12 14:33:21.206767
34166	CLSN	2020-09-29	0.7365	0.75	0.7159	0.7291	488637	{}	\N	2020-10-12 14:33:21.217607
34167	CLSN	2020-09-30	0.7387	0.758	0.7158	0.7321	304213	{}	\N	2020-10-12 14:33:21.22411
34168	CLSN	2020-10-01	0.7389	0.749	0.703	0.7177	368603	{}	\N	2020-10-12 14:33:21.230243
34169	CLSN	2020-10-02	0.7103	0.729	0.701	0.7139	290300	{}	\N	2020-10-12 14:33:21.235486
34170	CLSN	2020-10-05	0.705	0.725	0.69	0.6952	441678	{}	\N	2020-10-12 14:33:21.240428
34171	CLSN	2020-10-06	0.7	0.73	0.6954	0.7138	439129	{}	\N	2020-10-12 14:33:21.244628
34172	CLSN	2020-10-07	0.7077	0.7248	0.69	0.6982	560726	{}	\N	2020-10-12 14:33:21.248688
34173	CLSN	2020-10-08	0.6902	0.724	0.6902	0.7003	533987	{}	\N	2020-10-12 14:33:21.252836
34174	CLSN	2020-10-09	0.695	0.7175	0.6801	0.693	650897	{}	\N	2020-10-12 14:33:21.25682
34175	CLSN	2020-10-12	0.71	0.7114	0.681	0.681	209054	{}	\N	2020-10-12 14:33:21.260854
34176	CLXT	2020-09-28	6.2	6.25	5.85	5.92	46505	{}	\N	2020-10-12 14:33:25.088081
34177	CLXT	2020-09-29	5.89	5.93	5.59	5.74	73923	{}	\N	2020-10-12 14:33:25.100276
34178	CLXT	2020-09-30	5.71	5.7502	5.45	5.49	37140	{}	\N	2020-10-12 14:33:25.108321
34179	CLXT	2020-10-01	5.615	5.75	5.35	5.71	60178	{}	\N	2020-10-12 14:33:25.114616
34180	CLXT	2020-10-02	5.5	5.53	5.21	5.32	153541	{}	\N	2020-10-12 14:33:25.119715
34181	CLXT	2020-10-05	5.39	5.75	5.39	5.53	88245	{}	\N	2020-10-12 14:33:25.124814
34182	CLXT	2020-10-06	5.632	5.71	5.41	5.44	59196	{}	\N	2020-10-12 14:33:25.129458
34183	CLXT	2020-10-07	5.58	5.69	5.36	5.49	41221	{}	\N	2020-10-12 14:33:25.133826
34184	CLXT	2020-10-08	5.53	5.62	5.4	5.46	27285	{}	\N	2020-10-12 14:33:25.138526
34185	CLXT	2020-10-09	5.49	5.56	5.3475	5.47	80769	{}	\N	2020-10-12 14:33:25.142994
34186	CLXT	2020-10-12	5.51	5.6	5.46	5.46	4624	{}	\N	2020-10-12 14:33:25.147999
34187	CMCT	2020-09-28	9.43	9.88	9.43	9.65	7515	{}	\N	2020-10-12 14:33:26.118405
34188	CMCT	2020-09-29	9.78	9.78	9.3468	9.44	10346	{}	\N	2020-10-12 14:33:26.127028
34189	CMCT	2020-09-30	9.37	9.86	9.33	9.86	14612	{}	\N	2020-10-12 14:33:26.132578
34190	CMCT	2020-10-01	9.74	10.15	9.25	9.86	9907	{}	\N	2020-10-12 14:33:26.138246
34191	CMCT	2020-10-02	10.09	10.2	9.82	9.9	7351	{}	\N	2020-10-12 14:33:26.143472
34192	CMCT	2020-10-05	9.64	9.89	9.51	9.68	15835	{}	\N	2020-10-12 14:33:26.147664
34193	CMCT	2020-10-06	9.665	9.75	9.28	9.3	15334	{}	\N	2020-10-12 14:33:26.15226
34194	CMCT	2020-10-07	9.57	9.585	8.95	8.97	19896	{}	\N	2020-10-12 14:33:26.156464
34195	CMCT	2020-10-08	8.9	9.035	8.62	8.69	15123	{}	\N	2020-10-12 14:33:26.160848
34196	CMCT	2020-10-09	8.72	8.73	8.4	8.49	14048	{}	\N	2020-10-12 14:33:26.164826
34197	CMCT	2020-10-12	8.41	8.52	8.41	8.52	1679	{}	\N	2020-10-12 14:33:26.1691
34198	CNC	2020-09-28	56.21	56.84	55.25	55.52	4702698	{}	\N	2020-10-12 14:33:30.492147
34199	CNC	2020-09-29	55.74	56.175	55.24	55.86	5039288	{}	\N	2020-10-12 14:33:30.50581
34200	CNC	2020-09-30	56.26	58.885	56.21	58.33	7255299	{}	\N	2020-10-12 14:33:30.512873
34201	CNC	2020-10-01	58.36	58.94	57.69	58.82	7694800	{}	\N	2020-10-12 14:33:30.518678
34202	CNC	2020-10-02	58.08	60.91	57.6	59.98	7887313	{}	\N	2020-10-12 14:33:30.523366
34203	CNC	2020-10-05	60.49	62.7	60.28	62.26	7161933	{}	\N	2020-10-12 14:33:30.527789
34204	CNC	2020-10-06	62.33	63.67	61.74	62.25	8242328	{}	\N	2020-10-12 14:33:30.532274
34205	CNC	2020-10-07	62.76	65.14	62.58	64.62	6060390	{}	\N	2020-10-12 14:33:30.536119
34206	CNC	2020-10-08	65	65.26	63.84	64.68	4283111	{}	\N	2020-10-12 14:33:30.539959
34207	CNC	2020-10-09	64.99	65.23	63.555	64.2	4660570	{}	\N	2020-10-12 14:33:30.543706
34208	CNC	2020-10-12	64.69	65.595	64.47	65.595	891158	{}	\N	2020-10-12 14:33:30.547455
34209	CNTY	2020-09-28	5.67	5.75	5.51	5.66	285241	{}	\N	2020-10-12 14:33:31.91955
34210	CNTY	2020-09-29	5.66	5.68	5.33	5.55	253508	{}	\N	2020-10-12 14:33:31.928089
34211	CNTY	2020-09-30	5.55	5.795	5.36	5.48	535140	{}	\N	2020-10-12 14:33:31.93408
34212	CNTY	2020-10-01	5.5	5.6	5.38	5.51	391038	{}	\N	2020-10-12 14:33:31.939794
34213	CNTY	2020-10-02	5.29	5.41	5.2105	5.32	403691	{}	\N	2020-10-12 14:33:31.94484
34214	CNTY	2020-10-05	5.31	5.41	5.16	5.21	271444	{}	\N	2020-10-12 14:33:31.949656
34215	CNTY	2020-10-06	5.26	5.37	5.14	5.17	344767	{}	\N	2020-10-12 14:33:31.953502
34216	CNTY	2020-10-07	5.25	5.27	5.13	5.23	246225	{}	\N	2020-10-12 14:33:31.957743
34217	CNTY	2020-10-08	5.32	5.59	5.25	5.55	254958	{}	\N	2020-10-12 14:33:31.961477
34218	CNTY	2020-10-09	5.56	5.6	5.29	5.38	210615	{}	\N	2020-10-12 14:33:31.965321
34219	CNTY	2020-10-12	5.68	5.68	5.44	5.52	98366	{}	\N	2020-10-12 14:33:31.969247
34220	COHR	2020-09-28	109.89	110.98	108.87	109.82	176396	{}	\N	2020-10-12 14:33:33.365407
34221	COHR	2020-09-29	109.68	111.49	109.4	110.68	132968	{}	\N	2020-10-12 14:33:33.376065
34222	COHR	2020-09-30	111.75	112.475	110.41	110.93	173626	{}	\N	2020-10-12 14:33:33.383468
34223	COHR	2020-10-01	111.18	113.81	111.18	112.97	158812	{}	\N	2020-10-12 14:33:33.389403
34224	COHR	2020-10-02	111.02	112.81	107.9	110	126669	{}	\N	2020-10-12 14:33:33.39464
34225	COHR	2020-10-05	111.3	111.86	110.13	111.25	106781	{}	\N	2020-10-12 14:33:33.399702
34226	COHR	2020-10-06	111.74	114.38	111.26	111.57	169411	{}	\N	2020-10-12 14:33:33.404549
34227	COHR	2020-10-07	113.05	114.415	111.74	111.78	118270	{}	\N	2020-10-12 14:33:33.408837
34228	COHR	2020-10-08	112.59	113.76	111.61	113.62	82818	{}	\N	2020-10-12 14:33:33.413366
34229	COHR	2020-10-09	114.99	118.35	113.74	117.23	146644	{}	\N	2020-10-12 14:33:33.417563
34230	COHR	2020-10-12	117.86	118.805	117.38	117.54	4604	{}	\N	2020-10-12 14:33:33.421342
34231	CONE	2020-09-28	72.79	72.79	71.99	72.23	1045298	{}	\N	2020-10-12 14:33:34.327173
34232	CONE	2020-09-29	71.8	72.56	71.47	71.47	1338813	{}	\N	2020-10-12 14:33:34.33606
34233	CONE	2020-09-30	71.9	72.2	69.86	70.03	1245978	{}	\N	2020-10-12 14:33:34.348495
34234	CONE	2020-10-01	69.8	73.46	69.68	73.45	803228	{}	\N	2020-10-12 14:33:34.354284
34235	CONE	2020-10-02	72.67	75.83	72.67	75.45	1181793	{}	\N	2020-10-12 14:33:34.358498
34236	CONE	2020-10-05	75.9	76.61	73.54	76.26	1051370	{}	\N	2020-10-12 14:33:34.362613
34237	CONE	2020-10-06	76.61	78.08	75.51	77.31	1508028	{}	\N	2020-10-12 14:33:34.366363
34238	CONE	2020-10-07	77.83	78.93	77.21	78.855	1205654	{}	\N	2020-10-12 14:33:34.370472
34239	CONE	2020-10-08	79.53	81	79.2	80.72	1443059	{}	\N	2020-10-12 14:33:34.37456
34240	CONE	2020-10-09	81.2	81.53	80	80.08	1739183	{}	\N	2020-10-12 14:33:34.378363
34241	CONE	2020-10-12	80.09	80.4	79.69	80.19	42862	{}	\N	2020-10-12 14:33:34.38254
34242	COUP	2020-09-28	275.63	277.96	269.25	276.66	976756	{}	\N	2020-10-12 14:33:36.606084
34243	COUP	2020-09-29	277	277.46	271.075	274.41	587978	{}	\N	2020-10-12 14:33:36.617092
34244	COUP	2020-09-30	273	277.53	270.91	274.24	595037	{}	\N	2020-10-12 14:33:36.624369
34245	COUP	2020-10-01	278.63	281.99	274.005	279.84	1149706	{}	\N	2020-10-12 14:33:36.630686
34246	COUP	2020-10-02	274.66	284.84	272.4	277.59	806069	{}	\N	2020-10-12 14:33:36.636042
34247	COUP	2020-10-05	279.78	284.17	277.19	282.92	675308	{}	\N	2020-10-12 14:33:36.640931
34248	COUP	2020-10-06	282.91	296	281.8501	287.96	888259	{}	\N	2020-10-12 14:33:36.644962
34249	COUP	2020-10-07	290.98	295.18	284.08	294.86	1074303	{}	\N	2020-10-12 14:33:36.648679
34250	COUP	2020-10-08	298.88	299.88	286.2	291.38	711361	{}	\N	2020-10-12 14:33:36.652253
34251	COUP	2020-10-09	298.26	308.69	298	304.35	2312293	{}	\N	2020-10-12 14:33:36.656106
34252	COUP	2020-10-12	309.54	313.43	304.81	306.57	215025	{}	\N	2020-10-12 14:33:36.66005
34253	CPAH	2020-09-28	3.03	3.11	3.0249	3.08	11274	{}	\N	2020-10-12 14:33:37.800553
34254	CPAH	2020-09-29	3.15	3.3141	3.09	3.13	26342	{}	\N	2020-10-12 14:33:37.809782
34255	CPAH	2020-09-30	3.1886	3.1886	3.02	3.08	14796	{}	\N	2020-10-12 14:33:37.81645
34256	CPAH	2020-10-01	3.13	3.13	2.9554	3	28834	{}	\N	2020-10-12 14:33:37.822505
34257	CPAH	2020-10-02	3.15	3.15	2.961	3.0223	10208	{}	\N	2020-10-12 14:33:37.827474
34258	CPAH	2020-10-05	3.02	3.4323	2.9093	3	341642	{}	\N	2020-10-12 14:33:37.832548
34259	CPAH	2020-10-06	3.12	3.12	2.93	2.99	35604	{}	\N	2020-10-12 14:33:37.836842
34260	CPAH	2020-10-07	2.96	3.07	2.96	3.0336	15486	{}	\N	2020-10-12 14:33:37.840949
34261	CPAH	2020-10-08	2.99	2.99	2.905	2.92	24507	{}	\N	2020-10-12 14:33:37.845212
34262	CPAH	2020-10-09	2.98	3.09	2.91	3.08	46611	{}	\N	2020-10-12 14:33:37.849978
34263	CPAH	2020-10-12	3.14	3.14	3.09	3.09	5665	{}	\N	2020-10-12 14:33:37.854671
34264	CPLP	2020-09-28	6.57	6.85	6.57	6.79	85571	{}	\N	2020-10-12 14:33:38.83575
34265	CPLP	2020-09-29	6.79	6.9	6.7	6.9	76234	{}	\N	2020-10-12 14:33:38.84564
34266	CPLP	2020-09-30	6.99	7.04	6.58	6.6	105136	{}	\N	2020-10-12 14:33:38.851567
34267	CPLP	2020-10-01	6.73	6.78	6.51	6.675	81572	{}	\N	2020-10-12 14:33:38.857361
34268	CPLP	2020-10-02	6.67	6.85	6.64	6.83	74879	{}	\N	2020-10-12 14:33:38.862217
34269	CPLP	2020-10-05	6.83	7.11	6.75	7.04	79628	{}	\N	2020-10-12 14:33:38.86719
34270	CPLP	2020-10-06	7.01	7.23	6.853	6.96	65947	{}	\N	2020-10-12 14:33:38.871284
34271	CPLP	2020-10-07	6.95	7.17	6.82	7.06	92777	{}	\N	2020-10-12 14:33:38.87561
34272	CPLP	2020-10-08	7.06	7.3535	7.06	7.31	164391	{}	\N	2020-10-12 14:33:38.879516
34273	CPLP	2020-10-09	7.35	7.4123	7.225	7.2981	110640	{}	\N	2020-10-12 14:33:38.883246
34274	CPLP	2020-10-12	7.21	7.28	7.11	7.28	13862	{}	\N	2020-10-12 14:33:38.886903
34275	CRAK	2020-09-28	20	20.0769	20	20.0495	2748	{}	\N	2020-10-12 14:33:39.835222
34276	CRAK	2020-09-29	20.14	20.16	19.84	19.8841	5360	{}	\N	2020-10-12 14:33:39.844612
34277	CRAK	2020-09-30	19.855	19.98	19.83	19.83	3425	{}	\N	2020-10-12 14:33:39.850001
34278	CRAK	2020-10-01	19.6216	19.69	19.6216	19.66	1325	{}	\N	2020-10-12 14:33:39.855421
34279	CRAK	2020-10-02	19.26	19.695	19.26	19.695	1126	{}	\N	2020-10-12 14:33:39.860003
34280	CRAK	2020-10-05	20.06	20.1508	19.93	20.0979	7903	{}	\N	2020-10-12 14:33:39.864358
34281	CRAK	2020-10-06	20.35	20.423	20.04	20.09	4596	{}	\N	2020-10-12 14:33:39.868121
34282	CRAK	2020-10-07	20.26	20.33	20.26	20.33	613	{}	\N	2020-10-12 14:33:39.87171
34283	CRAK	2020-10-08	20.53	20.61	20.48	20.61	6699	{}	\N	2020-10-12 14:33:39.875277
34284	CRAK	2020-10-09	20.82	20.82	20.6	20.66	803	{}	\N	2020-10-12 14:33:39.878963
34285	CRAK	2020-10-12	20.46	20.46	20.46	20.46	322	{}	\N	2020-10-12 14:33:39.882661
34286	CRH	2020-09-28	36.79	37.03	36.7	36.86	739268	{}	\N	2020-10-12 14:33:41.720948
34287	CRH	2020-09-29	36.61	36.92	36.38	36.46	600341	{}	\N	2020-10-12 14:33:41.731517
34288	CRH	2020-09-30	36.18	36.48	35.87	36.09	691914	{}	\N	2020-10-12 14:33:41.737977
34289	CRH	2020-10-01	36.14	36.305	35.8	36.01	1544926	{}	\N	2020-10-12 14:33:41.743469
34290	CRH	2020-10-02	35.38	36.72	35.37	36.55	577810	{}	\N	2020-10-12 14:33:41.748286
34291	CRH	2020-10-05	36.85	37.57	36.81	37.55	563074	{}	\N	2020-10-12 14:33:41.752657
34292	CRH	2020-10-06	38.17	38.57	37.55	37.57	533723	{}	\N	2020-10-12 14:33:41.75703
34293	CRH	2020-10-07	38.28	39.35	38.26	39.03	1272669	{}	\N	2020-10-12 14:33:41.760962
34294	CRH	2020-10-08	39.72	39.98	39.51	39.8	559537	{}	\N	2020-10-12 14:33:41.764811
34295	CRH	2020-10-09	39.91	40	39.46	39.53	554152	{}	\N	2020-10-12 14:33:41.770306
34296	CRH	2020-10-12	39.7	39.75	39.54	39.67	132898	{}	\N	2020-10-12 14:33:41.774138
34297	CRMT	2020-09-28	85.57	87.87	85.425	87.38	73662	{}	\N	2020-10-12 14:33:42.703747
34298	CRMT	2020-09-29	86.99	86.99	84.71	84.9	32477	{}	\N	2020-10-12 14:33:42.714432
34299	CRMT	2020-09-30	85.27	87.4799	84.13	84.88	46806	{}	\N	2020-10-12 14:33:42.721607
34300	CRMT	2020-10-01	86.23	87.25	85.16	87.13	61858	{}	\N	2020-10-12 14:33:42.72766
34301	CRMT	2020-10-02	85.23	88.76	85.23	88.3	50005	{}	\N	2020-10-12 14:33:42.733202
34302	CRMT	2020-10-05	88.72	91.19	88.72	90.98	24427	{}	\N	2020-10-12 14:33:42.738089
34303	CRMT	2020-10-06	92	94.34	90.89	91.76	45846	{}	\N	2020-10-12 14:33:42.742603
34304	CRMT	2020-10-07	94.51	94.51	90.82	91.75	31986	{}	\N	2020-10-12 14:33:42.74701
34305	CRMT	2020-10-08	92.53	93.965	90.54	93.55	17312	{}	\N	2020-10-12 14:33:42.751202
34306	CRMT	2020-10-09	94.21	94.86	91.32	93.52	28784	{}	\N	2020-10-12 14:33:42.755334
34307	CRMT	2020-10-12	93.84	93.93	92.53	93.01	6494	{}	\N	2020-10-12 14:33:42.759326
34308	CRTO	2020-09-28	12.14	12.27	11.91	11.99	177165	{}	\N	2020-10-12 14:33:44.527291
34309	CRTO	2020-09-29	12.03	12.1225	11.6	11.79	246927	{}	\N	2020-10-12 14:33:44.539524
34310	CRTO	2020-09-30	11.8	12.26	11.8	12.19	342857	{}	\N	2020-10-12 14:33:44.546948
34311	CRTO	2020-10-01	12.3	12.56	12.08	12.13	178651	{}	\N	2020-10-12 14:33:44.552983
34312	CRTO	2020-10-02	12.02	12.65	12	12.47	242921	{}	\N	2020-10-12 14:33:44.558387
34313	CRTO	2020-10-05	12.47	12.72	12.39	12.62	165454	{}	\N	2020-10-12 14:33:44.563532
34314	CRTO	2020-10-06	12.67	12.74	12.39	12.49	195863	{}	\N	2020-10-12 14:33:44.568251
34315	CRTO	2020-10-07	12.75	12.835	12.37	12.42	372088	{}	\N	2020-10-12 14:33:44.572619
34316	CRTO	2020-10-08	12.48	12.78	12.34	12.68	196769	{}	\N	2020-10-12 14:33:44.577004
34317	CRTO	2020-10-09	12.62	12.8633	12.45	12.64	121051	{}	\N	2020-10-12 14:33:44.581327
34318	CRTO	2020-10-12	12.66	12.72	12.44	12.495	88518	{}	\N	2020-10-12 14:33:44.585379
34319	CMRX	2020-09-28	2.6	2.641	2.44	2.54	351091	{}	\N	2020-10-12 14:33:45.536157
34320	CMRX	2020-09-29	2.52	2.64	2.51	2.52	310274	{}	\N	2020-10-12 14:33:45.54872
34321	CMRX	2020-09-30	2.5438	2.6	2.48	2.49	361815	{}	\N	2020-10-12 14:33:45.556557
34322	CMRX	2020-10-01	2.52	2.57	2.47	2.54	196461	{}	\N	2020-10-12 14:33:45.563047
34323	CMRX	2020-10-02	2.46	2.55	2.43	2.43	257606	{}	\N	2020-10-12 14:33:45.568167
34324	CMRX	2020-10-05	2.495	2.62	2.49	2.54	229152	{}	\N	2020-10-12 14:33:45.573512
34325	CMRX	2020-10-06	2.56	2.635	2.5	2.52	324193	{}	\N	2020-10-12 14:33:45.578026
34326	CMRX	2020-10-07	2.57	3.18	2.56	3.02	1672149	{}	\N	2020-10-12 14:33:45.582183
34327	CMRX	2020-10-08	3.09	3.13	2.92	2.94	480287	{}	\N	2020-10-12 14:33:45.586617
34328	CMRX	2020-10-09	2.97	3.175	2.97	3.03	301100	{}	\N	2020-10-12 14:33:45.590839
34329	CMRX	2020-10-12	3.04	3.065	3	3.015	73617	{}	\N	2020-10-12 14:33:45.595013
34330	CNET	2020-09-28	1.22	1.35	1.18	1.33	418189	{}	\N	2020-10-12 14:33:47.609217
34331	CNET	2020-09-29	1.38	1.389	1.26	1.295	124002	{}	\N	2020-10-12 14:33:47.618088
34332	CNET	2020-09-30	1.29	2.11	1.2867	2.0499	2387279	{}	\N	2020-10-12 14:33:47.624218
34333	CNET	2020-10-01	1.69	1.7499	1.39	1.45	1443401	{}	\N	2020-10-12 14:33:47.630339
34334	CNET	2020-10-02	1.32	1.59	1.28	1.43	1410641	{}	\N	2020-10-12 14:33:47.63633
34335	CNET	2020-10-05	1.4	1.425	1.27	1.27	728000	{}	\N	2020-10-12 14:33:47.641189
34336	CNET	2020-10-06	1.305	1.37	1.2801	1.32	191228	{}	\N	2020-10-12 14:33:47.645563
34337	CNET	2020-10-07	1.33	1.43	1.33	1.41	305208	{}	\N	2020-10-12 14:33:47.649814
34338	CNET	2020-10-08	1.4	1.7	1.38	1.53	1970587	{}	\N	2020-10-12 14:33:47.654168
34339	CNET	2020-10-09	1.5	1.54	1.4	1.4	348289	{}	\N	2020-10-12 14:33:47.658464
34340	CNET	2020-10-12	1.39	1.4216	1.39	1.41	26594	{}	\N	2020-10-12 14:33:47.662797
34341	CNOB	2020-09-28	13.88	14.56	13.74	14.365	130947	{}	\N	2020-10-12 14:33:48.729056
34342	CNOB	2020-09-29	14.24	14.27	13.98	14.25	101576	{}	\N	2020-10-12 14:33:48.742481
34343	CNOB	2020-09-30	14.33	14.56	13.95	14.07	118140	{}	\N	2020-10-12 14:33:48.749723
34344	CNOB	2020-10-01	14.02	14.41	13.97	14.4	90765	{}	\N	2020-10-12 14:33:48.755543
34345	CNOB	2020-10-02	14.12	14.89	14.12	14.84	71414	{}	\N	2020-10-12 14:33:48.760395
34346	CNOB	2020-10-05	15	15.28	14.935	15.22	114605	{}	\N	2020-10-12 14:33:48.764736
34347	CNOB	2020-10-06	15.46	15.67	15	15.01	140713	{}	\N	2020-10-12 14:33:48.769069
34348	CNOB	2020-10-07	15.29	15.67	15.18	15.62	143007	{}	\N	2020-10-12 14:33:48.773065
34349	CNOB	2020-10-08	15.84	16.2	15.64	16.14	115562	{}	\N	2020-10-12 14:33:48.776714
34350	CNOB	2020-10-09	16.31	16.31	15.85	15.91	77778	{}	\N	2020-10-12 14:33:48.780376
34351	CNOB	2020-10-12	15.88	16.09	15.8	16.01	4165	{}	\N	2020-10-12 14:33:48.783942
34352	CODI	2020-09-28	17.9	18.01	17.77	18	207144	{}	\N	2020-10-12 14:33:50.75647
34353	CODI	2020-09-29	17.86	18.35	17.7596	18.31	601694	{}	\N	2020-10-12 14:33:50.767778
34354	CODI	2020-09-30	18.5	19.405	18.43	19.06	1813479	{}	\N	2020-10-12 14:33:50.774277
34355	CODI	2020-10-01	19.01	19.63	19.01	19.44	705092	{}	\N	2020-10-12 14:33:50.779212
34356	CODI	2020-10-02	19.26	19.52	19.03	19.15	439978	{}	\N	2020-10-12 14:33:50.783829
34357	CODI	2020-10-05	19.25	19.4899	19.01	19.07	344880	{}	\N	2020-10-12 14:33:50.788147
34358	CODI	2020-10-06	19.18	19.3199	18.95	19.09	356755	{}	\N	2020-10-12 14:33:50.792421
34359	CODI	2020-10-07	19.13	19.2715	19.12	19.21	245271	{}	\N	2020-10-12 14:33:50.796172
34360	CODI	2020-10-08	19.32	19.8	19.2	19.77	534393	{}	\N	2020-10-12 14:33:50.799824
34361	CODI	2020-10-09	19.8	19.86	19.32	19.56	368305	{}	\N	2020-10-12 14:33:50.803343
34362	CODI	2020-10-12	19.53	19.58	19.32	19.4	104178	{}	\N	2020-10-12 14:33:50.806848
34363	COHU	2020-09-28	17.05	17.72	16.97	17.6	156725	{}	\N	2020-10-12 14:33:51.994563
34364	COHU	2020-09-29	17.47	17.8	17.35	17.56	134868	{}	\N	2020-10-12 14:33:52.000814
34365	COHU	2020-09-30	17.59	17.94	17.12	17.18	173509	{}	\N	2020-10-12 14:33:52.005922
34366	COHU	2020-10-01	17.35	18.37	17.31	18.36	232605	{}	\N	2020-10-12 14:33:52.01052
34367	COHU	2020-10-02	17.77	18.71	17.4	18.12	320529	{}	\N	2020-10-12 14:33:52.015155
34368	COHU	2020-10-05	18.31	18.46	17.65	18.1	317966	{}	\N	2020-10-12 14:33:52.019068
34369	COHU	2020-10-06	18	18.69	17.87	18.155	193921	{}	\N	2020-10-12 14:33:52.023214
34370	COHU	2020-10-07	18.42	18.65	18.11	18.56	384732	{}	\N	2020-10-12 14:33:52.027215
34371	COHU	2020-10-08	18.88	19.12	18.6001	18.83	162755	{}	\N	2020-10-12 14:33:52.031329
34372	COHU	2020-10-09	19.18	20.27	19.075	19.69	232520	{}	\N	2020-10-12 14:33:52.035325
34373	COHU	2020-10-12	19.75	19.88	19.5	19.725	66906	{}	\N	2020-10-12 14:33:52.039254
34374	CORP	2020-09-28	114.47	115.1	114.45	115.06	50130	{}	\N	2020-10-12 14:33:53.264626
34375	CORP	2020-09-29	115.06	115.06	114.75	115.01	21320	{}	\N	2020-10-12 14:33:53.276097
34376	CORP	2020-09-30	115.07	115.07	114.55	114.66	127201	{}	\N	2020-10-12 14:33:53.283004
34377	CORP	2020-10-01	114.5	115.09	114.17	114.69	104692	{}	\N	2020-10-12 14:33:53.288557
34378	CORP	2020-10-02	114.59	115.01	114.46	114.87	71666	{}	\N	2020-10-12 14:33:53.293383
34379	CORP	2020-10-05	114.33	114.94	114.3101	114.59	27625	{}	\N	2020-10-12 14:33:53.29797
34380	CORP	2020-10-06	114.96	115.12	114.4143	114.59	44399	{}	\N	2020-10-12 14:33:53.302672
34381	CORP	2020-10-07	114.49	114.89	114.2642	114.44	35709	{}	\N	2020-10-12 14:33:53.306564
34382	CORP	2020-10-08	114.4562	114.84	114.4288	114.77	18387	{}	\N	2020-10-12 14:33:53.310359
34383	CORP	2020-10-09	114.72	115.2	114.36	115.2	38622	{}	\N	2020-10-12 14:33:53.314421
34384	CORP	2020-10-12	115.9	116.0999	115.01	115.1	10286	{}	\N	2020-10-12 14:33:53.318412
34385	COWN	2020-09-28	15.4	15.8	15.39	15.64	265135	{}	\N	2020-10-12 14:33:54.310245
34386	COWN	2020-09-29	15.57	16.11	15.405	15.99	226751	{}	\N	2020-10-12 14:33:54.319363
34387	COWN	2020-09-30	16.06	16.605	16.06	16.27	265363	{}	\N	2020-10-12 14:33:54.325461
34388	COWN	2020-10-01	16.43	16.57	16.15	16.49	187506	{}	\N	2020-10-12 14:33:54.330397
34389	COWN	2020-10-02	16.17	16.76	15.47	16.62	312381	{}	\N	2020-10-12 14:33:54.334925
34390	COWN	2020-10-05	16.8	17.11	16.72	17.02	319474	{}	\N	2020-10-12 14:33:54.342514
34391	COWN	2020-10-06	17.07	17.7	16.995	17.255	319290	{}	\N	2020-10-12 14:33:54.346712
34392	COWN	2020-10-07	17.5	17.99	17.48	17.85	547444	{}	\N	2020-10-12 14:33:54.350102
34393	COWN	2020-10-08	17.99	18.08	17.74	18.01	196052	{}	\N	2020-10-12 14:33:55.362083
34394	COWN	2020-10-09	18.33	18.8199	18.31	18.41	232423	{}	\N	2020-10-12 14:33:55.367663
34395	COWN	2020-10-12	18.49	18.84	18.44	18.6	68619	{}	\N	2020-10-12 14:33:55.372898
34396	CPHI	2020-09-28	0.3676	0.3924	0.3676	0.3751	306373	{}	\N	2020-10-12 14:33:57.343183
34397	CPHI	2020-09-29	0.376	0.4144	0.361	0.367549	1233911	{}	\N	2020-10-12 14:33:57.35306
34398	CPHI	2020-09-30	0.3622	0.384	0.361	0.3648	143091	{}	\N	2020-10-12 14:33:57.358765
34399	CPHI	2020-10-01	0.3646	0.373	0.3605	0.3616	101871	{}	\N	2020-10-12 14:33:57.363364
34400	CPHI	2020-10-02	0.3724	0.38	0.3616	0.374	275754	{}	\N	2020-10-12 14:33:57.367901
34401	CPHI	2020-10-05	0.351	0.38	0.351	0.3781	104292	{}	\N	2020-10-12 14:33:57.371571
34402	CPHI	2020-10-06	0.3622	0.378	0.362	0.3776	82704	{}	\N	2020-10-12 14:33:57.375247
34403	CPHI	2020-10-07	0.3784	0.3784	0.3642	0.3742	108787	{}	\N	2020-10-12 14:33:57.379123
34404	CPHI	2020-10-08	0.3606	0.395	0.3606	0.38	305165	{}	\N	2020-10-12 14:33:57.383017
34405	CPHI	2020-10-09	0.373	0.393	0.373	0.3859	89216	{}	\N	2020-10-12 14:33:57.386838
34406	CPHI	2020-10-12	0.3899	0.398	0.3899	0.394	62900	{}	\N	2020-10-12 14:33:57.390628
34407	CPK	2020-09-28	84.79	89.1	83.78	86.56	1047340	{}	\N	2020-10-12 14:33:58.709614
34408	CPK	2020-09-29	86.02	86.63	84.74	86.01	976741	{}	\N	2020-10-12 14:33:58.720056
34409	CPK	2020-09-30	86.3	87.7	83.5	84.3	6710592	{}	\N	2020-10-12 14:33:58.726524
34410	CPK	2020-10-01	84.22	84.3761	82.43	83.8	495238	{}	\N	2020-10-12 14:33:58.732066
34411	CPK	2020-10-02	83.03	85.38	82.95	85.25	354153	{}	\N	2020-10-12 14:33:58.736659
34412	CPK	2020-10-05	84.81	85.66	83.85	84.15	309889	{}	\N	2020-10-12 14:33:58.741114
34413	CPK	2020-10-06	84.61	87.46	83.65	86.47	365827	{}	\N	2020-10-12 14:33:58.74548
34414	CPK	2020-10-07	86.67	86.8	85.24	86.31	259045	{}	\N	2020-10-12 14:33:58.749396
34415	CPK	2020-10-08	87.17	89.06	86.09	89.04	230310	{}	\N	2020-10-12 14:33:58.75324
34416	CPK	2020-10-09	89.77	90.04	87.35	88.82	182043	{}	\N	2020-10-12 14:33:58.756941
34417	CPK	2020-10-12	88.68	89.51	88.34	89	15880	{}	\N	2020-10-12 14:33:58.761086
34418	CPTA	2020-09-28	9.6096	9.6777	9.51	9.54	13547	{}	\N	2020-10-12 14:33:59.746757
34419	CPTA	2020-09-29	9.52	9.59	9.3	9.35	9778	{}	\N	2020-10-12 14:33:59.756705
34420	CPTA	2020-09-30	9.3783	9.41	9.2	9.41	14174	{}	\N	2020-10-12 14:33:59.764255
34421	CPTA	2020-10-01	9.3999	9.7	9.2579	9.66	34387	{}	\N	2020-10-12 14:33:59.770228
34422	CPTA	2020-10-02	9.48	9.7	9.45	9.48	6431	{}	\N	2020-10-12 14:33:59.775087
34423	CPTA	2020-10-05	9.54	9.7	9.5	9.55	10713	{}	\N	2020-10-12 14:33:59.77934
34424	CPTA	2020-10-06	9.68	9.75	9.54	9.74	13745	{}	\N	2020-10-12 14:33:59.783227
34425	CPTA	2020-10-07	9.65	9.65	9.55	9.65	9607	{}	\N	2020-10-12 14:33:59.787131
34426	CPTA	2020-10-08	9.66	9.66	9.36	9.42	36884	{}	\N	2020-10-12 14:33:59.790823
34427	CPTA	2020-10-09	9.47	9.5826	9.36	9.56	15493	{}	\N	2020-10-12 14:33:59.79445
34428	CPTA	2020-10-12	9.6124	9.62	9.5201	9.585	1069	{}	\N	2020-10-12 14:33:59.798204
34429	CREG	2020-09-28	3.04	4.1	2.9001	3.61	3340171	{}	\N	2020-10-12 14:34:01.62227
34430	CREG	2020-09-29	3.51	3.7699	3.3177	3.6	695005	{}	\N	2020-10-12 14:34:01.632115
34431	CREG	2020-09-30	3.52	3.62	3.06	3.13	448751	{}	\N	2020-10-12 14:34:01.638844
34432	CREG	2020-10-01	3.13	3.37	3.13	3.18	297453	{}	\N	2020-10-12 14:34:01.644378
34433	CREG	2020-10-02	2.91	3.2699	2.9	3.09	213746	{}	\N	2020-10-12 14:34:01.649421
34434	CREG	2020-10-05	3.05	3.78	2.96	3.49	1632292	{}	\N	2020-10-12 14:34:01.654624
34435	CREG	2020-10-06	3.57	4.4	3.3701	3.92	2912768	{}	\N	2020-10-12 14:34:01.659085
34436	CREG	2020-10-07	4.31	4.98	3.81	3.87	2109377	{}	\N	2020-10-12 14:34:01.663017
34437	CREG	2020-10-08	3.87	4.05	3.31	3.43	773020	{}	\N	2020-10-12 14:34:01.667052
34438	CREG	2020-10-09	3.3648	3.82	3.36	3.78	428493	{}	\N	2020-10-12 14:34:01.671569
34439	CREG	2020-10-12	3.83	3.8999	3.63	3.6301	104199	{}	\N	2020-10-12 14:34:01.675829
34440	CNFR	2020-09-29	2.88	2.88	2.87	2.87	717	{}	\N	2020-10-12 14:34:04.034466
34441	CNFR	2020-09-30	2.86	2.88	2.86	2.88	637	{}	\N	2020-10-12 14:34:04.04259
34442	CNFR	2020-10-05	2.875	2.88	2.79	2.79	842	{}	\N	2020-10-12 14:34:04.048226
34443	CNFR	2020-10-06	2.955	3.01	2.76	2.79	5395	{}	\N	2020-10-12 14:34:04.053356
34444	CNFR	2020-10-07	2.77	2.8	2.625	2.625	1380	{}	\N	2020-10-12 14:34:04.058288
34445	CNFR	2020-10-08	2.6498	2.65	2.6498	2.6499	3900	{}	\N	2020-10-12 14:34:04.062806
34446	CNFR	2020-10-09	2.79	2.79	2.79	2.79	217	{}	\N	2020-10-12 14:34:04.06708
34447	CNO	2020-09-28	16.03	16.5	16.01	16.27	1390400	{}	\N	2020-10-12 14:34:05.027368
34448	CNO	2020-09-29	16.25	16.25	15.82	15.97	1169313	{}	\N	2020-10-12 14:34:05.036938
34449	CNO	2020-09-30	16.04	16.26	15.9	16.04	1544172	{}	\N	2020-10-12 14:34:05.043198
34450	CNO	2020-10-01	16.19	16.42	16.02	16.3	1923235	{}	\N	2020-10-12 14:34:05.048788
34451	CNO	2020-10-02	15.95	16.76	15.94	16.65	1938571	{}	\N	2020-10-12 14:34:05.053502
34452	CNO	2020-10-05	16.89	17.26	16.86	17.21	1294432	{}	\N	2020-10-12 14:34:05.058288
34453	CNO	2020-10-06	17.52	17.75	16.97	17.06	2131691	{}	\N	2020-10-12 14:34:05.062308
34454	CNO	2020-10-07	17.4	17.78	17.4	17.66	1681079	{}	\N	2020-10-12 14:34:05.066166
34455	CNO	2020-10-08	17.8	18.08	17.65	18.03	1610353	{}	\N	2020-10-12 14:34:05.070163
34456	CNO	2020-10-09	18.21	18.33	18.04	18.18	2285044	{}	\N	2020-10-12 14:34:05.07388
34457	CNO	2020-10-12	18.08	18.4	18.08	18.4	215613	{}	\N	2020-10-12 14:34:05.077587
34458	CNYA	2020-09-28	36.31	36.36	36.2	36.36	13935	{}	\N	2020-10-12 14:34:06.304771
34459	CNYA	2020-09-29	36.41	36.65	36.34	36.6	58872	{}	\N	2020-10-12 14:34:06.317351
34460	CNYA	2020-09-30	36.34	36.745	36.34	36.6	113962	{}	\N	2020-10-12 14:34:06.324488
34461	CNYA	2020-10-01	37.28	37.38	37.155	37.29	178308	{}	\N	2020-10-12 14:34:06.330105
34462	CNYA	2020-10-02	36.68	37.0612	36.68	36.72	14875	{}	\N	2020-10-12 14:34:06.335138
34463	CNYA	2020-10-05	37.11	37.33	37.09	37.23	25206	{}	\N	2020-10-12 14:34:06.343619
34464	CNYA	2020-10-06	37.35	37.465	37.25	37.3	13001	{}	\N	2020-10-12 14:34:06.348774
34465	CNYA	2020-10-07	37.6	37.69	37.565	37.64	24400	{}	\N	2020-10-12 14:34:06.353041
34466	CNYA	2020-10-08	37.75	37.75	37.6	37.69	34087	{}	\N	2020-10-12 14:34:06.357228
34467	CNYA	2020-10-09	37.91	38.13	37.91	38.001	19177	{}	\N	2020-10-12 14:34:06.361421
34468	CNYA	2020-10-12	38.85	38.85	38.59	38.7391	110930	{}	\N	2020-10-12 14:34:06.365536
34469	COHN	2020-09-28	20.39	21.1	19.73	20.73	433426	{}	\N	2020-10-12 14:34:07.578612
34470	COHN	2020-09-29	20.2	20.85	19.5	19.5	344809	{}	\N	2020-10-12 14:34:07.590278
34471	COHN	2020-09-30	19.49	19.9418	18.66	19.55	87632	{}	\N	2020-10-12 14:34:07.597468
34472	COHN	2020-10-01	19.28	20.1	19.24	19.25	38208	{}	\N	2020-10-12 14:34:07.602863
34473	COHN	2020-10-02	18.73	19.05	18.15	19.02	20078	{}	\N	2020-10-12 14:34:07.608242
34474	COHN	2020-10-05	18.94	19.4321	18	18.11	51170	{}	\N	2020-10-12 14:34:07.613703
34475	COHN	2020-10-06	18.09	18.4656	17.88	18.14	55731	{}	\N	2020-10-12 14:34:07.618711
34476	COHN	2020-10-07	18.15	19	17.8801	18.26	76533	{}	\N	2020-10-12 14:34:07.623218
34477	COHN	2020-10-08	18.84	20.31	18.4728	20.3098	195364	{}	\N	2020-10-12 14:34:07.627409
34478	COHN	2020-10-09	20.3	21.7	19.59	20.92	183355	{}	\N	2020-10-12 14:34:07.631643
34479	COHN	2020-10-12	20.6	22.06	20.21	21.83	52521	{}	\N	2020-10-12 14:34:07.635818
34480	CONN	2020-09-28	10.59	10.91	10.39	10.78	295938	{}	\N	2020-10-12 14:34:08.775003
34481	CONN	2020-09-29	10.73	10.795	10.4	10.58	314212	{}	\N	2020-10-12 14:34:08.783618
34482	CONN	2020-09-30	10.67	11	10.47	10.58	450195	{}	\N	2020-10-12 14:34:08.790286
34483	CONN	2020-10-01	10.58	10.78	9.8533	10.25	322822	{}	\N	2020-10-12 14:34:08.795507
34484	CONN	2020-10-02	9.86	10.45	9.86	10.41	312385	{}	\N	2020-10-12 14:34:08.800595
34485	CONN	2020-10-05	10.5	11.05	10.46	11.02	415867	{}	\N	2020-10-12 14:34:08.804759
34486	CONN	2020-10-06	11.02	11.7	11.02	11.14	543624	{}	\N	2020-10-12 14:34:08.808715
34487	CONN	2020-10-07	11.36	11.81	11.27	11.7	326755	{}	\N	2020-10-12 14:34:08.812169
34488	CONN	2020-10-08	11.96	12.25	11.74	12.21	352316	{}	\N	2020-10-12 14:34:08.815602
34489	CONN	2020-10-09	11.95	12.55	11.95	12.47	377595	{}	\N	2020-10-12 14:34:08.818998
34490	CONN	2020-10-12	12.61	13.04	12.48	13.03	71139	{}	\N	2020-10-12 14:34:08.822216
34491	CORT	2020-09-28	17.2	17.5	17.135	17.34	1014700	{}	\N	2020-10-12 14:34:10.040193
34492	CORT	2020-09-29	17.42	17.85	17.33	17.55	690155	{}	\N	2020-10-12 14:34:10.048797
34493	CORT	2020-09-30	17.53	17.98	17.27	17.405	1046821	{}	\N	2020-10-12 14:34:10.054235
34494	CORT	2020-10-01	17.41	17.78	17.23	17.38	644516	{}	\N	2020-10-12 14:34:10.059015
34495	CORT	2020-10-02	17.09	17.4	16.88	16.92	655023	{}	\N	2020-10-12 14:34:10.063487
34496	CORT	2020-10-05	17.11	17.75	16.84	17.64	661941	{}	\N	2020-10-12 14:34:10.06752
34497	CORT	2020-10-06	17.67	18.37	17.67	17.7136	570692	{}	\N	2020-10-12 14:34:10.071627
34498	CORT	2020-10-07	17.69	17.97	17.47	17.78	713859	{}	\N	2020-10-12 14:34:10.075016
34499	CORT	2020-10-08	17.97	18.62	17.94	18.32	699608	{}	\N	2020-10-12 14:34:10.078443
34500	CORT	2020-10-09	18.45	18.64	18.261	18.35	419974	{}	\N	2020-10-12 14:34:10.081935
34501	CORT	2020-10-12	18.54	18.6	18.24	18.27	29780	{}	\N	2020-10-12 14:34:10.085047
34502	CPB	2020-09-28	47.9998	48.365	47.445	47.98	2059338	{}	\N	2020-10-12 14:34:11.779786
34503	CPB	2020-09-29	48.12	48.19	47.72	48.02	3345250	{}	\N	2020-10-12 14:34:11.793027
34504	CPB	2020-09-30	48.21	48.7278	47.87	48.37	4723130	{}	\N	2020-10-12 14:34:11.800191
34505	CPB	2020-10-01	48.31	48.89	46.99	48.52	4759551	{}	\N	2020-10-12 14:34:11.80654
34506	CPB	2020-10-02	48.19	49.03	47.87	48.81	2523553	{}	\N	2020-10-12 14:34:11.811655
34507	CPB	2020-10-05	49	49.38	48.43	48.98	6290273	{}	\N	2020-10-12 14:34:11.816705
34508	CPB	2020-10-06	48.89	49.1541	48.48	48.58	3786737	{}	\N	2020-10-12 14:34:11.820658
34509	CPB	2020-10-07	48.23	48.59	47.81	48.25	5457842	{}	\N	2020-10-12 14:34:11.824861
34510	CPB	2020-10-08	48.29	48.7	47.86	48.42	3364575	{}	\N	2020-10-12 14:34:11.829051
34511	CPB	2020-10-09	48.35	48.775	48.08	48.47	2940869	{}	\N	2020-10-12 14:34:11.833336
34512	CPB	2020-10-12	48.7	49.11	48.47	48.79	783018	{}	\N	2020-10-12 14:34:11.837584
34513	CPSS	2020-09-28	3.18	3.32	3.17	3.25	8974	{}	\N	2020-10-12 14:34:13.243004
34514	CPSS	2020-09-29	3.19	3.33	3.19	3.29	11046	{}	\N	2020-10-12 14:34:13.252814
34515	CPSS	2020-09-30	3.23	3.36	3.23	3.3	2971	{}	\N	2020-10-12 14:34:13.259013
34516	CPSS	2020-10-01	3.32	3.46	3.27	3.4	11005	{}	\N	2020-10-12 14:34:13.264285
34517	CPSS	2020-10-02	3.3044	3.47	3.22	3.24	14186	{}	\N	2020-10-12 14:34:13.268819
34518	CPSS	2020-10-05	3.24	3.47	3.24	3.42	13902	{}	\N	2020-10-12 14:34:13.273512
34519	CPSS	2020-10-06	3.24	3.5	3.24	3.36	6739	{}	\N	2020-10-12 14:34:13.277702
34520	CPSS	2020-10-07	3.31	3.4	3.24	3.33	7569	{}	\N	2020-10-12 14:34:13.281355
34521	CPSS	2020-10-08	4.91	5.12	4.37	4.69	6129419	{}	\N	2020-10-12 14:34:13.285349
34522	CPSS	2020-10-09	4.7	4.78	4.51	4.64	354482	{}	\N	2020-10-12 14:34:13.289841
34523	CPSS	2020-10-12	4.6	4.66	4.55	4.6081	31853	{}	\N	2020-10-12 14:34:13.29386
34524	CQQQ	2020-09-28	67.67	67.79	67.24	67.62	188795	{}	\N	2020-10-12 14:34:15.312799
34525	CQQQ	2020-09-29	67.94	68.1075	67.64	67.96	132915	{}	\N	2020-10-12 14:34:15.323285
34526	CQQQ	2020-09-30	68.36	69.34	68.27	69.16	82968	{}	\N	2020-10-12 14:34:15.3302
34527	CQQQ	2020-10-01	69.91	70.28	69.62	70.23	105043	{}	\N	2020-10-12 14:34:15.337606
34528	CQQQ	2020-10-02	69.01	69.84	68.94	69.2	74047	{}	\N	2020-10-12 14:34:15.344102
34529	CQQQ	2020-10-05	69.28	69.61	69.19	69.61	52663	{}	\N	2020-10-12 14:34:15.349889
34530	CQQQ	2020-10-06	70.44	70.99	70.37	70.52	164984	{}	\N	2020-10-12 14:34:15.353853
34531	CQQQ	2020-10-07	71.66	71.92	71.4	71.7	120850	{}	\N	2020-10-12 14:34:15.357824
34532	CQQQ	2020-10-08	71.88	71.975	71.6	71.85	169645	{}	\N	2020-10-12 14:34:15.362057
34533	CQQQ	2020-10-09	72.35	72.87	72.34	72.74	88374	{}	\N	2020-10-12 14:34:15.366071
34534	CQQQ	2020-10-12	74.12	74.348	73.91	74.29	108425	{}	\N	2020-10-12 14:34:15.369899
34535	CRK	2020-09-28	4.7	4.715	4.5	4.67	2254000	{}	\N	2020-10-12 14:34:16.659211
34536	CRK	2020-09-29	4.66	4.68	4.29	4.54	7366649	{}	\N	2020-10-12 14:34:16.66822
34537	CRK	2020-09-30	4.51	4.535	4.295	4.38	5819343	{}	\N	2020-10-12 14:34:16.674256
34538	CRK	2020-10-01	4.36	4.459	4.3	4.36	6018709	{}	\N	2020-10-12 14:34:16.680501
34539	CRK	2020-10-02	4.26	4.52	4.26	4.45	3280983	{}	\N	2020-10-12 14:34:16.68606
34540	CRK	2020-10-05	4.51	4.82	4.502	4.77	5466170	{}	\N	2020-10-12 14:34:16.69092
34541	CRK	2020-10-06	4.9	4.98	4.755	4.82	5080383	{}	\N	2020-10-12 14:34:16.695121
34542	CRK	2020-10-07	4.9	5.33	4.83	5.28	7130189	{}	\N	2020-10-12 14:34:16.699134
34543	CRK	2020-10-08	5.31	5.37	5.19	5.36	5469305	{}	\N	2020-10-12 14:34:16.702971
34544	CRK	2020-10-09	5.52	5.67	5.41	5.65	8011966	{}	\N	2020-10-12 14:34:16.70709
34545	CRK	2020-10-12	5.61	5.62	5.44	5.51	881760	{}	\N	2020-10-12 14:34:16.71089
34546	CRT	2020-09-28	5.83	6.16	5.81	6.11	45593	{}	\N	2020-10-12 14:34:18.590941
34547	CRT	2020-09-29	5.95	6.1073	5.81	5.9576	56224	{}	\N	2020-10-12 14:34:18.596233
34548	CRT	2020-09-30	5.96	5.9723	5.46	5.5	88566	{}	\N	2020-10-12 14:34:18.600158
34549	CRT	2020-10-01	5.46	5.84	5.46	5.58	70368	{}	\N	2020-10-12 14:34:18.603964
34550	CRT	2020-10-02	5.42	5.495	5.42	5.46	24572	{}	\N	2020-10-12 14:34:18.608601
34551	CRT	2020-10-05	5.42	5.5289	5.34	5.45	49754	{}	\N	2020-10-12 14:34:18.613185
34552	CRT	2020-10-06	5.45	5.53	5.4101	5.4101	36548	{}	\N	2020-10-12 14:34:18.61737
34553	CRT	2020-10-07	5.4353	5.4353	5.16	5.3143	73036	{}	\N	2020-10-12 14:34:18.621177
34554	CRT	2020-10-08	5.28	5.5	5.26	5.5	32880	{}	\N	2020-10-12 14:34:18.624968
34555	CRT	2020-10-09	5.54	5.6	5.54	5.5898	28998	{}	\N	2020-10-12 14:34:18.628679
34556	CRT	2020-10-12	5.54	5.545	5.44	5.4635	18022	{}	\N	2020-10-12 14:34:18.632589
34557	CSBR	2020-09-28	9.35	9.49	9.14	9.2	15615	{}	\N	2020-10-12 14:34:21.492166
34558	CSBR	2020-09-29	9.25	9.3	9.23	9.26	9788	{}	\N	2020-10-12 14:34:21.502321
34559	CSBR	2020-09-30	9.23	9.35	9.195	9.25	11084	{}	\N	2020-10-12 14:34:21.508572
34560	CSBR	2020-10-01	9.2	9.2	8.99	8.99	11151	{}	\N	2020-10-12 14:34:21.513765
34561	CSBR	2020-10-02	8.95	8.95	8.7409	8.8	6524	{}	\N	2020-10-12 14:34:21.518241
34562	CSBR	2020-10-05	8.6	9.11	8.09	9.03	23389	{}	\N	2020-10-12 14:34:21.522573
34563	CSBR	2020-10-06	8.96	9.47	8.96	9.31	13844	{}	\N	2020-10-12 14:34:21.526778
34564	CSBR	2020-10-07	9.49	9.49	9.18	9.36	15427	{}	\N	2020-10-12 14:34:21.53006
34565	CSBR	2020-10-08	9.48	9.62	9.2601	9.46	13729	{}	\N	2020-10-12 14:34:21.534663
34566	CSBR	2020-10-09	9.47	9.47	8.9853	9.31	5143	{}	\N	2020-10-12 14:34:21.539043
34567	CSBR	2020-10-12	9.24	9.291	9.24	9.291	1931	{}	\N	2020-10-12 14:34:21.543165
34568	CNBKA	2020-09-28	64.32	65.08	64.09	64.1	7369	{}	\N	2020-10-12 14:34:22.464346
34569	CNBKA	2020-09-29	64.48	65.51	64	65	18874	{}	\N	2020-10-12 14:34:22.476209
34570	CNBKA	2020-09-30	64.79	66.86	64.79	66	8180	{}	\N	2020-10-12 14:34:22.483041
34571	CNBKA	2020-10-01	66	66.56	65.815	66.01	4296	{}	\N	2020-10-12 14:34:22.488928
34572	CNBKA	2020-10-02	66.8539	67.91	66.1801	67.91	2934	{}	\N	2020-10-12 14:34:22.493683
34573	CNBKA	2020-10-05	68.56	69	68.04	69	7271	{}	\N	2020-10-12 14:34:22.498116
34574	CNBKA	2020-10-06	68.75	68.975	67	67.58	4458	{}	\N	2020-10-12 14:34:22.502388
34575	CNBKA	2020-10-07	67.555	69	67.555	68.26	10551	{}	\N	2020-10-12 14:34:22.506525
34576	CNBKA	2020-10-08	69.9999	69.9999	68.3831	69.24	7494	{}	\N	2020-10-12 14:34:22.510573
34577	CNBKA	2020-10-09	70.08	70.08	67.6	68.7	2699	{}	\N	2020-10-12 14:34:22.514336
34578	CNBKA	2020-10-12	68.53	69.64	68.53	69.64	631	{}	\N	2020-10-12 14:34:22.518054
34579	CNQ	2020-09-28	16.5	16.85	16.48	16.72	1969917	{}	\N	2020-10-12 14:34:23.468329
34580	CNQ	2020-09-29	16.57	16.62	15.85	16.05	6780756	{}	\N	2020-10-12 14:34:23.473818
34581	CNQ	2020-09-30	16.12	16.41	15.985	16.01	6224785	{}	\N	2020-10-12 14:34:23.477994
34582	CNQ	2020-10-01	15.85	16.12	15.43	15.54	7496007	{}	\N	2020-10-12 14:34:23.482038
34583	CNQ	2020-10-02	14.99	15.92	14.85	15.64	7347779	{}	\N	2020-10-12 14:34:23.485997
34584	CNQ	2020-10-05	15.94	16.15	15.68	16.13	5662380	{}	\N	2020-10-12 14:34:23.489863
34585	CNQ	2020-10-06	16.4	16.73	15.99	16.25	8315885	{}	\N	2020-10-12 14:34:23.494003
34586	CNQ	2020-10-07	16.33	16.89	16.215	16.77	5422535	{}	\N	2020-10-12 14:34:23.497943
34587	CNQ	2020-10-08	16.96	18.175	16.91	17.97	10228666	{}	\N	2020-10-12 14:34:23.502504
34588	CNQ	2020-10-09	18.1	18.1144	17.45	17.88	10398338	{}	\N	2020-10-12 14:34:23.507024
34589	CNQ	2020-10-12	17.88	18.035	17.6701	17.94	724559	{}	\N	2020-10-12 14:34:23.511078
34590	COG	2020-09-28	18.24	18.29	18.04	18.07	3344180	{}	\N	2020-10-12 14:34:25.798245
34591	COG	2020-09-29	17.86	18.04	17.325	17.64	11907822	{}	\N	2020-10-12 14:34:25.812551
34592	COG	2020-09-30	17.65	17.75	17.185	17.36	8423928	{}	\N	2020-10-12 14:34:25.82025
34593	COG	2020-10-01	17.26	17.48	16.82	16.85	10838875	{}	\N	2020-10-12 14:34:25.825786
34594	COG	2020-10-02	16.47	17.215	16.41	17.01	8652515	{}	\N	2020-10-12 14:34:25.830766
34595	COG	2020-10-05	17.2	17.77	17.045	17.74	7718232	{}	\N	2020-10-12 14:34:25.835632
34596	COG	2020-10-06	17.91	18.175	17.58	17.61	7299310	{}	\N	2020-10-12 14:34:25.840271
34597	COG	2020-10-07	17.84	18.37	17.72	18.26	9431757	{}	\N	2020-10-12 14:34:25.844229
34598	COG	2020-10-08	18.11	18.52	18.03	18.49	8449762	{}	\N	2020-10-12 14:34:25.84855
34599	COG	2020-10-09	18.75	18.97	18.42	18.93	8579332	{}	\N	2020-10-12 14:34:25.852436
34600	COG	2020-10-12	18.9	19.145	18.71	19.14	1310517	{}	\N	2020-10-12 14:34:25.856223
34601	COLM	2020-09-28	91.15	91.4	89.14	89.19	312238	{}	\N	2020-10-12 14:34:26.931401
34602	COLM	2020-09-29	89.14	89.14	85.57	86.48	417336	{}	\N	2020-10-12 14:34:26.943658
34603	COLM	2020-09-30	86.42	88.22	86	86.98	458195	{}	\N	2020-10-12 14:34:26.950918
34604	COLM	2020-10-01	87.48	88.77	87.13	88.29	307036	{}	\N	2020-10-12 14:34:26.956286
34605	COLM	2020-10-02	85.7209	89.43	85.7209	88.94	313393	{}	\N	2020-10-12 14:34:26.960606
34606	COLM	2020-10-05	89.68	90.98	89.48	90.96	243216	{}	\N	2020-10-12 14:34:26.964913
34607	COLM	2020-10-06	90.705	92.06	89.49	89.69	160913	{}	\N	2020-10-12 14:34:26.969107
34608	COLM	2020-10-07	90.96	92.14	90.19	91.98	227317	{}	\N	2020-10-12 14:34:26.973093
34609	COLM	2020-10-08	92.04	93.4	91.83	93.14	174516	{}	\N	2020-10-12 14:34:26.976573
34610	COLM	2020-10-09	94.61	94.83	93.07	94.53	296510	{}	\N	2020-10-12 14:34:26.980594
34611	COLM	2020-10-12	94.53	94.75	93.65	93.68	22122	{}	\N	2020-10-12 14:34:26.98458
34612	COP	2020-09-28	34.34	34.74	33.615	33.71	8859987	{}	\N	2020-10-12 14:34:27.872578
34613	COP	2020-09-29	33.72	33.78	32.1703	32.46	13270911	{}	\N	2020-10-12 14:34:27.884428
34614	COP	2020-09-30	32.73	33.29	32.545	32.84	15990132	{}	\N	2020-10-12 14:34:27.891568
34615	COP	2020-10-01	32.23	32.74	31.6947	32.43	15528795	{}	\N	2020-10-12 14:34:27.897326
34616	COP	2020-10-02	31.58	33.29	31.45	33.07	13611340	{}	\N	2020-10-12 14:34:27.902473
34617	COP	2020-10-05	33.56	34.31	33.2316	34.28	11377662	{}	\N	2020-10-12 14:34:27.907013
34618	COP	2020-10-06	34.88	35.08	33.12	33.3	13001094	{}	\N	2020-10-12 14:34:27.911151
34619	COP	2020-10-07	33.47	34.53	33.315	34.33	13009644	{}	\N	2020-10-12 14:34:27.915166
34620	COP	2020-10-08	34.5	35.615	34.3	35.53	13441327	{}	\N	2020-10-12 14:34:27.919086
34621	COP	2020-10-09	35.94	35.97	34.835	35.13	12595227	{}	\N	2020-10-12 14:34:27.923152
34622	COP	2020-10-12	35.07	35.46	34.73	35.46	2048421	{}	\N	2020-10-12 14:34:27.927991
34623	COWNZ	2020-09-28	25.2	25.27	25.19	25.24	14803	{}	\N	2020-10-12 14:34:29.093109
34624	COWNZ	2020-09-29	25.29	25.29	25.2735	25.28	2015	{}	\N	2020-10-12 14:34:29.100273
34625	COWNZ	2020-09-30	25.2987	25.71	25.2987	25.71	24996	{}	\N	2020-10-12 14:34:29.10506
34626	COWNZ	2020-10-01	25.62	25.64	25.25	25.64	6746	{}	\N	2020-10-12 14:34:29.109245
34627	COWNZ	2020-10-02	25.5576	25.65	25.3	25.5	8177	{}	\N	2020-10-12 14:34:29.113436
34628	COWNZ	2020-10-05	25.545	25.545	25.48	25.49	4896	{}	\N	2020-10-12 14:34:29.118072
34629	COWNZ	2020-10-06	25.55	25.55	25.3254	25.3254	9981	{}	\N	2020-10-12 14:34:29.122265
34630	COWNZ	2020-10-07	25.4788	25.4788	25.3	25.415	6257	{}	\N	2020-10-12 14:34:29.126197
34631	COWNZ	2020-10-08	25.45	25.45	25.28	25.345	5991	{}	\N	2020-10-12 14:34:29.129997
34632	COWNZ	2020-10-09	25.3015	25.4485	25.3015	25.375	3323	{}	\N	2020-10-12 14:34:29.13391
34633	CPG	2020-09-28	1.26	1.3	1.24	1.28	1726191	{}	\N	2020-10-12 14:34:30.123923
34634	CPG	2020-09-29	1.28	1.28	1.17	1.19	19431598	{}	\N	2020-10-12 14:34:30.133437
34635	CPG	2020-09-30	1.2	1.25	1.18	1.22	20711061	{}	\N	2020-10-12 14:34:30.139344
34636	CPG	2020-10-01	1.21	1.21	1.1219	1.15	3010933	{}	\N	2020-10-12 14:34:30.144851
34637	CPG	2020-10-02	1.06	1.2	1.05	1.17	4044842	{}	\N	2020-10-12 14:34:30.149344
34638	CPG	2020-10-05	1.2	1.24	1.165	1.23	1717685	{}	\N	2020-10-12 14:34:30.153989
34639	CPG	2020-10-06	1.25	1.29	1.215	1.23	4950294	{}	\N	2020-10-12 14:34:30.157789
34640	CPG	2020-10-07	1.25	1.29	1.21	1.28	3325967	{}	\N	2020-10-12 14:34:30.161938
34641	CPG	2020-10-08	1.29	1.4	1.29	1.39	2610931	{}	\N	2020-10-12 14:34:30.165733
34642	CPG	2020-10-09	1.4	1.42	1.325	1.36	2436917	{}	\N	2020-10-12 14:34:30.169536
34643	CPG	2020-10-12	1.35	1.351	1.32	1.35	401708	{}	\N	2020-10-12 14:34:30.173178
34644	CQP	2020-09-28	33	33.69	32.61	33.52	164141	{}	\N	2020-10-12 14:34:31.606908
34645	CQP	2020-09-29	33.64	34.95	33.4594	34.78	218968	{}	\N	2020-10-12 14:34:31.620699
34646	CQP	2020-09-30	34.66	35.25	32.81	33.26	475369	{}	\N	2020-10-12 14:34:31.628433
34647	CQP	2020-10-01	33.06	33.86	33.0394	33.48	139179	{}	\N	2020-10-12 14:34:31.633962
34648	CQP	2020-10-02	33.17	34.65	33.03	34.23	235058	{}	\N	2020-10-12 14:34:31.639124
34649	CQP	2020-10-05	34.35	35.97	34.29	35.82	133994	{}	\N	2020-10-12 14:34:31.643946
34650	CQP	2020-10-06	35.52	36.17	34.295	34.84	147827	{}	\N	2020-10-12 14:34:31.648664
34651	CQP	2020-10-07	34.76	35.415	34.5	34.98	122036	{}	\N	2020-10-12 14:34:31.653056
34652	CQP	2020-10-08	34.96	35.95	34.81	35.76	160411	{}	\N	2020-10-12 14:34:31.657127
34653	CQP	2020-10-09	35.5	36.41	35.2	35.83	182606	{}	\N	2020-10-12 14:34:31.661093
34654	CQP	2020-10-12	36.13	36.2	35.1	35.18	22503	{}	\N	2020-10-12 14:34:31.665594
34655	CREE	2020-09-28	61.6	62.51	60.74	62.46	731254	{}	\N	2020-10-12 14:34:33.488206
34656	CREE	2020-09-29	62.37	63.46	61.975	62.71	639180	{}	\N	2020-10-12 14:34:33.49575
34657	CREE	2020-09-30	62.89	64.725	62.69	63.74	882567	{}	\N	2020-10-12 14:34:33.501137
34658	CREE	2020-10-01	64.515	66.48	64.515	65.83	802209	{}	\N	2020-10-12 14:34:33.506336
34659	CREE	2020-10-02	63.49	65.06	63.18	64.07	723777	{}	\N	2020-10-12 14:34:33.511212
34660	CREE	2020-10-05	65.19	66.79	65.19	66.3	689774	{}	\N	2020-10-12 14:34:33.515789
34661	CREE	2020-10-06	66.65	67.72	65.305	65.63	615700	{}	\N	2020-10-12 14:34:33.520175
34662	CREE	2020-10-07	67.39	68.38	66.73	67.92	1206483	{}	\N	2020-10-12 14:34:33.524132
34663	CREE	2020-10-08	68.78	69.33	68.2	69.03	693155	{}	\N	2020-10-12 14:34:33.527916
34664	CREE	2020-10-09	70	70.87	69.35	69.54	869065	{}	\N	2020-10-12 14:34:33.531783
34665	CREE	2020-10-12	69.87	70.39	69.2701	69.61	51356	{}	\N	2020-10-12 14:34:33.535601
34666	CRL	2020-09-28	219.54	224.3	219.09	223.42	227722	{}	\N	2020-10-12 14:34:34.681034
34667	CRL	2020-09-29	224.19	225.9179	222.53	222.93	797115	{}	\N	2020-10-12 14:34:34.693022
34668	CRL	2020-09-30	222.08	227.44	221.55	226.45	458305	{}	\N	2020-10-12 14:34:34.699734
34669	CRL	2020-10-01	228.02	230.64	227.145	230.47	818852	{}	\N	2020-10-12 14:34:34.706259
34670	CRL	2020-10-02	226.87	231.97	226.62	228.87	455049	{}	\N	2020-10-12 14:34:34.710852
34671	CRL	2020-10-05	232.07	235.99	231.945	234.97	427858	{}	\N	2020-10-12 14:34:34.715186
34672	CRL	2020-10-06	235.67	236.35	231.55	233.38	534602	{}	\N	2020-10-12 14:34:34.719379
34673	CRL	2020-10-07	236.05	237.73	235.39	237.35	603041	{}	\N	2020-10-12 14:34:34.723085
34674	CRL	2020-10-08	238.4	238.63	235.14	236.49	433731	{}	\N	2020-10-12 14:34:34.72669
34675	CRL	2020-10-09	238.98	243.46	238.36	242.57	778600	{}	\N	2020-10-12 14:34:34.730367
34676	CRL	2020-10-12	244.99	244.99	241.16	241.555	68320	{}	\N	2020-10-12 14:34:34.733957
34677	CRS	2020-09-28	18.72	19.14	18.5	18.65	473885	{}	\N	2020-10-12 14:34:36.274629
34678	CRS	2020-09-29	18.64	18.81	17.955	18.19	762619	{}	\N	2020-10-12 14:34:36.283623
34679	CRS	2020-09-30	18.21	18.96	18.02	18.16	696470	{}	\N	2020-10-12 14:34:36.289845
34680	CRS	2020-10-01	18.19	18.55	17.97	18.32	706386	{}	\N	2020-10-12 14:34:36.295167
34681	CRS	2020-10-02	17.9	18.96	17.83	18.86	706170	{}	\N	2020-10-12 14:34:36.299747
34682	CRS	2020-10-05	18.99	19.38	18.99	19.22	446758	{}	\N	2020-10-12 14:34:36.304565
34683	CRS	2020-10-06	19.59	19.69	18.23	18.32	754978	{}	\N	2020-10-12 14:34:36.308866
34684	CRS	2020-10-07	18.64	19.04	18.48	18.84	809871	{}	\N	2020-10-12 14:34:36.313067
34685	CRS	2020-10-08	19.16	19.16	18.64	19.06	683097	{}	\N	2020-10-12 14:34:36.31714
34686	CRS	2020-10-09	19.27	19.3065	18.59	18.88	501209	{}	\N	2020-10-12 14:34:36.321243
34687	CRS	2020-10-12	18.9	19.25	18.635	19.21	72077	{}	\N	2020-10-12 14:34:36.325164
34688	COE	2020-09-28	20.16	20.99	19.78	19.78	3730	{}	\N	2020-10-12 14:34:37.94404
34689	COE	2020-09-29	19.74	20.15	19.41	19.41	43620	{}	\N	2020-10-12 14:34:37.95437
34690	COE	2020-09-30	19.91	20.545	19.07	19.07	7438	{}	\N	2020-10-12 14:34:37.96102
34691	COE	2020-10-01	19.37	20.7	19.37	20.65	28194	{}	\N	2020-10-12 14:34:37.96685
34692	COE	2020-10-02	21.72	21.85	20.81	21.7776	60002	{}	\N	2020-10-12 14:34:37.971695
34693	COE	2020-10-05	22.09	25.27	22.09	25.08	118538	{}	\N	2020-10-12 14:34:37.976429
34694	COE	2020-10-06	25.24	25.98	24.6227	24.8	93183	{}	\N	2020-10-12 14:34:37.980471
34695	COE	2020-10-07	25.1	25.535	24	24.47	33648	{}	\N	2020-10-12 14:34:37.984401
34696	COE	2020-10-08	23.98	25.42	23.98	25.12	109853	{}	\N	2020-10-12 14:34:37.988195
34697	COE	2020-10-09	25.15	25.22	24.6755	25	10962	{}	\N	2020-10-12 14:34:37.992028
34698	COE	2020-10-12	24.3	25.14	24.3	24.71	3804	{}	\N	2020-10-12 14:34:37.995839
34699	COLD	2020-09-28	36.17	36.34	35.8	36.1	896909	{}	\N	2020-10-12 14:34:39.214323
34700	COLD	2020-09-29	35.75	35.95	35.19	35.51	2552037	{}	\N	2020-10-12 14:34:39.225101
34701	COLD	2020-09-30	35.82	35.985	35.31	35.75	3364785	{}	\N	2020-10-12 14:34:39.23222
34702	COLD	2020-10-01	35.85	36.45	35.185	36.45	3284863	{}	\N	2020-10-12 14:34:39.237957
34703	COLD	2020-10-02	36.01	36.82	35.81	36.44	3502092	{}	\N	2020-10-12 14:34:39.242736
34704	COLD	2020-10-05	36.61	37.525	35.91	37.39	2568389	{}	\N	2020-10-12 14:34:39.247424
34705	COLD	2020-10-06	37.58	37.8373	37.09	37.53	2287761	{}	\N	2020-10-12 14:34:39.251279
34706	COLD	2020-10-07	37.67	37.825	37.023	37.28	3055206	{}	\N	2020-10-12 14:34:39.255391
34707	COLD	2020-10-08	37.49	38.015	37.36	37.98	2629065	{}	\N	2020-10-12 14:34:39.259152
34708	COLD	2020-10-09	38.29	39.0738	38.06	38.89	2991908	{}	\N	2020-10-12 14:34:39.263268
34709	COLD	2020-10-12	38.86	38.89	38.41	38.74	105452	{}	\N	2020-10-12 14:34:39.267177
34710	COPX	2020-09-28	20.82	20.82	20.4701	20.58	122972	{}	\N	2020-10-12 14:34:40.536906
34711	COPX	2020-09-29	20.6	20.665	20.3	20.43	61962	{}	\N	2020-10-12 14:34:40.546718
34712	COPX	2020-09-30	20.35	20.77	20.35	20.72	63152	{}	\N	2020-10-12 14:34:40.553911
34713	COPX	2020-10-01	20.79	20.79	20.47	20.6	79450	{}	\N	2020-10-12 14:34:40.560015
34714	COPX	2020-10-02	20.3	20.7238	20.21	20.71	93308	{}	\N	2020-10-12 14:34:40.56491
34715	COPX	2020-10-05	20.72	21.035	20.72	21	53120	{}	\N	2020-10-12 14:34:40.570066
34716	COPX	2020-10-06	21.13	21.17	20.37	20.5	95756	{}	\N	2020-10-12 14:34:40.574217
34717	COPX	2020-10-07	20.66	21.13	20.66	20.94	130002	{}	\N	2020-10-12 14:34:40.578276
34718	COPX	2020-10-08	21.11	21.25	20.9456	21.16	40852	{}	\N	2020-10-12 14:34:40.582522
34719	COPX	2020-10-09	21.35	21.5	21.35	21.5	61836	{}	\N	2020-10-12 14:34:40.586835
34720	COPX	2020-10-12	21.4999	21.5	21.42	21.4383	37260	{}	\N	2020-10-12 14:34:40.591
35984	CULP	2020-09-28	12.44	12.74	12.29	12.57	13757	{}	\N	2020-10-12 14:37:48.056312
35985	CULP	2020-09-29	12.59	12.59	12.1	12.35	35312	{}	\N	2020-10-12 14:37:48.063359
35986	CULP	2020-09-30	12.46	12.53	12.25	12.42	34364	{}	\N	2020-10-12 14:37:48.068855
35987	CULP	2020-10-01	12.52	13.54	12.5	13.48	165300	{}	\N	2020-10-12 14:37:48.073643
35988	CULP	2020-10-02	13.45	14.1799	13.19	13.88	132368	{}	\N	2020-10-12 14:37:48.079719
35989	CULP	2020-10-05	13.84	14.025	13.18	13.45	97713	{}	\N	2020-10-12 14:37:48.083732
35990	CULP	2020-10-06	13.74	14.05	13.56	14	180545	{}	\N	2020-10-12 14:37:48.0882
35991	CULP	2020-10-07	14	14.575	13.8	13.9	171422	{}	\N	2020-10-12 14:37:48.092311
35992	CULP	2020-10-08	13.95	14.15	13.475	13.88	67903	{}	\N	2020-10-12 14:37:48.096277
35993	CULP	2020-10-09	13.83	14.18	13.83	14.08	23940	{}	\N	2020-10-12 14:37:48.100359
35994	CULP	2020-10-12	14.21	14.21	13.88	13.88	7026	{}	\N	2020-10-12 14:37:48.104743
35995	CVCO	2020-09-28	177.72	185.21	174.53	184.54	47687	{}	\N	2020-10-12 14:37:49.083828
35996	CVCO	2020-09-29	185.25	186.16	178.91	179.06	35012	{}	\N	2020-10-12 14:37:49.09326
35997	CVCO	2020-09-30	179.68	183.64	179.64	180.64	26297	{}	\N	2020-10-12 14:37:49.099196
35998	CVCO	2020-10-01	182.17	183.85	180.5	182.93	58843	{}	\N	2020-10-12 14:37:49.104552
35999	CVCO	2020-10-02	179	186.91	178.147	184.53	48677	{}	\N	2020-10-12 14:37:49.109161
34721	COST	2020-09-28	343.76	349.77	343.1	349.62	3304179	{}	\N	2020-10-12 14:34:41.77452
34722	COST	2020-09-29	348.95	354.29	347.1	352.14	2437167	{}	\N	2020-10-12 14:34:41.792157
34723	COST	2020-09-30	352.56	357.6986	351.1705	355	2320953	{}	\N	2020-10-12 14:34:41.798031
34724	COST	2020-10-01	356.26	359.685	354.82	358.46	1928196	{}	\N	2020-10-12 14:34:41.802707
34725	COST	2020-10-02	354.7277	357.84	352.85	355.01	1777433	{}	\N	2020-10-12 14:34:41.807254
34726	COST	2020-10-05	355.33	358.98	355.33	358.58	2052504	{}	\N	2020-10-12 14:34:41.811456
34727	COST	2020-10-06	361.9	363	357.0394	358.35	2589526	{}	\N	2020-10-12 14:34:41.815389
34728	COST	2020-10-07	360.43	364.74	359.66	363.02	1901059	{}	\N	2020-10-12 14:34:41.819211
34729	COST	2020-10-08	364.802	365.97	361.75	365.09	1745869	{}	\N	2020-10-12 14:34:41.823143
34730	COST	2020-10-09	366	369.75	365.71	369.46	1629761	{}	\N	2020-10-12 14:34:41.826887
34731	COST	2020-10-12	371	373.7899	370.77	372.81	641345	{}	\N	2020-10-12 14:34:41.83108
34732	CPAC	2020-09-28	7.59	7.59	7.59	7.59	460	{}	\N	2020-10-12 14:34:44.204369
34733	CPAC	2020-09-29	7.57	7.57	7.57	7.57	608	{}	\N	2020-10-12 14:34:44.211989
34734	CPAC	2020-09-30	7.8	7.8	7.7	7.8	7016	{}	\N	2020-10-12 14:34:44.218485
34735	CPAC	2020-10-01	7.8595	7.8595	7.8595	7.8595	572	{}	\N	2020-10-12 14:34:44.22356
34736	CPAC	2020-10-02	7.97	8.009	7.78	7.78	2758	{}	\N	2020-10-12 14:34:44.228661
34737	CPAC	2020-10-07	7.9673	7.9673	7.8	7.8	6768	{}	\N	2020-10-12 14:34:44.233291
34738	CPAC	2020-10-08	7.87	7.87	7.75	7.75	634	{}	\N	2020-10-12 14:34:44.237316
34739	CPIX	2020-09-28	3.25	3.31	3.18	3.29	7518	{}	\N	2020-10-12 14:34:45.197632
34740	CPIX	2020-09-29	3.21	3.25	3.18	3.2	6060	{}	\N	2020-10-12 14:34:45.20816
34741	CPIX	2020-09-30	3.2	3.2599	3.18	3.21	17404	{}	\N	2020-10-12 14:34:45.214764
34742	CPIX	2020-10-01	3.17	3.28	3.1509	3.22	10708	{}	\N	2020-10-12 14:34:45.220712
34743	CPIX	2020-10-02	3.22	3.35	3.22	3.3	9295	{}	\N	2020-10-12 14:34:45.225543
34744	CPIX	2020-10-05	3.23	3.36	3.23	3.3	25478	{}	\N	2020-10-12 14:34:45.230243
34745	CPIX	2020-10-06	3.305	3.36	3.18	3.18	32184	{}	\N	2020-10-12 14:34:45.234394
34746	CPIX	2020-10-07	3.22	3.24	3.16	3.2	27327	{}	\N	2020-10-12 14:34:45.23835
34747	CPIX	2020-10-08	3.2	3.2799	3.15	3.2	7829	{}	\N	2020-10-12 14:34:45.242351
34748	CPIX	2020-10-09	3.25	3.2798	3.13	3.2	41986	{}	\N	2020-10-12 14:34:45.246124
34749	CPIX	2020-10-12	3.18	3.25	3.165	3.2	1566	{}	\N	2020-10-12 14:34:45.249971
34750	CPSH	2020-09-28	1.73	1.7385	1.5	1.6	240501	{}	\N	2020-10-12 14:34:46.591346
34751	CPSH	2020-09-29	1.64	1.66	1.59	1.63	29062	{}	\N	2020-10-12 14:34:46.601067
34752	CPSH	2020-09-30	1.6	1.65	1.58	1.645	90708	{}	\N	2020-10-12 14:34:46.607627
34753	CPSH	2020-10-01	1.64	1.7	1.63	1.6467	290345	{}	\N	2020-10-12 14:34:46.613159
34754	CPSH	2020-10-02	1.64	1.8339	1.6101	1.785	326587	{}	\N	2020-10-12 14:34:46.618374
34755	CPSH	2020-10-05	1.77	1.98	1.72	1.89	339198	{}	\N	2020-10-12 14:34:46.623241
34756	CPSH	2020-10-06	1.89	2.0899	1.76	1.83	1281228	{}	\N	2020-10-12 14:34:46.627659
34757	CPSH	2020-10-07	1.87	2.22	1.86	1.87	1134270	{}	\N	2020-10-12 14:34:46.631748
34758	CPSH	2020-10-08	1.91	1.9297	1.7302	1.79	342185	{}	\N	2020-10-12 14:34:46.636091
34759	CPSH	2020-10-09	1.8444	1.86	1.7005	1.81	174459	{}	\N	2020-10-12 14:34:46.640404
34760	CPSH	2020-10-12	1.78	1.795	1.755	1.77	10960	{}	\N	2020-10-12 14:34:46.644824
34761	CPTAL	2020-09-28	22.6	22.905	22.6	22.71	2602	{}	\N	2020-10-12 14:34:47.734917
34762	CPTAL	2020-09-29	22.65	23.1999	22.65	22.65	2147	{}	\N	2020-10-12 14:34:47.742982
34763	CPTAL	2020-09-30	22.62	22.85	22.61	22.8	1892	{}	\N	2020-10-12 14:34:47.748983
34764	CPTAL	2020-10-01	22.8	22.9623	22.8	22.8	1500	{}	\N	2020-10-12 14:34:47.754085
34765	CPTAL	2020-10-02	22.6	22.895	22.6	22.85	849	{}	\N	2020-10-12 14:34:47.758868
34766	CPTAL	2020-10-05	23	23	22.75	22.9	1493	{}	\N	2020-10-12 14:34:47.764916
34767	CPTAL	2020-10-06	22.64	22.75	22.6	22.6	4778	{}	\N	2020-10-12 14:34:47.769237
34768	CPTAL	2020-10-07	22.52	22.75	22.5	22.6	2648	{}	\N	2020-10-12 14:34:47.774486
34769	CPTAL	2020-10-08	22.7001	22.9	22.585	22.8075	2828	{}	\N	2020-10-12 14:34:47.779022
34770	CPTAL	2020-10-09	22.82	22.82	22.6	22.6	400	{}	\N	2020-10-12 14:34:47.783173
34771	CPTAL	2020-10-12	22.7	22.7582	22.7	22.7	2025	{}	\N	2020-10-12 14:34:47.787661
34772	CRIS	2020-09-28	1.09	1.12	1.04	1.07	268260	{}	\N	2020-10-12 14:34:49.155454
34773	CRIS	2020-09-29	1.07	1.1	1.025	1.04	309075	{}	\N	2020-10-12 14:34:49.168812
34774	CRIS	2020-09-30	1.02	1.2	1.02	1.17	1094459	{}	\N	2020-10-12 14:34:49.176384
34775	CRIS	2020-10-01	1.1865	1.24	1.13	1.21	498835	{}	\N	2020-10-12 14:34:49.182022
34776	CRIS	2020-10-02	1.19	1.3	1.18	1.25	1259211	{}	\N	2020-10-12 14:34:49.187227
34777	CRIS	2020-10-05	1.26	1.26	1.12	1.2	974254	{}	\N	2020-10-12 14:34:49.191964
34778	CRIS	2020-10-06	1.21	1.28	1.1901	1.23	333319	{}	\N	2020-10-12 14:34:49.196667
34779	CRIS	2020-10-07	1.24	1.27	1.23	1.24	220170	{}	\N	2020-10-12 14:34:49.200661
34780	CRIS	2020-10-08	1.22	1.25	1.19	1.2	330700	{}	\N	2020-10-12 14:34:49.204648
34781	CRIS	2020-10-09	1.2	1.22	1.19	1.21	226083	{}	\N	2020-10-12 14:34:49.208712
34782	CRIS	2020-10-12	1.21	1.25	1.19	1.23	110018	{}	\N	2020-10-12 14:34:49.212675
34783	CROX	2020-09-28	43.13	43.48	41.94	41.99	762400	{}	\N	2020-10-12 14:34:50.142987
34784	CROX	2020-09-29	42.35	42.65	41.29	41.94	767810	{}	\N	2020-10-12 14:34:50.152423
34785	CROX	2020-09-30	42.295	43.16	41.78	42.73	967653	{}	\N	2020-10-12 14:34:50.158499
34786	CROX	2020-10-01	43.1	48.09	42.88	45.19	5489066	{}	\N	2020-10-12 14:34:50.163933
34787	CROX	2020-10-02	43.89	46.24	43.5	45.98	1444951	{}	\N	2020-10-12 14:34:50.169274
34788	CROX	2020-10-05	46.56	47.62	46.53	47.28	923276	{}	\N	2020-10-12 14:34:50.174225
34789	CROX	2020-10-06	47.46	48.34	46.15	46.8	1194161	{}	\N	2020-10-12 14:34:50.178441
34790	CROX	2020-10-07	47.71	50.98	46.21	47.1	2158536	{}	\N	2020-10-12 14:34:50.182686
34791	CROX	2020-10-08	47.79	48	46.8	47.05	589581	{}	\N	2020-10-12 14:34:50.186813
34792	CROX	2020-10-09	47.45	47.74	47.04	47.66	536460	{}	\N	2020-10-12 14:34:50.190988
34793	CROX	2020-10-12	48.5	48.8177	47.8	47.93	139491	{}	\N	2020-10-12 14:34:50.196301
34794	CSB	2020-09-28	37.59	38.22	37.59	38.01	5183	{}	\N	2020-10-12 14:34:52.779031
34795	CSB	2020-09-29	37.985	37.985	37.379	37.685	5468	{}	\N	2020-10-12 14:34:52.789092
34796	CSB	2020-09-30	37.83	38.1517	37.57	37.73	10721	{}	\N	2020-10-12 14:34:52.795361
34797	CSB	2020-10-01	37.57	37.9	37.57	37.84	9496	{}	\N	2020-10-12 14:34:52.800908
34798	CSB	2020-10-02	37.28	38.7408	37.28	38.7408	5040	{}	\N	2020-10-12 14:34:52.806381
34799	CSB	2020-10-05	39.22	39.59	39.132	39.59	18735	{}	\N	2020-10-12 14:34:52.811179
34800	CSB	2020-10-06	39.76	40.63	39.5271	39.5271	11779	{}	\N	2020-10-12 14:34:52.815588
34801	CSB	2020-10-07	39.99	40.35	39.8998	40.14	11091	{}	\N	2020-10-12 14:34:52.81998
34802	CSB	2020-10-08	40.67	40.876	40.4817	40.876	9439	{}	\N	2020-10-12 14:34:52.824387
34803	CSB	2020-10-09	41.27	41.29	40.6763	40.6763	12161	{}	\N	2020-10-12 14:34:52.828708
34804	CSB	2020-10-12	40.87	40.87	40.7643	40.815	2062	{}	\N	2020-10-12 14:34:52.832921
34805	COKE	2020-09-28	242.96	250	241.11	241.11	27151	{}	\N	2020-10-12 14:34:53.867092
34806	COKE	2020-09-29	241.72	243.7437	237.71	238.98	13840	{}	\N	2020-10-12 14:34:53.879441
34807	COKE	2020-09-30	239.11	241.7402	236.6317	240.68	17970	{}	\N	2020-10-12 14:34:53.88698
34808	COKE	2020-10-01	241.02	241.26	234.01	239.31	28662	{}	\N	2020-10-12 14:34:53.894009
34809	COKE	2020-10-02	235.84	242.67	235.835	242.6	24930	{}	\N	2020-10-12 14:34:53.89874
34810	COKE	2020-10-05	244.5	244.76	239.8	242.6	13867	{}	\N	2020-10-12 14:34:53.903487
34811	COKE	2020-10-06	243.65	244.545	237.185	241.62	33872	{}	\N	2020-10-12 14:34:53.907842
34812	COKE	2020-10-07	242.25	243.73	239.515	239.72	24875	{}	\N	2020-10-12 14:34:53.911937
34813	COKE	2020-10-08	240.31	242.43	238.99	240.33	16842	{}	\N	2020-10-12 14:34:53.915971
34814	COKE	2020-10-09	242.11	243.9232	240.5	242.29	24652	{}	\N	2020-10-12 14:34:53.920009
34815	COKE	2020-10-12	244.29	244.29	242.49	243.81	2892	{}	\N	2020-10-12 14:34:53.923861
34816	CORN	2020-09-28	12.69	12.85	12.59	12.77	325711	{}	\N	2020-10-12 14:34:55.121435
34817	CORN	2020-09-29	12.72	12.74	12.645	12.72	162367	{}	\N	2020-10-12 14:34:55.134289
34818	CORN	2020-09-30	12.72	13.28	12.69	13.11	1182528	{}	\N	2020-10-12 14:34:55.142145
34819	CORN	2020-10-01	13.25	13.3	13.2	13.24	146363	{}	\N	2020-10-12 14:34:55.148244
34820	CORN	2020-10-02	13.14	13.25	13.11	13.13	128118	{}	\N	2020-10-12 14:34:55.153386
34821	CORN	2020-10-05	13.2	13.2251	13.11	13.19	154099	{}	\N	2020-10-12 14:34:55.158005
34822	CORN	2020-10-06	13.26	13.4177	13.25	13.29	317972	{}	\N	2020-10-12 14:34:55.162375
34823	CORN	2020-10-07	13.4	13.48	13.3589	13.38	169393	{}	\N	2020-10-12 14:34:55.166265
34824	CORN	2020-10-08	13.48	13.5	13.2801	13.31	213266	{}	\N	2020-10-12 14:34:55.170295
34825	CORN	2020-10-09	13.37	13.5462	13.32	13.5	256524	{}	\N	2020-10-12 14:34:55.174458
34826	CORN	2020-10-12	13.49	13.5	13.33	13.38	236457	{}	\N	2020-10-12 14:34:55.17837
34827	CPA	2020-09-28	55.26	55.385	52.85	53.72	1165759	{}	\N	2020-10-12 14:34:57.35366
34828	CPA	2020-09-29	53.69	53.81	50.17	50.4	1928430	{}	\N	2020-10-12 14:34:57.362385
34829	CPA	2020-09-30	50.47	52.66	50.05	50.34	1286339	{}	\N	2020-10-12 14:34:57.367677
34830	CPA	2020-10-01	49.95	51.78	49.76	51.25	841757	{}	\N	2020-10-12 14:34:57.372159
34831	CPA	2020-10-02	49.75	51.42	49.17	49.69	1309981	{}	\N	2020-10-12 14:34:57.376345
34832	CPA	2020-10-05	50.4	50.89	49.38	50.46	1905884	{}	\N	2020-10-12 14:34:57.380457
34833	CPA	2020-10-06	51.54	53.11	51.23	52.17	1348395	{}	\N	2020-10-12 14:34:57.383997
34834	CPA	2020-10-07	53.2	54.35	51.86	51.96	741264	{}	\N	2020-10-12 14:34:57.388644
34835	CPA	2020-10-08	52.83	53.6	52.07	52.68	533285	{}	\N	2020-10-12 14:34:57.392262
34836	CPA	2020-10-09	52.68	53.02	51.6568	52.89	474706	{}	\N	2020-10-12 14:34:57.395858
34837	CPA	2020-10-12	52.59	53.23	52.33	52.9	76485	{}	\N	2020-10-12 14:34:57.399283
34838	CPHC	2020-09-28	12.19	12.22	12.12	12.22	1755	{}	\N	2020-10-12 14:34:58.400749
34839	CPHC	2020-09-29	12	12.07	12	12.07	426	{}	\N	2020-10-12 14:34:58.409818
34840	CPHC	2020-10-05	12.05	12.05	12.05	12.05	763	{}	\N	2020-10-12 14:34:58.416671
34841	CPHC	2020-10-06	12.4	12.4	12.045	12.05	1236	{}	\N	2020-10-12 14:34:58.422345
34842	CPHC	2020-10-07	11.965	12	11.965	12	533	{}	\N	2020-10-12 14:34:58.427399
34843	CPHC	2020-10-08	12.25	12.25	12.25	12.25	414	{}	\N	2020-10-12 14:34:58.431895
34844	CPHC	2020-10-12	12.25	12.25	12.25	12.25	278	{}	\N	2020-10-12 14:34:58.435954
34845	CPST	2020-09-28	4.19	4.73	4.19	4.65	159989	{}	\N	2020-10-12 14:35:02.507566
34846	CPST	2020-09-29	4.62	4.8503	4.52	4.75	135983	{}	\N	2020-10-12 14:35:02.5175
34847	CPST	2020-09-30	4.8	5.06	4.71	4.76	198864	{}	\N	2020-10-12 14:35:02.524194
34848	CPST	2020-10-01	4.77	4.94	4.7277	4.83	92382	{}	\N	2020-10-12 14:35:02.529679
34849	CPST	2020-10-02	4.86	4.92	4.46	4.85	152331	{}	\N	2020-10-12 14:35:02.535013
34850	CPST	2020-10-05	4.87	5.48	4.87	5.41	355673	{}	\N	2020-10-12 14:35:02.540224
34851	CPST	2020-10-06	5.45	5.64	5.07	5.2	197411	{}	\N	2020-10-12 14:35:02.544394
34852	CPST	2020-10-07	5.36	6.8	5.17	6.72	840108	{}	\N	2020-10-12 14:35:02.548694
34853	CPST	2020-10-08	6.3	6.63	5.62	5.78	431756	{}	\N	2020-10-12 14:35:02.553228
34854	CPST	2020-10-09	6	6.2	5.79	5.83	117949	{}	\N	2020-10-12 14:35:02.557452
34855	CPST	2020-10-12	5.9	5.9	5.62	5.7182	36833	{}	\N	2020-10-12 14:35:02.56163
34856	CR	2020-09-28	50.41	51.6	50.41	51.08	251077	{}	\N	2020-10-12 14:35:04.922513
34857	CR	2020-09-29	51.23	51.24	49.55	50.2	351925	{}	\N	2020-10-12 14:35:04.933289
34858	CR	2020-09-30	50.31	51.2441	49.45	50.13	843911	{}	\N	2020-10-12 14:35:04.93974
34859	CR	2020-10-01	50.36	50.98	49.23	49.41	482467	{}	\N	2020-10-12 14:35:04.944453
34860	CR	2020-10-02	48.23	50.655	48.19	49.74	590787	{}	\N	2020-10-12 14:35:04.948892
34861	CR	2020-10-05	50.66	50.7881	49.98	50.16	686073	{}	\N	2020-10-12 14:35:04.953351
34862	CR	2020-10-06	50.53	51.84	50.21	50.49	792554	{}	\N	2020-10-12 14:35:04.957444
34863	CR	2020-10-07	51.55	52.28	51.05	51.39	1263829	{}	\N	2020-10-12 14:35:04.960874
34864	CR	2020-10-08	52.05	52.14	50.9625	51.7	905321	{}	\N	2020-10-12 14:35:04.964347
34865	CR	2020-10-09	52.36	52.36	51.59	51.79	1143141	{}	\N	2020-10-12 14:35:04.96775
34866	CR	2020-10-12	52.01	52.64	51.63	52.46	104866	{}	\N	2020-10-12 14:35:04.97118
34867	CRESY	2020-09-28	3.08	3.08	2.78	2.82	183352	{}	\N	2020-10-12 14:35:06.871418
34868	CRESY	2020-09-29	2.85	2.85	2.75	2.75	125982	{}	\N	2020-10-12 14:35:06.882827
34869	CRESY	2020-09-30	2.75	2.885	2.71	2.81	118221	{}	\N	2020-10-12 14:35:06.889715
34870	CRESY	2020-10-01	2.83	3	2.82	2.96	108362	{}	\N	2020-10-12 14:35:06.895659
34871	CRESY	2020-10-02	2.92	2.95	2.81	2.87	79984	{}	\N	2020-10-12 14:35:06.900931
34872	CRESY	2020-10-05	2.89	2.96	2.87	2.91	59634	{}	\N	2020-10-12 14:35:06.905779
34873	CRESY	2020-10-06	2.95	3.05	2.93	3.04	130433	{}	\N	2020-10-12 14:35:06.91022
34874	CRESY	2020-10-07	3.05	3.1354	2.99	3.06	133611	{}	\N	2020-10-12 14:35:06.914376
34875	CRESY	2020-10-08	3.0373	3.2	3.0373	3.18	52224	{}	\N	2020-10-12 14:35:06.918597
34876	CRESY	2020-10-09	3.18	3.2	3.05	3.11	89081	{}	\N	2020-10-12 14:35:06.922935
34877	CRESY	2020-10-12	3.11	3.17	3.0301	3.1456	22534	{}	\N	2020-10-12 14:35:06.927359
34878	CRM	2020-09-28	247.56	248.58	242.21	246.67	4215901	{}	\N	2020-10-12 14:35:08.352521
34879	CRM	2020-09-29	245.26	247.8965	244.5	247.45	8105326	{}	\N	2020-10-12 14:35:08.36044
34880	CRM	2020-09-30	246.65	255.4068	246.221	251.32	16252115	{}	\N	2020-10-12 14:35:08.366627
34881	CRM	2020-10-01	253.18	254.67	250.81	253.45	8556728	{}	\N	2020-10-12 14:35:08.371896
34882	CRM	2020-10-02	249.75	254.95	247.21	247.85	8821120	{}	\N	2020-10-12 14:35:08.377726
34883	CRM	2020-10-05	250	253.28	249	251.53	6886755	{}	\N	2020-10-12 14:35:08.382499
34884	CRM	2020-10-06	252.73	257.29	249	250.14	8757966	{}	\N	2020-10-12 14:35:08.387247
34885	CRM	2020-10-07	252.2	260.74	252.2	259.98	9776546	{}	\N	2020-10-12 14:35:08.391852
34886	CRM	2020-10-08	264.2	264.62	257.08	260.22	8368717	{}	\N	2020-10-12 14:35:08.396308
34887	CRM	2020-10-09	262.9	266.05	261.5	265.98	8503224	{}	\N	2020-10-12 14:35:08.400846
34888	CRM	2020-10-12	270	270	265.2927	267.5978	2000480	{}	\N	2020-10-12 14:35:08.405336
34889	CRSP	2020-09-28	85.12	85.12	81.96	83.69	505014	{}	\N	2020-10-12 14:35:10.063852
34890	CRSP	2020-09-29	84	85.255	82.64	84.99	462956	{}	\N	2020-10-12 14:35:10.080181
34891	CRSP	2020-09-30	84.82	86.19	83.15	83.65	371627	{}	\N	2020-10-12 14:35:10.086807
34892	CRSP	2020-10-01	83.93	85.09	82.1075	83.18	657041	{}	\N	2020-10-12 14:35:10.092833
34893	CRSP	2020-10-02	81	82.7899	79.31	79.67	760900	{}	\N	2020-10-12 14:35:10.098211
34894	CRSP	2020-10-05	81.95	87.405	81.31	86.75	822755	{}	\N	2020-10-12 14:35:10.10345
34895	CRSP	2020-10-06	87	89.817	85.85	87.72	680369	{}	\N	2020-10-12 14:35:10.107828
34896	CRSP	2020-10-07	91.72	99.87	90.04	97.68	2357828	{}	\N	2020-10-12 14:35:10.112021
34897	CRSP	2020-10-08	100	100	94.26	95.39	1214809	{}	\N	2020-10-12 14:35:10.116458
34898	CRSP	2020-10-09	97.78	100.88	96.325	98.26	1512331	{}	\N	2020-10-12 14:35:10.120653
34899	CRSP	2020-10-12	99.75	100.25	96.61	97.005	323524	{}	\N	2020-10-12 14:35:10.124923
34900	CS	2020-09-28	9.8	9.97	9.78	9.9	2196276	{}	\N	2020-10-12 14:35:11.787257
34901	CS	2020-09-29	9.87	9.9	9.73	9.83	3470950	{}	\N	2020-10-12 14:35:11.796914
34902	CS	2020-09-30	9.91	10.065	9.9039	9.97	4642002	{}	\N	2020-10-12 14:35:11.803373
34903	CS	2020-10-01	9.99	10.01	9.85	9.91	2988460	{}	\N	2020-10-12 14:35:11.808449
34904	CS	2020-10-02	9.86	10.14	9.84	10.07	4730601	{}	\N	2020-10-12 14:35:11.812846
34905	CS	2020-10-05	10.28	10.35	10.255	10.32	4518885	{}	\N	2020-10-12 14:35:11.81703
34906	CS	2020-10-06	10.54	10.54	10.25	10.3	5554714	{}	\N	2020-10-12 14:35:11.821257
34907	CS	2020-10-07	10.46	10.52	10.425	10.48	4354728	{}	\N	2020-10-12 14:35:11.824835
34908	CS	2020-10-08	10.57	10.59	10.48	10.57	3206803	{}	\N	2020-10-12 14:35:11.828355
34909	CS	2020-10-09	10.61	10.61	10.44	10.47	2908391	{}	\N	2020-10-12 14:35:11.832193
34910	CS	2020-10-12	10.55	10.57	10.52	10.56	581398	{}	\N	2020-10-12 14:35:11.835788
34911	CSGS	2020-09-28	40.9	41.42	40.845	41.15	259208	{}	\N	2020-10-12 14:35:14.386511
34912	CSGS	2020-09-29	41.06	41.52	40.83	41.15	130556	{}	\N	2020-10-12 14:35:14.39609
34913	CSGS	2020-09-30	41.34	41.69	40.83	40.95	158410	{}	\N	2020-10-12 14:35:14.403147
34914	CSGS	2020-10-01	40.99	41.55	40.62	41.02	111185	{}	\N	2020-10-12 14:35:14.40935
34915	CSGS	2020-10-02	40.56	41.67	40.33	41.37	143867	{}	\N	2020-10-12 14:35:14.414486
34916	CSGS	2020-10-05	41.62	42.1	41.37	41.86	112490	{}	\N	2020-10-12 14:35:14.419327
34917	CSGS	2020-10-06	42.07	42.51	41.27	41.46	158155	{}	\N	2020-10-12 14:35:14.423512
34918	CSGS	2020-10-07	41.59	41.64	41.01	41.15	201446	{}	\N	2020-10-12 14:35:14.427692
34919	CSGS	2020-10-08	41.38	41.8806	41.14	41.54	193133	{}	\N	2020-10-12 14:35:14.436017
34920	CSGS	2020-10-09	41.81	42.165	41.62	41.64	114189	{}	\N	2020-10-12 14:35:14.441854
34921	CSGS	2020-10-12	41.58	42.03	41.2	41.62	15419	{}	\N	2020-10-12 14:35:14.446242
34922	CSTE	2020-09-28	9.31	9.76	9.31	9.72	61414	{}	\N	2020-10-12 14:35:21.456917
34923	CSTE	2020-09-29	9.72	10.16	9.59	10.08	97415	{}	\N	2020-10-12 14:35:21.468067
34924	CSTE	2020-09-30	10.08	10.2	9.78	9.8	46071	{}	\N	2020-10-12 14:35:21.475354
34925	CSTE	2020-10-01	10	10.175	9.8	10.1	102466	{}	\N	2020-10-12 14:35:21.481222
34926	CSTE	2020-10-02	10	10.28	10	10.26	53046	{}	\N	2020-10-12 14:35:21.486081
34927	CSTE	2020-10-05	10.38	10.62	10.34	10.59	66769	{}	\N	2020-10-12 14:35:21.490907
34928	CSTE	2020-10-06	10.71	10.96	10.35	10.91	79499	{}	\N	2020-10-12 14:35:21.494827
34929	CSTE	2020-10-07	10.95	11.38	10.905	11.36	79040	{}	\N	2020-10-12 14:35:21.498658
34930	CSTE	2020-10-08	11.4	11.99	11.28	11.85	105418	{}	\N	2020-10-12 14:35:21.502649
34931	CSTE	2020-10-09	11.94	12.19	11.68	11.99	95213	{}	\N	2020-10-12 14:35:21.5065
34932	CSTE	2020-10-12	12.05	12.16	11.8169	12.06	6035	{}	\N	2020-10-12 14:35:21.51039
34933	CTBB	2020-09-28	25.31	25.4	25.245	25.36	81215	{}	\N	2020-10-12 14:35:23.163115
34934	CTBB	2020-09-29	25.3	25.36	25.23	25.23	99098	{}	\N	2020-10-12 14:35:23.175231
34935	CTBB	2020-09-30	25.3	25.33	25.04	25.04	422266	{}	\N	2020-10-12 14:35:23.185054
34936	CTBB	2020-10-01	25.08	25.39	25.08	25.27	155916	{}	\N	2020-10-12 14:35:23.190787
34937	CTBB	2020-10-02	25.13	25.32	25.12	25.29	111808	{}	\N	2020-10-12 14:35:23.195624
34938	CTBB	2020-10-05	25.29	25.42	25.27	25.36	182906	{}	\N	2020-10-12 14:35:23.200621
34939	CTBB	2020-10-06	25.37	25.41	25.3	25.33	175217	{}	\N	2020-10-12 14:35:23.20491
34940	CTBB	2020-10-07	25.33	25.4	25.32	25.32	164024	{}	\N	2020-10-12 14:35:23.209279
34941	CTBB	2020-10-08	25.36	25.5	25.34	25.5	105427	{}	\N	2020-10-12 14:35:23.213505
34942	CTBB	2020-10-09	25.46	25.48	25.36	25.48	98272	{}	\N	2020-10-12 14:35:23.217508
34943	CTBB	2020-10-12	25.48	25.48	25.38	25.39	51958	{}	\N	2020-10-12 14:35:23.221564
34944	CTLT	2020-09-28	83.21	84.66	83.07	84.11	876709	{}	\N	2020-10-12 14:35:24.429076
34945	CTLT	2020-09-29	83.95	85.818	83.55	85.19	1228328	{}	\N	2020-10-12 14:35:24.439367
34946	CTLT	2020-09-30	84.98	86.48	84.82	85.66	2170956	{}	\N	2020-10-12 14:35:24.445764
34947	CTLT	2020-10-01	86.46	87.03	85.53	86.33	1715813	{}	\N	2020-10-12 14:35:24.451626
34948	CTLT	2020-10-02	84.94	86.5	82.88	85.88	1861069	{}	\N	2020-10-12 14:35:24.456729
34949	CTLT	2020-10-05	86.71	89.53	86.5	89.47	1800911	{}	\N	2020-10-12 14:35:24.461514
34950	CTLT	2020-10-06	89.27	91.22	88.51	88.85	1855461	{}	\N	2020-10-12 14:35:24.465993
34951	CTLT	2020-10-07	89.77	91.84	89.54	91.53	2034901	{}	\N	2020-10-12 14:35:24.470215
34952	CTLT	2020-10-08	92.25	93.32	91.5	92.71	1446411	{}	\N	2020-10-12 14:35:24.474541
34953	CTLT	2020-10-09	93.34	94.52	93.06	94.48	1142890	{}	\N	2020-10-12 14:35:24.478724
34954	CTLT	2020-10-12	94.99	95.22	93.73	95	485821	{}	\N	2020-10-12 14:35:24.482921
34955	COLL	2020-09-28	18.65	19.04	18.4	18.78	339938	{}	\N	2020-10-12 14:35:25.714369
34956	COLL	2020-09-29	18.76	20.25	18.75	19.88	410101	{}	\N	2020-10-12 14:35:25.722808
34957	COLL	2020-09-30	20.035	20.9	20	20.82	400071	{}	\N	2020-10-12 14:35:25.727945
34958	COLL	2020-10-01	20.82	21.75	20.6	21.68	377912	{}	\N	2020-10-12 14:35:25.73371
34959	COLL	2020-10-02	21.4	21.89	21	21.22	397247	{}	\N	2020-10-12 14:35:25.738432
34960	COLL	2020-10-05	21.22	21.43	20.53	20.89	313025	{}	\N	2020-10-12 14:35:25.742729
34961	COLL	2020-10-06	21.03	21.76	21.03	21.15	265007	{}	\N	2020-10-12 14:35:25.746374
34962	COLL	2020-10-07	21.26	21.63	20.83	21.28	253751	{}	\N	2020-10-12 14:35:25.750354
34963	COLL	2020-10-08	21.48	22.52	21.2777	21.94	351311	{}	\N	2020-10-12 14:35:25.753892
34964	COLL	2020-10-09	22.06	22.35	21.79	22.29	353905	{}	\N	2020-10-12 14:35:25.75752
34965	COLL	2020-10-12	22.4	22.465	21.93	22.09	30770	{}	\N	2020-10-12 14:35:25.761036
34966	CORE	2020-09-28	28.83	29.48	28.62	29.12	268486	{}	\N	2020-10-12 14:35:28.48464
34967	CORE	2020-09-29	28.95	29.21	28.52	29.13	257586	{}	\N	2020-10-12 14:35:28.495565
34968	CORE	2020-09-30	29.36	29.72	28.83	28.93	226699	{}	\N	2020-10-12 14:35:28.502641
34969	CORE	2020-10-01	28.96	29.19	28.49	29.04	173243	{}	\N	2020-10-12 14:35:28.508215
34970	CORE	2020-10-02	28.63	29.33	28.48	29.15	107920	{}	\N	2020-10-12 14:35:28.512359
34971	CORE	2020-10-05	29.3	29.6	28.44	28.75	206744	{}	\N	2020-10-12 14:35:28.516269
34972	CORE	2020-10-06	29.01	30.02	28.76	29.23	373911	{}	\N	2020-10-12 14:35:28.519707
34973	CORE	2020-10-07	29.36	29.6	28.97	29.39	236354	{}	\N	2020-10-12 14:35:28.5233
34974	CORE	2020-10-08	29.57	30.09	29.22	29.93	157803	{}	\N	2020-10-12 14:35:28.526672
34975	CORE	2020-10-09	30.21	30.63	29.63	29.67	181195	{}	\N	2020-10-12 14:35:28.529941
34976	CORE	2020-10-12	29.8	30.11	29.62	29.995	21265	{}	\N	2020-10-12 14:35:28.533365
34977	COTY	2020-09-28	2.75	2.83	2.69	2.75	13052512	{}	\N	2020-10-12 14:35:29.482649
34978	COTY	2020-09-29	2.77	2.77	2.65	2.65	20988418	{}	\N	2020-10-12 14:35:29.489892
34979	COTY	2020-09-30	2.75	2.82	2.685	2.7	26512864	{}	\N	2020-10-12 14:35:29.494965
34980	COTY	2020-10-01	2.7	2.88	2.6998	2.85	22055030	{}	\N	2020-10-12 14:35:29.49948
34981	COTY	2020-10-02	2.73	2.93	2.7	2.91	18016961	{}	\N	2020-10-12 14:35:29.503537
34982	COTY	2020-10-05	2.95	3.025	2.88	3	19126692	{}	\N	2020-10-12 14:35:29.508057
34983	COTY	2020-10-06	3.08	3.225	3.03	3.04	28648215	{}	\N	2020-10-12 14:35:29.51222
34984	COTY	2020-10-07	3.13	3.39	3.13	3.33	32124508	{}	\N	2020-10-12 14:35:29.516294
34985	COTY	2020-10-08	3.53	3.82	3.435	3.74	64436405	{}	\N	2020-10-12 14:35:29.520225
34986	COTY	2020-10-09	3.8	3.83	3.5	3.55	39200459	{}	\N	2020-10-12 14:35:29.524187
34987	COTY	2020-10-12	3.53	3.55	3.45	3.525	4885144	{}	\N	2020-10-12 14:35:29.528081
34988	CPER	2020-09-28	18.48	18.545	18.4	18.51	45747	{}	\N	2020-10-12 14:35:30.587616
34989	CPER	2020-09-29	18.42	18.57	18.3349	18.5	15776	{}	\N	2020-10-12 14:35:30.599789
34990	CPER	2020-09-30	18.72	18.78	18.63	18.7	21811	{}	\N	2020-10-12 14:35:30.607384
34991	CPER	2020-10-01	18.35	18.35	17.7	17.86	57496	{}	\N	2020-10-12 14:35:30.613202
34992	CPER	2020-10-02	18.03	18.5	18.03	18.3	134542	{}	\N	2020-10-12 14:35:30.618085
34993	CPER	2020-10-05	18.08	18.31	18.0301	18.24	23413	{}	\N	2020-10-12 14:35:30.623307
34994	CPER	2020-10-06	18.33	18.36	18.1	18.15	18604	{}	\N	2020-10-12 14:35:30.627523
34995	CPER	2020-10-07	18.69	18.813	18.63	18.68	30952	{}	\N	2020-10-12 14:35:30.632107
34996	CPER	2020-10-08	18.735	18.81	18.72	18.77	13582	{}	\N	2020-10-12 14:35:30.636394
34997	CPER	2020-10-09	18.98	19.1688	18.9	19.05	28240	{}	\N	2020-10-12 14:35:30.640575
34998	CPER	2020-10-12	19.09	19.09	18.95	18.9881	5740	{}	\N	2020-10-12 14:35:30.64457
34999	CPS	2020-09-28	13.99	14.34	13.54	13.63	216104	{}	\N	2020-10-12 14:35:31.934048
35000	CPS	2020-09-29	13.44	13.47	12.52	13.11	590128	{}	\N	2020-10-12 14:35:31.943827
35001	CPS	2020-09-30	13.16	13.855	12.98	13.21	449052	{}	\N	2020-10-12 14:35:31.949902
35002	CPS	2020-10-01	13.44	13.65	12.62	13.28	417339	{}	\N	2020-10-12 14:35:31.955598
35003	CPS	2020-10-02	12.8	14.4262	12.69	14.27	372719	{}	\N	2020-10-12 14:35:31.960271
35004	CPS	2020-10-05	14.54	15.37	14.49	15.27	296361	{}	\N	2020-10-12 14:35:31.965089
35005	CPS	2020-10-06	15.59	16.3	15.35	15.41	307615	{}	\N	2020-10-12 14:35:31.969186
35006	CPS	2020-10-07	15.62	16.36	15.62	16.12	235326	{}	\N	2020-10-12 14:35:31.973695
35007	CPS	2020-10-08	16.36	16.84	15.86	16.76	174745	{}	\N	2020-10-12 14:35:31.977878
35008	CPS	2020-10-09	16.89	17.25	16.42	16.5	314184	{}	\N	2020-10-12 14:35:31.982096
35009	CPS	2020-10-12	16.55	16.67	16.36	16.67	34700	{}	\N	2020-10-12 14:35:31.985977
35010	CPTAG	2020-09-28	22.89	23	22.84	22.85	3463	{}	\N	2020-10-12 14:35:33.453443
35011	CPTAG	2020-09-29	22.83	23	22.51	23	2264	{}	\N	2020-10-12 14:35:33.460461
35012	CPTAG	2020-09-30	22.88	22.89	22.7	22.7	3703	{}	\N	2020-10-12 14:35:33.466085
35013	CPTAG	2020-10-05	22.9	22.9	22.79	22.79	1256	{}	\N	2020-10-12 14:35:33.46711
35014	CPTAG	2020-10-06	22.63	23.34	22.63	23.34	408	{}	\N	2020-10-12 14:35:33.472758
35015	CPTAG	2020-10-07	22.7	23.34	22.7	23.34	604	{}	\N	2020-10-12 14:35:33.477038
35016	CPTAG	2020-10-08	23.25	23.34	23.25	23.34	1446	{}	\N	2020-10-12 14:35:33.481308
35017	CPTAG	2020-10-09	23.26	23.34	23.26	23.34	3239	{}	\N	2020-10-12 14:35:33.485877
35018	CRMD	2020-09-28	5.29	5.46	5.158	5.38	332925	{}	\N	2020-10-12 14:35:34.90044
35019	CRMD	2020-09-29	5.78	6.03	5.52	5.9	913004	{}	\N	2020-10-12 14:35:34.910249
35020	CRMD	2020-09-30	6.11	6.11	5.84	6.03	511080	{}	\N	2020-10-12 14:35:34.916897
35021	CRMD	2020-10-01	6.04	6.05	5.9001	6.01	500236	{}	\N	2020-10-12 14:35:34.922996
35022	CRMD	2020-10-02	5.82	6.09	5.75	5.85	454933	{}	\N	2020-10-12 14:35:34.927916
35023	CRMD	2020-10-05	5.9	6.03	5.9	5.97	186246	{}	\N	2020-10-12 14:35:34.933049
35024	CRMD	2020-10-06	5.92	6.12	5.87	5.91	446934	{}	\N	2020-10-12 14:35:34.937289
35025	CRMD	2020-10-07	6.02	6.11	5.93	6.05	300514	{}	\N	2020-10-12 14:35:34.941492
35026	CRMD	2020-10-08	6.08	6.25	6.04	6.04	387422	{}	\N	2020-10-12 14:35:34.94563
35027	CRMD	2020-10-09	6.05	6.08	5.94	6.04	231129	{}	\N	2020-10-12 14:35:34.950092
35028	CRMD	2020-10-12	6.03	6.04	5.95	5.99	70282	{}	\N	2020-10-12 14:35:34.954135
35029	CRWS	2020-09-28	5.535	5.65	5.52	5.56	3287	{}	\N	2020-10-12 14:35:35.979866
35030	CRWS	2020-09-29	5.49	5.685	5.49	5.66	12125	{}	\N	2020-10-12 14:35:35.99229
35031	CRWS	2020-09-30	5.58	5.6809	5.58	5.65	3899	{}	\N	2020-10-12 14:35:35.999498
35032	CRWS	2020-10-01	5.61	5.72	5.6	5.72	7176	{}	\N	2020-10-12 14:35:36.005846
35033	CRWS	2020-10-02	5.72	5.72	5.62	5.7	21308	{}	\N	2020-10-12 14:35:36.010697
35034	CRWS	2020-10-05	5.67	5.74	5.65	5.71	8861	{}	\N	2020-10-12 14:35:36.015536
35035	CRWS	2020-10-06	5.7	5.74	5.65	5.65	4464	{}	\N	2020-10-12 14:35:36.019803
35036	CRWS	2020-10-07	5.65	5.74	5.64	5.69	8240	{}	\N	2020-10-12 14:35:36.023959
35037	CRWS	2020-10-08	5.6818	5.78	5.6818	5.77	7675	{}	\N	2020-10-12 14:35:36.028004
35038	CRWS	2020-10-09	5.6889	5.78	5.66	5.76	7846	{}	\N	2020-10-12 14:35:36.032077
35039	CRWS	2020-10-12	5.76	5.84	5.76	5.84	16129	{}	\N	2020-10-12 14:35:36.036184
35040	CSF	2020-09-28	39.55	40	39.55	39.9021	1873	{}	\N	2020-10-12 14:35:38.343383
35041	CSF	2020-09-29	39.76	39.76	39.76	39.76	1060	{}	\N	2020-10-12 14:35:38.35185
35042	CSF	2020-09-30	40.06	40.13	39.75	39.75	2097	{}	\N	2020-10-12 14:35:38.358188
35043	CSF	2020-10-01	39.8708	40.1034	39.8002	40.1034	5943	{}	\N	2020-10-12 14:35:38.363765
35044	CSF	2020-10-02	39.54	40.58	39.54	40.56	2980	{}	\N	2020-10-12 14:35:38.36883
35045	CSF	2020-10-05	41.19	41.47	41.19	41.47	38189	{}	\N	2020-10-12 14:35:38.373358
35046	CSF	2020-10-06	41.57	42.245	41.42	41.42	41572	{}	\N	2020-10-12 14:35:38.377907
35047	CSF	2020-10-07	41.9	42.02	41.69	42.01	2092	{}	\N	2020-10-12 14:35:38.382693
35048	CSF	2020-10-08	42.15	42.42	42.15	42.42	17894	{}	\N	2020-10-12 14:35:38.38786
35049	CSF	2020-10-09	42.72	42.72	42.48	42.62	11677	{}	\N	2020-10-12 14:35:38.392074
35050	CSF	2020-10-12	42.72	42.78	42.72	42.78	2119	{}	\N	2020-10-12 14:35:38.396274
35051	CSML	2020-09-28	22.11	22.27	22.11	22.27	2063	{}	\N	2020-10-12 14:35:39.75969
35052	CSML	2020-09-29	22.15	22.1799	21.98	22.1	14198	{}	\N	2020-10-12 14:35:39.771414
35053	CSML	2020-09-30	22.28	22.43	22.15	22.15	4082	{}	\N	2020-10-12 14:35:39.7787
35054	CSML	2020-10-01	22.0859	22.4317	22.0859	22.4317	5432	{}	\N	2020-10-12 14:35:39.784699
35055	CSML	2020-10-02	21.96	22.81	21.96	22.81	1791	{}	\N	2020-10-12 14:35:39.789994
35056	CSML	2020-10-05	23.05	23.2314	23.0301	23.2314	3281	{}	\N	2020-10-12 14:35:39.795116
35057	CSML	2020-10-06	23.55	23.75	23.2303	23.2303	1557	{}	\N	2020-10-12 14:35:39.799855
35058	CSML	2020-10-07	23.58	23.68	23.504	23.68	4142	{}	\N	2020-10-12 14:35:39.804347
35059	CSML	2020-10-08	23.7	24	23.7	24	660488	{}	\N	2020-10-12 14:35:39.808824
35060	CSML	2020-10-09	24.07	24.22	24.07	24.13	415	{}	\N	2020-10-12 14:35:39.813207
35061	CSML	2020-10-12	24.13	24.15	24.13	24.15	359	{}	\N	2020-10-12 14:35:39.817506
35062	CSTM	2020-09-28	7.64	7.74	7.5	7.73	466292	{}	\N	2020-10-12 14:35:40.779117
35063	CSTM	2020-09-29	7.81	7.98	7.66	7.78	921487	{}	\N	2020-10-12 14:35:40.789912
35064	CSTM	2020-09-30	7.78	8.06	7.71	7.85	929326	{}	\N	2020-10-12 14:35:40.797048
35065	CSTM	2020-10-01	7.9	8.12	7.88	8.09	1069706	{}	\N	2020-10-12 14:35:40.803702
35066	CSTM	2020-10-02	7.87	8.33	7.84	8.28	1237812	{}	\N	2020-10-12 14:35:40.809023
35067	CSTM	2020-10-05	8.38	8.64	8.38	8.53	1636272	{}	\N	2020-10-12 14:35:40.814162
35068	CSTM	2020-10-06	8.65	8.77	7.96	8	2272522	{}	\N	2020-10-12 14:35:40.818436
35069	CSTM	2020-10-07	8.2	8.6	8.2	8.57	1798672	{}	\N	2020-10-12 14:35:40.822584
35070	CSTM	2020-10-08	8.64	8.92	8.56	8.91	1818459	{}	\N	2020-10-12 14:35:40.826628
35071	CSTM	2020-10-09	9.02	9.27	8.87	9.23	1777127	{}	\N	2020-10-12 14:35:40.830494
35072	CSTM	2020-10-12	9.16	9.19	8.99	9.01	176696	{}	\N	2020-10-12 14:35:40.834464
35073	CTHR	2020-09-28	0.83	0.84	0.8201	0.837	56972	{}	\N	2020-10-12 14:35:44.484465
35074	CTHR	2020-09-29	0.84	0.84	0.82	0.8299	38110	{}	\N	2020-10-12 14:35:44.493686
35075	CTHR	2020-09-30	0.82	0.8399	0.7886	0.8201	59768	{}	\N	2020-10-12 14:35:44.499292
35076	CTHR	2020-10-01	0.83	0.8901	0.83	0.8845	264349	{}	\N	2020-10-12 14:35:44.504511
35077	CTHR	2020-10-02	0.85	0.9025	0.842086	0.88	223183	{}	\N	2020-10-12 14:35:44.509253
35078	CTHR	2020-10-05	0.893	0.9	0.8805	0.889	70399	{}	\N	2020-10-12 14:35:44.513716
35079	CTHR	2020-10-06	0.9	1.005	0.893	0.938	577484	{}	\N	2020-10-12 14:35:44.51822
35080	CTHR	2020-10-07	0.995	0.995	0.913	0.915	190354	{}	\N	2020-10-12 14:35:44.522103
35081	CTHR	2020-10-08	0.953	0.963	0.886	0.8892	155024	{}	\N	2020-10-12 14:35:44.526085
35082	CTHR	2020-10-09	0.9135	0.9298	0.8956	0.91	49644	{}	\N	2020-10-12 14:35:44.530332
35083	CTHR	2020-10-12	0.9448	0.9448	0.9022	0.9138	54093	{}	\N	2020-10-12 14:35:44.534559
35084	COWZ	2020-09-28	28.05	28.24	28.03	28.17	39025	{}	\N	2020-10-12 14:35:46.578876
35085	COWZ	2020-09-29	28.23	28.23	27.8199	27.94	22968	{}	\N	2020-10-12 14:35:46.587188
35086	COWZ	2020-09-30	28.09	28.4308	28	28.19	57752	{}	\N	2020-10-12 14:35:46.592371
35087	COWZ	2020-10-01	28.4	28.409	28.04	28.22	191142	{}	\N	2020-10-12 14:35:46.597369
35088	COWZ	2020-10-02	27.66	28.525	27.66	28.5	13426	{}	\N	2020-10-12 14:35:46.601866
35089	COWZ	2020-10-05	28.75	29.0299	28.75	29	99809	{}	\N	2020-10-12 14:35:46.606213
35090	COWZ	2020-10-06	29.07	29.305	28.62	28.66	31699	{}	\N	2020-10-12 14:35:46.610134
35091	COWZ	2020-10-07	28.9	29.275	28.9	29.22	57168	{}	\N	2020-10-12 14:35:46.613908
35092	COWZ	2020-10-08	29.49	29.7146	29.49	29.7146	12502	{}	\N	2020-10-12 14:35:46.617669
35093	COWZ	2020-10-09	29.9	29.9	29.61	29.65	51614	{}	\N	2020-10-12 14:35:46.621299
35094	COWZ	2020-10-12	29.75	29.7574	29.65	29.74	3081	{}	\N	2020-10-12 14:35:46.624997
35095	CPI	2020-09-28	27.2592	27.2592	27.2592	27.2592	20	{}	\N	2020-10-12 14:35:48.237726
35096	CPI	2020-09-29	27.2438	27.2438	27.2438	27.2438	53	{}	\N	2020-10-12 14:35:48.244978
35097	CPI	2020-09-30	27.27	27.28	27.249	27.249	11942	{}	\N	2020-10-12 14:35:48.250677
35098	CPI	2020-10-01	27.28	27.2998	27.28	27.2998	3448	{}	\N	2020-10-12 14:35:48.255754
35099	CPI	2020-10-02	27.305	27.3248	27.305	27.3248	936	{}	\N	2020-10-12 14:35:48.260849
35100	CPI	2020-10-05	27.3349	27.3349	27.3349	27.3349	47	{}	\N	2020-10-12 14:35:48.265243
35101	CPI	2020-10-06	27.3211	27.3211	27.3211	27.3211	6	{}	\N	2020-10-12 14:35:48.269475
35102	CPI	2020-10-07	27.315	27.35	27.315	27.32	4422	{}	\N	2020-10-12 14:35:48.273545
35103	CPI	2020-10-08	27.3749	27.3749	27.3749	27.3749	11	{}	\N	2020-10-12 14:35:48.277546
35104	CPI	2020-10-09	27.3503	27.3748	27.3503	27.3748	230	{}	\N	2020-10-12 14:35:48.281547
35105	CPRX	2020-09-28	3.16	3.19	3.07	3.16	703512	{}	\N	2020-10-12 14:35:49.668906
35106	CPRX	2020-09-29	3.17	3.36	3.02	3.21	4157443	{}	\N	2020-10-12 14:35:49.682434
35107	CPRX	2020-09-30	3.18	3.25	2.91	2.97	4995019	{}	\N	2020-10-12 14:35:49.689883
35108	CPRX	2020-10-01	3.03	3.18	2.995	3.13	2676058	{}	\N	2020-10-12 14:35:49.695705
35109	CPRX	2020-10-02	3.05	3.12	3.02	3.05	1086046	{}	\N	2020-10-12 14:35:49.700656
35110	CPRX	2020-10-05	3.08	3.21	3.08	3.21	1186027	{}	\N	2020-10-12 14:35:49.705545
35111	CPRX	2020-10-06	3.17	3.3595	3.075	3.11	3444138	{}	\N	2020-10-12 14:35:49.709826
35112	CPRX	2020-10-07	3.14	3.27	3.14	3.25	1614514	{}	\N	2020-10-12 14:35:49.7139
35113	CPRX	2020-10-08	3.299	3.37	3.26	3.3	1177960	{}	\N	2020-10-12 14:35:49.718047
35114	CPRX	2020-10-09	3.33	3.33	3.23	3.29	853600	{}	\N	2020-10-12 14:35:49.72216
35115	CPRX	2020-10-12	3.31	3.32	3.235	3.27	197850	{}	\N	2020-10-12 14:35:49.726458
35116	CRBN	2020-09-28	130.45	130.83	130.195	130.68	123646	{}	\N	2020-10-12 14:35:51.010148
35117	CRBN	2020-09-29	130.67	130.8	130.15	130.44	2829	{}	\N	2020-10-12 14:35:51.019046
35118	CRBN	2020-09-30	130.65	131.69	130.5	131	2527	{}	\N	2020-10-12 14:35:51.024304
35119	CRBN	2020-10-01	131.91	131.945	131.2972	131.825	4667	{}	\N	2020-10-12 14:35:51.029778
35120	CRBN	2020-10-02	129.64	131.37	129.64	131.025	11821	{}	\N	2020-10-12 14:35:51.034553
35121	CRBN	2020-10-05	131.91	133.06	131.91	133	10063	{}	\N	2020-10-12 14:35:51.039135
35122	CRBN	2020-10-06	133.17	133.45	131.6578	131.6578	3673	{}	\N	2020-10-12 14:35:51.043324
35123	CRBN	2020-10-07	132.77	133.64	132.77	133.64	4497	{}	\N	2020-10-12 14:35:51.047233
35124	CRBN	2020-10-08	134.24	134.479	134.15	134.478	8823	{}	\N	2020-10-12 14:35:51.051087
35125	CRBN	2020-10-09	135.07	135.65	135.065	135.4205	2366	{}	\N	2020-10-12 14:35:51.05501
35126	CRBN	2020-10-12	136.61	136.61	136.245	136.45	1030	{}	\N	2020-10-12 14:35:51.058927
35127	CRHM	2020-09-28	2.16	2.16	2.05	2.06	24491	{}	\N	2020-10-12 14:35:52.072151
35128	CRHM	2020-09-29	2.05	2.09	2.03	2.08	16563	{}	\N	2020-10-12 14:35:52.08132
35129	CRHM	2020-09-30	2.1099	2.1799	2.08	2.17	147566	{}	\N	2020-10-12 14:35:52.087292
35130	CRHM	2020-10-01	2.18	2.18	2.15	2.17	18167	{}	\N	2020-10-12 14:35:52.092457
35131	CRHM	2020-10-02	2	2.17	2	2.12	16086	{}	\N	2020-10-12 14:35:52.097301
35132	CRHM	2020-10-05	2.175	2.26	2.17	2.25	59903	{}	\N	2020-10-12 14:35:52.102569
35133	CRHM	2020-10-06	2.22	2.35	2.22	2.25	41149	{}	\N	2020-10-12 14:35:52.106936
35134	CRHM	2020-10-07	2.26	2.34	2.26	2.31	53562	{}	\N	2020-10-12 14:35:52.110921
35135	CRHM	2020-10-08	2.34	2.35	2.26	2.35	55746	{}	\N	2020-10-12 14:35:52.114642
35136	CRHM	2020-10-09	2.35	2.35	2.3	2.31	14666	{}	\N	2020-10-12 14:35:52.118412
35137	CRHM	2020-10-12	2.33	2.33	2.33	2.33	878	{}	\N	2020-10-12 14:35:52.122397
35138	CSA	2020-09-28	40.72	41.22	40.72	41.18	739	{}	\N	2020-10-12 14:35:53.717201
35139	CSA	2020-09-29	40.73	40.89	40.73	40.89	2509	{}	\N	2020-10-12 14:35:53.728875
35140	CSA	2020-09-30	41.23	41.42	40.9481	40.9481	709	{}	\N	2020-10-12 14:35:53.73525
35141	CSA	2020-10-01	40.91	40.91	40.91	40.91	181	{}	\N	2020-10-12 14:35:53.741136
35142	CSA	2020-10-05	42.74	42.89	42.74	42.89	460	{}	\N	2020-10-12 14:35:53.746262
35143	CSA	2020-10-06	43.5785	43.5785	43.2035	43.2035	548	{}	\N	2020-10-12 14:35:53.751168
35144	CSA	2020-10-07	43.63	43.88	43.63	43.88	919	{}	\N	2020-10-12 14:35:53.755546
35145	CSA	2020-10-08	44.27	44.5265	44.27	44.5265	2166	{}	\N	2020-10-12 14:35:53.759806
35146	CSA	2020-10-09	44.5171	44.66	44.5	44.66	863	{}	\N	2020-10-12 14:35:53.764014
35147	CSA	2020-10-12	44.86	44.9035	44.86	44.8704	2800	{}	\N	2020-10-12 14:35:53.768288
35148	CSLT	2020-09-28	1.14	1.14	1.09	1.1	165671	{}	\N	2020-10-12 14:35:55.038388
35149	CSLT	2020-09-29	1.1	1.13	1.06	1.08	330530	{}	\N	2020-10-12 14:35:55.047419
35150	CSLT	2020-09-30	1.1	1.17	1.08	1.13	360781	{}	\N	2020-10-12 14:35:55.054009
35151	CSLT	2020-10-01	1.14	1.15	1.07	1.11	476916	{}	\N	2020-10-12 14:35:55.060658
35152	CSLT	2020-10-02	1.09	1.145	1.07	1.08	531264	{}	\N	2020-10-12 14:35:55.065175
35153	CSLT	2020-10-05	1.08	1.1	1.07	1.08	671994	{}	\N	2020-10-12 14:35:55.069853
35154	CSLT	2020-10-06	1.07	1.1	1.06	1.06	414646	{}	\N	2020-10-12 14:35:55.074043
35155	CSLT	2020-10-07	1.06	1.07	1.02	1.04	906508	{}	\N	2020-10-12 14:35:55.077866
35156	CSLT	2020-10-08	1.03	1.07	1.02	1.04	591203	{}	\N	2020-10-12 14:35:55.081933
35157	CSLT	2020-10-09	1.04	1.05	1.02	1.04	424060	{}	\N	2020-10-12 14:35:55.085853
35158	CSLT	2020-10-12	1.04	1.05	1.03	1.045	227131	{}	\N	2020-10-12 14:35:55.089818
35159	CSTR	2020-09-28	9.23	9.6796	9.23	9.59	21150	{}	\N	2020-10-12 14:35:55.990455
35160	CSTR	2020-09-29	9.66	9.66	9.43	9.62	18755	{}	\N	2020-10-12 14:35:55.997951
35161	CSTR	2020-09-30	9.73	9.955	9.645	9.81	23961	{}	\N	2020-10-12 14:35:56.003304
35162	CSTR	2020-10-01	9.89	10.005	9.63	9.71	18939	{}	\N	2020-10-12 14:35:56.008234
35163	CSTR	2020-10-02	10	10.0901	9.81	9.87	22265	{}	\N	2020-10-12 14:35:56.012828
35164	CSTR	2020-10-05	9.49	10.31	9.49	10.21	25077	{}	\N	2020-10-12 14:35:56.017092
35165	CSTR	2020-10-06	10.365	10.57	10.19	10.19	15321	{}	\N	2020-10-12 14:35:56.020914
35166	CSTR	2020-10-07	10.38	10.75	10.38	10.71	18882	{}	\N	2020-10-12 14:35:56.024309
35167	CSTR	2020-10-08	10.75	10.94	10.4106	10.83	34992	{}	\N	2020-10-12 14:35:56.027825
35168	CSTR	2020-10-09	10.785	10.82	10.61	10.65	4020	{}	\N	2020-10-12 14:35:56.031361
35169	CSTR	2020-10-12	10.7	10.7	10.405	10.51	901	{}	\N	2020-10-12 14:35:56.034942
35170	CSX	2020-09-28	79.16	79.72	78.5	79.33	2684890	{}	\N	2020-10-12 14:35:57.123864
35171	CSX	2020-09-29	78.96	79.46	77.57	77.6	3579171	{}	\N	2020-10-12 14:35:57.130909
35172	CSX	2020-09-30	78.39	79.23	77.38	77.67	4403158	{}	\N	2020-10-12 14:35:57.136479
35173	CSX	2020-10-01	78.35	78.35	76.27	76.86	3498873	{}	\N	2020-10-12 14:35:57.141147
35174	CSX	2020-10-02	76.78	77.87	76.31	77.15	3248699	{}	\N	2020-10-12 14:35:57.146207
35175	CSX	2020-10-05	77.49	78.29	77.31	77.74	2768863	{}	\N	2020-10-12 14:35:57.150603
35176	CSX	2020-10-06	77.97	78.4	76.47	76.72	2955175	{}	\N	2020-10-12 14:35:57.154824
35177	CSX	2020-10-07	77.11	79.15	77.04	78.69	3368855	{}	\N	2020-10-12 14:35:57.158846
35178	CSX	2020-10-08	78.9084	79.2	77.2	77.72	3083635	{}	\N	2020-10-12 14:35:57.163013
35179	CSX	2020-10-09	77.97	78.94	77.97	78.48	1941159	{}	\N	2020-10-12 14:35:57.166873
35180	CSX	2020-10-12	78.93	79.41	78.43	78.98	802602	{}	\N	2020-10-12 14:35:57.171115
35181	CP	2020-09-28	303.4	305.3995	300.59	301.7	416578	{}	\N	2020-10-12 14:35:58.467809
35182	CP	2020-09-29	301.37	304.66	300.6	303.8	669128	{}	\N	2020-10-12 14:35:58.480078
35183	CP	2020-09-30	304.3	307.13	302.75	304.43	754866	{}	\N	2020-10-12 14:35:58.486807
35184	CP	2020-10-01	305.85	306.57	301.18	302.16	560392	{}	\N	2020-10-12 14:35:58.492124
35185	CP	2020-10-02	298.48	304.15	298.48	303.55	503155	{}	\N	2020-10-12 14:35:58.496968
35186	CP	2020-10-05	305.8	307.06	304.22	305.53	370894	{}	\N	2020-10-12 14:35:58.502285
35187	CP	2020-10-06	306.26	307.25	300.96	301.41	709520	{}	\N	2020-10-12 14:35:58.506905
35188	CP	2020-10-07	302.99	311.1	302.99	309.68	518430	{}	\N	2020-10-12 14:35:58.510925
35189	CP	2020-10-08	311.42	312.435	308.66	309.98	410384	{}	\N	2020-10-12 14:35:58.514925
35190	CP	2020-10-09	312.61	316.96	312.15	313.94	668177	{}	\N	2020-10-12 14:35:58.518927
35191	CP	2020-10-12	315.7	317.11	314.67	315.23	45502	{}	\N	2020-10-12 14:35:58.522834
35192	CPF	2020-09-28	13.23	13.92	13.19	13.76	280695	{}	\N	2020-10-12 14:35:59.592056
35193	CPF	2020-09-29	13.73	13.73	13.24	13.44	194455	{}	\N	2020-10-12 14:35:59.601231
35194	CPF	2020-09-30	13.5	13.78	13.4	13.57	319706	{}	\N	2020-10-12 14:35:59.608092
35195	CPF	2020-10-01	13.41	13.63	13.165	13.44	255580	{}	\N	2020-10-12 14:35:59.613914
35196	CPF	2020-10-02	13.23	13.99	13.23	13.92	255978	{}	\N	2020-10-12 14:35:59.619149
35197	CPF	2020-10-05	14.12	14.4	13.44	14.36	234229	{}	\N	2020-10-12 14:35:59.624146
35198	CPF	2020-10-06	14.67	14.86	14.25	14.27	314236	{}	\N	2020-10-12 14:35:59.628696
35199	CPF	2020-10-07	14.5	14.75	14.32	14.51	321567	{}	\N	2020-10-12 14:35:59.633041
35200	CPF	2020-10-08	14.7	14.78	14.365	14.62	217884	{}	\N	2020-10-12 14:35:59.63708
35201	CPF	2020-10-09	14.64	14.81	14.4	14.42	199369	{}	\N	2020-10-12 14:35:59.641193
35202	CPF	2020-10-12	14.35	14.57	14.31	14.57	12510	{}	\N	2020-10-12 14:35:59.645387
35203	CPSI	2020-09-28	26.81	27.88	26.645	27.61	58615	{}	\N	2020-10-12 14:36:00.655017
35204	CPSI	2020-09-29	27.57	27.89	27.41	27.49	46868	{}	\N	2020-10-12 14:36:00.664874
35205	CPSI	2020-09-30	27.48	28.03	27.01	27.61	69640	{}	\N	2020-10-12 14:36:00.671813
35206	CPSI	2020-10-01	27.65	27.88	27.1325	27.68	48295	{}	\N	2020-10-12 14:36:00.677729
35207	CPSI	2020-10-02	27.16	27.715	26.96	27.29	55438	{}	\N	2020-10-12 14:36:00.682891
35208	CPSI	2020-10-05	27.53	27.94	27.45	27.75	39943	{}	\N	2020-10-12 14:36:00.687987
35209	CPSI	2020-10-06	27.55	28.75	27.45	27.66	78959	{}	\N	2020-10-12 14:36:00.692332
35210	CPSI	2020-10-07	27.99	28.03	27.27	27.27	68734	{}	\N	2020-10-12 14:36:00.696548
35211	CPSI	2020-10-08	27.46	28.19	27.24	28.13	78059	{}	\N	2020-10-12 14:36:00.701035
35212	CPSI	2020-10-09	28.44	28.54	27.94	28.25	29935	{}	\N	2020-10-12 14:36:00.705356
35213	CPSI	2020-10-12	28.2	28.28	27.65	28.17	4431	{}	\N	2020-10-12 14:36:00.709702
35214	CRAI	2020-09-28	38.65	39.69	38.2816	39.04	18507	{}	\N	2020-10-12 14:36:02.096662
35215	CRAI	2020-09-29	38.86	39.055	37.65	37.65	24141	{}	\N	2020-10-12 14:36:02.105567
35216	CRAI	2020-09-30	37.75	39.035	36.9275	37.47	58112	{}	\N	2020-10-12 14:36:02.111262
35217	CRAI	2020-10-01	37.61	38.22	37.3	37.63	24890	{}	\N	2020-10-12 14:36:02.116599
35218	CRAI	2020-10-02	37.07	38.85	36.32	38.11	26230	{}	\N	2020-10-12 14:36:02.121264
35219	CRAI	2020-10-05	38.28	39.72	38.28	39.72	29337	{}	\N	2020-10-12 14:36:02.126245
35220	CRAI	2020-10-06	39.5	39.98	38.39	38.46	44143	{}	\N	2020-10-12 14:36:02.130823
35221	CRAI	2020-10-07	38.83	39.52	37.5	38.49	22508	{}	\N	2020-10-12 14:36:02.134711
35222	CRAI	2020-10-08	38.9	39.345	37.65	39.03	17033	{}	\N	2020-10-12 14:36:02.138366
35223	CRAI	2020-10-09	39	39.5	38.45	39.14	29328	{}	\N	2020-10-12 14:36:02.141877
35224	CRAI	2020-10-12	39.13	39.13	38.56	39	1997	{}	\N	2020-10-12 14:36:02.14544
35225	CRF	2020-09-28	10.77	10.81	10.69	10.75	374811	{}	\N	2020-10-12 14:36:03.512769
35226	CRF	2020-09-29	10.85	10.86	10.78	10.86	258768	{}	\N	2020-10-12 14:36:03.524299
35227	CRF	2020-09-30	10.86	11.039	10.86	11	366666	{}	\N	2020-10-12 14:36:03.531103
35228	CRF	2020-10-01	11.09	11.1	10.98	11.05	390508	{}	\N	2020-10-12 14:36:03.536845
35229	CRF	2020-10-02	10.76	11.01	10.76	10.975	399545	{}	\N	2020-10-12 14:36:03.541827
35230	CRF	2020-10-05	11.04	11.15	11.0102	11.15	260002	{}	\N	2020-10-12 14:36:03.54702
35231	CRF	2020-10-06	11.13	11.1799	11.01	11.03	523850	{}	\N	2020-10-12 14:36:03.551499
35232	CRF	2020-10-07	11.11	11.14	11.05	11.1	282214	{}	\N	2020-10-12 14:36:03.555378
35233	CRF	2020-10-08	11.12	11.14	11.06	11.09	293891	{}	\N	2020-10-12 14:36:03.559156
35234	CRF	2020-10-09	11.15	11.18	11.08	11.13	309769	{}	\N	2020-10-12 14:36:03.563014
35235	CRF	2020-10-12	11.17	11.19	11.1317	11.19	97766	{}	\N	2020-10-12 14:36:03.56671
35236	CRNT	2020-09-28	2.44	2.49	2.4	2.46	360458	{}	\N	2020-10-12 14:36:04.996866
35237	CRNT	2020-09-29	2.45	2.52	2.42	2.49	224171	{}	\N	2020-10-12 14:36:05.006147
35238	CRNT	2020-09-30	2.49	2.54	2.46	2.49	297968	{}	\N	2020-10-12 14:36:05.011888
35239	CRNT	2020-10-01	2.52	2.53	2.42	2.42	311807	{}	\N	2020-10-12 14:36:05.016904
35240	CRNT	2020-10-02	2.36	2.46	2.31	2.44	218725	{}	\N	2020-10-12 14:36:05.021659
35241	CRNT	2020-10-05	2.48	2.52	2.45	2.49	200607	{}	\N	2020-10-12 14:36:05.025883
35242	CRNT	2020-10-06	2.5	2.53	2.45	2.47	195941	{}	\N	2020-10-12 14:36:05.029775
35243	CRNT	2020-10-07	2.48	2.5	2.4	2.42	237155	{}	\N	2020-10-12 14:36:05.03394
35244	CRNT	2020-10-08	2.43	2.525	2.42	2.51	250231	{}	\N	2020-10-12 14:36:05.037822
35245	CRNT	2020-10-09	2.53	2.55	2.44	2.47	602776	{}	\N	2020-10-12 14:36:05.041715
35246	CRNT	2020-10-12	2.5	2.51	2.41	2.4716	204000	{}	\N	2020-10-12 14:36:05.045575
35247	CRVS	2020-09-28	4.21	4.21	3.92	4.09	136333	{}	\N	2020-10-12 14:36:06.301258
35248	CRVS	2020-09-29	4.11	4.18	3.89	3.93	130112	{}	\N	2020-10-12 14:36:06.314358
35249	CRVS	2020-09-30	3.9	4.06	3.9	4.01	54044	{}	\N	2020-10-12 14:36:06.320316
35250	CRVS	2020-10-01	4.01	4.185	3.9723	4.09	89767	{}	\N	2020-10-12 14:36:06.32587
35251	CRVS	2020-10-02	4.06	4.14	3.96	4.07	69573	{}	\N	2020-10-12 14:36:06.331149
35252	CRVS	2020-10-05	5.71	5.74	4.7796	4.86	41139555	{}	\N	2020-10-12 14:36:06.335991
35253	CRVS	2020-10-06	4.705	4.775	4.16	4.17	1608851	{}	\N	2020-10-12 14:36:06.340517
35254	CRVS	2020-10-07	4.28	4.3297	4.15	4.21	477810	{}	\N	2020-10-12 14:36:06.344782
35255	CRVS	2020-10-08	4.16	4.25	4	4.04	678091	{}	\N	2020-10-12 14:36:06.349062
35256	CRVS	2020-10-09	4.06	4.1	3.96	3.99	390228	{}	\N	2020-10-12 14:36:06.353012
35257	CRVS	2020-10-12	4.01	4.03	3.78	3.79	258176	{}	\N	2020-10-12 14:36:06.356836
35258	CSCO	2020-09-28	39.195	39.29	38.79	39.13	23008446	{}	\N	2020-10-12 14:36:07.402794
35259	CSCO	2020-09-29	39.15	39.375	38.83	39.06	15510673	{}	\N	2020-10-12 14:36:07.412828
35260	CSCO	2020-09-30	39.09	39.7	39.02	39.39	24986646	{}	\N	2020-10-12 14:36:07.4195
35261	CSCO	2020-10-01	39.075	39.31	38.565	38.81	17104825	{}	\N	2020-10-12 14:36:07.42496
35262	CSCO	2020-10-02	38.046	38.55	37.97	38.27	22496554	{}	\N	2020-10-12 14:36:07.429977
35263	CSCO	2020-10-05	38.48	38.79	38.285	38.57	26557949	{}	\N	2020-10-12 14:36:07.435234
35264	CSCO	2020-10-06	38.5	39.35	38.4	38.57	22558748	{}	\N	2020-10-12 14:36:07.439138
35265	CSCO	2020-10-07	38.98	39.58	38.78	39.4	20043318	{}	\N	2020-10-12 14:36:07.443104
35266	CSCO	2020-10-08	39.8	40.18	39.66	39.79	18480232	{}	\N	2020-10-12 14:36:07.447084
35267	CSCO	2020-10-09	40.145	40.23	39.72	39.85	14842451	{}	\N	2020-10-12 14:36:07.450994
35268	CSCO	2020-10-12	39.93	40	39.63	39.915	4654643	{}	\N	2020-10-12 14:36:07.454775
35269	CSL	2020-09-28	121.17	122.42	120.6	121.22	272261	{}	\N	2020-10-12 14:36:09.423426
35270	CSL	2020-09-29	121.32	121.48	119.33	119.98	328080	{}	\N	2020-10-12 14:36:09.434998
35271	CSL	2020-09-30	120.92	123.41	120.74	122.37	806845	{}	\N	2020-10-12 14:36:09.441615
35272	CSL	2020-10-01	122.64	124.81	121.58	122.82	553378	{}	\N	2020-10-12 14:36:09.448041
35273	CSL	2020-10-02	120.77	124.78	120.54	124.17	338261	{}	\N	2020-10-12 14:36:09.453681
35274	CSL	2020-10-05	125.21	127.77	125.07	125.88	372459	{}	\N	2020-10-12 14:36:09.458561
35275	CSL	2020-10-06	126.09	128.68	125.01	125.51	417169	{}	\N	2020-10-12 14:36:09.463449
35276	CSL	2020-10-07	126.38	128.4	126.28	128.02	580920	{}	\N	2020-10-12 14:36:09.467921
35277	CSL	2020-10-08	128.68	129.84	127.81	128.28	562719	{}	\N	2020-10-12 14:36:09.472112
35278	CSL	2020-10-09	129.6	130	127.06	127.41	436571	{}	\N	2020-10-12 14:36:09.476472
35279	CSL	2020-10-12	127.58	129.64	127.3	129.43	58654	{}	\N	2020-10-12 14:36:09.482885
35280	CSU	2020-09-28	0.6	0.656	0.6	0.6252	112563	{}	\N	2020-10-12 14:36:11.701165
35281	CSU	2020-09-29	0.6306	0.6711	0.625	0.6543	97352	{}	\N	2020-10-12 14:36:11.715551
35282	CSU	2020-09-30	0.6583	0.68	0.6251	0.6282	107834	{}	\N	2020-10-12 14:36:11.725855
35283	CSU	2020-10-01	0.628	0.6678	0.62	0.6363	116289	{}	\N	2020-10-12 14:36:11.73507
35284	CSU	2020-10-02	0.6231	0.6487	0.62	0.6447	65584	{}	\N	2020-10-12 14:36:11.741276
35285	CSU	2020-10-05	0.67	0.67	0.625	0.6365	190394	{}	\N	2020-10-12 14:36:11.746281
35286	CSU	2020-10-06	0.6683	0.6683	0.6201	0.623	103129	{}	\N	2020-10-12 14:36:11.751377
35287	CSU	2020-10-07	0.6378	0.6471	0.6	0.6378	445372	{}	\N	2020-10-12 14:36:11.758201
35288	CSU	2020-10-08	0.6285	0.65	0.6252	0.65	158076	{}	\N	2020-10-12 14:36:11.766458
35289	CSU	2020-10-09	0.6581	0.6581	0.6246	0.6382	157545	{}	\N	2020-10-12 14:36:11.774798
35290	CSU	2020-10-12	0.63	0.647	0.6218	0.6401	11486	{}	\N	2020-10-12 14:36:11.780701
35291	CTDD	2020-09-28	26.17	26.1996	26.12	26.1566	43006	{}	\N	2020-10-12 14:36:12.74991
35292	CTDD	2020-09-29	26.07	26.1421	25.92	26.01	56162	{}	\N	2020-10-12 14:36:12.760173
35293	CTDD	2020-09-30	25.97	26.07	25.76	25.77	282196	{}	\N	2020-10-12 14:36:12.771986
35294	CTDD	2020-10-01	25.93	26.1	25.82	26.05	63876	{}	\N	2020-10-12 14:36:12.778633
35295	CTDD	2020-10-02	25.82	26.04	25.82	26.03	54275	{}	\N	2020-10-12 14:36:12.7836
35296	CTDD	2020-10-05	26	26.18	25.9701	26.1	80554	{}	\N	2020-10-12 14:36:12.788678
35297	CTDD	2020-10-06	26.11	26.22	25.91	26.2	83997	{}	\N	2020-10-12 14:36:12.793083
35298	CTDD	2020-10-07	26.19	26.3	26.18	26.28	66776	{}	\N	2020-10-12 14:36:12.796906
35299	CTDD	2020-10-08	26.3	26.305	26.26	26.3	53085	{}	\N	2020-10-12 14:36:12.800766
35300	CTDD	2020-10-09	26.25	26.33	26.25	26.33	33376	{}	\N	2020-10-12 14:36:12.804694
35301	CTDD	2020-10-12	26.26	26.33	26.26	26.26	8602	{}	\N	2020-10-12 14:36:12.808402
35302	CTO	2020-09-28	41.24	43.11	41.24	42.38	7830	{}	\N	2020-10-12 14:36:13.986809
35303	CTO	2020-09-29	42.7449	42.92	41.68	42.1	19998	{}	\N	2020-10-12 14:36:13.994323
35304	CTO	2020-09-30	43.5	46	43.5	44.1	45170	{}	\N	2020-10-12 14:36:13.999737
35305	CTO	2020-10-01	47	47	44.425	44.91	34539	{}	\N	2020-10-12 14:36:14.044759
35306	CTO	2020-10-02	45	46.745	45	45.64	35518	{}	\N	2020-10-12 14:36:14.056396
35307	CTO	2020-10-05	45.8	46.8699	45.525	46.42	15290	{}	\N	2020-10-12 14:36:14.06976
35308	CTO	2020-10-06	47	47.23	45.18	45.26	18124	{}	\N	2020-10-12 14:36:14.078059
35309	CTO	2020-10-07	46.17	46.3742	45.2	45.62	19330	{}	\N	2020-10-12 14:36:14.084521
35310	CTO	2020-10-08	45.22	46.78	45.22	46.26	17479	{}	\N	2020-10-12 14:36:14.090424
35311	CTO	2020-10-09	46.92	47.7889	46.4436	47.29	11249	{}	\N	2020-10-12 14:36:14.096376
35312	CTO	2020-10-12	47.15	47.665	47.1483	47.665	2863	{}	\N	2020-10-12 14:36:14.101352
35313	CTXR	2020-09-28	1	1.08	1	1.05	708115	{}	\N	2020-10-12 14:36:16.598338
35314	CTXR	2020-09-29	1.05	1.1	0.98	1.04	2049249	{}	\N	2020-10-12 14:36:16.606966
35315	CTXR	2020-09-30	1.09	1.09	1.03	1.04	1076199	{}	\N	2020-10-12 14:36:16.612514
35316	CTXR	2020-10-01	1.03	1.068	1.01	1.02	705966	{}	\N	2020-10-12 14:36:16.618338
35317	CTXR	2020-10-02	1	1.03	0.96	1	655556	{}	\N	2020-10-12 14:36:16.623505
35318	CTXR	2020-10-05	1.02	1.0299	0.981	0.983	495431	{}	\N	2020-10-12 14:36:16.628641
35319	CTXR	2020-10-06	0.982	1.02	0.98	1.01	828001	{}	\N	2020-10-12 14:36:16.632843
35320	CTXR	2020-10-07	1.061	1.11	1.05	1.08	2422134	{}	\N	2020-10-12 14:36:16.637103
35321	CTXR	2020-10-08	1.0766	1.1	1.07	1.08	1190850	{}	\N	2020-10-12 14:36:16.641081
35322	CTXR	2020-10-09	1.1	1.1	1.02	1.05	877795	{}	\N	2020-10-12 14:36:16.645151
35323	CTXR	2020-10-12	1.07	1.07	1.05	1.0601	253747	{}	\N	2020-10-12 14:36:16.649183
35324	CUBI	2020-09-28	11.09	11.57	11.09	11.31	133210	{}	\N	2020-10-12 14:36:18.509962
35325	CUBI	2020-09-29	11.3	11.3	10.91	11.25	194895	{}	\N	2020-10-12 14:36:18.520447
35326	CUBI	2020-09-30	11.27	11.48	11.06	11.2	370749	{}	\N	2020-10-12 14:36:18.52767
35327	CUBI	2020-10-01	11.2	11.41	11.06	11.34	236979	{}	\N	2020-10-12 14:36:18.533061
35328	CUBI	2020-10-02	11.06	11.8	11.06	11.75	197940	{}	\N	2020-10-12 14:36:18.537741
35329	CUBI	2020-10-05	11.89	12.42	11.865	12.39	196592	{}	\N	2020-10-12 14:36:18.541859
35330	CUBI	2020-10-06	12.59	13.16	12.52	12.59	305019	{}	\N	2020-10-12 14:36:18.545677
35331	CUBI	2020-10-07	12.83	13.045	12.58	12.67	249112	{}	\N	2020-10-12 14:36:18.549549
35332	CUBI	2020-10-08	12.85	13.14	12.67	13.01	166490	{}	\N	2020-10-12 14:36:18.553474
35333	CUBI	2020-10-09	13.19	13.205	12.66	12.69	208332	{}	\N	2020-10-12 14:36:18.557197
35334	CUBI	2020-10-12	12.61	12.86	12.61	12.745	9452	{}	\N	2020-10-12 14:36:18.561137
35335	CVA	2020-09-28	8.01	8.14	7.97	8.02	866190	{}	\N	2020-10-12 14:36:19.493294
35336	CVA	2020-09-29	8	8.1	7.835	7.91	1249230	{}	\N	2020-10-12 14:36:19.503448
35337	CVA	2020-09-30	7.94	8.03	7.71	7.75	1692330	{}	\N	2020-10-12 14:36:19.511076
35338	CVA	2020-10-01	7.77	7.905	7.73	7.87	1817478	{}	\N	2020-10-12 14:36:19.51695
35339	CVA	2020-10-02	7.68	8.01	7.68	7.94	1207065	{}	\N	2020-10-12 14:36:19.521565
35340	CVA	2020-10-05	7.97	8.07	7.835	7.87	1193545	{}	\N	2020-10-12 14:36:19.525758
35341	CVA	2020-10-06	7.88	8.07	7.81	7.81	2664934	{}	\N	2020-10-12 14:36:19.529975
35342	CVA	2020-10-07	7.92	8.135	7.92	8.09	1262618	{}	\N	2020-10-12 14:36:19.5341
35343	CVA	2020-10-08	8.18	8.32	8.175	8.3	1398958	{}	\N	2020-10-12 14:36:19.538122
35344	CVA	2020-10-09	8.4	8.45	8.13	8.15	928077	{}	\N	2020-10-12 14:36:19.542117
35345	CVA	2020-10-12	8.14	8.22	8.09	8.2111	169468	{}	\N	2020-10-12 14:36:19.546263
35346	CRVL	2020-09-28	84.04	85.8	84.04	85.25	32296	{}	\N	2020-10-12 14:36:21.408688
35347	CRVL	2020-09-29	85.1	85.91	84.14	84.85	27994	{}	\N	2020-10-12 14:36:21.422817
35348	CRVL	2020-09-30	84.74	86.52	84.74	85.42	31007	{}	\N	2020-10-12 14:36:21.430709
35349	CRVL	2020-10-01	85.865	87.55	85.52	87.54	50187	{}	\N	2020-10-12 14:36:21.435828
35350	CRVL	2020-10-02	86.22	88.76	85.47	87.72	23388	{}	\N	2020-10-12 14:36:21.441395
35351	CRVL	2020-10-05	88.14	89.57	87.9	88.48	22943	{}	\N	2020-10-12 14:36:21.445918
35352	CRVL	2020-10-06	89.09	89.85	87.66	88.47	37483	{}	\N	2020-10-12 14:36:21.450128
35353	CRVL	2020-10-07	89.04	89.88	87.587	89.44	41509	{}	\N	2020-10-12 14:36:21.45458
35354	CRVL	2020-10-08	89.79	90.3	88.54	89.46	34173	{}	\N	2020-10-12 14:36:21.459065
35355	CRVL	2020-10-09	90.25	90.25	88.62	89.55	19197	{}	\N	2020-10-12 14:36:21.463823
35356	CRVL	2020-10-12	89.55	89.55	88.955	88.955	1713	{}	\N	2020-10-12 14:36:21.469969
35357	CSGP	2020-09-28	851.08	859.1	846.63	857.13	116291	{}	\N	2020-10-12 14:36:22.385439
35358	CSGP	2020-09-29	857.64	866.4	850.87	856.85	109969	{}	\N	2020-10-12 14:36:22.39566
35359	CSGP	2020-09-30	859.83	869.58	843.71	848.51	176865	{}	\N	2020-10-12 14:36:22.402862
35360	CSGP	2020-10-01	859.8	871.78	850.765	865.38	208528	{}	\N	2020-10-12 14:36:22.409424
35361	CSGP	2020-10-02	854.98	866.25	848.2	855.1	237965	{}	\N	2020-10-12 14:36:22.413987
35362	CSGP	2020-10-05	860.73	870.09	858.62	866	152452	{}	\N	2020-10-12 14:36:22.418138
35363	CSGP	2020-10-06	867.83	871.5	857.715	863.92	141091	{}	\N	2020-10-12 14:36:22.422723
35364	CSGP	2020-10-07	869.09	879.21	867.43	875.72	122127	{}	\N	2020-10-12 14:36:22.427261
35365	CSGP	2020-10-08	880	882.76	867.01	868.89	153207	{}	\N	2020-10-12 14:36:22.431449
35366	CSGP	2020-10-09	872.45	881.9999	872.45	874.37	125562	{}	\N	2020-10-12 14:36:22.435733
35367	CSGP	2020-10-12	881.8	882.53	876.0103	877.57	5860	{}	\N	2020-10-12 14:36:22.439962
35368	CSPI	2020-09-28	8.9081	8.9081	8.58	8.8302	2239	{}	\N	2020-10-12 14:36:24.089886
35369	CSPI	2020-09-29	8.695	8.76	8.695	8.76	1057	{}	\N	2020-10-12 14:36:24.096457
35370	CSPI	2020-09-30	8.6001	8.86	8.6	8.63	16745	{}	\N	2020-10-12 14:36:24.101844
35371	CSPI	2020-10-01	8.6	8.6	8.43	8.515	1154	{}	\N	2020-10-12 14:36:24.106292
35372	CSPI	2020-10-02	8.28	8.5723	8.28	8.55	2221	{}	\N	2020-10-12 14:36:24.110714
35373	CSPI	2020-10-05	8.37	8.53	8.37	8.52	695	{}	\N	2020-10-12 14:36:24.114423
35374	CSPI	2020-10-06	8.6186	8.6186	8.32	8.35	1410	{}	\N	2020-10-12 14:36:24.117686
35375	CSPI	2020-10-07	8.75	8.75	8.4	8.4	665	{}	\N	2020-10-12 14:36:24.120823
35376	CSPI	2020-10-08	8.575	8.7081	8.575	8.7081	1389	{}	\N	2020-10-12 14:36:24.124335
35377	CSPI	2020-10-09	8.42	8.4563	8.347	8.4563	2901	{}	\N	2020-10-12 14:36:24.127995
35378	CTAS	2020-09-28	330	337.35	330	336	508342	{}	\N	2020-10-12 14:36:25.086222
35379	CTAS	2020-09-29	327.75	335.16	327.75	330.28	307630	{}	\N	2020-10-12 14:36:25.09688
35380	CTAS	2020-09-30	334.76	337.15	330.4	332.83	476132	{}	\N	2020-10-12 14:36:25.103699
35381	CTAS	2020-10-01	341.36	344.15	333.01	341.93	479864	{}	\N	2020-10-12 14:36:25.109441
35382	CTAS	2020-10-02	334.22	339.95	333.74	335	372987	{}	\N	2020-10-12 14:36:25.114651
35383	CTAS	2020-10-05	338.28	339.77	334.93	336.4	308614	{}	\N	2020-10-12 14:36:25.119435
35384	CTAS	2020-10-06	336.41	339.37	324.19	330.72	543857	{}	\N	2020-10-12 14:36:25.123708
35385	CTAS	2020-10-07	335.8	337.595	333.63	335.7	299360	{}	\N	2020-10-12 14:36:25.128005
35386	CTAS	2020-10-08	338.91	340.26	336.12	337.98	354924	{}	\N	2020-10-12 14:36:25.132367
35387	CTAS	2020-10-09	339.72	342.6	338.62	339.53	338639	{}	\N	2020-10-12 14:36:25.136863
35388	CTAS	2020-10-12	338.96	344.99	338.96	344.99	41138	{}	\N	2020-10-12 14:36:25.141315
35389	CTIC	2020-09-28	1.04	1.06	1.02	1.05	79687	{}	\N	2020-10-12 14:36:26.489803
35390	CTIC	2020-09-29	1.035	1.06	1.02	1.05	8818582	{}	\N	2020-10-12 14:36:26.497431
35391	CTIC	2020-09-30	2.31	2.6799	1.9	2.15	109073178	{}	\N	2020-10-12 14:36:26.503618
35392	CTIC	2020-10-01	1.97	2.58	1.97	2.475	20223796	{}	\N	2020-10-12 14:36:26.509111
35393	CTIC	2020-10-02	2.36	3.22	2.34	2.99	35069672	{}	\N	2020-10-12 14:36:26.514611
35394	CTIC	2020-10-05	3.11	3.34	2.91	3.28	9043929	{}	\N	2020-10-12 14:36:26.519109
35395	CTIC	2020-10-06	3.44	3.68	2.9	3.09	8058590	{}	\N	2020-10-12 14:36:26.5234
35396	CTIC	2020-10-07	3.05	3.19	2.85	2.99	3074543	{}	\N	2020-10-12 14:36:26.527809
35397	CTIC	2020-10-08	2.97	3.05	2.92	3.01	1261496	{}	\N	2020-10-12 14:36:26.531999
35398	CTIC	2020-10-09	3	3.3738	2.97	3.19	2306726	{}	\N	2020-10-12 14:36:26.536256
35399	CTIC	2020-10-12	3.17	3.24	3.03	3.1042	344897	{}	\N	2020-10-12 14:36:26.540648
35400	CTS	2020-09-28	21.82	22.35	21.82	22.11	124222	{}	\N	2020-10-12 14:36:27.476626
35401	CTS	2020-09-29	21.97	22.27	21.575	21.95	805288	{}	\N	2020-10-12 14:36:27.489584
35402	CTS	2020-09-30	21.95	22.37	21.9	22.03	564013	{}	\N	2020-10-12 14:36:27.496774
35403	CTS	2020-10-01	22.2	22.59	22.07	22.5	186888	{}	\N	2020-10-12 14:36:27.502927
35404	CTS	2020-10-02	22.1	22.56	22.03	22.38	210486	{}	\N	2020-10-12 14:36:27.507773
35405	CTS	2020-10-05	22.63	23.18	22.63	23.12	194472	{}	\N	2020-10-12 14:36:27.512408
35406	CTS	2020-10-06	23.54	24.29	23.27	23.59	276435	{}	\N	2020-10-12 14:36:27.516897
35407	CTS	2020-10-07	23.91	24.535	23.76	24.47	470354	{}	\N	2020-10-12 14:36:27.520794
35408	CTS	2020-10-08	24.82	25.13	24.54	24.88	267525	{}	\N	2020-10-12 14:36:27.525181
35409	CTS	2020-10-09	25.01	25.14	24.62	24.86	159617	{}	\N	2020-10-12 14:36:27.52999
35410	CTS	2020-10-12	24.84	25.005	24.76	24.97	13810	{}	\N	2020-10-12 14:36:27.534373
35411	CVGI	2020-09-28	5.24	6.11	5.1	6.08	744784	{}	\N	2020-10-12 14:36:29.277124
35412	CVGI	2020-09-29	5.98	6.09	5.76	6.02	385034	{}	\N	2020-10-12 14:36:29.287971
35413	CVGI	2020-09-30	6.07	6.56	5.985	6.53	600931	{}	\N	2020-10-12 14:36:29.294705
35414	CVGI	2020-10-01	6.54	6.67	6.19	6.57	517721	{}	\N	2020-10-12 14:36:29.300382
35415	CVGI	2020-10-02	6.32	6.6	6.01	6.54	443102	{}	\N	2020-10-12 14:36:29.305142
35416	CVGI	2020-10-05	6.73	6.9	6.4	6.48	392033	{}	\N	2020-10-12 14:36:29.309963
35417	CVGI	2020-10-06	6.6	6.65	6.2752	6.54	302621	{}	\N	2020-10-12 14:36:29.313938
35418	CVGI	2020-10-07	6.61	6.755	6.49	6.6	166765	{}	\N	2020-10-12 14:36:29.31809
35419	CVGI	2020-10-08	6.64	6.75	6.5	6.59	162037	{}	\N	2020-10-12 14:36:29.322048
35420	CVGI	2020-10-09	6.66	6.66	6.23	6.25	173409	{}	\N	2020-10-12 14:36:29.327754
35421	CVGI	2020-10-12	6.25	6.39	6.165	6.38	57925	{}	\N	2020-10-12 14:36:29.331853
35422	CWBR	2020-09-28	1.03	1.035	0.9301	0.9749	546105	{}	\N	2020-10-12 14:36:31.177992
35423	CWBR	2020-09-29	1.01	1.01	0.9525	0.981	226324	{}	\N	2020-10-12 14:36:31.190599
35424	CWBR	2020-09-30	0.986	0.9999	0.93	0.9496	414358	{}	\N	2020-10-12 14:36:31.197849
35425	CWBR	2020-10-01	0.96	0.99	0.95	0.98	356143	{}	\N	2020-10-12 14:36:31.204176
35426	CWBR	2020-10-02	0.96	1.02	0.95	0.982	391989	{}	\N	2020-10-12 14:36:31.209458
35427	CWBR	2020-10-05	0.97	1.01	0.9503	1	252797	{}	\N	2020-10-12 14:36:31.214694
35428	CWBR	2020-10-06	1.01	1.04	0.9594	0.98	444978	{}	\N	2020-10-12 14:36:31.218953
35429	CWBR	2020-10-07	0.9933	1.03	0.98	1.02	472525	{}	\N	2020-10-12 14:36:31.223166
35430	CWBR	2020-10-08	1.04	1.06	0.985	1	604650	{}	\N	2020-10-12 14:36:31.22728
35431	CWBR	2020-10-09	1	1.03	0.985	1.02	223179	{}	\N	2020-10-12 14:36:31.231414
35432	CWBR	2020-10-12	1.01	1.02	1	1.01	135586	{}	\N	2020-10-12 14:36:31.235596
35433	CXH	2020-09-28	9.3	9.39	9.293	9.39	32330	{}	\N	2020-10-12 14:36:32.712713
35434	CXH	2020-09-29	9.35	9.38	9.3387	9.38	13574	{}	\N	2020-10-12 14:36:32.723518
35435	CXH	2020-09-30	9.41	9.42	9.38	9.39	27756	{}	\N	2020-10-12 14:36:32.731669
35436	CXH	2020-10-01	9.4	9.44	9.4	9.44	44998	{}	\N	2020-10-12 14:36:32.738447
35437	CXH	2020-10-02	9.38	9.47	9.38	9.44	38416	{}	\N	2020-10-12 14:36:32.743998
35438	CXH	2020-10-05	9.45	9.45	9.4036	9.41	40712	{}	\N	2020-10-12 14:36:32.749065
35439	CXH	2020-10-06	9.4	9.44	9.39	9.43	21008	{}	\N	2020-10-12 14:36:32.753871
35440	CXH	2020-10-07	9.42	9.46	9.41	9.46	63624	{}	\N	2020-10-12 14:36:32.758272
35441	CXH	2020-10-08	9.45	9.4729	9.45	9.46	7060	{}	\N	2020-10-12 14:36:32.762941
35442	CXH	2020-10-09	9.46	9.5	9.43	9.44	37202	{}	\N	2020-10-12 14:36:32.767065
35443	CXH	2020-10-12	9.43	9.44	9.4	9.4	18308	{}	\N	2020-10-12 14:36:32.771096
35444	CZWI	2020-09-28	6.76	6.9799	6.76	6.82	19938	{}	\N	2020-10-12 14:36:34.284588
35445	CZWI	2020-09-29	6.9	6.91	6.86	6.86	1982	{}	\N	2020-10-12 14:36:34.292891
35446	CZWI	2020-09-30	6.9	6.91	6.8102	6.89	11719	{}	\N	2020-10-12 14:36:34.299347
35447	CZWI	2020-10-01	6.89	7.03	6.875	6.98	20612	{}	\N	2020-10-12 14:36:34.304455
35448	CZWI	2020-10-02	6.9481	7.16	6.93	7.03	13364	{}	\N	2020-10-12 14:36:34.309514
35449	CZWI	2020-10-05	7.08	7.21	7.06	7.06	9647	{}	\N	2020-10-12 14:36:34.313809
35450	CZWI	2020-10-06	7.05	7.21	7.04	7.2	9245	{}	\N	2020-10-12 14:36:34.318357
35451	CZWI	2020-10-07	7.13	7.23	6.8317	7.19	14953	{}	\N	2020-10-12 14:36:34.322559
35452	CZWI	2020-10-08	7.15	7.24	7.08	7.2	10865	{}	\N	2020-10-12 14:36:34.327245
35453	CZWI	2020-10-09	7.12	7.19	7.12	7.17	2045	{}	\N	2020-10-12 14:36:34.331505
35454	CRY	2020-09-28	18.75	18.99	18.43	18.46	253346	{}	\N	2020-10-12 14:36:35.519528
35455	CRY	2020-09-29	18.43	18.665	18.1	18.53	457255	{}	\N	2020-10-12 14:36:35.529206
35456	CRY	2020-09-30	18.64	19.07	18.345	18.47	449373	{}	\N	2020-10-12 14:36:35.535242
35457	CRY	2020-10-01	18.61	18.8922	18.21	18.34	358864	{}	\N	2020-10-12 14:36:35.541175
35458	CRY	2020-10-02	18.06	18.39	17.97	18.27	341719	{}	\N	2020-10-12 14:36:35.546346
35459	CRY	2020-10-05	18.44	18.65	18.15	18.35	241292	{}	\N	2020-10-12 14:36:35.550992
35460	CRY	2020-10-06	18.6	18.78	18.34	18.35	589807	{}	\N	2020-10-12 14:36:35.555318
35461	CRY	2020-10-07	18.49	18.6	18.11	18.47	660045	{}	\N	2020-10-12 14:36:35.559361
35462	CRY	2020-10-08	18.5	18.83	18.34	18.7	1122868	{}	\N	2020-10-12 14:36:35.563598
35463	CRY	2020-10-09	18.95	19.45	18.8	19.31	483260	{}	\N	2020-10-12 14:36:35.567718
35464	CRY	2020-10-12	19.38	19.61	19.165	19.61	104754	{}	\N	2020-10-12 14:36:35.572089
35465	CSOD	2020-09-28	34.93	35.4799	34.87	35.2	361210	{}	\N	2020-10-12 14:36:37.264954
35466	CSOD	2020-09-29	35.2	35.63	34.5	35.09	345705	{}	\N	2020-10-12 14:36:37.279053
35467	CSOD	2020-09-30	35.14	36.79	35.14	36.36	794195	{}	\N	2020-10-12 14:36:37.286188
35468	CSOD	2020-10-01	36.58	37.17	35.82	37.09	614119	{}	\N	2020-10-12 14:36:37.292118
35469	CSOD	2020-10-02	36.45	37.26	36.19	37.01	332701	{}	\N	2020-10-12 14:36:37.296763
35470	CSOD	2020-10-05	37.14	37.48	36.88	37.15	374026	{}	\N	2020-10-12 14:36:37.300849
35471	CSOD	2020-10-06	37.49	38.51	37.45	37.83	571223	{}	\N	2020-10-12 14:36:37.304853
35472	CSOD	2020-10-07	37.81	38.22	37.27	37.78	362929	{}	\N	2020-10-12 14:36:37.308977
35473	CSOD	2020-10-08	38.16	38.43	37.475	37.78	399445	{}	\N	2020-10-12 14:36:37.312933
35474	CSOD	2020-10-09	37.72	38.05	37.27	37.54	391655	{}	\N	2020-10-12 14:36:37.316891
35475	CSOD	2020-10-12	37.57	37.61	37.13	37.53	22012	{}	\N	2020-10-12 14:36:37.320842
35476	CSWC	2020-09-28	14.34	14.726	14.23	14.68	114807	{}	\N	2020-10-12 14:36:38.729514
35477	CSWC	2020-09-29	14.59	14.59	14.11	14.27	54963	{}	\N	2020-10-12 14:36:38.739168
35478	CSWC	2020-09-30	14.48	14.545	14.04	14.05	80516	{}	\N	2020-10-12 14:36:38.745291
35479	CSWC	2020-10-01	14.15	14.28	14	14.26	64363	{}	\N	2020-10-12 14:36:38.75109
35480	CSWC	2020-10-02	14.08	14.4262	14.08	14.4	43631	{}	\N	2020-10-12 14:36:38.755867
35481	CSWC	2020-10-05	14.46	14.51	14.26	14.34	42275	{}	\N	2020-10-12 14:36:38.760292
35482	CSWC	2020-10-06	14.34	14.5	14.3	14.3	58278	{}	\N	2020-10-12 14:36:38.76441
35483	CSWC	2020-10-07	14.33	14.4549	14.018	14.32	55929	{}	\N	2020-10-12 14:36:38.767919
35484	CSWC	2020-10-08	14.31	14.43	14.22	14.38	40584	{}	\N	2020-10-12 14:36:38.771779
35485	CSWC	2020-10-09	14.35	14.4	14.27	14.34	36089	{}	\N	2020-10-12 14:36:38.775323
35486	CSWC	2020-10-12	14.23	14.34	14.23	14.28	9376	{}	\N	2020-10-12 14:36:38.778924
35487	CTBI	2020-09-28	28.63	29.325	28.62	29.01	47825	{}	\N	2020-10-12 14:36:39.976867
35488	CTBI	2020-09-29	28.98	29.1886	28.47	29	57842	{}	\N	2020-10-12 14:36:39.987854
35489	CTBI	2020-09-30	28.76	29.07	28.09	28.26	76351	{}	\N	2020-10-12 14:36:39.995076
35490	CTBI	2020-10-01	28.43	28.6199	27.8	28.14	99618	{}	\N	2020-10-12 14:36:40.000935
35491	CTBI	2020-10-02	27.84	29.41	27.735	29.16	98753	{}	\N	2020-10-12 14:36:40.005916
35492	CTBI	2020-10-05	28.62	30.3486	28.62	30.26	47058	{}	\N	2020-10-12 14:36:40.010955
35493	CTBI	2020-10-06	30.572	31.43	30.27	30.37	65030	{}	\N	2020-10-12 14:36:40.015198
35494	CTBI	2020-10-07	30.73	31.15	30.39	31.04	45358	{}	\N	2020-10-12 14:36:40.019144
35495	CTBI	2020-10-08	31.29	31.63	30.95	31.44	43885	{}	\N	2020-10-12 14:36:40.022694
35496	CTBI	2020-10-09	31.78	31.78	30.98	31.21	28898	{}	\N	2020-10-12 14:36:40.026698
35497	CTBI	2020-10-12	31.06	31.64	31.02	31.64	3452	{}	\N	2020-10-12 14:36:40.030828
35498	CTSH	2020-09-28	68.97	69.65	68.73	69.32	3706894	{}	\N	2020-10-12 14:36:41.051621
35499	CTSH	2020-09-29	69.4117	70.08	68.63	69	2400619	{}	\N	2020-10-12 14:36:41.060479
35500	CTSH	2020-09-30	69.28	70.105	68.86	69.42	3535763	{}	\N	2020-10-12 14:36:41.066144
35501	CTSH	2020-10-01	70.22	70.595	68.975	69.33	3203742	{}	\N	2020-10-12 14:36:41.071221
35502	CTSH	2020-10-02	67.92	69.35	67.79	68.86	2165847	{}	\N	2020-10-12 14:36:41.075513
35503	CTSH	2020-10-05	69.62	71.26	69.5	70.93	4006822	{}	\N	2020-10-12 14:36:41.079845
35504	CTSH	2020-10-06	70.9	71.6	69.69	69.83	2818828	{}	\N	2020-10-12 14:36:41.083672
35505	CTSH	2020-10-07	70.73	72.48	70.6	72.24	3112780	{}	\N	2020-10-12 14:36:41.086935
35506	CTSH	2020-10-08	72.5	73.45	72.23	72.91	2218309	{}	\N	2020-10-12 14:36:41.090182
35507	CTSH	2020-10-09	73.65	74.06	72.75	73.08	3107639	{}	\N	2020-10-12 14:36:41.093562
35508	CTSH	2020-10-12	74.09	74.1	73.27	73.62	616971	{}	\N	2020-10-12 14:36:41.097123
35509	CUBA	2020-09-28	3.5	3.6011	3.46	3.56	34566	{}	\N	2020-10-12 14:36:42.081348
35510	CUBA	2020-09-29	3.58	3.59	3.52	3.55	8978	{}	\N	2020-10-12 14:36:42.092132
35511	CUBA	2020-09-30	3.56	3.64	3.535	3.6	32687	{}	\N	2020-10-12 14:36:42.099236
35512	CUBA	2020-10-01	3.615	3.65	3.59	3.65	40381	{}	\N	2020-10-12 14:36:42.104966
35513	CUBA	2020-10-02	3.62	3.74	3.6159	3.74	9134	{}	\N	2020-10-12 14:36:42.109622
35514	CUBA	2020-10-05	3.71	3.7729	3.67	3.73	25803	{}	\N	2020-10-12 14:36:42.114164
35515	CUBA	2020-10-06	3.77	3.8	3.67	3.67	29794	{}	\N	2020-10-12 14:36:42.118187
35516	CUBA	2020-10-07	3.75	3.8	3.72	3.8	12256	{}	\N	2020-10-12 14:36:42.121823
35517	CUBA	2020-10-08	3.7234	3.8045	3.7234	3.79	10743	{}	\N	2020-10-12 14:36:42.125709
35518	CUBA	2020-10-09	3.83	3.8427	3.8	3.82	15607	{}	\N	2020-10-12 14:36:42.129636
35519	CUBA	2020-10-12	3.81	3.83	3.805	3.825	1396	{}	\N	2020-10-12 14:36:42.133422
35520	CUZ	2020-09-28	28.03	29.11	27.625	28.83	955107	{}	\N	2020-10-12 14:36:43.419622
35521	CUZ	2020-09-29	28.62	28.85	28.19	28.6	1817372	{}	\N	2020-10-12 14:36:43.424952
35522	CUZ	2020-09-30	28.84	29.13	28.19	28.59	2752416	{}	\N	2020-10-12 14:36:43.429251
35523	CUZ	2020-10-01	28.6	29.24	28.34	29.24	1828538	{}	\N	2020-10-12 14:36:43.43299
35524	CUZ	2020-10-02	28.42	29.8	28.08	29.7	1512563	{}	\N	2020-10-12 14:36:43.436801
35525	CUZ	2020-10-05	29.96	30.21	29.44	29.8	1494001	{}	\N	2020-10-12 14:36:43.440878
35526	CUZ	2020-10-06	30.01	30.11	29.22	29.42	1909438	{}	\N	2020-10-12 14:36:43.446132
35527	CUZ	2020-10-07	29.55	29.56	28.74	28.87	1570960	{}	\N	2020-10-12 14:36:43.45019
35528	CUZ	2020-10-08	29.05	29.515	28.86	29.49	1984466	{}	\N	2020-10-12 14:36:43.453998
35529	CUZ	2020-10-09	29.78	29.78	29.15	29.27	613729	{}	\N	2020-10-12 14:36:43.457853
35530	CUZ	2020-10-12	29.33	29.39	29.01	29.36	100214	{}	\N	2020-10-12 14:36:43.461777
35531	CVGW	2020-09-28	68.86	68.96	66.54	67.04	103995	{}	\N	2020-10-12 14:36:44.855991
35532	CVGW	2020-09-29	67.31	67.62	64.93	65.49	115918	{}	\N	2020-10-12 14:36:44.8679
35533	CVGW	2020-09-30	65.52	66.76	65.52	66.27	120000	{}	\N	2020-10-12 14:36:44.874665
35534	CVGW	2020-10-01	66.95	67.77	66.015	67.71	115516	{}	\N	2020-10-12 14:36:44.880105
35535	CVGW	2020-10-02	66.58	68.316	66.485	67.76	70881	{}	\N	2020-10-12 14:36:44.884934
35536	CVGW	2020-10-05	68.2	69.205	67.47	68.81	95487	{}	\N	2020-10-12 14:36:44.890015
35537	CVGW	2020-10-06	69.41	69.485	67.535	68.355	190706	{}	\N	2020-10-12 14:36:44.894697
35538	CVGW	2020-10-07	68.44	68.74	66.28	67.92	139895	{}	\N	2020-10-12 14:36:44.898677
35539	CVGW	2020-10-08	68.67	69.04	67.915	68.6	95236	{}	\N	2020-10-12 14:36:44.902474
35540	CVGW	2020-10-09	69.03	69.1	68.44	68.62	66146	{}	\N	2020-10-12 14:36:44.906599
35541	CVGW	2020-10-12	68.77	69.47	68.63	68.83	4584	{}	\N	2020-10-12 14:36:44.910744
35542	CYAD	2020-09-28	9.385	9.385	9.27	9.295	1519	{}	\N	2020-10-12 14:36:46.300912
35543	CYAD	2020-09-29	9.86	9.9	9.75	9.9	7115	{}	\N	2020-10-12 14:36:46.312191
35544	CYAD	2020-09-30	10.29	10.29	9.875	9.88	4112	{}	\N	2020-10-12 14:36:46.319266
35545	CYAD	2020-10-01	9.65	9.66	9.56	9.5825	4826	{}	\N	2020-10-12 14:36:46.324727
35546	CYAD	2020-10-02	9.5	9.56	9.19	9.24	6226	{}	\N	2020-10-12 14:36:46.33044
35547	CYAD	2020-10-05	9.5	9.82	9.479	9.479	2208	{}	\N	2020-10-12 14:36:46.336485
35548	CYAD	2020-10-06	9.55	9.5702	9.414	9.486	5710	{}	\N	2020-10-12 14:36:46.340482
35549	CYAD	2020-10-07	9.36	9.4299	9.2204	9.3712	6193	{}	\N	2020-10-12 14:36:46.344715
35550	CYAD	2020-10-08	9.48	9.559	9.45	9.45	4719	{}	\N	2020-10-12 14:36:46.348855
35551	CYAD	2020-10-09	9.49	9.51	9.2601	9.4	9068	{}	\N	2020-10-12 14:36:46.353178
35552	CYAD	2020-10-12	9.4	9.4	9.24	9.3	4707	{}	\N	2020-10-12 14:36:46.357062
35553	CZNC	2020-09-28	15.98	16.49	15.8	16.32	35492	{}	\N	2020-10-12 14:36:48.745136
35554	CZNC	2020-09-29	16.3	17.09	16.06	16.85	17001	{}	\N	2020-10-12 14:36:48.751323
35555	CZNC	2020-09-30	17.0092	17.01	16.11	16.29	31843	{}	\N	2020-10-12 14:36:48.758476
35556	CZNC	2020-10-01	16.4	16.43	16.07	16.38	26173	{}	\N	2020-10-12 14:36:48.770037
35557	CZNC	2020-10-02	16.09	16.98	16.05	16.82	17117	{}	\N	2020-10-12 14:36:48.777844
35558	CZNC	2020-10-05	16.96	16.98	16.13	16.38	39058	{}	\N	2020-10-12 14:36:48.784379
35559	CZNC	2020-10-06	16.41	16.98	16.3	16.36	34037	{}	\N	2020-10-12 14:36:48.789888
35560	CZNC	2020-10-07	16.5928	17.48	16.52	17.38	33641	{}	\N	2020-10-12 14:36:48.794664
35561	CZNC	2020-10-08	17.35	18	17.1601	17.74	36558	{}	\N	2020-10-12 14:36:48.799616
35562	CZNC	2020-10-09	17.82	18.1836	17.64	17.97	13411	{}	\N	2020-10-12 14:36:48.804592
35563	CZNC	2020-10-12	17.84	17.9	17.59	17.7583	5939	{}	\N	2020-10-12 14:36:48.808929
35564	CSM	2020-09-28	75.5825	76.05	75.5825	75.8218	7532	{}	\N	2020-10-12 14:36:49.786984
35565	CSM	2020-09-29	76.112	76.112	75.535	75.67	8535	{}	\N	2020-10-12 14:36:49.795025
35566	CSM	2020-09-30	75.82	76.7352	75.6728	76.2454	5785	{}	\N	2020-10-12 14:36:49.801898
35567	CSM	2020-10-01	76.848	76.848	76.15	76.42	18753	{}	\N	2020-10-12 14:36:49.811812
35568	CSM	2020-10-02	76.05	76.17	75.55	76.09	6896	{}	\N	2020-10-12 14:36:49.821016
35569	CSM	2020-10-05	76.779	77.0472	76.55	77.0472	3463	{}	\N	2020-10-12 14:36:49.831873
35570	CSM	2020-10-06	77.1237	77.2094	75.6466	75.75	22515	{}	\N	2020-10-12 14:36:49.841579
35571	CSM	2020-10-07	76.55	77.102	76.55	77.0202	5363	{}	\N	2020-10-12 14:36:49.852725
35572	CSM	2020-10-08	77.595	77.7534	77.53	77.7264	5360	{}	\N	2020-10-12 14:36:49.860022
35573	CSM	2020-10-09	78.44	78.44	78.099	78.23	2331	{}	\N	2020-10-12 14:36:49.866851
35574	CSM	2020-10-12	78.58	78.99	74.65	78.99	63416	{}	\N	2020-10-12 14:36:49.872214
35575	CSSE	2020-09-28	14.15	15.09	14.15	14.5	61700	{}	\N	2020-10-12 14:36:51.212828
35576	CSSE	2020-09-29	14.6794	14.6794	14.31	14.39	35650	{}	\N	2020-10-12 14:36:51.222259
35577	CSSE	2020-09-30	14.31	14.9	14.2	14.43	51717	{}	\N	2020-10-12 14:36:51.228942
35578	CSSE	2020-10-01	14.52	14.8	14.35	14.59	23879	{}	\N	2020-10-12 14:36:51.235148
35579	CSSE	2020-10-02	14.1	14.99	14.1	14.88	15934	{}	\N	2020-10-12 14:36:51.240267
35580	CSSE	2020-10-05	14.91	15.33	14.59	14.69	63917	{}	\N	2020-10-12 14:36:51.245404
35581	CSSE	2020-10-06	14.75	15.29	14.7	14.95	23471	{}	\N	2020-10-12 14:36:51.250294
35582	CSSE	2020-10-07	15.2072	17.15	15.2072	16.2	58493	{}	\N	2020-10-12 14:36:51.255677
35583	CSSE	2020-10-08	16.89	16.89	14.96	15.69	68726	{}	\N	2020-10-12 14:36:51.25981
35584	CSSE	2020-10-09	15.88	16.475	14.51	14.75	47326	{}	\N	2020-10-12 14:36:51.264313
35585	CSSE	2020-10-12	14.75	15.66	14.75	15.1559	6915	{}	\N	2020-10-12 14:36:51.268946
35586	CSWI	2020-09-28	76.41	78	76.41	77.5	114955	{}	\N	2020-10-12 14:36:52.779983
35587	CSWI	2020-09-29	77.28	78.18	77.115	77.93	100155	{}	\N	2020-10-12 14:36:52.789516
35588	CSWI	2020-09-30	78.08	78.93	76.83	77.25	35196	{}	\N	2020-10-12 14:36:52.795484
35589	CSWI	2020-10-01	77.37	77.91	76.82	77.67	44680	{}	\N	2020-10-12 14:36:52.800862
35590	CSWI	2020-10-02	76.64	78.96	76.245	78.59	43200	{}	\N	2020-10-12 14:36:52.805523
35591	CSWI	2020-10-05	78.92	80.17	78.395	80	50751	{}	\N	2020-10-12 14:36:52.811089
35592	CSWI	2020-10-06	80.61	81.4	79.74	80.75	55889	{}	\N	2020-10-12 14:36:52.815032
35593	CSWI	2020-10-07	81.22	83.21	81.22	83	42904	{}	\N	2020-10-12 14:36:52.818956
35594	CSWI	2020-10-08	83.72	85.54	83.48	84.4	65915	{}	\N	2020-10-12 14:36:52.822676
35595	CSWI	2020-10-09	84.58	86.43	84.58	85.55	44441	{}	\N	2020-10-12 14:36:52.827131
35596	CSWI	2020-10-12	85.93	86.12	85.22	86.12	1267	{}	\N	2020-10-12 14:36:52.831633
35597	CTIB	2020-09-28	1.46	1.4875	1.41	1.44	79878	{}	\N	2020-10-12 14:36:54.91515
35598	CTIB	2020-09-29	1.47	1.84	1.465	1.6	756530	{}	\N	2020-10-12 14:36:54.925017
35599	CTIB	2020-09-30	1.68	1.69	1.4718	1.6495	303518	{}	\N	2020-10-12 14:36:54.932913
35600	CTIB	2020-10-01	1.65	1.65	1.53	1.53	95673	{}	\N	2020-10-12 14:36:54.939277
35601	CTIB	2020-10-02	1.53	1.53	1.45	1.5	52175	{}	\N	2020-10-12 14:36:54.944532
35602	CTIB	2020-10-05	1.48	1.5	1.37	1.3799	161349	{}	\N	2020-10-12 14:36:54.94959
35603	CTIB	2020-10-06	1.4103	1.49	1.39	1.4311	144143	{}	\N	2020-10-12 14:36:54.954027
35604	CTIB	2020-10-07	1.47	1.54	1.45	1.5	87380	{}	\N	2020-10-12 14:36:54.958565
35605	CTIB	2020-10-08	1.5	1.65	1.5	1.61	149964	{}	\N	2020-10-12 14:36:54.963296
35606	CTIB	2020-10-09	1.56	1.7646	1.56	1.61	111583	{}	\N	2020-10-12 14:36:54.967964
35607	CTIB	2020-10-12	1.61	1.6135	1.56	1.5814	13353	{}	\N	2020-10-12 14:36:54.972933
35608	CTT	2020-09-28	8.77	9.07	8.77	9.01	118412	{}	\N	2020-10-12 14:36:55.946289
35609	CTT	2020-09-29	9.01	9.25	8.9728	9.18	291651	{}	\N	2020-10-12 14:36:55.955365
35610	CTT	2020-09-30	9.21	9.24	8.82	8.93	469442	{}	\N	2020-10-12 14:36:55.96159
35611	CTT	2020-10-01	8.96	9.14	8.88	8.93	646806	{}	\N	2020-10-12 14:36:55.967617
35612	CTT	2020-10-02	8.75	9.0023	8.75	8.97	578013	{}	\N	2020-10-12 14:36:55.972407
35613	CTT	2020-10-05	9.04	9.23	8.82	8.92	331893	{}	\N	2020-10-12 14:36:55.97707
35614	CTT	2020-10-06	9.05	9.07	8.72	8.93	710222	{}	\N	2020-10-12 14:36:55.980886
35615	CTT	2020-10-07	8.97	9.16	8.97	9.12	377207	{}	\N	2020-10-12 14:36:55.98486
35616	CTT	2020-10-08	9.19	9.25	9.04	9.06	822784	{}	\N	2020-10-12 14:36:55.988361
35617	CTT	2020-10-09	9.12	9.15	8.885	8.9	614927	{}	\N	2020-10-12 14:36:55.992245
35618	CTT	2020-10-12	8.89	8.96	8.82	8.9	32394	{}	\N	2020-10-12 14:36:55.996022
35619	CUE	2020-09-28	16.28	16.28	14.84	15.16	214714	{}	\N	2020-10-12 14:36:57.048015
35620	CUE	2020-09-29	15.07	15.729	14.88	15.4	232219	{}	\N	2020-10-12 14:36:57.056407
35621	CUE	2020-09-30	15.4	15.79	14.85	15.02	143306	{}	\N	2020-10-12 14:36:57.06254
35622	CUE	2020-10-01	14.96	15.85	14.96	15.61	203551	{}	\N	2020-10-12 14:36:57.069521
35623	CUE	2020-10-02	15.47	16.03	15.42	15.6	119425	{}	\N	2020-10-12 14:36:57.075127
35624	CUE	2020-10-05	16.0308	16.31	15.425	16.3	169029	{}	\N	2020-10-12 14:36:57.080059
35625	CUE	2020-10-06	16.33	16.485	15.461	15.73	173204	{}	\N	2020-10-12 14:36:57.084564
35626	CUE	2020-10-07	15.84	16.48	15.71	16.4	170407	{}	\N	2020-10-12 14:36:57.088626
35627	CUE	2020-10-08	16.48	16.5	15.89	16.01	143508	{}	\N	2020-10-12 14:36:57.09283
35628	CUE	2020-10-09	16.135	16.37	15.9	16.29	91979	{}	\N	2020-10-12 14:36:57.097013
35629	CUE	2020-10-12	16.5	16.59	16	16.11	14710	{}	\N	2020-10-12 14:36:57.101003
35630	CURE	2020-09-28	60.44	61.65	60	60.0067	56630	{}	\N	2020-10-12 14:36:58.398807
35631	CURE	2020-09-29	60.34	61.53	59.44	59.99	29777	{}	\N	2020-10-12 14:36:58.405938
35632	CURE	2020-09-30	60.39	64.39	60.39	63.04	72963	{}	\N	2020-10-12 14:36:58.41253
35633	CURE	2020-10-01	63.68	64.57	61.49	62.2	72173	{}	\N	2020-10-12 14:36:58.419736
35634	CURE	2020-10-02	60.17	62.23	59.8	60.55	74545	{}	\N	2020-10-12 14:36:58.426195
35635	CURE	2020-10-05	62.13	64.5	62.06	64.25	69248	{}	\N	2020-10-12 14:36:58.430482
35636	CURE	2020-10-06	64.88	64.88	61.65	61.65	67249	{}	\N	2020-10-12 14:36:58.435529
35637	CURE	2020-10-07	63.04	65.78	63.04	65.16	63244	{}	\N	2020-10-12 14:36:58.439604
35638	CURE	2020-10-08	66.21	66.83	65.79	66.24	52093	{}	\N	2020-10-12 14:36:58.443993
35639	CURE	2020-10-09	67.14	68.58	66.83	67.98	55510	{}	\N	2020-10-12 14:36:58.44833
35640	CURE	2020-10-12	69.1	69.13	68.33	69.0076	9516	{}	\N	2020-10-12 14:36:58.452705
35641	CVE	2020-09-28	3.99	4.15	3.93	4.14	4682575	{}	\N	2020-10-12 14:36:59.584474
35642	CVE	2020-09-29	4.11	4.11	3.86	3.91	19773194	{}	\N	2020-10-12 14:36:59.592297
35643	CVE	2020-09-30	3.91	4.06	3.89	3.89	21825050	{}	\N	2020-10-12 14:36:59.597972
35644	CVE	2020-10-01	3.81	3.81	3.58	3.65	17156718	{}	\N	2020-10-12 14:36:59.602841
35645	CVE	2020-10-02	3.46	3.705	3.43	3.63	11998169	{}	\N	2020-10-12 14:36:59.607257
35646	CVE	2020-10-05	3.72	3.8	3.6	3.76	9443087	{}	\N	2020-10-12 14:36:59.61172
35647	CVE	2020-10-06	3.81	3.94	3.745	3.79	15911332	{}	\N	2020-10-12 14:36:59.615743
35648	CVE	2020-10-07	3.81	3.9	3.77	3.87	13088739	{}	\N	2020-10-12 14:36:59.619783
35649	CVE	2020-10-08	3.95	4.17	3.905	4.14	8987551	{}	\N	2020-10-12 14:36:59.623678
35650	CVE	2020-10-09	4.21	4.21	4	4.05	8967076	{}	\N	2020-10-12 14:36:59.627751
35651	CVE	2020-10-12	4.04	4.1	3.97	4.015	529394	{}	\N	2020-10-12 14:36:59.632058
35652	CVLY	2020-09-28	12.5601	13	12.5601	12.85	6799	{}	\N	2020-10-12 14:37:01.102948
35653	CVLY	2020-09-29	12.795	12.83	12.78	12.8	3294	{}	\N	2020-10-12 14:37:01.111123
35654	CVLY	2020-09-30	12.8	13.1	12.8	13.1	13800	{}	\N	2020-10-12 14:37:01.117573
35655	CVLY	2020-10-01	13.1	13.18	12.93	12.93	7857	{}	\N	2020-10-12 14:37:01.12321
35656	CVLY	2020-10-02	12.71	13.27	12.71	13.06	5854	{}	\N	2020-10-12 14:37:01.128576
35657	CVLY	2020-10-05	13.865	14.225	13.5914	14	6104	{}	\N	2020-10-12 14:37:01.133642
35658	CVLY	2020-10-06	13.96	14.715	13.96	14	9376	{}	\N	2020-10-12 14:37:01.137908
35659	CVLY	2020-10-07	14.07	14.11	14	14.1	11398	{}	\N	2020-10-12 14:37:01.142331
35660	CVLY	2020-10-08	14.15	14.15	13.92	14.02	6467	{}	\N	2020-10-12 14:37:01.146681
35661	CVLY	2020-10-09	13.75	14.25	13.75	14	5205	{}	\N	2020-10-12 14:37:01.151009
35662	CVLY	2020-10-12	14	14	13.98	13.98	860	{}	\N	2020-10-12 14:37:01.155111
35663	CVV	2020-09-28	3.115	3.4399	3.115	3.2997	90320	{}	\N	2020-10-12 14:37:03.251058
35664	CVV	2020-09-29	3.23	3.305	2.96	2.9955	27522	{}	\N	2020-10-12 14:37:03.258692
35665	CVV	2020-09-30	3.44	3.44	3.02	3.06	15707	{}	\N	2020-10-12 14:37:03.263878
35666	CVV	2020-10-01	3	3.025	2.88	2.9212	37075	{}	\N	2020-10-12 14:37:03.268708
35667	CVV	2020-10-02	2.8943	2.97	2.852	2.9	5725	{}	\N	2020-10-12 14:37:03.273086
35668	CVV	2020-10-05	3.04	3.1082	2.89	2.89	21891	{}	\N	2020-10-12 14:37:03.277581
35669	CVV	2020-10-06	2.89	3.0999	2.89	2.98	32494	{}	\N	2020-10-12 14:37:03.281678
35670	CVV	2020-10-07	3	7.47	3	5.74	45217217	{}	\N	2020-10-12 14:37:03.285643
35671	CVV	2020-10-08	4.815	5.17	4.3203	4.79	3001654	{}	\N	2020-10-12 14:37:03.289492
35672	CVV	2020-10-09	4.4	4.61	4.17	4.49	577307	{}	\N	2020-10-12 14:37:03.293454
35673	CVV	2020-10-12	4.46	4.48	4.23	4.34	42053	{}	\N	2020-10-12 14:37:03.297296
35674	CWST	2020-09-28	55.34	56.1	55.11	55.55	208152	{}	\N	2020-10-12 14:37:04.807802
35675	CWST	2020-09-29	55.55	56.29	55.5	56.07	196221	{}	\N	2020-10-12 14:37:04.819185
35676	CWST	2020-09-30	56.24	56.77	55.48	55.85	223959	{}	\N	2020-10-12 14:37:04.826105
35677	CWST	2020-10-01	55.81	56.74	55.39	56.72	270325	{}	\N	2020-10-12 14:37:04.832524
35678	CWST	2020-10-02	55.77	58.09	55.75	57.51	246757	{}	\N	2020-10-12 14:37:04.838012
35679	CWST	2020-10-05	57.83	58.31	57.01	57.7	138039	{}	\N	2020-10-12 14:37:04.843264
35680	CWST	2020-10-06	57.93	59.07	57.53	57.73	142276	{}	\N	2020-10-12 14:37:04.848309
35681	CWST	2020-10-07	58.5	59.04	57.84	58.88	266312	{}	\N	2020-10-12 14:37:04.852841
35682	CWST	2020-10-08	59	59.9652	58.715	59.42	265691	{}	\N	2020-10-12 14:37:04.857157
35683	CWST	2020-10-09	59.77	60	58.8	59.75	123027	{}	\N	2020-10-12 14:37:04.861734
35684	CWST	2020-10-12	59.96	60.69	59.96	60.53	17209	{}	\N	2020-10-12 14:37:04.866396
35685	CYCCP	2020-09-29	5.54	5.54	5.54	5.54	112	{}	\N	2020-10-12 14:37:07.509191
35686	CYCCP	2020-10-02	5.15	5.15	4.85	5	1128	{}	\N	2020-10-12 14:37:07.516796
35687	CYCCP	2020-10-05	5.12	5.12	5.12	5.12	150	{}	\N	2020-10-12 14:37:07.522525
35688	CYCCP	2020-10-06	5.66	5.66	5.66	5.66	118	{}	\N	2020-10-12 14:37:07.528116
35689	CYCCP	2020-10-07	5.68	5.68	5.5001	5.5001	357	{}	\N	2020-10-12 14:37:07.533304
35690	CYCCP	2020-10-08	5.54	5.54	5.05	5.05	1529	{}	\N	2020-10-12 14:37:07.537996
35691	CSQ	2020-09-28	13.2	13.33	13.18	13.26	191891	{}	\N	2020-10-12 14:37:08.625766
35692	CSQ	2020-09-29	13.2512	13.32	13.18	13.26	175073	{}	\N	2020-10-12 14:37:08.634587
35693	CSQ	2020-09-30	13.3	13.56	13.281	13.46	285591	{}	\N	2020-10-12 14:37:08.641385
35694	CSQ	2020-10-01	13.593	13.63	13.51	13.57	228820	{}	\N	2020-10-12 14:37:08.646787
35695	CSQ	2020-10-02	13.43	13.57	13.26	13.48	219921	{}	\N	2020-10-12 14:37:08.652405
35696	CSQ	2020-10-05	13.5	13.79	13.5	13.73	238533	{}	\N	2020-10-12 14:37:08.65677
35697	CSQ	2020-10-06	13.74	13.92	13.56	13.61	336941	{}	\N	2020-10-12 14:37:08.661116
35698	CSQ	2020-10-07	13.75	13.94	13.726	13.9	182328	{}	\N	2020-10-12 14:37:08.666113
35699	CSQ	2020-10-08	13.95	14.05	13.94	13.98	188163	{}	\N	2020-10-12 14:37:08.67041
35700	CSQ	2020-10-09	14.18	14.18	14.07	14.12	229663	{}	\N	2020-10-12 14:37:08.674751
35701	CSQ	2020-10-12	14.12	14.2652	14.0806	14.24	90253	{}	\N	2020-10-12 14:37:08.67904
35702	CSWCL	2020-09-28	25.135	25.22	25.135	25.22	1168	{}	\N	2020-10-12 14:37:10.345237
35703	CSWCL	2020-09-29	25.2	25.2	25.13	25.15	10302	{}	\N	2020-10-12 14:37:10.354593
35704	CSWCL	2020-09-30	25.08	25.15	25.08	25.09	15350	{}	\N	2020-10-12 14:37:10.3616
35705	CSWCL	2020-10-01	25.19	25.2	25.1066	25.2	6893	{}	\N	2020-10-12 14:37:10.366738
35706	CSWCL	2020-10-02	25.2	25.2	25.1483	25.186	16713	{}	\N	2020-10-12 14:37:10.371145
35707	CSWCL	2020-10-05	25.11	25.24	25.11	25.24	20373	{}	\N	2020-10-12 14:37:10.375871
35708	CSWCL	2020-10-06	25.2	25.27	25.07	25.07	9881	{}	\N	2020-10-12 14:37:10.380062
35709	CSWCL	2020-10-07	25.185	25.2684	25.12	25.12	8540	{}	\N	2020-10-12 14:37:10.38492
35710	CSWCL	2020-10-08	25.1916	25.2175	25.1916	25.2175	637	{}	\N	2020-10-12 14:37:10.389419
35711	CSWCL	2020-10-09	25.25	25.27	25.21	25.27	16627	{}	\N	2020-10-12 14:37:10.394204
35712	CTG	2020-09-28	4.95	4.9538	4.93	4.95	1789	{}	\N	2020-10-12 14:37:11.760363
35713	CTG	2020-09-29	4.95	4.95	4.88	4.89	7375	{}	\N	2020-10-12 14:37:11.771161
35714	CTG	2020-09-30	4.815	5.01	4.815	4.97	7961	{}	\N	2020-10-12 14:37:11.777637
35715	CTG	2020-10-01	5	5.09	4.96	5.08	19770	{}	\N	2020-10-12 14:37:11.783422
35716	CTG	2020-10-02	5.022	5.08	4.96	4.96	2457	{}	\N	2020-10-12 14:37:11.788549
35717	CTG	2020-10-05	4.99	4.99	4.9	4.9	9493	{}	\N	2020-10-12 14:37:11.793568
35718	CTG	2020-10-06	4.96	4.98	4.78	4.78	12884	{}	\N	2020-10-12 14:37:11.797686
35719	CTG	2020-10-07	4.79	4.925	4.77	4.8531	19529	{}	\N	2020-10-12 14:37:11.802026
35720	CTG	2020-10-08	4.82	4.82	4.71	4.72	21315	{}	\N	2020-10-12 14:37:11.805959
35721	CTG	2020-10-09	4.72	4.896	4.7	4.7	10113	{}	\N	2020-10-12 14:37:11.810331
35722	CTG	2020-10-12	4.68	4.76	4.68	4.76	11566	{}	\N	2020-10-12 14:37:11.814607
35723	CTRE	2020-09-28	17.52	18.45	17.52	18.275	1003628	{}	\N	2020-10-12 14:37:12.915505
35724	CTRE	2020-09-29	17.95	17.98	17.34	17.65	691353	{}	\N	2020-10-12 14:37:12.926149
35725	CTRE	2020-09-30	17.78	18.23	17.6	17.795	568244	{}	\N	2020-10-12 14:37:12.93336
35726	CTRE	2020-10-01	17.88	18.21	17.71	18.19	454639	{}	\N	2020-10-12 14:37:12.939801
35727	CTRE	2020-10-02	17.98	19.05	17.87	18.95	524912	{}	\N	2020-10-12 14:37:12.945185
35728	CTRE	2020-10-05	19.05	19.1999	18.55	18.74	285573	{}	\N	2020-10-12 14:37:12.950067
35729	CTRE	2020-10-06	18.92	19.16	18.59	18.7	891273	{}	\N	2020-10-12 14:37:12.95446
35730	CTRE	2020-10-07	18.82	18.88	18.52	18.63	428287	{}	\N	2020-10-12 14:37:12.958751
35731	CTRE	2020-10-08	18.69	18.88	18.55	18.72	452307	{}	\N	2020-10-12 14:37:12.962677
35732	CTRE	2020-10-09	18.91	18.91	18.5	18.56	242655	{}	\N	2020-10-12 14:37:12.966899
35733	CTRE	2020-10-12	18.51	18.75	18.39	18.72	26702	{}	\N	2020-10-12 14:37:12.970994
35734	CTXRW	2020-09-29	0.37	0.37	0.2201	0.2201	5790	{}	\N	2020-10-12 14:37:14.127356
35735	CTXRW	2020-10-02	0.35	0.35	0.35	0.35	2300	{}	\N	2020-10-12 14:37:14.13806
35736	CTXRW	2020-10-05	0.3	0.37	0.23	0.34225	1050	{}	\N	2020-10-12 14:37:14.144307
35737	CTXRW	2020-10-06	0.35	0.35	0.3499	0.35	5400	{}	\N	2020-10-12 14:37:14.149833
35738	CTXRW	2020-10-07	0.312501	0.335049	0.2551	0.2551	2650	{}	\N	2020-10-12 14:37:14.15537
35739	CTXRW	2020-10-08	0.35	0.36	0.29	0.32	5193	{}	\N	2020-10-12 14:37:14.16044
35740	CTXRW	2020-10-09	0.323643	0.34	0.311	0.34	6369	{}	\N	2020-10-12 14:37:14.16572
35741	CTXRW	2020-10-12	0.340049	0.37	0.340049	0.340949	4801	{}	\N	2020-10-12 14:37:14.17061
35742	CUK	2020-09-28	13.06	13.06	12.41	12.79	2529939	{}	\N	2020-10-12 14:37:15.302342
35743	CUK	2020-09-29	12.81	12.91	12.47	12.7	2292307	{}	\N	2020-10-12 14:37:15.312021
35744	CUK	2020-09-30	12.92	13.49	12.86	12.92	3835258	{}	\N	2020-10-12 14:37:15.318628
35745	CUK	2020-10-01	12.96	12.99	12.5601	12.87	2730183	{}	\N	2020-10-12 14:37:15.324584
35746	CUK	2020-10-02	12.04	12.97	12.04	12.94	3975551	{}	\N	2020-10-12 14:37:15.331903
35747	CUK	2020-10-05	12.96	13.019	12.6408	12.81	2009664	{}	\N	2020-10-12 14:37:15.339504
35748	CUK	2020-10-06	13.15	13.695	12.94	13.08	4258638	{}	\N	2020-10-12 14:37:15.344382
35749	CUK	2020-10-07	13.33	13.78	13.14	13.74	2741593	{}	\N	2020-10-12 14:37:15.348802
35750	CUK	2020-10-08	14.07	14.07	13.39	13.62	2848674	{}	\N	2020-10-12 14:37:15.352981
35751	CUK	2020-10-09	13.87	13.99	13.54	13.56	2666903	{}	\N	2020-10-12 14:37:15.357117
35752	CUK	2020-10-12	13.2	13.36	13.19	13.25	744936	{}	\N	2020-10-12 14:37:15.36142
35753	CVBF	2020-09-28	16.32	16.725	16.28	16.67	503410	{}	\N	2020-10-12 14:37:16.825137
35754	CVBF	2020-09-29	16.385	16.6	16.26	16.55	269405	{}	\N	2020-10-12 14:37:16.83567
35755	CVBF	2020-09-30	16.69	16.81	16.44	16.64	456370	{}	\N	2020-10-12 14:37:16.841986
35756	CVBF	2020-10-01	16.59	16.8	16.26	16.54	319278	{}	\N	2020-10-12 14:37:16.847775
35757	CVBF	2020-10-02	16.26	17.03	16.26	16.89	324101	{}	\N	2020-10-12 14:37:16.853276
35758	CVBF	2020-10-05	17.01	17.43	16.97	17.36	338722	{}	\N	2020-10-12 14:37:16.858697
35759	CVBF	2020-10-06	17.36	17.86	17.2	17.34	532051	{}	\N	2020-10-12 14:37:16.863034
35760	CVBF	2020-10-07	17.55	17.87	17.48	17.68	494274	{}	\N	2020-10-12 14:37:16.867634
35761	CVBF	2020-10-08	17.84	17.98	17.65	17.89	321533	{}	\N	2020-10-12 14:37:16.872
35762	CVBF	2020-10-09	18	18.07	17.84	17.9	218387	{}	\N	2020-10-12 14:37:16.876544
35763	CVBF	2020-10-12	17.79	18.18	17.2795	18.17	41863	{}	\N	2020-10-12 14:37:16.881002
35764	CVNA	2020-09-28	206.94	218.38	206.25	218.02	2380046	{}	\N	2020-10-12 14:37:17.947543
35765	CVNA	2020-09-29	220.51	232.9	220	226.03	3707696	{}	\N	2020-10-12 14:37:17.956629
35766	CVNA	2020-09-30	223.75	226.68	218.59	223.06	2253873	{}	\N	2020-10-12 14:37:17.961765
35767	CVNA	2020-10-01	227.3	242.15	223	238.1	4610211	{}	\N	2020-10-12 14:37:17.966566
35768	CVNA	2020-10-02	235.89	239.98	212.66	222.7	5642538	{}	\N	2020-10-12 14:37:17.970919
35769	CVNA	2020-10-05	226	229.7739	220.9	225.01	2632287	{}	\N	2020-10-12 14:37:17.975574
35770	CVNA	2020-10-06	229	232.43	210.72	214.65	3463805	{}	\N	2020-10-12 14:37:17.979383
35771	CVNA	2020-10-07	221.86	223.84	214.66	216.96	2358592	{}	\N	2020-10-12 14:37:17.983193
35772	CVNA	2020-10-08	217.86	219.5572	211.25	214	1988823	{}	\N	2020-10-12 14:37:17.986876
35773	CVNA	2020-10-09	214.87	215.51	211.69	213.45	2047646	{}	\N	2020-10-12 14:37:17.990717
35774	CVNA	2020-10-12	216	217	211.76	211.8	509564	{}	\N	2020-10-12 14:37:17.994338
35775	CVX	2020-09-28	73.64	74.76	73.15	73.93	13275308	{}	\N	2020-10-12 14:37:18.97909
35776	CVX	2020-09-29	74	74.14	71.11	71.9	21268661	{}	\N	2020-10-12 14:37:18.987622
35777	CVX	2020-09-30	72.25	72.89	71.61	72	20073955	{}	\N	2020-10-12 14:37:18.993581
35778	CVX	2020-10-01	71.51	71.88	70.03	70.42	29656952	{}	\N	2020-10-12 14:37:18.998775
35779	CVX	2020-10-02	68.92	71.67	68.82	71.19	19894941	{}	\N	2020-10-12 14:37:19.003176
35780	CVX	2020-10-05	71.52	72.73	70.71	72.7	24329190	{}	\N	2020-10-12 14:37:19.008282
35781	CVX	2020-10-06	73.84	74.32	72.245	72.3	29890573	{}	\N	2020-10-12 14:37:19.012308
35782	CVX	2020-10-07	72.17	73.92	71.56	73.78	30547285	{}	\N	2020-10-12 14:37:19.016431
35783	CVX	2020-10-08	74.39	75.55	73.99	75.22	22952097	{}	\N	2020-10-12 14:37:19.020437
35784	CVX	2020-10-09	75.86	76.09	73.98	74	24399859	{}	\N	2020-10-12 14:37:19.024522
35785	CVX	2020-10-12	73.63	74.543	73.2	74.435	3427467	{}	\N	2020-10-12 14:37:19.028453
35786	CXDC	2020-09-28	1.13	1.14	1.13	1.14	7796	{}	\N	2020-10-12 14:37:19.976833
35787	CXDC	2020-09-29	1.13	1.14	1.13	1.14	21992	{}	\N	2020-10-12 14:37:19.99067
35788	CXDC	2020-09-30	1.13	1.135	1.13	1.13	59591	{}	\N	2020-10-12 14:37:19.997707
35789	CXDC	2020-10-01	1.15	1.17	1.15	1.165	50015	{}	\N	2020-10-12 14:37:20.003845
35790	CXDC	2020-10-02	1.1638	1.17	1.16	1.17	28118	{}	\N	2020-10-12 14:37:20.009473
35791	CXDC	2020-10-05	1.15	1.165	1.15	1.1607	36042	{}	\N	2020-10-12 14:37:20.014601
35792	CXDC	2020-10-06	1.16	1.17	1.15	1.16	84398	{}	\N	2020-10-12 14:37:20.019258
35793	CXDC	2020-10-07	1.16	1.165	1.15	1.165	68156	{}	\N	2020-10-12 14:37:20.024229
35794	CXDC	2020-10-08	1.17	1.17	1.16	1.16	53385	{}	\N	2020-10-12 14:37:20.028852
35795	CXDC	2020-10-09	1.16	1.17	1.16	1.16	11236	{}	\N	2020-10-12 14:37:20.033732
35796	CXDC	2020-10-12	1.16	1.16	1.16	1.16	2648	{}	\N	2020-10-12 14:37:20.038095
35797	CYBE	2020-09-28	30.25	30.91	30.25	30.69	33319	{}	\N	2020-10-12 14:37:21.286077
35798	CYBE	2020-09-29	30.72	32.56	30.585	32.02	118481	{}	\N	2020-10-12 14:37:21.296875
35799	CYBE	2020-09-30	32.29	33.18	31.73	31.84	121322	{}	\N	2020-10-12 14:37:21.303861
35800	CYBE	2020-10-01	32.63	33.79	32.4074	33.75	91850	{}	\N	2020-10-12 14:37:21.309931
35801	CYBE	2020-10-02	32.82	34.32	32.75	33.04	92664	{}	\N	2020-10-12 14:37:21.315519
35802	CYBE	2020-10-05	33.42	33.82	33.05	33.5	56242	{}	\N	2020-10-12 14:37:21.320767
35803	CYBE	2020-10-06	33.81	36.04	33.53	34.93	74609	{}	\N	2020-10-12 14:37:21.326131
35804	CYBE	2020-10-07	35.14	36.3	35.13	36.16	52302	{}	\N	2020-10-12 14:37:21.332283
35805	CYBE	2020-10-08	36.75	37.21	35.85	36.98	61903	{}	\N	2020-10-12 14:37:21.340467
35806	CYBE	2020-10-09	37.28	38.18	37.06	38.09	50093	{}	\N	2020-10-12 14:37:21.344698
35807	CYBE	2020-10-12	38.69	39.48	38.5	39.19	18461	{}	\N	2020-10-12 14:37:21.348974
35808	CYTK	2020-09-28	23.75	23.78	22.11	22.37	584347	{}	\N	2020-10-12 14:37:22.337332
35809	CYTK	2020-09-29	22.27	23	22.235	22.33	634514	{}	\N	2020-10-12 14:37:22.347584
35810	CYTK	2020-09-30	22.585	22.585	21.26	21.65	1490340	{}	\N	2020-10-12 14:37:22.353284
35811	CYTK	2020-10-01	21.79	23.33	21.57	22.7	1136529	{}	\N	2020-10-12 14:37:22.358991
35812	CYTK	2020-10-02	22.15	24.52	21.535	23.99	2360086	{}	\N	2020-10-12 14:37:22.36391
35813	CYTK	2020-10-05	26.33	30.14	26.26	28.61	4013229	{}	\N	2020-10-12 14:37:22.368562
35814	CYTK	2020-10-06	29.035	29.249	27.8269	28.55	1575737	{}	\N	2020-10-12 14:37:22.372676
35815	CYTK	2020-10-07	28.9676	29.549	27.61	27.655	2540230	{}	\N	2020-10-12 14:37:22.376837
35816	CYTK	2020-10-08	17.45	17.73	15.16	16	21421405	{}	\N	2020-10-12 14:37:22.380937
35817	CYTK	2020-10-09	16.085	16.4999	15.01	15.26	8015949	{}	\N	2020-10-12 14:37:22.38502
35818	CYTK	2020-10-12	15.12	15.34	14.71	15.2843	815643	{}	\N	2020-10-12 14:37:22.388913
35819	CZZ	2020-09-28	15.4	15.51	14.67	14.77	796253	{}	\N	2020-10-12 14:37:24.266962
35820	CZZ	2020-09-29	14.67	14.75	13.98	14.28	1727379	{}	\N	2020-10-12 14:37:24.278461
35821	CZZ	2020-09-30	14.36	15.08	14.33	14.85	1895728	{}	\N	2020-10-12 14:37:24.285847
35822	CZZ	2020-10-01	14.71	15.28	14.35	15.09	1862426	{}	\N	2020-10-12 14:37:24.291689
35823	CZZ	2020-10-02	14.81	15.1	14.33	14.62	1146145	{}	\N	2020-10-12 14:37:24.296907
35824	CZZ	2020-10-05	14.7	15.05	14.55	15.04	1099673	{}	\N	2020-10-12 14:37:24.301798
35825	CZZ	2020-10-06	15.23	15.3	14.545	14.66	1298747	{}	\N	2020-10-12 14:37:24.306066
35826	CZZ	2020-10-07	14.64	14.7	14.19	14.47	982498	{}	\N	2020-10-12 14:37:24.310453
35827	CZZ	2020-10-08	14.58	14.93	14.36	14.87	1164587	{}	\N	2020-10-12 14:37:24.314554
35828	CZZ	2020-10-09	15.01	15.19	14.27	14.67	3191013	{}	\N	2020-10-12 14:37:24.318767
35829	CZZ	2020-10-12	14.82	14.89	14.44	14.56	62812	{}	\N	2020-10-12 14:37:24.323081
35830	DBE	2020-09-28	9.68	9.75	9.6	9.71	32087	{}	\N	2020-10-12 14:37:25.285916
35831	DBE	2020-09-29	9.68	9.68	9.34	9.39	39421	{}	\N	2020-10-12 14:37:25.295187
35832	DBE	2020-09-30	9.44	9.611	9.44	9.55	168555	{}	\N	2020-10-12 14:37:25.301893
35833	DBE	2020-10-01	9.39	9.39	9.16	9.33	19139	{}	\N	2020-10-12 14:37:25.307712
35834	DBE	2020-10-02	9.04	9.16	8.95	9.02	47907	{}	\N	2020-10-12 14:37:25.312708
35835	DBE	2020-10-05	9.51	9.58	9.48	9.51	9635	{}	\N	2020-10-12 14:37:25.317953
35836	DBE	2020-10-06	9.75	9.77	9.6285	9.64	8878	{}	\N	2020-10-12 14:37:25.32247
35837	DBE	2020-10-07	9.59	9.63	9.51	9.63	61216	{}	\N	2020-10-12 14:37:25.327057
35838	DBE	2020-10-08	9.85	9.85	9.7699	9.83	9628	{}	\N	2020-10-12 14:37:25.33659
35839	DBE	2020-10-09	9.79	9.86	9.715	9.72	26443	{}	\N	2020-10-12 14:37:25.344632
35840	DBE	2020-10-12	9.631	9.65	9.5	9.5127	7001	{}	\N	2020-10-12 14:37:25.351878
35841	DBP	2020-09-28	51.25	51.64	50.975	51.62	11069	{}	\N	2020-10-12 14:37:26.753421
35842	DBP	2020-09-29	51.56	52.2885	51.56	52.15	6875	{}	\N	2020-10-12 14:37:26.762824
35843	DBP	2020-09-30	52	52.0616	51.34	51.34	5769	{}	\N	2020-10-12 14:37:26.768428
35844	DBP	2020-10-01	52.13	52.44	51.802	52.16	7563	{}	\N	2020-10-12 14:37:26.773069
35845	DBP	2020-10-02	52.155	52.46	51.9701	52.1	44661	{}	\N	2020-10-12 14:37:26.777218
35846	DBP	2020-10-05	52.49	52.77	52.26	52.59	39518	{}	\N	2020-10-12 14:37:26.781062
35847	DBP	2020-10-06	52.77	52.77	51.61	51.61	4677	{}	\N	2020-10-12 14:37:26.784865
35848	DBP	2020-10-07	51.41	51.7459	51.41	51.7459	4513	{}	\N	2020-10-12 14:37:26.788284
35849	DBP	2020-10-08	52.11	52.11	51.62	51.77	4202	{}	\N	2020-10-12 14:37:26.792155
35850	DBP	2020-10-09	52.84	53.3038	52.55	53.19	4679	{}	\N	2020-10-12 14:37:26.795751
35851	DBP	2020-10-12	53.12	53.12	52.96	52.96	1991	{}	\N	2020-10-12 14:37:26.799443
35852	CTB	2020-09-28	30.6	31.61	30.57	31.26	334274	{}	\N	2020-10-12 14:37:30.159833
35853	CTB	2020-09-29	31.38	31.57	30.71	31.09	530044	{}	\N	2020-10-12 14:37:30.174862
35854	CTB	2020-09-30	31.12	32.405	31.12	31.7	843639	{}	\N	2020-10-12 14:37:30.181401
35855	CTB	2020-10-01	31.98	33.14	31.83	33.12	740636	{}	\N	2020-10-12 14:37:30.187131
35856	CTB	2020-10-02	32.25	34.12	32.25	33.78	709423	{}	\N	2020-10-12 14:37:30.19236
35857	CTB	2020-10-05	34.14	35.64	34.14	35.46	666473	{}	\N	2020-10-12 14:37:30.19817
35858	CTB	2020-10-06	35.62	36.47	35	35.04	699388	{}	\N	2020-10-12 14:37:30.202956
35859	CTB	2020-10-07	35.56	36.735	35.2486	36.27	773308	{}	\N	2020-10-12 14:37:30.207475
35860	CTB	2020-10-08	36.59	36.98	35.61	36.69	535883	{}	\N	2020-10-12 14:37:30.211884
35861	CTB	2020-10-09	37.09	37.63	36.84	37.55	776435	{}	\N	2020-10-12 14:37:30.216522
35862	CTB	2020-10-12	37.55	37.94	37.2601	37.65	63570	{}	\N	2020-10-12 14:37:30.22086
35863	CTMX	2020-09-28	6.82	6.84	6.63	6.69	185118	{}	\N	2020-10-12 14:37:31.235498
35864	CTMX	2020-09-29	6.69	6.795	6.63	6.69	183335	{}	\N	2020-10-12 14:37:31.245187
35865	CTMX	2020-09-30	6.7	6.83	6.59	6.64	324551	{}	\N	2020-10-12 14:37:31.251728
35866	CTMX	2020-10-01	6.68	6.93	6.68	6.91	260665	{}	\N	2020-10-12 14:37:31.257582
35867	CTMX	2020-10-02	6.76	6.84	6.4	6.47	533194	{}	\N	2020-10-12 14:37:31.262403
35868	CTMX	2020-10-05	6.59	7.06	6.5	7.02	324050	{}	\N	2020-10-12 14:37:31.267262
35869	CTMX	2020-10-06	7.07	7.07	6.69	6.7	290436	{}	\N	2020-10-12 14:37:31.271577
35870	CTMX	2020-10-07	6.73	7.01	6.71	6.94	254321	{}	\N	2020-10-12 14:37:31.275913
35871	CTMX	2020-10-08	7.01	7.08	6.87	6.9	238854	{}	\N	2020-10-12 14:37:31.279976
35872	CTMX	2020-10-09	6.94	6.99	6.76	6.815	177987	{}	\N	2020-10-12 14:37:31.284265
35873	CTMX	2020-10-12	6.83	6.94	6.81	6.81	41399	{}	\N	2020-10-12 14:37:31.288579
35874	CVI	2020-09-28	12.77	12.9	12.52	12.76	744950	{}	\N	2020-10-12 14:37:33.455482
35875	CVI	2020-09-29	12.7	12.86	12.12	12.39	1706834	{}	\N	2020-10-12 14:37:33.463874
35876	CVI	2020-09-30	12.35	12.78	12.25	12.38	1169227	{}	\N	2020-10-12 14:37:33.468956
35877	CVI	2020-10-01	12.16	12.61	11.832	12.07	2852624	{}	\N	2020-10-12 14:37:33.473768
35878	CVI	2020-10-02	11.67	12.55	11.55	12.45	1203472	{}	\N	2020-10-12 14:37:33.477928
35879	CVI	2020-10-05	12.74	13.09	12.53	12.81	1039856	{}	\N	2020-10-12 14:37:33.482382
35880	CVI	2020-10-06	13.09	13.23	12.35	12.5	1270540	{}	\N	2020-10-12 14:37:33.487673
35881	CVI	2020-10-07	12.55	12.74	12.31	12.57	1254543	{}	\N	2020-10-12 14:37:33.49167
35882	CVI	2020-10-08	12.81	13.45	12.68	13.44	1150050	{}	\N	2020-10-12 14:37:33.495296
35883	CVI	2020-10-09	13.54	13.6794	13.01	13.18	1507750	{}	\N	2020-10-12 14:37:33.499064
35884	CVI	2020-10-12	13.08	13.18	12.7	12.76	253641	{}	\N	2020-10-12 14:37:33.503379
35885	CVS	2020-09-28	57.6	58.595	57.6	57.94	5420047	{}	\N	2020-10-12 14:37:34.516019
35886	CVS	2020-09-29	57.95	57.95	57.05	57.54	11619682	{}	\N	2020-10-12 14:37:34.525591
35887	CVS	2020-09-30	57.64	59.095	57.6	58.4	15634309	{}	\N	2020-10-12 14:37:34.531724
35888	CVS	2020-10-01	58.43	58.67	57.355	57.84	14792102	{}	\N	2020-10-12 14:37:34.536937
35889	CVS	2020-10-02	57.09	58.69	56.92	57.83	13972840	{}	\N	2020-10-12 14:37:34.541585
35890	CVS	2020-10-05	58.26	58.97	58.19	58.95	10435022	{}	\N	2020-10-12 14:37:34.545869
35891	CVS	2020-10-06	59.11	59.5176	58.16	58.23	10800094	{}	\N	2020-10-12 14:37:34.549923
35892	CVS	2020-10-07	58.6	59.9789	58.56	59.42	11884064	{}	\N	2020-10-12 14:37:34.553488
35893	CVS	2020-10-08	59.63	60.5	59.328	60.47	9403797	{}	\N	2020-10-12 14:37:34.557254
35894	CVS	2020-10-09	60.8	60.8	59.21	59.4	11277453	{}	\N	2020-10-12 14:37:34.560684
35895	CVS	2020-10-12	59.5	59.95	59.2717	59.78	3959908	{}	\N	2020-10-12 14:37:34.564123
35896	CWCO	2020-09-28	10.79	10.977	10.655	10.74	84227	{}	\N	2020-10-12 14:37:36.332714
35897	CWCO	2020-09-29	10.73	10.83	10.5	10.58	63531	{}	\N	2020-10-12 14:37:36.341197
35898	CWCO	2020-09-30	10.58	10.66	10.33	10.41	42890	{}	\N	2020-10-12 14:37:36.346728
35899	CWCO	2020-10-01	10.41	10.56	10.13	10.32	75709	{}	\N	2020-10-12 14:37:36.352241
35900	CWCO	2020-10-02	10.14	10.6	10.14	10.51	47172	{}	\N	2020-10-12 14:37:36.357075
35901	CWCO	2020-10-05	10.65	11.01	10.65	11.01	58642	{}	\N	2020-10-12 14:37:36.361316
35902	CWCO	2020-10-06	11.1	11.15	10.76	10.82	73748	{}	\N	2020-10-12 14:37:36.365169
35903	CWCO	2020-10-07	10.86	11.14	10.86	11.03	42812	{}	\N	2020-10-12 14:37:36.369147
35904	CWCO	2020-10-08	11.1	11.21	10.93	11.05	51476	{}	\N	2020-10-12 14:37:36.373961
35905	CWCO	2020-10-09	11.16	11.24	10.88	10.89	50629	{}	\N	2020-10-12 14:37:36.377864
35906	CWCO	2020-10-12	10.97	11.05	10.92	11.05	10519	{}	\N	2020-10-12 14:37:36.381638
35907	CXO	2020-09-28	44.66	45.44	43.94	44.94	2822899	{}	\N	2020-10-12 14:37:37.378211
35908	CXO	2020-09-29	44.87	44.9999	43.22	44.4	3385738	{}	\N	2020-10-12 14:37:37.386921
35909	CXO	2020-09-30	44.61	45.25	43.81	44.12	3502422	{}	\N	2020-10-12 14:37:37.392981
35910	CXO	2020-10-01	43.42	43.98	42.14	42.35	2802347	{}	\N	2020-10-12 14:37:37.398552
35911	CXO	2020-10-02	40.99	43.89	40.82	43.07	5239327	{}	\N	2020-10-12 14:37:37.40304
35912	CXO	2020-10-05	43.76	44.56	43.31	44.51	2776691	{}	\N	2020-10-12 14:37:37.407828
35913	CXO	2020-10-06	45.36	46.05	43.35	43.38	2456436	{}	\N	2020-10-12 14:37:37.411614
35914	CXO	2020-10-07	43.75	44.7	43.04	44.59	3186250	{}	\N	2020-10-12 14:37:37.415919
35915	CXO	2020-10-08	44.99	45.63	44.36	45.59	2127455	{}	\N	2020-10-12 14:37:37.419832
35916	CXO	2020-10-09	46.03	46.4	44.75	45.03	5342882	{}	\N	2020-10-12 14:37:37.423887
35917	CXO	2020-10-12	44.69	44.76	43.46	44.215	547102	{}	\N	2020-10-12 14:37:37.427714
35918	CYRN	2020-09-28	1.09	1.1	0.99	1.05	22203	{}	\N	2020-10-12 14:37:38.679416
35919	CYRN	2020-09-29	1.0381	1.0899	0.9901	1.044	34707	{}	\N	2020-10-12 14:37:38.688011
35920	CYRN	2020-09-30	1.04	1.1399	1.03	1.08	27378	{}	\N	2020-10-12 14:37:38.694081
35921	CYRN	2020-10-01	1.11	1.11	1.0201	1.0711	24610	{}	\N	2020-10-12 14:37:38.699591
35922	CYRN	2020-10-02	1.07	1.07	1.02	1.0601	6645	{}	\N	2020-10-12 14:37:38.704789
35923	CYRN	2020-10-05	1.1	1.11	1.0301	1.11	10170	{}	\N	2020-10-12 14:37:38.709452
35924	CYRN	2020-10-06	1.09	1.1004	1.0202	1.06	10235	{}	\N	2020-10-12 14:37:38.713727
35925	CYRN	2020-10-07	1	1.08	0.95	1	74853	{}	\N	2020-10-12 14:37:38.718216
35926	CYRN	2020-10-08	1	1.02	0.98	0.995	12666	{}	\N	2020-10-12 14:37:38.722705
35927	CYRN	2020-10-09	0.99	1.03	0.99	0.9901	13306	{}	\N	2020-10-12 14:37:38.726974
35928	CYRN	2020-10-12	1	1.01	1	1.0068	2217	{}	\N	2020-10-12 14:37:38.731245
36000	CVCO	2020-10-05	186.26	188.3	181.09	186.69	59673	{}	\N	2020-10-12 14:37:49.113649
36001	CVCO	2020-10-06	187	190.35	183.6863	183.81	51152	{}	\N	2020-10-12 14:37:49.117956
36002	CVCO	2020-10-07	185.6	185.7	180.965	184	77967	{}	\N	2020-10-12 14:37:49.121986
36003	CVCO	2020-10-08	186.86	188.995	181.9	184.25	49205	{}	\N	2020-10-12 14:37:49.125554
36004	CVCO	2020-10-09	187.26	187.58	182.17	186.11	54619	{}	\N	2020-10-12 14:37:49.129301
36005	CVCO	2020-10-12	187.27	188.795	185	186.99	10748	{}	\N	2020-10-12 14:37:49.133191
36006	CVM	2020-09-28	12.65	12.68	11.9	12	415995	{}	\N	2020-10-12 14:37:50.22841
36007	CVM	2020-09-29	12.19	12.59	12.09	12.32	252147	{}	\N	2020-10-12 14:37:50.240668
36008	CVM	2020-09-30	12.27	12.82	12.04	12.75	328890	{}	\N	2020-10-12 14:37:50.247022
36009	CVM	2020-10-01	12.56	13.48	12.56	13.41	394382	{}	\N	2020-10-12 14:37:50.252633
36010	CVM	2020-10-02	12.95	13.78	12.8	13.57	341871	{}	\N	2020-10-12 14:37:50.257396
36011	CVM	2020-10-05	13.4	14.2	13.3	13.95	453371	{}	\N	2020-10-12 14:37:50.261939
36012	CVM	2020-10-06	13.99	14.8	13.69	14.12	633036	{}	\N	2020-10-12 14:37:50.266242
36013	CVM	2020-10-07	14.15	14.83	14.14	14.51	510673	{}	\N	2020-10-12 14:37:50.270034
36014	CVM	2020-10-08	14.62	15.61	14.5	14.86	964542	{}	\N	2020-10-12 14:37:50.273906
36015	CVM	2020-10-09	14.92	15.12	14.5801	14.87	288776	{}	\N	2020-10-12 14:37:50.277413
36016	CVM	2020-10-12	14.68	14.87	14.2	14.45	113288	{}	\N	2020-10-12 14:37:50.281284
36017	CWT	2020-09-28	42.96	43.98	42.86	43.55	244440	{}	\N	2020-10-12 14:37:51.341172
36018	CWT	2020-09-29	43.62	43.85	42.82	43.44	588341	{}	\N	2020-10-12 14:37:51.349016
36019	CWT	2020-09-30	43.83	43.935	42.95	43.45	741333	{}	\N	2020-10-12 14:37:51.354579
36020	CWT	2020-10-01	43.46	44.65	43.45	44.64	475932	{}	\N	2020-10-12 14:37:51.359209
36021	CWT	2020-10-02	44.19	45.12	44.05	44.9	369421	{}	\N	2020-10-12 14:37:51.363618
36022	CWT	2020-10-05	45.2	45.2	44.4	44.83	268339	{}	\N	2020-10-12 14:37:51.367964
36023	CWT	2020-10-06	45.14	45.7	44.78	45.02	445248	{}	\N	2020-10-12 14:37:51.371552
36024	CWT	2020-10-07	45.12	45.14	44.26	44.63	408534	{}	\N	2020-10-12 14:37:51.375231
36025	CWT	2020-10-08	44.98	45.56	44.57	45.01	380396	{}	\N	2020-10-12 14:37:51.378786
36026	CWT	2020-10-09	44.92	45.19	44.27	44.93	365211	{}	\N	2020-10-12 14:37:51.382711
36027	CWT	2020-10-12	44.9	45.655	44.9	45.46	80838	{}	\N	2020-10-12 14:37:51.386337
36028	CYBR	2020-09-28	100.14	102.17	98.89	100.35	348588	{}	\N	2020-10-12 14:37:52.382333
36029	CYBR	2020-09-29	100	102.38	98.23	102.14	402089	{}	\N	2020-10-12 14:37:52.393752
36030	CYBR	2020-09-30	101.72	103.86	101.54	103.42	459954	{}	\N	2020-10-12 14:37:52.40066
36031	CYBR	2020-10-01	104.78	108.57	104.37	108.07	555143	{}	\N	2020-10-12 14:37:52.407077
36032	CYBR	2020-10-02	105.91	109.14	105.22	107.39	428450	{}	\N	2020-10-12 14:37:52.412429
36033	CYBR	2020-10-05	107.84	109.5	107.46	109.13	458461	{}	\N	2020-10-12 14:37:52.417001
36034	CYBR	2020-10-06	109.17	113.46	109.17	110.7	395047	{}	\N	2020-10-12 14:37:52.421598
36035	CYBR	2020-10-07	111.04	112.74	110.7824	112.55	254263	{}	\N	2020-10-12 14:37:52.426212
36036	CYBR	2020-10-08	113.5	114.37	111.51	112.65	273011	{}	\N	2020-10-12 14:37:52.431016
36037	CYBR	2020-10-09	112.99	114.675	112.6	114	215296	{}	\N	2020-10-12 14:37:52.43622
36038	CYBR	2020-10-12	114.79	115.77	114.2	115.54	52696	{}	\N	2020-10-12 14:37:52.440567
36039	DAKT	2020-09-28	3.98	4.15	3.98	4.1	113903	{}	\N	2020-10-12 14:37:53.834922
36040	DAKT	2020-09-29	4.1	4.11	4.025	4.07	88944	{}	\N	2020-10-12 14:37:53.843239
36041	DAKT	2020-09-30	4.05	4.13	3.93	3.96	151179	{}	\N	2020-10-12 14:37:53.849749
36042	DAKT	2020-10-01	4.01	4.04	3.89	4.01	104829	{}	\N	2020-10-12 14:37:53.855309
36043	DAKT	2020-10-02	3.93	4.02	3.88	4	119158	{}	\N	2020-10-12 14:37:53.86039
36044	DAKT	2020-10-05	4.01	4.19	4	4.03	141210	{}	\N	2020-10-12 14:37:53.864863
36045	DAKT	2020-10-06	4.075	4.2	4.01	4.01	169139	{}	\N	2020-10-12 14:37:53.869166
36046	DAKT	2020-10-07	4.03	4.12	4.02	4.08	113369	{}	\N	2020-10-12 14:37:53.87346
36047	DAKT	2020-10-08	4.11	4.17	3.97	4.09	238592	{}	\N	2020-10-12 14:37:53.877546
36048	DAKT	2020-10-09	4.12	4.24	4.1	4.24	133581	{}	\N	2020-10-12 14:37:53.881982
36049	DAKT	2020-10-12	4.21	4.23	4.17	4.19	15177	{}	\N	2020-10-12 14:37:53.886546
36050	DBA	2020-09-28	14.63	14.6478	14.52	14.53	479370	{}	\N	2020-10-12 14:37:54.895928
36051	DBA	2020-09-29	14.51	14.59	14.48	14.54	431477	{}	\N	2020-10-12 14:37:54.907792
36052	DBA	2020-09-30	14.54	14.83	14.5399	14.74	1898160	{}	\N	2020-10-12 14:37:54.914745
36053	DBA	2020-10-01	14.72	14.72	14.62	14.65	290442	{}	\N	2020-10-12 14:37:54.921544
36054	DBA	2020-10-02	14.55	14.71	14.55	14.64	888904	{}	\N	2020-10-12 14:37:54.926925
36055	DBA	2020-10-05	14.7	14.725	14.64	14.715	383317	{}	\N	2020-10-12 14:37:54.931972
36056	DBA	2020-10-06	14.72	14.86	14.71	14.71	2151840	{}	\N	2020-10-12 14:37:54.936384
36057	DBA	2020-10-07	14.83	14.9411	14.83	14.92	1403058	{}	\N	2020-10-12 14:37:54.940533
36058	DBA	2020-10-08	15	15	14.893	14.91	555293	{}	\N	2020-10-12 14:37:54.944557
36059	DBA	2020-10-09	14.93	14.99	14.88	14.96	549114	{}	\N	2020-10-12 14:37:54.948894
36060	DBA	2020-10-12	14.96	14.96	14.84	14.8442	575329	{}	\N	2020-10-12 14:37:54.953083
36061	DBGR	2020-09-28	27.56	27.69	27.56	27.69	2356	{}	\N	2020-10-12 14:37:56.824025
36062	DBGR	2020-09-29	27.57	27.65	27.57	27.64	920	{}	\N	2020-10-12 14:37:56.831873
36063	DBGR	2020-09-30	27.58	27.59	27.46	27.49	2600	{}	\N	2020-10-12 14:37:56.837935
36064	DBGR	2020-10-01	27.49	27.5341	27.36	27.5341	1290	{}	\N	2020-10-12 14:37:56.843111
36065	DBGR	2020-10-02	27.29	27.4678	27.29	27.4678	219	{}	\N	2020-10-12 14:37:56.848206
36066	DBGR	2020-10-05	27.71	27.895	27.69	27.895	1433	{}	\N	2020-10-12 14:37:56.852918
35929	DAC	2020-09-28	6.26	6.78	6.21	6.71	204543	{}	\N	2020-10-12 14:37:40.008122
35930	DAC	2020-09-29	6.76	7.43	6.76	7.08	469638	{}	\N	2020-10-12 14:37:40.023248
35931	DAC	2020-09-30	6.98	7.55	6.98	7.34	679522	{}	\N	2020-10-12 14:37:40.030059
35932	DAC	2020-10-01	7.49	7.9399	7.445	7.89	415538	{}	\N	2020-10-12 14:37:40.034334
35933	DAC	2020-10-02	7.67	8.93	7.39	8.8	503846	{}	\N	2020-10-12 14:37:40.038413
35934	DAC	2020-10-05	8.97	9.08	8.2301	8.52	560610	{}	\N	2020-10-12 14:37:40.042053
35935	DAC	2020-10-06	8.5	8.505	7.65	7.78	460526	{}	\N	2020-10-12 14:37:40.045562
35936	DAC	2020-10-07	7.99	8.21	7.72	7.75	397224	{}	\N	2020-10-12 14:37:40.048991
35937	DAC	2020-10-08	7.82	9.05	7.8001	8.95	680972	{}	\N	2020-10-12 14:37:40.052512
35938	DAC	2020-10-09	8.8	8.95	8.06	8.41	806951	{}	\N	2020-10-12 14:37:40.056142
35939	DAC	2020-10-12	8.5	8.95	8.48	8.7701	259564	{}	\N	2020-10-12 14:37:40.059582
35940	DARE	2020-09-28	0.97	1.04	0.97	0.999	244597	{}	\N	2020-10-12 14:37:41.338891
35941	DARE	2020-09-29	1	1.025	0.9968	1.01	249862	{}	\N	2020-10-12 14:37:41.346725
35942	DARE	2020-09-30	1.01	1.04	0.997	0.9987	535414	{}	\N	2020-10-12 14:37:41.352434
35943	DARE	2020-10-01	1.03	1.03	1	1.015	499631	{}	\N	2020-10-12 14:37:41.35815
35944	DARE	2020-10-02	0.999	1.04	0.999	1.04	394163	{}	\N	2020-10-12 14:37:41.362809
35945	DARE	2020-10-05	1.03	1.06	1.0217	1.03	584149	{}	\N	2020-10-12 14:37:41.367511
35946	DARE	2020-10-06	1.03	1.09	1.03	1.04	434323	{}	\N	2020-10-12 14:37:41.371531
35947	DARE	2020-10-07	1.06	1.15	1.06	1.11	2722410	{}	\N	2020-10-12 14:37:41.375111
35948	DARE	2020-10-08	1.08	1.1	1.05	1.08	430552	{}	\N	2020-10-12 14:37:41.379023
35949	DARE	2020-10-09	1.09	1.1	1.07	1.0732	430464	{}	\N	2020-10-12 14:37:41.38289
35950	DARE	2020-10-12	1.08	1.11	1.07	1.1	190558	{}	\N	2020-10-12 14:37:41.386792
36067	DBGR	2020-10-06	27.96	27.96	27.64	27.64	689	{}	\N	2020-10-12 14:37:56.857402
36068	DBGR	2020-10-07	27.9362	27.9362	27.9362	27.9362	67	{}	\N	2020-10-12 14:37:56.861644
36069	DBGR	2020-10-08	28.13	28.2033	28.13	28.2033	200	{}	\N	2020-10-12 14:37:56.865938
36070	DBGR	2020-10-09	28.2014	28.2014	28.2014	28.2014	0	{}	\N	2020-10-12 14:37:56.870437
36071	CTRN	2020-09-28	24.21	25.25	24.21	24.96	210753	{}	\N	2020-10-12 14:37:59.473115
36072	CTRN	2020-09-29	24.84	25.04	24.25	24.74	191331	{}	\N	2020-10-12 14:37:59.48257
36073	CTRN	2020-09-30	24.88	25.8689	24.81	24.98	209785	{}	\N	2020-10-12 14:37:59.489039
36074	CTRN	2020-10-01	26.122	26.4	25.33	25.99	140378	{}	\N	2020-10-12 14:37:59.496614
36075	CTRN	2020-10-02	25.37	26.63	25.255	26.55	129553	{}	\N	2020-10-12 14:37:59.501734
36076	CTRN	2020-10-05	26.59	28.08	26.54	28.02	179433	{}	\N	2020-10-12 14:37:59.506591
36077	CTRN	2020-10-06	28.27	29.5	27.78	28	312340	{}	\N	2020-10-12 14:37:59.510551
36078	CTRN	2020-10-07	28.55	29.26	27.885	28.02	225078	{}	\N	2020-10-12 14:37:59.514587
36079	CTRN	2020-10-08	28.28	28.6099	27.41	27.6	178030	{}	\N	2020-10-12 14:37:59.518838
36080	CTRN	2020-10-09	27.75	29.11	27.39	27.59	144211	{}	\N	2020-10-12 14:37:59.522976
36081	CTRN	2020-10-12	27.8	28.155	27.75	27.97	15286	{}	\N	2020-10-12 14:37:59.527236
36082	CTY	2020-09-28	25.08	25.08	24.99	25.047	72256	{}	\N	2020-10-12 14:38:00.516074
36083	CTY	2020-09-29	25.01	25.01	24.88	24.92	91450	{}	\N	2020-10-12 14:38:00.525727
36084	CTY	2020-09-30	24.91	24.98	24.75	24.77	307592	{}	\N	2020-10-12 14:38:00.531614
36085	CTY	2020-10-01	24.81	25.1	24.81	25.0736	102038	{}	\N	2020-10-12 14:38:00.536489
36086	CTY	2020-10-02	24.88	25.01	24.873	25	112678	{}	\N	2020-10-12 14:38:00.541399
36087	CTY	2020-10-05	25	25.135	25	25.07	93306	{}	\N	2020-10-12 14:38:00.546079
36088	CTY	2020-10-06	25.07	25.1802	25	25.09	116168	{}	\N	2020-10-12 14:38:00.55019
36089	CTY	2020-10-07	25.09	25.14	25.02	25.08	90784	{}	\N	2020-10-12 14:38:00.554804
36090	CTY	2020-10-08	25.04	25.1417	25.04	25.14	92943	{}	\N	2020-10-12 14:38:00.558917
36091	CTY	2020-10-09	25.07	25.15	25.04	25.15	66512	{}	\N	2020-10-12 14:38:00.562806
36092	CTY	2020-10-12	25.1004	25.12	25.08	25.08	11190	{}	\N	2020-10-12 14:38:00.567325
36093	CURO	2020-09-28	6.98	7.26	6.96	7.14	164233	{}	\N	2020-10-12 14:38:01.899563
36094	CURO	2020-09-29	7.13	7.19	6.89	7.01	266167	{}	\N	2020-10-12 14:38:01.909413
36095	CURO	2020-09-30	6.94	7.2762	6.94	7.05	420363	{}	\N	2020-10-12 14:38:01.916276
36096	CURO	2020-10-01	7.06	7.3	6.99	7.3	304745	{}	\N	2020-10-12 14:38:01.922211
36097	CURO	2020-10-02	7.12	7.515	7.1	7.41	227220	{}	\N	2020-10-12 14:38:01.927166
36098	CURO	2020-10-05	7.47	7.83	7.45	7.77	221542	{}	\N	2020-10-12 14:38:01.932091
36099	CURO	2020-10-06	7.85	8.08	7.47	7.51	297811	{}	\N	2020-10-12 14:38:01.936427
36100	CURO	2020-10-07	7.66	7.805	7.56	7.74	236871	{}	\N	2020-10-12 14:38:01.940766
36101	CURO	2020-10-08	7.84	8.4	7.75	8.35	357992	{}	\N	2020-10-12 14:38:01.944928
36102	CURO	2020-10-09	8.46	8.56	8.25	8.41	306807	{}	\N	2020-10-12 14:38:01.94929
36103	CURO	2020-10-12	8.53	8.68	8.36	8.5182	41390	{}	\N	2020-10-12 14:38:01.953828
36104	CVEO	2020-09-28	0.6759	0.7	0.6682	0.6723	135583	{}	\N	2020-10-12 14:38:03.104992
36105	CVEO	2020-09-29	0.67	0.6937	0.65	0.66	436782	{}	\N	2020-10-12 14:38:03.112692
36106	CVEO	2020-09-30	0.66	0.6848	0.66	0.6751	316681	{}	\N	2020-10-12 14:38:03.118506
36107	CVEO	2020-10-01	0.69	0.69	0.650697	0.6777	309446	{}	\N	2020-10-12 14:38:03.123274
36108	CVEO	2020-10-02	0.68	0.6834	0.65	0.655	331224	{}	\N	2020-10-12 14:38:03.127436
36109	CVEO	2020-10-05	0.6578	0.69	0.65	0.655	474208	{}	\N	2020-10-12 14:38:03.13116
36110	CVEO	2020-10-06	0.6555	0.69	0.61	0.6824	1539022	{}	\N	2020-10-12 14:38:03.135655
36111	CVEO	2020-10-07	0.66	0.7	0.6415	0.6627	1040600	{}	\N	2020-10-12 14:38:03.139405
36112	CVEO	2020-10-08	0.65	0.685	0.645	0.679	705992	{}	\N	2020-10-12 14:38:03.143316
36113	CVEO	2020-10-09	0.67	0.6961	0.6513	0.6787	612714	{}	\N	2020-10-12 14:38:03.146851
36114	CVEO	2020-10-12	0.6912	0.6912	0.67	0.675	82726	{}	\N	2020-10-12 14:38:03.150807
36115	CVY	2020-09-28	16.21	16.5	16.21	16.4	28506	{}	\N	2020-10-12 14:38:04.299608
36116	CVY	2020-09-29	16.42	16.42	16.151	16.34	26716	{}	\N	2020-10-12 14:38:04.310363
36117	CVY	2020-09-30	16.4	16.499	16.21	16.34	21152	{}	\N	2020-10-12 14:38:04.348793
36118	CVY	2020-10-01	16.33	16.43	16.26	16.35	9871	{}	\N	2020-10-12 14:38:04.356004
36119	CVY	2020-10-02	16.16	16.6499	16.16	16.61	28761	{}	\N	2020-10-12 14:38:04.362077
36120	CVY	2020-10-05	16.7	16.91	16.7	16.91	14615	{}	\N	2020-10-12 14:38:04.367626
36121	CVY	2020-10-06	17	17.1586	16.78	16.83	16378	{}	\N	2020-10-12 14:38:04.372636
36122	CVY	2020-10-07	16.94	17.08	16.87	17.05	17185	{}	\N	2020-10-12 14:38:04.37665
36123	CVY	2020-10-08	17.14	17.33	17.1	17.3	24925	{}	\N	2020-10-12 14:38:04.380444
36124	CVY	2020-10-09	17.45	17.45	17.25	17.26	9930	{}	\N	2020-10-12 14:38:04.384264
36125	CVY	2020-10-12	17.3	17.3106	17.26	17.2925	4610	{}	\N	2020-10-12 14:38:04.388194
36126	CX	2020-09-28	3.64	3.74	3.61	3.72	5335235	{}	\N	2020-10-12 14:38:05.425396
36127	CX	2020-09-29	3.72	3.72	3.62	3.62	6030430	{}	\N	2020-10-12 14:38:05.434657
36128	CX	2020-09-30	3.62	3.84	3.6173	3.8	14650551	{}	\N	2020-10-12 14:38:05.440852
36129	CX	2020-10-01	3.82	3.86	3.6721	3.72	4849137	{}	\N	2020-10-12 14:38:05.446331
36130	CX	2020-10-02	3.64	3.8	3.6016	3.77	6864082	{}	\N	2020-10-12 14:38:05.451072
36131	CX	2020-10-05	3.91	3.99	3.8	3.97	26036956	{}	\N	2020-10-12 14:38:05.456081
36132	CX	2020-10-06	4.03	4.03	3.84	3.84	10323447	{}	\N	2020-10-12 14:38:05.460189
36133	CX	2020-10-07	3.92	3.98	3.86	3.88	14496408	{}	\N	2020-10-12 14:38:05.464154
36134	CX	2020-10-08	3.94	4.09	3.9	4.09	18387934	{}	\N	2020-10-12 14:38:05.468926
36135	CX	2020-10-09	4.1	4.1	4.03	4.05	12483464	{}	\N	2020-10-12 14:38:05.473339
36136	CX	2020-10-12	4.05	4.08	4.02	4.0418	1526874	{}	\N	2020-10-12 14:38:05.478349
36137	CXSE	2020-09-28	109.26	109.52	108.861	109.21	16102	{}	\N	2020-10-12 14:38:07.027514
36138	CXSE	2020-09-29	109.5	109.985	109.5	109.81	36572	{}	\N	2020-10-12 14:38:07.03929
36139	CXSE	2020-09-30	111.27	112.43	111.22	112.38	45809	{}	\N	2020-10-12 14:38:07.045358
36140	CXSE	2020-10-01	113.36	113.675	112.85	113.5	18207	{}	\N	2020-10-12 14:38:07.050898
36141	CXSE	2020-10-02	111.72	113.1685	111.47	112.02	23762	{}	\N	2020-10-12 14:38:07.056054
36142	CXSE	2020-10-05	112.39	113.09	112.39	113.02	46279	{}	\N	2020-10-12 14:38:07.060798
36143	CXSE	2020-10-06	114.01	115.0621	113.795	114.22	38497	{}	\N	2020-10-12 14:38:07.065289
36144	CXSE	2020-10-07	115.48	115.73	114.77	115.4	83733	{}	\N	2020-10-12 14:38:07.069597
36145	CXSE	2020-10-08	115.73	116.129	115.38	116.04	18946	{}	\N	2020-10-12 14:38:07.073594
36146	CXSE	2020-10-09	116.13	117.5565	116.13	117.46	25934	{}	\N	2020-10-12 14:38:07.077603
36147	CXSE	2020-10-12	119.18	119.298	118.87	119.26	11075	{}	\N	2020-10-12 14:38:07.081545
36148	CYCC	2020-09-28	3.5	3.53	3.45	3.48	84683	{}	\N	2020-10-12 14:38:08.020149
36149	CYCC	2020-09-29	3.4761	3.79	3.465	3.75	233135	{}	\N	2020-10-12 14:38:08.030375
36150	CYCC	2020-09-30	3.765	3.765	3.63	3.65	47791	{}	\N	2020-10-12 14:38:08.036689
36151	CYCC	2020-10-01	3.66	3.66	3.51	3.57	58028	{}	\N	2020-10-12 14:38:08.042514
36152	CYCC	2020-10-02	3.5738	3.72	3.5738	3.67	52024	{}	\N	2020-10-12 14:38:08.047613
36153	CYCC	2020-10-05	3.7	3.82	3.63	3.8	96074	{}	\N	2020-10-12 14:38:08.052684
36154	CYCC	2020-10-06	3.8	4.25	3.7852	4.05	999318	{}	\N	2020-10-12 14:38:08.057209
36155	CYCC	2020-10-07	4.05	4.0999	3.97	4.02	127754	{}	\N	2020-10-12 14:38:08.06141
36156	CYCC	2020-10-08	3.99	4.0922	3.845	3.97	179901	{}	\N	2020-10-12 14:38:08.065704
36157	CYCC	2020-10-09	4	4	3.85	3.94	77876	{}	\N	2020-10-12 14:38:08.070332
36158	CYCC	2020-10-12	3.92	3.95	3.88	3.92	13919	{}	\N	2020-10-12 14:38:08.074566
36159	D	2020-09-28	77.12	78.42	77.06	77.23	3403724	{}	\N	2020-10-12 14:38:09.367821
36160	D	2020-09-29	78.36	78.9	77.49	77.88	6966931	{}	\N	2020-10-12 14:38:09.376196
36161	D	2020-09-30	78.4	79.23	78.13	78.93	7057716	{}	\N	2020-10-12 14:38:09.381738
36162	D	2020-10-01	79.01	79.38	77.97	78.59	9596163	{}	\N	2020-10-12 14:38:09.386874
36163	D	2020-10-02	78.37	80.55	77.87	79.8	6701512	{}	\N	2020-10-12 14:38:09.391518
36164	D	2020-10-05	79.6	80.69	79.21	80.09	5897297	{}	\N	2020-10-12 14:38:09.395854
36165	D	2020-10-06	80.36	81.76	79.88	81.11	6976928	{}	\N	2020-10-12 14:38:09.399742
36166	D	2020-10-07	81.22	81.5	80.429	81.02	4239336	{}	\N	2020-10-12 14:38:09.403689
36167	D	2020-10-08	81.18	82.97	80.84	82.06	6110299	{}	\N	2020-10-12 14:38:09.407648
36168	D	2020-10-09	82.31	82.63	81.39	81.78	5281166	{}	\N	2020-10-12 14:38:09.411528
36169	D	2020-10-12	81.78	82.705	81.7	82.7	791432	{}	\N	2020-10-12 14:38:09.415399
36170	DAL	2020-09-28	30.6	31.96	30.1389	31.34	22486402	{}	\N	2020-10-12 14:38:10.708869
36171	DAL	2020-09-29	31.1	31.11	30.3	30.61	29694767	{}	\N	2020-10-12 14:38:10.717603
36172	DAL	2020-09-30	30.99	31.85	30.48	30.58	32251671	{}	\N	2020-10-12 14:38:10.723657
36173	DAL	2020-10-01	30.99	31.36	30.575	31.1	24307505	{}	\N	2020-10-12 14:38:10.728685
36174	DAL	2020-10-02	29.75	32.3	29.61	31.75	53912481	{}	\N	2020-10-12 14:38:10.733463
36175	DAL	2020-10-05	32.35	32.43	31.52	32	30018065	{}	\N	2020-10-12 14:38:10.738473
36176	DAL	2020-10-06	32.63	32.72	30.91	31.06	38730857	{}	\N	2020-10-12 14:38:10.742625
36177	DAL	2020-10-07	32.22	32.36	31.64	32.15	30685805	{}	\N	2020-10-12 14:38:10.746646
36178	DAL	2020-10-08	32.69	33	31.915	32.68	36062802	{}	\N	2020-10-12 14:38:10.751136
36179	DAL	2020-10-09	32.71	33.2742	32.29	32.81	33181981	{}	\N	2020-10-12 14:38:10.755476
36180	DAL	2020-10-12	32.68	32.8	32.3	32.6631	7268209	{}	\N	2020-10-12 14:38:10.759636
36181	DB	2020-09-28	8.29	8.505	8.26	8.43	4395714	{}	\N	2020-10-12 14:38:11.696845
36182	DB	2020-09-29	8.35	8.3921	8.225	8.27	8103487	{}	\N	2020-10-12 14:38:11.706703
36183	DB	2020-09-30	8.4	8.57	8.36	8.4	8885230	{}	\N	2020-10-12 14:38:11.712747
36184	DB	2020-10-01	8.34	8.42	8.275	8.36	8283332	{}	\N	2020-10-12 14:38:11.718074
36185	DB	2020-10-02	8.3	8.55	8.29	8.5	6569944	{}	\N	2020-10-12 14:38:11.723422
36186	DB	2020-10-05	8.56	8.71	8.5483	8.7	10910602	{}	\N	2020-10-12 14:38:11.727848
36187	DB	2020-10-06	9.07	9.195	8.995	9.01	15032923	{}	\N	2020-10-12 14:38:11.73232
36188	DB	2020-10-07	9.18	9.27	9.145	9.23	8278902	{}	\N	2020-10-12 14:38:11.736771
36189	DB	2020-10-08	9.37	9.4	9.29	9.36	9295002	{}	\N	2020-10-12 14:38:11.741491
36190	DB	2020-10-09	9.39	9.39	9.22	9.28	4316893	{}	\N	2020-10-12 14:38:11.745273
36191	DB	2020-10-12	9.24	9.26	9.2181	9.25	1083596	{}	\N	2020-10-12 14:38:11.749351
36192	DBEZ	2020-09-28	29.48	29.55	29.48	29.55	3501	{}	\N	2020-10-12 14:38:13.942425
36193	DBEZ	2020-09-29	29.42	29.42	29.42	29.42	2349	{}	\N	2020-10-12 14:38:13.950383
36194	DBEZ	2020-09-30	29.58	29.58	29.33	29.33	18211	{}	\N	2020-10-12 14:38:13.956528
36195	DBEZ	2020-10-01	29.38	29.5	29.38	29.49	1416	{}	\N	2020-10-12 14:38:13.961792
36196	DBEZ	2020-10-02	29.36	29.5229	29.36	29.5229	336	{}	\N	2020-10-12 14:38:13.967688
36197	DBEZ	2020-10-05	29.74	29.91	29.74	29.91	6551	{}	\N	2020-10-12 14:38:13.972419
36198	DBEZ	2020-10-06	30	30	29.67	29.67	773	{}	\N	2020-10-12 14:38:13.97703
36199	DBEZ	2020-10-07	29.85	29.9896	29.85	29.9896	443	{}	\N	2020-10-12 14:38:13.981605
36200	DBEZ	2020-10-08	30.2	30.2328	30.1999	30.2328	509	{}	\N	2020-10-12 14:38:13.986115
36201	DBEZ	2020-10-09	30.25	30.3183	30.25	30.3183	219	{}	\N	2020-10-12 14:38:13.990679
36202	CUB	2020-09-28	57.74	61.5298	57.68	60.98	622349	{}	\N	2020-10-12 14:38:15.080128
36203	CUB	2020-09-29	60.71	61.335	58.98	59.39	1742475	{}	\N	2020-10-12 14:38:15.092654
36204	CUB	2020-09-30	59.24	60.661	57.34	58.17	592090	{}	\N	2020-10-12 14:38:15.09948
36205	CUB	2020-10-01	59.24	59.8265	57.3	57.54	970509	{}	\N	2020-10-12 14:38:15.105927
36206	CUB	2020-10-02	56.1	58.23	55.84	57.25	1412706	{}	\N	2020-10-12 14:38:15.110947
36207	CUB	2020-10-05	58.22	59.2	57.34	58.12	445728	{}	\N	2020-10-12 14:38:15.116127
36208	CUB	2020-10-06	58.74	60.28	58.14	58.24	839247	{}	\N	2020-10-12 14:38:15.120793
36209	CUB	2020-10-07	58.83	60.765	58.64	59.94	885130	{}	\N	2020-10-12 14:38:15.125161
36210	CUB	2020-10-08	60.81	61.12	59.41	60.95	1046271	{}	\N	2020-10-12 14:38:15.129439
36211	CUB	2020-10-09	60.71	61.18	59.84	60.07	1006275	{}	\N	2020-10-12 14:38:15.133644
36212	CUB	2020-10-12	59.64	59.68	58.455	59.43	90992	{}	\N	2020-10-12 14:38:15.137928
36213	CVCY	2020-09-28	12.24	12.69	12.2	12.41	30114	{}	\N	2020-10-12 14:38:16.63213
36214	CVCY	2020-09-29	12.46	12.46	12.1	12.33	14511	{}	\N	2020-10-12 14:38:16.642633
36215	CVCY	2020-09-30	12.31	12.435	12.25	12.35	18658	{}	\N	2020-10-12 14:38:16.648933
36216	CVCY	2020-10-01	12.3	12.48	12.3	12.39	19134	{}	\N	2020-10-12 14:38:16.65465
36217	CVCY	2020-10-02	12.25	12.5	12.25	12.49	13845	{}	\N	2020-10-12 14:38:16.659685
36218	CVCY	2020-10-05	12.5251	12.97	12.52	12.89	23314	{}	\N	2020-10-12 14:38:16.665181
36219	CVCY	2020-10-06	12.98	13.28	12.9759	13.25	37286	{}	\N	2020-10-12 14:38:16.669909
36220	CVCY	2020-10-07	12.87	13.56	12.87	13.53	16014	{}	\N	2020-10-12 14:38:16.674455
36221	CVCY	2020-10-08	13.6	13.6	13.29	13.4	8686	{}	\N	2020-10-12 14:38:16.679005
36222	CVCY	2020-10-09	13.48	13.5228	13.28	13.46	10106	{}	\N	2020-10-12 14:38:16.683936
36223	CVCY	2020-10-12	13.39	13.39	13.3027	13.3027	2414	{}	\N	2020-10-12 14:38:16.688509
36224	CWBC	2020-09-28	8.66	8.66	8.66	8.66	281	{}	\N	2020-10-12 14:38:18.786705
36225	CWBC	2020-09-29	8.36	8.36	8.01	8.33	1575	{}	\N	2020-10-12 14:38:18.79735
36226	CWBC	2020-09-30	8.25	8.35	8.03	8.03	404	{}	\N	2020-10-12 14:38:18.803903
36227	CWBC	2020-10-01	8.4	8.4	8.4	8.4	103	{}	\N	2020-10-12 14:38:18.809494
36228	CWBC	2020-10-02	8.03	8.7	8.03	8.4787	27890	{}	\N	2020-10-12 14:38:18.814487
36229	CWBC	2020-10-05	8.25	8.47	8.25	8.47	621	{}	\N	2020-10-12 14:38:18.819765
36230	CWBC	2020-10-06	8.114	8.16	8.114	8.16	880	{}	\N	2020-10-12 14:38:18.824396
36231	CWBC	2020-10-08	8.31	8.35	8.3	8.3	1716	{}	\N	2020-10-12 14:38:18.828684
36232	CWBC	2020-10-12	8.25	8.25	8.225	8.25	3304	{}	\N	2020-10-12 14:38:18.833373
36233	CXW	2020-09-28	8.14	8.455	8.14	8.25	1560167	{}	\N	2020-10-12 14:38:21.18914
36234	CXW	2020-09-29	8.24	8.3279	8.04	8.16	2631547	{}	\N	2020-10-12 14:38:21.199708
36235	CXW	2020-09-30	8.15	8.3901	7.955	8	3076813	{}	\N	2020-10-12 14:38:21.206486
36236	CXW	2020-10-01	8.01	8.24	7.955	8.18	3439281	{}	\N	2020-10-12 14:38:21.212262
36237	CXW	2020-10-02	8	8.32	7.925	8.31	2986628	{}	\N	2020-10-12 14:38:21.217915
36238	CXW	2020-10-05	8.34	8.43	7.95	8.03	2836356	{}	\N	2020-10-12 14:38:21.223506
36239	CXW	2020-10-06	8.15	8.23	7.92	7.98	2918993	{}	\N	2020-10-12 14:38:21.228083
36240	CXW	2020-10-07	8.02	8.11	7.89	7.91	2960988	{}	\N	2020-10-12 14:38:21.23265
36241	CXW	2020-10-08	7.95	8	7.8	7.99	2497438	{}	\N	2020-10-12 14:38:21.237566
36242	CXW	2020-10-09	8.05	8.14	7.85	7.92	3292259	{}	\N	2020-10-12 14:38:21.241737
36243	CXW	2020-10-12	7.93	7.95	7.755	7.89	694020	{}	\N	2020-10-12 14:38:21.245882
36244	CYD	2020-09-28	18.7	19	18.18	18.65	58696	{}	\N	2020-10-12 14:38:22.193805
36245	CYD	2020-09-29	18.6	18.6	17.56	17.59	94950	{}	\N	2020-10-12 14:38:22.201036
36246	CYD	2020-09-30	17.66	18.395	17.66	18	74674	{}	\N	2020-10-12 14:38:22.206879
36247	CYD	2020-10-01	18.23	18.5	17.7	17.74	61540	{}	\N	2020-10-12 14:38:22.212268
36248	CYD	2020-10-02	17.49	18.36	17.49	18.08	32592	{}	\N	2020-10-12 14:38:22.21777
36249	CYD	2020-10-05	18.2	18.76	17.871	18.09	73272	{}	\N	2020-10-12 14:38:22.223496
36250	CYD	2020-10-06	18.1201	18.1859	17.8	17.86	40040	{}	\N	2020-10-12 14:38:22.229117
36251	CYD	2020-10-07	17.9	18.2	17.42	17.67	89146	{}	\N	2020-10-12 14:38:22.23466
36252	CYD	2020-10-08	17.82	17.895	17.45	17.65	37718	{}	\N	2020-10-12 14:38:22.240301
36253	CYD	2020-10-09	17.7204	17.7899	17.4959	17.64	37966	{}	\N	2020-10-12 14:38:22.245766
36254	CYD	2020-10-12	18.7	18.99	18.58	18.8	148238	{}	\N	2020-10-12 14:38:22.249987
36255	DAIO	2020-09-28	3.06	3.1201	3.04	3.11	11396	{}	\N	2020-10-12 14:38:23.838555
36256	DAIO	2020-09-29	3.145	3.18	3.1306	3.18	13844	{}	\N	2020-10-12 14:38:23.84783
36257	DAIO	2020-09-30	3.18	3.235	3.175	3.18	23053	{}	\N	2020-10-12 14:38:23.854408
36258	DAIO	2020-10-01	3.17	3.23	3.15	3.23	8041	{}	\N	2020-10-12 14:38:23.859418
36259	DAIO	2020-10-02	3.14	3.23	3.08	3.12	9702	{}	\N	2020-10-12 14:38:23.863728
36260	DAIO	2020-10-05	3.11	3.12	3.05	3.05	10592	{}	\N	2020-10-12 14:38:23.868848
36261	DAIO	2020-10-06	3.07	3.13	3.06	3.06	6532	{}	\N	2020-10-12 14:38:23.873042
36262	DAIO	2020-10-07	3.06	3.743	2.9901	3.6	198590	{}	\N	2020-10-12 14:38:23.87667
36263	DAIO	2020-10-08	3.62	3.7	3.49	3.63	28795	{}	\N	2020-10-12 14:38:23.880174
36264	DAIO	2020-10-09	3.48	3.6	3.48	3.52	9476	{}	\N	2020-10-12 14:38:23.883921
36265	DAIO	2020-10-12	3.49	3.58	3.49	3.58	221	{}	\N	2020-10-12 14:38:23.887769
36266	DBEM	2020-09-28	23.95	24.11	23.869	23.9573	9061	{}	\N	2020-10-12 14:38:25.615343
36267	DBEM	2020-09-29	23.82	24.05	23.82	24.05	15364	{}	\N	2020-10-12 14:38:25.623704
36268	DBEM	2020-09-30	24.1999	24.33	24.145	24.3053	18142	{}	\N	2020-10-12 14:38:25.62964
36269	DBEM	2020-10-01	24.45	24.5	24.315	24.5	10564	{}	\N	2020-10-12 14:38:25.635289
36270	DBEM	2020-10-02	24.22	24.5399	24.22	24.305	4059	{}	\N	2020-10-12 14:38:25.640384
36271	DBEM	2020-10-05	24.45	24.49	24.33	24.44	25356	{}	\N	2020-10-12 14:38:25.644533
36272	DBEM	2020-10-06	24.54	24.65	24.47	24.53	11565	{}	\N	2020-10-12 14:38:25.648749
36273	DBEM	2020-10-07	24.705	24.82	24.64	24.74	12166	{}	\N	2020-10-12 14:38:25.653262
36274	DBEM	2020-10-08	24.73	24.98	24.73	24.905	59018	{}	\N	2020-10-12 14:38:25.65767
36275	DBEM	2020-10-09	25.03	25.07	24.91	25.07	14711	{}	\N	2020-10-12 14:38:25.66202
36276	DBO	2020-09-28	7.22	7.31	7.172	7.28	705975	{}	\N	2020-10-12 14:38:27.38248
36277	DBO	2020-09-29	7.17	7.18	6.935	7.03	541187	{}	\N	2020-10-12 14:38:27.389969
36278	DBO	2020-09-30	7.1	7.23	7.08	7.19	234187	{}	\N	2020-10-12 14:38:27.39572
36279	DBO	2020-10-01	7.03	7.05	6.81	6.98	546927	{}	\N	2020-10-12 14:38:27.402223
36280	DBO	2020-10-02	6.68	6.83	6.64	6.71	701680	{}	\N	2020-10-12 14:38:27.407234
36281	DBO	2020-10-05	7	7.15	6.97	7.1	889080	{}	\N	2020-10-12 14:38:27.411499
36282	DBO	2020-10-06	7.27	7.33	7.19	7.21	675182	{}	\N	2020-10-12 14:38:27.415982
36283	DBO	2020-10-07	7.14	7.2	7.08	7.2	442308	{}	\N	2020-10-12 14:38:27.420645
36284	DBO	2020-10-08	7.34	7.4	7.29	7.38	697788	{}	\N	2020-10-12 14:38:27.425183
36285	DBO	2020-10-09	7.35	7.41	7.2575	7.28	830216	{}	\N	2020-10-12 14:38:27.429781
36286	DBO	2020-10-12	7.2	7.2	7.05	7.11	204178	{}	\N	2020-10-12 14:38:27.43473
36287	DDF	2020-09-28	8.12	8.1377	8.05	8.05	21206	{}	\N	2020-10-12 14:38:30.198431
36288	DDF	2020-09-29	8.05	8.0803	8	8	26173	{}	\N	2020-10-12 14:38:30.209161
36289	DDF	2020-09-30	8.13	8.13	8.03	8.07	49988	{}	\N	2020-10-12 14:38:30.215718
36290	DDF	2020-10-01	8.13	8.13	8.07	8.1	17787	{}	\N	2020-10-12 14:38:30.221249
36291	DDF	2020-10-02	8.12	8.18	8.0551	8.15	44496	{}	\N	2020-10-12 14:38:30.226178
36292	DDF	2020-10-05	8.14	8.25	8.14	8.21	26720	{}	\N	2020-10-12 14:38:30.230709
36293	DDF	2020-10-06	8.23	8.33	8.19	8.1901	52111	{}	\N	2020-10-12 14:38:30.234845
36294	DDF	2020-10-07	8.28	8.34	8.23	8.3	51910	{}	\N	2020-10-12 14:38:30.23866
36295	DDF	2020-10-08	8.39	8.39	8.31	8.34	66426	{}	\N	2020-10-12 14:38:30.242402
36296	DDF	2020-10-09	8.38	8.39	8.31	8.31	56190	{}	\N	2020-10-12 14:38:30.246521
36297	DDF	2020-10-12	8.32	8.38	8.32	8.37	9804	{}	\N	2020-10-12 14:38:30.25091
36298	CUBE	2020-09-28	32.4	32.76	32.185	32.55	739801	{}	\N	2020-10-12 14:38:31.161479
36299	CUBE	2020-09-29	32.45	32.69	32.065	32.32	1348102	{}	\N	2020-10-12 14:38:31.170358
36300	CUBE	2020-09-30	32.29	32.43	31.96	32.31	2605990	{}	\N	2020-10-12 14:38:31.176073
36301	CUBE	2020-10-01	32.33	33.13	32.3	32.96	3608311	{}	\N	2020-10-12 14:38:31.18097
36302	CUBE	2020-10-02	32.69	33.83	32.51	33.75	3341424	{}	\N	2020-10-12 14:38:31.185837
36303	CUBE	2020-10-05	33.99	34	33.07	34	2655243	{}	\N	2020-10-12 14:38:31.190745
36304	CUBE	2020-10-06	33.53	34.13	33.53	33.98	3573051	{}	\N	2020-10-12 14:38:31.194895
36305	CUBE	2020-10-07	34.15	34.26	33.77	34.06	2225329	{}	\N	2020-10-12 14:38:31.198949
36306	CUBE	2020-10-08	34.22	34.49	34.07	34.35	2212942	{}	\N	2020-10-12 14:38:31.203336
36307	CUBE	2020-10-09	34.66	34.66	34.155	34.33	1653238	{}	\N	2020-10-12 14:38:31.207488
36308	CUBE	2020-10-12	34.25	34.64	34.03	34.62	234784	{}	\N	2020-10-12 14:38:31.211462
36309	CUTR	2020-09-28	18.66	19.1099	18.245	18.33	123797	{}	\N	2020-10-12 14:38:32.368254
36310	CUTR	2020-09-29	18.32	18.83	17.8104	18.43	187080	{}	\N	2020-10-12 14:38:32.377092
36311	CUTR	2020-09-30	19.53	19.715	18.81	18.97	153101	{}	\N	2020-10-12 14:38:32.383184
36312	CUTR	2020-10-01	18.94	19.09	18.22	18.3	115816	{}	\N	2020-10-12 14:38:32.388629
36313	CUTR	2020-10-02	17.8	18.455	17.8	18.31	77199	{}	\N	2020-10-12 14:38:32.393483
36314	CUTR	2020-10-05	18.68	19.22	18.3475	18.63	99067	{}	\N	2020-10-12 14:38:32.398484
36315	CUTR	2020-10-06	18.85	19.4	18.56	18.6	142622	{}	\N	2020-10-12 14:38:32.402739
36316	CUTR	2020-10-07	18.925	19.15	18.61	18.79	109890	{}	\N	2020-10-12 14:38:32.406728
36317	CUTR	2020-10-08	19.17	20.94	18.92	20.61	170538	{}	\N	2020-10-12 14:38:32.410875
36318	CUTR	2020-10-09	20.66	21.22	19.81	19.92	93734	{}	\N	2020-10-12 14:38:32.415144
36319	CUTR	2020-10-12	20.03	20.61	19.97	20.36	19959	{}	\N	2020-10-12 14:38:32.41955
36320	CW	2020-09-28	95.1	97.05	94.85	95.2	164153	{}	\N	2020-10-12 14:38:33.358396
36321	CW	2020-09-29	95.26	95.61	93.7	94.11	282183	{}	\N	2020-10-12 14:38:33.366591
36322	CW	2020-09-30	94.04	95.62	92.8	93.26	404886	{}	\N	2020-10-12 14:38:33.372962
36323	CW	2020-10-01	94.14	95.88	93.45	93.93	396221	{}	\N	2020-10-12 14:38:33.378519
36324	CW	2020-10-02	92.19	96.07	92.19	95.48	374292	{}	\N	2020-10-12 14:38:33.383149
36325	CW	2020-10-05	96.82	97.43	94.12	94.84	556812	{}	\N	2020-10-12 14:38:33.387268
36326	CW	2020-10-06	96	97.04	94.16	94.32	433340	{}	\N	2020-10-12 14:38:33.391667
36327	CW	2020-10-07	95.51	96.48	94.43	95.47	454325	{}	\N	2020-10-12 14:38:33.395822
36328	CW	2020-10-08	96.12	97.91	95.75	97.91	377712	{}	\N	2020-10-12 14:38:33.400264
36329	CW	2020-10-09	99.45	99.95	97.72	97.83	406527	{}	\N	2020-10-12 14:38:33.404773
36330	CW	2020-10-12	97.83	98.495	97.385	98.09	45364	{}	\N	2020-10-12 14:38:33.408668
36331	CWEB	2020-09-28	47.3	47.543	46.38	47.47	10650	{}	\N	2020-10-12 14:38:34.349607
36332	CWEB	2020-09-29	46.9338	47.23	46.641	46.795	25949	{}	\N	2020-10-12 14:38:34.361198
36333	CWEB	2020-09-30	47.85	49.36	47.53	48.96	42731	{}	\N	2020-10-12 14:38:34.368123
36334	CWEB	2020-10-01	49.45	50.28	48.92	49.9	22877	{}	\N	2020-10-12 14:38:34.374158
36335	CWEB	2020-10-02	47.74	49.36	47.215	48.04	15983	{}	\N	2020-10-12 14:38:34.379402
36336	CWEB	2020-10-05	48.79	48.9697	48.155	48.95	30567	{}	\N	2020-10-12 14:38:34.384486
36337	CWEB	2020-10-06	49.72	51.85	49.72	50.69	137672	{}	\N	2020-10-12 14:38:34.38907
36338	CWEB	2020-10-07	51.78	52.4069	51.73	52.05	19660	{}	\N	2020-10-12 14:38:34.393599
36339	CWEB	2020-10-08	52.57	52.62	51.8299	52.27	18305	{}	\N	2020-10-12 14:38:34.398126
36340	CWEB	2020-10-09	52.7166	54.5	52.7166	54.5	44036	{}	\N	2020-10-12 14:38:34.403
36341	CWEB	2020-10-12	55.94	56.4	55.7466	56.4	16902	{}	\N	2020-10-12 14:38:34.407384
36342	CXE	2020-09-28	4.7	4.74	4.7	4.72	50846	{}	\N	2020-10-12 14:38:36.670946
36343	CXE	2020-09-29	4.73	4.74	4.72	4.73	54379	{}	\N	2020-10-12 14:38:36.678439
36344	CXE	2020-09-30	4.75	4.75	4.72	4.72	144138	{}	\N	2020-10-12 14:38:36.683457
36345	CXE	2020-10-01	4.73	4.8	4.72	4.79	160420	{}	\N	2020-10-12 14:38:36.688121
36346	CXE	2020-10-02	4.79	4.8	4.76	4.785	59954	{}	\N	2020-10-12 14:38:36.692854
36347	CXE	2020-10-05	4.79	4.819	4.75	4.78	116794	{}	\N	2020-10-12 14:38:36.697489
36348	CXE	2020-10-06	4.76	4.78	4.75	4.76	81295	{}	\N	2020-10-12 14:38:36.702076
36349	CXE	2020-10-07	4.76	4.82	4.75	4.75	111894	{}	\N	2020-10-12 14:38:36.706305
36350	CXE	2020-10-08	4.76	4.79	4.75	4.78	74876	{}	\N	2020-10-12 14:38:36.710441
36351	CXE	2020-10-09	4.77	4.81	4.76	4.76	91926	{}	\N	2020-10-12 14:38:36.714192
36352	CXE	2020-10-12	4.83	4.83	4.785	4.785	34996	{}	\N	2020-10-12 14:38:36.71851
36353	CYAN	2020-09-28	2.5747	2.5747	2.5747	2.5747	1408	{}	\N	2020-10-12 14:38:37.621036
36354	CYAN	2020-09-29	2.6	2.7	2.6	2.6	879	{}	\N	2020-10-12 14:38:37.628385
36355	CYAN	2020-09-30	2.5825	2.5825	2.43	2.45	1873	{}	\N	2020-10-12 14:38:37.634036
36356	CYAN	2020-10-01	2.43	2.6899	2.43	2.575	4758	{}	\N	2020-10-12 14:38:37.639001
36357	CYAN	2020-10-02	2.4	2.5172	2.37	2.5172	2491	{}	\N	2020-10-12 14:38:37.643519
36358	CYAN	2020-10-05	2.46	2.46	2.46	2.46	340	{}	\N	2020-10-12 14:38:37.647581
36359	CYAN	2020-10-06	2.5623	2.66	2.56	2.56	1884	{}	\N	2020-10-12 14:38:37.651868
36360	CYAN	2020-10-07	2.595	2.595	2.5	2.5	681	{}	\N	2020-10-12 14:38:37.6558
36361	CYAN	2020-10-08	2.67	2.6764	2.5227	2.55	4113	{}	\N	2020-10-12 14:38:37.660254
36362	CYAN	2020-10-09	2.55	2.59	2.5485	2.56	6425	{}	\N	2020-10-12 14:38:37.66433
36363	CYAN	2020-10-12	2.5601	2.5621	2.56	2.56	1288	{}	\N	2020-10-12 14:38:37.670107
36364	CYH	2020-09-28	4.18	4.32	4.17	4.24	1340388	{}	\N	2020-10-12 14:38:39.10979
36365	CYH	2020-09-29	4.25	4.25	4.01	4.14	1434247	{}	\N	2020-10-12 14:38:39.122787
36366	CYH	2020-09-30	4.22	4.41	4.2	4.22	2714137	{}	\N	2020-10-12 14:38:39.129832
36367	CYH	2020-10-01	4.26	4.35	4.17	4.32	1999012	{}	\N	2020-10-12 14:38:39.136262
36368	CYH	2020-10-02	4.2	4.48	4.14	4.44	2141360	{}	\N	2020-10-12 14:38:39.141301
36369	CYH	2020-10-05	4.48	4.91	4.46	4.85	2437018	{}	\N	2020-10-12 14:38:39.146202
36370	CYH	2020-10-06	4.95	5.115	4.76	4.76	3800491	{}	\N	2020-10-12 14:38:39.1505
36371	CYH	2020-10-07	4.85	4.9399	4.62	4.64	2575125	{}	\N	2020-10-12 14:38:39.154811
36372	CYH	2020-10-08	4.8	4.86	4.63	4.64	2531380	{}	\N	2020-10-12 14:38:39.159315
36373	CYH	2020-10-09	4.78	5.06	4.67	4.89	2980665	{}	\N	2020-10-12 14:38:39.163908
36374	CYH	2020-10-12	4.92	4.93	4.79	4.85	346216	{}	\N	2020-10-12 14:38:39.168843
36375	CZR	2020-09-28	58.42	59.73	55.23	58.21	6560028	{}	\N	2020-10-12 14:38:40.126605
36376	CZR	2020-09-29	56.3	57.4	53.535	54.49	23675184	{}	\N	2020-10-12 14:38:40.136074
36377	CZR	2020-09-30	55.9	57.97	54.63	56.06	21946903	{}	\N	2020-10-12 14:38:40.141512
36378	CZR	2020-10-01	56.44	57.55	55.07	57.1	7128813	{}	\N	2020-10-12 14:38:40.146434
36379	CZR	2020-10-02	54.62	58	54.42	56.83	8653280	{}	\N	2020-10-12 14:38:40.151808
36380	CZR	2020-10-05	57.71	58	55.15	57.07	4133018	{}	\N	2020-10-12 14:38:40.156596
36381	CZR	2020-10-06	57.4	57.7	54.88	55	5486276	{}	\N	2020-10-12 14:38:40.160695
36382	CZR	2020-10-07	55.69	56.48	54.6	55.74	4166828	{}	\N	2020-10-12 14:38:40.164681
36383	CZR	2020-10-08	56.55	56.97	54.625	54.74	3020747	{}	\N	2020-10-12 14:38:40.169267
36384	CZR	2020-10-09	55.02	55.73	52.01	52.35	4401145	{}	\N	2020-10-12 14:38:40.173603
36385	CZR	2020-10-12	53.06	53.93	52.35	53.33	675564	{}	\N	2020-10-12 14:38:40.177471
36386	DALT	2020-09-28	8.2103	8.4511	8.2103	8.41	16638	{}	\N	2020-10-12 14:38:41.780149
36387	DALT	2020-09-29	8.02	8.51	7.8801	8.46	49796	{}	\N	2020-10-12 14:38:41.789319
36388	DALT	2020-09-30	8.5174	8.57	8.5101	8.5101	15851	{}	\N	2020-10-12 14:38:41.795226
36389	DALT	2020-10-01	8.5799	8.5962	8.56	8.59	5356	{}	\N	2020-10-12 14:38:41.800809
36390	DALT	2020-10-02	8.5399	8.57	8.5295	8.545	13075	{}	\N	2020-10-12 14:38:41.806047
36391	DALT	2020-10-05	8.6119	8.65	8.6119	8.6475	47693	{}	\N	2020-10-12 14:38:41.810568
36392	DALT	2020-10-06	8.59	8.72	8.59	8.65	43171	{}	\N	2020-10-12 14:38:41.814385
36393	DALT	2020-10-07	8.69	8.725	8.68	8.7	32503	{}	\N	2020-10-12 14:38:41.818599
36394	DALT	2020-10-08	8.7899	8.79	8.76	8.76	4755	{}	\N	2020-10-12 14:38:41.822946
36395	DALT	2020-10-09	8.84	8.8498	8.84	8.8498	987	{}	\N	2020-10-12 14:38:41.827603
36396	DALT	2020-10-12	8.8601	8.88	8.86	8.88	7899	{}	\N	2020-10-12 14:38:41.83241
36397	DBEU	2020-09-28	27.69	27.73	27.63	27.67	61963	{}	\N	2020-10-12 14:38:43.492179
36398	DBEU	2020-09-29	27.62	27.69	27.5	27.54	25311	{}	\N	2020-10-12 14:38:43.500148
36399	DBEU	2020-09-30	27.58	27.6807	27.37	27.44	56140	{}	\N	2020-10-12 14:38:43.506821
36400	DBEU	2020-10-01	27.58	27.65	27.45	27.56	249684	{}	\N	2020-10-12 14:38:43.511938
36401	DBEU	2020-10-02	27.28	27.66	27.28	27.62	41179	{}	\N	2020-10-12 14:38:43.517005
36402	DBEU	2020-10-05	27.76	27.97	27.75	27.97	88252	{}	\N	2020-10-12 14:38:43.521687
36403	DBEU	2020-10-06	27.92	27.9496	27.6217	27.65	35487	{}	\N	2020-10-12 14:38:43.525716
36404	DBEU	2020-10-07	27.8	27.881	27.7664	27.86	24372	{}	\N	2020-10-12 14:38:43.529931
36405	DBEU	2020-10-08	28.07	28.14	27.95	28.14	178554	{}	\N	2020-10-12 14:38:43.534405
36406	DBEU	2020-10-09	28.2	28.26	28.13	28.18	70935	{}	\N	2020-10-12 14:38:43.539178
36407	DBEU	2020-10-12	28.28	28.33	28.28	28.3273	3043	{}	\N	2020-10-12 14:38:43.543928
36408	CVU	2020-09-28	2.8	2.87	2.76	2.85	59745	{}	\N	2020-10-12 14:38:45.176467
36409	CVU	2020-09-29	2.86	2.8878	2.63	2.68	40330	{}	\N	2020-10-12 14:38:45.186606
36410	CVU	2020-09-30	2.72	2.84	2.5345	2.68	67556	{}	\N	2020-10-12 14:38:45.192852
36411	CVU	2020-10-01	2.6096	2.62	2.485	2.56	182589	{}	\N	2020-10-12 14:38:45.198248
36412	CVU	2020-10-02	2.53	2.53	2.18	2.26	309717	{}	\N	2020-10-12 14:38:45.203544
36413	CVU	2020-10-05	2.27	2.3499	2.25	2.27	103616	{}	\N	2020-10-12 14:38:45.208214
36414	CVU	2020-10-06	2.28	2.37	2.25	2.3	70382	{}	\N	2020-10-12 14:38:45.212404
36415	CVU	2020-10-07	2.31	2.5	2.31	2.43	45086	{}	\N	2020-10-12 14:38:45.216728
36416	CVU	2020-10-08	2.38	2.53	2.38	2.46	44563	{}	\N	2020-10-12 14:38:45.221005
36417	CVU	2020-10-09	2.5	2.5	2.43	2.45	33670	{}	\N	2020-10-12 14:38:45.225288
36418	CVU	2020-10-12	2.44	2.44	2.44	2.44	989	{}	\N	2020-10-12 14:38:45.229608
36419	CXP	2020-09-28	10.98	11.3	10.98	11.16	590866	{}	\N	2020-10-12 14:38:46.728728
36420	CXP	2020-09-29	11.14	11.16	10.775	10.91	802312	{}	\N	2020-10-12 14:38:46.741904
36421	CXP	2020-09-30	11.01	11.355	10.785	10.91	2064421	{}	\N	2020-10-12 14:38:46.748506
36422	CXP	2020-10-01	10.91	11.24	10.88	11.2	971084	{}	\N	2020-10-12 14:38:46.756429
36423	CXP	2020-10-02	10.92	11.42	10.92	11.33	1326452	{}	\N	2020-10-12 14:38:46.761733
36424	CXP	2020-10-05	11.41	11.5	11.14	11.3	1609315	{}	\N	2020-10-12 14:38:46.767001
36425	CXP	2020-10-06	11.45	11.65	11.28	11.3	3683947	{}	\N	2020-10-12 14:38:46.771328
36426	CXP	2020-10-07	11.38	11.56	11.29	11.37	1191650	{}	\N	2020-10-12 14:38:46.775773
36427	CXP	2020-10-08	11.47	11.75	11.42	11.73	706765	{}	\N	2020-10-12 14:38:46.779844
36428	CXP	2020-10-09	11.89	11.94	11.68	11.73	1221010	{}	\N	2020-10-12 14:38:46.784492
36429	CXP	2020-10-12	11.63	11.91	11.57	11.88	107192	{}	\N	2020-10-12 14:38:46.788877
36430	CYRX	2020-09-28	48.19	49.75	47.35	49.55	757149	{}	\N	2020-10-12 14:38:48.105931
36431	CYRX	2020-09-29	49.46	49.835	48.5	48.97	517259	{}	\N	2020-10-12 14:38:48.115416
36432	CYRX	2020-09-30	48.99	49.99	47.0624	47.4	818139	{}	\N	2020-10-12 14:38:48.121972
36433	CYRX	2020-10-01	48	48.5	43.21	43.93	832237	{}	\N	2020-10-12 14:38:48.127117
36434	CYRX	2020-10-02	42.78	46.3525	41.83	45.2	1590770	{}	\N	2020-10-12 14:38:48.131992
36435	CYRX	2020-10-05	45.51	49.35	45.51	48.59	951978	{}	\N	2020-10-12 14:38:48.136663
36436	CYRX	2020-10-06	48.4	51.12	47.62	49.56	1142215	{}	\N	2020-10-12 14:38:48.140776
36437	CYRX	2020-10-07	50.01	52.35	49.9525	51.37	757686	{}	\N	2020-10-12 14:38:48.144748
36438	CYRX	2020-10-08	51.61	52.14	48.38	48.9	686575	{}	\N	2020-10-12 14:38:48.14866
36439	CYRX	2020-10-09	49.53	50.1	48.5035	49.06	377517	{}	\N	2020-10-12 14:38:48.153595
36440	CYRX	2020-10-12	49.5	49.85	48.77	48.93	98723	{}	\N	2020-10-12 14:38:48.157957
36441	DAX	2020-09-28	28.14	28.23	28.0196	28.23	10876	{}	\N	2020-10-12 14:38:49.993091
36442	DAX	2020-09-29	28.22	28.3006	28.06	28.3006	6140	{}	\N	2020-10-12 14:38:49.999526
36443	DAX	2020-09-30	28.07	28.2184	28.07	28.11	1690	{}	\N	2020-10-12 14:38:50.004686
36444	DAX	2020-10-01	28.145	28.145	27.91	28.06	5116	{}	\N	2020-10-12 14:38:50.010484
36445	DAX	2020-10-02	27.46	27.98	27.46	27.87	3989	{}	\N	2020-10-12 14:38:50.015144
36446	DAX	2020-10-05	28.3308	28.5471	28.3185	28.38	2743	{}	\N	2020-10-12 14:38:50.01979
36447	DAX	2020-10-06	28.69	28.69	28.225	28.225	1125	{}	\N	2020-10-12 14:38:50.024013
36448	DAX	2020-10-07	28.4204	28.61	28.355	28.61	4217	{}	\N	2020-10-12 14:38:50.028224
36449	DAX	2020-10-08	28.8	28.96	28.69	28.7759	3252	{}	\N	2020-10-12 14:38:50.032398
36450	DAX	2020-10-09	28.865	28.9563	28.776	28.925	8030	{}	\N	2020-10-12 14:38:50.036691
36451	DAX	2020-10-12	28.89	28.9412	28.89	28.9412	250	{}	\N	2020-10-12 14:38:50.040776
36452	DBB	2020-09-28	15.25	15.26	15.16	15.25	30506	{}	\N	2020-10-12 14:38:51.045451
36453	DBB	2020-09-29	15.18	15.23	15.155	15.23	50414	{}	\N	2020-10-12 14:38:51.061157
36454	DBB	2020-09-30	15.24	15.305	15.14	15.14	79155	{}	\N	2020-10-12 14:38:51.068172
36455	DBB	2020-10-01	14.99	14.99	14.61	14.7	194465	{}	\N	2020-10-12 14:38:51.074665
36456	DBB	2020-10-02	14.83	15	14.83	14.88	378283	{}	\N	2020-10-12 14:38:51.080112
36457	DBB	2020-10-05	14.9	15.02	14.8899	14.96	76842	{}	\N	2020-10-12 14:38:51.085791
36458	DBB	2020-10-06	15.035	15.07	14.86	14.88	19846	{}	\N	2020-10-12 14:38:51.090614
36459	DBB	2020-10-07	15.13	15.2358	15.11	15.16	31754	{}	\N	2020-10-12 14:38:51.095193
36460	DBB	2020-10-08	15.24	15.24	15.162	15.18	11495	{}	\N	2020-10-12 14:38:51.099792
36461	DBB	2020-10-09	15.42	15.52	15.39	15.48	127953	{}	\N	2020-10-12 14:38:51.104484
36462	DBB	2020-10-12	15.58	15.58	15.535	15.5441	3729	{}	\N	2020-10-12 14:38:51.109106
36463	DBJP	2020-09-28	40.91	41.063	40.91	41.02	5187	{}	\N	2020-10-12 14:38:52.488636
36464	DBJP	2020-09-29	41.19	41.2	41.02	41.08	49723	{}	\N	2020-10-12 14:38:52.499924
36465	DBJP	2020-09-30	40.64	40.85	40.62	40.72	75942	{}	\N	2020-10-12 14:38:52.506198
36466	DBJP	2020-10-01	40.81	40.81	40.65	40.8	159830	{}	\N	2020-10-12 14:38:52.511834
36467	DBJP	2020-10-02	40.12	40.52	40.12	40.52	14872	{}	\N	2020-10-12 14:38:52.517125
36468	DBJP	2020-10-05	40.805	40.94	40.77	40.91	122071	{}	\N	2020-10-12 14:38:52.522212
36469	DBJP	2020-10-06	40.93	41.04	40.7	40.86	55304	{}	\N	2020-10-12 14:38:52.526682
36470	DBJP	2020-10-07	41.13	41.21	41.0607	41.16	4812	{}	\N	2020-10-12 14:38:52.530877
36471	DBJP	2020-10-08	41.27	41.325	41.15	41.16	16723	{}	\N	2020-10-12 14:38:52.53551
36472	DBJP	2020-10-09	41.09	41.14	41	41	6047	{}	\N	2020-10-12 14:38:52.539872
36473	DBJP	2020-10-12	40.92	41.08	40.92	41.08	4779	{}	\N	2020-10-12 14:38:52.543983
36474	DCHF	2020-09-28	21.88	21.88	21.8673	21.8673	113	{}	\N	2020-10-12 14:38:53.692285
36475	DCHF	2020-09-29	21.3954	21.3954	21.3954	21.3954	40	{}	\N	2020-10-12 14:38:53.700167
36476	DCHF	2020-09-30	21.5257	21.5257	21.5257	21.5257	11	{}	\N	2020-10-12 14:38:53.706047
36477	DCHF	2020-10-01	21.3898	21.3898	21.3898	21.3898	125	{}	\N	2020-10-12 14:38:53.711406
36478	DCHF	2020-10-02	21.5196	21.5196	21.5196	21.5196	47	{}	\N	2020-10-12 14:38:53.716211
36479	DCHF	2020-10-05	21.0085	21.0085	21.0085	21.0085	4	{}	\N	2020-10-12 14:38:53.72065
36480	DCHF	2020-10-06	21.1255	21.1255	21.1255	21.1255	0	{}	\N	2020-10-12 14:38:53.725305
36481	DCHF	2020-10-07	21.1819	21.1819	21.1819	21.1819	2	{}	\N	2020-10-12 14:38:53.729578
36482	DCHF	2020-10-08	21.1668	21.1668	21.1668	21.1668	0	{}	\N	2020-10-12 14:38:53.734211
36483	DCHF	2020-10-09	20.7	20.75	20.5026	20.5026	5637	{}	\N	2020-10-12 14:38:53.739173
36484	DDD	2020-09-28	4.74	4.955	4.74	4.79	1960130	{}	\N	2020-10-12 14:38:54.919784
36485	DDD	2020-09-29	4.84	5.0699	4.77	4.92	4729294	{}	\N	2020-10-12 14:38:54.929949
36486	DDD	2020-09-30	4.93	5.125	4.86	4.91	4019548	{}	\N	2020-10-12 14:38:54.936338
36487	DDD	2020-10-01	4.94	4.9792	4.83	4.88	3066238	{}	\N	2020-10-12 14:38:54.941987
36488	DDD	2020-10-02	4.75	4.8986	4.6716	4.87	3355030	{}	\N	2020-10-12 14:38:54.947212
36489	DDD	2020-10-05	4.91	5.03	4.9	4.99	2480240	{}	\N	2020-10-12 14:38:54.952698
36490	DDD	2020-10-06	5.06	5.62	5.05	5.37	10918793	{}	\N	2020-10-12 14:38:54.957557
36491	DDD	2020-10-07	5.45	6.49	5.45	6.36	48398717	{}	\N	2020-10-12 14:38:54.962151
36492	DDD	2020-10-08	6.4	6.52	6.17	6.39	17125678	{}	\N	2020-10-12 14:38:54.966528
36493	DDD	2020-10-09	6.7	6.98	6.52	6.69	12801623	{}	\N	2020-10-12 14:38:54.970941
36494	DDD	2020-10-12	6.8	6.92	6.6573	6.695	2914890	{}	\N	2020-10-12 14:38:54.975291
36495	DEFA	2020-09-28	26.809	26.809	26.7844	26.7844	623	{}	\N	2020-10-12 14:38:56.646882
36496	DEFA	2020-09-29	26.81	26.81	26.7597	26.7597	427	{}	\N	2020-10-12 14:38:56.653761
36497	DEFA	2020-09-30	26.705	26.705	26.65	26.65	788	{}	\N	2020-10-12 14:38:56.659104
36498	DEFA	2020-10-01	26.7599	26.7657	26.7599	26.7657	178	{}	\N	2020-10-12 14:38:56.664078
36499	DEFA	2020-10-02	26.69	26.78	26.69	26.7747	1095	{}	\N	2020-10-12 14:38:56.668724
36500	DEFA	2020-10-05	26.97	27.1078	26.97	27.1078	872	{}	\N	2020-10-12 14:38:56.673034
36501	DEFA	2020-10-06	27.09	27.09	26.8579	26.8579	431	{}	\N	2020-10-12 14:38:56.677066
36502	DEFA	2020-10-07	27.03	27.0975	27.03	27.0975	642	{}	\N	2020-10-12 14:38:56.681303
36503	DEFA	2020-10-08	27.2634	27.2634	27.2634	27.2634	312	{}	\N	2020-10-12 14:38:56.685747
36504	DEFA	2020-10-09	27.44	27.4428	27.41	27.4428	653	{}	\N	2020-10-12 14:38:56.690355
36505	DESP	2020-09-28	6.5	6.69	6.34	6.43	283265	{}	\N	2020-10-12 14:38:57.581745
36506	DESP	2020-09-29	6.42	6.42	6.08	6.18	508158	{}	\N	2020-10-12 14:38:57.593134
36507	DESP	2020-09-30	6.14	6.4	6.14	6.36	1287190	{}	\N	2020-10-12 14:38:57.600103
36508	DESP	2020-10-01	6.39	6.8	6.39	6.78	933576	{}	\N	2020-10-12 14:38:57.605951
36509	DESP	2020-10-02	6.61	7.01	6.51	6.87	483010	{}	\N	2020-10-12 14:38:57.611164
36510	DESP	2020-10-05	6.92	7.05	6.63	6.72	911329	{}	\N	2020-10-12 14:38:57.616468
36511	DESP	2020-10-06	6.72	7.37	6.69	7.03	2137530	{}	\N	2020-10-12 14:38:57.621326
36512	DESP	2020-10-07	7.06	7.14	6.97	7.01	648188	{}	\N	2020-10-12 14:38:57.626011
36513	DESP	2020-10-08	7.1	7.205	6.89	6.95	1173813	{}	\N	2020-10-12 14:38:57.630673
36514	DESP	2020-10-09	7	7	6.81	6.84	1341455	{}	\N	2020-10-12 14:38:57.635251
36515	DESP	2020-10-12	6.89	6.97	6.86	6.9	106204	{}	\N	2020-10-12 14:38:57.639671
36516	DAR	2020-09-28	34.02	34.68	33.83	33.87	839195	{}	\N	2020-10-12 14:39:00.328384
36517	DAR	2020-09-29	34.5	35.57	34.12	35.43	4292122	{}	\N	2020-10-12 14:39:00.339284
36518	DAR	2020-09-30	35.39	36.58	35.215	36.03	3558473	{}	\N	2020-10-12 14:39:00.347159
36519	DAR	2020-10-01	36.41	37.47	36.09	37.33	3142968	{}	\N	2020-10-12 14:39:00.352611
36520	DAR	2020-10-02	36.55	37.31	36.19	36.82	2641481	{}	\N	2020-10-12 14:39:00.357499
36521	DAR	2020-10-05	37.03	37.95	36.92	37.94	1664540	{}	\N	2020-10-12 14:39:00.362697
36522	DAR	2020-10-06	38.22	39.24	37.9601	38.31	2291913	{}	\N	2020-10-12 14:39:00.369507
36523	DAR	2020-10-07	38.95	39.81	38.88	39.71	3527474	{}	\N	2020-10-12 14:39:00.376255
36524	DAR	2020-10-08	40.22	40.61	39.61	40.54	3363863	{}	\N	2020-10-12 14:39:00.380657
36525	DAR	2020-10-09	41.44	42.37	40.97	42.35	3778774	{}	\N	2020-10-12 14:39:00.385454
36526	DAR	2020-10-12	42.29	42.76	41.65	42.575	327768	{}	\N	2020-10-12 14:39:00.390213
36527	DBEF	2020-09-28	30.8	30.89	30.77	30.86	644087	{}	\N	2020-10-12 14:39:01.797175
36528	DBEF	2020-09-29	30.8	30.865	30.7	30.74	392882	{}	\N	2020-10-12 14:39:01.808151
36529	DBEF	2020-09-30	30.65	30.77	30.4901	30.6	766205	{}	\N	2020-10-12 14:39:01.815128
36530	DBEF	2020-10-01	30.69	30.74	30.59	30.69	482297	{}	\N	2020-10-12 14:39:01.821022
36531	DBEF	2020-10-02	30.36	30.72	30.33	30.67	308761	{}	\N	2020-10-12 14:39:01.82601
36532	DBEF	2020-10-05	30.84	31.04	30.84	31.04	510976	{}	\N	2020-10-12 14:39:01.830947
36533	DBEF	2020-10-06	31.06	31.105	30.76	30.78	471941	{}	\N	2020-10-12 14:39:01.835629
36534	DBEF	2020-10-07	31	31.11	30.99	31.09	330967	{}	\N	2020-10-12 14:39:01.839859
36535	DBEF	2020-10-08	31.21	31.3	31.19	31.27	404129	{}	\N	2020-10-12 14:39:01.84412
36536	DBEF	2020-10-09	31.33	31.36	31.24	31.28	1043554	{}	\N	2020-10-12 14:39:01.849087
36537	DBEF	2020-10-12	31.36	31.4	31.34	31.38	364877	{}	\N	2020-10-12 14:39:01.853689
36538	DCI	2020-09-28	46.59	47.15	46.555	46.68	301280	{}	\N	2020-10-12 14:39:03.114806
36539	DCI	2020-09-29	46.74	47.18	46.3448	46.38	474769	{}	\N	2020-10-12 14:39:03.121577
36540	DCI	2020-09-30	46.6	47.23	46.12	46.42	743531	{}	\N	2020-10-12 14:39:03.12739
36541	DCI	2020-10-01	46.71	47.48	46.71	46.93	879664	{}	\N	2020-10-12 14:39:03.13281
36542	DCI	2020-10-02	47.35	48.57	47.157	47.93	818292	{}	\N	2020-10-12 14:39:03.137868
36543	DCI	2020-10-05	48.62	50.62	48.62	50.46	1187664	{}	\N	2020-10-12 14:39:03.14314
36544	DCI	2020-10-06	50.35	51.2545	50.01	50.03	1003054	{}	\N	2020-10-12 14:39:03.147683
36545	DCI	2020-10-07	50.47	50.98	50.19	50.59	612288	{}	\N	2020-10-12 14:39:03.152344
36546	DCI	2020-10-08	51.04	51.1	50.24	50.82	809430	{}	\N	2020-10-12 14:39:03.157589
36547	DCI	2020-10-09	51.09	51.4	50.7	51.01	1617611	{}	\N	2020-10-12 14:39:03.162989
36548	DCI	2020-10-12	51.49	51.71	51.23	51.3694	63642	{}	\N	2020-10-12 14:39:03.167462
36549	DDS	2020-09-28	31.01	32.39	30.8997	31.29	327923	{}	\N	2020-10-12 14:39:04.091681
36550	DDS	2020-09-29	31.5	34.24	30.17	33.51	1504438	{}	\N	2020-10-12 14:39:04.101753
36551	DDS	2020-09-30	33.72	36.935	33.72	36.52	1670868	{}	\N	2020-10-12 14:39:04.107174
36552	DDS	2020-10-01	36.52	37.5	34.5223	36.96	983427	{}	\N	2020-10-12 14:39:04.110997
36553	DDS	2020-10-02	35.46	39.005	35	37.94	1481730	{}	\N	2020-10-12 14:39:04.114497
36554	DDS	2020-10-05	38.68	39.99	36.42	39.14	1657688	{}	\N	2020-10-12 14:39:04.118929
36555	DDS	2020-10-06	39.56	42	39.19	40.08	1444879	{}	\N	2020-10-12 14:39:04.122821
36556	DDS	2020-10-07	40.5	41.62	39.48	41.12	1343099	{}	\N	2020-10-12 14:39:04.12643
36557	DDS	2020-10-08	40.85	46.45	40.85	41.88	1255087	{}	\N	2020-10-12 14:39:04.130241
36558	DDS	2020-10-09	42.3	45.63	41.355	42.08	1129487	{}	\N	2020-10-12 14:39:04.134428
36559	DDS	2020-10-12	50.25	61.5	49.92	55.945	2894427	{}	\N	2020-10-12 14:39:04.138218
36560	DFEN	2020-09-28	10.98	11.45	10.86	11.14	4015499	{}	\N	2020-10-12 14:39:06.116354
36561	DFEN	2020-09-29	11.02	11.12	10.64	10.73	2455403	{}	\N	2020-10-12 14:39:06.126166
36562	DFEN	2020-09-30	10.8	11.12	10.16	10.4	3930487	{}	\N	2020-10-12 14:39:06.132453
36563	DFEN	2020-10-01	10.68	11.055	10.325	10.57	3199864	{}	\N	2020-10-12 14:39:06.138232
36564	DFEN	2020-10-02	9.95	10.99	9.91	10.9	4390839	{}	\N	2020-10-12 14:39:06.143185
36565	DFEN	2020-10-05	10.99	11.39	10.985	11.17	2605351	{}	\N	2020-10-12 14:39:06.147862
36566	DFEN	2020-10-06	11.29	11.54	10.56	10.6	4507866	{}	\N	2020-10-12 14:39:06.15215
36567	DFEN	2020-10-07	10.98	11.36	10.98	11.24	2713915	{}	\N	2020-10-12 14:39:06.156158
36568	DFEN	2020-10-08	11.43	11.805	11.185	11.8	2822012	{}	\N	2020-10-12 14:39:06.159798
36569	DFEN	2020-10-09	11.96	12.0399	11.53	11.55	2799220	{}	\N	2020-10-12 14:39:06.163556
36570	DFEN	2020-10-12	11.7	11.78	11.4	11.73	717434	{}	\N	2020-10-12 14:39:06.168029
36571	DGRS	2020-09-28	30.56	31.0399	30.56	30.9075	9571	{}	\N	2020-10-12 14:39:07.081137
36572	DGRS	2020-09-29	30.8913	30.8913	30.38	30.779	4730	{}	\N	2020-10-12 14:39:07.091534
36573	DGRS	2020-09-30	30.85	31.2	30.685	30.7044	7338	{}	\N	2020-10-12 14:39:07.097533
36574	DGRS	2020-10-01	30.99	31.06	30.775	30.96	24408	{}	\N	2020-10-12 14:39:07.102882
36575	DGRS	2020-10-02	30.5	31.7499	30.5	31.7499	5158	{}	\N	2020-10-12 14:39:07.107191
36576	DGRS	2020-10-05	31.96	32.405	31.96	32.405	7935	{}	\N	2020-10-12 14:39:07.111024
36577	DGRS	2020-10-06	32.59	33.15	32.3978	32.3978	15439	{}	\N	2020-10-12 14:39:07.115295
36578	DGRS	2020-10-07	32.73	33.065	32.65	32.9012	4220	{}	\N	2020-10-12 14:39:07.119351
36579	DGRS	2020-10-08	33.19	33.349	33.0322	33.2891	7296	{}	\N	2020-10-12 14:39:07.123216
36580	DGRS	2020-10-09	33.52	33.6401	33.2816	33.41	19396	{}	\N	2020-10-12 14:39:07.127011
36581	DGRS	2020-10-12	33.39	33.82	33.39	33.82	6523	{}	\N	2020-10-12 14:39:07.131023
36582	DHF	2020-09-28	2.72	2.75	2.72	2.74	2872399	{}	\N	2020-10-12 14:39:08.682487
36583	DHF	2020-09-29	2.75	2.75	2.73	2.74	2482524	{}	\N	2020-10-12 14:39:08.692663
36584	DHF	2020-09-30	2.75	2.76	2.71	2.71	5119580	{}	\N	2020-10-12 14:39:08.699623
36585	DHF	2020-10-01	2.72	2.74	2.715	2.72	1697338	{}	\N	2020-10-12 14:39:08.705397
36586	DHF	2020-10-02	2.72	2.74	2.7	2.74	2963272	{}	\N	2020-10-12 14:39:08.710026
36587	DHF	2020-10-05	2.75	2.77	2.74	2.75	3860878	{}	\N	2020-10-12 14:39:08.715716
36588	DHF	2020-10-06	2.75	2.78	2.74	2.75	4746604	{}	\N	2020-10-12 14:39:08.720479
36589	DHF	2020-10-07	2.77	2.8	2.76	2.78	2319130	{}	\N	2020-10-12 14:39:08.725359
36590	DHF	2020-10-08	2.78	2.81	2.76	2.77	6500198	{}	\N	2020-10-12 14:39:08.729307
36591	DHF	2020-10-09	2.78	2.8	2.76	2.76	7799694	{}	\N	2020-10-12 14:39:08.733593
36592	DHF	2020-10-12	2.77	2.79	2.77	2.77	1337062	{}	\N	2020-10-12 14:39:08.738121
36593	DHX	2020-09-28	2.25	2.3	2.25	2.28	49263	{}	\N	2020-10-12 14:39:10.470709
36594	DHX	2020-09-29	2.29	2.3002	2.24	2.26	148900	{}	\N	2020-10-12 14:39:10.479382
36595	DHX	2020-09-30	2.27	2.32	2.21	2.26	125597	{}	\N	2020-10-12 14:39:10.489075
36596	DHX	2020-10-01	2.24	2.41	2.235	2.39	178199	{}	\N	2020-10-12 14:39:10.497209
36597	DHX	2020-10-02	2.37	2.5	2.37	2.48	81832	{}	\N	2020-10-12 14:39:10.502919
36598	DHX	2020-10-05	2.48	2.56	2.48	2.54	84346	{}	\N	2020-10-12 14:39:10.508319
36599	DHX	2020-10-06	2.55	2.64	2.521	2.56	164780	{}	\N	2020-10-12 14:39:10.513994
36600	DHX	2020-10-07	2.56	2.65	2.53	2.55	145912	{}	\N	2020-10-12 14:39:10.519497
36601	DHX	2020-10-08	2.58	2.58	2.42	2.42	135831	{}	\N	2020-10-12 14:39:10.524677
36602	DHX	2020-10-09	2.47	2.47	2.4	2.4	66202	{}	\N	2020-10-12 14:39:10.530333
36603	DHX	2020-10-12	2.4	2.44	2.37	2.37	43866	{}	\N	2020-10-12 14:39:10.535657
36604	DIM	2020-09-28	56.117	56.16	56.0598	56.16	6716	{}	\N	2020-10-12 14:39:11.800238
36605	DIM	2020-09-29	56.02	56.0879	55.861	56.001	3844	{}	\N	2020-10-12 14:39:11.80951
36606	DIM	2020-09-30	55.82	56.17	55.69	55.71	4870	{}	\N	2020-10-12 14:39:11.816535
36607	DIM	2020-10-01	56.128	56.2599	55.98	56.22	3108	{}	\N	2020-10-12 14:39:11.822011
36608	DIM	2020-10-02	55.56	56.22	55.56	56.12	15623	{}	\N	2020-10-12 14:39:11.826324
36609	DIM	2020-10-05	56.83	57.0887	56.81	57.0887	10878	{}	\N	2020-10-12 14:39:11.830902
36610	DIM	2020-10-06	57.2013	57.24	56.652	56.652	1869	{}	\N	2020-10-12 14:39:11.835231
36611	DIM	2020-10-07	57.08	57.2233	57.08	57.2233	4234	{}	\N	2020-10-12 14:39:11.839058
36612	DIM	2020-10-08	57.36	57.4891	57.36	57.445	7950	{}	\N	2020-10-12 14:39:11.842717
36613	DIM	2020-10-09	57.56	57.7999	57.56	57.72	9126	{}	\N	2020-10-12 14:39:11.846302
36614	DIM	2020-10-12	57.79	57.815	57.78	57.815	2724	{}	\N	2020-10-12 14:39:11.850114
36615	DBD	2020-09-28	7.25	7.58	7.24	7.44	623481	{}	\N	2020-10-12 14:39:13.692527
36616	DBD	2020-09-29	7.45	7.635	7.28	7.62	1395771	{}	\N	2020-10-12 14:39:13.700309
36617	DBD	2020-09-30	7.59	7.84	7.57	7.64	1334909	{}	\N	2020-10-12 14:39:13.706388
36618	DBD	2020-10-01	7.67	7.845	7.6	7.7	985430	{}	\N	2020-10-12 14:39:13.710994
36619	DBD	2020-10-02	7.45	8.175	7.45	8.12	2098137	{}	\N	2020-10-12 14:39:13.716008
36620	DBD	2020-10-05	8.22	8.67	8.11	8.59	1429227	{}	\N	2020-10-12 14:39:13.720592
36621	DBD	2020-10-06	8.64	8.865	8.38	8.38	1648714	{}	\N	2020-10-12 14:39:13.724962
36622	DBD	2020-10-07	8.52	8.73	8.41	8.44	1275217	{}	\N	2020-10-12 14:39:13.729212
36623	DBD	2020-10-08	8.64	8.765	8.43	8.7	1126313	{}	\N	2020-10-12 14:39:13.733889
36624	DBD	2020-10-09	8.82	8.99	8.65	8.65	1288916	{}	\N	2020-10-12 14:39:13.73862
36625	DBD	2020-10-12	8.65	8.81	8.62	8.76	116072	{}	\N	2020-10-12 14:39:13.742898
36626	DBL	2020-09-28	18.98	19.08	18.89	18.975	27771	{}	\N	2020-10-12 14:39:14.7114
36627	DBL	2020-09-29	19.01	19.03	18.903	18.98	36526	{}	\N	2020-10-12 14:39:14.720936
36628	DBL	2020-09-30	19.08	19.27	19.03	19.06	89584	{}	\N	2020-10-12 14:39:14.727202
36629	DBL	2020-10-01	19.08	19.3	19.08	19.17	63776	{}	\N	2020-10-12 14:39:14.732873
36630	DBL	2020-10-02	19.125	19.29	19	19.16	101412	{}	\N	2020-10-12 14:39:14.737651
36631	DBL	2020-10-05	19.19	19.2721	19.09	19.17	60352	{}	\N	2020-10-12 14:39:14.74225
36632	DBL	2020-10-06	19.11	19.1385	19.055	19.12	43255	{}	\N	2020-10-12 14:39:14.746248
36633	DBL	2020-10-07	19.28	19.29	19.12	19.27	77072	{}	\N	2020-10-12 14:39:14.751002
36634	DBL	2020-10-08	19.21	19.27	19.1901	19.22	48384	{}	\N	2020-10-12 14:39:14.755206
36635	DBL	2020-10-09	19.15	19.29	19.15	19.25	25760	{}	\N	2020-10-12 14:39:14.760401
36636	DBL	2020-10-12	19.16	19.24	19.16	19.2194	12520	{}	\N	2020-10-12 14:39:14.766751
36637	DDWM	2020-09-28	25.5661	25.6836	25.56	25.63	19214	{}	\N	2020-10-12 14:39:16.824981
36638	DDWM	2020-09-29	25.6235	25.64	25.54	25.58	7509	{}	\N	2020-10-12 14:39:16.836988
36639	DDWM	2020-09-30	25.555	25.605	25.3969	25.43	9747	{}	\N	2020-10-12 14:39:16.84395
36640	DDWM	2020-10-01	25.5	25.535	25.41	25.49	18482	{}	\N	2020-10-12 14:39:16.850072
36641	DDWM	2020-10-02	25.4427	25.5747	25.39	25.51	11552	{}	\N	2020-10-12 14:39:16.855468
36642	DDWM	2020-10-05	25.76	25.84	25.71	25.84	30349	{}	\N	2020-10-12 14:39:16.860511
36643	DDWM	2020-10-06	25.8973	25.91	25.61	25.64	31187	{}	\N	2020-10-12 14:39:16.86512
36644	DDWM	2020-10-07	25.85	25.915	25.8332	25.86	8713	{}	\N	2020-10-12 14:39:16.869517
36645	DDWM	2020-10-08	25.95	26.0799	25.95	26.01	11713	{}	\N	2020-10-12 14:39:16.873998
36646	DDWM	2020-10-09	26.08	26.1295	26.031	26.105	65174	{}	\N	2020-10-12 14:39:16.878578
36647	DDWM	2020-10-12	26.14	26.1816	26.14	26.18	1617	{}	\N	2020-10-12 14:39:16.883635
36648	DFND	2020-09-28	35.81	36.09	35.81	36.04	10466	{}	\N	2020-10-12 14:39:19.151785
36649	DFND	2020-09-29	36.15	36.25	36.027	36.0686	4560	{}	\N	2020-10-12 14:39:19.161134
36650	DFND	2020-09-30	36.185	36.38	36.16	36.16	19211	{}	\N	2020-10-12 14:39:19.16734
36651	DFND	2020-10-01	36.534	36.61	36.45	36.53	14404	{}	\N	2020-10-12 14:39:19.172241
36652	DFND	2020-10-02	36.113	36.25	36.113	36.13	18228	{}	\N	2020-10-12 14:39:19.176605
36653	DFND	2020-10-05	36.49	36.49	36.105	36.46	11445	{}	\N	2020-10-12 14:39:19.181468
36654	DFND	2020-10-06	36.42	36.49	36.19	36.205	14794	{}	\N	2020-10-12 14:39:19.185879
36655	DFND	2020-10-07	36.445	36.49	36.425	36.47	7375	{}	\N	2020-10-12 14:39:19.190345
36656	DFND	2020-10-08	36.486	36.53	36.4	36.505	8200	{}	\N	2020-10-12 14:39:19.194324
36657	DFND	2020-10-09	36.82	36.88	36.686	36.82	41829	{}	\N	2020-10-12 14:39:19.1986
36658	DFND	2020-10-12	37.22	37.22	36.9075	36.925	6253	{}	\N	2020-10-12 14:39:19.203007
36659	DGX	2020-09-28	112.68	115.4	112.66	113.88	969179	{}	\N	2020-10-12 14:39:22.963847
36660	DGX	2020-09-29	114.01	114.73	112.65	113.65	1254654	{}	\N	2020-10-12 14:39:22.973273
36661	DGX	2020-09-30	114.32	115.35	113.56	114.49	1924831	{}	\N	2020-10-12 14:39:22.97874
36662	DGX	2020-10-01	115.09	115.64	112.38	112.7	1687673	{}	\N	2020-10-12 14:39:22.983995
36663	DGX	2020-10-02	111.53	115.15	109.5	113.87	1817454	{}	\N	2020-10-12 14:39:22.988845
36664	DGX	2020-10-05	114.96	116	114.14	115.39	1687266	{}	\N	2020-10-12 14:39:22.993326
36665	DGX	2020-10-06	115.76	117.62	114.8866	116.21	1577440	{}	\N	2020-10-12 14:39:22.997306
36666	DGX	2020-10-07	116.75	117.97	115.9	116.45	1369160	{}	\N	2020-10-12 14:39:23.001258
36667	DGX	2020-10-08	116.86	117.42	116.0641	116.28	1169659	{}	\N	2020-10-12 14:39:23.005406
36668	DGX	2020-10-09	116.81	117.6872	115.77	116.67	923078	{}	\N	2020-10-12 14:39:23.009454
36669	DGX	2020-10-12	117.23	117.67	116.54	117.52	200500	{}	\N	2020-10-12 14:39:23.013672
36670	DIG	2020-09-28	49.84	51.47	49.7726	50.54	126584	{}	\N	2020-10-12 14:39:25.323108
36671	DIG	2020-09-29	50.07	50.31	46.75	47.8	90063	{}	\N	2020-10-12 14:39:25.336615
36672	DIG	2020-09-30	48.4	49.38	47	47.48	121123	{}	\N	2020-10-12 14:39:25.343751
36673	DIG	2020-10-01	46.39	46.81	44.13	44.65	145237	{}	\N	2020-10-12 14:39:25.350888
36674	DIG	2020-10-02	42.01	46.1901	42.01	45.53	86984	{}	\N	2020-10-12 14:39:25.356155
36675	DIG	2020-10-05	46.49	48.34	46.2098	48.34	69373	{}	\N	2020-10-12 14:39:25.36073
36676	DIG	2020-10-06	49.74	50.2512	46.75	46.88	105049	{}	\N	2020-10-12 14:39:25.365378
36677	DIG	2020-10-07	47.81	48.5	46.43	48.23	41341	{}	\N	2020-10-12 14:39:25.369771
36678	DIG	2020-10-08	49.35	51.91	48.9955	51.9	103083	{}	\N	2020-10-12 14:39:25.374025
36679	DIG	2020-10-09	52.65	53.05	50.165	50.35	77941	{}	\N	2020-10-12 14:39:25.378118
36680	DIG	2020-10-12	49.99	50.275	49.05	50.04	12189	{}	\N	2020-10-12 14:39:25.382584
36681	FISK	2020-09-29	6.01	6.01	5.98	5.98	7904	{}	\N	2020-10-12 14:39:26.448996
36682	FISK	2020-10-01	6.31	6.31	6.26	6.26	613	{}	\N	2020-10-12 14:39:26.456602
36683	FISK	2020-10-02	6.31	6.39	6.28	6.39	6050	{}	\N	2020-10-12 14:39:26.4626
36684	FISK	2020-10-05	6.5	6.5	6.48	6.5	11353	{}	\N	2020-10-12 14:39:26.468235
36685	FISK	2020-10-06	6.6	6.6	6.6	6.6	321	{}	\N	2020-10-12 14:39:26.473524
36686	DBC	2020-09-28	12.99	13.09	12.95	13.06	806574	{}	\N	2020-10-12 14:39:28.172103
36687	DBC	2020-09-29	12.98	13.01	12.83	12.91	589551	{}	\N	2020-10-12 14:39:28.178384
36688	DBC	2020-09-30	12.94	13.1	12.91	13.06	1390071	{}	\N	2020-10-12 14:39:28.183649
36689	DBC	2020-10-01	12.96	12.975	12.775	12.89	1899252	{}	\N	2020-10-12 14:39:28.188434
36690	DBC	2020-10-02	12.68	12.82	12.64	12.74	1360356	{}	\N	2020-10-12 14:39:28.192762
36691	DBC	2020-10-05	12.97	13.09	12.97	13.07	1455317	{}	\N	2020-10-12 14:39:28.197406
36692	DBC	2020-10-06	13.2	13.28	13.1	13.12	781908	{}	\N	2020-10-12 14:39:28.201643
36693	DBC	2020-10-07	13.16	13.19	13.08	13.17	721741	{}	\N	2020-10-12 14:39:28.206206
36694	DBC	2020-10-08	13.29	13.33	13.23	13.3	1325959	{}	\N	2020-10-12 14:39:28.210442
36695	DBC	2020-10-09	13.33	13.415	13.3	13.34	1135706	{}	\N	2020-10-12 14:39:28.21476
36696	DBC	2020-10-12	13.33	13.33	13.18	13.21	528040	{}	\N	2020-10-12 14:39:28.219138
36697	DBS	2020-09-28	33.3745	33.8058	33.1	33.8058	2818	{}	\N	2020-10-12 14:39:30.008838
36698	DBS	2020-09-29	34.27	34.84	34.025	34.6562	2245	{}	\N	2020-10-12 14:39:30.021512
36699	DBS	2020-09-30	34.55	34.55	32.9828	33.06	5068	{}	\N	2020-10-12 14:39:30.028258
36700	DBS	2020-10-01	34.01	34.37	33.8982	33.8982	1881	{}	\N	2020-10-12 14:39:30.034748
36701	DBS	2020-10-02	33.83	34.47	33.83	34.06	3202	{}	\N	2020-10-12 14:39:30.039794
36702	DBS	2020-10-05	34.41	34.93	34.41	34.7416	1727	{}	\N	2020-10-12 14:39:30.044321
36703	DBS	2020-10-06	34.7	34.7	33.2899	33.2899	3178	{}	\N	2020-10-12 14:39:30.048843
36704	DBS	2020-10-07	33.85	33.9769	33.805	33.9769	2091	{}	\N	2020-10-12 14:39:30.053271
36705	DBS	2020-10-08	34.2759	34.2759	33.79	33.9903	6268	{}	\N	2020-10-12 14:39:30.057133
36706	DBS	2020-10-09	35.26	35.8894	35.26	35.8894	1616	{}	\N	2020-10-12 14:39:30.060859
36707	DBS	2020-10-12	36.14	36.14	35.56	35.6037	4423	{}	\N	2020-10-12 14:39:30.065267
36708	DCOM	2020-09-28	11.16	11.42	11.1391	11.29	72215	{}	\N	2020-10-12 14:39:31.017756
36709	DCOM	2020-09-29	11.21	11.21	10.86	11.19	76962	{}	\N	2020-10-12 14:39:31.027421
36710	DCOM	2020-09-30	11.395	11.58	11.2302	11.31	142022	{}	\N	2020-10-12 14:39:31.033691
36711	DCOM	2020-10-01	11.31	11.42	11.04	11.32	106483	{}	\N	2020-10-12 14:39:31.038722
36712	DCOM	2020-10-02	11.09	11.69	11.09	11.49	97873	{}	\N	2020-10-12 14:39:31.043167
36713	DCOM	2020-10-05	11.985	12.13	11.5692	12.07	130139	{}	\N	2020-10-12 14:39:31.047804
36714	DCOM	2020-10-06	12.25	12.48	11.99	12.01	238249	{}	\N	2020-10-12 14:39:31.051794
36715	DCOM	2020-10-07	12.16	12.33	12	12.18	107180	{}	\N	2020-10-12 14:39:31.055611
36716	DCOM	2020-10-08	12.295	12.49	12.18	12.41	75603	{}	\N	2020-10-12 14:39:31.059295
36717	DCOM	2020-10-09	12.46	12.52	12.26	12.47	73488	{}	\N	2020-10-12 14:39:31.063081
36718	DCOM	2020-10-12	12.45	12.48	12.3	12.48	2814	{}	\N	2020-10-12 14:39:31.066824
36719	DE	2020-09-28	222.06	224.3565	221.5	221.9	1225457	{}	\N	2020-10-12 14:39:33.392283
36720	DE	2020-09-29	221.65	222.39	218.07	220.18	1775529	{}	\N	2020-10-12 14:39:33.40104
36721	DE	2020-09-30	221.38	226.06	220.235	221.63	3556655	{}	\N	2020-10-12 14:39:33.407426
36722	DE	2020-10-01	223.5	223.78	218.57	219.04	2685147	{}	\N	2020-10-12 14:39:33.412394
36723	DE	2020-10-02	215.25	226.52	215.02	224.79	2712861	{}	\N	2020-10-12 14:39:33.417695
36724	DE	2020-10-05	227.26	231.12	224.85	226.96	4719058	{}	\N	2020-10-12 14:39:33.4223
36725	DE	2020-10-06	227.27	230.64	226.0101	226.12	2587024	{}	\N	2020-10-12 14:39:33.426439
36726	DE	2020-10-07	229.27	232.53	228.105	230.57	2001278	{}	\N	2020-10-12 14:39:33.431567
36727	DE	2020-10-08	230.99	234.81	230.71	234.24	2217114	{}	\N	2020-10-12 14:39:33.437095
36728	DE	2020-10-09	235.02	237.01	233.58	234.81	2129810	{}	\N	2020-10-12 14:39:33.441762
36729	DE	2020-10-12	237.14	237.26	236.01	236.2	719033	{}	\N	2020-10-12 14:39:33.446525
36730	DEUR	2020-09-28	28.69	28.89	28.69	28.71	16614	{}	\N	2020-10-12 14:39:35.117866
36731	DEUR	2020-09-29	28.04	28.116	28.04	28.07	7657	{}	\N	2020-10-12 14:39:35.12821
36732	DEUR	2020-09-30	28.46	28.47	28	28.24	22775	{}	\N	2020-10-12 14:39:35.135043
36733	DEUR	2020-10-01	28.1	28.1	27.97	27.97	5473	{}	\N	2020-10-12 14:39:35.141093
36734	DEUR	2020-10-02	28.31	28.3169	28.23	28.3169	5721	{}	\N	2020-10-12 14:39:35.145934
36735	DEUR	2020-10-05	27.5902	27.67	27.52	27.6502	2709	{}	\N	2020-10-12 14:39:35.150689
36736	DEUR	2020-10-06	27.4998	28.01	27.46	27.95	15247	{}	\N	2020-10-12 14:39:35.155249
36737	DEUR	2020-10-07	27.79	27.86	27.75	27.86	11338	{}	\N	2020-10-12 14:39:35.159125
36738	DEUR	2020-10-08	27.97	28.03	27.82	27.82	7431	{}	\N	2020-10-12 14:39:35.163201
36739	DEUR	2020-10-09	27.29	27.41	27.2102	27.2311	10971	{}	\N	2020-10-12 14:39:35.167589
36740	DEUR	2020-10-12	27.34	27.36	27.3	27.36	925	{}	\N	2020-10-12 14:39:35.171813
36741	DFE	2020-09-28	52.51	52.71	52.505	52.61	49266	{}	\N	2020-10-12 14:39:36.47834
36742	DFE	2020-09-29	52.81	52.95	52.595	52.76	17134	{}	\N	2020-10-12 14:39:36.487895
36743	DFE	2020-09-30	52.97	53.3767	52.97	53.1715	12889	{}	\N	2020-10-12 14:39:36.493981
36744	DFE	2020-10-01	53.69	53.7814	53.52	53.74	5825	{}	\N	2020-10-12 14:39:36.499583
36745	DFE	2020-10-02	53.14	53.7735	53.14	53.649	27941	{}	\N	2020-10-12 14:39:36.504924
36746	DFE	2020-10-05	54.5	54.95	54.38	54.86	8528	{}	\N	2020-10-12 14:39:36.509943
36747	DFE	2020-10-06	55.23	55.3284	54.46	54.525	33572	{}	\N	2020-10-12 14:39:36.514292
36748	DFE	2020-10-07	55.14	55.22	54.98	55.08	9655	{}	\N	2020-10-12 14:39:36.518608
36749	DFE	2020-10-08	55.5	55.55	55.405	55.525	11060	{}	\N	2020-10-12 14:39:36.522788
36750	DFE	2020-10-09	56.18	56.38	56.13	56.22	8715	{}	\N	2020-10-12 14:39:36.526748
36751	DFE	2020-10-12	56.34	56.45	56.23	56.41	9836	{}	\N	2020-10-12 14:39:36.530893
36752	DFVL	2020-09-28	86.31	86.31	86.31	86.31	101	{}	\N	2020-10-12 14:39:37.472077
36753	DFVL	2020-09-29	82.645	82.645	82.645	82.645	100	{}	\N	2020-10-12 14:39:37.489377
36754	DFVL	2020-09-30	87.02	87.02	87.02	87.02	288	{}	\N	2020-10-12 14:39:37.497282
36755	DFVL	2020-10-01	79.055	79.055	79.055	79.055	245	{}	\N	2020-10-12 14:39:37.504077
36756	DFVL	2020-10-02	102.985	102.985	102.985	102.985	161	{}	\N	2020-10-12 14:39:37.509585
36757	DFVL	2020-10-05	102.05	102.05	102.05	102.05	107	{}	\N	2020-10-12 14:39:37.514879
36758	DFVL	2020-10-06	72.405	72.405	72.405	72.405	103	{}	\N	2020-10-12 14:39:37.519444
36759	DFVL	2020-10-07	81.2	83.075	81.2	83.075	288	{}	\N	2020-10-12 14:39:37.523711
36760	DFVL	2020-10-08	91.665	91.665	91.665	91.665	142	{}	\N	2020-10-12 14:39:37.528053
36761	DFVL	2020-10-09	104.37	104.37	104.37	104.37	101	{}	\N	2020-10-12 14:39:37.532794
36762	DFVL	2020-10-12	93.94	93.94	93.94	93.94	324	{}	\N	2020-10-12 14:39:37.53731
36763	DGZ	2020-09-28	10.12	10.12	10.02	10.02	10177	{}	\N	2020-10-12 14:39:39.596421
36764	DGZ	2020-09-29	9.91	9.97	9.86	9.88	6287	{}	\N	2020-10-12 14:39:39.604244
36765	DGZ	2020-09-30	9.9	10.04	9.86	9.92	6359	{}	\N	2020-10-12 14:39:39.610152
36766	DGZ	2020-10-01	9.85	9.85	9.8	9.83	4236	{}	\N	2020-10-12 14:39:39.615043
36767	DGZ	2020-10-02	9.85	9.8588	9.83	9.8588	1117	{}	\N	2020-10-12 14:39:39.619924
36768	DGZ	2020-10-05	9.7715	9.8032	9.77	9.8032	2208	{}	\N	2020-10-12 14:39:39.624271
36769	DGZ	2020-10-06	9.79	9.9314	9.79	9.9314	3206	{}	\N	2020-10-12 14:39:39.628301
36770	DGZ	2020-10-07	9.97	9.975	9.94	9.975	1225	{}	\N	2020-10-12 14:39:39.632403
36771	DGZ	2020-10-08	9.94	9.94	9.9399	9.9399	276	{}	\N	2020-10-12 14:39:39.636773
36772	DGZ	2020-10-09	9.7	9.79	9.69	9.79	33590	{}	\N	2020-10-12 14:39:39.640938
36773	DCO	2020-09-28	33.73	34.42	33.73	33.98	61588	{}	\N	2020-10-12 14:39:40.657637
36774	DCO	2020-09-29	33.93	33.93	32.4	32.64	88090	{}	\N	2020-10-12 14:39:40.672904
36775	DCO	2020-09-30	32.68	33.33	32.64	32.92	148902	{}	\N	2020-10-12 14:39:40.678551
36776	DCO	2020-10-01	33.14	34.04	32.89	33.89	184524	{}	\N	2020-10-12 14:39:40.684077
36777	DCO	2020-10-02	33.11	34.65	32.792	34.13	88984	{}	\N	2020-10-12 14:39:40.689085
36778	DCO	2020-10-05	34.34	34.98	34	34.02	124068	{}	\N	2020-10-12 14:39:40.693805
36779	DCO	2020-10-06	34.55	35.91	33.91	34.2	183746	{}	\N	2020-10-12 14:39:40.698188
36780	DCO	2020-10-07	34.78	35.26	34.06	34.34	110047	{}	\N	2020-10-12 14:39:40.702513
36781	DCO	2020-10-08	34.81	35.59	34.36	35.23	69225	{}	\N	2020-10-12 14:39:40.7065
36782	DCO	2020-10-09	35.57	35.8	35.09	35.56	66065	{}	\N	2020-10-12 14:39:40.710654
36783	DCO	2020-10-12	35.56	35.59	35.28	35.45	2452	{}	\N	2020-10-12 14:39:40.714772
36784	DDG	2020-09-28	33.92	33.92	33.3442	33.6799	20050	{}	\N	2020-10-12 14:39:41.600678
36785	DDG	2020-09-29	34.2033	35.03	34.2033	34.6599	13274	{}	\N	2020-10-12 14:39:41.605267
36786	DDG	2020-09-30	34.63	35.09	34.2016	34.8301	7750	{}	\N	2020-10-12 14:39:41.60899
36787	DDG	2020-10-01	35.32	35.96	35.2901	35.898	9946	{}	\N	2020-10-12 14:39:41.612761
36788	DDG	2020-10-02	36.73	36.73	35.3	35.34	12366	{}	\N	2020-10-12 14:39:41.616303
36789	DDG	2020-10-05	34.8649	35.17	34	34.48	10092	{}	\N	2020-10-12 14:39:41.619989
36790	DDG	2020-10-06	34.07	34.93	34.01	34.855	4981	{}	\N	2020-10-12 14:39:41.623418
36791	DDG	2020-10-07	34.74	34.9041	34	34.3419	8730	{}	\N	2020-10-12 14:39:41.626814
36792	DDG	2020-10-08	34.01	34.2198	33.1546	33.1546	13715	{}	\N	2020-10-12 14:39:41.630794
36793	DDG	2020-10-09	33.1	33.6296	33.1	33.6296	4206	{}	\N	2020-10-12 14:39:41.63488
36794	DDG	2020-10-12	33.59	33.8	33.59	33.8	667	{}	\N	2020-10-12 14:39:41.638807
36795	DECK	2020-09-28	217.23	221.77	217.23	219.27	247252	{}	\N	2020-10-12 14:39:42.741635
36796	DECK	2020-09-29	218.28	219.635	214.2	217.91	477794	{}	\N	2020-10-12 14:39:42.748391
36797	DECK	2020-09-30	218.58	221.63	217.54	220.01	640155	{}	\N	2020-10-12 14:39:42.75302
36798	DECK	2020-10-01	222.65	238	221.94	235.6	1589558	{}	\N	2020-10-12 14:39:42.757627
36799	DECK	2020-10-02	229.76	238.47	229.76	237.38	601234	{}	\N	2020-10-12 14:39:42.762014
36800	DECK	2020-10-05	238.19	239.59	232.81	238.5	478481	{}	\N	2020-10-12 14:39:42.766286
36801	DECK	2020-10-06	240.11	240.32	233.84	234.97	668873	{}	\N	2020-10-12 14:39:42.771029
36802	DECK	2020-10-07	239.05	242.41	237.87	240.34	587536	{}	\N	2020-10-12 14:39:42.775164
36803	DECK	2020-10-08	243.33	247.0307	240.6901	245.26	593308	{}	\N	2020-10-12 14:39:42.779989
36804	DECK	2020-10-09	247.8	251.52	245.29	251.21	645960	{}	\N	2020-10-12 14:39:42.785622
36805	DECK	2020-10-12	256.45	257.8789	252.16	252.285	140190	{}	\N	2020-10-12 14:39:42.790875
36806	DEO	2020-09-28	138.45	138.585	136.93	137.55	911547	{}	\N	2020-10-12 14:39:43.815936
36807	DEO	2020-09-29	135.85	137.3	135.19	136.1	1071832	{}	\N	2020-10-12 14:39:43.824116
36808	DEO	2020-09-30	137.52	138.15	136.785	137.66	1038385	{}	\N	2020-10-12 14:39:43.829947
36809	DEO	2020-10-01	139	139.07	137.77	138.87	1266553	{}	\N	2020-10-12 14:39:43.834962
36810	DEO	2020-10-02	137.51	139.42	137.42	138.64	1025044	{}	\N	2020-10-12 14:39:43.839991
36811	DEO	2020-10-05	140	140.74	139.52	140.67	811715	{}	\N	2020-10-12 14:39:43.844632
36812	DEO	2020-10-06	139.29	139.855	137.65	138.33	901889	{}	\N	2020-10-12 14:39:43.849057
36813	DEO	2020-10-07	140.05	140.61	139.648	140.53	847207	{}	\N	2020-10-12 14:39:43.853583
36814	DEO	2020-10-08	140.17	141.32	139.97	140.86	674458	{}	\N	2020-10-12 14:39:43.857864
36815	DEO	2020-10-09	141.28	141.84	140.9	141.23	541628	{}	\N	2020-10-12 14:39:43.862082
36816	DEO	2020-10-12	140.46	141.01	140.14	140.81	150730	{}	\N	2020-10-12 14:39:43.866314
36817	DEX	2020-09-28	8.74	8.86	8.73	8.81	127643	{}	\N	2020-10-12 14:39:44.765783
36818	DEX	2020-09-29	8.81	8.86	8.77	8.85	51376	{}	\N	2020-10-12 14:39:44.776828
36819	DEX	2020-09-30	8.8	8.89	8.8	8.87	48732	{}	\N	2020-10-12 14:39:44.78437
36820	DEX	2020-10-01	8.88	9	8.82	8.86	177978	{}	\N	2020-10-12 14:39:44.790789
36821	DEX	2020-10-02	8.8	8.9	8.71	8.87	37332	{}	\N	2020-10-12 14:39:44.796572
36822	DEX	2020-10-05	8.85	9.03	8.85	9.03	97910	{}	\N	2020-10-12 14:39:44.80217
36823	DEX	2020-10-06	9.02	9.04	8.94	8.94	36026	{}	\N	2020-10-12 14:39:44.807131
36824	DEX	2020-10-07	9.08	9.08	8.99	9.04	38762	{}	\N	2020-10-12 14:39:44.812267
36825	DEX	2020-10-08	9.06	9.08	9.05	9.08	21894	{}	\N	2020-10-12 14:39:44.817231
36826	DEX	2020-10-09	9.08	9.085	9.0267	9.05	16866	{}	\N	2020-10-12 14:39:44.82185
36827	DEX	2020-10-12	9.08	9.095	9.08	9.09	5338	{}	\N	2020-10-12 14:39:44.826377
36828	DFP	2020-09-28	26.91	27.0199	26.8	26.94	124375	{}	\N	2020-10-12 14:39:46.63618
36829	DFP	2020-09-29	27	27.2	26.95	27.2	189733	{}	\N	2020-10-12 14:39:46.652166
36830	DFP	2020-09-30	27.3	27.4673	27.24	27.3	193076	{}	\N	2020-10-12 14:39:46.658921
36831	DFP	2020-10-01	27.5	27.53	27.28	27.51	169010	{}	\N	2020-10-12 14:39:46.664685
36832	DFP	2020-10-02	27.1	27.53	26.96	27.47	171562	{}	\N	2020-10-12 14:39:46.671253
36833	DFP	2020-10-05	27.47	27.88	27.47	27.84	128058	{}	\N	2020-10-12 14:39:46.67618
36834	DFP	2020-10-06	27.86	27.8787	27.6	27.66	167201	{}	\N	2020-10-12 14:39:46.680618
36835	DFP	2020-10-07	27.66	27.7999	27.61	27.77	73562	{}	\N	2020-10-12 14:39:46.685232
36836	DFP	2020-10-08	27.76	27.939	27.76	27.83	175410	{}	\N	2020-10-12 14:39:46.689506
36837	DFP	2020-10-09	27.88	27.95	27.77	27.83	179384	{}	\N	2020-10-12 14:39:46.693805
36838	DFP	2020-10-12	27.88	27.91	27.8	27.825	18576	{}	\N	2020-10-12 14:39:46.697952
36839	DGBP	2020-09-28	25.87	26.0918	25.87	26.0918	401	{}	\N	2020-10-12 14:39:48.523021
36840	DGBP	2020-09-29	26.03	26.03	25.8946	25.8946	247	{}	\N	2020-10-12 14:39:48.532407
36841	DGBP	2020-09-30	25.4675	25.4675	25.4675	25.4675	109	{}	\N	2020-10-12 14:39:48.538784
36842	DGBP	2020-10-01	25.6544	25.6544	25.6544	25.6544	1	{}	\N	2020-10-12 14:39:48.54363
36843	DGBP	2020-10-02	25.41	25.46	25.2807	25.2807	9034	{}	\N	2020-10-12 14:39:48.548143
36844	DGBP	2020-10-05	24.91	24.91	24.9024	24.9024	471	{}	\N	2020-10-12 14:39:48.552636
36845	DGBP	2020-10-06	25.4793	25.4793	25.4793	25.4793	203	{}	\N	2020-10-12 14:39:48.556457
36846	DGBP	2020-10-07	25.4598	25.4598	25.439	25.439	172	{}	\N	2020-10-12 14:39:48.560116
36847	DGBP	2020-10-08	25.2923	25.2923	25.2923	25.2923	56	{}	\N	2020-10-12 14:39:48.563839
36848	DGBP	2020-10-09	24.96	24.96	24.4461	24.4461	204	{}	\N	2020-10-12 14:39:48.5679
36849	DGBP	2020-10-12	24.44	24.44	24.4206	24.4206	204	{}	\N	2020-10-12 14:39:48.571953
36850	DHT	2020-09-28	5.21	5.22	5.02	5.11	2121320	{}	\N	2020-10-12 14:39:50.606526
36851	DHT	2020-09-29	5.08	5.16	5.05	5.14	2486339	{}	\N	2020-10-12 14:39:50.61465
36852	DHT	2020-09-30	5.15	5.29	5.14	5.16	3608837	{}	\N	2020-10-12 14:39:50.620705
36853	DHT	2020-10-01	5.17	5.24	5.09	5.19	2275639	{}	\N	2020-10-12 14:39:50.625921
36854	DHT	2020-10-02	5.07	5.2001	5.03	5.18	3752776	{}	\N	2020-10-12 14:39:50.630436
36855	DHT	2020-10-05	5.25	5.4	5.21	5.34	3335557	{}	\N	2020-10-12 14:39:50.63533
36856	DHT	2020-10-06	5.4	5.445	5.27	5.29	3607135	{}	\N	2020-10-12 14:39:50.63924
36857	DHT	2020-10-07	5.3	5.34	5.17	5.23	4172108	{}	\N	2020-10-12 14:39:50.64315
36858	DHT	2020-10-08	5.29	5.43	5.27	5.38	3347443	{}	\N	2020-10-12 14:39:50.647268
36859	DHT	2020-10-09	5.43	5.49	5.36	5.36	2849788	{}	\N	2020-10-12 14:39:50.651234
36860	DHT	2020-10-12	5.36	5.37	5.02	5.19	2541158	{}	\N	2020-10-12 14:39:50.65559
36861	DISCK	2020-09-28	19.62	19.895	19.22	19.73	2688224	{}	\N	2020-10-12 14:39:51.612524
36862	DISCK	2020-09-29	19.65	19.96	19.36	19.47	2042928	{}	\N	2020-10-12 14:39:51.621912
36863	DISCK	2020-09-30	19.6	20.07	19.48	19.6	2885039	{}	\N	2020-10-12 14:39:51.628161
36864	DISCK	2020-10-01	19.67	19.78	19.15	19.36	1795005	{}	\N	2020-10-12 14:39:51.633584
36865	DISCK	2020-10-02	19.09	19.53	18.99	19.4	1519666	{}	\N	2020-10-12 14:39:51.638615
36866	DISCK	2020-10-05	19.65	19.99	19.49	19.72	2481093	{}	\N	2020-10-12 14:39:51.64312
36867	DISCK	2020-10-06	19.81	19.93	19.3	19.335	3031177	{}	\N	2020-10-12 14:39:51.647429
36868	DISCK	2020-10-07	19.47	20.13	19.29	19.985	2469262	{}	\N	2020-10-12 14:39:51.651596
36869	DISCK	2020-10-08	20.04	20.485	19.99	20.47	1853981	{}	\N	2020-10-12 14:39:51.65545
36870	DISCK	2020-10-09	20.53	20.74	19.96	19.965	1707528	{}	\N	2020-10-12 14:39:51.659379
36871	DISCK	2020-10-12	19.86	19.97	19.71	19.82	262629	{}	\N	2020-10-12 14:39:51.663392
36872	DK	2020-09-28	11	11.505	10.98	11.44	1215639	{}	\N	2020-10-12 14:39:53.676567
36873	DK	2020-09-29	11.48	11.48	11.07	11.3	1956842	{}	\N	2020-10-12 14:39:53.684277
36874	DK	2020-09-30	11.25	11.53	11.01	11.13	2229847	{}	\N	2020-10-12 14:39:53.69037
36875	DK	2020-10-01	10.86	10.98	10.35	10.86	2689398	{}	\N	2020-10-12 14:39:53.695477
36876	DK	2020-10-02	10.49	11.51	10.47	11.41	2853443	{}	\N	2020-10-12 14:39:53.700607
36877	DK	2020-10-05	11.64	12.23	11.63	12.04	1734351	{}	\N	2020-10-12 14:39:53.705731
36878	DK	2020-10-06	12.37	12.47	11.67	11.79	1656252	{}	\N	2020-10-12 14:39:53.710085
36879	DK	2020-10-07	11.89	12.21	11.76	12.17	1404078	{}	\N	2020-10-12 14:39:53.714265
36880	DK	2020-10-08	12.39	13	12.29	13	1398036	{}	\N	2020-10-12 14:39:53.718497
36881	DK	2020-10-09	13.03	13.14	12.09	12.12	1926496	{}	\N	2020-10-12 14:39:53.723149
36882	DK	2020-10-12	11.91	12.12	11.56	11.75	315742	{}	\N	2020-10-12 14:39:53.727786
36883	DMB	2020-09-28	13.35	13.445	13.35	13.445	22839	{}	\N	2020-10-12 14:39:55.310656
36884	DMB	2020-09-29	13.4	13.48	13.4	13.4799	26808	{}	\N	2020-10-12 14:39:55.322616
36885	DMB	2020-09-30	13.44	13.52	13.39	13.45	41491	{}	\N	2020-10-12 14:39:55.329907
36886	DMB	2020-10-01	13.56	13.56	13.48	13.5324	48676	{}	\N	2020-10-12 14:39:55.335699
36887	DMB	2020-10-02	13.45	13.6	13.45	13.59	34502	{}	\N	2020-10-12 14:39:55.345206
36888	DMB	2020-10-05	13.58	13.62	13.55	13.62	51250	{}	\N	2020-10-12 14:39:55.351151
36889	DMB	2020-10-06	13.56	13.62	13.56	13.61	26978	{}	\N	2020-10-12 14:39:55.355406
36890	DMB	2020-10-07	13.6	13.68	13.6	13.68	27570	{}	\N	2020-10-12 14:39:55.359301
36891	DMB	2020-10-08	13.66	13.7	13.65	13.6999	40506	{}	\N	2020-10-12 14:39:55.363295
36892	DMB	2020-10-09	13.61	13.72	13.6	13.71	40430	{}	\N	2020-10-12 14:39:55.367416
36893	DMB	2020-10-12	13.65	13.7	13.65	13.6956	12948	{}	\N	2020-10-12 14:39:55.371506
36894	DNKN	2020-09-28	79.84	81.53	79.84	81.35	831282	{}	\N	2020-10-12 14:39:56.469351
36895	DNKN	2020-09-29	81.17	81.5101	79.96	80.85	739832	{}	\N	2020-10-12 14:39:56.479645
36896	DNKN	2020-09-30	81.24	82.14	80.86	81.91	802799	{}	\N	2020-10-12 14:39:56.486222
36897	DNKN	2020-10-01	82.37	84.59	82.37	84.5	1016089	{}	\N	2020-10-12 14:39:56.491631
36898	DNKN	2020-10-02	83.18	85.55	83	84.49	722562	{}	\N	2020-10-12 14:39:56.497314
36899	DNKN	2020-10-05	83.13	84.62	81.84	81.92	1032252	{}	\N	2020-10-12 14:39:56.502408
36900	DNKN	2020-10-06	82.16	82.95	80.305	80.64	792153	{}	\N	2020-10-12 14:39:56.506774
36901	DNKN	2020-10-07	81.64	82.73	81.22	82.2	533727	{}	\N	2020-10-12 14:39:56.511435
36902	DNKN	2020-10-08	82.89	83.64	82.177	83.25	614101	{}	\N	2020-10-12 14:39:56.516193
36903	DNKN	2020-10-09	83.7	84.37	82.905	84.3	532291	{}	\N	2020-10-12 14:39:56.520711
36904	DNKN	2020-10-12	84.49	84.49	83.25	83.825	86774	{}	\N	2020-10-12 14:39:56.525526
36905	DCPH	2020-09-28	49.97	50.82	48.77	49.95	240175	{}	\N	2020-10-12 14:39:57.422058
36906	DCPH	2020-09-29	50	50.01	48.65	49.55	432144	{}	\N	2020-10-12 14:39:57.431088
36907	DCPH	2020-09-30	49.96	51.36	49.24	51.3	382693	{}	\N	2020-10-12 14:39:57.437194
36908	DCPH	2020-10-01	51.99	52.4	50.57	52.35	713129	{}	\N	2020-10-12 14:39:57.442728
36909	DCPH	2020-10-02	51.15	52.68	50.855	51.77	307058	{}	\N	2020-10-12 14:39:57.448256
36910	DCPH	2020-10-05	52.54	55.78	51.83	55.55	484336	{}	\N	2020-10-12 14:39:57.453168
36911	DCPH	2020-10-06	55.54	57.87	54.42	56.2	408469	{}	\N	2020-10-12 14:39:57.457956
36912	DCPH	2020-10-07	56.77	58	55.755	57.42	397769	{}	\N	2020-10-12 14:39:57.462991
36913	DCPH	2020-10-08	58	58	55	55.17	270529	{}	\N	2020-10-12 14:39:57.467997
36914	DCPH	2020-10-09	56	56	53.1	53.46	288778	{}	\N	2020-10-12 14:39:57.472573
36915	DCPH	2020-10-12	53.59	55.15	53.32	54.67	19415	{}	\N	2020-10-12 14:39:57.477302
36916	DEI	2020-09-28	25.09	25.7	24.88	25.47	1372296	{}	\N	2020-10-12 14:39:58.429774
36917	DEI	2020-09-29	25.09	25.35	24.41	24.92	3388404	{}	\N	2020-10-12 14:39:58.438463
36918	DEI	2020-09-30	25.6	25.84	24.82	25.1	3409041	{}	\N	2020-10-12 14:39:58.444708
36919	DEI	2020-10-01	25.11	25.72	24.99	25.72	2637475	{}	\N	2020-10-12 14:39:58.449867
36920	DEI	2020-10-02	25.24	26.59	25.05	26.43	2904301	{}	\N	2020-10-12 14:39:58.455073
36921	DEI	2020-10-05	26.66	26.805	26.29	26.43	2895307	{}	\N	2020-10-12 14:39:58.459518
36922	DEI	2020-10-06	26.5	26.87	26.15	26.33	2458757	{}	\N	2020-10-12 14:39:58.463916
36923	DEI	2020-10-07	26.42	26.44	25.76	25.99	1732071	{}	\N	2020-10-12 14:39:58.468425
36924	DEI	2020-10-08	26.22	26.44	25.99	26.24	1948911	{}	\N	2020-10-12 14:39:58.473277
36925	DEI	2020-10-09	26.24	26.3	25.58	25.66	1751813	{}	\N	2020-10-12 14:39:58.477863
36926	DEI	2020-10-12	25.49	25.68	25.3	25.59	1575556	{}	\N	2020-10-12 14:39:58.482235
36927	DEUS	2020-09-28	33.7699	33.9553	33.7699	33.8479	3821	{}	\N	2020-10-12 14:39:59.465879
36928	DEUS	2020-09-29	33.892	33.892	33.67	33.7525	20016	{}	\N	2020-10-12 14:39:59.476121
36929	DEUS	2020-09-30	33.8627	34.1599	33.7201	33.93	42748	{}	\N	2020-10-12 14:39:59.48224
36930	DEUS	2020-10-01	34.1	34.1	33.89	33.97	15613	{}	\N	2020-10-12 14:39:59.487568
36931	DEUS	2020-10-02	33.6595	34.1772	33.6595	34.03	15971	{}	\N	2020-10-12 14:39:59.492267
36932	DEUS	2020-10-05	34.4137	34.7031	34.4137	34.6935	19430	{}	\N	2020-10-12 14:39:59.49698
36933	DEUS	2020-10-06	34.81	35.09	34.43	34.43	45516	{}	\N	2020-10-12 14:39:59.50122
36934	DEUS	2020-10-07	34.67	35.03	34.67	34.9789	2388	{}	\N	2020-10-12 14:39:59.505086
36935	DEUS	2020-10-08	35.2088	35.3375	35.2088	35.3375	3416	{}	\N	2020-10-12 14:39:59.508814
36936	DEUS	2020-10-09	35.4798	35.6599	35.4631	35.5	6560	{}	\N	2020-10-12 14:39:59.512814
36937	DEUS	2020-10-12	35.67	35.84	35.67	35.8	6813	{}	\N	2020-10-12 14:39:59.516807
36938	DFNL	2020-09-28	19.06	19.1238	18.9899	18.9899	5145	{}	\N	2020-10-12 14:40:01.590571
36939	DFNL	2020-09-29	18.9282	18.9282	18.72	18.8258	7243	{}	\N	2020-10-12 14:40:01.602636
36940	DFNL	2020-09-30	18.83	19.14	18.83	19.0478	7795	{}	\N	2020-10-12 14:40:01.609996
36941	DFNL	2020-10-01	19.14	19.15	18.99	19.15	9983	{}	\N	2020-10-12 14:40:01.61549
36942	DFNL	2020-10-02	18.86	19.415	18.86	19.37	9515	{}	\N	2020-10-12 14:40:01.620314
36943	DFNL	2020-10-05	19.57	19.7603	19.57	19.7603	4100	{}	\N	2020-10-12 14:40:01.624899
36944	DFNL	2020-10-06	19.92	20.05	19.525	19.54	8507	{}	\N	2020-10-12 14:40:01.629347
36945	DFNL	2020-10-07	19.95	19.95	19.7568	19.9144	2098	{}	\N	2020-10-12 14:40:01.633299
36946	DFNL	2020-10-08	20.16	20.255	20.115	20.255	3774	{}	\N	2020-10-12 14:40:01.637498
36947	DFNL	2020-10-09	20.35	20.35	20.1389	20.2369	50183	{}	\N	2020-10-12 14:40:01.641815
36948	DFNL	2020-10-12	20.27	20.35	20.27	20.35	970	{}	\N	2020-10-12 14:40:01.64599
36949	DGRW	2020-09-28	49.34	49.635	49.25	49.52	352405	{}	\N	2020-10-12 14:40:03.171707
36950	DGRW	2020-09-29	49.45	49.61	49.11	49.21	229523	{}	\N	2020-10-12 14:40:03.182018
36951	DGRW	2020-09-30	49.34	49.98	49.2701	49.57	213756	{}	\N	2020-10-12 14:40:03.188181
36952	DGRW	2020-10-01	49.8836	49.9355	49.4341	49.695	228928	{}	\N	2020-10-12 14:40:03.193991
36953	DGRW	2020-10-02	48.94	49.63	48.83	49.3	206736	{}	\N	2020-10-12 14:40:03.198382
36954	DGRW	2020-10-05	49.74	50.1899	49.61	50.11	323783	{}	\N	2020-10-12 14:40:03.202664
36955	DGRW	2020-10-06	50.19	50.5316	49.53	49.56	215340	{}	\N	2020-10-12 14:40:03.206544
36956	DGRW	2020-10-07	50.02	50.59	50.02	50.44	249614	{}	\N	2020-10-12 14:40:03.210437
36957	DGRW	2020-10-08	50.71	50.8266	50.5899	50.8	297226	{}	\N	2020-10-12 14:40:03.214158
36958	DGRW	2020-10-09	51.11	51.3396	51.01	51.2	312770	{}	\N	2020-10-12 14:40:03.217958
36959	DGRW	2020-10-12	51.61	51.6114	51.49	51.6114	73148	{}	\N	2020-10-12 14:40:03.222105
36960	DIOD	2020-09-28	51.5	53.175	51.39	53.12	306604	{}	\N	2020-10-12 14:40:05.334936
36961	DIOD	2020-09-29	53.22	55.21	52.84	55.16	374339	{}	\N	2020-10-12 14:40:05.34374
36962	DIOD	2020-09-30	55.09	57	54.62	56.45	557479	{}	\N	2020-10-12 14:40:05.350321
36963	DIOD	2020-10-01	57	59.3	56.36	58.98	552363	{}	\N	2020-10-12 14:40:05.355723
36964	DIOD	2020-10-02	57.08	58.77	56.68	57.78	245029	{}	\N	2020-10-12 14:40:05.360417
36965	DIOD	2020-10-05	58.44	59.45	57.8325	59.3	260885	{}	\N	2020-10-12 14:40:05.36482
36966	DIOD	2020-10-06	59.39	60.7678	59.16	59.89	262710	{}	\N	2020-10-12 14:40:05.369431
36967	DIOD	2020-10-07	60.52	61.34	59.49	61.09	336341	{}	\N	2020-10-12 14:40:05.373394
36968	DIOD	2020-10-08	61.5	62.88	61.295	62.46	337818	{}	\N	2020-10-12 14:40:05.377488
36969	DIOD	2020-10-09	63.19	64	62.93	63.47	195148	{}	\N	2020-10-12 14:40:05.381368
36970	DIOD	2020-10-12	63.75	64.28	63.52	64.19	14698	{}	\N	2020-10-12 14:40:05.385228
36971	DJP	2020-09-28	19.49	19.605	19.38	19.59	63137	{}	\N	2020-10-12 14:40:06.567432
36972	DJP	2020-09-29	19.38	19.38	19.185	19.25	167616	{}	\N	2020-10-12 14:40:06.576751
36973	DJP	2020-09-30	19.25	19.5888	19.25	19.48	201722	{}	\N	2020-10-12 14:40:06.583018
36974	DJP	2020-10-01	19.44	19.44	19.16	19.26	858351	{}	\N	2020-10-12 14:40:06.588175
36975	DJP	2020-10-02	19.04	19.311	19.04	19.13	100468	{}	\N	2020-10-12 14:40:06.592686
36976	DJP	2020-10-05	19.54	19.7	19.53	19.61	118039	{}	\N	2020-10-12 14:40:06.59721
36977	DJP	2020-10-06	19.69	19.9079	19.52	19.55	178058	{}	\N	2020-10-12 14:40:06.601422
36978	DJP	2020-10-07	19.81	19.9	19.6901	19.84	75479	{}	\N	2020-10-12 14:40:06.605208
36979	DJP	2020-10-08	20	20.05	19.8971	19.95	50358	{}	\N	2020-10-12 14:40:06.608879
36980	DJP	2020-10-09	20.27	20.4	20.242	20.33	153577	{}	\N	2020-10-12 14:40:06.612954
36981	DJP	2020-10-12	20.38	20.3899	20.21	20.22	24817	{}	\N	2020-10-12 14:40:06.616863
36982	DLNG	2020-09-28	2.36	2.64	2.35	2.64	86674	{}	\N	2020-10-12 14:40:07.575307
36983	DLNG	2020-09-29	2.63	2.63	2.44	2.44	158602	{}	\N	2020-10-12 14:40:07.584757
36984	DLNG	2020-09-30	2.47	2.55	2.47	2.5	35691	{}	\N	2020-10-12 14:40:07.591235
36985	DLNG	2020-10-01	2.55	2.65	2.4	2.41	60764	{}	\N	2020-10-12 14:40:07.596935
36986	DLNG	2020-10-02	2.4	2.47	2.2	2.39	138922	{}	\N	2020-10-12 14:40:07.601882
36987	DLNG	2020-10-05	2.35	2.46	2.34	2.4	65996	{}	\N	2020-10-12 14:40:07.60727
36988	DLNG	2020-10-06	2.35	2.64	2.35	2.64	332493	{}	\N	2020-10-12 14:40:07.611498
36989	DLNG	2020-10-07	2.68	2.68	2.35	2.6	116255	{}	\N	2020-10-12 14:40:07.615914
36990	DLNG	2020-10-08	2.54	2.82	2.54	2.6	184908	{}	\N	2020-10-12 14:40:07.620167
36991	DLNG	2020-10-09	2.58	2.6287	2.4	2.54	220718	{}	\N	2020-10-12 14:40:07.624391
36992	DLNG	2020-10-12	2.52	2.61	2.52	2.5363	36428	{}	\N	2020-10-12 14:40:07.628605
36993	FISV	2020-09-28	103.96	104.67	103.19	104.07	3468473	{}	\N	2020-10-12 14:40:09.014674
36994	FISV	2020-09-29	103.58	104.74	103.41	103.59	3460855	{}	\N	2020-10-12 14:40:09.024774
36995	FISV	2020-09-30	103.96	104.25	102.07	103.05	4758501	{}	\N	2020-10-12 14:40:09.036886
36996	FISV	2020-10-01	104.29	105.8	103.38	104.25	2860674	{}	\N	2020-10-12 14:40:09.043699
36997	FISV	2020-10-02	101.9	102.95	100.9	101.61	2709654	{}	\N	2020-10-12 14:40:09.049384
36998	FISV	2020-10-05	102.8	102.99	100.06	100.82	4201158	{}	\N	2020-10-12 14:40:09.056425
36999	FISV	2020-10-06	102.5	104.74	101.8612	102.36	4216848	{}	\N	2020-10-12 14:40:09.061906
37000	FISV	2020-10-07	102.96	103.633	101.78	102.42	3663576	{}	\N	2020-10-12 14:40:09.066094
37001	FISV	2020-10-08	103.33	104.2	102.64	103.39	2608252	{}	\N	2020-10-12 14:40:09.07017
37002	FISV	2020-10-09	103.88	105.27	103.15	105.11	2343918	{}	\N	2020-10-12 14:40:09.074014
37003	FISV	2020-10-12	105.59	105.99	104.56	105.68	736386	{}	\N	2020-10-12 14:40:09.077959
37004	DDM	2020-09-28	45.99	46.6265	45.84	46.2	446370	{}	\N	2020-10-12 14:40:10.059593
37005	DDM	2020-09-29	46.14	46.24	45.36	45.74	375654	{}	\N	2020-10-12 14:40:10.070514
37006	DDM	2020-09-30	45.91	47.6499	45.91	46.79	733257	{}	\N	2020-10-12 14:40:10.077052
37007	DDM	2020-10-01	47.39	47.71	46.45	46.99	489251	{}	\N	2020-10-12 14:40:10.082704
37008	DDM	2020-10-02	45.62	47.1	45.46	46.49	740688	{}	\N	2020-10-12 14:40:10.087665
37009	DDM	2020-10-05	47.18	48.13	47.18	48.08	801196	{}	\N	2020-10-12 14:40:10.092648
37010	DDM	2020-10-06	48.47	48.7687	46.65	46.76	872487	{}	\N	2020-10-12 14:40:10.096994
37011	DDM	2020-10-07	47.62	48.785	47.62	48.51	454776	{}	\N	2020-10-12 14:40:10.101081
37012	DDM	2020-10-08	49.08	49.12	48.47	49.03	612258	{}	\N	2020-10-12 14:40:10.105332
37013	DDM	2020-10-09	49.45	49.87	49.05	49.56	666114	{}	\N	2020-10-12 14:40:10.109423
37014	DDM	2020-10-12	49.96	50.27	49.8269	50.17	168191	{}	\N	2020-10-12 14:40:10.113809
37015	DEEF	2020-09-28	27.16	27.2	27.02	27.13	5602	{}	\N	2020-10-12 14:40:11.300797
37016	DEEF	2020-09-29	27.1375	27.1375	27.0201	27.0201	821	{}	\N	2020-10-12 14:40:11.308901
37017	DEEF	2020-09-30	27.1435	27.1435	26.86	26.9234	3122	{}	\N	2020-10-12 14:40:11.31506
37018	DEEF	2020-10-01	27.14	27.17	27.01	27.12	6545	{}	\N	2020-10-12 14:40:11.320442
37019	DEEF	2020-10-02	27.113	27.113	27.0357	27.0357	997	{}	\N	2020-10-12 14:40:11.328397
37020	DEEF	2020-10-05	27.4133	27.4133	27.3607	27.3607	1971	{}	\N	2020-10-12 14:40:11.334763
37021	DEEF	2020-10-06	27.305	27.34	27.1545	27.1545	814	{}	\N	2020-10-12 14:40:11.349187
37022	DEEF	2020-10-07	27.25	27.3547	27.25	27.3547	1127	{}	\N	2020-10-12 14:40:11.36529
37023	DEEF	2020-10-08	27.4888	27.5	27.38	27.49	2088	{}	\N	2020-10-12 14:40:11.383435
37024	DEEF	2020-10-09	27.555	27.575	27.555	27.575	471	{}	\N	2020-10-12 14:40:11.391234
37025	DES	2020-09-28	20.83	21.25	20.83	21.17	279085	{}	\N	2020-10-12 14:40:12.367794
37026	DES	2020-09-29	21.13	21.1899	20.805	21.01	347301	{}	\N	2020-10-12 14:40:12.37326
37027	DES	2020-09-30	21.06	21.4	20.93	21.08	450863	{}	\N	2020-10-12 14:40:12.378925
37028	DES	2020-10-01	21.13	21.25	20.92	21.23	364600	{}	\N	2020-10-12 14:40:12.384399
37029	DES	2020-10-02	20.9	21.7158	20.9	21.63	289132	{}	\N	2020-10-12 14:40:12.390549
37030	DES	2020-10-05	21.8	22.105	21.8	22.09	285498	{}	\N	2020-10-12 14:40:12.398582
37031	DES	2020-10-06	22.22	22.67	22.05	22.05	547032	{}	\N	2020-10-12 14:40:12.406469
37032	DES	2020-10-07	22.27	22.51	22.23	22.42	321942	{}	\N	2020-10-12 14:40:12.415532
37033	DES	2020-10-08	22.59	22.795	22.52	22.75	479661	{}	\N	2020-10-12 14:40:12.424552
37034	DES	2020-10-09	22.93	23	22.697	22.72	322595	{}	\N	2020-10-12 14:40:12.432495
37035	DES	2020-10-12	22.76	22.9399	22.68	22.9	51404	{}	\N	2020-10-12 14:40:12.439679
37036	DGICA	2020-09-28	13.92	14.4	13.92	14.11	42005	{}	\N	2020-10-12 14:40:15.462781
37037	DGICA	2020-09-29	14.04	14.17	13.9	14.06	25441	{}	\N	2020-10-12 14:40:15.473224
37038	DGICA	2020-09-30	14.015	14.26	13.99	14.07	19519	{}	\N	2020-10-12 14:40:15.479568
37039	DGICA	2020-10-01	14.125	14.43	14.11	14.35	30532	{}	\N	2020-10-12 14:40:15.485012
37040	DGICA	2020-10-02	14.31	14.48	14.26	14.39	25740	{}	\N	2020-10-12 14:40:15.4897
37041	DGICA	2020-10-05	14.48	14.57	14.41	14.41	18950	{}	\N	2020-10-12 14:40:15.494244
37042	DGICA	2020-10-06	14.53	14.74	14.4233	14.47	22811	{}	\N	2020-10-12 14:40:15.498195
37043	DGICA	2020-10-07	14.56	14.71	14.2501	14.5	26648	{}	\N	2020-10-12 14:40:15.502215
37044	DGICA	2020-10-08	14.6	14.61	14.27	14.46	50761	{}	\N	2020-10-12 14:40:15.506176
37045	DGICA	2020-10-09	14.48	14.65	14.25	14.54	30303	{}	\N	2020-10-12 14:40:15.510478
37046	DGICA	2020-10-12	14.55	14.62	14.33	14.6	4965	{}	\N	2020-10-12 14:40:15.514427
37047	DGT	2020-09-28	82.35	82.447	82.08	82.21	1866	{}	\N	2020-10-12 14:40:16.53136
37048	DGT	2020-09-29	81.96	82.04	81.69	81.7	2103	{}	\N	2020-10-12 14:40:16.540102
37049	DGT	2020-09-30	82.36	82.36	79.15	81.71	25840	{}	\N	2020-10-12 14:40:16.545916
37050	DGT	2020-10-01	82	82.0693	82	82.0693	633	{}	\N	2020-10-12 14:40:16.550683
37051	DGT	2020-10-02	81.64	82.18	81.43	82	1521	{}	\N	2020-10-12 14:40:16.556629
37052	DGT	2020-10-05	83.3886	83.3886	83.3886	83.3886	150	{}	\N	2020-10-12 14:40:16.561407
37053	DGT	2020-10-06	83.9	83.9	82.7835	82.7835	837	{}	\N	2020-10-12 14:40:16.565373
37054	DGT	2020-10-07	83.56	84.1179	83.56	84.1179	544	{}	\N	2020-10-12 14:40:16.569751
37055	DGT	2020-10-08	84.24	84.87	84.24	84.82	2796	{}	\N	2020-10-12 14:40:16.574926
37056	DGT	2020-10-09	84.97	85.026	84.9	85.026	4295	{}	\N	2020-10-12 14:40:16.579117
37057	DGT	2020-10-12	85.43	85.43	85.43	85.43	725	{}	\N	2020-10-12 14:40:16.583223
37058	DHIL	2020-09-28	125.94	126.5567	125.24	126.26	10071	{}	\N	2020-10-12 14:40:17.686046
37059	DHIL	2020-09-29	126.26	128.08	125.79	128.08	5789	{}	\N	2020-10-12 14:40:17.695119
37060	DHIL	2020-09-30	127	128	126.32	126.32	7979	{}	\N	2020-10-12 14:40:17.700965
37061	DHIL	2020-10-01	126.6	129	126.6	128.01	5566	{}	\N	2020-10-12 14:40:17.706731
37062	DHIL	2020-10-02	127.4	135	127.4	133.84	13152	{}	\N	2020-10-12 14:40:17.711786
37063	DHIL	2020-10-05	134.5	138.06	134.5	138.06	7945	{}	\N	2020-10-12 14:40:17.716113
37064	DHIL	2020-10-06	138	138	132.56	132.93	12856	{}	\N	2020-10-12 14:40:17.720399
37065	DHIL	2020-10-07	134.88	137	134.05	136.25	12578	{}	\N	2020-10-12 14:40:17.724682
37066	DHIL	2020-10-08	137.97	145	137.97	143.5	17131	{}	\N	2020-10-12 14:40:17.728955
37067	DHIL	2020-10-09	143	148.63	143	148.3	14611	{}	\N	2020-10-12 14:40:17.734034
37068	DHIL	2020-10-12	147.15	147.15	145.99	147.05	3331	{}	\N	2020-10-12 14:40:17.739275
37069	DIN	2020-09-28	56.51	57.74	55.74	56.97	517261	{}	\N	2020-10-12 14:40:18.964192
37070	DIN	2020-09-29	56.88	57.24	54.15	55.52	728624	{}	\N	2020-10-12 14:40:18.971104
37071	DIN	2020-09-30	55.89	57.59	54.47	54.59	934686	{}	\N	2020-10-12 14:40:18.976769
37072	DIN	2020-10-01	55.25	56.26	54.4	55.46	512178	{}	\N	2020-10-12 14:40:18.981103
37073	DIN	2020-10-02	52.73	57.74	52.62	57.6	889517	{}	\N	2020-10-12 14:40:18.985059
37074	DIN	2020-10-05	58	58.94	56.73	57.6	611883	{}	\N	2020-10-12 14:40:18.989174
37075	DIN	2020-10-06	58.5	59.66	56.03	56.16	631511	{}	\N	2020-10-12 14:40:18.993336
37076	DIN	2020-10-07	57.7	58.21	56.05	56.78	813375	{}	\N	2020-10-12 14:40:18.99697
37077	DIN	2020-10-08	57.39	58.35	56.35	58.18	433352	{}	\N	2020-10-12 14:40:19.000882
37078	DIN	2020-10-09	58.6	61.0472	58.505	60.59	911186	{}	\N	2020-10-12 14:40:19.004795
37079	DIN	2020-10-12	60.43	60.8065	59.36	59.6907	77081	{}	\N	2020-10-12 14:40:19.00915
37080	DIV	2020-09-28	14.96	15.13	14.96	15.04	138423	{}	\N	2020-10-12 14:40:20.571753
37081	DIV	2020-09-29	15.05	15.05	14.8578	15.02	77002	{}	\N	2020-10-12 14:40:20.581093
37082	DIV	2020-09-30	15.05	15.22	15.031	15.12	159831	{}	\N	2020-10-12 14:40:20.587552
37083	DIV	2020-10-01	15.13	15.24	15.015	15.24	143988	{}	\N	2020-10-12 14:40:20.593515
37084	DIV	2020-10-02	15.01	15.39	15	15.36	114832	{}	\N	2020-10-12 14:40:20.598443
37085	DIV	2020-10-05	15.34	15.415	15.2499	15.4	133311	{}	\N	2020-10-12 14:40:20.602975
37086	DIV	2020-10-06	15.42	15.59	15.31	15.34	184156	{}	\N	2020-10-12 14:40:20.607156
37087	DIV	2020-10-07	15.42	15.51	15.38	15.48	298841	{}	\N	2020-10-12 14:40:20.611292
37088	DIV	2020-10-08	15.54	15.77	15.52	15.77	189093	{}	\N	2020-10-12 14:40:20.615221
37089	DIV	2020-10-09	15.82	15.88	15.7	15.72	357417	{}	\N	2020-10-12 14:40:20.619522
37090	DIV	2020-10-12	15.72	15.76	15.6863	15.753	81609	{}	\N	2020-10-12 14:40:20.623542
37091	FIS	2020-09-28	147.63	148.76	147.12	147.93	1723730	{}	\N	2020-10-12 14:40:21.760435
37092	FIS	2020-09-29	148.29	149.19	147.73	148.8	3837781	{}	\N	2020-10-12 14:40:21.769799
37093	FIS	2020-09-30	148.75	149.28	145.92	147.21	6409272	{}	\N	2020-10-12 14:40:21.775937
37094	FIS	2020-10-01	148.99	150.31	145.52	146.87	6894199	{}	\N	2020-10-12 14:40:21.781265
37095	FIS	2020-10-02	145.06	145.92	143.03	145.15	3822638	{}	\N	2020-10-12 14:40:21.786141
37096	FIS	2020-10-05	146.13	146.8858	144.24	145.57	4208229	{}	\N	2020-10-12 14:40:21.791004
37097	FIS	2020-10-06	145.57	147.98	144.02	144.37	5969879	{}	\N	2020-10-12 14:40:21.795314
37098	FIS	2020-10-07	145.74	146.27	144.02	144.6	6302973	{}	\N	2020-10-12 14:40:21.799578
37099	FIS	2020-10-08	146.28	146.91	145.17	146.6	3949182	{}	\N	2020-10-12 14:40:21.804079
37100	FIS	2020-10-09	147.53	149.17	146.83	149.03	5473214	{}	\N	2020-10-12 14:40:21.808931
37101	FIS	2020-10-12	149.6	149.6	148.28	148.9265	731075	{}	\N	2020-10-12 14:40:21.813132
37102	DDT	2020-09-28	21.29	21.59	21.2417	21.59	26794	{}	\N	2020-10-12 14:40:23.362241
37103	DDT	2020-09-29	21.59	21.73	21.13	21.57	48508	{}	\N	2020-10-12 14:40:23.370657
37104	DDT	2020-09-30	21.49	22.53	21.49	22.05	120593	{}	\N	2020-10-12 14:40:23.376768
37105	DDT	2020-10-01	22.0101	23.2455	22.0101	22.97	46381	{}	\N	2020-10-12 14:40:23.381485
37106	DDT	2020-10-02	22.64	23.25	22.22	23	54280	{}	\N	2020-10-12 14:40:23.386275
37107	DDT	2020-10-05	22.83	23.977	22.494	23.56	60496	{}	\N	2020-10-12 14:40:23.391368
37108	DDT	2020-10-06	23.71	23.71	22.8583	23.03	51228	{}	\N	2020-10-12 14:40:23.396026
37109	DDT	2020-10-07	23.01	23.6592	23.01	23.65	25794	{}	\N	2020-10-12 14:40:23.400449
37110	DDT	2020-10-08	23.8998	24	23.755	24	33856	{}	\N	2020-10-12 14:40:23.405492
37111	DDT	2020-10-09	24	24	22.52	23.1	72276	{}	\N	2020-10-12 14:40:23.410322
37112	DDT	2020-10-12	23.47	24.44	23.47	24.28	40478	{}	\N	2020-10-12 14:40:23.414593
37113	DFS	2020-09-28	55.79	57.28	55.47	56.46	3862875	{}	\N	2020-10-12 14:40:25.812484
37114	DFS	2020-09-29	56.26	57.03	55.25	55.64	4556381	{}	\N	2020-10-12 14:40:25.823853
37115	DFS	2020-09-30	56.1	58.63	56.1	57.78	8056129	{}	\N	2020-10-12 14:40:25.831323
37116	DFS	2020-10-01	57.63	58.93	57.24	58.91	5142438	{}	\N	2020-10-12 14:40:25.836422
37117	DFS	2020-10-02	57.56	61.11	56.94	60.92	6354226	{}	\N	2020-10-12 14:40:25.841083
37118	DFS	2020-10-05	62.1	63.6	61.13	63.17	5645318	{}	\N	2020-10-12 14:40:25.84544
37119	DFS	2020-10-06	63.91	64.37	60.42	61.1	6219374	{}	\N	2020-10-12 14:40:25.84921
37120	DFS	2020-10-07	62.29	64.08	62.29	63.72	5290069	{}	\N	2020-10-12 14:40:25.853025
37121	DFS	2020-10-08	64.35	65.95	63.49	65.41	5803553	{}	\N	2020-10-12 14:40:25.856563
37122	DFS	2020-10-09	66.2	66.33	64.3873	64.89	4123243	{}	\N	2020-10-12 14:40:25.86015
37123	DFS	2020-10-12	65	65.21	64.555	64.94	571764	{}	\N	2020-10-12 14:40:25.863718
37124	DGRE	2020-09-28	24.0099	24.02	23.9119	24.02	11969	{}	\N	2020-10-12 14:40:28.494793
37125	DGRE	2020-09-29	23.9	24	23.8613	23.89	8540	{}	\N	2020-10-12 14:40:28.502094
37126	DGRE	2020-09-30	24.22	24.22	24.14	24.2	8915	{}	\N	2020-10-12 14:40:28.507894
37127	DGRE	2020-10-01	24.28	24.42	24.28	24.38	4220	{}	\N	2020-10-12 14:40:28.512497
37128	DGRE	2020-10-02	24.15	24.33	24.08	24.08	4263	{}	\N	2020-10-12 14:40:28.517005
37129	DGRE	2020-10-05	24.3834	24.4055	24.32	24.4055	23904	{}	\N	2020-10-12 14:40:28.521543
37130	DGRE	2020-10-06	24.5458	24.58	24.3576	24.3576	5585	{}	\N	2020-10-12 14:40:28.525759
37131	DGRE	2020-10-07	24.532	24.645	24.515	24.58	10246	{}	\N	2020-10-12 14:40:28.529521
37132	DGRE	2020-10-08	24.65	24.79	24.65	24.79	9408	{}	\N	2020-10-12 14:40:28.533348
37133	DGRE	2020-10-09	24.87	24.9694	24.87	24.92	6048	{}	\N	2020-10-12 14:40:28.538177
37134	DGRE	2020-10-12	24.975	24.975	24.94	24.9622	3155	{}	\N	2020-10-12 14:40:28.542479
37135	DIA	2020-09-28	275.26	277.12	274.72	275.9	2806624	{}	\N	2020-10-12 14:40:30.084505
37136	DIA	2020-09-29	275.6	275.98	273.29	274.42	1938492	{}	\N	2020-10-12 14:40:30.095157
37137	DIA	2020-09-30	275	280.15	275	277.5	3936535	{}	\N	2020-10-12 14:40:30.102254
37138	DIA	2020-10-01	279.33	280.35	276.57	278.14	2758650	{}	\N	2020-10-12 14:40:30.108268
37139	DIA	2020-10-02	274.17	278.51	273.6	276.8	4603026	{}	\N	2020-10-12 14:40:30.113678
37140	DIA	2020-10-05	278.79	281.59	278.75	281.51	1819739	{}	\N	2020-10-12 14:40:30.119358
37141	DIA	2020-10-06	282.54	283.52	277.24	277.72	4397746	{}	\N	2020-10-12 14:40:30.123996
37142	DIA	2020-10-07	280.28	283.67	280.23	282.92	2372008	{}	\N	2020-10-12 14:40:30.128504
37143	DIA	2020-10-08	284.52	284.68	282.69	284.29	2152345	{}	\N	2020-10-12 14:40:30.132989
37144	DIA	2020-10-09	285.61	286.81	284.41	285.93	2567619	{}	\N	2020-10-12 14:40:30.137675
37145	DIA	2020-10-12	286.95	287.98	286.68	287.6699	692712	{}	\N	2020-10-12 14:40:30.142296
37146	DISH	2020-09-28	29.16	30.03	29	29.875	2297031	{}	\N	2020-10-12 14:40:31.421869
37147	DISH	2020-09-29	29.71	30.0765	29.35	29.65	1449406	{}	\N	2020-10-12 14:40:31.430687
37148	DISH	2020-09-30	29.84	30.09	28.66	29.03	2308482	{}	\N	2020-10-12 14:40:31.436833
37149	DISH	2020-10-01	29.16	29.6	28.49	28.78	1806955	{}	\N	2020-10-12 14:40:31.442079
37150	DISH	2020-10-02	28.2	28.85	28.07	28.545	2217597	{}	\N	2020-10-12 14:40:31.446773
37151	DISH	2020-10-05	28.87	29.08	28.31	28.41	2528914	{}	\N	2020-10-12 14:40:31.451322
37152	DISH	2020-10-06	28.49	28.815	27.37	27.41	2028057	{}	\N	2020-10-12 14:40:31.455444
37153	DISH	2020-10-07	27.63	28.03	27.14	27.5	2889772	{}	\N	2020-10-12 14:40:31.459489
37154	DISH	2020-10-08	27.72	27.81	27.26	27.58	1660398	{}	\N	2020-10-12 14:40:31.463176
37155	DISH	2020-10-09	28.125	28.125	27.28	27.33	1739225	{}	\N	2020-10-12 14:40:31.467085
37156	DISH	2020-10-12	27.59	27.67	27.25	27.55	297534	{}	\N	2020-10-12 14:40:31.471069
37157	DJCI	2020-09-28	13.1422	13.22	13.1101	13.22	438	{}	\N	2020-10-12 14:40:32.628283
37158	DJCI	2020-09-29	13.075	13.075	13.06	13.06	510	{}	\N	2020-10-12 14:40:32.638539
37159	DJCI	2020-09-30	13.1549	13.1549	13.1549	13.1549	99	{}	\N	2020-10-12 14:40:32.646716
37160	DJCI	2020-10-01	13.04	13.0454	12.97	13.0454	6519	{}	\N	2020-10-12 14:40:32.653162
37161	DJCI	2020-10-02	13	13	12.9403	12.9403	416	{}	\N	2020-10-12 14:40:32.659649
37162	DJCI	2020-10-05	13.18	13.23	13.17	13.23	863	{}	\N	2020-10-12 14:40:32.664874
37163	DJCI	2020-10-06	13.38	13.38	13.24	13.24	1185	{}	\N	2020-10-12 14:40:32.67143
37164	DJCI	2020-10-07	13.36	13.36	13.36	13.36	102	{}	\N	2020-10-12 14:40:32.677124
37165	DJCI	2020-10-08	13.87	13.87	13.48	13.48	457	{}	\N	2020-10-12 14:40:32.68199
37166	DJCI	2020-10-09	13.62	13.65	13.59	13.62	10493	{}	\N	2020-10-12 14:40:32.687006
37167	DJCI	2020-10-12	13.71	13.71	13.71	13.71	202	{}	\N	2020-10-12 14:40:32.693292
37168	DLPNW	2020-09-28	0.06082	0.0617	0.0501	0.0501	4251	{}	\N	2020-10-12 14:40:34.497265
37169	DLPNW	2020-09-29	0.055101	0.0615	0.055	0.0615	3825	{}	\N	2020-10-12 14:40:34.505186
37170	DLPNW	2020-09-30	0.055	0.055	0.055	0.055	1400	{}	\N	2020-10-12 14:40:34.511367
37171	DLPNW	2020-10-01	0.08	0.08	0.0504	0.0507	41810	{}	\N	2020-10-12 14:40:34.516267
37172	DLPNW	2020-10-02	0.0536	0.0536	0.0535	0.0535	304	{}	\N	2020-10-12 14:40:34.521054
37173	DLPNW	2020-10-05	0.0525	0.0525	0.0525	0.0525	200	{}	\N	2020-10-12 14:40:34.525498
37174	DLPNW	2020-10-06	0.0525	0.0526	0.0525	0.0526	2434	{}	\N	2020-10-12 14:40:34.529805
37175	DLPNW	2020-10-07	0.0526	0.05978	0.0526	0.05978	282	{}	\N	2020-10-12 14:40:34.533779
37176	DLPNW	2020-10-09	0.0528	0.0528	0.0528	0.0528	3062	{}	\N	2020-10-12 14:40:34.538151
37177	DMRC	2020-09-28	17.29	18.01	17.13	17.45	58300	{}	\N	2020-10-12 14:40:35.854656
37178	DMRC	2020-09-29	17.75	23.82	17.59	21.9	803468	{}	\N	2020-10-12 14:40:35.865081
37179	DMRC	2020-09-30	21.63	22.9357	21.5801	22.33	236172	{}	\N	2020-10-12 14:40:35.871609
37180	DMRC	2020-10-01	22.56	28	22.54	27.47	601575	{}	\N	2020-10-12 14:40:35.877182
37181	DMRC	2020-10-02	27.24	28.34	25.3	26.89	267056	{}	\N	2020-10-12 14:40:35.882009
37182	DMRC	2020-10-05	27.01	28.94	26.12	26.29	139706	{}	\N	2020-10-12 14:40:35.886634
37183	DMRC	2020-10-06	27	29.45	27	28.37	245764	{}	\N	2020-10-12 14:40:35.89151
37184	DMRC	2020-10-07	28.75	31.47	28.437	30.5	261274	{}	\N	2020-10-12 14:40:35.895758
37185	DMRC	2020-10-08	30.95	31.7899	28.55	30.13	154687	{}	\N	2020-10-12 14:40:35.899256
37186	DMRC	2020-10-09	30.49	31.07	28.4	29.21	119966	{}	\N	2020-10-12 14:40:35.903089
37187	DMRC	2020-10-12	29.21	29.98	28.87	29.73	23988	{}	\N	2020-10-12 14:40:35.906865
37188	DEA	2020-09-28	22.82	23.27	22.5	23.15	534877	{}	\N	2020-10-12 14:40:36.931822
37189	DEA	2020-09-29	23.14	23.14	22.24	22.57	1522001	{}	\N	2020-10-12 14:40:36.943939
37190	DEA	2020-09-30	22.59	22.81	22.09	22.41	1593459	{}	\N	2020-10-12 14:40:36.950269
37191	DEA	2020-10-01	22.5	22.83	22.34	22.82	1039077	{}	\N	2020-10-12 14:40:36.95591
37192	DEA	2020-10-02	22.55	22.88	22.4	22.83	927968	{}	\N	2020-10-12 14:40:36.961206
37193	DEA	2020-10-05	22.93	23.02	22.465	22.75	1393225	{}	\N	2020-10-12 14:40:36.965624
37194	DEA	2020-10-06	22.84	22.9	22.42	22.65	1512659	{}	\N	2020-10-12 14:40:36.97013
37195	DEA	2020-10-07	22.65	22.74	22.375	22.46	1200536	{}	\N	2020-10-12 14:40:36.974321
37196	DEA	2020-10-08	22.6	22.84	22.44	22.6	760261	{}	\N	2020-10-12 14:40:36.978537
37197	DEA	2020-10-09	22.68	22.76	22.53	22.64	962787	{}	\N	2020-10-12 14:40:36.982948
37198	DEA	2020-10-12	22.7	22.84	22.43	22.78	99884	{}	\N	2020-10-12 14:40:36.986968
37199	DENN	2020-09-28	10.71	10.87	10.56	10.72	1244498	{}	\N	2020-10-12 14:40:38.182987
37200	DENN	2020-09-29	10.7	10.7	10.13	10.215	1173674	{}	\N	2020-10-12 14:40:38.191897
37201	DENN	2020-09-30	10.31	10.455	9.87	10.01	1579255	{}	\N	2020-10-12 14:40:38.196925
37202	DENN	2020-10-01	10.11	10.195	9.94	10.16	899076	{}	\N	2020-10-12 14:40:38.202103
37203	DENN	2020-10-02	9.8226	10.39	9.76	10.35	834239	{}	\N	2020-10-12 14:40:38.206911
37204	DENN	2020-10-05	10.48	10.5	10.05	10.15	769953	{}	\N	2020-10-12 14:40:38.211369
37205	DENN	2020-10-06	10.3	10.445	9.96	10.01	1352992	{}	\N	2020-10-12 14:40:38.215011
37206	DENN	2020-10-07	10.35	10.47	10.04	10.39	916802	{}	\N	2020-10-12 14:40:38.218822
37207	DENN	2020-10-08	10.5	10.58	10.22	10.41	1097752	{}	\N	2020-10-12 14:40:38.222793
37208	DENN	2020-10-09	10.45	10.515	10.33	10.49	588716	{}	\N	2020-10-12 14:40:38.226751
37209	DENN	2020-10-12	10.55	10.56	10.45	10.46	139266	{}	\N	2020-10-12 14:40:38.230435
37210	DEW	2020-09-28	37.85	37.99	37.759	37.91	6176	{}	\N	2020-10-12 14:40:40.067988
37211	DEW	2020-09-29	37.78	37.78	37.4883	37.6114	1661	{}	\N	2020-10-12 14:40:40.075894
37212	DEW	2020-09-30	37.705	37.995	37.705	37.995	2531	{}	\N	2020-10-12 14:40:40.081849
37213	DEW	2020-10-01	37.86	37.86	37.65	37.7599	1995	{}	\N	2020-10-12 14:40:40.087007
37214	DEW	2020-10-02	37.25	37.9571	37.25	37.82	18526	{}	\N	2020-10-12 14:40:40.092085
37215	DEW	2020-10-05	38.13	38.4	38.08	38.38	3615	{}	\N	2020-10-12 14:40:40.096336
37216	DEW	2020-10-06	38.54	38.7899	38.2174	38.2174	4796	{}	\N	2020-10-12 14:40:40.100187
37217	DEW	2020-10-07	38.47	38.7489	38.45	38.62	10405	{}	\N	2020-10-12 14:40:40.104155
37218	DEW	2020-10-08	38.89	39.0498	38.89	39.0498	3442	{}	\N	2020-10-12 14:40:40.108297
37219	DEW	2020-10-09	39.16	39.1937	39.01	39.01	5836	{}	\N	2020-10-12 14:40:40.112212
37220	DFFN	2020-09-28	0.83	0.83	0.79	0.8193	1072200	{}	\N	2020-10-12 14:40:41.717355
37221	DFFN	2020-09-29	0.8052	0.868	0.79	0.8439	928837	{}	\N	2020-10-12 14:40:41.723255
37222	DFFN	2020-09-30	0.8384	0.8495	0.8203	0.842	532085	{}	\N	2020-10-12 14:40:41.728279
40100	ENR	2020-09-28	39.98	40.42	39.57	39.64	558861	{}	\N	2020-10-12 14:47:43.54552
40101	ENR	2020-09-29	39.8	40.1	39.27	39.31	741303	{}	\N	2020-10-12 14:47:43.556623
40102	ENR	2020-09-30	39.2	39.85	38.87	39.14	1086044	{}	\N	2020-10-12 14:47:43.563665
40103	ENR	2020-10-01	39.21	40.48	38.95	40.24	1409682	{}	\N	2020-10-12 14:47:43.573552
40104	ENR	2020-10-02	39.69	41.05	39.56	40.46	1353922	{}	\N	2020-10-12 14:47:43.580619
40105	ENR	2020-10-05	40.9454	41.315	40.46	41.21	1207852	{}	\N	2020-10-12 14:47:43.588686
40106	ENR	2020-10-06	41.21	41.95	41.02	41.15	1180274	{}	\N	2020-10-12 14:47:43.595185
40107	ENR	2020-10-07	41.35	41.71	40.43	40.49	1066349	{}	\N	2020-10-12 14:47:43.601142
40108	ENR	2020-10-08	40.69	41.3	40.46	41.07	629911	{}	\N	2020-10-12 14:47:43.605794
40109	ENR	2020-10-09	41.14	41.58	40.52	41.22	752100	{}	\N	2020-10-12 14:47:43.61159
40110	ENR	2020-10-12	41.4	41.67	41.2	41.28	91782	{}	\N	2020-10-12 14:47:43.618314
40111	EPD	2020-09-28	15.89	16.21	15.715	16.02	7488521	{}	\N	2020-10-12 14:47:44.967924
40112	EPD	2020-09-29	16	16.03	15.76	15.93	12643149	{}	\N	2020-10-12 14:47:44.977138
40113	EPD	2020-09-30	15.93	16.23	15.69	15.79	17515474	{}	\N	2020-10-12 14:47:44.982163
40114	EPD	2020-10-01	15.76	15.95	15.65	15.67	14345679	{}	\N	2020-10-12 14:47:44.986938
40115	EPD	2020-10-02	15.5	16.09	15.35	16.04	15341669	{}	\N	2020-10-12 14:47:44.991707
40116	EPD	2020-10-05	16.15	16.455	16.02	16.45	12335329	{}	\N	2020-10-12 14:47:44.99616
40117	EPD	2020-10-06	16.52	16.8	16.2	16.39	22273358	{}	\N	2020-10-12 14:47:45.000292
40118	EPD	2020-10-07	16.4	16.59	16.35	16.4	13484963	{}	\N	2020-10-12 14:47:45.004331
40119	EPD	2020-10-08	16.46	17.25	16.45	17.14	14148593	{}	\N	2020-10-12 14:47:45.008666
40120	EPD	2020-10-09	17.29	17.35	16.855	17.01	12540046	{}	\N	2020-10-12 14:47:45.012688
40121	EPD	2020-10-12	17.2	17.27	16.96	17.055	3141959	{}	\N	2020-10-12 14:47:45.017027
37223	DFFN	2020-10-01	0.85	0.85	0.81	0.82	553275	{}	\N	2020-10-12 14:40:41.735043
37224	DFFN	2020-10-02	0.801	0.8399	0.8	0.8222	766151	{}	\N	2020-10-12 14:40:41.742265
37225	DFFN	2020-10-05	0.83	0.8689	0.8238	0.863	618356	{}	\N	2020-10-12 14:40:41.746368
37226	DFFN	2020-10-06	0.88	0.949	0.8612	0.897	1531322	{}	\N	2020-10-12 14:40:41.750543
37227	DFFN	2020-10-07	0.93	0.94	0.9	0.9277	702898	{}	\N	2020-10-12 14:40:41.755345
37228	DFFN	2020-10-08	0.9304	0.931	0.9	0.92	444378	{}	\N	2020-10-12 14:40:41.759466
37229	DFFN	2020-10-09	0.9112	0.94	0.9104	0.926	464492	{}	\N	2020-10-12 14:40:41.763356
37230	DFFN	2020-10-12	0.9199	0.9199	0.8888	0.904	229060	{}	\N	2020-10-12 14:40:41.767317
37231	DFVS	2020-09-28	12.265	12.265	12.265	12.265	100	{}	\N	2020-10-12 14:40:42.768911
37232	DFVS	2020-09-29	12.015	12.015	12.015	12.015	100	{}	\N	2020-10-12 14:40:42.775764
37233	DFVS	2020-09-30	12.015	12.015	12.015	12.015	100	{}	\N	2020-10-12 14:40:42.780414
37234	DFVS	2020-10-01	12.015	12.015	12.015	12.015	50	{}	\N	2020-10-12 14:40:42.784363
37235	DFVS	2020-10-02	12.015	12.015	12.015	12.015	100	{}	\N	2020-10-12 14:40:42.788424
37236	DFVS	2020-10-05	12.015	12.015	12.015	12.015	100	{}	\N	2020-10-12 14:40:42.792127
37237	DFVS	2020-10-06	12.015	12.015	12.015	12.015	100	{}	\N	2020-10-12 14:40:42.795538
37238	DFVS	2020-10-07	12.02	13.4	12.02	13.18	1580	{}	\N	2020-10-12 14:40:42.798913
37239	DFVS	2020-10-08	12.675	12.675	12.675	12.675	192	{}	\N	2020-10-12 14:40:42.80299
37240	DFVS	2020-10-09	12.45	12.45	12.45	12.45	100	{}	\N	2020-10-12 14:40:42.807358
37241	DFVS	2020-10-12	12.4	12.44	12.4	12.44	300	{}	\N	2020-10-12 14:40:42.811718
37242	DGII	2020-09-28	14.66	15.42	14.66	15.24	178677	{}	\N	2020-10-12 14:40:43.995369
37243	DGII	2020-09-29	15.46	16.1	15.4	15.985	266935	{}	\N	2020-10-12 14:40:44.00592
37244	DGII	2020-09-30	15.8939	16.06	15.53	15.63	165554	{}	\N	2020-10-12 14:40:44.013236
37245	DGII	2020-10-01	15.8	15.8877	15.2867	15.65	124326	{}	\N	2020-10-12 14:40:44.018934
37246	DGII	2020-10-02	15.34	15.77	15.34	15.65	102119	{}	\N	2020-10-12 14:40:44.023975
37247	DGII	2020-10-05	15.81	16.3	15.81	16.19	110675	{}	\N	2020-10-12 14:40:44.028751
37248	DGII	2020-10-06	15.77	16.6192	15.77	16.09	127014	{}	\N	2020-10-12 14:40:44.032649
37249	DGII	2020-10-07	16.24	16.49	16	16.21	105670	{}	\N	2020-10-12 14:40:44.036641
37250	DGII	2020-10-08	16.34	16.59	16.21	16.37	95280	{}	\N	2020-10-12 14:40:44.040643
37251	DGII	2020-10-09	16.59	16.76	16.36	16.68	89138	{}	\N	2020-10-12 14:40:44.044515
37252	DGII	2020-10-12	16.73	16.82	16.65	16.82	3541	{}	\N	2020-10-12 14:40:44.048173
37253	DGRO	2020-09-28	39.58	39.9	39.53	39.74	868802	{}	\N	2020-10-12 14:40:47.885107
37254	DGRO	2020-09-29	39.77	39.8	39.33	39.46	1116871	{}	\N	2020-10-12 14:40:47.895335
37255	DGRO	2020-09-30	39.56	40.144	39.52	39.79	1994660	{}	\N	2020-10-12 14:40:47.901393
37256	DGRO	2020-10-01	40	40.12	39.6082	39.83	3002558	{}	\N	2020-10-12 14:40:47.907496
37257	DGRO	2020-10-02	39.26	39.995	39.25	39.77	1412717	{}	\N	2020-10-12 14:40:47.913002
37258	DGRO	2020-10-05	40.06	40.4656	40.046	40.44	2427639	{}	\N	2020-10-12 14:40:47.917784
37259	DGRO	2020-10-06	40.55	40.915	40	40.07	3576231	{}	\N	2020-10-12 14:40:47.922497
37260	DGRO	2020-10-07	40.44	40.82	40.42	40.73	1175247	{}	\N	2020-10-12 14:40:47.926697
37261	DGRO	2020-10-08	40.93	41.15	40.8605	41.15	1049074	{}	\N	2020-10-12 14:40:47.930752
37262	DGRO	2020-10-09	41.37	41.49	41.1636	41.31	1501885	{}	\N	2020-10-12 14:40:47.934765
37263	DGRO	2020-10-12	41.43	41.6189	41.416	41.6	260853	{}	\N	2020-10-12 14:40:47.93938
37264	DHI	2020-09-28	73	74.68	72	74.49	3183253	{}	\N	2020-10-12 14:40:48.930204
37265	DHI	2020-09-29	74.55	74.8	73.665	74.02	4509515	{}	\N	2020-10-12 14:40:48.941709
37266	DHI	2020-09-30	74.46	76.6	74.11	75.63	7405033	{}	\N	2020-10-12 14:40:48.948315
37267	DHI	2020-10-01	76.71	77.63	75.78	77.54	6884797	{}	\N	2020-10-12 14:40:48.954313
37268	DHI	2020-10-02	75.78	78.37	75.55	77.89	4878177	{}	\N	2020-10-12 14:40:48.959477
37269	DHI	2020-10-05	78.5	78.78	74.65	75.7	8391454	{}	\N	2020-10-12 14:40:48.964404
37270	DHI	2020-10-06	75.91	76.67	74.65	75.11	4666117	{}	\N	2020-10-12 14:40:48.969142
37271	DHI	2020-10-07	76.12	76.86	75.26	75.76	4065015	{}	\N	2020-10-12 14:40:48.973792
37272	DHI	2020-10-08	76.65	77.92	76.6223	77.71	3544695	{}	\N	2020-10-12 14:40:48.977985
37273	DHI	2020-10-09	78.49	78.97	77.53	78.09	3842541	{}	\N	2020-10-12 14:40:48.982018
37274	DHI	2020-10-12	78.61	78.8052	77.4146	78.15	661629	{}	\N	2020-10-12 14:40:48.986228
37275	DHY	2020-09-28	2.14	2.17	2.12	2.155	3842085	{}	\N	2020-10-12 14:40:49.990701
37276	DHY	2020-09-29	2.17	2.17	2.14	2.15	2767439	{}	\N	2020-10-12 14:40:50.001087
37277	DHY	2020-09-30	2.18	2.18	2.11	2.11	3942022	{}	\N	2020-10-12 14:40:50.008392
37278	DHY	2020-10-01	2.14	2.14	2.11	2.12	1712468	{}	\N	2020-10-12 14:40:50.014616
37279	DHY	2020-10-02	2.05	2.14	2.05	2.12	4497886	{}	\N	2020-10-12 14:40:50.020065
37280	DHY	2020-10-05	2.14	2.16	2.13	2.14	2085393	{}	\N	2020-10-12 14:40:50.025383
37281	DHY	2020-10-06	2.13	2.16	2.13	2.14	2143478	{}	\N	2020-10-12 14:40:50.030179
37282	DHY	2020-10-07	2.18	2.18	2.15	2.16	2811636	{}	\N	2020-10-12 14:40:50.034538
37283	DHY	2020-10-08	2.18	2.18	2.14	2.14	2755358	{}	\N	2020-10-12 14:40:50.038886
37284	DHY	2020-10-09	2.15	2.17	2.14	2.17	2369166	{}	\N	2020-10-12 14:40:50.043155
37285	DHY	2020-10-12	2.18	2.18	2.16	2.165	225534	{}	\N	2020-10-12 14:40:50.047307
37286	DIS	2020-09-28	125.75	126.86	125.37	125.99	6557396	{}	\N	2020-10-12 14:40:51.077505
37287	DIS	2020-09-29	125.92	126.29	123.675	125.4	14950261	{}	\N	2020-10-12 14:40:51.091176
37288	DIS	2020-09-30	123.24	125.55	123.24	124.08	26403522	{}	\N	2020-10-12 14:40:51.099178
37289	DIS	2020-10-01	124.94	125.47	122.42	123.31	17659020	{}	\N	2020-10-12 14:40:51.105177
37290	DIS	2020-10-02	121.99	123.6775	121.69	122.55	12987370	{}	\N	2020-10-12 14:40:51.110531
37291	DIS	2020-10-05	123.26	123.595	122.33	123.37	11992839	{}	\N	2020-10-12 14:40:51.114987
37292	DIS	2020-10-06	123.13	124.19	120.61	120.93	17265341	{}	\N	2020-10-12 14:40:51.119481
37293	DIS	2020-10-07	121.95	123.3098	120.93	122.91	20086300	{}	\N	2020-10-12 14:40:51.123455
37294	DIS	2020-10-08	123.58	123.75	122.4201	123.09	14528832	{}	\N	2020-10-12 14:40:51.127285
37295	DIS	2020-10-09	123.85	125.5495	123.33	124.98	22803854	{}	\N	2020-10-12 14:40:51.130931
37296	DIS	2020-10-12	124.9	125.81	123.73	125.07	4433927	{}	\N	2020-10-12 14:40:51.134819
37297	DIVC	2020-09-28	28.6055	28.6055	28.6055	28.6055	0	{}	\N	2020-10-12 14:40:52.590503
37298	DIVC	2020-09-29	28.3297	28.3297	28.3297	28.3297	0	{}	\N	2020-10-12 14:40:52.597873
37299	DIVC	2020-09-30	28.4579	28.4579	28.4579	28.4579	0	{}	\N	2020-10-12 14:40:52.604379
37300	DIVC	2020-10-01	28.4924	28.4924	28.4924	28.4924	0	{}	\N	2020-10-12 14:40:52.609945
37301	DIVC	2020-10-02	29.3336	29.3336	29.3336	29.3336	4	{}	\N	2020-10-12 14:40:52.614871
37302	DIVC	2020-10-05	30.055	30.055	30.055	30.055	3	{}	\N	2020-10-12 14:40:52.619063
37303	DIVC	2020-10-06	30.132	30.132	30.132	30.132	1	{}	\N	2020-10-12 14:40:52.623267
37304	DIVC	2020-10-07	30.8	30.8	30.8	30.8	0	{}	\N	2020-10-12 14:40:52.627651
37305	DIVC	2020-10-08	31.755	31.755	31.755	31.755	0	{}	\N	2020-10-12 14:40:52.631738
37306	DIVC	2020-10-09	31.72	31.72	31.72	31.72	0	{}	\N	2020-10-12 14:40:52.636014
37307	FIT	2020-09-28	6.55	6.65	6.54	6.58	4386962	{}	\N	2020-10-12 14:40:53.539328
37308	FIT	2020-09-29	6.9	7.02	6.8427	6.96	57755203	{}	\N	2020-10-12 14:40:53.547619
37309	FIT	2020-09-30	6.99	7	6.95	6.96	15666534	{}	\N	2020-10-12 14:40:53.553773
37310	FIT	2020-10-01	7	7.02	6.98	6.98	21465403	{}	\N	2020-10-12 14:40:53.559067
37311	FIT	2020-10-02	6.9	6.98	6.87	6.92	14729107	{}	\N	2020-10-12 14:40:53.563947
37312	FIT	2020-10-05	6.92	6.967	6.87	6.89	9092966	{}	\N	2020-10-12 14:40:53.568331
37313	FIT	2020-10-06	6.88	6.92	6.85	6.85	5435265	{}	\N	2020-10-12 14:40:53.573
37314	FIT	2020-10-07	6.85	6.95	6.85	6.91	10571686	{}	\N	2020-10-12 14:40:53.57738
37315	FIT	2020-10-08	6.91	6.96	6.865	6.87	8097554	{}	\N	2020-10-12 14:40:53.581768
37316	FIT	2020-10-09	6.91	6.95	6.85	6.86	7590374	{}	\N	2020-10-12 14:40:53.586082
37317	FIT	2020-10-12	6.89	6.96	6.89	6.9599	868802	{}	\N	2020-10-12 14:40:53.590824
37318	DFJ	2020-09-28	70.89	70.9177	70.76	70.76	4931	{}	\N	2020-10-12 14:40:57.537042
37319	DFJ	2020-09-29	70.865	70.87	70.572	70.572	2888	{}	\N	2020-10-12 14:40:57.546275
37320	DFJ	2020-09-30	69.69	70.15	69.69	69.95	11512	{}	\N	2020-10-12 14:40:57.552788
37321	DFJ	2020-10-01	69.76	69.885	69.7	69.83	7282	{}	\N	2020-10-12 14:40:57.558059
37322	DFJ	2020-10-02	68.87	69.4149	68.87	69.4149	16638	{}	\N	2020-10-12 14:40:57.562638
37323	DFJ	2020-10-05	70.06	70.32	70.0595	70.1	9298	{}	\N	2020-10-12 14:40:57.566991
37324	DFJ	2020-10-06	70.151	70.33	69.9332	69.9332	4775	{}	\N	2020-10-12 14:40:57.571413
37325	DFJ	2020-10-07	70.23	70.235	70.0966	70.1501	3157	{}	\N	2020-10-12 14:40:57.575093
37326	DFJ	2020-10-08	70.485	70.485	70.18	70.21	7893	{}	\N	2020-10-12 14:40:57.578552
37327	DFJ	2020-10-09	70.12	70.29	70.12	70.26	5492	{}	\N	2020-10-12 14:40:57.582049
37328	DFJ	2020-10-12	70.28	70.28	70.12	70.1733	9588	{}	\N	2020-10-12 14:40:57.585825
37329	DGL	2020-09-28	54.89	55.195	54.7254	55.19	103078	{}	\N	2020-10-12 14:40:58.543737
37330	DGL	2020-09-29	55.315	55.675	55.315	55.57	1900	{}	\N	2020-10-12 14:40:58.55346
37331	DGL	2020-09-30	55.29	55.77	55.2498	55.36	13118	{}	\N	2020-10-12 14:40:58.558458
37332	DGL	2020-10-01	55.86	56.08	55.72	56.02	19100	{}	\N	2020-10-12 14:40:58.562874
37333	DGL	2020-10-02	55.91	55.91	55.78	55.81	3063	{}	\N	2020-10-12 14:40:58.566921
37334	DGL	2020-10-05	55.9	56.28	55.9	56.05	21791	{}	\N	2020-10-12 14:40:58.571212
37335	DGL	2020-10-06	56.31	56.31	55.32	55.32	22173	{}	\N	2020-10-12 14:40:58.575185
37336	DGL	2020-10-07	55.39	55.39	55.08	55.31	9108	{}	\N	2020-10-12 14:40:58.578824
37337	DGL	2020-10-08	55.65	55.65	55.24	55.56	202993	{}	\N	2020-10-12 14:40:58.582476
37338	DGL	2020-10-09	56.32	56.62	56.32	56.62	39214	{}	\N	2020-10-12 14:40:58.586212
37339	DGL	2020-10-12	56.39	56.39	56.33	56.33	2464	{}	\N	2020-10-12 14:40:58.590455
37340	DHS	2020-09-28	62.1	62.794	62.1	62.46	50273	{}	\N	2020-10-12 14:41:00.565649
37341	DHS	2020-09-29	62.48	62.49	61.72	62	57473	{}	\N	2020-10-12 14:41:00.577957
37342	DHS	2020-09-30	62.18	62.8865	62.08	62.49	51469	{}	\N	2020-10-12 14:41:00.584622
37343	DHS	2020-10-01	62.6	62.77	62.0964	62.5	44980	{}	\N	2020-10-12 14:41:00.591077
37344	DHS	2020-10-02	61.75	63.03	61.75	62.85	43380	{}	\N	2020-10-12 14:41:00.5954
37345	DHS	2020-10-05	63.12	63.71	63	63.6301	34002	{}	\N	2020-10-12 14:41:00.600469
37346	DHS	2020-10-06	63.81	64.41	63.24	63.28	146893	{}	\N	2020-10-12 14:41:00.605157
37347	DHS	2020-10-07	63.7	64.295	63.7	64.1801	43068	{}	\N	2020-10-12 14:41:00.609354
37348	DHS	2020-10-08	64.4	65.1348	64.4	65.075	39488	{}	\N	2020-10-12 14:41:00.613287
37349	DHS	2020-10-09	65.39	65.5	64.9	64.92	47381	{}	\N	2020-10-12 14:41:00.617222
37350	DHS	2020-10-12	65.06	65.1742	64.82	65.155	16142	{}	\N	2020-10-12 14:41:00.621371
37351	DIAL	2020-09-28	21.48	21.5	21.436	21.48	77458	{}	\N	2020-10-12 14:41:01.982127
37352	DIAL	2020-09-29	21.5	21.56	21.48	21.5	215905	{}	\N	2020-10-12 14:41:01.99185
37353	DIAL	2020-09-30	21.56	21.61	21.4601	21.51	652211	{}	\N	2020-10-12 14:41:01.99736
37354	DIAL	2020-10-01	21.49	21.6	21.44	21.53	219780	{}	\N	2020-10-12 14:41:02.002263
37355	DIAL	2020-10-02	21.49	21.5	21.4301	21.49	129651	{}	\N	2020-10-12 14:41:02.006901
37356	DIAL	2020-10-05	21.54	21.54	21.4638	21.51	200517	{}	\N	2020-10-12 14:41:02.01092
37357	DIAL	2020-10-06	21.53	21.62	21.49	21.55	209445	{}	\N	2020-10-12 14:41:02.014814
37358	DIAL	2020-10-07	21.54	21.57	21.47	21.5518	179593	{}	\N	2020-10-12 14:41:02.018783
37359	DIAL	2020-10-08	21.58	21.72	21.55	21.63	231084	{}	\N	2020-10-12 14:41:02.022797
37360	DIAL	2020-10-09	21.59	21.63	21.56	21.56	177263	{}	\N	2020-10-12 14:41:02.026628
37361	DIAL	2020-10-12	21.65	21.6599	21.619	21.6486	30081	{}	\N	2020-10-12 14:41:02.030291
37362	DISCA	2020-09-28	21.56	22.12	21.56	21.95	2995839	{}	\N	2020-10-12 14:41:03.580301
37363	DISCA	2020-09-29	21.93	22.13	21.585	21.75	2203596	{}	\N	2020-10-12 14:41:03.588158
37364	DISCA	2020-09-30	21.89	22.3	21.68	21.77	2561687	{}	\N	2020-10-12 14:41:03.594136
37365	DISCA	2020-10-01	21.89	22.02	21.24	21.51	3447742	{}	\N	2020-10-12 14:41:03.59915
37366	DISCA	2020-10-02	21.15	21.6	21.055	21.5	2365369	{}	\N	2020-10-12 14:41:03.604323
37367	DISCA	2020-10-05	21.96	22.13	21.6211	21.91	2117588	{}	\N	2020-10-12 14:41:03.60929
37368	DISCA	2020-10-06	22.05	22.115	21.491	21.52	2230762	{}	\N	2020-10-12 14:41:03.613736
37369	DISCA	2020-10-07	21.45	22.335	21.41	22.19	2530710	{}	\N	2020-10-12 14:41:03.618143
37370	DISCA	2020-10-08	22.33	22.79	22.28	22.77	3078304	{}	\N	2020-10-12 14:41:03.622885
37371	DISCA	2020-10-09	22.85	23.05	22.23	22.24	2948296	{}	\N	2020-10-12 14:41:03.627396
37372	DISCA	2020-10-12	22.17	22.23	21.94	22.065	332756	{}	\N	2020-10-12 14:41:03.63172
37373	DIVA	2020-09-28	20.3449	20.3449	20.3449	20.3449	37	{}	\N	2020-10-12 14:41:05.393252
37374	DIVA	2020-09-29	20.13	20.13	20.0993	20.0993	801	{}	\N	2020-10-12 14:41:05.401271
37375	DIVA	2020-09-30	20.1487	20.1487	20.1487	20.1487	6	{}	\N	2020-10-12 14:41:05.412729
37376	DIVA	2020-10-01	20.11	20.11	19.96	19.96	1124	{}	\N	2020-10-12 14:41:05.417713
37377	DIVA	2020-10-02	20.33	20.33	20.33	20.33	13	{}	\N	2020-10-12 14:41:05.421918
37378	DIVA	2020-10-05	20.53	20.53	20.53	20.53	109	{}	\N	2020-10-12 14:41:05.425984
37379	DIVA	2020-10-06	20.5066	20.5066	20.5066	20.5066	6	{}	\N	2020-10-12 14:41:05.430064
37380	DIVA	2020-10-07	20.74	20.74	20.6996	20.6996	386	{}	\N	2020-10-12 14:41:05.433904
37381	DIVA	2020-10-08	20.8699	20.8851	20.8699	20.8851	205	{}	\N	2020-10-12 14:41:05.438199
37382	DIVA	2020-10-09	20.74	20.74	20.74	20.74	7	{}	\N	2020-10-12 14:41:05.442467
37383	DJPY	2020-09-28	20.1953	20.1953	20.1953	20.1953	2	{}	\N	2020-10-12 14:41:07.990691
37384	DJPY	2020-09-29	20.3295	20.3295	20.3295	20.3295	2	{}	\N	2020-10-12 14:41:07.998756
37385	DJPY	2020-09-30	20.176	20.176	20.176	20.176	1	{}	\N	2020-10-12 14:41:08.00498
37386	DJPY	2020-10-01	20.2424	20.2424	20.2424	20.2424	0	{}	\N	2020-10-12 14:41:08.010368
37387	DJPY	2020-10-02	20.1204	20.1204	20.1204	20.1204	11	{}	\N	2020-10-12 14:41:08.015125
37388	DJPY	2020-10-05	20.4095	20.4095	20.4095	20.4095	6	{}	\N	2020-10-12 14:41:08.019143
37389	DJPY	2020-10-06	20.2386	20.2386	20.2386	20.2386	4	{}	\N	2020-10-12 14:41:08.023417
37390	DJPY	2020-10-07	20.6117	20.6117	20.6117	20.6117	0	{}	\N	2020-10-12 14:41:08.027659
37391	DJPY	2020-10-08	20.58	20.58	20.58	20.58	1033	{}	\N	2020-10-12 14:41:08.031815
37392	DJPY	2020-10-09	20.2862	20.2862	20.2862	20.2862	40	{}	\N	2020-10-12 14:41:08.035722
37393	DJPY	2020-10-12	20.0587	20.0587	20.0587	20.0587	302	{}	\N	2020-10-12 14:41:08.040174
37394	DLHC	2020-09-28	7.51	7.6	7.47	7.52	20123	{}	\N	2020-10-12 14:41:11.34216
37395	DLHC	2020-09-29	7.59	7.6	7.46	7.48	8164	{}	\N	2020-10-12 14:41:11.351714
37396	DLHC	2020-09-30	7.46	7.5369	7.2	7.25	16979	{}	\N	2020-10-12 14:41:11.358614
37397	DLHC	2020-10-01	7.32	7.56	7.32	7.4	16331	{}	\N	2020-10-12 14:41:11.364505
37398	DLHC	2020-10-02	7.52	7.89	7.4	7.89	15170	{}	\N	2020-10-12 14:41:11.371018
37399	DLHC	2020-10-05	7.9	8.9885	7.89	8.72	75322	{}	\N	2020-10-12 14:41:11.376128
37400	DLHC	2020-10-06	8.66	9.4	8.66	9.12	63951	{}	\N	2020-10-12 14:41:11.380955
37401	DLHC	2020-10-07	9.32	9.32	8.71	8.87	40741	{}	\N	2020-10-12 14:41:11.385512
37402	DLHC	2020-10-08	8.92	9.225	8.4	9.2	39420	{}	\N	2020-10-12 14:41:11.389994
37403	DLHC	2020-10-09	9.2	9.22	9	9.13	27500	{}	\N	2020-10-12 14:41:11.394176
37404	DLHC	2020-10-12	9.2	9.33	9.1595	9.25	18620	{}	\N	2020-10-12 14:41:11.398645
37405	DLTR	2020-09-28	90.085	91.3845	89.75	90.94	1612706	{}	\N	2020-10-12 14:41:13.139622
37406	DLTR	2020-09-29	90.9	91.2	89.81	89.87	1162152	{}	\N	2020-10-12 14:41:13.14813
37407	DLTR	2020-09-30	90.28	92.32	90.26	91.34	1761248	{}	\N	2020-10-12 14:41:13.15459
37408	DLTR	2020-10-01	91.65	93.38	91.59	91.87	1632930	{}	\N	2020-10-12 14:41:13.159989
37409	DLTR	2020-10-02	90.65	91.92	90.17	90.76	1678267	{}	\N	2020-10-12 14:41:13.16486
37410	DLTR	2020-10-05	91.22	91.6942	89.4507	89.98	2191789	{}	\N	2020-10-12 14:41:13.169238
37411	DLTR	2020-10-06	90.61	92.45	88.12	88.68	1787257	{}	\N	2020-10-12 14:41:13.17373
37412	DLTR	2020-10-07	89.84	90.925	89.45	90.55	1720545	{}	\N	2020-10-12 14:41:13.178076
37413	DLTR	2020-10-08	91	91.81	90.4307	91.53	1526381	{}	\N	2020-10-12 14:41:13.182255
37414	DLTR	2020-10-09	92.07	94.73	91.82	93.64	1968131	{}	\N	2020-10-12 14:41:13.186682
37415	DLTR	2020-10-12	94.28	95.2	93.765	94.75	406572	{}	\N	2020-10-12 14:41:13.191176
37416	FIV	2020-09-28	8.56	8.56	8.42	8.49	315370	{}	\N	2020-10-12 14:41:14.122348
37417	FIV	2020-09-29	8.47	8.47	8.42	8.43	318989	{}	\N	2020-10-12 14:41:14.132204
37418	FIV	2020-09-30	8.4	8.59	8.4	8.44	1303900	{}	\N	2020-10-12 14:41:14.140618
37419	FIV	2020-10-01	8.44	8.5	8.44	8.44	108200	{}	\N	2020-10-12 14:41:14.146626
37420	FIV	2020-10-02	8.42	8.5	8.42	8.47	103394	{}	\N	2020-10-12 14:41:14.152051
37421	FIV	2020-10-05	8.47	8.5	8.44	8.46	168637	{}	\N	2020-10-12 14:41:14.157945
37422	FIV	2020-10-06	8.51	8.51	8.45	8.46	216239	{}	\N	2020-10-12 14:41:14.162593
37423	FIV	2020-10-07	8.49	8.52	8.47	8.47	173940	{}	\N	2020-10-12 14:41:14.166849
37424	FIV	2020-10-08	8.48	8.55	8.48	8.51	222670	{}	\N	2020-10-12 14:41:14.171143
37425	FIV	2020-10-09	8.49	8.59	8.49	8.54	144508	{}	\N	2020-10-12 14:41:14.175233
37426	FIV	2020-10-12	8.54	8.5603	8.53	8.5603	4290	{}	\N	2020-10-12 14:41:14.179277
37427	DG	2020-09-28	208.72	210.89	206.1901	209.8	1552065	{}	\N	2020-10-12 14:41:15.157454
37428	DG	2020-09-29	209.5	211.98	209.25	209.64	2581498	{}	\N	2020-10-12 14:41:15.167867
37429	DG	2020-09-30	210.33	211.64	208.35	209.62	3247515	{}	\N	2020-10-12 14:41:15.175497
37430	DG	2020-10-01	211.26	213.92	210.97	211.52	3557341	{}	\N	2020-10-12 14:41:15.181154
37431	DG	2020-10-02	209.79	212.68	209.555	211.52	2529248	{}	\N	2020-10-12 14:41:15.186242
37432	DG	2020-10-05	212	213.58	210.82	213.19	2635351	{}	\N	2020-10-12 14:41:15.191273
37433	DG	2020-10-06	214.5	216.48	211.57	212.29	3153402	{}	\N	2020-10-12 14:41:15.195726
37434	DG	2020-10-07	213.87	215.73	213.6	214.53	3109188	{}	\N	2020-10-12 14:41:15.199827
37435	DG	2020-10-08	216.66	219.4	215.9	218.88	2813307	{}	\N	2020-10-12 14:41:15.204046
37436	DG	2020-10-09	219.85	222.78	219.72	220.98	2564826	{}	\N	2020-10-12 14:41:15.208158
37437	DG	2020-10-12	222.12	222.505	220.37	220.77	450497	{}	\N	2020-10-12 14:41:15.212133
37438	DGLY	2020-09-28	2.17	2.2	2	2.06	2614605	{}	\N	2020-10-12 14:41:16.654728
37439	DGLY	2020-09-29	2.02	2.29	2.01	2.2	1912637	{}	\N	2020-10-12 14:41:16.662461
37440	DGLY	2020-09-30	2.18	2.24	2.08	2.13	855891	{}	\N	2020-10-12 14:41:16.668791
37441	DGLY	2020-10-01	2.2	2.33	2.08	2.23	2537175	{}	\N	2020-10-12 14:41:16.674022
37442	DGLY	2020-10-02	2.1	2.22	2.1	2.14	1062648	{}	\N	2020-10-12 14:41:16.678855
37443	DGLY	2020-10-05	2.15	2.38	2.13	2.19	2795404	{}	\N	2020-10-12 14:41:16.683361
37444	DGLY	2020-10-06	2.16	2.25	2.12	2.16	698883	{}	\N	2020-10-12 14:41:16.688018
37445	DGLY	2020-10-07	2.16	2.42	2.16	2.25	2262285	{}	\N	2020-10-12 14:41:16.692478
37446	DGLY	2020-10-08	2.29	2.37	2.2607	2.31	1211910	{}	\N	2020-10-12 14:41:16.696701
37447	DGLY	2020-10-09	2.27	2.3493	2.27	2.3	539758	{}	\N	2020-10-12 14:41:16.701943
37448	DGLY	2020-10-12	2.31	2.33	2.26	2.27	261596	{}	\N	2020-10-12 14:41:16.706723
37449	DHR	2020-09-28	208.41	211.33	207.79	209.79	2109068	{}	\N	2020-10-12 14:41:18.328433
37450	DHR	2020-09-29	210	212.24	209.41	211.01	3464124	{}	\N	2020-10-12 14:41:18.340068
37451	DHR	2020-09-30	211.76	216.63	211.14	215.33	6263381	{}	\N	2020-10-12 14:41:18.348014
37452	DHR	2020-10-01	217.01	218.68	213.77	214.67	3964100	{}	\N	2020-10-12 14:41:18.357833
37453	DHR	2020-10-02	212.21	215.23	211.425	212.5	2814634	{}	\N	2020-10-12 14:41:18.362797
37454	DHR	2020-10-05	214.54	218.63	214.54	218.35	2519312	{}	\N	2020-10-12 14:41:18.36728
37455	DHR	2020-10-06	218.85	220.8139	217.107	218.24	3795694	{}	\N	2020-10-12 14:41:18.371727
37456	DHR	2020-10-07	219.52	223.2799	219.36	222.03	3431017	{}	\N	2020-10-12 14:41:18.375773
37457	DHR	2020-10-08	222.66	222.867	219.94	220.71	4262745	{}	\N	2020-10-12 14:41:18.379984
37458	DHR	2020-10-09	221.6	226.17	221.17	226.06	3208824	{}	\N	2020-10-12 14:41:18.384108
37459	DHR	2020-10-12	226.78	227.34	225.11	225.96	933329	{}	\N	2020-10-12 14:41:18.388224
37460	DISCB	2020-09-30	33.8147	33.8147	33.51	33.51	996	{}	\N	2020-10-12 14:41:19.837914
37461	DISCB	2020-10-06	33.5566	34.8328	33.51	33.51	393	{}	\N	2020-10-12 14:41:19.845573
37462	DISCB	2020-10-08	33.0501	33.0501	33.0501	33.0501	383	{}	\N	2020-10-12 14:41:19.851264
37463	DKS	2020-09-28	56.69	58.39	56.69	57.81	1971067	{}	\N	2020-10-12 14:41:21.304352
37464	DKS	2020-09-29	57.91	58.07	56.94	57.13	3145462	{}	\N	2020-10-12 14:41:21.315885
37465	DKS	2020-09-30	57.15	58.57	57.09	57.88	3440804	{}	\N	2020-10-12 14:41:21.322629
37466	DKS	2020-10-01	58.21	60.41	58.21	60.25	6949606	{}	\N	2020-10-12 14:41:21.328334
37467	DKS	2020-10-02	58.5	60.935	58.14	60.17	4026586	{}	\N	2020-10-12 14:41:21.333136
37468	DKS	2020-10-05	60.48	62.28	60.25	62.13	3954786	{}	\N	2020-10-12 14:41:21.338181
37469	DKS	2020-10-06	62.45	62.835	59.32	59.91	4873599	{}	\N	2020-10-12 14:41:21.342757
37470	DKS	2020-10-07	60.58	61.61	60.56	60.7	3063973	{}	\N	2020-10-12 14:41:21.347785
37471	DKS	2020-10-08	61.38	61.5	60.675	61.43	4419872	{}	\N	2020-10-12 14:41:21.351977
37472	DKS	2020-10-09	61.74	62.7	60.59	60.76	3242406	{}	\N	2020-10-12 14:41:21.356591
37473	DKS	2020-10-12	61.18	62.045	60.64	61.12	876172	{}	\N	2020-10-12 14:41:21.360866
37474	DLPH	2020-09-28	16.24	16.44	16.11	16.32	1291026	{}	\N	2020-10-12 14:41:22.719903
37475	DLPH	2020-09-29	16.45	16.55	16.32	16.34	3191294	{}	\N	2020-10-12 14:41:22.726961
37476	DLPH	2020-09-30	16.33	16.8	16.33	16.71	4110487	{}	\N	2020-10-12 14:41:22.733027
37477	DLPH	2020-10-01	16.97	17.2	16.83	17.02	19844733	{}	\N	2020-10-12 14:41:22.737846
37478	DMRI	2020-09-28	44.655	44.655	44.655	44.655	58	{}	\N	2020-10-12 14:41:24.123325
37479	DMRI	2020-09-29	44.72	44.72	44.72	44.72	120	{}	\N	2020-10-12 14:41:24.130407
37480	DMRI	2020-09-30	44.35	44.35	44.35	44.35	192	{}	\N	2020-10-12 14:41:24.136011
37481	DMRI	2020-10-01	44.65	44.65	44.65	44.65	161	{}	\N	2020-10-12 14:41:24.140865
37482	DMRI	2020-10-02	44.5254	44.62	44.3	44.62	1972	{}	\N	2020-10-12 14:41:24.145477
37483	DMRI	2020-10-05	45.035	45.035	45.035	45.035	132	{}	\N	2020-10-12 14:41:24.14944
37484	DMRI	2020-10-06	44.725	44.725	44.725	44.725	0	{}	\N	2020-10-12 14:41:24.153502
37485	DMRI	2020-10-07	44.7201	45	44.72	45	5553	{}	\N	2020-10-12 14:41:24.157646
37486	DMRI	2020-10-08	45.145	45.145	45.145	45.145	0	{}	\N	2020-10-12 14:41:24.161946
37487	DMRI	2020-10-09	45.3518	45.3518	45.3518	45.3518	36	{}	\N	2020-10-12 14:41:24.165974
37488	DNLI	2020-09-28	35.53	35.53	34.52	35.27	352726	{}	\N	2020-10-12 14:41:25.178729
37489	DNLI	2020-09-29	35.335	36.53	35.32	36.3	287574	{}	\N	2020-10-12 14:41:25.187984
37490	DNLI	2020-09-30	36.35	37.72	35.48	35.83	379671	{}	\N	2020-10-12 14:41:25.194862
37491	DNLI	2020-10-01	37.12	37.62	35.93	36.89	325278	{}	\N	2020-10-12 14:41:25.200941
37492	DNLI	2020-10-02	36.5	39.99	36.31	38.46	914559	{}	\N	2020-10-12 14:41:25.205763
37493	DNLI	2020-10-05	39.54	39.78	37.8816	38.04	778029	{}	\N	2020-10-12 14:41:25.210763
37494	DNLI	2020-10-06	37.87	38.52	37.01	37.45	417069	{}	\N	2020-10-12 14:41:25.215021
37495	DNLI	2020-10-07	38	39.54	37.75	39.16	904709	{}	\N	2020-10-12 14:41:25.219014
37496	DNLI	2020-10-08	39.4	40.35	38.87	40.16	658158	{}	\N	2020-10-12 14:41:25.223007
37497	DNLI	2020-10-09	40.27	41.82	40.11	41.56	395214	{}	\N	2020-10-12 14:41:25.227069
37498	DNLI	2020-10-12	42.19	43.22	41.56	42.9999	191307	{}	\N	2020-10-12 14:41:25.23139
37499	DOOR	2020-09-28	93.24	97.07	93.24	96.07	167675	{}	\N	2020-10-12 14:41:26.200747
37500	DOOR	2020-09-29	96.27	98.13	95.295	97.36	642829	{}	\N	2020-10-12 14:41:26.210822
37501	DOOR	2020-09-30	97.69	99.77	97.69	98.4	433644	{}	\N	2020-10-12 14:41:26.217546
37502	DOOR	2020-10-01	99.1	101.04	96.88	100.78	311204	{}	\N	2020-10-12 14:41:26.223216
37503	DOOR	2020-10-02	98.31	102.64	98.28	101.64	313759	{}	\N	2020-10-12 14:41:26.228034
37504	DOOR	2020-10-05	104.32	108.36	104.32	108.06	577889	{}	\N	2020-10-12 14:41:26.232862
37505	DOOR	2020-10-06	108.55	109.84	105.51	105.77	312961	{}	\N	2020-10-12 14:41:26.237192
37506	DOOR	2020-10-07	107.51	108.7247	104.42	106.21	302200	{}	\N	2020-10-12 14:41:26.24138
37507	DOOR	2020-10-08	107.6	107.91	106.33	107.25	204084	{}	\N	2020-10-12 14:41:26.245598
37508	DOOR	2020-10-09	108.49	109.59	105.35	106.15	227476	{}	\N	2020-10-12 14:41:26.249743
37509	DOOR	2020-10-12	106.91	107.72	105.35	107.29	72188	{}	\N	2020-10-12 14:41:26.253895
37510	DOX	2020-09-28	58.08	58.47	57.89	58.14	393990	{}	\N	2020-10-12 14:41:27.203156
37511	DOX	2020-09-29	57.8	58.1	57.26	57.45	404308	{}	\N	2020-10-12 14:41:27.211996
37512	DOX	2020-09-30	57.7	58	57.18	57.41	672497	{}	\N	2020-10-12 14:41:27.217418
37513	DOX	2020-10-01	57.7	58.02	56.95	57.54	619883	{}	\N	2020-10-12 14:41:27.222286
37514	DOX	2020-10-02	57.18	57.9	56.53	57.49	597526	{}	\N	2020-10-12 14:41:27.226802
37515	DOX	2020-10-05	57.96	58.33	57.44	57.88	684018	{}	\N	2020-10-12 14:41:27.231055
37516	DOX	2020-10-06	58	58.666	57.47	57.65	845406	{}	\N	2020-10-12 14:41:27.235048
37517	DOX	2020-10-07	57.82	58.69	57.82	58.28	617167	{}	\N	2020-10-12 14:41:27.238567
37518	DOX	2020-10-08	58.61	59.37	58.5	58.9	437059	{}	\N	2020-10-12 14:41:27.24202
37519	DOX	2020-10-09	58.99	59.573	58.615	59.4	434338	{}	\N	2020-10-12 14:41:27.245839
37520	DOX	2020-10-12	59.8	59.94	59.12	59.78	96452	{}	\N	2020-10-12 14:41:27.249382
37521	DRD	2020-09-28	11.67	11.88	11.45	11.71	321316	{}	\N	2020-10-12 14:41:28.411213
37522	DRD	2020-09-29	11.8	12.329	11.8	12.17	592158	{}	\N	2020-10-12 14:41:28.420018
37523	DRD	2020-09-30	12.03	12.08	11.61	11.85	527223	{}	\N	2020-10-12 14:41:28.426119
37524	DRD	2020-10-01	12.03	12.0666	11.77	11.83	412357	{}	\N	2020-10-12 14:41:28.431553
37525	DRD	2020-10-02	11.71	11.7899	11.3	11.4	708999	{}	\N	2020-10-12 14:41:28.436522
37526	DRD	2020-10-05	11.75	12.17	11.71	11.79	549719	{}	\N	2020-10-12 14:41:28.441058
37527	DRD	2020-10-06	11.61	11.75	11.02	11.06	703731	{}	\N	2020-10-12 14:41:28.445189
37528	DRD	2020-10-07	11.17	11.67	10.9	11.34	710908	{}	\N	2020-10-12 14:41:28.449091
37529	DRD	2020-10-08	11.46	11.69	11.36	11.53	291763	{}	\N	2020-10-12 14:41:28.453019
37530	DRD	2020-10-09	12	12.78	12	12.77	795081	{}	\N	2020-10-12 14:41:28.457208
37531	DRD	2020-10-12	12.73	12.75	12.375	12.42	183664	{}	\N	2020-10-12 14:41:28.4616
37532	DIVO	2020-09-28	30.2132	30.52	30.14	30.245	104841	{}	\N	2020-10-12 14:41:30.02753
37533	DIVO	2020-09-29	30.25	30.27	30.05	30.1499	31542	{}	\N	2020-10-12 14:41:30.033713
37534	DIVO	2020-09-30	30.27	30.71	30.27	30.49	59662	{}	\N	2020-10-12 14:41:30.038417
37535	DIVO	2020-10-01	30.87	30.87	30.4	30.51	50185	{}	\N	2020-10-12 14:41:30.042545
37536	DIVO	2020-10-02	30.35	30.55	30.155	30.44	53450	{}	\N	2020-10-12 14:41:30.046116
37537	DIVO	2020-10-05	30.63	30.85	30.53	30.85	14406	{}	\N	2020-10-12 14:41:30.0499
37538	DIVO	2020-10-06	31	31.011	30.49	30.52	35835	{}	\N	2020-10-12 14:41:30.053767
37539	DIVO	2020-10-07	30.8	31	30.71	30.94	90987	{}	\N	2020-10-12 14:41:30.057448
37540	DIVO	2020-10-08	30.99	31.05	30.8	31.03	59473	{}	\N	2020-10-12 14:41:30.061015
37541	DIVO	2020-10-09	31.03	31.27	31.03	31.2	48812	{}	\N	2020-10-12 14:41:30.064624
37542	DIVO	2020-10-12	31.5	31.5	31.27	31.3867	27515	{}	\N	2020-10-12 14:41:30.068632
37543	DL	2020-09-28	9.47	9.6981	9.35	9.61	26018	{}	\N	2020-10-12 14:41:31.044349
37544	DL	2020-09-29	9.7	9.7	9.56	9.56	9188	{}	\N	2020-10-12 14:41:31.049537
37545	DL	2020-09-30	9.6	9.8	9.53	9.58	28790	{}	\N	2020-10-12 14:41:31.054022
37546	DL	2020-10-01	9.64	9.83	9.571	9.75	35126	{}	\N	2020-10-12 14:41:31.05856
37547	DL	2020-10-02	9.4741	9.6468	9.4741	9.485	5512	{}	\N	2020-10-12 14:41:31.062708
37548	DL	2020-10-05	9.73	9.73	9.54	9.59	17654	{}	\N	2020-10-12 14:41:31.066881
37549	DL	2020-10-06	9.63	9.74	9.572	9.65	18122	{}	\N	2020-10-12 14:41:31.071071
37550	DL	2020-10-07	9.76	9.83	9.5	9.73	21164	{}	\N	2020-10-12 14:41:31.076158
37551	DL	2020-10-08	9.85	9.85	9.54	9.54	21692	{}	\N	2020-10-12 14:41:31.081141
37552	DL	2020-10-09	9.69	9.71	9.28	9.42	27580	{}	\N	2020-10-12 14:41:31.08514
37553	DL	2020-10-12	10.86	10.87	9.89	9.93	56378	{}	\N	2020-10-12 14:41:31.08987
37554	DLTH	2020-09-28	12.89	13.05	12.55	12.72	216102	{}	\N	2020-10-12 14:41:32.184601
37555	DLTH	2020-09-29	12.77	12.77	11.883	12.14	285248	{}	\N	2020-10-12 14:41:32.190408
37556	DLTH	2020-09-30	11.93	12.38	11.92	12.22	369245	{}	\N	2020-10-12 14:41:32.195062
37557	DLTH	2020-10-01	12.24	12.68	12.03	12.65	237736	{}	\N	2020-10-12 14:41:32.199189
37558	DLTH	2020-10-02	12.38	12.99	12.31	12.94	182905	{}	\N	2020-10-12 14:41:32.20314
37559	DLTH	2020-10-05	13.09	13.39	12.54	12.92	308458	{}	\N	2020-10-12 14:41:32.20725
37560	DLTH	2020-10-06	13	13.04	12.6	12.62	215274	{}	\N	2020-10-12 14:41:32.211549
37561	DLTH	2020-10-07	12.83	13.62	12.82	13.33	281641	{}	\N	2020-10-12 14:41:32.21572
37562	DLTH	2020-10-08	13.49	13.5	13.13	13.5	147038	{}	\N	2020-10-12 14:41:32.219579
37563	DLTH	2020-10-09	13.55	13.97	13.3	13.84	240799	{}	\N	2020-10-12 14:41:32.223725
37564	DLTH	2020-10-12	13.9	14.44	13.77	14.08	97497	{}	\N	2020-10-12 14:41:32.227595
37565	DNB	2020-09-28	24.72	25.28	24.58	25.26	730914	{}	\N	2020-10-12 14:41:33.233841
37566	DNB	2020-09-29	25.17	25.58	25.0301	25.24	2834407	{}	\N	2020-10-12 14:41:33.249127
37567	DNB	2020-09-30	25.3	25.72	25.11	25.66	2697472	{}	\N	2020-10-12 14:41:33.271959
37568	DNB	2020-10-01	25.8	25.96	25.22	25.39	2041168	{}	\N	2020-10-12 14:41:33.282783
37569	DNB	2020-10-02	25.2	25.65	24.81	25.54	1179323	{}	\N	2020-10-12 14:41:33.290666
37570	DNB	2020-10-05	25.7	26.4	25.7	26.38	2123197	{}	\N	2020-10-12 14:41:33.299967
37571	DNB	2020-10-06	25.98	26.5	25.91	25.91	2012946	{}	\N	2020-10-12 14:41:33.307714
37572	DNB	2020-10-07	26.09	26.38	25.87	25.92	2301318	{}	\N	2020-10-12 14:41:33.314858
37573	DNB	2020-10-08	26.51	28.42	26.51	28.13	4075469	{}	\N	2020-10-12 14:41:33.321935
37574	DNB	2020-10-09	27.8	28.64	27.26	28.16	1678372	{}	\N	2020-10-12 14:41:33.327604
37575	DNB	2020-10-12	28.4	28.4	28.1	28.1	49808	{}	\N	2020-10-12 14:41:33.333604
37576	DPW	2020-09-28	2.03	2.07	1.88	1.94	2777979	{}	\N	2020-10-12 14:41:38.090013
37577	DPW	2020-09-29	2	2.07	1.92	2	1214626	{}	\N	2020-10-12 14:41:38.098662
37578	DPW	2020-09-30	1.94	2.0302	1.94	2.01	498233	{}	\N	2020-10-12 14:41:38.103583
37579	DPW	2020-10-01	1.97	2	1.9	1.98	567319	{}	\N	2020-10-12 14:41:38.107992
37580	DPW	2020-10-02	2.24	2.44	2.06	2.26	28506547	{}	\N	2020-10-12 14:41:38.112032
37581	DPW	2020-10-05	2.06	2.13	1.97	2.04	2862922	{}	\N	2020-10-12 14:41:38.116197
37582	DPW	2020-10-06	2.09	2.28	2.04	2.16	4004261	{}	\N	2020-10-12 14:41:38.120549
37583	DPW	2020-10-07	2.21	2.7	2.12	2.55	11227754	{}	\N	2020-10-12 14:41:38.124645
37584	DPW	2020-10-08	2.88	3.03	2.46	2.57	31010297	{}	\N	2020-10-12 14:41:38.128577
37585	DPW	2020-10-09	2.3	2.45	2.23	2.35	3202956	{}	\N	2020-10-12 14:41:38.132608
37586	DPW	2020-10-12	2.27	2.32	2.17	2.23	679185	{}	\N	2020-10-12 14:41:38.136609
37587	DRNA	2020-09-28	18.87	18.995	17.83	18.085	353917	{}	\N	2020-10-12 14:41:39.033325
37588	DRNA	2020-09-29	18.03	18.49	17.82	18.39	234766	{}	\N	2020-10-12 14:41:39.03917
37589	DRNA	2020-09-30	18.42	18.67	17.82	17.99	360205	{}	\N	2020-10-12 14:41:39.043776
37590	DRNA	2020-10-01	18.1	18.71	17.76	18.69	539870	{}	\N	2020-10-12 14:41:39.047814
37591	DRNA	2020-10-02	18.29	18.86	18.02	18.08	425165	{}	\N	2020-10-12 14:41:39.051963
37592	DRNA	2020-10-05	18.22	19.48	18.21	19.45	567121	{}	\N	2020-10-12 14:41:39.055831
37593	DRNA	2020-10-06	19.53	19.63	18.34	18.48	452610	{}	\N	2020-10-12 14:41:39.059732
37594	DRNA	2020-10-07	18.54	19.4	18.5	19.32	458226	{}	\N	2020-10-12 14:41:39.063594
37595	DRNA	2020-10-08	19.74	20	19.229	19.71	414875	{}	\N	2020-10-12 14:41:39.067534
37596	DRNA	2020-10-09	19.75	20	19.435	19.88	496236	{}	\N	2020-10-12 14:41:39.071279
37597	DRNA	2020-10-12	19.97	19.97	19.5006	19.69	57743	{}	\N	2020-10-12 14:41:39.075082
37598	DSL	2020-09-28	15.7	15.93	15.7	15.87	394136	{}	\N	2020-10-12 14:41:40.593939
37599	DSL	2020-09-29	15.88	15.98	15.76	15.79	545946	{}	\N	2020-10-12 14:41:40.599852
37600	DSL	2020-09-30	15.89	16.07	15.83	16.02	917084	{}	\N	2020-10-12 14:41:40.604691
37601	DSL	2020-10-01	16.16	16.27	16.05	16.19	811186	{}	\N	2020-10-12 14:41:40.608899
37602	DSL	2020-10-02	16	16.1748	15.97	16.16	586424	{}	\N	2020-10-12 14:41:40.612839
37603	DSL	2020-10-05	16.25	16.28	16.1701	16.26	444698	{}	\N	2020-10-12 14:41:40.616721
37604	DSL	2020-10-06	16.29	16.3564	16.21	16.29	537134	{}	\N	2020-10-12 14:41:40.62061
37605	DSL	2020-10-07	16.43	16.43	16.29	16.32	484246	{}	\N	2020-10-12 14:41:40.624389
37606	DSL	2020-10-08	16.39	16.58	16.36	16.48	542820	{}	\N	2020-10-12 14:41:40.628174
37607	DSL	2020-10-09	16.54	16.5517	16.38	16.4	481083	{}	\N	2020-10-12 14:41:40.631766
37608	DSL	2020-10-12	16.46	16.4935	16.422	16.4394	83714	{}	\N	2020-10-12 14:41:40.635619
37609	DTEC	2020-09-28	37.99	38.2	37.88	38.09	18160	{}	\N	2020-10-12 14:41:41.815834
37610	DTEC	2020-09-29	38.32	38.4633	38.2544	38.31	14893	{}	\N	2020-10-12 14:41:41.827017
37611	DTEC	2020-09-30	38.52	38.8818	38.3	38.58	14482	{}	\N	2020-10-12 14:41:41.83401
37612	DTEC	2020-10-01	38.78	39.17	38.78	39.17	13938	{}	\N	2020-10-12 14:41:41.84
37613	DTEC	2020-10-02	38.79	38.94	38.4699	38.75	11955	{}	\N	2020-10-12 14:41:41.845002
37614	DTEC	2020-10-05	39.05	39.5055	39.05	39.5055	16203	{}	\N	2020-10-12 14:41:41.849806
37615	DTEC	2020-10-06	39.6	39.84	39.19	39.19	28954	{}	\N	2020-10-12 14:41:41.853931
37616	DTEC	2020-10-07	40	41.0397	39.49	39.965	30244	{}	\N	2020-10-12 14:41:41.857992
37617	DTEC	2020-10-08	40.47	40.59	40.24	40.34	26851	{}	\N	2020-10-12 14:41:41.861947
37618	DTEC	2020-10-09	40.52	41.085	40.52	40.87	19937	{}	\N	2020-10-12 14:41:41.865901
37619	DTEC	2020-10-12	41.17	41.17	40.925	40.9999	3830	{}	\N	2020-10-12 14:41:41.869714
37620	DWFI	2020-09-28	24.23	24.29	24.21	24.286	6619	{}	\N	2020-10-12 14:41:44.677635
37621	DWFI	2020-09-29	24.26	24.33	24.26	24.2968	3029	{}	\N	2020-10-12 14:41:44.685593
37622	DWFI	2020-09-30	24.2675	24.35	24.245	24.25	13219	{}	\N	2020-10-12 14:41:44.692972
37623	DWFI	2020-10-01	24.3	24.4099	24.3	24.4099	5379	{}	\N	2020-10-12 14:41:44.697627
37624	DWFI	2020-10-02	24.32	24.4	24.2901	24.37	14154	{}	\N	2020-10-12 14:41:44.701919
37625	DWFI	2020-10-05	24.39	24.46	24.38	24.46	7756	{}	\N	2020-10-12 14:41:44.706306
37626	DWFI	2020-10-06	24.39	24.49	24.39	24.485	8046	{}	\N	2020-10-12 14:41:44.709996
37627	DWFI	2020-10-07	24.4899	24.5435	24.45	24.505	47928	{}	\N	2020-10-12 14:41:44.713685
37628	DWFI	2020-10-08	24.54	24.59	24.54	24.5584	12836	{}	\N	2020-10-12 14:41:44.717288
37629	DWFI	2020-10-09	24.63	24.69	24.61	24.68	31950	{}	\N	2020-10-12 14:41:44.72064
37630	DWFI	2020-10-12	24.7234	24.7234	24.7012	24.7012	467	{}	\N	2020-10-12 14:41:44.724404
37631	DJCO	2020-09-28	251.25	252.01	245.5	247.11	4435	{}	\N	2020-10-12 14:41:46.074736
37632	DJCO	2020-09-29	245	247.94	243.98	247.94	1744	{}	\N	2020-10-12 14:41:46.084679
37633	DJCO	2020-09-30	245.01	246.75	242	242	839	{}	\N	2020-10-12 14:41:46.090273
37634	DJCO	2020-10-01	242.88	245.53	242	242	1508	{}	\N	2020-10-12 14:41:46.095418
37635	DJCO	2020-10-02	240.01	244.8	238	244.8	2407	{}	\N	2020-10-12 14:41:46.100239
37636	DJCO	2020-10-05	246.32	257.38	246.32	252.8	3509	{}	\N	2020-10-12 14:41:46.104261
37637	DJCO	2020-10-06	255	256.5	250.01	250.01	1889	{}	\N	2020-10-12 14:41:46.10847
37638	DJCO	2020-10-07	254.01	256	252.72	256	1386	{}	\N	2020-10-12 14:41:46.112609
37639	DJCO	2020-10-08	255.96	257.22	255.01	256	2423	{}	\N	2020-10-12 14:41:46.116614
37640	DJCO	2020-10-09	258.12	258.12	249.99	251.8	6274	{}	\N	2020-10-12 14:41:46.121317
37641	DJCO	2020-10-12	251	253	251	253	1097	{}	\N	2020-10-12 14:41:46.125847
37642	DLB	2020-09-28	65.65	66.33	65.48	65.99	264835	{}	\N	2020-10-12 14:41:47.059646
37643	DLB	2020-09-29	65.98	66.54	65.78	66.07	696634	{}	\N	2020-10-12 14:41:47.070572
37644	DLB	2020-09-30	66.39	66.88	65.95	66.28	732377	{}	\N	2020-10-12 14:41:47.077074
37645	DLB	2020-10-01	66.66	67.15	66.19	66.52	1051690	{}	\N	2020-10-12 14:41:47.082607
37646	DLB	2020-10-02	65.37	66.26	65.33	65.61	786616	{}	\N	2020-10-12 14:41:47.087294
37647	DLB	2020-10-05	65.99	66.47	65.58	65.93	448635	{}	\N	2020-10-12 14:41:47.091873
37648	DLB	2020-10-06	66.1	66.34	64.07	64.29	532397	{}	\N	2020-10-12 14:41:47.096115
37649	DLB	2020-10-07	64.73	67.1	64.635	66.86	1585676	{}	\N	2020-10-12 14:41:47.100037
37650	DLB	2020-10-08	67.12	68.72	66.93	68.71	1548032	{}	\N	2020-10-12 14:41:47.104079
37651	DLB	2020-10-09	69.05	69.238	68.32	68.5	916145	{}	\N	2020-10-12 14:41:47.108079
37652	DLB	2020-10-12	68.94	69.69	68.76	69.565	60978	{}	\N	2020-10-12 14:41:47.112083
37653	DLPN	2020-09-28	0.6626	0.67	0.65	0.6599	118146	{}	\N	2020-10-12 14:41:48.195087
37654	DLPN	2020-09-29	0.65	0.7393	0.6311	0.6837	1176938	{}	\N	2020-10-12 14:41:48.204119
37655	DLPN	2020-09-30	0.6837	0.7169	0.6501	0.69	424484	{}	\N	2020-10-12 14:41:48.21039
37656	DLPN	2020-10-01	0.7	0.7299	0.681	0.683	171553	{}	\N	2020-10-12 14:41:48.216071
37657	DLPN	2020-10-02	0.68	0.72	0.6783	0.6949	205496	{}	\N	2020-10-12 14:41:48.22078
37658	DLPN	2020-10-05	0.69	0.716793	0.6839	0.6856	111024	{}	\N	2020-10-12 14:41:48.225658
37659	DLPN	2020-10-06	0.68	0.7	0.6501	0.656	573543	{}	\N	2020-10-12 14:41:48.230151
37660	DLPN	2020-10-07	0.6601	0.6775	0.6601	0.6695	118569	{}	\N	2020-10-12 14:41:48.234164
37661	DLPN	2020-10-08	0.68	0.7354	0.671	0.7298	642755	{}	\N	2020-10-12 14:41:48.237825
37662	DLPN	2020-10-09	0.7014	0.729599	0.6785	0.7287	159494	{}	\N	2020-10-12 14:41:48.241842
37663	DLPN	2020-10-12	0.7287	0.7399	0.717	0.7399	186012	{}	\N	2020-10-12 14:41:48.246518
37664	DMF	2020-09-28	8.63	8.67	8.56	8.59	36859	{}	\N	2020-10-12 14:41:49.125948
37665	DMF	2020-09-29	8.57	8.62	8.54	8.62	31239	{}	\N	2020-10-12 14:41:49.135775
37666	DMF	2020-09-30	8.71	8.71	8.561	8.63	40571	{}	\N	2020-10-12 14:41:49.142314
37667	DMF	2020-10-01	8.58	8.58	8.42	8.485	114804	{}	\N	2020-10-12 14:41:49.148097
37668	DMF	2020-10-02	8.39	8.48	8.3301	8.48	51062	{}	\N	2020-10-12 14:41:49.15312
37669	DMF	2020-10-05	8.48	8.49	8.45	8.4502	41663	{}	\N	2020-10-12 14:41:49.157864
37670	DMF	2020-10-06	8.4	8.48	8.4	8.48	11274	{}	\N	2020-10-12 14:41:49.162067
37671	DMF	2020-10-07	8.44	8.48	8.44	8.47	24218	{}	\N	2020-10-12 14:41:49.16596
37672	DMF	2020-10-08	8.47	8.47	8.46	8.47	11604	{}	\N	2020-10-12 14:41:49.169885
37673	DMF	2020-10-09	8.46	8.48	8.4484	8.48	9036	{}	\N	2020-10-12 14:41:49.1741
37674	DMF	2020-10-12	8.49	8.49	8.46	8.49	10061	{}	\N	2020-10-12 14:41:49.178129
37675	DQ	2020-09-28	127.08	129.92	124.28	129.92	268415	{}	\N	2020-10-12 14:41:52.791869
37676	DQ	2020-09-29	129.95	133.65	127.87	130.55	856513	{}	\N	2020-10-12 14:41:52.803754
37677	DQ	2020-09-30	131.26	141.83	131.26	135.24	1289271	{}	\N	2020-10-12 14:41:52.810875
37678	DQ	2020-10-01	137.78	150.11	136.885	149.16	1289545	{}	\N	2020-10-12 14:41:52.816364
37679	DQ	2020-10-02	142.23	151.57	137.57	148.79	1134250	{}	\N	2020-10-12 14:41:52.821572
37680	DQ	2020-10-05	152.5	168.77	151.73	168	1348606	{}	\N	2020-10-12 14:41:52.826235
37681	DQ	2020-10-06	167.56	178.49	158.7	172	1029310	{}	\N	2020-10-12 14:41:52.830655
37682	DQ	2020-10-07	179.7	192.6671	173.41	188.78	1543159	{}	\N	2020-10-12 14:41:52.834586
37683	DQ	2020-10-08	193.3	199.67	182.31	186.54	1232353	{}	\N	2020-10-12 14:41:52.838504
37684	DQ	2020-10-09	182.41	198.85	176.81	196.53	977092	{}	\N	2020-10-12 14:41:52.842285
37685	DQ	2020-10-12	201.25	216.7	190.75	194.985	783926	{}	\N	2020-10-12 14:41:52.84607
37686	DRQ	2020-09-28	23.98	24.84	23.98	24.38	335474	{}	\N	2020-10-12 14:41:54.063863
37687	DRQ	2020-09-29	24.25	24.87	24.09	24.75	778192	{}	\N	2020-10-12 14:41:54.072134
37688	DRQ	2020-09-30	25.12	26.14	24.32	24.76	985389	{}	\N	2020-10-12 14:41:54.078576
37689	DRQ	2020-10-01	24.36	24.57	23.6	24.5	698251	{}	\N	2020-10-12 14:41:54.083956
37690	DRQ	2020-10-02	23.8	25.5	23.62	24.79	494468	{}	\N	2020-10-12 14:41:54.089035
37691	DRQ	2020-10-05	25.06	25.89	24.53	24.68	444594	{}	\N	2020-10-12 14:41:54.093692
37692	DRQ	2020-10-06	25.3	25.88	24.33	24.36	450869	{}	\N	2020-10-12 14:41:54.098305
37693	DRQ	2020-10-07	24.39	24.7	23.875	24.35	906371	{}	\N	2020-10-12 14:41:54.103429
37694	DRQ	2020-10-08	24.82	25.58	24.63	25.56	737495	{}	\N	2020-10-12 14:41:54.10806
37695	DRQ	2020-10-09	25.61	25.61	24.59	24.69	609385	{}	\N	2020-10-12 14:41:54.112307
37696	DRQ	2020-10-12	24.6	24.88	23.91	23.99	93618	{}	\N	2020-10-12 14:41:54.11683
37697	DSE	2020-09-28	0.3555	0.36	0.35	0.351	146173	{}	\N	2020-10-12 14:41:56.010269
37698	DSE	2020-09-29	0.35	0.354	0.35	0.3501	283676	{}	\N	2020-10-12 14:41:56.022366
37699	DSE	2020-09-30	0.3474	0.354	0.3474	0.3501	376599	{}	\N	2020-10-12 14:41:56.030975
37700	DSE	2020-10-01	0.354	0.354	0.3471	0.3502	265532	{}	\N	2020-10-12 14:41:56.037955
37701	DSE	2020-10-02	0.34	0.353	0.34	0.3505	264116	{}	\N	2020-10-12 14:41:56.043832
37702	DSE	2020-10-05	0.354	0.3662	0.3505	0.3636	328744	{}	\N	2020-10-12 14:41:56.048798
37703	DSE	2020-10-06	0.37	0.3792	0.3515	0.3633	374699	{}	\N	2020-10-12 14:41:56.053572
37704	DSE	2020-10-07	0.3661	0.3739	0.3633	0.3662	267024	{}	\N	2020-10-12 14:41:56.058762
37705	DSE	2020-10-08	0.366	0.3917	0.3637	0.386201	272616	{}	\N	2020-10-12 14:41:56.063488
37706	DSE	2020-10-09	0.403	0.4091	0.38	0.3861	257744	{}	\N	2020-10-12 14:41:56.067589
37707	DSE	2020-10-12	0.4011	0.47	0.400001	0.4025	3787948	{}	\N	2020-10-12 14:41:56.07162
37708	DSU	2020-09-28	9.94	10.07	9.94	10	188084	{}	\N	2020-10-12 14:41:57.008507
37709	DSU	2020-09-29	10.03	10.05	9.99	10	577690	{}	\N	2020-10-12 14:41:57.017377
37710	DSU	2020-09-30	10.05	10.05	9.94	9.94	464125	{}	\N	2020-10-12 14:41:57.022906
37711	DSU	2020-10-01	9.94	10.015	9.91	9.93	500779	{}	\N	2020-10-12 14:41:57.028255
37712	DSU	2020-10-02	9.93	10.05	9.9	10.05	363445	{}	\N	2020-10-12 14:41:57.03347
37713	DSU	2020-10-05	10.05	10.12	9.99	10.12	240304	{}	\N	2020-10-12 14:41:57.038212
37714	DSU	2020-10-06	10.09	10.15	10.07	10.1	161732	{}	\N	2020-10-12 14:41:57.042742
37715	DSU	2020-10-07	10.13	10.17	10.04	10.05	346626	{}	\N	2020-10-12 14:41:57.047007
37716	DSU	2020-10-08	10.09	10.13	10.03	10.06	470833	{}	\N	2020-10-12 14:41:57.051375
37717	DSU	2020-10-09	10.12	10.12	10.04	10.08	282580	{}	\N	2020-10-12 14:41:57.055559
37718	DSU	2020-10-12	10.11	10.11	10.08	10.095	23814	{}	\N	2020-10-12 14:41:57.060014
37719	DTH	2020-09-28	32.87	33.0768	32.8406	33.045	77689	{}	\N	2020-10-12 14:41:58.160115
37720	DTH	2020-09-29	33.07	33.085	32.89	33.08	224161	{}	\N	2020-10-12 14:41:58.17378
37721	DTH	2020-09-30	33.04	33.24	32.9	32.9	77552	{}	\N	2020-10-12 14:41:58.182702
37722	DTH	2020-10-01	32.91	32.995	32.88	32.995	6775	{}	\N	2020-10-12 14:41:58.189147
37723	DTH	2020-10-02	32.61	33.0899	32.61	33	32075	{}	\N	2020-10-12 14:41:58.194564
37724	DTH	2020-10-05	33.39	33.67	33.39	33.63	145899	{}	\N	2020-10-12 14:41:58.199437
37725	DTH	2020-10-06	33.73	33.73	33.27	33.27	65926	{}	\N	2020-10-12 14:41:58.203721
37726	DTH	2020-10-07	33.55	33.67	33.5461	33.67	38258	{}	\N	2020-10-12 14:41:58.208109
37727	DTH	2020-10-08	33.83	33.91	33.8	33.87	25487	{}	\N	2020-10-12 14:41:58.212745
37728	DTH	2020-10-09	34	34.07	33.89	34.05	126220	{}	\N	2020-10-12 14:41:58.217075
37729	DTH	2020-10-12	34.05	34.07	34.05	34.07	1616	{}	\N	2020-10-12 14:41:58.221372
37730	DLA	2020-09-28	14.7168	14.97	14.3	14.3	22018	{}	\N	2020-10-12 14:42:01.077609
37731	DLA	2020-09-29	14.19	14.27	13.79	14.27	15015	{}	\N	2020-10-12 14:42:01.086355
37732	DLA	2020-09-30	14.12	14.52	14.045	14.25	14110	{}	\N	2020-10-12 14:42:01.092672
37733	DLA	2020-10-01	14.1944	14.81	14.1944	14.73	10251	{}	\N	2020-10-12 14:42:01.097943
37734	DLA	2020-10-02	14.65	14.97	14.5191	14.68	16357	{}	\N	2020-10-12 14:42:01.10326
37735	DLA	2020-10-05	14.68	14.97	14.39	14.41	26503	{}	\N	2020-10-12 14:42:01.107738
37736	DLA	2020-10-06	14.5	14.71	14	14.1	27473	{}	\N	2020-10-12 14:42:01.111919
37737	DLA	2020-10-07	14.28	14.89	14.28	14.7	29238	{}	\N	2020-10-12 14:42:01.115803
37738	DLA	2020-10-08	14.97	15.24	14.72	15.1	30406	{}	\N	2020-10-12 14:42:01.119869
37739	DLA	2020-10-09	15.3	16	15.25	15.31	13651	{}	\N	2020-10-12 14:42:01.123763
37740	DLA	2020-10-12	15.59	16.32	15.5815	16.3	39540	{}	\N	2020-10-12 14:42:01.127489
37741	DLR	2020-09-28	144.18	146.14	143.84	145.94	1424274	{}	\N	2020-10-12 14:42:03.633512
37742	DLR	2020-09-29	146	148.17	145.74	147.52	2011255	{}	\N	2020-10-12 14:42:03.641633
37743	DLR	2020-09-30	148.14	148.79	145.44	146.76	3214709	{}	\N	2020-10-12 14:42:03.647362
37744	DLR	2020-10-01	146.89	148.25	145.2	147.25	2898725	{}	\N	2020-10-12 14:42:03.652652
37745	DLR	2020-10-02	146.4	149.78	145.395	149.35	3332612	{}	\N	2020-10-12 14:42:03.657866
37746	DLR	2020-10-05	149.07	152.42	147.14	151.98	2685057	{}	\N	2020-10-12 14:42:03.662237
37747	DLR	2020-10-06	151.98	153.48	150.18	153.03	2336874	{}	\N	2020-10-12 14:42:03.666017
37748	DLR	2020-10-07	153.58	154.2	152.185	152.58	2840361	{}	\N	2020-10-12 14:42:03.67025
37749	DLR	2020-10-08	153.26	158	153.14	156.85	1984938	{}	\N	2020-10-12 14:42:03.674414
37750	DLR	2020-10-09	157.59	158.7	156.82	158.34	2358524	{}	\N	2020-10-12 14:42:03.678398
37751	DLR	2020-10-12	158.27	158.48	156.93	157.815	284878	{}	\N	2020-10-12 14:42:03.68324
37752	DON	2020-09-28	28.78	29.18	28.78	29.04	344231	{}	\N	2020-10-12 14:42:05.563285
37753	DON	2020-09-29	29.02	29.0499	28.59	28.82	569793	{}	\N	2020-10-12 14:42:05.572116
37754	DON	2020-09-30	28.86	29.2433	28.741	28.92	472676	{}	\N	2020-10-12 14:42:05.578363
37755	DON	2020-10-01	29.01	29.2158	28.83	29.07	300504	{}	\N	2020-10-12 14:42:05.584177
37756	DON	2020-10-02	28.62	29.6301	28.62	29.51	495883	{}	\N	2020-10-12 14:42:05.58911
37757	DON	2020-10-05	29.76	30.1492	29.76	30.1	531093	{}	\N	2020-10-12 14:42:05.594029
37758	DON	2020-10-06	30.28	30.71	29.94	29.98	553143	{}	\N	2020-10-12 14:42:05.597798
37759	DON	2020-10-07	30.22	30.5599	30.22	30.41	459873	{}	\N	2020-10-12 14:42:05.601801
37760	DON	2020-10-08	30.63	30.94	30.61	30.92	588139	{}	\N	2020-10-12 14:42:05.605907
37761	DON	2020-10-09	31.11	31.195	30.79	30.87	318051	{}	\N	2020-10-12 14:42:05.609839
37762	DON	2020-10-12	30.93	31.0499	30.8401	31.01	45422	{}	\N	2020-10-12 14:42:05.613818
37763	DPZ	2020-09-28	416.44	423.11	414.88	421.54	424400	{}	\N	2020-10-12 14:42:07.620777
37764	DPZ	2020-09-29	422.2	428.541	420.645	423.46	772407	{}	\N	2020-10-12 14:42:07.629854
37765	DPZ	2020-09-30	425	428	421.77	425.28	897087	{}	\N	2020-10-12 14:42:07.637331
37766	DPZ	2020-10-01	427.11	430.91	424.53	426.99	1045494	{}	\N	2020-10-12 14:42:07.647111
37767	DPZ	2020-10-02	425.51	435.14	424.505	433.78	841544	{}	\N	2020-10-12 14:42:07.655226
37768	DPZ	2020-10-05	433.66	435.58	423.99	424.94	2064267	{}	\N	2020-10-12 14:42:07.662198
37769	DPZ	2020-10-06	426.55	433.52	425.3727	427.53	1269099	{}	\N	2020-10-12 14:42:07.669066
37770	DPZ	2020-10-07	429.8	433.45	425.77	431.05	1904444	{}	\N	2020-10-12 14:42:07.67462
37771	DPZ	2020-10-08	405.9	410.92	394.9	401.01	7585759	{}	\N	2020-10-12 14:42:07.679443
37772	DPZ	2020-10-09	403	404.55	386.64	390.95	5039433	{}	\N	2020-10-12 14:42:07.684216
37773	DPZ	2020-10-12	392	395.25	388.02	389.435	760023	{}	\N	2020-10-12 14:42:07.688959
37774	DRE	2020-09-28	36.71	37.015	36.47	36.93	1229733	{}	\N	2020-10-12 14:42:09.240589
37775	DRE	2020-09-29	36.67	37.07	36.48	36.96	3375939	{}	\N	2020-10-12 14:42:09.251087
37776	DRE	2020-09-30	37.21	37.48	36.52	36.9	4018202	{}	\N	2020-10-12 14:42:09.257452
37777	DRE	2020-10-01	37.05	37.51	36.6	37.49	4132939	{}	\N	2020-10-12 14:42:09.263129
37778	DRE	2020-10-02	37.09	38.57	36.91	38.44	4651190	{}	\N	2020-10-12 14:42:09.268074
37779	DRE	2020-10-05	38.68	39.348	38.22	39.22	3659390	{}	\N	2020-10-12 14:42:09.273167
37780	DRE	2020-10-06	39.22	39.59	38.76	38.99	3816130	{}	\N	2020-10-12 14:42:09.277094
37781	DRE	2020-10-07	39.16	39.32	38.93	38.99	2396333	{}	\N	2020-10-12 14:42:09.280809
37782	DRE	2020-10-08	39.19	39.7	39.11	39.67	2615923	{}	\N	2020-10-12 14:42:09.284622
37783	DRE	2020-10-09	39.92	39.97	39.44	39.82	2766337	{}	\N	2020-10-12 14:42:09.288141
37784	DRE	2020-10-12	39.98	40.13	39.69	40.1	245916	{}	\N	2020-10-12 14:42:09.291761
37785	DRUA	2020-09-28	26.11	26.13	25.91	25.97	64368	{}	\N	2020-10-12 14:42:10.622735
37786	DRUA	2020-09-29	26.02	26.05	25.83	25.85	86796	{}	\N	2020-10-12 14:42:10.632506
37787	DRUA	2020-09-30	25.84	25.93	25.64	25.64	315391	{}	\N	2020-10-12 14:42:10.63877
37788	DRUA	2020-10-01	25.73	25.88	25.7	25.87	104862	{}	\N	2020-10-12 14:42:10.644481
37789	DRUA	2020-10-02	25.71	25.96	25.71	25.95	93240	{}	\N	2020-10-12 14:42:10.649366
37790	DRUA	2020-10-05	25.97	26.05	25.94	26.04	105018	{}	\N	2020-10-12 14:42:10.654569
37791	DRUA	2020-10-06	26.0502	26.07	25.96	26.05	73931	{}	\N	2020-10-12 14:42:10.658597
37792	DRUA	2020-10-07	26.05	26.14	26.03	26.14	95788	{}	\N	2020-10-12 14:42:10.662569
37793	DRUA	2020-10-08	26.11	26.19	26.06	26.12	85258	{}	\N	2020-10-12 14:42:10.666779
37794	DRUA	2020-10-09	26.12	26.165	26.03	26.16	62152	{}	\N	2020-10-12 14:42:10.671816
37795	DRUA	2020-10-12	26.16	26.17	26.1	26.1	5962	{}	\N	2020-10-12 14:42:10.675803
37796	DSKE	2020-09-28	5.47	5.73	5.39	5.42	200989	{}	\N	2020-10-12 14:42:11.839196
37797	DSKE	2020-09-29	5.37	5.46	5.11	5.28	223596	{}	\N	2020-10-12 14:42:11.847245
37798	DSKE	2020-09-30	5.28	5.52	5.2157	5.37	373221	{}	\N	2020-10-12 14:42:11.853638
37799	DSKE	2020-10-01	5.35	5.69	5.25	5.65	175869	{}	\N	2020-10-12 14:42:11.859072
37800	DSKE	2020-10-02	5.52	6.06	5.4	6.03	400517	{}	\N	2020-10-12 14:42:11.864297
37801	DSKE	2020-10-05	6.12	6.377	6.04	6.3	358786	{}	\N	2020-10-12 14:42:11.868961
37802	DSKE	2020-10-06	6.4	6.52	6.05	6.08	339952	{}	\N	2020-10-12 14:42:11.87323
37803	DSKE	2020-10-07	6.1793	6.42	6.12	6.2	181150	{}	\N	2020-10-12 14:42:11.87873
37804	DSKE	2020-10-08	6.37	6.57	6.23	6.38	179118	{}	\N	2020-10-12 14:42:11.883667
37805	DSKE	2020-10-09	6.4	6.6	6.23	6.46	235206	{}	\N	2020-10-12 14:42:11.88804
37806	DSKE	2020-10-12	6.44	6.8	6.3757	6.7386	64754	{}	\N	2020-10-12 14:42:11.892241
37807	DTE	2020-09-28	115.56	115.79	114.36	114.81	895154	{}	\N	2020-10-12 14:42:12.838575
37808	DTE	2020-09-29	115.09	115.66	113.7	114.4	1659444	{}	\N	2020-10-12 14:42:12.850835
37809	DTE	2020-09-30	115.08	116.19	114.29	115.04	1613099	{}	\N	2020-10-12 14:42:12.859
37810	DTE	2020-10-01	114.84	115.62	112.54	113.44	2427973	{}	\N	2020-10-12 14:42:12.865461
37811	DTE	2020-10-02	112.45	114.86	111.539	114.16	3912267	{}	\N	2020-10-12 14:42:12.870944
37812	DTE	2020-10-05	114.48	115.25	113.49	114.51	1775605	{}	\N	2020-10-12 14:42:12.876254
37813	DTE	2020-10-06	114.89	117.625	114.45	115.4	1968022	{}	\N	2020-10-12 14:42:12.880842
37814	DTE	2020-10-07	116.1	116.64	113.86	114.51	3004455	{}	\N	2020-10-12 14:42:12.885074
37815	DTE	2020-10-08	118.99	122.93	118.67	121.87	5482145	{}	\N	2020-10-12 14:42:12.889176
37816	DTE	2020-10-09	122.44	122.53	119.57	120.29	1650064	{}	\N	2020-10-12 14:42:12.89285
37817	DTE	2020-10-12	120.22	120.52	119.59	120.3	147330	{}	\N	2020-10-12 14:42:12.896613
37818	DTW	2020-09-28	26.8	26.9099	26.61	26.72	27059	{}	\N	2020-10-12 14:42:14.247587
37819	DTW	2020-09-29	26.69	26.711	26.6	26.66	35294	{}	\N	2020-10-12 14:42:14.256624
37820	DTW	2020-09-30	26.65	26.66	26.38	26.38	141962	{}	\N	2020-10-12 14:42:14.263342
37821	DTW	2020-10-01	26.68	26.75	26.3501	26.72	33422	{}	\N	2020-10-12 14:42:14.269515
37822	DTW	2020-10-02	26.71	26.7599	26.6	26.75	62164	{}	\N	2020-10-12 14:42:14.274724
37823	DTW	2020-10-05	26.64	26.93	26.64	26.91	16722	{}	\N	2020-10-12 14:42:14.279534
37824	DTW	2020-10-06	26.93	26.99	26.5	26.59	95279	{}	\N	2020-10-12 14:42:14.28362
37825	DTW	2020-10-07	26.56	26.77	26.51	26.585	77308	{}	\N	2020-10-12 14:42:14.288094
37826	DTW	2020-10-08	26.6	26.76	26.58	26.59	40568	{}	\N	2020-10-12 14:42:14.292204
37827	DTW	2020-10-09	26.7495	26.81	26.71	26.81	13594	{}	\N	2020-10-12 14:42:14.296147
37828	DTW	2020-10-12	26.85	26.85	26.85	26.85	522	{}	\N	2020-10-12 14:42:14.300098
37829	DUKH	2020-09-28	25.89	25.945	25.755	25.82	29645	{}	\N	2020-10-12 14:42:15.332262
37830	DUKH	2020-09-29	25.87	25.8755	25.58	25.6	65226	{}	\N	2020-10-12 14:42:15.347129
37831	DUKH	2020-09-30	25.6	25.69	25.5	25.5	296346	{}	\N	2020-10-12 14:42:15.355375
37832	DUKH	2020-10-01	25.63	25.87	25.6	25.86	62896	{}	\N	2020-10-12 14:42:15.360873
37833	DUKH	2020-10-02	25.73	25.91	25.63	25.89	60798	{}	\N	2020-10-12 14:42:15.365229
37834	DUKH	2020-10-05	25.89	25.95	25.8081	25.93	39407	{}	\N	2020-10-12 14:42:15.369453
37835	DUKH	2020-10-06	25.93	25.95	25.83	25.95	73656	{}	\N	2020-10-12 14:42:15.373487
37836	DUKH	2020-10-07	25.93	25.96	25.88	25.96	89336	{}	\N	2020-10-12 14:42:15.377377
37837	DUKH	2020-10-08	25.89	26.2	25.89	26.14	107132	{}	\N	2020-10-12 14:42:15.381558
37838	DUKH	2020-10-09	26.14	26.21	26.0119	26.21	57760	{}	\N	2020-10-12 14:42:15.385335
37839	DUKH	2020-10-12	26.2	26.2084	26.065	26.2084	8476	{}	\N	2020-10-12 14:42:15.389519
37840	FISI	2020-09-28	15.19	15.82	15.17	15.51	70727	{}	\N	2020-10-12 14:42:16.341683
37841	FISI	2020-09-29	15.38	15.49	15.02	15.43	51579	{}	\N	2020-10-12 14:42:16.352842
37842	FISI	2020-09-30	15.43	15.705	15.28	15.4	46186	{}	\N	2020-10-12 14:42:16.359106
37843	FISI	2020-10-01	15.5	15.56	15.2	15.45	49582	{}	\N	2020-10-12 14:42:16.364332
37844	FISI	2020-10-02	15.1984	16.06	15.08	16	43394	{}	\N	2020-10-12 14:42:16.370016
37845	FISI	2020-10-05	16.24	16.47	16.05	16.4	44045	{}	\N	2020-10-12 14:42:16.375102
37846	FISI	2020-10-06	16.65	17.36	16.18	16.76	57528	{}	\N	2020-10-12 14:42:16.380203
37847	FISI	2020-10-07	16.95	17.105	16.61	16.89	45664	{}	\N	2020-10-12 14:42:16.384636
37848	FISI	2020-10-08	17.07	17.07	16.23	16.7	59718	{}	\N	2020-10-12 14:42:16.389768
37849	FISI	2020-10-09	16.86	16.86	16.17	16.205	55155	{}	\N	2020-10-12 14:42:16.394286
37850	FISI	2020-10-12	16.23	16.365	16.21	16.365	1913	{}	\N	2020-10-12 14:42:16.398523
37851	DLS	2020-09-28	59.87	60.03	59.79	59.98	76458	{}	\N	2020-10-12 14:42:19.125002
37852	DLS	2020-09-29	59.92	60.07	59.64	59.87	59642	{}	\N	2020-10-12 14:42:19.134826
37853	DLS	2020-09-30	59.71	60.28	59.71	59.9	54837	{}	\N	2020-10-12 14:42:19.141497
37854	DLS	2020-10-01	60.27	60.49	60.11	60.42	84840	{}	\N	2020-10-12 14:42:19.147631
37855	DLS	2020-10-02	59.62	60.32	59.62	60.23	70109	{}	\N	2020-10-12 14:42:19.152541
37856	DLS	2020-10-05	60.83	61.2695	60.83	61.23	123490	{}	\N	2020-10-12 14:42:19.157073
37857	DLS	2020-10-06	61.46	61.57	60.84	60.85	53509	{}	\N	2020-10-12 14:42:19.161294
37858	DLS	2020-10-07	61.31	61.4184	61.2	61.28	62181	{}	\N	2020-10-12 14:42:19.165056
37859	DLS	2020-10-08	61.63	61.72	61.49	61.62	78532	{}	\N	2020-10-12 14:42:19.16901
37860	DLS	2020-10-09	61.98	62.16	61.98	62.07	53214	{}	\N	2020-10-12 14:42:19.172786
37861	DLS	2020-10-12	62.1	62.162	62.1	62.154	7629	{}	\N	2020-10-12 14:42:19.17684
37862	DNP	2020-09-28	10.2	10.3	10.12	10.14	468681	{}	\N	2020-10-12 14:42:20.518727
37863	DNP	2020-09-29	10.07	10.1	10.01	10.06	705150	{}	\N	2020-10-12 14:42:20.528505
37864	DNP	2020-09-30	10.06	10.12	10.01	10.04	772064	{}	\N	2020-10-12 14:42:20.53521
37865	DNP	2020-10-01	10.08	10.1399	10	10.05	931990	{}	\N	2020-10-12 14:42:20.540824
37866	DNP	2020-10-02	9.93	10.16	9.72	10.13	1143396	{}	\N	2020-10-12 14:42:20.546251
37867	DNP	2020-10-05	10.17	10.2	10.07	10.09	680154	{}	\N	2020-10-12 14:42:20.551119
37868	DNP	2020-10-06	10.11	10.24	9.92	9.92	1483482	{}	\N	2020-10-12 14:42:20.555438
37869	DNP	2020-10-07	10.02	10.2	10.0101	10.07	975958	{}	\N	2020-10-12 14:42:20.559505
37870	DNP	2020-10-08	10.12	10.19	10.09	10.17	955826	{}	\N	2020-10-12 14:42:20.563993
37871	DNP	2020-10-09	10.2	10.22	10.09	10.17	772593	{}	\N	2020-10-12 14:42:20.569709
37872	DNP	2020-10-12	10.17	10.21	10.17	10.1997	164226	{}	\N	2020-10-12 14:42:20.573933
37873	DOO	2020-09-28	35.91	36.0386	35.87	36.03	9400	{}	\N	2020-10-12 14:42:21.748396
37874	DOO	2020-09-29	36.04	36.15	35.935	36.0816	3763	{}	\N	2020-10-12 14:42:21.761175
37875	DOO	2020-09-30	36.04	36.11	35.9759	35.9759	5252	{}	\N	2020-10-12 14:42:21.768804
37876	DOO	2020-10-01	35.921	35.97	35.825	35.9288	57731	{}	\N	2020-10-12 14:42:21.7741
37877	DOO	2020-10-02	35.69	36.1192	35.69	36.1192	9350	{}	\N	2020-10-12 14:42:21.778869
37878	DOO	2020-10-05	36.3	36.5603	36.3	36.5603	8218	{}	\N	2020-10-12 14:42:21.783113
37879	DOO	2020-10-06	36.66	36.6674	36.2	36.22	8148	{}	\N	2020-10-12 14:42:21.787067
37880	DOO	2020-10-07	36.54	36.6383	36.4	36.6293	9207	{}	\N	2020-10-12 14:42:21.790713
37881	DOO	2020-10-08	36.765	36.8665	36.765	36.849	4272	{}	\N	2020-10-12 14:42:21.794518
37882	DOO	2020-10-09	36.88	36.975	36.88	36.9699	5909	{}	\N	2020-10-12 14:42:21.798382
37883	DOO	2020-10-12	36.98	36.98	36.97	36.97	707	{}	\N	2020-10-12 14:42:21.802584
37884	DPST	2020-09-28	42.75	45.8	42.15	44.93	799439	{}	\N	2020-10-12 14:42:23.018468
37885	DPST	2020-09-29	44.52	44.52	40.915	43.08	621492	{}	\N	2020-10-12 14:42:23.025072
37886	DPST	2020-09-30	43.59	46.2	43.27	44.43	660891	{}	\N	2020-10-12 14:42:23.030199
37887	DPST	2020-10-01	45.31	46.25	43.4	45.46	696284	{}	\N	2020-10-12 14:42:23.035399
37888	DPST	2020-10-02	43.13	49.85	42.01	49.26	950044	{}	\N	2020-10-12 14:42:23.040069
37889	DPST	2020-10-05	50.97	55.53	50.97	55.36	874951	{}	\N	2020-10-12 14:42:23.04448
37890	DPST	2020-10-06	57.22	61.95	54.315	55.04	1697693	{}	\N	2020-10-12 14:42:23.048759
37891	DPST	2020-10-07	58.14	61.82	57.5	59.82	872148	{}	\N	2020-10-12 14:42:23.053445
37892	DPST	2020-10-08	61.2	63.1316	59.87	62.61	613148	{}	\N	2020-10-12 14:42:23.057539
37893	DPST	2020-10-09	64	64.69	58.99	60.45	739195	{}	\N	2020-10-12 14:42:23.061528
37894	DPST	2020-10-12	60.55	62.18	59.7	61.55	147403	{}	\N	2020-10-12 14:42:23.065723
37895	DRIO	2020-09-28	16.54	17.87	16.54	17.6	66000	{}	\N	2020-10-12 14:42:24.024781
37896	DRIO	2020-09-29	17.56	17.94	16.06	16.6	87116	{}	\N	2020-10-12 14:42:24.033048
37897	DRIO	2020-09-30	16.4	17.295	16.25	16.78	66302	{}	\N	2020-10-12 14:42:24.038818
37898	DRIO	2020-10-01	17.01	18	16.838	17.07	148608	{}	\N	2020-10-12 14:42:24.045491
37899	DRIO	2020-10-02	16.78	16.8499	15.9	16.5	123078	{}	\N	2020-10-12 14:42:24.051554
37900	DRIO	2020-10-05	16.4	16.51	16.13	16.36	156017	{}	\N	2020-10-12 14:42:24.057124
37901	DRIO	2020-10-06	16.54	16.6	15.3	15.34	53350	{}	\N	2020-10-12 14:42:24.062077
37902	DRIO	2020-10-07	15.51	16.175	15.35	15.89	57007	{}	\N	2020-10-12 14:42:24.067228
37903	DRIO	2020-10-08	15.84	16	14.91	15.29	52045	{}	\N	2020-10-12 14:42:24.072563
37904	DRIO	2020-10-09	15.38	15.98	14.87	14.97	50476	{}	\N	2020-10-12 14:42:24.077728
37905	DRIO	2020-10-12	15.13	15.1668	14.78	15.09	6741	{}	\N	2020-10-12 14:42:24.08234
37906	DRV	2020-09-28	14.46	14.51	13.9001	14.11	454696	{}	\N	2020-10-12 14:42:25.359026
37907	DRV	2020-09-29	14.1	14.75	14.08	14.39	189883	{}	\N	2020-10-12 14:42:25.364707
37908	DRV	2020-09-30	14.44	14.73	13.7807	14.32	138823	{}	\N	2020-10-12 14:42:25.369743
37909	DRV	2020-10-01	14.17	14.3014	13.4	13.4	212345	{}	\N	2020-10-12 14:42:25.373976
37910	DRV	2020-10-02	14.12	14.12	12.61	12.72	310218	{}	\N	2020-10-12 14:42:25.378306
37911	DRV	2020-10-05	12.65	13.25	12.45	12.51	263631	{}	\N	2020-10-12 14:42:25.38307
37912	DRV	2020-10-06	12.47	12.9	12.22	12.75	261905	{}	\N	2020-10-12 14:42:25.387532
37913	DRV	2020-10-07	12.47	12.87	12.46	12.7	191499	{}	\N	2020-10-12 14:42:25.391838
37914	DRV	2020-10-08	12.66	12.66	11.97	12.09	362642	{}	\N	2020-10-12 14:42:25.395829
37915	DRV	2020-10-09	11.93	12.319	11.8	12.19	146896	{}	\N	2020-10-12 14:42:25.399835
37916	DRV	2020-10-12	12.1	12.4	12.05	12.15	47050	{}	\N	2020-10-12 14:42:25.404023
37917	DTEA	2020-09-28	0.98	0.98	0.937	0.9471	94998	{}	\N	2020-10-12 14:42:27.152135
37918	DTEA	2020-09-29	0.96	1.03	0.96	0.9901	140572	{}	\N	2020-10-12 14:42:27.163075
37919	DTEA	2020-09-30	1.02	1.08	1	1.0799	243247	{}	\N	2020-10-12 14:42:27.170842
37920	DTEA	2020-10-01	1.09	1.18	1.07	1.1416	446063	{}	\N	2020-10-12 14:42:27.176772
37921	DTEA	2020-10-02	1.11	1.22	1.11	1.165	199200	{}	\N	2020-10-12 14:42:27.182212
37922	DTEA	2020-10-05	1.14	1.17	1.11	1.1581	99334	{}	\N	2020-10-12 14:42:27.187099
37923	DTEA	2020-10-06	1.14	1.16	1.12	1.16	93265	{}	\N	2020-10-12 14:42:27.191137
37924	DTEA	2020-10-07	1.15	1.25	1.145	1.18	179689	{}	\N	2020-10-12 14:42:27.195235
37925	DTEA	2020-10-08	1.17	1.25	1.17	1.205	123411	{}	\N	2020-10-12 14:42:27.199251
37926	DTEA	2020-10-09	1.21	1.2798	1.2062	1.25	149446	{}	\N	2020-10-12 14:42:27.203517
37927	DTEA	2020-10-12	1.25	1.27	1.2401	1.2401	5843	{}	\N	2020-10-12 14:42:27.207759
37928	DUC	2020-09-28	9.16	9.22	9.16	9.18	13924	{}	\N	2020-10-12 14:42:28.311254
37929	DUC	2020-09-29	9.15	9.22	9.1	9.12	165132	{}	\N	2020-10-12 14:42:28.320885
37930	DUC	2020-09-30	9.16	9.35	9.12	9.23	765073	{}	\N	2020-10-12 14:42:28.32707
37931	DUC	2020-10-01	9.18	9.23	9.18	9.19	34832	{}	\N	2020-10-12 14:42:28.333489
37932	DUC	2020-10-02	9.14	9.17	9.134	9.16	263344	{}	\N	2020-10-12 14:42:28.338132
37933	DUC	2020-10-05	9.19	9.19	9.14	9.16	50143	{}	\N	2020-10-12 14:42:28.342594
37934	DUC	2020-10-06	9.16	9.1966	9.16	9.16	20115	{}	\N	2020-10-12 14:42:28.348822
37935	DUC	2020-10-07	9.17	9.22	9.17	9.2066	23352	{}	\N	2020-10-12 14:42:28.353493
37936	DUC	2020-10-08	9.2	9.21	9.14	9.1798	99024	{}	\N	2020-10-12 14:42:28.357553
37937	DUC	2020-10-09	9.13	9.2069	9.13	9.19	107908	{}	\N	2020-10-12 14:42:28.361993
37938	DUC	2020-10-12	9.18	9.22	9.18	9.22	17742	{}	\N	2020-10-12 14:42:28.366243
37939	DVAX	2020-09-28	4.52	4.54	4.26	4.36	2141844	{}	\N	2020-10-12 14:42:29.414058
37940	DVAX	2020-09-29	4.35	4.4592	4.3101	4.4	1336522	{}	\N	2020-10-12 14:42:29.426486
37941	DVAX	2020-09-30	4.4	4.495	4.2601	4.32	2513336	{}	\N	2020-10-12 14:42:29.433814
37942	DVAX	2020-10-01	4.3	4.459	4.17	4.21	2506311	{}	\N	2020-10-12 14:42:29.440111
37943	DVAX	2020-10-02	4.1	4.335	4.07	4.22	1883033	{}	\N	2020-10-12 14:42:29.445508
37944	DVAX	2020-10-05	4.29	4.53	4.28	4.53	2161008	{}	\N	2020-10-12 14:42:29.450516
37945	DVAX	2020-10-06	4.6	4.6598	4.35	4.41	2042816	{}	\N	2020-10-12 14:42:29.454893
37946	DVAX	2020-10-07	4.42	4.69	4.41	4.61	1789709	{}	\N	2020-10-12 14:42:29.459113
37947	DVAX	2020-10-08	4.66	4.77	4.6051	4.61	1831590	{}	\N	2020-10-12 14:42:29.463481
37948	DVAX	2020-10-09	4.68	4.72	4.425	4.44	2048330	{}	\N	2020-10-12 14:42:29.467836
37949	DVAX	2020-10-12	4.4899	4.52	4.375	4.5001	509520	{}	\N	2020-10-12 14:42:29.472085
37950	DLX	2020-09-28	25.53	26.35	25.53	25.97	302174	{}	\N	2020-10-12 14:42:30.745234
37951	DLX	2020-09-29	25.86	25.99	25	25.62	376121	{}	\N	2020-10-12 14:42:30.756709
37952	DLX	2020-09-30	25.94	26.61	25.62	25.73	457114	{}	\N	2020-10-12 14:42:30.767038
37953	DLX	2020-10-01	25.7	26.22	25.3	25.69	378108	{}	\N	2020-10-12 14:42:30.773053
37954	DLX	2020-10-02	25.15	26.5	25.15	26.36	376494	{}	\N	2020-10-12 14:42:30.777926
37955	DLX	2020-10-05	26.57	26.95	26.33	26.55	295414	{}	\N	2020-10-12 14:42:30.782425
37956	DLX	2020-10-06	26.85	27.08	25.98	26	476868	{}	\N	2020-10-12 14:42:30.786387
37957	DLX	2020-10-07	26.23	26.81	25.73	26.1	381093	{}	\N	2020-10-12 14:42:30.79035
37958	DLX	2020-10-08	26.43	26.53	25.795	26.31	411900	{}	\N	2020-10-12 14:42:30.794599
37959	DLX	2020-10-09	26.54	26.61	24.88	24.92	891619	{}	\N	2020-10-12 14:42:30.79874
37960	DLX	2020-10-12	24.92	25.55	24.47	25.55	276518	{}	\N	2020-10-12 14:42:30.802974
37961	DMRL	2020-09-28	59.21	59.3799	59.2	59.3799	2046	{}	\N	2020-10-12 14:42:32.29635
37962	DMRL	2020-09-29	59.4243	59.4243	59.2983	59.2983	3673	{}	\N	2020-10-12 14:42:32.304061
37963	DMRL	2020-09-30	59.2	59.895	59.2	59.51	7194	{}	\N	2020-10-12 14:42:32.309099
37964	DMRL	2020-10-01	59.7119	59.7119	59.7059	59.7059	258	{}	\N	2020-10-12 14:42:32.313765
37965	DMRL	2020-10-02	59.5757	59.5757	59.5139	59.5139	356	{}	\N	2020-10-12 14:42:32.318311
37966	DMRL	2020-10-05	59.74	59.74	59.62	59.662	1393	{}	\N	2020-10-12 14:42:32.322869
37967	DMRL	2020-10-06	59.9079	59.9079	59.562	59.562	502	{}	\N	2020-10-12 14:42:32.326669
37968	DMRL	2020-10-07	59.85	60.1367	59.73	60.1367	5220	{}	\N	2020-10-12 14:42:32.330564
37969	DMRL	2020-10-08	60.21	60.265	60.21	60.265	405	{}	\N	2020-10-12 14:42:32.334484
37970	DMRL	2020-10-09	60.7736	60.7736	60.7736	60.7736	119	{}	\N	2020-10-12 14:42:32.341572
37971	DNN	2020-09-28	0.42	0.4344	0.4068	0.4219	1221313	{}	\N	2020-10-12 14:42:33.759639
37972	DNN	2020-09-29	0.4236	0.427	0.408	0.408	1303493	{}	\N	2020-10-12 14:42:33.773005
37973	DNN	2020-09-30	0.41	0.4227	0.3867	0.4066	1955315	{}	\N	2020-10-12 14:42:33.780669
37974	DNN	2020-10-01	0.426	0.426	0.4	0.4001	692364	{}	\N	2020-10-12 14:42:33.786965
37975	DNN	2020-10-02	0.4	0.4137	0.3911	0.4059	775073	{}	\N	2020-10-12 14:42:33.791858
37976	DNN	2020-10-05	0.42	0.4245	0.4021	0.4222	687722	{}	\N	2020-10-12 14:42:33.796593
37977	DNN	2020-10-06	0.4294	0.43	0.41	0.4202	1159024	{}	\N	2020-10-12 14:42:33.801089
37978	DNN	2020-10-07	0.4198	0.5299	0.41	0.4572	18141659	{}	\N	2020-10-12 14:42:33.804963
37979	DNN	2020-10-08	0.38	0.39	0.36	0.3701	14856029	{}	\N	2020-10-12 14:42:33.808634
37980	DNN	2020-10-09	0.3701	0.3771	0.36	0.3603	4440954	{}	\N	2020-10-12 14:42:33.812146
37981	DNN	2020-10-12	0.3551	0.3595	0.3495	0.3538	1490495	{}	\N	2020-10-12 14:42:33.815812
37982	DORM	2020-09-28	86.46	89.5	86.46	89.17	167665	{}	\N	2020-10-12 14:42:35.724063
37983	DORM	2020-09-29	89.3	89.68	88.34	88.74	137497	{}	\N	2020-10-12 14:42:35.735596
37984	DORM	2020-09-30	89.52	91.97	88.27	90.38	257922	{}	\N	2020-10-12 14:42:35.742452
37985	DORM	2020-10-01	90.89	93.37	89.515	93.26	181917	{}	\N	2020-10-12 14:42:35.748197
37986	DORM	2020-10-02	91.55	94.54	90.53	94.21	159679	{}	\N	2020-10-12 14:42:35.752907
37987	DORM	2020-10-05	94.04	95	88.51	94.69	176920	{}	\N	2020-10-12 14:42:35.757297
37988	DORM	2020-10-06	95.69	97.42	94.34	95.18	140605	{}	\N	2020-10-12 14:42:35.761329
37989	DORM	2020-10-07	96.15	96.15	93.85	94.31	162142	{}	\N	2020-10-12 14:42:35.765235
37990	DORM	2020-10-08	95.02	95.2	93.33	93.86	167244	{}	\N	2020-10-12 14:42:35.76913
37991	DORM	2020-10-09	94.46	94.8888	92.8001	92.96	139647	{}	\N	2020-10-12 14:42:35.772861
37992	DORM	2020-10-12	93.22	94.34	93.03	93.51	23377	{}	\N	2020-10-12 14:42:35.776519
37993	DRIP	2020-09-28	67.51	69.3	64.0955	65.76	274213	{}	\N	2020-10-12 14:42:37.212088
37994	DRIP	2020-09-29	66.65	72.49	66.49	69.99	291087	{}	\N	2020-10-12 14:42:37.220087
37995	DRIP	2020-09-30	69.66	71.91	67.92	71.04	230375	{}	\N	2020-10-12 14:42:37.225667
37996	DRIP	2020-10-01	72.85	76.6	72.04	75.91	350636	{}	\N	2020-10-12 14:42:37.231142
37997	DRIP	2020-10-02	81.2	81.49	70.715	72.06	497672	{}	\N	2020-10-12 14:42:37.236392
37998	DRIP	2020-10-05	69.32	70.55	65.55	65.55	455915	{}	\N	2020-10-12 14:42:37.240753
37999	DRIP	2020-10-06	63.44	69.1585	61.6181	68.89	553808	{}	\N	2020-10-12 14:42:37.244582
38000	DRIP	2020-10-07	67.41	68.3299	64.5	64.82	337960	{}	\N	2020-10-12 14:42:37.248378
38001	DRIP	2020-10-08	63.72	64.8	59.21	59.25	354178	{}	\N	2020-10-12 14:42:37.252592
38002	DRIP	2020-10-09	58.22	62.09	57.34	61.11	433493	{}	\N	2020-10-12 14:42:37.256382
38003	DRIP	2020-10-12	61.7	63.66	61.55	62.48	111076	{}	\N	2020-10-12 14:42:37.26007
38004	DSM	2020-09-28	7.29	7.31	7.25	7.27	144406	{}	\N	2020-10-12 14:42:38.25959
38005	DSM	2020-09-29	7.29	7.3	7.27	7.3	100006	{}	\N	2020-10-12 14:42:38.273339
38006	DSM	2020-09-30	7.31	7.35	7.29	7.29	162944	{}	\N	2020-10-12 14:42:38.280682
38007	DSM	2020-10-01	7.32	7.39	7.32	7.36	285030	{}	\N	2020-10-12 14:42:38.287267
38008	DSM	2020-10-02	7.35	7.39	7.2938	7.38	117636	{}	\N	2020-10-12 14:42:38.2923
38009	DSM	2020-10-05	7.37	7.37	7.31	7.34	156930	{}	\N	2020-10-12 14:42:38.297242
38010	DSM	2020-10-06	7.33	7.36	7.31	7.34	105564	{}	\N	2020-10-12 14:42:38.301657
38011	DSM	2020-10-07	7.34	7.39	7.32	7.39	235304	{}	\N	2020-10-12 14:42:38.305806
38012	DSM	2020-10-08	7.41	7.4116	7.38	7.41	146966	{}	\N	2020-10-12 14:42:38.309881
38013	DSM	2020-10-09	7.42	7.42	7.37	7.4	160444	{}	\N	2020-10-12 14:42:38.314034
38014	DSM	2020-10-12	7.43	7.43	7.39	7.4004	67896	{}	\N	2020-10-12 14:42:38.31811
38015	DTD	2020-09-28	93.07	94.045	93.07	93.58	22039	{}	\N	2020-10-12 14:42:39.405615
38016	DTD	2020-09-29	93.61	93.6697	92.6999	93.02	8626	{}	\N	2020-10-12 14:42:39.414371
38017	DTD	2020-09-30	93.1	94.48	93.1	93.44	13638	{}	\N	2020-10-12 14:42:39.420215
38018	DTD	2020-10-01	93.89	94.115	93.4276	94.02	18697	{}	\N	2020-10-12 14:42:39.425448
38019	DTD	2020-10-02	92.56	94.33	92.31	93.87	12321	{}	\N	2020-10-12 14:42:39.430122
38020	DTD	2020-10-05	94.49	95.46	94.49	95.46	10243	{}	\N	2020-10-12 14:42:39.434699
38021	DTD	2020-10-06	95.64	96.38	94.41	94.41	20318	{}	\N	2020-10-12 14:42:39.438792
38022	DTD	2020-10-07	95.26	96.2	95.26	95.9	15253	{}	\N	2020-10-12 14:42:39.442699
38023	DTD	2020-10-08	96.48	97.0363	96.48	97.0363	9667	{}	\N	2020-10-12 14:42:39.446801
38024	DTD	2020-10-09	97.62	97.72	97.24	97.39	15496	{}	\N	2020-10-12 14:42:39.450856
38025	DTD	2020-10-12	97.65	97.77	97.65	97.77	3420	{}	\N	2020-10-12 14:42:39.45531
38026	DTN	2020-09-28	74.97	75.85	74.97	75.4	20376	{}	\N	2020-10-12 14:42:40.817419
38027	DTN	2020-09-29	75.2	75.57	74.513	75.0239	15612	{}	\N	2020-10-12 14:42:40.829389
38028	DTN	2020-09-30	75.06	75.87	75.06	75.42	15274	{}	\N	2020-10-12 14:42:40.834961
38029	DTN	2020-10-01	75.5	75.67	74.67	75.03	17132	{}	\N	2020-10-12 14:42:40.8404
38030	DTN	2020-10-02	74.25	76.06	74.25	76.06	20554	{}	\N	2020-10-12 14:42:40.845164
38031	DTN	2020-10-05	75.79	76.95	75.79	76.87	11467	{}	\N	2020-10-12 14:42:40.84974
38032	DTN	2020-10-06	77.27	77.6839	76.271	76.271	11849	{}	\N	2020-10-12 14:42:40.853849
38033	DTN	2020-10-07	76.85	77.6075	76.85	77.5059	7081	{}	\N	2020-10-12 14:42:40.857864
38034	DTN	2020-10-08	77.85	78.5835	77.85	78.5835	25550	{}	\N	2020-10-12 14:42:40.861889
38035	DTN	2020-10-09	79	79.1649	78.47	78.47	28540	{}	\N	2020-10-12 14:42:40.865861
38036	DTN	2020-10-12	78.46	78.639	78.46	78.58	2760	{}	\N	2020-10-12 14:42:40.86967
38037	DNOW	2020-09-28	4.92	5.28	4.92	5.03	1703245	{}	\N	2020-10-12 14:42:42.148986
38038	DNOW	2020-09-29	5.01	5.02	4.73	4.87	2497545	{}	\N	2020-10-12 14:42:42.158207
38039	DNOW	2020-09-30	4.88	4.905	4.51	4.54	4654788	{}	\N	2020-10-12 14:42:42.165306
38040	DNOW	2020-10-01	4.48	4.6	4.385	4.43	3812486	{}	\N	2020-10-12 14:42:42.171193
38041	DNOW	2020-10-02	4.31	5.03	4.27	4.88	5523138	{}	\N	2020-10-12 14:42:42.176488
38042	DNOW	2020-10-05	4.93	5.19	4.905	5.04	3996332	{}	\N	2020-10-12 14:42:42.182019
38043	DNOW	2020-10-06	5.14	5.29	4.99	5.06	3375352	{}	\N	2020-10-12 14:42:42.186511
38044	DNOW	2020-10-07	5.07	5.14	4.885	5.08	2620742	{}	\N	2020-10-12 14:42:42.190896
38045	DNOW	2020-10-08	5.21	5.37	5.155	5.3	2368196	{}	\N	2020-10-12 14:42:42.195351
38046	DNOW	2020-10-09	5.35	5.42	5.2	5.32	2807622	{}	\N	2020-10-12 14:42:42.199599
38047	DNOW	2020-10-12	5.29	5.32	5.16	5.195	1052518	{}	\N	2020-10-12 14:42:42.203872
38048	DOL	2020-09-28	41.28	41.53	41.28	41.46	27412	{}	\N	2020-10-12 14:42:43.57032
38049	DOL	2020-09-29	41.46	41.6011	41.315	41.38	18291	{}	\N	2020-10-12 14:42:43.580707
38050	DOL	2020-09-30	41.38	41.571	41.23	41.33	34646	{}	\N	2020-10-12 14:42:43.587352
38051	DOL	2020-10-01	41.335	41.41	41.22	41.35	13975	{}	\N	2020-10-12 14:42:43.592713
38052	DOL	2020-10-02	41	41.445	41	41.43	8974	{}	\N	2020-10-12 14:42:43.597507
38053	DOL	2020-10-05	41.66	41.95	41.66	41.93	12256	{}	\N	2020-10-12 14:42:43.602186
38054	DOL	2020-10-06	42.03	42.03	41.4701	41.54	25719	{}	\N	2020-10-12 14:42:43.606763
38055	DOL	2020-10-07	41.84	41.959	41.84	41.9	6240	{}	\N	2020-10-12 14:42:43.610693
38056	DOL	2020-10-08	42.12	42.1964	42.07	42.155	20837	{}	\N	2020-10-12 14:42:43.614627
38057	DOL	2020-10-09	42.34	42.455	42.29	42.38	14768	{}	\N	2020-10-12 14:42:43.618651
38058	DOL	2020-10-12	42.48	42.517	42.4771	42.485	4339	{}	\N	2020-10-12 14:42:43.622649
38059	DRAD	2020-09-28	2.56	2.563	2.4536	2.51	122562	{}	\N	2020-10-12 14:42:45.659003
38060	DRAD	2020-09-29	2.49	2.59	2.49	2.58	33478	{}	\N	2020-10-12 14:42:45.666581
38061	DRAD	2020-09-30	2.5362	2.58	2.48	2.49	67306	{}	\N	2020-10-12 14:42:45.672816
38062	DRAD	2020-10-01	2.5	2.55	2.4699	2.51	26765	{}	\N	2020-10-12 14:42:45.678401
38063	DRAD	2020-10-02	2.5	2.55	2.45	2.55	52861	{}	\N	2020-10-12 14:42:45.683504
38064	DRAD	2020-10-05	2.56	2.59	2.54	2.58	21255	{}	\N	2020-10-12 14:42:45.688778
38065	DRAD	2020-10-06	2.6	2.7	2.58	2.62	80269	{}	\N	2020-10-12 14:42:45.693484
38066	DRAD	2020-10-07	2.59	2.7401	2.59	2.69	99017	{}	\N	2020-10-12 14:42:45.697763
38067	DRAD	2020-10-08	2.7	2.75	2.66	2.72	46248	{}	\N	2020-10-12 14:42:45.702818
38068	DRAD	2020-10-09	2.72	2.75	2.66	2.75	25143	{}	\N	2020-10-12 14:42:45.707429
38069	DRAD	2020-10-12	2.72	2.74	2.6918	2.73	6298	{}	\N	2020-10-12 14:42:45.711744
38070	DRRX	2020-09-28	1.77	1.7817	1.675	1.74	861384	{}	\N	2020-10-12 14:42:46.693486
38071	DRRX	2020-09-29	1.72	1.76	1.67	1.75	547598	{}	\N	2020-10-12 14:42:46.703017
38072	DRRX	2020-09-30	1.73	1.78	1.69	1.71	726801	{}	\N	2020-10-12 14:42:46.708783
38073	DRRX	2020-10-01	1.73	1.81	1.72	1.75	771474	{}	\N	2020-10-12 14:42:46.713532
38074	DRRX	2020-10-02	1.71	1.82	1.71	1.81	537207	{}	\N	2020-10-12 14:42:46.717804
38075	DRRX	2020-10-05	1.84	1.91	1.825	1.89	819120	{}	\N	2020-10-12 14:42:46.721805
38076	DRRX	2020-10-06	1.9	1.93	1.75	1.76	632716	{}	\N	2020-10-12 14:42:46.725986
38077	DRRX	2020-10-07	1.78	2.06	1.7756	1.97	1613516	{}	\N	2020-10-12 14:42:46.729841
38078	DRRX	2020-10-08	2.0601	2.1	1.915	1.93	923754	{}	\N	2020-10-12 14:42:46.733476
38079	DRRX	2020-10-09	1.97	2	1.87	1.89	491543	{}	\N	2020-10-12 14:42:46.737383
38080	DRRX	2020-10-12	1.94	1.99	1.89	1.975	228234	{}	\N	2020-10-12 14:42:46.740703
38081	DSI	2020-09-28	127.3	127.71	126.88	127.32	47945	{}	\N	2020-10-12 14:42:47.729979
38082	DSI	2020-09-29	127.52	127.71	126.709	127.05	32836	{}	\N	2020-10-12 14:42:47.736996
38083	DSI	2020-09-30	127.34	129.08	127.34	127.95	49302	{}	\N	2020-10-12 14:42:47.742962
38084	DSI	2020-10-01	129.03	129.38	128.17	128.83	38957	{}	\N	2020-10-12 14:42:47.747575
38085	DSI	2020-10-02	126.5	128.28	126.5	127.5235	384675	{}	\N	2020-10-12 14:42:47.752005
38086	DSI	2020-10-05	128.4	129.77	128.4	129.77	33603	{}	\N	2020-10-12 14:42:47.757265
38087	DSI	2020-10-06	129.77	130.7599	127.934	128.06	35359	{}	\N	2020-10-12 14:42:47.762348
38088	DSI	2020-10-07	129.26	130.4157	129.26	130.12	87502	{}	\N	2020-10-12 14:42:47.767073
38089	DSI	2020-10-08	130.92	131.32	130.77	131.24	32623	{}	\N	2020-10-12 14:42:47.772264
38090	DSI	2020-10-09	132.09	132.594	131.7897	132.47	49674	{}	\N	2020-10-12 14:42:47.777475
38091	DSI	2020-10-12	133.28	133.45	133.183	133.45	5111	{}	\N	2020-10-12 14:42:47.782665
38092	DTQ	2020-09-28	25.17	25.19	25.15	25.185	30307	{}	\N	2020-10-12 14:42:49.458429
38093	DTQ	2020-09-29	25.18	25.19	25.16	25.18	18154	{}	\N	2020-10-12 14:42:49.465068
38094	DTQ	2020-09-30	25.18	25.19	25.16	25.16	187704	{}	\N	2020-10-12 14:42:49.4705
38095	DTQ	2020-10-01	25.19	25.19	25.16	25.19	86184	{}	\N	2020-10-12 14:42:49.475409
38096	DTQ	2020-10-02	25.16	25.19	25.16	25.185	62778	{}	\N	2020-10-12 14:42:49.48039
38097	DTQ	2020-10-05	25.17	25.19	25.17	25.19	28946	{}	\N	2020-10-12 14:42:49.484213
38098	DTQ	2020-10-06	25.19	25.19	25.17	25.19	22688	{}	\N	2020-10-12 14:42:49.487833
38099	DTQ	2020-10-07	25.17	25.19	25.17	25.19	25582	{}	\N	2020-10-12 14:42:49.491637
38100	DTQ	2020-10-08	25.172	25.19	25.17	25.19	67291	{}	\N	2020-10-12 14:42:49.495494
38101	DTQ	2020-10-09	25.19	25.19	25.16	25.18	258344	{}	\N	2020-10-12 14:42:49.500186
38102	DUG	2020-09-28	55.3	55.3	53	54.37	103026	{}	\N	2020-10-12 14:42:51.365532
38103	DUG	2020-09-29	54.45	58.1909	54.45	57.1	77421	{}	\N	2020-10-12 14:42:51.375249
38104	DUG	2020-09-30	57.02	58.02	55.262	57.44	149898	{}	\N	2020-10-12 14:42:51.381058
38105	DUG	2020-10-01	58.42	61.56	58.31	60.96	191087	{}	\N	2020-10-12 14:42:51.386483
38106	DUG	2020-10-02	64.3	64.3	58.7331	59.71	155004	{}	\N	2020-10-12 14:42:51.391892
38107	DUG	2020-10-05	58.44	58.82	56.02	56.06	51411	{}	\N	2020-10-12 14:42:51.396702
38108	DUG	2020-10-06	54.5	57.8	53.785	57.68	87676	{}	\N	2020-10-12 14:42:51.40095
38109	DUG	2020-10-07	57.25	58.24	55.69	55.94	44383	{}	\N	2020-10-12 14:42:51.405515
38110	DUG	2020-10-08	54.72	55.2225	51.73	51.795	115243	{}	\N	2020-10-12 14:42:51.409989
38111	DUG	2020-10-09	50.81	53.47	50.63	53.37	87809	{}	\N	2020-10-12 14:42:51.414641
38112	DUG	2020-10-12	53.41	54.69	53.41	53.72	16640	{}	\N	2020-10-12 14:42:51.418953
38113	DVD	2020-09-28	1.42	1.426	1.35	1.42	18756	{}	\N	2020-10-12 14:42:53.110851
38114	DVD	2020-09-29	1.42	1.44	1.41	1.41	28364	{}	\N	2020-10-12 14:42:53.120778
38115	DVD	2020-09-30	1.3908	1.44	1.35	1.41	94844	{}	\N	2020-10-12 14:42:53.12825
38116	DVD	2020-10-01	1.4064	1.41	1.32	1.335	79720	{}	\N	2020-10-12 14:42:53.134975
38117	DVD	2020-10-02	1.34	1.43	1.34	1.43	87233	{}	\N	2020-10-12 14:42:53.141708
38118	DVD	2020-10-05	1.42	1.43	1.39	1.41	67984	{}	\N	2020-10-12 14:42:53.146712
38119	DVD	2020-10-06	1.43	1.43	1.38	1.39	17256	{}	\N	2020-10-12 14:42:53.151803
38120	DVD	2020-10-07	1.38	1.43	1.38	1.43	15788	{}	\N	2020-10-12 14:42:53.156368
38121	DVD	2020-10-08	1.43	1.43	1.39	1.4	12932	{}	\N	2020-10-12 14:42:53.160748
38122	DVD	2020-10-09	1.43	1.45	1.4	1.4	16686	{}	\N	2020-10-12 14:42:53.165434
38123	DVD	2020-10-12	1.43	1.43	1.4185	1.43	11146	{}	\N	2020-10-12 14:42:53.170226
38124	DWAS	2020-09-28	58.64	59.07	58.64	59.07	6836	{}	\N	2020-10-12 14:42:54.113404
38125	DWAS	2020-09-29	59.1	59.6944	58.9	59.3192	39754	{}	\N	2020-10-12 14:42:54.121482
38126	DWAS	2020-09-30	59.51	60.3299	59.1254	59.37	44307	{}	\N	2020-10-12 14:42:54.127472
38127	DWAS	2020-10-01	60.1	60.58	59.5315	60.58	9078	{}	\N	2020-10-12 14:42:54.132704
38128	DWAS	2020-10-02	59.24	60.77	59.24	60.58	8124	{}	\N	2020-10-12 14:42:54.137916
38129	DWAS	2020-10-05	61.2	62.7	61.2	62.64	30868	{}	\N	2020-10-12 14:42:54.142654
38130	DWAS	2020-10-06	63.15	64.12	62.1201	62.48	32502	{}	\N	2020-10-12 14:42:54.146708
38131	DWAS	2020-10-07	63.5	64.4	63.5	64.23	25327	{}	\N	2020-10-12 14:42:54.150998
38132	DWAS	2020-10-08	65	65	64.1104	64.53	25048	{}	\N	2020-10-12 14:42:54.15486
38133	DWAS	2020-10-09	65.21	65.62	64.9988	65.56	23718	{}	\N	2020-10-12 14:42:54.158835
38134	DWAS	2020-10-12	66.28	66.28	65.54	65.97	8656	{}	\N	2020-10-12 14:42:54.162853
38135	DOC	2020-09-28	17.75	18.01	17.6	18.01	1608056	{}	\N	2020-10-12 14:42:55.429577
38136	DOC	2020-09-29	18.01	18.01	17.56	17.87	2630224	{}	\N	2020-10-12 14:42:55.435648
38137	DOC	2020-09-30	17.95	18.07	17.71	17.91	3210665	{}	\N	2020-10-12 14:42:55.440202
38138	DOC	2020-10-01	17.72	17.94	17.57	17.93	3632761	{}	\N	2020-10-12 14:42:55.444654
38139	DOC	2020-10-02	17.69	18.56	17.56	18.51	4074858	{}	\N	2020-10-12 14:42:55.449384
38140	DOC	2020-10-05	18.65	18.678	18.17	18.39	2528928	{}	\N	2020-10-12 14:42:55.453997
38141	DOC	2020-10-06	18.5	18.75	18.23	18.41	5149619	{}	\N	2020-10-12 14:42:55.459302
38142	DOC	2020-10-07	18.52	18.5699	18.225	18.3	2331130	{}	\N	2020-10-12 14:42:55.463795
38143	DOC	2020-10-08	18.38	18.62	18.32	18.49	2871391	{}	\N	2020-10-12 14:42:55.468334
38144	DOC	2020-10-09	18.67	18.67	18.22	18.24	1967275	{}	\N	2020-10-12 14:42:55.472679
38145	DOC	2020-10-12	18.24	18.4	18.14	18.35	1184682	{}	\N	2020-10-12 14:42:55.477341
38146	DRH	2020-09-28	5.07	5.31	5.07	5.24	1768812	{}	\N	2020-10-12 14:42:57.869078
38147	DRH	2020-09-29	5.26	5.26	4.96	5.05	3159597	{}	\N	2020-10-12 14:42:57.881467
38148	DRH	2020-09-30	5.09	5.325	5	5.07	4235880	{}	\N	2020-10-12 14:42:57.889378
38149	DRH	2020-10-01	5.06	5.29	5.06	5.26	4752107	{}	\N	2020-10-12 14:42:57.895408
38150	DRH	2020-10-02	5.09	5.295	5.04	5.28	4044109	{}	\N	2020-10-12 14:42:57.900545
38151	DRH	2020-10-05	5.34	5.42	5.15	5.26	2858236	{}	\N	2020-10-12 14:42:57.905316
38152	DRH	2020-10-06	5.35	5.5	5.265	5.3	3828637	{}	\N	2020-10-12 14:42:57.909731
38153	DRH	2020-10-07	5.37	5.38	5.245	5.32	2862323	{}	\N	2020-10-12 14:42:57.914419
38154	DRH	2020-10-08	5.37	5.52	5.31	5.51	2198129	{}	\N	2020-10-12 14:42:57.918727
38155	DRH	2020-10-09	5.58	5.58	5.35	5.38	1906521	{}	\N	2020-10-12 14:42:57.923993
38156	DRH	2020-10-12	5.37	5.37	5.235	5.34	299160	{}	\N	2020-10-12 14:42:57.929024
38157	DS	2020-09-28	1.2	1.24	1.13	1.17	1932113	{}	\N	2020-10-12 14:42:59.099422
38158	DS	2020-09-29	1.22	1.22	1.115	1.16	1118244	{}	\N	2020-10-12 14:42:59.107828
38159	DS	2020-09-30	1.14	1.17	1.0722	1.12	1230704	{}	\N	2020-10-12 14:42:59.113558
38160	DS	2020-10-01	1.12	1.18	1.09	1.18	1194391	{}	\N	2020-10-12 14:42:59.119032
38161	DS	2020-10-02	1.12	1.18	1.12	1.15	836577	{}	\N	2020-10-12 14:42:59.124106
38162	DS	2020-10-05	1.17	1.18	1.105	1.16	990922	{}	\N	2020-10-12 14:42:59.128952
38163	DS	2020-10-06	1.17	1.19	1.13	1.13	864987	{}	\N	2020-10-12 14:42:59.133464
38164	DS	2020-10-07	1.16	1.19	1.1489	1.17	674754	{}	\N	2020-10-12 14:42:59.137842
38165	DS	2020-10-08	1.19	1.2	1.13	1.19	965176	{}	\N	2020-10-12 14:42:59.14191
38166	DS	2020-10-09	1.22	1.23	1.18	1.18	632074	{}	\N	2020-10-12 14:42:59.146282
38167	DS	2020-10-12	1.19	1.2	1.15	1.17	130840	{}	\N	2020-10-12 14:42:59.150529
38168	DSX	2020-09-28	1.42	1.43	1.4	1.41	130130	{}	\N	2020-10-12 14:43:00.474006
38169	DSX	2020-09-29	1.4	1.43	1.39	1.42	240080	{}	\N	2020-10-12 14:43:00.482866
38170	DSX	2020-09-30	1.38	1.435	1.38	1.41	546860	{}	\N	2020-10-12 14:43:00.490854
38171	DSX	2020-10-01	1.44	1.47	1.42	1.43	463670	{}	\N	2020-10-12 14:43:00.496172
38172	DSX	2020-10-02	1.42	1.45	1.4	1.44	687242	{}	\N	2020-10-12 14:43:00.500977
38173	DSX	2020-10-05	1.46	1.52	1.45	1.51	976922	{}	\N	2020-10-12 14:43:00.50507
38174	DSX	2020-10-06	1.48	1.52	1.48	1.5	460859	{}	\N	2020-10-12 14:43:00.509463
38175	DSX	2020-10-07	1.5	1.51	1.47	1.49	455858	{}	\N	2020-10-12 14:43:00.51352
38176	DSX	2020-10-08	1.49	1.5	1.46	1.46	384569	{}	\N	2020-10-12 14:43:00.517638
38177	DSX	2020-10-09	1.46	1.47	1.42	1.45	533376	{}	\N	2020-10-12 14:43:00.52149
38178	DSX	2020-10-12	1.45	1.48	1.44	1.4489	56486	{}	\N	2020-10-12 14:43:00.525648
38179	DUK	2020-09-28	82.97	84.08	82.66	82.87	2456408	{}	\N	2020-10-12 14:43:02.147021
38180	DUK	2020-09-29	83.07	83.4	82.04	82.41	4349743	{}	\N	2020-10-12 14:43:02.157749
38181	DUK	2020-09-30	88	89.4899	86.43	88.56	37285781	{}	\N	2020-10-12 14:43:02.165503
38182	DUK	2020-10-01	88.03	90.36	87.6	90.05	12380054	{}	\N	2020-10-12 14:43:02.171783
38183	DUK	2020-10-02	89.22	92.15	88.69	91.58	11594730	{}	\N	2020-10-12 14:43:02.177069
38184	DUK	2020-10-05	90.98	92.15	89.9	91.74	6149984	{}	\N	2020-10-12 14:43:02.182453
38185	DUK	2020-10-06	91.72	92.065	90.46	90.77	5821998	{}	\N	2020-10-12 14:43:02.187738
38186	DUK	2020-10-07	91.61	92.125	90.7	91.3	4926453	{}	\N	2020-10-12 14:43:02.191978
38187	DUK	2020-10-08	91.49	93.43	90.9	93.15	6420913	{}	\N	2020-10-12 14:43:02.19679
38188	DUK	2020-10-09	93.14	93.92	92.04	93.3	7344198	{}	\N	2020-10-12 14:43:02.20187
38189	DUK	2020-10-12	93.19	93.235	92.59	93.08	781023	{}	\N	2020-10-12 14:43:02.206651
38190	DVYA	2020-09-28	32.49	32.56	32.49	32.501	2499	{}	\N	2020-10-12 14:43:03.373956
38191	DVYA	2020-09-29	32.46	32.46	32.25	32.27	5759	{}	\N	2020-10-12 14:43:03.383315
38192	DVYA	2020-09-30	32.39	32.63	32.39	32.5	3167	{}	\N	2020-10-12 14:43:03.389631
38193	DVYA	2020-10-01	32.56	32.68	32.56	32.68	917	{}	\N	2020-10-12 14:43:03.39531
38194	DVYA	2020-10-02	32.785	32.785	32.6531	32.71	2646	{}	\N	2020-10-12 14:43:03.403071
38195	DVYA	2020-10-05	32.843	33.0169	32.843	33.0169	1437	{}	\N	2020-10-12 14:43:03.407255
38196	DVYA	2020-10-06	33.2721	33.2721	32.93	33.05	3615	{}	\N	2020-10-12 14:43:03.411273
38197	DVYA	2020-10-07	33.28	33.35	33.171	33.297	5287	{}	\N	2020-10-12 14:43:03.415807
38198	DVYA	2020-10-08	33.29	33.44	33.25	33.36	2719	{}	\N	2020-10-12 14:43:03.419749
38199	DVYA	2020-10-09	33.3139	33.41	33.2811	33.41	4604	{}	\N	2020-10-12 14:43:03.42352
38200	DVYA	2020-10-12	33.48	33.48	33.48	33.48	1122	{}	\N	2020-10-12 14:43:03.427535
38201	DXJ	2020-09-28	49.53	49.63	49.46	49.62	565180	{}	\N	2020-10-12 14:43:04.88886
38202	DXJ	2020-09-29	49.65	49.7901	49.52	49.53	251760	{}	\N	2020-10-12 14:43:04.900945
38203	DXJ	2020-09-30	48.9	49.24	48.875	49	515924	{}	\N	2020-10-12 14:43:04.907438
38204	DXJ	2020-10-01	49	49.08	48.885	49.04	368282	{}	\N	2020-10-12 14:43:04.913055
38205	DXJ	2020-10-02	48.37	48.915	48.325	48.81	470407	{}	\N	2020-10-12 14:43:04.917727
38206	DXJ	2020-10-05	49.06	49.47	49.06	49.37	214334	{}	\N	2020-10-12 14:43:04.922126
38207	DXJ	2020-10-06	49.52	49.64	49.16	49.23	487775	{}	\N	2020-10-12 14:43:04.926475
38208	DXJ	2020-10-07	49.64	49.79	49.575	49.74	611970	{}	\N	2020-10-12 14:43:04.930653
38209	DXJ	2020-10-08	49.88	49.915	49.7	49.79	455810	{}	\N	2020-10-12 14:43:04.934651
38210	DXJ	2020-10-09	49.58	49.64	49.505	49.54	207381	{}	\N	2020-10-12 14:43:04.938767
38211	DXJ	2020-10-12	49.32	49.35	49.21	49.29	42659	{}	\N	2020-10-12 14:43:04.942783
38212	DZSI	2020-09-28	9.6274	9.705	9.2901	9.52	27086	{}	\N	2020-10-12 14:43:07.087983
38213	DZSI	2020-09-29	9.39	9.7775	9.39	9.66	15834	{}	\N	2020-10-12 14:43:07.098692
38214	DZSI	2020-09-30	9.8	9.855	9.27	9.37	25884	{}	\N	2020-10-12 14:43:07.105368
38215	DZSI	2020-10-01	9.515	9.84	9.32	9.56	14889	{}	\N	2020-10-12 14:43:07.111035
38216	DZSI	2020-10-02	9.5499	9.77	9.29	9.41	15885	{}	\N	2020-10-12 14:43:07.11636
38217	DZSI	2020-10-05	9.44	9.9	9.3201	9.86	30051	{}	\N	2020-10-12 14:43:07.121055
38218	DZSI	2020-10-06	9.7	10.49	9.7	10.47	38839	{}	\N	2020-10-12 14:43:07.12533
38219	DZSI	2020-10-07	10.47	10.7	10.25	10.57	43615	{}	\N	2020-10-12 14:43:07.129891
38220	DZSI	2020-10-08	10.63	10.7	10.29	10.58	21351	{}	\N	2020-10-12 14:43:07.134017
38221	DZSI	2020-10-09	10.2886	10.75	10.2886	10.7	22961	{}	\N	2020-10-12 14:43:07.138444
38222	DZSI	2020-10-12	10.48	10.75	10.48	10.75	570	{}	\N	2020-10-12 14:43:07.14283
38223	EAI	2020-09-28	25.89	26.14	25.89	25.95	24038	{}	\N	2020-10-12 14:43:08.197941
38224	EAI	2020-09-29	25.95	26.05	25.89	26.03	25942	{}	\N	2020-10-12 14:43:08.209643
38225	EAI	2020-09-30	26.03	26.03	25.63	25.63	146371	{}	\N	2020-10-12 14:43:08.217032
38226	EAI	2020-10-01	25.68	25.91	25.64	25.84	52572	{}	\N	2020-10-12 14:43:08.222601
38227	EAI	2020-10-02	25.8	26.01	25.8	26.01	42294	{}	\N	2020-10-12 14:43:08.227534
38228	EAI	2020-10-05	25.92	26.07	25.91	26.05	72562	{}	\N	2020-10-12 14:43:08.232589
38229	EAI	2020-10-06	26.09	26.11	25.9543	26.11	50946	{}	\N	2020-10-12 14:43:08.236691
38230	EAI	2020-10-07	26.11	26.11	25.965	26.035	28318	{}	\N	2020-10-12 14:43:08.240406
38231	EAI	2020-10-08	25.94	26.085	25.9	26.03	50961	{}	\N	2020-10-12 14:43:08.244226
38232	EAI	2020-10-09	26.02	26.06	25.95	26.03	43644	{}	\N	2020-10-12 14:43:08.248214
38233	EAI	2020-10-12	25.92	25.92	25.88	25.88	27234	{}	\N	2020-10-12 14:43:08.251856
38234	EBIX	2020-09-28	20.3	20.99	20.3	20.48	206936	{}	\N	2020-10-12 14:43:09.512099
38235	EBIX	2020-09-29	20.41	20.56	19.9483	20.51	142017	{}	\N	2020-10-12 14:43:09.519265
38236	EBIX	2020-09-30	20.71	21.38	20.48	20.6	204945	{}	\N	2020-10-12 14:43:09.52466
38237	EBIX	2020-10-01	20.78	21.95	20.52	21.94	313209	{}	\N	2020-10-12 14:43:09.529078
38238	EBIX	2020-10-02	20.99	21.56	20.9	21.35	151035	{}	\N	2020-10-12 14:43:09.533733
38239	EBIX	2020-10-05	21.67	22.27	21.53	21.88	193535	{}	\N	2020-10-12 14:43:09.537772
38240	EBIX	2020-10-06	22.17	22.35	20.67	20.78	396722	{}	\N	2020-10-12 14:43:09.5416
38241	EBIX	2020-10-07	20.79	21.55	20.4503	20.97	532215	{}	\N	2020-10-12 14:43:09.545611
38242	EBIX	2020-10-08	21.31	21.69	21.23	21.37	134771	{}	\N	2020-10-12 14:43:09.54949
38243	EBIX	2020-10-09	21.59	21.8549	21.2394	21.36	163102	{}	\N	2020-10-12 14:43:09.553248
38244	EBIX	2020-10-12	21.51	21.51	21.14	21.32	23354	{}	\N	2020-10-12 14:43:09.557031
38245	EBSB	2020-09-28	10.09	10.52	10.09	10.38	97599	{}	\N	2020-10-12 14:43:10.914105
38246	EBSB	2020-09-29	10.32	10.38	10.05	10.24	88962	{}	\N	2020-10-12 14:43:10.923315
38247	EBSB	2020-09-30	10.36	10.47	10.24	10.35	134418	{}	\N	2020-10-12 14:43:10.929392
38248	EBSB	2020-10-01	10.35	10.53	10.25	10.5	97871	{}	\N	2020-10-12 14:43:10.934897
38249	EBSB	2020-10-02	10.264	10.83	10.264	10.78	75687	{}	\N	2020-10-12 14:43:10.939628
38250	EBSB	2020-10-05	10.83	11.25	10.78	11.2	138966	{}	\N	2020-10-12 14:43:10.944025
38251	EBSB	2020-10-06	11.33	11.65	11.16	11.17	102899	{}	\N	2020-10-12 14:43:10.948538
38252	EBSB	2020-10-07	11.28	11.57	11.28	11.53	123384	{}	\N	2020-10-12 14:43:10.95239
38253	EBSB	2020-10-08	11.64	11.75	11.4	11.7	142033	{}	\N	2020-10-12 14:43:10.956444
38254	EBSB	2020-10-09	11.77	11.84	11.57	11.68	83276	{}	\N	2020-10-12 14:43:10.960141
38255	EBSB	2020-10-12	11.66	11.82	11.65	11.75	11901	{}	\N	2020-10-12 14:43:10.964126
38256	DOV	2020-09-28	110.42	110.76	108.64	108.78	775063	{}	\N	2020-10-12 14:43:12.443277
38257	DOV	2020-09-29	108.84	109.095	107.61	108.24	999241	{}	\N	2020-10-12 14:43:12.452842
38258	DOV	2020-09-30	108.86	110.3099	107.26	108.34	1695695	{}	\N	2020-10-12 14:43:12.459082
38259	DOV	2020-10-01	109	109.61	106.3	107.14	1208820	{}	\N	2020-10-12 14:43:12.464473
38260	DOV	2020-10-02	105.51	111.12	105.4	110.05	1537728	{}	\N	2020-10-12 14:43:12.469136
38261	DOV	2020-10-05	111.29	112.59	110.7	111.33	1043707	{}	\N	2020-10-12 14:43:12.473733
38262	DOV	2020-10-06	111.53	112.67	109.9	110.09	817121	{}	\N	2020-10-12 14:43:12.47804
38263	DOV	2020-10-07	111.47	113.0475	110.88	111.7	1007848	{}	\N	2020-10-12 14:43:12.483931
38264	DOV	2020-10-08	112.37	112.97	111.02	112.93	849999	{}	\N	2020-10-12 14:43:12.488194
38265	DOV	2020-10-09	113.81	114.29	112.31	113.08	967813	{}	\N	2020-10-12 14:43:12.49221
38266	DOV	2020-10-12	113.89	114.2	113.14	114	103639	{}	\N	2020-10-12 14:43:12.496243
38267	DRI	2020-09-28	98.67	101.29	97.27	100.58	3615155	{}	\N	2020-10-12 14:43:14.117375
38268	DRI	2020-09-29	101.28	101.75	98.39	100.8	4539294	{}	\N	2020-10-12 14:43:14.130362
38269	DRI	2020-09-30	100.18	103.06	100.18	100.74	4621055	{}	\N	2020-10-12 14:43:14.138289
38270	DRI	2020-10-01	101.8	104.98	100.84	104.88	4004840	{}	\N	2020-10-12 14:43:14.147263
38271	DRI	2020-10-02	101.5	105.65	101.23	104.21	4050829	{}	\N	2020-10-12 14:43:14.154886
38272	DRI	2020-10-05	105.06	105.7	102.43	103.71	3188445	{}	\N	2020-10-12 14:43:14.163884
38273	DRI	2020-10-06	103.81	104.61	100.83	101.25	4509372	{}	\N	2020-10-12 14:43:14.175241
38274	DRI	2020-10-07	102.55	103.37	100.77	101.46	2953138	{}	\N	2020-10-12 14:43:14.184028
38275	DRI	2020-10-08	101.98	103.72	100.41	103.38	2690547	{}	\N	2020-10-12 14:43:14.194298
38276	DRI	2020-10-09	103.26	103.28	101.79	101.99	2721290	{}	\N	2020-10-12 14:43:14.204449
38277	DRI	2020-10-12	101.4	101.79	100.09	100.695	763113	{}	\N	2020-10-12 14:43:14.21306
38278	DSPG	2020-09-28	12.84	13.09	12.82	12.95	151710	{}	\N	2020-10-12 14:43:16.140964
38279	DSPG	2020-09-29	12.95	13.37	12.88	13.28	144352	{}	\N	2020-10-12 14:43:16.147733
38280	DSPG	2020-09-30	13.26	13.37	12.98	13.18	153651	{}	\N	2020-10-12 14:43:16.153583
38281	DSPG	2020-10-01	13.26	13.74	12.7291	13.63	124101	{}	\N	2020-10-12 14:43:16.158643
38282	DSPG	2020-10-02	13.24	13.46	13.07	13.09	85307	{}	\N	2020-10-12 14:43:16.16362
38283	DSPG	2020-10-05	13.18	13.67	13.18	13.63	80773	{}	\N	2020-10-12 14:43:16.169833
38284	DSPG	2020-10-06	13.21	13.93	13.21	13.46	52017	{}	\N	2020-10-12 14:43:16.174954
38285	DSPG	2020-10-07	13.55	13.87	13.41	13.54	91918	{}	\N	2020-10-12 14:43:16.180351
38286	DSPG	2020-10-08	13.67	13.975	13.55	13.91	75569	{}	\N	2020-10-12 14:43:16.186495
38287	DSPG	2020-10-09	13.99	14.44	13.57	14.39	105699	{}	\N	2020-10-12 14:43:16.191305
38288	DSPG	2020-10-12	14.45	14.8	14.21	14.29	19058	{}	\N	2020-10-12 14:43:16.196399
38289	DTJ	2020-09-28	25.71	25.86	25.6422	25.85	14663	{}	\N	2020-10-12 14:43:17.199178
38290	DTJ	2020-09-29	25.79	25.79	25.64	25.77	17234	{}	\N	2020-10-12 14:43:17.207842
38291	DTJ	2020-09-30	25.77	25.77	25.4	25.4	219652	{}	\N	2020-10-12 14:43:17.213912
38292	DTJ	2020-10-01	25.49	25.66	25.47	25.66	84066	{}	\N	2020-10-12 14:43:17.220046
38293	DTJ	2020-10-02	25.54	25.65	25.54	25.6	44126	{}	\N	2020-10-12 14:43:17.225807
38294	DTJ	2020-10-05	25.62	25.685	25.62	25.65	22792	{}	\N	2020-10-12 14:43:17.230294
38295	DTJ	2020-10-06	25.61	25.72	25.5879	25.72	35782	{}	\N	2020-10-12 14:43:17.235619
38296	DTJ	2020-10-07	25.7	25.74	25.69	25.74	24326	{}	\N	2020-10-12 14:43:17.240072
38297	DTJ	2020-10-08	25.7	25.7807	25.68	25.68	32462	{}	\N	2020-10-12 14:43:17.244433
38298	DTJ	2020-10-09	25.71	25.74	25.68	25.74	16480	{}	\N	2020-10-12 14:43:17.248959
38299	DTJ	2020-10-12	25.69	25.69	25.69	25.69	4148	{}	\N	2020-10-12 14:43:17.253187
38300	DWX	2020-09-28	33.76	33.879	33.74	33.84	25950	{}	\N	2020-10-12 14:43:22.142678
38301	DWX	2020-09-29	33.79	33.81	33.56	33.66	21068	{}	\N	2020-10-12 14:43:22.15261
38302	DWX	2020-09-30	33.7	33.92	33.62	33.73	43455	{}	\N	2020-10-12 14:43:22.160547
38303	DWX	2020-10-01	33.85	33.95	33.7201	33.95	23917	{}	\N	2020-10-12 14:43:22.166301
38304	DWX	2020-10-02	33.56	33.97	33.56	33.88	24160	{}	\N	2020-10-12 14:43:22.171513
38305	DWX	2020-10-05	34.05	34.28	34.05	34.28	17829	{}	\N	2020-10-12 14:43:22.176243
38306	DWX	2020-10-06	34.27	34.27	33.89	33.94	38681	{}	\N	2020-10-12 14:43:22.180757
38307	DWX	2020-10-07	34.1	34.1715	34	34.14	33822	{}	\N	2020-10-12 14:43:22.184666
38308	DWX	2020-10-08	34.26	34.3	34.16	34.27	49180	{}	\N	2020-10-12 14:43:22.188792
38309	DWX	2020-10-09	34.35	34.49	34.29	34.38	33414	{}	\N	2020-10-12 14:43:22.193742
38310	DWX	2020-10-12	34.49	34.49	34.401	34.4206	6918	{}	\N	2020-10-12 14:43:22.19782
38311	DXGE	2020-09-28	28.66	28.8	28.66	28.8	2370	{}	\N	2020-10-12 14:43:24.542631
38312	DXGE	2020-09-29	28.61	28.61	28.61	28.61	105	{}	\N	2020-10-12 14:43:24.548467
38313	DXGE	2020-09-30	28.6984	28.6984	28.6586	28.6586	1063	{}	\N	2020-10-12 14:43:24.554385
38314	DXGE	2020-10-02	28.64	28.68	28.64	28.68	401	{}	\N	2020-10-12 14:43:24.5596
38315	DXGE	2020-10-05	28.98	29.16	28.98	29.146	1200	{}	\N	2020-10-12 14:43:24.564532
38316	DXGE	2020-10-06	29.3	29.3	29.005	29.005	227	{}	\N	2020-10-12 14:43:24.569999
38317	DXGE	2020-10-08	29.56	29.5873	29.56	29.5873	727	{}	\N	2020-10-12 14:43:24.574841
38318	DXGE	2020-10-09	29.4808	29.4808	29.4808	29.4808	503	{}	\N	2020-10-12 14:43:24.579662
38319	DY	2020-09-28	55.82	57.37	55.32	55.84	297704	{}	\N	2020-10-12 14:43:25.848638
38320	DY	2020-09-29	55.63	56.4	54.1527	54.51	615902	{}	\N	2020-10-12 14:43:25.857408
38321	DY	2020-09-30	55.05	55.345	52.58	52.82	720987	{}	\N	2020-10-12 14:43:25.864101
38322	DY	2020-10-01	53.05	53.945	51.88	52.05	1217382	{}	\N	2020-10-12 14:43:25.870274
38323	DY	2020-10-02	50.53	55.07	50.53	54.86	745197	{}	\N	2020-10-12 14:43:25.874631
38324	DY	2020-10-05	55.62	58.97	55.28	58.54	698036	{}	\N	2020-10-12 14:43:25.879069
38325	DY	2020-10-06	59.16	61.73	58.56	58.97	843253	{}	\N	2020-10-12 14:43:25.883951
38326	DY	2020-10-07	60.17	61.82	59.795	60.97	909424	{}	\N	2020-10-12 14:43:25.888165
38327	DY	2020-10-08	62.08	64.32	60.64	64.28	823937	{}	\N	2020-10-12 14:43:25.89241
38328	DY	2020-10-09	64.99	67.23	64.01	66.99	1114390	{}	\N	2020-10-12 14:43:25.897053
38329	DY	2020-10-12	67.07	69.12	67	69.12	167454	{}	\N	2020-10-12 14:43:25.901618
38330	E	2020-09-28	15.83	16.05	15.79	15.92	323998	{}	\N	2020-10-12 14:43:27.281532
38331	E	2020-09-29	15.83	15.83	15.42	15.56	906838	{}	\N	2020-10-12 14:43:27.289059
38332	E	2020-09-30	15.69	15.79	15.53	15.54	648028	{}	\N	2020-10-12 14:43:27.295839
38333	E	2020-10-01	15.39	15.42	15.04	15.04	646138	{}	\N	2020-10-12 14:43:27.30195
38334	E	2020-10-02	14.82	15.29	14.82	15.25	636450	{}	\N	2020-10-12 14:43:27.308309
38335	E	2020-10-05	15.48	15.871	15.43	15.83	1034471	{}	\N	2020-10-12 14:43:27.314333
38336	E	2020-10-06	16.32	16.46	15.67	15.67	1946766	{}	\N	2020-10-12 14:43:27.318956
38337	E	2020-10-07	15.82	15.91	15.67	15.87	962562	{}	\N	2020-10-12 14:43:27.323104
38338	E	2020-10-08	16.03	16.35	16.03	16.31	1059447	{}	\N	2020-10-12 14:43:27.327798
38339	E	2020-10-09	16.43	16.44	16.07	16.07	645623	{}	\N	2020-10-12 14:43:27.331946
38340	E	2020-10-12	16.02	16.0369	15.95	15.98	60676	{}	\N	2020-10-12 14:43:27.336222
38341	EAT	2020-09-28	45.74	46.09	44.78	45.43	1468016	{}	\N	2020-10-12 14:43:29.073429
38342	EAT	2020-09-29	45.32	45.32	43.02	43.57	3802451	{}	\N	2020-10-12 14:43:29.082384
38343	EAT	2020-09-30	43.66	44.83	42.37	42.72	3810759	{}	\N	2020-10-12 14:43:29.088429
38344	EAT	2020-10-01	43.09	43.84	42.55	43.01	3683765	{}	\N	2020-10-12 14:43:29.093963
38345	EAT	2020-10-02	41	43.63	40.66	43.56	4438218	{}	\N	2020-10-12 14:43:29.098964
38346	EAT	2020-10-05	44.36	45.17	43.21	43.6	2533640	{}	\N	2020-10-12 14:43:29.103994
38347	EAT	2020-10-06	44.24	45.75	42.9	42.95	2845209	{}	\N	2020-10-12 14:43:29.108346
38348	EAT	2020-10-07	44.18	45.27	43.32	45.04	3842679	{}	\N	2020-10-12 14:43:29.112473
38349	EAT	2020-10-08	45.59	47	44.28	46.84	2223542	{}	\N	2020-10-12 14:43:29.116908
38350	EAT	2020-10-09	47	48.38	46.34	47.31	3201138	{}	\N	2020-10-12 14:43:29.12127
38351	EAT	2020-10-12	47.31	47.31	46.06	46.335	573114	{}	\N	2020-10-12 14:43:29.125802
38352	DPG	2020-09-28	10.17	10.22	10.1001	10.13	98752	{}	\N	2020-10-12 14:43:30.933929
38353	DPG	2020-09-29	10.07	10.29	10.07	10.25	245976	{}	\N	2020-10-12 14:43:30.939936
38354	DPG	2020-09-30	10.35	10.45	10.21	10.32	567823	{}	\N	2020-10-12 14:43:30.944855
38355	DPG	2020-10-01	10.25	10.5	10.25	10.39	414035	{}	\N	2020-10-12 14:43:30.949704
38356	DPG	2020-10-02	10.3	10.48	10.25	10.47	272119	{}	\N	2020-10-12 14:43:30.955215
38357	DPG	2020-10-05	10.5	10.74	10.47	10.69	272564	{}	\N	2020-10-12 14:43:30.960874
38358	DPG	2020-10-06	10.76	10.93	10.69	10.76	324965	{}	\N	2020-10-12 14:43:30.965928
38359	DPG	2020-10-07	10.83	10.87	10.65	10.74	438392	{}	\N	2020-10-12 14:43:30.970615
38360	DPG	2020-10-08	10.8	10.92	10.79	10.92	323112	{}	\N	2020-10-12 14:43:30.975299
38361	DPG	2020-10-09	10.98	10.98	10.81	10.92	296679	{}	\N	2020-10-12 14:43:30.980428
38362	DPG	2020-10-12	10.87	10.95	10.87	10.9	32468	{}	\N	2020-10-12 14:43:30.985551
38363	DRN	2020-09-28	9.83	10.275	9.83	10.15	605857	{}	\N	2020-10-12 14:43:33.027836
38364	DRN	2020-09-29	10.18	10.2689	9.69	9.93	289219	{}	\N	2020-10-12 14:43:33.038516
38365	DRN	2020-09-30	10.06	10.36	9.71	10.03	418575	{}	\N	2020-10-12 14:43:33.045305
38366	DRN	2020-10-01	10.16	10.63	10.01	10.62	480287	{}	\N	2020-10-12 14:43:33.051575
38367	DRN	2020-10-02	10.06	11.26	10.03	11.14	798007	{}	\N	2020-10-12 14:43:33.057657
38368	DRN	2020-10-05	11.3	11.44	10.74	11.36	449084	{}	\N	2020-10-12 14:43:33.063059
38369	DRN	2020-10-06	11.39	11.63	11.01	11.16	402000	{}	\N	2020-10-12 14:43:33.067753
38370	DRN	2020-10-07	11.39	11.41	11.03	11.19	263639	{}	\N	2020-10-12 14:43:33.072495
38371	DRN	2020-10-08	11.33	11.84	11.33	11.73	366428	{}	\N	2020-10-12 14:43:33.077241
38372	DRN	2020-10-09	11.88	11.96	11.48	11.62	239003	{}	\N	2020-10-12 14:43:33.081689
38373	DRN	2020-10-12	11.72	11.7598	11.43	11.63	63532	{}	\N	2020-10-12 14:43:33.086628
38374	DSGX	2020-09-28	54.1	54.37	53.31	53.75	153495	{}	\N	2020-10-12 14:43:34.518331
38375	DSGX	2020-09-29	53.91	55.83	53.91	55.7	164776	{}	\N	2020-10-12 14:43:34.527705
38376	DSGX	2020-09-30	55.69	57.27	55.44	56.98	428237	{}	\N	2020-10-12 14:43:34.533856
38377	DSGX	2020-10-01	57.51	58.85	57.51	57.77	204569	{}	\N	2020-10-12 14:43:34.539624
38378	DSGX	2020-10-02	56.76	57.38	55.28	55.84	118441	{}	\N	2020-10-12 14:43:34.544412
38379	DSGX	2020-10-05	56.37	56.66	55.79	56.22	59874	{}	\N	2020-10-12 14:43:34.549017
38380	DSGX	2020-10-06	56.02	56.81	55.28	56.06	67138	{}	\N	2020-10-12 14:43:34.552827
38381	DSGX	2020-10-07	56.52	56.7499	55.58	55.86	96463	{}	\N	2020-10-12 14:43:34.556814
38382	DSGX	2020-10-08	56.11	56.57	55.06	56.11	93428	{}	\N	2020-10-12 14:43:34.560395
38383	DSGX	2020-10-09	56.4	57.54	55.88	57.09	58054	{}	\N	2020-10-12 14:43:34.564244
38384	DSGX	2020-10-12	57.74	57.92	56.97	57.17	8583	{}	\N	2020-10-12 14:43:34.568488
38385	DSWL	2020-09-28	2.54	2.58	2.52	2.555	7997	{}	\N	2020-10-12 14:43:36.14129
38386	DSWL	2020-09-29	2.53	3.25	2.53	2.88	551035	{}	\N	2020-10-12 14:43:36.148416
38387	DSWL	2020-09-30	2.85	2.99	2.71	2.71	13726	{}	\N	2020-10-12 14:43:36.153235
38388	DSWL	2020-10-01	2.71	2.79	2.5487	2.66	13733	{}	\N	2020-10-12 14:43:36.158327
38389	DSWL	2020-10-02	2.76	2.76	2.71	2.75	3181	{}	\N	2020-10-12 14:43:36.163263
38390	DSWL	2020-10-05	2.6105	2.74	2.6105	2.6973	14611	{}	\N	2020-10-12 14:43:36.167731
38391	DSWL	2020-10-06	2.7	2.78	2.61	2.61	4620	{}	\N	2020-10-12 14:43:36.17264
38392	DSWL	2020-10-07	2.69	2.69	2.65	2.66	1458	{}	\N	2020-10-12 14:43:36.177698
38393	DSWL	2020-10-08	2.7	2.72	2.68	2.7014	7881	{}	\N	2020-10-12 14:43:36.181929
38394	DSWL	2020-10-09	2.61	2.72	2.61	2.72	2171	{}	\N	2020-10-12 14:43:36.186725
38395	DSWL	2020-10-12	2.71	2.73	2.71	2.73	1185	{}	\N	2020-10-12 14:43:36.191734
38396	DWSN	2020-09-28	1.74	1.96	1.68	1.75	105411	{}	\N	2020-10-12 14:43:39.071286
38397	DWSN	2020-09-29	1.81	1.83	1.7201	1.83	13673	{}	\N	2020-10-12 14:43:39.080797
38398	DWSN	2020-09-30	1.8251	1.85	1.75	1.76	18468	{}	\N	2020-10-12 14:43:39.087398
38399	DWSN	2020-10-01	1.7385	1.7799	1.705	1.71	13289	{}	\N	2020-10-12 14:43:39.093639
38400	DWSN	2020-10-02	1.71	1.8	1.66	1.77	32661	{}	\N	2020-10-12 14:43:39.098767
38401	DWSN	2020-10-05	1.79	1.85	1.79	1.81	24773	{}	\N	2020-10-12 14:43:39.10414
38402	DWSN	2020-10-06	1.8	2.2	1.7916	2.0599	445477	{}	\N	2020-10-12 14:43:39.108717
38403	DWSN	2020-10-07	2.02	2.1	1.85	2.1	458447	{}	\N	2020-10-12 14:43:39.113179
38404	DWSN	2020-10-08	2.09	2.11	1.985	2.08	422351	{}	\N	2020-10-12 14:43:39.117525
38405	DWSN	2020-10-09	2.12	2.1434	1.91	2.04	44828	{}	\N	2020-10-12 14:43:39.122096
38406	DWSN	2020-10-12	2.06	2.1045	1.98	1.98	7801	{}	\N	2020-10-12 14:43:39.126638
38407	DXD	2020-09-28	15.94	16.0001	15.72	15.86	1610961	{}	\N	2020-10-12 14:43:40.464832
38408	DXD	2020-09-29	15.89	16.16	15.86	16.04	1302457	{}	\N	2020-10-12 14:43:40.475702
38409	DXD	2020-09-30	15.98	15.98	15.3608	15.66	3257843	{}	\N	2020-10-12 14:43:40.483208
38410	DXD	2020-10-01	15.46	15.77	15.36	15.6	1211028	{}	\N	2020-10-12 14:43:40.489446
38411	DXD	2020-10-02	16.07	16.11	15.56	15.76	2557908	{}	\N	2020-10-12 14:43:40.494344
38412	DXD	2020-10-05	15.5242	15.53	15.21	15.22	1208348	{}	\N	2020-10-12 14:43:40.499689
38413	DXD	2020-10-06	15.1	15.68	15.0019	15.64	2682736	{}	\N	2020-10-12 14:43:40.504527
38414	DXD	2020-10-07	15.35	15.35	14.9516	15.03	1629584	{}	\N	2020-10-12 14:43:40.50897
38415	DXD	2020-10-08	14.88	15.065	14.8508	14.89	1285816	{}	\N	2020-10-12 14:43:40.513416
38416	DXD	2020-10-09	14.75	14.8709	14.6301	14.73	1506164	{}	\N	2020-10-12 14:43:40.517837
38417	DXD	2020-10-12	14.61	14.64	14.5148	14.55	263124	{}	\N	2020-10-12 14:43:40.522148
38418	DZK	2020-09-28	43.01	43.3	42.9201	43.3	2949	{}	\N	2020-10-12 14:43:41.911336
38419	DZK	2020-09-29	43.2	43.2	42.87	42.87	825	{}	\N	2020-10-12 14:43:41.919212
38420	DZK	2020-09-30	42.9352	43.07	42.7464	42.7464	938	{}	\N	2020-10-12 14:43:41.925044
38421	DZK	2020-10-01	43.25	43.25	42.55	43.07	9010	{}	\N	2020-10-12 14:43:41.930538
38422	DZK	2020-10-02	43.091	43.091	42.9714	42.9714	781	{}	\N	2020-10-12 14:43:41.93569
38423	DZK	2020-10-05	44.2601	44.8687	44.2601	44.8687	780	{}	\N	2020-10-12 14:43:41.941172
38424	DZK	2020-10-06	44.5304	45	43.587	43.587	1106	{}	\N	2020-10-12 14:43:41.945683
38425	DZK	2020-10-07	44.28	44.74	44.28	44.74	7099	{}	\N	2020-10-12 14:43:41.950147
38426	DZK	2020-10-08	45.36	45.5291	45.2447	45.5291	1604	{}	\N	2020-10-12 14:43:41.954451
38427	DZK	2020-10-09	46.23	46.77	46.1704	46.51	4188	{}	\N	2020-10-12 14:43:41.958789
38428	EBAY	2020-09-28	53.2	54.36	53.1	54.04	6167296	{}	\N	2020-10-12 14:43:42.947622
38429	EBAY	2020-09-29	53.98	54.3	52.59	52.65	6715129	{}	\N	2020-10-12 14:43:42.957559
38430	EBAY	2020-09-30	52.98	53.62	51.51	52.1	8402505	{}	\N	2020-10-12 14:43:42.964314
38431	EBAY	2020-10-01	52.37	52.86	52	52.38	6375429	{}	\N	2020-10-12 14:43:42.970272
38432	EBAY	2020-10-02	51.68	52.4299	51.19	51.68	5065640	{}	\N	2020-10-12 14:43:42.975551
38433	EBAY	2020-10-05	51.94	52.105	51.15	51.77	4895064	{}	\N	2020-10-12 14:43:42.980664
38434	EBAY	2020-10-06	51.77	51.78	49.485	49.66	8230537	{}	\N	2020-10-12 14:43:42.984917
38435	EBAY	2020-10-07	50.55	51.11	49.98	50.94	5921521	{}	\N	2020-10-12 14:43:42.989093
38436	EBAY	2020-10-08	51.3	52.18	50.98	52.08	6615743	{}	\N	2020-10-12 14:43:42.993499
38437	EBAY	2020-10-09	52.43	55.86	52.32	55.45	12492355	{}	\N	2020-10-12 14:43:42.997673
38438	EBAY	2020-10-12	56.2	56.2	54.53	54.92	2202841	{}	\N	2020-10-12 14:43:43.001703
38439	DVA	2020-09-28	84.41	85.86	84.06	84.5	913195	{}	\N	2020-10-12 14:43:44.622487
38440	DVA	2020-09-29	84.63	85.16	83.38	84.14	1691941	{}	\N	2020-10-12 14:43:44.632424
38441	DVA	2020-09-30	83.82	86.61	83.76	85.65	2156897	{}	\N	2020-10-12 14:43:44.639373
38442	DVA	2020-10-01	86.48	86.87	80.85	82.82	5167084	{}	\N	2020-10-12 14:43:44.645432
38443	DVA	2020-10-02	81.66	84.9	81.02	84.29	2166807	{}	\N	2020-10-12 14:43:44.650652
38444	DVA	2020-10-05	84.67	87.23	84.67	86.63	2059682	{}	\N	2020-10-12 14:43:44.6555
38445	DVA	2020-10-06	86.71	87.11	84.81	85.31	2329599	{}	\N	2020-10-12 14:43:44.659895
38446	DVA	2020-10-07	85.55	87.9	85.55	87.19	1764147	{}	\N	2020-10-12 14:43:44.664056
38447	DVA	2020-10-08	87.65	88.84	87.48	88.6	1361381	{}	\N	2020-10-12 14:43:44.667988
38448	DVA	2020-10-09	89.46	89.74	86.56	87.86	1831927	{}	\N	2020-10-12 14:43:44.672171
38449	DVA	2020-10-12	87.89	89.42	87.89	88.73	321976	{}	\N	2020-10-12 14:43:44.676051
38450	DVY	2020-09-28	81.41	82.395	81.3	81.93	615095	{}	\N	2020-10-12 14:43:45.987862
38451	DVY	2020-09-29	81.95	82	80.6	81.14	519151	{}	\N	2020-10-12 14:43:45.99937
38452	DVY	2020-09-30	81.45	82.345	81.04	81.59	746537	{}	\N	2020-10-12 14:43:46.007233
38453	DVY	2020-10-01	81.64	82.02	80.74	81.33	481904	{}	\N	2020-10-12 14:43:46.013432
38454	DVY	2020-10-02	80.12	83.01	80.06	82.57	678707	{}	\N	2020-10-12 14:43:46.019289
38455	DVY	2020-10-05	83.16	84.38	83.16	84.24	544296	{}	\N	2020-10-12 14:43:46.024141
38456	DVY	2020-10-06	84.72	86.11	83.83	84.01	544288	{}	\N	2020-10-12 14:43:46.029073
38457	DVY	2020-10-07	84.86	85.79	84.86	85.47	557379	{}	\N	2020-10-12 14:43:46.033642
38458	DVY	2020-10-08	85.94	87.04	85.84	86.96	430058	{}	\N	2020-10-12 14:43:46.038091
38459	DVY	2020-10-09	87.48	87.7611	86.47	86.6	452004	{}	\N	2020-10-12 14:43:46.04239
38460	DVY	2020-10-12	86.86	87.17	86.55	87.0767	73780	{}	\N	2020-10-12 14:43:46.047317
38461	DWPP	2020-09-28	25.1701	25.1701	25.17	25.17	232	{}	\N	2020-10-12 14:43:49.06274
38462	DWPP	2020-09-29	25.0708	25.0732	25.05	25.05	1511	{}	\N	2020-10-12 14:43:49.072686
38463	DWPP	2020-09-30	25.3001	25.41	25.09	25.09	11265	{}	\N	2020-10-12 14:43:49.078935
38464	DWPP	2020-10-01	25.41	25.41	25.365	25.37	2860	{}	\N	2020-10-12 14:43:49.083994
38465	DWPP	2020-10-02	25.21	25.285	25.2	25.2	1698	{}	\N	2020-10-12 14:43:49.0882
38466	DWPP	2020-10-05	25.47	25.6402	25.465	25.6402	1362	{}	\N	2020-10-12 14:43:49.092789
38467	DWPP	2020-10-07	25.6599	25.71	25.6599	25.71	551	{}	\N	2020-10-12 14:43:49.097422
38468	DWPP	2020-10-08	25.88	25.908	25.865	25.908	2202	{}	\N	2020-10-12 14:43:49.101474
38469	DWPP	2020-10-09	26.2196	26.2196	26.1627	26.1627	754	{}	\N	2020-10-12 14:43:49.10564
38470	DXJS	2020-09-28	39.81	39.9	39.802	39.9	2428	{}	\N	2020-10-12 14:43:50.489693
38471	DXJS	2020-09-29	39.85	39.85	39.85	39.85	630	{}	\N	2020-10-12 14:43:50.497223
38472	DXJS	2020-09-30	39.3415	39.41	39.27	39.41	1419	{}	\N	2020-10-12 14:43:50.503492
38473	DXJS	2020-10-01	39.37	39.37	39.3694	39.3694	408	{}	\N	2020-10-12 14:43:50.508492
38474	DXJS	2020-10-02	38.9025	39.06	38.895	39.06	1363	{}	\N	2020-10-12 14:43:50.513182
38475	DXJS	2020-10-05	39.6007	39.6007	39.51	39.51	813	{}	\N	2020-10-12 14:43:50.517636
38476	DXJS	2020-10-06	39.58	39.58	39.58	39.58	248	{}	\N	2020-10-12 14:43:50.522189
38477	DXJS	2020-10-07	39.56	39.6845	39.56	39.6	880	{}	\N	2020-10-12 14:43:50.526801
38478	DXJS	2020-10-08	39.71	39.875	39.67	39.685	6034	{}	\N	2020-10-12 14:43:50.531312
38479	DXJS	2020-10-09	39.51	39.51	39.51	39.51	144	{}	\N	2020-10-12 14:43:50.535574
38480	EAD	2020-09-28	7.31	7.3607	7.2955	7.36	182452	{}	\N	2020-10-12 14:43:52.999015
38481	EAD	2020-09-29	7.37	7.37	7.31	7.35	181956	{}	\N	2020-10-12 14:43:53.009004
38482	EAD	2020-09-30	7.37	7.39	7.3	7.3	283891	{}	\N	2020-10-12 14:43:53.016277
38483	EAD	2020-10-01	7.39	7.42	7.35	7.4	403791	{}	\N	2020-10-12 14:43:53.023052
38484	EAD	2020-10-02	7.37	7.44	7.3571	7.44	82300	{}	\N	2020-10-12 14:43:53.028719
38485	EAD	2020-10-05	7.44	7.49	7.43	7.48	199428	{}	\N	2020-10-12 14:43:53.033156
38486	EAD	2020-10-06	7.46	7.5	7.45	7.46	153062	{}	\N	2020-10-12 14:43:53.037549
38487	EAD	2020-10-07	7.49	7.4967	7.47	7.47	75833	{}	\N	2020-10-12 14:43:53.041451
38488	EAD	2020-10-08	7.48	7.51	7.48	7.5	104988	{}	\N	2020-10-12 14:43:53.04559
38489	EAD	2020-10-09	7.49	7.54	7.49	7.54	190771	{}	\N	2020-10-12 14:43:53.049649
38490	EAD	2020-10-12	7.55	7.55	7.53	7.54	25476	{}	\N	2020-10-12 14:43:53.053612
38491	EBF	2020-09-28	17.36	17.76	17.33	17.61	158953	{}	\N	2020-10-12 14:43:54.345171
38492	EBF	2020-09-29	17.59	17.63	17.22	17.47	147912	{}	\N	2020-10-12 14:43:54.354948
38493	EBF	2020-09-30	17.5	17.77	17.44	17.44	262443	{}	\N	2020-10-12 14:43:54.360346
38494	EBF	2020-10-01	17.45	17.64	17.34	17.63	204942	{}	\N	2020-10-12 14:43:54.365739
38495	EBF	2020-10-02	17.44	17.94	17.35	17.83	141111	{}	\N	2020-10-12 14:43:54.370507
38496	EBF	2020-10-05	17.97	17.99	17.76	17.8	171573	{}	\N	2020-10-12 14:43:54.374749
38497	EBF	2020-10-06	17.83	18.28	17.74	17.84	221964	{}	\N	2020-10-12 14:43:54.379197
38498	EBF	2020-10-07	17.96	18.19	17.84	17.97	208226	{}	\N	2020-10-12 14:43:54.383285
38499	EBF	2020-10-08	17.81	18	17.69	17.78	185991	{}	\N	2020-10-12 14:43:54.387273
38500	EBF	2020-10-09	17.98	17.99	17.52	17.54	191800	{}	\N	2020-10-12 14:43:54.391421
38501	EBF	2020-10-12	17.52	17.69	17.35	17.51	50134	{}	\N	2020-10-12 14:43:54.395958
38502	EC	2020-09-28	10.08	10.23	10.0586	10.15	679455	{}	\N	2020-10-12 14:43:55.327761
38503	EC	2020-09-29	10.1	10.12	9.79	9.79	1829381	{}	\N	2020-10-12 14:43:55.339538
38504	EC	2020-09-30	9.83	9.97	9.8	9.84	1891300	{}	\N	2020-10-12 14:43:55.351835
38505	EC	2020-10-01	9.76	9.83	9.62	9.69	2831349	{}	\N	2020-10-12 14:43:55.358315
38506	EC	2020-10-02	9.49	9.9	9.45	9.8	1377810	{}	\N	2020-10-12 14:43:55.362783
38507	EC	2020-10-05	9.9	10.07	9.87	10.02	1990346	{}	\N	2020-10-12 14:43:55.367429
38508	EC	2020-10-06	10.17	10.26	9.9	9.93	1920167	{}	\N	2020-10-12 14:43:55.372313
38509	EC	2020-10-07	9.97	9.995	9.68	9.92	1863820	{}	\N	2020-10-12 14:43:55.377152
38510	EC	2020-10-08	9.95	10.22	9.95	10.18	2588636	{}	\N	2020-10-12 14:43:55.382519
38511	EC	2020-10-09	10.31	10.35	10.011	10.05	1518153	{}	\N	2020-10-12 14:43:55.387484
38512	EC	2020-10-12	9.99	9.99	9.785	9.81	257138	{}	\N	2020-10-12 14:43:55.392646
38513	ECL	2020-09-28	199.91	201.44	197.69	198.27	1339257	{}	\N	2020-10-12 14:43:56.333571
38514	ECL	2020-09-29	199	199.26	196.82	197.45	1516660	{}	\N	2020-10-12 14:43:56.344894
38515	ECL	2020-09-30	198.69	201.63	198.08	199.84	2636794	{}	\N	2020-10-12 14:43:56.352394
38516	ECL	2020-10-01	201.78	203.75	196.12	197.08	2321998	{}	\N	2020-10-12 14:43:56.359144
38517	ECL	2020-10-02	195.07	199.84	194.5	198.06	2187928	{}	\N	2020-10-12 14:43:56.364473
38518	ECL	2020-10-05	200.48	202.29	199.09	199.39	2597875	{}	\N	2020-10-12 14:43:56.369566
38519	ECL	2020-10-06	200	201.9	196.19	196.43	2474057	{}	\N	2020-10-12 14:43:56.373939
38520	ECL	2020-10-07	199.64	203.28	199.57	201.4	2670096	{}	\N	2020-10-12 14:43:56.378056
38521	ECL	2020-10-08	203.34	203.47	201.32	201.46	1648568	{}	\N	2020-10-12 14:43:56.382469
38522	ECL	2020-10-09	203.64	204.715	202.91	203.16	1305570	{}	\N	2020-10-12 14:43:56.386557
38523	ECL	2020-10-12	204.35	204.91	203.01	203.15	287290	{}	\N	2020-10-12 14:43:56.390823
38524	EDC	2020-09-28	55.59	55.93	54.51	55.23	68807	{}	\N	2020-10-12 14:43:58.167415
38525	EDC	2020-09-29	54.72	55.585	54.67	55.23	43396	{}	\N	2020-10-12 14:43:58.176795
38526	EDC	2020-09-30	56.42	58.22	56.42	57.65	105113	{}	\N	2020-10-12 14:43:58.183193
38527	EDC	2020-10-01	59.15	59.51	58.13	59.3	108792	{}	\N	2020-10-12 14:43:58.189348
38528	EDC	2020-10-02	56.71	58.8	56.6	57.38	90988	{}	\N	2020-10-12 14:43:58.194964
38529	EDC	2020-10-05	58.36	59.65	58.33	59.62	115726	{}	\N	2020-10-12 14:43:58.199808
38530	EDC	2020-10-06	60.3	61.3332	59.44	59.9	103074	{}	\N	2020-10-12 14:43:58.203837
38531	EDC	2020-10-07	61.62	62.5	61.22	62.05	148873	{}	\N	2020-10-12 14:43:58.20783
38532	EDC	2020-10-08	62.68	63.65	62.281	63.45	77558	{}	\N	2020-10-12 14:43:58.212163
38533	EDC	2020-10-09	63.96	65.27	63.44	64.85	93829	{}	\N	2020-10-12 14:43:58.216319
38534	EDC	2020-10-12	65.772	66.1752	65.5	66.1752	31290	{}	\N	2020-10-12 14:43:58.220405
38535	FITB	2020-09-28	20.92	21.5189	20.78	21.34	6140908	{}	\N	2020-10-12 14:44:00.380784
38536	FITB	2020-09-29	21.25	21.36	20.69	20.99	5358638	{}	\N	2020-10-12 14:44:00.389562
38537	FITB	2020-09-30	21.16	21.62	21.115	21.32	5275042	{}	\N	2020-10-12 14:44:00.395558
38538	FITB	2020-10-01	21.15	21.29	20.83	21.03	4060443	{}	\N	2020-10-12 14:44:00.401584
38539	FITB	2020-10-02	20.52	22.045	20.52	21.89	5777425	{}	\N	2020-10-12 14:44:00.406675
38540	FITB	2020-10-05	21.99	22.93	21.99	22.75	5364007	{}	\N	2020-10-12 14:44:00.411642
38541	FITB	2020-10-06	23.06	23.625	22.39	22.52	5072411	{}	\N	2020-10-12 14:44:00.415967
38542	FITB	2020-10-07	22.99	23.6	22.8001	23.33	4255061	{}	\N	2020-10-12 14:44:00.419869
38543	FITB	2020-10-08	23.53	23.815	23.305	23.71	4067414	{}	\N	2020-10-12 14:44:00.424424
38544	FITB	2020-10-09	23.92	23.93	23.225	23.38	5276502	{}	\N	2020-10-12 14:44:00.428924
38545	FITB	2020-10-12	23.26	23.52	23.22	23.41	1438492	{}	\N	2020-10-12 14:44:00.433531
38546	DTY	2020-09-28	26.31	26.38	26.24	26.32	20856	{}	\N	2020-10-12 14:44:01.424563
38547	DTY	2020-09-29	26.33	26.3499	26.1656	26.23	49614	{}	\N	2020-10-12 14:44:01.434015
38548	DTY	2020-09-30	26.15	26.35	26.09	26.35	158908	{}	\N	2020-10-12 14:44:01.440149
38549	DTY	2020-10-01	26.24	26.46	26.11	26.46	47550	{}	\N	2020-10-12 14:44:01.445849
38550	DTY	2020-10-02	26.4	26.4599	26.3	26.44	45522	{}	\N	2020-10-12 14:44:01.450552
38551	DTY	2020-10-05	26.44	26.44	26.27	26.4	31762	{}	\N	2020-10-12 14:44:01.455361
38552	DTY	2020-10-06	26.355	26.4	26.22	26.37	33110	{}	\N	2020-10-12 14:44:01.459673
38553	DTY	2020-10-07	26.32	26.39	26.32	26.38	22414	{}	\N	2020-10-12 14:44:01.464363
38554	DTY	2020-10-08	26.375	26.435	26.375	26.4	21395	{}	\N	2020-10-12 14:44:01.468279
38555	DTY	2020-10-09	26.365	26.41	26.29	26.39	15148	{}	\N	2020-10-12 14:44:01.472404
38556	DTY	2020-10-12	26.39	26.3984	26.26	26.325	7460	{}	\N	2020-10-12 14:44:01.477103
38557	DVN	2020-09-28	9.56	10.02	8.985	9.8	39940718	{}	\N	2020-10-12 14:44:02.486121
38558	DVN	2020-09-29	9.76	9.77	8.97	9.57	50769309	{}	\N	2020-10-12 14:44:02.498087
38559	DVN	2020-09-30	9.61	9.76	9.4	9.46	23722482	{}	\N	2020-10-12 14:44:02.504882
38560	DVN	2020-10-01	9.41	9.54	9.02	9.1	23928693	{}	\N	2020-10-12 14:44:02.515687
38561	DVN	2020-10-02	8.7	9.86	8.65	9.6	28495589	{}	\N	2020-10-12 14:44:02.521088
38562	DVN	2020-10-05	9.87	10.16	9.62	10.15	24346268	{}	\N	2020-10-12 14:44:02.52619
38563	DVN	2020-10-06	10.45	10.59	9.69	9.73	22249973	{}	\N	2020-10-12 14:44:02.530927
38564	DVN	2020-10-07	9.8	9.95	9.59	9.87	18154305	{}	\N	2020-10-12 14:44:02.535427
38565	DVN	2020-10-08	9.97	10.33	9.84	10.32	17253974	{}	\N	2020-10-12 14:44:02.539682
38566	DVN	2020-10-09	10.41	10.46	10	10.07	17715451	{}	\N	2020-10-12 14:44:02.544001
38567	DVN	2020-10-12	10	10.03	9.83	9.92	3100829	{}	\N	2020-10-12 14:44:02.548514
38568	DWAT	2020-10-06	11.26	11.26	11.26	11.26	349	{}	\N	2020-10-12 14:44:04.144332
38569	DWAT	2020-10-09	11.24	11.2999	11.12	11.12	3625	{}	\N	2020-10-12 14:44:04.151036
38570	ECC	2020-09-28	8.17	8.49	8.1	8.4	79327	{}	\N	2020-10-12 14:44:06.075835
38571	ECC	2020-09-29	8.32	8.48	8.32	8.415	84886	{}	\N	2020-10-12 14:44:06.085383
38572	ECC	2020-09-30	8.45	8.63	8.43	8.62	209630	{}	\N	2020-10-12 14:44:06.092114
38573	ECC	2020-10-01	8.6	8.8	8.5797	8.71	305777	{}	\N	2020-10-12 14:44:06.097992
38574	ECC	2020-10-02	8.56	8.6427	8.4	8.58	168720	{}	\N	2020-10-12 14:44:06.104208
38575	ECC	2020-10-05	8.55	8.94	8.55	8.86	356616	{}	\N	2020-10-12 14:44:06.109031
38576	ECC	2020-10-06	8.96	9.09	8.87	8.96	238027	{}	\N	2020-10-12 14:44:06.113626
38577	ECC	2020-10-07	9.12	9.12	8.94	9.02	156704	{}	\N	2020-10-12 14:44:06.118264
38578	ECC	2020-10-08	9	9.2	9	9.19	273774	{}	\N	2020-10-12 14:44:06.12293
38579	ECC	2020-10-09	9.2	9.535	9.12	9.49	412625	{}	\N	2020-10-12 14:44:06.128605
38580	ECC	2020-10-12	9.43	9.46	9.03	9.0781	216462	{}	\N	2020-10-12 14:44:06.13329
38581	EDEN	2020-09-28	83.86	84.025	83.2	83.99	6292	{}	\N	2020-10-12 14:44:08.120815
38582	EDEN	2020-09-29	84.75	85.44	84.75	85.275	115836	{}	\N	2020-10-12 14:44:08.12915
38583	EDEN	2020-09-30	85.06	85.06	84.5	84.655	11355	{}	\N	2020-10-12 14:44:08.134691
38584	EDEN	2020-10-01	86.02	86.14	85.51	86.1	39556	{}	\N	2020-10-12 14:44:08.140019
38585	EDEN	2020-10-02	85.5	85.85	85.29	85.655	14838	{}	\N	2020-10-12 14:44:08.144775
38586	EDEN	2020-10-05	86.4	87.53	86.11	87.46	17841	{}	\N	2020-10-12 14:44:08.149433
38587	EDEN	2020-10-06	86.885	87	86.01	86.12	10631	{}	\N	2020-10-12 14:44:08.153267
38588	EDEN	2020-10-07	87.24	87.25	86.65	87.085	12996	{}	\N	2020-10-12 14:44:08.156919
38589	EDEN	2020-10-08	87.23	88.29	87.12	88.22	9267	{}	\N	2020-10-12 14:44:08.160513
38590	EDEN	2020-10-09	88.98	89.57	88.98	89.49	23187	{}	\N	2020-10-12 14:44:08.164251
38591	EDEN	2020-10-12	90.575	90.64	90.57	90.64	1010	{}	\N	2020-10-12 14:44:08.167905
38592	EDU	2020-09-28	151.78	151.795	147.41	149.66	893604	{}	\N	2020-10-12 14:44:09.191707
38593	EDU	2020-09-29	149.62	149.62	145.33	147.23	1686411	{}	\N	2020-10-12 14:44:09.20008
38594	EDU	2020-09-30	146.89	149.53	146.41	149.5	1683341	{}	\N	2020-10-12 14:44:09.206086
38595	EDU	2020-10-01	151.53	156.59	150.64	154.56	1174528	{}	\N	2020-10-12 14:44:09.212133
38596	EDU	2020-10-02	151.76	155.89	151.58	154.15	883800	{}	\N	2020-10-12 14:44:09.217475
38597	EDU	2020-10-05	153.92	154.67	151.49	151.78	1549831	{}	\N	2020-10-12 14:44:09.222283
38598	EDU	2020-10-06	153.77	164.75	153.48	163.81	2482115	{}	\N	2020-10-12 14:44:09.226418
38599	EDU	2020-10-07	164.27	167.08	161.25	165.46	2844256	{}	\N	2020-10-12 14:44:09.23049
38600	EDU	2020-10-08	166.63	168.06	164.1701	167.71	1422352	{}	\N	2020-10-12 14:44:09.234503
38601	EDU	2020-10-09	167.64	171.56	167.05	170.09	1251349	{}	\N	2020-10-12 14:44:09.238328
38602	EDU	2020-10-12	170.7	172	170.1	170.9	409485	{}	\N	2020-10-12 14:44:09.243503
38603	EELV	2020-09-28	19.94	20.01	19.8401	19.88	410225	{}	\N	2020-10-12 14:44:11.052653
38604	EELV	2020-09-29	19.905	19.97	19.87	19.91	11739	{}	\N	2020-10-12 14:44:11.063401
38605	EELV	2020-09-30	20.04	20.15	20.0001	20.13	34145	{}	\N	2020-10-12 14:44:11.070414
38606	EELV	2020-10-01	20.22	20.23	20.12	20.15	57115	{}	\N	2020-10-12 14:44:11.076333
38607	EELV	2020-10-02	20.12	20.13	20.0104	20.08	41960	{}	\N	2020-10-12 14:44:11.081976
38608	EELV	2020-10-05	20.12	20.21	20.08	20.21	41486	{}	\N	2020-10-12 14:44:11.086887
38609	EELV	2020-10-06	20.1994	20.29	20.165	20.19	13582	{}	\N	2020-10-12 14:44:11.09128
38610	EELV	2020-10-07	20.3	20.3	20.2278	20.26	23496	{}	\N	2020-10-12 14:44:11.095398
38611	EELV	2020-10-08	20.32	20.42	20.3	20.39	69431	{}	\N	2020-10-12 14:44:11.099602
38612	EELV	2020-10-09	20.42	20.55	20.42	20.55	40288	{}	\N	2020-10-12 14:44:11.103644
38613	EELV	2020-10-12	20.487	20.487	20.487	20.487	1189	{}	\N	2020-10-12 14:44:11.107662
38614	EEX	2020-09-28	1.78	2.41	1.78	2.24	826523	{}	\N	2020-10-12 14:44:12.701788
38615	EEX	2020-09-29	2.21	2.2133	1.92	2	328199	{}	\N	2020-10-12 14:44:12.714141
38616	EEX	2020-09-30	2.02	2.12	2	2.04	168977	{}	\N	2020-10-12 14:44:12.721411
38617	EEX	2020-10-01	2.04	2.07	1.95	2.02	864952	{}	\N	2020-10-12 14:44:12.727219
38618	EEX	2020-10-02	1.92	2.26	1.92	2.23	224190	{}	\N	2020-10-12 14:44:12.732672
38619	EEX	2020-10-05	2.23	2.38	2.22	2.32	1456172	{}	\N	2020-10-12 14:44:12.737394
38620	EEX	2020-10-06	2.44	2.85	2.3956	2.67	3068764	{}	\N	2020-10-12 14:44:12.741834
38621	EEX	2020-10-07	2.73	3	2.6907	2.85	1065711	{}	\N	2020-10-12 14:44:12.746236
38622	EEX	2020-10-08	2.95	2.9623	2.77	2.89	641034	{}	\N	2020-10-12 14:44:12.750972
38623	EEX	2020-10-09	2.79	2.85	2.66	2.81	362173	{}	\N	2020-10-12 14:44:12.755261
38624	EEX	2020-10-12	2.76	2.85	2.76	2.82	49686	{}	\N	2020-10-12 14:44:12.759427
38625	DX	2020-09-28	15.32	15.79	15.32	15.61	318012	{}	\N	2020-10-12 14:44:14.68836
38626	DX	2020-09-29	15.62	15.655	15.08	15.2	539356	{}	\N	2020-10-12 14:44:14.695073
38627	DX	2020-09-30	15.2	15.43	15.13	15.21	368819	{}	\N	2020-10-12 14:44:14.700341
38628	DX	2020-10-01	15.24	15.29	15.03	15.29	505444	{}	\N	2020-10-12 14:44:14.706438
38629	DX	2020-10-02	14.98	15.65	14.98	15.6	620735	{}	\N	2020-10-12 14:44:14.71408
38630	DX	2020-10-05	15.64	15.78	15.59	15.72	386375	{}	\N	2020-10-12 14:44:14.718606
38631	DX	2020-10-06	15.75	16.06	15.64	15.67	503115	{}	\N	2020-10-12 14:44:14.723653
38632	DX	2020-10-07	15.8	15.87	15.65	15.78	470310	{}	\N	2020-10-12 14:44:14.728339
38633	DX	2020-10-08	15.88	16.35	15.86	16.27	672181	{}	\N	2020-10-12 14:44:14.73342
38634	DX	2020-10-09	16.39	16.4	16.03	16.09	467571	{}	\N	2020-10-12 14:44:14.737817
38635	DX	2020-10-12	16.08	16.14	15.98	16.09	55546	{}	\N	2020-10-12 14:44:14.742356
38636	DXPE	2020-09-28	16.14	17.04	15.87	16.65	72721	{}	\N	2020-10-12 14:44:15.792683
38637	DXPE	2020-09-29	16.68	17.0399	16.1	16.39	60516	{}	\N	2020-10-12 14:44:15.802391
38638	DXPE	2020-09-30	16.43	17	15.83	16.13	95191	{}	\N	2020-10-12 14:44:15.808822
38639	DXPE	2020-10-01	16.23	16.5	15.98	16.34	85340	{}	\N	2020-10-12 14:44:15.814917
38640	DXPE	2020-10-02	15.88	17.03	15.85	16.89	106189	{}	\N	2020-10-12 14:44:15.819756
38641	DXPE	2020-10-05	17.25	18.05	17.14	17.71	76114	{}	\N	2020-10-12 14:44:15.824945
38642	DXPE	2020-10-06	18.26	18.675	17.39	17.73	73560	{}	\N	2020-10-12 14:44:15.829043
38643	DXPE	2020-10-07	17.235	18.4476	17.235	17.93	96900	{}	\N	2020-10-12 14:44:15.833248
38644	DXPE	2020-10-08	18.19	18.54	17.8	18.32	68684	{}	\N	2020-10-12 14:44:15.837112
38645	DXPE	2020-10-09	18.54	18.89	17.73	17.97	77689	{}	\N	2020-10-12 14:44:15.840973
38646	DXPE	2020-10-12	18.11	18.16	17.9	18.16	9854	{}	\N	2020-10-12 14:44:15.844665
38647	DYNT	2020-09-28	0.6887	0.6887	0.6602	0.6799	59860	{}	\N	2020-10-12 14:44:17.078845
38648	DYNT	2020-09-29	0.69	0.731	0.6601	0.6886	260264	{}	\N	2020-10-12 14:44:17.088906
38649	DYNT	2020-09-30	0.7001	0.7198	0.6901	0.7024	50270	{}	\N	2020-10-12 14:44:17.096477
38650	DYNT	2020-10-01	0.7052	0.7052	0.68021	0.6905	36669	{}	\N	2020-10-12 14:44:17.102597
38651	DYNT	2020-10-02	0.686	0.758899	0.686	0.7314	656337	{}	\N	2020-10-12 14:44:17.107768
38652	DYNT	2020-10-05	0.72	0.7305	0.6911	0.6953	134509	{}	\N	2020-10-12 14:44:17.112887
38653	DYNT	2020-10-06	0.686	0.7084	0.67	0.6903	80597	{}	\N	2020-10-12 14:44:17.117436
38654	DYNT	2020-10-07	0.6901	0.7199	0.6901	0.7136	65632	{}	\N	2020-10-12 14:44:17.122269
38655	DYNT	2020-10-08	0.7141	0.74	0.7101	0.73	61672	{}	\N	2020-10-12 14:44:17.126605
38656	DYNT	2020-10-09	0.7302	0.7398	0.7051	0.7209	293701	{}	\N	2020-10-12 14:44:17.130943
38657	DYNT	2020-10-12	0.7256	0.7299	0.7227	0.7238	4730	{}	\N	2020-10-12 14:44:17.135257
38658	EAB	2020-09-28	25.15	25.15	25.14	25.15	19474	{}	\N	2020-10-12 14:44:19.027733
38659	EAB	2020-09-29	25.15	25.15	25.13	25.1379	21984	{}	\N	2020-10-12 14:44:19.034038
38660	EAB	2020-09-30	25.14	25.15	25.13	25.13	86501	{}	\N	2020-10-12 14:44:19.039512
38661	EAB	2020-10-01	25.12	25.15	25.12	25.15	28728	{}	\N	2020-10-12 14:44:19.044264
38662	EAB	2020-10-02	25.15	25.15	25.14	25.145	20450	{}	\N	2020-10-12 14:44:19.049482
38663	EAB	2020-10-05	25.13	25.15	25.13	25.145	13312	{}	\N	2020-10-12 14:44:19.053917
38664	EAB	2020-10-06	25.13	25.15	25.13	25.15	27832	{}	\N	2020-10-12 14:44:19.058667
38665	EAB	2020-10-07	25.13	25.15	25.13	25.15	15010	{}	\N	2020-10-12 14:44:19.063042
38666	EAB	2020-10-08	25.15	25.15	25.13	25.15	13174	{}	\N	2020-10-12 14:44:19.067261
38667	EAB	2020-10-09	25.14	25.15	25.14	25.15	18476	{}	\N	2020-10-12 14:44:19.071327
38668	EAB	2020-10-12	25.15	25.15	25.15	25.15	1580	{}	\N	2020-10-12 14:44:19.075224
38669	EARS	2020-09-28	0.77	0.8	0.76	0.7951	274326	{}	\N	2020-10-12 14:44:20.066158
38670	EARS	2020-09-29	0.809	0.8195	0.7786	0.8193	98621	{}	\N	2020-10-12 14:44:20.074989
38671	EARS	2020-09-30	0.8178	0.82405	0.8002	0.807999	143098	{}	\N	2020-10-12 14:44:20.080415
38672	EARS	2020-10-01	0.8109	0.83	0.79305	0.8	84627	{}	\N	2020-10-12 14:44:20.085552
38673	EARS	2020-10-02	0.8	0.8006	0.7612	0.7923	71677	{}	\N	2020-10-12 14:44:20.09031
38674	EARS	2020-10-05	0.7935	0.85	0.77	0.8298	107760	{}	\N	2020-10-12 14:44:20.09453
38675	EARS	2020-10-06	0.827	0.8298	0.8	0.8005	106483	{}	\N	2020-10-12 14:44:20.097994
38676	EARS	2020-10-07	0.815	0.845	0.8	0.822	137540	{}	\N	2020-10-12 14:44:20.101683
38677	EARS	2020-10-08	0.825	0.8417	0.8054	0.84	122354	{}	\N	2020-10-12 14:44:20.105482
38678	EARS	2020-10-09	0.8496	0.865	0.8197	0.8473	184028	{}	\N	2020-10-12 14:44:20.109429
38679	EARS	2020-10-12	0.87	0.88	0.823	0.8449	48259	{}	\N	2020-10-12 14:44:20.113202
38680	EBMT	2020-09-28	17.02	17.48	17	17.46	17407	{}	\N	2020-10-12 14:44:21.080512
38681	EBMT	2020-09-29	17.45	17.66	17.2758	17.66	4200	{}	\N	2020-10-12 14:44:21.089705
38682	EBMT	2020-09-30	17.64	17.75	17.62	17.63	3393	{}	\N	2020-10-12 14:44:21.096807
38683	EBMT	2020-10-01	17.6	17.8	17.58	17.58	14423	{}	\N	2020-10-12 14:44:21.102949
38684	EBMT	2020-10-02	17.43	18.5	17.43	18.3	16535	{}	\N	2020-10-12 14:44:21.10752
38685	EBMT	2020-10-05	18.49	18.72	18.1519	18.58	13101	{}	\N	2020-10-12 14:44:21.112526
38686	EBMT	2020-10-06	18.5	18.92	18.41	18.6	25596	{}	\N	2020-10-12 14:44:21.116771
38687	EBMT	2020-10-07	18.64	18.82	18.52	18.82	8719	{}	\N	2020-10-12 14:44:21.120565
38688	EBMT	2020-10-08	18.855	18.99	18.6201	18.91	14265	{}	\N	2020-10-12 14:44:21.124495
38689	EBMT	2020-10-09	18.92	19.1382	18.82	19.1	11802	{}	\N	2020-10-12 14:44:21.12859
38690	EBMT	2020-10-12	19.1123	19.118	19.1123	19.118	743	{}	\N	2020-10-12 14:44:21.132259
38691	ECNS	2020-09-28	42.5	42.535	42.5	42.535	643	{}	\N	2020-10-12 14:44:22.97872
38692	ECNS	2020-09-29	42.54	42.9	42.54	42.9	3245	{}	\N	2020-10-12 14:44:22.989604
38693	ECNS	2020-09-30	43.22	43.4619	43.22	43.4619	1286	{}	\N	2020-10-12 14:44:22.996938
38694	ECNS	2020-10-01	43.87	44.37	43.82	44.16	14300	{}	\N	2020-10-12 14:44:23.002838
38695	ECNS	2020-10-02	44.045	44.045	44.045	44.045	417	{}	\N	2020-10-12 14:44:23.007411
38696	ECNS	2020-10-05	43.645	43.7153	43.645	43.7153	664	{}	\N	2020-10-12 14:44:23.0122
38697	ECNS	2020-10-06	44.49	44.61	44.45	44.45	7213	{}	\N	2020-10-12 14:44:23.016338
38698	ECNS	2020-10-07	44.68	44.68	44.57	44.57	1303	{}	\N	2020-10-12 14:44:23.020533
38699	ECNS	2020-10-08	45.36	45.36	45.25	45.29	2603	{}	\N	2020-10-12 14:44:23.024517
38700	ECNS	2020-10-09	44.93	45.27	44.93	45.225	4364	{}	\N	2020-10-12 14:44:23.028436
38701	ECNS	2020-10-12	45.95	45.95	45.5001	45.5001	2039	{}	\N	2020-10-12 14:44:23.032234
38702	EDAP	2020-09-28	4.33	4.59	4.2	4.51	272684	{}	\N	2020-10-12 14:44:24.034966
38703	EDAP	2020-09-29	4.6	4.75	4.49	4.64	163544	{}	\N	2020-10-12 14:44:24.041361
38704	EDAP	2020-09-30	4.58	4.74	4.51	4.6	46885	{}	\N	2020-10-12 14:44:24.04593
38705	EDAP	2020-10-01	4.6	4.68	4.4	4.505	85921	{}	\N	2020-10-12 14:44:24.050484
38706	EDAP	2020-10-02	4.49	4.5599	4.4	4.5	24094	{}	\N	2020-10-12 14:44:24.056656
38707	EDAP	2020-10-05	4.57	4.73	4.55	4.67	66577	{}	\N	2020-10-12 14:44:24.061607
38708	EDAP	2020-10-06	4.7	4.7	4.52	4.565	50681	{}	\N	2020-10-12 14:44:24.066081
38709	EDAP	2020-10-07	4.42	4.55	4.35	4.45	80030	{}	\N	2020-10-12 14:44:24.070441
38710	EDAP	2020-10-08	4.5	4.5154	4.4	4.5	34693	{}	\N	2020-10-12 14:44:24.074803
38711	EDAP	2020-10-09	4.5	4.5	4.32	4.4186	89732	{}	\N	2020-10-12 14:44:24.079067
38712	EDAP	2020-10-12	4.4	4.55	4.4	4.45	14959	{}	\N	2020-10-12 14:44:24.083212
38713	EDF	2020-09-28	6.28	6.4301	6.21	6.43	200554	{}	\N	2020-10-12 14:44:25.920265
38714	EDF	2020-09-29	6.41	6.48	6.34	6.43	214730	{}	\N	2020-10-12 14:44:25.93131
38715	EDF	2020-09-30	6.42	6.53	6.38	6.48	542682	{}	\N	2020-10-12 14:44:25.937712
38716	EDF	2020-10-01	6.48	6.59	6.36	6.4	440076	{}	\N	2020-10-12 14:44:25.944615
38717	EDF	2020-10-02	6.35	6.43	6.35	6.4	130082	{}	\N	2020-10-12 14:44:25.949796
38718	EDF	2020-10-05	6.4	6.44	6.36	6.36	208480	{}	\N	2020-10-12 14:44:25.954532
38719	EDF	2020-10-06	6.39	6.44	6.35	6.4	248773	{}	\N	2020-10-12 14:44:25.958587
38720	EDF	2020-10-07	6.42	6.47	6.39	6.42	311212	{}	\N	2020-10-12 14:44:25.962424
38721	EDF	2020-10-08	6.49	6.55	6.34	6.43	321193	{}	\N	2020-10-12 14:44:25.966102
38722	EDF	2020-10-09	6.46	6.52	6.46	6.47	140224	{}	\N	2020-10-12 14:44:25.970097
38723	EDF	2020-10-12	6.5	6.55	6.5	6.5399	60912	{}	\N	2020-10-12 14:44:25.974005
38724	DXLG	2020-09-28	0.24	0.244	0.232	0.2347	376990	{}	\N	2020-10-12 14:44:27.771
38725	DXLG	2020-09-29	0.2337	0.285	0.232	0.2749	7719557	{}	\N	2020-10-12 14:44:27.777794
38726	DXLG	2020-09-30	0.2567	0.2744	0.25	0.2576	1253101	{}	\N	2020-10-12 14:44:27.783335
38727	DXLG	2020-10-01	0.273	0.2828	0.2502	0.274	1946183	{}	\N	2020-10-12 14:44:27.789078
38728	DXLG	2020-10-02	0.255	0.2984	0.255	0.271	3598109	{}	\N	2020-10-12 14:44:27.795018
38729	DXLG	2020-10-05	0.2888	0.2888	0.265	0.2671	848946	{}	\N	2020-10-12 14:44:27.799894
38730	DXLG	2020-10-06	0.2802	0.2802	0.252501	0.2575	1302164	{}	\N	2020-10-12 14:44:27.805415
38731	DXLG	2020-10-07	0.253	0.2699	0.253	0.2668	701184	{}	\N	2020-10-12 14:44:27.811303
38732	DXLG	2020-10-08	0.26	0.345	0.257	0.32	9495570	{}	\N	2020-10-12 14:44:27.816
38733	DXLG	2020-10-09	0.31	0.325	0.295	0.3082	1813045	{}	\N	2020-10-12 14:44:27.822404
38734	DXLG	2020-10-12	0.3025	0.3661	0.3025	0.345	4763670	{}	\N	2020-10-12 14:44:27.828461
38735	EAST	2020-09-28	1.14	1.18	1.12	1.16	82655	{}	\N	2020-10-12 14:44:29.255618
38736	EAST	2020-09-29	1.18	1.18	1.14	1.16	46394	{}	\N	2020-10-12 14:44:29.265191
38737	EAST	2020-09-30	1.18	1.18	1.1301	1.14	11707	{}	\N	2020-10-12 14:44:29.272178
38738	EAST	2020-10-01	1.131	1.18	1.131	1.16	16230	{}	\N	2020-10-12 14:44:29.277721
38739	EAST	2020-10-02	1.15	1.19	1.15	1.16	41176	{}	\N	2020-10-12 14:44:29.283615
38740	EAST	2020-10-05	1.1593	1.213	1.15	1.17	78319	{}	\N	2020-10-12 14:44:29.288801
38741	EAST	2020-10-06	1.1701	1.24	1.17	1.17	57776	{}	\N	2020-10-12 14:44:29.293316
38742	EAST	2020-10-07	1.17	1.23	1.17	1.22	9384	{}	\N	2020-10-12 14:44:29.297454
38743	EAST	2020-10-08	1.2	1.23	1.16	1.19	36222	{}	\N	2020-10-12 14:44:29.30172
38744	EAST	2020-10-09	1.21	1.22	1.16	1.19	20917	{}	\N	2020-10-12 14:44:29.306044
38745	EAST	2020-10-12	1.21	1.21	1.18	1.19	7236	{}	\N	2020-10-12 14:44:29.310249
38746	EBS	2020-09-28	102.28	103.8	101.18	103.64	407200	{}	\N	2020-10-12 14:44:30.632627
38747	EBS	2020-09-29	103.29	105	102.2	104.34	654399	{}	\N	2020-10-12 14:44:30.641214
38748	EBS	2020-09-30	104.11	105.37	102.6301	103.33	815599	{}	\N	2020-10-12 14:44:30.646977
38749	EBS	2020-10-01	103.94	106.01	103.2	105.89	609264	{}	\N	2020-10-12 14:44:30.652658
38750	EBS	2020-10-02	103.14	104.18	99.29	100.28	826462	{}	\N	2020-10-12 14:44:30.658344
38751	EBS	2020-10-05	101.88	106.08	101.15	105.94	784282	{}	\N	2020-10-12 14:44:30.662725
38752	EBS	2020-10-06	106.83	115.97	106.83	111.31	1900395	{}	\N	2020-10-12 14:44:30.666763
38753	EBS	2020-10-07	113.14	114.78	107.24	108.72	1531253	{}	\N	2020-10-12 14:44:30.670724
38754	EBS	2020-10-08	110.01	110.7695	107.23	110.41	604578	{}	\N	2020-10-12 14:44:30.675763
38755	EBS	2020-10-09	110.41	110.99	106.5	109.51	515555	{}	\N	2020-10-12 14:44:30.680287
38756	EBS	2020-10-12	110.6	110.88	107.49	108.81	238848	{}	\N	2020-10-12 14:44:30.686304
38757	ECF	2020-09-28	11.67	11.72	11.61	11.68	34377	{}	\N	2020-10-12 14:44:31.643015
38758	ECF	2020-09-29	11.67	11.69	11.53	11.59	29505	{}	\N	2020-10-12 14:44:31.649892
38759	ECF	2020-09-30	11.53	11.65	11.53	11.55	50534	{}	\N	2020-10-12 14:44:31.655141
38760	ECF	2020-10-01	11.58	11.75	11.58	11.75	32111	{}	\N	2020-10-12 14:44:31.660633
38761	ECF	2020-10-02	11.7	11.78	11.55	11.62	38503	{}	\N	2020-10-12 14:44:31.665661
38762	ECF	2020-10-05	11.62	11.81	11.62	11.78	37053	{}	\N	2020-10-12 14:44:31.669975
38763	ECF	2020-10-06	11.86	11.96	11.7602	11.79	34693	{}	\N	2020-10-12 14:44:31.67466
38764	ECF	2020-10-07	11.79	12.07	11.79	12.06	24777	{}	\N	2020-10-12 14:44:31.679159
38765	ECF	2020-10-08	12.18	12.27	11.9375	12.141	51486	{}	\N	2020-10-12 14:44:31.683266
38766	ECF	2020-10-09	12.15	12.24	12.15	12.17	23107	{}	\N	2020-10-12 14:44:31.687346
38767	ECF	2020-10-12	12.22	12.23	12.21	12.23	4956	{}	\N	2020-10-12 14:44:31.691907
38768	EDN	2020-09-28	3.05	3.175	3.05	3.1	5885	{}	\N	2020-10-12 14:44:33.008248
38769	EDN	2020-09-29	3.02	3.1205	3	3.09	38754	{}	\N	2020-10-12 14:44:33.018468
38770	EDN	2020-09-30	3.075	3.17	3.075	3.11	26854	{}	\N	2020-10-12 14:44:33.025233
38771	EDN	2020-10-01	3.13	3.17	3.09	3.17	15010	{}	\N	2020-10-12 14:44:33.031059
38772	EDN	2020-10-02	3.07	3.21	3.07	3.12	32839	{}	\N	2020-10-12 14:44:33.036511
38773	EDN	2020-10-05	3.12	3.165	3.03	3.06	27210	{}	\N	2020-10-12 14:44:33.041816
38774	EDN	2020-10-06	3.09	3.19	2.9	2.9	29716	{}	\N	2020-10-12 14:44:33.046335
38775	EDN	2020-10-07	2.984	2.984	2.86	2.8801	24026	{}	\N	2020-10-12 14:44:33.050504
38776	EDN	2020-10-08	2.9606	2.9606	2.8702	2.92	33227	{}	\N	2020-10-12 14:44:33.055066
38777	EDN	2020-10-09	2.9505	2.99	2.88	2.95	43228	{}	\N	2020-10-12 14:44:33.060031
38778	EDN	2020-10-12	2.91	2.925	2.9096	2.925	2754	{}	\N	2020-10-12 14:44:33.064305
38779	EEA	2020-09-28	9.4	9.516	9.1973	9.1973	7121	{}	\N	2020-10-12 14:44:34.197656
38780	EEA	2020-09-29	9.42	9.42	9.15	9.16	53962	{}	\N	2020-10-12 14:44:34.205172
38781	EEA	2020-09-30	9.29	9.459	9.29	9.459	12506	{}	\N	2020-10-12 14:44:34.21083
38782	EEA	2020-10-01	9.39	9.39	9.23	9.23	11056	{}	\N	2020-10-12 14:44:34.216239
38783	EEA	2020-10-02	9.17	9.17	9.17	9.17	1314	{}	\N	2020-10-12 14:44:34.221072
38784	EEA	2020-10-05	9.2299	9.335	9.2299	9.332	12200	{}	\N	2020-10-12 14:44:34.225479
38785	EEA	2020-10-06	9.32	9.38	9.3	9.31	22548	{}	\N	2020-10-12 14:44:34.230123
38786	EEA	2020-10-07	9.38	9.3801	9.15	9.33	76114	{}	\N	2020-10-12 14:44:34.234382
38787	EEA	2020-10-08	9.36	9.39	9.3084	9.34	36178	{}	\N	2020-10-12 14:44:34.238525
38788	EEA	2020-10-09	9.4	9.45	9.27	9.32	69974	{}	\N	2020-10-12 14:44:34.243234
38789	EEA	2020-10-12	9.38	9.38	9.38	9.38	402	{}	\N	2020-10-12 14:44:34.248193
38790	EEMO	2020-09-28	15.6319	15.6319	15.57	15.57	287	{}	\N	2020-10-12 14:44:35.66856
38791	EEMO	2020-09-29	15.64	15.6507	15.64	15.6507	200	{}	\N	2020-10-12 14:44:35.676778
38792	EEMO	2020-09-30	15.95	15.9699	15.95	15.9699	402	{}	\N	2020-10-12 14:44:35.682262
38793	EEMO	2020-10-01	16.01	16.09	16.01	16.035	3009	{}	\N	2020-10-12 14:44:35.687134
38794	EEMO	2020-10-02	15.93	15.98	15.86	15.86	870	{}	\N	2020-10-12 14:44:35.69148
38795	EEMO	2020-10-05	16.05	16.05	16.05	16.05	119	{}	\N	2020-10-12 14:44:35.695976
38796	EEMO	2020-10-06	16.24	16.24	16.138	16.138	368	{}	\N	2020-10-12 14:44:35.700155
38797	EEMO	2020-10-07	16.423	16.51	16.423	16.49	17026	{}	\N	2020-10-12 14:44:35.704266
38798	EEMO	2020-10-08	16.5426	16.5426	16.5426	16.5426	7	{}	\N	2020-10-12 14:44:35.708242
38799	EEMO	2020-10-09	16.62	16.7508	16.62	16.7508	403	{}	\N	2020-10-12 14:44:35.712468
38800	EEV	2020-09-28	26.7	27.02	26.61	26.8111	12801	{}	\N	2020-10-12 14:44:36.651699
38801	EEV	2020-09-29	26.8	26.95	26.72	26.79	6318	{}	\N	2020-10-12 14:44:36.657528
38802	EEV	2020-09-30	26.42	26.42	25.87	25.96	18460	{}	\N	2020-10-12 14:44:36.662601
38803	EEV	2020-10-01	25.65	25.83	25.45	25.45	15559	{}	\N	2020-10-12 14:44:36.668228
38804	EEV	2020-10-02	26.23	26.23	25.6532	26.05	21866	{}	\N	2020-10-12 14:44:36.673513
38805	EEV	2020-10-05	25.6499	25.67	25.3899	25.4	9736	{}	\N	2020-10-12 14:44:36.678182
38806	EEV	2020-10-06	25.18	25.35	24.89	25.3	18495	{}	\N	2020-10-12 14:44:36.683086
38807	EEV	2020-10-07	24.76	24.89	24.61	24.7	18835	{}	\N	2020-10-12 14:44:36.687804
38808	EEV	2020-10-08	24.49	24.49	24.25	24.32	5626	{}	\N	2020-10-12 14:44:36.69224
38809	EEV	2020-10-09	24.08	24.09	23.85	23.94	7237	{}	\N	2020-10-12 14:44:36.696997
38810	EEV	2020-10-12	23.7039	23.7039	23.65	23.65	468	{}	\N	2020-10-12 14:44:36.70175
38811	EFZ	2020-09-28	23.75	23.75	23.7	23.7391	148733	{}	\N	2020-10-12 14:44:38.018967
38812	EFZ	2020-09-29	23.68	23.765	23.641	23.715	16023	{}	\N	2020-10-12 14:44:38.027226
38813	EFZ	2020-09-30	23.8	23.84	23.7193	23.81	4060	{}	\N	2020-10-12 14:44:38.033485
38814	EFZ	2020-10-01	23.7223	23.77	23.71	23.755	21282	{}	\N	2020-10-12 14:44:38.039592
38815	EFZ	2020-10-02	23.83	23.83	23.7	23.72	19328	{}	\N	2020-10-12 14:44:38.045289
38816	EFZ	2020-10-05	23.485	23.498	23.38	23.38	28877	{}	\N	2020-10-12 14:44:38.050894
38817	EFZ	2020-10-06	23.427	23.6706	23.37	23.61	141464	{}	\N	2020-10-12 14:44:38.056101
38818	EFZ	2020-10-07	23.43	23.441	23.3988	23.405	4524	{}	\N	2020-10-12 14:44:38.06061
38819	EFZ	2020-10-08	23.3	23.32	23.2651	23.2651	6766	{}	\N	2020-10-12 14:44:38.065277
38820	EFZ	2020-10-09	23.15	23.15	23.07	23.08	19690	{}	\N	2020-10-12 14:44:38.071097
38821	EFZ	2020-10-12	23.0281	23.03	23.0281	23.03	11207	{}	\N	2020-10-12 14:44:38.075682
38822	DXCM	2020-09-28	399	401.5	387.01	400.19	1036540	{}	\N	2020-10-12 14:44:39.465285
38823	DXCM	2020-09-29	399.79	409.555	397.95	402.18	541896	{}	\N	2020-10-12 14:44:39.471538
38824	DXCM	2020-09-30	401.02	415.63	397.51	412.23	580895	{}	\N	2020-10-12 14:44:39.476355
38825	DXCM	2020-10-01	417.08	420	409.83	413.84	473249	{}	\N	2020-10-12 14:44:39.481295
38826	DXCM	2020-10-02	396.99	401.6	379.11	384.52	1290410	{}	\N	2020-10-12 14:44:39.487127
38827	DXCM	2020-10-05	388	389.755	378.735	382.51	824492	{}	\N	2020-10-12 14:44:39.493429
38828	DXCM	2020-10-06	379.67	389.35	373.41	374.89	861539	{}	\N	2020-10-12 14:44:39.499145
38829	DXCM	2020-10-07	376.75	394.43	376.75	393.82	1009771	{}	\N	2020-10-12 14:44:39.5034
38830	DXCM	2020-10-08	396.75	399.8	387.17	387.5	521872	{}	\N	2020-10-12 14:44:39.507963
38831	DXCM	2020-10-09	393.53	396.98	385.2	386	641938	{}	\N	2020-10-12 14:44:39.512056
38832	DXCM	2020-10-12	392.52	394.78	388.02	393.4	264422	{}	\N	2020-10-12 14:44:39.51648
38833	EARN	2020-09-28	11.45	11.79	11.4	11.59	77458	{}	\N	2020-10-12 14:44:41.702898
38834	EARN	2020-09-29	11.32	11.33	11.07	11.2	90354	{}	\N	2020-10-12 14:44:41.709995
38835	EARN	2020-09-30	11.21	11.28	11.05	11.1	65327	{}	\N	2020-10-12 14:44:41.714276
38836	EARN	2020-10-01	11.15	11.16	10.9	11.12	116175	{}	\N	2020-10-12 14:44:41.718505
38837	EARN	2020-10-02	10.98	11.25	10.94	11.21	52034	{}	\N	2020-10-12 14:44:41.722591
38838	EARN	2020-10-05	11.26	11.26	11.12	11.21	45616	{}	\N	2020-10-12 14:44:41.727157
38839	EARN	2020-10-06	11.24	11.312	10.97	11.01	156842	{}	\N	2020-10-12 14:44:41.73149
38840	EARN	2020-10-07	11.07	11.15	10.9	11.13	72748	{}	\N	2020-10-12 14:44:41.735483
38841	EARN	2020-10-08	11.23	11.45	11.08	11.41	95366	{}	\N	2020-10-12 14:44:41.739853
38842	EARN	2020-10-09	11.54	11.54	11.18	11.24	49348	{}	\N	2020-10-12 14:44:41.744301
38843	EARN	2020-10-12	11.26	11.26	11.18	11.235	2404	{}	\N	2020-10-12 14:44:41.749143
40122	EQWL	2020-09-28	59.12	59.9209	59.12	59.6513	3639	{}	\N	2020-10-12 14:47:48.716509
40123	EQWL	2020-09-29	59.026	59.2504	59.026	59.2504	1114	{}	\N	2020-10-12 14:47:48.725383
40124	EQWL	2020-09-30	59.63	59.9002	59.55	59.9002	1061	{}	\N	2020-10-12 14:47:48.731951
40125	EQWL	2020-10-01	59.89	59.9968	59.85	59.9002	1759	{}	\N	2020-10-12 14:47:48.736391
40126	EQWL	2020-10-02	59.735	59.911	59.735	59.9	1277	{}	\N	2020-10-12 14:47:48.741286
40127	EQWL	2020-10-05	60.556	60.556	60.556	60.556	428	{}	\N	2020-10-12 14:47:48.746391
40128	EQWL	2020-10-06	60.9454	61.15	60.024	60.024	1391	{}	\N	2020-10-12 14:47:48.751269
40129	EQWL	2020-10-07	60.82	61.2638	60.82	61.2638	533	{}	\N	2020-10-12 14:47:48.755734
40130	EQWL	2020-10-08	61.678	61.91	61.678	61.91	1106	{}	\N	2020-10-12 14:47:48.760054
40131	EQWL	2020-10-09	62.28	62.28	62.1	62.18	26432	{}	\N	2020-10-12 14:47:48.76454
40132	ERIE	2020-09-28	206.51	210.04	206.51	208.49	54207	{}	\N	2020-10-12 14:47:49.833324
40133	ERIE	2020-09-29	207.63	211.1	207.63	210.07	38615	{}	\N	2020-10-12 14:47:49.840749
40134	ERIE	2020-09-30	210.09	212.2	208.65	210.28	72169	{}	\N	2020-10-12 14:47:49.846351
40135	ERIE	2020-10-01	211.32	212.45	207.48	211.17	74261	{}	\N	2020-10-12 14:47:49.85151
40136	ERIE	2020-10-02	208.07	215.04	199.15	213.25	37888	{}	\N	2020-10-12 14:47:49.856254
40137	ERIE	2020-10-05	214.99	218.79	212.84	218.32	65832	{}	\N	2020-10-12 14:47:49.860626
40138	ERIE	2020-10-06	219.16	221.73	216.51	218.54	51508	{}	\N	2020-10-12 14:47:49.865123
40139	ERIE	2020-10-07	221.09	224.55	219.53	220.33	56489	{}	\N	2020-10-12 14:47:49.869928
40140	ERIE	2020-10-08	220.52	221.87	219.11	221.29	31214	{}	\N	2020-10-12 14:47:49.874501
40141	ERIE	2020-10-09	221.47	224.005	220.96	221.59	39089	{}	\N	2020-10-12 14:47:49.879291
40142	ERIE	2020-10-12	222.38	224	221.045	224	3388	{}	\N	2020-10-12 14:47:49.883609
40143	ESCA	2020-09-28	17.3	18.79	17.2601	18.62	56114	{}	\N	2020-10-12 14:47:50.960127
40144	ESCA	2020-09-29	18.58	18.7638	18.13	18.48	16438	{}	\N	2020-10-12 14:47:50.96862
40145	ESCA	2020-09-30	18.51	18.85	18.17	18.29	26011	{}	\N	2020-10-12 14:47:50.97498
40146	ESCA	2020-10-01	18.5	19.71	18.3	19.28	40013	{}	\N	2020-10-12 14:47:50.980471
40147	ESCA	2020-10-02	19.03	19.66	18.65	18.73	14344	{}	\N	2020-10-12 14:47:50.98561
40148	ESCA	2020-10-05	18.99	19.99	18.99	19.93	53502	{}	\N	2020-10-12 14:47:50.990167
40149	ESCA	2020-10-06	20.1	20.5	19.32	19.66	102919	{}	\N	2020-10-12 14:47:50.994712
40150	ESCA	2020-10-07	19.65	20.1	19.395	20.06	33477	{}	\N	2020-10-12 14:47:50.999528
38844	ECH	2020-09-28	23.9	24.145	23.7	23.86	96103	{}	\N	2020-10-12 14:44:43.066372
38845	ECH	2020-09-29	23.98	23.98	23.36	23.44	151787	{}	\N	2020-10-12 14:44:43.076955
38846	ECH	2020-09-30	23.5499	24.15	23.5465	24.02	579854	{}	\N	2020-10-12 14:44:43.081773
38847	ECH	2020-10-01	24.44	24.44	23.9	24.02	400588	{}	\N	2020-10-12 14:44:43.087024
38848	ECH	2020-10-02	23.87	24.26	23.76	24.05	276911	{}	\N	2020-10-12 14:44:43.092454
38849	ECH	2020-10-05	24.4	24.4	23.92	24.16	240300	{}	\N	2020-10-12 14:44:43.096988
38850	ECH	2020-10-06	23.96	24.08	23.51	23.65	364834	{}	\N	2020-10-12 14:44:43.101463
38851	ECH	2020-10-07	23.6901	23.76	23.475	23.482	198354	{}	\N	2020-10-12 14:44:43.107887
38852	ECH	2020-10-08	23.46	23.75	23.41	23.71	384160	{}	\N	2020-10-12 14:44:43.114527
38853	ECH	2020-10-09	24.1	24.27	23.83	24.04	410504	{}	\N	2020-10-12 14:44:43.121201
38854	ECH	2020-10-12	24.19	24.29	24.04	24.06	11338	{}	\N	2020-10-12 14:44:43.128841
38855	EES	2020-09-28	29.55	30.22	29.55	30.15	65018	{}	\N	2020-10-12 14:44:45.765811
38856	EES	2020-09-29	29.9136	29.9195	29.53	29.83	19786	{}	\N	2020-10-12 14:44:45.773126
38857	EES	2020-09-30	29.76	30.25	29.705	29.87	37214	{}	\N	2020-10-12 14:44:45.779036
38858	EES	2020-10-01	30.11	30.25	29.7921	30.25	24460	{}	\N	2020-10-12 14:44:45.783918
38859	EES	2020-10-02	29.55	30.9099	29.55	30.84	14983	{}	\N	2020-10-12 14:44:45.789006
38860	EES	2020-10-05	30.95	31.63	30.94	31.59	137526	{}	\N	2020-10-12 14:44:45.793807
38861	EES	2020-10-06	31.87	32.3775	31.431	31.431	108461	{}	\N	2020-10-12 14:44:45.798577
38862	EES	2020-10-07	31.95	32.2545	31.88	32.12	24979	{}	\N	2020-10-12 14:44:45.803128
38863	EES	2020-10-08	32.4	32.615	32.23	32.6	35181	{}	\N	2020-10-12 14:44:45.807634
38864	EES	2020-10-09	32.81	32.956	32.55	32.55	31842	{}	\N	2020-10-12 14:44:45.812447
38865	EES	2020-10-12	32.6175	32.7699	32.54	32.7699	2932	{}	\N	2020-10-12 14:44:45.817056
38866	EFC	2020-09-28	12.74	13.06	12.74	12.83	178279	{}	\N	2020-10-12 14:44:46.805736
38867	EFC	2020-09-29	12.62	12.73	12.28	12.42	389638	{}	\N	2020-10-12 14:44:46.81359
38868	EFC	2020-09-30	12.5	12.58	12.23	12.26	448388	{}	\N	2020-10-12 14:44:46.820014
38869	EFC	2020-10-01	12.26	12.4	12.08	12.39	496438	{}	\N	2020-10-12 14:44:46.825288
38870	EFC	2020-10-02	12.21	12.77	12.2	12.74	318385	{}	\N	2020-10-12 14:44:46.831063
38871	EFC	2020-10-05	12.88	12.88	12.4683	12.58	449808	{}	\N	2020-10-12 14:44:46.835991
38872	EFC	2020-10-06	12.67	12.94	12.46	12.53	447419	{}	\N	2020-10-12 14:44:46.840384
38873	EFC	2020-10-07	12.62	12.72	12.52	12.6	284411	{}	\N	2020-10-12 14:44:46.844945
38874	EFC	2020-10-08	12.71	13.12	12.61	13.11	642531	{}	\N	2020-10-12 14:44:46.849587
38875	EFC	2020-10-09	13.3	13.35	12.83	12.95	409385	{}	\N	2020-10-12 14:44:46.854142
38876	EFC	2020-10-12	12.9	13.09	12.84	13.06	82076	{}	\N	2020-10-12 14:44:46.858613
38877	EFO	2020-09-28	33.25	33.3599	33.24	33.3599	5556	{}	\N	2020-10-12 14:44:48.004191
38878	EFO	2020-09-29	33.4088	33.4088	33.3092	33.3092	1430	{}	\N	2020-10-12 14:44:48.012603
38879	EFO	2020-09-30	33.26	33.4	32.94	33.03	25407	{}	\N	2020-10-12 14:44:48.018757
38880	EFO	2020-10-01	33.26	33.318	33.0394	33.318	1788	{}	\N	2020-10-12 14:44:48.024381
38881	EFO	2020-10-02	33.07	33.39	33.03	33.25	4156	{}	\N	2020-10-12 14:44:48.029939
38882	EFO	2020-10-05	34.14	34.25	34.14	34.25	3124	{}	\N	2020-10-12 14:44:48.034829
38883	EFO	2020-10-06	34.15	34.29	33.44	33.56	9210	{}	\N	2020-10-12 14:44:48.039035
38884	EFO	2020-10-07	34.085	34.15	34.085	34.12	1233	{}	\N	2020-10-12 14:44:48.043126
38885	EFO	2020-10-08	34.51	34.535	34.51	34.535	594	{}	\N	2020-10-12 14:44:48.048251
38886	EFO	2020-10-09	35.19	35.19	35.0766	35.0766	566	{}	\N	2020-10-12 14:44:48.052454
38887	EFO	2020-10-12	35.17	35.17	35.17	35.17	424	{}	\N	2020-10-12 14:44:48.056619
38888	DXYN	2020-09-28	0.81	0.8476	0.81	0.835422	58802	{}	\N	2020-10-12 14:44:49.206045
38889	DXYN	2020-09-29	0.8354	0.85	0.77	0.8118	36066	{}	\N	2020-10-12 14:44:49.214275
38890	DXYN	2020-09-30	0.8118	0.9298	0.8118	0.899	81253	{}	\N	2020-10-12 14:44:49.220436
38891	DXYN	2020-10-01	0.9149	0.9149	0.78	0.852	149396	{}	\N	2020-10-12 14:44:49.225816
38892	DXYN	2020-10-02	0.9	0.915	0.88	0.915	40894	{}	\N	2020-10-12 14:44:49.231211
38893	DXYN	2020-10-05	0.91	0.9385	0.89	0.8901	66701	{}	\N	2020-10-12 14:44:49.23592
38894	DXYN	2020-10-06	0.895049	0.9359	0.875	0.875	56814	{}	\N	2020-10-12 14:44:49.240169
38895	DXYN	2020-10-07	0.888	0.888	0.8101	0.86	30033	{}	\N	2020-10-12 14:44:49.244292
38896	DXYN	2020-10-08	0.84	0.93	0.8	0.8984	64598	{}	\N	2020-10-12 14:44:49.248629
38897	DXYN	2020-10-09	0.92	0.93	0.86	0.86	30493	{}	\N	2020-10-12 14:44:49.25302
38898	DXYN	2020-10-12	0.86	0.89	0.86	0.89	923	{}	\N	2020-10-12 14:44:49.257294
38899	DZZ	2020-09-28	2.69	2.711	2.67	2.69	27193	{}	\N	2020-10-12 14:44:51.583751
38900	DZZ	2020-09-29	2.65	2.6674	2.62	2.65	32205	{}	\N	2020-10-12 14:44:51.591272
38901	DZZ	2020-09-30	2.63	2.68	2.62	2.655	9209	{}	\N	2020-10-12 14:44:51.596795
38902	DZZ	2020-10-01	2.619	2.64	2.58	2.6	6158	{}	\N	2020-10-12 14:44:51.602404
38903	DZZ	2020-10-02	2.6	2.615	2.6	2.615	1111	{}	\N	2020-10-12 14:44:51.607613
38904	DZZ	2020-10-05	2.62	2.62	2.5601	2.57	7962	{}	\N	2020-10-12 14:44:51.612088
38905	DZZ	2020-10-06	2.51	2.65	2.51	2.65	5746	{}	\N	2020-10-12 14:44:51.616688
38906	DZZ	2020-10-07	2.62	2.6852	2.62	2.68	6688	{}	\N	2020-10-12 14:44:51.621153
38907	DZZ	2020-10-08	2.62	2.66	2.62	2.66	10779	{}	\N	2020-10-12 14:44:51.62582
38908	DZZ	2020-10-09	2.55	2.59	2.52	2.55	18014	{}	\N	2020-10-12 14:44:51.630196
38909	EAE	2020-09-28	25.13	25.15	25.12	25.12	6963	{}	\N	2020-10-12 14:44:53.131062
38910	EAE	2020-09-29	25.15	25.15	25.12	25.12	7280	{}	\N	2020-10-12 14:44:53.138677
38911	EAE	2020-09-30	25.12	25.15	25.12	25.12	66556	{}	\N	2020-10-12 14:44:53.145271
38912	EAE	2020-10-01	25.15	25.15	25.136	25.1477	14756	{}	\N	2020-10-12 14:44:53.150725
38913	EAE	2020-10-02	25.14	25.15	25.12	25.15	31072	{}	\N	2020-10-12 14:44:53.155893
38914	EAE	2020-10-05	25.12	25.15	25.12	25.15	5092	{}	\N	2020-10-12 14:44:53.160917
38915	EAE	2020-10-06	25.15	25.15	25.13	25.15	14192	{}	\N	2020-10-12 14:44:53.16525
38916	EAE	2020-10-07	25.13	25.15	25.13	25.15	5966	{}	\N	2020-10-12 14:44:53.171365
38917	EAE	2020-10-08	25.15	25.15	25.14	25.15	5326	{}	\N	2020-10-12 14:44:53.175891
38918	EAE	2020-10-09	25.14	25.15	25.14	25.15	22852	{}	\N	2020-10-12 14:44:53.180325
38919	EAE	2020-10-12	25.14	25.14	25.14	25.14	400	{}	\N	2020-10-12 14:44:53.18469
38920	EBAYL	2020-09-28	25.73	25.8	25.73	25.8	58370	{}	\N	2020-10-12 14:44:54.075424
38921	EBAYL	2020-09-29	25.76	25.79	25.72	25.73	124308	{}	\N	2020-10-12 14:44:54.083351
38922	EBAYL	2020-09-30	25.75	25.78	25.61	25.61	161019	{}	\N	2020-10-12 14:44:54.09034
38923	EBAYL	2020-10-01	25.7	25.79	25.68	25.74	51473	{}	\N	2020-10-12 14:44:54.095925
38924	EBAYL	2020-10-02	25.65	25.78	25.65	25.74	76806	{}	\N	2020-10-12 14:44:54.10011
38925	EBAYL	2020-10-05	25.76	25.835	25.68	25.7	101991	{}	\N	2020-10-12 14:44:54.104436
38926	EBAYL	2020-10-06	25.74	25.78	25.72	25.76	78690	{}	\N	2020-10-12 14:44:54.109431
38927	EBAYL	2020-10-07	25.8	25.8051	25.73	25.78	84169	{}	\N	2020-10-12 14:44:54.113681
38928	EBAYL	2020-10-08	25.76	25.815	25.71	25.74	52264	{}	\N	2020-10-12 14:44:54.119945
38929	EBAYL	2020-10-09	25.76	25.8	25.72	25.8	54994	{}	\N	2020-10-12 14:44:54.124503
38930	EBAYL	2020-10-12	25.8	25.89	25.8	25.89	14878	{}	\N	2020-10-12 14:44:54.128708
38931	ECCB	2020-09-28	25.1655	25.3	24.95	24.95	4275	{}	\N	2020-10-12 14:44:55.130037
38932	ECCB	2020-09-29	25.05	25.2049	25.05	25.06	3242	{}	\N	2020-10-12 14:44:55.137175
38933	ECCB	2020-09-30	25.06	25.2	25.06	25.2	3838	{}	\N	2020-10-12 14:44:55.142969
38934	ECCB	2020-10-01	25.29	25.29	24.88	24.95	27978	{}	\N	2020-10-12 14:44:55.148506
38935	ECCB	2020-10-02	25.06	25.06	24.92	25	7000	{}	\N	2020-10-12 14:44:55.153352
38936	ECCB	2020-10-05	25.162	25.2	25.05	25.2	10806	{}	\N	2020-10-12 14:44:55.157863
38937	ECCB	2020-10-06	25.18	25.27	25	25.27	18276	{}	\N	2020-10-12 14:44:55.162341
38938	ECCB	2020-10-07	25.08	25.15	25.0381	25.095	34714	{}	\N	2020-10-12 14:44:55.166151
38939	ECCB	2020-10-08	25.12	25.12	24.98	25.01	14046	{}	\N	2020-10-12 14:44:55.170214
38940	ECCB	2020-10-09	24.97	25.2441	24.7821	24.89	10928	{}	\N	2020-10-12 14:44:55.174386
38941	ECCB	2020-10-12	24.9	24.94	24.9	24.94	966	{}	\N	2020-10-12 14:44:55.178875
38942	ECOL	2020-09-28	32.03	32.485	31.68	32.25	121004	{}	\N	2020-10-12 14:44:56.370243
38943	ECOL	2020-09-29	32.33	32.45	31.55	32.26	159186	{}	\N	2020-10-12 14:44:56.377333
38944	ECOL	2020-09-30	32.35	33.15	32.29	32.67	269253	{}	\N	2020-10-12 14:44:56.382613
38945	ECOL	2020-10-01	32.85	33.15	32.05	32.57	138464	{}	\N	2020-10-12 14:44:56.387473
38946	ECOL	2020-10-02	31.92	33.69	30.75	33.43	185171	{}	\N	2020-10-12 14:44:56.392248
38947	ECOL	2020-10-05	33.85	34.375	33.47	33.81	179316	{}	\N	2020-10-12 14:44:56.396371
38948	ECOL	2020-10-06	34.24	34.56	33.3301	33.55	219693	{}	\N	2020-10-12 14:44:56.400206
38949	ECOL	2020-10-07	32.845	34.185	32.845	34.02	181749	{}	\N	2020-10-12 14:44:56.404085
38950	ECOL	2020-10-08	34.47	34.88	34.0205	34.48	127908	{}	\N	2020-10-12 14:44:56.407834
38951	ECOL	2020-10-09	34.88	34.91	33.74	33.99	170187	{}	\N	2020-10-12 14:44:56.412203
38952	ECOL	2020-10-12	34	34.59	33.7	34.49	19566	{}	\N	2020-10-12 14:44:56.416072
38953	EDIT	2020-09-28	30.3	30.51	28.72	29.14	829171	{}	\N	2020-10-12 14:44:58.784989
38954	EDIT	2020-09-29	29.04	29.73	28.81	29.04	638705	{}	\N	2020-10-12 14:44:58.793986
38955	EDIT	2020-09-30	29.23	29.37	27.69	28.06	1044758	{}	\N	2020-10-12 14:44:58.79982
38956	EDIT	2020-10-01	28.29	28.69	27.57	28.17	810287	{}	\N	2020-10-12 14:44:58.80495
38957	EDIT	2020-10-02	27.2	28.05	27.01	27.07	686753	{}	\N	2020-10-12 14:44:58.809463
38958	EDIT	2020-10-05	27.5	29.5	27.45	29.14	725600	{}	\N	2020-10-12 14:44:58.814473
38959	EDIT	2020-10-06	29.18	29.76	28.55	28.76	616146	{}	\N	2020-10-12 14:44:58.818512
38960	EDIT	2020-10-07	29.27	32.28	29.2246	31.04	1692820	{}	\N	2020-10-12 14:44:58.822509
38961	EDIT	2020-10-08	32.28	32.34	30.42	30.71	982973	{}	\N	2020-10-12 14:44:58.826407
38962	EDIT	2020-10-09	30.97	33.3	30.635	33.09	1065716	{}	\N	2020-10-12 14:44:58.831154
38963	EDIT	2020-10-12	33.6	33.96	31.81	31.81	317215	{}	\N	2020-10-12 14:44:58.835151
38964	EEMX	2020-09-28	65.15	67.76	64.98	65	3480	{}	\N	2020-10-12 14:45:01.055247
38965	EEMX	2020-09-29	64.81	65.18	64.81	65.08	4474	{}	\N	2020-10-12 14:45:01.061703
38966	EEMX	2020-09-30	65.29	66.17	65.28	66.0899	1085	{}	\N	2020-10-12 14:45:01.067164
38967	EEMX	2020-10-01	66.52	66.5524	66.3	66.5524	1110	{}	\N	2020-10-12 14:45:01.072712
38968	EEMX	2020-10-02	65.72	66.03	65.72	65.9767	4017	{}	\N	2020-10-12 14:45:01.079647
38969	EEMX	2020-10-05	66.26	66.72	66.25	66.72	2601	{}	\N	2020-10-12 14:45:01.084857
38970	EEMX	2020-10-06	67.22	67.27	66.6273	66.6273	1610	{}	\N	2020-10-12 14:45:01.091178
38971	EEMX	2020-10-07	67.36	67.52	67.22	67.52	3076	{}	\N	2020-10-12 14:45:01.097
38972	EEMX	2020-10-08	67.48	67.945	67.45	67.945	3282	{}	\N	2020-10-12 14:45:01.102729
38973	EEMX	2020-10-09	68.3	68.5955	68.3	68.55	1815	{}	\N	2020-10-12 14:45:01.109058
38974	EFAX	2020-09-28	68.34	68.67	68.34	68.66	11927	{}	\N	2020-10-12 14:45:02.214199
38975	EFAX	2020-09-29	68.59	68.84	68.49	68.71	12743	{}	\N	2020-10-12 14:45:02.220502
38976	EFAX	2020-09-30	68.31	68.441	68.27	68.285	1427	{}	\N	2020-10-12 14:45:02.225861
38977	EFAX	2020-10-01	68.705	68.705	68.54	68.54	864	{}	\N	2020-10-12 14:45:02.230118
38978	EFAX	2020-10-02	68.64	68.65	68.48	68.48	3804	{}	\N	2020-10-12 14:45:02.234146
38979	EFAX	2020-10-05	69.2102	69.61	69.18	69.6	1301	{}	\N	2020-10-12 14:45:02.238624
38980	EFAX	2020-10-06	69.42	69.431	68.87	68.87	697	{}	\N	2020-10-12 14:45:02.243891
38981	EFAX	2020-10-07	69.41	69.41	69.365	69.365	798	{}	\N	2020-10-12 14:45:02.247837
38982	EFAX	2020-10-08	69.7301	69.7746	69.73	69.7746	870	{}	\N	2020-10-12 14:45:02.252271
38983	EFAX	2020-10-09	70.205	70.35	70.205	70.22	1037	{}	\N	2020-10-12 14:45:02.257488
38984	EFR	2020-09-28	11.7	11.7172	11.67	11.69	66338	{}	\N	2020-10-12 14:45:03.416523
38985	EFR	2020-09-29	11.69	11.6955	11.65	11.69	171484	{}	\N	2020-10-12 14:45:03.422437
38986	EFR	2020-09-30	11.74	11.79	11.63	11.63	272613	{}	\N	2020-10-12 14:45:03.428
38987	EFR	2020-10-01	11.7	11.78	11.7	11.74	139627	{}	\N	2020-10-12 14:45:03.433174
38988	EFR	2020-10-02	11.64	11.82	11.64	11.77	192856	{}	\N	2020-10-12 14:45:03.437742
38989	EFR	2020-10-05	11.8	11.9	11.77	11.9	190587	{}	\N	2020-10-12 14:45:03.441992
38990	EFR	2020-10-06	11.86	11.92	11.8542	11.91	184653	{}	\N	2020-10-12 14:45:03.446569
38991	EFR	2020-10-07	11.94	11.98	11.91	11.92	532294	{}	\N	2020-10-12 14:45:03.450531
38992	EFR	2020-10-08	11.92	12.2299	11.92	12.2	740424	{}	\N	2020-10-12 14:45:03.454514
38993	EFR	2020-10-09	12.2	12.28	12.2	12.21	277832	{}	\N	2020-10-12 14:45:03.459035
38994	EFR	2020-10-12	12.26	12.2899	12.22	12.245	33294	{}	\N	2020-10-12 14:45:03.463979
38995	EA	2020-09-28	133.36	133.85	130.87	132.28	1669415	{}	\N	2020-10-12 14:45:04.413996
38996	EA	2020-09-29	132.06	132.43	129.5	130.02	1707843	{}	\N	2020-10-12 14:45:04.420179
38997	EA	2020-09-30	129.73	131.99	129.45	130.41	1983616	{}	\N	2020-10-12 14:45:04.425416
38998	EA	2020-10-01	132.07	134.055	131.46	132.78	1744812	{}	\N	2020-10-12 14:45:04.429791
38999	EA	2020-10-02	131.37	132.24	127.71	129.07	1621197	{}	\N	2020-10-12 14:45:04.433906
39000	EA	2020-10-05	130.09	132.95	129.07	130.72	1247828	{}	\N	2020-10-12 14:45:04.43769
39001	EA	2020-10-06	130.36	130.4	124.52	124.84	3257752	{}	\N	2020-10-12 14:45:04.441441
39002	EA	2020-10-07	126.41	126.5	123.97	125.01	3183944	{}	\N	2020-10-12 14:45:04.445011
39003	EA	2020-10-08	126.1	126.38	124.42	125.99	1932695	{}	\N	2020-10-12 14:45:04.448956
39004	EA	2020-10-09	126.86	128.44	125.915	127.45	1921215	{}	\N	2020-10-12 14:45:04.452875
39005	EA	2020-10-12	128.98	129.4	127.83	128.78	377049	{}	\N	2020-10-12 14:45:04.456803
39006	EBTC	2020-09-28	20.62	21.5	20.42	20.66	10273	{}	\N	2020-10-12 14:45:05.443795
39007	EBTC	2020-09-29	21.06	21.38	20.63	21.2	9411	{}	\N	2020-10-12 14:45:05.453148
39008	EBTC	2020-09-30	21.005	21.365	20.9193	21.02	4238	{}	\N	2020-10-12 14:45:05.459056
39009	EBTC	2020-10-01	21.56	21.56	20.83	20.83	3339	{}	\N	2020-10-12 14:45:05.464111
39010	EBTC	2020-10-02	20.41	21.39	20.41	21.06	4669	{}	\N	2020-10-12 14:45:05.46861
39011	EBTC	2020-10-05	21.12	22.25	20.7253	21.96	5797	{}	\N	2020-10-12 14:45:05.473118
39012	EBTC	2020-10-06	22.34	23.1499	21.507	22.25	12344	{}	\N	2020-10-12 14:45:05.477219
39013	EBTC	2020-10-07	21.3791	23.7	21.3791	23.7	8157	{}	\N	2020-10-12 14:45:05.480835
39014	EBTC	2020-10-08	23.5	23.86	22.99	23.11	11944	{}	\N	2020-10-12 14:45:05.484451
39015	EBTC	2020-10-09	22.9435	23.18	22.7	23.15	4736	{}	\N	2020-10-12 14:45:05.48816
39016	EBTC	2020-10-12	22.6736	22.6736	22.6736	22.6736	682	{}	\N	2020-10-12 14:45:05.493193
39017	ECOM	2020-09-28	14.53	15.18	14.53	14.99	360908	{}	\N	2020-10-12 14:45:06.862849
39018	ECOM	2020-09-29	14.99	15.09	14.56	14.78	535918	{}	\N	2020-10-12 14:45:06.869959
39019	ECOM	2020-09-30	14.83	14.99	14.38	14.47	490105	{}	\N	2020-10-12 14:45:06.875403
39020	ECOM	2020-10-01	14.67	15.34	14.62	15.24	638942	{}	\N	2020-10-12 14:45:06.879903
39021	ECOM	2020-10-02	14.69	15.13	14.65	14.72	589060	{}	\N	2020-10-12 14:45:06.884192
39022	ECOM	2020-10-05	14.89	15.32	14.63	15.23	531889	{}	\N	2020-10-12 14:45:06.888499
39023	ECOM	2020-10-06	15.31	15.7	15.06	15.22	437291	{}	\N	2020-10-12 14:45:06.893048
39024	ECOM	2020-10-07	15.55	16.53	15.31	16.5	1029062	{}	\N	2020-10-12 14:45:06.896811
39025	ECOM	2020-10-08	16.79	16.8399	16.05	16.32	764893	{}	\N	2020-10-12 14:45:06.900421
39026	ECOM	2020-10-09	16.5	16.8	16.36	16.64	523963	{}	\N	2020-10-12 14:45:06.904151
39027	ECOM	2020-10-12	16.81	16.98	16.525	16.665	82386	{}	\N	2020-10-12 14:45:06.908106
39028	EDI	2020-09-28	7.14	7.2099	7.12	7.18	68408	{}	\N	2020-10-12 14:45:08.649197
39029	EDI	2020-09-29	7.18	7.22	7.13	7.15	109564	{}	\N	2020-10-12 14:45:08.660472
39030	EDI	2020-09-30	7.15	7.19	7.05	7.05	286297	{}	\N	2020-10-12 14:45:08.667381
39031	EDI	2020-10-01	7.1	7.19	7.1	7.15	102854	{}	\N	2020-10-12 14:45:08.673029
39032	EDI	2020-10-02	7.06	7.18	7.06	7.11	43784	{}	\N	2020-10-12 14:45:08.678324
39033	EDI	2020-10-05	7.15	7.19	7.11	7.17	65506	{}	\N	2020-10-12 14:45:08.683713
39034	EDI	2020-10-06	7.11	7.27	7.11	7.19	47333	{}	\N	2020-10-12 14:45:08.688051
39035	EDI	2020-10-07	7.22	7.32	7.2007	7.26	85264	{}	\N	2020-10-12 14:45:08.692285
39036	EDI	2020-10-08	7.27	7.37	7.27	7.33	53692	{}	\N	2020-10-12 14:45:08.696075
39037	EDI	2020-10-09	7.35	7.44	7.35	7.43	87497	{}	\N	2020-10-12 14:45:08.700069
39038	EDI	2020-10-12	7.54	7.589	7.4535	7.51	15950	{}	\N	2020-10-12 14:45:08.704198
39039	EEMV	2020-09-28	54.7	54.82	54.58	54.77	291354	{}	\N	2020-10-12 14:45:10.151282
39040	EEMV	2020-09-29	54.53	54.78	54.53	54.69	275228	{}	\N	2020-10-12 14:45:10.160155
39041	EEMV	2020-09-30	54.8824	55.3068	54.8824	55.17	358530	{}	\N	2020-10-12 14:45:10.16585
39042	EEMV	2020-10-01	55.43	55.63	55.32	55.61	267629	{}	\N	2020-10-12 14:45:10.172518
39043	EEMV	2020-10-02	55.02	55.43	55.02	55.15	250193	{}	\N	2020-10-12 14:45:10.178789
39044	EEMV	2020-10-05	55.58	55.72	55.5	55.705	382026	{}	\N	2020-10-12 14:45:10.184228
39045	EEMV	2020-10-06	55.825	55.95	55.55	55.63	373339	{}	\N	2020-10-12 14:45:10.188639
39046	EEMV	2020-10-07	56.0281	56.19	55.9105	56.11	454651	{}	\N	2020-10-12 14:45:10.192615
39047	EEMV	2020-10-08	56.18	56.45	56.18	56.43	492031	{}	\N	2020-10-12 14:45:10.196826
39048	EEMV	2020-10-09	56.38	56.77	56.38	56.64	269899	{}	\N	2020-10-12 14:45:10.200909
39049	EEMV	2020-10-12	56.69	56.862	56.67	56.862	114328	{}	\N	2020-10-12 14:45:10.205122
39050	EFSC	2020-09-28	26.87	27.67	26.84	27.48	81997	{}	\N	2020-10-12 14:45:12.441593
39051	EFSC	2020-09-29	27.39	27.47	26.86	27.42	93474	{}	\N	2020-10-12 14:45:12.44726
39052	EFSC	2020-09-30	27.66	27.875	27.06	27.27	71786	{}	\N	2020-10-12 14:45:12.451916
39053	EFSC	2020-10-01	27.32	27.67	27	27.64	66134	{}	\N	2020-10-12 14:45:12.456465
39054	EFSC	2020-10-02	26.99	28.52	26.99	28.17	76799	{}	\N	2020-10-12 14:45:12.461594
39055	EFSC	2020-10-05	28.4	28.91	28.1386	28.84	118158	{}	\N	2020-10-12 14:45:12.466115
39056	EFSC	2020-10-06	29.3	30.57	28.9	29.18	112533	{}	\N	2020-10-12 14:45:12.470593
39057	EFSC	2020-10-07	29.73	30.3	29.44	29.9	123172	{}	\N	2020-10-12 14:45:12.47688
39058	EFSC	2020-10-08	30.28	31.405	29.6348	30.32	86146	{}	\N	2020-10-12 14:45:12.481391
39059	EFSC	2020-10-09	30.4	30.73	30.04	30.11	62172	{}	\N	2020-10-12 14:45:12.485587
39060	EFSC	2020-10-12	30.1	30.66	30.1	30.61	16307	{}	\N	2020-10-12 14:45:12.48971
39061	EIX	2020-09-28	52.01	52.64	51.4699	51.89	2701653	{}	\N	2020-10-12 14:45:14.254901
39062	EIX	2020-09-29	51.42	51.7	50.93	51.02	2784437	{}	\N	2020-10-12 14:45:14.262907
39063	EIX	2020-09-30	51.44	51.63	50.515	50.84	4618072	{}	\N	2020-10-12 14:45:14.268847
39064	EIX	2020-10-01	50.8	51.14	50.26	50.44	4421517	{}	\N	2020-10-12 14:45:14.274381
39065	EIX	2020-10-02	50.14	51.55	50	51.5	5408834	{}	\N	2020-10-12 14:45:14.279362
39066	EIX	2020-10-05	51.76	52.86	51.59	52.61	5785722	{}	\N	2020-10-12 14:45:14.283709
39067	EIX	2020-10-06	52.7	55.12	52.305	53.86	5031115	{}	\N	2020-10-12 14:45:14.288817
39068	EIX	2020-10-07	54.11	55.17	54.07	54.94	3654884	{}	\N	2020-10-12 14:45:14.292992
39069	EIX	2020-10-08	54.98	56.13	54.95	56.1	5103160	{}	\N	2020-10-12 14:45:14.297266
39070	EIX	2020-10-09	56.6	56.73	55.78	56.09	3261665	{}	\N	2020-10-12 14:45:14.301541
39071	EIX	2020-10-12	56.16	56.26	55.655	56.105	792954	{}	\N	2020-10-12 14:45:14.30651
39072	ELLO	2020-09-29	28.69	28.69	28.21	28.21	531	{}	\N	2020-10-12 14:45:15.28442
39073	ELLO	2020-09-30	29	29.01	27	29	431512	{}	\N	2020-10-12 14:45:15.294385
39074	ELLO	2020-10-02	30.68	33.7399	29.0199	29.0199	2489	{}	\N	2020-10-12 14:45:15.302245
39075	ELLO	2020-10-06	31.5089	31.5089	30.47	31.505	853	{}	\N	2020-10-12 14:45:15.30783
39076	ELLO	2020-10-08	30.25	30.25	30.25	30.25	761	{}	\N	2020-10-12 14:45:15.312228
39077	ELLO	2020-10-09	31.555	31.555	31.555	31.555	265	{}	\N	2020-10-12 14:45:15.316468
39078	ELLO	2020-10-12	32.24	32.24	32.24	32.24	451	{}	\N	2020-10-12 14:45:15.321201
39079	FITBI	2020-09-28	27.16	27.2	26.92	26.94	121578	{}	\N	2020-10-12 14:45:16.774799
39080	FITBI	2020-09-29	27.03	27.07	26.9166	26.96	37354	{}	\N	2020-10-12 14:45:16.785936
39081	FITBI	2020-09-30	27.14	27.14	26.35	26.71	398442	{}	\N	2020-10-12 14:45:16.793671
39082	FITBI	2020-10-01	26.84	27.08	26.77	27.08	57572	{}	\N	2020-10-12 14:45:16.799885
39083	FITBI	2020-10-02	26.95	27.27	26.8884	27.21	40040	{}	\N	2020-10-12 14:45:16.805608
39084	FITBI	2020-10-05	27.24	27.3	27.1	27.22	79239	{}	\N	2020-10-12 14:45:16.810726
39085	FITBI	2020-10-06	27.24	27.422	26.97	27.23	75698	{}	\N	2020-10-12 14:45:16.815367
39086	FITBI	2020-10-07	27.35	27.54	27.35	27.38	60243	{}	\N	2020-10-12 14:45:16.819732
39087	FITBI	2020-10-08	27.5	27.72	27.44	27.57	37315	{}	\N	2020-10-12 14:45:16.824827
39088	FITBI	2020-10-09	27.72	27.73	27.4293	27.51	31437	{}	\N	2020-10-12 14:45:16.829177
39089	FITBI	2020-10-12	27.67	27.67	27.36	27.37	10857	{}	\N	2020-10-12 14:45:16.833368
39090	ECCY	2020-09-28	24.97	24.97	24.91	24.91	893	{}	\N	2020-10-12 14:45:17.976656
39091	ECCY	2020-09-29	24.95	25	24.63	24.63	4042	{}	\N	2020-10-12 14:45:17.983875
39092	ECCY	2020-09-30	24.8127	24.8127	24.68	24.68	2506	{}	\N	2020-10-12 14:45:17.989412
39093	ECCY	2020-10-01	24.7582	24.899	24.5801	24.5801	4560	{}	\N	2020-10-12 14:45:17.994936
39094	ECCY	2020-10-02	24.976	24.976	24.5801	24.62	1906	{}	\N	2020-10-12 14:45:18.000509
39095	ECCY	2020-10-05	24.73	24.76	24.7	24.7325	1960	{}	\N	2020-10-12 14:45:18.005491
39096	ECCY	2020-10-06	24.7201	24.77	24.7201	24.75	4010	{}	\N	2020-10-12 14:45:18.010726
39097	ECCY	2020-10-07	24.75	24.75	24.75	24.75	582	{}	\N	2020-10-12 14:45:18.015991
39098	ECCY	2020-10-08	24.78	24.85	24.75	24.75	2624	{}	\N	2020-10-12 14:45:18.02351
39099	ECCY	2020-10-09	24.85	24.9	24.7	24.7	4752	{}	\N	2020-10-12 14:45:18.031454
39100	ECON	2020-09-28	24.7	24.74	24.595	24.69	10221	{}	\N	2020-10-12 14:45:19.479918
39101	ECON	2020-09-29	24.62	24.64	24.55	24.57	9577	{}	\N	2020-10-12 14:45:19.486198
39102	ECON	2020-09-30	24.9	25.1356	24.9	25.06	16776	{}	\N	2020-10-12 14:45:19.491241
39103	ECON	2020-10-01	25.33	25.34	25.17	25.33	26556	{}	\N	2020-10-12 14:45:19.496972
39104	ECON	2020-10-02	25.025	25.2122	24.96	24.96	18053	{}	\N	2020-10-12 14:45:19.504084
39105	ECON	2020-10-05	25.1565	25.22	25.1	25.18	4902	{}	\N	2020-10-12 14:45:19.510928
39106	ECON	2020-10-06	25.46	25.515	25.24	25.24	16282	{}	\N	2020-10-12 14:45:19.518287
39107	ECON	2020-10-07	25.51	25.57	25.44	25.46	14418	{}	\N	2020-10-12 14:45:19.525637
39108	ECON	2020-10-08	25.58	25.67	25.52	25.61	6834	{}	\N	2020-10-12 14:45:19.531959
39109	ECON	2020-10-09	25.6604	25.8832	25.6604	25.79	10496	{}	\N	2020-10-12 14:45:19.539271
39110	ECON	2020-10-12	25.998	26	25.97	26	5738	{}	\N	2020-10-12 14:45:19.546805
39111	EDD	2020-09-28	5.64	5.68	5.6278	5.67	161880	{}	\N	2020-10-12 14:45:20.583712
39112	EDD	2020-09-29	5.58	5.58	5.51	5.51	478032	{}	\N	2020-10-12 14:45:20.5962
39113	EDD	2020-09-30	5.52	5.57	5.52	5.56	240362	{}	\N	2020-10-12 14:45:20.603798
39114	EDD	2020-10-01	5.56	5.6	5.56	5.59	278420	{}	\N	2020-10-12 14:45:20.609708
39115	EDD	2020-10-02	5.58	5.59	5.55	5.57	136260	{}	\N	2020-10-12 14:45:20.614745
39116	EDD	2020-10-05	5.55	5.59	5.55	5.59	397222	{}	\N	2020-10-12 14:45:20.620108
39117	EDD	2020-10-06	5.6	5.66	5.6	5.63	272006	{}	\N	2020-10-12 14:45:20.624517
39118	EDD	2020-10-07	5.64	5.68	5.5995	5.68	519846	{}	\N	2020-10-12 14:45:20.629
39119	EDD	2020-10-08	5.68	5.68	5.62	5.64	451296	{}	\N	2020-10-12 14:45:20.633469
39120	EDD	2020-10-09	5.63	5.67	5.63	5.63	230232	{}	\N	2020-10-12 14:45:20.637854
39121	EDD	2020-10-12	5.67	5.68	5.6501	5.67	64208	{}	\N	2020-10-12 14:45:20.642062
39122	EDUC	2020-09-28	14.35	15.7	14.2998	15.06	79638	{}	\N	2020-10-12 14:45:21.605799
39123	EDUC	2020-09-29	14.93	15.69	14.77	15.69	41153	{}	\N	2020-10-12 14:45:21.612695
39124	EDUC	2020-09-30	15.79	17.1899	15.51	16.77	224882	{}	\N	2020-10-12 14:45:21.61856
39125	EDUC	2020-10-01	17.13	17.72	16.63	17.72	90035	{}	\N	2020-10-12 14:45:21.62373
39126	EDUC	2020-10-02	17.31	18.19	17	17.18	53460	{}	\N	2020-10-12 14:45:21.628524
39127	EDUC	2020-10-05	17.31	18.2	17.18	17.92	60577	{}	\N	2020-10-12 14:45:21.633319
39128	EDUC	2020-10-06	18	18	17.155	17.62	38095	{}	\N	2020-10-12 14:45:21.63883
39129	EDUC	2020-10-07	18.25	20	18.01	18.81	283523	{}	\N	2020-10-12 14:45:21.643813
39130	EDUC	2020-10-08	18.94	19.265	18.25	18.7	49594	{}	\N	2020-10-12 14:45:21.648563
39131	EDUC	2020-10-09	18.89	19.5	18.25	18.98	53779	{}	\N	2020-10-12 14:45:21.655253
39132	EDUC	2020-10-12	19.5	19.5	18.7	18.77	20034	{}	\N	2020-10-12 14:45:21.659965
39133	EFAD	2020-09-28	40	40	39.83	39.9	17531	{}	\N	2020-10-12 14:45:22.888701
39134	EFAD	2020-09-29	39.92	39.99	39.78	39.915	4607	{}	\N	2020-10-12 14:45:22.898998
39135	EFAD	2020-09-30	39.76	39.7945	39.54	39.56	22913	{}	\N	2020-10-12 14:45:22.905691
39136	EFAD	2020-10-01	39.81	39.8867	39.73	39.86	18015	{}	\N	2020-10-12 14:45:22.911693
39137	EFAD	2020-10-02	39.78	39.96	39.76	39.915	4259	{}	\N	2020-10-12 14:45:22.917031
39138	EFAD	2020-10-05	40.14	40.46	40.14	40.45	6849	{}	\N	2020-10-12 14:45:22.921912
39139	EFAD	2020-10-06	40.27	40.4	39.8	39.8	5909	{}	\N	2020-10-12 14:45:22.925976
39140	EFAD	2020-10-07	40.005	40.1099	39.975	40.05	10338	{}	\N	2020-10-12 14:45:22.929927
39141	EFAD	2020-10-08	40.12	40.1945	40.12	40.1945	1344	{}	\N	2020-10-12 14:45:22.933959
39142	EFAD	2020-10-09	40.42	40.5	40.3402	40.45	5659	{}	\N	2020-10-12 14:45:22.938071
39143	EFAD	2020-10-12	40.62	40.64	40.62	40.63	825	{}	\N	2020-10-12 14:45:22.942294
39144	EFV	2020-09-28	40.51	40.71	40.475	40.71	1018084	{}	\N	2020-10-12 14:45:24.595351
39145	EFV	2020-09-29	40.59	40.67	40.32	40.47	1666209	{}	\N	2020-10-12 14:45:24.605075
39146	EFV	2020-09-30	40.39	40.67	40.2	40.35	2273272	{}	\N	2020-10-12 14:45:24.611485
39147	EFV	2020-10-01	40.39	40.5171	40.18	40.4	1046244	{}	\N	2020-10-12 14:45:24.617132
39148	EFV	2020-10-02	39.94	40.5486	39.9	40.46	1095592	{}	\N	2020-10-12 14:45:24.621733
39149	EFV	2020-10-05	40.92	41.185	40.845	41.18	915487	{}	\N	2020-10-12 14:45:24.626236
39150	EFV	2020-10-06	41.41	41.47	40.84	40.935	598697	{}	\N	2020-10-12 14:45:24.630271
39151	EFV	2020-10-07	41.27	41.44	41.16	41.32	2063858	{}	\N	2020-10-12 14:45:24.634298
39152	EFV	2020-10-08	41.49	41.69	41.42	41.65	2574146	{}	\N	2020-10-12 14:45:24.637825
39153	EFV	2020-10-09	41.74	41.84	41.64	41.77	1802341	{}	\N	2020-10-12 14:45:24.64144
39154	EFV	2020-10-12	41.765	41.78	41.73	41.76	239858	{}	\N	2020-10-12 14:45:24.644951
39155	EGPT	2020-09-28	24.72	24.8647	24.72	24.8647	140	{}	\N	2020-10-12 14:45:26.541147
39156	EGPT	2020-09-29	24.55	25.01	24.55	24.69	1898	{}	\N	2020-10-12 14:45:26.55171
39157	EGPT	2020-09-30	24.95	25.19	24.93	25.19	1888	{}	\N	2020-10-12 14:45:26.56021
39158	EGPT	2020-10-01	25.3246	25.3246	24.97	25.25	2497	{}	\N	2020-10-12 14:45:26.570083
39159	EGPT	2020-10-02	24.77	25.0796	24.77	25.0796	859	{}	\N	2020-10-12 14:45:26.579218
39160	EGPT	2020-10-05	25.07	25.1502	25	25.1502	1779	{}	\N	2020-10-12 14:45:26.586367
39161	EGPT	2020-10-06	25.18	25.67	24.9	25.42	3238	{}	\N	2020-10-12 14:45:26.595292
39162	EGPT	2020-10-07	25.38	26.0999	25.11	26.0999	4259	{}	\N	2020-10-12 14:45:26.602045
39163	EGPT	2020-10-08	25.67	26.078	24.591	26.05	3996	{}	\N	2020-10-12 14:45:26.611067
39164	EGPT	2020-10-09	25.63	26.15	25.23	25.8761	3738	{}	\N	2020-10-12 14:45:26.616729
39165	EGPT	2020-10-12	26	26	26	26	208	{}	\N	2020-10-12 14:45:26.624408
39166	EHTH	2020-09-28	75.49	76.13	73.9	74.66	455095	{}	\N	2020-10-12 14:45:27.939147
39167	EHTH	2020-09-29	73.87	76.4868	73.87	76.14	461594	{}	\N	2020-10-12 14:45:27.945353
39168	EHTH	2020-09-30	76.14	80.389	76.07	79.07	731022	{}	\N	2020-10-12 14:45:27.953096
39169	EHTH	2020-10-01	80.97	85.76	80.15	85.33	1387051	{}	\N	2020-10-12 14:45:27.963082
39170	EHTH	2020-10-02	82.5	89.53	82.38	88.76	945576	{}	\N	2020-10-12 14:45:27.970379
39171	EHTH	2020-10-05	89.1915	93.805	89.18	93.3	668645	{}	\N	2020-10-12 14:45:27.979768
39172	EHTH	2020-10-06	93.6	94.41	84.78	85.55	2009860	{}	\N	2020-10-12 14:45:27.988058
39173	EHTH	2020-10-07	86.71	87	84.29	85.89	843502	{}	\N	2020-10-12 14:45:27.996226
39174	EHTH	2020-10-08	86.63	87.41	85.265	85.69	347048	{}	\N	2020-10-12 14:45:28.003778
39175	EHTH	2020-10-09	86	86.735	85.5	86.04	310930	{}	\N	2020-10-12 14:45:28.010579
39176	EHTH	2020-10-12	87.17	87.23	85.33	86.0852	67582	{}	\N	2020-10-12 14:45:28.018303
39177	EL	2020-09-28	217.11	217.86	214.97	215.75	903394	{}	\N	2020-10-12 14:45:29.411502
39178	EL	2020-09-29	217.87	217.8999	212.73	214.78	1922825	{}	\N	2020-10-12 14:45:29.417929
39179	EL	2020-09-30	215.81	219.97	215.38	218.25	1937638	{}	\N	2020-10-12 14:45:29.423614
39180	EL	2020-10-01	219.69	220.57	217.33	219.53	1741134	{}	\N	2020-10-12 14:45:29.429861
39181	EL	2020-10-02	217.15	220.76	215.79	218.76	1387449	{}	\N	2020-10-12 14:45:29.435017
39182	EL	2020-10-05	221.66	223.055	220.14	222.05	1660860	{}	\N	2020-10-12 14:45:29.439246
39183	EL	2020-10-06	222	223.02	216.84	217.66	1741398	{}	\N	2020-10-12 14:45:29.445195
39184	EL	2020-10-07	219.42	222.14	219.065	221.48	1513330	{}	\N	2020-10-12 14:45:29.449994
39185	EL	2020-10-08	222.24	224.81	221.3	224.21	1525804	{}	\N	2020-10-12 14:45:29.455016
39186	EL	2020-10-09	224.58	226.27	222.51	224.6	1455972	{}	\N	2020-10-12 14:45:29.461734
39187	EL	2020-10-12	226.43	226.88	224.6	225.65	221271	{}	\N	2020-10-12 14:45:29.468052
39188	ELSE	2020-09-28	3.41	3.505	3.4	3.5	9185	{}	\N	2020-10-12 14:45:30.340882
39189	ELSE	2020-09-29	3.5	3.53	3.5	3.53	1712	{}	\N	2020-10-12 14:45:30.35144
39190	ELSE	2020-09-30	3.4687	3.5	3.4	3.4618	3252	{}	\N	2020-10-12 14:45:30.357229
39191	ELSE	2020-10-01	3.445	3.55	3.445	3.54	2284	{}	\N	2020-10-12 14:45:30.363811
39192	ELSE	2020-10-02	3.48	3.48	3.48	3.48	257	{}	\N	2020-10-12 14:45:30.368491
39193	ELSE	2020-10-05	3.485	3.485	3.45	3.45	674	{}	\N	2020-10-12 14:45:30.373086
39194	ELSE	2020-10-06	3.5	8.8	3.45	8.4	18682051	{}	\N	2020-10-12 14:45:30.379301
39195	ELSE	2020-10-07	5.96	6.17	4.92	5.18	3201611	{}	\N	2020-10-12 14:45:30.384366
39196	ELSE	2020-10-08	5.01	5.37	4.6305	4.7	508249	{}	\N	2020-10-12 14:45:30.390804
39197	ELSE	2020-10-09	4.63	4.92	4.4	4.64	404219	{}	\N	2020-10-12 14:45:30.397517
39198	ELSE	2020-10-12	4.42	4.5	4.4	4.4767	67914	{}	\N	2020-10-12 14:45:30.404328
39199	EEMA	2020-09-28	74.01	74.01	73.415	73.79	29046	{}	\N	2020-10-12 14:45:31.854199
39200	EEMA	2020-09-29	73.37	73.91	73.37	73.79	32914	{}	\N	2020-10-12 14:45:31.861708
39201	EEMA	2020-09-30	74.61	75.0693	74.3	74.68	41882	{}	\N	2020-10-12 14:45:31.867662
39202	EEMA	2020-10-01	75.69	75.88	75.38	75.8371	29975	{}	\N	2020-10-12 14:45:31.872999
39203	EEMA	2020-10-02	74.39	75.33	74.39	74.855	11044	{}	\N	2020-10-12 14:45:31.879487
39204	EEMA	2020-10-05	75.44	75.66	75.18	75.57	39359	{}	\N	2020-10-12 14:45:31.885436
39205	EEMA	2020-10-06	75.75	76.445	75.69	75.69	36099	{}	\N	2020-10-12 14:45:31.891029
39206	EEMA	2020-10-07	76.49	76.965	76.49	76.95	16580	{}	\N	2020-10-12 14:45:31.897314
39207	EEMA	2020-10-08	77.21	77.619	76.92	77.6	46661	{}	\N	2020-10-12 14:45:31.903234
39208	EEMA	2020-10-09	77.28	78.16	77.28	77.84	85292	{}	\N	2020-10-12 14:45:31.909388
39209	EEMA	2020-10-12	78.01	78.23	78.01	78.22	1051	{}	\N	2020-10-12 14:45:31.916087
39210	EFAS	2020-09-28	11.98	12.0019	11.98	12	792	{}	\N	2020-10-12 14:45:33.035624
39211	EFAS	2020-09-29	11.925	11.9483	11.915	11.915	2211	{}	\N	2020-10-12 14:45:33.045371
39212	EFAS	2020-09-30	11.96	11.9849	11.94	11.94	1048	{}	\N	2020-10-12 14:45:33.054527
39213	EFAS	2020-10-01	11.9301	11.98	11.924	11.924	8603	{}	\N	2020-10-12 14:45:33.062368
39214	EFAS	2020-10-02	11.9207	12.05	11.9207	12.05	415	{}	\N	2020-10-12 14:45:33.072071
39215	EFAS	2020-10-05	12.24	12.261	12.22	12.25	6323	{}	\N	2020-10-12 14:45:33.079706
39216	EFAS	2020-10-06	12.4071	12.4071	12.2269	12.2269	4301	{}	\N	2020-10-12 14:45:33.087915
39217	EFAS	2020-10-08	12.371	12.371	12.371	12.371	675	{}	\N	2020-10-12 14:45:33.096057
39218	EFAS	2020-10-09	12.49	12.5498	12.49	12.5498	1830	{}	\N	2020-10-12 14:45:33.102784
39219	EFOI	2020-09-28	7.65	7.65	6.84	7.14	81780	{}	\N	2020-10-12 14:45:34.078552
39220	EFOI	2020-09-29	7.26	7.36	6.935	7.04	32564	{}	\N	2020-10-12 14:45:34.086254
39221	EFOI	2020-09-30	7.22	7.22	6.85	6.87	23810	{}	\N	2020-10-12 14:45:34.093399
39222	EFOI	2020-10-01	6.9402	6.9527	6.515	6.7	38638	{}	\N	2020-10-12 14:45:34.100815
39223	EFOI	2020-10-02	6.57	7.1199	6.57	6.7	41475	{}	\N	2020-10-12 14:45:34.109723
39224	EFOI	2020-10-05	6.68	7.15	6.68	6.97	32450	{}	\N	2020-10-12 14:45:34.123799
39225	EFOI	2020-10-06	7.09	7.3599	6.98	7.23	49260	{}	\N	2020-10-12 14:45:34.142443
39226	EFOI	2020-10-07	7.3	7.8	7.2977	7.69	111062	{}	\N	2020-10-12 14:45:34.152287
39227	EFOI	2020-10-08	7.76	7.88	7.62	7.63	28501	{}	\N	2020-10-12 14:45:34.159319
39228	EFOI	2020-10-09	7.82	8.88	7.66	8.16	136552	{}	\N	2020-10-12 14:45:34.167085
39229	EFOI	2020-10-12	8.25	8.3	8.12	8.13	7771	{}	\N	2020-10-12 14:45:34.17377
39230	EGAN	2020-09-28	13.29	13.42	13.08	13.25	114123	{}	\N	2020-10-12 14:45:35.520119
39231	EGAN	2020-09-29	13.25	14.04	13.2	13.86	303340	{}	\N	2020-10-12 14:45:35.525064
39232	EGAN	2020-09-30	13.85	14.26	13.65	14.17	358830	{}	\N	2020-10-12 14:45:35.53047
39233	EGAN	2020-10-01	14.32	14.44	13.96	14.4	150215	{}	\N	2020-10-12 14:45:35.538607
39234	EGAN	2020-10-02	13.99	14.44	13.82	13.89	82900	{}	\N	2020-10-12 14:45:35.561003
39235	EGAN	2020-10-05	14.32	15.07	14.2	14.94	254272	{}	\N	2020-10-12 14:45:35.573052
39236	EGAN	2020-10-06	15.16	15.8	14.89	15.52	269435	{}	\N	2020-10-12 14:45:35.580542
39237	EGAN	2020-10-07	15.82	16.25	15.72	15.99	276750	{}	\N	2020-10-12 14:45:35.588044
39238	EGAN	2020-10-08	16.24	16.255	15.65	16.21	177244	{}	\N	2020-10-12 14:45:35.594861
39239	EGAN	2020-10-09	16.27	16.98	16.24	16.97	261637	{}	\N	2020-10-12 14:45:35.601613
39240	EGAN	2020-10-12	17.4	18.015	17.3	18.015	201876	{}	\N	2020-10-12 14:45:35.606805
39241	EHI	2020-09-28	9.6	9.635	9.6	9.6	49908	{}	\N	2020-10-12 14:45:37.20827
39242	EHI	2020-09-29	9.6	9.625	9.6	9.61	117198	{}	\N	2020-10-12 14:45:37.213803
39243	EHI	2020-09-30	9.6	9.69	9.595	9.64	546425	{}	\N	2020-10-12 14:45:37.218483
39244	EHI	2020-10-01	9.68	9.7099	9.6	9.61	297692	{}	\N	2020-10-12 14:45:37.223413
39245	EHI	2020-10-02	9.59	9.69	9.57	9.65	69088	{}	\N	2020-10-12 14:45:37.229497
39246	EHI	2020-10-05	9.66	9.71	9.65	9.7	150550	{}	\N	2020-10-12 14:45:37.235173
39247	EHI	2020-10-06	9.7	9.77	9.669	9.7	47212	{}	\N	2020-10-12 14:45:37.24041
39248	EHI	2020-10-07	9.72	9.75	9.7	9.71	109050	{}	\N	2020-10-12 14:45:37.245724
39249	EHI	2020-10-08	9.71	9.7898	9.71	9.73	97568	{}	\N	2020-10-12 14:45:37.251515
39250	EHI	2020-10-09	9.72	9.76	9.72	9.73	111994	{}	\N	2020-10-12 14:45:37.257284
39251	EHI	2020-10-12	9.73	9.78	9.73	9.7716	11266	{}	\N	2020-10-12 14:45:37.264652
39252	EIM	2020-09-28	13.01	13.06	12.9961	13.05	63532	{}	\N	2020-10-12 14:45:39.125144
39253	EIM	2020-09-29	13.22	13.22	13.05	13.1	65511	{}	\N	2020-10-12 14:45:39.132523
39254	EIM	2020-09-30	13.1	13.17	13.09	13.17	76125	{}	\N	2020-10-12 14:45:39.138453
39255	EIM	2020-10-01	13.11	13.17	13.045	13.09	199696	{}	\N	2020-10-12 14:45:39.144381
39256	EIM	2020-10-02	13.09	13.09	13.06	13.06	50511	{}	\N	2020-10-12 14:45:39.149946
39257	EIM	2020-10-05	13.11	13.12	13.05	13.05	66889	{}	\N	2020-10-12 14:45:39.156054
39258	EIM	2020-10-06	13.02	13.1	13.02	13.06	63190	{}	\N	2020-10-12 14:45:39.161019
39259	EIM	2020-10-07	13.06	13.1022	13.03	13.055	127962	{}	\N	2020-10-12 14:45:39.165702
39260	EIM	2020-10-08	13.08	13.1	13.03	13.07	97699	{}	\N	2020-10-12 14:45:39.170958
39261	EIM	2020-10-09	13.09	13.1	13.06	13.07	58251	{}	\N	2020-10-12 14:45:39.175809
39262	EIM	2020-10-12	13.05	13.09	13.05	13.08	14319	{}	\N	2020-10-12 14:45:39.180433
39263	ELU	2020-09-28	25.5805	25.64	25.465	25.465	7448	{}	\N	2020-10-12 14:45:41.36978
39264	ELU	2020-09-29	25.64	25.64	25.56	25.6	2996	{}	\N	2020-10-12 14:45:41.376159
39265	ELU	2020-09-30	25.55	25.55	25.22	25.22	47800	{}	\N	2020-10-12 14:45:41.380519
39266	ELU	2020-10-01	25.33	25.38	25.24	25.36	9358	{}	\N	2020-10-12 14:45:41.384965
39267	ELU	2020-10-02	25.3555	25.3555	25.2789	25.31	9314	{}	\N	2020-10-12 14:45:41.389352
39268	ELU	2020-10-05	25.38	25.42	25.38	25.42	6396	{}	\N	2020-10-12 14:45:41.393903
39269	ELU	2020-10-06	25.5	25.56	25.4953	25.56	10512	{}	\N	2020-10-12 14:45:41.398275
39270	ELU	2020-10-07	25.56	25.61	25.45	25.5	5246	{}	\N	2020-10-12 14:45:41.402427
39271	ELU	2020-10-08	25.51	25.51	25.46	25.46	3046	{}	\N	2020-10-12 14:45:41.406917
39272	ELU	2020-10-09	25.45	25.45	25.41	25.41	2962	{}	\N	2020-10-12 14:45:41.411266
39273	EMHY	2020-09-28	43.55	43.6852	43.4354	43.46	29700	{}	\N	2020-10-12 14:45:44.108112
39274	EMHY	2020-09-29	43.4619	43.509	43.25	43.33	68041	{}	\N	2020-10-12 14:45:44.115815
39275	EMHY	2020-09-30	43.33	43.74	43.3	43.64	26348	{}	\N	2020-10-12 14:45:44.121842
39276	EMHY	2020-10-01	43.7	43.71	43.5001	43.57	62743	{}	\N	2020-10-12 14:45:44.126755
39277	EMHY	2020-10-02	43.33	43.5	43.17	43.38	17909	{}	\N	2020-10-12 14:45:44.131166
39278	EMHY	2020-10-05	43.34	43.675	43.34	43.641	22173	{}	\N	2020-10-12 14:45:44.136988
39279	EMHY	2020-10-06	43.7	43.79	43.6	43.63	17130	{}	\N	2020-10-12 14:45:44.141648
39280	EMHY	2020-10-07	43.77	43.82	43.53	43.8	131735	{}	\N	2020-10-12 14:45:44.146478
39281	EMHY	2020-10-08	43.8	44.16	43.8	44.16	142013	{}	\N	2020-10-12 14:45:44.152185
39282	EMHY	2020-10-09	44.12	44.27	44.025	44.2653	28015	{}	\N	2020-10-12 14:45:44.157329
39283	EMHY	2020-10-12	44.34	44.34	44.15	44.2425	3829	{}	\N	2020-10-12 14:45:44.161752
39284	EMKR	2020-09-28	3.3	3.345	3.29	3.3	36948	{}	\N	2020-10-12 14:45:45.184251
39285	EMKR	2020-09-29	3.31	3.4	3.27	3.32	18815	{}	\N	2020-10-12 14:45:45.190332
39286	EMKR	2020-09-30	3.31	3.36	3.25	3.25	16829	{}	\N	2020-10-12 14:45:45.195696
39287	EMKR	2020-10-01	3.2399	3.25	3.21	3.22	12461	{}	\N	2020-10-12 14:45:45.201443
39288	EMKR	2020-10-02	3.1759	3.26	3.1759	3.25	13010	{}	\N	2020-10-12 14:45:45.207028
39289	EMKR	2020-10-05	3.3476	3.3476	3.25	3.29	10815	{}	\N	2020-10-12 14:45:45.213029
39290	EMKR	2020-10-06	3.37	3.4	3.27	3.27	25079	{}	\N	2020-10-12 14:45:45.219369
39291	EMKR	2020-10-07	3.29	3.29	3.1	3.2	66215	{}	\N	2020-10-12 14:45:45.225214
39292	EMKR	2020-10-08	3.21	3.3673	3.2	3.28	38677	{}	\N	2020-10-12 14:45:45.230753
39293	EMKR	2020-10-09	3.29	3.36	3.21	3.33	15964	{}	\N	2020-10-12 14:45:45.235795
39294	EMKR	2020-10-12	3.29	3.32	3.244	3.2817	4075	{}	\N	2020-10-12 14:45:45.240322
39295	FIVN	2020-09-28	125.57	127.285	124.56	126.82	463024	{}	\N	2020-10-12 14:45:46.545874
39296	FIVN	2020-09-29	126.82	129.61	126.205	128.74	416089	{}	\N	2020-10-12 14:45:46.552368
39297	FIVN	2020-09-30	128.05	131.94	127.15	129.54	674527	{}	\N	2020-10-12 14:45:46.55895
39298	FIVN	2020-10-01	131.32	133.16	128.71	131.95	655206	{}	\N	2020-10-12 14:45:46.566639
39299	FIVN	2020-10-02	128.88	134.96	128.009	132.06	651618	{}	\N	2020-10-12 14:45:46.576066
39300	FIVN	2020-10-05	132.24	134.705	131.59	133.42	470946	{}	\N	2020-10-12 14:45:46.58063
39301	FIVN	2020-10-06	133	138	133	135.87	730214	{}	\N	2020-10-12 14:45:46.584934
39302	FIVN	2020-10-07	137.7	141.65	136.5847	139.57	766953	{}	\N	2020-10-12 14:45:46.589764
39303	FIVN	2020-10-08	140.66	141.76	135.31	136.27	660354	{}	\N	2020-10-12 14:45:46.595434
39304	FIVN	2020-10-09	136.43	141.3	136.43	140.08	386032	{}	\N	2020-10-12 14:45:46.600048
39305	FIVN	2020-10-12	142.71	144.73	141.11	142.36	46002	{}	\N	2020-10-12 14:45:46.605986
39306	ED	2020-09-28	75.98	77.155	75.51	76.32	1673645	{}	\N	2020-10-12 14:45:47.687293
39307	ED	2020-09-29	76.73	77.7	76.2508	76.98	3177822	{}	\N	2020-10-12 14:45:47.692775
39308	ED	2020-09-30	77.45	78.06	77.16	77.8	3409186	{}	\N	2020-10-12 14:45:47.698561
39309	ED	2020-10-01	78.05	79.08	77.74	79.04	3050712	{}	\N	2020-10-12 14:45:47.704062
39310	ED	2020-10-02	78.53	80.21	78.26	79.77	3316748	{}	\N	2020-10-12 14:45:47.755485
39311	ED	2020-10-05	79.6	80.334	79.105	79.93	2739129	{}	\N	2020-10-12 14:45:47.765424
39312	ED	2020-10-06	80.06	81.22	79.46	80.76	2964762	{}	\N	2020-10-12 14:45:47.773491
39313	ED	2020-10-07	81.09	81.23	79.8401	80.83	2609344	{}	\N	2020-10-12 14:45:47.782362
39314	ED	2020-10-08	81.25	82.35	80.89	82.27	2409940	{}	\N	2020-10-12 14:45:47.790065
39315	ED	2020-10-09	82.39	82.92	81.5176	82.55	2326088	{}	\N	2020-10-12 14:45:47.795924
39316	ED	2020-10-12	82.45	83.0777	82.4	82.865	242294	{}	\N	2020-10-12 14:45:47.801363
39317	EFA	2020-09-28	63.89	64.05	63.775	64.03	18123969	{}	\N	2020-10-12 14:45:50.246385
39318	EFA	2020-09-29	64.03	64.23	63.72	63.96	14389283	{}	\N	2020-10-12 14:45:50.257312
39319	EFA	2020-09-30	63.72	64.13	63.425	63.65	28847581	{}	\N	2020-10-12 14:45:50.262406
39320	EFA	2020-10-01	64	64.07	63.64	64.02	33556876	{}	\N	2020-10-12 14:45:50.267358
39321	EFA	2020-10-02	63.24	64.045	63.2	63.89	18428417	{}	\N	2020-10-12 14:45:50.272208
39322	EFA	2020-10-05	64.41	64.88	64.39	64.87	12650417	{}	\N	2020-10-12 14:45:50.276177
39323	EFA	2020-10-06	64.91	64.95	64.04	64.17	16960942	{}	\N	2020-10-12 14:45:50.280393
39324	EFA	2020-10-07	64.62	64.805	64.49	64.72	14960767	{}	\N	2020-10-12 14:45:50.284865
39325	EFA	2020-10-08	65.01	65.19	64.94	65.17	13190680	{}	\N	2020-10-12 14:45:50.288967
39326	EFA	2020-10-09	65.51	65.725	65.42	65.68	21237709	{}	\N	2020-10-12 14:45:50.293315
39327	EFA	2020-10-12	65.81	65.88	65.765	65.84	1471259	{}	\N	2020-10-12 14:45:50.297513
39328	EFL	2020-09-28	8.8	8.83	8.76	8.76	43676	{}	\N	2020-10-12 14:45:51.718078
39329	EFL	2020-09-29	8.74	8.82	8.72	8.72	60326	{}	\N	2020-10-12 14:45:51.725211
39330	EFL	2020-09-30	8.74	8.87	8.74	8.84	69656	{}	\N	2020-10-12 14:45:51.730322
39331	EFL	2020-10-01	8.77	8.84	8.73	8.79	113672	{}	\N	2020-10-12 14:45:51.73481
39332	EFL	2020-10-02	8.72	8.8197	8.72	8.76	38154	{}	\N	2020-10-12 14:45:51.738761
39333	EFL	2020-10-05	8.7301	8.77	8.62	8.66	128806	{}	\N	2020-10-12 14:45:51.742523
39334	EFL	2020-10-06	8.69	8.76	8.6	8.62	133820	{}	\N	2020-10-12 14:45:51.746523
39335	EFL	2020-10-07	8.68	8.69	8.65	8.65	65430	{}	\N	2020-10-12 14:45:51.751032
39336	EFL	2020-10-08	8.68	8.84	8.68	8.83	76788	{}	\N	2020-10-12 14:45:51.754961
39337	EFL	2020-10-09	8.74	8.79	8.62	8.66	163588	{}	\N	2020-10-12 14:45:51.758737
39338	EFL	2020-10-12	8.71	8.74	8.71	8.74	14732	{}	\N	2020-10-12 14:45:51.762886
39339	EGP	2020-09-28	130.81	132.8	130.34	132.41	144880	{}	\N	2020-10-12 14:45:53.458448
39340	EGP	2020-09-29	131.21	132.17	129.16	129.77	435687	{}	\N	2020-10-12 14:45:53.466659
39341	EGP	2020-09-30	130.68	131.87	128.08	129.33	428213	{}	\N	2020-10-12 14:45:53.472339
39342	EGP	2020-10-01	129.96	133.93	129.06	133.83	386705	{}	\N	2020-10-12 14:45:53.476887
39343	EGP	2020-10-02	131.76	137.26	131.01	136.77	455455	{}	\N	2020-10-12 14:45:53.48125
39344	EGP	2020-10-05	138.15	139.86	135.8454	139.73	319573	{}	\N	2020-10-12 14:45:53.485669
39345	EGP	2020-10-06	141.02	142.73	138.68	139.9	437469	{}	\N	2020-10-12 14:45:53.490017
39346	EGP	2020-10-07	140.43	141.48	138.98	141.04	374062	{}	\N	2020-10-12 14:45:53.493937
39347	EGP	2020-10-08	142.09	144.07	141.53	143.95	516679	{}	\N	2020-10-12 14:45:53.497863
39348	EGP	2020-10-09	144.95	145.56	143.125	145.45	237614	{}	\N	2020-10-12 14:45:53.501979
39349	EGP	2020-10-12	145.66	146.01	144.845	146.01	20480	{}	\N	2020-10-12 14:45:53.506126
39350	EIGI	2020-09-28	5.34	6.47	5.34	6.25	3129974	{}	\N	2020-10-12 14:45:55.238562
39351	EIGI	2020-09-29	6.39	6.39	5.855	5.89	1047350	{}	\N	2020-10-12 14:45:55.24666
39352	EIGI	2020-09-30	5.92	6.0292	5.72	5.74	679684	{}	\N	2020-10-12 14:45:55.252459
39353	EIGI	2020-10-01	5.76	5.92	5.56	5.91	632289	{}	\N	2020-10-12 14:45:55.257459
39354	EIGI	2020-10-02	5.76	5.92	5.675	5.72	326155	{}	\N	2020-10-12 14:45:55.262648
39355	EIGI	2020-10-05	5.77	5.98	5.77	5.9	628786	{}	\N	2020-10-12 14:45:55.267494
39356	EIGI	2020-10-06	5.94	6.19	5.81	5.87	495465	{}	\N	2020-10-12 14:45:55.272631
39357	EIGI	2020-10-07	5.95	6.17	5.95	6.115	966938	{}	\N	2020-10-12 14:45:55.27709
39358	EIGI	2020-10-08	6.13	6.18	5.75	5.77	592644	{}	\N	2020-10-12 14:45:55.281529
39359	EIGI	2020-10-09	5.92	6.16	5.81	6.15	371235	{}	\N	2020-10-12 14:45:55.28626
39360	EIGI	2020-10-12	6.24	6.2998	6.08	6.13	61589	{}	\N	2020-10-12 14:45:55.291127
39361	ELD	2020-09-28	31.53	31.83	31.38	31.45	62255	{}	\N	2020-10-12 14:45:56.499327
39362	ELD	2020-09-29	31.62	31.6644	31.41	31.6	5008	{}	\N	2020-10-12 14:45:56.51011
39363	ELD	2020-09-30	31.51	31.78	31.51	31.74	3597	{}	\N	2020-10-12 14:45:56.515673
39364	ELD	2020-10-01	31.8	32.06	31.7701	32.06	11581	{}	\N	2020-10-12 14:45:56.52128
39365	ELD	2020-10-02	31.68	32.05	31.68	31.6901	44173	{}	\N	2020-10-12 14:45:56.526123
39366	ELD	2020-10-05	32	32.1399	31.8401	31.95	23448	{}	\N	2020-10-12 14:45:56.530938
39367	ELD	2020-10-06	32.18	32.2	31.985	31.985	16469	{}	\N	2020-10-12 14:45:56.535744
39368	ELD	2020-10-07	32.13	32.1399	31.77	32.07	9015	{}	\N	2020-10-12 14:45:56.540631
39369	ELD	2020-10-08	32.0614	32.209	32.061	32.209	11191	{}	\N	2020-10-12 14:45:56.545311
39370	ELD	2020-10-09	32.04	32.6553	32.04	32.2713	15875	{}	\N	2020-10-12 14:45:56.55023
39371	ELD	2020-10-12	32.57	32.7	32.32	32.32	2972	{}	\N	2020-10-12 14:45:56.554477
39372	ELS	2020-09-28	63.16	63.84	62.67	63.45	503205	{}	\N	2020-10-12 14:45:57.641439
39373	ELS	2020-09-29	63.19	63.38	61.59	61.65	1569137	{}	\N	2020-10-12 14:45:57.648595
39374	ELS	2020-09-30	62.04	62.08	60.74	61.3	4896338	{}	\N	2020-10-12 14:45:57.654132
39375	ELS	2020-10-01	61.7	64	61.3	63.72	4186713	{}	\N	2020-10-12 14:45:57.659873
39376	ELS	2020-10-02	63.04	64.63	62.665	64.17	2101487	{}	\N	2020-10-12 14:45:57.664981
39377	ELS	2020-10-05	64.63	64.91	63.43	64.55	2299910	{}	\N	2020-10-12 14:45:57.670405
39378	ELS	2020-10-06	64.97	65.03	63.72	64.16	1788736	{}	\N	2020-10-12 14:45:57.675259
39379	ELS	2020-10-07	64.41	64.6	63.33	64.15	1436397	{}	\N	2020-10-12 14:45:57.680058
39380	ELS	2020-10-08	64.54	65.64	64.15	65	1462306	{}	\N	2020-10-12 14:45:57.685096
39381	ELS	2020-10-09	65.4	65.61	64.465	64.85	1601288	{}	\N	2020-10-12 14:45:57.689884
39382	ELS	2020-10-12	65.04	65.2	64.46	65.04	294425	{}	\N	2020-10-12 14:45:57.694586
39383	EMCF	2020-09-28	27.62	27.62	27.62	27.62	879	{}	\N	2020-10-12 14:45:59.482722
39384	EMCF	2020-09-29	25.8407	27	25.8407	26.96	2079	{}	\N	2020-10-12 14:45:59.491007
39385	EMCF	2020-09-30	26.755	26.755	25.01	25.11	1643	{}	\N	2020-10-12 14:45:59.49744
39386	EMCF	2020-10-01	25.01	25.2	24.3	24.7	3238	{}	\N	2020-10-12 14:45:59.502747
39387	EMCF	2020-10-02	25.01	26.25	25	26.25	726	{}	\N	2020-10-12 14:45:59.507843
39388	EMCF	2020-10-05	25	25	24.06	24.06	942	{}	\N	2020-10-12 14:45:59.512228
39389	EMCF	2020-10-06	24.6048	24.6048	24.06	24.06	587	{}	\N	2020-10-12 14:45:59.516396
39390	EMCF	2020-10-07	24.7675	24.7675	24	24.315	537	{}	\N	2020-10-12 14:45:59.520992
39391	EMCF	2020-10-08	23.4356	24.13	23.0601	23.595	2046	{}	\N	2020-10-12 14:45:59.525371
39392	EMCF	2020-10-09	23.5	23.5001	23.5	23.5001	380	{}	\N	2020-10-12 14:45:59.52978
39393	EME	2020-09-28	67.49	68.64	67.245	68.27	253116	{}	\N	2020-10-12 14:46:02.598155
39394	EME	2020-09-29	68.05	68.22	66.5225	67.49	412827	{}	\N	2020-10-12 14:46:02.606195
39395	EME	2020-09-30	67.77	68.96	67.61	67.71	594584	{}	\N	2020-10-12 14:46:02.612068
39396	EME	2020-10-01	67.87	68.33	67	68.11	474732	{}	\N	2020-10-12 14:46:02.616935
39397	EME	2020-10-02	67.05	69.643	67.05	69.14	299711	{}	\N	2020-10-12 14:46:02.621618
39398	EME	2020-10-05	70.16	71.26	69.895	70.79	784860	{}	\N	2020-10-12 14:46:02.627293
39399	EME	2020-10-06	71.65	73.2825	70.53	70.86	528911	{}	\N	2020-10-12 14:46:02.632461
39400	EME	2020-10-07	71.62	72.77	70.78	70.99	596304	{}	\N	2020-10-12 14:46:02.637451
39401	EME	2020-10-08	71.71	72.25	70.785	71.64	460969	{}	\N	2020-10-12 14:46:02.641733
39402	EME	2020-10-09	72.53	72.53	70.64	71.55	358300	{}	\N	2020-10-12 14:46:02.646462
39403	EME	2020-10-12	71.71	73.035	71.71	73.035	26700	{}	\N	2020-10-12 14:46:02.65136
39404	EMR	2020-09-28	65.55	66.23	65.19	65.52	1952825	{}	\N	2020-10-12 14:46:04.747132
39405	EMR	2020-09-29	65.74	65.95	64.71	65.16	2678297	{}	\N	2020-10-12 14:46:04.753015
39406	EMR	2020-09-30	65.62	66.64	65.03	65.57	5110476	{}	\N	2020-10-12 14:46:04.757002
39407	EMR	2020-10-01	66.23	66.34	64.16	64.63	4393520	{}	\N	2020-10-12 14:46:04.760981
39408	EMR	2020-10-02	63.38	66.31	63.26	65.54	3838181	{}	\N	2020-10-12 14:46:04.76485
39409	EMR	2020-10-05	66.5	67.84	66.3064	67.59	4530598	{}	\N	2020-10-12 14:46:04.770775
39410	EMR	2020-10-06	67.78	69.13	67.17	67.28	4195994	{}	\N	2020-10-12 14:46:04.776654
39411	EMR	2020-10-07	68.21	69.07	67.87	68.88	3726589	{}	\N	2020-10-12 14:46:04.78317
39412	EMR	2020-10-08	69.23	69.58	68.7	69.28	3238957	{}	\N	2020-10-12 14:46:04.789185
39413	EMR	2020-10-09	70	71.2	69.905	69.95	5893913	{}	\N	2020-10-12 14:46:04.794291
39414	EMR	2020-10-12	70.3	70.47	69.69	69.79	730233	{}	\N	2020-10-12 14:46:04.799905
39415	EDZ	2020-09-28	18.96	19.38	18.87	19.12	207389	{}	\N	2020-10-12 14:46:07.844924
39416	EDZ	2020-09-29	19.3	19.34	18.98	19.13	106393	{}	\N	2020-10-12 14:46:07.852904
39417	EDZ	2020-09-30	18.72	18.72	18.06	18.25	204896	{}	\N	2020-10-12 14:46:07.859518
39418	EDZ	2020-10-01	17.81	18.135	17.69	17.71	153407	{}	\N	2020-10-12 14:46:07.866535
39419	EDZ	2020-10-02	18.53	18.5701	17.93	18.34	108550	{}	\N	2020-10-12 14:46:07.873532
39420	EDZ	2020-10-05	17.99	18.01	17.6	17.6	118885	{}	\N	2020-10-12 14:46:07.879669
39421	EDZ	2020-10-06	17.3903	17.66	17.1	17.53	129989	{}	\N	2020-10-12 14:46:07.88611
39422	EDZ	2020-10-07	17	17.16	16.771	16.93	200226	{}	\N	2020-10-12 14:46:07.892552
39423	EDZ	2020-10-08	16.76	16.8303	16.46	16.54	170969	{}	\N	2020-10-12 14:46:07.900094
39424	EDZ	2020-10-09	16.4	16.52	16.035	16.15	159123	{}	\N	2020-10-12 14:46:07.906839
39425	EDZ	2020-10-12	15.89	15.98	15.82	15.82	50116	{}	\N	2020-10-12 14:46:07.915133
39426	EEM	2020-09-28	43.51	43.6	43.21	43.42	45540701	{}	\N	2020-10-12 14:46:09.210297
39427	EEM	2020-09-29	43.29	43.53	43.245	43.39	21822392	{}	\N	2020-10-12 14:46:09.21592
39428	EEM	2020-09-30	43.73	44.21	43.7	44.09	40786472	{}	\N	2020-10-12 14:46:09.221247
39429	EEM	2020-10-01	44.42	44.53	44.16	44.5	50906049	{}	\N	2020-10-12 14:46:09.227189
39430	EEM	2020-10-02	43.83	44.375	43.78	43.99	24608006	{}	\N	2020-10-12 14:46:09.232652
39431	EEM	2020-10-05	44.23	44.58	44.23	44.58	32632804	{}	\N	2020-10-12 14:46:09.23879
39432	EEM	2020-10-06	44.72	45	44.51	44.66	32759958	{}	\N	2020-10-12 14:46:09.24451
39433	EEM	2020-10-07	45.07	45.29	44.94	45.15	40395692	{}	\N	2020-10-12 14:46:09.250017
39434	EEM	2020-10-08	45.31	45.57	44.9484	45.52	29461937	{}	\N	2020-10-12 14:46:09.256428
39435	EEM	2020-10-09	45.63	45.955	45.5	45.83	30635120	{}	\N	2020-10-12 14:46:09.264356
39436	EEM	2020-10-12	46.08	46.17	45.98	46.145	8733831	{}	\N	2020-10-12 14:46:09.272483
39437	EFAV	2020-09-28	68.21	68.49	68.103	68.49	936737	{}	\N	2020-10-12 14:46:11.687315
39438	EFAV	2020-09-29	68.41	68.65	68.2401	68.43	626821	{}	\N	2020-10-12 14:46:11.693173
39439	EFAV	2020-09-30	68.22	68.5099	67.98	68.11	850923	{}	\N	2020-10-12 14:46:11.697311
39440	EFAV	2020-10-01	68.41	68.5327	68.24	68.51	674013	{}	\N	2020-10-12 14:46:11.701063
39441	EFAV	2020-10-02	67.88	68.5	67.86	68.415	607721	{}	\N	2020-10-12 14:46:11.704956
39442	EFAV	2020-10-05	68.82	69.22	68.82	69.22	587842	{}	\N	2020-10-12 14:46:11.708964
39443	EFAV	2020-10-06	69.02	69.1722	68.3401	68.435	546007	{}	\N	2020-10-12 14:46:11.712872
39444	EFAV	2020-10-07	68.744	68.78	68.5401	68.68	764181	{}	\N	2020-10-12 14:46:11.716685
39445	EFAV	2020-10-08	68.76	68.93	68.76	68.9	831130	{}	\N	2020-10-12 14:46:11.720663
39446	EFAV	2020-10-09	69	69.3382	69	69.3	511423	{}	\N	2020-10-12 14:46:11.724653
39447	EFAV	2020-10-12	69.47	69.6	69.47	69.58	43904	{}	\N	2020-10-12 14:46:11.728761
39448	EFNL	2020-09-28	40.19	40.2675	40.07	40.19	17470	{}	\N	2020-10-12 14:46:13.14831
39449	EFNL	2020-09-29	40.69	40.69	40.3025	40.3025	3414	{}	\N	2020-10-12 14:46:13.154381
39450	EFNL	2020-09-30	40.42	40.51	40.155	40.22	10276	{}	\N	2020-10-12 14:46:13.1625
39451	EFNL	2020-10-01	40.8	41.03	40.65	40.88	32313	{}	\N	2020-10-12 14:46:13.169177
39452	EFNL	2020-10-02	40.22	40.644	40.22	40.644	3197	{}	\N	2020-10-12 14:46:13.175536
39453	EFNL	2020-10-05	41.38	41.5	41.13	41.5	26346	{}	\N	2020-10-12 14:46:13.181294
39454	EFNL	2020-10-06	41.615	41.615	40.8	40.8	9995	{}	\N	2020-10-12 14:46:13.186484
39455	EFNL	2020-10-07	41.23	41.35	41.05	41.31	47647	{}	\N	2020-10-12 14:46:13.191376
39456	EFNL	2020-10-08	41.22	41.5199	41.22	41.5199	4483	{}	\N	2020-10-12 14:46:13.195677
39457	EFNL	2020-10-09	41.86	42	41.8	41.97	9657	{}	\N	2020-10-12 14:46:13.200051
39458	EFNL	2020-10-12	42.25	42.25	42.09	42.2201	2842	{}	\N	2020-10-12 14:46:13.204309
39459	EFX	2020-09-28	158.68	159.43	157.48	158.11	468868	{}	\N	2020-10-12 14:46:14.107102
39460	EFX	2020-09-29	158.6	161.18	157.61	157.84	735807	{}	\N	2020-10-12 14:46:14.118696
39461	EFX	2020-09-30	158.26	159.185	156.11	156.9	994819	{}	\N	2020-10-12 14:46:14.126085
39462	EFX	2020-10-01	157.57	158.94	157.02	158.01	1151646	{}	\N	2020-10-12 14:46:14.140981
39463	EFX	2020-10-02	156.7	160.71	155.79	158.96	929960	{}	\N	2020-10-12 14:46:14.156276
39464	EFX	2020-10-05	159.23	160.36	158.24	159.87	794157	{}	\N	2020-10-12 14:46:14.165958
39465	EFX	2020-10-06	161.19	161.46	156.7	157.27	1410354	{}	\N	2020-10-12 14:46:14.17255
39466	EFX	2020-10-07	159.03	161.465	158.255	161.07	1222345	{}	\N	2020-10-12 14:46:14.179201
39467	EFX	2020-10-08	162.44	165.555	161.79	161.79	800979	{}	\N	2020-10-12 14:46:14.18619
39468	EFX	2020-10-09	163.21	165.34	161.69	164.28	626904	{}	\N	2020-10-12 14:46:14.193448
39469	EFX	2020-10-12	165.29	165.53	164.31	165.4	66272	{}	\N	2020-10-12 14:46:14.202166
39470	EGLE	2020-09-28	17.1	17.284	16.5	16.56	59471	{}	\N	2020-10-12 14:46:15.468501
39471	EGLE	2020-09-29	16.51	16.79	15.88	15.96	50186	{}	\N	2020-10-12 14:46:15.473163
39472	EGLE	2020-09-30	16.06	17.09	16.06	16.37	46410	{}	\N	2020-10-12 14:46:15.477538
39473	EGLE	2020-10-01	16.45	17.2	16.45	16.97	39597	{}	\N	2020-10-12 14:46:15.483937
39474	EGLE	2020-10-02	16.39	18.06	16.39	17.89	74302	{}	\N	2020-10-12 14:46:15.488243
39475	EGLE	2020-10-05	18.23	19.015	18.06	18.36	75013	{}	\N	2020-10-12 14:46:15.492541
39476	EGLE	2020-10-06	18.6061	18.77	18.03	18.03	66569	{}	\N	2020-10-12 14:46:15.496789
39477	EGLE	2020-10-07	18.24	18.24	16.91	17.2	58000	{}	\N	2020-10-12 14:46:15.503034
39478	EGLE	2020-10-08	17.49	18	16.23	17.29	98034	{}	\N	2020-10-12 14:46:15.508929
39479	EGLE	2020-10-09	17.59	17.61	16.835	17.25	29569	{}	\N	2020-10-12 14:46:15.513875
39480	EGLE	2020-10-12	17.28	17.3378	16.765	16.765	3582	{}	\N	2020-10-12 14:46:15.519762
39481	EHC	2020-09-28	63.37	64.035	62.785	63.06	445678	{}	\N	2020-10-12 14:46:16.952973
39482	EHC	2020-09-29	62.96	63.42	62.53	62.98	654289	{}	\N	2020-10-12 14:46:16.958757
39483	EHC	2020-09-30	63.25	65.435	63.02	64.98	1469693	{}	\N	2020-10-12 14:46:16.964212
39484	EHC	2020-10-01	65.25	66.43	65.12	65.83	1171778	{}	\N	2020-10-12 14:46:16.969019
39485	EHC	2020-10-02	64.44	66.2	64.44	65.45	667001	{}	\N	2020-10-12 14:46:16.974257
39486	EHC	2020-10-05	66.03	66.4	65.48	66.13	900897	{}	\N	2020-10-12 14:46:16.978786
39487	EHC	2020-10-06	66.36	66.93	65.13	65.33	804509	{}	\N	2020-10-12 14:46:16.983355
39488	EHC	2020-10-07	65.68	66.22	65.255	65.83	530353	{}	\N	2020-10-12 14:46:16.988857
39489	EHC	2020-10-08	66.31	66.93	65.65	66.85	533241	{}	\N	2020-10-12 14:46:16.99373
39490	EHC	2020-10-09	67.76	68.48	66.76	67.03	854792	{}	\N	2020-10-12 14:46:16.998335
39491	EHC	2020-10-12	67.26	67.51	66.97	67.095	47592	{}	\N	2020-10-12 14:46:17.003054
39492	EIG	2020-09-28	29.76	30.74	29.76	30.33	125288	{}	\N	2020-10-12 14:46:17.989544
39493	EIG	2020-09-29	30.24	30.27	29.82	30.03	173360	{}	\N	2020-10-12 14:46:17.994975
39494	EIG	2020-09-30	30.07	30.44	30.05	30.25	254274	{}	\N	2020-10-12 14:46:17.999469
39495	EIG	2020-10-01	30.23	30.47	29.91	30.19	191578	{}	\N	2020-10-12 14:46:18.003482
39496	EIG	2020-10-02	29.89	30.95	29.8	30.74	285011	{}	\N	2020-10-12 14:46:18.007469
39497	EIG	2020-10-05	31.02	31.4	31.02	31.22	154501	{}	\N	2020-10-12 14:46:18.011874
39498	EIG	2020-10-06	31.49	31.49	30.9	31.18	307668	{}	\N	2020-10-12 14:46:18.015918
39499	EIG	2020-10-07	31.41	31.7	30.945	31.19	375011	{}	\N	2020-10-12 14:46:18.021129
39500	EIG	2020-10-08	31.47	31.72	31	31.33	205945	{}	\N	2020-10-12 14:46:18.026309
39501	EIG	2020-10-09	31.34	31.63	31.17	31.42	167579	{}	\N	2020-10-12 14:46:18.030929
39502	EIG	2020-10-12	31.3	31.77	31.18	31.64	10828	{}	\N	2020-10-12 14:46:18.035052
39503	ELJ	2020-09-28	26.05	26.42	26.05	26.35	12892	{}	\N	2020-10-12 14:46:19.570394
39504	ELJ	2020-09-29	26.12	26.16	26.05	26.147	18860	{}	\N	2020-10-12 14:46:19.575399
39505	ELJ	2020-09-30	26.14	26.14	25.66	25.66	83348	{}	\N	2020-10-12 14:46:19.579296
39506	ELJ	2020-10-01	25.76	26.09	25.72	26	35798	{}	\N	2020-10-12 14:46:19.583072
39507	ELJ	2020-10-02	25.96	26.2	25.8488	26.2	30302	{}	\N	2020-10-12 14:46:19.58792
39508	ELJ	2020-10-05	26.13	26.155	25.93	26.01	33036	{}	\N	2020-10-12 14:46:19.592745
39509	ELJ	2020-10-06	25.93	26.02	25.93	26	17714	{}	\N	2020-10-12 14:46:19.596849
39510	ELJ	2020-10-07	26	26.01	25.95	25.99	15160	{}	\N	2020-10-12 14:46:19.601056
39511	ELJ	2020-10-08	26	26	25.9	25.96	11224	{}	\N	2020-10-12 14:46:19.60483
39512	ELJ	2020-10-09	25.9502	26.14	25.9	26.14	7662	{}	\N	2020-10-12 14:46:19.60865
39513	EMAG	2020-09-28	20.68	20.68	20.5642	20.5642	309	{}	\N	2020-10-12 14:46:20.978025
39514	EMAG	2020-09-29	20.64	20.64	20.64	20.64	242	{}	\N	2020-10-12 14:46:20.985927
39515	EMAG	2020-09-30	20.685	20.685	20.685	20.685	7	{}	\N	2020-10-12 14:46:20.992503
39516	EMAG	2020-10-01	20.76	20.76	20.615	20.615	212	{}	\N	2020-10-12 14:46:20.998623
39517	EMAG	2020-10-02	20.56	20.56	20.56	20.56	11	{}	\N	2020-10-12 14:46:21.007365
39518	EMAG	2020-10-05	20.4101	20.67	20.41	20.67	3128	{}	\N	2020-10-12 14:46:21.013666
39519	EMAG	2020-10-06	20.66	20.66	20.66	20.66	65	{}	\N	2020-10-12 14:46:21.020811
39520	EMAG	2020-10-07	20.51	20.635	20.435	20.635	3227	{}	\N	2020-10-12 14:46:21.029852
39521	EMAG	2020-10-08	20.765	20.765	20.765	20.765	6	{}	\N	2020-10-12 14:46:21.036801
39522	EMAG	2020-10-09	21.015	21.015	21.015	21.015	3	{}	\N	2020-10-12 14:46:21.04267
39523	EEFT	2020-09-28	89.72	89.99	87.8475	88.22	400234	{}	\N	2020-10-12 14:46:22.723023
39524	EEFT	2020-09-29	87.94	89.03	87.2715	88.58	302242	{}	\N	2020-10-12 14:46:22.727869
39525	EEFT	2020-09-30	88.89	91.83	88.6	91.1	569348	{}	\N	2020-10-12 14:46:22.732212
39526	EEFT	2020-10-01	91.61	93.37	90.98	92.3	819975	{}	\N	2020-10-12 14:46:22.736511
39527	EEFT	2020-10-02	90.06	93.0172	90	92.41	201347	{}	\N	2020-10-12 14:46:22.741783
39528	EEFT	2020-10-05	93.46	94.3	91.9491	93.58	181444	{}	\N	2020-10-12 14:46:22.746807
39529	EEFT	2020-10-06	94.995	94.995	90.355	90.82	439478	{}	\N	2020-10-12 14:46:22.75125
39530	EEFT	2020-10-07	92.38	92.425	88.74	89.13	624511	{}	\N	2020-10-12 14:46:22.756357
39531	EEFT	2020-10-08	90.71	93.295	90.11	93.14	469551	{}	\N	2020-10-12 14:46:22.761261
39532	EEFT	2020-10-09	94.32	94.56	91.66	92.24	433339	{}	\N	2020-10-12 14:46:22.765542
39533	EEFT	2020-10-12	92.45	93.36	91.6501	92.815	46603	{}	\N	2020-10-12 14:46:22.769776
39534	EEMS	2020-09-28	43.49	43.49	43.24	43.3889	2353	{}	\N	2020-10-12 14:46:24.082295
39535	EEMS	2020-09-29	43.5657	43.67	43.28	43.6	5298	{}	\N	2020-10-12 14:46:24.087099
39536	EEMS	2020-09-30	43.86	44.0945	43.75	43.79	7734	{}	\N	2020-10-12 14:46:24.091178
39537	EEMS	2020-10-01	44.305	44.39	44.03	44.39	8871	{}	\N	2020-10-12 14:46:24.09549
39538	EEMS	2020-10-02	44.21	44.21	43.86	44.13	7560	{}	\N	2020-10-12 14:46:24.100357
39539	EEMS	2020-10-05	44.5256	44.5256	44.23	44.38	17700	{}	\N	2020-10-12 14:46:24.105393
39540	EEMS	2020-10-06	45.4	45.4	44.485	44.5526	12365	{}	\N	2020-10-12 14:46:24.10966
39541	EEMS	2020-10-07	45.0084	45.01	44.8511	44.93	2863	{}	\N	2020-10-12 14:46:24.114406
39542	EEMS	2020-10-08	45	45.173	44.98	45.005	4540	{}	\N	2020-10-12 14:46:24.118733
39543	EEMS	2020-10-09	45.29	45.67	45.29	45.4	5939	{}	\N	2020-10-12 14:46:24.123162
39544	EEMS	2020-10-12	45.32	45.32	45.32	45.32	934	{}	\N	2020-10-12 14:46:24.127087
39545	EFF	2020-09-28	14.08	14.1195	14.04	14.07	19419	{}	\N	2020-10-12 14:46:25.641711
39546	EFF	2020-09-29	14.03	14.05	14.02	14.0283	14708	{}	\N	2020-10-12 14:46:25.647561
39547	EFF	2020-09-30	14.03	14.07	14.02	14.02	14950	{}	\N	2020-10-12 14:46:25.652592
39548	EFF	2020-10-01	14.02	14.05	14.02	14.02	14746	{}	\N	2020-10-12 14:46:25.656935
39549	EFF	2020-10-02	14.02	14.02	13.99	13.99	11586	{}	\N	2020-10-12 14:46:25.662393
39550	EFF	2020-10-05	14.01	14.05	14	14.05	33728	{}	\N	2020-10-12 14:46:25.667805
39551	EFF	2020-10-06	14.02	14.0734	13.97	14.02	23424	{}	\N	2020-10-12 14:46:25.672458
39552	EFF	2020-10-07	14.02	14.0901	14.02	14.03	32696	{}	\N	2020-10-12 14:46:25.677306
39553	EFF	2020-10-08	14.12	14.81	14.12	14.59	378143	{}	\N	2020-10-12 14:46:25.683327
39554	EFF	2020-10-09	14.56	14.77	14.53	14.57	32760	{}	\N	2020-10-12 14:46:25.687783
39555	EFF	2020-10-12	14.62	14.65	14.59	14.6	10984	{}	\N	2020-10-12 14:46:25.692223
39556	EFU	2020-09-28	17.69	17.6971	17.63	17.6547	1587	{}	\N	2020-10-12 14:46:27.118104
39557	EFU	2020-09-29	17.76	17.76	17.6739	17.6739	289	{}	\N	2020-10-12 14:46:27.123757
39558	EFU	2020-09-30	17.8002	17.8002	17.7599	17.7662	615	{}	\N	2020-10-12 14:46:27.128286
39559	EFU	2020-10-01	17.62	17.6634	17.62	17.6634	1144	{}	\N	2020-10-12 14:46:27.132973
39560	EFU	2020-10-02	18.08	18.08	17.71	17.71	1584	{}	\N	2020-10-12 14:46:27.137771
39561	EFU	2020-10-05	17.43	17.43	17.14	17.14	6390	{}	\N	2020-10-12 14:46:27.142274
39562	EFU	2020-10-06	17.2805	17.5079	17.2805	17.5079	466	{}	\N	2020-10-12 14:46:27.147145
39563	EFU	2020-10-07	17.2043	17.2043	17.2043	17.2043	272	{}	\N	2020-10-12 14:46:27.152209
39564	EFU	2020-10-08	16.9945	16.9945	16.9945	16.9945	86	{}	\N	2020-10-12 14:46:27.156784
39565	EFU	2020-10-09	16.78	16.78	16.701	16.701	1829	{}	\N	2020-10-12 14:46:27.161412
39566	EGRX	2020-09-28	41.67	41.67	40	41.27	153603	{}	\N	2020-10-12 14:46:29.158976
39567	EGRX	2020-09-29	41.14	42.82	41.14	41.88	355333	{}	\N	2020-10-12 14:46:29.165288
39568	EGRX	2020-09-30	42.09	43	41.555	42.545	200045	{}	\N	2020-10-12 14:46:29.170416
39569	EGRX	2020-10-01	42.67	43.75	42.495	43.01	165056	{}	\N	2020-10-12 14:46:29.177578
39570	EGRX	2020-10-02	42.47	42.97	40.39	41.72	159432	{}	\N	2020-10-12 14:46:29.184367
39571	EGRX	2020-10-05	42.02	43.36	42.02	42.87	100017	{}	\N	2020-10-12 14:46:29.190729
39572	EGRX	2020-10-06	42.89	43.7	42.315	42.9	168237	{}	\N	2020-10-12 14:46:29.195253
39573	EGRX	2020-10-07	43.42	43.94	42.905	43.63	124540	{}	\N	2020-10-12 14:46:29.199869
39574	EGRX	2020-10-08	44.2	45.62	44.15	45.25	200691	{}	\N	2020-10-12 14:46:29.205263
39575	EGRX	2020-10-09	44.625	46.78	44.625	46.35	174130	{}	\N	2020-10-12 14:46:29.209581
39576	EGRX	2020-10-12	46.22	46.47	44.041	45.85	14103	{}	\N	2020-10-12 14:46:29.21403
39577	EIRL	2020-09-28	40.65	40.7	40.55	40.67	2585	{}	\N	2020-10-12 14:46:31.165576
39578	EIRL	2020-09-29	40.47	40.758	40.42	40.74	3887	{}	\N	2020-10-12 14:46:31.171109
39579	EIRL	2020-09-30	40.5327	40.765	40.06	40.06	29675	{}	\N	2020-10-12 14:46:31.176074
39580	EIRL	2020-10-01	40.82	40.85	40.705	40.85	6255	{}	\N	2020-10-12 14:46:31.180958
39581	EIRL	2020-10-02	40.66	40.9	40.43	40.57	3996	{}	\N	2020-10-12 14:46:31.185283
39582	EIRL	2020-10-05	41.1723	41.44	41.086	41.3437	1693	{}	\N	2020-10-12 14:46:31.190175
39583	EIRL	2020-10-06	41.6016	41.99	41.39	41.39	2654	{}	\N	2020-10-12 14:46:31.194952
39584	EIRL	2020-10-07	41.76	42.2993	41.76	42.2993	2052	{}	\N	2020-10-12 14:46:31.199523
39585	EIRL	2020-10-08	42.47	42.6516	42.4606	42.6516	1663	{}	\N	2020-10-12 14:46:31.204166
39586	EIRL	2020-10-09	42.8173	43.32	42.8173	43.18	2402	{}	\N	2020-10-12 14:46:31.20866
39587	EMAN	2020-09-28	1.17	1.22	1.17	1.2	519591	{}	\N	2020-10-12 14:46:32.842177
39588	EMAN	2020-09-29	1.1602	1.19	1.16	1.16	399563	{}	\N	2020-10-12 14:46:32.847917
39589	EMAN	2020-09-30	1.18	1.27	1.18	1.27	1008007	{}	\N	2020-10-12 14:46:32.852481
39590	EMAN	2020-10-01	1.28	1.3	1.2	1.25	713133	{}	\N	2020-10-12 14:46:32.857694
39591	EMAN	2020-10-02	1.15	1.23	1.15	1.23	436131	{}	\N	2020-10-12 14:46:32.862607
39592	EMAN	2020-10-05	1.23	1.2582	1.18	1.19	334921	{}	\N	2020-10-12 14:46:32.867216
39593	EMAN	2020-10-06	1.1955	1.22	1.16	1.17	303568	{}	\N	2020-10-12 14:46:32.871965
39594	EMAN	2020-10-07	1.191	1.2	1.17	1.2	367661	{}	\N	2020-10-12 14:46:32.876524
39595	EMAN	2020-10-08	1.2	1.24	1.17	1.22	641304	{}	\N	2020-10-12 14:46:32.881165
39596	EMAN	2020-10-09	1.21	1.22	1.18	1.19	447882	{}	\N	2020-10-12 14:46:32.8857
39597	EMAN	2020-10-12	1.18	1.21	1.18	1.19	96043	{}	\N	2020-10-12 14:46:32.890594
39598	EMGF	2020-09-28	43.52	43.59	43.22	43.33	30446	{}	\N	2020-10-12 14:46:33.830639
39599	EMGF	2020-09-29	43.29	43.5081	43.279	43.35	40297	{}	\N	2020-10-12 14:46:33.836478
39600	EMGF	2020-09-30	43.76	44.03	43.76	43.88	50112	{}	\N	2020-10-12 14:46:33.84185
39601	EMGF	2020-10-01	44.22	44.4295	44.082	44.38	31101	{}	\N	2020-10-12 14:46:33.849944
39602	EMGF	2020-10-02	43.88	44.24	43.85	43.85	44973	{}	\N	2020-10-12 14:46:33.855134
39603	EMGF	2020-10-05	44.21	44.37	44.14	44.32	28918	{}	\N	2020-10-12 14:46:33.859651
39604	EMGF	2020-10-06	44.55	44.682	44.27	44.27	27135	{}	\N	2020-10-12 14:46:33.864499
39605	EMGF	2020-10-07	44.7	44.78	44.54	44.7	62870	{}	\N	2020-10-12 14:46:33.869015
39606	EMGF	2020-10-08	45.04	45.28	44.9263	45.26	68560	{}	\N	2020-10-12 14:46:33.873585
39607	EMGF	2020-10-09	45.26	45.7104	45.26	45.64	223653	{}	\N	2020-10-12 14:46:33.877982
39608	EMGF	2020-10-12	45.76	45.87	45.7589	45.87	4611	{}	\N	2020-10-12 14:46:33.88293
39609	EMN	2020-09-28	78.44	79.43	78.11	78.51	1027280	{}	\N	2020-10-12 14:46:34.809383
39610	EMN	2020-09-29	78.44	78.96	76.9168	77.15	1007197	{}	\N	2020-10-12 14:46:34.816207
39611	EMN	2020-09-30	77.45	79.13	77.45	78.12	1958658	{}	\N	2020-10-12 14:46:34.820676
39612	EMN	2020-10-01	78.08	78.76	76.01	76.31	1674163	{}	\N	2020-10-12 14:46:34.825305
39613	EMN	2020-10-02	75.01	78.99	74.84	78.17	1646028	{}	\N	2020-10-12 14:46:34.830364
39614	EMN	2020-10-05	78.89	81.01	78.82	80.51	1758956	{}	\N	2020-10-12 14:46:34.835854
39615	EMN	2020-10-06	81.24	82.9	79.93	80.25	2537999	{}	\N	2020-10-12 14:46:34.840368
39616	EMN	2020-10-07	81.85	84.24	81.85	83.52	1716034	{}	\N	2020-10-12 14:46:34.844869
39617	EMN	2020-10-08	84	85.44	83.705	85.4	1597271	{}	\N	2020-10-12 14:46:34.849446
39618	EMN	2020-10-09	86.07	86.72	85.455	86.16	1400313	{}	\N	2020-10-12 14:46:34.854082
39619	EMN	2020-10-12	86.66	86.66	85.33	85.66	299766	{}	\N	2020-10-12 14:46:34.858832
39620	ENB	2020-09-28	30.2	30.49	29.95	30.08	2371318	{}	\N	2020-10-12 14:46:37.014527
39621	ENB	2020-09-29	30.01	30.02	29.39	29.55	4507285	{}	\N	2020-10-12 14:46:37.021847
39622	ENB	2020-09-30	29.64	29.795	29.19	29.2	6898635	{}	\N	2020-10-12 14:46:37.026538
39623	ENB	2020-10-01	29.22	29.445	28.92	29.04	6755154	{}	\N	2020-10-12 14:46:37.030918
39624	ENB	2020-10-02	28.5	29.16	28.45	28.96	5754436	{}	\N	2020-10-12 14:46:37.036294
39625	ENB	2020-10-05	29.3	29.59	29.08	29.54	4508586	{}	\N	2020-10-12 14:46:37.041058
39626	ENB	2020-10-06	29.84	29.86	28.865	28.96	8023623	{}	\N	2020-10-12 14:46:37.045851
39627	ENB	2020-10-07	29.04	29.29	28.89	29.21	7296209	{}	\N	2020-10-12 14:46:37.050459
39628	ENB	2020-10-08	29.37	30.33	29.34	30.32	5756382	{}	\N	2020-10-12 14:46:37.054681
39629	ENB	2020-10-09	30.42	30.57	29.805	29.83	5147979	{}	\N	2020-10-12 14:46:37.059006
39630	ENB	2020-10-12	29.91	30.16	29.61	29.66	1141826	{}	\N	2020-10-12 14:46:37.063354
39631	EGHT	2020-09-28	15.5	15.89	15.49	15.71	977344	{}	\N	2020-10-12 14:46:38.056235
39632	EGHT	2020-09-29	15.63	15.81	15.47	15.58	1287539	{}	\N	2020-10-12 14:46:38.061503
39633	EGHT	2020-09-30	15.57	15.77	15.445	15.55	2066942	{}	\N	2020-10-12 14:46:38.065841
39634	EGHT	2020-10-01	15.72	15.87	15.33	15.78	3612581	{}	\N	2020-10-12 14:46:38.070704
39635	EGHT	2020-10-02	15.46	15.82	15.35	15.66	3149436	{}	\N	2020-10-12 14:46:38.075952
39636	EGHT	2020-10-05	15.73	16.07	15.66	15.75	1754937	{}	\N	2020-10-12 14:46:38.081181
39637	EGHT	2020-10-06	15.88	16.39	15.69	15.83	1625237	{}	\N	2020-10-12 14:46:38.086504
39638	EGHT	2020-10-07	15.95	16.13	15.65	15.96	1428920	{}	\N	2020-10-12 14:46:38.091496
39639	EGHT	2020-10-08	16.13	16.35	15.85	16.25	1116803	{}	\N	2020-10-12 14:46:38.096423
39640	EGHT	2020-10-09	16.26	16.54	16.17	16.49	863464	{}	\N	2020-10-12 14:46:38.101318
39641	EGHT	2020-10-12	16.65	16.8	16.49	16.75	205454	{}	\N	2020-10-12 14:46:38.106028
39642	EHT	2020-09-28	9.4804	9.4875	9.4	9.45	53863	{}	\N	2020-10-12 14:46:39.50826
39643	EHT	2020-09-29	9.42	9.47	9.37	9.4302	111156	{}	\N	2020-10-12 14:46:39.513493
39644	EHT	2020-09-30	9.4401	9.51	9.44	9.47	81748	{}	\N	2020-10-12 14:46:39.517178
39645	EHT	2020-10-01	9.51	9.5299	9.43	9.47	99416	{}	\N	2020-10-12 14:46:39.520726
39646	EHT	2020-10-02	9.5299	9.57	9.42	9.57	172332	{}	\N	2020-10-12 14:46:39.524463
39647	EHT	2020-10-05	9.52	9.57	9.4895	9.57	85088	{}	\N	2020-10-12 14:46:39.528273
39648	EHT	2020-10-06	9.54	9.59	9.51	9.55	77783	{}	\N	2020-10-12 14:46:39.532284
39649	EHT	2020-10-07	9.55	9.5959	9.55	9.5931	74576	{}	\N	2020-10-12 14:46:39.536427
39650	EHT	2020-10-08	9.57	9.658	9.57	9.65	51430	{}	\N	2020-10-12 14:46:39.540709
39651	EHT	2020-10-09	9.64	9.6899	9.61	9.64	49298	{}	\N	2020-10-12 14:46:39.54488
39652	EHT	2020-10-12	9.61	9.6773	9.61	9.63	12730	{}	\N	2020-10-12 14:46:39.548816
39653	ELC	2020-09-28	26.12	26.2	26.05	26.05	13330	{}	\N	2020-10-12 14:46:41.465605
39654	ELC	2020-09-29	26.01	26.27	25.99	26.27	17149	{}	\N	2020-10-12 14:46:41.472533
39655	ELC	2020-09-30	26.19	26.19	25.75	25.75	92487	{}	\N	2020-10-12 14:46:41.477529
39656	ELC	2020-10-01	25.84	26.0325	25.82	26.03	43524	{}	\N	2020-10-12 14:46:41.482286
39657	ELC	2020-10-02	25.93	26.06	25.91	26.01	24350	{}	\N	2020-10-12 14:46:41.486701
39658	ELC	2020-10-05	25.92	26.23	25.92	26.21	17096	{}	\N	2020-10-12 14:46:41.491599
39659	ELC	2020-10-06	26.23	26.36	26.16	26.3	35923	{}	\N	2020-10-12 14:46:41.495706
39660	ELC	2020-10-07	26.2	26.29	26.09	26.28	23858	{}	\N	2020-10-12 14:46:41.499674
39661	ELC	2020-10-08	26.21	26.3756	26.16	26.33	22480	{}	\N	2020-10-12 14:46:41.503886
39662	ELC	2020-10-09	26.36	26.36	26.11	26.22	16892	{}	\N	2020-10-12 14:46:41.507864
39663	ELC	2020-10-12	26.17	26.17	26.17	26.17	1972	{}	\N	2020-10-12 14:46:41.511801
39664	ELTK	2020-09-28	4.7	4.8	4.6512	4.78	2942	{}	\N	2020-10-12 14:46:43.113447
39665	ELTK	2020-09-29	4.8164	4.8164	4.7	4.7143	3400	{}	\N	2020-10-12 14:46:43.119552
39666	ELTK	2020-09-30	4.71	4.83	4.71	4.75	7504	{}	\N	2020-10-12 14:46:43.124805
39667	ELTK	2020-10-01	4.7388	4.81	4.7	4.7	1452	{}	\N	2020-10-12 14:46:43.1287
39668	ELTK	2020-10-02	4.69	4.8	4.665	4.68	10029	{}	\N	2020-10-12 14:46:43.132886
39669	ELTK	2020-10-05	4.7	4.78	4.668	4.74	4232	{}	\N	2020-10-12 14:46:43.137661
39670	ELTK	2020-10-06	4.7538	5.19	4.7538	5.1	51746	{}	\N	2020-10-12 14:46:43.142237
39671	ELTK	2020-10-07	5.0646	5.09	4.95	4.95	5602	{}	\N	2020-10-12 14:46:43.146297
39672	ELTK	2020-10-08	4.95	5.05	4.94	4.94	6877	{}	\N	2020-10-12 14:46:43.150891
39673	ELTK	2020-10-09	4.92	4.95	4.88	4.91	2413	{}	\N	2020-10-12 14:46:43.154803
39674	ELTK	2020-10-12	4.9498	5.15	4.9	4.95	19154	{}	\N	2020-10-12 14:46:43.159122
39675	EMP	2020-09-28	26.42	26.65	26.34	26.35	22675	{}	\N	2020-10-12 14:46:45.936948
39676	EMP	2020-09-29	26.15	26.2636	25.92	25.99	44930	{}	\N	2020-10-12 14:46:45.944322
39677	EMP	2020-09-30	26.11	26.11	25.46	25.52	562293	{}	\N	2020-10-12 14:46:45.950304
39678	EMP	2020-10-01	25.68	25.89	25.65	25.89	133664	{}	\N	2020-10-12 14:46:45.954296
39679	EMP	2020-10-02	25.84	25.84	25.61	25.77	111316	{}	\N	2020-10-12 14:46:45.95883
39680	EMP	2020-10-05	25.86	25.98	25.85	25.97	59120	{}	\N	2020-10-12 14:46:45.963015
39681	EMP	2020-10-06	25.99	25.99	25.6679	25.75	68958	{}	\N	2020-10-12 14:46:45.967611
39682	EMP	2020-10-07	25.82	25.92	25.81	25.87	46286	{}	\N	2020-10-12 14:46:45.971741
39683	EMP	2020-10-08	25.88	25.96	25.83	25.89	42870	{}	\N	2020-10-12 14:46:45.975954
39684	EMP	2020-10-09	25.9	25.93	25.77	25.84	32190	{}	\N	2020-10-12 14:46:45.980644
39685	EMP	2020-10-12	25.92	25.92	25.82	25.91	4478	{}	\N	2020-10-12 14:46:45.984899
39686	ENBL	2020-09-28	4.02	4.1365	3.9403	3.98	824782	{}	\N	2020-10-12 14:46:47.032706
39687	ENBL	2020-09-29	4.02	4.205	3.99	4.11	1354483	{}	\N	2020-10-12 14:46:47.039212
39688	ENBL	2020-09-30	4.19	4.29	4.1	4.14	3128564	{}	\N	2020-10-12 14:46:47.043018
39689	ENBL	2020-10-01	4.18	4.49	4.12	4.46	2493063	{}	\N	2020-10-12 14:46:47.046976
39690	ENBL	2020-10-02	4.38	4.56	4.31	4.45	1845635	{}	\N	2020-10-12 14:46:47.051686
39691	ENBL	2020-10-05	4.49	4.64	4.41	4.45	1286710	{}	\N	2020-10-12 14:46:47.05626
39692	ENBL	2020-10-06	4.46	4.62	4.36	4.38	1191873	{}	\N	2020-10-12 14:46:47.060546
39693	ENBL	2020-10-07	4.41	4.47	4.315	4.43	860527	{}	\N	2020-10-12 14:46:47.064577
39694	ENBL	2020-10-08	4.46	4.75	4.4	4.72	1395061	{}	\N	2020-10-12 14:46:47.068725
39695	ENBL	2020-10-09	4.82	4.825	4.56	4.74	1412676	{}	\N	2020-10-12 14:46:47.072636
39696	ENBL	2020-10-12	4.73	4.73	4.62	4.68	125641	{}	\N	2020-10-12 14:46:47.076541
39697	ENSG	2020-09-28	56.65	57.45	56.2	56.49	202379	{}	\N	2020-10-12 14:46:49.257095
39698	ENSG	2020-09-29	56.46	56.95	55.32	56.04	131483	{}	\N	2020-10-12 14:46:49.262788
39699	ENSG	2020-09-30	56.1	57.75	56.1	57.06	278359	{}	\N	2020-10-12 14:46:49.267706
39700	ENSG	2020-10-01	57.37	58.2622	55.8	56.68	267987	{}	\N	2020-10-12 14:46:49.27177
39701	ENSG	2020-10-02	55.39	56.9	55	56.39	125057	{}	\N	2020-10-12 14:46:49.276397
39702	ENSG	2020-10-05	57.03	58.12	56.66	57.85	149105	{}	\N	2020-10-12 14:46:49.281392
39703	ENSG	2020-10-06	58.65	59.09	57.2047	57.29	242730	{}	\N	2020-10-12 14:46:49.285563
39704	ENSG	2020-10-07	57.83	58.51	57.23	57.98	214879	{}	\N	2020-10-12 14:46:49.289651
39705	ENSG	2020-10-08	58.73	59.65	58.605	59.37	123599	{}	\N	2020-10-12 14:46:49.293831
39706	ENSG	2020-10-09	60	60.29	58.73	59.5	130466	{}	\N	2020-10-12 14:46:49.297752
39707	ENSG	2020-10-12	59.19	59.23	58.18	58.631	37472	{}	\N	2020-10-12 14:46:49.302817
39708	ENZ	2020-09-28	2.16	2.2099	2.13	2.14	154723	{}	\N	2020-10-12 14:46:50.24727
39709	ENZ	2020-09-29	2.13	2.15	2.0645	2.08	423310	{}	\N	2020-10-12 14:46:50.255173
39710	ENZ	2020-09-30	2.09	2.15	2.06	2.11	295856	{}	\N	2020-10-12 14:46:50.26076
39711	ENZ	2020-10-01	2.12	2.17	2.08	2.15	256164	{}	\N	2020-10-12 14:46:50.266134
39712	ENZ	2020-10-02	2.09	2.16	2.06	2.1	318829	{}	\N	2020-10-12 14:46:50.271272
39713	ENZ	2020-10-05	2.08	2.24	2.08	2.21	392234	{}	\N	2020-10-12 14:46:50.27579
39714	ENZ	2020-10-06	2.26	2.35	2.15	2.18	386179	{}	\N	2020-10-12 14:46:50.280551
39715	ENZ	2020-10-07	2.22	2.28	2.19	2.24	254280	{}	\N	2020-10-12 14:46:50.285212
39716	ENZ	2020-10-08	2.26	2.42	2.25	2.36	570132	{}	\N	2020-10-12 14:46:50.289816
39717	ENZ	2020-10-09	2.36	2.39	2.27	2.31	282119	{}	\N	2020-10-12 14:46:50.294421
39718	ENZ	2020-10-12	2.35	2.41	2.3317	2.4	74418	{}	\N	2020-10-12 14:46:50.299014
39719	EPC	2020-09-28	27.66	28.22	27.63	27.77	361085	{}	\N	2020-10-12 14:46:51.327178
39720	EPC	2020-09-29	27.83	27.93	27.22	27.42	750184	{}	\N	2020-10-12 14:46:51.333398
39721	EPC	2020-09-30	27.56	28.14	27.42	27.88	796039	{}	\N	2020-10-12 14:46:51.338605
39722	EPC	2020-10-01	27.95	28.33	27.45	28.22	685349	{}	\N	2020-10-12 14:46:51.342845
39723	EPC	2020-10-02	27.93	29.25	27.74	28.94	1216634	{}	\N	2020-10-12 14:46:51.350056
39724	EPC	2020-10-05	29.2	29.2	28.36	28.91	696257	{}	\N	2020-10-12 14:46:51.355149
39725	EPC	2020-10-06	29.15	30.1	28.93	29.58	694325	{}	\N	2020-10-12 14:46:51.359543
39726	EPC	2020-10-07	29.55	30.4	29.14	29.18	1086190	{}	\N	2020-10-12 14:46:51.364599
39727	EPC	2020-10-08	29.48	29.75	29.27	29.66	588211	{}	\N	2020-10-12 14:46:51.369999
39728	EPC	2020-10-09	29.59	30.2	29.48	29.76	571482	{}	\N	2020-10-12 14:46:51.374671
39729	EPC	2020-10-12	29.71	30.2	29.69	30	76745	{}	\N	2020-10-12 14:46:51.379256
39730	EPRF	2020-09-28	24.5	24.5	24.4024	24.4701	58369	{}	\N	2020-10-12 14:46:52.99803
39731	EPRF	2020-09-29	24.3517	24.39	24.29	24.33	127526	{}	\N	2020-10-12 14:46:53.005877
39732	EPRF	2020-09-30	24.3838	24.41	24.24	24.24	112252	{}	\N	2020-10-12 14:46:53.012428
39733	EPRF	2020-10-01	24.2	24.3999	24.2	24.38	49339	{}	\N	2020-10-12 14:46:53.019755
39734	EPRF	2020-10-02	24.32	24.46	24.3	24.46	34800	{}	\N	2020-10-12 14:46:53.024939
39735	EPRF	2020-10-05	24.58	24.58	24.47	24.53	85867	{}	\N	2020-10-12 14:46:53.030458
39736	EPRF	2020-10-06	24.55	24.6176	24.5081	24.57	59321	{}	\N	2020-10-12 14:46:53.035399
39737	EPRF	2020-10-07	24.63	24.63	24.55	24.55	50039	{}	\N	2020-10-12 14:46:53.040808
39738	EPRF	2020-10-08	24.74	24.74	24.595	24.6333	140943	{}	\N	2020-10-12 14:46:53.045429
39739	EPRF	2020-10-09	24.6	24.6699	24.56	24.6	46305	{}	\N	2020-10-12 14:46:53.049832
39740	EPRF	2020-10-12	24.73	24.73	24.6	24.6	21988	{}	\N	2020-10-12 14:46:53.054487
39741	EGOV	2020-09-28	19.88	20.175	19.6288	20.07	371744	{}	\N	2020-10-12 14:46:54.567101
39742	EGOV	2020-09-29	20.04	20.22	19.79	19.82	455983	{}	\N	2020-10-12 14:46:54.572869
39743	EGOV	2020-09-30	19.9	20.14	19.59	19.7	341420	{}	\N	2020-10-12 14:46:54.578116
39744	EGOV	2020-10-01	19.55	19.88	19.41	19.87	299077	{}	\N	2020-10-12 14:46:54.582141
39745	EGOV	2020-10-02	19.63	20.28	19.63	20.18	258591	{}	\N	2020-10-12 14:46:54.586168
39746	EGOV	2020-10-05	20.26	20.69	20.1749	20.59	262161	{}	\N	2020-10-12 14:46:54.589879
39747	EGOV	2020-10-06	20.63	21	20.405	20.72	239120	{}	\N	2020-10-12 14:46:54.593932
39748	EGOV	2020-10-07	21	21	20.445	20.62	397996	{}	\N	2020-10-12 14:46:54.597963
39749	EGOV	2020-10-08	20.8	21.25	20.62	20.92	192241	{}	\N	2020-10-12 14:46:54.601655
39750	EGOV	2020-10-09	21.07	21.2	20.9	21.15	190308	{}	\N	2020-10-12 14:46:54.605408
39751	EGOV	2020-10-12	21.31	21.37	21	21.12	25253	{}	\N	2020-10-12 14:46:54.609215
39752	EKSO	2020-09-28	4.34	4.5299	4.3201	4.37	72386	{}	\N	2020-10-12 14:46:56.367929
39753	EKSO	2020-09-29	4.36	4.53	4.325	4.445	45892	{}	\N	2020-10-12 14:46:56.373501
39754	EKSO	2020-09-30	4.5	4.87	4.5	4.75	76891	{}	\N	2020-10-12 14:46:56.37868
39755	EKSO	2020-10-01	4.8	4.86	4.2665	4.42	271485	{}	\N	2020-10-12 14:46:56.383441
39756	EKSO	2020-10-02	4.26	4.44	4.25	4.39	50925	{}	\N	2020-10-12 14:46:56.387825
39757	EKSO	2020-10-05	4.49	4.7	4.47	4.5	68754	{}	\N	2020-10-12 14:46:56.391645
39758	EKSO	2020-10-06	4.45	4.59	4.38	4.38	61133	{}	\N	2020-10-12 14:46:56.396147
39759	EKSO	2020-10-07	4.44	4.7	4.4202	4.68	101776	{}	\N	2020-10-12 14:46:56.400105
39760	EKSO	2020-10-08	4.72	5.06	4.7	5.04	92920	{}	\N	2020-10-12 14:46:56.404303
39761	EKSO	2020-10-09	5.09	5.5	5.05	5.08	110024	{}	\N	2020-10-12 14:46:56.408585
39762	EKSO	2020-10-12	5.11	5.37	5.11	5.33	15058	{}	\N	2020-10-12 14:46:56.413031
39763	ELP	2020-09-28	11.54	11.58	10.84	10.94	581521	{}	\N	2020-10-12 14:46:58.053933
39764	ELP	2020-09-29	10.8	11.06	10.7	10.95	1259030	{}	\N	2020-10-12 14:46:58.060103
39765	ELP	2020-09-30	10.97	11.12	10.8	11.02	1006490	{}	\N	2020-10-12 14:46:58.065689
39766	ELP	2020-10-01	10.92	11	10.64	10.97	1307743	{}	\N	2020-10-12 14:46:58.070423
39767	ELP	2020-10-02	10.92	11.09	10.78	10.9	1265609	{}	\N	2020-10-12 14:46:58.074674
39768	ELP	2020-10-05	10.91	11.33	10.805	11.22	1146765	{}	\N	2020-10-12 14:46:58.079072
39769	ELP	2020-10-06	11.37	11.605	11.1	11.25	1371233	{}	\N	2020-10-12 14:46:58.083868
39770	ELP	2020-10-07	11.24	11.31	11.025	11.24	1793805	{}	\N	2020-10-12 14:46:58.08796
39771	ELP	2020-10-08	11.3	11.63	11.23	11.63	1994262	{}	\N	2020-10-12 14:46:58.092271
39772	ELP	2020-10-09	11.72	11.965	11.64	11.74	1028395	{}	\N	2020-10-12 14:46:58.096758
39773	ELP	2020-10-12	11.75	11.81	11.38	11.53	204340	{}	\N	2020-10-12 14:46:58.101373
39774	EMB	2020-09-28	110.73	110.868	110.49	110.6	3876746	{}	\N	2020-10-12 14:46:59.121273
39775	EMB	2020-09-29	110.57	110.62	110.27	110.3884	3030844	{}	\N	2020-10-12 14:46:59.128588
39776	EMB	2020-09-30	110.36	111.05	110.28	110.89	5545259	{}	\N	2020-10-12 14:46:59.134366
39777	EMB	2020-10-01	110.94	111	110.675	110.73	13639297	{}	\N	2020-10-12 14:46:59.138746
39778	EMB	2020-10-02	110.23	110.56	110.022	110.34	7170459	{}	\N	2020-10-12 14:46:59.143541
39779	EMB	2020-10-05	110.5	110.92	110.46	110.77	3952535	{}	\N	2020-10-12 14:46:59.148155
39780	EMB	2020-10-06	110.98	111.15	110.72	110.85	5388336	{}	\N	2020-10-12 14:46:59.153069
39781	EMB	2020-10-07	111.16	111.25	110.89	111.09	4400495	{}	\N	2020-10-12 14:46:59.157482
39782	EMB	2020-10-08	111.65	112.34	111.535	112.17	4778481	{}	\N	2020-10-12 14:46:59.162124
39783	EMB	2020-10-09	112.44	112.55	112.31	112.44	3937792	{}	\N	2020-10-12 14:46:59.167721
39784	EMB	2020-10-12	112.71	112.81	112.47	112.57	614062	{}	\N	2020-10-12 14:46:59.172714
39785	EMLC	2020-09-28	30.64	30.67	30.3919	30.55	2613787	{}	\N	2020-10-12 14:47:01.011348
39786	EMLC	2020-09-29	30.6	30.71	30.53	30.63	1287529	{}	\N	2020-10-12 14:47:01.017408
39787	EMLC	2020-09-30	30.7	30.9	30.67	30.84	1827695	{}	\N	2020-10-12 14:47:01.0226
39788	EMLC	2020-10-01	30.8	30.9	30.76	30.85	1519214	{}	\N	2020-10-12 14:47:01.028164
39789	EMLC	2020-10-02	30.73	30.85	30.685	30.73	2643113	{}	\N	2020-10-12 14:47:01.034709
39790	EMLC	2020-10-05	30.84	30.94	30.83	30.87	1518546	{}	\N	2020-10-12 14:47:01.039419
39791	EMLC	2020-10-06	31.005	31.04	30.7547	30.81	820585	{}	\N	2020-10-12 14:47:01.043682
39792	EMLC	2020-10-07	30.8638	30.91	30.81	30.85	635585	{}	\N	2020-10-12 14:47:01.04785
39793	EMLC	2020-10-08	30.88	30.995	30.88	30.99	764949	{}	\N	2020-10-12 14:47:01.052464
39794	EMLC	2020-10-09	31.11	31.23	31.11	31.21	431029	{}	\N	2020-10-12 14:47:01.056518
39795	EMLC	2020-10-12	31.15	31.15	31.1	31.12	186786	{}	\N	2020-10-12 14:47:01.060933
39796	EMX	2020-09-28	2.61	2.68	2.61	2.68	167093	{}	\N	2020-10-12 14:47:02.349079
39797	EMX	2020-09-29	2.7233	2.73	2.69	2.72	135504	{}	\N	2020-10-12 14:47:02.358355
39798	EMX	2020-09-30	2.7	2.7	2.62	2.63	89557	{}	\N	2020-10-12 14:47:02.363759
39799	EMX	2020-10-01	2.66	2.71	2.6	2.68	125301	{}	\N	2020-10-12 14:47:02.368727
39800	EMX	2020-10-02	2.66	2.71	2.5715	2.64	174696	{}	\N	2020-10-12 14:47:02.373198
39801	EMX	2020-10-05	2.68	2.78	2.63	2.78	210373	{}	\N	2020-10-12 14:47:02.377601
39802	EMX	2020-10-06	2.76	2.79	2.65	2.66	174798	{}	\N	2020-10-12 14:47:02.382524
39803	EMX	2020-10-07	2.64	3	2.64	2.8	316418	{}	\N	2020-10-12 14:47:02.387725
39804	EMX	2020-10-08	2.81	2.9	2.7	2.75	193721	{}	\N	2020-10-12 14:47:02.392759
39805	EMX	2020-10-09	2.81	2.87	2.76	2.84	152628	{}	\N	2020-10-12 14:47:02.397275
39806	EMX	2020-10-12	2.8927	2.95	2.85	2.91	43542	{}	\N	2020-10-12 14:47:02.402127
39807	ENG	2020-09-28	0.78	0.82	0.779551	0.8	61304	{}	\N	2020-10-12 14:47:04.363436
39808	ENG	2020-09-29	0.84	0.91	0.8	0.899	755210	{}	\N	2020-10-12 14:47:04.368909
39809	ENG	2020-09-30	0.882	0.889	0.8122	0.859	211684	{}	\N	2020-10-12 14:47:04.373198
39810	ENG	2020-10-01	0.87	0.87	0.8107	0.865	42917	{}	\N	2020-10-12 14:47:04.377732
39811	ENG	2020-10-02	0.815	0.85	0.765148	0.8262	51670	{}	\N	2020-10-12 14:47:04.381644
39812	ENG	2020-10-05	0.82	0.840501	0.8	0.8072	63083	{}	\N	2020-10-12 14:47:04.385789
39813	ENG	2020-10-06	0.8175	0.88	0.8075	0.832	135418	{}	\N	2020-10-12 14:47:04.389541
39814	ENG	2020-10-07	0.85	1.33	0.8337	1.22	10424090	{}	\N	2020-10-12 14:47:04.393346
39815	ENG	2020-10-08	1.04	1.2301	1	1.08	3562675	{}	\N	2020-10-12 14:47:04.397552
39816	ENG	2020-10-09	1.05	1.16	1.03	1.1	3117035	{}	\N	2020-10-12 14:47:04.401579
39817	ENG	2020-10-12	1.13	1.14	1.05	1.07	472766	{}	\N	2020-10-12 14:47:04.405438
39818	ENS	2020-09-28	65.29	67.35	64.7	66.43	182713	{}	\N	2020-10-12 14:47:05.387102
39819	ENS	2020-09-29	66.64	67.79	66.16	67.25	380144	{}	\N	2020-10-12 14:47:05.392661
39820	ENS	2020-09-30	67.32	68.29	66.52	67.12	337083	{}	\N	2020-10-12 14:47:05.398301
39821	ENS	2020-10-01	67.97	68.41	67.07	68.06	431911	{}	\N	2020-10-12 14:47:05.402807
39822	ENS	2020-10-02	66.38	69.29	66.38	68.57	292612	{}	\N	2020-10-12 14:47:05.407142
39823	ENS	2020-10-05	69.49	71.7	69.49	71.37	384931	{}	\N	2020-10-12 14:47:05.413079
39824	ENS	2020-10-06	72.44	72.95	70.48	70.53	426687	{}	\N	2020-10-12 14:47:05.419565
39825	ENS	2020-10-07	71.7	72.85	71.35	72.52	360150	{}	\N	2020-10-12 14:47:05.426627
39826	ENS	2020-10-08	73.66	74.19	72.73	73.51	364316	{}	\N	2020-10-12 14:47:05.433428
39827	ENS	2020-10-09	74.6	75.41	72.62	73.18	389905	{}	\N	2020-10-12 14:47:05.438623
39828	ENS	2020-10-12	73.56	74.65	73.15	74.033	73399	{}	\N	2020-10-12 14:47:05.444472
39829	ENX	2020-09-28	11.84	11.84	11.8008	11.83	36282	{}	\N	2020-10-12 14:47:06.625352
39830	ENX	2020-09-29	11.84	11.85	11.8002	11.81	29696	{}	\N	2020-10-12 14:47:06.63164
39831	ENX	2020-09-30	11.8373	11.87	11.8	11.8	7365	{}	\N	2020-10-12 14:47:06.636614
39832	ENX	2020-10-01	11.82	11.83	11.74	11.77	35488	{}	\N	2020-10-12 14:47:06.641029
39833	ENX	2020-10-02	11.74	11.76	11.68	11.68	19820	{}	\N	2020-10-12 14:47:06.645665
39834	ENX	2020-10-05	11.73	11.73	11.584	11.59	34680	{}	\N	2020-10-12 14:47:06.650222
39835	ENX	2020-10-06	11.52	11.588	11.52	11.56	33683	{}	\N	2020-10-12 14:47:06.654569
39836	ENX	2020-10-07	11.54	11.6	11.54	11.56	22807	{}	\N	2020-10-12 14:47:06.658833
39837	ENX	2020-10-08	11.7	11.89	11.69	11.89	125943	{}	\N	2020-10-12 14:47:06.664294
39838	ENX	2020-10-09	11.86	11.93	11.86	11.91	36911	{}	\N	2020-10-12 14:47:06.669915
39839	ENX	2020-10-12	11.91	12.02	11.91	11.92	4407	{}	\N	2020-10-12 14:47:06.675289
39840	EPAM	2020-09-28	325.54	331.98	325.365	328.96	215011	{}	\N	2020-10-12 14:47:07.721155
39841	EPAM	2020-09-29	330.22	332.74	324.09	325.56	461126	{}	\N	2020-10-12 14:47:07.727514
39842	EPAM	2020-09-30	328.65	330.39	322.69	323.28	599337	{}	\N	2020-10-12 14:47:07.732675
39843	EPAM	2020-10-01	328.32	335.32	326.5101	334.12	691525	{}	\N	2020-10-12 14:47:07.737022
39844	EPAM	2020-10-02	326.51	333.52	324.6701	326.92	356312	{}	\N	2020-10-12 14:47:07.741346
39845	EPAM	2020-10-05	330.34	335	328.6537	333.78	519651	{}	\N	2020-10-12 14:47:07.74602
39846	EPAM	2020-10-06	334.02	344.36	330.01	335.34	961991	{}	\N	2020-10-12 14:47:07.750649
39847	EPAM	2020-10-07	339.22	344.98	339.01	344.32	313421	{}	\N	2020-10-12 14:47:07.755055
39848	EPAM	2020-10-08	346	348.125	331.5	334.64	784539	{}	\N	2020-10-12 14:47:07.759638
39849	EPAM	2020-10-09	337.87	342.9999	336.52	337.25	514779	{}	\N	2020-10-12 14:47:07.764642
39850	EPAM	2020-10-12	343	345	337.6312	341.7	58229	{}	\N	2020-10-12 14:47:07.769201
39851	EPP	2020-09-28	40.89	41.01	40.78	41.01	386165	{}	\N	2020-10-12 14:47:08.802609
39852	EPP	2020-09-29	40.64	40.77	40.5	40.61	561521	{}	\N	2020-10-12 14:47:08.807526
39853	EPP	2020-09-30	40.47	40.81	40.45	40.54	242470	{}	\N	2020-10-12 14:47:08.812212
39854	EPP	2020-10-01	40.93	40.99	40.68	40.94	372597	{}	\N	2020-10-12 14:47:08.816493
39855	EPP	2020-10-02	40.53	40.9176	40.53	40.81	613577	{}	\N	2020-10-12 14:47:08.820361
39856	EPP	2020-10-05	41.18	41.5	41.18	41.5	203418	{}	\N	2020-10-12 14:47:08.824345
39857	EPP	2020-10-06	41.55	41.5986	41.08	41.13	299613	{}	\N	2020-10-12 14:47:08.828425
39858	EPP	2020-10-07	41.74	41.86	41.65	41.73	157997	{}	\N	2020-10-12 14:47:08.832691
39859	EPP	2020-10-08	41.95	42.13	41.87	42.09	282485	{}	\N	2020-10-12 14:47:08.836819
39860	EPP	2020-10-09	42.19	42.32	42.12	42.24	80873	{}	\N	2020-10-12 14:47:08.840761
39861	EPP	2020-10-12	42.39	42.45	42.35	42.43	74173	{}	\N	2020-10-12 14:47:08.844772
39862	EGY	2020-09-28	0.94	1	0.93	0.97	123822	{}	\N	2020-10-12 14:47:09.832709
39863	EGY	2020-09-29	0.97	1.02	0.97	0.99	150170	{}	\N	2020-10-12 14:47:09.841732
39864	EGY	2020-09-30	0.9839	1.04	0.97	1	2385316	{}	\N	2020-10-12 14:47:09.846952
39865	EGY	2020-10-01	1.1	1.14	1.05	1.12	3335230	{}	\N	2020-10-12 14:47:09.85173
39866	EGY	2020-10-02	1.05	1.1	1.02	1.06	435562	{}	\N	2020-10-12 14:47:09.855943
39867	EGY	2020-10-05	1.04	1.05	1.02	1.04	237154	{}	\N	2020-10-12 14:47:09.859746
39868	EGY	2020-10-06	1.02	1.0201	0.98	0.9833	394790	{}	\N	2020-10-12 14:47:09.863754
39869	EGY	2020-10-07	0.99	1.02	0.98	0.99	243080	{}	\N	2020-10-12 14:47:09.868145
39870	EGY	2020-10-08	0.9987	1.03	0.99	1.01	349660	{}	\N	2020-10-12 14:47:09.872163
39871	EGY	2020-10-09	1.14	1.339	1.05	1.09	5101388	{}	\N	2020-10-12 14:47:09.876365
39872	EGY	2020-10-12	1.09	1.1	1.05	1.055	243558	{}	\N	2020-10-12 14:47:09.88038
39873	EIDO	2020-09-28	17.48	17.48	17.29	17.37	311637	{}	\N	2020-10-12 14:47:10.837386
39874	EIDO	2020-09-29	17.23	17.33	17.17	17.26	491277	{}	\N	2020-10-12 14:47:10.844598
39875	EIDO	2020-09-30	17.22	17.415	17.22	17.265	439741	{}	\N	2020-10-12 14:47:10.850188
39876	EIDO	2020-10-01	17.57	17.74	17.515	17.7	568662	{}	\N	2020-10-12 14:47:10.855713
39877	EIDO	2020-10-02	17.35	17.54	17.19	17.41	347828	{}	\N	2020-10-12 14:47:10.86034
39878	EIDO	2020-10-05	17.7362	17.97	17.7362	17.89	626205	{}	\N	2020-10-12 14:47:10.864814
39879	EIDO	2020-10-06	17.91	18.17	17.91	17.99	565845	{}	\N	2020-10-12 14:47:10.869588
39880	EIDO	2020-10-07	18.05	18.1088	18	18.06	266879	{}	\N	2020-10-12 14:47:10.873802
39881	EIDO	2020-10-08	18.13	18.255	18.13	18.21	520885	{}	\N	2020-10-12 14:47:10.8782
39882	EIDO	2020-10-09	18.27	18.395	18.18	18.36	299477	{}	\N	2020-10-12 14:47:10.882348
39883	EIDO	2020-10-12	18.38	18.3899	18.315	18.38	68734	{}	\N	2020-10-12 14:47:10.886644
39884	EIS	2020-09-28	51.47	51.68	51.27	51.5318	11959	{}	\N	2020-10-12 14:47:12.409835
39885	EIS	2020-09-29	51.81	51.815	51.7007	51.815	2421	{}	\N	2020-10-12 14:47:12.416943
39886	EIS	2020-09-30	52.05	52.79	51.82	52.43	26915	{}	\N	2020-10-12 14:47:12.424651
39887	EIS	2020-10-01	52.7	53.085	52.67	52.98	58968	{}	\N	2020-10-12 14:47:12.430702
39888	EIS	2020-10-02	52.71	52.95	52.47	52.61	3771	{}	\N	2020-10-12 14:47:12.435393
39889	EIS	2020-10-05	54.0608	54.5799	54.0608	54.56	3031	{}	\N	2020-10-12 14:47:12.439952
39890	EIS	2020-10-06	54.315	54.55	53.63	53.98	30628	{}	\N	2020-10-12 14:47:12.444334
39891	EIS	2020-10-07	54.445	55.26	54.333	55.26	222579	{}	\N	2020-10-12 14:47:12.448542
39892	EIS	2020-10-08	55.26	55.59	55.1	55.59	6160	{}	\N	2020-10-12 14:47:12.45334
39893	EIS	2020-10-09	55.95	56.488	55.74	55.93	11205	{}	\N	2020-10-12 14:47:12.457699
39894	ELMD	2020-09-28	10.21	10.6	10.17	10.43	71560	{}	\N	2020-10-12 14:47:13.408645
39895	ELMD	2020-09-29	10.445	10.93	10.25	10.5	47320	{}	\N	2020-10-12 14:47:13.420471
39896	ELMD	2020-09-30	10.55	10.71	10.28	10.41	53261	{}	\N	2020-10-12 14:47:13.429014
39897	ELMD	2020-10-01	10.42	10.49	10.07	10.07	75006	{}	\N	2020-10-12 14:47:13.437142
39898	ELMD	2020-10-02	9.44	9.55	8.54	9.11	374798	{}	\N	2020-10-12 14:47:13.444397
39899	ELMD	2020-10-05	9.04	9.25	8.6801	9	199935	{}	\N	2020-10-12 14:47:13.452159
39900	ELMD	2020-10-06	8.95	9.48	8.1	8.92	556734	{}	\N	2020-10-12 14:47:13.459229
39901	ELMD	2020-10-07	8.87	9.24	8.8404	9.18	231827	{}	\N	2020-10-12 14:47:13.46597
39902	ELMD	2020-10-08	9.21	9.3	9.095	9.19	91415	{}	\N	2020-10-12 14:47:13.472166
39903	ELMD	2020-10-09	9.18	9.59	9.18	9.59	124131	{}	\N	2020-10-12 14:47:13.477508
39904	ELMD	2020-10-12	8.7838	8.99	8.5501	8.94	95446	{}	\N	2020-10-12 14:47:13.481924
39905	ELY	2020-09-28	18.62	19.15	18.59	18.78	898690	{}	\N	2020-10-12 14:47:14.471507
39906	ELY	2020-09-29	19.09	19.35	18.55	18.82	1648255	{}	\N	2020-10-12 14:47:14.478114
39907	ELY	2020-09-30	18.77	19.31	18.77	19.14	2853171	{}	\N	2020-10-12 14:47:14.482896
39908	ELY	2020-10-01	19.32	19.78	19.08	19.57	2904930	{}	\N	2020-10-12 14:47:14.486297
39909	ELY	2020-10-02	18.79	19.79	18.3	19.53	1588195	{}	\N	2020-10-12 14:47:14.489763
39910	ELY	2020-10-05	19.72	20.28	19.655	20.04	2708172	{}	\N	2020-10-12 14:47:14.493582
39911	ELY	2020-10-06	20.24	20.62	19.7208	19.88	4142061	{}	\N	2020-10-12 14:47:14.497017
39912	ELY	2020-10-07	20	20.36	19.83	20.12	2865677	{}	\N	2020-10-12 14:47:14.50037
39913	ELY	2020-10-08	20.42	20.58	20.03	20.1	1420526	{}	\N	2020-10-12 14:47:14.503643
39914	ELY	2020-10-09	20.25	20.4	19.89	19.91	2368873	{}	\N	2020-10-12 14:47:14.507869
39915	ELY	2020-10-12	20.03	20.0676	19.46	19.535	344948	{}	\N	2020-10-12 14:47:14.512408
39916	EMTY	2020-09-28	28.31	28.3717	28.08	28.1771	12897	{}	\N	2020-10-12 14:47:16.565136
39917	EMTY	2020-09-29	28.71	28.71	28.45	28.58	4025	{}	\N	2020-10-12 14:47:16.570422
39918	EMTY	2020-09-30	28.45	28.45	27.9001	28.175	5663	{}	\N	2020-10-12 14:47:16.575021
39919	EMTY	2020-10-01	28.07	28.07	27.328	27.4901	21758	{}	\N	2020-10-12 14:47:16.579423
39920	EMTY	2020-10-02	27.97	27.97	27.31	27.33	8618	{}	\N	2020-10-12 14:47:16.584112
39921	EMTY	2020-10-05	27.01	27.066	26.7	26.7472	25680	{}	\N	2020-10-12 14:47:16.588542
39922	EMTY	2020-10-06	26.6128	27.4	26.6128	27.37	12459	{}	\N	2020-10-12 14:47:16.59272
39923	EMTY	2020-10-07	27.01	27.01	26.52	26.64	6734	{}	\N	2020-10-12 14:47:16.597476
39924	EMTY	2020-10-08	26.42	26.53	26.2801	26.3005	19275	{}	\N	2020-10-12 14:47:16.601924
39925	EMTY	2020-10-09	26.035	26.37	26.035	26.37	13127	{}	\N	2020-10-12 14:47:16.606477
39926	EMTY	2020-10-12	26.13	26.1682	26.13	26.1682	427	{}	\N	2020-10-12 14:47:16.611062
39927	ENJ	2020-09-28	25.3	25.5	25.2893	25.5	3406	{}	\N	2020-10-12 14:47:18.361001
39928	ENJ	2020-09-29	25.49	25.5	25.49	25.5	798	{}	\N	2020-10-12 14:47:18.369019
39929	ENJ	2020-09-30	25.29	25.51	25.28	25.3419	11756	{}	\N	2020-10-12 14:47:18.373307
39930	ENJ	2020-10-01	25.3309	25.3309	25.3309	25.3309	2000	{}	\N	2020-10-12 14:47:18.377862
39931	ENJ	2020-10-02	25.4101	25.46	25.37	25.37	2600	{}	\N	2020-10-12 14:47:18.382697
39932	ENJ	2020-10-05	25.3897	25.48	25.378	25.378	2458	{}	\N	2020-10-12 14:47:18.386712
39933	ENJ	2020-10-07	25.46	25.47	25.3425	25.47	2440	{}	\N	2020-10-12 14:47:18.390714
39934	ENJ	2020-10-08	25.24	25.3774	25.24	25.35	3314	{}	\N	2020-10-12 14:47:18.395025
39935	ENJ	2020-10-09	25.34	25.34	25.34	25.34	1000	{}	\N	2020-10-12 14:47:18.399215
39936	ENSV	2020-09-28	0.1743	0.1888	0.1451	0.1556	24564421	{}	\N	2020-10-12 14:47:19.381772
39937	ENSV	2020-09-29	0.1661	0.178	0.1577	0.1722	14859499	{}	\N	2020-10-12 14:47:19.38754
39938	ENSV	2020-09-30	0.1565	0.1596	0.1494	0.1519	4716535	{}	\N	2020-10-12 14:47:19.392067
39939	ENSV	2020-10-01	0.153	0.157	0.1501	0.1505	2492056	{}	\N	2020-10-12 14:47:19.395951
39940	ENSV	2020-10-02	0.16	0.1601	0.15111	0.1561	2966091	{}	\N	2020-10-12 14:47:19.40049
39941	ENSV	2020-10-05	0.1521	0.1682	0.151	0.1545	6194029	{}	\N	2020-10-12 14:47:19.404589
39942	ENSV	2020-10-06	0.152	0.168	0.152	0.158	9348737	{}	\N	2020-10-12 14:47:19.408952
39943	ENSV	2020-10-07	0.165	0.185	0.1565	0.1756	13202655	{}	\N	2020-10-12 14:47:19.412684
39944	ENSV	2020-10-08	0.18	0.1987	0.166	0.1934	21606242	{}	\N	2020-10-12 14:47:19.417624
39945	ENSV	2020-10-09	0.203	0.23	0.168	0.1697	33532620	{}	\N	2020-10-12 14:47:19.422376
39946	ENSV	2020-10-12	0.1705	0.1779	0.1601	0.163	3282874	{}	\N	2020-10-12 14:47:19.42666
39947	ENVA	2020-09-28	15.97	16.48	15.96	16.01	183545	{}	\N	2020-10-12 14:47:20.551498
39948	ENVA	2020-09-29	15.99	16.1725	15.7	15.9	358089	{}	\N	2020-10-12 14:47:20.557777
39949	ENVA	2020-09-30	15.94	16.58	15.94	16.39	583511	{}	\N	2020-10-12 14:47:20.562641
39950	ENVA	2020-10-01	16.42	16.89	15.815	16.86	551633	{}	\N	2020-10-12 14:47:20.567707
39951	ENVA	2020-10-02	16.38	17.5	16.1	17.2	466275	{}	\N	2020-10-12 14:47:20.572481
39952	ENVA	2020-10-05	17.22	17.79	17.22	17.76	574619	{}	\N	2020-10-12 14:47:20.577008
39953	ENVA	2020-10-06	17.76	18.33	17.4	17.42	633516	{}	\N	2020-10-12 14:47:20.581258
39954	ENVA	2020-10-07	17.72	18.08	17.28	17.78	734543	{}	\N	2020-10-12 14:47:20.585639
39955	ENVA	2020-10-08	18	18.91	17.83	18.84	692688	{}	\N	2020-10-12 14:47:20.589974
39956	ENVA	2020-10-09	19.08	20.12	18.84	19.19	1636511	{}	\N	2020-10-12 14:47:20.594882
39957	ENVA	2020-10-12	19.25	19.5199	18.88	19.34	247872	{}	\N	2020-10-12 14:47:20.600074
39958	EOS	2020-09-28	18.21	18.34	18.14	18.33	149116	{}	\N	2020-10-12 14:47:21.636006
39959	EOS	2020-09-29	18.36	18.37	18.19	18.35	232987	{}	\N	2020-10-12 14:47:21.643021
39960	EOS	2020-09-30	18.41	18.59	18.41	18.45	336511	{}	\N	2020-10-12 14:47:21.647566
39961	EOS	2020-10-01	18.59	18.76	18.59	18.72	283373	{}	\N	2020-10-12 14:47:21.652136
39962	EOS	2020-10-02	18.54	18.7142	18.43	18.67	327115	{}	\N	2020-10-12 14:47:21.655827
39963	EOS	2020-10-05	18.74	18.84	18.61	18.81	267000	{}	\N	2020-10-12 14:47:21.659954
39964	EOS	2020-10-06	18.84	19.23	18.8	18.88	494234	{}	\N	2020-10-12 14:47:21.663848
39965	EOS	2020-10-07	18.93	19.1239	18.9	19.03	373760	{}	\N	2020-10-12 14:47:21.668954
39966	EOS	2020-10-08	19.23	19.28	18.68	18.95	478073	{}	\N	2020-10-12 14:47:21.673688
39967	EOS	2020-10-09	18.95	19.14	18.9	19.12	373879	{}	\N	2020-10-12 14:47:21.678131
39968	EOS	2020-10-12	19.29	19.29	19.205	19.265	74070	{}	\N	2020-10-12 14:47:21.682704
39969	EPOL	2020-09-28	16.42	16.48	16.34	16.46	60556	{}	\N	2020-10-12 14:47:22.605482
39970	EPOL	2020-09-29	16.61	16.725	16.59	16.68	81558	{}	\N	2020-10-12 14:47:22.611106
39971	EPOL	2020-09-30	16.48	16.6	16.435	16.46	123475	{}	\N	2020-10-12 14:47:22.616127
39972	EPOL	2020-10-01	16.63	16.73	16.46	16.72	64380	{}	\N	2020-10-12 14:47:22.620733
39973	EPOL	2020-10-02	16.444	16.57	16.34	16.37	770901	{}	\N	2020-10-12 14:47:22.625748
39974	EPOL	2020-10-05	16.85	17.01	16.85	16.99	190885	{}	\N	2020-10-12 14:47:22.631094
39975	EPOL	2020-10-06	17.11	17.11	16.94	16.975	163787	{}	\N	2020-10-12 14:47:22.636461
39976	EPOL	2020-10-07	16.91	17.09	16.86	17.03	39698	{}	\N	2020-10-12 14:47:22.640799
39977	EPOL	2020-10-08	16.85	16.85	16.73	16.835	386567	{}	\N	2020-10-12 14:47:22.645952
39978	EPOL	2020-10-09	16.7	16.8	16.62	16.75	125695	{}	\N	2020-10-12 14:47:22.65057
39979	EPOL	2020-10-12	16.49	16.55	16.48	16.53	54139	{}	\N	2020-10-12 14:47:22.654885
39980	EIGR	2020-09-28	9.47	9.54	7.67	7.96	2222618	{}	\N	2020-10-12 14:47:23.670051
39981	EIGR	2020-09-29	8.0457	8.54	8	8.27	567064	{}	\N	2020-10-12 14:47:23.677437
39982	EIGR	2020-09-30	8.18	8.5	8.07	8.14	395667	{}	\N	2020-10-12 14:47:23.681835
39983	EIGR	2020-10-01	8.14	8.4134	7.925	8.02	944768	{}	\N	2020-10-12 14:47:23.685528
39984	EIGR	2020-10-02	8.07	8.34	7.57	7.98	491404	{}	\N	2020-10-12 14:47:23.689228
39985	EIGR	2020-10-05	8	8.5369	8	8.41	352242	{}	\N	2020-10-12 14:47:23.693358
39986	EIGR	2020-10-06	8.4	8.77	8.25	8.36	342710	{}	\N	2020-10-12 14:47:23.697643
39987	EIGR	2020-10-07	8.47	8.7	8.31	8.47	270170	{}	\N	2020-10-12 14:47:23.701392
39988	EIGR	2020-10-08	8.58	8.66	8.27	8.38	346467	{}	\N	2020-10-12 14:47:23.705273
39989	EIGR	2020-10-09	8.48	8.5291	8.08	8.14	266221	{}	\N	2020-10-12 14:47:23.709323
39990	EIGR	2020-10-12	8.16	8.4499	8.0507	8.25	106904	{}	\N	2020-10-12 14:47:23.713106
39991	ELF	2020-09-28	18.57	18.97	18.46	18.75	451088	{}	\N	2020-10-12 14:47:25.441142
39992	ELF	2020-09-29	18.75	18.91	18.08	18.32	1229947	{}	\N	2020-10-12 14:47:25.448579
39993	ELF	2020-09-30	18.36	18.72	18.125	18.37	950804	{}	\N	2020-10-12 14:47:25.453352
39994	ELF	2020-10-01	18.44	19.2	18.3	18.96	1182738	{}	\N	2020-10-12 14:47:25.458289
39995	ELF	2020-10-02	18.57	18.93	18.38	18.76	830219	{}	\N	2020-10-12 14:47:25.463134
39996	ELF	2020-10-05	18.91	19.71	18.9	19.6	1264973	{}	\N	2020-10-12 14:47:25.467529
39997	ELF	2020-10-06	19.8	20.71	19.8	20.31	1645791	{}	\N	2020-10-12 14:47:25.471735
39998	ELF	2020-10-07	20.41	21.0299	20.0827	20.5	5604287	{}	\N	2020-10-12 14:47:25.47599
39999	ELF	2020-10-08	20.25	21.17	20.18	21.14	1620655	{}	\N	2020-10-12 14:47:25.480295
40000	ELF	2020-10-09	21.38	22.09	20.97	21.64	1103711	{}	\N	2020-10-12 14:47:25.484475
40001	ELF	2020-10-12	21.71	22.14	21.6	21.66	295480	{}	\N	2020-10-12 14:47:25.488691
40002	ELVT	2020-09-28	2.35	2.5	2.31	2.49	529202	{}	\N	2020-10-12 14:47:26.613652
40003	ELVT	2020-09-29	2.51	2.55	2.43	2.51	790099	{}	\N	2020-10-12 14:47:26.623414
40004	ELVT	2020-09-30	2.52	2.59	2.36	2.57	931453	{}	\N	2020-10-12 14:47:26.629943
40005	ELVT	2020-10-01	2.56	2.665	2.55	2.59	799487	{}	\N	2020-10-12 14:47:26.636742
40006	ELVT	2020-10-02	2.54	2.765	2.4	2.71	985636	{}	\N	2020-10-12 14:47:26.651541
40007	ELVT	2020-10-05	2.71	2.75	2.51	2.66	959252	{}	\N	2020-10-12 14:47:26.659358
40008	ELVT	2020-10-06	2.68	2.82	2.59	2.65	1394757	{}	\N	2020-10-12 14:47:26.67497
40009	ELVT	2020-10-07	2.7	2.9978	2.69	2.85	2103722	{}	\N	2020-10-12 14:47:26.689668
40010	ELVT	2020-10-08	2.87	2.995	2.82	2.88	960760	{}	\N	2020-10-12 14:47:26.698804
40011	ELVT	2020-10-09	2.9	3.32	2.88	3.18	1133600	{}	\N	2020-10-12 14:47:26.705552
40012	ELVT	2020-10-12	3.2	3.2198	2.91	2.96	429874	{}	\N	2020-10-12 14:47:26.712163
40013	EMIF	2020-09-28	20.83	20.84	20.83	20.84	271	{}	\N	2020-10-12 14:47:28.111284
40014	EMIF	2020-09-29	20.82	20.82	20.61	20.69	5838	{}	\N	2020-10-12 14:47:28.117544
40015	EMIF	2020-09-30	20.8539	20.8539	20.7	20.71	1453	{}	\N	2020-10-12 14:47:28.123094
40016	EMIF	2020-10-01	20.73	20.8227	20.71	20.8227	2351	{}	\N	2020-10-12 14:47:28.1287
40017	EMIF	2020-10-02	20.7	20.7	20.6	20.6	573	{}	\N	2020-10-12 14:47:28.133966
40018	EMIF	2020-10-05	20.78	20.97	20.78	20.97	71568	{}	\N	2020-10-12 14:47:28.138586
40019	EMIF	2020-10-06	21.0983	21.0983	20.8	20.87	732	{}	\N	2020-10-12 14:47:28.143529
40020	EMIF	2020-10-07	21.01	21.01	20.8701	20.93	1933	{}	\N	2020-10-12 14:47:28.148769
40021	EMIF	2020-10-08	21.02	21.15	21.02	21.15	513	{}	\N	2020-10-12 14:47:28.15316
40022	EMIF	2020-10-09	21.26	21.35	21.2	21.2	2014	{}	\N	2020-10-12 14:47:28.158119
40023	EMIF	2020-10-12	21.28	21.2918	21.2	21.2	518	{}	\N	2020-10-12 14:47:28.163499
40024	EMO	2020-09-28	8.82	9.09	8.69	8.92	47989	{}	\N	2020-10-12 14:47:29.869182
40025	EMO	2020-09-29	8.92	9.18	8.8042	9.08	237886	{}	\N	2020-10-12 14:47:29.876467
40026	EMO	2020-09-30	9.13	9.27	8.735	8.77	164151	{}	\N	2020-10-12 14:47:29.880824
40027	EMO	2020-10-01	8.77	8.8461	8.51	8.62	189973	{}	\N	2020-10-12 14:47:29.885603
40028	EMO	2020-10-02	8.31	8.781	8.31	8.74	141617	{}	\N	2020-10-12 14:47:29.88919
40029	EMO	2020-10-05	8.86	9.31	8.86	9.25	124143	{}	\N	2020-10-12 14:47:29.893772
40030	EMO	2020-10-06	9.35	9.5	9.22	9.28	199761	{}	\N	2020-10-12 14:47:29.89758
40031	EMO	2020-10-07	9.25	9.51	9.24	9.41	107162	{}	\N	2020-10-12 14:47:29.901669
40032	EMO	2020-10-08	9.49	10.03	9.4842	10	164552	{}	\N	2020-10-12 14:47:29.905588
40033	EMO	2020-10-09	10.05	10.15	9.9147	10.09	170692	{}	\N	2020-10-12 14:47:29.909379
40034	EMO	2020-10-12	10.01	10.1	10	10.04	13584	{}	\N	2020-10-12 14:47:29.913741
40035	ENDP	2020-09-28	3.26	3.35	3.12	3.2	4053108	{}	\N	2020-10-12 14:47:31.726387
40036	ENDP	2020-09-29	3.21	3.325	3.155	3.27	2729319	{}	\N	2020-10-12 14:47:31.733785
40037	ENDP	2020-09-30	3.25	3.39	3.21	3.32	2847895	{}	\N	2020-10-12 14:47:31.73848
40038	ENDP	2020-10-01	3.31	3.455	3.31	3.33	3326102	{}	\N	2020-10-12 14:47:31.742994
40039	ENDP	2020-10-02	3.27	3.465	3.2022	3.42	4100313	{}	\N	2020-10-12 14:47:31.74704
40040	ENDP	2020-10-05	3.47	3.76	3.36	3.725	6666911	{}	\N	2020-10-12 14:47:31.751546
40041	ENDP	2020-10-06	3.78	4.46	3.74	4.25	18594053	{}	\N	2020-10-12 14:47:31.756219
40042	ENDP	2020-10-07	4.27	4.585	4.2	4.44	9253824	{}	\N	2020-10-12 14:47:31.761105
40043	ENDP	2020-10-08	4.5	4.96	4.48	4.85	9349292	{}	\N	2020-10-12 14:47:31.765444
40044	ENDP	2020-10-09	4.86	5.16	4.64	4.98	8832697	{}	\N	2020-10-12 14:47:31.769822
40045	ENDP	2020-10-12	4.9	5.01	4.78	4.845	1493820	{}	\N	2020-10-12 14:47:31.7738
40046	ENPH	2020-09-28	75.14	79.73	74.7	78.14	3390200	{}	\N	2020-10-12 14:47:32.730127
40047	ENPH	2020-09-29	78.2	83.37	78.2	80.8	4153627	{}	\N	2020-10-12 14:47:32.736807
40048	ENPH	2020-09-30	81.37	84.72	81.27	82.59	3260306	{}	\N	2020-10-12 14:47:32.741763
40049	ENPH	2020-10-01	85.56	89.52	84.41	89.51	3975589	{}	\N	2020-10-12 14:47:32.746566
40050	ENPH	2020-10-02	85.61	92.57	85.16	90.43	3996441	{}	\N	2020-10-12 14:47:32.751438
40051	ENPH	2020-10-05	93	98.42	92.9	98.05	4208408	{}	\N	2020-10-12 14:47:32.756045
40052	ENPH	2020-10-06	98.06	100.28	95.06	98.58	5441984	{}	\N	2020-10-12 14:47:32.760097
40053	ENPH	2020-10-07	101.5	108.75	100.46	107.82	4971408	{}	\N	2020-10-12 14:47:32.764755
40054	ENPH	2020-10-08	109.6	109.8	102.5638	105.5	6025002	{}	\N	2020-10-12 14:47:32.768849
40055	ENPH	2020-10-09	107.18	111.27	105.29	109.18	2971270	{}	\N	2020-10-12 14:47:32.773894
40056	ENPH	2020-10-12	110.4	110.41	103.55	103.68	1501261	{}	\N	2020-10-12 14:47:32.778162
40057	ENZL	2020-09-28	58.12	58.32	57.8601	58.05	4686	{}	\N	2020-10-12 14:47:34.158712
40058	ENZL	2020-09-29	57.45	57.539	57.1	57.14	5118	{}	\N	2020-10-12 14:47:34.167823
40059	ENZL	2020-09-30	57.56	58.28	57.39	57.54	11217	{}	\N	2020-10-12 14:47:34.174609
40060	ENZL	2020-10-01	57.98	58.72	57.75	58.3	34007	{}	\N	2020-10-12 14:47:34.179616
40061	ENZL	2020-10-02	58.02	58.76	57.78	58.27	18320	{}	\N	2020-10-12 14:47:34.183549
40062	ENZL	2020-10-05	58.86	59.41	58.86	59.41	4946	{}	\N	2020-10-12 14:47:34.187362
40063	ENZL	2020-10-06	59.4	59.5732	58.22	58.31	4666	{}	\N	2020-10-12 14:47:34.191823
40064	ENZL	2020-10-07	59.23	59.8	59.01	59.8	304610	{}	\N	2020-10-12 14:47:34.196076
40065	ENZL	2020-10-08	60.24	60.42	60	60.385	15201	{}	\N	2020-10-12 14:47:34.199819
40066	ENZL	2020-10-09	60.96	61.51	60.96	61.37	23172	{}	\N	2020-10-12 14:47:34.203522
40067	ENZL	2020-10-12	61.77	61.82	61.545	61.545	2495	{}	\N	2020-10-12 14:47:34.207771
40068	EPU	2020-09-28	29.56	29.84	29.21	29.3	18197	{}	\N	2020-10-12 14:47:36.554513
40069	EPU	2020-09-29	29.23	29.39	29.22	29.31	1611	{}	\N	2020-10-12 14:47:36.559408
40070	EPU	2020-09-30	29.44	29.6099	29.44	29.6099	3396	{}	\N	2020-10-12 14:47:36.564621
40071	EPU	2020-10-01	29.5825	29.79	29.3	29.31	15804	{}	\N	2020-10-12 14:47:36.568113
40072	EPU	2020-10-02	28.97	29.52	28.97	29.39	7703	{}	\N	2020-10-12 14:47:36.571915
40073	EPU	2020-10-05	29.8	29.81	29.69	29.69	2974	{}	\N	2020-10-12 14:47:36.5765
40074	EPU	2020-10-06	29.88	29.99	29.43	29.48	15657	{}	\N	2020-10-12 14:47:36.580397
40075	EPU	2020-10-07	29.55	29.6586	29.415	29.415	3344	{}	\N	2020-10-12 14:47:36.584116
40076	EPU	2020-10-08	29.62	29.7347	29.44	29.7347	13447	{}	\N	2020-10-12 14:47:36.587748
40077	EPU	2020-10-09	29.82	30.3	29.82	30	3656	{}	\N	2020-10-12 14:47:36.591511
40078	EPU	2020-10-12	29.9031	29.9031	29.9031	29.9031	271	{}	\N	2020-10-12 14:47:36.595968
40079	ERM	2020-09-28	17.64	17.69	17.6362	17.6362	14544	{}	\N	2020-10-12 14:47:38.311211
40080	ERM	2020-09-29	17.5389	17.5389	17.5389	17.5389	575	{}	\N	2020-10-12 14:47:38.317577
40081	ERM	2020-09-30	17.67	17.68	17.6678	17.6678	1461	{}	\N	2020-10-12 14:47:38.323507
40082	ERM	2020-10-01	17.7	17.71	17.6551	17.6551	6687	{}	\N	2020-10-12 14:47:38.329056
40083	ERM	2020-10-02	17.7393	17.7393	17.7393	17.7393	1	{}	\N	2020-10-12 14:47:38.334083
40084	ERM	2020-10-05	17.72	18.02	17.72	18.02	853	{}	\N	2020-10-12 14:47:38.338386
40085	ERM	2020-10-06	18.09	18.09	17.8396	17.8396	209	{}	\N	2020-10-12 14:47:38.342612
40086	ERM	2020-10-07	18.13	18.1798	18.13	18.1798	165	{}	\N	2020-10-12 14:47:38.347191
40087	ERM	2020-10-08	18.35	18.42	18.35	18.42	11384	{}	\N	2020-10-12 14:47:38.352701
40088	ERM	2020-10-09	18.435	18.435	18.435	18.435	1	{}	\N	2020-10-12 14:47:38.358189
40089	ERM	2020-10-12	18.41	18.43	18.41	18.42	874	{}	\N	2020-10-12 14:47:38.362884
40090	EMSH	2020-09-28	72.65	72.65	72.65	72.65	100	{}	\N	2020-10-12 14:47:41.660927
40091	EMSH	2020-09-29	72.65	72.65	72.65	72.65	0	{}	\N	2020-10-12 14:47:41.672633
40092	EMSH	2020-09-30	72.505	72.505	72.505	72.505	100	{}	\N	2020-10-12 14:47:41.67737
40093	EMSH	2020-10-01	72.59	72.59	72.19	72.5301	2310	{}	\N	2020-10-12 14:47:41.682141
40094	EMSH	2020-10-02	72.54	72.54	72.54	72.54	100	{}	\N	2020-10-12 14:47:41.686583
40095	EMSH	2020-10-05	72.27	72.4288	72.25	72.4288	800	{}	\N	2020-10-12 14:47:41.69138
40096	EMSH	2020-10-06	72.71	72.71	72.475	72.475	461	{}	\N	2020-10-12 14:47:41.696173
40097	EMSH	2020-10-07	72.61	72.61	72.61	72.61	108	{}	\N	2020-10-12 14:47:41.700989
40098	EMSH	2020-10-08	72.6	72.6	72.515	72.515	315	{}	\N	2020-10-12 14:47:41.705402
40099	EMSH	2020-10-09	72.625	72.625	72.625	72.625	127	{}	\N	2020-10-12 14:47:41.709705
40151	ESCA	2020-10-08	20.12	20.1998	19.6511	20.03	28431	{}	\N	2020-10-12 14:47:51.00418
40152	ESCA	2020-10-09	19.99	20.08	19.649	19.92	28090	{}	\N	2020-10-12 14:47:51.008946
40153	ESCA	2020-10-12	19.96	20.1	19.502	19.502	10589	{}	\N	2020-10-12 14:47:51.013893
40154	ESGG	2020-09-28	109.41	109.6178	109.41	109.6178	660	{}	\N	2020-10-12 14:47:52.171842
40155	ESGG	2020-09-29	109.325	109.325	109.2387	109.2387	298	{}	\N	2020-10-12 14:47:52.179017
40156	ESGG	2020-09-30	109.93	109.93	109.841	109.841	275	{}	\N	2020-10-12 14:47:52.184105
40157	ESGG	2020-10-01	110.1609	110.1609	110.1609	110.1609	463	{}	\N	2020-10-12 14:47:52.189557
40158	ESGG	2020-10-02	109.64	109.64	109.3054	109.3054	1434	{}	\N	2020-10-12 14:47:52.194112
40159	ESGG	2020-10-05	111.039	111.039	111.039	111.039	663	{}	\N	2020-10-12 14:47:52.198785
40160	ESGG	2020-10-06	111.02	111.14	109.65	109.65	1505	{}	\N	2020-10-12 14:47:52.20367
40161	ESGG	2020-10-07	110.94	111.1975	110.87	111.1975	1021	{}	\N	2020-10-12 14:47:52.208435
40162	ESGG	2020-10-08	112.07	112.13	111.911	112.078	1741	{}	\N	2020-10-12 14:47:52.212792
40163	ESGG	2020-10-09	113.14	113.15	113.0799	113.0799	3565	{}	\N	2020-10-12 14:47:52.217147
40164	KFRC	2020-09-28	32.66	33.19	32.61	32.78	80537	{}	\N	2020-10-12 14:47:53.444186
40165	KFRC	2020-09-29	32.77	32.77	32.02	32.45	74714	{}	\N	2020-10-12 14:47:53.450688
40166	KFRC	2020-09-30	32.66	33.07	32.08	32.17	100319	{}	\N	2020-10-12 14:47:53.45532
40167	KFRC	2020-10-01	32.18	32.33	31.84	32.17	82700	{}	\N	2020-10-12 14:47:53.459828
40168	KFRC	2020-10-02	31.7	32.92	31.7	32.77	64212	{}	\N	2020-10-12 14:47:53.464559
40169	KFRC	2020-10-05	33	34.45	33	34.08	141474	{}	\N	2020-10-12 14:47:53.469402
40170	KFRC	2020-10-06	34.47	34.76	33.9406	34.25	66423	{}	\N	2020-10-12 14:47:53.474068
40171	KFRC	2020-10-07	34.63	35.63	34.63	35.62	102653	{}	\N	2020-10-12 14:47:53.47903
40172	KFRC	2020-10-08	35.97	36.19	35.335	35.89	139880	{}	\N	2020-10-12 14:47:53.483405
40173	KFRC	2020-10-09	36.06	36.41	35.87	36.31	81762	{}	\N	2020-10-12 14:47:53.487804
40174	KFRC	2020-10-12	36.68	37.39	36.37	37.33	17057	{}	\N	2020-10-12 14:47:53.492077
40175	EMFM	2020-09-28	16.73	16.73	16.565	16.565	292	{}	\N	2020-10-12 14:47:54.985081
40176	EMFM	2020-09-29	16.4801	16.4974	16.4801	16.4974	1901	{}	\N	2020-10-12 14:47:54.992241
40177	EMFM	2020-09-30	16.5843	16.5843	16.5843	16.5843	155	{}	\N	2020-10-12 14:47:54.997063
40178	EMFM	2020-10-01	16.67	16.67	16.662	16.662	297	{}	\N	2020-10-12 14:47:55.001108
40179	EMFM	2020-10-02	16.6001	16.6001	16.54	16.54	1653	{}	\N	2020-10-12 14:47:55.005275
40180	EMFM	2020-10-05	16.809	16.809	16.7498	16.76	2710	{}	\N	2020-10-12 14:47:55.009331
40181	EMFM	2020-10-06	16.75	16.75	16.75	16.75	171	{}	\N	2020-10-12 14:47:55.013206
40182	EMFM	2020-10-07	16.78	16.7999	16.78	16.785	998	{}	\N	2020-10-12 14:47:55.017126
40183	EMFM	2020-10-08	16.95	16.98	16.91	16.91	3909	{}	\N	2020-10-12 14:47:55.02097
40184	EMFM	2020-10-09	16.99	17.0682	16.98	17.0682	3277	{}	\N	2020-10-12 14:47:55.025392
40185	EML	2020-09-28	19.78	19.99	19.5801	19.6	3622	{}	\N	2020-10-12 14:47:55.973368
40186	EML	2020-09-29	20	20	19.51	19.56	2970	{}	\N	2020-10-12 14:47:55.981381
40187	EML	2020-09-30	19.95	19.95	19.52	19.52	1590	{}	\N	2020-10-12 14:47:55.986857
40188	EML	2020-10-01	19.3411	19.67	19.26	19.36	9090	{}	\N	2020-10-12 14:47:55.991933
40189	EML	2020-10-02	19.44	20.79	19.44	19.86	4145	{}	\N	2020-10-12 14:47:55.996391
40190	EML	2020-10-05	20.3	21.1	20.3	21.1	4523	{}	\N	2020-10-12 14:47:56.000533
40191	EML	2020-10-06	21.12	21.2	20.8875	21.17	4547	{}	\N	2020-10-12 14:47:56.004594
40192	EML	2020-10-07	22.085	22.085	21.135	21.85	3761	{}	\N	2020-10-12 14:47:56.008685
40193	EML	2020-10-08	21.89	21.89	21.42	21.42	2732	{}	\N	2020-10-12 14:47:56.012805
40194	EML	2020-10-09	21.675	21.95	21.56	21.56	1956	{}	\N	2020-10-12 14:47:56.016803
40195	EML	2020-10-12	21.43	22.135	21.43	22.13	1591	{}	\N	2020-10-12 14:47:56.020899
40196	EMQQ	2020-09-28	51.82	51.91	51.25	51.71	145646	{}	\N	2020-10-12 14:47:56.949584
40197	EMQQ	2020-09-29	51.55	51.63	51.17	51.29	182676	{}	\N	2020-10-12 14:47:56.955312
40198	EMQQ	2020-09-30	51.69	52.5109	51.69	52.23	365614	{}	\N	2020-10-12 14:47:56.959956
40199	EMQQ	2020-10-01	52.77	52.95	52.45	52.91	339120	{}	\N	2020-10-12 14:47:56.96398
40200	EMQQ	2020-10-02	51.86	52.69	51.86	51.91	204724	{}	\N	2020-10-12 14:47:56.968049
40201	EMQQ	2020-10-05	52.25	52.53	52.165	52.51	102729	{}	\N	2020-10-12 14:47:56.972145
40202	EMQQ	2020-10-06	52.94	53.635	52.81	52.91	231686	{}	\N	2020-10-12 14:47:56.97679
40203	EMQQ	2020-10-07	53.63	53.8299	53.33	53.69	202740	{}	\N	2020-10-12 14:47:56.980832
40204	EMQQ	2020-10-08	53.84	53.9	53.5457	53.75	259252	{}	\N	2020-10-12 14:47:56.984986
40205	EMQQ	2020-10-09	54.14	54.95	54.04	54.94	351873	{}	\N	2020-10-12 14:47:56.990114
40206	EMQQ	2020-10-12	55.535	55.535	55.24	55.505	136763	{}	\N	2020-10-12 14:47:56.994651
40207	ENIC	2020-09-28	3.4	3.47	3.38	3.4	180584	{}	\N	2020-10-12 14:47:58.486765
40208	ENIC	2020-09-29	3.4	3.41	3.31	3.31	421690	{}	\N	2020-10-12 14:47:58.493825
40209	ENIC	2020-09-30	3.31	3.46	3.31	3.44	1356166	{}	\N	2020-10-12 14:47:58.499305
40210	ENIC	2020-10-01	3.46	3.48	3.41	3.44	311818	{}	\N	2020-10-12 14:47:58.503763
40211	ENIC	2020-10-02	3.4	3.49	3.4	3.41	328417	{}	\N	2020-10-12 14:47:58.508114
40212	ENIC	2020-10-05	3.43	3.465	3.35	3.42	557963	{}	\N	2020-10-12 14:47:58.512824
40213	ENIC	2020-10-06	3.4	3.42	3.325	3.33	385679	{}	\N	2020-10-12 14:47:58.517064
40214	ENIC	2020-10-07	3.39	3.39	3.2991	3.32	404806	{}	\N	2020-10-12 14:47:58.52151
40215	ENIC	2020-10-08	3.31	3.395	3.31	3.37	481284	{}	\N	2020-10-12 14:47:58.525939
40216	ENIC	2020-10-09	3.37	3.46	3.37	3.44	481217	{}	\N	2020-10-12 14:47:58.530995
40217	ENIC	2020-10-12	3.46	3.5	3.441	3.465	84144	{}	\N	2020-10-12 14:47:58.535392
40218	ENV	2020-09-28	77.17	78.98	77.17	78.7	227136	{}	\N	2020-10-12 14:48:00.276596
40219	ENV	2020-09-29	78.85	79.01	76.96	77.25	597790	{}	\N	2020-10-12 14:48:00.283408
40220	ENV	2020-09-30	77.38	78.33	76.34	77.16	350332	{}	\N	2020-10-12 14:48:00.288431
40221	ENV	2020-10-01	77.98	78.78	77.33	77.59	708395	{}	\N	2020-10-12 14:48:00.293518
40222	ENV	2020-10-02	76.21	77.94	76.21	77.12	376645	{}	\N	2020-10-12 14:48:00.298195
40223	ENV	2020-10-05	77.58	78.95	77.58	78.87	331998	{}	\N	2020-10-12 14:48:00.302424
40224	ENV	2020-10-06	79.74	80.52	78.27	78.36	411018	{}	\N	2020-10-12 14:48:00.306296
40225	ENV	2020-10-07	79.13	80.75	79.13	80.56	411237	{}	\N	2020-10-12 14:48:00.310315
40226	ENV	2020-10-08	81.13	84.49	80.985	83.86	948146	{}	\N	2020-10-12 14:48:00.314522
40227	ENV	2020-10-09	84.36	85.38	82.99	84.03	572470	{}	\N	2020-10-12 14:48:00.318724
40228	ENV	2020-10-12	84.56	84.56	83.53	83.85	98833	{}	\N	2020-10-12 14:48:00.323716
40229	EOG	2020-09-28	38.37	38.37	37.26	37.44	4179588	{}	\N	2020-10-12 14:48:02.819954
40230	EOG	2020-09-29	37.24	37.47	34.9	36.19	13653551	{}	\N	2020-10-12 14:48:02.825824
40231	EOG	2020-09-30	36.44	36.76	35.8	35.94	11616473	{}	\N	2020-10-12 14:48:02.830175
40232	EOG	2020-10-01	35.33	35.82	34.62	34.86	9768626	{}	\N	2020-10-12 14:48:02.834065
40233	EOG	2020-10-02	33.79	35.22	33.63	34.81	10299197	{}	\N	2020-10-12 14:48:02.838208
40234	EOG	2020-10-05	35.46	36.49	35.07	36.46	8190313	{}	\N	2020-10-12 14:48:02.842028
40235	EOG	2020-10-06	37.18	37.72	35.73	35.99	11928552	{}	\N	2020-10-12 14:48:02.845693
40236	EOG	2020-10-07	36	36.87	35.71	36.71	8849484	{}	\N	2020-10-12 14:48:02.849553
40237	EOG	2020-10-08	37.11	37.87	36.6706	37.67	7355668	{}	\N	2020-10-12 14:48:02.853631
40238	EOG	2020-10-09	37.98	38.17	37.175	37.56	6593257	{}	\N	2020-10-12 14:48:02.857708
40239	EOG	2020-10-12	37.51	37.68	36.66	37.59	1643548	{}	\N	2020-10-12 14:48:02.861434
40240	EPM	2020-09-28	2.29	2.5	2.29	2.46	96011	{}	\N	2020-10-12 14:48:03.792175
40241	EPM	2020-09-29	2.4	2.4099	2.27	2.29	120729	{}	\N	2020-10-12 14:48:03.799658
40242	EPM	2020-09-30	2.345	2.36	2.24	2.24	72424	{}	\N	2020-10-12 14:48:03.804784
40243	EPM	2020-10-01	2.27	2.32	2.22	2.29	105842	{}	\N	2020-10-12 14:48:03.809821
40244	EPM	2020-10-02	2.25	2.35	2.24	2.28	66332	{}	\N	2020-10-12 14:48:03.813976
40245	EPM	2020-10-05	2.2994	2.39	2.2966	2.32	72395	{}	\N	2020-10-12 14:48:03.817869
40246	EPM	2020-10-06	2.4	2.4	2.17	2.2	83366	{}	\N	2020-10-12 14:48:03.822679
40247	EPM	2020-10-07	2.19	2.27	2.17	2.19	105847	{}	\N	2020-10-12 14:48:03.826866
40248	EPM	2020-10-08	2.23	2.3	2.18	2.27	118210	{}	\N	2020-10-12 14:48:03.830602
40249	EPM	2020-10-09	2.29	2.31	2.19	2.2	140438	{}	\N	2020-10-12 14:48:03.834492
40250	EPM	2020-10-12	2.2	2.25	2.19	2.25	12848	{}	\N	2020-10-12 14:48:03.839031
40251	EQC	2020-09-28	29.99	30.29	29.68	30.24	808399	{}	\N	2020-10-12 14:48:05.019441
40252	EQC	2020-09-29	30.25	30.3099	29.67	29.87	1486903	{}	\N	2020-10-12 14:48:05.029929
40253	EQC	2020-09-30	26.52	27.46	26.41	26.63	3826707	{}	\N	2020-10-12 14:48:05.035128
40254	EQC	2020-10-01	26.72	27.22	26.72	27.21	2252725	{}	\N	2020-10-12 14:48:05.039825
40255	EQC	2020-10-02	27	27.385	26.88	27.28	1898117	{}	\N	2020-10-12 14:48:05.044881
40256	EQC	2020-10-05	27.39	27.49	27.15	27.42	1614976	{}	\N	2020-10-12 14:48:05.050764
40257	EQC	2020-10-06	27.3	27.58	27.1	27.36	1547096	{}	\N	2020-10-12 14:48:05.05506
40258	EQC	2020-10-07	27.31	27.38	27.11	27.12	1910806	{}	\N	2020-10-12 14:48:05.059643
40259	EQC	2020-10-08	27.26	27.38	27.13	27.31	1940842	{}	\N	2020-10-12 14:48:05.063793
40260	EQC	2020-10-09	27.43	27.73	27.28	27.49	1926903	{}	\N	2020-10-12 14:48:05.067752
40261	EQC	2020-10-12	27.59	27.65	27.35	27.465	160911	{}	\N	2020-10-12 14:48:05.071926
40262	ERUS	2020-09-28	32.35	32.35	31.97	32.21	214043	{}	\N	2020-10-12 14:48:06.550675
40263	ERUS	2020-09-29	32.06	32.22	31.73	32.08	51151	{}	\N	2020-10-12 14:48:06.557834
40264	ERUS	2020-09-30	32.18	32.6	32.18	32.46	63646	{}	\N	2020-10-12 14:48:06.563346
40265	ERUS	2020-10-01	32.46	32.48	32.21	32.34	112516	{}	\N	2020-10-12 14:48:06.568236
40266	ERUS	2020-10-02	31.89	32.06	31.82	31.95	128072	{}	\N	2020-10-12 14:48:06.573609
40267	ERUS	2020-10-05	32.05	32.41	32.05	32.41	197704	{}	\N	2020-10-12 14:48:06.578644
40268	ERUS	2020-10-06	32.71	32.805	32.31	32.44	80175	{}	\N	2020-10-12 14:48:06.583226
40269	ERUS	2020-10-07	32.1	32.22	31.93	32.09	229753	{}	\N	2020-10-12 14:48:06.587333
40270	ERUS	2020-10-08	32.34	32.5	32.24	32.39	100949	{}	\N	2020-10-12 14:48:06.591851
40271	ERUS	2020-10-09	32.48	32.65	32.45	32.48	131400	{}	\N	2020-10-12 14:48:06.596412
40272	ERUS	2020-10-12	32.65	32.68	32.5456	32.62	29943	{}	\N	2020-10-12 14:48:06.600643
40273	ESG	2020-09-28	80.7236	80.95	80.5	80.9	3273	{}	\N	2020-10-12 14:48:07.783291
40274	ESG	2020-09-29	81.06	81.06	80.5586	80.665	1648	{}	\N	2020-10-12 14:48:07.789626
40275	ESG	2020-09-30	80.46	81.4465	80.46	81.4465	385	{}	\N	2020-10-12 14:48:07.795281
40276	ESG	2020-10-01	82.02	82.02	81.3673	81.6677	19219	{}	\N	2020-10-12 14:48:07.799874
40277	ESG	2020-10-02	81.0479	81.27	80.8938	80.8938	886	{}	\N	2020-10-12 14:48:07.804768
40278	ESG	2020-10-05	81.67	82.2055	81.67	82.2055	6689	{}	\N	2020-10-12 14:48:07.809535
40279	ESG	2020-10-06	82.09	82.36	81.148	81.148	5779	{}	\N	2020-10-12 14:48:07.814191
40280	ESG	2020-10-07	81.95	82.78	81.95	82.78	5090	{}	\N	2020-10-12 14:48:07.818486
40281	ESG	2020-10-08	83.1989	83.24	83.015	83.19	1700	{}	\N	2020-10-12 14:48:07.823118
40282	ESG	2020-10-09	83.61	84.05	83.61	83.87	11691	{}	\N	2020-10-12 14:48:07.82788
40283	EMXC	2020-09-28	45.87	46.01	45.7954	45.7954	7215	{}	\N	2020-10-12 14:48:08.850632
40284	EMXC	2020-09-29	45.7	45.96	45.7	45.85	5596	{}	\N	2020-10-12 14:48:08.8571
40285	EMXC	2020-09-30	46.03	46.41	46.03	46.27	3873	{}	\N	2020-10-12 14:48:08.863071
40286	EMXC	2020-10-01	46.59	46.72	46.4	46.72	9912	{}	\N	2020-10-12 14:48:08.867714
40287	EMXC	2020-10-02	45.99	46.36	45.99	46.2197	4387	{}	\N	2020-10-12 14:48:08.87263
40288	EMXC	2020-10-05	46.7389	47.04	46.665	47.04	12260	{}	\N	2020-10-12 14:48:08.877143
40289	EMXC	2020-10-06	47.1175	47.1883	46.7439	46.78	5503	{}	\N	2020-10-12 14:48:08.881621
40290	EMXC	2020-10-07	47.19	47.48	47.19	47.4283	5256	{}	\N	2020-10-12 14:48:08.885917
40291	EMXC	2020-10-08	47.51	47.73	47.51	47.73	6871	{}	\N	2020-10-12 14:48:08.890147
40292	EMXC	2020-10-09	48.0299	48.2488	47.925	48.2	8167	{}	\N	2020-10-12 14:48:08.894917
40293	EMXC	2020-10-12	48.16	48.16	48.01	48.0857	872	{}	\N	2020-10-12 14:48:08.899385
40294	ENLC	2020-09-28	2.4	2.435	2.25	2.28	3875379	{}	\N	2020-10-12 14:48:09.99223
40295	ENLC	2020-09-29	2.29	2.37	2.22	2.35	4237543	{}	\N	2020-10-12 14:48:10.00115
40296	ENLC	2020-09-30	2.37	2.44	2.32	2.35	8191410	{}	\N	2020-10-12 14:48:10.007233
40297	ENLC	2020-10-01	2.37	2.39	2.29	2.31	5003434	{}	\N	2020-10-12 14:48:10.012465
40298	ENLC	2020-10-02	2.2	2.425	2.2	2.39	3071563	{}	\N	2020-10-12 14:48:10.017092
40299	ENLC	2020-10-05	2.41	2.5	2.38	2.45	4348695	{}	\N	2020-10-12 14:48:10.021897
40300	ENLC	2020-10-06	2.52	2.58	2.355	2.41	3472726	{}	\N	2020-10-12 14:48:10.026446
40301	ENLC	2020-10-07	2.44	2.455	2.34	2.38	4273350	{}	\N	2020-10-12 14:48:10.031876
40302	ENLC	2020-10-08	2.4	2.72	2.4	2.71	6131869	{}	\N	2020-10-12 14:48:10.036938
40303	ENLC	2020-10-09	2.72	2.75	2.6	2.68	4269091	{}	\N	2020-10-12 14:48:10.041741
40304	ENLC	2020-10-12	2.68	2.68	2.6	2.615	555803	{}	\N	2020-10-12 14:48:10.045702
40305	ENTA	2020-09-28	43.43	44.29	43.27	44.15	142474	{}	\N	2020-10-12 14:48:11.10141
40306	ENTA	2020-09-29	44.24	45.06	43.88	44.83	115380	{}	\N	2020-10-12 14:48:11.109287
40307	ENTA	2020-09-30	45.22	45.94	44.8925	45.78	175705	{}	\N	2020-10-12 14:48:11.114865
40308	ENTA	2020-10-01	46.02	46.67	45.02	46.35	170884	{}	\N	2020-10-12 14:48:11.119813
40309	ENTA	2020-10-02	45.9	46.255	44.495	45.41	175576	{}	\N	2020-10-12 14:48:11.124623
40310	ENTA	2020-10-05	45.78	47.195	43.545	46.91	98345	{}	\N	2020-10-12 14:48:11.128602
40311	ENTA	2020-10-06	47.42	47.6	46.49	47.04	165039	{}	\N	2020-10-12 14:48:11.133061
40312	ENTA	2020-10-07	47.24	47.27	46.49	46.73	134866	{}	\N	2020-10-12 14:48:11.137079
40313	ENTA	2020-10-08	46.6	46.99	45.09	46.61	110668	{}	\N	2020-10-12 14:48:11.141324
40314	ENTA	2020-10-09	47.21	47.425	45.78	46.16	85475	{}	\N	2020-10-12 14:48:11.145415
40315	ENTA	2020-10-12	46.13	46.19	45.61	45.905	9098	{}	\N	2020-10-12 14:48:11.149502
40316	EOI	2020-09-28	14.55	14.64	14.5065	14.56	94830	{}	\N	2020-10-12 14:48:12.427687
40317	EOI	2020-09-29	14.56	14.57	14.44	14.55	144622	{}	\N	2020-10-12 14:48:12.432871
40318	EOI	2020-09-30	14.66	14.73	14.54	14.64	349608	{}	\N	2020-10-12 14:48:12.437501
40319	EOI	2020-10-01	14.8	14.82	14.69	14.7	141072	{}	\N	2020-10-12 14:48:12.441586
40320	EOI	2020-10-02	14.61	14.69	14.53	14.69	156645	{}	\N	2020-10-12 14:48:12.445656
40321	EOI	2020-10-05	14.73	14.84	14.69	14.82	173794	{}	\N	2020-10-12 14:48:12.449688
40322	EOI	2020-10-06	14.86	14.97	14.6	14.69	186635	{}	\N	2020-10-12 14:48:12.453676
40323	EOI	2020-10-07	14.82	14.8895	14.72	14.87	167647	{}	\N	2020-10-12 14:48:12.457538
40324	EOI	2020-10-08	14.96	14.98	14.79	14.87	155177	{}	\N	2020-10-12 14:48:12.461596
40325	EOI	2020-10-09	14.97	15.02	14.88	15.01	244731	{}	\N	2020-10-12 14:48:12.46593
40326	EOI	2020-10-12	15.07	15.105	15.07	15.0908	24936	{}	\N	2020-10-12 14:48:12.469891
40327	EPIX	2020-09-28	6.86	7.08	6.5395	7.03	188599	{}	\N	2020-10-12 14:48:13.885705
40328	EPIX	2020-09-29	7.03	7.1	6.93	7	81996	{}	\N	2020-10-12 14:48:13.891976
40329	EPIX	2020-09-30	7	7.15	6.61	6.86	538974	{}	\N	2020-10-12 14:48:13.89661
40330	EPIX	2020-10-01	7	7.19	6.76	6.94	77302	{}	\N	2020-10-12 14:48:13.90066
40331	EPIX	2020-10-02	6.86	7.08	6.28	6.52	67153	{}	\N	2020-10-12 14:48:13.90461
40332	EPIX	2020-10-05	6.54	6.88	6.38	6.7	57860	{}	\N	2020-10-12 14:48:13.908456
40333	EPIX	2020-10-06	6.42	6.88	6.42	6.75	25309	{}	\N	2020-10-12 14:48:13.912291
40334	EPIX	2020-10-07	6.76	6.82	6.2	6.43	287747	{}	\N	2020-10-12 14:48:13.916274
40335	EPIX	2020-10-08	6.42	6.64	6.145	6.34	147848	{}	\N	2020-10-12 14:48:13.921477
40336	EPIX	2020-10-09	6.32	6.48	6.21	6.21	21116	{}	\N	2020-10-12 14:48:13.926264
40337	EPIX	2020-10-12	6.32	6.6	6.32	6.6	1314	{}	\N	2020-10-12 14:48:13.930112
40338	EQBK	2020-09-28	14.27	15.17	14.27	15.09	41923	{}	\N	2020-10-12 14:48:15.763107
40339	EQBK	2020-09-29	14.9	15.32	14.75	15.27	30772	{}	\N	2020-10-12 14:48:15.768483
40340	EQBK	2020-09-30	15.21	15.635	15.21	15.52	32909	{}	\N	2020-10-12 14:48:15.773511
40341	EQBK	2020-10-01	15.5	15.61	15.2	15.59	119131	{}	\N	2020-10-12 14:48:15.777828
40342	EQBK	2020-10-02	15.46	16.1	15.3	15.85	25163	{}	\N	2020-10-12 14:48:15.782183
40343	EQBK	2020-10-05	15.83	16.395	15.83	16.21	50836	{}	\N	2020-10-12 14:48:15.786542
40344	EQBK	2020-10-06	16.42	16.485	15.7	16.17	72015	{}	\N	2020-10-12 14:48:15.791259
40345	EQBK	2020-10-07	16.25	16.77	16.25	16.7	29251	{}	\N	2020-10-12 14:48:15.795396
40346	EQBK	2020-10-08	16.93	17.05	16.41	16.95	30763	{}	\N	2020-10-12 14:48:15.799577
40347	EQBK	2020-10-09	17	17	16.52	16.53	47740	{}	\N	2020-10-12 14:48:15.80356
40348	EQBK	2020-10-12	16.36	16.55	16.26	16.54	3073	{}	\N	2020-10-12 14:48:15.807926
40349	EQS	2020-09-28	1.16	1.2452	1.16	1.19	8847	{}	\N	2020-10-12 14:48:17.149821
40350	EQS	2020-09-29	1.159	1.18	1.15	1.18	39440	{}	\N	2020-10-12 14:48:17.15699
40351	EQS	2020-09-30	1.2	1.22	1.17	1.21	6936	{}	\N	2020-10-12 14:48:17.162944
40352	EQS	2020-10-01	1.1188	1.24	1.1188	1.24	2542	{}	\N	2020-10-12 14:48:17.169265
40353	EQS	2020-10-02	1.1961	1.25	1.14	1.23	11618	{}	\N	2020-10-12 14:48:17.175401
40354	EQS	2020-10-05	1.1708	1.24	1.1708	1.24	3150	{}	\N	2020-10-12 14:48:17.180162
40355	EQS	2020-10-06	1.2499	1.2499	1.2499	1.2499	456	{}	\N	2020-10-12 14:48:17.185389
40356	EQS	2020-10-07	1.125	1.15	1.125	1.15	16208	{}	\N	2020-10-12 14:48:17.197249
40357	EQS	2020-10-08	1.15	1.21	1.149	1.17	6898	{}	\N	2020-10-12 14:48:17.204209
40358	EQS	2020-10-09	1.1406	1.17	1.135	1.17	9296	{}	\N	2020-10-12 14:48:17.210497
40359	EQS	2020-10-12	1.2	1.21	1.16	1.18	2820	{}	\N	2020-10-12 14:48:17.216121
40360	ERJ	2020-09-28	4.46	4.68	4.43	4.44	2987083	{}	\N	2020-10-12 14:48:18.411705
40361	ERJ	2020-09-29	4.48	4.5	4.29	4.29	3806513	{}	\N	2020-10-12 14:48:18.41847
40362	ERJ	2020-09-30	4.3	4.46	4.3	4.41	3447285	{}	\N	2020-10-12 14:48:18.425091
40363	ERJ	2020-10-01	4.38	4.53	4.36	4.52	3514810	{}	\N	2020-10-12 14:48:18.430043
40364	ERJ	2020-10-02	4.45	4.6	4.421	4.56	4127925	{}	\N	2020-10-12 14:48:18.437714
40365	ERJ	2020-10-05	4.6	4.675	4.505	4.56	2565503	{}	\N	2020-10-12 14:48:18.444431
40366	ERJ	2020-10-06	4.82	4.97	4.64	4.7	6728940	{}	\N	2020-10-12 14:48:18.449617
40367	ERJ	2020-10-07	4.64	4.73	4.47	4.47	5186797	{}	\N	2020-10-12 14:48:18.454188
40368	ERJ	2020-10-08	4.48	4.75	4.4699	4.74	3873086	{}	\N	2020-10-12 14:48:18.458744
40369	ERJ	2020-10-09	4.77	4.89	4.67	4.8	3566328	{}	\N	2020-10-12 14:48:18.462847
40370	ERJ	2020-10-12	4.84	4.84	4.7011	4.73	1110659	{}	\N	2020-10-12 14:48:18.466815
40371	ESBA	2020-09-28	6.2	6.3	6.14	6.19	17783	{}	\N	2020-10-12 14:48:19.765379
40372	ESBA	2020-09-29	6.1104	6.1104	5.988	6.05	4547	{}	\N	2020-10-12 14:48:19.77359
40373	ESBA	2020-09-30	6.2073	6.22	6.0193	6.06	17119	{}	\N	2020-10-12 14:48:19.779484
40374	ESBA	2020-10-01	6.11	6.3	6.11	6.27	11013	{}	\N	2020-10-12 14:48:19.785086
40375	ESBA	2020-10-02	6.25	6.51	6.24	6.43	24091	{}	\N	2020-10-12 14:48:19.79051
40376	ESBA	2020-10-05	6.68	6.68	6.3	6.31	9184	{}	\N	2020-10-12 14:48:19.794982
40377	ESBA	2020-10-06	6.56	6.66	6.47	6.5	4721	{}	\N	2020-10-12 14:48:19.799527
40378	ESBA	2020-10-07	6.57	6.62	6.5	6.62	1330	{}	\N	2020-10-12 14:48:19.80395
40379	ESBA	2020-10-08	6.75	6.75	6.67	6.69	4349	{}	\N	2020-10-12 14:48:19.808263
40380	ESBA	2020-10-09	6.66	6.69	6.63	6.63	1105	{}	\N	2020-10-12 14:48:19.814606
40381	ESBA	2020-10-12	6.69	6.69	6.69	6.69	201	{}	\N	2020-10-12 14:48:19.818906
40382	ESGE	2020-09-28	35.19	35.21	34.95	35.09	470147	{}	\N	2020-10-12 14:48:20.995656
40383	ESGE	2020-09-29	35.03	35.1899	34.973	35.06	445190	{}	\N	2020-10-12 14:48:21.00152
40384	ESGE	2020-09-30	35.35	35.75	35.33	35.61	404685	{}	\N	2020-10-12 14:48:21.006611
40385	ESGE	2020-10-01	35.91	36.02	35.705	36	3379561	{}	\N	2020-10-12 14:48:21.011712
40386	ESGE	2020-10-02	35.52	35.87	35.52	35.59	942565	{}	\N	2020-10-12 14:48:21.018033
40387	ESGE	2020-10-05	35.8	36.05	35.8	36.05	423069	{}	\N	2020-10-12 14:48:21.024352
40388	ESGE	2020-10-06	36.19	36.37	35.97	36.07	1049085	{}	\N	2020-10-12 14:48:21.029455
40389	ESGE	2020-10-07	36.49	36.62	36.342	36.52	15201884	{}	\N	2020-10-12 14:48:21.034296
40390	ESGE	2020-10-08	36.69	36.84	36.5699	36.84	2424259	{}	\N	2020-10-12 14:48:21.038953
40391	ESGE	2020-10-09	36.91	37.1699	36.825	37.1	2042221	{}	\N	2020-10-12 14:48:21.043513
40392	ESGE	2020-10-12	37.32	37.36	37.22	37.36	90890	{}	\N	2020-10-12 14:48:21.048559
40393	ESS	2020-09-28	208.07	209.57	205.29	207.52	402187	{}	\N	2020-10-12 14:48:22.756488
40394	ESS	2020-09-29	205	206.33	198.985	199.6	640669	{}	\N	2020-10-12 14:48:22.763236
40395	ESS	2020-09-30	201.58	203.04	198.41	200.79	852762	{}	\N	2020-10-12 14:48:22.767946
40396	ESS	2020-10-01	201.03	206.99	201	206.82	829794	{}	\N	2020-10-12 14:48:22.772851
40397	ESS	2020-10-02	203.84	213.56	200.67	212.1	1423387	{}	\N	2020-10-12 14:48:22.777949
40398	ESS	2020-10-05	213.89	214.84	208.82	213.4	1335557	{}	\N	2020-10-12 14:48:22.782578
40399	ESS	2020-10-06	214.73	214.73	209.44	210.77	970751	{}	\N	2020-10-12 14:48:22.787251
40400	ESS	2020-10-07	211.3	212.36	207.23	210.41	930232	{}	\N	2020-10-12 14:48:22.791488
40401	ESS	2020-10-08	212.06	216.87	211.17	215.98	669913	{}	\N	2020-10-12 14:48:22.795607
40402	ESS	2020-10-09	217.75	218.325	213.045	215.45	685643	{}	\N	2020-10-12 14:48:22.79994
40403	ESS	2020-10-12	215.56	216.35	213.6	215.075	78243	{}	\N	2020-10-12 14:48:22.804063
40404	MUX	2020-09-28	0.9939	1.04	0.9754	1.02	2776955	{}	\N	2020-10-12 14:48:23.700546
40405	MUX	2020-09-29	1.05	1.118	1.03	1.08	7822545	{}	\N	2020-10-12 14:48:23.706963
40406	MUX	2020-09-30	1.08	1.1	1.05	1.06	3759335	{}	\N	2020-10-12 14:48:23.712069
40407	MUX	2020-10-01	1.07	1.11	1.06	1.08	4614968	{}	\N	2020-10-12 14:48:23.716537
40408	MUX	2020-10-02	1.07	1.09	1.04	1.06	4358379	{}	\N	2020-10-12 14:48:23.721402
40409	MUX	2020-10-05	1.06	1.12	1.05	1.11	5215659	{}	\N	2020-10-12 14:48:23.725741
40410	MUX	2020-10-06	1.12	1.15	1.05	1.06	8015685	{}	\N	2020-10-12 14:48:23.730275
40411	MUX	2020-10-07	1.09	1.0999	1.035	1.07	5488266	{}	\N	2020-10-12 14:48:23.734703
40412	MUX	2020-10-08	1.09	1.12	1.07	1.1	3660606	{}	\N	2020-10-12 14:48:23.739463
40413	MUX	2020-10-09	1.13	1.17	1.1	1.17	7251573	{}	\N	2020-10-12 14:48:23.743661
40414	MUX	2020-10-12	1.1408	1.17	1.14	1.145	1869972	{}	\N	2020-10-12 14:48:23.747469
40415	ENFR	2020-09-28	12.5869	12.8	12.51	12.67	21959	{}	\N	2020-10-12 14:48:25.048763
40416	ENFR	2020-09-29	12.61	12.75	12.549	12.68	22964	{}	\N	2020-10-12 14:48:25.055918
40417	ENFR	2020-09-30	12.7	12.8	12.3	12.33	226219	{}	\N	2020-10-12 14:48:25.061183
40418	ENFR	2020-10-01	12.45	12.45	12.19	12.25	51704	{}	\N	2020-10-12 14:48:25.065641
40419	ENFR	2020-10-02	12.06	12.43	12	12.43	22132	{}	\N	2020-10-12 14:48:25.070317
40420	ENFR	2020-10-05	12.6	12.6998	12.45	12.6998	24350	{}	\N	2020-10-12 14:48:25.074707
40421	ENFR	2020-10-06	12.75	12.9599	12.44	12.44	10910	{}	\N	2020-10-12 14:48:25.078656
40422	ENFR	2020-10-07	12.55	12.63	12.4511	12.61	14836	{}	\N	2020-10-12 14:48:25.083864
40423	ENFR	2020-10-08	12.65	13.219	12.65	13.19	13438	{}	\N	2020-10-12 14:48:25.087644
40424	ENFR	2020-10-09	13.28	13.28	13.0701	13.11	11473	{}	\N	2020-10-12 14:48:25.091374
40425	ENFR	2020-10-12	13.11	13.12	13.0201	13.0527	2463	{}	\N	2020-10-12 14:48:25.095058
40426	ENOR	2020-09-28	20.1993	20.26	20.115	20.25	20072	{}	\N	2020-10-12 14:48:27.128929
40427	ENOR	2020-09-29	20.28	20.31	20.16	20.297	2672	{}	\N	2020-10-12 14:48:27.135566
40428	ENOR	2020-09-30	20.43	20.64	20.43	20.64	3928	{}	\N	2020-10-12 14:48:27.140747
40429	ENOR	2020-10-01	20.7796	20.7796	20.52	20.61	232134	{}	\N	2020-10-12 14:48:27.145529
40430	ENOR	2020-10-02	20.39	20.8189	20.39	20.79	15463	{}	\N	2020-10-12 14:48:27.150207
40431	ENOR	2020-10-05	21	21.35	21	21.35	5967	{}	\N	2020-10-12 14:48:27.154672
40432	ENOR	2020-10-06	21.36	21.36	21.053	21.053	753	{}	\N	2020-10-12 14:48:27.159418
40433	ENOR	2020-10-07	21.19	21.2395	21.11	21.2395	878	{}	\N	2020-10-12 14:48:27.163986
40434	ENOR	2020-10-08	21.363	21.4849	21.363	21.4849	3009	{}	\N	2020-10-12 14:48:27.169625
40435	ENOR	2020-10-09	21.55	21.6399	21.55	21.6399	4066	{}	\N	2020-10-12 14:48:27.174607
40436	ENTX	2020-09-28	1.3	1.315	1.25	1.27	43766	{}	\N	2020-10-12 14:48:28.499931
40437	ENTX	2020-09-29	1.32	1.32	1.25	1.25	16448	{}	\N	2020-10-12 14:48:28.505672
40438	ENTX	2020-09-30	1.25	1.25	1.24	1.24	62220	{}	\N	2020-10-12 14:48:28.511658
40439	ENTX	2020-10-01	1.2474	1.25	1.22	1.22	15273	{}	\N	2020-10-12 14:48:28.515842
40440	ENTX	2020-10-02	1.2141	1.24	1.21	1.24	16955	{}	\N	2020-10-12 14:48:28.520533
40441	ENTX	2020-10-05	1.25	1.26	1.18	1.2	31348	{}	\N	2020-10-12 14:48:28.52569
40442	ENTX	2020-10-06	1.192	1.1976	1.13	1.15	32672	{}	\N	2020-10-12 14:48:28.530274
40443	ENTX	2020-10-07	1.1742	1.1742	1.1201	1.15	44138	{}	\N	2020-10-12 14:48:28.535112
40444	ENTX	2020-10-08	1.1501	1.18	1.1501	1.17	37211	{}	\N	2020-10-12 14:48:28.540005
40445	ENTX	2020-10-09	1.19	1.19	1.14	1.17	31739	{}	\N	2020-10-12 14:48:28.544715
40446	ENTX	2020-10-12	1.13	1.16	1.13	1.13	7871	{}	\N	2020-10-12 14:48:28.549108
40447	EPAY	2020-09-28	42.73	42.95	42.43	42.82	130645	{}	\N	2020-10-12 14:48:30.245419
40448	EPAY	2020-09-29	42.83	43.29	42.36	42.6	129853	{}	\N	2020-10-12 14:48:30.252178
40449	EPAY	2020-09-30	42.64	43.16	42.04	42.16	273230	{}	\N	2020-10-12 14:48:30.257119
40450	EPAY	2020-10-01	42.33	43.07	41.825	43.03	297252	{}	\N	2020-10-12 14:48:30.262304
40451	EPAY	2020-10-02	42.23	42.83	41.77	42.28	190885	{}	\N	2020-10-12 14:48:30.267077
40452	EPAY	2020-10-05	42.76	43.365	42.45	42.9	117932	{}	\N	2020-10-12 14:48:30.271483
40453	EPAY	2020-10-06	43.3	43.82	42.2948	42.73	188421	{}	\N	2020-10-12 14:48:30.275704
40454	EPAY	2020-10-07	43.12	43.17	42.53	43.09	417823	{}	\N	2020-10-12 14:48:30.28
40455	EPAY	2020-10-08	43.54	44.62	43.24	44	234174	{}	\N	2020-10-12 14:48:30.284936
40456	EPAY	2020-10-09	44.24	44.86	44.24	44.86	140294	{}	\N	2020-10-12 14:48:30.289249
40457	EPAY	2020-10-12	45.14	45.32	44.43	45.19	19796	{}	\N	2020-10-12 14:48:30.293856
40458	EPR	2020-09-28	27.57	28.62	27.5	28.18	615633	{}	\N	2020-10-12 14:48:31.422754
40459	EPR	2020-09-29	28	28.04	26.974	27.54	923742	{}	\N	2020-10-12 14:48:31.429742
40460	EPR	2020-09-30	27.79	28.41	27.135	27.5	1643200	{}	\N	2020-10-12 14:48:31.43512
40461	EPR	2020-10-01	27.57	28.06	27.2638	27.89	1620398	{}	\N	2020-10-12 14:48:31.440601
40462	EPR	2020-10-02	26.67	28.69	26.67	28.63	2889121	{}	\N	2020-10-12 14:48:31.445974
40463	EPR	2020-10-05	27.38	27.48	25.2984	25.4	5344346	{}	\N	2020-10-12 14:48:31.450799
40464	EPR	2020-10-06	25.32	25.48	24.25	24.39	3341748	{}	\N	2020-10-12 14:48:31.455803
40465	EPR	2020-10-07	24.51	25.08	24.14	24.39	2260711	{}	\N	2020-10-12 14:48:31.460895
40466	EPR	2020-10-08	24.76	25.38	24.61	25.33	1528340	{}	\N	2020-10-12 14:48:31.465657
40467	EPR	2020-10-09	25.51	25.54	24.43	24.76	1849789	{}	\N	2020-10-12 14:48:31.470774
40468	EPR	2020-10-12	24.81	24.92	24.26	24.55	329155	{}	\N	2020-10-12 14:48:31.475679
40469	ERH	2020-09-28	11.84	11.9411	11.84	11.89	20434	{}	\N	2020-10-12 14:48:33.064861
40470	ERH	2020-09-29	11.87	11.99	11.84	11.87	16868	{}	\N	2020-10-12 14:48:33.071601
40471	ERH	2020-09-30	12.12	12.12	11.92	12.01	20082	{}	\N	2020-10-12 14:48:33.076711
40472	ERH	2020-10-01	12.12	12.12	11.98	12.09	29469	{}	\N	2020-10-12 14:48:33.081786
40473	ERH	2020-10-02	12.11	12.12	12.05	12.105	21475	{}	\N	2020-10-12 14:48:33.086881
40474	ERH	2020-10-05	12.07	12.24	12.07	12.205	15020	{}	\N	2020-10-12 14:48:33.091267
40475	ERH	2020-10-06	12.2063	12.37	12.1823	12.27	7995	{}	\N	2020-10-12 14:48:33.096034
40476	ERH	2020-10-07	12.355	12.425	12.355	12.405	15416	{}	\N	2020-10-12 14:48:33.100747
40477	ERH	2020-10-08	12.39	12.5	12.39	12.49	17869	{}	\N	2020-10-12 14:48:33.106059
40478	ERH	2020-10-09	12.5	12.5295	12.41	12.4983	11417	{}	\N	2020-10-12 14:48:33.110846
40479	ERH	2020-10-12	12.4955	12.55	12.1	12.4938	10323	{}	\N	2020-10-12 14:48:33.115557
40480	ERY	2020-09-28	66.78	66.83	64.48	65.93	323279	{}	\N	2020-10-12 14:48:34.305155
40481	ERY	2020-09-29	66.77	70.91	66.1199	69.5	218615	{}	\N	2020-10-12 14:48:34.312778
40482	ERY	2020-09-30	68.73	70.55	67.16	69.81	185692	{}	\N	2020-10-12 14:48:34.318146
40483	ERY	2020-10-01	71.51	75.1696	71.02	74.47	342924	{}	\N	2020-10-12 14:48:34.323577
40484	ERY	2020-10-02	78.53	78.53	71.63	72.84	218610	{}	\N	2020-10-12 14:48:34.328963
40485	ERY	2020-10-05	71.63	71.95	68.48	68.61	196172	{}	\N	2020-10-12 14:48:34.33397
40486	ERY	2020-10-06	66.6	70.91	65.75	70.59	198054	{}	\N	2020-10-12 14:48:34.338642
40487	ERY	2020-10-07	70.28	71.36	68.24	68.62	131296	{}	\N	2020-10-12 14:48:34.343335
40488	ERY	2020-10-08	67.35	68	63.35	63.36	259893	{}	\N	2020-10-12 14:48:34.347865
40489	ERY	2020-10-09	62.32	65.61	61.97	65.5	292540	{}	\N	2020-10-12 14:48:34.354312
40490	ERY	2020-10-12	65.91	67.23	65.35	66.0401	61054	{}	\N	2020-10-12 14:48:34.360043
40491	ESGS	2020-09-28	25.2625	25.2625	25.2625	25.2625	106	{}	\N	2020-10-12 14:48:36.015724
40492	ESGS	2020-09-29	25.64	25.64	25.0564	25.0564	352	{}	\N	2020-10-12 14:48:36.021681
40493	ESGS	2020-09-30	25.4266	25.4266	25.2705	25.2705	282	{}	\N	2020-10-12 14:48:36.025491
40494	ESGS	2020-10-01	25.1366	25.1366	25.1366	25.1366	217	{}	\N	2020-10-12 14:48:36.029831
40495	ESGS	2020-10-02	25.56	25.56	25.4814	25.4814	378	{}	\N	2020-10-12 14:48:36.034079
40496	ESGS	2020-10-05	26.0261	26.0261	26.0261	26.0261	8	{}	\N	2020-10-12 14:48:36.038119
40497	ESGS	2020-10-06	25.8748	25.8748	25.8748	25.8748	92	{}	\N	2020-10-12 14:48:36.042016
40498	ESGS	2020-10-07	26.3768	26.3768	26.3768	26.3768	5	{}	\N	2020-10-12 14:48:36.046216
40499	ESGS	2020-10-08	26.87	26.87	26.7622	26.7622	323	{}	\N	2020-10-12 14:48:36.050266
40500	ESGS	2020-10-09	26.75	26.78	26.7208	26.7208	13320	{}	\N	2020-10-12 14:48:36.054545
40501	ESNT	2020-09-28	36.59	37.54	36.485	37.15	448901	{}	\N	2020-10-12 14:48:37.459812
40502	ESNT	2020-09-29	37.05	37.05	35.75	36.44	744180	{}	\N	2020-10-12 14:48:37.464618
40503	ESNT	2020-09-30	36.49	37.31	36.49	37.01	1146359	{}	\N	2020-10-12 14:48:37.468783
40504	ESNT	2020-10-01	37.12	37.86	36.39	37.8	1170508	{}	\N	2020-10-12 14:48:37.474123
40505	ESNT	2020-10-02	36.77	40.13	36.77	39.69	1427040	{}	\N	2020-10-12 14:48:37.478684
40506	ESNT	2020-10-05	40.23	40.98	39.81	40.95	911162	{}	\N	2020-10-12 14:48:37.483605
40507	ESNT	2020-10-06	41.65	43.11	41.2	41.81	1944387	{}	\N	2020-10-12 14:48:37.488474
40508	ESNT	2020-10-07	42.44	43.74	42	42.33	1591054	{}	\N	2020-10-12 14:48:37.493228
40509	ESNT	2020-10-08	43.09	43.09	42.24	42.7	962588	{}	\N	2020-10-12 14:48:37.497324
40510	ESNT	2020-10-09	43.41	43.47	42.29	42.75	1245685	{}	\N	2020-10-12 14:48:37.501588
40511	ESNT	2020-10-12	43.17	43.5	42.7	43.48	227644	{}	\N	2020-10-12 14:48:37.50698
40512	ETM	2020-09-28	1.5	1.59	1.47	1.57	1567376	{}	\N	2020-10-12 14:48:38.93418
40513	ETM	2020-09-29	1.59	1.67	1.55	1.62	2101428	{}	\N	2020-10-12 14:48:38.940434
40514	ETM	2020-09-30	1.57	1.71	1.56	1.61	1854759	{}	\N	2020-10-12 14:48:38.94545
40515	ETM	2020-10-01	1.57	1.65	1.57	1.61	1352730	{}	\N	2020-10-12 14:48:38.950682
40516	ETM	2020-10-02	1.56	1.64	1.5232	1.59	1427578	{}	\N	2020-10-12 14:48:38.955453
40517	ETM	2020-10-05	1.56	1.62	1.49	1.54	1180352	{}	\N	2020-10-12 14:48:38.960263
40518	ETM	2020-10-06	1.59	1.61	1.49	1.49	1369250	{}	\N	2020-10-12 14:48:38.964441
40519	ETM	2020-10-07	1.49	1.56	1.49	1.54	1580773	{}	\N	2020-10-12 14:48:38.968858
40520	ETM	2020-10-08	1.5	1.64	1.49	1.62	1513379	{}	\N	2020-10-12 14:48:38.97327
40521	ETM	2020-10-09	1.62	1.66	1.5	1.53	1555083	{}	\N	2020-10-12 14:48:38.977735
40522	ETM	2020-10-12	1.5	1.56	1.5	1.5575	286854	{}	\N	2020-10-12 14:48:38.981858
40523	ENO	2020-09-28	26.95	27.2996	26.84	27.2996	10193	{}	\N	2020-10-12 14:48:40.620584
40524	ENO	2020-09-29	26.98	27.19	26.88	27.19	8078	{}	\N	2020-10-12 14:48:40.626673
40525	ENO	2020-09-30	27.16	27.23	26.65	26.65	53508	{}	\N	2020-10-12 14:48:40.63145
40526	ENO	2020-10-01	26.76	27.0085	26.76	26.8	15861	{}	\N	2020-10-12 14:48:40.636546
40527	ENO	2020-10-02	26.77	26.8465	26.24	26.345	40326	{}	\N	2020-10-12 14:48:40.640823
40528	ENO	2020-10-05	26.37	26.37	26.05	26.17	28650	{}	\N	2020-10-12 14:48:40.645574
40529	ENO	2020-10-06	26.13	26.1882	26.1	26.16	8458	{}	\N	2020-10-12 14:48:40.649594
40530	ENO	2020-10-07	26.17	26.38	26.17	26.37	23960	{}	\N	2020-10-12 14:48:40.654467
40531	ENO	2020-10-08	26.38	26.6	26.36	26.6	12074	{}	\N	2020-10-12 14:48:40.658605
40532	ENO	2020-10-09	26.57	26.635	26.09	26.3	54166	{}	\N	2020-10-12 14:48:40.662608
40533	ENO	2020-10-12	26.29	26.29	26.22	26.22	6624	{}	\N	2020-10-12 14:48:40.667271
40534	ENTR	2020-09-28	25.3	25.3	24.445	24.59	3630	{}	\N	2020-10-12 14:48:41.681731
40535	ENTR	2020-09-29	24.74	24.79	24.61	24.7609	3280	{}	\N	2020-10-12 14:48:41.687278
40536	ENTR	2020-09-30	24.98	24.995	24.8316	24.8334	1333	{}	\N	2020-10-12 14:48:41.691296
40537	ENTR	2020-10-01	25.26	25.4304	25.2314	25.4304	8760	{}	\N	2020-10-12 14:48:41.695676
40538	ENTR	2020-10-02	25.25	25.525	25	25.01	4141	{}	\N	2020-10-12 14:48:41.700051
40539	ENTR	2020-10-05	25.22	25.67	25.22	25.67	4754	{}	\N	2020-10-12 14:48:41.704784
40540	ENTR	2020-10-06	26.1	26.1	25.3326	25.35	4160	{}	\N	2020-10-12 14:48:41.708828
40541	ENTR	2020-10-07	25.9	25.9	25.87	25.87	603	{}	\N	2020-10-12 14:48:41.71352
40542	ENTR	2020-10-08	25.88	26.06	25.88	25.93	6335	{}	\N	2020-10-12 14:48:41.717633
40543	ENTR	2020-10-09	26.2001	26.38	26.2001	26.38	1924	{}	\N	2020-10-12 14:48:41.721696
40544	ENTR	2020-10-12	26.73	26.73	26.6399	26.6399	707	{}	\N	2020-10-12 14:48:41.725528
40545	EOD	2020-09-28	4.18	4.21	4.15	4.16	334409	{}	\N	2020-10-12 14:48:44.126676
40546	EOD	2020-09-29	4.18	4.21	4.15	4.2	527262	{}	\N	2020-10-12 14:48:44.133585
40547	EOD	2020-09-30	4.19	4.27	4.19	4.23	1027269	{}	\N	2020-10-12 14:48:44.138964
40548	EOD	2020-10-01	4.32	4.32	4.25	4.28	1456164	{}	\N	2020-10-12 14:48:44.144031
40549	EOD	2020-10-02	4.23	4.31	4.2064	4.31	508632	{}	\N	2020-10-12 14:48:44.149177
40550	EOD	2020-10-05	4.32	4.37	4.29	4.31	520326	{}	\N	2020-10-12 14:48:44.154479
40551	EOD	2020-10-06	4.33	4.36	4.27	4.27	487394	{}	\N	2020-10-12 14:48:44.159411
40552	EOD	2020-10-07	4.29	4.36	4.29	4.36	372691	{}	\N	2020-10-12 14:48:44.164108
40553	EOD	2020-10-08	4.35	4.4	4.34	4.38	346010	{}	\N	2020-10-12 14:48:44.170323
40554	EOD	2020-10-09	4.38	4.43	4.38	4.42	369628	{}	\N	2020-10-12 14:48:44.175718
40555	EOD	2020-10-12	4.4	4.43	4.39	4.43	151006	{}	\N	2020-10-12 14:48:44.180642
40556	EPV	2020-09-28	22.39	22.495	22.22	22.22	6963	{}	\N	2020-10-12 14:48:45.18466
40557	EPV	2020-09-29	22.28	22.4678	22.21	22.439	3707	{}	\N	2020-10-12 14:48:45.190727
40558	EPV	2020-09-30	22.2083	22.53	22.15	22.52	8016	{}	\N	2020-10-12 14:48:45.1954
40559	EPV	2020-10-01	22.1621	22.415	22.14	22.1782	9154	{}	\N	2020-10-12 14:48:45.199712
40560	EPV	2020-10-02	22.39	22.4	22.0916	22.16	4374	{}	\N	2020-10-12 14:48:45.204763
40561	EPV	2020-10-05	21.78	21.78	21.4178	21.4178	7127	{}	\N	2020-10-12 14:48:45.209484
40562	EPV	2020-10-06	21.48	22.005	21.3801	21.91	5306	{}	\N	2020-10-12 14:48:45.213495
40563	EPV	2020-10-07	21.6	21.6745	21.58	21.58	1504	{}	\N	2020-10-12 14:48:45.217556
40564	EPV	2020-10-08	21.3282	21.35	21.21	21.21	2752	{}	\N	2020-10-12 14:48:45.222169
40565	EPV	2020-10-09	20.8835	20.9599	20.755	20.78	4011	{}	\N	2020-10-12 14:48:45.226362
40566	EPV	2020-10-12	20.69	20.69	20.66	20.66	2232	{}	\N	2020-10-12 14:48:45.230367
40567	EQT	2020-09-28	13.97	14.135	13.235	13.99	3610602	{}	\N	2020-10-12 14:48:46.548718
40568	EQT	2020-09-29	13.73	13.78	12.75	12.84	13209642	{}	\N	2020-10-12 14:48:46.554398
40569	EQT	2020-09-30	12.73	13.17	12.575	12.93	8790183	{}	\N	2020-10-12 14:48:46.559336
40570	EQT	2020-10-01	12.81	13.08	12.46	12.68	5692332	{}	\N	2020-10-12 14:48:46.56355
40571	EQT	2020-10-02	12.29	13.075	12.27	12.73	4956737	{}	\N	2020-10-12 14:48:46.567994
40572	EQT	2020-10-05	13.04	13.73	12.99	13.69	8918829	{}	\N	2020-10-12 14:48:46.573394
40573	EQT	2020-10-06	13.88	14.24	13.4	13.46	6132218	{}	\N	2020-10-12 14:48:46.577305
40574	EQT	2020-10-07	13.67	14.21	13.505	14.13	7047188	{}	\N	2020-10-12 14:48:46.581014
40575	EQT	2020-10-08	14.27	14.505	13.885	14.5	6118853	{}	\N	2020-10-12 14:48:46.584942
40576	EQT	2020-10-09	14.91	15.24	14.385	14.69	8270022	{}	\N	2020-10-12 14:48:46.589198
40577	EQT	2020-10-12	14.76	15.17	14.59	15.015	1893498	{}	\N	2020-10-12 14:48:46.593167
40578	ERYP	2020-09-28	6.12	6.12	5.98	5.98	550	{}	\N	2020-10-12 14:48:48.683964
40579	ERYP	2020-09-30	6.25	6.25	6.0001	6.0001	856	{}	\N	2020-10-12 14:48:48.688826
40580	ERYP	2020-10-01	6	6	6	6	148	{}	\N	2020-10-12 14:48:48.693068
40581	ERYP	2020-10-02	6	6.075	5.56	5.56	2598	{}	\N	2020-10-12 14:48:48.696887
40582	ERYP	2020-10-05	6.02	6.02	5.72	5.72	890	{}	\N	2020-10-12 14:48:48.700606
40583	ERYP	2020-10-06	6	6.59	6	6.59	1150	{}	\N	2020-10-12 14:48:48.706821
40584	ERYP	2020-10-07	6.59	6.59	5.85	6.1213	4937	{}	\N	2020-10-12 14:48:48.716055
40585	ERYP	2020-10-08	5.9	5.9	5.8866	5.8866	557	{}	\N	2020-10-12 14:48:48.721764
40586	ERYP	2020-10-09	5.88	5.95	5.52	5.75	11180	{}	\N	2020-10-12 14:48:48.725718
40587	ETO	2020-09-28	20.72	20.925	20.56	20.65	61496	{}	\N	2020-10-12 14:48:52.391183
40588	ETO	2020-09-29	20.66	20.78	20.56	20.61	114191	{}	\N	2020-10-12 14:48:52.396635
40589	ETO	2020-09-30	20.65	20.84	20.61	20.79	129112	{}	\N	2020-10-12 14:48:52.401461
40590	ETO	2020-10-01	20.86	20.9066	20.79	20.88	76324	{}	\N	2020-10-12 14:48:52.405325
40591	ETO	2020-10-02	20.71	20.86	20.65	20.81	66003	{}	\N	2020-10-12 14:48:52.409298
40592	ETO	2020-10-05	20.81	21.26	20.81	21.25	107374	{}	\N	2020-10-12 14:48:52.41324
40593	ETO	2020-10-06	21.32	21.49	20.91	20.99	98932	{}	\N	2020-10-12 14:48:52.417802
40594	ETO	2020-10-07	21.26	21.488	21.2122	21.42	123950	{}	\N	2020-10-12 14:48:52.422928
40595	ETO	2020-10-08	21.56	21.71	21.56	21.7	73382	{}	\N	2020-10-12 14:48:52.42795
40596	ETO	2020-10-09	21.79	21.84	21.69	21.71	81637	{}	\N	2020-10-12 14:48:52.432277
40597	ETO	2020-10-12	21.92	21.99	21.8729	21.97	23150	{}	\N	2020-10-12 14:48:52.436523
40598	EUDV	2020-09-28	43.4441	43.4441	43.4441	43.4441	238	{}	\N	2020-10-12 14:48:54.425065
40599	EUDV	2020-09-29	43.5983	43.5983	43.5983	43.5983	115	{}	\N	2020-10-12 14:48:54.431458
40600	EUDV	2020-09-30	43.559	43.559	43.3838	43.3838	329	{}	\N	2020-10-12 14:48:54.436952
40601	EUDV	2020-10-01	43.73	43.7727	43.68	43.7727	1220	{}	\N	2020-10-12 14:48:54.440546
40602	EUDV	2020-10-02	43.8197	43.8197	43.8197	43.8197	206	{}	\N	2020-10-12 14:48:54.444427
40603	EUDV	2020-10-05	44.3748	44.3748	44.3748	44.3748	136	{}	\N	2020-10-12 14:48:54.448506
40604	EUDV	2020-10-06	43.6375	43.6375	43.6375	43.6375	47	{}	\N	2020-10-12 14:48:54.452792
40605	EUDV	2020-10-07	43.7442	43.7442	43.7442	43.7442	243	{}	\N	2020-10-12 14:48:54.456378
40606	EUDV	2020-10-08	44.05	44.05	44.0444	44.0444	334	{}	\N	2020-10-12 14:48:54.460178
40607	EUDV	2020-10-09	44.45	44.487	44.45	44.487	722	{}	\N	2020-10-12 14:48:54.463922
40608	ENTG	2020-09-28	69.8	71.7	69.72	71.4	947781	{}	\N	2020-10-12 14:48:55.41559
40609	ENTG	2020-09-29	71.85	74.15	71.5	73.19	858225	{}	\N	2020-10-12 14:48:55.426286
40610	ENTG	2020-09-30	72.92	75.16	72.6	74.43	1046520	{}	\N	2020-10-12 14:48:55.432355
40611	ENTG	2020-10-01	76.01	78.31	75.03	76.65	1331603	{}	\N	2020-10-12 14:48:55.437136
40612	ENTG	2020-10-02	75.04	76.98	75.035	75.92	964467	{}	\N	2020-10-12 14:48:55.441053
40613	ENTG	2020-10-05	77.325	78.93	76.47	78.9	1001328	{}	\N	2020-10-12 14:48:55.444735
40614	ENTG	2020-10-06	78.76	81.76	78.76	80.09	1007217	{}	\N	2020-10-12 14:48:55.448527
40615	ENTG	2020-10-07	81.06	82.03	79.68	81.76	853523	{}	\N	2020-10-12 14:48:55.452518
40616	ENTG	2020-10-08	81.74	82.86	81.24	82.15	561640	{}	\N	2020-10-12 14:48:55.456603
40617	ENTG	2020-10-09	82.79	83.4127	82.5	82.74	698877	{}	\N	2020-10-12 14:48:55.460189
40618	ENTG	2020-10-12	84.14	84.4	83.28	83.3	123588	{}	\N	2020-10-12 14:48:55.463775
40619	EOT	2020-09-28	20.23	20.38	20.215	20.215	24704	{}	\N	2020-10-12 14:48:56.406758
40620	EOT	2020-09-29	20.21	20.32	20.08	20.18	51644	{}	\N	2020-10-12 14:48:56.411768
40621	EOT	2020-09-30	20.23	20.37	20.04	20.04	54042	{}	\N	2020-10-12 14:48:56.41707
40622	EOT	2020-10-01	20.12	20.1978	20.04	20.13	32428	{}	\N	2020-10-12 14:48:56.422406
40623	EOT	2020-10-02	20.02	20.28	20.02	20.1618	25218	{}	\N	2020-10-12 14:48:56.426561
40624	EOT	2020-10-05	20.13	20.16	19.885	20.13	25740	{}	\N	2020-10-12 14:48:56.430355
40625	EOT	2020-10-06	20.13	20.14	19.94	19.95	42252	{}	\N	2020-10-12 14:48:56.434608
40626	EOT	2020-10-07	19.96	20.17	19.96	20.09	39280	{}	\N	2020-10-12 14:48:56.43884
40627	EOT	2020-10-08	20.19	20.19	19.96	20.06	30522	{}	\N	2020-10-12 14:48:56.442994
40628	EOT	2020-10-09	20.1	20.18	20.1	20.15	17704	{}	\N	2020-10-12 14:48:56.446822
40629	EOT	2020-10-12	20.37	20.37	20.1665	20.2053	25228	{}	\N	2020-10-12 14:48:56.451065
40630	EPI	2020-09-28	23.49	23.5	23.37	23.42	819215	{}	\N	2020-10-12 14:48:57.798659
40631	EPI	2020-09-29	23.4	23.495	23.37	23.42	429114	{}	\N	2020-10-12 14:48:57.805803
40632	EPI	2020-09-30	23.51	23.71	23.51	23.6	1070669	{}	\N	2020-10-12 14:48:57.811758
40633	EPI	2020-10-01	23.81	23.95	23.76	23.94	788630	{}	\N	2020-10-12 14:48:57.815918
40634	EPI	2020-10-02	23.505	23.84	23.5	23.73	469660	{}	\N	2020-10-12 14:48:57.820375
40635	EPI	2020-10-05	24.07	24.2	24.07	24.15	490879	{}	\N	2020-10-12 14:48:57.824878
40636	EPI	2020-10-06	24.37	24.3827	24.16	24.21	499781	{}	\N	2020-10-12 14:48:57.829111
40637	EPI	2020-10-07	24.35	24.44	24.3	24.35	802558	{}	\N	2020-10-12 14:48:57.832852
40638	EPI	2020-10-08	24.4399	24.51	24.38	24.43	649651	{}	\N	2020-10-12 14:48:57.836859
40639	EPI	2020-10-09	24.68	24.77	24.58	24.71	518211	{}	\N	2020-10-12 14:48:57.841675
40640	EPI	2020-10-12	24.62	24.62	24.52	24.59	111697	{}	\N	2020-10-12 14:48:57.846431
40641	EPS	2020-09-28	35.16	35.44	35.16	35.35	25714	{}	\N	2020-10-12 14:48:59.220234
40642	EPS	2020-09-29	35.18	35.244	35.05	35.1761	29468	{}	\N	2020-10-12 14:48:59.226167
40643	EPS	2020-09-30	35.15	35.6301	35.15	35.42	56610	{}	\N	2020-10-12 14:48:59.231056
40644	EPS	2020-10-01	35.61	35.6988	35.3673	35.55	20837	{}	\N	2020-10-12 14:48:59.235873
40645	EPS	2020-10-02	35.11	35.535	35.1001	35.37	23147	{}	\N	2020-10-12 14:48:59.240474
40646	EPS	2020-10-05	35.705	35.96	35.6644	35.96	23449	{}	\N	2020-10-12 14:48:59.244624
40647	EPS	2020-10-06	36.04	36.27	35.52	35.52	18071	{}	\N	2020-10-12 14:48:59.248805
40648	EPS	2020-10-07	35.9876	36.197	35.91	36.1699	54626	{}	\N	2020-10-12 14:48:59.253258
40649	EPS	2020-10-08	36.26	36.49	36.26	36.48	34768	{}	\N	2020-10-12 14:48:59.257501
40650	EPS	2020-10-09	36.58	36.79	36.55	36.68	228561	{}	\N	2020-10-12 14:48:59.261916
40651	EPS	2020-10-12	36.93	37.0469	36.91	37.02	4446	{}	\N	2020-10-12 14:48:59.266813
40652	EQL	2020-09-28	77.0841	77.2435	76.9118	76.9118	1032	{}	\N	2020-10-12 14:49:00.393707
40653	EQL	2020-09-29	76.845	76.923	76.16	76.651	2336	{}	\N	2020-10-12 14:49:00.399689
40654	EQL	2020-09-30	77.335	77.55	76.68	76.68	4734	{}	\N	2020-10-12 14:49:00.4048
40655	EQL	2020-10-01	77.279	77.297	76.96	77.0542	2240	{}	\N	2020-10-12 14:49:00.409558
40656	EQL	2020-10-02	75.66	77.35	75.66	77.07	5297	{}	\N	2020-10-12 14:49:00.413887
40657	EQL	2020-10-05	77.905	78.3856	77.6549	78.3512	4148	{}	\N	2020-10-12 14:49:00.419602
40658	EQL	2020-10-06	78.54	78.9513	77.5645	77.5645	1260	{}	\N	2020-10-12 14:49:00.425639
40659	EQL	2020-10-07	78.3807	78.735	78.3807	78.735	3266	{}	\N	2020-10-12 14:49:00.430869
40660	EQL	2020-10-08	78.98	79.6268	78.98	79.6268	1210	{}	\N	2020-10-12 14:49:00.435452
40661	EQL	2020-10-09	80.051	80.17	79.76	79.97	7457	{}	\N	2020-10-12 14:49:00.440333
40662	EQL	2020-10-12	80.325	80.325	80.325	80.325	197	{}	\N	2020-10-12 14:49:00.44475
40663	ERF	2020-09-28	1.83	1.92	1.82	1.91	1626408	{}	\N	2020-10-12 14:49:01.39245
40664	ERF	2020-09-29	1.91	1.91	1.73	1.81	9416181	{}	\N	2020-10-12 14:49:01.399987
40665	ERF	2020-09-30	1.83	1.89	1.7713	1.86	11085587	{}	\N	2020-10-12 14:49:01.40486
40666	ERF	2020-10-01	1.81	1.84	1.76	1.8	5901980	{}	\N	2020-10-12 14:49:01.409834
40667	ERF	2020-10-02	1.75	1.88	1.72	1.85	3393246	{}	\N	2020-10-12 14:49:01.414166
40668	ERF	2020-10-05	1.89	1.96	1.82	1.96	2900913	{}	\N	2020-10-12 14:49:01.419818
40669	ERF	2020-10-06	2.03	2.0698	1.9	1.91	3327514	{}	\N	2020-10-12 14:49:01.425439
40670	ERF	2020-10-07	1.97	2.01	1.87	1.94	2328324	{}	\N	2020-10-12 14:49:01.42953
40671	ERF	2020-10-08	1.96	2.11	1.9345	2.1	2718042	{}	\N	2020-10-12 14:49:01.433728
40672	ERF	2020-10-09	2.13	2.13	2	2.04	2206709	{}	\N	2020-10-12 14:49:01.438034
40673	ERF	2020-10-12	2.02	2.02	1.975	2	193942	{}	\N	2020-10-12 14:49:01.442519
40674	ERX	2020-09-28	9.82	10.15	9.81	9.93	3517783	{}	\N	2020-10-12 14:49:02.731211
40675	ERX	2020-09-29	9.85	9.93	9.2	9.41	2686966	{}	\N	2020-10-12 14:49:02.739987
40676	ERX	2020-09-30	9.53	9.73	9.25	9.36	2547385	{}	\N	2020-10-12 14:49:02.745475
40677	ERX	2020-10-01	9.13	9.21	8.65	8.77	4807037	{}	\N	2020-10-12 14:49:02.750584
40678	ERX	2020-10-02	8.28	9.0799	8.26	8.94	3234412	{}	\N	2020-10-12 14:49:02.755285
40679	ERX	2020-10-05	9.12	9.47	9.04	9.47	3226986	{}	\N	2020-10-12 14:49:02.759849
40680	ERX	2020-10-06	9.74	9.85	9.13	9.16	2975003	{}	\N	2020-10-12 14:49:02.764545
40681	ERX	2020-10-07	9.3	9.49	9.0713	9.45	2719864	{}	\N	2020-10-12 14:49:02.769044
40682	ERX	2020-10-08	9.65	10.16	9.51	10.15	4895661	{}	\N	2020-10-12 14:49:02.774646
40683	ERX	2020-10-09	10.35	10.38	9.79	9.82	4146309	{}	\N	2020-10-12 14:49:02.779599
40684	ERX	2020-10-12	9.765	9.835	9.55	9.74	680249	{}	\N	2020-10-12 14:49:02.78428
40685	ESGD	2020-09-28	63.77	63.9121	63.637	63.91	123787	{}	\N	2020-10-12 14:49:03.718579
40686	ESGD	2020-09-29	63.89	64.0899	63.6501	63.8	145046	{}	\N	2020-10-12 14:49:03.730807
40687	ESGD	2020-09-30	63.62	64.0199	63.365	63.48	368507	{}	\N	2020-10-12 14:49:03.7407
40688	ESGD	2020-10-01	63.88	63.99	63.58	63.94	187928	{}	\N	2020-10-12 14:49:03.749642
40689	ESGD	2020-10-02	63.16	63.93	63.16	63.78	371460	{}	\N	2020-10-12 14:49:03.756252
40690	ESGD	2020-10-05	64.31	64.7399	64.31	64.71	131113	{}	\N	2020-10-12 14:49:03.767482
40691	ESGD	2020-10-06	64.77	64.91	64	64.16	229167	{}	\N	2020-10-12 14:49:03.774426
40692	ESGD	2020-10-07	64.58	64.7099	64.38	64.63	895117	{}	\N	2020-10-12 14:49:03.780193
40693	ESGD	2020-10-08	64.91	65.11	64.85	65.09	197428	{}	\N	2020-10-12 14:49:03.784788
40694	ESGD	2020-10-09	65.35	65.5503	65.24	65.53	335602	{}	\N	2020-10-12 14:49:03.789606
40695	ESGD	2020-10-12	65.69	65.7	65.6182	65.68	14126	{}	\N	2020-10-12 14:49:03.794064
40696	ESGR	2020-09-28	157.57	163.1099	157.57	160.89	54853	{}	\N	2020-10-12 14:49:04.993013
40697	ESGR	2020-09-29	159.7	161.55	158.48	160.73	34613	{}	\N	2020-10-12 14:49:05.001268
40698	ESGR	2020-09-30	161.89	164.32	161.12	161.5	49905	{}	\N	2020-10-12 14:49:05.007455
40699	ESGR	2020-10-01	162	163.56	160.69	163.04	35980	{}	\N	2020-10-12 14:49:05.01266
40700	ESGR	2020-10-02	161	165.6275	161	164.37	32106	{}	\N	2020-10-12 14:49:05.017686
40701	ESGR	2020-10-05	165.95	170.01	165.95	168.62	37809	{}	\N	2020-10-12 14:49:05.023047
40702	ESGR	2020-10-06	169.79	171.985	165.0399	169.19	30569	{}	\N	2020-10-12 14:49:05.027996
40703	ESGR	2020-10-07	170.59	171.795	167.71	169.99	53204	{}	\N	2020-10-12 14:49:05.033123
40704	ESGR	2020-10-08	171.48	174.4	170.395	171.04	31476	{}	\N	2020-10-12 14:49:05.038092
40705	ESGR	2020-10-09	170.5	171.93	170	170.48	19132	{}	\N	2020-10-12 14:49:05.042284
40706	ESGR	2020-10-12	170.3	173.9	166.9377	173.9	3592	{}	\N	2020-10-12 14:49:05.047016
40707	ESTE	2020-09-28	2.68	2.79	2.67	2.72	314685	{}	\N	2020-10-12 14:49:06.14445
40708	ESTE	2020-09-29	2.7	2.8	2.6301	2.74	630377	{}	\N	2020-10-12 14:49:06.153414
40709	ESTE	2020-09-30	2.76	2.76	2.59	2.59	240408	{}	\N	2020-10-12 14:49:06.15839
40710	ESTE	2020-10-01	2.6	2.76	2.56	2.65	730398	{}	\N	2020-10-12 14:49:06.163314
40711	ESTE	2020-10-02	2.6	2.77	2.6	2.68	517524	{}	\N	2020-10-12 14:49:06.168694
40712	ESTE	2020-10-05	2.7	2.9	2.7	2.84	367366	{}	\N	2020-10-12 14:49:06.174054
40713	ESTE	2020-10-06	2.87	2.89	2.76	2.78	498582	{}	\N	2020-10-12 14:49:06.17861
40714	ESTE	2020-10-07	2.79	2.89	2.77	2.88	268572	{}	\N	2020-10-12 14:49:06.18319
40715	ESTE	2020-10-08	2.91	3.08	2.86	3.02	404216	{}	\N	2020-10-12 14:49:06.188069
40716	ESTE	2020-10-09	3.04	3.0492	2.87	2.91	325032	{}	\N	2020-10-12 14:49:06.192317
40717	ESTE	2020-10-12	2.89	2.92	2.84	2.92	18184	{}	\N	2020-10-12 14:49:06.196416
40718	ETHO	2020-09-28	43.65	44.09	43.65	44.09	3638	{}	\N	2020-10-12 14:49:07.543582
40719	ETHO	2020-09-29	44.125	44.125	43.89	44.0257	4123	{}	\N	2020-10-12 14:49:07.549193
40720	ETHO	2020-09-30	44.3	44.6	44.04	44.29	5802	{}	\N	2020-10-12 14:49:07.554204
40721	ETHO	2020-10-01	44.39	44.555	44.34	44.4889	2108	{}	\N	2020-10-12 14:49:07.558597
40722	ETHO	2020-10-02	44.46	44.5499	44.3963	44.3963	4600	{}	\N	2020-10-12 14:49:07.562483
40723	ETHO	2020-10-05	44.98	45.2992	44.76	45.2992	9960	{}	\N	2020-10-12 14:49:07.566683
40724	ETHO	2020-10-06	45.54	45.725	44.9892	44.9892	7025	{}	\N	2020-10-12 14:49:07.571131
40725	ETHO	2020-10-07	45.31	45.8568	45.31	45.8568	7315	{}	\N	2020-10-12 14:49:07.575224
40726	ETHO	2020-10-08	46.06	46.35	46.06	46.33	4641	{}	\N	2020-10-12 14:49:07.579171
40727	ETHO	2020-10-09	46.54	46.7635	46.54	46.75	1770	{}	\N	2020-10-12 14:49:07.583155
40728	ETHO	2020-10-12	46.9	47	46.9	46.9	435	{}	\N	2020-10-12 14:49:07.587431
40729	ERC	2020-09-28	10.88	10.8941	10.81	10.84	63320	{}	\N	2020-10-12 14:49:08.573013
40730	ERC	2020-09-29	10.82	10.86	10.69	10.74	164332	{}	\N	2020-10-12 14:49:08.578177
40731	ERC	2020-09-30	10.79	10.855	10.75	10.75	156896	{}	\N	2020-10-12 14:49:08.582577
40732	ERC	2020-10-01	10.94	10.94	10.81	10.87	119590	{}	\N	2020-10-12 14:49:08.586193
40733	ERC	2020-10-02	10.835	10.92	10.7	10.8	303987	{}	\N	2020-10-12 14:49:08.590237
40734	ERC	2020-10-05	10.83	10.91	10.81	10.9	93856	{}	\N	2020-10-12 14:49:08.593852
40735	ERC	2020-10-06	10.9	11.015	10.87	10.92	111019	{}	\N	2020-10-12 14:49:08.597714
40736	ERC	2020-10-07	10.99	11.04	10.95	11.03	78930	{}	\N	2020-10-12 14:49:08.602064
40737	ERC	2020-10-08	11.02	11.13	11.02	11.1	91980	{}	\N	2020-10-12 14:49:08.607042
40738	ERC	2020-10-09	11.14	11.1622	11.1	11.16	81323	{}	\N	2020-10-12 14:49:08.611055
40739	ERC	2020-10-12	11.1728	11.2	11.1606	11.19	18309	{}	\N	2020-10-12 14:49:08.61493
40740	ESE	2020-09-28	80.03	81.96	80.03	80.67	99925	{}	\N	2020-10-12 14:49:10.30703
40741	ESE	2020-09-29	80.77	82.02	80.51	81.04	244054	{}	\N	2020-10-12 14:49:10.312689
40742	ESE	2020-09-30	81.01	82.22	80.14	80.56	200632	{}	\N	2020-10-12 14:49:10.317116
40743	ESE	2020-10-01	80.93	81.92	79.45	81.76	213548	{}	\N	2020-10-12 14:49:10.321485
40744	ESE	2020-10-02	80.27	83.28	80.27	82.59	249761	{}	\N	2020-10-12 14:49:10.326607
40745	ESE	2020-10-05	83.21	84.685	83.21	84.44	197113	{}	\N	2020-10-12 14:49:10.332003
40746	ESE	2020-10-06	85.44	86	83.9	84.13	382325	{}	\N	2020-10-12 14:49:10.336902
40747	ESE	2020-10-07	84.94	85.87	83.76	84.35	281502	{}	\N	2020-10-12 14:49:10.341725
40748	ESE	2020-10-08	85.35	86.1	84.7937	85.2	112928	{}	\N	2020-10-12 14:49:10.346247
40749	ESE	2020-10-09	85.96	87.075	85.54	85.83	100979	{}	\N	2020-10-12 14:49:10.352118
40750	ESE	2020-10-12	85.83	86.31	85.83	86.29	10360	{}	\N	2020-10-12 14:49:10.360046
40751	ETN	2020-09-28	101.53	103.5167	101.2	102.34	1625445	{}	\N	2020-10-12 14:49:12.315778
40752	ETN	2020-09-29	102.49	102.49	100.87	101.18	2011302	{}	\N	2020-10-12 14:49:12.322755
40753	ETN	2020-09-30	101.71	103.65	100.95	102.03	3171626	{}	\N	2020-10-12 14:49:12.328013
40754	ETN	2020-10-01	102.71	103.205	99.66	100.59	2573155	{}	\N	2020-10-12 14:49:12.33272
40755	ETN	2020-10-02	99.13	104.21	98.985	102.65	3581936	{}	\N	2020-10-12 14:49:12.337241
40756	ETN	2020-10-05	103.88	105.47	103.6	105.09	2894231	{}	\N	2020-10-12 14:49:12.341544
40757	ETN	2020-10-06	105.31	106.79	104.33	104.44	2556426	{}	\N	2020-10-12 14:49:12.345798
40758	ETN	2020-10-07	105.79	107.13	105.455	106.95	2897285	{}	\N	2020-10-12 14:49:12.350288
40759	ETN	2020-10-08	107.29	107.92	106.23	107.48	1980307	{}	\N	2020-10-12 14:49:12.35646
40760	ETN	2020-10-09	108.46	109.2	107.3	107.53	2970000	{}	\N	2020-10-12 14:49:12.360614
40761	ETN	2020-10-12	108.61	109.63	108	108.7	596358	{}	\N	2020-10-12 14:49:12.365284
40762	EUDG	2020-09-28	27.6617	27.705	27.6259	27.6303	4424	{}	\N	2020-10-12 14:49:13.342703
40763	EUDG	2020-09-29	27.745	27.79	27.6565	27.76	12331	{}	\N	2020-10-12 14:49:13.408275
40764	EUDG	2020-09-30	27.595	27.749	27.5599	27.65	3836	{}	\N	2020-10-12 14:49:13.413202
40765	EUDG	2020-10-01	27.9	27.9	27.8373	27.865	2462	{}	\N	2020-10-12 14:49:13.420407
40766	EUDG	2020-10-02	27.73	27.91	27.73	27.9	4535	{}	\N	2020-10-12 14:49:13.428071
40767	EUDG	2020-10-05	28.1617	28.3	28.13	28.27	4489	{}	\N	2020-10-12 14:49:13.433492
40768	EUDG	2020-10-06	28.145	28.145	27.7434	27.7434	32245	{}	\N	2020-10-12 14:49:13.439374
40769	EUDG	2020-10-07	28.0305	28.0835	28.025	28.0835	2413	{}	\N	2020-10-12 14:49:13.443721
40770	EUDG	2020-10-08	28.1	28.19	28.0702	28.16	3247	{}	\N	2020-10-12 14:49:13.447537
40771	EUDG	2020-10-09	28.49	28.558	28.48	28.5467	35238	{}	\N	2020-10-12 14:49:13.454679
40772	EUDG	2020-10-12	28.6801	28.6961	28.68	28.6961	2669	{}	\N	2020-10-12 14:49:13.461222
40773	EURZ	2020-09-28	21.1477	21.1477	21.1477	21.1477	100	{}	\N	2020-10-12 14:49:14.788924
40774	EURZ	2020-09-29	21.1973	21.1973	21.1973	21.1973	1	{}	\N	2020-10-12 14:49:14.794183
40775	EURZ	2020-09-30	21.1178	21.1178	21.1178	21.1178	100	{}	\N	2020-10-12 14:49:14.798453
40776	EURZ	2020-10-01	21.2378	21.2378	21.2378	21.2378	100	{}	\N	2020-10-12 14:49:14.802401
40777	EURZ	2020-10-02	21.1932	21.1932	21.1932	21.1932	0	{}	\N	2020-10-12 14:49:14.806031
40778	EURZ	2020-10-05	21.6063	21.6063	21.6063	21.6063	15	{}	\N	2020-10-12 14:49:14.80983
40779	EURZ	2020-10-06	21.3879	21.3879	21.3879	21.3879	1	{}	\N	2020-10-12 14:49:14.813229
40780	EURZ	2020-10-07	21.6257	21.6257	21.6257	21.6257	0	{}	\N	2020-10-12 14:49:14.817062
40781	EURZ	2020-10-08	21.8039	21.8039	21.8039	21.8039	101	{}	\N	2020-10-12 14:49:14.82079
40782	EURZ	2020-10-09	21.972	21.972	21.972	21.972	100	{}	\N	2020-10-12 14:49:14.824253
40783	EVBN	2020-09-28	22.3	22.68	22.295	22.68	5355	{}	\N	2020-10-12 14:49:15.799267
40784	EVBN	2020-09-29	22.57	22.57	22.2	22.26	9023	{}	\N	2020-10-12 14:49:15.805448
40785	EVBN	2020-09-30	22.4758	22.525	22.25	22.25	3691	{}	\N	2020-10-12 14:49:15.810009
40786	EVBN	2020-10-01	22.2501	22.48	22.25	22.26	5695	{}	\N	2020-10-12 14:49:15.814419
40787	EVBN	2020-10-02	22.07	22.5	22.07	22.26	9748	{}	\N	2020-10-12 14:49:15.818737
40788	EVBN	2020-10-05	22.38	22.6	22.38	22.5	7280	{}	\N	2020-10-12 14:49:15.82255
40789	EVBN	2020-10-06	22.69	22.98	22.31	22.98	12710	{}	\N	2020-10-12 14:49:15.826553
40790	EVBN	2020-10-07	22.77	25.55	22.77	24.5	13734	{}	\N	2020-10-12 14:49:15.830526
40791	EVBN	2020-10-08	24.73	24.73	24.02	24.02	9858	{}	\N	2020-10-12 14:49:15.835104
40792	EVBN	2020-10-09	24.58	24.58	24.11	24.33	6431	{}	\N	2020-10-12 14:49:15.839142
40793	EVBN	2020-10-12	24.37	24.37	24	24.2	889	{}	\N	2020-10-12 14:49:15.842905
40794	EVI	2020-09-28	25.12	26.8	25.12	25.69	11308	{}	\N	2020-10-12 14:49:17.035832
40795	EVI	2020-09-29	25.82	26.11	25.56	26.11	4328	{}	\N	2020-10-12 14:49:17.041478
40796	EVI	2020-09-30	25.065	26.67	25	26.61	30301	{}	\N	2020-10-12 14:49:17.046265
40797	EVI	2020-10-01	26.37	28.56	25.815	27.3	30102	{}	\N	2020-10-12 14:49:17.050928
40798	EVI	2020-10-02	27.15	28.25	26.3	27.51	5762	{}	\N	2020-10-12 14:49:17.055131
40799	EVI	2020-10-05	27.36	28.58	27.2175	28.3589	13587	{}	\N	2020-10-12 14:49:17.05886
40800	EVI	2020-10-06	28.42	28.75	27.16	27.29	5974	{}	\N	2020-10-12 14:49:17.062419
40801	EVI	2020-10-07	27.39	28.3	26.87	28.21	5670	{}	\N	2020-10-12 14:49:17.066262
40802	EVI	2020-10-08	28.13	28.13	26.66	26.69	5119	{}	\N	2020-10-12 14:49:17.070132
40803	EVI	2020-10-09	27.28	27.92	26.27	26.97	9451	{}	\N	2020-10-12 14:49:17.073909
40804	EVI	2020-10-12	28.31	28.31	26.7176	27.608	1069	{}	\N	2020-10-12 14:49:17.077403
40805	EVT	2020-09-28	19.56	19.81	19.5049	19.64	139138	{}	\N	2020-10-12 14:49:18.54595
40806	EVT	2020-09-29	19.61	19.62	19.39	19.5	161495	{}	\N	2020-10-12 14:49:18.552245
40807	EVT	2020-09-30	19.62	19.88	19.55	19.7	456339	{}	\N	2020-10-12 14:49:18.557265
40808	EVT	2020-10-01	19.83	19.87	19.66	19.75	286419	{}	\N	2020-10-12 14:49:18.561457
40809	EVT	2020-10-02	19.34	19.79	19.34	19.75	326952	{}	\N	2020-10-12 14:49:18.56555
40810	EVT	2020-10-05	19.77	20.09	19.77	20	271997	{}	\N	2020-10-12 14:49:18.569443
40811	EVT	2020-10-06	20	20.16	19.79	19.8	321310	{}	\N	2020-10-12 14:49:18.573508
40812	EVT	2020-10-07	19.89	20.06	19.89	20	268510	{}	\N	2020-10-12 14:49:18.577541
40813	EVT	2020-10-08	20.1	20.27	19.9	20	658374	{}	\N	2020-10-12 14:49:18.581276
40814	EVT	2020-10-09	20.08	20.23	20	20.05	616839	{}	\N	2020-10-12 14:49:18.585207
40815	EVT	2020-10-12	20.1	20.18	20.05	20.1	120316	{}	\N	2020-10-12 14:49:18.589264
40816	FIZZ	2020-09-28	70.89	71.24	69.235	69.38	262203	{}	\N	2020-10-12 14:49:19.535369
40817	FIZZ	2020-09-29	69.39	69.6494	68.32	68.53	235238	{}	\N	2020-10-12 14:49:19.541901
40818	FIZZ	2020-09-30	68.71	70.58	67.945	68.01	291774	{}	\N	2020-10-12 14:49:19.546885
40819	FIZZ	2020-10-01	68.77	71.35	68.02	71.28	289612	{}	\N	2020-10-12 14:49:19.5511
40820	FIZZ	2020-10-02	69.86	71.34	69.67	69.96	193394	{}	\N	2020-10-12 14:49:19.555009
40821	FIZZ	2020-10-05	70.24	71.3409	70.224	70.96	177622	{}	\N	2020-10-12 14:49:19.558866
40822	FIZZ	2020-10-06	70.69	74.64	70.69	74.22	331719	{}	\N	2020-10-12 14:49:19.56245
40823	FIZZ	2020-10-07	75.58	78.22	74.72	76.5	857070	{}	\N	2020-10-12 14:49:19.566302
40824	FIZZ	2020-10-08	76.8	77.3	75.37	77.05	218827	{}	\N	2020-10-12 14:49:19.570056
40825	FIZZ	2020-10-09	77.87	79.56	77.3395	79.31	226768	{}	\N	2020-10-12 14:49:19.573534
40826	FIZZ	2020-10-12	79.99	80.22	78.62	78.83	53853	{}	\N	2020-10-12 14:49:19.57699
40827	ERII	2020-09-28	7.955	8.3	7.955	8.24	151453	{}	\N	2020-10-12 14:49:20.932384
40828	ERII	2020-09-29	8.22	8.23	8.03	8.17	158606	{}	\N	2020-10-12 14:49:20.938716
40829	ERII	2020-09-30	8.23	8.33	8.155	8.2	160577	{}	\N	2020-10-12 14:49:20.943758
40830	ERII	2020-10-01	8.29	8.38	8.12	8.34	125742	{}	\N	2020-10-12 14:49:20.948744
40831	ERII	2020-10-02	8.15	8.48	8.15	8.39	112956	{}	\N	2020-10-12 14:49:20.952927
40832	ERII	2020-10-05	8.4	8.66	8.4	8.61	106589	{}	\N	2020-10-12 14:49:20.957288
40833	ERII	2020-10-06	8.67	8.68	8.45	8.47	186765	{}	\N	2020-10-12 14:49:20.961345
40834	ERII	2020-10-07	8.55	8.74	8.55	8.68	150632	{}	\N	2020-10-12 14:49:20.965495
40835	ERII	2020-10-08	8.73	8.77	8.67	8.74	81752	{}	\N	2020-10-12 14:49:20.969582
40836	ERII	2020-10-09	8.86	8.9	8.74	8.84	211036	{}	\N	2020-10-12 14:49:20.9738
40837	ERII	2020-10-12	8.9	9.2	8.85	9.16	71853	{}	\N	2020-10-12 14:49:20.978133
40838	ES	2020-09-28	83	83.74	81.915	82.65	1312049	{}	\N	2020-10-12 14:49:22.692457
40839	ES	2020-09-29	82.94	83.67	82.61	83.01	1558339	{}	\N	2020-10-12 14:49:22.700076
40840	ES	2020-09-30	83.49	84.17	82.64	83.55	3012741	{}	\N	2020-10-12 14:49:22.705317
40841	ES	2020-10-01	83.84	85.48	83.6	85.45	3070846	{}	\N	2020-10-12 14:49:22.710211
40842	ES	2020-10-02	84.93	87.63	84.64	87.05	3256219	{}	\N	2020-10-12 14:49:22.714782
40843	ES	2020-10-05	87.42	89.56	86.83	89.49	3632465	{}	\N	2020-10-12 14:49:22.719335
40844	ES	2020-10-06	89.49	91.37	88.7419	90.78	3291194	{}	\N	2020-10-12 14:49:22.723806
40845	ES	2020-10-07	91.37	91.79	90.26	91.4	2511729	{}	\N	2020-10-12 14:49:22.72836
40846	ES	2020-10-08	91.54	92.95	91.24	92.43	2318929	{}	\N	2020-10-12 14:49:22.732801
40847	ES	2020-10-09	92.75	92.99	91.48	92.19	2483666	{}	\N	2020-10-12 14:49:22.73737
40848	ES	2020-10-12	92.17	93.18	92.17	93.15	247648	{}	\N	2020-10-12 14:49:22.741884
40849	ESLT	2020-09-28	122.48	122.48	121.05	121.19	13423	{}	\N	2020-10-12 14:49:25.14725
40850	ESLT	2020-09-29	119.47	119.69	118.23	118.35	26828	{}	\N	2020-10-12 14:49:25.154484
40851	ESLT	2020-09-30	119.36	121.54	119.36	120.3	29066	{}	\N	2020-10-12 14:49:25.159256
40852	ESLT	2020-10-01	121.57	121.9	120.78	121.02	17794	{}	\N	2020-10-12 14:49:25.163645
40853	ESLT	2020-10-02	119.55	121.5569	119.55	120.96	11058	{}	\N	2020-10-12 14:49:25.169444
40854	ESLT	2020-10-05	124	125.25	124	125.25	17248	{}	\N	2020-10-12 14:49:25.174625
40855	ESLT	2020-10-06	124.91	125.115	123	123	10811	{}	\N	2020-10-12 14:49:25.178605
40856	ESLT	2020-10-07	125.09	125.36	124.66	125.09	9012	{}	\N	2020-10-12 14:49:25.183106
40857	ESLT	2020-10-08	125.31	125.46	124.4	125.3	20111	{}	\N	2020-10-12 14:49:25.187336
40858	ESLT	2020-10-09	126	126.5	124.01	124.8	48768	{}	\N	2020-10-12 14:49:25.191397
40859	ESLT	2020-10-12	124.07	124.62	123.84	124.41	6561	{}	\N	2020-10-12 14:49:25.195244
40860	ESQ	2020-09-28	14.56	15.4999	14	14.46	16819	{}	\N	2020-10-12 14:49:27.117319
40861	ESQ	2020-09-29	14.58	14.75	14.48	14.57	35612	{}	\N	2020-10-12 14:49:27.125804
40862	ESQ	2020-09-30	14.63	15.08	14.54	15	14582	{}	\N	2020-10-12 14:49:27.130608
40863	ESQ	2020-10-01	15.1	15.88	15.1	15.88	5515	{}	\N	2020-10-12 14:49:27.135678
40864	ESQ	2020-10-02	15.36	15.43	15.115	15.43	4057	{}	\N	2020-10-12 14:49:27.139965
40865	ESQ	2020-10-05	15.51	15.51	15.51	15.51	2712	{}	\N	2020-10-12 14:49:27.144116
40866	ESQ	2020-10-06	16.02	16.34	15.765	15.81	40750	{}	\N	2020-10-12 14:49:27.148382
40867	ESQ	2020-10-07	15.81	16.55	15.81	16.55	7693	{}	\N	2020-10-12 14:49:27.152976
40868	ESQ	2020-10-08	16.5	17.79	16.5	17.69	10867	{}	\N	2020-10-12 14:49:27.157061
40869	ESQ	2020-10-09	18.04	18.5624	15.76	16.7	36677	{}	\N	2020-10-12 14:49:27.161179
40870	ESQ	2020-10-12	16.62	16.62	16.62	16.62	194	{}	\N	2020-10-12 14:49:27.1657
40871	ETFC	2020-09-28	49.59	50.83	49.59	50.1	1333696	{}	\N	2020-10-12 14:49:30.047107
40872	ETFC	2020-09-29	49.83	49.96	48.88	48.93	1526362	{}	\N	2020-10-12 14:49:30.053093
40873	ETFC	2020-09-30	49.24	50.7	48.98	50.05	2447681	{}	\N	2020-10-12 14:49:30.057676
40874	ETFC	2020-10-01	50.76	51.18	48.91	49.26	20256683	{}	\N	2020-10-12 14:49:30.061852
40875	ETSY	2020-09-28	120.37	123.95	119.19	123.69	3351874	{}	\N	2020-10-12 14:49:31.148719
40876	ETSY	2020-09-29	123.61	125.7	121.26	123.23	2546906	{}	\N	2020-10-12 14:49:31.157645
40877	ETSY	2020-09-30	123.19	125.59	120.14	121.63	2361710	{}	\N	2020-10-12 14:49:31.162598
40878	ETSY	2020-10-01	124.1	133.09	123.25	131.69	5624009	{}	\N	2020-10-12 14:49:31.168813
40879	ETSY	2020-10-02	127.5	133.33	127.27	131.63	3588232	{}	\N	2020-10-12 14:49:31.175118
40880	ETSY	2020-10-05	132.69	136.84	130.7	136.69	2735379	{}	\N	2020-10-12 14:49:31.179526
40881	ETSY	2020-10-06	137	139.26	134.1	134.56	2907273	{}	\N	2020-10-12 14:49:31.184152
40882	ETSY	2020-10-07	136.63	143.1	136.45	142.66	3788081	{}	\N	2020-10-12 14:49:31.188555
40883	ETSY	2020-10-08	145.01	145.15	139.16	139.54	3134859	{}	\N	2020-10-12 14:49:31.192785
40884	ETSY	2020-10-09	140.7	147.3	140.07	146.67	3320030	{}	\N	2020-10-12 14:49:31.196883
40885	ETSY	2020-10-12	148.96	150.75	146.9602	150.4999	1204439	{}	\N	2020-10-12 14:49:31.201323
40886	EUFX	2020-09-28	43.305	43.305	43.305	43.305	201	{}	\N	2020-10-12 14:49:33.215776
40887	EUFX	2020-09-29	43.1008	43.1008	43.02	43.02	1398	{}	\N	2020-10-12 14:49:33.222478
40888	EUFX	2020-09-30	43.275	43.275	43.275	43.275	0	{}	\N	2020-10-12 14:49:33.227621
40889	EUFX	2020-10-01	43.235	43.235	43.235	43.235	0	{}	\N	2020-10-12 14:49:33.232451
40890	EUFX	2020-10-02	44.04	44.04	43.32	43.32	912	{}	\N	2020-10-12 14:49:33.238095
40891	EUFX	2020-10-05	43.16	43.6	43.16	43.54	7988	{}	\N	2020-10-12 14:49:33.24321
40892	EUFX	2020-10-06	43.5399	43.7	43.44	43.68	2390	{}	\N	2020-10-12 14:49:33.248079
40893	EUFX	2020-10-07	43.6257	43.6257	43.6257	43.6257	199	{}	\N	2020-10-12 14:49:33.252594
40894	EUFX	2020-10-08	43.68	43.68	43.68	43.68	1	{}	\N	2020-10-12 14:49:33.257287
40895	EUFX	2020-10-09	43.47	43.4928	43.39	43.4928	1808	{}	\N	2020-10-12 14:49:33.261671
40896	EUFX	2020-10-12	43.43	43.43	43.43	43.43	353	{}	\N	2020-10-12 14:49:33.267024
40897	EV	2020-09-28	37.57	38.46	37.47	38.42	398952	{}	\N	2020-10-12 14:49:34.349792
40898	EV	2020-09-29	38.37	38.37	37.44	38.1	1024170	{}	\N	2020-10-12 14:49:34.360499
40899	EV	2020-09-30	38.44	38.8896	37.52	38.15	1080973	{}	\N	2020-10-12 14:49:34.366039
40900	EV	2020-10-01	39.23	39.63	38.14	38.51	906221	{}	\N	2020-10-12 14:49:34.370658
40901	EV	2020-10-02	37.94	39.51	37.94	39.36	1219547	{}	\N	2020-10-12 14:49:34.375003
40902	EV	2020-10-05	39.83	40.35	39.665	40.21	1166550	{}	\N	2020-10-12 14:49:34.378818
40903	EV	2020-10-06	40.53	41.29	39.895	40.16	1316230	{}	\N	2020-10-12 14:49:34.383212
40904	EV	2020-10-07	40.58	41.28	40.445	40.94	1020129	{}	\N	2020-10-12 14:49:34.387033
40905	EV	2020-10-08	60.11	60.85	59.04	60.65	29334357	{}	\N	2020-10-12 14:49:34.390777
40906	EV	2020-10-09	60.77	61.25	60.23	60.31	11753051	{}	\N	2020-10-12 14:49:34.394569
40907	EV	2020-10-12	60.39	61.04	60.31	61.02	3123737	{}	\N	2020-10-12 14:49:34.398653
40908	EVN	2020-09-28	12.71	12.77	12.71	12.74	66407	{}	\N	2020-10-12 14:49:36.424252
40909	EVN	2020-09-29	12.75	12.765	12.72	12.74	111508	{}	\N	2020-10-12 14:49:36.429323
40910	EVN	2020-09-30	12.77	12.81	12.73	12.81	115384	{}	\N	2020-10-12 14:49:36.434143
40911	EVN	2020-10-01	12.85	12.85	12.74	12.8	79130	{}	\N	2020-10-12 14:49:36.439183
40912	EVN	2020-10-02	12.8	12.805	12.7	12.74	138042	{}	\N	2020-10-12 14:49:36.444204
40913	EVN	2020-10-05	12.7	12.73	12.65	12.72	124852	{}	\N	2020-10-12 14:49:36.450063
40914	EVN	2020-10-06	12.73	12.79	12.7	12.75	361096	{}	\N	2020-10-12 14:49:36.454553
40915	EVN	2020-10-07	12.79	12.83	12.7309	12.77	172582	{}	\N	2020-10-12 14:49:36.459154
40916	EVN	2020-10-08	12.79	12.7995	12.72	12.78	195422	{}	\N	2020-10-12 14:49:36.463981
40917	EVN	2020-10-09	12.76	12.76	12.68	12.69	305040	{}	\N	2020-10-12 14:49:36.468833
40918	EVN	2020-10-12	12.75	12.76	12.72	12.76	24844	{}	\N	2020-10-12 14:49:36.473401
40919	EVV	2020-09-28	11.38	11.43	11.31	11.35	232281	{}	\N	2020-10-12 14:49:37.992272
40920	EVV	2020-09-29	11.355	11.3623	11.28	11.32	213682	{}	\N	2020-10-12 14:49:37.999061
40921	EVV	2020-09-30	11.34	11.38	11.31	11.32	336085	{}	\N	2020-10-12 14:49:38.005234
40922	EVV	2020-10-01	11.32	11.38	11.31	11.38	181068	{}	\N	2020-10-12 14:49:38.009821
40923	EVV	2020-10-02	11.3	11.43	11.2885	11.41	270733	{}	\N	2020-10-12 14:49:38.015986
40924	EVV	2020-10-05	11.43	11.5	11.4101	11.47	227494	{}	\N	2020-10-12 14:49:38.021059
40925	EVV	2020-10-06	11.45	11.58	11.45	11.53	222821	{}	\N	2020-10-12 14:49:38.027132
40926	EVV	2020-10-07	11.57	11.71	11.57	11.67	243777	{}	\N	2020-10-12 14:49:38.033323
40927	EVV	2020-10-08	11.74	11.87	11.72	11.87	597459	{}	\N	2020-10-12 14:49:38.03871
40928	EVV	2020-10-09	11.85	11.85	11.62	11.75	252596	{}	\N	2020-10-12 14:49:38.042794
40929	EVV	2020-10-12	11.76	11.8	11.73	11.7884	80170	{}	\N	2020-10-12 14:49:38.047132
40930	ERIC	2020-09-28	10.65	10.715	10.63	10.71	6601982	{}	\N	2020-10-12 14:49:39.941564
40931	ERIC	2020-09-29	10.83	10.92	10.81	10.88	5415849	{}	\N	2020-10-12 14:49:39.94845
40932	ERIC	2020-09-30	10.97	11.015	10.85	10.89	5210250	{}	\N	2020-10-12 14:49:39.953529
40933	ERIC	2020-10-01	10.77	10.8	10.64	10.72	6123833	{}	\N	2020-10-12 14:49:39.959
40934	ERIC	2020-10-02	10.51	10.655	10.5	10.61	5186301	{}	\N	2020-10-12 14:49:39.963598
40935	ERIC	2020-10-05	10.76	10.79	10.7	10.77	4450842	{}	\N	2020-10-12 14:49:39.967862
40936	ERIC	2020-10-06	10.84	10.855	10.62	10.67	4569850	{}	\N	2020-10-12 14:49:39.972299
40937	ERIC	2020-10-07	10.66	10.7	10.61	10.65	5985794	{}	\N	2020-10-12 14:49:39.976397
40938	ERIC	2020-10-08	10.8	10.8	10.71	10.78	5128893	{}	\N	2020-10-12 14:49:39.980622
40939	ERIC	2020-10-09	10.89	10.9299	10.84	10.875	4427806	{}	\N	2020-10-12 14:49:39.985301
40940	ERIC	2020-10-12	10.91	10.93	10.84	10.8499	2718817	{}	\N	2020-10-12 14:49:39.98961
40941	ESEA	2020-09-28	2.37	2.445	2.3	2.37	87848	{}	\N	2020-10-12 14:49:41.170143
40942	ESEA	2020-09-29	2.42	2.6	2.345	2.4	249338	{}	\N	2020-10-12 14:49:41.178678
40943	ESEA	2020-09-30	2.45	2.54	2.4	2.47	126966	{}	\N	2020-10-12 14:49:41.185493
40944	ESEA	2020-10-01	2.53	2.8299	2.5	2.72	256325	{}	\N	2020-10-12 14:49:41.190542
40945	ESEA	2020-10-02	2.52	2.73	2.52	2.65	69494	{}	\N	2020-10-12 14:49:41.195282
40946	ESEA	2020-10-05	2.73	2.78	2.53	2.68	197017	{}	\N	2020-10-12 14:49:41.200105
40947	ESEA	2020-10-06	2.6522	2.87	2.54	2.63	273586	{}	\N	2020-10-12 14:49:41.204632
40948	ESEA	2020-10-07	2.67	2.92	2.61	2.7	287142	{}	\N	2020-10-12 14:49:41.209103
40949	ESEA	2020-10-08	2.74	2.85	2.7101	2.85	69297	{}	\N	2020-10-12 14:49:41.213602
40950	ESEA	2020-10-09	2.81	3.25	2.81	2.96	782568	{}	\N	2020-10-12 14:49:41.217973
40951	ESEA	2020-10-12	3.2	3.2799	3.024	3.1127	218778	{}	\N	2020-10-12 14:49:41.22219
45115	GBAB	2020-09-28	23.79	23.8	23.64	23.7	22312	{}	\N	2020-10-12 14:59:43.996951
45116	GBAB	2020-09-29	23.75	23.75	23.56	23.68	93786	{}	\N	2020-10-12 14:59:44.012738
45117	GBAB	2020-09-30	23.82	23.83	23.57	23.57	88617	{}	\N	2020-10-12 14:59:44.019076
45118	GBAB	2020-10-01	23.59	23.79	23.565	23.79	51210	{}	\N	2020-10-12 14:59:44.024313
45119	GBAB	2020-10-02	23.84	24.08	23.7476	24.03	67408	{}	\N	2020-10-12 14:59:44.029291
45120	GBAB	2020-10-05	24.05	24.05	23.8614	23.96	115610	{}	\N	2020-10-12 14:59:44.034493
45121	GBAB	2020-10-06	23.97	24.1	23.903	24.04	67735	{}	\N	2020-10-12 14:59:44.039437
45122	GBAB	2020-10-07	24.04	24.28	24.02	24.26	73948	{}	\N	2020-10-12 14:59:44.044423
45123	GBAB	2020-10-08	24.42	24.57	24.26	24.57	93202	{}	\N	2020-10-12 14:59:44.049239
45124	GBAB	2020-10-09	24.51	24.599	24.08	24.45	79286	{}	\N	2020-10-12 14:59:44.054622
45125	GBAB	2020-10-12	24.4	24.5	24.27	24.372	23182	{}	\N	2020-10-12 14:59:44.061262
45126	GBT	2020-09-28	54.87	55.44	53.36	53.63	692816	{}	\N	2020-10-12 14:59:45.295442
45127	GBT	2020-09-29	53.31	54.77	53.26	53.94	421127	{}	\N	2020-10-12 14:59:45.306887
45128	GBT	2020-09-30	54.17	55.63	53.4531	55.14	599074	{}	\N	2020-10-12 14:59:45.314961
45129	GBT	2020-10-01	55.445	58.27	55.3394	57.96	846463	{}	\N	2020-10-12 14:59:45.320958
45130	GBT	2020-10-02	57.2	58.73	56.5001	57	670175	{}	\N	2020-10-12 14:59:45.325907
45131	GBT	2020-10-05	56.81	63.05	56.81	61.79	1041542	{}	\N	2020-10-12 14:59:45.33101
45132	GBT	2020-10-06	63.05	64.09	60.87	61.88	854536	{}	\N	2020-10-12 14:59:45.335557
45133	GBT	2020-10-07	61.94	64.23	61.7746	63.77	519996	{}	\N	2020-10-12 14:59:45.339741
45134	GBT	2020-10-08	64.43	64.489	61.1	61.55	732714	{}	\N	2020-10-12 14:59:45.344056
45135	GBT	2020-10-09	61.81	63.92	61.02	62.36	685203	{}	\N	2020-10-12 14:59:45.348911
45136	GBT	2020-10-12	62.31	62.84	60.01	60.34	122438	{}	\N	2020-10-12 14:59:45.354731
45137	GCV	2020-09-28	5.28	5.49	5.28	5.395	39304	{}	\N	2020-10-12 14:59:46.412387
45138	GCV	2020-09-29	5.49	5.49	5.36	5.385	22196	{}	\N	2020-10-12 14:59:46.424381
45139	GCV	2020-09-30	5.38	5.49	5.33	5.4	54224	{}	\N	2020-10-12 14:59:46.431669
45140	GCV	2020-10-01	5.4	5.48	5.4	5.44	54564	{}	\N	2020-10-12 14:59:46.438173
45141	GCV	2020-10-02	5.39	5.47	5.39	5.46	7732	{}	\N	2020-10-12 14:59:46.443004
45142	GCV	2020-10-05	5.5363	5.5363	5.43	5.445	53265	{}	\N	2020-10-12 14:59:46.447458
45143	GCV	2020-10-06	5.41	5.522	5.4	5.43	75510	{}	\N	2020-10-12 14:59:46.452561
45144	GCV	2020-10-07	5.41	5.62	5.41	5.5	67352	{}	\N	2020-10-12 14:59:46.457421
45145	GCV	2020-10-08	5.54	5.57	5.5	5.5202	106890	{}	\N	2020-10-12 14:59:46.462317
45146	GCV	2020-10-09	5.5224	5.5622	5.5224	5.56	55356	{}	\N	2020-10-12 14:59:46.467111
45147	GCV	2020-10-12	5.59	5.6049	5.58	5.58	12370	{}	\N	2020-10-12 14:59:46.472177
45148	GDVD	2020-09-28	28.37	28.58	28.349	28.349	2434	{}	\N	2020-10-12 14:59:47.809186
45149	GDVD	2020-09-29	28.3014	28.3267	28.29	28.3267	1109	{}	\N	2020-10-12 14:59:47.816988
45150	GDVD	2020-09-30	28.44	28.53	28.4207	28.4207	1685	{}	\N	2020-10-12 14:59:47.823231
45151	GDVD	2020-10-01	28.12	28.26	28.0301	28.0301	8993	{}	\N	2020-10-12 14:59:47.828813
45152	GDVD	2020-10-02	28.13	28.2789	28.13	28.2789	937	{}	\N	2020-10-12 14:59:47.835199
45153	GDVD	2020-10-05	28.46	28.58	28.449	28.58	3747	{}	\N	2020-10-12 14:59:47.840852
45154	GDVD	2020-10-06	28.54	28.88	28.4255	28.4255	4139	{}	\N	2020-10-12 14:59:47.845214
45155	GDVD	2020-10-07	28.525	28.708	28.51	28.708	927	{}	\N	2020-10-12 14:59:47.849478
45156	GDVD	2020-10-08	28.88	29.0832	28.82	29.0832	1003	{}	\N	2020-10-12 14:59:47.854226
45157	GDVD	2020-10-09	29.155	29.41	29.14	29.3202	2291	{}	\N	2020-10-12 14:59:47.859155
40952	ESGU	2020-09-28	75.89	76.2278	75.61	76.08	247414	{}	\N	2020-10-12 14:49:42.127381
40953	ESGU	2020-09-29	76.11	76.18	75.5529	75.67	329567	{}	\N	2020-10-12 14:49:42.148945
40954	ESGU	2020-09-30	75.86	76.995	75.8103	76.21	533219	{}	\N	2020-10-12 14:49:42.155834
40955	ESGU	2020-10-01	76.9	77.08	76.33	76.7	414718	{}	\N	2020-10-12 14:49:42.16163
40956	ESGU	2020-10-02	75.87	76.49	75.38	76.05	387836	{}	\N	2020-10-12 14:49:42.167714
40957	ESGU	2020-10-05	76.59	77.42	76.59	77.37	269959	{}	\N	2020-10-12 14:49:42.17522
40958	ESGU	2020-10-06	77.42	77.98	76.2166	76.37	759991	{}	\N	2020-10-12 14:49:42.186901
40959	ESGU	2020-10-07	77.18	77.89	77.1094	77.72	16265019	{}	\N	2020-10-12 14:49:42.194468
40960	ESGU	2020-10-08	78.22	78.41	77.98	78.38	2487861	{}	\N	2020-10-12 14:49:42.200529
40961	ESGU	2020-10-09	78.87	79.22	78.65	79.11	2362039	{}	\N	2020-10-12 14:49:42.206134
40962	ESGU	2020-10-12	79.82	79.92	79.66	79.87	48195	{}	\N	2020-10-12 14:49:42.211805
40963	ESPR	2020-09-28	37.84	37.84	36.93	37.12	259324	{}	\N	2020-10-12 14:49:43.126489
40964	ESPR	2020-09-29	36.4	37.66	35.51	37.46	297622	{}	\N	2020-10-12 14:49:43.131325
40965	ESPR	2020-09-30	37.22	37.97	36.79	37.19	425918	{}	\N	2020-10-12 14:49:43.135366
40966	ESPR	2020-10-01	37.38	37.9686	35.66	35.76	380372	{}	\N	2020-10-12 14:49:43.139349
40967	ESPR	2020-10-02	35.2	35.76	34.1	34.18	404701	{}	\N	2020-10-12 14:49:43.143833
40968	ESPR	2020-10-05	34.56	36.1781	34.56	35.55	357365	{}	\N	2020-10-12 14:49:43.14868
40969	ESPR	2020-10-06	35.71	35.97	34.5	34.69	407976	{}	\N	2020-10-12 14:49:43.153909
40970	ESPR	2020-10-07	34.62	35.425	34.3	34.9	457589	{}	\N	2020-10-12 14:49:43.158305
40971	ESPR	2020-10-08	35.23	35.51	33.44	33.74	504143	{}	\N	2020-10-12 14:49:43.162489
40972	ESPR	2020-10-09	33.82	34.045	32.75	33.81	568079	{}	\N	2020-10-12 14:49:43.166639
40973	ESPR	2020-10-12	33.81	33.97	33	33.385	86016	{}	\N	2020-10-12 14:49:43.171341
40974	ESXB	2020-09-28	5.12	5.2	5.06	5.08	24993	{}	\N	2020-10-12 14:49:45.296411
40975	ESXB	2020-09-29	5.05	5.08	4.94	5.04	14197	{}	\N	2020-10-12 14:49:45.30507
40976	ESXB	2020-09-30	5.04	5.14	5.04	5.08	11273	{}	\N	2020-10-12 14:49:45.311223
40977	ESXB	2020-10-01	5.08	5.13	5.04	5.1	23731	{}	\N	2020-10-12 14:49:45.316476
40978	ESXB	2020-10-02	5.02	5.26	5.02	5.16	33744	{}	\N	2020-10-12 14:49:45.321202
40979	ESXB	2020-10-05	5.27	5.27	5.14	5.18	58181	{}	\N	2020-10-12 14:49:45.326111
40980	ESXB	2020-10-06	5.23	5.34	5.14	5.14	38007	{}	\N	2020-10-12 14:49:45.332425
40981	ESXB	2020-10-07	5.18	5.3	5.16	5.25	24602	{}	\N	2020-10-12 14:49:45.336957
40982	ESXB	2020-10-08	5.3	5.34	5.14	5.29	34869	{}	\N	2020-10-12 14:49:45.341473
40983	ESXB	2020-10-09	5.24	5.32	5.23	5.29	72195	{}	\N	2020-10-12 14:49:45.3471
40984	ESXB	2020-10-12	5.28	5.32	5.28	5.3	5490	{}	\N	2020-10-12 14:49:45.351803
40985	ETV	2020-09-28	13.8	13.87	13.7871	13.83	227967	{}	\N	2020-10-12 14:49:46.561129
40986	ETV	2020-09-29	13.84	13.85	13.73	13.83	358548	{}	\N	2020-10-12 14:49:46.567678
40987	ETV	2020-09-30	13.94	13.99	13.83	13.83	595397	{}	\N	2020-10-12 14:49:46.572606
40988	ETV	2020-10-01	13.93	13.99	13.83	13.92	525922	{}	\N	2020-10-12 14:49:46.577792
40989	ETV	2020-10-02	13.7	13.8684	13.68	13.71	728729	{}	\N	2020-10-12 14:49:46.582215
40990	ETV	2020-10-05	13.75	13.86	13.75	13.82	397786	{}	\N	2020-10-12 14:49:46.586987
40991	ETV	2020-10-06	13.85	13.92	13.7	13.73	350555	{}	\N	2020-10-12 14:49:46.591292
40992	ETV	2020-10-07	13.8	13.97	13.8	13.95	495422	{}	\N	2020-10-12 14:49:46.595757
40993	ETV	2020-10-08	14.03	14.09	13.99	14.04	297176	{}	\N	2020-10-12 14:49:46.600627
40994	ETV	2020-10-09	14.04	14.11	13.97	14.02	400380	{}	\N	2020-10-12 14:49:46.605524
40995	ETV	2020-10-12	14.11	14.12	14.06	14.1	140644	{}	\N	2020-10-12 14:49:46.609926
40996	EVBG	2020-09-28	126.56	130.7125	124.37	130.45	368906	{}	\N	2020-10-12 14:49:49.196889
40997	EVBG	2020-09-29	129.71	131.9	126.62	127.2	300371	{}	\N	2020-10-12 14:49:49.203274
40998	EVBG	2020-09-30	126.88	128.805	124.59	125.73	288776	{}	\N	2020-10-12 14:49:49.20757
40999	EVBG	2020-10-01	126.095	129.8462	125.03	128.12	431097	{}	\N	2020-10-12 14:49:49.211978
41000	EVBG	2020-10-02	125.22	127.645	119.65	121.53	548816	{}	\N	2020-10-12 14:49:49.21798
41001	EVBG	2020-10-05	123	125.61	119.145	123.64	991344	{}	\N	2020-10-12 14:49:49.22303
41002	EVBG	2020-10-06	124.99	127.3499	118.61	120.23	442859	{}	\N	2020-10-12 14:49:49.228085
41003	EVBG	2020-10-07	121.89	122.68	118.11	120.58	393514	{}	\N	2020-10-12 14:49:49.234081
41004	EVBG	2020-10-08	122	122.86	117.02	118.36	393616	{}	\N	2020-10-12 14:49:49.238998
41005	EVBG	2020-10-09	119.6199	126.54	117.7701	126.5	466430	{}	\N	2020-10-12 14:49:49.243772
41006	EVBG	2020-10-12	128.6	130.88	126.285	130.38	97082	{}	\N	2020-10-12 14:49:49.248339
41007	EVK	2020-09-28	0.95	0.9576	0.82	0.839	21696	{}	\N	2020-10-12 14:49:50.14084
41008	EVK	2020-09-29	0.838	0.9	0.82	0.895	77450	{}	\N	2020-10-12 14:49:50.147428
41009	EVK	2020-09-30	0.8205	0.8776	0.82	0.857799	7670	{}	\N	2020-10-12 14:49:50.152027
41010	EVK	2020-10-01	0.82	0.874	0.8067	0.8116	4676	{}	\N	2020-10-12 14:49:50.156294
41011	EVK	2020-10-02	0.846	0.846	0.801187	0.81	563	{}	\N	2020-10-12 14:49:50.160191
41012	EVK	2020-10-05	0.84	0.888	0.84	0.860601	4399	{}	\N	2020-10-12 14:49:50.164336
41013	EVK	2020-10-06	0.89	0.949	0.85	0.8937	62633	{}	\N	2020-10-12 14:49:50.16884
41014	EVK	2020-10-07	0.9383	0.9383	0.8543	0.9	1104	{}	\N	2020-10-12 14:49:50.173688
41015	EVK	2020-10-08	0.9	0.936	0.9	0.9027	2977	{}	\N	2020-10-12 14:49:50.177858
41016	EVK	2020-10-09	0.935	0.935	0.935	0.935	309	{}	\N	2020-10-12 14:49:50.181766
41017	EVK	2020-10-12	0.925	0.925	0.91	0.92	5276	{}	\N	2020-10-12 14:49:50.185805
41018	EVRI	2020-09-28	8.45	8.76	8.26	8.73	1888838	{}	\N	2020-10-12 14:49:51.40774
41019	EVRI	2020-09-29	8.69	8.69	8.185	8.27	3796968	{}	\N	2020-10-12 14:49:51.414417
41020	EVRI	2020-09-30	8.32	8.68	8.16	8.25	3934286	{}	\N	2020-10-12 14:49:51.420483
41021	EVRI	2020-10-01	8.43	8.51	8.33	8.5	2751831	{}	\N	2020-10-12 14:49:51.426129
41022	EVRI	2020-10-02	8.04	8.665	8.04	8.55	2812784	{}	\N	2020-10-12 14:49:51.430604
41023	EVRI	2020-10-05	8.6	8.66	8.37	8.65	3176265	{}	\N	2020-10-12 14:49:51.435015
41024	EVRI	2020-10-06	8.74	9.21	8.53	8.59	6849209	{}	\N	2020-10-12 14:49:51.439872
41025	EVRI	2020-10-07	8.82	8.9099	8.49	8.5	3897526	{}	\N	2020-10-12 14:49:51.444255
41026	EVRI	2020-10-08	8.57	8.6453	8.4	8.55	1978336	{}	\N	2020-10-12 14:49:51.448969
41027	EVRI	2020-10-09	8.6	8.7	8.41	8.54	2147967	{}	\N	2020-10-12 14:49:51.453264
41028	EVRI	2020-10-12	8.63	9.21	8.61	9.18	2161191	{}	\N	2020-10-12 14:49:51.457491
41029	EWA	2020-09-28	20.02	20.07	19.94	20.06	4684298	{}	\N	2020-10-12 14:49:52.439069
41030	EWA	2020-09-29	19.88	19.93	19.765	19.85	3371697	{}	\N	2020-10-12 14:49:52.448227
41031	EWA	2020-09-30	19.7	19.89	19.65	19.72	2670099	{}	\N	2020-10-12 14:49:52.452943
41032	EWA	2020-10-01	19.94	19.94	19.75	19.88	3066784	{}	\N	2020-10-12 14:49:52.458099
41033	EWA	2020-10-02	19.61	19.89	19.57	19.85	2765936	{}	\N	2020-10-12 14:49:52.461939
41034	EWA	2020-10-05	20.11	20.26	20.09	20.25	2013280	{}	\N	2020-10-12 14:49:52.466012
41035	EWA	2020-10-06	20.24	20.29	19.92	20	3424837	{}	\N	2020-10-12 14:49:52.470425
41036	EWA	2020-10-07	20.34	20.45	20.3	20.39	2993689	{}	\N	2020-10-12 14:49:52.474672
41037	EWA	2020-10-08	20.58	20.69	20.545	20.69	9896136	{}	\N	2020-10-12 14:49:52.478901
41038	EWA	2020-10-09	20.78	20.85	20.75	20.82	2379412	{}	\N	2020-10-12 14:49:52.482844
41039	EWA	2020-10-12	20.88	20.91	20.85	20.905	268168	{}	\N	2020-10-12 14:49:52.48719
41040	EWN	2020-09-28	34.74	34.86	34.69	34.81	94835	{}	\N	2020-10-12 14:49:53.840607
41041	EWN	2020-09-29	34.89	35.08	34.79	34.91	176382	{}	\N	2020-10-12 14:49:53.848157
41042	EWN	2020-09-30	34.9	35.145	34.74	34.83	166537	{}	\N	2020-10-12 14:49:53.854196
41043	EWN	2020-10-01	35.27	35.45	35.11	35.42	266717	{}	\N	2020-10-12 14:49:53.858867
41044	EWN	2020-10-02	35.09	35.46	35.08	35.38	246338	{}	\N	2020-10-12 14:49:53.863463
41045	EWN	2020-10-05	35.77	36.01	35.76	36.01	181876	{}	\N	2020-10-12 14:49:53.867748
41046	EWN	2020-10-06	35.81	35.83	35.25	35.33	164676	{}	\N	2020-10-12 14:49:53.872113
41047	EWN	2020-10-07	35.67	35.79	35.65	35.75	171423	{}	\N	2020-10-12 14:49:53.876728
41048	EWN	2020-10-08	35.83	36.11	35.82	36.04	1044762	{}	\N	2020-10-12 14:49:53.88153
41049	EWN	2020-10-09	36.42	36.72	36.42	36.7	156858	{}	\N	2020-10-12 14:49:53.886427
41050	EWN	2020-10-12	36.92	37.03	36.9	37.02	14811	{}	\N	2020-10-12 14:49:53.890815
41051	ESRT	2020-09-28	6.14	6.35	6.12	6.24	1668983	{}	\N	2020-10-12 14:49:55.304974
41052	ESRT	2020-09-29	6.23	6.24	6.01	6.09	2822060	{}	\N	2020-10-12 14:49:55.310761
41053	ESRT	2020-09-30	6.1	6.3034	6.03	6.12	4125613	{}	\N	2020-10-12 14:49:55.315971
41054	ESRT	2020-10-01	6.15	6.39	6.15	6.38	4516962	{}	\N	2020-10-12 14:49:55.320384
41055	ESRT	2020-10-02	6.23	6.58	6.15	6.58	2509456	{}	\N	2020-10-12 14:49:55.325055
41056	ESRT	2020-10-05	6.63	6.68	6.47	6.6	3039980	{}	\N	2020-10-12 14:49:55.330086
41057	ESRT	2020-10-06	6.66	6.74	6.515	6.58	2842098	{}	\N	2020-10-12 14:49:55.334905
41058	ESRT	2020-10-07	6.61	6.7	6.545	6.65	2633730	{}	\N	2020-10-12 14:49:55.339065
41059	ESRT	2020-10-08	6.7	6.8182	6.64	6.78	3777870	{}	\N	2020-10-12 14:49:55.343043
41060	ESRT	2020-10-09	6.84	6.85	6.64	6.67	2534770	{}	\N	2020-10-12 14:49:55.347162
41061	ESRT	2020-10-12	6.64	6.74	6.605	6.695	538617	{}	\N	2020-10-12 14:49:55.351111
41062	ETH	2020-09-28	13.78	14.15	13.74	13.76	365993	{}	\N	2020-10-12 14:49:56.971856
41063	ETH	2020-09-29	13.75	14.09	13.53	13.59	833802	{}	\N	2020-10-12 14:49:56.978783
41064	ETH	2020-09-30	13.65	13.97	13.48	13.54	879342	{}	\N	2020-10-12 14:49:56.984255
41065	ETH	2020-10-01	13.54	14.04	13.54	13.77	748118	{}	\N	2020-10-12 14:49:56.989615
41066	ETH	2020-10-02	13.6	14.05	13.52	14.01	655782	{}	\N	2020-10-12 14:49:56.994641
41067	ETH	2020-10-05	14.07	14.5031	14.07	14.4	533844	{}	\N	2020-10-12 14:49:57.000244
41068	ETH	2020-10-06	14.55	15.22	14.46	14.47	879673	{}	\N	2020-10-12 14:49:57.004995
41069	ETH	2020-10-07	14.4	15.22	14.4	15.13	867596	{}	\N	2020-10-12 14:49:57.009167
41070	ETH	2020-10-08	15.23	15.59	14.68	15.15	569163	{}	\N	2020-10-12 14:49:57.01347
41071	ETH	2020-10-09	15.2	15.59	15.2	15.49	470356	{}	\N	2020-10-12 14:49:57.017733
41072	ETH	2020-10-12	15.46	15.88	15.43	15.795	104569	{}	\N	2020-10-12 14:49:57.022434
41073	ETW	2020-09-28	8.6	8.64	8.56	8.56	313101	{}	\N	2020-10-12 14:49:58.21209
41074	ETW	2020-09-29	8.59	8.61	8.5	8.5	500360	{}	\N	2020-10-12 14:49:58.218582
41075	ETW	2020-09-30	8.54	8.66	8.5368	8.54	849036	{}	\N	2020-10-12 14:49:58.22392
41076	ETW	2020-10-01	8.61	8.67	8.57	8.63	601597	{}	\N	2020-10-12 14:49:58.229868
41077	ETW	2020-10-02	8.47	8.6369	8.47	8.57	522785	{}	\N	2020-10-12 14:49:58.235948
41078	ETW	2020-10-05	8.64	8.77	8.62	8.68	687670	{}	\N	2020-10-12 14:49:58.241221
41079	ETW	2020-10-06	8.73	8.76	8.65	8.65	502901	{}	\N	2020-10-12 14:49:58.246664
41080	ETW	2020-10-07	8.69	8.77	8.68	8.75	448485	{}	\N	2020-10-12 14:49:58.252886
41081	ETW	2020-10-08	8.77	8.79	8.73	8.76	408552	{}	\N	2020-10-12 14:49:58.258504
41082	ETW	2020-10-09	8.78	8.83	8.71	8.79	476834	{}	\N	2020-10-12 14:49:58.267155
41083	ETW	2020-10-12	8.84	8.8612	8.8101	8.82	140040	{}	\N	2020-10-12 14:49:58.275799
41084	EUO	2020-09-28	24.75	24.8204	24.7	24.71	45670	{}	\N	2020-10-12 14:50:00.397885
41085	EUO	2020-09-29	24.57	24.57	24.34	24.39	153238	{}	\N	2020-10-12 14:50:00.405268
41086	EUO	2020-09-30	24.55	24.63	24.3505	24.49	63066	{}	\N	2020-10-12 14:50:00.413309
41087	EUO	2020-10-01	24.39	24.4749	24.35	24.38	91867	{}	\N	2020-10-12 14:50:00.426733
41088	EUO	2020-10-02	24.5	24.55	24.49	24.55	55043	{}	\N	2020-10-12 14:50:00.438277
41089	EUO	2020-10-05	24.24	24.275	24.1907	24.24	24746	{}	\N	2020-10-12 14:50:00.44698
41090	EUO	2020-10-06	24.18	24.45	24.18	24.38	73763	{}	\N	2020-10-12 14:50:00.456624
41091	EUO	2020-10-07	24.285	24.34	24.28	24.34	36411	{}	\N	2020-10-12 14:50:00.463665
41092	EUO	2020-10-08	24.42	24.43	24.345	24.355	20582	{}	\N	2020-10-12 14:50:00.471083
41093	EUO	2020-10-09	24.15	24.175	24.06	24.07	95122	{}	\N	2020-10-12 14:50:00.481381
41094	EUO	2020-10-12	24.145	24.145	24.1004	24.13	3610	{}	\N	2020-10-12 14:50:00.497666
41095	EVFM	2020-09-28	2.29	2.29	2.15	2.25	2245761	{}	\N	2020-10-12 14:50:02.164853
41096	EVFM	2020-09-29	2.21	2.4	2.2094	2.37	3204147	{}	\N	2020-10-12 14:50:02.173714
41097	EVFM	2020-09-30	2.36	2.48	2.32	2.36	3071746	{}	\N	2020-10-12 14:50:02.182312
41098	EVFM	2020-10-01	2.4	2.42	2.31	2.39	1932665	{}	\N	2020-10-12 14:50:02.18812
41099	EVFM	2020-10-02	2.27	2.6	2.26	2.56	3790304	{}	\N	2020-10-12 14:50:02.192707
41100	EVFM	2020-10-05	2.58	2.78	2.51	2.71	4110640	{}	\N	2020-10-12 14:50:02.198172
41101	EVFM	2020-10-06	2.75	2.82	2.6	2.7	3659448	{}	\N	2020-10-12 14:50:02.202847
41102	EVFM	2020-10-07	2.7	2.87	2.68	2.84	3443125	{}	\N	2020-10-12 14:50:02.207778
41103	EVFM	2020-10-08	2.81	2.98	2.81	2.91	2987625	{}	\N	2020-10-12 14:50:02.21308
41104	EVFM	2020-10-09	2.91	2.93	2.84	2.9	1880066	{}	\N	2020-10-12 14:50:02.219636
41105	EVFM	2020-10-12	2.89	2.9	2.81	2.865	415479	{}	\N	2020-10-12 14:50:02.225021
41106	EVOK	2020-09-28	4.68	4.95	4.68	4.73	174440	{}	\N	2020-10-12 14:50:03.881196
41107	EVOK	2020-09-29	4.69	4.9	4.69	4.73	146639	{}	\N	2020-10-12 14:50:03.888408
41108	EVOK	2020-09-30	4.72	4.88	4.67	4.69	187649	{}	\N	2020-10-12 14:50:03.893691
41109	EVOK	2020-10-01	4.75	4.755	4.5	4.54	160967	{}	\N	2020-10-12 14:50:03.900123
41110	EVOK	2020-10-02	4.37	4.51	4.32	4.39	197192	{}	\N	2020-10-12 14:50:03.905404
41111	EVOK	2020-10-05	4.38	4.7	4.3	4.49	182821	{}	\N	2020-10-12 14:50:03.911695
41112	EVOK	2020-10-06	4.51	4.6	4.41	4.41	168257	{}	\N	2020-10-12 14:50:03.920882
41113	EVOK	2020-10-07	4.43	4.68	4.43	4.56	238138	{}	\N	2020-10-12 14:50:03.926089
41114	EVOK	2020-10-08	4.55	4.57	4.3273	4.38	346741	{}	\N	2020-10-12 14:50:03.931214
41115	EVOK	2020-10-09	4.42	4.45	4.355	4.41	88699	{}	\N	2020-10-12 14:50:03.936571
41116	EVOK	2020-10-12	4.6	4.6	4.32	4.35	119820	{}	\N	2020-10-12 14:50:03.941464
41117	EVTC	2020-09-28	34.82	35.31	34.73	34.84	273751	{}	\N	2020-10-12 14:50:04.841231
41118	EVTC	2020-09-29	34.87	35.075	34.67	34.87	411345	{}	\N	2020-10-12 14:50:04.846959
41119	EVTC	2020-09-30	35.03	35.3	34.415	34.71	803238	{}	\N	2020-10-12 14:50:04.852007
41120	EVTC	2020-10-01	34.73	35.2	34.73	35.08	689446	{}	\N	2020-10-12 14:50:04.85632
41121	EVTC	2020-10-02	34.55	35.42	34.36	35.26	1056406	{}	\N	2020-10-12 14:50:04.860478
41122	EVTC	2020-10-05	35.75	36.01	35.41	36	767608	{}	\N	2020-10-12 14:50:04.864988
41123	EVTC	2020-10-06	36.3	36.66	35.26	35.36	984961	{}	\N	2020-10-12 14:50:04.870185
41124	EVTC	2020-10-07	35.59	35.63	35.09	35.36	814181	{}	\N	2020-10-12 14:50:04.874815
41125	EVTC	2020-10-08	35.69	36.1	35.42	35.88	905766	{}	\N	2020-10-12 14:50:04.879825
41126	EVTC	2020-10-09	36.21	36.85	36.11	36.69	496045	{}	\N	2020-10-12 14:50:04.88448
41127	EVTC	2020-10-12	36.99	37.14	36.26	36.35	85859	{}	\N	2020-10-12 14:50:04.889184
41128	EWG	2020-09-28	29.18	29.28	29.14	29.27	4401792	{}	\N	2020-10-12 14:50:06.905468
41129	EWG	2020-09-29	29.3	29.445	29.185	29.37	6477094	{}	\N	2020-10-12 14:50:06.912472
41130	EWG	2020-09-30	29.25	29.45	29.04	29.14	8151023	{}	\N	2020-10-12 14:50:06.918634
41131	EWG	2020-10-01	29.29	29.34	29.1	29.32	8155568	{}	\N	2020-10-12 14:50:06.92313
41132	EWG	2020-10-02	28.83	29.22	28.83	29.17	4547072	{}	\N	2020-10-12 14:50:06.927068
41133	EWG	2020-10-05	29.48	29.78	29.48	29.78	3347881	{}	\N	2020-10-12 14:50:06.93123
41134	EWG	2020-10-06	29.9	29.9025	29.37	29.46	3927733	{}	\N	2020-10-12 14:50:06.935631
41135	EWG	2020-10-07	29.58	29.8	29.57	29.73	3138490	{}	\N	2020-10-12 14:50:06.940084
41136	EWG	2020-10-08	29.96	30.055	29.92	30.04	9805245	{}	\N	2020-10-12 14:50:06.944191
41137	EWG	2020-10-09	30.18	30.225	30.06	30.2	4192305	{}	\N	2020-10-12 14:50:06.948188
41138	EWG	2020-10-12	30.25	30.32	30.25	30.3151	495890	{}	\N	2020-10-12 14:50:06.952072
41139	EWP	2020-09-28	21.83	22.005	21.83	21.97	567519	{}	\N	2020-10-12 14:50:08.03139
41140	EWP	2020-09-29	21.99	22.06	21.75	21.85	503008	{}	\N	2020-10-12 14:50:08.037812
41141	EWP	2020-09-30	21.92	22.025	21.72	21.76	806711	{}	\N	2020-10-12 14:50:08.042604
41142	EWP	2020-10-01	21.91	21.96	21.78	21.91	607577	{}	\N	2020-10-12 14:50:08.047428
41143	EWP	2020-10-02	21.73	22.07	21.73	22.01	270164	{}	\N	2020-10-12 14:50:08.051887
41144	EWP	2020-10-05	22.22	22.46	22.22	22.45	564043	{}	\N	2020-10-12 14:50:08.055916
41145	EWP	2020-10-06	22.848	22.848	22.36	22.38	741941	{}	\N	2020-10-12 14:50:08.059811
41146	EWP	2020-10-07	22.66	22.73	22.56	22.61	407470	{}	\N	2020-10-12 14:50:08.063605
41147	EWP	2020-10-08	22.81	22.92	22.765	22.91	527048	{}	\N	2020-10-12 14:50:08.067931
41148	EWP	2020-10-09	22.92	22.92	22.76	22.86	428177	{}	\N	2020-10-12 14:50:08.071727
41149	EWP	2020-10-12	22.78	22.81	22.76	22.78	33298	{}	\N	2020-10-12 14:50:08.076062
41150	EWV	2020-09-28	19.61	19.61	19.4642	19.4642	9558	{}	\N	2020-10-12 14:50:09.180746
41151	EWV	2020-09-29	19.37	19.4735	19.351	19.4735	679	{}	\N	2020-10-12 14:50:09.185998
41152	EWV	2020-09-30	19.86	19.86	19.71	19.71	4650	{}	\N	2020-10-12 14:50:09.190853
41153	EWV	2020-10-01	19.8	19.8	19.76	19.76	2629	{}	\N	2020-10-12 14:50:09.195769
41154	EWV	2020-10-02	20.07	20.07	19.9651	19.9651	2297	{}	\N	2020-10-12 14:50:09.200851
41155	EWV	2020-10-05	19.77	19.77	19.63	19.71	4010	{}	\N	2020-10-12 14:50:09.205379
41156	EWV	2020-10-06	19.5301	19.67	19.48	19.67	2423	{}	\N	2020-10-12 14:50:09.210203
41157	EWV	2020-10-07	19.54	19.54	19.524	19.524	384	{}	\N	2020-10-12 14:50:09.21467
41158	EWV	2020-10-08	19.4	19.51	19.4	19.505	2192	{}	\N	2020-10-12 14:50:09.219807
41159	EWV	2020-10-09	19.5	19.54	19.4646	19.47	3412	{}	\N	2020-10-12 14:50:09.224304
41160	EWV	2020-10-12	19.51	19.51	19.51	19.51	1236	{}	\N	2020-10-12 14:50:09.228726
41161	EXI	2020-09-28	93.5	94.05	93.5	93.87	5470	{}	\N	2020-10-12 14:50:10.211913
41162	EXI	2020-09-29	96.25	96.25	93.4266	93.775	70987	{}	\N	2020-10-12 14:50:10.216712
41163	EXI	2020-09-30	93.42	94.04	93.01	93.2021	14622	{}	\N	2020-10-12 14:50:10.220704
41164	EXI	2020-10-01	93.52	93.88	92.6247	93.22	16916	{}	\N	2020-10-12 14:50:10.224627
41165	EXI	2020-10-02	91.76	93.785	91.76	93.673	4080	{}	\N	2020-10-12 14:50:10.228648
41166	EXI	2020-10-05	94.36	95.15	94.36	95.15	15080	{}	\N	2020-10-12 14:50:10.23297
41167	EXI	2020-10-06	95.3	95.61	94.115	94.13	6688	{}	\N	2020-10-12 14:50:10.237027
41168	EXI	2020-10-07	95.11	95.7	95.11	95.7	9864	{}	\N	2020-10-12 14:50:10.241064
41169	EXI	2020-10-08	96	96.27	95.74	96.27	7567	{}	\N	2020-10-12 14:50:10.245935
41170	EXI	2020-10-09	96.62	97.006	96.4554	96.645	7604	{}	\N	2020-10-12 14:50:10.250047
41171	EXI	2020-10-12	97	97	96.87	96.93	5784	{}	\N	2020-10-12 14:50:10.253903
41172	ESSA	2020-09-28	12.23	12.44	11.81	12.35	16038	{}	\N	2020-10-12 14:50:11.259985
41173	ESSA	2020-09-29	12.22	12.62	12.001	12.54	10606	{}	\N	2020-10-12 14:50:11.26531
41174	ESSA	2020-09-30	12.67	12.67	12.33	12.33	8770	{}	\N	2020-10-12 14:50:11.270458
41175	ESSA	2020-10-01	12.2356	12.44	12.045	12.25	9499	{}	\N	2020-10-12 14:50:11.275188
41176	ESSA	2020-10-02	12.16	12.5	12.16	12.3	7922	{}	\N	2020-10-12 14:50:11.280429
41177	ESSA	2020-10-05	12.38	12.805	11.99	12.16	13119	{}	\N	2020-10-12 14:50:11.285475
41178	ESSA	2020-10-06	12.35	12.985	12.25	12.75	14391	{}	\N	2020-10-12 14:50:11.290482
41179	ESSA	2020-10-07	12.85	13	12.8	13	13510	{}	\N	2020-10-12 14:50:11.295024
41180	ESSA	2020-10-08	13	13.225	12.74	12.76	11088	{}	\N	2020-10-12 14:50:11.299966
41181	ESSA	2020-10-09	12.77	13.19	12.77	13.105	3083	{}	\N	2020-10-12 14:50:11.305602
41182	ESSA	2020-10-12	13.16	13.39	13.16	13.165	6363	{}	\N	2020-10-12 14:50:11.313412
41183	ETJ	2020-09-28	9.68	9.7	9.57	9.58	246168	{}	\N	2020-10-12 14:50:12.84968
41184	ETJ	2020-09-29	9.55	9.58	9.46	9.54	610734	{}	\N	2020-10-12 14:50:12.856972
41185	ETJ	2020-09-30	9.6	9.68	9.57	9.57	753374	{}	\N	2020-10-12 14:50:12.861715
41186	ETJ	2020-10-01	9.64	9.74	9.62	9.71	453508	{}	\N	2020-10-12 14:50:12.867146
41187	ETJ	2020-10-02	9.62	9.69	9.55	9.69	334294	{}	\N	2020-10-12 14:50:12.871398
41188	ETJ	2020-10-05	9.71	9.76	9.65	9.74	436310	{}	\N	2020-10-12 14:50:12.875399
41189	ETJ	2020-10-06	9.77	9.77	9.6269	9.68	370360	{}	\N	2020-10-12 14:50:12.879345
41190	ETJ	2020-10-07	9.73	9.76	9.71	9.75	373178	{}	\N	2020-10-12 14:50:12.883279
41191	ETJ	2020-10-08	9.78	9.79	9.74	9.75	481390	{}	\N	2020-10-12 14:50:12.88793
41192	ETJ	2020-10-09	9.79	9.85	9.78	9.83	374142	{}	\N	2020-10-12 14:50:12.891957
41193	ETJ	2020-10-12	9.89	9.89	9.85	9.86	136092	{}	\N	2020-10-12 14:50:12.895923
41194	EUSA	2020-09-28	61.2617	61.2617	61.1625	61.1625	2561	{}	\N	2020-10-12 14:50:14.093123
41195	EUSA	2020-09-29	61.135	61.2753	60.695	60.98	5120	{}	\N	2020-10-12 14:50:14.097952
41196	EUSA	2020-09-30	61.335	61.6	60.73	61.07	9793	{}	\N	2020-10-12 14:50:14.101995
41197	EUSA	2020-10-01	61.56	61.57	61.15	61.3942	13087	{}	\N	2020-10-12 14:50:14.105994
41198	EUSA	2020-10-02	61.3257	61.93	61.32	61.71	5761	{}	\N	2020-10-12 14:50:14.110006
41199	EUSA	2020-10-05	62.41	62.8299	62.41	62.8074	4236	{}	\N	2020-10-12 14:50:14.11427
41200	EUSA	2020-10-06	62.87	63.579	62.3193	62.3193	6106	{}	\N	2020-10-12 14:50:14.118569
41201	EUSA	2020-10-07	63.1497	63.47	63.1101	63.43	19561	{}	\N	2020-10-12 14:50:14.122303
41202	EUSA	2020-10-08	63.6	64.1024	63.6	64.1024	4887	{}	\N	2020-10-12 14:50:14.12584
41203	EUSA	2020-10-09	64.3	64.59	64.2408	64.38	9689	{}	\N	2020-10-12 14:50:14.129422
41204	EUSA	2020-10-12	64.5366	64.6799	64.5366	64.6669	2498	{}	\N	2020-10-12 14:50:14.132986
41205	EVH	2020-09-28	12.38	12.53	12.25	12.31	642722	{}	\N	2020-10-12 14:50:15.708574
41206	EVH	2020-09-29	12.45	13.11	12.39	12.86	2032819	{}	\N	2020-10-12 14:50:15.726429
41207	EVH	2020-09-30	12.79	12.95	12.32	12.41	1637979	{}	\N	2020-10-12 14:50:15.732248
41208	EVH	2020-10-01	12.42	12.63	11.84	12.62	2471543	{}	\N	2020-10-12 14:50:15.739707
41209	EVH	2020-10-02	12.28	12.59	12.26	12.39	1020759	{}	\N	2020-10-12 14:50:15.746644
41210	EVH	2020-10-05	12.51	12.545	11.49	11.57	3515157	{}	\N	2020-10-12 14:50:15.753644
41211	EVH	2020-10-06	11.75	12.03	11.55	11.64	2281717	{}	\N	2020-10-12 14:50:15.759916
41212	EVH	2020-10-07	11.75	12.055	11.44	11.54	2779441	{}	\N	2020-10-12 14:50:15.764533
41213	EVH	2020-10-08	11.55	11.73	11.02	11.36	3278094	{}	\N	2020-10-12 14:50:15.769065
41214	EVH	2020-10-09	11.37	11.56	11.18	11.21	1724186	{}	\N	2020-10-12 14:50:15.772603
41215	EVH	2020-10-12	11.29	11.48	11.1201	11.19	294818	{}	\N	2020-10-12 14:50:15.776029
41216	EW	2020-09-28	78.35	78.72	77.49	77.71	2809619	{}	\N	2020-10-12 14:50:17.698007
41217	EW	2020-09-29	78.34	80.0321	77.9702	78.31	5131588	{}	\N	2020-10-12 14:50:17.703846
41218	EW	2020-09-30	78.33	80.99	78.32	79.82	6276290	{}	\N	2020-10-12 14:50:17.708846
41219	EW	2020-10-01	80.38	80.7399	78.44	78.98	3725425	{}	\N	2020-10-12 14:50:17.71311
41220	EW	2020-10-02	77.95	79.92	77.67	78.81	2941147	{}	\N	2020-10-12 14:50:17.717693
41221	EW	2020-10-05	79.12	80.5	79.04	80.15	3355138	{}	\N	2020-10-12 14:50:17.722173
41222	EW	2020-10-06	80.28	80.43	78.474	78.59	2943785	{}	\N	2020-10-12 14:50:17.726345
41223	EW	2020-10-07	79.58	81.72	79.4526	81.5	3892698	{}	\N	2020-10-12 14:50:17.7304
41224	EW	2020-10-08	82	82.25	80.93	81.65	3572385	{}	\N	2020-10-12 14:50:17.73451
41225	EW	2020-10-09	82.6	83.95	81.81	83.9	4453426	{}	\N	2020-10-12 14:50:17.738839
41226	EW	2020-10-12	84.44	84.65	83.18	84.24	868650	{}	\N	2020-10-12 14:50:17.74297
41227	EWH	2020-09-28	21.76	21.82	21.72	21.82	3917946	{}	\N	2020-10-12 14:50:18.72266
41228	EWH	2020-09-29	21.67	21.81	21.64	21.78	6875286	{}	\N	2020-10-12 14:50:18.728478
41229	EWH	2020-09-30	21.79	21.945	21.74	21.9	7664044	{}	\N	2020-10-12 14:50:18.733404
41230	EWH	2020-10-01	22.04	22.2	21.96	22.16	3905824	{}	\N	2020-10-12 14:50:18.73756
41231	EWH	2020-10-02	21.91	22.16	21.91	22.04	3829410	{}	\N	2020-10-12 14:50:18.742205
41232	EWH	2020-10-05	22.1	22.305	22.0779	22.3	5103520	{}	\N	2020-10-12 14:50:18.746368
41233	EWH	2020-10-06	22.38	22.4	22.25	22.28	4358564	{}	\N	2020-10-12 14:50:18.750577
41234	EWH	2020-10-07	22.5	22.525	22.405	22.47	4951484	{}	\N	2020-10-12 14:50:18.754907
41235	EWH	2020-10-08	22.38	22.52	22.36	22.5	6128997	{}	\N	2020-10-12 14:50:18.759709
41236	EWH	2020-10-09	22.31	22.44	22.28	22.4	3360394	{}	\N	2020-10-12 14:50:18.763771
41237	EWH	2020-10-12	22.54	22.54	22.4516	22.48	2364298	{}	\N	2020-10-12 14:50:18.767855
41238	EWRE	2020-09-28	26.04	26.121	25.9977	25.9977	3914	{}	\N	2020-10-12 14:50:19.717989
41239	EWRE	2020-09-29	25.7175	25.7175	25.715	25.715	572	{}	\N	2020-10-12 14:50:19.726073
41240	EWRE	2020-09-30	26.07	26.0879	25.715	25.82	3899	{}	\N	2020-10-12 14:50:19.732704
41241	EWRE	2020-10-01	26.02	26.3355	25.84	26.3355	8411	{}	\N	2020-10-12 14:50:19.740961
41242	EWRE	2020-10-02	26.24	27.03	26.24	27.03	2146	{}	\N	2020-10-12 14:50:19.747816
41243	EWRE	2020-10-05	27.02	27.0952	26.67	27.0952	4929	{}	\N	2020-10-12 14:50:19.755096
41244	EWRE	2020-10-06	27.3703	27.3703	26.9546	26.9546	671	{}	\N	2020-10-12 14:50:19.762316
41245	EWRE	2020-10-07	27.01	27.16	26.84	27.08	9248	{}	\N	2020-10-12 14:50:19.768559
41246	EWRE	2020-10-08	27.45	27.56	27.45	27.56	3235	{}	\N	2020-10-12 14:50:19.773973
41247	EWRE	2020-10-09	27.27	27.3101	27.27	27.3101	782	{}	\N	2020-10-12 14:50:19.779068
41248	EWRE	2020-10-12	27.165	27.165	27.165	27.165	151	{}	\N	2020-10-12 14:50:19.786543
41249	EWZ	2020-09-28	29.02	29.1	27.255	27.46	36925979	{}	\N	2020-10-12 14:50:20.96459
41250	EWZ	2020-09-29	27.32	27.9	27.115	27.17	26291339	{}	\N	2020-10-12 14:50:20.971138
41251	EWZ	2020-09-30	27.37	27.89	27.37	27.66	26976479	{}	\N	2020-10-12 14:50:20.975985
41252	EWZ	2020-10-01	27.48	27.735	27.1	27.67	24203810	{}	\N	2020-10-12 14:50:20.980427
41253	EWZ	2020-10-02	27.5	28.02	27.14	27.16	21235486	{}	\N	2020-10-12 14:50:20.985226
41254	EWZ	2020-10-05	27.47	28.45	27.3318	28.38	21767775	{}	\N	2020-10-12 14:50:20.989331
41255	EWZ	2020-10-06	28.71	29.085	27.81	27.97	34736357	{}	\N	2020-10-12 14:50:20.99337
41256	EWZ	2020-10-07	28.2	28.345	27.64	27.91	26727698	{}	\N	2020-10-12 14:50:20.99738
41257	EWZ	2020-10-08	27.92	28.815	27.85	28.72	20679788	{}	\N	2020-10-12 14:50:21.001467
41258	EWZ	2020-10-09	28.87	29.35	28.725	28.96	23654168	{}	\N	2020-10-12 14:50:21.005482
41259	EWZ	2020-10-12	29.17	29.19	28.991	29.07	1463100	{}	\N	2020-10-12 14:50:21.009323
41260	EXLS	2020-09-28	64.18	65.96	64.18	65.48	161583	{}	\N	2020-10-12 14:50:22.539067
41261	EXLS	2020-09-29	65.42	66.17	65.12	66.05	113638	{}	\N	2020-10-12 14:50:22.545142
41262	EXLS	2020-09-30	66.375	67.09	65.6	65.94	220694	{}	\N	2020-10-12 14:50:22.550407
41263	EXLS	2020-10-01	66.26	66.96	64.77	65.86	167149	{}	\N	2020-10-12 14:50:22.555901
41264	EXLS	2020-10-02	64.22	66.76	64.22	66.24	87290	{}	\N	2020-10-12 14:50:22.560365
41265	EXLS	2020-10-05	66.72	67.73	66.38	67.73	102251	{}	\N	2020-10-12 14:50:22.56567
41266	EXLS	2020-10-06	68.46	69.28	67.63	68.4	146897	{}	\N	2020-10-12 14:50:22.57125
41267	EXLS	2020-10-07	68.69	69.34	67.15	67.85	182146	{}	\N	2020-10-12 14:50:22.577076
41268	EXLS	2020-10-08	68.68	69.77	68.04	69.17	192627	{}	\N	2020-10-12 14:50:22.583024
41269	EXLS	2020-10-09	69.49	70.44	69.38	70.1	100052	{}	\N	2020-10-12 14:50:22.588472
41270	EXLS	2020-10-12	70.04	70.64	69.7229	70.11	18599	{}	\N	2020-10-12 14:50:22.592733
41271	EXTR	2020-09-28	3.96	4.16	3.96	4.09	1119324	{}	\N	2020-10-12 14:50:23.77184
41272	EXTR	2020-09-29	4.09	4.155	4.02	4.15	755244	{}	\N	2020-10-12 14:50:23.77743
41273	EXTR	2020-09-30	4.15	4.205	4.02	4.02	981217	{}	\N	2020-10-12 14:50:23.782199
41274	EXTR	2020-10-01	4.05	4.06	3.93	4.005	717689	{}	\N	2020-10-12 14:50:23.786277
41275	EXTR	2020-10-02	3.84	4.02	3.84	4	383190	{}	\N	2020-10-12 14:50:23.790747
41276	EXTR	2020-10-05	4.05	4.13	4.02	4.07	358618	{}	\N	2020-10-12 14:50:23.79524
41277	EXTR	2020-10-06	4.11	4.15	3.96	3.985	671950	{}	\N	2020-10-12 14:50:23.799844
41278	EXTR	2020-10-07	4.03	4.085	4	4.07	519858	{}	\N	2020-10-12 14:50:23.803645
41279	EXTR	2020-10-08	4.14	4.16	4.0377	4.11	556927	{}	\N	2020-10-12 14:50:23.807452
41280	EXTR	2020-10-09	4.65	4.96	4.51	4.61	3512415	{}	\N	2020-10-12 14:50:23.81118
41281	EXTR	2020-10-12	4.6	4.93	4.5305	4.85	687734	{}	\N	2020-10-12 14:50:23.815084
41282	ETG	2020-09-28	14.77	14.8902	14.76	14.77	137747	{}	\N	2020-10-12 14:50:25.038974
41283	ETG	2020-09-29	14.83	14.86	14.75	14.79	214000	{}	\N	2020-10-12 14:50:25.044076
41284	ETG	2020-09-30	14.86	15.02	14.78	14.87	442921	{}	\N	2020-10-12 14:50:25.048349
41285	ETG	2020-10-01	14.93	15.08	14.88	14.96	269283	{}	\N	2020-10-12 14:50:25.052037
41286	ETG	2020-10-02	14.84	14.98	14.79	14.85	451528	{}	\N	2020-10-12 14:50:25.055896
41287	ETG	2020-10-05	14.92	15.18	14.9087	15.18	489506	{}	\N	2020-10-12 14:50:25.059943
41288	ETG	2020-10-06	15.24	15.43	15.09	15.14	335811	{}	\N	2020-10-12 14:50:25.063942
41289	ETG	2020-10-07	15.31	15.3351	15.21	15.33	279630	{}	\N	2020-10-12 14:50:25.067819
41290	ETG	2020-10-08	15.47	15.47	15.3	15.36	339350	{}	\N	2020-10-12 14:50:25.071642
41291	ETG	2020-10-09	15.46	15.55	15.4	15.44	245382	{}	\N	2020-10-12 14:50:25.075452
41292	ETG	2020-10-12	15.59	15.63	15.5639	15.61	58858	{}	\N	2020-10-12 14:50:25.079216
41293	ETX	2020-09-28	21.36	21.36	21.055	21.065	7601	{}	\N	2020-10-12 14:50:26.554326
41294	ETX	2020-09-29	21.18	21.22	21	21.21	27880	{}	\N	2020-10-12 14:50:26.560241
41295	ETX	2020-09-30	21.14	21.18	21.035	21.05	27918	{}	\N	2020-10-12 14:50:26.564297
41296	ETX	2020-10-01	21.1564	21.24	20.92	20.92	44992	{}	\N	2020-10-12 14:50:26.569043
41297	ETX	2020-10-02	21.1132	21.1132	20.77	20.8441	38158	{}	\N	2020-10-12 14:50:26.574054
41298	ETX	2020-10-05	20.8	21.05	20.785	20.7962	49946	{}	\N	2020-10-12 14:50:26.577638
41299	ETX	2020-10-06	20.785	20.989	20.785	20.87	12292	{}	\N	2020-10-12 14:50:26.581279
41300	ETX	2020-10-07	20.9	20.99	20.8139	20.98	33124	{}	\N	2020-10-12 14:50:26.585306
41301	ETX	2020-10-08	21.01	21.04	20.89	20.953	32292	{}	\N	2020-10-12 14:50:26.590165
41302	ETX	2020-10-09	21.06	21.27	20.95	21.18	29618	{}	\N	2020-10-12 14:50:26.594154
41303	EURL	2020-09-28	18.2	21.48	17.42	17.55	49105	{}	\N	2020-10-12 14:50:27.981547
41304	EURL	2020-09-29	17.6812	18.0675	17.5	17.52	8184	{}	\N	2020-10-12 14:50:27.987909
41305	EURL	2020-09-30	17.54	17.87	17.26	17.41	24149	{}	\N	2020-10-12 14:50:27.992962
41306	EURL	2020-10-01	17.77	17.775	17.542	17.75	14159	{}	\N	2020-10-12 14:50:27.99788
41307	EURL	2020-10-02	17.13	17.85	17.13	17.77	4740	{}	\N	2020-10-12 14:50:28.002519
41308	EURL	2020-10-05	18.43	18.72	18.34	18.69	15603	{}	\N	2020-10-12 14:50:28.006864
41309	EURL	2020-10-06	18.7965	18.7965	17.96	17.99	14076	{}	\N	2020-10-12 14:50:28.011275
41310	EURL	2020-10-07	18.36	18.43	18.2337	18.4	4552	{}	\N	2020-10-12 14:50:28.015377
41311	EURL	2020-10-08	18.642	18.881	18.642	18.881	11447	{}	\N	2020-10-12 14:50:28.019683
41312	EURL	2020-10-09	19.27	19.47	19.18	19.4	42582	{}	\N	2020-10-12 14:50:28.025021
41313	EURL	2020-10-12	19.57	19.651	19.57	19.651	3910	{}	\N	2020-10-12 14:50:28.029244
41314	EVF	2020-09-28	5.87	5.89	5.86	5.88	65634	{}	\N	2020-10-12 14:50:29.036023
41315	EVF	2020-09-29	5.86	5.8766	5.83	5.85	83404	{}	\N	2020-10-12 14:50:29.046338
41316	EVF	2020-09-30	5.83	5.88	5.83	5.87	203325	{}	\N	2020-10-12 14:50:29.052341
41317	EVF	2020-10-01	5.88	5.89	5.86	5.89	66395	{}	\N	2020-10-12 14:50:29.05792
41318	EVF	2020-10-02	5.85	5.9	5.85	5.88	22436	{}	\N	2020-10-12 14:50:29.062187
41319	EVF	2020-10-05	5.88	6	5.8774	5.99	40936	{}	\N	2020-10-12 14:50:29.066634
41320	EVF	2020-10-06	5.95	5.97	5.91	5.95	153916	{}	\N	2020-10-12 14:50:29.071051
41321	EVF	2020-10-07	5.95	5.95	5.8822	5.905	104620	{}	\N	2020-10-12 14:50:29.075816
41322	EVF	2020-10-08	5.91	6.1	5.91	6.06	315972	{}	\N	2020-10-12 14:50:29.080105
41323	EVF	2020-10-09	6.05	6.1	6.0147	6.1	234754	{}	\N	2020-10-12 14:50:29.084475
41324	EVF	2020-10-12	6.07	6.09	6.07	6.07	23966	{}	\N	2020-10-12 14:50:29.088623
41325	EVOL	2020-09-28	1.0168	1.04	0.97	1.04	15567	{}	\N	2020-10-12 14:50:31.720307
41326	EVOL	2020-09-29	1.01	1.12	1.01	1.09	25003	{}	\N	2020-10-12 14:50:31.726598
41327	EVOL	2020-09-30	1.11	1.25	1.0777	1.16	71397	{}	\N	2020-10-12 14:50:31.731726
41328	EVOL	2020-10-01	1.1512	1.18	1.13	1.1617	12035	{}	\N	2020-10-12 14:50:31.737079
41329	EVOL	2020-10-02	1.1636	1.1859	1.12	1.14	20226	{}	\N	2020-10-12 14:50:31.741926
41330	EVOL	2020-10-05	1.1138	1.1597	1.11	1.11	7190	{}	\N	2020-10-12 14:50:31.746176
41331	EVOL	2020-10-06	1.13	1.15	1.13	1.15	7716	{}	\N	2020-10-12 14:50:31.750158
41332	EVOL	2020-10-07	1.15	1.16	1.11	1.125	29805	{}	\N	2020-10-12 14:50:31.754268
41333	EVOL	2020-10-08	1.12	1.2	1.06	1.17	59535	{}	\N	2020-10-12 14:50:31.75854
41334	EVOL	2020-10-09	1.17	1.2	1.12	1.15	18695	{}	\N	2020-10-12 14:50:31.763283
41335	EVOL	2020-10-12	1.12	1.14	1.12	1.14	1142	{}	\N	2020-10-12 14:50:31.768234
41336	EWC	2020-09-28	27.58	27.65	27.46	27.57	2015857	{}	\N	2020-10-12 14:50:33.490105
41337	EWC	2020-09-29	27.5	27.52	27.25	27.42	2588330	{}	\N	2020-10-12 14:50:33.495437
41338	EWC	2020-09-30	27.49	27.65	27.36	27.41	3765372	{}	\N	2020-10-12 14:50:33.500451
41339	EWC	2020-10-01	27.51	27.655	27.39	27.57	2190975	{}	\N	2020-10-12 14:50:33.504483
41340	EWC	2020-10-02	27.21	27.63	27.18	27.55	1990487	{}	\N	2020-10-12 14:50:33.508738
41341	EWC	2020-10-05	27.76	28.07	27.71	28.07	1640213	{}	\N	2020-10-12 14:50:33.513269
41342	EWC	2020-10-06	28.08	28.13	27.58	27.64	2266164	{}	\N	2020-10-12 14:50:33.517957
41343	EWC	2020-10-07	27.94	28.13	27.88	28.05	1713285	{}	\N	2020-10-12 14:50:33.52289
41344	EWC	2020-10-08	28.27	28.47	28.25	28.46	5375032	{}	\N	2020-10-12 14:50:33.529591
41345	EWC	2020-10-09	28.63	28.71	28.48	28.6	1832608	{}	\N	2020-10-12 14:50:33.5368
41346	EWC	2020-10-12	28.77	28.8	28.65	28.68	696902	{}	\N	2020-10-12 14:50:33.541175
41347	EWQ	2020-09-28	27.94	28.04	27.9	27.99	834683	{}	\N	2020-10-12 14:50:36.021
41348	EWQ	2020-09-29	28.05	28.18	27.92	28.07	941139	{}	\N	2020-10-12 14:50:36.027533
41349	EWQ	2020-09-30	28.04	28.22	27.81	27.94	1091681	{}	\N	2020-10-12 14:50:36.032016
41350	EWQ	2020-10-01	28.12	28.18	27.97	28.1	1032966	{}	\N	2020-10-12 14:50:36.036826
41351	EWQ	2020-10-02	27.71	28.16	27.71	28.1	587578	{}	\N	2020-10-12 14:50:36.040976
41352	EWQ	2020-10-05	28.4	28.62	28.4	28.62	605953	{}	\N	2020-10-12 14:50:36.045614
41353	EWQ	2020-10-06	28.82	28.82	28.27	28.35	1113427	{}	\N	2020-10-12 14:50:36.049766
41354	EWQ	2020-10-07	28.55	28.65	28.46	28.61	587462	{}	\N	2020-10-12 14:50:36.053829
41355	EWQ	2020-10-08	28.73	28.825	28.69	28.82	968317	{}	\N	2020-10-12 14:50:36.057966
41356	EWQ	2020-10-09	29.07	29.15	28.98	29.09	659868	{}	\N	2020-10-12 14:50:36.061789
41357	EWQ	2020-10-12	29.19	29.23	29.185	29.2	81917	{}	\N	2020-10-12 14:50:36.065836
41358	EWUS	2020-09-28	33.5401	33.5401	33.46	33.46	6810	{}	\N	2020-10-12 14:50:38.029432
41359	EWUS	2020-09-29	33.19	33.2427	33.19	33.2427	256	{}	\N	2020-10-12 14:50:38.035073
41360	EWUS	2020-09-30	33.475	33.5923	33.475	33.5923	674	{}	\N	2020-10-12 14:50:38.039634
41361	EWUS	2020-10-01	33.57	33.76	33.57	33.75	1898	{}	\N	2020-10-12 14:50:38.043697
41362	EWUS	2020-10-02	33.68	33.9199	33.68	33.87	808	{}	\N	2020-10-12 14:50:38.047538
41363	EWUS	2020-10-05	34.35	34.47	34.29	34.47	1886	{}	\N	2020-10-12 14:50:38.051362
41364	EWUS	2020-10-06	34.61	34.61	34.35	34.35	818	{}	\N	2020-10-12 14:50:38.055738
41365	EWUS	2020-10-07	34.7	34.7	34.39	34.54	3641	{}	\N	2020-10-12 14:50:38.059458
41366	EWUS	2020-10-08	34.82	34.82	34.7	34.7881	2526	{}	\N	2020-10-12 14:50:38.063285
41367	EWUS	2020-10-09	35.245	35.3744	35.245	35.3744	1191	{}	\N	2020-10-12 14:50:38.067722
41368	EXD	2020-09-28	8.89	8.965	8.88	8.92	30142	{}	\N	2020-10-12 14:50:39.245375
41369	EXD	2020-09-29	8.93	9.0336	8.83	8.85	111516	{}	\N	2020-10-12 14:50:39.251652
41370	EXD	2020-09-30	8.91	9.0629	8.91	8.96	84796	{}	\N	2020-10-12 14:50:39.256479
41371	EXD	2020-10-01	8.92	9.0793	8.92	9.06	46892	{}	\N	2020-10-12 14:50:39.261077
41372	EXD	2020-10-02	8.94	9.04	8.9335	9.001	35811	{}	\N	2020-10-12 14:50:39.266673
41373	EXD	2020-10-05	8.94	9.1	8.94	9.1	41895	{}	\N	2020-10-12 14:50:39.271402
41374	EXD	2020-10-06	9.05	9.12	8.97	8.97	38436	{}	\N	2020-10-12 14:50:39.277054
41375	EXD	2020-10-07	9.08	9.11	9.0101	9.09	33932	{}	\N	2020-10-12 14:50:39.283526
41376	EXD	2020-10-08	9.1	9.2098	9.1	9.16	53080	{}	\N	2020-10-12 14:50:39.288491
41377	EXD	2020-10-09	9.2	9.27	9.14	9.15	53404	{}	\N	2020-10-12 14:50:39.294569
41378	EXD	2020-10-12	9.21	9.35	9.21	9.24	15306	{}	\N	2020-10-12 14:50:39.299456
41379	EXP	2020-09-28	83.47	85.74	83.25	84.82	280033	{}	\N	2020-10-12 14:50:40.419964
41380	EXP	2020-09-29	84.52	85.32	83.19	84.42	470019	{}	\N	2020-10-12 14:50:40.426061
41381	EXP	2020-09-30	84.78	87.525	84.685	86.32	735365	{}	\N	2020-10-12 14:50:40.431007
41382	EXP	2020-10-01	86.31	88.43	85.995	86.97	506536	{}	\N	2020-10-12 14:50:40.435553
41383	EXP	2020-10-02	84.86	89.29	84.59	87.76	1472815	{}	\N	2020-10-12 14:50:40.440081
41384	EXP	2020-10-05	89.44	91.61	87.9972	89.29	1592651	{}	\N	2020-10-12 14:50:40.444419
41385	EXP	2020-10-06	90.78	92.97	88.96	89.26	1089438	{}	\N	2020-10-12 14:50:40.448976
41386	EXP	2020-10-07	90.9	92.91	90.09	91.32	639892	{}	\N	2020-10-12 14:50:40.453203
41387	EXP	2020-10-08	93.06	94.99	92.84	93.83	481180	{}	\N	2020-10-12 14:50:40.457415
41388	EXP	2020-10-09	94.36	94.66	91.73	93.36	582860	{}	\N	2020-10-12 14:50:40.461675
41389	EXP	2020-10-12	94.41	94.68	93.05	94.34	109225	{}	\N	2020-10-12 14:50:40.466061
41390	EYE	2020-09-28	37.25	38.5	37.15	38.24	386801	{}	\N	2020-10-12 14:50:41.73235
41391	EYE	2020-09-29	38.12	39.08	37.19	38.63	562092	{}	\N	2020-10-12 14:50:41.739748
41392	EYE	2020-09-30	38.68	39.4	37.92	38.24	253512	{}	\N	2020-10-12 14:50:41.745419
41393	EYE	2020-10-01	38.64	39.46	38.24	39.45	263142	{}	\N	2020-10-12 14:50:41.751642
41394	EYE	2020-10-02	38.52	39.954	37.3369	39.68	286274	{}	\N	2020-10-12 14:50:41.75642
41395	EYE	2020-10-05	40	40.885	39.4173	40.87	621809	{}	\N	2020-10-12 14:50:41.76077
41396	EYE	2020-10-06	41.22	41.697	39.27	39.33	298526	{}	\N	2020-10-12 14:50:41.765513
41397	EYE	2020-10-07	40.05	41.31	37.7901	41.07	831434	{}	\N	2020-10-12 14:50:41.771249
41398	EYE	2020-10-08	41.39	41.39	39.57	39.6	464846	{}	\N	2020-10-12 14:50:41.775827
41399	EYE	2020-10-09	40.36	40.635	39.57	40.07	626279	{}	\N	2020-10-12 14:50:41.78048
41400	EYE	2020-10-12	40.36	40.53	39.88	40.24	42854	{}	\N	2020-10-12 14:50:41.784836
41401	ETR	2020-09-28	97.6	98.02	96.54	96.68	994409	{}	\N	2020-10-12 14:50:43.534952
41402	ETR	2020-09-29	96.95	97.28	95.145	95.78	1588608	{}	\N	2020-10-12 14:50:43.541215
41403	ETR	2020-09-30	96.6	98.95	96.51	98.53	3184812	{}	\N	2020-10-12 14:50:43.546548
41404	ETR	2020-10-01	98.82	101.35	98.67	101.28	3080334	{}	\N	2020-10-12 14:50:43.55098
41405	ETR	2020-10-02	100.5	104.22	100.17	103.65	4391834	{}	\N	2020-10-12 14:50:43.555229
41406	ETR	2020-10-05	103.42	105.06	102.335	104.81	2249440	{}	\N	2020-10-12 14:50:43.559818
41407	ETR	2020-10-06	105.25	107.86	104.65	106.05	2993802	{}	\N	2020-10-12 14:50:43.564423
41408	ETR	2020-10-07	106.9	107	104.245	104.7	3077781	{}	\N	2020-10-12 14:50:43.569502
41409	ETR	2020-10-08	105.11	107.38	104.93	107.21	1759722	{}	\N	2020-10-12 14:50:43.574236
41410	ETR	2020-10-09	107.3	107.36	105.14	105.85	1865327	{}	\N	2020-10-12 14:50:43.57933
41411	ETR	2020-10-12	105.97	106.23	105.41	105.97	496585	{}	\N	2020-10-12 14:50:43.583813
41412	EVA	2020-09-28	38.6	38.96	38.24	38.57	118321	{}	\N	2020-10-12 14:50:44.697841
41413	EVA	2020-09-29	38.75	39.23	38.4666	39.09	112590	{}	\N	2020-10-12 14:50:44.706742
41414	EVA	2020-09-30	39.4	40.63	39.07	40.26	132938	{}	\N	2020-10-12 14:50:44.712531
41415	EVA	2020-10-01	40.57	41.37	39.86	41.21	743441	{}	\N	2020-10-12 14:50:44.717991
41416	EVA	2020-10-02	40.16	40.8	40	40.35	99426	{}	\N	2020-10-12 14:50:44.722074
41417	EVA	2020-10-05	40.35	40.85	40.0079	40.29	108305	{}	\N	2020-10-12 14:50:44.726387
41418	EVA	2020-10-06	40.28	40.855	39.935	40.26	73863	{}	\N	2020-10-12 14:50:44.73067
41419	EVA	2020-10-07	40.5	40.88	40.2	40.6	103223	{}	\N	2020-10-12 14:50:44.734567
41420	EVA	2020-10-08	41	41.678	40.73	41.2	157059	{}	\N	2020-10-12 14:50:44.738561
41421	EVA	2020-10-09	41.39	41.72	41.06	41.39	94682	{}	\N	2020-10-12 14:50:44.742279
41422	EVA	2020-10-12	41.75	41.98	41.5701	41.84	31239	{}	\N	2020-10-12 14:50:44.746359
41423	EVGN	2020-09-28	4.08	5.3	3.9497	3.97	4398887	{}	\N	2020-10-12 14:50:45.762981
41424	EVGN	2020-09-29	3.94	4.47	3.85	4.08	2403671	{}	\N	2020-10-12 14:50:45.771644
41425	EVGN	2020-09-30	4.39	4.57	3.93	3.96	1636340	{}	\N	2020-10-12 14:50:45.777163
41426	EVGN	2020-10-01	4.06	4.24	3.97	4.05	1048118	{}	\N	2020-10-12 14:50:45.782152
41427	EVGN	2020-10-02	3.69	3.88	3.55	3.7	1377060	{}	\N	2020-10-12 14:50:45.787152
41428	EVGN	2020-10-05	3.905	4.38	3.805	4.16	1411198	{}	\N	2020-10-12 14:50:45.791551
41429	EVGN	2020-10-06	4.26	4.35	3.91	3.96	875394	{}	\N	2020-10-12 14:50:45.795783
41430	EVGN	2020-10-07	3.98	4.31	3.82	4.13	963252	{}	\N	2020-10-12 14:50:45.800064
41431	EVGN	2020-10-08	4.12	4.38	3.97	3.99	681524	{}	\N	2020-10-12 14:50:45.804129
41432	EVGN	2020-10-09	4.05	4.16	3.9	3.9	528056	{}	\N	2020-10-12 14:50:45.808575
41433	EVGN	2020-10-12	3.89	3.89	3.73	3.8577	338922	{}	\N	2020-10-12 14:50:45.812998
41434	EVR	2020-09-28	62.33	63.54	62.17	63.5	568944	{}	\N	2020-10-12 14:50:47.631365
41435	EVR	2020-09-29	63.45	64.75	63.09	63.51	2051238	{}	\N	2020-10-12 14:50:47.636712
41436	EVR	2020-09-30	64.11	65.7	64.11	65.46	797199	{}	\N	2020-10-12 14:50:47.641753
41437	EVR	2020-10-01	66.29	66.78	65.26	66.15	735467	{}	\N	2020-10-12 14:50:47.646121
41438	EVR	2020-10-02	66.4	69.23	65.86	69.01	768778	{}	\N	2020-10-12 14:50:47.650338
41439	EVR	2020-10-05	70	70.685	69.75	70.43	632844	{}	\N	2020-10-12 14:50:47.654628
41440	EVR	2020-10-06	71.28	72.44	69.52	70.18	594565	{}	\N	2020-10-12 14:50:47.658676
41441	EVR	2020-10-07	71.15	71.7525	69.5	70.19	931955	{}	\N	2020-10-12 14:50:47.663503
41442	EVR	2020-10-08	71	72.37	70.65	72.23	540358	{}	\N	2020-10-12 14:50:47.66873
41443	EVR	2020-10-09	73.48	74.28	73.0776	73.83	841659	{}	\N	2020-10-12 14:50:47.673977
41444	EVR	2020-10-12	74.35	76.15	74.15	75.575	112643	{}	\N	2020-10-12 14:50:47.679499
41445	EVX	2020-09-28	99.92	99.9218	99.6259	99.6259	632	{}	\N	2020-10-12 14:50:49.490781
41446	EVX	2020-09-29	98.93	99.3013	98.93	99.3013	583	{}	\N	2020-10-12 14:50:49.498365
41447	EVX	2020-09-30	98.88	100.39	98.88	100.39	491	{}	\N	2020-10-12 14:50:49.504263
41448	EVX	2020-10-01	99.84	99.84	99.8128	99.8128	574	{}	\N	2020-10-12 14:50:49.509058
41449	EVX	2020-10-02	99.4373	100.45	99.4373	100.45	595	{}	\N	2020-10-12 14:50:49.513348
41450	EVX	2020-10-05	100.73	101.1435	100.73	101.1435	1649	{}	\N	2020-10-12 14:50:49.518003
41451	EVX	2020-10-06	102.21	102.21	100.8109	100.8109	986	{}	\N	2020-10-12 14:50:49.522256
41452	EVX	2020-10-07	102.7441	102.7441	102.7441	102.7441	303	{}	\N	2020-10-12 14:50:49.526616
41453	EVX	2020-10-08	103.212	103.212	103	103	972	{}	\N	2020-10-12 14:50:49.531101
41454	EVX	2020-10-09	103.73	104.2	103.58	104.2	1050	{}	\N	2020-10-12 14:50:49.535296
41455	EWJ	2020-09-28	59.41	59.58	59.27	59.58	5906496	{}	\N	2020-10-12 14:50:50.589997
41456	EWJ	2020-09-29	59.66	59.79	59.42	59.53	6495166	{}	\N	2020-10-12 14:50:50.597657
41457	EWJ	2020-09-30	58.88	59.28	58.84	59.07	5548480	{}	\N	2020-10-12 14:50:50.602049
41458	EWJ	2020-10-01	58.94	59.125	58.86	59.08	5595567	{}	\N	2020-10-12 14:50:50.606947
41459	EWJ	2020-10-02	58.311	58.83	58.31	58.79	5086188	{}	\N	2020-10-12 14:50:50.61084
41460	EWJ	2020-10-05	59.055	59.27	59.035	59.25	5184277	{}	\N	2020-10-12 14:50:50.614921
41461	EWJ	2020-10-06	59.39	59.49	58.99	59.14	4697779	{}	\N	2020-10-12 14:50:50.619082
41462	EWJ	2020-10-07	59.345	59.45	59.3	59.435	7602397	{}	\N	2020-10-12 14:50:50.623056
41463	EWJ	2020-10-08	59.62	59.655	59.37	59.48	8764028	{}	\N	2020-10-12 14:50:50.626815
41464	EWJ	2020-10-09	59.4	59.51	59.38	59.46	5947993	{}	\N	2020-10-12 14:50:50.630764
41465	EWJ	2020-10-12	59.5	59.5	59.39	59.47	651327	{}	\N	2020-10-12 14:50:50.634734
41466	EWU	2020-09-28	25.76	25.83	25.635	25.7	2313090	{}	\N	2020-10-12 14:50:51.697351
41467	EWU	2020-09-29	25.58	25.69	25.43	25.52	3086898	{}	\N	2020-10-12 14:50:51.705174
41468	EWU	2020-09-30	25.58	25.74	25.405	25.485	2964057	{}	\N	2020-10-12 14:50:51.710213
41469	EWU	2020-10-01	25.56	25.585	25.36	25.54	2659875	{}	\N	2020-10-12 14:50:51.715989
41470	EWU	2020-10-02	25.37	25.81	25.34	25.74	2617201	{}	\N	2020-10-12 14:50:51.720902
41471	EWU	2020-10-05	25.935	26.09	25.89	26.07	1707315	{}	\N	2020-10-12 14:50:51.725708
41472	EWU	2020-10-06	26.03	26.06	25.61	25.68	2902372	{}	\N	2020-10-12 14:50:51.73013
41473	EWU	2020-10-07	25.82	25.9	25.75	25.85	2922289	{}	\N	2020-10-12 14:50:51.73464
41474	EWU	2020-10-08	26.03	26.135	25.98	26.13	5534320	{}	\N	2020-10-12 14:50:51.739652
41475	EWU	2020-10-09	26.35	26.41	26.25	26.37	1973181	{}	\N	2020-10-12 14:50:51.744587
41476	EWU	2020-10-12	26.37	26.38	26.32	26.36	316205	{}	\N	2020-10-12 14:50:51.74924
41477	EXG	2020-09-28	7.37	7.42	7.37	7.37	550704	{}	\N	2020-10-12 14:50:52.974824
41478	EXG	2020-09-29	7.39	7.4	7.34	7.34	1089042	{}	\N	2020-10-12 14:50:52.982469
41479	EXG	2020-09-30	7.4	7.45	7.34	7.36	2299316	{}	\N	2020-10-12 14:50:52.988509
41480	EXG	2020-10-01	7.4	7.45	7.38	7.41	1026863	{}	\N	2020-10-12 14:50:52.993398
41481	EXG	2020-10-02	7.36	7.53	7.34	7.51	4083580	{}	\N	2020-10-12 14:50:52.998757
41482	EXG	2020-10-05	7.55	7.64	7.55	7.63	1988756	{}	\N	2020-10-12 14:50:53.003966
41483	EXG	2020-10-06	7.7	7.72	7.58	7.58	2073434	{}	\N	2020-10-12 14:50:53.008905
41484	EXG	2020-10-07	7.67	7.74	7.653	7.74	1924608	{}	\N	2020-10-12 14:50:53.015052
41485	EXG	2020-10-08	7.78	7.7827	7.725	7.74	1236674	{}	\N	2020-10-12 14:50:53.020187
41486	EXG	2020-10-09	7.79	7.83	7.76	7.82	1146107	{}	\N	2020-10-12 14:50:53.024998
41487	EXG	2020-10-12	7.87	7.87	7.84	7.85	335041	{}	\N	2020-10-12 14:50:53.030607
41488	EYEN	2020-09-28	3.25	3.25	3.06	3.2	125520	{}	\N	2020-10-12 14:50:54.32481
41489	EYEN	2020-09-29	3.23	3.25	3.16	3.21	61730	{}	\N	2020-10-12 14:50:54.331602
41490	EYEN	2020-09-30	3.2	3.2	3.01	3.12	81080	{}	\N	2020-10-12 14:50:54.336321
41491	EYEN	2020-10-01	3.18	3.18	3.01	3.11	44223	{}	\N	2020-10-12 14:50:54.341617
41492	EYEN	2020-10-02	3.1	3.168	3.06	3.13	89067	{}	\N	2020-10-12 14:50:54.346639
41493	EYEN	2020-10-05	3.15	3.33	3.15	3.26	158128	{}	\N	2020-10-12 14:50:54.351384
41494	EYEN	2020-10-06	3.36	3.43	3.26	3.29	173805	{}	\N	2020-10-12 14:50:54.355777
41495	EYEN	2020-10-07	3.33	3.4	3.27	3.35	115668	{}	\N	2020-10-12 14:50:54.35999
41496	EYEN	2020-10-08	3.39	3.61	3.3647	3.56	192437	{}	\N	2020-10-12 14:50:54.364357
41497	EYEN	2020-10-09	3.6	3.6	3.39	3.39	137383	{}	\N	2020-10-12 14:50:54.368404
41498	EYEN	2020-10-12	3.59	3.708	3.49	3.61	375177	{}	\N	2020-10-12 14:50:54.372703
41499	FAD	2020-09-28	85.789	85.9499	85.44	85.7885	4447	{}	\N	2020-10-12 14:50:55.375678
41500	FAD	2020-09-29	85.989	86.22	85.9123	85.9123	1438	{}	\N	2020-10-12 14:50:55.381641
41501	FAD	2020-09-30	86.6116	86.87	85.45	86.06	4150	{}	\N	2020-10-12 14:50:55.386998
41502	FAD	2020-10-01	87.02	87.5895	86.8203	87.44	24278	{}	\N	2020-10-12 14:50:55.392483
41503	FAD	2020-10-02	86.6457	87.3528	86.6457	87.2436	15061	{}	\N	2020-10-12 14:50:55.397752
41504	FAD	2020-10-05	88.46	88.8741	88.46	88.8741	843	{}	\N	2020-10-12 14:50:55.402165
41505	FAD	2020-10-06	89.19	89.85	88.57	88.57	2463	{}	\N	2020-10-12 14:50:55.406423
41506	FAD	2020-10-07	89.47	90.0354	89.47	90.0354	1863	{}	\N	2020-10-12 14:50:55.41072
41507	FAD	2020-10-08	90.2331	90.3383	90.0521	90.165	3054	{}	\N	2020-10-12 14:50:55.415078
41508	FAD	2020-10-09	91.22	91.5691	91.22	91.5691	2568	{}	\N	2020-10-12 14:50:55.419601
41509	FAD	2020-10-12	91.85	92.042	91.85	92.042	26509	{}	\N	2020-10-12 14:50:55.423884
41510	FAS	2020-09-28	32.98	34.73	32.9301	34.01	3590928	{}	\N	2020-10-12 14:50:56.563134
41511	FAS	2020-09-29	33.89	33.89	32.26	32.87	2750966	{}	\N	2020-10-12 14:50:56.569576
41512	FAS	2020-09-30	33.09	34.81	33.08	33.87	3354922	{}	\N	2020-10-12 14:50:56.573908
41513	FAS	2020-10-01	34.48	35.14	33.48	34.28	2003582	{}	\N	2020-10-12 14:50:56.577889
41514	FAS	2020-10-02	32.37	35.66	32.35	35.04	3855816	{}	\N	2020-10-12 14:50:56.582261
41515	FAS	2020-10-05	35.98	37.175	35.8701	36.86	3234967	{}	\N	2020-10-12 14:50:56.58702
41516	FAS	2020-10-06	37.51	38.57	35.5311	35.8	4504503	{}	\N	2020-10-12 14:50:56.592586
41517	FAS	2020-10-07	37.19	38.1	36.79	37.54	3002577	{}	\N	2020-10-12 14:50:56.598155
41518	FAS	2020-10-08	38.22	39.42	38.04	39.12	2656031	{}	\N	2020-10-12 14:50:56.603205
41519	FAS	2020-10-09	39.77	39.91	38.4666	39.21	2841891	{}	\N	2020-10-12 14:50:56.607419
41520	FAS	2020-10-12	39.54	40.22	39.24	40.09	834475	{}	\N	2020-10-12 14:50:56.611843
41521	EURN	2020-09-28	8.85	8.87	8.62	8.68	1426087	{}	\N	2020-10-12 14:50:58.11137
41522	EURN	2020-09-29	8.77	8.85	8.68	8.77	2685709	{}	\N	2020-10-12 14:50:58.118496
41523	EURN	2020-09-30	8.82	8.93	8.8	8.83	1887383	{}	\N	2020-10-12 14:50:58.123688
41524	EURN	2020-10-01	8.8	8.88	8.71	8.85	1286377	{}	\N	2020-10-12 14:50:58.127708
41525	EURN	2020-10-02	8.79	8.97	8.7	8.88	1815418	{}	\N	2020-10-12 14:50:58.131477
41526	EURN	2020-10-05	8.99	9.19	8.9601	9.18	1740231	{}	\N	2020-10-12 14:50:58.135467
41527	EURN	2020-10-06	9.15	9.235	9.01	9.06	1772109	{}	\N	2020-10-12 14:50:58.139372
41528	EURN	2020-10-07	8.95	9.02	8.87	8.88	2475994	{}	\N	2020-10-12 14:50:58.143842
41529	EURN	2020-10-08	8.9	9.01	8.875	8.95	2503410	{}	\N	2020-10-12 14:50:58.149241
41530	EURN	2020-10-09	9.21	9.255	9.06	9.11	2686767	{}	\N	2020-10-12 14:50:58.153974
41531	EURN	2020-10-12	8.9	8.905	8.745	8.805	1487370	{}	\N	2020-10-12 14:50:58.159275
41532	EVC	2020-09-28	1.28	1.34	1.28	1.32	146060	{}	\N	2020-10-12 14:50:59.103668
41533	EVC	2020-09-29	1.31	1.52	1.31	1.49	761626	{}	\N	2020-10-12 14:50:59.110812
41534	EVC	2020-09-30	1.51	1.55	1.48	1.52	668713	{}	\N	2020-10-12 14:50:59.115587
41535	EVC	2020-10-01	1.51	1.57	1.51	1.55	350603	{}	\N	2020-10-12 14:50:59.119821
41536	EVC	2020-10-02	1.52	1.56	1.5	1.55	239122	{}	\N	2020-10-12 14:50:59.124013
41537	EVC	2020-10-05	1.55	1.57	1.52	1.54	1025313	{}	\N	2020-10-12 14:50:59.127969
41538	EVC	2020-10-06	1.56	1.6	1.53	1.59	420696	{}	\N	2020-10-12 14:50:59.132027
41539	EVC	2020-10-07	1.59	1.72	1.58	1.7	824891	{}	\N	2020-10-12 14:50:59.13593
41540	EVC	2020-10-08	1.72	1.72	1.59	1.69	445654	{}	\N	2020-10-12 14:50:59.140046
41541	EVC	2020-10-09	1.71	1.7681	1.69	1.76	436763	{}	\N	2020-10-12 14:50:59.144118
41542	EVC	2020-10-12	1.78	1.78	1.75	1.753	40152	{}	\N	2020-10-12 14:50:59.148194
41543	EWBC	2020-09-28	32.4	32.965	32.21	32.64	677897	{}	\N	2020-10-12 14:51:00.920029
41544	EWBC	2020-09-29	32.43	32.58	31.74	32.46	1238848	{}	\N	2020-10-12 14:51:00.926573
41545	EWBC	2020-09-30	32.27	33.22	32.27	32.74	848316	{}	\N	2020-10-12 14:51:00.931508
41546	EWBC	2020-10-01	32.83	33.38	32.56	32.82	593807	{}	\N	2020-10-12 14:51:00.936733
41547	EWBC	2020-10-02	32	33.9	31.9	33.57	512033	{}	\N	2020-10-12 14:51:00.941386
41548	EWBC	2020-10-05	34.06	35.66	33.84	35.61	919111	{}	\N	2020-10-12 14:51:00.946154
41549	EWBC	2020-10-06	35.695	37.49	35.69	35.77	1162654	{}	\N	2020-10-12 14:51:00.950557
41550	EWBC	2020-10-07	36.14	37.53	36.14	36.9	751352	{}	\N	2020-10-12 14:51:00.955185
41551	EWBC	2020-10-08	37.43	37.54	36.84	37.47	590587	{}	\N	2020-10-12 14:51:00.959987
41552	EWBC	2020-10-09	37.2	37.99	36.93	37.2	694548	{}	\N	2020-10-12 14:51:00.966737
41553	EWBC	2020-10-12	36.9	37.33	36.731	37.1	81607	{}	\N	2020-10-12 14:51:00.971791
41554	EWL	2020-09-28	41.17	41.34	41.095	41.28	738032	{}	\N	2020-10-12 14:51:02.150454
41555	EWL	2020-09-29	41.28	41.4	41.09	41.22	524080	{}	\N	2020-10-12 14:51:02.158277
41556	EWL	2020-09-30	41.25	41.49	40.955	41.06	620848	{}	\N	2020-10-12 14:51:02.16298
41557	EWL	2020-10-01	41.54	41.54	41.255	41.37	752051	{}	\N	2020-10-12 14:51:02.167611
41558	EWL	2020-10-02	41.09	41.57	41.09	41.5	702009	{}	\N	2020-10-12 14:51:02.17182
41559	EWL	2020-10-05	41.75	41.98	41.74	41.98	373394	{}	\N	2020-10-12 14:51:02.175999
41560	EWL	2020-10-06	41.73	41.735	41.15	41.25	960836	{}	\N	2020-10-12 14:51:02.180273
41561	EWL	2020-10-07	41.4	41.46	41.25	41.41	693078	{}	\N	2020-10-12 14:51:02.184521
41562	EWL	2020-10-08	41.6	41.73	41.55	41.72	558871	{}	\N	2020-10-12 14:51:02.188653
41563	EWL	2020-10-09	42.04	42.25	41.98	42.24	758974	{}	\N	2020-10-12 14:51:02.192624
41564	EWL	2020-10-12	42.38	42.4613	42.33	42.43	103706	{}	\N	2020-10-12 14:51:02.196696
41565	EWW	2020-09-28	32.6	32.86	32.175	32.65	2105280	{}	\N	2020-10-12 14:51:04.074322
41566	EWW	2020-09-29	32.85	32.92	32.42	32.59	1806376	{}	\N	2020-10-12 14:51:04.08038
41567	EWW	2020-09-30	32.88	33.71	32.79	33.36	2694543	{}	\N	2020-10-12 14:51:04.085444
41568	EWW	2020-10-01	33.81	33.91	32.96	33.1	3086467	{}	\N	2020-10-12 14:51:04.089659
41569	EWW	2020-10-02	32.55	33.52	32.5	33.26	2163019	{}	\N	2020-10-12 14:51:04.093438
41570	EWW	2020-10-05	33.68	34.27	33.68	33.98	2274337	{}	\N	2020-10-12 14:51:04.097335
41571	EWW	2020-10-06	34.19	34.27	33.4	33.58	2716510	{}	\N	2020-10-12 14:51:04.101671
41572	EWW	2020-10-07	34.23	34.39	34.13	34.3	2159206	{}	\N	2020-10-12 14:51:04.105498
41573	EWW	2020-10-08	34.52	35.38	34.31	35.35	4801711	{}	\N	2020-10-12 14:51:04.109134
41574	EWW	2020-10-09	35.62	36.03	35.37	35.82	4201250	{}	\N	2020-10-12 14:51:04.113054
41575	EWW	2020-10-12	35.92	35.92	35.6	35.725	592943	{}	\N	2020-10-12 14:51:04.116978
41576	EXEL	2020-09-28	24.18	24.23	23.44	23.73	1093985	{}	\N	2020-10-12 14:51:05.536112
41577	EXEL	2020-09-29	23.825	24.72	23.7	24.5	2013450	{}	\N	2020-10-12 14:51:05.544184
41578	EXEL	2020-09-30	24.49	24.715	24	24.45	1559690	{}	\N	2020-10-12 14:51:05.551039
41579	EXEL	2020-10-01	24.58	24.8	24.335	24.54	1140522	{}	\N	2020-10-12 14:51:05.554879
41580	EXEL	2020-10-02	24.14	24.5011	23.37	23.4	1710847	{}	\N	2020-10-12 14:51:05.558869
41581	EXEL	2020-10-05	23.65	24.19	23.65	23.98	1516842	{}	\N	2020-10-12 14:51:05.562821
41582	EXEL	2020-10-06	24.03	24.35	23.475	23.5136	1008442	{}	\N	2020-10-12 14:51:05.566938
41583	EXEL	2020-10-07	23.65	24.3	23.61	23.855	1084747	{}	\N	2020-10-12 14:51:05.570991
41584	EXEL	2020-10-08	24.19	25.08	23.98	24.795	1365441	{}	\N	2020-10-12 14:51:05.574687
41585	EXEL	2020-10-09	25.06	25.1199	23.7	23.91	2457072	{}	\N	2020-10-12 14:51:05.578255
41586	EXEL	2020-10-12	24	24.19	23.885	24.06	380859	{}	\N	2020-10-12 14:51:05.582635
41587	EXR	2020-09-28	107.02	108.73	106.37	108.12	808450	{}	\N	2020-10-12 14:51:08.347221
41588	EXR	2020-09-29	107.63	108.56	106.39	107.24	1220608	{}	\N	2020-10-12 14:51:08.354989
41589	EXR	2020-09-30	107.69	108.3	106.135	106.99	2035945	{}	\N	2020-10-12 14:51:08.360295
41590	EXR	2020-10-01	107.31	109.78	107.09	109.24	2069882	{}	\N	2020-10-12 14:51:08.364726
41591	EXR	2020-10-02	108.69	112.86	107.48	112.39	2735302	{}	\N	2020-10-12 14:51:08.369079
41592	EXR	2020-10-05	113	113.14	108.61	112.79	2358874	{}	\N	2020-10-12 14:51:08.373084
41593	EXR	2020-10-06	113.43	114.29	110.95	111.95	3414036	{}	\N	2020-10-12 14:51:08.376914
41594	EXR	2020-10-07	112.58	113.19	111.28	112.53	2117918	{}	\N	2020-10-12 14:51:08.380838
41595	EXR	2020-10-08	112.76	113.96	112.59	113.75	2050450	{}	\N	2020-10-12 14:51:08.384807
41596	EXR	2020-10-09	114.47	114.54	112.18	113.35	2233828	{}	\N	2020-10-12 14:51:08.388779
41597	EXR	2020-10-12	113.42	113.82	111.75	113.075	261943	{}	\N	2020-10-12 14:51:08.392845
41598	EZJ	2020-09-28	36.2412	36.2412	36.2412	36.2412	294	{}	\N	2020-10-12 14:51:10.013336
41599	EZJ	2020-09-29	36.34	36.43	36.2	36.23	898	{}	\N	2020-10-12 14:51:10.022373
41600	EZJ	2020-09-30	35.795	35.84	35.7585	35.7585	1899	{}	\N	2020-10-12 14:51:10.031902
41601	EZJ	2020-10-01	35.65	35.73	35.55	35.55	2715	{}	\N	2020-10-12 14:51:10.039577
41602	EZJ	2020-10-02	35.1325	35.2913	35.1325	35.2913	253	{}	\N	2020-10-12 14:51:10.048253
41603	EZJ	2020-10-05	35.68	35.8605	35.64	35.8605	2435	{}	\N	2020-10-12 14:51:10.056147
41604	EZJ	2020-10-06	36.0302	36.14	35.74	35.86	2000	{}	\N	2020-10-12 14:51:10.06455
41605	EZJ	2020-10-07	36.35	36.35	36.04	36.04	1598	{}	\N	2020-10-12 14:51:10.071969
41606	EZJ	2020-10-08	36.0881	36.0881	36.0881	36.0881	16	{}	\N	2020-10-12 14:51:10.079032
41607	EZJ	2020-10-09	36.1	36.131	36.1	36.131	1154	{}	\N	2020-10-12 14:51:10.084573
41608	FALN	2020-09-28	27.17	27.27	27.09	27.19	110689	{}	\N	2020-10-12 14:51:11.55024
41609	FALN	2020-09-29	27.17	27.18	27.11	27.18	134833	{}	\N	2020-10-12 14:51:11.555891
41610	FALN	2020-09-30	27.26	27.3	27.18	27.3	152627	{}	\N	2020-10-12 14:51:11.560543
41611	FALN	2020-10-01	27.25	27.35	27.19	27.2717	1003191	{}	\N	2020-10-12 14:51:11.564848
41612	FALN	2020-10-02	27.21	27.32	27.1883	27.31	116713	{}	\N	2020-10-12 14:51:11.569199
41613	FALN	2020-10-05	27.26	27.48	27.25	27.415	81874	{}	\N	2020-10-12 14:51:11.573486
41614	FALN	2020-10-06	27.485	27.54	27.43	27.435	75109	{}	\N	2020-10-12 14:51:11.578304
41615	FALN	2020-10-07	27.57	27.58	27.48	27.55	318084	{}	\N	2020-10-12 14:51:11.582749
41616	FALN	2020-10-08	27.63	27.66	27.53	27.66	205822	{}	\N	2020-10-12 14:51:11.587369
41617	FALN	2020-10-09	27.67	27.7	27.64	27.69	154919	{}	\N	2020-10-12 14:51:11.591421
41618	FALN	2020-10-12	27.77	27.77	27.655	27.73	38711	{}	\N	2020-10-12 14:51:11.595467
41619	FATE	2020-09-28	38.23	40.52	37.89	40.5	936572	{}	\N	2020-10-12 14:51:12.809487
41620	FATE	2020-09-29	40.25	41.36	39.09	39.4	1365350	{}	\N	2020-10-12 14:51:12.815771
41621	FATE	2020-09-30	39.4	41.18	39.22	39.97	805033	{}	\N	2020-10-12 14:51:12.820935
41622	FATE	2020-10-01	40.3	42.223	39.65	40.6	1147924	{}	\N	2020-10-12 14:51:12.82552
41623	FATE	2020-10-02	39.16	40.08	38.02	38.09	1153507	{}	\N	2020-10-12 14:51:12.830368
41624	FATE	2020-10-05	38.39	42	38.09	41.89	1100212	{}	\N	2020-10-12 14:51:12.834447
41625	FATE	2020-10-06	41.64	42.55	41.38	41.56	823131	{}	\N	2020-10-12 14:51:12.838991
41626	FATE	2020-10-07	42.31	44.3491	41.76	44.04	757407	{}	\N	2020-10-12 14:51:12.843226
41627	FATE	2020-10-08	44.72	45.3	44.1925	44.98	523655	{}	\N	2020-10-12 14:51:12.849029
41628	FATE	2020-10-09	45.09	48.1	44.64	48.01	740938	{}	\N	2020-10-12 14:51:12.853743
41629	FATE	2020-10-12	48.73	49.05	47.34	48.3134	112785	{}	\N	2020-10-12 14:51:12.85782
41630	FBMS	2020-09-28	20.1	20.91	19.88	20.83	58455	{}	\N	2020-10-12 14:51:13.878667
41631	FBMS	2020-09-29	20.57	21.1	20.42	21.03	53265	{}	\N	2020-10-12 14:51:13.886889
41632	FBMS	2020-09-30	21.12	21.41	20.75	20.97	45191	{}	\N	2020-10-12 14:51:13.89275
41633	FBMS	2020-10-01	21.05	21.42	20.66	21.37	48630	{}	\N	2020-10-12 14:51:13.898707
41634	FBMS	2020-10-02	21.09	21.905	21.09	21.82	38132	{}	\N	2020-10-12 14:51:13.903821
41635	FBMS	2020-10-05	21.97	22.75	21.38	22.44	44236	{}	\N	2020-10-12 14:51:13.908075
41636	FBMS	2020-10-06	22.75	23.05	22.045	22.28	91042	{}	\N	2020-10-12 14:51:13.912382
41637	FBMS	2020-10-07	22.63	23.41	22.03	23.38	56680	{}	\N	2020-10-12 14:51:13.916536
41638	FBMS	2020-10-08	23.38	23.55	22.65	23.42	39292	{}	\N	2020-10-12 14:51:13.920779
41639	FBMS	2020-10-09	23.5	23.61	23.14	23.58	32463	{}	\N	2020-10-12 14:51:13.925126
41640	FBMS	2020-10-12	23.25	23.76	23.095	23.75	3175	{}	\N	2020-10-12 14:51:13.929683
41641	EUSC	2020-09-28	27.26	27.31	27.26	27.3	7014	{}	\N	2020-10-12 14:51:15.226088
41642	EUSC	2020-09-29	27.1359	27.1759	27.1359	27.1759	1155	{}	\N	2020-10-12 14:51:15.231831
41643	EUSC	2020-09-30	27.351	27.351	27.21	27.265	5543	{}	\N	2020-10-12 14:51:15.23668
41644	EUSC	2020-10-01	27.455	27.455	27.335	27.35	1658	{}	\N	2020-10-12 14:51:15.240652
41645	EUSC	2020-10-02	27.18	27.454	27.18	27.454	1898	{}	\N	2020-10-12 14:51:15.245889
41646	EUSC	2020-10-05	27.65	27.78	27.65	27.78	9122	{}	\N	2020-10-12 14:51:15.250511
41647	EUSC	2020-10-06	27.96	27.96	27.7	27.71	9317	{}	\N	2020-10-12 14:51:15.255734
41648	EUSC	2020-10-07	27.83	27.8517	27.79	27.84	8703	{}	\N	2020-10-12 14:51:15.260267
41649	EUSC	2020-10-08	27.97	28.042	27.96	27.98	2653	{}	\N	2020-10-12 14:51:15.264935
41650	EUSC	2020-10-09	28.05	28.14	28.015	28.03	8572	{}	\N	2020-10-12 14:51:15.269207
41651	EUSC	2020-10-12	28.16	28.16	28.142	28.142	954	{}	\N	2020-10-12 14:51:15.273737
41652	EVG	2020-09-28	11.49	11.49	11.42	11.44	27150	{}	\N	2020-10-12 14:51:16.266356
41653	EVG	2020-09-29	11.47	11.47	11.38	11.41	21496	{}	\N	2020-10-12 14:51:16.27291
41654	EVG	2020-09-30	11.47	11.49	11.41	11.45	43898	{}	\N	2020-10-12 14:51:16.278311
41655	EVG	2020-10-01	11.42	11.48	11.42	11.48	40802	{}	\N	2020-10-12 14:51:16.283189
41656	EVG	2020-10-02	11.42	11.5062	11.42	11.5	84558	{}	\N	2020-10-12 14:51:16.287535
41657	EVG	2020-10-05	11.5	11.56	11.4691	11.56	25354	{}	\N	2020-10-12 14:51:16.291975
41658	EVG	2020-10-06	11.56	11.62	11.4786	11.62	62054	{}	\N	2020-10-12 14:51:16.297297
41659	EVG	2020-10-07	11.64	11.72	11.6355	11.67	53571	{}	\N	2020-10-12 14:51:16.301945
41660	EVG	2020-10-08	11.77	11.99	11.77	11.99	603342	{}	\N	2020-10-12 14:51:16.306879
41661	EVG	2020-10-09	11.95	12.06	11.95	12.06	294752	{}	\N	2020-10-12 14:51:16.311196
41662	EVG	2020-10-12	12.08	12.0924	12.06	12.06	38078	{}	\N	2020-10-12 14:51:16.315718
41663	EVM	2020-09-28	11.44	11.47	11.34	11.39	71428	{}	\N	2020-10-12 14:51:17.994318
41664	EVM	2020-09-29	11.31	11.39	11.31	11.39	24910	{}	\N	2020-10-12 14:51:18.001499
41665	EVM	2020-09-30	11.45	11.45	11.32	11.36	36534	{}	\N	2020-10-12 14:51:18.00644
41666	EVM	2020-10-01	11.32	11.43	11.31	11.43	33765	{}	\N	2020-10-12 14:51:18.012139
41667	EVM	2020-10-02	11.33	11.47	11.31	11.46	40091	{}	\N	2020-10-12 14:51:18.016292
41668	EVM	2020-10-05	11.435	11.4652	11.33	11.45	12189	{}	\N	2020-10-12 14:51:18.020906
41669	EVM	2020-10-06	11.35	11.52	11.33	11.48	20454	{}	\N	2020-10-12 14:51:18.026078
41670	EVM	2020-10-07	11.59	11.62	11.41	11.41	41094	{}	\N	2020-10-12 14:51:18.030925
41671	EVM	2020-10-08	11.56	11.68	11.56	11.68	43569	{}	\N	2020-10-12 14:51:18.036276
41672	EVM	2020-10-09	11.69	11.69	11.63	11.68	22690	{}	\N	2020-10-12 14:51:18.042156
41673	EVM	2020-10-12	11.69	11.69	11.68	11.68	918	{}	\N	2020-10-12 14:51:18.048346
41674	EVY	2020-09-28	12.76	12.86	12.76	12.77	12371	{}	\N	2020-10-12 14:51:19.806121
41675	EVY	2020-09-29	12.74	12.7401	12.74	12.74	2280	{}	\N	2020-10-12 14:51:19.811766
41676	EVY	2020-09-30	12.75	12.8	12.75	12.8	1332	{}	\N	2020-10-12 14:51:19.816726
41677	EVY	2020-10-01	12.8	12.83	12.77	12.7917	5469	{}	\N	2020-10-12 14:51:19.82095
41678	EVY	2020-10-02	12.75	12.76	12.75	12.76	969	{}	\N	2020-10-12 14:51:19.824986
41679	EVY	2020-10-05	12.82	12.82	12.69	12.72	9377	{}	\N	2020-10-12 14:51:19.829046
41680	EVY	2020-10-06	12.8	12.8	12.7361	12.7361	1106	{}	\N	2020-10-12 14:51:19.833434
41681	EVY	2020-10-07	12.72	12.77	12.72	12.77	2492	{}	\N	2020-10-12 14:51:19.837583
41682	EVY	2020-10-08	12.89	13.05	12.89	12.9907	54331	{}	\N	2020-10-12 14:51:19.842086
41683	EVY	2020-10-09	12.99	13.03	12.99	13.02	15496	{}	\N	2020-10-12 14:51:19.846219
41684	EVY	2020-10-12	13	13.0256	13	13.02	5065	{}	\N	2020-10-12 14:51:19.850066
41685	EWI	2020-09-28	24.46	24.55	24.43	24.55	391113	{}	\N	2020-10-12 14:51:21.112696
41686	EWI	2020-09-29	24.6	24.67	24.39	24.5	1266922	{}	\N	2020-10-12 14:51:21.11687
41687	EWI	2020-09-30	24.45	24.6	24.32	24.38	200110	{}	\N	2020-10-12 14:51:21.120825
41688	EWI	2020-10-01	24.48	24.5	24.3	24.49	493348	{}	\N	2020-10-12 14:51:21.124855
41689	EWI	2020-10-02	24.13	24.51	24.13	24.47	270025	{}	\N	2020-10-12 14:51:21.128641
41690	EWI	2020-10-05	24.65	24.97	24.64	24.97	290576	{}	\N	2020-10-12 14:51:21.132679
41691	EWI	2020-10-06	25.11	25.13	24.67	24.72	918097	{}	\N	2020-10-12 14:51:21.136837
41692	EWI	2020-10-07	24.97	25.075	24.93	25.04	248007	{}	\N	2020-10-12 14:51:21.141028
41693	EWI	2020-10-08	25.08	25.185	25.07	25.18	617181	{}	\N	2020-10-12 14:51:21.145823
41694	EWI	2020-10-09	25.28	25.31	25.15	25.24	433833	{}	\N	2020-10-12 14:51:21.150097
41695	EWI	2020-10-12	25.3	25.32	25.27	25.31	25132	{}	\N	2020-10-12 14:51:21.15385
41696	EWS	2020-09-28	18.65	18.69	18.58	18.67	488517	{}	\N	2020-10-12 14:51:22.092971
41697	EWS	2020-09-29	18.53	18.63	18.505	18.59	1320700	{}	\N	2020-10-12 14:51:22.100106
41698	EWS	2020-09-30	18.61	18.785	18.59	18.71	516208	{}	\N	2020-10-12 14:51:22.106455
41699	EWS	2020-10-01	18.88	18.97	18.821	18.94	1063287	{}	\N	2020-10-12 14:51:22.112321
41700	EWS	2020-10-02	18.8	18.975	18.8	18.93	1039637	{}	\N	2020-10-12 14:51:22.117842
41701	EWS	2020-10-05	19	19.13	18.99	19.13	790136	{}	\N	2020-10-12 14:51:22.122561
41702	EWS	2020-10-06	19.1808	19.25	19.1	19.115	1733720	{}	\N	2020-10-12 14:51:22.12854
41703	EWS	2020-10-07	19.21	19.25	19.18	19.23	1486882	{}	\N	2020-10-12 14:51:22.133124
41704	EWS	2020-10-08	19.28	19.29	19.19	19.28	498756	{}	\N	2020-10-12 14:51:22.136935
41705	EWS	2020-10-09	19.26	19.365	19.21	19.34	759711	{}	\N	2020-10-12 14:51:22.14085
41706	EWS	2020-10-12	19.38	19.39	19.34	19.3889	242484	{}	\N	2020-10-12 14:51:22.144664
41707	EXAS	2020-09-28	98.2883	100.9	96.55	99.7	2644611	{}	\N	2020-10-12 14:51:23.456913
41708	EXAS	2020-09-29	99.99	102.75	99.65	102.01	2461926	{}	\N	2020-10-12 14:51:23.463477
41709	EXAS	2020-09-30	102	103	101.02	102	2124015	{}	\N	2020-10-12 14:51:23.46875
41710	EXAS	2020-10-01	103.05	105.39	102.06	105.21	1800704	{}	\N	2020-10-12 14:51:23.474034
41711	EXAS	2020-10-02	103	108.78	103	105.6	2404171	{}	\N	2020-10-12 14:51:23.479015
41712	EXAS	2020-10-05	106.14	110.26	106	106.9	1669179	{}	\N	2020-10-12 14:51:23.482997
41713	EXAS	2020-10-06	106.91	107.8699	102.52	103.75	1765332	{}	\N	2020-10-12 14:51:23.486621
41714	EXAS	2020-10-07	105.22	109.035	105	108.43	1590410	{}	\N	2020-10-12 14:51:23.490778
41715	EXAS	2020-10-08	110.74	110.74	106.6	108.38	1005905	{}	\N	2020-10-12 14:51:23.494759
41716	EXAS	2020-10-09	108.35	110.595	107.36	107.94	609955	{}	\N	2020-10-12 14:51:23.498334
41717	EXAS	2020-10-12	108.54	108.86	107.42	108.29	96720	{}	\N	2020-10-12 14:51:23.501955
41718	EXPE	2020-09-28	92.99	95.2587	91.67	94.29	2152937	{}	\N	2020-10-12 14:51:24.825974
41719	EXPE	2020-09-29	94.04	94.05	91.06	91.78	2151415	{}	\N	2020-10-12 14:51:24.833208
41720	EXPE	2020-09-30	92.3	93.8653	90.42	91.69	2210356	{}	\N	2020-10-12 14:51:24.837999
41721	EXPE	2020-10-01	92.55	94.725	91.95	92.06	2309671	{}	\N	2020-10-12 14:51:24.84347
41722	EXPE	2020-10-02	89.215	92.51	88.42	92.1	1639058	{}	\N	2020-10-12 14:51:24.847939
41723	EXPE	2020-10-05	93.1	93.47	90.9	92.16	1258054	{}	\N	2020-10-12 14:51:24.852115
41724	EXPE	2020-10-06	92.55	95.57	91.4866	91.87	1926918	{}	\N	2020-10-12 14:51:24.856097
41725	EXPE	2020-10-07	93.73	94.27	92.43	93.49	1366627	{}	\N	2020-10-12 14:51:24.860391
41726	EXPE	2020-10-08	94.06	96.05	93.022	95.95	1242295	{}	\N	2020-10-12 14:51:24.864359
41727	EXPE	2020-10-09	97.22	97.49	95.095	96.3	1219338	{}	\N	2020-10-12 14:51:24.868173
41728	EXPE	2020-10-12	97.62	97.63	96.77	97.39	233363	{}	\N	2020-10-12 14:51:24.871986
41729	EYEG	2020-09-28	3.92	3.92	3.77	3.8	10439	{}	\N	2020-10-12 14:51:26.175796
41730	EYEG	2020-09-29	3.89	3.92	3.8543	3.91	31273	{}	\N	2020-10-12 14:51:26.181841
41731	EYEG	2020-09-30	3.86	3.98	3.86	3.92	5510	{}	\N	2020-10-12 14:51:26.186431
41732	EYEG	2020-10-01	3.95	4.18	3.88	3.95	23502	{}	\N	2020-10-12 14:51:26.190676
41733	EYEG	2020-10-02	3.88	4.04	3.68	3.82	21631	{}	\N	2020-10-12 14:51:26.194855
41734	EYEG	2020-10-05	3.8379	3.86	3.8379	3.86	1216	{}	\N	2020-10-12 14:51:26.199
41735	EYEG	2020-10-06	3.9339	3.97	3.86	3.88	7102	{}	\N	2020-10-12 14:51:26.203415
41736	EYEG	2020-10-07	3.82	3.8785	3.72	3.87	13204	{}	\N	2020-10-12 14:51:26.207287
41737	EYEG	2020-10-08	3.78	3.9062	3.78	3.89	3073	{}	\N	2020-10-12 14:51:26.211194
41738	EYEG	2020-10-09	3.8631	4.012	3.789	3.84	8532	{}	\N	2020-10-12 14:51:26.215067
41739	EZT	2020-09-28	25.26	25.28	25.22	25.26	46522	{}	\N	2020-10-12 14:51:28.711985
41740	EZT	2020-09-29	25.26	25.26	25.21	25.26	75444	{}	\N	2020-10-12 14:51:28.717982
41741	EZT	2020-09-30	25.26	25.2628	25.19	25.19	73658	{}	\N	2020-10-12 14:51:28.722756
41742	EZT	2020-10-01	25.26	25.265	25.23	25.26	68789	{}	\N	2020-10-12 14:51:28.727176
41743	EZT	2020-10-02	25.23	25.26	25.23	25.26	39026	{}	\N	2020-10-12 14:51:28.732081
41744	EZT	2020-10-05	25.25	25.26	25.23	25.25	14388	{}	\N	2020-10-12 14:51:28.736884
41745	EZT	2020-10-06	25.24	25.24	25.2	25.24	94692	{}	\N	2020-10-12 14:51:28.741387
41746	EZT	2020-10-07	25.24	25.24	25.21	25.24	12538	{}	\N	2020-10-12 14:51:28.746072
41747	EZT	2020-10-08	25.24	25.24	25.22	25.2375	18330	{}	\N	2020-10-12 14:51:28.750384
41748	EZT	2020-10-09	25.24	25.24	25.2132	25.2399	12072	{}	\N	2020-10-12 14:51:28.754698
41749	EZT	2020-10-12	25.24	25.24	25.22	25.22	4202	{}	\N	2020-10-12 14:51:28.758858
41750	FAM	2020-09-28	9.69	9.757	9.68	9.73	39919	{}	\N	2020-10-12 14:51:29.699861
41751	FAM	2020-09-29	9.73	9.74	9.68	9.69	15858	{}	\N	2020-10-12 14:51:29.706172
41752	FAM	2020-09-30	9.6827	9.7024	9.57	9.57	122353	{}	\N	2020-10-12 14:51:29.711452
41753	FAM	2020-10-01	9.6	9.69	9.6	9.63	20596	{}	\N	2020-10-12 14:51:29.716158
41754	FAM	2020-10-02	9.52	9.57	9.47	9.55	121039	{}	\N	2020-10-12 14:51:29.721246
41755	FAM	2020-10-05	9.56	9.6	9.555	9.59	94253	{}	\N	2020-10-12 14:51:29.727022
41756	FAM	2020-10-06	9.56	9.6999	9.55	9.6	37360	{}	\N	2020-10-12 14:51:29.731556
41757	FAM	2020-10-07	9.72	9.72	9.631	9.66	37690	{}	\N	2020-10-12 14:51:29.736245
41758	FAM	2020-10-08	9.67	9.73	9.63	9.71	72830	{}	\N	2020-10-12 14:51:29.740483
41759	FAM	2020-10-09	9.69	9.755	9.69	9.71	38614	{}	\N	2020-10-12 14:51:29.745028
41760	FAM	2020-10-12	9.71	9.7764	9.71	9.71	6432	{}	\N	2020-10-12 14:51:29.750025
41761	FAST	2020-09-28	45.44	45.985	45.24	45.77	2627447	{}	\N	2020-10-12 14:51:30.663833
41762	FAST	2020-09-29	45.72	46.13	45.1669	45.33	2812377	{}	\N	2020-10-12 14:51:30.671123
41763	FAST	2020-09-30	45.59	45.855	44.84	45.09	3270516	{}	\N	2020-10-12 14:51:30.676325
41764	FAST	2020-10-01	45.1	45.38	44.65	44.75	2477060	{}	\N	2020-10-12 14:51:30.681203
41765	FAST	2020-10-02	44.26	44.83	44.1	44.54	3487860	{}	\N	2020-10-12 14:51:30.686207
41766	FAST	2020-10-05	44.69	45.985	44.69	45.83	2934900	{}	\N	2020-10-12 14:51:30.690778
41767	FAST	2020-10-06	45.7	46.14	45.04	45.13	3230196	{}	\N	2020-10-12 14:51:30.694822
41768	FAST	2020-10-07	45.44	46.34	45.355	46.21	2620229	{}	\N	2020-10-12 14:51:30.700718
41769	FAST	2020-10-08	46.4	46.585	45.92	46	2813990	{}	\N	2020-10-12 14:51:30.704953
41770	FAST	2020-10-09	46.22	47.085	46.17	46.63	2937249	{}	\N	2020-10-12 14:51:30.708968
41771	FAST	2020-10-12	47.11	47.86	47.02	47.83	1275098	{}	\N	2020-10-12 14:51:30.713252
41772	FB	2020-09-28	259.4	259.6	254.8208	256.82	18213618	{}	\N	2020-10-12 14:51:32.030889
41773	FB	2020-09-29	257.81	263.27	256.9	261.79	19779692	{}	\N	2020-10-12 14:51:32.038544
41774	FB	2020-09-30	261.98	266.085	260.1	261.9	19068837	{}	\N	2020-10-12 14:51:32.044977
41775	FB	2020-10-01	265.35	268.33	264.8	266.63	19345403	{}	\N	2020-10-12 14:51:32.050406
41776	FB	2020-10-02	261.18	265.15	259.18	259.94	15503283	{}	\N	2020-10-12 14:51:32.055168
41777	FB	2020-10-05	262.2	264.865	260.8401	264.65	12834929	{}	\N	2020-10-12 14:51:32.059996
41778	FB	2020-10-06	262.01	265.69	258.24	258.66	17703302	{}	\N	2020-10-12 14:51:32.064522
41779	FB	2020-10-07	259.2	260.18	254.82	258.12	22992326	{}	\N	2020-10-12 14:51:32.069282
41780	FB	2020-10-08	259.63	264.62	259.1488	263.76	16036226	{}	\N	2020-10-12 14:51:32.073392
41781	FB	2020-10-09	264.52	264.75	262.17	264.45	13621081	{}	\N	2020-10-12 14:51:32.077798
41782	FB	2020-10-12	270.2	272.279	267.87	271.43	6927865	{}	\N	2020-10-12 14:51:32.082281
41783	EWD	2020-09-28	35.01	35.35	35.01	35.29	105451	{}	\N	2020-10-12 14:51:35.199056
41784	EWD	2020-09-29	35.5	35.67	35.36	35.55	592869	{}	\N	2020-10-12 14:51:35.20723
41785	EWD	2020-09-30	35.35	35.64	35.11	35.31	62109	{}	\N	2020-10-12 14:51:35.214572
41786	EWD	2020-10-01	35.7	35.76	35.4	35.73	468095	{}	\N	2020-10-12 14:51:35.220981
41787	EWD	2020-10-02	34.89	35.52	34.89	35.35	170581	{}	\N	2020-10-12 14:51:35.228533
41788	EWD	2020-10-05	35.84	36.18	35.84	36.12	193812	{}	\N	2020-10-12 14:51:35.235508
41789	EWD	2020-10-06	36.17	36.17	35.49	35.54	130991	{}	\N	2020-10-12 14:51:35.241254
41790	EWD	2020-10-07	36	36.15	35.94	36.05	159210	{}	\N	2020-10-12 14:51:35.247159
41791	EWD	2020-10-08	36.15	36.34	36.09	36.31	161178	{}	\N	2020-10-12 14:51:35.253058
41792	EWD	2020-10-09	36.43	36.58	36.375	36.5	157151	{}	\N	2020-10-12 14:51:35.258444
41793	EWD	2020-10-12	36.67	36.72	36.55	36.72	15252	{}	\N	2020-10-12 14:51:35.265111
41794	EWMC	2020-09-28	59.7624	60.0814	59.7624	60.0814	965	{}	\N	2020-10-12 14:51:36.413166
41795	EWMC	2020-09-29	59.4932	59.604	59.4932	59.604	876	{}	\N	2020-10-12 14:51:36.420404
41796	EWMC	2020-09-30	60.14	60.465	59.9692	59.9692	3262	{}	\N	2020-10-12 14:51:36.425164
41797	EWMC	2020-10-01	60.14	60.5969	60.1	60.5969	1356	{}	\N	2020-10-12 14:51:36.429644
41798	EWMC	2020-10-02	61.223	61.5683	61.223	61.5683	491	{}	\N	2020-10-12 14:51:36.433778
41799	EWMC	2020-10-05	62.5103	62.93	62.5103	62.93	1450	{}	\N	2020-10-12 14:51:36.437823
41800	EWMC	2020-10-06	62.43	64.1207	62.43	62.6618	17363	{}	\N	2020-10-12 14:51:36.441888
41801	EWMC	2020-10-07	63.8339	63.8339	63.8339	63.8339	170	{}	\N	2020-10-12 14:51:36.44573
41802	EWMC	2020-10-08	64.585	64.585	64.585	64.585	194	{}	\N	2020-10-12 14:51:36.449687
41803	EWMC	2020-10-09	64.6922	64.6972	64.5952	64.5952	890	{}	\N	2020-10-12 14:51:36.453701
41804	EWT	2020-09-28	44.33	44.37	44.11	44.31	2957648	{}	\N	2020-10-12 14:51:37.783026
41805	EWT	2020-09-29	44.33	44.585	44.23	44.42	4027024	{}	\N	2020-10-12 14:51:37.789443
41806	EWT	2020-09-30	44.75	45.045	44.67	44.9	5425376	{}	\N	2020-10-12 14:51:37.795205
41807	EWT	2020-10-01	45.31	45.61	45.28	45.54	3404573	{}	\N	2020-10-12 14:51:37.799931
41808	EWT	2020-10-02	44.83	45.24	44.795	44.97	2526604	{}	\N	2020-10-12 14:51:37.80436
41809	EWT	2020-10-05	45.05	45.44	45.05	45.44	4278064	{}	\N	2020-10-12 14:51:37.808848
41810	EWT	2020-10-06	45.65	45.7	45.285	45.44	3444583	{}	\N	2020-10-12 14:51:37.813183
41811	EWT	2020-10-07	45.98	46.265	45.98	46.16	4332274	{}	\N	2020-10-12 14:51:37.817426
41812	EWT	2020-10-08	46.5	46.515	46.3	46.39	2996208	{}	\N	2020-10-12 14:51:37.821463
41813	EWT	2020-10-09	46.49	46.815	46.38	46.77	3560779	{}	\N	2020-10-12 14:51:37.825352
41814	EWT	2020-10-12	46.431	46.61	46.431	46.58	2109835	{}	\N	2020-10-12 14:51:37.829598
41815	EXC	2020-09-28	35.82	36.23	35.64	35.74	4414437	{}	\N	2020-10-12 14:51:39.459282
41816	EXC	2020-09-29	35.84	35.94	35.15	35.32	3075868	{}	\N	2020-10-12 14:51:39.469946
41817	EXC	2020-09-30	35.57	36.11	35.51	35.75	4967806	{}	\N	2020-10-12 14:51:39.475005
41818	EXC	2020-10-01	35.8693	36.15	35.54	35.825	3756553	{}	\N	2020-10-12 14:51:39.480236
41819	EXC	2020-10-02	35.5517	36.195	35.22	35.94	4747594	{}	\N	2020-10-12 14:51:39.484716
41820	EXC	2020-10-05	36.05	36.89	35.73	36.79	4886276	{}	\N	2020-10-12 14:51:39.489166
41821	EXC	2020-10-06	36.7	37.83	36.69	37.17	5189955	{}	\N	2020-10-12 14:51:39.493837
41822	EXC	2020-10-07	37.38	37.945	37.26	37.85	6035591	{}	\N	2020-10-12 14:51:39.498208
41823	EXC	2020-10-08	37.436	39.36	37.436	39.32	6851865	{}	\N	2020-10-12 14:51:39.502345
41824	EXC	2020-10-09	40.49	40.62	38.92	39.2	5773890	{}	\N	2020-10-12 14:51:39.506088
41825	EXC	2020-10-12	39.21	39.67	39.09	39.65	977524	{}	\N	2020-10-12 14:51:39.510057
41826	EXPD	2020-09-28	91.46	92.19	90.97	91.37	714444	{}	\N	2020-10-12 14:51:40.405881
41827	EXPD	2020-09-29	91.49	92.12	90.9	91.63	659534	{}	\N	2020-10-12 14:51:40.41233
41828	EXPD	2020-09-30	91.76	92.18	89.945	90.52	1092772	{}	\N	2020-10-12 14:51:40.417119
41829	EXPD	2020-10-01	91.12	91.47	88.86	89.3	869245	{}	\N	2020-10-12 14:51:40.421386
41830	EXPD	2020-10-02	88.53	90.1	88.31	89.65	791746	{}	\N	2020-10-12 14:51:40.425304
41831	EXPD	2020-10-05	90.23	91.19	90.23	91.09	805053	{}	\N	2020-10-12 14:51:40.429242
41832	EXPD	2020-10-06	91.3	91.36	89.2	89.31	645772	{}	\N	2020-10-12 14:51:40.43329
41833	EXPD	2020-10-07	90.33	91.42	89.925	90.74	836987	{}	\N	2020-10-12 14:51:40.436957
41834	EXPD	2020-10-08	91.05	91.82	90.36	91.66	933964	{}	\N	2020-10-12 14:51:40.440854
41835	EXPD	2020-10-09	92.07	92.6	91.75	92.46	638430	{}	\N	2020-10-12 14:51:40.444737
41836	EXPD	2020-10-12	92.84	93.12	92.43	92.66	176997	{}	\N	2020-10-12 14:51:40.448596
41837	EYESW	2020-09-28	0.06	0.0914	0.06	0.07	3249	{}	\N	2020-10-12 14:51:42.222901
41838	EYESW	2020-09-29	0.0753	0.085	0.07	0.085	2426	{}	\N	2020-10-12 14:51:42.228347
41839	EYESW	2020-09-30	0.0501	0.06	0.0501	0.06	1740	{}	\N	2020-10-12 14:51:42.233116
41840	EYESW	2020-10-02	0.0794	0.0794	0.0685	0.0685	1895	{}	\N	2020-10-12 14:51:42.237064
41841	EYESW	2020-10-06	0.0685	0.0686	0.0685	0.0685	5188	{}	\N	2020-10-12 14:51:42.240992
41842	EYESW	2020-10-07	0.0685	0.0685	0.065	0.0651	25663	{}	\N	2020-10-12 14:51:42.244939
41843	EYESW	2020-10-08	0.069699	0.069699	0.069699	0.069699	100	{}	\N	2020-10-12 14:51:42.249043
41844	FAAR	2020-09-28	26.435	26.55	26.2113	26.41	9037	{}	\N	2020-10-12 14:51:43.231926
41845	FAAR	2020-09-29	26.5689	26.67	26.1901	26.67	27812	{}	\N	2020-10-12 14:51:43.240492
41846	FAAR	2020-09-30	26.21	26.9399	26.21	26.8064	63136	{}	\N	2020-10-12 14:51:43.246911
41847	FAAR	2020-10-01	26.38	26.945	26.0901	26.945	18261	{}	\N	2020-10-12 14:51:43.253321
41848	FAAR	2020-10-02	26.875	27.71	26.21	27.11	7959	{}	\N	2020-10-12 14:51:43.258767
41849	FAAR	2020-10-05	27.51	27.51	26.11	27.2399	41683	{}	\N	2020-10-12 14:51:43.263507
41850	FAAR	2020-10-06	26.45	27.1099	26.2101	27.1	30074	{}	\N	2020-10-12 14:51:43.268319
41851	FAAR	2020-10-07	26.7	27.17	26.2401	26.4084	34778	{}	\N	2020-10-12 14:51:43.272857
41852	FAAR	2020-10-08	26.59	27.2	26.2401	26.26	19154	{}	\N	2020-10-12 14:51:43.27753
41853	FAAR	2020-10-09	26.3301	27.08	26.3301	26.6205	24963	{}	\N	2020-10-12 14:51:43.282373
41854	FAAR	2020-10-12	26.6783	26.83	26.64	26.7082	1755	{}	\N	2020-10-12 14:51:43.287056
41855	FAUS	2020-09-28	26.88	26.92	26.88	26.92	347	{}	\N	2020-10-12 14:51:46.503732
41856	FAUS	2020-09-29	26.7077	26.7077	26.7077	26.7077	0	{}	\N	2020-10-12 14:51:46.511296
41857	FAUS	2020-09-30	26.66	26.7125	26.66	26.7125	560	{}	\N	2020-10-12 14:51:46.518368
41858	FAUS	2020-10-01	26.9719	26.9719	26.9719	26.9719	58	{}	\N	2020-10-12 14:51:46.523927
41859	FAUS	2020-10-02	26.93	26.9852	26.91	26.9852	301	{}	\N	2020-10-12 14:51:46.530714
41860	FAUS	2020-10-05	27.549	27.55	27.549	27.5498	1052	{}	\N	2020-10-12 14:51:46.536506
41861	FAUS	2020-10-06	27.57	27.57	27.2623	27.2623	1543	{}	\N	2020-10-12 14:51:46.54212
41862	FAUS	2020-10-07	27.72	27.72	27.64	27.64	307	{}	\N	2020-10-12 14:51:46.548747
41863	FAUS	2020-10-09	28.5	28.62	28.5	28.62	352	{}	\N	2020-10-12 14:51:46.554986
41864	FBHS	2020-09-28	84.58	86.6	84.22	85.82	910624	{}	\N	2020-10-12 14:51:48.618387
41865	FBHS	2020-09-29	86.17	86.57	84.92	85.25	1350112	{}	\N	2020-10-12 14:51:48.625249
41866	FBHS	2020-09-30	85.52	87.38	85.52	86.52	3060324	{}	\N	2020-10-12 14:51:48.62974
41867	FBHS	2020-10-01	87.42	88.21	86.13	87.67	2277803	{}	\N	2020-10-12 14:51:48.634452
41868	FBHS	2020-10-02	86.09	88.07	85.2667	87.52	1585222	{}	\N	2020-10-12 14:51:48.638585
41869	FBHS	2020-10-05	88.31	88.882	88.07	88.81	2368670	{}	\N	2020-10-12 14:51:48.643273
41870	FBHS	2020-10-06	89	89.56	87.07	87.18	1597690	{}	\N	2020-10-12 14:51:48.647751
41871	FBHS	2020-10-07	88.43	88.6	86.8	87.6	2089834	{}	\N	2020-10-12 14:51:48.652085
41872	FBHS	2020-10-08	87.95	89.605	87.95	89.33	1414855	{}	\N	2020-10-12 14:51:48.656303
41873	FBHS	2020-10-09	90	90.28	88.52	88.69	1123869	{}	\N	2020-10-12 14:51:48.660576
41874	FBHS	2020-10-12	89.51	89.51	88.56	89	309847	{}	\N	2020-10-12 14:51:48.665049
41875	EWO	2020-09-28	14.61	14.67	14.58	14.6435	10657	{}	\N	2020-10-12 14:51:50.221078
41876	EWO	2020-09-29	14.552	14.595	14.51	14.595	2117	{}	\N	2020-10-12 14:51:50.227316
41877	EWO	2020-09-30	14.61	14.75	14.5693	14.63	20716	{}	\N	2020-10-12 14:51:50.232216
41878	EWO	2020-10-01	14.62	14.64	14.52	14.62	28262	{}	\N	2020-10-12 14:51:50.236747
41879	EWO	2020-10-02	14.5195	14.69	14.5195	14.63	58613	{}	\N	2020-10-12 14:51:50.241294
41880	EWO	2020-10-05	14.9	15.03	14.9	15.03	18358	{}	\N	2020-10-12 14:51:50.245976
41881	EWO	2020-10-06	15.34	15.4	15.08	15.08	12995	{}	\N	2020-10-12 14:51:50.250107
41882	EWO	2020-10-07	15.32	15.33	15.202	15.28	10238	{}	\N	2020-10-12 14:51:50.254112
41883	EWO	2020-10-08	15.4	15.56	15.4	15.55	13936	{}	\N	2020-10-12 14:51:50.258363
41884	EWO	2020-10-09	15.64	15.64	15.49	15.53	333449	{}	\N	2020-10-12 14:51:50.262613
41885	EWO	2020-10-12	15.605	15.605	15.605	15.605	1222	{}	\N	2020-10-12 14:51:50.266662
41886	EWZS	2020-09-28	13.2629	13.2629	12.6	12.62	25356	{}	\N	2020-10-12 14:51:52.608
41887	EWZS	2020-09-29	12.5	12.75	12.44	12.44	47233	{}	\N	2020-10-12 14:51:52.613878
41888	EWZS	2020-09-30	12.585	12.7575	12.55	12.67	13753	{}	\N	2020-10-12 14:51:52.618648
41889	EWZS	2020-10-01	12.65	12.88	12.65	12.88	6760	{}	\N	2020-10-12 14:51:52.623103
41890	EWZS	2020-10-02	12.8	12.9579	12.61	12.61	54516	{}	\N	2020-10-12 14:51:52.627352
41891	EWZS	2020-10-05	12.78	13.1	12.72	13.09	107088	{}	\N	2020-10-12 14:51:52.631209
41892	EWZS	2020-10-06	13.2662	13.38	12.9001	12.95	87871	{}	\N	2020-10-12 14:51:52.636013
41893	EWZS	2020-10-07	13.1029	13.1029	12.86	12.91	35091	{}	\N	2020-10-12 14:51:52.640983
41894	EWZS	2020-10-08	12.925	13.1803	12.91	13.15	14209	{}	\N	2020-10-12 14:51:52.645205
41895	EWZS	2020-10-09	13.385	13.5	13.32	13.36	57784	{}	\N	2020-10-12 14:51:52.649257
41896	EWZS	2020-10-12	13.595	13.6	13.48	13.5384	3542	{}	\N	2020-10-12 14:51:52.653197
41897	EXK	2020-09-28	3.5	3.54	3.36	3.46	2437949	{}	\N	2020-10-12 14:51:55.422818
41898	EXK	2020-09-29	3.52	3.63	3.5	3.59	6591982	{}	\N	2020-10-12 14:51:55.430205
41899	EXK	2020-09-30	3.57	3.59	3.48	3.51	3479376	{}	\N	2020-10-12 14:51:55.434814
41900	EXK	2020-10-01	3.51	3.575	3.4	3.56	5276397	{}	\N	2020-10-12 14:51:55.439599
41901	EXK	2020-10-02	3.44	3.52	3.38	3.39	5515243	{}	\N	2020-10-12 14:51:55.443849
41902	EXK	2020-10-05	3.41	3.53	3.39	3.47	3323116	{}	\N	2020-10-12 14:51:55.447918
41903	EXK	2020-10-06	3.5	3.535	3.29	3.29	4344044	{}	\N	2020-10-12 14:51:55.452005
41904	EXK	2020-10-07	3.34	3.4	3.31	3.35	3163114	{}	\N	2020-10-12 14:51:55.456451
41905	EXK	2020-10-08	3.4	3.44	3.34	3.42	2941083	{}	\N	2020-10-12 14:51:55.460473
41906	EXK	2020-10-09	3.49	3.63	3.45	3.62	8075843	{}	\N	2020-10-12 14:51:55.464495
41907	EXK	2020-10-12	3.65	3.65	3.565	3.59	2584249	{}	\N	2020-10-12 14:51:55.468685
41908	EXPO	2020-09-28	72.935	73.5	72.305	73.08	224082	{}	\N	2020-10-12 14:51:57.01269
41909	EXPO	2020-09-29	73.05	73.8	72.58	72.8	354711	{}	\N	2020-10-12 14:51:57.020633
41910	EXPO	2020-09-30	73.38	73.45	71.78	72.07	192797	{}	\N	2020-10-12 14:51:57.025867
41911	EXPO	2020-10-01	72.87	72.87	71.64	71.97	292558	{}	\N	2020-10-12 14:51:57.030092
41912	EXPO	2020-10-02	70.96	72.07	70.72	70.74	197485	{}	\N	2020-10-12 14:51:57.034811
41913	EXPO	2020-10-05	71.22	72.11	70.42	70.9	185910	{}	\N	2020-10-12 14:51:57.038954
41914	EXPO	2020-10-06	71.545	72.9	70.75	71.87	280028	{}	\N	2020-10-12 14:51:57.042485
41915	EXPO	2020-10-07	72.25	72.94	71.68	72.7	264208	{}	\N	2020-10-12 14:51:57.04616
41916	EXPO	2020-10-08	73.43	73.83	72.2	73.17	131799	{}	\N	2020-10-12 14:51:57.050835
41917	EXPO	2020-10-09	73.62	74.833	72.97	73.99	162390	{}	\N	2020-10-12 14:51:57.054916
41918	EXPO	2020-10-12	74.29	75.18	74.29	75.13	18310	{}	\N	2020-10-12 14:51:57.058759
41919	EYES	2020-09-28	0.86	0.87	0.832	0.87	133437	{}	\N	2020-10-12 14:51:58.144027
41920	EYES	2020-09-29	0.88	0.898	0.8701	0.8884	192937	{}	\N	2020-10-12 14:51:58.15089
41921	EYES	2020-09-30	0.89	0.9197	0.8501	0.8661	309135	{}	\N	2020-10-12 14:51:58.156463
41922	EYES	2020-10-01	0.871084	0.873	0.8595	0.8663	69892	{}	\N	2020-10-12 14:51:58.16119
41923	EYES	2020-10-02	0.85	0.92	0.85	0.8937	224691	{}	\N	2020-10-12 14:51:58.166261
41924	EYES	2020-10-05	0.9	0.9196	0.87	0.9082	237115	{}	\N	2020-10-12 14:51:58.170866
41925	EYES	2020-10-06	0.89	0.9399	0.8842	0.8891	286703	{}	\N	2020-10-12 14:51:58.174904
41926	EYES	2020-10-07	0.92	0.9326	0.880201	0.9001	179104	{}	\N	2020-10-12 14:51:58.178967
41927	EYES	2020-10-08	0.91	0.93	0.905	0.9249	171914	{}	\N	2020-10-12 14:51:58.182831
41928	EYES	2020-10-09	0.93	0.94	0.9001	0.901	187537	{}	\N	2020-10-12 14:51:58.186842
41929	EYES	2020-10-12	0.9055	0.9229	0.9	0.9027	46247	{}	\N	2020-10-12 14:51:58.190776
41930	EZU	2020-09-28	38.005	38.119	37.93	38.11	3442988	{}	\N	2020-10-12 14:51:59.652068
41931	EZU	2020-09-29	38.16	38.32	37.97	38.17	2457423	{}	\N	2020-10-12 14:51:59.658272
41932	EZU	2020-09-30	38.08	38.34	37.8	37.89	6411514	{}	\N	2020-10-12 14:51:59.663644
41933	EZU	2020-10-01	38.23	38.27	37.98	38.23	5561438	{}	\N	2020-10-12 14:51:59.667414
41934	EZU	2020-10-02	37.7	38.24	37.67	38.15	4484968	{}	\N	2020-10-12 14:51:59.672782
41935	EZU	2020-10-05	38.53	38.92	38.53	38.91	2720320	{}	\N	2020-10-12 14:51:59.677476
41936	EZU	2020-10-06	39.06	39.1	38.4	38.48	3682727	{}	\N	2020-10-12 14:51:59.681694
41937	EZU	2020-10-07	38.77	38.949	38.69	38.86	2073383	{}	\N	2020-10-12 14:51:59.685662
41938	EZU	2020-10-08	39.06	39.24	39.04	39.23	2455984	{}	\N	2020-10-12 14:51:59.689951
41939	EZU	2020-10-09	39.49	39.61	39.3811	39.56	2919970	{}	\N	2020-10-12 14:51:59.695406
41940	EZU	2020-10-12	39.67	39.74	39.66	39.73	260553	{}	\N	2020-10-12 14:51:59.700929
41941	FBM	2020-09-28	14.95	15.69	14.95	15.38	143463	{}	\N	2020-10-12 14:52:02.446314
41942	FBM	2020-09-29	15.44	15.64	15.21	15.61	238197	{}	\N	2020-10-12 14:52:02.458317
41943	FBM	2020-09-30	15.65	16.13	15.53	15.72	276901	{}	\N	2020-10-12 14:52:02.46465
41944	FBM	2020-10-01	15.89	16.11	15.55	16.02	181641	{}	\N	2020-10-12 14:52:02.470184
41945	FBM	2020-10-02	15.61	16.23	15.61	16.2	190680	{}	\N	2020-10-12 14:52:02.474873
41946	FBM	2020-10-05	16.32	16.75	16.15	16.72	195671	{}	\N	2020-10-12 14:52:02.479888
41947	FBM	2020-10-06	16.99	17.235	16.56	16.61	263885	{}	\N	2020-10-12 14:52:02.484719
41948	FBM	2020-10-07	16.79	17.05	16.29	16.63	333446	{}	\N	2020-10-12 14:52:02.489449
41949	FBM	2020-10-08	16.82	17.33	16.53	17.15	291174	{}	\N	2020-10-12 14:52:02.494898
41950	FBM	2020-10-09	17.3	17.3	17.01	17.17	240577	{}	\N	2020-10-12 14:52:02.499957
41951	FBM	2020-10-12	17.25	17.49	17.15	17.34	22714	{}	\N	2020-10-12 14:52:02.505597
41952	FCAN	2020-09-29	20.52	20.55	20.52	20.55	490	{}	\N	2020-10-12 14:52:07.048167
41953	FCAN	2020-09-30	20.6631	20.73	20.5564	20.5564	481	{}	\N	2020-10-12 14:52:07.056085
41954	FCAN	2020-10-08	21.04	21.04	21.04	21.04	336	{}	\N	2020-10-12 14:52:07.061778
41955	FCF	2020-09-28	7.55	7.82	7.54	7.74	646141	{}	\N	2020-10-12 14:52:09.332458
41956	FCF	2020-09-29	7.71	7.77	7.535	7.62	1469306	{}	\N	2020-10-12 14:52:09.337935
41957	FCF	2020-09-30	7.65	7.87	7.57	7.74	1664802	{}	\N	2020-10-12 14:52:09.342141
41958	FCF	2020-10-01	7.67	7.84	7.67	7.82	1330456	{}	\N	2020-10-12 14:52:09.345949
41959	FCF	2020-10-02	7.68	8.09	7.68	8.05	1386794	{}	\N	2020-10-12 14:52:09.349638
41960	FCF	2020-10-05	8.16	8.39	8.16	8.36	1345015	{}	\N	2020-10-12 14:52:09.353518
41961	FCF	2020-10-06	8.52	8.715	8.395	8.41	1506054	{}	\N	2020-10-12 14:52:09.357227
41962	FCF	2020-10-07	8.51	8.695	8.415	8.56	1577668	{}	\N	2020-10-12 14:52:09.360695
41963	FCF	2020-10-08	8.66	8.81	8.57	8.77	1000300	{}	\N	2020-10-12 14:52:09.364191
41964	FCF	2020-10-09	8.83	8.84	8.64	8.64	950213	{}	\N	2020-10-12 14:52:09.367752
41965	FCF	2020-10-12	8.61	8.7527	8.595	8.73	92938	{}	\N	2020-10-12 14:52:09.371568
41966	FLDM	2020-09-28	7.1	7.45	7.01	7.24	1833191	{}	\N	2020-10-12 14:52:10.867257
41967	FLDM	2020-09-29	7.54	7.86	7.3	7.68	3382524	{}	\N	2020-10-12 14:52:10.873465
41968	FLDM	2020-09-30	7.71	7.96	7.32	7.43	3542086	{}	\N	2020-10-12 14:52:10.880096
41969	FLDM	2020-10-01	7.34	8.15	7.34	7.91	3140178	{}	\N	2020-10-12 14:52:10.886238
41970	FLDM	2020-10-02	7.63	8.4901	7.55	8.41	2729525	{}	\N	2020-10-12 14:52:10.897018
41971	FLDM	2020-10-05	8.41	8.82	8.33	8.65	2079422	{}	\N	2020-10-12 14:52:10.911583
41972	FLDM	2020-10-06	8.74	9.01	8.27	8.52	3131983	{}	\N	2020-10-12 14:52:10.919187
41973	FLDM	2020-10-07	8.65	8.87	8.47	8.54	1338371	{}	\N	2020-10-12 14:52:10.92657
41974	FLDM	2020-10-08	9.24	9.24	8.02	8.1	3969674	{}	\N	2020-10-12 14:52:10.934014
41975	FLDM	2020-10-09	8.15	8.27	7.76	7.81	1477280	{}	\N	2020-10-12 14:52:10.940835
41976	FLDM	2020-10-12	7.9	7.99	7.61	7.75	507174	{}	\N	2020-10-12 14:52:10.948042
41977	EWY	2020-09-28	64.24	64.42	63.94	64.35	3923002	{}	\N	2020-10-12 14:52:12.797064
41978	EWY	2020-09-29	64.39	64.75	64.25	64.53	2205692	{}	\N	2020-10-12 14:52:12.806196
41979	EWY	2020-09-30	64.92	65.61	64.87	65.34	5212351	{}	\N	2020-10-12 14:52:12.811832
41980	EWY	2020-10-01	65.83	66.04	65.54	66.01	3049491	{}	\N	2020-10-12 14:52:12.816783
41981	EWY	2020-10-02	64.675	65.67	64.6	65.19	2377619	{}	\N	2020-10-12 14:52:12.8211
41982	EWY	2020-10-05	66.07	66.67	66.06	66.63	2706000	{}	\N	2020-10-12 14:52:12.826138
41983	EWY	2020-10-06	66.23	66.565	65.7	65.8	3425421	{}	\N	2020-10-12 14:52:12.832273
41984	EWY	2020-10-07	66.97	67.49	66.95	67.37	3119089	{}	\N	2020-10-12 14:52:12.836759
41985	EWY	2020-10-08	67.5	67.82	67.3	67.71	3887409	{}	\N	2020-10-12 14:52:12.840966
41986	EWY	2020-10-09	68.32	68.49	68.01	68.34	2616374	{}	\N	2020-10-12 14:52:12.845346
41987	EWY	2020-10-12	67.88	68.01	67.72	67.97	1527607	{}	\N	2020-10-12 14:52:12.849853
41988	EXFO	2020-09-28	3.3	3.3	3.21	3.2399	13851	{}	\N	2020-10-12 14:52:14.18962
41989	EXFO	2020-09-29	3.21	3.26	3.21	3.24	21131	{}	\N	2020-10-12 14:52:14.196213
41990	EXFO	2020-09-30	3.235	3.25	3.21	3.25	13634	{}	\N	2020-10-12 14:52:14.201351
41991	EXFO	2020-10-01	3.22	3.23	3.2	3.21	6104	{}	\N	2020-10-12 14:52:14.206052
41992	EXFO	2020-10-02	3.11	3.2398	3.11	3.2398	11065	{}	\N	2020-10-12 14:52:14.210752
41993	EXFO	2020-10-05	3.25	3.2999	3.09	3.14	54155	{}	\N	2020-10-12 14:52:14.215209
41994	EXFO	2020-10-06	3.2	3.25	3.14	3.2	121665	{}	\N	2020-10-12 14:52:14.219934
41995	EXFO	2020-10-07	3.3	3.3	3.11	3.15	130335	{}	\N	2020-10-12 14:52:14.224457
41996	EXFO	2020-10-08	3	3.08	2.81	2.97	100025	{}	\N	2020-10-12 14:52:14.229298
41997	EXFO	2020-10-09	2.95	3.07	2.8456	3.07	138919	{}	\N	2020-10-12 14:52:14.23429
41998	EXFO	2020-10-12	3.05	3.05	2.9878	2.988	8100	{}	\N	2020-10-12 14:52:14.238959
41999	EZM	2020-09-28	33.9	34.37	33.9	34.27	30096	{}	\N	2020-10-12 14:52:17.500486
42000	EZM	2020-09-29	34.13	34.18	33.69	33.9	50642	{}	\N	2020-10-12 14:52:17.507245
42001	EZM	2020-09-30	34.25	34.51	33.913	34.19	81518	{}	\N	2020-10-12 14:52:17.513008
42002	EZM	2020-10-01	34.26	34.58	34.26	34.56	39576	{}	\N	2020-10-12 14:52:17.519592
42003	EZM	2020-10-02	34.3099	35.2	34.3099	35.04	45480	{}	\N	2020-10-12 14:52:17.527739
42004	EZM	2020-10-05	35.44	35.92	35.44	35.89	132127	{}	\N	2020-10-12 14:52:17.532137
42005	EZM	2020-10-06	36.04	36.6492	35.64	35.65	55413	{}	\N	2020-10-12 14:52:17.538019
42006	EZM	2020-10-07	36.11	36.5419	36.11	36.43	36241	{}	\N	2020-10-12 14:52:17.543093
42007	EZM	2020-10-08	36.63	36.92	36.5	36.9	33994	{}	\N	2020-10-12 14:52:17.547749
42008	EZM	2020-10-09	37.04	37.285	36.775	36.89	29716	{}	\N	2020-10-12 14:52:17.552535
42009	EZM	2020-10-12	37.0499	37.135	37.0499	37.135	1837	{}	\N	2020-10-12 14:52:17.557429
42010	FANG	2020-09-28	30.79	32.025	30.66	31.49	2550437	{}	\N	2020-10-12 14:52:19.957663
42011	FANG	2020-09-29	31.07	31.31	30.26	31.2	1939274	{}	\N	2020-10-12 14:52:19.964712
42012	FANG	2020-09-30	31.32	31.77	29.99	30.12	2713108	{}	\N	2020-10-12 14:52:19.970054
42013	FANG	2020-10-01	29.99	30	28.59	29.09	4201120	{}	\N	2020-10-12 14:52:19.974866
42014	FANG	2020-10-02	28.17	30.97	28.05	30.47	3639723	{}	\N	2020-10-12 14:52:19.979751
42015	FANG	2020-10-05	30.99	31.28	30.225	30.87	2350375	{}	\N	2020-10-12 14:52:19.983945
42016	FANG	2020-10-06	31.63	32.43	29.04	29.21	3424977	{}	\N	2020-10-12 14:52:19.98827
42017	FANG	2020-10-07	29.65	29.8695	28.9202	29.47	3209582	{}	\N	2020-10-12 14:52:19.993184
42018	FANG	2020-10-08	29.85	31.15	29.5	31.12	2519951	{}	\N	2020-10-12 14:52:19.997316
42019	FANG	2020-10-09	31.5	31.86	30.42	30.69	2741109	{}	\N	2020-10-12 14:52:20.001477
42020	FANG	2020-10-12	30.34	30.743	29.57	30.6551	448909	{}	\N	2020-10-12 14:52:20.005862
42021	FAZ	2020-09-28	15.65	15.68	14.79	15.11	3788118	{}	\N	2020-10-12 14:52:23.01018
42022	FAZ	2020-09-29	15.22	15.925	15.2	15.67	3646607	{}	\N	2020-10-12 14:52:23.01893
42023	FAZ	2020-09-30	15.56	15.57	14.73	15.13	4417225	{}	\N	2020-10-12 14:52:23.026592
42024	FAZ	2020-10-01	14.93	15.35	14.6	14.99	3440292	{}	\N	2020-10-12 14:52:23.032743
42025	FAZ	2020-10-02	15.87	15.87	14.385	14.62	4471886	{}	\N	2020-10-12 14:52:23.037782
42026	FAZ	2020-10-05	14.22	14.3045	13.765	13.9	3533235	{}	\N	2020-10-12 14:52:23.042604
42027	FAZ	2020-10-06	13.65	14.385	13.245	14.28	5264020	{}	\N	2020-10-12 14:52:23.047511
42028	FAZ	2020-10-07	13.7	13.9	13.37	13.57	3219447	{}	\N	2020-10-12 14:52:23.051884
42029	FAZ	2020-10-08	13.37	13.41	12.92	12.99	4567086	{}	\N	2020-10-12 14:52:23.055836
42030	FAZ	2020-10-09	12.81	13.2201	12.75	12.99	2634133	{}	\N	2020-10-12 14:52:23.059817
42031	FAZ	2020-10-12	12.88	12.96	12.64	12.645	873953	{}	\N	2020-10-12 14:52:23.064007
42032	FCN	2020-09-28	108.01	109.74	108.01	108.92	300778	{}	\N	2020-10-12 14:52:24.96537
42033	FCN	2020-09-29	108.7	108.92	106.68	107	695264	{}	\N	2020-10-12 14:52:24.972083
42034	FCN	2020-09-30	107.29	107.59	105.34	105.97	766723	{}	\N	2020-10-12 14:52:24.977117
42035	FCN	2020-10-01	105.86	107.06	105.635	106.85	828370	{}	\N	2020-10-12 14:52:24.982385
42036	FCN	2020-10-02	106.39	108.81	106.1582	108	658513	{}	\N	2020-10-12 14:52:24.988054
42037	FCN	2020-10-05	108.68	110.53	108.37	110.53	787899	{}	\N	2020-10-12 14:52:24.992789
42038	FCN	2020-10-06	110.34	112.43	109.48	111.26	626493	{}	\N	2020-10-12 14:52:24.998725
42039	FCN	2020-10-07	111.47	112.29	110.835	111.76	453159	{}	\N	2020-10-12 14:52:25.004245
42040	FCN	2020-10-08	112.54	112.97	111.54	111.75	625733	{}	\N	2020-10-12 14:52:25.009471
42041	FCN	2020-10-09	112.11	113.145	112.11	112.17	505124	{}	\N	2020-10-12 14:52:25.01448
42042	FCN	2020-10-12	112.6	113.9	111.82	113.64	48511	{}	\N	2020-10-12 14:52:25.020537
42043	FDL	2020-09-28	25.86	26.185	25.86	26.04	379342	{}	\N	2020-10-12 14:52:28.164514
42044	FDL	2020-09-29	26.01	26.0499	25.7	25.82	556016	{}	\N	2020-10-12 14:52:28.170706
42045	FDL	2020-09-30	25.93	26.26	25.885	26.04	352061	{}	\N	2020-10-12 14:52:28.175393
42046	FDL	2020-10-01	26.15	26.21	25.9167	26.07	204325	{}	\N	2020-10-12 14:52:28.179949
42047	FDL	2020-10-02	25.74	26.3686	25.71	26.23	262056	{}	\N	2020-10-12 14:52:28.18465
42048	FDL	2020-10-05	26.35	26.71	26.35	26.65	197506	{}	\N	2020-10-12 14:52:28.189346
42049	FDL	2020-10-06	26.74	27.09	26.4901	26.55	219804	{}	\N	2020-10-12 14:52:28.193969
42050	FDL	2020-10-07	26.77	27.035	26.77	26.96	279143	{}	\N	2020-10-12 14:52:28.198483
42051	FDL	2020-10-08	27.06	27.39	27.06	27.33	232915	{}	\N	2020-10-12 14:52:28.202734
42052	FDL	2020-10-09	27.43	27.48	27.21	27.24	227478	{}	\N	2020-10-12 14:52:28.207731
42053	FDL	2020-10-12	27.2301	27.3652	27.2	27.35	50081	{}	\N	2020-10-12 14:52:28.212151
42054	FDVV	2020-09-28	27.72	27.94	27.7	27.81	219830	{}	\N	2020-10-12 14:52:29.635131
42055	FDVV	2020-09-29	27.81	27.81	27.4243	27.54	77004	{}	\N	2020-10-12 14:52:29.641153
42056	FDVV	2020-09-30	27.62	27.9352	27.54	27.71	121694	{}	\N	2020-10-12 14:52:29.646053
42057	FDVV	2020-10-01	27.83	27.9051	27.565	27.75	70108	{}	\N	2020-10-12 14:52:29.650451
42058	FDVV	2020-10-02	27.31	27.87	27.271	27.72	324138	{}	\N	2020-10-12 14:52:29.654635
42059	FDVV	2020-10-05	27.95	28.23	27.95	28.21	117542	{}	\N	2020-10-12 14:52:29.659257
42060	FDVV	2020-10-06	28.31	28.4932	27.8717	27.95	91275	{}	\N	2020-10-12 14:52:29.663396
42061	FDVV	2020-10-07	28.18	28.46	28.18	28.38	103187	{}	\N	2020-10-12 14:52:29.667402
42062	FDVV	2020-10-08	28.5	28.65	28.48	28.65	63179	{}	\N	2020-10-12 14:52:29.671613
42063	FDVV	2020-10-09	28.79	28.8449	28.62	28.72	73291	{}	\N	2020-10-12 14:52:29.67607
42064	FDVV	2020-10-12	28.87	28.959	28.79	28.94	30828	{}	\N	2020-10-12 14:52:29.680191
42065	EXTN	2020-09-28	4.27	4.495	4.22	4.41	217523	{}	\N	2020-10-12 14:52:31.833096
42066	EXTN	2020-09-29	4.4	4.4	4.17	4.26	444441	{}	\N	2020-10-12 14:52:31.839039
42067	EXTN	2020-09-30	4.25	4.33	4.04	4.16	669300	{}	\N	2020-10-12 14:52:31.844015
42068	EXTN	2020-10-01	4.15	4.51	3.99	4.43	837039	{}	\N	2020-10-12 14:52:31.848258
42069	EXTN	2020-10-02	4.29	4.5	4.15	4.48	366514	{}	\N	2020-10-12 14:52:31.852765
42070	EXTN	2020-10-05	4.52	4.62	4.42	4.54	365468	{}	\N	2020-10-12 14:52:31.857291
42071	EXTN	2020-10-06	4.62	4.71	4.41	4.47	223470	{}	\N	2020-10-12 14:52:31.861946
42072	EXTN	2020-10-07	4.48	4.59	4.41	4.54	259730	{}	\N	2020-10-12 14:52:31.866598
42073	EXTN	2020-10-08	4.61	4.79	4.55	4.76	267569	{}	\N	2020-10-12 14:52:31.87144
42074	EXTN	2020-10-09	4.76	5.17	4.65	4.99	473646	{}	\N	2020-10-12 14:52:31.876431
42075	EXTN	2020-10-12	5	5	4.85	4.885	55600	{}	\N	2020-10-12 14:52:31.880987
42076	EZA	2020-09-28	36.49	36.5778	35.87	36.27	237665	{}	\N	2020-10-12 14:52:33.505866
42077	EZA	2020-09-29	36.42	36.99	36.42	36.73	310706	{}	\N	2020-10-12 14:52:33.515115
42078	EZA	2020-09-30	36.91	37.27	36.74	36.93	372689	{}	\N	2020-10-12 14:52:33.522876
42079	EZA	2020-10-01	37.52	37.71	37.11	37.66	288239	{}	\N	2020-10-12 14:52:33.529163
42080	EZA	2020-10-02	37.4	38.08	37.332	37.51	323201	{}	\N	2020-10-12 14:52:33.535074
42081	EZA	2020-10-05	37.61	38.12	37.46	37.71	324331	{}	\N	2020-10-12 14:52:33.539775
42082	EZA	2020-10-06	37.74	37.77	36.88	37.12	742359	{}	\N	2020-10-12 14:52:33.544918
42083	EZA	2020-10-07	37.46	37.72	37.26	37.54	295143	{}	\N	2020-10-12 14:52:33.548993
42084	EZA	2020-10-08	37.45	37.86	37.365	37.73	188078	{}	\N	2020-10-12 14:52:33.552913
42085	EZA	2020-10-09	38.19	38.53	38.07	38.34	183611	{}	\N	2020-10-12 14:52:33.556732
42086	EZA	2020-10-12	38.56	38.63	38.38	38.59	44696	{}	\N	2020-10-12 14:52:33.560569
42087	FAF	2020-09-28	50.94	51.5599	50.65	51.17	567533	{}	\N	2020-10-12 14:52:34.941061
42088	FAF	2020-09-29	51.23	51.31	50.15	50.71	855951	{}	\N	2020-10-12 14:52:34.946151
42089	FAF	2020-09-30	50.73	51.41	50.62	50.91	1043576	{}	\N	2020-10-12 14:52:34.951383
42090	FAF	2020-10-01	51.06	51.477	50.69	51.21	1082634	{}	\N	2020-10-12 14:52:34.956444
42091	FAF	2020-10-02	50.57	52.59	50.3	52.2	676428	{}	\N	2020-10-12 14:52:34.96073
42092	FAF	2020-10-05	52.59	53.64	52.59	53.22	806108	{}	\N	2020-10-12 14:52:34.965553
42093	FAF	2020-10-06	53.63	53.725	52.27	52.41	1055761	{}	\N	2020-10-12 14:52:34.969894
42094	FAF	2020-10-07	52.78	53.41	52.17	52.24	826053	{}	\N	2020-10-12 14:52:34.974352
42095	FAF	2020-10-08	52.56	52.83	51.8	52.27	790342	{}	\N	2020-10-12 14:52:34.9784
42096	FAF	2020-10-09	53.18	53.48	52.51	52.78	972112	{}	\N	2020-10-12 14:52:34.982281
42097	FAF	2020-10-12	52.95	53.58	52.65	53.52	92025	{}	\N	2020-10-12 14:52:34.98627
42098	FAT	2020-09-28	6.1	6.1	5.75	6.02	26306	{}	\N	2020-10-12 14:52:37.081993
42099	FAT	2020-09-29	6.1	6.1	5.56	5.99	18927	{}	\N	2020-10-12 14:52:37.088836
42100	FAT	2020-09-30	5.7074	6.05	5.4	5.64	45997	{}	\N	2020-10-12 14:52:37.094338
42101	FAT	2020-10-01	5.63	5.86	5.5	5.66	10827	{}	\N	2020-10-12 14:52:37.098843
42102	FAT	2020-10-02	5.5561	5.65	5.3	5.52	23628	{}	\N	2020-10-12 14:52:37.10381
42103	FAT	2020-10-05	5.57	5.57	5.33	5.41	8736	{}	\N	2020-10-12 14:52:37.108337
42104	FAT	2020-10-06	5.3927	5.41	4.71	4.9359	47503	{}	\N	2020-10-12 14:52:37.11301
42105	FAT	2020-10-07	5.03	5.223	5	5.12	19903	{}	\N	2020-10-12 14:52:37.118661
42106	FAT	2020-10-08	5.25	5.25	5.09	5.11	7158	{}	\N	2020-10-12 14:52:37.131593
42107	FAT	2020-10-09	5.215	5.215	5.02	5.16	8620	{}	\N	2020-10-12 14:52:37.138974
42108	FAT	2020-10-12	5.9	6.1	5.55	5.565	52352	{}	\N	2020-10-12 14:52:37.146849
42109	FBC	2020-09-28	29.62	30.4	29.46	30.02	312614	{}	\N	2020-10-12 14:52:40.650383
42110	FBC	2020-09-29	30.07	30.07	29.18	29.77	620003	{}	\N	2020-10-12 14:52:40.657495
42111	FBC	2020-09-30	29.84	30.32	29.31	29.63	890960	{}	\N	2020-10-12 14:52:40.662905
42112	FBC	2020-10-01	29.7	30.11	29.36	30.04	892900	{}	\N	2020-10-12 14:52:40.667461
42113	FBC	2020-10-02	29.62	31.27	29.61	31.04	657851	{}	\N	2020-10-12 14:52:40.671559
42114	FBC	2020-10-05	31.41	31.77	30.845	31.48	630729	{}	\N	2020-10-12 14:52:40.675707
42115	FBC	2020-10-06	32.04	32.19	31.14	31.25	1055992	{}	\N	2020-10-12 14:52:40.679499
42116	FBC	2020-10-07	31.63	32.065	31.44	31.48	859838	{}	\N	2020-10-12 14:52:40.683165
42117	FBC	2020-10-08	31.74	32.47	31.54	32.29	858680	{}	\N	2020-10-12 14:52:40.687505
42118	FBC	2020-10-09	32.64	32.65	32.0313	32.5	627066	{}	\N	2020-10-12 14:52:40.691443
42119	FBC	2020-10-12	32.35	32.93	32.22	32.905	71550	{}	\N	2020-10-12 14:52:40.695139
42120	FBP	2020-09-28	5.19	5.34	5.185	5.26	1177666	{}	\N	2020-10-12 14:52:42.391967
42121	FBP	2020-09-29	5.22	5.2562	5.08	5.21	1321739	{}	\N	2020-10-12 14:52:42.402776
42122	FBP	2020-09-30	5.22	5.34	5.17	5.22	1877118	{}	\N	2020-10-12 14:52:42.407607
42123	FBP	2020-10-01	5.2	5.35	5.1601	5.34	2433349	{}	\N	2020-10-12 14:52:42.412328
42124	FBP	2020-10-02	5.24	5.7	5.21	5.65	3149137	{}	\N	2020-10-12 14:52:42.4179
42125	FBP	2020-10-05	5.72	6.1	5.69	6.07	4367245	{}	\N	2020-10-12 14:52:42.422936
42126	FBP	2020-10-06	6.18	6.37	5.99	6	4487330	{}	\N	2020-10-12 14:52:42.427978
42127	FBP	2020-10-07	6.09	6.26	6.05	6.24	3699561	{}	\N	2020-10-12 14:52:42.432348
42128	FBP	2020-10-08	6.3	6.38	6.1918	6.34	2878745	{}	\N	2020-10-12 14:52:42.43679
42129	FBP	2020-10-09	6.4	6.47	6.26	6.33	3538371	{}	\N	2020-10-12 14:52:42.441368
42130	FBP	2020-10-12	6.29	6.35	6.22	6.285	259122	{}	\N	2020-10-12 14:52:42.445899
42131	FCO	2020-09-28	6.21	6.58	6.21	6.58	42556	{}	\N	2020-10-12 14:52:44.217294
42132	FCO	2020-09-29	6.58	6.5963	6.42	6.58	31698	{}	\N	2020-10-12 14:52:44.222966
42133	FCO	2020-09-30	6.73	6.8499	6.62	6.84	70006	{}	\N	2020-10-12 14:52:44.22829
42134	FCO	2020-10-01	6.675	6.77	6.54	6.7	124979	{}	\N	2020-10-12 14:52:44.233688
42135	FCO	2020-10-02	6.47	6.675	6.47	6.61	29361	{}	\N	2020-10-12 14:52:44.24008
42136	FCO	2020-10-05	6.7	6.94	6.6799	6.94	40365	{}	\N	2020-10-12 14:52:44.247562
42137	FCO	2020-10-06	6.94	6.97	6.9	6.93	32002	{}	\N	2020-10-12 14:52:44.256538
42138	FCO	2020-10-07	7.0208	7.0301	6.9307	6.99	24317	{}	\N	2020-10-12 14:52:44.262375
42139	FCO	2020-10-08	7.09	7.1	6.99	7.1	12185	{}	\N	2020-10-12 14:52:44.266563
42140	FCO	2020-10-09	7.1	7.1	7.02	7.09	11429	{}	\N	2020-10-12 14:52:44.2711
42141	FCO	2020-10-12	7.19	7.21	7.1475	7.19	8246	{}	\N	2020-10-12 14:52:44.275101
42142	FDS	2020-09-28	331.5	336.72	331.5	333.29	253927	{}	\N	2020-10-12 14:52:47.619831
42143	FDS	2020-09-29	332.25	337.5	332.25	333.96	432081	{}	\N	2020-10-12 14:52:47.625315
42144	FDS	2020-09-30	333.96	338.7	331.97	334.88	514935	{}	\N	2020-10-12 14:52:47.630003
42145	FDS	2020-10-01	335.18	339.18	326.96	328	668058	{}	\N	2020-10-12 14:52:47.6344
42146	FDS	2020-10-02	323.86	327.97	322.51	323.72	680557	{}	\N	2020-10-12 14:52:47.638476
42147	FDS	2020-10-05	325.93	328.95	320.57	323.15	580620	{}	\N	2020-10-12 14:52:47.642762
42148	FDS	2020-10-06	324.38	327.7234	321.15	321.77	540708	{}	\N	2020-10-12 14:52:47.646946
42149	FDS	2020-10-07	324.18	326.62	323.3	325.38	566923	{}	\N	2020-10-12 14:52:47.651057
42150	FDS	2020-10-08	326.79	330.86	324.72	328.6	483984	{}	\N	2020-10-12 14:52:47.655171
42151	FDS	2020-10-09	330.99	332.35	326.915	327.47	334793	{}	\N	2020-10-12 14:52:47.659054
42152	FDS	2020-10-12	328.38	330.01	326.72	328.2	68603	{}	\N	2020-10-12 14:52:47.663158
42153	FEIM	2020-09-28	10.05	10.55	10.05	10.55	7503	{}	\N	2020-10-12 14:52:51.253507
42154	FEIM	2020-09-29	10.72	10.72	10.05	10.3	21226	{}	\N	2020-10-12 14:52:51.260053
42155	FEIM	2020-09-30	10.34	10.4048	10.1001	10.11	6088	{}	\N	2020-10-12 14:52:51.264358
42156	FEIM	2020-10-01	10.05	10.16	10.05	10.065	6253	{}	\N	2020-10-12 14:52:51.268434
42157	FEIM	2020-10-02	10.1	10.29	9.845	10.13	7859	{}	\N	2020-10-12 14:52:51.272377
42158	FEIM	2020-10-05	10	10.28	9.985	10.1	9280	{}	\N	2020-10-12 14:52:51.276278
42159	FEIM	2020-10-06	9.97	10.25	9.8194	10.17	13289	{}	\N	2020-10-12 14:52:51.280477
42160	FEIM	2020-10-07	10.1	10.29	10.1	10.1	4982	{}	\N	2020-10-12 14:52:51.284278
42161	FEIM	2020-10-08	10.2	10.2	10	10.0001	1877	{}	\N	2020-10-12 14:52:51.287893
42162	FEIM	2020-10-09	10.0944	10.0944	9.93	9.93	377	{}	\N	2020-10-12 14:52:51.292259
42163	FLEU	2020-09-28	110.65	110.65	110.4189	110.4189	700	{}	\N	2020-10-12 14:52:52.616479
42164	FLEU	2020-09-29	110.2355	110.2355	110.2355	110.2355	0	{}	\N	2020-10-12 14:52:52.621922
42165	FLEU	2020-09-30	110.19	110.19	109.27	109.27	1302	{}	\N	2020-10-12 14:52:52.626373
42166	FLEU	2020-10-01	109.1263	109.1263	109.1263	109.1263	2	{}	\N	2020-10-12 14:52:52.63021
42167	FLEU	2020-10-02	108.99	109.5619	108.99	109.5619	210	{}	\N	2020-10-12 14:52:52.634316
42168	FLEU	2020-10-05	111.4217	112.4604	111.4217	112.4604	136	{}	\N	2020-10-12 14:52:52.638435
42169	FLEU	2020-10-06	109.3258	109.3258	109.3258	109.3258	0	{}	\N	2020-10-12 14:52:52.642383
42170	FLEU	2020-10-07	111	111	109.61	109.68	2010	{}	\N	2020-10-12 14:52:52.646574
42171	FLEU	2020-10-08	111.84	113.03	111.84	113.03	1700	{}	\N	2020-10-12 14:52:52.650526
42172	FLEU	2020-10-09	114.5815	114.5815	114.5815	114.5815	0	{}	\N	2020-10-12 14:52:52.654522
42173	FAB	2020-09-28	46.79	47.205	46.79	47.04	3402	{}	\N	2020-10-12 14:52:54.145121
42174	FAB	2020-09-29	46.3	46.579	46.3	46.579	1517	{}	\N	2020-10-12 14:52:54.152534
42175	FAB	2020-09-30	46.89	47.23	46.47	46.47	2379	{}	\N	2020-10-12 14:52:54.15767
42176	FAB	2020-10-01	46.795	46.89	46.795	46.81	1373	{}	\N	2020-10-12 14:52:54.162207
42177	FAB	2020-10-02	46.11	47.8141	46.11	47.8141	971	{}	\N	2020-10-12 14:52:54.166598
42178	FAB	2020-10-05	48.5383	48.8	48.5383	48.74	1106	{}	\N	2020-10-12 14:52:54.171158
42179	FAB	2020-10-06	49.28	49.6443	48.28	48.28	3879	{}	\N	2020-10-12 14:52:54.175556
42180	FAB	2020-10-07	49.46	49.5	49.11	49.5	2446	{}	\N	2020-10-12 14:52:54.179959
42181	FAB	2020-10-08	49.9159	50.2	49.87	50.2	2027	{}	\N	2020-10-12 14:52:54.184353
42182	FAB	2020-10-09	50.1513	50.3	49.96	49.96	2094	{}	\N	2020-10-12 14:52:54.188654
42183	FAB	2020-10-12	50.1324	50.35	50.1324	50.35	767	{}	\N	2020-10-12 14:52:54.193124
42184	FARO	2020-09-28	57.93	59.43	57.93	59.23	66007	{}	\N	2020-10-12 14:52:57.306044
42185	FARO	2020-09-29	58.305	61.21	58.305	60.92	87018	{}	\N	2020-10-12 14:52:57.312656
42186	FARO	2020-09-30	61.01	62.1	60.61	60.98	89073	{}	\N	2020-10-12 14:52:57.318273
42187	FARO	2020-10-01	61.68	62	60.53	62	69470	{}	\N	2020-10-12 14:52:57.32318
42188	FARO	2020-10-02	60.94	62.41	60.94	61.94	42859	{}	\N	2020-10-12 14:52:57.328392
42189	FARO	2020-10-05	62.35	63.21	62.025	62.9	39661	{}	\N	2020-10-12 14:52:57.333263
42190	FARO	2020-10-06	62.89	64.62	62.31	62.56	109811	{}	\N	2020-10-12 14:52:57.337831
42191	FARO	2020-10-07	62.83	63.81	62.52	62.85	71090	{}	\N	2020-10-12 14:52:57.343099
42192	FARO	2020-10-08	63.28	63.51	62.64	62.98	62697	{}	\N	2020-10-12 14:52:57.347785
42193	FARO	2020-10-09	63.35	63.93	63.0501	63.75	58931	{}	\N	2020-10-12 14:52:57.353694
42194	FARO	2020-10-12	63.82	64.9	63.01	64.135	4561	{}	\N	2020-10-12 14:52:57.358061
42195	FBIZ	2020-09-28	14.35	14.775	14.35	14.52	14126	{}	\N	2020-10-12 14:52:59.26437
42196	FBIZ	2020-09-29	14.47	14.638	14.285	14.33	11556	{}	\N	2020-10-12 14:52:59.27117
42197	FBIZ	2020-09-30	14.33	14.33	14.15	14.29	16611	{}	\N	2020-10-12 14:52:59.276121
42198	FBIZ	2020-10-01	14.2	14.35	14.2	14.25	12229	{}	\N	2020-10-12 14:52:59.280634
42199	FBIZ	2020-10-02	14.12	14.75	14.12	14.7	9691	{}	\N	2020-10-12 14:52:59.28487
42200	FBIZ	2020-10-05	15	15.05	14.7729	15	21735	{}	\N	2020-10-12 14:52:59.289243
42201	FBIZ	2020-10-06	14.85	15.29	14.55	15	26617	{}	\N	2020-10-12 14:52:59.29348
42202	FBIZ	2020-10-07	15.32	16.24	15.32	15.94	14513	{}	\N	2020-10-12 14:52:59.297582
42203	FBIZ	2020-10-08	16.11	16.57	16.07	16.33	12321	{}	\N	2020-10-12 14:52:59.301677
42204	FBIZ	2020-10-09	16.3	16.63	16.3	16.37	22868	{}	\N	2020-10-12 14:52:59.305875
42205	FBIZ	2020-10-12	16.4	16.4	16.4	16.4	400	{}	\N	2020-10-12 14:52:59.310587
42206	FBZ	2020-09-28	10.75	10.75	10.46	10.46	940	{}	\N	2020-10-12 14:53:00.254106
42207	FBZ	2020-09-29	10.48	10.48	10.425	10.425	436	{}	\N	2020-10-12 14:53:00.261806
42208	FBZ	2020-09-30	10.415	10.5399	10.415	10.5399	1426	{}	\N	2020-10-12 14:53:00.266853
42209	FBZ	2020-10-01	10.512	10.552	10.42	10.47	1001	{}	\N	2020-10-12 14:53:00.271738
42210	FBZ	2020-10-02	10.6514	10.6514	10.3608	10.3738	6886	{}	\N	2020-10-12 14:53:00.276335
42211	FBZ	2020-10-05	10.6684	10.75	10.6684	10.71	1384	{}	\N	2020-10-12 14:53:00.280646
42212	FBZ	2020-10-06	10.86	11.04	10.56	10.56	2325	{}	\N	2020-10-12 14:53:00.284988
42213	FBZ	2020-10-07	10.59	10.69	10.58	10.69	6654	{}	\N	2020-10-12 14:53:00.289205
42214	FBZ	2020-10-08	10.76	10.835	10.76	10.835	1829	{}	\N	2020-10-12 14:53:00.293845
42215	FBZ	2020-10-09	11.0908	11.0908	10.8801	11.0485	1324	{}	\N	2020-10-12 14:53:00.298909
42216	FBZ	2020-10-12	11.017	11.017	11.017	11.017	483	{}	\N	2020-10-12 14:53:00.303374
42217	FCCO	2020-09-28	13.89	13.96	13.68	13.68	4200	{}	\N	2020-10-12 14:53:03.762427
42218	FCCO	2020-09-29	13.845	13.845	13.32	13.77	2141	{}	\N	2020-10-12 14:53:03.769075
42219	FCCO	2020-09-30	13.72	13.72	13.36	13.63	3465	{}	\N	2020-10-12 14:53:03.774317
42220	FCCO	2020-10-01	13.55	13.955	13.43	13.43	6182	{}	\N	2020-10-12 14:53:03.778892
42221	FCCO	2020-10-02	13.5	13.705	12.95	13.66	5358	{}	\N	2020-10-12 14:53:03.783483
42222	FCCO	2020-10-05	13.79	13.9	13.56	13.65	11373	{}	\N	2020-10-12 14:53:03.788151
42223	FCCO	2020-10-06	13.55	13.9	13.405	13.71	31710	{}	\N	2020-10-12 14:53:03.792945
42224	FCCO	2020-10-07	13.85	13.85	13.66	13.82	11563	{}	\N	2020-10-12 14:53:03.797659
42225	FCCO	2020-10-08	13.87	14.61	13.865	13.99	53376	{}	\N	2020-10-12 14:53:03.802026
42226	FCCO	2020-10-09	14.12	14.2735	13.9	14	19520	{}	\N	2020-10-12 14:53:03.806344
42227	FCOR	2020-09-28	56.04	56.19	55.7008	55.95	51212	{}	\N	2020-10-12 14:53:06.423053
42228	FCOR	2020-09-29	55.89	56.07	55.89	55.96	25052	{}	\N	2020-10-12 14:53:06.429831
42229	FCOR	2020-09-30	56.01	56.03	55.85	55.91	41156	{}	\N	2020-10-12 14:53:06.43454
42230	FCOR	2020-10-01	55.9	56.02	55.865	55.93	50630	{}	\N	2020-10-12 14:53:06.438587
42231	FCOR	2020-10-02	55.89	56	55.89	55.95	24962	{}	\N	2020-10-12 14:53:06.441968
42232	FCOR	2020-10-05	55.95	55.9752	55.78	55.78	45692	{}	\N	2020-10-12 14:53:06.445413
42233	FCOR	2020-10-06	55.75	55.935	55.75	55.83	38437	{}	\N	2020-10-12 14:53:06.448775
42234	FCOR	2020-10-07	55.6	55.9952	55.6	55.9	33379	{}	\N	2020-10-12 14:53:06.452216
42235	FCOR	2020-10-08	55.9	56	55.9	55.94	28850	{}	\N	2020-10-12 14:53:06.455525
42236	FCOR	2020-10-09	55.9	55.992	55.9	55.99	18974	{}	\N	2020-10-12 14:53:06.458906
42237	FCOR	2020-10-12	55.8	56.12	55.8	56.07	6879	{}	\N	2020-10-12 14:53:06.462764
42238	FDBC	2020-09-28	47.75	48.62	47.195	47.81	6088	{}	\N	2020-10-12 14:53:10.685584
42239	FDBC	2020-09-29	52.83	52.83	45.87	48.89	3625	{}	\N	2020-10-12 14:53:10.691119
42240	FDBC	2020-09-30	48.3	49	47.7	48.71	1766	{}	\N	2020-10-12 14:53:10.69556
42241	FDBC	2020-10-01	48.1	49.44	47.129	48.53	5190	{}	\N	2020-10-12 14:53:10.704002
42242	FDBC	2020-10-02	46.57	48.11	46.57	46.71	4670	{}	\N	2020-10-12 14:53:10.70827
42243	FDBC	2020-10-05	47.39	47.7075	46.65	46.65	2340	{}	\N	2020-10-12 14:53:10.712361
42244	FDBC	2020-10-06	48.675	48.675	46.28	46.65	1908	{}	\N	2020-10-12 14:53:10.716349
42245	FDBC	2020-10-07	47.5	48.84	46.61	48.84	3746	{}	\N	2020-10-12 14:53:10.720441
42246	FDBC	2020-10-08	47.76	47.81	46.705	47.3	1983	{}	\N	2020-10-12 14:53:10.724525
42247	FDBC	2020-10-09	48.7	48.85	45.505	48	2807	{}	\N	2020-10-12 14:53:10.728726
42248	FDMO	2020-09-28	40	40.01	39.7	39.955	30440	{}	\N	2020-10-12 14:53:12.130918
42249	FDMO	2020-09-29	39.91	40.15	39.91	39.98	13377	{}	\N	2020-10-12 14:53:12.139492
42250	FDMO	2020-09-30	40.07	40.6	40.01	40.35	19301	{}	\N	2020-10-12 14:53:12.147236
42251	FDMO	2020-10-01	40.64	40.8185	40.5	40.76	24533	{}	\N	2020-10-12 14:53:12.15174
42252	FDMO	2020-10-02	40.21	40.61	39.9	40.21	23643	{}	\N	2020-10-12 14:53:12.15622
42253	FDMO	2020-10-05	40.45	41.023	40.45	41.023	25383	{}	\N	2020-10-12 14:53:12.160627
42254	FDMO	2020-10-06	40.99	41.1271	40.4	40.5523	19103	{}	\N	2020-10-12 14:53:12.164961
42255	FDMO	2020-10-07	40.94	41.3225	40.92	41.24	36901	{}	\N	2020-10-12 14:53:12.16927
42256	FDMO	2020-10-08	41.66	41.66	41.33	41.42	12853	{}	\N	2020-10-12 14:53:12.173918
42257	FDMO	2020-10-09	41.69	41.99	41.67	41.94	23920	{}	\N	2020-10-12 14:53:12.178643
42258	FDMO	2020-10-12	42.31	42.465	42.25	42.4	9082	{}	\N	2020-10-12 14:53:12.18295
42259	FDUS	2020-09-28	9.79	10.33	9.79	10.09	292876	{}	\N	2020-10-12 14:53:15.169009
42260	FDUS	2020-09-29	10.08	10.156	9.77	9.85	123668	{}	\N	2020-10-12 14:53:15.175772
42261	FDUS	2020-09-30	9.8601	10.1	9.85	9.85	128161	{}	\N	2020-10-12 14:53:15.18072
42262	FDUS	2020-10-01	9.94	10.1	9.845	10.08	108477	{}	\N	2020-10-12 14:53:15.185949
42263	FDUS	2020-10-02	10.04	10.3998	9.96	10.37	166551	{}	\N	2020-10-12 14:53:15.190427
42264	FDUS	2020-10-05	10.52	10.56	10.35	10.44	221388	{}	\N	2020-10-12 14:53:15.194517
42265	FDUS	2020-10-06	10.5	10.62	10.37	10.4	138075	{}	\N	2020-10-12 14:53:15.198878
42266	FDUS	2020-10-07	10.51	10.6	10.38	10.45	113142	{}	\N	2020-10-12 14:53:15.203615
42267	FDUS	2020-10-08	10.57	10.5999	10.43	10.48	123952	{}	\N	2020-10-12 14:53:15.207987
42268	FDUS	2020-10-09	10.57	10.59	10.49	10.5	100485	{}	\N	2020-10-12 14:53:15.21225
42269	FDUS	2020-10-12	10.51	10.61	10.5	10.56	22736	{}	\N	2020-10-12 14:53:15.216216
42270	FEM	2020-09-28	21.51	21.56	21.35	21.44	61549	{}	\N	2020-10-12 14:53:17.170392
42271	FEM	2020-09-29	21.34	21.4899	21.34	21.44	79331	{}	\N	2020-10-12 14:53:17.175693
42272	FEM	2020-09-30	21.54	21.69	21.505	21.64	95009	{}	\N	2020-10-12 14:53:17.180791
42273	FEM	2020-10-01	21.77	21.87	21.68	21.87	226698	{}	\N	2020-10-12 14:53:17.185484
42274	FEM	2020-10-02	21.51	21.77	21.51	21.58	93076	{}	\N	2020-10-12 14:53:17.190628
42275	FEM	2020-10-05	21.59	21.77	21.59	21.77	63568	{}	\N	2020-10-12 14:53:17.195432
42276	FEM	2020-10-06	22.09	22.1099	21.9	21.96	144396	{}	\N	2020-10-12 14:53:17.199999
42277	FEM	2020-10-07	21.93	22	21.88	21.95	156843	{}	\N	2020-10-12 14:53:17.204299
42278	FEM	2020-10-08	22.0639	22.1854	22.0301	22.18	49901	{}	\N	2020-10-12 14:53:17.208813
42279	FEM	2020-10-09	22.19	22.32	22.15	22.25	63223	{}	\N	2020-10-12 14:53:17.213625
42280	FEM	2020-10-12	22.32	22.34	22.25	22.2864	35094	{}	\N	2020-10-12 14:53:17.218243
42281	FEP	2020-09-28	33.14	33.23	33.0601	33.19	24270	{}	\N	2020-10-12 14:53:18.774131
42282	FEP	2020-09-29	33.28	33.46	33.1349	33.28	28526	{}	\N	2020-10-12 14:53:18.78013
42283	FEP	2020-09-30	33.3021	33.6081	33.19	33.22	80117	{}	\N	2020-10-12 14:53:18.785793
42284	FEP	2020-10-01	33.695	33.85	33.51	33.84	113281	{}	\N	2020-10-12 14:53:18.790524
42285	FEP	2020-10-02	33.34	33.845	33.34	33.73	41915	{}	\N	2020-10-12 14:53:18.795035
42286	FEP	2020-10-05	34.16	34.453	34.16	34.39	20641	{}	\N	2020-10-12 14:53:18.799955
42287	FEP	2020-10-06	34.47	34.4845	33.96	33.99	33957	{}	\N	2020-10-12 14:53:18.804946
42288	FEP	2020-10-07	34.28	34.4308	34.27	34.31	31796	{}	\N	2020-10-12 14:53:18.809361
42289	FEP	2020-10-08	34.6	34.6799	34.549	34.64	24462	{}	\N	2020-10-12 14:53:18.813656
42290	FEP	2020-10-09	34.9	34.9999	34.84	34.95	19849	{}	\N	2020-10-12 14:53:18.818651
42291	FEP	2020-10-12	35.13	35.2168	35.11	35.2168	2708	{}	\N	2020-10-12 14:53:18.823146
42292	EZPW	2020-09-28	4.95	5.2	4.91	5.13	239607	{}	\N	2020-10-12 14:53:21.108325
42293	EZPW	2020-09-29	5.11	5.15	5.03	5.05	207766	{}	\N	2020-10-12 14:53:21.113007
42294	EZPW	2020-09-30	5.07	5.21	5.01	5.03	336955	{}	\N	2020-10-12 14:53:21.117344
42295	EZPW	2020-10-01	5.04	5.17	4.97	5.01	217413	{}	\N	2020-10-12 14:53:21.122556
42296	EZPW	2020-10-02	4.92	5.07	4.89	5	231873	{}	\N	2020-10-12 14:53:21.126738
42297	EZPW	2020-10-05	5.06	5.225	4.99	5.13	312103	{}	\N	2020-10-12 14:53:21.130859
42298	EZPW	2020-10-06	5.17	5.42	5.17	5.31	613470	{}	\N	2020-10-12 14:53:21.135319
42299	EZPW	2020-10-07	5.35	5.51	5.35	5.41	383869	{}	\N	2020-10-12 14:53:21.140558
42300	EZPW	2020-10-08	5.47	5.5	5.24	5.3	283604	{}	\N	2020-10-12 14:53:21.146432
42301	EZPW	2020-10-09	5.32	5.34	5.08	5.08	337039	{}	\N	2020-10-12 14:53:21.150659
42302	EZPW	2020-10-12	5.1	5.21	5.07	5.12	116726	{}	\N	2020-10-12 14:53:21.155807
42303	FAX	2020-09-28	3.87	3.93	3.87	3.93	721273	{}	\N	2020-10-12 14:53:22.796741
42304	FAX	2020-09-29	3.95	3.95	3.9	3.94	702948	{}	\N	2020-10-12 14:53:22.804365
42305	FAX	2020-09-30	3.96	4.01	3.94	3.99	1487991	{}	\N	2020-10-12 14:53:22.809616
42306	FAX	2020-10-01	4.01	4.03	3.99	3.99	800972	{}	\N	2020-10-12 14:53:22.814841
42307	FAX	2020-10-02	3.96	4.02	3.95	3.97	505378	{}	\N	2020-10-12 14:53:22.819416
42308	FAX	2020-10-05	4	4.01	3.985	4	470630	{}	\N	2020-10-12 14:53:22.824486
42309	FAX	2020-10-06	4	4.02	3.99	4	403878	{}	\N	2020-10-12 14:53:22.829777
42310	FAX	2020-10-07	4.02	4.06	3.99	3.99	682154	{}	\N	2020-10-12 14:53:22.834393
42311	FAX	2020-10-08	4.01	4.02	3.97	3.98	657998	{}	\N	2020-10-12 14:53:22.83878
42312	FAX	2020-10-09	4	4	3.94	3.96	843587	{}	\N	2020-10-12 14:53:22.84349
42313	FAX	2020-10-12	3.96	3.97	3.93	3.93	415595	{}	\N	2020-10-12 14:53:22.84777
42314	FBIOP	2020-09-28	18.7079	18.8877	18.56	18.66	9496	{}	\N	2020-10-12 14:53:23.935242
42315	FBIOP	2020-09-29	18.66	18.7	18.4001	18.7	10755	{}	\N	2020-10-12 14:53:23.941807
42316	FBIOP	2020-09-30	18.69	18.9506	18.65	18.85	16848	{}	\N	2020-10-12 14:53:23.946735
42317	FBIOP	2020-10-01	18.73	18.95	18.73	18.95	17705	{}	\N	2020-10-12 14:53:23.950915
42318	FBIOP	2020-10-02	18.95	18.95	18.75	18.89	11182	{}	\N	2020-10-12 14:53:23.95473
42319	FBIOP	2020-10-05	19	19.0539	18.89	18.975	31440	{}	\N	2020-10-12 14:53:23.960354
42320	FBIOP	2020-10-06	18.95	18.99	18.9	18.9	9420	{}	\N	2020-10-12 14:53:23.968426
42321	FBIOP	2020-10-07	19.05	19.15	18.9	18.97	21561	{}	\N	2020-10-12 14:53:23.97692
42322	FBIOP	2020-10-08	19.09	19.15	19	19.14	26838	{}	\N	2020-10-12 14:53:23.986774
42323	FBIOP	2020-10-09	19.14	19.14	18.95	19.14	13299	{}	\N	2020-10-12 14:53:23.996084
42324	FBIOP	2020-10-12	18.835	18.9	18.45	18.6981	49864	{}	\N	2020-10-12 14:53:24.002904
42325	FBSS	2020-09-28	15.0634	15.227	14.9357	15.05	6978	{}	\N	2020-10-12 14:53:25.403705
42326	FBSS	2020-10-01	15.28	15.89	15.28	15.5	15481	{}	\N	2020-10-12 14:53:25.410731
42327	FBSS	2020-10-02	15.09	15.54	15.09	15.53	5155	{}	\N	2020-10-12 14:53:25.41544
42328	FBSS	2020-10-05	15.5	15.5	15.33	15.34	29604	{}	\N	2020-10-12 14:53:25.419568
42329	FBSS	2020-10-06	15.24	15.59	15.24	15.5	20063	{}	\N	2020-10-12 14:53:25.423944
42330	FBSS	2020-10-07	15.21	15.634	15.2	15.5502	5128	{}	\N	2020-10-12 14:53:25.428285
42331	FBSS	2020-10-08	15.5	15.55	15.465	15.5423	32056	{}	\N	2020-10-12 14:53:25.432924
42332	FBSS	2020-10-09	15.6	15.91	15.6	15.91	3018	{}	\N	2020-10-12 14:53:25.436643
42333	FBSS	2020-10-12	15.8272	15.958	15.64	15.81	7713	{}	\N	2020-10-12 14:53:25.440439
42334	FCBC	2020-09-28	17.79	18.41	17.76	18.13	31523	{}	\N	2020-10-12 14:53:27.631766
42335	FCBC	2020-09-29	18.01	18.13	17.71	18.11	21424	{}	\N	2020-10-12 14:53:27.63817
42336	FCBC	2020-09-30	18.06	18.27	17.945	18.05	19997	{}	\N	2020-10-12 14:53:27.642846
42337	FCBC	2020-10-01	18.15	18.25	17.76	18.17	21086	{}	\N	2020-10-12 14:53:27.646744
42338	FCBC	2020-10-02	17.78	18.5	17.78	18.39	21788	{}	\N	2020-10-12 14:53:27.65072
42339	FCBC	2020-10-05	18.41	19.1	18.31	18.92	23381	{}	\N	2020-10-12 14:53:27.654683
42340	FCBC	2020-10-06	19.19	19.425	18.68	18.79	27810	{}	\N	2020-10-12 14:53:27.659139
42341	FCBC	2020-10-07	18.7301	19.44	18.7301	19.28	28770	{}	\N	2020-10-12 14:53:27.663159
42342	FCBC	2020-10-08	18.835	19.91	18.71	19.74	32324	{}	\N	2020-10-12 14:53:27.667197
42343	FCBC	2020-10-09	19.99	19.99	19.26	19.56	19805	{}	\N	2020-10-12 14:53:27.671117
42344	FCNCA	2020-09-28	314.05	322.26	314.05	318.4	33331	{}	\N	2020-10-12 14:53:28.626948
42345	FCNCA	2020-09-29	315.84	316.76	310.27	314.83	26758	{}	\N	2020-10-12 14:53:28.63238
42346	FCNCA	2020-09-30	315	326.98	315	318.78	30564	{}	\N	2020-10-12 14:53:28.637226
42347	FCNCA	2020-10-01	318.78	323.265	317.45	322.62	28667	{}	\N	2020-10-12 14:53:28.641531
42348	FCNCA	2020-10-02	315.75	333.355	315.75	329.7	31969	{}	\N	2020-10-12 14:53:28.646221
42349	FCNCA	2020-10-05	334.57	343.27	333.83	339.71	50270	{}	\N	2020-10-12 14:53:28.650913
42350	FCNCA	2020-10-06	344.09	354.8	336.19	342.34	27666	{}	\N	2020-10-12 14:53:28.655243
42351	FCNCA	2020-10-07	345.12	352.1	341.89	346.59	28737	{}	\N	2020-10-12 14:53:28.659422
42352	FCNCA	2020-10-08	349.7	352.92	346.41	348.75	15539	{}	\N	2020-10-12 14:53:28.663748
42353	FCNCA	2020-10-09	351.79	351.94	345.81	347.6	20066	{}	\N	2020-10-12 14:53:28.668509
42354	FCNCA	2020-10-12	345.8	359.62	345.8	352	2414	{}	\N	2020-10-12 14:53:28.672828
42355	FDIV	2020-09-28	44.33	44.54	44.33	44.539	1111	{}	\N	2020-10-12 14:53:30.001322
42356	FDIV	2020-09-29	44.2687	44.41	44.21	44.41	1288	{}	\N	2020-10-12 14:53:30.010691
42357	FDIV	2020-09-30	44.551	44.6188	44.345	44.35	3813	{}	\N	2020-10-12 14:53:30.017249
42358	FDIV	2020-10-01	44.46	44.525	44.38	44.48	2585	{}	\N	2020-10-12 14:53:30.025842
42359	FDIV	2020-10-02	44.625	44.7934	44.6208	44.7934	2113	{}	\N	2020-10-12 14:53:30.032387
42360	FDIV	2020-10-05	44.91	45.2	44.91	45.2	2488	{}	\N	2020-10-12 14:53:30.03902
42361	FDIV	2020-10-06	45.171	45.4	45.1651	45.1651	3645	{}	\N	2020-10-12 14:53:30.04356
42362	FDIV	2020-10-07	45.3599	45.3599	45.15	45.2855	9622	{}	\N	2020-10-12 14:53:30.048097
42363	FDIV	2020-10-08	45.4831	45.68	45.4831	45.68	4094	{}	\N	2020-10-12 14:53:30.052395
42364	FDIV	2020-10-09	45.26	45.8739	45.26	45.71	6992	{}	\N	2020-10-12 14:53:30.056609
42365	FDIV	2020-10-12	45.82	45.82	45.82	45.82	262	{}	\N	2020-10-12 14:53:30.060777
42366	FEDU	2020-09-28	0.778	0.85	0.6475	0.7082	2773233	{}	\N	2020-10-12 14:53:32.075891
42367	FEDU	2020-09-29	0.68	0.7051	0.652	0.6758	724281	{}	\N	2020-10-12 14:53:32.082163
42368	FEDU	2020-09-30	0.696	0.696	0.6475	0.65	526980	{}	\N	2020-10-12 14:53:32.086242
42369	FEDU	2020-10-01	0.678	0.678	0.6475	0.667	314778	{}	\N	2020-10-12 14:53:32.090578
42370	FEDU	2020-10-02	0.644	0.678	0.6255	0.66	333736	{}	\N	2020-10-12 14:53:32.094884
42371	FEDU	2020-10-05	0.6749	0.79	0.65	0.744	1919252	{}	\N	2020-10-12 14:53:32.099733
42372	FEDU	2020-10-06	0.7	0.767	0.7	0.767	585316	{}	\N	2020-10-12 14:53:32.104074
42373	FEDU	2020-10-07	0.7346	0.7819	0.73	0.7601	409058	{}	\N	2020-10-12 14:53:32.10882
42374	FEDU	2020-10-08	0.7284	0.7866	0.7284	0.75	189642	{}	\N	2020-10-12 14:53:32.112732
42375	FEDU	2020-10-09	0.75	0.778	0.73	0.7601	227554	{}	\N	2020-10-12 14:53:32.117034
42376	FEDU	2020-10-12	0.791	0.8	0.7601	0.791	142172	{}	\N	2020-10-12 14:53:32.123933
42377	FENC	2020-09-28	5.74	5.89	5.57	5.84	128781	{}	\N	2020-10-12 14:53:33.502024
42378	FENC	2020-09-29	5.87	6.027	5.69	5.97	104110	{}	\N	2020-10-12 14:53:33.511431
42379	FENC	2020-09-30	6	6.08	5.79	6.06	184063	{}	\N	2020-10-12 14:53:33.524231
42380	FENC	2020-10-01	6.1	6.3516	6.06	6.25	227662	{}	\N	2020-10-12 14:53:33.53397
42381	FENC	2020-10-02	6.18	6.3278	5.935	5.99	110719	{}	\N	2020-10-12 14:53:33.541838
42382	FENC	2020-10-05	6.08	6.8	6.08	6.72	280952	{}	\N	2020-10-12 14:53:33.547913
42383	FENC	2020-10-06	6.82	7.15	6.71	7.06	272315	{}	\N	2020-10-12 14:53:33.554378
42384	FENC	2020-10-07	7.07	7.56	7.01	7.5	274339	{}	\N	2020-10-12 14:53:33.560369
42385	FENC	2020-10-08	7.59	8.18	7.52	8.09	353771	{}	\N	2020-10-12 14:53:33.568765
42386	FENC	2020-10-09	8.1	8.17	7.55	7.59	164471	{}	\N	2020-10-12 14:53:33.575181
42387	FENC	2020-10-12	7.65	7.7	7.3901	7.6	26427	{}	\N	2020-10-12 14:53:33.581442
42388	FFBC	2020-09-28	11.88	12.23	11.85	12.08	348583	{}	\N	2020-10-12 14:53:34.639774
42389	FFBC	2020-09-29	11.98	12.02	11.56	11.85	250554	{}	\N	2020-10-12 14:53:34.645397
42390	FFBC	2020-09-30	11.96	12.22	11.9	12.005	422199	{}	\N	2020-10-12 14:53:34.650756
42391	FFBC	2020-10-01	11.98	12.18	11.85	12.07	265355	{}	\N	2020-10-12 14:53:34.65516
42392	FFBC	2020-10-02	11.89	12.43	11.89	12.35	239124	{}	\N	2020-10-12 14:53:34.65958
42393	FFBC	2020-10-05	12.57	13.38	12.57	13.11	575450	{}	\N	2020-10-12 14:53:34.664713
42394	FFBC	2020-10-06	13.3	13.9	13.16	13.23	825345	{}	\N	2020-10-12 14:53:34.669303
42395	FFBC	2020-10-07	13.37	13.79	13.345	13.6	555816	{}	\N	2020-10-12 14:53:34.673546
42396	FFBC	2020-10-08	13.79	13.97	13.5599	13.84	253061	{}	\N	2020-10-12 14:53:34.67786
42397	FFBC	2020-10-09	13.93	14	13.64	13.69	325821	{}	\N	2020-10-12 14:53:34.682501
42398	FFBC	2020-10-12	13.69	13.88	13.67	13.81	21952	{}	\N	2020-10-12 14:53:34.686854
42399	FFHL	2020-09-28	3.847	3.87	3.7141	3.85	23919	{}	\N	2020-10-12 14:53:35.573521
42400	FFHL	2020-09-29	3.86	3.878	3.76	3.77	16060	{}	\N	2020-10-12 14:53:35.581433
42401	FFHL	2020-09-30	3.88	3.9	3.74	3.74	9963	{}	\N	2020-10-12 14:53:35.586357
42402	FFHL	2020-10-01	3.77	3.852	3.63	3.72	8594	{}	\N	2020-10-12 14:53:35.591465
42403	FFHL	2020-10-02	3.65	3.79	3.52	3.725	8701	{}	\N	2020-10-12 14:53:35.595705
42404	FFHL	2020-10-05	3.7	4.2	3.7	3.87	44953	{}	\N	2020-10-12 14:53:35.600248
42405	FFHL	2020-10-06	3.98	4.145	3.98	4.14	25043	{}	\N	2020-10-12 14:53:35.604861
42406	FFHL	2020-10-07	4.13	4.2311	4.0601	4.19	31928	{}	\N	2020-10-12 14:53:35.609252
42407	FFHL	2020-10-08	4.19	4.5	4.19	4.31	26184	{}	\N	2020-10-12 14:53:35.613406
42408	FFHL	2020-10-09	4.4	4.48	4.1701	4.19	21704	{}	\N	2020-10-12 14:53:35.617694
42409	FFHL	2020-10-12	4.16	4.28	4.16	4.28	1394	{}	\N	2020-10-12 14:53:35.622034
42410	FFR	2020-09-28	38.68	38.9477	38.675	38.87	4361	{}	\N	2020-10-12 14:53:36.528246
42411	FFR	2020-09-29	38.47	38.72	38.3	38.72	2592	{}	\N	2020-10-12 14:53:36.533529
42412	FFR	2020-09-30	38.88	38.98	38.57	38.73	15942	{}	\N	2020-10-12 14:53:36.538979
42413	FFR	2020-10-01	38.91	39.4128	38.91	39.4128	581	{}	\N	2020-10-12 14:53:36.545534
42414	FFR	2020-10-02	38.98	39.8987	38.98	39.8987	1894	{}	\N	2020-10-12 14:53:36.551812
42415	FFR	2020-10-05	40.1	40.2352	39.88	40.2352	1554	{}	\N	2020-10-12 14:53:36.556483
42416	FFR	2020-10-06	40.16	40.16	40.005	40.005	329	{}	\N	2020-10-12 14:53:36.561589
42417	FFR	2020-10-07	40.05	40.1401	39.99	40.12	2034	{}	\N	2020-10-12 14:53:36.566115
42418	FFR	2020-10-08	40.24	40.4558	40.23	40.4558	3104	{}	\N	2020-10-12 14:53:36.571185
42419	FFR	2020-10-09	39.271	40.53	39.271	40.33	2195	{}	\N	2020-10-12 14:53:36.577472
42420	FFR	2020-10-12	38.89	40.39	38.89	40.39	562	{}	\N	2020-10-12 14:53:36.582785
42421	FARM	2020-09-28	4.45	4.765	4.45	4.68	86440	{}	\N	2020-10-12 14:53:37.504307
42422	FARM	2020-09-29	4.54	4.65	4.475	4.49	139413	{}	\N	2020-10-12 14:53:37.51181
42423	FARM	2020-09-30	4.53	4.6899	4.333	4.42	82460	{}	\N	2020-10-12 14:53:37.516918
42424	FARM	2020-10-01	4.575	4.59	4.38	4.51	81879	{}	\N	2020-10-12 14:53:37.521529
42425	FARM	2020-10-02	4.47	4.68	4.42	4.64	68367	{}	\N	2020-10-12 14:53:37.526485
42426	FARM	2020-10-05	4.68	4.75	4.57	4.64	49217	{}	\N	2020-10-12 14:53:37.530796
42427	FARM	2020-10-06	4.73	4.73	4.46	4.46	71240	{}	\N	2020-10-12 14:53:37.535056
42428	FARM	2020-10-07	4.52	4.7	4.42	4.69	89703	{}	\N	2020-10-12 14:53:37.539275
42429	FARM	2020-10-08	4.72	4.93	4.62	4.86	118782	{}	\N	2020-10-12 14:53:37.544429
42430	FARM	2020-10-09	4.87	4.96	4.72	4.74	76359	{}	\N	2020-10-12 14:53:37.548614
42431	FARM	2020-10-12	4.72	4.85	4.6089	4.78	22507	{}	\N	2020-10-12 14:53:37.55245
42432	FBIO	2020-09-28	4.1	4.1	3.8202	3.94	1259875	{}	\N	2020-10-12 14:53:38.712077
42433	FBIO	2020-09-29	3.95	4	3.82	3.9	704380	{}	\N	2020-10-12 14:53:38.719903
42434	FBIO	2020-09-30	3.95	4.21	3.91	4.03	890435	{}	\N	2020-10-12 14:53:38.725376
42435	FBIO	2020-10-01	4.05	4.08	3.8883	4.03	959145	{}	\N	2020-10-12 14:53:38.730109
42436	FBIO	2020-10-02	3.85	4.13	3.85	3.95	941943	{}	\N	2020-10-12 14:53:38.734509
42437	FBIO	2020-10-05	4.02	4.32	3.9907	4.22	1417721	{}	\N	2020-10-12 14:53:38.738851
42438	FBIO	2020-10-06	4.27	4.34	4.1005	4.12	1039174	{}	\N	2020-10-12 14:53:38.74309
42439	FBIO	2020-10-07	4.19	4.47	4.14	4.31	1121167	{}	\N	2020-10-12 14:53:38.747146
42440	FBIO	2020-10-08	4.46	4.77	4.4101	4.45	2731181	{}	\N	2020-10-12 14:53:38.751399
42441	FBIO	2020-10-09	4.4888	4.58	4.11	4.17	2159261	{}	\N	2020-10-12 14:53:38.755854
42442	FBIO	2020-10-12	3	3.48	2.99	3.22	3171494	{}	\N	2020-10-12 14:53:38.760729
42443	FCAU	2020-09-28	12.39	12.445	12.315	12.37	1943267	{}	\N	2020-10-12 14:53:39.784507
42444	FCAU	2020-09-29	12.25	12.29	12.11	12.16	4153877	{}	\N	2020-10-12 14:53:39.791063
42445	FCAU	2020-09-30	12.17	12.38	12.155	12.22	3263591	{}	\N	2020-10-12 14:53:39.796194
42446	FCAU	2020-10-01	12.23	12.26	12.0716	12.23	4379580	{}	\N	2020-10-12 14:53:39.800863
42447	FCAU	2020-10-02	11.92	12.23	11.91	12.2	4179944	{}	\N	2020-10-12 14:53:39.805995
42448	FCAU	2020-10-05	12.34	12.46	12.295	12.41	2501775	{}	\N	2020-10-12 14:53:39.810677
42449	FCAU	2020-10-06	12.51	12.67	12.315	12.33	6459726	{}	\N	2020-10-12 14:53:39.815078
42450	FCAU	2020-10-07	12.47	12.64	12.46	12.59	2833951	{}	\N	2020-10-12 14:53:39.819424
42451	FCAU	2020-10-08	12.48	12.635	12.48	12.61	3543446	{}	\N	2020-10-12 14:53:39.823813
42452	FCAU	2020-10-09	12.51	12.53	12.37	12.49	2680661	{}	\N	2020-10-12 14:53:39.828119
42453	FCAU	2020-10-12	12.59	12.62	12.54	12.615	1060849	{}	\N	2020-10-12 14:53:39.832328
42454	FCFS	2020-09-28	57.01	57.66	56.9	56.99	203886	{}	\N	2020-10-12 14:53:40.836227
42455	FCFS	2020-09-29	57.21	57.21	55.78	56.2	133932	{}	\N	2020-10-12 14:53:40.844496
42456	FCFS	2020-09-30	56.53	57.34	56.27	57.16	147008	{}	\N	2020-10-12 14:53:40.850733
42457	FCFS	2020-10-01	57.02	59.9186	57.02	59.19	299908	{}	\N	2020-10-12 14:53:40.856142
42458	FCFS	2020-10-02	58.19	60.82	58.1	60.54	185151	{}	\N	2020-10-12 14:53:40.860768
42459	FCFS	2020-10-05	60.92	61.13	59.42	59.69	300861	{}	\N	2020-10-12 14:53:40.865108
42460	FCFS	2020-10-06	60.4	60.65	58.76	59.27	235346	{}	\N	2020-10-12 14:53:40.869635
42461	FCFS	2020-10-07	59.62	60.045	58.7877	59.07	246314	{}	\N	2020-10-12 14:53:40.874372
42462	FCFS	2020-10-08	59.82	60.61	59.18	59.75	152568	{}	\N	2020-10-12 14:53:40.878904
42463	FCFS	2020-10-09	60.15	60.5	57.68	58.61	353690	{}	\N	2020-10-12 14:53:40.883765
42464	FCFS	2020-10-12	58.4	59.34	58.16	59.27	57249	{}	\N	2020-10-12 14:53:40.888253
42465	FCT	2020-09-28	11.05	11.14	11.01	11.07	81086	{}	\N	2020-10-12 14:53:42.241043
42466	FCT	2020-09-29	11.05	11.07	11.0043	11.04	215762	{}	\N	2020-10-12 14:53:42.245785
42467	FCT	2020-09-30	11.07	11.07	10.97	10.97	354134	{}	\N	2020-10-12 14:53:42.250197
42468	FCT	2020-10-01	10.89	10.92	10.8	10.88	172835	{}	\N	2020-10-12 14:53:42.254746
42469	FCT	2020-10-02	10.86	10.95	10.86	10.95	112419	{}	\N	2020-10-12 14:53:42.260592
42470	FCT	2020-10-05	10.92	10.9699	10.8801	10.94	106484	{}	\N	2020-10-12 14:53:42.265332
42471	FCT	2020-10-06	10.92	10.96	10.9	10.92	239839	{}	\N	2020-10-12 14:53:42.270487
42472	FCT	2020-10-07	10.97	10.97	10.9	10.92	209277	{}	\N	2020-10-12 14:53:42.274935
42473	FCT	2020-10-08	10.94	10.97	10.92	10.92	123288	{}	\N	2020-10-12 14:53:42.279673
42474	FCT	2020-10-09	10.95	10.96	10.93	10.96	129452	{}	\N	2020-10-12 14:53:42.284882
42475	FCT	2020-10-12	10.94	10.9678	10.94	10.965	29784	{}	\N	2020-10-12 14:53:42.289193
42476	FDEU	2020-09-28	10.01	10.11	10	10.05	46798	{}	\N	2020-10-12 14:53:43.614335
42477	FDEU	2020-09-29	10.09	10.12	10.03	10.08	98486	{}	\N	2020-10-12 14:53:43.620663
42478	FDEU	2020-09-30	10.02	10.08	10.02	10.05	264872	{}	\N	2020-10-12 14:53:43.62606
42479	FDEU	2020-10-01	10.0221	10.12	9.99	10.07	107407	{}	\N	2020-10-12 14:53:43.630547
42480	FDEU	2020-10-02	9.98	10.12	9.9	10.09	70342	{}	\N	2020-10-12 14:53:43.635084
42481	FDEU	2020-10-05	10.15	10.29	10.15	10.22	109082	{}	\N	2020-10-12 14:53:43.639999
42482	FDEU	2020-10-06	10.285	10.34	10.17	10.2	85176	{}	\N	2020-10-12 14:53:43.645019
42483	FDEU	2020-10-07	10.3	10.37	10.23	10.31	110398	{}	\N	2020-10-12 14:53:43.651294
42484	FDEU	2020-10-08	10.35	10.365	10.32	10.32	39630	{}	\N	2020-10-12 14:53:43.658603
42485	FDEU	2020-10-09	10.38	10.39	10.36	10.38	47572	{}	\N	2020-10-12 14:53:43.663251
42486	FDEU	2020-10-12	10.44	10.44	10.4319	10.4319	4174	{}	\N	2020-10-12 14:53:43.667124
42487	FDP	2020-09-28	22.4	23	22.395	22.56	151815	{}	\N	2020-10-12 14:53:44.760806
42488	FDP	2020-09-29	22.6	22.83	22.46	22.5	229203	{}	\N	2020-10-12 14:53:44.766884
42489	FDP	2020-09-30	22.59	23.11	22.59	22.92	335181	{}	\N	2020-10-12 14:53:44.77321
42490	FDP	2020-10-01	23	23	22.4	22.57	179549	{}	\N	2020-10-12 14:53:44.778998
42491	FDP	2020-10-02	22.24	22.93	22.24	22.87	231077	{}	\N	2020-10-12 14:53:44.784879
42492	FDP	2020-10-05	22.98	23.42	22.89	23.37	255826	{}	\N	2020-10-12 14:53:44.790364
42493	FDP	2020-10-06	23.56	24.15	23.435	23.47	329285	{}	\N	2020-10-12 14:53:44.794985
42494	FDP	2020-10-07	23.61	23.8	23.37	23.4	262244	{}	\N	2020-10-12 14:53:44.799101
42495	FDP	2020-10-08	23.63	23.79	23.3	23.53	181800	{}	\N	2020-10-12 14:53:44.802802
42496	FDP	2020-10-09	23.76	23.9	23.38	23.52	169843	{}	\N	2020-10-12 14:53:44.807683
42497	FDP	2020-10-12	23.36	23.75	23.36	23.75	35856	{}	\N	2020-10-12 14:53:44.815359
42498	FEI	2020-09-28	4.67	4.77	4.66	4.71	247892	{}	\N	2020-10-12 14:53:45.900229
42499	FEI	2020-09-29	4.73	4.74	4.65	4.74	332572	{}	\N	2020-10-12 14:53:45.906691
42500	FEI	2020-09-30	4.73	4.7899	4.61	4.64	857065	{}	\N	2020-10-12 14:53:45.912071
42501	FEI	2020-10-01	4.57	4.63	4.55	4.58	957210	{}	\N	2020-10-12 14:53:45.916591
42502	FEI	2020-10-02	4.5	4.7	4.5	4.69	511528	{}	\N	2020-10-12 14:53:45.920963
42503	FEI	2020-10-05	4.85	5.03	4.76	5	1244453	{}	\N	2020-10-12 14:53:45.925365
42504	FEI	2020-10-06	5.05	5.09	4.96	5.02	875078	{}	\N	2020-10-12 14:53:45.92951
42505	FEI	2020-10-07	5.03	5.1	4.98	5.1	539830	{}	\N	2020-10-12 14:53:45.933601
42506	FEI	2020-10-08	5.11	5.32	5.11	5.3	712334	{}	\N	2020-10-12 14:53:45.937477
42507	FEI	2020-10-09	5.33	5.37	5.28	5.34	742355	{}	\N	2020-10-12 14:53:45.941848
42508	FEI	2020-10-12	5.35	5.35	5.29	5.3	103435	{}	\N	2020-10-12 14:53:45.946072
42509	FFBW	2020-09-28	9.4469	9.5799	9.41	9.5699	4938	{}	\N	2020-10-12 14:53:47.254744
42510	FFBW	2020-09-29	9.5	9.5	9.22	9.22	8671	{}	\N	2020-10-12 14:53:47.262672
42511	FFBW	2020-09-30	9.4533	9.49	9.4533	9.49	1548	{}	\N	2020-10-12 14:53:47.268236
42512	FFBW	2020-10-01	9.396	9.565	9.396	9.52	1405	{}	\N	2020-10-12 14:53:47.273621
42513	FFBW	2020-10-05	9.43	9.55	9.26	9.47	4908	{}	\N	2020-10-12 14:53:47.288209
42514	FFBW	2020-10-06	9.25	9.5	9.25	9.28	5448	{}	\N	2020-10-12 14:53:47.292389
42515	FFBW	2020-10-07	9.44	9.5	9.25	9.5	9370	{}	\N	2020-10-12 14:53:47.29714
42516	FFBW	2020-10-08	9.4185	9.5	9.35	9.5	2622	{}	\N	2020-10-12 14:53:47.301967
42517	FFBW	2020-10-09	9.42	9.49	9.4142	9.49	961	{}	\N	2020-10-12 14:53:47.306268
42518	FFBW	2020-10-12	9.375	9.375	9.375	9.375	234	{}	\N	2020-10-12 14:53:47.310441
42519	FFTG	2020-09-28	27.44	27.47	27.3	27.405	12057	{}	\N	2020-10-12 14:53:48.888038
42520	FFTG	2020-09-29	27.4736	27.51	27.36	27.41	13875	{}	\N	2020-10-12 14:53:48.894336
42521	FFTG	2020-09-30	27.46	27.5214	27.35	27.37	11368	{}	\N	2020-10-12 14:53:48.899273
42522	FFTG	2020-10-01	27.34	27.6599	27.34	27.4878	10174	{}	\N	2020-10-12 14:53:48.903498
42523	FFTG	2020-10-02	27.46	27.4757	27.39	27.4715	8199	{}	\N	2020-10-12 14:53:48.907322
42524	FFTG	2020-10-05	27.7	27.72	27.64	27.673	9872	{}	\N	2020-10-12 14:53:48.911149
42525	FFTG	2020-10-06	27.52	27.7599	27.32	27.43	15841	{}	\N	2020-10-12 14:53:48.914992
42526	FFTG	2020-10-07	27.58	27.5951	27.56	27.56	3811	{}	\N	2020-10-12 14:53:48.918607
42527	FFTG	2020-10-08	27.705	27.7999	27.69	27.76	12598	{}	\N	2020-10-12 14:53:48.92228
42528	FFTG	2020-10-09	28.09	28.1999	28.09	28.1999	14186	{}	\N	2020-10-12 14:53:48.925962
42529	FGBI	2020-09-28	12.2355	12.39	11.93	12.06	6291	{}	\N	2020-10-12 14:53:50.602935
42530	FGBI	2020-09-29	12.16	12.18	12.06	12.18	2158	{}	\N	2020-10-12 14:53:50.608492
42531	FGBI	2020-09-30	12.18	12.45	11.86	12.11	10174	{}	\N	2020-10-12 14:53:50.615156
42532	FGBI	2020-10-01	12.23	12.84	12.23	12.48	4445	{}	\N	2020-10-12 14:53:50.62055
42533	FGBI	2020-10-02	12.54	12.59	12.54	12.59	1992	{}	\N	2020-10-12 14:53:50.624875
42534	FGBI	2020-10-05	12.72	12.98	12.52	12.62	4000	{}	\N	2020-10-12 14:53:50.629612
42535	FGBI	2020-10-06	12.75	13	12.75	12.84	5666	{}	\N	2020-10-12 14:53:50.634648
42536	FGBI	2020-10-07	13.14	13.69	13.14	13.59	6340	{}	\N	2020-10-12 14:53:50.640161
42537	FGBI	2020-10-08	13.73	13.96	13.73	13.8	5282	{}	\N	2020-10-12 14:53:50.644785
42538	FGBI	2020-10-09	13.8	13.8	13.65	13.7	2215	{}	\N	2020-10-12 14:53:50.650033
42539	FGBI	2020-10-12	13.84	14.2	13.84	14.2	363	{}	\N	2020-10-12 14:53:50.654808
42540	FC	2020-09-28	17.5	18.2756	17.47	18.05	135407	{}	\N	2020-10-12 14:53:51.673424
42541	FC	2020-09-29	18.27	18.27	17.9	17.96	110072	{}	\N	2020-10-12 14:53:51.680902
42542	FC	2020-09-30	18.02	18.25	17.62	17.74	148604	{}	\N	2020-10-12 14:53:51.686196
42543	FC	2020-10-01	17.7	18.01	17.4	17.4	174688	{}	\N	2020-10-12 14:53:51.690799
42544	FC	2020-10-02	17.09	17.41	16.64	17.25	199104	{}	\N	2020-10-12 14:53:51.694876
42545	FC	2020-10-05	17.43	17.565	17.21	17.51	119788	{}	\N	2020-10-12 14:53:51.699479
42546	FC	2020-10-06	17.69	18.4	17.55	17.86	107770	{}	\N	2020-10-12 14:53:51.7039
42547	FC	2020-10-07	17.99	18.48	17.87	18.33	92496	{}	\N	2020-10-12 14:53:51.708122
42548	FC	2020-10-08	18.4	18.4	18.11	18.17	110475	{}	\N	2020-10-12 14:53:51.712579
42549	FC	2020-10-09	18.3	18.58	18.03	18.58	58588	{}	\N	2020-10-12 14:53:51.716696
42550	FC	2020-10-12	18.59	18.59	18.59	18.59	918	{}	\N	2020-10-12 14:53:51.720909
42551	FCG	2020-09-28	6.19	6.37	6.14	6.3	478391	{}	\N	2020-10-12 14:53:52.650476
42552	FCG	2020-09-29	6.27	6.27	5.99	6.15	355264	{}	\N	2020-10-12 14:53:52.6575
42553	FCG	2020-09-30	6.15	6.265	6.055	6.08	350886	{}	\N	2020-10-12 14:53:52.663402
42554	FCG	2020-10-01	6.05	6.08	5.89	5.92	379262	{}	\N	2020-10-12 14:53:52.668716
42555	FCG	2020-10-02	5.75	6.11	5.6905	6.05	529279	{}	\N	2020-10-12 14:53:52.673388
42556	FCG	2020-10-05	6.1805	6.37	6.13	6.37	423244	{}	\N	2020-10-12 14:53:52.678838
42557	FCG	2020-10-06	6.45	6.56	6.17	6.2	751489	{}	\N	2020-10-12 14:53:52.683572
42558	FCG	2020-10-07	6.25	6.4	6.23	6.385	680077	{}	\N	2020-10-12 14:53:52.688433
42559	FCG	2020-10-08	6.45	6.7	6.4	6.7	597496	{}	\N	2020-10-12 14:53:52.692975
42560	FCG	2020-10-09	6.75	6.8105	6.54	6.6	543584	{}	\N	2020-10-12 14:53:52.697506
42561	FCG	2020-10-12	6.59	6.59	6.48	6.54	131327	{}	\N	2020-10-12 14:53:52.702068
42562	FCX	2020-09-28	15.64	15.94	15.425	15.9	17466844	{}	\N	2020-10-12 14:53:53.687478
42563	FCX	2020-09-29	15.81	15.92	15.31	15.32	26313879	{}	\N	2020-10-12 14:53:53.695736
42564	FCX	2020-09-30	15.4	15.8078	15.36	15.64	35433034	{}	\N	2020-10-12 14:53:53.700971
42565	FCX	2020-10-01	15.7	15.8	15.3	15.55	59227092	{}	\N	2020-10-12 14:53:53.706781
42566	FCX	2020-10-02	15.26	16.06	15.22	15.94	45548540	{}	\N	2020-10-12 14:53:53.711952
42567	FCX	2020-10-05	16.04	16.54	16.04	16.32	31238224	{}	\N	2020-10-12 14:53:53.717014
42568	FCX	2020-10-06	16.4299	16.58	15.715	15.72	39546964	{}	\N	2020-10-12 14:53:53.721392
42569	FCX	2020-10-07	16.25	17	16.25	16.88	57554584	{}	\N	2020-10-12 14:53:53.725909
42570	FCX	2020-10-08	17.05	17.24	16.76	16.8	33290010	{}	\N	2020-10-12 14:53:53.73021
42571	FCX	2020-10-09	17.2	17.26	16.835	17.23	31159001	{}	\N	2020-10-12 14:53:53.734802
42572	FCX	2020-10-12	17.16	17.26	16.9	17.04	9445772	{}	\N	2020-10-12 14:53:53.739488
42573	FDLO	2020-09-28	39.27	39.536	39.27	39.37	41672	{}	\N	2020-10-12 14:53:55.282201
42574	FDLO	2020-09-29	39.43	39.4744	39.2092	39.27	23379	{}	\N	2020-10-12 14:53:55.288923
42575	FDLO	2020-09-30	39.39	39.835	39.35	39.56	65000	{}	\N	2020-10-12 14:53:55.293657
42576	FDLO	2020-10-01	39.72	39.8848	39.44	39.65	72353	{}	\N	2020-10-12 14:53:55.29799
42577	FDLO	2020-10-02	39.08	39.6	39.08	39.42	86621	{}	\N	2020-10-12 14:53:55.302766
42578	FDLO	2020-10-05	39.66	39.88	39.605	39.87	40854	{}	\N	2020-10-12 14:53:55.307297
42579	FDLO	2020-10-06	39.94	40.19	39.45	39.54	203506	{}	\N	2020-10-12 14:53:55.31199
42580	FDLO	2020-10-07	39.8	40.095	39.8	39.97	65425	{}	\N	2020-10-12 14:53:55.316856
42581	FDLO	2020-10-08	40.22	40.36	40.1423	40.36	36388	{}	\N	2020-10-12 14:53:55.321544
42582	FDLO	2020-10-09	40.5166	40.75	40.4591	40.64	57516	{}	\N	2020-10-12 14:53:55.326993
42583	FDLO	2020-10-12	40.93	40.944	40.8	40.944	28521	{}	\N	2020-10-12 14:53:55.331662
42584	FEO	2020-09-28	12.06	12.06	11.91	11.94	14568	{}	\N	2020-10-12 14:53:57.08983
42585	FEO	2020-09-29	11.91	11.98	11.86	11.96	11178	{}	\N	2020-10-12 14:53:57.099114
42586	FEO	2020-09-30	12.03	12.065	11.93	12.045	38194	{}	\N	2020-10-12 14:53:57.10536
42587	FEO	2020-10-01	12.09	12.6	12.09	12.288	27752	{}	\N	2020-10-12 14:53:57.111288
42588	FEO	2020-10-02	12.28	12.3	12.1601	12.2155	8038	{}	\N	2020-10-12 14:53:57.116672
42589	FEO	2020-10-05	12.19	12.48	12.091	12.25	42414	{}	\N	2020-10-12 14:53:57.12126
42590	FEO	2020-10-06	12.28	12.51	12.215	12.51	11686	{}	\N	2020-10-12 14:53:57.12637
42591	FEO	2020-10-07	12.45	12.485	12.45	12.48	7436	{}	\N	2020-10-12 14:53:57.13135
42592	FEO	2020-10-08	12.53	12.84	12.39	12.68	19854	{}	\N	2020-10-12 14:53:57.136963
42593	FEO	2020-10-09	12.65	12.65	12.41	12.41	8760	{}	\N	2020-10-12 14:53:57.1417
42594	FEO	2020-10-12	12.42	12.42	12.35	12.38	16992	{}	\N	2020-10-12 14:53:57.147587
42595	FEX	2020-09-28	62.74	63.3976	62.74	63.21	52388	{}	\N	2020-10-12 14:53:58.319652
42596	FEX	2020-09-29	63.12	63.335	62.7336	62.9386	18824	{}	\N	2020-10-12 14:53:58.325287
42597	FEX	2020-09-30	63.11	63.84	62.88	63.21	51508	{}	\N	2020-10-12 14:53:58.330714
42598	FEX	2020-10-01	63.8691	63.8691	63.278	63.6	113249	{}	\N	2020-10-12 14:53:58.335258
42599	FEX	2020-10-02	63.1044	63.93	63.1044	63.69	12619	{}	\N	2020-10-12 14:53:58.339852
42600	FEX	2020-10-05	64.14	64.93	64.14	64.87	15253	{}	\N	2020-10-12 14:53:58.346669
42601	FEX	2020-10-06	65.18	65.66	64.16	64.31	30511	{}	\N	2020-10-12 14:53:58.350992
42602	FEX	2020-10-07	64.77	65.5601	64.77	65.44	33761	{}	\N	2020-10-12 14:53:58.35552
42603	FEX	2020-10-08	65.95	66.09	65.7135	66.08	28567	{}	\N	2020-10-12 14:53:58.359891
42604	FEX	2020-10-09	66.48	66.6764	66.32	66.39	16517	{}	\N	2020-10-12 14:53:58.364766
42605	FEX	2020-10-12	66.8	66.815	66.79	66.79	1244	{}	\N	2020-10-12 14:53:58.369212
42606	FFG	2020-09-28	48.3	49.73	48.3	48.37	120747	{}	\N	2020-10-12 14:53:59.336562
42607	FFG	2020-09-29	48.1	48.44	47.57	48.11	192997	{}	\N	2020-10-12 14:53:59.342993
42608	FFG	2020-09-30	48.3	48.83	48	48.2	155424	{}	\N	2020-10-12 14:53:59.349189
42609	FFG	2020-10-01	48.1	49.71	48.1	49.6	204176	{}	\N	2020-10-12 14:53:59.354527
42610	FFG	2020-10-02	49.49	50.23	48.7	49.95	204402	{}	\N	2020-10-12 14:53:59.360108
42611	FFG	2020-10-05	50.03	51	49.8	50.88	346542	{}	\N	2020-10-12 14:53:59.366701
42612	FFG	2020-10-06	50.96	51.18	50.27	50.53	196514	{}	\N	2020-10-12 14:53:59.372291
42613	FFG	2020-10-07	50.86	51.21	49.77	50.41	179777	{}	\N	2020-10-12 14:53:59.377568
42614	FFG	2020-10-08	50.83	51.16	50.07	50.82	120281	{}	\N	2020-10-12 14:53:59.383372
42615	FFG	2020-10-09	51.08	51.08	50.51	50.82	69406	{}	\N	2020-10-12 14:53:59.388642
42616	FFG	2020-10-12	50.68	50.97	50.68	50.97	7338	{}	\N	2020-10-12 14:53:59.394824
42617	FFNW	2020-09-28	9.65	9.65	9.22	9.25	2517	{}	\N	2020-10-12 14:54:00.666039
42618	FFNW	2020-09-29	9.25	9.25	9.11	9.12	3485	{}	\N	2020-10-12 14:54:00.673714
42619	FFNW	2020-09-30	9.05	9.19	9.05	9.12	5519	{}	\N	2020-10-12 14:54:00.678808
42620	FFNW	2020-10-01	9.12	9.21	9.02	9.02	6740	{}	\N	2020-10-12 14:54:00.68402
42621	FFNW	2020-10-02	9.08	9.32	9.03	9.15	6018	{}	\N	2020-10-12 14:54:00.689043
42622	FFNW	2020-10-05	9.14	9.35	9.14	9.18	6232	{}	\N	2020-10-12 14:54:00.693423
42623	FFNW	2020-10-06	9.31	9.35	9.11	9.2	6829	{}	\N	2020-10-12 14:54:00.69806
42624	FFNW	2020-10-07	9.2594	9.2594	9.11	9.12	5087	{}	\N	2020-10-12 14:54:00.702845
42625	FFNW	2020-10-08	9.2	9.34	9.2	9.34	3938	{}	\N	2020-10-12 14:54:00.70741
42626	FFNW	2020-10-09	9.26	9.36	9.26	9.27	1935	{}	\N	2020-10-12 14:54:00.712919
42627	FFNW	2020-10-12	9.255	9.255	9.255	9.255	555	{}	\N	2020-10-12 14:54:00.717799
42628	FGD	2020-09-28	17.79	17.95	17.79	17.9	65308	{}	\N	2020-10-12 14:54:01.97788
42629	FGD	2020-09-29	17.805	17.83	17.635	17.75	205325	{}	\N	2020-10-12 14:54:01.98645
42630	FGD	2020-09-30	17.84	17.94	17.77	17.82	71583	{}	\N	2020-10-12 14:54:01.994511
42631	FGD	2020-10-01	17.85	17.877	17.7549	17.85	75263	{}	\N	2020-10-12 14:54:02.001597
42632	FGD	2020-10-02	17.67	18.0799	17.67	18.01	57783	{}	\N	2020-10-12 14:54:02.007578
42633	FGD	2020-10-05	18.22	18.41	18.22	18.41	243424	{}	\N	2020-10-12 14:54:02.015092
42634	FGD	2020-10-06	18.55	18.6482	18.3	18.32	53912	{}	\N	2020-10-12 14:54:02.022509
42635	FGD	2020-10-07	18.51	18.5974	18.49	18.57	128789	{}	\N	2020-10-12 14:54:02.027665
42636	FGD	2020-10-08	18.7	18.86	18.7	18.85	136857	{}	\N	2020-10-12 14:54:02.032547
42637	FGD	2020-10-09	18.895	18.895	18.757	18.82	72131	{}	\N	2020-10-12 14:54:02.037129
42638	FGD	2020-10-12	18.83	18.85	18.8063	18.841	10291	{}	\N	2020-10-12 14:54:02.041641
42639	FI	2020-09-28	1.69	1.725	1.66	1.68	695126	{}	\N	2020-10-12 14:54:03.551926
42640	FI	2020-09-29	1.7	1.7	1.6001	1.63	1212711	{}	\N	2020-10-12 14:54:03.558511
42641	FI	2020-09-30	1.63	1.65	1.51	1.54	1694920	{}	\N	2020-10-12 14:54:03.563265
42642	FI	2020-10-01	1.56	1.6	1.5	1.57	1911411	{}	\N	2020-10-12 14:54:03.568107
42643	FI	2020-10-02	1.53	1.68	1.51	1.64	1514911	{}	\N	2020-10-12 14:54:03.573248
42644	FI	2020-10-05	1.65	1.735	1.65	1.68	1316316	{}	\N	2020-10-12 14:54:03.578435
42645	FI	2020-10-06	1.71	1.73	1.615	1.65	1608275	{}	\N	2020-10-12 14:54:03.583287
42646	FI	2020-10-07	1.67	1.71	1.65	1.69	1249615	{}	\N	2020-10-12 14:54:03.587608
42647	FI	2020-10-08	1.7	1.85	1.69	1.83	1254621	{}	\N	2020-10-12 14:54:03.592237
42648	FI	2020-10-09	1.88	1.8862	1.69	1.73	1512277	{}	\N	2020-10-12 14:54:03.597031
42649	FI	2020-10-12	1.73	1.74	1.69	1.7078	264460	{}	\N	2020-10-12 14:54:03.60141
42650	FCEF	2020-09-28	19.58	19.58	19.4805	19.5098	5825	{}	\N	2020-10-12 14:54:04.955697
42651	FCEF	2020-09-29	19.44	19.6199	19.43	19.47	4919	{}	\N	2020-10-12 14:54:04.961451
42652	FCEF	2020-09-30	19.42	19.7026	19.42	19.6526	5479	{}	\N	2020-10-12 14:54:04.966437
42653	FCEF	2020-10-01	19.75	19.79	19.745	19.77	3062	{}	\N	2020-10-12 14:54:04.970841
42654	FCEF	2020-10-02	19.83	19.83	19.66	19.695	2050	{}	\N	2020-10-12 14:54:04.978145
42655	FCEF	2020-10-05	19.835	20	19.835	20	5511	{}	\N	2020-10-12 14:54:04.985389
42656	FCEF	2020-10-06	20.005	20.15	19.97	20.0089	45956	{}	\N	2020-10-12 14:54:04.991528
42657	FCEF	2020-10-07	20.14	20.21	20.0701	20.13	8862	{}	\N	2020-10-12 14:54:04.998216
42658	FCEF	2020-10-08	20.255	20.34	20.255	20.2693	13781	{}	\N	2020-10-12 14:54:05.002914
42659	FCEF	2020-10-09	20.38	20.44	20.345	20.345	29274	{}	\N	2020-10-12 14:54:05.008786
42660	FCEF	2020-10-12	20.3952	20.5097	20.31	20.465	2554	{}	\N	2020-10-12 14:54:05.016464
42661	FCOM	2020-09-28	38.58	38.58	38.235	38.49	70479	{}	\N	2020-10-12 14:54:05.976928
42662	FCOM	2020-09-29	38.49	38.64	38.3115	38.55	70279	{}	\N	2020-10-12 14:54:05.983831
42663	FCOM	2020-09-30	38.52	38.9574	38.37	38.58	135419	{}	\N	2020-10-12 14:54:05.989697
42664	FCOM	2020-10-01	38.89	39.2495	38.81	39.18	60613	{}	\N	2020-10-12 14:54:05.995392
42665	FCOM	2020-10-02	38.44	38.97	38.3469	38.5	63546	{}	\N	2020-10-12 14:54:06.000495
42666	FCOM	2020-10-05	38.74	39.15	38.74	39.15	75312	{}	\N	2020-10-12 14:54:06.004422
42667	FCOM	2020-10-06	38.96	39.251	38.38	38.47	66869	{}	\N	2020-10-12 14:54:06.009694
42668	FCOM	2020-10-07	38.79	38.85	38.47	38.79	64939	{}	\N	2020-10-12 14:54:06.015433
42669	FCOM	2020-10-08	39.05	39.25	38.91	39.25	44053	{}	\N	2020-10-12 14:54:06.020609
42670	FCOM	2020-10-09	39.39	39.4999	39.26	39.45	56387	{}	\N	2020-10-12 14:54:06.028924
42671	FCOM	2020-10-12	39.96	39.9877	39.791	39.9776	16289	{}	\N	2020-10-12 14:54:06.035911
42672	FDD	2020-09-28	10.17	10.25	10.17	10.23	100834	{}	\N	2020-10-12 14:54:07.279314
42673	FDD	2020-09-29	10.2	10.24	10.1399	10.19	210283	{}	\N	2020-10-12 14:54:07.345795
42674	FDD	2020-09-30	10.27	10.37	10.22	10.26	194703	{}	\N	2020-10-12 14:54:07.384865
42675	FDD	2020-10-01	10.35	10.3835	10.3	10.345	77680	{}	\N	2020-10-12 14:54:07.403253
42676	FDD	2020-10-02	10.27	10.455	10.27	10.425	98855	{}	\N	2020-10-12 14:54:07.417828
42677	FDD	2020-10-05	10.59	10.68	10.58	10.68	64613	{}	\N	2020-10-12 14:54:07.42588
42678	FDD	2020-10-06	10.79	10.815	10.63	10.66	89988	{}	\N	2020-10-12 14:54:07.438362
42679	FDD	2020-10-07	10.7	10.765	10.69	10.74	79247	{}	\N	2020-10-12 14:54:07.447107
42680	FDD	2020-10-08	10.87	10.89	10.83	10.89	75400	{}	\N	2020-10-12 14:54:07.457428
42681	FDD	2020-10-09	10.91	10.98	10.85	10.89	61115	{}	\N	2020-10-12 14:54:07.464216
42682	FDD	2020-10-12	10.9245	10.9399	10.92	10.935	5400	{}	\N	2020-10-12 14:54:07.470809
42683	FDX	2020-09-28	255.27	259.95	254.08	254.44	3394066	{}	\N	2020-10-12 14:54:08.501623
42684	FDX	2020-09-29	254.99	258.2	249.12	253.5	4265228	{}	\N	2020-10-12 14:54:08.509279
42685	FDX	2020-09-30	253.47	257.5061	249.5	251.52	4811859	{}	\N	2020-10-12 14:54:08.514265
42686	FDX	2020-10-01	255.26	257.79	252.88	254.08	5509388	{}	\N	2020-10-12 14:54:08.519264
42687	FDX	2020-10-02	250.55	258.55	250	255.2	5696655	{}	\N	2020-10-12 14:54:08.52472
42688	FDX	2020-10-05	258.8	260.96	257.73	259.21	4776860	{}	\N	2020-10-12 14:54:08.528682
42689	FDX	2020-10-06	260.12	264.98	258.18	259.27	6234303	{}	\N	2020-10-12 14:54:08.532973
42690	FDX	2020-10-07	263.85	270.66	263.5	268.26	5949203	{}	\N	2020-10-12 14:54:08.53775
42691	FDX	2020-10-08	269.73	272.525	266.3501	271.06	4019962	{}	\N	2020-10-12 14:54:08.541812
42692	FDX	2020-10-09	274.4	275	269.54	271.55	4230061	{}	\N	2020-10-12 14:54:08.545556
42693	FDX	2020-10-12	272.26	274.07	271.32	272.5	1433187	{}	\N	2020-10-12 14:54:08.550105
42694	FEMS	2020-09-28	33.42	33.49	33.3	33.3832	17386	{}	\N	2020-10-12 14:54:09.941487
42695	FEMS	2020-09-29	33.31	33.33	33.13	33.33	7721	{}	\N	2020-10-12 14:54:09.950215
42696	FEMS	2020-09-30	33.5	33.72	33.3762	33.53	22911	{}	\N	2020-10-12 14:54:09.957726
42697	FEMS	2020-10-01	33.7706	33.97	33.63	33.97	88930	{}	\N	2020-10-12 14:54:09.964937
42698	FEMS	2020-10-02	33.8952	33.9901	33.65	33.65	21313	{}	\N	2020-10-12 14:54:09.972206
42699	FEMS	2020-10-05	34	34.2921	34	34.19	19741	{}	\N	2020-10-12 14:54:09.979246
42700	FEMS	2020-10-06	34.465	34.465	34.155	34.18	5395	{}	\N	2020-10-12 14:54:09.985699
42701	FEMS	2020-10-07	34.35	34.425	34.28	34.31	40268	{}	\N	2020-10-12 14:54:09.992043
42702	FEMS	2020-10-08	34.295	34.6413	34.286	34.4871	14030	{}	\N	2020-10-12 14:54:09.999168
42703	FEMS	2020-10-09	34.6123	34.87	34.6123	34.82	3877	{}	\N	2020-10-12 14:54:10.006293
42704	FEMS	2020-10-12	34.77	34.77	34.7	34.77	1709	{}	\N	2020-10-12 14:54:10.012887
42705	FEZ	2020-09-28	36.44	36.579	36.39	36.57	3103535	{}	\N	2020-10-12 14:54:11.18278
42706	FEZ	2020-09-29	36.63	36.8	36.44	36.61	1944227	{}	\N	2020-10-12 14:54:11.188959
42707	FEZ	2020-09-30	36.51	36.75	36.16	36.35	4269690	{}	\N	2020-10-12 14:54:11.193856
42708	FEZ	2020-10-01	36.5	36.56	36.25	36.49	3785914	{}	\N	2020-10-12 14:54:11.198054
42709	FEZ	2020-10-02	35.94	36.485	35.93	36.38	2830264	{}	\N	2020-10-12 14:54:11.20207
42710	FEZ	2020-10-05	36.7	37.06	36.7	37.06	2471940	{}	\N	2020-10-12 14:54:11.206767
42711	FEZ	2020-10-06	37.2	37.21	36.54	36.6	3561362	{}	\N	2020-10-12 14:54:11.210759
42712	FEZ	2020-10-07	36.85	37.06	36.78	36.98	2597429	{}	\N	2020-10-12 14:54:11.214896
42713	FEZ	2020-10-08	37.14	37.31	37.12	37.27	1673624	{}	\N	2020-10-12 14:54:11.21957
42714	FEZ	2020-10-09	37.54	37.7	37.455	37.61	1950524	{}	\N	2020-10-12 14:54:11.224137
42715	FEZ	2020-10-12	37.76	37.82	37.75	37.82	510491	{}	\N	2020-10-12 14:54:11.228279
42716	FFIU	2020-09-28	26.485	26.485	26.485	26.485	3	{}	\N	2020-10-12 14:54:12.71654
42717	FFIU	2020-09-29	26.45	26.4607	26.45	26.45	4529	{}	\N	2020-10-12 14:54:12.723947
42718	FFIU	2020-09-30	26.34	26.425	26.34	26.425	835	{}	\N	2020-10-12 14:54:12.729009
42719	FFIU	2020-10-01	26.37	26.42	26.37	26.42	1152	{}	\N	2020-10-12 14:54:12.733363
42720	FFIU	2020-10-02	26.435	26.435	26.435	26.435	92	{}	\N	2020-10-12 14:54:12.737585
42721	FFIU	2020-10-05	26.41	26.43	26.345	26.345	268	{}	\N	2020-10-12 14:54:12.741737
42722	FFIU	2020-10-06	26.42	26.42	26.4186	26.4186	318	{}	\N	2020-10-12 14:54:12.746106
42723	FFIU	2020-10-07	26.49	26.49	26.385	26.385	32793	{}	\N	2020-10-12 14:54:12.750445
42724	FFIU	2020-10-08	26.45	26.47	26.42	26.47	32986	{}	\N	2020-10-12 14:54:12.754642
42725	FFIU	2020-10-09	26.47	26.47	26.447	26.447	27094	{}	\N	2020-10-12 14:54:12.75887
42726	FGEN	2020-09-28	42.58	43.8	42.37	43.01	588857	{}	\N	2020-10-12 14:54:14.339299
42727	FGEN	2020-09-29	43.01	43.07	41.49	41.62	647184	{}	\N	2020-10-12 14:54:14.346512
42728	FGEN	2020-09-30	41.61	41.8	40.94	41.12	780554	{}	\N	2020-10-12 14:54:14.351832
42729	FGEN	2020-10-01	41.31	41.63	40.25	40.97	917276	{}	\N	2020-10-12 14:54:14.356739
42730	FGEN	2020-10-02	40.46	41.67	40.28	41.37	898000	{}	\N	2020-10-12 14:54:14.360948
42731	FGEN	2020-10-05	41.9	44.3	41.71	43.9	596091	{}	\N	2020-10-12 14:54:14.365555
42732	FGEN	2020-10-06	44.28	44.79	42.55	42.63	370238	{}	\N	2020-10-12 14:54:14.370004
42733	FGEN	2020-10-07	43	44.81	42.89	44.58	595761	{}	\N	2020-10-12 14:54:14.374803
42734	FGEN	2020-10-08	45	46.73	44.62	46.62	525511	{}	\N	2020-10-12 14:54:14.379642
42735	FGEN	2020-10-09	46.62	49.23	45.66	48.82	676179	{}	\N	2020-10-12 14:54:14.384061
42736	FGEN	2020-10-12	49.21	49.7	48.55	49.095	108572	{}	\N	2020-10-12 14:54:14.388807
42737	FHN	2020-09-28	9	9.25	8.99	9.15	4023623	{}	\N	2020-10-12 14:54:15.838882
42738	FHN	2020-09-29	9.1	9.28	9	9.22	11728491	{}	\N	2020-10-12 14:54:15.845533
42739	FHN	2020-09-30	9.26	9.56	9.26	9.43	19420539	{}	\N	2020-10-12 14:54:15.850675
42740	FHN	2020-10-01	9.48	9.64	9.41	9.59	13216503	{}	\N	2020-10-12 14:54:15.855642
42741	FHN	2020-10-02	9.35	10.015	9.33	9.95	10688523	{}	\N	2020-10-12 14:54:15.859967
42742	FHN	2020-10-05	10.03	10.4	10.03	10.4	13431258	{}	\N	2020-10-12 14:54:15.863983
42743	FHN	2020-10-06	10.5	10.88	10.28	10.36	11646702	{}	\N	2020-10-12 14:54:15.868371
42744	FHN	2020-10-07	10.52	10.82	10.44	10.66	15949372	{}	\N	2020-10-12 14:54:15.8725
42745	FHN	2020-10-08	10.9	10.9	10.69	10.85	11665940	{}	\N	2020-10-12 14:54:15.876846
42746	FHN	2020-10-09	10.93	11.04	10.68	10.72	13683634	{}	\N	2020-10-12 14:54:15.881093
42747	FHN	2020-10-12	10.71	10.86	10.65	10.82	1039958	{}	\N	2020-10-12 14:54:15.885174
42748	FIF	2020-09-28	8.95	9.07	8.89	9.02	109359	{}	\N	2020-10-12 14:54:16.796243
42749	FIF	2020-09-29	9.04	9.09	9	9.05	205964	{}	\N	2020-10-12 14:54:16.806367
42750	FIF	2020-09-30	9.12	9.14	8.9001	8.95	254403	{}	\N	2020-10-12 14:54:16.813352
42751	FIF	2020-10-01	8.91	8.96	8.82	8.87	244870	{}	\N	2020-10-12 14:54:16.819124
42752	FIF	2020-10-02	8.6	8.95	8.6	8.88	274687	{}	\N	2020-10-12 14:54:16.824128
42753	FIF	2020-10-05	8.94	9.12	8.92	9.03	635878	{}	\N	2020-10-12 14:54:16.82884
42754	FIF	2020-10-06	9.08	9.22	9.03	9.07	232315	{}	\N	2020-10-12 14:54:16.832918
42755	FIF	2020-10-07	9.1	9.31	9.09	9.22	227472	{}	\N	2020-10-12 14:54:16.836989
42756	FIF	2020-10-08	9.22	9.68	9.22	9.65	198448	{}	\N	2020-10-12 14:54:16.841291
42757	FIF	2020-10-09	9.68	9.705	9.58	9.6	115930	{}	\N	2020-10-12 14:54:16.84552
42758	FIF	2020-10-12	9.55	9.65	9.55	9.62	45384	{}	\N	2020-10-12 14:54:16.849327
42759	FCCY	2020-09-28	11.35	11.9	11.35	11.72	6682	{}	\N	2020-10-12 14:54:18.120717
42760	FCCY	2020-09-29	11.68	12.45	11.68	11.95	1170	{}	\N	2020-10-12 14:54:18.128845
42761	FCCY	2020-09-30	11.9	11.9	11.9	11.9	1938	{}	\N	2020-10-12 14:54:18.134905
42762	FCCY	2020-10-01	11.93	12.37	11.83	11.83	4730	{}	\N	2020-10-12 14:54:18.14035
42763	FCCY	2020-10-02	11.66	12.065	11.62	11.79	6266	{}	\N	2020-10-12 14:54:18.146101
42764	FCCY	2020-10-05	12.175	12.175	11.91	12	5855	{}	\N	2020-10-12 14:54:18.150352
42765	FCCY	2020-10-06	12.09	12.44	12.09	12.22	5448	{}	\N	2020-10-12 14:54:18.154958
42766	FCCY	2020-10-07	12.0975	12.1817	11.9	11.95	13410	{}	\N	2020-10-12 14:54:18.158838
42767	FCCY	2020-10-08	12	12.4	11.9001	12.4	11440	{}	\N	2020-10-12 14:54:18.162866
42768	FCCY	2020-10-09	12.44	12.65	12.16	12.65	11011	{}	\N	2020-10-12 14:54:18.16717
42769	FCCY	2020-10-12	12.73	12.73	12.62	12.62	2130	{}	\N	2020-10-12 14:54:18.171509
42770	FDT	2020-09-28	50.23	50.32	50.1575	50.26	18611	{}	\N	2020-10-12 14:54:19.841704
42771	FDT	2020-09-29	50.38	50.465	50.1	50.3	256674	{}	\N	2020-10-12 14:54:19.849346
42772	FDT	2020-09-30	50.22	50.44	50.07	50.18	83077	{}	\N	2020-10-12 14:54:19.854846
42773	FDT	2020-10-01	50.52	50.62	50.32	50.61	134800	{}	\N	2020-10-12 14:54:19.860589
42774	FDT	2020-10-02	49.88	50.505	49.88	50.36	57710	{}	\N	2020-10-12 14:54:19.866358
42775	FDT	2020-10-05	50.93	51.29	50.93	51.26	40567	{}	\N	2020-10-12 14:54:19.870721
42776	FDT	2020-10-06	51.31	51.3575	50.78	50.85	89792	{}	\N	2020-10-12 14:54:19.875568
42777	FDT	2020-10-07	51.17	51.3599	51.1113	51.27	93802	{}	\N	2020-10-12 14:54:19.88065
42778	FDT	2020-10-08	51.59	51.7123	51.5	51.68	101206	{}	\N	2020-10-12 14:54:19.885344
42779	FDT	2020-10-09	51.81	51.9636	51.75	51.9375	21557	{}	\N	2020-10-12 14:54:19.890065
42780	FDT	2020-10-12	51.81	51.94	51.81	51.9	52150	{}	\N	2020-10-12 14:54:19.895213
42781	FENG	2020-09-28	1.24	1.26	1.24	1.24	84712	{}	\N	2020-10-12 14:54:21.403156
42782	FENG	2020-09-29	1.24	1.26	1.23	1.24	208618	{}	\N	2020-10-12 14:54:21.408841
42783	FENG	2020-09-30	1.24	1.2527	1.2	1.23	375424	{}	\N	2020-10-12 14:54:21.450092
42784	FENG	2020-10-01	1.26	1.32	1.24	1.32	847284	{}	\N	2020-10-12 14:54:21.455459
42785	FENG	2020-10-02	1.3	1.39	1.29	1.39	347748	{}	\N	2020-10-12 14:54:21.46197
42786	FENG	2020-10-05	1.39	1.44	1.39	1.39	339958	{}	\N	2020-10-12 14:54:21.466668
42787	FENG	2020-10-06	1.39	1.4	1.34	1.38	149734	{}	\N	2020-10-12 14:54:21.471561
42788	FENG	2020-10-07	1.37	1.43	1.36	1.41	149140	{}	\N	2020-10-12 14:54:21.476391
42789	FENG	2020-10-08	1.41	1.43	1.34	1.4	258610	{}	\N	2020-10-12 14:54:21.481452
42790	FENG	2020-10-09	1.4	1.42	1.37	1.41	56408	{}	\N	2020-10-12 14:54:21.48617
42791	FENG	2020-10-12	1.4	1.4099	1.38	1.4	19576	{}	\N	2020-10-12 14:54:21.490536
42792	FFA	2020-09-28	14.8	14.94	14.735	14.79	31986	{}	\N	2020-10-12 14:54:22.698676
42793	FFA	2020-09-29	14.88	14.97	14.781	14.92	78286	{}	\N	2020-10-12 14:54:22.708068
42794	FFA	2020-09-30	15.1	15.19	14.93	15.14	294519	{}	\N	2020-10-12 14:54:22.715117
42795	FFA	2020-10-01	15.25	15.315	15.06	15.21	117032	{}	\N	2020-10-12 14:54:22.72118
42796	FFA	2020-10-02	15.1	15.1	14.81	15.05	64132	{}	\N	2020-10-12 14:54:22.726664
42797	FFA	2020-10-05	15.06	15.17	15	15.12	86970	{}	\N	2020-10-12 14:54:22.731638
42798	FFA	2020-10-06	15.23	15.3	14.98	15.14	42077	{}	\N	2020-10-12 14:54:22.735789
42799	FFA	2020-10-07	15.3	15.44	15.1545	15.43	144073	{}	\N	2020-10-12 14:54:22.740393
42800	FFA	2020-10-08	15.62	15.62	15.42	15.51	69624	{}	\N	2020-10-12 14:54:22.744539
42801	FFA	2020-10-09	15.65	15.65	15.49	15.55	52634	{}	\N	2020-10-12 14:54:22.748888
42802	FFA	2020-10-12	15.55	15.61	15.52	15.61	36818	{}	\N	2020-10-12 14:54:22.753214
42803	FFIC	2020-09-28	10.48	10.89	10.48	10.71	100450	{}	\N	2020-10-12 14:54:24.125556
42804	FFIC	2020-09-29	10.65	10.65	10.27	10.54	154976	{}	\N	2020-10-12 14:54:24.134296
42805	FFIC	2020-09-30	10.64	10.77	10.43	10.52	89418	{}	\N	2020-10-12 14:54:24.140333
42806	FFIC	2020-10-01	10.5	10.78	10.35	10.72	180275	{}	\N	2020-10-12 14:54:24.145761
42807	FFIC	2020-10-02	10.58	11.14	10.55	11.13	76647	{}	\N	2020-10-12 14:54:24.151232
42808	FFIC	2020-10-05	11.19	11.55	11.17	11.51	113378	{}	\N	2020-10-12 14:54:24.15629
42809	FFIC	2020-10-06	11.68	12	11.52	11.55	123840	{}	\N	2020-10-12 14:54:24.160519
42810	FFIC	2020-10-07	11.06	11.9	11.06	11.7	109232	{}	\N	2020-10-12 14:54:24.164624
42811	FFIC	2020-10-08	11.83	11.925	11.67	11.84	70353	{}	\N	2020-10-12 14:54:24.169004
42812	FFIC	2020-10-09	11.54	12.01	11.54	11.62	70114	{}	\N	2020-10-12 14:54:24.173199
42813	FFIC	2020-10-12	11.6	11.67	11.51	11.67	4796	{}	\N	2020-10-12 14:54:24.177374
42814	FFWM	2020-09-28	12.71	13.31	12.71	13.24	129116	{}	\N	2020-10-12 14:54:25.557239
42815	FFWM	2020-09-29	13.36	13.36	12.93	13.14	120630	{}	\N	2020-10-12 14:54:25.56781
42816	FFWM	2020-09-30	13.16	13.35	13.05	13.07	143698	{}	\N	2020-10-12 14:54:25.575019
42817	FFWM	2020-10-01	13.05	13.3	12.99	13.22	95881	{}	\N	2020-10-12 14:54:25.581205
42818	FFWM	2020-10-02	13.05	13.47	12.97	13.44	114573	{}	\N	2020-10-12 14:54:25.587519
42819	FFWM	2020-10-05	13.6	13.97	13.53	13.93	75880	{}	\N	2020-10-12 14:54:25.592356
42820	FFWM	2020-10-06	14.05	14.37	13.865	13.87	169470	{}	\N	2020-10-12 14:54:25.597
42821	FFWM	2020-10-07	14.02	14.28	13.74	14.01	147433	{}	\N	2020-10-12 14:54:25.601269
42822	FFWM	2020-10-08	14.15	14.45	14.094	14.38	107367	{}	\N	2020-10-12 14:54:25.605751
42823	FFWM	2020-10-09	14.55	14.73	14.23	14.31	72760	{}	\N	2020-10-12 14:54:25.6115
42824	FFWM	2020-10-12	14.26	14.33	14.14	14.33	14627	{}	\N	2020-10-12 14:54:25.615876
42825	FGM	2020-09-28	44.33	44.52	44.3205	44.3205	1652	{}	\N	2020-10-12 14:54:26.683158
42826	FGM	2020-10-02	44.49	45.0467	44.49	45.0467	2093	{}	\N	2020-10-12 14:54:26.693591
42827	FGM	2020-10-05	46.0941	46.0941	45.965	45.965	1411	{}	\N	2020-10-12 14:54:26.70113
42828	FGM	2020-10-07	46.1	46.2038	46.1	46.2038	501	{}	\N	2020-10-12 14:54:26.706153
42829	FGM	2020-10-08	46.58	46.88	46.58	46.88	2203	{}	\N	2020-10-12 14:54:26.712384
42830	FGM	2020-10-09	46.6214	46.6214	46.6214	46.6214	1300	{}	\N	2020-10-12 14:54:26.71758
42831	FGM	2020-10-12	46.89	46.89	46.89	46.89	185	{}	\N	2020-10-12 14:54:26.721547
42832	FIDI	2020-09-28	15.85	15.899	15.77	15.84	15584	{}	\N	2020-10-12 14:54:27.677484
42833	FIDI	2020-09-29	15.8097	15.82	15.8	15.8	11119	{}	\N	2020-10-12 14:54:27.68733
42834	FIDI	2020-09-30	15.86	15.8721	15.8025	15.85	5828	{}	\N	2020-10-12 14:54:27.69386
42835	FIDI	2020-10-01	15.85	15.88	15.75	15.75	11007	{}	\N	2020-10-12 14:54:27.699312
42836	FIDI	2020-10-02	15.67	15.861	15.64	15.8116	19527	{}	\N	2020-10-12 14:54:27.703865
42837	FIDI	2020-10-05	15.9199	16	15.8666	16	12941	{}	\N	2020-10-12 14:54:27.708549
42838	FIDI	2020-10-06	16.0922	16.2	15.95	15.96	10155	{}	\N	2020-10-12 14:54:27.713295
42839	FIDI	2020-10-07	16.1	16.11	16.045	16.102	3809	{}	\N	2020-10-12 14:54:27.717187
42840	FIDI	2020-10-08	16.1386	16.19	16.1386	16.19	1499	{}	\N	2020-10-12 14:54:27.720967
42841	FIDI	2020-10-09	16.24	16.2599	16.23	16.23	7011	{}	\N	2020-10-12 14:54:27.726471
42842	FIDI	2020-10-12	16.25	16.25	16.21	16.21	1169	{}	\N	2020-10-12 14:54:27.731182
42843	FICO	2020-09-28	426.04	432.14	424.64	427.52	218531	{}	\N	2020-10-12 14:54:29.210129
42844	FICO	2020-09-29	426.32	431.66	425.95	427.21	230089	{}	\N	2020-10-12 14:54:29.217197
42845	FICO	2020-09-30	427.21	433.79	418.11	425.38	606605	{}	\N	2020-10-12 14:54:29.223189
42846	FICO	2020-10-01	431.62	439.68	431.495	436.99	338880	{}	\N	2020-10-12 14:54:29.228223
42847	FICO	2020-10-02	429.5	438.27	421.45	433.72	367944	{}	\N	2020-10-12 14:54:29.234207
42848	FICO	2020-10-05	436.95	439.55	432.22	437.35	204096	{}	\N	2020-10-12 14:54:29.238543
42849	FICO	2020-10-06	437.17	445.35	433.17	435.83	302312	{}	\N	2020-10-12 14:54:29.242501
42850	FICO	2020-10-07	441.07	446.495	438.03	446.44	270059	{}	\N	2020-10-12 14:54:29.247172
42851	FICO	2020-10-08	450.25	450.25	437.83	439.19	325315	{}	\N	2020-10-12 14:54:29.251459
42852	FICO	2020-10-09	440.95	446.9	438.56	442.3	240037	{}	\N	2020-10-12 14:54:29.255914
42853	FICO	2020-10-12	448	448	442.42	443.71	20702	{}	\N	2020-10-12 14:54:29.260145
42854	FIHD	2020-09-28	150.64	150.64	150.1241	150.1241	308	{}	\N	2020-10-12 14:54:30.70195
42855	FIHD	2020-09-29	149	149	148.5	148.5	636	{}	\N	2020-10-12 14:54:30.709833
42856	FIHD	2020-09-30	149.7	150.555	149	149.95	957	{}	\N	2020-10-12 14:54:30.716419
42857	FIHD	2020-10-01	148.48	150.18	148.48	148.51	695	{}	\N	2020-10-12 14:54:30.722148
42858	FIHD	2020-10-02	149.79	151.1988	149.79	150.755	1161	{}	\N	2020-10-12 14:54:30.728899
42859	FIHD	2020-10-05	148.12	154.61	148.12	154.61	1986	{}	\N	2020-10-12 14:54:30.735005
42860	FIHD	2020-10-06	152.0922	152.0922	152.0922	152.0922	290	{}	\N	2020-10-12 14:54:30.740028
42861	FIHD	2020-10-07	153.9103	155.8113	153.9103	155.8113	981	{}	\N	2020-10-12 14:54:30.744997
42862	FIHD	2020-10-08	156.9067	158.32	156.9067	158.32	444	{}	\N	2020-10-12 14:54:30.749523
42863	FIHD	2020-10-09	158.4	160	158.4	158.55	698	{}	\N	2020-10-12 14:54:30.753798
42864	FCEL	2020-09-28	2.3722	2.49	2.34	2.43	8831568	{}	\N	2020-10-12 14:54:31.689161
42865	FCEL	2020-09-29	2.51	2.57	2.43	2.56	16270114	{}	\N	2020-10-12 14:54:31.695265
42866	FCEL	2020-09-30	2.19	2.28	2.12	2.14	55434988	{}	\N	2020-10-12 14:54:31.700187
42867	FCEL	2020-10-01	2.19	2.19	2.1	2.11	21170181	{}	\N	2020-10-12 14:54:31.70438
42868	FCEL	2020-10-02	2.03	2.07	1.98	2.02	22746877	{}	\N	2020-10-12 14:54:31.709292
42869	FCEL	2020-10-05	2.09	2.12	1.58	1.89	95377899	{}	\N	2020-10-12 14:54:31.715696
42870	FCEL	2020-10-06	1.95	2.02	1.85	1.93	29718975	{}	\N	2020-10-12 14:54:31.720595
42871	FCEL	2020-10-07	1.9659	2.22	1.93	2.16	35914635	{}	\N	2020-10-12 14:54:31.724808
42872	FCEL	2020-10-08	2.45	2.61	2.31	2.38	68903182	{}	\N	2020-10-12 14:54:31.729511
42873	FCEL	2020-10-09	2.54	2.66	2.43	2.62	62006723	{}	\N	2020-10-12 14:54:31.734384
42874	FCEL	2020-10-12	2.74	2.75	2.45	2.5081	18131309	{}	\N	2020-10-12 14:54:31.73892
42875	FCPT	2020-09-28	25.68	26.16	25.68	26.13	446947	{}	\N	2020-10-12 14:54:32.729712
42876	FCPT	2020-09-29	25.8	25.86	25.25	25.64	776365	{}	\N	2020-10-12 14:54:32.736034
42877	FCPT	2020-09-30	25.71	26.31	25.21	25.59	1383742	{}	\N	2020-10-12 14:54:32.742591
42878	FCPT	2020-10-01	25.66	26.3	25.44	26.27	874709	{}	\N	2020-10-12 14:54:32.749218
42879	FCPT	2020-10-02	25.95	27.22	25.64	27.11	1352028	{}	\N	2020-10-12 14:54:32.755426
42880	FCPT	2020-10-05	27.48	28.19	27.04	27.88	1351635	{}	\N	2020-10-12 14:54:32.760837
42881	FCPT	2020-10-06	28.16	28.67	27.4999	28.06	939274	{}	\N	2020-10-12 14:54:32.766876
42882	FCPT	2020-10-07	28.29	28.49	27.5	27.76	1463272	{}	\N	2020-10-12 14:54:32.772543
42883	FCPT	2020-10-08	28.13	28.62	28.07	28.33	663313	{}	\N	2020-10-12 14:54:32.778784
42884	FCPT	2020-10-09	28.55	28.67	27.92	28.03	536324	{}	\N	2020-10-12 14:54:32.784418
42885	FCPT	2020-10-12	28.04	28.16	27.74	28.01	133441	{}	\N	2020-10-12 14:54:32.78945
42886	FELE	2020-09-28	57.58	58.875	57.58	58.43	135284	{}	\N	2020-10-12 14:54:34.354755
42887	FELE	2020-09-29	60.72	61.25	59.54	59.62	142265	{}	\N	2020-10-12 14:54:34.362486
42888	FELE	2020-09-30	59.4	60.62	58.53	58.83	202053	{}	\N	2020-10-12 14:54:34.368637
42889	FELE	2020-10-01	59.04	59.8725	58.4	58.88	128256	{}	\N	2020-10-12 14:54:34.373132
42890	FELE	2020-10-02	57.75	59.89	57.75	59.31	95234	{}	\N	2020-10-12 14:54:34.377915
42891	FELE	2020-10-05	59.74	61	59.74	60.91	140187	{}	\N	2020-10-12 14:54:34.382233
42892	FELE	2020-10-06	61	62.91	60.505	62.13	186291	{}	\N	2020-10-12 14:54:34.386276
42893	FELE	2020-10-07	62.56	63.14	61.44	61.86	191869	{}	\N	2020-10-12 14:54:34.390777
42894	FELE	2020-10-08	62.06	63.03	61.91	62.95	118551	{}	\N	2020-10-12 14:54:34.397623
42895	FELE	2020-10-09	63.41	63.885	62.49	62.64	111575	{}	\N	2020-10-12 14:54:34.401841
42896	FELE	2020-10-12	63.01	64.07	62.8	63.73	108444	{}	\N	2020-10-12 14:54:34.406284
42897	FF	2020-09-28	11.01	11.51	10.9	11.41	140755	{}	\N	2020-10-12 14:54:35.389373
42898	FF	2020-09-29	11.41	11.49	11.17	11.32	216858	{}	\N	2020-10-12 14:54:35.396665
42899	FF	2020-09-30	11.4	11.57	11.35	11.37	274459	{}	\N	2020-10-12 14:54:35.40154
42900	FF	2020-10-01	11.54	11.57	11.09	11.21	252066	{}	\N	2020-10-12 14:54:35.406896
42901	FF	2020-10-02	11.075	11.68	11.03	11.39	222133	{}	\N	2020-10-12 14:54:35.411899
42902	FF	2020-10-05	11.63	11.9	11.58	11.77	311939	{}	\N	2020-10-12 14:54:35.416255
42903	FF	2020-10-06	11.96	11.96	11.54	11.55	324592	{}	\N	2020-10-12 14:54:35.421134
42904	FF	2020-10-07	11.64	11.81	11.46	11.71	306012	{}	\N	2020-10-12 14:54:35.426064
42905	FF	2020-10-08	11.93	11.93	11.56	11.64	252911	{}	\N	2020-10-12 14:54:35.430652
42906	FF	2020-10-09	11.75	11.89	11.56	11.8	213392	{}	\N	2020-10-12 14:54:35.435233
42907	FF	2020-10-12	11.83	11.85	11.67	11.76	22314	{}	\N	2020-10-12 14:54:35.440258
42908	FFIV	2020-09-28	121.77	123.49	121.5375	123.37	569464	{}	\N	2020-10-12 14:54:36.400777
42909	FFIV	2020-09-29	122.99	125.025	122.48	124.01	694395	{}	\N	2020-10-12 14:54:36.408568
42910	FFIV	2020-09-30	124.03	124.65	122.31	122.77	608797	{}	\N	2020-10-12 14:54:36.414416
42911	FFIV	2020-10-01	123.38	123.86	122.19	123.07	433747	{}	\N	2020-10-12 14:54:36.419854
42912	FFIV	2020-10-02	122.41	123.38	121.77	122.5	452899	{}	\N	2020-10-12 14:54:36.425697
42913	FFIV	2020-10-05	123.87	125.45	123.49	125.37	758301	{}	\N	2020-10-12 14:54:36.432124
42914	FFIV	2020-10-06	126.38	128.2	125.085	125.42	475432	{}	\N	2020-10-12 14:54:36.437682
42915	FFIV	2020-10-07	127.07	128.38	126.57	127.63	547808	{}	\N	2020-10-12 14:54:36.444378
42916	FFIV	2020-10-08	128.09	128.96	127.42	128.06	635886	{}	\N	2020-10-12 14:54:36.45061
42917	FFIV	2020-10-09	128.43	128.735	125.56	127.56	518759	{}	\N	2020-10-12 14:54:36.458888
42918	FFIV	2020-10-12	127.83	128.5	125.84	126.445	183928	{}	\N	2020-10-12 14:54:36.467536
42919	FHK	2020-10-05	33.72	33.74	33.72	33.74	346	{}	\N	2020-10-12 14:54:38.126671
42920	FHK	2020-10-08	34.91	34.91	34.91	34.91	330	{}	\N	2020-10-12 14:54:38.13312
42921	FHLC	2020-09-28	51.94	52.25	51.78	51.86	115647	{}	\N	2020-10-12 14:54:39.965333
42922	FHLC	2020-09-29	51.97	52.2689	51.69	51.85	76011	{}	\N	2020-10-12 14:54:39.972564
42923	FHLC	2020-09-30	52.06	53.07	52.06	52.6	172606	{}	\N	2020-10-12 14:54:39.977595
42924	FHLC	2020-10-01	52.88	53.09	52.3	52.51	176805	{}	\N	2020-10-12 14:54:39.982235
42925	FHLC	2020-10-02	51.85	52.5099	51.8	52	112438	{}	\N	2020-10-12 14:54:39.986903
42926	FHLC	2020-10-05	52.5	53.28	52.32	53.27	134330	{}	\N	2020-10-12 14:54:39.991031
42927	FHLC	2020-10-06	53.38	53.5292	52.61	52.67	859477	{}	\N	2020-10-12 14:54:39.995149
42928	FHLC	2020-10-07	53.03	53.85	53.03	53.68	116779	{}	\N	2020-10-12 14:54:39.999161
42929	FHLC	2020-10-08	54	54.14	53.87	53.98	106529	{}	\N	2020-10-12 14:54:40.00293
42930	FHLC	2020-10-09	54.24	54.57	54.17	54.5	210414	{}	\N	2020-10-12 14:54:40.006564
42931	FHLC	2020-10-12	54.77	54.785	54.555	54.7682	41126	{}	\N	2020-10-12 14:54:40.010313
42932	FIDU	2020-09-28	40.19	40.689	40.19	40.4108	56267	{}	\N	2020-10-12 14:54:41.132109
42933	FIDU	2020-09-29	40.36	40.5223	40.05	40.14	39110	{}	\N	2020-10-12 14:54:41.138418
42934	FIDU	2020-09-30	40.24	40.6799	39.7997	40.14	36314	{}	\N	2020-10-12 14:54:41.143423
42935	FIDU	2020-10-01	40.28	40.46	39.805	40.03	56014	{}	\N	2020-10-12 14:54:41.147834
42936	FIDU	2020-10-02	39.45	40.65	39.4	40.48	109120	{}	\N	2020-10-12 14:54:41.152307
42937	FIDU	2020-10-05	40.83	41.225	40.83	41.19	42433	{}	\N	2020-10-12 14:54:41.156821
42938	FIDU	2020-10-06	41.41	41.6825	40.771	40.8	50914	{}	\N	2020-10-12 14:54:41.161057
42939	FIDU	2020-10-07	41.17	41.73	41.17	41.62	86348	{}	\N	2020-10-12 14:54:41.165424
42940	FIDU	2020-10-08	41.9	42.01	41.629	42.01	45916	{}	\N	2020-10-12 14:54:41.169553
42941	FIDU	2020-10-09	42.38	42.4399	42.08	42.17	206143	{}	\N	2020-10-12 14:54:41.173654
42942	FIDU	2020-10-12	42.46	42.46	42.29	42.4	35894	{}	\N	2020-10-12 14:54:41.177805
45158	GDVD	2020-10-12	29.42	29.42	29.42	29.42	201	{}	\N	2020-10-12 14:59:47.863717
45159	GENE	2020-09-28	3.3	3.3785	3.2516	3.29	259373	{}	\N	2020-10-12 14:59:49.269638
45160	GENE	2020-09-29	3.32	3.45	3.26	3.41	733964	{}	\N	2020-10-12 14:59:49.279071
45161	GENE	2020-09-30	3.47	3.6	3.38	3.46	1195453	{}	\N	2020-10-12 14:59:49.284894
45162	GENE	2020-10-01	3.77	3.77	3.38	3.52	2039553	{}	\N	2020-10-12 14:59:49.290961
45163	GENE	2020-10-02	3.39	3.47	3.36	3.41	313873	{}	\N	2020-10-12 14:59:49.295802
45164	GENE	2020-10-05	3.41	3.45	3.27	3.36	295149	{}	\N	2020-10-12 14:59:49.300125
45165	GENE	2020-10-06	3.33	3.3899	3.31	3.31	260200	{}	\N	2020-10-12 14:59:49.305983
45166	GENE	2020-10-07	3.35	3.5794	3.33	3.5	839606	{}	\N	2020-10-12 14:59:49.311883
45167	GENE	2020-10-08	3.54	3.54	3.37	3.41	834459	{}	\N	2020-10-12 14:59:49.316272
45168	GENE	2020-10-09	3.46	3.49	3.41	3.48	242996	{}	\N	2020-10-12 14:59:49.320791
45169	GENE	2020-10-12	3.47	3.52	3.43	3.48	139704	{}	\N	2020-10-12 14:59:49.3254
45170	FUTY	2020-09-28	37.75	38.02	37.6133	37.64	190660	{}	\N	2020-10-12 14:59:51.087274
45171	FUTY	2020-09-29	37.76	37.995	37.45	37.65	110000	{}	\N	2020-10-12 14:59:51.095661
45172	FUTY	2020-09-30	37.88	38.15	37.7227	37.94	104312	{}	\N	2020-10-12 14:59:51.101053
45173	FUTY	2020-10-01	38.07	38.4	37.95	38.33	86719	{}	\N	2020-10-12 14:59:51.106561
45174	FUTY	2020-10-02	37.97	38.98	37.95	38.81	101807	{}	\N	2020-10-12 14:59:51.111494
45175	FUTY	2020-10-05	38.9	39.36	38.71	39.24	334595	{}	\N	2020-10-12 14:59:51.115757
45176	FUTY	2020-10-06	39.2901	40.07	39.165	39.58	137458	{}	\N	2020-10-12 14:59:51.119748
45177	FUTY	2020-10-07	39.77	39.99	39.58	39.83	144838	{}	\N	2020-10-12 14:59:51.124132
45178	FUTY	2020-10-08	40.01	40.63	40	40.57	88250	{}	\N	2020-10-12 14:59:51.128171
45179	FUTY	2020-10-09	40.75	40.79	40.32	40.57	84136	{}	\N	2020-10-12 14:59:51.131957
45180	FUTY	2020-10-12	40.58	40.79	40.5	40.75	45186	{}	\N	2020-10-12 14:59:51.136029
45181	FXA	2020-09-28	70.3701	70.53	70.36	70.52	9428	{}	\N	2020-10-12 14:59:52.908888
45182	FXA	2020-09-29	71.02	71.16	70.96	71.05	12629	{}	\N	2020-10-12 14:59:52.918172
45183	FXA	2020-09-30	71.1	71.54	71.1	71.38	10419	{}	\N	2020-10-12 14:59:52.924606
45184	FXA	2020-10-01	71.66	71.75	71.42	71.72	21173	{}	\N	2020-10-12 14:59:52.930579
45185	FXA	2020-10-02	71.27	71.4499	71.25	71.38	30238	{}	\N	2020-10-12 14:59:52.936218
45186	FXA	2020-10-05	71.6	71.71	71.531	71.61	23809	{}	\N	2020-10-12 14:59:52.941492
45187	FXA	2020-10-06	71.42	71.46	70.92	70.96	9793	{}	\N	2020-10-12 14:59:52.945747
45188	FXA	2020-10-07	71.19	71.26	71.12	71.13	6724	{}	\N	2020-10-12 14:59:52.950146
45189	FXA	2020-10-08	71.43	71.4641	71.323	71.43	6541	{}	\N	2020-10-12 14:59:52.954451
45190	FXA	2020-10-09	71.97	72.18	71.97	72.18	24065	{}	\N	2020-10-12 14:59:52.958735
45191	FXA	2020-10-12	71.91	71.94	71.85	71.94	3493	{}	\N	2020-10-12 14:59:52.963495
42943	FILL	2020-09-28	10.63	10.77	10.63	10.6653	30438	{}	\N	2020-10-12 14:54:43.529808
42944	FILL	2020-09-29	10.58	10.58	10.36	10.48	15113	{}	\N	2020-10-12 14:54:43.542339
42945	FILL	2020-09-30	10.48	10.5316	10.3901	10.45	8864	{}	\N	2020-10-12 14:54:43.546273
42946	FILL	2020-10-01	10.37	10.37	10.15	10.22	78742	{}	\N	2020-10-12 14:54:43.550391
42947	FILL	2020-10-02	10.01	10.3	10	10.25	193754	{}	\N	2020-10-12 14:54:43.554465
42948	FILL	2020-10-05	10.3	10.56	10.29	10.48	88008	{}	\N	2020-10-12 14:54:43.558347
42949	FILL	2020-10-06	10.75	10.78	10.44	10.44	46040	{}	\N	2020-10-12 14:54:43.56256
42950	FILL	2020-10-07	10.48	10.555	10.4401	10.53	19504	{}	\N	2020-10-12 14:54:43.567632
42951	FILL	2020-10-08	10.64	10.85	10.64	10.8065	40716	{}	\N	2020-10-12 14:54:43.571727
42952	FILL	2020-10-09	10.94	10.94	10.7374	10.82	14906	{}	\N	2020-10-12 14:54:43.575754
42953	FILL	2020-10-12	10.75	10.75	10.695	10.75	10350	{}	\N	2020-10-12 14:54:43.580226
42954	FDIS	2020-09-28	61.73	62.1333	61.359	61.995	162851	{}	\N	2020-10-12 14:54:46.805017
42955	FDIS	2020-09-29	62.01	62.0622	61.51	61.51	88220	{}	\N	2020-10-12 14:54:46.81643
42956	FDIS	2020-09-30	61.75	62.75	61.6769	62.03	155243	{}	\N	2020-10-12 14:54:46.823037
42957	FDIS	2020-10-01	62.85	63.3	62.4325	63.3	251258	{}	\N	2020-10-12 14:54:46.828493
42958	FDIS	2020-10-02	61.91	62.82	61.627	62.44	142056	{}	\N	2020-10-12 14:54:46.833404
42959	FDIS	2020-10-05	62.9	63.4733	62.8503	63.4	147093	{}	\N	2020-10-12 14:54:46.837963
42960	FDIS	2020-10-06	63.5007	63.635	61.96	62.16	826280	{}	\N	2020-10-12 14:54:46.842223
42961	FDIS	2020-10-07	62.81	63.775	62.81	63.63	110518	{}	\N	2020-10-12 14:54:46.845917
42962	FDIS	2020-10-08	64.24	64.24	63.6846	63.97	83362	{}	\N	2020-10-12 14:54:46.849556
42963	FDIS	2020-10-09	64.38	65	64.33	64.94	123219	{}	\N	2020-10-12 14:54:46.853673
42964	FDIS	2020-10-12	65.77	65.9	65.4978	65.9	58538	{}	\N	2020-10-12 14:54:46.858508
42965	FDN	2020-09-28	188.35	188.83	186.23	188.21	791427	{}	\N	2020-10-12 14:54:47.86239
42966	FDN	2020-09-29	188.19	189.06	187.12	187.95	565885	{}	\N	2020-10-12 14:54:47.873038
42967	FDN	2020-09-30	188.07	190.355	187	188.16	588242	{}	\N	2020-10-12 14:54:47.880637
42968	FDN	2020-10-01	190.62	192.33	189.6	191.88	479275	{}	\N	2020-10-12 14:54:47.886717
42969	FDN	2020-10-02	187.7	191.72	187.56	188.66	449861	{}	\N	2020-10-12 14:54:47.892371
42970	FDN	2020-10-05	189.85	191.93	189.5001	191.74	271907	{}	\N	2020-10-12 14:54:47.897706
42971	FDN	2020-10-06	191.52	193.04	188.4	189.09	500083	{}	\N	2020-10-12 14:54:47.902288
42972	FDN	2020-10-07	190.84	192.95	190.2273	192.48	340221	{}	\N	2020-10-12 14:54:47.906719
42973	FDN	2020-10-08	194.43	194.6	192.55	193.5	219559	{}	\N	2020-10-12 14:54:47.911414
42974	FDN	2020-10-09	194.83	197.05	194.71	196.76	254133	{}	\N	2020-10-12 14:54:47.915898
42975	FDN	2020-10-12	199.81	200.6	198.38	199.98	121161	{}	\N	2020-10-12 14:54:47.920279
42976	FE	2020-09-28	28.87	28.89	28.43	28.45	3251626	{}	\N	2020-10-12 14:54:49.543753
42977	FE	2020-09-29	28.53	28.73	28.12	28.6	4902338	{}	\N	2020-10-12 14:54:49.55406
42978	FE	2020-09-30	28.73	28.98	28.48	28.71	9822379	{}	\N	2020-10-12 14:54:49.561614
42979	FE	2020-10-01	28.8	29.02	28.45	28.76	5314802	{}	\N	2020-10-12 14:54:49.567206
42980	FE	2020-10-02	28.57	29.76	28.4	29.52	7637947	{}	\N	2020-10-12 14:54:49.571354
42981	FE	2020-10-05	29.66	30.04	29.415	29.91	5741372	{}	\N	2020-10-12 14:54:49.575302
42982	FE	2020-10-06	29.95	30.77	29.82	30.41	10488164	{}	\N	2020-10-12 14:54:49.579442
42983	FE	2020-10-07	30.58	30.7661	30.37	30.56	7688672	{}	\N	2020-10-12 14:54:49.583426
42984	FE	2020-10-08	30.67	31.36	30.6	31.28	6778239	{}	\N	2020-10-12 14:54:49.587383
42985	FE	2020-10-09	31.36	31.5799	31.06	31.22	8579341	{}	\N	2020-10-12 14:54:49.59143
42986	FE	2020-10-12	31.13	31.53	31.08	31.28	1984651	{}	\N	2020-10-12 14:54:49.595378
42987	FEN	2020-09-28	9.17	9.2387	9.07	9.16	103056	{}	\N	2020-10-12 14:54:50.758014
42988	FEN	2020-09-29	9.1	9.18	9.06	9.16	66809	{}	\N	2020-10-12 14:54:50.768773
42989	FEN	2020-09-30	9.14	9.29	9.02	9.06	61764	{}	\N	2020-10-12 14:54:50.774913
42990	FEN	2020-10-01	9.18	9.18	8.9501	8.99	74797	{}	\N	2020-10-12 14:54:50.780703
42991	FEN	2020-10-02	8.8	9.29	8.8	9.23	85576	{}	\N	2020-10-12 14:54:50.785637
42992	FEN	2020-10-05	9.29	9.64	9.29	9.63	138339	{}	\N	2020-10-12 14:54:50.791191
42993	FEN	2020-10-06	9.57	9.68	9.57	9.6	119742	{}	\N	2020-10-12 14:54:50.796437
42994	FEN	2020-10-07	9.7499	9.8	9.63	9.74	108902	{}	\N	2020-10-12 14:54:50.800229
42995	FEN	2020-10-08	9.84	10.15	9.812	10.11	143624	{}	\N	2020-10-12 14:54:50.804034
42996	FEN	2020-10-09	10.11	10.18	10.03	10.1	90668	{}	\N	2020-10-12 14:54:50.807878
42997	FEN	2020-10-12	10.17	10.17	10.07	10.11	35538	{}	\N	2020-10-12 14:54:50.811617
42998	FEUZ	2020-09-28	36.3064	36.385	36.2053	36.2634	1544	{}	\N	2020-10-12 14:54:53.30578
42999	FEUZ	2020-09-29	36.2999	36.35	36.095	36.35	1072	{}	\N	2020-10-12 14:54:53.313882
43000	FEUZ	2020-09-30	36.52	36.52	36.39	36.44	2312	{}	\N	2020-10-12 14:54:53.320165
43001	FEUZ	2020-10-01	36.65	36.6599	36.5675	36.6599	925	{}	\N	2020-10-12 14:54:53.325972
43002	FEUZ	2020-10-02	36.605	36.66	36.585	36.585	453	{}	\N	2020-10-12 14:54:53.332533
43003	FEUZ	2020-10-05	37.15	37.5597	37.15	37.19	2116	{}	\N	2020-10-12 14:54:53.337358
43004	FEUZ	2020-10-06	37.58	37.58	37.075	37.075	533	{}	\N	2020-10-12 14:54:53.342146
43005	FEUZ	2020-10-08	37.76	37.76	37.76	37.76	229	{}	\N	2020-10-12 14:54:53.347407
43006	FEUZ	2020-10-09	38.025	38.025	38.02	38.02	597	{}	\N	2020-10-12 14:54:53.356181
43007	FEUZ	2020-10-12	38.1785	38.1785	38.1785	38.1785	282	{}	\N	2020-10-12 14:54:53.361942
43008	FFSG	2020-09-28	26.66	26.79	26.6072	26.6072	14149	{}	\N	2020-10-12 14:54:54.822959
43009	FFSG	2020-09-29	26.76	26.7699	26.32	26.42	11299	{}	\N	2020-10-12 14:54:54.833342
43010	FFSG	2020-09-30	26.77	26.8641	26.44	26.57	25583	{}	\N	2020-10-12 14:54:54.840165
43011	FFSG	2020-10-01	26.9799	27.015	26.64	26.84	6813	{}	\N	2020-10-12 14:54:54.846182
43012	FFSG	2020-10-02	26.73	26.9011	26.65	26.78	8706	{}	\N	2020-10-12 14:54:54.851567
43013	FFSG	2020-10-05	27.34	27.3859	27.15	27.3859	10420	{}	\N	2020-10-12 14:54:54.856722
43014	FFSG	2020-10-06	27.58	27.58	26.99	26.99	4191	{}	\N	2020-10-12 14:54:54.86106
43015	FFSG	2020-10-07	27.6	27.6	27.316	27.45	11732	{}	\N	2020-10-12 14:54:54.865229
43016	FFSG	2020-10-08	27.84	27.9899	27.55	27.92	17252	{}	\N	2020-10-12 14:54:54.870198
43017	FFSG	2020-10-09	28.06	28.0898	27.8	27.9199	4899	{}	\N	2020-10-12 14:54:54.875495
43018	FGB	2020-09-28	2.31	2.33	2.22	2.32	153292	{}	\N	2020-10-12 14:54:55.91237
43019	FGB	2020-09-29	2.33	2.34	2.26	2.31	230390	{}	\N	2020-10-12 14:54:55.921824
43020	FGB	2020-09-30	2.29	2.33	2.28	2.29	225218	{}	\N	2020-10-12 14:54:55.927419
43021	FGB	2020-10-01	2.34	2.34	2.29	2.34	227972	{}	\N	2020-10-12 14:54:55.932466
43022	FGB	2020-10-02	2.32	2.39	2.315	2.38	237924	{}	\N	2020-10-12 14:54:55.936882
43023	FGB	2020-10-05	2.32	2.41	2.32	2.37	154444	{}	\N	2020-10-12 14:54:55.941429
43024	FGB	2020-10-06	2.39	2.58	2.34	2.55	686757	{}	\N	2020-10-12 14:54:55.945388
43025	FGB	2020-10-07	2.6	2.84	2.56	2.81	659276	{}	\N	2020-10-12 14:54:55.948822
43026	FGB	2020-10-08	2.75	2.82	2.71	2.81	380144	{}	\N	2020-10-12 14:54:55.952553
43027	FGB	2020-10-09	2.83	2.84	2.66	2.7	261128	{}	\N	2020-10-12 14:54:55.956074
43028	FGB	2020-10-12	2.7	2.7075	2.68	2.68	32528	{}	\N	2020-10-12 14:54:55.959765
43029	FIBK	2020-09-28	31.42	32.19	31.42	31.91	273490	{}	\N	2020-10-12 14:54:57.006998
43030	FIBK	2020-09-29	31.62	31.95	31.305	31.79	219821	{}	\N	2020-10-12 14:54:57.012655
43031	FIBK	2020-09-30	32.03	32.5	31.78	31.85	234777	{}	\N	2020-10-12 14:54:57.018137
43032	FIBK	2020-10-01	31.91	32.345	31.46	32.16	219092	{}	\N	2020-10-12 14:54:57.022743
43033	FIBK	2020-10-02	31.49	33.3	31.49	33.03	179184	{}	\N	2020-10-12 14:54:57.028012
43034	FIBK	2020-10-05	33.3	34.2	33.3	34	205577	{}	\N	2020-10-12 14:54:57.032622
43035	FIBK	2020-10-06	34.5	36	34.27	34.87	303540	{}	\N	2020-10-12 14:54:57.036584
43036	FIBK	2020-10-07	35	36.04	35	35.72	251001	{}	\N	2020-10-12 14:54:57.040688
43037	FIBK	2020-10-08	35.82	36.43	35.59	36.09	185370	{}	\N	2020-10-12 14:54:57.044959
43038	FIBK	2020-10-09	36.36	36.48	35.74	36.12	122950	{}	\N	2020-10-12 14:54:57.048525
43039	FIBK	2020-10-12	35.89	36.7	35.89	36.53	35444	{}	\N	2020-10-12 14:54:57.052338
43040	FIEE	2020-09-28	128.62	129.3863	128.62	129.3863	233	{}	\N	2020-10-12 14:54:58.184934
43041	FIEE	2020-09-29	128.7054	128.7054	128.7054	128.7054	127	{}	\N	2020-10-12 14:54:58.191316
43042	FIEE	2020-09-30	126.593	126.593	126.593	126.593	0	{}	\N	2020-10-12 14:54:58.196316
43043	FIEE	2020-10-01	126.23	127.0114	126.23	127.0114	317	{}	\N	2020-10-12 14:54:58.20042
43044	FIEE	2020-10-02	127.4874	127.4874	127.4874	127.4874	2	{}	\N	2020-10-12 14:54:58.204324
43045	FIEE	2020-10-05	129.71	130.7187	129.71	130.7187	140	{}	\N	2020-10-12 14:54:58.208737
43046	FIEE	2020-10-06	127.4094	127.4094	127.4094	127.4094	4	{}	\N	2020-10-12 14:54:58.212397
43047	FIEE	2020-10-07	129.174	129.174	129.174	129.174	67	{}	\N	2020-10-12 14:54:58.215815
43048	FIEE	2020-10-08	131.0662	131.0662	131.0662	131.0662	0	{}	\N	2020-10-12 14:54:58.219166
43049	FIEE	2020-10-09	133.885	133.885	133.885	133.885	0	{}	\N	2020-10-12 14:54:58.222543
43050	ACGL	2020-09-28	28.96	29.72	28.88	29.26	1738377	{}	\N	2020-10-12 14:55:03.036374
43051	ACGL	2020-09-29	29.09	29.31	28.51	28.87	1196217	{}	\N	2020-10-12 14:55:03.042355
43052	ACGL	2020-09-30	29.2	29.72	29.01	29.25	2201278	{}	\N	2020-10-12 14:55:03.047481
43053	ACGL	2020-10-01	29.35	29.7	29.115	29.415	966679	{}	\N	2020-10-12 14:55:03.052655
43054	ACGL	2020-10-02	28.86	29.94	28.6	29.7	646000	{}	\N	2020-10-12 14:55:03.057086
43055	ACGL	2020-10-05	30.02	30.69	29.68	30.64	950861	{}	\N	2020-10-12 14:55:03.061445
43056	ACGL	2020-10-06	30.95	30.95	30.025	30.18	845477	{}	\N	2020-10-12 14:55:03.066659
43057	ACGL	2020-10-07	30.4	30.85	30.105	30.24	1240793	{}	\N	2020-10-12 14:55:03.0711
43058	ACGL	2020-10-08	30.39	31.51	30.39	31.5	1184426	{}	\N	2020-10-12 14:55:03.075031
43059	ACGL	2020-10-09	31.98	32	31.48	31.53	1677866	{}	\N	2020-10-12 14:55:03.078801
43060	ACGL	2020-10-12	31.33	31.65	31.04	31.555	452984	{}	\N	2020-10-12 14:55:03.082777
43061	ACMR	2020-09-28	68.81	69.97	66.83	68.32	263106	{}	\N	2020-10-12 14:55:04.002475
43062	ACMR	2020-09-29	68.3	70.06	67.5651	69.94	179149	{}	\N	2020-10-12 14:55:04.010828
43063	ACMR	2020-09-30	71.82	72.95	68.8	69.1	230724	{}	\N	2020-10-12 14:55:04.016654
43064	ACMR	2020-10-01	70.53	76.69	70	75.38	362650	{}	\N	2020-10-12 14:55:04.024914
43065	ACMR	2020-10-02	71.51	74.31	70.69	73	295335	{}	\N	2020-10-12 14:55:04.030991
43066	ACMR	2020-10-05	73.93	77.15	73.77	75.92	330468	{}	\N	2020-10-12 14:55:04.036339
43067	ACMR	2020-10-06	76	76.5	70.16	70.22	453328	{}	\N	2020-10-12 14:55:04.041152
43068	ACMR	2020-10-07	71.71	75.18	71.09	71.88	355812	{}	\N	2020-10-12 14:55:04.04552
43069	ACMR	2020-10-08	68.4	71.02	66.23	70.79	555862	{}	\N	2020-10-12 14:55:04.049707
43070	ACMR	2020-10-09	71.49	75.93	71.01	75.45	349782	{}	\N	2020-10-12 14:55:04.05413
43071	ACMR	2020-10-12	76.5	82.67	76.465	79.88	241816	{}	\N	2020-10-12 14:55:04.058154
43072	ACT	2020-09-28	25.83	26.25	25.83	26.25	1040	{}	\N	2020-10-12 14:55:07.082691
43073	ACT	2020-09-29	26.56	26.56	26.045	26.14	2847	{}	\N	2020-10-12 14:55:07.09059
43074	ACT	2020-09-30	26.13	26.2373	26.015	26.18	2342	{}	\N	2020-10-12 14:55:07.095752
43075	ACT	2020-10-01	26.06	26.09	26.03	26.031	1561	{}	\N	2020-10-12 14:55:07.100561
43076	ACT	2020-10-02	25.69	26.205	25.46	26.205	3905	{}	\N	2020-10-12 14:55:07.105257
43077	ACT	2020-10-05	26.43	26.715	26.43	26.715	1152	{}	\N	2020-10-12 14:55:07.109789
43078	ACT	2020-10-06	26.8142	26.939	26.5447	26.5447	2928	{}	\N	2020-10-12 14:55:07.113597
43079	ACT	2020-10-07	26.9	26.95	26.9	26.95	926	{}	\N	2020-10-12 14:55:07.117364
43080	ACT	2020-10-08	26.89	27.22	26.89	27.22	2407	{}	\N	2020-10-12 14:55:07.121185
43081	ACT	2020-10-09	27.42	27.68	27.42	27.5073	3143	{}	\N	2020-10-12 14:55:07.124898
43082	ADBE	2020-09-28	487.88	489.39	481.18	488.51	2699145	{}	\N	2020-10-12 14:55:08.278627
43083	ADBE	2020-09-29	489.94	493.45	484.17	489.33	1981547	{}	\N	2020-10-12 14:55:08.290909
43084	ADBE	2020-09-30	488.13	497.24	486.22	490.43	3279990	{}	\N	2020-10-12 14:55:08.299013
43085	ADBE	2020-10-01	497.4252	501.5	495.76	499.51	2139586	{}	\N	2020-10-12 14:55:08.304686
43086	ADBE	2020-10-02	487.99	496.78	478.64	478.99	2584427	{}	\N	2020-10-12 14:55:08.310417
43087	ADBE	2020-10-05	483.25	486.69	478.99	486.47	1934469	{}	\N	2020-10-12 14:55:08.315076
43088	ADBE	2020-10-06	485	491.51	477.5	478.98	1881106	{}	\N	2020-10-12 14:55:08.32024
43089	ADBE	2020-10-07	484.35	493.98	481.43	493.15	1960331	{}	\N	2020-10-12 14:55:08.325071
43090	ADBE	2020-10-08	499.05	499.489	487.56	490.84	1545592	{}	\N	2020-10-12 14:55:08.32977
43091	ADBE	2020-10-09	491.77	502.17	491.176	502.16	1741736	{}	\N	2020-10-12 14:55:08.333904
43092	ADBE	2020-10-12	508.26	509.1	499.54	505	745775	{}	\N	2020-10-12 14:55:08.338028
43093	ADNT	2020-09-28	16.5	17.73	16.5	17.16	1123362	{}	\N	2020-10-12 14:55:09.795284
43094	ADNT	2020-09-29	17.21	17.29	16.91	17.13	1077355	{}	\N	2020-10-12 14:55:09.803663
43095	ADNT	2020-09-30	17.16	17.67	17.04	17.33	1254791	{}	\N	2020-10-12 14:55:09.810016
43096	ADNT	2020-10-01	17.58	19.58	17.58	19.41	6757252	{}	\N	2020-10-12 14:55:09.815159
43097	ADNT	2020-10-02	18.8	19.97	18.55	19.76	2358422	{}	\N	2020-10-12 14:55:09.820241
43098	ADNT	2020-10-05	20	21.54	20	21.33	4505129	{}	\N	2020-10-12 14:55:09.825234
43099	ADNT	2020-10-06	22	22.3	20.93	21.06	2597197	{}	\N	2020-10-12 14:55:09.829364
43100	ADNT	2020-10-07	21.44	22.14	21.39	21.88	2103118	{}	\N	2020-10-12 14:55:09.833915
43101	ADNT	2020-10-08	22.3	22.68	21.61	22.11	2340170	{}	\N	2020-10-12 14:55:09.838314
43102	ADNT	2020-10-09	22.32	22.48	21.61	21.71	1335128	{}	\N	2020-10-12 14:55:09.842803
43103	ADNT	2020-10-12	21.835	22.125	21.5013	21.86	158551	{}	\N	2020-10-12 14:55:09.847003
43104	ADUS	2020-09-28	94.12	95.7	93.02	93.67	72487	{}	\N	2020-10-12 14:55:11.083537
43105	ADUS	2020-09-29	93.66	94.33	91.9	93.37	73596	{}	\N	2020-10-12 14:55:11.093214
43106	ADUS	2020-09-30	93.07	95.45	93.07	94.51	92365	{}	\N	2020-10-12 14:55:11.098967
43107	ADUS	2020-10-01	95.31	97.75	94.2	97.63	105492	{}	\N	2020-10-12 14:55:11.103825
43108	ADUS	2020-10-02	96.08	97.69	94.2	95.05	75710	{}	\N	2020-10-12 14:55:11.108579
43109	ADUS	2020-10-05	95.98	98.04	95.575	97.42	58798	{}	\N	2020-10-12 14:55:11.113076
43110	ADUS	2020-10-06	98.08	98.53	96.34	96.67	76048	{}	\N	2020-10-12 14:55:11.117075
43111	ADUS	2020-10-07	97.57	97.79	96.34	96.46	125532	{}	\N	2020-10-12 14:55:11.121239
43112	ADUS	2020-10-08	97.46	98.65	96.03	96.22	83417	{}	\N	2020-10-12 14:55:11.12532
43113	ADUS	2020-10-09	97.04	98.25	95.63	96.55	193260	{}	\N	2020-10-12 14:55:11.129661
43114	ADUS	2020-10-12	97.42	97.9	97.22	97.8	4694	{}	\N	2020-10-12 14:55:11.133945
43115	AFT	2020-09-28	12.8	12.84	12.7866	12.83	72222	{}	\N	2020-10-12 14:55:12.962586
43116	AFT	2020-09-29	12.8	12.88	12.78	12.88	131808	{}	\N	2020-10-12 14:55:12.971141
43117	AFT	2020-09-30	12.82	12.88	12.81	12.82	189678	{}	\N	2020-10-12 14:55:12.975883
43118	AFT	2020-10-01	12.78	12.95	12.78	12.91	155633	{}	\N	2020-10-12 14:55:12.980441
43119	AFT	2020-10-02	12.82	12.91	12.81	12.91	68328	{}	\N	2020-10-12 14:55:12.984395
43120	AFT	2020-10-05	12.91	13.06	12.91	13.04	62598	{}	\N	2020-10-12 14:55:12.988125
43121	AFT	2020-10-06	13.06	13.07	12.98	13.07	63484	{}	\N	2020-10-12 14:55:12.991871
43122	AFT	2020-10-07	13.12	13.12	13.07	13.11	96815	{}	\N	2020-10-12 14:55:12.995453
43123	AFT	2020-10-08	13.09	13.2	13.09	13.18	97538	{}	\N	2020-10-12 14:55:12.999154
43124	AFT	2020-10-09	13.16	13.25	13.15	13.19	82209	{}	\N	2020-10-12 14:55:13.002855
43125	AFT	2020-10-12	13.25	13.31	13.24	13.25	26624	{}	\N	2020-10-12 14:55:13.006644
43126	AGR	2020-09-28	49.62	49.81	49.2	49.35	467590	{}	\N	2020-10-12 14:55:14.714289
43127	AGR	2020-09-29	49.42	49.85	48.97	49.26	871721	{}	\N	2020-10-12 14:55:14.725723
43128	AGR	2020-09-30	49.54	50.7	49.445	50.46	1786671	{}	\N	2020-10-12 14:55:14.732771
43129	AGR	2020-10-01	50.61	51.69	50.37	51.21	1145169	{}	\N	2020-10-12 14:55:14.738468
43130	AGR	2020-10-02	50.78	52.625	50.5026	52.38	1196047	{}	\N	2020-10-12 14:55:14.743601
43131	AGR	2020-10-05	52.71	53.665	52.36	53.59	1181807	{}	\N	2020-10-12 14:55:14.748027
43132	AGR	2020-10-06	53.56	54.38	52.835	53.83	858874	{}	\N	2020-10-12 14:55:14.752423
43133	AGR	2020-10-07	54.25	54.89	54.09	54.69	732066	{}	\N	2020-10-12 14:55:14.756703
43134	AGR	2020-10-08	54.79	56.2	54.62	55.63	920228	{}	\N	2020-10-12 14:55:14.76106
43135	AGR	2020-10-09	55.74	55.91	54.9	55.23	1127231	{}	\N	2020-10-12 14:55:14.765396
43136	AGR	2020-10-12	55.25	55.71	55.03	55.67	74335	{}	\N	2020-10-12 14:55:14.770115
43137	AGZD	2020-09-28	46.93	46.93	46.8	46.8312	3352	{}	\N	2020-10-12 14:55:17.192066
43138	AGZD	2020-09-29	46.826	46.868	46.76	46.76	7528	{}	\N	2020-10-12 14:55:17.198187
43139	AGZD	2020-09-30	46.7538	46.8	46.68	46.79	11689	{}	\N	2020-10-12 14:55:17.203056
43140	AGZD	2020-10-01	46.81	46.97	46.76	46.88	41860	{}	\N	2020-10-12 14:55:17.207498
43141	AGZD	2020-10-02	46.8999	46.8999	46.82	46.89	2837	{}	\N	2020-10-12 14:55:17.211925
43142	AGZD	2020-10-05	46.8	46.851	46.665	46.83	5601	{}	\N	2020-10-12 14:55:17.21672
43143	AGZD	2020-10-06	46.89	47.0099	46.85	46.87	7964	{}	\N	2020-10-12 14:55:17.220994
43144	AGZD	2020-10-07	46.9434	47.05	46.94	47.05	4554	{}	\N	2020-10-12 14:55:17.22537
43145	AGZD	2020-10-08	46.9742	46.9934	46.8901	46.9934	4733	{}	\N	2020-10-12 14:55:17.229369
43146	AGZD	2020-10-09	46.86	46.96	46.84	46.84	2737	{}	\N	2020-10-12 14:55:17.233438
43147	AGZD	2020-10-12	47.0599	47.0599	46.86	46.95	1069	{}	\N	2020-10-12 14:55:17.23765
43148	AIF	2020-09-28	12.695	12.71	12.63	12.63	32653	{}	\N	2020-10-12 14:55:18.329245
43149	AIF	2020-09-29	12.67	12.77	12.6	12.77	107636	{}	\N	2020-10-12 14:55:18.336108
43150	AIF	2020-09-30	12.74	12.77	12.64	12.71	139663	{}	\N	2020-10-12 14:55:18.34155
43151	AIF	2020-10-01	12.7	12.8	12.7	12.74	129911	{}	\N	2020-10-12 14:55:18.346502
43152	AIF	2020-10-02	12.71	12.73	12.67	12.73	47672	{}	\N	2020-10-12 14:55:18.351536
43153	AIF	2020-10-05	12.75	12.91	12.75	12.91	106156	{}	\N	2020-10-12 14:55:18.356241
43154	AIF	2020-10-06	12.86	12.95	12.86	12.9	69286	{}	\N	2020-10-12 14:55:18.360698
43155	AIF	2020-10-07	12.96	13.04	12.96	13.04	63484	{}	\N	2020-10-12 14:55:18.365684
43156	AIF	2020-10-08	13.04	13.14	12.9984	13.14	98226	{}	\N	2020-10-12 14:55:18.370287
43157	AIF	2020-10-09	13.08	13.1306	13.08	13.1	57184	{}	\N	2020-10-12 14:55:18.3747
43158	AIF	2020-10-12	13.13	13.13	13.07	13.08	5234	{}	\N	2020-10-12 14:55:18.379201
43159	FIVE	2020-09-28	131.33	131.7447	128.06	128.33	567989	{}	\N	2020-10-12 14:55:19.403658
43160	FIVE	2020-09-29	128.98	129.58	123.45	124.66	778219	{}	\N	2020-10-12 14:55:19.411262
43161	FIVE	2020-09-30	126.89	128.59	125.67	127	709591	{}	\N	2020-10-12 14:55:19.416313
43162	FIVE	2020-10-01	127.63	130.04	127.58	129.13	661563	{}	\N	2020-10-12 14:55:19.420955
43163	FIVE	2020-10-02	126.001	130.85	125.31	129.42	610560	{}	\N	2020-10-12 14:55:19.424833
43164	FIVE	2020-10-05	130.32	131.49	128.4	130.04	777325	{}	\N	2020-10-12 14:55:19.428722
43165	FIVE	2020-10-06	131.11	131.35	124.48	125.35	968614	{}	\N	2020-10-12 14:55:19.432633
43166	FIVE	2020-10-07	127.4	130	127.4	128.26	554014	{}	\N	2020-10-12 14:55:19.437265
43167	FIVE	2020-10-08	128.82	132.5	127.57	132.46	419661	{}	\N	2020-10-12 14:55:19.441169
43168	FIVE	2020-10-09	133.98	136.285	133.01	135.78	551311	{}	\N	2020-10-12 14:55:19.445069
43169	FIVE	2020-10-12	135.88	136.3	133.47	133.66	175994	{}	\N	2020-10-12 14:55:19.449056
43170	FLAU	2020-09-28	23.64	23.64	23.55	23.6	3570	{}	\N	2020-10-12 14:55:21.372479
43171	FLAU	2020-09-29	23.45	23.45	23.3978	23.3978	1311	{}	\N	2020-10-12 14:55:21.379114
43172	FLAU	2020-09-30	23.38	23.39	23.23	23.23	4399	{}	\N	2020-10-12 14:55:21.384102
43173	FLAU	2020-10-01	23.4001	23.45	23.3652	23.39	2178	{}	\N	2020-10-12 14:55:21.388726
43174	FLAU	2020-10-02	23.3682	23.3685	23.3682	23.3685	353	{}	\N	2020-10-12 14:55:21.393556
43175	FLAU	2020-10-05	23.8703	23.8703	23.8703	23.8703	168	{}	\N	2020-10-12 14:55:21.39812
43176	FLAU	2020-10-06	23.93	23.93	23.58	23.59	4776	{}	\N	2020-10-12 14:55:21.402805
43177	FLAU	2020-10-07	24.021	24.0514	24.021	24.0514	174	{}	\N	2020-10-12 14:55:21.407805
43178	FLAU	2020-10-08	24.225	24.3699	24.225	24.35	1446	{}	\N	2020-10-12 14:55:21.412404
43179	FLAU	2020-10-09	24.5	24.59	24.48	24.55	1547	{}	\N	2020-10-12 14:55:21.416746
43180	FLAU	2020-10-12	24.58	24.67	24.58	24.67	324	{}	\N	2020-10-12 14:55:21.421271
43181	FLEX	2020-09-28	10.77	11.055	10.7184	10.93	2740644	{}	\N	2020-10-12 14:55:22.401239
43182	FLEX	2020-09-29	10.9	11	10.855	10.89	1717029	{}	\N	2020-10-12 14:55:22.407457
43183	FLEX	2020-09-30	10.92	11.16	10.91	11.14	2619899	{}	\N	2020-10-12 14:55:22.412476
43184	FLEX	2020-10-01	11.25	11.49	11.23	11.38	3664503	{}	\N	2020-10-12 14:55:22.416705
43185	FLEX	2020-10-02	11.09	11.32	11.0521	11.17	1835316	{}	\N	2020-10-12 14:55:22.421004
43186	FLEX	2020-10-05	11.34	11.65	11.34	11.61	2927322	{}	\N	2020-10-12 14:55:22.425042
43187	FLEX	2020-10-06	11.65	12.15	11.56	11.79	2473487	{}	\N	2020-10-12 14:55:22.429107
43188	FLEX	2020-10-07	11.95	12.19	11.83	12	2285238	{}	\N	2020-10-12 14:55:22.432855
43189	FLEX	2020-10-08	12.09	12.26	12.015	12.19	1301219	{}	\N	2020-10-12 14:55:22.436919
43190	FLEX	2020-10-09	12.36	12.44	12.12	12.14	1088917	{}	\N	2020-10-12 14:55:22.441383
43191	FLEX	2020-10-12	12.28	12.34	12.17	12.33	643858	{}	\N	2020-10-12 14:55:22.445525
43192	FLIC	2020-09-28	14.83	15.05	14.71	14.75	50933	{}	\N	2020-10-12 14:55:23.412499
43193	FLIC	2020-09-29	15.46	15.46	14.41	14.87	44680	{}	\N	2020-10-12 14:55:23.420383
43194	FLIC	2020-09-30	15.01	15.16	14.68	14.81	42208	{}	\N	2020-10-12 14:55:23.425394
43195	FLIC	2020-10-01	14.79	14.931	14.56	14.83	37005	{}	\N	2020-10-12 14:55:23.430368
43196	FLIC	2020-10-02	14.65	15.4	14.65	15.14	74838	{}	\N	2020-10-12 14:55:23.434978
43197	FLIC	2020-10-05	15.23	15.84	15.23	15.82	70409	{}	\N	2020-10-12 14:55:23.439314
43198	FLIC	2020-10-06	15.95	16.68	15.83	16.14	57193	{}	\N	2020-10-12 14:55:23.443731
43199	FLIC	2020-10-07	16.21	16.89	16.13	16.83	91239	{}	\N	2020-10-12 14:55:23.447889
43200	FLIC	2020-10-08	16.85	16.8655	16.53	16.74	63768	{}	\N	2020-10-12 14:55:23.452518
43201	FLIC	2020-10-09	16.71	16.71	16.11	16.265	41443	{}	\N	2020-10-12 14:55:23.457068
43202	FLIC	2020-10-12	16.33	16.33	16.1	16.2	3834	{}	\N	2020-10-12 14:55:23.461179
43203	FLO	2020-09-28	23.81	24.115	23.81	24.02	961809	{}	\N	2020-10-12 14:55:26.140928
43204	FLO	2020-09-29	24.01	24.2	23.92	24.1	965679	{}	\N	2020-10-12 14:55:26.147335
43205	FLO	2020-09-30	24.21	24.4995	24.03	24.33	2630758	{}	\N	2020-10-12 14:55:26.15357
43206	FLO	2020-10-01	24.38	24.4716	24.18	24.42	1292905	{}	\N	2020-10-12 14:55:26.160896
43207	FLO	2020-10-02	24.34	24.705	24.27	24.63	1208753	{}	\N	2020-10-12 14:55:26.168182
43208	FLO	2020-10-05	24.75	24.985	24.58	24.92	1131677	{}	\N	2020-10-12 14:55:26.174349
43209	FLO	2020-10-06	24.8	24.93	24.532	24.63	1471729	{}	\N	2020-10-12 14:55:26.179896
43210	FLO	2020-10-07	24.64	24.75	24.34	24.59	1626157	{}	\N	2020-10-12 14:55:26.184834
43211	FLO	2020-10-08	24.7	24.74	24.43	24.51	1445968	{}	\N	2020-10-12 14:55:26.190512
43212	FLO	2020-10-09	24.51	24.83	24.48	24.69	1225867	{}	\N	2020-10-12 14:55:26.196561
43213	FLO	2020-10-12	24.76	24.99	24.73	24.965	131111	{}	\N	2020-10-12 14:55:26.202871
43214	FLQL	2020-09-28	34.47	34.65	34.42	34.53	22553	{}	\N	2020-10-12 14:55:27.403703
43215	FLQL	2020-09-29	34.5	34.68	34.42	34.47	15517	{}	\N	2020-10-12 14:55:27.410986
43216	FLQL	2020-09-30	34.47	34.941	34.47	34.76	199264	{}	\N	2020-10-12 14:55:27.415597
43217	FLQL	2020-10-01	34.83	35	34.7298	34.8301	19262	{}	\N	2020-10-12 14:55:27.419704
43218	FLQL	2020-10-02	34.6	34.82	34.47	34.66	19301	{}	\N	2020-10-12 14:55:27.425599
43219	FLQL	2020-10-05	34.85	35.2361	34.85	35.19	86418	{}	\N	2020-10-12 14:55:27.429631
43220	FLQL	2020-10-06	35.2	35.5	34.8406	34.9	26592	{}	\N	2020-10-12 14:55:27.434119
43221	FLQL	2020-10-07	35.27	35.4862	35.2	35.42	24550	{}	\N	2020-10-12 14:55:27.438629
43222	FLQL	2020-10-08	35.54	35.7	35.54	35.7	25556	{}	\N	2020-10-12 14:55:27.444114
43223	FLQL	2020-10-09	35.96	36.02	35.85	35.94	11328	{}	\N	2020-10-12 14:55:27.449564
43224	FLQL	2020-10-12	36.24	36.24	36.11	36.2	3658	{}	\N	2020-10-12 14:55:27.453908
43225	FLTW	2020-09-28	30.82	30.835	30.78	30.79	1884	{}	\N	2020-10-12 14:55:28.569532
43226	FLTW	2020-09-29	30.795	30.98	30.7702	30.96	2901	{}	\N	2020-10-12 14:55:28.581092
43227	FLTW	2020-09-30	31.2812	31.2812	31.2812	31.2812	151	{}	\N	2020-10-12 14:55:28.588316
43228	FLTW	2020-10-01	31.55	31.6613	31.52	31.6613	1651	{}	\N	2020-10-12 14:55:28.594686
43229	FLTW	2020-10-02	31.32	31.41	31.2405	31.29	1468	{}	\N	2020-10-12 14:55:28.600442
43230	FLTW	2020-10-05	31.39	31.5843	31.39	31.5843	2568	{}	\N	2020-10-12 14:55:28.605616
43231	FLTW	2020-10-06	31.64	31.7301	31.5563	31.5563	1059	{}	\N	2020-10-12 14:55:28.610199
43232	FLTW	2020-10-07	32.04	32.1427	32.04	32.1427	1118	{}	\N	2020-10-12 14:55:28.614332
43233	FLTW	2020-10-08	32.41	32.41	32.2714	32.2714	1480	{}	\N	2020-10-12 14:55:28.618235
43234	FLTW	2020-10-09	32.54	32.5597	32.54	32.5597	225	{}	\N	2020-10-12 14:55:28.622558
43235	FLTW	2020-10-12	32.49	32.49	32.49	32.49	281	{}	\N	2020-10-12 14:55:28.62672
43236	FMBH	2020-09-28	23.03	23.99	23.02	23.76	18313	{}	\N	2020-10-12 14:55:30.561511
43237	FMBH	2020-09-29	24.18	25.07	24.17	24.93	30424	{}	\N	2020-10-12 14:55:30.571066
43238	FMBH	2020-09-30	24.93	25.34	24.73	24.95	33456	{}	\N	2020-10-12 14:55:30.577497
43239	FMBH	2020-10-01	24.91	25.355	24.43	25.09	24185	{}	\N	2020-10-12 14:55:30.583526
43240	FMBH	2020-10-02	24.965	25.82	24.6	25.59	11060	{}	\N	2020-10-12 14:55:30.588342
43241	FMBH	2020-10-05	25.79	26.67	25.79	26.47	14494	{}	\N	2020-10-12 14:55:30.59304
43242	FMBH	2020-10-06	26.64	28.38	26.205	26.74	19834	{}	\N	2020-10-12 14:55:30.59729
43243	FMBH	2020-10-07	27.26	28.41	27.1	27.93	25647	{}	\N	2020-10-12 14:55:30.601306
43244	FMBH	2020-10-08	28	28.36	27.255	28	21399	{}	\N	2020-10-12 14:55:30.60619
43245	FMBH	2020-10-09	28.39	28.45	26.7706	28.05	13925	{}	\N	2020-10-12 14:55:30.610946
43246	FMBH	2020-10-12	28.09	28.24	28.02	28.24	1580	{}	\N	2020-10-12 14:55:30.615278
43247	FMO	2020-09-28	7.15	7.32	7.12	7.31	26605	{}	\N	2020-10-12 14:55:31.616949
43248	FMO	2020-09-29	7.25	7.36	7.18	7.29	65187	{}	\N	2020-10-12 14:55:31.628159
43249	FMO	2020-09-30	7.32	7.43	7.15	7.2	69625	{}	\N	2020-10-12 14:55:31.635459
43250	FMO	2020-10-01	7.2	7.28	7	7.05	104774	{}	\N	2020-10-12 14:55:31.641493
43251	FMO	2020-10-02	6.82	7.16	6.82	7.12	61589	{}	\N	2020-10-12 14:55:31.646441
43252	FMO	2020-10-05	7.23	7.6	7.2	7.6	88952	{}	\N	2020-10-12 14:55:31.65133
43253	FMO	2020-10-06	7.67	7.7	7.33	7.36	241574	{}	\N	2020-10-12 14:55:31.65546
43254	FMO	2020-10-07	7.37	7.4999	7.37	7.45	86076	{}	\N	2020-10-12 14:55:31.659819
43255	FMO	2020-10-08	7.48	7.9	7.44	7.87	101380	{}	\N	2020-10-12 14:55:31.66422
43256	FMO	2020-10-09	7.94	7.96	7.8	7.92	62444	{}	\N	2020-10-12 14:55:31.668364
43257	FMO	2020-10-12	7.92	7.93	7.8001	7.87	14430	{}	\N	2020-10-12 14:55:31.672454
43258	FNDE	2020-09-28	23.69	23.77	23.54	23.61	605501	{}	\N	2020-10-12 14:55:33.028615
43259	FNDE	2020-09-29	23.5034	23.59	23.4263	23.51	437532	{}	\N	2020-10-12 14:55:33.038594
43260	FNDE	2020-09-30	23.62	23.83	23.62	23.72	600791	{}	\N	2020-10-12 14:55:33.044827
43261	FNDE	2020-10-01	23.88	23.96	23.77	23.92	482977	{}	\N	2020-10-12 14:55:33.051503
43262	FNDE	2020-10-02	23.61	23.85	23.61	23.67	630828	{}	\N	2020-10-12 14:55:33.056137
43263	FNDE	2020-10-05	23.75	23.98	23.75	23.96	410609	{}	\N	2020-10-12 14:55:33.060693
43264	FNDE	2020-10-06	24.07	24.16	23.87	23.9	645409	{}	\N	2020-10-12 14:55:33.0644
43265	FNDE	2020-10-07	24.04	24.12	23.965	24.05	511646	{}	\N	2020-10-12 14:55:33.068075
43266	FNDE	2020-10-08	24.13	24.3162	24.09	24.29	494671	{}	\N	2020-10-12 14:55:33.071911
43267	FNDE	2020-10-09	24.32	24.49	24.27	24.39	464018	{}	\N	2020-10-12 14:55:33.075668
43268	FNDE	2020-10-12	24.52	24.55	24.4642	24.55	81503	{}	\N	2020-10-12 14:55:33.079347
43269	FNGU	2020-09-28	183.48	183.48	175.8138	182.48	549438	{}	\N	2020-10-12 14:55:34.078197
43270	FNGU	2020-09-29	180.6083	185.8	179.66	183.3	407607	{}	\N	2020-10-12 14:55:34.087063
43271	FNGU	2020-09-30	184.87	196.44	183.5853	192.07	678347	{}	\N	2020-10-12 14:55:34.093694
43272	FNGU	2020-10-01	199.61	203.8487	194.9	202.64	542191	{}	\N	2020-10-12 14:55:34.099537
43273	FNGU	2020-10-02	187	198.68	184.51	185.58	1013446	{}	\N	2020-10-12 14:55:34.10591
43274	FNGU	2020-10-05	190.02	197.7299	188.7901	197.31	586159	{}	\N	2020-10-12 14:55:34.110771
43275	FNGU	2020-10-06	196.0065	199.7336	183.51	187.01	848035	{}	\N	2020-10-12 14:55:34.115484
43276	FNGU	2020-10-07	193.8	198.5895	191.165	197.24	534555	{}	\N	2020-10-12 14:55:34.120013
43277	FNGU	2020-10-08	202.53	202.98	197.778	199.88	343522	{}	\N	2020-10-12 14:55:34.124427
43278	FNGU	2020-10-09	201.75	204.9	200	204.12	351073	{}	\N	2020-10-12 14:55:34.12899
43279	FNGU	2020-10-12	217.46	221.4862	214.05	221.335	306296	{}	\N	2020-10-12 14:55:34.133409
43280	FL	2020-09-28	33.33	33.79	32.34	32.39	1641481	{}	\N	2020-10-12 14:55:35.097009
43281	FL	2020-09-29	32.41	32.41	31.41	32.11	3758350	{}	\N	2020-10-12 14:55:35.106541
43282	FL	2020-09-30	32.25	33.7	32.25	33.03	5024713	{}	\N	2020-10-12 14:55:35.112869
43283	FL	2020-10-01	33.1	34.25	33.1	34.11	3839633	{}	\N	2020-10-12 14:55:35.118989
43284	FL	2020-10-02	33.41	35.8	33.23	35.63	4077538	{}	\N	2020-10-12 14:55:35.124284
43285	FL	2020-10-05	35.98	37.28	35.9	37.2	5708354	{}	\N	2020-10-12 14:55:35.129067
43286	FL	2020-10-06	37.45	38.315	36.48	36.55	5742369	{}	\N	2020-10-12 14:55:35.133484
43287	FL	2020-10-07	37.72	38.78	37.34	37.54	4586014	{}	\N	2020-10-12 14:55:35.137858
43288	FL	2020-10-08	37.49	38.42	36.88	38.34	4059228	{}	\N	2020-10-12 14:55:35.142077
43289	FL	2020-10-09	38.42	38.79	37.72	38.03	3494586	{}	\N	2020-10-12 14:55:35.146374
43290	FL	2020-10-12	38.13	38.635	37.995	38.49	447832	{}	\N	2020-10-12 14:55:35.150488
43291	FLCA	2020-09-28	26	26	24.51	24.64	3169	{}	\N	2020-10-12 14:55:36.571257
43292	FLCA	2020-09-29	24.63	24.65	24.5649	24.5649	3782	{}	\N	2020-10-12 14:55:36.582522
43293	FLCA	2020-09-30	24.5289	24.5289	24.5289	24.5289	127	{}	\N	2020-10-12 14:55:36.5895
43294	FLCA	2020-10-01	24.6297	24.6297	24.6297	24.6297	213	{}	\N	2020-10-12 14:55:36.595566
43295	FLCA	2020-10-02	24.63	24.69	24.61	24.69	3688	{}	\N	2020-10-12 14:55:36.600733
43296	FLCA	2020-10-05	24.77	25.0799	24.77	25.0799	977	{}	\N	2020-10-12 14:55:36.606045
43297	FLCA	2020-10-06	25.0744	25.0744	24.7781	24.7781	2921	{}	\N	2020-10-12 14:55:36.61054
43298	FLCA	2020-10-07	25.1528	25.1528	25.1528	25.1528	39	{}	\N	2020-10-12 14:55:36.61545
43299	FLCA	2020-10-08	25.42	25.44	25.4077	25.4077	5172	{}	\N	2020-10-12 14:55:36.620199
43300	FLCA	2020-10-09	25.52	25.615	25.52	25.5537	3075	{}	\N	2020-10-12 14:55:36.625166
43301	FLCA	2020-10-12	25.64	25.8534	25.61	25.8534	342	{}	\N	2020-10-12 14:55:36.629485
43302	FLHK	2020-09-28	22.94	22.94	22.94	22.94	122	{}	\N	2020-10-12 14:55:38.739488
43303	FLHK	2020-09-29	22.9	22.915	22.9	22.915	399	{}	\N	2020-10-12 14:55:38.74718
43304	FLHK	2020-09-30	22.995	22.995	22.995	22.995	21	{}	\N	2020-10-12 14:55:38.753311
43305	FLHK	2020-10-01	23.13	23.195	23.13	23.195	301	{}	\N	2020-10-12 14:55:38.758441
43306	FLHK	2020-10-02	23.1986	23.1986	23.1986	23.1986	0	{}	\N	2020-10-12 14:55:38.763849
43307	FLHK	2020-10-05	23.31	23.329	23.31	23.31	330	{}	\N	2020-10-12 14:55:38.76846
43308	FLHK	2020-10-06	23.47	23.51	23.42	23.42	1569	{}	\N	2020-10-12 14:55:38.774462
43309	FLHK	2020-10-07	23.57	23.61	23.57	23.61	506	{}	\N	2020-10-12 14:55:38.779341
43310	FLHK	2020-10-08	23.63	23.63	23.63	23.63	12	{}	\N	2020-10-12 14:55:38.78463
43311	FLHK	2020-10-09	23.55	23.55	23.43	23.43	1045	{}	\N	2020-10-12 14:55:38.790405
43312	FLKR	2020-09-28	22.79	22.79	22.7101	22.765	6543	{}	\N	2020-10-12 14:55:39.844568
43313	FLKR	2020-09-29	22.87	22.92	22.76	22.8513	6561	{}	\N	2020-10-12 14:55:39.853685
43314	FLKR	2020-09-30	23.1259	23.22	23.12	23.12	1898	{}	\N	2020-10-12 14:55:39.860644
43315	FLKR	2020-10-01	23.22	23.31	23.16	23.31	3454	{}	\N	2020-10-12 14:55:39.866362
43316	FLKR	2020-10-02	22.89	23.145	22.89	23.145	1666	{}	\N	2020-10-12 14:55:39.871447
43317	FLKR	2020-10-05	23.28	23.58	23.28	23.58	6556	{}	\N	2020-10-12 14:55:39.876401
43318	FLKR	2020-10-06	23.42	23.5	23.31	23.31	6307	{}	\N	2020-10-12 14:55:39.880111
43319	FLKR	2020-10-07	23.67	23.81	23.67	23.81	5177	{}	\N	2020-10-12 14:55:39.884215
43320	FLKR	2020-10-08	23.795	23.96	23.76	23.9299	6933	{}	\N	2020-10-12 14:55:39.888093
43321	FLKR	2020-10-09	24.1	24.17	24.0263	24.17	5474	{}	\N	2020-10-12 14:55:39.892311
43322	FLKR	2020-10-12	24	24	23.9577	23.9577	2511	{}	\N	2020-10-12 14:55:39.89603
43323	FLMI	2020-09-28	25.601	25.64	25.601	25.64	630	{}	\N	2020-10-12 14:55:40.994654
43324	FLMI	2020-09-29	25.68	25.68	25.63	25.63	200600	{}	\N	2020-10-12 14:55:41.003539
43325	FLMI	2020-09-30	25.575	25.575	25.575	25.575	4	{}	\N	2020-10-12 14:55:41.011311
43326	FLMI	2020-10-01	25.57	25.57	25.54	25.54	504	{}	\N	2020-10-12 14:55:41.017616
43327	FLMI	2020-10-02	25.56	25.56	25.56	25.56	4	{}	\N	2020-10-12 14:55:41.022927
43328	FLMI	2020-10-05	25.55	25.55	25.55	25.55	2	{}	\N	2020-10-12 14:55:41.027951
43329	FLMI	2020-10-06	25.59	25.59	25.57	25.57	4020	{}	\N	2020-10-12 14:55:41.033577
43330	FLMI	2020-10-07	25.49	25.5	25.48	25.5	834	{}	\N	2020-10-12 14:55:41.038252
43331	FLMI	2020-10-08	25.54	25.54	25.54	25.54	1	{}	\N	2020-10-12 14:55:41.042821
43332	FLMI	2020-10-09	25.511	25.55	25.51	25.55	7790	{}	\N	2020-10-12 14:55:41.04725
43333	FLMI	2020-10-12	25.59	25.59	25.59	25.59	315	{}	\N	2020-10-12 14:55:41.051475
43334	FLQH	2020-09-28	24.2617	24.3253	24.2617	24.3253	1809	{}	\N	2020-10-12 14:55:42.72824
43335	FLQH	2020-09-29	24.16	24.2789	24.16	24.23	1303	{}	\N	2020-10-12 14:55:42.736625
43336	FLQH	2020-09-30	24.19	24.19	24.142	24.142	3799	{}	\N	2020-10-12 14:55:42.742405
43337	FLQH	2020-10-01	24.05	24.18	24.05	24.135	4117	{}	\N	2020-10-12 14:55:42.747774
43338	FLQH	2020-10-02	24.06	24.2	24.06	24.2	2963	{}	\N	2020-10-12 14:55:42.753301
43339	FLQH	2020-10-05	24.29	24.365	24.26	24.365	2607	{}	\N	2020-10-12 14:55:42.757969
43340	FLQH	2020-10-06	24.37	24.37	24.2	24.2	7417	{}	\N	2020-10-12 14:55:42.762174
43341	FLQH	2020-10-07	24.42	24.42	24.341	24.3484	2919	{}	\N	2020-10-12 14:55:42.766357
43342	FLQH	2020-10-08	24.35	24.52	24.35	24.52	1473	{}	\N	2020-10-12 14:55:42.770511
43343	FLQH	2020-10-09	24.615	24.615	24.615	24.615	142	{}	\N	2020-10-12 14:55:42.774616
43344	FLTR	2020-09-28	25.19	25.22	25.19	25.2	56144	{}	\N	2020-10-12 14:55:43.797342
43345	FLTR	2020-09-29	25.2	25.22	25.2	25.21	38078	{}	\N	2020-10-12 14:55:43.807967
43346	FLTR	2020-09-30	25.22	25.23	25.19	25.21	63437	{}	\N	2020-10-12 14:55:43.814707
43347	FLTR	2020-10-01	25.21	25.21	25.18	25.205	27894	{}	\N	2020-10-12 14:55:43.820428
43348	FLTR	2020-10-02	25.21	25.25	25.17	25.19	375615	{}	\N	2020-10-12 14:55:43.82539
43349	FLTR	2020-10-05	25.22	25.22	25.18	25.2	231055	{}	\N	2020-10-12 14:55:43.829769
43350	FLTR	2020-10-06	25.2	25.23	25.18	25.185	117427	{}	\N	2020-10-12 14:55:43.834234
43351	FLTR	2020-10-07	25.21	25.24	25.17	25.18	267200	{}	\N	2020-10-12 14:55:43.838382
43352	FLTR	2020-10-08	25.18	25.21	25.18	25.205	82091	{}	\N	2020-10-12 14:55:43.8424
43353	FLTR	2020-10-09	25.22	25.22	25.2	25.21	109281	{}	\N	2020-10-12 14:55:43.846544
43354	FLTR	2020-10-12	25.2	25.21	25.2	25.2003	10870	{}	\N	2020-10-12 14:55:43.850763
43355	FMB	2020-09-28	55.66	55.73	55.65	55.69	120093	{}	\N	2020-10-12 14:55:44.981982
43356	FMB	2020-09-29	55.7025	55.75	55.68	55.705	113704	{}	\N	2020-10-12 14:55:44.990758
43357	FMB	2020-09-30	55.7	55.7	55.5758	55.59	142652	{}	\N	2020-10-12 14:55:44.996645
43358	FMB	2020-10-01	55.62	55.64	55.55	55.58	294228	{}	\N	2020-10-12 14:55:45.005679
43359	FMB	2020-10-02	55.62	55.63	55.55	55.6	104215	{}	\N	2020-10-12 14:55:45.011251
43360	FMB	2020-10-05	55.595	55.63	55.53	55.55	89414	{}	\N	2020-10-12 14:55:45.015839
43361	FMB	2020-10-06	55.5414	55.62	55.49	55.53	135252	{}	\N	2020-10-12 14:55:45.020399
43362	FMB	2020-10-07	55.41	55.5	55.3911	55.435	88029	{}	\N	2020-10-12 14:55:45.027203
43363	FMB	2020-10-08	55.44	55.44	55.38	55.38	75505	{}	\N	2020-10-12 14:55:45.032703
43364	FMB	2020-10-09	55.39	55.45	55.35	55.39	123871	{}	\N	2020-10-12 14:55:45.037504
43365	FMB	2020-10-12	55.44	55.4448	55.405	55.4412	17728	{}	\N	2020-10-12 14:55:45.04201
43366	FMK	2020-09-28	38.6136	38.6607	38.4343	38.62	5169	{}	\N	2020-10-12 14:55:46.496028
43367	FMK	2020-09-29	38.55	38.55	38.4155	38.4155	375	{}	\N	2020-10-12 14:55:46.502876
43368	FMK	2020-09-30	38.8001	39.0457	38.635	38.635	87861	{}	\N	2020-10-12 14:55:46.507888
43369	FMK	2020-10-01	38.7843	39.096	38.7843	39.096	3956	{}	\N	2020-10-12 14:55:46.512823
43370	FMK	2020-10-02	38.25	38.675	38.25	38.675	1988	{}	\N	2020-10-12 14:55:46.517253
43371	FMK	2020-10-05	39.115	39.235	39.1	39.235	1560	{}	\N	2020-10-12 14:55:46.521263
43372	FMK	2020-10-06	39.3887	39.3887	38.7347	38.7347	1048	{}	\N	2020-10-12 14:55:46.525234
43373	FMK	2020-10-07	39.19	39.398	39.17	39.398	5451	{}	\N	2020-10-12 14:55:46.529246
43374	FMK	2020-10-08	39.56	39.65	39.56	39.65	542	{}	\N	2020-10-12 14:55:46.533662
43375	FMK	2020-10-09	39.87	39.87	39.8282	39.8282	471	{}	\N	2020-10-12 14:55:46.538169
43376	FNDB	2020-09-28	37.92	38.1699	37.92	38.165	6962	{}	\N	2020-10-12 14:55:47.722543
43377	FNDB	2020-09-29	38.04	38.04	37.625	37.795	15533	{}	\N	2020-10-12 14:55:47.733018
43378	FNDB	2020-09-30	37.79	38.4	37.79	37.9944	16584	{}	\N	2020-10-12 14:55:47.739435
43379	FNDB	2020-10-01	38.3	38.315	37.99	38.21	10645	{}	\N	2020-10-12 14:55:47.744986
43380	FNDB	2020-10-02	37.98	38.3735	37.98	38.18	5632	{}	\N	2020-10-12 14:55:47.750302
43381	FNDB	2020-10-05	38.47	38.94	38.47	38.94	10369	{}	\N	2020-10-12 14:55:47.755212
43382	FNDB	2020-10-06	39.03	39.37	38.5	38.55	16921	{}	\N	2020-10-12 14:55:47.759196
43383	FNDB	2020-10-07	39	39.24	38.95	39.24	11440	{}	\N	2020-10-12 14:55:47.763309
43384	FNDB	2020-10-08	39.4694	39.6968	39.45	39.6968	7180	{}	\N	2020-10-12 14:55:47.7673
43385	FNDB	2020-10-09	40	40.02	39.79	39.8282	23734	{}	\N	2020-10-12 14:55:47.771497
43386	FNDB	2020-10-12	40.08	40.08	40.08	40.08	386	{}	\N	2020-10-12 14:55:47.775592
43387	FKU	2020-09-28	31.01	31.28	30.96	30.96	6077	{}	\N	2020-10-12 14:55:49.491026
43388	FKU	2020-09-29	30.9	30.96	30.72	30.85	3320	{}	\N	2020-10-12 14:55:49.498681
43389	FKU	2020-09-30	31.03	31.05	30.81	30.88	5411	{}	\N	2020-10-12 14:55:49.50324
43390	FKU	2020-10-01	30.91	31.1	30.8	30.955	15860	{}	\N	2020-10-12 14:55:49.507836
43391	FKU	2020-10-02	30.98	31.51	30.98	31.51	5634	{}	\N	2020-10-12 14:55:49.511922
43392	FKU	2020-10-05	31.92	32.02	31.87	32.02	744	{}	\N	2020-10-12 14:55:49.516301
43393	FKU	2020-10-06	31.92	31.935	31.63	31.63	2111	{}	\N	2020-10-12 14:55:49.519958
43394	FKU	2020-10-07	31.86	31.87	31.8	31.83	1255	{}	\N	2020-10-12 14:55:49.524306
43395	FKU	2020-10-08	32.28	32.33	32.26	32.33	2212	{}	\N	2020-10-12 14:55:49.528748
43396	FLCH	2020-09-28	28.34	28.37	28.28	28.37	12907	{}	\N	2020-10-12 14:55:50.618702
43397	FLCH	2020-09-29	28.36	28.46	28.31	28.41	10125	{}	\N	2020-10-12 14:55:50.625675
43398	FLCH	2020-09-30	28.76	29.0914	28.76	29.0914	15905	{}	\N	2020-10-12 14:55:50.630373
43399	FLCH	2020-10-01	29.38	29.38	29.15	29.2699	12422	{}	\N	2020-10-12 14:55:50.634635
43400	FLCH	2020-10-02	28.87	29.29	28.87	28.96	13088	{}	\N	2020-10-12 14:55:50.64017
43401	FLCH	2020-10-05	29.01	29.0928	28.954	29.0928	11525	{}	\N	2020-10-12 14:55:50.64467
43402	FLCH	2020-10-06	29.3	29.5	29.3	29.4	5513	{}	\N	2020-10-12 14:55:50.648916
43403	FLCH	2020-10-07	29.65	29.71	29.59	29.6575	5689	{}	\N	2020-10-12 14:55:50.654509
43404	FLCH	2020-10-08	29.74	29.887	29.74	29.8799	10790	{}	\N	2020-10-12 14:55:50.659433
43405	FLCH	2020-10-09	29.83	30.055	29.816	30.05	3227	{}	\N	2020-10-12 14:55:50.663917
43406	FLCH	2020-10-12	30.54	30.585	30.475	30.585	5192	{}	\N	2020-10-12 14:55:50.668139
43407	FLGR	2020-09-28	22.73	22.778	22.72	22.778	1876	{}	\N	2020-10-12 14:55:51.800871
43408	FLGR	2020-09-29	22.8999	22.8999	22.835	22.8899	1033	{}	\N	2020-10-12 14:55:51.811985
43409	FLGR	2020-09-30	22.8161	22.8161	22.7977	22.7977	2699	{}	\N	2020-10-12 14:55:51.818957
43410	FLGR	2020-10-01	22.8	22.815	22.8	22.815	509	{}	\N	2020-10-12 14:55:51.825027
43411	FLGR	2020-10-02	22.5	22.7277	22.5	22.7277	335	{}	\N	2020-10-12 14:55:51.830783
43412	FLGR	2020-10-05	23.2107	23.2107	23.2107	23.2107	36	{}	\N	2020-10-12 14:55:51.836028
43413	FLGR	2020-10-06	23.2396	23.2396	22.9763	22.9763	689	{}	\N	2020-10-12 14:55:51.841097
43414	FLGR	2020-10-07	23.1918	23.1918	23.1918	23.1918	232	{}	\N	2020-10-12 14:55:51.845995
43415	FLGR	2020-10-08	23.42	23.42	23.4165	23.4165	375	{}	\N	2020-10-12 14:55:51.85075
43416	FLGR	2020-10-09	23.51	23.5457	23.51	23.5457	617	{}	\N	2020-10-12 14:55:51.855495
43417	FLGR	2020-10-12	23.61	23.61	23.61	23.61	264	{}	\N	2020-10-12 14:55:51.860984
43418	FLOW	2020-09-28	42.06	43.96	42.06	43.38	132417	{}	\N	2020-10-12 14:55:53.730806
43419	FLOW	2020-09-29	43.27	43.48	42.49	42.92	189395	{}	\N	2020-10-12 14:55:53.738659
43420	FLOW	2020-09-30	43.05	43.685	42.245	42.82	524698	{}	\N	2020-10-12 14:55:53.744794
43421	FLOW	2020-10-01	42.78	43.43	41.88	42.35	284346	{}	\N	2020-10-12 14:55:53.749874
43422	FLOW	2020-10-02	41.42	43.92	41.42	43.54	436855	{}	\N	2020-10-12 14:55:53.754918
43423	FLOW	2020-10-05	43.97	44.98	43.52	44.27	723680	{}	\N	2020-10-12 14:55:53.759258
43424	FLOW	2020-10-06	44.56	46.06	44.05	45.12	517148	{}	\N	2020-10-12 14:55:53.76363
43425	FLOW	2020-10-07	45.67	46.09	44.83	45.6	355470	{}	\N	2020-10-12 14:55:53.768029
43426	FLOW	2020-10-08	46.26	46.46	45.18	45.77	341274	{}	\N	2020-10-12 14:55:53.772246
43427	FLOW	2020-10-09	46.32	46.63	45.52	45.61	230380	{}	\N	2020-10-12 14:55:53.776375
43428	FLOW	2020-10-12	45.64	45.855	45.25	45.3975	42174	{}	\N	2020-10-12 14:55:53.780645
43429	FLRN	2020-09-28	30.59	30.6	30.57	30.57	966898	{}	\N	2020-10-12 14:55:55.022394
43430	FLRN	2020-09-29	30.58	30.59	30.57	30.59	323310	{}	\N	2020-10-12 14:55:55.03351
43431	FLRN	2020-09-30	30.6	30.6	30.58	30.58	386754	{}	\N	2020-10-12 14:55:55.040569
43432	FLRN	2020-10-01	30.58	30.58	30.56	30.575	260192	{}	\N	2020-10-12 14:55:55.047021
43433	FLRN	2020-10-02	30.56	30.58	30.56	30.57	197755	{}	\N	2020-10-12 14:55:55.052903
43434	FLRN	2020-10-05	30.56	30.59	30.56	30.59	631751	{}	\N	2020-10-12 14:55:55.057838
43435	FLRN	2020-10-06	30.58	30.59	30.57	30.57	1119376	{}	\N	2020-10-12 14:55:55.062786
43436	FLRN	2020-10-07	30.56	30.57	30.56	30.57	510164	{}	\N	2020-10-12 14:55:55.067711
43437	FLRN	2020-10-08	30.57	30.58	30.56	30.57	361973	{}	\N	2020-10-12 14:55:55.072684
43438	FLRN	2020-10-09	30.57	30.59	30.57	30.58	397398	{}	\N	2020-10-12 14:55:55.078117
43439	FLRN	2020-10-12	30.57	30.59	30.57	30.59	37326	{}	\N	2020-10-12 14:55:55.083042
43440	FLY	2020-09-28	7.04	7.9799	6.72	7.64	1267465	{}	\N	2020-10-12 14:55:56.073193
43441	FLY	2020-09-29	7.87	7.9	7.1501	7.44	1627497	{}	\N	2020-10-12 14:55:56.080624
43442	FLY	2020-09-30	7.54	7.84	7.15	7.26	810143	{}	\N	2020-10-12 14:55:56.086518
43443	FLY	2020-10-01	7.29	7.86	7.29	7.82	639758	{}	\N	2020-10-12 14:55:56.094998
43444	FLY	2020-10-02	7.39	7.88	7.39	7.67	552573	{}	\N	2020-10-12 14:55:56.100001
43445	FLY	2020-10-05	7.74	7.96	7.54	7.57	407765	{}	\N	2020-10-12 14:55:56.1053
43446	FLY	2020-10-06	7.56	7.79	6.96	7.1	592137	{}	\N	2020-10-12 14:55:56.110617
43447	FLY	2020-10-07	7.32	7.52	7.18	7.27	383160	{}	\N	2020-10-12 14:55:56.11532
43448	FLY	2020-10-08	7.36	7.74	7.3	7.54	519416	{}	\N	2020-10-12 14:55:56.12028
43449	FLY	2020-10-09	7.5	7.61	7.3	7.39	398012	{}	\N	2020-10-12 14:55:56.124981
43450	FLY	2020-10-12	7.26	7.47	7.241	7.38	106210	{}	\N	2020-10-12 14:55:56.129627
43451	FMCIW	2020-09-28	9	9.7	8.81	8.99	1197325	{}	\N	2020-10-12 14:55:57.160126
43452	FMCIW	2020-09-29	9.3	9.77	9.3	9.5502	708501	{}	\N	2020-10-12 14:55:57.17125
43453	FMCIW	2020-09-30	9.64	9.64	8.32	8.87	788523	{}	\N	2020-10-12 14:55:57.178324
43454	FMCIW	2020-10-01	8.95	9.2	8.5	8.94	589122	{}	\N	2020-10-12 14:55:57.183993
43455	FMCIW	2020-10-02	8.33	8.7999	8.0702	8.3897	416327	{}	\N	2020-10-12 14:55:57.189373
43456	FMCIW	2020-10-05	8.52	8.92	8.12	8.49	585285	{}	\N	2020-10-12 14:55:57.194818
43457	FMCIW	2020-10-06	8.6	8.61	7.26	7.3	802999	{}	\N	2020-10-12 14:55:57.199148
43458	FMCIW	2020-10-07	7.75	7.89	7.185	7.46	559139	{}	\N	2020-10-12 14:55:57.203876
43459	FMCIW	2020-10-08	7.5	8.51	7.5	8.4701	651494	{}	\N	2020-10-12 14:55:57.207774
43460	FMCIW	2020-10-09	8.69	8.85	8.3101	8.8	488625	{}	\N	2020-10-12 14:55:57.211919
43461	FMCIW	2020-10-12	8.95	9.105	8.56	8.63	122852	{}	\N	2020-10-12 14:55:57.216473
43462	FMN	2020-09-28	14.5	14.55	14.4	14.55	18825	{}	\N	2020-10-12 14:55:58.172545
43463	FMN	2020-09-29	14.6	14.6	14.26	14.27	31408	{}	\N	2020-10-12 14:55:58.180794
43464	FMN	2020-09-30	14.36	14.36	14.11	14.27	28318	{}	\N	2020-10-12 14:55:58.186893
43465	FMN	2020-10-01	14.27	14.27	14.155	14.27	27018	{}	\N	2020-10-12 14:55:58.191783
43466	FMN	2020-10-02	14.36	14.6325	14.09	14.36	38852	{}	\N	2020-10-12 14:55:58.197297
43467	FMN	2020-10-05	14.42	14.65	13.95	14.12	25304	{}	\N	2020-10-12 14:55:58.202026
43468	FMN	2020-10-06	14.55	14.55	13.98	14.01	42770	{}	\N	2020-10-12 14:55:58.206057
43469	FMN	2020-10-07	14.05	14.2	14.05	14.05	22678	{}	\N	2020-10-12 14:55:58.210288
43470	FMN	2020-10-08	14.12	14.16	14.06	14.06	26422	{}	\N	2020-10-12 14:55:58.214648
43471	FMN	2020-10-09	14.07	14.17	14	14	29596	{}	\N	2020-10-12 14:55:58.218754
43472	FMN	2020-10-12	14.09	14.09	13.94	14.0399	4456	{}	\N	2020-10-12 14:55:58.22294
43473	FNDA	2020-09-28	32.89	33.4455	32.89	33.31	242729	{}	\N	2020-10-12 14:55:59.163002
43474	FNDA	2020-09-29	33.239	33.239	32.67	32.99	172130	{}	\N	2020-10-12 14:55:59.171305
43475	FNDA	2020-09-30	33.05	33.53	32.84	33.05	168681	{}	\N	2020-10-12 14:55:59.177001
43476	FNDA	2020-10-01	33.2	33.53	33.02	33.5	206189	{}	\N	2020-10-12 14:55:59.182097
43477	FNDA	2020-10-02	32.86	34.05	32.845	33.91	256821	{}	\N	2020-10-12 14:55:59.187504
43478	FNDA	2020-10-05	34.22	34.67	34.22	34.67	159897	{}	\N	2020-10-12 14:55:59.191855
43479	FNDA	2020-10-06	34.95	35.44	34.4401	34.46	125445	{}	\N	2020-10-12 14:55:59.196257
43480	FNDA	2020-10-07	34.82	35.21	34.82	35.09	144290	{}	\N	2020-10-12 14:55:59.200494
43481	FNDA	2020-10-08	35.45	35.68	35.34	35.67	176982	{}	\N	2020-10-12 14:55:59.20453
43482	FNDA	2020-10-09	35.93	35.99	35.6	35.71	194453	{}	\N	2020-10-12 14:55:59.208786
43483	FNDA	2020-10-12	35.8	35.9727	35.682	35.93	41530	{}	\N	2020-10-12 14:55:59.212815
43484	FNKO	2020-09-28	5.84	5.9	5.57	5.72	628006	{}	\N	2020-10-12 14:56:00.937079
43485	FNKO	2020-09-29	5.67	5.8	5.635	5.71	513138	{}	\N	2020-10-12 14:56:00.946602
43486	FNKO	2020-09-30	5.78	5.985	5.73	5.79	320006	{}	\N	2020-10-12 14:56:00.954202
43487	FNKO	2020-10-01	5.819	5.9565	5.81	5.86	233778	{}	\N	2020-10-12 14:56:00.959672
43488	FNKO	2020-10-02	5.67	5.75	5.52	5.74	573067	{}	\N	2020-10-12 14:56:00.964719
43489	FNKO	2020-10-05	5.81	6.05	5.76	5.91	652899	{}	\N	2020-10-12 14:56:00.97008
43490	FNKO	2020-10-06	5.94	6.24	5.86	5.96	566670	{}	\N	2020-10-12 14:56:00.974974
43491	FNKO	2020-10-07	6.08	6.25	6.05	6.12	316241	{}	\N	2020-10-12 14:56:00.979422
43492	FNKO	2020-10-08	6.19	6.2485	5.96	6.16	433233	{}	\N	2020-10-12 14:56:00.984116
43493	FNKO	2020-10-09	6.2	6.28	6.1601	6.19	246799	{}	\N	2020-10-12 14:56:00.989384
43494	FNKO	2020-10-12	6.29	6.29	6.182	6.19	70312	{}	\N	2020-10-12 14:56:00.99366
43495	FORK	2020-09-28	2.32	2.33	2.32	2.32	5433	{}	\N	2020-10-12 14:56:02.592094
43496	FORK	2020-09-29	2.33	2.35	2.32	2.3452	6609	{}	\N	2020-10-12 14:56:02.599536
43497	FORK	2020-09-30	2.31	2.34	2.31	2.33	7771	{}	\N	2020-10-12 14:56:02.605838
43498	FORK	2020-10-01	2.34	2.36	2.32	2.33	4775	{}	\N	2020-10-12 14:56:02.610873
43499	FORK	2020-10-02	2.31	2.34	2.31	2.34	3836	{}	\N	2020-10-12 14:56:02.615338
43500	FORK	2020-10-05	2.34	2.34	2.33	2.33	1180	{}	\N	2020-10-12 14:56:02.62054
43501	FORK	2020-10-06	2.32	2.33	2.32	2.33	3413	{}	\N	2020-10-12 14:56:02.625134
43502	FORK	2020-10-07	2.33	2.33	2.31	2.32	1675	{}	\N	2020-10-12 14:56:02.629665
43503	FORK	2020-10-08	2.33	2.33	2.32	2.32	4550	{}	\N	2020-10-12 14:56:02.634917
43504	FORK	2020-10-09	2.31	2.33	2.3	2.33	9149	{}	\N	2020-10-12 14:56:02.640661
43505	FOXA	2020-09-28	27.205	27.86	27.07	27.84	2911595	{}	\N	2020-10-12 14:56:03.959885
43506	FOXA	2020-09-29	27.71	27.97	27.12	27.77	3232739	{}	\N	2020-10-12 14:56:03.965917
43507	FOXA	2020-09-30	27.99	28.52	27.5	27.83	4186262	{}	\N	2020-10-12 14:56:03.971043
43508	FOXA	2020-10-01	27.96	28.22	27.505	28.02	3269068	{}	\N	2020-10-12 14:56:03.976732
43509	FOXA	2020-10-02	27.72	28.36	26.915	28.08	3999879	{}	\N	2020-10-12 14:56:03.981136
43510	FOXA	2020-10-05	28.23	28.78	28.02	28.71	2545176	{}	\N	2020-10-12 14:56:03.985411
43511	FOXA	2020-10-06	28.78	29.06	27.81	27.93	3013878	{}	\N	2020-10-12 14:56:03.989895
43512	FOXA	2020-10-07	28.2	29.26	28.1	28.97	4416068	{}	\N	2020-10-12 14:56:03.994284
43513	FOXA	2020-10-08	29.25	29.57	28.915	29.56	2438105	{}	\N	2020-10-12 14:56:03.998187
43514	FOXA	2020-10-09	29.77	30	29.04	29.19	3085138	{}	\N	2020-10-12 14:56:04.002113
43515	FOXA	2020-10-12	29.44	29.52	28.98	28.99	409400	{}	\N	2020-10-12 14:56:04.006148
43516	FLGB	2020-09-28	19.51	19.51	19.42	19.4201	9652	{}	\N	2020-10-12 14:56:08.063419
43517	FLGB	2020-09-29	19.29	19.2913	19.2564	19.2913	910	{}	\N	2020-10-12 14:56:08.070195
43518	FLGB	2020-09-30	19.433	19.433	19.24	19.25	8381	{}	\N	2020-10-12 14:56:08.07716
43519	FLGB	2020-10-01	19.28	19.35	19.22	19.3	11864	{}	\N	2020-10-12 14:56:08.082651
43520	FLGB	2020-10-02	19.31	19.52	19.3007	19.5083	2477	{}	\N	2020-10-12 14:56:08.087444
43521	FLGB	2020-10-05	19.7	19.76	19.64	19.76	5065	{}	\N	2020-10-12 14:56:08.094067
43522	FLGB	2020-10-06	19.73	19.74	19.4406	19.47	2752	{}	\N	2020-10-12 14:56:08.09964
43523	FLGB	2020-10-07	19.58	19.6399	19.5692	19.6399	951	{}	\N	2020-10-12 14:56:08.105493
43524	FLGB	2020-10-08	19.7285	19.7899	19.7285	19.74	1930	{}	\N	2020-10-12 14:56:08.113349
43525	FLGB	2020-10-09	19.9097	20.02	19.9097	20.02	5475	{}	\N	2020-10-12 14:56:08.118685
43526	FLGB	2020-10-12	19.95	20.0092	19.95	19.98	1862	{}	\N	2020-10-12 14:56:08.125071
43527	FLIR	2020-09-28	35.46	36.17	35.46	35.98	1016593	{}	\N	2020-10-12 14:56:09.16518
43528	FLIR	2020-09-29	36	36.23	35.11	35.58	734226	{}	\N	2020-10-12 14:56:09.172443
43529	FLIR	2020-09-30	35.73	36.29	35.55	35.85	1247376	{}	\N	2020-10-12 14:56:09.179491
43530	FLIR	2020-10-01	36.15	36.235	35.28	35.44	947039	{}	\N	2020-10-12 14:56:09.184367
43531	FLIR	2020-10-02	34.97	35.925	34.8485	35.49	882527	{}	\N	2020-10-12 14:56:09.18932
43532	FLIR	2020-10-05	35.95	36.54	35.95	36.46	1092528	{}	\N	2020-10-12 14:56:09.193749
43533	FLIR	2020-10-06	36.54	37.54	36.46	36.66	1085971	{}	\N	2020-10-12 14:56:09.198447
43534	FLIR	2020-10-07	36.88	37.18	36.55	36.73	629671	{}	\N	2020-10-12 14:56:09.203725
43535	FLIR	2020-10-08	37.3	37.53	36.76	37.42	796535	{}	\N	2020-10-12 14:56:09.207403
43536	FLIR	2020-10-09	37.6	38.05	37.17	37.53	871958	{}	\N	2020-10-12 14:56:09.211347
43537	FLIR	2020-10-12	37.73	38.21	37.67	38.13	182890	{}	\N	2020-10-12 14:56:09.214996
43538	FLLV	2020-09-28	38.78	38.85	38.57	38.73	8012	{}	\N	2020-10-12 14:56:10.536229
43539	FLLV	2020-09-29	38.77	38.81	38.55	38.6399	7871	{}	\N	2020-10-12 14:56:10.543539
43540	FLLV	2020-09-30	38.69	39.11	38.6039	38.87	7608	{}	\N	2020-10-12 14:56:10.548964
43541	FLLV	2020-10-01	38.9015	39.035	38.835	38.835	4772	{}	\N	2020-10-12 14:56:10.555482
43542	FLLV	2020-10-02	38.48	38.9061	38.48	38.75	6498	{}	\N	2020-10-12 14:56:10.560273
43543	FLLV	2020-10-05	39.01	39.25	39.01	39.25	8405	{}	\N	2020-10-12 14:56:10.564915
43544	FLLV	2020-10-06	39.241	39.47	38.8668	38.8668	2061	{}	\N	2020-10-12 14:56:10.570219
43545	FLLV	2020-10-07	39.18	39.39	39.17	39.37	7383	{}	\N	2020-10-12 14:56:10.574874
43546	FLLV	2020-10-08	39.71	39.77	39.59	39.705	6046	{}	\N	2020-10-12 14:56:10.579142
43547	FLLV	2020-10-09	39.88	40.08	39.88	39.95	6609	{}	\N	2020-10-12 14:56:10.583516
43548	FLLV	2020-10-12	40.33	40.33	40.1926	40.2951	2885	{}	\N	2020-10-12 14:56:10.589233
43549	FLQD	2020-09-28	27.8309	27.84	27.7805	27.7805	947	{}	\N	2020-10-12 14:56:11.792466
43550	FLQD	2020-09-29	27.69	27.78	27.65	27.65	2714	{}	\N	2020-10-12 14:56:11.799789
43551	FLQD	2020-09-30	27.77	27.82	27.77	27.82	395	{}	\N	2020-10-12 14:56:11.805348
43552	FLQD	2020-10-01	27.811	27.82	27.695	27.82	2199	{}	\N	2020-10-12 14:56:11.810506
43553	FLQD	2020-10-02	27.9259	27.95	27.8316	27.8316	876	{}	\N	2020-10-12 14:56:11.815358
43554	FLQD	2020-10-05	28.1545	28.2427	28.1545	28.2427	445	{}	\N	2020-10-12 14:56:11.820797
43555	FLQD	2020-10-06	28.32	28.44	28.0142	28.0142	1641	{}	\N	2020-10-12 14:56:11.824925
43556	FLQD	2020-10-07	28.4265	28.4265	28.4265	28.4265	123	{}	\N	2020-10-12 14:56:11.828986
43557	FLQD	2020-10-08	28.585	28.61	28.54	28.61	553	{}	\N	2020-10-12 14:56:11.832976
43558	FLQD	2020-10-09	28.8388	28.8388	28.7	28.7	3669	{}	\N	2020-10-12 14:56:11.83745
43559	FLT	2020-09-28	240.83	243.445	239.59	241.71	520350	{}	\N	2020-10-12 14:56:13.547579
43560	FLT	2020-09-29	242.88	243.75	240.31	240.75	1970467	{}	\N	2020-10-12 14:56:13.558266
43561	FLT	2020-09-30	240.34	243.13	235.78	238.1	1211707	{}	\N	2020-10-12 14:56:13.564285
43562	FLT	2020-10-01	241.39	243.3	236.62	237.18	962625	{}	\N	2020-10-12 14:56:13.570128
43563	FLT	2020-10-02	232.88	235.96	231.4	234.46	1104683	{}	\N	2020-10-12 14:56:13.575174
43564	FLT	2020-10-05	236.63	239.1375	234.94	238.92	1217011	{}	\N	2020-10-12 14:56:13.580543
43565	FLT	2020-10-06	241.06	246.255	236.84	238.94	907563	{}	\N	2020-10-12 14:56:13.584766
43566	FLT	2020-10-07	240.93	241.92	239.1	240.9	749180	{}	\N	2020-10-12 14:56:13.589061
43567	FLT	2020-10-08	241.41	245.86	241.41	245.12	805602	{}	\N	2020-10-12 14:56:13.593528
43568	FLT	2020-10-09	247.75	250.29	245.02	247.11	1082058	{}	\N	2020-10-12 14:56:13.597483
43569	FLT	2020-10-12	249.28	252.915	248.3628	252.69	170901	{}	\N	2020-10-12 14:56:13.601737
43570	FM	2020-09-28	25.82	25.93	25.65	25.775	134923	{}	\N	2020-10-12 14:56:15.455047
43571	FM	2020-09-29	25.73	25.735	25.4	25.69	46688	{}	\N	2020-10-12 14:56:15.46387
43572	FM	2020-09-30	25.34	25.7504	25.34	25.59	42254	{}	\N	2020-10-12 14:56:15.470188
43573	FM	2020-10-01	25.55	25.87	25.465	25.81	89514	{}	\N	2020-10-12 14:56:15.476505
43574	FM	2020-10-02	25.71	25.8	25.56	25.7	28675	{}	\N	2020-10-12 14:56:15.481392
43575	FM	2020-10-05	25.82	26.265	25.795	26.265	39090	{}	\N	2020-10-12 14:56:15.485861
43576	FM	2020-10-06	26.23	26.5	26.03	26.4	615964	{}	\N	2020-10-12 14:56:15.489931
43577	FM	2020-10-07	26.39	26.628	26.39	26.6	25791	{}	\N	2020-10-12 14:56:15.493807
43578	FM	2020-10-08	26.52	26.74	26.515	26.6	123678	{}	\N	2020-10-12 14:56:15.49765
43579	FM	2020-10-09	26.84	26.93	26.52	26.93	18371	{}	\N	2020-10-12 14:56:15.50125
43580	FM	2020-10-12	26.94	26.94	26.6601	26.6601	4888	{}	\N	2020-10-12 14:56:15.505004
43581	FMF	2020-09-28	43.4	43.6587	43.29	43.52	3505	{}	\N	2020-10-12 14:56:16.687875
43582	FMF	2020-09-29	43.575	43.82	43.3601	43.575	7822	{}	\N	2020-10-12 14:56:16.696046
43583	FMF	2020-09-30	43.77	43.77	43.416	43.5675	3161	{}	\N	2020-10-12 14:56:16.702084
43584	FMF	2020-10-01	43.93	43.93	43.4	43.59	8918	{}	\N	2020-10-12 14:56:16.708037
43585	FMF	2020-10-02	43.6799	43.6799	43.4	43.4995	3429	{}	\N	2020-10-12 14:56:16.713694
43586	FMF	2020-10-05	43.62	43.75	43.4301	43.4301	5243	{}	\N	2020-10-12 14:56:16.718341
43587	FMF	2020-10-06	43.685	43.89	43.45	43.85	12253	{}	\N	2020-10-12 14:56:16.722919
43588	FMF	2020-10-07	43.67	43.85	43.4	43.41	5106	{}	\N	2020-10-12 14:56:16.727085
43589	FMF	2020-10-08	44.14	44.14	43.785	43.785	3117	{}	\N	2020-10-12 14:56:16.730975
43590	FMF	2020-10-09	43.659	44.09	43.6201	43.8512	2934	{}	\N	2020-10-12 14:56:16.735154
43591	FMF	2020-10-12	43.8429	44.03	43.5401	43.5401	1062	{}	\N	2020-10-12 14:56:16.739993
43592	FMY	2020-09-28	13.63	13.7112	13.618	13.7	11747	{}	\N	2020-10-12 14:56:17.772312
43593	FMY	2020-09-29	13.8	13.8	13.6	13.62	6816	{}	\N	2020-10-12 14:56:17.781087
43594	FMY	2020-09-30	13.6141	13.6141	13.58	13.59	11092	{}	\N	2020-10-12 14:56:17.787092
43595	FMY	2020-10-01	13.7179	13.89	13.59	13.64	21756	{}	\N	2020-10-12 14:56:17.791909
43596	FMY	2020-10-02	13.6	13.77	13.6	13.7232	5862	{}	\N	2020-10-12 14:56:17.79731
43597	FMY	2020-10-05	13.65	14	13.62	13.96	52598	{}	\N	2020-10-12 14:56:17.802079
43598	FMY	2020-10-06	13.9	13.9	13.75	13.79	11574	{}	\N	2020-10-12 14:56:17.806114
43599	FMY	2020-10-07	13.77	13.8324	13.65	13.65	4162	{}	\N	2020-10-12 14:56:17.810337
43600	FMY	2020-10-08	13.67	13.87	13.67	13.85	28434	{}	\N	2020-10-12 14:56:17.81415
43601	FMY	2020-10-09	13.81	13.8999	13.66	13.8417	15368	{}	\N	2020-10-12 14:56:17.817904
43602	FMY	2020-10-12	13.93	13.93	13.7725	13.7725	1956	{}	\N	2020-10-12 14:56:17.822328
43603	FNDC	2020-09-28	29.9	30.02	29.87	29.99	154300	{}	\N	2020-10-12 14:56:18.904126
43604	FNDC	2020-09-29	29.9	30.001	29.8	29.88	134914	{}	\N	2020-10-12 14:56:18.908633
43605	FNDC	2020-09-30	29.82	30.079	29.82	29.9	151801	{}	\N	2020-10-12 14:56:18.912977
43606	FNDC	2020-10-01	30.05	30.1799	29.98	30.143	137742	{}	\N	2020-10-12 14:56:18.916624
43607	FNDC	2020-10-02	29.74	30.08	29.74	29.98	145838	{}	\N	2020-10-12 14:56:18.920457
43608	FNDC	2020-10-05	30.29	30.54	30.29	30.51	128335	{}	\N	2020-10-12 14:56:18.924173
43609	FNDC	2020-10-06	30.56	30.6499	30.2699	30.29	138845	{}	\N	2020-10-12 14:56:18.927779
43610	FNDC	2020-10-07	30.47	30.58	30.432	30.5	144184	{}	\N	2020-10-12 14:56:18.931325
43611	FNDC	2020-10-08	30.69	30.78	30.67	30.77	154509	{}	\N	2020-10-12 14:56:18.936109
43612	FNDC	2020-10-09	30.86	30.99	30.8578	30.91	107411	{}	\N	2020-10-12 14:56:18.940798
43613	FNDC	2020-10-12	30.92	30.97	30.9	30.96	21429	{}	\N	2020-10-12 14:56:18.944568
43614	FNHC	2020-09-28	6.62	7.38	6.62	6.94	61271	{}	\N	2020-10-12 14:56:20.211503
43615	FNHC	2020-09-29	7	7	6.47	6.69	44468	{}	\N	2020-10-12 14:56:20.219631
43616	FNHC	2020-09-30	6.66	6.79	6.3	6.32	30565	{}	\N	2020-10-12 14:56:20.225931
43617	FNHC	2020-10-01	6.4	6.4	6	6.24	97462	{}	\N	2020-10-12 14:56:20.231302
43618	FNHC	2020-10-02	6.18	6.42	6.16	6.16	27566	{}	\N	2020-10-12 14:56:20.236803
43619	FNHC	2020-10-05	6.515	6.515	6.06	6.09	39576	{}	\N	2020-10-12 14:56:20.241738
43620	FNHC	2020-10-06	6.14	6.14	5.88	5.89	66818	{}	\N	2020-10-12 14:56:20.246201
43621	FNHC	2020-10-07	6.28	6.28	5.95	5.99	54256	{}	\N	2020-10-12 14:56:20.250831
43622	FNHC	2020-10-08	6	6.16	5.99	6.07	36632	{}	\N	2020-10-12 14:56:20.255574
43623	FNHC	2020-10-09	6.07	6.13	5.95	6.04	32896	{}	\N	2020-10-12 14:56:20.259713
43624	FNHC	2020-10-12	6.25	6.25	5.95	5.95	11787	{}	\N	2020-10-12 14:56:20.264112
43625	FLCO	2020-09-28	26.7	26.7	26.68	26.7	6335	{}	\N	2020-10-12 14:56:21.288794
43626	FLCO	2020-09-29	26.76	26.77	26.71	26.71	19072	{}	\N	2020-10-12 14:56:21.297713
43627	FLCO	2020-09-30	26.69	26.69	26.62	26.62	42972	{}	\N	2020-10-12 14:56:21.3037
43628	FLCO	2020-10-01	26.64	26.7	26.61	26.7	24490	{}	\N	2020-10-12 14:56:21.310861
43629	FLCO	2020-10-02	26.68	26.7	26.63	26.66	5356	{}	\N	2020-10-12 14:56:21.31741
43630	FLCO	2020-10-05	26.591	26.61	26.575	26.58	14433	{}	\N	2020-10-12 14:56:21.322656
43631	FLCO	2020-10-06	26.57	26.6489	26.57	26.61	9219	{}	\N	2020-10-12 14:56:21.326744
43632	FLCO	2020-10-07	26.615	26.6456	26.605	26.605	8886	{}	\N	2020-10-12 14:56:21.331166
43633	FLCO	2020-10-08	26.63	26.71	26.63	26.69	6169	{}	\N	2020-10-12 14:56:21.335632
43634	FLCO	2020-10-09	26.69	26.73	26.65	26.72	9566	{}	\N	2020-10-12 14:56:21.340508
43635	FLCO	2020-10-12	26.79	26.84	26.79	26.8187	1082	{}	\N	2020-10-12 14:56:21.344755
43636	FLQS	2020-09-28	25.795	25.7964	25.795	25.7964	421	{}	\N	2020-10-12 14:56:24.094647
43637	FLQS	2020-09-29	25.6239	25.6239	25.6239	25.6239	183	{}	\N	2020-10-12 14:56:24.101588
43638	FLQS	2020-09-30	25.9	25.9	25.6904	25.6904	1667	{}	\N	2020-10-12 14:56:24.107189
43639	FLQS	2020-10-01	25.83	26.07	25.82	26.07	4547	{}	\N	2020-10-12 14:56:24.112473
43640	FLQS	2020-10-02	26.3544	26.3544	26.3544	26.3544	120	{}	\N	2020-10-12 14:56:24.117034
43641	FLQS	2020-10-05	26.7507	26.8375	26.68	26.8375	1614	{}	\N	2020-10-12 14:56:24.121973
43642	FLQS	2020-10-06	26.7621	26.7621	26.7621	26.7621	115	{}	\N	2020-10-12 14:56:24.126176
43643	FLQS	2020-10-07	27.2084	27.2084	27.2084	27.2084	100	{}	\N	2020-10-12 14:56:24.130406
43644	FLQS	2020-10-08	27.4	27.4686	27.4	27.4686	830	{}	\N	2020-10-12 14:56:24.134831
43645	FLQS	2020-10-09	27.6412	27.6412	27.6412	27.6412	100	{}	\N	2020-10-12 14:56:24.139345
43646	FLWS	2020-09-28	25.37	25.81	24.73	25.64	619216	{}	\N	2020-10-12 14:56:25.12503
43647	FLWS	2020-09-29	25.62	25.91	24.83	25.16	615928	{}	\N	2020-10-12 14:56:25.134073
43648	FLWS	2020-09-30	25.24	25.55	24.75	24.94	549613	{}	\N	2020-10-12 14:56:25.139917
43649	FLWS	2020-10-01	24.99	25.6	24.43	25.54	469878	{}	\N	2020-10-12 14:56:25.145178
43650	FLWS	2020-10-02	25.15	26.37	24.93	26.23	483108	{}	\N	2020-10-12 14:56:25.150895
43651	FLWS	2020-10-05	26.42	27.44	26.21	26.99	581424	{}	\N	2020-10-12 14:56:25.15603
43652	FLWS	2020-10-06	27.08	27.12	25.55	25.89	656576	{}	\N	2020-10-12 14:56:25.160391
43653	FLWS	2020-10-07	26.27	26.68	25.92	26.57	645109	{}	\N	2020-10-12 14:56:25.164585
43654	FLWS	2020-10-08	26.65	26.95	26.07	26.26	399126	{}	\N	2020-10-12 14:56:25.168993
43655	FLWS	2020-10-09	26.52	27.34	26.35	27.285	719593	{}	\N	2020-10-12 14:56:25.173182
43656	FLWS	2020-10-12	27.59	27.9799	27.31	27.84	168476	{}	\N	2020-10-12 14:56:25.177508
43657	FMC	2020-09-28	106.88	107.99	106.06	106.32	608262	{}	\N	2020-10-12 14:56:26.808871
43658	FMC	2020-09-29	106.14	106.7	104.78	105.31	765457	{}	\N	2020-10-12 14:56:26.820106
43659	FMC	2020-09-30	105.81	107.565	105.07	105.91	1061827	{}	\N	2020-10-12 14:56:26.827836
43660	FMC	2020-10-01	106.36	107.16	102.62	103.19	1884188	{}	\N	2020-10-12 14:56:26.833943
43661	FMC	2020-10-02	101.65	105.12	101.421	103.61	1482144	{}	\N	2020-10-12 14:56:26.839026
43662	FMC	2020-10-05	104.62	106.35	104.57	105.33	1583918	{}	\N	2020-10-12 14:56:26.843613
43663	FMC	2020-10-06	105.86	106.98	103.64	103.82	1509829	{}	\N	2020-10-12 14:56:26.848399
43664	FMC	2020-10-07	105.45	106.855	105.33	106.28	1088705	{}	\N	2020-10-12 14:56:26.853412
43665	FMC	2020-10-08	106.95	107.63	105.92	106.06	906108	{}	\N	2020-10-12 14:56:26.85814
43666	FMC	2020-10-09	106.31	108.93	106.06	108.02	1589700	{}	\N	2020-10-12 14:56:26.863178
43667	FMC	2020-10-12	108.11	108.68	107.74	108.57	144036	{}	\N	2020-10-12 14:56:26.867909
43668	FNB	2020-09-28	6.64	6.89	6.64	6.84	2229694	{}	\N	2020-10-12 14:56:28.462865
43669	FNB	2020-09-29	6.76	6.81	6.6	6.73	5533629	{}	\N	2020-10-12 14:56:28.469059
43670	FNB	2020-09-30	6.78	6.89	6.72	6.78	5717569	{}	\N	2020-10-12 14:56:28.473978
43671	FNB	2020-10-01	6.76	6.88	6.665	6.71	4623036	{}	\N	2020-10-12 14:56:28.478797
43672	FNB	2020-10-02	6.58	6.99	6.57	6.98	4704587	{}	\N	2020-10-12 14:56:28.483355
43673	FNB	2020-10-05	7.07	7.28	7.055	7.19	8289764	{}	\N	2020-10-12 14:56:28.488661
43674	FNB	2020-10-06	7.29	7.56	7.21	7.25	6628831	{}	\N	2020-10-12 14:56:28.493906
43675	FNB	2020-10-07	7.35	7.63	7.3	7.5	5720275	{}	\N	2020-10-12 14:56:28.498939
43676	FNB	2020-10-08	7.62	7.645	7.495	7.62	3921565	{}	\N	2020-10-12 14:56:28.503835
43677	FNB	2020-10-09	7.68	7.73	7.47	7.51	4281661	{}	\N	2020-10-12 14:56:28.508045
43678	FNB	2020-10-12	7.52	7.625	7.505	7.59	415919	{}	\N	2020-10-12 14:56:28.512845
43679	FOF	2020-09-28	10.96	11.07	10.9018	11.03	130651	{}	\N	2020-10-12 14:56:30.6174
43680	FOF	2020-09-29	11	11.0496	10.92	11.01	179062	{}	\N	2020-10-12 14:56:30.625972
43681	FOF	2020-09-30	11.1	11.2	11.07	11.11	164514	{}	\N	2020-10-12 14:56:30.631891
43682	FOF	2020-10-01	11.16	11.25	11.07	11.15	138653	{}	\N	2020-10-12 14:56:30.637817
43683	FOF	2020-10-02	11.09	11.27	11.062	11.25	72175	{}	\N	2020-10-12 14:56:30.64248
43684	FOF	2020-10-05	11.31	11.4562	11.31	11.37	113586	{}	\N	2020-10-12 14:56:30.647349
43685	FOF	2020-10-06	11.34	11.42	11.28	11.28	121963	{}	\N	2020-10-12 14:56:30.651598
43686	FOF	2020-10-07	11.35	11.45	11.1	11.16	468718	{}	\N	2020-10-12 14:56:30.655837
43687	FOF	2020-10-08	11.2	11.27	11.1604	11.25	115066	{}	\N	2020-10-12 14:56:30.659853
43688	FOF	2020-10-09	11.25	11.3299	11.24	11.28	212902	{}	\N	2020-10-12 14:56:30.663701
43689	FOF	2020-10-12	11.34	11.34	11.24	11.29	46648	{}	\N	2020-10-12 14:56:30.668092
43690	FPA	2020-09-28	26.35	26.614	26.25	26.614	1410	{}	\N	2020-10-12 14:56:34.421255
43691	FPA	2020-09-29	26.58	26.6	26.58	26.6	427	{}	\N	2020-10-12 14:56:34.426567
43692	FPA	2020-09-30	26.26	26.7372	26.26	26.7372	939	{}	\N	2020-10-12 14:56:34.431062
43693	FPA	2020-10-02	26.535	26.65	26.535	26.65	180	{}	\N	2020-10-12 14:56:34.435488
43694	FPA	2020-10-06	27.18	27.18	27.07	27.07	258	{}	\N	2020-10-12 14:56:34.440057
43695	FPA	2020-10-07	27.37	27.37	27.1448	27.1448	765	{}	\N	2020-10-12 14:56:34.444275
43696	FPA	2020-10-08	27.5835	27.62	27.4601	27.62	1866	{}	\N	2020-10-12 14:56:34.448783
43697	FPA	2020-10-09	27.785	27.785	27.7394	27.7394	498	{}	\N	2020-10-12 14:56:34.45339
43698	FPXI	2020-09-28	56.81	56.86	56.42	56.72	118449	{}	\N	2020-10-12 14:56:36.138212
43699	FPXI	2020-09-29	56.73	56.98	56.62	56.81	356146	{}	\N	2020-10-12 14:56:36.147712
43700	FPXI	2020-09-30	56.75	57.0899	56.5	56.84	118560	{}	\N	2020-10-12 14:56:36.15392
43701	FPXI	2020-10-01	57.2	57.6699	56.82	57.52	204111	{}	\N	2020-10-12 14:56:36.16039
43702	FPXI	2020-10-02	56.87	57.4199	56.87	57.14	150510	{}	\N	2020-10-12 14:56:36.165527
43703	FPXI	2020-10-05	57.52	58.1932	57.52	58.1932	83298	{}	\N	2020-10-12 14:56:36.170118
43704	FPXI	2020-10-06	58.4	58.71	57.91	57.91	108039	{}	\N	2020-10-12 14:56:36.174598
43705	FPXI	2020-10-07	58.49	58.69	58.36	58.41	114842	{}	\N	2020-10-12 14:56:36.179005
43706	FPXI	2020-10-08	58.73	59.12	58.69	59.04	107415	{}	\N	2020-10-12 14:56:36.183869
43707	FPXI	2020-10-09	59.09	59.67	59.07	59.55	129024	{}	\N	2020-10-12 14:56:36.188632
43708	FPXI	2020-10-12	60.32	60.385	60.0415	60.31	14134	{}	\N	2020-10-12 14:56:36.193254
43709	FREL	2020-09-28	23.41	23.7	23.41	23.61	297131	{}	\N	2020-10-12 14:56:37.547299
43710	FREL	2020-09-29	23.61	23.61	23.235	23.41	388210	{}	\N	2020-10-12 14:56:37.556032
43711	FREL	2020-09-30	23.51	23.77	23.265	23.5	383125	{}	\N	2020-10-12 14:56:37.561981
43712	FREL	2020-10-01	23.57	23.98	23.49	23.97	428670	{}	\N	2020-10-12 14:56:37.568161
43713	FREL	2020-10-02	23.65	24.45	23.5501	24.39	408968	{}	\N	2020-10-12 14:56:37.573647
43714	FREL	2020-10-05	24.46	24.6	24.07	24.53	255643	{}	\N	2020-10-12 14:56:37.578605
43715	FREL	2020-10-06	24.57	24.72	24.28	24.38	434007	{}	\N	2020-10-12 14:56:37.582812
43716	FREL	2020-10-07	24.51	24.53	24.295	24.41	316467	{}	\N	2020-10-12 14:56:37.587519
43717	FREL	2020-10-08	24.51	24.88	24.51	24.82	392512	{}	\N	2020-10-12 14:56:37.591955
43718	FREL	2020-10-09	24.9565	24.9565	24.63	24.7	348445	{}	\N	2020-10-12 14:56:37.596845
43719	FREL	2020-10-12	24.75	24.85	24.58	24.745	88913	{}	\N	2020-10-12 14:56:37.601611
43720	FLEE	2020-09-28	22.88	22.93	22.88	22.93	4109	{}	\N	2020-10-12 14:56:39.326504
43721	FLEE	2020-09-29	23.0188	23.0188	22.9	22.92	17127	{}	\N	2020-10-12 14:56:39.332973
43722	FLEE	2020-09-30	22.905	22.97	22.88	22.88	5121	{}	\N	2020-10-12 14:56:39.337399
43723	FLEE	2020-10-01	22.98	22.99	22.974	22.974	2253	{}	\N	2020-10-12 14:56:39.341968
43724	FLEE	2020-10-02	22.78	23.02	22.78	22.96	6449	{}	\N	2020-10-12 14:56:39.349838
43725	FLEE	2020-10-05	23.22	23.3812	23.22	23.3812	918	{}	\N	2020-10-12 14:56:39.355108
43726	FLEE	2020-10-06	23.0867	23.0867	23.0867	23.0867	236	{}	\N	2020-10-12 14:56:39.361128
43727	FLEE	2020-10-07	23.2799	23.2945	23.2799	23.2945	415	{}	\N	2020-10-12 14:56:39.367348
43728	FLEE	2020-10-08	23.41	23.4723	23.41	23.4723	529	{}	\N	2020-10-12 14:56:39.372753
43729	FLEE	2020-10-09	23.7295	23.7295	23.672	23.672	889	{}	\N	2020-10-12 14:56:39.377111
43730	FLEE	2020-10-12	23.83	23.83	23.83	23.83	341	{}	\N	2020-10-12 14:56:39.382229
43731	FLGE	2020-09-28	442.302	442.302	442.302	442.302	100	{}	\N	2020-10-12 14:56:40.715328
43732	FLGE	2020-09-29	444.57	444.57	441.8442	441.8442	1954	{}	\N	2020-10-12 14:56:40.725819
43733	FLGE	2020-09-30	449.76	449.76	449.76	449.76	282	{}	\N	2020-10-12 14:56:40.733986
43734	FLGE	2020-10-01	458.0499	458.0499	457.0397	457.0397	471	{}	\N	2020-10-12 14:56:40.740456
43735	FLGE	2020-10-02	448.39	448.39	440.4224	440.4224	7645	{}	\N	2020-10-12 14:56:40.745914
43736	FLGE	2020-10-05	446.83	457.2855	446.83	457.2855	5040	{}	\N	2020-10-12 14:56:40.751315
43737	FLGE	2020-10-06	484.25	484.25	442.03	444.8	10072	{}	\N	2020-10-12 14:56:40.75662
43738	FLGE	2020-10-07	454.18	459.749	453.34	459.749	1995	{}	\N	2020-10-12 14:56:40.761268
43739	FLGE	2020-10-08	464.715	464.715	462	462.64	2880	{}	\N	2020-10-12 14:56:40.766106
43740	FLGE	2020-10-09	468.9	475.9885	468.9	475.9885	826	{}	\N	2020-10-12 14:56:40.770986
43741	FLGE	2020-10-12	487.4	487.4	487.4	487.4	379	{}	\N	2020-10-12 14:56:40.775807
43742	FLJP	2020-09-28	26.38	26.47	26.3265	26.47	21460	{}	\N	2020-10-12 14:56:41.765512
43743	FLJP	2020-09-29	26.47	26.505	26.37	26.39	31371	{}	\N	2020-10-12 14:56:41.776587
43744	FLJP	2020-09-30	26.11	26.2801	26.11	26.17	21754	{}	\N	2020-10-12 14:56:41.783457
43745	FLJP	2020-10-01	26.12	26.28	26.11	26.2175	645805	{}	\N	2020-10-12 14:56:41.788704
43746	FLJP	2020-10-02	25.84	26.1	25.84	26.1	110387	{}	\N	2020-10-12 14:56:41.793564
43747	FLJP	2020-10-05	26.2	26.285	26.18	26.285	20184	{}	\N	2020-10-12 14:56:41.798663
43748	FLJP	2020-10-06	26.3	26.371	26.18	26.23	11698	{}	\N	2020-10-12 14:56:41.803094
43749	FLJP	2020-10-07	26.3231	26.35	26.29	26.335	12754	{}	\N	2020-10-12 14:56:41.807271
43750	FLJP	2020-10-08	26.4399	26.44	26.3401	26.36	17743	{}	\N	2020-10-12 14:56:41.811776
43751	FLJP	2020-10-09	26.33	26.385	26.3	26.36	16120	{}	\N	2020-10-12 14:56:41.816189
43752	FLJP	2020-10-12	26.39	26.39	26.3379	26.355	7354	{}	\N	2020-10-12 14:56:41.820889
43753	FLQG	2020-09-28	32.7439	32.7439	32.7439	32.7439	2	{}	\N	2020-10-12 14:56:43.134325
43754	FLQG	2020-09-29	32.7799	32.7799	32.6982	32.6982	1167	{}	\N	2020-10-12 14:56:43.14199
43755	FLQG	2020-09-30	32.84	32.9041	32.84	32.9041	2813	{}	\N	2020-10-12 14:56:43.147314
43756	FLQG	2020-10-01	32.98	32.98	32.98	32.98	101	{}	\N	2020-10-12 14:56:43.152533
43757	FLQG	2020-10-02	32.845	32.86	32.8217	32.8217	1110	{}	\N	2020-10-12 14:56:43.157216
43758	FLQG	2020-10-05	33.24	33.27	33.24	33.27	1100	{}	\N	2020-10-12 14:56:43.161199
43759	FLQG	2020-10-06	33.2302	33.35	32.9124	32.9124	728	{}	\N	2020-10-12 14:56:43.165723
43760	FLQG	2020-10-07	33.3599	33.3599	33.3599	33.3599	0	{}	\N	2020-10-12 14:56:43.170223
43761	FLQG	2020-10-08	33.5694	33.5694	33.5694	33.5694	194	{}	\N	2020-10-12 14:56:43.174631
43762	FLQG	2020-10-09	33.81	33.8414	33.81	33.8414	664	{}	\N	2020-10-12 14:56:43.178768
43763	FLTB	2020-09-28	52.5	52.5	52.44	52.48	24299	{}	\N	2020-10-12 14:56:44.362306
43764	FLTB	2020-09-29	52.485	52.54	52.44	52.44	19803	{}	\N	2020-10-12 14:56:44.372263
43765	FLTB	2020-09-30	52.49	52.54	52.45	52.4851	8769	{}	\N	2020-10-12 14:56:44.3785
43766	FLTB	2020-10-01	52.4964	52.52	52.466	52.5	28340	{}	\N	2020-10-12 14:56:44.384502
43767	FLTB	2020-10-02	52.54	52.54	52.49	52.5239	11191	{}	\N	2020-10-12 14:56:44.389414
43768	FLTB	2020-10-05	52.49	52.5297	52.45	52.47	50446	{}	\N	2020-10-12 14:56:44.394312
43769	FLTB	2020-10-06	52.49	52.535	52.49	52.5345	10253	{}	\N	2020-10-12 14:56:44.39869
43770	FLTB	2020-10-07	52.54	52.54	52.4685	52.5	18028	{}	\N	2020-10-12 14:56:44.402997
43771	FLTB	2020-10-08	52.44	52.55	52.44	52.55	51329	{}	\N	2020-10-12 14:56:44.407679
43772	FLTB	2020-10-09	52.57	52.58	52.45	52.52	26233	{}	\N	2020-10-12 14:56:44.411744
43773	FLTB	2020-10-12	52.53	52.5549	52.53	52.535	4292	{}	\N	2020-10-12 14:56:44.415908
43774	FMCIU	2020-09-28	32.5	35	32.5	35	1756	{}	\N	2020-10-12 14:56:46.323612
43775	FMCIU	2020-09-29	36	36	36	36	861	{}	\N	2020-10-12 14:56:46.331718
43776	FMCIU	2020-09-30	33.5	33.51	33.5	33.51	425	{}	\N	2020-10-12 14:56:46.337725
43777	FMCIU	2020-10-07	26.1	27.5	26.1	27.5	514	{}	\N	2020-10-12 14:56:46.343417
43778	FMCIU	2020-10-12	33.35	33.5	32.75	32.76	1528	{}	\N	2020-10-12 14:56:46.348814
43779	FMS	2020-09-28	42.2	42.25	42	42.18	128767	{}	\N	2020-10-12 14:56:47.330321
43780	FMS	2020-09-29	42.23	42.54	42.23	42.43	219039	{}	\N	2020-10-12 14:56:47.33984
43781	FMS	2020-09-30	42.29	42.54	42.14	42.48	247814	{}	\N	2020-10-12 14:56:47.346681
43782	FMS	2020-10-01	42.19	42.22	41.735	41.86	525846	{}	\N	2020-10-12 14:56:47.354084
43783	FMS	2020-10-02	41.22	41.64	41.14	41.4	285448	{}	\N	2020-10-12 14:56:47.360009
43784	FMS	2020-10-05	42.09	42.65	42.09	42.58	198038	{}	\N	2020-10-12 14:56:47.364074
43785	FMS	2020-10-06	43.08	43.38	42.772	42.91	259685	{}	\N	2020-10-12 14:56:47.368089
43786	FMS	2020-10-07	43.1	43.25	42.89	43.19	240632	{}	\N	2020-10-12 14:56:47.37188
43787	FMS	2020-10-08	42.95	43.26	42.88	43.19	260458	{}	\N	2020-10-12 14:56:47.375709
43788	FMS	2020-10-09	43.11	43.38	43.03	43.17	159492	{}	\N	2020-10-12 14:56:47.38032
43789	FMS	2020-10-12	43.09	43.19	42.95	42.95	85202	{}	\N	2020-10-12 14:56:47.38474
43790	FND	2020-09-28	73.34	74.15	72.81	73.78	633815	{}	\N	2020-10-12 14:56:48.59756
43791	FND	2020-09-29	74.38	75.18	73.33	73.39	1157311	{}	\N	2020-10-12 14:56:48.606166
43792	FND	2020-09-30	74.13	76.5	73.18	74.8	2040137	{}	\N	2020-10-12 14:56:48.612627
43793	FND	2020-10-01	75.77	76.64	75	76.39	1459260	{}	\N	2020-10-12 14:56:48.619247
43794	FND	2020-10-02	74.67	76.38	74.205	75.53	1187829	{}	\N	2020-10-12 14:56:48.623806
43795	FND	2020-10-05	76.02	77.08	73.86	74.45	1487241	{}	\N	2020-10-12 14:56:48.627628
43796	FND	2020-10-06	74.79	74.89	72.65	73.24	1986010	{}	\N	2020-10-12 14:56:48.631749
43797	FND	2020-10-07	74.52	76.57	73.99	76.5	1527939	{}	\N	2020-10-12 14:56:48.636155
43798	FND	2020-10-08	77.05	79.22	76.6034	79.01	1531491	{}	\N	2020-10-12 14:56:48.640046
43799	FND	2020-10-09	79.8	81.86	79.29	79.64	2112844	{}	\N	2020-10-12 14:56:48.644071
43800	FND	2020-10-12	81.02	81.74	79.86	80.675	273844	{}	\N	2020-10-12 14:56:48.647973
43801	FNK	2020-09-28	27.31	27.82	27.31	27.7601	7471	{}	\N	2020-10-12 14:56:49.764248
43802	FNK	2020-09-29	27.14	27.39	27.14	27.336	1614	{}	\N	2020-10-12 14:56:49.77557
43803	FNK	2020-09-30	27.51	27.72	27.3854	27.3854	14298	{}	\N	2020-10-12 14:56:49.783665
43804	FNK	2020-10-01	27.4801	27.7685	27.4801	27.641	4151	{}	\N	2020-10-12 14:56:49.789265
43805	FNK	2020-10-02	27.23	28.3995	27.23	28.3995	4506	{}	\N	2020-10-12 14:56:49.794654
43806	FNK	2020-10-05	28.77	28.9986	28.77	28.9986	5362	{}	\N	2020-10-12 14:56:49.799971
43807	FNK	2020-10-06	29.35	29.69	28.77	28.77	3629	{}	\N	2020-10-12 14:56:49.804554
43808	FNK	2020-10-07	29.47	29.53	29.2377	29.465	5093	{}	\N	2020-10-12 14:56:49.808789
43809	FNK	2020-10-08	29.687	29.8936	29.687	29.8936	4777	{}	\N	2020-10-12 14:56:49.813255
43810	FNK	2020-10-09	29.945	29.956	29.7656	29.7656	4961	{}	\N	2020-10-12 14:56:49.81765
43811	FNK	2020-10-12	29.9233	29.9233	29.9233	29.9233	892	{}	\N	2020-10-12 14:56:49.821973
43812	FNWB	2020-09-28	10.02	10.02	9.71	9.76	28022	{}	\N	2020-10-12 14:56:50.727852
43813	FNWB	2020-09-29	9.72	9.86	9.65	9.86	12477	{}	\N	2020-10-12 14:56:50.739834
43814	FNWB	2020-09-30	9.74	9.945	9.73	9.9	3803	{}	\N	2020-10-12 14:56:50.748573
43815	FNWB	2020-10-01	9.7	9.865	9.695	9.83	8659	{}	\N	2020-10-12 14:56:50.754166
43816	FNWB	2020-10-02	9.71	9.97	9.65	9.77	28915	{}	\N	2020-10-12 14:56:50.760474
43817	FNWB	2020-10-05	9.825	10.25	9.765	10.23	13891	{}	\N	2020-10-12 14:56:50.765747
43818	FNWB	2020-10-06	10.01	11.05	10.01	10.21	8088	{}	\N	2020-10-12 14:56:50.770273
43819	FNWB	2020-10-07	10.67	10.88	10.33	10.88	7852	{}	\N	2020-10-12 14:56:50.774713
43820	FNWB	2020-10-08	10.9	11.98	10.53	11.77	83960	{}	\N	2020-10-12 14:56:50.779846
43821	FNWB	2020-10-09	11.81	12	11.68	11.68	2215	{}	\N	2020-10-12 14:56:50.784554
43822	FNWB	2020-10-12	11.6	11.6	11.185	11.185	937	{}	\N	2020-10-12 14:56:50.7889
43823	FONR	2020-09-28	21.83	22.8924	21.735	22.25	23844	{}	\N	2020-10-12 14:56:51.955922
43824	FONR	2020-09-29	22.07	22.66	21.91	22.15	11305	{}	\N	2020-10-12 14:56:51.963672
43825	FONR	2020-09-30	22.05	22.24	20.4	20.88	39884	{}	\N	2020-10-12 14:56:51.969002
43826	FONR	2020-10-01	21	21.39	20.63	21.38	13466	{}	\N	2020-10-12 14:56:51.974565
43827	FONR	2020-10-02	20.87	22.22	20.67	21.69	12311	{}	\N	2020-10-12 14:56:51.97925
43828	FONR	2020-10-05	22	22.49	21.205	22.15	10749	{}	\N	2020-10-12 14:56:51.98381
43829	FONR	2020-10-06	21.51	22.47	21.51	21.69	10695	{}	\N	2020-10-12 14:56:51.987903
43830	FONR	2020-10-07	21.63	21.9416	20.68	21.11	21081	{}	\N	2020-10-12 14:56:51.992436
43831	FONR	2020-10-08	20.96	21.215	20.01	20.1	21491	{}	\N	2020-10-12 14:56:51.997014
43832	FONR	2020-10-09	20.15	20.4	20.05	20.38	22096	{}	\N	2020-10-12 14:56:52.001559
43833	FONR	2020-10-12	20.38	20.38	20.1029	20.18	533	{}	\N	2020-10-12 14:56:52.006057
43834	FORTY	2020-10-01	84.71	85.1336	84.71	85.1336	434	{}	\N	2020-10-12 14:56:53.2922
43835	FORTY	2020-10-08	81.74	81.74	81.74	81.74	195	{}	\N	2020-10-12 14:56:53.300126
43836	FORTY	2020-10-12	86.77	86.77	86.77	86.77	769	{}	\N	2020-10-12 14:56:53.306402
43837	FLEH	2020-09-28	22.11	22.64	22.0834	22.0834	8187	{}	\N	2020-10-12 14:56:54.662558
43838	FLEH	2020-09-29	22.04	22.4	21.9943	21.9943	11402	{}	\N	2020-10-12 14:56:54.669842
43839	FLEH	2020-09-30	21.98	21.98	21.9285	21.9285	156	{}	\N	2020-10-12 14:56:54.674706
43840	FLEH	2020-10-01	22.05	22.05	21.92	21.97	2949	{}	\N	2020-10-12 14:56:54.679183
43841	FLEH	2020-10-02	22.1	22.11	22.0581	22.0581	4534	{}	\N	2020-10-12 14:56:54.683176
43842	FLEH	2020-10-05	22.2583	22.2882	22.2583	22.2882	1049	{}	\N	2020-10-12 14:56:54.687647
43843	FLEH	2020-10-06	22.2666	22.2666	22.0981	22.0981	613	{}	\N	2020-10-12 14:56:54.691634
43844	FLEH	2020-10-07	22.277	22.277	22.277	22.277	22	{}	\N	2020-10-12 14:56:54.695708
43845	FLEH	2020-10-08	22.48	22.8	22.44	22.51	16335	{}	\N	2020-10-12 14:56:54.69985
43846	FLEH	2020-10-09	22.57	22.57	22.5533	22.57	4259	{}	\N	2020-10-12 14:56:54.703846
43847	FLN	2020-09-28	14.955	14.955	14.65	14.81	15362	{}	\N	2020-10-12 14:56:55.726936
43848	FLN	2020-09-29	14.7	14.7651	14.615	14.615	3681	{}	\N	2020-10-12 14:56:55.735331
43849	FLN	2020-09-30	14.729	14.81	14.729	14.81	466	{}	\N	2020-10-12 14:56:55.741356
43850	FLN	2020-10-01	15	15	14.8199	14.8894	6085	{}	\N	2020-10-12 14:56:55.747202
43851	FLN	2020-10-02	14.9721	14.9721	14.75	14.75	2212	{}	\N	2020-10-12 14:56:55.752285
43852	FLN	2020-10-05	14.79	15.12	14.79	15.12	8761	{}	\N	2020-10-12 14:56:55.757447
43853	FLN	2020-10-08	15.31	15.4	15.31	15.345	11525	{}	\N	2020-10-12 14:56:55.761845
43854	FLN	2020-10-09	15.6	15.625	15.5551	15.5551	644	{}	\N	2020-10-12 14:56:55.766034
43855	FLN	2020-10-12	15.54	15.54	15.54	15.54	200	{}	\N	2020-10-12 14:56:55.769982
43856	FLR	2020-09-28	9.64	9.98	9.05	9.15	3118795	{}	\N	2020-10-12 14:56:56.866926
43857	FLR	2020-09-29	8.96	9.41	8.8101	9.23	5142858	{}	\N	2020-10-12 14:56:56.873788
43858	FLR	2020-09-30	9.28	9.43	8.76	8.81	4729938	{}	\N	2020-10-12 14:56:56.879616
43859	FLR	2020-10-01	8.86	9.24	8.78	9.15	3969827	{}	\N	2020-10-12 14:56:56.886403
43860	FLR	2020-10-02	8.88	9.45	8.81	9.4	6982190	{}	\N	2020-10-12 14:56:56.893964
43861	FLR	2020-10-05	9.59	9.7899	9.475	9.71	6524027	{}	\N	2020-10-12 14:56:56.899189
43862	FLR	2020-10-06	9.84	10.135	9.45	9.48	4371628	{}	\N	2020-10-12 14:56:56.904316
43863	FLR	2020-10-07	9.58	11.1795	9.575	10.9	9677816	{}	\N	2020-10-12 14:56:56.909453
43864	FLR	2020-10-08	10.98	11.08	10.64	11.03	7453887	{}	\N	2020-10-12 14:56:56.91463
43865	FLR	2020-10-09	11.13	11.19	10.795	10.92	3949696	{}	\N	2020-10-12 14:56:56.92018
43866	FLR	2020-10-12	10.99	11.22	10.86	10.95	845738	{}	\N	2020-10-12 14:56:56.925685
43867	FLXS	2020-09-28	23.04	23.5	22.97	23.09	44256	{}	\N	2020-10-12 14:56:57.886344
43868	FLXS	2020-09-29	23	24.19	23	23.51	121081	{}	\N	2020-10-12 14:56:57.893798
43869	FLXS	2020-09-30	23.98	24.34	23.7	24	61858	{}	\N	2020-10-12 14:56:57.899522
43870	FLXS	2020-10-01	24.14	24.63	24.14	24.39	39475	{}	\N	2020-10-12 14:56:57.904697
43871	FLXS	2020-10-02	24.42	25.28	23.84	24.11	134737	{}	\N	2020-10-12 14:56:57.908911
43872	FLXS	2020-10-05	24.01	27.34	24.01	26.83	63716	{}	\N	2020-10-12 14:56:57.913526
43873	FLXS	2020-10-06	27.06	27.14	25.3906	25.41	52089	{}	\N	2020-10-12 14:56:57.917843
43874	FLXS	2020-10-07	25.8	26.953	25.8	26.34	33421	{}	\N	2020-10-12 14:56:57.921998
43875	FLXS	2020-10-08	26.33	26.9399	25.99	26.17	34407	{}	\N	2020-10-12 14:56:57.926232
43876	FLXS	2020-10-09	26.08	29.96	26.08	28.87	97908	{}	\N	2020-10-12 14:56:57.930921
43877	FLXS	2020-10-12	29.07	29.73	28.92	29.19	51911	{}	\N	2020-10-12 14:56:57.935144
43878	FMCI	2020-09-28	25.41	26.7	24.5224	24.96	2880675	{}	\N	2020-10-12 14:56:59.274175
43879	FMCI	2020-09-29	25.26	26	25.1	25.54	1070026	{}	\N	2020-10-12 14:56:59.283635
43880	FMCI	2020-09-30	25.4	25.5	23.51	23.64	1423175	{}	\N	2020-10-12 14:56:59.28967
43881	FMCI	2020-10-01	24.02	24.57	22.9002	23.97	888251	{}	\N	2020-10-12 14:56:59.29506
43882	FMCI	2020-10-02	22.067	23.5	22.02	22.8442	833904	{}	\N	2020-10-12 14:56:59.30046
43883	FMCI	2020-10-05	23.5	24.18	22.3	23.23	996689	{}	\N	2020-10-12 14:56:59.305831
43884	FMCI	2020-10-06	23.19	23.61	20.93	21.06	1819761	{}	\N	2020-10-12 14:56:59.310422
43885	FMCI	2020-10-07	22.6	22.88	21.15	22.16	1347393	{}	\N	2020-10-12 14:56:59.315265
43886	FMCI	2020-10-08	22.2	24.94	22.2	24.37	1591189	{}	\N	2020-10-12 14:56:59.319748
43887	FMCI	2020-10-09	24.41	25.83	23.7	24.32	1095336	{}	\N	2020-10-12 14:56:59.323952
43888	FMCI	2020-10-12	25	25.2	23.9	24.12	378783	{}	\N	2020-10-12 14:56:59.328951
43889	FMHI	2020-09-28	51.743	51.89	51.7294	51.78	19238	{}	\N	2020-10-12 14:57:00.213972
43890	FMHI	2020-09-29	51.75	51.86	51.74	51.8599	13284	{}	\N	2020-10-12 14:57:00.22407
43891	FMHI	2020-09-30	51.86	51.86	51.6901	51.75	11400	{}	\N	2020-10-12 14:57:00.232669
43892	FMHI	2020-10-01	51.68	51.88	51.67	51.7994	45277	{}	\N	2020-10-12 14:57:00.238292
43893	FMHI	2020-10-02	51.7801	51.88	51.771	51.88	26265	{}	\N	2020-10-12 14:57:00.243446
43894	FMHI	2020-10-05	51.89	51.89	51.65	51.65	13182	{}	\N	2020-10-12 14:57:00.248673
43895	FMHI	2020-10-06	51.8699	51.8699	51.77	51.77	3249	{}	\N	2020-10-12 14:57:00.253707
43896	FMHI	2020-10-07	51.82	51.82	51.6901	51.735	5648	{}	\N	2020-10-12 14:57:00.25893
43897	FMHI	2020-10-08	51.74	51.82	51.6901	51.73	9280	{}	\N	2020-10-12 14:57:00.263935
43898	FMHI	2020-10-09	51.84	51.87	51.61	51.87	10753	{}	\N	2020-10-12 14:57:00.268702
43899	FMHI	2020-10-12	51.84	51.84	51.84	51.84	308	{}	\N	2020-10-12 14:57:00.272951
43900	FNCL	2020-09-28	33.44	34.06	33.44	33.82	81101	{}	\N	2020-10-12 14:57:03.169001
43901	FNCL	2020-09-29	33.84	33.84	33.2401	33.5	85040	{}	\N	2020-10-12 14:57:03.173779
43902	FNCL	2020-09-30	33.54	34.16	33.54	33.82	92812	{}	\N	2020-10-12 14:57:03.17828
43903	FNCL	2020-10-01	33.96	34.25	33.725	33.98	68376	{}	\N	2020-10-12 14:57:03.182882
43904	FNCL	2020-10-02	33.43	34.5273	33.43	34.32	108908	{}	\N	2020-10-12 14:57:03.188044
43905	FNCL	2020-10-05	34.64	35.0325	34.626	34.98	96671	{}	\N	2020-10-12 14:57:03.192397
43906	FNCL	2020-10-06	35.25	35.59	34.5701	34.68	81951	{}	\N	2020-10-12 14:57:03.197096
43907	FNCL	2020-10-07	35	35.42	34.981	35.27	74633	{}	\N	2020-10-12 14:57:03.202203
43908	FNCL	2020-10-08	35.5	35.8354	35.4107	35.78	80305	{}	\N	2020-10-12 14:57:03.206216
43909	FNCL	2020-10-09	35.97	36	35.6	35.79	73749	{}	\N	2020-10-12 14:57:03.210452
43910	FNCL	2020-10-12	35.78	36.0804	35.78	36.066	24356	{}	\N	2020-10-12 14:57:03.21548
43911	FNGD	2020-09-28	9.41	9.67	9.27	9.3	1624202	{}	\N	2020-10-12 14:57:04.869244
43912	FNGD	2020-09-29	9.37	9.43	9.12	9.25	1485835	{}	\N	2020-10-12 14:57:04.879157
43913	FNGD	2020-09-30	9.27	9.27	8.59	8.82	2928219	{}	\N	2020-10-12 14:57:04.886291
43914	FNGD	2020-10-01	8.48	8.69	8.2615	8.3	2616074	{}	\N	2020-10-12 14:57:04.891759
43915	FNGD	2020-10-02	8.95	9.06	8.47	8.99	4194475	{}	\N	2020-10-12 14:57:04.897532
43916	FNGD	2020-10-05	8.79	8.82	8.4	8.4	1533305	{}	\N	2020-10-12 14:57:04.902779
43917	FNGD	2020-10-06	8.48	8.98	8.32	8.85	2106651	{}	\N	2020-10-12 14:57:04.906911
43918	FNGD	2020-10-07	8.54	8.6581	8.31	8.36	1559590	{}	\N	2020-10-12 14:57:04.910906
43919	FNGD	2020-10-08	8.17	8.3489	8.12	8.28	1142446	{}	\N	2020-10-12 14:57:04.915931
43920	FNGD	2020-10-09	8.18	8.25	8.04	8.04	771227	{}	\N	2020-10-12 14:57:04.920324
43921	FNGD	2020-10-12	7.56	7.66	7.38	7.385	671607	{}	\N	2020-10-12 14:57:04.924445
43922	FNV	2020-09-28	139.21	140.34	136.985	138.76	536011	{}	\N	2020-10-12 14:57:06.598069
43923	FNV	2020-09-29	139.89	141.12	138.9806	139.52	958311	{}	\N	2020-10-12 14:57:06.604808
43924	FNV	2020-09-30	139.66	141.16	137.55	139.58	1071001	{}	\N	2020-10-12 14:57:06.609992
43925	FNV	2020-10-01	140.74	143.29	140.29	141.27	1334721	{}	\N	2020-10-12 14:57:06.615372
43926	FNV	2020-10-02	141.73	142.52	139.63	139.69	897190	{}	\N	2020-10-12 14:57:06.620342
43927	FNV	2020-10-05	139.96	142.95	139.85	141.14	788976	{}	\N	2020-10-12 14:57:06.624724
43928	FNV	2020-10-06	141.1	142.34	134.81	134.91	1288533	{}	\N	2020-10-12 14:57:06.629409
43929	FNV	2020-10-07	136.04	137	134.86	135.63	993625	{}	\N	2020-10-12 14:57:06.635313
43930	FNV	2020-10-08	136.06	138.32	136.06	137.19	976018	{}	\N	2020-10-12 14:57:06.640201
43931	FNV	2020-10-09	138.86	141.34	138.7	140.97	1284367	{}	\N	2020-10-12 14:57:06.644363
43932	FNV	2020-10-12	141.61	142.7316	140.48	142.3916	217493	{}	\N	2020-10-12 14:57:06.648928
43933	FORR	2020-09-28	33.53	34.55	33.48	34.33	38295	{}	\N	2020-10-12 14:57:08.314495
43934	FORR	2020-09-29	34.14	34.35	32.81	33.14	52724	{}	\N	2020-10-12 14:57:08.323175
43935	FORR	2020-09-30	33.61	34.06	32.64	32.79	78024	{}	\N	2020-10-12 14:57:08.329106
43936	FORR	2020-10-01	32.92	33.6762	32.21	33.35	52937	{}	\N	2020-10-12 14:57:08.334755
43937	FORR	2020-10-02	32.77	33.79	32.05	33.55	12033	{}	\N	2020-10-12 14:57:08.33934
43938	FORR	2020-10-05	33.65	34	33.455	33.69	25575	{}	\N	2020-10-12 14:57:08.344072
43939	FORR	2020-10-06	33.205	34.375	32.25	33.37	60778	{}	\N	2020-10-12 14:57:08.350338
43940	FORR	2020-10-07	34	35.56	33.725	35.11	53197	{}	\N	2020-10-12 14:57:08.354486
43941	FORR	2020-10-08	35.72	35.96	35.04	35.81	33333	{}	\N	2020-10-12 14:57:08.358393
43942	FORR	2020-10-09	36	36.91	35.35	36.5	35504	{}	\N	2020-10-12 14:57:08.362184
43943	FORR	2020-10-12	36.43	36.49	36.36	36.49	4489	{}	\N	2020-10-12 14:57:08.366562
43944	FPAY	2020-09-28	1.77	1.89	1.77	1.83	109245	{}	\N	2020-10-12 14:57:09.990569
43945	FPAY	2020-09-29	1.82	1.83	1.76	1.76	47346	{}	\N	2020-10-12 14:57:09.999243
43946	FPAY	2020-09-30	1.75	1.8199	1.73	1.77	45590	{}	\N	2020-10-12 14:57:10.005411
43947	FPAY	2020-10-01	1.7786	1.7786	1.7201	1.75	37741	{}	\N	2020-10-12 14:57:10.011304
43948	FPAY	2020-10-02	1.72	1.755	1.7182	1.74	115487	{}	\N	2020-10-12 14:57:10.017104
43949	FPAY	2020-10-05	1.75	1.8099	1.74	1.77	34818	{}	\N	2020-10-12 14:57:10.021804
43950	FPAY	2020-10-06	1.77	1.8	1.77	1.77	37588	{}	\N	2020-10-12 14:57:10.026456
43951	FPAY	2020-10-07	1.78	1.8	1.67	1.73	161880	{}	\N	2020-10-12 14:57:10.031781
43952	FPAY	2020-10-08	1.71	1.81	1.71	1.78	111803	{}	\N	2020-10-12 14:57:10.038097
43953	FPAY	2020-10-09	1.8	1.8	1.72	1.7892	71485	{}	\N	2020-10-12 14:57:10.043566
43954	FPAY	2020-10-12	1.79	1.89	1.79	1.84	46328	{}	\N	2020-10-12 14:57:10.049312
43955	GLTR	2020-09-28	93.11	94.0196	92.71	93.99	29384	{}	\N	2020-10-12 14:57:11.037993
43956	GLTR	2020-09-29	94.45	95.4	94.2427	95.31	33694	{}	\N	2020-10-12 14:57:11.04695
43957	GLTR	2020-09-30	94.69	95.39	93.75	93.96	27684	{}	\N	2020-10-12 14:57:11.053808
43958	GLTR	2020-10-01	94.83	95.68	94.3965	95.07	47332	{}	\N	2020-10-12 14:57:11.059757
43959	GLTR	2020-10-02	95.105	95.465	94.7555	94.89	13952	{}	\N	2020-10-12 14:57:11.065476
43960	GLTR	2020-10-05	95.41	96.3	95.41	96.16	34540	{}	\N	2020-10-12 14:57:11.070203
43961	GLTR	2020-10-06	96.19	96.3716	94.1	94.19	34248	{}	\N	2020-10-12 14:57:11.075074
43962	GLTR	2020-10-07	94.52	94.84	94.3	94.71	65346	{}	\N	2020-10-12 14:57:11.080092
43963	GLTR	2020-10-08	95.25	95.59	94.5601	95.11	76195	{}	\N	2020-10-12 14:57:11.085198
43964	GLTR	2020-10-09	96.67	97.87	96.67	97.83	35434	{}	\N	2020-10-12 14:57:11.090014
43965	GLTR	2020-10-12	97.65	97.6699	97.18	97.6699	8637	{}	\N	2020-10-12 14:57:11.094869
43966	FLFR	2020-09-28	23.029	23.04	23.029	23.03	2462	{}	\N	2020-10-12 14:57:12.240224
43967	FLFR	2020-09-29	23.15	23.15	23.1	23.1	1260	{}	\N	2020-10-12 14:57:12.247729
43968	FLFR	2020-09-30	23.1	23.1	23.0622	23.0622	262	{}	\N	2020-10-12 14:57:12.253055
43969	FLFR	2020-10-01	23.088	23.1502	23.088	23.1502	530	{}	\N	2020-10-12 14:57:12.257797
43970	FLFR	2020-10-02	22.85	23.1714	22.85	23.13	1000	{}	\N	2020-10-12 14:57:12.262633
43971	FLFR	2020-10-05	23.566	23.566	23.566	23.566	89	{}	\N	2020-10-12 14:57:12.266876
43972	FLFR	2020-10-06	23.3489	23.3489	23.3489	23.3489	74	{}	\N	2020-10-12 14:57:12.271426
43973	FLFR	2020-10-07	23.5649	23.5649	23.5649	23.5649	352	{}	\N	2020-10-12 14:57:12.275317
43974	FLFR	2020-10-08	23.7	23.71	23.7	23.71	341	{}	\N	2020-10-12 14:57:12.279152
43975	FLFR	2020-10-09	23.93	23.98	23.93	23.93	1258	{}	\N	2020-10-12 14:57:12.28363
43976	FLIY	2020-09-28	20.7713	20.7713	20.7713	20.7713	6	{}	\N	2020-10-12 14:57:13.967787
43977	FLIY	2020-09-29	20.7658	20.7658	20.7658	20.7658	19	{}	\N	2020-10-12 14:57:13.974654
43978	FLIY	2020-09-30	20.703	20.703	20.703	20.703	11	{}	\N	2020-10-12 14:57:13.980625
43979	FLIY	2020-10-01	20.7152	20.7152	20.7152	20.7152	0	{}	\N	2020-10-12 14:57:13.986135
43980	FLIY	2020-10-02	20.6873	20.6873	20.6873	20.6873	1	{}	\N	2020-10-12 14:57:13.990984
43981	FLIY	2020-10-05	21.0811	21.0811	21.0811	21.0811	3	{}	\N	2020-10-12 14:57:13.99518
43982	FLIY	2020-10-06	20.89	20.89	20.89	20.89	283	{}	\N	2020-10-12 14:57:13.999208
43983	FLIY	2020-10-07	21.1769	21.1769	21.1769	21.1769	0	{}	\N	2020-10-12 14:57:14.003526
43984	FLIY	2020-10-08	21.2966	21.2966	21.2966	21.2966	1	{}	\N	2020-10-12 14:57:14.00761
43985	FLIY	2020-10-09	21.3656	21.3656	21.3656	21.3656	6	{}	\N	2020-10-12 14:57:14.011593
43986	FLQE	2020-09-28	25.7	25.7606	25.7	25.7606	1168	{}	\N	2020-10-12 14:57:15.136396
43987	FLQE	2020-09-29	25.63	25.7254	25.63	25.7254	907	{}	\N	2020-10-12 14:57:15.143544
43988	FLQE	2020-09-30	25.84	25.92	25.82	25.88	1954	{}	\N	2020-10-12 14:57:15.149173
43989	FLQE	2020-10-01	25.97	26.0508	25.97	26.0508	3796	{}	\N	2020-10-12 14:57:15.154132
43990	FLQE	2020-10-02	25.93	25.97	25.836	25.836	1783	{}	\N	2020-10-12 14:57:15.159525
43991	FLQE	2020-10-05	26.09	26.1668	26.09	26.1668	3597	{}	\N	2020-10-12 14:57:15.164158
43992	FLQE	2020-10-06	26.21	26.21	26.0673	26.0673	1152	{}	\N	2020-10-12 14:57:15.168568
43993	FLQE	2020-10-07	26.21	26.2691	26.21	26.2691	491	{}	\N	2020-10-12 14:57:15.172659
43994	FLQE	2020-10-08	26.42	26.4499	26.41	26.41	419	{}	\N	2020-10-12 14:57:15.176668
43995	FLQE	2020-10-09	26.51	26.6495	26.49	26.6495	3611	{}	\N	2020-10-12 14:57:15.180843
43996	FLS	2020-09-28	27.15	27.94	27.15	27.43	685957	{}	\N	2020-10-12 14:57:16.519218
43997	FLS	2020-09-29	27.51	27.59	26.725	26.93	1690797	{}	\N	2020-10-12 14:57:16.528895
43998	FLS	2020-09-30	27.13	27.84	27.01	27.29	2403739	{}	\N	2020-10-12 14:57:16.537291
43999	FLS	2020-10-01	27.36	27.51	26.305	26.67	2028696	{}	\N	2020-10-12 14:57:16.542989
44000	FLS	2020-10-02	25.96	27.5	25.87	27.23	980354	{}	\N	2020-10-12 14:57:16.548698
44001	FLS	2020-10-05	27.5	27.99	27.47	27.9	1207305	{}	\N	2020-10-12 14:57:16.553935
44002	FLS	2020-10-06	28.23	28.5	27.5	27.55	920820	{}	\N	2020-10-12 14:57:16.557567
44003	FLS	2020-10-07	27.83	28.65	27.83	28.51	1323394	{}	\N	2020-10-12 14:57:16.561543
44004	FLS	2020-10-08	28.82	29.18	28.4	29.16	1086803	{}	\N	2020-10-12 14:57:16.565709
44005	FLS	2020-10-09	29.49	29.71	29.1	29.48	1519559	{}	\N	2020-10-12 14:57:16.56921
44006	FLS	2020-10-12	29.47	29.51	29.1	29.1	173524	{}	\N	2020-10-12 14:57:16.572501
44007	FMAT	2020-09-28	34.08	34.5426	34.05	34.24	34247	{}	\N	2020-10-12 14:57:17.483048
44008	FMAT	2020-09-29	34.2	34.3499	33.95	33.9963	27053	{}	\N	2020-10-12 14:57:17.492976
44009	FMAT	2020-09-30	34.24	34.5997	34.02	34.3565	17934	{}	\N	2020-10-12 14:57:17.500092
44010	FMAT	2020-10-01	34.4	34.57	33.835	33.93	119913	{}	\N	2020-10-12 14:57:17.505635
44011	FMAT	2020-10-02	33.51	34.48	33.5	34.3	63082	{}	\N	2020-10-12 14:57:17.510459
44012	FMAT	2020-10-05	34.63	35.2	34.63	35.01	56441	{}	\N	2020-10-12 14:57:17.515593
44013	FMAT	2020-10-06	35.27	35.47	34.5813	34.6	28615	{}	\N	2020-10-12 14:57:17.519932
44014	FMAT	2020-10-07	35.04	35.58	35.04	35.5199	137325	{}	\N	2020-10-12 14:57:17.524691
44015	FMAT	2020-10-08	35.89	35.915	35.729	35.87	30463	{}	\N	2020-10-12 14:57:17.528982
44016	FMAT	2020-10-09	36.19	36.305	35.929	36.105	213831	{}	\N	2020-10-12 14:57:17.533319
44017	FMAT	2020-10-12	36.31	36.31	36.1	36.21	16183	{}	\N	2020-10-12 14:57:17.537499
44018	FN	2020-09-28	62.55	63.35	62.1492	63	156277	{}	\N	2020-10-12 14:57:18.951324
44019	FN	2020-09-29	63.3	64.1689	63.02	63.33	253341	{}	\N	2020-10-12 14:57:18.960714
44020	FN	2020-09-30	63.15	64.51	62.73	63.03	436526	{}	\N	2020-10-12 14:57:18.969669
44021	FN	2020-10-01	63.53	63.6	62.47	62.97	312815	{}	\N	2020-10-12 14:57:18.975828
44022	FN	2020-10-02	61.53	63.23	61.48	62.57	342923	{}	\N	2020-10-12 14:57:18.980977
44023	FN	2020-10-05	63.24	64.23	62.73	63.55	319444	{}	\N	2020-10-12 14:57:18.986039
44024	FN	2020-10-06	63.73	64.91	63.51	63.55	368964	{}	\N	2020-10-12 14:57:18.99031
44025	FN	2020-10-07	64.12	64.12	62.79	63.6	453807	{}	\N	2020-10-12 14:57:18.994323
44026	FN	2020-10-08	64.06	65.13	64.04	64.37	210647	{}	\N	2020-10-12 14:57:18.998646
44027	FN	2020-10-09	65.02	65.89	64.23	64.67	445498	{}	\N	2020-10-12 14:57:19.003233
44028	FN	2020-10-12	64.62	65.3	64.62	65.1	32358	{}	\N	2020-10-12 14:57:19.007464
44029	FNF	2020-09-28	31.75	32.08	31.52	31.59	1941188	{}	\N	2020-10-12 14:57:20.111507
44030	FNF	2020-09-29	31.38	31.61	30.83	30.99	2469253	{}	\N	2020-10-12 14:57:20.121613
44031	FNF	2020-09-30	31.07	31.76	31.07	31.31	3628186	{}	\N	2020-10-12 14:57:20.128095
44032	FNF	2020-10-01	31.35	32.21	31.34	31.89	3152400	{}	\N	2020-10-12 14:57:20.140545
44033	FNF	2020-10-02	31.32	33.2	31.17	33.05	3248689	{}	\N	2020-10-12 14:57:20.148213
44034	FNF	2020-10-05	33.49	34.13	33.38	33.55	2296860	{}	\N	2020-10-12 14:57:20.154728
44035	FNF	2020-10-06	33.82	34.09	33.4	33.44	2929378	{}	\N	2020-10-12 14:57:20.162473
44036	FNF	2020-10-07	33.7	34.15	33.25	33.38	2311817	{}	\N	2020-10-12 14:57:20.167198
44037	FNF	2020-10-08	33.74	33.78	33.29	33.52	1704822	{}	\N	2020-10-12 14:57:20.172036
44038	FNF	2020-10-09	33.98	34.05	33.48	33.48	3629280	{}	\N	2020-10-12 14:57:20.176621
44039	FNF	2020-10-12	33.52	33.98	33.41	33.85	322094	{}	\N	2020-10-12 14:57:20.181797
44040	GRBK	2020-09-28	15.85	16.39	15.78	16.27	248087	{}	\N	2020-10-12 14:57:22.815448
44041	GRBK	2020-09-29	16.23	16.28	15.93	16.06	329799	{}	\N	2020-10-12 14:57:22.826291
44042	GRBK	2020-09-30	16.47	16.47	15.91	16.1	397214	{}	\N	2020-10-12 14:57:22.831914
44043	GRBK	2020-10-01	16.26	16.41	15.8501	16.4	209637	{}	\N	2020-10-12 14:57:22.8365
44044	GRBK	2020-10-02	15.97	16.82	15.95	16.72	321125	{}	\N	2020-10-12 14:57:22.840786
44045	GRBK	2020-10-05	16.99	17.28	16.37	17.04	320634	{}	\N	2020-10-12 14:57:22.845135
44046	GRBK	2020-10-06	17.34	17.83	17.18	17.21	435704	{}	\N	2020-10-12 14:57:22.849479
44047	GRBK	2020-10-07	17.53	17.69	16.9125	17.1	213316	{}	\N	2020-10-12 14:57:22.852798
44048	GRBK	2020-10-08	17.85	18.85	17.7909	18.47	434610	{}	\N	2020-10-12 14:57:22.856397
44049	GRBK	2020-10-09	19	19.805	18.8	18.94	326067	{}	\N	2020-10-12 14:57:22.859778
44050	GRBK	2020-10-12	19.1	19.21	18.63	19.07	118252	{}	\N	2020-10-12 14:57:22.863881
44051	FLMX	2020-09-28	16.0101	16.2255	16.0101	16.2255	756	{}	\N	2020-10-12 14:57:24.155271
44052	FLMX	2020-09-29	16.26	16.26	16.208	16.208	752	{}	\N	2020-10-12 14:57:24.163488
44053	FLMX	2020-09-30	16.5692	16.5692	16.5692	16.5692	122	{}	\N	2020-10-12 14:57:24.169018
44054	FLMX	2020-10-01	16.65	16.699	16.4	16.41	1526	{}	\N	2020-10-12 14:57:24.173335
44055	FLMX	2020-10-02	16.5241	16.5241	16.5241	16.5241	38	{}	\N	2020-10-12 14:57:24.177681
44056	FLMX	2020-10-05	16.82	16.8242	16.82	16.8242	278	{}	\N	2020-10-12 14:57:24.18253
44057	FLMX	2020-10-06	16.6466	16.6466	16.6466	16.6466	4	{}	\N	2020-10-12 14:57:24.1868
44058	FLMX	2020-10-07	16.93	17.0071	16.9	17.0071	844	{}	\N	2020-10-12 14:57:24.19063
44059	FLMX	2020-10-08	17.49	17.5258	17.49	17.5258	903	{}	\N	2020-10-12 14:57:24.194958
44060	FLMX	2020-10-09	17.58	17.84	17.58	17.71	2128	{}	\N	2020-10-12 14:57:24.19897
44061	FLQM	2020-09-28	32.87	33.02	32.87	32.9796	1319	{}	\N	2020-10-12 14:57:25.655501
44062	FLQM	2020-09-29	32.85	32.85	32.81	32.81	710	{}	\N	2020-10-12 14:57:25.665644
44063	FLQM	2020-09-30	33.0716	33.27	32.97	33.0434	3600	{}	\N	2020-10-12 14:57:25.671418
44064	FLQM	2020-10-01	33.13	33.33	33.13	33.2537	3760	{}	\N	2020-10-12 14:57:25.676789
44065	FLQM	2020-10-02	32.95	33.42	32.95	33.362	1299	{}	\N	2020-10-12 14:57:25.682007
44066	FLQM	2020-10-05	33.7402	33.9105	33.7402	33.9105	739	{}	\N	2020-10-12 14:57:25.68681
44067	FLQM	2020-10-06	33.675	33.675	33.6233	33.6233	816	{}	\N	2020-10-12 14:57:25.691623
44068	FLQM	2020-10-07	34.04	34.23	34.0163	34.23	9681	{}	\N	2020-10-12 14:57:25.695795
44069	FLQM	2020-10-08	34.5	34.5	34.33	34.45	3858	{}	\N	2020-10-12 14:57:25.699837
44070	FLQM	2020-10-09	34.6	34.81	34.6	34.715	2170	{}	\N	2020-10-12 14:57:25.704397
44071	FLQM	2020-10-12	34.8602	34.9	34.8602	34.9	408	{}	\N	2020-10-12 14:57:25.708338
44072	FLXN	2020-09-28	11.24	11.24	10.74	10.83	341855	{}	\N	2020-10-12 14:57:26.639466
44073	FLXN	2020-09-29	10.65	10.81	10.23	10.4	606608	{}	\N	2020-10-12 14:57:26.649942
44074	FLXN	2020-09-30	10.44	10.7	10.28	10.41	631088	{}	\N	2020-10-12 14:57:26.656546
44075	FLXN	2020-10-01	10.545	10.6	10.3	10.45	399427	{}	\N	2020-10-12 14:57:26.662194
44076	FLXN	2020-10-02	10.2	10.52	10.08	10.13	454793	{}	\N	2020-10-12 14:57:26.668134
44077	FLXN	2020-10-05	10.33	10.6599	10.14	10.35	380193	{}	\N	2020-10-12 14:57:26.672801
44078	FLXN	2020-10-06	10.5	10.65	10.05	10.175	592172	{}	\N	2020-10-12 14:57:26.676986
44079	FLXN	2020-10-07	10.27	10.91	10.21	10.78	468250	{}	\N	2020-10-12 14:57:26.681558
44080	FLXN	2020-10-08	10.93	11.34	10.87	11.25	353700	{}	\N	2020-10-12 14:57:26.686673
44081	FLXN	2020-10-09	11.38	11.51	11.1554	11.36	301941	{}	\N	2020-10-12 14:57:26.691079
44082	FLXN	2020-10-12	11.34	11.47	11.16	11.47	46552	{}	\N	2020-10-12 14:57:26.6955
44083	FMBI	2020-09-28	10.81	11.25	10.81	11.07	965013	{}	\N	2020-10-12 14:57:27.599593
44084	FMBI	2020-09-29	11.01	11.01	10.55	10.715	530283	{}	\N	2020-10-12 14:57:27.61101
44085	FMBI	2020-09-30	10.82	11.01	10.675	10.78	641117	{}	\N	2020-10-12 14:57:27.618655
44086	FMBI	2020-10-01	10.8	10.88	10.58	10.82	519277	{}	\N	2020-10-12 14:57:27.623935
44087	FMBI	2020-10-02	10.61	11.15	10.61	11.1	455778	{}	\N	2020-10-12 14:57:27.629355
44088	FMBI	2020-10-05	11.29	11.66	11.19	11.57	510076	{}	\N	2020-10-12 14:57:27.634276
44089	FMBI	2020-10-06	11.73	12.12	11.51	11.6	1230780	{}	\N	2020-10-12 14:57:27.639069
44090	FMBI	2020-10-07	11.7	12.1	11.7	11.98	834349	{}	\N	2020-10-12 14:57:27.643506
44091	FMBI	2020-10-08	12.17	12.23	11.93	12.17	459795	{}	\N	2020-10-12 14:57:27.648106
44092	FMBI	2020-10-09	12.29	12.29	11.87	11.87	491470	{}	\N	2020-10-12 14:57:27.652609
44093	FMBI	2020-10-12	11.83	11.98	11.8	11.95	50805	{}	\N	2020-10-12 14:57:27.657453
44094	FMX	2020-09-28	55.25	55.93	55.07	55.7	691998	{}	\N	2020-10-12 14:57:28.555844
44095	FMX	2020-09-29	55.6	56.1	54.73	55.23	1351482	{}	\N	2020-10-12 14:57:28.567663
44096	FMX	2020-09-30	55.61	56.83	55.33	56.19	2099509	{}	\N	2020-10-12 14:57:28.57503
44097	FMX	2020-10-01	56.71	56.81	55.57	55.81	678328	{}	\N	2020-10-12 14:57:28.580742
44098	FMX	2020-10-02	55.21	56.27	55.06	56.01	643288	{}	\N	2020-10-12 14:57:28.585986
44099	FMX	2020-10-05	56.56	57.04	56.1575	56.58	822054	{}	\N	2020-10-12 14:57:28.590739
44100	FMX	2020-10-06	57.03	57.42	56.45	56.7	1201046	{}	\N	2020-10-12 14:57:28.595509
44101	FMX	2020-10-07	57.22	57.655	56.85	57.1	455064	{}	\N	2020-10-12 14:57:28.599647
44102	FMX	2020-10-08	57.21	59.68	57.21	59.51	1943024	{}	\N	2020-10-12 14:57:28.603505
44103	FMX	2020-10-09	59.76	60.54	59.265	60.39	1168878	{}	\N	2020-10-12 14:57:28.60746
44104	FMX	2020-10-12	60.5	60.71	60.255	60.5	121272	{}	\N	2020-10-12 14:57:28.611582
44105	FNDX	2020-09-28	38.59	38.9528	38.59	38.79	258675	{}	\N	2020-10-12 14:57:30.809744
44106	FNDX	2020-09-29	38.71	38.79	38.315	38.44	321269	{}	\N	2020-10-12 14:57:30.819229
44107	FNDX	2020-09-30	38.52	39.16	38.52	38.73	380888	{}	\N	2020-10-12 14:57:30.82533
44108	FNDX	2020-10-01	38.92	39.085	38.66	38.86	223194	{}	\N	2020-10-12 14:57:30.83102
44109	FNDX	2020-10-02	38.17	39.0746	38.14	38.88	275561	{}	\N	2020-10-12 14:57:30.835675
44110	FNDX	2020-10-05	39.13	39.604	39.13	39.57	171402	{}	\N	2020-10-12 14:57:30.840211
44111	FNDX	2020-10-06	39.66	39.98	39.07	39.11	195537	{}	\N	2020-10-12 14:57:30.844561
44112	FNDX	2020-10-07	39.42	39.92	39.42	39.83	190017	{}	\N	2020-10-12 14:57:30.849032
44113	FNDX	2020-10-08	40.03	40.37	40.01	40.36	281566	{}	\N	2020-10-12 14:57:30.85419
44114	FNDX	2020-10-09	40.56	40.65	40.34	40.44	271517	{}	\N	2020-10-12 14:57:30.858898
44115	FNDX	2020-10-12	40.61	40.7928	40.59	40.7776	46765	{}	\N	2020-10-12 14:57:30.864329
44116	FNLC	2020-09-28	19.84	20.62	19.8	20.51	11251	{}	\N	2020-10-12 14:57:31.826052
44117	FNLC	2020-09-29	20.59	20.99	19.7193	20.98	4165	{}	\N	2020-10-12 14:57:31.835389
44118	FNLC	2020-09-30	20.19	21.16	20.19	21.065	6043	{}	\N	2020-10-12 14:57:31.841587
44119	FNLC	2020-10-01	21.2	21.35	20.505	21.01	13315	{}	\N	2020-10-12 14:57:31.847756
44120	FNLC	2020-10-02	20.74	22	20.705	21.86	13051	{}	\N	2020-10-12 14:57:31.851943
44121	FNLC	2020-10-05	21.8	21.99	21.59	21.9	11363	{}	\N	2020-10-12 14:57:31.855912
44122	FNLC	2020-10-06	21.91	22.23	21.53	21.53	11378	{}	\N	2020-10-12 14:57:31.859764
44123	FNLC	2020-10-07	21.78	22.22	21.43	22.09	12253	{}	\N	2020-10-12 14:57:31.863757
44124	FNLC	2020-10-08	22.25	22.25	21.7227	21.85	6934	{}	\N	2020-10-12 14:57:31.867538
44125	FNLC	2020-10-09	21.9	22.17	21.9	22.15	3885	{}	\N	2020-10-12 14:57:31.871126
44126	FNLC	2020-10-12	21.9	22.0719	21.84	22.0719	1561	{}	\N	2020-10-12 14:57:31.874686
44127	FOXF	2020-09-28	74.89	76.02	73.475	75.8	334272	{}	\N	2020-10-12 14:57:32.842367
44128	FOXF	2020-09-29	75.77	77.02	73.82	74.09	289920	{}	\N	2020-10-12 14:57:32.850479
44129	FOXF	2020-09-30	74.81	76.74	73.14	74.33	731946	{}	\N	2020-10-12 14:57:32.855926
44130	FOXF	2020-10-01	74.47	76.54	73.525	76.43	480781	{}	\N	2020-10-12 14:57:32.861321
44131	FOXF	2020-10-02	74.33	78.72	74.01	78.28	276792	{}	\N	2020-10-12 14:57:32.866771
44132	FOXF	2020-10-05	79.44	82.47	79.435	81.47	380384	{}	\N	2020-10-12 14:57:32.871889
44133	FOXF	2020-10-06	81.95	83.36	80.35	80.99	228523	{}	\N	2020-10-12 14:57:32.875881
44134	FOXF	2020-10-07	82.01	85.55	82.01	85.46	316060	{}	\N	2020-10-12 14:57:32.880058
44135	FOXF	2020-10-08	87.08	87.42	84.03	85.06	342695	{}	\N	2020-10-12 14:57:32.883902
44136	FOXF	2020-10-09	85.9	88.4875	83.69	87.7	419997	{}	\N	2020-10-12 14:57:32.887735
44137	FOXF	2020-10-12	87.74	88.48	87.2	87.3	27637	{}	\N	2020-10-12 14:57:32.891547
44138	FPL	2020-09-28	3.68	3.7	3.595	3.67	75956	{}	\N	2020-10-12 14:57:34.025299
44139	FPL	2020-09-29	3.67	3.69	3.63	3.65	185162	{}	\N	2020-10-12 14:57:34.036455
44140	FPL	2020-09-30	3.68	3.71	3.58	3.6	153842	{}	\N	2020-10-12 14:57:34.043416
44141	FPL	2020-10-01	3.53	3.55	3.47	3.4883	243973	{}	\N	2020-10-12 14:57:34.048464
44142	FPL	2020-10-02	3.44	3.57	3.43	3.57	263230	{}	\N	2020-10-12 14:57:34.053778
44143	FPL	2020-10-05	3.59	3.77	3.59	3.73	588840	{}	\N	2020-10-12 14:57:34.058086
44144	FPL	2020-10-06	3.79	3.7919	3.49	3.67	319411	{}	\N	2020-10-12 14:57:34.062328
44145	FPL	2020-10-07	3.75	3.79	3.71	3.79	335089	{}	\N	2020-10-12 14:57:34.066277
44146	FPL	2020-10-08	3.8	3.97	3.8	3.94	460890	{}	\N	2020-10-12 14:57:34.070529
44147	FPL	2020-10-09	4	4	3.9201	3.96	265468	{}	\N	2020-10-12 14:57:34.075116
44148	FPL	2020-10-12	3.93	3.98	3.93	3.96	41122	{}	\N	2020-10-12 14:57:34.079947
44149	FRBA	2020-09-28	6.3	6.445	6.21	6.35	17603	{}	\N	2020-10-12 14:57:36.18141
44150	FRBA	2020-09-29	6.46	6.46	6.14	6.24	6180	{}	\N	2020-10-12 14:57:36.192699
44151	FRBA	2020-09-30	6.26	6.26	6.11	6.2	10065	{}	\N	2020-10-12 14:57:36.199538
44152	FRBA	2020-10-01	6.18	6.34	6.15	6.25	21680	{}	\N	2020-10-12 14:57:36.204875
44153	FRBA	2020-10-02	6.41	6.4685	6.2825	6.44	9877	{}	\N	2020-10-12 14:57:36.209826
44154	FRBA	2020-10-05	6.45	6.74	6.233	6.67	16248	{}	\N	2020-10-12 14:57:36.214995
44155	FRBA	2020-10-06	6.72	7.02	6.58	6.87	36295	{}	\N	2020-10-12 14:57:36.21969
44156	FRBA	2020-10-07	6.95	7.07	6.8401	6.98	22218	{}	\N	2020-10-12 14:57:36.223777
44157	FRBA	2020-10-08	6.98	7.17	6.83	7.11	216155	{}	\N	2020-10-12 14:57:36.227933
44158	FRBA	2020-10-09	7.14	7.204	6.87	6.97	4592	{}	\N	2020-10-12 14:57:36.232246
44159	FRBA	2020-10-12	7.14	7.14	6.94	6.94	812	{}	\N	2020-10-12 14:57:36.236292
44160	GRF	2020-09-28	6.9499	6.9801	6.9499	6.9801	2606	{}	\N	2020-10-12 14:57:38.546972
44161	GRF	2020-09-30	7	7.09	7	7.09	4135	{}	\N	2020-10-12 14:57:38.556229
44162	GRF	2020-10-01	7.05	7.0788	7.05	7.0788	4773	{}	\N	2020-10-12 14:57:38.563464
44163	GRF	2020-10-02	6.88	6.91	6.8	6.8	9508	{}	\N	2020-10-12 14:57:38.569108
44164	GRF	2020-10-05	6.88	6.95	6.88	6.94	12144	{}	\N	2020-10-12 14:57:38.574711
44165	GRF	2020-10-06	7	7.03	7	7.0297	2945	{}	\N	2020-10-12 14:57:38.579662
44166	GRF	2020-10-07	7.05	7.07	7	7.02	9289	{}	\N	2020-10-12 14:57:38.584104
44167	GRF	2020-10-08	7.065	7.09	7.0628	7.09	1618	{}	\N	2020-10-12 14:57:38.588669
44168	GRF	2020-10-09	7.0967	7.135	6.88	6.89	14223	{}	\N	2020-10-12 14:57:38.592871
44169	GRF	2020-10-12	6.9	6.963	6.9	6.963	2301	{}	\N	2020-10-12 14:57:38.597917
44170	FLRT	2020-09-28	47.91	47.91	47.91	47.91	0	{}	\N	2020-10-12 14:57:40.116303
44171	FLRT	2020-09-29	47.78	47.78	47.78	47.78	0	{}	\N	2020-10-12 14:57:40.123635
44172	FLRT	2020-09-30	47.5	47.635	47.5	47.635	163	{}	\N	2020-10-12 14:57:40.128097
44173	FLRT	2020-10-01	47.71	47.71	47.71	47.71	18	{}	\N	2020-10-12 14:57:40.132325
44174	FLRT	2020-10-02	47.625	47.625	47.625	47.625	3	{}	\N	2020-10-12 14:57:40.136276
44175	FLRT	2020-10-05	47.65	47.65	47.65	47.65	2	{}	\N	2020-10-12 14:57:40.140396
44176	FLRT	2020-10-06	47.68	47.68	47.68	47.68	0	{}	\N	2020-10-12 14:57:40.144922
44177	FLRT	2020-10-07	47.75	47.75	47.75	47.75	0	{}	\N	2020-10-12 14:57:40.149222
44178	FLRT	2020-10-08	47.805	47.805	47.805	47.805	27	{}	\N	2020-10-12 14:57:40.153145
44179	FLRT	2020-10-09	47.7	47.855	47.7	47.855	800	{}	\N	2020-10-12 14:57:40.157174
44180	FMAO	2020-09-28	20.3	20.48	20.05	20.21	5514	{}	\N	2020-10-12 14:57:41.128426
44181	FMAO	2020-09-29	20.01	20.55	20.01	20.55	1977	{}	\N	2020-10-12 14:57:41.138063
44182	FMAO	2020-09-30	20.2	20.2	20.01	20.01	2556	{}	\N	2020-10-12 14:57:41.145213
44183	FMAO	2020-10-01	20.14	20.2758	20.02	20.02	8131	{}	\N	2020-10-12 14:57:41.151571
44184	FMAO	2020-10-02	19.85	20.5	19.85	20.21	5106	{}	\N	2020-10-12 14:57:41.156583
44185	FMAO	2020-10-05	20.475	20.6	20.17	20.28	5841	{}	\N	2020-10-12 14:57:41.161255
44186	FMAO	2020-10-06	20.55	20.88	20.15	20.15	10049	{}	\N	2020-10-12 14:57:41.165487
44187	FMAO	2020-10-07	20.75	20.81	20.3201	20.56	17557	{}	\N	2020-10-12 14:57:41.169717
44188	FMAO	2020-10-08	20.83	20.83	20.29	20.29	4873	{}	\N	2020-10-12 14:57:41.173845
44189	FMAO	2020-10-09	20.63	20.63	20.2	20.2	4940	{}	\N	2020-10-12 14:57:41.178167
44190	FMAO	2020-10-12	20.52	20.52	20.0344	20.2171	2905	{}	\N	2020-10-12 14:57:41.18276
44191	FMNB	2020-09-28	10.71	11.24	10.6522	11.15	97768	{}	\N	2020-10-12 14:57:43.035448
44192	FMNB	2020-09-29	11.12	11.22	10.64	10.87	56739	{}	\N	2020-10-12 14:57:43.048754
44193	FMNB	2020-09-30	10.87	11.06	10.84	10.92	62947	{}	\N	2020-10-12 14:57:43.05382
44194	FMNB	2020-10-01	11.0523	11.1	10.88	11.065	44266	{}	\N	2020-10-12 14:57:43.058472
44195	FMNB	2020-10-02	10.965	11.46	10.5465	11.42	108929	{}	\N	2020-10-12 14:57:43.065473
44196	FMNB	2020-10-05	11.41	11.63	10.9104	11.61	78401	{}	\N	2020-10-12 14:57:43.069723
44197	FMNB	2020-10-06	11.67	12	11.55	11.61	63079	{}	\N	2020-10-12 14:57:43.073654
44198	FMNB	2020-10-07	11.7	11.85	11.6	11.78	56037	{}	\N	2020-10-12 14:57:43.079308
44199	FMNB	2020-10-08	11.84	11.885	11.57	11.72	54707	{}	\N	2020-10-12 14:57:43.083791
44200	FMNB	2020-10-09	11.84	11.84	11.63	11.71	16722	{}	\N	2020-10-12 14:57:43.08795
44201	FMNB	2020-10-12	11.71	11.8	11.67	11.78	3144	{}	\N	2020-10-12 14:57:43.091844
44202	FNDF	2020-09-28	24.7	24.81	24.7	24.79	941299	{}	\N	2020-10-12 14:57:44.002002
44203	FNDF	2020-09-29	24.74	24.77	24.58	24.675	939274	{}	\N	2020-10-12 14:57:44.008803
44204	FNDF	2020-09-30	24.62	24.79	24.53	24.57	1026803	{}	\N	2020-10-12 14:57:44.013974
44205	FNDF	2020-10-01	24.67	24.6917	24.52	24.65	447438	{}	\N	2020-10-12 14:57:44.019357
44206	FNDF	2020-10-02	24.34	24.71	24.3	24.64	1306112	{}	\N	2020-10-12 14:57:44.023699
44207	FNDF	2020-10-05	25	25.15	24.945	25.12	277724	{}	\N	2020-10-12 14:57:44.029534
44208	FNDF	2020-10-06	25.26	25.3	24.92	24.98	435839	{}	\N	2020-10-12 14:57:44.03481
44209	FNDF	2020-10-07	25.18	25.31	25.16	25.24	309711	{}	\N	2020-10-12 14:57:44.039073
44210	FNDF	2020-10-08	25.36	25.45	25.32	25.43	447673	{}	\N	2020-10-12 14:57:44.043656
44211	FNDF	2020-10-09	25.55	25.58	25.455	25.54	349695	{}	\N	2020-10-12 14:57:44.04908
44212	FNDF	2020-10-12	25.55	25.55	25.51	25.55	63621	{}	\N	2020-10-12 14:57:44.054131
44213	FOE	2020-09-28	12.07	12.67	12.07	12.36	620879	{}	\N	2020-10-12 14:57:45.959448
44214	FOE	2020-09-29	12.53	12.73	12.34	12.46	572792	{}	\N	2020-10-12 14:57:45.966484
44215	FOE	2020-09-30	12.5	12.83	12.34	12.4	606801	{}	\N	2020-10-12 14:57:45.971965
44216	FOE	2020-10-01	12.41	12.64	12.18	12.42	438792	{}	\N	2020-10-12 14:57:45.978332
44217	FOE	2020-10-02	12.14	12.75	12.02	12.5	820623	{}	\N	2020-10-12 14:57:45.983586
44218	FOE	2020-10-05	12.68	13.14	12.68	12.78	446660	{}	\N	2020-10-12 14:57:45.988084
44219	FOE	2020-10-06	13	13.15	12.77	12.78	723834	{}	\N	2020-10-12 14:57:45.992351
44220	FOE	2020-10-07	13.04	13.27	12.92	13.21	702678	{}	\N	2020-10-12 14:57:45.997257
44221	FOE	2020-10-08	13.46	13.6706	13.185	13.27	717719	{}	\N	2020-10-12 14:57:46.00159
44222	FOE	2020-10-09	13.39	13.61	13.13	13.13	439572	{}	\N	2020-10-12 14:57:46.00593
44223	FOE	2020-10-12	13.11	13.33	13.07	13.275	98538	{}	\N	2020-10-12 14:57:46.011188
44224	FORD	2020-09-28	1.41	1.42	1.3826	1.42	59308	{}	\N	2020-10-12 14:57:46.986416
44225	FORD	2020-09-29	1.41	1.42	1.39	1.41	30365	{}	\N	2020-10-12 14:57:46.996899
44226	FORD	2020-09-30	1.4	1.42	1.39	1.39	57631	{}	\N	2020-10-12 14:57:47.006229
44227	FORD	2020-10-01	1.41	1.42	1.39	1.41	54788	{}	\N	2020-10-12 14:57:47.011658
44228	FORD	2020-10-02	1.41	1.45	1.38	1.44	66145	{}	\N	2020-10-12 14:57:47.015898
44229	FORD	2020-10-05	1.46	1.51	1.43	1.47	103551	{}	\N	2020-10-12 14:57:47.02059
44230	FORD	2020-10-06	1.47	1.5	1.44	1.44	70720	{}	\N	2020-10-12 14:57:47.0251
44231	FORD	2020-10-07	1.45	1.52	1.45	1.47	54124	{}	\N	2020-10-12 14:57:47.030256
44232	FORD	2020-10-08	1.48	1.5	1.4501	1.5	32796	{}	\N	2020-10-12 14:57:47.036404
44233	FORD	2020-10-09	1.48	1.73	1.4769	1.63	1075297	{}	\N	2020-10-12 14:57:47.042206
44234	FORD	2020-10-12	1.6417	1.65	1.5401	1.55	115375	{}	\N	2020-10-12 14:57:47.048598
44235	FOSL	2020-09-28	6.22	6.42	6.14	6.21	1047456	{}	\N	2020-10-12 14:57:48.111498
44236	FOSL	2020-09-29	6.2	6.23	5.62	5.78	1677860	{}	\N	2020-10-12 14:57:48.117255
44237	FOSL	2020-09-30	5.84	6.01	5.69	5.74	1173691	{}	\N	2020-10-12 14:57:48.121791
44238	FOSL	2020-10-01	5.91	5.91	5.695	5.77	860166	{}	\N	2020-10-12 14:57:48.127373
44239	FOSL	2020-10-02	5.55	5.95	5.51	5.91	627694	{}	\N	2020-10-12 14:57:48.133249
44240	FOSL	2020-10-05	5.87	6.615	5.87	6.535	3345626	{}	\N	2020-10-12 14:57:48.137732
44241	FOSL	2020-10-06	6.5	6.85	6.28	6.3	1650032	{}	\N	2020-10-12 14:57:48.142724
44242	FOSL	2020-10-07	6.45	7.02	6.33	7	2203426	{}	\N	2020-10-12 14:57:48.149787
44243	FOSL	2020-10-08	7.12	7.22	6.33	6.4	2423591	{}	\N	2020-10-12 14:57:48.15565
44244	FOSL	2020-10-09	6.44	6.92	6.36	6.73	2011399	{}	\N	2020-10-12 14:57:48.161503
44245	FOSL	2020-10-12	6.76	6.879	6.59	6.61	237412	{}	\N	2020-10-12 14:57:48.166109
44246	FPI	2020-09-28	6.57	6.67	6.57	6.65	99186	{}	\N	2020-10-12 14:57:49.637367
44247	FPI	2020-09-29	6.66	6.66	6.52	6.6	127641	{}	\N	2020-10-12 14:57:49.645987
44248	FPI	2020-09-30	6.6	6.76	6.6	6.66	193874	{}	\N	2020-10-12 14:57:49.652406
44249	FPI	2020-10-01	6.66	6.77	6.58	6.76	182001	{}	\N	2020-10-12 14:57:49.657968
44250	FPI	2020-10-02	6.7	6.87	6.6	6.84	274956	{}	\N	2020-10-12 14:57:49.664529
44251	FPI	2020-10-05	6.81	6.82	6.46	6.62	411119	{}	\N	2020-10-12 14:57:49.669769
44252	FPI	2020-10-06	6.62	6.69	6.48	6.58	394779	{}	\N	2020-10-12 14:57:49.674613
44253	FPI	2020-10-07	6.59	6.76	6.51	6.7	203018	{}	\N	2020-10-12 14:57:49.679474
44254	FPI	2020-10-08	6.73	6.85	6.5	6.55	427034	{}	\N	2020-10-12 14:57:49.684904
44255	FPI	2020-10-09	6.62	6.72	6.58	6.64	192714	{}	\N	2020-10-12 14:57:49.689782
44256	FPI	2020-10-12	6.64	6.71	6.55	6.661	27830	{}	\N	2020-10-12 14:57:49.694234
44257	FRA	2020-09-28	11.55	11.58	11.51	11.53	130632	{}	\N	2020-10-12 14:57:50.714239
44258	FRA	2020-09-29	11.53	11.53	11.48	11.48	326264	{}	\N	2020-10-12 14:57:50.722737
44259	FRA	2020-09-30	11.52	11.56	11.5	11.5	299275	{}	\N	2020-10-12 14:57:50.730338
44260	FRA	2020-10-01	11.52	11.6	11.52	11.55	112586	{}	\N	2020-10-12 14:57:50.738342
44261	FRA	2020-10-02	11.52	11.66	11.5011	11.66	184329	{}	\N	2020-10-12 14:57:50.743216
44262	FRA	2020-10-05	11.67	11.755	11.67	11.73	151782	{}	\N	2020-10-12 14:57:50.747819
44263	FRA	2020-10-06	11.73	11.77	11.69	11.7	112598	{}	\N	2020-10-12 14:57:50.752292
44264	FRA	2020-10-07	11.76	11.78	11.73	11.73	222833	{}	\N	2020-10-12 14:57:50.757037
44265	FRA	2020-10-08	11.78	11.85	11.78	11.81	104814	{}	\N	2020-10-12 14:57:50.761896
44266	FRA	2020-10-09	11.85	11.85	11.79	11.83	171732	{}	\N	2020-10-12 14:57:50.766539
44267	FRA	2020-10-12	11.9	11.9	11.822	11.83	93074	{}	\N	2020-10-12 14:57:50.770978
44268	FRI	2020-09-28	21.2269	21.33	21.14	21.275	9396	{}	\N	2020-10-12 14:57:51.745992
44269	FRI	2020-09-29	21.17	21.218	20.9006	21.07	10342	{}	\N	2020-10-12 14:57:51.755186
44270	FRI	2020-09-30	21.14	21.34	20.9265	21.1015	43076	{}	\N	2020-10-12 14:57:51.765669
44271	FRI	2020-10-01	21.2295	21.57	21.11	21.57	69476	{}	\N	2020-10-12 14:57:51.77178
44272	FRI	2020-10-02	21.3818	22.065	21.33	22.06	10324	{}	\N	2020-10-12 14:57:51.776505
44273	FRI	2020-10-05	22.12	22.1889	21.83	22.1101	10759	{}	\N	2020-10-12 14:57:51.781293
44274	FRI	2020-10-06	22.1	22.4	22	22.03	8816	{}	\N	2020-10-12 14:57:51.786779
44275	FRI	2020-10-07	22.2	22.2	21.9635	22.08	23566	{}	\N	2020-10-12 14:57:51.791756
44276	FRI	2020-10-08	22.17	22.485	22.17	22.45	4627	{}	\N	2020-10-12 14:57:51.797421
44277	FRI	2020-10-09	22.3501	22.4999	22.32	22.38	32142	{}	\N	2020-10-12 14:57:51.802026
44278	FRI	2020-10-12	22.27	22.33	22.27	22.33	4291	{}	\N	2020-10-12 14:57:51.806885
44279	FRTA	2020-09-28	11.56	11.94	11.46	11.85	692929	{}	\N	2020-10-12 14:57:53.342899
44280	FRTA	2020-09-29	11.84	12.04	11.63	11.65	509390	{}	\N	2020-10-12 14:57:53.352506
44281	FRTA	2020-09-30	11.7	12.12	11.66	11.82	569866	{}	\N	2020-10-12 14:57:53.358819
44282	FRTA	2020-10-01	11.9	12.15	11.71	12.145	476647	{}	\N	2020-10-12 14:57:53.364904
44283	FRTA	2020-10-02	11.67	12.58	11.45	12.39	559335	{}	\N	2020-10-12 14:57:53.369969
44284	FRTA	2020-10-05	12.5	12.979	12.5	12.78	898473	{}	\N	2020-10-12 14:57:53.374466
44285	FRTA	2020-10-06	12.9	13.41	12.81	13.04	773988	{}	\N	2020-10-12 14:57:53.380566
44286	FRTA	2020-10-07	13.21	13.85	13.17	13.56	836371	{}	\N	2020-10-12 14:57:53.384891
44287	FRTA	2020-10-08	13.75	14.63	13.75	14.5	1177866	{}	\N	2020-10-12 14:57:53.389412
44288	FRTA	2020-10-09	14.84	14.84	14.06	14.23	513816	{}	\N	2020-10-12 14:57:53.394592
44289	FRTA	2020-10-12	14.22	14.68	14.22	14.56	179991	{}	\N	2020-10-12 14:57:53.398832
44290	FOLD	2020-09-28	13.95	14.015	13.68	13.9	2532958	{}	\N	2020-10-12 14:57:54.853078
44291	FOLD	2020-09-29	13.96	14.12	13.59	14.07	1092520	{}	\N	2020-10-12 14:57:54.861168
44292	FOLD	2020-09-30	14.11	14.545	14.05	14.12	1465665	{}	\N	2020-10-12 14:57:54.866306
44293	FOLD	2020-10-01	14.22	14.35	13.85	14.08	1790751	{}	\N	2020-10-12 14:57:54.870672
44294	FOLD	2020-10-02	13.79	14.01	13.47	13.47	1253366	{}	\N	2020-10-12 14:57:54.875844
44295	FOLD	2020-10-05	13.54	14.37	13.49	14.36	1491536	{}	\N	2020-10-12 14:57:54.881352
44296	FOLD	2020-10-06	14.6	14.81	14.34	14.38	2471296	{}	\N	2020-10-12 14:57:54.886042
44297	FOLD	2020-10-07	14.41	14.87	14.18	14.765	1733685	{}	\N	2020-10-12 14:57:54.890504
44298	FOLD	2020-10-08	14.9	14.92	14.42	14.895	1543010	{}	\N	2020-10-12 14:57:54.901746
44299	FOLD	2020-10-09	14.96	15.15	14.66	15.115	1412666	{}	\N	2020-10-12 14:57:54.90666
44300	FOLD	2020-10-12	15.22	15.625	15.115	15.61	388649	{}	\N	2020-10-12 14:57:54.911531
44301	FORM	2020-09-28	24.46	24.79	24.145	24.75	556140	{}	\N	2020-10-12 14:57:55.956803
44302	FORM	2020-09-29	25.0665	25.32	24.7201	25.04	349512	{}	\N	2020-10-12 14:57:55.964254
44303	FORM	2020-09-30	24.96	25.33	24.8	24.93	500109	{}	\N	2020-10-12 14:57:55.970197
44304	FORM	2020-10-01	25.33	25.99	25.26	25.89	508499	{}	\N	2020-10-12 14:57:55.975587
44305	FORM	2020-10-02	24.99	25.66	24.89	25.25	493667	{}	\N	2020-10-12 14:57:55.981446
44306	FORM	2020-10-05	25.66	26.1	25.38	26.07	352345	{}	\N	2020-10-12 14:57:55.986013
44307	FORM	2020-10-06	26.38	26.81	25.895	26.24	395924	{}	\N	2020-10-12 14:57:55.991111
44308	FORM	2020-10-07	26.59	26.74	26.315	26.495	383593	{}	\N	2020-10-12 14:57:55.995827
44309	FORM	2020-10-08	26.8	27.53	26.65	27.49	445891	{}	\N	2020-10-12 14:57:56.000139
44310	FORM	2020-10-09	27.86	29.32	27.86	29.08	697111	{}	\N	2020-10-12 14:57:56.004406
44311	FORM	2020-10-12	29.19	29.3961	28.83	28.92	60663	{}	\N	2020-10-12 14:57:56.008855
44312	FR	2020-09-28	40.25	40.83	40.13	40.78	1035228	{}	\N	2020-10-12 14:57:57.061676
44313	FR	2020-09-29	40.38	40.63	39.95	40.26	1760349	{}	\N	2020-10-12 14:57:57.071856
44314	FR	2020-09-30	40.47	40.55	39.36	39.8	2501430	{}	\N	2020-10-12 14:57:57.080522
44315	FR	2020-10-01	40.12	40.71	39.7	40.7	2534752	{}	\N	2020-10-12 14:57:57.086732
44316	FR	2020-10-02	40.12	42	39.88	41.86	1644566	{}	\N	2020-10-12 14:57:57.091964
44317	FR	2020-10-05	42.07	43.11	41.74	42.92	1729117	{}	\N	2020-10-12 14:57:57.096828
44318	FR	2020-10-06	43.11	43.38	42.42	42.67	1479984	{}	\N	2020-10-12 14:57:57.101299
44319	FR	2020-10-07	42.86	42.91	42.5	42.64	909112	{}	\N	2020-10-12 14:57:57.105716
44320	FR	2020-10-08	42.85	43.49	42.76	43.36	753621	{}	\N	2020-10-12 14:57:57.110297
44321	FR	2020-10-09	43.74	43.8	43.01	43.49	795443	{}	\N	2020-10-12 14:57:57.115081
44322	FR	2020-10-12	43.64	43.89	43.34	43.73	145131	{}	\N	2020-10-12 14:57:57.120082
44323	FRO	2020-09-28	6.5	6.5151	6.29	6.37	2143998	{}	\N	2020-10-12 14:57:59.128593
44324	FRO	2020-09-29	6.37	6.53	6.21	6.34	5611869	{}	\N	2020-10-12 14:57:59.14108
44325	FRO	2020-09-30	6.35	6.7	6.35	6.5	4299206	{}	\N	2020-10-12 14:57:59.148884
44326	FRO	2020-10-01	6.57	6.615	6.43	6.51	3364288	{}	\N	2020-10-12 14:57:59.15587
44327	FRO	2020-10-02	6.48	6.59	6.39	6.52	3655315	{}	\N	2020-10-12 14:57:59.161777
44328	FRO	2020-10-05	6.68	6.85	6.62	6.78	3149580	{}	\N	2020-10-12 14:57:59.166464
44329	FRO	2020-10-06	6.8	6.88	6.67	6.68	2242679	{}	\N	2020-10-12 14:57:59.171916
44330	FRO	2020-10-07	6.58	6.685	6.47	6.63	3789837	{}	\N	2020-10-12 14:57:59.178615
44331	FRO	2020-10-08	6.7	6.9	6.67	6.86	2732818	{}	\N	2020-10-12 14:57:59.185071
44332	FRO	2020-10-09	6.95	6.97	6.735	6.76	2200312	{}	\N	2020-10-12 14:57:59.191319
44333	FRO	2020-10-12	6.79	6.799	6.62	6.72	1425087	{}	\N	2020-10-12 14:57:59.198574
44334	FSLR	2020-09-28	64.301	65.58	63.9301	65.37	1722813	{}	\N	2020-10-12 14:58:00.620152
44335	FSLR	2020-09-29	65.5	66	64.7	65.76	1651186	{}	\N	2020-10-12 14:58:00.625309
44336	FSLR	2020-09-30	65.795	68.14	65.65	66.2	2792881	{}	\N	2020-10-12 14:58:00.630969
44337	FSLR	2020-10-01	67.27	68.88	66.78	68.57	2209950	{}	\N	2020-10-12 14:58:00.635886
44338	FSLR	2020-10-02	67.01	69.32	66.71	67.21	1157518	{}	\N	2020-10-12 14:58:00.64064
44339	FSLR	2020-10-05	68.55	71.78	68.29	71.5	2347401	{}	\N	2020-10-12 14:58:00.646661
44340	FSLR	2020-10-06	72.575	75.57	71.8	73.44	2375668	{}	\N	2020-10-12 14:58:00.651309
44341	FSLR	2020-10-07	75.5	81.24	75	78.71	5304697	{}	\N	2020-10-12 14:58:00.655611
44342	FSLR	2020-10-08	80.5	80.6	76.5	77.12	3184679	{}	\N	2020-10-12 14:58:00.660586
44343	FSLR	2020-10-09	77.95	81.43	76.88	78.81	2991507	{}	\N	2020-10-12 14:58:00.66487
44344	FSLR	2020-10-12	81.97	83.3	77.31	77.69	1733258	{}	\N	2020-10-12 14:58:00.671313
44345	FT	2020-09-28	6.87	6.875	6.82	6.84	65219	{}	\N	2020-10-12 14:58:02.017432
44346	FT	2020-09-29	6.82	6.855	6.815	6.84	66908	{}	\N	2020-10-12 14:58:02.024509
44347	FT	2020-09-30	6.85	6.87	6.82	6.82	102250	{}	\N	2020-10-12 14:58:02.030649
44348	FT	2020-10-01	6.84	6.8998	6.83	6.85	101754	{}	\N	2020-10-12 14:58:02.036876
44349	FT	2020-10-02	6.76	6.8554	6.76	6.85	106828	{}	\N	2020-10-12 14:58:02.045534
44350	FT	2020-10-05	6.8898	6.8999	6.87	6.88	124892	{}	\N	2020-10-12 14:58:02.067834
44351	FT	2020-10-06	6.88	6.98	6.88	6.94	141099	{}	\N	2020-10-12 14:58:02.101466
44352	FT	2020-10-07	6.97	7.004	6.96	6.9927	64822	{}	\N	2020-10-12 14:58:02.109688
44353	FT	2020-10-08	6.98	7.02	6.98	7	393018	{}	\N	2020-10-12 14:58:02.11769
44354	FT	2020-10-09	7.02	7.04	7.01	7.01	37553	{}	\N	2020-10-12 14:58:02.125058
44355	FT	2020-10-12	7.03	7.0499	7.03	7.04	11304	{}	\N	2020-10-12 14:58:02.131928
44356	FTF	2020-09-28	8.83	8.97	8.83	8.84	79607	{}	\N	2020-10-12 14:58:03.552558
44357	FTF	2020-09-29	8.81	8.8216	8.53	8.55	134824	{}	\N	2020-10-12 14:58:03.560188
44358	FTF	2020-09-30	8.53	8.5839	8.5	8.53	221528	{}	\N	2020-10-12 14:58:03.566417
44359	FTF	2020-10-01	8.53	8.64	8.36	8.6	87987	{}	\N	2020-10-12 14:58:03.572166
44360	FTF	2020-10-02	8.58	8.63	8.5	8.59	48958	{}	\N	2020-10-12 14:58:03.577334
44361	FTF	2020-10-05	8.58	8.665	8.52	8.57	93664	{}	\N	2020-10-12 14:58:03.581772
44362	FTF	2020-10-06	8.59	8.67	8.57	8.63	129048	{}	\N	2020-10-12 14:58:03.586167
44363	FTF	2020-10-07	8.6299	8.65	8.56	8.61	127570	{}	\N	2020-10-12 14:58:03.590174
44364	FTF	2020-10-08	8.57	8.63	8.57	8.62	150598	{}	\N	2020-10-12 14:58:03.594251
44365	FTF	2020-10-09	8.63	8.6578	8.59	8.655	44494	{}	\N	2020-10-12 14:58:03.598816
44366	FTF	2020-10-12	8.67	8.67	8.6	8.64	30982	{}	\N	2020-10-12 14:58:03.60281
44367	FTXG	2020-09-28	20.7954	20.7954	20.76	20.76	1133	{}	\N	2020-10-12 14:58:05.59886
44368	FTXG	2020-09-29	20.72	20.72	20.6685	20.6685	1058	{}	\N	2020-10-12 14:58:05.607025
44369	FTXG	2020-10-01	21.0613	21.0613	21.0613	21.0613	333	{}	\N	2020-10-12 14:58:05.612443
44370	FTXG	2020-10-06	21.37	21.4497	21.3556	21.3556	1514	{}	\N	2020-10-12 14:58:05.617496
44371	FTXG	2020-10-07	21.431	21.6	21.431	21.55	977	{}	\N	2020-10-12 14:58:05.622353
44372	FTXG	2020-10-08	21.41	21.7	21.41	21.68	2712	{}	\N	2020-10-12 14:58:05.626657
44373	FUL	2020-09-28	46.65	47.44	46.225	46.74	419610	{}	\N	2020-10-12 14:58:06.882689
44374	FUL	2020-09-29	46.84	47.06	45.68	45.84	854943	{}	\N	2020-10-12 14:58:06.893893
44375	FUL	2020-09-30	45.9	47.03	45.41	45.78	1026627	{}	\N	2020-10-12 14:58:06.900851
44376	FUL	2020-10-01	46.09	46.545	44.54	44.82	676528	{}	\N	2020-10-12 14:58:06.907316
44377	FUL	2020-10-02	43.93	46.63	43.91	46.21	744326	{}	\N	2020-10-12 14:58:06.913017
44378	FUL	2020-10-05	46.6	47.93	46.6	47.76	710862	{}	\N	2020-10-12 14:58:06.918102
44379	FUL	2020-10-06	48.15	49.51	47.81	48.07	796816	{}	\N	2020-10-12 14:58:06.92251
44380	FUL	2020-10-07	48.73	49.4	48.405	49.14	675113	{}	\N	2020-10-12 14:58:06.92698
44381	FUL	2020-10-08	49.69	50.37	49.53	49.99	566679	{}	\N	2020-10-12 14:58:06.931594
44382	FUL	2020-10-09	50.5	50.55	49.06	49.42	656118	{}	\N	2020-10-12 14:58:06.935742
44383	FUL	2020-10-12	49.5	49.8	49.27	49.435	67458	{}	\N	2020-10-12 14:58:06.940011
44384	FVAL	2020-09-28	35.48	35.62	35.4	35.49	217369	{}	\N	2020-10-12 14:58:07.963242
44385	FVAL	2020-09-29	35.485	35.55	35.215	35.33	15261	{}	\N	2020-10-12 14:58:07.971946
44386	FVAL	2020-09-30	35.43	35.9019	35.38	35.62	21267	{}	\N	2020-10-12 14:58:07.977421
44387	FVAL	2020-10-01	35.79	35.85	35.57	35.72	34674	{}	\N	2020-10-12 14:58:07.98271
44388	FVAL	2020-10-02	35.18	35.83	35	35.7	25281	{}	\N	2020-10-12 14:58:07.988033
44389	FVAL	2020-10-05	35.94	36.23	35.9	36.23	30105	{}	\N	2020-10-12 14:58:07.992131
44390	FVAL	2020-10-06	36.22	36.55	35.71	35.8	20492	{}	\N	2020-10-12 14:58:07.996575
44391	FVAL	2020-10-07	35.94	36.49	35.94	36.46	16025	{}	\N	2020-10-12 14:58:08.000925
44392	FVAL	2020-10-08	36.55	36.79	36.55	36.79	46267	{}	\N	2020-10-12 14:58:08.005261
44393	FVAL	2020-10-09	37.01	37.07	36.84	36.95	37067	{}	\N	2020-10-12 14:58:08.009402
44394	FVAL	2020-10-12	37.15	37.33	37.15	37.295	8179	{}	\N	2020-10-12 14:58:08.013512
44395	FXC	2020-09-28	73.6	73.67	73.51	73.58	28256	{}	\N	2020-10-12 14:58:08.983854
44396	FXC	2020-09-29	73.596	73.61	73.37	73.51	20042	{}	\N	2020-10-12 14:58:08.991454
44397	FXC	2020-09-30	73.58	73.98	73.54	73.94	13259	{}	\N	2020-10-12 14:58:08.997059
44398	FXC	2020-10-01	73.9904	74.19	73.99	74.15	13182	{}	\N	2020-10-12 14:58:09.002563
44399	FXC	2020-10-02	73.87	74.0169	73.87	73.97	18103	{}	\N	2020-10-12 14:58:09.007755
44400	FXC	2020-10-05	74.17	74.24	74.14	74.23	9513	{}	\N	2020-10-12 14:58:09.011936
44401	FXC	2020-10-06	74.227	74.27	73.93	74.04	26303	{}	\N	2020-10-12 14:58:09.016114
44402	FXC	2020-10-07	74.169	74.2101	74.0953	74.2	4937	{}	\N	2020-10-12 14:58:09.020806
44403	FXC	2020-10-08	74.34	74.61	74.34	74.57	10590	{}	\N	2020-10-12 14:58:09.025164
44404	FXC	2020-10-09	74.9	75.035	74.8483	74.98	27955	{}	\N	2020-10-12 14:58:09.029535
44405	FXC	2020-10-12	75.06	75.1	75.05	75.1	6693	{}	\N	2020-10-12 14:58:09.036643
44406	MVT	2020-09-28	13.97	14.04	13.88	13.94	28800	{}	\N	2020-10-12 14:58:10.200853
44407	MVT	2020-09-29	13.92	14.01	13.87	14	74554	{}	\N	2020-10-12 14:58:10.2113
44408	MVT	2020-09-30	13.98	14.16	13.91	13.94	57144	{}	\N	2020-10-12 14:58:10.217915
44409	MVT	2020-10-01	14.06	14.1	13.97	14.02	51848	{}	\N	2020-10-12 14:58:10.224914
44410	MVT	2020-10-02	14.04	14.16	14.04	14.14	25300	{}	\N	2020-10-12 14:58:10.230117
44411	MVT	2020-10-05	14.14	14.42	13.95	14.04	25431	{}	\N	2020-10-12 14:58:10.234979
44412	MVT	2020-10-06	14.09	14.15	13.96	14.07	22210	{}	\N	2020-10-12 14:58:10.239772
44413	MVT	2020-10-07	14.13	14.23	13.97	14.17	27290	{}	\N	2020-10-12 14:58:10.24454
44414	MVT	2020-10-08	14.23	14.26	13.995	14.23	21122	{}	\N	2020-10-12 14:58:10.248969
44415	MVT	2020-10-09	14.17	14.31	14	14.19	37442	{}	\N	2020-10-12 14:58:10.253569
44416	MVT	2020-10-12	14.26	14.26	14.225	14.225	1522	{}	\N	2020-10-12 14:58:10.258268
44417	FOR	2020-09-28	18.13	18.54	18.13	18.49	88542	{}	\N	2020-10-12 14:58:11.247749
44418	FOR	2020-09-29	18.6	18.73	17.92	18.12	142136	{}	\N	2020-10-12 14:58:11.255764
44419	FOR	2020-09-30	18.04	18.26	17.5204	17.7	300730	{}	\N	2020-10-12 14:58:11.261752
44420	FOR	2020-10-01	17.62	17.98	17.45	17.93	356782	{}	\N	2020-10-12 14:58:11.267277
44421	FOR	2020-10-02	17.47	18.73	17.47	18.57	154439	{}	\N	2020-10-12 14:58:11.27235
44422	FOR	2020-10-05	18.71	18.71	17.99	18.48	186615	{}	\N	2020-10-12 14:58:11.277017
44423	FOR	2020-10-06	18.69	18.95	18.34	18.46	166908	{}	\N	2020-10-12 14:58:11.281227
44424	FOR	2020-10-07	18.69	18.72	18.29	18.5	375740	{}	\N	2020-10-12 14:58:11.285135
44425	FOR	2020-10-08	18.81	19.07	18.55	18.99	204735	{}	\N	2020-10-12 14:58:11.288856
44426	FOR	2020-10-09	18.99	19.1	18.73	18.8	139076	{}	\N	2020-10-12 14:58:11.292867
44427	FOR	2020-10-12	18.94	18.99	18.87	18.99	3980	{}	\N	2020-10-12 14:58:11.29672
44428	FPE	2020-09-28	19.06	19.13	19.015	19.13	1420822	{}	\N	2020-10-12 14:58:12.298414
44429	FPE	2020-09-29	19.14	19.16	19.1042	19.12	2240520	{}	\N	2020-10-12 14:58:12.307477
44430	FPE	2020-09-30	19.17	19.21	19.1458	19.17	2300792	{}	\N	2020-10-12 14:58:12.313928
44431	FPE	2020-10-01	19.22	19.28	19.1712	19.27	2304602	{}	\N	2020-10-12 14:58:12.319374
44432	FPE	2020-10-02	19.21	19.26	19.18	19.26	1444892	{}	\N	2020-10-12 14:58:12.324413
44433	FPE	2020-10-05	19.28	19.32	19.21	19.31	2473583	{}	\N	2020-10-12 14:58:12.32899
44434	FPE	2020-10-06	19.39	19.39	19.29	19.33	1258171	{}	\N	2020-10-12 14:58:12.33285
44435	FPE	2020-10-07	19.33	19.38	19.33	19.38	896347	{}	\N	2020-10-12 14:58:12.336974
44436	FPE	2020-10-08	19.4	19.41	19.345	19.41	691067	{}	\N	2020-10-12 14:58:12.341469
44437	FPE	2020-10-09	19.44	19.44	19.39	19.42	1078444	{}	\N	2020-10-12 14:58:12.346292
44438	FPE	2020-10-12	19.44	19.44	19.4	19.4064	111600	{}	\N	2020-10-12 14:58:12.351699
44439	FRME	2020-09-28	22.97	23.59	22.97	23.42	189911	{}	\N	2020-10-12 14:58:13.577358
44440	FRME	2020-09-29	22.95	23.29	22.83	23.17	222502	{}	\N	2020-10-12 14:58:13.587743
44441	FRME	2020-09-30	23.2	23.77	22.98	23.16	180085	{}	\N	2020-10-12 14:58:13.594449
44442	FRME	2020-10-01	23.36	23.54	22.89	23.49	245940	{}	\N	2020-10-12 14:58:13.599948
44443	FRME	2020-10-02	23.19	24.53	23.19	24.34	201112	{}	\N	2020-10-12 14:58:13.60462
44444	FRME	2020-10-05	24.62	25.16	24.13	25.09	152169	{}	\N	2020-10-12 14:58:13.608905
44445	FRME	2020-10-06	25.5	26.32	25.0532	25.31	175861	{}	\N	2020-10-12 14:58:13.613122
44446	FRME	2020-10-07	25.67	26.145	25.37	25.92	186620	{}	\N	2020-10-12 14:58:13.616834
44447	FRME	2020-10-08	26.23	26.48	25.79	26.12	137201	{}	\N	2020-10-12 14:58:13.620338
44448	FRME	2020-10-09	26.4	26.4	25.59	25.65	100460	{}	\N	2020-10-12 14:58:13.623801
44449	FRME	2020-10-12	25.56	26.02	25.55	25.96	85244	{}	\N	2020-10-12 14:58:13.627391
44450	FSM	2020-09-28	6.39	6.43	6.185	6.35	3595493	{}	\N	2020-10-12 14:58:16.114523
44451	FSM	2020-09-29	6.4	6.63	6.39	6.48	7737862	{}	\N	2020-10-12 14:58:16.1244
44452	FSM	2020-09-30	6.4	6.5302	6.245	6.36	7345891	{}	\N	2020-10-12 14:58:16.131648
44453	FSM	2020-10-01	6.48	6.555	6.31	6.5	5860522	{}	\N	2020-10-12 14:58:16.137901
44454	FSM	2020-10-02	6.42	6.5411	6.32	6.37	5090335	{}	\N	2020-10-12 14:58:16.142707
44455	FSM	2020-10-05	6.44	6.655	6.43	6.57	5840353	{}	\N	2020-10-12 14:58:16.147946
44456	FSM	2020-10-06	6.57	6.625	6.15	6.17	6495848	{}	\N	2020-10-12 14:58:16.152108
44457	FSM	2020-10-07	6.33	6.405	6.2077	6.25	4487406	{}	\N	2020-10-12 14:58:16.156245
44458	FSM	2020-10-08	6.37	6.505	6.295	6.42	4651665	{}	\N	2020-10-12 14:58:16.160384
44459	FSM	2020-10-09	6.7	7.09	6.62	7	7655797	{}	\N	2020-10-12 14:58:16.164408
44460	FSM	2020-10-12	7.04	7.08	6.84	6.96	1462037	{}	\N	2020-10-12 14:58:16.168318
44461	FSZ	2020-09-28	53.12	53.355	53.12	53.355	1176	{}	\N	2020-10-12 14:58:17.720146
44462	FSZ	2020-09-29	53.48	53.56	53.48	53.56	1296	{}	\N	2020-10-12 14:58:17.728439
44463	FSZ	2020-09-30	53.68	54.1925	53.68	53.81	8969	{}	\N	2020-10-12 14:58:17.734186
44464	FSZ	2020-10-01	53.999	54.26	53.999	54.12	3275	{}	\N	2020-10-12 14:58:17.73954
44465	FSZ	2020-10-05	54.76	54.98	54.71	54.98	3499	{}	\N	2020-10-12 14:58:17.744549
44466	FSZ	2020-10-06	54.62	54.62	54.13	54.13	370	{}	\N	2020-10-12 14:58:17.749177
44467	FSZ	2020-10-07	54.44	54.44	54.14	54.4	9856	{}	\N	2020-10-12 14:58:17.753453
44468	FSZ	2020-10-08	54.7	54.93	54.665	54.665	2377	{}	\N	2020-10-12 14:58:17.757633
44469	FSZ	2020-10-09	55.2	55.2571	55.04	55.2571	3190	{}	\N	2020-10-12 14:58:17.761957
44470	FTEC	2020-09-28	91.56	91.68	90.68	91.59	340986	{}	\N	2020-10-12 14:58:18.713529
44471	FTEC	2020-09-29	91.5	91.94	91.27	91.37	192056	{}	\N	2020-10-12 14:58:18.719353
44472	FTEC	2020-09-30	91.37	92.99	91.37	92.06	378586	{}	\N	2020-10-12 14:58:18.72539
44473	FTEC	2020-10-01	93.27	93.47	92.6233	93.12	275880	{}	\N	2020-10-12 14:58:18.730258
44474	FTEC	2020-10-02	91	92.54	90.68	91.06	366621	{}	\N	2020-10-12 14:58:18.73518
44475	FTEC	2020-10-05	91.75	93.1	91.705	93.07	344900	{}	\N	2020-10-12 14:58:18.739837
44476	FTEC	2020-10-06	92.99	93.9	91.545	91.84	255088	{}	\N	2020-10-12 14:58:18.744526
44477	FTEC	2020-10-07	92.79	93.73	92.79	93.54	258126	{}	\N	2020-10-12 14:58:18.749118
44478	FTEC	2020-10-08	94.3	94.47	93.73	93.98	262395	{}	\N	2020-10-12 14:58:18.753672
44479	FTEC	2020-10-09	94.51	95.53	94.5	95.5	317110	{}	\N	2020-10-12 14:58:18.758099
44480	FTEC	2020-10-12	96.95	97.19	96.4	97.128	134992	{}	\N	2020-10-12 14:58:18.762863
44481	FTV	2020-09-28	75.79	76.57	75.055	75.73	1914210	{}	\N	2020-10-12 14:58:20.731891
44482	FTV	2020-09-29	75.77	76.94	75.28	76.21	6228432	{}	\N	2020-10-12 14:58:20.743727
44483	FTV	2020-09-30	76.7	77.34	75.585	76.21	4951972	{}	\N	2020-10-12 14:58:20.751532
44484	FTV	2020-10-01	76.84	77.03	74.79	75.34	4105362	{}	\N	2020-10-12 14:58:20.758351
44485	FTV	2020-10-02	74.54	78.57	74.48	77.4	3690230	{}	\N	2020-10-12 14:58:20.764062
44486	FTV	2020-10-05	78.72	79.145	77.83	79.06	6289982	{}	\N	2020-10-12 14:58:20.768905
44487	FTV	2020-10-06	79.9	80.645	78.41	78.65	5013575	{}	\N	2020-10-12 14:58:20.77444
44488	FTV	2020-10-07	79.25	81.48	79.25	81.04	6612280	{}	\N	2020-10-12 14:58:20.779415
44489	FTV	2020-10-08	81.47	82.12	80.785	81.79	6044021	{}	\N	2020-10-12 14:58:20.784242
44490	FTV	2020-10-09	70	71.02	69.3	69.82	27091944	{}	\N	2020-10-12 14:58:20.7896
44491	FTV	2020-10-12	69.75	69.99	67.98	68.05	2159863	{}	\N	2020-10-12 14:58:20.794361
44492	FTXN	2020-09-28	9.74	9.8537	9.64	9.845	8691	{}	\N	2020-10-12 14:58:21.832926
44493	FTXN	2020-09-29	9.531	9.545	9.36	9.48	12801	{}	\N	2020-10-12 14:58:21.844435
44494	FTXN	2020-09-30	9.51	9.52	9.265	9.265	7057	{}	\N	2020-10-12 14:58:21.851048
44495	FTXN	2020-10-01	9.2774	9.2774	9.04	9.04	1210	{}	\N	2020-10-12 14:58:21.856919
44496	FTXN	2020-10-02	8.88	9.221	8.88	9.16	29885	{}	\N	2020-10-12 14:58:21.861702
44497	FTXN	2020-10-05	9.27	9.5	9.27	9.5	4026	{}	\N	2020-10-12 14:58:21.866911
44498	FTXN	2020-10-06	9.731	9.7383	9.3883	9.3883	3485	{}	\N	2020-10-12 14:58:21.871445
44499	FTXN	2020-10-07	9.55	9.6778	9.5471	9.66	9288	{}	\N	2020-10-12 14:58:21.875954
44500	FTXN	2020-10-08	9.68	10.03	9.68	10.03	114955	{}	\N	2020-10-12 14:58:21.88015
44501	FTXN	2020-10-09	10.17	10.17	9.89	9.96	6445	{}	\N	2020-10-12 14:58:21.884315
44502	FTXN	2020-10-12	9.91	9.91	9.9	9.9	225	{}	\N	2020-10-12 14:58:21.888191
44503	FUV	2020-09-28	6.34	6.54	6.2	6.49	542615	{}	\N	2020-10-12 14:58:22.914559
44504	FUV	2020-09-29	6.31	6.84	6.3	6.65	1852729	{}	\N	2020-10-12 14:58:22.9278
44505	FUV	2020-09-30	6.56	6.82	6.3617	6.58	520265	{}	\N	2020-10-12 14:58:22.935353
44506	FUV	2020-10-01	6.78	6.98	6.5101	6.71	747008	{}	\N	2020-10-12 14:58:22.94071
44507	FUV	2020-10-02	6.3	6.59	6.03	6.2	780015	{}	\N	2020-10-12 14:58:22.945179
44508	FUV	2020-10-05	6.29	6.69	6.25	6.36	784240	{}	\N	2020-10-12 14:58:22.949499
44509	FUV	2020-10-06	6.35	6.35	6.05	6.12	381943	{}	\N	2020-10-12 14:58:22.953939
44510	FUV	2020-10-07	6.21	6.525	6.2	6.28	448888	{}	\N	2020-10-12 14:58:22.957988
44511	FUV	2020-10-08	6.345	6.43	6.265	6.34	285500	{}	\N	2020-10-12 14:58:22.961326
44512	FUV	2020-10-09	6.33	6.49	6.21	6.37	369000	{}	\N	2020-10-12 14:58:22.964763
44513	FUV	2020-10-12	6.46	6.94	6.41	6.9	375690	{}	\N	2020-10-12 14:58:22.96843
44514	FWONK	2020-09-28	36	36.32	35.65	35.72	731160	{}	\N	2020-10-12 14:58:24.156178
44515	FWONK	2020-09-29	35.74	35.85	34.96	35.37	552036	{}	\N	2020-10-12 14:58:24.164888
44516	FWONK	2020-09-30	35.51	37.28	35.41	36.27	2060892	{}	\N	2020-10-12 14:58:24.170535
44517	FWONK	2020-10-01	36.75	37.59	36.49	37.15	1287157	{}	\N	2020-10-12 14:58:24.176871
44518	FWONK	2020-10-02	36.43	37.61	36.06	37.07	557033	{}	\N	2020-10-12 14:58:24.182011
44519	FWONK	2020-10-05	37.09	37.7025	36.84	37.49	711225	{}	\N	2020-10-12 14:58:24.186409
44520	FWONK	2020-10-06	37.82	38.09	37.34	37.54	715316	{}	\N	2020-10-12 14:58:24.190863
44521	FWONK	2020-10-07	38	38.24	37.12	37.81	1232063	{}	\N	2020-10-12 14:58:24.195327
44522	FWONK	2020-10-08	38.11	38.23	37.38	37.59	341428	{}	\N	2020-10-12 14:58:24.199552
44523	FWONK	2020-10-09	37.8356	37.85	37.11	37.22	153689	{}	\N	2020-10-12 14:58:24.203717
44524	FWONK	2020-10-12	37.09	37.5	37.0069	37.23	24870	{}	\N	2020-10-12 14:58:24.208456
44525	FPRX	2020-09-28	4.52	4.81	4.52	4.56	451399	{}	\N	2020-10-12 14:58:25.43217
44526	FPRX	2020-09-29	4.57	4.7793	4.51	4.58	207307	{}	\N	2020-10-12 14:58:25.443509
44527	FPRX	2020-09-30	4.57	4.88	4.57	4.7	329929	{}	\N	2020-10-12 14:58:25.45078
44528	FPRX	2020-10-01	4.76	5.02	4.7	4.92	283191	{}	\N	2020-10-12 14:58:25.457243
44529	FPRX	2020-10-02	4.75	4.98	4.65	4.87	398317	{}	\N	2020-10-12 14:58:25.462181
44530	FPRX	2020-10-05	4.89	5.05	4.81	4.86	405616	{}	\N	2020-10-12 14:58:25.467127
44531	FPRX	2020-10-06	4.91	5.1552	4.775	5.02	1005725	{}	\N	2020-10-12 14:58:25.471329
44532	FPRX	2020-10-07	5.0501	5.81	5.05	5.46	555848	{}	\N	2020-10-12 14:58:25.475978
44533	FPRX	2020-10-08	5.51	5.55	5	5.12	425164	{}	\N	2020-10-12 14:58:25.480581
44534	FPRX	2020-10-09	5.65	6.1	5.3	5.38	1031009	{}	\N	2020-10-12 14:58:25.484743
44535	FPRX	2020-10-12	5.48	5.99	5.41	5.975	173356	{}	\N	2020-10-12 14:58:25.488887
44536	FRD	2020-09-28	5.87	6.085	5.68	5.81	20300	{}	\N	2020-10-12 14:58:27.126343
44537	FRD	2020-09-29	5.9125	6	5.9125	5.92	4312	{}	\N	2020-10-12 14:58:27.132363
44538	FRD	2020-09-30	5.8	5.91	5.8	5.8	4578	{}	\N	2020-10-12 14:58:27.136566
44539	FRD	2020-10-01	5.85	6	5.8	5.8	8508	{}	\N	2020-10-12 14:58:27.1407
44540	FRD	2020-10-02	5.88	5.95	5.74	5.74	6942	{}	\N	2020-10-12 14:58:27.14524
44541	FRD	2020-10-05	5.85	5.85	5.71	5.75	8091	{}	\N	2020-10-12 14:58:27.149971
44542	FRD	2020-10-06	5.71	5.91	5.7	5.7	7142	{}	\N	2020-10-12 14:58:27.154447
44543	FRD	2020-10-07	5.97	5.97	5.81	5.81	6339	{}	\N	2020-10-12 14:58:27.158661
44544	FRD	2020-10-08	5.95	5.95	5.86	5.86	3110	{}	\N	2020-10-12 14:58:27.163409
44545	FRD	2020-10-09	5.9	5.96	5.9	5.95	2094	{}	\N	2020-10-12 14:58:27.167838
44546	FRD	2020-10-12	5.91	5.91	5.9	5.9	2360	{}	\N	2020-10-12 14:58:27.171642
44547	FRBK	2020-09-28	2.05	2.1352	2.03	2.07	135511	{}	\N	2020-10-12 14:58:28.683399
44548	FRBK	2020-09-29	2.07	2.08	2.032	2.08	25056	{}	\N	2020-10-12 14:58:28.692291
44549	FRBK	2020-09-30	2.0847	2.1	1.96	1.98	339307	{}	\N	2020-10-12 14:58:28.697744
44550	FRBK	2020-10-01	2	2.05	1.98	2.035	135967	{}	\N	2020-10-12 14:58:28.702975
44551	FRBK	2020-10-02	2	2.12	1.9552	2.09	199437	{}	\N	2020-10-12 14:58:28.708346
44552	FRBK	2020-10-05	2.09	2.1175	2.06	2.1	69035	{}	\N	2020-10-12 14:58:28.712656
44553	FRBK	2020-10-06	2.1	2.15	2.05	2.07	129511	{}	\N	2020-10-12 14:58:28.71644
44554	FRBK	2020-10-07	2.15	2.15	2.09	2.13	82098	{}	\N	2020-10-12 14:58:28.720357
44555	FRBK	2020-10-08	2.15	2.19	2.04	2.17	68856	{}	\N	2020-10-12 14:58:28.724262
44556	FRBK	2020-10-09	2.19	2.23	2.14	2.22	74682	{}	\N	2020-10-12 14:58:28.735686
44557	FRBK	2020-10-12	2.22	2.25	2.19	2.23	10625	{}	\N	2020-10-12 14:58:28.739878
44558	FRPT	2020-09-28	104.57	107.74	104.57	107.48	251480	{}	\N	2020-10-12 14:58:31.08999
44559	FRPT	2020-09-29	107.84	110.59	107.27	109.03	192979	{}	\N	2020-10-12 14:58:31.100783
44560	FRPT	2020-09-30	109.5	112.12	109.1	111.65	230159	{}	\N	2020-10-12 14:58:31.106974
44561	FRPT	2020-10-01	112.77	113.36	110.74	113.33	223639	{}	\N	2020-10-12 14:58:31.112604
44562	FRPT	2020-10-02	110.64	113.94	110.001	112.83	231532	{}	\N	2020-10-12 14:58:31.117795
44563	FRPT	2020-10-05	114.44	117.25	114.365	116.23	301933	{}	\N	2020-10-12 14:58:31.123067
44564	FRPT	2020-10-06	116.7	118.62	115.61	116.58	282967	{}	\N	2020-10-12 14:58:31.127292
44565	FRPT	2020-10-07	118.45	123.19	117.72	121.08	384848	{}	\N	2020-10-12 14:58:31.131524
44566	FRPT	2020-10-08	122.9	122.98	119.27	120.03	241472	{}	\N	2020-10-12 14:58:31.136258
44567	FRPT	2020-10-09	120.22	121.9699	120.13	121.71	234224	{}	\N	2020-10-12 14:58:31.140508
44568	FRPT	2020-10-12	122.94	124	122.15	122.59	27800	{}	\N	2020-10-12 14:58:31.144704
44569	FRSX	2020-09-28	0.9	0.9089	0.844	0.8633	341578	{}	\N	2020-10-12 14:58:32.069887
44570	FRSX	2020-09-29	0.9199	0.97	0.9	0.94	1375207	{}	\N	2020-10-12 14:58:32.076582
44571	FRSX	2020-09-30	0.9105	0.94	0.89	0.9177	862234	{}	\N	2020-10-12 14:58:32.083489
44572	FRSX	2020-10-01	0.92	0.9271	0.8816	0.9271	379571	{}	\N	2020-10-12 14:58:32.090192
44573	FRSX	2020-10-02	0.9319	0.98	0.9102	0.9212	1730103	{}	\N	2020-10-12 14:58:32.095837
44574	FRSX	2020-10-05	0.93	0.94	0.9	0.91	400540	{}	\N	2020-10-12 14:58:32.101535
44575	FRSX	2020-10-06	0.9	0.9285	0.87	0.8798	636051	{}	\N	2020-10-12 14:58:32.10697
44576	FRSX	2020-10-07	0.9001	1.04	0.89	0.9424	2597772	{}	\N	2020-10-12 14:58:32.114327
44577	FRSX	2020-10-08	0.97	0.97	0.91	0.93	886216	{}	\N	2020-10-12 14:58:32.119523
44578	FRSX	2020-10-09	0.94	0.99	0.93	0.9699	704693	{}	\N	2020-10-12 14:58:32.124971
44579	FRSX	2020-10-12	0.99	0.99	0.9501	0.9523	201113	{}	\N	2020-10-12 14:58:32.130118
44580	FSD	2020-09-28	13.84	13.96	13.84	13.95	62972	{}	\N	2020-10-12 14:58:33.517458
44581	FSD	2020-09-29	13.92	13.9799	13.89	13.96	334538	{}	\N	2020-10-12 14:58:33.52458
44582	FSD	2020-09-30	13.98	14.01	13.92	13.92	260573	{}	\N	2020-10-12 14:58:33.529883
44583	FSD	2020-10-01	13.83	13.9	13.8	13.88	119683	{}	\N	2020-10-12 14:58:33.535191
44584	FSD	2020-10-02	13.76	13.89	13.76	13.87	93631	{}	\N	2020-10-12 14:58:33.539854
44585	FSD	2020-10-05	13.9	13.92	13.8734	13.91	56748	{}	\N	2020-10-12 14:58:33.543888
44586	FSD	2020-10-06	13.91	13.98	13.91	13.95	104671	{}	\N	2020-10-12 14:58:33.548288
44587	FSD	2020-10-07	13.96	14.09	13.96	14.08	199122	{}	\N	2020-10-12 14:58:33.55262
44588	FSD	2020-10-08	14.1	14.17	14.0629	14.16	73410	{}	\N	2020-10-12 14:58:33.557232
44589	FSD	2020-10-09	14.14	14.23	14.14	14.19	102174	{}	\N	2020-10-12 14:58:33.561582
44590	FSD	2020-10-12	14.1901	14.25	14.1901	14.25	1742	{}	\N	2020-10-12 14:58:33.565721
44591	FSTR	2020-09-28	12.37	12.79	12.37	12.67	30729	{}	\N	2020-10-12 14:58:35.07978
44592	FSTR	2020-09-29	12.68	12.88	12.19	12.52	22150	{}	\N	2020-10-12 14:58:35.086066
44593	FSTR	2020-09-30	12.69	13.845	12.69	13.42	39986	{}	\N	2020-10-12 14:58:35.090717
44594	FSTR	2020-10-01	13.645	13.645	13.09	13.25	18685	{}	\N	2020-10-12 14:58:35.094717
44595	FSTR	2020-10-02	13.1	14.17	12.151	13.98	28562	{}	\N	2020-10-12 14:58:35.098707
44596	FSTR	2020-10-05	15	15	13.99	14.24	35659	{}	\N	2020-10-12 14:58:35.10243
44597	FSTR	2020-10-06	14.55	16.08	14.46	15.45	69248	{}	\N	2020-10-12 14:58:35.105768
44598	FSTR	2020-10-07	15.84	15.84	13.94	15.04	74404	{}	\N	2020-10-12 14:58:35.109387
44599	FSTR	2020-10-08	15.36	15.36	14.7	15.03	17372	{}	\N	2020-10-12 14:58:35.112755
44600	FSTR	2020-10-09	15.34	15.65	14.88	14.99	28999	{}	\N	2020-10-12 14:58:35.116631
44601	FSTR	2020-10-12	15	15.34	14.8	14.925	5566	{}	\N	2020-10-12 14:58:35.11988
44602	FTAG	2020-09-30	21.8	21.8	21.8	21.8	1262	{}	\N	2020-10-12 14:58:36.295495
44603	FTAG	2020-10-02	21.16	21.16	21.16	21.16	505	{}	\N	2020-10-12 14:58:36.30498
44604	FTAG	2020-10-05	21.65	21.9976	21.65	21.9976	2172	{}	\N	2020-10-12 14:58:36.310963
44605	FTAG	2020-10-07	22.11	22.1693	22.11	22.1693	658	{}	\N	2020-10-12 14:58:36.315639
44606	FTAG	2020-10-09	22.54	22.56	22.48	22.48	316	{}	\N	2020-10-12 14:58:36.320417
44607	FTLB	2020-09-29	19.1856	19.1856	19.09	19.1798	1214	{}	\N	2020-10-12 14:58:38.335957
44608	FTLB	2020-09-30	19.2	19.2841	19.2	19.28	3539	{}	\N	2020-10-12 14:58:38.343751
44609	FTLB	2020-10-01	19.1711	19.235	19.1711	19.235	493	{}	\N	2020-10-12 14:58:38.349589
44610	FTLB	2020-10-05	19.3826	19.435	19.3826	19.435	405	{}	\N	2020-10-12 14:58:38.356919
44611	FTLB	2020-10-06	19.4	19.42	19.2549	19.2549	1039	{}	\N	2020-10-12 14:58:38.362335
44612	FTLB	2020-10-07	19.31	19.38	19.3	19.38	1420	{}	\N	2020-10-12 14:58:38.366691
44613	FTLB	2020-10-08	19.4857	19.4857	19.485	19.485	1000	{}	\N	2020-10-12 14:58:38.370984
44614	FTLB	2020-10-09	19.6104	19.6104	19.58	19.58	1775	{}	\N	2020-10-12 14:58:38.37532
44615	FTSI	2020-09-28	3.29	3.4653	3.21	3.21	46317	{}	\N	2020-10-12 14:58:39.883272
44616	FTSI	2020-09-29	3.23	3.56	3.23	3.45	123541	{}	\N	2020-10-12 14:58:39.892695
44617	FTSI	2020-09-30	3.44	3.45	3.2892	3.32	26823	{}	\N	2020-10-12 14:58:39.899164
44618	FTSI	2020-10-01	3.35	3.5	3.33	3.34	21774	{}	\N	2020-10-12 14:58:39.904789
44619	FTSI	2020-10-02	3.16	3.3898	3.1269	3.26	20059	{}	\N	2020-10-12 14:58:39.909822
44620	FTSI	2020-10-05	3.38	3.42	3.24	3.35	35361	{}	\N	2020-10-12 14:58:39.914447
44621	FTSI	2020-10-06	3.34	3.4766	3.17	3.24	23765	{}	\N	2020-10-12 14:58:39.918943
44622	FTSI	2020-10-07	3.15	3.36	3.15	3.3	17416	{}	\N	2020-10-12 14:58:39.922893
44623	FTSI	2020-10-08	3.34	4.92	3.25	4.422	592149	{}	\N	2020-10-12 14:58:39.926897
44624	FTSI	2020-10-09	4.42	4.6699	3.86	4.06	779688	{}	\N	2020-10-12 14:58:39.930884
44625	FTSI	2020-10-12	4.33	4.33	3.86	3.8774	29719	{}	\N	2020-10-12 14:58:39.934825
44626	FRAN	2020-09-28	2.71	3	2.71	2.82	205335	{}	\N	2020-10-12 14:58:41.692494
44627	FRAN	2020-09-29	2.75	2.85	2.6	2.73	159278	{}	\N	2020-10-12 14:58:41.703038
44628	FRAN	2020-09-30	2.78	2.99	2.68	2.68	522874	{}	\N	2020-10-12 14:58:41.710577
44629	FRAN	2020-10-01	2.71	2.97	2.71	2.85	481788	{}	\N	2020-10-12 14:58:41.715597
44630	FRAN	2020-10-02	2.78	2.82	2.65	2.74	206890	{}	\N	2020-10-12 14:58:41.720334
44631	FRAN	2020-10-05	2.8	2.84	2.74	2.79	99232	{}	\N	2020-10-12 14:58:41.724837
44632	FRAN	2020-10-06	2.8	2.8	2.66	2.7	111710	{}	\N	2020-10-12 14:58:41.72884
44633	FRAN	2020-10-07	2.8	2.81	2.685	2.8	129911	{}	\N	2020-10-12 14:58:41.732621
44634	FRAN	2020-10-08	2.84	2.8498	2.7201	2.81	143207	{}	\N	2020-10-12 14:58:41.73651
44635	FRAN	2020-10-09	2.9	2.99	2.8	2.88	176235	{}	\N	2020-10-12 14:58:41.740592
44636	FRAN	2020-10-12	2.88	3.7089	2.87	3.1991	1173347	{}	\N	2020-10-12 14:58:41.74452
44637	FRGI	2020-09-28	9.66	10.085	9.58	9.95	116154	{}	\N	2020-10-12 14:58:42.664159
44638	FRGI	2020-09-29	9.96	9.985	9.68	9.81	140000	{}	\N	2020-10-12 14:58:42.675222
44639	FRGI	2020-09-30	9.87	10.11	9.33	9.37	157674	{}	\N	2020-10-12 14:58:42.68288
44640	FRGI	2020-10-01	9.35	9.475	8.95	9.2	125784	{}	\N	2020-10-12 14:58:42.689341
44641	FRGI	2020-10-02	9.01	9.18	8.91	8.93	81588	{}	\N	2020-10-12 14:58:42.694366
44642	FRGI	2020-10-05	9.02	9.2263	8.77	8.95	158875	{}	\N	2020-10-12 14:58:42.699358
44643	FRGI	2020-10-06	9.04	10.22	9.04	9.59	206311	{}	\N	2020-10-12 14:58:42.70369
44644	FRGI	2020-10-07	9.68	10.63	9.68	10.3	290350	{}	\N	2020-10-12 14:58:42.707895
44645	FRGI	2020-10-08	10.42	10.48	10.2	10.45	67610	{}	\N	2020-10-12 14:58:42.712655
44646	FRGI	2020-10-09	10.6	11.3	10.6	11.03	157076	{}	\N	2020-10-12 14:58:42.717777
44647	FRGI	2020-10-12	11.02	11.11	10.59	10.59	27725	{}	\N	2020-10-12 14:58:42.724392
44648	FTFT	2020-09-28	2	2.11	1.9	2.11	46705	{}	\N	2020-10-12 14:58:46.248056
44649	FTFT	2020-09-29	2.16	2.205	2.04	2.07	81812	{}	\N	2020-10-12 14:58:46.259575
44650	FTFT	2020-09-30	2.18	2.2	2.1001	2.19	29390	{}	\N	2020-10-12 14:58:46.266029
44651	FTFT	2020-10-01	2.17	2.22	2	2.2	36890	{}	\N	2020-10-12 14:58:46.271242
44652	FTFT	2020-10-02	2.08	2.19	2.03	2.13	26301	{}	\N	2020-10-12 14:58:46.276558
44653	FTFT	2020-10-05	2.17	2.38	2.1	2.29	25786	{}	\N	2020-10-12 14:58:46.281916
44654	FTFT	2020-10-06	2.33	2.33	2.205	2.24	20540	{}	\N	2020-10-12 14:58:46.286844
44655	FTFT	2020-10-07	2.23	2.28	2.2	2.22	45085	{}	\N	2020-10-12 14:58:46.291232
44656	FTFT	2020-10-08	2.25	2.25	2.125	2.18	27532	{}	\N	2020-10-12 14:58:46.295276
44657	FTFT	2020-10-09	2.185	2.25	2.11	2.11	82457	{}	\N	2020-10-12 14:58:46.300141
44658	FTFT	2020-10-12	2.14	2.15	2.05	2.075	14557	{}	\N	2020-10-12 14:58:46.308902
44659	FTI	2020-09-28	6.29	6.45	6.28	6.37	5480036	{}	\N	2020-10-12 14:58:47.236416
44660	FTI	2020-09-29	6.3	6.35	6.08	6.21	9841059	{}	\N	2020-10-12 14:58:47.247622
44661	FTI	2020-09-30	6.37	6.53	6.23	6.31	13017859	{}	\N	2020-10-12 14:58:47.254007
44662	FTI	2020-10-01	6.23	6.33	6.09	6.27	10907937	{}	\N	2020-10-12 14:58:47.259967
44663	FTI	2020-10-02	6.13	6.53	6.085	6.38	14562699	{}	\N	2020-10-12 14:58:47.264665
44664	FTI	2020-10-05	6.67	6.76	6.59	6.72	8998150	{}	\N	2020-10-12 14:58:47.269693
44665	FTI	2020-10-06	6.93	6.955	6.465	6.53	14750184	{}	\N	2020-10-12 14:58:47.273947
44666	FTI	2020-10-07	6.68	6.73	6.515	6.67	8881480	{}	\N	2020-10-12 14:58:47.278047
44667	FTI	2020-10-08	6.88	7.09	6.835	7.08	8781692	{}	\N	2020-10-12 14:58:47.282424
44668	FTI	2020-10-09	7.2	7.24	7.01	7.07	9318310	{}	\N	2020-10-12 14:58:47.287064
44669	FTI	2020-10-12	7.07	7.09	6.91	6.92	2330311	{}	\N	2020-10-12 14:58:47.29182
44670	FTK	2020-09-28	2.8	2.9	2.69	2.82	324443	{}	\N	2020-10-12 14:58:48.554672
44671	FTK	2020-09-29	2.85	2.89	2.71	2.84	613531	{}	\N	2020-10-12 14:58:48.564834
44672	FTK	2020-09-30	2.84	2.9417	2.64	2.71	1649537	{}	\N	2020-10-12 14:58:48.571492
44673	FTK	2020-10-01	2.72	2.84	2.65	2.68	914169	{}	\N	2020-10-12 14:58:48.577152
44674	FTK	2020-10-02	2.61	2.81	2.56	2.71	510892	{}	\N	2020-10-12 14:58:48.582002
44675	FTK	2020-10-05	2.74	2.78	2.62	2.66	400753	{}	\N	2020-10-12 14:58:48.586773
44676	FTK	2020-10-06	2.7	2.88	2.651	2.72	776291	{}	\N	2020-10-12 14:58:48.591121
44677	FTK	2020-10-07	2.76	2.885	2.7333	2.74	640458	{}	\N	2020-10-12 14:58:48.595141
44678	FTK	2020-10-08	2.76	2.8	2.7	2.71	340317	{}	\N	2020-10-12 14:58:48.598983
44679	FTK	2020-10-09	2.7	2.78	2.67	2.68	658325	{}	\N	2020-10-12 14:58:48.60301
44680	FTK	2020-10-12	2.71	2.7754	2.64	2.77	180104	{}	\N	2020-10-12 14:58:48.607375
44681	FTSM	2020-09-28	60.08	60.1	60.04	60.08	1492822	{}	\N	2020-10-12 14:58:49.5771
44682	FTSM	2020-09-29	60.07	60.085	60.06	60.08	671715	{}	\N	2020-10-12 14:58:49.587551
44683	FTSM	2020-09-30	60.08	60.08	60.02	60.02	959988	{}	\N	2020-10-12 14:58:49.593599
44684	FTSM	2020-10-01	60.03	60.06	60.02	60.05	1184966	{}	\N	2020-10-12 14:58:49.598826
44685	FTSM	2020-10-02	60.03	60.0466	60.02	60.03	724450	{}	\N	2020-10-12 14:58:49.603441
44686	FTSM	2020-10-05	60.04	60.07	60.03	60.06	747009	{}	\N	2020-10-12 14:58:49.60807
44687	FTSM	2020-10-06	60.06	60.0699	60.03	60.04	1284015	{}	\N	2020-10-12 14:58:49.612065
44688	FTSM	2020-10-07	60.04	60.04	60.03	60.04	642358	{}	\N	2020-10-12 14:58:49.615689
44689	FTSM	2020-10-08	60.04	60.04	60.02	60.04	1007361	{}	\N	2020-10-12 14:58:49.619491
44690	FTSM	2020-10-09	60.04	60.06	60.03	60.06	470367	{}	\N	2020-10-12 14:58:49.623272
44691	FTSM	2020-10-12	60.04	60.05	60.04	60.0473	58320	{}	\N	2020-10-12 14:58:49.626951
44692	FRPH	2020-09-28	41.49	42.25	41.45	41.86	31893	{}	\N	2020-10-12 14:58:52.451224
44693	FRPH	2020-09-29	41.67	42	41.67	41.95	9324	{}	\N	2020-10-12 14:58:52.457276
44694	FRPH	2020-09-30	41.78	42.27	41.67	41.67	13402	{}	\N	2020-10-12 14:58:52.461024
44695	FRPH	2020-10-01	41.79	42.07	41.4	41.51	12522	{}	\N	2020-10-12 14:58:52.464793
44696	FRPH	2020-10-02	40.95	42.67	40.3	42.44	11070	{}	\N	2020-10-12 14:58:52.469128
44697	FRPH	2020-10-05	42.57	43.39	41.2	43.33	11015	{}	\N	2020-10-12 14:58:52.473726
44698	FRPH	2020-10-06	42.76	43.96	42.65	43.53	17000	{}	\N	2020-10-12 14:58:52.478211
44699	FRPH	2020-10-07	43.9144	44.48	43.55	43.57	14075	{}	\N	2020-10-12 14:58:52.482182
44700	FRPH	2020-10-08	43.9	44.6	43.27	44.25	22418	{}	\N	2020-10-12 14:58:52.486311
44701	FRPH	2020-10-09	44.02	44.22	42.79	43.3	24539	{}	\N	2020-10-12 14:58:52.490948
44702	FRPH	2020-10-12	43.53	43.53	43.53	43.53	509	{}	\N	2020-10-12 14:58:52.494757
44703	FSBW	2020-09-28	39.9	40.79	39.9	40.01	8576	{}	\N	2020-10-12 14:58:53.451427
44704	FSBW	2020-09-29	40.18	42.9749	40.18	41.2	3619	{}	\N	2020-10-12 14:58:53.4605
44705	FSBW	2020-09-30	41.61	41.95	41	41	4741	{}	\N	2020-10-12 14:58:53.466955
44706	FSBW	2020-10-01	41.3	41.5195	41.3	41.39	5436	{}	\N	2020-10-12 14:58:53.473085
44707	FSBW	2020-10-02	40.3	42.1	39.25	42.1	4364	{}	\N	2020-10-12 14:58:53.478584
44708	FSBW	2020-10-05	39.3001	43.69	39.3001	43.69	17524	{}	\N	2020-10-12 14:58:53.483647
44709	FSBW	2020-10-06	43.52	44.36	42.73	43.35	10736	{}	\N	2020-10-12 14:58:53.488218
44710	FSBW	2020-10-07	43.015	45.01	43	44.79	6066	{}	\N	2020-10-12 14:58:53.493238
44711	FSBW	2020-10-08	45.22	45.22	42.675	43.2	10699	{}	\N	2020-10-12 14:58:53.497856
44712	FSBW	2020-10-09	43.95	44.02	43.11	43.11	11358	{}	\N	2020-10-12 14:58:53.50933
44713	FSBW	2020-10-12	43.54	43.54	43.54	43.54	542	{}	\N	2020-10-12 14:58:53.514434
44714	FSFG	2020-09-28	47.95	48.6	47.95	48.54	3537	{}	\N	2020-10-12 14:58:55.963033
44715	FSFG	2020-09-29	48.5899	49.9	48.5899	49.78	3423	{}	\N	2020-10-12 14:58:55.972147
44716	FSFG	2020-09-30	49.73	57	49.73	54.34	10163	{}	\N	2020-10-12 14:58:55.978147
44717	FSFG	2020-10-01	54.23	54.395	52.65	52.81	8026	{}	\N	2020-10-12 14:58:55.983686
44718	FSFG	2020-10-02	51.81	53.1	51.81	53.1	1319	{}	\N	2020-10-12 14:58:55.989383
44719	FSFG	2020-10-05	53.05	53.05	53.05	53.05	984	{}	\N	2020-10-12 14:58:55.994325
44720	FSFG	2020-10-06	52.5	54	51.1001	52.19	6117	{}	\N	2020-10-12 14:58:55.998629
44721	FSFG	2020-10-07	52.19	52.55	52.19	52.55	1292	{}	\N	2020-10-12 14:58:56.002949
44722	FSFG	2020-10-08	53.9	53.9	52.14	52.14	3305	{}	\N	2020-10-12 14:58:56.006966
44723	FSFG	2020-10-09	52	53	51.9904	53	753	{}	\N	2020-10-12 14:58:56.011053
44724	FSFG	2020-10-12	53.5	53.5	53.5	53.5	852	{}	\N	2020-10-12 14:58:56.01534
44725	FSP	2020-09-28	3.55	4.02	3.55	3.85	1104600	{}	\N	2020-10-12 14:58:57.220483
44726	FSP	2020-09-29	3.81	3.86	3.64	3.69	502236	{}	\N	2020-10-12 14:58:57.226555
44727	FSP	2020-09-30	3.71	3.83	3.61	3.66	557423	{}	\N	2020-10-12 14:58:57.230742
44728	FSP	2020-10-01	3.7	3.78	3.58	3.77	370093	{}	\N	2020-10-12 14:58:57.234983
44729	FSP	2020-10-02	3.7	3.89	3.69	3.86	304233	{}	\N	2020-10-12 14:58:57.239442
44730	FSP	2020-10-05	3.92	3.95	3.71	3.76	283407	{}	\N	2020-10-12 14:58:57.243584
44731	FSP	2020-10-06	3.82	3.94	3.75	3.76	324023	{}	\N	2020-10-12 14:58:57.247652
44732	FSP	2020-10-07	3.74	3.83	3.715	3.79	591927	{}	\N	2020-10-12 14:58:57.251472
44733	FSP	2020-10-08	3.84	3.94	3.8	3.9	244257	{}	\N	2020-10-12 14:58:57.255543
44734	FSP	2020-10-09	3.95	3.95	3.79	3.865	314207	{}	\N	2020-10-12 14:58:57.259488
44735	FSP	2020-10-12	3.86	3.88	3.76	3.86	33837	{}	\N	2020-10-12 14:58:57.263187
44736	FTC	2020-09-28	83.33	83.727	83.06	83.69	38894	{}	\N	2020-10-12 14:58:58.229794
44737	FTC	2020-09-29	83.7	84.22	83.62	83.7	176471	{}	\N	2020-10-12 14:58:58.238703
44738	FTC	2020-09-30	83.67	84.68	83.39	83.995	158010	{}	\N	2020-10-12 14:58:58.245105
44739	FTC	2020-10-01	84.69	85.3	84.52	85.07	63571	{}	\N	2020-10-12 14:58:58.251664
44740	FTC	2020-10-02	83.38	84.88	83.38	84.09	63778	{}	\N	2020-10-12 14:58:58.257227
44741	FTC	2020-10-05	84.73	85.785	84.73	85.78	41877	{}	\N	2020-10-12 14:58:58.262241
44742	FTC	2020-10-06	85.8936	86.5268	84.7697	84.93	54022	{}	\N	2020-10-12 14:58:58.266725
44743	FTC	2020-10-07	85.71	86.57	85.71	86.433	81715	{}	\N	2020-10-12 14:58:58.271174
44744	FTC	2020-10-08	87.02	87.02	86.3332	86.71	72380	{}	\N	2020-10-12 14:58:58.275802
44745	FTC	2020-10-09	87.4829	87.94	87.39	87.8	20597	{}	\N	2020-10-12 14:58:58.280696
44746	FTC	2020-10-12	88.7	88.7899	88.1999	88.6301	6485	{}	\N	2020-10-12 14:58:58.285091
44747	FTGC	2020-09-28	16.95	16.985	16.85	16.97	117222	{}	\N	2020-10-12 14:58:59.44748
44748	FTGC	2020-09-29	16.92	16.92	16.758	16.8	33847	{}	\N	2020-10-12 14:58:59.46482
44749	FTGC	2020-09-30	16.84	17.0499	16.82	17	75957	{}	\N	2020-10-12 14:58:59.472155
44750	FTGC	2020-10-01	16.9359	16.9359	16.71	16.81	41632	{}	\N	2020-10-12 14:58:59.477535
44751	FTGC	2020-10-02	16.64	16.7854	16.6155	16.71	171615	{}	\N	2020-10-12 14:58:59.482088
44752	FTGC	2020-10-05	16.855	17.02	16.84	16.98	295979	{}	\N	2020-10-12 14:58:59.48602
44753	FTGC	2020-10-06	17.1623	17.1786	17.005	17.0292	18705	{}	\N	2020-10-12 14:58:59.490181
44754	FTGC	2020-10-07	17.1905	17.21	17.1001	17.19	50927	{}	\N	2020-10-12 14:58:59.494708
44755	FTGC	2020-10-08	17.35	17.3699	17.23	17.3	96149	{}	\N	2020-10-12 14:58:59.499547
44756	FTGC	2020-10-09	17.45	17.53	17.3601	17.5	98568	{}	\N	2020-10-12 14:58:59.503879
44757	FTGC	2020-10-12	17.38	17.38	17.3204	17.3204	10829	{}	\N	2020-10-12 14:58:59.508084
44758	FTHI	2020-09-28	19.12	19.1235	18.97	19.08	23022	{}	\N	2020-10-12 14:59:00.612245
44759	FTHI	2020-09-29	19.08	19.195	18.92	19.13	426109	{}	\N	2020-10-12 14:59:00.620958
44760	FTHI	2020-09-30	19.15	19.2298	19.1201	19.1391	7251	{}	\N	2020-10-12 14:59:00.627142
44761	FTHI	2020-10-01	19.19	19.237	19.13	19.13	12964	{}	\N	2020-10-12 14:59:00.633738
44762	FTHI	2020-10-02	18.98	19.2499	18.98	19.17	9218	{}	\N	2020-10-12 14:59:00.639351
44763	FTHI	2020-10-05	19.15	19.4057	19.15	19.4057	60834	{}	\N	2020-10-12 14:59:00.644028
44764	FTHI	2020-10-06	19.38	19.45	19.19	19.19	17394	{}	\N	2020-10-12 14:59:00.648633
44765	FTHI	2020-10-07	19.26	19.42	19.26	19.42	16885	{}	\N	2020-10-12 14:59:00.652566
44766	FTHI	2020-10-08	19.41	19.54	19.41	19.49	9881	{}	\N	2020-10-12 14:59:00.656325
44767	FTHI	2020-10-09	19.575	19.64	19.5702	19.6207	17654	{}	\N	2020-10-12 14:59:00.660408
44768	FTHI	2020-10-12	19.79	19.79	19.79	19.79	136	{}	\N	2020-10-12 14:59:00.664838
44769	FTNT	2020-09-28	118.27	118.5	116.41	117.31	868362	{}	\N	2020-10-12 14:59:01.600211
44770	FTNT	2020-09-29	117.59	118.82	116.65	117.1	998261	{}	\N	2020-10-12 14:59:01.609311
44771	FTNT	2020-09-30	117.07	118.65	116.17	117.81	1281009	{}	\N	2020-10-12 14:59:01.616069
44772	FTNT	2020-10-01	118.643	120.93	118.28	120.25	1178328	{}	\N	2020-10-12 14:59:01.622132
44773	FTNT	2020-10-02	118.14	121.77	116.3	116.38	1062252	{}	\N	2020-10-12 14:59:01.6274
44774	FTNT	2020-10-05	118.8733	119.64	116.39	119.09	1736992	{}	\N	2020-10-12 14:59:01.632057
44775	FTNT	2020-10-06	119.73	121.31	117.92	118.3	947201	{}	\N	2020-10-12 14:59:01.637299
44776	FTNT	2020-10-07	118.735	120.99	118.36	120.69	985133	{}	\N	2020-10-12 14:59:01.641847
44777	FTNT	2020-10-08	121.69	122.93	120.825	121.64	759441	{}	\N	2020-10-12 14:59:01.646424
44778	FTNT	2020-10-09	123.62	126.98	123.355	126.18	1243508	{}	\N	2020-10-12 14:59:01.65097
44779	FTNT	2020-10-12	127.7	129.28	127.0459	127.22	356575	{}	\N	2020-10-12 14:59:01.65505
44780	FTSD	2020-09-28	95.82	95.9	95.82	95.86	12254	{}	\N	2020-10-12 14:59:02.610964
44781	FTSD	2020-09-29	95.89	95.92	95.82	95.83	20867	{}	\N	2020-10-12 14:59:02.622005
44782	FTSD	2020-09-30	95.83	95.9	95.76	95.84	109519	{}	\N	2020-10-12 14:59:02.628626
44783	FTSD	2020-10-01	95.75	95.85	95.6838	95.75	30112	{}	\N	2020-10-12 14:59:02.635565
44784	FTSD	2020-10-02	95.819	95.82	95.7578	95.78	13323	{}	\N	2020-10-12 14:59:02.640988
44785	FTSD	2020-10-05	95.79	95.83	95.74	95.78	14296	{}	\N	2020-10-12 14:59:02.646175
44786	FTSD	2020-10-06	95.79	95.81	95.7306	95.8	59227	{}	\N	2020-10-12 14:59:02.650634
44787	FTSD	2020-10-07	95.75	95.77	95.67	95.77	14238	{}	\N	2020-10-12 14:59:02.654929
44788	FTSD	2020-10-08	95.715	95.76	95.65	95.71	14311	{}	\N	2020-10-12 14:59:02.659366
44789	FTSD	2020-10-09	95.75	95.78	95.67	95.77	31736	{}	\N	2020-10-12 14:59:02.663693
44790	FTSD	2020-10-12	95.72	95.72	95.6783	95.69	6323	{}	\N	2020-10-12 14:59:02.667889
44791	FTXD	2020-09-28	26.61	26.61	26.3601	26.3601	312	{}	\N	2020-10-12 14:59:03.818057
44792	FTXD	2020-09-30	26.75	26.82	26.66	26.82	1111	{}	\N	2020-10-12 14:59:03.827377
44793	FTXD	2020-10-01	26.82	27.067	26.7987	27.067	1405	{}	\N	2020-10-12 14:59:03.834054
44794	FTXD	2020-10-02	26.775	26.8104	26.775	26.8104	550	{}	\N	2020-10-12 14:59:03.840255
44795	FTXD	2020-10-05	27.08	27.31	27.08	27.31	611	{}	\N	2020-10-12 14:59:03.84596
44796	FTXD	2020-10-06	27.229	27.229	26.6942	26.6942	1038	{}	\N	2020-10-12 14:59:03.851179
44797	FTXD	2020-10-07	27	27.0255	26.87	26.99	1299	{}	\N	2020-10-12 14:59:03.855762
44798	FTXD	2020-10-08	27.41	27.41	27.2311	27.2311	1376	{}	\N	2020-10-12 14:59:03.860202
44799	FTXD	2020-10-09	27.7099	27.7099	27.55	27.668	2768	{}	\N	2020-10-12 14:59:03.864586
44800	FTXD	2020-10-12	27.7801	27.84	27.7801	27.84	601	{}	\N	2020-10-12 14:59:03.869138
44801	FUN	2020-09-28	28.42	29.3	28.06	28.66	330482	{}	\N	2020-10-12 14:59:05.169555
44802	FUN	2020-09-29	28.51	28.5207	27.06	27.4	1237022	{}	\N	2020-10-12 14:59:05.179163
44803	FUN	2020-09-30	27.3	28.24	27.3	28.07	697858	{}	\N	2020-10-12 14:59:05.185516
44804	FUN	2020-10-01	28.36	29.09	27.485	27.99	1111943	{}	\N	2020-10-12 14:59:05.19148
44805	FUN	2020-10-02	27.35	28.485	27	28.38	701590	{}	\N	2020-10-12 14:59:05.196281
44806	FUN	2020-10-05	28.29	28.6199	26.86	27.59	1744809	{}	\N	2020-10-12 14:59:05.201184
44807	FUN	2020-10-06	27.83	28.3	27.02	27.12	1474791	{}	\N	2020-10-12 14:59:05.205321
44808	FUN	2020-10-07	27.6	27.98	27.28	27.86	600623	{}	\N	2020-10-12 14:59:05.209227
44809	FUN	2020-10-08	27.96	28.365	27.64	27.85	624769	{}	\N	2020-10-12 14:59:05.21419
44810	FUN	2020-10-09	27.88	28.55	27.78	28.15	963022	{}	\N	2020-10-12 14:59:05.219526
44811	FUN	2020-10-12	27.96	28.16	27.56	27.84	97326	{}	\N	2020-10-12 14:59:05.224356
44812	FTCS	2020-09-28	63.19	63.3992	62.91	63.22	661031	{}	\N	2020-10-12 14:59:06.449353
44813	FTCS	2020-09-29	63.22	63.37	62.86	63.01	396314	{}	\N	2020-10-12 14:59:06.45905
44814	FTCS	2020-09-30	63.2	63.89	62.99	63.27	545783	{}	\N	2020-10-12 14:59:06.465013
44815	FTCS	2020-10-01	63.8	63.91	63.3264	63.62	504262	{}	\N	2020-10-12 14:59:06.470636
44816	FTCS	2020-10-02	62.87	63.3432	62.62	62.89	510066	{}	\N	2020-10-12 14:59:06.476096
44817	FTCS	2020-10-05	63.32	63.98	63.32	63.9574	306415	{}	\N	2020-10-12 14:59:06.480806
44818	FTCS	2020-10-06	64.02	64.1595	62.96	63.07	537384	{}	\N	2020-10-12 14:59:06.484712
44819	FTCS	2020-10-07	63.6025	64.23	63.6025	64.04	308520	{}	\N	2020-10-12 14:59:06.489127
44820	FTCS	2020-10-08	64.37	64.538	64.23	64.538	427059	{}	\N	2020-10-12 14:59:06.493316
44821	FTCS	2020-10-09	64.87	65.32	64.78	65.24	303807	{}	\N	2020-10-12 14:59:06.497317
44822	FTCS	2020-10-12	65.7	65.7778	65.49	65.709	39094	{}	\N	2020-10-12 14:59:06.501471
44823	FTRI	2020-09-28	9.81	9.81	9.7	9.7301	2525	{}	\N	2020-10-12 14:59:07.496093
44824	FTRI	2020-09-29	9.65	9.65	9.65	9.65	258	{}	\N	2020-10-12 14:59:07.505188
44825	FTRI	2020-09-30	9.71	9.73	9.61	9.62	20574	{}	\N	2020-10-12 14:59:07.511556
44826	FTRI	2020-10-01	9.61	9.61	9.55	9.55	1437	{}	\N	2020-10-12 14:59:07.516823
44827	FTRI	2020-10-02	9.41	9.62	9.41	9.62	1914	{}	\N	2020-10-12 14:59:07.521993
44828	FTRI	2020-10-05	9.695	9.79	9.695	9.79	539	{}	\N	2020-10-12 14:59:07.526552
44829	FTRI	2020-10-06	9.79	9.79	9.6509	9.6509	1808	{}	\N	2020-10-12 14:59:07.530508
44830	FTRI	2020-10-08	9.85	9.94	9.85	9.94	603	{}	\N	2020-10-12 14:59:07.534539
44831	FTRI	2020-10-09	9.97	9.98	9.97	9.98	400	{}	\N	2020-10-12 14:59:07.538594
44832	FTRI	2020-10-12	9.96	9.96	9.96	9.96	480	{}	\N	2020-10-12 14:59:07.542808
44833	FTXL	2020-09-28	46.8447	47.43	46.8447	47.43	5089	{}	\N	2020-10-12 14:59:08.472035
44834	FTXL	2020-09-29	47.18	48	47.18	47.72	30148	{}	\N	2020-10-12 14:59:08.478371
44835	FTXL	2020-09-30	47.91	48.2547	47.75	47.75	2220	{}	\N	2020-10-12 14:59:08.483484
44836	FTXL	2020-10-01	48.81	48.89	48.265	48.73	8421	{}	\N	2020-10-12 14:59:08.487913
44837	FTXL	2020-10-02	47.42	48.3364	47.42	47.53	6228	{}	\N	2020-10-12 14:59:08.492344
44838	FTXL	2020-10-05	48.2211	48.98	48.2211	48.98	6920	{}	\N	2020-10-12 14:59:08.497116
44839	FTXL	2020-10-06	49.1771	50	48.65	48.8346	7001	{}	\N	2020-10-12 14:59:08.501572
44840	FTXL	2020-10-07	49.5923	49.8111	49.51	49.8111	8953	{}	\N	2020-10-12 14:59:08.506657
44841	FTXL	2020-10-08	50.07	50.43	50.07	50.254	9261	{}	\N	2020-10-12 14:59:08.511433
44842	FTXL	2020-10-09	51.36	51.575	51.1065	51.54	57651	{}	\N	2020-10-12 14:59:08.515894
44843	FTXL	2020-10-12	52.22	52.22	51.9085	52.06	2509	{}	\N	2020-10-12 14:59:08.520706
44844	FUNC	2020-09-28	11.6033	11.93	11.6	11.72	10432	{}	\N	2020-10-12 14:59:10.385508
44845	FUNC	2020-09-29	11.85	11.89	11.62	11.73	7638	{}	\N	2020-10-12 14:59:10.393627
44846	FUNC	2020-09-30	11.81	11.81	11.71	11.71	2152	{}	\N	2020-10-12 14:59:10.398675
44847	FUNC	2020-10-01	11.6	12.13	11.6	12.125	24687	{}	\N	2020-10-12 14:59:10.403486
44848	FUNC	2020-10-02	11.68	11.87	11.655	11.81	5510	{}	\N	2020-10-12 14:59:10.408726
44849	FUNC	2020-10-05	11.94	12.12	11.9101	12.07	12248	{}	\N	2020-10-12 14:59:10.415857
44850	FUNC	2020-10-06	12.08	12.7	12.0201	12.38	12835	{}	\N	2020-10-12 14:59:10.421236
44851	FUNC	2020-10-07	12.5	12.71	12.0723	12.69	8297	{}	\N	2020-10-12 14:59:10.4268
44852	FUNC	2020-10-08	12.13	12.84	12.13	12.75	10644	{}	\N	2020-10-12 14:59:10.432702
44853	FUNC	2020-10-09	12.455	12.72	12.455	12.57	1246	{}	\N	2020-10-12 14:59:10.439843
44854	FUNC	2020-10-12	12.39	12.565	12.39	12.54	2239	{}	\N	2020-10-12 14:59:10.445498
44855	FWDB	2020-09-28	25.72	25.72	25.72	25.72	12	{}	\N	2020-10-12 14:59:12.09377
44856	FWDB	2020-09-29	25.54	25.725	25.54	25.725	507	{}	\N	2020-10-12 14:59:12.104613
44857	FWDB	2020-09-30	25.76	25.775	25.7205	25.7374	3347	{}	\N	2020-10-12 14:59:12.115976
44858	FWDB	2020-10-01	25.74	25.74	25.59	25.6	7175	{}	\N	2020-10-12 14:59:12.123265
44859	FWDB	2020-10-02	25.63	25.738	25.63	25.738	3902	{}	\N	2020-10-12 14:59:12.13063
44860	FWDB	2020-10-05	25.76	25.8298	25.75	25.75	2435	{}	\N	2020-10-12 14:59:12.138919
44861	FWDB	2020-10-06	25.5901	25.7805	25.59	25.7805	30829	{}	\N	2020-10-12 14:59:12.146538
44862	FWDB	2020-10-07	25.76	25.8093	25.74	25.8093	10283	{}	\N	2020-10-12 14:59:12.151682
44863	FWDB	2020-10-08	25.8201	25.861	25.77	25.8288	14205	{}	\N	2020-10-12 14:59:12.156866
44864	FWDB	2020-10-09	25.72	25.76	25.72	25.76	4933	{}	\N	2020-10-12 14:59:12.16177
44865	FWDB	2020-10-12	25.805	25.805	25.66	25.805	936	{}	\N	2020-10-12 14:59:12.166496
44866	FXD	2020-09-28	42.41	42.86	42.381	42.76	49475	{}	\N	2020-10-12 14:59:13.317483
44867	FXD	2020-09-29	42.7716	42.7716	42.149	42.26	214698	{}	\N	2020-10-12 14:59:13.325616
44868	FXD	2020-09-30	42.42	43.03	42.28	42.61	70033	{}	\N	2020-10-12 14:59:13.332385
44869	FXD	2020-10-01	42.91	43.4044	42.77	43.4	164974	{}	\N	2020-10-12 14:59:13.339438
44870	FXD	2020-10-02	42.65	43.5599	42.65	43.38	286107	{}	\N	2020-10-12 14:59:13.34436
44871	FXD	2020-10-05	43.67	44.1342	43.67	44.09	98093	{}	\N	2020-10-12 14:59:13.349884
44872	FXD	2020-10-06	44.23	44.61	43.42	43.46	40674	{}	\N	2020-10-12 14:59:13.354523
44873	FXD	2020-10-07	43.9825	44.475	43.9825	44.37	31254	{}	\N	2020-10-12 14:59:13.360004
44874	FXD	2020-10-08	44.77	44.87	44.46	44.87	74138	{}	\N	2020-10-12 14:59:13.367502
44875	FXD	2020-10-09	45.17	45.22	44.928	45.03	186209	{}	\N	2020-10-12 14:59:13.372337
44876	FXD	2020-10-12	45.25	45.2596	45.06	45.16	15953	{}	\N	2020-10-12 14:59:13.378933
44877	FXY	2020-09-28	89.64	89.67	89.5	89.67	23466	{}	\N	2020-10-12 14:59:14.328286
44878	FXY	2020-09-29	89.57	89.57	89.456	89.5	20325	{}	\N	2020-10-12 14:59:14.336128
44879	FXY	2020-09-30	89.48	89.6679	89.42	89.62	24491	{}	\N	2020-10-12 14:59:14.341511
44880	FXY	2020-10-01	89.45	89.6104	89.45	89.59	19239	{}	\N	2020-10-12 14:59:14.346585
44881	FXY	2020-10-02	89.78	89.83	89.7	89.7	32984	{}	\N	2020-10-12 14:59:14.353172
44882	FXY	2020-10-05	89.58	89.58	89.37	89.4	23353	{}	\N	2020-10-12 14:59:14.358015
44883	FXY	2020-10-06	89.498	89.595	89.47	89.57	27931	{}	\N	2020-10-12 14:59:14.364362
44884	FXY	2020-10-07	89.27	89.27	89.11	89.15	35473	{}	\N	2020-10-12 14:59:14.369533
44885	FXY	2020-10-08	89.21	89.2384	89.16	89.2	35603	{}	\N	2020-10-12 14:59:14.374186
44886	FXY	2020-10-09	89.35	89.56	89.33	89.56	26093	{}	\N	2020-10-12 14:59:14.378482
44887	FXY	2020-10-12	89.74	89.809	89.71	89.809	3203	{}	\N	2020-10-12 14:59:14.383493
44888	GABC	2020-09-28	26.83	27.5	26.83	27.33	32608	{}	\N	2020-10-12 14:59:16.298433
44889	GABC	2020-09-29	27.24	27.24	26.59	27.03	22554	{}	\N	2020-10-12 14:59:16.306526
44890	GABC	2020-09-30	27.1222	27.38	26.98	27.14	44270	{}	\N	2020-10-12 14:59:16.312563
44891	GABC	2020-10-01	27.1	27.63	26.86	27.47	57010	{}	\N	2020-10-12 14:59:16.318898
44892	GABC	2020-10-02	27.05	28.15	27.05	27.95	39273	{}	\N	2020-10-12 14:59:16.326432
44893	GABC	2020-10-05	28.31	28.49	27.78	28.43	32721	{}	\N	2020-10-12 14:59:16.33089
44894	GABC	2020-10-06	28.55	29.2	28.2	28.41	45488	{}	\N	2020-10-12 14:59:16.334829
44895	GABC	2020-10-07	28.78	29.47	28.69	29.33	118807	{}	\N	2020-10-12 14:59:16.339811
44896	GABC	2020-10-08	29.66	29.75	29.24	29.61	41108	{}	\N	2020-10-12 14:59:16.344136
44897	GABC	2020-10-09	29.76	29.76	28.7501	29.31	36674	{}	\N	2020-10-12 14:59:16.348018
44898	GABC	2020-10-12	29.24	29.37	28.99	29.37	3104	{}	\N	2020-10-12 14:59:16.352168
44899	GAMR	2020-09-28	67	67.01	66.59	66.95	23347	{}	\N	2020-10-12 14:59:18.369219
44900	GAMR	2020-09-29	67.63	67.828	67.365	67.6199	30964	{}	\N	2020-10-12 14:59:18.376993
44901	GAMR	2020-09-30	67.7	68.49	67.7	67.94	23869	{}	\N	2020-10-12 14:59:18.382419
44902	GAMR	2020-10-01	68.5	68.61	68.2	68.38	15444	{}	\N	2020-10-12 14:59:18.387293
44903	GAMR	2020-10-02	67.27	67.91	67	67.44	8519	{}	\N	2020-10-12 14:59:18.392203
44904	GAMR	2020-10-05	67.58	68.08	67.52	68	23861	{}	\N	2020-10-12 14:59:18.39703
44905	GAMR	2020-10-06	68.06	68.15	67.25	67.42	22214	{}	\N	2020-10-12 14:59:18.401443
44906	GAMR	2020-10-07	67.97	68.2	67.97	68.0901	22874	{}	\N	2020-10-12 14:59:18.406054
44907	GAMR	2020-10-08	68.45	68.465	68.045	68.28	11177	{}	\N	2020-10-12 14:59:18.410657
44908	GAMR	2020-10-09	68.61	69.24	68.61	69.02	17757	{}	\N	2020-10-12 14:59:18.4148
44909	GAMR	2020-10-12	69.39	69.54	69.14	69.325	9596	{}	\N	2020-10-12 14:59:18.41923
44910	GBIL	2020-09-28	100.45	100.45	100.43	100.43	492586	{}	\N	2020-10-12 14:59:19.401001
44911	GBIL	2020-09-29	100.45	100.45	100.43	100.45	478623	{}	\N	2020-10-12 14:59:19.407687
44912	GBIL	2020-09-30	100.45	100.45	100.43	100.43	273342	{}	\N	2020-10-12 14:59:19.412491
44913	GBIL	2020-10-01	100.12	100.13	100.12	100.12	186646	{}	\N	2020-10-12 14:59:19.416657
44914	GBIL	2020-10-02	100.14	100.14	100.12	100.12	272334	{}	\N	2020-10-12 14:59:19.421368
44915	GBIL	2020-10-05	100.12	100.13	100.12	100.12	403042	{}	\N	2020-10-12 14:59:19.426751
44916	GBIL	2020-10-06	100.14	100.14	100.12	100.13	163925	{}	\N	2020-10-12 14:59:19.431109
44917	GBIL	2020-10-07	100.13	100.14	100.12	100.13	567494	{}	\N	2020-10-12 14:59:19.435314
44918	GBIL	2020-10-08	100.13	100.14	100.12	100.125	156074	{}	\N	2020-10-12 14:59:19.439335
44919	GBIL	2020-10-09	100.12	100.132	100.12	100.13	156517	{}	\N	2020-10-12 14:59:19.443675
44920	GBIL	2020-10-12	100.14	100.14	100.12	100.1245	47195	{}	\N	2020-10-12 14:59:19.447811
44921	GBR	2020-09-28	1.23	1.29	1.23	1.26	7115	{}	\N	2020-10-12 14:59:20.432362
44922	GBR	2020-09-29	1.47	2.13	1.3699	1.72	3029984	{}	\N	2020-10-12 14:59:20.439995
44923	GBR	2020-09-30	1.51	1.61	1.3601	1.43	274350	{}	\N	2020-10-12 14:59:20.445917
44924	GBR	2020-10-01	1.4	1.4	1.27	1.35	209872	{}	\N	2020-10-12 14:59:20.450786
44925	GBR	2020-10-02	1.35	1.68	1.27	1.56	716454	{}	\N	2020-10-12 14:59:20.455666
44926	GBR	2020-10-05	1.38	1.5549	1.38	1.46	143772	{}	\N	2020-10-12 14:59:20.460934
44927	GBR	2020-10-06	1.49	1.58	1.4768	1.52	90114	{}	\N	2020-10-12 14:59:20.466706
44928	GBR	2020-10-07	1.5	1.64	1.4708	1.5829	72306	{}	\N	2020-10-12 14:59:20.47115
44929	GBR	2020-10-08	1.55	1.94	1.53	1.6	779324	{}	\N	2020-10-12 14:59:20.475139
44930	GBR	2020-10-09	1.6	1.79	1.5431	1.6	408538	{}	\N	2020-10-12 14:59:20.479368
44931	GBR	2020-10-12	1.54	1.555	1.51	1.55	43281	{}	\N	2020-10-12 14:59:20.48325
44932	FVC	2020-09-28	26.08	26.23	26.051	26.18	10796	{}	\N	2020-10-12 14:59:22.486804
44933	FVC	2020-09-29	26.25	26.2699	26.08	26.17	27889	{}	\N	2020-10-12 14:59:22.495741
44934	FVC	2020-09-30	26.15	26.42	26.02	26.17	29610	{}	\N	2020-10-12 14:59:22.502137
44935	FVC	2020-10-01	26.5	26.635	26.395	26.61	297881	{}	\N	2020-10-12 14:59:22.507918
44936	FVC	2020-10-02	25.84	26.4758	25.84	26.32	28945	{}	\N	2020-10-12 14:59:22.51288
44937	FVC	2020-10-05	26.7201	26.8599	26.7	26.82	10031	{}	\N	2020-10-12 14:59:22.518235
44938	FVC	2020-10-06	26.85	27.1	26.59	26.59	9378	{}	\N	2020-10-12 14:59:22.522374
44939	FVC	2020-10-07	26.98	27.18	26.98	27.11	18522	{}	\N	2020-10-12 14:59:22.526838
44940	FVC	2020-10-08	27.19	27.3399	27.17	27.3	67718	{}	\N	2020-10-12 14:59:22.532018
44941	FVC	2020-10-09	27.39	27.67	27.39	27.58	21962	{}	\N	2020-10-12 14:59:22.53641
44942	FVC	2020-10-12	27.8	27.9399	27.8	27.9399	8781	{}	\N	2020-10-12 14:59:22.540639
44943	FWRD	2020-09-28	55.89	56.89	55.025	56.47	134268	{}	\N	2020-10-12 14:59:23.941879
44944	FWRD	2020-09-29	58.5	58.76	57.39	58	284342	{}	\N	2020-10-12 14:59:23.951765
44945	FWRD	2020-09-30	57.81	59	56.76	57.38	204903	{}	\N	2020-10-12 14:59:23.958928
44946	FWRD	2020-10-01	58.065	58.57	57.53	58.56	211259	{}	\N	2020-10-12 14:59:23.965223
44947	FWRD	2020-10-02	57.57	59.545	57.06	59.21	190704	{}	\N	2020-10-12 14:59:23.971019
44948	FWRD	2020-10-05	59.53	60.505	59.53	60	195459	{}	\N	2020-10-12 14:59:23.975667
44949	FWRD	2020-10-06	60.82	61.2	59.71	59.93	408008	{}	\N	2020-10-12 14:59:23.980725
44950	FWRD	2020-10-07	59.13	61.93	59.13	61.79	240573	{}	\N	2020-10-12 14:59:23.986052
44951	FWRD	2020-10-08	62.49	62.84	61.2	62.75	148742	{}	\N	2020-10-12 14:59:23.990632
44952	FWRD	2020-10-09	62.97	63.03	61.79	62.37	84473	{}	\N	2020-10-12 14:59:23.995339
44953	FWRD	2020-10-12	62.4	62.92	62.39	62.7482	6777	{}	\N	2020-10-12 14:59:24.000578
44954	FXI	2020-09-28	41.32	41.35	41.15	41.33	15623775	{}	\N	2020-10-12 14:59:24.86907
44955	FXI	2020-09-29	41.01	41.27	40.98	41.17	9987159	{}	\N	2020-10-12 14:59:24.877168
44956	FXI	2020-09-30	41.65	42.06	41.61	42	18078420	{}	\N	2020-10-12 14:59:24.883405
44957	FXI	2020-10-01	42.38	42.58	42.185	42.51	18886617	{}	\N	2020-10-12 14:59:24.889408
44958	FXI	2020-10-02	41.88	42.49	41.86	42.03	19157019	{}	\N	2020-10-12 14:59:24.894389
44959	FXI	2020-10-05	41.92	42.21	41.885	42.19	15933884	{}	\N	2020-10-12 14:59:24.900026
44960	FXI	2020-10-06	42.5	42.72	42.355	42.47	21738408	{}	\N	2020-10-12 14:59:24.90577
44961	FXI	2020-10-07	42.89	43.02	42.705	42.92	19278277	{}	\N	2020-10-12 14:59:24.910457
44962	FXI	2020-10-08	42.93	43.19	42.89	43.13	11415380	{}	\N	2020-10-12 14:59:24.916433
44963	FXI	2020-10-09	42.97	43.34	42.97	43.24	12049280	{}	\N	2020-10-12 14:59:24.921508
44964	FXI	2020-10-12	44.26	44.34	44.14	44.325	5787770	{}	\N	2020-10-12 14:59:24.926266
44965	FYC	2020-09-28	46.33	47.08	46.33	46.95	19987	{}	\N	2020-10-12 14:59:25.780109
44966	FYC	2020-09-29	47.2725	47.3699	46.78	47.035	67155	{}	\N	2020-10-12 14:59:25.788434
44967	FYC	2020-09-30	47.13	47.86	46.9601	47.26	89510	{}	\N	2020-10-12 14:59:25.794055
44968	FYC	2020-10-01	47.75	48.136	47.42	48.136	58227	{}	\N	2020-10-12 14:59:25.798944
44969	FYC	2020-10-02	47.09	48.4599	47.08	48.25	20551	{}	\N	2020-10-12 14:59:25.80378
44970	FYC	2020-10-05	48.54	49.44	48.54	49.42	17200	{}	\N	2020-10-12 14:59:25.808066
44971	FYC	2020-10-06	49.72	50.46	49.14	49.25	46828	{}	\N	2020-10-12 14:59:25.812179
44972	FYC	2020-10-07	49.77	50.36	49.77	50.25	31055	{}	\N	2020-10-12 14:59:25.81599
44973	FYC	2020-10-08	50.76	50.76	50.31	50.47	18603	{}	\N	2020-10-12 14:59:25.820275
44974	FYC	2020-10-09	50.98	51.3	50.85	51.1566	11239	{}	\N	2020-10-12 14:59:25.824554
44975	FYC	2020-10-12	51.54	51.54	51.15	51.3135	1525	{}	\N	2020-10-12 14:59:25.828374
44976	GALT	2020-09-28	2.77	3.85	2.63	3.64	4974294	{}	\N	2020-10-12 14:59:26.800395
44977	GALT	2020-09-29	3.7	3.74	3.27	3.29	1843725	{}	\N	2020-10-12 14:59:26.807751
44978	GALT	2020-09-30	3.25	3.265	2.66	2.67	1018419	{}	\N	2020-10-12 14:59:26.812702
44979	GALT	2020-10-01	2.68	2.74	2.63	2.64	252223	{}	\N	2020-10-12 14:59:26.817284
44980	GALT	2020-10-02	2.64	2.75	2.6	2.63	254755	{}	\N	2020-10-12 14:59:26.822497
44981	GALT	2020-10-05	2.67	2.77	2.64	2.7	280749	{}	\N	2020-10-12 14:59:26.826987
44982	GALT	2020-10-06	2.71	2.79	2.61	2.74	332103	{}	\N	2020-10-12 14:59:26.830692
44983	GALT	2020-10-07	2.72	2.88	2.6544	2.74	326957	{}	\N	2020-10-12 14:59:26.834426
44984	GALT	2020-10-08	2.7501	2.84	2.72	2.77	141796	{}	\N	2020-10-12 14:59:26.838713
44985	GALT	2020-10-09	2.81	2.81	2.71	2.74	137743	{}	\N	2020-10-12 14:59:26.842467
44986	GALT	2020-10-12	2.8	2.8	2.7	2.74	35029	{}	\N	2020-10-12 14:59:26.846131
44987	GBLI	2020-09-28	21.35	21.6799	20.4028	21.45	5428	{}	\N	2020-10-12 14:59:27.940506
44988	GBLI	2020-09-29	21.01	21.43	20.91	20.91	3230	{}	\N	2020-10-12 14:59:27.949222
44989	GBLI	2020-09-30	20.89	21.03	20.6	20.79	15100	{}	\N	2020-10-12 14:59:27.95573
44990	GBLI	2020-10-01	20.72	21.255	20.55	20.82	23424	{}	\N	2020-10-12 14:59:27.961521
44991	GBLI	2020-10-02	20.43	20.99	20.43	20.71	6446	{}	\N	2020-10-12 14:59:27.966202
44992	GBLI	2020-10-05	20.99	21.98	20.99	21.54	46071	{}	\N	2020-10-12 14:59:27.970936
44993	GBLI	2020-10-06	21.47	21.91	20.81	21.21	42774	{}	\N	2020-10-12 14:59:27.976121
44994	GBLI	2020-10-07	21.3	22.14	21.3	21.76	19452	{}	\N	2020-10-12 14:59:27.980913
44995	GBLI	2020-10-08	21.81	22.21	21.79	22	10331	{}	\N	2020-10-12 14:59:27.985611
44996	GBLI	2020-10-09	22.01	22.34	22.01	22.31	23515	{}	\N	2020-10-12 14:59:27.991287
44997	GBLI	2020-10-12	22.14	22.45	22.14	22.45	981	{}	\N	2020-10-12 14:59:27.996375
44998	GCOW	2020-09-28	24.43	24.7	24.43	24.7	15744	{}	\N	2020-10-12 14:59:28.843968
44999	GCOW	2020-09-29	24.59	24.67	24.441	24.4721	15693	{}	\N	2020-10-12 14:59:28.852247
45000	GCOW	2020-09-30	24.6	24.655	24.46	24.57	10319	{}	\N	2020-10-12 14:59:28.859104
45001	GCOW	2020-10-01	24.5872	24.59	24.4	24.47	19571	{}	\N	2020-10-12 14:59:28.864668
45002	GCOW	2020-10-02	24.31	24.6524	24.31	24.6041	8565	{}	\N	2020-10-12 14:59:28.87014
45003	GCOW	2020-10-05	24.87	25	24.81	24.9973	16719	{}	\N	2020-10-12 14:59:28.875258
45004	GCOW	2020-10-06	25.13	25.1542	24.76	24.845	13642	{}	\N	2020-10-12 14:59:28.879735
45005	GCOW	2020-10-07	24.97	25.11	24.935	25.05	17277	{}	\N	2020-10-12 14:59:28.884045
45006	GCOW	2020-10-08	25.17	25.3059	25.17	25.3059	18928	{}	\N	2020-10-12 14:59:28.889033
45007	GCOW	2020-10-09	25.3	25.3634	25.231	25.28	20052	{}	\N	2020-10-12 14:59:28.894689
45008	GCOW	2020-10-12	25.29	25.29	25.1912	25.23	8338	{}	\N	2020-10-12 14:59:28.899114
45009	GDS	2020-09-28	82.28	83.2	79.51	80.49	766420	{}	\N	2020-10-12 14:59:30.444117
45010	GDS	2020-09-29	80.09	81.16	79.24	80.16	290296	{}	\N	2020-10-12 14:59:30.448959
45011	GDS	2020-09-30	80.81	82.41	80.26	81.83	597255	{}	\N	2020-10-12 14:59:30.454262
45012	GDS	2020-10-01	82.71	83.2	81.68	82.82	571897	{}	\N	2020-10-12 14:59:30.45983
45013	GDS	2020-10-02	81.44	83.49	81.16	82.76	669441	{}	\N	2020-10-12 14:59:30.465312
45014	GDS	2020-10-05	83.68	84.24	82.6	83.95	599780	{}	\N	2020-10-12 14:59:30.472341
45015	GDS	2020-10-06	83.94	86.7	83.85	86	1078489	{}	\N	2020-10-12 14:59:30.477359
45016	GDS	2020-10-07	87.51	88.75	85.96	88.08	884801	{}	\N	2020-10-12 14:59:30.482515
45017	GDS	2020-10-08	88.35	89.22	85.65	86.02	546299	{}	\N	2020-10-12 14:59:30.487437
45018	GDS	2020-10-09	86.81	88.28	86.37	86.63	440317	{}	\N	2020-10-12 14:59:30.492397
45019	GDS	2020-10-12	87.45	88.09	86.77	86.99	287116	{}	\N	2020-10-12 14:59:30.49738
45020	GEM	2020-09-28	32.47	32.49	32.22	32.325	57687	{}	\N	2020-10-12 14:59:31.416754
45021	GEM	2020-09-29	32.24	32.3697	32.2002	32.3	57609	{}	\N	2020-10-12 14:59:31.422733
45022	GEM	2020-09-30	32.56	32.8499	32.53	32.79	79440	{}	\N	2020-10-12 14:59:31.428215
45023	GEM	2020-10-01	33.02	33.11	32.8639	33.11	79527	{}	\N	2020-10-12 14:59:31.43267
45024	GEM	2020-10-02	32.63	32.9825	32.63	32.73	73885	{}	\N	2020-10-12 14:59:31.437946
45025	GEM	2020-10-05	32.96	33.15	32.95	33.15	74657	{}	\N	2020-10-12 14:59:31.443146
45026	GEM	2020-10-06	33.24	33.3791	33.04	33.09	82617	{}	\N	2020-10-12 14:59:31.448216
45027	GEM	2020-10-07	33.4	33.51	33.32	33.435	107683	{}	\N	2020-10-12 14:59:31.452716
45028	GEM	2020-10-08	33.52	33.7199	33.49	33.6771	344818	{}	\N	2020-10-12 14:59:31.457381
45029	GEM	2020-10-09	33.76	33.99	33.7162	33.94	63905	{}	\N	2020-10-12 14:59:31.462361
45030	GEM	2020-10-12	34.02	34.0999	33.99	34.0999	466432	{}	\N	2020-10-12 14:59:31.466663
45031	GFED	2020-09-28	13.9	14.1	13.85	14.09	4766	{}	\N	2020-10-12 14:59:34.072502
45032	GFED	2020-09-30	13.85	14.013	13.85	14.013	1421	{}	\N	2020-10-12 14:59:34.079333
45033	GFED	2020-10-01	13.87	14.25	13.85	14.25	11256	{}	\N	2020-10-12 14:59:34.084507
45034	GFED	2020-10-02	13.9	14.085	13.9	14.085	947	{}	\N	2020-10-12 14:59:34.089247
45035	GFED	2020-10-05	14.09	14.1099	13.91	13.95	1804	{}	\N	2020-10-12 14:59:34.09372
45036	GFED	2020-10-06	14.06	14.1101	14.05	14.11	1156	{}	\N	2020-10-12 14:59:34.09884
45037	GFED	2020-10-07	13.9301	14.51	13.9301	14.25	1202	{}	\N	2020-10-12 14:59:34.103984
45038	GGM	2020-09-28	17.8	17.8	17.51	17.63	70968	{}	\N	2020-10-12 14:59:35.39255
45039	GGM	2020-09-29	17.68	17.71	17.66	17.7	51440	{}	\N	2020-10-12 14:59:35.400403
45040	GGM	2020-09-30	17.86	17.86	17.75	17.75	94308	{}	\N	2020-10-12 14:59:35.406627
45041	GGM	2020-10-01	17.71	17.92	17.71	17.85	61477	{}	\N	2020-10-12 14:59:35.41251
45042	GGM	2020-10-02	17.81	18.18	17.74	18.18	83433	{}	\N	2020-10-12 14:59:35.419038
45043	GGM	2020-10-05	18.25	18.3775	18.25	18.34	120196	{}	\N	2020-10-12 14:59:35.423853
45044	GGM	2020-10-06	18.35	18.3812	18.29	18.38	60614	{}	\N	2020-10-12 14:59:35.428456
45045	GGM	2020-10-07	18.44	18.44	18.34	18.42	47668	{}	\N	2020-10-12 14:59:35.433126
45046	GGM	2020-10-08	18.35	18.4644	18.35	18.46	47082	{}	\N	2020-10-12 14:59:35.437563
45047	GGM	2020-10-09	18.5	18.51	18.45	18.5	58612	{}	\N	2020-10-12 14:59:35.441956
45048	GGM	2020-10-12	18.46	18.8	18.46	18.56	35642	{}	\N	2020-10-12 14:59:35.44674
45049	FVD	2020-09-28	31.34	31.65	31.34	31.47	1099203	{}	\N	2020-10-12 14:59:36.359082
45050	FVD	2020-09-29	31.49	31.54	31.1567	31.27	1134430	{}	\N	2020-10-12 14:59:36.369229
45051	FVD	2020-09-30	31.4	31.72	31.2963	31.46	1430809	{}	\N	2020-10-12 14:59:36.375575
45052	FVD	2020-10-01	31.62	31.72	31.36	31.56	1245613	{}	\N	2020-10-12 14:59:36.380725
45053	FVD	2020-10-02	31.1724	31.9	31.1724	31.78	905898	{}	\N	2020-10-12 14:59:36.385657
45054	FVD	2020-10-05	31.973	32.245	31.9274	32.18	1033994	{}	\N	2020-10-12 14:59:36.390573
45055	FVD	2020-10-06	32.28	32.65	32.0507	32.12	1497372	{}	\N	2020-10-12 14:59:36.394615
45056	FVD	2020-10-07	32.34	32.55	32.27	32.45	1898200	{}	\N	2020-10-12 14:59:36.398619
45057	FVD	2020-10-08	32.6079	32.8517	32.6079	32.84	919457	{}	\N	2020-10-12 14:59:36.402854
45058	FVD	2020-10-09	32.99	33.05	32.77	32.89	1175639	{}	\N	2020-10-12 14:59:36.406981
45059	FVD	2020-10-12	32.97	33.0976	32.8958	33.04	167737	{}	\N	2020-10-12 14:59:36.411317
45060	FWP	2020-09-28	6.22	6.22	6.05	6.11	2397	{}	\N	2020-10-12 14:59:37.555079
45061	FWP	2020-09-29	6.16	6.4399	6.16	6.18	32967	{}	\N	2020-10-12 14:59:37.564007
45062	FWP	2020-09-30	6.225	6.33	6.1445	6.2	5506	{}	\N	2020-10-12 14:59:37.571408
45063	FWP	2020-10-01	6.2965	6.2965	6.22	6.28	15682	{}	\N	2020-10-12 14:59:37.576711
45064	FWP	2020-10-02	6.28	6.36	6.2	6.22	3145	{}	\N	2020-10-12 14:59:37.581768
45065	FWP	2020-10-05	6.2605	6.755	6.24	6.27	18033	{}	\N	2020-10-12 14:59:37.58623
45066	FWP	2020-10-06	6.42	6.77	6.31	6.4698	47152	{}	\N	2020-10-12 14:59:37.591866
45067	FWP	2020-10-07	6.5462	6.83	6.42	6.7	51482	{}	\N	2020-10-12 14:59:37.596112
45068	FWP	2020-10-08	6.76	6.9777	6.63	6.73	34007	{}	\N	2020-10-12 14:59:37.600039
45069	FWP	2020-10-09	6.68	6.77	6.4136	6.51	13862	{}	\N	2020-10-12 14:59:37.60495
45070	FWP	2020-10-12	6.4472	6.81	6.4472	6.81	2260	{}	\N	2020-10-12 14:59:37.609455
45071	FXH	2020-09-28	93.37	93.95	92.97	93.28	26301	{}	\N	2020-10-12 14:59:38.575212
45072	FXH	2020-09-29	93.38	94.2099	93.38	93.7321	65099	{}	\N	2020-10-12 14:59:38.585163
45073	FXH	2020-09-30	93.82	95.475	93.82	94.74	53190	{}	\N	2020-10-12 14:59:38.592141
45074	FXH	2020-10-01	95.31	95.8197	94.6094	95.19	227785	{}	\N	2020-10-12 14:59:38.598006
45075	FXH	2020-10-02	93.74	95.1057	93.5	94.11	47646	{}	\N	2020-10-12 14:59:38.604168
45076	FXH	2020-10-05	94.75	96.84	94.75	96.75	91696	{}	\N	2020-10-12 14:59:38.609423
45077	FXH	2020-10-06	96.97	97.612	96.04	96.23	41624	{}	\N	2020-10-12 14:59:38.613963
45078	FXH	2020-10-07	96.75	98.41	96.75	98.15	34422	{}	\N	2020-10-12 14:59:38.618732
45079	FXH	2020-10-08	98.73	98.92	98.23	98.92	136669	{}	\N	2020-10-12 14:59:38.62423
45080	FXH	2020-10-09	99.58	99.868	99.27	99.74	78903	{}	\N	2020-10-12 14:59:38.629254
45081	FXH	2020-10-12	100.27	100.33	99.8	100.2	4645	{}	\N	2020-10-12 14:59:38.633788
45082	FXR	2020-09-28	40.79	41.342	40.79	41.14	102001	{}	\N	2020-10-12 14:59:39.581681
45083	FXR	2020-09-29	41.145	41.3	40.82	40.9206	87699	{}	\N	2020-10-12 14:59:39.588908
45084	FXR	2020-09-30	41.13	41.505	40.76	40.96	61909	{}	\N	2020-10-12 14:59:39.594751
45085	FXR	2020-10-01	41.3278	41.54	40.9146	41.12	45427	{}	\N	2020-10-12 14:59:39.600238
45086	FXR	2020-10-02	40.775	41.7751	40.775	41.57	126246	{}	\N	2020-10-12 14:59:39.605791
45087	FXR	2020-10-05	42	42.62	42	42.55	36080	{}	\N	2020-10-12 14:59:39.610564
45088	FXR	2020-10-06	42.79	43.175	42.2	42.21	53903	{}	\N	2020-10-12 14:59:39.614783
45089	FXR	2020-10-07	42.82	43.2	42.82	43.13	84778	{}	\N	2020-10-12 14:59:39.619314
45090	FXR	2020-10-08	43.4	43.49	43.19	43.49	124782	{}	\N	2020-10-12 14:59:39.624239
45091	FXR	2020-10-09	43.8431	43.8431	43.4934	43.59	141113	{}	\N	2020-10-12 14:59:39.628678
45092	FXR	2020-10-12	43.7914	43.92	43.76	43.82	22838	{}	\N	2020-10-12 14:59:39.632942
45093	FYT	2020-09-28	30.52	31.25	30.52	31.18	6198	{}	\N	2020-10-12 14:59:40.540137
45094	FYT	2020-09-29	30.93	30.94	30.3905	30.84	13451	{}	\N	2020-10-12 14:59:40.547272
45095	FYT	2020-09-30	30.91	31.3462	30.7204	30.8	6853	{}	\N	2020-10-12 14:59:40.552557
45096	FYT	2020-10-01	30.96	31.27	30.74	31.27	13005	{}	\N	2020-10-12 14:59:40.55826
45097	FYT	2020-10-02	30.8762	32	30.8762	31.9304	17643	{}	\N	2020-10-12 14:59:40.562942
45098	FYT	2020-10-05	32.43	32.65	32.37	32.6029	24389	{}	\N	2020-10-12 14:59:40.567633
45099	FYT	2020-10-06	32.82	33.38	32.49	32.49	9797	{}	\N	2020-10-12 14:59:40.572904
45100	FYT	2020-10-07	32.98	33.3499	32.91	33.2257	5699	{}	\N	2020-10-12 14:59:40.577952
45101	FYT	2020-10-08	33.59	33.7752	33.3164	33.7752	8697	{}	\N	2020-10-12 14:59:40.582752
45102	FYT	2020-10-09	33.85	33.8699	33.625	33.6709	2873	{}	\N	2020-10-12 14:59:40.587643
45103	FYT	2020-10-12	33.73	33.73	33.73	33.73	174	{}	\N	2020-10-12 14:59:40.592683
45104	GAIA	2020-09-28	9.57	10.09	9.43	10.03	33264	{}	\N	2020-10-12 14:59:41.725037
45105	GAIA	2020-09-29	9.97	10.06	9.81	9.91	23042	{}	\N	2020-10-12 14:59:41.734824
45106	GAIA	2020-09-30	9.91	9.965	9.68	9.83	43910	{}	\N	2020-10-12 14:59:41.744673
45107	GAIA	2020-10-01	9.92	10.15	9.76	10.12	26007	{}	\N	2020-10-12 14:59:41.751654
45108	GAIA	2020-10-02	9.9	10.31	9.85	9.94	34155	{}	\N	2020-10-12 14:59:41.7568
45109	GAIA	2020-10-05	10.03	10.48	9.87	10.02	29869	{}	\N	2020-10-12 14:59:41.762233
45110	GAIA	2020-10-06	10.09	10.38	9.97	9.99	26494	{}	\N	2020-10-12 14:59:41.766728
45111	GAIA	2020-10-07	10.25	10.39	10.15	10.33	27134	{}	\N	2020-10-12 14:59:41.771435
45112	GAIA	2020-10-08	10.39	10.66	10.33	10.53	89458	{}	\N	2020-10-12 14:59:41.776189
45113	GAIA	2020-10-09	10.73	11.38	10.7	11.32	28463	{}	\N	2020-10-12 14:59:41.780405
45114	GAIA	2020-10-12	11.38	11.38	11.09	11.23	6992	{}	\N	2020-10-12 14:59:41.784673
45192	FXL	2020-09-28	90.13	90.48	89.24	90.22	106010	{}	\N	2020-10-12 14:59:55.079194
45193	FXL	2020-09-29	90.9323	91.24	90.4412	90.84	155769	{}	\N	2020-10-12 14:59:55.086678
45194	FXL	2020-09-30	90.65	91.7499	90.26	90.92	156156	{}	\N	2020-10-12 14:59:55.09159
45195	FXL	2020-10-01	91.79	93.11	91.77	92.65	207599	{}	\N	2020-10-12 14:59:55.097622
45196	FXL	2020-10-02	90.26	92.7987	90.26	91.58	153381	{}	\N	2020-10-12 14:59:55.102321
45197	FXL	2020-10-05	92.3	93.64	92.1	93.57	121488	{}	\N	2020-10-12 14:59:55.106949
45198	FXL	2020-10-06	93.8	95.2864	92.99	93.35	73602	{}	\N	2020-10-12 14:59:55.111405
45199	FXL	2020-10-07	94.11	95.62	94.11	95.45	88848	{}	\N	2020-10-12 14:59:55.115768
45200	FXL	2020-10-08	96.16	96.3	95.45	95.68	84213	{}	\N	2020-10-12 14:59:55.120112
45201	FXL	2020-10-09	96.41	97.44	96.41	97.15	125272	{}	\N	2020-10-12 14:59:55.124451
45202	FXL	2020-10-12	98.3966	98.645	97.8343	98.42	25837	{}	\N	2020-10-12 14:59:55.129515
45203	GAA	2020-09-28	26.74	26.74	26.24	26.275	3983	{}	\N	2020-10-12 14:59:57.457252
45204	GAA	2020-09-29	26.85	26.86	26.17	26.4	5368	{}	\N	2020-10-12 14:59:57.463895
45205	GAA	2020-09-30	26.51	26.555	26.27	26.27	27748	{}	\N	2020-10-12 14:59:57.469817
45206	GAA	2020-10-01	26.4917	26.6299	26.4917	26.53	2131	{}	\N	2020-10-12 14:59:57.47533
45207	GAA	2020-10-02	26.44	26.63	26.44	26.575	2503	{}	\N	2020-10-12 14:59:57.479958
45208	GAA	2020-10-05	26.72	26.7666	26.5307	26.5307	2063	{}	\N	2020-10-12 14:59:57.484793
45209	GAA	2020-10-06	26.8294	26.8294	26.7478	26.7478	612	{}	\N	2020-10-12 14:59:57.489355
45210	GAA	2020-10-07	26.89	27.009	26.89	27.009	2887	{}	\N	2020-10-12 14:59:57.49336
45211	GAA	2020-10-08	26.9459	26.9699	26.9459	26.9699	4661	{}	\N	2020-10-12 14:59:57.497525
45212	GAA	2020-10-09	27.09	27.1273	26.96	27.1273	3048	{}	\N	2020-10-12 14:59:57.502136
45213	GAA	2020-10-12	27.07	27.0791	27.07	27.0791	927	{}	\N	2020-10-12 14:59:57.506631
45214	GARS	2020-09-28	3.8	3.9	3.8	3.9	30421	{}	\N	2020-10-12 14:59:58.463966
45215	GARS	2020-09-29	3.86	3.9	3.77	3.845	27355	{}	\N	2020-10-12 14:59:58.472535
45216	GARS	2020-09-30	3.8601	3.9	3.8001	3.82	29916	{}	\N	2020-10-12 14:59:58.477956
45217	GARS	2020-10-01	3.8207	3.99	3.815	3.84	6871	{}	\N	2020-10-12 14:59:58.483675
45218	GARS	2020-10-02	3.76	3.87	3.759	3.87	9187	{}	\N	2020-10-12 14:59:58.491199
45219	GARS	2020-10-05	3.84	3.9	3.84	3.875	50671	{}	\N	2020-10-12 14:59:58.495651
45220	GARS	2020-10-06	3.9	3.9	3.77	3.9	39702	{}	\N	2020-10-12 14:59:58.499888
45221	GARS	2020-10-07	3.84	3.9	3.7801	3.87	25827	{}	\N	2020-10-12 14:59:58.504857
45222	GARS	2020-10-08	3.85	3.9	3.84	3.86	7493	{}	\N	2020-10-12 14:59:58.509404
45223	GARS	2020-10-09	3.82	3.89	3.815	3.89	16232	{}	\N	2020-10-12 14:59:58.513469
45224	GARS	2020-10-12	3.85	3.9	3.85	3.9	8350	{}	\N	2020-10-12 14:59:58.518144
45225	GBL	2020-09-28	11.59	11.92	11.18	11.35	20677	{}	\N	2020-10-12 14:59:59.41792
45226	GBL	2020-09-29	11.4	11.74	11.2816	11.46	24986	{}	\N	2020-10-12 14:59:59.423834
45227	GBL	2020-09-30	11.46	11.86	11.46	11.57	14085	{}	\N	2020-10-12 14:59:59.428373
45228	GBL	2020-10-01	11.69	11.84	11.52	11.61	11774	{}	\N	2020-10-12 14:59:59.432483
45229	GBL	2020-10-02	11.26	11.85	11.26	11.65	14000	{}	\N	2020-10-12 14:59:59.437444
45230	GBL	2020-10-05	11.85	11.85	11.78	11.81	10542	{}	\N	2020-10-12 14:59:59.443934
45231	GBL	2020-10-06	11.85	12.17	11.85	11.91	26222	{}	\N	2020-10-12 14:59:59.448912
45232	GBL	2020-10-07	12.05	12.13	11.68	12.01	12782	{}	\N	2020-10-12 14:59:59.453246
45233	GBL	2020-10-08	12.09	12.33	11.495	12.24	39350	{}	\N	2020-10-12 14:59:59.458795
45234	GBL	2020-10-09	12.35	12.75	12.35	12.35	7590	{}	\N	2020-10-12 14:59:59.464328
45235	GBL	2020-10-12	12.35	12.58	12.35	12.58	2996	{}	\N	2020-10-12 14:59:59.468471
45236	GCO	2020-09-28	23.21	23.63	22.33	22.36	563155	{}	\N	2020-10-12 15:00:01.751506
45237	GCO	2020-09-29	22.32	22.32	20.8	21.68	789443	{}	\N	2020-10-12 15:00:01.761225
45238	GCO	2020-09-30	21.82	22.83	21.52	21.54	621538	{}	\N	2020-10-12 15:00:01.766702
45239	GCO	2020-10-01	21.56	22.44	21.35	21.85	821540	{}	\N	2020-10-12 15:00:01.772062
45240	GCO	2020-10-02	20.97	22.85	20.94	22.71	1149468	{}	\N	2020-10-12 15:00:01.777274
45241	GCO	2020-10-05	23.04	23.52	22.06	22.59	553000	{}	\N	2020-10-12 15:00:01.781967
45242	GCO	2020-10-06	22.84	22.84	21.16	21.18	540696	{}	\N	2020-10-12 15:00:01.786035
45243	GCO	2020-10-07	21.63	22.17	21.325	21.83	471534	{}	\N	2020-10-12 15:00:01.790417
45244	GCO	2020-10-08	22.18	22.18	21.32	21.96	299240	{}	\N	2020-10-12 15:00:01.794172
45245	GCO	2020-10-09	21.95	22.27	21.27	21.69	462333	{}	\N	2020-10-12 15:00:01.797817
45246	GCO	2020-10-12	21.86	22.27	21.5	21.55	254628	{}	\N	2020-10-12 15:00:01.801736
45247	GDO	2020-09-28	16.73	16.77	16.64	16.77	18432	{}	\N	2020-10-12 15:00:03.168876
45248	GDO	2020-09-29	16.77	16.7999	16.7	16.78	30792	{}	\N	2020-10-12 15:00:03.177893
45249	GDO	2020-09-30	16.75	16.8787	16.56	16.56	53774	{}	\N	2020-10-12 15:00:03.184596
45250	GDO	2020-10-01	16.86	16.91	16.6901	16.72	74548	{}	\N	2020-10-12 15:00:03.19041
45251	GDO	2020-10-02	16.64	16.89	16.56	16.73	56798	{}	\N	2020-10-12 15:00:03.196185
45252	GDO	2020-10-05	16.68	16.78	16.68	16.76	48106	{}	\N	2020-10-12 15:00:03.201221
45253	GDO	2020-10-06	16.79	17.04	16.76	17.04	54653	{}	\N	2020-10-12 15:00:03.205494
45254	GDO	2020-10-07	17.16	17.39	17.06	17.34	85706	{}	\N	2020-10-12 15:00:03.209977
45255	GDO	2020-10-08	17.38	17.45	17.301	17.45	64120	{}	\N	2020-10-12 15:00:03.214249
45256	GDO	2020-10-09	17.55	17.59	17.428	17.56	61806	{}	\N	2020-10-12 15:00:03.218878
45257	GDO	2020-10-12	17.59	17.59	17.55	17.55	7094	{}	\N	2020-10-12 15:00:03.223151
45258	GE	2020-09-28	6.23	6.31	6.18	6.2	83970650	{}	\N	2020-10-12 15:00:04.275656
45259	GE	2020-09-29	6.21	6.23	6.06	6.12	143933457	{}	\N	2020-10-12 15:00:04.285417
45260	GE	2020-09-30	6.14	6.29	6.14	6.23	186112477	{}	\N	2020-10-12 15:00:04.2922
45261	GE	2020-10-01	6.27	6.29	6.11	6.24	149728856	{}	\N	2020-10-12 15:00:04.298754
45262	GE	2020-10-02	6.07	6.4	6.05	6.39	165374131	{}	\N	2020-10-12 15:00:04.304134
45263	GE	2020-10-05	6.39	6.45	6.32	6.41	118260461	{}	\N	2020-10-12 15:00:04.308671
45264	GE	2020-10-06	6.43	6.58	6.11	6.17	334366932	{}	\N	2020-10-12 15:00:04.313703
45265	GE	2020-10-07	6.22	6.4	6.21	6.31	165835321	{}	\N	2020-10-12 15:00:04.318675
45266	GE	2020-10-08	6.36	6.67	6.34	6.65	205197268	{}	\N	2020-10-12 15:00:04.32306
45267	GE	2020-10-09	7.07	7.07	6.7	6.84	345389130	{}	\N	2020-10-12 15:00:04.328381
45268	GE	2020-10-12	6.92	6.92	6.74	6.75	67146857	{}	\N	2020-10-12 15:00:04.333497
45269	GEO	2020-09-28	11.1	11.58	11.08	11.52	1800561	{}	\N	2020-10-12 15:00:07.701461
45270	GEO	2020-09-29	11.49	11.66	11.22	11.35	2201838	{}	\N	2020-10-12 15:00:07.712345
45271	GEO	2020-09-30	11.3	11.68	11.24	11.34	1982625	{}	\N	2020-10-12 15:00:07.720406
45272	GEO	2020-10-01	11.35	11.58	11.255	11.57	2155857	{}	\N	2020-10-12 15:00:07.727257
45273	GEO	2020-10-02	11.27	11.71	11.27	11.69	1882856	{}	\N	2020-10-12 15:00:07.733073
45274	GEO	2020-10-05	11.83	11.95	11.21	11.31	4102610	{}	\N	2020-10-12 15:00:07.738022
45275	GEO	2020-10-06	11.36	11.51	11.1601	11.21	3910045	{}	\N	2020-10-12 15:00:07.742965
45276	GEO	2020-10-07	11.4	11.4766	11.29	11.37	2009269	{}	\N	2020-10-12 15:00:07.748254
45277	GEO	2020-10-08	11.38	11.44	11.185	11.28	3790577	{}	\N	2020-10-12 15:00:07.753393
45278	GEO	2020-10-09	11.32	11.49	11.095	11.26	4797466	{}	\N	2020-10-12 15:00:07.757796
45279	GEO	2020-10-12	11.3	11.31	11.12	11.2	991771	{}	\N	2020-10-12 15:00:07.762221
45280	GF	2020-09-28	16.56	16.691	16.56	16.6	7946	{}	\N	2020-10-12 15:00:08.718096
45281	GF	2020-09-29	16.52	16.69	16.52	16.655	68702	{}	\N	2020-10-12 15:00:08.725849
45282	GF	2020-09-30	16.71	16.8983	16.6301	16.88	26172	{}	\N	2020-10-12 15:00:08.731146
45283	GF	2020-10-01	16.97	17.12	16.97	17.12	7702	{}	\N	2020-10-12 15:00:08.737338
45284	GF	2020-10-02	16.99	16.99	16.864	16.93	16190	{}	\N	2020-10-12 15:00:08.742404
45285	GF	2020-10-05	17.0101	17.39	17.0101	17.354	19974	{}	\N	2020-10-12 15:00:08.746286
45286	GF	2020-10-06	17.44	17.51	17.1	17.1	24262	{}	\N	2020-10-12 15:00:08.75102
45287	GF	2020-10-07	17.38	17.5364	17.38	17.45	31010	{}	\N	2020-10-12 15:00:08.755908
45288	GF	2020-10-08	17.3534	17.49	17.3534	17.49	1626	{}	\N	2020-10-12 15:00:08.760034
45289	GF	2020-10-09	17.46	17.54	17.41	17.5	5052	{}	\N	2020-10-12 15:00:08.764007
45290	GF	2020-10-12	17.6773	17.7	17.6773	17.7	2724	{}	\N	2020-10-12 15:00:08.769272
45291	GFY	2020-09-28	16.325	16.5	16.325	16.4	4653	{}	\N	2020-10-12 15:00:11.09013
45292	GFY	2020-09-29	16.393	16.393	16.3	16.3211	3046	{}	\N	2020-10-12 15:00:11.094955
45293	GFY	2020-09-30	16.34	16.4	16.34	16.4	2776	{}	\N	2020-10-12 15:00:11.098832
45294	GFY	2020-10-01	16.48	16.48	16.35	16.35	2602	{}	\N	2020-10-12 15:00:11.102949
45295	GFY	2020-10-02	16.3999	16.3999	16.32	16.35	7676	{}	\N	2020-10-12 15:00:11.107045
45296	GFY	2020-10-05	16.3804	16.4124	16.33	16.4124	12750	{}	\N	2020-10-12 15:00:11.111188
45297	GFY	2020-10-06	16.32	16.4246	16.32	16.4246	4720	{}	\N	2020-10-12 15:00:11.115424
45298	GFY	2020-10-07	16.36	16.4611	16.33	16.34	26556	{}	\N	2020-10-12 15:00:11.119892
45299	GFY	2020-10-09	16.35	16.38	16.35	16.38	12870	{}	\N	2020-10-12 15:00:11.125617
45300	FXB	2020-09-28	124.63	124.63	124.11	124.13	18044	{}	\N	2020-10-12 15:00:13.172801
45301	FXB	2020-09-29	124.33	124.54	124.18	124.44	30292	{}	\N	2020-10-12 15:00:13.182874
45302	FXB	2020-09-30	124.38	125.08	124.38	124.83	10911	{}	\N	2020-10-12 15:00:13.190038
45303	FXB	2020-10-01	124.76	124.8269	124.44	124.64	19177	{}	\N	2020-10-12 15:00:13.196843
45304	FXB	2020-10-02	125	125.2	125	125.08	14966	{}	\N	2020-10-12 15:00:13.2025
45305	FXB	2020-10-05	125.5599	125.6	125.4	125.6	42779	{}	\N	2020-10-12 15:00:13.207117
45306	FXB	2020-10-06	125.3606	125.4628	124.73	124.88	31091	{}	\N	2020-10-12 15:00:13.212026
45307	FXB	2020-10-07	124.48	124.955	124.48	124.92	19571	{}	\N	2020-10-12 15:00:13.216512
45308	FXB	2020-10-08	124.9536	125.1767	124.9	125.05	5848	{}	\N	2020-10-12 15:00:13.22073
45309	FXB	2020-10-09	125.46	126.1304	125.4038	126.12	12853	{}	\N	2020-10-12 15:00:13.225925
45310	FXB	2020-10-12	126.07	126.3483	126.07	126.3483	3202	{}	\N	2020-10-12 15:00:13.230749
45311	FXO	2020-09-28	25	25.57	25	25.38	101835	{}	\N	2020-10-12 15:00:14.271623
45312	FXO	2020-09-29	25.28	25.28	24.8196	25.08	118530	{}	\N	2020-10-12 15:00:14.283425
45313	FXO	2020-09-30	25.22	25.62	25.18	25.37	120333	{}	\N	2020-10-12 15:00:14.28953
45314	FXO	2020-10-01	25.43	25.67	25.2542	25.5	96702	{}	\N	2020-10-12 15:00:14.294987
45315	FXO	2020-10-02	25.1	26.22	25.1	26.1	90853	{}	\N	2020-10-12 15:00:14.299772
45316	FXO	2020-10-05	26.65	26.92	26.52	26.855	173257	{}	\N	2020-10-12 15:00:14.304921
45317	FXO	2020-10-06	27.1212	27.63	26.6826	26.76	157444	{}	\N	2020-10-12 15:00:14.309633
45318	FXO	2020-10-07	27.05	27.57	27.05	27.3	98508	{}	\N	2020-10-12 15:00:14.313863
45319	FXO	2020-10-08	27.51	27.81	27.4701	27.74	95274	{}	\N	2020-10-12 15:00:14.318173
45320	FXO	2020-10-09	27.96	28	27.52	27.65	84678	{}	\N	2020-10-12 15:00:14.32296
45321	FXO	2020-10-12	27.69	27.91	27.65	27.8705	11988	{}	\N	2020-10-12 15:00:14.328202
45322	FYX	2020-09-28	56.8	58.1869	56.8	57.99	18536	{}	\N	2020-10-12 15:00:15.433919
45323	FYX	2020-09-29	58.07	58.07	57.37	57.75	6697	{}	\N	2020-10-12 15:00:15.443233
45324	FYX	2020-09-30	58.02	58.87	57.52	57.88	23469	{}	\N	2020-10-12 15:00:15.452553
45325	FYX	2020-10-01	58.29	58.85	57.9101	58.85	12824	{}	\N	2020-10-12 15:00:15.459136
45326	FYX	2020-10-02	57.38	59.7339	57.38	59.64	13223	{}	\N	2020-10-12 15:00:15.464187
45327	FYX	2020-10-05	60.2	60.9999	60.2	60.98	11964	{}	\N	2020-10-12 15:00:15.46825
45328	FYX	2020-10-06	61.6799	62.39	60.71	60.76	93440	{}	\N	2020-10-12 15:00:15.472383
45329	FYX	2020-10-07	61.65	62.15	61.4901	62.01	531279	{}	\N	2020-10-12 15:00:15.476899
45330	FYX	2020-10-08	62.585	62.85	62.195	62.745	28296	{}	\N	2020-10-12 15:00:15.481879
45331	FYX	2020-10-09	63.29	63.45	62.82	63.049	164327	{}	\N	2020-10-12 15:00:15.486538
45332	FYX	2020-10-12	63.3442	63.3819	63.04	63.3571	942	{}	\N	2020-10-12 15:00:15.490376
45333	GBF	2020-09-28	125.53	125.53	125.2899	125.51	7213	{}	\N	2020-10-12 15:00:17.22977
45334	GBF	2020-09-29	125.57	125.78	125.5	125.6	7648	{}	\N	2020-10-12 15:00:17.236858
45335	GBF	2020-09-30	125.3118	125.4195	125.205	125.33	3443	{}	\N	2020-10-12 15:00:17.241925
45336	GBF	2020-10-01	125.1	125.37	124.965	125.145	7568	{}	\N	2020-10-12 15:00:17.247524
45337	GBF	2020-10-02	125.03	125.29	125	125.1864	21316	{}	\N	2020-10-12 15:00:17.252938
45338	GBF	2020-10-05	124.81	125.0321	124.65	124.65	39167	{}	\N	2020-10-12 15:00:17.258106
45339	GBF	2020-10-06	124.47	125.09	124.47	125.09	5748	{}	\N	2020-10-12 15:00:17.262694
45340	GBF	2020-10-07	124.4	124.89	124.4	124.865	10401	{}	\N	2020-10-12 15:00:17.267701
45341	GBF	2020-10-08	124.95	125.015	124.63	124.83	7188	{}	\N	2020-10-12 15:00:17.27206
45342	GBF	2020-10-09	124.9741	124.9741	124.8295	124.88	6922	{}	\N	2020-10-12 15:00:17.276856
45343	GBF	2020-10-12	125.3767	125.3767	125.335	125.335	970	{}	\N	2020-10-12 15:00:17.281852
45344	GCC	2020-09-28	17.26	17.265	17.14	17.19	13706	{}	\N	2020-10-12 15:00:18.229568
45345	GCC	2020-09-29	17.12	17.169	17.08	17.12	98914	{}	\N	2020-10-12 15:00:18.238087
45346	GCC	2020-09-30	17.1401	17.39	17.1401	17.29	17088	{}	\N	2020-10-12 15:00:18.243874
45347	GCC	2020-10-01	17.11	17.23	17.11	17.14	179296	{}	\N	2020-10-12 15:00:18.249281
45348	GCC	2020-10-02	17.09	17.13	17.04	17.04	25613	{}	\N	2020-10-12 15:00:18.255592
45349	GCC	2020-10-05	17.15	17.3425	17.15	17.32	28507	{}	\N	2020-10-12 15:00:18.260025
45350	GCC	2020-10-06	17.43	17.455	17.285	17.285	31205	{}	\N	2020-10-12 15:00:18.264645
45351	GCC	2020-10-07	17.38	17.48	17	17.4	30446	{}	\N	2020-10-12 15:00:18.26958
45352	GCC	2020-10-08	17.59	17.6	17.51	17.52	11424	{}	\N	2020-10-12 15:00:18.274032
45353	GCC	2020-10-09	17.66	17.74	17.66	17.72	13789	{}	\N	2020-10-12 15:00:18.278454
45354	GCC	2020-10-12	17.81	17.81	17.587	17.587	13244	{}	\N	2020-10-12 15:00:18.283239
45355	GD	2020-09-28	139.7	142.395	139.7	140.92	730139	{}	\N	2020-10-12 15:00:20.375973
45356	GD	2020-09-29	140.43	140.58	137.89	138.55	2052336	{}	\N	2020-10-12 15:00:20.383251
45357	GD	2020-09-30	139.16	140.14	137.07	138.43	2304675	{}	\N	2020-10-12 15:00:20.389534
45358	GD	2020-10-01	138.89	140.08	136.8	138.22	1665301	{}	\N	2020-10-12 15:00:20.395151
45359	GD	2020-10-02	133.72	140.9411	133.6	140.49	2544758	{}	\N	2020-10-12 15:00:20.400059
45360	GD	2020-10-05	141.98	143.35	141.11	142.73	1859701	{}	\N	2020-10-12 15:00:20.404407
45361	GD	2020-10-06	144.08	146.97	142.03	142.31	2619552	{}	\N	2020-10-12 15:00:20.408722
45362	GD	2020-10-07	143.67	145.65	143.53	144.87	2386073	{}	\N	2020-10-12 15:00:20.412696
45363	GD	2020-10-08	145	145.66	143.6	145.04	1725436	{}	\N	2020-10-12 15:00:20.416834
45364	GD	2020-10-09	146.11	146.11	143.53	143.66	3830649	{}	\N	2020-10-12 15:00:20.421844
45365	GD	2020-10-12	144.8	146.57	143.8401	145.73	686511	{}	\N	2020-10-12 15:00:20.426004
45366	GDV	2020-09-28	18.19	18.34	18.14	18.16	265847	{}	\N	2020-10-12 15:00:21.382472
45367	GDV	2020-09-29	18.21	18.24	18.04	18.11	482530	{}	\N	2020-10-12 15:00:21.388382
45368	GDV	2020-09-30	18.14	18.48	18.14	18.35	612975	{}	\N	2020-10-12 15:00:21.393575
45369	GDV	2020-10-01	18.48	18.66	18.39	18.56	505820	{}	\N	2020-10-12 15:00:21.398465
45370	GDV	2020-10-02	18.4	18.8	18.3297	18.68	1190665	{}	\N	2020-10-12 15:00:21.403205
45371	GDV	2020-10-05	18.81	18.98	18.81	18.96	597227	{}	\N	2020-10-12 15:00:21.407984
45372	GDV	2020-10-06	19.09	19.13	18.6899	18.77	713272	{}	\N	2020-10-12 15:00:21.41225
45373	GDV	2020-10-07	18.91	19.04	18.91	18.96	685752	{}	\N	2020-10-12 15:00:21.417791
45374	GDV	2020-10-08	19.13	19.23	18.98	19.09	468142	{}	\N	2020-10-12 15:00:21.422924
45375	GDV	2020-10-09	19.13	19.24	19.04	19.08	383390	{}	\N	2020-10-12 15:00:21.428253
45376	GDV	2020-10-12	19.25	19.2994	19.13	19.27	157735	{}	\N	2020-10-12 15:00:21.43345
45377	GECCM	2020-09-28	22.2	22.44	22.0918	22.329	2196	{}	\N	2020-10-12 15:00:22.386176
45378	GECCM	2020-09-29	22.2	22.2	21.9724	22.15	1379	{}	\N	2020-10-12 15:00:22.395288
45379	GECCM	2020-09-30	22.225	22.515	22.225	22.515	3039	{}	\N	2020-10-12 15:00:22.401934
45380	GECCM	2020-10-01	22.9	22.9699	22.6	22.81	4193	{}	\N	2020-10-12 15:00:22.407527
45381	GECCM	2020-10-02	22.973	23	22.5	22.5	1842	{}	\N	2020-10-12 15:00:22.412566
45382	GECCM	2020-10-05	22.5	22.72	21.85	22.2557	3097	{}	\N	2020-10-12 15:00:22.417383
45383	GECCM	2020-10-06	21.85	22	21.85	22	2228	{}	\N	2020-10-12 15:00:22.422374
45384	GECCM	2020-10-07	22	22.125	21.6	21.8691	10868	{}	\N	2020-10-12 15:00:22.427645
45385	GECCM	2020-10-08	21.51	21.9	21.43	21.9	15653	{}	\N	2020-10-12 15:00:22.432581
45386	GECCM	2020-10-09	22.01	22.1899	21.4801	22.11	1140	{}	\N	2020-10-12 15:00:22.437319
45387	GECCM	2020-10-12	21.9	21.9	21.7	21.7	765	{}	\N	2020-10-12 15:00:22.442404
45388	GENY	2020-09-28	48.15	48.24	47.9701	48.114	2976	{}	\N	2020-10-12 15:00:23.480363
45389	GENY	2020-09-29	48.35	48.355	48.07	48.07	851	{}	\N	2020-10-12 15:00:23.489343
45390	GENY	2020-09-30	48.77	48.94	48.461	48.69	6288	{}	\N	2020-10-12 15:00:23.496544
45391	GENY	2020-10-01	49.29	49.37	49.29	49.37	731	{}	\N	2020-10-12 15:00:23.50322
45392	GENY	2020-10-02	48.88	49.06	48.66	49.06	644	{}	\N	2020-10-12 15:00:23.508758
45393	GENY	2020-10-05	49.6735	50.04	49.63	50.04	955	{}	\N	2020-10-12 15:00:23.513467
45394	GENY	2020-10-06	49.9	49.9	49.2658	49.2658	2019	{}	\N	2020-10-12 15:00:23.518307
45395	GENY	2020-10-09	50.62	50.8	50.62	50.7899	1608	{}	\N	2020-10-12 15:00:23.523196
45396	GENY	2020-10-12	51.19	51.19	51.15	51.15	632	{}	\N	2020-10-12 15:00:23.528253
45397	OCFC	2020-09-28	13.79	14.17	13.79	14.08	112194	{}	\N	2020-10-12 15:00:24.633508
45398	OCFC	2020-09-29	13.81	13.91	13.56	13.79	180089	{}	\N	2020-10-12 15:00:24.641921
45399	OCFC	2020-09-30	13.89	14.055	13.59	13.69	152340	{}	\N	2020-10-12 15:00:24.648559
45400	OCFC	2020-10-01	13.83	13.845	13.53	13.74	159069	{}	\N	2020-10-12 15:00:24.654176
45401	OCFC	2020-10-02	13.55	14.61	13.53	14.525	285116	{}	\N	2020-10-12 15:00:24.660165
45402	OCFC	2020-10-05	14.57	14.8	14.44	14.53	237996	{}	\N	2020-10-12 15:00:24.665152
45403	OCFC	2020-10-06	14.73	15.43	14.62	14.91	225635	{}	\N	2020-10-12 15:00:24.670012
45404	OCFC	2020-10-07	15.02	15.33	14.87	15.19	212554	{}	\N	2020-10-12 15:00:24.675259
45405	OCFC	2020-10-08	15.32	15.62	15.11	15.43	135348	{}	\N	2020-10-12 15:00:24.679828
45406	OCFC	2020-10-09	15.54	15.64	15.12	15.21	73568	{}	\N	2020-10-12 15:00:24.684361
45407	OCFC	2020-10-12	15.13	15.355	15.08	15.325	8079	{}	\N	2020-10-12 15:00:24.688715
45408	FWONA	2020-09-28	33.37	33.37	32.92	33.04	72886	{}	\N	2020-10-12 15:00:26.322632
45409	FWONA	2020-09-29	33.09	33.09	32.26	32.65	92920	{}	\N	2020-10-12 15:00:26.330844
45410	FWONA	2020-09-30	33.07	34.42	32.66	33.51	117482	{}	\N	2020-10-12 15:00:26.337719
45411	FWONA	2020-10-01	33.99	34.71	33.95	34.33	108940	{}	\N	2020-10-12 15:00:26.3434
45412	FWONA	2020-10-02	34.06	34.8	33.82	34.25	98173	{}	\N	2020-10-12 15:00:26.348767
45413	FWONA	2020-10-05	34.3	34.74	33.98	34.57	89648	{}	\N	2020-10-12 15:00:26.353721
45414	FWONA	2020-10-06	34.59	35.11	34.44	34.58	108732	{}	\N	2020-10-12 15:00:26.358354
45415	FWONA	2020-10-07	35.05	35.31	34.31	34.94	73006	{}	\N	2020-10-12 15:00:26.363464
45416	FWONA	2020-10-08	35.1	35.1	34.56	34.65	72419	{}	\N	2020-10-12 15:00:26.369544
45417	FWONA	2020-10-09	34.88	35	34.28	34.29	38076	{}	\N	2020-10-12 15:00:26.374931
45418	FWONA	2020-10-12	34.32	34.38	34.19	34.335	8355	{}	\N	2020-10-12 15:00:26.37942
45419	FXP	2020-09-28	44.17	44.51	44.09	44.13	27634	{}	\N	2020-10-12 15:00:28.317281
45420	FXP	2020-09-29	44.81	44.86	44.26	44.5	31246	{}	\N	2020-10-12 15:00:28.329404
45421	FXP	2020-09-30	43.4	43.505	42.57	42.69	51003	{}	\N	2020-10-12 15:00:28.337566
45422	FXP	2020-10-01	41.88	42.26	41.52	41.65	40862	{}	\N	2020-10-12 15:00:28.343808
45423	FXP	2020-10-02	43	43	41.68	42.64	60620	{}	\N	2020-10-12 15:00:28.350324
45424	FXP	2020-10-05	42.83	42.83	42.2	42.2	33994	{}	\N	2020-10-12 15:00:28.35489
45425	FXP	2020-10-06	41.5757	41.88	41.21	41.62	64501	{}	\N	2020-10-12 15:00:28.360018
45426	FXP	2020-10-07	40.79	41.2	40.64	40.72	39242	{}	\N	2020-10-12 15:00:28.365436
45427	FXP	2020-10-08	40.7	40.83	40.31	40.36	17231	{}	\N	2020-10-12 15:00:28.370495
45428	FXP	2020-10-09	40.6	40.64	39.99	40.19	19652	{}	\N	2020-10-12 15:00:28.37477
45429	FXP	2020-10-12	38.25	38.47	38.13	38.15	11970	{}	\N	2020-10-12 15:00:28.378916
45430	FYLD	2020-09-28	19.92	20.01	19.76	20.01	1013	{}	\N	2020-10-12 15:00:29.37584
45431	FYLD	2020-09-29	20.001	20.1499	19.89	19.89	2953	{}	\N	2020-10-12 15:00:29.384035
45432	FYLD	2020-09-30	19.975	19.975	19.9125	19.9125	523	{}	\N	2020-10-12 15:00:29.39195
45433	FYLD	2020-10-01	19.9	20.1866	19.9	20.1866	1384	{}	\N	2020-10-12 15:00:29.398857
45434	FYLD	2020-10-02	19.94	20.15	19.91	20.15	5445	{}	\N	2020-10-12 15:00:29.405011
45435	FYLD	2020-10-05	20.36	20.39	20.2701	20.39	3801	{}	\N	2020-10-12 15:00:29.409248
45436	FYLD	2020-10-06	20.49	20.81	20.3555	20.3555	1209	{}	\N	2020-10-12 15:00:29.413581
45437	FYLD	2020-10-07	20.58	20.65	20.48	20.48	3221	{}	\N	2020-10-12 15:00:29.418441
45438	FYLD	2020-10-08	20.61	20.7163	20.61	20.64	6243	{}	\N	2020-10-12 15:00:29.422961
45439	FYLD	2020-10-09	20.76	20.785	20.74	20.785	2246	{}	\N	2020-10-12 15:00:29.427702
45440	FYLD	2020-10-12	20.8	20.81	20.8	20.81	433	{}	\N	2020-10-12 15:00:29.432036
45441	GAIN	2020-09-28	9.13	9.45	9.11	9.37	284685	{}	\N	2020-10-12 15:00:30.420948
45442	GAIN	2020-09-29	9.35	9.42	9.19	9.2	132239	{}	\N	2020-10-12 15:00:30.429826
45443	GAIN	2020-09-30	9.3	9.34	9.1	9.1	174182	{}	\N	2020-10-12 15:00:30.436251
45444	GAIN	2020-10-01	9.15	9.21	9.09	9.1	135219	{}	\N	2020-10-12 15:00:30.441563
45445	GAIN	2020-10-02	9	9.2	8.97	9.1	117159	{}	\N	2020-10-12 15:00:30.446947
45446	GAIN	2020-10-05	9.12	9.2206	9.0706	9.1	145001	{}	\N	2020-10-12 15:00:30.451617
45447	GAIN	2020-10-06	9.1044	9.18	9.06	9.07	105986	{}	\N	2020-10-12 15:00:30.456338
45448	GAIN	2020-10-07	9.14	9.22	9.09	9.14	89019	{}	\N	2020-10-12 15:00:30.460588
45449	GAIN	2020-10-08	9.1	9.24	9.1	9.22	87116	{}	\N	2020-10-12 15:00:30.465175
45450	GAIN	2020-10-09	9.21	9.25	9.11	9.17	65498	{}	\N	2020-10-12 15:00:30.469843
45451	GAIN	2020-10-12	9.19	9.25	9.17	9.18	36591	{}	\N	2020-10-12 15:00:30.474513
45452	GASS	2020-09-28	2.72	2.8	2.65	2.8	27815	{}	\N	2020-10-12 15:00:31.407505
45453	GASS	2020-09-29	2.84	2.97	2.7354	2.94	71817	{}	\N	2020-10-12 15:00:31.416616
45454	GASS	2020-09-30	2.945	2.99	2.9013	2.97	115907	{}	\N	2020-10-12 15:00:31.423563
45455	GASS	2020-10-01	3	3.0105	2.91	2.93	22876	{}	\N	2020-10-12 15:00:31.429518
45456	GASS	2020-10-02	2.8	2.92	2.8	2.9	22251	{}	\N	2020-10-12 15:00:31.434431
45457	GASS	2020-10-05	2.95	2.96	2.8801	2.95	14206	{}	\N	2020-10-12 15:00:31.439246
45458	GASS	2020-10-06	2.945	2.97	2.9	2.9	20189	{}	\N	2020-10-12 15:00:31.443184
45459	GASS	2020-10-07	2.9	2.92	2.765	2.86	22115	{}	\N	2020-10-12 15:00:31.447482
45460	GASS	2020-10-08	2.89	2.89	2.795	2.8425	10071	{}	\N	2020-10-12 15:00:31.451135
45461	GASS	2020-10-09	2.86	2.8762	2.83	2.83	23931	{}	\N	2020-10-12 15:00:31.455835
45462	GASS	2020-10-12	2.81	2.835	2.7824	2.7824	8882	{}	\N	2020-10-12 15:00:31.459656
45463	GBLIL	2020-09-28	26.84	26.985	26.76	26.965	15671	{}	\N	2020-10-12 15:00:33.000394
45464	GBLIL	2020-09-29	26.97	27.27	26.88	27.27	9940	{}	\N	2020-10-12 15:00:33.00802
45465	GBLIL	2020-09-30	26.41	26.4628	25.72	25.75	98306	{}	\N	2020-10-12 15:00:33.013182
45466	GBLIL	2020-10-01	25.95	26.2499	25.86	26.07	17615	{}	\N	2020-10-12 15:00:33.018005
45467	GBLIL	2020-10-02	25.89	26.0077	25.75	25.89	10684	{}	\N	2020-10-12 15:00:33.022873
45468	GBLIL	2020-10-05	25.89	26.17	25.88	26.04	4218	{}	\N	2020-10-12 15:00:33.027293
45469	GBLIL	2020-10-06	25.935	26.05	25.8	25.92	10742	{}	\N	2020-10-12 15:00:33.031388
45470	GBLIL	2020-10-07	25.98	26.07	25.94	26	6158	{}	\N	2020-10-12 15:00:33.036202
45471	GBLIL	2020-10-08	25.995	26.15	25.98	26.15	3724	{}	\N	2020-10-12 15:00:33.043327
45472	GBLIL	2020-10-09	26.0353	26.1157	26	26.08	6153	{}	\N	2020-10-12 15:00:33.047818
45473	GDEN	2020-09-28	14.34	14.82	14.16	14.22	185113	{}	\N	2020-10-12 15:00:35.213441
45474	GDEN	2020-09-29	14.19	14.45	13.34	13.59	265600	{}	\N	2020-10-12 15:00:35.221541
45475	GDEN	2020-09-30	13.69	14.525	13.5014	13.81	238224	{}	\N	2020-10-12 15:00:35.227772
45476	GDEN	2020-10-01	13.95	14.21	13.73	14.07	129758	{}	\N	2020-10-12 15:00:35.233209
45477	GDEN	2020-10-02	13.47	14.33	13.37	14.13	96894	{}	\N	2020-10-12 15:00:35.237891
45478	GDEN	2020-10-05	14.37	14.55	13.85	14.14	103476	{}	\N	2020-10-12 15:00:35.242125
45479	GDEN	2020-10-06	14.19	14.51	13.74	13.78	146817	{}	\N	2020-10-12 15:00:35.246253
45480	GDEN	2020-10-07	14.16	14.38	13.79	13.86	190509	{}	\N	2020-10-12 15:00:35.250055
45481	GDEN	2020-10-08	14.03	14.39	13.62	13.85	121501	{}	\N	2020-10-12 15:00:35.253533
45482	GDEN	2020-10-09	13.76	14.25	13.59	13.62	122440	{}	\N	2020-10-12 15:00:35.257347
45483	GDEN	2020-10-12	13.65	13.6699	13.0901	13.36	83604	{}	\N	2020-10-12 15:00:35.261416
45484	GDXJ	2020-09-28	55.32	55.68	54.4	55.26	7166412	{}	\N	2020-10-12 15:00:36.707517
45485	GDXJ	2020-09-29	55.43	56.36	55.3	56.11	6600505	{}	\N	2020-10-12 15:00:36.71512
45486	GDXJ	2020-09-30	55.58	56.07	54.81	55.36	6653420	{}	\N	2020-10-12 15:00:36.721532
45487	GDXJ	2020-10-01	56.21	56.95	55.6101	56.31	6863513	{}	\N	2020-10-12 15:00:36.726732
45488	GDXJ	2020-10-02	55.5235	56.33	54.98	55.32	6601613	{}	\N	2020-10-12 15:00:36.732177
45489	GDXJ	2020-10-05	55.88	57	55.77	56.58	7409567	{}	\N	2020-10-12 15:00:36.736817
45490	GDXJ	2020-10-06	57.23	57.5	54.531	54.66	8860131	{}	\N	2020-10-12 15:00:36.740911
45491	GDXJ	2020-10-07	55.47	56	54.88	55.28	4155451	{}	\N	2020-10-12 15:00:36.745251
45492	GDXJ	2020-10-08	55.91	56.6598	55.56	56.49	5309231	{}	\N	2020-10-12 15:00:36.750128
45493	GDXJ	2020-10-09	57.85	59.59	57.82	59.5	10765463	{}	\N	2020-10-12 15:00:36.754438
45494	GDXJ	2020-10-12	59.13	59.42	58.59	59.09	1516553	{}	\N	2020-10-12 15:00:36.758386
45495	GEL	2020-09-28	4.65	4.71	4.37	4.61	846844	{}	\N	2020-10-12 15:00:38.972312
45496	GEL	2020-09-29	4.55	4.71	4.45	4.55	2478811	{}	\N	2020-10-12 15:00:38.983353
45497	GEL	2020-09-30	4.53	4.7206	4.42	4.5	3158409	{}	\N	2020-10-12 15:00:38.990299
45498	GEL	2020-10-01	4.5	4.67	4.36	4.62	1533699	{}	\N	2020-10-12 15:00:38.995115
45499	GEL	2020-10-02	4.49	4.8	4.32	4.71	995997	{}	\N	2020-10-12 15:00:38.999319
45500	GEL	2020-10-05	4.79	4.99	4.75	4.99	1347991	{}	\N	2020-10-12 15:00:39.004133
45501	GEL	2020-10-06	4.99	5.18	4.73	4.85	1115301	{}	\N	2020-10-12 15:00:39.008078
45502	GEL	2020-10-07	4.94	4.99	4.7	4.76	1417570	{}	\N	2020-10-12 15:00:39.011654
45503	GEL	2020-10-08	4.83	5	4.72	4.99	1586197	{}	\N	2020-10-12 15:00:39.015094
45504	GEL	2020-10-09	5.03	5.05	4.84	4.88	1417969	{}	\N	2020-10-12 15:00:39.019311
45505	GEL	2020-10-12	4.9	4.91	4.73	4.79	250631	{}	\N	2020-10-12 15:00:39.023343
45506	GERN	2020-09-28	1.73	1.73	1.67	1.71	1382082	{}	\N	2020-10-12 15:00:39.999805
45507	GERN	2020-09-29	1.72	1.82	1.7	1.76	2504258	{}	\N	2020-10-12 15:00:40.010438
45508	GERN	2020-09-30	1.78	1.835	1.715	1.73	2619096	{}	\N	2020-10-12 15:00:40.015693
45509	GERN	2020-10-01	1.77	1.79	1.74	1.77	2841207	{}	\N	2020-10-12 15:00:40.023519
45510	GERN	2020-10-02	1.77	1.88	1.74	1.82	3106282	{}	\N	2020-10-12 15:00:40.032439
45511	GERN	2020-10-05	1.85	1.99	1.84	1.99	3347545	{}	\N	2020-10-12 15:00:40.046017
45512	GERN	2020-10-06	1.94	2.04	1.935	1.97	2798055	{}	\N	2020-10-12 15:00:40.0533
45513	GERN	2020-10-07	1.99	2.05	1.98	2.04	3444955	{}	\N	2020-10-12 15:00:40.058363
45514	GERN	2020-10-08	2.05	2.09	1.98	1.99	5605764	{}	\N	2020-10-12 15:00:40.063026
45515	GERN	2020-10-09	2	2.0394	1.96	2	2391081	{}	\N	2020-10-12 15:00:40.068414
45516	GERN	2020-10-12	2	2.04	1.96	2.04	583560	{}	\N	2020-10-12 15:00:40.077528
45517	GFNCP	2020-09-28	101	102.1985	101	102.1985	723	{}	\N	2020-10-12 15:00:41.100744
45518	GFNCP	2020-09-29	100.5	103.05	100.5	103.05	534	{}	\N	2020-10-12 15:00:41.107367
45519	GFNCP	2020-09-30	101.68	102.475	101.68	101.9001	1422	{}	\N	2020-10-12 15:00:41.111941
45520	GFNCP	2020-10-01	102	102	101.25	101.6744	703	{}	\N	2020-10-12 15:00:41.116541
45521	GFNCP	2020-10-05	101.9	101.9	101.3	101.3	745	{}	\N	2020-10-12 15:00:41.122464
45522	GFNCP	2020-10-06	101.3	101.3	101.26	101.26	502	{}	\N	2020-10-12 15:00:41.126916
45523	GFNCP	2020-10-09	101.26	101.3895	99	99	1350	{}	\N	2020-10-12 15:00:41.131112
45524	GFNCP	2020-10-12	101.49	101.49	101.49	101.49	290	{}	\N	2020-10-12 15:00:41.135846
45525	GGO	2020-09-28	10.515	10.515	10.515	10.515	320	{}	\N	2020-10-12 15:00:42.298678
45526	GGO	2020-10-06	10.21	10.21	10.2	10.21	1978	{}	\N	2020-10-12 15:00:42.306582
45527	GGO	2020-10-09	10.9587	10.9587	10.9587	10.9587	153	{}	\N	2020-10-12 15:00:42.31302
45528	FXF	2020-09-28	98.59	98.91	98.41	98.91	24354	{}	\N	2020-10-12 15:00:44.396164
45529	FXF	2020-09-29	99.11	99.415	99.11	99.37	33003	{}	\N	2020-10-12 15:00:44.408441
45530	FXF	2020-09-30	99.1	99.63	98.88	99.18	28678	{}	\N	2020-10-12 15:00:44.415503
45531	FXF	2020-10-01	99.51	99.57	99.38	99.52	30774	{}	\N	2020-10-12 15:00:44.420206
45532	FXF	2020-10-02	99.23	99.33	99.2	99.25	10462	{}	\N	2020-10-12 15:00:44.424767
45533	FXF	2020-10-05	99.76	99.91	99.75	99.84	49515	{}	\N	2020-10-12 15:00:44.429529
45534	FXF	2020-10-06	100.03	100.03	99.55	99.68	22594	{}	\N	2020-10-12 15:00:44.433764
45535	FXF	2020-10-07	99.711	99.7368	99.6	99.62	28563	{}	\N	2020-10-12 15:00:44.437433
45536	FXF	2020-10-08	99.41	99.66	99.38	99.64	28504	{}	\N	2020-10-12 15:00:44.441565
45537	FXF	2020-10-09	100.18	100.45	100.109	100.43	27475	{}	\N	2020-10-12 15:00:44.445173
45538	FXF	2020-10-12	100.49	100.53	100.43	100.5183	9404	{}	\N	2020-10-12 15:00:44.448817
45539	G	2020-09-28	38.67	39.36	38.65	38.98	583205	{}	\N	2020-10-12 15:00:46.071571
45540	G	2020-09-29	38.98	39.15	38.46	38.68	761942	{}	\N	2020-10-12 15:00:46.080012
45541	G	2020-09-30	38.7	39.36	38.7	38.95	1390097	{}	\N	2020-10-12 15:00:46.086519
45542	G	2020-10-01	39.05	39.53	38.66	38.74	1816174	{}	\N	2020-10-12 15:00:46.092215
45543	G	2020-10-02	38.12	39.02	38.12	38.57	1725285	{}	\N	2020-10-12 15:00:46.098196
45544	G	2020-10-05	39.01	39.08	38.5	38.83	1970876	{}	\N	2020-10-12 15:00:46.103444
45545	G	2020-10-06	39.05	39.17	37.96	38.18	2516022	{}	\N	2020-10-12 15:00:46.109507
45546	G	2020-10-07	38.5	39.08	38.37	38.5	4537620	{}	\N	2020-10-12 15:00:46.117805
45547	G	2020-10-08	38.8	39.025	38.68	38.89	4467108	{}	\N	2020-10-12 15:00:46.123315
45548	G	2020-10-09	39.32	39.6904	38.96	39.4	3054446	{}	\N	2020-10-12 15:00:46.128931
45549	G	2020-10-12	39.74	39.8676	39.345	39.45	270172	{}	\N	2020-10-12 15:00:46.135442
45550	GAL	2020-09-28	38.739	38.86	38.739	38.86	1909	{}	\N	2020-10-12 15:00:47.098955
45551	GAL	2020-09-29	38.85	38.855	38.6501	38.82	6901	{}	\N	2020-10-12 15:00:47.105181
45552	GAL	2020-09-30	38.7	39.1	38.7	38.94	13147	{}	\N	2020-10-12 15:00:47.110217
45553	GAL	2020-10-01	39.045	39.3	39	39.3	1787	{}	\N	2020-10-12 15:00:47.115477
45554	GAL	2020-10-02	38.86	39.009	38.86	39	9174	{}	\N	2020-10-12 15:00:47.120749
45555	GAL	2020-10-05	39.2887	39.38	39.256	39.38	7843	{}	\N	2020-10-12 15:00:47.12547
45556	GAL	2020-10-06	39.4224	39.45	39.0896	39.0896	2506	{}	\N	2020-10-12 15:00:47.130331
45557	GAL	2020-10-07	39.31	39.39	39.3001	39.39	4227	{}	\N	2020-10-12 15:00:47.134714
45558	GAL	2020-10-08	39.5382	39.677	39.5382	39.67	4601	{}	\N	2020-10-12 15:00:47.138634
45559	GAL	2020-10-09	39.9	39.92	39.875	39.8865	2918	{}	\N	2020-10-12 15:00:47.143521
45560	GAL	2020-10-12	40.05	40.099	39.99	40.099	8968	{}	\N	2020-10-12 15:00:47.149535
45561	GCI	2020-09-28	1.3	1.43	1.3	1.38	1149499	{}	\N	2020-10-12 15:00:49.00165
45562	GCI	2020-09-29	1.41	1.47	1.34	1.34	1994914	{}	\N	2020-10-12 15:00:49.009135
45563	GCI	2020-09-30	1.34	1.4	1.295	1.3	2511745	{}	\N	2020-10-12 15:00:49.014696
45564	GCI	2020-10-01	1.31	1.36	1.29	1.36	1644256	{}	\N	2020-10-12 15:00:49.020712
45565	GCI	2020-10-02	1.27	1.37	1.26	1.35	1734573	{}	\N	2020-10-12 15:00:49.027137
45566	GCI	2020-10-05	1.35	1.42	1.35	1.39	1375741	{}	\N	2020-10-12 15:00:49.036974
45567	GCI	2020-10-06	1.41	1.54	1.4	1.46	3223927	{}	\N	2020-10-12 15:00:49.047911
45568	GCI	2020-10-07	1.49	1.49	1.38	1.41	2475764	{}	\N	2020-10-12 15:00:49.084254
45569	GCI	2020-10-08	1.43	1.51	1.43	1.5	1781755	{}	\N	2020-10-12 15:00:49.095081
45570	GCI	2020-10-09	1.52	1.53	1.43	1.44	1830889	{}	\N	2020-10-12 15:00:49.102297
45571	GCI	2020-10-12	1.43	1.4405	1.38	1.38	659327	{}	\N	2020-10-12 15:00:49.109346
45572	GDDY	2020-09-28	76.37	76.68	73.96	74.93	1799731	{}	\N	2020-10-12 15:00:50.010728
45573	GDDY	2020-09-29	74.98	77.98	74.7	76.14	4791664	{}	\N	2020-10-12 15:00:50.016435
45574	GDDY	2020-09-30	75.8	76.69	75.38	75.97	2232810	{}	\N	2020-10-12 15:00:50.022251
45575	GDDY	2020-10-01	76.99	77.97	76.03	77.39	2389242	{}	\N	2020-10-12 15:00:50.026938
45576	GDDY	2020-10-02	75.62	76.68	74.93	75.16	1812424	{}	\N	2020-10-12 15:00:50.03617
45577	GDDY	2020-10-05	75.62	77.6	74.78	77.09	2141503	{}	\N	2020-10-12 15:00:50.045143
45578	GDDY	2020-10-06	77.5	78.34	73.55	74.81	3675926	{}	\N	2020-10-12 15:00:50.052932
45579	GDDY	2020-10-07	75.36	76.76	74.6	75.4	1561738	{}	\N	2020-10-12 15:00:50.059856
45580	GDDY	2020-10-08	76.5	76.5	74.46	75.05	1448108	{}	\N	2020-10-12 15:00:50.065758
45581	GDDY	2020-10-09	75.43	77.26	75.21	76.83	1561427	{}	\N	2020-10-12 15:00:50.071486
45582	GDDY	2020-10-12	78.1	78.44	76.735	77.17	177197	{}	\N	2020-10-12 15:00:50.077066
45583	GDX	2020-09-28	38.92	39.16	38.3699	38.87	17265974	{}	\N	2020-10-12 15:00:51.026921
45584	GDX	2020-09-29	39.07	39.62	38.94	39.37	16708719	{}	\N	2020-10-12 15:00:51.077156
45585	GDX	2020-09-30	39.1	39.52	38.56	39.16	17292967	{}	\N	2020-10-12 15:00:51.108423
45586	GDX	2020-10-01	39.55	39.94	39.16	39.57	21628557	{}	\N	2020-10-12 15:00:51.132294
45587	GDX	2020-10-02	39.11	39.57	38.76	38.97	16399664	{}	\N	2020-10-12 15:00:51.145834
45588	GDX	2020-10-05	39.17	39.98	39.07	39.57	19607337	{}	\N	2020-10-12 15:00:51.15352
45589	GDX	2020-10-06	39.87	40.06	38.07	38.11	26027256	{}	\N	2020-10-12 15:00:51.161163
45590	GDX	2020-10-07	38.5	38.805	38.15	38.35	22009501	{}	\N	2020-10-12 15:00:51.167155
45591	GDX	2020-10-08	38.72	39.17	38.48	38.97	18418017	{}	\N	2020-10-12 15:00:51.17371
45592	GDX	2020-10-09	39.91	40.795	39.83	40.78	26680612	{}	\N	2020-10-12 15:00:51.179855
45593	GDX	2020-10-12	40.68	40.8	40.35	40.6	4695563	{}	\N	2020-10-12 15:00:51.185475
45594	GEF	2020-09-28	36.62	36.97	36.04	36.79	240632	{}	\N	2020-10-12 15:00:52.538247
45595	GEF	2020-09-29	36.85	36.86	35.98	36.22	333571	{}	\N	2020-10-12 15:00:52.548206
45596	GEF	2020-09-30	36.25	36.78	35.97	36.21	385624	{}	\N	2020-10-12 15:00:52.555301
45597	GEF	2020-10-01	36.26	36.61	35.45	35.93	399577	{}	\N	2020-10-12 15:00:52.561092
45598	GEF	2020-10-02	35.18	37.855	35.15	37.66	421894	{}	\N	2020-10-12 15:00:52.566803
45599	GEF	2020-10-05	37.9	39.33	37.885	39.22	424544	{}	\N	2020-10-12 15:00:52.571811
45600	GEF	2020-10-06	39.63	40.3325	39.07	39.09	521894	{}	\N	2020-10-12 15:00:52.577409
45601	GEF	2020-10-07	39.61	40.52	39.385	40.45	302179	{}	\N	2020-10-12 15:00:52.581916
45602	GEF	2020-10-08	40.8	41.67	40.7	41.49	406419	{}	\N	2020-10-12 15:00:52.586736
45603	GEF	2020-10-09	41.76	41.88	40.7068	40.92	319971	{}	\N	2020-10-12 15:00:52.591679
45604	GEF	2020-10-12	42.32	43.87	42.18	43.02	148182	{}	\N	2020-10-12 15:00:52.596798
45605	GER	2020-09-28	6.13	6.249	6.0839	6.18	52522	{}	\N	2020-10-12 15:00:53.518594
45606	GER	2020-09-29	6.21	6.25	6.13	6.19	80542	{}	\N	2020-10-12 15:00:53.524806
45607	GER	2020-09-30	6.15	6.26	5.98	5.98	190492	{}	\N	2020-10-12 15:00:53.529742
45608	GER	2020-10-01	5.99	5.99	5.84	5.88	244380	{}	\N	2020-10-12 15:00:53.534706
45609	GER	2020-10-02	5.7	5.99	5.7	5.95	92209	{}	\N	2020-10-12 15:00:53.540094
45610	GER	2020-10-05	6	6.28	6	6.28	145607	{}	\N	2020-10-12 15:00:53.544825
45611	GER	2020-10-06	6.3	6.4	6.17	6.19	184168	{}	\N	2020-10-12 15:00:53.549287
45612	GER	2020-10-07	6.2	6.3	6.175	6.21	89296	{}	\N	2020-10-12 15:00:53.553822
45613	GER	2020-10-08	6.23	6.56	6.23	6.56	138650	{}	\N	2020-10-12 15:00:53.558156
45614	GER	2020-10-09	6.58	6.585	6.47	6.51	176888	{}	\N	2020-10-12 15:00:53.562438
45615	GER	2020-10-12	6.46	6.5	6.41	6.42	51894	{}	\N	2020-10-12 15:00:53.56694
45616	GFF	2020-09-28	19.97	20.31	19.88	20.16	336325	{}	\N	2020-10-12 15:00:54.566655
45617	GFF	2020-09-29	20.17	20.27	19.68	19.68	471843	{}	\N	2020-10-12 15:00:54.576942
45618	GFF	2020-09-30	19.77	20.03	19.29	19.54	573888	{}	\N	2020-10-12 15:00:54.584154
45619	GFF	2020-10-01	19.62	19.97	19.29	19.74	439604	{}	\N	2020-10-12 15:00:54.590412
45620	GFF	2020-10-02	19.24	20.46	19.15	20.34	625193	{}	\N	2020-10-12 15:00:54.596344
45621	GFF	2020-10-05	20.41	20.98	20.41	20.87	377284	{}	\N	2020-10-12 15:00:54.601298
45622	GFF	2020-10-06	21.1	21.49	20.83	20.99	574088	{}	\N	2020-10-12 15:00:54.605994
45623	GFF	2020-10-07	21.25	21.74	21.01	21.63	1137622	{}	\N	2020-10-12 15:00:54.610604
45624	GFF	2020-10-08	22	22.78	21.94	22.69	807236	{}	\N	2020-10-12 15:00:54.615151
45625	GFF	2020-10-09	22.7	23.39	22.59	23.3	821562	{}	\N	2020-10-12 15:00:54.619667
45626	GFF	2020-10-12	23.37	24.44	23.32	24.08	247510	{}	\N	2020-10-12 15:00:54.62447
45627	GGB	2020-09-28	3.77	3.79	3.58	3.62	8585033	{}	\N	2020-10-12 15:00:55.984729
45628	GGB	2020-09-29	3.64	3.68	3.52	3.52	13492358	{}	\N	2020-10-12 15:00:55.992314
45629	GGB	2020-09-30	3.62	3.77	3.62	3.7	10315606	{}	\N	2020-10-12 15:00:55.999308
45630	GGB	2020-10-01	3.66	3.685	3.57	3.62	8028820	{}	\N	2020-10-12 15:00:56.005485
45631	GGB	2020-10-02	3.57	3.68	3.57	3.62	7069954	{}	\N	2020-10-12 15:00:56.009979
45632	GGB	2020-10-05	3.7	3.92	3.68	3.91	11078099	{}	\N	2020-10-12 15:00:56.015341
45633	GGB	2020-10-06	3.95	3.97	3.8	3.84	11314937	{}	\N	2020-10-12 15:00:56.019188
45634	GGB	2020-10-07	4	4.055	3.91	3.92	11128731	{}	\N	2020-10-12 15:00:56.022839
45635	GGB	2020-10-08	3.97	4.035	3.9	3.97	11410789	{}	\N	2020-10-12 15:00:56.026471
45636	GGB	2020-10-09	3.98	3.995	3.87	3.92	16113776	{}	\N	2020-10-12 15:00:56.031336
45637	GGB	2020-10-12	3.96	3.97	3.915	3.925	2273150	{}	\N	2020-10-12 15:00:56.035174
45638	GAM	2020-09-28	33.68	34.01	33.68	33.86	42693	{}	\N	2020-10-12 15:00:57.340757
45639	GAM	2020-09-29	33.9399	33.9399	33.6201	33.74	20148	{}	\N	2020-10-12 15:00:57.353936
45640	GAM	2020-09-30	33.92	34.16	33.88	33.95	40050	{}	\N	2020-10-12 15:00:57.361927
45641	GAM	2020-10-01	34.14	34.43	34.11	34.36	82845	{}	\N	2020-10-12 15:00:57.368794
45642	GAM	2020-10-02	33.99	34.3145	33.5227	34.236	38412	{}	\N	2020-10-12 15:00:57.37499
45643	GAM	2020-10-05	34.4899	34.69	34.1015	34.69	71601	{}	\N	2020-10-12 15:00:57.379985
45644	GAM	2020-10-06	34.69	34.69	34.03	34.03	132722	{}	\N	2020-10-12 15:00:57.383947
45645	GAM	2020-10-07	34.26	34.74	34.26	34.74	56004	{}	\N	2020-10-12 15:00:57.388302
45646	GAM	2020-10-08	34.88	35.02	34.71	34.86	86642	{}	\N	2020-10-12 15:00:57.392631
45647	GAM	2020-10-09	35.13	35.2117	34.77	35.09	48498	{}	\N	2020-10-12 15:00:57.397303
45648	GAM	2020-10-12	35.09	35.21	34.9	35.21	15866	{}	\N	2020-10-12 15:00:57.402481
45649	GBCI	2020-09-28	31.82	32.48	31.6601	32.42	342312	{}	\N	2020-10-12 15:00:58.384494
45650	GBCI	2020-09-29	32.28	32.28	31.155	32.14	288905	{}	\N	2020-10-12 15:00:58.394938
45651	GBCI	2020-09-30	32.32	32.61	31.745	32.05	278673	{}	\N	2020-10-12 15:00:58.402233
45652	GBCI	2020-10-01	31.85	32.225	31.29	32.18	290330	{}	\N	2020-10-12 15:00:58.40873
45653	GBCI	2020-10-02	31.79	33.78	31.4966	33.59	337136	{}	\N	2020-10-12 15:00:58.41455
45654	GBCI	2020-10-05	33.99	34.83	33.78	34.77	291931	{}	\N	2020-10-12 15:00:58.419707
45655	GBCI	2020-10-06	35	36.36	34.77	35.02	262184	{}	\N	2020-10-12 15:00:58.424538
45656	GBCI	2020-10-07	35.34	35.99	35.02	35.4	285917	{}	\N	2020-10-12 15:00:58.429469
45657	GBCI	2020-10-08	35.79	36.31	35.35	36	282166	{}	\N	2020-10-12 15:00:58.434597
45658	GBCI	2020-10-09	36.03	36.07	35.22	35.5	179414	{}	\N	2020-10-12 15:00:58.43911
45659	GBCI	2020-10-12	35.35	35.905	35.28	35.8903	33508	{}	\N	2020-10-12 15:00:58.443569
45660	GBX	2020-09-28	29.7	30.5	29.5	29.78	449727	{}	\N	2020-10-12 15:00:59.82322
45661	GBX	2020-09-29	29.84	30.05	29.131	29.57	555365	{}	\N	2020-10-12 15:00:59.832302
45662	GBX	2020-09-30	29.74	30.525	29.235	29.4	928784	{}	\N	2020-10-12 15:00:59.838787
45663	GBX	2020-10-01	29.4	30.29	29.11	30.24	978493	{}	\N	2020-10-12 15:00:59.845538
45664	GBX	2020-10-02	29.55	31.87	29.55	31.35	1402203	{}	\N	2020-10-12 15:00:59.850936
45665	GBX	2020-10-05	31.75	33.5899	31.615	33.14	2185815	{}	\N	2020-10-12 15:00:59.85543
45666	GBX	2020-10-06	33.53	33.75	32.5	32.6	1342399	{}	\N	2020-10-12 15:00:59.85965
45667	GBX	2020-10-07	32.84	33.955	32.81	33.45	1053530	{}	\N	2020-10-12 15:00:59.864344
45668	GBX	2020-10-08	33.76	34.25	33.0228	33.77	615166	{}	\N	2020-10-12 15:00:59.868394
45669	GBX	2020-10-09	34.21	34.77	33.57	33.73	1647543	{}	\N	2020-10-12 15:00:59.872566
45670	GBX	2020-10-12	33.87	34.26	33.18	33.35	139240	{}	\N	2020-10-12 15:00:59.876893
45671	GCP	2020-09-28	20.65	21.255	20.63	21	275428	{}	\N	2020-10-12 15:01:01.840395
45672	GCP	2020-09-29	20.93	21.27	20.825	20.86	368259	{}	\N	2020-10-12 15:01:01.849514
45673	GCP	2020-09-30	20.86	21.5	20.72	20.95	515636	{}	\N	2020-10-12 15:01:01.855439
45674	GCP	2020-10-01	20.98	21.31	20.65	20.96	454544	{}	\N	2020-10-12 15:01:01.860579
45675	GCP	2020-10-02	20.6	21.495	20.6	21.22	328087	{}	\N	2020-10-12 15:01:01.86516
45676	GCP	2020-10-05	21.37	22.12	21.21	21.95	327953	{}	\N	2020-10-12 15:01:01.869539
45677	GCP	2020-10-06	22.1	22.78	22.02	22.08	416583	{}	\N	2020-10-12 15:01:01.873798
45678	GCP	2020-10-07	22.33	22.79	22.33	22.51	482514	{}	\N	2020-10-12 15:01:01.877488
45679	GCP	2020-10-08	22.81	23.0722	22.45	22.57	251926	{}	\N	2020-10-12 15:01:01.881179
45680	GCP	2020-10-09	22.72	23.07	22.22	22.25	291882	{}	\N	2020-10-12 15:01:01.885131
45681	GCP	2020-10-12	22.34	22.35	22.04	22.04	25520	{}	\N	2020-10-12 15:01:01.889026
45682	GDOT	2020-09-28	49.99	50.98	49.61	49.99	499761	{}	\N	2020-10-12 15:01:03.233288
45683	GDOT	2020-09-29	49.9	50.7274	49.37	49.98	538858	{}	\N	2020-10-12 15:01:03.241973
45684	GDOT	2020-09-30	49.47	51.046	49.4	50.61	906499	{}	\N	2020-10-12 15:01:03.248521
45685	GDOT	2020-10-01	51.44	52.41	51.11	52.33	887072	{}	\N	2020-10-12 15:01:03.255524
45686	GDOT	2020-10-02	51.1	59.86	50.8819	59.3	3304100	{}	\N	2020-10-12 15:01:03.262649
45687	GDOT	2020-10-05	60.01	63.57	59.34	61.51	2239004	{}	\N	2020-10-12 15:01:03.268895
45688	GDOT	2020-10-06	61.83	63.66	60.01	60.71	1213090	{}	\N	2020-10-12 15:01:03.275696
45689	GDOT	2020-10-07	61.56	62.77	60.93	61.5	930715	{}	\N	2020-10-12 15:01:03.282127
45690	GDOT	2020-10-08	63.07	64.73	62.03	63.88	1235456	{}	\N	2020-10-12 15:01:03.288414
45691	GDOT	2020-10-09	64.13	64.33	62.6	63.34	930854	{}	\N	2020-10-12 15:01:03.295864
45692	GDOT	2020-10-12	63.51	63.96	62.08	63.58	203468	{}	\N	2020-10-12 15:01:03.307205
45693	GEC	2020-09-28	2.63	2.89	2.3801	2.39	8320	{}	\N	2020-10-12 15:01:05.470722
45694	GEC	2020-09-29	2.45	2.55	2.34	2.38	3713	{}	\N	2020-10-12 15:01:05.480753
45695	GEC	2020-09-30	2.46	2.46	2.36	2.36	2757	{}	\N	2020-10-12 15:01:05.48657
45696	GEC	2020-10-01	2.33	2.3516	2.31	2.32	2381	{}	\N	2020-10-12 15:01:05.492562
45697	GEC	2020-10-02	2.3	2.57	2.3	2.49	2307	{}	\N	2020-10-12 15:01:05.499097
45698	GEC	2020-10-05	2.53	2.67	2.49	2.49	3436	{}	\N	2020-10-12 15:01:05.504071
45699	GEC	2020-10-06	2.48	2.48	2.39	2.39	811	{}	\N	2020-10-12 15:01:05.510367
45700	GEC	2020-10-07	2.37	2.48	2.37	2.38	2326	{}	\N	2020-10-12 15:01:05.518691
45701	GEC	2020-10-09	2.41	2.41	2.34	2.35	1420	{}	\N	2020-10-12 15:01:05.527271
45702	GENC	2020-09-28	10.98	11.2	10.98	11.09	16963	{}	\N	2020-10-12 15:01:06.54086
45703	GENC	2020-09-29	11.145	11.23	11.045	11.14	8239	{}	\N	2020-10-12 15:01:06.551392
45704	GENC	2020-09-30	11.185	11.29	10.91	11.03	23502	{}	\N	2020-10-12 15:01:06.557301
45705	GENC	2020-10-01	11.0337	11.16	10.93	11.06	10131	{}	\N	2020-10-12 15:01:06.5631
45706	GENC	2020-10-02	11.14	11.65	11.1101	11.42	15257	{}	\N	2020-10-12 15:01:06.567275
45707	GENC	2020-10-05	10.55	11.89	10.55	11.56	10691	{}	\N	2020-10-12 15:01:06.571664
45708	GENC	2020-10-06	11.66	11.85	11.5	11.53	18668	{}	\N	2020-10-12 15:01:06.576711
45709	GENC	2020-10-07	11.74	11.98	11.595	11.91	15528	{}	\N	2020-10-12 15:01:06.583951
45710	GENC	2020-10-08	11.99	12.01	11.81	11.98	24210	{}	\N	2020-10-12 15:01:06.598333
45711	GENC	2020-10-09	12.14	12.24	11.93	12.1	15414	{}	\N	2020-10-12 15:01:06.608211
45712	GENC	2020-10-12	12.21	12.2299	11.5395	12.02	5577	{}	\N	2020-10-12 15:01:06.617337
45713	GEVO	2020-09-28	0.92	0.929	0.8801	0.89	8212797	{}	\N	2020-10-12 15:01:08.037858
45714	GEVO	2020-09-29	0.8773	1.17	0.855	1.11	28163760	{}	\N	2020-10-12 15:01:08.045047
45715	GEVO	2020-09-30	1.01	1.09	0.9685	0.9996	15610530	{}	\N	2020-10-12 15:01:08.050377
45716	GEVO	2020-10-01	1	1.01	0.9486	0.965	5504419	{}	\N	2020-10-12 15:01:08.055073
45717	GEVO	2020-10-02	0.8975	0.94375	0.8911	0.908	5719432	{}	\N	2020-10-12 15:01:08.059565
45718	GEVO	2020-10-05	0.9244	1.02	0.915	0.9873	12768031	{}	\N	2020-10-12 15:01:08.064295
45719	GEVO	2020-10-06	0.97	1.04	0.96	0.9841	8554854	{}	\N	2020-10-12 15:01:08.070873
45720	GEVO	2020-10-07	1.05	1.25	1.03	1.21	56607532	{}	\N	2020-10-12 15:01:08.075806
45721	GEVO	2020-10-08	1.32	1.42	1.02	1.02	101325165	{}	\N	2020-10-12 15:01:08.087472
45722	GEVO	2020-10-09	1.06	1.17	1.02	1.04	29392619	{}	\N	2020-10-12 15:01:08.094323
45723	GEVO	2020-10-12	1.11	1.14	1.08	1.11	5206534	{}	\N	2020-10-12 15:01:08.102402
45724	GGAL	2020-09-28	7.44	7.73	7.22	7.23	1274942	{}	\N	2020-10-12 15:01:09.529184
45725	GGAL	2020-09-29	7.01	7.35	6.9688	7.16	1092027	{}	\N	2020-10-12 15:01:09.537816
45726	GGAL	2020-09-30	7.3912	7.478	7.13	7.24	763810	{}	\N	2020-10-12 15:01:09.544449
45727	GGAL	2020-10-01	7.24	7.51	7.17	7.42	921642	{}	\N	2020-10-12 15:01:09.550494
45728	GGAL	2020-10-02	7.29	7.46	7.16	7.29	554143	{}	\N	2020-10-12 15:01:09.555981
45729	GGAL	2020-10-05	7.34	7.485	7.025	7.12	1336666	{}	\N	2020-10-12 15:01:09.560899
45730	GGAL	2020-10-06	7.17	7.39	6.88	6.91	1087327	{}	\N	2020-10-12 15:01:09.565828
45731	GGAL	2020-10-07	6.97	7.2	6.83	7.06	660819	{}	\N	2020-10-12 15:01:09.570313
45732	GGAL	2020-10-08	7.13	7.335	7.01	7.25	575465	{}	\N	2020-10-12 15:01:09.574664
45733	GGAL	2020-10-09	7.28	7.3	6.98	7.06	677542	{}	\N	2020-10-12 15:01:09.579198
45734	GGAL	2020-10-12	7.01	7.02	6.82	6.915	194759	{}	\N	2020-10-12 15:01:09.583602
45735	GHL	2020-09-28	11.08	11.51	11.08	11.18	96029	{}	\N	2020-10-12 15:01:10.635164
45736	GHL	2020-09-29	11.18	11.43	10.96	11.25	143638	{}	\N	2020-10-12 15:01:10.645433
45737	GHL	2020-09-30	11.27	11.51	11.13	11.35	169625	{}	\N	2020-10-12 15:01:10.653275
45738	GHL	2020-10-01	11.29	11.31	10.88	10.97	131335	{}	\N	2020-10-12 15:01:10.65933
45739	GHL	2020-10-02	10.8	11.545	10.78	11.42	170800	{}	\N	2020-10-12 15:01:10.664417
45740	GHL	2020-10-05	11.5	11.85	11.5	11.71	159334	{}	\N	2020-10-12 15:01:10.668497
45741	GHL	2020-10-06	11.89	12.59	11.75	12.13	151898	{}	\N	2020-10-12 15:01:10.672366
45742	GHL	2020-10-07	12.37	12.6	11.87	12.05	148115	{}	\N	2020-10-12 15:01:10.676229
45743	GHL	2020-10-08	11.98	12.82	11.86	12.72	102891	{}	\N	2020-10-12 15:01:10.68019
45744	GHL	2020-10-09	13.42	13.67	13.21	13.52	239623	{}	\N	2020-10-12 15:01:10.684282
45745	GHL	2020-10-12	13.65	14.16	13.65	14.015	28510	{}	\N	2020-10-12 15:01:10.688033
45746	GII	2020-09-28	44.33	44.46	44.25	44.28	16580	{}	\N	2020-10-12 15:01:12.137031
45747	GII	2020-09-29	44.37	44.56	44.1449	44.35	26524	{}	\N	2020-10-12 15:01:12.147428
45748	GII	2020-09-30	44.37	44.635	44.3	44.45	26432	{}	\N	2020-10-12 15:01:12.155155
45749	GII	2020-10-01	44.3994	44.48	44.235	44.48	14231	{}	\N	2020-10-12 15:01:12.160943
45750	GII	2020-10-02	43.99	44.81	43.99	44.72	23433	{}	\N	2020-10-12 15:01:12.166185
45751	GII	2020-10-05	44.96	45.3259	44.95	45.2884	229751	{}	\N	2020-10-12 15:01:12.170387
45752	GII	2020-10-06	45.62	45.7103	45	45.1	141891	{}	\N	2020-10-12 15:01:12.175499
45753	GII	2020-10-07	45.49	45.65	45.36	45.55	16004	{}	\N	2020-10-12 15:01:12.180452
45754	GII	2020-10-08	45.67	46.21	45.67	46.17	16137	{}	\N	2020-10-12 15:01:12.18712
45755	GII	2020-10-09	46.23	46.39	46.11	46.21	29049	{}	\N	2020-10-12 15:01:12.191221
45756	GII	2020-10-12	46.27	46.41	46.27	46.41	6954	{}	\N	2020-10-12 15:01:12.195481
45757	GJO	2020-09-28	23.05	23.05	23	23	2100	{}	\N	2020-10-12 15:01:14.010454
45758	GJO	2020-09-29	23.4	23.5	23	23	3404	{}	\N	2020-10-12 15:01:14.015741
45759	GJO	2020-09-30	23.2	23.2	23.2	23.2	410	{}	\N	2020-10-12 15:01:14.020556
45760	GJO	2020-10-01	23.4	23.4	23	23.076	6900	{}	\N	2020-10-12 15:01:14.026171
45761	GJO	2020-10-02	23	23.2	23	23.2	2204	{}	\N	2020-10-12 15:01:14.033228
45762	GJO	2020-10-05	22.98	22.98	22.98	22.98	1000	{}	\N	2020-10-12 15:01:14.039771
45763	GJO	2020-10-06	23.07	23.2	23	23.2	3080	{}	\N	2020-10-12 15:01:14.046822
45764	FXZ	2020-09-28	37.52	38.01	37.52	37.605	17575	{}	\N	2020-10-12 15:01:15.105056
45765	FXZ	2020-09-29	37.76	37.76	37.12	37.19	9028	{}	\N	2020-10-12 15:01:15.115254
45766	FXZ	2020-09-30	37.33	37.87	37.25	37.53	14948	{}	\N	2020-10-12 15:01:15.121262
45767	FXZ	2020-10-01	37.71	37.87	37.12	37.24	233199	{}	\N	2020-10-12 15:01:15.126829
45768	FXZ	2020-10-02	37.13	38.19	37.12	37.98	43956	{}	\N	2020-10-12 15:01:15.131652
45769	FXZ	2020-10-05	38.8	39.119	38.8	39.05	17823	{}	\N	2020-10-12 15:01:15.136384
45770	FXZ	2020-10-06	39.38	39.82	38.82	38.83	31616	{}	\N	2020-10-12 15:01:15.140496
45771	FXZ	2020-10-07	39.34	40.02	39.34	39.83	22606	{}	\N	2020-10-12 15:01:15.144688
45772	FXZ	2020-10-08	40.17	40.288	39.9106	40.288	33392	{}	\N	2020-10-12 15:01:15.148609
45773	FXZ	2020-10-09	40.65	40.65	40.22	40.41	23593	{}	\N	2020-10-12 15:01:15.1528
45774	FXZ	2020-10-12	40.67	40.67	40.325	40.362	3008	{}	\N	2020-10-12 15:01:15.15694
45775	GAINM	2020-09-28	24.65	24.75	24.5553	24.74	2619	{}	\N	2020-10-12 15:01:16.125262
45776	GAINM	2020-09-29	24.761	24.807	24.695	24.7299	1923	{}	\N	2020-10-12 15:01:16.134748
45777	GAINM	2020-09-30	24.71	24.81	24.63	24.63	2993	{}	\N	2020-10-12 15:01:16.140685
45778	GAINM	2020-10-01	24.7203	24.78	24.7	24.7157	1945	{}	\N	2020-10-12 15:01:16.145788
45779	GAINM	2020-10-02	24.75	24.81	24.6	24.695	1726	{}	\N	2020-10-12 15:01:16.150964
45780	GAINM	2020-10-05	24.7885	24.8003	24.6925	24.795	30670	{}	\N	2020-10-12 15:01:16.155432
45781	GAINM	2020-10-06	24.82	24.84	24.705	24.82	4759	{}	\N	2020-10-12 15:01:16.159673
45782	GAINM	2020-10-07	24.88	24.99	24.85	24.99	3496	{}	\N	2020-10-12 15:01:16.163657
45783	GAINM	2020-10-08	24.82	24.96	24.8	24.9021	4356	{}	\N	2020-10-12 15:01:16.167382
45784	GAINM	2020-10-09	24.89	24.94	24.78	24.84	4818	{}	\N	2020-10-12 15:01:16.172426
45785	GAINM	2020-10-12	24.91	25	24.91	24.98	700	{}	\N	2020-10-12 15:01:16.177219
45786	GATX	2020-09-28	63.6	64.95	63.59	64.14	264110	{}	\N	2020-10-12 15:01:17.547996
45787	GATX	2020-09-29	64.11	64.13	62.64	63.64	274214	{}	\N	2020-10-12 15:01:17.559517
45788	GATX	2020-09-30	64	65.94	63.33	63.75	700480	{}	\N	2020-10-12 15:01:17.570139
45789	GATX	2020-10-01	63.68	64.28	62.15	64.05	391225	{}	\N	2020-10-12 15:01:17.576512
45790	GATX	2020-10-02	63.23	65.66	62.66	65.44	267681	{}	\N	2020-10-12 15:01:17.581695
45791	GATX	2020-10-05	66.33	67.33	66.03	66.9	221554	{}	\N	2020-10-12 15:01:17.586005
45792	GATX	2020-10-06	67.84	68.58	66.4	66.58	406230	{}	\N	2020-10-12 15:01:17.589882
45793	GATX	2020-10-07	67.34	68.51	67.14	68.28	321085	{}	\N	2020-10-12 15:01:17.593654
45794	GATX	2020-10-08	69.25	69.25	67.715	68.7	229556	{}	\N	2020-10-12 15:01:17.597981
45795	GATX	2020-10-09	69.21	69.265	67.56	67.82	303714	{}	\N	2020-10-12 15:01:17.601817
45796	GATX	2020-10-12	67.98	69.32	67.5	68.64	48765	{}	\N	2020-10-12 15:01:17.605741
45797	GFI	2020-09-28	12.09	12.11	11.715	11.89	4208791	{}	\N	2020-10-12 15:01:21.133846
45798	GFI	2020-09-29	12.12	12.595	11.97	12.49	11501218	{}	\N	2020-10-12 15:01:21.143323
45799	GFI	2020-09-30	12.37	12.41	12.015	12.29	9831384	{}	\N	2020-10-12 15:01:21.150166
45800	GFI	2020-10-01	12.41	12.54	12.2113	12.4	7836159	{}	\N	2020-10-12 15:01:21.15657
45801	GFI	2020-10-02	12.23	12.435	12.075	12.12	5294394	{}	\N	2020-10-12 15:01:21.165426
45802	GFI	2020-10-05	12.31	12.63	12.215	12.31	9422495	{}	\N	2020-10-12 15:01:21.170065
45803	GFI	2020-10-06	12.46	12.55	11.87	11.87	12541490	{}	\N	2020-10-12 15:01:21.174094
45804	GFI	2020-10-07	11.8	12.04	11.63	11.86	8875467	{}	\N	2020-10-12 15:01:21.178144
45805	GFI	2020-10-08	12.09	12.31	11.99	12.28	6862203	{}	\N	2020-10-12 15:01:21.182296
45806	GFI	2020-10-09	12.71	13.21	12.7	13.17	11079663	{}	\N	2020-10-12 15:01:21.186138
45807	GFI	2020-10-12	13	13.07	12.76	12.91	2502026	{}	\N	2020-10-12 15:01:21.190155
45808	GGZ	2020-09-28	10.16	10.16	10	10	52508	{}	\N	2020-10-12 15:01:23.143953
45809	GGZ	2020-09-29	10	10.02	9.93	9.94	27668	{}	\N	2020-10-12 15:01:23.15392
45810	GGZ	2020-09-30	9.99	10.09	9.96	10.09	19190	{}	\N	2020-10-12 15:01:23.160363
45811	GGZ	2020-10-01	10.06	10.119	10.06	10.07	17934	{}	\N	2020-10-12 15:01:23.166572
45812	GGZ	2020-10-02	10.01	10.126	9.65	10.06	23888	{}	\N	2020-10-12 15:01:23.171993
45813	GGZ	2020-10-05	10.1	10.22	10.0904	10.19	62966	{}	\N	2020-10-12 15:01:23.177011
45814	GGZ	2020-10-06	10.26	10.459	10.0796	10.23	58942	{}	\N	2020-10-12 15:01:23.181181
45815	GGZ	2020-10-07	10.33	10.47	10.33	10.41	43420	{}	\N	2020-10-12 15:01:23.185553
45816	GGZ	2020-10-08	10.437	10.5	10.437	10.47	13744	{}	\N	2020-10-12 15:01:23.189811
45817	GGZ	2020-10-09	10.51	10.58	10.4802	10.49	24990	{}	\N	2020-10-12 15:01:23.194395
45818	GGZ	2020-10-12	10.57	10.67	10.57	10.6616	1770	{}	\N	2020-10-12 15:01:23.198615
45819	GIB	2020-09-28	67.59	68.54	67.42	67.56	136520	{}	\N	2020-10-12 15:01:24.129901
45820	GIB	2020-09-29	67.34	68.16	66.97	67	345543	{}	\N	2020-10-12 15:01:24.138839
45821	GIB	2020-09-30	66.9	68.16	66.9	67.77	340471	{}	\N	2020-10-12 15:01:24.144871
45822	GIB	2020-10-01	67.91	69.45	67.79	69.34	358675	{}	\N	2020-10-12 15:01:24.151023
45823	GIB	2020-10-02	68.41	69.24	68.4	68.78	235972	{}	\N	2020-10-12 15:01:24.155932
45824	GIB	2020-10-05	68.8	69.81	68.72	69.77	187600	{}	\N	2020-10-12 15:01:24.160505
45825	GIB	2020-10-06	69.7	69.91	67.54	67.57	270317	{}	\N	2020-10-12 15:01:24.164853
45826	GIB	2020-10-07	68.23	68.53	67.97	68.18	269464	{}	\N	2020-10-12 15:01:24.169322
45827	GIB	2020-10-08	68.62	69.22	68.365	69.18	151864	{}	\N	2020-10-12 15:01:24.173362
45828	GIB	2020-10-09	69.69	70.26	69.14	69.23	297678	{}	\N	2020-10-12 15:01:24.177831
45829	GIB	2020-10-12	69.24	69.39	69.24	69.39	21606	{}	\N	2020-10-12 15:01:24.18195
45830	GILD	2020-09-28	63.04	63.41	62.52	62.85	7054119	{}	\N	2020-10-12 15:01:25.105681
45831	GILD	2020-09-29	62.96	63.025	61.95	62.1	8387132	{}	\N	2020-10-12 15:01:25.115928
45832	GILD	2020-09-30	62.35	63.425	62.11	63.19	9239793	{}	\N	2020-10-12 15:01:25.123282
45833	GILD	2020-10-01	63.33	63.71	62.71	63.32	7927488	{}	\N	2020-10-12 15:01:25.128699
45834	GILD	2020-10-02	63.2969	63.84	62.15	62.17	7194415	{}	\N	2020-10-12 15:01:25.133273
45835	GILD	2020-10-05	63.71	64.09	63.1	63.6	10114294	{}	\N	2020-10-12 15:01:25.138171
45836	GILD	2020-10-06	63.66	63.76	62.35	62.54	7131115	{}	\N	2020-10-12 15:01:25.142343
45837	GILD	2020-10-07	62.73	63.17	62.29	62.84	6143502	{}	\N	2020-10-12 15:01:25.1464
45838	GILD	2020-10-08	63.33	64.05	63.02	63.32	6285221	{}	\N	2020-10-12 15:01:25.150317
45839	GILD	2020-10-09	64.61	64.67	63.72	63.84	9037322	{}	\N	2020-10-12 15:01:25.154474
45840	GILD	2020-10-12	64.07	64.39	63.55	64.305	2089926	{}	\N	2020-10-12 15:01:25.158489
45841	GJP	2020-09-28	24.75	24.75	24.7	24.75	2200	{}	\N	2020-10-12 15:01:26.601665
45842	GJP	2020-09-30	24.74	24.74	24.045	24.74	3200	{}	\N	2020-10-12 15:01:26.609505
45843	GJP	2020-10-01	24.3	24.3	24.27	24.27	1020	{}	\N	2020-10-12 15:01:26.614618
45844	GJP	2020-10-02	23.43	24.05	23.39	24.05	2096	{}	\N	2020-10-12 15:01:26.619901
45845	GJP	2020-10-05	24.56	24.56	24.4	24.55	4414	{}	\N	2020-10-12 15:01:26.624645
45846	GJP	2020-10-06	24.54	24.55	24.54	24.55	1200	{}	\N	2020-10-12 15:01:26.629035
45847	GJP	2020-10-07	24.5	24.5	24.3	24.5	4588	{}	\N	2020-10-12 15:01:26.633043
45848	GJP	2020-10-08	24.55	24.55	24.55	24.55	2000	{}	\N	2020-10-12 15:01:26.637455
45849	GLP	2020-09-28	12.9	13.12	12.61	12.68	110032	{}	\N	2020-10-12 15:01:28.569974
45850	GLP	2020-09-29	12.93	13.46	12.81	13.45	329885	{}	\N	2020-10-12 15:01:28.574328
45851	GLP	2020-09-30	13.45	13.6	13.07	13.11	267313	{}	\N	2020-10-12 15:01:28.578728
45852	GLP	2020-10-01	13.15	13.38	13.04	13.16	123119	{}	\N	2020-10-12 15:01:28.58309
45853	GLP	2020-10-02	13.09	13.49	13.02	13.4	146324	{}	\N	2020-10-12 15:01:28.587454
45854	GLP	2020-10-05	13.45	13.78	13.45	13.72	130202	{}	\N	2020-10-12 15:01:28.591478
45855	GLP	2020-10-06	13.72	13.93	13.63	13.74	130880	{}	\N	2020-10-12 15:01:28.5962
45856	GLP	2020-10-07	13.88	13.9299	13.46	13.55	179430	{}	\N	2020-10-12 15:01:28.600834
45857	GLP	2020-10-08	13.57	13.85	13.57	13.82	102656	{}	\N	2020-10-12 15:01:28.607057
45858	GLP	2020-10-09	13.96	14.5899	13.85	14.56	497424	{}	\N	2020-10-12 15:01:28.611997
45859	GLP	2020-10-12	15	15	14.66	14.7767	188075	{}	\N	2020-10-12 15:01:28.616457
45860	GEOS	2020-09-28	5.82	6.415	5.82	6.25	49151	{}	\N	2020-10-12 15:01:30.081303
45861	GEOS	2020-09-29	6.02	6.22	6.0166	6.16	81470	{}	\N	2020-10-12 15:01:30.090538
45862	GEOS	2020-09-30	6.16	6.25	6.07	6.18	48285	{}	\N	2020-10-12 15:01:30.096792
45863	GEOS	2020-10-01	6.16	6.21	5.99	6.08	106355	{}	\N	2020-10-12 15:01:30.102309
45864	GEOS	2020-10-02	5.95	6.2	5.95	6.18	25783	{}	\N	2020-10-12 15:01:30.10722
45865	GEOS	2020-10-05	6.32	6.48	6.26	6.28	28808	{}	\N	2020-10-12 15:01:30.112061
45866	GEOS	2020-10-06	6.28	6.465	6.17	6.17	46409	{}	\N	2020-10-12 15:01:30.116173
45867	GEOS	2020-10-07	6.29	6.29	5.92	5.97	43353	{}	\N	2020-10-12 15:01:30.120141
45868	GEOS	2020-10-08	6	6.18	5.99	6.02	29365	{}	\N	2020-10-12 15:01:30.124093
45869	GEOS	2020-10-09	6	6.11	5.92	6.11	41348	{}	\N	2020-10-12 15:01:30.128244
45870	GEOS	2020-10-12	6.12	6.195	6.12	6.195	484	{}	\N	2020-10-12 15:01:30.13216
45871	GFNSL	2020-09-28	25.41	25.4201	25.41	25.4201	1294	{}	\N	2020-10-12 15:01:31.093898
45872	GFNSL	2020-09-29	25.38	25.39	25.35	25.35	900	{}	\N	2020-10-12 15:01:31.103147
45873	GFNSL	2020-09-30	25.44	25.44	25.37	25.37	1901	{}	\N	2020-10-12 15:01:31.109643
45874	GFNSL	2020-10-02	25.36	25.41	25.36	25.401	864	{}	\N	2020-10-12 15:01:31.115104
45875	GFNSL	2020-10-05	25.41	25.45	25.3	25.45	1790	{}	\N	2020-10-12 15:01:31.121073
45876	GFNSL	2020-10-06	25.4	25.4	25.38	25.4	2105	{}	\N	2020-10-12 15:01:31.126616
45877	GFNSL	2020-10-07	25.45	25.57	25.4499	25.4499	1802	{}	\N	2020-10-12 15:01:31.132144
45878	GFNSL	2020-10-08	25.41	25.5599	25.4	25.4808	3526	{}	\N	2020-10-12 15:01:31.137055
45879	GFNSL	2020-10-09	25.44	25.4414	25.41	25.41	1850	{}	\N	2020-10-12 15:01:31.142044
45880	GFNSL	2020-10-12	25.44	25.44	25.4	25.41	5394	{}	\N	2020-10-12 15:01:31.147054
45881	GGT	2020-09-28	6.63	6.7	6.63	6.63	78599	{}	\N	2020-10-12 15:01:32.283015
45882	GGT	2020-09-29	6.64	6.68	6.465	6.5	252690	{}	\N	2020-10-12 15:01:32.292545
45883	GGT	2020-09-30	6.58	6.58	6.4	6.46	325654	{}	\N	2020-10-12 15:01:32.298995
45884	GGT	2020-10-01	6.45	6.515	6.43	6.5	205526	{}	\N	2020-10-12 15:01:32.30497
45885	GGT	2020-10-02	6.44	6.47	6.36	6.42	162034	{}	\N	2020-10-12 15:01:32.3108
45886	GGT	2020-10-05	6.44	6.62	6.44	6.62	77730	{}	\N	2020-10-12 15:01:32.315783
45887	GGT	2020-10-06	6.6	6.68	6.56	6.59	237240	{}	\N	2020-10-12 15:01:32.320328
45888	GGT	2020-10-07	6.61	6.674	6.61	6.62	83292	{}	\N	2020-10-12 15:01:32.324877
45889	GGT	2020-10-08	6.68	6.75	6.6464	6.75	146568	{}	\N	2020-10-12 15:01:32.329237
45890	GGT	2020-10-09	6.8	6.8	6.72	6.72	166038	{}	\N	2020-10-12 15:01:32.333786
45891	GGT	2020-10-12	6.76	6.81	6.76	6.8024	37426	{}	\N	2020-10-12 15:01:32.338275
45892	GJR	2020-09-30	22.5	22.9998	22.5	22.9998	1200	{}	\N	2020-10-12 15:01:33.971265
45893	GJR	2020-10-01	22.3	22.3	22.3	22.3	1792	{}	\N	2020-10-12 15:01:33.979557
45894	GJR	2020-10-02	22.5	22.97	22.5	22.97	2600	{}	\N	2020-10-12 15:01:33.9853
45895	GJR	2020-10-07	22.5162	22.5162	22.37	22.37	2790	{}	\N	2020-10-12 15:01:33.990424
45896	GJR	2020-10-09	22.35	22.35	22.35	22.35	400	{}	\N	2020-10-12 15:01:33.995262
45897	GLOB	2020-09-28	178.41	181.55	177	181.02	145422	{}	\N	2020-10-12 15:01:34.966756
45898	GLOB	2020-09-29	181.82	182.9799	177.4	178.77	249288	{}	\N	2020-10-12 15:01:34.973858
45899	GLOB	2020-09-30	180.48	184.55	178.45	179.22	411411	{}	\N	2020-10-12 15:01:34.980988
45900	GLOB	2020-10-01	182.27	184.69	180.44	184.44	361502	{}	\N	2020-10-12 15:01:34.992783
45901	GLOB	2020-10-02	179.56	184.645	179	182.68	333562	{}	\N	2020-10-12 15:01:35.000822
45902	GLOB	2020-10-05	185	193.585	184.135	191.26	850242	{}	\N	2020-10-12 15:01:35.006892
45903	GLOB	2020-10-06	193	193.465	188.0901	189.67	497607	{}	\N	2020-10-12 15:01:35.012688
45904	GLOB	2020-10-07	191.75	195.76	187.26	194.86	305780	{}	\N	2020-10-12 15:01:35.018731
45905	GLOB	2020-10-08	195.82	197.5	190.165	191.21	304073	{}	\N	2020-10-12 15:01:35.024202
45906	GLOB	2020-10-09	193	195.91	193	195.82	201212	{}	\N	2020-10-12 15:01:35.029692
45907	GLOB	2020-10-12	199.46	199.46	195.82	197.05	50022	{}	\N	2020-10-12 15:01:35.035218
45908	GLUU	2020-09-28	7.48	7.54	7.29	7.46	3167053	{}	\N	2020-10-12 15:01:36.431712
45909	GLUU	2020-09-29	7.43	7.755	7.4	7.62	3271481	{}	\N	2020-10-12 15:01:36.441144
45910	GLUU	2020-09-30	7.6	7.9	7.57	7.675	2923518	{}	\N	2020-10-12 15:01:36.447648
45911	GLUU	2020-10-01	7.72	7.86	7.685	7.79	2601421	{}	\N	2020-10-12 15:01:36.453768
45912	GLUU	2020-10-02	7.51	7.895	7.465	7.48	3426830	{}	\N	2020-10-12 15:01:36.45962
45913	GLUU	2020-10-05	7.6	7.71	7.4	7.64	5323618	{}	\N	2020-10-12 15:01:36.46407
45914	GLUU	2020-10-06	7.68	7.97	7.675	7.8	3984385	{}	\N	2020-10-12 15:01:36.468564
45915	GLUU	2020-10-07	7.81	7.85	7.6	7.75	3454418	{}	\N	2020-10-12 15:01:36.473116
45916	GLUU	2020-10-08	7.81	7.91	7.74	7.77	5353645	{}	\N	2020-10-12 15:01:36.477839
45917	GLUU	2020-10-09	7.76	8.09	7.76	7.85	3934215	{}	\N	2020-10-12 15:01:36.482537
45918	GLUU	2020-10-12	7.9	8.01	7.81	7.93	596767	{}	\N	2020-10-12 15:01:36.487064
45919	GLW	2020-09-28	31.76	32.187	31.63	32.03	3008964	{}	\N	2020-10-12 15:01:38.65759
45920	GLW	2020-09-29	32.15	32.325	31.93	31.97	5949194	{}	\N	2020-10-12 15:01:38.67101
45921	GLW	2020-09-30	32.15	32.79	32.06	32.41	9847639	{}	\N	2020-10-12 15:01:38.677743
45922	GLW	2020-10-01	32.75	32.77	32.055	32.53	8742164	{}	\N	2020-10-12 15:01:38.683226
45923	GLW	2020-10-02	32.03	33.11	32	32.81	7109245	{}	\N	2020-10-12 15:01:38.688243
45924	GLW	2020-10-05	33.25	33.725	33	33.53	7305119	{}	\N	2020-10-12 15:01:38.692919
45925	GLW	2020-10-06	33.64	34.335	33.57	33.62	10208718	{}	\N	2020-10-12 15:01:38.697744
45926	GLW	2020-10-07	33.93	35.095	33.881	35.06	9011378	{}	\N	2020-10-12 15:01:38.702098
45927	GLW	2020-10-08	35.2	35.83	35.1	35.23	8481148	{}	\N	2020-10-12 15:01:38.706482
45928	GLW	2020-10-09	35.4	35.55	34.71	34.78	7231419	{}	\N	2020-10-12 15:01:38.710775
45929	GLW	2020-10-12	34.98	35.04	34.72	34.78	1500319	{}	\N	2020-10-12 15:01:38.715308
45930	GMRE	2020-09-28	13.53	13.72	13.5	13.59	205267	{}	\N	2020-10-12 15:01:39.769136
45931	GMRE	2020-09-29	13.63	13.74	13.27	13.59	715999	{}	\N	2020-10-12 15:01:39.780263
45932	GMRE	2020-09-30	13.76	13.99	13.46	13.5	1419879	{}	\N	2020-10-12 15:01:39.787033
45933	GMRE	2020-10-01	13.4	13.75	13.36	13.73	366505	{}	\N	2020-10-12 15:01:39.793198
45934	GMRE	2020-10-02	13.49	14.3	13.42	14.29	635289	{}	\N	2020-10-12 15:01:39.798365
45935	GMRE	2020-10-05	14.22	14.26	13.9	14.21	404120	{}	\N	2020-10-12 15:01:39.803253
45936	GMRE	2020-10-06	14.3	14.43	14.005	14.14	456597	{}	\N	2020-10-12 15:01:39.807299
45937	GMRE	2020-10-07	14.3	14.3	13.97	14.1	323812	{}	\N	2020-10-12 15:01:39.811699
45938	GMRE	2020-10-08	14.12	14.4398	14.12	14.38	309528	{}	\N	2020-10-12 15:01:39.815798
45939	GMRE	2020-10-09	14.5	14.6	14.25	14.36	376474	{}	\N	2020-10-12 15:01:39.819722
45940	GMRE	2020-10-12	14.38	14.46	14.26	14.32	38750	{}	\N	2020-10-12 15:01:39.823614
45941	GMTA	2020-09-28	26.84	27	26.8	26.9	8955	{}	\N	2020-10-12 15:01:41.318242
45942	GMTA	2020-09-29	26.8752	27.18	26.79	27.18	7798	{}	\N	2020-10-12 15:01:41.323846
45943	GMTA	2020-09-30	26.9514	27.091	26.46	26.46	68738	{}	\N	2020-10-12 15:01:41.328169
45944	GMTA	2020-10-01	26.53	26.9018	26.53	26.68	29370	{}	\N	2020-10-12 15:01:41.332881
45945	GMTA	2020-10-02	26.47	26.7397	26.47	26.63	14502	{}	\N	2020-10-12 15:01:41.337733
45946	GMTA	2020-10-05	26.88	26.9175	26.66	26.86	10572	{}	\N	2020-10-12 15:01:41.342582
45947	GMTA	2020-10-06	26.6	26.91	26.34	26.66	22624	{}	\N	2020-10-12 15:01:41.347644
45948	GMTA	2020-10-07	27.062	27.062	26.715	26.96	11298	{}	\N	2020-10-12 15:01:41.35285
45949	GMTA	2020-10-08	26.89	26.98	26.81	26.98	11548	{}	\N	2020-10-12 15:01:41.357749
45950	GMTA	2020-10-09	26.965	27.03	26.965	27.03	5380	{}	\N	2020-10-12 15:01:41.362338
45951	GNK	2020-09-28	6.79	6.84	6.7	6.73	89230	{}	\N	2020-10-12 15:01:43.108677
45952	GNK	2020-09-29	6.72	6.96	6.67	6.77	170164	{}	\N	2020-10-12 15:01:43.114313
45953	GNK	2020-09-30	6.86	7.17	6.82	6.9	276188	{}	\N	2020-10-12 15:01:43.119254
45954	GNK	2020-10-01	6.93	7.31	6.93	7.28	229817	{}	\N	2020-10-12 15:01:43.123644
45955	GNK	2020-10-02	7.13	7.8	7.077	7.77	288423	{}	\N	2020-10-12 15:01:43.128192
45956	GNK	2020-10-05	7.89	8.15	7.7727	8.04	437516	{}	\N	2020-10-12 15:01:43.132698
45957	GNK	2020-10-06	8.07	8.14	7.79	7.79	320517	{}	\N	2020-10-12 15:01:43.136782
45958	GNK	2020-10-07	7.73	7.89	7.43	7.44	438351	{}	\N	2020-10-12 15:01:43.141221
45959	GNK	2020-10-08	7.5	7.81	7.5	7.69	238534	{}	\N	2020-10-12 15:01:43.145629
45960	GNK	2020-10-09	7.77	7.83	7.4419	7.49	218766	{}	\N	2020-10-12 15:01:43.149722
45961	GNK	2020-10-12	7.44	7.45	7.29	7.29	35016	{}	\N	2020-10-12 15:01:43.153731
45962	GNT	2020-09-28	4.99	4.99	4.92	4.99	71981	{}	\N	2020-10-12 15:01:44.428034
45963	GNT	2020-09-29	4.98	5	4.9	4.94	197014	{}	\N	2020-10-12 15:01:44.437897
45964	GNT	2020-09-30	4.95	5	4.92	4.94	147132	{}	\N	2020-10-12 15:01:44.445198
45965	GNT	2020-10-01	4.95	5.03	4.95	5.02	250702	{}	\N	2020-10-12 15:01:44.451324
45966	GNT	2020-10-02	4.97	5	4.94	4.98	111012	{}	\N	2020-10-12 15:01:44.456239
45967	GNT	2020-10-05	4.98	5.03	4.9734	5.02	144340	{}	\N	2020-10-12 15:01:44.461636
45968	GNT	2020-10-06	5.01	5.02	4.91	4.91	225315	{}	\N	2020-10-12 15:01:44.466091
45969	GNT	2020-10-07	4.88	4.97	4.88	4.88	180838	{}	\N	2020-10-12 15:01:44.470242
45970	GNT	2020-10-08	4.92	5	4.92	5	119328	{}	\N	2020-10-12 15:01:44.474521
45971	GNT	2020-10-09	5	5.04	5	5.02	115158	{}	\N	2020-10-12 15:01:44.478967
45972	GNT	2020-10-12	5.01	5.0363	5.01	5.0217	43236	{}	\N	2020-10-12 15:01:44.483338
45973	GECCL	2020-09-28	23.0611	23.32	23.0611	23.32	665	{}	\N	2020-10-12 15:01:45.903674
45974	GECCL	2020-09-29	23.35	23.49	23.35	23.49	3612	{}	\N	2020-10-12 15:01:45.915169
45975	GECCL	2020-10-01	23.62	23.65	23.574	23.6008	5023	{}	\N	2020-10-12 15:01:45.926088
45976	GECCL	2020-10-02	23.3501	24.11	23.3501	24	6236	{}	\N	2020-10-12 15:01:45.931846
45977	GECCL	2020-10-05	23.8095	23.875	23.75	23.86	898	{}	\N	2020-10-12 15:01:45.939025
45978	GECCL	2020-10-06	23.745	23.745	23.61	23.74	3287	{}	\N	2020-10-12 15:01:45.944737
45979	GECCL	2020-10-07	23.64	23.74	23.64	23.73	2847	{}	\N	2020-10-12 15:01:45.952148
45980	GECCL	2020-10-08	23.5	23.98	23.5	23.98	3354	{}	\N	2020-10-12 15:01:45.961504
45981	GECCL	2020-10-09	23.905	23.99	23.79	23.79	2743	{}	\N	2020-10-12 15:01:45.971797
45982	GECCL	2020-10-12	23.8866	23.8866	23.8866	23.8866	120	{}	\N	2020-10-12 15:01:45.979511
45983	GEN	2020-09-28	0.5834	0.605799	0.54	0.5481	1138014	{}	\N	2020-10-12 15:01:46.884614
45984	GEN	2020-09-29	0.55	0.5599	0.54	0.55	1679077	{}	\N	2020-10-12 15:01:46.894327
45985	GEN	2020-09-30	0.56	0.56	0.54	0.5442	1259642	{}	\N	2020-10-12 15:01:46.900336
45986	GEN	2020-10-01	0.5745	0.5745	0.53	0.54	1571006	{}	\N	2020-10-12 15:01:46.905759
45987	GEN	2020-10-02	0.538	0.55	0.522	0.5394	823282	{}	\N	2020-10-12 15:01:46.911063
45988	GEN	2020-10-05	0.542	0.5472	0.5038	0.521	1714324	{}	\N	2020-10-12 15:01:46.915766
45989	GEN	2020-10-06	0.53	0.5399	0.5151	0.5243	1096366	{}	\N	2020-10-12 15:01:46.919713
45990	GEN	2020-10-07	0.53	0.538	0.52	0.5274	1354795	{}	\N	2020-10-12 15:01:46.923695
45991	GEN	2020-10-08	0.53	0.553	0.5298	0.54	1491962	{}	\N	2020-10-12 15:01:46.927849
45992	GEN	2020-10-09	0.511	0.55	0.511	0.5379	885044	{}	\N	2020-10-12 15:01:46.931525
45993	GEN	2020-10-12	0.53	0.555	0.53	0.54	399382	{}	\N	2020-10-12 15:01:46.935382
45994	GES	2020-09-28	11.82	12.18	11.78	11.93	978071	{}	\N	2020-10-12 15:01:47.986296
45995	GES	2020-09-29	11.99	12	11.16	11.37	3276698	{}	\N	2020-10-12 15:01:47.996697
45996	GES	2020-09-30	11.41	12.115	11.41	11.62	2555928	{}	\N	2020-10-12 15:01:48.003007
45997	GES	2020-10-01	11.67	12.43	11.65	12.14	2599266	{}	\N	2020-10-12 15:01:48.009026
45998	GES	2020-10-02	11.54	12.84	11.54	12.78	1716438	{}	\N	2020-10-12 15:01:48.014036
45999	GES	2020-10-05	12.97	13.85	12.92	13.79	2077435	{}	\N	2020-10-12 15:01:48.018839
46000	GES	2020-10-06	13.91	13.97	13.22	13.22	2554425	{}	\N	2020-10-12 15:01:48.022998
46001	GES	2020-10-07	13.58	14	13.39	13.83	2209969	{}	\N	2020-10-12 15:01:48.027009
46002	GES	2020-10-08	13.98	14.19	13.48	13.77	1338977	{}	\N	2020-10-12 15:01:48.030785
46003	GES	2020-10-09	13.9	14.22	13.85	14.07	1442203	{}	\N	2020-10-12 15:01:48.034794
46004	GES	2020-10-12	14.05	14.28	13.76	13.76	789837	{}	\N	2020-10-12 15:01:48.039154
46005	GGG	2020-09-28	61.15	61.825	61.025	61.16	408329	{}	\N	2020-10-12 15:01:49.376408
46006	GGG	2020-09-29	61.37	61.99	61.07	61.35	803253	{}	\N	2020-10-12 15:01:49.389239
46007	GGG	2020-09-30	61.57	62.01	60.73	61.35	1302956	{}	\N	2020-10-12 15:01:49.39672
46008	GGG	2020-10-01	61.74	62.17	61.17	61.55	779018	{}	\N	2020-10-12 15:01:49.402824
46009	GGG	2020-10-02	60.52	61.68	60.52	60.93	1084440	{}	\N	2020-10-12 15:01:49.407912
46010	GGG	2020-10-05	61.68	61.99	61.09	61.66	1530385	{}	\N	2020-10-12 15:01:49.412769
46011	GGG	2020-10-06	61.67	62.8	61.26	61.85	1209650	{}	\N	2020-10-12 15:01:49.416951
46012	GGG	2020-10-07	62.51	63.59	62.27	63.33	1167541	{}	\N	2020-10-12 15:01:49.421067
46013	GGG	2020-10-08	63.65	63.99	63.33	63.98	1132612	{}	\N	2020-10-12 15:01:49.425309
46014	GGG	2020-10-09	64.22	64.92	64.16	64.55	1122381	{}	\N	2020-10-12 15:01:49.42935
46015	GGG	2020-10-12	64.58	65	64.44	64.59	147388	{}	\N	2020-10-12 15:01:49.433449
46016	GHM	2020-09-28	12.95	13.32	12.85	13.18	25207	{}	\N	2020-10-12 15:01:50.398055
46017	GHM	2020-09-29	13.1	13.13	12.87	13.05	16233	{}	\N	2020-10-12 15:01:50.408273
46018	GHM	2020-09-30	13.19	13.3	12.43	12.77	32349	{}	\N	2020-10-12 15:01:50.415045
46019	GHM	2020-10-01	12.72	12.95	12.6	12.63	103581	{}	\N	2020-10-12 15:01:50.421318
46020	GHM	2020-10-02	12.95	13.45	12.64	13.3	23324	{}	\N	2020-10-12 15:01:50.426832
46021	GHM	2020-10-05	13.43	13.7139	13.25	13.41	52850	{}	\N	2020-10-12 15:01:50.431949
46022	GHM	2020-10-06	13.38	13.91	13.21	13.81	49132	{}	\N	2020-10-12 15:01:50.436349
46023	GHM	2020-10-07	14.1	14.6	13.59	14.5	89835	{}	\N	2020-10-12 15:01:50.440758
46024	GHM	2020-10-08	14.65	14.9028	14.41	14.41	64098	{}	\N	2020-10-12 15:01:50.445354
46025	GHM	2020-10-09	14.6	14.81	14.51	14.62	21333	{}	\N	2020-10-12 15:01:50.450005
46026	GHM	2020-10-12	14.7	14.7	14.38	14.38	4686	{}	\N	2020-10-12 15:01:50.454383
46027	GIGM	2020-09-28	2.85	2.945	2.8	2.83	40096	{}	\N	2020-10-12 15:01:51.486102
46028	GIGM	2020-09-29	2.84	2.84	2.8	2.84	23668	{}	\N	2020-10-12 15:01:51.497414
46029	GIGM	2020-09-30	2.86	2.88	2.81	2.8401	8945	{}	\N	2020-10-12 15:01:51.50428
46030	GIGM	2020-10-01	2.85	2.85	2.81	2.85	2634	{}	\N	2020-10-12 15:01:51.51281
46031	GIGM	2020-10-02	2.8	2.89	2.75	2.85	29943	{}	\N	2020-10-12 15:01:51.518931
46032	GIGM	2020-10-05	2.825	2.885	2.7801	2.7901	12374	{}	\N	2020-10-12 15:01:51.525725
46033	GIGM	2020-10-06	2.92	2.92	2.81	2.82	5391	{}	\N	2020-10-12 15:01:51.532407
46034	GIGM	2020-10-07	2.85	2.93	2.8195	2.9	7742	{}	\N	2020-10-12 15:01:51.53765
46035	GIGM	2020-10-08	2.9	2.9	2.84	2.84	3751	{}	\N	2020-10-12 15:01:51.542361
46036	GIGM	2020-10-09	2.95	2.98	2.87	2.87	10720	{}	\N	2020-10-12 15:01:51.546719
46037	GIGM	2020-10-12	2.854	2.8806	2.8463	2.8463	1989	{}	\N	2020-10-12 15:01:51.563096
46038	GIS	2020-09-28	59.87	60.495	59.15	59.9	2896698	{}	\N	2020-10-12 15:01:52.846047
46039	GIS	2020-09-29	59.87	60.24	59.43	59.86	4390087	{}	\N	2020-10-12 15:01:52.857222
46040	GIS	2020-09-30	59.93	62.03	59.79	61.68	14873369	{}	\N	2020-10-12 15:01:52.864507
46041	GIS	2020-10-01	61.81	62.68	60.77	62.57	12048972	{}	\N	2020-10-12 15:01:52.870391
46042	GIS	2020-10-02	62.35	63.29	61.98	62.52	8254462	{}	\N	2020-10-12 15:01:52.875572
46043	GIS	2020-10-05	62.41	63.03	62.25	62.68	5023083	{}	\N	2020-10-12 15:01:52.880664
46044	GIS	2020-10-06	62.39	62.6199	60.97	61.23	6673141	{}	\N	2020-10-12 15:01:52.884941
46045	GIS	2020-10-07	61.5	61.86	60.8	61.59	6424366	{}	\N	2020-10-12 15:01:52.888971
46046	GIS	2020-10-08	61.26	61.59	60.74	61.39	4879422	{}	\N	2020-10-12 15:01:52.89311
46047	GIS	2020-10-09	61.51	62.23	61.13	61.87	5978747	{}	\N	2020-10-12 15:01:52.897223
46048	GIS	2020-10-12	62.01	62.54	61.8	62.52	1204078	{}	\N	2020-10-12 15:01:52.900988
46049	GKOS	2020-09-28	49.4	51.71	49.39	50.55	502979	{}	\N	2020-10-12 15:01:54.014029
46050	GKOS	2020-09-29	50.2	52.03	50.175	50.97	1040296	{}	\N	2020-10-12 15:01:54.023493
46051	GKOS	2020-09-30	51.16	51.5	48.71	49.52	1104708	{}	\N	2020-10-12 15:01:54.030089
46052	GKOS	2020-10-01	50.22	50.74	49.635	49.85	493781	{}	\N	2020-10-12 15:01:54.035808
46053	GKOS	2020-10-02	49.0121	50.36	48.68	49.16	1055860	{}	\N	2020-10-12 15:01:54.042801
46054	GKOS	2020-10-05	49.59	50.22	49.16	49.48	575620	{}	\N	2020-10-12 15:01:54.048036
46055	GKOS	2020-10-06	49.7	50.41	48.88	48.9	908338	{}	\N	2020-10-12 15:01:54.052863
46056	GKOS	2020-10-07	49.46	53.02	49.4307	52.54	1154450	{}	\N	2020-10-12 15:01:54.057404
46057	GKOS	2020-10-08	52	52.99	51.43	52.46	1148724	{}	\N	2020-10-12 15:01:54.062047
46058	GKOS	2020-10-09	53	55.43	52.77	54.59	1239440	{}	\N	2020-10-12 15:01:54.066245
46059	GKOS	2020-10-12	54.76	55.03	53.58	54.1	93102	{}	\N	2020-10-12 15:01:54.070311
46060	GLDD	2020-09-28	9.53	9.64	9.435	9.6	270670	{}	\N	2020-10-12 15:01:55.007147
46061	GLDD	2020-09-29	9.61	9.68	9.445	9.575	220486	{}	\N	2020-10-12 15:01:55.01533
46062	GLDD	2020-09-30	9.61	9.79	9.46	9.51	372189	{}	\N	2020-10-12 15:01:55.020892
46063	GLDD	2020-10-01	9.51	9.66	9.29	9.35	248858	{}	\N	2020-10-12 15:01:55.025926
46064	GLDD	2020-10-02	9.21	9.47	9.101	9.4	167107	{}	\N	2020-10-12 15:01:55.030403
46065	GLDD	2020-10-05	9.7	9.99	9.7	9.97	311962	{}	\N	2020-10-12 15:01:55.034871
46066	GLDD	2020-10-06	10	10.28	9.97	10.095	363555	{}	\N	2020-10-12 15:01:55.039178
46067	GLDD	2020-10-07	10.1	10.43	10.1	10.38	303278	{}	\N	2020-10-12 15:01:55.043327
46068	GLDD	2020-10-08	10.44	10.69	10.2	10.6	240350	{}	\N	2020-10-12 15:01:55.04702
46069	GLDD	2020-10-09	10.6	10.75	10.49	10.51	223965	{}	\N	2020-10-12 15:01:55.050858
46070	GLDD	2020-10-12	10.6	10.67	10.48	10.591	38046	{}	\N	2020-10-12 15:01:55.054766
46071	GLQ	2020-09-28	11.05	11.16	10.89	11.12	69743	{}	\N	2020-10-12 15:01:56.984214
46072	GLQ	2020-09-29	11.15	11.1816	11.08	11.17	41453	{}	\N	2020-10-12 15:01:56.993945
46073	GLQ	2020-09-30	11.19	11.33	11.16	11.21	65641	{}	\N	2020-10-12 15:01:57.000615
46074	GLQ	2020-10-01	11.28	11.4	11.22	11.4	54462	{}	\N	2020-10-12 15:01:57.007156
46075	GLQ	2020-10-02	11.25	11.4	11.16	11.38	53462	{}	\N	2020-10-12 15:01:57.012663
46076	GLQ	2020-10-05	11.38	11.54	11.38	11.54	42028	{}	\N	2020-10-12 15:01:57.016938
46077	GLQ	2020-10-06	11.55	11.69	11.496	11.51	52303	{}	\N	2020-10-12 15:01:57.020557
46078	GLQ	2020-10-07	11.67	11.71	11.6372	11.68	59805	{}	\N	2020-10-12 15:01:57.024248
46079	GLQ	2020-10-08	11.78	11.87	11.72	11.87	41281	{}	\N	2020-10-12 15:01:57.027698
46080	GLQ	2020-10-09	11.92	12.01	11.9	11.93	37989	{}	\N	2020-10-12 15:01:57.031199
46081	GLQ	2020-10-12	12.15	12.15	12.01	12.07	33190	{}	\N	2020-10-12 15:01:57.03474
46082	GME	2020-09-28	10.16	10.26	9.55	10.09	6831663	{}	\N	2020-10-12 15:01:59.585335
46083	GME	2020-09-29	10	10.65	9.93	10.35	10462415	{}	\N	2020-10-12 15:01:59.596102
46084	GME	2020-09-30	10.25	10.7597	10.06	10.2	12122650	{}	\N	2020-10-12 15:01:59.603635
46085	GME	2020-10-01	10.09	10.25	9.69	9.77	9122848	{}	\N	2020-10-12 15:01:59.60971
46086	GME	2020-10-02	9.38	9.78	9.3	9.39	8569623	{}	\N	2020-10-12 15:01:59.614968
46087	GME	2020-10-05	9.44	9.59	9.2502	9.46	5702305	{}	\N	2020-10-12 15:01:59.619937
46088	GME	2020-10-06	9.56	9.835	9.1	9.13	8871366	{}	\N	2020-10-12 15:01:59.624819
46089	GME	2020-10-07	9.23	9.56	9.1678	9.36	6602699	{}	\N	2020-10-12 15:01:59.629182
46090	GME	2020-10-08	9.54	13.64	9.19	13.49	152870857	{}	\N	2020-10-12 15:01:59.633692
46091	GME	2020-10-09	12.83	14.8	11.9	12.02	154880460	{}	\N	2020-10-12 15:01:59.638027
46092	GME	2020-10-12	11.66	12.7662	11.4	12.3399	24523590	{}	\N	2020-10-12 15:01:59.642698
46093	GMOM	2020-09-28	24.0315	24.1058	23.995	24.1058	6427	{}	\N	2020-10-12 15:02:01.508192
46094	GMOM	2020-09-29	24.2099	24.2099	24.09	24.09	3250	{}	\N	2020-10-12 15:02:01.518052
46095	GMOM	2020-09-30	24.17	24.19	23.985	23.985	5954	{}	\N	2020-10-12 15:02:01.524192
46096	GMOM	2020-10-01	24.1	24.12	24.08	24.08	4098	{}	\N	2020-10-12 15:02:01.529386
46097	GMOM	2020-10-02	24.05	24.0775	23.9608	24.03	7691	{}	\N	2020-10-12 15:02:01.534061
46098	GMOM	2020-10-05	24.2283	24.2283	24.13	24.1601	8292	{}	\N	2020-10-12 15:02:01.538752
46099	GMOM	2020-10-06	24.16	24.189	23.95	23.9574	5760	{}	\N	2020-10-12 15:02:01.542904
46100	GMOM	2020-10-07	24.0094	24.1599	23.3	24.1347	15985	{}	\N	2020-10-12 15:02:01.546705
46101	GMOM	2020-10-08	24.13	24.2682	24.13	24.235	12769	{}	\N	2020-10-12 15:02:01.551674
46102	GMOM	2020-10-09	24.65	24.65	24.04	24.5191	6392	{}	\N	2020-10-12 15:02:01.55579
46103	GMOM	2020-10-12	24.5699	24.5699	24.38	24.38	2544	{}	\N	2020-10-12 15:02:01.559832
46104	GMO	2020-09-28	0.1133	0.1167	0.112	0.1139	1045193	{}	\N	2020-10-12 15:02:03.144371
46105	GMO	2020-09-29	0.114	0.1171	0.1127	0.1148	1186123	{}	\N	2020-10-12 15:02:03.152059
46106	GMO	2020-09-30	0.117	0.117	0.1114	0.112	1538984	{}	\N	2020-10-12 15:02:03.157078
46107	GMO	2020-10-01	0.113	0.113	0.113	0.113	10000	{}	\N	2020-10-12 15:02:03.162023
46108	KEY	2020-09-28	11.94	12.19	11.92	12.13	7518329	{}	\N	2020-10-12 15:02:04.1289
46109	KEY	2020-09-29	12.09	12.09	11.6	11.83	12997868	{}	\N	2020-10-12 15:02:04.13763
46110	KEY	2020-09-30	11.93	12.105	11.81	11.93	20397558	{}	\N	2020-10-12 15:02:04.14391
46111	KEY	2020-10-01	11.97	12.08	11.75	11.92	13059659	{}	\N	2020-10-12 15:02:04.149223
46112	KEY	2020-10-02	11.69	12.34	11.6	12.22	15646777	{}	\N	2020-10-12 15:02:04.154682
46113	KEY	2020-10-05	12.4	12.78	12.32	12.76	13409488	{}	\N	2020-10-12 15:02:04.159513
46114	KEY	2020-10-06	12.96	13.21	12.55	12.6	16354671	{}	\N	2020-10-12 15:02:04.164012
46115	KEY	2020-10-07	12.79	13.33	12.79	13.16	17117044	{}	\N	2020-10-12 15:02:04.16828
46116	KEY	2020-10-08	13.23	13.395	13.06	13.31	13681813	{}	\N	2020-10-12 15:02:04.172507
46117	KEY	2020-10-09	13.34	13.41	13.04	13.12	7649943	{}	\N	2020-10-12 15:02:04.176861
46118	KEY	2020-10-12	13.14	13.29	13.09	13.205	2337714	{}	\N	2020-10-12 15:02:04.181133
46119	GIL	2020-09-28	20.33	20.45	20.11	20.21	926320	{}	\N	2020-10-12 15:02:06.023026
46120	GIL	2020-09-29	20.25	20.33	19.99	20.2	954960	{}	\N	2020-10-12 15:02:06.034371
46121	GIL	2020-09-30	20.25	20.445	19.58	19.67	1070171	{}	\N	2020-10-12 15:02:06.043087
46122	GIL	2020-10-01	19.75	20.53	19.66	20.5	1675665	{}	\N	2020-10-12 15:02:06.049612
46123	GIL	2020-10-02	20	20.75	19.97	20.72	662162	{}	\N	2020-10-12 15:02:06.055701
46124	GIL	2020-10-05	20.95	21.04	20.655	20.79	1037106	{}	\N	2020-10-12 15:02:06.060843
46125	GIL	2020-10-06	20.91	20.91	20.1	20.17	3032768	{}	\N	2020-10-12 15:02:06.066258
46126	GIL	2020-10-07	20.48	21.14	20.265	21.03	1996789	{}	\N	2020-10-12 15:02:06.071231
46127	GIL	2020-10-08	21.34	22.29	21.29	22.25	2680087	{}	\N	2020-10-12 15:02:06.075877
46128	GIL	2020-10-09	22.5	22.69	22.26	22.63	1850821	{}	\N	2020-10-12 15:02:06.083843
46129	GIL	2020-10-12	22.68	22.68	22.33	22.39	119686	{}	\N	2020-10-12 15:02:06.087939
46130	GJT	2020-09-30	19.66	19.79	19.66	19.79	8286	{}	\N	2020-10-12 15:02:06.940114
46131	GJT	2020-10-01	19.8	19.8	19.52	19.7355	4154	{}	\N	2020-10-12 15:02:06.947532
46132	GJT	2020-10-02	19.78	19.79	19.76	19.79	6302	{}	\N	2020-10-12 15:02:06.95299
46133	GJT	2020-10-05	19.79	19.79	19.56	19.733	5004	{}	\N	2020-10-12 15:02:06.957803
46134	GJT	2020-10-06	19.6	19.6	19.4366	19.46	4200	{}	\N	2020-10-12 15:02:06.962761
46135	GLRE	2020-09-28	6.46	6.67	6.45	6.58	85388	{}	\N	2020-10-12 15:02:08.314408
46136	GLRE	2020-09-29	6.53	6.69	6.47	6.64	64761	{}	\N	2020-10-12 15:02:08.32242
46137	GLRE	2020-09-30	6.68	6.76	6.605	6.72	59079	{}	\N	2020-10-12 15:02:08.327695
46138	GLRE	2020-10-01	6.7	6.8	6.61	6.73	131293	{}	\N	2020-10-12 15:02:08.33311
46139	GLRE	2020-10-02	6.58	6.967	6.48	6.88	86802	{}	\N	2020-10-12 15:02:08.337918
46140	GLRE	2020-10-05	6.92	7.14	6.91	7.03	123377	{}	\N	2020-10-12 15:02:08.342636
46141	GLRE	2020-10-06	7.04	7.13	6.86	6.95	73979	{}	\N	2020-10-12 15:02:08.346543
46142	GLRE	2020-10-07	7	7.12	6.85	6.95	65808	{}	\N	2020-10-12 15:02:08.350371
46143	GLRE	2020-10-08	7.02	7.08	6.94	7.01	58799	{}	\N	2020-10-12 15:02:08.35448
46144	GLRE	2020-10-09	7.08	7.21	6.86	6.93	142848	{}	\N	2020-10-12 15:02:08.35842
46145	GLRE	2020-10-12	6.86	7.05	6.86	7.03	14541	{}	\N	2020-10-12 15:02:08.36399
46146	GLYC	2020-09-28	3.26	3.31	3.18	3.26	196979	{}	\N	2020-10-12 15:02:09.301662
46147	GLYC	2020-09-29	3.25	3.27	3.11	3.12	185645	{}	\N	2020-10-12 15:02:09.310964
46148	GLYC	2020-09-30	3.17	3.19	3.06	3.07	263098	{}	\N	2020-10-12 15:02:09.31736
46149	GLYC	2020-10-01	3.09	3.22	3.06	3.18	213654	{}	\N	2020-10-12 15:02:09.322741
46150	GLYC	2020-10-02	3.0806	3.22	3.06	3.13	114085	{}	\N	2020-10-12 15:02:09.327586
46151	GLYC	2020-10-05	3.17	3.3	3.17	3.27	1305907	{}	\N	2020-10-12 15:02:09.332094
46152	GLYC	2020-10-06	3.58	3.58	3.32	3.41	746816	{}	\N	2020-10-12 15:02:09.336183
46153	GLYC	2020-10-07	3.43	3.53	3.4	3.5	248106	{}	\N	2020-10-12 15:02:09.340459
46154	GLYC	2020-10-08	3.51	3.62	3.45	3.53	217036	{}	\N	2020-10-12 15:02:09.344478
46155	GLYC	2020-10-09	3.52	3.6	3.48	3.55	126783	{}	\N	2020-10-12 15:02:09.348551
46156	GLYC	2020-10-12	3.54	3.57	3.48	3.55	19450	{}	\N	2020-10-12 15:02:09.352514
46157	GMLPP	2020-09-28	18.85	18.88	18.2584	18.57	16055	{}	\N	2020-10-12 15:02:10.870397
46158	GMLPP	2020-09-29	18.57	18.93	17.73	18.93	8955	{}	\N	2020-10-12 15:02:10.879906
46159	GMLPP	2020-09-30	18.5	19.25	18.2866	18.93	12026	{}	\N	2020-10-12 15:02:10.886263
46160	GMLPP	2020-10-01	19.15	19.5	18.937	19.39	12199	{}	\N	2020-10-12 15:02:10.892184
46161	GMLPP	2020-10-02	19.1093	19.49	18.63	19.32	7005	{}	\N	2020-10-12 15:02:10.897504
46162	GMLPP	2020-10-05	19.1	19.605	19.1	19.605	15457	{}	\N	2020-10-12 15:02:10.902284
46163	GMLPP	2020-10-06	19.07	19.65	19.05	19.45	14682	{}	\N	2020-10-12 15:02:10.906841
46164	GMLPP	2020-10-07	19.45	19.685	19.13	19.41	21634	{}	\N	2020-10-12 15:02:10.911222
46165	GMLPP	2020-10-08	19.39	19.6882	19.39	19.5189	8185	{}	\N	2020-10-12 15:02:10.915957
46166	GMLPP	2020-10-09	19.466	19.5	19	19.305	8980	{}	\N	2020-10-12 15:02:10.920316
46167	GMLPP	2020-10-12	19.4144	19.57	19.4144	19.5	3634	{}	\N	2020-10-12 15:02:10.924691
46168	GNCA	2020-09-28	2.38	2.38	2.17	2.2	310786	{}	\N	2020-10-12 15:02:11.862057
46169	GNCA	2020-09-29	2.21	2.32	2.18	2.23	188823	{}	\N	2020-10-12 15:02:11.872205
46170	GNCA	2020-09-30	2.26	2.32	2.25	2.29	245539	{}	\N	2020-10-12 15:02:11.879335
46171	GNCA	2020-10-01	2.29	2.292	2.15	2.23	409508	{}	\N	2020-10-12 15:02:11.885106
46172	GNCA	2020-10-02	2.18	2.39	2.17	2.32	376755	{}	\N	2020-10-12 15:02:11.890564
46173	GNCA	2020-10-05	2.38	2.49	2.3674	2.41	342194	{}	\N	2020-10-12 15:02:11.895491
46174	GNCA	2020-10-06	2.43	2.45	2.38	2.39	165075	{}	\N	2020-10-12 15:02:11.899621
46175	GNCA	2020-10-07	2.42	2.5	2.39	2.46	247512	{}	\N	2020-10-12 15:02:11.903953
46176	GNCA	2020-10-08	2.48	2.52	2.44	2.49	141270	{}	\N	2020-10-12 15:02:11.907976
46177	GNCA	2020-10-09	2.51	2.57	2.47	2.5	149222	{}	\N	2020-10-12 15:02:11.911894
46178	GNCA	2020-10-12	2.55	2.55	2.48	2.54	39295	{}	\N	2020-10-12 15:02:11.916154
46179	GNUS	2020-09-28	1.19	1.26	1.16	1.18	36868681	{}	\N	2020-10-12 15:02:13.007976
46180	GNUS	2020-09-29	1.17	1.33	1.17	1.29	41230341	{}	\N	2020-10-12 15:02:13.019777
46181	GNUS	2020-09-30	1.28	1.28	1.18	1.19	28186310	{}	\N	2020-10-12 15:02:13.026546
46182	GNUS	2020-10-01	1.21	1.23	1.18	1.23	17519273	{}	\N	2020-10-12 15:02:13.032417
46183	GNUS	2020-10-02	1.17	1.28	1.17	1.21	32575095	{}	\N	2020-10-12 15:02:13.037737
46184	GNUS	2020-10-05	1.2	1.25	1.19	1.2	17152081	{}	\N	2020-10-12 15:02:13.042508
46185	GNUS	2020-10-06	1.21	1.24	1.15	1.16	18946863	{}	\N	2020-10-12 15:02:13.047001
46186	GNUS	2020-10-07	1.18	1.33	1.17	1.27	32231203	{}	\N	2020-10-12 15:02:13.050799
46187	GNUS	2020-10-08	1.34	1.38	1.28	1.29	24911053	{}	\N	2020-10-12 15:02:13.055004
46188	GNUS	2020-10-09	1.34	1.45	1.31	1.405	54379740	{}	\N	2020-10-12 15:02:13.059319
46189	GNUS	2020-10-12	1.46	1.49	1.41	1.445	13145340	{}	\N	2020-10-12 15:02:13.063328
46190	GOL	2020-09-28	6.57	6.76	6.18	6.24	1573560	{}	\N	2020-10-12 15:02:14.244035
46191	GOL	2020-09-29	6.09	6.135	5.835	5.86	4064059	{}	\N	2020-10-12 15:02:14.250202
46192	GOL	2020-09-30	5.9	6.215	5.9	6.1	2506111	{}	\N	2020-10-12 15:02:14.255138
46193	GOL	2020-10-01	6.27	6.45	6.2	6.37	3017969	{}	\N	2020-10-12 15:02:14.25972
46194	GOL	2020-10-02	6.23	6.33	6.1	6.21	2365767	{}	\N	2020-10-12 15:02:14.264664
46195	GOL	2020-10-05	6.35	6.42	6.1719	6.37	1851253	{}	\N	2020-10-12 15:02:14.268898
46196	GOL	2020-10-06	6.7	7.05	6.66	6.79	4986936	{}	\N	2020-10-12 15:02:14.272931
46197	GOL	2020-10-07	6.74	6.87	6.47	6.52	2694121	{}	\N	2020-10-12 15:02:14.277338
46198	GOL	2020-10-08	6.6	6.69	6.5	6.68	1965499	{}	\N	2020-10-12 15:02:14.281049
46199	GOL	2020-10-09	6.85	7.19	6.8401	7	4279498	{}	\N	2020-10-12 15:02:14.285003
46200	GOL	2020-10-12	7.01	7.02	6.785	6.94	359396	{}	\N	2020-10-12 15:02:14.289179
46201	GOOD	2020-09-28	16.79	17.38	16.75	17.27	135821	{}	\N	2020-10-12 15:02:15.498787
46202	GOOD	2020-09-29	17.19	17.35	16.69	17	142148	{}	\N	2020-10-12 15:02:15.504816
46203	GOOD	2020-09-30	17.15	17.49	16.76	16.85	174176	{}	\N	2020-10-12 15:02:15.509277
46204	GOOD	2020-10-01	16.89	17.31	16.81	17.29	145801	{}	\N	2020-10-12 15:02:15.512843
46205	GOOD	2020-10-02	16.87	17.69	16.68	17.625	168629	{}	\N	2020-10-12 15:02:15.516963
46206	GOOD	2020-10-05	17.72	17.84	17.2237	17.47	128571	{}	\N	2020-10-12 15:02:15.521075
46207	GOOD	2020-10-06	17.63	17.9	17.3609	17.46	133831	{}	\N	2020-10-12 15:02:15.524673
46208	GOOD	2020-10-07	17.54	17.659	17.43	17.52	120842	{}	\N	2020-10-12 15:02:15.528825
46209	GOOD	2020-10-08	17.71	17.8559	17.56	17.68	135810	{}	\N	2020-10-12 15:02:15.532879
46210	GOOD	2020-10-09	17.7	17.82	17.47	17.53	142092	{}	\N	2020-10-12 15:02:15.536936
46211	GOOD	2020-10-12	17.55	17.6	17.2	17.4599	43863	{}	\N	2020-10-12 15:02:15.541178
46212	GORO	2020-09-28	3.26	3.33	3.22	3.29	582918	{}	\N	2020-10-12 15:02:17.009655
46213	GORO	2020-09-29	3.3	3.48	3.3	3.46	1091558	{}	\N	2020-10-12 15:02:17.019633
46214	GORO	2020-09-30	3.46	3.5	3.37	3.41	727833	{}	\N	2020-10-12 15:02:17.02625
46215	GORO	2020-10-01	3.46	3.54	3.43	3.505	1039705	{}	\N	2020-10-12 15:02:17.031804
46216	GORO	2020-10-02	3.49	3.6	3.44	3.6	1117406	{}	\N	2020-10-12 15:02:17.036505
46217	GORO	2020-10-05	3.78	3.78	3.3	3.32	2468148	{}	\N	2020-10-12 15:02:17.0415
46218	GORO	2020-10-06	3.32	3.34	2.79	2.79	4386959	{}	\N	2020-10-12 15:02:17.046102
46219	GORO	2020-10-07	2.89	2.93	2.69	2.72	3615283	{}	\N	2020-10-12 15:02:17.050176
46220	GORO	2020-10-08	2.77	3.02	2.77	3	3307885	{}	\N	2020-10-12 15:02:17.054058
46221	GORO	2020-10-09	3.02	3.1	2.97	3.09	1346253	{}	\N	2020-10-12 15:02:17.058061
46222	GORO	2020-10-12	3.15	3.16	2.97	2.975	432336	{}	\N	2020-10-12 15:02:17.061926
46223	GGN	2020-09-28	3.41	3.44	3.38	3.41	762734	{}	\N	2020-10-12 15:02:19.002869
46224	GGN	2020-09-29	3.405	3.44	3.39	3.39	640677	{}	\N	2020-10-12 15:02:19.011843
46225	GGN	2020-09-30	3.38	3.43	3.38	3.41	544836	{}	\N	2020-10-12 15:02:19.017936
46226	GGN	2020-10-01	3.425	3.44	3.39	3.43	648574	{}	\N	2020-10-12 15:02:19.023871
46227	GGN	2020-10-02	3.4	3.42	3.38	3.38	893748	{}	\N	2020-10-12 15:02:19.029007
46228	GGN	2020-10-05	3.42	3.44	3.4	3.44	489470	{}	\N	2020-10-12 15:02:19.033636
46229	GGN	2020-10-06	3.44	3.47	3.38	3.39	1106547	{}	\N	2020-10-12 15:02:19.038218
46230	GGN	2020-10-07	3.41	3.43	3.395	3.42	470261	{}	\N	2020-10-12 15:02:19.042599
46231	GGN	2020-10-08	3.42	3.47	3.42	3.46	558191	{}	\N	2020-10-12 15:02:19.046761
46232	GGN	2020-10-09	3.49	3.52	3.48	3.51	825022	{}	\N	2020-10-12 15:02:19.050915
46233	GGN	2020-10-12	3.53	3.54	3.51	3.53	175274	{}	\N	2020-10-12 15:02:19.055017
46234	GILT	2020-09-28	5.63	5.66	5.62	5.62	80341	{}	\N	2020-10-12 15:02:20.188989
46235	GILT	2020-09-29	5.6234	5.69	5.61	5.66	73586	{}	\N	2020-10-12 15:02:20.199782
46236	GILT	2020-09-30	5.65	5.67	5.3501	5.45	296435	{}	\N	2020-10-12 15:02:20.206631
46237	GILT	2020-10-01	5.45	5.48	5.12	5.27	494490	{}	\N	2020-10-12 15:02:20.212486
46238	GILT	2020-10-02	5.18	5.26	5.12	5.21	184677	{}	\N	2020-10-12 15:02:20.217037
46239	GILT	2020-10-05	5.31	5.32	4.8	5.31	4561717	{}	\N	2020-10-12 15:02:20.221617
46240	GILT	2020-10-06	5.4	5.49	5.21	5.41	2223602	{}	\N	2020-10-12 15:02:20.225737
46241	GILT	2020-10-07	5.46	5.51	5.41	5.5	816218	{}	\N	2020-10-12 15:02:20.229731
46242	GILT	2020-10-08	5.48	5.49	5.16	5.25	852670	{}	\N	2020-10-12 15:02:20.233406
46243	GILT	2020-10-09	5.28	5.29	5.12	5.24	447038	{}	\N	2020-10-12 15:02:20.237304
46244	GILT	2020-10-12	5.24	5.24	5.08	5.13	126035	{}	\N	2020-10-12 15:02:20.241439
46245	GLOG	2020-09-28	2.72	2.73	2.58	2.68	993257	{}	\N	2020-10-12 15:02:22.646155
46246	GLOG	2020-09-29	2.68	2.715	2.55	2.62	1850880	{}	\N	2020-10-12 15:02:22.655093
46247	GLOG	2020-09-30	2.61	2.775	2.61	2.71	1139247	{}	\N	2020-10-12 15:02:22.661436
46248	GLOG	2020-10-01	2.72	2.86	2.72	2.84	868708	{}	\N	2020-10-12 15:02:22.667009
46249	GLOG	2020-10-02	2.78	2.8	2.65	2.8	1364878	{}	\N	2020-10-12 15:02:22.67269
46250	GLOG	2020-10-05	2.85	3	2.85	2.93	971455	{}	\N	2020-10-12 15:02:22.677282
46251	GLOG	2020-10-06	2.99	3.025	2.72	2.73	1202937	{}	\N	2020-10-12 15:02:22.681996
46252	GLOG	2020-10-07	2.77	2.8178	2.71	2.76	819742	{}	\N	2020-10-12 15:02:22.686378
46253	GLOG	2020-10-08	2.77	2.91	2.76	2.89	707722	{}	\N	2020-10-12 15:02:22.691057
46254	GLOG	2020-10-09	2.92	2.92	2.78	2.78	510526	{}	\N	2020-10-12 15:02:22.695889
46255	GLOG	2020-10-12	2.79	2.8309	2.71	2.725	381196	{}	\N	2020-10-12 15:02:22.700089
46256	GLT	2020-09-28	13.81	14.02	13.73	13.78	227114	{}	\N	2020-10-12 15:02:23.670797
46257	GLT	2020-09-29	13.76	13.98	13.44	13.73	249037	{}	\N	2020-10-12 15:02:23.679236
46258	GLT	2020-09-30	13.79	14	13.68	13.77	295743	{}	\N	2020-10-12 15:02:23.685814
46259	GLT	2020-10-01	13.85	14.03	13.71	13.89	293938	{}	\N	2020-10-12 15:02:23.692726
46260	GLT	2020-10-02	13.62	14	13.5315	13.87	233096	{}	\N	2020-10-12 15:02:23.700245
46261	GLT	2020-10-05	14.01	14.5	14.01	14.14	261909	{}	\N	2020-10-12 15:02:23.707142
46262	GLT	2020-10-06	14.34	14.61	14.04	14.08	325633	{}	\N	2020-10-12 15:02:23.716051
46263	GLT	2020-10-07	14.3	14.35	13.99	14.24	393101	{}	\N	2020-10-12 15:02:23.722428
46264	GLT	2020-10-08	14.42	14.65	14.11	14.23	272428	{}	\N	2020-10-12 15:02:23.728417
46265	GLT	2020-10-09	14.39	14.51	14.19	14.37	295497	{}	\N	2020-10-12 15:02:23.733585
46266	GLT	2020-10-12	14.38	14.72	14.38	14.58	18864	{}	\N	2020-10-12 15:02:23.740745
46267	GNTY	2020-09-28	23.81	25.18	23.695	24.85	13080	{}	\N	2020-10-12 15:02:27.694109
46268	GNTY	2020-09-29	24.66	24.66	24.345	24.41	7154	{}	\N	2020-10-12 15:02:27.705294
46269	GNTY	2020-09-30	24.1201	25.32	24.1201	24.89	18616	{}	\N	2020-10-12 15:02:27.712885
46270	GNTY	2020-10-01	24.89	25.5	24.84	25.05	19141	{}	\N	2020-10-12 15:02:27.719928
46271	GNTY	2020-10-02	25	25.79	25	25.45	47482	{}	\N	2020-10-12 15:02:27.726133
46272	GNTY	2020-10-05	24.65	26.47	24.65	26.47	21529	{}	\N	2020-10-12 15:02:27.731054
46273	GNTY	2020-10-06	26.23	27.615	25.54	26.6	12340	{}	\N	2020-10-12 15:02:27.735429
46274	GNTY	2020-10-07	26.74	27.4	25.95	26.7	15287	{}	\N	2020-10-12 15:02:27.740599
46275	GNTY	2020-10-08	26.94	27.74	26.34	27.59	12317	{}	\N	2020-10-12 15:02:27.744971
46276	GNTY	2020-10-09	26.72	27.8	26.72	27.6	6635	{}	\N	2020-10-12 15:02:27.749302
46277	GNTY	2020-10-12	27.5	27.5	27.085	27.0875	2969	{}	\N	2020-10-12 15:02:27.753893
46278	GOLD	2020-09-28	28	28.04	27.45	27.76	12613200	{}	\N	2020-10-12 15:02:28.923053
46279	GOLD	2020-09-29	27.97	28.48	27.8701	28.28	24831433	{}	\N	2020-10-12 15:02:28.931086
46280	GOLD	2020-09-30	28.12	28.39	27.8	28.11	29526202	{}	\N	2020-10-12 15:02:28.936684
46281	GOLD	2020-10-01	28.39	28.55	28.07	28.26	20493686	{}	\N	2020-10-12 15:02:28.941954
46282	GOLD	2020-10-02	28.19	28.3899	27.82	27.89	19182369	{}	\N	2020-10-12 15:02:28.946784
46283	GOLD	2020-10-05	28.01	28.74	27.9701	28.37	21622133	{}	\N	2020-10-12 15:02:28.95157
46284	GOLD	2020-10-06	28.41	28.54	27.04	27.06	36089173	{}	\N	2020-10-12 15:02:28.955621
46285	GOLD	2020-10-07	27.39	27.53	26.88	26.95	36910481	{}	\N	2020-10-12 15:02:28.959525
46286	GOLD	2020-10-08	27.12	27.52	26.9	27.27	27089372	{}	\N	2020-10-12 15:02:28.963135
46287	GOLD	2020-10-09	27.77	28.05	27.52	28.05	29078687	{}	\N	2020-10-12 15:02:28.966618
46288	GOLD	2020-10-12	27.95	28.14	27.8	27.83	8239210	{}	\N	2020-10-12 15:02:28.970416
46289	GOLF	2020-09-28	33.42	33.43	33.025	33.1	156970	{}	\N	2020-10-12 15:02:30.257512
46290	GOLF	2020-09-29	33.36	33.39	32.93	33.17	286127	{}	\N	2020-10-12 15:02:30.267481
46291	GOLF	2020-09-30	33.23	33.82	33.23	33.61	509639	{}	\N	2020-10-12 15:02:30.27381
46292	GOLF	2020-10-01	33.82	34.78	33.59	34.7	630203	{}	\N	2020-10-12 15:02:30.279475
46293	GOLF	2020-10-02	33.99	35.15	33.99	34.87	363214	{}	\N	2020-10-12 15:02:30.284163
46294	GOLF	2020-10-05	35.03	36.44	35.0043	36.4	438640	{}	\N	2020-10-12 15:02:30.288774
46295	GOLF	2020-10-06	36.73	36.79	35.36	35.5	442156	{}	\N	2020-10-12 15:02:30.293216
46296	GOLF	2020-10-07	35.8	36.25	35.62	35.63	491123	{}	\N	2020-10-12 15:02:30.296914
46297	GOLF	2020-10-08	36.02	36.02	34.95	35.1	627843	{}	\N	2020-10-12 15:02:30.300861
46298	GOLF	2020-10-09	35.26	35.61	35.16	35.25	338690	{}	\N	2020-10-12 15:02:30.304826
46299	GOLF	2020-10-12	35.25	35.376	34.97	35.07	45865	{}	\N	2020-10-12 15:02:30.309252
46300	GOVT	2020-09-28	28.01	28.01	27.98	27.99	3902737	{}	\N	2020-10-12 15:02:31.733226
46301	GOVT	2020-09-29	28	28.03	28	28.01	4415556	{}	\N	2020-10-12 15:02:31.743606
46302	GOVT	2020-09-30	27.98	27.98	27.9	27.95	5261764	{}	\N	2020-10-12 15:02:31.750186
46303	GOVT	2020-10-01	27.89	27.95	27.86	27.93	4278504	{}	\N	2020-10-12 15:02:31.756299
46304	GOVT	2020-10-02	27.95	27.95	27.88	27.895	1999338	{}	\N	2020-10-12 15:02:31.761629
46305	GOVT	2020-10-05	27.82	27.82	27.74	27.745	6175756	{}	\N	2020-10-12 15:02:31.766303
46306	GOVT	2020-10-06	27.7368	27.83	27.71	27.8	7467563	{}	\N	2020-10-12 15:02:31.770391
46307	GOVT	2020-10-07	27.73	27.77	27.7	27.725	13986333	{}	\N	2020-10-12 15:02:31.774835
46308	GOVT	2020-10-08	27.76	27.775	27.74	27.775	9375976	{}	\N	2020-10-12 15:02:31.778908
46309	GOVT	2020-10-09	27.75	27.78	27.7056	27.755	5741398	{}	\N	2020-10-12 15:02:31.782903
46310	GOVT	2020-10-12	27.779	27.78	27.77	27.77	262053	{}	\N	2020-10-12 15:02:31.786754
46311	GIFI	2020-09-28	3.03	3.1	3.03	3.09	10624	{}	\N	2020-10-12 15:02:32.716495
46312	GIFI	2020-09-29	3.108	3.12	3.07	3.12	21538	{}	\N	2020-10-12 15:02:32.727121
46313	GIFI	2020-09-30	3.1199	3.175	3.1199	3.13	10753	{}	\N	2020-10-12 15:02:32.734934
46314	GIFI	2020-10-01	3.1595	3.5	3.1595	3.38	105374	{}	\N	2020-10-12 15:02:32.741971
46315	GIFI	2020-10-02	3.4	3.4	3.35	3.4	38211	{}	\N	2020-10-12 15:02:32.747359
46316	GIFI	2020-10-05	3.45	3.52	3.414	3.5	11365	{}	\N	2020-10-12 15:02:32.752668
46317	GIFI	2020-10-06	3.54	3.6	3.44	3.44	43527	{}	\N	2020-10-12 15:02:32.757931
46318	GIFI	2020-10-07	3.44	3.5	3.3	3.3	20330	{}	\N	2020-10-12 15:02:32.762466
46319	GIFI	2020-10-08	3.4	3.4	3.28	3.31	9193	{}	\N	2020-10-12 15:02:32.766664
46320	GIFI	2020-10-09	3.32	3.35	3.28	3.305	21779	{}	\N	2020-10-12 15:02:32.771086
46321	GIFI	2020-10-12	3.34	3.35	3.34	3.35	1037	{}	\N	2020-10-12 15:02:32.775143
46322	GIM	2020-09-28	5.34	5.39	5.34	5.39	232542	{}	\N	2020-10-12 15:02:33.787773
46323	GIM	2020-09-29	5.39	5.41	5.37	5.38	763827	{}	\N	2020-10-12 15:02:33.796451
46324	GIM	2020-09-30	5.37	5.39	5.335	5.35	1057946	{}	\N	2020-10-12 15:02:33.802819
46325	GIM	2020-10-01	5.35	5.37	5.34	5.35	446181	{}	\N	2020-10-12 15:02:33.808853
46326	GIM	2020-10-02	5.32	5.36	5.32	5.33	701349	{}	\N	2020-10-12 15:02:33.814288
46327	GIM	2020-10-05	5.35	5.41	5.34	5.39	565427	{}	\N	2020-10-12 15:02:33.819107
46328	GIM	2020-10-06	5.39	5.39	5.31	5.31	388736	{}	\N	2020-10-12 15:02:33.824045
46329	GIM	2020-10-07	5.32	5.39	5.31	5.32	612630	{}	\N	2020-10-12 15:02:33.828559
46330	GIM	2020-10-08	5.33	5.37	5.32	5.35	409888	{}	\N	2020-10-12 15:02:33.832908
46331	GIM	2020-10-09	5.39	5.39	5.32	5.33	641589	{}	\N	2020-10-12 15:02:33.837569
46332	GIM	2020-10-12	5.34	5.36	5.34	5.3525	97156	{}	\N	2020-10-12 15:02:33.841693
46333	GLOP	2020-09-28	3.67	3.8	3.65	3.7	144218	{}	\N	2020-10-12 15:02:34.864391
46334	GLOP	2020-09-29	3.64	3.765	3.63	3.72	278967	{}	\N	2020-10-12 15:02:34.875458
46335	GLOP	2020-09-30	3.7	3.8	3.35	3.36	1396607	{}	\N	2020-10-12 15:02:34.88286
46336	GLOP	2020-10-01	3.47	3.56	3.445	3.51	400307	{}	\N	2020-10-12 15:02:34.88922
46337	GLOP	2020-10-02	3.4	3.57	3.37	3.55	422610	{}	\N	2020-10-12 15:02:34.894688
46338	GLOP	2020-10-05	3.6	3.71	3.55	3.57	270146	{}	\N	2020-10-12 15:02:34.899653
46339	GLOP	2020-10-06	3.65	3.65	3.44	3.46	375763	{}	\N	2020-10-12 15:02:34.903699
46340	GLOP	2020-10-07	3.45	3.51	3.36	3.47	488218	{}	\N	2020-10-12 15:02:34.908131
46341	GLOP	2020-10-08	3.51	3.635	3.47	3.6	374910	{}	\N	2020-10-12 15:02:34.912666
46342	GLOP	2020-10-09	3.62	3.65	3.5	3.52	363944	{}	\N	2020-10-12 15:02:34.916894
46343	GLOP	2020-10-12	3.52	3.59	3.46	3.47	121100	{}	\N	2020-10-12 15:02:34.92136
46344	GLV	2020-09-28	9.06	9.17	9.05	9.15	24541	{}	\N	2020-10-12 15:02:35.901475
46345	GLV	2020-09-29	9.18	9.21	9.095	9.17	31750	{}	\N	2020-10-12 15:02:35.910758
46346	GLV	2020-09-30	9.24	9.28	9.18	9.23	32302	{}	\N	2020-10-12 15:02:35.916811
46347	GLV	2020-10-01	9.29	9.36	9.23	9.33	40386	{}	\N	2020-10-12 15:02:35.922462
46348	GLV	2020-10-02	9.25	9.3198	9.23	9.3	24209	{}	\N	2020-10-12 15:02:35.927683
46349	GLV	2020-10-05	9.31	9.42	9.2909	9.41	28399	{}	\N	2020-10-12 15:02:35.93236
46350	GLV	2020-10-06	9.37	9.44	9.34	9.34	19556	{}	\N	2020-10-12 15:02:35.936655
46351	GLV	2020-10-07	9.54	9.54	9.38	9.46	40228	{}	\N	2020-10-12 15:02:35.940784
46352	GLV	2020-10-08	9.53	9.53	9.4605	9.47	10168	{}	\N	2020-10-12 15:02:35.944744
46353	GLV	2020-10-09	9.54	9.6	9.4886	9.58	27048	{}	\N	2020-10-12 15:02:35.948535
46354	GLV	2020-10-12	9.72	9.7222	9.615	9.6322	4801	{}	\N	2020-10-12 15:02:35.952628
46355	GMLP	2020-09-28	2.1	2.1899	2.04	2.05	111893	{}	\N	2020-10-12 15:02:37.15931
46356	GMLP	2020-09-29	2.05	2.06	2.03	2.06	91626	{}	\N	2020-10-12 15:02:37.168339
46357	GMLP	2020-09-30	2.04	2.09	2	2.03	274194	{}	\N	2020-10-12 15:02:37.174373
46358	GMLP	2020-10-01	2	2.06	2	2.04	186347	{}	\N	2020-10-12 15:02:37.179909
46359	GMLP	2020-10-02	1.96	2.06	1.95	2.03	282754	{}	\N	2020-10-12 15:02:37.184589
46360	GMLP	2020-10-05	2.01	2.14	2	2.12	314045	{}	\N	2020-10-12 15:02:37.189123
46361	GMLP	2020-10-06	2.11	2.14	2.0801	2.12	78031	{}	\N	2020-10-12 15:02:37.19339
46362	GMLP	2020-10-07	2.12	2.1201	2.02	2.09	224649	{}	\N	2020-10-12 15:02:37.197386
46363	GMLP	2020-10-08	2.08	2.119	2.05	2.07	111481	{}	\N	2020-10-12 15:02:37.201173
46364	GMLP	2020-10-09	2.09	2.1	2.06	2.08	547280	{}	\N	2020-10-12 15:02:37.205136
46365	GMLP	2020-10-12	2.09	2.09	2.07	2.075	120906	{}	\N	2020-10-12 15:02:37.208787
46366	GNE	2020-09-28	8.12	8.2799	8.11	8.12	29748	{}	\N	2020-10-12 15:02:38.194922
46367	GNE	2020-09-29	8.12	8.24	8.1	8.19	294651	{}	\N	2020-10-12 15:02:38.205635
46368	GNE	2020-09-30	8.2016	8.21	7.93	8	81438	{}	\N	2020-10-12 15:02:38.212559
46369	GNE	2020-10-01	8	8.14	7.96	8.03	73363	{}	\N	2020-10-12 15:02:38.218291
46370	GNE	2020-10-02	7.95	8.128	7.75	8	191087	{}	\N	2020-10-12 15:02:38.22331
46371	GNE	2020-10-05	8.01	8.25	8.01	8.14	60914	{}	\N	2020-10-12 15:02:38.22768
46372	GNE	2020-10-06	8.16	8.58	8.09	8.29	147955	{}	\N	2020-10-12 15:02:38.231893
46373	GNE	2020-10-07	8.33	8.445	8.22	8.31	66700	{}	\N	2020-10-12 15:02:38.235527
46374	GNE	2020-10-08	8.34	8.46	8.22	8.37	78095	{}	\N	2020-10-12 15:02:38.239382
46375	GNE	2020-10-09	8.34	8.57	8.34	8.43	67818	{}	\N	2020-10-12 15:02:38.24301
46376	GNE	2020-10-12	8.5	8.5	8.45	8.47	10074	{}	\N	2020-10-12 15:02:38.246531
46377	GNR	2020-09-28	37.8	37.97	37.71	37.79	110891	{}	\N	2020-10-12 15:02:39.534312
46378	GNR	2020-09-29	37.72	37.75	37.27	37.43	117901	{}	\N	2020-10-12 15:02:39.544227
46379	GNR	2020-09-30	37.47	37.78	37.38	37.55	62605	{}	\N	2020-10-12 15:02:39.550881
46380	GNR	2020-10-01	37.63	37.65	37.2	37.37	152533	{}	\N	2020-10-12 15:02:39.556899
46381	GNR	2020-10-02	36.86	37.54	36.8301	37.39	121957	{}	\N	2020-10-12 15:02:39.561875
46382	GNR	2020-10-05	37.72	38.265	37.72	38.26	58912	{}	\N	2020-10-12 15:02:39.567029
46383	GNR	2020-10-06	38.4	38.535	37.56	37.62	177891	{}	\N	2020-10-12 15:02:39.571925
46384	GNR	2020-10-07	37.9	38.226	37.89	38.17	119757	{}	\N	2020-10-12 15:02:39.575977
46385	GNR	2020-10-08	38.33	38.68	38.3	38.68	95264	{}	\N	2020-10-12 15:02:39.57988
46386	GNR	2020-10-09	39.1	39.22	38.89	39.06	361913	{}	\N	2020-10-12 15:02:39.58388
46387	GNR	2020-10-12	39.03	39.05	38.9001	38.967	12326	{}	\N	2020-10-12 15:02:39.587927
46388	GOF	2020-09-28	17.67	17.95	17.46	17.58	237577	{}	\N	2020-10-12 15:02:40.743454
46389	GOF	2020-09-29	17.61	17.65	17.5	17.61	301901	{}	\N	2020-10-12 15:02:40.754263
46390	GOF	2020-09-30	17.67	17.7	17.56	17.6	520460	{}	\N	2020-10-12 15:02:40.760471
46391	GOF	2020-10-01	17.65	17.94	17.63	17.87	369424	{}	\N	2020-10-12 15:02:40.765536
46392	GOF	2020-10-02	17.7	17.91	17.7	17.73	352443	{}	\N	2020-10-12 15:02:40.770534
46393	GOF	2020-10-05	17.8	17.9	17.75	17.86	619768	{}	\N	2020-10-12 15:02:40.775336
46394	GOF	2020-10-06	17.96	18.06	17.7518	17.98	444790	{}	\N	2020-10-12 15:02:40.779928
46395	GOF	2020-10-07	17.97	18.08	17.97	18.04	313648	{}	\N	2020-10-12 15:02:40.783855
46396	GOF	2020-10-08	18.05	18.25	18.0399	18.18	535799	{}	\N	2020-10-12 15:02:40.787771
46397	GOF	2020-10-09	18.15	18.4	18.15	18.16	335083	{}	\N	2020-10-12 15:02:40.791562
46398	GOF	2020-10-12	18.25	18.29	18.16	18.265	96602	{}	\N	2020-10-12 15:02:40.79519
46399	GOGO	2020-09-28	9.1	9.46	8.96	9.205	1327598	{}	\N	2020-10-12 15:02:42.15407
46400	GOGO	2020-09-29	9.12	10.08	8.95	9.81	3678385	{}	\N	2020-10-12 15:02:42.164015
46401	GOGO	2020-09-30	9.68	10.06	9.17	9.24	3634794	{}	\N	2020-10-12 15:02:42.170831
46402	GOGO	2020-10-01	9.14	9.94	8.9	9.6	3393000	{}	\N	2020-10-12 15:02:42.177045
46403	GOGO	2020-10-02	9.23	10.51	9.1	10.01	5413653	{}	\N	2020-10-12 15:02:42.182299
46404	GOGO	2020-10-05	10.05	10.55	9.66	10.29	3823377	{}	\N	2020-10-12 15:02:42.187394
46405	GOGO	2020-10-06	10.39	10.65	9.91	10.11	3224621	{}	\N	2020-10-12 15:02:42.191596
46406	GOGO	2020-10-07	10.31	11.15	9.95	10.07	5111925	{}	\N	2020-10-12 15:02:42.195772
46407	GOGO	2020-10-08	10.25	10.53	10.05	10.16	1993776	{}	\N	2020-10-12 15:02:42.199818
46408	GOGO	2020-10-09	10.19	11.05	10.07	11.01	3135333	{}	\N	2020-10-12 15:02:42.204161
46409	GOGO	2020-10-12	11	11.25	10.31	10.445	1399227	{}	\N	2020-10-12 15:02:42.208371
46410	GOOS	2020-09-28	29.89	30.43	29.4147	29.83	1145835	{}	\N	2020-10-12 15:02:44.206774
46411	GOOS	2020-09-29	29.89	30.25	29.37	29.68	1889015	{}	\N	2020-10-12 15:02:44.221092
46412	GOOS	2020-09-30	31.2	33.29	30.93	32.17	8986765	{}	\N	2020-10-12 15:02:44.226378
46413	GOOS	2020-10-01	32.48	33.73	32.3165	33.2	4413091	{}	\N	2020-10-12 15:02:44.231314
46414	GOOS	2020-10-02	32.09	33.2899	32.02	33.05	2557922	{}	\N	2020-10-12 15:02:44.235642
46415	GOOS	2020-10-05	33.28	33.78	32.91	33.7	1688919	{}	\N	2020-10-12 15:02:44.240081
46416	GOOS	2020-10-06	33.8	33.96	32.87	33	2268100	{}	\N	2020-10-12 15:02:44.244206
46417	GOOS	2020-10-07	33.68	34.57	33.645	34.29	2119441	{}	\N	2020-10-12 15:02:44.248104
46418	GOOS	2020-10-08	34.5	35.82	34.5	35.66	2505998	{}	\N	2020-10-12 15:02:44.252035
46419	GOOS	2020-10-09	36	36.06	35.11	35.87	1999434	{}	\N	2020-10-12 15:02:44.255884
46420	GOOS	2020-10-12	35.87	35.99	35.5	35.54	438168	{}	\N	2020-10-12 15:02:44.259438
46421	GPM	2020-09-28	5.34	5.43	5.34	5.4	420044	{}	\N	2020-10-12 15:02:46.021168
46422	GPM	2020-09-29	5.4	5.43	5.36	5.4	578356	{}	\N	2020-10-12 15:02:46.031021
46423	GPM	2020-09-30	5.41	5.5	5.4	5.45	1593550	{}	\N	2020-10-12 15:02:46.037539
46424	GPM	2020-10-01	5.48	5.5148	5.46	5.48	698523	{}	\N	2020-10-12 15:02:46.042832
46425	GPM	2020-10-02	5.35	5.5	5.35	5.5	517809	{}	\N	2020-10-12 15:02:46.047052
46426	GPM	2020-10-05	5.5	5.58	5.5	5.58	219114	{}	\N	2020-10-12 15:02:46.051105
46427	GPM	2020-10-06	5.59	5.62	5.52	5.53	316259	{}	\N	2020-10-12 15:02:46.055754
46428	GPM	2020-10-07	5.55	5.6201	5.55	5.57	500559	{}	\N	2020-10-12 15:02:46.059449
46429	GPM	2020-10-08	5.6	5.65	5.595	5.65	397243	{}	\N	2020-10-12 15:02:46.062966
46430	GPM	2020-10-09	5.65	5.7373	5.65	5.72	412658	{}	\N	2020-10-12 15:02:46.066558
46431	GPM	2020-10-12	5.72	5.725	5.7	5.7131	103990	{}	\N	2020-10-12 15:02:46.07027
46432	GJH	2020-09-30	10.19	10.2	10.17	10.17	6934	{}	\N	2020-10-12 15:02:48.217036
46433	GJH	2020-10-01	10.23	10.3032	10.23	10.3032	2000	{}	\N	2020-10-12 15:02:48.224692
46434	GJH	2020-10-02	10.4	10.4112	10.22	10.22	1280	{}	\N	2020-10-12 15:02:48.23043
46435	GJH	2020-10-05	10.22	10.342	10.15	10.17	29924	{}	\N	2020-10-12 15:02:48.235286
46436	GJH	2020-10-06	10.26	10.26	10.2349	10.25	3508	{}	\N	2020-10-12 15:02:48.240022
46437	GJH	2020-10-07	10.2925	10.36	10.21	10.32	6962	{}	\N	2020-10-12 15:02:48.244429
46438	GJH	2020-10-08	10.25	10.2701	10.25	10.2701	5960	{}	\N	2020-10-12 15:02:48.24872
46439	GLAD	2020-09-28	7.36	7.625	7.36	7.51	146748	{}	\N	2020-10-12 15:02:50.137132
46440	GLAD	2020-09-29	7.59	7.6	7.44	7.44	103518	{}	\N	2020-10-12 15:02:50.147841
46441	GLAD	2020-09-30	7.52	7.6	7.36	7.41	148034	{}	\N	2020-10-12 15:02:50.154922
46442	GLAD	2020-10-01	7.48	7.6	7.45	7.56	83850	{}	\N	2020-10-12 15:02:50.161103
46443	GLAD	2020-10-02	7.5	7.69	7.454	7.64	103195	{}	\N	2020-10-12 15:02:50.166445
46444	GLAD	2020-10-05	7.66	7.7	7.59	7.61	98449	{}	\N	2020-10-12 15:02:50.171584
46445	GLAD	2020-10-06	7.6899	7.72	7.61	7.61	74034	{}	\N	2020-10-12 15:02:50.175916
46446	GLAD	2020-10-07	7.62	7.6699	7.53	7.59	133034	{}	\N	2020-10-12 15:02:50.180125
46447	GLAD	2020-10-08	7.65	7.7	7.6061	7.645	78437	{}	\N	2020-10-12 15:02:50.184204
46448	GLAD	2020-10-09	7.6612	7.6925	7.63	7.66	70032	{}	\N	2020-10-12 15:02:50.188587
46449	GLAD	2020-10-12	7.68	7.6977	7.6201	7.66	56443	{}	\N	2020-10-12 15:02:50.192876
46450	GLL	2020-09-28	32.7	32.9754	32.301	32.42	109815	{}	\N	2020-10-12 15:02:51.07909
46451	GLL	2020-09-29	32.16	32.23	31.75	31.81	81235	{}	\N	2020-10-12 15:02:51.087401
46452	GLL	2020-09-30	31.96	32.25	31.6406	32.19	75474	{}	\N	2020-10-12 15:02:51.093545
46453	GLL	2020-10-01	31.62	31.8025	31.291	31.5467	328828	{}	\N	2020-10-12 15:02:51.099351
46454	GLL	2020-10-02	31.59	31.76	31.39	31.64	58082	{}	\N	2020-10-12 15:02:51.104589
46455	GLL	2020-10-05	31.49	31.49	31.06	31.3	50609	{}	\N	2020-10-12 15:02:51.11006
46456	GLL	2020-10-06	31.05	32.17	30.99	32.17	70446	{}	\N	2020-10-12 15:02:51.114209
46457	GLL	2020-10-07	32.22	32.46	32.109	32.19	88596	{}	\N	2020-10-12 15:02:51.118518
46458	GLL	2020-10-08	31.78	32.31	31.68	31.9	26581	{}	\N	2020-10-12 15:02:51.122669
46459	GLL	2020-10-09	31.02	31.0906	30.6556	30.68	101607	{}	\N	2020-10-12 15:02:51.126946
46460	GLL	2020-10-12	30.87	30.96	30.79	30.91	35349	{}	\N	2020-10-12 15:02:51.131478
46461	GLU	2020-09-28	15.68	15.7602	15.68	15.74	11508	{}	\N	2020-10-12 15:02:52.502692
46462	GLU	2020-09-29	15.72	15.78	15.69	15.7	6373	{}	\N	2020-10-12 15:02:52.509802
46463	GLU	2020-09-30	15.72	16.19	15.6901	16.1	13650	{}	\N	2020-10-12 15:02:52.51495
46464	GLU	2020-10-01	16.14	16.2	16.08	16.1764	8054	{}	\N	2020-10-12 15:02:52.51957
46465	GLU	2020-10-02	15.9104	16.125	15.86	15.998	4288	{}	\N	2020-10-12 15:02:52.5238
46466	GLU	2020-10-05	16.12	16.125	16	16.06	13345	{}	\N	2020-10-12 15:02:52.528115
46467	GLU	2020-10-06	16.232	16.38	16.15	16.26	14445	{}	\N	2020-10-12 15:02:52.531882
46468	GLU	2020-10-07	16.3383	16.38	16.24	16.26	14321	{}	\N	2020-10-12 15:02:52.535494
46469	GLU	2020-10-08	16.25	16.38	16.25	16.3	7328	{}	\N	2020-10-12 15:02:52.538917
46470	GLU	2020-10-09	16.26	16.39	16.26	16.32	6369	{}	\N	2020-10-12 15:02:52.54239
46471	GLU	2020-10-12	16.39	16.39	16.32	16.36	6724	{}	\N	2020-10-12 15:02:52.546124
46472	GMF	2020-09-28	109.65	109.74	109.075	109.48	32314	{}	\N	2020-10-12 15:02:54.626598
46473	GMF	2020-09-29	109.36	109.8371	109.2	109.725	13584	{}	\N	2020-10-12 15:02:54.638034
46474	GMF	2020-09-30	110.46	111.6299	110.46	111.39	32972	{}	\N	2020-10-12 15:02:54.644203
46475	GMF	2020-10-01	111.93	112.6871	111.93	112.53	10284	{}	\N	2020-10-12 15:02:54.649473
46476	GMF	2020-10-02	111.34	112.05	111.1601	111.37	19658	{}	\N	2020-10-12 15:02:54.654356
46477	GMF	2020-10-05	111.59	112.41	111.59	112.34	60349	{}	\N	2020-10-12 15:02:54.658919
46478	GMF	2020-10-06	112.92	113.5999	112.73	112.94	41262	{}	\N	2020-10-12 15:02:54.662554
46479	GMF	2020-10-07	114.02	114.46	113.79	114.19	29016	{}	\N	2020-10-12 15:02:54.666602
46480	GMF	2020-10-08	114.69	115.0068	114.49	114.79	16939	{}	\N	2020-10-12 15:02:54.670224
46481	GMF	2020-10-09	114.97	115.8171	114.97	115.6852	9632	{}	\N	2020-10-12 15:02:54.673674
46482	GMF	2020-10-12	116.365	116.57	116.36	116.57	8927	{}	\N	2020-10-12 15:02:54.677383
46483	GNMK	2020-09-28	13.5	13.94	13.19	13.65	676328	{}	\N	2020-10-12 15:02:56.059696
46484	GNMK	2020-09-29	14.0435	14.3	13.42	14	567240	{}	\N	2020-10-12 15:02:56.071309
46485	GNMK	2020-09-30	14.02	14.38	13.93	14.2	506966	{}	\N	2020-10-12 15:02:56.078593
46486	GNMK	2020-10-01	14.46	14.46	13.9	14.09	706829	{}	\N	2020-10-12 15:02:56.085226
46487	GNMK	2020-10-02	13.81	14.2	13.32	14	836936	{}	\N	2020-10-12 15:02:56.090705
46488	GNMK	2020-10-05	13.02	13.93	13.01	13.75	907120	{}	\N	2020-10-12 15:02:56.095851
46489	GNMK	2020-10-06	13.93	14.58	13.83	14.14	548064	{}	\N	2020-10-12 15:02:56.100596
46490	GNMK	2020-10-07	14.21	15.15	14.21	15.11	1228930	{}	\N	2020-10-12 15:02:56.104969
46491	GNMK	2020-10-08	15.278	15.349	14	14.3	1143058	{}	\N	2020-10-12 15:02:56.108981
46492	GNMK	2020-10-09	15.71	15.75	13.6	14.11	4368931	{}	\N	2020-10-12 15:02:56.11303
46493	GNMK	2020-10-12	14.19	14.3	12.77	12.85	770840	{}	\N	2020-10-12 15:02:56.116897
46494	GNW	2020-09-28	3.11	3.305	3.07	3.27	5534342	{}	\N	2020-10-12 15:02:57.42616
46495	GNW	2020-09-29	3.24	3.53	3.08	3.5	12852739	{}	\N	2020-10-12 15:02:57.437146
46496	GNW	2020-09-30	3.41	3.46	3.3	3.35	8354776	{}	\N	2020-10-12 15:02:57.443869
46497	GNW	2020-10-01	2.95	3.25	2.95	3.16	16780233	{}	\N	2020-10-12 15:02:57.450013
46498	GNW	2020-10-02	3.09	3.465	3.09	3.41	10310904	{}	\N	2020-10-12 15:02:57.455223
46499	GNW	2020-10-05	3.43	3.545	3.43	3.47	5508667	{}	\N	2020-10-12 15:02:57.46009
46500	GNW	2020-10-06	3.5	3.53	3.4	3.44	5932458	{}	\N	2020-10-12 15:02:57.464455
46501	GNW	2020-10-07	3.5	3.53	3.4	3.4	6250260	{}	\N	2020-10-12 15:02:57.468924
46502	GNW	2020-10-08	3.46	3.49	3.405	3.45	3290304	{}	\N	2020-10-12 15:02:57.473181
46503	GNW	2020-10-09	3.52	3.52	3.39	3.43	6972118	{}	\N	2020-10-12 15:02:57.477429
46504	GNW	2020-10-12	3.42	3.58	3.415	3.545	1558057	{}	\N	2020-10-12 15:02:57.481654
46505	GOGL	2020-09-28	3.61	3.64	3.58	3.61	189509	{}	\N	2020-10-12 15:02:59.199759
46506	GOGL	2020-09-29	3.67	3.75	3.665	3.74	259958	{}	\N	2020-10-12 15:02:59.208461
46507	GOGL	2020-09-30	3.71	3.84	3.7	3.82	222459	{}	\N	2020-10-12 15:02:59.214222
46508	GOGL	2020-10-01	3.84	3.92	3.83	3.9	247036	{}	\N	2020-10-12 15:02:59.21986
46509	GOGL	2020-10-02	4	4.12	4	4.08	336821	{}	\N	2020-10-12 15:02:59.224882
46510	GOGL	2020-10-05	4.15	4.2	4.1	4.12	826400	{}	\N	2020-10-12 15:02:59.22923
46511	GOGL	2020-10-06	4.12	4.12	3.98	4.04	849792	{}	\N	2020-10-12 15:02:59.233556
46512	GOGL	2020-10-07	3.98	4.01	3.94	3.99	332086	{}	\N	2020-10-12 15:02:59.237971
46513	GOGL	2020-10-08	3.99	4.04	3.9721	4.01	289434	{}	\N	2020-10-12 15:02:59.241935
46514	GOGL	2020-10-09	3.93	4.02	3.9	3.98	427456	{}	\N	2020-10-12 15:02:59.246207
46515	GOGL	2020-10-12	3.94	3.94	3.89	3.89	86720	{}	\N	2020-10-12 15:02:59.250702
46516	GOOG	2020-09-28	1474.21	1476.8	1449.3012	1464.52	1874329	{}	\N	2020-10-12 15:03:00.145403
46517	GOOG	2020-09-29	1466.71	1476.6632	1458.805	1469.33	914903	{}	\N	2020-10-12 15:03:00.152999
46518	GOOG	2020-09-30	1466.28	1489.75	1459.88	1469.6	1552561	{}	\N	2020-10-12 15:03:00.158203
46519	GOOG	2020-10-01	1484.27	1499.04	1479.21	1490.09	1691200	{}	\N	2020-10-12 15:03:00.162937
46520	GOOG	2020-10-02	1462.03	1483.2	1450.92	1458.42	1157095	{}	\N	2020-10-12 15:03:00.167561
46521	GOOG	2020-10-05	1466.21	1488.21	1464.27	1486.02	1081302	{}	\N	2020-10-12 15:03:00.171847
46522	GOOG	2020-10-06	1475.58	1486.76	1448.5901	1453.44	1122714	{}	\N	2020-10-12 15:03:00.175931
46523	GOOG	2020-10-07	1464.29	1468.96	1436	1460.29	1693161	{}	\N	2020-10-12 15:03:00.179891
46524	GOOG	2020-10-08	1465.09	1490	1465.09	1485.93	1143070	{}	\N	2020-10-12 15:03:00.183855
46525	GOOG	2020-10-09	1494.7	1516.52	1489.45	1515.22	1338109	{}	\N	2020-10-12 15:03:00.187834
46526	GOOG	2020-10-12	1543	1551.75	1532.57	1548.31	564930	{}	\N	2020-10-12 15:03:00.191618
46527	GPI	2020-09-28	86.25	90.07	86.25	89.36	156047	{}	\N	2020-10-12 15:03:02.785108
46528	GPI	2020-09-29	89.15	89.15	84.84	86.92	244219	{}	\N	2020-10-12 15:03:02.795168
46529	GPI	2020-09-30	87.11	89.07	87.11	88.39	242052	{}	\N	2020-10-12 15:03:02.802557
46530	GPI	2020-10-01	89.19	93.03	86.85	92.76	522299	{}	\N	2020-10-12 15:03:02.808488
46531	GPI	2020-10-02	89.38	97.72	89.38	97.33	308084	{}	\N	2020-10-12 15:03:02.813133
46532	GPI	2020-10-05	98.74	103.18	98.3	102	460731	{}	\N	2020-10-12 15:03:02.817731
46533	GPI	2020-10-06	112	123.2	111	118.62	2092954	{}	\N	2020-10-12 15:03:02.821736
46534	GPI	2020-10-07	121.51	128.06	120.5	124.21	1037813	{}	\N	2020-10-12 15:03:02.825591
46535	GPI	2020-10-08	126.41	127.9	122.56	127.32	583695	{}	\N	2020-10-12 15:03:02.829564
46536	GPI	2020-10-09	128.46	128.978	123.765	124.62	428875	{}	\N	2020-10-12 15:03:02.83321
46537	GPI	2020-10-12	125.08	127.06	123.8158	126.15	62512	{}	\N	2020-10-12 15:03:02.836928
46538	GPK	2020-09-28	13.99	14.1	13.86	13.89	3292076	{}	\N	2020-10-12 15:03:03.776388
46539	GPK	2020-09-29	13.91	13.965	13.725	13.8	2637498	{}	\N	2020-10-12 15:03:03.787373
46540	GPK	2020-09-30	13.8	14.14	13.8	14.09	7230456	{}	\N	2020-10-12 15:03:03.794537
46541	GPK	2020-10-01	14.16	14.31	13.97	14.02	4859278	{}	\N	2020-10-12 15:03:03.800763
46542	GPK	2020-10-02	13.71	14.15	13.71	14.03	5067338	{}	\N	2020-10-12 15:03:03.805724
46543	GPK	2020-10-05	14.16	14.475	14.1	14.4	3408556	{}	\N	2020-10-12 15:03:03.810475
46544	GPK	2020-10-06	14.46	14.51	14.175	14.19	4625323	{}	\N	2020-10-12 15:03:03.814839
46545	GPK	2020-10-07	14.41	14.715	14.41	14.52	4444101	{}	\N	2020-10-12 15:03:03.819234
46546	GPK	2020-10-08	14.57	14.66	14.39	14.54	4179377	{}	\N	2020-10-12 15:03:03.823961
46547	GPK	2020-10-09	14.7	14.7	14.38	14.42	2863222	{}	\N	2020-10-12 15:03:03.828232
46548	GPK	2020-10-12	14.47	14.59	14.45	14.505	696097	{}	\N	2020-10-12 15:03:03.83219
46549	GLDI	2020-09-28	9.75	9.788	9.6932	9.77	33034	{}	\N	2020-10-12 15:03:06.844996
46550	GLDI	2020-09-29	9.79	9.88	9.7887	9.86	46806	{}	\N	2020-10-12 15:03:06.855999
46551	GLDI	2020-09-30	9.86	9.8701	9.8	9.825	33904	{}	\N	2020-10-12 15:03:06.862732
46552	GLDI	2020-10-01	9.8541	9.9699	9.8541	9.9031	24509	{}	\N	2020-10-12 15:03:06.868771
46553	GLDI	2020-10-02	9.895	9.928	9.88	9.89	36445	{}	\N	2020-10-12 15:03:06.874207
46554	GLDI	2020-10-05	9.93	9.97	9.9293	9.94	53399	{}	\N	2020-10-12 15:03:06.878978
46555	GLDI	2020-10-06	9.975	9.975	9.83	9.83	54114	{}	\N	2020-10-12 15:03:06.883266
46556	GLDI	2020-10-07	9.86	9.86	9.81	9.83	37997	{}	\N	2020-10-12 15:03:06.887694
46557	GLDI	2020-10-08	9.87	9.88	9.8	9.845	77544	{}	\N	2020-10-12 15:03:06.893605
46558	GLDI	2020-10-09	9.94	10	9.9	9.995	29359	{}	\N	2020-10-12 15:03:06.897895
46559	GLDI	2020-10-12	10.05	10.05	10.02	10.04	129030	{}	\N	2020-10-12 15:03:06.902127
46560	GLPG	2020-09-28	145	145	140.54	141.8	200134	{}	\N	2020-10-12 15:03:09.145983
46561	GLPG	2020-09-29	143	144.06	141.18	141.75	133256	{}	\N	2020-10-12 15:03:09.15642
46562	GLPG	2020-09-30	140.52	143.11	140.35	142.225	79727	{}	\N	2020-10-12 15:03:09.164274
46563	GLPG	2020-10-01	141.88	142.73	141.31	142.16	87022	{}	\N	2020-10-12 15:03:09.170347
46564	GLPG	2020-10-02	139.77	143.6	139.71	142.02	139114	{}	\N	2020-10-12 15:03:09.17599
46565	GLPG	2020-10-05	142.34	147.47	142.28	147.05	141912	{}	\N	2020-10-12 15:03:09.180792
46566	GLPG	2020-10-06	144.48	146.02	142	142.5	117851	{}	\N	2020-10-12 15:03:09.185477
46567	GLPG	2020-10-07	138.51	139.5	137.32	138.81	118727	{}	\N	2020-10-12 15:03:09.189889
46568	GLPG	2020-10-08	142.29	142.92	140.92	141.25	119829	{}	\N	2020-10-12 15:03:09.194482
46569	GLPG	2020-10-09	144.44	146.995	144.02	146.59	112054	{}	\N	2020-10-12 15:03:09.198881
46570	GLPG	2020-10-12	146.62	148.59	146.4	147.7911	53848	{}	\N	2020-10-12 15:03:09.203403
46571	GMED	2020-09-28	49.79	50	49.375	49.67	313205	{}	\N	2020-10-12 15:03:10.157975
46572	GMED	2020-09-29	50.05	50.1916	48.38	48.9	954319	{}	\N	2020-10-12 15:03:10.167172
46573	GMED	2020-09-30	49.1	50.4	48.82	49.52	1169938	{}	\N	2020-10-12 15:03:10.173222
46574	GMED	2020-10-01	49.73	49.9199	48.58	49.06	825098	{}	\N	2020-10-12 15:03:10.179097
46575	GMED	2020-10-02	48.16	49.4	48	48.89	774422	{}	\N	2020-10-12 15:03:10.184236
46576	GMED	2020-10-05	49.23	50.29	49.2	49.7	1136873	{}	\N	2020-10-12 15:03:10.188853
46577	GMED	2020-10-06	49.96	51.92	49.84	51.02	1420361	{}	\N	2020-10-12 15:03:10.192968
46578	GMED	2020-10-07	51.65	52.58	51.22	52.2	1104365	{}	\N	2020-10-12 15:03:10.19697
46579	GMED	2020-10-08	52.74	53.75	52.6485	53.55	1159019	{}	\N	2020-10-12 15:03:10.200857
46580	GMED	2020-10-09	54.17	54.34	53.53	53.56	2273049	{}	\N	2020-10-12 15:03:10.204777
46581	GMED	2020-10-12	53.98	54.99	53.36	54.85	265462	{}	\N	2020-10-12 15:03:10.208749
46582	GNRC	2020-09-28	186.88	191.43	186.11	190.18	764392	{}	\N	2020-10-12 15:03:12.038277
46583	GNRC	2020-09-29	190.98	192.28	187.73	188.14	1217223	{}	\N	2020-10-12 15:03:12.048702
46584	GNRC	2020-09-30	187.76	194.17	187.76	193.64	1775435	{}	\N	2020-10-12 15:03:12.055174
46585	GNRC	2020-10-01	195.5	196.9492	191.74	196.16	1427306	{}	\N	2020-10-12 15:03:12.060253
46586	GNRC	2020-10-02	191.27	198.77	190.1	197.75	1212014	{}	\N	2020-10-12 15:03:12.065587
46587	GNRC	2020-10-05	199.25	208.26	198.53	207.86	1622697	{}	\N	2020-10-12 15:03:12.069936
46588	GNRC	2020-10-06	211.05	213.23	204.885	207.37	2325391	{}	\N	2020-10-12 15:03:12.074434
46589	GNRC	2020-10-07	212.55	213.91	205.16	207.04	1472028	{}	\N	2020-10-12 15:03:12.078663
46590	GNRC	2020-10-08	209.99	210.58	203.62	204.5	1515991	{}	\N	2020-10-12 15:03:12.082698
46591	GNRC	2020-10-09	205.16	208.74	205.16	205.53	1382000	{}	\N	2020-10-12 15:03:12.086559
46592	GNRC	2020-10-12	206.65	207.75	200.38	202.89	380602	{}	\N	2020-10-12 15:03:12.090865
46593	GOEX	2020-09-28	35.11	35.11	34.55	34.79	6958	{}	\N	2020-10-12 15:03:13.15096
46594	GOEX	2020-09-29	35.02	35.57	35.02	35.53	8646	{}	\N	2020-10-12 15:03:13.160533
46595	GOEX	2020-09-30	35.23	35.5193	34.91	35.05	10720	{}	\N	2020-10-12 15:03:13.167041
46596	GOEX	2020-10-01	35.405	35.9	35.405	35.51	2846	{}	\N	2020-10-12 15:03:13.172233
46597	GOEX	2020-10-02	35.05	35.085	34.713	34.73	11794	{}	\N	2020-10-12 15:03:13.17696
46598	GOEX	2020-10-05	35.22	35.65	35.1581	35.61	18208	{}	\N	2020-10-12 15:03:13.181681
46599	GOEX	2020-10-06	35.98	36.2899	34.55	34.56	30826	{}	\N	2020-10-12 15:03:13.186477
46600	GOEX	2020-10-07	35.08	35.3217	34.82	34.82	6373	{}	\N	2020-10-12 15:03:13.190293
46601	GOEX	2020-10-08	35.5099	35.6999	35.208	35.33	4335	{}	\N	2020-10-12 15:03:13.194085
46602	GOEX	2020-10-09	36.44	37.5	36.44	37.5	18747	{}	\N	2020-10-12 15:03:13.198173
46603	GOEX	2020-10-12	37.43	37.43	37.16	37.2	3103	{}	\N	2020-10-12 15:03:13.20215
46604	GOOGL	2020-09-28	1471.99	1472.99	1446.28	1458.66	1406609	{}	\N	2020-10-12 15:03:14.408987
46605	GOOGL	2020-09-29	1463.91	1472	1454	1466.02	1520705	{}	\N	2020-10-12 15:03:14.418099
46606	GOOGL	2020-09-30	1461.08	1486.1975	1455.755	1465.6	1860114	{}	\N	2020-10-12 15:03:14.422968
46607	GOOGL	2020-10-01	1483.17	1495.07	1476.8	1487.9	1537181	{}	\N	2020-10-12 15:03:14.428288
46608	GOOGL	2020-10-02	1459.63	1479.03	1447.35	1455.6	1422580	{}	\N	2020-10-12 15:03:14.433004
46609	GOOGL	2020-10-05	1462.65	1484.7	1460.09	1482.83	1056949	{}	\N	2020-10-12 15:03:14.437431
46610	GOOGL	2020-10-06	1475.5	1484.52	1445.01	1451.02	1209012	{}	\N	2020-10-12 15:03:14.441478
46611	GOOGL	2020-10-07	1459.31	1468.97	1433.23	1459.14	1767082	{}	\N	2020-10-12 15:03:14.445273
46612	GOOGL	2020-10-08	1465.45	1488.0275	1465.45	1483.43	1261642	{}	\N	2020-10-12 15:03:14.449233
46613	GOOGL	2020-10-09	1492.86	1511.85	1487	1510.45	1553079	{}	\N	2020-10-12 15:03:14.453325
46614	GOOGL	2020-10-12	1538	1548.37	1529.3	1545.21	606634	{}	\N	2020-10-12 15:03:14.457412
46615	GPC	2020-09-28	95.32	96.33	94.83	95.74	685503	{}	\N	2020-10-12 15:03:15.683221
46616	GPC	2020-09-29	95.94	95.94	94.61	94.74	1006864	{}	\N	2020-10-12 15:03:15.696644
46617	GPC	2020-09-30	95.15	96	94.28	95.17	1526224	{}	\N	2020-10-12 15:03:15.703882
46618	GPC	2020-10-01	95.69	96.61	94.22	94.94	1306760	{}	\N	2020-10-12 15:03:15.70989
46619	GPC	2020-10-02	93.5	96.19	93.5	95.85	1107084	{}	\N	2020-10-12 15:03:15.715149
46620	GPC	2020-10-05	96.93	97.828	96.3	97.4	1445130	{}	\N	2020-10-12 15:03:15.720055
46621	GPC	2020-10-06	97.53	99.12	96.2	96.52	984421	{}	\N	2020-10-12 15:03:15.724285
46622	GPC	2020-10-07	97.58	99.43	97.58	98.75	976698	{}	\N	2020-10-12 15:03:15.728696
46623	GPC	2020-10-08	99.15	101.2	98.8028	101.15	1301556	{}	\N	2020-10-12 15:03:15.732815
46624	GPC	2020-10-09	101.58	102.32	100.3	100.39	1264235	{}	\N	2020-10-12 15:03:15.737228
46625	GPC	2020-10-12	100.28	100.99	99.89	100.3	129628	{}	\N	2020-10-12 15:03:15.741705
46626	GPN	2020-09-28	177.47	180.1	176.805	178.56	1516860	{}	\N	2020-10-12 15:03:17.422604
46627	GPN	2020-09-29	179.18	179.469	177.16	177.5	1848074	{}	\N	2020-10-12 15:03:17.432653
46628	GPN	2020-09-30	178.02	180.65	175.55	177.58	2718480	{}	\N	2020-10-12 15:03:17.439302
46629	GPN	2020-10-01	179.11	182.6	176.71	179.24	2135518	{}	\N	2020-10-12 15:03:17.444755
46630	GPN	2020-10-02	175.93	180.15	175.2	178.43	2427177	{}	\N	2020-10-12 15:03:17.449573
46631	GPN	2020-10-05	181.29	181.29	174.82	176.52	2460132	{}	\N	2020-10-12 15:03:17.454577
46632	GPN	2020-10-06	176.9	180.21	174.52	174.92	1931700	{}	\N	2020-10-12 15:03:17.458512
46633	GPN	2020-10-07	176.65	177.67	175.52	176.81	1547646	{}	\N	2020-10-12 15:03:17.462759
46634	GPN	2020-10-08	177.46	181.69	177.45	181.55	1998808	{}	\N	2020-10-12 15:03:17.466782
46635	GPN	2020-10-09	182.76	184.86	182.255	183.78	2041007	{}	\N	2020-10-12 15:03:17.470586
46636	GPN	2020-10-12	184.48	184.935	182.03	182.33	1042646	{}	\N	2020-10-12 15:03:17.474511
46637	GPRE	2020-09-28	14.43	15.22	14.38	15.03	361476	{}	\N	2020-10-12 15:03:20.923534
46638	GPRE	2020-09-29	15	15.71	14.865	15.59	355369	{}	\N	2020-10-12 15:03:20.932541
46639	GPRE	2020-09-30	15.66	15.96	15.26	15.48	333376	{}	\N	2020-10-12 15:03:20.937755
46640	GPRE	2020-10-01	15.51	15.55	15.01	15.31	357074	{}	\N	2020-10-12 15:03:20.942598
46641	GPRE	2020-10-02	14.93	15.9	14.57	15.76	289906	{}	\N	2020-10-12 15:03:20.947209
46642	GPRE	2020-10-05	16.03	16.3899	15.9	16.02	478184	{}	\N	2020-10-12 15:03:20.951663
46643	GPRE	2020-10-06	16.15	16.24	15.75	15.75	343811	{}	\N	2020-10-12 15:03:20.955447
46644	GPRE	2020-10-07	16	16.7	15.9	16.32	384221	{}	\N	2020-10-12 15:03:20.959269
46645	GPRE	2020-10-08	16.29	16.59	16	16.21	217610	{}	\N	2020-10-12 15:03:20.962971
46646	GPRE	2020-10-09	16.31	16.52	15.89	16.285	240538	{}	\N	2020-10-12 15:03:20.966866
46647	GPRE	2020-10-12	16.38	16.485	15.92	15.955	37003	{}	\N	2020-10-12 15:03:20.970832
46648	GPS	2020-09-28	16.72	17.45	16.64	17.2	9018994	{}	\N	2020-10-12 15:03:23.697347
46649	GPS	2020-09-29	17.19	17.19	16.43	16.71	14019641	{}	\N	2020-10-12 15:03:23.707337
46650	GPS	2020-09-30	16.89	17.45	16.86	17.03	18286363	{}	\N	2020-10-12 15:03:23.713719
46651	GPS	2020-10-01	17.17	17.99	17.12	17.87	15925155	{}	\N	2020-10-12 15:03:23.719139
46652	GPS	2020-10-02	17.31	18.47	17.23	18.35	16406084	{}	\N	2020-10-12 15:03:23.724138
46653	GPS	2020-10-05	18.55	19.36	18.55	19.23	17705414	{}	\N	2020-10-12 15:03:23.728994
46654	GPS	2020-10-06	19.52	19.69	18.57	18.63	16322725	{}	\N	2020-10-12 15:03:23.733509
46655	GPS	2020-10-07	19.3	19.875	19.21	19.78	21217596	{}	\N	2020-10-12 15:03:23.737718
46656	GPS	2020-10-08	20	20.07	19.24	19.83	22955731	{}	\N	2020-10-12 15:03:23.74177
46657	GPS	2020-10-09	20	20.055	19.31	19.81	26305054	{}	\N	2020-10-12 15:03:23.745808
46658	GPS	2020-10-12	19.88	20.34	19.72	19.73	6649386	{}	\N	2020-10-12 15:03:23.7501
46659	GJS	2020-09-29	19.631	19.7	19.631	19.677	3200	{}	\N	2020-10-12 15:03:26.191361
46660	GJS	2020-09-30	19.65	19.66	19.65	19.66	2200	{}	\N	2020-10-12 15:03:26.199543
46661	GJS	2020-10-01	19.7	19.7	19.47	19.47	2144	{}	\N	2020-10-12 15:03:26.205011
46662	GJS	2020-10-02	19.47	20.08	19.47	20.08	2800	{}	\N	2020-10-12 15:03:26.209506
46663	GJS	2020-10-05	19.7417	19.7417	19.7417	19.7417	2000	{}	\N	2020-10-12 15:03:26.213911
46664	GJS	2020-10-06	19.72	19.72	19.72	19.72	1200	{}	\N	2020-10-12 15:03:26.218294
46665	GJS	2020-10-08	19.5	19.5	19.5	19.5	934	{}	\N	2020-10-12 15:03:26.222193
46666	GLBZ	2020-09-28	9.84	9.84	9.84	9.84	219	{}	\N	2020-10-12 15:03:27.273745
46667	GLBZ	2020-09-30	10.3	10.3	10.27	10.27	2808	{}	\N	2020-10-12 15:03:27.283652
46668	GLBZ	2020-10-01	10.28	10.3	10	10.15	1695	{}	\N	2020-10-12 15:03:27.290183
46669	GLBZ	2020-10-02	10.67	10.67	10.67	10.67	496	{}	\N	2020-10-12 15:03:27.296022
46670	GLBZ	2020-10-05	10.1221	10.1221	10.1221	10.1221	1630	{}	\N	2020-10-12 15:03:27.301328
46671	GLBZ	2020-10-07	9.98	10	9.98	10	493	{}	\N	2020-10-12 15:03:27.306202
46672	GLBZ	2020-10-09	10.03	10.3188	10.03	10.3188	6393	{}	\N	2020-10-12 15:03:27.310324
46673	GLBZ	2020-10-12	10.271	10.271	10.271	10.271	311	{}	\N	2020-10-12 15:03:27.314722
46674	GLNG	2020-09-28	6.97	7	6.01	6.06	5875379	{}	\N	2020-10-12 15:03:28.272651
46675	GLNG	2020-09-29	6.15	6.49	5.665	6.01	4937195	{}	\N	2020-10-12 15:03:28.284108
46676	GLNG	2020-09-30	5.91	6.265	5.67	6.055	3315553	{}	\N	2020-10-12 15:03:28.291804
46677	GLNG	2020-10-01	6.3	7.06	6.3	6.94	4797344	{}	\N	2020-10-12 15:03:28.298315
46678	GLNG	2020-10-02	6.38	7.2	6.31	7.15	2463306	{}	\N	2020-10-12 15:03:28.304287
46679	GLNG	2020-10-05	7.11	7.66	7.09	7.6	1927800	{}	\N	2020-10-12 15:03:28.309302
46680	GLNG	2020-10-06	7.79	7.88	7.07	7.185	1881697	{}	\N	2020-10-12 15:03:28.313909
46681	GLNG	2020-10-07	7.15	7.59	7.13	7.52	1291005	{}	\N	2020-10-12 15:03:28.318647
46682	GLNG	2020-10-08	7.67	7.9623	7.61	7.79	1178583	{}	\N	2020-10-12 15:03:28.323377
46683	GLNG	2020-10-09	7.86	8.05	7.601	7.64	957691	{}	\N	2020-10-12 15:03:28.327985
46684	GLNG	2020-10-12	7.2	7.51	6.88	7.46	999144	{}	\N	2020-10-12 15:03:28.332443
46685	GLPI	2020-09-28	38.78	38.89	37.92	38.02	800998	{}	\N	2020-10-12 15:03:29.44878
46686	GLPI	2020-09-29	37.79	37.79	36.48	36.9	768935	{}	\N	2020-10-12 15:03:29.458196
46687	GLPI	2020-09-30	37.22	38.16	36.75	36.93	1297177	{}	\N	2020-10-12 15:03:29.464475
46688	GLPI	2020-10-01	37.05	38.19	36.77	38.18	801868	{}	\N	2020-10-12 15:03:29.470532
46689	GLPI	2020-10-02	37.26	38.25	37.01	37.81	1182563	{}	\N	2020-10-12 15:03:29.475787
46690	GLPI	2020-10-05	38.13	38.45	36.91	37.52	805394	{}	\N	2020-10-12 15:03:29.48072
46691	GLPI	2020-10-06	37.77	37.87	36.73	37.375	957228	{}	\N	2020-10-12 15:03:29.485243
46692	GLPI	2020-10-07	37.83	38.01	37.38	37.53	784902	{}	\N	2020-10-12 15:03:29.489678
46693	GLPI	2020-10-08	37.7	38.53	37.59	38.29	632015	{}	\N	2020-10-12 15:03:29.494042
46694	GLPI	2020-10-09	38.71	38.71	37.65	38.28	550836	{}	\N	2020-10-12 15:03:29.498353
46695	GLPI	2020-10-12	38.62	38.62	37.78	37.96	104233	{}	\N	2020-10-12 15:03:29.502672
46696	GM	2020-09-28	29.82	30.04	29.38	29.44	8555761	{}	\N	2020-10-12 15:03:33.218913
46697	GM	2020-09-29	29.3	29.4	28.5	28.74	13875462	{}	\N	2020-10-12 15:03:33.229664
46698	GM	2020-09-30	28.89	29.82	28.89	29.59	22762644	{}	\N	2020-10-12 15:03:33.235575
46699	GM	2020-10-01	29.52	30.51	29.15	30.38	29544276	{}	\N	2020-10-12 15:03:33.240803
46700	GM	2020-10-02	29.65	30.9	29.59	30.46	17755198	{}	\N	2020-10-12 15:03:33.245494
46701	GM	2020-10-05	30.85	31.29	30.8	30.96	16573458	{}	\N	2020-10-12 15:03:33.25038
46702	GM	2020-10-06	31.27	31.45	30.3	30.4	19441997	{}	\N	2020-10-12 15:03:33.254274
46703	GM	2020-10-07	30.85	31.76	30.81	31.62	22732512	{}	\N	2020-10-12 15:03:33.258322
46704	GM	2020-10-08	31.9	32.635	31.795	32.21	34270859	{}	\N	2020-10-12 15:03:33.262462
46705	GM	2020-10-09	32.3	32.68	32.015	32.16	18411817	{}	\N	2020-10-12 15:03:33.266511
46706	GM	2020-10-12	32.36	32.58	32.1217	32.46	4605101	{}	\N	2020-10-12 15:03:33.270566
46707	GMS	2020-09-28	22.94	24.14	22.94	23.92	266698	{}	\N	2020-10-12 15:03:35.254256
46708	GMS	2020-09-29	23.92	24.03	23.5	23.92	456849	{}	\N	2020-10-12 15:03:35.266814
46709	GMS	2020-09-30	24.07	24.61	23.9	24.1	525740	{}	\N	2020-10-12 15:03:35.273725
46710	GMS	2020-10-01	24.35	24.79	23.78	24.62	445258	{}	\N	2020-10-12 15:03:35.279592
46711	GMS	2020-10-02	24.05	25.075	23.92	24.86	614454	{}	\N	2020-10-12 15:03:35.284288
46712	GMS	2020-10-05	25.24	25.775	25.09	25.43	564796	{}	\N	2020-10-12 15:03:35.288931
46713	GMS	2020-10-06	25.85	26.73	25.85	25.92	824432	{}	\N	2020-10-12 15:03:35.293037
46714	GMS	2020-10-07	26.28	26.77	26.18	26.5	625745	{}	\N	2020-10-12 15:03:35.296985
46715	GMS	2020-10-08	26.91	26.96	26.45	26.8	395660	{}	\N	2020-10-12 15:03:35.300679
46716	GMS	2020-10-09	27.16	27.23	26.49	26.78	330055	{}	\N	2020-10-12 15:03:35.304291
46717	GMS	2020-10-12	26.79	27.35	26.64	27.19	39611	{}	\N	2020-10-12 15:03:35.307872
46718	GNL	2020-09-28	15.64	16.47	15.605	16.22	439602	{}	\N	2020-10-12 15:03:36.626018
46719	GNL	2020-09-29	16.21	16.21	15.72	15.84	609962	{}	\N	2020-10-12 15:03:36.63976
46720	GNL	2020-09-30	15.81	16.22	15.69	15.9	972488	{}	\N	2020-10-12 15:03:36.647401
46721	GNL	2020-10-01	15.98	16.47	15.96	16.45	948793	{}	\N	2020-10-12 15:03:36.653117
46722	GNL	2020-10-02	16.19	16.87	16.09	16.7	701884	{}	\N	2020-10-12 15:03:36.658081
46723	GNL	2020-10-05	16.8	16.9	16.63	16.71	657814	{}	\N	2020-10-12 15:03:36.662715
46724	GNL	2020-10-06	16.85	17.15	16.6	16.73	1276085	{}	\N	2020-10-12 15:03:36.667614
46725	GNL	2020-10-07	16.82	16.92	16.63	16.68	1142414	{}	\N	2020-10-12 15:03:36.671826
46726	GNL	2020-10-08	16.32	16.67	16.24	16.61	650553	{}	\N	2020-10-12 15:03:36.675633
46727	GNL	2020-10-09	16.78	16.8159	16.2	16.28	612586	{}	\N	2020-10-12 15:03:36.679447
46728	GNL	2020-10-12	16.21	16.32	16.15	16.27	74752	{}	\N	2020-10-12 15:03:36.683342
46729	GOAU	2020-09-28	22.25	22.265	21.9861	22.14	31824	{}	\N	2020-10-12 15:03:37.850679
46730	GOAU	2020-09-29	22.35	22.68	22.35	22.49	65201	{}	\N	2020-10-12 15:03:37.859271
46731	GOAU	2020-09-30	22.39	22.5586	22.0622	22.36	36687	{}	\N	2020-10-12 15:03:37.865233
46732	GOAU	2020-10-01	22.59	22.83	22.43	22.55	311884	{}	\N	2020-10-12 15:03:37.871465
46733	GOAU	2020-10-02	22.31	22.56	22.0729	22.12	54018	{}	\N	2020-10-12 15:03:37.876013
46734	GOAU	2020-10-05	22.26	22.74	22.26	22.47	49852	{}	\N	2020-10-12 15:03:37.879941
46735	GOAU	2020-10-06	22.62	22.62	21.5991	21.6	53924	{}	\N	2020-10-12 15:03:37.883951
46736	GOAU	2020-10-07	21.79	21.97	21.62	21.68	43595	{}	\N	2020-10-12 15:03:37.887554
46737	GOAU	2020-10-08	21.99	22.1683	21.9	22	32979	{}	\N	2020-10-12 15:03:37.891765
46738	GOAU	2020-10-09	22.48	23.1	22.48	23.1	87263	{}	\N	2020-10-12 15:03:37.895401
46739	GOAU	2020-10-12	23.09	23.23	23.02	23.1666	42590	{}	\N	2020-10-12 15:03:37.898845
46740	GOODM	2020-09-28	25	25.14	25	25.1	6016	{}	\N	2020-10-12 15:03:40.494623
46741	GOODM	2020-09-29	25.02	25.1276	25.01	25.07	3489	{}	\N	2020-10-12 15:03:40.503757
46742	GOODM	2020-09-30	25.14	25.19	25.05	25.19	15435	{}	\N	2020-10-12 15:03:40.509589
46743	GOODM	2020-10-01	25.1849	25.24	25.1015	25.16	1459	{}	\N	2020-10-12 15:03:40.514654
46744	GOODM	2020-10-02	25.17	25.36	25.1441	25.17	1977	{}	\N	2020-10-12 15:03:40.519051
46745	GOODM	2020-10-05	25.2	25.47	25.15	25.41	10104	{}	\N	2020-10-12 15:03:40.523344
46746	GOODM	2020-10-06	25.23	25.45	25.22	25.32	4063	{}	\N	2020-10-12 15:03:40.527159
46747	GOODM	2020-10-07	25.18	25.42	25.18	25.399	2618	{}	\N	2020-10-12 15:03:40.530769
46748	GOODM	2020-10-08	25.26	25.34	25.15	25.3	1930	{}	\N	2020-10-12 15:03:40.534543
46749	GOODM	2020-10-09	25.2912	25.32	25.275	25.29	9269	{}	\N	2020-10-12 15:03:40.538099
46750	GOODM	2020-10-12	25.3	25.3	25.19	25.21	498	{}	\N	2020-10-12 15:03:40.541414
46751	GPJA	2020-09-28	26.74	26.96	26.74	26.96	21436	{}	\N	2020-10-12 15:03:41.791122
46752	GPJA	2020-09-29	26.83	26.83	26.5944	26.71	31442	{}	\N	2020-10-12 15:03:41.80023
46753	GPJA	2020-09-30	26.72	26.8126	26.21	26.21	226338	{}	\N	2020-10-12 15:03:41.806683
46754	GPJA	2020-10-01	26.34	26.55	26.3098	26.55	82342	{}	\N	2020-10-12 15:03:41.812397
46755	GPJA	2020-10-02	26.38	26.51	26.3693	26.46	50315	{}	\N	2020-10-12 15:03:41.816759
46756	GPJA	2020-10-05	26.64	26.64	26.49	26.64	29846	{}	\N	2020-10-12 15:03:41.82154
46757	GPJA	2020-10-06	26.6184	26.75	26.44	26.54	49163	{}	\N	2020-10-12 15:03:41.825635
46758	GPJA	2020-10-07	26.59	26.65	26.53	26.57	29536	{}	\N	2020-10-12 15:03:41.829539
46759	GPJA	2020-10-08	26.6	26.706	26.59	26.65	29164	{}	\N	2020-10-12 15:03:41.833658
46760	GPJA	2020-10-09	26.64	26.67	26.48	26.57	28179	{}	\N	2020-10-12 15:03:41.837306
46761	GPJA	2020-10-12	26.57	26.57	26.57	26.57	2660	{}	\N	2020-10-12 15:03:41.841169
46762	GPL	2020-09-28	0.9	0.9099	0.8642	0.8945	1486551	{}	\N	2020-10-12 15:04:02.728995
46763	GPL	2020-09-29	0.9	0.9299	0.892	0.9139	1490901	{}	\N	2020-10-12 15:04:02.739456
46764	GPL	2020-09-30	0.9001	0.9099	0.8651	0.8879	1134891	{}	\N	2020-10-12 15:04:02.745923
46765	GPL	2020-10-01	0.9	0.9097	0.8838	0.885	1227576	{}	\N	2020-10-12 15:04:02.751389
46766	GPL	2020-10-02	0.87	0.8961	0.85	0.8734	1159809	{}	\N	2020-10-12 15:04:02.756789
46767	GPL	2020-10-05	0.87	0.8989	0.87	0.8949	1512065	{}	\N	2020-10-12 15:04:02.761547
46768	GPL	2020-10-06	0.9	0.9055	0.871	0.881	1169703	{}	\N	2020-10-12 15:04:02.765888
46769	GPL	2020-10-07	0.885	0.895	0.8709	0.8811	860534	{}	\N	2020-10-12 15:04:02.770084
46770	GPL	2020-10-08	0.8741	0.91	0.8741	0.8943	1009022	{}	\N	2020-10-12 15:04:02.774104
46771	GPL	2020-10-09	0.91	0.9299	0.865	0.8996	4027711	{}	\N	2020-10-12 15:04:02.778714
46772	GPL	2020-10-12	0.9	0.9	0.881	0.8844	624412	{}	\N	2020-10-12 15:04:02.783071
46773	GPOR	2020-09-28	0.5194	0.5757	0.5095	0.5691	3291818	{}	\N	2020-10-12 15:04:04.635768
46774	GPOR	2020-09-29	0.5681	0.569	0.5212	0.5522	2986406	{}	\N	2020-10-12 15:04:04.648198
46775	GPOR	2020-09-30	0.55	0.5763	0.524	0.5271	4450863	{}	\N	2020-10-12 15:04:04.655737
46776	GPOR	2020-10-01	0.5286	0.5305	0.4625	0.4627	8532542	{}	\N	2020-10-12 15:04:04.662093
46777	GPOR	2020-10-02	0.465	0.5055	0.4601	0.4695	6374497	{}	\N	2020-10-12 15:04:04.667763
46778	GPOR	2020-10-05	0.51	0.6075	0.4933	0.5363	17976516	{}	\N	2020-10-12 15:04:04.672908
46779	GPOR	2020-10-06	0.57	0.592	0.524	0.5509	12147633	{}	\N	2020-10-12 15:04:04.677596
46780	GPOR	2020-10-07	0.5622	0.62	0.55	0.5836	11928014	{}	\N	2020-10-12 15:04:04.682047
46781	GPOR	2020-10-08	0.5948	0.69	0.58021	0.6628	13288688	{}	\N	2020-10-12 15:04:04.687377
46782	GPOR	2020-10-09	0.77	0.826	0.63	0.6591	24603603	{}	\N	2020-10-12 15:04:04.691507
46783	GPOR	2020-10-12	0.69	0.7099	0.6339	0.635	4222393	{}	\N	2020-10-12 15:04:04.696208
46784	FRC	2020-09-28	106.36	109.2	105.935	108.17	994298	{}	\N	2020-10-12 15:04:39.105374
46785	FRC	2020-09-29	107.64	108.97	107.31	107.67	1768364	{}	\N	2020-10-12 15:04:39.112012
46786	FRC	2020-09-30	108	110.26	107.78	109.06	1577129	{}	\N	2020-10-12 15:04:39.117076
46787	FRC	2020-10-01	109.81	110.88	108.43	110.37	1233035	{}	\N	2020-10-12 15:04:39.122041
46788	FRC	2020-10-02	109.2	113.22	108.67	112.94	994046	{}	\N	2020-10-12 15:04:39.126766
46789	FRC	2020-10-05	114.03	117.49	114.02	117.37	1204184	{}	\N	2020-10-12 15:04:39.130666
46790	FRC	2020-10-06	118.86	121.66	117.88	118.79	2496243	{}	\N	2020-10-12 15:04:39.134777
46791	FRC	2020-10-07	120	123.9	119.89	122.73	2353950	{}	\N	2020-10-12 15:04:39.138775
46792	FRC	2020-10-08	123.08	123.91	120.3223	123.76	1449088	{}	\N	2020-10-12 15:04:39.143092
46793	FRC	2020-10-09	124.19	124.8	122.09	123.72	1090910	{}	\N	2020-10-12 15:04:39.146978
46794	FRC	2020-10-12	123.63	125.22	123.49	124.545	507034	{}	\N	2020-10-12 15:04:39.151232
46795	FRT	2020-09-28	74.54	75.81	74.13	74.94	589636	{}	\N	2020-10-12 15:04:42.481155
46796	FRT	2020-09-29	74.58	74.64	71.7475	72.99	821925	{}	\N	2020-10-12 15:04:42.488242
46797	FRT	2020-09-30	73.74	74.55	72.3	73.44	1221979	{}	\N	2020-10-12 15:04:42.493795
46798	FRT	2020-10-01	74.36	76.1	73.6	76.05	948822	{}	\N	2020-10-12 15:04:42.498752
46799	FRT	2020-10-02	74.46	78.91	74.08	78.43	1093796	{}	\N	2020-10-12 15:04:42.503884
46800	FRT	2020-10-05	79.1	79.64	77.51	78.55	729307	{}	\N	2020-10-12 15:04:42.507746
46801	FRT	2020-10-06	79.26	80.095	77.7429	78.4	713608	{}	\N	2020-10-12 15:04:42.511709
46802	FRT	2020-10-07	78.63	79.01	77.01	77.65	672343	{}	\N	2020-10-12 15:04:42.515069
46803	FRT	2020-10-08	78.23	79.76	77.9	79.72	599442	{}	\N	2020-10-12 15:04:42.519008
46804	FRT	2020-10-09	80.3	80.34	78.13	78.38	808383	{}	\N	2020-10-12 15:04:42.523734
46805	FRT	2020-10-12	78.95	79.42	77.56	78.9	222715	{}	\N	2020-10-12 15:04:42.528163
46806	FSI	2020-09-28	2.05	2.19	2.03	2.11	21579	{}	\N	2020-10-12 15:04:45.74661
46807	FSI	2020-09-29	2.1699	2.1699	2.01	2.06	16111	{}	\N	2020-10-12 15:04:45.758121
46808	FSI	2020-09-30	2.04	2.09	1.95	2.04	43641	{}	\N	2020-10-12 15:04:45.765332
46809	FSI	2020-10-01	2.04	2.09	1.995	2.065	9070	{}	\N	2020-10-12 15:04:45.770338
46810	FSI	2020-10-02	1.92	2.07	1.92	2.06	24391	{}	\N	2020-10-12 15:04:45.774795
46811	FSI	2020-10-05	2.06	2.09	1.98	2.08	22034	{}	\N	2020-10-12 15:04:45.779053
46812	FSI	2020-10-06	2.01	2.09	2.01	2.08	13720	{}	\N	2020-10-12 15:04:45.783153
46813	FSI	2020-10-07	2.1	2.18	2.07	2.17	34605	{}	\N	2020-10-12 15:04:45.78685
46814	FSI	2020-10-08	2.23	2.36	2.2	2.27	30129	{}	\N	2020-10-12 15:04:45.790628
46815	FSI	2020-10-09	2.18	2.29	2.16	2.2	33399	{}	\N	2020-10-12 15:04:45.79464
46816	FSI	2020-10-12	2.29	2.29	2.18	2.21	6884	{}	\N	2020-10-12 15:04:45.798396
46817	FSV	2020-09-28	129.54	132.145	129.49	130.76	30958	{}	\N	2020-10-12 15:04:48.963701
46818	FSV	2020-09-29	131.1	131.71	129.58	131.43	26234	{}	\N	2020-10-12 15:04:48.97022
46819	FSV	2020-09-30	132.17	132.36	130.8	131.89	42205	{}	\N	2020-10-12 15:04:48.97534
46820	FSV	2020-10-01	132.66	133.33	131.63	133.04	47012	{}	\N	2020-10-12 15:04:48.980215
46821	FSV	2020-10-02	132.3	132.77	131.38	132.09	48587	{}	\N	2020-10-12 15:04:48.984619
46822	FSV	2020-10-05	132.58	134.92	132.58	134.41	68415	{}	\N	2020-10-12 15:04:48.989226
46823	FSV	2020-10-06	134.18	134.37	132.28	133.12	29551	{}	\N	2020-10-12 15:04:48.993429
46824	FSV	2020-10-07	135.2	135.2	133.01	133.2	32617	{}	\N	2020-10-12 15:04:48.997789
46825	FSV	2020-10-08	134.18	136.105	133.57	135.41	51327	{}	\N	2020-10-12 15:04:49.001712
46826	FSV	2020-10-09	135.67	136.785	135.34	136.65	23026	{}	\N	2020-10-12 15:04:49.005725
46827	FTEK	2020-09-28	0.8	0.83	0.763	0.79	237965	{}	\N	2020-10-12 15:04:50.217712
46828	FTEK	2020-09-29	0.814	0.8282	0.81	0.825	57450	{}	\N	2020-10-12 15:04:50.223434
46829	FTEK	2020-09-30	0.8238	0.855	0.82	0.8415	125648	{}	\N	2020-10-12 15:04:50.228279
46830	FTEK	2020-10-01	0.84	0.84	0.82	0.84	37154	{}	\N	2020-10-12 15:04:50.232841
46831	FTEK	2020-10-02	0.821	0.83105	0.8149	0.83	79023	{}	\N	2020-10-12 15:04:50.237342
46832	FTEK	2020-10-05	0.8	0.85	0.76995	0.8001	55265	{}	\N	2020-10-12 15:04:50.241142
46833	FTEK	2020-10-06	0.79	0.839899	0.78	0.8139	68533	{}	\N	2020-10-12 15:04:50.245005
46834	FTEK	2020-10-07	0.7984	0.828	0.7789	0.7981	66933	{}	\N	2020-10-12 15:04:50.248405
46835	FTEK	2020-10-08	0.8	0.8	0.7901	0.8	53937	{}	\N	2020-10-12 15:04:50.252135
46836	FTEK	2020-10-09	0.79	0.86	0.79	0.8463	155835	{}	\N	2020-10-12 15:04:50.256638
46837	FTEK	2020-10-12	0.8408	0.85	0.83	0.8499	21777	{}	\N	2020-10-12 15:04:50.261255
46838	FTS	2020-09-28	40.41	40.84	40.33	40.76	212292	{}	\N	2020-10-12 15:04:51.319362
46839	FTS	2020-09-29	40.84	40.93	40.43	40.83	494876	{}	\N	2020-10-12 15:04:51.328648
46840	FTS	2020-09-30	41	41.09	40.74	40.86	819531	{}	\N	2020-10-12 15:04:51.334154
46841	FTS	2020-10-01	40.92	41.29	40.72	41.05	415356	{}	\N	2020-10-12 15:04:51.340019
46842	FTS	2020-10-02	40.59	41.3	40.59	41.25	639618	{}	\N	2020-10-12 15:04:51.345368
46843	FTS	2020-10-05	41.38	41.92	41.1593	41.64	381596	{}	\N	2020-10-12 15:04:51.350428
46844	FTS	2020-10-06	41.75	41.75	41.25	41.51	436638	{}	\N	2020-10-12 15:04:51.355227
46845	FTS	2020-10-07	41.53	42.06	41.53	41.89	400888	{}	\N	2020-10-12 15:04:51.360122
46846	FTS	2020-10-08	42.09	42.4086	42.06	42.13	416654	{}	\N	2020-10-12 15:04:51.364495
46847	FTS	2020-10-09	42.28	42.47	41.7	41.79	684986	{}	\N	2020-10-12 15:04:51.368787
46848	FTS	2020-10-12	41.83	41.9396	41.79	41.82	37003	{}	\N	2020-10-12 15:04:51.376276
46849	FUSB	2020-09-28	6.56	6.6	6.56	6.6	1623	{}	\N	2020-10-12 15:04:54.703085
46850	FUSB	2020-09-29	6.71	6.71	6.71	6.71	623	{}	\N	2020-10-12 15:04:54.715212
46851	FUSB	2020-09-30	6.63	6.63	6.63	6.63	223	{}	\N	2020-10-12 15:04:54.724503
46852	FUSB	2020-10-01	6.5	6.5	6.35	6.425	2117	{}	\N	2020-10-12 15:04:54.731544
46853	FUSB	2020-10-02	6.38	6.4721	6.37	6.44	3834	{}	\N	2020-10-12 15:04:54.737075
46854	FUSB	2020-10-05	6.6115	6.8	6.55	6.55	2505	{}	\N	2020-10-12 15:04:54.742139
46855	FUSB	2020-10-06	7.08	7.1	6.7829	6.8	1768	{}	\N	2020-10-12 15:04:54.74662
46856	FUSB	2020-10-07	7.19	7.2	7.16	7.2	5236	{}	\N	2020-10-12 15:04:54.75114
46857	FUSB	2020-10-08	7.18	7.18	7.17	7.18	4349	{}	\N	2020-10-12 15:04:54.75545
46858	FUSB	2020-10-09	7.14	7.36	7.14	7.34	4721	{}	\N	2020-10-12 15:04:54.75975
46859	FUSB	2020-10-12	6.9337	6.9337	6.9337	6.9337	2192	{}	\N	2020-10-12 15:04:54.764089
46860	FXE	2020-09-28	109.87	109.95	109.63	109.85	76007	{}	\N	2020-10-12 15:04:57.65174
46861	FXE	2020-09-29	110.19	110.565	110.19	110.51	124808	{}	\N	2020-10-12 15:04:57.66415
46862	FXE	2020-09-30	110.24	110.59	110	110.37	100828	{}	\N	2020-10-12 15:04:57.67198
46863	FXE	2020-10-01	110.48	110.66	110.37	110.6	70276	{}	\N	2020-10-12 15:04:57.678423
46864	FXE	2020-10-02	110.23	110.37	110.16	110.22	57235	{}	\N	2020-10-12 15:04:57.68357
46865	FXE	2020-10-05	110.77	111.03	110.77	110.93	201491	{}	\N	2020-10-12 15:04:57.688466
46866	FXE	2020-10-06	111.0491	111.05	110.47	110.58	156140	{}	\N	2020-10-12 15:04:57.693319
46867	FXE	2020-10-07	110.81	110.84	110.62	110.68	310921	{}	\N	2020-10-12 15:04:57.698481
46868	FXE	2020-10-08	110.55	110.6731	110.42	110.65	310264	{}	\N	2020-10-12 15:04:57.702792
46869	FXE	2020-10-09	111.16	111.31	111.05	111.28	230046	{}	\N	2020-10-12 15:04:57.70703
46870	FXE	2020-10-12	111.11	111.19	111.08	111.16	120840	{}	\N	2020-10-12 15:04:57.711368
46871	FXU	2020-09-28	26.21	26.37	26.12	26.13	27679	{}	\N	2020-10-12 15:04:58.671498
46872	FXU	2020-09-29	26.3021	26.39	26.03	26.16	103972	{}	\N	2020-10-12 15:04:58.677909
46873	FXU	2020-09-30	26.41	26.5501	26.32	26.47	75403	{}	\N	2020-10-12 15:04:58.683752
46874	FXU	2020-10-01	26.53	26.6699	26.3981	26.57	142604	{}	\N	2020-10-12 15:04:58.69017
46875	FXU	2020-10-02	26.32	27.09	26.2889	26.97	116148	{}	\N	2020-10-12 15:04:58.697437
46876	FXU	2020-10-05	27.02	27.295	26.934	27.23	59508	{}	\N	2020-10-12 15:04:58.704104
46877	FXU	2020-10-06	27.25	27.85	27.2423	27.52	48630	{}	\N	2020-10-12 15:04:58.710738
46878	FXU	2020-10-07	27.58	27.72	27.4011	27.61	38344	{}	\N	2020-10-12 15:04:58.716284
46879	FXU	2020-10-08	27.69	28.14	27.69	28.12	136254	{}	\N	2020-10-12 15:04:58.722337
46880	FXU	2020-10-09	28.33	28.33	27.92	28.02	25452	{}	\N	2020-10-12 15:04:58.73085
46881	FXU	2020-10-12	28.0022	28.14	28.0022	28.14	2707	{}	\N	2020-10-12 15:04:58.739975
46882	GBDC	2020-09-28	13.21	13.28	13.1	13.18	415448	{}	\N	2020-10-12 15:05:00.508365
46883	GBDC	2020-09-29	13.26	13.33	13	13.02	427463	{}	\N	2020-10-12 15:05:00.513269
46884	GBDC	2020-09-30	13.01	13.36	12.98	13.24	1702205	{}	\N	2020-10-12 15:05:00.517904
46885	GBDC	2020-10-01	13.26	13.94	13.235	13.92	885008	{}	\N	2020-10-12 15:05:00.522982
46886	GBDC	2020-10-02	13.7806	13.89	13.65	13.83	561819	{}	\N	2020-10-12 15:05:00.528944
46887	GBDC	2020-10-05	13.83	13.9258	13.56	13.66	283179	{}	\N	2020-10-12 15:05:00.541763
46888	GBDC	2020-10-06	13.76	13.82	13.5585	13.58	217796	{}	\N	2020-10-12 15:05:00.549966
46889	GBDC	2020-10-07	13.6	13.6644	13.49	13.55	199921	{}	\N	2020-10-12 15:05:00.558596
46890	GBDC	2020-10-08	13.59	13.96	13.55	13.77	233297	{}	\N	2020-10-12 15:05:00.564931
46891	GBDC	2020-10-09	13.8	13.8462	13.65	13.8	223743	{}	\N	2020-10-12 15:05:00.570183
46892	GBDC	2020-10-12	13.88	13.925	13.64	13.74	56807	{}	\N	2020-10-12 15:05:00.575605
46893	GRID	2020-09-28	61.95	62.619	61.9	62.53	14751	{}	\N	2020-10-12 15:05:01.541497
46894	GRID	2020-09-29	62.9783	63.18	62.77	62.9784	7655	{}	\N	2020-10-12 15:05:01.548298
46895	GRID	2020-09-30	63	63.643	62.85	62.85	3289	{}	\N	2020-10-12 15:05:01.554663
46896	GRID	2020-10-01	63.53	64.03	63.39	63.855	2746	{}	\N	2020-10-12 15:05:01.563347
46897	GRID	2020-10-02	63.21	64.325	63.21	64.03	12836	{}	\N	2020-10-12 15:05:01.571499
46898	GRID	2020-10-05	64.59	65.8034	64.59	65.8034	7174	{}	\N	2020-10-12 15:05:01.57806
46899	GRID	2020-10-06	65.96	66.26	65.03	65.0769	6056	{}	\N	2020-10-12 15:05:01.584715
46900	GRID	2020-10-07	65.5	67.02	65.5	67.02	13849	{}	\N	2020-10-12 15:05:01.591395
46901	GRID	2020-10-08	67.42	67.55	67.04	67.3	5980	{}	\N	2020-10-12 15:05:01.597472
46902	GRID	2020-10-09	67.76	68.22	67.64	67.855	16510	{}	\N	2020-10-12 15:05:01.60325
46903	GRID	2020-10-12	68.39	68.39	68	68	5194	{}	\N	2020-10-12 15:05:01.610119
46904	GROW	2020-09-28	2.5	2.59	2.43	2.44	20865	{}	\N	2020-10-12 15:05:03.836678
46905	GROW	2020-09-29	2.43	2.4699	2.32	2.32	29217	{}	\N	2020-10-12 15:05:03.843418
46906	GROW	2020-09-30	2.33	2.47	2.33	2.38	21588	{}	\N	2020-10-12 15:05:03.849767
46907	GROW	2020-10-01	2.425	2.5062	2.38	2.41	27453	{}	\N	2020-10-12 15:05:03.854822
46908	GROW	2020-10-02	2.45	2.52	2.425	2.5	22246	{}	\N	2020-10-12 15:05:03.860342
46909	GROW	2020-10-05	2.51	2.687	2.5	2.5	43194	{}	\N	2020-10-12 15:05:03.865059
46910	GROW	2020-10-06	2.53	2.61	2.48	2.5	46026	{}	\N	2020-10-12 15:05:03.869462
46911	GROW	2020-10-07	2.59	2.59	2.4171	2.525	18843	{}	\N	2020-10-12 15:05:03.875782
46912	GROW	2020-10-08	2.54	2.6195	2.54	2.56	25308	{}	\N	2020-10-12 15:05:03.882656
46913	GROW	2020-10-09	2.63	2.75	2.57	2.68	44071	{}	\N	2020-10-12 15:05:03.889429
46914	GROW	2020-10-12	2.65	2.745	2.65	2.6939	3148	{}	\N	2020-10-12 15:05:03.894573
46915	GSAT	2020-09-28	0.31	0.3111	0.2985	0.304	1104826	{}	\N	2020-10-12 15:05:05.152032
46916	GSAT	2020-09-29	0.3035	0.3169	0.3035	0.3101	894756	{}	\N	2020-10-12 15:05:05.161302
46917	GSAT	2020-09-30	0.31	0.3159	0.303	0.3068	865582	{}	\N	2020-10-12 15:05:05.167281
46918	GSAT	2020-10-01	0.306	0.311899	0.3002	0.3068	736597	{}	\N	2020-10-12 15:05:05.173068
46919	GSAT	2020-10-02	0.3	0.31	0.3	0.30615	825967	{}	\N	2020-10-12 15:05:05.177884
46920	GSAT	2020-10-05	0.3171	0.3171	0.302	0.302	1055495	{}	\N	2020-10-12 15:05:05.182757
46921	GSAT	2020-10-06	0.31	0.3399	0.303	0.3051	5793714	{}	\N	2020-10-12 15:05:05.186846
46922	GSAT	2020-10-07	0.3184	0.3189	0.305	0.305	987440	{}	\N	2020-10-12 15:05:05.190788
46923	GSAT	2020-10-08	0.311	0.32125	0.31	0.3132	792482	{}	\N	2020-10-12 15:05:05.194608
46924	GSAT	2020-10-09	0.3121	0.3277	0.3103	0.3138	1432764	{}	\N	2020-10-12 15:05:05.198605
46925	GSAT	2020-10-12	0.3138	0.3193	0.3121	0.3123	166019	{}	\N	2020-10-12 15:05:05.202798
46926	GSG	2020-09-28	10.74	10.81	10.68	10.795	172067	{}	\N	2020-10-12 15:05:06.8095
46927	GSG	2020-09-29	10.7	10.71	10.51	10.58	253824	{}	\N	2020-10-12 15:05:06.81578
46928	GSG	2020-09-30	10.63	10.8	10.61	10.75	663594	{}	\N	2020-10-12 15:05:06.822991
46929	GSG	2020-10-01	10.63	10.65	10.43	10.55	1773712	{}	\N	2020-10-12 15:05:06.829436
46930	GSG	2020-10-02	10.34	10.48	10.3005	10.37	404314	{}	\N	2020-10-12 15:05:06.835799
46931	GSG	2020-10-05	10.54	10.75	10.54	10.72	407417	{}	\N	2020-10-12 15:05:06.843707
46932	GSG	2020-10-06	10.85	10.945	10.77	10.79	718877	{}	\N	2020-10-12 15:05:06.84997
46933	GSG	2020-10-07	10.81	10.87	10.76	10.86	134800	{}	\N	2020-10-12 15:05:06.855065
46934	GSG	2020-10-08	10.95	11.0262	10.94	11.02	282302	{}	\N	2020-10-12 15:05:06.859861
46935	GSG	2020-10-09	11.05	11.12	11.0077	11.03	197178	{}	\N	2020-10-12 15:05:06.865067
46936	GSG	2020-10-12	10.97	11.01	10.86	10.8769	109256	{}	\N	2020-10-12 15:05:06.871322
46937	GSLC	2020-09-28	67.61	67.92	67.4015	67.81	358847	{}	\N	2020-10-12 15:05:07.802694
46938	GSLC	2020-09-29	67.78	67.97	67.4201	67.5	245021	{}	\N	2020-10-12 15:05:07.809017
46939	GSLC	2020-09-30	67.67	68.66	67.64	68.05	338596	{}	\N	2020-10-12 15:05:07.814035
46940	GSLC	2020-10-01	68.57	68.79	68.17	68.58	572441	{}	\N	2020-10-12 15:05:07.818763
46941	GSLC	2020-10-02	67.34	68.24	67.34	67.91	801375	{}	\N	2020-10-12 15:05:07.823578
46942	GSLC	2020-10-05	68.34	69.07	68.34	69.07	229485	{}	\N	2020-10-12 15:05:07.828987
46943	GSLC	2020-10-06	69.13	69.51	68.02	68.11	373888	{}	\N	2020-10-12 15:05:07.83381
46944	GSLC	2020-10-07	68.74	69.4091	68.68	69.2802	224352	{}	\N	2020-10-12 15:05:07.838256
46945	GSLC	2020-10-08	69.64	69.85	69.48	69.8	397529	{}	\N	2020-10-12 15:05:07.842864
46946	GSLC	2020-10-09	70.15	70.5386	70.11	70.39	270858	{}	\N	2020-10-12 15:05:07.847993
46947	GSLC	2020-10-12	71	71.2696	70.9	71.26	65060	{}	\N	2020-10-12 15:05:07.852423
46948	GTLS	2020-09-28	70.07	70.68	69.0067	69.67	329449	{}	\N	2020-10-12 15:05:08.900914
46949	GTLS	2020-09-29	69.61	70.14	68.555	69.32	166747	{}	\N	2020-10-12 15:05:08.910173
46950	GTLS	2020-09-30	69.41	71.47	69.3	70.27	319922	{}	\N	2020-10-12 15:05:08.916853
46951	GTLS	2020-10-01	70.82	72.29	69.8	72.01	224490	{}	\N	2020-10-12 15:05:08.922624
46952	GTLS	2020-10-02	69.91	74	69.0003	73.75	369572	{}	\N	2020-10-12 15:05:08.927829
46953	GTLS	2020-10-05	74.71	76.15	74.35	75.91	407408	{}	\N	2020-10-12 15:05:08.932932
46954	GTLS	2020-10-06	76.62	76.74	70.15	70.69	494054	{}	\N	2020-10-12 15:05:08.937477
46955	GTLS	2020-10-07	72.63	74.1	71.45	73.91	492099	{}	\N	2020-10-12 15:05:08.942159
46956	GTLS	2020-10-08	75.215	75.82	72.5	72.63	243821	{}	\N	2020-10-12 15:05:08.946898
46957	GTLS	2020-10-09	73.42	73.85	72.775	73.17	231810	{}	\N	2020-10-12 15:05:08.951339
46958	GTLS	2020-10-12	73.98	74.13	72.34	73.35	53870	{}	\N	2020-10-12 15:05:08.955821
46959	GUNR	2020-09-28	27.95	28.1	27.87	27.92	882588	{}	\N	2020-10-12 15:05:10.41855
46960	GUNR	2020-09-29	27.88	27.88	27.52	27.63	811214	{}	\N	2020-10-12 15:05:10.429329
46961	GUNR	2020-09-30	27.69	27.935	27.6299	27.76	1108676	{}	\N	2020-10-12 15:05:10.438156
46962	GUNR	2020-10-01	27.76	27.78	27.46	27.59	451724	{}	\N	2020-10-12 15:05:10.443619
46963	GUNR	2020-10-02	27.2	27.75	27.2	27.6	614308	{}	\N	2020-10-12 15:05:10.448816
46964	GUNR	2020-10-05	27.85	28.19	27.85	28.19	692229	{}	\N	2020-10-12 15:05:10.453309
46965	GUNR	2020-10-06	28.38	28.45	27.74	27.8	759601	{}	\N	2020-10-12 15:05:10.457503
46966	GUNR	2020-10-07	28.05	28.17	27.94	28.09	389922	{}	\N	2020-10-12 15:05:10.46154
46967	GUNR	2020-10-08	28.3037	28.56	28.3037	28.54	1418514	{}	\N	2020-10-12 15:05:10.46589
46968	GUNR	2020-10-09	28.87	28.9	28.69	28.8	850422	{}	\N	2020-10-12 15:05:10.470025
46969	GUNR	2020-10-12	28.78	28.81	28.69	28.77	57448	{}	\N	2020-10-12 15:05:10.474063
46970	GVI	2020-09-28	117.63	117.87	117.53	117.67	51137	{}	\N	2020-10-12 15:05:11.386489
46971	GVI	2020-09-29	117.65	117.84	117.645	117.82	144160	{}	\N	2020-10-12 15:05:11.392474
46972	GVI	2020-09-30	117.81	117.9	117.6006	117.74	65276	{}	\N	2020-10-12 15:05:11.398155
46973	GVI	2020-10-01	117.35	117.66	117.26	117.66	50220	{}	\N	2020-10-12 15:05:11.402911
46974	GVI	2020-10-02	117.7	117.7	117.39	117.63	82424	{}	\N	2020-10-12 15:05:11.407741
46975	GVI	2020-10-05	117.42	117.6774	117.4	117.42	39422	{}	\N	2020-10-12 15:05:11.412667
46976	GVI	2020-10-06	117.4	117.6442	117.36	117.51	57841	{}	\N	2020-10-12 15:05:11.417207
46977	GVI	2020-10-07	117.3	117.5199	117.3	117.42	55856	{}	\N	2020-10-12 15:05:11.421566
46978	GVI	2020-10-08	117.4	117.57	117.3801	117.54	46758	{}	\N	2020-10-12 15:05:11.425782
46979	GVI	2020-10-09	117.35	117.57	117.35	117.54	43196	{}	\N	2020-10-12 15:05:11.430415
46980	GVI	2020-10-12	117.44	117.7	117.44	117.6327	11163	{}	\N	2020-10-12 15:05:11.434886
46981	GWW	2020-09-28	351.42	357.03	351.42	353.67	152455	{}	\N	2020-10-12 15:05:13.118732
46982	GWW	2020-09-29	353.58	354.46	349.08	352.08	227483	{}	\N	2020-10-12 15:05:13.132317
46983	GWW	2020-09-30	356.08	359.765	353.61	356.77	657631	{}	\N	2020-10-12 15:05:13.150786
46984	GWW	2020-10-01	358.17	360.16	352.845	358.03	500921	{}	\N	2020-10-12 15:05:13.155926
46985	GWW	2020-10-02	355.11	362.02	353.47	359.39	312964	{}	\N	2020-10-12 15:05:13.160628
46986	GWW	2020-10-05	361.93	372.68	361.93	371.16	389198	{}	\N	2020-10-12 15:05:13.165787
46987	GWW	2020-10-06	372.44	374.12	367.35	367.71	614161	{}	\N	2020-10-12 15:05:13.170381
46988	GWW	2020-10-07	371.51	379.3	370.82	376.4	398767	{}	\N	2020-10-12 15:05:13.174664
46989	GWW	2020-10-08	376.73	389.17	376.39	385.99	936092	{}	\N	2020-10-12 15:05:13.179209
46990	GWW	2020-10-09	386.68	392.35	383.1201	385.42	827029	{}	\N	2020-10-12 15:05:13.183733
46991	GWW	2020-10-12	388.54	389.255	385.79	386.42	89146	{}	\N	2020-10-12 15:05:13.188017
46992	HACK	2020-09-28	46.78	46.98	46.49	46.9	61719	{}	\N	2020-10-12 15:05:14.181945
46993	HACK	2020-09-29	46.91	47.0199	46.57	46.64	100984	{}	\N	2020-10-12 15:05:14.191516
46994	HACK	2020-09-30	46.6	47.1	46.36	46.54	75299	{}	\N	2020-10-12 15:05:14.197828
46995	HACK	2020-10-01	46.96	47.32	46.87	47.25	605062	{}	\N	2020-10-12 15:05:14.203271
46996	HACK	2020-10-02	46.5	47.21	46.33	46.78	148679	{}	\N	2020-10-12 15:05:14.207995
46997	HACK	2020-10-05	47.03	47.63	46.96	47.6	363614	{}	\N	2020-10-12 15:05:14.212534
46998	HACK	2020-10-06	47.51	48.485	47.38	47.53	570264	{}	\N	2020-10-12 15:05:14.217423
46999	HACK	2020-10-07	47.81	48.355	47.7801	48.23	221498	{}	\N	2020-10-12 15:05:14.221393
47000	HACK	2020-10-08	48.58	48.7665	48.2211	48.39	158682	{}	\N	2020-10-12 15:05:14.225473
47001	HACK	2020-10-09	48.71	49.17	48.71	49.13	190375	{}	\N	2020-10-12 15:05:14.229266
47002	HACK	2020-10-12	49.48	49.78	49.34	49.7463	69872	{}	\N	2020-10-12 15:05:14.233034
47003	GRVY	2020-09-28	134.19	143.99	132.61	143.99	139756	{}	\N	2020-10-12 15:05:16.002691
47004	GRVY	2020-09-29	148.71	150	137.5	140.34	159463	{}	\N	2020-10-12 15:05:16.013246
47005	GRVY	2020-09-30	141.41	146.9	140.9691	143.35	64266	{}	\N	2020-10-12 15:05:16.020202
47006	GRVY	2020-10-01	147.48	147.85	122.07	127	250724	{}	\N	2020-10-12 15:05:16.026003
47007	GRVY	2020-10-02	120	123	112.6711	115.02	235647	{}	\N	2020-10-12 15:05:16.031414
47008	GRVY	2020-10-05	119.04	121.3	102.69	115	243891	{}	\N	2020-10-12 15:05:16.036881
47009	GRVY	2020-10-06	115	118.99	109	113.7	83722	{}	\N	2020-10-12 15:05:16.041319
47010	GRVY	2020-10-07	116.27	122.97	116.27	119	78356	{}	\N	2020-10-12 15:05:16.045487
47011	GRVY	2020-10-08	120	126.99	120	124.31	62102	{}	\N	2020-10-12 15:05:16.050193
47012	GRVY	2020-10-09	127	132	124.4619	125.53	66737	{}	\N	2020-10-12 15:05:16.054543
47013	GRVY	2020-10-12	128	131.9899	128	129	23282	{}	\N	2020-10-12 15:05:16.058732
47014	GSH	2020-09-28	8.46	8.5	8.41	8.41	36409	{}	\N	2020-10-12 15:05:17.079143
47015	GSH	2020-09-29	8.48	8.55	8.48	8.48	30082	{}	\N	2020-10-12 15:05:17.090417
47016	GSH	2020-09-30	8.42	8.4499	8.36	8.36	48962	{}	\N	2020-10-12 15:05:17.09666
47017	GSH	2020-10-01	8.4	8.415	8.37	8.37	25584	{}	\N	2020-10-12 15:05:17.101619
47018	GSH	2020-10-02	8.34	8.39	8.33	8.33	22794	{}	\N	2020-10-12 15:05:17.106627
47019	GSH	2020-10-05	8.38	8.38	8.25	8.35	47438	{}	\N	2020-10-12 15:05:17.111384
47020	GSH	2020-10-06	8.32	8.32	8.21	8.25	54609	{}	\N	2020-10-12 15:05:17.115787
47021	GSH	2020-10-07	8.15	8.15	8.07	8.09	60318	{}	\N	2020-10-12 15:05:17.120068
47022	GSH	2020-10-08	8.2	8.29	8.12	8.24	118983	{}	\N	2020-10-12 15:05:17.124106
47023	GSH	2020-10-09	8.19	8.29	8.19	8.29	27634	{}	\N	2020-10-12 15:05:17.128283
47024	GSH	2020-10-12	8.56	8.68	8.56	8.655	40552	{}	\N	2020-10-12 15:05:17.132099
47025	GSM	2020-09-28	0.69	0.757	0.69	0.7201	478410	{}	\N	2020-10-12 15:05:18.921285
47026	GSM	2020-09-29	0.7202	0.7501	0.7003	0.7338	508585	{}	\N	2020-10-12 15:05:18.931204
47027	GSM	2020-09-30	0.7256	0.75	0.65	0.669	1253806	{}	\N	2020-10-12 15:05:18.937763
47028	GSM	2020-10-01	0.6597	0.81	0.63655	0.65	784234	{}	\N	2020-10-12 15:05:18.943636
47029	GSM	2020-10-02	0.6379	0.67	0.62	0.64	491789	{}	\N	2020-10-12 15:05:18.948565
47030	GSM	2020-10-05	0.6412	0.672	0.6212	0.6315	464783	{}	\N	2020-10-12 15:05:18.953243
47031	GSM	2020-10-06	0.6315	0.6487	0.6066	0.6086	273983	{}	\N	2020-10-12 15:05:18.95706
47032	GSM	2020-10-07	0.643	0.749	0.6111	0.6174	2759410	{}	\N	2020-10-12 15:05:18.960862
47033	GSM	2020-10-08	0.67	0.6773	0.6122	0.617	531978	{}	\N	2020-10-12 15:05:18.964779
47034	GSM	2020-10-09	0.6	0.6623	0.6	0.64	1721855	{}	\N	2020-10-12 15:05:18.968631
47035	GSM	2020-10-12	0.6562	0.658799	0.62	0.62	122841	{}	\N	2020-10-12 15:05:18.972547
47036	GT	2020-09-28	7.55	7.84	7.55	7.73	2718890	{}	\N	2020-10-12 15:05:19.890102
47037	GT	2020-09-29	7.68	7.735	7.525	7.63	1911733	{}	\N	2020-10-12 15:05:19.899306
47038	GT	2020-09-30	7.69	7.89	7.62	7.67	3017855	{}	\N	2020-10-12 15:05:19.905335
47039	GT	2020-10-01	8.09	8.33	7.97	8.32	4550002	{}	\N	2020-10-12 15:05:19.911079
47040	GT	2020-10-02	8	8.44	7.96	8.355	2977407	{}	\N	2020-10-12 15:05:19.916208
47041	GT	2020-10-05	8.44	8.87	8.43	8.86	3110499	{}	\N	2020-10-12 15:05:19.920868
47042	GT	2020-10-06	8.93	9.265	8.82	8.84	3300208	{}	\N	2020-10-12 15:05:19.925456
47043	GT	2020-10-07	8.94	9.56	8.94	9.53	4071408	{}	\N	2020-10-12 15:05:19.930022
47044	GT	2020-10-08	9.58	9.65	9.25	9.62	4086312	{}	\N	2020-10-12 15:05:19.934662
47045	GT	2020-10-09	9.68	9.79	9.56	9.64	2373227	{}	\N	2020-10-12 15:05:19.939064
47046	GT	2020-10-12	9.64	9.825	9.59	9.73	658779	{}	\N	2020-10-12 15:05:19.943539
47047	GTT	2020-09-28	5.07	5.41	4.92	5.28	240493	{}	\N	2020-10-12 15:05:21.962252
47048	GTT	2020-09-29	5.25	5.251	4.82	5.04	671116	{}	\N	2020-10-12 15:05:21.972074
47049	GTT	2020-09-30	5.03	5.3	5.01	5.16	616329	{}	\N	2020-10-12 15:05:21.978534
47050	GTT	2020-10-01	5.11	5.27	4.9	5.03	554387	{}	\N	2020-10-12 15:05:21.984209
47051	GTT	2020-10-02	4.95	5.37	4.76	5.21	649886	{}	\N	2020-10-12 15:05:21.988799
47052	GTT	2020-10-05	5.05	5.29	5.02	5.24	562418	{}	\N	2020-10-12 15:05:21.993351
47053	GTT	2020-10-06	5.24	5.35	5.03	5.07	620350	{}	\N	2020-10-12 15:05:21.998008
47054	GTT	2020-10-07	5.15	5.17	4.91	5.1	624368	{}	\N	2020-10-12 15:05:22.002526
47055	GTT	2020-10-08	5.14	5.31	4.94	5.18	750119	{}	\N	2020-10-12 15:05:22.006914
47056	GTT	2020-10-09	5.24	5.2629	4.9	4.99	620379	{}	\N	2020-10-12 15:05:22.010934
47057	GTT	2020-10-12	4.9	4.95	4.74	4.75	272217	{}	\N	2020-10-12 15:05:22.014637
47058	GVIP	2020-09-28	76.78	77.2468	76.6291	77.132	5927	{}	\N	2020-10-12 15:05:26.033324
47059	GVIP	2020-09-29	77.2026	77.21	76.8512	76.942	4016	{}	\N	2020-10-12 15:05:26.040938
47060	GVIP	2020-09-30	77.455	78.0221	76.88	77.5899	10428	{}	\N	2020-10-12 15:05:26.046713
47061	GVIP	2020-10-01	78.36	78.805	78.36	78.805	2596	{}	\N	2020-10-12 15:05:26.052093
47062	GVIP	2020-10-02	78.0767	78.154	77.06	77.2764	4916	{}	\N	2020-10-12 15:05:26.057648
47063	GVIP	2020-10-05	78.1	78.79	78.1	78.68	5646	{}	\N	2020-10-12 15:05:26.062036
47064	GVIP	2020-10-06	78.95	79.08	77.52	77.93	11604	{}	\N	2020-10-12 15:05:26.066275
47065	GVIP	2020-10-07	78.94	79.28	78.94	79.17	4383	{}	\N	2020-10-12 15:05:26.070308
47066	GVIP	2020-10-08	79.95	79.95	79.28	79.5	7427	{}	\N	2020-10-12 15:05:26.0742
47067	GVIP	2020-10-09	80.219	80.28	79.7785	80.14	5695	{}	\N	2020-10-12 15:05:26.078164
47068	GVIP	2020-10-12	81.16	81.17	80.7216	81.17	6932	{}	\N	2020-10-12 15:05:26.082239
47069	GWX	2020-09-28	30.29	30.3022	30.198	30.26	48918	{}	\N	2020-10-12 15:05:27.361722
47070	GWX	2020-09-29	30.3231	30.374	30.18	30.33	25204	{}	\N	2020-10-12 15:05:27.370077
47071	GWX	2020-09-30	30.25	30.5343	30.225	30.38	72858	{}	\N	2020-10-12 15:05:27.375496
47072	GWX	2020-10-01	30.5	30.57	30.37	30.5	49588	{}	\N	2020-10-12 15:05:27.380937
47073	GWX	2020-10-02	30.02	30.4021	30.02	30.33	26432	{}	\N	2020-10-12 15:05:27.385295
47074	GWX	2020-10-05	30.75	30.92	30.72	30.86	23929	{}	\N	2020-10-12 15:05:27.389722
47075	GWX	2020-10-06	30.9	31.0165	30.6201	30.67	44655	{}	\N	2020-10-12 15:05:27.394099
47076	GWX	2020-10-07	30.86	31.03	30.84	31.01	44150	{}	\N	2020-10-12 15:05:27.398576
47077	GWX	2020-10-08	31.06	31.2	31.06	31.2	33324	{}	\N	2020-10-12 15:05:27.402751
47078	GWX	2020-10-09	31.31	31.4581	31.27	31.4427	81932	{}	\N	2020-10-12 15:05:27.406973
47079	GWX	2020-10-12	31.31	31.49	31.31	31.49	5392	{}	\N	2020-10-12 15:05:27.411235
47080	H	2020-09-28	54.78	55.1195	54.06	54.49	419229	{}	\N	2020-10-12 15:05:28.591942
47081	H	2020-09-29	54.57	54.57	53.18	53.25	1321120	{}	\N	2020-10-12 15:05:28.600282
47082	H	2020-09-30	53.94	55	53.05	53.37	1337190	{}	\N	2020-10-12 15:05:28.605435
47083	H	2020-10-01	54.06	54.53	52.74	53.64	1164934	{}	\N	2020-10-12 15:05:28.61047
47084	H	2020-10-02	51.53	54.22	51.5	53.68	1436825	{}	\N	2020-10-12 15:05:28.615263
47085	H	2020-10-05	54.11	55.13	53.73	54.76	1343817	{}	\N	2020-10-12 15:05:28.620339
47086	H	2020-10-06	55.77	57.185	55.28	55.32	1753075	{}	\N	2020-10-12 15:05:28.624639
47087	H	2020-10-07	56.22	57.9	55.87	57.15	1849450	{}	\N	2020-10-12 15:05:28.629554
47088	H	2020-10-08	57.3	59.18	57.23	59.01	1552466	{}	\N	2020-10-12 15:05:28.682193
47089	H	2020-10-09	59.62	60.26	57.29	57.42	1752019	{}	\N	2020-10-12 15:05:28.689128
47090	H	2020-10-12	57.45	57.82	57.0801	57.58	273224	{}	\N	2020-10-12 15:05:28.695895
47091	HALL	2020-09-28	2.75	2.8353	2.62	2.67	335379	{}	\N	2020-10-12 15:05:30.07297
47092	HALL	2020-09-29	2.68	2.95	2.58	2.75	474089	{}	\N	2020-10-12 15:05:30.084925
47093	HALL	2020-09-30	2.75	2.83	2.595	2.62	187038	{}	\N	2020-10-12 15:05:30.092723
47094	HALL	2020-10-01	2.63	2.83	2.5895	2.67	422430	{}	\N	2020-10-12 15:05:30.099136
47095	HALL	2020-10-02	2.62	2.77	2.57	2.73	153025	{}	\N	2020-10-12 15:05:30.103464
47096	HALL	2020-10-05	2.73	2.88	2.66	2.85	108664	{}	\N	2020-10-12 15:05:30.107807
47097	HALL	2020-10-06	2.87	2.95	2.72	2.72	199633	{}	\N	2020-10-12 15:05:30.112772
47098	HALL	2020-10-07	2.77	2.8	2.67	2.73	77023	{}	\N	2020-10-12 15:05:30.116879
47099	HALL	2020-10-08	2.76	2.87	2.75	2.85	201753	{}	\N	2020-10-12 15:05:30.121285
47100	HALL	2020-10-09	2.83	2.8433	2.7101	2.76	98750	{}	\N	2020-10-12 15:05:30.125122
47101	HALL	2020-10-12	2.75	2.8071	2.71	2.75	46111	{}	\N	2020-10-12 15:05:30.129039
47102	KF	2020-09-28	29.2	29.55	29.2	29.49	3478	{}	\N	2020-10-12 15:05:31.800045
47103	KF	2020-09-29	29.36	29.363	29.01	29.259	11406	{}	\N	2020-10-12 15:05:31.81152
47104	KF	2020-09-30	29.44	29.65	29.44	29.65	4362	{}	\N	2020-10-12 15:05:31.818247
47105	KF	2020-10-01	29.78	29.854	29.55	29.78	8110	{}	\N	2020-10-12 15:05:31.823974
47106	KF	2020-10-02	29.41	30	29.2	29.7	14568	{}	\N	2020-10-12 15:05:31.829262
47107	KF	2020-10-05	29.91	30.12	29.91	30.12	12478	{}	\N	2020-10-12 15:05:31.833998
47108	KF	2020-10-06	30.23	30.64	30.2029	30.39	5857	{}	\N	2020-10-12 15:05:31.837925
47109	KF	2020-10-07	30.74	30.8695	30.6627	30.83	14266	{}	\N	2020-10-12 15:05:31.841788
47110	KF	2020-10-08	30.91	31.15	30.7101	31.15	3982	{}	\N	2020-10-12 15:05:31.846014
47111	KF	2020-10-12	31.1	31.45	30.6042	31.31	11034	{}	\N	2020-10-12 15:05:31.850192
47112	GRIF	2020-09-28	52.15	52.15	52	52	2974	{}	\N	2020-10-12 15:05:33.593439
47113	GRIF	2020-09-29	52	53.09	52	52.5	2386	{}	\N	2020-10-12 15:05:33.60582
47114	GRIF	2020-09-30	52.45	53.45	52.45	53.45	1513	{}	\N	2020-10-12 15:05:33.612925
47115	GRIF	2020-10-01	54.01	57.72	53.45	54.62	3120	{}	\N	2020-10-12 15:05:33.618696
47116	GRIF	2020-10-02	54.96	56.5779	54.24	54.76	3947	{}	\N	2020-10-12 15:05:33.62367
47117	GRIF	2020-10-05	55.53	56.45	55.1868	55.94	5489	{}	\N	2020-10-12 15:05:33.628913
47118	GRIF	2020-10-06	56.05	56.05	55.06	55.06	1807	{}	\N	2020-10-12 15:05:33.633132
47119	GRIF	2020-10-07	55.53	56.73	55.499	56.73	6107	{}	\N	2020-10-12 15:05:33.637105
47120	GRIF	2020-10-08	56.33	60.45	56.25	58.84	9378	{}	\N	2020-10-12 15:05:33.641002
47121	GRIF	2020-10-09	60.7827	64.345	59.2518	60.44	9782	{}	\N	2020-10-12 15:05:33.645055
47122	GRIF	2020-10-12	60.44	60.625	59.63	60.625	900	{}	\N	2020-10-12 15:05:33.648946
47123	GRN	2020-09-28	55.165	55.165	55.165	55.165	134	{}	\N	2020-10-12 15:05:35.181952
47124	GRN	2020-09-29	53.335	53.335	53.335	53.335	6	{}	\N	2020-10-12 15:05:35.188488
47125	GRN	2020-09-30	53.81	53.81	53.6392	53.6392	285	{}	\N	2020-10-12 15:05:35.193463
47126	GRN	2020-10-01	52.83	52.83	52.83	52.83	0	{}	\N	2020-10-12 15:05:35.197868
47127	GRN	2020-10-02	53.64	53.805	53.64	53.805	111	{}	\N	2020-10-12 15:05:35.203628
47128	GRN	2020-10-05	53.525	53.525	53.525	53.525	28	{}	\N	2020-10-12 15:05:35.207359
47129	GRN	2020-10-06	53.425	53.425	53.425	53.425	12	{}	\N	2020-10-12 15:05:35.211143
47130	GRN	2020-10-07	53.505	53.505	53.505	53.505	20	{}	\N	2020-10-12 15:05:35.214831
47131	GRN	2020-10-08	52.375	52.375	52.375	52.375	5	{}	\N	2020-10-12 15:05:35.218498
47132	GRN	2020-10-09	51.405	51.405	51.405	51.405	47	{}	\N	2020-10-12 15:05:35.221964
47133	GRX	2020-09-28	10.2	10.3	10.2	10.22	43446	{}	\N	2020-10-12 15:05:36.433797
47134	GRX	2020-09-29	10.18	10.26	10.15	10.15	83554	{}	\N	2020-10-12 15:05:36.446311
47135	GRX	2020-09-30	10.17	10.44	10.17	10.32	137889	{}	\N	2020-10-12 15:05:36.453486
47136	GRX	2020-10-01	10.32	10.4	10.26	10.29	88918	{}	\N	2020-10-12 15:05:36.459649
47137	GRX	2020-10-02	10.21	10.34	10.181	10.27	71258	{}	\N	2020-10-12 15:05:36.46408
47138	GRX	2020-10-05	10.32	10.54	10.32	10.48	53434	{}	\N	2020-10-12 15:05:36.468722
47139	GRX	2020-10-06	10.51	10.71	10.43	10.47	96879	{}	\N	2020-10-12 15:05:36.473314
47140	GRX	2020-10-07	10.56	10.74	10.54	10.67	111496	{}	\N	2020-10-12 15:05:36.478065
47141	GRX	2020-10-08	10.72	10.725	10.67	10.68	37650	{}	\N	2020-10-12 15:05:36.482234
47142	GRX	2020-10-09	10.75	10.8223	10.67	10.82	85242	{}	\N	2020-10-12 15:05:36.486687
47143	GRX	2020-10-12	10.85	10.88	10.805	10.83	40886	{}	\N	2020-10-12 15:05:36.491064
47144	GSIE	2020-09-28	28.17	28.24	28.11	28.2	168405	{}	\N	2020-10-12 15:05:38.102955
47145	GSIE	2020-09-29	28.2799	28.31	28.1127	28.2	317658	{}	\N	2020-10-12 15:05:38.114035
47146	GSIE	2020-09-30	28.12	28.28	28.05	28.15	1142381	{}	\N	2020-10-12 15:05:38.120679
47147	GSIE	2020-10-01	28.28	28.33	28.14	28.29	459801	{}	\N	2020-10-12 15:05:38.126672
47148	GSIE	2020-10-02	28.03	28.32	28.03	28.29	272853	{}	\N	2020-10-12 15:05:38.131302
47149	GSIE	2020-10-05	28.5	28.7	28.5	28.69	173453	{}	\N	2020-10-12 15:05:38.135991
47150	GSIE	2020-10-06	28.67	28.7	28.31	28.38	336685	{}	\N	2020-10-12 15:05:38.140249
47151	GSIE	2020-10-07	28.57	28.64	28.5	28.57	201773	{}	\N	2020-10-12 15:05:38.144448
47152	GSIE	2020-10-08	28.73	28.81	28.7	28.81	333035	{}	\N	2020-10-12 15:05:38.148176
47153	GSIE	2020-10-09	28.97	29.052	28.92	29.04	136570	{}	\N	2020-10-12 15:05:38.15204
47154	GSIE	2020-10-12	29.1	29.16	29.0663	29.16	34995	{}	\N	2020-10-12 15:05:38.155906
47155	GSP	2020-09-28	9.22	9.32	9.17	9.32	8615	{}	\N	2020-10-12 15:05:39.483265
47156	GSP	2020-09-29	8.96	9.05	8.96	9.045	4796	{}	\N	2020-10-12 15:05:39.494247
47157	GSP	2020-09-30	9.12	9.24	9.11	9.225	6026	{}	\N	2020-10-12 15:05:39.504099
47158	GSP	2020-10-01	9.11	9.11	8.91	8.98	19514	{}	\N	2020-10-12 15:05:39.510855
47159	GSP	2020-10-02	8.75	8.88	8.75	8.82	2424	{}	\N	2020-10-12 15:05:39.515131
47160	GSP	2020-10-05	9.25	9.25	9.16	9.16	3628	{}	\N	2020-10-12 15:05:39.518868
47161	GSP	2020-10-06	9.38	9.405	9.26	9.27	1933	{}	\N	2020-10-12 15:05:39.523494
47162	GSP	2020-10-07	9.28	9.36	9.28	9.36	284	{}	\N	2020-10-12 15:05:39.527376
47163	GSP	2020-10-08	9.46	9.56	9.46	9.49	8297	{}	\N	2020-10-12 15:05:39.5312
47164	GSP	2020-10-09	9.56	9.6	9.54	9.54	375	{}	\N	2020-10-12 15:05:39.534845
47165	GSP	2020-10-12	9.53	9.53	9.53	9.53	682	{}	\N	2020-10-12 15:05:39.538386
47166	GSY	2020-09-28	50.54	50.54	50.5	50.53	1016629	{}	\N	2020-10-12 15:05:40.778375
47167	GSY	2020-09-29	50.5335	50.54	50.52	50.53	377463	{}	\N	2020-10-12 15:05:40.789407
47168	GSY	2020-09-30	50.54	50.54	50.52	50.52	328998	{}	\N	2020-10-12 15:05:40.796585
47169	GSY	2020-10-01	50.52	50.54	50.51	50.52	699009	{}	\N	2020-10-12 15:05:40.802936
47170	GSY	2020-10-02	50.54	50.54	50.52	50.53	352674	{}	\N	2020-10-12 15:05:40.807831
47171	GSY	2020-10-05	50.54	50.54	50.52	50.54	423846	{}	\N	2020-10-12 15:05:40.812576
47172	GSY	2020-10-06	50.52	50.54	50.52	50.54	797315	{}	\N	2020-10-12 15:05:40.816951
48380	HIBB	2020-10-05	45.04	49.18	44.91	48.58	1217422	{}	\N	2020-10-12 15:08:43.078452
48381	HIBB	2020-10-06	48.8	49.1	45.155	45.86	629683	{}	\N	2020-10-12 15:08:43.082449
48382	HIBB	2020-10-07	47.02	49.9599	46.92	48.07	592779	{}	\N	2020-10-12 15:08:43.086188
48383	HIBB	2020-10-08	48.4	51.34	47.08	51.11	652406	{}	\N	2020-10-12 15:08:43.089938
48384	HIBB	2020-10-09	51.98	55.96	51.3401	55.4	965448	{}	\N	2020-10-12 15:08:43.093719
48385	HIBB	2020-10-12	55.28	55.5199	54	54.98	131642	{}	\N	2020-10-12 15:08:43.097489
48386	HIMX	2020-09-28	3.38	3.45	3.35	3.44	462991	{}	\N	2020-10-12 15:08:44.126142
48387	HIMX	2020-09-29	3.44	3.4668	3.37	3.43	448079	{}	\N	2020-10-12 15:08:44.136685
48388	HIMX	2020-09-30	3.46	3.62	3.445	3.56	972828	{}	\N	2020-10-12 15:08:44.143183
48389	HIMX	2020-10-01	3.61	3.65	3.54	3.65	425449	{}	\N	2020-10-12 15:08:44.148676
48390	HIMX	2020-10-02	3.55	3.61	3.45	3.47	696110	{}	\N	2020-10-12 15:08:44.15329
48391	HIMX	2020-10-05	3.56	3.56	3.4	3.46	739441	{}	\N	2020-10-12 15:08:44.157533
48392	HIMX	2020-10-06	3.75	3.865	3.65	3.73	2189242	{}	\N	2020-10-12 15:08:44.161475
48393	HIMX	2020-10-07	3.76	3.84	3.71	3.75	731954	{}	\N	2020-10-12 15:08:44.164854
48394	HIMX	2020-10-08	3.78	3.83	3.7	3.82	875763	{}	\N	2020-10-12 15:08:44.168436
48395	HIMX	2020-10-09	3.83	3.8901	3.79	3.87	651210	{}	\N	2020-10-12 15:08:44.172025
48396	HIMX	2020-10-12	3.88	3.88	3.76	3.8	397981	{}	\N	2020-10-12 15:08:44.175806
48397	HLG	2020-09-28	48.22	49.3	48.2	48.51	7968	{}	\N	2020-10-12 15:08:45.40293
48398	HLG	2020-09-29	48.5	49.05	48.32	48.53	8080	{}	\N	2020-10-12 15:08:45.41225
48399	HLG	2020-09-30	48.7873	49.65	45.61	49.3	7263	{}	\N	2020-10-12 15:08:45.418915
48400	HLG	2020-10-01	49.9899	49.9899	49.234	49.6999	3403	{}	\N	2020-10-12 15:08:45.424343
48401	HLG	2020-10-02	47.67	51	47.67	51	5882	{}	\N	2020-10-12 15:08:45.429243
48402	HLG	2020-10-05	50.55	52.22	50.43	51.04	6153	{}	\N	2020-10-12 15:08:45.434015
48403	HLG	2020-10-06	51.86	51.86	50.78	51.5	5723	{}	\N	2020-10-12 15:08:45.438147
48404	HLG	2020-10-07	51.88	53.8	51.59	52.61	3681	{}	\N	2020-10-12 15:08:45.442347
48405	HLG	2020-10-08	53.5	54.8	52.6972	54.4999	8239	{}	\N	2020-10-12 15:08:45.446338
48406	HLG	2020-10-09	54.98	56	54.55	55.7999	3940	{}	\N	2020-10-12 15:08:45.450345
48407	HLG	2020-10-12	55.8	55.8	55.67	55.7999	1202	{}	\N	2020-10-12 15:08:45.454396
48408	HMN	2020-09-28	33.55	34.41	33.45	33.71	163847	{}	\N	2020-10-12 15:08:46.902293
48409	HMN	2020-09-29	33.61	33.89	33.05	33.5	301212	{}	\N	2020-10-12 15:08:46.912175
48410	HMN	2020-09-30	33.6	33.95	33.11	33.4	358275	{}	\N	2020-10-12 15:08:46.91874
48411	HMN	2020-10-01	33.33	33.75	33.08	33.46	299258	{}	\N	2020-10-12 15:08:46.925144
48412	HMN	2020-10-02	33.04	34.02	33.04	33.75	351231	{}	\N	2020-10-12 15:08:46.93087
48413	HMN	2020-10-05	34.07	34.77	34.07	34.64	248686	{}	\N	2020-10-12 15:08:46.935672
48414	HMN	2020-10-06	35.12	35.29	34.15	34.16	411562	{}	\N	2020-10-12 15:08:46.940086
48415	HMN	2020-10-07	34.07	34.63	33.92	34.29	556067	{}	\N	2020-10-12 15:08:46.944345
48416	HMN	2020-10-08	34.59	35.25	34.42	34.44	361097	{}	\N	2020-10-12 15:08:46.948492
48417	HMN	2020-10-09	34.71	34.97	34.135	34.24	659753	{}	\N	2020-10-12 15:08:46.952446
48418	HMN	2020-10-12	34.07	35.1	34.07	35.09	36548	{}	\N	2020-10-12 15:08:46.956587
48419	HOMB	2020-09-28	15.17	15.45	15.1	15.33	480751	{}	\N	2020-10-12 15:08:47.984983
48420	HOMB	2020-09-29	15.28	15.33	14.92	15.15	288834	{}	\N	2020-10-12 15:08:47.995214
48421	HOMB	2020-09-30	15.345	15.48	15.06	15.16	486131	{}	\N	2020-10-12 15:08:48.002567
48422	HOMB	2020-10-01	15.29	15.46	15.02	15.45	602478	{}	\N	2020-10-12 15:08:48.008599
48423	HOMB	2020-10-02	15.22	15.92	15.06	15.81	397802	{}	\N	2020-10-12 15:08:48.013846
48424	HOMB	2020-10-05	16.05	16.63	15.9855	16.58	453756	{}	\N	2020-10-12 15:08:48.018626
48425	HOMB	2020-10-06	16.88	17.37	16.5	16.5	630000	{}	\N	2020-10-12 15:08:48.023041
48426	HOMB	2020-10-07	16.7	17.18	16.64	16.93	530635	{}	\N	2020-10-12 15:08:48.027518
47173	GSY	2020-10-07	50.52	50.56	50.52	50.54	382160	{}	\N	2020-10-12 15:05:40.841963
47174	GSY	2020-10-08	50.55	50.56	50.52	50.53	475896	{}	\N	2020-10-12 15:05:40.852936
47175	GSY	2020-10-09	50.53	50.56	50.51	50.56	1325083	{}	\N	2020-10-12 15:05:40.856911
47176	GSY	2020-10-12	50.54	50.55	50.54	50.545	68473	{}	\N	2020-10-12 15:05:40.860831
47177	GTIM	2020-09-28	1.36	1.4199	1.342	1.41	15438	{}	\N	2020-10-12 15:05:42.063691
47178	GTIM	2020-09-29	1.47	1.47	1.385	1.42	17998	{}	\N	2020-10-12 15:05:42.069044
47179	GTIM	2020-09-30	1.35	1.53	1.35	1.46	7397	{}	\N	2020-10-12 15:05:42.073464
47180	GTIM	2020-10-01	1.49	1.49	1.44	1.48	4105	{}	\N	2020-10-12 15:05:42.077801
47181	GTIM	2020-10-02	1.47	1.6399	1.46	1.51	10855	{}	\N	2020-10-12 15:05:42.082523
47182	GTIM	2020-10-05	1.56	1.56	1.4001	1.48	19796	{}	\N	2020-10-12 15:05:42.087862
47183	GTIM	2020-10-06	1.48	1.55	1.4	1.5	23809	{}	\N	2020-10-12 15:05:42.093669
47184	GTIM	2020-10-07	1.55	1.72	1.51	1.55	90147	{}	\N	2020-10-12 15:05:42.098373
47185	GTIM	2020-10-08	1.61	1.66	1.52	1.64	19050	{}	\N	2020-10-12 15:05:42.102519
47186	GTIM	2020-10-09	1.69	1.69	1.495	1.61	19680	{}	\N	2020-10-12 15:05:42.106812
47187	GTIM	2020-10-12	1.54	1.62	1.54	1.61	982	{}	\N	2020-10-12 15:05:42.11124
47188	GTY	2020-09-28	25.83	26.42	25.78	26.12	142383	{}	\N	2020-10-12 15:05:43.086945
47189	GTY	2020-09-29	26.19	26.19	25.28	25.78	253751	{}	\N	2020-10-12 15:05:43.098288
47190	GTY	2020-09-30	25.93	26.26	25.74	26.01	481906	{}	\N	2020-10-12 15:05:43.104865
47191	GTY	2020-10-01	26.27	26.5	25.84	26.47	269820	{}	\N	2020-10-12 15:05:43.110259
47192	GTY	2020-10-02	25.99	26.9	25.98	26.69	344563	{}	\N	2020-10-12 15:05:43.115466
47193	GTY	2020-10-05	26.88	27.15	26.03	26.43	201066	{}	\N	2020-10-12 15:05:43.120428
47194	GTY	2020-10-06	26.55	27.02	26.16	26.36	314983	{}	\N	2020-10-12 15:05:43.12459
47195	GTY	2020-10-07	26.54	26.74	26.15	26.54	456106	{}	\N	2020-10-12 15:05:43.128694
47196	GTY	2020-10-08	26.89	27.38	26.61	26.99	206240	{}	\N	2020-10-12 15:05:43.132592
47197	GTY	2020-10-09	27.14	27.189	26.83	26.93	189756	{}	\N	2020-10-12 15:05:43.137297
47198	GTY	2020-10-12	26.85	27.0012	26.64	26.85	22784	{}	\N	2020-10-12 15:05:43.141675
47199	GUDB	2020-09-28	52.235	52.235	52.235	52.235	0	{}	\N	2020-10-12 15:05:44.745964
47200	GUDB	2020-09-29	52.28	52.28	52.28	52.28	0	{}	\N	2020-10-12 15:05:44.753643
47201	GUDB	2020-09-30	52.27	52.27	52.27	52.27	10	{}	\N	2020-10-12 15:05:44.759013
47202	GUDB	2020-10-02	52.28	52.28	52.28	52.28	0	{}	\N	2020-10-12 15:05:44.763804
47203	GUDB	2020-10-05	52.065	52.065	52.065	52.065	14	{}	\N	2020-10-12 15:05:44.768379
47204	GUDB	2020-10-06	52.26	52.26	52.26	52.26	0	{}	\N	2020-10-12 15:05:44.773018
47205	GUDB	2020-10-07	52.31	52.31	52.31	52.31	78	{}	\N	2020-10-12 15:05:44.77762
47206	GUDB	2020-10-08	52.335	52.335	52.335	52.335	25	{}	\N	2020-10-12 15:05:44.783017
47207	GUDB	2020-10-09	52.36	52.36	52.36	52.36	3	{}	\N	2020-10-12 15:05:44.787097
47208	GV	2020-09-28	4.25	4.45	4.23	4.23	69973	{}	\N	2020-10-12 15:05:46.029561
47209	GV	2020-09-29	4.2	4.4	4.19	4.19	58229	{}	\N	2020-10-12 15:05:46.035227
47210	GV	2020-09-30	4.22	4.33	4.21	4.23	68793	{}	\N	2020-10-12 15:05:46.040159
47211	GV	2020-10-01	4.27	4.4	4.25	4.26	66236	{}	\N	2020-10-12 15:05:46.044357
47212	GV	2020-10-02	4.22	4.37	4.2156	4.35	59761	{}	\N	2020-10-12 15:05:46.049063
47213	GV	2020-10-05	4.4	4.57	4.33	4.53	137132	{}	\N	2020-10-12 15:05:46.053704
47214	GV	2020-10-06	4.6	4.78	4.44	4.44	120381	{}	\N	2020-10-12 15:05:46.058163
47215	GV	2020-10-07	4.53	4.77	4.53	4.68	147416	{}	\N	2020-10-12 15:05:46.062384
47216	GV	2020-10-08	4.68	4.85	4.5183	4.66	96882	{}	\N	2020-10-12 15:05:46.066384
47217	GV	2020-10-09	4.7	4.79	4.62	4.68	78729	{}	\N	2020-10-12 15:05:46.070628
47218	GV	2020-10-12	4.73	4.73	4.64	4.64	28047	{}	\N	2020-10-12 15:05:46.074719
47219	GRPN	2020-09-28	23.49	24.31	23.2111	23.82	1189334	{}	\N	2020-10-12 15:05:47.087674
47220	GRPN	2020-09-29	23.74	23.74	21.1901	21.575	2064588	{}	\N	2020-10-12 15:05:47.097332
47221	GRPN	2020-09-30	21.92	21.975	20.375	20.41	1951897	{}	\N	2020-10-12 15:05:47.105591
47222	GRPN	2020-10-01	20.5	21.46	20.15	21.23	1167902	{}	\N	2020-10-12 15:05:47.113231
47223	GRPN	2020-10-02	20.1803	21.63	20.03	21.42	866740	{}	\N	2020-10-12 15:05:47.122236
47224	GRPN	2020-10-05	21.7	22.28	21.6171	22.06	901915	{}	\N	2020-10-12 15:05:47.130219
47225	GRPN	2020-10-06	22.24	22.3	20.14	20.3	1095761	{}	\N	2020-10-12 15:05:47.138034
47226	GRPN	2020-10-07	20.58	21.24	20.38	20.94	799468	{}	\N	2020-10-12 15:05:47.14628
47227	GRPN	2020-10-08	21.24	21.31	20.74	21.21	756341	{}	\N	2020-10-12 15:05:47.153485
47228	GRPN	2020-10-09	21.47	21.66	20.84	21.14	766237	{}	\N	2020-10-12 15:05:47.161147
47229	GRPN	2020-10-12	21.25	21.29	20.64	20.91	311643	{}	\N	2020-10-12 15:05:47.16973
47230	GSEU	2020-09-28	29.0601	29.0909	29.0601	29.0909	437	{}	\N	2020-10-12 15:05:50.64222
47231	GSEU	2020-09-29	29.221	29.221	29.12	29.1681	1405	{}	\N	2020-10-12 15:05:50.650518
47232	GSEU	2020-09-30	29.229	29.229	29.078	29.078	417	{}	\N	2020-10-12 15:05:50.65696
47233	GSEU	2020-10-01	29.22	29.27	29.22	29.27	1482	{}	\N	2020-10-12 15:05:50.662536
47234	GSEU	2020-10-02	29.2	29.3764	29.1998	29.36	1602	{}	\N	2020-10-12 15:05:50.667936
47235	GSEU	2020-10-05	29.6699	29.7807	29.6699	29.7807	297	{}	\N	2020-10-12 15:05:50.672653
47236	GSEU	2020-10-06	29.3289	29.3289	29.3289	29.3289	109	{}	\N	2020-10-12 15:05:50.677898
47237	GSEU	2020-10-07	29.6264	29.6264	29.6264	29.6264	57	{}	\N	2020-10-12 15:05:50.682835
47238	GSEU	2020-10-08	29.789	29.8396	29.781	29.8396	942	{}	\N	2020-10-12 15:05:50.687498
47239	GSEU	2020-10-09	30.1333	30.1333	30.1333	30.1333	3	{}	\N	2020-10-12 15:05:50.692974
47240	GSK	2020-09-28	37.8	38.16	37.775	37.86	2981477	{}	\N	2020-10-12 15:05:52.011642
47241	GSK	2020-09-29	37.6	37.8573	37.27	37.43	5645319	{}	\N	2020-10-12 15:05:52.017812
47242	GSK	2020-09-30	38.11	38.12	37.5	37.64	7352778	{}	\N	2020-10-12 15:05:52.023279
47243	GSK	2020-10-01	37.61	37.7	36.98	37.14	7629376	{}	\N	2020-10-12 15:05:52.029068
47244	GSK	2020-10-02	37.16	37.4762	37.085	37.24	6476339	{}	\N	2020-10-12 15:05:52.034619
47245	GSK	2020-10-05	37.38	37.795	37.37	37.69	4860514	{}	\N	2020-10-12 15:05:52.039334
47246	GSK	2020-10-06	37.47	37.52	36.72	36.81	7723942	{}	\N	2020-10-12 15:05:52.044309
47247	GSK	2020-10-07	36.64	36.76	36.5	36.59	7488555	{}	\N	2020-10-12 15:05:52.049303
47248	GSK	2020-10-08	36.92	37.25	36.87	37.16	6672451	{}	\N	2020-10-12 15:05:52.054247
47249	GSK	2020-10-09	37.58	37.89	37.48	37.57	7640820	{}	\N	2020-10-12 15:05:52.059152
47250	GSK	2020-10-12	37.7	37.7	37.42	37.5135	2611429	{}	\N	2020-10-12 15:05:52.063831
47251	GSS	2020-09-28	4.21	4.29	4.14	4.23	224930	{}	\N	2020-10-12 15:05:53.039968
47252	GSS	2020-09-29	4.28	4.46	4.26	4.38	350099	{}	\N	2020-10-12 15:05:53.047494
47253	GSS	2020-09-30	4.37	4.39	4.26	4.31	261960	{}	\N	2020-10-12 15:05:53.052589
47254	GSS	2020-10-01	4.32	4.44	4.3	4.41	247376	{}	\N	2020-10-12 15:05:53.056922
47255	GSS	2020-10-02	4.42	4.42	4.22	4.3	347316	{}	\N	2020-10-12 15:05:53.061215
47256	GSS	2020-10-05	4.34	4.435	4.27	4.34	279465	{}	\N	2020-10-12 15:05:53.065593
47257	GSS	2020-10-06	4.36	4.38	4.06	4.1	410922	{}	\N	2020-10-12 15:05:53.069915
47258	GSS	2020-10-07	4.12	4.17	4.065	4.1	414429	{}	\N	2020-10-12 15:05:53.074059
47259	GSS	2020-10-08	4.14	4.23	4.1	4.12	272519	{}	\N	2020-10-12 15:05:53.078196
47260	GSS	2020-10-09	4.25	4.59	4.25	4.57	749968	{}	\N	2020-10-12 15:05:53.081917
47261	GSS	2020-10-12	4.57	4.73	4.508	4.6018	182503	{}	\N	2020-10-12 15:05:53.085655
47262	GTE	2020-09-28	0.252	0.252	0.2355	0.2379	1287583	{}	\N	2020-10-12 15:05:56.190508
47263	GTE	2020-09-29	0.249	0.249	0.2285	0.2366	2129165	{}	\N	2020-10-12 15:05:56.199074
47264	GTE	2020-09-30	0.235	0.241799	0.23	0.2323	1446616	{}	\N	2020-10-12 15:05:56.204908
47265	GTE	2020-10-01	0.229	0.2338	0.2268	0.2276	995461	{}	\N	2020-10-12 15:05:56.210942
47266	GTE	2020-10-02	0.2251	0.233399	0.2205	0.229	1540587	{}	\N	2020-10-12 15:05:56.216011
47267	GTE	2020-10-05	0.2336	0.2364	0.2269	0.2312	1221306	{}	\N	2020-10-12 15:05:56.220954
47268	GTE	2020-10-06	0.233	0.24	0.2262	0.2271	1665430	{}	\N	2020-10-12 15:05:56.225501
47269	GTE	2020-10-07	0.2271	0.235	0.2151	0.2233	1544136	{}	\N	2020-10-12 15:05:56.229358
47270	GTE	2020-10-08	0.2251	0.2309	0.2209	0.2265	3291157	{}	\N	2020-10-12 15:05:56.232719
47271	GTE	2020-10-09	0.24	0.259	0.2277	0.2312	16090329	{}	\N	2020-10-12 15:05:56.236371
47272	GTE	2020-10-12	0.23	0.23	0.222	0.226399	1940705	{}	\N	2020-10-12 15:05:56.239824
47273	GTN	2020-09-28	14.13	14.29	13.98	14.15	656926	{}	\N	2020-10-12 15:05:57.806396
47274	GTN	2020-09-29	14.03	14.16	13.47	13.8	1819719	{}	\N	2020-10-12 15:05:57.81559
47275	GTN	2020-09-30	13.91	14.23	13.67	13.77	777970	{}	\N	2020-10-12 15:05:57.821922
47276	GTN	2020-10-01	13.77	13.93	13.353	13.63	970413	{}	\N	2020-10-12 15:05:57.827636
47277	GTN	2020-10-02	13.29	13.73	13.13	13.57	991866	{}	\N	2020-10-12 15:05:57.832612
47278	GTN	2020-10-05	13.77	14.055	13.63	13.8	1146126	{}	\N	2020-10-12 15:05:57.837539
47279	GTN	2020-10-06	13.75	14.07	13.44	13.52	1430229	{}	\N	2020-10-12 15:05:57.841894
47280	GTN	2020-10-07	13.64	13.93	13.43	13.56	1537605	{}	\N	2020-10-12 15:05:57.845828
47281	GTN	2020-10-08	13.89	14.035	13.47	13.81	1678241	{}	\N	2020-10-12 15:05:57.849768
47282	GTN	2020-10-09	14	14.15	13.58	13.59	1235933	{}	\N	2020-10-12 15:05:57.853744
47283	GTN	2020-10-12	13.6	13.88	13.36	13.7482	292832	{}	\N	2020-10-12 15:05:57.857676
47284	GTYH	2020-09-28	2.39	2.61	2.3801	2.6	27718	{}	\N	2020-10-12 15:05:58.889573
47285	GTYH	2020-09-29	2.6	2.65	2.55	2.64	28534	{}	\N	2020-10-12 15:05:58.897123
47286	GTYH	2020-09-30	2.68	2.77	2.62	2.65	20026	{}	\N	2020-10-12 15:05:58.901969
47287	GTYH	2020-10-01	2.58	2.7	2.4801	2.66	27107	{}	\N	2020-10-12 15:05:58.90654
47288	GTYH	2020-10-02	2.6	2.69	2.4724	2.65	31104	{}	\N	2020-10-12 15:05:58.910941
47289	GTYH	2020-10-05	2.67	2.82	2.6	2.78	41662	{}	\N	2020-10-12 15:05:58.915116
47290	GTYH	2020-10-06	2.82	3	2.56	2.58	79747	{}	\N	2020-10-12 15:05:58.919073
47291	GTYH	2020-10-07	2.58	2.75	2.58	2.71	54459	{}	\N	2020-10-12 15:05:58.923031
47292	GTYH	2020-10-08	2.78	3.15	2.7437	2.93	27548	{}	\N	2020-10-12 15:05:58.926782
47293	GTYH	2020-10-09	3.01	3.01	2.81	2.91	22530	{}	\N	2020-10-12 15:05:58.930522
47294	GTYH	2020-10-12	2.88	2.98	2.88	2.98	754	{}	\N	2020-10-12 15:05:58.934313
47295	GUSH	2020-09-28	22.74	23.78	22.16	23.1922	2566186	{}	\N	2020-10-12 15:06:00.391433
47296	GUSH	2020-09-29	22.91	23.05	20.9015	21.83	3070823	{}	\N	2020-10-12 15:06:00.399835
47297	GUSH	2020-09-30	21.89	22.45	21.18	21.46	2839554	{}	\N	2020-10-12 15:06:00.405806
47298	GUSH	2020-10-01	21	21.1817	19.7799	20.03	3709334	{}	\N	2020-10-12 15:06:00.411449
47299	GUSH	2020-10-02	18.6	21.3673	18.5165	21	3668820	{}	\N	2020-10-12 15:06:00.418167
47300	GUSH	2020-10-05	21.7965	22.9	21.4118	22.9	2769623	{}	\N	2020-10-12 15:06:00.423133
47301	GUSH	2020-10-06	23.64	24.28	21.63	21.72	3857432	{}	\N	2020-10-12 15:06:00.427723
47302	GUSH	2020-10-07	22.17	23.1189	21.89	22.99	2737807	{}	\N	2020-10-12 15:06:00.432363
47303	GUSH	2020-10-08	23.38	25	23.03	24.97	3095752	{}	\N	2020-10-12 15:06:00.437426
47304	GUSH	2020-10-09	25.36	25.8	23.73	24.13	3068686	{}	\N	2020-10-12 15:06:00.442114
47305	GUSH	2020-10-12	23.98	24	23.16	23.71	810133	{}	\N	2020-10-12 15:06:00.446916
47306	GVP	2020-09-28	1.03	1.04	1	1.03	17745	{}	\N	2020-10-12 15:06:01.460118
47307	GVP	2020-09-29	1.03	1.08	1.03	1.05	23337	{}	\N	2020-10-12 15:06:01.470808
47308	GVP	2020-09-30	1.05	1.09	1.01	1.04	42628	{}	\N	2020-10-12 15:06:01.47772
47309	GVP	2020-10-01	1.04	1.06	1.01	1.04	18449	{}	\N	2020-10-12 15:06:01.483976
47310	GVP	2020-10-02	1.03	1.08	1.01	1.05	12346	{}	\N	2020-10-12 15:06:01.488987
47311	GVP	2020-10-05	1.05	1.05	1	1.02	20529	{}	\N	2020-10-12 15:06:01.493761
47312	GVP	2020-10-06	1.03	1.04	1	1	39870	{}	\N	2020-10-12 15:06:01.498274
47313	GVP	2020-10-07	1	1.03	0.9951	1.01	53186	{}	\N	2020-10-12 15:06:01.502536
47314	GVP	2020-10-08	1.03	1.03	0.9922	1.02	39847	{}	\N	2020-10-12 15:06:01.506842
47315	GVP	2020-10-09	1.02	1.07	1.01	1.03	77794	{}	\N	2020-10-12 15:06:01.510838
47316	GVP	2020-10-12	1.0299	1.04	1.01	1.03	12861	{}	\N	2020-10-12 15:06:01.514823
47317	GWGH	2020-09-28	8.64	8.65	8.22	8.27	5240	{}	\N	2020-10-12 15:06:04.012418
47318	GWGH	2020-09-29	8.55	8.6	8.55	8.55	1206	{}	\N	2020-10-12 15:06:04.02328
47319	GWGH	2020-09-30	8.5553	8.6	8.4	8.6	4683	{}	\N	2020-10-12 15:06:04.031138
47320	GWGH	2020-10-01	8.51	8.75	8.4	8.69	14001	{}	\N	2020-10-12 15:06:04.037786
47321	GWGH	2020-10-02	8.54	8.68	8.54	8.68	1791	{}	\N	2020-10-12 15:06:04.043748
47322	GWGH	2020-10-05	8.59	9.12	8.55	9.12	3853	{}	\N	2020-10-12 15:06:04.0489
47323	GWGH	2020-10-06	9	9.15	8.61	8.98	18081	{}	\N	2020-10-12 15:06:04.053791
47324	GWGH	2020-10-07	9.03	9.03	8.658	9.02	3092	{}	\N	2020-10-12 15:06:04.058152
47325	GWGH	2020-10-08	8.72	8.86	8.41	8.86	1713	{}	\N	2020-10-12 15:06:04.06261
47326	GWGH	2020-10-09	9	9.095	9	9.02	4678	{}	\N	2020-10-12 15:06:04.066988
47327	GWGH	2020-10-12	8.8773	8.8773	8.8773	8.8773	451	{}	\N	2020-10-12 15:06:04.071148
47328	GXG	2020-09-28	23.66	23.82	23.5166	23.5166	1895	{}	\N	2020-10-12 15:06:05.208196
47329	GXG	2020-09-29	23.52	23.52	23.325	23.325	1401	{}	\N	2020-10-12 15:06:05.217401
47330	GXG	2020-09-30	23.39	23.5639	23.39	23.45	2692	{}	\N	2020-10-12 15:06:05.223412
47331	GXG	2020-10-01	23.32	23.3917	23.21	23.3	2891	{}	\N	2020-10-12 15:06:05.228802
47332	GXG	2020-10-02	23.4279	23.4279	23	23.32	8645	{}	\N	2020-10-12 15:06:05.233619
47333	GXG	2020-10-05	23.35	23.63	23.35	23.63	6172	{}	\N	2020-10-12 15:06:05.238206
47334	GXG	2020-10-06	24.02	24.0595	23.45	23.45	1088	{}	\N	2020-10-12 15:06:05.243743
47335	GXG	2020-10-07	23.55	23.55	23.3	23.53	3366	{}	\N	2020-10-12 15:06:05.247546
47336	GXG	2020-10-08	23.84	24.08	23.642	23.68	4676	{}	\N	2020-10-12 15:06:05.251484
47337	GXG	2020-10-09	23.96	24.34	23.96	24.29	12972	{}	\N	2020-10-12 15:06:05.255635
47338	GXG	2020-10-12	23.79	23.84	23.75	23.84	630	{}	\N	2020-10-12 15:06:05.259735
47339	GRUB	2020-09-28	72.59	72.94	71.97	72.3	1024434	{}	\N	2020-10-12 15:06:07.099234
47340	GRUB	2020-09-29	72.9	73.7	72.73	73.14	2730940	{}	\N	2020-10-12 15:06:07.108706
47341	GRUB	2020-09-30	72.76	73.41	71.8	72.33	3835089	{}	\N	2020-10-12 15:06:07.115189
47342	GRUB	2020-10-01	72.46	73.74	72.1	73.23	6153888	{}	\N	2020-10-12 15:06:07.120754
47343	GRUB	2020-10-02	72.59	73.9292	72.35	73.43	3283767	{}	\N	2020-10-12 15:06:07.125811
47344	GRUB	2020-10-05	73.2	74.63	72.99	74.42	1868698	{}	\N	2020-10-12 15:06:07.13049
47345	GRUB	2020-10-06	74.42	74.57	73	73.53	1128907	{}	\N	2020-10-12 15:06:07.134723
47346	GRUB	2020-10-07	74.13	74.75	73.96	74.52	5089561	{}	\N	2020-10-12 15:06:07.139267
47347	GRUB	2020-10-08	74.47	74.72	73.41	73.88	1668874	{}	\N	2020-10-12 15:06:07.143406
47348	GRUB	2020-10-09	74.25	75.29	74.23	74.5	1168398	{}	\N	2020-10-12 15:06:07.147546
47349	GRUB	2020-10-12	75.27	76.45	75.155	76.43	561305	{}	\N	2020-10-12 15:06:07.151398
47350	GSIT	2020-09-28	5.78	6.08	5.77	5.88	37428	{}	\N	2020-10-12 15:06:08.883587
47351	GSIT	2020-09-29	5.78	5.922	5.71	5.81	20644	{}	\N	2020-10-12 15:06:08.891419
47352	GSIT	2020-09-30	5.78	5.862	5.62	5.64	28384	{}	\N	2020-10-12 15:06:08.896301
47353	GSIT	2020-10-01	5.76	5.76	5.56	5.61	33427	{}	\N	2020-10-12 15:06:08.901533
47354	GSIT	2020-10-02	5.56	5.72	5.56	5.6	20599	{}	\N	2020-10-12 15:06:08.906805
47355	GSIT	2020-10-05	5.7	5.91	5.62	5.9	37512	{}	\N	2020-10-12 15:06:08.910736
47356	GSIT	2020-10-06	5.96	6.08	5.71	5.85	13898	{}	\N	2020-10-12 15:06:08.914732
47357	GSIT	2020-10-07	6.04	6.08	5.92	6.03	17905	{}	\N	2020-10-12 15:06:08.918684
47358	GSIT	2020-10-08	6.12	6.26	5.89	6.19	24724	{}	\N	2020-10-12 15:06:08.922809
47359	GSIT	2020-10-09	6.38	6.4	6.26	6.39	32657	{}	\N	2020-10-12 15:06:08.926782
47360	GSIT	2020-10-12	6.3767	6.3767	6.2984	6.359	1600	{}	\N	2020-10-12 15:06:08.930829
47361	GSUM	2020-09-28	1.14	1.25	1.14	1.19	199068	{}	\N	2020-10-12 15:06:10.491382
47362	GSUM	2020-09-29	1.24	1.24	1.1773	1.19	76071	{}	\N	2020-10-12 15:06:10.500754
47363	GSUM	2020-09-30	1.1985	1.3	1.19	1.21	312298	{}	\N	2020-10-12 15:06:10.506828
47364	GSUM	2020-10-01	1.724	1.88	1.58	1.69	14838999	{}	\N	2020-10-12 15:06:10.512018
47365	GSUM	2020-10-02	1.67	1.87	1.6507	1.85	2694633	{}	\N	2020-10-12 15:06:10.516834
47366	GSUM	2020-10-05	1.85	1.85	1.74	1.83	462268	{}	\N	2020-10-12 15:06:10.521624
47367	GSUM	2020-10-06	1.81	1.83	1.77	1.79	250870	{}	\N	2020-10-12 15:06:10.525679
47368	GSUM	2020-10-07	1.78	1.82	1.72	1.82	503343	{}	\N	2020-10-12 15:06:10.529742
47369	GSUM	2020-10-08	1.79	1.81	1.74	1.8	215897	{}	\N	2020-10-12 15:06:10.53361
47370	GSUM	2020-10-09	1.77	1.8301	1.77	1.82	320468	{}	\N	2020-10-12 15:06:10.537507
47371	GSUM	2020-10-12	1.8	1.82	1.8	1.81	43184	{}	\N	2020-10-12 15:06:10.54122
47372	GTHX	2020-09-28	12.31	12.7	12.13	12.23	681728	{}	\N	2020-10-12 15:06:11.540225
47373	GTHX	2020-09-29	12.14	12.4	11.83	11.92	600237	{}	\N	2020-10-12 15:06:11.548364
47374	GTHX	2020-09-30	11.96	12.2	11.44	11.55	805202	{}	\N	2020-10-12 15:06:11.555619
47375	GTHX	2020-10-01	11.55	11.91	11.4	11.67	931886	{}	\N	2020-10-12 15:06:11.560867
47376	GTHX	2020-10-02	11.44	12.05	11.33	11.705	666721	{}	\N	2020-10-12 15:06:11.565567
47377	GTHX	2020-10-05	12.28	12.76	11.86	12.43	990480	{}	\N	2020-10-12 15:06:11.569902
47378	GTHX	2020-10-06	12.51	12.79	12.27	12.37	801761	{}	\N	2020-10-12 15:06:11.574212
47379	GTHX	2020-10-07	12.44	12.92	12.44	12.7	984515	{}	\N	2020-10-12 15:06:11.578309
47380	GTHX	2020-10-08	13.06	13.36	12.76	13.04	772586	{}	\N	2020-10-12 15:06:11.582632
47381	GTHX	2020-10-09	13.12	13.23	12.56	12.77	551469	{}	\N	2020-10-12 15:06:11.586512
47382	GTHX	2020-10-12	12.99	12.99	12.29	12.88	276359	{}	\N	2020-10-12 15:06:11.590339
47383	GURU	2020-09-28	37.75	37.87	37.5	37.86	2213	{}	\N	2020-10-12 15:06:13.445433
47384	GURU	2020-09-29	37.81	37.81	37.649	37.649	1496	{}	\N	2020-10-12 15:06:13.454136
47385	GURU	2020-09-30	37.47	38.17	37.47	38.04	3883	{}	\N	2020-10-12 15:06:13.459928
47386	GURU	2020-10-01	38.42	38.42	38.08	38.31	2482	{}	\N	2020-10-12 15:06:13.465518
47387	GURU	2020-10-02	37.56	38.5	37.56	38.18	3891	{}	\N	2020-10-12 15:06:13.469975
47388	GURU	2020-10-05	38.77	38.99	38.5401	38.935	5998	{}	\N	2020-10-12 15:06:13.474848
47389	GURU	2020-10-06	39.0381	39.16	38.52	38.535	2157	{}	\N	2020-10-12 15:06:13.479877
47390	GURU	2020-10-07	38.82	39.27	38.82	39.27	2398	{}	\N	2020-10-12 15:06:13.483897
47391	GURU	2020-10-08	39.31	39.54	39.25	39.54	3068	{}	\N	2020-10-12 15:06:13.488027
47392	GURU	2020-10-09	39.86	39.945	39.61	39.88	4330	{}	\N	2020-10-12 15:06:13.491899
47393	GURU	2020-10-12	40	40.48	39.78	39.78	1316	{}	\N	2020-10-12 15:06:13.496031
47394	GWRE	2020-09-28	105.91	107.54	105.82	106.12	257982	{}	\N	2020-10-12 15:06:14.958128
47395	GWRE	2020-09-29	106.27	107.12	105.49	105.53	501564	{}	\N	2020-10-12 15:06:14.970122
47396	GWRE	2020-09-30	105.25	107.215	103.05	104.27	1201960	{}	\N	2020-10-12 15:06:14.977817
47397	GWRE	2020-10-01	105.15	107.195	105	106.51	941486	{}	\N	2020-10-12 15:06:14.98383
47398	GWRE	2020-10-02	104.41	106.26	102.7459	103.15	808183	{}	\N	2020-10-12 15:06:14.989471
47399	GWRE	2020-10-05	103.48	103.91	102.2	102.94	670848	{}	\N	2020-10-12 15:06:14.994428
47400	GWRE	2020-10-06	103.28	106.995	103	106.4	1016963	{}	\N	2020-10-12 15:06:14.998695
47401	GWRE	2020-10-07	107.09	112.41	107.09	112.11	1333920	{}	\N	2020-10-12 15:06:15.003091
47402	GWRE	2020-10-08	114.32	115	111.5	114.28	954632	{}	\N	2020-10-12 15:06:15.007939
47403	GWRE	2020-10-09	114.54	116.75	114.54	115.47	978616	{}	\N	2020-10-12 15:06:15.013403
47404	GWRE	2020-10-12	116.53	117.89	115.49	115.91	104994	{}	\N	2020-10-12 15:06:15.018549
47405	GYLD	2020-09-28	11.051	11.14	10.69	11.08	17056	{}	\N	2020-10-12 15:06:17.200249
47406	GYLD	2020-09-29	11.23	11.23	10.89	11.13	9255	{}	\N	2020-10-12 15:06:17.209998
47407	GYLD	2020-09-30	11.26	11.285	11.13	11.1903	4195	{}	\N	2020-10-12 15:06:17.216866
47408	GYLD	2020-10-01	11.1301	11.1857	11.035	11.1857	5266	{}	\N	2020-10-12 15:06:17.223257
47409	GYLD	2020-10-02	11.25	11.25	10.96	11.17	2151	{}	\N	2020-10-12 15:06:17.22826
47410	GYLD	2020-10-05	11.13	11.33	11.13	11.33	5610	{}	\N	2020-10-12 15:06:17.233553
47411	GYLD	2020-10-06	11.5	11.5	10.52	11.22	9254	{}	\N	2020-10-12 15:06:17.2381
47412	GYLD	2020-10-07	11.19	11.3559	11.19	11.23	4096	{}	\N	2020-10-12 15:06:17.242244
47413	GYLD	2020-10-08	11.29	11.41	11.2645	11.41	3992	{}	\N	2020-10-12 15:06:17.246217
47414	GYLD	2020-10-09	11.4	11.63	11.39	11.58	12944	{}	\N	2020-10-12 15:06:17.250541
47415	GYLD	2020-10-12	11.605	11.6169	11.605	11.6169	610	{}	\N	2020-10-12 15:06:17.25473
47416	HAFC	2020-09-28	7.94	8.28	7.94	8.07	169187	{}	\N	2020-10-12 15:06:18.205232
47417	HAFC	2020-09-29	8.02	8.13	7.88	8.04	112896	{}	\N	2020-10-12 15:06:18.215511
47418	HAFC	2020-09-30	8.08	8.28	8.04	8.21	201208	{}	\N	2020-10-12 15:06:18.22131
47419	HAFC	2020-10-01	8.13	8.18	7.91	8.12	145256	{}	\N	2020-10-12 15:06:18.226718
47420	HAFC	2020-10-02	7.93	8.39	7.93	8.38	234160	{}	\N	2020-10-12 15:06:18.231968
47421	HAFC	2020-10-05	8.5	8.75	8.5	8.69	123464	{}	\N	2020-10-12 15:06:18.238479
47422	HAFC	2020-10-06	8.7	9.01	8.67	8.73	137707	{}	\N	2020-10-12 15:06:18.24338
47423	HAFC	2020-10-07	8.83	9.015	8.64	8.805	156211	{}	\N	2020-10-12 15:06:18.248041
47424	HAFC	2020-10-08	8.91	8.99	8.71	8.97	106552	{}	\N	2020-10-12 15:06:18.253472
47425	HAFC	2020-10-09	9.07	9.11	8.775	8.79	109676	{}	\N	2020-10-12 15:06:18.258429
47426	HAFC	2020-10-12	8.8	8.86	8.73	8.83	18000	{}	\N	2020-10-12 15:06:18.263125
47427	HAL	2020-09-28	12.7	12.92	12.55	12.76	16361947	{}	\N	2020-10-12 15:06:19.313981
47428	HAL	2020-09-29	12.72	12.775	12	12.07	25667722	{}	\N	2020-10-12 15:06:19.323519
47429	HAL	2020-09-30	12.18	12.47	11.95	12.05	23972458	{}	\N	2020-10-12 15:06:19.329696
47430	HAL	2020-10-01	11.86	11.9	11	11.17	52997710	{}	\N	2020-10-12 15:06:19.336533
47431	HAL	2020-10-02	10.65	11.39	10.6	11.31	34180603	{}	\N	2020-10-12 15:06:19.342211
47432	HAL	2020-10-05	11.55	11.97	11.41	11.93	27648286	{}	\N	2020-10-12 15:06:19.347365
47433	HAL	2020-10-06	12.23	12.56	11.78	11.85	37649775	{}	\N	2020-10-12 15:06:19.35214
47434	HAL	2020-10-07	11.87	12.04	11.7	11.93	25995895	{}	\N	2020-10-12 15:06:19.356354
47435	HAL	2020-10-08	12.12	12.81	12.053	12.81	27456040	{}	\N	2020-10-12 15:06:19.362208
47436	HAL	2020-10-09	12.91	12.96	12.44	12.58	26395036	{}	\N	2020-10-12 15:06:19.366461
47437	HAL	2020-10-12	12.58	12.59	12.36	12.435	6589283	{}	\N	2020-10-12 15:06:19.370568
47438	HBAN	2020-09-28	9.06	9.21	8.945	9.12	5806747	{}	\N	2020-10-12 15:06:20.320162
47439	HBAN	2020-09-29	9.0716	9.18	8.88	9.05	5646030	{}	\N	2020-10-12 15:06:20.330047
47440	HBAN	2020-09-30	9.08	9.31	9.0698	9.17	7597780	{}	\N	2020-10-12 15:06:20.336383
47441	HBAN	2020-10-01	9.29	9.29	9.06	9.16	6459009	{}	\N	2020-10-12 15:06:20.342001
47442	HBAN	2020-10-02	8.95	9.38	8.91	9.33	8360178	{}	\N	2020-10-12 15:06:20.347544
47443	HBAN	2020-10-05	9.45	9.675	9.38	9.67	6897152	{}	\N	2020-10-12 15:06:20.354078
47444	HBAN	2020-10-06	9.77	9.99	9.5	9.54	7304303	{}	\N	2020-10-12 15:06:20.358414
47445	HBAN	2020-10-07	9.68	9.99	9.68	9.88	7996021	{}	\N	2020-10-12 15:06:20.362999
47446	HBAN	2020-10-08	9.95	10.055	9.855	10.03	5523762	{}	\N	2020-10-12 15:06:20.367091
47447	HBAN	2020-10-09	10.06	10.16	9.81	9.85	7100159	{}	\N	2020-10-12 15:06:20.371536
47448	HBAN	2020-10-12	9.83	9.975	9.83	9.93	1555184	{}	\N	2020-10-12 15:06:20.375642
47449	GS	2020-09-28	197.78	201.865	197.15	199.07	3162281	{}	\N	2020-10-12 15:06:22.197854
47450	GS	2020-09-29	198.16	198.77	195.43	196.79	4599202	{}	\N	2020-10-12 15:06:22.206277
47451	GS	2020-09-30	198.68	203.08	198.36	200.97	6076699	{}	\N	2020-10-12 15:06:22.212624
47452	GS	2020-10-01	201.53	203.13	197.8738	198.55	4402288	{}	\N	2020-10-12 15:06:22.219406
47453	GS	2020-10-02	195.34	200.6999	195.29	199.9	4781390	{}	\N	2020-10-12 15:06:22.22539
47454	GS	2020-10-05	201.93	202.84	199.59	201.8	5112329	{}	\N	2020-10-12 15:06:22.230798
47455	GS	2020-10-06	204.25	206.09	199.93	201.09	5348580	{}	\N	2020-10-12 15:06:22.236544
47456	GS	2020-10-07	203.9	205	201.17	203.6	4464176	{}	\N	2020-10-12 15:06:22.243305
47457	GS	2020-10-08	205.15	208.16	204.11	207.98	4634676	{}	\N	2020-10-12 15:06:22.250253
47458	GS	2020-10-09	208.76	209.81	206.07	207.54	5958793	{}	\N	2020-10-12 15:06:22.25506
47459	GS	2020-10-12	207.41	213.6	207.41	213.38	2630109	{}	\N	2020-10-12 15:06:22.259303
47460	GSL	2020-09-28	6.07	6.3	6.07	6.25	72005	{}	\N	2020-10-12 15:06:23.486947
47461	GSL	2020-09-29	6.35	6.955	6.35	6.9	325252	{}	\N	2020-10-12 15:06:23.496089
47462	GSL	2020-09-30	7.02	7.02	6.79	6.9	134034	{}	\N	2020-10-12 15:06:23.50238
47463	GSL	2020-10-01	6.9	6.97	6.6156	6.9	108444	{}	\N	2020-10-12 15:06:23.508284
47464	GSL	2020-10-02	6.77	6.98	6.66	6.95	63638	{}	\N	2020-10-12 15:06:23.513713
47465	GSL	2020-10-05	7	7.5	7	7.2	120842	{}	\N	2020-10-12 15:06:23.518699
47466	GSL	2020-10-06	7.2	7.2	6.84	6.9	111874	{}	\N	2020-10-12 15:06:23.523121
47467	GSL	2020-10-07	6.92	7.3	6.92	7.22	38934	{}	\N	2020-10-12 15:06:23.527487
47468	GSL	2020-10-08	7.25	7.64	7.22	7.62	120218	{}	\N	2020-10-12 15:06:23.532139
47469	GSL	2020-10-09	7.6	7.84	7.58	7.8	99570	{}	\N	2020-10-12 15:06:23.53693
47470	GSL	2020-10-12	7.8	7.82	7.71	7.77	17574	{}	\N	2020-10-12 15:06:23.541874
47471	GSSC	2020-09-28	41.8	42.29	41.76	42.15	18004	{}	\N	2020-10-12 15:06:25.009961
47472	GSSC	2020-09-29	42.02	42.27	41.67	42.05	19776	{}	\N	2020-10-12 15:06:25.020489
47473	GSSC	2020-09-30	42.34	42.54	41.8	42.03	22749	{}	\N	2020-10-12 15:06:25.027323
47474	GSSC	2020-10-01	42.28	42.94	41.98	42.94	22768	{}	\N	2020-10-12 15:06:25.034464
47475	GSSC	2020-10-02	42.46	43.21	42.22	43.16	13747	{}	\N	2020-10-12 15:06:25.040306
47476	GSSC	2020-10-05	43.39	44.27	43.39	44.27	16427	{}	\N	2020-10-12 15:06:25.045316
47477	GSSC	2020-10-06	44.51	45.31	44.07	44.21	25262	{}	\N	2020-10-12 15:06:25.050009
47478	GSSC	2020-10-07	44.97	45.09	44.6701	45.08	14688	{}	\N	2020-10-12 15:06:25.054802
47479	GSSC	2020-10-08	45.53	45.64	45.23	45.61	18644	{}	\N	2020-10-12 15:06:25.059416
47480	GSSC	2020-10-09	46.055	46.06	45.5737	45.91	23051	{}	\N	2020-10-12 15:06:25.063804
47481	GSSC	2020-10-12	46.49	46.49	45.6007	46	6402	{}	\N	2020-10-12 15:06:25.068839
47482	GTES	2020-09-28	10.98	11.25	10.98	11.08	122399	{}	\N	2020-10-12 15:06:26.486182
47483	GTES	2020-09-29	10.98	11.2	10.92	11.13	319877	{}	\N	2020-10-12 15:06:26.494808
47484	GTES	2020-09-30	11.08	11.52	10.99	11.12	423141	{}	\N	2020-10-12 15:06:26.500615
47485	GTES	2020-10-01	11.17	11.3	10.86	10.98	254331	{}	\N	2020-10-12 15:06:26.506412
47486	GTES	2020-10-02	10.66	11.34	10.66	11.22	219812	{}	\N	2020-10-12 15:06:26.511928
47487	GTES	2020-10-05	11.34	11.95	11.34	11.71	396313	{}	\N	2020-10-12 15:06:26.516808
47488	GTES	2020-10-06	11.74	11.88	11.5	11.53	375587	{}	\N	2020-10-12 15:06:26.521915
47489	GTES	2020-10-07	11.69	12.09	11.48	11.94	601693	{}	\N	2020-10-12 15:06:26.526627
47490	GTES	2020-10-08	12.06	12.36	11.85	12.05	443068	{}	\N	2020-10-12 15:06:26.531209
47491	GTES	2020-10-09	12.1	12.29	11.8	11.81	216387	{}	\N	2020-10-12 15:06:26.535774
47492	GTES	2020-10-12	11.86	12.19	11.84	12.04	49272	{}	\N	2020-10-12 15:06:26.540329
47493	GTS	2020-09-28	17.94	18.17	17.67	17.71	91081	{}	\N	2020-10-12 15:06:27.493946
47494	GTS	2020-09-29	17.75	17.97	17.42	17.87	205984	{}	\N	2020-10-12 15:06:27.50507
47495	GTS	2020-09-30	17.77	18.14	17.73	17.87	252703	{}	\N	2020-10-12 15:06:27.512688
47496	GTS	2020-10-01	17.94	18.73	17.55	18.53	694911	{}	\N	2020-10-12 15:06:27.518974
47497	GTS	2020-10-02	18.19	19.035	18.19	18.89	175670	{}	\N	2020-10-12 15:06:27.524866
47498	GTS	2020-10-05	19.06	19.93	18.83	19.72	177418	{}	\N	2020-10-12 15:06:27.529999
47499	GTS	2020-10-06	19.99	20.255	19.5	19.6	158975	{}	\N	2020-10-12 15:06:27.53466
47500	GTS	2020-10-07	19.85	20.27	19.74	20.21	183994	{}	\N	2020-10-12 15:06:27.539302
47501	GTS	2020-10-08	20.46	20.8	20.38	20.49	139119	{}	\N	2020-10-12 15:06:27.54378
47502	GTS	2020-10-09	20.76	20.76	20.43	20.5	96006	{}	\N	2020-10-12 15:06:27.548412
47503	GTS	2020-10-12	20.5	20.86	20.49	20.81	32054	{}	\N	2020-10-12 15:06:27.553175
47504	GUT	2020-09-28	7.7	7.8	7.7	7.75	86241	{}	\N	2020-10-12 15:06:29.566796
47505	GUT	2020-09-29	7.7638	7.85	7.56	7.58	128325	{}	\N	2020-10-12 15:06:29.578558
47506	GUT	2020-09-30	7.59	7.749	7.56	7.56	168328	{}	\N	2020-10-12 15:06:29.584972
47507	GUT	2020-10-01	7.62	7.75	7.61	7.73	119244	{}	\N	2020-10-12 15:06:29.590555
47508	GUT	2020-10-02	7.55	7.72	7.52	7.68	226192	{}	\N	2020-10-12 15:06:29.595939
47509	GUT	2020-10-05	7.72	7.74	7.6	7.64	123482	{}	\N	2020-10-12 15:06:29.601138
47510	GUT	2020-10-06	7.65	7.7173	7.6	7.6	158212	{}	\N	2020-10-12 15:06:29.605509
47511	GUT	2020-10-07	7.61	7.67	7.55	7.5899	219330	{}	\N	2020-10-12 15:06:29.610141
47512	GUT	2020-10-08	7.6	7.6819	7.6	7.64	106423	{}	\N	2020-10-12 15:06:29.614152
47513	GUT	2020-10-09	7.66	7.74	7.55	7.6	302538	{}	\N	2020-10-12 15:06:29.617902
47514	GUT	2020-10-12	7.61	7.6499	7.61	7.64	31454	{}	\N	2020-10-12 15:06:29.622007
47515	GWPH	2020-09-28	101.14	101.7261	98.8	99.5	384414	{}	\N	2020-10-12 15:06:31.328567
47516	GWPH	2020-09-29	98.79	99.52	96.8812	97.82	384968	{}	\N	2020-10-12 15:06:31.338948
47517	GWPH	2020-09-30	97.56	99.2509	97.01	97.35	297711	{}	\N	2020-10-12 15:06:31.345574
47518	GWPH	2020-10-01	97.91	98.8	95.6	96.55	299948	{}	\N	2020-10-12 15:06:31.351647
47519	GWPH	2020-10-02	95.28	96.85	92.55	92.67	373717	{}	\N	2020-10-12 15:06:31.357431
47520	GWPH	2020-10-05	95.98	96.46	93.11	94.7	351158	{}	\N	2020-10-12 15:06:31.364609
47521	GWPH	2020-10-06	94.71	98.46	94.71	96.15	466659	{}	\N	2020-10-12 15:06:31.368942
47522	GWPH	2020-10-07	96.41	98.175	96.1212	97.3	230938	{}	\N	2020-10-12 15:06:31.372954
47523	GWPH	2020-10-08	97.86	99.3	96.74	97.11	340055	{}	\N	2020-10-12 15:06:31.377278
47524	GWPH	2020-10-09	97.37	97.95	92.89	94.63	686419	{}	\N	2020-10-12 15:06:31.38186
47525	GWPH	2020-10-12	95.76	96.7	94.3843	96.3756	79965	{}	\N	2020-10-12 15:06:31.386047
47526	GWRS	2020-09-28	11	11.02	10.67	10.94	12344	{}	\N	2020-10-12 15:06:32.399192
47527	GWRS	2020-09-29	10.95	11.15	10.75	10.97	4700	{}	\N	2020-10-12 15:06:32.410688
47528	GWRS	2020-09-30	11.08	11.08	10.73	10.78	8779	{}	\N	2020-10-12 15:06:32.417989
47529	GWRS	2020-10-01	10.75	11	10.685	10.86	15303	{}	\N	2020-10-12 15:06:32.423572
47530	GWRS	2020-10-02	10.68	10.92	10.63	10.63	9579	{}	\N	2020-10-12 15:06:32.428791
47531	GWRS	2020-10-05	10.93	11.0199	10.7282	10.93	11789	{}	\N	2020-10-12 15:06:32.433415
47532	GWRS	2020-10-06	10.97	10.975	10.75	10.85	10309	{}	\N	2020-10-12 15:06:32.437957
47533	GWRS	2020-10-07	11.02	11.225	10.82	11.09	12479	{}	\N	2020-10-12 15:06:32.442342
47534	GWRS	2020-10-08	11.19	11.23	10.82	11.02	10046	{}	\N	2020-10-12 15:06:32.446786
47535	GWRS	2020-10-09	11.22	11.24	11.06	11.13	6626	{}	\N	2020-10-12 15:06:32.451329
47536	GWRS	2020-10-12	11.21	11.25	11.2	11.2	3553	{}	\N	2020-10-12 15:06:32.45561
47537	GYC	2020-09-28	24	24.55	24	24.41	1652	{}	\N	2020-10-12 15:06:33.394753
47538	GYC	2020-09-29	24.36	24.4	24.35	24.35	1600	{}	\N	2020-10-12 15:06:33.402586
47539	GYC	2020-09-30	24.1717	24.75	24.17	24.75	3198	{}	\N	2020-10-12 15:06:33.408415
47540	GYC	2020-10-01	24.25	24.35	24.25	24.35	3272	{}	\N	2020-10-12 15:06:33.414048
47541	GYC	2020-10-02	24.29	24.55	23.81	24.07	8272	{}	\N	2020-10-12 15:06:33.419575
47542	GYC	2020-10-05	24.66	24.66	24.19	24.2	2010	{}	\N	2020-10-12 15:06:33.425194
47543	GYC	2020-10-06	23.9744	24.55	23.86	23.91	6186	{}	\N	2020-10-12 15:06:33.4299
47544	GYC	2020-10-07	24.02	24.5494	24.02	24.5	9796	{}	\N	2020-10-12 15:06:33.434503
47545	GYC	2020-10-08	24.54	24.55	24.18	24.18	2352	{}	\N	2020-10-12 15:06:33.439156
47546	GYC	2020-10-09	24.44	24.44	24.25	24.4053	3262	{}	\N	2020-10-12 15:06:33.443999
47547	GYC	2020-10-12	25.25	25.25	25.25	25.25	3240	{}	\N	2020-10-12 15:06:33.448663
47548	HA	2020-09-28	13.37	13.7806	13.13	13.38	1068023	{}	\N	2020-10-12 15:06:34.701972
47549	HA	2020-09-29	13.24	13.25	12.85	12.86	913464	{}	\N	2020-10-12 15:06:34.712007
47550	HA	2020-09-30	13.01	13.54	12.86	12.89	894620	{}	\N	2020-10-12 15:06:34.719922
47551	HA	2020-10-01	12.91	13.1	12.65	13.075	1009189	{}	\N	2020-10-12 15:06:34.72633
47552	HA	2020-10-02	12.65	13.53	12.633	13.43	991703	{}	\N	2020-10-12 15:06:34.731672
47553	HA	2020-10-05	13.64	13.65	13.17	13.33	592861	{}	\N	2020-10-12 15:06:34.736789
47554	HA	2020-10-06	13.3855	13.7	12.91	12.93	729946	{}	\N	2020-10-12 15:06:34.741312
47555	HA	2020-10-07	13.27	13.39	12.99	13.25	1095644	{}	\N	2020-10-12 15:06:34.745718
47556	HA	2020-10-08	13.4101	14.06	13.375	13.66	763268	{}	\N	2020-10-12 15:06:34.750071
47557	HA	2020-10-09	13.69	13.87	13.38	13.71	812389	{}	\N	2020-10-12 15:06:34.754728
47558	HA	2020-10-12	13.64	13.83	13.4427	13.56	237049	{}	\N	2020-10-12 15:06:34.759314
47559	HAIN	2020-09-28	33.21	33.92	33.185	33.56	733903	{}	\N	2020-10-12 15:06:37.758402
47560	HAIN	2020-09-29	33.52	33.8	33.26	33.65	530956	{}	\N	2020-10-12 15:06:37.771781
47561	HAIN	2020-09-30	33.62	34.66	33.57	34.3	627703	{}	\N	2020-10-12 15:06:37.779485
47562	HAIN	2020-10-01	34.61	35.23	34.13	35.17	696352	{}	\N	2020-10-12 15:06:37.787371
47563	HAIN	2020-10-02	35	35.8	34.325	35.63	503334	{}	\N	2020-10-12 15:06:37.794478
47564	HAIN	2020-10-05	35.52	36.89	35.474	36.77	498813	{}	\N	2020-10-12 15:06:37.800521
47565	HAIN	2020-10-06	36.83	36.9	35.62	35.68	642723	{}	\N	2020-10-12 15:06:37.805686
47566	HAIN	2020-10-07	35.9	36.26	35.68	35.84	456526	{}	\N	2020-10-12 15:06:37.810183
47567	HAIN	2020-10-08	35.92	36	35	35.21	431750	{}	\N	2020-10-12 15:06:37.813819
47568	HAIN	2020-10-09	35.25	36	35.22	35.84	337442	{}	\N	2020-10-12 15:06:37.818084
47569	HAIN	2020-10-12	35.97	36.1	35.88	36.03	35238	{}	\N	2020-10-12 15:06:37.822091
47570	HBCP	2020-09-28	24.005	24.92	24.005	24.8	10228	{}	\N	2020-10-12 15:06:39.514174
47571	HBCP	2020-09-29	25.22	25.65	24.2	24.48	4437	{}	\N	2020-10-12 15:06:39.522697
47572	HBCP	2020-09-30	24.6	24.89	24.15	24.15	4519	{}	\N	2020-10-12 15:06:39.528088
47573	HBCP	2020-10-01	24.05	24.55	24.05	24.38	11343	{}	\N	2020-10-12 15:06:39.533199
47574	HBCP	2020-10-02	24.58	25.3	24.55	25.16	5778	{}	\N	2020-10-12 15:06:39.538289
47575	HBCP	2020-10-05	25.02	26.16	25.02	25.66	8057	{}	\N	2020-10-12 15:06:39.542682
47576	HBCP	2020-10-06	26.11	27.1	25.67	25.67	12935	{}	\N	2020-10-12 15:06:39.546503
47577	HBCP	2020-10-07	26.46	26.46	26.06	26.23	12957	{}	\N	2020-10-12 15:06:39.550577
47578	HBCP	2020-10-08	26.025	26.405	24.96	25.85	15865	{}	\N	2020-10-12 15:06:39.554585
47579	HBCP	2020-10-09	25.61	26.12	25.61	25.98	6580	{}	\N	2020-10-12 15:06:39.558895
47580	HBCP	2020-10-12	25.67	25.69	25.21	25.69	5452	{}	\N	2020-10-12 15:06:39.562776
47581	GSJY	2020-09-28	33.21	33.3762	33.21	33.3762	302	{}	\N	2020-10-12 15:06:41.597493
47582	GSJY	2020-09-29	33.4276	33.4276	33.4276	33.4276	3	{}	\N	2020-10-12 15:06:41.605737
47583	GSJY	2020-09-30	33.06	33.1918	33.06	33.1918	8212	{}	\N	2020-10-12 15:06:41.612514
47584	GSJY	2020-10-01	33.08	33.1282	33.04	33.1282	6153	{}	\N	2020-10-12 15:06:41.619602
47585	GSJY	2020-10-02	32.8658	32.93	32.86	32.93	1833	{}	\N	2020-10-12 15:06:41.624501
47586	GSJY	2020-10-05	33.0999	33.17	33.08	33.09	4216	{}	\N	2020-10-12 15:06:41.629397
47587	GSJY	2020-10-06	33.25	33.25	33.0799	33.0799	791	{}	\N	2020-10-12 15:06:41.634051
47588	GSJY	2020-10-07	33.2	33.25	33.17	33.1701	2605	{}	\N	2020-10-12 15:06:41.638513
47589	GSJY	2020-10-08	33.28	33.28	33.21	33.21	384	{}	\N	2020-10-12 15:06:41.64322
47590	GSJY	2020-10-09	33.2301	33.2301	33.2208	33.2208	5731	{}	\N	2020-10-12 15:06:41.647495
47591	GSJY	2020-10-12	33.27	33.27	33.22	33.2394	7105	{}	\N	2020-10-12 15:06:41.652174
47592	GSV	2020-09-28	0.745	0.7784	0.722	0.7781	510673	{}	\N	2020-10-12 15:06:42.700539
47593	GSV	2020-09-29	0.78	0.787268	0.7708	0.7811	489784	{}	\N	2020-10-12 15:06:42.708313
47594	GSV	2020-09-30	0.7969	0.7969	0.78	0.78	335317	{}	\N	2020-10-12 15:06:42.713965
47595	GSV	2020-10-01	0.8	0.8	0.7751	0.7939	783966	{}	\N	2020-10-12 15:06:42.719964
47596	GSV	2020-10-02	0.7931	0.8	0.77	0.7899	260559	{}	\N	2020-10-12 15:06:42.725561
47597	GSV	2020-10-05	0.77	0.7866	0.7657	0.7764	490943	{}	\N	2020-10-12 15:06:42.730904
47598	GSV	2020-10-06	0.78	0.7892	0.7638	0.7718	320924	{}	\N	2020-10-12 15:06:42.73652
47599	GSV	2020-10-07	0.7793	0.7799	0.765	0.77	221674	{}	\N	2020-10-12 15:06:42.742205
47600	GSV	2020-10-08	0.77	0.78	0.7508	0.7749	448728	{}	\N	2020-10-12 15:06:42.748647
47601	GSV	2020-10-09	0.775	0.7765	0.75	0.77	805707	{}	\N	2020-10-12 15:06:42.756692
47602	GSV	2020-10-12	0.777	0.7834	0.76	0.7722	149437	{}	\N	2020-10-12 15:06:42.763152
48427	HOMB	2020-10-08	17.05	17.23	16.69	17.14	485107	{}	\N	2020-10-12 15:08:48.0318
48428	HOMB	2020-10-09	17.34	17.56	16.79	16.93	467976	{}	\N	2020-10-12 15:08:48.037327
48429	HOMB	2020-10-12	16.86	17.2	16.82	17.2	52486	{}	\N	2020-10-12 15:08:48.042116
48430	HOPE	2020-09-28	7.47	7.695	7.47	7.65	843648	{}	\N	2020-10-12 15:08:49.404855
48431	HOPE	2020-09-29	7.6	7.65	7.41	7.55	529166	{}	\N	2020-10-12 15:08:49.414761
48432	HOPE	2020-09-30	7.64	7.73	7.45	7.585	832111	{}	\N	2020-10-12 15:08:49.421095
48433	HOPE	2020-10-01	7.64	7.64	7.37	7.54	704339	{}	\N	2020-10-12 15:08:49.426549
48434	HOPE	2020-10-02	7.4	7.77	7.4	7.74	727629	{}	\N	2020-10-12 15:08:49.431302
48435	HOPE	2020-10-05	7.87	8.04	7.84	8	550707	{}	\N	2020-10-12 15:08:49.436203
48436	HOPE	2020-10-06	8.13	8.38	8.02	8.04	625494	{}	\N	2020-10-12 15:08:49.440701
48437	HOPE	2020-10-07	8.1	8.29	8.025	8.12	807019	{}	\N	2020-10-12 15:08:49.445049
48438	HOPE	2020-10-08	8.2	8.32	8.14	8.26	388232	{}	\N	2020-10-12 15:08:49.448993
48439	HOPE	2020-10-09	8.33	8.36	8.05	8.09	683300	{}	\N	2020-10-12 15:08:49.45311
48440	HOPE	2020-10-12	8.05	8.16	7.92	8.145	129375	{}	\N	2020-10-12 15:08:49.457433
48441	HPS	2020-09-28	15.95	16.098	15.897	15.98	69902	{}	\N	2020-10-12 15:08:51.025053
48442	HPS	2020-09-29	15.99	16.04	15.91	16	110236	{}	\N	2020-10-12 15:08:51.037606
48443	HPS	2020-09-30	16.11	16.14	15.91	15.98	204289	{}	\N	2020-10-12 15:08:51.045019
48444	HPS	2020-10-01	16.05	16.07	15.91	16.07	94097	{}	\N	2020-10-12 15:08:51.050402
48445	HPS	2020-10-02	15.97	16.0934	15.92	16.03	71703	{}	\N	2020-10-12 15:08:51.054357
48446	HPS	2020-10-05	16.01	16.29	16.01	16.24	121674	{}	\N	2020-10-12 15:08:51.058863
47603	GTO	2020-09-28	57.91	57.91	57.86	57.89	51985	{}	\N	2020-10-12 15:06:43.838704
47604	GTO	2020-09-29	57.93	57.97	57.86	57.94	45796	{}	\N	2020-10-12 15:06:43.852022
47605	GTO	2020-09-30	57.92	57.92	57.73	57.74	44172	{}	\N	2020-10-12 15:06:43.858476
47606	GTO	2020-10-01	57.78	57.933	57.7401	57.88	62427	{}	\N	2020-10-12 15:06:43.86406
47607	GTO	2020-10-02	57.82	57.9399	57.57	57.89	53990	{}	\N	2020-10-12 15:06:43.869095
47608	GTO	2020-10-05	57.8	57.81	57.69	57.69	121981	{}	\N	2020-10-12 15:06:43.873416
47609	GTO	2020-10-06	57.76	57.94	57.73	57.92	37312	{}	\N	2020-10-12 15:06:43.878379
47610	GTO	2020-10-07	57.7	57.9	57.7	57.78	103466	{}	\N	2020-10-12 15:06:43.88375
47611	GTO	2020-10-08	57.87	57.93	57.8283	57.925	47018	{}	\N	2020-10-12 15:06:43.889542
47612	GTO	2020-10-09	57.85	57.91	57.7648	57.89	81010	{}	\N	2020-10-12 15:06:43.894545
47613	GTO	2020-10-12	57.72	58.21	57.72	57.8171	8524	{}	\N	2020-10-12 15:06:43.899506
47614	GURE	2020-09-28	4.98	5.04	4.81	5.04	13530	{}	\N	2020-10-12 15:06:46.789683
47615	GURE	2020-09-29	4.8	5.02	4.8	4.99	8637	{}	\N	2020-10-12 15:06:46.80181
47616	GURE	2020-09-30	5.075	5.8	4.96	5.4999	24409	{}	\N	2020-10-12 15:06:46.808579
47617	GURE	2020-10-01	5.28	5.6201	5.22	5.48	9181	{}	\N	2020-10-12 15:06:46.815069
47618	GURE	2020-10-02	5.5	5.545	5.287	5.49	13640	{}	\N	2020-10-12 15:06:46.820408
47619	GURE	2020-10-05	5.49	5.81	5.49	5.7999	36960	{}	\N	2020-10-12 15:06:46.825195
47620	GURE	2020-10-06	5.8	5.8	5.46	5.75	2300	{}	\N	2020-10-12 15:06:46.828974
47621	GURE	2020-10-07	5.8	5.95	5.64	5.88	29833	{}	\N	2020-10-12 15:06:46.833076
47622	GURE	2020-10-08	5.9	5.93	5.55	5.675	18192	{}	\N	2020-10-12 15:06:46.837254
47623	GURE	2020-10-09	5.79	5.79	5.21	5.37	13566	{}	\N	2020-10-12 15:06:46.841176
47624	GURE	2020-10-12	5.4	5.615	5.33	5.33	1167	{}	\N	2020-10-12 15:06:46.845211
47625	GVAL	2020-09-28	17.06	17.21	17.05	17.17	5446	{}	\N	2020-10-12 15:06:48.502022
47626	GVAL	2020-09-29	17.1199	17.12	16.98	17.04	13282	{}	\N	2020-10-12 15:06:48.509602
47627	GVAL	2020-09-30	17.09	17.2722	17.05	17.23	11882	{}	\N	2020-10-12 15:06:48.51503
47628	GVAL	2020-10-01	17.16	17.3	17.1575	17.24	18274	{}	\N	2020-10-12 15:06:48.519913
47629	GVAL	2020-10-02	17.08	17.18	17.08	17.13	4634	{}	\N	2020-10-12 15:06:48.525481
47630	GVAL	2020-10-05	17.38	17.4074	17.26	17.33	73989	{}	\N	2020-10-12 15:06:48.529667
47631	GVAL	2020-10-06	17.58	17.5925	17.23	17.3151	36079	{}	\N	2020-10-12 15:06:48.533662
47632	GVAL	2020-10-07	17.51	17.5364	17.3	17.37	40247	{}	\N	2020-10-12 15:06:48.537747
47633	GVAL	2020-10-08	17.34	17.47	17.34	17.405	3602	{}	\N	2020-10-12 15:06:48.541691
47634	GVAL	2020-10-09	17.48	17.5231	17.439	17.47	8504	{}	\N	2020-10-12 15:06:48.545613
47635	GVAL	2020-10-12	17.55	17.55	17.47	17.481	8848	{}	\N	2020-10-12 15:06:48.549521
47636	GXC	2020-09-28	116.18	116.39	115.76	116.38	17172	{}	\N	2020-10-12 15:06:49.563597
47637	GXC	2020-09-29	116.1	116.45	115.9268	116.22	17251	{}	\N	2020-10-12 15:06:49.574877
47638	GXC	2020-09-30	117.5	118.99	117.5	118.775	27473	{}	\N	2020-10-12 15:06:49.581315
47639	GXC	2020-10-01	120.08	120.08	119.3193	119.955	22950	{}	\N	2020-10-12 15:06:49.586915
47640	GXC	2020-10-02	118.19	119.41	118.19	118.51	21917	{}	\N	2020-10-12 15:06:49.592518
47641	GXC	2020-10-05	118.7	119.39	118.65	119.39	538904	{}	\N	2020-10-12 15:06:49.598056
47642	GXC	2020-10-06	119.95	121.125	119.89	120.14	26261	{}	\N	2020-10-12 15:06:49.604243
47643	GXC	2020-10-07	121.36	121.7189	120.86	121.4632	21395	{}	\N	2020-10-12 15:06:49.609602
47644	GXC	2020-10-08	121.57	122.26	121.5	122.05	24334	{}	\N	2020-10-12 15:06:49.613811
47645	GXC	2020-10-09	122	123.06	121.975	122.87	35829	{}	\N	2020-10-12 15:06:49.618093
47646	GXC	2020-10-12	124.92	125.32	124.58	125.29	18094	{}	\N	2020-10-12 15:06:49.622327
47647	GXF	2020-09-28	24.16	24.405	24.16	24.405	3332	{}	\N	2020-10-12 15:06:51.039237
47648	GXF	2020-09-29	24.585	24.86	24.475	24.77	6478	{}	\N	2020-10-12 15:06:51.048399
47649	GXF	2020-09-30	24.52	24.81	24.36	24.36	111027	{}	\N	2020-10-12 15:06:51.055605
47650	GXF	2020-10-01	24.585	24.74	24.52	24.709	1281	{}	\N	2020-10-12 15:06:51.063193
47651	GXF	2020-10-02	24.54	24.86	24.43	24.86	5695	{}	\N	2020-10-12 15:06:51.06875
47652	GXF	2020-10-05	24.69	25.189	24.68	24.95	28772	{}	\N	2020-10-12 15:06:51.073192
47653	GXF	2020-10-06	25.05	25.05	24.6923	24.6923	2106	{}	\N	2020-10-12 15:06:51.077551
47654	GXF	2020-10-07	25.037	25.037	25.037	25.037	219	{}	\N	2020-10-12 15:06:51.082779
47655	GXF	2020-10-08	25.07	25.25	25.0254	25.13	7633	{}	\N	2020-10-12 15:06:51.087587
47656	GXF	2020-10-09	25.32	25.5	25.3	25.3	4344	{}	\N	2020-10-12 15:06:51.092497
47657	GXF	2020-10-12	25.72	25.8	25.72	25.74	956	{}	\N	2020-10-12 15:06:51.097395
47658	HAE	2020-09-28	88.49	90.52	87.56	87.77	501612	{}	\N	2020-10-12 15:06:52.552554
47659	HAE	2020-09-29	87.93	88.35	86.25	86.86	655942	{}	\N	2020-10-12 15:06:52.55788
47660	HAE	2020-09-30	86.8	88.1	85.61	87.25	604506	{}	\N	2020-10-12 15:06:52.563083
47661	HAE	2020-10-01	87.74	88.76	86.95	88.75	580166	{}	\N	2020-10-12 15:06:52.567441
47662	HAE	2020-10-02	87.71	89.35	87.05	88.79	585174	{}	\N	2020-10-12 15:06:52.57142
47663	HAE	2020-10-05	89.8	92.75	89.8	92.68	835293	{}	\N	2020-10-12 15:06:52.576046
47664	HAE	2020-10-06	92.91	95.72	92.5	94.1	874125	{}	\N	2020-10-12 15:06:52.580397
47665	HAE	2020-10-07	94.96	95.54	92.52	92.57	871722	{}	\N	2020-10-12 15:06:52.584721
47666	HAE	2020-10-08	93.31	94.37	91.97	94.08	439372	{}	\N	2020-10-12 15:06:52.588761
47667	HAE	2020-10-09	95.01	95.77	93.91	95.52	353673	{}	\N	2020-10-12 15:06:52.592846
47668	HAE	2020-10-12	95.94	97.32	94.65	97.32	88233	{}	\N	2020-10-12 15:06:52.597246
47669	HALO	2020-09-28	26.12	26.38	25.73	25.93	738001	{}	\N	2020-10-12 15:06:54.468088
47670	HALO	2020-09-29	25.89	26.74	25.69	26.51	1001631	{}	\N	2020-10-12 15:06:54.477827
47671	HALO	2020-09-30	26.66	26.81	26.07	26.28	1084372	{}	\N	2020-10-12 15:06:54.484998
47672	HALO	2020-10-01	26.3	26.69	25.99	26.55	1024007	{}	\N	2020-10-12 15:06:54.490993
47673	HALO	2020-10-02	26.18	26.41	25.7	25.81	498779	{}	\N	2020-10-12 15:06:54.496081
47674	HALO	2020-10-05	26	26.8	25.93	26.64	622706	{}	\N	2020-10-12 15:06:54.50129
47675	HALO	2020-10-06	26.7	27.14	26.45	26.5	639696	{}	\N	2020-10-12 15:06:54.506162
47676	HALO	2020-10-07	26.56	27.73	26.56	27.56	748139	{}	\N	2020-10-12 15:06:54.510457
47677	HALO	2020-10-08	27.87	28.04	27.41	27.86	787696	{}	\N	2020-10-12 15:06:54.514531
47678	HALO	2020-10-09	28.16	28.695	27.61	28.55	894882	{}	\N	2020-10-12 15:06:54.518536
47679	HALO	2020-10-12	28.82	28.99	28.3	28.98	111400	{}	\N	2020-10-12 15:06:54.522604
47680	HAS	2020-09-28	78	78.65	77.29	78.25	467635	{}	\N	2020-10-12 15:06:55.844057
47681	HAS	2020-09-29	79.18	81.9	78.84	81.24	912221	{}	\N	2020-10-12 15:06:55.854147
47682	HAS	2020-09-30	84.03	85.82	82.285	82.72	1344756	{}	\N	2020-10-12 15:06:55.861175
47683	HAS	2020-10-01	82.97	84.605	82.38	82.95	720587	{}	\N	2020-10-12 15:06:55.866894
47684	HAS	2020-10-02	81.37	84.13	80.85	82.97	753483	{}	\N	2020-10-12 15:06:55.87212
47685	HAS	2020-10-05	83.2	84.58	82.835	83.09	791121	{}	\N	2020-10-12 15:06:55.877251
47686	HAS	2020-10-06	83.92	84.99	82.78	83.17	709895	{}	\N	2020-10-12 15:06:55.882104
47687	HAS	2020-10-07	84.29	85.17	83.66	84.93	503914	{}	\N	2020-10-12 15:06:55.887959
47688	HAS	2020-10-08	85.4816	88	85.02	87.96	719338	{}	\N	2020-10-12 15:06:55.892664
47689	HAS	2020-10-09	88.15	90	88.11	89.16	817974	{}	\N	2020-10-12 15:06:55.897858
47690	HAS	2020-10-12	89.44	89.66	87.41	87.835	234310	{}	\N	2020-10-12 15:06:55.902487
47691	HBANO	2020-09-28	25.788	25.8	25.71	25.71	58617	{}	\N	2020-10-12 15:06:57.247575
47692	HBANO	2020-09-29	25.79	25.8	25.6986	25.77	87114	{}	\N	2020-10-12 15:06:57.255506
47693	HBANO	2020-09-30	25.45	25.46	25.26	25.28	120361	{}	\N	2020-10-12 15:06:57.261055
47694	HBANO	2020-10-01	25.38	25.49	25.33	25.43	53296	{}	\N	2020-10-12 15:06:57.265914
47695	HBANO	2020-10-02	25.43	25.51	25.31	25.46	57041	{}	\N	2020-10-12 15:06:57.271442
47696	HBANO	2020-10-05	25.54	25.54	25.47	25.47	33490	{}	\N	2020-10-12 15:06:57.275536
47697	HBANO	2020-10-06	25.55	25.55	25.45	25.47	31023	{}	\N	2020-10-12 15:06:57.279817
47698	HBANO	2020-10-07	25.51	25.5237	25.43	25.5	46506	{}	\N	2020-10-12 15:06:57.284194
47699	HBANO	2020-10-08	25.53	25.569	25.45	25.45	47508	{}	\N	2020-10-12 15:06:57.288413
47700	HBANO	2020-10-09	25.45	25.54	25.41	25.5	36121	{}	\N	2020-10-12 15:06:57.293605
47701	HBANO	2020-10-12	25.46	25.502	25.45	25.45	8657	{}	\N	2020-10-12 15:06:57.297777
47702	HCAPZ	2020-09-28	23.755	24.1	23.71	24	7556	{}	\N	2020-10-12 15:07:02.099695
47703	HCAPZ	2020-09-29	24.04	24.25	24.01	24.095	2051	{}	\N	2020-10-12 15:07:02.106973
47704	HCAPZ	2020-09-30	24.03	24.24	24.01	24.24	3604	{}	\N	2020-10-12 15:07:02.112534
47705	HCAPZ	2020-10-01	24.1	24.1412	24.1	24.1412	413	{}	\N	2020-10-12 15:07:02.117829
47706	HCAPZ	2020-10-02	24.12	24.23	24.12	24.145	1940	{}	\N	2020-10-12 15:07:02.122743
47707	HCAPZ	2020-10-05	24.2	24.25	24.2	24.25	1030	{}	\N	2020-10-12 15:07:02.127282
47708	HCAPZ	2020-10-06	24.24	24.24	24.24	24.24	729	{}	\N	2020-10-12 15:07:02.131303
47709	HCAPZ	2020-10-07	24	24.24	24	24.18	3058	{}	\N	2020-10-12 15:07:02.135197
47710	HCAPZ	2020-10-08	23.875	24.2	23.74	24.2	2847	{}	\N	2020-10-12 15:07:02.138853
47711	HCAPZ	2020-10-12	24.44	24.44	23.75	23.75	4150	{}	\N	2020-10-12 15:07:02.142502
47712	HDG	2020-09-28	45.7	45.78	45.62	45.78	2064	{}	\N	2020-10-12 15:07:04.328376
47713	HDG	2020-09-29	45.67	45.75	45.65	45.67	1288	{}	\N	2020-10-12 15:07:04.335443
47714	HDG	2020-09-30	45.79	45.91	45.78	45.9066	17645	{}	\N	2020-10-12 15:07:04.342794
47715	HDG	2020-10-01	45.91	46.01	45.79	46.01	1880	{}	\N	2020-10-12 15:07:04.347118
47716	HDG	2020-10-02	45.98	46.07	45.87	46.07	1592	{}	\N	2020-10-12 15:07:04.351514
47717	HDG	2020-10-05	46.21	46.36	46.2	46.36	11095	{}	\N	2020-10-12 15:07:04.355704
47718	HDG	2020-10-06	46.47	46.73	46.3901	46.3901	17856	{}	\N	2020-10-12 15:07:04.363302
47719	HDG	2020-10-07	46.63	46.7599	46.63	46.7599	2716	{}	\N	2020-10-12 15:07:04.367218
47720	HDG	2020-10-08	46.8899	46.8899	46.8899	46.8899	38	{}	\N	2020-10-12 15:07:04.37075
47721	HDG	2020-10-09	46.94	47.08	46.9	47.045	2190	{}	\N	2020-10-12 15:07:04.374551
47722	HDS	2020-09-28	41.26	41.47	40.805	40.87	1203283	{}	\N	2020-10-12 15:07:05.331396
47723	HDS	2020-09-29	40.88	40.97	40.11	40.45	773621	{}	\N	2020-10-12 15:07:05.341996
47724	HDS	2020-09-30	40.71	42.08	40.52	41.24	1611934	{}	\N	2020-10-12 15:07:05.354174
47725	HDS	2020-10-01	41.6	41.98	41.04	41.68	1049774	{}	\N	2020-10-12 15:07:05.35985
47726	HDS	2020-10-02	40.85	42.29	40.85	42.08	876976	{}	\N	2020-10-12 15:07:05.36532
47727	HDS	2020-10-05	42.41	42.87	42.35	42.55	853007	{}	\N	2020-10-12 15:07:05.370009
47728	HDS	2020-10-06	42.75	42.795	41.52	41.53	955405	{}	\N	2020-10-12 15:07:05.373975
47729	HDS	2020-10-07	41.98	42.49	40.9023	42.3	665499	{}	\N	2020-10-12 15:07:05.378211
47730	HDS	2020-10-08	42.58	42.98	42.19	42.92	621420	{}	\N	2020-10-12 15:07:05.382703
47731	HDS	2020-10-09	43	43.52	42.4239	42.88	541075	{}	\N	2020-10-12 15:07:05.386947
47732	HDS	2020-10-12	43	43.39	42.8912	43.2136	138566	{}	\N	2020-10-12 15:07:05.390962
47733	HEFA	2020-09-28	27.76	27.82	27.72	27.8	233766	{}	\N	2020-10-12 15:07:06.381609
47734	HEFA	2020-09-29	27.77	27.79	27.65	27.695	370063	{}	\N	2020-10-12 15:07:06.393174
47735	HEFA	2020-09-30	27.65	27.73	27.475	27.54	1316199	{}	\N	2020-10-12 15:07:06.399738
47736	HEFA	2020-10-01	27.64	27.69	27.55	27.66	487335	{}	\N	2020-10-12 15:07:06.405073
47737	HEFA	2020-10-02	27.35	27.685	27.35	27.64	299662	{}	\N	2020-10-12 15:07:06.409446
47738	HEFA	2020-10-05	27.78	27.98	27.78	27.98	154879	{}	\N	2020-10-12 15:07:06.413589
47739	HEFA	2020-10-06	27.98	28.02	27.71	27.73	693331	{}	\N	2020-10-12 15:07:06.418031
47740	HEFA	2020-10-07	27.94	28.02	27.92	28.01	196435	{}	\N	2020-10-12 15:07:06.421574
47741	HEFA	2020-10-08	28.11	28.19	28.11	28.18	237567	{}	\N	2020-10-12 15:07:06.42509
47742	HEFA	2020-10-09	28.21	28.27	28.18	28.23	423892	{}	\N	2020-10-12 15:07:06.428845
47743	HEFA	2020-10-12	28.33	28.33	28.27	28.3133	19975	{}	\N	2020-10-12 15:07:06.432829
47744	HFWA	2020-09-28	18.24	19	17.92	18.63	120537	{}	\N	2020-10-12 15:07:09.28971
47745	HFWA	2020-09-29	18.53	18.78	18.21	18.7	110434	{}	\N	2020-10-12 15:07:09.298067
47746	HFWA	2020-09-30	18.84	19	18.36	18.39	186733	{}	\N	2020-10-12 15:07:09.303891
47747	HFWA	2020-10-01	18.34	18.55	17.985	18.28	110889	{}	\N	2020-10-12 15:07:09.30953
47748	HFWA	2020-10-02	18.08	19.04	17.85	18.97	151726	{}	\N	2020-10-12 15:07:09.315102
47749	HFWA	2020-10-05	19.21	19.73	19.19	19.68	92743	{}	\N	2020-10-12 15:07:09.319776
47750	HFWA	2020-10-06	19.99	20.61	19.76	19.94	107704	{}	\N	2020-10-12 15:07:09.32384
47751	HFWA	2020-10-07	20.17	20.33	19.805	20.18	172895	{}	\N	2020-10-12 15:07:09.327737
47752	HFWA	2020-10-08	20.47	20.74	20.29	20.61	88855	{}	\N	2020-10-12 15:07:09.331709
47753	HFWA	2020-10-09	20.8	20.8	20.32	20.44	63626	{}	\N	2020-10-12 15:07:09.335934
47754	HFWA	2020-10-12	20.33	20.76	20.33	20.64	5999	{}	\N	2020-10-12 15:07:09.340714
47755	HEEM	2020-09-28	26.7299	26.7299	26.6684	26.7187	14620	{}	\N	2020-10-12 15:07:13.198928
47756	HEEM	2020-09-29	26.6559	26.77	26.65	26.77	5698	{}	\N	2020-10-12 15:07:13.208241
47757	HEEM	2020-09-30	26.99	27.12	26.95	27.03	17448	{}	\N	2020-10-12 15:07:13.214103
47758	HEEM	2020-10-01	27.07	27.2707	27.07	27.24	45913	{}	\N	2020-10-12 15:07:13.219795
47759	HEEM	2020-10-02	26.95	27.06	26.89	26.89	24920	{}	\N	2020-10-12 15:07:13.224844
47760	HEEM	2020-10-05	27.0973	27.22	27.09	27.185	6646	{}	\N	2020-10-12 15:07:13.229737
47761	HEEM	2020-10-06	27.31	27.48	27.26	27.3072	13386	{}	\N	2020-10-12 15:07:13.233977
47762	HEEM	2020-10-07	27.5257	27.5908	27.5	27.5769	10181	{}	\N	2020-10-12 15:07:13.23843
47763	HEEM	2020-10-08	27.63	27.77	27.63	27.73	16409	{}	\N	2020-10-12 15:07:13.242467
47764	HEEM	2020-10-09	27.75	27.91	27.69	27.85	36027	{}	\N	2020-10-12 15:07:13.246572
47765	HEEM	2020-10-12	28	28.06	28	28.06	436	{}	\N	2020-10-12 15:07:13.250698
47766	HEWJ	2020-09-28	32.13	32.22	32.06	32.22	40186	{}	\N	2020-10-12 15:07:14.930655
47767	HEWJ	2020-09-29	32.29	32.35	32.24	32.27	24416	{}	\N	2020-10-12 15:07:14.938944
47768	HEWJ	2020-09-30	31.87	32.035	31.87	31.95	6822	{}	\N	2020-10-12 15:07:14.944335
47769	HEWJ	2020-10-01	31.9699	31.9814	31.92	31.95	42524	{}	\N	2020-10-12 15:07:14.949872
47770	HEWJ	2020-10-02	31.53	31.78	31.53	31.78	67154	{}	\N	2020-10-12 15:07:14.954939
47771	HEWJ	2020-10-05	31.94	32.12	31.94	32.11	77225	{}	\N	2020-10-12 15:07:14.959717
47772	HEWJ	2020-10-06	32.18	32.23	31.95	32.03	1404065	{}	\N	2020-10-12 15:07:14.963977
47773	HEWJ	2020-10-07	32.26	32.31	32.25	32.3	159065	{}	\N	2020-10-12 15:07:14.969317
47774	HEWJ	2020-10-08	32.315	32.41	32.27	32.3	223346	{}	\N	2020-10-12 15:07:14.973559
47775	HEWJ	2020-10-09	32.21	32.26	32.17	32.2	152419	{}	\N	2020-10-12 15:07:14.9778
47776	HEWJ	2020-10-12	32.12	32.168	32.0713	32.168	25185	{}	\N	2020-10-12 15:07:14.981813
47777	HHC	2020-09-28	58	59.95	57.95	58.2	354527	{}	\N	2020-10-12 15:07:16.619412
47778	HHC	2020-09-29	58	58.2	55.96	56.85	599723	{}	\N	2020-10-12 15:07:16.628706
47779	HHC	2020-09-30	57.25	58.85	56.8908	57.6	478737	{}	\N	2020-10-12 15:07:16.63577
47780	HHC	2020-10-01	58.14	59.73	57.34	59.7	906334	{}	\N	2020-10-12 15:07:16.641625
47781	HHC	2020-10-02	58.32	61.825	58.2421	61.1	1092201	{}	\N	2020-10-12 15:07:16.646365
47782	HHC	2020-10-05	61.79	61.79	59.61	60.53	753146	{}	\N	2020-10-12 15:07:16.651523
47783	HHC	2020-10-06	61	62.6199	60.1201	60.59	468291	{}	\N	2020-10-12 15:07:16.657449
47784	HHC	2020-10-07	60.97	61.45	60.15	60.69	341236	{}	\N	2020-10-12 15:07:16.661471
47785	HHC	2020-10-08	60.8	62.2981	60.31	61.96	484977	{}	\N	2020-10-12 15:07:16.665414
47786	HHC	2020-10-09	62.98	64.1123	61.38	62.58	844440	{}	\N	2020-10-12 15:07:16.669239
47787	HHC	2020-10-12	62.54	62.94	61.73	62	94162	{}	\N	2020-10-12 15:07:16.673269
47788	HII	2020-09-28	143.22	145.24	141.7	143.13	334894	{}	\N	2020-10-12 15:07:18.543803
47789	HII	2020-09-29	142.62	143.52	140.87	141.21	457498	{}	\N	2020-10-12 15:07:18.552329
47790	HII	2020-09-30	142.09	142.93	140.04	140.75	724826	{}	\N	2020-10-12 15:07:18.558012
47791	HII	2020-10-01	140.76	142.86	139.3	139.98	711752	{}	\N	2020-10-12 15:07:18.563541
47792	HII	2020-10-02	137.21	144.32	137.21	144.13	765271	{}	\N	2020-10-12 15:07:18.568641
47793	HII	2020-10-05	146.11	147.07	144.5757	146.32	678944	{}	\N	2020-10-12 15:07:18.573346
47794	HII	2020-10-06	147.91	151.28	146.58	147.13	799582	{}	\N	2020-10-12 15:07:18.577601
47795	HII	2020-10-07	148.42	150.64	147.37	147.94	629186	{}	\N	2020-10-12 15:07:18.581886
47796	HII	2020-10-08	148.78	149.7	146.15	149.64	556297	{}	\N	2020-10-12 15:07:18.585944
47797	HII	2020-10-09	150.92	151.39	149.01	149.98	802685	{}	\N	2020-10-12 15:07:18.589974
47798	HII	2020-10-12	149.98	152.27	148.35	151.17	85826	{}	\N	2020-10-12 15:07:18.594259
47799	HLI	2020-09-28	56.9161	57.33	56.2561	57.06	355053	{}	\N	2020-10-12 15:07:20.986393
47800	HLI	2020-09-29	57.16	58.215	56.89	58.04	600686	{}	\N	2020-10-12 15:07:20.999765
47801	HLI	2020-09-30	58.11	59.28	58.02	59.05	716200	{}	\N	2020-10-12 15:07:21.00767
47802	HLI	2020-10-01	59.3	59.46	58.61	59.42	550219	{}	\N	2020-10-12 15:07:21.013866
47803	HLI	2020-10-02	58	59.58	57.82	59.43	554145	{}	\N	2020-10-12 15:07:21.019046
47804	HLI	2020-10-05	59.89	60.43	59.62	60.39	614879	{}	\N	2020-10-12 15:07:21.024115
47805	HLI	2020-10-06	60.93	62.04	60.28	60.75	1038011	{}	\N	2020-10-12 15:07:21.028665
47806	HLI	2020-10-07	61.52	61.95	60.495	61.42	934433	{}	\N	2020-10-12 15:07:21.033803
47807	HLI	2020-10-08	61.67	62.85	61.23	62.26	668710	{}	\N	2020-10-12 15:07:21.038487
47808	HLI	2020-10-09	62.72	63.84	62.43	63.49	698368	{}	\N	2020-10-12 15:07:21.04272
47809	HLI	2020-10-12	63.89	64.55	63.66	64.445	93512	{}	\N	2020-10-12 15:07:21.046867
47810	HMNF	2020-09-28	13.3	13.3	13.3	13.3	599	{}	\N	2020-10-12 15:07:22.059067
47811	HMNF	2020-09-29	13.4	13.4	13.4	13.4	2257	{}	\N	2020-10-12 15:07:22.069509
47812	HMNF	2020-09-30	13.2117	13.57	13.2117	13.26	1374	{}	\N	2020-10-12 15:07:22.078191
47813	HMNF	2020-10-01	13.33	13.33	13.33	13.33	259	{}	\N	2020-10-12 15:07:22.085216
47814	HMNF	2020-10-05	13.25	13.25	13.25	13.25	333	{}	\N	2020-10-12 15:07:22.091115
47815	HMNF	2020-10-06	13.6	13.6	13.26	13.26	1637	{}	\N	2020-10-12 15:07:22.096456
47816	HMNF	2020-10-07	13.415	13.415	13.415	13.415	134	{}	\N	2020-10-12 15:07:22.102226
47817	HMNF	2020-10-08	13.27	13.5076	13.27	13.5076	1966	{}	\N	2020-10-12 15:07:22.10746
47818	HMNF	2020-10-09	13.59	13.64	13.55	13.601	4865	{}	\N	2020-10-12 15:07:22.11264
47819	HMNF	2020-10-12	13.105	13.105	13.105	13.105	309	{}	\N	2020-10-12 15:07:22.117642
47820	HNI	2020-09-28	31.57	31.69	31.2	31.38	165615	{}	\N	2020-10-12 15:07:24.094197
47821	HNI	2020-09-29	31.52	31.52	30.49	31.1	164017	{}	\N	2020-10-12 15:07:24.101449
47822	HNI	2020-09-30	31.13	31.96	31.11	31.38	296831	{}	\N	2020-10-12 15:07:24.106738
47823	HNI	2020-10-01	31.47	32.33	31.39	32.3	284585	{}	\N	2020-10-12 15:07:24.111516
47824	HNI	2020-10-02	31.63	34.16	31.63	34.07	422108	{}	\N	2020-10-12 15:07:24.116161
47825	HNI	2020-10-05	34.32	34.67	34.12	34.6	283964	{}	\N	2020-10-12 15:07:24.1202
47826	HNI	2020-10-06	35.08	35.9	34.37	34.6	287470	{}	\N	2020-10-12 15:07:24.123895
47827	HNI	2020-10-07	35.21	35.89	34.615	34.86	273658	{}	\N	2020-10-12 15:07:24.127465
47828	HNI	2020-10-08	35.41	35.42	34.41	34.83	194685	{}	\N	2020-10-12 15:07:24.131256
47829	HNI	2020-10-09	35.14	35.2	34.43	34.95	154514	{}	\N	2020-10-12 15:07:24.134664
47830	HNI	2020-10-12	35.06	35.46	34.62	35.395	16692	{}	\N	2020-10-12 15:07:24.138287
47831	HOLX	2020-09-28	65.73	66.61	65.1	65.77	1543828	{}	\N	2020-10-12 15:07:26.411096
47832	HOLX	2020-09-29	65.82	66.73	65.49	66.12	1194539	{}	\N	2020-10-12 15:07:26.419547
47833	HOLX	2020-09-30	66.04	67.45	65.56	66.47	1565325	{}	\N	2020-10-12 15:07:26.425254
47834	HOLX	2020-10-01	67	67.86	65.97	66.35	1408188	{}	\N	2020-10-12 15:07:26.430758
47835	HOLX	2020-10-02	65.63	67.31	65.6	66.88	1018843	{}	\N	2020-10-12 15:07:26.435595
47836	HOLX	2020-10-05	67.56	68.44	67.46	67.9	1338974	{}	\N	2020-10-12 15:07:26.440041
47837	HOLX	2020-10-06	68.18	68.8	67.02	67.29	1196863	{}	\N	2020-10-12 15:07:26.444232
47838	HOLX	2020-10-07	68.13	69.31	67.81	68.93	1360846	{}	\N	2020-10-12 15:07:26.448058
47839	HOLX	2020-10-08	69.32	69.4399	68.02	68.29	1613442	{}	\N	2020-10-12 15:07:26.451479
47840	HOLX	2020-10-09	69.03	69.705	68.06	69.39	1500526	{}	\N	2020-10-12 15:07:26.455178
47841	HOLX	2020-10-12	69.54	69.71	68.65	69.3	250637	{}	\N	2020-10-12 15:07:26.459391
47842	HOV	2020-09-28	30.0365	32.82	29.8187	32.41	210102	{}	\N	2020-10-12 15:07:27.823183
47843	HOV	2020-09-29	31.87	32.04	30.3	30.85	207090	{}	\N	2020-10-12 15:07:27.833084
47844	HOV	2020-09-30	30.87	33.2599	30.6255	32.52	257488	{}	\N	2020-10-12 15:07:27.839661
47845	HOV	2020-10-01	32.76	34.66	32.31	34.51	308554	{}	\N	2020-10-12 15:07:27.845328
47846	HOV	2020-10-02	33.34	36.7155	32.7101	35.91	360599	{}	\N	2020-10-12 15:07:27.850206
47847	HOV	2020-10-05	36.83	37.99	34.3329	34.77	361053	{}	\N	2020-10-12 15:07:27.854909
47848	HOV	2020-10-06	35.61	35.69	33.2001	33.56	297653	{}	\N	2020-10-12 15:07:27.859213
47849	HOV	2020-10-07	34.5	35.4672	34.2401	34.81	142219	{}	\N	2020-10-12 15:07:27.863513
47850	HOV	2020-10-08	35.55	37.5	35.1372	37.13	208996	{}	\N	2020-10-12 15:07:27.86738
47851	HOV	2020-10-09	37.79	38.1799	35.81	36.54	160745	{}	\N	2020-10-12 15:07:27.871301
47852	HOV	2020-10-12	37.1	37.1	36.02	36.68	46326	{}	\N	2020-10-12 15:07:27.875162
47853	HQL	2020-09-28	17.48	17.49	17.32	17.47	35960	{}	\N	2020-10-12 15:07:29.178463
47854	HQL	2020-09-29	17.51	17.6	17.42	17.53	79142	{}	\N	2020-10-12 15:07:29.187719
47855	HQL	2020-09-30	17.53	17.81	17.51	17.58	136072	{}	\N	2020-10-12 15:07:29.194035
47856	HQL	2020-10-01	17.68	17.75	17.5877	17.66	132238	{}	\N	2020-10-12 15:07:29.20033
47857	HQL	2020-10-02	17.59	17.65	17.31	17.31	196249	{}	\N	2020-10-12 15:07:29.205553
47858	HQL	2020-10-05	17.54	18	17.54	17.95	231706	{}	\N	2020-10-12 15:07:29.210128
47859	HQL	2020-10-06	18.02	18.17	17.864	17.99	142790	{}	\N	2020-10-12 15:07:29.214547
47860	HQL	2020-10-07	18.04	18.37	18.04	18.31	238530	{}	\N	2020-10-12 15:07:29.218684
47861	HQL	2020-10-08	18.55	18.59	18.19	18.5	225336	{}	\N	2020-10-12 15:07:29.222714
47862	HQL	2020-10-09	18.59	18.64	18.42	18.42	245745	{}	\N	2020-10-12 15:07:29.226691
47863	HQL	2020-10-12	18.63	18.63	18.455	18.54	59546	{}	\N	2020-10-12 15:07:29.230639
47864	HDAW	2020-09-28	20.94	21.01	20.94	20.9479	939	{}	\N	2020-10-12 15:07:30.48315
47865	HDAW	2020-09-29	20.95	20.95	20.8732	20.8732	685	{}	\N	2020-10-12 15:07:30.492546
47866	HDAW	2020-09-30	21	21.0595	20.625	20.625	6008	{}	\N	2020-10-12 15:07:30.499505
47867	HDAW	2020-10-01	20.9281	20.95	20.74	20.92	10728	{}	\N	2020-10-12 15:07:30.50537
47868	HDAW	2020-10-02	20.84	20.9568	20.72	20.9568	1989	{}	\N	2020-10-12 15:07:30.510578
47869	HDAW	2020-10-05	21.1737	21.2231	21.1737	21.2231	1237	{}	\N	2020-10-12 15:07:30.515516
47870	HDAW	2020-10-06	21.3	21.3	21.1313	21.1313	355	{}	\N	2020-10-12 15:07:30.519669
47871	HDAW	2020-10-07	21.25	21.25	21.2351	21.2351	577	{}	\N	2020-10-12 15:07:30.52379
47872	HDAW	2020-10-08	21.331	21.378	21.331	21.34	3518	{}	\N	2020-10-12 15:07:30.527781
47873	HDAW	2020-10-09	21.5188	21.55	21.4778	21.4778	2790	{}	\N	2020-10-12 15:07:30.531791
47874	HDAW	2020-10-12	21.4901	21.5638	21.44	21.5638	9966	{}	\N	2020-10-12 15:07:30.535869
47875	HEES	2020-09-28	19.28	20.38	19.28	19.77	298739	{}	\N	2020-10-12 15:07:32.494658
47876	HEES	2020-09-29	19.64	20.205	19.16	19.3	282438	{}	\N	2020-10-12 15:07:32.507061
47877	HEES	2020-09-30	19.51	20.07	19.47	19.66	306552	{}	\N	2020-10-12 15:07:32.51455
47878	HEES	2020-10-01	19.54	20.27	19.45	20.2	254443	{}	\N	2020-10-12 15:07:32.520178
47879	HEES	2020-10-02	19.65	21.89	19.51	21.79	290526	{}	\N	2020-10-12 15:07:32.525154
47880	HEES	2020-10-05	21.96	23.34	21.94	23.26	357753	{}	\N	2020-10-12 15:07:32.53006
47881	HEES	2020-10-06	23.62	23.7499	22.52	22.59	370349	{}	\N	2020-10-12 15:07:32.53465
47882	HEES	2020-10-07	22.96	23.74	22.77	23.17	320139	{}	\N	2020-10-12 15:07:32.538784
47883	HEES	2020-10-08	23.54	23.8	23.01	23.57	169411	{}	\N	2020-10-12 15:07:32.542764
47884	HEES	2020-10-09	23.75	23.93	22.79	22.87	281485	{}	\N	2020-10-12 15:07:32.546758
47885	HEES	2020-10-12	22.86	23.28	22.56	23.08	78244	{}	\N	2020-10-12 15:07:32.550564
47886	HEWG	2020-09-28	27.93	28.03	27.88	27.99	73806	{}	\N	2020-10-12 15:07:33.464583
47887	HEWG	2020-09-29	27.9601	27.99	27.84	27.94	62365	{}	\N	2020-10-12 15:07:33.472675
47888	HEWG	2020-09-30	27.9	28.03	27.68	27.76	312401	{}	\N	2020-10-12 15:07:33.478735
47889	HEWG	2020-10-01	27.88	27.88	27.69	27.85	253591	{}	\N	2020-10-12 15:07:33.485889
47890	HEWG	2020-10-02	27.49	27.84	27.49	27.81	463139	{}	\N	2020-10-12 15:07:33.490978
47891	HEWG	2020-10-05	28	28.22	27.98	28.22	63085	{}	\N	2020-10-12 15:07:33.49575
47892	HEWG	2020-10-06	28.26	28.3	27.94	27.99	173713	{}	\N	2020-10-12 15:07:33.500063
47893	HEWG	2020-10-07	28.1	28.26	28.06	28.22	19589	{}	\N	2020-10-12 15:07:33.504319
47894	HEWG	2020-10-08	28.47	28.58	28.43	28.52	57622	{}	\N	2020-10-12 15:07:33.508347
47895	HEWG	2020-10-09	28.52	28.55	28.42	28.5153	244624	{}	\N	2020-10-12 15:07:33.512222
47896	HEWG	2020-10-12	28.62	28.69	28.6	28.69	19604	{}	\N	2020-10-12 15:07:33.516418
47897	HFBL	2020-09-28	23.5	23.5	23.375	23.3832	711	{}	\N	2020-10-12 15:07:34.539547
47898	HFBL	2020-09-30	23.05	23.05	23.05	23.05	1097	{}	\N	2020-10-12 15:07:34.5487
47899	HFBL	2020-10-05	23	23	23	23	209	{}	\N	2020-10-12 15:07:34.554902
47900	HFBL	2020-10-06	22.95	22.95	22.62	22.62	1799	{}	\N	2020-10-12 15:07:34.560187
47901	HFBL	2020-10-07	22.95	23.3	22.95	23.2	1243	{}	\N	2020-10-12 15:07:34.564831
47902	HFBL	2020-10-08	22.81	22.8117	22.81	22.81	1497	{}	\N	2020-10-12 15:07:34.569449
47903	HFBL	2020-10-12	24.4815	24.4815	23.575	23.575	1066	{}	\N	2020-10-12 15:07:34.574266
47904	HIE	2020-09-28	6.07	6.14	6.03	6.03	72305	{}	\N	2020-10-12 15:07:35.552913
47905	HIE	2020-09-29	6.02	6.09	6.02	6.04	102836	{}	\N	2020-10-12 15:07:35.563736
47906	HIE	2020-09-30	6.11	6.21	6.07	6.16	198549	{}	\N	2020-10-12 15:07:35.570546
47907	HIE	2020-10-01	6.15	6.22	6.15	6.18	177594	{}	\N	2020-10-12 15:07:35.576852
47908	HIE	2020-10-02	6.04	6.2207	6.01	6.08	179222	{}	\N	2020-10-12 15:07:35.581969
47909	HIE	2020-10-05	6.12	6.24	6.12	6.21	123048	{}	\N	2020-10-12 15:07:35.587202
47910	HIE	2020-10-06	6.24	6.3696	6.22	6.22	95331	{}	\N	2020-10-12 15:07:35.591531
47911	HIE	2020-10-07	6.22	6.3378	6.22	6.29	193380	{}	\N	2020-10-12 15:07:35.595518
47912	HIE	2020-10-08	6.32	6.33	6.3	6.31	185122	{}	\N	2020-10-12 15:07:35.599376
47913	HIE	2020-10-09	6.35	6.41	6.33	6.33	362206	{}	\N	2020-10-12 15:07:35.603497
47914	HIE	2020-10-12	6.33	6.37	6.31	6.33	237552	{}	\N	2020-10-12 15:07:35.607775
47915	HIPS	2020-09-28	11.8031	11.92	11.8031	11.92	2271	{}	\N	2020-10-12 15:07:36.91897
47916	HIPS	2020-09-29	11.82	11.88	11.7201	11.81	12353	{}	\N	2020-10-12 15:07:36.928215
47917	HIPS	2020-09-30	11.9	11.9	11.795	11.81	3482	{}	\N	2020-10-12 15:07:36.934844
47918	HIPS	2020-10-01	11.82	11.9	11.8147	11.9	3150	{}	\N	2020-10-12 15:07:36.940685
47919	HIPS	2020-10-02	11.94	11.9608	11.7545	11.9608	11227	{}	\N	2020-10-12 15:07:36.945864
47920	HIPS	2020-10-05	12.08	12.08	12.01	12.04	4837	{}	\N	2020-10-12 15:07:36.950588
47921	HIPS	2020-10-06	12.0941	12.135	11.975	11.975	6228	{}	\N	2020-10-12 15:07:36.954633
47922	HIPS	2020-10-07	12.02	12.06	11.9801	12.04	6394	{}	\N	2020-10-12 15:07:36.958708
47923	HIPS	2020-10-08	12.12	12.23	12.12	12.23	3372	{}	\N	2020-10-12 15:07:36.963067
47924	HIPS	2020-10-09	12.3	12.3	12.1602	12.23	18701	{}	\N	2020-10-12 15:07:36.967413
47925	HIPS	2020-10-12	12.3	12.3	12.1817	12.21	3826	{}	\N	2020-10-12 15:07:36.971774
47926	HLF	2020-09-28	48.52	48.54	47.05	47.41	1027665	{}	\N	2020-10-12 15:07:38.309833
47927	HLF	2020-09-29	47.6323	47.67	46.085	46.11	2682702	{}	\N	2020-10-12 15:07:38.316652
47928	HLF	2020-09-30	46.08	47.09	46.08	46.65	1940407	{}	\N	2020-10-12 15:07:38.321979
47929	HLF	2020-10-01	46.75	48	46.63	47.89	1497074	{}	\N	2020-10-12 15:07:38.327187
47930	HLF	2020-10-02	47.63	49.68	47.465	49.2	1753164	{}	\N	2020-10-12 15:07:38.332422
47931	HLF	2020-10-05	49.53	50.295	49.26	50.19	1626875	{}	\N	2020-10-12 15:07:38.33687
47932	HLF	2020-10-06	50.2	50.975	49.79	50.26	1323493	{}	\N	2020-10-12 15:07:38.34091
47933	HLF	2020-10-07	50.73	51.585	50.29	50.52	1361713	{}	\N	2020-10-12 15:07:38.345398
47934	HLF	2020-10-08	50.66	51.36	50	50.87	1446006	{}	\N	2020-10-12 15:07:38.352417
47935	HLF	2020-10-09	51.1	51.48	50.82	51.35	894348	{}	\N	2020-10-12 15:07:38.356741
47936	HLF	2020-10-12	51.27	51.27	49.7	49.78	220476	{}	\N	2020-10-12 15:07:38.362013
47937	HMHC	2020-09-28	1.72	1.86	1.71	1.72	2876787	{}	\N	2020-10-12 15:07:39.37868
47938	HMHC	2020-09-29	1.71	1.74	1.61	1.68	4002015	{}	\N	2020-10-12 15:07:39.39003
47939	HMHC	2020-09-30	1.68	1.79	1.68	1.73	1496487	{}	\N	2020-10-12 15:07:39.397397
47940	HMHC	2020-10-01	1.95	2.24	1.88	2.01	9083975	{}	\N	2020-10-12 15:07:39.40355
47941	HMHC	2020-10-02	2.1	2.5	2.0901	2.49	10894996	{}	\N	2020-10-12 15:07:39.408816
47942	HMHC	2020-10-05	2.62	2.64	2.24	2.51	4689389	{}	\N	2020-10-12 15:07:39.414341
47943	HMHC	2020-10-06	2.41	2.75	2.4	2.53	3773336	{}	\N	2020-10-12 15:07:39.418799
47944	HMHC	2020-10-07	2.55	2.7197	2.5	2.65	2381206	{}	\N	2020-10-12 15:07:39.423255
47945	HMHC	2020-10-08	2.66	2.74	2.54	2.71	1860795	{}	\N	2020-10-12 15:07:39.427633
47946	HMHC	2020-10-09	2.72	2.93	2.655	2.83	3002167	{}	\N	2020-10-12 15:07:39.432458
47947	HMHC	2020-10-12	2.84	2.8541	2.76	2.795	322180	{}	\N	2020-10-12 15:07:39.436905
47948	HOFT	2020-09-28	25.58	26.335	25.46	26.09	47674	{}	\N	2020-10-12 15:07:40.919337
47949	HOFT	2020-09-29	25.72	26.035	25.38	25.66	39534	{}	\N	2020-10-12 15:07:40.929522
47950	HOFT	2020-09-30	26.25	26.42	25.46	25.86	49469	{}	\N	2020-10-12 15:07:40.936411
47951	HOFT	2020-10-01	26.32	26.67	25.82	26.47	67370	{}	\N	2020-10-12 15:07:40.94219
47952	HOFT	2020-10-02	25.99	27.57	25.99	27.39	81995	{}	\N	2020-10-12 15:07:40.947504
47953	HOFT	2020-10-05	27.54	28.16	27.43	27.99	89181	{}	\N	2020-10-12 15:07:40.952337
47954	HOFT	2020-10-06	28.09	28.72	27.72	27.72	54689	{}	\N	2020-10-12 15:07:40.956831
47955	HOFT	2020-10-07	28.63	28.74	28.3	28.66	121718	{}	\N	2020-10-12 15:07:40.961448
47956	HOFT	2020-10-08	28.765	28.84	27.81	28.52	98925	{}	\N	2020-10-12 15:07:40.965863
47957	HOFT	2020-10-09	28.9	28.9	28.12	28.72	48405	{}	\N	2020-10-12 15:07:40.970289
47958	HOFT	2020-10-12	28.73	29.6	28.275	29.43	12460	{}	\N	2020-10-12 15:07:40.97453
47959	HDGE	2020-09-28	4.48	4.48	4.42	4.46	1051265	{}	\N	2020-10-12 15:07:42.796562
47960	HDGE	2020-09-29	4.46	4.55	4.46	4.5	857562	{}	\N	2020-10-12 15:07:42.806802
47961	HDGE	2020-09-30	4.51	4.53	4.449	4.49	1275468	{}	\N	2020-10-12 15:07:42.814068
47962	HDGE	2020-10-01	4.47	4.5052	4.455	4.47	302244	{}	\N	2020-10-12 15:07:42.820377
47963	HDGE	2020-10-02	4.57	4.58	4.37	4.37	590313	{}	\N	2020-10-12 15:07:42.82538
47964	HDGE	2020-10-05	4.33	4.368	4.31	4.31	384404	{}	\N	2020-10-12 15:07:42.830404
47965	HDGE	2020-10-06	4.31	4.3558	4.2204	4.34	1205115	{}	\N	2020-10-12 15:07:42.835391
47966	HDGE	2020-10-07	4.31	4.31	4.26	4.27	779134	{}	\N	2020-10-12 15:07:42.839897
47967	HDGE	2020-10-08	4.27	4.27	4.17	4.19	1128756	{}	\N	2020-10-12 15:07:42.844265
47968	HDGE	2020-10-09	4.16	4.22	4.14	4.22	525564	{}	\N	2020-10-12 15:07:42.84869
47969	HDGE	2020-10-12	4.18	4.21	4.18	4.2	73634	{}	\N	2020-10-12 15:07:42.853035
47970	HDSN	2020-09-28	1.12	1.17	1.12	1.16	36036	{}	\N	2020-10-12 15:07:44.172236
47971	HDSN	2020-09-29	1.154	1.17	1.1	1.14	29584	{}	\N	2020-10-12 15:07:44.178147
47972	HDSN	2020-09-30	1.15	1.18	1.15	1.17	29030	{}	\N	2020-10-12 15:07:44.183344
47973	HDSN	2020-10-01	1.15	1.1601	1.1	1.14	39035	{}	\N	2020-10-12 15:07:44.18792
47974	HDSN	2020-10-02	1.14	1.14	1.055	1.12	108868	{}	\N	2020-10-12 15:07:44.19237
47975	HDSN	2020-10-05	1.11	1.14	1.1029	1.11	42123	{}	\N	2020-10-12 15:07:44.198245
47976	HDSN	2020-10-06	1.1163	1.16	1.11	1.13	89361	{}	\N	2020-10-12 15:07:44.203923
47977	HDSN	2020-10-07	1.14	1.19	1.1	1.17	158995	{}	\N	2020-10-12 15:07:44.209264
47978	HDSN	2020-10-08	1.17	1.2	1.14	1.2	18124	{}	\N	2020-10-12 15:07:44.215215
47979	HDSN	2020-10-09	1.21	1.21	1.15	1.18	56221	{}	\N	2020-10-12 15:07:44.220627
47980	HDSN	2020-10-12	1.2	1.22	1.16	1.205	56025	{}	\N	2020-10-12 15:07:44.225889
47981	HEI	2020-09-28	106.15	107.39	105.125	105.44	380587	{}	\N	2020-10-12 15:07:46.009658
47982	HEI	2020-09-29	104.47	105.71	104.04	104.44	516001	{}	\N	2020-10-12 15:07:46.020553
47983	HEI	2020-09-30	104.73	106.28	103.47	104.66	1036422	{}	\N	2020-10-12 15:07:46.027843
47984	HEI	2020-10-01	106.15	106.9	104.63	105.42	532264	{}	\N	2020-10-12 15:07:46.034472
47985	HEI	2020-10-02	102.07	106.37	102.01	106.1	483341	{}	\N	2020-10-12 15:07:46.041207
47986	HEI	2020-10-05	107.55	108.395	106.36	106.67	453234	{}	\N	2020-10-12 15:07:46.047374
47987	HEI	2020-10-06	107.87	109.05	105.89	106.04	686899	{}	\N	2020-10-12 15:07:46.052716
47988	HEI	2020-10-07	109.01	111.06	108.56	110.48	667110	{}	\N	2020-10-12 15:07:46.056929
47989	HEI	2020-10-08	111.87	112.09	110.66	111.78	423053	{}	\N	2020-10-12 15:07:46.061223
47990	HEI	2020-10-09	112	113.2228	111.29	111.66	482767	{}	\N	2020-10-12 15:07:46.065526
47991	HEI	2020-10-12	111.71	111.93	110.78	111.43	75156	{}	\N	2020-10-12 15:07:46.069692
47992	HFC	2020-09-28	20.59	20.98	20.37	20.89	2357270	{}	\N	2020-10-12 15:07:47.565919
47993	HFC	2020-09-29	20.8	20.97	19.965	20.18	3503592	{}	\N	2020-10-12 15:07:47.578167
47994	HFC	2020-09-30	20.14	20.31	19.6187	19.71	4912495	{}	\N	2020-10-12 15:07:47.586028
47995	HFC	2020-10-01	19.39	19.6	18.59	18.94	5970513	{}	\N	2020-10-12 15:07:47.592303
47996	HFC	2020-10-02	18.5	19.81	18.38	19.6	5074453	{}	\N	2020-10-12 15:07:47.597551
47997	HFC	2020-10-05	19.94	20.28	19.5686	20.03	4799506	{}	\N	2020-10-12 15:07:47.602317
47998	HFC	2020-10-06	20.38	21.02	20.06	20.13	4404193	{}	\N	2020-10-12 15:07:47.606714
47999	HFC	2020-10-07	20.26	20.79	19.85	20.7	4572157	{}	\N	2020-10-12 15:07:47.610727
48000	HFC	2020-10-08	20.9	21.66	20.63	21.5	3634284	{}	\N	2020-10-12 15:07:47.614828
48001	HFC	2020-10-09	21.64	21.85	20.82	20.87	3740169	{}	\N	2020-10-12 15:07:47.618959
48002	HFC	2020-10-12	20.78	20.78	20.34	20.475	509401	{}	\N	2020-10-12 15:07:47.62286
48003	HIFS	2020-09-28	184.5	185.65	184.5	185.65	2277	{}	\N	2020-10-12 15:07:49.09759
48004	HIFS	2020-09-29	183.13	183.75	180.75	183.67	3329	{}	\N	2020-10-12 15:07:49.107431
48005	HIFS	2020-09-30	182	184.8687	182	184	3439	{}	\N	2020-10-12 15:07:49.114731
48006	HIFS	2020-10-01	184.31	188	184.15	185.51	3314	{}	\N	2020-10-12 15:07:49.120361
48007	HIFS	2020-10-02	184	185.01	182.65	183.8	4522	{}	\N	2020-10-12 15:07:49.124782
48008	HIFS	2020-10-05	185	186	183.155	186	6256	{}	\N	2020-10-12 15:07:49.128966
48009	HIFS	2020-10-06	186.2	189.59	186.2	189.59	2768	{}	\N	2020-10-12 15:07:49.133372
48010	HIFS	2020-10-07	191.98	192	191.87	192	2265	{}	\N	2020-10-12 15:07:49.137438
48011	HIFS	2020-10-08	192	195.18	192	195.18	2636	{}	\N	2020-10-12 15:07:49.141212
48012	HIFS	2020-10-09	197.551	199.65	197.5	198.95	2699	{}	\N	2020-10-12 15:07:49.145014
48013	HIFS	2020-10-12	199.85	203.46	199.85	202	3253	{}	\N	2020-10-12 15:07:49.148972
48014	HL	2020-09-28	5.05	5.095	4.9001	5.01	6901790	{}	\N	2020-10-12 15:07:50.852201
48015	HL	2020-09-29	5.06	5.13	4.95	5.07	12883646	{}	\N	2020-10-12 15:07:50.862502
48016	HL	2020-09-30	5.02	5.17	4.98	5.08	14152415	{}	\N	2020-10-12 15:07:50.869619
48017	HL	2020-10-01	5.14	5.24	5.02	5.21	15776892	{}	\N	2020-10-12 15:07:50.875515
48018	HL	2020-10-02	5.12	5.2322	5.0316	5.08	13035715	{}	\N	2020-10-12 15:07:50.881573
48019	HL	2020-10-05	5.15	5.35	5.13	5.29	13758859	{}	\N	2020-10-12 15:07:50.887169
48020	HL	2020-10-06	5.32	5.36	5.01	5.02	20750079	{}	\N	2020-10-12 15:07:50.89275
48021	HL	2020-10-07	5.13	5.2	5	5.16	14689011	{}	\N	2020-10-12 15:07:50.897529
48022	HL	2020-10-08	5.22	5.26	5.1	5.2	8759250	{}	\N	2020-10-12 15:07:50.902556
48023	HL	2020-10-09	5.37	5.53	5.26	5.53	16900954	{}	\N	2020-10-12 15:07:50.907178
48024	HL	2020-10-12	5.56	5.59	5.43	5.485	5013910	{}	\N	2020-10-12 15:07:50.911749
48025	HMSY	2020-09-28	23.78	24.21	23.58	23.66	408841	{}	\N	2020-10-12 15:07:53.939255
48026	HMSY	2020-09-29	23.61	23.835	23.33	23.74	594849	{}	\N	2020-10-12 15:07:53.951467
48027	HMSY	2020-09-30	23.96	24.2	23.7	23.95	694402	{}	\N	2020-10-12 15:07:53.958187
48028	HMSY	2020-10-01	23.85	24.1	23.49	23.91	404605	{}	\N	2020-10-12 15:07:53.964343
48029	HMSY	2020-10-02	23.65	24.63	23.363	24.37	516449	{}	\N	2020-10-12 15:07:53.970213
48030	HMSY	2020-10-05	27	27.845	26.62	27.39	1762608	{}	\N	2020-10-12 15:07:53.975456
48031	HMSY	2020-10-06	27.42	28.48	27.11	28.18	1286961	{}	\N	2020-10-12 15:07:53.979945
48032	HMSY	2020-10-07	28.36	28.63	27.94	28.11	723882	{}	\N	2020-10-12 15:07:53.984514
48033	HMSY	2020-10-08	28.2525	29.13	28.15	29	443750	{}	\N	2020-10-12 15:07:53.988662
48034	HMSY	2020-10-09	29.02	29.35	28.53	28.86	404622	{}	\N	2020-10-12 15:07:53.992774
48035	HMSY	2020-10-12	28.87	29.63	28.53	29.63	51244	{}	\N	2020-10-12 15:07:53.997472
48036	HOG	2020-09-28	23.599	24.06	23.42	23.91	2127695	{}	\N	2020-10-12 15:07:55.357443
48037	HOG	2020-09-29	23.79	23.97	23.315	23.61	2969357	{}	\N	2020-10-12 15:07:55.36852
48038	HOG	2020-09-30	23.78	24.63	23.73	24.54	6713194	{}	\N	2020-10-12 15:07:55.374776
48039	HOG	2020-10-01	24.71	25.16	24.33	25.16	3928864	{}	\N	2020-10-12 15:07:55.380433
48040	HOG	2020-10-02	24.26	25.89	24.21	25.81	4775069	{}	\N	2020-10-12 15:07:55.385441
48041	HOG	2020-10-05	26.28	27.17	26.19	26.98	7610934	{}	\N	2020-10-12 15:07:55.38965
48042	HOG	2020-10-06	27.45	27.84	26.66	26.77	4243479	{}	\N	2020-10-12 15:07:55.393819
48043	HOG	2020-10-07	27.26	27.965	26.86	27.56	3371293	{}	\N	2020-10-12 15:07:55.398171
48044	HOG	2020-10-08	28.28	29.41	28.16	28.98	5878823	{}	\N	2020-10-12 15:07:55.402917
48045	HOG	2020-10-09	29.17	29.36	27.965	28.02	3590363	{}	\N	2020-10-12 15:07:55.407451
48046	HOG	2020-10-12	28.25	28.67	28.12	28.5905	686168	{}	\N	2020-10-12 15:07:55.412947
48047	HEBT	2020-09-28	14.01	15.1	14.01	15.05	23800	{}	\N	2020-10-12 15:07:56.730057
48048	HEBT	2020-09-29	14.83	15.48	14.67	15.47	18102	{}	\N	2020-10-12 15:07:56.737676
48049	HEBT	2020-09-30	15.4	15.75	14.82	15.415	54122	{}	\N	2020-10-12 15:07:56.744006
48050	HEBT	2020-10-01	15.4962	15.8564	14.76	15.47	64929	{}	\N	2020-10-12 15:07:56.749738
48051	HEBT	2020-10-02	15.4528	16.2354	14.7265	16	51390	{}	\N	2020-10-12 15:07:56.754551
48052	HEBT	2020-10-05	16.2	17.8	16.155	17.78	70374	{}	\N	2020-10-12 15:07:56.758622
48053	HEBT	2020-10-06	18.5665	18.5665	16.25	16.32	36811	{}	\N	2020-10-12 15:07:56.762944
48054	HEBT	2020-10-07	16.5	16.65	15.64	15.66	19517	{}	\N	2020-10-12 15:07:56.766922
48055	HEBT	2020-10-08	16.26	16.56	15.43	16.24	34556	{}	\N	2020-10-12 15:07:56.771108
48056	HEBT	2020-10-09	16.6671	16.6671	16.08	16.54	26491	{}	\N	2020-10-12 15:07:56.775257
48057	HEBT	2020-10-12	16.79	16.79	16.73	16.73	649	{}	\N	2020-10-12 15:07:56.779256
48058	HEP	2020-09-28	12.51	12.82	12	12.35	247985	{}	\N	2020-10-12 15:07:57.811279
48059	HEP	2020-09-29	12.26	12.85	12.1504	12.71	660538	{}	\N	2020-10-12 15:07:57.820889
48060	HEP	2020-09-30	12.77	12.88	12.09	12.13	955157	{}	\N	2020-10-12 15:07:57.827357
48061	HEP	2020-10-01	12.16	12.19	11.57	11.93	597208	{}	\N	2020-10-12 15:07:57.832756
48062	HEP	2020-10-02	11.58	12.24	11.58	12.08	637283	{}	\N	2020-10-12 15:07:57.838101
48063	HEP	2020-10-05	12.23	12.62	11.83	12.6	582768	{}	\N	2020-10-12 15:07:57.842769
48064	HEP	2020-10-06	12.72	12.81	12.09	12.22	714679	{}	\N	2020-10-12 15:07:57.847179
48065	HEP	2020-10-07	12.29	12.305	11.85	12.03	427840	{}	\N	2020-10-12 15:07:57.851447
48066	HEP	2020-10-08	12.13	12.69	11.94	12.61	696571	{}	\N	2020-10-12 15:07:57.855925
48067	HEP	2020-10-09	12.75	13.18	12.43	13.15	624698	{}	\N	2020-10-12 15:07:57.860242
48068	HEP	2020-10-12	13.25	13.4	12.91	13.08	122340	{}	\N	2020-10-12 15:07:57.864614
48069	HI	2020-09-28	27.73	28.72	27.73	28.2	318670	{}	\N	2020-10-12 15:07:59.742931
48070	HI	2020-09-29	28.09	28.385	27.78	28.2	427065	{}	\N	2020-10-12 15:07:59.749701
48071	HI	2020-09-30	28.17	28.93	28.04	28.36	884996	{}	\N	2020-10-12 15:07:59.754637
48072	HI	2020-10-01	28.38	28.72	28.02	28.62	795277	{}	\N	2020-10-12 15:07:59.759377
48073	HI	2020-10-02	28	29.3	27.85	29.15	650915	{}	\N	2020-10-12 15:07:59.763946
48074	HI	2020-10-05	29.46	30.35	29.46	30.33	553146	{}	\N	2020-10-12 15:07:59.768521
48075	HI	2020-10-06	30.5	31.72	30.3856	30.66	901965	{}	\N	2020-10-12 15:07:59.772406
48076	HI	2020-10-07	30.97	31.73	30.97	31.35	709184	{}	\N	2020-10-12 15:07:59.776508
48077	HI	2020-10-08	31.78	31.82	31.12	31.38	518166	{}	\N	2020-10-12 15:07:59.780719
48078	HI	2020-10-09	31.72	31.79	30.87	30.88	695573	{}	\N	2020-10-12 15:07:59.784487
48079	HI	2020-10-12	30.79	31.07	30.7	30.91	58749	{}	\N	2020-10-12 15:07:59.788228
48080	HJPX	2020-09-28	29.5483	29.5483	29.5483	29.5483	5	{}	\N	2020-10-12 15:08:01.824206
48081	HJPX	2020-09-29	29.5928	29.5928	29.5928	29.5928	22	{}	\N	2020-10-12 15:08:01.832793
48082	HJPX	2020-09-30	29.4	29.4	29.3627	29.3627	511	{}	\N	2020-10-12 15:08:01.838584
48083	HJPX	2020-10-01	29.3486	29.3486	29.3486	29.3486	3	{}	\N	2020-10-12 15:08:01.844681
48084	HJPX	2020-10-02	29.146	29.146	29.146	29.146	5	{}	\N	2020-10-12 15:08:01.850897
48085	HJPX	2020-10-05	29.5211	29.5211	29.5211	29.5211	2	{}	\N	2020-10-12 15:08:01.855521
48086	HJPX	2020-10-06	29.62	29.62	29.3945	29.3945	115	{}	\N	2020-10-12 15:08:01.860163
48087	HJPX	2020-10-07	29.68	29.68	29.68	29.68	3	{}	\N	2020-10-12 15:08:01.864709
48088	HJPX	2020-10-08	29.6634	29.6634	29.6634	29.6634	3	{}	\N	2020-10-12 15:08:01.869096
48089	HJPX	2020-10-09	29.5716	29.5716	29.5716	29.5716	2	{}	\N	2020-10-12 15:08:01.873914
48090	HJPX	2020-10-12	29.56	29.56	29.56	29.56	401	{}	\N	2020-10-12 15:08:01.879354
48091	HMTV	2020-09-28	8.75	8.99	8.75	8.85	26985	{}	\N	2020-10-12 15:08:04.427343
48092	HMTV	2020-09-29	8.79	9.09	8.72	8.88	18592	{}	\N	2020-10-12 15:08:04.440959
48093	HMTV	2020-09-30	9.045	9.045	8.66	8.69	9387	{}	\N	2020-10-12 15:08:04.448814
48094	HMTV	2020-10-01	8.64	9	8.62	8.77	12604	{}	\N	2020-10-12 15:08:04.454825
48095	HMTV	2020-10-02	8.43	8.74	8.43	8.65	11651	{}	\N	2020-10-12 15:08:04.459733
48096	HMTV	2020-10-05	8.66	8.985	8.66	8.94	14076	{}	\N	2020-10-12 15:08:04.464607
48097	HMTV	2020-10-06	8.88	9.39	8.85	9.07	17726	{}	\N	2020-10-12 15:08:04.469275
48098	HMTV	2020-10-07	9.06	9.41	9.06	9.14	17762	{}	\N	2020-10-12 15:08:04.473594
48099	HMTV	2020-10-08	9.23	9.49	9.07	9.29	12178	{}	\N	2020-10-12 15:08:04.47777
48100	HMTV	2020-10-09	9.44	9.49	9.07	9.25	13124	{}	\N	2020-10-12 15:08:04.482125
48101	HMTV	2020-10-12	9.21	9.21	9.06	9.06	896	{}	\N	2020-10-12 15:08:04.48644
48102	HOLI	2020-09-28	10.645	11.03	10.645	10.96	154022	{}	\N	2020-10-12 15:08:05.799665
48103	HOLI	2020-09-29	10.98	11.08	10.81	11.03	210735	{}	\N	2020-10-12 15:08:05.809422
48104	HOLI	2020-09-30	11.06	11.22	11	11.05	206897	{}	\N	2020-10-12 15:08:05.816405
48105	HOLI	2020-10-01	11.09	11.22	11.09	11.2	146557	{}	\N	2020-10-12 15:08:05.822709
48106	HOLI	2020-10-02	11.09	11.21	11	11.17	218339	{}	\N	2020-10-12 15:08:05.828165
48107	HOLI	2020-10-05	11.17	11.54	11.17	11.53	151559	{}	\N	2020-10-12 15:08:05.832955
48108	HOLI	2020-10-06	11.5	11.585	11.35	11.45	166083	{}	\N	2020-10-12 15:08:05.837193
48109	HOLI	2020-10-07	11.53	11.66	11.43	11.56	133887	{}	\N	2020-10-12 15:08:05.841223
48110	HOLI	2020-10-08	11.62	11.88	11.45	11.88	180889	{}	\N	2020-10-12 15:08:05.845581
48111	HOLI	2020-10-09	11.9	11.95	11.531	11.855	153657	{}	\N	2020-10-12 15:08:05.849729
48112	HOLI	2020-10-12	11.86	12.09	11.78	12.05	84651	{}	\N	2020-10-12 15:08:05.853853
48113	HCC	2020-09-28	17.96	18.58	17.76	18.13	397912	{}	\N	2020-10-12 15:08:07.075251
48114	HCC	2020-09-29	18.12	18.23	17.3	17.48	727040	{}	\N	2020-10-12 15:08:07.087446
48115	HCC	2020-09-30	17.65	17.84	16.9	17.08	1082318	{}	\N	2020-10-12 15:08:07.093884
48116	HCC	2020-10-01	17.17	17.61	17.05	17.36	1161476	{}	\N	2020-10-12 15:08:07.099422
48117	HCC	2020-10-02	16.96	17.72	16.81	17.5	843770	{}	\N	2020-10-12 15:08:07.104639
48118	HCC	2020-10-05	17.58	18.1	17.565	17.9	991350	{}	\N	2020-10-12 15:08:07.109465
48119	HCC	2020-10-06	18.18	18.18	17.09	17.11	1454879	{}	\N	2020-10-12 15:08:07.114149
48120	HCC	2020-10-07	17.23	17.2849	16.29	16.98	2061179	{}	\N	2020-10-12 15:08:07.118182
48121	HCC	2020-10-08	17.25	17.79	16.9001	17.38	1485522	{}	\N	2020-10-12 15:08:07.121976
48122	HCC	2020-10-09	17.62	17.69	16.07	16.57	1873898	{}	\N	2020-10-12 15:08:07.125846
48123	HCC	2020-10-12	16.51	16.92	16.19	16.46	191066	{}	\N	2020-10-12 15:08:07.129602
48124	HDMV	2020-09-28	28.71	28.82	28.7	28.7734	24169	{}	\N	2020-10-12 15:08:08.123924
48125	HDMV	2020-09-29	28.73	28.8527	28.648	28.72	17814	{}	\N	2020-10-12 15:08:08.136222
48126	HDMV	2020-09-30	28.69	28.7833	28.52	28.655	38185	{}	\N	2020-10-12 15:08:08.143153
48127	HDMV	2020-10-01	28.77	28.8	28.71	28.76	12189	{}	\N	2020-10-12 15:08:08.149049
48128	HDMV	2020-10-02	28.55	28.8	28.55	28.74	17742	{}	\N	2020-10-12 15:08:08.153548
48129	HDMV	2020-10-05	28.86	29.01	28.86	29	14934	{}	\N	2020-10-12 15:08:08.158587
48130	HDMV	2020-10-06	29.06	29.06	28.6803	28.71	11831	{}	\N	2020-10-12 15:08:08.163241
48131	HDMV	2020-10-07	28.82	28.92	28.81	28.9	162323	{}	\N	2020-10-12 15:08:08.167436
48132	HDMV	2020-10-08	28.89	28.94	28.86	28.88	87793	{}	\N	2020-10-12 15:08:08.172099
48133	HDMV	2020-10-09	28.98	29.1191	28.98	29.075	39809	{}	\N	2020-10-12 15:08:08.176469
48134	HDMV	2020-10-12	29.16	29.23	29.16	29.23	5454	{}	\N	2020-10-12 15:08:08.180761
48135	HEAR	2020-09-28	17.15	17.74	17.07	17.71	358115	{}	\N	2020-10-12 15:08:09.600455
48136	HEAR	2020-09-29	17.67	17.8	17.23	17.3	207655	{}	\N	2020-10-12 15:08:09.609828
48137	HEAR	2020-09-30	17.85	19.1768	17.62	18.2	670162	{}	\N	2020-10-12 15:08:09.616156
48138	HEAR	2020-10-01	18.53	19.24	18.47	18.88	495279	{}	\N	2020-10-12 15:08:09.621449
48139	HEAR	2020-10-02	18.22	18.76	17.97	18.17	330170	{}	\N	2020-10-12 15:08:09.627026
48140	HEAR	2020-10-05	18.28	18.52	18.03	18.32	291004	{}	\N	2020-10-12 15:08:09.631574
48141	HEAR	2020-10-06	18.41	18.48	17.84	17.89	293119	{}	\N	2020-10-12 15:08:09.635302
48142	HEAR	2020-10-07	18.05	18.7	18.05	18.39	484978	{}	\N	2020-10-12 15:08:09.63901
48143	HEAR	2020-10-08	18.49	18.49	17.95	17.97	231216	{}	\N	2020-10-12 15:08:09.6431
48144	HEAR	2020-10-09	18.12	20	17.97	19.84	776665	{}	\N	2020-10-12 15:08:09.646511
48145	HEAR	2020-10-12	20	20	19.27	19.8	174982	{}	\N	2020-10-12 15:08:09.650137
48146	HEQ	2020-09-28	10.42	10.57	10.41	10.55	64224	{}	\N	2020-10-12 15:08:10.656836
48147	HEQ	2020-09-29	10.57	10.57	10.47	10.53	70566	{}	\N	2020-10-12 15:08:10.666756
48148	HEQ	2020-09-30	10.54	10.67	10.54	10.56	138842	{}	\N	2020-10-12 15:08:10.673363
48149	HEQ	2020-10-01	10.66	10.71	10.61	10.63	84706	{}	\N	2020-10-12 15:08:10.679322
48150	HEQ	2020-10-02	10.56	10.64	10.36	10.64	57715	{}	\N	2020-10-12 15:08:10.68528
48151	HEQ	2020-10-05	10.66	10.8	10.66	10.68	75136	{}	\N	2020-10-12 15:08:10.691308
48152	HEQ	2020-10-06	10.69	10.73	10.54	10.57	137552	{}	\N	2020-10-12 15:08:10.697839
48153	HEQ	2020-10-07	10.61	10.7099	10.61	10.65	120704	{}	\N	2020-10-12 15:08:10.702713
48154	HEQ	2020-10-08	10.7	10.7799	10.61	10.66	96126	{}	\N	2020-10-12 15:08:10.707792
48155	HEQ	2020-10-09	10.66	10.71	10.64	10.65	162224	{}	\N	2020-10-12 15:08:10.713222
48156	HEQ	2020-10-12	10.65	10.71	10.6306	10.71	43192	{}	\N	2020-10-12 15:08:10.718517
48157	HIHO	2020-09-28	2.92	3.03	2.82	2.92	42696	{}	\N	2020-10-12 15:08:13.299801
48158	HIHO	2020-09-29	2.81	2.8738	2.635	2.75	95146	{}	\N	2020-10-12 15:08:13.308474
48159	HIHO	2020-09-30	2.71	2.84	2.6508	2.78	19322	{}	\N	2020-10-12 15:08:13.314845
48160	HIHO	2020-10-01	2.78	2.81	2.755	2.76	5603	{}	\N	2020-10-12 15:08:13.320052
48161	HIHO	2020-10-02	2.85	2.85	2.74	2.82	35735	{}	\N	2020-10-12 15:08:13.325408
48162	HIHO	2020-10-05	2.79	2.995	2.79	2.93	39465	{}	\N	2020-10-12 15:08:13.329672
48163	HIHO	2020-10-06	2.875	3.21	2.875	3.1	74047	{}	\N	2020-10-12 15:08:13.334004
48164	HIHO	2020-10-07	3.11	3.24	3.075	3.21	50085	{}	\N	2020-10-12 15:08:13.338244
48165	HIHO	2020-10-08	3.2	3.6299	2.98	3.62	96039	{}	\N	2020-10-12 15:08:13.343237
48166	HIHO	2020-10-09	3.44	3.98	3.3	3.79	942980	{}	\N	2020-10-12 15:08:13.347775
48167	HIHO	2020-10-12	3.55	3.55	3.26	3.3055	81260	{}	\N	2020-10-12 15:08:13.352216
48168	HLX	2020-09-28	2.45	2.65	2.45	2.52	2654297	{}	\N	2020-10-12 15:08:15.100577
48169	HLX	2020-09-29	2.55	2.56	2.37	2.43	5370756	{}	\N	2020-10-12 15:08:15.110674
48170	HLX	2020-09-30	2.43	2.52	2.32	2.41	8041494	{}	\N	2020-10-12 15:08:15.117716
48171	HLX	2020-10-01	2.42	2.44	2.26	2.35	5017325	{}	\N	2020-10-12 15:08:15.123584
48172	HLX	2020-10-02	2.2	2.4504	2.2	2.38	4492469	{}	\N	2020-10-12 15:08:15.12882
48173	HLX	2020-10-05	2.44	2.57	2.374	2.56	5117124	{}	\N	2020-10-12 15:08:15.133778
48174	HLX	2020-10-06	2.7	2.77	2.47	2.5	7344733	{}	\N	2020-10-12 15:08:15.137959
48175	HLX	2020-10-07	2.51	2.585	2.47	2.58	5419834	{}	\N	2020-10-12 15:08:15.142223
48176	HLX	2020-10-08	2.64	2.82	2.6	2.81	5206999	{}	\N	2020-10-12 15:08:15.146615
48177	HLX	2020-10-09	2.86	2.9	2.67	2.75	7058730	{}	\N	2020-10-12 15:08:15.150902
48178	HLX	2020-10-12	2.79	2.79	2.65	2.7301	1356357	{}	\N	2020-10-12 15:08:15.154997
48179	HNP	2020-09-28	15.4517	15.4517	15.37	15.37	4822	{}	\N	2020-10-12 15:08:17.165844
48180	HNP	2020-09-29	15.16	15.16	15.01	15.01	11488	{}	\N	2020-10-12 15:08:17.178868
48181	HNP	2020-09-30	15.43	15.44	15.39	15.43	27364	{}	\N	2020-10-12 15:08:17.186422
48182	HNP	2020-10-01	15.58	15.61	15.489	15.59	12888	{}	\N	2020-10-12 15:08:17.192324
48183	HNP	2020-10-02	15.53	15.53	15.41	15.41	14036	{}	\N	2020-10-12 15:08:17.197156
48184	HNP	2020-10-05	15.33	15.37	15.28	15.33	14535	{}	\N	2020-10-12 15:08:17.202237
48185	HNP	2020-10-06	15.79	15.92	15.79	15.83	11826	{}	\N	2020-10-12 15:08:17.206361
48186	HNP	2020-10-07	16.02	16.18	16.02	16.18	13074	{}	\N	2020-10-12 15:08:17.210667
48187	HNP	2020-10-08	16.18	16.18	16.06	16.18	8810	{}	\N	2020-10-12 15:08:17.214596
48188	HNP	2020-10-09	15.53	15.6399	15.49	15.63	19954	{}	\N	2020-10-12 15:08:17.218346
48189	HNP	2020-10-12	15.72	15.82	15.7049	15.7049	2956	{}	\N	2020-10-12 15:08:17.221911
48190	HOME	2020-09-28	14.58	15.15	14.21	14.23	1907542	{}	\N	2020-10-12 15:08:18.254044
48191	HOME	2020-09-29	14.75	15.62	14.6646	14.84	4635773	{}	\N	2020-10-12 15:08:18.262669
48192	HOME	2020-09-30	15.1	15.72	14.76	14.86	8843125	{}	\N	2020-10-12 15:08:18.26837
48193	HOME	2020-10-01	15.11	15.75	14.89	15.2	7722981	{}	\N	2020-10-12 15:08:18.273907
48194	HOME	2020-10-02	14.49	15.91	14.4	15.75	5414307	{}	\N	2020-10-12 15:08:18.278579
48195	HOME	2020-10-05	16.15	18.07	16.145	18.06	8784107	{}	\N	2020-10-12 15:08:18.283331
48196	HOME	2020-10-06	18.07	18.59	17.61	17.84	7101080	{}	\N	2020-10-12 15:08:18.287247
48197	HOME	2020-10-07	18.34	19.38	18.18	19.22	5345326	{}	\N	2020-10-12 15:08:18.291042
48198	HOME	2020-10-08	19.35	19.67	18.77	19.22	5774537	{}	\N	2020-10-12 15:08:18.294905
48199	HOME	2020-10-09	19.292	19.52	18.76	19.22	4358863	{}	\N	2020-10-12 15:08:18.298595
48200	HOME	2020-10-12	19.68	20.25	19.5	19.6672	1379466	{}	\N	2020-10-12 15:08:18.30223
48201	HPI	2020-09-28	18.72	18.88	18.71	18.8	60240	{}	\N	2020-10-12 15:08:19.251271
48202	HPI	2020-09-29	18.7899	18.8199	18.67	18.79	81876	{}	\N	2020-10-12 15:08:19.261063
48203	HPI	2020-09-30	18.92	18.956	18.72	18.82	146740	{}	\N	2020-10-12 15:08:19.268036
48204	HPI	2020-10-01	18.92	18.9228	18.72	18.88	120884	{}	\N	2020-10-12 15:08:19.274138
48205	HPI	2020-10-02	18.7	18.93	18.7	18.84	83409	{}	\N	2020-10-12 15:08:19.279335
48206	HPI	2020-10-05	19	19.14	18.95	19.14	102714	{}	\N	2020-10-12 15:08:19.284383
48207	HPI	2020-10-06	19.14	19.29	19.06	19.14	110370	{}	\N	2020-10-12 15:08:19.288415
48208	HPI	2020-10-07	19.23	19.44	19.2075	19.44	132079	{}	\N	2020-10-12 15:08:19.292395
48209	HPI	2020-10-08	19.38	19.47	19.37	19.43	111922	{}	\N	2020-10-12 15:08:19.296535
48210	HPI	2020-10-09	19.37	19.41	19.24	19.26	81430	{}	\N	2020-10-12 15:08:19.30072
48211	HPI	2020-10-12	19.26	19.26	19.18	19.2499	20988	{}	\N	2020-10-12 15:08:19.304763
48212	HCSG	2020-09-28	21.28	21.435	20.81	20.93	468635	{}	\N	2020-10-12 15:08:20.359656
48213	HCSG	2020-09-29	20.85	21.5	20.85	21.36	435225	{}	\N	2020-10-12 15:08:20.37438
48214	HCSG	2020-09-30	21.4	21.75	21.3	21.53	412858	{}	\N	2020-10-12 15:08:20.379754
48215	HCSG	2020-10-01	21.72	21.95	21.329	21.73	507262	{}	\N	2020-10-12 15:08:20.38466
48216	HCSG	2020-10-02	21.36	22.46	21.36	22.39	361224	{}	\N	2020-10-12 15:08:20.38918
48217	HCSG	2020-10-05	22.55	22.7	22.21	22.56	356850	{}	\N	2020-10-12 15:08:20.394423
48218	HCSG	2020-10-06	22.6921	23.29	22.46	22.655	482560	{}	\N	2020-10-12 15:08:20.398329
48219	HCSG	2020-10-07	22.85	23.41	22.8208	23.37	328264	{}	\N	2020-10-12 15:08:20.402254
48220	HCSG	2020-10-08	23.25	24.03	23.1404	23.97	651382	{}	\N	2020-10-12 15:08:20.406089
48221	HCSG	2020-10-09	24.24	24.24	23.56	23.58	271140	{}	\N	2020-10-12 15:08:20.409776
48222	HCSG	2020-10-12	23.66	23.875	23.33	23.78	68388	{}	\N	2020-10-12 15:08:20.413787
48223	HEWC	2020-09-28	25.2761	25.2761	25.2761	25.2761	0	{}	\N	2020-10-12 15:08:21.508176
48224	HEWC	2020-09-29	25.185	25.185	25.185	25.185	0	{}	\N	2020-10-12 15:08:21.516012
48225	HEWC	2020-09-30	25.07	25.0905	25.07	25.0905	200	{}	\N	2020-10-12 15:08:21.521923
48226	HEWC	2020-10-01	25.09	25.1068	25.09	25.1068	202	{}	\N	2020-10-12 15:08:21.527105
48227	HEWC	2020-10-02	25.05	25.1988	25.05	25.1988	203	{}	\N	2020-10-12 15:08:21.531831
48228	HEWC	2020-10-05	25.41	25.5209	25.39	25.5209	304	{}	\N	2020-10-12 15:08:21.536152
48229	HEWC	2020-10-06	25.529	25.53	25.46	25.46	2876	{}	\N	2020-10-12 15:08:21.540325
48230	HEWC	2020-10-07	25.5906	25.5906	25.5906	25.5906	2	{}	\N	2020-10-12 15:08:21.544622
48231	HEWC	2020-10-08	25.7895	25.7994	25.7895	25.7994	388	{}	\N	2020-10-12 15:08:21.549007
48232	HEWC	2020-10-09	25.77	25.77	25.77	25.77	3	{}	\N	2020-10-12 15:08:21.55294
48233	HEZU	2020-09-28	28	28.12	27.98	28.07	106784	{}	\N	2020-10-12 15:08:23.085463
48234	HEZU	2020-09-29	28.04	28.07	27.88	27.95	97279	{}	\N	2020-10-12 15:08:23.096451
48235	HEZU	2020-09-30	27.97	28.11	27.77	27.84	123552	{}	\N	2020-10-12 15:08:23.102374
48236	HEZU	2020-10-01	27.99	28.02	27.85	27.96	1608321	{}	\N	2020-10-12 15:08:23.107846
48237	HEZU	2020-10-02	27.68	28.06	27.68	28.02	602890	{}	\N	2020-10-12 15:08:23.113181
48238	HEZU	2020-10-05	28.17	28.4	28.17	28.4	107510	{}	\N	2020-10-12 15:08:23.117793
48239	HEZU	2020-10-06	28.49	28.5	28.13	28.16	609712	{}	\N	2020-10-12 15:08:23.121901
48240	HEZU	2020-10-07	28.37	28.47	28.3	28.43	148846	{}	\N	2020-10-12 15:08:23.126149
48241	HEZU	2020-10-08	28.62	28.72	28.6	28.69	102490	{}	\N	2020-10-12 15:08:23.130026
48242	HEZU	2020-10-09	28.76	28.83	28.69	28.79	972553	{}	\N	2020-10-12 15:08:23.133784
48243	HEZU	2020-10-12	28.92	28.96	28.891	28.96	30489	{}	\N	2020-10-12 15:08:23.138078
48244	HGV	2020-09-28	21.53	22.105	21.12	21.98	457269	{}	\N	2020-10-12 15:08:25.261631
48245	HGV	2020-09-29	21.98	21.98	20.73	21.21	1655048	{}	\N	2020-10-12 15:08:25.271705
48246	HGV	2020-09-30	21.35	22.07	20.81	20.98	1086953	{}	\N	2020-10-12 15:08:25.278381
48247	HGV	2020-10-01	21.01	21.42	20.68	21.38	1197659	{}	\N	2020-10-12 15:08:25.28464
48248	HGV	2020-10-02	20.75	21.65	20.72	21.57	1070361	{}	\N	2020-10-12 15:08:25.289751
48249	HGV	2020-10-05	21.78	22	21.19	21.75	721292	{}	\N	2020-10-12 15:08:25.295032
48250	HGV	2020-10-06	22.07	22.9625	21.97	22.02	976682	{}	\N	2020-10-12 15:08:25.29905
48251	HGV	2020-10-07	22.43	23.1	22.33	22.79	2050837	{}	\N	2020-10-12 15:08:25.303094
48252	HGV	2020-10-08	23.11	23.11	22.47	22.91	1409516	{}	\N	2020-10-12 15:08:25.307206
48253	HGV	2020-10-09	23.1	23.11	22.4	22.83	802759	{}	\N	2020-10-12 15:08:25.311352
48254	HGV	2020-10-12	23.04	23.08	22.6	22.97	376175	{}	\N	2020-10-12 15:08:25.315236
48255	HIG	2020-09-28	36.6	37.39	36.53	36.88	1867287	{}	\N	2020-10-12 15:08:28.151235
48256	HIG	2020-09-29	36.54	36.76	35.62	36.53	3472626	{}	\N	2020-10-12 15:08:28.161764
48257	HIG	2020-09-30	36.82	37.265	36.48	36.86	5468759	{}	\N	2020-10-12 15:08:28.168612
48258	HIG	2020-10-01	36.89	37.22	36.28	36.77	3206735	{}	\N	2020-10-12 15:08:28.174902
48259	HIG	2020-10-02	36.14	37.83	36.1	37.56	4232989	{}	\N	2020-10-12 15:08:28.180058
48260	HIG	2020-10-05	38.78	39.63	38.57	39.23	7027892	{}	\N	2020-10-12 15:08:28.185092
48261	HIG	2020-10-06	39.4	39.63	37.87	37.91	4816531	{}	\N	2020-10-12 15:08:28.189246
48262	HIG	2020-10-07	38.3	38.93	37.89	37.95	4905057	{}	\N	2020-10-12 15:08:28.193762
48263	HIG	2020-10-08	38.32	39.36	38.05	39.33	5679309	{}	\N	2020-10-12 15:08:28.198237
48264	HIG	2020-10-09	39.39	39.47	38.52	38.61	6255725	{}	\N	2020-10-12 15:08:28.20249
48265	HIG	2020-10-12	38.59	39.06	38.39	38.95	472467	{}	\N	2020-10-12 15:08:28.20671
48266	HIW	2020-09-28	33.86	34.72	33.735	34.11	815738	{}	\N	2020-10-12 15:08:29.602324
48267	HIW	2020-09-29	33.92	34.1	32.84	33.58	1545954	{}	\N	2020-10-12 15:08:29.612263
48268	HIW	2020-09-30	33.5	34.15	33.25	33.57	6127574	{}	\N	2020-10-12 15:08:29.619037
48269	HIW	2020-10-01	33.5	34.42	33.44	34.4	2515243	{}	\N	2020-10-12 15:08:29.625154
48270	HIW	2020-10-02	33.82	35.24	33.67	35.18	1441407	{}	\N	2020-10-12 15:08:29.630121
48271	HIW	2020-10-05	35.51	35.69	34.665	35.51	1079013	{}	\N	2020-10-12 15:08:29.635332
48272	HIW	2020-10-06	35.76	36.22	35.175	35.49	2324917	{}	\N	2020-10-12 15:08:29.639379
48273	HIW	2020-10-07	35.7	35.81	34.89	35.24	1512738	{}	\N	2020-10-12 15:08:29.643683
48274	HIW	2020-10-08	35.59	35.9	35.31	35.83	2359920	{}	\N	2020-10-12 15:08:29.64803
48275	HIW	2020-10-09	36.22	36.22	35.2	35.35	2481986	{}	\N	2020-10-12 15:08:29.652409
48276	HIW	2020-10-12	35.43	35.78	34.96	35.61	880116	{}	\N	2020-10-12 15:08:29.656539
48277	HLT	2020-09-28	87.82	89.44	86.8	89.06	2221549	{}	\N	2020-10-12 15:08:31.42234
48278	HLT	2020-09-29	88.9	88.98	86.43	86.6	5755937	{}	\N	2020-10-12 15:08:31.434799
48279	HLT	2020-09-30	87.54	88.88	85.17	85.32	6170233	{}	\N	2020-10-12 15:08:31.44227
48280	HLT	2020-10-01	85.92	87.24	85.55	87.22	3522517	{}	\N	2020-10-12 15:08:31.448782
48281	HLT	2020-10-02	84.91	88.02	84.61	87.96	3165721	{}	\N	2020-10-12 15:08:31.453754
48282	HLT	2020-10-05	88.45	90.0063	87.86	89.3	2662749	{}	\N	2020-10-12 15:08:31.45888
48283	HLT	2020-10-06	90.28	92.39	89.38	89.6	4067838	{}	\N	2020-10-12 15:08:31.463035
48284	HLT	2020-10-07	90.41	92.15	90.41	91.72	4168840	{}	\N	2020-10-12 15:08:31.46731
48285	HLT	2020-10-08	92.28	92.93	91.14	92.54	4380813	{}	\N	2020-10-12 15:08:31.471463
48286	HLT	2020-10-09	93	94.1	91.255	91.37	3225219	{}	\N	2020-10-12 15:08:31.475671
48287	HLT	2020-10-12	91.44	91.515	90.385	90.79	558253	{}	\N	2020-10-12 15:08:31.47982
48288	HMOP	2020-09-28	41.32	41.33	41.267	41.285	17159	{}	\N	2020-10-12 15:08:32.514316
48289	HMOP	2020-09-29	41.205	41.23	41.1601	41.1833	6766	{}	\N	2020-10-12 15:08:32.523817
48290	HMOP	2020-09-30	41.18	41.19	41.1125	41.1382	5505	{}	\N	2020-10-12 15:08:32.530243
48291	HMOP	2020-10-01	41.17	41.18	41.125	41.125	7560	{}	\N	2020-10-12 15:08:32.535818
48292	HMOP	2020-10-02	41.165	41.21	41.1101	41.18	12854	{}	\N	2020-10-12 15:08:32.541372
48293	HMOP	2020-10-05	41.19	41.19	41.05	41.0855	7937	{}	\N	2020-10-12 15:08:32.546663
48294	HMOP	2020-10-06	41.09	41.11	41.0345	41.0345	16063	{}	\N	2020-10-12 15:08:32.551035
48295	HMOP	2020-10-07	41.07	41.0757	41.0092	41.04	16887	{}	\N	2020-10-12 15:08:32.555342
48296	HMOP	2020-10-08	41.05	41.07	40.9874	41.0101	8699	{}	\N	2020-10-12 15:08:32.559873
48297	HMOP	2020-10-09	41.09	41.09	40.99	41.0146	9685	{}	\N	2020-10-12 15:08:32.564193
48298	HMOP	2020-10-12	41.06	41.075	41.015	41.015	5374	{}	\N	2020-10-12 15:08:32.568516
48299	HOLD	2020-09-28	98.66	98.66	98.64	98.64	969	{}	\N	2020-10-12 15:08:34.052924
48300	HOLD	2020-09-29	98.7	98.72	98.68	98.72	1367	{}	\N	2020-10-12 15:08:34.061543
48301	HOLD	2020-09-30	98.72	98.762	98.705	98.705	449	{}	\N	2020-10-12 15:08:34.067437
48302	HOLD	2020-10-01	98.72	98.72	98.705	98.705	915	{}	\N	2020-10-12 15:08:34.073067
48303	HOLD	2020-10-02	98.755	98.755	98.755	98.755	51	{}	\N	2020-10-12 15:08:34.077871
48304	HOLD	2020-10-05	98.74	98.74	98.74	98.74	0	{}	\N	2020-10-12 15:08:34.082265
48305	HOLD	2020-10-06	98.79	98.79	98.79	98.79	0	{}	\N	2020-10-12 15:08:34.086582
48306	HOLD	2020-10-07	98.77	98.77	98.77	98.77	0	{}	\N	2020-10-12 15:08:34.091103
48307	HOLD	2020-10-08	98.8	98.8	98.8	98.8	0	{}	\N	2020-10-12 15:08:34.095432
48308	HOLD	2020-10-09	98.81	98.81	98.78	98.78	443	{}	\N	2020-10-12 15:08:34.099639
48309	HONE	2020-09-28	7.84	7.995	7.8	7.93	169946	{}	\N	2020-10-12 15:08:35.481911
48310	HONE	2020-09-29	7.8	8.08	7.75	7.97	154884	{}	\N	2020-10-12 15:08:35.494007
48311	HONE	2020-09-30	8	8.16	7.96	8.07	165925	{}	\N	2020-10-12 15:08:35.502253
48312	HONE	2020-10-01	8.09	8.11	7.92	8.08	88321	{}	\N	2020-10-12 15:08:35.508675
48313	HONE	2020-10-02	7.93	8.35	7.89	8.31	120055	{}	\N	2020-10-12 15:08:35.513984
48314	HONE	2020-10-05	8.4	8.64	8.18	8.62	156232	{}	\N	2020-10-12 15:08:35.518494
48315	HONE	2020-10-06	8.73	8.83	8.51	8.56	160328	{}	\N	2020-10-12 15:08:35.522713
48316	HONE	2020-10-07	8.64	8.855	8.44	8.82	175991	{}	\N	2020-10-12 15:08:35.527019
48317	HONE	2020-10-08	8.74	9.14	8.4904	8.95	158389	{}	\N	2020-10-12 15:08:35.53117
48318	HONE	2020-10-09	9.03	9.06	8.8	8.86	108436	{}	\N	2020-10-12 15:08:35.535268
48319	HONE	2020-10-12	8.84	9	8.815	9	13021	{}	\N	2020-10-12 15:08:35.539416
48320	HRI	2020-09-28	37.84	40.03	37.7	39.13	129077	{}	\N	2020-10-12 15:08:36.961303
48321	HRI	2020-09-29	39.28	39.6	38.14	38.91	210587	{}	\N	2020-10-12 15:08:36.973668
48322	HRI	2020-09-30	39.03	40.95	39.03	39.61	335787	{}	\N	2020-10-12 15:08:36.981324
48323	HRI	2020-10-01	39.93	40.45	39.33	40.38	225684	{}	\N	2020-10-12 15:08:36.98807
48324	HRI	2020-10-02	39	42.3125	39	42.08	262801	{}	\N	2020-10-12 15:08:36.993643
48325	HRI	2020-10-05	42.99	45.64	42.32	45	423455	{}	\N	2020-10-12 15:08:36.999397
48326	HRI	2020-10-06	45	45.87	43.8	44	344526	{}	\N	2020-10-12 15:08:37.004415
48327	HRI	2020-10-07	45.28	45.8	44.49	45.55	270677	{}	\N	2020-10-12 15:08:37.009328
48328	HRI	2020-10-08	46.2	46.34	45.185	46.12	187801	{}	\N	2020-10-12 15:08:37.014007
48329	HRI	2020-10-09	46.99	46.99	45.5446	45.9	160190	{}	\N	2020-10-12 15:08:37.018632
48330	HRI	2020-10-12	46.1	46.305	45.6	45.82	23396	{}	\N	2020-10-12 15:08:37.023222
48331	HDB	2020-09-28	48.9	49.29	48.51	48.54	1411568	{}	\N	2020-10-12 15:08:37.996737
48332	HDB	2020-09-29	48.78	49.11	48.41	48.96	2950942	{}	\N	2020-10-12 15:08:38.006662
48333	HDB	2020-09-30	49.59	50.59	49.21	49.96	3469829	{}	\N	2020-10-12 15:08:38.01421
48334	HDB	2020-10-01	51.05	51.98	50.995	51.78	2515745	{}	\N	2020-10-12 15:08:38.020135
48335	HDB	2020-10-02	51.44	52.18	51.075	51.77	1907508	{}	\N	2020-10-12 15:08:38.025095
48336	HDB	2020-10-05	52.24	52.89	52.16	52.52	2883942	{}	\N	2020-10-12 15:08:38.029516
48337	HDB	2020-10-06	53.75	54.11	53.25	53.29	2337101	{}	\N	2020-10-12 15:08:38.033155
48338	HDB	2020-10-07	54.05	54.4	53.85	53.95	1972040	{}	\N	2020-10-12 15:08:38.037206
48339	HDB	2020-10-08	54.89	55.3	54	54.85	3590352	{}	\N	2020-10-12 15:08:38.04084
48340	HDB	2020-10-09	56.31	56.68	56.08	56.22	2578099	{}	\N	2020-10-12 15:08:38.044395
48341	HDB	2020-10-12	55	55.96	55	55.72	532882	{}	\N	2020-10-12 15:08:38.048177
48342	HE	2020-09-28	33.29	33.525	33.15	33.18	285245	{}	\N	2020-10-12 15:08:39.234658
24147	AAPL	2020-09-28	114.98	115.32	112.78	114.96	134468196	{}	\N	2020-10-12 14:08:10.594341
24148	AAPL	2020-09-29	114.55	115.31	113.57	114.09	97892142	{}	\N	2020-10-12 14:08:10.613329
24149	AAPL	2020-09-30	113.77	117.25	113.62	115.81	136055079	{}	\N	2020-10-12 14:08:10.618739
24150	AAPL	2020-10-01	117.64	117.72	115.83	116.79	112824590	{}	\N	2020-10-12 14:08:10.623312
24151	AAPL	2020-10-02	112.89	115.37	112.22	113.02	139442161	{}	\N	2020-10-12 14:08:10.627807
24152	AAPL	2020-10-05	113.92	116.65	113.55	116.5	106010004	{}	\N	2020-10-12 14:08:10.633164
24153	AAPL	2020-10-06	115.7	116.12	112.25	113.16	155506372	{}	\N	2020-10-12 14:08:10.637783
24154	AAPL	2020-10-07	114.62	115.51	114.13	115.08	95862909	{}	\N	2020-10-12 14:08:10.641555
24155	AAPL	2020-10-08	116.27	116.4	114.5901	114.97	82756166	{}	\N	2020-10-12 14:08:10.646127
24156	AAPL	2020-10-09	115.28	117	114.92	116.97	98222295	{}	\N	2020-10-12 14:08:10.650709
24157	AAPL	2020-10-12	120.06	121.49	119.2845	120.7	40288800	{}	\N	2020-10-12 14:08:10.654691
24158	ABC	2020-09-28	97.69	98.76	96.79	97.58	640447	{}	\N	2020-10-12 14:08:12.231753
24159	ABC	2020-09-29	97.47	97.58	95.92	96.16	1260117	{}	\N	2020-10-12 14:08:12.239853
24160	ABC	2020-09-30	96.17	97.85	95.69	96.92	1888931	{}	\N	2020-10-12 14:08:12.246852
24161	ABC	2020-10-01	97.32	97.6265	94.73	95.36	1317261	{}	\N	2020-10-12 14:08:12.2526
24162	ABC	2020-10-02	94.05	96.55	93.96	95.38	1163947	{}	\N	2020-10-12 14:08:12.257944
24163	ABC	2020-10-05	95.77	97.01	94.8	96.39	1413319	{}	\N	2020-10-12 14:08:12.262087
24164	ABC	2020-10-06	97.15	97.72	95.46	95.63	1561891	{}	\N	2020-10-12 14:08:12.266549
24165	ABC	2020-10-07	96.02	96.45	95.56	95.59	1357488	{}	\N	2020-10-12 14:08:12.271316
24166	ABC	2020-10-08	96.1	98.98	95.82	98.49	2010980	{}	\N	2020-10-12 14:08:12.275851
24167	ABC	2020-10-09	98.9	98.99	96.57	96.86	1443346	{}	\N	2020-10-12 14:08:12.280393
24168	ABC	2020-10-12	97.38	97.38	96.45	97.3	123757	{}	\N	2020-10-12 14:08:12.285216
24169	ABEO	2020-09-28	1.45	1.46	1.07	1.09	14045577	{}	\N	2020-10-12 14:08:14.292938
24170	ABEO	2020-09-29	1.15	1.16	1.09	1.105	6688329	{}	\N	2020-10-12 14:08:14.303423
24171	ABEO	2020-09-30	1.1101	1.13	1	1.02	6551469	{}	\N	2020-10-12 14:08:14.314261
24172	ABEO	2020-10-01	1.05	1.06	1	1.05	4825651	{}	\N	2020-10-12 14:08:14.319923
24173	ABEO	2020-10-02	1.03	1.12	1.02	1.05	5633493	{}	\N	2020-10-12 14:08:14.324835
24174	ABEO	2020-10-05	1.05	1.11	1.05	1.09	2302380	{}	\N	2020-10-12 14:08:14.328954
24175	ABEO	2020-10-06	1.12	1.27	1.11	1.22	6002575	{}	\N	2020-10-12 14:08:14.333589
24176	ABEO	2020-10-07	1.2	1.335	1.17	1.23	3197397	{}	\N	2020-10-12 14:08:14.338031
24177	ABEO	2020-10-08	1.26	1.4	1.23	1.38	3051118	{}	\N	2020-10-12 14:08:14.342346
24178	ABEO	2020-10-09	1.46	1.58	1.36	1.38	4235638	{}	\N	2020-10-12 14:08:14.346699
24179	ABEO	2020-10-12	1.4	1.44	1.33	1.375	563669	{}	\N	2020-10-12 14:08:14.351213
24180	ABM	2020-09-28	37.29	37.86	37.1	37.46	485359	{}	\N	2020-10-12 14:08:18.365364
24181	ABM	2020-09-29	37.47	37.58	36.41	36.92	683303	{}	\N	2020-10-12 14:08:18.37673
24182	ABM	2020-09-30	36.95	37.47	36.36	36.66	790409	{}	\N	2020-10-12 14:08:18.384441
24183	ABM	2020-10-01	36.77	37.05	36.35	36.74	520725	{}	\N	2020-10-12 14:08:18.390916
24184	ABM	2020-10-02	36.08	37.04	36.08	36.7	649903	{}	\N	2020-10-12 14:08:18.396868
24185	ABM	2020-10-05	36.9	37.1	36.58	36.83	757940	{}	\N	2020-10-12 14:08:18.402081
24186	ABM	2020-10-06	36.97	37.775	36.62	36.92	1438761	{}	\N	2020-10-12 14:08:18.406981
24187	ABM	2020-10-07	37.19	37.46	36.745	37.39	1167002	{}	\N	2020-10-12 14:08:18.411669
24188	ABM	2020-10-08	37.59	37.9	37.14	37.48	631607	{}	\N	2020-10-12 14:08:18.41624
24189	ABM	2020-10-09	37.6	37.69	36.73	37.01	530672	{}	\N	2020-10-12 14:08:18.420609
24190	ABM	2020-10-12	36.91	37.28	36.78	37.2375	32791	{}	\N	2020-10-12 14:08:18.425088
24191	ABR	2020-09-28	11.82	12.06	11.73	11.88	1087222	{}	\N	2020-10-12 14:08:20.946763
24192	ABR	2020-09-29	11.9	11.925	11.425	11.51	1446727	{}	\N	2020-10-12 14:08:20.956133
24193	ABR	2020-09-30	11.6	11.77	11.375	11.47	1504271	{}	\N	2020-10-12 14:08:20.962386
24194	ABR	2020-10-01	11.4901	11.74	11.3135	11.72	1270854	{}	\N	2020-10-12 14:08:20.967894
24195	ABR	2020-10-02	11.32	11.95	11.02	11.85	1806100	{}	\N	2020-10-12 14:08:20.973506
24196	ABR	2020-10-05	11.9	12.08	11.74	11.83	2074550	{}	\N	2020-10-12 14:08:20.978355
24197	ABR	2020-10-06	11.9	11.985	11.53	11.55	1958604	{}	\N	2020-10-12 14:08:20.982968
24198	ABR	2020-10-07	11.71	11.96	11.54	11.73	1681135	{}	\N	2020-10-12 14:08:20.987426
24199	ABR	2020-10-08	11.74	11.93	11.6946	11.89	2453273	{}	\N	2020-10-12 14:08:20.991752
24200	ABR	2020-10-09	11.9	12.01	11.59	11.74	1828991	{}	\N	2020-10-12 14:08:20.995796
24201	ABR	2020-10-12	11.8	11.8936	11.74	11.88	300975	{}	\N	2020-10-12 14:08:21.00093
24202	AC	2020-09-28	36.28	37.3	35.9	36.89	17048	{}	\N	2020-10-12 14:08:22.479971
24203	AC	2020-09-29	36.59	36.76	36.31	36.58	17539	{}	\N	2020-10-12 14:08:22.490163
24204	AC	2020-09-30	36.06	36.72	35.905	36.13	20535	{}	\N	2020-10-12 14:08:22.496502
24205	AC	2020-10-01	35.96	36.58	35.38	35.38	15032	{}	\N	2020-10-12 14:08:22.501883
24206	AC	2020-10-02	35.2	36.18	34.39	36.18	24747	{}	\N	2020-10-12 14:08:22.507491
24207	AC	2020-10-05	36.23	36.51	35.51	35.76	12472	{}	\N	2020-10-12 14:08:22.512616
24208	AC	2020-10-06	35.9	36.01	33.07	33.07	35868	{}	\N	2020-10-12 14:08:22.517834
24209	AC	2020-10-07	33.1	34.47	33.1	33.51	15108	{}	\N	2020-10-12 14:08:22.522617
24210	AC	2020-10-08	33.86	35.25	33.86	34.11	21335	{}	\N	2020-10-12 14:08:22.527366
24211	AC	2020-10-09	34.6	35.32	34.3101	34.52	13809	{}	\N	2020-10-12 14:08:22.532384
24212	AC	2020-10-12	35.31	35.31	35.09	35.1	1440	{}	\N	2020-10-12 14:08:22.537388
24213	ABTX	2020-09-28	22.47	23.26	22.47	23.02	46114	{}	\N	2020-10-12 14:08:24.32733
24214	ABTX	2020-09-29	22.86	23.54	22.25	23.25	79614	{}	\N	2020-10-12 14:08:24.337823
24215	ABTX	2020-09-30	23.38	23.86	23.14	23.37	70432	{}	\N	2020-10-12 14:08:24.344498
24216	ABTX	2020-10-01	23.4	23.63	23.05	23.59	31226	{}	\N	2020-10-12 14:08:24.35086
24217	ABTX	2020-10-02	22.73	24.4	22.73	24.2	41154	{}	\N	2020-10-12 14:08:24.355961
24218	ABTX	2020-10-05	24.44	25.24	24.165	25.22	49360	{}	\N	2020-10-12 14:08:24.360586
24219	ABTX	2020-10-06	25.4	26.21	24.84	24.88	77421	{}	\N	2020-10-12 14:08:24.364653
24220	ABTX	2020-10-07	25.06	25.86	24.91	25.77	58302	{}	\N	2020-10-12 14:08:24.368672
24221	ABTX	2020-10-08	25.98	26.26	25.65	26.02	63196	{}	\N	2020-10-12 14:08:24.372609
24222	ABTX	2020-10-09	26.22	26.35	25.9873	26.19	34610	{}	\N	2020-10-12 14:08:24.376582
24223	ABTX	2020-10-12	26.19	26.36	26.125	26.25	4325	{}	\N	2020-10-12 14:08:24.380626
24224	ACBI	2020-09-28	10.77	11.38	10.77	11.24	85274	{}	\N	2020-10-12 14:08:25.816858
24225	ACBI	2020-09-29	11.32	11.48	11.08	11.31	72635	{}	\N	2020-10-12 14:08:25.825581
24226	ACBI	2020-09-30	11.38	11.53	11.23	11.35	61876	{}	\N	2020-10-12 14:08:25.831608
24227	ACBI	2020-10-01	11.37	11.59	11.33	11.49	81307	{}	\N	2020-10-12 14:08:25.836831
24228	ACBI	2020-10-02	11.2	12.065	11.1	12.01	86868	{}	\N	2020-10-12 14:08:25.841162
24229	ACBI	2020-10-05	12.11	12.45	12.11	12.43	83551	{}	\N	2020-10-12 14:08:25.846158
24230	ACBI	2020-10-06	12.49	13.22	12.49	12.64	79278	{}	\N	2020-10-12 14:08:25.850545
24231	ACBI	2020-10-07	12.79	13.22	12.79	13.01	77393	{}	\N	2020-10-12 14:08:25.854978
24232	ACBI	2020-10-08	13.19	13.43	13.09	13.25	76721	{}	\N	2020-10-12 14:08:25.859562
24233	ACBI	2020-10-09	13.24	13.4	13.14	13.26	46823	{}	\N	2020-10-12 14:08:25.865135
24234	ACBI	2020-10-12	13.23	13.42	12.945	13.41	7305	{}	\N	2020-10-12 14:08:25.869857
24235	AAP	2020-09-28	151.03	154.165	150.01	153.39	756747	{}	\N	2020-10-12 14:08:27.084738
24236	AAP	2020-09-29	153.89	153.89	151.66	151.71	1156658	{}	\N	2020-10-12 14:08:27.094623
24237	AAP	2020-09-30	151.48	154.865	151.445	153.5	1011950	{}	\N	2020-10-12 14:08:27.100933
24238	AAP	2020-10-01	153.56	156.22	153.195	154.85	1210392	{}	\N	2020-10-12 14:08:27.106524
24239	AAP	2020-10-02	152.24	156.35	151.66	155.5	873855	{}	\N	2020-10-12 14:08:27.111653
24240	AAP	2020-10-05	155.74	158.16	154.92	155.86	1111860	{}	\N	2020-10-12 14:08:27.116175
24241	AAP	2020-10-06	160.26	160.775	155.06	155.16	1559098	{}	\N	2020-10-12 14:08:27.120094
24242	AAP	2020-10-07	156.54	158.5004	155.2828	156.81	2015922	{}	\N	2020-10-12 14:08:27.125068
24243	AAP	2020-10-08	157.8	158.5	155.115	157.66	802474	{}	\N	2020-10-12 14:08:27.129708
24244	AAP	2020-10-09	158.58	158.58	155.07	155.32	1602534	{}	\N	2020-10-12 14:08:27.1342
24245	AAP	2020-10-12	155.51	155.752	153.63	154.52	182384	{}	\N	2020-10-12 14:08:27.138544
24246	ABBV	2020-09-28	87.29	88.2	86.87	87.28	6148153	{}	\N	2020-10-12 14:08:28.385261
24247	ABBV	2020-09-29	87.11	87.28	86.31	86.89	10477521	{}	\N	2020-10-12 14:08:28.390881
24248	ABBV	2020-09-30	87	88.44	86.81	87.59	15409355	{}	\N	2020-10-12 14:08:28.396151
24249	ABBV	2020-10-01	88.17	88.17	86.73	87.14	16341905	{}	\N	2020-10-12 14:08:28.400385
24250	ABBV	2020-10-02	86.5	87.35	86.05	86.12	10025248	{}	\N	2020-10-12 14:08:28.405537
24251	ABBV	2020-10-05	86.49	88.18	86.21	87.93	12128250	{}	\N	2020-10-12 14:08:28.410309
24252	ABBV	2020-10-06	87.97	88.06	85.71	85.89	15376128	{}	\N	2020-10-12 14:08:28.415406
24253	ABBV	2020-10-07	86.16	87.2977	86.03	87.07	11289917	{}	\N	2020-10-12 14:08:28.41982
24254	ABBV	2020-10-08	87.27	88.58	86.92	87.34	11090154	{}	\N	2020-10-12 14:08:28.424572
24255	ABBV	2020-10-09	87.41	88.7	87.35	87.7	11554320	{}	\N	2020-10-12 14:08:28.429739
24256	ABBV	2020-10-12	88.17	88.23	87.32	87.54	2743928	{}	\N	2020-10-12 14:08:28.43457
24257	ABCB	2020-09-28	22.11	23.08	22.11	22.81	372320	{}	\N	2020-10-12 14:08:29.891714
24258	ABCB	2020-09-29	22.64	22.785	22.0736	22.76	358394	{}	\N	2020-10-12 14:08:29.903986
24259	ABCB	2020-09-30	23.08	23.35	22.54	22.78	358874	{}	\N	2020-10-12 14:08:29.91669
24260	ABCB	2020-10-01	22.8	23.08	22.37	23.03	330398	{}	\N	2020-10-12 14:08:29.928515
24261	ABCB	2020-10-02	22.61	23.915	22.61	23.82	287375	{}	\N	2020-10-12 14:08:29.937658
24262	ABCB	2020-10-05	24	25.06	23.45	24.84	386133	{}	\N	2020-10-12 14:08:29.949808
24263	ABCB	2020-10-06	25.38	26.44	24.82	25.38	771539	{}	\N	2020-10-12 14:08:29.960803
24264	ABCB	2020-10-07	25.87	26.5	25.39	25.62	473884	{}	\N	2020-10-12 14:08:29.975949
24265	ABCB	2020-10-08	25.97	26.63	25.43	26.43	258229	{}	\N	2020-10-12 14:08:29.986572
24266	ABCB	2020-10-09	26.59	27.11	25.7	25.88	299525	{}	\N	2020-10-12 14:08:29.994629
24267	ABCB	2020-10-12	25.81	26.22	25.7	26.195	9283	{}	\N	2020-10-12 14:08:30.001723
24268	ABG	2020-09-28	95.34	98.93	95.34	98.66	172294	{}	\N	2020-10-12 14:08:32.695508
24269	ABG	2020-09-29	99.17	99.17	93.79	95.5	261998	{}	\N	2020-10-12 14:08:32.702259
24270	ABG	2020-09-30	95.7	98.55	95.51	97.45	427675	{}	\N	2020-10-12 14:08:32.708261
24271	ABG	2020-10-01	97.53	101.01	95.99	100.3	337990	{}	\N	2020-10-12 14:08:32.713704
24272	ABG	2020-10-02	99.24	105.31	98.18	103.73	399765	{}	\N	2020-10-12 14:08:32.717953
24273	ABG	2020-10-05	104.26	110.37	104.26	109.88	238433	{}	\N	2020-10-12 14:08:32.721922
24274	ABG	2020-10-06	112.62	121.155	111.91	112.56	612001	{}	\N	2020-10-12 14:08:32.725969
24275	ABG	2020-10-07	114.05	117.445	112.13	113.71	445530	{}	\N	2020-10-12 14:08:32.7303
24276	ABG	2020-10-08	115.42	116.085	113.11	115.15	265118	{}	\N	2020-10-12 14:08:32.735841
24277	ABG	2020-10-09	116.11	117.33	113.71	114.7	191816	{}	\N	2020-10-12 14:08:32.739888
24278	ABG	2020-10-12	114.84	115.34	113.9	114.4	10999	{}	\N	2020-10-12 14:08:32.743995
24279	ABIO	2020-09-28	4.6127	4.65	4.3888	4.55	507148	{}	\N	2020-10-12 14:08:34.270144
24280	ABIO	2020-09-29	4.49	4.535	4.37	4.44	371823	{}	\N	2020-10-12 14:08:34.281642
24281	ABIO	2020-09-30	4.38	4.555	4.38	4.42	217263	{}	\N	2020-10-12 14:08:34.288808
24282	ABIO	2020-10-01	4.45	4.4899	4.27	4.43	273861	{}	\N	2020-10-12 14:08:34.294839
24283	ABIO	2020-10-02	4.32	4.6	4.2038	4.57	556713	{}	\N	2020-10-12 14:08:34.299893
24284	ABIO	2020-10-05	4.62	4.715	4.42	4.6	428657	{}	\N	2020-10-12 14:08:34.306969
24285	ABIO	2020-10-06	4.64	5.23	4.6012	5.18	1988165	{}	\N	2020-10-12 14:08:34.311043
24286	ABIO	2020-10-07	5.35	5.3999	4.62	4.74	4938368	{}	\N	2020-10-12 14:08:34.315373
24287	ABIO	2020-10-08	4.58	4.58	4.32	4.38	1406341	{}	\N	2020-10-12 14:08:34.319327
24288	ABIO	2020-10-09	4.4	4.48	4.26	4.38	569198	{}	\N	2020-10-12 14:08:34.323206
24289	ABIO	2020-10-12	4.43	4.4301	4.28	4.3311	84367	{}	\N	2020-10-12 14:08:34.327319
24290	ABMD	2020-09-28	269.98	275.32	268.48	273.55	291425	{}	\N	2020-10-12 14:08:35.537747
24291	ABMD	2020-09-29	271.31	279.44	271.31	272.46	181397	{}	\N	2020-10-12 14:08:35.542225
24292	ABMD	2020-09-30	275.12	280.875	272.22	277.06	328711	{}	\N	2020-10-12 14:08:35.546975
24293	ABMD	2020-10-01	277.14	280.73	258.29	260.45	965716	{}	\N	2020-10-12 14:08:35.551573
24294	ABMD	2020-10-02	256.48	265.41	255.405	262.12	385865	{}	\N	2020-10-12 14:08:35.555835
24295	ABMD	2020-10-05	263.91	270.69	263.01	270.69	312233	{}	\N	2020-10-12 14:08:35.56026
24296	ABMD	2020-10-06	268.81	271.17	263.565	264.82	345551	{}	\N	2020-10-12 14:08:35.564473
24297	ABMD	2020-10-07	270	272.795	267.83	268.54	193811	{}	\N	2020-10-12 14:08:35.568759
24298	ABMD	2020-10-08	271	272.93	267.13	271.2	225946	{}	\N	2020-10-12 14:08:35.573351
24299	ABMD	2020-10-09	273.3	276.24	271.56	273.56	144389	{}	\N	2020-10-12 14:08:35.57732
24300	ABMD	2020-10-12	274.8	275.9032	273.05	273.985	35591	{}	\N	2020-10-12 14:08:35.581371
24301	ABT	2020-09-28	104.24	105.25	103.82	104.44	5431388	{}	\N	2020-10-12 14:08:36.980898
24302	ABT	2020-09-29	106.01	106.41	104.92	105.19	8646280	{}	\N	2020-10-12 14:08:36.990893
24303	ABT	2020-09-30	105.99	109.48	105.74	108.83	18000377	{}	\N	2020-10-12 14:08:36.997086
24304	ABT	2020-10-01	109.18	110.33	108.38	108.64	13228283	{}	\N	2020-10-12 14:08:37.002669
24305	ABT	2020-10-02	107.68	108.66	106.33	106.5	10170900	{}	\N	2020-10-12 14:08:37.007957
24306	ABT	2020-10-05	107.18	108.65	107.18	108.56	7240307	{}	\N	2020-10-12 14:08:37.012953
24307	ABT	2020-10-06	108.56	108.77	106.05	106.25	7173170	{}	\N	2020-10-12 14:08:37.017419
24308	ABT	2020-10-07	107.37	108.18	106.93	107.74	7958948	{}	\N	2020-10-12 14:08:37.02206
24309	ABT	2020-10-08	108.14	109.13	107.42	108.54	8352826	{}	\N	2020-10-12 14:08:37.026181
24310	ABT	2020-10-09	109.34	109.97	108.8	109.65	5843805	{}	\N	2020-10-12 14:08:37.03026
24311	ABT	2020-10-12	110.25	110.64	109.76	110.37	1336720	{}	\N	2020-10-12 14:08:37.034532
24312	ABUS	2020-09-28	3.12	3.13	2.95	3.03	981533	{}	\N	2020-10-12 14:08:38.191693
24313	ABUS	2020-09-29	3	3.24	2.95	3.18	1352518	{}	\N	2020-10-12 14:08:38.202269
24314	ABUS	2020-09-30	3.17	3.2	3.06	3.13	860455	{}	\N	2020-10-12 14:08:38.209565
24315	ABUS	2020-10-01	3.17	3.17	3.055	3.09	660005	{}	\N	2020-10-12 14:08:38.215163
24316	ABUS	2020-10-02	3	3.13	2.97	3.01	1264020	{}	\N	2020-10-12 14:08:38.21967
24317	ABUS	2020-10-05	3.04	3.26	3	3.26	1536448	{}	\N	2020-10-12 14:08:38.224094
24318	ABUS	2020-10-06	3.26	3.3	3.03	3.08	1042981	{}	\N	2020-10-12 14:08:38.228275
24319	ABUS	2020-10-07	3.1	3.38	3.09	3.32	1683592	{}	\N	2020-10-12 14:08:38.232468
24320	ABUS	2020-10-08	3.39	3.44	3.13	3.14	1681599	{}	\N	2020-10-12 14:08:38.236398
24321	ABUS	2020-10-09	3.16	3.23	3.12	3.2	824614	{}	\N	2020-10-12 14:08:38.240041
24322	ABUS	2020-10-12	3.21	3.21	3.1	3.1298	136312	{}	\N	2020-10-12 14:08:38.244304
24323	ACAD	2020-09-28	40.58	40.67	39.67	40.23	616464	{}	\N	2020-10-12 14:08:39.450496
24324	ACAD	2020-09-29	40.1	40.93	39.8802	40.41	576054	{}	\N	2020-10-12 14:08:39.467
24325	ACAD	2020-09-30	41.11	41.64	40.56	41.25	824685	{}	\N	2020-10-12 14:08:39.473614
24326	ACAD	2020-10-01	41.36	42.36	41.24	42.14	668160	{}	\N	2020-10-12 14:08:39.481885
24327	ACAD	2020-10-02	41.39	42.22	40.87	41.36	976116	{}	\N	2020-10-12 14:08:39.486593
24328	ACAD	2020-10-05	41.82	42.395	41.54	42.14	1158466	{}	\N	2020-10-12 14:08:39.491283
24329	ACAD	2020-10-06	42.2	42.7	41.48	41.7	545801	{}	\N	2020-10-12 14:08:39.495413
24330	ACAD	2020-10-07	41.95	43.15	41.93	42.89	720818	{}	\N	2020-10-12 14:08:39.499758
24331	ACAD	2020-10-08	43.21	43.21	42.16	43.18	629534	{}	\N	2020-10-12 14:08:39.503858
24332	ACAD	2020-10-09	43.17	43.3	42.41	42.73	502472	{}	\N	2020-10-12 14:08:39.508265
24333	ACAD	2020-10-12	43.19	43.3	42.5701	43.15	51891	{}	\N	2020-10-12 14:08:39.513116
24334	ACER	2020-09-28	2.54	2.8	2.49	2.75	234049	{}	\N	2020-10-12 14:08:40.784287
24335	ACER	2020-09-29	2.86	3.18	2.82	3.13	786872	{}	\N	2020-10-12 14:08:40.796319
24336	ACER	2020-09-30	3.3	3.31	2.86	2.88	650279	{}	\N	2020-10-12 14:08:40.803634
24337	ACER	2020-10-01	2.81	3.0577	2.79	2.97	175403	{}	\N	2020-10-12 14:08:40.809407
24338	ACER	2020-10-02	2.88	2.986	2.72	2.8401	125962	{}	\N	2020-10-12 14:08:40.815078
24339	ACER	2020-10-05	2.88	3.05	2.87	2.91	150215	{}	\N	2020-10-12 14:08:40.819791
24340	ACER	2020-10-06	2.91	3.06	2.91	2.92	174345	{}	\N	2020-10-12 14:08:40.824653
24341	ACER	2020-10-07	2.94	3.04	2.94	2.98	113562	{}	\N	2020-10-12 14:08:40.830274
24342	ACER	2020-10-08	3.06	3.06	2.95	2.96	59033	{}	\N	2020-10-12 14:08:40.835149
24343	ACER	2020-10-09	3.02	3.04	2.89	2.96	93183	{}	\N	2020-10-12 14:08:40.840226
24344	ACER	2020-10-12	2.96	2.9899	2.91	2.9591	18188	{}	\N	2020-10-12 14:08:40.845049
48343	HE	2020-09-29	33.21	33.34	32.74	33.02	535372	{}	\N	2020-10-12 15:08:39.247683
48344	HE	2020-09-30	33.27	33.39	32.95	33.24	1011828	{}	\N	2020-10-12 15:08:39.254341
48345	HE	2020-10-01	33.22	33.52	33.1	33.23	601577	{}	\N	2020-10-12 15:08:39.260526
48346	HE	2020-10-02	33	34.05	33	33.89	543041	{}	\N	2020-10-12 15:08:39.265453
48347	HE	2020-10-05	34.02	34.355	33.72	34.3	564883	{}	\N	2020-10-12 15:08:39.271182
48348	HE	2020-10-06	34.3	35.24	34.15	34.76	812073	{}	\N	2020-10-12 15:08:39.275852
48349	HE	2020-10-07	34.81	35.01	34.32	34.59	579454	{}	\N	2020-10-12 15:08:39.279914
48350	HE	2020-10-08	34.65	34.94	34.42	34.68	422763	{}	\N	2020-10-12 15:08:39.284137
48351	HE	2020-10-09	34.8	34.84	34.28	34.47	665502	{}	\N	2020-10-12 15:08:39.28809
48352	HE	2020-10-12	34.46	34.84	34.37	34.83	82510	{}	\N	2020-10-12 15:08:39.292194
48353	HES	2020-09-28	41.31	41.545	40.28	41.03	1893147	{}	\N	2020-10-12 15:08:40.482369
48354	HES	2020-09-29	40.83	40.885	39.77	40.69	3678274	{}	\N	2020-10-12 15:08:40.489388
48355	HES	2020-09-30	40.47	41.58	40.47	40.93	3648194	{}	\N	2020-10-12 15:08:40.495234
48356	HES	2020-10-01	40	40.38	38.66	38.98	6079233	{}	\N	2020-10-12 15:08:40.501129
48357	HES	2020-10-02	37.49	38.66	37.47	37.71	6298187	{}	\N	2020-10-12 15:08:40.506795
48358	HES	2020-10-05	38.32	39.38	38.18	39.36	5167791	{}	\N	2020-10-12 15:08:40.510939
48359	HES	2020-10-06	40.15	40.56	37.8438	37.97	5164862	{}	\N	2020-10-12 15:08:40.514873
48360	HES	2020-10-07	38.21	38.65	36.62	37.68	7106090	{}	\N	2020-10-12 15:08:40.518659
48361	HES	2020-10-08	38.17	38.71	37.675	38.59	4354049	{}	\N	2020-10-12 15:08:40.522717
48362	HES	2020-10-09	39.13	39.3974	37.73	38.06	3229137	{}	\N	2020-10-12 15:08:40.526691
48363	HES	2020-10-12	37.78	38.14	37.02	38.0856	1029222	{}	\N	2020-10-12 15:08:40.530467
48364	HGH	2020-09-28	28.25	28.29	28.15	28.22	40987	{}	\N	2020-10-12 15:08:42.00443
48365	HGH	2020-09-29	28.2	28.336	28.2	28.26	98370	{}	\N	2020-10-12 15:08:42.014366
48366	HGH	2020-09-30	27.94	27.9835	27.61	27.67	231333	{}	\N	2020-10-12 15:08:42.021521
48367	HGH	2020-10-01	27.75	27.9	27.71	27.76	119174	{}	\N	2020-10-12 15:08:42.026811
48368	HGH	2020-10-02	27.66	27.94	27.614	27.94	93406	{}	\N	2020-10-12 15:08:42.031692
48369	HGH	2020-10-05	27.97	27.97	27.87	27.93	129704	{}	\N	2020-10-12 15:08:42.036086
48370	HGH	2020-10-06	27.95	27.9799	27.77	27.85	80822	{}	\N	2020-10-12 15:08:42.040633
48371	HGH	2020-10-07	27.89	27.97	27.85	27.88	57765	{}	\N	2020-10-12 15:08:42.045119
48372	HGH	2020-10-08	27.82	27.94	27.8	27.8	93842	{}	\N	2020-10-12 15:08:42.050224
48373	HGH	2020-10-09	27.85	27.9278	27.84	27.91	63400	{}	\N	2020-10-12 15:08:42.054485
48374	HGH	2020-10-12	27.85	27.92	27.85	27.92	4584	{}	\N	2020-10-12 15:08:42.058736
48375	HIBB	2020-09-28	38.08	39.39	37.8801	38.62	347857	{}	\N	2020-10-12 15:08:43.046218
48376	HIBB	2020-09-29	38.77	38.84	37.36	37.99	355517	{}	\N	2020-10-12 15:08:43.056987
48377	HIBB	2020-09-30	38.19	40.81	38.17	39.22	994120	{}	\N	2020-10-12 15:08:43.063884
48378	HIBB	2020-10-01	39.5	42.05	38.512	41.57	592825	{}	\N	2020-10-12 15:08:43.069298
48379	HIBB	2020-10-02	40.75	44.94	40.75	44.61	906302	{}	\N	2020-10-12 15:08:43.074022
48447	HPS	2020-10-06	16.21	16.3695	16.21	16.27	108757	{}	\N	2020-10-12 15:08:51.062878
48448	HPS	2020-10-07	16.29	16.3896	16.2873	16.33	98206	{}	\N	2020-10-12 15:08:51.067092
48449	HPS	2020-10-08	16.43	16.47	16.3501	16.45	95776	{}	\N	2020-10-12 15:08:51.070472
48450	HPS	2020-10-09	16.37	16.437	16.25	16.26	115086	{}	\N	2020-10-12 15:08:51.07448
48451	HPS	2020-10-12	16.25	16.2573	16.2265	16.2547	23568	{}	\N	2020-10-12 15:08:51.078068
48452	HR	2020-09-28	29.38	30.075	29.19	29.91	1118429	{}	\N	2020-10-12 15:08:52.001022
48453	HR	2020-09-29	29.87	29.87	29.18	29.64	1841077	{}	\N	2020-10-12 15:08:52.008977
48454	HR	2020-09-30	29.92	30.45	29.825	30.12	3522993	{}	\N	2020-10-12 15:08:52.014548
48455	HR	2020-10-01	30.06	30.53	29.7	30.5	2653271	{}	\N	2020-10-12 15:08:52.019149
48456	HR	2020-10-02	30	31.12	29.78	30.92	3530022	{}	\N	2020-10-12 15:08:52.023997
48457	HR	2020-10-05	31.1	31.315	30.36	30.73	2047356	{}	\N	2020-10-12 15:08:52.027864
48458	HR	2020-10-06	30.86	31.26	30.27	30.55	2601644	{}	\N	2020-10-12 15:08:52.031743
48459	HR	2020-10-07	30.64	30.64	30.1	30.34	2258428	{}	\N	2020-10-12 15:08:52.035829
48460	HR	2020-10-08	30.49	30.97	30.32	30.67	1895363	{}	\N	2020-10-12 15:08:52.040058
48461	HR	2020-10-09	30.85	30.96	30.13	30.34	938131	{}	\N	2020-10-12 15:08:52.043933
48462	HR	2020-10-12	30.26	30.48	30.05	30.3932	178411	{}	\N	2020-10-12 15:08:52.048341
48463	HSIC	2020-09-28	59.84	60.41	59.39	59.65	679697	{}	\N	2020-10-12 15:08:53.70483
48464	HSIC	2020-09-29	59.64	59.64	58.41	58.57	717786	{}	\N	2020-10-12 15:08:53.715957
48465	HSIC	2020-09-30	58.6	59.55	58.037	58.78	1077143	{}	\N	2020-10-12 15:08:53.722965
48466	HSIC	2020-10-01	58.98	59.665	57.52	57.92	772249	{}	\N	2020-10-12 15:08:53.729332
48467	HSIC	2020-10-02	56.97	58.145	56.75	57.97	1199389	{}	\N	2020-10-12 15:08:53.735241
48468	HSIC	2020-10-05	58.67	60.635	58.215	60.51	873215	{}	\N	2020-10-12 15:08:53.741123
48469	HSIC	2020-10-06	60.51	60.61	58.56	58.84	778319	{}	\N	2020-10-12 15:08:53.745829
48470	HSIC	2020-10-07	59.17	60.2334	58.89	59.46	1059568	{}	\N	2020-10-12 15:08:53.750349
48471	HSIC	2020-10-08	60.13	61.32	59.56	61.09	766922	{}	\N	2020-10-12 15:08:53.754742
48472	HSIC	2020-10-09	61.63	61.815	60.28	60.46	444942	{}	\N	2020-10-12 15:08:53.759386
48473	HSIC	2020-10-12	60.55	60.56	59.72	60.37	80413	{}	\N	2020-10-12 15:08:53.764161
48474	HD	2020-09-28	271.93	274.22	270.3	272.33	3061050	{}	\N	2020-10-12 15:08:55.10288
48475	HD	2020-09-29	273.4	273.8415	271.1138	272.11	3927331	{}	\N	2020-10-12 15:08:55.115164
48476	HD	2020-09-30	273.48	279.915	272.52	277.71	9456748	{}	\N	2020-10-12 15:08:55.122217
48477	HD	2020-10-01	279.44	280	275.85	277.62	5295300	{}	\N	2020-10-12 15:08:55.128142
48478	HD	2020-10-02	274.94	281.08	274.1032	279.31	5504140	{}	\N	2020-10-12 15:08:55.132995
48479	HD	2020-10-05	280.92	282.42	279.7	282.1	4671260	{}	\N	2020-10-12 15:08:55.137705
48480	HD	2020-10-06	282.5	283	275.77	276.47	5626386	{}	\N	2020-10-12 15:08:55.142187
48481	HD	2020-10-07	280.85	284	280	282.79	6617439	{}	\N	2020-10-12 15:08:55.146414
48482	HD	2020-10-08	283.84	284.95	281.1707	284.52	4436593	{}	\N	2020-10-12 15:08:55.150589
48483	HD	2020-10-09	285.47	287.28	284.09	285.92	4345089	{}	\N	2020-10-12 15:08:55.154572
48484	HD	2020-10-12	287.39	287.45	286.06	287	1015725	{}	\N	2020-10-12 15:08:55.158693
48485	HDV	2020-09-28	80.24	80.9917	80.24	80.57	365098	{}	\N	2020-10-12 15:08:56.47743
48486	HDV	2020-09-29	80.6	80.6	79.53	79.83	332764	{}	\N	2020-10-12 15:08:56.49009
48487	HDV	2020-09-30	80.35	81.04	80.01	80.45	247458	{}	\N	2020-10-12 15:08:56.496998
48488	HDV	2020-10-01	80.59	80.72	79.65	80.02	355748	{}	\N	2020-10-12 15:08:56.503159
48489	HDV	2020-10-02	79.16	80.58	79.01	80.07	405584	{}	\N	2020-10-12 15:08:56.50857
48490	HDV	2020-10-05	80.64	81.23	80.27	81.16	179644	{}	\N	2020-10-12 15:08:56.513741
48491	HDV	2020-10-06	81.51	82.0575	80.53	80.65	246730	{}	\N	2020-10-12 15:08:56.517785
48492	HDV	2020-10-07	81.11	81.7391	81.01	81.53	358282	{}	\N	2020-10-12 15:08:56.521853
48493	HDV	2020-10-08	81.92	82.75	81.86	82.68	193359	{}	\N	2020-10-12 15:08:56.525952
48494	HDV	2020-10-09	83.04	83.2299	82.44	82.59	261953	{}	\N	2020-10-12 15:08:56.529878
48495	HDV	2020-10-12	82.69	82.96	82.4	82.85	82764	{}	\N	2020-10-12 15:08:56.534065
48496	HELE	2020-09-28	187.57	197.25	187.1	197.23	172908	{}	\N	2020-10-12 15:08:58.580263
48497	HELE	2020-09-29	197.8	201.39	196.16	199.4	289965	{}	\N	2020-10-12 15:08:58.589789
48498	HELE	2020-09-30	201.41	201.41	192.08	193.52	161330	{}	\N	2020-10-12 15:08:58.595953
48499	HELE	2020-10-01	193.29	202.89	190.11	199.69	242012	{}	\N	2020-10-12 15:08:58.601629
48500	HELE	2020-10-02	193.7901	201.46	193.7901	199.83	138356	{}	\N	2020-10-12 15:08:58.606481
48501	HELE	2020-10-05	201.5	207.46	201.5	207.19	225321	{}	\N	2020-10-12 15:08:58.610932
48502	HELE	2020-10-06	209.88	209.88	202.5	203.31	228454	{}	\N	2020-10-12 15:08:58.615093
48503	HELE	2020-10-07	206.05	207.9	203.61	205	345519	{}	\N	2020-10-12 15:08:58.619129
48504	HELE	2020-10-08	200.33	201.55	185.05	194.5	962295	{}	\N	2020-10-12 15:08:58.623219
48505	HELE	2020-10-09	197.19	200.87	196.22	198.11	481693	{}	\N	2020-10-12 15:08:58.627087
48506	HELE	2020-10-12	198.11	200	198.11	199.725	58332	{}	\N	2020-10-12 15:08:58.630867
48507	HFRO	2020-09-28	8.72	8.9075	8.72	8.79	293579	{}	\N	2020-10-12 15:08:59.558694
48508	HFRO	2020-09-29	8.87	8.89	8.68	8.69	635499	{}	\N	2020-10-12 15:08:59.568109
48509	HFRO	2020-09-30	8.8	8.8	8.47	8.59	3947944	{}	\N	2020-10-12 15:08:59.574886
48510	HFRO	2020-10-01	8.59	8.63	8.55	8.61	355336	{}	\N	2020-10-12 15:08:59.580671
48511	HFRO	2020-10-02	8.61	8.79	8.5605	8.79	288963	{}	\N	2020-10-12 15:08:59.585692
48512	HFRO	2020-10-05	8.8	8.85	8.7	8.76	540286	{}	\N	2020-10-12 15:08:59.591148
48513	HFRO	2020-10-06	8.77	8.78	8.68	8.7	488625	{}	\N	2020-10-12 15:08:59.595086
48514	HFRO	2020-10-07	8.73	8.78	8.66	8.7	363302	{}	\N	2020-10-12 15:08:59.599167
48515	HFRO	2020-10-08	8.7	8.76	8.6501	8.69	407949	{}	\N	2020-10-12 15:08:59.602986
48516	HFRO	2020-10-09	8.71	8.71	8.645	8.67	234457	{}	\N	2020-10-12 15:08:59.607157
48517	HFRO	2020-10-12	8.7	8.7	8.6	8.61	124252	{}	\N	2020-10-12 15:08:59.611149
48518	HGSH	2020-09-28	0.94	0.99	0.8621	0.989899	12330	{}	\N	2020-10-12 15:09:02.076599
48519	HGSH	2020-09-29	0.99	1.05	0.95	1.05	22077	{}	\N	2020-10-12 15:09:02.087726
48520	HGSH	2020-09-30	1.1	1.1	0.9821	1.08	44444	{}	\N	2020-10-12 15:09:02.094229
48521	HGSH	2020-10-01	1.06	1.28	1.03	1.2194	127617	{}	\N	2020-10-12 15:09:02.100027
48522	HGSH	2020-10-02	1.26	1.28	1.155	1.16	29023	{}	\N	2020-10-12 15:09:02.105132
48523	HGSH	2020-10-05	1.24	1.24	1.0594	1.105	15546	{}	\N	2020-10-12 15:09:02.11077
48524	HGSH	2020-10-06	1.14	1.15	1.0301	1.1	17691	{}	\N	2020-10-12 15:09:02.115092
48525	HGSH	2020-10-07	1.06	1.15	1.041	1.08	7868	{}	\N	2020-10-12 15:09:02.118865
48526	HGSH	2020-10-08	1.04	1.08	1.01	1.06	12716	{}	\N	2020-10-12 15:09:02.122591
48527	HGSH	2020-10-09	1.0327	1.1099	0.99	1.03	44518	{}	\N	2020-10-12 15:09:02.126301
48528	HGSH	2020-10-12	0.9991	1.0382	0.9991	1.0382	2970	{}	\N	2020-10-12 15:09:02.12992
48529	HIL	2020-09-28	1.33	1.37	1.29	1.35	133638	{}	\N	2020-10-12 15:09:03.317059
48530	HIL	2020-09-29	1.35	1.4	1.34	1.36	477092	{}	\N	2020-10-12 15:09:03.32831
48531	HIL	2020-09-30	1.346	1.45	1.31	1.32	238644	{}	\N	2020-10-12 15:09:03.335389
48532	HIL	2020-10-01	1.35	1.409	1.325	1.385	191652	{}	\N	2020-10-12 15:09:03.342184
48533	HIL	2020-10-02	1.38	1.4	1.3477	1.38	186846	{}	\N	2020-10-12 15:09:03.347193
48534	HIL	2020-10-05	1.4	1.45	1.38	1.43	371668	{}	\N	2020-10-12 15:09:03.352017
48535	HIL	2020-10-06	1.41	1.76	1.41	1.57	7643148	{}	\N	2020-10-12 15:09:03.358064
48536	HIL	2020-10-07	1.73	1.85	1.48	1.5	10321379	{}	\N	2020-10-12 15:09:03.362756
48537	HIL	2020-10-08	1.45	1.5	1.41	1.49	1468101	{}	\N	2020-10-12 15:09:03.367566
48538	HIL	2020-10-09	1.47	1.51	1.43	1.48	1136076	{}	\N	2020-10-12 15:09:03.371685
48539	HIL	2020-10-12	1.46	1.48	1.44	1.45	308038	{}	\N	2020-10-12 15:09:03.375845
48540	HJLI	2020-09-28	0.4075	0.4278	0.4075	0.4255	1135131	{}	\N	2020-10-12 15:09:04.61628
48541	HJLI	2020-09-29	0.421	0.47	0.4161	0.444	3227296	{}	\N	2020-10-12 15:09:04.626142
48542	HJLI	2020-09-30	0.46	0.4695	0.43	0.433	1830948	{}	\N	2020-10-12 15:09:04.632706
48543	HJLI	2020-10-01	0.4274	0.45	0.422	0.4389	1144284	{}	\N	2020-10-12 15:09:04.638612
48544	HJLI	2020-10-02	0.4485	0.465	0.4231	0.431	1988405	{}	\N	2020-10-12 15:09:04.643886
48545	HJLI	2020-10-05	0.426	0.44	0.415	0.42	1155123	{}	\N	2020-10-12 15:09:04.648695
48546	HJLI	2020-10-06	0.4201	0.4203	0.4	0.409	1878665	{}	\N	2020-10-12 15:09:04.652655
48547	HJLI	2020-10-07	0.37	0.3988	0.345	0.3988	7045474	{}	\N	2020-10-12 15:09:04.656855
48548	HJLI	2020-10-08	0.38	0.4	0.379	0.4	4126603	{}	\N	2020-10-12 15:09:04.661147
48549	HJLI	2020-10-09	0.3959	0.3989	0.38	0.389	1843631	{}	\N	2020-10-12 15:09:04.665302
48550	HJLI	2020-10-12	0.3802	0.389	0.3795	0.3802	384197	{}	\N	2020-10-12 15:09:04.669405
48551	HLNE	2020-09-28	62.63	63.84	62.05	63.14	125840	{}	\N	2020-10-12 15:09:05.879283
48552	HLNE	2020-09-29	63.1	63.475	62.17	63.25	210670	{}	\N	2020-10-12 15:09:05.89141
48553	HLNE	2020-09-30	63.45	65.855	63.41	64.59	254829	{}	\N	2020-10-12 15:09:05.898158
48554	HLNE	2020-10-01	64.6	65.58	64.47	65.45	125131	{}	\N	2020-10-12 15:09:05.903668
48555	HLNE	2020-10-02	64.36	66.37	64.36	66.08	187723	{}	\N	2020-10-12 15:09:05.908536
48556	HLNE	2020-10-05	66.7	68.7	66.15	68.14	166801	{}	\N	2020-10-12 15:09:05.913305
48557	HLNE	2020-10-06	68.65	69.17	66.99	67.15	121168	{}	\N	2020-10-12 15:09:05.917799
48558	HLNE	2020-10-07	68.1	69.085	66.96	68.07	105991	{}	\N	2020-10-12 15:09:05.921735
48559	HLNE	2020-10-08	68	68.28	67.54	67.85	183280	{}	\N	2020-10-12 15:09:05.92534
48560	HLNE	2020-10-09	68.535	69.93	67.79	69.83	129589	{}	\N	2020-10-12 15:09:05.929019
48561	HLNE	2020-10-12	70.39	70.39	67.85	69.515	20726	{}	\N	2020-10-12 15:09:05.93262
48562	HMST	2020-09-28	25.39	26.16	25.32	25.93	157039	{}	\N	2020-10-12 15:09:08.370536
48563	HMST	2020-09-29	25.76	26.01	25.36	25.76	121395	{}	\N	2020-10-12 15:09:08.378525
48564	HMST	2020-09-30	25.64	26.2	25.64	25.76	164410	{}	\N	2020-10-12 15:09:08.383984
48565	HMST	2020-10-01	25.8	26.21	25.5	26.21	138779	{}	\N	2020-10-12 15:09:08.388707
48566	HMST	2020-10-02	28.1	29.63	27.09	29.46	297489	{}	\N	2020-10-12 15:09:08.393447
48567	HMST	2020-10-05	29.46	30.36	29.46	29.79	200623	{}	\N	2020-10-12 15:09:08.397639
48568	HMST	2020-10-06	30.21	31.0264	29.81	30.14	218416	{}	\N	2020-10-12 15:09:08.401629
48569	HMST	2020-10-07	30.3	30.68	29.42	29.82	156916	{}	\N	2020-10-12 15:09:08.405544
48570	HMST	2020-10-08	30.12	31.2	29.48	30.05	72932	{}	\N	2020-10-12 15:09:08.409478
48571	HMST	2020-10-09	30.56	31.22	29.61	29.74	48935	{}	\N	2020-10-12 15:09:08.413368
48572	HMST	2020-10-12	29.66	29.89	29.54	29.79	12708	{}	\N	2020-10-12 15:09:08.41725
48573	HNRG	2020-09-28	0.66175	0.6998	0.6501	0.6501	63599	{}	\N	2020-10-12 15:09:09.900267
48574	HNRG	2020-09-29	0.6726	0.7792	0.65	0.67	274748	{}	\N	2020-10-12 15:09:09.912184
48575	HNRG	2020-09-30	0.6712	0.6938	0.62	0.6519	184355	{}	\N	2020-10-12 15:09:09.9192
48576	HNRG	2020-10-01	0.6547	0.69	0.63	0.66	63232	{}	\N	2020-10-12 15:09:09.925109
48577	HNRG	2020-10-02	0.64	0.685	0.6363	0.6659	72242	{}	\N	2020-10-12 15:09:09.929627
48578	HNRG	2020-10-05	0.685	0.73	0.665	0.7136	129021	{}	\N	2020-10-12 15:09:09.934037
48579	HNRG	2020-10-06	0.7053	0.737	0.6735	0.72	233206	{}	\N	2020-10-12 15:09:09.938397
48580	HNRG	2020-10-07	0.74	0.79	0.7187	0.7401	474970	{}	\N	2020-10-12 15:09:09.942519
48581	HNRG	2020-10-08	0.74	0.77	0.715	0.7445	165546	{}	\N	2020-10-12 15:09:09.946218
48582	HNRG	2020-10-09	0.765	0.7888	0.7486	0.765	136514	{}	\N	2020-10-12 15:09:09.949849
48583	HNRG	2020-10-12	0.77	0.78	0.752101	0.7795	56970	{}	\N	2020-10-12 15:09:09.953551
48584	HPE	2020-09-28	9.26	9.5	9.24	9.48	13207268	{}	\N	2020-10-12 15:09:11.066249
48585	HPE	2020-09-29	9.49	9.56	9.32	9.38	15210041	{}	\N	2020-10-12 15:09:11.076373
48586	HPE	2020-09-30	9.4	9.54	9.3	9.37	16520416	{}	\N	2020-10-12 15:09:11.083279
48587	HPE	2020-10-01	9.43	9.445	9.15	9.23	17223888	{}	\N	2020-10-12 15:09:11.089403
48588	HPE	2020-10-02	9.05	9.455	9.05	9.42	17623711	{}	\N	2020-10-12 15:09:11.094523
48589	HPE	2020-10-05	9.51	9.79	9.5	9.77	25466407	{}	\N	2020-10-12 15:09:11.099584
48590	HPE	2020-10-06	9.81	9.905	9.52	9.56	19609576	{}	\N	2020-10-12 15:09:11.103946
48591	HPE	2020-10-07	9.66	9.79	9.655	9.77	12440822	{}	\N	2020-10-12 15:09:11.109453
48592	HPE	2020-10-08	9.8	10.07	9.8	9.99	28203064	{}	\N	2020-10-12 15:09:11.113319
48593	HPE	2020-10-09	10.05	10.08	9.74	9.81	15430582	{}	\N	2020-10-12 15:09:11.117255
48594	HPE	2020-10-12	9.83	9.845	9.715	9.82	2996199	{}	\N	2020-10-12 15:09:11.121261
48595	HQH	2020-09-28	20.61	20.61	20.2801	20.45	135942	{}	\N	2020-10-12 15:09:12.083358
48596	HQH	2020-09-29	20.49	20.57	20.38	20.47	230549	{}	\N	2020-10-12 15:09:12.09279
48597	HQH	2020-09-30	20.56	20.7	20.49	20.62	282724	{}	\N	2020-10-12 15:09:12.099222
48598	HQH	2020-10-01	20.7	20.7	20.56	20.69	270967	{}	\N	2020-10-12 15:09:12.10574
48599	HQH	2020-10-02	20.34	20.64	20.28	20.47	248077	{}	\N	2020-10-12 15:09:12.11064
48600	HQH	2020-10-05	20.6	21.055	20.6	21.02	406430	{}	\N	2020-10-12 15:09:12.115402
48601	HQH	2020-10-06	21.06	21.15	20.8	20.87	292807	{}	\N	2020-10-12 15:09:12.11941
48602	HQH	2020-10-07	21	21.3	20.966	21.28	399579	{}	\N	2020-10-12 15:09:12.123516
48603	HQH	2020-10-08	21.4	21.4	21.25	21.34	303561	{}	\N	2020-10-12 15:09:12.127489
48604	HQH	2020-10-09	21.39	21.5	21.35	21.47	380884	{}	\N	2020-10-12 15:09:12.131744
48605	HQH	2020-10-12	21.57	21.6	21.4	21.5499	97952	{}	\N	2020-10-12 15:09:12.135651
48606	HRTX	2020-09-28	14.77	15.46	14.57	15.22	1668963	{}	\N	2020-10-12 15:09:13.66075
48607	HRTX	2020-09-29	15.17	15.32	14.72	14.84	547210	{}	\N	2020-10-12 15:09:13.671176
48608	HRTX	2020-09-30	14.83	15.18	14.64	14.83	727580	{}	\N	2020-10-12 15:09:13.677521
48609	HRTX	2020-10-01	14.92	15.12	14.79	15	864384	{}	\N	2020-10-12 15:09:13.682692
48610	HRTX	2020-10-02	14.71	14.91	14.4283	14.61	603439	{}	\N	2020-10-12 15:09:13.687112
48611	HRTX	2020-10-05	14.81	15.27	14.47	15.01	755455	{}	\N	2020-10-12 15:09:13.691349
48612	HRTX	2020-10-06	15.095	15.18	14.59	14.61	672944	{}	\N	2020-10-12 15:09:13.695548
48613	HRTX	2020-10-07	14.8	15.7	14.61	15.52	923571	{}	\N	2020-10-12 15:09:13.699344
48614	HRTX	2020-10-08	15.83	16.06	15.595	15.8	457799	{}	\N	2020-10-12 15:09:13.702809
48615	HRTX	2020-10-09	15.82	15.93	15.51	15.79	474844	{}	\N	2020-10-12 15:09:13.706348
48616	HRTX	2020-10-12	15.91	15.96	15.44	15.65	94673	{}	\N	2020-10-12 15:09:13.710024
48617	HLIT	2020-09-28	5.29	5.42	5.27	5.34	405839	{}	\N	2020-10-12 15:09:14.834699
48618	HLIT	2020-09-29	5.42	5.645	5.39	5.62	470919	{}	\N	2020-10-12 15:09:14.844992
48619	HLIT	2020-09-30	5.61	5.66	5.51	5.58	589054	{}	\N	2020-10-12 15:09:14.85185
48620	HLIT	2020-10-01	5.63	5.65	5.4	5.475	320855	{}	\N	2020-10-12 15:09:14.857661
48621	HLIT	2020-10-02	5.39	5.54	5.33	5.52	640382	{}	\N	2020-10-12 15:09:14.861816
48622	HLIT	2020-10-05	5.56	5.68	5.56	5.67	289567	{}	\N	2020-10-12 15:09:14.866222
48623	HLIT	2020-10-06	5.7	5.82	5.615	5.66	455160	{}	\N	2020-10-12 15:09:14.870119
48624	HLIT	2020-10-07	5.71	5.74	5.61	5.73	469442	{}	\N	2020-10-12 15:09:14.874538
48625	HLIT	2020-10-08	5.82	6.22	5.81	6.21	931649	{}	\N	2020-10-12 15:09:14.878417
48626	HLIT	2020-10-09	6.23	6.46	6.22	6.45	793546	{}	\N	2020-10-12 15:09:14.882374
48627	HLIT	2020-10-12	6.48	6.51	6.29	6.3157	225312	{}	\N	2020-10-12 15:09:14.886585
48628	HMLP	2020-09-28	10.78	10.89	10.63	10.63	79636	{}	\N	2020-10-12 15:09:15.831866
48629	HMLP	2020-09-29	10.65	10.885	10.605	10.61	152612	{}	\N	2020-10-12 15:09:15.842186
48630	HMLP	2020-09-30	10.66	10.88	10.61	10.61	155552	{}	\N	2020-10-12 15:09:15.849087
48631	HMLP	2020-10-01	10.66	10.98	10.62	10.96	175133	{}	\N	2020-10-12 15:09:15.855219
48632	HMLP	2020-10-02	10.72	11.16	10.67	11.16	156931	{}	\N	2020-10-12 15:09:15.861067
48633	HMLP	2020-10-05	11.09	11.4692	11.09	11.42	149486	{}	\N	2020-10-12 15:09:15.866429
48634	HMLP	2020-10-06	11.48	11.645	11.15	11.26	153233	{}	\N	2020-10-12 15:09:15.871264
48635	HMLP	2020-10-07	11.38	11.46	11.02	11.35	193758	{}	\N	2020-10-12 15:09:15.875999
48636	HMLP	2020-10-08	11.37	11.67	11.3	11.61	227516	{}	\N	2020-10-12 15:09:15.880494
48637	HMLP	2020-10-09	11.75	11.79	11.3	11.51	104712	{}	\N	2020-10-12 15:09:15.884977
48638	HMLP	2020-10-12	11.63	11.65	11.25	11.27	59689	{}	\N	2020-10-12 15:09:15.889648
48639	HMY	2020-09-28	5.28	5.29	5.08	5.18	5916142	{}	\N	2020-10-12 15:09:16.910963
48640	HMY	2020-09-29	5.33	5.46	5.28	5.42	9735891	{}	\N	2020-10-12 15:09:16.923938
48641	HMY	2020-09-30	5.36	5.38	5.22	5.27	13503961	{}	\N	2020-10-12 15:09:16.931499
48642	HMY	2020-10-01	5.34	5.405	5.255	5.29	8707367	{}	\N	2020-10-12 15:09:16.937846
48643	HMY	2020-10-02	5.3	5.4	5.2223	5.26	6840146	{}	\N	2020-10-12 15:09:16.943019
48644	HMY	2020-10-05	5.39	5.57	5.31	5.41	12446205	{}	\N	2020-10-12 15:09:16.947962
48645	HMY	2020-10-06	5.5	5.57	5.135	5.15	15296595	{}	\N	2020-10-12 15:09:16.952577
48646	HMY	2020-10-07	5.19	5.295	5.085	5.23	9272034	{}	\N	2020-10-12 15:09:16.956908
48647	HMY	2020-10-08	5.33	5.45	5.31	5.44	6891183	{}	\N	2020-10-12 15:09:16.961185
48648	HMY	2020-10-09	5.67	5.96	5.6454	5.96	16190976	{}	\N	2020-10-12 15:09:16.965304
48649	HMY	2020-10-12	5.87	5.89	5.75	5.82	4901792	{}	\N	2020-10-12 15:09:16.969409
48650	HNW	2020-09-28	13.72	13.72	13.41	13.44	5550	{}	\N	2020-10-12 15:09:18.431544
48651	HNW	2020-09-29	13.4	13.49	13.38	13.4	12530	{}	\N	2020-10-12 15:09:18.44184
48652	HNW	2020-09-30	13.9	14.29	13.43	13.51	32874	{}	\N	2020-10-12 15:09:18.448466
48653	HNW	2020-10-01	13.5	13.88	13.5	13.6	39451	{}	\N	2020-10-12 15:09:18.454479
48654	HNW	2020-10-02	13.9	13.977	13.53	13.6	24194	{}	\N	2020-10-12 15:09:18.459274
48655	HNW	2020-10-05	13.68	13.68	13.46	13.55	16069	{}	\N	2020-10-12 15:09:18.464051
48656	HNW	2020-10-06	13.455	13.58	13.31	13.42	46448	{}	\N	2020-10-12 15:09:18.468382
48657	HNW	2020-10-07	13.56	13.6682	13.47	13.5	16859	{}	\N	2020-10-12 15:09:18.472813
48658	HNW	2020-10-08	13.75	13.75	13.49	13.6	25972	{}	\N	2020-10-12 15:09:18.476704
48659	HNW	2020-10-09	13.6	13.72	13.6	13.6905	52533	{}	\N	2020-10-12 15:09:18.48078
48660	HNW	2020-10-12	13.69	13.69	13.56	13.59	11778	{}	\N	2020-10-12 15:09:18.485451
48661	HPF	2020-09-28	17.69	17.96	17.69	17.85	30951	{}	\N	2020-10-12 15:09:19.890302
48662	HPF	2020-09-29	17.92	17.92	17.75	17.91	69632	{}	\N	2020-10-12 15:09:19.9019
48663	HPF	2020-09-30	17.93	17.959	17.79	17.82	119122	{}	\N	2020-10-12 15:09:19.909734
48664	HPF	2020-10-01	17.96	17.96	17.7892	17.94	84539	{}	\N	2020-10-12 15:09:19.915621
48665	HPF	2020-10-02	17.62	17.9201	17.62	17.87	71892	{}	\N	2020-10-12 15:09:19.920781
48666	HPF	2020-10-05	18.01	18.06	17.9718	18.03	87169	{}	\N	2020-10-12 15:09:19.926086
48667	HPF	2020-10-06	17.98	18.15	17.98	18.04	117226	{}	\N	2020-10-12 15:09:19.930157
48668	HPF	2020-10-07	18.17	18.229	18.12	18.18	55146	{}	\N	2020-10-12 15:09:19.93463
48669	HPF	2020-10-08	18.24	18.48	18.181	18.45	143682	{}	\N	2020-10-12 15:09:19.939058
48670	HPF	2020-10-09	18.38	18.38	18.11	18.23	88288	{}	\N	2020-10-12 15:09:19.943336
48671	HPF	2020-10-12	18.17	18.2	18.115	18.1667	50742	{}	\N	2020-10-12 15:09:19.947696
48672	HRTG	2020-09-28	10.32	10.69	10.3	10.43	93910	{}	\N	2020-10-12 15:09:21.506024
48673	HRTG	2020-09-29	10.37	10.41	10.02	10.21	154353	{}	\N	2020-10-12 15:09:21.517233
48674	HRTG	2020-09-30	10.19	10.28	9.93	10.12	203245	{}	\N	2020-10-12 15:09:21.523621
48675	HRTG	2020-10-01	10.12	10.19	9.93	10.08	200175	{}	\N	2020-10-12 15:09:21.529448
48676	HRTG	2020-10-02	10.03	10.35	9.99	10.23	129426	{}	\N	2020-10-12 15:09:21.534743
48677	HRTG	2020-10-05	10.26	10.73	10.26	10.66	176395	{}	\N	2020-10-12 15:09:21.539699
48678	HRTG	2020-10-06	10.75	10.92	10.52	10.55	198841	{}	\N	2020-10-12 15:09:21.543871
48679	HRTG	2020-10-07	10.64	10.69	10.2	10.28	165351	{}	\N	2020-10-12 15:09:21.548104
48680	HRTG	2020-10-08	10.39	10.405	10.12	10.26	149500	{}	\N	2020-10-12 15:09:21.552539
48681	HRTG	2020-10-09	10.38	10.56	10.16	10.5	302282	{}	\N	2020-10-12 15:09:21.557191
48682	HRTG	2020-10-12	10.44	10.65	10.375	10.62	27878	{}	\N	2020-10-12 15:09:21.562259
48683	HSON	2020-09-28	9.745	9.745	9.745	9.745	118	{}	\N	2020-10-12 15:09:23.224174
48684	HSON	2020-09-29	9.6212	9.64	9.54	9.54	2509	{}	\N	2020-10-12 15:09:23.234304
48685	HSON	2020-09-30	9.675	9.675	9.675	9.675	694	{}	\N	2020-10-12 15:09:23.241026
48686	HSON	2020-10-01	9.69	9.69	9.38	9.38	2659	{}	\N	2020-10-12 15:09:23.246935
48687	HSON	2020-10-02	9.37	9.825	9.37	9.71	11747	{}	\N	2020-10-12 15:09:23.25167
48688	HSON	2020-10-05	9.87	10.125	9.795	10.06	1930	{}	\N	2020-10-12 15:09:23.256697
48689	HSON	2020-10-06	9.95	10.1499	9.87	9.97	7440	{}	\N	2020-10-12 15:09:23.261216
48690	HSON	2020-10-07	10.13	10.13	10.045	10.05	1429	{}	\N	2020-10-12 15:09:23.26517
48691	HSON	2020-10-08	10	10	9.9	9.9	2343	{}	\N	2020-10-12 15:09:23.269072
48692	HSON	2020-10-09	10.005	10.005	9.85	9.91	4441	{}	\N	2020-10-12 15:09:23.273184
48693	HSON	2020-10-12	9.9123	9.94	9.9	9.92	2613	{}	\N	2020-10-12 15:09:23.276967
48694	HTBK	2020-09-28	6.43	6.71	6.43	6.66	279465	{}	\N	2020-10-12 15:09:24.171791
48695	HTBK	2020-09-29	6.63	6.64	6.46	6.62	194869	{}	\N	2020-10-12 15:09:24.183178
48696	HTBK	2020-09-30	6.6	6.76	6.59	6.655	149584	{}	\N	2020-10-12 15:09:24.189098
48697	HTBK	2020-10-01	6.65	6.69	6.56	6.67	149853	{}	\N	2020-10-12 15:09:24.194796
48698	HTBK	2020-10-02	6.54	6.89	6.54	6.87	116634	{}	\N	2020-10-12 15:09:24.199799
48699	HTBK	2020-10-05	6.94	7.08	6.88	7.04	126164	{}	\N	2020-10-12 15:09:24.204473
48700	HTBK	2020-10-06	7.14	7.37	7.01	7.05	215093	{}	\N	2020-10-12 15:09:24.208277
48701	HTBK	2020-10-07	7.14	7.34	7.11	7.295	158819	{}	\N	2020-10-12 15:09:24.212086
48702	HTBK	2020-10-08	7.38	7.4	7.22	7.31	138693	{}	\N	2020-10-12 15:09:24.215916
48703	HTBK	2020-10-09	7.35	7.42	7.21	7.23	65005	{}	\N	2020-10-12 15:09:24.219914
48704	HTBK	2020-10-12	7.19	7.33	7.16	7.26	15859	{}	\N	2020-10-12 15:09:24.224382
48705	HTLF	2020-09-28	29.67	30.81	29.67	30.53	59207	{}	\N	2020-10-12 15:09:26.345729
48706	HTLF	2020-09-29	30.45	30.45	29.51	30.205	46126	{}	\N	2020-10-12 15:09:26.358065
48707	HTLF	2020-09-30	30.45	30.75	29.8	29.99	38689	{}	\N	2020-10-12 15:09:26.370577
48708	HTLF	2020-10-01	30.23	30.89	29.83	30.79	157941	{}	\N	2020-10-12 15:09:26.375363
48709	HTLF	2020-10-02	30.29	31.8	30.21	31.69	67625	{}	\N	2020-10-12 15:09:26.379715
48710	HTLF	2020-10-05	32.01	33.2	32.01	33.06	76713	{}	\N	2020-10-12 15:09:26.383982
48711	HTLF	2020-10-06	33.55	34.43	33.04	33.08	78014	{}	\N	2020-10-12 15:09:26.388077
48712	HTLF	2020-10-07	33.63	34.09	33.08	33.57	118193	{}	\N	2020-10-12 15:09:26.391865
48713	HTLF	2020-10-08	34.1	34.39	33.59	34.18	64147	{}	\N	2020-10-12 15:09:26.395794
48714	HTLF	2020-10-09	34.42	34.42	33.555	33.68	50688	{}	\N	2020-10-12 15:09:26.399393
48715	HTLF	2020-10-12	33.52	34.17	33.52	34.145	6827	{}	\N	2020-10-12 15:09:26.403291
48716	HUN	2020-09-28	21.99	22.35	21.8	22.22	1500891	{}	\N	2020-10-12 15:09:27.375902
48717	HUN	2020-09-29	22.11	22.29	21.83	21.9	2191094	{}	\N	2020-10-12 15:09:27.382722
48718	HUN	2020-09-30	22.9	23.14	22.0799	22.21	5185876	{}	\N	2020-10-12 15:09:27.386903
48719	HUN	2020-10-01	22.2	22.5	21.91	22.05	2583014	{}	\N	2020-10-12 15:09:27.390643
48720	HUN	2020-10-02	21.68	22.92	21.65	22.73	2864233	{}	\N	2020-10-12 15:09:27.394578
48721	HUN	2020-10-05	22.93	23.48	22.85	23.45	2924897	{}	\N	2020-10-12 15:09:27.398211
48722	HUN	2020-10-06	23.65	24.07	23.36	23.36	3569309	{}	\N	2020-10-12 15:09:27.401987
48723	HUN	2020-10-07	24.06	24.34	23.79	24.1	2643695	{}	\N	2020-10-12 15:09:27.405422
48724	HUN	2020-10-08	24.2	24.455	23.96	24.38	1547892	{}	\N	2020-10-12 15:09:27.408697
48725	HUN	2020-10-09	24.5	24.82	24.335	24.48	2143212	{}	\N	2020-10-12 15:09:27.412251
48726	HUN	2020-10-12	24.67	24.7	24.47	24.545	324781	{}	\N	2020-10-12 15:09:27.415601
48727	HY	2020-09-28	37.37	38.2	37.37	37.63	42316	{}	\N	2020-10-12 15:09:28.682262
48728	HY	2020-09-29	37.36	37.97	37.15	37.51	59592	{}	\N	2020-10-12 15:09:28.691972
48729	HY	2020-09-30	37.54	38.19	36.93	37.15	78739	{}	\N	2020-10-12 15:09:28.699261
48730	HY	2020-10-01	37.33	37.7	36.83	37.47	85475	{}	\N	2020-10-12 15:09:28.705058
48731	HY	2020-10-02	36.82	39.07	36.37	38.9	99702	{}	\N	2020-10-12 15:09:28.709775
48732	HY	2020-10-05	39.42	40.71	38.9	40.58	101400	{}	\N	2020-10-12 15:09:28.71449
48733	HY	2020-10-06	40.53	42.16	39.96	40.6	108316	{}	\N	2020-10-12 15:09:28.718462
48734	HY	2020-10-07	41.26	42.06	40.88	41.85	58141	{}	\N	2020-10-12 15:09:28.722357
48735	HY	2020-10-08	42	42.71	41.89	42.49	78909	{}	\N	2020-10-12 15:09:28.726339
48736	HY	2020-10-09	43	43	41.85	42.46	55322	{}	\N	2020-10-12 15:09:28.730177
48737	HY	2020-10-12	42.63	43	42.63	42.65	13128	{}	\N	2020-10-12 15:09:28.733855
48738	HMC	2020-09-28	23.5	23.7575	23.5	23.66	765019	{}	\N	2020-10-12 15:09:30.073571
48739	HMC	2020-09-29	23.41	23.48	23.31	23.38	1111573	{}	\N	2020-10-12 15:09:30.086225
48740	HMC	2020-09-30	23.59	23.835	23.57	23.7	1152906	{}	\N	2020-10-12 15:09:30.093018
48741	HMC	2020-10-01	23.64	23.74	23.57	23.7	917908	{}	\N	2020-10-12 15:09:30.098805
48742	HMC	2020-10-02	23.73	24.15	23.65	24.12	1277409	{}	\N	2020-10-12 15:09:30.10409
48743	HMC	2020-10-05	24.41	24.46	24.32	24.42	864386	{}	\N	2020-10-12 15:09:30.108831
48744	HMC	2020-10-06	24.75	24.86	24.43	24.48	1127349	{}	\N	2020-10-12 15:09:30.112985
48745	HMC	2020-10-07	24.79	24.88	24.71	24.81	711734	{}	\N	2020-10-12 15:09:30.117131
48746	HMC	2020-10-08	24.68	24.7389	24.61	24.68	831298	{}	\N	2020-10-12 15:09:30.121232
48747	HMC	2020-10-09	24.52	24.575	24.33	24.38	1365170	{}	\N	2020-10-12 15:09:30.125278
48748	HMC	2020-10-12	24.19	24.195	24.06	24.195	388128	{}	\N	2020-10-12 15:09:30.129508
48749	HNNA	2020-09-28	8.83	8.83	8.504	8.504	2930	{}	\N	2020-10-12 15:09:31.108902
48750	HNNA	2020-09-29	8.48	8.5	8.2	8.5	12520	{}	\N	2020-10-12 15:09:31.120134
48751	HNNA	2020-09-30	8.1901	8.56	8.1901	8.29	1310	{}	\N	2020-10-12 15:09:31.126703
48752	HNNA	2020-10-01	8.355	8.56	8.345	8.4	1852	{}	\N	2020-10-12 15:09:31.131818
48753	HNNA	2020-10-02	8.1	8.24	8.1	8.2299	1762	{}	\N	2020-10-12 15:09:31.136953
48754	HNNA	2020-10-05	8.3698	8.53	8.2	8.42	3882	{}	\N	2020-10-12 15:09:31.141312
48755	HNNA	2020-10-06	8.42	8.42	8.2419	8.33	4603	{}	\N	2020-10-12 15:09:31.145509
48756	HNNA	2020-10-07	8.355	8.4	8.31	8.3999	2691	{}	\N	2020-10-12 15:09:31.149994
48757	HNNA	2020-10-08	8.435	8.54	8.38	8.48	3491	{}	\N	2020-10-12 15:09:31.154042
48758	HNNA	2020-10-09	8.45	8.56	8.45	8.56	1167	{}	\N	2020-10-12 15:09:31.158403
48759	HNNA	2020-10-12	8.6	8.6	8.52	8.52	704	{}	\N	2020-10-12 15:09:31.162874
48760	HON	2020-09-28	164.44	166.15	163.23	164.64	2464066	{}	\N	2020-10-12 15:09:32.14966
48761	HON	2020-09-29	164.56	166.23	163.47	164.51	4117916	{}	\N	2020-10-12 15:09:32.159013
48762	HON	2020-09-30	165.77	167.92	163.02	164.61	5714119	{}	\N	2020-10-12 15:09:32.165571
48763	HON	2020-10-01	166.2	167.14	162.1167	163.68	4697207	{}	\N	2020-10-12 15:09:32.170976
48764	HON	2020-10-02	161.62	167	161.35	165.61	4464434	{}	\N	2020-10-12 15:09:32.17568
48765	HON	2020-10-05	166.99	169.25	166.74	168.72	3525137	{}	\N	2020-10-12 15:09:32.180411
48766	HON	2020-10-06	169.11	171.42	166.62	166.89	4439573	{}	\N	2020-10-12 15:09:32.184486
48767	HON	2020-10-07	168.99	172.195	168.52	171.55	3934801	{}	\N	2020-10-12 15:09:32.188322
48768	HON	2020-10-08	172.38	173.87	171.48	173.78	3954656	{}	\N	2020-10-12 15:09:32.192244
48769	HON	2020-10-09	174.4	176.06	173.265	174.38	5153919	{}	\N	2020-10-12 15:09:32.196117
48770	HON	2020-10-12	175.06	176.98	174.12	175.35	1828321	{}	\N	2020-10-12 15:09:32.200092
48771	HPQ	2020-09-28	18.6	18.945	18.56	18.86	11229944	{}	\N	2020-10-12 15:09:33.187785
48772	HPQ	2020-09-29	18.85	19.15	18.835	18.92	15464580	{}	\N	2020-10-12 15:09:33.198061
48773	HPQ	2020-09-30	18.88	19.32	18.84	18.99	23072252	{}	\N	2020-10-12 15:09:33.20462
48774	HPQ	2020-10-01	19.1	19.28	18.93	19	20430128	{}	\N	2020-10-12 15:09:33.210594
48775	HPQ	2020-10-02	18.53	19.305	18.5	19.09	15565072	{}	\N	2020-10-12 15:09:33.215444
48776	HPQ	2020-10-05	19.32	19.59	19.1	19.31	18622012	{}	\N	2020-10-12 15:09:33.220607
48777	HPQ	2020-10-06	19.5	19.6	18.945	18.96	22766250	{}	\N	2020-10-12 15:09:33.224812
48778	HPQ	2020-10-07	19.16	19.31	19.04	19.23	23272670	{}	\N	2020-10-12 15:09:33.228716
48779	HPQ	2020-10-08	19.36	19.51	19.275	19.4	16497344	{}	\N	2020-10-12 15:09:33.23281
48780	HPQ	2020-10-09	19.61	19.68	19.24	19.37	15815052	{}	\N	2020-10-12 15:09:33.23694
48781	HPQ	2020-10-12	19.57	19.7	19.335	19.61	3977390	{}	\N	2020-10-12 15:09:33.241107
48782	HRL	2020-09-28	49	49.31	48.73	48.96	1136918	{}	\N	2020-10-12 15:09:34.277825
48783	HRL	2020-09-29	48.93	49.29	48.39	48.51	2627796	{}	\N	2020-10-12 15:09:34.287338
48784	HRL	2020-09-30	48.7	49.23	48.5501	48.89	2921458	{}	\N	2020-10-12 15:09:34.293732
48785	HRL	2020-10-01	49.07	49.395	48.45	49.28	2593193	{}	\N	2020-10-12 15:09:34.299624
48786	HRL	2020-10-02	49.18	49.75	48.82	49.02	1874590	{}	\N	2020-10-12 15:09:34.304569
48787	HRL	2020-10-05	48.99	49.51	48.73	49.5	1814905	{}	\N	2020-10-12 15:09:34.309136
48788	HRL	2020-10-06	49.44	49.66	48.69	48.85	2260869	{}	\N	2020-10-12 15:09:34.313233
48789	HRL	2020-10-07	48.87	49.6	48.74	49.33	2616842	{}	\N	2020-10-12 15:09:34.31695
48790	HRL	2020-10-08	49.46	49.81	48.875	49.1	2081101	{}	\N	2020-10-12 15:09:34.320683
48791	HRL	2020-10-09	49.11	49.4099	48.77	48.96	3259361	{}	\N	2020-10-12 15:09:34.324251
48792	HRL	2020-10-12	49.36	50.01	49.05	49.97	974028	{}	\N	2020-10-12 15:09:34.328157
48793	HSII	2020-09-28	19.25	20.15	19.25	19.8	97871	{}	\N	2020-10-12 15:09:35.455221
48794	HSII	2020-09-29	19.75	20.25	19.65	20	128950	{}	\N	2020-10-12 15:09:35.463236
48795	HSII	2020-09-30	20.11	20.35	19.5	19.65	122124	{}	\N	2020-10-12 15:09:35.469229
48796	HSII	2020-10-01	19.51	19.71	19.18	19.54	102585	{}	\N	2020-10-12 15:09:35.475141
48797	HSII	2020-10-02	19.25	19.64	19.12	19.61	68705	{}	\N	2020-10-12 15:09:35.480136
48798	HSII	2020-10-05	19.75	20.4	19.75	20.27	76000	{}	\N	2020-10-12 15:09:35.48444
48799	HSII	2020-10-06	20.49	20.76	20.0275	20.22	271012	{}	\N	2020-10-12 15:09:35.488502
48800	HSII	2020-10-07	20.31	20.76	20.22	20.7	107362	{}	\N	2020-10-12 15:09:35.492378
48801	HSII	2020-10-08	20.88	21.28	20.44	21.26	103379	{}	\N	2020-10-12 15:09:35.496322
48802	HSII	2020-10-09	21.47	21.6	20.98	21.47	69829	{}	\N	2020-10-12 15:09:35.500363
48803	HSII	2020-10-12	21.42	21.95	20.86	21.79	23306	{}	\N	2020-10-12 15:09:35.504463
48804	HTHT	2020-09-28	43	44.495	42.7	44.26	2206591	{}	\N	2020-10-12 15:09:36.44647
48805	HTHT	2020-09-29	44	44.07	42.6	43.52	1215767	{}	\N	2020-10-12 15:09:36.458748
48806	HTHT	2020-09-30	44.2	45.42	42.84	43.24	2104374	{}	\N	2020-10-12 15:09:36.465975
48807	HTHT	2020-10-01	43.24	43.87	42.84	43.73	856510	{}	\N	2020-10-12 15:09:36.472352
48808	HTHT	2020-10-02	43.1	44.67	43.0679	44.01	512672	{}	\N	2020-10-12 15:09:36.477329
48809	HTHT	2020-10-05	45.88	46.3	45.07	45.67	1630797	{}	\N	2020-10-12 15:09:36.482259
48810	HTHT	2020-10-06	45.61	45.99	44.1	44.59	1321126	{}	\N	2020-10-12 15:09:36.486528
48811	HTHT	2020-10-07	45.08	45.19	44.4	44.46	1542236	{}	\N	2020-10-12 15:09:36.490547
48812	HTHT	2020-10-08	44.99	44.99	43.6	44.51	1252959	{}	\N	2020-10-12 15:09:36.494616
48813	HTHT	2020-10-09	44.22	44.99	43.63	44.58	1092536	{}	\N	2020-10-12 15:09:36.498739
48814	HTHT	2020-10-12	44.62	44.785	43.6	43.93	359703	{}	\N	2020-10-12 15:09:36.502811
48815	HUBS	2020-09-28	296.55	303.67	295.46	301.86	384017	{}	\N	2020-10-12 15:09:37.592154
48816	HUBS	2020-09-29	302.19	303.8633	297.65	299.59	545771	{}	\N	2020-10-12 15:09:37.601466
48817	HUBS	2020-09-30	298.18	299.36	289.06	292.23	784458	{}	\N	2020-10-12 15:09:37.608419
48818	HUBS	2020-10-01	296.19	298.09	289.24	293.44	937453	{}	\N	2020-10-12 15:09:37.614766
48819	HUBS	2020-10-02	288.25	291	281.57	285.45	1130387	{}	\N	2020-10-12 15:09:37.620275
48820	HUBS	2020-10-05	287.44	293.33	286.03	289.52	940067	{}	\N	2020-10-12 15:09:37.625021
48821	HUBS	2020-10-06	289.51	299.9199	289.51	293.34	888344	{}	\N	2020-10-12 15:09:37.629104
48822	HUBS	2020-10-07	299	305.98	295.54	304.61	1033286	{}	\N	2020-10-12 15:09:37.633746
48823	HUBS	2020-10-08	311.83	311.83	302.3546	309.73	869156	{}	\N	2020-10-12 15:09:37.638755
48824	HUBS	2020-10-09	313.02	322.71	313.02	321.57	982122	{}	\N	2020-10-12 15:09:37.642676
48825	HUBS	2020-10-12	328.99	330.7158	320.11	324.385	195487	{}	\N	2020-10-12 15:09:37.646431
48826	HVT	2020-09-28	20.55	20.86	20.42	20.43	85689	{}	\N	2020-10-12 15:09:38.903445
48827	HVT	2020-09-29	20.39	20.84	20.26	20.67	275500	{}	\N	2020-10-12 15:09:38.91609
48828	HVT	2020-09-30	20.6	21.31	20.6	20.94	187763	{}	\N	2020-10-12 15:09:38.923611
48829	HVT	2020-10-01	21.17	21.39	20.77	21.3	167417	{}	\N	2020-10-12 15:09:38.929423
48830	HVT	2020-10-02	20.85	21.995	20.79	21.79	192600	{}	\N	2020-10-12 15:09:38.934523
48831	HVT	2020-10-05	21.88	22.4999	21.6	22.42	171971	{}	\N	2020-10-12 15:09:38.939306
48832	HVT	2020-10-06	22.65	22.87	22.07	22.08	172409	{}	\N	2020-10-12 15:09:38.944038
48833	HVT	2020-10-07	22.34	23.85	22.3	23.77	578176	{}	\N	2020-10-12 15:09:38.947779
48834	HVT	2020-10-08	24.45	24.58	23.29	24.08	397402	{}	\N	2020-10-12 15:09:38.951548
48835	HVT	2020-10-09	24.18	24.76	24.01	24.02	296120	{}	\N	2020-10-12 15:09:38.955223
48836	HVT	2020-10-12	24.13	24.28	23.87	24.08	28594	{}	\N	2020-10-12 15:09:38.958941
48837	HYD	2020-09-28	59.62	59.71	59.58	59.69	292528	{}	\N	2020-10-12 15:09:40.702924
48838	HYD	2020-09-29	59.61	59.75	59.61	59.71	293317	{}	\N	2020-10-12 15:09:40.712391
48839	HYD	2020-09-30	59.81	59.81	59.65	59.69	214891	{}	\N	2020-10-12 15:09:40.71916
48840	HYD	2020-10-01	59.64	59.75	59.52	59.75	826167	{}	\N	2020-10-12 15:09:40.724556
48841	HYD	2020-10-02	59.624	59.7	59.6	59.6	128966	{}	\N	2020-10-12 15:09:40.729739
48842	HYD	2020-10-05	59.69	59.69	59.43	59.43	496257	{}	\N	2020-10-12 15:09:40.734901
48843	HYD	2020-10-06	59.46	59.52	59.42	59.5	218681	{}	\N	2020-10-12 15:09:40.739282
48844	HYD	2020-10-07	59.55	59.55	59.4	59.42	209605	{}	\N	2020-10-12 15:09:40.743986
48845	HYD	2020-10-08	59.45	59.53	59.4	59.4	177824	{}	\N	2020-10-12 15:09:40.748206
48846	HYD	2020-10-09	59.45	59.62	59.44	59.52	507930	{}	\N	2020-10-12 15:09:40.752708
48847	HYD	2020-10-12	59.59	59.6	59.48	59.5403	64617	{}	\N	2020-10-12 15:09:40.756865
48848	HYI	2020-09-28	14	14.07	13.961	14.06	151114	{}	\N	2020-10-12 15:09:42.051669
48849	HYI	2020-09-29	14.08	14.108	13.95	14.05	139272	{}	\N	2020-10-12 15:09:42.060701
48850	HYI	2020-09-30	13.99	14.13	13.99	14.03	251212	{}	\N	2020-10-12 15:09:42.067038
48851	HYI	2020-10-01	14.18	14.2	14.1	14.17	94715	{}	\N	2020-10-12 15:09:42.073198
48852	HYI	2020-10-02	14.11	14.3	14.0432	14.25	108442	{}	\N	2020-10-12 15:09:42.078762
48853	HYI	2020-10-05	14.3	14.3	14.21	14.29	52320	{}	\N	2020-10-12 15:09:42.083486
48854	HYI	2020-10-06	14.26	14.36	14.26	14.34	98325	{}	\N	2020-10-12 15:09:42.088357
48855	HYI	2020-10-07	14.36	14.39	14.32	14.34	155574	{}	\N	2020-10-12 15:09:42.093084
48856	HYI	2020-10-08	14.42	14.45	14.37	14.37	96510	{}	\N	2020-10-12 15:09:42.097881
48857	HYI	2020-10-09	14.49	14.5559	14.383	14.46	116308	{}	\N	2020-10-12 15:09:42.102771
48858	HYI	2020-10-12	14.59	14.59	14.53	14.56	52528	{}	\N	2020-10-12 15:09:42.107308
48859	HYUP	2020-09-28	45.5039	45.5039	45.5039	45.5039	93	{}	\N	2020-10-12 15:09:43.315264
48860	HYUP	2020-09-29	45.5049	45.5049	45.5049	45.5049	12	{}	\N	2020-10-12 15:09:43.322195
48861	HYUP	2020-09-30	45.726	45.726	45.726	45.726	181	{}	\N	2020-10-12 15:09:43.327729
48862	HYUP	2020-10-01	45.6352	45.6352	45.6352	45.6352	2	{}	\N	2020-10-12 15:09:43.332533
48863	HYUP	2020-10-02	45.5911	45.595	45.5911	45.595	560	{}	\N	2020-10-12 15:09:43.337752
48864	HYUP	2020-10-05	45.945	45.945	45.945	45.945	30	{}	\N	2020-10-12 15:09:43.341952
48865	HYUP	2020-10-06	45.8022	45.8022	45.8022	45.8022	26	{}	\N	2020-10-12 15:09:43.34601
48866	HYUP	2020-10-07	46.085	46.085	46.085	46.085	27	{}	\N	2020-10-12 15:09:43.350197
48867	HYUP	2020-10-08	46.1849	46.1849	46.1849	46.1849	16	{}	\N	2020-10-12 15:09:43.354157
48868	HYUP	2020-10-09	46.265	46.265	46.265	46.265	81	{}	\N	2020-10-12 15:09:43.358683
48869	IAF	2020-09-28	4.57	4.57	4.46	4.5367	141658	{}	\N	2020-10-12 15:09:44.440203
48870	IAF	2020-09-29	4.5	4.52	4.49	4.52	36215	{}	\N	2020-10-12 15:09:44.453075
48871	IAF	2020-09-30	4.49	4.5	4.45	4.46	58053	{}	\N	2020-10-12 15:09:44.458429
48872	IAF	2020-10-01	4.52	4.52	4.46	4.48	63520	{}	\N	2020-10-12 15:09:44.46306
48873	IAF	2020-10-02	4.41	4.48	4.41	4.45	29574	{}	\N	2020-10-12 15:09:44.467469
48874	IAF	2020-10-05	4.52	4.5386	4.479	4.52	86435	{}	\N	2020-10-12 15:09:44.47193
48875	IAF	2020-10-06	4.45	4.55	4.45	4.48	144160	{}	\N	2020-10-12 15:09:44.475603
48876	IAF	2020-10-07	4.53	4.5585	4.53	4.54	56846	{}	\N	2020-10-12 15:09:44.479332
48877	IAF	2020-10-08	4.61	4.64	4.58	4.59	58494	{}	\N	2020-10-12 15:09:44.483537
48878	IAF	2020-10-09	4.63	4.67	4.61	4.63	75700	{}	\N	2020-10-12 15:09:44.487987
48879	IAF	2020-10-12	4.66	4.71	4.66	4.71	35375	{}	\N	2020-10-12 15:09:44.491781
48880	IPI	2020-09-28	8.8	9	8.46	8.52	135154	{}	\N	2020-10-12 15:09:45.860589
48881	IPI	2020-09-29	8.54	8.715	7.94	8.06	334718	{}	\N	2020-10-12 15:09:45.870381
48882	IPI	2020-09-30	8.17	8.85	8.17	8.44	240249	{}	\N	2020-10-12 15:09:45.87662
48883	IPI	2020-10-01	8.48	8.62	8.2117	8.43	244130	{}	\N	2020-10-12 15:09:45.881624
48884	IPI	2020-10-02	8.16	9.185	8.16	8.87	234109	{}	\N	2020-10-12 15:09:45.886467
48885	IPI	2020-10-05	8.92	9.32	8.8517	9.17	174582	{}	\N	2020-10-12 15:09:45.89074
48886	IPI	2020-10-06	9.22	9.48	8.93	9.11	196846	{}	\N	2020-10-12 15:09:45.894966
48887	IPI	2020-10-07	9.13	9.79	9.12	9.66	295069	{}	\N	2020-10-12 15:09:45.898516
48888	IPI	2020-10-08	9.78	10.2199	9.61	10.06	270282	{}	\N	2020-10-12 15:09:45.902076
48889	IPI	2020-10-09	10.23	10.23	9.4	9.72	211600	{}	\N	2020-10-12 15:09:45.905777
48890	IPI	2020-10-12	9.69	9.79	9.47	9.5099	21492	{}	\N	2020-10-12 15:09:45.90953
48891	HPP	2020-09-28	21.79	22.36	21.66	22.16	1023303	{}	\N	2020-10-12 15:09:48.230089
48892	HPP	2020-09-29	22.12	22.12	21.4	21.52	2547287	{}	\N	2020-10-12 15:09:48.240008
48893	HPP	2020-09-30	21.7	22.13	21.67	21.93	4316150	{}	\N	2020-10-12 15:09:48.246621
48894	HPP	2020-10-01	21.9	22.325	21.79	22.28	3037937	{}	\N	2020-10-12 15:09:48.252564
48895	HPP	2020-10-02	21.75	22.72	21.6	22.63	2862584	{}	\N	2020-10-12 15:09:48.257332
48896	HPP	2020-10-05	22.89	23.04	22.42	22.49	1704542	{}	\N	2020-10-12 15:09:48.261959
48897	HPP	2020-10-06	22.69	22.8	22.21	22.31	2054359	{}	\N	2020-10-12 15:09:48.266158
48898	HPP	2020-10-07	22.49	22.53	22.01	22.24	1385970	{}	\N	2020-10-12 15:09:48.271268
48899	HPP	2020-10-08	22.36	22.97	22.3	22.97	1338428	{}	\N	2020-10-12 15:09:48.275123
48900	HPP	2020-10-09	23.13	23.24	22.56	22.67	1057856	{}	\N	2020-10-12 15:09:48.27886
48901	HPP	2020-10-12	22.7	22.7	22.3901	22.6	237706	{}	\N	2020-10-12 15:09:48.282594
48902	HRC	2020-09-28	83.05	84.08	82.7149	82.92	583993	{}	\N	2020-10-12 15:09:49.260843
48903	HRC	2020-09-29	83.18	83.72	81.88	81.94	897632	{}	\N	2020-10-12 15:09:49.273081
48904	HRC	2020-09-30	82.09	84.39	82.01	83.51	948282	{}	\N	2020-10-12 15:09:49.27993
48905	HRC	2020-10-01	83.34	84.46	81.76	82.1	1030249	{}	\N	2020-10-12 15:09:49.285288
48906	HRC	2020-10-02	81.74	83.48	81.2206	82.91	725167	{}	\N	2020-10-12 15:09:49.289919
48907	HRC	2020-10-05	83.19	85.46	82.9501	85.21	1064736	{}	\N	2020-10-12 15:09:49.293912
48908	HRC	2020-10-06	85.64	88.38	84.93	86.61	1293641	{}	\N	2020-10-12 15:09:49.297882
48909	HRC	2020-10-07	87.26	87.93	86.86	87.92	868974	{}	\N	2020-10-12 15:09:49.301601
48910	HRC	2020-10-08	88.15	89.08	88.05	89	695674	{}	\N	2020-10-12 15:09:49.305132
48911	HRC	2020-10-09	89.33	89.97	88.58	89.46	982460	{}	\N	2020-10-12 15:09:49.30828
48912	HRC	2020-10-12	89.48	91.44	89.16	91.44	137536	{}	\N	2020-10-12 15:09:49.311719
48913	HTBX	2020-09-28	1.29	1.3	1.21	1.25	4712410	{}	\N	2020-10-12 15:09:51.884062
48914	HTBX	2020-09-29	1.22	1.24	1.2	1.23	3675387	{}	\N	2020-10-12 15:09:51.89497
48915	HTBX	2020-09-30	1.2	1.24	1.18	1.24	4006544	{}	\N	2020-10-12 15:09:51.901818
48916	HTBX	2020-10-01	1.22	1.243	1.19	1.23	2727547	{}	\N	2020-10-12 15:09:51.907714
48917	HTBX	2020-10-02	1.19	1.24	1.18	1.21	3450199	{}	\N	2020-10-12 15:09:51.912237
48918	HTBX	2020-10-05	1.19	1.28	1.18	1.27	5352999	{}	\N	2020-10-12 15:09:51.917008
48919	HTBX	2020-10-06	1.259	1.28	1.2	1.22	4769625	{}	\N	2020-10-12 15:09:51.921181
48920	HTBX	2020-10-07	1.3	1.3	1.25	1.27	4382594	{}	\N	2020-10-12 15:09:51.925134
48921	HTBX	2020-10-08	1.28	1.31	1.25	1.27	3883136	{}	\N	2020-10-12 15:09:51.928797
48922	HTBX	2020-10-09	1.27	1.28	1.24	1.25	2408589	{}	\N	2020-10-12 15:09:51.933031
48923	HTBX	2020-10-12	1.25	1.25	1.22	1.2297	1019198	{}	\N	2020-10-12 15:09:51.936863
48924	HTY	2020-09-28	5.05	5.09	5	5	44536	{}	\N	2020-10-12 15:09:53.103055
48925	HTY	2020-09-29	5.01	5.03	4.97	5.02	83780	{}	\N	2020-10-12 15:09:53.115948
48926	HTY	2020-09-30	5.03	5.09	5.025	5.09	67622	{}	\N	2020-10-12 15:09:53.124226
48927	HTY	2020-10-01	5.1	5.12	5.0778	5.11	42460	{}	\N	2020-10-12 15:09:53.130584
48928	HTY	2020-10-02	5.06	5.13	5.06	5.1	42887	{}	\N	2020-10-12 15:09:53.135861
48929	HTY	2020-10-05	5.13	5.17	5.13	5.16	19738	{}	\N	2020-10-12 15:09:53.141464
48930	HTY	2020-10-06	5.16	5.21	5.13	5.13	65317	{}	\N	2020-10-12 15:09:53.145934
48931	HTY	2020-10-07	5.18	5.22	5.175	5.2	44270	{}	\N	2020-10-12 15:09:53.150268
48932	HTY	2020-10-08	5.2	5.23	5.17	5.21	36924	{}	\N	2020-10-12 15:09:53.154546
48933	HTY	2020-10-09	5.23	5.24	5.22	5.24	68768	{}	\N	2020-10-12 15:09:53.158826
48934	HTY	2020-10-12	5.24	5.2768	5.24	5.2768	12312	{}	\N	2020-10-12 15:09:53.163166
48935	HX	2020-09-28	1.53	1.605	1.46	1.52	25295	{}	\N	2020-10-12 15:09:55.539182
48936	HX	2020-09-29	1.54	1.5899	1.45	1.5517	50631	{}	\N	2020-10-12 15:09:55.550538
48937	HX	2020-09-30	1.5839	1.5839	1.46	1.48	56856	{}	\N	2020-10-12 15:09:55.55703
48938	HX	2020-10-01	1.4901	1.54	1.42	1.51	20004	{}	\N	2020-10-12 15:09:55.562797
48939	HX	2020-10-02	1.47	1.53	1.4301	1.49	25569	{}	\N	2020-10-12 15:09:55.56783
48940	HX	2020-10-05	1.54	1.55	1.47	1.5401	15744	{}	\N	2020-10-12 15:09:55.572648
48941	HX	2020-10-06	1.52	1.61	1.49	1.53	25361	{}	\N	2020-10-12 15:09:55.576935
48942	HX	2020-10-07	1.5	1.8	1.5	1.69	327693	{}	\N	2020-10-12 15:09:55.58101
48943	HX	2020-10-08	1.6538	1.75	1.52	1.64	101954	{}	\N	2020-10-12 15:09:55.585158
48944	HX	2020-10-09	1.64	1.6984	1.5501	1.6402	99466	{}	\N	2020-10-12 15:09:55.589188
48945	HX	2020-10-12	1.67	1.82	1.665	1.75	78622	{}	\N	2020-10-12 15:09:55.593109
48946	HYG	2020-09-28	83.38	83.62	83.19	83.49	18319835	{}	\N	2020-10-12 15:09:56.599844
48947	HYG	2020-09-29	83.56	83.61	83.29	83.56	19485651	{}	\N	2020-10-12 15:09:56.609625
48948	HYG	2020-09-30	83.5494	84.03	83.5494	83.9	25416204	{}	\N	2020-10-12 15:09:56.616477
48949	HYG	2020-10-01	83.81	83.89	83.69	83.83	30367931	{}	\N	2020-10-12 15:09:56.622439
48950	HYG	2020-10-02	83.28	83.87	83.28	83.83	23377114	{}	\N	2020-10-12 15:09:56.627065
48951	HYG	2020-10-05	83.99	84.48	83.92	84.46	25008249	{}	\N	2020-10-12 15:09:56.631383
48952	HYG	2020-10-06	84.48	84.75	84.22	84.27	36237815	{}	\N	2020-10-12 15:09:56.635565
48953	HYG	2020-10-07	84.52	84.63	84.465	84.57	19913990	{}	\N	2020-10-12 15:09:56.639603
48954	HYG	2020-10-08	84.85	84.85	84.64	84.76	17691368	{}	\N	2020-10-12 15:09:56.643553
48955	HYG	2020-10-09	84.89	85.03	84.763	84.95	20951718	{}	\N	2020-10-12 15:09:56.647396
48956	HYG	2020-10-12	85.22	85.32	85.13	85.32	2170055	{}	\N	2020-10-12 15:09:56.651892
48957	HYLV	2020-09-28	24.65	24.7166	24.65	24.69	12824	{}	\N	2020-10-12 15:09:57.860464
48958	HYLV	2020-09-29	24.67	24.77	24.67	24.72	7505	{}	\N	2020-10-12 15:09:57.868517
48959	HYLV	2020-09-30	24.809	24.81	24.75	24.78	2983	{}	\N	2020-10-12 15:09:57.87431
48960	HYLV	2020-10-01	24.725	24.785	24.71	24.76	8495	{}	\N	2020-10-12 15:09:57.879653
48961	HYLV	2020-10-02	24.65	24.8034	24.64	24.7841	26303	{}	\N	2020-10-12 15:09:57.885565
48962	HYLV	2020-10-05	24.86	24.89	24.86	24.89	2724	{}	\N	2020-10-12 15:09:57.890299
48963	HYLV	2020-10-06	24.87	24.9799	24.83	24.8507	14397	{}	\N	2020-10-12 15:09:57.89499
48964	HYLV	2020-10-07	25.0099	25.0099	24.97	25	13003	{}	\N	2020-10-12 15:09:57.899483
48965	HYLV	2020-10-08	25	25.01	24.9768	24.9768	27652	{}	\N	2020-10-12 15:09:57.904097
48966	HYLV	2020-10-09	25	25.09	24.96	24.99	115497	{}	\N	2020-10-12 15:09:57.908338
48967	HYLV	2020-10-12	25.15	25.15	25.0601	25.1156	3386	{}	\N	2020-10-12 15:09:57.912667
48968	IBD	2020-09-28	26.3678	26.44	26.3	26.39	19711	{}	\N	2020-10-12 15:09:59.806211
48969	IBD	2020-09-29	26.4009	26.49	26.39	26.39	160713	{}	\N	2020-10-12 15:09:59.816231
48970	IBD	2020-09-30	26.46	26.49	26.3	26.3	21982	{}	\N	2020-10-12 15:09:59.82345
48971	IBD	2020-10-01	26.27	26.45	26.27	26.35	36793	{}	\N	2020-10-12 15:09:59.829597
48972	IBD	2020-10-02	26.44	26.49	26.3887	26.4	24115	{}	\N	2020-10-12 15:09:59.834735
48973	IBD	2020-10-05	26.39	26.54	26.27	26.27	16703	{}	\N	2020-10-12 15:09:59.840274
48974	IBD	2020-10-06	26.5	26.5	26.21	26.28	118436	{}	\N	2020-10-12 15:09:59.844948
48975	IBD	2020-10-07	26.2864	26.39	26.1601	26.25	67712	{}	\N	2020-10-12 15:09:59.849405
48976	IBD	2020-10-08	26.36	26.4	26.2861	26.31	19994	{}	\N	2020-10-12 15:09:59.853471
48977	IBD	2020-10-09	26.385	26.43	26.34	26.36	39458	{}	\N	2020-10-12 15:09:59.857706
48978	IBD	2020-10-12	26.4773	26.48	26.4773	26.48	1045	{}	\N	2020-10-12 15:09:59.861863
48979	HRB	2020-09-28	15.46	16.03	15.425	15.81	5103490	{}	\N	2020-10-12 15:10:01.671502
48980	HRB	2020-09-29	15.75	16.34	15.75	16.2	8975146	{}	\N	2020-10-12 15:10:01.681553
48981	HRB	2020-09-30	16.3	16.6	16.28	16.29	7935803	{}	\N	2020-10-12 15:10:01.688275
48982	HRB	2020-10-01	16.35	16.685	16.215	16.66	8439406	{}	\N	2020-10-12 15:10:01.694688
48983	HRB	2020-10-02	16.3	16.67	16.3	16.49	6529809	{}	\N	2020-10-12 15:10:01.699567
48984	HRB	2020-10-05	16.54	16.82	16.54	16.64	6901303	{}	\N	2020-10-12 15:10:01.704561
48985	HRB	2020-10-06	16.78	17.17	16.74	16.8	6229523	{}	\N	2020-10-12 15:10:01.708753
48986	HRB	2020-10-07	17.05	17.38	17.035	17.24	6001326	{}	\N	2020-10-12 15:10:01.712719
48987	HRB	2020-10-08	17.36	17.77	17.285	17.73	5821260	{}	\N	2020-10-12 15:10:01.716941
48988	HRB	2020-10-09	17.8	17.85	17.41	17.41	4541103	{}	\N	2020-10-12 15:10:01.720978
48989	HRB	2020-10-12	17.4	17.7	17.35	17.69	1073504	{}	\N	2020-10-12 15:10:01.725148
48990	HSC	2020-09-28	13.46	14.21	13.46	13.97	697369	{}	\N	2020-10-12 15:10:03.695397
48991	HSC	2020-09-29	13.96	14.21	13.65	13.78	1063254	{}	\N	2020-10-12 15:10:03.705986
48992	HSC	2020-09-30	13.85	14.48	13.76	13.91	948381	{}	\N	2020-10-12 15:10:03.712867
48993	HSC	2020-10-01	14	14.17	13.78	14.03	692414	{}	\N	2020-10-12 15:10:03.718974
48994	HSC	2020-10-02	13.6	14.54	13.5101	14.43	518108	{}	\N	2020-10-12 15:10:03.724173
48995	HSC	2020-10-05	14.53	14.93	14.53	14.85	511473	{}	\N	2020-10-12 15:10:03.728991
48996	HSC	2020-10-06	15.1	15.5	14.67	14.68	734737	{}	\N	2020-10-12 15:10:03.733458
48997	HSC	2020-10-07	14.9	15.24	14.62	14.89	628416	{}	\N	2020-10-12 15:10:03.737967
48998	HSC	2020-10-08	15.17	15.32	14.92	15.25	404067	{}	\N	2020-10-12 15:10:03.742358
48999	HSC	2020-10-09	15.48	15.56	15	15.11	346275	{}	\N	2020-10-12 15:10:03.746831
49000	HSC	2020-10-12	15.08	15.35	14.93	15.25	47516	{}	\N	2020-10-12 15:10:03.75118
49001	HSY	2020-09-28	140.31	141.92	139.65	140.41	585324	{}	\N	2020-10-12 15:10:05.067905
49002	HSY	2020-09-29	142.3	142.78	140.3	141.19	1417463	{}	\N	2020-10-12 15:10:05.076905
49003	HSY	2020-09-30	141.43	144.4	141.22	143.34	1568252	{}	\N	2020-10-12 15:10:05.082984
49004	HSY	2020-10-01	143.49	144.52	141.71	143.93	1380411	{}	\N	2020-10-12 15:10:05.089177
49005	HSY	2020-10-02	143.5	144.05	142.09	142.92	985465	{}	\N	2020-10-12 15:10:05.094839
49006	HSY	2020-10-05	143.18	144.14	142.6196	143.77	880154	{}	\N	2020-10-12 15:10:05.099337
49007	HSY	2020-10-06	143.42	143.76	141.09	141.56	1283688	{}	\N	2020-10-12 15:10:05.103904
49008	HSY	2020-10-07	142.02	144.08	141.75	143.41	1203514	{}	\N	2020-10-12 15:10:05.108302
49009	HSY	2020-10-08	143.16	144.16	142.68	143.51	971472	{}	\N	2020-10-12 15:10:05.11272
49010	HSY	2020-10-09	144.25	146.41	143.8495	145.63	1213777	{}	\N	2020-10-12 15:10:05.117197
49011	HSY	2020-10-12	147.02	148.62	146.53	148.4	377425	{}	\N	2020-10-12 15:10:05.121566
49012	HTH	2020-09-28	20.69	21	20.58	20.64	477644	{}	\N	2020-10-12 15:10:07.133395
49013	HTH	2020-09-29	20.65	20.89	20.39	20.83	944759	{}	\N	2020-10-12 15:10:07.144554
49014	HTH	2020-09-30	20.98	21.12	20.56	20.58	1015628	{}	\N	2020-10-12 15:10:07.151658
49015	HTH	2020-10-01	20.61	20.95	20.52	20.87	715691	{}	\N	2020-10-12 15:10:07.1578
49016	HTH	2020-10-02	20.57	21.24	20.5279	21.02	918326	{}	\N	2020-10-12 15:10:07.163366
49017	HTH	2020-10-05	21.21	21.35	21.02	21.17	1039839	{}	\N	2020-10-12 15:10:07.168796
49018	HTH	2020-10-06	21.48	21.6	21.02	21.02	1515727	{}	\N	2020-10-12 15:10:07.173315
49019	HTH	2020-10-07	21.24	21.48	20.97	21.09	1203775	{}	\N	2020-10-12 15:10:07.177768
49020	HTH	2020-10-08	21.28	21.5	20.99	21.36	1685964	{}	\N	2020-10-12 15:10:07.182312
49021	HTH	2020-10-09	21.6	21.6	21	21.25	668736	{}	\N	2020-10-12 15:10:07.186889
49022	HTH	2020-10-12	21.16	21.625	21.09	21.58	172400	{}	\N	2020-10-12 15:10:07.191316
49023	HUBG	2020-09-28	49.96	51.075	49.74	50.55	181535	{}	\N	2020-10-12 15:10:08.196426
49024	HUBG	2020-09-29	50.57	50.88	50.2	50.52	159049	{}	\N	2020-10-12 15:10:08.208762
49025	HUBG	2020-09-30	50.74	51.02	49.63	50.195	230064	{}	\N	2020-10-12 15:10:08.216658
49026	HUBG	2020-10-01	50.57	51.27	49.74	50.04	131804	{}	\N	2020-10-12 15:10:08.222692
49027	HUBG	2020-10-02	49.58	51.36	49.32	51.2	206669	{}	\N	2020-10-12 15:10:08.227941
49028	HUBG	2020-10-05	51.5	52.22	51.44	51.99	145327	{}	\N	2020-10-12 15:10:08.233201
49029	HUBG	2020-10-06	52.43	53.37	51.54	52.81	297200	{}	\N	2020-10-12 15:10:08.237917
49030	HUBG	2020-10-07	53.32	54.48	53.08	53.71	319707	{}	\N	2020-10-12 15:10:08.242466
49031	HUBG	2020-10-08	53.88	54.1422	53.38	53.9	121405	{}	\N	2020-10-12 15:10:08.24687
49032	HUBG	2020-10-09	54.29	54.69	53.51	53.7	184189	{}	\N	2020-10-12 15:10:08.251094
49033	HUBG	2020-10-12	53.65	54.43	53.65	54.18	29383	{}	\N	2020-10-12 15:10:08.255384
49034	HUSA	2020-09-28	1.15	1.17	1.06	1.0918	202516	{}	\N	2020-10-12 15:10:09.845673
49035	HUSA	2020-09-29	3.05	6.36	2.75	2.98	146018847	{}	\N	2020-10-12 15:10:09.856035
49036	HUSA	2020-09-30	2.35	2.55	1.92	1.94	4240268	{}	\N	2020-10-12 15:10:09.862024
49037	HUSA	2020-10-01	1.81	1.9	1.6	1.62	2089375	{}	\N	2020-10-12 15:10:09.867611
49038	HUSA	2020-10-02	1.72	1.73	1.51	1.53	1279895	{}	\N	2020-10-12 15:10:09.872579
49039	HUSA	2020-10-05	1.48	1.55	1.41	1.53	1254387	{}	\N	2020-10-12 15:10:09.876748
49040	HUSA	2020-10-06	1.57	1.78	1.4828	1.63	3498804	{}	\N	2020-10-12 15:10:09.881002
49041	HUSA	2020-10-07	1.68	1.92	1.56	1.91	2147236	{}	\N	2020-10-12 15:10:09.884757
49042	HUSA	2020-10-08	1.8003	2.72	1.8	2.34	12067877	{}	\N	2020-10-12 15:10:09.888223
49043	HUSA	2020-10-09	2.17	2.2	1.75	1.89	2640269	{}	\N	2020-10-12 15:10:09.89167
49044	HUSA	2020-10-12	1.9	1.93	1.76	1.8	398231	{}	\N	2020-10-12 15:10:09.89515
49045	HYDB	2020-09-28	48.58	48.6099	48.45	48.47	1743	{}	\N	2020-10-12 15:10:11.465404
49046	HYDB	2020-09-29	48.51	48.5367	48.5	48.5	1360	{}	\N	2020-10-12 15:10:11.477592
49047	HYDB	2020-09-30	48.65	48.65	48.605	48.605	639	{}	\N	2020-10-12 15:10:11.484866
49048	HYDB	2020-10-01	48.38	48.61	48.38	48.544	2285	{}	\N	2020-10-12 15:10:11.4914
49049	HYDB	2020-10-02	48.5099	48.58	48.4699	48.52	2027	{}	\N	2020-10-12 15:10:11.49618
49050	HYDB	2020-10-05	48.769	48.84	48.769	48.77	69681	{}	\N	2020-10-12 15:10:11.500926
49051	HYDB	2020-10-06	48.84	48.9799	48.72	48.7336	4706	{}	\N	2020-10-12 15:10:11.505287
49052	HYDB	2020-10-07	49.0499	49.14	48.963	49.14	4478	{}	\N	2020-10-12 15:10:11.509221
49053	HYDB	2020-10-08	49.2	49.24	49.09	49.21	4724	{}	\N	2020-10-12 15:10:11.513011
49054	HYDB	2020-10-09	49.28	49.29	49.14	49.28	5556	{}	\N	2020-10-12 15:10:11.516776
49055	HYDB	2020-10-12	49.49	49.49	49.3428	49.3693	3660	{}	\N	2020-10-12 15:10:11.520671
49056	HYHG	2020-09-28	59.459	59.459	59.3747	59.3747	905	{}	\N	2020-10-12 15:10:13.092891
49057	HYHG	2020-09-29	59.43	59.698	59.43	59.4719	2476	{}	\N	2020-10-12 15:10:13.102739
49058	HYHG	2020-09-30	59.65	59.81	59.65	59.7	1860	{}	\N	2020-10-12 15:10:13.109397
49059	HYHG	2020-10-01	59.62	59.7587	59.32	59.32	4924	{}	\N	2020-10-12 15:10:13.115861
49060	HYHG	2020-10-02	59.46	59.5475	59.42	59.42	1484	{}	\N	2020-10-12 15:10:13.120971
49061	HYHG	2020-10-05	59.56	60.25	59.56	60.04	4086	{}	\N	2020-10-12 15:10:13.125986
49062	HYHG	2020-10-06	60.08	60.18	59.85	59.85	3990	{}	\N	2020-10-12 15:10:13.130098
49063	HYHG	2020-10-07	59.49	60.54	59.49	60.34	5733	{}	\N	2020-10-12 15:10:13.134188
49064	HYHG	2020-10-08	60.38	60.5276	60.38	60.4612	2604	{}	\N	2020-10-12 15:10:13.138621
49065	HYHG	2020-10-09	60.35	60.4475	60.35	60.41	2000	{}	\N	2020-10-12 15:10:13.142638
49066	HYHG	2020-10-12	60.5866	60.5866	60.5866	60.5866	335	{}	\N	2020-10-12 15:10:13.146934
49067	HYT	2020-09-28	10.6	10.66	10.55	10.56	380209	{}	\N	2020-10-12 15:10:16.171915
49068	HYT	2020-09-29	10.5693	10.57	10.5	10.55	566983	{}	\N	2020-10-12 15:10:16.18395
49069	HYT	2020-09-30	10.62	10.68	10.57	10.68	1863230	{}	\N	2020-10-12 15:10:16.190507
49070	HYT	2020-10-01	10.7	10.75	10.66	10.7	748668	{}	\N	2020-10-12 15:10:16.195507
49071	HYT	2020-10-02	10.61	10.75	10.61	10.75	391785	{}	\N	2020-10-12 15:10:16.200371
49072	HYT	2020-10-05	10.8	10.91	10.7881	10.91	505870	{}	\N	2020-10-12 15:10:16.204672
49073	HYT	2020-10-06	10.91	10.95	10.87	10.91	773534	{}	\N	2020-10-12 15:10:16.20911
49074	HYT	2020-10-07	10.96	11.061	10.96	11.05	724823	{}	\N	2020-10-12 15:10:16.212906
49075	HYT	2020-10-08	11.09	11.15	11.06	11.11	442772	{}	\N	2020-10-12 15:10:16.216822
49076	HYT	2020-10-09	11.16	11.1653	11.08	11.11	625652	{}	\N	2020-10-12 15:10:16.220474
49077	HYT	2020-10-12	11.14	11.19	11.14	11.1663	194086	{}	\N	2020-10-12 15:10:16.224046
49078	IAE	2020-09-28	7.861	7.9	7.77	7.81	37299	{}	\N	2020-10-12 15:10:17.224105
49079	IAE	2020-09-29	7.83	7.92	7.63	7.9	122414	{}	\N	2020-10-12 15:10:17.234991
49080	IAE	2020-09-30	7.91	7.93	7.79	7.92	177258	{}	\N	2020-10-12 15:10:17.242173
49081	IAE	2020-10-01	7.77	7.77	7.43	7.6	274830	{}	\N	2020-10-12 15:10:17.248387
49082	IAE	2020-10-02	7.5	7.59	7.43	7.52	102918	{}	\N	2020-10-12 15:10:17.25397
49083	IAE	2020-10-05	7.54	7.66	7.495	7.66	139718	{}	\N	2020-10-12 15:10:17.259153
49084	IAE	2020-10-06	7.66	7.66	7.55	7.56	72751	{}	\N	2020-10-12 15:10:17.263624
49085	IAE	2020-10-07	7.59	7.69	7.59	7.64	70876	{}	\N	2020-10-12 15:10:17.268019
49086	IAE	2020-10-08	7.69	7.77	7.5305	7.75	35646	{}	\N	2020-10-12 15:10:17.272549
49087	IAE	2020-10-09	7.76	7.83	7.65	7.83	64488	{}	\N	2020-10-12 15:10:17.276912
49088	IAE	2020-10-12	7.8	7.8801	7.78	7.88	34124	{}	\N	2020-10-12 15:10:17.281173
49089	IAT	2020-09-28	32.17	32.87	32.1497	32.63	165897	{}	\N	2020-10-12 15:10:18.246535
49090	IAT	2020-09-29	32.3	32.58	31.72	32.22	59548	{}	\N	2020-10-12 15:10:18.256351
49091	IAT	2020-09-30	32.53	33.05	32.39	32.7	99904	{}	\N	2020-10-12 15:10:18.263306
49092	IAT	2020-10-01	32.86	33.07	32.5	32.9	78108	{}	\N	2020-10-12 15:10:18.269808
49093	IAT	2020-10-02	32.36	33.96	32.3301	33.84	78420	{}	\N	2020-10-12 15:10:18.275082
49094	IAT	2020-10-05	34.35	35.05	34.28	35.04	101106	{}	\N	2020-10-12 15:10:18.280011
49095	IAT	2020-10-06	35.35	36.25	34.73	34.85	251912	{}	\N	2020-10-12 15:10:18.284663
49096	IAT	2020-10-07	35.38	36.22	35.38	35.93	88424	{}	\N	2020-10-12 15:10:18.28893
49097	IAT	2020-10-08	36.06	36.5	35.95	36.47	71544	{}	\N	2020-10-12 15:10:18.293346
49098	IAT	2020-10-09	36.67	36.77	35.87	36.15	91860	{}	\N	2020-10-12 15:10:18.297935
49099	IAT	2020-10-12	36.16	36.44	36.07	36.35	31833	{}	\N	2020-10-12 15:10:18.30255
49100	HSKA	2020-09-28	99.67	101.66	99.37	100.44	73270	{}	\N	2020-10-12 15:10:20.632306
49101	HSKA	2020-09-29	99.89	101.33	97.6825	98.99	60979	{}	\N	2020-10-12 15:10:20.640434
49102	HSKA	2020-09-30	99.63	100.295	97.73	98.79	67946	{}	\N	2020-10-12 15:10:20.646605
49103	HSKA	2020-10-01	99.46	99.93	97.62	98.58	44624	{}	\N	2020-10-12 15:10:20.652543
49104	HSKA	2020-10-02	97.12	99.11	96.1	96.1	42044	{}	\N	2020-10-12 15:10:20.658038
49105	HSKA	2020-10-05	97.29	101.98	97.29	98.58	48737	{}	\N	2020-10-12 15:10:20.66281
49106	HSKA	2020-10-06	98.87	101.235	98.07	99.27	73942	{}	\N	2020-10-12 15:10:20.667297
49107	HSKA	2020-10-07	100.98	104.78	100.35	104.16	67331	{}	\N	2020-10-12 15:10:20.67161
49108	HSKA	2020-10-08	105.22	109	104.9	106.33	62522	{}	\N	2020-10-12 15:10:20.675877
49109	HSKA	2020-10-09	107.38	107.67	104.7	107.33	29622	{}	\N	2020-10-12 15:10:20.680598
49110	HSKA	2020-10-12	107.99	107.99	104.45	105.055	15266	{}	\N	2020-10-12 15:10:20.685061
49111	HTD	2020-09-28	19.04	19.34	19.022	19.24	60907	{}	\N	2020-10-12 15:10:21.772018
49112	HTD	2020-09-29	19.19	19.22	18.9906	19.1	124616	{}	\N	2020-10-12 15:10:21.781464
49113	HTD	2020-09-30	19.2	19.38	19.06	19.09	283227	{}	\N	2020-10-12 15:10:21.78828
49114	HTD	2020-10-01	19.11	19.35	19.03	19.28	198536	{}	\N	2020-10-12 15:10:21.793801
49115	HTD	2020-10-02	19.15	19.57	19.0872	19.56	160684	{}	\N	2020-10-12 15:10:21.798504
49116	HTD	2020-10-05	19.59	19.88	19.51	19.79	193542	{}	\N	2020-10-12 15:10:21.803453
49117	HTD	2020-10-06	19.85	19.98	19.73	19.77	239137	{}	\N	2020-10-12 15:10:21.807767
49118	HTD	2020-10-07	19.94	20.0796	19.854	20.02	161403	{}	\N	2020-10-12 15:10:21.811577
49119	HTD	2020-10-08	20.07	20.28	20.065	20.25	137818	{}	\N	2020-10-12 15:10:21.815598
49120	HTD	2020-10-09	20.31	20.33	19.95	20	158050	{}	\N	2020-10-12 15:10:21.819427
49121	HTD	2020-10-12	20.01	20.1	19.96	20	97812	{}	\N	2020-10-12 15:10:21.823203
49122	HTLD	2020-09-28	18.84	19.13	18.65	19.08	785584	{}	\N	2020-10-12 15:10:22.751688
49123	HTLD	2020-09-29	19.02	19.15	18.74	18.82	804036	{}	\N	2020-10-12 15:10:22.760267
49124	HTLD	2020-09-30	18.89	19.1	18.515	18.6	336109	{}	\N	2020-10-12 15:10:22.765843
49125	HTLD	2020-10-01	18.58	18.835	18.44	18.75	210228	{}	\N	2020-10-12 15:10:22.771275
49126	HTLD	2020-10-02	18.64	19.37	18.575	19.12	249105	{}	\N	2020-10-12 15:10:22.77619
49127	HTLD	2020-10-05	19.26	19.43	19.16	19.37	296988	{}	\N	2020-10-12 15:10:22.780115
49128	HTLD	2020-10-06	19.46	19.62	19.31	19.46	319629	{}	\N	2020-10-12 15:10:22.784107
49129	HTLD	2020-10-07	19.46	19.76	19.33	19.63	288709	{}	\N	2020-10-12 15:10:22.787847
49130	HTLD	2020-10-08	19.8	19.94	19.52	19.8	204630	{}	\N	2020-10-12 15:10:22.791848
49131	HTLD	2020-10-09	20.33	20.33	19.85	20	246581	{}	\N	2020-10-12 15:10:22.796636
49132	HTLD	2020-10-12	19.95	20.2299	19.95	20.17	40285	{}	\N	2020-10-12 15:10:22.800775
49133	HUM	2020-09-28	403.97	409.47	399.27	400	688938	{}	\N	2020-10-12 15:10:23.851458
49134	HUM	2020-09-29	400.59	404.24	398.6417	401.5	1657050	{}	\N	2020-10-12 15:10:23.862409
49135	HUM	2020-09-30	403.2	419.06	401.125	413.89	1384978	{}	\N	2020-10-12 15:10:23.868867
49136	HUM	2020-10-01	415.92	417.14	409.69	415.36	1066734	{}	\N	2020-10-12 15:10:23.87441
49137	HUM	2020-10-02	414.33	415.28	405.14	408.31	1016755	{}	\N	2020-10-12 15:10:23.879439
49138	HUM	2020-10-05	412.8	419.88	410.635	419.62	714633	{}	\N	2020-10-12 15:10:23.884116
49139	HUM	2020-10-06	420.67	421.56	414.34	416.2	1365624	{}	\N	2020-10-12 15:10:23.888156
49140	HUM	2020-10-07	418.26	430.23	417.6392	429.41	952395	{}	\N	2020-10-12 15:10:23.891668
49141	HUM	2020-10-08	430.1	434.065	425.51	427.32	893871	{}	\N	2020-10-12 15:10:23.895135
49142	HUM	2020-10-09	429.59	435.77	429.04	433.44	684603	{}	\N	2020-10-12 15:10:23.899228
49143	HUM	2020-10-12	436.6	440.35	433.78	437.515	335954	{}	\N	2020-10-12 15:10:23.90294
49144	HYAC	2020-09-28	10	10.0432	10	10	8204	{}	\N	2020-10-12 15:10:27.038792
49145	HYAC	2020-09-29	10.0007	10.02	9.99	10	429249	{}	\N	2020-10-12 15:10:27.049102
49146	HYAC	2020-09-30	9.99	10.05	9.99	10	262244	{}	\N	2020-10-12 15:10:27.054838
49147	HYAC	2020-10-01	10.0125	10.04	9.99	10	2187940	{}	\N	2020-10-12 15:10:27.059951
49148	HYAC	2020-10-02	10	10.07	10	10.02	142145	{}	\N	2020-10-12 15:10:27.064268
49149	HYAC	2020-10-05	10.03	10.03	10	10	831473	{}	\N	2020-10-12 15:10:27.068376
49150	HYAC	2020-10-06	10.005	10.05	10	10	615857	{}	\N	2020-10-12 15:10:27.072575
49151	HYAC	2020-10-07	10.0626	10.0626	10.01	10.02	559818	{}	\N	2020-10-12 15:10:27.075927
49152	HYAC	2020-10-08	10.04	10.13	10	10.1	1949139	{}	\N	2020-10-12 15:10:27.079541
49153	HYAC	2020-10-09	10.11	10.11	10.04	10.09	237858	{}	\N	2020-10-12 15:10:27.083035
49154	HYAC	2020-10-12	10.08	10.08	10.01	10.01	5142	{}	\N	2020-10-12 15:10:27.086403
49155	HYDW	2020-09-28	49.49	49.58	49.49	49.5799	7537	{}	\N	2020-10-12 15:10:28.283967
49156	HYDW	2020-09-29	49.53	49.62	49.53	49.6	2942	{}	\N	2020-10-12 15:10:28.290278
49157	HYDW	2020-09-30	49.75	49.78	49.6925	49.73	4540	{}	\N	2020-10-12 15:10:28.295117
49158	HYDW	2020-10-01	49.57	49.71	49.57	49.7	1556	{}	\N	2020-10-12 15:10:28.299036
49159	HYDW	2020-10-02	49.53	49.72	49.53	49.72	5829	{}	\N	2020-10-12 15:10:28.303156
49160	HYDW	2020-10-05	49.9201	50.0235	49.9	50.02	2772	{}	\N	2020-10-12 15:10:28.306984
49161	HYDW	2020-10-06	50.08	50.34	49.93	50.02	60064	{}	\N	2020-10-12 15:10:28.310649
49162	HYDW	2020-10-07	50.04	50.19	50.04	50.19	8874	{}	\N	2020-10-12 15:10:28.313949
49163	HYDW	2020-10-08	50.11	50.2122	50.11	50.2122	5252	{}	\N	2020-10-12 15:10:28.317194
49164	HYDW	2020-10-09	50.2101	50.32	50.21	50.27	6500	{}	\N	2020-10-12 15:10:28.320822
49165	HYLS	2020-09-28	46.94	47	46.78	46.92	317689	{}	\N	2020-10-12 15:10:29.434998
49166	HYLS	2020-09-29	47.09	47.18	46.81	47.01	1622136	{}	\N	2020-10-12 15:10:29.443694
49167	HYLS	2020-09-30	47.09	47.309	46.95	47.18	1162279	{}	\N	2020-10-12 15:10:29.44958
49168	HYLS	2020-10-01	47.29	47.4	47.16	47.35	476924	{}	\N	2020-10-12 15:10:29.454632
49169	HYLS	2020-10-02	47.01	47.37	47	47.33	374143	{}	\N	2020-10-12 15:10:29.459001
49170	HYLS	2020-10-05	47.27	47.55	47.27	47.5	1043548	{}	\N	2020-10-12 15:10:29.463423
49171	HYLS	2020-10-06	47.44	47.59	47.27	47.27	713067	{}	\N	2020-10-12 15:10:29.467429
49172	HYLS	2020-10-07	47.41	47.65	47.32	47.54	635741	{}	\N	2020-10-12 15:10:29.471114
49173	HYLS	2020-10-08	47.68	47.6915	47.54	47.58	384548	{}	\N	2020-10-12 15:10:29.4747
49174	HYLS	2020-10-09	47.6071	47.7499	47.5902	47.67	397600	{}	\N	2020-10-12 15:10:29.478405
49175	HYLS	2020-10-12	47.82	47.82	47.7201	47.7766	39143	{}	\N	2020-10-12 15:10:29.482596
49176	HZNP	2020-09-28	80.5	80.54	78.01	79.67	2017425	{}	\N	2020-10-12 15:10:31.106119
49177	HZNP	2020-09-29	79.31	80.015	78.0242	78.16	1507282	{}	\N	2020-10-12 15:10:31.116671
49178	HZNP	2020-09-30	78.16	78.73	77.29	77.68	1779338	{}	\N	2020-10-12 15:10:31.123539
49179	HZNP	2020-10-01	79.1	81.21	78.32	80.42	3471567	{}	\N	2020-10-12 15:10:31.129142
49180	HZNP	2020-10-02	78.4	80.835	78.38	79.86	1401987	{}	\N	2020-10-12 15:10:31.134356
49181	HZNP	2020-10-05	80.48	82.79	80.24	81.56	1872379	{}	\N	2020-10-12 15:10:31.139361
49182	HZNP	2020-10-06	79.4423	81.94	79.4	80.1	1454940	{}	\N	2020-10-12 15:10:31.143994
49183	HZNP	2020-10-07	81.08	81.25	79.0132	80.54	2126136	{}	\N	2020-10-12 15:10:31.148145
49184	HZNP	2020-10-08	81.67	84.17	80.18	83.45	1858522	{}	\N	2020-10-12 15:10:31.152295
49185	HZNP	2020-10-09	84.67	86.67	82.845	84.39	1954199	{}	\N	2020-10-12 15:10:31.156562
49186	HZNP	2020-10-12	85.4	85.42	81.61	83.19	662943	{}	\N	2020-10-12 15:10:31.160796
49187	IBA	2020-09-28	37.49	37.49	36.3834	37.11	3603	{}	\N	2020-10-12 15:10:32.35189
49188	IBA	2020-09-29	36.99	38.8	36.99	37.02	11344	{}	\N	2020-10-12 15:10:32.35933
49189	IBA	2020-09-30	37.43	38.295	37.25	38.295	8200	{}	\N	2020-10-12 15:10:32.370045
49190	IBA	2020-10-01	37.83	38.39	37.83	38.26	3596	{}	\N	2020-10-12 15:10:32.37647
49191	IBA	2020-10-02	38.34	38.85	38.28	38.45	7440	{}	\N	2020-10-12 15:10:32.381542
49192	IBA	2020-10-05	38.46	38.67	38.435	38.67	3418	{}	\N	2020-10-12 15:10:32.38548
49193	IBA	2020-10-06	38.49	38.49	37.7	37.7	5518	{}	\N	2020-10-12 15:10:32.389499
49194	IBA	2020-10-07	37.85	38.605	37.85	38.4	8638	{}	\N	2020-10-12 15:10:32.393377
49195	IBA	2020-10-08	38.37	39.6	38.12	39.56	9254	{}	\N	2020-10-12 15:10:32.397255
49196	IBA	2020-10-09	39.72	39.97	39.1186	39.92	6132	{}	\N	2020-10-12 15:10:32.401026
49197	IBA	2020-10-12	39.54	39.54	39.54	39.54	414	{}	\N	2020-10-12 15:10:32.404952
49198	IBOC	2020-09-28	26.12	26.61	25.59	26.45	171160	{}	\N	2020-10-12 15:10:35.452035
49199	IBOC	2020-09-29	26.3	26.38	25.56	25.98	116455	{}	\N	2020-10-12 15:10:35.460955
49200	IBOC	2020-09-30	26.07	26.66	25.69	26.06	176106	{}	\N	2020-10-12 15:10:35.466908
49201	IBOC	2020-10-01	26.38	26.455	25.65	26.05	167603	{}	\N	2020-10-12 15:10:35.472139
49202	IBOC	2020-10-02	25.44	26.66	25.44	26.46	156327	{}	\N	2020-10-12 15:10:35.476567
49203	IBOC	2020-10-05	26.9	27.76	26.9	27.71	175096	{}	\N	2020-10-12 15:10:35.480693
49204	IBOC	2020-10-06	28.21	29.2	27.92	28.22	266383	{}	\N	2020-10-12 15:10:35.48475
49205	IBOC	2020-10-07	28.275	28.86	27.955	28.685	160756	{}	\N	2020-10-12 15:10:35.488531
49206	IBOC	2020-10-08	29.09	29.24	28.58	29.11	133798	{}	\N	2020-10-12 15:10:35.492283
49207	IBOC	2020-10-09	29.39	29.39	28.49	28.54	112223	{}	\N	2020-10-12 15:10:35.495964
49208	IBOC	2020-10-12	28.5	29.12	28.07	29.12	14679	{}	\N	2020-10-12 15:10:35.499781
49209	HSTM	2020-09-28	20.13	20.46	20.07	20.13	139482	{}	\N	2020-10-12 15:10:37.474282
49210	HSTM	2020-09-29	20.13	20.32	19.93	20.05	98070	{}	\N	2020-10-12 15:10:37.485747
49211	HSTM	2020-09-30	20.1	20.41	20	20.07	162910	{}	\N	2020-10-12 15:10:37.491983
49212	HSTM	2020-10-01	20.3	20.3	19.98	20.07	138963	{}	\N	2020-10-12 15:10:37.496826
49213	HSTM	2020-10-02	19.8	20.37	19.66	20.18	145835	{}	\N	2020-10-12 15:10:37.501295
49214	HSTM	2020-10-05	20.26	20.85	20.26	20.59	89348	{}	\N	2020-10-12 15:10:37.505789
49215	HSTM	2020-10-06	20.73	21.22	20.57	20.78	158085	{}	\N	2020-10-12 15:10:37.510524
49216	HSTM	2020-10-07	20.9	21.18	20.75	20.87	88644	{}	\N	2020-10-12 15:10:37.514516
49217	HSTM	2020-10-08	20.96	21.3	20.9	21.18	67423	{}	\N	2020-10-12 15:10:37.51851
49218	HSTM	2020-10-09	21.23	21.88	21	21.38	66079	{}	\N	2020-10-12 15:10:37.522555
49219	HSTM	2020-10-12	21.39	21.565	21.3	21.56	6485	{}	\N	2020-10-12 15:10:37.526436
49220	HTGM	2020-09-28	0.3051	0.3394	0.3051	0.333	2380753	{}	\N	2020-10-12 15:10:38.754355
49221	HTGM	2020-09-29	0.333	0.3877	0.324	0.3503	9502834	{}	\N	2020-10-12 15:10:38.767272
49222	HTGM	2020-09-30	0.3466	0.3636	0.3251	0.3304	4566127	{}	\N	2020-10-12 15:10:38.774335
49223	HTGM	2020-10-01	0.33	0.335	0.307219	0.3218	1824674	{}	\N	2020-10-12 15:10:38.780038
49224	HTGM	2020-10-02	0.314	0.3333	0.306	0.3144	1863729	{}	\N	2020-10-12 15:10:38.785063
49225	HTGM	2020-10-05	0.3151	0.324	0.306	0.3156	1204844	{}	\N	2020-10-12 15:10:38.789469
49226	HTGM	2020-10-06	0.31	0.325	0.3	0.305	1466671	{}	\N	2020-10-12 15:10:38.794166
49227	HTGM	2020-10-07	0.31	0.3295	0.3008	0.32	1939424	{}	\N	2020-10-12 15:10:38.798445
49228	HTGM	2020-10-08	0.3286	0.338	0.32	0.3297	1600619	{}	\N	2020-10-12 15:10:38.802576
49229	HTGM	2020-10-09	0.33	0.3444	0.3256	0.341	1448801	{}	\N	2020-10-12 15:10:38.806642
49230	HTGM	2020-10-12	0.34	0.3598	0.330201	0.333	697609	{}	\N	2020-10-12 15:10:38.810474
49231	HWCC	2020-09-28	2.58	2.705	2.485	2.585	7691	{}	\N	2020-10-12 15:10:41.264903
49232	HWCC	2020-09-29	2.52	2.58	2.46	2.51	6673	{}	\N	2020-10-12 15:10:41.278466
49233	HWCC	2020-09-30	2.5273	2.63	2.42	2.5	27269	{}	\N	2020-10-12 15:10:41.284458
49234	HWCC	2020-10-01	2.52	2.62	2.44	2.52	29433	{}	\N	2020-10-12 15:10:41.289393
49235	HWCC	2020-10-02	2.45	2.65	2.4	2.6	6984	{}	\N	2020-10-12 15:10:41.294065
49236	HWCC	2020-10-05	2.65	2.84	2.63	2.71	19917	{}	\N	2020-10-12 15:10:41.298213
49237	HWCC	2020-10-06	2.8327	2.8327	2.733	2.74	6151	{}	\N	2020-10-12 15:10:41.302471
49238	HWCC	2020-10-07	2.83	2.8499	2.635	2.795	12111	{}	\N	2020-10-12 15:10:41.306738
49239	HWCC	2020-10-08	2.785	2.8	2.63	2.79	9445	{}	\N	2020-10-12 15:10:41.310715
49240	HWCC	2020-10-09	2.73	2.85	2.63	2.76	25687	{}	\N	2020-10-12 15:10:41.314893
49241	HWCC	2020-10-12	2.7	2.705	2.62	2.65	6740	{}	\N	2020-10-12 15:10:41.318938
49242	HYEM	2020-09-28	22.51	22.67	22.37	22.64	230061	{}	\N	2020-10-12 15:10:43.15186
49243	HYEM	2020-09-29	22.58	22.72	22.51	22.56	91011	{}	\N	2020-10-12 15:10:43.160557
49244	HYEM	2020-09-30	22.62	22.77	22.5601	22.74	175794	{}	\N	2020-10-12 15:10:43.166504
49245	HYEM	2020-10-01	22.53	22.71	22.2008	22.59	654422	{}	\N	2020-10-12 15:10:43.172461
49246	HYEM	2020-10-02	22.52	22.64	22.3814	22.57	55617	{}	\N	2020-10-12 15:10:43.177696
49247	HYEM	2020-10-05	22.56	22.7	22.5015	22.64	404993	{}	\N	2020-10-12 15:10:43.182471
49248	HYEM	2020-10-06	22.68	22.7769	22.5	22.61	81168	{}	\N	2020-10-12 15:10:43.186615
49249	HYEM	2020-10-07	22.7	22.82	22.59	22.77	744513	{}	\N	2020-10-12 15:10:43.190817
49250	HYEM	2020-10-08	22.8	22.89	22.7501	22.89	576884	{}	\N	2020-10-12 15:10:43.195035
49251	HYEM	2020-10-09	22.91	22.94	22.76	22.9	142468	{}	\N	2020-10-12 15:10:43.198888
49252	HYEM	2020-10-12	22.9	22.9593	22.69	22.8236	34404	{}	\N	2020-10-12 15:10:43.203176
49253	HYZD	2020-09-28	20.6208	21.14	20.6208	20.71	47703	{}	\N	2020-10-12 15:10:46.46656
49254	HYZD	2020-09-29	20.6501	20.75	20.64	20.64	13617	{}	\N	2020-10-12 15:10:46.476243
49255	HYZD	2020-09-30	20.66	20.77	20.66	20.77	17229	{}	\N	2020-10-12 15:10:46.482613
49256	HYZD	2020-10-01	20.74	20.79	20.74	20.77	18712	{}	\N	2020-10-12 15:10:46.488609
49257	HYZD	2020-10-02	20.7	20.7844	20.7	20.76	11032	{}	\N	2020-10-12 15:10:46.493752
49258	HYZD	2020-10-05	20.77	20.9466	20.68	20.9466	39445	{}	\N	2020-10-12 15:10:46.498716
49259	HYZD	2020-10-06	20.8905	20.9826	20.8905	20.95	7805	{}	\N	2020-10-12 15:10:46.503233
49260	HYZD	2020-10-07	20.89	21.6	20.89	21.48	61853	{}	\N	2020-10-12 15:10:46.507423
49261	HYZD	2020-10-08	21	21.09	21	21.04	15181	{}	\N	2020-10-12 15:10:46.511696
49262	HYZD	2020-10-09	21.02	21.0873	21.02	21.07	30451	{}	\N	2020-10-12 15:10:46.516151
49263	HYZD	2020-10-12	21.08	21.1	21.08	21.1	10838	{}	\N	2020-10-12 15:10:46.520863
49264	IAI	2020-09-28	61.22	62.14	61.22	61.76	8005	{}	\N	2020-10-12 15:10:47.547711
49265	IAI	2020-09-29	61.82	61.94	60.9	61.07	10138	{}	\N	2020-10-12 15:10:47.558034
49266	IAI	2020-09-30	61.7023	62.295	61.44	61.49	11800	{}	\N	2020-10-12 15:10:47.565581
49267	IAI	2020-10-01	62.24	62.6271	61.92	62.02	7375	{}	\N	2020-10-12 15:10:47.572228
49268	IAI	2020-10-02	60.96	62.7066	60.96	62.45	7020	{}	\N	2020-10-12 15:10:47.577343
49269	IAI	2020-10-05	63.23	63.23	62.6912	62.94	15484	{}	\N	2020-10-12 15:10:47.583186
49270	IAI	2020-10-06	63.2	63.77	62.63	62.63	11251	{}	\N	2020-10-12 15:10:47.587152
49271	IAI	2020-10-07	63.4	63.5132	62.9	63.39	4855	{}	\N	2020-10-12 15:10:47.59154
49272	IAI	2020-10-08	63.74	64.2	63.3578	64.19	6539	{}	\N	2020-10-12 15:10:47.595646
49273	IAI	2020-10-09	64.91	64.91	64.06	64.25	4432	{}	\N	2020-10-12 15:10:47.600554
49274	IAI	2020-10-12	64.41	65.21	64.41	65.21	1241	{}	\N	2020-10-12 15:10:47.604994
49275	IBB	2020-09-28	134.71	134.95	133.27	134.48	1601032	{}	\N	2020-10-12 15:10:49.273231
49276	IBB	2020-09-29	134.585	135.485	133.79	134.61	1616146	{}	\N	2020-10-12 15:10:49.282916
49277	IBB	2020-09-30	135.34	136.73	134.5	135.41	2635855	{}	\N	2020-10-12 15:10:49.289567
49278	IBB	2020-10-01	136.1	136.85	135.39	136.67	2046947	{}	\N	2020-10-12 15:10:49.295193
49279	IBB	2020-10-02	134.75	136.66	133.23	133.87	2393532	{}	\N	2020-10-12 15:10:49.300615
49280	IBB	2020-10-05	136.13	139.76	136.11	139.57	2956357	{}	\N	2020-10-12 15:10:49.305448
49281	IBB	2020-10-06	139.84	140.37	137.71	137.88	2554380	{}	\N	2020-10-12 15:10:49.309757
49282	IBB	2020-10-07	139.44	141.49	138.72	140.99	1683984	{}	\N	2020-10-12 15:10:49.3144
49283	IBB	2020-10-08	141.75	141.91	140.6	141.38	1482431	{}	\N	2020-10-12 15:10:49.318826
49284	IBB	2020-10-09	142.43	142.62	141.245	142.53	2196779	{}	\N	2020-10-12 15:10:49.322914
49285	IBB	2020-10-12	143.75	143.82	142.01	143.53	977496	{}	\N	2020-10-12 15:10:49.327021
49286	IBDM	2020-09-28	25.04	25.06	25.04	25.06	311759	{}	\N	2020-10-12 15:10:50.543989
49287	IBDM	2020-09-29	25.06	25.07	25.05	25.05	405983	{}	\N	2020-10-12 15:10:50.554184
49288	IBDM	2020-09-30	25.075	25.075	25.04	25.05	120789	{}	\N	2020-10-12 15:10:50.56089
49289	IBDM	2020-10-01	25.03	25.03	25	25.02	128738	{}	\N	2020-10-12 15:10:50.566888
49290	IBDM	2020-10-02	25.02	25.02	24.99	25.02	186711	{}	\N	2020-10-12 15:10:50.572223
49291	IBDM	2020-10-05	25.02	25.025	25	25.02	184004	{}	\N	2020-10-12 15:10:50.577038
49292	IBDM	2020-10-06	25	25.02	25	25.01	138833	{}	\N	2020-10-12 15:10:50.581913
49293	IBDM	2020-10-07	25.02	25.02	25	25.02	151454	{}	\N	2020-10-12 15:10:50.586307
49294	IBDM	2020-10-08	25.02	25.02	25	25.02	143772	{}	\N	2020-10-12 15:10:50.590689
49295	IBDM	2020-10-09	25.02	25.02	25	25.02	98485	{}	\N	2020-10-12 15:10:50.595211
49296	IBDM	2020-10-12	25.01	25.02	25.01	25.01	13408	{}	\N	2020-10-12 15:10:50.599295
49297	HST	2020-09-28	10.89	11.13	10.835	10.93	6283874	{}	\N	2020-10-12 15:10:51.824722
49298	HST	2020-09-29	10.3	10.79	10.3	10.73	13501788	{}	\N	2020-10-12 15:10:51.835044
49299	HST	2020-09-30	10.74	11.09	10.63	10.79	17452030	{}	\N	2020-10-12 15:10:51.841832
49300	HST	2020-10-01	10.88	11.16	10.78	11.14	20406149	{}	\N	2020-10-12 15:10:51.848108
49301	HST	2020-10-02	10.74	11.325	10.71	11.29	12210208	{}	\N	2020-10-12 15:10:51.852778
49302	HST	2020-10-05	11.39	11.47	11.16	11.38	11751597	{}	\N	2020-10-12 15:10:51.857754
49303	HST	2020-10-06	11.5	11.66	11.27	11.3	19122287	{}	\N	2020-10-12 15:10:51.861906
49304	HST	2020-10-07	11.38	11.665	11.365	11.61	19928849	{}	\N	2020-10-12 15:10:51.865973
49305	HST	2020-10-08	11.76	11.875	11.6	11.85	21090611	{}	\N	2020-10-12 15:10:51.870008
49306	HST	2020-10-09	11.94	11.98	11.52	11.58	15156694	{}	\N	2020-10-12 15:10:51.873943
49307	HST	2020-10-12	11.58	11.63	11.365	11.43	1727676	{}	\N	2020-10-12 15:10:51.877857
49308	HUD	2020-09-28	7.58	7.615	7.58	7.59	327171	{}	\N	2020-10-12 15:10:53.314683
49309	HUD	2020-09-29	7.59	7.6	7.56	7.57	1065368	{}	\N	2020-10-12 15:10:53.325027
49310	HUD	2020-09-30	7.59	7.61	7.57	7.6	1975035	{}	\N	2020-10-12 15:10:53.332033
49311	HUD	2020-10-01	7.61	7.64	7.6	7.63	1133777	{}	\N	2020-10-12 15:10:53.338143
49312	HUD	2020-10-02	7.55	7.63	7.55	7.62	1748365	{}	\N	2020-10-12 15:10:53.343187
49313	HUD	2020-10-05	7.64	7.65	7.6299	7.64	1973116	{}	\N	2020-10-12 15:10:53.348367
49314	HUD	2020-10-06	7.65	7.66	7.64	7.65	1775828	{}	\N	2020-10-12 15:10:53.352504
49315	HUD	2020-10-07	7.66	7.66	7.65	7.66	1525666	{}	\N	2020-10-12 15:10:53.356615
49316	HUD	2020-10-08	7.66	7.68	7.64	7.68	3390850	{}	\N	2020-10-12 15:10:53.360835
49317	HUD	2020-10-09	7.67	7.68	7.66	7.67	395320	{}	\N	2020-10-12 15:10:53.369078
49318	HUD	2020-10-12	7.68	7.69	7.65	7.68	323798	{}	\N	2020-10-12 15:10:53.373438
49319	HVBC	2020-09-30	13.4	13.4	13.4	13.4	170	{}	\N	2020-10-12 15:10:54.448323
49320	HVBC	2020-10-05	12.76	12.76	12.65	12.76	563	{}	\N	2020-10-12 15:10:54.460059
49321	HVBC	2020-10-06	13.4	13.5	13.2088	13.4361	2122	{}	\N	2020-10-12 15:10:54.468157
49322	HVBC	2020-10-07	13.16	13.16	13.16	13.16	211	{}	\N	2020-10-12 15:10:54.474636
49323	HVBC	2020-10-08	13.2453	13.2453	13.2453	13.2453	583	{}	\N	2020-10-12 15:10:54.480413
49324	HVBC	2020-10-12	12.9101	12.9101	12.9101	12.9101	110	{}	\N	2020-10-12 15:10:54.485334
49325	HYB	2020-09-28	8.24	8.24	8.15	8.19	78103	{}	\N	2020-10-12 15:10:55.426064
49326	HYB	2020-09-29	8.2	8.2	8.12	8.18	101293	{}	\N	2020-10-12 15:10:55.436668
49327	HYB	2020-09-30	8.21	8.27	8.18	8.18	158366	{}	\N	2020-10-12 15:10:55.443498
49328	HYB	2020-10-01	8.23	8.25	8.17	8.24	271724	{}	\N	2020-10-12 15:10:55.449788
49329	HYB	2020-10-02	8.24	8.24	8.15	8.22	134360	{}	\N	2020-10-12 15:10:55.454423
49330	HYB	2020-10-05	8.22	8.27	8.2	8.27	137994	{}	\N	2020-10-12 15:10:55.459003
49331	HYB	2020-10-06	8.29	8.3	8.26	8.27	73303	{}	\N	2020-10-12 15:10:55.463264
49332	HYB	2020-10-07	8.3	8.32	8.2701	8.32	84828	{}	\N	2020-10-12 15:10:55.467348
49333	HYB	2020-10-08	8.36	8.36	8.29	8.36	56426	{}	\N	2020-10-12 15:10:55.471199
49334	HYB	2020-10-09	8.36	8.38	8.35	8.37	67972	{}	\N	2020-10-12 15:10:55.475199
49335	HYB	2020-10-12	8.38	8.4	8.37	8.4	20038	{}	\N	2020-10-12 15:10:55.479118
49336	IAC	2020-09-28	117.41	119.57	116.62	117.93	461979	{}	\N	2020-10-12 15:10:56.503599
49337	IAC	2020-09-29	117.76	120.19	117.44	118.96	404141	{}	\N	2020-10-12 15:10:56.516623
49338	IAC	2020-09-30	118.36	121.82	117.27	119.78	782400	{}	\N	2020-10-12 15:10:56.523774
49339	IAC	2020-10-01	120.81	125.92	120.81	124.78	679127	{}	\N	2020-10-12 15:10:56.52987
49340	IAC	2020-10-02	121.32	124.73	121.25	123.48	651046	{}	\N	2020-10-12 15:10:56.535447
49341	IAC	2020-10-05	124.25	128	123.86	126.64	1025927	{}	\N	2020-10-12 15:10:56.540206
49342	IAC	2020-10-06	126.58	128.48	125.05	126.47	714782	{}	\N	2020-10-12 15:10:56.544844
49343	IAC	2020-10-07	128.18	128.98	126.7	128.06	354492	{}	\N	2020-10-12 15:10:56.549309
49344	IAC	2020-10-08	128.67	129.9	123.62	124.11	895208	{}	\N	2020-10-12 15:10:56.553548
49345	IAC	2020-10-09	125.27	126.59	123.81	125.09	767456	{}	\N	2020-10-12 15:10:56.557463
49346	IAC	2020-10-12	127.25	127.89	125.7253	126.34	83808	{}	\N	2020-10-12 15:10:56.561628
49347	IBCE	2020-09-28	25.04	25.0892	25.04	25.0892	283	{}	\N	2020-10-12 15:10:58.520759
49348	IBCE	2020-09-29	25.0875	25.0875	25.0875	25.0875	104	{}	\N	2020-10-12 15:10:58.530174
49349	IBCE	2020-09-30	25.0901	25.0901	25.0901	25.0901	96	{}	\N	2020-10-12 15:10:58.535838
49350	IBCE	2020-10-01	25.07	25.07	25.07	25.07	70	{}	\N	2020-10-12 15:10:58.540881
49351	IBCE	2020-10-02	25.07	25.07	25.07	25.07	5	{}	\N	2020-10-12 15:10:58.545963
49352	IBCE	2020-10-05	25.1	25.1	25.1	25.1	30	{}	\N	2020-10-12 15:10:58.550186
49353	IBCE	2020-10-06	25.1102	25.1102	25.0502	25.0502	830	{}	\N	2020-10-12 15:10:58.554456
49354	IBCE	2020-10-08	25.075	25.075	25.075	25.075	91	{}	\N	2020-10-12 15:10:58.558389
49355	IBCE	2020-10-09	25.1	25.1	25.09	25.09	404	{}	\N	2020-10-12 15:10:58.562434
49356	IBDO	2020-09-28	26.21	26.27	26.18	26.25	107278	{}	\N	2020-10-12 15:10:59.525708
49357	IBDO	2020-09-29	26.2587	26.265	26.2023	26.23	316176	{}	\N	2020-10-12 15:10:59.534718
49358	IBDO	2020-09-30	26.2	26.24	26.2	26.23	217833	{}	\N	2020-10-12 15:10:59.540747
49359	IBDO	2020-10-01	26.21	26.39	26.16	26.2	324712	{}	\N	2020-10-12 15:10:59.546139
49360	IBDO	2020-10-02	26.16	26.2	26.16	26.2	90228	{}	\N	2020-10-12 15:10:59.550838
49361	IBDO	2020-10-05	26.21	26.25	26.16	26.18	189413	{}	\N	2020-10-12 15:10:59.555217
49362	IBDO	2020-10-06	26.17	26.23	26.15	26.19	505203	{}	\N	2020-10-12 15:10:59.559271
49363	IBDO	2020-10-07	26.19	26.23	26.18	26.21	170282	{}	\N	2020-10-12 15:10:59.563118
49364	IBDO	2020-10-08	26.21	26.24	26.1811	26.22	123455	{}	\N	2020-10-12 15:10:59.566854
49365	IBDO	2020-10-09	26.22	26.2499	26.205	26.23	84399	{}	\N	2020-10-12 15:10:59.570816
49366	IBDO	2020-10-12	26.25	26.25	26.2101	26.24	48524	{}	\N	2020-10-12 15:10:59.574721
49367	ICAD	2020-09-28	9.34	9.47	8.69	8.74	207760	{}	\N	2020-10-12 15:11:00.995509
49368	ICAD	2020-09-29	8.73	9.35	8.66	8.83	181779	{}	\N	2020-10-12 15:11:01.005757
49369	ICAD	2020-09-30	8.83	9.15	8.65	8.81	95543	{}	\N	2020-10-12 15:11:01.012717
49370	ICAD	2020-10-01	8.81	8.9142	8.48	8.73	192203	{}	\N	2020-10-12 15:11:01.018649
49371	ICAD	2020-10-02	8.54	9.29	8.54	9.265	160704	{}	\N	2020-10-12 15:11:01.023489
49372	ICAD	2020-10-05	9.37	9.41	9.04	9.29	196888	{}	\N	2020-10-12 15:11:01.028518
49373	ICAD	2020-10-06	9.33	9.49	9.06	9.18	125426	{}	\N	2020-10-12 15:11:01.032841
49374	ICAD	2020-10-07	9.3	9.9301	9.2059	9.91	160776	{}	\N	2020-10-12 15:11:01.037858
49375	ICAD	2020-10-08	9.83	10.45	9.71	10.31	216761	{}	\N	2020-10-12 15:11:01.042535
49376	ICAD	2020-10-09	10.43	10.43	9.8	9.95	93323	{}	\N	2020-10-12 15:11:01.047751
49377	ICAD	2020-10-12	9.92	10.035	9.76	9.77	18693	{}	\N	2020-10-12 15:11:01.051985
49378	ICF	2020-09-28	101.45	102.89	101.45	102.35	102703	{}	\N	2020-10-12 15:11:02.060653
49379	ICF	2020-09-29	102.1255	102.47	100.96	101.69	36461	{}	\N	2020-10-12 15:11:02.066685
49380	ICF	2020-09-30	102.079	102.8376	100.97	102.02	66348	{}	\N	2020-10-12 15:11:02.072552
49381	ICF	2020-10-01	102.5	104.01	102.095	104	56136	{}	\N	2020-10-12 15:11:02.076876
49382	ICF	2020-10-02	102.31	106.3	102.28	105.92	67605	{}	\N	2020-10-12 15:11:02.081256
49383	ICF	2020-10-05	106.2	106.79	104.89	106.66	39946	{}	\N	2020-10-12 15:11:02.085852
49384	ICF	2020-10-06	106.71	107.264	105.47	105.88	42509	{}	\N	2020-10-12 15:11:02.09013
49385	ICF	2020-10-07	106.85	106.85	105.5	106.01	60296	{}	\N	2020-10-12 15:11:02.094056
49386	ICF	2020-10-08	106.22	108.18	106.22	107.81	70564	{}	\N	2020-10-12 15:11:02.098308
49387	ICF	2020-10-09	108.35	108.35	106.92	107.36	48287	{}	\N	2020-10-12 15:11:02.102576
49388	ICF	2020-10-12	107.17	107.51	106.77	107.38	14470	{}	\N	2020-10-12 15:11:02.106886
49389	KFY	2020-09-28	29.46	29.85	29.22	29.27	278664	{}	\N	2020-10-12 15:11:03.055963
49390	KFY	2020-09-29	29.35	29.45	28.77	29.17	513856	{}	\N	2020-10-12 15:11:03.065183
49391	KFY	2020-09-30	29.28	29.67	28.68	29	741475	{}	\N	2020-10-12 15:11:03.070318
49392	KFY	2020-10-01	29	29.25	28.63	28.99	592559	{}	\N	2020-10-12 15:11:03.075556
49393	KFY	2020-10-02	28.51	29.72	28.44	29.52	429546	{}	\N	2020-10-12 15:11:03.080198
49394	KFY	2020-10-05	29.75	30.37	29.75	30.26	511074	{}	\N	2020-10-12 15:11:03.084685
49395	KFY	2020-10-06	30.5	30.93	30.225	30.35	599563	{}	\N	2020-10-12 15:11:03.088394
49396	KFY	2020-10-07	30.62	31.66	30.6	31.57	915528	{}	\N	2020-10-12 15:11:03.092234
49397	KFY	2020-10-08	31.99	32.06	31.43	31.6	534974	{}	\N	2020-10-12 15:11:03.096043
49398	KFY	2020-10-09	31.87	32.04	31.195	31.23	476487	{}	\N	2020-10-12 15:11:03.099743
49399	KFY	2020-10-12	31.26	31.88	31.18	31.87	44631	{}	\N	2020-10-12 15:11:03.103282
49400	HTGC	2020-09-28	11.37	11.54	11.36	11.46	656863	{}	\N	2020-10-12 15:11:04.044846
49401	HTGC	2020-09-29	11.46	11.56	11.37	11.48	1029514	{}	\N	2020-10-12 15:11:04.054005
49402	HTGC	2020-09-30	11.5	11.7	11.5	11.57	1284523	{}	\N	2020-10-12 15:11:04.060027
49403	HTGC	2020-10-01	11.67	11.87	11.54	11.86	984823	{}	\N	2020-10-12 15:11:04.065694
49404	HTGC	2020-10-02	11.71	11.85	11.33	11.76	1546300	{}	\N	2020-10-12 15:11:04.070513
49405	HTGC	2020-10-05	11.85	11.955	11.83	11.91	1336806	{}	\N	2020-10-12 15:11:04.075501
49406	HTGC	2020-10-06	12	12	11.735	11.77	1257689	{}	\N	2020-10-12 15:11:04.079666
49407	HTGC	2020-10-07	11.89	11.925	11.75	11.8	920975	{}	\N	2020-10-12 15:11:04.083432
49408	HTGC	2020-10-08	11.83	12.06	11.8	12.05	1440292	{}	\N	2020-10-12 15:11:04.087066
49409	HTGC	2020-10-09	12.05	12.1111	11.97	12.04	1070340	{}	\N	2020-10-12 15:11:04.090906
49410	HTGC	2020-10-12	12.12	12.15	11.95	11.99	364693	{}	\N	2020-10-12 15:11:04.09482
49411	HURN	2020-09-28	39.48	40.19	39.48	39.76	63569	{}	\N	2020-10-12 15:11:06.331051
49412	HURN	2020-09-29	39.59	39.97	38.69	39.88	122149	{}	\N	2020-10-12 15:11:06.343405
49413	HURN	2020-09-30	39.92	40.57	39.08	39.33	129396	{}	\N	2020-10-12 15:11:06.35057
49414	HURN	2020-10-01	39.38	40.23	39.24	40.09	97606	{}	\N	2020-10-12 15:11:06.356531
49415	HURN	2020-10-02	39.38	40.905	39.38	40.51	82211	{}	\N	2020-10-12 15:11:06.364443
49416	HURN	2020-10-05	40.72	41.29	40.65	41	39461	{}	\N	2020-10-12 15:11:06.370018
49417	HURN	2020-10-06	41.34	41.9584	39.62	41.1	84048	{}	\N	2020-10-12 15:11:06.374963
49418	HURN	2020-10-07	41.39	42.05	40.28	41.8	99971	{}	\N	2020-10-12 15:11:06.379211
49419	HURN	2020-10-08	42.26	42.76	41.34	42.36	58608	{}	\N	2020-10-12 15:11:06.383476
49420	HURN	2020-10-09	42.72	42.85	41.35	42.04	51615	{}	\N	2020-10-12 15:11:06.387721
49421	HURN	2020-10-12	42.22	42.54	41.74	42.33	5441	{}	\N	2020-10-12 15:11:06.391915
49422	HXL	2020-09-28	34.28	35.15	34.205	34.35	775888	{}	\N	2020-10-12 15:11:07.448814
49423	HXL	2020-09-29	34.03	34.28	32.8	33.5	1762697	{}	\N	2020-10-12 15:11:07.461867
49424	HXL	2020-09-30	33.87	34.61	33.03	33.55	1732359	{}	\N	2020-10-12 15:11:07.468537
49425	HXL	2020-10-01	33.98	35.74	33.95	35.3	2808896	{}	\N	2020-10-12 15:11:07.473916
49426	HXL	2020-10-02	34.18	37.445	34.18	37.15	1964483	{}	\N	2020-10-12 15:11:07.478183
49427	HXL	2020-10-05	38.08	38.08	36.82	37.46	2191647	{}	\N	2020-10-12 15:11:07.482765
49428	HXL	2020-10-06	37.99	38.16	36.63	36.91	2268871	{}	\N	2020-10-12 15:11:07.487757
49429	HXL	2020-10-07	37.6	38.01	37.53	37.63	1560337	{}	\N	2020-10-12 15:11:07.491615
49430	HXL	2020-10-08	38.18	38.24	37.1	38.19	1841374	{}	\N	2020-10-12 15:11:07.495596
49431	HXL	2020-10-09	37.2	37.51	36.3	37.02	2056480	{}	\N	2020-10-12 15:11:07.499456
49432	HXL	2020-10-12	37.23	37.5	36.61	37.08	433103	{}	\N	2020-10-12 15:11:07.503176
49433	HYLB	2020-09-28	47.8	47.95	47.745	47.86	1371223	{}	\N	2020-10-12 15:11:09.954859
49434	HYLB	2020-09-29	47.91	47.96	47.76	47.915	1407113	{}	\N	2020-10-12 15:11:09.96523
49435	HYLB	2020-09-30	47.93	48.19	47.925	48.1	1424979	{}	\N	2020-10-12 15:11:09.971422
49436	HYLB	2020-10-01	48.05	48.13	47.99	48.06	9493649	{}	\N	2020-10-12 15:11:09.97691
49437	HYLB	2020-10-02	47.82	48.07	47.82	48.01	2724806	{}	\N	2020-10-12 15:11:09.981737
49438	HYLB	2020-10-05	48.1	48.37	48.1	48.32	5124614	{}	\N	2020-10-12 15:11:09.986134
49439	HYLB	2020-10-06	48.42	48.58	48.22	48.22	4741337	{}	\N	2020-10-12 15:11:09.99015
49440	HYLB	2020-10-07	48.43	48.46	48.38	48.43	1547795	{}	\N	2020-10-12 15:11:09.993913
49441	HYLB	2020-10-08	48.58	48.59	48.48	48.59	2175847	{}	\N	2020-10-12 15:11:09.997786
49442	HYLB	2020-10-09	48.63	48.69	48.54	48.65	2372957	{}	\N	2020-10-12 15:11:10.001367
49443	HYLB	2020-10-12	48.78	48.875	48.76	48.875	290454	{}	\N	2020-10-12 15:11:10.005167
49444	HZO	2020-09-28	25.55	26.43	25.405	26.4	208881	{}	\N	2020-10-12 15:11:10.954476
49445	HZO	2020-09-29	26.38	26.4	25.45	25.55	417360	{}	\N	2020-10-12 15:11:10.96788
49446	HZO	2020-09-30	25.61	26.34	25.37	25.67	499038	{}	\N	2020-10-12 15:11:10.975577
49447	HZO	2020-10-01	26.27	27.45	26.155	26.46	961002	{}	\N	2020-10-12 15:11:10.982611
49448	HZO	2020-10-02	25.81	26.55	25.55	26.06	692510	{}	\N	2020-10-12 15:11:10.987732
49449	HZO	2020-10-05	26.35	26.98	26.18	26.78	884226	{}	\N	2020-10-12 15:11:10.992659
49450	HZO	2020-10-06	26.99	27.805	26.27	26.66	928502	{}	\N	2020-10-12 15:11:10.997292
49451	HZO	2020-10-07	27	27.47	26.1	26.25	837669	{}	\N	2020-10-12 15:11:11.001586
49452	HZO	2020-10-08	26.58	26.625	25.63	25.86	507337	{}	\N	2020-10-12 15:11:11.005966
49453	HZO	2020-10-09	26.08	26.1	25.54	25.76	456473	{}	\N	2020-10-12 15:11:11.009991
49454	HZO	2020-10-12	26.1	26.32	25.75	25.91	73247	{}	\N	2020-10-12 15:11:11.014097
49455	IART	2020-09-28	46.7	47.735	46.27	46.76	495630	{}	\N	2020-10-12 15:11:12.467711
49456	IART	2020-09-29	48.35	48.51	46.1	46.42	588187	{}	\N	2020-10-12 15:11:12.47931
49457	IART	2020-09-30	45.6982	47.83	45.4	47.22	609528	{}	\N	2020-10-12 15:11:12.48652
49458	IART	2020-10-01	47.46	47.96	46.83	47.27	508953	{}	\N	2020-10-12 15:11:12.492107
49459	IART	2020-10-02	46.41	47.45	45.7848	47.23	229813	{}	\N	2020-10-12 15:11:12.496718
49460	IART	2020-10-05	47.78	48.52	47.51	48.47	292186	{}	\N	2020-10-12 15:11:12.50134
49461	IART	2020-10-06	48.79	49.49	48.47	48.87	338657	{}	\N	2020-10-12 15:11:12.505328
49462	IART	2020-10-07	50.47	52.14	50.47	51.88	846579	{}	\N	2020-10-12 15:11:12.509946
49463	IART	2020-10-08	50.41	51.11	50.22	50.54	658710	{}	\N	2020-10-12 15:11:12.513976
49464	IART	2020-10-09	50.62	51.45	49.32	49.8	511003	{}	\N	2020-10-12 15:11:12.517879
49465	IART	2020-10-12	50.03	50.1	49.28	50.011	136195	{}	\N	2020-10-12 15:11:12.521692
49466	IBDR	2020-09-28	26.74	26.82	26.72	26.77	34772	{}	\N	2020-10-12 15:11:15.012526
49467	IBDR	2020-09-29	26.73	26.84	26.72	26.8	242957	{}	\N	2020-10-12 15:11:15.022558
49468	IBDR	2020-09-30	26.73	26.83	26.73	26.82	91941	{}	\N	2020-10-12 15:11:15.029088
49469	IBDR	2020-10-01	26.7	26.8048	26.6992	26.77	79082	{}	\N	2020-10-12 15:11:15.034189
49470	IBDR	2020-10-02	26.79	26.82	26.75	26.81	114204	{}	\N	2020-10-12 15:11:15.039774
49471	IBDR	2020-10-05	26.83	26.83	26.73	26.74	82963	{}	\N	2020-10-12 15:11:15.051156
49472	IBDR	2020-10-06	26.8	26.82	26.72	26.74	104961	{}	\N	2020-10-12 15:11:15.058844
49473	IBDR	2020-10-07	26.76	26.8499	26.7421	26.77	52390	{}	\N	2020-10-12 15:11:15.065723
49474	IBDR	2020-10-08	26.795	26.84	26.75	26.84	31161	{}	\N	2020-10-12 15:11:15.071449
49475	IBDR	2020-10-09	26.8185	26.84	26.79	26.84	31638	{}	\N	2020-10-12 15:11:15.076867
49476	IBDR	2020-10-12	26.85	26.91	26.8101	26.86	15794	{}	\N	2020-10-12 15:11:15.081038
49477	HUBB	2020-09-28	136.07	139.31	136.07	136.96	228397	{}	\N	2020-10-12 15:11:17.253654
49478	HUBB	2020-09-29	137.31	139.05	135.92	136.79	521546	{}	\N	2020-10-12 15:11:17.263058
49479	HUBB	2020-09-30	136.99	138.91	135.97	136.84	652979	{}	\N	2020-10-12 15:11:17.269604
49480	HUBB	2020-10-01	138.23	139.23	135.38	136.17	324311	{}	\N	2020-10-12 15:11:17.275617
49481	HUBB	2020-10-02	133.51	139.59	133.51	137.51	460036	{}	\N	2020-10-12 15:11:17.280863
49482	HUBB	2020-10-05	138.77	140.91	138.77	140.23	449321	{}	\N	2020-10-12 15:11:17.28561
49483	HUBB	2020-10-06	140.5	141.75	138.8	139.23	614184	{}	\N	2020-10-12 15:11:17.289946
49484	HUBB	2020-10-07	141.29	142.565	140.135	141.25	811110	{}	\N	2020-10-12 15:11:17.294076
49485	HUBB	2020-10-08	142.55	143.82	141.675	143.1	635175	{}	\N	2020-10-12 15:11:17.297869
49486	HUBB	2020-10-09	143.72	145.25	143.51	144.08	611831	{}	\N	2020-10-12 15:11:17.301606
49487	HUBB	2020-10-12	145	145.81	144.33	145.33	88641	{}	\N	2020-10-12 15:11:17.305232
49488	HWBK	2020-09-28	17.91	18.62	17.88	18.31	5288	{}	\N	2020-10-12 15:11:18.846375
49489	HWBK	2020-09-29	18.46	18.81	18.106	18.8	2059	{}	\N	2020-10-12 15:11:18.852909
49490	HWBK	2020-09-30	18.7	18.94	18.35	18.94	2082	{}	\N	2020-10-12 15:11:18.858715
49491	HWBK	2020-10-01	18.96	19.76	18.7	19	18170	{}	\N	2020-10-12 15:11:18.863049
49492	HWBK	2020-10-02	18.9	19.09	18.75	18.89	2378	{}	\N	2020-10-12 15:11:18.867693
49493	HWBK	2020-10-05	19.03	19.485	19.02	19.12	4780	{}	\N	2020-10-12 15:11:18.871749
49494	HWBK	2020-10-06	18.69	18.75	18.41	18.43	3228	{}	\N	2020-10-12 15:11:18.875529
49495	HWBK	2020-10-07	19.1314	19.45	18.44	19.44	5784	{}	\N	2020-10-12 15:11:18.879247
49496	HWBK	2020-10-08	19.42	19.4599	18.64	18.64	6229	{}	\N	2020-10-12 15:11:18.882957
49497	HWBK	2020-10-09	18.93	19	18.4323	18.76	4455	{}	\N	2020-10-12 15:11:18.886876
49498	HYACW	2020-09-28	1	1.1147	1	1.1147	4414	{}	\N	2020-10-12 15:11:20.16124
49499	HYACW	2020-09-29	1.05	1.1199	0.95	1.08	8072	{}	\N	2020-10-12 15:11:20.171731
49500	HYACW	2020-09-30	1.12	1.12	0.99	0.99	106387	{}	\N	2020-10-12 15:11:20.17794
49501	HYACW	2020-10-01	1	1	0.8786	0.88	104015	{}	\N	2020-10-12 15:11:20.183412
49502	HYACW	2020-10-02	0.8	0.9	0.8	0.8778	24340	{}	\N	2020-10-12 15:11:20.188123
49503	HYACW	2020-10-05	0.88	0.89	0.81	0.843999	44975	{}	\N	2020-10-12 15:11:20.192894
49504	HYACW	2020-10-06	0.8	0.9794	0.7	0.93	25509	{}	\N	2020-10-12 15:11:20.197605
49505	HYACW	2020-10-07	0.84	1.026	0.84	1.026	7510	{}	\N	2020-10-12 15:11:20.201379
49506	HYACW	2020-10-08	0.96	1.0291	0.91	0.97	5995	{}	\N	2020-10-12 15:11:20.205112
49507	HYACW	2020-10-09	1.1	1.1	0.98	1.02	63499	{}	\N	2020-10-12 15:11:20.209346
49508	HYACW	2020-10-12	1.02	1.12	0.97	0.97	7652	{}	\N	2020-10-12 15:11:20.21307
49509	HYMB	2020-09-28	56.86	56.99	56.78	56.87	184245	{}	\N	2020-10-12 15:11:22.124884
49510	HYMB	2020-09-29	56.9371	56.98	56.89	56.96	86807	{}	\N	2020-10-12 15:11:22.133764
49511	HYMB	2020-09-30	56.92	56.9862	56.81	56.86	124407	{}	\N	2020-10-12 15:11:22.140054
49512	HYMB	2020-10-01	56.8	56.873	56.65	56.82	143656	{}	\N	2020-10-12 15:11:22.147051
49513	HYMB	2020-10-02	56.82	56.82	56.7201	56.8	88430	{}	\N	2020-10-12 15:11:22.152489
49514	HYMB	2020-10-05	56.76	56.8	56.68	56.75	98048	{}	\N	2020-10-12 15:11:22.157121
49515	HYMB	2020-10-06	56.75	56.7893	56.6	56.71	130174	{}	\N	2020-10-12 15:11:22.161854
49516	HYMB	2020-10-07	56.7	56.7	56.545	56.62	111084	{}	\N	2020-10-12 15:11:22.166842
49517	HYMB	2020-10-08	56.67	56.67	56.58	56.59	125647	{}	\N	2020-10-12 15:11:22.17212
49518	HYMB	2020-10-09	56.59	56.69	56.58	56.58	111162	{}	\N	2020-10-12 15:11:22.176666
49519	HYMB	2020-10-12	56.63	56.72	56.63	56.6583	18787	{}	\N	2020-10-12 15:11:22.180913
49520	HYXU	2020-09-28	51.81	52.0151	51.81	52.0151	2612	{}	\N	2020-10-12 15:11:23.913181
49521	HYXU	2020-09-29	52.18	52.3199	52.1367	52.3199	1186	{}	\N	2020-10-12 15:11:23.923218
49522	HYXU	2020-09-30	52.22	52.31	52.075	52.075	2609	{}	\N	2020-10-12 15:11:23.930633
49523	HYXU	2020-10-01	52.32	52.83	52.32	52.63	70559	{}	\N	2020-10-12 15:11:23.936975
49524	HYXU	2020-10-02	52.43	52.62	52.2706	52.52	3005	{}	\N	2020-10-12 15:11:23.942062
49525	HYXU	2020-10-05	52.61	53.06	52.61	52.82	10784	{}	\N	2020-10-12 15:11:23.947088
49526	HYXU	2020-10-06	53.3	53.3	52.8036	52.8036	1029	{}	\N	2020-10-12 15:11:23.951431
49527	HYXU	2020-10-07	52.95	53.0716	52.93	53.005	2395	{}	\N	2020-10-12 15:11:23.955694
49528	HYXU	2020-10-08	53.065	53.1	52.9506	53.03	1666	{}	\N	2020-10-12 15:11:23.959943
49529	HYXU	2020-10-09	53.378	53.378	53.21	53.23	9386	{}	\N	2020-10-12 15:11:23.964201
49530	HYXU	2020-10-12	53.38	53.38	53.38	53.38	396	{}	\N	2020-10-12 15:11:23.969012
49531	IAGG	2020-09-28	55.71	55.78	55.69	55.77	241890	{}	\N	2020-10-12 15:11:26.051149
49532	IAGG	2020-09-29	55.785	55.83	55.785	55.825	134552	{}	\N	2020-10-12 15:11:26.059016
49533	IAGG	2020-09-30	55.77	55.77	55.66	55.66	181190	{}	\N	2020-10-12 15:11:26.064667
49534	IAGG	2020-10-01	55.56	55.71	55.56	55.69	183235	{}	\N	2020-10-12 15:11:26.070103
49535	IAGG	2020-10-02	55.72	55.74	55.68	55.68	167985	{}	\N	2020-10-12 15:11:26.074935
49536	IAGG	2020-10-05	55.66	55.671	55.6	55.6	118559	{}	\N	2020-10-12 15:11:26.079662
49537	IAGG	2020-10-06	55.66	55.73	55.64	55.67	267678	{}	\N	2020-10-12 15:11:26.083833
49538	IAGG	2020-10-07	55.6879	55.69	55.61	55.63	94400	{}	\N	2020-10-12 15:11:26.088094
49539	IAGG	2020-10-08	55.68	55.74	55.67	55.705	243332	{}	\N	2020-10-12 15:11:26.092173
49540	IAGG	2020-10-09	55.73	55.79	55.71	55.79	74193	{}	\N	2020-10-12 15:11:26.096499
49541	IAGG	2020-10-12	55.81	55.81	55.72	55.723	20369	{}	\N	2020-10-12 15:11:26.100634
49542	IBDN	2020-09-28	25.61	25.64	25.61	25.63	279627	{}	\N	2020-10-12 15:11:27.59822
49543	IBDN	2020-09-29	25.62	25.67	25.62	25.635	584436	{}	\N	2020-10-12 15:11:27.607482
49544	IBDN	2020-09-30	25.62	25.64	25.62	25.62	167356	{}	\N	2020-10-12 15:11:27.613613
49545	IBDN	2020-10-01	25.6	25.6	25.57	25.57	179842	{}	\N	2020-10-12 15:11:27.618843
49546	IBDN	2020-10-02	25.57	25.6	25.57	25.595	104122	{}	\N	2020-10-12 15:11:27.623681
49547	IBDN	2020-10-05	25.6	25.64	25.59	25.59	141750	{}	\N	2020-10-12 15:11:27.628327
49548	IBDN	2020-10-06	25.61	25.61	25.59	25.6	527781	{}	\N	2020-10-12 15:11:27.632766
49549	IBDN	2020-10-07	25.6	25.61	25.59	25.605	183449	{}	\N	2020-10-12 15:11:27.636754
49550	IBDN	2020-10-08	25.6	25.61	25.6	25.61	122442	{}	\N	2020-10-12 15:11:27.640361
49551	IBDN	2020-10-09	25.61	25.62	25.6	25.62	74267	{}	\N	2020-10-12 15:11:27.644045
49552	IBDN	2020-10-12	25.63	25.63	25.6116	25.62	27776	{}	\N	2020-10-12 15:11:27.647936
49553	IBM	2020-09-28	120.57	122.33	120.41	121.73	3433929	{}	\N	2020-10-12 15:11:28.730705
49554	IBM	2020-09-29	121.41	122.1858	120.21	120.94	4078929	{}	\N	2020-10-12 15:11:28.740678
49555	IBM	2020-09-30	121.38	122.91	120.8	121.67	6369821	{}	\N	2020-10-12 15:11:28.74856
49556	IBM	2020-10-01	122.36	123.3	120.355	121.09	6255588	{}	\N	2020-10-12 15:11:28.754146
49557	IBM	2020-10-02	119.04	121.75	118.82	120.57	5621439	{}	\N	2020-10-12 15:11:28.759186
49558	IBM	2020-10-05	121.84	122.75	121.05	122.01	6164984	{}	\N	2020-10-12 15:11:28.763629
49559	IBM	2020-10-06	122.58	124.83	121.59	121.97	7477034	{}	\N	2020-10-12 15:11:28.76799
49560	IBM	2020-10-07	122.67	124.39	122.32	124.07	5510469	{}	\N	2020-10-12 15:11:28.772354
49561	IBM	2020-10-08	130.86	135.5	129.77	131.49	51097617	{}	\N	2020-10-12 15:11:28.776705
49562	IBM	2020-10-09	132	132	127.6	127.79	16516557	{}	\N	2020-10-12 15:11:28.781217
49563	IBM	2020-10-12	128.07	128.25	126.44	126.98	3181481	{}	\N	2020-10-12 15:11:28.785477
49564	IBTX	2020-09-28	43.27	44.53	43.27	44	220414	{}	\N	2020-10-12 15:11:30.301877
49565	IBTX	2020-09-29	43.94	43.94	42.61	43.51	149352	{}	\N	2020-10-12 15:11:30.312634
49566	IBTX	2020-09-30	43.84	44.87	43.51	44.18	168921	{}	\N	2020-10-12 15:11:30.319527
49567	IBTX	2020-10-01	44.18	44.78	43.77	44.29	134811	{}	\N	2020-10-12 15:11:30.32556
49568	IBTX	2020-10-02	43.17	45.98	43.17	45.77	131328	{}	\N	2020-10-12 15:11:30.330304
49569	IBTX	2020-10-05	46.5	48.72	44.3768	48.53	244251	{}	\N	2020-10-12 15:11:30.335361
49570	IBTX	2020-10-06	47.3	48.89	46.78	46.97	310194	{}	\N	2020-10-12 15:11:30.339876
49571	IBTX	2020-10-07	48.05	48.76	47.04	47.56	310689	{}	\N	2020-10-12 15:11:30.344139
49572	IBTX	2020-10-08	48	48.74	47.26	48.34	129974	{}	\N	2020-10-12 15:11:30.348201
49573	IBTX	2020-10-09	48.72	48.72	47.08	47.49	108567	{}	\N	2020-10-12 15:11:30.352792
49574	IBTX	2020-10-12	47.52	48.23	46.705	48.09	19559	{}	\N	2020-10-12 15:11:30.356786
49575	ICLK	2020-09-28	6.57	6.7	6.52	6.6	995587	{}	\N	2020-10-12 15:11:31.405104
49576	ICLK	2020-09-29	6.59	6.89	6.53	6.74	1061147	{}	\N	2020-10-12 15:11:31.414889
49577	ICLK	2020-09-30	6.77	6.885	6.59	6.67	826569	{}	\N	2020-10-12 15:11:31.420794
49578	ICLK	2020-10-01	6.68	6.95	6.52	6.89	809329	{}	\N	2020-10-12 15:11:31.425688
49579	ICLK	2020-10-02	6.66	7.03	6.61	6.91	932574	{}	\N	2020-10-12 15:11:31.430441
49580	ICLK	2020-10-05	6.92	6.94	6.65	6.79	516246	{}	\N	2020-10-12 15:11:31.434764
49581	ICLK	2020-10-06	6.98	7.2	6.84	7.12	794379	{}	\N	2020-10-12 15:11:31.438951
49582	ICLK	2020-10-07	7.23	7.42	7.15	7.21	756960	{}	\N	2020-10-12 15:11:31.442599
49583	ICLK	2020-10-08	7.28	7.31	7.1573	7.28	545076	{}	\N	2020-10-12 15:11:31.446108
49584	ICLK	2020-10-09	7.275	7.67	7.275	7.59	1553807	{}	\N	2020-10-12 15:11:31.449666
49585	ICLK	2020-10-12	7.55	7.71	7.42	7.47	312957	{}	\N	2020-10-12 15:11:31.453234
49586	ICUI	2020-09-28	188.28	189.12	185.1	185.48	84584	{}	\N	2020-10-12 15:11:33.143627
49587	ICUI	2020-09-29	185.19	188.09	183.01	183.15	65033	{}	\N	2020-10-12 15:11:33.155114
49588	ICUI	2020-09-30	183.22	184.735	181.385	182.76	104853	{}	\N	2020-10-12 15:11:33.162598
49589	ICUI	2020-10-01	182.58	185.07	179.41	180.27	105529	{}	\N	2020-10-12 15:11:33.168599
49590	ICUI	2020-10-02	178.35	180.48	176.78	177.6	130921	{}	\N	2020-10-12 15:11:33.173074
49591	ICUI	2020-10-05	178.59	182.5899	178.59	181.45	101255	{}	\N	2020-10-12 15:11:33.178047
49592	ICUI	2020-10-06	181.35	184.41	180.39	182.15	86468	{}	\N	2020-10-12 15:11:33.182316
49593	ICUI	2020-10-07	182.63	187.63	182.63	187.2	64712	{}	\N	2020-10-12 15:11:33.186119
49594	ICUI	2020-10-08	188.34	194.34	187.59	194.03	139701	{}	\N	2020-10-12 15:11:33.189923
49595	ICUI	2020-10-09	194.14	195.77	191.26	191.85	76266	{}	\N	2020-10-12 15:11:33.193858
49596	ICUI	2020-10-12	193.25	194.23	192.4	194.23	5714	{}	\N	2020-10-12 15:11:33.19771
49597	HWKN	2020-09-28	45.72	47.16	45.65	46.54	24831	{}	\N	2020-10-12 15:11:35.102149
49598	HWKN	2020-09-29	46.51	47.98	46.23	46.59	23760	{}	\N	2020-10-12 15:11:35.1146
49599	HWKN	2020-09-30	46.6	47.4	45.47	46.1	39871	{}	\N	2020-10-12 15:11:35.121582
49600	HWKN	2020-10-01	46.29	46.43	45.32	45.99	33035	{}	\N	2020-10-12 15:11:35.127832
49601	HWKN	2020-10-02	45.96	47.11	45.92	46.51	25926	{}	\N	2020-10-12 15:11:35.132745
49602	HWKN	2020-10-05	46.74	47.47	46.555	47.47	16187	{}	\N	2020-10-12 15:11:35.137637
49603	HWKN	2020-10-06	47.92	48.67	46.29	46.93	41473	{}	\N	2020-10-12 15:11:35.141734
49604	HWKN	2020-10-07	47.03	47.29	46.01	46.29	48060	{}	\N	2020-10-12 15:11:35.145794
49605	HWKN	2020-10-08	46.46	46.59	45.64	46.2	53164	{}	\N	2020-10-12 15:11:35.149766
49606	HWKN	2020-10-09	46.57	48.47	45.73	48.01	38009	{}	\N	2020-10-12 15:11:35.153784
49607	HWKN	2020-10-12	48.35	49.04	47.185	47.535	5932	{}	\N	2020-10-12 15:11:35.157576
49608	HYACU	2020-09-28	10.49	10.49	10.49	10.49	150	{}	\N	2020-10-12 15:11:35.990186
49609	HYACU	2020-09-29	10.415	10.415	10.38	10.38	5305	{}	\N	2020-10-12 15:11:35.997484
49610	HYACU	2020-10-02	10.25	10.25	10.25	10.25	9301	{}	\N	2020-10-12 15:11:36.003021
49611	HYACU	2020-10-05	10.35	10.5	10.28	10.5	11600	{}	\N	2020-10-12 15:11:36.007525
49612	HYACU	2020-10-06	10.45	10.5	10.4	10.45	9700	{}	\N	2020-10-12 15:11:36.011986
49613	HYLD	2020-09-28	30.25	30.32	29.41	29.7	46250	{}	\N	2020-10-12 15:11:39.255189
49614	HYLD	2020-09-29	29.41	29.7	29.41	29.66	12979	{}	\N	2020-10-12 15:11:39.266276
49615	HYLD	2020-09-30	29.75	30.28	29.4726	30.145	22041	{}	\N	2020-10-12 15:11:39.273319
49616	HYLD	2020-10-01	30.0798	30.3095	30.01	30.155	8999	{}	\N	2020-10-12 15:11:39.278494
49617	HYLD	2020-10-02	29.9	30.2699	29.34	29.9425	47333	{}	\N	2020-10-12 15:11:39.283142
49618	HYLD	2020-10-05	29.85	30.25	29.85	30.02	16344	{}	\N	2020-10-12 15:11:39.287402
49619	HYLD	2020-10-06	29.92	30.105	29.88	30.05	8190	{}	\N	2020-10-12 15:11:39.291442
49620	HYLD	2020-10-07	29.95	30.15	29.95	30.13	15388	{}	\N	2020-10-12 15:11:39.295645
49621	HYLD	2020-10-08	30.01	30.152	30.01	30.13	7631	{}	\N	2020-10-12 15:11:39.299841
49622	HYLD	2020-10-09	30.135	30.23	30.13	30.23	14062	{}	\N	2020-10-12 15:11:39.303929
49623	HYLD	2020-10-12	30.45	30.49	30.4082	30.4082	5820	{}	\N	2020-10-12 15:11:39.307929
49624	HZN	2020-09-28	5.17	5.43	5.17	5.28	36288	{}	\N	2020-10-12 15:11:42.348661
49625	HZN	2020-09-29	5.3	5.54	5.3	5.46	68174	{}	\N	2020-10-12 15:11:42.360615
49626	HZN	2020-09-30	5.5	5.75	5.5	5.75	170959	{}	\N	2020-10-12 15:11:42.374601
49627	HZN	2020-10-01	5.75	5.87	5.75	5.83	80788	{}	\N	2020-10-12 15:11:42.38141
49628	HZN	2020-10-02	5.65	5.95	5.6	5.88	82128	{}	\N	2020-10-12 15:11:42.386604
49629	HZN	2020-10-05	5.9	6.07	5.745	5.97	90596	{}	\N	2020-10-12 15:11:42.390574
49630	HZN	2020-10-06	5.98	5.98	5.75	5.75	30298	{}	\N	2020-10-12 15:11:42.395079
49631	HZN	2020-10-07	5.86	5.92	5.81	5.83	23386	{}	\N	2020-10-12 15:11:42.399493
49632	HZN	2020-10-08	5.92	5.92	5.64	5.67	44348	{}	\N	2020-10-12 15:11:42.403673
49633	HZN	2020-10-09	5.66	5.83	5.66	5.74	73516	{}	\N	2020-10-12 15:11:42.407812
49634	HZN	2020-10-12	5.7	5.725	5.5801	5.725	4238	{}	\N	2020-10-12 15:11:42.411863
49635	IAK	2020-09-28	55.09	56.1018	55.09	55.68	8480	{}	\N	2020-10-12 15:11:43.623342
49636	IAK	2020-09-29	55.575	55.575	54.93	55	4063	{}	\N	2020-10-12 15:11:43.634399
49637	IAK	2020-09-30	55.4473	56.065	55.297	55.74	2588	{}	\N	2020-10-12 15:11:43.641557
49638	IAK	2020-10-01	55.35	55.9619	55.35	55.4928	26254	{}	\N	2020-10-12 15:11:43.64769
49639	IAK	2020-10-02	54.82	56.59	54.82	56.4401	1742	{}	\N	2020-10-12 15:11:43.652797
49640	IAK	2020-10-05	57.1607	57.71	57.1607	57.6002	2618	{}	\N	2020-10-12 15:11:43.657868
49641	IAK	2020-10-06	57.37	58.2233	56.96	56.96	4926	{}	\N	2020-10-12 15:11:43.66247
49642	IAK	2020-10-07	57.7658	58.11	57.425	57.885	1844	{}	\N	2020-10-12 15:11:43.66688
49643	IAK	2020-10-08	57.79	58.73	57.79	58.55	2555	{}	\N	2020-10-12 15:11:43.672084
49644	IAK	2020-10-09	58.5	58.91	58.4901	58.58	366556	{}	\N	2020-10-12 15:11:43.676433
49645	IAK	2020-10-12	58.67	59.16	58.67	59.16	3575	{}	\N	2020-10-12 15:11:43.680938
49646	IBDL	2020-09-28	25.21	25.23	25.2	25.22	450004	{}	\N	2020-10-12 15:11:45.969797
49647	IBDL	2020-09-29	25.215	25.24	25.21	25.23	116760	{}	\N	2020-10-12 15:11:45.979719
49648	IBDL	2020-09-30	25.24	25.25	25.22	25.23	180038	{}	\N	2020-10-12 15:11:45.987142
49649	IBDL	2020-10-01	25.22	25.22	25.19	25.21	247192	{}	\N	2020-10-12 15:11:45.993266
49650	IBDL	2020-10-02	25.19	25.21	25.19	25.21	133996	{}	\N	2020-10-12 15:11:45.998718
49651	IBDL	2020-10-05	25.19	25.215	25.19	25.2	119765	{}	\N	2020-10-12 15:11:46.003601
49652	IBDL	2020-10-06	25.2	25.21	25.19	25.19	240853	{}	\N	2020-10-12 15:11:46.008223
49653	IBDL	2020-10-07	25.196	25.2	25.19	25.195	72034	{}	\N	2020-10-12 15:11:46.013152
49654	IBDL	2020-10-08	25.2	25.21	25.19	25.2	88397	{}	\N	2020-10-12 15:11:46.018029
49655	IBDL	2020-10-09	25.19	25.21	25.19	25.19	404083	{}	\N	2020-10-12 15:11:46.022628
49656	IBDL	2020-10-12	25.19	25.2241	25.19	25.22	109141	{}	\N	2020-10-12 15:11:46.027305
49657	IBIO	2020-09-28	2.07	2.08	1.97	2.08	9166561	{}	\N	2020-10-12 15:11:46.984573
49658	IBIO	2020-09-29	2.01	2.18	1.99	2.09	14890682	{}	\N	2020-10-12 15:11:46.994651
49659	IBIO	2020-09-30	2.05	2.12	2	2.03	6459964	{}	\N	2020-10-12 15:11:47.000891
49660	IBIO	2020-10-01	2.03	2.08	1.99	2.05	7640416	{}	\N	2020-10-12 15:11:47.007391
49661	IBIO	2020-10-02	2.15	2.18	2.06	2.06	14458474	{}	\N	2020-10-12 15:11:47.013121
49662	IBIO	2020-10-05	2.081	2.11	2.0411	2.1	6852141	{}	\N	2020-10-12 15:11:47.017577
49663	IBIO	2020-10-06	2.09	2.1189	2.02	2.02	6211671	{}	\N	2020-10-12 15:11:47.021518
49664	IBIO	2020-10-07	2.051	2.11	2.02	2.08	5607203	{}	\N	2020-10-12 15:11:47.025583
49665	IBIO	2020-10-08	2.09	2.1	2.05	2.06	4185423	{}	\N	2020-10-12 15:11:47.029581
49666	IBIO	2020-10-09	2.06	2.0799	2.03	2.05	5206451	{}	\N	2020-10-12 15:11:47.033373
49667	IBIO	2020-10-12	2.07	2.07	2.04	2.0601	1365801	{}	\N	2020-10-12 15:11:47.037573
49668	IBML	2020-09-28	26.29	26.3	26.26	26.28	15862	{}	\N	2020-10-12 15:11:48.642521
49669	IBML	2020-09-29	26.2726	26.31	26.2644	26.29	162214	{}	\N	2020-10-12 15:11:48.652163
49670	IBML	2020-09-30	26.28	26.299	26.2401	26.28	35167	{}	\N	2020-10-12 15:11:48.658249
49671	IBML	2020-10-01	26.26	26.27	26.2301	26.26	22704	{}	\N	2020-10-12 15:11:48.663669
49672	IBML	2020-10-02	26.256	26.256	26.2	26.2201	16148	{}	\N	2020-10-12 15:11:48.668899
49673	IBML	2020-10-05	26.225	26.26	26.21	26.21	17491	{}	\N	2020-10-12 15:11:48.673544
49674	IBML	2020-10-06	26.2282	26.25	26.2	26.2	52678	{}	\N	2020-10-12 15:11:48.678173
49675	IBML	2020-10-07	26.206	26.25	26.2	26.25	49091	{}	\N	2020-10-12 15:11:48.682332
49676	IBML	2020-10-08	26.2331	26.25	26.2201	26.24	10156	{}	\N	2020-10-12 15:11:48.686523
49677	IBML	2020-10-09	26.25	26.25	26.19	26.19	137313	{}	\N	2020-10-12 15:11:48.690755
49678	IBML	2020-10-12	26.2235	26.2235	26.21	26.215	1652	{}	\N	2020-10-12 15:11:48.69516
49679	ICHR	2020-09-28	21.2	22	21.16	21.7	372208	{}	\N	2020-10-12 15:11:51.359304
49680	ICHR	2020-09-29	21.7945	22.22	21.5497	21.7	374747	{}	\N	2020-10-12 15:11:51.37033
49681	ICHR	2020-09-30	21.74	22.2	21.47	21.57	250330	{}	\N	2020-10-12 15:11:51.376561
49682	ICHR	2020-10-01	21.7	22.91	21.7	22.61	228923	{}	\N	2020-10-12 15:11:51.382014
49683	ICHR	2020-10-02	21.83	22.57	21.7	22.27	219169	{}	\N	2020-10-12 15:11:51.386908
49684	ICHR	2020-10-05	22.4	22.88	22.31	22.81	174934	{}	\N	2020-10-12 15:11:51.391732
49685	ICHR	2020-10-06	22.95	23.69	22.69	22.95	176995	{}	\N	2020-10-12 15:11:51.396466
49686	ICHR	2020-10-07	23.12	23.52	23	23.12	171917	{}	\N	2020-10-12 15:11:51.401211
49687	ICHR	2020-10-08	23.36	24.67	23.26	24.42	380386	{}	\N	2020-10-12 15:11:51.40551
49688	ICHR	2020-10-09	24.69	26.33	24.5	26.18	499104	{}	\N	2020-10-12 15:11:51.409636
49689	ICHR	2020-10-12	26.19	26.47	25.89	26.1	52042	{}	\N	2020-10-12 15:11:51.413624
49690	ICVT	2020-09-28	81.08	81.08	80.4	81.01	145287	{}	\N	2020-10-12 15:11:52.310397
49691	ICVT	2020-09-29	80.65	81.01	80.43	80.79	97070	{}	\N	2020-10-12 15:11:52.32004
49692	ICVT	2020-09-30	80.75	81.5	80.64	80.95	150271	{}	\N	2020-10-12 15:11:52.325973
49693	ICVT	2020-10-01	81.63	83.0238	81.6	82.5058	198150	{}	\N	2020-10-12 15:11:52.331212
49694	ICVT	2020-10-02	80.92	82.32	80.74	81.35	80367	{}	\N	2020-10-12 15:11:52.335938
49695	ICVT	2020-10-05	82.21	82.66	82.02	82.64	109596	{}	\N	2020-10-12 15:11:52.340332
49696	ICVT	2020-10-06	82.32	83.52	81.6757	81.93	154305	{}	\N	2020-10-12 15:11:52.344652
49697	ICVT	2020-10-07	83.28	83.79	82.6801	83.3	240950	{}	\N	2020-10-12 15:11:52.348445
49698	ICVT	2020-10-08	84.065	84.13	83.18	83.54	70630	{}	\N	2020-10-12 15:11:52.352453
49699	ICVT	2020-10-09	84.05	84.36	83.82	84.35	70567	{}	\N	2020-10-12 15:11:52.356232
49700	ICVT	2020-10-12	85.16	85.16	84.565	84.94	26347	{}	\N	2020-10-12 15:11:52.36002
49701	IDN	2020-09-28	6.85	6.97	6.645	6.7	95642	{}	\N	2020-10-12 15:11:53.850083
49702	IDN	2020-09-29	6.77	6.92	6.74	6.85	46011	{}	\N	2020-10-12 15:11:53.858668
49703	IDN	2020-09-30	6.83	6.905	6.65	6.67	70156	{}	\N	2020-10-12 15:11:53.864579
49704	IDN	2020-10-01	6.66	6.96	6.66	6.94	97981	{}	\N	2020-10-12 15:11:53.870497
49705	IDN	2020-10-02	6.8	7.03	6.7959	6.86	75649	{}	\N	2020-10-12 15:11:53.876113
49706	IDN	2020-10-05	7.03	7.36	6.77	7.28	139985	{}	\N	2020-10-12 15:11:53.880706
49707	IDN	2020-10-06	7.29	7.3	7.025	7.11	51534	{}	\N	2020-10-12 15:11:53.885044
49708	IDN	2020-10-07	7.23	7.43	7.11	7.38	77818	{}	\N	2020-10-12 15:11:53.889457
49709	IDN	2020-10-08	7.43	7.43	7.26	7.3	31330	{}	\N	2020-10-12 15:11:53.893847
49710	IDN	2020-10-09	7.31	7.38	7.05	7.07	42372	{}	\N	2020-10-12 15:11:53.897967
49711	IDN	2020-10-12	7.16	7.17	7.03	7.0913	13083	{}	\N	2020-10-12 15:11:53.90211
49712	IAG	2020-09-28	3.84	3.88	3.72	3.79	2761800	{}	\N	2020-10-12 15:11:55.292816
49713	IAG	2020-09-29	3.825	3.9199	3.79	3.86	5634949	{}	\N	2020-10-12 15:11:55.303989
49714	IAG	2020-09-30	3.82	3.9181	3.775	3.83	7133753	{}	\N	2020-10-12 15:11:55.310786
49715	IAG	2020-10-01	3.89	3.94	3.84	3.9	7782859	{}	\N	2020-10-12 15:11:55.316916
49716	IAG	2020-10-02	3.89	3.92	3.805	3.89	7849468	{}	\N	2020-10-12 15:11:55.32251
49717	IAG	2020-10-05	3.93	3.995	3.88	3.93	8637575	{}	\N	2020-10-12 15:11:55.327845
49718	IAG	2020-10-06	3.94	4	3.72	3.72	8438795	{}	\N	2020-10-12 15:11:55.332057
49719	IAG	2020-10-07	3.77	3.8014	3.7	3.7	6512276	{}	\N	2020-10-12 15:11:55.336264
49720	IAG	2020-10-08	3.77	3.94	3.75	3.92	6588279	{}	\N	2020-10-12 15:11:55.340714
49721	IAG	2020-10-09	3.99	4.09	3.96	4.08	10717624	{}	\N	2020-10-12 15:11:55.344915
49722	IAG	2020-10-12	4.07	4.22	4.05	4.215	3196524	{}	\N	2020-10-12 15:11:55.34902
49723	IBCP	2020-09-28	12.51	13.07	12.51	12.94	88245	{}	\N	2020-10-12 15:11:56.515185
49724	IBCP	2020-09-29	12.78	13.05	12.415	12.76	85179	{}	\N	2020-10-12 15:11:56.524931
49725	IBCP	2020-09-30	12.81	13	12.55	12.57	64352	{}	\N	2020-10-12 15:11:56.531915
49726	IBCP	2020-10-01	12.68	12.79	12.43	12.71	68958	{}	\N	2020-10-12 15:11:56.538145
49727	IBCP	2020-10-02	12.43	13.11	12.42	13.08	37623	{}	\N	2020-10-12 15:11:56.54292
49728	IBCP	2020-10-05	13.21	13.54	13.105	13.49	66993	{}	\N	2020-10-12 15:11:56.547386
49729	IBCP	2020-10-06	13.5	14.4	13.41	13.71	110865	{}	\N	2020-10-12 15:11:56.551484
49730	IBCP	2020-10-07	13.81	14.06	13.66	14.02	69525	{}	\N	2020-10-12 15:11:56.555414
49731	IBCP	2020-10-08	14.17	14.58	14	14.16	67134	{}	\N	2020-10-12 15:11:56.559284
49732	IBCP	2020-10-09	14.26	14.26	13.93	14.18	49286	{}	\N	2020-10-12 15:11:56.563805
49733	IBCP	2020-10-12	14.14	14.36	14.04	14.28	5090	{}	\N	2020-10-12 15:11:56.568772
49734	IBDQ	2020-09-28	27.06	27.14	27.04	27.11	74771	{}	\N	2020-10-12 15:11:57.49358
49735	IBDQ	2020-09-29	27.1	27.16	27.05	27.14	220221	{}	\N	2020-10-12 15:11:57.503176
49736	IBDQ	2020-09-30	27.15	27.15	27.09	27.13	69148	{}	\N	2020-10-12 15:11:57.509852
49737	IBDQ	2020-10-01	27.13	27.13	27.05	27.11	113037	{}	\N	2020-10-12 15:11:57.515456
49738	IBDQ	2020-10-02	27.1	27.12	27.07	27.11	89510	{}	\N	2020-10-12 15:11:57.520245
49739	IBDQ	2020-10-05	27.12	27.14	27.07	27.08	134092	{}	\N	2020-10-12 15:11:57.524991
49740	IBDQ	2020-10-06	27.1	27.12	27.08	27.12	224805	{}	\N	2020-10-12 15:11:57.528962
49741	IBDQ	2020-10-07	27.12	27.14	27.09	27.11	141723	{}	\N	2020-10-12 15:11:57.532598
49742	IBDQ	2020-10-08	27.09	27.14	27.09	27.13	100385	{}	\N	2020-10-12 15:11:57.536139
49743	IBDQ	2020-10-09	27.07	27.1497	27.07	27.1	56336	{}	\N	2020-10-12 15:11:57.539728
49744	IBDQ	2020-10-12	27.15	27.21	27.106	27.19	22968	{}	\N	2020-10-12 15:11:57.54366
49745	IBMK	2020-09-28	26.48	26.49	26.4528	26.49	17269	{}	\N	2020-10-12 15:11:58.5301
49746	IBMK	2020-09-29	26.49	26.51	26.46	26.4766	68622	{}	\N	2020-10-12 15:11:58.536147
49747	IBMK	2020-09-30	26.48	26.51	26.453	26.5	63355	{}	\N	2020-10-12 15:11:58.540719
49748	IBMK	2020-10-01	26.46	26.47	26.4296	26.45	52801	{}	\N	2020-10-12 15:11:58.544994
49749	IBMK	2020-10-02	26.45	26.46	26.41	26.45	28869	{}	\N	2020-10-12 15:11:58.548965
49750	IBMK	2020-10-05	26.45	26.45	26.39	26.43	37484	{}	\N	2020-10-12 15:11:58.552744
49751	IBMK	2020-10-06	26.45	26.45	26.39	26.39	37182	{}	\N	2020-10-12 15:11:58.556499
49752	IBMK	2020-10-07	26.45	26.45	26.4	26.45	46051	{}	\N	2020-10-12 15:11:58.560513
49753	IBMK	2020-10-08	26.435	26.45	26.422	26.44	16293	{}	\N	2020-10-12 15:11:58.564563
49754	IBMK	2020-10-09	26.44	26.45	26.4	26.43	109885	{}	\N	2020-10-12 15:11:58.56807
49755	IBMK	2020-10-12	26.39	26.4249	26.38	26.38	5823	{}	\N	2020-10-12 15:11:58.571471
49756	ICL	2020-09-28	3.5	3.6	3.5	3.56	258280	{}	\N	2020-10-12 15:12:00.310932
49757	ICL	2020-09-29	3.52	3.52	3.41	3.44	291196	{}	\N	2020-10-12 15:12:00.322689
49758	ICL	2020-09-30	3.48	3.54	3.48	3.49	722438	{}	\N	2020-10-12 15:12:00.329843
49759	ICL	2020-10-01	3.77	3.805	3.71	3.78	499199	{}	\N	2020-10-12 15:12:00.33584
49760	ICL	2020-10-02	3.68	3.78	3.68	3.7	508989	{}	\N	2020-10-12 15:12:00.340823
49761	ICL	2020-10-05	3.79	3.89	3.79	3.85	201290	{}	\N	2020-10-12 15:12:00.345564
49762	ICL	2020-10-06	3.8	3.9	3.8	3.83	219457	{}	\N	2020-10-12 15:12:00.349785
49763	ICL	2020-10-07	3.82	3.9	3.81	3.84	167039	{}	\N	2020-10-12 15:12:00.353598
49764	ICL	2020-10-08	3.82	3.89	3.82	3.87	149115	{}	\N	2020-10-12 15:12:00.35757
49765	ICL	2020-10-09	3.87	3.95	3.85	3.94	278883	{}	\N	2020-10-12 15:12:00.364106
49766	ICL	2020-10-12	3.78	3.81	3.76	3.76	142790	{}	\N	2020-10-12 15:12:00.368083
49767	IDCC	2020-09-28	55.65	57.3	55.65	57.12	166204	{}	\N	2020-10-12 15:12:02.020691
49768	IDCC	2020-09-29	57.15	57.885	56.96	57.3	88555	{}	\N	2020-10-12 15:12:02.026961
49769	IDCC	2020-09-30	57.9	57.9	56.6	57.06	152999	{}	\N	2020-10-12 15:12:02.031411
49770	IDCC	2020-10-01	57.3	57.83	56.86	57.385	155364	{}	\N	2020-10-12 15:12:02.036226
49771	IDCC	2020-10-02	56.55	57.06	56.3697	56.77	223491	{}	\N	2020-10-12 15:12:02.041659
49772	IDCC	2020-10-05	57.26	58.29	57.23	58.21	87479	{}	\N	2020-10-12 15:12:02.04592
49773	IDCC	2020-10-06	58.54	58.97	57.13	57.25	150788	{}	\N	2020-10-12 15:12:02.049927
49774	IDCC	2020-10-07	57.65	58.75	57.65	58.3	113535	{}	\N	2020-10-12 15:12:02.053946
49775	IDCC	2020-10-08	59.25	59.58	58.05	59.42	159589	{}	\N	2020-10-12 15:12:02.058124
49776	IDCC	2020-10-09	59.76	60.68	59.335	60.66	131113	{}	\N	2020-10-12 15:12:02.06245
49777	IDCC	2020-10-12	61.01	61.07	60.25	60.765	35754	{}	\N	2020-10-12 15:12:02.066455
49778	IDRA	2020-09-28	2.38	2.39	2.25	2.34	157713	{}	\N	2020-10-12 15:12:03.730499
49779	IDRA	2020-09-29	2.34	2.34	2.2	2.22	51610	{}	\N	2020-10-12 15:12:03.739858
49780	IDRA	2020-09-30	2.22	2.22	2.11	2.16	90050	{}	\N	2020-10-12 15:12:03.745489
49781	IDRA	2020-10-01	2.17	2.25	2.1	2.25	91899	{}	\N	2020-10-12 15:12:03.751562
49782	IDRA	2020-10-02	2.22	2.33	2.1101	2.29	124514	{}	\N	2020-10-12 15:12:03.756094
49783	IDRA	2020-10-05	2.25	2.36	2.21	2.21	107864	{}	\N	2020-10-12 15:12:03.760959
49784	IDRA	2020-10-06	2.32	2.33	2.21	2.21	66052	{}	\N	2020-10-12 15:12:03.764809
49785	IDRA	2020-10-07	2.24	2.29	2.2	2.23	80300	{}	\N	2020-10-12 15:12:03.768679
49786	IDRA	2020-10-08	2.26	2.28	2.2	2.255	101170	{}	\N	2020-10-12 15:12:03.772552
49787	IDRA	2020-10-09	2.27	2.42	2.25	2.36	154088	{}	\N	2020-10-12 15:12:03.776583
49788	IDRA	2020-10-12	2.38	2.4	2.3	2.38	37506	{}	\N	2020-10-12 15:12:03.780557
49789	IEC	2020-09-28	8.4726	8.9	8.4726	8.75	21436	{}	\N	2020-10-12 15:12:06.694292
49790	IEC	2020-09-29	8.73	8.88	8.61	8.76	7952	{}	\N	2020-10-12 15:12:06.704323
49791	IEC	2020-09-30	8.75	8.75	8.6	8.72	17092	{}	\N	2020-10-12 15:12:06.711595
49792	IEC	2020-10-01	8.66	8.75	8.6447	8.7	3100	{}	\N	2020-10-12 15:12:06.717346
49793	IEC	2020-10-02	8.31	8.78	8.31	8.76	29075	{}	\N	2020-10-12 15:12:06.722262
49794	IEC	2020-10-05	8.91	9	8.89	8.96	30981	{}	\N	2020-10-12 15:12:06.727214
49795	IEC	2020-10-06	8.89	9.05	8.8	8.81	26706	{}	\N	2020-10-12 15:12:06.731663
49796	IEC	2020-10-07	8.81	8.97	8.8	8.86	14878	{}	\N	2020-10-12 15:12:06.735624
49797	IEC	2020-10-08	8.87	9.07	8.87	9.07	26299	{}	\N	2020-10-12 15:12:06.739799
49798	IEC	2020-10-09	9.07	9.24	9.05	9.15	27487	{}	\N	2020-10-12 15:12:06.74495
49799	IEC	2020-10-12	9.15	9.15	9.05	9.05	6185	{}	\N	2020-10-12 15:12:06.748968
49800	IESC	2020-09-28	30.46	31.44	30.46	31.27	46329	{}	\N	2020-10-12 15:12:07.99905
49801	IESC	2020-09-29	31.25	31.81	30.905	31.53	37874	{}	\N	2020-10-12 15:12:08.011484
49802	IESC	2020-09-30	32.01	32.35	31.51	31.77	74679	{}	\N	2020-10-12 15:12:08.01893
49803	IESC	2020-10-01	32.33	32.76	31.53	32.65	55287	{}	\N	2020-10-12 15:12:08.025564
49804	IESC	2020-10-02	32.11	33.025	32.11	32.85	39440	{}	\N	2020-10-12 15:12:08.031077
49805	IESC	2020-10-05	33.05	34.8099	33.05	34.63	78643	{}	\N	2020-10-12 15:12:08.036217
49806	IESC	2020-10-06	35.38	37.8	35.3701	36.76	157676	{}	\N	2020-10-12 15:12:08.04172
49807	IESC	2020-10-07	37.5727	38.81	37.11	37.93	105350	{}	\N	2020-10-12 15:12:08.046556
49808	IESC	2020-10-08	38.73	39.45	37.49	37.58	61028	{}	\N	2020-10-12 15:12:08.051165
49809	IESC	2020-10-09	38.51	38.51	36	37.39	82081	{}	\N	2020-10-12 15:12:08.055728
49810	IESC	2020-10-12	37.79	37.79	37.0001	37.615	18425	{}	\N	2020-10-12 15:12:08.060271
49811	IGI	2020-09-28	20.9	20.925	20.71	20.86	6954	{}	\N	2020-10-12 15:12:09.993331
49812	IGI	2020-09-29	20.93	20.9633	20.87	20.905	57288	{}	\N	2020-10-12 15:12:10.003035
49813	IGI	2020-09-30	20.85	20.97	20.74	20.74	44282	{}	\N	2020-10-12 15:12:10.010004
49814	IGI	2020-10-01	20.9	20.9	20.4273	20.64	49834	{}	\N	2020-10-12 15:12:10.01604
49815	IGI	2020-10-02	20.52	20.75	20.29	20.425	79088	{}	\N	2020-10-12 15:12:10.020972
49816	IGI	2020-10-05	20.47	20.64	20.4	20.53	25662	{}	\N	2020-10-12 15:12:10.02622
49817	IGI	2020-10-06	20.57	20.6911	20.31	20.49	71876	{}	\N	2020-10-12 15:12:10.030608
49818	IGI	2020-10-07	20.64	20.7	20.55	20.56	59728	{}	\N	2020-10-12 15:12:10.034884
49819	IGI	2020-10-08	20.57	20.859	20.57	20.78	52788	{}	\N	2020-10-12 15:12:10.039537
49820	IGI	2020-10-09	20.86	20.86	20.69	20.815	64804	{}	\N	2020-10-12 15:12:10.044723
49821	IGI	2020-10-12	20.76	20.7906	20.74	20.74	8822	{}	\N	2020-10-12 15:12:10.049441
49822	IBKR	2020-09-28	48.02	48.74	48.02	48.37	428051	{}	\N	2020-10-12 15:12:11.413922
49823	IBKR	2020-09-29	48.5	48.5	47.99	48.1	518743	{}	\N	2020-10-12 15:12:11.425061
49824	IBKR	2020-09-30	48.31	48.95	47.871	48.33	517432	{}	\N	2020-10-12 15:12:11.431745
49825	IBKR	2020-10-01	48.49	48.71	48.25	48.58	471889	{}	\N	2020-10-12 15:12:11.43747
49826	IBKR	2020-10-02	47.93	49.555	47.93	48.93	705232	{}	\N	2020-10-12 15:12:11.442716
49827	IBKR	2020-10-05	49.16	49.69	49.07	49.6	398936	{}	\N	2020-10-12 15:12:11.447614
49828	IBKR	2020-10-06	49.5	51	49.5	50.07	480644	{}	\N	2020-10-12 15:12:11.45164
49829	IBKR	2020-10-07	50.56	51.1	50.15	50.61	292151	{}	\N	2020-10-12 15:12:11.455755
49830	IBKR	2020-10-08	51.08	51.52	50.6	51.43	383475	{}	\N	2020-10-12 15:12:11.459832
49831	IBKR	2020-10-09	51.295	51.73	50.58	50.66	306066	{}	\N	2020-10-12 15:12:11.463897
49832	IBKR	2020-10-12	51.24	52.07	51	51.985	126923	{}	\N	2020-10-12 15:12:11.468017
49833	IBND	2020-09-28	35.37	35.46	35.28	35.32	18892	{}	\N	2020-10-12 15:12:12.420874
49834	IBND	2020-09-29	35.59	35.6369	35.54	35.61	18015	{}	\N	2020-10-12 15:12:12.432945
49835	IBND	2020-09-30	35.6399	35.6599	35.4928	35.59	10231	{}	\N	2020-10-12 15:12:12.440599
49836	IBND	2020-10-01	35.59	35.72	35.59	35.72	106635	{}	\N	2020-10-12 15:12:12.446881
49837	IBND	2020-10-02	35.65	35.69	35.5826	35.67	21177	{}	\N	2020-10-12 15:12:12.452293
49838	IBND	2020-10-05	35.85	35.88	35.71	35.7788	14478	{}	\N	2020-10-12 15:12:12.457926
49839	IBND	2020-10-06	35.87	35.95	35.7601	35.84	120984	{}	\N	2020-10-12 15:12:12.462888
49840	IBND	2020-10-07	35.8493	35.91	35.78	35.8017	15176	{}	\N	2020-10-12 15:12:12.468048
49841	IBND	2020-10-08	35.83	35.91	35.76	35.91	16688	{}	\N	2020-10-12 15:12:12.472928
49842	IBND	2020-10-09	36.06	36.24	35.97	36.19	34630	{}	\N	2020-10-12 15:12:12.47789
49843	IBND	2020-10-12	36.12	36.12	35.995	36.0675	2920	{}	\N	2020-10-12 15:12:12.482231
49844	ICCH	2020-09-28	11.75	11.75	11.74	11.74	360	{}	\N	2020-10-12 15:12:14.062927
49845	ICCH	2020-09-29	11.75	11.75	11.65	11.7023	3233	{}	\N	2020-10-12 15:12:14.069877
49846	ICCH	2020-09-30	11.7001	11.7001	11.7001	11.7001	150	{}	\N	2020-10-12 15:12:14.075285
49847	ICCH	2020-10-01	11.75	11.75	11.75	11.75	3301	{}	\N	2020-10-12 15:12:14.080149
49848	ICCH	2020-10-02	11.65	11.7501	11.65	11.75	1478	{}	\N	2020-10-12 15:12:14.085052
49849	ICCH	2020-10-05	11.74	11.77	11.74	11.74	724	{}	\N	2020-10-12 15:12:14.089107
49850	ICCH	2020-10-06	12	12.15	11.81	11.84	15370	{}	\N	2020-10-12 15:12:14.092884
49851	ICCH	2020-10-08	12.49	12.5	12.46	12.497	10537	{}	\N	2020-10-12 15:12:14.096948
49852	ICLN	2020-09-28	17.26	17.59	17.221	17.56	2561755	{}	\N	2020-10-12 15:12:15.398903
49853	ICLN	2020-09-29	17.82	18.05	17.75	17.98	1815555	{}	\N	2020-10-12 15:12:15.408834
49854	ICLN	2020-09-30	18.22	18.6689	18.16	18.49	2988836	{}	\N	2020-10-12 15:12:15.414768
49855	ICLN	2020-10-01	18.9	19.1	18.72	19.09	3148390	{}	\N	2020-10-12 15:12:15.420104
49856	ICLN	2020-10-02	18.71	19.27	18.63	19.08	2872028	{}	\N	2020-10-12 15:12:15.425468
49857	ICLN	2020-10-05	19.41	19.97	19.3218	19.95	4837557	{}	\N	2020-10-12 15:12:15.430409
49858	ICLN	2020-10-06	20.23	20.45	19.85	20.09	6147032	{}	\N	2020-10-12 15:12:15.434525
49859	ICLN	2020-10-07	20.52	21.17	20.4	21.09	7713842	{}	\N	2020-10-12 15:12:15.43833
49860	ICLN	2020-10-08	21.73	21.74	20.8201	21.07	6344933	{}	\N	2020-10-12 15:12:15.442189
49861	ICLN	2020-10-09	20.92	21.48	20.88	21.39	3108809	{}	\N	2020-10-12 15:12:15.446062
49862	ICLN	2020-10-12	21.57	21.59	20.71	20.795	3231836	{}	\N	2020-10-12 15:12:15.449965
49863	IDHD	2020-09-28	23.37	23.37	23	23.072	4481	{}	\N	2020-10-12 15:12:16.585741
49864	IDHD	2020-09-29	23.1597	23.1597	23.09	23.09	2769	{}	\N	2020-10-12 15:12:16.596299
49865	IDHD	2020-09-30	23.01	23.1268	22.9	22.9573	4904	{}	\N	2020-10-12 15:12:16.603276
49866	IDHD	2020-10-01	23.275	23.275	23.03	23.03	1548	{}	\N	2020-10-12 15:12:16.609361
49867	IDHD	2020-10-02	23.07	23.14	22.83	23.1399	3405	{}	\N	2020-10-12 15:12:16.614116
49868	IDHD	2020-10-05	23.59	23.59	23.4401	23.4401	2653	{}	\N	2020-10-12 15:12:16.618905
49869	IDHD	2020-10-06	23.61	23.61	23.2826	23.35	2626	{}	\N	2020-10-12 15:12:16.623122
49870	IDHD	2020-10-07	23.3	23.45	23.241	23.45	12341	{}	\N	2020-10-12 15:12:16.627151
49871	IDHD	2020-10-08	23.4937	23.51	23.4937	23.51	707	{}	\N	2020-10-12 15:12:16.630974
49872	IDHD	2020-10-09	23.65	23.65	23.5767	23.5767	879	{}	\N	2020-10-12 15:12:16.634775
49873	IDHD	2020-10-12	23.57	23.57	23.57	23.57	365	{}	\N	2020-10-12 15:12:16.639685
49874	IDX	2020-09-28	15.98	16.0216	15.98	16.02	10570	{}	\N	2020-10-12 15:12:17.792981
49875	IDX	2020-09-29	15.78	15.9393	15.78	15.83	39175	{}	\N	2020-10-12 15:12:17.802461
49876	IDX	2020-09-30	15.94	15.97	15.872	15.89	1696	{}	\N	2020-10-12 15:12:17.808665
49877	IDX	2020-10-01	16.13	16.3	16.13	16.3	18952	{}	\N	2020-10-12 15:12:17.814512
49878	IDX	2020-10-02	16.19	16.19	16.08	16.11	5799	{}	\N	2020-10-12 15:12:17.820272
49879	IDX	2020-10-05	16.5	16.53	16.43	16.47	17914	{}	\N	2020-10-12 15:12:17.825036
49880	IDX	2020-10-06	16.52	16.65	16.44	16.52	28696	{}	\N	2020-10-12 15:12:17.829479
49881	IDX	2020-10-07	16.5	16.59	16.5	16.565	7126	{}	\N	2020-10-12 15:12:17.833881
49882	IDX	2020-10-08	16.5575	16.7219	16.545	16.67	18175	{}	\N	2020-10-12 15:12:17.838092
49883	IDX	2020-10-09	16.67	16.83	16.67	16.8	42014	{}	\N	2020-10-12 15:12:17.842444
49884	IDX	2020-10-12	16.69	16.8	16.2	16.8	44927	{}	\N	2020-10-12 15:12:17.846568
49885	IEUR	2020-09-28	44.77	44.94	44.74	44.88	208346	{}	\N	2020-10-12 15:12:18.757647
49886	IEUR	2020-09-29	44.9	45.0399	44.68	44.87	284654	{}	\N	2020-10-12 15:12:18.769064
49887	IEUR	2020-09-30	44.85	45.12	44.5633	44.72	203298	{}	\N	2020-10-12 15:12:18.77768
49888	IEUR	2020-10-01	45.08	45.11	44.79	45.05	333217	{}	\N	2020-10-12 15:12:18.783674
49889	IEUR	2020-10-02	44.51	45.17	44.49	45.06	133789	{}	\N	2020-10-12 15:12:18.788283
49890	IEUR	2020-10-05	45.61	45.86	45.55	45.86	223143	{}	\N	2020-10-12 15:12:18.793218
49891	IEUR	2020-10-06	45.86	45.89	45.17	45.21	225903	{}	\N	2020-10-12 15:12:18.797241
49892	IEUR	2020-10-07	45.54	45.6889	45.4449	45.58	147601	{}	\N	2020-10-12 15:12:18.801116
49893	IEUR	2020-10-08	45.85	46.03	45.82	46	181358	{}	\N	2020-10-12 15:12:18.805116
49894	IEUR	2020-10-09	46.38	46.52	46.279	46.48	68337	{}	\N	2020-10-12 15:12:18.809137
49895	IEUR	2020-10-12	46.6	46.73	46.58	46.73	198862	{}	\N	2020-10-12 15:12:18.813149
49896	IGA	2020-09-28	8.3	8.4	8.3	8.35	72354	{}	\N	2020-10-12 15:12:20.405224
49897	IGA	2020-09-29	8.38	8.38	8.3	8.35	170642	{}	\N	2020-10-12 15:12:20.410228
49898	IGA	2020-09-30	8.32	8.415	8.32	8.34	304844	{}	\N	2020-10-12 15:12:20.415421
49899	IGA	2020-10-01	8.19	8.24	8.15	8.17	304986	{}	\N	2020-10-12 15:12:20.420042
49900	IGA	2020-10-02	8.11	8.15	8.08	8.09	183844	{}	\N	2020-10-12 15:12:20.424838
49901	IGA	2020-10-05	8.13	8.22	8.12	8.2	180604	{}	\N	2020-10-12 15:12:20.429398
49902	IGA	2020-10-06	8.2	8.25	8.13	8.14	120475	{}	\N	2020-10-12 15:12:20.433528
49903	IGA	2020-10-07	8.18	8.3	8.17	8.28	183388	{}	\N	2020-10-12 15:12:20.437812
49904	IGA	2020-10-08	8.3	8.3	8.23	8.27	80902	{}	\N	2020-10-12 15:12:20.442178
49905	IGA	2020-10-09	8.29	8.36	8.29	8.29	159400	{}	\N	2020-10-12 15:12:20.44627
49906	IGA	2020-10-12	8.35	8.35	8.33	8.33	54264	{}	\N	2020-10-12 15:12:20.450505
49907	IGOV	2020-09-28	52.63	52.66	52.54	52.61	125948	{}	\N	2020-10-12 15:12:21.39262
49908	IGOV	2020-09-29	52.85	52.96	52.84	52.9	70347	{}	\N	2020-10-12 15:12:21.400366
49909	IGOV	2020-09-30	52.88	52.99	52.81	52.87	125570	{}	\N	2020-10-12 15:12:21.40626
49910	IGOV	2020-10-01	52.92	53.1	52.85	53.03	289104	{}	\N	2020-10-12 15:12:21.411879
49911	IGOV	2020-10-02	52.97	53.0432	52.92	52.94	91778	{}	\N	2020-10-12 15:12:21.417383
49912	IGOV	2020-10-05	53.09	53.18	53.01	53.01	66934	{}	\N	2020-10-12 15:12:21.42226
49913	IGOV	2020-10-06	53.12	53.1522	52.91	52.95	81475	{}	\N	2020-10-12 15:12:21.426622
49914	IGOV	2020-10-07	52.97	52.97	52.84	52.85	363298	{}	\N	2020-10-12 15:12:21.430995
49915	IGOV	2020-10-08	52.91	53.0399	52.91	52.98	143631	{}	\N	2020-10-12 15:12:21.43533
49916	IGOV	2020-10-09	53.3	53.37	53.18	53.36	120467	{}	\N	2020-10-12 15:12:21.439618
49917	IGOV	2020-10-12	53.42	53.45	53.32	53.4258	17960	{}	\N	2020-10-12 15:12:21.443697
49918	IHT	2020-09-28	1.0973	1.18	1.085	1.095	27066	{}	\N	2020-10-12 15:12:22.759542
49919	IHT	2020-09-29	1.08	1.09	1.05	1.07	16194	{}	\N	2020-10-12 15:12:22.768694
49920	IHT	2020-10-02	1.12	1.14	1.12	1.14	1144	{}	\N	2020-10-12 15:12:22.774913
49921	IHT	2020-10-05	1.19	1.19	1.14	1.14	994	{}	\N	2020-10-12 15:12:22.780013
49922	IHT	2020-10-06	1.17	1.17	1.0675	1.11	2865	{}	\N	2020-10-12 15:12:22.78423
49923	IHT	2020-10-07	1.2	1.27	1.2	1.27	3823	{}	\N	2020-10-12 15:12:22.788927
49924	IHT	2020-10-08	1.18	1.425	1.18	1.365	10204	{}	\N	2020-10-12 15:12:22.792886
49925	IHT	2020-10-09	1.55	1.55	1.3741	1.41	8426	{}	\N	2020-10-12 15:12:22.796463
49926	IHT	2020-10-12	1.41	1.41	1.41	1.41	425	{}	\N	2020-10-12 15:12:22.800283
49927	IBMJ	2020-09-28	25.885	25.8914	25.86	25.86	10824	{}	\N	2020-10-12 15:12:24.940835
49928	IBMJ	2020-09-29	25.895	25.9022	25.86	25.8699	18335	{}	\N	2020-10-12 15:12:24.950668
49929	IBMJ	2020-09-30	25.87	25.91	25.86	25.91	116822	{}	\N	2020-10-12 15:12:24.957245
49930	IBMJ	2020-10-01	25.8501	25.8621	25.845	25.86	15309	{}	\N	2020-10-12 15:12:24.963278
49931	IBMJ	2020-10-02	25.8327	25.87	25.811	25.82	35482	{}	\N	2020-10-12 15:12:24.967913
49932	IBMJ	2020-10-05	25.84	25.87	25.8204	25.85	30421	{}	\N	2020-10-12 15:12:24.972768
49933	IBMJ	2020-10-06	25.87	25.87	25.8269	25.87	21798	{}	\N	2020-10-12 15:12:24.976771
49934	IBMJ	2020-10-07	25.87	25.88	25.8501	25.86	39731	{}	\N	2020-10-12 15:12:24.980474
49935	IBMJ	2020-10-08	25.87	25.87	25.8501	25.87	12236	{}	\N	2020-10-12 15:12:24.98421
49936	IBMJ	2020-10-09	25.83	25.8652	25.82	25.8303	22087	{}	\N	2020-10-12 15:12:24.987808
49937	IBMJ	2020-10-12	25.8395	25.86	25.8395	25.845	2351	{}	\N	2020-10-12 15:12:24.991427
49938	ICBK	2020-09-28	18.16	18.75	18.16	18.34	2776	{}	\N	2020-10-12 15:12:26.123834
49939	ICBK	2020-09-29	18.801	18.96	18.48	18.96	1101	{}	\N	2020-10-12 15:12:26.136953
49940	ICBK	2020-09-30	19.26	19.39	18.8	18.8	1799	{}	\N	2020-10-12 15:12:26.144939
49941	ICBK	2020-10-01	18.33	18.33	18.33	18.33	2059	{}	\N	2020-10-12 15:12:26.151167
49942	ICBK	2020-10-02	18.69	19.2	18.24	18.24	3475	{}	\N	2020-10-12 15:12:26.156413
49943	ICBK	2020-10-05	18.96	19.36	18.7	19.36	4130	{}	\N	2020-10-12 15:12:26.161235
49944	ICBK	2020-10-06	19.36	20	18.2001	19.8	6069	{}	\N	2020-10-12 15:12:26.165673
49945	ICBK	2020-10-07	19.87	19.91	19.6963	19.85	6184	{}	\N	2020-10-12 15:12:26.170873
49946	ICBK	2020-10-08	20	20	19.77	19.99	15182	{}	\N	2020-10-12 15:12:26.175051
49947	ICBK	2020-10-09	20	20.22	19.45	19.87	8005	{}	\N	2020-10-12 15:12:26.178936
49948	ICBK	2020-10-12	19.87	20	19.87	19.87	1871	{}	\N	2020-10-12 15:12:26.183416
49949	ICOL	2020-09-28	8.33	8.3999	8.33	8.3999	4887	{}	\N	2020-10-12 15:12:27.204574
49950	ICOL	2020-09-29	8.2599	8.26	8.225	8.25	4739	{}	\N	2020-10-12 15:12:27.213367
49951	ICOL	2020-09-30	8.226	8.37	8.226	8.35	4376	{}	\N	2020-10-12 15:12:27.219479
49952	ICOL	2020-10-01	8.54	8.54	8.28	8.32	9540	{}	\N	2020-10-12 15:12:27.224778
49953	ICOL	2020-10-02	8.247	8.265	8.18	8.265	9831	{}	\N	2020-10-12 15:12:27.229338
49954	ICOL	2020-10-05	8.46	8.46	8.33	8.34	2480	{}	\N	2020-10-12 15:12:27.233477
49955	ICOL	2020-10-06	8.4278	8.4278	8.3	8.3	3421	{}	\N	2020-10-12 15:12:27.237745
49956	ICOL	2020-10-07	8.25	8.25	8.25	8.25	641	{}	\N	2020-10-12 15:12:27.241495
49957	ICOL	2020-10-08	8.3599	8.3945	8.345	8.3945	1788	{}	\N	2020-10-12 15:12:27.245405
49958	ICOL	2020-10-09	8.43	8.454	8.385	8.41	6703	{}	\N	2020-10-12 15:12:27.248899
49959	ICOL	2020-10-12	8.58	8.58	8.4799	8.4799	3164	{}	\N	2020-10-12 15:12:27.252468
49960	IDEV	2020-09-28	54.14	54.27	54.0388	54.23	211525	{}	\N	2020-10-12 15:12:28.782858
49961	IDEV	2020-09-29	54.23	54.35	53.989	54.11	67822	{}	\N	2020-10-12 15:12:28.7925
49962	IDEV	2020-09-30	54.02	54.29	53.795	53.97	81590	{}	\N	2020-10-12 15:12:28.799004
49963	IDEV	2020-10-01	54.26	54.29	53.9785	54.29	40082	{}	\N	2020-10-12 15:12:28.804843
49964	IDEV	2020-10-02	53.64	54.2674	53.61	54.16	53510	{}	\N	2020-10-12 15:12:28.809925
49965	IDEV	2020-10-05	54.67	55.01	54.67	55.01	38019	{}	\N	2020-10-12 15:12:28.814774
49966	IDEV	2020-10-06	55.08	55.11	54.36	54.49	205000	{}	\N	2020-10-12 15:12:28.856513
49967	IDEV	2020-10-07	54.89	55.01	54.7678	54.96	46453	{}	\N	2020-10-12 15:12:28.866735
49968	IDEV	2020-10-08	55.22	55.349	55.15	55.33	49482	{}	\N	2020-10-12 15:12:28.873219
49969	IDEV	2020-10-09	55.6	55.77	55.52	55.75	49669	{}	\N	2020-10-12 15:12:28.878949
49970	IDEV	2020-10-12	55.88	55.9388	55.8193	55.9388	10229	{}	\N	2020-10-12 15:12:28.883763
49971	IEF	2020-09-28	122.04	122.075	121.995	122.06	4177891	{}	\N	2020-10-12 15:12:30.211171
49972	IEF	2020-09-29	122.13	122.22	122.093	122.15	2411532	{}	\N	2020-10-12 15:12:30.220914
49973	IEF	2020-09-30	122.03	122.03	121.66	121.82	4292668	{}	\N	2020-10-12 15:12:30.228018
49974	IEF	2020-10-01	121.53	121.8989	121.43	121.81	8706447	{}	\N	2020-10-12 15:12:30.233935
49975	IEF	2020-10-02	121.91	121.93	121.595	121.685	7355773	{}	\N	2020-10-12 15:12:30.239383
49976	IEF	2020-10-05	121.35	121.36	120.92	120.92	6149638	{}	\N	2020-10-12 15:12:30.244225
49977	IEF	2020-10-06	120.945	121.4	120.87	121.21	8184265	{}	\N	2020-10-12 15:12:30.248382
49978	IEF	2020-10-07	120.97	121.1	120.78	120.85	7161518	{}	\N	2020-10-12 15:12:30.252227
49979	IEF	2020-10-08	120.99	121.09	120.94	121.07	3548397	{}	\N	2020-10-12 15:12:30.256275
49980	IEF	2020-10-09	120.989	121.13	120.78	121.005	4177947	{}	\N	2020-10-12 15:12:30.26043
49981	IEF	2020-10-12	121.06	121.08	120.99	120.995	734823	{}	\N	2020-10-12 15:12:30.264431
49982	IEV	2020-09-28	42.01	42.13	41.98	42.1	236277	{}	\N	2020-10-12 15:12:31.754255
49983	IEV	2020-09-29	42.14	42.29	41.94	42.09	115681	{}	\N	2020-10-12 15:12:31.763386
49984	IEV	2020-09-30	42.06	42.29	41.74	41.86	290739	{}	\N	2020-10-12 15:12:31.769469
49985	IEV	2020-10-01	42.2	42.25	41.94	42.16	125593	{}	\N	2020-10-12 15:12:31.775442
49986	IEV	2020-10-02	41.68	42.2899	41.68	42.17	93780	{}	\N	2020-10-12 15:12:31.78055
49987	IEV	2020-10-05	42.64	42.91	42.61	42.91	99059	{}	\N	2020-10-12 15:12:31.785496
49988	IEV	2020-10-06	42.97	42.97	42.25	42.29	115624	{}	\N	2020-10-12 15:12:31.789711
49989	IEV	2020-10-07	42.59	42.74	42.5	42.66	233848	{}	\N	2020-10-12 15:12:31.79361
49990	IEV	2020-10-08	42.91	43.08	42.87	43.07	229764	{}	\N	2020-10-12 15:12:31.797527
49991	IEV	2020-10-09	43.38	43.53	43.3	43.47	80389	{}	\N	2020-10-12 15:12:31.801341
49992	IEV	2020-10-12	43.6	43.68	43.59	43.67	17238	{}	\N	2020-10-12 15:12:31.805082
49993	IHD	2020-09-28	6.53	6.6	6.51	6.6	34284	{}	\N	2020-10-12 15:12:33.535818
49994	IHD	2020-09-29	6.6	6.7274	6.52	6.58	126622	{}	\N	2020-10-12 15:12:33.543048
49995	IHD	2020-09-30	6.59	6.64	6.58	6.61	132730	{}	\N	2020-10-12 15:12:33.551188
49996	IHD	2020-10-01	6.49	6.52	6.41	6.44	149546	{}	\N	2020-10-12 15:12:33.559329
49997	IHD	2020-10-02	6.33	6.38	6.3	6.35	265277	{}	\N	2020-10-12 15:12:33.566516
49998	IHD	2020-10-05	6.35	6.3999	6.28	6.38	370503	{}	\N	2020-10-12 15:12:33.572696
49999	IHD	2020-10-06	6.39	6.45	6.07	6.42	223390	{}	\N	2020-10-12 15:12:33.578697
50000	IHD	2020-10-07	6.46	6.5	6.45	6.49	159838	{}	\N	2020-10-12 15:12:33.58493
50001	IHD	2020-10-08	6.54	6.6	6.4919	6.55	200324	{}	\N	2020-10-12 15:12:33.591759
50002	IHD	2020-10-09	6.59	6.6	6.55	6.58	229786	{}	\N	2020-10-12 15:12:33.597462
50003	IHD	2020-10-12	6.61	6.64	6.5991	6.62	27570	{}	\N	2020-10-12 15:12:33.602623
50004	III	2020-09-28	2.06	2.1	2.06	2.07	23788	{}	\N	2020-10-12 15:12:34.67039
50005	III	2020-09-29	2.08	2.1	2.05	2.1	30893	{}	\N	2020-10-12 15:12:34.680577
50006	III	2020-09-30	2.1447	2.16	2.07	2.11	26734	{}	\N	2020-10-12 15:12:34.686928
50007	III	2020-10-01	2.11	2.11	2.06	2.09	29051	{}	\N	2020-10-12 15:12:34.692793
50008	III	2020-10-02	2.07	2.1	2.05	2.09	30240	{}	\N	2020-10-12 15:12:34.697351
50009	III	2020-10-05	2.1	2.15	2.09	2.15	50665	{}	\N	2020-10-12 15:12:34.701262
50010	III	2020-10-06	2.15	2.2	2.12	2.13	32623	{}	\N	2020-10-12 15:12:34.705571
50011	III	2020-10-07	2.14	2.19	2.12	2.15	43682	{}	\N	2020-10-12 15:12:34.709303
50012	III	2020-10-08	2.19	2.19	2.13	2.13	26192	{}	\N	2020-10-12 15:12:34.713002
50013	III	2020-10-09	2.17	2.19	2.1318	2.15	18698	{}	\N	2020-10-12 15:12:34.716723
50014	III	2020-10-12	2.15	2.18	2.11	2.12	25954	{}	\N	2020-10-12 15:12:34.720392
50015	ISDR	2020-09-28	19.81	20	19.25	19.66	62687	{}	\N	2020-10-12 15:12:36.078145
50016	ISDR	2020-09-29	19.89	20.19	19.6167	19.82	55278	{}	\N	2020-10-12 15:12:36.087199
50017	ISDR	2020-09-30	20.19	20.19	19.7701	19.88	24822	{}	\N	2020-10-12 15:12:36.093253
50018	ISDR	2020-10-01	20.1539	20.25	19.97	19.97	56051	{}	\N	2020-10-12 15:12:36.099221
50019	ISDR	2020-10-02	19.86	20.03	19.645	19.82	53516	{}	\N	2020-10-12 15:12:36.10431
50020	ISDR	2020-10-05	19.83	20.35	19.83	20.175	33079	{}	\N	2020-10-12 15:12:36.109566
50021	ISDR	2020-10-06	20.3	21.2	20.01	20.92	35659	{}	\N	2020-10-12 15:12:36.113646
50022	ISDR	2020-10-07	21.26	22.4999	21.06	21.54	29118	{}	\N	2020-10-12 15:12:36.117555
50023	ISDR	2020-10-08	21.7981	21.7999	20.74	21.15	16676	{}	\N	2020-10-12 15:12:36.122437
50024	ISDR	2020-10-09	21.49	21.49	21.0164	21.21	13387	{}	\N	2020-10-12 15:12:36.127042
50025	ISDR	2020-10-12	21.21	21.63	21.21	21.6001	5322	{}	\N	2020-10-12 15:12:36.13147
50026	IBN	2020-09-28	9.9	9.935	9.81	9.82	6511428	{}	\N	2020-10-12 15:12:37.476766
50027	IBN	2020-09-29	9.83	9.88	9.765	9.85	9238877	{}	\N	2020-10-12 15:12:37.486264
50028	IBN	2020-09-30	9.76	9.89	9.7	9.83	8361424	{}	\N	2020-10-12 15:12:37.492515
50029	IBN	2020-10-01	10.11	10.28	10.11	10.25	6191820	{}	\N	2020-10-12 15:12:37.498685
50030	IBN	2020-10-02	10.08	10.33	10.05	10.24	12324512	{}	\N	2020-10-12 15:12:37.504051
50031	IBN	2020-10-05	10.28	10.47	10.28	10.45	12677821	{}	\N	2020-10-12 15:12:37.508746
50032	IBN	2020-10-06	10.53	10.585	10.38	10.39	17611045	{}	\N	2020-10-12 15:12:37.513149
50033	IBN	2020-10-07	10.52	10.66	10.5	10.63	10481552	{}	\N	2020-10-12 15:12:37.517388
50034	IBN	2020-10-08	10.74	10.9	10.7	10.81	15972170	{}	\N	2020-10-12 15:12:37.521527
50035	IBN	2020-10-09	11.08	11.16	10.97	11.04	11410859	{}	\N	2020-10-12 15:12:37.52565
50036	IBN	2020-10-12	11.01	11.06	11	11.035	3347622	{}	\N	2020-10-12 15:12:37.529782
50037	ICCC	2020-09-28	5.65	5.65	5.65	5.65	694	{}	\N	2020-10-12 15:12:38.621631
50038	ICCC	2020-09-29	5.45	5.5815	5.45	5.48	2838	{}	\N	2020-10-12 15:12:38.633694
50039	ICCC	2020-09-30	5.325	5.65	5.325	5.41	822	{}	\N	2020-10-12 15:12:38.640452
50040	ICCC	2020-10-01	5.65	5.65	5.5253	5.55	2563	{}	\N	2020-10-12 15:12:38.646312
50041	ICCC	2020-10-02	5.65	5.65	5.47	5.47	3677	{}	\N	2020-10-12 15:12:38.650792
50042	ICCC	2020-10-05	5.533	5.64	5.46	5.55	3026	{}	\N	2020-10-12 15:12:38.65515
50043	ICCC	2020-10-06	5.525	5.91	5.4891	5.64	19753	{}	\N	2020-10-12 15:12:38.659412
50044	ICCC	2020-10-07	5.7593	5.76	5.64	5.7024	4839	{}	\N	2020-10-12 15:12:38.663121
50045	ICCC	2020-10-08	5.57	5.7816	5.57	5.62	5284	{}	\N	2020-10-12 15:12:38.666849
50046	ICCC	2020-10-09	5.47	5.88	5.47	5.695	3230	{}	\N	2020-10-12 15:12:38.670412
50047	ICCC	2020-10-12	5.85	5.85	5.85	5.85	345	{}	\N	2020-10-12 15:12:38.674145
50048	ICLR	2020-09-28	184.61	186.8578	177.545	186.04	153462	{}	\N	2020-10-12 15:12:40.371073
50049	ICLR	2020-09-29	184.36	189.47	184.36	188.71	235645	{}	\N	2020-10-12 15:12:40.379212
50050	ICLR	2020-09-30	190.01	192.355	188.05	191.09	177792	{}	\N	2020-10-12 15:12:40.384466
50051	ICLR	2020-10-01	188.145	197.4	188.145	196.78	303993	{}	\N	2020-10-12 15:12:40.389174
50052	ICLR	2020-10-02	193.64	198.96	190.005	196.64	311406	{}	\N	2020-10-12 15:12:40.393721
50053	ICLR	2020-10-05	198.47	203.4	197.82	201.68	267979	{}	\N	2020-10-12 15:12:40.398607
50054	ICLR	2020-10-06	202.78	206.61	201.01	203.06	259476	{}	\N	2020-10-12 15:12:40.402454
50055	ICLR	2020-10-07	204.64	208.53	200.78	206.7	339034	{}	\N	2020-10-12 15:12:40.406462
50056	ICLR	2020-10-08	207.7	209.4	207.34	208.43	204746	{}	\N	2020-10-12 15:12:40.410324
50057	ICLR	2020-10-09	210.59	212.78	206.45	211.97	329332	{}	\N	2020-10-12 15:12:40.414194
50058	ICLR	2020-10-12	213.4	214.36	210.5	213.57	22465	{}	\N	2020-10-12 15:12:40.417778
50059	IEFA	2020-09-28	60.48	60.64	60.39	60.64	7753017	{}	\N	2020-10-12 15:12:41.925571
50060	IEFA	2020-09-29	60.62	60.8	60.33	60.53	7099842	{}	\N	2020-10-12 15:12:41.937329
50061	IEFA	2020-09-30	60.34	60.739	60.105	60.28	10511698	{}	\N	2020-10-12 15:12:41.944578
50062	IEFA	2020-10-01	60.58	60.71	60.32	60.66	9671077	{}	\N	2020-10-12 15:12:41.9505
50063	IEFA	2020-10-02	59.91	60.7	59.89	60.57	6570400	{}	\N	2020-10-12 15:12:41.955535
50064	IEFA	2020-10-05	61.04	61.5	61.02	61.5	3906781	{}	\N	2020-10-12 15:12:41.960382
50065	IEFA	2020-10-06	61.52	61.61	60.7376	60.87	6949759	{}	\N	2020-10-12 15:12:41.964759
50066	IEFA	2020-10-07	61.31	61.43	61.125	61.3383	14289295	{}	\N	2020-10-12 15:12:41.968805
50067	IEFA	2020-10-08	61.66	61.78	61.56	61.76	5327002	{}	\N	2020-10-12 15:12:41.972931
50068	IEFA	2020-10-09	62.06	62.245	61.965	62.17	8038624	{}	\N	2020-10-12 15:12:41.976905
50069	IEFA	2020-10-12	62.31	62.4776	62.26	62.465	1149935	{}	\N	2020-10-12 15:12:41.98083
50070	IFN	2020-09-28	16.31	16.406	16.27	16.34	87463	{}	\N	2020-10-12 15:12:44.259876
50071	IFN	2020-09-29	16.44	16.44	16.22	16.31	157429	{}	\N	2020-10-12 15:12:44.281433
50072	IFN	2020-09-30	16.4	16.5	16.36	16.5	247640	{}	\N	2020-10-12 15:12:44.287143
50073	IFN	2020-10-01	16.61	16.86	16.61	16.8	206374	{}	\N	2020-10-12 15:12:44.297038
50074	IFN	2020-10-02	16.59	16.78	16.5328	16.57	102556	{}	\N	2020-10-12 15:12:44.302467
50075	IFN	2020-10-05	16.63	16.88	16.63	16.77	120646	{}	\N	2020-10-12 15:12:44.308014
50076	IFN	2020-10-06	16.87	16.95	16.81	16.85	143204	{}	\N	2020-10-12 15:12:44.314029
50077	IFN	2020-10-07	16.96	17.0893	16.951	17.04	190313	{}	\N	2020-10-12 15:12:44.319569
50078	IFN	2020-10-08	17.16	17.25	17.09	17.23	151945	{}	\N	2020-10-12 15:12:44.325155
50079	IFN	2020-10-09	17.26	17.35	17.26	17.31	104030	{}	\N	2020-10-12 15:12:44.330691
50080	IFN	2020-10-12	17.31	17.32	17.21	17.25	45198	{}	\N	2020-10-12 15:12:44.33643
50081	IGM	2020-09-28	307.3	307.77	304.94	307.77	73475	{}	\N	2020-10-12 15:12:45.687838
50082	IGM	2020-09-29	307.89	309.1345	306.99	307.68	29252	{}	\N	2020-10-12 15:12:45.698774
50083	IGM	2020-09-30	307.08	312.23	307.08	308.8	46952	{}	\N	2020-10-12 15:12:45.705393
50084	IGM	2020-10-01	313	314.58	311.48	313.94	175235	{}	\N	2020-10-12 15:12:45.711345
50085	IGM	2020-10-02	306.92	311.96	306.1	306.92	46389	{}	\N	2020-10-12 15:12:45.716148
50086	IGM	2020-10-05	308.77	313.31	308.77	313.18	39601	{}	\N	2020-10-12 15:12:45.720753
50087	IGM	2020-10-06	312.85	315.41	307.7435	308.96	29316	{}	\N	2020-10-12 15:12:45.724525
50088	IGM	2020-10-07	311.83	314.76	311.49	314.25	27282	{}	\N	2020-10-12 15:12:45.728723
50089	IGM	2020-10-08	316.39	316.79	315.605	316.44	23224	{}	\N	2020-10-12 15:12:45.732638
50090	IGM	2020-10-09	317.95	321.34	317.95	321.13	30984	{}	\N	2020-10-12 15:12:45.736537
50091	IGM	2020-10-12	326	327.48	324.37	327.48	19343	{}	\N	2020-10-12 15:12:45.740521
50092	IHDG	2020-09-28	36.73	36.83	36.705	36.76	80919	{}	\N	2020-10-12 15:12:46.916201
50093	IHDG	2020-09-29	36.85	36.91	36.735	36.75	57477	{}	\N	2020-10-12 15:12:46.926294
50094	IHDG	2020-09-30	36.78	36.825	36.4701	36.62	32560	{}	\N	2020-10-12 15:12:46.933145
50095	IHDG	2020-10-01	36.77	36.97	36.77	36.96	40686	{}	\N	2020-10-12 15:12:46.939305
50096	IHDG	2020-10-02	36.6	36.97	36.6	36.9	41629	{}	\N	2020-10-12 15:12:46.9439
50097	IHDG	2020-10-05	36.9427	37.175	36.92	37.15	53006	{}	\N	2020-10-12 15:12:46.948447
50098	IHDG	2020-10-06	36.98	37.09	36.65	36.72	80337	{}	\N	2020-10-12 15:12:46.953102
50099	IHDG	2020-10-07	37.04	37.13	36.88	37.09	51387	{}	\N	2020-10-12 15:12:46.957316
50100	IHDG	2020-10-08	37.23	37.34	37.22	37.335	54010	{}	\N	2020-10-12 15:12:46.96118
50101	IHDG	2020-10-09	37.5	37.57	37.4319	37.51	52355	{}	\N	2020-10-12 15:12:46.96516
50102	IHDG	2020-10-12	37.66	37.73	37.645	37.73	7205	{}	\N	2020-10-12 15:12:46.96917
50103	IIIN	2020-09-28	18.4	19.03	18.4	18.68	51237	{}	\N	2020-10-12 15:12:48.006739
50104	IIIN	2020-09-29	18.6	18.87	18.12	18.79	33710	{}	\N	2020-10-12 15:12:48.01823
50105	IIIN	2020-09-30	18.78	19.14	18.61	18.7	57931	{}	\N	2020-10-12 15:12:48.02616
50106	IIIN	2020-10-01	18.68	18.96	18.36	18.83	38216	{}	\N	2020-10-12 15:12:48.032327
50107	IIIN	2020-10-02	18.3	19.13	18.11	18.95	43125	{}	\N	2020-10-12 15:12:48.037438
50108	IIIN	2020-10-05	18.95	19.87	18.92	19.7	49991	{}	\N	2020-10-12 15:12:48.042677
50109	IIIN	2020-10-06	19.88	20.42	19.27	20.01	83644	{}	\N	2020-10-12 15:12:48.046696
50110	IIIN	2020-10-07	20.09	20.34	19.82	20.15	97639	{}	\N	2020-10-12 15:12:48.050741
50111	IIIN	2020-10-08	20.54	21	19.49	20.74	48363	{}	\N	2020-10-12 15:12:48.054815
50112	IIIN	2020-10-09	20.95	21.25	20.31	20.51	61801	{}	\N	2020-10-12 15:12:48.059061
50113	IIIN	2020-10-12	20.55	20.55	20.075	20.235	3698	{}	\N	2020-10-12 15:12:48.063104
50114	IJT	2020-09-28	173.73	176.86	173.73	176.35	40358	{}	\N	2020-10-12 15:12:49.255515
50115	IJT	2020-09-29	176.16	177.151	174.45	175.86	67351	{}	\N	2020-10-12 15:12:49.266908
50116	IJT	2020-09-30	177.02	178.93	175.24	176.44	75338	{}	\N	2020-10-12 15:12:49.274262
50117	IJT	2020-10-01	177.14	178.79	176.2382	178.79	98012	{}	\N	2020-10-12 15:12:49.279685
50118	IJT	2020-10-02	175.2	180.33	175.08	179.6	66327	{}	\N	2020-10-12 15:12:49.284362
50119	IJT	2020-10-05	181.05	183.7	181.05	183.63	43820	{}	\N	2020-10-12 15:12:49.289321
50120	IJT	2020-10-06	184.87	187.6095	183.0913	183.43	70300	{}	\N	2020-10-12 15:12:49.293672
50121	IJT	2020-10-07	185.43	186.36	184.4065	185.53	113957	{}	\N	2020-10-12 15:12:49.297677
50122	IJT	2020-10-08	186.85	188.0702	186.261	188.07	58706	{}	\N	2020-10-12 15:12:49.301413
50123	IJT	2020-10-09	189.48	190.33	188.45	189.63	49342	{}	\N	2020-10-12 15:12:49.305213
50124	IJT	2020-10-12	190.08	191.25	189.695	191.1696	38969	{}	\N	2020-10-12 15:12:49.309573
50125	IMGN	2020-09-28	3.63	3.63	3.48	3.58	1636020	{}	\N	2020-10-12 15:12:50.429553
50126	IMGN	2020-09-29	3.53	3.6	3.46	3.555	3714421	{}	\N	2020-10-12 15:12:50.438846
50127	IMGN	2020-09-30	3.55	3.71	3.52	3.6	1343406	{}	\N	2020-10-12 15:12:50.444737
50128	IMGN	2020-10-01	3.57	3.7	3.57	3.7	1411352	{}	\N	2020-10-12 15:12:50.449861
50129	IMGN	2020-10-02	3.82	4.2	3.785	3.87	2944918	{}	\N	2020-10-12 15:12:50.454405
50130	IMGN	2020-10-05	4.307	4.48	3.99	4.42	8642497	{}	\N	2020-10-12 15:12:50.45845
50131	IMGN	2020-10-06	4.35	4.45	4.22	4.29	3637144	{}	\N	2020-10-12 15:12:50.462762
50132	IMGN	2020-10-07	4.32	4.3579	4.23	4.3	1501568	{}	\N	2020-10-12 15:12:50.467247
50133	IMGN	2020-10-08	4.36	4.36	4.26	4.3	1238717	{}	\N	2020-10-12 15:12:50.471549
50134	IMGN	2020-10-09	4.3375	4.84	4.28	4.82	16080268	{}	\N	2020-10-12 15:12:50.475698
50135	IMGN	2020-10-12	4.6001	4.81	4.58	4.675	829616	{}	\N	2020-10-12 15:12:50.479893
50136	ICSH	2020-09-28	50.56	50.57	50.55	50.57	761282	{}	\N	2020-10-12 15:12:53.85733
50137	ICSH	2020-09-29	50.57	50.58	50.55	50.58	1687997	{}	\N	2020-10-12 15:12:53.868047
50138	ICSH	2020-09-30	50.59	50.59	50.55	50.55	863458	{}	\N	2020-10-12 15:12:53.875193
50139	ICSH	2020-10-01	50.56	50.565	50.54	50.55	1439838	{}	\N	2020-10-12 15:12:53.881513
50140	ICSH	2020-10-02	50.55	50.55	50.53	50.54	1525064	{}	\N	2020-10-12 15:12:53.886826
50141	ICSH	2020-10-05	50.55	50.554	50.54	50.54	442458	{}	\N	2020-10-12 15:12:53.891866
50142	ICSH	2020-10-06	50.55	50.55	50.54	50.545	691527	{}	\N	2020-10-12 15:12:53.896214
50143	ICSH	2020-10-07	50.56	50.56	50.54	50.54	855455	{}	\N	2020-10-12 15:12:53.900573
50144	ICSH	2020-10-08	50.54	50.56	50.54	50.56	646640	{}	\N	2020-10-12 15:12:53.905215
50145	ICSH	2020-10-09	50.54	50.56	50.54	50.55	576518	{}	\N	2020-10-12 15:12:53.909569
50146	ICSH	2020-10-12	50.54	50.56	50.54	50.555	150239	{}	\N	2020-10-12 15:12:53.914187
50147	IDHQ	2020-09-28	26.12	26.3187	26.1	26.21	12209	{}	\N	2020-10-12 15:12:55.192128
50148	IDHQ	2020-09-29	26.23	26.59	26.2	26.275	30863	{}	\N	2020-10-12 15:12:55.201395
50149	IDHQ	2020-09-30	26.1796	26.3494	26.135	26.27	62183	{}	\N	2020-10-12 15:12:55.20771
50150	IDHQ	2020-10-01	26.35	26.399	26.27	26.39	7909	{}	\N	2020-10-12 15:12:55.213237
50151	IDHQ	2020-10-02	25.94	26.339	25.94	26.25	10645	{}	\N	2020-10-12 15:12:55.218176
50152	IDHQ	2020-10-05	26.53	26.5652	26.45	26.54	15097	{}	\N	2020-10-12 15:12:55.223052
50153	IDHQ	2020-10-06	26.4298	26.45	26.11	26.11	10374	{}	\N	2020-10-12 15:12:55.227275
50154	IDHQ	2020-10-07	26.36	26.45	26.295	26.4	7214	{}	\N	2020-10-12 15:12:55.231122
50155	IDHQ	2020-10-08	26.49	27.22	26.465	26.57	134778	{}	\N	2020-10-12 15:12:55.23481
50156	IDHQ	2020-10-09	26.65	26.92	26.65	26.77	17278	{}	\N	2020-10-12 15:12:55.238557
50157	IDHQ	2020-10-12	26.89	26.89	26.855	26.855	849	{}	\N	2020-10-12 15:12:55.242311
50158	IEP	2020-09-28	49.25	49.34	48.22	48.95	82348	{}	\N	2020-10-12 15:12:58.245093
50159	IEP	2020-09-29	48.83	49.445	48.7	49.2	74328	{}	\N	2020-10-12 15:12:58.255653
50160	IEP	2020-09-30	49.45	49.93	48.86	49.32	101522	{}	\N	2020-10-12 15:12:58.262373
50161	IEP	2020-10-01	49.75	49.82	48.71	49.62	78933	{}	\N	2020-10-12 15:12:58.267883
50162	IEP	2020-10-02	49.45	50.21	49.08	50.05	63446	{}	\N	2020-10-12 15:12:58.272475
50163	IEP	2020-10-05	50.07	51.29	50.07	50.64	126695	{}	\N	2020-10-12 15:12:58.277577
50164	IEP	2020-10-06	51.14	51.5	50.3301	50.63	83483	{}	\N	2020-10-12 15:12:58.281616
50165	IEP	2020-10-07	51.05	51.59	50.715	50.91	79294	{}	\N	2020-10-12 15:12:58.285795
50166	IEP	2020-10-08	50.78	52.15	50.78	52.02	99555	{}	\N	2020-10-12 15:12:58.289795
50167	IEP	2020-10-09	52.089	52.48	51.21	51.31	92564	{}	\N	2020-10-12 15:12:58.293814
50168	IEP	2020-10-12	51.25	51.78	50.84	51.03	28833	{}	\N	2020-10-12 15:12:58.298036
50169	IGEB	2020-09-28	54.4	54.5	54.4	54.5	11106	{}	\N	2020-10-12 15:12:59.958055
50170	IGEB	2020-09-29	54.5206	54.58	54.5	54.5	12276	{}	\N	2020-10-12 15:12:59.96933
50171	IGEB	2020-09-30	54.44	54.44	54.365	54.44	14087	{}	\N	2020-10-12 15:12:59.975941
50172	IGEB	2020-10-01	54.25	54.4525	54.25	54.4525	11488	{}	\N	2020-10-12 15:12:59.981384
50173	IGEB	2020-10-02	54.3152	54.4297	54.3152	54.4297	13125	{}	\N	2020-10-12 15:12:59.989071
50174	IGEB	2020-10-05	54.3	54.349	54.2534	54.2711	9231	{}	\N	2020-10-12 15:12:59.993861
50175	IGEB	2020-10-06	54.315	54.44	54.2462	54.27	19577	{}	\N	2020-10-12 15:12:59.997934
50176	IGEB	2020-10-07	54.34	54.45	54.3363	54.3363	8929	{}	\N	2020-10-12 15:13:00.002175
50177	IGEB	2020-10-08	54.44	54.495	54.44	54.495	5479	{}	\N	2020-10-12 15:13:00.006657
50178	IGEB	2020-10-09	54.46	54.59	54.46	54.585	30689	{}	\N	2020-10-12 15:13:00.010883
50179	IGEB	2020-10-12	54.63	54.7051	54.63	54.7051	1461	{}	\N	2020-10-12 15:13:00.014877
50180	IGN	2020-09-28	46.39	46.6699	46.37	46.55	3112	{}	\N	2020-10-12 15:13:01.200042
50181	IGN	2020-09-29	46.38	47.0831	46.38	46.88	3579	{}	\N	2020-10-12 15:13:01.208504
50182	IGN	2020-09-30	46.74	46.89	46.42	46.62	7206	{}	\N	2020-10-12 15:13:01.215078
50183	IGN	2020-10-01	46.675	46.87	46.54	46.87	237322	{}	\N	2020-10-12 15:13:01.220502
50184	IGN	2020-10-02	45.83	46.78	45.83	46.78	10875	{}	\N	2020-10-12 15:13:01.225899
50185	IGN	2020-10-05	47.32	47.84	47.22	47.84	4904	{}	\N	2020-10-12 15:13:01.230725
50186	IGN	2020-10-06	47.94	48.36	47.62	47.62	4260	{}	\N	2020-10-12 15:13:01.234771
50187	IGN	2020-10-07	47.85	48.4462	47.85	48.4462	2543	{}	\N	2020-10-12 15:13:01.239151
50188	IGN	2020-10-08	49.02	49.4003	49.02	49.4003	2639	{}	\N	2020-10-12 15:13:01.243373
50189	IGN	2020-10-09	49.84	50.13	49.84	49.8687	1909	{}	\N	2020-10-12 15:13:01.247706
50190	IGN	2020-10-12	50.13	50.13	49.8001	49.8001	1468	{}	\N	2020-10-12 15:13:01.251872
50191	IHI	2020-09-28	294.39	296.2981	293.06	294.34	121296	{}	\N	2020-10-12 15:13:02.181709
50192	IHI	2020-09-29	295.52	298.197	294.07	294.815	172760	{}	\N	2020-10-12 15:13:02.190918
50193	IHI	2020-09-30	296.7432	302.775	295.61	299.81	152889	{}	\N	2020-10-12 15:13:02.197389
50194	IHI	2020-10-01	302	304.29	297.78	298.88	191583	{}	\N	2020-10-12 15:13:02.20279
50195	IHI	2020-10-02	295.46	298.4	294	295.44	190307	{}	\N	2020-10-12 15:13:02.207276
50196	IHI	2020-10-05	297.1	302.18	297.1	302.04	112927	{}	\N	2020-10-12 15:13:02.211689
50197	IHI	2020-10-06	303.04	304.6	299.245	299.46	97186	{}	\N	2020-10-12 15:13:02.215454
50198	IHI	2020-10-07	301.91	306.6629	301.91	305.64	162484	{}	\N	2020-10-12 15:13:02.219249
50199	IHI	2020-10-08	307.89	308.19	305.3961	307.45	142184	{}	\N	2020-10-12 15:13:02.223021
50200	IHI	2020-10-09	308.95	311.95	308.7808	311.64	129481	{}	\N	2020-10-12 15:13:02.226637
50201	IHI	2020-10-12	313	314.9676	311.99	314.94	40495	{}	\N	2020-10-12 15:13:02.230296
50202	IIVI	2020-09-28	40.86	41.36	40.37	41.21	935136	{}	\N	2020-10-12 15:13:03.53257
50203	IIVI	2020-09-29	41.22	41.99	40.36	40.99	930733	{}	\N	2020-10-12 15:13:03.544571
50204	IIVI	2020-09-30	41	41.76	40.21	40.56	1239439	{}	\N	2020-10-12 15:13:03.552931
50205	IIVI	2020-10-01	41.245	41.55	40.4853	41.54	708792	{}	\N	2020-10-12 15:13:03.559669
50206	IIVI	2020-10-02	40.33	42.13	40.17	41.75	586515	{}	\N	2020-10-12 15:13:03.56441
50207	IIVI	2020-10-05	42.52	43.1	42.02	42.96	913101	{}	\N	2020-10-12 15:13:03.569432
50208	IIVI	2020-10-06	43.22	44.14	42.32	42.54	823937	{}	\N	2020-10-12 15:13:03.573612
50209	IIVI	2020-10-07	43.86	44.67	43.63	43.71	1137699	{}	\N	2020-10-12 15:13:03.577373
50210	IIVI	2020-10-08	44.1	44.67	43.93	44.56	480875	{}	\N	2020-10-12 15:13:03.581121
50211	IIVI	2020-10-09	44.9	45.91	44.9	45.45	951997	{}	\N	2020-10-12 15:13:03.584973
50212	IIVI	2020-10-12	45.79	46.21	45	45.29	265679	{}	\N	2020-10-12 15:13:03.589093
50213	ILPT	2020-09-28	21.39	22.015	21.39	21.93	224870	{}	\N	2020-10-12 15:13:04.786622
50214	ILPT	2020-09-29	21.74	21.925	21.3901	21.7	167969	{}	\N	2020-10-12 15:13:04.797484
50215	ILPT	2020-09-30	21.75	22.22	21.705	21.87	303376	{}	\N	2020-10-12 15:13:04.805107
50216	ILPT	2020-10-01	21.9	22.3	21.55	22.29	219766	{}	\N	2020-10-12 15:13:04.811252
50217	ILPT	2020-10-02	21.9	22.9	21.53	22.79	173611	{}	\N	2020-10-12 15:13:04.816407
50218	ILPT	2020-10-05	22.93	23	22.17	22.2	281713	{}	\N	2020-10-12 15:13:04.821301
50219	ILPT	2020-10-06	22.35	22.79	22.08	22.33	329265	{}	\N	2020-10-12 15:13:04.825884
50220	ILPT	2020-10-07	22.49	22.58	21.96	22	280944	{}	\N	2020-10-12 15:13:04.830122
50221	ILPT	2020-10-08	22.15	22.51	22	22.34	280965	{}	\N	2020-10-12 15:13:04.834509
50222	ILPT	2020-10-09	22.42	22.45	22	22.15	243367	{}	\N	2020-10-12 15:13:04.838693
50223	ILPT	2020-10-12	22.25	22.7	22.05	22.57	43415	{}	\N	2020-10-12 15:13:04.84287
50224	IMMU	2020-09-28	85.54	85.57	85.25	85.52	3588734	{}	\N	2020-10-12 15:13:06.015364
50225	IMMU	2020-09-29	85.4	85.48	85.16	85.25	3376696	{}	\N	2020-10-12 15:13:06.026694
50226	IMMU	2020-09-30	85.21	85.33	84.84	85.03	5594814	{}	\N	2020-10-12 15:13:06.037392
50227	IMMU	2020-10-01	85.16	85.35	84.9	84.92	3111431	{}	\N	2020-10-12 15:13:06.046627
50228	IMMU	2020-10-02	84.95	85.3	84.9	85.08	3292164	{}	\N	2020-10-12 15:13:06.052477
50229	IMMU	2020-10-05	85.16	85.87	85.03	85.87	3683238	{}	\N	2020-10-12 15:13:06.058629
50230	IMMU	2020-10-06	85.6	85.85	85.5	85.61	2712363	{}	\N	2020-10-12 15:13:06.06513
50231	IMMU	2020-10-07	85.72	85.79	85.36	85.36	2975536	{}	\N	2020-10-12 15:13:06.07123
50232	IMMU	2020-10-08	85.45	85.75	85.41	85.73	2035661	{}	\N	2020-10-12 15:13:06.076805
50233	IMMU	2020-10-09	85.74	86.05	85.67	86	3283738	{}	\N	2020-10-12 15:13:06.081325
50234	IMMU	2020-10-12	86.2	86.49	86.06	86.06	1145557	{}	\N	2020-10-12 15:13:06.085826
50235	ICFI	2020-09-28	63.63	64.04	62.93	63.75	78543	{}	\N	2020-10-12 15:13:09.740376
50236	ICFI	2020-09-29	63.74	63.99	61.08	62.2	67745	{}	\N	2020-10-12 15:13:09.748458
50237	ICFI	2020-09-30	62.48	63.13	61.11	61.53	106073	{}	\N	2020-10-12 15:13:09.754039
50238	ICFI	2020-10-01	61.53	62.35	61.06	61.67	101990	{}	\N	2020-10-12 15:13:09.759247
50239	ICFI	2020-10-02	60.69	63.24	60.02	62.92	94728	{}	\N	2020-10-12 15:13:09.764378
50240	ICFI	2020-10-05	63.41	63.93	62.86	63.72	53193	{}	\N	2020-10-12 15:13:09.769391
50241	ICFI	2020-10-06	63.7	66.13	63.51	64.66	103098	{}	\N	2020-10-12 15:13:09.773346
50242	ICFI	2020-10-07	65.17	66.71	64.76	66.68	107899	{}	\N	2020-10-12 15:13:09.777032
50243	ICFI	2020-10-08	67.42	68.03	66.53	67.57	79604	{}	\N	2020-10-12 15:13:09.780727
50244	ICFI	2020-10-09	67.99	69.23	66.74	67.29	81312	{}	\N	2020-10-12 15:13:09.784563
50245	ICFI	2020-10-12	67.24	68.27	66.75	67.4	58490	{}	\N	2020-10-12 15:13:09.788381
50246	ICPT	2020-09-28	40.61	41.19	39.84	40.53	475573	{}	\N	2020-10-12 15:13:11.435834
50247	ICPT	2020-09-29	40.48	41.925	40.48	41.78	500767	{}	\N	2020-10-12 15:13:11.448705
50248	ICPT	2020-09-30	41.7254	42.9	41.05	41.46	378102	{}	\N	2020-10-12 15:13:11.457184
50249	ICPT	2020-10-01	41.6	42.58	41.4	42.08	346928	{}	\N	2020-10-12 15:13:11.462727
50250	ICPT	2020-10-02	41.01	42.02	40.67	41.13	291992	{}	\N	2020-10-12 15:13:11.467376
50251	ICPT	2020-10-05	41.72	43.05	41.47	42.47	496433	{}	\N	2020-10-12 15:13:11.473013
50252	ICPT	2020-10-06	42.78	43.16	40.4	40.51	558179	{}	\N	2020-10-12 15:13:11.477348
50253	ICPT	2020-10-07	40.45	41.575	40.45	40.99	286635	{}	\N	2020-10-12 15:13:11.481423
50254	ICPT	2020-10-08	41.5	41.82	37.58	37.69	2618339	{}	\N	2020-10-12 15:13:11.485663
50255	ICPT	2020-10-09	37.88	38.235	36.5	36.67	689164	{}	\N	2020-10-12 15:13:11.49006
50256	ICPT	2020-10-12	37.26	37.26	34.92	35.06	301224	{}	\N	2020-10-12 15:13:11.494241
50257	IDLV	2020-09-28	28.62	28.7	28.585	28.68	166924	{}	\N	2020-10-12 15:13:12.774564
50258	IDLV	2020-09-29	28.59	28.65	28.495	28.57	77384	{}	\N	2020-10-12 15:13:12.784355
50259	IDLV	2020-09-30	28.44	28.56	28.38	28.42	132928	{}	\N	2020-10-12 15:13:12.790568
50260	IDLV	2020-10-01	28.5	28.6	28.45	28.56	103184	{}	\N	2020-10-12 15:13:12.796087
50261	IDLV	2020-10-02	28.3	28.52	28.3	28.44	138035	{}	\N	2020-10-12 15:13:12.800858
50262	IDLV	2020-10-05	28.61	28.7299	28.489	28.71	88704	{}	\N	2020-10-12 15:13:12.808001
50263	IDLV	2020-10-06	28.66	28.7099	28.44	28.48	99625	{}	\N	2020-10-12 15:13:12.812572
50264	IDLV	2020-10-07	28.55	28.608	28.43	28.56	69202	{}	\N	2020-10-12 15:13:12.816543
50265	IDLV	2020-10-08	28.57	28.62	28.53	28.58	131633	{}	\N	2020-10-12 15:13:12.820562
50266	IDLV	2020-10-09	28.62	28.73	28.61	28.7	71819	{}	\N	2020-10-12 15:13:12.824743
50267	IDLV	2020-10-12	28.73	28.825	28.7	28.825	16863	{}	\N	2020-10-12 15:13:12.829526
50268	IDXG	2020-09-28	3.1	3.25	3.09	3.15	42767	{}	\N	2020-10-12 15:13:13.801949
50269	IDXG	2020-09-29	3.15	3.35	3.15	3.32	63783	{}	\N	2020-10-12 15:13:13.811732
50270	IDXG	2020-09-30	3.31	3.31	3.07	3.09	68202	{}	\N	2020-10-12 15:13:13.818574
50271	IDXG	2020-10-01	3.13	3.1499	2.97	3.06	46737	{}	\N	2020-10-12 15:13:13.824447
50272	IDXG	2020-10-02	2.99	3.09	2.99	3.05	17938	{}	\N	2020-10-12 15:13:13.829521
50273	IDXG	2020-10-05	3.05	3.133	2.99	3	27416	{}	\N	2020-10-12 15:13:13.835067
50274	IDXG	2020-10-06	2.99	3.085	2.91	2.92	42923	{}	\N	2020-10-12 15:13:13.839362
50275	IDXG	2020-10-07	2.92	3.05	2.92	3.02	34327	{}	\N	2020-10-12 15:13:13.843589
50276	IDXG	2020-10-08	3.03	3.1799	3	3.02	27527	{}	\N	2020-10-12 15:13:13.847804
50277	IDXG	2020-10-09	3.06	3.0796	3	3.04	26365	{}	\N	2020-10-12 15:13:13.852068
50278	IDXG	2020-10-12	3.06	3.065	2.93	2.97	72975	{}	\N	2020-10-12 15:13:13.856179
50279	IEUS	2020-09-28	51.06	51.1	50.86	51.1	3205	{}	\N	2020-10-12 15:13:15.981135
50280	IEUS	2020-09-29	51.245	51.31	51.245	51.31	583	{}	\N	2020-10-12 15:13:15.986292
50281	IEUS	2020-09-30	51.27	51.77	51.27	51.3101	6724	{}	\N	2020-10-12 15:13:15.99096
50282	IEUS	2020-10-01	51.97	51.9962	51.86	51.91	3951	{}	\N	2020-10-12 15:13:15.995794
50283	IEUS	2020-10-02	51.8	52	51.7	51.97	15344	{}	\N	2020-10-12 15:13:16.001002
50284	IEUS	2020-10-05	52.64	52.9045	52.5925	52.9045	4063	{}	\N	2020-10-12 15:13:16.005926
50285	IEUS	2020-10-06	53.2	53.2	52.5585	52.5585	1687	{}	\N	2020-10-12 15:13:16.010548
50286	IEUS	2020-10-07	52.77	52.95	52.77	52.89	927	{}	\N	2020-10-12 15:13:16.015544
50287	IEUS	2020-10-08	53.25	53.25	53.25	53.25	475	{}	\N	2020-10-12 15:13:16.020557
50288	IEUS	2020-10-09	53.65	53.8818	53.65	53.8818	1933	{}	\N	2020-10-12 15:13:16.025336
50289	IEUS	2020-10-12	54.11	54.5	54.02	54.272	19071	{}	\N	2020-10-12 15:13:16.031663
50290	IGF	2020-09-28	38.6678	38.815	38.54	38.7	415123	{}	\N	2020-10-12 15:13:17.002259
50291	IGF	2020-09-29	38.68	38.89	38.49	38.69	482525	{}	\N	2020-10-12 15:13:17.013271
50292	IGF	2020-09-30	38.75	38.94	38.49	38.58	3264093	{}	\N	2020-10-12 15:13:17.02412
50293	IGF	2020-10-01	38.64	38.87	38.53	38.81	214657	{}	\N	2020-10-12 15:13:17.030125
50294	IGF	2020-10-02	38.26	39.1	38.26	39.02	209090	{}	\N	2020-10-12 15:13:17.036794
50295	IGF	2020-10-05	39.25	39.58	39.141	39.55	191155	{}	\N	2020-10-12 15:13:17.043241
50296	IGF	2020-10-06	39.79	39.9	39.28	39.36	216374	{}	\N	2020-10-12 15:13:17.051096
50297	IGF	2020-10-07	39.65	39.82	39.52	39.72	685692	{}	\N	2020-10-12 15:13:17.058791
50298	IGF	2020-10-08	39.82	40.37	39.82	40.33	171235	{}	\N	2020-10-12 15:13:17.066559
50299	IGF	2020-10-09	40.46	40.55	40.17	40.37	328985	{}	\N	2020-10-12 15:13:17.072441
50300	IGF	2020-10-12	40.42	40.53	40.38	40.515	62499	{}	\N	2020-10-12 15:13:17.079726
50301	IGR	2020-09-28	5.99	6.01	5.95	5.95	289196	{}	\N	2020-10-12 15:13:18.349455
50302	IGR	2020-09-29	5.97	5.99	5.9	5.93	453788	{}	\N	2020-10-12 15:13:18.358817
50303	IGR	2020-09-30	5.97	6.07	5.96	5.98	905012	{}	\N	2020-10-12 15:13:18.365658
50304	IGR	2020-10-01	5.96	6.13	5.96	6.1	785417	{}	\N	2020-10-12 15:13:18.371278
50305	IGR	2020-10-02	6.05	6.17	6.05	6.16	500740	{}	\N	2020-10-12 15:13:18.375999
50306	IGR	2020-10-05	6.18	6.23	6.12	6.23	530680	{}	\N	2020-10-12 15:13:18.380552
50307	IGR	2020-10-06	6.21	6.235	6.16	6.16	757486	{}	\N	2020-10-12 15:13:18.386095
50308	IGR	2020-10-07	6.18	6.235	6.17	6.19	878627	{}	\N	2020-10-12 15:13:18.390344
50309	IGR	2020-10-08	6.21	6.28	6.2003	6.28	458131	{}	\N	2020-10-12 15:13:18.394461
50310	IGR	2020-10-09	6.29	6.29	6.24	6.24	589590	{}	\N	2020-10-12 15:13:18.398622
50311	IGR	2020-10-12	6.27	6.29	6.243	6.253	164006	{}	\N	2020-10-12 15:13:18.402896
50312	IHE	2020-09-28	162.22	162.581	161.441	161.96	3500	{}	\N	2020-10-12 15:13:20.370562
50313	IHE	2020-09-29	161.7265	162.6038	161.4498	161.4498	9074	{}	\N	2020-10-12 15:13:20.387527
50314	IHE	2020-09-30	162.2382	164.11	162.2382	163.3799	3086	{}	\N	2020-10-12 15:13:20.394913
50315	IHE	2020-10-01	163.59	163.59	162.4869	162.4869	7188	{}	\N	2020-10-12 15:13:20.401382
50316	IHE	2020-10-02	160.45	162.7426	160.45	162.7426	4751	{}	\N	2020-10-12 15:13:20.408068
50317	IHE	2020-10-05	165.52	167.08	165.52	167.08	4173	{}	\N	2020-10-12 15:13:20.413872
50318	IHE	2020-10-06	167.59	167.59	164.9	165.04	2701	{}	\N	2020-10-12 15:13:20.419957
50319	IHE	2020-10-07	166.0289	168.232	165.852	168.232	3687	{}	\N	2020-10-12 15:13:20.425128
50320	IHE	2020-10-08	169.33	170.67	169.33	170.67	4103	{}	\N	2020-10-12 15:13:20.429134
50321	IHE	2020-10-09	170.94	172.42	170.94	171.92	4142	{}	\N	2020-10-12 15:13:20.433806
50322	IHE	2020-10-12	172.22	172.31	171.89	172.31	1159	{}	\N	2020-10-12 15:13:20.438201
50323	IHY	2020-09-28	24.21	24.25	24.17	24.2	40715	{}	\N	2020-10-12 15:13:21.437558
50324	IHY	2020-09-29	24.29	24.35	24.28	24.32	35540	{}	\N	2020-10-12 15:13:21.444533
50325	IHY	2020-09-30	24.33	24.44	24.33	24.44	18871	{}	\N	2020-10-12 15:13:21.450907
50326	IHY	2020-10-01	24.41	24.44	24.31	24.35	37991	{}	\N	2020-10-12 15:13:21.455784
50327	IHY	2020-10-02	24.3	24.3	24.25	24.27	24662	{}	\N	2020-10-12 15:13:21.460736
50328	IHY	2020-10-05	24.36	24.49	24.36	24.43	87165	{}	\N	2020-10-12 15:13:21.465684
50329	IHY	2020-10-06	24.52	24.535	24.4727	24.4727	41636	{}	\N	2020-10-12 15:13:21.47012
50330	IHY	2020-10-07	24.6	24.6	24.51	24.51	11559	{}	\N	2020-10-12 15:13:21.47419
50331	IHY	2020-10-08	24.5783	24.66	24.5433	24.625	29560	{}	\N	2020-10-12 15:13:21.478477
50332	IHY	2020-10-09	24.6769	24.7236	24.6456	24.71	10640	{}	\N	2020-10-12 15:13:21.482614
50333	IHY	2020-10-12	24.74	24.74	24.7115	24.7115	964	{}	\N	2020-10-12 15:13:21.487485
50334	IJH	2020-09-28	183.32	185.73	183.32	185.27	1423682	{}	\N	2020-10-12 15:13:22.810143
50335	IJH	2020-09-29	185.18	185.8	182.96	184.14	849866	{}	\N	2020-10-12 15:13:22.81917
50336	IJH	2020-09-30	185	187.19	183.89	185.31	1231299	{}	\N	2020-10-12 15:13:22.825003
50337	IJH	2020-10-01	186.26	187.79	185.53	187.77	1297401	{}	\N	2020-10-12 15:13:22.830543
50338	IJH	2020-10-02	184.66	190.33	184.12	189.56	968922	{}	\N	2020-10-12 15:13:22.836479
50339	IJH	2020-10-05	191.24	193.98	191.19	193.86	931292	{}	\N	2020-10-12 15:13:22.841081
50340	IJH	2020-10-06	195	197.61	192.56	193.06	1540167	{}	\N	2020-10-12 15:13:22.845288
50341	IJH	2020-10-07	195.14	197.01	195.08	196.35	709706	{}	\N	2020-10-12 15:13:22.84943
50342	IJH	2020-10-08	198.14	198.5	196.96	198.42	631426	{}	\N	2020-10-12 15:13:22.854202
50343	IJH	2020-10-09	199.97	200.47	198.357	199.01	670577	{}	\N	2020-10-12 15:13:22.858711
50344	IJH	2020-10-12	199.78	200.3335	199.01	200.25	189373	{}	\N	2020-10-12 15:13:22.862923
50345	ICON	2020-09-28	0.7222	0.77	0.7222	0.751	60074	{}	\N	2020-10-12 15:13:23.781071
50346	ICON	2020-09-29	0.75	0.78	0.75	0.75525	304151	{}	\N	2020-10-12 15:13:23.793691
50347	ICON	2020-09-30	0.7	0.748	0.6965	0.713	236960	{}	\N	2020-10-12 15:13:23.801037
50348	ICON	2020-10-01	0.704	0.73	0.6805	0.7	138746	{}	\N	2020-10-12 15:13:23.80689
50349	ICON	2020-10-02	0.7	0.71	0.69	0.7003	61608	{}	\N	2020-10-12 15:13:23.81324
50350	ICON	2020-10-05	0.699	0.713199	0.6856	0.6901	102888	{}	\N	2020-10-12 15:13:23.819786
50351	ICON	2020-10-06	0.666	0.72	0.666	0.6983	91642	{}	\N	2020-10-12 15:13:23.824952
50352	ICON	2020-10-07	0.7099	0.769	0.698	0.701	186064	{}	\N	2020-10-12 15:13:23.829698
50353	ICON	2020-10-08	0.7122	0.747	0.71	0.7361	115355	{}	\N	2020-10-12 15:13:23.834933
50354	ICON	2020-10-09	0.73	0.755999	0.7131	0.7264	134966	{}	\N	2020-10-12 15:13:23.839857
50355	ICON	2020-10-12	0.7627	0.989	0.7348	0.9098	3111108	{}	\N	2020-10-12 15:13:23.844655
50356	IDE	2020-09-28	9.45	9.5727	9.45	9.52	93017	{}	\N	2020-10-12 15:13:24.7503
50357	IDE	2020-09-29	9.55	9.5899	9.46	9.5	104263	{}	\N	2020-10-12 15:13:24.760548
50358	IDE	2020-09-30	9.49	9.7	9.44	9.51	259970	{}	\N	2020-10-12 15:13:24.768681
50359	IDE	2020-10-01	9.41	9.41	9.29	9.32	194826	{}	\N	2020-10-12 15:13:24.774285
50360	IDE	2020-10-02	9.21	9.37	9.21	9.36	79452	{}	\N	2020-10-12 15:13:24.778785
50361	IDE	2020-10-05	9.42	9.53	9.42	9.49	117374	{}	\N	2020-10-12 15:13:24.783836
50362	IDE	2020-10-06	9.53	9.58	9.461	9.52	143917	{}	\N	2020-10-12 15:13:24.788914
50363	IDE	2020-10-07	9.59	9.6718	9.59	9.63	172478	{}	\N	2020-10-12 15:13:24.794469
50364	IDE	2020-10-08	9.67	9.72	9.6	9.63	84919	{}	\N	2020-10-12 15:13:24.800054
50365	IDE	2020-10-09	9.65	9.81	9.65	9.77	203978	{}	\N	2020-10-12 15:13:24.806896
50366	IDE	2020-10-12	9.77	9.8899	9.77	9.87	86866	{}	\N	2020-10-12 15:13:24.81326
50367	IDXX	2020-09-28	381.81	387.68	380.16	386.37	422465	{}	\N	2020-10-12 15:13:27.260611
50368	IDXX	2020-09-29	388.22	395.21	385	385.06	288473	{}	\N	2020-10-12 15:13:27.272767
50369	IDXX	2020-09-30	383.45	397.26	383.45	393.11	496318	{}	\N	2020-10-12 15:13:27.282658
50370	IDXX	2020-10-01	396.29	398.155	391.88	394.71	292682	{}	\N	2020-10-12 15:13:27.289135
50371	IDXX	2020-10-02	389.7	393.82	382.68	383.85	297330	{}	\N	2020-10-12 15:13:27.294023
50372	IDXX	2020-10-05	385.27	392.83	385.27	392.37	285946	{}	\N	2020-10-12 15:13:27.298371
50373	IDXX	2020-10-06	391.3382	393.54	383.12	384.31	269670	{}	\N	2020-10-12 15:13:27.302753
50374	IDXX	2020-10-07	388.02	391.55	385.26	386.23	278257	{}	\N	2020-10-12 15:13:27.306641
50375	IDXX	2020-10-08	388.94	394.39	385.81	393.51	198678	{}	\N	2020-10-12 15:13:27.310581
50376	IDXX	2020-10-09	395	401.96	393.02	400.65	271935	{}	\N	2020-10-12 15:13:27.314943
50377	IDXX	2020-10-12	404.88	414.935	404.88	414.3101	77331	{}	\N	2020-10-12 15:13:27.319172
50378	IEZ	2020-09-28	7.36	7.62	7.36	7.5	115439	{}	\N	2020-10-12 15:13:28.509519
50379	IEZ	2020-09-29	7.45	7.45	7.15	7.25	52636	{}	\N	2020-10-12 15:13:28.518125
50380	IEZ	2020-09-30	7.3327	7.45	7.17	7.22	90896	{}	\N	2020-10-12 15:13:28.524408
50381	IEZ	2020-10-01	7.15	7.15	6.88	6.95	155528	{}	\N	2020-10-12 15:13:28.529804
50382	IEZ	2020-10-02	6.69	7.19	6.66	7.13	91024	{}	\N	2020-10-12 15:13:28.534852
50383	IEZ	2020-10-05	7.26	7.4	7.24	7.39	76411	{}	\N	2020-10-12 15:13:28.539304
50384	IEZ	2020-10-06	7.53	7.62	7.25	7.29	198595	{}	\N	2020-10-12 15:13:28.5431
50385	IEZ	2020-10-07	7.31	7.375	7.19	7.34	190894	{}	\N	2020-10-12 15:13:28.546643
50386	IEZ	2020-10-08	7.45	7.7262	7.45	7.71	124758	{}	\N	2020-10-12 15:13:28.550399
50387	IEZ	2020-10-09	7.78	7.78	7.55	7.6	95926	{}	\N	2020-10-12 15:13:28.554093
50388	IEZ	2020-10-12	7.57	7.57	7.43	7.4473	42404	{}	\N	2020-10-12 15:13:28.557669
50389	IGHG	2020-09-28	70.78	71.35	70.78	71.259	7052	{}	\N	2020-10-12 15:13:30.259446
50390	IGHG	2020-09-29	71.22	71.35	71.21	71.28	10040	{}	\N	2020-10-12 15:13:30.268765
50391	IGHG	2020-09-30	71.35	71.56	71.02	71.44	28628	{}	\N	2020-10-12 15:13:30.275287
50392	IGHG	2020-10-01	71.49	71.71	71.03	71.35	75610	{}	\N	2020-10-12 15:13:30.280662
50393	IGHG	2020-10-02	70.4	71.67	70.4	71.67	6253	{}	\N	2020-10-12 15:13:30.28567
50394	IGHG	2020-10-05	71.777	72.1	71.6601	72.09	6744	{}	\N	2020-10-12 15:13:30.290013
50395	IGHG	2020-10-06	72	72.16	71.12	71.12	19442	{}	\N	2020-10-12 15:13:30.29372
50396	IGHG	2020-10-07	71.45	72.3258	71.45	72.11	4526	{}	\N	2020-10-12 15:13:30.297455
50397	IGHG	2020-10-08	72.2715	72.4	72.1524	72.2601	61777	{}	\N	2020-10-12 15:13:30.301778
50398	IGHG	2020-10-09	72.07	72.51	71.56	72.51	7942	{}	\N	2020-10-12 15:13:30.306351
50399	IGHG	2020-10-12	72.49	72.5	72.49	72.5	1963	{}	\N	2020-10-12 15:13:30.310226
50400	IGRO	2020-09-28	54.645	55.1	54.56	54.8	11658	{}	\N	2020-10-12 15:13:31.287591
50401	IGRO	2020-09-29	54.88	55.29	54.52	54.8121	2948	{}	\N	2020-10-12 15:13:31.297723
50402	IGRO	2020-09-30	54.63	54.91	54.4146	54.4146	6770	{}	\N	2020-10-12 15:13:31.303703
50403	IGRO	2020-10-01	54.92	54.92	54.6229	54.685	3329	{}	\N	2020-10-12 15:13:31.308968
50404	IGRO	2020-10-02	53.92	54.91	53.92	54.7934	8665	{}	\N	2020-10-12 15:13:31.313336
50405	IGRO	2020-10-05	55.5407	55.6384	55.245	55.5129	5304	{}	\N	2020-10-12 15:13:31.317758
50406	IGRO	2020-10-06	55.61	55.783	54.96	55.1749	9704	{}	\N	2020-10-12 15:13:31.321992
50407	IGRO	2020-10-07	55.51	55.88	55.51	55.7693	13925	{}	\N	2020-10-12 15:13:31.325881
50408	IGRO	2020-10-08	56.105	56.43	56.0038	56.205	2709	{}	\N	2020-10-12 15:13:31.329693
50409	IGRO	2020-10-09	56.55	56.55	56.425	56.4582	2321	{}	\N	2020-10-12 15:13:31.336377
50410	IGRO	2020-10-12	56.76	56.76	56.74	56.74	2689	{}	\N	2020-10-12 15:13:31.34115
50411	IHF	2020-09-28	197.54	198.93	196.41	196.74	10674	{}	\N	2020-10-12 15:13:32.287559
50412	IHF	2020-09-29	196.74	197.9599	195.6639	197.34	13994	{}	\N	2020-10-12 15:13:32.296157
50413	IHF	2020-09-30	197.87	203.35	197.87	201.82	21437	{}	\N	2020-10-12 15:13:32.302681
50414	IHF	2020-10-01	202.77	203.04	200.45	202.12	40977	{}	\N	2020-10-12 15:13:32.307703
50415	IHF	2020-10-02	198.73	204.6	198.73	203.03	10837	{}	\N	2020-10-12 15:13:32.312492
50416	IHF	2020-10-05	205.15	208.09	205.08	208.05	16652	{}	\N	2020-10-12 15:13:32.316549
50417	IHF	2020-10-06	208.73	210.13	206.03	206.03	14340	{}	\N	2020-10-12 15:13:32.320549
50418	IHF	2020-10-07	208.355	210.96	207.99	210.3632	12192	{}	\N	2020-10-12 15:13:32.324266
50419	IHF	2020-10-08	211.61	211.61	210.154	211.09	15107	{}	\N	2020-10-12 15:13:32.328401
50420	IHF	2020-10-09	212.89	213.81	211.26	211.69	16804	{}	\N	2020-10-12 15:13:32.332287
50421	IHF	2020-10-12	213.19	214.12	213.0423	214.04	5001	{}	\N	2020-10-12 15:13:32.336404
50422	IID	2020-09-28	4.32	4.41	4.3	4.41	41049	{}	\N	2020-10-12 15:13:33.411275
50423	IID	2020-09-29	4.39	4.43	4.39	4.39	52310	{}	\N	2020-10-12 15:13:33.422807
50424	IID	2020-09-30	4.31	4.46	4.31	4.43	152224	{}	\N	2020-10-12 15:13:33.429255
50425	IID	2020-10-01	4.37	4.4146	4.27	4.32	175878	{}	\N	2020-10-12 15:13:33.435467
50426	IID	2020-10-02	4.3	4.3517	4.3	4.35	42263	{}	\N	2020-10-12 15:13:33.440441
50427	IID	2020-10-05	4.34	4.43	4.34	4.42	51206	{}	\N	2020-10-12 15:13:33.444948
50428	IID	2020-10-06	4.43	4.465	4.34	4.34	38810	{}	\N	2020-10-12 15:13:33.448911
50429	IID	2020-10-07	4.39	4.42	4.33	4.42	68578	{}	\N	2020-10-12 15:13:33.453871
50430	IID	2020-10-08	4.41	4.43	4.35	4.36	30470	{}	\N	2020-10-12 15:13:33.459018
50431	IID	2020-10-09	4.37	4.4108	4.35	4.4	75566	{}	\N	2020-10-12 15:13:33.46414
50432	IID	2020-10-12	4.41	4.44	4.41	4.44	3400	{}	\N	2020-10-12 15:13:33.470297
50433	IJK	2020-09-28	236.24	239.02	235.935	238.57	51856	{}	\N	2020-10-12 15:13:34.604272
50434	IJK	2020-09-29	238.6	239.79	237.11	237.94	34479	{}	\N	2020-10-12 15:13:34.614967
50435	IJK	2020-09-30	238.39	241.51	238.1	239.18	111193	{}	\N	2020-10-12 15:13:34.621371
50436	IJK	2020-10-01	240.25	242.52	239.77	242.52	241133	{}	\N	2020-10-12 15:13:34.626673
50437	IJK	2020-10-02	237.53	244.42	237.53	243.48	106658	{}	\N	2020-10-12 15:13:34.631009
50438	IJK	2020-10-05	245.9105	248.8001	245.9105	248.66	77968	{}	\N	2020-10-12 15:13:34.6361
50439	IJK	2020-10-06	249.95	252.878	247.15	247.61	65594	{}	\N	2020-10-12 15:13:34.639993
50440	IJK	2020-10-07	249.8	252.63	249.8	251.81	80359	{}	\N	2020-10-12 15:13:34.643379
50441	IJK	2020-10-08	254.31	254.32	252.715	254.1	37768	{}	\N	2020-10-12 15:13:34.646678
50442	IJK	2020-10-09	255.62	256.6499	255.07	255.67	41287	{}	\N	2020-10-12 15:13:34.650206
50443	IJK	2020-10-12	256.78	257.21	255.75	257.17	26271	{}	\N	2020-10-12 15:13:34.654403
50444	IMH	2020-09-28	1.1527	1.47	1.1527	1.43	232506	{}	\N	2020-10-12 15:13:36.160509
50445	IMH	2020-09-29	1.46	1.4999	1.335	1.39	187514	{}	\N	2020-10-12 15:13:36.166453
50446	IMH	2020-09-30	1.4	1.45	1.27	1.27	46360	{}	\N	2020-10-12 15:13:36.171719
50447	IMH	2020-10-01	1.2699	1.47	1.26	1.38	49322	{}	\N	2020-10-12 15:13:36.176336
50448	IMH	2020-10-02	1.35	1.691	1.35	1.53	396647	{}	\N	2020-10-12 15:13:36.180633
50449	IMH	2020-10-05	1.52	1.5389	1.4	1.47	88784	{}	\N	2020-10-12 15:13:36.185056
50450	IMH	2020-10-06	1.44	1.5668	1.37	1.4	43827	{}	\N	2020-10-12 15:13:36.189002
50451	IMH	2020-10-07	1.49	1.49	1.42	1.45	15115	{}	\N	2020-10-12 15:13:36.19263
50452	IMH	2020-10-08	1.47	1.49	1.47	1.49	25309	{}	\N	2020-10-12 15:13:36.196208
50453	IMH	2020-10-09	1.5	1.59	1.41	1.42	44465	{}	\N	2020-10-12 15:13:36.199985
50454	IMH	2020-10-12	1.42	1.49	1.4198	1.47	15498	{}	\N	2020-10-12 15:13:36.203893
50455	IMOS	2020-09-28	19.07	19.2754	19.07	19.1	6471	{}	\N	2020-10-12 15:13:37.200221
50456	IMOS	2020-09-29	18.47	19.24	18.47	19.03	10515	{}	\N	2020-10-12 15:13:37.209174
50457	IMOS	2020-09-30	19.39	19.39	19.22	19.26	11295	{}	\N	2020-10-12 15:13:37.21647
50458	IMOS	2020-10-01	19.54	19.72	19.37	19.55	15489	{}	\N	2020-10-12 15:13:37.223082
50459	IMOS	2020-10-02	19.3701	19.67	19.2774	19.297	2807	{}	\N	2020-10-12 15:13:37.228219
50460	IMOS	2020-10-05	19.37	19.68	19.37	19.51	3602	{}	\N	2020-10-12 15:13:37.232977
50461	IMOS	2020-10-06	19.75	19.9	19.6201	19.744	7565	{}	\N	2020-10-12 15:13:37.238352
50462	IMOS	2020-10-07	19.82	20	19.82	19.925	13067	{}	\N	2020-10-12 15:13:37.243267
50463	IMOS	2020-10-08	20.141	20.3	19.83	19.85	10821	{}	\N	2020-10-12 15:13:37.247508
50464	IMOS	2020-10-09	20.34	20.44	19.51	19.51	6911	{}	\N	2020-10-12 15:13:37.252135
50465	IMOS	2020-10-12	20.2	20.23	20.1	20.18	3599	{}	\N	2020-10-12 15:13:37.256397
50466	IDMO	2020-09-28	30.01	30.01	29.9512	29.9512	229	{}	\N	2020-10-12 15:13:40.216239
50467	IDMO	2020-09-29	30.23	30.2592	30.23	30.2592	1069	{}	\N	2020-10-12 15:13:40.227145
50468	IDMO	2020-09-30	30.1703	30.1703	30.1095	30.1095	176	{}	\N	2020-10-12 15:13:40.23467
50469	IDMO	2020-10-01	30.41	30.46	30.35	30.45	10249108	{}	\N	2020-10-12 15:13:40.240595
50470	IDMO	2020-10-02	30.22	30.2499	30.11	30.19	11570	{}	\N	2020-10-12 15:13:40.245706
50471	IDMO	2020-10-05	30.27	30.51	30.27	30.43	104695	{}	\N	2020-10-12 15:13:40.250591
50472	IDMO	2020-10-06	30.35	30.35	30.01	30.01	30042	{}	\N	2020-10-12 15:13:40.255002
50473	IDMO	2020-10-07	30.23	30.33	30.16	30.22	40004	{}	\N	2020-10-12 15:13:40.259936
50474	IDMO	2020-10-08	30.32	30.47	30.32	30.39	32957	{}	\N	2020-10-12 15:13:40.264524
50475	IDMO	2020-10-09	30.57	30.83	30.57	30.75	74742	{}	\N	2020-10-12 15:13:40.269221
50476	IDMO	2020-10-12	30.9468	30.98	30.9302	30.9789	824	{}	\N	2020-10-12 15:13:40.273537
50477	IDU	2020-09-28	145.45	145.926	144.52	144.78	17088	{}	\N	2020-10-12 15:13:42.91998
50478	IDU	2020-09-29	145.48	145.91	144.03	144.65	45039	{}	\N	2020-10-12 15:13:42.926701
50479	IDU	2020-09-30	146.13	146.41	144.82	146.02	19574	{}	\N	2020-10-12 15:13:42.932875
50480	IDU	2020-10-01	146.01	147.5438	146.01	147.09	20894	{}	\N	2020-10-12 15:13:42.938892
50481	IDU	2020-10-02	145.77	149.58	145.77	148.93	21808	{}	\N	2020-10-12 15:13:42.944475
50482	IDU	2020-10-05	148.77	151.21	148.77	150.7	20166	{}	\N	2020-10-12 15:13:42.949911
50483	IDU	2020-10-06	150.98	153.85	150.61	152.41	31653	{}	\N	2020-10-12 15:13:42.955465
50484	IDU	2020-10-07	152.77	153.55	151.9196	152.91	19940	{}	\N	2020-10-12 15:13:42.960386
50485	IDU	2020-10-08	153.53	156.04	153.53	155.92	26399	{}	\N	2020-10-12 15:13:42.965004
50486	IDU	2020-10-09	156.4	156.4	154.88	155.72	11837	{}	\N	2020-10-12 15:13:42.969611
50487	IDU	2020-10-12	155.8	156.71	155.57	156.59	12148	{}	\N	2020-10-12 15:13:42.97423
50488	IEO	2020-09-28	27.35	27.73	27.1	27.39	170903	{}	\N	2020-10-12 15:13:44.29865
50489	IEO	2020-09-29	27.36	27.36	26.24	26.63	122514	{}	\N	2020-10-12 15:13:44.309371
50490	IEO	2020-09-30	26.72	27	26.28	26.4	71851	{}	\N	2020-10-12 15:13:44.316551
50491	IEO	2020-10-01	26.13	26.24	25.48	25.62	171025	{}	\N	2020-10-12 15:13:44.322249
50492	IEO	2020-10-02	24.88	26.29	24.4402	26.09	209727	{}	\N	2020-10-12 15:13:44.326651
50493	IEO	2020-10-05	26.54	27.1024	26.37	27.1	113694	{}	\N	2020-10-12 15:13:44.330893
50494	IEO	2020-10-06	27.5	27.75	26.41	26.5	167348	{}	\N	2020-10-12 15:13:44.335315
50495	IEO	2020-10-07	26.7764	27.175	26.48	27.07	99440	{}	\N	2020-10-12 15:13:44.339126
50496	IEO	2020-10-08	27.27	28.11	27.24	28.11	99402	{}	\N	2020-10-12 15:13:44.342842
50497	IEO	2020-10-09	28.29	28.44	27.57	27.75	1155277	{}	\N	2020-10-12 15:13:44.346336
50498	IEO	2020-10-12	27.77	27.77	27.3	27.58	261833	{}	\N	2020-10-12 15:13:44.350326
50499	IFMK	2020-09-28	0.7122	0.7459	0.71	0.7221	44580	{}	\N	2020-10-12 15:13:45.701442
50500	IFMK	2020-09-29	0.71	0.739	0.671	0.70959	113430	{}	\N	2020-10-12 15:13:45.71122
50501	IFMK	2020-09-30	0.718	0.718	0.671	0.6879	111102	{}	\N	2020-10-12 15:13:45.718375
50502	IFMK	2020-10-01	0.685	0.72	0.67	0.69	96344	{}	\N	2020-10-12 15:13:45.723942
50503	IFMK	2020-10-02	0.6722	0.7001	0.67	0.695	661103	{}	\N	2020-10-12 15:13:45.72865
50504	IFMK	2020-10-05	0.688	0.72	0.68	0.716	199112	{}	\N	2020-10-12 15:13:45.732937
50505	IFMK	2020-10-06	0.7174	0.7199	0.697	0.697	88949	{}	\N	2020-10-12 15:13:45.737307
50506	IFMK	2020-10-07	0.78	0.78	0.7	0.7298	348883	{}	\N	2020-10-12 15:13:45.741262
50507	IFMK	2020-10-08	0.7297	0.73	0.7101	0.72	86180	{}	\N	2020-10-12 15:13:45.745383
50508	IFMK	2020-10-09	0.71	0.77	0.71	0.7574	194883	{}	\N	2020-10-12 15:13:45.74959
50509	IFMK	2020-10-12	0.7444	0.755	0.7301	0.75	19496	{}	\N	2020-10-12 15:13:45.753821
50510	IGE	2020-09-28	20.14	20.29	20.02	20.14	199823	{}	\N	2020-10-12 15:13:46.850912
50511	IGE	2020-09-29	20.0878	20.15	19.7	19.82	127124	{}	\N	2020-10-12 15:13:46.86045
50512	IGE	2020-09-30	19.85	20.03	19.71	19.81	183517	{}	\N	2020-10-12 15:13:46.866756
50513	IGE	2020-10-01	19.71	19.83	19.4	19.46	631470	{}	\N	2020-10-12 15:13:46.873215
50514	IGE	2020-10-02	19.1	19.6899	19.1	19.585	754808	{}	\N	2020-10-12 15:13:46.878309
50515	IGE	2020-10-05	19.74	20.06	19.74	20.06	180954	{}	\N	2020-10-12 15:13:46.883066
50516	IGE	2020-10-06	20.26	20.32	19.63	19.66	1123058	{}	\N	2020-10-12 15:13:46.88758
50517	IGE	2020-10-07	19.86	19.98	19.73	19.95	304420	{}	\N	2020-10-12 15:13:46.892262
50518	IGE	2020-10-08	20.08	20.57	20.08	20.57	416691	{}	\N	2020-10-12 15:13:46.896631
50519	IGE	2020-10-09	20.74	20.848	20.45	20.53	254998	{}	\N	2020-10-12 15:13:46.901497
50520	IGE	2020-10-12	20.51	20.55	20.43	20.49	87673	{}	\N	2020-10-12 15:13:46.906124
50521	IGT	2020-09-28	11.4	11.75	11.23	11.55	2341358	{}	\N	2020-10-12 15:13:48.869571
50522	IGT	2020-09-29	11.49	11.55	10.967	11.03	5660075	{}	\N	2020-10-12 15:13:48.879389
50523	IGT	2020-09-30	11.08	11.64	11.06	11.13	5784443	{}	\N	2020-10-12 15:13:48.886939
50524	IGT	2020-10-01	11.32	11.56	11.25	11.39	4784365	{}	\N	2020-10-12 15:13:48.892592
50525	IGT	2020-10-02	10.78	11.75	10.76	11.64	5389938	{}	\N	2020-10-12 15:13:48.897692
50526	IGT	2020-10-05	11.81	11.92	11.61	11.76	3887597	{}	\N	2020-10-12 15:13:48.902711
50527	IGT	2020-10-06	11.9	12.19	11.46	11.51	5526662	{}	\N	2020-10-12 15:13:48.907133
50528	IGT	2020-10-07	11.64	11.85	11.505	11.67	3796021	{}	\N	2020-10-12 15:13:48.911378
50529	IGT	2020-10-08	11.85	12.1	11.67	11.96	3823568	{}	\N	2020-10-12 15:13:48.916087
50530	IGT	2020-10-09	12.04	12.04	11.475	11.51	3050658	{}	\N	2020-10-12 15:13:48.920441
50531	IGT	2020-10-12	11.6	11.6	11.35	11.38	809539	{}	\N	2020-10-12 15:13:48.924928
50532	IHTA	2020-09-28	8.11	8.11	7.99	8.06	6802	{}	\N	2020-10-12 15:13:50.641918
50533	IHTA	2020-09-29	8.0293	8.05	7.94	7.95	29496	{}	\N	2020-10-12 15:13:50.656499
50534	IHTA	2020-09-30	8.11	8.11	7.87	7.91	65070	{}	\N	2020-10-12 15:13:50.661504
50535	IHTA	2020-10-01	7.9993	8.13	7.985	8.13	18900	{}	\N	2020-10-12 15:13:50.666872
50536	IHTA	2020-10-02	8.1	8.13	8.092	8.13	12400	{}	\N	2020-10-12 15:13:50.67191
50537	IHTA	2020-10-05	8.1074	8.1554	8.05	8.05	48074	{}	\N	2020-10-12 15:13:50.676389
50538	IHTA	2020-10-06	8.03	8.1076	8.01	8.05	67149	{}	\N	2020-10-12 15:13:50.680287
50539	IHTA	2020-10-07	8.08	8.18	8.0742	8.18	35134	{}	\N	2020-10-12 15:13:50.684709
50540	IHTA	2020-10-08	8.4	8.4	8.195	8.2	8530	{}	\N	2020-10-12 15:13:50.689477
50541	IHTA	2020-10-09	8.25	8.25	8.0803	8.23	51124	{}	\N	2020-10-12 15:13:50.693889
50542	IIPR	2020-09-28	125.6417	129.4	125.08	128.3	405398	{}	\N	2020-10-12 15:13:53.702682
50543	IIPR	2020-09-29	127.06	128.21	124.25	124.75	1006247	{}	\N	2020-10-12 15:13:53.712671
50544	IIPR	2020-09-30	125.66	126.895	122.1501	124.11	787119	{}	\N	2020-10-12 15:13:53.719014
50545	IIPR	2020-10-01	125	125.49	122.59	125.13	535480	{}	\N	2020-10-12 15:13:53.724704
50546	IIPR	2020-10-02	121.83	125.5	121.78	125.31	1132515	{}	\N	2020-10-12 15:13:53.730205
50547	IIPR	2020-10-05	126.24	127.43	125.34	126.6	622612	{}	\N	2020-10-12 15:13:53.735523
50548	IIPR	2020-10-06	127	127.25	123.32	124.43	493472	{}	\N	2020-10-12 15:13:53.740372
50549	IIPR	2020-10-07	125.95	131.41	124.82	129.96	1731277	{}	\N	2020-10-12 15:13:53.744375
50550	IIPR	2020-10-08	131.3	132.25	128.513	129.78	1962941	{}	\N	2020-10-12 15:13:53.748556
50551	IIPR	2020-10-09	131.05	132.005	130.2584	131.22	547823	{}	\N	2020-10-12 15:13:53.753247
50552	IIPR	2020-10-12	132.7	133.17	130.97	132.2	150783	{}	\N	2020-10-12 15:13:53.758078
50553	ILF	2020-09-28	21.6	21.71	20.79	20.9	1568192	{}	\N	2020-10-12 15:13:55.685632
50554	ILF	2020-09-29	20.87	21.08	20.625	20.63	1218335	{}	\N	2020-10-12 15:13:55.695439
50555	ILF	2020-09-30	20.81	21.19	20.81	21.06	1700624	{}	\N	2020-10-12 15:13:55.703331
50556	ILF	2020-10-01	21.04	21.04	20.7	20.95	1960459	{}	\N	2020-10-12 15:13:55.709335
50557	ILF	2020-10-02	20.77	21.12	20.751	20.81	2262757	{}	\N	2020-10-12 15:13:55.714109
50558	ILF	2020-10-05	21.09	21.55	21.02	21.52	692654	{}	\N	2020-10-12 15:13:55.7193
50559	ILF	2020-10-06	21.67	21.865	21.16	21.23	1164695	{}	\N	2020-10-12 15:13:55.723183
50560	ILF	2020-10-07	21.45	21.54	21.21	21.34	834722	{}	\N	2020-10-12 15:13:55.726822
50561	ILF	2020-10-08	21.4	22.105	21.385	22.09	1710878	{}	\N	2020-10-12 15:13:55.730433
50562	ILF	2020-10-09	22.16	22.52	22.07	22.32	2150045	{}	\N	2020-10-12 15:13:55.734267
50563	ILF	2020-10-12	22.43	22.43	22.28	22.37	294397	{}	\N	2020-10-12 15:13:55.738525
50564	IMRNW	2020-09-28	1.25	1.278	1.2	1.27	7243	{}	\N	2020-10-12 15:13:57.015355
50565	IMRNW	2020-09-29	1.2	1.25	1.2	1.2492	1272	{}	\N	2020-10-12 15:13:57.023504
50566	IMRNW	2020-10-01	1.2045	1.29	1.103	1.29	6219	{}	\N	2020-10-12 15:13:57.029047
50567	IMRNW	2020-10-02	1.15	1.1955	1.15	1.1955	650	{}	\N	2020-10-12 15:13:57.034396
50568	IMRNW	2020-10-05	1.2	1.2507	1.2	1.2507	1648	{}	\N	2020-10-12 15:13:57.039625
50569	IMRNW	2020-10-07	1.25	1.25	1.2135	1.25	2209	{}	\N	2020-10-12 15:13:57.044229
50570	IMRNW	2020-10-08	1.1	1.21	1	1.2	6560	{}	\N	2020-10-12 15:13:57.0484
50571	IMRNW	2020-10-09	1.99	2	1.25	1.4	7264	{}	\N	2020-10-12 15:13:57.052864
50572	IMRNW	2020-10-12	1.644	1.644	1.42	1.6194	7097	{}	\N	2020-10-12 15:13:57.057136
50573	INCY	2020-09-28	87.62	90.45	87.37	90.36	1382376	{}	\N	2020-10-12 15:13:58.538048
50574	INCY	2020-09-29	90.22	90.48	87.26	87.43	1093566	{}	\N	2020-10-12 15:13:58.546557
50575	INCY	2020-09-30	88.18	90.77	87.87	89.74	1228010	{}	\N	2020-10-12 15:13:58.552696
50576	INCY	2020-10-01	90.1	93.08	89.98	92.46	1071378	{}	\N	2020-10-12 15:13:58.558477
50577	INCY	2020-10-02	91.19	91.91	87.25	88.75	1176985	{}	\N	2020-10-12 15:13:58.564322
50578	INCY	2020-10-05	89.26	92.45	89.26	92.3	1070853	{}	\N	2020-10-12 15:13:58.570042
50579	INCY	2020-10-06	91.97	92.34	88.315	88.6	1027749	{}	\N	2020-10-12 15:13:58.574807
50580	INCY	2020-10-07	89.68	91.18	88.62	90.45	912721	{}	\N	2020-10-12 15:13:58.579068
50581	INCY	2020-10-08	90.71	93.22	90.09	92.78	1113519	{}	\N	2020-10-12 15:13:58.583283
50582	INCY	2020-10-09	92.99	96.76	92.63	95.74	1532541	{}	\N	2020-10-12 15:13:58.588232
50583	INCY	2020-10-12	97.03	97.69	95.58	97.24	335734	{}	\N	2020-10-12 15:13:58.592811
50584	KALV	2020-09-28	12.95	13.09	12.75	12.82	19351	{}	\N	2020-10-12 15:14:00.353945
50585	KALV	2020-09-29	12.44	13.16	12.44	12.63	19748	{}	\N	2020-10-12 15:14:00.362968
50586	KALV	2020-09-30	12.64	13.13	12.06	12.59	39883	{}	\N	2020-10-12 15:14:00.375654
50587	KALV	2020-10-01	12.48	12.98	12.43	12.88	68317	{}	\N	2020-10-12 15:14:00.381271
50588	KALV	2020-10-02	12.68	13.18	12.01	12.97	30599	{}	\N	2020-10-12 15:14:00.386087
50589	KALV	2020-10-05	12.36	13.9	12.36	13.82	29053	{}	\N	2020-10-12 15:14:00.390065
50590	KALV	2020-10-06	13.92	14.09	12.48	13.84	31627	{}	\N	2020-10-12 15:14:00.39427
50591	KALV	2020-10-07	14.09	14.29	14.04	14.23	31822	{}	\N	2020-10-12 15:14:00.398469
50592	KALV	2020-10-08	14.48	14.48	13.98	14.02	36074	{}	\N	2020-10-12 15:14:00.403125
50593	KALV	2020-10-09	14.06	14.228	13.065	13.51	54889	{}	\N	2020-10-12 15:14:00.40816
50594	KALV	2020-10-12	12.88	13.76	12.88	13.41	8078	{}	\N	2020-10-12 15:14:00.4135
50595	IEI	2020-09-28	133.67	133.7	133.66	133.67	915494	{}	\N	2020-10-12 15:14:04.367403
50596	IEI	2020-09-29	133.73	133.76	133.7	133.73	518178	{}	\N	2020-10-12 15:14:04.379834
50597	IEI	2020-09-30	133.71	133.71	133.56	133.59	994432	{}	\N	2020-10-12 15:14:04.386997
50598	IEI	2020-10-01	133.42	133.58	133.36	133.52	2787782	{}	\N	2020-10-12 15:14:04.392374
50599	IEI	2020-10-02	133.6	133.6	133.43	133.47	929235	{}	\N	2020-10-12 15:14:04.397549
50600	IEI	2020-10-05	133.37	133.37	133.14	133.15	1336210	{}	\N	2020-10-12 15:14:04.401559
50601	IEI	2020-10-06	133.19	133.34	133.15	133.26	612158	{}	\N	2020-10-12 15:14:04.40615
50602	IEI	2020-10-07	133.14	133.185	133.0601	133.12	1110015	{}	\N	2020-10-12 15:14:04.40977
50603	IEI	2020-10-08	133.16	133.23	133.13	133.21	786421	{}	\N	2020-10-12 15:14:04.414062
50604	IEI	2020-10-09	133.19	133.25	133.095	133.16	642170	{}	\N	2020-10-12 15:14:04.418732
50605	IEI	2020-10-12	133.17	133.2	133.15	133.15	95563	{}	\N	2020-10-12 15:14:04.424908
50606	IEX	2020-09-28	180.87	183.47	180.83	181.32	203245	{}	\N	2020-10-12 15:14:05.476277
50607	IEX	2020-09-29	180.68	182.665	180.68	181.75	388118	{}	\N	2020-10-12 15:14:05.483928
50608	IEX	2020-09-30	182.87	183.81	180.34	182.41	971427	{}	\N	2020-10-12 15:14:05.489998
50609	IEX	2020-10-01	184.1	185.21	177.1275	178.12	694221	{}	\N	2020-10-12 15:14:05.494987
50610	IEX	2020-10-02	176.52	181.7	176.225	179.92	445160	{}	\N	2020-10-12 15:14:05.500317
50611	IEX	2020-10-05	181.65	183.71	181.65	182.91	327557	{}	\N	2020-10-12 15:14:05.504917
50612	IEX	2020-10-06	182.96	186.91	181.93	183.8	523883	{}	\N	2020-10-12 15:14:05.509053
50613	IEX	2020-10-07	185.66	189.35	185.55	189.18	668822	{}	\N	2020-10-12 15:14:05.513091
50614	IEX	2020-10-08	190.02	190.9589	189.105	190	656648	{}	\N	2020-10-12 15:14:05.517306
50615	IEX	2020-10-09	190.47	191.36	188.91	190.41	506239	{}	\N	2020-10-12 15:14:05.521319
50616	IEX	2020-10-12	190.76	190.76	184.94	186.38	411928	{}	\N	2020-10-12 15:14:05.52556
50617	IFRX	2020-09-28	4.5	4.57	4.29	4.33	178923	{}	\N	2020-10-12 15:14:06.728011
50618	IFRX	2020-09-29	4.33	5.38	4.12	4.59	5613985	{}	\N	2020-10-12 15:14:06.74097
50619	IFRX	2020-09-30	4.54	4.63	4.1	4.26	648617	{}	\N	2020-10-12 15:14:06.747896
50620	IFRX	2020-10-01	4.211	4.39	4.07	4.28	309622	{}	\N	2020-10-12 15:14:06.754345
50621	IFRX	2020-10-02	4.12	4.69	4.12	4.62	337376	{}	\N	2020-10-12 15:14:06.759331
50622	IFRX	2020-10-05	4.73	4.8	4.51	4.62	235672	{}	\N	2020-10-12 15:14:06.764438
50623	IFRX	2020-10-06	4.69	4.74	4.33	4.55	240570	{}	\N	2020-10-12 15:14:06.769975
50624	IFRX	2020-10-07	4.58	4.7	4.47	4.59	107397	{}	\N	2020-10-12 15:14:06.774043
50625	IFRX	2020-10-08	4.74	4.9	4.6326	4.68	447391	{}	\N	2020-10-12 15:14:06.77836
50626	IFRX	2020-10-09	4.7765	5.0328	4.72	4.85	316836	{}	\N	2020-10-12 15:14:06.782658
50627	IFRX	2020-10-12	4.85	4.98	4.7846	4.7846	79447	{}	\N	2020-10-12 15:14:06.787221
50628	IHG	2020-09-28	53.06	53.49	52.48	53.21	172956	{}	\N	2020-10-12 15:14:08.358209
50629	IHG	2020-09-29	53.07	53.35	52.28	52.57	321264	{}	\N	2020-10-12 15:14:08.367255
50630	IHG	2020-09-30	52.46	53.3	52.22	52.48	225009	{}	\N	2020-10-12 15:14:08.37797
50631	IHG	2020-10-01	51.23	51.53	50.65	51.44	340794	{}	\N	2020-10-12 15:14:08.382872
50632	IHG	2020-10-02	50.64	52.37	50.48	52.12	271787	{}	\N	2020-10-12 15:14:08.387822
50633	IHG	2020-10-05	53.1	53.5	52.82	53.36	261325	{}	\N	2020-10-12 15:14:08.391721
50634	IHG	2020-10-06	54.7	56.36	54.63	54.89	393322	{}	\N	2020-10-12 15:14:08.395449
50635	IHG	2020-10-07	56.2	56.55	55.73	56.09	317798	{}	\N	2020-10-12 15:14:08.399233
50636	IHG	2020-10-08	55.88	55.94	55.03	55.72	248002	{}	\N	2020-10-12 15:14:08.4033
50637	IHG	2020-10-09	56.24	56.72	55.91	56.29	186331	{}	\N	2020-10-12 15:14:08.407109
50638	IHG	2020-10-12	55.6	55.96	55.55	55.88	64371	{}	\N	2020-10-12 15:14:08.410911
50639	IIM	2020-09-28	14.75	14.75	14.63	14.67	138628	{}	\N	2020-10-12 15:14:09.979282
50640	IIM	2020-09-29	14.68	14.74	14.65	14.72	191112	{}	\N	2020-10-12 15:14:09.997154
50641	IIM	2020-09-30	14.78	14.78	14.7	14.71	226214	{}	\N	2020-10-12 15:14:10.004875
50642	IIM	2020-10-01	14.74	14.81	14.73	14.79	120026	{}	\N	2020-10-12 15:14:10.010691
50643	IIM	2020-10-02	14.79	14.83	14.73	14.83	108742	{}	\N	2020-10-12 15:14:10.015908
50644	IIM	2020-10-05	14.83	14.84	14.7122	14.79	138134	{}	\N	2020-10-12 15:14:10.020794
50645	IIM	2020-10-06	14.78	14.82	14.75	14.79	97510	{}	\N	2020-10-12 15:14:10.02546
50646	IIM	2020-10-07	14.79	14.91	14.78	14.8	403116	{}	\N	2020-10-12 15:14:10.030966
50647	IIM	2020-10-08	14.8	14.852	14.77	14.83	115238	{}	\N	2020-10-12 15:14:10.035141
50648	IIM	2020-10-09	14.87	14.88	14.82	14.87	104786	{}	\N	2020-10-12 15:14:10.0399
50649	IIM	2020-10-12	14.98	14.98	14.9	14.9	40997	{}	\N	2020-10-12 15:14:10.044577
50650	IKNX	2020-09-28	3.4	3.47	3.25	3.47	4457	{}	\N	2020-10-12 15:14:11.073023
50651	IKNX	2020-09-29	3.5	3.75	3.32	3.52	11632	{}	\N	2020-10-12 15:14:11.079462
50652	IKNX	2020-09-30	3.59	3.59	3.3801	3.57	8478	{}	\N	2020-10-12 15:14:11.085031
50653	IKNX	2020-10-01	3.39	3.42	3.3133	3.41	4748	{}	\N	2020-10-12 15:14:11.090153
50654	IKNX	2020-10-02	3.45	3.5	3.3165	3.48	2396	{}	\N	2020-10-12 15:14:11.0946
50655	IKNX	2020-10-05	3.33	3.5677	3.32	3.3408	13024	{}	\N	2020-10-12 15:14:11.099245
50656	IKNX	2020-10-06	3.4399	4.1	3.395	3.8	208054	{}	\N	2020-10-12 15:14:11.104306
50657	IKNX	2020-10-07	3.8	3.8	3.4606	3.53	27347	{}	\N	2020-10-12 15:14:11.108905
50658	IKNX	2020-10-08	3.52	3.63	3.3801	3.5545	12145	{}	\N	2020-10-12 15:14:11.113437
50659	IKNX	2020-10-09	3.54	3.5855	3.41	3.4565	6620	{}	\N	2020-10-12 15:14:11.118276
50660	IKNX	2020-10-12	3.59	3.65	3.59	3.59	5023	{}	\N	2020-10-12 15:14:11.122867
50661	IMOM	2020-09-28	28.46	28.5	28.36	28.41	26248	{}	\N	2020-10-12 15:14:14.310533
50662	IMOM	2020-09-29	28.87	28.9136	28.65	28.8446	20378	{}	\N	2020-10-12 15:14:14.319075
50663	IMOM	2020-09-30	28.77	28.77	28.6299	28.6299	3907	{}	\N	2020-10-12 15:14:14.324841
50664	IMOM	2020-10-01	28.945	28.9751	28.86	28.86	113249	{}	\N	2020-10-12 15:14:14.330201
50665	IMOM	2020-10-02	28.8199	28.9303	28.77	28.89	7370	{}	\N	2020-10-12 15:14:14.335633
50666	IMOM	2020-10-05	29.33	29.4	29.22	29.4	6292	{}	\N	2020-10-12 15:14:14.340872
50667	IMOM	2020-10-06	29.24	29.24	28.8758	28.8758	3736	{}	\N	2020-10-12 15:14:14.34537
50668	IMOM	2020-10-07	29.135	29.2681	29.135	29.2681	5647	{}	\N	2020-10-12 15:14:14.349777
50669	IMOM	2020-10-08	29.39	29.61	29.39	29.61	4751	{}	\N	2020-10-12 15:14:14.354818
50670	IMOM	2020-10-09	29.81	29.99	29.81	29.885	3167	{}	\N	2020-10-12 15:14:14.35914
50671	IMOM	2020-10-12	30.185	30.34	30.185	30.34	1964	{}	\N	2020-10-12 15:14:14.363751
50672	INBK	2020-09-28	14.72	15.31	14.72	15.18	31674	{}	\N	2020-10-12 15:14:15.691646
50673	INBK	2020-09-29	15.2	15.2	14.36	14.62	13960	{}	\N	2020-10-12 15:14:15.701831
50674	INBK	2020-09-30	14.76	14.87	14.5601	14.73	19705	{}	\N	2020-10-12 15:14:15.708233
50675	INBK	2020-10-01	14.8	14.82	14.48	14.58	68843	{}	\N	2020-10-12 15:14:15.714011
50676	INBK	2020-10-02	14.4386	15.91	14.4386	15.815	31884	{}	\N	2020-10-12 15:14:15.718922
50677	INBK	2020-10-05	15.71	16.44	15.71	16.34	24580	{}	\N	2020-10-12 15:14:15.724012
50678	INBK	2020-10-06	16.67	16.8	16.075	16.14	49010	{}	\N	2020-10-12 15:14:15.728023
50679	INBK	2020-10-07	16.17	17.43	16.17	17.35	28228	{}	\N	2020-10-12 15:14:15.732127
50680	INBK	2020-10-08	17.54	17.95	17.32	17.76	22200	{}	\N	2020-10-12 15:14:15.736962
50681	INBK	2020-10-09	17.72	17.86	17.5	17.72	28335	{}	\N	2020-10-12 15:14:15.741307
50682	INBK	2020-10-12	17.59	17.73	17.005	17.72	2711	{}	\N	2020-10-12 15:14:15.745604
50683	ISRG	2020-09-28	685	695.56	680.03	693.16	611970	{}	\N	2020-10-12 15:14:17.165988
50684	ISRG	2020-09-29	698	715.89	690.55	692.93	522048	{}	\N	2020-10-12 15:14:17.180188
50685	ISRG	2020-09-30	696.14	717.89	690.55	709.54	585454	{}	\N	2020-10-12 15:14:17.187764
50686	ISRG	2020-10-01	713.11	718.4273	704.015	710.52	382868	{}	\N	2020-10-12 15:14:17.193889
50687	ISRG	2020-10-02	700.08	707.57	692.101	696.28	495269	{}	\N	2020-10-12 15:14:17.199236
50688	ISRG	2020-10-05	699.1	708.56	696.822	707.7	387914	{}	\N	2020-10-12 15:14:17.204816
50689	ISRG	2020-10-06	709.53	716.22	696.295	699.02	382355	{}	\N	2020-10-12 15:14:17.209493
50690	ISRG	2020-10-07	708.44	721.99	708.44	720.28	378224	{}	\N	2020-10-12 15:14:17.213935
50691	ISRG	2020-10-08	725	728.62	720.12	723.02	361109	{}	\N	2020-10-12 15:14:17.218775
50692	ISRG	2020-10-09	730.185	741.27	728	738.19	381042	{}	\N	2020-10-12 15:14:17.223275
50693	ISRG	2020-10-12	748.74	750.56	743.23	749.9	199748	{}	\N	2020-10-12 15:14:17.22771
50694	IEMG	2020-09-28	52.08	52.2	51.74	51.97	7034258	{}	\N	2020-10-12 15:14:18.297185
50695	IEMG	2020-09-29	51.86	52.14	51.8	51.99	5913319	{}	\N	2020-10-12 15:14:18.308596
50696	IEMG	2020-09-30	52.37	52.92	52.34	52.8	16379904	{}	\N	2020-10-12 15:14:18.315246
50697	IEMG	2020-10-01	53.17	53.315	52.855	53.3	10460458	{}	\N	2020-10-12 15:14:18.320489
50698	IEMG	2020-10-02	52.47	53.12	52.43	52.65	5811078	{}	\N	2020-10-12 15:14:18.325442
50699	IEMG	2020-10-05	52.9999	53.375	52.95	53.35	5155645	{}	\N	2020-10-12 15:14:18.329469
50700	IEMG	2020-10-06	53.57	53.86	53.285	53.43	7519268	{}	\N	2020-10-12 15:14:18.333623
50701	IEMG	2020-10-07	53.94	54.18	53.79	54.015	7292386	{}	\N	2020-10-12 15:14:18.337336
50702	IEMG	2020-10-08	54.22	54.505	54.1	54.44	4408881	{}	\N	2020-10-12 15:14:18.340895
50703	IEMG	2020-10-09	54.57	54.96	54.45	54.86	5685755	{}	\N	2020-10-12 15:14:18.344827
50704	IEMG	2020-10-12	55.07	55.315	54.95	55.28	2077379	{}	\N	2020-10-12 15:14:18.349277
50705	IFF	2020-09-28	120.8	123.18	120.56	121.4	704776	{}	\N	2020-10-12 15:14:20.123866
50706	IFF	2020-09-29	121.52	123.005	120.41	121.5	1278605	{}	\N	2020-10-12 15:14:20.134756
50707	IFF	2020-09-30	122.05	123.89	121.3622	122.45	1894634	{}	\N	2020-10-12 15:14:20.142015
50708	IFF	2020-10-01	122.99	124.98	119.57	120.09	2610003	{}	\N	2020-10-12 15:14:20.148645
50709	IFF	2020-10-02	119.3	120.94	118.81	119.17	1801442	{}	\N	2020-10-12 15:14:20.154111
50710	IFF	2020-10-05	119.56	121.21	117.73	118.36	1792522	{}	\N	2020-10-12 15:14:20.15925
50711	IFF	2020-10-06	119.19	119.69	114.85	115.31	4652003	{}	\N	2020-10-12 15:14:20.163952
50712	IFF	2020-10-07	116.43	117.69	115.3	116.51	2223423	{}	\N	2020-10-12 15:14:20.168434
50713	IFF	2020-10-08	117.69	118.29	114.85	116.2	1802353	{}	\N	2020-10-12 15:14:20.173046
50714	IFF	2020-10-09	117.75	121.715	117.75	119.41	2850421	{}	\N	2020-10-12 15:14:20.177309
50715	IFF	2020-10-12	120.18	120.18	117.7	118.63	348741	{}	\N	2020-10-12 15:14:20.181956
50716	IGD	2020-09-28	4.87	4.93	4.8504	4.92	367724	{}	\N	2020-10-12 15:14:21.53693
50717	IGD	2020-09-29	4.915	4.94	4.88	4.92	923381	{}	\N	2020-10-12 15:14:21.545791
50718	IGD	2020-09-30	4.93	5.01	4.93	4.95	1387468	{}	\N	2020-10-12 15:14:21.55245
50719	IGD	2020-10-01	4.9	4.96	4.9	4.94	857211	{}	\N	2020-10-12 15:14:21.557628
50720	IGD	2020-10-02	4.89	4.94	4.85	4.94	363789	{}	\N	2020-10-12 15:14:21.563544
50721	IGD	2020-10-05	4.95	4.9679	4.91	4.94	528566	{}	\N	2020-10-12 15:14:21.567988
50722	IGD	2020-10-06	4.95	4.98	4.92	4.92	454539	{}	\N	2020-10-12 15:14:21.572196
50723	IGD	2020-10-07	4.93	5.01	4.93	5.01	665156	{}	\N	2020-10-12 15:14:21.576971
50724	IGD	2020-10-08	5.03	5.03	4.96	4.97	638025	{}	\N	2020-10-12 15:14:21.580549
50725	IGD	2020-10-09	4.98	5.027	4.97	5.02	308293	{}	\N	2020-10-12 15:14:21.583868
50726	IGD	2020-10-12	5.02	5.02	4.96	5.02	150278	{}	\N	2020-10-12 15:14:21.587842
50727	IGV	2020-09-28	313.57	314	308.85	311.52	883542	{}	\N	2020-10-12 15:14:22.582015
50728	IGV	2020-09-29	312.02	313.145	309.82	310.55	715014	{}	\N	2020-10-12 15:14:22.592125
50729	IGV	2020-09-30	310.51	315.185	309.09	311.12	448692	{}	\N	2020-10-12 15:14:22.59853
50730	IGV	2020-10-01	316.06	318.57	314.59	317.12	586539	{}	\N	2020-10-12 15:14:22.604417
50731	IGV	2020-10-02	310.99	317.78	309.73	310.86	520586	{}	\N	2020-10-12 15:14:22.609338
50732	IGV	2020-10-05	313.54	316.3	312.09	315.65	369612	{}	\N	2020-10-12 15:14:22.615059
50733	IGV	2020-10-06	316.03	319.58	312.57	313.22	485622	{}	\N	2020-10-12 15:14:22.619331
50734	IGV	2020-10-07	317.19	321.233	316.39	320.3	289690	{}	\N	2020-10-12 15:14:22.623521
50735	IGV	2020-10-08	323	323.97	318.91	320.56	382770	{}	\N	2020-10-12 15:14:22.627565
50736	IGV	2020-10-09	323.46	326.8959	322.6286	326.52	377807	{}	\N	2020-10-12 15:14:22.631537
50737	IGV	2020-10-12	331.45	332.11	327.665	330.39	317276	{}	\N	2020-10-12 15:14:22.635393
50738	IHIT	2020-09-28	7.83	7.88	7.81	7.88	49881	{}	\N	2020-10-12 15:14:23.768478
50739	IHIT	2020-09-29	7.85	7.92	7.815	7.9	106250	{}	\N	2020-10-12 15:14:23.778921
50740	IHIT	2020-09-30	7.93	7.94	7.88	7.93	183783	{}	\N	2020-10-12 15:14:23.785996
50741	IHIT	2020-10-01	7.9	7.97	7.9	7.97	80844	{}	\N	2020-10-12 15:14:23.792277
50742	IHIT	2020-10-02	7.9	7.99	7.9	7.96	202956	{}	\N	2020-10-12 15:14:23.797806
50743	IHIT	2020-10-05	7.97	8	7.94	7.97	89190	{}	\N	2020-10-12 15:14:23.803402
50744	IHIT	2020-10-06	7.94	8.05	7.94	8.03	176062	{}	\N	2020-10-12 15:14:23.808284
50745	IHIT	2020-10-07	8.01	8.11	8.01	8.11	52244	{}	\N	2020-10-12 15:14:23.81288
50746	IHIT	2020-10-08	8.08	8.23	8.08	8.15	172152	{}	\N	2020-10-12 15:14:23.817519
50747	IHIT	2020-10-09	8.11	8.17	8.07	8.1	82755	{}	\N	2020-10-12 15:14:23.822029
50748	IHIT	2020-10-12	8.08	8.17	8.08	8.146	18762	{}	\N	2020-10-12 15:14:23.826349
50749	ILTB	2020-09-28	75.78	75.88	75.7	75.88	17565	{}	\N	2020-10-12 15:14:25.28165
50750	ILTB	2020-09-29	76.03	76.07	75.714	75.81	13420	{}	\N	2020-10-12 15:14:25.287918
50751	ILTB	2020-09-30	75.52	75.68	75.28	75.45	20212	{}	\N	2020-10-12 15:14:25.29259
50752	ILTB	2020-10-01	75.02	75.5399	75	75.42	51824	{}	\N	2020-10-12 15:14:25.29733
50753	ILTB	2020-10-02	75.4	75.42	75.115	75.41	36545	{}	\N	2020-10-12 15:14:25.301608
50754	ILTB	2020-10-05	74.8668	74.96	74.59	74.68	65791	{}	\N	2020-10-12 15:14:25.306293
50755	ILTB	2020-10-06	74.86	75.1899	74.5003	74.69	48152	{}	\N	2020-10-12 15:14:25.311155
50756	ILTB	2020-10-07	74.69	74.8793	74.37	74.42	29789	{}	\N	2020-10-12 15:14:25.316133
50757	ILTB	2020-10-08	74.74	74.99	74.72	74.95	12071	{}	\N	2020-10-12 15:14:25.320621
50758	ILTB	2020-10-09	74.82	75.0488	74.6301	75	21435	{}	\N	2020-10-12 15:14:25.324972
50759	ILTB	2020-10-12	75.36	75.47	75.28	75.3141	10425	{}	\N	2020-10-12 15:14:25.329154
50760	IMMR	2020-09-28	7.01	7.14	6.92	6.99	177170	{}	\N	2020-10-12 15:14:26.648686
50761	IMMR	2020-09-29	7.13	7.24	6.87	7.02	218961	{}	\N	2020-10-12 15:14:26.659508
50762	IMMR	2020-09-30	7.01	7.06	6.92	7.05	208566	{}	\N	2020-10-12 15:14:26.666282
50763	IMMR	2020-10-01	7.12	7.13	6.985	7.04	304193	{}	\N	2020-10-12 15:14:26.67233
50764	IMMR	2020-10-02	6.93	6.99	6.58	6.6	386918	{}	\N	2020-10-12 15:14:26.676908
50765	IMMR	2020-10-05	6.71	6.85	6.61	6.73	268109	{}	\N	2020-10-12 15:14:26.68198
50766	IMMR	2020-10-06	6.74	6.94	6.58	6.58	213787	{}	\N	2020-10-12 15:14:26.686214
50767	IMMR	2020-10-07	6.63	6.77	6.515	6.74	490633	{}	\N	2020-10-12 15:14:26.690311
50768	IMMR	2020-10-08	6.82	6.97	6.67	6.76	529762	{}	\N	2020-10-12 15:14:26.6942
50769	IMMR	2020-10-09	6.79	6.95	6.7	6.8	307147	{}	\N	2020-10-12 15:14:26.698406
50770	IMMR	2020-10-12	6.89	7.05	6.86	7.02	82388	{}	\N	2020-10-12 15:14:26.70242
50771	IMTM	2020-09-28	33.85	33.95	33.7413	33.94	232534	{}	\N	2020-10-12 15:14:29.61636
50772	IMTM	2020-09-29	34.07	34.25	34.05	34.175	377568	{}	\N	2020-10-12 15:14:29.627074
50773	IMTM	2020-09-30	34.07	34.1531	33.88	33.96	105907	{}	\N	2020-10-12 15:14:29.634917
50774	IMTM	2020-10-01	34.25	34.33	34.14	34.32	207945	{}	\N	2020-10-12 15:14:29.640467
50775	IMTM	2020-10-02	33.83	34.1768	33.7	34.09	94869	{}	\N	2020-10-12 15:14:29.645355
50776	IMTM	2020-10-05	34.27	34.5	34.27	34.5	104413	{}	\N	2020-10-12 15:14:29.64995
50777	IMTM	2020-10-06	34.37	34.37	33.8632	33.93	64994	{}	\N	2020-10-12 15:14:29.653657
50778	IMTM	2020-10-07	34.17	34.2	34.0372	34.17	73175	{}	\N	2020-10-12 15:14:29.657346
50779	IMTM	2020-10-08	34.33	34.43	34.26	34.42	108693	{}	\N	2020-10-12 15:14:29.660974
50780	IMTM	2020-10-09	34.59	34.84	34.59	34.84	68118	{}	\N	2020-10-12 15:14:29.664769
50781	IMTM	2020-10-12	34.98	35.1	34.98	35.0934	15298	{}	\N	2020-10-12 15:14:29.668735
50782	INCO	2020-09-28	41.91	41.92	41.5264	41.5264	7531	{}	\N	2020-10-12 15:14:31.418907
50783	INCO	2020-09-29	41.9406	41.99	41.64	41.78	8251	{}	\N	2020-10-12 15:14:31.430436
50784	INCO	2020-09-30	42.02	42.4547	42.02	42.3317	5260	{}	\N	2020-10-12 15:14:31.436754
50785	INCO	2020-10-01	42.37	42.5	42.1573	42.5	1050	{}	\N	2020-10-12 15:14:31.441438
50786	INCO	2020-10-02	42.12	42.5	42.12	42.3397	4136	{}	\N	2020-10-12 15:14:31.446431
50787	INCO	2020-10-05	42.5	42.81	42.4679	42.72	18674	{}	\N	2020-10-12 15:14:31.450306
50788	INCO	2020-10-06	42.74	42.81	42.39	42.39	6739	{}	\N	2020-10-12 15:14:31.454274
50789	INCO	2020-10-07	43	43	42.605	42.6861	3650	{}	\N	2020-10-12 15:14:31.458336
50790	INCO	2020-10-08	42.84	42.9499	42.8201	42.8201	2471	{}	\N	2020-10-12 15:14:31.46231
50791	INCO	2020-10-09	42.94	42.9999	42.8685	42.91	5729	{}	\N	2020-10-12 15:14:31.466077
50792	INCO	2020-10-12	42.41	42.61	42.41	42.61	1774	{}	\N	2020-10-12 15:14:31.470166
50793	INFO	2020-09-28	79.03	80.64	78.99	80.06	2878504	{}	\N	2020-10-12 15:14:33.869273
50794	INFO	2020-09-29	79.7	80.52	77.41	77.42	5423186	{}	\N	2020-10-12 15:14:33.879554
50795	INFO	2020-09-30	79.2	79.92	77.82	78.51	7898108	{}	\N	2020-10-12 15:14:33.885947
50796	INFO	2020-10-01	78.62	79.32	78.06	78.56	4600007	{}	\N	2020-10-12 15:14:33.891707
50797	INFO	2020-10-02	77.7	78.96	77.29	78.52	3350370	{}	\N	2020-10-12 15:14:33.896973
50798	INFO	2020-10-05	78.92	79.3895	78.5	78.59	2783018	{}	\N	2020-10-12 15:14:33.902068
50799	INFO	2020-10-06	78.89	78.89	77.17	77.36	3928779	{}	\N	2020-10-12 15:14:33.906705
50800	INFO	2020-10-07	77.78	78.13	77.39	77.92	3970449	{}	\N	2020-10-12 15:14:33.911207
50801	INFO	2020-10-08	78.64	79.33	78.13	78.24	3647744	{}	\N	2020-10-12 15:14:33.915446
50802	INFO	2020-10-09	78.85	79.465	78.62	78.68	2788640	{}	\N	2020-10-12 15:14:33.920046
50803	INFO	2020-10-12	79.4	79.83	78.93	79.795	342399	{}	\N	2020-10-12 15:14:33.925037
50804	IIF	2020-09-28	17.02	17.1899	17.02	17.18	19873	{}	\N	2020-10-12 15:14:36.252945
50805	IIF	2020-09-29	17.16	17.25	17.14	17.2	35977	{}	\N	2020-10-12 15:14:36.260639
50806	IIF	2020-09-30	17.35	17.47	17.26	17.35	29890	{}	\N	2020-10-12 15:14:36.266682
50807	IIF	2020-10-01	17.46	17.54	17.4	17.54	82656	{}	\N	2020-10-12 15:14:36.272107
50808	IIF	2020-10-02	17.29	17.46	17.29	17.34	27498	{}	\N	2020-10-12 15:14:36.276771
50809	IIF	2020-10-05	17.51	17.65	17.43	17.64	26682	{}	\N	2020-10-12 15:14:36.281679
50810	IIF	2020-10-06	17.67	17.77	17.65	17.67	56954	{}	\N	2020-10-12 15:14:36.285789
50811	IIF	2020-10-07	17.82	18	17.73	17.84	45256	{}	\N	2020-10-12 15:14:36.29041
50812	IIF	2020-10-08	17.9	17.9	17.72	17.86	118756	{}	\N	2020-10-12 15:14:36.294552
50813	IIF	2020-10-09	17.91	18.07	17.91	18.05	73362	{}	\N	2020-10-12 15:14:36.298914
50814	IIF	2020-10-12	18.1	18.1	17.8601	17.985	14620	{}	\N	2020-10-12 15:14:36.30335
50815	IMAX	2020-09-28	12.42	12.53	12.07	12.16	462957	{}	\N	2020-10-12 15:14:38.064921
50816	IMAX	2020-09-29	12.13	12.15	11.84	11.95	983479	{}	\N	2020-10-12 15:14:38.075979
50817	IMAX	2020-09-30	11.96	12.26	11.9326	11.96	1055294	{}	\N	2020-10-12 15:14:38.082429
50818	IMAX	2020-10-01	12.2	12.2	11.58	11.61	1756518	{}	\N	2020-10-12 15:14:38.088003
50819	IMAX	2020-10-02	11.3	11.48	11.08	11.27	1265171	{}	\N	2020-10-12 15:14:38.0929
50820	IMAX	2020-10-05	11.27	11.305	10.5	11.06	2441156	{}	\N	2020-10-12 15:14:38.098392
50821	IMAX	2020-10-06	11.17	11.91	11.16	11.43	1717941	{}	\N	2020-10-12 15:14:38.102717
50822	IMAX	2020-10-07	11.58	11.7	11.11	11.37	1352071	{}	\N	2020-10-12 15:14:38.107452
50823	IMAX	2020-10-08	11.5	11.64	11.27	11.62	887303	{}	\N	2020-10-12 15:14:38.112834
50824	IMAX	2020-10-09	11.85	12.11	11.52	11.62	842678	{}	\N	2020-10-12 15:14:38.117472
50825	IMAX	2020-10-12	11.37	11.59	11.2	11.56	442625	{}	\N	2020-10-12 15:14:38.121963
50826	IMO	2020-09-28	12.5	12.72	12.44	12.48	594835	{}	\N	2020-10-12 15:14:39.725478
50827	IMO	2020-09-29	12.53	12.53	11.805	12.04	502939	{}	\N	2020-10-12 15:14:39.734282
50828	IMO	2020-09-30	12.12	12.26	11.94	11.96	533333	{}	\N	2020-10-12 15:14:39.740421
50829	IMO	2020-10-01	12.045	12.14	11.42	11.54	794369	{}	\N	2020-10-12 15:14:39.745465
50830	IMO	2020-10-02	11.25	11.77	11.16	11.68	877935	{}	\N	2020-10-12 15:14:39.749933
50831	IMO	2020-10-05	11.91	12.03	11.59	11.89	877826	{}	\N	2020-10-12 15:14:39.75438
50832	IMO	2020-10-06	12.12	12.25	11.7	11.82	465402	{}	\N	2020-10-12 15:14:39.758432
50833	IMO	2020-10-07	11.86	12.18	11.82	12.07	496378	{}	\N	2020-10-12 15:14:39.762429
50834	IMO	2020-10-08	12.33	12.91	12.23	12.83	506509	{}	\N	2020-10-12 15:14:39.7662
50835	IMO	2020-10-09	12.99	13.1	12.49	12.6	384381	{}	\N	2020-10-12 15:14:39.770191
50836	IMO	2020-10-12	12.54	12.59	12.42	12.53	92582	{}	\N	2020-10-12 15:14:39.774104
50837	INDY	2020-09-28	34.49	34.6	34.41	34.5	70374	{}	\N	2020-10-12 15:14:41.24453
50838	INDY	2020-09-29	34.52	34.61	34.43	34.43	14116	{}	\N	2020-10-12 15:14:41.253172
50839	INDY	2020-09-30	34.66	34.96	34.66	34.83	50628	{}	\N	2020-10-12 15:14:41.259704
50840	INDY	2020-10-01	35.28	35.4	35.08	35.4	61939	{}	\N	2020-10-12 15:14:41.265302
50841	INDY	2020-10-02	34.57	35.19	34.57	35.06	7537	{}	\N	2020-10-12 15:14:41.27015
50842	INDY	2020-10-05	35.56	35.78	35.56	35.75	218754	{}	\N	2020-10-12 15:14:41.274932
50843	INDY	2020-10-06	36.035	36.15	35.761	35.83	31188	{}	\N	2020-10-12 15:14:41.279263
50844	INDY	2020-10-07	36.16	36.4387	36.16	36.34	59177	{}	\N	2020-10-12 15:14:41.283103
50845	INDY	2020-10-08	36.44	36.52	36.37	36.41	41389	{}	\N	2020-10-12 15:14:41.2871
50846	INDY	2020-10-09	36.785	37.05	36.77	36.95	47700	{}	\N	2020-10-12 15:14:41.292073
50847	INDY	2020-10-12	36.5	36.845	36.465	36.83	6512	{}	\N	2020-10-12 15:14:41.299227
50848	INGN	2020-09-28	28.81	29.23	28.31	28.41	203429	{}	\N	2020-10-12 15:14:42.730823
50849	INGN	2020-09-29	28.45	29.39	28.39	29.14	214121	{}	\N	2020-10-12 15:14:42.740785
50850	INGN	2020-09-30	29.21	29.94	28.64	29	186172	{}	\N	2020-10-12 15:14:42.746655
50851	INGN	2020-10-01	29.08	29.1543	28.26	28.79	105221	{}	\N	2020-10-12 15:14:42.751138
50852	INGN	2020-10-02	28.34	30.07	28.34	29.76	347970	{}	\N	2020-10-12 15:14:42.755899
50853	INGN	2020-10-05	29.93	30.39	29.54	29.87	144664	{}	\N	2020-10-12 15:14:42.760427
50854	INGN	2020-10-06	30.11	30.6	29.61	29.755	150366	{}	\N	2020-10-12 15:14:42.764813
50855	INGN	2020-10-07	29.78	30.39	29.46	29.74	133117	{}	\N	2020-10-12 15:14:42.768811
50856	INGN	2020-10-08	30	30.81	29.82	30.7	138102	{}	\N	2020-10-12 15:14:42.773034
50857	INGN	2020-10-09	30.85	31.17	30.59	31.05	76649	{}	\N	2020-10-12 15:14:42.777065
50858	INGN	2020-10-12	31.06	31.395	30.975	31.29	12375	{}	\N	2020-10-12 15:14:42.781227
50859	INGR	2020-09-28	77.42	78.33	76.53	76.6	395943	{}	\N	2020-10-12 15:14:43.973229
50860	INGR	2020-09-29	76.61	76.88	74.37	75.03	700681	{}	\N	2020-10-12 15:14:43.987753
50861	INGR	2020-09-30	74.91	76.82	74.91	75.68	1355148	{}	\N	2020-10-12 15:14:43.99656
50862	INGR	2020-10-01	75.81	76.2	73.96	74.75	1247275	{}	\N	2020-10-12 15:14:44.003024
50863	INGR	2020-10-02	73.63	76.475	73.62	75.77	1055571	{}	\N	2020-10-12 15:14:44.008106
50864	INGR	2020-10-05	76.29	76.9	75.58	76.78	782437	{}	\N	2020-10-12 15:14:44.012596
50865	INGR	2020-10-06	77.22	77.88	76.06	76.35	839565	{}	\N	2020-10-12 15:14:44.016532
50866	INGR	2020-10-07	76.65	77.37	76.2301	76.83	837354	{}	\N	2020-10-12 15:14:44.020729
50867	INGR	2020-10-08	77.18	77.43	76.16	76.93	589830	{}	\N	2020-10-12 15:14:44.024888
50868	INGR	2020-10-09	77.52	77.6	76.52	76.8	459055	{}	\N	2020-10-12 15:14:44.029117
50869	INGR	2020-10-12	76.89	77.6	76.7	77.515	96547	{}	\N	2020-10-12 15:14:44.032997
50870	INO	2020-09-28	10.66	13.8	10.28	12.14	67258623	{}	\N	2020-10-12 15:14:45.104456
50871	INO	2020-09-29	11.93	12.55	11.29	11.3078	22912102	{}	\N	2020-10-12 15:14:45.114416
50872	INO	2020-09-30	11.2	11.84	11.1	11.6	14117173	{}	\N	2020-10-12 15:14:45.121202
50873	INO	2020-10-01	11.74	12.1	11.42	11.74	9540606	{}	\N	2020-10-12 15:14:45.12659
50874	INO	2020-10-02	11.85	12.2	11.53	11.65	10069886	{}	\N	2020-10-12 15:14:45.131171
50875	INO	2020-10-05	11.76	11.94	11.21	11.94	8511239	{}	\N	2020-10-12 15:14:45.135656
50876	INO	2020-10-06	11.89	13.11	11.89	12.41	15951150	{}	\N	2020-10-12 15:14:45.140005
50877	INO	2020-10-07	12.71	12.84	12.08	12.58	7801871	{}	\N	2020-10-12 15:14:45.143922
50878	INO	2020-10-08	12.68	12.8	12.35	12.48	5526678	{}	\N	2020-10-12 15:14:45.148196
50879	INO	2020-10-09	12.45	12.4955	11.95	12.31	7000222	{}	\N	2020-10-12 15:14:45.151862
50880	INO	2020-10-12	12.25	12.6849	12.17	12.428	3279802	{}	\N	2020-10-12 15:14:45.155741
50881	INOV	2020-09-28	26.49	26.86	26.315	26.43	417117	{}	\N	2020-10-12 15:14:46.317087
50882	INOV	2020-09-29	26.57	26.96	26.2	26.25	316821	{}	\N	2020-10-12 15:14:46.3279
50883	INOV	2020-09-30	26.22	26.71	26.15	26.48	289490	{}	\N	2020-10-12 15:14:46.334656
50884	INOV	2020-10-01	26.66	26.96	26.26	26.75	399055	{}	\N	2020-10-12 15:14:46.340813
50885	INOV	2020-10-02	26.18	26.99	26.18	26.735	249529	{}	\N	2020-10-12 15:14:46.346182
50886	INOV	2020-10-05	26.93	27.53	26.58	27.4	345981	{}	\N	2020-10-12 15:14:46.350935
50887	INOV	2020-10-06	27.62	27.93	27.01	27.11	703411	{}	\N	2020-10-12 15:14:46.355417
50888	INOV	2020-10-07	27.31	27.49	26.68	26.97	349168	{}	\N	2020-10-12 15:14:46.359726
50889	INOV	2020-10-08	27.12	27.37	26.86	26.97	294392	{}	\N	2020-10-12 15:14:46.36391
50890	INOV	2020-10-09	27.11	27.54	26.98	27.49	242575	{}	\N	2020-10-12 15:14:46.370796
50891	INOV	2020-10-12	27.54	27.9	27.21	27.67	43115	{}	\N	2020-10-12 15:14:46.380535
50892	INSW	2020-09-28	14.505	14.75	14.04	14.37	254643	{}	\N	2020-10-12 15:14:47.797368
50893	INSW	2020-09-29	14.54	14.785	14.0949	14.49	355721	{}	\N	2020-10-12 15:14:47.806777
50894	INSW	2020-09-30	14.68	15.095	14.52	14.61	344758	{}	\N	2020-10-12 15:14:47.812592
50895	INSW	2020-10-01	14.64	14.91	14.38	14.88	244895	{}	\N	2020-10-12 15:14:47.8175
50896	INSW	2020-10-02	14.63	15.08	14.4904	15.06	207951	{}	\N	2020-10-12 15:14:47.822458
50897	INSW	2020-10-05	15.11	15.89	15.11	15.82	368089	{}	\N	2020-10-12 15:14:47.826731
50898	INSW	2020-10-06	16	16.05	15.21	15.3	327183	{}	\N	2020-10-12 15:14:47.830984
50899	INSW	2020-10-07	15.44	15.62	15.18	15.28	342906	{}	\N	2020-10-12 15:14:47.835003
50900	INSW	2020-10-08	15.35	15.9	15.1	15.81	509301	{}	\N	2020-10-12 15:14:47.839308
50901	INSW	2020-10-09	16.1	16.45	15.67	15.74	278436	{}	\N	2020-10-12 15:14:47.843392
50902	INSW	2020-10-12	15.61	15.71	14.98	15.1	101844	{}	\N	2020-10-12 15:14:47.847617
50903	INTT	2020-09-28	4.31	4.6	4.31	4.36	36741	{}	\N	2020-10-12 15:14:49.524131
50904	INTT	2020-09-29	4.44	4.44	4.33	4.35	7254	{}	\N	2020-10-12 15:14:49.53308
50905	INTT	2020-09-30	4.33	4.5	4.33	4.5	25798	{}	\N	2020-10-12 15:14:49.538782
50906	INTT	2020-10-01	4.72	4.76	4.69	4.76	3484	{}	\N	2020-10-12 15:14:49.543781
50907	INTT	2020-10-02	4.76	4.76	4.5	4.54	45998	{}	\N	2020-10-12 15:14:49.547852
50908	INTT	2020-10-05	4.34	4.64	4.34	4.55	12345	{}	\N	2020-10-12 15:14:49.55191
50909	INTT	2020-10-06	4.65	4.65	4.41	4.41	9299	{}	\N	2020-10-12 15:14:49.555929
50910	INTT	2020-10-07	4.53	4.54	4.42	4.52	8745	{}	\N	2020-10-12 15:14:49.559456
50911	INTT	2020-10-08	4.65	4.65	4.45	4.5	8047	{}	\N	2020-10-12 15:14:49.562936
50912	INTT	2020-10-09	4.5754	4.58	4.5	4.55	7759	{}	\N	2020-10-12 15:14:49.56631
50913	INTT	2020-10-12	4.64	4.64	4.595	4.595	353	{}	\N	2020-10-12 15:14:49.569999
50914	KAMN	2020-09-28	39.09	40.41	39.09	39.91	192011	{}	\N	2020-10-12 15:14:51.060314
50915	KAMN	2020-09-29	39.61	39.76	38.75	39.06	259602	{}	\N	2020-10-12 15:14:51.07052
50916	KAMN	2020-09-30	39.07	40.38	38.65	38.97	396153	{}	\N	2020-10-12 15:14:51.077527
50917	KAMN	2020-10-01	38.96	39.79	38.56	38.9	373774	{}	\N	2020-10-12 15:14:51.082914
50918	KAMN	2020-10-02	38	40.37	38	40.28	306619	{}	\N	2020-10-12 15:14:51.087609
50919	KAMN	2020-10-05	40.7	41.46	40.46	41.31	297081	{}	\N	2020-10-12 15:14:51.09238
50920	KAMN	2020-10-06	42.03	42.19	40.48	40.51	350525	{}	\N	2020-10-12 15:14:51.096966
50921	KAMN	2020-10-07	40.88	41.61	40.68	41.06	269805	{}	\N	2020-10-12 15:14:51.100884
50922	KAMN	2020-10-08	41.59	42.24	40.9	42.22	142200	{}	\N	2020-10-12 15:14:51.104804
50923	KAMN	2020-10-09	42.575	42.575	41.75	42.24	270268	{}	\N	2020-10-12 15:14:51.109366
50924	KAMN	2020-10-12	42.2	42.86	42.18	42.72	21983	{}	\N	2020-10-12 15:14:51.113495
50925	ILMN	2020-09-28	307.25	307.61	297.67	305.46	1833934	{}	\N	2020-10-12 15:14:52.021396
50926	ILMN	2020-09-29	304.6	309.7	302.37	305.5	1566753	{}	\N	2020-10-12 15:14:52.027902
50927	ILMN	2020-09-30	305.18	310.61	303.61	309.08	1463712	{}	\N	2020-10-12 15:14:52.033552
50928	ILMN	2020-10-01	312.775	313.1375	306.15	309.61	1709940	{}	\N	2020-10-12 15:14:52.039266
50929	ILMN	2020-10-02	306.15	311.78	305	308.43	1453911	{}	\N	2020-10-12 15:14:52.043972
50930	ILMN	2020-10-05	310.74	320.62	310.08	316.17	1417618	{}	\N	2020-10-12 15:14:52.04849
50931	ILMN	2020-10-06	317.79	323.93	315.1	317.02	1097211	{}	\N	2020-10-12 15:14:52.05288
50932	ILMN	2020-10-07	319.32	325.32	314.5488	324.34	1134716	{}	\N	2020-10-12 15:14:52.057698
50933	ILMN	2020-10-08	327.78	334	322.64	333.78	1342542	{}	\N	2020-10-12 15:14:52.062293
50934	ILMN	2020-10-09	337.99	339.6	333.9672	339.04	1371504	{}	\N	2020-10-12 15:14:52.066877
50935	ILMN	2020-10-12	341.06	348.6	339.63	348.6	367208	{}	\N	2020-10-12 15:14:52.071761
50936	IMMP	2020-09-28	1.71	1.74	1.65	1.65	138349	{}	\N	2020-10-12 15:14:53.38978
50937	IMMP	2020-09-29	1.69	1.69	1.66	1.68	64612	{}	\N	2020-10-12 15:14:53.404514
50938	IMMP	2020-09-30	1.68	1.73	1.66	1.73	133688	{}	\N	2020-10-12 15:14:53.411804
50939	IMMP	2020-10-01	1.75	1.77	1.67	1.69	74845	{}	\N	2020-10-12 15:14:53.416914
50940	IMMP	2020-10-02	1.65	1.69	1.53	1.65	132994	{}	\N	2020-10-12 15:14:53.421822
50941	IMMP	2020-10-05	1.66	1.68	1.65	1.68	68880	{}	\N	2020-10-12 15:14:53.426389
50942	IMMP	2020-10-06	1.69	1.76	1.69	1.73	110542	{}	\N	2020-10-12 15:14:53.430884
50943	IMMP	2020-10-07	1.73	1.83	1.73	1.79	99271	{}	\N	2020-10-12 15:14:53.435029
50944	IMMP	2020-10-08	1.78	1.8	1.74	1.77	41683	{}	\N	2020-10-12 15:14:53.439449
50945	IMMP	2020-10-09	1.74	1.7588	1.68	1.71	39101	{}	\N	2020-10-12 15:14:53.443636
50946	IMMP	2020-10-12	1.71	1.74	1.69	1.7	14551	{}	\N	2020-10-12 15:14:53.447511
50947	IMTE	2020-09-28	3.93	3.93	3.75	3.7553	16410	{}	\N	2020-10-12 15:14:54.388272
50948	IMTE	2020-09-29	3.7619	3.77	3.57	3.61	26122	{}	\N	2020-10-12 15:14:54.397661
50949	IMTE	2020-09-30	3.63	4.04	3.62	3.76	72245	{}	\N	2020-10-12 15:14:54.410247
50950	IMTE	2020-10-01	3.98	4.01	3.7	3.73	16522	{}	\N	2020-10-12 15:14:54.418213
50951	IMTE	2020-10-02	3.72	3.91	3.6	3.74	27794	{}	\N	2020-10-12 15:14:54.424547
50952	IMTE	2020-10-05	3.7	3.83	3.7	3.82	6712	{}	\N	2020-10-12 15:14:54.429172
50953	IMTE	2020-10-06	3.75	3.99	3.74	3.99	5764	{}	\N	2020-10-12 15:14:54.434772
50954	IMTE	2020-10-07	3.85	4.88	3.755	4.2	461675	{}	\N	2020-10-12 15:14:54.439587
50955	IMTE	2020-10-08	4.17	4.2799	3.7	3.82	52222	{}	\N	2020-10-12 15:14:54.444338
50956	IMTE	2020-10-09	3.82	3.97	3.781	3.84	10175	{}	\N	2020-10-12 15:14:54.448761
50957	IMTE	2020-10-12	3.77	3.96	3.77	3.85	3616	{}	\N	2020-10-12 15:14:54.453359
50958	INDA	2020-09-28	33.58	33.665	33.455	33.57	2496455	{}	\N	2020-10-12 15:14:55.432286
50959	INDA	2020-09-29	33.55	33.7	33.52	33.6	2732679	{}	\N	2020-10-12 15:14:55.442777
50960	INDA	2020-09-30	33.879	34.015	33.78	33.86	3331639	{}	\N	2020-10-12 15:14:55.448482
50961	INDA	2020-10-01	34.25	34.38	34.075	34.36	3608520	{}	\N	2020-10-12 15:14:55.453612
50962	INDA	2020-10-02	33.751	34.21	33.751	34.08	2577177	{}	\N	2020-10-12 15:14:55.458379
50963	INDA	2020-10-05	34.56	34.74	34.53	34.72	3136787	{}	\N	2020-10-12 15:14:55.46276
50964	INDA	2020-10-06	34.99	35.03	34.73	34.79	3900786	{}	\N	2020-10-12 15:14:55.466304
50965	INDA	2020-10-07	35.15	35.23	35.11	35.14	6431480	{}	\N	2020-10-12 15:14:55.470328
50966	INDA	2020-10-08	35.23	35.335	35.16	35.21	6020797	{}	\N	2020-10-12 15:14:55.474207
50967	INDA	2020-10-09	35.5	35.63	35.345	35.585	2757265	{}	\N	2020-10-12 15:14:55.478244
50968	INDA	2020-10-12	35.34	35.48	35.295	35.46	661665	{}	\N	2020-10-12 15:14:55.48241
50969	INFU	2020-09-28	13.83	14.2399	13.32	13.69	114221	{}	\N	2020-10-12 15:14:57.264119
50970	INFU	2020-09-29	13.6	14.1	13.54	13.84	130853	{}	\N	2020-10-12 15:14:57.273622
50971	INFU	2020-09-30	11.95	13.49	11.6513	12.69	590412	{}	\N	2020-10-12 15:14:57.279361
50972	INFU	2020-10-01	12.73	13.23	12.5	12.89	89300	{}	\N	2020-10-12 15:14:57.284107
50973	INFU	2020-10-02	12.77	13.35	12.75	12.75	49302	{}	\N	2020-10-12 15:14:57.288432
50974	INFU	2020-10-05	12.93	13.59	12.57	13.56	132803	{}	\N	2020-10-12 15:14:57.292962
50975	INFU	2020-10-06	13.55	13.8903	13.29	13.59	66027	{}	\N	2020-10-12 15:14:57.29694
50976	INFU	2020-10-07	13.7179	13.98	13.52	13.7	91082	{}	\N	2020-10-12 15:14:57.300823
50977	INFU	2020-10-08	13.79	14	13.43	13.6	83568	{}	\N	2020-10-12 15:14:57.305246
50978	INFU	2020-10-09	13.77	13.77	13.51	13.6	161960	{}	\N	2020-10-12 15:14:57.309001
50979	INFU	2020-10-12	13.7	13.98	13.25	13.61	19152	{}	\N	2020-10-12 15:14:57.312656
50980	INPX	2020-09-28	1.0168	1.04	1.01	1.01	727738	{}	\N	2020-10-12 15:14:58.952088
50981	INPX	2020-09-29	1.01	1.15	1.01	1.1	3630272	{}	\N	2020-10-12 15:14:58.962114
50982	INPX	2020-09-30	1.08	1.13	1.044	1.1	2253908	{}	\N	2020-10-12 15:14:58.968381
50983	INPX	2020-10-01	1.1	1.1002	1.06	1.08	733129	{}	\N	2020-10-12 15:14:58.974497
50984	INPX	2020-10-02	1.09	1.29	1.06	1.18	14825333	{}	\N	2020-10-12 15:14:58.979646
50985	INPX	2020-10-05	1.19	1.3586	1.14	1.23	8605062	{}	\N	2020-10-12 15:14:58.983884
50986	INPX	2020-10-06	1.16	1.16	1.1	1.105	3027385	{}	\N	2020-10-12 15:14:58.988619
50987	INPX	2020-10-07	1.1	1.16	1.06	1.12	1690530	{}	\N	2020-10-12 15:14:58.993333
50988	INPX	2020-10-08	1.11	1.13	1.09	1.1	1780384	{}	\N	2020-10-12 15:14:58.99752
50989	INPX	2020-10-09	1.11	1.1173	1.09	1.09	1701958	{}	\N	2020-10-12 15:14:59.001866
50990	INPX	2020-10-12	1.12	1.13	1.09	1.0912	607699	{}	\N	2020-10-12 15:14:59.006406
50991	INSG	2020-09-28	11.21	11.28	10.4	10.67	3446007	{}	\N	2020-10-12 15:15:00.323997
50992	INSG	2020-09-29	10.5904	10.7	10.4	10.4	2100126	{}	\N	2020-10-12 15:15:00.334237
50993	INSG	2020-09-30	10.38	10.5101	10.22	10.32	1844313	{}	\N	2020-10-12 15:15:00.341393
50994	INSG	2020-10-01	10.35	10.36	10.07	10.32	1921192	{}	\N	2020-10-12 15:15:00.348036
50995	INSG	2020-10-02	10.05	10.36	10.02	10.11	1455331	{}	\N	2020-10-12 15:15:00.353343
50996	INSG	2020-10-05	10.2	10.2399	9.89	10.06	2392546	{}	\N	2020-10-12 15:15:00.358458
50997	INSG	2020-10-06	10.06	10.46	9.95	10.16	1649973	{}	\N	2020-10-12 15:15:00.362999
50998	INSG	2020-10-07	10.26	10.35	10	10.1	1753497	{}	\N	2020-10-12 15:15:00.367446
50999	INSG	2020-10-08	10.13	10.29	9.9702	10.11	1496923	{}	\N	2020-10-12 15:15:00.377423
51000	INSG	2020-10-09	10.15	10.6	10.1181	10.3	2128880	{}	\N	2020-10-12 15:15:00.382782
51001	INSG	2020-10-12	10.4303	10.5	10.28	10.35	540580	{}	\N	2020-10-12 15:15:00.387789
51002	INTC	2020-09-28	50.53	51.46	50.16	51.43	27699508	{}	\N	2020-10-12 15:15:01.401232
51003	INTC	2020-09-29	51.3	51.735	50.95	51.19	18323165	{}	\N	2020-10-12 15:15:01.41314
51004	INTC	2020-09-30	51.24	52.38	51.15	51.78	24777940	{}	\N	2020-10-12 15:15:01.420872
51005	INTC	2020-10-01	52.4	52.72	51.99	52.24	20580040	{}	\N	2020-10-12 15:15:01.427
51006	INTC	2020-10-02	51.5	51.9789	50.99	51.01	23446077	{}	\N	2020-10-12 15:15:01.43247
51007	INTC	2020-10-05	51.3	51.81	51.27	51.69	20379339	{}	\N	2020-10-12 15:15:01.437461
51008	INTC	2020-10-06	51.71	52.7	51.23	51.37	22378737	{}	\N	2020-10-12 15:15:01.442138
51009	INTC	2020-10-07	51.85	52.9	51.79	52.67	21110589	{}	\N	2020-10-12 15:15:01.446878
51010	INTC	2020-10-08	52.86	53.58	52.67	53.37	22090741	{}	\N	2020-10-12 15:15:01.451444
51011	INTC	2020-10-09	53.55	53.85	52.67	52.82	22672112	{}	\N	2020-10-12 15:15:01.456107
51012	INTC	2020-10-12	53.46	53.62	53.21	53.545	6686086	{}	\N	2020-10-12 15:15:01.460716
51013	INTG	2020-09-29	31.259	31.259	31	31.259	2605	{}	\N	2020-10-12 15:15:02.968417
51014	INTG	2020-10-01	31.25	31.25	31.25	31.25	522	{}	\N	2020-10-12 15:15:02.974781
51015	INTG	2020-10-05	31.0027	32.64	31.0027	32.64	526	{}	\N	2020-10-12 15:15:02.979727
51016	INTG	2020-10-06	32.6999	32.7	32.6999	32.7	532	{}	\N	2020-10-12 15:15:02.984586
51017	INTG	2020-10-08	31.235	31.235	31.235	31.235	563	{}	\N	2020-10-12 15:15:02.989862
51018	INTG	2020-10-09	31.29	31.63	30	31.63	1856	{}	\N	2020-10-12 15:15:02.995154
51019	INUV	2020-09-28	0.357	0.3747	0.3416	0.3595	2005977	{}	\N	2020-10-12 15:15:04.061839
51020	INUV	2020-09-29	0.3578	0.3746	0.345	0.3573	1738704	{}	\N	2020-10-12 15:15:04.075213
51021	INUV	2020-09-30	0.3598	0.385	0.3518	0.3655	3237763	{}	\N	2020-10-12 15:15:04.084707
51022	INUV	2020-10-01	0.36	0.3699	0.35	0.351	1268533	{}	\N	2020-10-12 15:15:04.091902
51023	INUV	2020-10-02	0.32	0.347	0.32	0.3373	2726242	{}	\N	2020-10-12 15:15:04.097202
51024	INUV	2020-10-05	0.3401	0.3475	0.3314	0.3354	1077947	{}	\N	2020-10-12 15:15:04.101566
51025	INUV	2020-10-06	0.33	0.35	0.324	0.324	2270208	{}	\N	2020-10-12 15:15:04.105645
51026	INUV	2020-10-07	0.32	0.3432	0.32	0.325	1739545	{}	\N	2020-10-12 15:15:04.10966
51027	INUV	2020-10-08	0.33	0.345	0.3251	0.3412	1324124	{}	\N	2020-10-12 15:15:04.113677
51028	INUV	2020-10-09	0.39	0.4	0.3362	0.339	5797611	{}	\N	2020-10-12 15:15:04.117255
51029	INUV	2020-10-12	0.34	0.349	0.3275	0.344	609364	{}	\N	2020-10-12 15:15:04.121012
51030	KBLM	2020-09-29	11.08	11.09	10.9	10.9	1620	{}	\N	2020-10-12 15:15:06.603568
51031	KBLM	2020-09-30	10.92	10.965	10.9	10.965	1956	{}	\N	2020-10-12 15:15:06.613944
51032	KBLM	2020-10-02	11.05	11.1	11.05	11.0999	862	{}	\N	2020-10-12 15:15:06.619924
51033	KBLM	2020-10-05	11.05	11.05	10.9212	10.9212	2357	{}	\N	2020-10-12 15:15:06.625449
51034	KBLM	2020-10-06	10.9991	10.9991	10.91	10.91	802	{}	\N	2020-10-12 15:15:06.630233
51035	KBLM	2020-10-08	10.9528	10.98	10.9216	10.93	1283	{}	\N	2020-10-12 15:15:06.634766
51036	KBLM	2020-10-09	11.28	11.3	11.17	11.17	1928	{}	\N	2020-10-12 15:15:06.639083
51037	KBLM	2020-10-12	10.95	11.02	10.95	11.02	940	{}	\N	2020-10-12 15:15:06.643696
51038	IMKTA	2020-09-28	36.68	37.67	36.37	37.43	94314	{}	\N	2020-10-12 15:15:08.941968
51039	IMKTA	2020-09-29	37.35	38.005	36.9	37.87	86614	{}	\N	2020-10-12 15:15:08.949889
51040	IMKTA	2020-09-30	37.88	38.23	37.6301	38.09	78526	{}	\N	2020-10-12 15:15:08.955693
51041	IMKTA	2020-10-01	38.11	39.28	38.065	39.17	104840	{}	\N	2020-10-12 15:15:08.960782
51042	IMKTA	2020-10-02	38.7	39.2	38.11	38.86	75469	{}	\N	2020-10-12 15:15:08.965688
51043	IMKTA	2020-10-05	38.8	39.5	38.8	39.21	54016	{}	\N	2020-10-12 15:15:08.970444
51044	IMKTA	2020-10-06	39.5	39.67	38.775	39.03	104918	{}	\N	2020-10-12 15:15:08.974854
51045	IMKTA	2020-10-07	39.27	39.32	38.5	38.56	81976	{}	\N	2020-10-12 15:15:08.979255
51046	IMKTA	2020-10-08	38.86	38.86	37.85	38.07	77067	{}	\N	2020-10-12 15:15:08.9833
51047	IMKTA	2020-10-09	38.08	38.81	38.01	38.55	56476	{}	\N	2020-10-12 15:15:08.987714
51048	IMKTA	2020-10-12	38.54	38.772	38.45	38.45	6606	{}	\N	2020-10-12 15:15:08.991987
51049	IMRN	2020-09-28	7.5066	7.5066	7.08	7.08	41445	{}	\N	2020-10-12 15:15:10.330461
51050	IMRN	2020-09-29	7.2222	7.35	6.9866	7.35	26073	{}	\N	2020-10-12 15:15:10.338786
51051	IMRN	2020-09-30	7.0901	7.5	7.0901	7.49	19399	{}	\N	2020-10-12 15:15:10.344713
51052	IMRN	2020-10-01	7.8	7.8	7.3141	7.33	11160	{}	\N	2020-10-12 15:15:10.349506
51053	IMRN	2020-10-02	6.95	7.33	6.84	7.05	32913	{}	\N	2020-10-12 15:15:10.354419
51054	IMRN	2020-10-05	6.99	7.29	6.99	7.1	18715	{}	\N	2020-10-12 15:15:10.358993
51055	IMRN	2020-10-06	7.2116	7.37	7.03	7.11	30827	{}	\N	2020-10-12 15:15:10.362903
51056	IMRN	2020-10-07	7.01	7.3057	7.01	7.01	33376	{}	\N	2020-10-12 15:15:10.366578
51057	IMRN	2020-10-08	7.0971	7.7	7.02	7.37	40820	{}	\N	2020-10-12 15:15:10.373977
51058	IMRN	2020-10-09	8.23	8.4	7.53	7.78	118940	{}	\N	2020-10-12 15:15:10.379718
51059	IMRN	2020-10-12	8.41	8.8	8	8.55	120360	{}	\N	2020-10-12 15:15:10.384342
51060	INBKL	2020-09-28	24.7	24.79	24.7	24.79	1633	{}	\N	2020-10-12 15:15:11.760087
51061	INBKL	2020-09-29	24.9	24.91	24.9	24.9	352	{}	\N	2020-10-12 15:15:11.770063
51062	INBKL	2020-09-30	25.99	25.99	25.59	25.59	675	{}	\N	2020-10-12 15:15:11.77673
51063	INBKL	2020-10-02	25.03	25.03	25.01	25.01	499	{}	\N	2020-10-12 15:15:11.782055
51064	INBKL	2020-10-05	25.4559	25.4559	25	25	830	{}	\N	2020-10-12 15:15:11.787132
51065	INBKL	2020-10-08	25.1	25.2	25.1	25.2	1025	{}	\N	2020-10-12 15:15:11.792273
51066	INBKL	2020-10-12	25.1	25.1	25.1	25.1	150	{}	\N	2020-10-12 15:15:11.79688
51067	ING	2020-09-28	7.11	7.1899	7.0901	7.12	3892036	{}	\N	2020-10-12 15:15:12.876922
51068	ING	2020-09-29	7.12	7.14	6.99	7.07	7130537	{}	\N	2020-10-12 15:15:12.886096
51069	ING	2020-09-30	7.08	7.17	7.05	7.08	8168187	{}	\N	2020-10-12 15:15:12.892558
51070	ING	2020-10-01	7.09	7.125	7	7.08	10359165	{}	\N	2020-10-12 15:15:12.897842
51071	ING	2020-10-02	7.05	7.28	7.04	7.24	7660643	{}	\N	2020-10-12 15:15:12.902808
51072	ING	2020-10-05	7.34	7.41	7.33	7.4	5749297	{}	\N	2020-10-12 15:15:12.907382
51073	ING	2020-10-06	7.69	7.76	7.54	7.54	12836499	{}	\N	2020-10-12 15:15:12.912245
51074	ING	2020-10-07	7.68	7.73	7.645	7.71	8410821	{}	\N	2020-10-12 15:15:12.916231
51075	ING	2020-10-08	7.86	7.91	7.81	7.86	7937975	{}	\N	2020-10-12 15:15:12.920441
51076	ING	2020-10-09	7.96	7.97	7.84	7.87	5370192	{}	\N	2020-10-12 15:15:12.92474
51077	ING	2020-10-12	7.78	7.7981	7.745	7.7719	2351536	{}	\N	2020-10-12 15:15:12.929123
51078	INS	2020-09-28	39.99	39.99	38.08	39.23	37199	{}	\N	2020-10-12 15:15:14.430157
51079	INS	2020-09-29	38.92	39.83	38.59	39.32	13944	{}	\N	2020-10-12 15:15:14.441275
51080	INS	2020-09-30	39.45	39.47	38.165	38.98	27114	{}	\N	2020-10-12 15:15:14.45028
51081	INS	2020-10-01	39.06	45.47	37.3	44.05	273431	{}	\N	2020-10-12 15:15:14.456893
51082	INS	2020-10-02	43.26	44.53	42.26	42.32	76436	{}	\N	2020-10-12 15:15:14.461714
51083	INS	2020-10-05	44	44.55	41.39	41.77	99346	{}	\N	2020-10-12 15:15:14.465647
51084	INS	2020-10-06	42.58	43.23	40.76	40.86	54835	{}	\N	2020-10-12 15:15:14.470181
51085	INS	2020-10-07	41.59	42.42	40.76	41.85	68379	{}	\N	2020-10-12 15:15:14.474689
51086	INS	2020-10-08	42.48	42.48	40.66	41.02	48496	{}	\N	2020-10-12 15:15:14.478816
51087	INS	2020-10-09	41.25	42.5	40.995	41.8	49908	{}	\N	2020-10-12 15:15:14.482888
51088	INS	2020-10-12	42.49	43	42.1173	43	13927	{}	\N	2020-10-12 15:15:14.486848
51089	INSM	2020-09-28	33.95	34	32.685	32.97	653690	{}	\N	2020-10-12 15:15:17.07721
51090	INSM	2020-09-29	32.81	33.195	32.11	32.755	907058	{}	\N	2020-10-12 15:15:17.082426
51091	INSM	2020-09-30	32.76	33.43	31.79	32.14	580969	{}	\N	2020-10-12 15:15:17.087091
51092	INSM	2020-10-01	32.78	33.4	32.18	33.29	806900	{}	\N	2020-10-12 15:15:17.091289
51093	INSM	2020-10-02	32.35	33.17	31.6	32	826955	{}	\N	2020-10-12 15:15:17.095445
51094	INSM	2020-10-05	32.42	33.44	32.39	33.15	442478	{}	\N	2020-10-12 15:15:17.099576
51095	INSM	2020-10-06	33.39	33.67	32.55	33.07	549024	{}	\N	2020-10-12 15:15:17.104214
51096	INSM	2020-10-07	33.25	34.88	32.98	34.54	1109795	{}	\N	2020-10-12 15:15:17.108833
51097	INSM	2020-10-08	34.72	35.69	34.32	35.53	860131	{}	\N	2020-10-12 15:15:17.113407
51098	INSM	2020-10-09	35.98	36.48	35.53	36.05	867986	{}	\N	2020-10-12 15:15:17.11769
51099	INSM	2020-10-12	36.39	36.59	35.63	36.29	165476	{}	\N	2020-10-12 15:15:17.121913
51100	INTF	2020-09-28	24.45	24.514	24.4	24.45	97476	{}	\N	2020-10-12 15:15:18.148048
51101	INTF	2020-09-29	24.46	24.52	24.3677	24.4101	65253	{}	\N	2020-10-12 15:15:18.158176
51102	INTF	2020-09-30	24.3	24.44	24.24	24.3	155315	{}	\N	2020-10-12 15:15:18.165085
51103	INTF	2020-10-01	24.41	24.52	24.36	24.5	121703	{}	\N	2020-10-12 15:15:18.170867
51104	INTF	2020-10-02	24.2649	24.4742	24.2649	24.43	95817	{}	\N	2020-10-12 15:15:18.176102
51105	INTF	2020-10-05	24.6578	24.84	24.6578	24.84	126276	{}	\N	2020-10-12 15:15:18.181205
51106	INTF	2020-10-06	24.79	24.8	24.47	24.47	88641	{}	\N	2020-10-12 15:15:18.185557
51107	INTF	2020-10-07	24.62	24.7278	24.6042	24.65	77321	{}	\N	2020-10-12 15:15:18.190409
51108	INTF	2020-10-08	24.75	24.84	24.75	24.84	117250	{}	\N	2020-10-12 15:15:18.195317
51109	INTF	2020-10-09	25	25.0673	24.96	25.06	1134258	{}	\N	2020-10-12 15:15:18.200219
51110	INTF	2020-10-12	25.07	25.1245	25.0658	25.12	26453	{}	\N	2020-10-12 15:15:18.205275
51111	INVE	2020-09-28	6	6.42	5.94	6.38	196904	{}	\N	2020-10-12 15:15:19.122077
51112	INVE	2020-09-29	6.25	6.5	6.25	6.3	110164	{}	\N	2020-10-12 15:15:19.131419
51113	INVE	2020-09-30	6.32	6.47	6.26	6.26	133751	{}	\N	2020-10-12 15:15:19.13768
51114	INVE	2020-10-01	6.32	6.377	6.02	6.06	142407	{}	\N	2020-10-12 15:15:19.14312
51115	INVE	2020-10-02	5.845	6.04	5.845	6	73256	{}	\N	2020-10-12 15:15:19.147488
51116	INVE	2020-10-05	6.06	6.2	6.02	6.13	85906	{}	\N	2020-10-12 15:15:19.152017
51117	INVE	2020-10-06	6.13	6.62	6.13	6.35	125953	{}	\N	2020-10-12 15:15:19.15613
51118	INVE	2020-10-07	6.43	6.75	6.43	6.64	54790	{}	\N	2020-10-12 15:15:19.160295
51119	INVE	2020-10-08	6.73	6.79	6.36	6.48	58216	{}	\N	2020-10-12 15:15:19.164203
51120	INVE	2020-10-09	6.475	6.54	6.32	6.43	60823	{}	\N	2020-10-12 15:15:19.167865
51121	INVE	2020-10-12	6.48	6.5	6.43	6.5	8526	{}	\N	2020-10-12 15:15:19.171931
51122	IOO	2020-09-28	56.2	56.38	56.0499	56.3	103178	{}	\N	2020-10-12 15:15:20.143109
51123	IOO	2020-09-29	56.3	56.32	55.8997	55.99	69296	{}	\N	2020-10-12 15:15:20.151268
51124	IOO	2020-09-30	56	56.8	56	56.3	107774	{}	\N	2020-10-12 15:15:20.157755
51125	IOO	2020-10-01	56.8	56.8	56.3	56.59	53007	{}	\N	2020-10-12 15:15:20.16293
51126	IOO	2020-10-02	55.64	56.1999	55.64	55.88	217501	{}	\N	2020-10-12 15:15:20.167766
51127	IOO	2020-10-05	56.36	56.87	56.33	56.86	34634	{}	\N	2020-10-12 15:15:20.173335
51128	IOO	2020-10-06	56.84	56.87	55.88	55.91	879579	{}	\N	2020-10-12 15:15:20.177557
51129	IOO	2020-10-07	56.37	56.79	56.29	56.72	15753	{}	\N	2020-10-12 15:15:20.181336
51130	IOO	2020-10-08	57.07	57.15	56.95	57.12	44403	{}	\N	2020-10-12 15:15:20.185356
51131	IOO	2020-10-09	57.42	57.83	57.42	57.79	49618	{}	\N	2020-10-12 15:15:20.189202
51132	IOO	2020-10-12	58.33	58.61	58.32	58.61	8324	{}	\N	2020-10-12 15:15:20.193216
51133	IOSP	2020-09-28	64.3	66.7396	63.63	63.95	189922	{}	\N	2020-10-12 15:15:22.04164
51134	IOSP	2020-09-29	63.91	64.17	62.61	62.99	92809	{}	\N	2020-10-12 15:15:22.05156
51135	IOSP	2020-09-30	63.2	64.32	62.99	63.32	127930	{}	\N	2020-10-12 15:15:22.058275
51136	IOSP	2020-10-01	63.5	64.18	62.51	62.97	99068	{}	\N	2020-10-12 15:15:22.063825
51137	IOSP	2020-10-02	61.85	64.61	61.85	63.56	136596	{}	\N	2020-10-12 15:15:22.068943
51138	IOSP	2020-10-05	64.38	65.83	63.7	65.59	138651	{}	\N	2020-10-12 15:15:22.075057
51139	IOSP	2020-10-06	66.61	67.46	65.505	65.86	184341	{}	\N	2020-10-12 15:15:22.079868
51140	IOSP	2020-10-07	66.59	66.765	65.16	65.78	163738	{}	\N	2020-10-12 15:15:22.083975
51141	IOSP	2020-10-08	66.74	67.13	66.04	66.8	90206	{}	\N	2020-10-12 15:15:22.088439
51142	IOSP	2020-10-09	67.57	68.15	66.555	67.07	82390	{}	\N	2020-10-12 15:15:22.09343
51143	IOSP	2020-10-12	67.1	67.835	66.775	67.7	23793	{}	\N	2020-10-12 15:15:22.097831
51144	IPB	2020-09-28	29.05	29.3	29.01	29.01	2836	{}	\N	2020-10-12 15:15:23.026191
51145	INFI	2020-09-28	1.09	1.13	1.07	1.13	321066	{}	\N	2020-10-12 15:15:25.843481
51146	INFI	2020-09-29	1.14	1.16	1.08	1.15	1225123	{}	\N	2020-10-12 15:15:25.854226
51147	INFI	2020-09-30	1.14	1.18	1.13	1.17	559016	{}	\N	2020-10-12 15:15:25.860831
51148	INFI	2020-10-01	1.18	1.18	1.11	1.16	328244	{}	\N	2020-10-12 15:15:25.866429
51149	INFI	2020-10-02	1.1432	1.15	1.11	1.135	178873	{}	\N	2020-10-12 15:15:25.871711
51150	INFI	2020-10-05	1.12	1.16	1.11	1.14	393223	{}	\N	2020-10-12 15:15:25.876555
51151	INFI	2020-10-06	1.15	1.17	1.13	1.13	327103	{}	\N	2020-10-12 15:15:25.880883
51152	INFI	2020-10-07	1.13	1.1425	1.11	1.13	163269	{}	\N	2020-10-12 15:15:25.88526
51153	INFI	2020-10-08	1.13	1.18	1.09	1.17	509156	{}	\N	2020-10-12 15:15:25.889627
51154	INFI	2020-10-09	1.18	1.2	1.165	1.175	379982	{}	\N	2020-10-12 15:15:25.894153
51155	INFI	2020-10-12	1.11	1.17	1.11	1.155	180840	{}	\N	2020-10-12 15:15:25.898351
51156	INN	2020-09-28	5.04	5.36	5.04	5.26	676330	{}	\N	2020-10-12 15:15:28.593713
51157	INN	2020-09-29	5.26	5.33	5.151	5.23	1826788	{}	\N	2020-10-12 15:15:28.606229
51158	INN	2020-09-30	5.28	5.48	5.08	5.18	1503725	{}	\N	2020-10-12 15:15:28.613048
51159	INN	2020-10-01	5.18	5.36	5.175	5.35	1077338	{}	\N	2020-10-12 15:15:28.618207
51160	INN	2020-10-02	5.2	5.46	5.1	5.39	1526384	{}	\N	2020-10-12 15:15:28.623644
51161	INN	2020-10-05	5.39	5.46	5.21	5.29	2098128	{}	\N	2020-10-12 15:15:28.628536
51162	INN	2020-10-06	5.39	5.46	5.26	5.28	1674843	{}	\N	2020-10-12 15:15:28.6326
51163	INN	2020-10-07	5.3	5.43	5.27	5.38	2926800	{}	\N	2020-10-12 15:15:28.6366
51164	INN	2020-10-08	5.47	5.51	5.365	5.47	1238161	{}	\N	2020-10-12 15:15:28.640822
51165	INN	2020-10-09	5.52	5.55	5.38	5.49	1714484	{}	\N	2020-10-12 15:15:28.644784
51166	INN	2020-10-12	5.47	5.49	5.36	5.48	297177	{}	\N	2020-10-12 15:15:28.64892
51167	INSI	2020-09-28	20.5	20.7533	20.5	20.55	15945	{}	\N	2020-10-12 15:15:30.475585
51168	INSI	2020-09-29	20.68	20.99	20.5	20.575	40454	{}	\N	2020-10-12 15:15:30.483563
51169	INSI	2020-09-30	20.57	20.9595	20.23	20.23	58106	{}	\N	2020-10-12 15:15:30.489201
51170	INSI	2020-10-01	20.44	20.59	20.375	20.48	30216	{}	\N	2020-10-12 15:15:30.494054
51171	INSI	2020-10-02	20.48	20.65	20.32	20.37	13308	{}	\N	2020-10-12 15:15:30.499245
51172	INSI	2020-10-05	20.51	20.67	20.51	20.67	17784	{}	\N	2020-10-12 15:15:30.503989
51173	INSI	2020-10-06	20.7	20.7	20.33	20.36	19013	{}	\N	2020-10-12 15:15:30.508728
51174	INSI	2020-10-07	20.52	20.617	20.5	20.51	30552	{}	\N	2020-10-12 15:15:30.513865
51175	INSI	2020-10-08	20.61	20.61	20.39	20.39	22109	{}	\N	2020-10-12 15:15:30.517919
51176	INSI	2020-10-09	20.41	20.4345	20.34	20.43	29966	{}	\N	2020-10-12 15:15:30.522126
51177	INSI	2020-10-12	20.43	20.43	20.36	20.37	7300	{}	\N	2020-10-12 15:15:30.526493
51178	INTU	2020-09-28	325.43	326.13	322.26	325.55	1042525	{}	\N	2020-10-12 15:15:31.770132
51179	INTU	2020-09-29	325.54	329.22	323.05	323.64	1034866	{}	\N	2020-10-12 15:15:31.780369
51180	INTU	2020-09-30	323.02	329.14	319.67	326.21	1646411	{}	\N	2020-10-12 15:15:31.786465
51181	INTU	2020-10-01	330.6	333.98	329.85	332.34	1002461	{}	\N	2020-10-12 15:15:31.792263
51182	INTU	2020-10-02	326.16	329.48	319.91	321.89	1037809	{}	\N	2020-10-12 15:15:31.796894
51183	INTU	2020-10-05	330.33	332	327.12	331.36	1256772	{}	\N	2020-10-12 15:15:31.80174
51184	INTU	2020-10-06	331.31	333.87	323.91	325.38	947433	{}	\N	2020-10-12 15:15:31.806442
51185	INTU	2020-10-07	327.74	336.91	327.08	334.91	1186299	{}	\N	2020-10-12 15:15:31.810649
51186	INTU	2020-10-08	336.115	337.76	333.26	334.99	746098	{}	\N	2020-10-12 15:15:31.814722
51187	INTU	2020-10-09	336.62	342.2	335.773	341.81	771809	{}	\N	2020-10-12 15:15:31.8187
51188	INTU	2020-10-12	345.18	348.48	342.21	348.1	410724	{}	\N	2020-10-12 15:15:31.823135
51189	IO	2020-09-28	1.47	1.67	1.47	1.55	260224	{}	\N	2020-10-12 15:15:32.821225
51190	IO	2020-09-29	1.56	1.6091	1.48	1.51	169072	{}	\N	2020-10-12 15:15:32.831198
51191	IO	2020-09-30	1.5	1.5898	1.49	1.5	251140	{}	\N	2020-10-12 15:15:32.837637
51192	IO	2020-10-01	1.5	1.55	1.38	1.45	389019	{}	\N	2020-10-12 15:15:32.843894
51193	IO	2020-10-02	1.42	1.51	1.42	1.47	191691	{}	\N	2020-10-12 15:15:32.84857
51194	IO	2020-10-05	1.5	1.56	1.44	1.47	320646	{}	\N	2020-10-12 15:15:32.853072
51195	IO	2020-10-06	1.59	1.59	1.46	1.47	271494	{}	\N	2020-10-12 15:15:32.857677
51196	IO	2020-10-07	1.46	2.23	1.46	2.08	15627485	{}	\N	2020-10-12 15:15:32.86182
51197	IO	2020-10-08	2.2	2.2	1.8	1.88	8454048	{}	\N	2020-10-12 15:15:32.865992
51198	IO	2020-10-09	1.86	1.88	1.7	1.73	2153559	{}	\N	2020-10-12 15:15:32.870101
51199	IO	2020-10-12	1.71	1.78	1.61	1.6193	452608	{}	\N	2020-10-12 15:15:32.874325
51200	IOVA	2020-09-28	32.32	32.99	31.41	31.92	1288262	{}	\N	2020-10-12 15:15:34.014464
51201	IOVA	2020-09-29	31.88	32.64	31.56	32.3	647117	{}	\N	2020-10-12 15:15:34.022936
51202	IOVA	2020-09-30	32.34	33.38	32.26	32.92	940563	{}	\N	2020-10-12 15:15:34.028908
51203	IOVA	2020-10-01	32.98	33.69	32.8401	33.64	801419	{}	\N	2020-10-12 15:15:34.035072
51204	IOVA	2020-10-02	33.28	33.97	31.44	31.5	712606	{}	\N	2020-10-12 15:15:34.040142
51205	IOVA	2020-10-05	31.85	32.54	31.6	32.06	1278400	{}	\N	2020-10-12 15:15:34.044773
51206	IOVA	2020-10-06	24.95	29.4399	24.79	28.1	13516234	{}	\N	2020-10-12 15:15:34.049415
51207	IOVA	2020-10-07	29.06	29.215	27.41	28.04	2797787	{}	\N	2020-10-12 15:15:34.053236
51208	IOVA	2020-10-08	28.41	29.78	28.0101	29.42	1434483	{}	\N	2020-10-12 15:15:34.057362
51209	IOVA	2020-10-09	29.85	31.44	29.61	31.19	2322123	{}	\N	2020-10-12 15:15:34.061553
51210	IOVA	2020-10-12	31.2	31.39	30.46	31.12	600664	{}	\N	2020-10-12 15:15:34.065521
51211	IPAR	2020-09-28	37.53	38.16	37.39	37.5	102024	{}	\N	2020-10-12 15:15:35.136264
51212	IPAR	2020-09-29	37.78	37.78	36.86	37.06	101061	{}	\N	2020-10-12 15:15:35.147419
51213	IPAR	2020-09-30	37.54	38.09	36.86	37.35	131234	{}	\N	2020-10-12 15:15:35.15429
51214	IPAR	2020-10-01	37.4	39.73	36.96	37.63	86681	{}	\N	2020-10-12 15:15:35.160323
51215	IPAR	2020-10-02	37	38.89	36.63	38.58	111841	{}	\N	2020-10-12 15:15:35.164995
51216	IPAR	2020-10-05	38.94	40.14	38.675	39.62	102140	{}	\N	2020-10-12 15:15:35.169743
51217	IPAR	2020-10-06	39.99	40.43	39.27	39.94	87993	{}	\N	2020-10-12 15:15:35.174549
51218	IPAR	2020-10-07	40.21	40.62	39.37	40.01	131496	{}	\N	2020-10-12 15:15:35.17891
51219	IPAR	2020-10-08	40.36	40.94	40.02	40.91	76959	{}	\N	2020-10-12 15:15:35.182942
51220	IPAR	2020-10-09	41.24	41.32	40.48	41.19	65700	{}	\N	2020-10-12 15:15:35.1869
51221	IPAR	2020-10-12	41.17	41.83	41.07	41.47	6866	{}	\N	2020-10-12 15:15:35.191118
51222	IPFF	2020-09-28	13.8947	13.9092	13.88	13.905	5773	{}	\N	2020-10-12 15:15:36.445867
51223	IPFF	2020-09-29	13.945	13.945	13.8899	13.8899	1357	{}	\N	2020-10-12 15:15:36.456688
51224	IPFF	2020-09-30	13.92	14.0645	13.92	14.0645	5033	{}	\N	2020-10-12 15:15:36.462891
51225	IPFF	2020-10-01	13.95	14.12	13.95	14.12	7499	{}	\N	2020-10-12 15:15:36.468119
51226	IPFF	2020-10-02	14.03	14.05	13.98	13.98	6319	{}	\N	2020-10-12 15:15:36.473527
51227	IPFF	2020-10-05	14.09	14.1199	13.962	13.98	17500	{}	\N	2020-10-12 15:15:36.477634
51228	IPFF	2020-10-06	14.02	14.05	13.95	13.95	3217	{}	\N	2020-10-12 15:15:36.481091
51229	IPFF	2020-10-07	14.14	14.14	14.0201	14.12	6817	{}	\N	2020-10-12 15:15:36.48478
51230	IPFF	2020-10-08	14.1366	14.16	14.1366	14.145	3859	{}	\N	2020-10-12 15:15:36.488327
51231	IPFF	2020-10-09	14.23	14.2658	14.23	14.2658	4715	{}	\N	2020-10-12 15:15:36.492129
51232	IPFF	2020-10-12	14.14	14.3182	14.14	14.315	2309	{}	\N	2020-10-12 15:15:36.495897
51233	IPG	2020-09-28	17.21	17.25	16.875	17.05	3166988	{}	\N	2020-10-12 15:15:37.384917
51234	IPG	2020-09-29	17.01	17.1	16.54	16.62	5252822	{}	\N	2020-10-12 15:15:37.394435
51235	IPG	2020-09-30	16.75	16.99	16.47	16.67	9229115	{}	\N	2020-10-12 15:15:37.401204
51236	IPG	2020-10-01	16.8	16.98	16.5999	16.77	7044458	{}	\N	2020-10-12 15:15:37.406537
51237	IPG	2020-10-02	16.56	17.505	16.56	17.41	7255491	{}	\N	2020-10-12 15:15:37.410783
51238	IPG	2020-10-05	17.63	17.91	17.5	17.62	5327100	{}	\N	2020-10-12 15:15:37.414801
51239	IPG	2020-10-06	17.81	18.035	17.435	17.48	5168415	{}	\N	2020-10-12 15:15:37.418762
51240	IPG	2020-10-07	17.65	18.135	17.65	18.03	4835921	{}	\N	2020-10-12 15:15:37.422997
51241	IPG	2020-10-08	18.14	18.42	17.97	18.34	5552076	{}	\N	2020-10-12 15:15:37.427119
51242	IPG	2020-10-09	18.54	18.67	18.215	18.36	10121454	{}	\N	2020-10-12 15:15:37.431161
51243	IPG	2020-10-12	18.43	18.645	18.395	18.46	2222840	{}	\N	2020-10-12 15:15:37.435373
51244	INDL	2020-09-28	30.94	31.0289	30.5	30.82	76753	{}	\N	2020-10-12 15:15:38.353059
51245	INDL	2020-09-29	30.67	31.15	30.63	30.8	42953	{}	\N	2020-10-12 15:15:38.364231
51246	INDL	2020-09-30	31.24	32	31.12	31.53	55329	{}	\N	2020-10-12 15:15:38.370195
51247	INDL	2020-10-01	32.77	32.99	32.2	32.97	88084	{}	\N	2020-10-12 15:15:38.381265
51248	INDL	2020-10-02	31.3	32.49	31.2523	32.12	125705	{}	\N	2020-10-12 15:15:38.38586
51249	INDL	2020-10-05	33.42	34.01	33.42	33.9	124898	{}	\N	2020-10-12 15:15:38.390042
51250	INDL	2020-10-06	34.71	34.86	34	34.21	84508	{}	\N	2020-10-12 15:15:38.394122
51251	INDL	2020-10-07	35.23	35.4694	35.135	35.24	95135	{}	\N	2020-10-12 15:15:38.398026
51252	INDL	2020-10-08	35.56	35.79	35.2885	35.31	65304	{}	\N	2020-10-12 15:15:38.401939
51253	INDL	2020-10-09	36.39	36.71	35.84	36.55	81326	{}	\N	2020-10-12 15:15:38.405814
51254	INDL	2020-10-12	35.75	36.16	35.6505	36.16	46253	{}	\N	2020-10-12 15:15:38.40989
51255	INSE	2020-09-28	2.99	3	2.89	2.98	47348	{}	\N	2020-10-12 15:15:39.433616
51256	INSE	2020-09-29	3	3.01	2.91	2.94	64753	{}	\N	2020-10-12 15:15:39.445827
51257	INSE	2020-09-30	2.9747	3.045	2.9199	2.96	21164	{}	\N	2020-10-12 15:15:39.452329
51258	INSE	2020-10-01	2.97	3.1	2.9	3.02	20601	{}	\N	2020-10-12 15:15:39.458129
51259	INSE	2020-10-02	2.93	3.2381	2.93	3.22	36244	{}	\N	2020-10-12 15:15:39.463287
51260	INSE	2020-10-05	3.2009	3.28	3.06	3.16	27317	{}	\N	2020-10-12 15:15:39.467951
51261	INSE	2020-10-06	3.1	3.22	3.0765	3.12	33676	{}	\N	2020-10-12 15:15:39.472403
51262	INSE	2020-10-07	3.15	3.26	3.12	3.2	14594	{}	\N	2020-10-12 15:15:39.476293
51263	INSE	2020-10-08	3.1894	3.2806	3.14	3.2	7741	{}	\N	2020-10-12 15:15:39.480348
51264	INSE	2020-10-09	3.2	3.52	3.2	3.49	182007	{}	\N	2020-10-12 15:15:39.484449
51265	INSE	2020-10-12	3.45	3.68	3.45	3.5401	88327	{}	\N	2020-10-12 15:15:39.488243
51266	INVA	2020-09-28	10.9	10.985	10.52	10.67	514408	{}	\N	2020-10-12 15:15:40.798512
51267	INVA	2020-09-29	10.64	10.67	10.315	10.33	582469	{}	\N	2020-10-12 15:15:40.808165
51268	INVA	2020-09-30	10.335	10.72	10.335	10.45	660813	{}	\N	2020-10-12 15:15:40.815262
51269	INVA	2020-10-01	10.43	10.71	10.1	10.28	960786	{}	\N	2020-10-12 15:15:40.819915
51270	INVA	2020-10-02	10.14	10.36	10.08	10.22	529158	{}	\N	2020-10-12 15:15:40.825617
51271	INVA	2020-10-05	10.31	10.61	10.29	10.52	474400	{}	\N	2020-10-12 15:15:40.83133
51272	INVA	2020-10-06	10.58	10.71	10.24	10.25	425913	{}	\N	2020-10-12 15:15:40.835137
51273	INVA	2020-10-07	10.27	10.48	10.15	10.39	810598	{}	\N	2020-10-12 15:15:40.839296
51274	INVA	2020-10-08	10.51	10.71	10.36	10.57	317889	{}	\N	2020-10-12 15:15:40.843686
51275	INVA	2020-10-09	10.6	10.7	10.4	10.42	248020	{}	\N	2020-10-12 15:15:40.8477
51276	INVA	2020-10-12	10.42	10.53	10.41	10.48	82270	{}	\N	2020-10-12 15:15:40.851376
51277	INWK	2020-09-28	3	3	2.99	2.99	230666	{}	\N	2020-10-12 15:15:41.995248
51278	INWK	2020-09-29	3	3	2.99	2.995	87410	{}	\N	2020-10-12 15:15:42.007096
51279	INWK	2020-09-30	3	3	2.99	2.99	544075	{}	\N	2020-10-12 15:15:42.01281
51280	IOR	2020-09-30	12.22	12.95	11.84	11.84	799	{}	\N	2020-10-12 15:15:43.984249
51281	IOR	2020-10-05	12.3	13	11.02	11.02	4984	{}	\N	2020-10-12 15:15:43.998001
51282	IOR	2020-10-06	11.4164	12	11.4164	11.45	1226	{}	\N	2020-10-12 15:15:44.004361
51283	IOR	2020-10-07	11.45	11.45	11.45	11.45	312	{}	\N	2020-10-12 15:15:44.010232
51284	IOR	2020-10-08	11.5	11.5	11.5	11.5	1104	{}	\N	2020-10-12 15:15:44.016292
51285	IOR	2020-10-09	12.29	12.7	12.28	12.7	746	{}	\N	2020-10-12 15:15:44.021665
51286	IPDN	2020-09-28	1.19	1.25	0.9511	1.09	2456075	{}	\N	2020-10-12 15:15:46.438004
51287	IPDN	2020-09-29	0.9505	1.1799	0.8968	1.055	1745668	{}	\N	2020-10-12 15:15:46.449102
51288	IPDN	2020-09-30	0.9562	1.06	0.9411	0.9613	226488	{}	\N	2020-10-12 15:15:46.455163
51289	IPDN	2020-10-01	0.963	1.0398	0.951052	0.956829	119422	{}	\N	2020-10-12 15:15:46.460872
51290	IPDN	2020-10-02	0.92	1	0.92	0.9771	157597	{}	\N	2020-10-12 15:15:46.465865
51291	IPDN	2020-10-05	0.95	0.97	0.9101	0.94	200128	{}	\N	2020-10-12 15:15:46.471058
51292	IPDN	2020-10-06	0.93	1.04	0.9101	0.9417	296574	{}	\N	2020-10-12 15:15:46.475801
51293	IPDN	2020-10-07	0.949	1.465	0.9301	1.41	4330739	{}	\N	2020-10-12 15:15:46.479961
51294	IPDN	2020-10-08	1.24	1.31	1.08	1.16	5132133	{}	\N	2020-10-12 15:15:46.484359
51295	IPDN	2020-10-09	1.1	1.1622	1.05	1.12	822548	{}	\N	2020-10-12 15:15:46.488281
51296	IPDN	2020-10-12	1.08	1.13	1.08	1.0951	126956	{}	\N	2020-10-12 15:15:46.492508
51297	IPGP	2020-09-28	167.93	167.93	165.56	167.22	120731	{}	\N	2020-10-12 15:15:47.434003
51298	IPGP	2020-09-29	167.81	169.335	166.345	168.26	100676	{}	\N	2020-10-12 15:15:47.447693
51299	IPGP	2020-09-30	168.61	171.85	166.59	169.97	156742	{}	\N	2020-10-12 15:15:47.453303
51300	IPGP	2020-10-01	171.78	176.97	171.39	173	194560	{}	\N	2020-10-12 15:15:47.458975
51301	IPGP	2020-10-02	168.83	170.73	167.88	168.83	137211	{}	\N	2020-10-12 15:15:47.463972
51302	IPGP	2020-10-05	170.77	174.52	170.77	173.88	144327	{}	\N	2020-10-12 15:15:47.468995
51303	IPGP	2020-10-06	173.2	177.35	172.01	173.71	91834	{}	\N	2020-10-12 15:15:47.47406
51304	IPGP	2020-10-07	176.23	177.8226	175.27	176.45	170048	{}	\N	2020-10-12 15:15:47.478322
51305	IPGP	2020-10-08	175.88	179.9	175.88	179.83	112508	{}	\N	2020-10-12 15:15:47.482581
51306	IPGP	2020-10-09	180.88	182.67	180.06	181.03	99279	{}	\N	2020-10-12 15:15:47.486757
51307	IPGP	2020-10-12	183.01	184.56	181.67	181.87	26523	{}	\N	2020-10-12 15:15:47.492212
51308	IPOS	2020-09-28	32.48	32.48	32.11	32.33	31188	{}	\N	2020-10-12 15:15:49.160835
51309	IPOS	2020-09-29	32.42	32.42	32.1	32.31	23219	{}	\N	2020-10-12 15:15:49.173116
51310	IPOS	2020-09-30	32.23	32.39	32.12	32.2921	318875	{}	\N	2020-10-12 15:15:49.179986
51311	IPOS	2020-10-01	32.81	32.81	32.39	32.74	49245	{}	\N	2020-10-12 15:15:49.185422
51312	IPOS	2020-10-02	32.46	32.69	32.25	32.44	40623	{}	\N	2020-10-12 15:15:49.190661
51313	IPOS	2020-10-05	32.75	32.91	32.7	32.9	15550	{}	\N	2020-10-12 15:15:49.195351
51314	IPOS	2020-10-06	33.15	33.39	33.0038	33.01	21777	{}	\N	2020-10-12 15:15:49.199935
51315	IPOS	2020-10-07	33.3	33.3499	33.15	33.2343	24415	{}	\N	2020-10-12 15:15:49.204121
51316	IPOS	2020-10-08	33.12	33.39	33.12	33.34	18037	{}	\N	2020-10-12 15:15:49.20843
51317	IPOS	2020-10-09	33.43	33.9699	33.31	33.7471	37766	{}	\N	2020-10-12 15:15:49.212513
51318	IPOS	2020-10-12	34.15	34.298	34.11	34.22	19415	{}	\N	2020-10-12 15:15:49.216618
51319	INFY	2020-09-28	13.67	13.705	13.6	13.63	5718252	{}	\N	2020-10-12 15:15:51.21364
51320	INFY	2020-09-29	13.64	13.69	13.49	13.56	15148409	{}	\N	2020-10-12 15:15:51.223111
51321	INFY	2020-09-30	13.58	13.94	13.56	13.81	27734355	{}	\N	2020-10-12 15:15:51.229127
51322	INFY	2020-10-01	13.84	13.9	13.71	13.76	10685662	{}	\N	2020-10-12 15:15:51.235714
51323	INFY	2020-10-02	13.69	13.74	13.57	13.67	14698188	{}	\N	2020-10-12 15:15:51.240887
51324	INFY	2020-10-05	14.17	14.2	14.02	14.05	20592118	{}	\N	2020-10-12 15:15:51.245615
51325	INFY	2020-10-06	14.14	14.19	13.89	13.89	21247292	{}	\N	2020-10-12 15:15:51.249529
51326	INFY	2020-10-07	14.31	14.58	14.29	14.54	35788464	{}	\N	2020-10-12 15:15:51.25353
51327	INFY	2020-10-08	14.84	14.9	14.72	14.8	22499480	{}	\N	2020-10-12 15:15:51.258141
51328	INFY	2020-10-09	14.97	15.255	14.915	15.2	16571476	{}	\N	2020-10-12 15:15:51.262614
51329	INFY	2020-10-12	15.35	15.5	15.34	15.395	11998379	{}	\N	2020-10-12 15:15:51.266953
51330	INOD	2020-09-28	2.34	2.36	2.14	2.33	24173	{}	\N	2020-10-12 15:15:53.623905
51331	INOD	2020-09-29	2.36	3.45	2.3395	3.25	2102501	{}	\N	2020-10-12 15:15:53.635592
51332	INOD	2020-09-30	2.9035	3.21	2.9	3.09	345922	{}	\N	2020-10-12 15:15:53.644919
51333	INOD	2020-10-01	3.14	3.83	3.05	3.09	492095	{}	\N	2020-10-12 15:15:53.650404
51334	INOD	2020-10-02	3.01	3.5	2.9	3.35	252691	{}	\N	2020-10-12 15:15:53.655831
51335	INOD	2020-10-05	3.35	3.8	3.34	3.38	110087	{}	\N	2020-10-12 15:15:53.660577
51336	INOD	2020-10-06	3.4	3.4485	3.0601	3.2	90288	{}	\N	2020-10-12 15:15:53.665305
51337	INOD	2020-10-07	3.27	3.295	3.1	3.18	57206	{}	\N	2020-10-12 15:15:53.669503
51338	INOD	2020-10-08	3.2202	3.24	3.05	3.13	97117	{}	\N	2020-10-12 15:15:53.674366
51339	INOD	2020-10-09	3.19	3.3	3.0926	3.16	46100	{}	\N	2020-10-12 15:15:53.678575
51340	INOD	2020-10-12	3.21	3.2499	3.11	3.11	9495	{}	\N	2020-10-12 15:15:53.682594
51341	IONS	2020-09-28	49.32	49.915	48.84	49.03	489730	{}	\N	2020-10-12 15:15:55.724013
51342	IONS	2020-09-29	48.84	49.2	47.86	48.48	1095743	{}	\N	2020-10-12 15:15:55.733666
51343	IONS	2020-09-30	48.6	49	47.19	47.45	1023888	{}	\N	2020-10-12 15:15:55.740751
51344	IONS	2020-10-01	47.69	48.6	47.32	47.76	1578736	{}	\N	2020-10-12 15:15:55.746015
51345	IONS	2020-10-02	47.22	47.49	45.12	45.65	1757555	{}	\N	2020-10-12 15:15:55.750344
51346	IONS	2020-10-05	46.06	47.95	46.04	47.58	2182610	{}	\N	2020-10-12 15:15:55.755015
51347	IONS	2020-10-06	47.89	48.05	45.09	45.3	1240591	{}	\N	2020-10-12 15:15:55.758884
51348	IONS	2020-10-07	45.5	46.63	45.47	46.42	957905	{}	\N	2020-10-12 15:15:55.762729
51349	IONS	2020-10-08	46.75	47.245	46.5	46.86	880864	{}	\N	2020-10-12 15:15:55.766545
51350	IONS	2020-10-09	47	47.08	45.8744	45.93	596626	{}	\N	2020-10-12 15:15:55.771393
51351	IONS	2020-10-12	46.43	46.6034	45.6	46.6	198342	{}	\N	2020-10-12 15:15:55.776141
51352	IPHI	2020-09-28	110.99	112.08	109.48	111.59	503934	{}	\N	2020-10-12 15:15:57.615693
51353	IPHI	2020-09-29	111.59	114.2	111.3	112.96	986044	{}	\N	2020-10-12 15:15:57.623898
51354	IPHI	2020-09-30	112.49	114.03	111	112.25	1682485	{}	\N	2020-10-12 15:15:57.629362
51355	IPHI	2020-10-01	114.51	114.68	111.305	113.53	1622362	{}	\N	2020-10-12 15:15:57.634031
51356	IPHI	2020-10-02	110.28	112.58	109.19	109.36	2021903	{}	\N	2020-10-12 15:15:57.638861
51357	IPHI	2020-10-05	110	114.8	109.58	114.17	2019568	{}	\N	2020-10-12 15:15:57.643485
51358	IPHI	2020-10-06	113.49	118.62	113.49	114.03	1352873	{}	\N	2020-10-12 15:15:57.648465
51359	IPHI	2020-10-07	115.43	116.2875	112.8	113.45	1326981	{}	\N	2020-10-12 15:15:57.652894
51360	IPHI	2020-10-08	114.29	117.14	114.29	116.13	1141349	{}	\N	2020-10-12 15:15:57.659155
51361	IPHI	2020-10-09	117.94	122.94	117.805	122.09	2020862	{}	\N	2020-10-12 15:15:57.663246
51362	IPHI	2020-10-12	123.77	124.895	122.36	123.56	278949	{}	\N	2020-10-12 15:15:57.66816
51363	IPWR	2020-09-28	6.8	7.21	6.3	6.3	27115	{}	\N	2020-10-12 15:15:59.729869
51364	IPWR	2020-09-29	6.45	6.6675	5.9	6.11	38757	{}	\N	2020-10-12 15:15:59.738574
51365	IPWR	2020-09-30	6.11	6.4	6	6.2	31342	{}	\N	2020-10-12 15:15:59.744684
51366	IPWR	2020-10-01	6.24	6.4497	6.12	6.12	9900	{}	\N	2020-10-12 15:15:59.749632
51367	IPWR	2020-10-02	6.03	6.1475	5.93	6.1	11789	{}	\N	2020-10-12 15:15:59.753997
51368	IPWR	2020-10-05	6.05	6.6999	5.9657	6.49	70619	{}	\N	2020-10-12 15:15:59.758732
51369	IPWR	2020-10-06	6.6401	7.2485	6.49	6.49	49826	{}	\N	2020-10-12 15:15:59.762825
51370	IPWR	2020-10-07	6.61	7.15	6.61	6.94	26088	{}	\N	2020-10-12 15:15:59.7668
51371	IPWR	2020-10-08	7.1	7.1999	6.6945	6.86	29507	{}	\N	2020-10-12 15:15:59.770818
51372	IPWR	2020-10-09	6.89	7.1	6.76	7.1	11537	{}	\N	2020-10-12 15:15:59.774839
51373	IPWR	2020-10-12	7.09	7.09	6.88	6.9901	3965	{}	\N	2020-10-12 15:15:59.779671
51374	IQI	2020-09-28	12.18	12.23	12.179	12.21	47589	{}	\N	2020-10-12 15:16:00.955543
51375	IQI	2020-09-29	12.2221	12.26	12.22	12.26	146188	{}	\N	2020-10-12 15:16:00.966117
51376	IQI	2020-09-30	12.32	12.36	12.21	12.36	220811	{}	\N	2020-10-12 15:16:00.971783
51377	IQI	2020-10-01	12.35	12.4	12.31	12.4	109749	{}	\N	2020-10-12 15:16:00.97705
51378	IQI	2020-10-02	12.38	12.39	12.31	12.37	157837	{}	\N	2020-10-12 15:16:00.982153
51379	IQI	2020-10-05	12.38	12.38	12.25	12.29	136876	{}	\N	2020-10-12 15:16:00.986877
51380	IQI	2020-10-06	12.3	12.3	12.28	12.29	162498	{}	\N	2020-10-12 15:16:00.991807
51381	IQI	2020-10-07	12.3	12.33	12.26	12.26	274406	{}	\N	2020-10-12 15:16:00.996378
51382	IQI	2020-10-08	12.33	12.34	12.3	12.33	63618	{}	\N	2020-10-12 15:16:01.000731
51383	IQI	2020-10-09	12.33	12.397	12.29	12.29	93532	{}	\N	2020-10-12 15:16:01.005153
51384	IQI	2020-10-12	12.38	12.38	12.335	12.3699	35010	{}	\N	2020-10-12 15:16:01.009704
51385	IRIX	2020-09-28	1.89	1.95	1.89	1.91	4706	{}	\N	2020-10-12 15:16:02.342411
51386	IRIX	2020-09-29	1.915	1.9509	1.915	1.94	5432	{}	\N	2020-10-12 15:16:02.348315
51387	IRIX	2020-09-30	1.96	1.9698	1.945	1.96	25379	{}	\N	2020-10-12 15:16:02.353754
51388	IRIX	2020-10-01	1.93	2	1.9	1.94	3079	{}	\N	2020-10-12 15:16:02.358357
51389	IRIX	2020-10-02	1.9	2.015	1.9	1.91	3792	{}	\N	2020-10-12 15:16:02.362712
51390	IRIX	2020-10-05	2.02	2.02	1.96	1.96	2944	{}	\N	2020-10-12 15:16:02.366774
51391	IRIX	2020-10-06	2.028	2.028	1.92	1.92	3256	{}	\N	2020-10-12 15:16:02.374568
51392	IRIX	2020-10-07	1.945	1.969	1.9	1.92	6566	{}	\N	2020-10-12 15:16:02.380885
51393	IRIX	2020-10-08	1.91	2	1.91	2	36160	{}	\N	2020-10-12 15:16:02.384852
51394	IRIX	2020-10-09	2	2.04	1.99	2.02	15722	{}	\N	2020-10-12 15:16:02.389057
51395	IRIX	2020-10-12	2.03	2.05	2.03	2.05	1352	{}	\N	2020-10-12 15:16:02.393145
51396	GWB	2020-09-28	12.35	12.99	12.35	12.82	305567	{}	\N	2020-10-12 15:16:03.598928
51397	GWB	2020-09-29	12.57	12.74	12.13	12.41	471017	{}	\N	2020-10-12 15:16:03.607787
51398	GWB	2020-09-30	12.48	12.75	12.29	12.45	751209	{}	\N	2020-10-12 15:16:03.615727
51399	GWB	2020-10-01	12.63	12.81	12.24	12.64	615553	{}	\N	2020-10-12 15:16:03.623663
51400	GWB	2020-10-02	12.3	13.05	12.3	13.01	521898	{}	\N	2020-10-12 15:16:03.631636
51401	GWB	2020-10-05	13.26	13.7	13.23	13.69	612640	{}	\N	2020-10-12 15:16:03.637717
51402	GWB	2020-10-06	14.01	14.3	13.6	13.67	734212	{}	\N	2020-10-12 15:16:03.644804
51403	GWB	2020-10-07	13.87	14.31	13.87	14.23	785651	{}	\N	2020-10-12 15:16:03.650452
51404	GWB	2020-10-08	14.41	14.63	14.17	14.59	560659	{}	\N	2020-10-12 15:16:03.655124
51405	GWB	2020-10-09	14.77	14.83	14.11	14.22	553244	{}	\N	2020-10-12 15:16:03.659839
51406	GWB	2020-10-12	14.17	14.48	14.09	14.48	63606	{}	\N	2020-10-12 15:16:03.665762
51407	HAP	2020-09-28	32.6401	32.6401	32.512	32.512	2008	{}	\N	2020-10-12 15:16:05.967505
51408	HAP	2020-09-29	32.372	32.451	32.279	32.279	1014	{}	\N	2020-10-12 15:16:05.974161
51409	HAP	2020-09-30	32.54	32.54	32.3918	32.3918	1716	{}	\N	2020-10-12 15:16:05.979437
51410	HAP	2020-10-01	32.21	32.32	32.21	32.3	2325	{}	\N	2020-10-12 15:16:05.984267
51411	HAP	2020-10-02	31.72	32.4532	31.72	32.32	9660	{}	\N	2020-10-12 15:16:05.988826
51412	HAP	2020-10-05	32.86	32.995	32.86	32.99	1050	{}	\N	2020-10-12 15:16:05.993085
51413	HAP	2020-10-06	33.18	33.18	32.6826	32.6826	615	{}	\N	2020-10-12 15:16:05.997272
51414	HAP	2020-10-07	33.05	33.22	33.03	33.17	12728	{}	\N	2020-10-12 15:16:06.001871
51415	HAP	2020-10-08	33.45	33.64	33.41	33.64	1091	{}	\N	2020-10-12 15:16:06.006638
51416	HAP	2020-10-09	33.7409	33.7812	33.7409	33.7812	303	{}	\N	2020-10-12 15:16:06.011233
51417	HAP	2020-10-12	33.8	33.8	33.8	33.8	482	{}	\N	2020-10-12 15:16:06.015671
51418	HBB	2020-09-28	19.24	19.82	19.24	19.41	82130	{}	\N	2020-10-12 15:16:07.141305
51419	HBB	2020-09-29	19.3471	19.59	18.5	19.43	206357	{}	\N	2020-10-12 15:16:07.147012
51420	HBB	2020-09-30	19.41	19.7784	19.22	19.45	150690	{}	\N	2020-10-12 15:16:07.151709
51421	HBB	2020-10-01	19.65	20	19.45	19.59	83067	{}	\N	2020-10-12 15:16:07.156725
51422	HBB	2020-10-02	19.44	21.345	19.44	20.82	139329	{}	\N	2020-10-12 15:16:07.160648
51423	HBB	2020-10-05	21.04	21.1	20.02	20.45	85328	{}	\N	2020-10-12 15:16:07.164701
51424	HBB	2020-10-06	20.48	20.615	19.52	19.52	62565	{}	\N	2020-10-12 15:16:07.168639
51425	HBB	2020-10-07	19.76	20.45	19.76	20.16	48193	{}	\N	2020-10-12 15:16:07.174236
51426	HBB	2020-10-08	20.29	20.82	20.05	20.34	46568	{}	\N	2020-10-12 15:16:07.179207
51427	HBB	2020-10-09	20.57	21.48	20.19	21.05	54872	{}	\N	2020-10-12 15:16:07.184677
51428	HBB	2020-10-12	21.16	21.97	21.16	21.53	23478	{}	\N	2020-10-12 15:16:07.190191
51429	HDEF	2020-09-28	20.37	20.39	20.3	20.39	130223	{}	\N	2020-10-12 15:16:09.424479
51430	HDEF	2020-09-29	20.37	20.42	20.274	20.375	82367	{}	\N	2020-10-12 15:16:09.430964
51431	HDEF	2020-09-30	20.43	20.53	20.3481	20.36	65345	{}	\N	2020-10-12 15:16:09.436975
51432	HDEF	2020-10-01	20.39	20.4447	20.3401	20.42	91000	{}	\N	2020-10-12 15:16:09.442037
51433	HDEF	2020-10-02	20.26	20.57	20.26	20.5	351066	{}	\N	2020-10-12 15:16:09.446854
51434	HDEF	2020-10-05	20.67	20.82	20.67	20.82	66872	{}	\N	2020-10-12 15:16:09.451301
51435	HDEF	2020-10-06	20.89	20.89	20.61	20.63	69167	{}	\N	2020-10-12 15:16:09.45628
51436	HDEF	2020-10-07	20.74	20.8034	20.74	20.79	88330	{}	\N	2020-10-12 15:16:09.460827
51437	HDEF	2020-10-08	20.87	20.89	20.8182	20.87	62725	{}	\N	2020-10-12 15:16:09.465373
51438	HDEF	2020-10-09	20.95	21.02	20.9301	21.01	49313	{}	\N	2020-10-12 15:16:09.47021
51439	HDEF	2020-10-12	21.07	21.07	21	21.048	15280	{}	\N	2020-10-12 15:16:09.474998
51440	HEDJ	2020-09-28	62.22	62.44	62.11	62.35	163299	{}	\N	2020-10-12 15:16:10.501254
51441	HEDJ	2020-09-29	62.25	62.41	62	62.15	193867	{}	\N	2020-10-12 15:16:10.510502
51442	HEDJ	2020-09-30	62.1	62.38	61.59	61.66	410869	{}	\N	2020-10-12 15:16:10.515397
51443	HEDJ	2020-10-01	62.05	62.1	61.72	62	198104	{}	\N	2020-10-12 15:16:10.52072
51444	HEDJ	2020-10-02	61.49	62.19	61.48	62.06	277649	{}	\N	2020-10-12 15:16:10.525188
51445	HEDJ	2020-10-05	62.45	62.94	62.45	62.94	195394	{}	\N	2020-10-12 15:16:10.529418
51446	HEDJ	2020-10-06	62.9	62.95	62.18	62.22	237654	{}	\N	2020-10-12 15:16:10.533588
51447	HEDJ	2020-10-07	62.67	63.06	62.63	62.97	117296	{}	\N	2020-10-12 15:16:10.538045
51448	HEDJ	2020-10-08	63.29	63.585	63.28	63.56	165342	{}	\N	2020-10-12 15:16:10.542958
51449	HEDJ	2020-10-09	63.62	63.8	63.52	63.72	110829	{}	\N	2020-10-12 15:16:10.547056
51450	HEDJ	2020-10-12	64	64.15	63.97	64.11	56617	{}	\N	2020-10-12 15:16:10.551016
51451	HESM	2020-09-28	15.23	15.36	14.85	15.25	206533	{}	\N	2020-10-12 15:16:11.499829
51452	HESM	2020-09-29	15.11	15.18	14.71	15.11	593361	{}	\N	2020-10-12 15:16:11.508571
51453	HESM	2020-09-30	15.12	15.77	15.06	15.1	196906	{}	\N	2020-10-12 15:16:11.514748
51454	HESM	2020-10-01	15.06	15.23	14.59	14.94	138481	{}	\N	2020-10-12 15:16:11.519969
51455	HESM	2020-10-02	14.56	15.1	14.56	15.02	121799	{}	\N	2020-10-12 15:16:11.524971
51456	HESM	2020-10-05	15.58	15.76	15.26	15.47	216846	{}	\N	2020-10-12 15:16:11.529597
51457	HESM	2020-10-06	15.55	15.8538	15.11	15.33	201873	{}	\N	2020-10-12 15:16:11.533884
51458	HESM	2020-10-07	15.43	15.43	14.95	15.23	173172	{}	\N	2020-10-12 15:16:11.538964
51459	HESM	2020-10-08	15.47	16.11	15.14	16.1	320904	{}	\N	2020-10-12 15:16:11.543296
51460	HESM	2020-10-09	16.27	16.66	16.11	16.62	174606	{}	\N	2020-10-12 15:16:11.547784
51461	HESM	2020-10-12	16.55	16.725	16.5	16.675	33768	{}	\N	2020-10-12 15:16:11.552073
51462	HEWW	2020-09-28	13.71	13.7152	13.7	13.7152	1701	{}	\N	2020-10-12 15:16:12.706961
51463	HEWW	2020-09-29	13.7224	13.7224	13.7224	13.7224	155	{}	\N	2020-10-12 15:16:12.712202
51464	HEWW	2020-09-30	13.82	13.88	13.818	13.85	3577	{}	\N	2020-10-12 15:16:12.716537
51465	HEWW	2020-10-01	13.5405	13.5405	13.5405	13.5405	76	{}	\N	2020-10-12 15:16:12.720961
51466	HEWW	2020-10-02	13.5404	13.5404	13.5404	13.5404	15	{}	\N	2020-10-12 15:16:12.725694
51467	HEWW	2020-10-05	13.5999	13.5999	13.5999	13.5999	1	{}	\N	2020-10-12 15:16:12.731937
51468	HEWW	2020-10-06	13.66	13.66	13.6407	13.6407	459	{}	\N	2020-10-12 15:16:12.73768
51469	HEWW	2020-10-07	13.9123	13.9123	13.9123	13.9123	53	{}	\N	2020-10-12 15:16:12.743838
51470	HEWW	2020-10-08	13.94	14.2392	13.9399	14.2392	3744	{}	\N	2020-10-12 15:16:12.749229
51471	HEWW	2020-10-09	14.39	14.4	14.2889	14.2889	912	{}	\N	2020-10-12 15:16:12.754418
51472	HFXI	2020-09-28	19.95	20.05	19.92	19.95	17782	{}	\N	2020-10-12 15:16:14.14263
51473	HFXI	2020-09-29	19.99	20.07	19.871	19.9	43196	{}	\N	2020-10-12 15:16:14.15169
51474	HFXI	2020-09-30	19.9601	19.9601	19.815	19.89	6098	{}	\N	2020-10-12 15:16:14.158448
51475	HFXI	2020-10-01	19.93	19.93	19.83	19.86	21963	{}	\N	2020-10-12 15:16:14.163709
51476	HFXI	2020-10-02	19.66	19.949	19.66	19.87	23056	{}	\N	2020-10-12 15:16:14.168349
51477	HFXI	2020-10-05	20.0731	20.15	20.05	20.15	10540	{}	\N	2020-10-12 15:16:14.173661
51478	HFXI	2020-10-06	20.2367	20.2367	19.97	19.99	11869	{}	\N	2020-10-12 15:16:14.177533
51479	HFXI	2020-10-07	20.17	20.24	20.08	20.23	48327	{}	\N	2020-10-12 15:16:14.181475
51480	HFXI	2020-10-08	20.3093	20.37	20.24	20.35	71578	{}	\N	2020-10-12 15:16:14.185415
51481	HFXI	2020-10-09	20.45	20.49	20.35	20.36	19481	{}	\N	2020-10-12 15:16:14.189861
51482	HFXI	2020-10-12	20.37	20.47	20.37	20.47	3525	{}	\N	2020-10-12 15:16:14.193876
51483	HIO	2020-09-28	4.94	4.94	4.91	4.93	149029	{}	\N	2020-10-12 15:16:17.542123
51484	HIO	2020-09-29	4.94	4.95	4.89	4.89	452088	{}	\N	2020-10-12 15:16:17.55168
51485	HIO	2020-09-30	4.91	4.93	4.89	4.9	1488421	{}	\N	2020-10-12 15:16:17.55863
51486	HIO	2020-10-01	4.93	4.96	4.9282	4.94	356119	{}	\N	2020-10-12 15:16:17.565249
51487	HIO	2020-10-02	4.93	5	4.925	4.94	491707	{}	\N	2020-10-12 15:16:17.571292
51488	HIO	2020-10-05	4.97	5	4.97	4.99	274878	{}	\N	2020-10-12 15:16:17.575961
51489	HIO	2020-10-06	5	5	4.97	4.97	196183	{}	\N	2020-10-12 15:16:17.58051
51490	HIO	2020-10-07	4.99	5.01	4.98	4.98	334592	{}	\N	2020-10-12 15:16:17.584712
51491	HIO	2020-10-08	5	5	4.98	4.98	566128	{}	\N	2020-10-12 15:16:17.589089
51492	HIO	2020-10-09	4.98	5.03	4.97	5	696432	{}	\N	2020-10-12 15:16:17.593203
51493	HIO	2020-10-12	5.03	5.03	5.01	5.02	48934	{}	\N	2020-10-12 15:16:17.597276
51494	IRCP	2020-09-28	7.41	7.595	7.06	7.595	1685	{}	\N	2020-10-12 15:16:19.205024
51495	IRCP	2020-09-29	7.65	7.99	7.0201	7.98	117729	{}	\N	2020-10-12 15:16:19.211167
51496	IRCP	2020-09-30	7.3	7.78	7.2	7.765	234316	{}	\N	2020-10-12 15:16:19.216443
51497	IRCP	2020-10-01	7.77	7.8	7.5	7.585	327358	{}	\N	2020-10-12 15:16:19.220985
51498	IRCP	2020-10-02	7.44	7.44	7.44	7.44	853	{}	\N	2020-10-12 15:16:19.225474
51499	IRCP	2020-10-05	7.4377	7.4377	7.17	7.17	998	{}	\N	2020-10-12 15:16:19.230169
51500	IRCP	2020-10-06	7.18	7.2886	6.97	6.97	3895	{}	\N	2020-10-12 15:16:19.234617
51501	IRCP	2020-10-07	7.12	7.1837	6.97	7.13	18465	{}	\N	2020-10-12 15:16:19.239114
51502	IRCP	2020-10-08	7.21	7.34	7	7.32	30130	{}	\N	2020-10-12 15:16:19.243765
51503	IRCP	2020-10-09	7.01	8.06	7.01	7.85	23699	{}	\N	2020-10-12 15:16:19.248095
51504	IRM	2020-09-28	27.02	27.0458	26.52	26.6	2766215	{}	\N	2020-10-12 15:16:20.262654
51505	IRM	2020-09-29	26.53	26.82	26.35	26.76	3179433	{}	\N	2020-10-12 15:16:20.271226
51506	IRM	2020-09-30	27	27.275	26.49	26.79	4387002	{}	\N	2020-10-12 15:16:20.27769
51507	IRM	2020-10-01	26.84	27.055	26.55	27.01	4017435	{}	\N	2020-10-12 15:16:20.283389
51508	IRM	2020-10-02	26.68	27.39	26.57	27.34	5465663	{}	\N	2020-10-12 15:16:20.289587
51509	IRM	2020-10-05	27.48	27.55	26.995	27.35	4111172	{}	\N	2020-10-12 15:16:20.294053
51510	IRM	2020-10-06	27.46	27.63	26.97	27.11	4134713	{}	\N	2020-10-12 15:16:20.2987
51511	IRM	2020-10-07	27.11	27.37	26.94	27.1	3519175	{}	\N	2020-10-12 15:16:20.303811
51512	IRM	2020-10-08	27.2	28.31	27.13	28.24	5594071	{}	\N	2020-10-12 15:16:20.308794
51513	IRM	2020-10-09	28.51	28.69	28.065	28.25	4493858	{}	\N	2020-10-12 15:16:20.313795
51514	IRM	2020-10-12	28.25	28.335	28.155	28.2589	655575	{}	\N	2020-10-12 15:16:20.318385
51515	ISBC	2020-09-28	7.15	7.4	7.12	7.29	1147927	{}	\N	2020-10-12 15:16:21.308788
51516	ISBC	2020-09-29	7.24	7.27	7.03	7.255	1220187	{}	\N	2020-10-12 15:16:21.318994
51517	ISBC	2020-09-30	7.32	7.405	7.19	7.26	960236	{}	\N	2020-10-12 15:16:21.326006
51518	ISBC	2020-10-01	7.32	7.34	7.16	7.29	1225412	{}	\N	2020-10-12 15:16:21.331569
51519	ISBC	2020-10-02	7.16	7.55	7.131	7.51	1025010	{}	\N	2020-10-12 15:16:21.338322
51520	ISBC	2020-10-05	7.55	7.79	7.55	7.71	1100887	{}	\N	2020-10-12 15:16:21.343161
51521	ISBC	2020-10-06	7.86	7.99	7.64	7.66	1220767	{}	\N	2020-10-12 15:16:21.347614
51522	ISBC	2020-10-07	7.71	7.95	7.71	7.91	2100540	{}	\N	2020-10-12 15:16:21.351852
51523	ISBC	2020-10-08	8	8.05	7.89	7.89	1032140	{}	\N	2020-10-12 15:16:21.357075
51524	ISBC	2020-10-09	7.97	8.02	7.71	7.76	1052838	{}	\N	2020-10-12 15:16:21.36141
51525	ISBC	2020-10-12	7.71	7.88	7.7	7.86	301087	{}	\N	2020-10-12 15:16:21.36565
51526	ISNS	2020-09-28	3.56	3.69	3.56	3.575	2831	{}	\N	2020-10-12 15:16:22.35044
51527	ISNS	2020-09-29	3.615	3.69	3.52	3.66	3771	{}	\N	2020-10-12 15:16:22.35983
51528	ISNS	2020-09-30	3.7	3.7	3.61	3.63	4089	{}	\N	2020-10-12 15:16:22.366586
51529	ISNS	2020-10-01	3.61	3.63	3.59	3.61	2181	{}	\N	2020-10-12 15:16:22.379749
51530	ISNS	2020-10-02	3.52	3.67	3.52	3.6	3437	{}	\N	2020-10-12 15:16:22.385662
51531	ISNS	2020-10-05	3.7	3.7	3.5101	3.56	3847	{}	\N	2020-10-12 15:16:22.391013
51532	ISNS	2020-10-06	3.523	3.98	3.523	3.8	84367	{}	\N	2020-10-12 15:16:22.395995
51533	ISNS	2020-10-07	3.7365	3.94	3.7365	3.9	10905	{}	\N	2020-10-12 15:16:22.401233
51534	ISNS	2020-10-08	3.86	3.92	3.74	3.89	7541	{}	\N	2020-10-12 15:16:22.406811
51535	ISNS	2020-10-09	3.825	3.89	3.6401	3.85	3922	{}	\N	2020-10-12 15:16:22.411743
51536	ISNS	2020-10-12	3.83	3.86	3.83	3.83	1573	{}	\N	2020-10-12 15:16:22.416434
51537	ITCI	2020-09-28	27.01	27.405	26.2	26.33	586818	{}	\N	2020-10-12 15:16:23.343742
51538	ITCI	2020-09-29	26.14	26.38	25.42	25.54	703252	{}	\N	2020-10-12 15:16:23.352708
51539	ITCI	2020-09-30	25.48	26.03	25.16	25.66	1264966	{}	\N	2020-10-12 15:16:23.359395
51540	ITCI	2020-10-01	25.89	27.01	25.35	25.74	1214997	{}	\N	2020-10-12 15:16:23.364477
51541	ITCI	2020-10-02	25	25.33	23.75	23.81	1506669	{}	\N	2020-10-12 15:16:23.369786
51542	ITCI	2020-10-05	24.3	26.49	24.28	26	1609235	{}	\N	2020-10-12 15:16:23.379464
51543	ITCI	2020-10-06	26.04	26.425	25.34	25.84	772964	{}	\N	2020-10-12 15:16:23.384473
51544	ITCI	2020-10-07	25.98	26.76	25.91	26.66	720913	{}	\N	2020-10-12 15:16:23.389433
51545	ITCI	2020-10-08	26.81	27.41	26.33	27.06	746457	{}	\N	2020-10-12 15:16:23.394837
51546	ITCI	2020-10-09	27.11	27.61	26.65	27.16	491575	{}	\N	2020-10-12 15:16:23.401099
51547	ITCI	2020-10-12	27.45	27.5	26.01	26.74	180666	{}	\N	2020-10-12 15:16:23.408844
51548	ITRN	2020-09-28	14.14	14.445	13.97	14.03	14372	{}	\N	2020-10-12 15:16:25.496443
51549	ITRN	2020-09-29	14.02	14.56	13.8196	14.5	28345	{}	\N	2020-10-12 15:16:25.506718
51550	ITRN	2020-09-30	14.55	14.61	13.76	13.92	32851	{}	\N	2020-10-12 15:16:25.512525
51551	ITRN	2020-10-01	13.905	14.91	13.905	14.62	30000	{}	\N	2020-10-12 15:16:25.517764
51552	ITRN	2020-10-02	14.42	14.98	14.2086	14.79	8563	{}	\N	2020-10-12 15:16:25.522823
51553	ITRN	2020-10-05	14.605	15.18	14.605	14.88	11649	{}	\N	2020-10-12 15:16:25.527224
51554	ITRN	2020-10-06	14.8	14.83	14.37	14.37	16969	{}	\N	2020-10-12 15:16:25.531277
51555	ITRN	2020-10-07	14.58	14.7531	14.17	14.18	28909	{}	\N	2020-10-12 15:16:25.535587
51556	ITRN	2020-10-08	14.22	14.6061	13.74	13.8	39508	{}	\N	2020-10-12 15:16:25.540307
51557	ITRN	2020-10-09	14	14.43	13.91	14.38	13300	{}	\N	2020-10-12 15:16:25.544479
51558	ITRN	2020-10-12	14.34	14.3973	14.275	14.3131	6198	{}	\N	2020-10-12 15:16:25.548681
51559	IVE	2020-09-28	111.83	112.925	111.83	112.4	497353	{}	\N	2020-10-12 15:16:26.501916
51560	IVE	2020-09-29	112.39	112.46	111.098	111.53	366706	{}	\N	2020-10-12 15:16:26.512799
51561	IVE	2020-09-30	111.92	113.56	111.8002	112.45	573904	{}	\N	2020-10-12 15:16:26.519647
51562	IVE	2020-10-01	113	113.38	111.925	112.57	571680	{}	\N	2020-10-12 15:16:26.525449
51563	IVE	2020-10-02	110.97	113.51	110.95	112.81	640548	{}	\N	2020-10-12 15:16:26.530478
51564	IVE	2020-10-05	113.77	114.695	113.5194	114.58	483130	{}	\N	2020-10-12 15:16:26.535505
51565	IVE	2020-10-06	114.89	115.82	113.34	113.48	452160	{}	\N	2020-10-12 15:16:26.540402
51566	IVE	2020-10-07	114.4	115.675	114.4	115.32	602088	{}	\N	2020-10-12 15:16:26.544945
51567	IVE	2020-10-08	115.96	116.79	115.9	116.77	395006	{}	\N	2020-10-12 15:16:26.549919
51568	IVE	2020-10-09	117.46	117.62	116.6199	116.96	403936	{}	\N	2020-10-12 15:16:26.555159
51569	IVE	2020-10-12	117.19	117.79	117.08	117.79	101590	{}	\N	2020-10-12 15:16:26.559504
51570	IVR	2020-09-28	2.85	2.97	2.8	2.82	12049320	{}	\N	2020-10-12 15:16:27.842106
51571	IVR	2020-09-29	2.86	2.87	2.7	2.75	8658759	{}	\N	2020-10-12 15:16:27.852381
51572	IVR	2020-09-30	2.74	2.77	2.7	2.71	11067451	{}	\N	2020-10-12 15:16:27.858852
51573	IVR	2020-10-01	2.78	2.79	2.7	2.77	6684657	{}	\N	2020-10-12 15:16:27.864027
51574	IVR	2020-10-02	2.67	2.9	2.65	2.89	8292664	{}	\N	2020-10-12 15:16:27.869098
51575	IVR	2020-10-05	2.9	2.93	2.82	2.88	7764321	{}	\N	2020-10-12 15:16:27.874574
51576	IVR	2020-10-06	2.88	2.92	2.8	2.8	11429060	{}	\N	2020-10-12 15:16:27.879028
51577	IVR	2020-10-07	2.81	2.8459	2.76	2.8	8632087	{}	\N	2020-10-12 15:16:27.883448
51578	IVR	2020-10-08	2.79	2.9	2.76	2.85	8169299	{}	\N	2020-10-12 15:16:27.888172
51579	IVR	2020-10-09	2.86	2.8777	2.8	2.84	5117099	{}	\N	2020-10-12 15:16:27.892469
51580	IVR	2020-10-12	2.83	2.84	2.78	2.8017	2292141	{}	\N	2020-10-12 15:16:27.896769
51581	IWN	2020-09-28	98.33	100.41	98.33	100.02	1039271	{}	\N	2020-10-12 15:16:29.244014
51582	IWN	2020-09-29	99.9	99.9	97.73	99.09	1070041	{}	\N	2020-10-12 15:16:29.255415
51583	IWN	2020-09-30	99.45	101.1099	98.6	99.37	657206	{}	\N	2020-10-12 15:16:29.262715
51584	IWN	2020-10-01	99.99	100.9	98.99	100.9	477625	{}	\N	2020-10-12 15:16:29.269008
51585	IWN	2020-10-02	98.89	102.94	98.89	102.52	1067622	{}	\N	2020-10-12 15:16:29.27471
51586	IWN	2020-10-05	103.58	105.16	103.58	105.03	847675	{}	\N	2020-10-12 15:16:29.279565
51587	IWN	2020-10-06	105.95	107.93	104.48	104.62	2247263	{}	\N	2020-10-12 15:16:29.284164
51588	IWN	2020-10-07	105.8	107.06	105.74	106.77	1308049	{}	\N	2020-10-12 15:16:29.28894
51589	IWN	2020-10-08	107.83	108.66	107.16	108.48	1145760	{}	\N	2020-10-12 15:16:29.293541
51590	IWN	2020-10-09	109.55	109.86	108.04	108.47	1942825	{}	\N	2020-10-12 15:16:29.297599
51591	IWN	2020-10-12	108.91	109.33	108.27	109.25	382144	{}	\N	2020-10-12 15:16:29.301731
51592	IX	2020-09-28	65.84	66.84	65.54	65.59	51878	{}	\N	2020-10-12 15:16:30.7387
51593	IX	2020-09-29	65.08	65.08	64.26	64.47	45996	{}	\N	2020-10-12 15:16:30.751502
51594	IX	2020-09-30	62.62	63.24	62.07	62.57	97477	{}	\N	2020-10-12 15:16:30.758573
51595	IX	2020-10-01	62.61	63.06	62.38	62.91	58186	{}	\N	2020-10-12 15:16:30.76394
51596	IX	2020-10-02	62.44	63.53	62.44	63.34	110788	{}	\N	2020-10-12 15:16:30.768676
51597	IX	2020-10-05	64.48	65.437	64.48	65.31	69427	{}	\N	2020-10-12 15:16:30.774818
51598	IX	2020-10-06	65.28	65.49	64.66	64.79	80190	{}	\N	2020-10-12 15:16:30.779456
51599	IX	2020-10-07	65.36	65.72	65.16	65.39	58159	{}	\N	2020-10-12 15:16:30.783192
51600	IX	2020-10-08	65.62	65.62	65.15	65.3	42447	{}	\N	2020-10-12 15:16:30.787487
51601	IX	2020-10-09	65.56	65.82	65.32	65.53	48281	{}	\N	2020-10-12 15:16:30.79147
51602	IX	2020-10-12	65.53	65.72	65.4	65.7	18355	{}	\N	2020-10-12 15:16:30.795344
51603	IYH	2020-09-28	224.2	225.46	223.49	223.95	46872	{}	\N	2020-10-12 15:16:32.089887
51604	IYH	2020-09-29	224.72	225.369	223.18	223.93	26247	{}	\N	2020-10-12 15:16:32.098574
51605	IYH	2020-09-30	224.88	229	224.88	227.2	74390	{}	\N	2020-10-12 15:16:32.105706
51606	IYH	2020-10-01	228.72	228.9291	226.015	226.87	26144	{}	\N	2020-10-12 15:16:32.111446
51607	IYH	2020-10-02	224.4	226.67	223.201	224.67	24062	{}	\N	2020-10-12 15:16:32.115833
51608	IYH	2020-10-05	226.56	229.51	226.56	229.51	20096	{}	\N	2020-10-12 15:16:32.120627
51609	IYH	2020-10-06	230.38	230.38	226.545	226.69	21446	{}	\N	2020-10-12 15:16:32.124553
51610	IYH	2020-10-07	228.5	231.615	228.5	231.25	52473	{}	\N	2020-10-12 15:16:32.129092
51611	IYH	2020-10-08	232.75	232.75	231.83	232.42	16665	{}	\N	2020-10-12 15:16:32.13292
51612	IYH	2020-10-09	233.78	234.65	233.21	233.98	12399	{}	\N	2020-10-12 15:16:32.136692
51613	IYH	2020-10-12	235.57	236.43	234.9157	236.412	10418	{}	\N	2020-10-12 15:16:32.140725
51614	IZRL	2020-09-28	24.73	24.9699	24.59	24.59	19072	{}	\N	2020-10-12 15:16:33.055405
51615	IZRL	2020-09-29	24.78	24.9699	24.72	24.84	9651	{}	\N	2020-10-12 15:16:33.062118
51616	IZRL	2020-09-30	25	25.4	25	25.14	47118	{}	\N	2020-10-12 15:16:33.067573
51617	IZRL	2020-10-01	25.23	25.5384	25.2	25.38	30229	{}	\N	2020-10-12 15:16:33.0728
51618	IZRL	2020-10-02	25.4	25.43	25.0301	25.11	9431	{}	\N	2020-10-12 15:16:33.076785
51619	IZRL	2020-10-05	25.55	25.9263	25.54	25.86	9709	{}	\N	2020-10-12 15:16:33.080576
51620	IZRL	2020-10-06	25.83	26.065	25.5326	25.6758	13832	{}	\N	2020-10-12 15:16:33.084718
51621	IZRL	2020-10-07	25.84	26.18	25.71	26.08	43578	{}	\N	2020-10-12 15:16:33.089416
51622	IZRL	2020-10-08	26.22	26.5	26.16	26.42	19261	{}	\N	2020-10-12 15:16:33.093701
51623	IZRL	2020-10-09	26.63	26.99	26.51	26.83	41656	{}	\N	2020-10-12 15:16:33.097699
51624	IZRL	2020-10-12	27.18	27.18	26.68	26.78	13196	{}	\N	2020-10-12 15:16:33.101584
51625	JBGS	2020-09-28	26.67	27.395	26.57	27.23	645315	{}	\N	2020-10-12 15:16:34.010144
51626	JBGS	2020-09-29	27.09	27.21	26.69	27.04	1472514	{}	\N	2020-10-12 15:16:34.020579
51627	JBGS	2020-09-30	27.23	27.56	26.44	26.74	1579737	{}	\N	2020-10-12 15:16:34.027259
51628	JBGS	2020-10-01	26.76	27.32	26.68	27.32	835480	{}	\N	2020-10-12 15:16:34.032296
51629	JBGS	2020-10-02	26.79	27.71	26.7	27.62	1104843	{}	\N	2020-10-12 15:16:34.037417
51630	JBGS	2020-10-05	27.89	27.93	27.38	27.84	620254	{}	\N	2020-10-12 15:16:34.0423
51631	JBGS	2020-10-06	28.04	28.1	27.45	27.61	913394	{}	\N	2020-10-12 15:16:34.046737
51632	JBGS	2020-10-07	27.67	27.82	27.3	27.42	698608	{}	\N	2020-10-12 15:16:34.050902
51633	JBGS	2020-10-08	27.57	27.84	27.44	27.8	1259294	{}	\N	2020-10-12 15:16:34.05673
51634	JBGS	2020-10-09	28	28	27.24	27.62	757011	{}	\N	2020-10-12 15:16:34.061558
51635	JBGS	2020-10-12	27.55	27.64	27.23	27.55	78807	{}	\N	2020-10-12 15:16:34.065909
51636	IRDM	2020-09-28	24.9	25.3	24.84	25.21	726374	{}	\N	2020-10-12 15:16:35.630812
51637	IRDM	2020-09-29	25.23	25.81	25.23	25.7699	437819	{}	\N	2020-10-12 15:16:35.63876
51638	IRDM	2020-09-30	25.92	25.94	25.12	25.58	661014	{}	\N	2020-10-12 15:16:35.644603
51639	IRDM	2020-10-01	25.7	26.29	25.03	26.27	645062	{}	\N	2020-10-12 15:16:35.649543
51640	IRDM	2020-10-02	25.74	26.5	25.6157	26.3	326157	{}	\N	2020-10-12 15:16:35.654431
51641	IRDM	2020-10-05	26.81	27.52	26.73	27.48	606310	{}	\N	2020-10-12 15:16:35.659127
51642	IRDM	2020-10-06	27.75	28.4767	27.41	27.48	788026	{}	\N	2020-10-12 15:16:35.663013
51643	IRDM	2020-10-07	27.84	28.7	27.4	28.57	1062838	{}	\N	2020-10-12 15:16:35.666692
51644	IRDM	2020-10-08	28.92	29.1	28.08	28.62	620807	{}	\N	2020-10-12 15:16:35.670772
51645	IRDM	2020-10-09	28.61	29.05	28.595	28.91	409307	{}	\N	2020-10-12 15:16:35.6748
51646	IRDM	2020-10-12	28.9	29.5	28.82	29.3	130807	{}	\N	2020-10-12 15:16:35.678658
51647	IRT	2020-09-28	11.46	11.77	11.46	11.65	489249	{}	\N	2020-10-12 15:16:38.251864
51648	IRT	2020-09-29	11.61	11.69	11.41	11.5	1261446	{}	\N	2020-10-12 15:16:38.261869
51649	IRT	2020-09-30	11.53	11.945	11.45	11.59	1911289	{}	\N	2020-10-12 15:16:38.268208
51650	IRT	2020-10-01	11.5	11.88	11.475	11.87	737780	{}	\N	2020-10-12 15:16:38.274084
51651	IRT	2020-10-02	11.69	12.14	11.59	11.95	1501706	{}	\N	2020-10-12 15:16:38.278925
51652	IRT	2020-10-05	12.04	12.125	11.875	12.04	725763	{}	\N	2020-10-12 15:16:38.28365
51653	IRT	2020-10-06	12.15	12.28	11.93	11.94	1104673	{}	\N	2020-10-12 15:16:38.288004
51654	IRT	2020-10-07	11.91	12.08	11.79	11.94	740303	{}	\N	2020-10-12 15:16:38.292171
51655	IRT	2020-10-08	12	12.16	11.9	12.15	957789	{}	\N	2020-10-12 15:16:38.296106
51656	IRT	2020-10-09	12.26	12.35	12.02	12.07	669822	{}	\N	2020-10-12 15:16:38.300054
51657	IRT	2020-10-12	12.02	12.14	11.9	12	132133	{}	\N	2020-10-12 15:16:38.304541
51658	ISTB	2020-09-28	51.52	51.55	51.5	51.55	287660	{}	\N	2020-10-12 15:16:39.848746
51659	ISTB	2020-09-29	51.54	51.59	51.51	51.52	349172	{}	\N	2020-10-12 15:16:39.859754
51660	ISTB	2020-09-30	51.57	51.58	51.51	51.55	401383	{}	\N	2020-10-12 15:16:39.866519
51661	ISTB	2020-10-01	51.46	51.51	51.42	51.49	360657	{}	\N	2020-10-12 15:16:39.872417
51662	ISTB	2020-10-02	51.49	51.5162	51.431	51.45	425666	{}	\N	2020-10-12 15:16:39.877627
51663	ISTB	2020-10-05	51.48	51.53	51.4	51.48	448628	{}	\N	2020-10-12 15:16:39.882414
51664	ISTB	2020-10-06	51.48	51.54	51.43	51.5	439554	{}	\N	2020-10-12 15:16:39.88726
51665	ISTB	2020-10-07	51.4699	51.54	51.45	51.52	498516	{}	\N	2020-10-12 15:16:39.891594
51666	ISTB	2020-10-08	51.47	51.58	51.47	51.51	469244	{}	\N	2020-10-12 15:16:39.89576
51667	ISTB	2020-10-09	51.49	51.56	51.45	51.54	480514	{}	\N	2020-10-12 15:16:39.89985
51668	ISTB	2020-10-12	51.52	51.56	51.52	51.545	62983	{}	\N	2020-10-12 15:16:39.904843
51669	ITCB	2020-09-28	3.9732	3.99	3.85	3.86	16185	{}	\N	2020-10-12 15:16:41.455779
51670	ITCB	2020-09-29	3.9	4.02	3.88	3.93	15898	{}	\N	2020-10-12 15:16:41.465841
51671	ITCB	2020-09-30	3.93	4.05	3.81	3.86	12304	{}	\N	2020-10-12 15:16:41.473042
51672	ITCB	2020-10-01	3.86	4	3.86	4	20032	{}	\N	2020-10-12 15:16:41.478928
51673	ITCB	2020-10-02	3.99	4.1471	3.93	4.09	84848	{}	\N	2020-10-12 15:16:41.484628
51674	ITCB	2020-10-05	4.11	4.13	3.91	3.98	107962	{}	\N	2020-10-12 15:16:41.489828
51675	ITCB	2020-10-06	3.97	3.98	3.81	3.84	65426	{}	\N	2020-10-12 15:16:41.494244
51676	ITCB	2020-10-07	3.88	3.97	3.82	3.85	37464	{}	\N	2020-10-12 15:16:41.499562
51677	ITCB	2020-10-08	3.91	3.91	3.75	3.87	37080	{}	\N	2020-10-12 15:16:41.504223
51678	ITCB	2020-10-09	3.95	4.04	3.86	3.89	68442	{}	\N	2020-10-12 15:16:41.509375
51679	ITCB	2020-10-12	3.9365	3.95	3.85	3.895	7250	{}	\N	2020-10-12 15:16:41.513659
51680	ITI	2020-09-28	4.15	4.2	4.13	4.14	130612	{}	\N	2020-10-12 15:16:42.382917
51681	ITI	2020-09-29	4.25	4.25	4.14	4.19	91316	{}	\N	2020-10-12 15:16:42.394199
51682	ITI	2020-09-30	4.19	4.2	4.05	4.1	97413	{}	\N	2020-10-12 15:16:42.399838
51683	ITI	2020-10-01	4.11	4.15	3.99	4.09	183633	{}	\N	2020-10-12 15:16:42.405313
51684	ITI	2020-10-02	4.01	4.11	3.96	4.05	154658	{}	\N	2020-10-12 15:16:42.409745
51685	ITI	2020-10-05	4.1	4.14	4.07	4.09	219390	{}	\N	2020-10-12 15:16:42.413982
51686	ITI	2020-10-06	4.095	4.14	4	4.065	211781	{}	\N	2020-10-12 15:16:42.418353
51687	ITI	2020-10-07	4.1	4.18	4.09	4.15	187486	{}	\N	2020-10-12 15:16:42.422769
51688	ITI	2020-10-08	4.22	4.38	4.2	4.36	224644	{}	\N	2020-10-12 15:16:42.426991
51689	ITI	2020-10-09	4.38	4.44	4.33	4.38	101636	{}	\N	2020-10-12 15:16:42.431406
51690	ITI	2020-10-12	4.37	4.41	4.34	4.35	41912	{}	\N	2020-10-12 15:16:42.436045
51691	IUSG	2020-09-28	79.33	79.72	79.09	79.69	435506	{}	\N	2020-10-12 15:16:43.362837
51692	IUSG	2020-09-29	79.7	79.89	79.2673	79.42	321485	{}	\N	2020-10-12 15:16:43.388006
51693	IUSG	2020-09-30	79.57	80.74	79.4999	79.95	480844	{}	\N	2020-10-12 15:16:43.393053
51694	IUSG	2020-10-01	80.79	80.98	80.2673	80.74	335490	{}	\N	2020-10-12 15:16:43.398142
51695	IUSG	2020-10-02	79.09	80.17	78.99	79.39	352850	{}	\N	2020-10-12 15:16:43.404492
51696	IUSG	2020-10-05	79.98	81	79.98	80.94	345469	{}	\N	2020-10-12 15:16:43.409328
51697	IUSG	2020-10-06	81	81.3699	79.52	79.61	479622	{}	\N	2020-10-12 15:16:43.414349
51698	IUSG	2020-10-07	80.49	81.26	80.46	81.12	262825	{}	\N	2020-10-12 15:16:43.419348
51699	IUSG	2020-10-08	81.67	81.75	81.2767	81.57	301677	{}	\N	2020-10-12 15:16:43.424103
51700	IUSG	2020-10-09	81.93	82.7	81.93	82.7	276841	{}	\N	2020-10-12 15:16:43.428482
51701	IUSG	2020-10-12	83.6	84.125	83.415	84.125	83378	{}	\N	2020-10-12 15:16:43.43235
51702	IVAL	2020-09-28	24.98	25.09	24.98	25.045	2850	{}	\N	2020-10-12 15:16:44.383024
51703	IVAL	2020-09-29	25.1099	25.1099	24.9976	24.9976	28600	{}	\N	2020-10-12 15:16:44.391799
51704	IVAL	2020-09-30	24.87	24.94	24.81	24.86	4024	{}	\N	2020-10-12 15:16:44.397859
51705	IVAL	2020-10-01	25.07	25.07	24.82	24.91	113258	{}	\N	2020-10-12 15:16:44.403537
51706	IVAL	2020-10-02	24.699	24.8299	24.699	24.76	8250	{}	\N	2020-10-12 15:16:44.409416
51707	IVAL	2020-10-05	25.18	25.21	25.1201	25.21	13157	{}	\N	2020-10-12 15:16:44.414246
51708	IVAL	2020-10-06	25.32	25.38	25.11	25.11	9667	{}	\N	2020-10-12 15:16:44.419522
51709	IVAL	2020-10-07	25.16	25.34	25.16	25.34	17721	{}	\N	2020-10-12 15:16:44.423951
51710	IVAL	2020-10-08	25.43	25.45	25.3481	25.44	27928	{}	\N	2020-10-12 15:16:44.428325
51711	IVAL	2020-10-09	25.41	25.49	25.4069	25.48	28815	{}	\N	2020-10-12 15:16:44.433402
51712	IVAL	2020-10-12	25.4	25.4	25.29	25.338	1035	{}	\N	2020-10-12 15:16:44.437905
51713	IVOO	2020-09-28	124.07	125.83	124.07	125.54	94700	{}	\N	2020-10-12 15:16:45.586824
51714	IVOO	2020-09-29	125.47	125.875	124.095	124.88	22381	{}	\N	2020-10-12 15:16:45.59558
51715	IVOO	2020-09-30	125.24	126.83	124.91	125.54	34569	{}	\N	2020-10-12 15:16:45.601027
51716	IVOO	2020-10-01	125.91	127.2028	125.91	126.935	26307	{}	\N	2020-10-12 15:16:45.606337
51717	IVOO	2020-10-02	124.98	128.88	124.31	128.5875	19273	{}	\N	2020-10-12 15:16:45.610951
51718	IVOO	2020-10-05	130.2	131.45	130.2	131.4	42111	{}	\N	2020-10-12 15:16:45.615396
51719	IVOO	2020-10-06	132.11	133.895	130.47	130.75	43464	{}	\N	2020-10-12 15:16:45.620017
51720	IVOO	2020-10-07	132.1	133.47	132.1	133.11	28488	{}	\N	2020-10-12 15:16:45.624549
51721	IVOO	2020-10-08	134.21	134.53	133.655	134.39	15175	{}	\N	2020-10-12 15:16:45.629401
51722	IVOO	2020-10-09	135.4	135.66	134.45	134.91	21496	{}	\N	2020-10-12 15:16:45.634081
51723	IVOO	2020-10-12	135.41	135.79	134.901	135.79	6538	{}	\N	2020-10-12 15:16:45.638951
51724	IVV	2020-09-28	334.36	336.1	333.31	335.36	3189579	{}	\N	2020-10-12 15:16:46.711319
51725	IVV	2020-09-29	335.21	335.895	332.8001	333.51	2432807	{}	\N	2020-10-12 15:16:46.72069
51726	IVV	2020-09-30	334.31	339.4688	334.1	336.06	4206346	{}	\N	2020-10-12 15:16:46.729997
51727	IVV	2020-10-01	338.83	339.87	336.18	338.24	3205342	{}	\N	2020-10-12 15:16:46.735139
51728	IVV	2020-10-02	332.82	337.09	332.385	335.05	3685696	{}	\N	2020-10-12 15:16:46.740425
51729	IVV	2020-10-05	337.21	341.12	337.2	340.9	2883064	{}	\N	2020-10-12 15:16:46.744583
51730	IVV	2020-10-06	341.13	343.35	335.56	336.06	3563142	{}	\N	2020-10-12 15:16:46.748759
51731	IVV	2020-10-07	339.28	342.8	339.28	341.9	2848489	{}	\N	2020-10-12 15:16:46.794744
51732	IVV	2020-10-08	344.07	345.03	343.06	344.98	4027309	{}	\N	2020-10-12 15:16:46.803896
51733	IVV	2020-10-09	346.79	348.54	346.09	348.01	3548573	{}	\N	2020-10-12 15:16:46.81055
51734	IVV	2020-10-12	350.78	352.93	350.28	352.93	562254	{}	\N	2020-10-12 15:16:46.815666
51735	IRTC	2020-09-28	230.13	233.35	225.615	228.74	162884	{}	\N	2020-10-12 15:16:48.117061
51736	IRTC	2020-09-29	230.77	239.01	230.67	233.58	160584	{}	\N	2020-10-12 15:16:48.127031
51737	IRTC	2020-09-30	231.76	241.93	230.09	238.11	338859	{}	\N	2020-10-12 15:16:48.133709
51738	IRTC	2020-10-01	238.66	241	226.02	230.62	282356	{}	\N	2020-10-12 15:16:48.139134
51739	IRTC	2020-10-02	228.65	228.65	222.84	227.08	223159	{}	\N	2020-10-12 15:16:48.143714
51740	IRTC	2020-10-05	228	229.22	221.61	225.72	313825	{}	\N	2020-10-12 15:16:48.148303
51741	IRTC	2020-10-06	226.36	238.99	223.0533	234	307686	{}	\N	2020-10-12 15:16:48.152523
51742	IRTC	2020-10-07	237.96	246.35	234.54	245.26	241915	{}	\N	2020-10-12 15:16:48.157356
51743	IRTC	2020-10-08	246.155	248.2144	238.33	244.77	202818	{}	\N	2020-10-12 15:16:48.161364
51744	IRTC	2020-10-09	246.68	252.11	245.365	251.88	169391	{}	\N	2020-10-12 15:16:48.165127
51745	IRTC	2020-10-12	258.89	258.89	251.66	256.29	43487	{}	\N	2020-10-12 15:16:48.169269
51746	ISHG	2020-09-29	81.53	81.53	81.34	81.3801	2111	{}	\N	2020-10-12 15:16:49.978567
51747	ISHG	2020-09-30	81.41	81.7359	81.41	81.56	2690	{}	\N	2020-10-12 15:16:49.987619
51748	ISHG	2020-10-05	82.17	82.17	81.9106	81.9106	3906	{}	\N	2020-10-12 15:16:49.99469
51749	ISHG	2020-10-06	81.89	81.92	81.5873	81.61	2079	{}	\N	2020-10-12 15:16:49.999706
51750	ISHG	2020-10-07	81.72	81.75	81.63	81.75	2058	{}	\N	2020-10-12 15:16:50.004692
51751	ISHG	2020-10-09	81.83	82.405	81.83	82.3	9161	{}	\N	2020-10-12 15:16:50.009478
51752	ISHG	2020-10-12	81.98	81.98	81.98	81.98	1004	{}	\N	2020-10-12 15:16:50.013343
51753	ISTR	2020-09-28	12.66	13	12.66	12.94	4788	{}	\N	2020-10-12 15:16:51.420224
51754	ISTR	2020-09-29	13	13	12.54	12.99	2598	{}	\N	2020-10-12 15:16:51.430473
51755	ISTR	2020-09-30	13	13.27	12.6	12.82	5905	{}	\N	2020-10-12 15:16:51.437058
51756	ISTR	2020-10-01	12.93	13.565	12.67	12.86	11289	{}	\N	2020-10-12 15:16:51.4427
51757	ISTR	2020-10-02	12.83	13.095	12.65	12.96	18628	{}	\N	2020-10-12 15:16:51.447352
51758	ISTR	2020-10-05	13	13.99	13	13.99	43438	{}	\N	2020-10-12 15:16:51.45196
51759	ISTR	2020-10-06	14.05	14.15	13.3274	13.49	17780	{}	\N	2020-10-12 15:16:51.456431
51760	ISTR	2020-10-07	13.81	14.4	13.77	14.35	14703	{}	\N	2020-10-12 15:16:51.460556
51761	ISTR	2020-10-08	14.5299	14.65	14.13	14.45	15027	{}	\N	2020-10-12 15:16:51.464542
51762	ISTR	2020-10-09	14.53	14.53	13.95	14.2	5075	{}	\N	2020-10-12 15:16:51.468657
51763	ISTR	2020-10-12	14.13	14.27	13.8	14.22	2361	{}	\N	2020-10-12 15:16:51.472807
51764	ITGR	2020-09-28	59.49	60.19	58.59	58.8	434099	{}	\N	2020-10-12 15:16:52.673833
51765	ITGR	2020-09-29	58.76	60.21	58.14	58.68	396475	{}	\N	2020-10-12 15:16:52.684513
51766	ITGR	2020-09-30	58.8	59.48	58.65	59.01	658862	{}	\N	2020-10-12 15:16:52.69218
51767	ITGR	2020-10-01	59.24	60.22	57.87	58.16	316141	{}	\N	2020-10-12 15:16:52.698441
51768	ITGR	2020-10-02	56.86	58.645	56.62	57.76	427729	{}	\N	2020-10-12 15:16:52.704803
51769	ITGR	2020-10-05	58.38	59.83	58.38	59.32	317531	{}	\N	2020-10-12 15:16:52.710479
51770	ITGR	2020-10-06	59.93	62.155	59.74	60.53	469619	{}	\N	2020-10-12 15:16:52.715358
51771	ITGR	2020-10-07	58.17	61.94	58.17	61.65	455115	{}	\N	2020-10-12 15:16:52.720003
51772	ITGR	2020-10-08	62.57	63.79	62.34	63.38	455232	{}	\N	2020-10-12 15:16:52.724572
51773	ITGR	2020-10-09	64.08	64.95	63.22	64.77	269793	{}	\N	2020-10-12 15:16:52.729906
51774	ITGR	2020-10-12	64.69	65.85	64.5463	65.81	51845	{}	\N	2020-10-12 15:16:52.733982
51775	IVAC	2020-09-28	5.51	5.85	5.51	5.79	45870	{}	\N	2020-10-12 15:16:53.649095
51776	IVAC	2020-09-29	5.77	5.7701	5.7	5.72	11714	{}	\N	2020-10-12 15:16:53.66658
51777	IVAC	2020-09-30	5.72	5.81	5.5	5.51	23344	{}	\N	2020-10-12 15:16:53.674195
51778	IVAC	2020-10-01	5.54	5.67	5.4795	5.61	28830	{}	\N	2020-10-12 15:16:53.679773
51779	IVAC	2020-10-02	5.5	5.67	4.96	5.44	47498	{}	\N	2020-10-12 15:16:53.685377
51780	IVAC	2020-10-05	5.42	5.64	5.42	5.56	28850	{}	\N	2020-10-12 15:16:53.690477
51781	IVAC	2020-10-06	5.47	5.78	5.26	5.28	67773	{}	\N	2020-10-12 15:16:53.694832
51782	IVAC	2020-10-07	5.33	5.57	5.3	5.42	70682	{}	\N	2020-10-12 15:16:53.698848
51783	IVAC	2020-10-08	5.5	5.5	5.12	5.36	96910	{}	\N	2020-10-12 15:16:53.702929
51784	IVAC	2020-10-09	5.35	5.5	5.28	5.5	22251	{}	\N	2020-10-12 15:16:53.707872
51785	IVAC	2020-10-12	5.57	5.57	5.52	5.55	5490	{}	\N	2020-10-12 15:16:53.712467
51786	IWD	2020-09-28	117.65	118.78	117.64	118.18	1529475	{}	\N	2020-10-12 15:16:54.728956
51787	IWD	2020-09-29	118.31	118.32	116.83	117.27	2094621	{}	\N	2020-10-12 15:16:54.741625
51788	IWD	2020-09-30	117.61	119.23	117.37	118.13	1978402	{}	\N	2020-10-12 15:16:54.748326
51789	IWD	2020-10-01	118.68	119.23	117.654	118.42	1393322	{}	\N	2020-10-12 15:16:54.754055
51790	IWD	2020-10-02	116.7	119.41	116.41	118.76	2370795	{}	\N	2020-10-12 15:16:54.759869
51791	IWD	2020-10-05	119.57	120.8343	119.57	120.68	1511037	{}	\N	2020-10-12 15:16:54.76482
51792	IWD	2020-10-06	121.11	122.13	119.37	119.58	2070857	{}	\N	2020-10-12 15:16:54.769014
51793	IWD	2020-10-07	120.62	121.83	120.58	121.48	1331702	{}	\N	2020-10-12 15:16:54.773428
51794	IWD	2020-10-08	122.11	123.15	121.94	123.07	1722077	{}	\N	2020-10-12 15:16:54.7776
51795	IWD	2020-10-09	123.81	123.9406	122.92	123.28	1383494	{}	\N	2020-10-12 15:16:54.782
51796	IWD	2020-10-12	123.6	124.22	123.43	124.22	418971	{}	\N	2020-10-12 15:16:54.786404
51797	IWP	2020-09-28	170.96	172.2588	170.55	171.8	562995	{}	\N	2020-10-12 15:16:55.732986
51798	IWP	2020-09-29	171.79	173.2399	171.6301	171.88	232963	{}	\N	2020-10-12 15:16:55.743345
51799	IWP	2020-09-30	172.38	174.3	171.77	172.87	307917	{}	\N	2020-10-12 15:16:55.749744
51800	IWP	2020-10-01	174.04	175.62	174.04	175.1	936174	{}	\N	2020-10-12 15:16:55.755378
51801	IWP	2020-10-02	171.79	175.21	171.79	173.64	409431	{}	\N	2020-10-12 15:16:55.76075
51802	IWP	2020-10-05	174.59	177.07	174.59	177.04	231573	{}	\N	2020-10-12 15:16:55.765163
51803	IWP	2020-10-06	177.31	179.4299	175.63	176.06	322785	{}	\N	2020-10-12 15:16:55.770095
51804	IWP	2020-10-07	177.72	180.01	177.72	179.65	215700	{}	\N	2020-10-12 15:16:55.774532
51805	IWP	2020-10-08	180.62	181.01	179.94	180.68	343386	{}	\N	2020-10-12 15:16:55.778469
51806	IWP	2020-10-09	182.02	183.66	182.02	183.08	261754	{}	\N	2020-10-12 15:16:55.782647
51807	IWP	2020-10-12	184.86	185.11	183.721	185.08	86976	{}	\N	2020-10-12 15:16:55.787172
51808	IXN	2020-09-28	261.71	262.17	259.84	262.16	43905	{}	\N	2020-10-12 15:16:58.22358
51809	IXN	2020-09-29	261.95	263.058	261.61	262.02	148926	{}	\N	2020-10-12 15:16:58.232554
51810	IXN	2020-09-30	261.68	266.0699	261.68	263.67	102924	{}	\N	2020-10-12 15:16:58.238482
51811	IXN	2020-10-01	267.05	267.38	265.375	266.68	86270	{}	\N	2020-10-12 15:16:58.243478
51812	IXN	2020-10-02	260.53	264.17	260.08	260.45	78400	{}	\N	2020-10-12 15:16:58.248177
51813	IXN	2020-10-05	262.33	266.09	262.33	266	64963	{}	\N	2020-10-12 15:16:58.252891
51814	IXN	2020-10-06	265.26	266.6631	261.3331	261.75	71705	{}	\N	2020-10-12 15:16:58.25705
51815	IXN	2020-10-07	264.7	267	264.6801	266.54	299757	{}	\N	2020-10-12 15:16:58.261108
51816	IXN	2020-10-08	268.84	268.84	267.61	268.07	87913	{}	\N	2020-10-12 15:16:58.265255
51817	IXN	2020-10-09	269.56	272.2	269.56	272.2	42874	{}	\N	2020-10-12 15:16:58.269601
51818	IXN	2020-10-12	275.83	277.08	274.78	277.07	21434	{}	\N	2020-10-12 15:16:58.273967
51819	KBH	2020-09-28	37.18	38.775	36.76	38.64	1982824	{}	\N	2020-10-12 15:16:59.201619
51820	KBH	2020-09-29	38.69	38.7499	37.58	37.69	1819692	{}	\N	2020-10-12 15:16:59.212624
51821	KBH	2020-09-30	37.7	39.29	37.64	38.39	2947964	{}	\N	2020-10-12 15:16:59.219069
51822	KBH	2020-10-01	38.93	39.76	38.43	39.71	3530402	{}	\N	2020-10-12 15:16:59.224388
51823	KBH	2020-10-02	38.48	40.09	38.32	39.81	2049024	{}	\N	2020-10-12 15:16:59.229615
51824	KBH	2020-10-05	40.4	40.62	38.445	39.56	2204640	{}	\N	2020-10-12 15:16:59.23405
51825	KBH	2020-10-06	40	40.28	38.53	38.7	2142241	{}	\N	2020-10-12 15:16:59.238448
51826	KBH	2020-10-07	39.36	39.8	37.86	38.42	3051496	{}	\N	2020-10-12 15:16:59.242379
51827	KBH	2020-10-08	39.19	40.31	39.03	40.06	2329941	{}	\N	2020-10-12 15:16:59.246046
51828	KBH	2020-10-09	40.67	41.48	40.44	40.82	2287962	{}	\N	2020-10-12 15:16:59.249963
51829	KBH	2020-10-12	41	41.41	40.51	41.31	382968	{}	\N	2020-10-12 15:16:59.253823
51830	IRL	2020-09-28	8.485	8.72	8.33	8.37	8889	{}	\N	2020-10-12 15:17:01.152437
51831	IRL	2020-09-29	8.2823	8.2823	8.1401	8.1401	7264	{}	\N	2020-10-12 15:17:01.163792
51832	IRL	2020-09-30	8.36	8.44	8.36	8.41	1828	{}	\N	2020-10-12 15:17:01.171128
51833	IRL	2020-10-01	8.4608	8.4608	8.4608	8.4608	1068	{}	\N	2020-10-12 15:17:01.177076
51834	IRL	2020-10-02	8.4	8.4	8.4	8.4	1543	{}	\N	2020-10-12 15:17:01.182175
51835	IRL	2020-10-05	8.53	8.58	8.47	8.47	4864	{}	\N	2020-10-12 15:17:01.18736
51836	IRL	2020-10-06	8.52	8.57	8.23	8.23	20827	{}	\N	2020-10-12 15:17:01.191736
51837	IRL	2020-10-07	8.4	8.43	8.26	8.26	36533	{}	\N	2020-10-12 15:17:01.195834
51838	IRL	2020-10-08	8.45	8.4535	8.45	8.45	8164	{}	\N	2020-10-12 15:17:01.200506
51839	IRL	2020-10-09	8.54	8.638	8.54	8.57	3230	{}	\N	2020-10-12 15:17:01.205324
51840	IRL	2020-10-12	8.52	8.52	8.52	8.52	2302	{}	\N	2020-10-12 15:17:01.210046
51841	ISSC	2020-09-28	6.89	7.13	6.89	7.1	15803	{}	\N	2020-10-12 15:17:02.629714
51842	ISSC	2020-09-29	7.19	7.19	6.95	7.04	13163	{}	\N	2020-10-12 15:17:02.640768
51843	ISSC	2020-09-30	7.01	7.09	6.84	6.84	2732	{}	\N	2020-10-12 15:17:02.648841
51844	ISSC	2020-10-01	6.87	7.5	6.84	7	42828	{}	\N	2020-10-12 15:17:02.65429
51845	ISSC	2020-10-02	6.98	7.21	6.98	7.18	12862	{}	\N	2020-10-12 15:17:02.658957
51846	ISSC	2020-10-05	7.238	7.238	6.88	6.88	9036	{}	\N	2020-10-12 15:17:02.66365
51847	ISSC	2020-10-06	7.1	7.13	6.72	6.72	9696	{}	\N	2020-10-12 15:17:02.667725
51848	ISSC	2020-10-07	6.7616	6.85	6.48	6.5	28582	{}	\N	2020-10-12 15:17:02.671786
51849	ISSC	2020-10-08	6.63	6.78	6.42	6.69	22599	{}	\N	2020-10-12 15:17:02.675969
51850	ISSC	2020-10-09	6.7098	6.88	6.68	6.68	12280	{}	\N	2020-10-12 15:17:02.680002
51851	ISSC	2020-10-12	6.68	6.68	6.62	6.63	2681	{}	\N	2020-10-12 15:17:02.684485
51852	IVC	2020-09-28	7.5	7.6523	7.36	7.45	420477	{}	\N	2020-10-12 15:17:04.380023
51853	IVC	2020-09-29	7.44	7.53	7.34	7.4	188987	{}	\N	2020-10-12 15:17:04.389849
51854	IVC	2020-09-30	7.35	7.6	7.35	7.52	609047	{}	\N	2020-10-12 15:17:04.395672
51855	IVC	2020-10-01	7.59	7.96	7.5	7.87	355175	{}	\N	2020-10-12 15:17:04.400839
51856	IVC	2020-10-02	7.75	7.99	7.52	7.86	378546	{}	\N	2020-10-12 15:17:04.405893
51857	IVC	2020-10-05	7.89	8.36	7.89	8.04	585866	{}	\N	2020-10-12 15:17:04.410227
51858	IVC	2020-10-06	8.12	8.35	7.8	7.87	525461	{}	\N	2020-10-12 15:17:04.414395
51859	IVC	2020-10-07	7.96	8.3	7.91	8.1	383425	{}	\N	2020-10-12 15:17:04.419116
51860	IVC	2020-10-08	8.23	8.28	7.95	8.23	340766	{}	\N	2020-10-12 15:17:04.423555
51861	IVC	2020-10-09	8.33	8.54	8.2	8.36	305128	{}	\N	2020-10-12 15:17:04.427865
51862	IVC	2020-10-12	8.35	8.35	8.13	8.23	63466	{}	\N	2020-10-12 15:17:04.432263
51863	IVOV	2020-09-28	103.31	105.35	103.31	104.97	159775	{}	\N	2020-10-12 15:17:06.202125
51864	IVOV	2020-09-29	104.83	104.9	103.15	104.03	43275	{}	\N	2020-10-12 15:17:06.212325
51865	IVOV	2020-09-30	104.14	105.955	104.0966	104.84	49122	{}	\N	2020-10-12 15:17:06.218679
51866	IVOV	2020-10-01	105.15	106.18	105.15	106.17	22990	{}	\N	2020-10-12 15:17:06.22419
51867	IVOV	2020-10-02	105.43	108.29	105.39	107.89	25239	{}	\N	2020-10-12 15:17:06.228584
51868	IVOV	2020-10-05	108.93	110.54	108.93	110.41	32174	{}	\N	2020-10-12 15:17:06.233232
51869	IVOV	2020-10-06	111.12	112.955	109.835	110.1	36818	{}	\N	2020-10-12 15:17:06.237339
51870	IVOV	2020-10-07	111.405	112.36	111.405	112.01	15609	{}	\N	2020-10-12 15:17:06.241245
51871	IVOV	2020-10-08	113	113.22	112.25	113.21	14747	{}	\N	2020-10-12 15:17:06.244853
51872	IVOV	2020-10-09	114	114.52	112.99	113.08	32839	{}	\N	2020-10-12 15:17:06.248623
51873	IVOV	2020-10-12	113.38	114.03	112.98	114.03	12778	{}	\N	2020-10-12 15:17:06.252667
51874	IWF	2020-09-28	215.52	215.74	214.04	215.69	2281950	{}	\N	2020-10-12 15:17:08.160125
51875	IWF	2020-09-29	215.69	216.4007	214.85	215.13	727531	{}	\N	2020-10-12 15:17:08.171197
51876	IWF	2020-09-30	215.32	219.04	215.32	216.89	1920495	{}	\N	2020-10-12 15:17:08.179439
51877	IWF	2020-10-01	219.45	219.92	218.18	219.64	1516142	{}	\N	2020-10-12 15:17:08.185123
51878	IWF	2020-10-02	215.06	218.055	214.25	215.22	1409310	{}	\N	2020-10-12 15:17:08.190619
51879	IWF	2020-10-05	216.52	219.55	216.52	219.44	828784	{}	\N	2020-10-12 15:17:08.195185
51880	IWF	2020-10-06	219.01	220.3765	215.3	215.96	2692184	{}	\N	2020-10-12 15:17:08.199646
51881	IWF	2020-10-07	217.83	220.37	217.83	219.96	882398	{}	\N	2020-10-12 15:17:08.2043
51882	IWF	2020-10-08	221.48	221.5064	220.22	220.83	934119	{}	\N	2020-10-12 15:17:08.209131
51883	IWF	2020-10-09	221.95	224.2495	221.8	224.19	1622060	{}	\N	2020-10-12 15:17:08.213271
51884	IWF	2020-10-12	227.23	228.57	226.299	228.57	295432	{}	\N	2020-10-12 15:17:08.220751
51885	IWO	2020-09-28	219.34	221.3	218.56	220.83	188459	{}	\N	2020-10-12 15:17:09.18015
51886	IWO	2020-09-29	220.72	222.5093	219.68	221.07	238711	{}	\N	2020-10-12 15:17:09.189866
51887	IWO	2020-09-30	221.36	224.4	219.76	221.52	470831	{}	\N	2020-10-12 15:17:09.196484
51888	IWO	2020-10-01	223	225.06	221.515	224.94	427979	{}	\N	2020-10-12 15:17:09.201058
51889	IWO	2020-10-02	220.87	224.99	220.86	223.89	223830	{}	\N	2020-10-12 15:17:09.205968
51890	IWO	2020-10-05	226.14	230.93	226.14	230.7	314439	{}	\N	2020-10-12 15:17:09.210386
51891	IWO	2020-10-06	232.22	235.9	229.72	230.36	411468	{}	\N	2020-10-12 15:17:09.214619
51892	IWO	2020-10-07	232.85	236.16	232.83	235.23	265272	{}	\N	2020-10-12 15:17:09.219089
51893	IWO	2020-10-08	237.8	238.09	236.05	237.06	291300	{}	\N	2020-10-12 15:17:09.223373
51894	IWO	2020-10-09	238.86	239.95	238.05	239.54	244959	{}	\N	2020-10-12 15:17:09.22752
51895	IWO	2020-10-12	240.47	241.64	239.4465	241.49	87032	{}	\N	2020-10-12 15:17:09.232003
51896	IWY	2020-09-28	120.79	120.79	119.79	120.72	169642	{}	\N	2020-10-12 15:17:11.206872
51897	IWY	2020-09-29	120.69	120.93	120.18	120.32	75423	{}	\N	2020-10-12 15:17:11.217247
51898	IWY	2020-09-30	120.39	122.589	120.39	121.37	88073	{}	\N	2020-10-12 15:17:11.225667
51899	IWY	2020-10-01	122.56	123	122.0687	122.75	507243	{}	\N	2020-10-12 15:17:11.231369
51900	IWY	2020-10-02	120.15	121.8	119.83	120.03	339517	{}	\N	2020-10-12 15:17:11.236296
51901	IWY	2020-10-05	120.96	122.57	120.89	122.48	842267	{}	\N	2020-10-12 15:17:11.24153
51902	IWY	2020-10-06	122.31	122.7509	119.91	120.24	148957	{}	\N	2020-10-12 15:17:11.24593
51903	IWY	2020-10-07	121.33	122.67	121.33	122.4	743064	{}	\N	2020-10-12 15:17:11.25068
51904	IWY	2020-10-08	123.39	123.39	122.6627	122.96	127213	{}	\N	2020-10-12 15:17:11.254911
51905	IWY	2020-10-09	123.55	124.88	123.52	124.88	119428	{}	\N	2020-10-12 15:17:11.258674
51906	IWY	2020-10-12	126.43	127.38	126.14	127.363	31739	{}	\N	2020-10-12 15:17:11.262687
51907	IRS	2020-09-28	3.11	3.11	2.83	2.86	445180	{}	\N	2020-10-12 15:17:13.523937
51908	IRS	2020-09-29	2.97	2.97	2.75	2.81	107868	{}	\N	2020-10-12 15:17:13.530958
51909	IRS	2020-09-30	2.83	2.9199	2.8	2.82	50396	{}	\N	2020-10-12 15:17:13.536693
51910	IRS	2020-10-01	2.8	2.99	2.8	2.99	85078	{}	\N	2020-10-12 15:17:13.541914
51911	IRS	2020-10-02	2.89	2.96	2.84	2.9	83465	{}	\N	2020-10-12 15:17:13.546976
51912	IRS	2020-10-05	2.98	3	2.9403	2.97	45504	{}	\N	2020-10-12 15:17:13.55173
51913	IRS	2020-10-06	2.96	3.05	2.95	2.95	40935	{}	\N	2020-10-12 15:17:13.555998
51914	IRS	2020-10-07	3	3.08	2.94	3.05	35830	{}	\N	2020-10-12 15:17:13.560798
51915	IRS	2020-10-08	3.04	3.09	3	3.04	13996	{}	\N	2020-10-12 15:17:13.565521
51916	IRS	2020-10-09	3.09	3.12	3.01	3.12	84815	{}	\N	2020-10-12 15:17:13.569696
51917	IRS	2020-10-12	3.11	3.11	2.96	2.99	3876	{}	\N	2020-10-12 15:17:13.575878
51918	ISIG	2020-09-28	0.6188	0.634	0.6071	0.6298	16087	{}	\N	2020-10-12 15:17:14.719974
51919	ISIG	2020-09-29	0.63	0.6497	0.63	0.6497	27081	{}	\N	2020-10-12 15:17:14.731849
51920	ISIG	2020-09-30	0.6545	0.7052	0.6511	0.666	45745	{}	\N	2020-10-12 15:17:14.738121
51921	ISIG	2020-10-01	0.654201	0.678	0.6541	0.674868	15215	{}	\N	2020-10-12 15:17:14.743839
51922	ISIG	2020-10-02	0.6749	0.686	0.6601	0.6617	13598	{}	\N	2020-10-12 15:17:14.748548
51923	ISIG	2020-10-05	0.6617	0.6798	0.6601	0.6601	5677	{}	\N	2020-10-12 15:17:14.75314
51924	ISIG	2020-10-06	0.6601	0.6779	0.62	0.6203	20105	{}	\N	2020-10-12 15:17:14.757485
51925	ISIG	2020-10-07	0.65	0.66	0.6	0.61	45166	{}	\N	2020-10-12 15:17:14.761667
51926	ISIG	2020-10-08	0.6172	0.669	0.61001	0.669	29933	{}	\N	2020-10-12 15:17:14.765702
51927	ISIG	2020-10-09	0.6686	0.68	0.63201	0.68	46845	{}	\N	2020-10-12 15:17:14.769625
51928	ISIG	2020-10-12	0.6903	0.69165	0.6401	0.69165	20132	{}	\N	2020-10-12 15:17:14.773885
51929	ISZE	2020-09-28	25.8527	25.8527	25.8527	25.8527	0	{}	\N	2020-10-12 15:17:16.533764
51930	ISZE	2020-09-29	25.8151	25.8151	25.8151	25.8151	0	{}	\N	2020-10-12 15:17:16.540335
51931	ISZE	2020-09-30	25.7629	25.7629	25.7629	25.7629	2	{}	\N	2020-10-12 15:17:16.544894
51932	ISZE	2020-10-01	25.8696	25.8696	25.8696	25.8696	2	{}	\N	2020-10-12 15:17:16.549684
51933	ISZE	2020-10-02	25.814	25.814	25.814	25.814	5	{}	\N	2020-10-12 15:17:16.554036
51934	ISZE	2020-10-05	26.2494	26.2494	26.2494	26.2494	0	{}	\N	2020-10-12 15:17:16.55837
51935	ISZE	2020-10-07	26.39	26.39	26.3289	26.3289	373	{}	\N	2020-10-12 15:17:16.562534
51936	ISZE	2020-10-08	26.4749	26.4749	26.4749	26.4749	5	{}	\N	2020-10-12 15:17:16.56689
51937	ISZE	2020-10-09	26.6017	26.6017	26.6017	26.6017	68	{}	\N	2020-10-12 15:17:16.570886
51938	ITEQ	2020-09-28	53.74	54.35	53.74	54.34	17145	{}	\N	2020-10-12 15:17:18.65771
51939	ITEQ	2020-09-29	54.44	55.06	54.44	55	20420	{}	\N	2020-10-12 15:17:18.66868
51940	ITEQ	2020-09-30	55.16	55.975	55.14	55.55	13958	{}	\N	2020-10-12 15:17:18.675349
51941	ITEQ	2020-10-01	55.62	56.58	55.62	56.58	49270	{}	\N	2020-10-12 15:17:18.680798
51942	ITEQ	2020-10-02	55.49	56.66	55.49	56.1413	14785	{}	\N	2020-10-12 15:17:18.685286
51943	ITEQ	2020-10-05	56.89	57.91	56.89	57.91	18539	{}	\N	2020-10-12 15:17:18.690098
51944	ITEQ	2020-10-06	57.79	58.17	57.33	57.63	12472	{}	\N	2020-10-12 15:17:18.694237
51945	ITEQ	2020-10-07	58.5	59.99	58.34	59.99	34232	{}	\N	2020-10-12 15:17:18.698734
51946	ITEQ	2020-10-08	60.46	60.46	59.61	59.9787	15269	{}	\N	2020-10-12 15:17:18.702761
51947	ITEQ	2020-10-09	60.46	61.83	60.46	61.77	19960	{}	\N	2020-10-12 15:17:18.706851
51948	ITEQ	2020-10-12	62.08	62.08	61.31	61.8	8859	{}	\N	2020-10-12 15:17:18.710712
51949	ITW	2020-09-28	194.82	198.7466	194.42	195.68	925821	{}	\N	2020-10-12 15:17:19.802913
51950	ITW	2020-09-29	194.86	196.42	192.72	193.47	1495140	{}	\N	2020-10-12 15:17:19.814095
51951	ITW	2020-09-30	194.47	197.14	191.52	193.21	3103531	{}	\N	2020-10-12 15:17:19.821381
51952	ITW	2020-10-01	194.42	194.95	188.685	190.9	2188603	{}	\N	2020-10-12 15:17:19.827755
51953	ITW	2020-10-02	188.73	194.17	188.14	191.85	1581633	{}	\N	2020-10-12 15:17:19.833206
51954	ITW	2020-10-05	194.16	198.27	193.57	197.78	1610578	{}	\N	2020-10-12 15:17:19.838526
51955	ITW	2020-10-06	198.13	201.24	195.76	196.41	1809792	{}	\N	2020-10-12 15:17:19.843137
51956	ITW	2020-10-07	198.36	200.18	197.46	199.64	1515875	{}	\N	2020-10-12 15:17:19.847213
51957	ITW	2020-10-08	200.25	201.27	197.72	200.98	1290198	{}	\N	2020-10-12 15:17:19.851703
51958	ITW	2020-10-09	203.06	206.72	201.51	205.71	2356646	{}	\N	2020-10-12 15:17:19.856048
51959	ITW	2020-10-12	206.4	206.91	204.07	205.29	599352	{}	\N	2020-10-12 15:17:19.860515
51960	IVOG	2020-09-28	146.46	148.67	146.14	148.31	159470	{}	\N	2020-10-12 15:17:21.469888
51961	IVOG	2020-09-29	147.97	149.25	147.415	147.9	50198	{}	\N	2020-10-12 15:17:21.476828
51962	IVOG	2020-09-30	148.24	150.205	147.9901	149.1028	28616	{}	\N	2020-10-12 15:17:21.482589
51963	IVOG	2020-10-01	149.45	150.69	149.27	150.69	23501	{}	\N	2020-10-12 15:17:21.487538
51964	IVOG	2020-10-02	147.73	152.025	147.73	151.55	18926	{}	\N	2020-10-12 15:17:21.492501
51965	IVOG	2020-10-05	152.82	154.71	152.25	154.68	35717	{}	\N	2020-10-12 15:17:21.496605
51966	IVOG	2020-10-06	155.32	157	153.705	153.96	16493	{}	\N	2020-10-12 15:17:21.500758
51967	IVOG	2020-10-07	155.55	156.8527	155.55	156.77	19810	{}	\N	2020-10-12 15:17:21.504861
51968	IVOG	2020-10-08	158	158.22	157.4	158.1	9206	{}	\N	2020-10-12 15:17:21.509116
51969	IVOG	2020-10-09	158.99	159.63	158.585	158.9582	20510	{}	\N	2020-10-12 15:17:21.513022
51970	IVOG	2020-10-12	159.48	159.99	159.27	159.99	5923	{}	\N	2020-10-12 15:17:21.517143
51971	IVZ	2020-09-28	11.03	11.575	11.02	11.49	11208833	{}	\N	2020-10-12 15:17:22.46162
51972	IVZ	2020-09-29	11.39	11.52	11.1134	11.43	18607619	{}	\N	2020-10-12 15:17:22.471712
51973	IVZ	2020-09-30	11.49	11.645	11.25	11.41	13670132	{}	\N	2020-10-12 15:17:22.479335
51974	IVZ	2020-10-01	11.46	11.55	11.165	11.25	10739768	{}	\N	2020-10-12 15:17:22.485067
51975	IVZ	2020-10-02	11.88	12.21	11.66	11.85	28369930	{}	\N	2020-10-12 15:17:22.490173
51976	IVZ	2020-10-05	11.96	12.03	11.69	11.87	16410631	{}	\N	2020-10-12 15:17:22.494709
51977	IVZ	2020-10-06	12.08	12.3	11.8	11.85	12644996	{}	\N	2020-10-12 15:17:22.499646
51978	IVZ	2020-10-07	11.98	12.69	11.97	12.62	15556182	{}	\N	2020-10-12 15:17:22.503471
51979	IVZ	2020-10-08	13.05	13.325	12.805	12.95	22339574	{}	\N	2020-10-12 15:17:22.507773
51980	IVZ	2020-10-09	13.03	13.3	12.92	13.11	11573008	{}	\N	2020-10-12 15:17:22.511664
51981	IVZ	2020-10-12	13.24	13.43	13.14	13.32	3411386	{}	\N	2020-10-12 15:17:22.516044
51982	IWR	2020-09-28	57.07	57.59	57.02	57.43	1177538	{}	\N	2020-10-12 15:17:23.679791
51983	IWR	2020-09-29	57.47	57.5775	56.91	57.27	1949579	{}	\N	2020-10-12 15:17:23.691676
51984	IWR	2020-09-30	57.27	57.93	56.99	57.38	1702732	{}	\N	2020-10-12 15:17:23.698781
51985	IWR	2020-10-01	57.8	58.08	57.5	57.95	2158545	{}	\N	2020-10-12 15:17:23.70411
51986	IWR	2020-10-02	56.94	58.38	56.87	58.14	1662071	{}	\N	2020-10-12 15:17:23.709492
51987	IWR	2020-10-05	58.55	59.3	58.55	59.28	693996	{}	\N	2020-10-12 15:17:23.714592
51988	IWR	2020-10-06	59.51	60.13	58.73	58.83	1281010	{}	\N	2020-10-12 15:17:23.718533
51989	IWR	2020-10-07	59.39	60.05	59.39	59.92	712407	{}	\N	2020-10-12 15:17:23.722405
51990	IWR	2020-10-08	60.33	60.64	60.18	60.61	1037543	{}	\N	2020-10-12 15:17:23.726486
51991	IWR	2020-10-09	61.06	61.22	60.74	60.87	806005	{}	\N	2020-10-12 15:17:23.731184
51992	IWR	2020-10-12	61.18	61.39	61.01	61.39	277678	{}	\N	2020-10-12 15:17:23.735519
51993	IYLD	2020-09-28	22.615	22.66	22.58	22.61	43210	{}	\N	2020-10-12 15:17:26.382307
51994	IYLD	2020-09-29	22.65	22.65	22.5	22.54	30472	{}	\N	2020-10-12 15:17:26.389325
51995	IYLD	2020-09-30	22.56	22.62	22.54	22.55	75521	{}	\N	2020-10-12 15:17:26.394754
51996	IYLD	2020-10-01	22.52	22.6399	22.52	22.63	68985	{}	\N	2020-10-12 15:17:26.399766
51997	IYLD	2020-10-02	22.46	22.5299	22.4	22.5	65362	{}	\N	2020-10-12 15:17:26.40434
51998	IYLD	2020-10-05	22.57	22.57	22.48	22.505	45025	{}	\N	2020-10-12 15:17:26.408946
51999	IYLD	2020-10-06	22.57	22.619	22.48	22.52	58867	{}	\N	2020-10-12 15:17:26.412871
52000	IYLD	2020-10-07	22.63	22.63	22.495	22.54	54155	{}	\N	2020-10-12 15:17:26.416878
52001	IYLD	2020-10-08	22.61	22.69	22.57	22.68	32017	{}	\N	2020-10-12 15:17:26.421651
52002	IYLD	2020-10-09	22.63	22.7294	22.63	22.665	35705	{}	\N	2020-10-12 15:17:26.425619
52003	IYLD	2020-10-12	22.63	22.7199	22.63	22.715	4593	{}	\N	2020-10-12 15:17:26.429591
52004	IRMD	2020-09-28	21.09	21.58	21	21.02	25004	{}	\N	2020-10-12 15:17:27.917393
52005	IRMD	2020-09-29	21.15	21.29	20.9	21.16	52603	{}	\N	2020-10-12 15:17:27.929988
52006	IRMD	2020-09-30	21.21	21.72	21.015	21.4	62504	{}	\N	2020-10-12 15:17:27.937611
52007	IRMD	2020-10-01	21.55	21.75	21.34	21.44	18204	{}	\N	2020-10-12 15:17:27.943772
52008	IRMD	2020-10-02	21.29	21.3	20.56	20.62	29581	{}	\N	2020-10-12 15:17:27.950291
52009	IRMD	2020-10-05	21.02	21.26	20.72	20.96	26045	{}	\N	2020-10-12 15:17:27.955404
52010	IRMD	2020-10-06	21.11	21.3	20.61	20.99	31442	{}	\N	2020-10-12 15:17:27.95985
52011	IRMD	2020-10-07	21	21.13	20.68	21.13	32954	{}	\N	2020-10-12 15:17:27.964951
52012	IRMD	2020-10-08	21.3	21.52	20.85	21.11	15229	{}	\N	2020-10-12 15:17:27.968988
52013	IRMD	2020-10-09	21.11	22.17	21.06	21.93	51430	{}	\N	2020-10-12 15:17:27.973521
52014	IRMD	2020-10-12	22.13	22.13	21.63	21.78	2358	{}	\N	2020-10-12 15:17:27.977817
52015	ISD	2020-09-28	13.8	13.81	13.72	13.77	167310	{}	\N	2020-10-12 15:17:29.377584
52016	ISD	2020-09-29	13.73	13.8	13.72	13.78	252252	{}	\N	2020-10-12 15:17:29.391841
52017	ISD	2020-09-30	13.8	13.89	13.79	13.81	161942	{}	\N	2020-10-12 15:17:29.399014
52018	ISD	2020-10-01	13.82	13.99	13.82	13.96	392608	{}	\N	2020-10-12 15:17:29.4047
52019	ISD	2020-10-02	13.86	14	13.8328	14	155619	{}	\N	2020-10-12 15:17:29.409664
52020	ISD	2020-10-05	14.03	14.09	14	14.09	161441	{}	\N	2020-10-12 15:17:29.41399
52021	ISD	2020-10-06	14.09	14.1487	14.05	14.08	180782	{}	\N	2020-10-12 15:17:29.418117
52022	ISD	2020-10-07	14.12	14.19	14.12	14.17	251414	{}	\N	2020-10-12 15:17:29.422519
52023	ISD	2020-10-08	14.2	14.2	14.1	14.13	288455	{}	\N	2020-10-12 15:17:29.426552
52024	ISD	2020-10-09	14.11	14.16	14.09	14.09	158080	{}	\N	2020-10-12 15:17:29.430659
52025	ISD	2020-10-12	14.09	14.2	14.09	14.1915	77262	{}	\N	2020-10-12 15:17:29.434601
52026	ISRA	2020-09-28	34.9226	35.35	34.85	34.94	3553	{}	\N	2020-10-12 15:17:30.418289
52027	ISRA	2020-09-29	35.18	35.3837	35.0108	35.15	1494	{}	\N	2020-10-12 15:17:30.429051
52028	ISRA	2020-09-30	35.62	36.23	35.62	36.09	9241	{}	\N	2020-10-12 15:17:30.435635
52029	ISRA	2020-10-01	36.1501	36.415	36.1501	36.415	1179	{}	\N	2020-10-12 15:17:30.441165
52030	ISRA	2020-10-02	36.035	36.035	36.035	36.035	45	{}	\N	2020-10-12 15:17:30.445728
52031	ISRA	2020-10-05	36.8	37.36	36.725	37.36	3739	{}	\N	2020-10-12 15:17:30.450548
52032	ISRA	2020-10-06	37.48	37.6699	37.0553	37.45	10306	{}	\N	2020-10-12 15:17:30.454938
52033	ISRA	2020-10-07	37.6657	38.45	37.6657	38.45	3127	{}	\N	2020-10-12 15:17:30.459016
52034	ISRA	2020-10-08	38.4647	38.55	38.3743	38.55	560	{}	\N	2020-10-12 15:17:30.462782
52035	ISRA	2020-10-09	39.2	39.24	38.93	39.24	5589	{}	\N	2020-10-12 15:17:30.466659
52036	ISRA	2020-10-12	39.38	39.38	39.38	39.38	619	{}	\N	2020-10-12 15:17:30.470437
52037	ITB	2020-09-28	55.11	56.35	54.79	56.23	2640795	{}	\N	2020-10-12 15:17:32.016582
52038	ITB	2020-09-29	56.25	56.41	55.43	55.61	2561110	{}	\N	2020-10-12 15:17:32.029653
52039	ITB	2020-09-30	55.71	57.36	55.69	56.65	3330203	{}	\N	2020-10-12 15:17:32.036273
52040	ITB	2020-10-01	57.24	57.8	56.56	57.76	3200241	{}	\N	2020-10-12 15:17:32.041708
52041	ITB	2020-10-02	56.5301	58.39	56.31	58.08	3852536	{}	\N	2020-10-12 15:17:32.046438
52042	ITB	2020-10-05	58.59	58.97	57.14	58.01	3463974	{}	\N	2020-10-12 15:17:32.050308
52043	ITB	2020-10-06	58.18	58.485	56.7383	56.98	3411342	{}	\N	2020-10-12 15:17:32.053998
52044	ITB	2020-10-07	57.69	58.21	57.09	57.61	2407624	{}	\N	2020-10-12 15:17:32.057723
52045	ITB	2020-10-08	58.15	59.18	58.15	59.1	2213400	{}	\N	2020-10-12 15:17:32.061466
52046	ITB	2020-10-09	59.54	59.97	59.16	59.57	2111704	{}	\N	2020-10-12 15:17:32.065406
52047	ITB	2020-10-12	59.965	59.99	59.2963	59.92	631761	{}	\N	2020-10-12 15:17:32.069121
52048	ITRI	2020-09-28	59.28	59.43	57.86	59.2	497365	{}	\N	2020-10-12 15:17:33.591693
52049	ITRI	2020-09-29	59.1	59.775	58.44	59.58	456680	{}	\N	2020-10-12 15:17:33.601144
52050	ITRI	2020-09-30	59.56	61.73	59.405	60.74	396358	{}	\N	2020-10-12 15:17:33.607156
52051	ITRI	2020-10-01	61.745	63.475	61.56	63.35	305867	{}	\N	2020-10-12 15:17:33.612639
52052	ITRI	2020-10-02	61.74	63.79	61.35	63.15	203049	{}	\N	2020-10-12 15:17:33.618492
52053	ITRI	2020-10-05	63.95	64.895	63.83	64.8	246959	{}	\N	2020-10-12 15:17:33.624029
52054	ITRI	2020-10-06	64.46	66.39	64.45	64.68	470796	{}	\N	2020-10-12 15:17:33.62869
52055	ITRI	2020-10-07	65.7	65.7	64.46	65.2	340283	{}	\N	2020-10-12 15:17:33.633802
52056	ITRI	2020-10-08	66.03	66.75	65.3	66.2	393890	{}	\N	2020-10-12 15:17:33.638349
52057	ITRI	2020-10-09	66.87	67.79	66.095	67.52	164105	{}	\N	2020-10-12 15:17:33.642352
52058	ITRI	2020-10-12	67.96	68.485	67.24	68.11	66386	{}	\N	2020-10-12 15:17:33.646272
52059	ITUB	2020-09-28	4.25	4.28	3.96	4.01	24487949	{}	\N	2020-10-12 15:17:34.626542
52060	ITUB	2020-09-29	3.97	4.05	3.89	3.89	44005991	{}	\N	2020-10-12 15:17:34.636135
52061	ITUB	2020-09-30	3.93	4.01	3.92	3.98	34532570	{}	\N	2020-10-12 15:17:34.643273
52062	ITUB	2020-10-01	3.95	3.96	3.885	3.95	23987646	{}	\N	2020-10-12 15:17:34.649261
52063	ITUB	2020-10-02	3.94	4.12	3.93	3.98	56557481	{}	\N	2020-10-12 15:17:34.654075
52064	ITUB	2020-10-05	3.95	4.11	3.93	4.09	31514894	{}	\N	2020-10-12 15:17:34.658857
52065	ITUB	2020-10-06	4.13	4.23	4.01	4.07	59601265	{}	\N	2020-10-12 15:17:34.662828
52066	ITUB	2020-10-07	4.08	4.1	3.98	3.99	46028388	{}	\N	2020-10-12 15:17:34.666504
52067	ITUB	2020-10-08	4.03	4.31	4	4.27	72507548	{}	\N	2020-10-12 15:17:34.669999
52068	ITUB	2020-10-09	4.25	4.43	4.215	4.31	79448843	{}	\N	2020-10-12 15:17:34.673933
52069	ITUB	2020-10-12	4.3	4.33	4.28	4.31	3729833	{}	\N	2020-10-12 15:17:34.677431
52070	IVH	2020-09-28	11.91	12.03	11.91	12.01	52035	{}	\N	2020-10-12 15:17:35.751298
52071	IVH	2020-09-29	11.97	12.02	11.95	12.02	102424	{}	\N	2020-10-12 15:17:35.762054
52072	IVH	2020-09-30	12.03	12.03	11.9	11.9	121484	{}	\N	2020-10-12 15:17:35.769894
52073	IVH	2020-10-01	11.9	11.9797	11.9	11.93	62714	{}	\N	2020-10-12 15:17:35.776851
52074	IVH	2020-10-02	11.88	11.94	11.83	11.94	64694	{}	\N	2020-10-12 15:17:35.7828
52075	IVH	2020-10-05	11.94	12.05	11.93	12.05	50760	{}	\N	2020-10-12 15:17:35.787654
52076	IVH	2020-10-06	12	12.11	11.97	12.03	62539	{}	\N	2020-10-12 15:17:35.792799
52077	IVH	2020-10-07	12.1	12.11	12.02	12.08	41554	{}	\N	2020-10-12 15:17:35.797275
52078	IVH	2020-10-08	12.11	12.11	12.04	12.11	118116	{}	\N	2020-10-12 15:17:35.801659
52079	IVH	2020-10-09	12.12	12.17	12.12	12.16	80385	{}	\N	2020-10-12 15:17:35.806095
52080	IVH	2020-10-12	12.16	12.2294	12.16	12.17	30160	{}	\N	2020-10-12 15:17:35.810373
52081	IWB	2020-09-28	186.09	187.02	185.6	186.75	555050	{}	\N	2020-10-12 15:17:37.710682
52082	IWB	2020-09-29	186.58	186.9694	185.39	185.77	499095	{}	\N	2020-10-12 15:17:37.72065
52083	IWB	2020-09-30	186	188.9299	186	187.15	544868	{}	\N	2020-10-12 15:17:37.727132
52084	IWB	2020-10-01	188.48	189.19	187.4468	188.51	510440	{}	\N	2020-10-12 15:17:37.732825
52085	IWB	2020-10-02	185.25	188.03	184.98	186.94	421572	{}	\N	2020-10-12 15:17:37.738126
52086	IWB	2020-10-05	188.21	190.35	188.21	190.27	419538	{}	\N	2020-10-12 15:17:37.743315
52087	IWB	2020-10-06	190.42	191.6975	187.43	187.7	402175	{}	\N	2020-10-12 15:17:37.748467
52088	IWB	2020-10-07	189.45	191.4531	189.45	190.93	222240	{}	\N	2020-10-12 15:17:37.752612
52089	IWB	2020-10-08	192.28	192.71	191.74	192.59	467871	{}	\N	2020-10-12 15:17:37.757015
52090	IWB	2020-10-09	193.76	194.69	193.37	194.32	431968	{}	\N	2020-10-12 15:17:37.761238
52091	IWB	2020-10-12	195.84	197.04	195.63	197.04	32995	{}	\N	2020-10-12 15:17:37.765724
52092	IWV	2020-09-28	194.67	195.8	194.38	195.42	119554	{}	\N	2020-10-12 15:17:38.709328
52093	IWV	2020-09-29	195.47	195.73	194.04	194.44	88734	{}	\N	2020-10-12 15:17:38.729218
52094	IWV	2020-09-30	194.71	197.81	194.71	195.71	500562	{}	\N	2020-10-12 15:17:38.735802
52095	IWV	2020-10-01	197.36	198.1	196.31	197.5	214050	{}	\N	2020-10-12 15:17:38.741002
52096	IWV	2020-10-02	194.09	197.05	193.77	195.96	392207	{}	\N	2020-10-12 15:17:38.745653
52097	IWV	2020-10-05	197.17	199.58	197.17	199.46	238651	{}	\N	2020-10-12 15:17:38.750022
52098	IWV	2020-10-06	199.79	201.2799	196.78	196.97	221190	{}	\N	2020-10-12 15:17:38.754482
52099	IWV	2020-10-07	198.81	200.98	198.81	200.67	103343	{}	\N	2020-10-12 15:17:38.758705
52100	IWV	2020-10-08	201.67	202.3	201.22	202.3	81978	{}	\N	2020-10-12 15:17:38.762773
52101	IWV	2020-10-09	203.31	204.34	203.01	204.08	73163	{}	\N	2020-10-12 15:17:38.767278
52102	IWV	2020-10-12	205.44	206.5594	205.3136	206.5	42110	{}	\N	2020-10-12 15:17:38.771351
52103	IYC	2020-09-28	248.01	249.8627	247.8	249.52	25901	{}	\N	2020-10-12 15:17:41.029187
52104	IYC	2020-09-29	249.71	249.71	247.32	247.585	8909	{}	\N	2020-10-12 15:17:41.039988
52105	IYC	2020-09-30	250.25	251.92	248.85	249.064	18346	{}	\N	2020-10-12 15:17:41.047982
52106	IYC	2020-10-01	252.47	253.21	251.44	253.05	61386	{}	\N	2020-10-12 15:17:41.053529
52107	IYC	2020-10-02	248.27	252.51	248.27	251.6	21367	{}	\N	2020-10-12 15:17:41.058846
52108	IYC	2020-10-05	253.27	254.8	252.95	254.64	12508	{}	\N	2020-10-12 15:17:41.063253
52109	IYC	2020-10-06	255.35	255.87	250.0518	250.24	16513	{}	\N	2020-10-12 15:17:41.067637
52110	IYC	2020-10-07	253.43	255.89	253.43	255.29	11031	{}	\N	2020-10-12 15:17:41.071967
52111	IYC	2020-10-08	257.21	257.21	255.4935	256.96	26438	{}	\N	2020-10-12 15:17:41.076486
52112	IYC	2020-10-09	259.02	259.32	257.941	258.3	16019	{}	\N	2020-10-12 15:17:41.08055
52113	IYC	2020-10-12	261.06	261.06	259.75	260.9	7384	{}	\N	2020-10-12 15:17:41.0845
52114	IYR	2020-09-28	79.75	80.51	79.51	80.18	3960749	{}	\N	2020-10-12 15:17:44.011655
52115	IYR	2020-09-29	80.12	80.21	79	79.58	4515338	{}	\N	2020-10-12 15:17:44.019679
52116	IYR	2020-09-30	80	80.69	79.04	79.84	4186811	{}	\N	2020-10-12 15:17:44.026066
52117	IYR	2020-10-01	80.12	81.45	79.81	81.38	18650088	{}	\N	2020-10-12 15:17:44.031358
52118	IYR	2020-10-02	80.26	82.94	79.9	82.68	21322453	{}	\N	2020-10-12 15:17:44.036288
52119	IYR	2020-10-05	82.9	83.42	81.64	83.14	12626401	{}	\N	2020-10-12 15:17:44.041349
52120	IYR	2020-10-06	83.33	83.77	82.29	82.62	20420388	{}	\N	2020-10-12 15:17:44.045627
52121	IYR	2020-10-07	83.12	83.155	82.33	82.71	3412510	{}	\N	2020-10-12 15:17:44.050114
52122	IYR	2020-10-08	83.17	84.32	82.97	84.01	4157321	{}	\N	2020-10-12 15:17:44.054747
52123	IYR	2020-10-09	84.33	84.57	83.4	83.65	10842274	{}	\N	2020-10-12 15:17:44.060506
52124	IYR	2020-10-12	83.76	83.94	83.3	83.87	1277729	{}	\N	2020-10-12 15:17:44.065079
52125	ISMD	2020-09-28	22.9599	23.03	22.89	22.92	13280	{}	\N	2020-10-12 15:17:46.095477
52126	ISMD	2020-09-29	22.91	22.91	21.66	22.7	61777	{}	\N	2020-10-12 15:17:46.105357
52127	ISMD	2020-09-30	22.9503	22.9503	22.665	22.7	3842	{}	\N	2020-10-12 15:17:46.112292
52128	ISMD	2020-10-01	22.88	23.04	22.625	22.84	16249	{}	\N	2020-10-12 15:17:46.118644
52129	ISMD	2020-10-02	22.32	23.4199	22.32	23.4199	6442	{}	\N	2020-10-12 15:17:46.124027
52130	ISMD	2020-10-05	23.63	23.79	23.21	23.7541	15614	{}	\N	2020-10-12 15:17:46.129088
52131	ISMD	2020-10-06	24.09	24.24	23.7297	23.7297	28288	{}	\N	2020-10-12 15:17:46.133423
52132	ISMD	2020-10-07	24.01	24.13	22.19	24.13	41187	{}	\N	2020-10-12 15:17:46.137467
52133	ISMD	2020-10-08	24.33	25.68	24.33	24.46	444737	{}	\N	2020-10-12 15:17:46.142152
52134	ISMD	2020-10-09	24.64	24.64	24.5272	24.5272	16675	{}	\N	2020-10-12 15:17:46.146433
52135	ISMD	2020-10-12	24.662	24.662	24.63	24.63	381	{}	\N	2020-10-12 15:17:46.150685
52136	IT	2020-09-28	127.52	127.96	126.33	126.53	330793	{}	\N	2020-10-12 15:17:47.161333
52137	IT	2020-09-29	126.39	127.915	125.11	125.23	564440	{}	\N	2020-10-12 15:17:47.170719
52138	IT	2020-09-30	125.92	127.22	124.59	124.95	705442	{}	\N	2020-10-12 15:17:47.177694
52139	IT	2020-10-01	125.84	127.28	124.63	125.72	549089	{}	\N	2020-10-12 15:17:47.183353
52140	IT	2020-10-02	123.82	126.44	123.26	125.34	547445	{}	\N	2020-10-12 15:17:47.188317
52141	IT	2020-10-05	126.74	126.95	125.33	125.98	593276	{}	\N	2020-10-12 15:17:47.193567
52142	IT	2020-10-06	126.43	127.965	123.66	124.21	748729	{}	\N	2020-10-12 15:17:47.197891
52143	IT	2020-10-07	124.5	125.995	123.86	125.62	914649	{}	\N	2020-10-12 15:17:47.202006
52144	IT	2020-10-08	126.97	128.6	126.45	126.83	1093876	{}	\N	2020-10-12 15:17:47.206493
52145	IT	2020-10-09	128.12	128.569	126	126.4	1278745	{}	\N	2020-10-12 15:17:47.210816
52146	IT	2020-10-12	127.2	127.54	125.06	127.54	251014	{}	\N	2020-10-12 15:17:47.215264
52147	ITIC	2020-09-28	136.06	138.3	135.15	135.15	5639	{}	\N	2020-10-12 15:17:48.1507
52148	ITIC	2020-09-29	135.79	135.81	134.92	134.92	1653	{}	\N	2020-10-12 15:17:48.160649
52149	ITIC	2020-09-30	135.14	135.14	129.79	130.06	8222	{}	\N	2020-10-12 15:17:48.167704
52150	ITIC	2020-10-01	131.2	134.01	129.85	134.01	11108	{}	\N	2020-10-12 15:17:48.174118
52151	ITIC	2020-10-02	133.46	138.13	133.46	137.85	8345	{}	\N	2020-10-12 15:17:48.179767
52152	ITIC	2020-10-05	139.5	141.93	135	138	11617	{}	\N	2020-10-12 15:17:48.184312
52153	ITIC	2020-10-06	136.44	138.5	135.91	136.35	5089	{}	\N	2020-10-12 15:17:48.18919
52154	ITIC	2020-10-07	140.12	140.12	137.04	139.85	7336	{}	\N	2020-10-12 15:17:48.193596
52155	ITIC	2020-10-08	139.8	143.99	139.8	142.83	4972	{}	\N	2020-10-12 15:17:48.198077
52156	ITIC	2020-10-09	143.93	150.32	143.93	149.7	16644	{}	\N	2020-10-12 15:17:48.201973
52157	ITIC	2020-10-12	151.5	151.8	150.68	150.8	1591	{}	\N	2020-10-12 15:17:48.205912
52158	IUSV	2020-09-28	53.85	54.429	53.85	54.18	673813	{}	\N	2020-10-12 15:17:50.176246
52159	IUSV	2020-09-29	54.19	54.19	53.534	53.71	303617	{}	\N	2020-10-12 15:17:50.184751
52160	IUSV	2020-09-30	53.88	54.72	53.88	54.29	611252	{}	\N	2020-10-12 15:17:50.190476
52161	IUSV	2020-10-01	54.44	54.63	53.95	54.33	531818	{}	\N	2020-10-12 15:17:50.195578
52162	IUSV	2020-10-02	53.47	54.7501	53.47	54.48	526238	{}	\N	2020-10-12 15:17:50.200392
52163	IUSV	2020-10-05	54.84	55.39	54.84	55.35	314519	{}	\N	2020-10-12 15:17:50.20461
52164	IUSV	2020-10-06	55.47	55.9832	54.7692	54.88	294630	{}	\N	2020-10-12 15:17:50.208677
52165	IUSV	2020-10-07	55.3703	55.89	55.33	55.69	438550	{}	\N	2020-10-12 15:17:50.212692
52166	IUSV	2020-10-08	56	56.41	55.96	56.41	285628	{}	\N	2020-10-12 15:17:50.216583
52167	IUSV	2020-10-09	56.76	56.7999	56.33	56.47	310195	{}	\N	2020-10-12 15:17:50.220318
52168	IUSV	2020-10-12	56.71	56.94	56.5458	56.9301	95203	{}	\N	2020-10-12 15:17:50.224393
52169	IWL	2020-09-28	80.82	81.05	80.5276	80.93	23130	{}	\N	2020-10-12 15:17:52.341625
52170	IWL	2020-09-29	81.01	81.0336	80.421	80.6	18578	{}	\N	2020-10-12 15:17:52.349602
52171	IWL	2020-09-30	80.74	81.98	80.74	81.2	22408	{}	\N	2020-10-12 15:17:52.355417
52172	IWL	2020-10-01	81.91	82.1346	81.38	81.75	90585	{}	\N	2020-10-12 15:17:52.360202
52173	IWL	2020-10-02	80.36	81.2499	80.36	80.74	31257	{}	\N	2020-10-12 15:17:52.365052
52174	IWL	2020-10-05	81.41	82.18	81.41	82.18	368275	{}	\N	2020-10-12 15:17:52.369805
52175	IWL	2020-10-06	82.24	82.61	80.8	81.04	53046	{}	\N	2020-10-12 15:17:52.379197
52176	IWL	2020-10-07	81.72	82.4501	81.68	82.29	55319	{}	\N	2020-10-12 15:17:52.384789
52177	IWL	2020-10-08	82.91	82.93	82.65	82.91	166246	{}	\N	2020-10-12 15:17:52.389472
52178	IWL	2020-10-09	83.33	83.81	83.33	83.79	42013	{}	\N	2020-10-12 15:17:52.393301
52179	IWL	2020-10-12	84.54	85.11	84.4	85.11	52413	{}	\N	2020-10-12 15:17:52.397516
52180	IXC	2020-09-28	16.92	16.995	16.8	16.85	786837	{}	\N	2020-10-12 15:17:54.629775
52181	IXC	2020-09-29	16.76	16.76	16.35	16.48	609591	{}	\N	2020-10-12 15:17:54.640617
52182	IXC	2020-09-30	16.5	16.68	16.35	16.4	794242	{}	\N	2020-10-12 15:17:54.647573
52183	IXC	2020-10-01	16.36	16.36	15.934	16.02	582514	{}	\N	2020-10-12 15:17:54.652667
52184	IXC	2020-10-02	15.77	16.18	15.61	16.1	1115966	{}	\N	2020-10-12 15:17:54.657734
52185	IXC	2020-10-05	16.26	16.535	16.21	16.52	625135	{}	\N	2020-10-12 15:17:54.662664
52186	IXC	2020-10-06	16.75	16.87	16.31	16.35	675239	{}	\N	2020-10-12 15:17:54.666818
52187	IXC	2020-10-07	16.49	16.571	16.28	16.53	1259549	{}	\N	2020-10-12 15:17:54.672349
52188	IXC	2020-10-08	16.67	17.06	16.665	17.06	1052378	{}	\N	2020-10-12 15:17:54.676716
52189	IXC	2020-10-09	17.19	17.27	16.9	16.92	1122670	{}	\N	2020-10-12 15:17:54.681208
52190	IXC	2020-10-12	16.93	16.97	16.77	16.86	273184	{}	\N	2020-10-12 15:17:54.68526
52191	IYG	2020-09-28	120.86	122.71	120.86	121.99	39736	{}	\N	2020-10-12 15:17:55.639666
52192	IYG	2020-09-29	121.94	121.9839	120.4525	120.75	58162	{}	\N	2020-10-12 15:17:55.650297
52193	IYG	2020-09-30	121.17	123.24	121.17	121.86	68646	{}	\N	2020-10-12 15:17:55.656221
52194	IYG	2020-10-01	122.65	123.77	122.1757	123.12	396398	{}	\N	2020-10-12 15:17:55.662131
52195	IYG	2020-10-02	120.89	124.51	120.89	123.85	110645	{}	\N	2020-10-12 15:17:55.666703
52196	IYG	2020-10-05	125.05	126.04	124.78	125.88	63407	{}	\N	2020-10-12 15:17:55.671141
52197	IYG	2020-10-06	127.05	128.0299	124.29	124.59	106825	{}	\N	2020-10-12 15:17:55.675419
52198	IYG	2020-10-07	125.71	127.19	125.66	126.6	4703275	{}	\N	2020-10-12 15:17:55.680304
52199	IYG	2020-10-08	127.38	128.7	127.2601	128.47	719873	{}	\N	2020-10-12 15:17:55.684242
52200	IYG	2020-10-09	129.12	129.69	128.01	129.04	620821	{}	\N	2020-10-12 15:17:55.687865
52201	IYG	2020-10-12	129.26	130.0628	129.26	130.0628	26561	{}	\N	2020-10-12 15:17:55.6916
52202	IYT	2020-09-28	202.29	203.82	201.81	202.46	153527	{}	\N	2020-10-12 15:17:57.070938
52203	IYT	2020-09-29	202.9	202.97	200.04	200.04	256301	{}	\N	2020-10-12 15:17:57.07946
52204	IYT	2020-09-30	201.42	203.13	197.66	198.93	322954	{}	\N	2020-10-12 15:17:57.084869
52205	IYT	2020-10-01	200.93	200.93	197.41	198.57	163227	{}	\N	2020-10-12 15:17:57.090271
52206	IYT	2020-10-02	194.69	201.75	194.26	199.97	319186	{}	\N	2020-10-12 15:17:57.095182
52207	IYT	2020-10-05	200.54	203.781	200.54	203.37	111828	{}	\N	2020-10-12 15:17:57.099803
52208	IYT	2020-10-06	204.49	206.17	201.53	201.62	300478	{}	\N	2020-10-12 15:17:57.103666
52209	IYT	2020-10-07	205.08	207.74	204.13	207	202191	{}	\N	2020-10-12 15:17:57.108354
52210	IYT	2020-10-08	208.2	208.65	206.13	208.17	216722	{}	\N	2020-10-12 15:17:57.112277
52211	IYT	2020-10-09	210	210.9	208.4108	209.87	266251	{}	\N	2020-10-12 15:17:57.116413
52212	IYT	2020-10-12	211.84	211.84	209.94	210.798	47269	{}	\N	2020-10-12 15:17:57.120166
52213	IYY	2020-09-28	165.49	166.52	165.345	165.96	11828	{}	\N	2020-10-12 15:17:58.232585
52214	IYY	2020-09-29	166.35	166.4005	165.14	165.37	17053	{}	\N	2020-10-12 15:17:58.242687
52215	IYY	2020-09-30	165.46	168.1	165.46	166.67	21142	{}	\N	2020-10-12 15:17:58.249143
52216	IYY	2020-10-01	168.14	168.39	167.04	167.95	39829	{}	\N	2020-10-12 15:17:58.254545
52217	IYY	2020-10-02	164.99	167.319	164.99	166.48	19956	{}	\N	2020-10-12 15:17:58.259374
52218	IYY	2020-10-05	167.83	169.43	167.83	169.42	26325	{}	\N	2020-10-12 15:17:58.264131
52219	IYY	2020-10-06	169.83	170.02	167.0013	167.383	13371	{}	\N	2020-10-12 15:17:58.268166
52220	IYY	2020-10-07	168.62	170.4156	168.62	170.21	15172	{}	\N	2020-10-12 15:17:58.272375
52221	IYY	2020-10-08	171.12	171.56	170.9494	171.56	11293	{}	\N	2020-10-12 15:17:58.27662
52222	IYY	2020-10-09	172.24	173.34	172.24	172.85	21131	{}	\N	2020-10-12 15:17:58.281027
52223	IYY	2020-10-12	174.19	175	174.19	174.96	37015	{}	\N	2020-10-12 15:17:58.285229
52224	ISCF	2020-09-28	29.89	29.98	29.85	29.9389	15884	{}	\N	2020-10-12 15:18:00.619849
52225	ISCF	2020-09-29	29.98	30.02	29.861	29.9496	16069	{}	\N	2020-10-12 15:18:00.631806
52226	ISCF	2020-09-30	29.85	30.14	29.81	30.062	37911	{}	\N	2020-10-12 15:18:00.638817
52227	ISCF	2020-10-01	30.095	30.2399	30.095	30.1535	7764	{}	\N	2020-10-12 15:18:00.645984
52228	ISCF	2020-10-02	30.0074	30.14	29.96	30.0411	14674	{}	\N	2020-10-12 15:18:00.652615
52229	ISCF	2020-10-05	30.29	30.465	30.26	30.465	13973	{}	\N	2020-10-12 15:18:00.65859
52230	ISCF	2020-10-06	30.44	30.499	30.1206	30.18	23048	{}	\N	2020-10-12 15:18:00.664028
52231	ISCF	2020-10-07	30.3	30.46	30.28	30.33	10559	{}	\N	2020-10-12 15:18:00.668579
52232	ISCF	2020-10-08	30.5	30.54	30.42	30.48	11174	{}	\N	2020-10-12 15:18:00.673291
52233	ISCF	2020-10-09	30.71	30.89	30.71	30.879	13795	{}	\N	2020-10-12 15:18:00.677564
52234	ISCF	2020-10-12	30.89	31	30.885	31	17083	{}	\N	2020-10-12 15:18:00.68173
52235	ISR	2020-09-28	0.53	0.5829	0.53	0.5697	167401	{}	\N	2020-10-12 15:18:01.578396
52236	ISR	2020-09-29	0.5801	0.6	0.5696	0.5795	166382	{}	\N	2020-10-12 15:18:01.585979
52237	ISR	2020-09-30	0.5849	0.5948	0.555601	0.5829	122451	{}	\N	2020-10-12 15:18:01.592124
52238	ISR	2020-10-01	0.58	0.5829	0.5651	0.5658	142369	{}	\N	2020-10-12 15:18:01.597615
52239	ISR	2020-10-02	0.56	0.568	0.56	0.5651	136562	{}	\N	2020-10-12 15:18:01.602812
52240	ISR	2020-10-05	0.56	0.57	0.55	0.5593	216986	{}	\N	2020-10-12 15:18:01.60763
52241	ISR	2020-10-06	0.5595	0.5712	0.55	0.5532	171281	{}	\N	2020-10-12 15:18:01.612125
52242	ISR	2020-10-07	0.5512	0.56	0.5512	0.5564	196220	{}	\N	2020-10-12 15:18:01.616378
52243	ISR	2020-10-08	0.5508	0.5755	0.5508	0.5676	156655	{}	\N	2020-10-12 15:18:01.620623
52244	ISR	2020-10-09	0.5851	0.5877	0.56	0.57	197937	{}	\N	2020-10-12 15:18:01.625102
52245	ISR	2020-10-12	0.58	0.589	0.5569	0.57	116692	{}	\N	2020-10-12 15:18:01.629513
52246	ITA	2020-09-28	160.44	163.54	160.44	161.82	188182	{}	\N	2020-10-12 15:18:02.579464
52247	ITA	2020-09-29	161.93	161.93	159.59	159.86	130064	{}	\N	2020-10-12 15:18:02.58715
52248	ITA	2020-09-30	160.26	161.9	157.18	158.41	150841	{}	\N	2020-10-12 15:18:02.593541
52249	ITA	2020-10-01	158.9	161.6	157.96	159.19	153455	{}	\N	2020-10-12 15:18:02.59877
52250	ITA	2020-10-02	156.62	161.32	156.52	160.88	169259	{}	\N	2020-10-12 15:18:02.603713
52251	ITA	2020-10-05	161.94	163.31	161.3319	162.18	109150	{}	\N	2020-10-12 15:18:02.608049
52252	ITA	2020-10-06	162.52	164.06	159.19	159.19	136020	{}	\N	2020-10-12 15:18:02.612696
52253	ITA	2020-10-07	160.72	163.23	160.72	162.68	112703	{}	\N	2020-10-12 15:18:02.617122
52254	ITA	2020-10-08	163.3	165.44	162.7675	165.44	104363	{}	\N	2020-10-12 15:18:02.62155
52255	ITA	2020-10-09	166.08	166.5899	164.1401	164.25	130621	{}	\N	2020-10-12 15:18:02.62591
52256	ITA	2020-10-12	164.57	165.22	163.61	165.1	42916	{}	\N	2020-10-12 15:18:02.630866
52257	ITOT	2020-09-28	75.01	75.45	74.85	75.3	752139	{}	\N	2020-10-12 15:18:03.739451
52258	ITOT	2020-09-29	75.28	75.425	74.75	74.93	1071587	{}	\N	2020-10-12 15:18:03.749597
52259	ITOT	2020-09-30	75.1	76.18	74.98	75.49	972952	{}	\N	2020-10-12 15:18:03.756431
52260	ITOT	2020-10-01	76.09	76.33	75.61	76.07	1260512	{}	\N	2020-10-12 15:18:03.762315
52261	ITOT	2020-10-02	74.69	75.91	74.6	75.51	1219317	{}	\N	2020-10-12 15:18:03.77188
52262	ITOT	2020-10-05	76.03	76.89	76.0117	76.88	728459	{}	\N	2020-10-12 15:18:03.776783
52263	ITOT	2020-10-06	76.94	77.535	75.79	75.85	842294	{}	\N	2020-10-12 15:18:03.782001
52264	ITOT	2020-10-07	76.6	77.425	76.6	77.2	1041796	{}	\N	2020-10-12 15:18:03.786615
52265	ITOT	2020-10-08	77.78	77.96	77.52	77.96	754569	{}	\N	2020-10-12 15:18:03.791673
52266	ITOT	2020-10-09	78.32	78.73	78.201	78.63	846441	{}	\N	2020-10-12 15:18:03.796714
52267	ITOT	2020-10-12	79.16	79.6557	79.07	79.63	216159	{}	\N	2020-10-12 15:18:03.800692
52268	IUSB	2020-09-28	54.35	54.42	54.34	54.38	513908	{}	\N	2020-10-12 15:18:05.157115
52269	IUSB	2020-09-29	54.43	54.48	54.406	54.43	315871	{}	\N	2020-10-12 15:18:05.169874
52270	IUSB	2020-09-30	54.38	54.41	54.3124	54.38	617860	{}	\N	2020-10-12 15:18:05.177271
52271	IUSB	2020-10-01	54.22	54.3399	54.205	54.27	466603	{}	\N	2020-10-12 15:18:05.183258
52272	IUSB	2020-10-02	54.36	54.36	54.235	54.24	361581	{}	\N	2020-10-12 15:18:05.188418
52273	IUSB	2020-10-05	54.22	54.23	54.15	54.15	610488	{}	\N	2020-10-12 15:18:05.1945
52274	IUSB	2020-10-06	54.13	54.29	54.13	54.22	507955	{}	\N	2020-10-12 15:18:05.198697
52275	IUSB	2020-10-07	54.15	54.24	54.13	54.14	350363	{}	\N	2020-10-12 15:18:05.203
52276	IUSB	2020-10-08	54.18	54.26	54.18	54.2	211583	{}	\N	2020-10-12 15:18:05.207725
52277	IUSB	2020-10-09	54.2	54.28	54.18	54.26	212074	{}	\N	2020-10-12 15:18:05.212163
52278	IUSB	2020-10-12	54.27	54.35	54.27	54.32	67156	{}	\N	2020-10-12 15:18:05.216503
52279	IVLU	2020-09-28	19.77	19.85	19.7514	19.84	106274	{}	\N	2020-10-12 15:18:06.994125
52280	IVLU	2020-09-29	19.75	19.82	19.63	19.68	156045	{}	\N	2020-10-12 15:18:07.005486
52281	IVLU	2020-09-30	19.645	19.73	19.53	19.57	43490	{}	\N	2020-10-12 15:18:07.01254
52282	IVLU	2020-10-01	19.55	19.5984	19.49	19.51	282431	{}	\N	2020-10-12 15:18:07.018522
52283	IVLU	2020-10-02	19.3925	19.5999	19.3901	19.54	76394	{}	\N	2020-10-12 15:18:07.024294
52284	IVLU	2020-10-05	19.82	19.96	19.81	19.91	71500	{}	\N	2020-10-12 15:18:07.029552
52285	IVLU	2020-10-06	20.05	20.09	19.83	19.83	35997	{}	\N	2020-10-12 15:18:07.033643
52286	IVLU	2020-10-07	19.96	20.0499	19.96	20.04	41924	{}	\N	2020-10-12 15:18:07.037702
52287	IVLU	2020-10-08	20.11	20.1599	20.07	20.11	62020	{}	\N	2020-10-12 15:18:07.04306
52288	IVLU	2020-10-09	20.19	20.2	20.12	20.14	184013	{}	\N	2020-10-12 15:18:07.04844
52289	IVLU	2020-10-12	20.14	20.18	20.082	20.17	33891	{}	\N	2020-10-12 15:18:07.053298
52290	IWC	2020-09-28	89.87	90.59	89.665	90.27	13935	{}	\N	2020-10-12 15:18:08.204335
52291	IWC	2020-09-29	90.72	90.96	89.5822	90.56	55590	{}	\N	2020-10-12 15:18:08.21828
52292	IWC	2020-09-30	91.1167	91.72	89.821	90.47	12338	{}	\N	2020-10-12 15:18:08.225713
52293	IWC	2020-10-01	90.91	91.86	90.63	91.86	77399	{}	\N	2020-10-12 15:18:08.231494
52294	IWC	2020-10-02	90.938	92.3812	90.92	92.06	16455	{}	\N	2020-10-12 15:18:08.236573
52295	IWC	2020-10-05	92.63	94.97	92.63	94.95	96125	{}	\N	2020-10-12 15:18:08.241636
52296	IWC	2020-10-06	95.74	97.2	94.65	94.66	53109	{}	\N	2020-10-12 15:18:08.246533
52297	IWC	2020-10-07	95.78	97.32	95.65	97.25	13664	{}	\N	2020-10-12 15:18:08.250953
52298	IWC	2020-10-08	97.96	98.39	97.41	98.06	85907	{}	\N	2020-10-12 15:18:08.25544
52299	IWC	2020-10-09	98.9627	99.05	98.35	98.82	15599	{}	\N	2020-10-12 15:18:08.260291
52300	IWC	2020-10-12	99.07	99.16	98.59	99.0703	3794	{}	\N	2020-10-12 15:18:08.265033
52301	IWX	2020-09-28	50.865	51.2481	50.865	51.01	54476	{}	\N	2020-10-12 15:18:09.43461
52302	IWX	2020-09-29	51.05	51.05	50.51	50.65	210140	{}	\N	2020-10-12 15:18:09.446846
52303	IWX	2020-09-30	50.83	51.49	50.83	51.07	34410	{}	\N	2020-10-12 15:18:09.453597
52304	IWX	2020-10-01	51.41	51.41	50.795	51.08	34783	{}	\N	2020-10-12 15:18:09.459343
52305	IWX	2020-10-02	50.32	51.35	50.32	51.01	215953	{}	\N	2020-10-12 15:18:09.464908
52306	IWX	2020-10-05	51.43	51.8	51.37	51.77	1407539	{}	\N	2020-10-12 15:18:09.470182
52307	IWX	2020-10-06	51.89	52.24	51.1909	51.26	108630	{}	\N	2020-10-12 15:18:09.474736
52308	IWX	2020-10-07	51.795	52.21	51.76	52.03	1464197	{}	\N	2020-10-12 15:18:09.479234
52309	IWX	2020-10-08	52.29	52.66	52.29	52.65	63852	{}	\N	2020-10-12 15:18:09.483538
52310	IWX	2020-10-09	52.95	53.0056	52.66	52.77	101473	{}	\N	2020-10-12 15:18:09.488078
52311	IWX	2020-10-12	53.01	53.26	52.92	53.26	16233	{}	\N	2020-10-12 15:18:09.492543
52312	IYF	2020-09-28	112.64	114.25	112.64	113.66	118869	{}	\N	2020-10-12 15:18:11.759653
52313	IYF	2020-09-29	113.54	113.54	112	112.47	90097	{}	\N	2020-10-12 15:18:11.774085
52314	IYF	2020-09-30	112.78	114.36	112.64	113.42	149937	{}	\N	2020-10-12 15:18:11.782017
52315	IYF	2020-10-01	113.98	114.89	113.37	114.38	128553	{}	\N	2020-10-12 15:18:11.788426
52316	IYF	2020-10-02	112.54	115.85	112.54	115.36	177876	{}	\N	2020-10-12 15:18:11.794661
52317	IYF	2020-10-05	116.14	117.1	115.66	117	184201	{}	\N	2020-10-12 15:18:11.799349
52318	IYF	2020-10-06	117.43	118.52	115.57	115.91	130435	{}	\N	2020-10-12 15:18:11.803839
52319	IYF	2020-10-07	116.78	117.79	116.63	117.35	175620	{}	\N	2020-10-12 15:18:11.808931
52320	IYF	2020-10-08	117.81	119.15	117.81	118.92	123600	{}	\N	2020-10-12 15:18:11.813689
52321	IYF	2020-10-09	119.44	119.688	118.38	119.1	128547	{}	\N	2020-10-12 15:18:11.818168
52322	IYF	2020-10-12	119.44	120.12	119.08	120.065	24232	{}	\N	2020-10-12 15:18:11.82258
52323	JAKK	2020-09-28	3.82	3.8705	3.63	3.82	127347	{}	\N	2020-10-12 15:18:12.89753
52324	JAKK	2020-09-29	3.82	3.95	3.7013	3.87	44580	{}	\N	2020-10-12 15:18:12.907746
52325	JAKK	2020-09-30	3.85	4.09	3.8	3.8	53198	{}	\N	2020-10-12 15:18:12.914687
52326	JAKK	2020-10-01	3.83	3.96	3.79	3.89	107922	{}	\N	2020-10-12 15:18:12.920559
52327	JAKK	2020-10-02	3.84	3.96	3.7956	3.96	65641	{}	\N	2020-10-12 15:18:12.925469
52328	JAKK	2020-10-05	4	4.16	3.72	3.89	127405	{}	\N	2020-10-12 15:18:12.930524
52329	JAKK	2020-10-06	3.8415	4	3.82	3.84	38770	{}	\N	2020-10-12 15:18:12.93503
52330	JAKK	2020-10-07	3.86	3.99	3.83	3.91	37715	{}	\N	2020-10-12 15:18:12.939386
52331	JAKK	2020-10-08	3.9299	4	3.8507	3.95	65682	{}	\N	2020-10-12 15:18:12.943769
52332	JAKK	2020-10-09	3.98	4.04	3.86	3.98	37734	{}	\N	2020-10-12 15:18:12.947992
52333	JAKK	2020-10-12	3.98	5.91	3.97	5.3503	8775485	{}	\N	2020-10-12 15:18:12.952227
52334	JD	2020-09-28	75.89	76.1	73.95	76.1	7940125	{}	\N	2020-10-12 15:18:14.383063
52335	JD	2020-09-29	76.12	76.4	74.74	75.16	5732389	{}	\N	2020-10-12 15:18:14.391268
52336	JD	2020-09-30	76.26	78.82	75.64	77.61	10827122	{}	\N	2020-10-12 15:18:14.39675
52337	JD	2020-10-01	78.75	79	77.47	78.57	7094371	{}	\N	2020-10-12 15:18:14.401263
52338	JD	2020-10-02	76.74	78.1384	75.76	76.1	8205702	{}	\N	2020-10-12 15:18:14.40607
52339	JD	2020-10-05	76.88	77.54	75.59	76.4	5996989	{}	\N	2020-10-12 15:18:14.410336
52340	JD	2020-10-06	76.89	78.52	76.31	76.69	9218733	{}	\N	2020-10-12 15:18:14.414223
52341	JD	2020-10-07	78.56	79.41	77.678	78.12	11838231	{}	\N	2020-10-12 15:18:14.417909
52342	JD	2020-10-08	78.51	78.59	77.1001	78.15	10837363	{}	\N	2020-10-12 15:18:14.421549
52343	JD	2020-10-09	79.09	80.15	78.67	79.79	10871909	{}	\N	2020-10-12 15:18:14.425342
52344	JD	2020-10-12	82.14	84.36	81.9	84.33	7494335	{}	\N	2020-10-12 15:18:14.429238
52345	JHMI	2020-09-28	38.7602	38.7602	38.7602	38.7602	376	{}	\N	2020-10-12 15:18:16.415834
52346	JHMI	2020-09-29	38.61	38.6376	38.5399	38.5399	2566	{}	\N	2020-10-12 15:18:16.423775
52347	JHMI	2020-09-30	39.08	39.1582	38.6246	38.6246	1325	{}	\N	2020-10-12 15:18:16.43003
52348	JHMI	2020-10-01	38.5323	38.5323	38.5323	38.5323	195	{}	\N	2020-10-12 15:18:16.435846
52349	JHMI	2020-10-02	38.9501	38.9501	38.9501	38.9501	140	{}	\N	2020-10-12 15:18:16.440926
52350	JHMI	2020-10-05	39.51	39.6082	39.51	39.6082	1107	{}	\N	2020-10-12 15:18:16.445694
52351	JHMI	2020-10-06	39.93	40	39.2744	39.2744	943	{}	\N	2020-10-12 15:18:16.4499
52352	JHMI	2020-10-07	40.0608	40.0608	40.0608	40.0608	157	{}	\N	2020-10-12 15:18:16.453819
52353	JHMI	2020-10-08	40.25	40.3673	40.1681	40.3673	400	{}	\N	2020-10-12 15:18:16.458263
52354	JHMI	2020-10-09	40.54	40.66	40.54	40.66	1066	{}	\N	2020-10-12 15:18:16.462695
52355	JILL	2020-09-28	0.5201	0.5585	0.52	0.545	413459	{}	\N	2020-10-12 15:18:17.379093
52356	JILL	2020-09-29	0.545	0.5706	0.5212	0.5476	736330	{}	\N	2020-10-12 15:18:17.390226
52357	JILL	2020-09-30	0.5305	0.55	0.5305	0.5395	4160250	{}	\N	2020-10-12 15:18:17.396591
52358	JILL	2020-10-01	0.57	0.7	0.54	0.5998	10038821	{}	\N	2020-10-12 15:18:17.401703
52359	JILL	2020-10-02	0.546	0.588299	0.546	0.5837	1210602	{}	\N	2020-10-12 15:18:17.406648
52360	JILL	2020-10-05	0.56	0.5822	0.56	0.5776	852200	{}	\N	2020-10-12 15:18:17.411564
52361	JILL	2020-10-06	0.56	0.6298	0.56	0.5722	1619937	{}	\N	2020-10-12 15:18:17.415601
52362	JILL	2020-10-07	0.583	0.615	0.5788	0.58	1042318	{}	\N	2020-10-12 15:18:17.419663
52363	JILL	2020-10-08	0.6	0.6347	0.5901	0.6199	1353324	{}	\N	2020-10-12 15:18:17.423888
52364	JILL	2020-10-09	0.61	0.675	0.61	0.6599	1618970	{}	\N	2020-10-12 15:18:17.428267
52365	JILL	2020-10-12	0.73	0.73	0.6501	0.6592	1308374	{}	\N	2020-10-12 15:18:17.432651
52366	JJU	2020-09-28	37.8	38.32	37.8	38.32	318	{}	\N	2020-10-12 15:18:19.38611
52367	JJU	2020-09-29	38.119	38.28	38.119	38.28	170	{}	\N	2020-10-12 15:18:19.393675
52368	JJU	2020-09-30	37.602	37.87	37.602	37.87	167	{}	\N	2020-10-12 15:18:19.399693
52369	JJU	2020-10-01	36.98	37.275	36.97	37.275	1177	{}	\N	2020-10-12 15:18:19.404748
52370	JJU	2020-10-02	37.77	37.77	37.77	37.77	9	{}	\N	2020-10-12 15:18:19.410314
52371	JJU	2020-10-05	38.175	38.175	38.175	38.175	1	{}	\N	2020-10-12 15:18:19.414832
52372	JJU	2020-10-06	38.04	38.04	38.04	38.04	1	{}	\N	2020-10-12 15:18:19.419245
52373	JJU	2020-10-07	38.51	38.51	38.51	38.51	49	{}	\N	2020-10-12 15:18:19.423895
52374	JJU	2020-10-08	38.85	38.85	38.84	38.84	166	{}	\N	2020-10-12 15:18:19.428494
52375	JJU	2020-10-09	39.7	39.9754	39.7	39.9754	283	{}	\N	2020-10-12 15:18:19.432993
52376	JLS	2020-09-28	19.124	19.2905	19.124	19.21	11485	{}	\N	2020-10-12 15:18:20.99597
52377	JLS	2020-09-29	19.2055	19.2055	19.06	19.14	45294	{}	\N	2020-10-12 15:18:21.004289
52378	JLS	2020-09-30	19.1994	19.2	18.93	18.93	89318	{}	\N	2020-10-12 15:18:21.010603
52379	JLS	2020-10-01	19.05	19.172	18.88	18.88	39677	{}	\N	2020-10-12 15:18:21.016239
52380	JLS	2020-10-02	18.87	18.87	18.73	18.84	17442	{}	\N	2020-10-12 15:18:21.021467
52381	JLS	2020-10-05	18.84	19.2161	18.51	19.08	97615	{}	\N	2020-10-12 15:18:21.026587
52382	JLS	2020-10-06	19.05	19.2799	19.05	19.12	41144	{}	\N	2020-10-12 15:18:21.031365
52383	JLS	2020-10-07	19.122	19.1578	19.08	19.1	14826	{}	\N	2020-10-12 15:18:21.035935
52384	JLS	2020-10-08	19.0706	19.2272	19.0706	19.2272	3734	{}	\N	2020-10-12 15:18:21.040789
52385	JLS	2020-10-09	19.1708	19.22	19.143	19.16	28449	{}	\N	2020-10-12 15:18:21.045775
52386	JP	2020-09-28	1.8	1.8	1.7	1.7	18307	{}	\N	2020-10-12 15:18:23.296516
52387	JP	2020-09-29	1.67	1.69	1.66	1.69	15400	{}	\N	2020-10-12 15:18:23.3074
52388	JP	2020-09-30	1.75	1.75	1.68	1.685	9118	{}	\N	2020-10-12 15:18:23.314142
52389	JP	2020-10-01	1.74	1.74	1.67	1.74	4698	{}	\N	2020-10-12 15:18:23.319764
52390	JP	2020-10-02	1.65	1.6965	1.65	1.68	10460	{}	\N	2020-10-12 15:18:23.324893
52391	JP	2020-10-05	1.69	1.7	1.6769	1.68	9158	{}	\N	2020-10-12 15:18:23.329863
52392	JP	2020-10-06	1.68	1.76	1.67	1.6701	7674	{}	\N	2020-10-12 15:18:23.33433
52393	JP	2020-10-07	1.7	1.7002	1.6752	1.6752	10364	{}	\N	2020-10-12 15:18:23.338486
52394	JP	2020-10-08	1.65	1.6846	1.58	1.64	99288	{}	\N	2020-10-12 15:18:23.34321
52395	JP	2020-10-09	1.7	1.7	1.63	1.67	32158	{}	\N	2020-10-12 15:18:23.347466
52396	JP	2020-10-12	1.64	1.675	1.64	1.658	1844	{}	\N	2020-10-12 15:18:23.351709
52397	JPI	2020-09-28	22.01	22.14	21.94	22.1	62100	{}	\N	2020-10-12 15:18:24.322287
52398	JPI	2020-09-29	22.07	22.2	21.92	21.96	94130	{}	\N	2020-10-12 15:18:24.331598
52399	JPI	2020-09-30	21.98	22.15	21.98	21.98	188705	{}	\N	2020-10-12 15:18:24.337028
52400	JPI	2020-10-01	22.11	22.21	22	22.15	70857	{}	\N	2020-10-12 15:18:24.343083
52401	JPI	2020-10-02	22.08	22.31	22.0221	22.28	56911	{}	\N	2020-10-12 15:18:24.347952
52402	JPI	2020-10-05	22.3	22.4694	22.3	22.39	79976	{}	\N	2020-10-12 15:18:24.352546
52403	JPI	2020-10-06	22.4	22.5726	22.34	22.4	103803	{}	\N	2020-10-12 15:18:24.356876
52404	JPI	2020-10-07	22.4	22.6	22.4	22.57	77608	{}	\N	2020-10-12 15:18:24.361268
52405	JPI	2020-10-08	22.54	22.66	22.54	22.62	47644	{}	\N	2020-10-12 15:18:24.365342
52406	JPI	2020-10-09	22.59	22.71	22.59	22.65	119690	{}	\N	2020-10-12 15:18:24.369505
52407	JPI	2020-10-12	22.75	22.79	22.6742	22.71	40918	{}	\N	2020-10-12 15:18:24.373769
52408	JPS	2020-09-28	8.73	8.81	8.72	8.78	465705	{}	\N	2020-10-12 15:18:26.535844
52409	JPS	2020-09-29	8.75	8.805	8.71	8.72	1146195	{}	\N	2020-10-12 15:18:26.543882
52410	JPS	2020-09-30	8.7	8.86	8.68	8.81	2488363	{}	\N	2020-10-12 15:18:26.548827
52411	JPS	2020-10-01	8.89	8.91	8.84	8.91	1031011	{}	\N	2020-10-12 15:18:26.553679
52412	JPS	2020-10-02	8.75	8.92	8.7	8.9	1438721	{}	\N	2020-10-12 15:18:26.558342
52413	JPS	2020-10-05	8.9	9	8.85	8.99	764070	{}	\N	2020-10-12 15:18:26.562534
52414	JPS	2020-10-06	8.99	9.02	8.93	8.94	627903	{}	\N	2020-10-12 15:18:26.5666
52415	JPS	2020-10-07	8.95	9	8.94	8.94	816322	{}	\N	2020-10-12 15:18:26.570776
52416	JPS	2020-10-08	8.94	8.99	8.94	8.98	486568	{}	\N	2020-10-12 15:18:26.576674
52417	JPS	2020-10-09	9.01	9.04	8.96	8.99	592699	{}	\N	2020-10-12 15:18:26.581371
52418	JPS	2020-10-12	9	9.03	8.9927	9.025	151614	{}	\N	2020-10-12 15:18:26.586168
52419	JSM	2020-09-28	21.65	22.805	21.51	22.8	22308	{}	\N	2020-10-12 15:18:27.600201
52420	JSM	2020-09-29	22.25	22.65	22.03	22.65	8084	{}	\N	2020-10-12 15:18:27.609316
52421	JSM	2020-09-30	22.7	22.86	21.1	21.39	102645	{}	\N	2020-10-12 15:18:27.615009
52422	JSM	2020-10-01	21.54	21.8248	21.23	21.46	37823	{}	\N	2020-10-12 15:18:27.619854
52423	JSM	2020-10-02	21.24	21.35	20.8	21.24	38719	{}	\N	2020-10-12 15:18:27.625185
52424	JSM	2020-10-05	21.39	21.74	21.1004	21.29	20112	{}	\N	2020-10-12 15:18:27.629692
52425	JSM	2020-10-06	21.29	21.844	21.2852	21.56	26074	{}	\N	2020-10-12 15:18:27.63339
52426	JSM	2020-10-07	21.74	21.85	21.675	21.755	13237	{}	\N	2020-10-12 15:18:27.637241
52427	JSM	2020-10-08	21.88	22.33	21.87	22.1	20378	{}	\N	2020-10-12 15:18:27.64171
52428	JSM	2020-10-09	22.23	22.39	21.51	22.12	21906	{}	\N	2020-10-12 15:18:27.645648
52429	JSM	2020-10-12	22.21	22.6594	22.05	22.435	6291	{}	\N	2020-10-12 15:18:27.649448
52430	JSMD	2020-09-28	51.43	51.7639	51.33	51.45	7966	{}	\N	2020-10-12 15:18:29.262587
52431	JSMD	2020-09-29	51.29	52.05	51.29	51.6714	11659	{}	\N	2020-10-12 15:18:29.274459
52432	JSMD	2020-09-30	51.18	52.7927	51.15	51.695	17638	{}	\N	2020-10-12 15:18:29.281405
52433	JSMD	2020-10-01	51.88	52.6099	51.88	52.3	3476	{}	\N	2020-10-12 15:18:29.286944
52434	JSMD	2020-10-02	51.5	52.58	51.5	52.38	8468	{}	\N	2020-10-12 15:18:29.292342
52435	JSMD	2020-10-05	53.3264	53.76	53.2555	53.65	31057	{}	\N	2020-10-12 15:18:29.296701
52436	JSMD	2020-10-06	53.9844	54.69	53.65	53.67	10622	{}	\N	2020-10-12 15:18:29.301644
52437	JSMD	2020-10-07	54.92	54.92	54.425	54.8986	3288	{}	\N	2020-10-12 15:18:29.305505
52438	JSMD	2020-10-08	55.39	55.53	55.0501	55.45	3881	{}	\N	2020-10-12 15:18:29.309535
52439	JSMD	2020-10-09	56.01	56.31	55.8629	56.116	6141	{}	\N	2020-10-12 15:18:29.313501
52440	JSMD	2020-10-12	56.6	56.6	56.1501	56.2948	9318	{}	\N	2020-10-12 15:18:29.317541
52441	JCAP	2020-09-28	17.18	17.19	17.14	17.15	249805	{}	\N	2020-10-12 15:18:31.242235
52442	JCAP	2020-09-29	17.15	17.17	17.13	17.17	451464	{}	\N	2020-10-12 15:18:31.252477
52443	JCAP	2020-09-30	17.16	17.18	17.14	17.14	493995	{}	\N	2020-10-12 15:18:31.259204
52444	JCAP	2020-10-01	17.17	17.17	17.14	17.15	648878	{}	\N	2020-10-12 15:18:31.264692
52445	JCAP	2020-10-02	17.13	17.2	17.13	17.17	414957	{}	\N	2020-10-12 15:18:31.2696
52446	JCAP	2020-10-05	17.18	17.2	17.14	17.17	786868	{}	\N	2020-10-12 15:18:31.274387
52447	JCAP	2020-10-06	17.17	17.17	17.13	17.13	856556	{}	\N	2020-10-12 15:18:31.278824
52448	JCAP	2020-10-07	17.17	17.17	17.14	17.16	899516	{}	\N	2020-10-12 15:18:31.283108
52449	JCAP	2020-10-08	17.17	17.2	17.145	17.16	1052993	{}	\N	2020-10-12 15:18:31.287749
52450	JCAP	2020-10-09	17.17	17.19	17.16	17.19	394503	{}	\N	2020-10-12 15:18:31.297777
52451	JCAP	2020-10-12	17.17	17.21	17.17	17.2	88122	{}	\N	2020-10-12 15:18:31.302659
52452	JE	2020-09-28	0.39	0.4	0.285	0.291	144526598	{}	\N	2020-10-12 15:18:32.400803
52453	JE	2020-09-29	7.44	7.44	5.1	5.75	18722440	{}	\N	2020-10-12 15:18:32.411594
52454	JE	2020-09-30	5.98	5.98	5.18	5.21	6902644	{}	\N	2020-10-12 15:18:32.425368
52455	JE	2020-10-01	5.25	5.3	4.8	4.87	6320068	{}	\N	2020-10-12 15:18:32.431575
52456	JE	2020-10-02	4.48	5.96	4.27	5.01	14350202	{}	\N	2020-10-12 15:18:32.436883
52457	JE	2020-10-05	4.94	5.33	4.86	5.17	6983117	{}	\N	2020-10-12 15:18:32.44136
52458	JE	2020-10-06	5	5.6829	4.92	5.32	13286092	{}	\N	2020-10-12 15:18:32.44628
52459	JE	2020-10-07	5.26	5.59	5.22	5.32	5432281	{}	\N	2020-10-12 15:18:32.4513
52460	JE	2020-10-08	5.44	5.76	5.23	5.6	11678848	{}	\N	2020-10-12 15:18:32.457104
52461	JE	2020-10-09	6.47	8.5	6.1	6.41	93555920	{}	\N	2020-10-12 15:18:32.461811
52462	JE	2020-10-12	6.85	6.98	6.5	6.6233	6619903	{}	\N	2020-10-12 15:18:32.467043
52463	JHMT	2020-09-28	65.3795	65.699	65.255	65.635	2249	{}	\N	2020-10-12 15:18:33.85968
52464	JHMT	2020-09-29	65.75	65.91	65.6056	65.8973	2558	{}	\N	2020-10-12 15:18:33.869754
52465	JHMT	2020-09-30	66.27	66.53	65.9734	65.9734	2957	{}	\N	2020-10-12 15:18:33.876151
52466	JHMT	2020-10-01	66.88	66.88	66.6	66.842	1804	{}	\N	2020-10-12 15:18:33.88197
52467	JHMT	2020-10-02	66.57	66.57	65.8184	65.8184	615	{}	\N	2020-10-12 15:18:33.886684
52468	JHMT	2020-10-05	66.6701	67.24	66.6701	67.24	1665	{}	\N	2020-10-12 15:18:33.891659
52469	JHMT	2020-10-06	67.37	67.37	66.7812	66.7812	1194	{}	\N	2020-10-12 15:18:33.896001
52470	JHMT	2020-10-07	68.0282	68.3381	68.01	68.3	2186	{}	\N	2020-10-12 15:18:33.900611
52471	JHMT	2020-10-08	68.7214	68.8048	68.7214	68.8048	1490	{}	\N	2020-10-12 15:18:33.904743
52472	JHMT	2020-10-09	69.645	69.8739	69.645	69.7899	1532	{}	\N	2020-10-12 15:18:33.908695
52473	JHMT	2020-10-12	70.99	70.99	70.48	70.48	688	{}	\N	2020-10-12 15:18:33.912685
52474	JJS	2020-09-28	37.35	37.35	37.0522	37.0522	390	{}	\N	2020-10-12 15:18:35.302744
52475	JJS	2020-09-29	37.56	37.56	37.56	37.56	2	{}	\N	2020-10-12 15:18:35.311098
52476	JJS	2020-09-30	37.7904	37.7904	37.7904	37.7904	150	{}	\N	2020-10-12 15:18:35.317006
52477	JJS	2020-10-01	37.58	37.58	37.355	37.355	402	{}	\N	2020-10-12 15:18:35.321796
52478	JJS	2020-10-02	37.78	37.78	37.78	37.78	0	{}	\N	2020-10-12 15:18:35.32663
52479	JJS	2020-10-05	37.82	37.84	37.635	37.6529	1118	{}	\N	2020-10-12 15:18:35.331365
52480	JJS	2020-10-06	38.015	38.015	38.015	38.015	5	{}	\N	2020-10-12 15:18:35.335515
52481	JJS	2020-10-07	38.5236	38.5236	38.5236	38.5236	190	{}	\N	2020-10-12 15:18:35.339725
52482	JJS	2020-10-08	38.775	38.775	38.775	38.775	181	{}	\N	2020-10-12 15:18:35.344097
52483	JJS	2020-10-09	39.025	39.025	39.025	39.025	1	{}	\N	2020-10-12 15:18:35.348601
52484	JKJ	2020-09-28	157.04	157.1572	156.825	156.825	796	{}	\N	2020-10-12 15:19:07.758894
52485	JKJ	2020-09-29	157.18	157.18	154.875	154.875	1311	{}	\N	2020-10-12 15:19:07.770322
52486	JKJ	2020-09-30	157.89	157.89	155.73	155.73	1672	{}	\N	2020-10-12 15:19:07.777883
52487	JKJ	2020-10-01	157.65	158.49	157.65	158.33	2692	{}	\N	2020-10-12 15:19:07.783616
52488	JKJ	2020-10-02	157.779	160.38	157.779	160.38	1700	{}	\N	2020-10-12 15:19:07.788753
52489	JKJ	2020-10-05	161.3	163.6252	161.3	163.4	2341	{}	\N	2020-10-12 15:19:07.793848
52490	JKJ	2020-10-06	164.23	166.44	163.2815	163.2815	1490	{}	\N	2020-10-12 15:19:07.798356
52491	JKJ	2020-10-07	164.1	165.98	164.1	165.84	4109	{}	\N	2020-10-12 15:19:07.802464
52492	JKJ	2020-10-08	167.305	167.5851	166.79	167.5851	1901	{}	\N	2020-10-12 15:19:07.806331
52493	JKJ	2020-10-09	168.071	168.071	167.917	167.917	573	{}	\N	2020-10-12 15:19:07.810764
52494	JKJ	2020-10-12	168.55	168.8243	168.17	168.8243	1146	{}	\N	2020-10-12 15:19:07.814633
52495	JNJ	2020-09-28	146.73	148.29	146.42	147.11	6205946	{}	\N	2020-10-12 15:19:13.169762
52496	JNJ	2020-09-29	147.6	147.92	146.06	147.06	13190069	{}	\N	2020-10-12 15:19:13.181562
52497	JNJ	2020-09-30	147.45	149.73	146.84	148.88	16840438	{}	\N	2020-10-12 15:19:13.189288
52498	JNJ	2020-10-01	149.31	149.65	146.46	147.32	11494667	{}	\N	2020-10-12 15:19:13.196574
52499	JNJ	2020-10-02	146.21	148.41	145.83	146.24	9825301	{}	\N	2020-10-12 15:19:13.202149
52500	JNJ	2020-10-05	147	148.38	146.42	148.23	9850987	{}	\N	2020-10-12 15:19:13.206586
52501	JNJ	2020-10-06	148.28	148.61	145.87	146.26	9396212	{}	\N	2020-10-12 15:19:13.210558
52502	JNJ	2020-10-07	146.49	148.23	145.81	147.88	10756548	{}	\N	2020-10-12 15:19:13.21465
52503	JNJ	2020-10-08	148.5	149.85	148.04	148.89	8713181	{}	\N	2020-10-12 15:19:13.218521
52504	JNJ	2020-10-09	149.67	151.48	149.445	150.97	9368613	{}	\N	2020-10-12 15:19:13.222497
52505	JNJ	2020-10-12	151.6	153.142	151.02	153	4768178	{}	\N	2020-10-12 15:19:13.226196
52506	JOUT	2020-09-28	83.04	83.73	82.5	82.68	30502	{}	\N	2020-10-12 15:19:14.763675
52507	JOUT	2020-09-29	82.79	83.1259	81.51	82.1	41622	{}	\N	2020-10-12 15:19:14.77404
52508	JOUT	2020-09-30	82.21	83.25	81.52	81.89	39805	{}	\N	2020-10-12 15:19:14.780776
52509	JOUT	2020-10-01	82.57	83.2	81.48	82.785	27272	{}	\N	2020-10-12 15:19:14.785827
52510	JOUT	2020-10-02	81.5	87.34	81.06	86.18	114398	{}	\N	2020-10-12 15:19:14.790501
52511	JOUT	2020-10-05	86.82	89.95	86.82	88.91	29225	{}	\N	2020-10-12 15:19:14.795358
52512	JOUT	2020-10-06	89.87	89.94	86.7	86.7	33057	{}	\N	2020-10-12 15:19:14.799959
52513	JOUT	2020-10-07	88.47	90.93	87.79	89.83	52179	{}	\N	2020-10-12 15:19:14.805072
52514	JOUT	2020-10-08	91.24	91.43	86.86	87.58	37364	{}	\N	2020-10-12 15:19:14.809336
52515	JOUT	2020-10-09	88.29	91.6	88.29	90.8	36712	{}	\N	2020-10-12 15:19:14.81351
52516	JOUT	2020-10-12	90.45	91.58	90.425	90.425	2236	{}	\N	2020-10-12 15:19:14.817529
52517	JPN	2020-09-28	29.18	29.2095	29.15	29.1633	2909	{}	\N	2020-10-12 15:19:16.599359
52518	JPN	2020-09-29	29.31	29.31	29.14	29.1769	12975	{}	\N	2020-10-12 15:19:16.611238
52519	JPN	2020-09-30	28.9	28.99	28.9	28.91	1162	{}	\N	2020-10-12 15:19:16.617807
52520	JPN	2020-10-01	28.88	28.93	28.88	28.93	347	{}	\N	2020-10-12 15:19:16.622918
52521	JPN	2020-10-02	28.6839	28.85	28.6839	28.85	2250	{}	\N	2020-10-12 15:19:16.627495
52522	JPN	2020-10-05	28.91	29.0707	28.91	29.0707	2888	{}	\N	2020-10-12 15:19:16.631428
52523	JPN	2020-10-06	29.14	29.14	29.0296	29.0296	567	{}	\N	2020-10-12 15:19:16.63561
52524	JPN	2020-10-07	29.1	29.1755	29.09	29.1755	1693	{}	\N	2020-10-12 15:19:16.639519
52525	JPN	2020-10-08	29.18	29.18	29.07	29.07	1209	{}	\N	2020-10-12 15:19:16.642978
52526	JPN	2020-10-09	29.15	29.168	29.15	29.168	559	{}	\N	2020-10-12 15:19:16.64697
52527	JPN	2020-10-12	29.17	29.17	29.17	29.17	217	{}	\N	2020-10-12 15:19:16.651201
52528	JBSS	2020-09-28	74.46	75.37	74.46	74.84	61319	{}	\N	2020-10-12 15:19:18.150588
52529	JBSS	2020-09-29	75.33	75.39	74.56	75.13	23259	{}	\N	2020-10-12 15:19:18.162381
52530	JBSS	2020-09-30	75.58	76.22	74.9	75.38	43912	{}	\N	2020-10-12 15:19:18.169064
52531	JBSS	2020-10-01	76.47	76.47	75.01	75.55	43998	{}	\N	2020-10-12 15:19:18.174264
52532	JBSS	2020-10-02	74.92	76.43	73.78	75.79	51457	{}	\N	2020-10-12 15:19:18.179256
52533	JBSS	2020-10-05	75.98	76.82	75.91	76.48	31708	{}	\N	2020-10-12 15:19:18.183569
52534	JBSS	2020-10-06	76.98	77.78	76.25	76.67	53139	{}	\N	2020-10-12 15:19:18.188137
52535	JBSS	2020-10-07	77.04	77.04	75.83	76.13	42244	{}	\N	2020-10-12 15:19:18.191986
52536	JBSS	2020-10-08	76.55	77.68	76.1	76.65	50546	{}	\N	2020-10-12 15:19:18.195723
52537	JBSS	2020-10-09	76.75	77.43	76.23	76.89	36005	{}	\N	2020-10-12 15:19:18.199379
52538	JBSS	2020-10-12	77.01	77.62	77.01	77.55	4153	{}	\N	2020-10-12 15:19:18.203115
52539	JCS	2020-09-28	4.02	4.06	3.945	4	22594	{}	\N	2020-10-12 15:19:20.896587
52540	JCS	2020-09-29	4	4.04	3.8492	3.8492	14349	{}	\N	2020-10-12 15:19:20.907991
52541	JCS	2020-09-30	3.81	4	3.81	3.83	22047	{}	\N	2020-10-12 15:19:20.914162
52542	JCS	2020-10-01	3.99	4.0244	3.9156	3.99	22932	{}	\N	2020-10-12 15:19:20.919084
52543	JCS	2020-10-02	4.02	4.14	3.8723	4.09	23694	{}	\N	2020-10-12 15:19:20.923889
52544	JCS	2020-10-05	4.03	4.295	3.88	4.1	15713	{}	\N	2020-10-12 15:19:20.928322
52545	JCS	2020-10-06	4.02	4.18	3.96	3.96	4042	{}	\N	2020-10-12 15:19:20.932638
52546	JCS	2020-10-07	3.955	4.1596	3.9	3.98	34780	{}	\N	2020-10-12 15:19:20.936517
52547	JCS	2020-10-08	4.025	4.045	3.85	4	125904	{}	\N	2020-10-12 15:19:20.940011
52548	JCS	2020-10-09	3.99	4	3.85	3.93	10722	{}	\N	2020-10-12 15:19:20.94363
52549	JCS	2020-10-12	3.95	3.9824	3.8901	3.8901	2340	{}	\N	2020-10-12 15:19:20.947198
52550	JEMD	2020-09-28	7.5528	7.5528	7.27	7.41	67593	{}	\N	2020-10-12 15:19:22.315654
52551	JEMD	2020-09-29	7.4678	7.49	7.43	7.47	14076	{}	\N	2020-10-12 15:19:22.32311
52552	JEMD	2020-09-30	7.43	7.69	7.4201	7.69	156574	{}	\N	2020-10-12 15:19:22.329298
52553	JEMD	2020-10-01	7.71	7.74	7.54	7.74	27448	{}	\N	2020-10-12 15:19:22.334858
52554	JEMD	2020-10-02	7.6162	7.71	7.51	7.71	44904	{}	\N	2020-10-12 15:19:22.340513
52555	JEMD	2020-10-05	7.6726	7.7	7.56	7.57	37866	{}	\N	2020-10-12 15:19:22.345987
52556	JEMD	2020-10-06	7.625	7.7	7.42	7.42	69952	{}	\N	2020-10-12 15:19:22.351187
52557	JEMD	2020-10-07	7.41	7.54	7.41	7.54	134970	{}	\N	2020-10-12 15:19:22.356076
52558	JEMD	2020-10-08	7.5	7.52	7.38	7.39	95954	{}	\N	2020-10-12 15:19:22.361103
52559	JEMD	2020-10-09	7.4154	7.47	7.4013	7.47	17062	{}	\N	2020-10-12 15:19:22.365514
52560	JEMD	2020-10-12	7.46	7.5	7.4599	7.5	12242	{}	\N	2020-10-12 15:19:22.374067
52561	JHMC	2020-09-28	38.69	38.8872	38.69	38.8872	321	{}	\N	2020-10-12 15:19:24.136528
52562	JHMC	2020-09-29	38.7308	38.7308	38.5951	38.5951	238	{}	\N	2020-10-12 15:19:24.148349
52563	JHMC	2020-09-30	38.9721	39.24	38.9721	39.145	1534	{}	\N	2020-10-12 15:19:24.155709
52564	JHMC	2020-10-01	39.5949	39.5949	39.5949	39.5949	124	{}	\N	2020-10-12 15:19:24.16119
52565	JHMC	2020-10-02	39.31	39.68	39.2342	39.605	5865	{}	\N	2020-10-12 15:19:24.166231
52566	JHMC	2020-10-05	39.7701	40.0955	39.7701	40.0955	17860	{}	\N	2020-10-12 15:19:24.171017
52567	JHMC	2020-10-06	40.14	40.16	39.4827	39.4827	3559	{}	\N	2020-10-12 15:19:24.174824
52568	JHMC	2020-10-07	39.84	40.38	39.84	40.36	763	{}	\N	2020-10-12 15:19:24.178765
52569	JHMC	2020-10-08	40.48	40.6351	40.48	40.6351	190	{}	\N	2020-10-12 15:19:24.182933
52570	JHMC	2020-10-09	41.04	41.04	40.83	40.83	3604	{}	\N	2020-10-12 15:19:24.186903
52571	JHMC	2020-10-12	41.16	41.16	41.155	41.155	366	{}	\N	2020-10-12 15:19:24.19075
52572	JHS	2020-09-28	15.45	15.45	15.05	15.34	30809	{}	\N	2020-10-12 15:19:27.323573
52573	JHS	2020-09-29	15.35	15.35	14.81	15.08	37292	{}	\N	2020-10-12 15:19:27.341083
52574	JHS	2020-09-30	15.62	15.62	15	15.145	46564	{}	\N	2020-10-12 15:19:27.349308
52575	JHS	2020-10-01	15.2	15.2	15.15	15.153	44656	{}	\N	2020-10-12 15:19:27.404299
52576	JHS	2020-10-02	15.25	15.76	14.81	15.4	60923	{}	\N	2020-10-12 15:19:27.415785
52577	JHS	2020-10-05	15.38	15.38	15.08	15.1	23084	{}	\N	2020-10-12 15:19:27.423153
52578	JHS	2020-10-06	15.38	15.38	15.16	15.16	36750	{}	\N	2020-10-12 15:19:27.428421
52579	JHS	2020-10-07	15.26	15.41	15.13	15.16	31008	{}	\N	2020-10-12 15:19:27.433182
52580	JHS	2020-10-08	15.23	15.31	15.13	15.17	23688	{}	\N	2020-10-12 15:19:27.437457
52581	JHS	2020-10-09	15.22	15.3	15.11	15.2	15310	{}	\N	2020-10-12 15:19:27.441515
52582	JHS	2020-10-12	15.12	15.65	15.08	15.51	16774	{}	\N	2020-10-12 15:19:27.445516
52583	JJP	2020-09-28	65.85	66.2968	65.85	66.2968	334	{}	\N	2020-10-12 15:19:28.836556
52584	JJP	2020-09-29	67.17	67.17	67.17	67.17	24	{}	\N	2020-10-12 15:19:28.846196
52585	JJP	2020-09-30	66.71	66.71	66.3	66.3	309	{}	\N	2020-10-12 15:19:28.852934
52586	JJP	2020-10-01	67.3682	67.3682	67.3682	67.3682	433	{}	\N	2020-10-12 15:19:28.858865
52587	JJP	2020-10-02	67.3129	67.3129	67.3129	67.3129	3	{}	\N	2020-10-12 15:19:28.863778
52588	JJP	2020-10-05	68	68.08	67.8475	67.8475	274	{}	\N	2020-10-12 15:19:28.868631
52589	JJP	2020-10-06	67.92	67.92	66.4312	66.4312	572	{}	\N	2020-10-12 15:19:28.87303
52590	JJP	2020-10-07	66.6314	66.6314	66.6314	66.6314	75	{}	\N	2020-10-12 15:19:28.877607
52591	JJP	2020-10-08	66.8	66.8302	66.8	66.8302	212	{}	\N	2020-10-12 15:19:28.881945
52592	JJP	2020-10-09	68.7	68.7	68.7	68.7	129	{}	\N	2020-10-12 15:19:28.886139
52593	JJP	2020-10-12	68.21	68.56	67.951	68.56	2850	{}	\N	2020-10-12 15:19:28.890257
52594	JKH	2020-09-28	319.44	321.3523	318.4	319.93	16359	{}	\N	2020-10-12 15:19:31.789426
52595	JKH	2020-09-29	320.01	322.21	319.96	319.97	21349	{}	\N	2020-10-12 15:19:31.80037
52596	JKH	2020-09-30	319.92	323.79	319.44	320.45	35242	{}	\N	2020-10-12 15:19:31.806814
52597	JKH	2020-10-01	323.97	326.04	323.72	325.37	70421	{}	\N	2020-10-12 15:19:31.811887
52598	JKH	2020-10-02	320.02	325.29	320.02	322.8	21522	{}	\N	2020-10-12 15:19:31.81628
52599	JKH	2020-10-05	325.5	328.8	325.41	328.8	28847	{}	\N	2020-10-12 15:19:31.820302
52600	JKH	2020-10-06	328.94	333.38	326.09	327.32	30697	{}	\N	2020-10-12 15:19:31.824355
52601	JKH	2020-10-07	330.68	334.3716	330.68	333.94	10070	{}	\N	2020-10-12 15:19:31.827954
52602	JKH	2020-10-08	336.32	336.32	334.2111	335.83	15793	{}	\N	2020-10-12 15:19:31.831375
52603	JKH	2020-10-09	338.31	341.24	338.31	340.48	21454	{}	\N	2020-10-12 15:19:31.834913
52604	JKH	2020-10-12	344.16	344.7827	342.37	344.33	14350	{}	\N	2020-10-12 15:19:31.838186
52605	JLL	2020-09-28	97.37	98.87	96.61	97.47	531936	{}	\N	2020-10-12 15:19:32.868341
52606	JLL	2020-09-29	97.59	98.15	95.34	96.12	703753	{}	\N	2020-10-12 15:19:32.877571
52607	JLL	2020-09-30	97.09	98.42	94.92	95.66	1013190	{}	\N	2020-10-12 15:19:32.883893
52608	JLL	2020-10-01	96.77	98.24	95.63	96.5	805933	{}	\N	2020-10-12 15:19:32.889241
52609	JLL	2020-10-02	94	99.41	93.91	99.2	975662	{}	\N	2020-10-12 15:19:32.894298
52610	JLL	2020-10-05	100.3	101.86	99.73	101.57	631128	{}	\N	2020-10-12 15:19:32.899005
52611	JLL	2020-10-06	102.54	106.53	101.8	104.03	747701	{}	\N	2020-10-12 15:19:32.903438
52612	JLL	2020-10-07	105.49	105.82	103.57	105.45	570199	{}	\N	2020-10-12 15:19:32.907601
52613	JLL	2020-10-08	104	108.05	104	107.97	877428	{}	\N	2020-10-12 15:19:32.911598
52614	JLL	2020-10-09	109.11	109.85	106.72	107.13	637571	{}	\N	2020-10-12 15:19:32.915644
52615	JLL	2020-10-12	108.16	108.16	105.7224	107.425	62903	{}	\N	2020-10-12 15:19:32.919643
52616	JOB	2020-09-28	0.99	1.02	0.9403	0.97	90956	{}	\N	2020-10-12 15:27:00.697973
52617	JOB	2020-09-29	0.970001	0.99	0.97	0.9872	49524	{}	\N	2020-10-12 15:27:00.708168
52618	JOB	2020-09-30	0.99	1.15	0.9835	1	1024630	{}	\N	2020-10-12 15:27:00.714535
52619	JOB	2020-10-01	1.03	1.2	1	1.19	2456227	{}	\N	2020-10-12 15:27:00.721294
52620	JOB	2020-10-02	1.05	1.1	1.01	1.03	639213	{}	\N	2020-10-12 15:27:00.726673
52621	JOB	2020-10-05	1.03	1.0328	1.01	1.01	169854	{}	\N	2020-10-12 15:27:00.732195
52622	JOB	2020-10-06	1	1.02	0.9811	0.99	217596	{}	\N	2020-10-12 15:27:00.73846
52623	JOB	2020-10-07	0.9781	1.03	0.9781	1	232850	{}	\N	2020-10-12 15:27:00.743315
52624	JOB	2020-10-08	1.02	1.03	1.0011	1.01	188017	{}	\N	2020-10-12 15:27:00.748278
52625	JOB	2020-10-09	1	1.03	0.9928	1	175145	{}	\N	2020-10-12 15:27:00.755259
52626	JOB	2020-10-12	0.981	1.02	0.981	1.005	23309	{}	\N	2020-10-12 15:27:00.760498
52627	JPIN	2020-09-28	51.83	51.96	51.75	51.92	109556	{}	\N	2020-10-12 15:27:03.75909
52628	JPIN	2020-09-29	51.87	51.9679	51.64	51.73	58747	{}	\N	2020-10-12 15:27:03.76624
52629	JPIN	2020-09-30	51.58	51.915	51.54	51.61	46985	{}	\N	2020-10-12 15:27:03.771974
52630	JPIN	2020-10-01	51.88	51.96	51.7	51.96	56615	{}	\N	2020-10-12 15:27:03.777683
52631	JPIN	2020-10-02	51.3	51.93	51.3	51.8	39425	{}	\N	2020-10-12 15:27:03.782952
52632	JPIN	2020-10-05	52.36	52.55	52.26	52.5	28424	{}	\N	2020-10-12 15:27:03.790842
52633	JPIN	2020-10-06	52.46	52.55	51.88	52.02	51812	{}	\N	2020-10-12 15:27:03.796108
52634	JPIN	2020-10-07	52.33	52.46	52.25	52.36	59692	{}	\N	2020-10-12 15:27:03.801472
52635	JPIN	2020-10-08	52.62	52.74	52.58	52.71	30170	{}	\N	2020-10-12 15:27:03.806989
52636	JPIN	2020-10-09	52.81	53.0499	52.81	52.99	28273	{}	\N	2020-10-12 15:27:03.812083
52637	JPIN	2020-10-12	52.95	53.14	52.95	53.1015	9137	{}	\N	2020-10-12 15:27:03.817037
52638	JBLU	2020-09-28	11.93	12.275	11.6919	11.95	8486780	{}	\N	2020-10-12 15:27:05.487462
52639	JBLU	2020-09-29	11.92	11.92	11.28	11.4199	7648805	{}	\N	2020-10-12 15:27:05.498093
52640	JBLU	2020-09-30	11.55	11.87	11.32	11.33	8933398	{}	\N	2020-10-12 15:27:05.505792
52641	JBLU	2020-10-01	11.51	11.615	11.32	11.46	7738681	{}	\N	2020-10-12 15:27:05.51281
52642	JBLU	2020-10-02	11	11.87	10.95	11.77	10121971	{}	\N	2020-10-12 15:27:05.518122
52643	JBLU	2020-10-05	11.9	11.9733	11.68	11.76	5298161	{}	\N	2020-10-12 15:27:05.523794
52644	JBLU	2020-10-06	11.94	12.14	11.49	11.51	7311108	{}	\N	2020-10-12 15:27:05.529074
52645	JBLU	2020-10-07	12.31	12.48	12.06	12.29	12800057	{}	\N	2020-10-12 15:27:05.53427
52646	JBLU	2020-10-08	12.62	12.93	12.32	12.75	12420975	{}	\N	2020-10-12 15:27:05.540187
52647	JBLU	2020-10-09	12.72	12.86	12.51	12.74	8659446	{}	\N	2020-10-12 15:27:05.545986
52648	JBLU	2020-10-12	12.7	12.74	12.49	12.57	2620154	{}	\N	2020-10-12 15:27:05.553409
52649	JCO	2020-09-28	8.04	8.04	7.85	7.95	82732	{}	\N	2020-10-12 15:27:07.61973
52650	JCO	2020-09-29	7.93	7.95	7.88	7.95	74156	{}	\N	2020-10-12 15:27:07.630933
52651	JCO	2020-09-30	7.96	7.98	7.85	7.89	146100	{}	\N	2020-10-12 15:27:07.638038
52652	JCO	2020-10-01	7.97	7.97	7.89	7.94	116621	{}	\N	2020-10-12 15:27:07.644297
52653	JCO	2020-10-02	7.85	7.95	7.85	7.95	69278	{}	\N	2020-10-12 15:27:07.649566
52654	JCO	2020-10-05	7.95	8	7.93	7.99	168783	{}	\N	2020-10-12 15:27:07.655047
52655	JCO	2020-10-06	7.99	8.03	7.9251	8	152190	{}	\N	2020-10-12 15:27:07.660046
52656	JCO	2020-10-07	8.02	8.0599	8	8.03	149294	{}	\N	2020-10-12 15:27:07.665472
52657	JCO	2020-10-08	8.03	8.08	8.03	8.05	54788	{}	\N	2020-10-12 15:27:07.67022
52658	JCO	2020-10-09	8.03	8.066	8	8	69118	{}	\N	2020-10-12 15:27:07.675097
52659	JCO	2020-10-12	8.01	8.01	7.98	8	31402	{}	\N	2020-10-12 15:27:07.680481
52660	JFR	2020-09-28	8.26	8.32	8.26	8.3	202092	{}	\N	2020-10-12 15:27:09.131099
52661	JFR	2020-09-29	8.3	8.33	8.28	8.3	1007040	{}	\N	2020-10-12 15:27:09.141893
52662	JFR	2020-09-30	8.29	8.36	8.29	8.31	311442	{}	\N	2020-10-12 15:27:09.149829
52663	JFR	2020-10-01	8.34	8.39	8.31	8.31	528228	{}	\N	2020-10-12 15:27:09.156597
52664	JFR	2020-10-02	8.27	8.38	8.27	8.38	273456	{}	\N	2020-10-12 15:27:09.162615
52665	JFR	2020-10-05	8.4	8.45	8.39	8.43	232188	{}	\N	2020-10-12 15:27:09.1679
52666	JFR	2020-10-06	8.37	8.43	8.37	8.37	456781	{}	\N	2020-10-12 15:27:09.173175
52667	JFR	2020-10-07	8.39	8.42	8.33	8.34	419883	{}	\N	2020-10-12 15:27:09.178465
52668	JFR	2020-10-08	8.34	8.42	8.34	8.36	316296	{}	\N	2020-10-12 15:27:09.184221
52669	JFR	2020-10-09	8.38	8.41	8.35	8.35	258468	{}	\N	2020-10-12 15:27:09.189184
52670	JFR	2020-10-12	8.37	8.41	8.37	8.39	116474	{}	\N	2020-10-12 15:27:09.19452
52671	JHMA	2020-09-28	34.2393	34.2393	34.2393	34.2393	1	{}	\N	2020-10-12 15:27:10.591955
52672	JHMA	2020-09-29	34.14	34.14	33.9576	33.9576	31400	{}	\N	2020-10-12 15:27:10.597697
52673	JHMA	2020-09-30	34.3238	34.3238	34.3238	34.3238	0	{}	\N	2020-10-12 15:27:10.602979
52674	JHMA	2020-10-01	33.9409	33.9409	33.9409	33.9409	50	{}	\N	2020-10-12 15:27:10.607568
52675	JHMA	2020-10-02	34.381	34.381	34.381	34.381	5	{}	\N	2020-10-12 15:27:10.611997
52676	JHMA	2020-10-05	35.1175	35.1175	35.1175	35.1175	1	{}	\N	2020-10-12 15:27:10.617253
52677	JHMA	2020-10-06	35.3	35.3	34.7514	34.7514	188	{}	\N	2020-10-12 15:27:10.621881
52678	JHMA	2020-10-07	35.49	35.6826	35.49	35.6826	662	{}	\N	2020-10-12 15:27:10.626621
52679	JHMA	2020-10-08	36.0721	36.0721	35.98	36.0721	785	{}	\N	2020-10-12 15:27:10.63072
52680	JHMA	2020-10-09	36.5	36.5	36.2459	36.2459	303	{}	\N	2020-10-12 15:27:10.635468
52681	JHMA	2020-10-12	36.3542	36.4	36.3542	36.4	582	{}	\N	2020-10-12 15:27:10.640037
52682	JHMU	2020-09-28	29.2906	29.2906	29.1411	29.1411	174	{}	\N	2020-10-12 15:27:11.863463
52683	JHMU	2020-09-29	29.06	29.1919	29.06	29.1919	809	{}	\N	2020-10-12 15:27:11.872871
52684	JHMU	2020-09-30	29.5574	29.5574	29.5574	29.5574	48	{}	\N	2020-10-12 15:27:11.880319
52685	JHMU	2020-10-01	29.7124	29.7124	29.7124	29.7124	168	{}	\N	2020-10-12 15:27:11.886525
52686	JHMU	2020-10-02	30.1622	30.1622	30.1622	30.1622	13	{}	\N	2020-10-12 15:27:11.892033
52687	JHMU	2020-10-05	30.4733	30.4733	30.4647	30.4647	505	{}	\N	2020-10-12 15:27:11.897247
52688	JHMU	2020-10-06	30.85	30.85	30.72	30.7305	825	{}	\N	2020-10-12 15:27:11.902644
52689	JHMU	2020-10-07	30.83	30.8972	30.83	30.8972	331	{}	\N	2020-10-12 15:27:11.908107
52690	JHMU	2020-10-08	30.92	31.4521	30.92	31.4521	1984	{}	\N	2020-10-12 15:27:11.913264
52691	JHMU	2020-10-09	31.58	31.58	31.3767	31.3767	230	{}	\N	2020-10-12 15:27:11.917893
52692	JJT	2020-09-28	45.27	45.27	45.27	45.27	83	{}	\N	2020-10-12 15:27:13.421838
52693	JJT	2020-09-29	45.57	45.57	45.57	45.57	1	{}	\N	2020-10-12 15:27:13.428067
52694	JJT	2020-09-30	45.575	45.575	45.575	45.575	0	{}	\N	2020-10-12 15:27:13.43463
52695	JJT	2020-10-01	45.5	45.5	45.5	45.5	0	{}	\N	2020-10-12 15:27:13.440137
52696	JJT	2020-10-02	46.585	46.585	46.585	46.585	20	{}	\N	2020-10-12 15:27:13.446225
52697	JJT	2020-10-05	47.1	47.1	47.1	47.1	0	{}	\N	2020-10-12 15:27:13.45188
52698	JJT	2020-10-06	47.175	47.175	47.175	47.175	3	{}	\N	2020-10-12 15:27:13.45784
52699	JJT	2020-10-07	47.105	47.105	47.105	47.105	1	{}	\N	2020-10-12 15:27:13.463037
52700	JJT	2020-10-08	47.3191	47.3191	47.3191	47.3191	1	{}	\N	2020-10-12 15:27:13.467803
52701	JJT	2020-10-09	47.36	47.49	47.35	47.49	1201	{}	\N	2020-10-12 15:27:13.472623
52702	JKL	2020-09-28	105.5	107.4893	105.5	106.94	6414	{}	\N	2020-10-12 15:27:14.4171
52703	JKL	2020-09-29	107.34	107.34	104.687	105.24	4696	{}	\N	2020-10-12 15:27:14.427523
52704	JKL	2020-09-30	105.26	107.36	105.26	106.09	10194	{}	\N	2020-10-12 15:27:14.435815
52705	JKL	2020-10-01	106.42	107.1464	106.065	107.1464	7646	{}	\N	2020-10-12 15:27:14.443069
52706	JKL	2020-10-02	105.24	109.63	105.24	109.37	2321	{}	\N	2020-10-12 15:27:14.448859
52707	JKL	2020-10-05	110.46	111.9	110.46	111.82	4502	{}	\N	2020-10-12 15:27:14.454788
52708	JKL	2020-10-06	113.06	113.99	111.2	111.21	2472	{}	\N	2020-10-12 15:27:14.460619
52709	JKL	2020-10-07	112.25	113.8018	112.25	113.58	3120	{}	\N	2020-10-12 15:27:14.466248
52710	JKL	2020-10-08	114.1	115.345	114.1	115.345	4597	{}	\N	2020-10-12 15:27:14.471854
52711	JKL	2020-10-09	116.01	116.5495	114.806	114.941	4727	{}	\N	2020-10-12 15:27:14.477461
52712	JKL	2020-10-12	115.366	115.366	115.366	115.366	730	{}	\N	2020-10-12 15:27:14.48262
52713	JMM	2020-09-28	6.9245	7.07	6.8975	6.9826	8242	{}	\N	2020-10-12 15:27:16.123556
52714	JMM	2020-09-29	6.96	7.005	6.95	6.98	26912	{}	\N	2020-10-12 15:27:16.130198
52715	JMM	2020-09-30	6.9473	7.005	6.8851	6.9809	10268	{}	\N	2020-10-12 15:27:16.135975
52716	JMM	2020-10-01	7.13	7.13	6.99	7.04	6922	{}	\N	2020-10-12 15:27:16.141202
52717	JMM	2020-10-02	6.9716	7.05	6.94	7.04	12794	{}	\N	2020-10-12 15:27:16.147311
52718	JMM	2020-10-05	7.0399	7.0399	6.938	6.98	10540	{}	\N	2020-10-12 15:27:16.154115
52719	JMM	2020-10-06	7.035	7.0603	6.95	6.95	26598	{}	\N	2020-10-12 15:27:16.15934
52720	JMM	2020-10-07	7.03	7.12	6.965	7.12	34578	{}	\N	2020-10-12 15:27:16.164329
52721	JMM	2020-10-08	7.13	7.13	7.065	7.065	6364	{}	\N	2020-10-12 15:27:16.169545
52722	JMM	2020-10-09	6.9465	7.065	6.9465	7.036	3782	{}	\N	2020-10-12 15:27:16.174614
52723	JMM	2020-10-12	7.0153	7.076	7.0153	7.055	2784	{}	\N	2020-10-12 15:27:16.180766
52724	JNUG	2020-09-28	128.95	130.47	124.68	128.3	1373323	{}	\N	2020-10-12 15:27:17.160055
52725	JNUG	2020-09-29	129.09	133.697	128.89	131.96	1076780	{}	\N	2020-10-12 15:27:17.167034
52726	JNUG	2020-09-30	129.57	132.2899	126.34	129	953409	{}	\N	2020-10-12 15:27:17.174504
52727	JNUG	2020-10-01	132.9	136.33	130.11	132.96	1198021	{}	\N	2020-10-12 15:27:17.181017
52728	JNUG	2020-10-02	130	133.39	127.057	129.3	1174461	{}	\N	2020-10-12 15:27:17.187035
52729	JNUG	2020-10-05	131.27	136.46	130.5202	133.96	1065121	{}	\N	2020-10-12 15:27:17.19285
52730	JNUG	2020-10-06	137.41	138.87	124.6193	125.36	1360515	{}	\N	2020-10-12 15:27:17.198324
52731	JNUG	2020-10-07	128.83	131.57	126.3501	128.55	911361	{}	\N	2020-10-12 15:27:17.204328
52732	JNUG	2020-10-08	131.36	134.57	129.5	133.86	973834	{}	\N	2020-10-12 15:27:17.209998
52733	JNUG	2020-10-09	140	148.4499	139.89	147.75	1758520	{}	\N	2020-10-12 15:27:17.215582
52734	JNUG	2020-10-12	146.71	147.8	143.5	145.71	419381	{}	\N	2020-10-12 15:27:17.221094
52735	JPMB	2020-09-28	48.3599	48.3599	48.2557	48.2557	2428	{}	\N	2020-10-12 15:27:19.860087
52736	JPMB	2020-09-29	48.1	48.25	48.1	48.23	4773	{}	\N	2020-10-12 15:27:19.871367
52737	JPMB	2020-09-30	48.23	48.35	48.22	48.2501	38393	{}	\N	2020-10-12 15:27:19.879473
52738	JPMB	2020-10-01	48.27	48.35	48.251	48.26	1268	{}	\N	2020-10-12 15:27:19.886207
52739	JPMB	2020-10-02	48.2178	48.2884	48.2178	48.2884	299	{}	\N	2020-10-12 15:27:19.893611
52740	JPMB	2020-10-05	48.4203	48.4203	48.4203	48.4203	237	{}	\N	2020-10-12 15:27:19.899048
52741	JPMB	2020-10-06	48.5001	48.5601	48.5001	48.5241	1769	{}	\N	2020-10-12 15:27:19.9048
52742	JPMB	2020-10-07	48.73	48.77	48.6121	48.75	1556	{}	\N	2020-10-12 15:27:19.910782
52743	JPMB	2020-10-08	48.83	48.995	48.8201	48.995	3251	{}	\N	2020-10-12 15:27:19.916223
52744	JPMB	2020-10-09	49.05	49.26	49.05	49.2349	3008	{}	\N	2020-10-12 15:27:19.92182
52745	JPMB	2020-10-12	49.132	49.2447	49.132	49.2447	876	{}	\N	2020-10-12 15:27:19.927779
52746	JJA	2020-09-28	42.16	42.37	42.16	42.37	427	{}	\N	2020-10-12 15:27:22.827709
52747	JJA	2020-09-29	41.98	42.1126	41.98	42.1126	750	{}	\N	2020-10-12 15:27:22.836903
52748	JJA	2020-09-30	42.6	43.43	42.52	43.43	615	{}	\N	2020-10-12 15:27:22.843315
52749	JJA	2020-10-01	43.345	43.345	43.345	43.345	108	{}	\N	2020-10-12 15:27:22.849276
52750	JJA	2020-10-02	43.09	43.34	43.09	43.34	354	{}	\N	2020-10-12 15:27:22.854652
52751	JJA	2020-10-05	43.46	43.46	43.46	43.46	139	{}	\N	2020-10-12 15:27:22.859638
52752	JJA	2020-10-06	44.18	44.18	44.05	44.05	479	{}	\N	2020-10-12 15:27:22.864881
52753	JJA	2020-10-07	44.67	44.8	44.67	44.8	311	{}	\N	2020-10-12 15:27:22.870456
52754	JJA	2020-10-08	44.72	44.77	44.605	44.605	814	{}	\N	2020-10-12 15:27:22.875419
52755	JJA	2020-10-09	44.92	45.1201	44.92	45.1201	811	{}	\N	2020-10-12 15:27:22.88046
52756	JKD	2020-09-28	192.86	194.4498	192.86	193.81	6889	{}	\N	2020-10-12 15:27:23.852487
52757	JKD	2020-09-29	193.62	194.37	192.915	193.2	6456	{}	\N	2020-10-12 15:27:23.86548
52758	JKD	2020-09-30	194.1965	196.03	193.8612	195	8270	{}	\N	2020-10-12 15:27:23.872374
52759	JKD	2020-10-01	196.7013	196.7013	195	195.5413	6075	{}	\N	2020-10-12 15:27:23.878921
52760	JKD	2020-10-02	191.97	194.89	191.97	193.61	5106	{}	\N	2020-10-12 15:27:23.88418
52761	JKD	2020-10-05	195.39	196.97	195.3701	196.8855	36936	{}	\N	2020-10-12 15:27:23.890145
52762	JKD	2020-10-06	196.24	198.0692	194.4855	194.4855	17334	{}	\N	2020-10-12 15:27:23.896721
52763	JKD	2020-10-07	196.44	198.2875	196.44	197.945	4758	{}	\N	2020-10-12 15:27:23.901753
52764	JKD	2020-10-08	199.82	199.82	198.36	198.67	32621	{}	\N	2020-10-12 15:27:23.908191
52765	JKD	2020-10-09	199.27	200.765	199.27	200.3556	5793	{}	\N	2020-10-12 15:27:23.913914
52766	JKD	2020-10-12	202.6	203.515	202.208	203.515	1722	{}	\N	2020-10-12 15:27:23.919136
52767	JKS	2020-09-28	36.28	37.79	35.16	37.77	2762794	{}	\N	2020-10-12 15:27:25.060172
52768	JKS	2020-09-29	37.86	39.41	36.77	39.25	9171969	{}	\N	2020-10-12 15:27:25.072515
52769	JKS	2020-09-30	39.25	41.12	38.62	39.76	8174348	{}	\N	2020-10-12 15:27:25.080619
52770	JKS	2020-10-01	40.48	43.28	39.54	42.74	8312010	{}	\N	2020-10-12 15:27:25.087179
52771	JKS	2020-10-02	41.52	44.98	41.1	44.13	9804606	{}	\N	2020-10-12 15:27:25.094002
52772	JKS	2020-10-05	45.25	48.51	43.71	48.32	12074531	{}	\N	2020-10-12 15:27:25.098959
52773	JKS	2020-10-06	47.76	48.67	44.22	47.77	17322963	{}	\N	2020-10-12 15:27:25.103913
52774	JKS	2020-10-07	48.84	56.4	48.16	55.91	27000459	{}	\N	2020-10-12 15:27:25.109403
52775	JKS	2020-10-08	56	59.62	52.07	54.96	61348967	{}	\N	2020-10-12 15:27:25.114965
52776	JKS	2020-10-09	53.645	63.29	52.6096	62.84	45413735	{}	\N	2020-10-12 15:27:25.11971
52777	JKS	2020-10-12	61.14	61.84	56.83	58.1064	17847500	{}	\N	2020-10-12 15:27:25.125154
52778	JOE	2020-09-28	20.52	21.11	20.52	20.85	124174	{}	\N	2020-10-12 15:27:26.580837
52779	JOE	2020-09-29	20.83	20.895	20.42	20.6	203117	{}	\N	2020-10-12 15:27:26.591644
52780	JOE	2020-09-30	20.64	21.03	20.52	20.63	262254	{}	\N	2020-10-12 15:27:26.600038
52781	JOE	2020-10-01	20.78	20.9	20.33	20.87	227361	{}	\N	2020-10-12 15:27:26.60682
52782	JOE	2020-10-02	20.64	21.32	20.48	21.25	223337	{}	\N	2020-10-12 15:27:26.611736
52783	JOE	2020-10-05	22.63	24.49	22.5	24.34	1738472	{}	\N	2020-10-12 15:27:26.616835
52784	JOE	2020-10-06	24.56	25.44	23.75	24.91	1095345	{}	\N	2020-10-12 15:27:26.62152
52785	JOE	2020-10-07	25.13	26.25	25.1035	25.47	987718	{}	\N	2020-10-12 15:27:26.626413
52786	JOE	2020-10-08	25.81	26.72	25.71	26.12	614788	{}	\N	2020-10-12 15:27:26.631778
52787	JOE	2020-10-09	26.39	26.98	26.345	26.5	545864	{}	\N	2020-10-12 15:27:26.63651
52788	JOE	2020-10-12	26.53	27.13	26.18	26.99	185659	{}	\N	2020-10-12 15:27:26.641383
52789	JPSE	2020-09-28	28.22	28.5499	28.22	28.46	11337	{}	\N	2020-10-12 15:27:28.754511
52790	JPSE	2020-09-29	28.3469	28.48	28.1577	28.4187	12626	{}	\N	2020-10-12 15:27:28.762425
52791	JPSE	2020-09-30	28.21	28.8	28.21	28.39	6357	{}	\N	2020-10-12 15:27:28.768834
52792	JPSE	2020-10-01	28.55	28.81	28.435	28.78	11237	{}	\N	2020-10-12 15:27:28.774573
52793	JPSE	2020-10-02	28.23	29.21	28.23	29.11	14582	{}	\N	2020-10-12 15:27:28.779543
52794	JPSE	2020-10-05	29.55	29.78	29.55	29.78	34435	{}	\N	2020-10-12 15:27:28.784542
52795	JPSE	2020-10-06	30	30.4299	29.64	29.64	96250	{}	\N	2020-10-12 15:27:28.789767
52796	JPSE	2020-10-07	30.225	30.29	30.11	30.25	13481	{}	\N	2020-10-12 15:27:28.794914
52797	JPSE	2020-10-08	30.61	30.73	30.5229	30.73	8925	{}	\N	2020-10-12 15:27:28.800384
52798	JPSE	2020-10-09	30.8	30.9213	30.68	30.9	33406	{}	\N	2020-10-12 15:27:28.806915
52799	JPSE	2020-10-12	30.84	31.025	30.84	31.025	2085	{}	\N	2020-10-12 15:27:28.812162
52800	JRS	2020-09-28	7.47	7.64	7.44	7.61	109025	{}	\N	2020-10-12 15:27:29.772714
52801	JRS	2020-09-29	7.54	7.59	7.46	7.49	314966	{}	\N	2020-10-12 15:27:29.782464
52802	JRS	2020-09-30	7.54	7.594	7.4702	7.55	291754	{}	\N	2020-10-12 15:27:29.790557
52803	JRS	2020-10-01	7.66	7.69	7.4919	7.66	454017	{}	\N	2020-10-12 15:27:29.796821
52804	JRS	2020-10-02	7.6	7.81	7.57	7.78	264916	{}	\N	2020-10-12 15:27:29.801903
52805	JRS	2020-10-05	7.84	7.94	7.77	7.91	244995	{}	\N	2020-10-12 15:27:29.806437
52806	JRS	2020-10-06	7.93	8	7.86	7.89	164834	{}	\N	2020-10-12 15:27:29.810887
52807	JRS	2020-10-07	7.93	7.96	7.85	7.92	181628	{}	\N	2020-10-12 15:27:29.815466
52808	JRS	2020-10-08	7.9	8.05	7.9	8	212424	{}	\N	2020-10-12 15:27:29.821047
52809	JRS	2020-10-09	8.05	8.05	7.98	8	125753	{}	\N	2020-10-12 15:27:29.826541
52810	JRS	2020-10-12	8.09	8.09	7.96	8	53814	{}	\N	2020-10-12 15:27:29.831625
52811	JT	2020-09-28	0.55	0.58	0.5228	0.553	123171	{}	\N	2020-10-12 15:27:30.67207
52812	JT	2020-09-29	0.58	0.58	0.5128	0.53	343218	{}	\N	2020-10-12 15:27:30.681393
52813	JT	2020-09-30	0.48	0.5219	0.35	0.5086	562663	{}	\N	2020-10-12 15:27:30.688872
52814	JT	2020-10-01	0.4933	0.52	0.4156	0.49	231125	{}	\N	2020-10-12 15:27:30.694604
52815	JT	2020-10-02	0.52	0.58	0.5145	0.536	362478	{}	\N	2020-10-12 15:27:30.699891
52816	JT	2020-10-05	0.538	0.57	0.506	0.5252	249613	{}	\N	2020-10-12 15:27:30.705048
52817	JT	2020-10-06	0.52	0.57	0.51	0.51	221237	{}	\N	2020-10-12 15:27:30.710511
52818	JT	2020-10-07	0.5171	0.57	0.5103	0.54	555894	{}	\N	2020-10-12 15:27:30.715027
52819	JT	2020-10-08	0.55	0.55	0.49	0.5401	283126	{}	\N	2020-10-12 15:27:30.719892
52820	JT	2020-10-09	0.51	0.547	0.5046	0.5351	509770	{}	\N	2020-10-12 15:27:30.724746
52821	JT	2020-10-12	0.519	0.544	0.519	0.539101	59056	{}	\N	2020-10-12 15:27:30.729729
52822	JDD	2020-09-28	7.75	7.77	7.64	7.74	106206	{}	\N	2020-10-12 15:27:31.706767
52823	JDD	2020-09-29	7.79	7.79	7.66	7.74	192874	{}	\N	2020-10-12 15:27:31.718561
52824	JDD	2020-09-30	7.72	7.79	7.65	7.77	317949	{}	\N	2020-10-12 15:27:31.727814
52825	JDD	2020-10-01	7.9	7.9087	7.795	7.87	476614	{}	\N	2020-10-12 15:27:31.734842
52826	JDD	2020-10-02	7.82	7.9	7.71	7.88	144704	{}	\N	2020-10-12 15:27:31.740165
52827	JDD	2020-10-05	7.86	8.04	7.86	8.04	313761	{}	\N	2020-10-12 15:27:31.745624
52828	JDD	2020-10-06	8.07	8.11	8.03	8.04	170786	{}	\N	2020-10-12 15:27:31.75061
52829	JDD	2020-10-07	8.02	8.1132	8.005	8.04	155384	{}	\N	2020-10-12 15:27:31.756735
52830	JDD	2020-10-08	8.1	8.165	8.0301	8.14	47428	{}	\N	2020-10-12 15:27:31.7619
52831	JDD	2020-10-09	8.07	8.24	8.07	8.17	145320	{}	\N	2020-10-12 15:27:31.767117
52832	JDD	2020-10-12	8.31	8.31	7.97	8.23	221149	{}	\N	2020-10-12 15:27:31.772855
52833	JHB	2020-09-28	8.98	9.0146	8.93	8.95	108336	{}	\N	2020-10-12 15:27:32.695525
52834	JHB	2020-09-29	8.94	8.97	8.87	8.9	240850	{}	\N	2020-10-12 15:27:32.706068
52835	JHB	2020-09-30	8.94	8.9599	8.91	8.95	96892	{}	\N	2020-10-12 15:27:32.714667
52836	JHB	2020-10-01	8.96	8.99	8.94	8.99	155562	{}	\N	2020-10-12 15:27:32.720917
52837	JHB	2020-10-02	8.95	8.99	8.94	8.98	278772	{}	\N	2020-10-12 15:27:32.726938
52838	JHB	2020-10-05	8.98	8.99	8.96	8.98	148880	{}	\N	2020-10-12 15:27:32.732112
52839	JHB	2020-10-06	8.99	9.04	8.978	9.03	115592	{}	\N	2020-10-12 15:27:32.737697
52840	JHB	2020-10-07	9.03	9.05	9.01	9.05	79926	{}	\N	2020-10-12 15:27:32.74329
52841	JHB	2020-10-08	9.03	9.09	9.03	9.08	164182	{}	\N	2020-10-12 15:27:32.748698
52842	JHB	2020-10-09	9.08	9.11	9.07	9.11	156754	{}	\N	2020-10-12 15:27:32.75325
52843	JHB	2020-10-12	9.08	9.094	9.08	9.08	43172	{}	\N	2020-10-12 15:27:32.75839
52844	JHY	2020-09-28	9.466	9.466	9.44	9.45	72704	{}	\N	2020-10-12 15:27:33.825534
52845	JHY	2020-09-29	9.47	9.49	9.45	9.47	38312	{}	\N	2020-10-12 15:27:33.834957
52846	JHY	2020-09-30	9.4672	9.48	9.45	9.454	49768	{}	\N	2020-10-12 15:27:33.844468
52847	JHY	2020-10-01	9.46	9.49	9.46	9.48	37578	{}	\N	2020-10-12 15:27:33.85118
52848	JHY	2020-10-02	9.46	9.49	9.46	9.47	76112	{}	\N	2020-10-12 15:27:33.856931
52849	JHY	2020-10-05	9.471	9.49	9.46	9.47	48964	{}	\N	2020-10-12 15:27:33.86293
52850	JHY	2020-10-06	9.4845	9.5	9.48	9.4917	52474	{}	\N	2020-10-12 15:27:33.868989
52851	JHY	2020-10-07	9.48	9.5	9.47	9.48	45162	{}	\N	2020-10-12 15:27:33.873923
52852	JHY	2020-10-08	9.484	9.5	9.48	9.4938	37940	{}	\N	2020-10-12 15:27:33.87971
52853	JHY	2020-10-09	9.48	9.52	9.48	9.51	26170	{}	\N	2020-10-12 15:27:33.886623
52854	JHY	2020-10-12	9.519	9.519	9.5038	9.5038	5172	{}	\N	2020-10-12 15:27:33.890813
52855	JKE	2020-09-28	264.73	264.73	262.5	264.44	15531	{}	\N	2020-10-12 15:27:35.968368
52856	JKE	2020-09-29	264.09	265.5	264.076	264.77	10982	{}	\N	2020-10-12 15:27:35.978757
52857	JKE	2020-09-30	265.04	269	265.04	267.52	96856	{}	\N	2020-10-12 15:27:35.987316
52858	JKE	2020-10-01	270.46	270.6296	268.56	270.26	71170	{}	\N	2020-10-12 15:27:35.993919
52859	JKE	2020-10-02	264.05	268.36	263.8791	265.2703	74140	{}	\N	2020-10-12 15:27:36.000009
52860	JKE	2020-10-05	266.73	269.5873	266.54	269.5873	24666	{}	\N	2020-10-12 15:27:36.00562
52861	JKE	2020-10-06	269.76	270.4881	265.26	265.45	17653	{}	\N	2020-10-12 15:27:36.011336
52862	JKE	2020-10-07	268.08	270.3	267.7829	269.92	44875	{}	\N	2020-10-12 15:27:36.017757
52863	JKE	2020-10-08	271.79	271.99	270.87	271.3835	16039	{}	\N	2020-10-12 15:27:36.023484
52864	JKE	2020-10-09	272.68	274.9171	272.68	274.9171	17418	{}	\N	2020-10-12 15:27:36.028306
52865	JKE	2020-10-12	278.54	279	276.821	278.9	5459	{}	\N	2020-10-12 15:27:36.034862
52866	JKK	2020-09-28	234.88	236.0181	234.44	235.9	6310	{}	\N	2020-10-12 15:27:37.627934
52867	JKK	2020-09-29	235.0381	236.739	235.0381	235.83	6656	{}	\N	2020-10-12 15:27:37.636584
52868	JKK	2020-09-30	235.8	239.9	235.8	238.57	10279	{}	\N	2020-10-12 15:27:37.642769
52869	JKK	2020-10-01	241.187	242.12	240.56	241.78	4136	{}	\N	2020-10-12 15:27:37.648487
52870	JKK	2020-10-02	238.04	241.13	235.94	240.89	4943	{}	\N	2020-10-12 15:27:37.656962
52871	JKK	2020-10-05	243.07	246.9627	243.07	246.9627	9629	{}	\N	2020-10-12 15:27:37.661721
52872	JKK	2020-10-06	248.33	251.7419	247.1985	247.1985	6800	{}	\N	2020-10-12 15:27:37.66602
52873	JKK	2020-10-07	250.96	253.17	250.56	252.93	9055	{}	\N	2020-10-12 15:27:37.671798
52874	JKK	2020-10-08	253.9	254.78	253.5	254.11	10914	{}	\N	2020-10-12 15:27:37.676067
52875	JKK	2020-10-09	256.2584	258.21	256.2584	257.63	3854	{}	\N	2020-10-12 15:27:37.680232
52876	JKK	2020-10-12	260.24	260.24	257.6	259.77	2061	{}	\N	2020-10-12 15:27:37.68477
52877	JMOM	2020-09-28	34.3216	34.51	34.2675	34.47	21035	{}	\N	2020-10-12 15:27:38.699132
52878	JMOM	2020-09-29	34.45	34.6199	34.41	34.5269	2304	{}	\N	2020-10-12 15:27:38.708643
52879	JMOM	2020-09-30	34.59	34.93	34.5	34.5	9214	{}	\N	2020-10-12 15:27:38.714265
52880	JMOM	2020-10-01	35	35.08	34.9	35.005	27991	{}	\N	2020-10-12 15:27:38.721237
52881	JMOM	2020-10-02	34.35	34.9224	34.35	34.8095	40255	{}	\N	2020-10-12 15:27:38.726802
52882	JMOM	2020-10-05	35.09	35.4	35.02	35.4	6318	{}	\N	2020-10-12 15:27:38.731664
52883	JMOM	2020-10-06	35.55	35.6896	35.015	35.09	12774	{}	\N	2020-10-12 15:27:38.736499
52884	JMOM	2020-10-07	35.55	35.761	35.52	35.7251	20599	{}	\N	2020-10-12 15:27:38.741818
52885	JMOM	2020-10-08	36.07	36.07	35.7826	35.87	35392	{}	\N	2020-10-12 15:27:38.746399
52886	JMOM	2020-10-09	36.39	36.39	36.14	36.2795	48858	{}	\N	2020-10-12 15:27:38.750684
52887	JMOM	2020-10-12	36.53	36.75	36.505	36.75	6000	{}	\N	2020-10-12 15:27:38.755166
52888	JNPR	2020-09-28	21.74	22.02	21.65	21.98	3840406	{}	\N	2020-10-12 15:27:39.704688
52889	JNPR	2020-09-29	21.94	22.135	21.735	21.86	3960989	{}	\N	2020-10-12 15:27:39.712863
52890	JNPR	2020-09-30	21.73	21.77	21.29	21.5	8463786	{}	\N	2020-10-12 15:27:39.718638
52891	JNPR	2020-10-01	21.51	21.6	21.225	21.35	4941535	{}	\N	2020-10-12 15:27:39.723972
52892	JNPR	2020-10-02	21.04	21.425	21.03	21.32	5141091	{}	\N	2020-10-12 15:27:39.728567
52893	JNPR	2020-10-05	21.56	22.015	21.5128	21.96	5292245	{}	\N	2020-10-12 15:27:39.732707
52894	JNPR	2020-10-06	21.96	22.25	21.68	21.74	4826170	{}	\N	2020-10-12 15:27:39.736629
52895	JNPR	2020-10-07	21.91	22.315	21.89	22.29	4248469	{}	\N	2020-10-12 15:27:39.742094
52896	JNPR	2020-10-08	22.46	22.78	22.44	22.64	8325567	{}	\N	2020-10-12 15:27:39.746256
52897	JNPR	2020-10-09	22.84	22.93	22.52	22.66	4189793	{}	\N	2020-10-12 15:27:39.750731
52898	JNPR	2020-10-12	22.61	22.66	22.29	22.51	1073331	{}	\N	2020-10-12 15:27:39.755042
52899	JPME	2020-09-28	63.82	64.185	63.66	64.095	19856	{}	\N	2020-10-12 15:27:42.960617
52900	JPME	2020-09-29	64.09	64.19	63.39	63.7	13260	{}	\N	2020-10-12 15:27:42.97321
52901	JPME	2020-09-30	64.136	64.55	63.94	64.075	10719	{}	\N	2020-10-12 15:27:42.981943
52902	JPME	2020-10-01	64.33	64.647	64.08	64.45	13352	{}	\N	2020-10-12 15:27:42.988327
52903	JPME	2020-10-02	64.15	64.9	64.09	64.75	12874	{}	\N	2020-10-12 15:27:42.994093
52904	JPME	2020-10-05	65.13	66.08	65.13	66.08	8601	{}	\N	2020-10-12 15:27:42.999411
52905	JPME	2020-10-06	66.23	66.98	65.59	65.68	14381	{}	\N	2020-10-12 15:27:43.005108
52906	JPME	2020-10-07	66.19	66.83	66.19	66.83	34845	{}	\N	2020-10-12 15:27:43.009613
52907	JPME	2020-10-08	67.16	67.5599	67.12	67.5599	8397	{}	\N	2020-10-12 15:27:43.014488
52908	JPME	2020-10-09	67.765	68.13	67.765	67.9	33379	{}	\N	2020-10-12 15:27:43.019745
52909	JPME	2020-10-12	68.0096	68.1141	68.0096	68.1141	1144	{}	\N	2020-10-12 15:27:43.024986
52910	JPT	2020-09-28	22.59	22.7468	22.59	22.7	6805	{}	\N	2020-10-12 15:27:44.565827
52911	JPT	2020-09-29	22.74	22.83	22.6059	22.725	9524	{}	\N	2020-10-12 15:27:44.5773
52912	JPT	2020-09-30	22.64	22.93	22.6201	22.93	3992	{}	\N	2020-10-12 15:27:44.584621
52913	JPT	2020-10-01	23.16	23.16	22.745	23.03	13526	{}	\N	2020-10-12 15:27:44.591534
52914	JPT	2020-10-02	22.92	23.0866	22.7601	23.08	10592	{}	\N	2020-10-12 15:27:44.597887
52915	JPT	2020-10-05	22.88	23.09	22.88	23.0101	9072	{}	\N	2020-10-12 15:27:44.612788
52916	JPT	2020-10-06	22.9012	23.09	22.9012	23.05	11558	{}	\N	2020-10-12 15:27:44.619857
52917	JPT	2020-10-07	23.0351	23.08	23	23.08	15114	{}	\N	2020-10-12 15:27:44.625139
52918	JPT	2020-10-08	23	23.0736	23	23.07	16210	{}	\N	2020-10-12 15:27:44.631189
52919	JPT	2020-10-09	23.045	23.125	23.0001	23.12	19878	{}	\N	2020-10-12 15:27:44.636945
52920	JPT	2020-10-12	23.07	23.14	23.07	23.14	2170	{}	\N	2020-10-12 15:27:44.642481
52921	JRVR	2020-09-28	43.77	44.41	43.64	44.08	139419	{}	\N	2020-10-12 15:27:45.735573
52922	JRVR	2020-09-29	44.13	44.27	43.0505	43.96	136285	{}	\N	2020-10-12 15:27:45.747955
52923	JRVR	2020-09-30	44.21	44.66	43.68	44.53	181985	{}	\N	2020-10-12 15:27:45.757046
52924	JRVR	2020-10-01	44.8	44.8	43.88	44.41	89323	{}	\N	2020-10-12 15:27:45.763429
52925	JRVR	2020-10-02	43.81	44.8	43.61	44.65	123649	{}	\N	2020-10-12 15:27:45.76938
52926	JRVR	2020-10-05	44.98	45.98	44.98	45.38	83037	{}	\N	2020-10-12 15:27:45.774946
52927	JRVR	2020-10-06	45.72	45.72	44.52	44.83	202113	{}	\N	2020-10-12 15:27:45.779934
52928	JRVR	2020-10-07	45.32	45.66	44.87	45.37	155167	{}	\N	2020-10-12 15:27:45.784857
52929	JRVR	2020-10-08	45.91	47.29	45.91	46.97	98305	{}	\N	2020-10-12 15:27:45.789991
52930	JRVR	2020-10-09	47.47	48.13	46.795	46.98	117535	{}	\N	2020-10-12 15:27:45.795113
52931	JRVR	2020-10-12	47.02	47.67	46.68	47.6225	17638	{}	\N	2020-10-12 15:27:45.800154
52932	JCI	2020-09-28	41.08	41.955	41.05	41.44	4661112	{}	\N	2020-10-12 15:27:47.343952
52933	JCI	2020-09-29	41.41	41.46	40.73	40.96	9700637	{}	\N	2020-10-12 15:27:47.356721
52934	JCI	2020-09-30	41.16	41.5509	40.6001	40.85	9401171	{}	\N	2020-10-12 15:27:47.365896
52935	JCI	2020-10-01	41.24	41.39	40.475	40.68	9207918	{}	\N	2020-10-12 15:27:47.374007
52936	JCI	2020-10-02	40.15	42.11	40.15	41.59	11085961	{}	\N	2020-10-12 15:27:47.379999
52937	JCI	2020-10-05	41.95	42.57	41.915	42.15	7344664	{}	\N	2020-10-12 15:27:47.385048
52938	JCI	2020-10-06	42.35	42.61	41.56	41.65	6919291	{}	\N	2020-10-12 15:27:47.390834
52939	JCI	2020-10-07	42.31	42.875	42.24	42.5	6447497	{}	\N	2020-10-12 15:27:47.396096
52940	JCI	2020-10-08	42.75	43.265	42.68	43.21	8928847	{}	\N	2020-10-12 15:27:47.401593
52941	JCI	2020-10-09	43.44	43.64	42.925	42.97	7921493	{}	\N	2020-10-12 15:27:47.407458
52942	JCI	2020-10-12	43	43.505	42.93	43.42	1520064	{}	\N	2020-10-12 15:27:47.412797
52943	JHSC	2020-09-28	24.31	24.53	24.27	24.48	12748	{}	\N	2020-10-12 15:27:49.277468
52944	JHSC	2020-09-29	24.5	24.51	24.18	24.34	12369	{}	\N	2020-10-12 15:27:49.286906
52945	JHSC	2020-09-30	24.43	24.64	24.221	24.38	19084	{}	\N	2020-10-12 15:27:49.293633
52946	JHSC	2020-10-01	24.52	24.69	24.418	24.69	67174	{}	\N	2020-10-12 15:27:49.300106
52947	JHSC	2020-10-02	24.25	24.969	24.25	24.87	6247	{}	\N	2020-10-12 15:27:49.306033
52948	JHSC	2020-10-05	25.08	25.46	25.08	25.44	22853	{}	\N	2020-10-12 15:27:49.311568
52949	JHSC	2020-10-06	25.61	25.99	25.29	25.35	64422	{}	\N	2020-10-12 15:27:49.31711
52950	JHSC	2020-10-07	25.66	25.9	25.66	25.8399	159935	{}	\N	2020-10-12 15:27:49.322467
52951	JHSC	2020-10-08	26.12	26.17	25.96	26.14	77960	{}	\N	2020-10-12 15:27:49.327474
52952	JHSC	2020-10-09	26.41	26.4605	26.17	26.26	114786	{}	\N	2020-10-12 15:27:49.332534
52953	JHSC	2020-10-12	26.35	26.42	26.35	26.42	1775	{}	\N	2020-10-12 15:27:49.337588
52954	JKG	2020-09-28	199.59	201.28	199.59	200.61	4730	{}	\N	2020-10-12 15:27:51.683868
52955	JKG	2020-09-29	201.2	201.2	198.95	199.4	5105	{}	\N	2020-10-12 15:27:51.694169
52956	JKG	2020-09-30	199.43	201.9824	198.9992	200.32	7306	{}	\N	2020-10-12 15:27:51.702293
52957	JKG	2020-10-01	201.756	202.0091	201.756	202.0091	1452	{}	\N	2020-10-12 15:27:51.708076
52958	JKG	2020-10-02	201.79	204.01	201.54	203.16	4255	{}	\N	2020-10-12 15:27:51.715347
52959	JKG	2020-10-05	205.99	206.9659	205.781	206.93	3902	{}	\N	2020-10-12 15:27:51.721891
52960	JKG	2020-10-06	206.97	208.0935	204.5785	204.92	3722	{}	\N	2020-10-12 15:27:51.727332
52961	JKG	2020-10-07	207.52	208.165	207.1585	207.87	2426	{}	\N	2020-10-12 15:27:51.732443
52962	JKG	2020-10-08	209.81	210.3359	209.3069	210.3359	5377	{}	\N	2020-10-12 15:27:51.738452
52963	JKG	2020-10-09	211.39	211.78	210.8601	210.8601	1661	{}	\N	2020-10-12 15:27:51.743852
52964	JOF	2020-09-28	8.76	8.8	8.755	8.8	42531	{}	\N	2020-10-12 15:27:53.551541
52965	JOF	2020-09-29	8.83	8.8604	8.75	8.75	53476	{}	\N	2020-10-12 15:27:53.559798
52966	JOF	2020-09-30	8.66	8.6842	8.6	8.67	29678	{}	\N	2020-10-12 15:27:53.565295
52967	JOF	2020-10-01	8.63	8.68	8.55	8.58	82620	{}	\N	2020-10-12 15:27:53.571628
52968	JOF	2020-10-02	8.5	8.92	8.47	8.71	80848	{}	\N	2020-10-12 15:27:53.577059
52969	JOF	2020-10-05	8.66	8.89	8.66	8.74	66644	{}	\N	2020-10-12 15:27:53.581831
52970	JOF	2020-10-06	8.79	8.86	8.62	8.67	129523	{}	\N	2020-10-12 15:27:53.587133
52971	JOF	2020-10-07	8.7	8.72	8.63	8.65	49499	{}	\N	2020-10-12 15:27:53.591976
52972	JOF	2020-10-08	8.75	8.75	8.51	8.51	69658	{}	\N	2020-10-12 15:27:53.597107
52973	JOF	2020-10-09	8.54	8.5993	8.53	8.53	59514	{}	\N	2020-10-12 15:27:53.601833
52974	JOF	2020-10-12	8.6	8.67	8.5565	8.585	3524	{}	\N	2020-10-12 15:27:53.606783
52975	JPEM	2020-09-28	46.82	46.82	46.27	46.4	24562	{}	\N	2020-10-12 15:27:54.574784
52976	JPEM	2020-09-29	46.3	46.4475	46.1682	46.29	28229	{}	\N	2020-10-12 15:27:54.588157
52977	JPEM	2020-09-30	46.45	46.8214	46.45	46.74	14895	{}	\N	2020-10-12 15:27:54.596478
52978	JPEM	2020-10-01	46.85	47.06	46.74	47.04	41105	{}	\N	2020-10-12 15:27:54.602131
52979	JPEM	2020-10-02	46.39	46.88	46.39	46.5805	29538	{}	\N	2020-10-12 15:27:54.607638
52980	JPEM	2020-10-05	46.84	47.3	46.84	47.3	470051	{}	\N	2020-10-12 15:27:54.612042
52981	JPEM	2020-10-06	47.54	47.61	47.09	47.2	44171	{}	\N	2020-10-12 15:27:54.616531
52982	JPEM	2020-10-07	47.53	47.5605	47.3	47.43	15160	{}	\N	2020-10-12 15:27:54.621157
52983	JPEM	2020-10-08	47.57	47.85	47.54	47.79	24634	{}	\N	2020-10-12 15:27:54.625849
52984	JPEM	2020-10-09	47.99	48.34	47.99	48.23	14642	{}	\N	2020-10-12 15:27:54.630703
52985	JPEM	2020-10-12	48.32	48.32	48.23	48.3	8903	{}	\N	2020-10-12 15:27:54.635172
52986	JPM	2020-09-28	95.06	96.93	94.35	96.16	20866039	{}	\N	2020-10-12 15:27:55.847984
52987	JPM	2020-09-29	96.03	96.03	94.332	95.35	24586249	{}	\N	2020-10-12 15:27:55.859266
52988	JPM	2020-09-30	95.35	97.1975	95.26	96.27	31876037	{}	\N	2020-10-12 15:27:55.867473
52989	JPM	2020-10-01	97.12	97.77	96.03	96.97	28725131	{}	\N	2020-10-12 15:27:55.874124
52990	JPM	2020-10-02	95.25	98.59	95.09	97.89	27035518	{}	\N	2020-10-12 15:27:55.879938
52991	JPM	2020-10-05	98.32	99.25	97.53	99.04	26602982	{}	\N	2020-10-12 15:27:55.885649
52992	JPM	2020-10-06	100.07	100.86	97.5101	98.02	34052179	{}	\N	2020-10-12 15:27:55.891756
52993	JPM	2020-10-07	98.96	100.35	98.6658	99.73	29890998	{}	\N	2020-10-12 15:27:55.897979
52994	JPM	2020-10-08	100.51	101.92	99.895	101.78	30749739	{}	\N	2020-10-12 15:27:55.90339
52995	JPM	2020-10-09	102	102.105	100.11	101.2	25396128	{}	\N	2020-10-12 15:27:55.90872
52996	JPM	2020-10-12	101.1	102.02	100.85	101.97	9300458	{}	\N	2020-10-12 15:27:55.914662
52997	JPST	2020-09-28	50.775	50.79	50.77	50.77	3798027	{}	\N	2020-10-12 15:27:56.860763
52998	JPST	2020-09-29	50.79	50.81	50.78	50.79	2706406	{}	\N	2020-10-12 15:27:56.87352
52999	JPST	2020-09-30	50.8	50.8	50.78	50.795	3154919	{}	\N	2020-10-12 15:27:56.882028
53000	JPST	2020-10-01	50.755	50.76	50.74	50.75	2918952	{}	\N	2020-10-12 15:27:56.888934
53001	JPST	2020-10-02	50.75	50.76	50.74	50.75	2131146	{}	\N	2020-10-12 15:27:56.895127
53002	JPST	2020-10-05	50.76	50.76	50.75	50.76	2429416	{}	\N	2020-10-12 15:27:56.901036
53003	JPST	2020-10-06	50.76	50.77	50.74	50.76	4840412	{}	\N	2020-10-12 15:27:56.909371
53004	JPST	2020-10-07	50.75	50.77	50.75	50.77	2118889	{}	\N	2020-10-12 15:27:56.915423
53005	JPST	2020-10-08	50.76	50.78	50.76	50.77	2724512	{}	\N	2020-10-12 15:27:56.921994
53006	JPST	2020-10-09	50.77	50.78	50.77	50.775	2123994	{}	\N	2020-10-12 15:27:56.928004
53007	JPST	2020-10-12	50.78	50.78	50.77	50.775	463483	{}	\N	2020-10-12 15:27:56.933843
53008	JRO	2020-09-28	8.2	8.23	8.17	8.18	134729	{}	\N	2020-10-12 15:27:58.257577
53009	JRO	2020-09-29	8.18	8.21	8.17	8.18	227058	{}	\N	2020-10-12 15:27:58.267719
53010	JRO	2020-09-30	8.17	8.24	8.17	8.2	342098	{}	\N	2020-10-12 15:27:58.276139
53011	JRO	2020-10-01	8.22	8.29	8.18	8.19	4133987	{}	\N	2020-10-12 15:27:58.283184
53012	JRO	2020-10-02	8.12	8.26	8.12	8.24	2872272	{}	\N	2020-10-12 15:27:58.290161
53013	JRO	2020-10-05	8.25	8.32	8.25	8.28	419028	{}	\N	2020-10-12 15:27:58.296303
53014	JRO	2020-10-06	8.26	8.32	8.22	8.24	2708385	{}	\N	2020-10-12 15:27:58.302403
53015	JRO	2020-10-07	8.26	8.31	8.23	8.25	1954208	{}	\N	2020-10-12 15:27:58.308634
53016	JRO	2020-10-08	8.25	8.31	8.24	8.24	3425395	{}	\N	2020-10-12 15:27:58.31546
53017	JRO	2020-10-09	8.25	8.3	8.24	8.26	2063613	{}	\N	2020-10-12 15:27:58.321457
53018	JRO	2020-10-12	8.28	8.29	8.26	8.27	579622	{}	\N	2020-10-12 15:27:58.327732
53019	JTA	2020-09-28	7.91	8.06	7.9014	8.01	42019	{}	\N	2020-10-12 15:27:59.537653
53020	JTA	2020-09-29	8.07	8.07	7.9	7.94	68168	{}	\N	2020-10-12 15:27:59.544702
53021	JTA	2020-09-30	7.92	8.0499	7.92	8.01	116186	{}	\N	2020-10-12 15:27:59.549688
53022	JTA	2020-10-01	8.16	8.47	7.965	8.09	235078	{}	\N	2020-10-12 15:27:59.554914
53023	JTA	2020-10-02	8.08	8.08	7.8916	8.06	192038	{}	\N	2020-10-12 15:27:59.560583
53024	JTA	2020-10-05	8.05	8.13	8.05	8.07	134712	{}	\N	2020-10-12 15:27:59.565503
53025	JTA	2020-10-06	8.07	8.27	8.07	8.11	140937	{}	\N	2020-10-12 15:27:59.57055
53026	JTA	2020-10-07	8.19	8.19	8.1	8.1	156813	{}	\N	2020-10-12 15:27:59.576068
53027	JTA	2020-10-08	8.14	8.2	8.1202	8.14	59398	{}	\N	2020-10-12 15:27:59.58118
53028	JTA	2020-10-09	8.16	8.261	8.16	8.23	57422	{}	\N	2020-10-12 15:27:59.585788
53029	JTA	2020-10-12	8.25	8.303	8.22	8.295	59034	{}	\N	2020-10-12 15:27:59.590627
53030	JCOM	2020-09-28	70.08	70.65	70.08	70.34	275316	{}	\N	2020-10-12 15:28:01.153745
53031	JCOM	2020-09-29	70.28	70.97	68.63	68.99	350569	{}	\N	2020-10-12 15:28:01.164535
53032	JCOM	2020-09-30	69.33	70.065	68.79	69.22	343922	{}	\N	2020-10-12 15:28:01.171242
53033	JCOM	2020-10-01	70.04	70.99	69.065	70.18	655005	{}	\N	2020-10-12 15:28:01.177831
53034	JCOM	2020-10-02	69	70.75	68.67	70.34	383652	{}	\N	2020-10-12 15:28:01.183514
53035	JCOM	2020-10-05	71.1	71.61	70.41	70.95	402795	{}	\N	2020-10-12 15:28:01.18814
53036	JCOM	2020-10-06	70.4225	71.95	68.4732	70.08	346367	{}	\N	2020-10-12 15:28:01.193062
53037	JCOM	2020-10-07	70.74	70.99	69.55	70.06	240980	{}	\N	2020-10-12 15:28:01.197968
53038	JCOM	2020-10-08	70.8	71.67	70.601	70.87	308650	{}	\N	2020-10-12 15:28:01.203799
53039	JCOM	2020-10-09	72	72	70.975	71.24	353298	{}	\N	2020-10-12 15:28:01.208681
53040	JCOM	2020-10-12	71.01	72.77	71.01	72.675	93074	{}	\N	2020-10-12 15:28:01.21364
53041	JELD	2020-09-28	21.48	22.61	21.47	22.3	645658	{}	\N	2020-10-12 15:28:02.197175
53042	JELD	2020-09-29	22.14	22.42	21.95	22.28	504484	{}	\N	2020-10-12 15:28:02.212037
53043	JELD	2020-09-30	22.17	23.12	22.17	22.6	1085815	{}	\N	2020-10-12 15:28:02.220202
53044	JELD	2020-10-01	23.01	23.18	22.49	23.12	588790	{}	\N	2020-10-12 15:28:02.225745
53045	JELD	2020-10-02	22.26	23.645	22.26	23.42	756645	{}	\N	2020-10-12 15:28:02.23134
53046	JELD	2020-10-05	25	27.32	24.82	26.18	2536869	{}	\N	2020-10-12 15:28:02.235803
53047	JELD	2020-10-06	26.23	26.775	25.305	25.46	1457302	{}	\N	2020-10-12 15:28:02.240012
53048	JELD	2020-10-07	26.19	26.84	25.66	26.25	1228160	{}	\N	2020-10-12 15:28:02.244518
53049	JELD	2020-10-08	26.75	27.15	26.1201	26.74	1069376	{}	\N	2020-10-12 15:28:02.24903
53050	JELD	2020-10-09	26.95	27.17	25.76	25.87	1032555	{}	\N	2020-10-12 15:28:02.253907
53051	JELD	2020-10-12	26.32	26.495	25.918	26.09	120858	{}	\N	2020-10-12 15:28:02.25976
53052	JHI	2020-09-28	15.82	15.88	15.73	15.87	14672	{}	\N	2020-10-12 15:28:03.604381
53053	JHI	2020-09-29	15.82	15.88	15.62	15.83	27602	{}	\N	2020-10-12 15:28:03.615481
53054	JHI	2020-09-30	16	16.09	15.81	15.83	72030	{}	\N	2020-10-12 15:28:03.622778
53055	JHI	2020-10-01	15.85	16.26	15.83	16.19	62498	{}	\N	2020-10-12 15:28:03.629824
53056	JHI	2020-10-02	16.07	16.1	15.85	15.86	24405	{}	\N	2020-10-12 15:28:03.636587
53057	JHI	2020-10-05	15.87	15.94	15.81	15.935	49958	{}	\N	2020-10-12 15:28:03.642078
53058	JHI	2020-10-06	15.99	16.12	15.93	16.03	26230	{}	\N	2020-10-12 15:28:03.647335
53059	JHI	2020-10-07	16.15	16.21	16.03	16.16	17704	{}	\N	2020-10-12 15:28:03.652623
53060	JHI	2020-10-08	16.21	16.32	16.13	16.13	23865	{}	\N	2020-10-12 15:28:03.658592
53061	JHI	2020-10-09	16.1	16.48	16.1	16.38	43698	{}	\N	2020-10-12 15:28:03.664028
53062	JHI	2020-10-12	16.5	16.5	16.3308	16.37	9984	{}	\N	2020-10-12 15:28:03.669104
53063	JHMH	2020-09-28	39.32	39.32	39.23	39.23	1414	{}	\N	2020-10-12 15:28:05.05522
53064	JHMH	2020-09-29	39.2101	39.3812	39.2101	39.3007	1494	{}	\N	2020-10-12 15:28:05.063532
53065	JHMH	2020-09-30	39.8699	40.0863	39.6216	39.6216	1115	{}	\N	2020-10-12 15:28:05.069989
53066	JHMH	2020-10-01	40.065	40.065	39.7421	39.7421	323	{}	\N	2020-10-12 15:28:05.076221
53067	JHMH	2020-10-02	39.665	39.715	39.4722	39.4722	1037	{}	\N	2020-10-12 15:28:05.081254
53068	JHMH	2020-10-05	39.99	40.3379	39.99	40.3379	1030	{}	\N	2020-10-12 15:28:05.086635
53069	JHMH	2020-10-06	40.46	40.46	39.9624	39.9624	544	{}	\N	2020-10-12 15:28:05.092033
53070	JHMH	2020-10-07	40.4	40.7585	40.4	40.7585	692	{}	\N	2020-10-12 15:28:05.097425
53071	JHMH	2020-10-08	40.91	41.0324	40.8809	40.9682	4136	{}	\N	2020-10-12 15:28:05.102321
53072	JHMH	2020-10-09	41.2593	41.385	41.2593	41.29	1012	{}	\N	2020-10-12 15:28:05.107081
53073	JHMH	2020-10-12	41.43	41.43	41.43	41.43	205	{}	\N	2020-10-12 15:28:05.11182
53074	JHX	2020-09-28	23	23.54	23	23.52	19485	{}	\N	2020-10-12 15:28:07.735495
53075	JHX	2020-09-29	23.58	23.64	23.36	23.42	21989	{}	\N	2020-10-12 15:28:07.751371
53076	JHX	2020-09-30	23.79	24.04	23.78	23.81	22036	{}	\N	2020-10-12 15:28:07.761768
53077	JHX	2020-10-01	24.45	24.57	24.3	24.47	26895	{}	\N	2020-10-12 15:28:07.768892
53078	JHX	2020-10-02	24.01	24.62	23.98	24.62	47750	{}	\N	2020-10-12 15:28:07.77475
53079	JHX	2020-10-05	24.72	24.84	24.62	24.68	27968	{}	\N	2020-10-12 15:28:07.78013
53080	JHX	2020-10-06	24.6	24.6	24.02	24.02	42978	{}	\N	2020-10-12 15:28:07.785464
53081	JHX	2020-10-07	24.41	25	24.41	24.72	23670	{}	\N	2020-10-12 15:28:07.790574
53082	JHX	2020-10-08	25.05	25.4	25.05	25.4	18734	{}	\N	2020-10-12 15:28:07.795454
53083	JHX	2020-10-09	25.5	25.88	25.5	25.72	23946	{}	\N	2020-10-12 15:28:07.800431
53084	JHX	2020-10-12	26.1	26.1	25.56	25.7415	21382	{}	\N	2020-10-12 15:28:07.805684
53085	JJSF	2020-09-28	129.85	131.79	129.795	130.38	108066	{}	\N	2020-10-12 15:28:08.969854
53086	JJSF	2020-09-29	130.6	130.6	128.92	129.35	68824	{}	\N	2020-10-12 15:28:08.982682
53087	JJSF	2020-09-30	130.8	131.925	129.46	130.39	110147	{}	\N	2020-10-12 15:28:08.990426
53088	JJSF	2020-10-01	130.73	132.5	130.25	131.905	103527	{}	\N	2020-10-12 15:28:08.996957
53089	JJSF	2020-10-02	130.31	133.515	129.28	132.53	118581	{}	\N	2020-10-12 15:28:09.003173
53090	JJSF	2020-10-05	133.52	133.52	131.74	132.93	69126	{}	\N	2020-10-12 15:28:09.008287
53091	JJSF	2020-10-06	133.97	134.85	131.93	132.67	71311	{}	\N	2020-10-12 15:28:09.013581
53092	JJSF	2020-10-07	133.57	134.4	131.53	133.85	102846	{}	\N	2020-10-12 15:28:09.019066
53093	JJSF	2020-10-08	134.84	134.84	132.135	133.83	112793	{}	\N	2020-10-12 15:28:09.024215
53094	JJSF	2020-10-09	134.38	135.49	132.55	133.32	57812	{}	\N	2020-10-12 15:28:09.029583
53095	JJSF	2020-10-12	133.79	134.1666	133.04	134.1666	9095	{}	\N	2020-10-12 15:28:09.034924
53096	JMP	2020-09-28	2.4	2.45	2.4	2.4	3193	{}	\N	2020-10-12 15:28:10.7961
53097	JMP	2020-09-29	2.4	2.4398	2.36	2.43	14192	{}	\N	2020-10-12 15:28:10.808343
53098	JMP	2020-09-30	2.39	2.5351	2.39	2.53	45150	{}	\N	2020-10-12 15:28:10.817214
53099	JMP	2020-10-01	2.55	2.55	2.41	2.5	14844	{}	\N	2020-10-12 15:28:10.82378
53100	JMP	2020-10-02	2.3572	2.42	2.3572	2.42	3386	{}	\N	2020-10-12 15:28:10.830042
53101	JMP	2020-10-05	2.55	2.55	2.455	2.4735	14900	{}	\N	2020-10-12 15:28:10.835048
53102	JMP	2020-10-06	2.5413	2.5413	2.5	2.525	12776	{}	\N	2020-10-12 15:28:10.840219
53103	JMP	2020-10-07	2.495	2.53	2.485	2.485	5436	{}	\N	2020-10-12 15:28:10.845431
53104	JMP	2020-10-08	2.52	2.5333	2.4992	2.5175	12448	{}	\N	2020-10-12 15:28:10.851153
53105	JMP	2020-10-09	2.505	2.505	2.49	2.49	990	{}	\N	2020-10-12 15:28:10.856025
53106	JMP	2020-10-12	2.49	2.5299	2.49	2.5208	6540	{}	\N	2020-10-12 15:28:10.86115
53107	JOBS	2020-09-28	77.69	79.77	77.41	79.77	257594	{}	\N	2020-10-12 15:28:12.94952
53108	JOBS	2020-09-29	79.2	79.55	77.875	78.51	226376	{}	\N	2020-10-12 15:28:12.960753
53109	JOBS	2020-09-30	78	79.06	77.01	77.99	484677	{}	\N	2020-10-12 15:28:12.967553
53110	JOBS	2020-10-01	77.84	78.87	74.29	75.03	346852	{}	\N	2020-10-12 15:28:12.973656
53111	JOBS	2020-10-02	74.26	76.9	74.26	75.39	354739	{}	\N	2020-10-12 15:28:12.979533
53112	JOBS	2020-10-05	75.47	76.205	74.5	74.805	126048	{}	\N	2020-10-12 15:28:12.984272
53113	JOBS	2020-10-06	75.2	75.49	74.53	75.06	176059	{}	\N	2020-10-12 15:28:12.989236
53114	JOBS	2020-10-07	75.72	78.475	74.85	77.55	101667	{}	\N	2020-10-12 15:28:12.994446
53115	JOBS	2020-10-08	77	79.58	76.56	77.08	99433	{}	\N	2020-10-12 15:28:12.999321
53116	JOBS	2020-10-09	77.4	77.925	76.12	76.77	79470	{}	\N	2020-10-12 15:28:13.004087
53117	JOBS	2020-10-12	76.95	76.95	76.16	76.57	24562	{}	\N	2020-10-12 15:28:13.009031
53118	JRJC	2020-09-28	7.51	7.95	7.3701	7.87	47351	{}	\N	2020-10-12 15:28:16.087701
53119	JRJC	2020-09-29	7.83	9.185	7.58	8.37	193297	{}	\N	2020-10-12 15:28:16.097836
53120	JRJC	2020-09-30	8.37	8.37	7.89	7.89	30513	{}	\N	2020-10-12 15:28:16.10475
53121	JRJC	2020-10-01	8.13	8.2741	7.87	8.04	13841	{}	\N	2020-10-12 15:28:16.110614
53122	JRJC	2020-10-02	7.75	8.07	7.75	7.99	4025	{}	\N	2020-10-12 15:28:16.11728
53123	JRJC	2020-10-05	7.905	8.39	7.905	8.26	16901	{}	\N	2020-10-12 15:28:16.1226
53124	JRJC	2020-10-06	8.37	8.9799	8.0301	8.0301	47678	{}	\N	2020-10-12 15:28:16.127593
53125	JRJC	2020-10-07	8.22	8.88	8.2	8.72	88979	{}	\N	2020-10-12 15:28:16.136182
53126	JRJC	2020-10-08	8.7	8.7794	8.48	8.6	10633	{}	\N	2020-10-12 15:28:16.141226
53127	JRJC	2020-10-09	8.705	8.72	8.3	8.36	14306	{}	\N	2020-10-12 15:28:16.146608
53128	JRJC	2020-10-12	8.37	8.68	8.04	8.49	29559	{}	\N	2020-10-12 15:28:16.151196
53129	JVAL	2020-09-28	25.1718	25.1718	25.1718	25.1718	183	{}	\N	2020-10-12 15:28:18.052315
53130	JVAL	2020-09-29	24.9631	25.0012	24.9631	25.0012	210	{}	\N	2020-10-12 15:28:18.065475
53131	JVAL	2020-09-30	25.305	25.355	25.2242	25.2242	4237	{}	\N	2020-10-12 15:28:18.07405
53132	JVAL	2020-10-01	25.28	25.2811	25.19	25.2811	4733	{}	\N	2020-10-12 15:28:18.081251
53133	JVAL	2020-10-02	25.09	25.45	25.0444	25.45	4653	{}	\N	2020-10-12 15:28:18.088462
53134	JVAL	2020-10-05	25.7	25.84	25.6744	25.79	3826	{}	\N	2020-10-12 15:28:18.0951
53135	JVAL	2020-10-06	25.93	26.1302	25.6158	25.6158	3734	{}	\N	2020-10-12 15:28:18.101
53136	JVAL	2020-10-07	26.03	26.105	26.0092	26.0729	10337	{}	\N	2020-10-12 15:28:18.106469
53137	JVAL	2020-10-08	26.321	26.3652	26.3201	26.3652	358	{}	\N	2020-10-12 15:28:18.112245
53138	JVAL	2020-10-09	26.48	26.53	26.39	26.39	4491	{}	\N	2020-10-12 15:28:18.118588
53139	JVAL	2020-10-12	26.69	26.69	26.685	26.685	687	{}	\N	2020-10-12 15:28:18.125852
53140	JCTCF	2020-09-28	7.79	7.79	7.4	7.4	539	{}	\N	2020-10-12 15:28:19.026918
53141	JCTCF	2020-09-29	7.36	7.89	7.36	7.46	777	{}	\N	2020-10-12 15:28:19.038976
53142	JCTCF	2020-09-30	7.88	7.88	7.4	7.71	857	{}	\N	2020-10-12 15:28:19.046529
53143	JCTCF	2020-10-01	7.45	7.83	7.45	7.83	375	{}	\N	2020-10-12 15:28:19.053236
53144	JCTCF	2020-10-02	7.45	7.45	7.42	7.42	2194	{}	\N	2020-10-12 15:28:19.06001
53145	JCTCF	2020-10-05	7.46	7.46	7.23	7.41	2042	{}	\N	2020-10-12 15:28:19.065711
53146	JCTCF	2020-10-06	7.6639	7.6639	7.5379	7.5379	1647	{}	\N	2020-10-12 15:28:19.071019
53147	JCTCF	2020-10-07	7.6632	7.9	7.58	7.58	9831	{}	\N	2020-10-12 15:28:19.076094
53148	JCTCF	2020-10-08	7.8	7.8483	7.8	7.81	5731	{}	\N	2020-10-12 15:28:19.081916
53149	JCTCF	2020-10-09	7.82	8.358	7.78	8.358	26590	{}	\N	2020-10-12 15:28:19.093805
53150	JCTCF	2020-10-12	8.25	8.25	8.25	8.25	123	{}	\N	2020-10-12 15:28:19.103594
53151	JEQ	2020-09-28	8.25	8.42	8.25	8.3	23808	{}	\N	2020-10-12 15:28:20.274512
53152	JEQ	2020-09-29	8.36	8.4427	8.28	8.39	29950	{}	\N	2020-10-12 15:28:20.283814
53153	JEQ	2020-09-30	8.29	8.34	8.225	8.275	65354	{}	\N	2020-10-12 15:28:20.291475
53154	JEQ	2020-10-01	8.24	8.355	8.22	8.355	52932	{}	\N	2020-10-12 15:28:20.297999
53155	JEQ	2020-10-02	8.32	8.33	8.235	8.235	3450	{}	\N	2020-10-12 15:28:20.303501
53156	JEQ	2020-10-05	8.375	8.375	8.25	8.28	11236	{}	\N	2020-10-12 15:28:20.309047
53157	JEQ	2020-10-06	8.4496	8.47	8.21	8.33	35770	{}	\N	2020-10-12 15:28:20.314313
53158	JEQ	2020-10-08	8.39	8.5	8.2832	8.2832	40476	{}	\N	2020-10-12 15:28:20.319121
53159	JEQ	2020-10-09	8.33	8.4	8.33	8.4	10844	{}	\N	2020-10-12 15:28:20.323632
53160	JEQ	2020-10-12	8.4	8.42	8.373	8.42	6512	{}	\N	2020-10-12 15:28:20.328783
53161	JHG	2020-09-28	20.52	21.54	20.48	21.43	3039097	{}	\N	2020-10-12 15:28:22.319546
53162	JHG	2020-09-29	21.19	21.83	20.82	21.79	7599779	{}	\N	2020-10-12 15:28:22.331986
53163	JHG	2020-09-30	21.7	22.1	21.31	21.72	6410877	{}	\N	2020-10-12 15:28:22.339378
53164	JHG	2020-10-01	21.88	21.91	21.44	21.6	3906886	{}	\N	2020-10-12 15:28:22.345693
53165	JHG	2020-10-02	23.95	25.94	23.925	25.05	22028273	{}	\N	2020-10-12 15:28:22.352122
53166	JHG	2020-10-05	25	25.6	24.71	25.4	3671020	{}	\N	2020-10-12 15:28:22.357088
53167	JHG	2020-10-06	25.2	25.215	24.45	24.65	4867328	{}	\N	2020-10-12 15:28:22.362062
53168	JHG	2020-10-07	24.86	25.27	24.77	25.04	3356561	{}	\N	2020-10-12 15:28:22.366924
53169	JHG	2020-10-08	25.77	27.09	25.7499	26.93	5114534	{}	\N	2020-10-12 15:28:22.371722
53170	JHG	2020-10-09	27.34	27.8	26.97	27.57	3958879	{}	\N	2020-10-12 15:28:22.376336
53171	JHG	2020-10-12	27.55	28.02	27.35	28	903385	{}	\N	2020-10-12 15:28:22.380941
53172	JJM	2020-09-28	44.4812	44.4812	44.4812	44.4812	80	{}	\N	2020-10-12 15:28:24.139413
53173	JJM	2020-09-29	43.89	44.39	43.89	44.39	227	{}	\N	2020-10-12 15:28:24.153216
53174	JJM	2020-09-30	44.7	44.7	44.3415	44.3415	251	{}	\N	2020-10-12 15:28:24.161866
53175	JJM	2020-10-01	42.82	42.8511	42.82	42.8511	425	{}	\N	2020-10-12 15:28:24.167406
53176	JJM	2020-10-02	43.55	43.55	43.55	43.55	161	{}	\N	2020-10-12 15:28:24.17339
53177	JJM	2020-10-05	43.37	43.8	43.37	43.8	198	{}	\N	2020-10-12 15:28:24.178395
53178	JJM	2020-10-06	43.63	43.665	43.63	43.665	241	{}	\N	2020-10-12 15:28:24.183077
53179	JJM	2020-10-07	44.5	44.5	44.4999	44.4999	213	{}	\N	2020-10-12 15:28:24.18785
53180	JJM	2020-10-08	44.695	44.695	44.695	44.695	79	{}	\N	2020-10-12 15:28:24.192808
53181	JJM	2020-10-09	45.61	45.61	45.61	45.61	49	{}	\N	2020-10-12 15:28:24.19765
53182	JJM	2020-10-12	45.44	45.44	45.44	45.44	254	{}	\N	2020-10-12 15:28:24.202258
53183	JKHY	2020-09-28	162.83	165.38	161.82	162.72	426329	{}	\N	2020-10-12 15:28:25.177005
53184	JKHY	2020-09-29	162.75	164.54	162.13	162.13	339146	{}	\N	2020-10-12 15:28:25.188827
53185	JKHY	2020-09-30	161.38	164.09	161.38	162.59	342167	{}	\N	2020-10-12 15:28:25.197407
53186	JKHY	2020-10-01	163.71	165.585	162.5	163.54	442919	{}	\N	2020-10-12 15:28:25.204585
53187	JKHY	2020-10-02	161.6	163.085	160.7961	161.56	328097	{}	\N	2020-10-12 15:28:25.210952
53188	JKHY	2020-10-05	163.24	163.49	161.28	162.09	372784	{}	\N	2020-10-12 15:28:25.216895
53189	JKHY	2020-10-06	163	165.2796	161.84	162.06	261703	{}	\N	2020-10-12 15:28:25.222606
53190	JKHY	2020-10-07	163	165	162.575	164.44	328086	{}	\N	2020-10-12 15:28:25.228614
53191	JKHY	2020-10-08	165.4	166.4899	163.13	164.32	296392	{}	\N	2020-10-12 15:28:25.233691
53192	JKHY	2020-10-09	164.62	165.775	164.08	164.49	502347	{}	\N	2020-10-12 15:28:25.238285
53193	JKHY	2020-10-12	165.97	165.97	163.57	165.36	119433	{}	\N	2020-10-12 15:28:25.243411
53194	JMIN	2020-09-28	29.11	29.2699	29.11	29.2206	1144	{}	\N	2020-10-12 15:28:26.267704
53195	JMIN	2020-09-29	29.2499	29.2499	29	29.11	3350	{}	\N	2020-10-12 15:28:26.279848
53196	JMIN	2020-09-30	29.4	29.47	29.2171	29.3054	990	{}	\N	2020-10-12 15:28:26.287608
53197	JMIN	2020-10-01	29.3699	29.3699	29.17	29.235	2636	{}	\N	2020-10-12 15:28:26.294518
53198	JMIN	2020-10-02	29.3681	29.3681	29.3681	29.3681	110	{}	\N	2020-10-12 15:28:26.300163
53199	JMIN	2020-10-05	29.644	29.7683	29.644	29.7683	1808	{}	\N	2020-10-12 15:28:26.305912
53200	JMIN	2020-10-06	29.78	29.96	29.6008	29.64	1965	{}	\N	2020-10-12 15:28:26.311047
53201	JMIN	2020-10-07	29.89	30.0084	29.84	30.0084	4751	{}	\N	2020-10-12 15:28:26.315994
53202	JMIN	2020-10-08	30.222	30.282	30.221	30.2689	2258	{}	\N	2020-10-12 15:28:26.321057
53203	JMIN	2020-10-09	30.371	30.5	30.33	30.455	9024	{}	\N	2020-10-12 15:28:26.326219
53204	JMIN	2020-10-12	30.551	30.6746	30.551	30.6746	717	{}	\N	2020-10-12 15:28:26.331492
53205	JNK	2020-09-28	103.58	103.96	103.44	103.81	5807548	{}	\N	2020-10-12 15:28:27.870177
53206	JNK	2020-09-29	103.84	103.97	103.54	103.89	4748411	{}	\N	2020-10-12 15:28:27.881154
53207	JNK	2020-09-30	103.87	104.485	103.86	104.27	7971631	{}	\N	2020-10-12 15:28:27.887066
53208	JNK	2020-10-01	104.12	104.285	104	104.15	11605053	{}	\N	2020-10-12 15:28:27.892426
53209	JNK	2020-10-02	103.71	104.205	103.58	104.1	8125731	{}	\N	2020-10-12 15:28:27.89835
53210	JNK	2020-10-05	104.29	104.87	104.25	104.81	12580411	{}	\N	2020-10-12 15:28:27.902863
53211	JNK	2020-10-06	104.92	105.3	104.57	104.58	13208711	{}	\N	2020-10-12 15:28:27.907268
53212	JNK	2020-10-07	104.95	105.08	104.89	104.99	5041301	{}	\N	2020-10-12 15:28:27.911126
53213	JNK	2020-10-08	105.34	105.34	105.11	105.32	4393444	{}	\N	2020-10-12 15:28:27.915267
53214	JNK	2020-10-09	105.37	105.59	105.275	105.48	8156435	{}	\N	2020-10-12 15:28:27.919153
53215	JNK	2020-10-12	105.6	106.03	105.6	105.99	1553139	{}	\N	2020-10-12 15:28:27.923958
53216	JPHY	2020-09-28	49.75	49.8199	49.6855	49.78	16452	{}	\N	2020-10-12 15:28:29.715714
53217	JPHY	2020-09-29	49.86	49.95	49.695	49.85	25244	{}	\N	2020-10-12 15:28:29.724214
53218	JPHY	2020-09-30	49.72	50.25	49.72	50.0287	26896	{}	\N	2020-10-12 15:28:29.731861
53219	JPHY	2020-10-01	49.82	50.17	49.82	50.03	179593	{}	\N	2020-10-12 15:28:29.737401
53220	JPHY	2020-10-02	49.88	50.01	49.8201	49.95	9424	{}	\N	2020-10-12 15:28:29.743228
53221	JPHY	2020-10-05	50.09	50.45	49.9901	50.26	21209	{}	\N	2020-10-12 15:28:29.748201
53222	JPHY	2020-10-06	50.34	50.46	50.16	50.16	21596	{}	\N	2020-10-12 15:28:29.753156
53223	JPHY	2020-10-07	50.4	50.515	50.2665	50.385	12356	{}	\N	2020-10-12 15:28:29.759226
53224	JPHY	2020-10-08	50.53	50.63	50.4238	50.48	18673	{}	\N	2020-10-12 15:28:29.764136
53225	JPHY	2020-10-09	50.498	50.81	50.498	50.62	13211	{}	\N	2020-10-12 15:28:29.769107
53226	JPHY	2020-10-12	50.77	50.78	50.6101	50.7201	7162	{}	\N	2020-10-12 15:28:29.774317
53227	JQC	2020-09-28	5.92	5.93	5.885	5.9	301786	{}	\N	2020-10-12 15:28:31.153832
53228	JQC	2020-09-29	5.91	5.925	5.8901	5.91	485325	{}	\N	2020-10-12 15:28:31.167153
53229	JQC	2020-09-30	5.93	5.94	5.893	5.91	1959310	{}	\N	2020-10-12 15:28:31.17503
53230	JQC	2020-10-01	5.97	5.99	5.95	5.98	1107135	{}	\N	2020-10-12 15:28:31.181647
53231	JQC	2020-10-02	5.93	6.03	5.92	6.03	1367451	{}	\N	2020-10-12 15:28:31.1875
53232	JQC	2020-10-05	6.05	6.09	6.03	6.09	740128	{}	\N	2020-10-12 15:28:31.192658
53233	JQC	2020-10-06	6.08	6.1	6.04	6.04	1420344	{}	\N	2020-10-12 15:28:31.197332
53234	JQC	2020-10-07	6.07	6.095	6.03	6.06	1226147	{}	\N	2020-10-12 15:28:31.20226
53235	JQC	2020-10-08	6.06	6.14	6.0427	6.1	1696737	{}	\N	2020-10-12 15:28:31.207313
53236	JQC	2020-10-09	6.12	6.185	6.105	6.17	1554521	{}	\N	2020-10-12 15:28:31.21234
53237	JQC	2020-10-12	6.19	6.19	6.15	6.16	384362	{}	\N	2020-10-12 15:28:31.217852
53238	JVA	2020-09-28	3.3412	3.44	3.3242	3.44	5541	{}	\N	2020-10-12 15:28:32.168466
53239	JVA	2020-09-29	3.4	3.75	3.4	3.56	60997	{}	\N	2020-10-12 15:28:32.180318
53240	JVA	2020-09-30	3.6101	3.65	3.4	3.48	44540	{}	\N	2020-10-12 15:28:32.188422
53241	JVA	2020-10-01	3.54	3.6599	3.4601	3.6	13632	{}	\N	2020-10-12 15:28:32.196156
53242	JVA	2020-10-02	3.46	3.57	3.4	3.41	30777	{}	\N	2020-10-12 15:28:32.203132
53243	JVA	2020-10-05	3.525	3.65	3.4	3.41	67143	{}	\N	2020-10-12 15:28:32.208329
53244	JVA	2020-10-06	3.44	3.5211	3.44	3.47	11605	{}	\N	2020-10-12 15:28:32.213793
53245	JVA	2020-10-07	3.59	3.65	3.46	3.46	29008	{}	\N	2020-10-12 15:28:32.218859
53246	JVA	2020-10-08	3.5	3.64	3.5	3.6	5376	{}	\N	2020-10-12 15:28:32.224034
53247	JVA	2020-10-09	3.6	3.6	3.42	3.505	19849	{}	\N	2020-10-12 15:28:32.228885
53248	JVA	2020-10-12	3.51	3.54	3.4483	3.4483	3445	{}	\N	2020-10-12 15:28:32.234728
53249	JDIV	2020-09-28	23.17	23.3094	23.17	23.3064	2015	{}	\N	2020-10-12 15:28:33.334905
53250	JDIV	2020-09-29	23.0488	23.0488	23.0488	23.0488	33	{}	\N	2020-10-12 15:28:33.344028
53251	JDIV	2020-09-30	23.31	23.34	23.14	23.2267	17745	{}	\N	2020-10-12 15:28:33.350611
53252	JDIV	2020-10-01	23.161	23.1869	23.161	23.1869	1463	{}	\N	2020-10-12 15:28:33.356242
53253	JDIV	2020-10-02	23.22	23.5	23.0275	23.5	11841	{}	\N	2020-10-12 15:28:33.362124
53254	JDIV	2020-10-05	23.8285	23.8518	23.8285	23.8518	371	{}	\N	2020-10-12 15:28:33.366975
53255	JDIV	2020-10-06	24.04	24.04	23.73	23.73	442	{}	\N	2020-10-12 15:28:33.371889
53256	JDIV	2020-10-07	24.01	24.1115	23.9643	24.1115	2439	{}	\N	2020-10-12 15:28:33.377322
53257	JDIV	2020-10-08	24.27	24.4775	24.2601	24.4775	1825	{}	\N	2020-10-12 15:28:33.382461
53258	JDIV	2020-10-09	24.49	24.59	24.44	24.55	6180	{}	\N	2020-10-12 15:28:33.387548
53259	JGH	2020-09-28	13.51	13.63	13.51	13.57	80428	{}	\N	2020-10-12 15:28:34.643679
53260	JGH	2020-09-29	13.61	13.61	13.47	13.52	168682	{}	\N	2020-10-12 15:28:34.660462
53261	JGH	2020-09-30	13.5	13.58	13.49	13.53	337954	{}	\N	2020-10-12 15:28:34.666152
53262	JGH	2020-10-01	13.62	13.64	13.55	13.57	79710	{}	\N	2020-10-12 15:28:34.670583
53263	JGH	2020-10-02	13.4	13.61	13.4	13.59	119948	{}	\N	2020-10-12 15:28:34.675172
53264	JGH	2020-10-05	13.61	13.73	13.61	13.73	136300	{}	\N	2020-10-12 15:28:34.6798
53265	JGH	2020-10-06	13.73	13.82	13.73	13.77	65230	{}	\N	2020-10-12 15:28:34.684365
53266	JGH	2020-10-07	13.81	13.875	13.81	13.84	109488	{}	\N	2020-10-12 15:28:34.689163
53267	JGH	2020-10-08	13.9	13.96	13.89	13.9	46904	{}	\N	2020-10-12 15:28:34.69363
53268	JGH	2020-10-09	13.96	13.98	13.91	13.91	74306	{}	\N	2020-10-12 15:28:34.698252
53269	JGH	2020-10-12	13.92	14	13.92	13.995	14340	{}	\N	2020-10-12 15:28:34.702718
53270	JHMD	2020-09-28	27.31	27.346	27.2614	27.325	13686	{}	\N	2020-10-12 15:28:36.058441
53271	JHMD	2020-09-29	27.37	27.3798	27.25	27.3598	6630	{}	\N	2020-10-12 15:28:36.064312
53272	JHMD	2020-09-30	27.25	27.35	27.14	27.16	17097	{}	\N	2020-10-12 15:28:36.073827
53273	JHMD	2020-10-01	27.3	27.36	27.1901	27.3	31285	{}	\N	2020-10-12 15:28:36.081407
53274	JHMD	2020-10-02	27.02	27.35	27.02	27.318	2455	{}	\N	2020-10-12 15:28:36.086396
53275	JHMD	2020-10-05	27.6193	27.6588	27.6193	27.65	3256	{}	\N	2020-10-12 15:28:36.09072
53276	JHMD	2020-10-06	27.7567	27.7567	27.4391	27.45	7461	{}	\N	2020-10-12 15:28:36.095915
53277	JHMD	2020-10-07	27.6926	27.78	27.655	27.7551	13911	{}	\N	2020-10-12 15:28:36.100265
53278	JHMD	2020-10-08	27.83	27.87	27.7772	27.84	6181	{}	\N	2020-10-12 15:28:36.104633
53279	JHMD	2020-10-09	27.976	28.0696	27.976	28.01	15258	{}	\N	2020-10-12 15:28:36.109062
53280	JHMS	2020-09-28	30.4649	30.4649	30.4576	30.4576	211	{}	\N	2020-10-12 15:28:37.472842
53281	JHMS	2020-09-29	30.355	30.355	30.3187	30.3187	292	{}	\N	2020-10-12 15:28:37.481163
53282	JHMS	2020-09-30	30.4967	30.7	30.4967	30.68	1501	{}	\N	2020-10-12 15:28:37.487749
53283	JHMS	2020-10-01	30.69	30.7698	30.6166	30.7698	1332	{}	\N	2020-10-12 15:28:37.493458
53284	JHMS	2020-10-02	30.75	30.79	30.7334	30.7334	653	{}	\N	2020-10-12 15:28:37.498995
53285	JHMS	2020-10-05	30.89	30.965	30.87	30.962	1187	{}	\N	2020-10-12 15:28:37.504107
53286	JHMS	2020-10-06	31.0462	31.1222	30.8267	30.84	3337	{}	\N	2020-10-12 15:28:37.509778
53287	JHMS	2020-10-07	31.0201	31.1779	31.02	31.1779	884	{}	\N	2020-10-12 15:28:37.514864
53288	JHMS	2020-10-08	31.25	31.369	31.25	31.369	823	{}	\N	2020-10-12 15:28:37.519993
53289	JHMS	2020-10-09	31.66	31.66	31.6311	31.6311	367	{}	\N	2020-10-12 15:28:37.525138
53290	JJE	2020-09-28	27.1713	27.1713	27.1713	27.1713	0	{}	\N	2020-10-12 15:28:40.290537
53291	JJE	2020-09-29	25.5432	25.5432	25.5432	25.5432	1	{}	\N	2020-10-12 15:28:40.298288
53292	JJE	2020-09-30	25.9599	25.9599	25.9599	25.9599	0	{}	\N	2020-10-12 15:28:40.304081
53293	JJE	2020-10-01	25.33	25.33	25.33	25.33	0	{}	\N	2020-10-12 15:28:40.309535
53294	JJE	2020-10-02	24.656	24.656	24.656	24.656	0	{}	\N	2020-10-12 15:28:40.314654
53295	JJE	2020-10-05	26.1998	26.1998	26.1998	26.1998	0	{}	\N	2020-10-12 15:28:40.31925
53296	JJE	2020-10-06	26.045	26.045	26.045	26.045	0	{}	\N	2020-10-12 15:28:40.324114
53297	JJE	2020-10-07	26.325	26.325	26.325	26.325	0	{}	\N	2020-10-12 15:28:40.328755
53298	JJE	2020-10-08	26.895	26.895	26.895	26.895	2	{}	\N	2020-10-12 15:28:40.333491
53299	JJE	2020-10-09	26.985	26.985	26.985	26.985	1	{}	\N	2020-10-12 15:28:40.338312
53300	JKF	2020-09-28	98.61	99.345	98.44	98.9	5296	{}	\N	2020-10-12 15:28:41.293514
53301	JKF	2020-09-29	99.04	99.04	97.61	98.1797	25144	{}	\N	2020-10-12 15:28:41.304595
53302	JKF	2020-09-30	98.72	99.57	98.36	98.85	13947	{}	\N	2020-10-12 15:28:41.312074
53303	JKF	2020-10-01	99.22	99.22	98.4	98.82	10300	{}	\N	2020-10-12 15:28:41.318907
53304	JKF	2020-10-02	97.09	99.21	97.09	99.15	11290	{}	\N	2020-10-12 15:28:41.324798
53305	JKF	2020-10-05	99.99	100.2501	99.9	100.1923	5133	{}	\N	2020-10-12 15:28:41.329697
53306	JKF	2020-10-06	100.44	101.18	99.22	99.4765	7685	{}	\N	2020-10-12 15:28:41.334589
53307	JKF	2020-10-07	100.5	101.0276	100.32	101.0276	9996	{}	\N	2020-10-12 15:28:41.339489
53308	JKF	2020-10-08	101.62	102.1603	101.3	102.1603	13507	{}	\N	2020-10-12 15:28:41.344253
53309	JKF	2020-10-09	102.74	102.74	102.1444	102.155	4909	{}	\N	2020-10-12 15:28:41.349135
53310	JKF	2020-10-12	102.85	102.88	102.85	102.88	1911	{}	\N	2020-10-12 15:28:41.354061
53311	JNCE	2020-09-28	8.21	8.5	7.94	8.16	418889	{}	\N	2020-10-12 15:28:43.570807
53312	JNCE	2020-09-29	8.2087	8.42	7.97	8.22	408294	{}	\N	2020-10-12 15:28:43.580615
53313	JNCE	2020-09-30	8.24	8.4225	8.01	8.16	396258	{}	\N	2020-10-12 15:28:43.588241
53314	JNCE	2020-10-01	8.39	9.06	8.32	9.02	659497	{}	\N	2020-10-12 15:28:43.595278
53315	JNCE	2020-10-02	8.61	9	8.5	8.73	256499	{}	\N	2020-10-12 15:28:43.600646
53316	JNCE	2020-10-05	8.81	9.33	8.81	9.21	403843	{}	\N	2020-10-12 15:28:43.606639
53317	JNCE	2020-10-06	9.39	9.59	9.08	9.25	362244	{}	\N	2020-10-12 15:28:43.611757
53318	JNCE	2020-10-07	9.34	9.57	9.14	9.43	551325	{}	\N	2020-10-12 15:28:43.616948
53319	JNCE	2020-10-08	9.53	9.71	9.2	9.67	282248	{}	\N	2020-10-12 15:28:43.622552
53320	JNCE	2020-10-09	9.84	10.03	9.5101	9.61	272124	{}	\N	2020-10-12 15:28:43.628082
53321	JNCE	2020-10-12	9.5001	9.84	9.26	9.69	122324	{}	\N	2020-10-12 15:28:43.633122
53322	JPC	2020-09-28	8.5	8.55	8.449	8.47	342696	{}	\N	2020-10-12 15:28:44.618112
53323	JPC	2020-09-29	8.49	8.5	8.42	8.43	505142	{}	\N	2020-10-12 15:28:44.631639
53324	JPC	2020-09-30	8.43	8.5	8.34	8.49	1082726	{}	\N	2020-10-12 15:28:44.639198
53325	JPC	2020-10-01	8.52	8.5315	8.45	8.48	774209	{}	\N	2020-10-12 15:28:44.64502
53326	JPC	2020-10-02	8.39	8.48	8.39	8.45	617894	{}	\N	2020-10-12 15:28:44.651422
53327	JPC	2020-10-05	8.46	8.56	8.45	8.52	503694	{}	\N	2020-10-12 15:28:44.657213
53328	JPC	2020-10-06	8.55	8.59	8.5333	8.56	444257	{}	\N	2020-10-12 15:28:44.663297
53329	JPC	2020-10-07	8.58	8.63	8.5764	8.61	289026	{}	\N	2020-10-12 15:28:44.671052
53330	JPC	2020-10-08	8.62	8.66	8.605	8.65	396380	{}	\N	2020-10-12 15:28:44.676602
53331	JPC	2020-10-09	8.66	8.68	8.64	8.64	307958	{}	\N	2020-10-12 15:28:44.682042
53332	JPC	2020-10-12	8.67	8.67	8.6	8.6699	182108	{}	\N	2020-10-12 15:28:44.687404
53333	JSML	2020-09-28	46.25	47.27	46.25	47.14	9360	{}	\N	2020-10-12 15:28:46.47468
53334	JSML	2020-09-29	47.3423	47.43	46.9184	47.2455	6494	{}	\N	2020-10-12 15:28:46.482333
53335	JSML	2020-09-30	47.67	47.94	47.09	47.2133	4165	{}	\N	2020-10-12 15:28:46.489107
53336	JSML	2020-10-01	47.74	48.21	47.4401	48.106	20787	{}	\N	2020-10-12 15:28:46.495236
53337	JSML	2020-10-02	47.6666	48.3372	47.6666	48.3372	1090	{}	\N	2020-10-12 15:28:46.501108
53338	JSML	2020-10-05	48.8716	49.3889	48.8716	49.3889	1295	{}	\N	2020-10-12 15:28:46.506928
53339	JSML	2020-10-06	49.75	50.42	49.4563	49.4563	8627	{}	\N	2020-10-12 15:28:46.511455
53340	JSML	2020-10-07	49.59	50.1301	49.59	50.1301	5571	{}	\N	2020-10-12 15:28:46.517458
53341	JSML	2020-10-08	50.6424	50.7	50.47	50.65	2489	{}	\N	2020-10-12 15:28:46.522466
53342	JSML	2020-10-09	50.9743	51.2239	50.66	51.2239	3687	{}	\N	2020-10-12 15:28:46.527918
53343	JSML	2020-10-12	51.81	51.81	51.49	51.49	239	{}	\N	2020-10-12 15:28:46.533495
53344	JRI	2020-09-28	12.04	12.17	12.02	12.17	98277	{}	\N	2020-10-12 15:28:47.571596
53345	JRI	2020-09-29	12.21	12.21	12	12.16	183115	{}	\N	2020-10-12 15:28:47.584182
53346	JRI	2020-09-30	12.1	12.13	12.02	12.06	189519	{}	\N	2020-10-12 15:28:47.594338
53347	JRI	2020-10-01	12.17	12.18	12.04	12.18	166192	{}	\N	2020-10-12 15:28:47.600319
53348	JRI	2020-10-02	12.07	12.25	12.0201	12.24	106624	{}	\N	2020-10-12 15:28:47.607202
53349	JRI	2020-10-05	12.17	12.37	12.17	12.35	170586	{}	\N	2020-10-12 15:28:47.61333
53350	JRI	2020-10-06	12.33	12.3699	12.28	12.31	162381	{}	\N	2020-10-12 15:28:47.618805
53351	JRI	2020-10-07	12.32	12.36	12.3	12.32	118056	{}	\N	2020-10-12 15:28:47.62491
53352	JRI	2020-10-08	12.33	12.48	12.3248	12.44	147438	{}	\N	2020-10-12 15:28:47.630106
53353	JRI	2020-10-09	12.46	12.48	12.44	12.46	125456	{}	\N	2020-10-12 15:28:47.635399
53354	JRI	2020-10-12	12.44	12.46	12.39	12.46	61106	{}	\N	2020-10-12 15:28:47.64156
53355	JXI	2020-09-28	55.66	55.875	55.6163	55.6163	2402	{}	\N	2020-10-12 15:28:48.776944
53356	JXI	2020-09-29	56.029	56.038	55.8544	55.8544	3452	{}	\N	2020-10-12 15:28:48.789677
53357	JXI	2020-09-30	55.97	56.275	55.78	56.13	60512	{}	\N	2020-10-12 15:28:48.79743
53358	JXI	2020-10-01	56.33	56.84	56.3	56.72	13159	{}	\N	2020-10-12 15:28:48.803292
53359	JXI	2020-10-02	56.33	57.49	56.33	57.35	6241	{}	\N	2020-10-12 15:28:48.80825
53360	JXI	2020-10-05	57.29	57.809	57.22	57.809	5987	{}	\N	2020-10-12 15:28:48.812876
53361	JXI	2020-10-06	57.87	58.52	57.82	58.02	11507	{}	\N	2020-10-12 15:28:48.81736
53362	JXI	2020-10-07	58.42	58.71	58.23	58.7	5371	{}	\N	2020-10-12 15:28:48.821537
53363	JXI	2020-10-08	59.04	59.315	58.955	59.26	4945	{}	\N	2020-10-12 15:28:48.825431
53364	JXI	2020-10-09	59.44	59.44	59.0768	59.17	5808	{}	\N	2020-10-12 15:28:48.82941
53365	JXI	2020-10-12	59.42	59.865	59.42	59.865	3027	{}	\N	2020-10-12 15:28:48.833078
53366	KAR	2020-09-28	14.31	14.52	14.27	14.35	1580561	{}	\N	2020-10-12 15:28:49.898559
53367	KAR	2020-09-29	14.38	14.66	14.205	14.33	1822039	{}	\N	2020-10-12 15:28:49.907636
53368	KAR	2020-09-30	14.33	14.57	14.255	14.4	4469324	{}	\N	2020-10-12 15:28:49.914614
53369	KAR	2020-10-01	14.99	15.35	14.72	15.13	4276461	{}	\N	2020-10-12 15:28:49.922609
53370	KAR	2020-10-02	14.81	15.56	14.72	15.41	2747733	{}	\N	2020-10-12 15:28:49.929534
53371	KAR	2020-10-05	15.56	15.81	15.5201	15.79	2381251	{}	\N	2020-10-12 15:28:49.935449
53372	KAR	2020-10-06	16.04	16.395	15.85	15.95	4757219	{}	\N	2020-10-12 15:28:49.940559
53373	KAR	2020-10-07	16.15	16.49	16.01	16.19	3334262	{}	\N	2020-10-12 15:28:49.946251
53374	KAR	2020-10-08	16.4	16.52	16.3	16.47	1812450	{}	\N	2020-10-12 15:28:49.951207
53375	KAR	2020-10-09	16.68	16.99	16.56	16.82	2939929	{}	\N	2020-10-12 15:28:49.956054
53376	KAR	2020-10-12	16.82	17.16	16.63	17.115	674074	{}	\N	2020-10-12 15:28:49.96069
53377	KARS	2020-09-28	28.38	28.695	28.38	28.695	13371	{}	\N	2020-10-12 15:28:50.832435
53378	KARS	2020-09-29	28.6	29.06	28.6	28.97	9457	{}	\N	2020-10-12 15:28:50.840702
53379	KARS	2020-09-30	29.065	29.4095	29.065	29.255	6926	{}	\N	2020-10-12 15:28:50.84817
53380	KARS	2020-10-01	29.785	29.9	29.65	29.7952	8567	{}	\N	2020-10-12 15:28:50.854289
53381	KARS	2020-10-02	29.19	29.68	28.8401	29.37	12824	{}	\N	2020-10-12 15:28:50.859097
53382	KARS	2020-10-05	29.79	30.15	29.775	30.12	7448	{}	\N	2020-10-12 15:28:50.866464
53383	KARS	2020-10-06	30.39	30.6	30	30	12506	{}	\N	2020-10-12 15:28:50.871449
53384	KARS	2020-10-07	30.13	30.77	30.13	30.75	9408	{}	\N	2020-10-12 15:28:50.876397
53385	KARS	2020-10-08	31.04	31.04	30.8033	30.8033	17369	{}	\N	2020-10-12 15:28:50.881493
53386	KARS	2020-10-09	30.91	31.12	30.91	31.08	25872	{}	\N	2020-10-12 15:28:50.886436
53387	KARS	2020-10-12	31.56	31.56	31.2	31.343	10071	{}	\N	2020-10-12 15:28:50.891371
53388	KBLMR	2020-09-28	0.229999	0.2415	0.19	0.2	110732	{}	\N	2020-10-12 15:28:51.946781
53389	KBLMR	2020-09-29	0.202	0.27	0.2	0.2	171329	{}	\N	2020-10-12 15:28:51.953927
53390	KBLMR	2020-09-30	0.23	0.23	0.1999	0.2	59996	{}	\N	2020-10-12 15:28:51.961217
53391	KBLMR	2020-10-01	0.2105	0.219999	0.2	0.21	70728	{}	\N	2020-10-12 15:28:51.967714
53392	KBLMR	2020-10-02	0.21	0.23	0.1916	0.21	34020	{}	\N	2020-10-12 15:28:51.97438
53393	KBLMR	2020-10-05	0.24	0.24	0.2	0.2	153163	{}	\N	2020-10-12 15:28:51.981147
53394	KBLMR	2020-10-06	0.22	0.22	0.19	0.19	342701	{}	\N	2020-10-12 15:28:51.987335
53395	KBLMR	2020-10-07	0.198	0.2078	0.19	0.2	188055	{}	\N	2020-10-12 15:28:51.994537
53396	KBLMR	2020-10-08	0.2	0.21	0.1796	0.19	642498	{}	\N	2020-10-12 15:28:52.001111
53397	KBLMR	2020-10-09	0.36	0.36	0.1701	0.1701	663088	{}	\N	2020-10-12 15:28:52.008898
53398	KBLMR	2020-10-12	0.35	0.35	0.28	0.3046	1011090	{}	\N	2020-10-12 15:28:52.018677
53399	KBWR	2020-09-28	31.977	32.39	31.89	32.39	5434	{}	\N	2020-10-12 15:28:52.96834
53400	KBWR	2020-09-29	32	32	31.2566	31.8429	3370	{}	\N	2020-10-12 15:28:52.979017
53401	KBWR	2020-09-30	32.02	32.37	31.97	31.97	779	{}	\N	2020-10-12 15:28:52.985111
53402	KBWR	2020-10-01	32.25	32.25	31.99	32.1	1929	{}	\N	2020-10-12 15:28:52.991644
53403	KBWR	2020-10-02	31.52	33.24	31.52	33.2215	5578	{}	\N	2020-10-12 15:28:52.997499
53404	KBWR	2020-10-05	33.38	34.5191	33.38	34.5191	3855	{}	\N	2020-10-12 15:28:53.003376
53405	KBWR	2020-10-06	34.73	35.84	34.565	34.565	5317	{}	\N	2020-10-12 15:28:53.009176
53406	KBWR	2020-10-07	35.1911	35.57	35.11	35.4159	6921	{}	\N	2020-10-12 15:28:53.013799
53407	KBWR	2020-10-08	35.75	36.0447	35.75	36.0447	2206	{}	\N	2020-10-12 15:28:53.019116
53408	KBWR	2020-10-09	36.19	36.19	35.6271	35.6271	13290	{}	\N	2020-10-12 15:28:53.025405
53409	KBWR	2020-10-12	35.832	35.96	35.832	35.96	676	{}	\N	2020-10-12 15:28:53.030646
53410	KBWD	2020-09-28	13.36	13.71	13.343	13.61	113902	{}	\N	2020-10-12 15:28:54.298998
53411	KBWD	2020-09-29	13.59	13.61	13.3	13.405	74677	{}	\N	2020-10-12 15:28:54.30894
53412	KBWD	2020-09-30	13.47	13.6	13.36	13.42	108455	{}	\N	2020-10-12 15:28:54.315126
53413	KBWD	2020-10-01	13.45	13.51	13.3301	13.51	107726	{}	\N	2020-10-12 15:28:54.319595
53414	KBWD	2020-10-02	13.32	13.85	13.32	13.82	102819	{}	\N	2020-10-12 15:28:54.323998
53415	KBWD	2020-10-05	13.8995	14	13.87	13.98	90560	{}	\N	2020-10-12 15:28:54.328227
53416	KBWD	2020-10-06	14.05	14.17	13.83	13.86	226797	{}	\N	2020-10-12 15:28:54.33328
53417	KBWD	2020-10-07	13.99	14.06	13.89	13.985	75223	{}	\N	2020-10-12 15:28:54.338815
53418	KBWD	2020-10-08	14.09	14.279	14.09	14.24	85757	{}	\N	2020-10-12 15:28:54.343825
53419	KBWD	2020-10-09	14.2908	14.31	14.12	14.14	93466	{}	\N	2020-10-12 15:28:54.3496
53420	KBWD	2020-10-12	14.17	14.18	14.12	14.131	25670	{}	\N	2020-10-12 15:28:54.355776
53421	KCE	2020-09-28	57.845	57.845	57.7713	57.7713	474	{}	\N	2020-10-12 15:28:55.60457
53422	KCE	2020-09-29	57.7281	57.78	57.16	57.78	8544	{}	\N	2020-10-12 15:28:55.612957
53423	KCE	2020-09-30	58.26	58.26	57.92	57.92	443	{}	\N	2020-10-12 15:28:55.62054
53424	KCE	2020-10-01	58.475	58.475	57.915	58.2299	4334	{}	\N	2020-10-12 15:28:55.633375
53425	KCE	2020-10-02	58.8	59.33	58.8	59.2	7284	{}	\N	2020-10-12 15:28:55.643796
53426	KCE	2020-10-05	59.8479	60.12	59.78	60.12	11209	{}	\N	2020-10-12 15:28:55.654521
53427	KCE	2020-10-06	60.28	60.9	59.61	59.61	3576	{}	\N	2020-10-12 15:28:55.670072
53428	KCE	2020-10-07	60.295	60.6462	60.2	60.6462	4185	{}	\N	2020-10-12 15:28:55.689344
53429	KCE	2020-10-08	61.63	62.8	61.63	62.69	10685	{}	\N	2020-10-12 15:28:55.711036
53430	KCE	2020-10-09	63.275	63.275	63.1781	63.1781	908	{}	\N	2020-10-12 15:28:55.737837
53431	KCE	2020-10-12	63.815	63.815	63.815	63.815	459	{}	\N	2020-10-12 15:28:55.767954
53432	KDMN	2020-09-28	3.83	3.85	3.7	3.76	1947943	{}	\N	2020-10-12 15:28:57.252368
53433	KDMN	2020-09-29	3.77	3.77	3.605	3.72	7850026	{}	\N	2020-10-12 15:28:57.264045
53434	KDMN	2020-09-30	3.72	3.93	3.67	3.92	7737918	{}	\N	2020-10-12 15:28:57.271256
53435	KDMN	2020-10-01	3.99	4.18	3.94	4.06	4847012	{}	\N	2020-10-12 15:28:57.278039
53436	KDMN	2020-10-02	3.98	4.11	3.92	3.94	2676905	{}	\N	2020-10-12 15:28:57.283012
53437	KDMN	2020-10-05	3.98	4.185	3.94	4.1	2886827	{}	\N	2020-10-12 15:28:57.288062
53438	KDMN	2020-10-06	4.1	4.165	4.015	4.03	3315692	{}	\N	2020-10-12 15:28:57.292102
53439	KDMN	2020-10-07	4.11	4.14	4.02	4.06	2633777	{}	\N	2020-10-12 15:28:57.296273
53440	KDMN	2020-10-08	4.11	4.19	4.05	4.08	1677953	{}	\N	2020-10-12 15:28:57.300395
53441	KDMN	2020-10-09	4.14	4.195	4.09	4.14	2238380	{}	\N	2020-10-12 15:28:57.304935
53442	KDMN	2020-10-12	4.15	4.15	3.9482	3.95	914796	{}	\N	2020-10-12 15:28:57.309373
53443	KEP	2020-09-28	8.59	8.77	8.59	8.72	89198	{}	\N	2020-10-12 15:28:58.503861
53444	KEP	2020-09-29	8.73	8.75	8.67	8.67	145304	{}	\N	2020-10-12 15:28:58.513687
53445	KEP	2020-09-30	8.71	8.75	8.6698	8.71	669137	{}	\N	2020-10-12 15:28:58.521589
53446	KEP	2020-10-01	8.73	8.83	8.72	8.83	161131	{}	\N	2020-10-12 15:28:58.528507
53447	KEP	2020-10-02	8.66	8.82	8.64	8.81	349835	{}	\N	2020-10-12 15:28:58.53482
53448	KEP	2020-10-05	8.87	8.99	8.87	8.99	211825	{}	\N	2020-10-12 15:28:58.540967
53449	KEP	2020-10-06	8.77	8.8	8.63	8.69	167544	{}	\N	2020-10-12 15:28:58.547774
53450	KEP	2020-10-07	8.91	8.96	8.77	8.84	346829	{}	\N	2020-10-12 15:28:58.554328
53451	KEP	2020-10-08	9.08	9.15	9.02	9.1	104642	{}	\N	2020-10-12 15:28:58.561629
53452	KEP	2020-10-09	9.15	9.185	9.07	9.16	137942	{}	\N	2020-10-12 15:28:58.568168
53453	KEP	2020-10-12	9.15	9.15	9.0001	9.04	24284	{}	\N	2020-10-12 15:28:58.5752
53454	JTD	2020-09-28	12.84	12.84	12.67	12.76	35202	{}	\N	2020-10-12 15:28:59.50313
53455	JTD	2020-09-29	12.86	12.86	12.7	12.78	102116	{}	\N	2020-10-12 15:28:59.516926
53456	JTD	2020-09-30	12.86	13.1	12.82	13.07	113761	{}	\N	2020-10-12 15:28:59.524452
53457	JTD	2020-10-01	13.18	13.18	12.94	13	145021	{}	\N	2020-10-12 15:28:59.53286
53458	JTD	2020-10-02	12.97	12.99	12.7626	12.99	71516	{}	\N	2020-10-12 15:28:59.538808
53459	JTD	2020-10-05	13	13.1	12.99	13.05	60346	{}	\N	2020-10-12 15:28:59.543894
53460	JTD	2020-10-06	13.12	13.15	12.98	13.04	53641	{}	\N	2020-10-12 15:28:59.548863
53461	JTD	2020-10-07	13.12	13.12	13	13.06	78228	{}	\N	2020-10-12 15:28:59.553533
53462	JTD	2020-10-08	13.12	13.12	13	13.04	60818	{}	\N	2020-10-12 15:28:59.558504
53463	JTD	2020-10-09	13.04	13.1	13.035	13.04	89664	{}	\N	2020-10-12 15:28:59.563207
53464	JTD	2020-10-12	13.13	13.1868	13.1	13.1852	58066	{}	\N	2020-10-12 15:28:59.568593
53465	KAI	2020-09-28	108.85	112.51	108.85	112.22	77488	{}	\N	2020-10-12 15:29:00.596463
53466	KAI	2020-09-29	112.49	113.505	109.96	110.72	78490	{}	\N	2020-10-12 15:29:00.605161
53467	KAI	2020-09-30	111.71	113.91	108.66	109.62	132961	{}	\N	2020-10-12 15:29:00.612564
53468	KAI	2020-10-01	109.66	111.39	108.48	110.95	104223	{}	\N	2020-10-12 15:29:00.620413
53469	KAI	2020-10-02	109.24	112.14	108.63	110.41	103653	{}	\N	2020-10-12 15:29:00.626982
53470	KAI	2020-10-05	110.94	114.07	110.94	112.36	90603	{}	\N	2020-10-12 15:29:00.634106
53471	KAI	2020-10-06	113.27	115.99	111.12	112.75	44780	{}	\N	2020-10-12 15:29:00.641825
53472	KAI	2020-10-07	113.96	115.04	112.1	114.01	73757	{}	\N	2020-10-12 15:29:00.649146
53473	KAI	2020-10-08	115.73	115.73	113.74	114.51	143117	{}	\N	2020-10-12 15:29:00.65716
53474	KAI	2020-10-09	116.01	117.7	114.43	117.22	56176	{}	\N	2020-10-12 15:29:00.663651
53475	KAI	2020-10-12	117.07	119.1	117.07	118	11162	{}	\N	2020-10-12 15:29:00.670838
53476	KALA	2020-09-28	7.82	8.13	7.61	7.97	1103515	{}	\N	2020-10-12 15:29:01.577119
53477	KALA	2020-09-29	7.94	8.04	7.66	7.93	638906	{}	\N	2020-10-12 15:29:01.586524
53478	KALA	2020-09-30	7.89	8.16	7.46	7.5	2198158	{}	\N	2020-10-12 15:29:01.594525
53479	KALA	2020-10-01	7.745	7.79	7.49	7.66	1082400	{}	\N	2020-10-12 15:29:01.601484
53480	KALA	2020-10-02	7.51	7.77	7.24	7.36	1174735	{}	\N	2020-10-12 15:29:01.607499
53481	KALA	2020-10-05	7.4	7.95	7.4	7.79	1957379	{}	\N	2020-10-12 15:29:01.613168
53482	KALA	2020-10-06	7.8794	8.635	7.8794	8.27	2342529	{}	\N	2020-10-12 15:29:01.618162
53483	KALA	2020-10-07	8.54	9.14	8.35	8.79	2494341	{}	\N	2020-10-12 15:29:01.623851
53484	KALA	2020-10-08	9.2	9.225	8.53	8.67	1698589	{}	\N	2020-10-12 15:29:01.630181
53485	KALA	2020-10-09	8.84	8.84	8.4	8.61	896506	{}	\N	2020-10-12 15:29:01.636478
53486	KALA	2020-10-12	8.61	8.65	8.2636	8.3	438090	{}	\N	2020-10-12 15:29:01.642082
53487	KB	2020-09-28	32.74	33.09	32.66	32.73	90832	{}	\N	2020-10-12 15:29:02.516779
53488	KB	2020-09-29	32.29	32.29	31.76	31.78	179340	{}	\N	2020-10-12 15:29:02.525189
53489	KB	2020-09-30	31.81	32.31	31.76	32.12	244328	{}	\N	2020-10-12 15:29:02.533442
53490	KB	2020-10-01	32.1	32.57	32.1	32.53	168166	{}	\N	2020-10-12 15:29:02.539903
53491	KB	2020-10-02	31.84	32.71	31.84	32.31	420270	{}	\N	2020-10-12 15:29:02.54589
53492	KB	2020-10-05	33.36	34.16	33.36	33.83	286927	{}	\N	2020-10-12 15:29:02.552342
53493	KB	2020-10-06	33.76	33.99	33.29	33.36	447572	{}	\N	2020-10-12 15:29:02.558053
53494	KB	2020-10-07	34.18	34.49	34.18	34.38	212707	{}	\N	2020-10-12 15:29:02.564319
53495	KB	2020-10-08	34.5	34.5709	34.16	34.2	174340	{}	\N	2020-10-12 15:29:02.570302
53496	KB	2020-10-09	34.4	34.66	34.36	34.61	262435	{}	\N	2020-10-12 15:29:02.576244
53497	KB	2020-10-12	35.1	35.4	35.01	35.36	97292	{}	\N	2020-10-12 15:29:02.581339
53498	KBAL	2020-09-28	10.57	10.7044	10.46	10.47	135591	{}	\N	2020-10-12 15:29:03.481165
53499	KBAL	2020-09-29	10.49	10.62	10.22	10.39	355009	{}	\N	2020-10-12 15:29:03.489889
53500	KBAL	2020-09-30	10.57	10.81	10.43	10.54	214947	{}	\N	2020-10-12 15:29:03.497365
53501	KBAL	2020-10-01	10.55	10.7	10.47	10.67	108670	{}	\N	2020-10-12 15:29:03.504879
53502	KBAL	2020-10-02	10.43	10.93	10.32	10.87	161519	{}	\N	2020-10-12 15:29:03.51143
53503	KBAL	2020-10-05	10.97	11.26	10.8	11.12	140087	{}	\N	2020-10-12 15:29:03.517822
53504	KBAL	2020-10-06	11.01	11.61	10.96	11.14	236282	{}	\N	2020-10-12 15:29:03.523614
53505	KBAL	2020-10-07	11.25	11.6	11.07	11.12	112866	{}	\N	2020-10-12 15:29:03.531793
53506	KBAL	2020-10-08	11.53	11.53	10.97	11.07	85030	{}	\N	2020-10-12 15:29:03.539608
53507	KBAL	2020-10-09	11.23	11.45	10.94	11.08	102615	{}	\N	2020-10-12 15:29:03.547048
53508	KBAL	2020-10-12	11.04	11.251	10.972	11.251	25453	{}	\N	2020-10-12 15:29:03.553695
53509	KBLMW	2020-09-28	0.3	0.3	0.247	0.26	29932	{}	\N	2020-10-12 15:29:04.658611
53510	KBLMW	2020-09-29	0.2535	0.28	0.2501	0.26	98045	{}	\N	2020-10-12 15:29:04.669074
53511	KBLMW	2020-09-30	0.27	0.2701	0.2201	0.23	392210	{}	\N	2020-10-12 15:29:04.676253
53512	KBLMW	2020-10-01	0.25	0.279	0.235	0.2437	92990	{}	\N	2020-10-12 15:29:04.682015
53513	KBLMW	2020-10-02	0.2554	0.2701	0.2302	0.27	146838	{}	\N	2020-10-12 15:29:04.68755
53514	KBLMW	2020-10-05	0.28	0.2948	0.265	0.274	135692	{}	\N	2020-10-12 15:29:04.69276
53515	KBLMW	2020-10-06	0.26	0.27	0.23	0.24	138067	{}	\N	2020-10-12 15:29:04.696857
53516	KBLMW	2020-10-07	0.25	0.2501	0.2222	0.2434	244186	{}	\N	2020-10-12 15:29:04.701062
53517	KBLMW	2020-10-08	0.2228	0.2481	0.22	0.23	116868	{}	\N	2020-10-12 15:29:04.705281
53518	KBLMW	2020-10-09	0.239999	0.24	0.2045	0.2379	343216	{}	\N	2020-10-12 15:29:04.710968
53519	KBLMW	2020-10-12	0.2505	0.2505	0.2201	0.2201	108080	{}	\N	2020-10-12 15:29:04.716324
53520	KBR	2020-09-28	22.63	23	22.57	22.72	2418464	{}	\N	2020-10-12 15:29:05.565814
53521	KBR	2020-09-29	22.71	22.92	22.155	22.7	2254900	{}	\N	2020-10-12 15:29:05.576737
53522	KBR	2020-09-30	22.75	23.09	22.16	22.36	2665623	{}	\N	2020-10-12 15:29:05.584767
53523	KBR	2020-10-01	22.52	22.71	21.8	22.17	2840446	{}	\N	2020-10-12 15:29:05.59128
53524	KBR	2020-10-02	21.72	22.59	21.72	22.43	2431675	{}	\N	2020-10-12 15:29:05.596992
53525	KBR	2020-10-05	22.67	23.35	22.58	23.33	2395639	{}	\N	2020-10-12 15:29:05.601652
53526	KBR	2020-10-06	23.55	23.77	22.62	22.91	4440051	{}	\N	2020-10-12 15:29:05.606355
53527	KBR	2020-10-07	23.15	23.68	23.15	23.4	1887516	{}	\N	2020-10-12 15:29:05.611147
53528	KBR	2020-10-08	23.78	23.85	23.29	23.59	3095346	{}	\N	2020-10-12 15:29:05.615697
53529	KBR	2020-10-09	24.02	24.54	23.72	24.37	3586337	{}	\N	2020-10-12 15:29:05.620368
53530	KBR	2020-10-12	24.52	24.78	24.3	24.72	676846	{}	\N	2020-10-12 15:29:05.624777
53531	KBWY	2020-09-28	17.38	17.93	17.38	17.75	136732	{}	\N	2020-10-12 15:29:07.170372
53532	KBWY	2020-09-29	17.76	17.76	17.23	17.47	82636	{}	\N	2020-10-12 15:29:07.177323
53533	KBWY	2020-09-30	17.55	17.9	17.39	17.52	99722	{}	\N	2020-10-12 15:29:07.183072
53534	KBWY	2020-10-01	17.54	17.99	17.48	17.99	58209	{}	\N	2020-10-12 15:29:07.190497
53535	KBWY	2020-10-02	17.62	18.45	17.54	18.4	135176	{}	\N	2020-10-12 15:29:07.197516
53536	KBWY	2020-10-05	18.55	18.63	18.13	18.29	86227	{}	\N	2020-10-12 15:29:07.203806
53537	KBWY	2020-10-06	18.39	18.76	18.26	18.28	79169	{}	\N	2020-10-12 15:29:07.211248
53538	KBWY	2020-10-07	18.45	18.49	18.2471	18.37	64371	{}	\N	2020-10-12 15:29:07.218043
53539	KBWY	2020-10-08	18.49	18.81	18.41	18.8	81920	{}	\N	2020-10-12 15:29:07.224472
53540	KBWY	2020-10-09	18.89	18.95	18.521	18.56	45734	{}	\N	2020-10-12 15:29:07.229718
53541	KBWY	2020-10-12	18.56	18.605	18.36	18.56	25894	{}	\N	2020-10-12 15:29:07.236498
53542	KCAPL	2020-09-28	24.5	24.5	24.5	24.5	515	{}	\N	2020-10-12 15:29:08.588785
53543	KCAPL	2020-09-29	24.66	24.66	24.5	24.66	796	{}	\N	2020-10-12 15:29:08.598084
53544	KCAPL	2020-09-30	24.66	24.66	24.66	24.66	503	{}	\N	2020-10-12 15:29:08.607289
53545	KCAPL	2020-10-01	24.54	24.66	24.54	24.66	9923	{}	\N	2020-10-12 15:29:08.61959
53546	KCAPL	2020-10-02	24.6599	24.6599	24.6599	24.6599	138	{}	\N	2020-10-12 15:29:08.629466
53547	KCAPL	2020-10-05	24.48	24.5867	24.41	24.5603	700	{}	\N	2020-10-12 15:29:08.639039
53548	KCAPL	2020-10-06	24.5	24.6	24.5	24.52	4678	{}	\N	2020-10-12 15:29:08.649862
53549	KCAPL	2020-10-07	24.6	24.6	24.5324	24.6	1479921	{}	\N	2020-10-12 15:29:08.659999
53550	KCAPL	2020-10-08	24.4101	24.46	24.4101	24.44	1505	{}	\N	2020-10-12 15:29:08.67054
53551	KCAPL	2020-10-09	24.505	24.6	24.42	24.6	1725	{}	\N	2020-10-12 15:29:08.681807
53552	KCAPL	2020-10-12	24.55	24.55	24.55	24.55	101	{}	\N	2020-10-12 15:29:08.692108
53553	KELYA	2020-09-28	16.5	16.98	16.5	16.89	102770	{}	\N	2020-10-12 15:29:10.049596
53554	KELYA	2020-09-29	16.83	16.96	16.67	16.89	93198	{}	\N	2020-10-12 15:29:10.057704
53555	KELYA	2020-09-30	17.145	17.55	16.95	17.04	247434	{}	\N	2020-10-12 15:29:10.065882
53556	KELYA	2020-10-01	17.05	17.21	16.77	16.87	73134	{}	\N	2020-10-12 15:29:10.079426
53557	KELYA	2020-10-02	16.42	17.09	15.56	17.04	115918	{}	\N	2020-10-12 15:29:10.089105
53558	KELYA	2020-10-05	17.18	17.63	17.18	17.6	96316	{}	\N	2020-10-12 15:29:10.101976
53559	KELYA	2020-10-06	17.61	18.23	17.61	17.91	192527	{}	\N	2020-10-12 15:29:10.109823
53560	KELYA	2020-10-07	18.09	18.39	17.72	18.32	182703	{}	\N	2020-10-12 15:29:10.117211
53561	KELYA	2020-10-08	18.52	18.73	18.255	18.73	99925	{}	\N	2020-10-12 15:29:10.126682
53562	KELYA	2020-10-09	18.92	18.9399	18.445	18.58	68095	{}	\N	2020-10-12 15:29:10.136157
53563	KELYA	2020-10-12	18.57	18.94	18.37	18.88	19635	{}	\N	2020-10-12 15:29:10.142341
53564	KE	2020-09-28	11.41	12.08	11.41	11.96	49326	{}	\N	2020-10-12 15:29:11.240684
53565	KE	2020-09-29	12.23	12.23	11.7618	11.91	37730	{}	\N	2020-10-12 15:29:11.252354
53566	KE	2020-09-30	11.94	12.01	11.51	11.56	20828	{}	\N	2020-10-12 15:29:11.259757
53567	KE	2020-10-01	11.53	11.71	11.43	11.6	36951	{}	\N	2020-10-12 15:29:11.266906
53568	KE	2020-10-02	11.4	11.62	11.14	11.42	17638	{}	\N	2020-10-12 15:29:11.273118
53569	KE	2020-10-05	11.62	12.14	11.605	12.07	59178	{}	\N	2020-10-12 15:29:11.278701
53570	KE	2020-10-06	12.17	12.71	12.09	12.24	150411	{}	\N	2020-10-12 15:29:11.284067
53571	KE	2020-10-07	12.44	12.5	12.21	12.32	58677	{}	\N	2020-10-12 15:29:11.289385
53572	KE	2020-10-08	12.43	12.5018	12.11	12.36	40013	{}	\N	2020-10-12 15:29:11.294359
53573	KE	2020-10-09	12.44	12.56	12.23	12.29	37220	{}	\N	2020-10-12 15:29:11.299783
53574	KE	2020-10-12	12.32	12.4067	12.26	12.37	6141	{}	\N	2020-10-12 15:29:11.305215
53575	KEQU	2020-09-28	9.12	9.5	9.01	9.3145	2514	{}	\N	2020-10-12 15:29:12.953094
53576	KEQU	2020-09-29	8.98	8.98	8.98	8.98	265	{}	\N	2020-10-12 15:29:12.965251
53577	KEQU	2020-09-30	8.9657	8.9788	8.92	8.94	2455	{}	\N	2020-10-12 15:29:12.972089
53578	KEQU	2020-10-01	8.99	8.99	8.95	8.95	611	{}	\N	2020-10-12 15:29:12.978772
53579	KEQU	2020-10-02	8.92	9	8.8	9	1913	{}	\N	2020-10-12 15:29:12.985404
53580	KEQU	2020-10-05	8.94	9.25	8.91	8.96	2552	{}	\N	2020-10-12 15:29:12.990998
53581	KEQU	2020-10-06	9	9	8.72	8.72	8537	{}	\N	2020-10-12 15:29:12.99511
53582	KEQU	2020-10-07	8.8387	8.99	8.75	8.75	5256	{}	\N	2020-10-12 15:29:13.000022
53583	KEQU	2020-10-08	8.82	8.95	8.82	8.85	6068	{}	\N	2020-10-12 15:29:13.005109
53584	KEQU	2020-10-09	8.755	9	8.75	8.8	7766	{}	\N	2020-10-12 15:29:13.010616
53585	KEQU	2020-10-12	8.79	8.79	8.79	8.79	317	{}	\N	2020-10-12 15:29:13.016139
53586	JYNT	2020-09-28	17.52	17.9	17.27	17.52	54839	{}	\N	2020-10-12 15:29:14.421669
53587	JYNT	2020-09-29	17.52	17.71	17.18	17.4	24101	{}	\N	2020-10-12 15:29:14.430685
53588	JYNT	2020-09-30	17.48	17.82	17.14	17.39	29323	{}	\N	2020-10-12 15:29:14.44021
53589	JYNT	2020-10-01	17.53	17.7	17.135	17.51	36589	{}	\N	2020-10-12 15:29:14.448667
53590	JYNT	2020-10-02	17.12	18.25	17.05	17.95	70439	{}	\N	2020-10-12 15:29:14.455999
53591	JYNT	2020-10-05	17.99	18.25	17.35	18	86959	{}	\N	2020-10-12 15:29:14.462847
53592	JYNT	2020-10-06	17.665	18.25	17.665	17.9	76316	{}	\N	2020-10-12 15:29:14.4693
53593	JYNT	2020-10-07	18.2	18.69	16.38	18.51	36105	{}	\N	2020-10-12 15:29:14.475977
53594	JYNT	2020-10-08	18.6	18.66	18.23	18.38	20068	{}	\N	2020-10-12 15:29:14.483115
53595	JYNT	2020-10-09	18.69	18.83	18.31	18.68	39749	{}	\N	2020-10-12 15:29:14.488579
53596	JYNT	2020-10-12	18.75	18.98	18.601	18.894	2349	{}	\N	2020-10-12 15:29:14.494366
53597	KBA	2020-09-28	39.13	39.14	38.93	39.09	223262	{}	\N	2020-10-12 15:29:16.774612
53598	KBA	2020-09-29	39.2	39.48	39.11	39.34	344270	{}	\N	2020-10-12 15:29:16.782368
53599	KBA	2020-09-30	39.2	39.5116	39.1836	39.47	112992	{}	\N	2020-10-12 15:29:16.789408
53600	KBA	2020-10-01	39.93	40.12	39.9	40.09	159791	{}	\N	2020-10-12 15:29:16.798113
53601	KBA	2020-10-02	39.36	39.8	39.201	39.66	178291	{}	\N	2020-10-12 15:29:16.804914
53602	KBA	2020-10-05	39.88	40.13	39.88	40.12	83878	{}	\N	2020-10-12 15:29:16.811459
53603	KBA	2020-10-06	40.26	40.29	39.87	40.04	138860	{}	\N	2020-10-12 15:29:16.817588
53604	KBA	2020-10-07	40.39	40.547	40.35	40.48	119205	{}	\N	2020-10-12 15:29:16.823298
53605	KBA	2020-10-08	40.47	40.61	40.42	40.54	147220	{}	\N	2020-10-12 15:29:16.828569
53606	KBA	2020-10-09	40.81	40.9765	40.81	40.94	504438	{}	\N	2020-10-12 15:29:16.833796
53607	KBA	2020-10-12	41.6406	41.7099	41.59	41.7099	82453	{}	\N	2020-10-12 15:29:16.838823
53608	KBSF	2020-09-28	1.81	1.94	1.81	1.86	15440	{}	\N	2020-10-12 15:29:17.868302
53609	KBSF	2020-09-29	1.8235	1.97	1.7701	1.91	55249	{}	\N	2020-10-12 15:29:17.881636
53610	KBSF	2020-09-30	1.92	1.93	1.87	1.885	7545	{}	\N	2020-10-12 15:29:17.890362
53611	KBSF	2020-10-01	1.9199	2.04	1.895	1.93	17376	{}	\N	2020-10-12 15:29:17.896912
53612	KBSF	2020-10-02	1.9	1.9167	1.78	1.91	20856	{}	\N	2020-10-12 15:29:17.902572
53613	KBSF	2020-10-05	1.98	1.98	1.86	1.94	18372	{}	\N	2020-10-12 15:29:17.908429
53614	KBSF	2020-10-06	1.87	2.05	1.8601	1.99	20384	{}	\N	2020-10-12 15:29:17.913388
53615	KBSF	2020-10-07	2.02	2.07	1.97	2.07	21378	{}	\N	2020-10-12 15:29:17.919134
53616	KBSF	2020-10-08	2.1	2.185	2.01	2.1	23300	{}	\N	2020-10-12 15:29:17.924483
53617	KBSF	2020-10-09	2.0999	2.1	1.865	2.02	33788	{}	\N	2020-10-12 15:29:17.929508
53618	KBSF	2020-10-12	1.99	2.25	1.99	2.14	91976	{}	\N	2020-10-12 15:29:17.935128
53619	KCNY	2020-09-28	33.41	33.53	33.41	33.53	445	{}	\N	2020-10-12 15:29:19.399375
53620	KCNY	2020-09-29	33.3924	33.3924	33.3924	33.3924	44	{}	\N	2020-10-12 15:29:19.414838
53621	KCNY	2020-09-30	33.5395	33.55	33.5395	33.55	208	{}	\N	2020-10-12 15:29:19.427533
53622	KCNY	2020-10-01	33.6	33.75	33.6	33.67	551	{}	\N	2020-10-12 15:29:19.434731
53623	KCNY	2020-10-02	33.72	33.72	33.72	33.72	151	{}	\N	2020-10-12 15:29:19.440369
53624	KCNY	2020-10-05	33.8372	33.8372	33.7501	33.7501	943	{}	\N	2020-10-12 15:29:19.445243
53625	KCNY	2020-10-06	33.8203	33.84	33.8	33.84	4048	{}	\N	2020-10-12 15:29:19.450296
53626	KCNY	2020-10-07	33.85	33.85	33.85	33.85	5	{}	\N	2020-10-12 15:29:19.455072
53627	KCNY	2020-10-08	33.72	33.767	33.72	33.767	519	{}	\N	2020-10-12 15:29:19.460577
53628	KCNY	2020-10-09	33.96	33.96	33.96	33.96	0	{}	\N	2020-10-12 15:29:19.465675
53629	KCNY	2020-10-12	33.96	33.96	33.96	33.96	200	{}	\N	2020-10-12 15:29:19.471556
53630	KELYB	2020-09-28	16.59	17.9998	16.31	16.84	14714	{}	\N	2020-10-12 15:29:21.182117
53631	KELYB	2020-09-29	16.7001	17.6871	15.5001	17.0072	19224	{}	\N	2020-10-12 15:29:21.189743
53632	KELYB	2020-09-30	17.2	17.86	16.6786	17.395	12980	{}	\N	2020-10-12 15:29:21.197839
53633	KELYB	2020-10-01	17.34	17.5	16.29	17.4999	11859	{}	\N	2020-10-12 15:29:21.205725
53634	KELYB	2020-10-02	17.05	17.3	16.695	17.015	5800	{}	\N	2020-10-12 15:29:21.213349
53635	KELYB	2020-10-05	16.3995	17.5	16.3995	17.48	10177	{}	\N	2020-10-12 15:29:21.220558
53636	KELYB	2020-10-06	17.2627	18.09	17.2325	17.743	5171	{}	\N	2020-10-12 15:29:21.22755
53637	KELYB	2020-10-07	18.35	19.45	17.7902	18.01	7105	{}	\N	2020-10-12 15:29:21.235238
53638	KELYB	2020-10-08	17.8868	18.5	17.8689	18.46	6930	{}	\N	2020-10-12 15:29:21.241141
53639	KELYB	2020-10-09	19.15	19.225	18.35	18.35	6431	{}	\N	2020-10-12 15:29:21.250232
53640	KELYB	2020-10-12	18.47	18.47	18.47	18.47	778	{}	\N	2020-10-12 15:29:21.257749
53641	KEMQ	2020-09-28	28.18	28.18	27.89	28.09	8333	{}	\N	2020-10-12 15:29:22.412297
53642	KEMQ	2020-09-29	27.914	27.9599	27.8239	27.93	3765	{}	\N	2020-10-12 15:29:22.418411
53643	KEMQ	2020-09-30	27.93	28.28	27.93	28.2034	6125	{}	\N	2020-10-12 15:29:22.423188
53644	KEMQ	2020-10-01	28.37	28.555	28.35	28.52	8194	{}	\N	2020-10-12 15:29:22.428457
53645	KEMQ	2020-10-02	28.18	28.42	28.18	28.25	12272	{}	\N	2020-10-12 15:29:22.435558
53646	KEMQ	2020-10-05	28.37	28.5	28.355	28.5	22100	{}	\N	2020-10-12 15:29:22.440984
53647	KEMQ	2020-10-06	28.61	28.83	28.5275	28.5275	29689	{}	\N	2020-10-12 15:29:22.446683
53648	KEMQ	2020-10-07	28.84	28.94	28.748	28.94	6067	{}	\N	2020-10-12 15:29:22.452473
53649	KEMQ	2020-10-08	28.96	29.0404	28.95	29.0404	2277	{}	\N	2020-10-12 15:29:22.457246
53650	KEMQ	2020-10-09	29.2	29.46	29.03	29.3117	24801	{}	\N	2020-10-12 15:29:22.462803
53651	KEMQ	2020-10-12	29.49	29.49	29.39	29.45	7275	{}	\N	2020-10-12 15:29:22.468192
53652	ITT	2020-09-28	59.83	60.3	59.2	59.98	583679	{}	\N	2020-10-12 15:29:26.197197
53653	ITT	2020-09-29	60.17	60.405	59.34	59.45	909031	{}	\N	2020-10-12 15:29:26.208872
53654	ITT	2020-09-30	59.7	60.31	58.54	59.05	1319973	{}	\N	2020-10-12 15:29:26.215876
53655	ITT	2020-10-01	59.52	59.98	58.01	58.54	726833	{}	\N	2020-10-12 15:29:26.222216
53656	ITT	2020-10-02	57.15	59.83	57.13	58.96	844765	{}	\N	2020-10-12 15:29:26.227436
53657	ITT	2020-10-05	59.91	61.03	59.87	60.64	845094	{}	\N	2020-10-12 15:29:26.232685
53658	ITT	2020-10-06	60.89	62.5	60.15	60.96	914647	{}	\N	2020-10-12 15:29:26.237612
53659	ITT	2020-10-07	61.8	62.92	61.74	62.66	714522	{}	\N	2020-10-12 15:29:26.242634
53660	ITT	2020-10-08	63	63.44	62.59	63.31	619172	{}	\N	2020-10-12 15:29:26.247266
53661	ITT	2020-10-09	64.08	65.06	63.56	64.25	1899580	{}	\N	2020-10-12 15:29:26.251735
53662	ITT	2020-10-12	64.66	64.705	63.495	63.74	152959	{}	\N	2020-10-12 15:29:26.258023
53663	IWM	2020-09-28	148.37	150.44	146.404	150.02	17640276	{}	\N	2020-10-12 15:29:28.047135
53664	IWM	2020-09-29	149.85	150.28	148	149.34	18742080	{}	\N	2020-10-12 15:29:28.055415
53665	IWM	2020-09-30	149.9	151.97	148.49	149.79	29068779	{}	\N	2020-10-12 15:29:28.062655
53666	IWM	2020-10-01	150.81	152.2	149.49	152.18	25725178	{}	\N	2020-10-12 15:29:28.070639
53667	IWM	2020-10-02	149.41	153.58	148.99	152.85	29428825	{}	\N	2020-10-12 15:29:28.076763
53668	IWM	2020-10-05	154.49	157.305	154.48	157.07	18734029	{}	\N	2020-10-12 15:29:28.082253
53669	IWM	2020-10-06	158.5	161.01	156.34	156.62	36314920	{}	\N	2020-10-12 15:29:28.087426
53670	IWM	2020-10-07	158.59	160.56	158.4638	160.04	27065779	{}	\N	2020-10-12 15:29:28.092801
53671	IWM	2020-10-08	161.72	162.28	160.58	161.86	38059982	{}	\N	2020-10-12 15:29:28.0983
53672	IWM	2020-10-09	163.35	163.71	161.805	162.7	24901697	{}	\N	2020-10-12 15:29:28.104397
53673	IWM	2020-10-12	163.4	164.115	162.6	163.8	7074799	{}	\N	2020-10-12 15:29:28.11136
53674	IXG	2020-09-28	52.98	53.48	52.9512	53.28	65921	{}	\N	2020-10-12 15:29:29.166715
53675	IXG	2020-09-29	52.99	53.05	52.375	52.58	35394	{}	\N	2020-10-12 15:29:29.176255
53676	IXG	2020-09-30	52.65	53.27	52.65	52.99	8387	{}	\N	2020-10-12 15:29:29.183186
53677	IXG	2020-10-01	53.17	53.34	52.83	53.14	26787	{}	\N	2020-10-12 15:29:29.188639
53678	IXG	2020-10-02	52.48	53.55	52.48	53.39	5592	{}	\N	2020-10-12 15:29:29.194562
53679	IXG	2020-10-05	53.95	54.4	53.95	54.4	20730	{}	\N	2020-10-12 15:29:29.20054
53680	IXG	2020-10-06	54.77	55.025	53.995	54.07	54569	{}	\N	2020-10-12 15:29:29.205802
53681	IXG	2020-10-07	54.58	54.9	54.474	54.76	7924	{}	\N	2020-10-12 15:29:29.21109
53682	IXG	2020-10-08	55.11	55.385	55	55.37	20980	{}	\N	2020-10-12 15:29:29.216417
53683	IXG	2020-10-09	55.57	55.57	55.309	55.385	6387	{}	\N	2020-10-12 15:29:29.221774
53684	IXG	2020-10-12	55.61	55.77	55.57	55.77	9423	{}	\N	2020-10-12 15:29:29.227321
53685	IYJ	2020-09-28	167.1	168.605	167.08	167.82	22924	{}	\N	2020-10-12 15:29:30.166186
53686	IYJ	2020-09-29	168.12	168.12	166.745	167.13	29015	{}	\N	2020-10-12 15:29:30.173781
53687	IYJ	2020-09-30	167.22	169.2594	166.09	167.15	29088	{}	\N	2020-10-12 15:29:30.180389
53688	IYJ	2020-10-01	168.55	169.0952	166.16	166.82	59288	{}	\N	2020-10-12 15:29:30.186765
53689	IYJ	2020-10-02	163.52	168.52	163.52	167.72	28743	{}	\N	2020-10-12 15:29:30.192139
53690	IYJ	2020-10-05	169.38	170.88	169.38	170.6	43126	{}	\N	2020-10-12 15:29:30.198257
53691	IYJ	2020-10-06	170.86	172.67	168.61	168.75	29018	{}	\N	2020-10-12 15:29:30.204669
53692	IYJ	2020-10-07	170.41	172.44	170.41	172.19	26726	{}	\N	2020-10-12 15:29:30.210656
53693	IYJ	2020-10-08	173.96	173.96	172.18	173.73	325672	{}	\N	2020-10-12 15:29:30.216786
53694	IYJ	2020-10-09	174.48	175.625	174.4	174.86	103856	{}	\N	2020-10-12 15:29:30.223377
53695	IYJ	2020-10-12	175.45	176.28	175.438	176.28	11887	{}	\N	2020-10-12 15:29:30.229149
53696	IYZ	2020-09-28	27.05	27.3363	27.05	27.21	274185	{}	\N	2020-10-12 15:29:31.462189
53697	IYZ	2020-09-29	27.26	27.32	27.13	27.21	327178	{}	\N	2020-10-12 15:29:31.475799
53698	IYZ	2020-09-30	27.33	27.35	27.03	27.2	629701	{}	\N	2020-10-12 15:29:31.484522
53699	IYZ	2020-10-01	27.2	27.31	27.08	27.24	359835	{}	\N	2020-10-12 15:29:31.492231
53700	IYZ	2020-10-02	26.88	27.265	26.81	27.17	515432	{}	\N	2020-10-12 15:29:31.497632
53701	IYZ	2020-10-05	27.3	27.485	27.24	27.47	202226	{}	\N	2020-10-12 15:29:31.502515
53702	IYZ	2020-10-06	27.5	27.81	27.33	27.35	282346	{}	\N	2020-10-12 15:29:31.508761
53703	IYZ	2020-10-07	27.64	27.68	27.48	27.64	139350	{}	\N	2020-10-12 15:29:31.514481
53704	IYZ	2020-10-08	27.8	28	27.8	27.97	70607	{}	\N	2020-10-12 15:29:31.518993
53705	IYZ	2020-10-09	28.01	28.21	28.01	28.13	449026	{}	\N	2020-10-12 15:29:31.523963
53706	IYZ	2020-10-12	28.17	28.18	28.06	28.15	37086	{}	\N	2020-10-12 15:29:31.529526
53707	JBL	2020-09-28	33.9	34.18	33.13	33.89	2327383	{}	\N	2020-10-12 15:29:32.52
53708	JBL	2020-09-29	33.87	33.87	33.08	33.29	2501933	{}	\N	2020-10-12 15:29:32.532381
53709	JBL	2020-09-30	33.44	34.77	33.405	34.26	3295756	{}	\N	2020-10-12 15:29:32.540471
53710	JBL	2020-10-01	34.74	35.15	34.4	34.63	2923590	{}	\N	2020-10-12 15:29:32.546915
53711	JBL	2020-10-02	33.56	34.67	33.48	34.37	2270293	{}	\N	2020-10-12 15:29:32.552681
53712	JBL	2020-10-05	34.9	35.7	34.78	35.41	3299649	{}	\N	2020-10-12 15:29:32.55827
53713	JBL	2020-10-06	35.52	36.05	34.84	34.86	2050593	{}	\N	2020-10-12 15:29:32.563228
53714	JBL	2020-10-07	35.46	36.26	35.41	35.98	2142464	{}	\N	2020-10-12 15:29:32.567671
53715	JBL	2020-10-08	36.28	36.53	35.85	36.4	1874673	{}	\N	2020-10-12 15:29:32.572313
53716	JBL	2020-10-09	37	37.0596	36.34	36.37	1928107	{}	\N	2020-10-12 15:29:32.576538
53717	JBL	2020-10-12	36.63	36.8028	36.281	36.6	770007	{}	\N	2020-10-12 15:29:32.581214
53718	KHC	2020-09-28	29.66	29.99	29.43	29.83	4440788	{}	\N	2020-10-12 15:29:34.660749
53719	KHC	2020-09-29	29.76	29.88	28.5	29.59	4136791	{}	\N	2020-10-12 15:29:34.667695
53720	KHC	2020-09-30	29.61	30.21	29.61	29.95	5395318	{}	\N	2020-10-12 15:29:34.673403
53721	KHC	2020-10-01	30	30.4	29.665	30.37	5656928	{}	\N	2020-10-12 15:29:34.678623
53722	KHC	2020-10-02	29.94	30.58	29.93	30.47	4222368	{}	\N	2020-10-12 15:29:34.684088
53723	KHC	2020-10-05	30.64	31.07	30.62	31	4416445	{}	\N	2020-10-12 15:29:34.689473
53724	KHC	2020-10-06	31.02	31.27	30.5904	30.67	4415693	{}	\N	2020-10-12 15:29:34.694504
53725	KHC	2020-10-07	30.81	31	28.5	30.84	3724104	{}	\N	2020-10-12 15:29:34.699482
53726	KHC	2020-10-08	30.99	31.34	30.9	31.22	4367549	{}	\N	2020-10-12 15:29:34.704918
53727	KHC	2020-10-09	31.29	31.475	31	31.11	3771103	{}	\N	2020-10-12 15:29:34.709606
53728	KHC	2020-10-12	31.43	31.98	31.21	31.98	2234390	{}	\N	2020-10-12 15:29:34.714653
53729	KLDW	2020-09-28	37.7	37.85	37.65	37.785	9976	{}	\N	2020-10-12 15:29:35.686057
53730	KLDW	2020-09-29	37.855	37.9099	37.6586	37.8909	10622	{}	\N	2020-10-12 15:29:35.695241
53731	KLDW	2020-09-30	37.74	37.9598	37.67	37.855	3694	{}	\N	2020-10-12 15:29:35.701477
53732	KLDW	2020-10-01	38.07	38.07	37.835	37.97	4258	{}	\N	2020-10-12 15:29:35.707661
53733	KLDW	2020-10-02	37.7053	37.855	37.6503	37.78	9215	{}	\N	2020-10-12 15:29:35.713044
53734	KLDW	2020-10-05	38.185	38.35	38.16	38.35	6909	{}	\N	2020-10-12 15:29:35.718177
53735	KLDW	2020-10-06	38.32	38.38	37.845	37.845	3879	{}	\N	2020-10-12 15:29:35.722843
53736	KLDW	2020-10-07	38.37	38.488	38.37	38.488	4170	{}	\N	2020-10-12 15:29:35.728067
53737	KLDW	2020-10-08	38.71	38.76	38.675	38.7484	4683	{}	\N	2020-10-12 15:29:35.733569
53738	KLDW	2020-10-09	38.89	39.035	38.88	38.9998	9089	{}	\N	2020-10-12 15:29:35.739394
53739	KLDW	2020-10-12	39.1	39.11	38.99	39.1	1699	{}	\N	2020-10-12 15:29:35.744456
53740	KNL	2020-09-28	12.08	12.53	12.01	12.25	206874	{}	\N	2020-10-12 15:29:37.714346
53741	KNL	2020-09-29	12.23	12.29	11.81	12.14	298917	{}	\N	2020-10-12 15:29:37.72777
53742	KNL	2020-09-30	12.25	12.58	11.95	12.06	443240	{}	\N	2020-10-12 15:29:37.737346
53743	KNL	2020-10-01	12.13	12.44	12.08	12.39	464101	{}	\N	2020-10-12 15:29:37.749298
53744	KNL	2020-10-02	12.08	13.21	11.93	13.12	927226	{}	\N	2020-10-12 15:29:37.757381
53745	KNL	2020-10-05	13.26	13.27	12.65	12.77	513428	{}	\N	2020-10-12 15:29:37.765632
53746	KNL	2020-10-06	12.78	13.005	12.5	12.52	1133932	{}	\N	2020-10-12 15:29:37.771971
53747	KNL	2020-10-07	12.8	13.13	12.53	12.73	876020	{}	\N	2020-10-12 15:29:37.777512
53748	KNL	2020-10-08	12.92	12.95	12.63	12.84	420805	{}	\N	2020-10-12 15:29:37.78295
53749	KNL	2020-10-09	12.99	13.02	12.56	12.59	402834	{}	\N	2020-10-12 15:29:37.788577
53750	KNL	2020-10-12	12.64	12.93	12.51	12.9	104089	{}	\N	2020-10-12 15:29:37.794436
53751	KRYS	2020-09-28	41.67	42.53	40.65	40.8	219417	{}	\N	2020-10-12 15:29:39.202672
53752	KRYS	2020-09-29	40.95	42.39	40.76	42.18	104353	{}	\N	2020-10-12 15:29:39.209312
53753	KRYS	2020-09-30	41.73	43.66	41.595	43.05	69922	{}	\N	2020-10-12 15:29:39.214188
53754	KRYS	2020-10-01	43.48	44.055	42.77	43.75	98289	{}	\N	2020-10-12 15:29:39.21887
53755	KRYS	2020-10-02	43.17	44.04	42.415	43.16	127250	{}	\N	2020-10-12 15:29:39.224595
53756	KRYS	2020-10-05	43.49	46.59	42.605	46.28	140078	{}	\N	2020-10-12 15:29:39.230337
53757	KRYS	2020-10-06	46.59	48.09	45	46.56	77640	{}	\N	2020-10-12 15:29:39.235981
53758	KRYS	2020-10-07	47.24	48.95	47.1	47.99	99677	{}	\N	2020-10-12 15:29:39.241215
53759	KRYS	2020-10-08	48.66	49.15	47.58	48.13	67835	{}	\N	2020-10-12 15:29:39.246627
53760	KRYS	2020-10-09	49.19	50.98	48.5	50.7	84407	{}	\N	2020-10-12 15:29:39.252184
53761	KRYS	2020-10-12	50.81	51.03	49.78	50.51	24627	{}	\N	2020-10-12 15:29:39.257715
53762	L	2020-09-28	34.46	35.19	34.12	34.73	705626	{}	\N	2020-10-12 15:29:40.977896
53763	L	2020-09-29	34.46	34.7	33.83	34.2	1330240	{}	\N	2020-10-12 15:29:40.989848
53764	L	2020-09-30	34.49	35.06	34.33	34.75	1955173	{}	\N	2020-10-12 15:29:40.998447
53765	L	2020-10-01	34.91	35.12	34.37	34.86	1411206	{}	\N	2020-10-12 15:29:41.00454
53766	L	2020-10-02	34.27	35.7976	34	35.5	1012049	{}	\N	2020-10-12 15:29:41.010075
53767	L	2020-10-05	36.09	36.45	35.54	36.15	890281	{}	\N	2020-10-12 15:29:41.014894
53768	L	2020-10-06	36.55	36.59	35.48	35.5	1240549	{}	\N	2020-10-12 15:29:41.020708
53769	L	2020-10-07	35.78	36.26	35.73	36.1	1101211	{}	\N	2020-10-12 15:29:41.024882
53770	L	2020-10-08	36.48	37.06	36.32	36.99	904318	{}	\N	2020-10-12 15:29:41.029555
53771	L	2020-10-09	37.31	37.44	36.67	36.98	963468	{}	\N	2020-10-12 15:29:41.034005
53772	L	2020-10-12	36.86	37.41	36.77	37.3547	209067	{}	\N	2020-10-12 15:29:41.03814
53773	LBAI	2020-09-28	10.01	10.34	10.01	10.19	192993	{}	\N	2020-10-12 15:29:43.407823
53774	LBAI	2020-09-29	10.15	10.15	9.82	9.98	107333	{}	\N	2020-10-12 15:29:43.416509
53775	LBAI	2020-09-30	10.03	10.2	9.89	9.95	174659	{}	\N	2020-10-12 15:29:43.424641
53776	LBAI	2020-10-01	10	10.07	9.79	9.99	107284	{}	\N	2020-10-12 15:29:43.431044
53777	LBAI	2020-10-02	9.79	10.28	9.79	10.23	95661	{}	\N	2020-10-12 15:29:43.437756
53778	LBAI	2020-10-05	10.28	10.56	10.26	10.5	374170	{}	\N	2020-10-12 15:29:43.443609
53779	LBAI	2020-10-06	10.67	10.97	10.51	10.53	167175	{}	\N	2020-10-12 15:29:43.448741
53780	LBAI	2020-10-07	10.95	11	10.66	10.77	199714	{}	\N	2020-10-12 15:29:43.454344
53781	LBAI	2020-10-08	10.9	11	10.77	10.92	152408	{}	\N	2020-10-12 15:29:43.459337
53782	LBAI	2020-10-09	11	11.05	10.55	10.6	142544	{}	\N	2020-10-12 15:29:43.464555
53783	LBAI	2020-10-12	10.55	10.79	10.52	10.74	13441	{}	\N	2020-10-12 15:29:43.469766
53784	LBRT	2020-09-28	7.49	7.775	7.33	7.6	734218	{}	\N	2020-10-12 15:29:44.680019
53785	LBRT	2020-09-29	7.6	7.89	7.37	7.88	1754413	{}	\N	2020-10-12 15:29:44.691948
53786	LBRT	2020-09-30	7.98	8.42	7.78	7.99	2622105	{}	\N	2020-10-12 15:29:44.699679
53787	LBRT	2020-10-01	7.95	8.22	7.665	7.96	1742959	{}	\N	2020-10-12 15:29:44.706229
53788	LBRT	2020-10-02	7.67	8.82	7.6481	8.68	1611601	{}	\N	2020-10-12 15:29:44.711994
53789	LBRT	2020-10-05	9.14	9.61	8.93	9.6	1568572	{}	\N	2020-10-12 15:29:44.716998
53790	LBRT	2020-10-06	9.84	10.53	9.46	9.46	3642458	{}	\N	2020-10-12 15:29:44.721923
53791	LBRT	2020-10-07	8.7	9.085	8.53	8.62	5547755	{}	\N	2020-10-12 15:29:44.726873
53792	LBRT	2020-10-08	8.85	8.93	8.43	8.77	2527926	{}	\N	2020-10-12 15:29:44.732354
53793	LBRT	2020-10-09	8.79	8.99	8.3307	8.35	1437035	{}	\N	2020-10-12 15:29:44.737027
53794	LBRT	2020-10-12	8.3	8.34	8.05	8.12	308682	{}	\N	2020-10-12 15:29:44.742028
53795	LCII	2020-09-28	107.63	109.02	104.26	105.32	244881	{}	\N	2020-10-12 15:29:45.918047
53796	LCII	2020-09-29	105.18	108.08	104.03	104.41	203866	{}	\N	2020-10-12 15:29:45.925893
53797	LCII	2020-09-30	104.37	110.48	104.37	106.29	921861	{}	\N	2020-10-12 15:29:45.933021
53798	LCII	2020-10-01	107.26	111.54	107.15	111.19	511350	{}	\N	2020-10-12 15:29:45.9393
53799	LCII	2020-10-02	109	113.51	109	112.64	165128	{}	\N	2020-10-12 15:29:45.945594
53800	LCII	2020-10-05	113.73	117.49	113.73	117.21	183755	{}	\N	2020-10-12 15:29:45.950938
53801	LCII	2020-10-06	118.66	121	116.3727	116.83	223364	{}	\N	2020-10-12 15:29:45.956846
53802	LCII	2020-10-07	118.13	119.99	117.01	118.99	253679	{}	\N	2020-10-12 15:29:45.962305
53803	LCII	2020-10-08	120.29	120.55	115.68	115.99	120811	{}	\N	2020-10-12 15:29:45.968384
53804	LCII	2020-10-09	116.8	118.17	116.24	117.96	133684	{}	\N	2020-10-12 15:29:45.973878
53805	LCII	2020-10-12	118.11	119.1	117.545	118.47	32478	{}	\N	2020-10-12 15:29:45.98009
53806	LE	2020-09-28	14.73	14.89	14.31	14.47	177036	{}	\N	2020-10-12 15:29:47.18134
53807	LE	2020-09-29	14.53	14.53	13.12	13.29	209792	{}	\N	2020-10-12 15:29:47.192028
53808	LE	2020-09-30	13.43	13.9	12.91	13.03	273625	{}	\N	2020-10-12 15:29:47.200643
53809	LE	2020-10-01	13.33	13.34	12.22	12.75	260263	{}	\N	2020-10-12 15:29:47.207051
53810	LE	2020-10-02	12.37	13.11	12.2004	12.54	166541	{}	\N	2020-10-12 15:29:47.213192
53811	LE	2020-10-05	12.65	13.25	12.6432	12.87	128531	{}	\N	2020-10-12 15:29:47.218399
53812	LE	2020-10-06	12.99	13.4	12.46	12.51	167238	{}	\N	2020-10-12 15:29:47.223579
53813	LE	2020-10-07	12.66	13.84	12.66	13.73	233570	{}	\N	2020-10-12 15:29:47.228882
53814	LE	2020-10-08	13.88	14.36	13.55	13.98	278817	{}	\N	2020-10-12 15:29:47.23399
53815	LE	2020-10-09	14.05	14.78	13.51	13.74	221596	{}	\N	2020-10-12 15:29:47.239238
53816	LE	2020-10-12	13.8	14.25	13.53	14.04	29027	{}	\N	2020-10-12 15:29:47.244563
53817	LEN	2020-09-28	78.19	79.735	77.275	79.64	2278666	{}	\N	2020-10-12 15:29:48.144211
53818	LEN	2020-09-29	79.63	80.34	78.41	78.97	4274951	{}	\N	2020-10-12 15:29:48.15135
53819	LEN	2020-09-30	79.25	82.72	79.13	81.68	9167719	{}	\N	2020-10-12 15:29:48.157405
53820	LEN	2020-10-01	82.46	83.1999	81.3	82.93	6008538	{}	\N	2020-10-12 15:29:48.165848
53821	LEN	2020-10-02	81.1	84.19	80.801	83.86	4242736	{}	\N	2020-10-12 15:29:48.174365
53822	LEN	2020-10-05	84.35	85.57	80.82	83.01	5590750	{}	\N	2020-10-12 15:29:48.185131
53823	LEN	2020-10-06	83.41	83.69	80.02	80.66	4560066	{}	\N	2020-10-12 15:29:48.200122
53824	LEN	2020-10-07	81.84	82.35	79.72	80.43	5184940	{}	\N	2020-10-12 15:29:48.209561
53825	LEN	2020-10-08	81.35	84.04	81.35	84.01	4699759	{}	\N	2020-10-12 15:29:48.216739
53826	LEN	2020-10-09	84.61	85.155	83.38	84.07	4017727	{}	\N	2020-10-12 15:29:48.224169
53827	LEN	2020-10-12	84.45	84.57	83	84.18	764066	{}	\N	2020-10-12 15:29:48.229481
53828	KIM	2020-09-28	11.55	11.75	11.405	11.56	4047588	{}	\N	2020-10-12 15:29:49.449542
53829	KIM	2020-09-29	11.48	11.48	10.96	11.28	9963309	{}	\N	2020-10-12 15:29:49.456347
53830	KIM	2020-09-30	11.37	11.6	11.1	11.26	8256162	{}	\N	2020-10-12 15:29:49.464972
53831	KIM	2020-10-01	11.32	11.65	11.165	11.61	10013892	{}	\N	2020-10-12 15:29:49.474777
53832	KIM	2020-10-02	11.27	11.92	11.2106	11.9	10720885	{}	\N	2020-10-12 15:29:49.482635
53833	KIM	2020-10-05	12.03	12.1	11.655	11.95	8198653	{}	\N	2020-10-12 15:29:49.489465
53834	KIM	2020-10-06	12.11	12.32	11.865	11.95	8419673	{}	\N	2020-10-12 15:29:49.494548
53835	KIM	2020-10-07	12.01	12.1	11.68	11.76	9869482	{}	\N	2020-10-12 15:29:49.500452
53836	KIM	2020-10-08	11.89	12.02	11.78	12.02	10068681	{}	\N	2020-10-12 15:29:49.505871
53837	KIM	2020-10-09	12.13	12.13	11.6	11.62	6433271	{}	\N	2020-10-12 15:29:49.510736
53838	KIM	2020-10-12	11.62	11.655	11.44	11.635	1529457	{}	\N	2020-10-12 15:29:49.516882
53839	KL	2020-09-28	49.13	49.32	47.9	48.69	1368256	{}	\N	2020-10-12 15:29:51.173862
53840	KL	2020-09-29	48.82	49.45	48.64	48.91	2305598	{}	\N	2020-10-12 15:29:51.182739
53841	KL	2020-09-30	48.85	49.4	47.64	48.73	3379676	{}	\N	2020-10-12 15:29:51.190335
53842	KL	2020-10-01	49.33	49.82	48.53	49.32	3502888	{}	\N	2020-10-12 15:29:51.196539
53843	KL	2020-10-02	48.99	49.32	48.25	48.54	2119943	{}	\N	2020-10-12 15:29:51.202198
53844	KL	2020-10-05	48.74	50.03	48.64	48.93	2327752	{}	\N	2020-10-12 15:29:51.208426
53845	KL	2020-10-06	49.3	49.49	46.84	46.91	3786489	{}	\N	2020-10-12 15:29:51.21443
53846	KL	2020-10-07	47.53	48	47.27	47.64	3131389	{}	\N	2020-10-12 15:29:51.220998
53847	KL	2020-10-08	48.01	48.5	47.5944	48.27	2241871	{}	\N	2020-10-12 15:29:51.22692
53848	KL	2020-10-09	49.47	51.32	49.42	51.17	3627595	{}	\N	2020-10-12 15:29:51.233812
53849	KL	2020-10-12	51	51.41	50.6187	50.92	504288	{}	\N	2020-10-12 15:29:51.239923
53850	KMI	2020-09-28	12.58	12.76	12.45	12.61	11308068	{}	\N	2020-10-12 15:29:52.418281
53851	KMI	2020-09-29	12.6	12.715	12.33	12.45	16445158	{}	\N	2020-10-12 15:29:52.427721
53852	KMI	2020-09-30	12.56	12.61	12.27	12.33	36316231	{}	\N	2020-10-12 15:29:52.434283
53853	KMI	2020-10-01	12.2	12.32	12.05	12.24	26624077	{}	\N	2020-10-12 15:29:52.440914
53854	KMI	2020-10-02	12.05	12.44	12.03	12.31	21424393	{}	\N	2020-10-12 15:29:52.446251
53855	KMI	2020-10-05	12.43	12.68	12.305	12.6	22215994	{}	\N	2020-10-12 15:29:52.450946
53856	KMI	2020-10-06	12.73	12.76	12.25	12.27	27760457	{}	\N	2020-10-12 15:29:52.45584
53857	KMI	2020-10-07	12.28	12.67	12.19	12.59	41349036	{}	\N	2020-10-12 15:29:52.460681
53858	KMI	2020-10-08	12.7	13.15	12.595	13.13	30218417	{}	\N	2020-10-12 15:29:52.465428
53859	KMI	2020-10-09	13.21	13.245	12.75	12.82	28739843	{}	\N	2020-10-12 15:29:52.470242
53860	KMI	2020-10-12	12.85	12.88	12.7	12.86	8098513	{}	\N	2020-10-12 15:29:52.474942
53861	KMX	2020-09-28	95.96	96.28	94.44	95.05	1717416	{}	\N	2020-10-12 15:29:53.422812
53862	KMX	2020-09-29	94.71	94.8918	92.65	93.08	4127064	{}	\N	2020-10-12 15:29:53.433583
53863	KMX	2020-09-30	93.03	93.49	91.14	91.91	5337637	{}	\N	2020-10-12 15:29:53.440936
53864	KMX	2020-10-01	92.84	93.53	91.13	93.49	3317412	{}	\N	2020-10-12 15:29:53.450065
53865	KMX	2020-10-02	93.55	95.86	93.03	95.55	4870502	{}	\N	2020-10-12 15:29:53.457714
53866	KMX	2020-10-05	96.95	99.19	96.46	97.91	4475088	{}	\N	2020-10-12 15:29:53.465939
53867	KMX	2020-10-06	98.82	98.8425	93	93.15	3815720	{}	\N	2020-10-12 15:29:53.470956
53868	KMX	2020-10-07	93.86	95.04	93.44	94.1	3000466	{}	\N	2020-10-12 15:29:53.475999
53869	KMX	2020-10-08	94.57	96.1	93.53	95.94	1512958	{}	\N	2020-10-12 15:29:53.481436
53870	KMX	2020-10-09	96.96	97.29	94.65	94.96	4162118	{}	\N	2020-10-12 15:29:53.486973
53871	KMX	2020-10-12	94.98	95.2	93.52	93.92	1140473	{}	\N	2020-10-12 15:29:53.491821
53872	KNOP	2020-09-28	12.39	12.66	12.39	12.56	70547	{}	\N	2020-10-12 15:29:54.965479
53873	KNOP	2020-09-29	12.47	12.57	12.32	12.5	148881	{}	\N	2020-10-12 15:29:54.975836
53874	KNOP	2020-09-30	12.61	12.95	12.61	12.9	304500	{}	\N	2020-10-12 15:29:54.983513
53875	KNOP	2020-10-01	12.9	13	12.715	12.94	150682	{}	\N	2020-10-12 15:29:54.989981
53876	KNOP	2020-10-02	12.76	13.04	12.76	12.97	216193	{}	\N	2020-10-12 15:29:54.996336
53877	KNOP	2020-10-05	13	13.13	12.96	13.12	155011	{}	\N	2020-10-12 15:29:55.001226
53878	KNOP	2020-10-06	13.2	13.2728	12.81	12.89	187198	{}	\N	2020-10-12 15:29:55.00567
53879	KNOP	2020-10-07	13.09	13.22	12.87	13.16	318665	{}	\N	2020-10-12 15:29:55.010527
53880	KNOP	2020-10-08	13.18	13.28	13.03	13.17	231991	{}	\N	2020-10-12 15:29:55.015362
53881	KNOP	2020-10-09	13.3	13.46	13.14	13.3	147295	{}	\N	2020-10-12 15:29:55.020366
53882	KNOP	2020-10-12	13.28	13.37	13.1	13.2399	72400	{}	\N	2020-10-12 15:29:55.024994
53883	KODK	2020-09-28	9.36	9.605	8.95	9.31	7407861	{}	\N	2020-10-12 15:29:56.781832
53884	KODK	2020-09-29	9.12	9.29	8.86	9	10334864	{}	\N	2020-10-12 15:29:56.794387
53885	KODK	2020-09-30	8.85	9.49	8.71	8.82	17236098	{}	\N	2020-10-12 15:29:56.802195
53886	KODK	2020-10-01	8.88	9.1278	8.67	8.82	12583031	{}	\N	2020-10-12 15:29:56.808262
53887	KODK	2020-10-02	8.43	9.05	8.32	8.76	15541952	{}	\N	2020-10-12 15:29:56.813862
53888	KODK	2020-10-05	8.76	9.36	8.58	9.03	20945433	{}	\N	2020-10-12 15:29:56.819181
53889	KODK	2020-10-06	8.97	9.32	8.71	8.75	16586625	{}	\N	2020-10-12 15:29:56.823561
53890	KODK	2020-10-07	8.91	9.06	8.74	8.98	11917754	{}	\N	2020-10-12 15:29:56.827553
53891	KODK	2020-10-08	9	9.085	8.74	8.86	9284613	{}	\N	2020-10-12 15:29:56.831573
53892	KODK	2020-10-09	8.87	10.77	8.85	10.01	78515487	{}	\N	2020-10-12 15:29:56.836151
53893	KODK	2020-10-12	10.03	10.49	9.62	9.745	12626312	{}	\N	2020-10-12 15:29:56.842068
53894	KOIN	2020-09-28	33.53	33.6799	33.52	33.6799	6292	{}	\N	2020-10-12 15:29:57.893087
53895	KOIN	2020-09-29	33.69	33.7687	33.6166	33.6166	1801	{}	\N	2020-10-12 15:29:57.901732
53896	KOIN	2020-09-30	33.562	33.895	33.562	33.89	1040	{}	\N	2020-10-12 15:29:57.908474
53897	KOIN	2020-10-01	34.02	34.02	34.02	34.02	365	{}	\N	2020-10-12 15:29:57.915086
53898	KOIN	2020-10-02	33.49	33.874	33.49	33.61	14288	{}	\N	2020-10-12 15:29:57.920554
53899	KOIN	2020-10-05	34.0374	34.25	33.95	34.25	5120	{}	\N	2020-10-12 15:29:57.925547
53900	KOIN	2020-10-06	34.1725	34.376	33.848	33.848	2354	{}	\N	2020-10-12 15:29:57.930196
53901	KOIN	2020-10-07	34.18	34.5548	34.18	34.5548	2612	{}	\N	2020-10-12 15:29:57.934862
53902	KOIN	2020-10-08	34.83	34.94	34.7801	34.7801	1728	{}	\N	2020-10-12 15:29:57.939788
53903	KOIN	2020-10-09	35.089	35.16	34.94	35.16	8834	{}	\N	2020-10-12 15:29:57.94459
53904	KOIN	2020-10-12	35.36	35.58	35.36	35.4282	30908	{}	\N	2020-10-12 15:29:57.949383
53905	KORP	2020-09-28	52.61	52.61	52.5593	52.585	1264	{}	\N	2020-10-12 15:29:59.154296
53906	KORP	2020-09-29	52.5983	52.6043	52.59	52.59	1667	{}	\N	2020-10-12 15:29:59.160294
53907	KORP	2020-09-30	52.69	52.69	52.5701	52.625	1995	{}	\N	2020-10-12 15:29:59.165537
53908	KORP	2020-10-01	52.52	52.55	52.4849	52.4849	1888	{}	\N	2020-10-12 15:29:59.170751
53909	KORP	2020-10-02	52.5419	52.59	52.535	52.535	2685	{}	\N	2020-10-12 15:29:59.177406
53910	KORP	2020-10-05	52.59	52.62	52.5151	52.62	3827	{}	\N	2020-10-12 15:29:59.184258
53911	KORP	2020-10-06	52.62	52.62	52.51	52.5935	12361	{}	\N	2020-10-12 15:29:59.190685
53912	KORP	2020-10-07	52.61	52.64	52.61	52.6137	4555	{}	\N	2020-10-12 15:29:59.203555
53913	KORP	2020-10-08	52.634	52.72	52.6257	52.7	3373	{}	\N	2020-10-12 15:29:59.212649
53914	KORP	2020-10-09	52.6892	52.73	52.6892	52.73	681	{}	\N	2020-10-12 15:29:59.219997
53915	KORP	2020-10-12	52.6201	52.79	52.6201	52.78	2143	{}	\N	2020-10-12 15:29:59.226647
53916	KPTI	2020-09-28	15.04	15.29	14.62	14.91	1315409	{}	\N	2020-10-12 15:30:00.525654
53917	KPTI	2020-09-29	14.87	15.18	14.44	14.59	894084	{}	\N	2020-10-12 15:30:00.538187
53918	KPTI	2020-09-30	14.62	15.02	14.39	14.6	1107449	{}	\N	2020-10-12 15:30:00.546732
53919	KPTI	2020-10-01	14.63	14.829	14.25	14.49	753061	{}	\N	2020-10-12 15:30:00.554197
53920	KPTI	2020-10-02	14.25	14.65	14.18	14.23	979076	{}	\N	2020-10-12 15:30:00.559788
53921	KPTI	2020-10-05	14.33	15.02	14.24	14.88	797503	{}	\N	2020-10-12 15:30:00.564787
53922	KPTI	2020-10-06	14.95	15.07	14.38	14.46	694623	{}	\N	2020-10-12 15:30:00.570372
53923	KPTI	2020-10-07	14.5	14.73	14.32	14.63	772512	{}	\N	2020-10-12 15:30:00.575951
53924	KPTI	2020-10-08	14.83	14.95	14.425	14.59	691652	{}	\N	2020-10-12 15:30:00.580887
53925	KPTI	2020-10-09	14.79	15.04	14.32	14.83	1145417	{}	\N	2020-10-12 15:30:00.585749
53926	KPTI	2020-10-12	15.16	15.26	14.68	14.75	426143	{}	\N	2020-10-12 15:30:00.590578
53927	KREF	2020-09-28	16.86	17.41	16.86	17.23	315620	{}	\N	2020-10-12 15:30:01.519062
53928	KREF	2020-09-29	16.77	16.8	16.34	16.54	429051	{}	\N	2020-10-12 15:30:01.53176
53929	KREF	2020-09-30	16.47	16.86	16.37	16.53	354480	{}	\N	2020-10-12 15:30:01.539364
53930	KREF	2020-10-01	16.62	16.7	16.33	16.7	304377	{}	\N	2020-10-12 15:30:01.54536
53931	KREF	2020-10-02	16.24	16.94	16.23	16.9	367638	{}	\N	2020-10-12 15:30:01.551545
53932	KREF	2020-10-05	17.04	17.18	16.85	17.01	317183	{}	\N	2020-10-12 15:30:01.557585
53933	KREF	2020-10-06	17.14	17.28	16.57	16.66	390201	{}	\N	2020-10-12 15:30:01.56332
53934	KREF	2020-10-07	16.81	16.84	16.43	16.67	493015	{}	\N	2020-10-12 15:30:01.569416
53935	KREF	2020-10-08	16.81	17.4	16.81	17.35	354510	{}	\N	2020-10-12 15:30:01.576908
53936	KREF	2020-10-09	17.54	17.705	16.91	17.28	308424	{}	\N	2020-10-12 15:30:01.583498
53937	KREF	2020-10-12	17.24	17.55	17.24	17.44	77974	{}	\N	2020-10-12 15:30:01.589753
53938	KSU	2020-09-28	185.25	186.52	182.095	184.88	505312	{}	\N	2020-10-12 15:30:04.047202
53939	KSU	2020-09-29	185.41	185.81	182.73	182.77	891997	{}	\N	2020-10-12 15:30:04.054701
53940	KSU	2020-09-30	184.15	185.28	180.08	180.83	1695720	{}	\N	2020-10-12 15:30:04.060591
53941	KSU	2020-10-01	182.24	183.93	178.5	179.45	1640608	{}	\N	2020-10-12 15:30:04.068934
53942	KSU	2020-10-02	177.57	183.705	176.53	180.92	2008176	{}	\N	2020-10-12 15:30:04.07872
53943	KSU	2020-10-05	182.26	183.105	179.59	181.17	2115915	{}	\N	2020-10-12 15:30:04.08466
53944	KSU	2020-10-06	182.95	182.95	178.95	179.6	1966271	{}	\N	2020-10-12 15:30:04.091899
53945	KSU	2020-10-07	180.36	184.81	180.22	183.38	1389482	{}	\N	2020-10-12 15:30:04.098511
53946	KSU	2020-10-08	183.82	185.25	183.57	184.06	1046685	{}	\N	2020-10-12 15:30:04.105757
53947	KSU	2020-10-09	185.71	187.15	183.67	186.64	1433160	{}	\N	2020-10-12 15:30:04.112194
53948	KSU	2020-10-12	186.91	188.61	185.9	186.67	255441	{}	\N	2020-10-12 15:30:04.118731
53949	KT	2020-09-28	9.57	9.625	9.55	9.6	430527	{}	\N	2020-10-12 15:30:05.080382
53950	KT	2020-09-29	9.71	9.79	9.67	9.69	640805	{}	\N	2020-10-12 15:30:05.088098
53951	KT	2020-09-30	9.72	9.755	9.61	9.61	1424760	{}	\N	2020-10-12 15:30:05.096393
53952	KT	2020-10-01	9.66	9.71	9.58	9.65	1278815	{}	\N	2020-10-12 15:30:05.10502
53953	KT	2020-10-02	9.51	9.685	9.49	9.65	714776	{}	\N	2020-10-12 15:30:05.111622
53954	KT	2020-10-05	9.8	9.86	9.74	9.8	1689044	{}	\N	2020-10-12 15:30:05.117039
53955	KT	2020-10-06	9.79	9.86	9.73	9.73	729616	{}	\N	2020-10-12 15:30:05.123316
53956	KT	2020-10-07	9.81	9.87	9.76	9.81	735995	{}	\N	2020-10-12 15:30:05.129046
53957	KT	2020-10-08	9.85	9.86	9.81	9.85	937118	{}	\N	2020-10-12 15:30:05.136558
53958	KT	2020-10-09	9.89	9.97	9.89	9.96	552206	{}	\N	2020-10-12 15:30:05.143752
53959	KT	2020-10-12	9.92	9.99	9.92	9.98	215286	{}	\N	2020-10-12 15:30:05.150114
53960	KW	2020-09-28	14.71	14.93	14.71	14.79	922072	{}	\N	2020-10-12 15:30:07.645165
53961	KW	2020-09-29	14.46	14.73	14.38	14.57	1375873	{}	\N	2020-10-12 15:30:07.653763
53962	KW	2020-09-30	14.63	14.95	14.48	14.52	1050875	{}	\N	2020-10-12 15:30:07.659654
53963	KW	2020-10-01	14.48	14.865	14.48	14.75	1211787	{}	\N	2020-10-12 15:30:07.665674
53964	KW	2020-10-02	14.42	15.16	14.42	15.01	1560119	{}	\N	2020-10-12 15:30:07.672462
53965	KW	2020-10-05	15.17	15.32	14.9	15.1	5180460	{}	\N	2020-10-12 15:30:07.679499
53966	KW	2020-10-06	15.26	15.26	14.61	14.62	1596689	{}	\N	2020-10-12 15:30:07.688679
53967	KW	2020-10-07	14.71	14.93	14.53	14.62	1469750	{}	\N	2020-10-12 15:30:07.693908
53968	KW	2020-10-08	14.75	14.93	14.62	14.91	1394501	{}	\N	2020-10-12 15:30:07.699309
53969	KW	2020-10-09	14.98	15.02	14.655	14.68	768350	{}	\N	2020-10-12 15:30:07.704483
53970	KW	2020-10-12	14.62	14.77	14.47	14.73	96467	{}	\N	2020-10-12 15:30:07.709857
53971	KIN	2020-09-28	4.55	4.66	4.46	4.49	141622	{}	\N	2020-10-12 15:30:08.705989
53972	KIN	2020-09-29	4.42	4.51	4.27	4.39	153994	{}	\N	2020-10-12 15:30:08.716823
53973	KIN	2020-09-30	4.39	4.59	4.26	4.29	228084	{}	\N	2020-10-12 15:30:08.72413
53974	KIN	2020-10-01	4.38	4.46	4.21	4.36	268172	{}	\N	2020-10-12 15:30:08.730477
53975	KIN	2020-10-02	4.15	4.41	4.14	4.26	333156	{}	\N	2020-10-12 15:30:08.736853
53976	KIN	2020-10-05	4.28	4.56	4.28	4.5	221869	{}	\N	2020-10-12 15:30:08.743443
53977	KIN	2020-10-06	4.53	4.68	4.46	4.47	251275	{}	\N	2020-10-12 15:30:08.749519
53978	KIN	2020-10-07	4.78	5.05	4.67	4.84	632396	{}	\N	2020-10-12 15:30:08.755683
53979	KIN	2020-10-08	5.01	5.05	4.84	4.95	407856	{}	\N	2020-10-12 15:30:08.761305
53980	KIN	2020-10-09	4.97	5.02	4.75	4.85	219709	{}	\N	2020-10-12 15:30:08.766802
53981	KIN	2020-10-12	4.85	4.85	4.7252	4.76	73734	{}	\N	2020-10-12 15:30:08.771868
53982	KLAC	2020-09-28	187.77	193.68	186.8	193.25	1265697	{}	\N	2020-10-12 15:30:10.48308
53983	KLAC	2020-09-29	193.26	195.81	192.6	193.33	692195	{}	\N	2020-10-12 15:30:10.496753
53984	KLAC	2020-09-30	192.67	196.45	191.05	193.74	913353	{}	\N	2020-10-12 15:30:10.503945
53985	KLAC	2020-10-01	197.62	200.21	195.66	198.98	844009	{}	\N	2020-10-12 15:30:10.510321
53986	KLAC	2020-10-02	192.09	196.32	190.8	191.96	997467	{}	\N	2020-10-12 15:30:10.516391
53987	KLAC	2020-10-05	193.28	200	192.72	199.95	1210157	{}	\N	2020-10-12 15:30:10.521459
53988	KLAC	2020-10-06	199.89	205.83	198.68	199.83	1181880	{}	\N	2020-10-12 15:30:10.527087
53989	KLAC	2020-10-07	203	205.69	201.11	202.9	1123077	{}	\N	2020-10-12 15:30:10.531735
53990	KLAC	2020-10-08	205.49	206.95	204.47	206.69	663741	{}	\N	2020-10-12 15:30:10.536074
53991	KLAC	2020-10-09	209.98	212.11	208.79	211.04	1125720	{}	\N	2020-10-12 15:30:10.540763
53992	KLAC	2020-10-12	213.96	214.92	212.6	213.93	374415	{}	\N	2020-10-12 15:30:10.5456
53993	KN	2020-09-28	14.86	15.08	14.76	15.04	895959	{}	\N	2020-10-12 15:30:13.049265
53994	KN	2020-09-29	15.1	15.28	14.905	15.11	761697	{}	\N	2020-10-12 15:30:13.063449
53995	KN	2020-09-30	15.19	15.27	14.8	14.9	1287773	{}	\N	2020-10-12 15:30:13.073422
53996	KN	2020-10-01	15.04	15.24	14.97	15.14	892832	{}	\N	2020-10-12 15:30:13.080996
53997	KN	2020-10-02	14.86	15.2	14.82	15.07	722645	{}	\N	2020-10-12 15:30:13.08757
53998	KN	2020-10-05	15.13	15.44	15.105	15.26	663824	{}	\N	2020-10-12 15:30:13.094224
53999	KN	2020-10-06	15.33	15.47	15.05	15.12	1115538	{}	\N	2020-10-12 15:30:13.101902
54000	KN	2020-10-07	15.23	15.36	15.115	15.23	787054	{}	\N	2020-10-12 15:30:13.109993
54001	KN	2020-10-08	15.42	15.54	15.28	15.44	690299	{}	\N	2020-10-12 15:30:13.118305
54002	KN	2020-10-09	15.49	15.95	15.44	15.9	985344	{}	\N	2020-10-12 15:30:13.124909
54003	KN	2020-10-12	16	16.32	15.97	16.27	465872	{}	\N	2020-10-12 15:30:13.133188
54004	KNSL	2020-09-28	177.68	183.67	177.68	182.77	131554	{}	\N	2020-10-12 15:30:14.181477
54005	KNSL	2020-09-29	182.63	186.53	182.52	186.28	109952	{}	\N	2020-10-12 15:30:14.191816
54006	KNSL	2020-09-30	186.28	191.91	186.28	190.18	278299	{}	\N	2020-10-12 15:30:14.198946
54007	KNSL	2020-10-01	191.01	192.05	187.39	189.65	139637	{}	\N	2020-10-12 15:30:14.205454
54008	KNSL	2020-10-02	186.39	199.1099	185	195.9	168582	{}	\N	2020-10-12 15:30:14.210979
54009	KNSL	2020-10-05	198.79	203.62	197.7	203.08	105662	{}	\N	2020-10-12 15:30:14.216088
54010	KNSL	2020-10-06	204.5	205.86	200	201.38	136365	{}	\N	2020-10-12 15:30:14.222796
54011	KNSL	2020-10-07	204.685	208.47	203.42	207.28	146092	{}	\N	2020-10-12 15:30:14.228518
54012	KNSL	2020-10-08	209.22	210.42	205.27	205.73	70657	{}	\N	2020-10-12 15:30:14.233409
54013	KNSL	2020-10-09	206.46	209.72	206.44	208.2	63232	{}	\N	2020-10-12 15:30:14.239835
54014	KNSL	2020-10-12	208.82	214.35	208.82	212.17	23834	{}	\N	2020-10-12 15:30:14.245735
54015	KOL	2020-09-28	78.19	78.2799	78.1853	78.1853	1908	{}	\N	2020-10-12 15:30:16.305122
54016	KOL	2020-09-29	77.8	78.28	77.785	78.2023	2060	{}	\N	2020-10-12 15:30:16.312153
54017	KOL	2020-09-30	78.408	78.44	77.91	77.91	1294	{}	\N	2020-10-12 15:30:16.31734
54018	KOL	2020-10-01	78.31	78.83	78.28	78.765	1554	{}	\N	2020-10-12 15:30:16.323219
54019	KOL	2020-10-02	77.25	78.86	77.25	78.4063	2771	{}	\N	2020-10-12 15:30:16.327836
54020	KOL	2020-10-05	78.21	78.575	78.21	78.385	4089	{}	\N	2020-10-12 15:30:16.332308
54021	KOL	2020-10-06	79.09	79.09	77.64	77.64	2013	{}	\N	2020-10-12 15:30:16.336768
54022	KOL	2020-10-07	78.0675	78.093	78.0675	78.093	595	{}	\N	2020-10-12 15:30:16.341797
54023	KOL	2020-10-08	78.24	78.24	78.05	78.1	1605	{}	\N	2020-10-12 15:30:16.345827
54024	KOL	2020-10-09	78.67	78.67	78.4866	78.49	2065	{}	\N	2020-10-12 15:30:16.350552
54025	KOL	2020-10-12	78.33	78.33	78.08	78.1	1194	{}	\N	2020-10-12 15:30:16.355702
54026	KOPN	2020-09-28	1.3	1.3888	1.29	1.32	463594	{}	\N	2020-10-12 15:30:17.912983
54027	KOPN	2020-09-29	1.32	1.41	1.3	1.41	476880	{}	\N	2020-10-12 15:30:17.92414
54028	KOPN	2020-09-30	1.42	1.44	1.34	1.38	233511	{}	\N	2020-10-12 15:30:17.931316
54029	KOPN	2020-10-01	1.385	1.41	1.3672	1.39	209722	{}	\N	2020-10-12 15:30:17.937401
54030	KOPN	2020-10-02	1.33	1.41	1.3	1.365	194414	{}	\N	2020-10-12 15:30:17.942612
54031	KOPN	2020-10-05	1.4	1.43	1.33	1.35	226961	{}	\N	2020-10-12 15:30:17.94759
54032	KOPN	2020-10-06	1.35	1.39	1.28	1.28	326036	{}	\N	2020-10-12 15:30:17.952215
54033	KOPN	2020-10-07	1.31	1.43	1.31	1.38	698811	{}	\N	2020-10-12 15:30:17.956576
54034	KOPN	2020-10-08	1.42	1.47	1.41	1.44	394020	{}	\N	2020-10-12 15:30:17.963721
54035	KOPN	2020-10-09	1.46	1.49	1.44	1.46	327542	{}	\N	2020-10-12 15:30:17.971874
54036	KOPN	2020-10-12	1.48	1.48	1.445	1.4561	108715	{}	\N	2020-10-12 15:30:17.980038
54037	KORU	2020-09-28	16.41	16.49	16.13	16.43	114078	{}	\N	2020-10-12 15:30:18.942639
54038	KORU	2020-09-29	16.43	16.73	16.3852	16.6	90707	{}	\N	2020-10-12 15:30:18.953994
54039	KORU	2020-09-30	16.92	17.38	16.88	17.31	63598	{}	\N	2020-10-12 15:30:18.961638
54040	KORU	2020-10-01	17.65	17.73	17.37	17.67	91347	{}	\N	2020-10-12 15:30:18.967742
54041	KORU	2020-10-02	16.75	17.41	16.645	17.1	67450	{}	\N	2020-10-12 15:30:18.973666
54042	KORU	2020-10-05	17.7	18.2	17.7	18.17	73781	{}	\N	2020-10-12 15:30:18.978728
54043	KORU	2020-10-06	17.84	18.11	17.45	17.55	67249	{}	\N	2020-10-12 15:30:18.983883
54044	KORU	2020-10-07	18.46	18.84	18.46	18.73	101636	{}	\N	2020-10-12 15:30:18.989315
54045	KORU	2020-10-08	18.86	19.1134	18.7	19.05	52178	{}	\N	2020-10-12 15:30:18.994691
54046	KORU	2020-10-09	19.5	19.69	19.3	19.56	81013	{}	\N	2020-10-12 15:30:18.999891
54047	KORU	2020-10-12	19.19	19.34	19.051	19.28	74058	{}	\N	2020-10-12 15:30:19.005025
54048	KRA	2020-09-28	16.43	17.56	16.31	17.1	300297	{}	\N	2020-10-12 15:30:19.935617
54049	KRA	2020-09-29	17.11	17.69	16.94	17.5	513088	{}	\N	2020-10-12 15:30:19.946651
54050	KRA	2020-09-30	17.73	18.5	17.54	17.82	796095	{}	\N	2020-10-12 15:30:19.953984
54051	KRA	2020-10-01	18.02	18.125	17.37	17.64	610493	{}	\N	2020-10-12 15:30:19.960761
54052	KRA	2020-10-02	16.5	19.1	16.5	19.01	538256	{}	\N	2020-10-12 15:30:19.966756
54053	KRA	2020-10-05	19.25	20.605	19.13	20.32	1188384	{}	\N	2020-10-12 15:30:19.971995
54054	KRA	2020-10-06	21.1	21.9	20.88	21.25	1217427	{}	\N	2020-10-12 15:30:19.976783
54055	KRA	2020-10-07	21.59	23.3	21.25	23.22	1057221	{}	\N	2020-10-12 15:30:19.981452
54056	KRA	2020-10-08	23.83	25.18	23.13	24.94	1384005	{}	\N	2020-10-12 15:30:19.986375
54057	KRA	2020-10-09	24.51	25.44	24.51	25.33	1001272	{}	\N	2020-10-12 15:30:19.991127
54058	KRA	2020-10-12	25.45	25.715	25.12	25.58	114411	{}	\N	2020-10-12 15:30:19.995832
54059	KRE	2020-09-28	35.08	36.04	35.08	35.78	6096881	{}	\N	2020-10-12 15:30:20.944902
54060	KRE	2020-09-29	35.71	35.71	34.72	35.29	4366707	{}	\N	2020-10-12 15:30:20.955876
54061	KRE	2020-09-30	35.37	36.16	35.36	35.68	7737311	{}	\N	2020-10-12 15:30:20.963067
54062	KRE	2020-10-01	35.88	36.16	35.38	35.92	8529236	{}	\N	2020-10-12 15:30:20.969016
54063	KRE	2020-10-02	35.17	37.12	35.17	36.94	8624761	{}	\N	2020-10-12 15:30:20.974896
54064	KRE	2020-10-05	37.39	38.53	37.39	38.48	13832645	{}	\N	2020-10-12 15:30:20.979462
54065	KRE	2020-10-06	38.9	40.01	38.23	38.41	15886759	{}	\N	2020-10-12 15:30:20.983763
54066	KRE	2020-10-07	38.98	39.98	38.96	39.5	8714935	{}	\N	2020-10-12 15:30:20.988341
54067	KRE	2020-10-08	39.8	40.255	39.52	40.11	7656403	{}	\N	2020-10-12 15:30:20.992973
54068	KRE	2020-10-09	40.36	40.575	39.47	39.66	12858760	{}	\N	2020-10-12 15:30:20.997429
54069	KRE	2020-10-12	39.67	40.05	39.49	39.95	2312192	{}	\N	2020-10-12 15:30:21.00209
54070	KRP	2020-09-28	6.3	6.38	6.15	6.16	129174	{}	\N	2020-10-12 15:30:23.164868
54071	KRP	2020-09-29	6.19	6.19	6.03	6.04	168548	{}	\N	2020-10-12 15:30:23.176461
54072	KRP	2020-09-30	6.24	6.2436	5.95	6.11	247498	{}	\N	2020-10-12 15:30:23.183255
54073	KRP	2020-10-01	6.04	6.13	5.57	5.71	480327	{}	\N	2020-10-12 15:30:23.192031
54074	KRP	2020-10-02	5.57	6.327	5.57	6.28	247346	{}	\N	2020-10-12 15:30:23.196975
54075	KRP	2020-10-05	6.42	6.73	6.19	6.47	271782	{}	\N	2020-10-12 15:30:23.201846
54076	KRP	2020-10-06	6.63	6.6602	6.1	6.17	187052	{}	\N	2020-10-12 15:30:23.20722
54077	KRP	2020-10-07	6.24	6.28	5.9511	6.26	136525	{}	\N	2020-10-12 15:30:23.21224
54078	KRP	2020-10-08	6.24	6.63	6.1646	6.59	153132	{}	\N	2020-10-12 15:30:23.216775
54079	KRP	2020-10-09	6.57	6.63	6.2	6.26	135586	{}	\N	2020-10-12 15:30:23.221903
54080	KRP	2020-10-12	6.24	6.33	6.11	6.255	120846	{}	\N	2020-10-12 15:30:23.229168
54081	KSM	2020-09-28	10.82	10.83	10.72	10.8	23885	{}	\N	2020-10-12 15:30:24.312134
54082	KSM	2020-09-29	10.84	10.85	10.82	10.85	32706	{}	\N	2020-10-12 15:30:24.322194
54083	KSM	2020-09-30	10.9	10.91	10.83	10.85	35636	{}	\N	2020-10-12 15:30:24.330858
54084	KSM	2020-10-01	10.86	10.91	10.82	10.83	29954	{}	\N	2020-10-12 15:30:24.339062
54085	KSM	2020-10-02	10.88	10.88	10.82	10.88	33423	{}	\N	2020-10-12 15:30:24.346749
54086	KSM	2020-10-05	10.8	10.86	10.77	10.77	68773	{}	\N	2020-10-12 15:30:24.352446
54087	KSM	2020-10-06	10.82	10.835	10.71	10.71	51110	{}	\N	2020-10-12 15:30:24.358944
54088	KSM	2020-10-07	10.77	10.81	10.71	10.76	26034	{}	\N	2020-10-12 15:30:24.365666
54089	KSM	2020-10-08	10.75	10.8	10.73	10.76	64938	{}	\N	2020-10-12 15:30:24.370636
54090	KSM	2020-10-09	10.78	10.8202	10.71	10.71	56447	{}	\N	2020-10-12 15:30:24.376686
54091	KSM	2020-10-12	10.7	10.8	10.7	10.75	16142	{}	\N	2020-10-12 15:30:24.382157
54092	KTCC	2020-09-28	9.02	9.9599	8.99	9.58	97687	{}	\N	2020-10-12 15:30:26.985947
54093	KTCC	2020-09-29	9.8	10.1	9.56	9.91	73907	{}	\N	2020-10-12 15:30:26.99643
54094	KTCC	2020-09-30	9.82	9.9661	9.65	9.85	32692	{}	\N	2020-10-12 15:30:27.004513
54095	KTCC	2020-10-01	9.9999	10.03	9.9	10	80606	{}	\N	2020-10-12 15:30:27.01091
54096	KTCC	2020-10-02	10	10.05	9.69	9.77	96493	{}	\N	2020-10-12 15:30:27.016578
54097	KTCC	2020-10-05	9.76	10.25	9.76	10.04	68747	{}	\N	2020-10-12 15:30:27.021433
54098	KTCC	2020-10-06	10.25	10.48	9.62	9.9	43401	{}	\N	2020-10-12 15:30:27.025728
54099	KTCC	2020-10-07	9.87	9.87	9.35	9.45	33146	{}	\N	2020-10-12 15:30:27.030261
54100	KTCC	2020-10-08	9.45	9.5812	9.16	9.29	21571	{}	\N	2020-10-12 15:30:27.035322
54101	KTCC	2020-10-09	9.33	9.8615	9.3	9.59	13282	{}	\N	2020-10-12 15:30:27.039904
54102	KTCC	2020-10-12	9.7	9.82	9.355	9.355	9740	{}	\N	2020-10-12 15:30:27.04443
54103	KMDA	2020-09-28	8.42	8.59	7.96	8.14	228231	{}	\N	2020-10-12 15:30:27.959946
54104	KMDA	2020-09-29	8.12	8.14	7.91	8.02	170531	{}	\N	2020-10-12 15:30:27.97205
54105	KMDA	2020-09-30	8.33	8.77	8.24	8.36	277109	{}	\N	2020-10-12 15:30:27.980115
54106	KMDA	2020-10-01	8.215	8.25	8	8.08	144732	{}	\N	2020-10-12 15:30:27.986631
54107	KMDA	2020-10-02	8.2	8.2738	7.96	8.01	156684	{}	\N	2020-10-12 15:30:27.992269
54108	KMDA	2020-10-05	8.12	8.35	8.11	8.27	89782	{}	\N	2020-10-12 15:30:27.997392
54109	KMDA	2020-10-06	8.32	8.45	8.09	8.13	95756	{}	\N	2020-10-12 15:30:28.002664
54110	KMDA	2020-10-07	8.06	8.0745	7.9	7.96	175983	{}	\N	2020-10-12 15:30:28.007479
54111	KMDA	2020-10-08	8	8.1	7.92	8.03	102371	{}	\N	2020-10-12 15:30:28.012534
54112	KMDA	2020-10-09	8.03	8.1	7.8738	8.09	128978	{}	\N	2020-10-12 15:30:28.017755
54113	KMDA	2020-10-12	8.02	8.02	7.82	7.93	90203	{}	\N	2020-10-12 15:30:28.022748
54114	KMT	2020-09-28	28.75	29.84	28.75	29.57	627645	{}	\N	2020-10-12 15:30:29.70566
54115	KMT	2020-09-29	29.64	29.64	28.74	29.04	543867	{}	\N	2020-10-12 15:30:29.711383
54116	KMT	2020-09-30	29.36	29.8	28.58	28.94	1196049	{}	\N	2020-10-12 15:30:29.717617
54117	KMT	2020-10-01	29.02	29.3	28.45	28.83	1142708	{}	\N	2020-10-12 15:30:29.72336
54118	KMT	2020-10-02	28.09	30.24	28.09	29.78	1398491	{}	\N	2020-10-12 15:30:29.729556
54119	KMT	2020-10-05	30.35	31.89	30.21	31.83	973615	{}	\N	2020-10-12 15:30:29.737001
54120	KMT	2020-10-06	32	33.74	31.75	32.65	1638925	{}	\N	2020-10-12 15:30:29.744014
54121	KMT	2020-10-07	33.09	34.135	32.69	33.75	1145634	{}	\N	2020-10-12 15:30:29.749398
54122	KMT	2020-10-08	34	34	33.11	33.92	913366	{}	\N	2020-10-12 15:30:29.7551
54123	KMT	2020-10-09	34.14	34.625	33.79	34.14	1062570	{}	\N	2020-10-12 15:30:29.760416
54124	KMT	2020-10-12	34.35	34.695	34.09	34.655	289305	{}	\N	2020-10-12 15:30:29.765856
54125	KO	2020-09-28	49.31	49.71	49.14	49.28	11424535	{}	\N	2020-10-12 15:30:30.716267
54126	KO	2020-09-29	49.39	49.555	48.91	48.92	24360908	{}	\N	2020-10-12 15:30:30.725933
54127	KO	2020-09-30	49.27	49.62	48.8916	49.37	30901387	{}	\N	2020-10-12 15:30:30.734954
54128	KO	2020-10-01	49.44	49.6577	48.59	49.18	32516779	{}	\N	2020-10-12 15:30:30.75565
54129	KO	2020-10-02	48.61	49.58	48.55	49.36	24030804	{}	\N	2020-10-12 15:30:30.778209
54130	KO	2020-10-05	49.43	49.55	49.12	49.38	20542043	{}	\N	2020-10-12 15:30:30.791188
54131	KO	2020-10-06	49.55	49.725	48.83	48.94	17562304	{}	\N	2020-10-12 15:30:30.801907
54132	KO	2020-10-07	49.02	49.76	48.95	49.56	19441212	{}	\N	2020-10-12 15:30:30.809967
54133	KO	2020-10-08	49.71	50.54	49.535	50.46	24121114	{}	\N	2020-10-12 15:30:30.816713
54134	KO	2020-10-09	50.67	51.23	50.6	50.81	19421944	{}	\N	2020-10-12 15:30:30.826802
54135	KO	2020-10-12	50.84	51.43	50.83	51.39	6774684	{}	\N	2020-10-12 15:30:30.834675
54136	KR	2020-09-28	33.5	33.725	33.33	33.7	5088229	{}	\N	2020-10-12 15:30:31.963764
54137	KR	2020-09-29	33.43	33.98	33.29	33.77	12794231	{}	\N	2020-10-12 15:30:31.974873
54138	KR	2020-09-30	33.7	34.53	33.445	33.91	25357051	{}	\N	2020-10-12 15:30:31.983431
54139	KR	2020-10-01	33.95	34.23	33.52	34.02	16284217	{}	\N	2020-10-12 15:30:31.989549
54140	KR	2020-10-02	33.75	34.335	33.7	34.07	9996186	{}	\N	2020-10-12 15:30:31.995501
54141	KR	2020-10-05	34.1	34.57	34.1	34.53	10433018	{}	\N	2020-10-12 15:30:32.000568
54142	KR	2020-10-06	34.61	34.97	33.835	33.95	14020455	{}	\N	2020-10-12 15:30:32.005375
54143	KR	2020-10-07	34.06	34.425	34	34.03	11793869	{}	\N	2020-10-12 15:30:32.010475
54144	KR	2020-10-08	34.22	34.685	34.15	34.29	8730172	{}	\N	2020-10-12 15:30:32.015131
54145	KR	2020-10-09	34.5	34.76	34.37	34.45	12280178	{}	\N	2020-10-12 15:30:32.020097
54146	KR	2020-10-12	34.57	34.92	34.49	34.875	3112839	{}	\N	2020-10-12 15:30:32.024827
54147	KRG	2020-09-28	11.1	11.79	11.1	11.65	390585	{}	\N	2020-10-12 15:30:32.864242
54148	KRG	2020-09-29	11.64	11.66	11.23	11.55	734574	{}	\N	2020-10-12 15:30:32.876836
54149	KRG	2020-09-30	11.64	11.97	11.465	11.58	1316625	{}	\N	2020-10-12 15:30:32.885671
54150	KRG	2020-10-01	11.53	11.94	11.52	11.94	790709	{}	\N	2020-10-12 15:30:32.893521
54151	KRG	2020-10-02	11.5	12.27	11.33	12.19	717044	{}	\N	2020-10-12 15:30:32.8981
54152	KRG	2020-10-05	12.36	12.49	11.95	12.06	813532	{}	\N	2020-10-12 15:30:32.902228
54153	KRG	2020-10-06	12.24	12.52	12.02	12.14	866373	{}	\N	2020-10-12 15:30:32.906138
54154	KRG	2020-10-07	12.37	12.41	11.97	12.04	1418463	{}	\N	2020-10-12 15:30:32.879542
54155	KRG	2020-10-08	12.15	12.27	12.048	12.22	807655	{}	\N	2020-10-12 15:30:32.886019
54156	KRG	2020-10-09	12.39	12.42	11.84	11.91	767609	{}	\N	2020-10-12 15:30:32.890959
54157	KRG	2020-10-12	11.9	12.065	11.74	11.93	70318	{}	\N	2020-10-12 15:30:32.895991
54158	KSS	2020-09-28	19.95	20.27	19.45	19.48	6653825	{}	\N	2020-10-12 15:30:34.410396
54159	KSS	2020-09-29	19.65	19.65	18.28	18.41	25846801	{}	\N	2020-10-12 15:30:34.420769
54160	KSS	2020-09-30	18.46	19.03	18.435	18.53	18383461	{}	\N	2020-10-12 15:30:34.429109
54161	KSS	2020-10-01	18.56	19.625	18.3601	19.46	18019019	{}	\N	2020-10-12 15:30:34.44399
54162	KSS	2020-10-02	18.54	20.06	18.5	19.96	20243159	{}	\N	2020-10-12 15:30:34.458473
54163	KSS	2020-10-05	20.33	20.75	20.05	20.42	12511441	{}	\N	2020-10-12 15:30:34.477981
54164	KSS	2020-10-06	20.75	20.75	19.36	19.49	13743429	{}	\N	2020-10-12 15:30:34.498767
54165	KSS	2020-10-07	19.84	20.735	19.83	20.64	12984898	{}	\N	2020-10-12 15:30:34.513704
54166	KSS	2020-10-08	20.87	21.9	20.72	21.89	16137126	{}	\N	2020-10-12 15:30:34.538445
54167	KSS	2020-10-09	22	22.24	21.425	21.53	13098304	{}	\N	2020-10-12 15:30:34.548135
54168	KSS	2020-10-12	21.75	22.39	21.62	21.67	5147136	{}	\N	2020-10-12 15:30:34.558903
54169	KURA	2020-09-28	30.18	30.49	29.62	30.07	473013	{}	\N	2020-10-12 15:30:35.654865
54170	KURA	2020-09-29	30.01	30.7	29.68	30.55	378878	{}	\N	2020-10-12 15:30:35.665441
54171	KURA	2020-09-30	30.94	31.32	30.341	30.64	360715	{}	\N	2020-10-12 15:30:35.673895
54172	KURA	2020-10-01	30.64	30.87	29.7018	30.71	550524	{}	\N	2020-10-12 15:30:35.681533
54173	KURA	2020-10-02	30.06	30.8	29.9	30	390252	{}	\N	2020-10-12 15:30:35.687669
54174	KURA	2020-10-05	30.12	31.47	30.12	30.96	439281	{}	\N	2020-10-12 15:30:35.694189
54175	KURA	2020-10-06	31	32	30.72	31.14	682620	{}	\N	2020-10-12 15:30:35.699817
54176	KURA	2020-10-07	31.46	34	31.46	33.56	1170105	{}	\N	2020-10-12 15:30:35.706391
54177	KURA	2020-10-08	34	34.25	32.62	33.02	900547	{}	\N	2020-10-12 15:30:35.712423
54178	KURA	2020-10-09	33.35	33.84	32.31	32.97	426560	{}	\N	2020-10-12 15:30:35.718015
54179	KURA	2020-10-12	34.33	34.33	32.52	33.4	131459	{}	\N	2020-10-12 15:30:35.724354
54180	KWR	2020-09-28	173.97	179.82	173.97	179.02	139118	{}	\N	2020-10-12 15:30:36.604294
54181	KWR	2020-09-29	178.96	180.29	176.31	176.36	107114	{}	\N	2020-10-12 15:30:36.616587
54182	KWR	2020-09-30	177.91	182.35	177.91	179.71	174908	{}	\N	2020-10-12 15:30:36.624624
54183	KWR	2020-10-01	179.94	181.15	177.71	179.49	118854	{}	\N	2020-10-12 15:30:36.631195
54184	KWR	2020-10-02	176.65	184.26	176.65	182.69	99175	{}	\N	2020-10-12 15:30:36.636969
54185	KWR	2020-10-05	184.95	188.91	184.95	188.14	159104	{}	\N	2020-10-12 15:30:36.642411
54186	KWR	2020-10-06	190.73	193.77	186.88	186.88	165736	{}	\N	2020-10-12 15:30:36.64726
54187	KWR	2020-10-07	188.98	192.31	187.955	189.83	173314	{}	\N	2020-10-12 15:30:36.651918
54188	KWR	2020-10-08	191.98	195.15	191.58	194.28	97774	{}	\N	2020-10-12 15:30:36.656406
54189	KWR	2020-10-09	196.11	197.43	193.455	194.47	112299	{}	\N	2020-10-12 15:30:36.661563
54190	KWR	2020-10-12	194.25	196	193.135	195.905	14134	{}	\N	2020-10-12 15:30:36.66586
54191	LABD	2020-09-28	58.69	61.88	58.69	60.35	441062	{}	\N	2020-10-12 15:30:37.692232
54192	LABD	2020-09-29	60.28	61.57	57.92	58.93	508227	{}	\N	2020-10-12 15:30:37.702287
54193	LABD	2020-09-30	58.83	60.89	56.59	59.68	509238	{}	\N	2020-10-12 15:30:37.711835
54194	LABD	2020-10-01	58.59	59.9377	57	57	601314	{}	\N	2020-10-12 15:30:37.718307
54195	LABD	2020-10-02	60.34	61.65	56.805	61.14	724928	{}	\N	2020-10-12 15:30:37.72407
54196	LABD	2020-10-05	58.16	58.33	51.5501	51.71	1640433	{}	\N	2020-10-12 15:30:37.729393
54197	LABD	2020-10-06	51.73	53.6899	49.8	53.1	887408	{}	\N	2020-10-12 15:30:37.735737
54198	LABD	2020-10-07	51.42	51.7892	47.17	47.9	1128332	{}	\N	2020-10-12 15:30:37.742312
54199	LABD	2020-10-08	46.52	48.35	46.5	47.3	673278	{}	\N	2020-10-12 15:30:37.747963
54200	LABD	2020-10-09	46.46	47.5324	45.58	45.76	585059	{}	\N	2020-10-12 15:30:37.752864
54201	LABD	2020-10-12	44.66	46.36	44.52	45.59	313007	{}	\N	2020-10-12 15:30:37.758147
54202	LAMR	2020-09-28	67.88	68.8	67.32	67.46	331770	{}	\N	2020-10-12 15:30:38.676275
54203	LAMR	2020-09-29	67.01	67.91	65.29	66.27	340291	{}	\N	2020-10-12 15:30:38.684814
54204	LAMR	2020-09-30	66	67.515	65.65	66.17	458209	{}	\N	2020-10-12 15:30:38.691429
54205	LAMR	2020-10-01	66.17	68.81	65.77	68.81	535676	{}	\N	2020-10-12 15:30:38.697743
54206	LAMR	2020-10-02	67.08	69.77	67.02	69.605	649485	{}	\N	2020-10-12 15:30:38.70422
54207	LAMR	2020-10-05	70.17	72.44	70	72.06	948643	{}	\N	2020-10-12 15:30:38.711026
54208	LAMR	2020-10-06	73.0963	73.11	71.06	71.29	837642	{}	\N	2020-10-12 15:30:38.717589
54209	LAMR	2020-10-07	72.27	72.27	69.74	69.82	455106	{}	\N	2020-10-12 15:30:38.725657
54210	LAMR	2020-10-08	70.53	71.12	69.62	71.02	395634	{}	\N	2020-10-12 15:30:38.734112
54211	LAMR	2020-10-09	71.64	72.365	69.84	70.26	476682	{}	\N	2020-10-12 15:30:38.740286
54212	LAMR	2020-10-12	70.38	70.5	69.028	69.45	144018	{}	\N	2020-10-12 15:30:38.745525
54213	LAUR	2020-09-28	13.09	13.22	12.95	12.99	670604	{}	\N	2020-10-12 15:30:39.62608
54214	LAUR	2020-09-29	12.85	13.11	12.82	13.03	581650	{}	\N	2020-10-12 15:30:39.637389
54215	LAUR	2020-09-30	13.23	13.49	13.12	13.28	891036	{}	\N	2020-10-12 15:30:39.646672
54216	LAUR	2020-10-01	13.29	13.61	12.94	13.59	452073	{}	\N	2020-10-12 15:30:39.653954
54217	LAUR	2020-10-02	13.28	13.6	12.921	13.47	377347	{}	\N	2020-10-12 15:30:39.660422
54218	LAUR	2020-10-05	13.59	13.63	13.02	13.24	796157	{}	\N	2020-10-12 15:30:39.666497
54219	LAUR	2020-10-06	13.38	13.57	13.11	13.14	453082	{}	\N	2020-10-12 15:30:39.671887
54220	LAUR	2020-10-07	13.23	13.36	12.99	12.99	587303	{}	\N	2020-10-12 15:30:39.678738
54221	LAUR	2020-10-08	12.92	13.5	12.86	13.34	747733	{}	\N	2020-10-12 15:30:39.686804
54222	LAUR	2020-10-09	13.3	13.72	13.2	13.62	485933	{}	\N	2020-10-12 15:30:39.694514
54223	LAUR	2020-10-12	13.63	13.75	13.57	13.61	77140	{}	\N	2020-10-12 15:30:39.700524
54224	MSBI	2020-09-28	12.88	13.39	12.88	13.1	178413	{}	\N	2020-10-12 15:30:40.680278
54225	MSBI	2020-09-29	13.3	13.3	12.68	12.9	111199	{}	\N	2020-10-12 15:30:40.688444
54226	MSBI	2020-09-30	12.96	13.12	12.8	12.85	122601	{}	\N	2020-10-12 15:30:40.694
54227	MSBI	2020-10-01	13.02	13.32	12.77	13.23	294168	{}	\N	2020-10-12 15:30:40.699038
54228	MSBI	2020-10-02	13.625	13.72	13	13.61	120297	{}	\N	2020-10-12 15:30:40.705413
54229	MSBI	2020-10-05	13.72	13.99	13.485	13.87	92456	{}	\N	2020-10-12 15:30:40.713799
54230	MSBI	2020-10-06	13.93	14.41	12.8001	13.98	112039	{}	\N	2020-10-12 15:30:40.719955
54231	MSBI	2020-10-07	13.415	14.55	13.415	14.42	107709	{}	\N	2020-10-12 15:30:40.725176
54232	MSBI	2020-10-08	14.59	14.789	14.26	14.68	97883	{}	\N	2020-10-12 15:30:40.730341
54233	MSBI	2020-10-09	14.84	14.84	14.24	14.32	48751	{}	\N	2020-10-12 15:30:40.736098
54234	MSBI	2020-10-12	14.3	14.425	14.181	14.39	7554	{}	\N	2020-10-12 15:30:40.74279
54235	KIQ	2020-09-28	0.5099	0.5107	0.4848	0.5041	18645	{}	\N	2020-10-12 15:30:41.606351
54236	KIQ	2020-09-29	0.5025	0.524	0.4902	0.5096	35573	{}	\N	2020-10-12 15:30:41.618173
54237	KIQ	2020-09-30	0.5434	0.5475	0.52	0.5475	54718	{}	\N	2020-10-12 15:30:41.626022
54238	KIQ	2020-10-01	0.5475	0.5499	0.5255	0.529	14007	{}	\N	2020-10-12 15:30:41.632947
54239	KIQ	2020-10-02	0.55	0.55	0.4871	0.536	57111	{}	\N	2020-10-12 15:30:41.639885
54240	KIQ	2020-10-05	0.537	0.54	0.5241	0.54	5003	{}	\N	2020-10-12 15:30:41.647289
54241	KIQ	2020-10-06	0.477	0.5401	0.477	0.5171	107416	{}	\N	2020-10-12 15:30:41.652986
54242	KIQ	2020-10-07	0.5421	0.5421	0.51	0.5312	8328	{}	\N	2020-10-12 15:30:41.665532
54243	KIQ	2020-10-08	0.52	0.54	0.5124	0.53	44629	{}	\N	2020-10-12 15:30:41.676426
54244	KIQ	2020-10-09	0.5101	0.52	0.47	0.515	61892	{}	\N	2020-10-12 15:30:41.684309
54245	KIQ	2020-10-12	0.515	0.5421	0.515	0.5416	36904	{}	\N	2020-10-12 15:30:41.694042
54246	KLIC	2020-09-28	22.3	23.04	22.13	22.54	466221	{}	\N	2020-10-12 15:30:42.855116
54247	KLIC	2020-09-29	22.61	22.97	22.61	22.64	681498	{}	\N	2020-10-12 15:30:42.867347
54248	KLIC	2020-09-30	22.58	22.8	22.22	22.4	386285	{}	\N	2020-10-12 15:30:42.875241
54249	KLIC	2020-10-01	22.53	22.79	22.5	22.7	274774	{}	\N	2020-10-12 15:30:42.881866
54250	KLIC	2020-10-02	22.1	22.4	21.87	21.89	527475	{}	\N	2020-10-12 15:30:42.888633
54251	KLIC	2020-10-05	22.04	23.23	22.04	23.2	469406	{}	\N	2020-10-12 15:30:42.894301
54252	KLIC	2020-10-06	23.6	24.46	23.26	23.92	568677	{}	\N	2020-10-12 15:30:42.8993
54253	KLIC	2020-10-07	24.11	24.46	23.64	23.88	487498	{}	\N	2020-10-12 15:30:42.904821
54254	KLIC	2020-10-08	24	24.59	23.85	24.55	362740	{}	\N	2020-10-12 15:30:42.911353
54255	KLIC	2020-10-09	24.83	25.5	24.55	25.08	533260	{}	\N	2020-10-12 15:30:42.916818
54256	KLIC	2020-10-12	25.2	25.32	24.85	25	121100	{}	\N	2020-10-12 15:30:42.922778
54257	KNX	2020-09-28	42.5	42.75	41.36	42.3	2277753	{}	\N	2020-10-12 15:30:43.850879
54258	KNX	2020-09-29	42.29	42.7	41.28	41.31	3072609	{}	\N	2020-10-12 15:30:43.863391
54259	KNX	2020-09-30	41.46	41.46	40.44	40.7	4544237	{}	\N	2020-10-12 15:30:43.872068
54260	KNX	2020-10-01	40.7	40.87	40.04	40.5	5391583	{}	\N	2020-10-12 15:30:43.879793
54261	KNX	2020-10-02	40.11	42	40.09	41.61	5328635	{}	\N	2020-10-12 15:30:43.885028
54262	KNX	2020-10-05	41.79	42.26	41.18	42.25	7826347	{}	\N	2020-10-12 15:30:43.889864
54263	KNX	2020-10-06	42.25	42.36	41.13	41.21	6025586	{}	\N	2020-10-12 15:30:43.894632
54264	KNX	2020-10-07	41.52	42	41.26	41.31	7092242	{}	\N	2020-10-12 15:30:43.900718
54265	KNX	2020-10-08	41.66	41.855	40.91	41.28	5934831	{}	\N	2020-10-12 15:30:43.905998
54266	KNX	2020-10-09	40.08	40.83	39.79	40.25	9159310	{}	\N	2020-10-12 15:30:43.911161
54267	KNX	2020-10-12	40.57	41.45	40.17	40.89	2697070	{}	\N	2020-10-12 15:30:43.917295
54268	KOSS	2020-09-28	2.2586	2.2586	2.07	2.2	29691	{}	\N	2020-10-12 15:30:46.142163
54269	KOSS	2020-09-29	2.14	2.1765	2.14	2.14	7461	{}	\N	2020-10-12 15:30:46.152567
54270	KOSS	2020-09-30	2.19	2.19	2.14	2.14	1766	{}	\N	2020-10-12 15:30:46.160599
54271	KOSS	2020-10-01	2.14	2.18	2.01	2.17	9991	{}	\N	2020-10-12 15:30:46.168426
54272	KOSS	2020-10-05	2.1401	2.18	2.13	2.15	8440	{}	\N	2020-10-12 15:30:46.176846
54273	KOSS	2020-10-06	2.17	2.21	2.15	2.15	9249	{}	\N	2020-10-12 15:30:46.185664
54274	KOSS	2020-10-07	2.1558	2.2337	2.1558	2.2	6884	{}	\N	2020-10-12 15:30:46.194279
54275	KOSS	2020-10-08	2.15	2.23	2.15	2.19	21465	{}	\N	2020-10-12 15:30:46.20114
54276	KOSS	2020-10-09	2.1844	2.19	2.1586	2.17	5956	{}	\N	2020-10-12 15:30:46.207271
54277	KOSS	2020-10-12	2.1622	2.4108	2.06	2.25	135152	{}	\N	2020-10-12 15:30:46.213158
54278	KRNY	2020-09-28	7.2	7.41	7.2	7.39	276975	{}	\N	2020-10-12 15:30:47.152265
54279	KRNY	2020-09-29	7.44	7.46	7.19	7.26	171364	{}	\N	2020-10-12 15:30:47.164391
54280	KRNY	2020-09-30	7.31	7.37	7.16	7.21	317887	{}	\N	2020-10-12 15:30:47.172051
54281	KRNY	2020-10-01	7.21	7.355	7.12	7.27	226293	{}	\N	2020-10-12 15:30:47.180092
54282	KRNY	2020-10-02	7.15	7.5	7.15	7.46	194295	{}	\N	2020-10-12 15:30:47.186739
54283	KRNY	2020-10-05	7.5	7.65	7.49	7.64	149798	{}	\N	2020-10-12 15:30:47.191975
54284	KRNY	2020-10-06	7.74	7.955	7.6848	7.72	243463	{}	\N	2020-10-12 15:30:47.197445
54285	KRNY	2020-10-07	7.79	7.98	7.73	7.86	220587	{}	\N	2020-10-12 15:30:47.203125
54286	KRNY	2020-10-08	7.94	8.12	7.94	8.08	311958	{}	\N	2020-10-12 15:30:47.209317
54287	KRNY	2020-10-09	8.12	8.15	7.94	7.95	203965	{}	\N	2020-10-12 15:30:47.215675
54288	KRNY	2020-10-12	7.94	8.1	7.9	8.08	25243	{}	\N	2020-10-12 15:30:47.222653
54289	KTF	2020-09-28	11.09	11.09	11.01	11.04	25763	{}	\N	2020-10-12 15:30:48.19457
54290	KTF	2020-09-29	11.04	11.08	11.02	11.08	117820	{}	\N	2020-10-12 15:30:48.203383
54291	KTF	2020-09-30	11.11	11.11	11.05	11.08	120348	{}	\N	2020-10-12 15:30:48.209846
54292	KTF	2020-10-01	11.12	11.12	10.6	11.09	92159	{}	\N	2020-10-12 15:30:48.215858
54293	KTF	2020-10-02	11.1	11.11	11.0436	11.09	75378	{}	\N	2020-10-12 15:30:48.221784
54294	KTF	2020-10-05	11.09	11.15	10.84	11.12	161218	{}	\N	2020-10-12 15:30:48.228097
54295	KTF	2020-10-06	11.07	11.1282	11.01	11.05	358311	{}	\N	2020-10-12 15:30:48.232777
54296	KTF	2020-10-07	11.06	11.125	11.03	11.03	107829	{}	\N	2020-10-12 15:30:48.238253
54297	KTF	2020-10-08	11.05	11.07	11.02	11.07	89836	{}	\N	2020-10-12 15:30:48.243339
54298	KTF	2020-10-09	11.03	11.1188	11.03	11.1	124070	{}	\N	2020-10-12 15:30:48.24913
54299	KTF	2020-10-12	11.15	11.15	11.09	11.09	15528	{}	\N	2020-10-12 15:30:48.254117
54300	KURE	2020-09-28	33.09	33.1	32.6	32.8	36497	{}	\N	2020-10-12 15:30:49.097847
54301	KURE	2020-09-29	32.73	33.215	32.73	33.11	29586	{}	\N	2020-10-12 15:30:49.107216
54302	KURE	2020-09-30	33.58	33.63	33.44	33.5594	25824	{}	\N	2020-10-12 15:30:49.114934
54303	KURE	2020-10-01	34.12	34.18	33.8282	34.18	35945	{}	\N	2020-10-12 15:30:49.121748
54304	KURE	2020-10-02	33.88	34.095	33.571	33.6	20847	{}	\N	2020-10-12 15:30:49.127313
54305	KURE	2020-10-05	33.58	34	33.58	33.85	28399	{}	\N	2020-10-12 15:30:49.13199
54306	KURE	2020-10-06	34.25	34.5	34.2075	34.49	60351	{}	\N	2020-10-12 15:30:49.137065
54307	KURE	2020-10-07	34.92	34.92	34.5	34.55	36688	{}	\N	2020-10-12 15:30:49.14153
54308	KURE	2020-10-08	34.43	34.89	34.43	34.62	28504	{}	\N	2020-10-12 15:30:49.145951
54309	KURE	2020-10-09	35	35.3184	34.9336	35.14	40932	{}	\N	2020-10-12 15:30:49.150876
54310	KURE	2020-10-12	35.58	36.01	35.58	36.008	29291	{}	\N	2020-10-12 15:30:49.15598
54311	LAC	2020-09-28	10.11	11.15	9.41	10.5	17634290	{}	\N	2020-10-12 15:30:50.890513
54312	LAC	2020-09-29	9.85	10.83	9.67	10.61	18000589	{}	\N	2020-10-12 15:30:50.903069
54313	LAC	2020-09-30	10.59	12.93	10.47	11.39	31303318	{}	\N	2020-10-12 15:30:50.910563
54314	LAC	2020-10-01	12.25	12.87	11.74	12.5	20270686	{}	\N	2020-10-12 15:30:50.916858
54315	LAC	2020-10-02	11.4	15.3	11.36	14.7	54083316	{}	\N	2020-10-12 15:30:50.922842
54316	LAC	2020-10-05	16.2	16.97	15.52	15.86	36243092	{}	\N	2020-10-12 15:30:50.92817
54317	LAC	2020-10-06	15.87	15.99	13.6	13.79	27738475	{}	\N	2020-10-12 15:30:50.933357
54318	LAC	2020-10-07	14.57	15.71	14.16	14.81	17259965	{}	\N	2020-10-12 15:30:50.938949
54319	LAC	2020-10-08	15.27	15.34	14.2722	14.78	8684472	{}	\N	2020-10-12 15:30:50.944445
54320	LAC	2020-10-09	14.8	14.9	14.01	14.44	9123451	{}	\N	2020-10-12 15:30:50.949966
54321	LAC	2020-10-12	14.29	14.3999	13.03	13.13	6366692	{}	\N	2020-10-12 15:30:50.955757
54322	LAKE	2020-09-28	18.5	18.87	18.25	18.83	302185	{}	\N	2020-10-12 15:30:51.876637
54323	LAKE	2020-09-29	19.01	19.53	18.81	19.45	370506	{}	\N	2020-10-12 15:30:51.887949
54324	LAKE	2020-09-30	19.45	20.06	19.22	19.8	348793	{}	\N	2020-10-12 15:30:51.894858
54325	LAKE	2020-10-01	19.97	20.55	19.55	20.5	378723	{}	\N	2020-10-12 15:30:51.900812
54326	LAKE	2020-10-02	20.81	21.39	20.32	21.32	659709	{}	\N	2020-10-12 15:30:51.906234
54327	LAKE	2020-10-05	21.25	22.28	21.05	22.24	491883	{}	\N	2020-10-12 15:30:51.91084
54328	LAKE	2020-10-06	22.45	22.84	21.75	22.52	480297	{}	\N	2020-10-12 15:30:51.914958
54329	LAKE	2020-10-07	22.48	23.41	22.47	22.77	403824	{}	\N	2020-10-12 15:30:51.918899
54330	LAKE	2020-10-08	23.05	23.11	21.3593	22.07	525712	{}	\N	2020-10-12 15:30:51.923167
54331	LAKE	2020-10-09	22.01	23	22.01	22.86	311687	{}	\N	2020-10-12 15:30:51.927373
54332	LAKE	2020-10-12	23.25	23.8499	23.085	23.71	187239	{}	\N	2020-10-12 15:30:51.931749
54333	LARK	2020-09-28	20.85	21	20.85	20.95	3912	{}	\N	2020-10-12 15:30:53.446273
54334	LARK	2020-09-29	20.89	20.89	20.89	20.89	607	{}	\N	2020-10-12 15:30:53.456581
54335	LARK	2020-09-30	21.1	21.35	21.09	21.35	4532	{}	\N	2020-10-12 15:30:53.463864
54336	LARK	2020-10-01	21.695	21.775	21.35	21.715	3474	{}	\N	2020-10-12 15:30:53.470918
54337	LARK	2020-10-02	21.35	21.4546	20.76	20.76	6348	{}	\N	2020-10-12 15:30:53.476952
54338	LARK	2020-10-05	20.91	21.5	20.91	21.27	3664	{}	\N	2020-10-12 15:30:53.482147
54339	LARK	2020-10-06	21.3	21.98	21.24	21.65	10497	{}	\N	2020-10-12 15:30:53.487797
54340	LARK	2020-10-07	21.63	22.15	21.51	22.09	5890	{}	\N	2020-10-12 15:30:53.493308
54341	LARK	2020-10-08	22.31	22.85	22.31	22.53	3871	{}	\N	2020-10-12 15:30:53.498624
54342	LARK	2020-10-09	22.82	22.92	22.25	22.78	3458	{}	\N	2020-10-12 15:30:53.503919
54343	LARK	2020-10-12	23.01	23.01	23.01	23.01	445	{}	\N	2020-10-12 15:30:53.50944
54344	LAWS	2020-09-28	41.48	42.15	41.05	41.81	7146	{}	\N	2020-10-12 15:30:55.963747
54345	LAWS	2020-09-29	41.65	41.65	40.92	41.01	10217	{}	\N	2020-10-12 15:30:55.975919
54346	LAWS	2020-09-30	41.69	41.69	41.03	41.03	10796	{}	\N	2020-10-12 15:30:55.98385
54347	LAWS	2020-10-01	41.1	42.485	40.7701	41.34	9067	{}	\N	2020-10-12 15:30:55.990348
54348	LAWS	2020-10-02	41	44.935	41	43.5	19190	{}	\N	2020-10-12 15:30:55.996231
54349	LAWS	2020-10-05	43.99	44.29	43.7	44.28	9070	{}	\N	2020-10-12 15:30:56.0013
54350	LAWS	2020-10-06	44.99	45.14	43.845	44.5	11780	{}	\N	2020-10-12 15:30:56.006439
54351	LAWS	2020-10-07	44.54	46.49	44.5	46.49	9228	{}	\N	2020-10-12 15:30:56.011531
54352	LAWS	2020-10-08	46.99	47.47	42.3	44.82	19166	{}	\N	2020-10-12 15:30:56.016497
54353	LAWS	2020-10-09	45.55	45.55	44.39	44.55	5492	{}	\N	2020-10-12 15:30:56.021652
54354	LAWS	2020-10-12	44.96	44.96	44.96	44.96	515	{}	\N	2020-10-12 15:30:56.02692
54355	KIRK	2020-09-28	8.04	8.48	7.8574	8.28	811698	{}	\N	2020-10-12 15:30:57.245652
54356	KIRK	2020-09-29	8.24	8.6	7.99	8.11	686212	{}	\N	2020-10-12 15:30:57.257889
54357	KIRK	2020-09-30	8.1	8.7734	8	8.21	1330530	{}	\N	2020-10-12 15:30:57.265739
54358	KIRK	2020-10-01	8.28	9.24	8.271	9.17	2163789	{}	\N	2020-10-12 15:30:57.272959
54359	KIRK	2020-10-02	8.59	9.52	8.35	9.09	1191534	{}	\N	2020-10-12 15:30:57.278905
54360	KIRK	2020-10-05	9.18	9.85	9.161	9.74	1213093	{}	\N	2020-10-12 15:30:57.283737
54361	KIRK	2020-10-06	9.84	10.87	9.76	10.32	2073520	{}	\N	2020-10-12 15:30:57.288141
54362	KIRK	2020-10-07	10.51	10.8	10.04	10.37	1304119	{}	\N	2020-10-12 15:30:57.29292
54363	KIRK	2020-10-08	10.34	10.4168	9.52	9.72	1241481	{}	\N	2020-10-12 15:30:57.297693
54364	KIRK	2020-10-09	9.9	11	9.7	10.82	1248406	{}	\N	2020-10-12 15:30:57.302255
54365	KIRK	2020-10-12	10.85	11.24	10.39	10.86	566023	{}	\N	2020-10-12 15:30:57.30682
54366	KMB	2020-09-28	146.97	148.43	146.3	147.56	1481582	{}	\N	2020-10-12 15:30:58.28026
54367	KMB	2020-09-29	147.6	147.94	146.02	146.34	1878951	{}	\N	2020-10-12 15:30:58.292747
54368	KMB	2020-09-30	146.94	148.32	146.68	147.66	3110508	{}	\N	2020-10-12 15:30:58.300133
54369	KMB	2020-10-01	148.08	148.65	146.5	147.63	2182583	{}	\N	2020-10-12 15:30:58.306936
54370	KMB	2020-10-02	146.62	148.51	145.83	147.43	2063246	{}	\N	2020-10-12 15:30:58.312629
54371	KMB	2020-10-05	147.34	149.62	147.34	149.53	2639159	{}	\N	2020-10-12 15:30:58.317779
54372	KMB	2020-10-06	149.53	151.46	149.22	150	2968804	{}	\N	2020-10-12 15:30:58.322451
54373	KMB	2020-10-07	150.48	151.685	149.565	150.85	2172022	{}	\N	2020-10-12 15:30:58.328459
54374	KMB	2020-10-08	151.2	151.2	149.77	150.31	2643575	{}	\N	2020-10-12 15:30:58.333194
54375	KMB	2020-10-09	150.6	152.75	150.5	152.49	2355126	{}	\N	2020-10-12 15:30:58.338103
54376	KMB	2020-10-12	153.07	153.775	152.53	153.7039	452243	{}	\N	2020-10-12 15:30:58.343045
54377	KNOW	2020-09-28	33.85	34.25	33.85	34.09	10039	{}	\N	2020-10-12 15:30:59.333631
54378	KNOW	2020-09-29	33.95	34.2	33.7907	33.92	6157	{}	\N	2020-10-12 15:30:59.341855
54379	KNOW	2020-09-30	34.11	34.31	33.9	34.07	6213	{}	\N	2020-10-12 15:30:59.347287
54380	KNOW	2020-10-01	34.02	34.4	33.97	34.1	11376	{}	\N	2020-10-12 15:30:59.352844
54381	KNOW	2020-10-02	33.47	34.3911	33.47	34.3	6108	{}	\N	2020-10-12 15:30:59.357867
54382	KNOW	2020-10-05	34.33	34.9279	34.33	34.9279	10450	{}	\N	2020-10-12 15:30:59.362777
54383	KNOW	2020-10-06	35.051	35.37	34.7	34.73	9767	{}	\N	2020-10-12 15:30:59.367056
54384	KNOW	2020-10-07	34.95	35.385	34.95	35.371	5364	{}	\N	2020-10-12 15:30:59.371278
54385	KNOW	2020-10-08	35.45	35.7	35.45	35.7	3658	{}	\N	2020-10-12 15:30:59.375584
54386	KNOW	2020-10-09	36	36.0101	35.7616	35.9191	4265	{}	\N	2020-10-12 15:30:59.379857
54387	KNOW	2020-10-12	36.0204	36.0717	35.88	36.04	10654	{}	\N	2020-10-12 15:30:59.383878
54388	KOP	2020-09-28	20.61	21.255	20.61	20.88	115607	{}	\N	2020-10-12 15:31:02.141681
54389	KOP	2020-09-29	20.85	21.34	20.71	20.82	335923	{}	\N	2020-10-12 15:31:02.156282
54390	KOP	2020-09-30	20.96	21.7	20.58	20.91	259848	{}	\N	2020-10-12 15:31:02.164961
54391	KOP	2020-10-01	21.07	21.33	20.75	21.02	345798	{}	\N	2020-10-12 15:31:02.172713
54392	KOP	2020-10-02	20.38	22.27	20.38	22.04	226362	{}	\N	2020-10-12 15:31:02.178879
54393	KOP	2020-10-05	22.35	22.72	21.9	22.38	212667	{}	\N	2020-10-12 15:31:02.184609
54394	KOP	2020-10-06	22.88	23.8	22.51	22.7	256696	{}	\N	2020-10-12 15:31:02.190094
54395	KOP	2020-10-07	23.01	23.625	22.85	23.14	343700	{}	\N	2020-10-12 15:31:02.195199
54396	KOP	2020-10-08	23.65	23.75	23.02	23.7	179434	{}	\N	2020-10-12 15:31:02.200581
54397	KOP	2020-10-09	23.91	24.255	23.555	23.8	193193	{}	\N	2020-10-12 15:31:02.205769
54398	KOP	2020-10-12	23.71	24.2	23.6416	23.955	30118	{}	\N	2020-10-12 15:31:02.21074
54399	KRO	2020-09-28	12.54	12.85	12.54	12.7	169063	{}	\N	2020-10-12 15:31:03.297629
54400	KRO	2020-09-29	12.66	12.88	12.5184	12.82	651246	{}	\N	2020-10-12 15:31:03.311466
54401	KRO	2020-09-30	12.76	13.09	12.69	12.86	918665	{}	\N	2020-10-12 15:31:03.320269
54402	KRO	2020-10-01	12.92	13.04	12.61	12.76	254829	{}	\N	2020-10-12 15:31:03.327342
54403	KRO	2020-10-02	12.59	13.09	12.43	13.06	202012	{}	\N	2020-10-12 15:31:03.333607
54404	KRO	2020-10-05	13.22	13.535	13.095	13.22	289901	{}	\N	2020-10-12 15:31:03.338391
54405	KRO	2020-10-06	13.42	13.6	13.22	13.24	258024	{}	\N	2020-10-12 15:31:03.343144
54406	KRO	2020-10-07	13.48	13.855	13.43	13.61	300067	{}	\N	2020-10-12 15:31:03.347906
54407	KRO	2020-10-08	13.82	13.945	13.68	13.79	283730	{}	\N	2020-10-12 15:31:03.352936
54408	KRO	2020-10-09	13.96	13.99	13.7	13.71	344964	{}	\N	2020-10-12 15:31:03.358019
54409	KRO	2020-10-12	13.7	14.09	13.7	14.055	74209	{}	\N	2020-10-12 15:31:03.363118
54410	KTH	2020-09-28	34.25	34.25	34.1	34.1001	615	{}	\N	2020-10-12 15:31:04.537131
54411	KTH	2020-09-30	34.1	34.1	34.1	34.1	1182	{}	\N	2020-10-12 15:31:04.547073
54412	KTH	2020-10-02	34.01	34.01	34.01	34.01	1552	{}	\N	2020-10-12 15:31:04.553381
54413	KTH	2020-10-06	34.87	34.87	34.155	34.16	4696	{}	\N	2020-10-12 15:31:04.55856
54414	KTH	2020-10-08	34.23	34.23	33.74	33.9539	9212	{}	\N	2020-10-12 15:31:04.563901
54415	KTH	2020-10-09	34.0547	34.0547	34.0547	34.0547	2000	{}	\N	2020-10-12 15:31:04.571463
54416	KYN	2020-09-28	3.97	4.09	3.93	4.05	872722	{}	\N	2020-10-12 15:31:05.510703
54417	KYN	2020-09-29	4.01	4.09	3.98	4.04	2076233	{}	\N	2020-10-12 15:31:05.52363
54418	KYN	2020-09-30	4.11	4.14	3.95	3.99	2611643	{}	\N	2020-10-12 15:31:05.530847
54419	KYN	2020-10-01	3.97	3.99	3.9	3.94	2709036	{}	\N	2020-10-12 15:31:05.536879
54420	KYN	2020-10-02	3.78	4	3.78	4	2846906	{}	\N	2020-10-12 15:31:05.54239
54421	KYN	2020-10-05	4.16	4.41	4.1101	4.4	6495693	{}	\N	2020-10-12 15:31:05.54772
54422	KYN	2020-10-06	4.49	4.62	4.35	4.44	9770551	{}	\N	2020-10-12 15:31:05.552459
54423	KYN	2020-10-07	4.45	4.51	4.33	4.4	4360095	{}	\N	2020-10-12 15:31:05.557136
54424	KYN	2020-10-08	4.45	4.65	4.44	4.65	4681650	{}	\N	2020-10-12 15:31:05.561487
54425	KYN	2020-10-09	4.68	4.74	4.55	4.65	3610275	{}	\N	2020-10-12 15:31:05.565702
54426	KYN	2020-10-12	4.64	4.64	4.51	4.55	876460	{}	\N	2020-10-12 15:31:05.570128
54427	LANC	2020-09-28	181.52	184.21	181.035	183.02	90803	{}	\N	2020-10-12 15:31:06.908221
54428	LANC	2020-09-29	183.18	183.18	178.06	178.24	88178	{}	\N	2020-10-12 15:31:06.91838
54429	LANC	2020-09-30	178.36	181.37	177.71	178.56	80901	{}	\N	2020-10-12 15:31:06.924805
54430	LANC	2020-10-01	178.7	178.7	175.07	177.63	71894	{}	\N	2020-10-12 15:31:06.930213
54431	LANC	2020-10-02	175.76	177.76	174.94	176.76	78099	{}	\N	2020-10-12 15:31:06.935472
54432	LANC	2020-10-05	178.16	179.22	177.59	178.72	55433	{}	\N	2020-10-12 15:31:06.941102
54433	LANC	2020-10-06	178.585	179.61	174.975	176.04	77037	{}	\N	2020-10-12 15:31:06.945324
54434	LANC	2020-10-07	177.21	177.21	174.82	176.33	123104	{}	\N	2020-10-12 15:31:06.949564
54435	LANC	2020-10-08	177.71	180.13	177	177.4	71853	{}	\N	2020-10-12 15:31:06.954065
54436	LANC	2020-10-09	178.86	179.19	175.73	176.89	65184	{}	\N	2020-10-12 15:31:06.958297
54437	LANC	2020-10-12	177.1	179.83	176.97	179.745	9625	{}	\N	2020-10-12 15:31:06.962405
54438	LB	2020-09-28	31.1	31.78	30.7	31.53	3997412	{}	\N	2020-10-12 15:31:08.04976
54439	LB	2020-09-29	31.52	31.52	30.265	30.36	9835122	{}	\N	2020-10-12 15:31:08.062874
54440	LB	2020-09-30	31.64	33.12	31.3	31.81	12102625	{}	\N	2020-10-12 15:31:08.071272
54441	LB	2020-10-01	32.05	33.46	32.05	33.22	11102861	{}	\N	2020-10-12 15:31:08.078033
54442	LB	2020-10-02	32.15	33.43	32	33.13	5302557	{}	\N	2020-10-12 15:31:08.086096
54443	LB	2020-10-05	33.35	33.48	32.48	33.21	5692330	{}	\N	2020-10-12 15:31:08.093069
54444	LB	2020-10-06	33.42	33.57	32.24	32.3	5631493	{}	\N	2020-10-12 15:31:08.099056
54445	LB	2020-10-07	33.01	33.39	32.58	32.99	6337549	{}	\N	2020-10-12 15:31:08.104649
54446	LB	2020-10-08	33.56	33.66	32.38	32.58	11441937	{}	\N	2020-10-12 15:31:08.109673
54447	LB	2020-10-09	32.99	33.09	31.8	32.26	7542706	{}	\N	2020-10-12 15:31:08.114432
54448	LB	2020-10-12	32.37	33.09	31.96	32.16	3827011	{}	\N	2020-10-12 15:31:08.119985
54449	LCA	2020-09-28	15.1	15.4899	14.36	15.15	2090211	{}	\N	2020-10-12 15:31:09.775449
54450	LCA	2020-09-29	14.97	15.07	14.5	14.59	1521436	{}	\N	2020-10-12 15:31:09.785999
54451	LCA	2020-09-30	14.675	14.79	13.8	14.37	1808814	{}	\N	2020-10-12 15:31:09.793237
54452	LCA	2020-10-01	14.63	14.74	14.3	14.4	1201898	{}	\N	2020-10-12 15:31:09.79984
54453	LCA	2020-10-02	13.62	14.68	13.61	14.56	1303515	{}	\N	2020-10-12 15:31:09.806015
54454	LCA	2020-10-05	14.42	14.69	13.9	14.26	1590309	{}	\N	2020-10-12 15:31:09.811168
54455	LCA	2020-10-06	14.12	14.26	13.51	13.8	2457426	{}	\N	2020-10-12 15:31:09.816194
54456	LCA	2020-10-07	13.845	14.1	13.5231	13.95	1227366	{}	\N	2020-10-12 15:31:09.821185
54457	LCA	2020-10-08	13.9	14.42	13.75	14.15	1131707	{}	\N	2020-10-12 15:31:09.826118
54458	LCA	2020-10-09	14.1	14.33	13.94	14.1	765620	{}	\N	2020-10-12 15:31:09.831166
54459	LCA	2020-10-12	14.1	14.14	13.83	13.91	509654	{}	\N	2020-10-12 15:31:09.83614
54460	KKR	2020-09-28	34.45	35.08	33.95	34.95	2817767	{}	\N	2020-10-12 15:31:10.804049
54461	KKR	2020-09-29	35.01	35.06	34.11	34.19	5582752	{}	\N	2020-10-12 15:31:10.818963
54462	KKR	2020-09-30	34.34	34.87	34.03	34.34	6120656	{}	\N	2020-10-12 15:31:10.827761
54463	KKR	2020-10-01	34.64	35.01	34.375	34.69	5358695	{}	\N	2020-10-12 15:31:10.835315
54464	KKR	2020-10-02	34.2	35.48	34.01	35.11	3882396	{}	\N	2020-10-12 15:31:10.841597
54465	KKR	2020-10-05	35.46	36.21	35.32	36.06	3977610	{}	\N	2020-10-12 15:31:10.846648
54466	KKR	2020-10-06	36.1	36.23	34.95	34.95	5292903	{}	\N	2020-10-12 15:31:10.851791
54467	KKR	2020-10-07	35.26	35.72	35.205	35.25	4099861	{}	\N	2020-10-12 15:31:10.856936
54468	KKR	2020-10-08	35.72	36.11	35.49	36.05	3503229	{}	\N	2020-10-12 15:31:10.861779
54469	KKR	2020-10-09	36.21	36.42	35.715	35.83	7152409	{}	\N	2020-10-12 15:31:10.866794
54470	KKR	2020-10-12	36.28	36.39	35.95	36.2	607805	{}	\N	2020-10-12 15:31:10.871802
54471	KNDI	2020-09-28	6.1	6.98	6.02	6.63	4698559	{}	\N	2020-10-12 15:31:11.791576
54472	KNDI	2020-09-29	6.445	7.41	6.3501	7.18	11800690	{}	\N	2020-10-12 15:31:11.802484
54473	KNDI	2020-09-30	7.14	7.14	6.13	6.2	3969044	{}	\N	2020-10-12 15:31:11.8104
54474	KNDI	2020-10-01	6.34	6.37	5.95	6.1	1458036	{}	\N	2020-10-12 15:31:11.817349
54475	KNDI	2020-10-02	5.87	6.4	5.87	6.27	1745478	{}	\N	2020-10-12 15:31:11.824402
54476	KNDI	2020-10-05	6.3	7.17	6.2501	7.11	3398581	{}	\N	2020-10-12 15:31:11.829562
54477	KNDI	2020-10-06	7.12	7.1332	6.46	6.69	1467406	{}	\N	2020-10-12 15:31:11.834412
54478	KNDI	2020-10-07	6.8	7.37	6.66	7.09	3197585	{}	\N	2020-10-12 15:31:11.839472
54479	KNDI	2020-10-08	7.21	8.19	6.9	7.58	8323742	{}	\N	2020-10-12 15:31:11.84428
54480	KNDI	2020-10-09	7.6	7.7	7.28	7.455	2048198	{}	\N	2020-10-12 15:31:11.849198
54481	KNDI	2020-10-12	7.54	7.88	7.31	7.68	956989	{}	\N	2020-10-12 15:31:11.854194
54482	KRMA	2020-09-28	24.035	24.18	24.02	24.18	7357	{}	\N	2020-10-12 15:31:13.610896
54483	KRMA	2020-09-29	24.12	24.17	23.9067	24.05	18482	{}	\N	2020-10-12 15:31:13.620628
54484	KRMA	2020-09-30	24.22	24.36	24.09	24.31	5692	{}	\N	2020-10-12 15:31:13.627728
54485	KRMA	2020-10-01	24.48	24.48	24.14	24.28	23877	{}	\N	2020-10-12 15:31:13.634493
54486	KRMA	2020-10-02	24.1243	24.36	24.0716	24.17	6033	{}	\N	2020-10-12 15:31:13.639976
54487	KRMA	2020-10-05	24.37	24.71	24.37	24.69	6851	{}	\N	2020-10-12 15:31:13.645075
54488	KRMA	2020-10-06	24.6469	24.78	24.29	24.29	11195	{}	\N	2020-10-12 15:31:13.649615
54489	KRMA	2020-10-07	24.7025	24.87	24.6401	24.84	17425	{}	\N	2020-10-12 15:31:13.654471
54490	KRMA	2020-10-08	24.9745	24.999	24.9681	24.99	3432	{}	\N	2020-10-12 15:31:13.659507
54491	KRMA	2020-10-09	25	25.2789	25	25.18	9469	{}	\N	2020-10-12 15:31:13.66438
54492	KRMA	2020-10-12	25.33	25.54	25.33	25.54	783	{}	\N	2020-10-12 15:31:13.66926
54493	KTOV	2020-09-28	4.39	4.59	4.13	4.26	978436	{}	\N	2020-10-12 15:31:15.036938
54494	KTOV	2020-09-29	4.33	4.48	4.27	4.36	549398	{}	\N	2020-10-12 15:31:15.048384
54495	KTOV	2020-09-30	4.3	4.42	4.27	4.41	303904	{}	\N	2020-10-12 15:31:15.056371
54496	KTOV	2020-10-01	4.38	4.4272	4.23	4.36	286917	{}	\N	2020-10-12 15:31:15.063199
54497	KTOV	2020-10-02	4.23	4.38	4.215	4.33	232352	{}	\N	2020-10-12 15:31:15.068931
54498	KTOV	2020-10-05	4.39	4.58	4.28	4.42	418347	{}	\N	2020-10-12 15:31:15.074722
54499	KTOV	2020-10-06	4.45	4.5	4.36	4.44	360226	{}	\N	2020-10-12 15:31:15.080411
54500	KTOV	2020-10-07	4.45	4.55	4.38	4.52	339207	{}	\N	2020-10-12 15:31:15.086578
54501	KTOV	2020-10-08	4.5	4.59	4.4251	4.51	438463	{}	\N	2020-10-12 15:31:15.092392
54502	KTOV	2020-10-09	4.53	4.57	4.4	4.46	205656	{}	\N	2020-10-12 15:31:15.098059
54503	KTOV	2020-10-12	4.43	4.48	4.3801	4.435	63743	{}	\N	2020-10-12 15:31:15.103625
54504	KZIA	2020-09-28	6.77	7.0042	6.77	6.8	24835	{}	\N	2020-10-12 15:31:16.055753
54505	KZIA	2020-09-29	6.89	6.89	6.67	6.75	30741	{}	\N	2020-10-12 15:31:16.067263
54506	KZIA	2020-10-02	6.35	6.5	6	6.2	138912	{}	\N	2020-10-12 15:31:16.074554
54507	KZIA	2020-10-05	6.37	6.37	6.01	6.05	75214	{}	\N	2020-10-12 15:31:16.080659
54508	KZIA	2020-10-06	6	6.02	5.91	5.95	55402	{}	\N	2020-10-12 15:31:16.086862
54509	KZIA	2020-10-07	6.54	6.54	6.02	6.1	19391	{}	\N	2020-10-12 15:31:16.093708
54510	KZIA	2020-10-08	6.25	6.33	6.06	6.13	13201	{}	\N	2020-10-12 15:31:16.099292
54511	KZIA	2020-10-09	6.2	6.22	5.91	6	38152	{}	\N	2020-10-12 15:31:16.104478
54512	KZIA	2020-10-12	6	6.115	5.91	5.99	6115	{}	\N	2020-10-12 15:31:16.109498
54513	LAND	2020-09-28	15.09	15.35	15.04	15.31	88495	{}	\N	2020-10-12 15:31:17.896308
54514	LAND	2020-09-29	15.26	15.33	14.9	15.06	66194	{}	\N	2020-10-12 15:31:17.907627
54515	LAND	2020-09-30	15.2	15.336	14.93	15.02	84262	{}	\N	2020-10-12 15:31:17.914684
54516	LAND	2020-10-01	15.2	15.31	15	15.3	89017	{}	\N	2020-10-12 15:31:17.921593
54517	LAND	2020-10-02	15.12	15.51	15.12	15.5	77503	{}	\N	2020-10-12 15:31:17.926944
54518	LAND	2020-10-05	15.55	15.69	15.15	15.25	78365	{}	\N	2020-10-12 15:31:17.931926
54519	LAND	2020-10-06	15.16	15.74	15.1	15.39	102062	{}	\N	2020-10-12 15:31:17.936337
54520	LAND	2020-10-07	15.47	15.47	15	15.08	191729	{}	\N	2020-10-12 15:31:17.941308
54521	LAND	2020-10-08	14.24	14.35	14	14.3	1471665	{}	\N	2020-10-12 15:31:17.945702
54522	LAND	2020-10-09	14.3	14.49	14.25	14.41	461279	{}	\N	2020-10-12 15:31:17.950386
54523	LAND	2020-10-12	14.45	14.5	14.38	14.43	85472	{}	\N	2020-10-12 15:31:17.955664
54524	LAZ	2020-09-28	31.82	32.28	31.5	32.01	611946	{}	\N	2020-10-12 15:31:19.057286
54525	LAZ	2020-09-29	32.1	32.96	31.86	32.52	1089681	{}	\N	2020-10-12 15:31:19.070546
54526	LAZ	2020-09-30	32.68	33.295	32.54	33.05	927794	{}	\N	2020-10-12 15:31:19.077903
54527	LAZ	2020-10-01	33.25	33.4	32.54	33.1	533906	{}	\N	2020-10-12 15:31:19.084625
54528	LAZ	2020-10-02	32.95	34.57	32.95	34.34	819253	{}	\N	2020-10-12 15:31:19.091115
54529	LAZ	2020-10-05	34.9	35.1	34.53	34.61	792342	{}	\N	2020-10-12 15:31:19.097844
54530	LAZ	2020-10-06	34.73	35.01	34.165	34.5	1120428	{}	\N	2020-10-12 15:31:19.103194
54531	LAZ	2020-10-07	34.93	35.63	34.65	34.95	828136	{}	\N	2020-10-12 15:31:19.108248
54532	LAZ	2020-10-08	35.2	36.39	35.12	36.38	1096745	{}	\N	2020-10-12 15:31:19.113458
54533	LAZ	2020-10-09	36.7	36.93	36.25	36.51	990923	{}	\N	2020-10-12 15:31:19.118853
54534	LAZ	2020-10-12	36.61	37.6	36.53	37.29	167339	{}	\N	2020-10-12 15:31:19.124504
54535	LBC	2020-09-28	8.08	8.45	8.04	8.38	28876	{}	\N	2020-10-12 15:31:20.008598
54536	LBC	2020-09-29	8.31	8.31	8.1	8.27	15821	{}	\N	2020-10-12 15:31:20.019183
54537	LBC	2020-09-30	8.32	8.4823	8.29	8.33	22423	{}	\N	2020-10-12 15:31:20.027192
54538	LBC	2020-10-01	8.18	8.36	8.11	8.27	33891	{}	\N	2020-10-12 15:31:20.033689
54539	LBC	2020-10-02	8.15	8.55	8.12	8.55	16775	{}	\N	2020-10-12 15:31:20.039529
54540	LBC	2020-10-05	8.63	8.89	8.63	8.87	21157	{}	\N	2020-10-12 15:31:20.044723
54541	LBC	2020-10-06	8.93	9.1	8.51	8.75	35157	{}	\N	2020-10-12 15:31:20.049858
54542	LBC	2020-10-07	8.87	9.05	8.87	9.02	28384	{}	\N	2020-10-12 15:31:20.054951
54543	LBC	2020-10-08	9.14	9.18	8.91	9.17	23935	{}	\N	2020-10-12 15:31:20.059906
54544	LBC	2020-10-09	9.28	9.29	9.03	9.06	26373	{}	\N	2020-10-12 15:31:20.064871
54545	LBC	2020-10-12	9.02	9.1	9.02	9.05	10115	{}	\N	2020-10-12 15:31:20.070022
54546	LBRDK	2020-09-28	141.98	144.139	141.32	143.37	732560	{}	\N	2020-10-12 15:31:21.307861
54547	LBRDK	2020-09-29	143.92	144.55	142.69	143.08	726894	{}	\N	2020-10-12 15:31:21.319264
54548	LBRDK	2020-09-30	143.25	144.45	142.21	142.87	688919	{}	\N	2020-10-12 15:31:21.32646
54549	LBRDK	2020-10-01	143.195	146.36	143.195	145.64	678974	{}	\N	2020-10-12 15:31:21.338497
54550	LBRDK	2020-10-02	143.96	145.11	141.67	142.85	1015461	{}	\N	2020-10-12 15:31:21.346319
54551	LBRDK	2020-10-05	143.64	147.08	143.51	146.35	1024779	{}	\N	2020-10-12 15:31:21.353907
54552	LBRDK	2020-10-06	145.53	146.51	142.98	144.25	1013596	{}	\N	2020-10-12 15:31:21.365925
54553	LBRDK	2020-10-07	145.67	146.05	143.91	145.26	587247	{}	\N	2020-10-12 15:31:21.376513
54554	LBRDK	2020-10-08	145.88	147.18	144.67	146.85	698810	{}	\N	2020-10-12 15:31:21.385897
54555	LBRDK	2020-10-09	147.12	148.21	147.01	147.26	366972	{}	\N	2020-10-12 15:31:21.394057
54556	LBRDK	2020-10-12	148.32	150.44	148.15	150.08	252663	{}	\N	2020-10-12 15:31:21.410277
54557	LBTYK	2020-09-28	20.78	21.02	20.67	20.89	2272135	{}	\N	2020-10-12 15:31:22.386261
54558	LBTYK	2020-09-29	21.01	21.26	20.625	20.9	4564411	{}	\N	2020-10-12 15:31:22.399719
54559	LBTYK	2020-09-30	21.01	21.48	20.48	20.535	2326531	{}	\N	2020-10-12 15:31:22.408602
54560	LBTYK	2020-10-01	20.585	20.67	19.98	20.09	2671537	{}	\N	2020-10-12 15:31:22.416082
54561	LBTYK	2020-10-02	19.77	20.08	19.39	19.73	3448240	{}	\N	2020-10-12 15:31:22.426094
54562	LBTYK	2020-10-05	19.75	20.32	19.745	20.24	1791113	{}	\N	2020-10-12 15:31:22.431846
54563	LBTYK	2020-10-06	20.41	20.81	20.29	20.32	3047842	{}	\N	2020-10-12 15:31:22.437493
54564	LBTYK	2020-10-07	20.32	20.76	20.28	20.55	2335283	{}	\N	2020-10-12 15:31:22.442854
54565	LBTYK	2020-10-08	21.41	21.41	20.38	20.43	1500468	{}	\N	2020-10-12 15:31:22.448209
54566	LBTYK	2020-10-09	20.52	21.665	20.52	21.62	3355421	{}	\N	2020-10-12 15:31:22.453701
54567	LBTYK	2020-10-12	21.76	22.09	21.6	21.99	1587610	{}	\N	2020-10-12 15:31:22.458495
54568	LCNB	2020-09-28	13.38	13.6	13.38	13.54	20737	{}	\N	2020-10-12 15:31:24.213874
54569	LCNB	2020-09-29	13.43	13.52	13.38	13.52	1900	{}	\N	2020-10-12 15:31:24.225536
54570	LCNB	2020-09-30	13.7699	13.8	13.65	13.65	10125	{}	\N	2020-10-12 15:31:24.232555
54571	LCNB	2020-10-01	13.7	13.98	13.7	13.8	11555	{}	\N	2020-10-12 15:31:24.238357
54572	LCNB	2020-10-02	13.53	14.39	13.53	14.24	8609	{}	\N	2020-10-12 15:31:24.243253
54573	LCNB	2020-10-05	14.33	14.33	13.95	14.22	15975	{}	\N	2020-10-12 15:31:24.24768
54574	LCNB	2020-10-06	14.34	14.5	13.815	14.12	11269	{}	\N	2020-10-12 15:31:24.252368
54575	LCNB	2020-10-07	14.2	14.37	13.82	14.1	48570	{}	\N	2020-10-12 15:31:24.256238
54576	LCNB	2020-10-08	14.09	14.6	14.09	14.37	11881	{}	\N	2020-10-12 15:31:24.260171
54577	LCNB	2020-10-09	14.58	14.58	14.12	14.18	31568	{}	\N	2020-10-12 15:31:24.264015
54578	LCNB	2020-10-12	14.21	14.32	14.14	14.32	1856	{}	\N	2020-10-12 15:31:24.267976
54579	LCI	2020-09-28	5.87	6.0058	5.85	5.89	247248	{}	\N	2020-10-12 15:31:25.208067
54580	LCI	2020-09-29	5.9	6.02	5.85	5.95	447304	{}	\N	2020-10-12 15:31:25.219277
54581	LCI	2020-09-30	5.95	6.2056	5.943	6.11	637826	{}	\N	2020-10-12 15:31:25.22786
54582	LCI	2020-10-01	6.11	6.19	6.0231	6.08	326882	{}	\N	2020-10-12 15:31:25.234599
54583	LCI	2020-10-02	6	6.25	5.9304	6.13	466110	{}	\N	2020-10-12 15:31:25.240987
54584	LCI	2020-10-05	6.13	6.3	6.1	6.14	464588	{}	\N	2020-10-12 15:31:25.246901
54585	LCI	2020-10-06	6.13	6.43	6.13	6.25	899140	{}	\N	2020-10-12 15:31:25.25199
54586	LCI	2020-10-07	6.28	6.4334	6.25	6.39	672014	{}	\N	2020-10-12 15:31:25.257037
54587	LCI	2020-10-08	6.41	7	6.4	6.96	1076484	{}	\N	2020-10-12 15:31:25.261987
54588	LCI	2020-10-09	6.99	7.04	6.84	6.94	852010	{}	\N	2020-10-12 15:31:25.267081
54589	LCI	2020-10-12	6.93	7.08	6.73	7.065	179289	{}	\N	2020-10-12 15:31:25.272348
54590	LC	2020-09-28	4.63	4.8	4.62	4.71	707783	{}	\N	2020-10-12 15:31:27.568361
54591	LC	2020-09-29	4.72	4.795	4.59	4.65	1117422	{}	\N	2020-10-12 15:31:27.581885
54592	LC	2020-09-30	4.67	4.82	4.65	4.71	1155073	{}	\N	2020-10-12 15:31:27.589406
54593	LC	2020-10-01	4.75	4.93	4.64	4.92	1571006	{}	\N	2020-10-12 15:31:27.595568
54594	LC	2020-10-02	4.76	5.05	4.72	4.96	1815054	{}	\N	2020-10-12 15:31:27.601436
54595	LC	2020-10-05	4.97	5.1	4.84	4.91	1621459	{}	\N	2020-10-12 15:31:27.606815
54596	LC	2020-10-06	5.02	5.16	4.88	4.9	1834893	{}	\N	2020-10-12 15:31:27.611855
54597	LC	2020-10-07	5	5.223	4.99	5.12	1885183	{}	\N	2020-10-12 15:31:27.617589
54598	LC	2020-10-08	5.21	5.47	5.175	5.38	1710192	{}	\N	2020-10-12 15:31:27.624249
54599	LC	2020-10-09	5.49	5.54	5.25	5.28	1678225	{}	\N	2020-10-12 15:31:27.629626
54600	LC	2020-10-12	5.26	5.53	5.25	5.41	754592	{}	\N	2020-10-12 15:31:27.635267
54601	LDOS	2020-09-28	92.55	93.13	91.44	91.57	708030	{}	\N	2020-10-12 15:31:30.339453
54602	LDOS	2020-09-29	91.54	91.76	89.99	90.57	857911	{}	\N	2020-10-12 15:31:30.350339
54603	LDOS	2020-09-30	90.58	90.89	88.71	89.15	1398614	{}	\N	2020-10-12 15:31:30.357863
54604	LDOS	2020-10-01	89.83	90.51	87.98	90.42	1512336	{}	\N	2020-10-12 15:31:30.36358
54605	LDOS	2020-10-02	88.85	90.94	88.67	90.39	1521161	{}	\N	2020-10-12 15:31:30.369592
54606	LDOS	2020-10-05	90.64	90.92	88.96	89.82	1123426	{}	\N	2020-10-12 15:31:30.374432
54607	LDOS	2020-10-06	89.98	91.71	88.83	89.01	1176076	{}	\N	2020-10-12 15:31:30.379344
54608	LDOS	2020-10-07	89.35	89.85	87.7	88.48	1812488	{}	\N	2020-10-12 15:31:30.383997
54609	LDOS	2020-10-08	89.16	90.575	88.84	89.19	1173824	{}	\N	2020-10-12 15:31:30.388448
54610	LDOS	2020-10-09	90.11	91.28	89.78	90.87	1859456	{}	\N	2020-10-12 15:31:30.392851
54611	LDOS	2020-10-12	91.14	91.14	89.24	90.39	309159	{}	\N	2020-10-12 15:31:30.397478
54612	LEMB	2020-09-28	41.47	41.52	41.21	41.46	261541	{}	\N	2020-10-12 15:31:32.268772
54613	LEMB	2020-09-29	41.3906	41.6	41.35	41.48	49400	{}	\N	2020-10-12 15:31:32.277462
54614	LEMB	2020-09-30	41.51	41.84	41.51	41.83	571366	{}	\N	2020-10-12 15:31:32.284079
54615	LEMB	2020-10-01	41.95	42.0235	41.8422	42.01	788040	{}	\N	2020-10-12 15:31:32.28999
54616	LEMB	2020-10-02	41.84	41.94	41.83	41.86	223352	{}	\N	2020-10-12 15:31:32.296201
54617	LEMB	2020-10-05	41.92	42.079	41.92	42.06	40076	{}	\N	2020-10-12 15:31:32.301469
54618	LEMB	2020-10-06	42.15	42.2	41.82	41.92	81198	{}	\N	2020-10-12 15:31:32.306992
54619	LEMB	2020-10-07	42.04	42.04	41.77	41.99	77054	{}	\N	2020-10-12 15:31:32.312552
54620	LEMB	2020-10-08	42.04	42.11	41.92	42.09	142297	{}	\N	2020-10-12 15:31:32.317448
54621	LEMB	2020-10-09	42.3	42.36	42.24	42.24	76038	{}	\N	2020-10-12 15:31:32.322068
54622	LEMB	2020-10-12	42.3	42.3	42.2208	42.2208	4893	{}	\N	2020-10-12 15:31:32.326806
54623	LH	2020-09-28	183.05	186.22	183.05	184.07	422816	{}	\N	2020-10-12 15:31:33.658166
54624	LH	2020-09-29	185.29	188.64	184.2909	186.47	1086240	{}	\N	2020-10-12 15:31:33.6707
54625	LH	2020-09-30	185.98	189.84	185.83	188.27	1284411	{}	\N	2020-10-12 15:31:33.678451
54626	LH	2020-10-01	189.12	191.47	185.88	186.91	864089	{}	\N	2020-10-12 15:31:33.685111
54627	LH	2020-10-02	184.5	189.9	183.41	187.99	821477	{}	\N	2020-10-12 15:31:33.690679
54628	LH	2020-10-05	189.72	194.3	189.6	193.06	1426464	{}	\N	2020-10-12 15:31:33.695828
54629	LH	2020-10-06	193.3	195.96	191.07	191.11	705164	{}	\N	2020-10-12 15:31:33.700754
54630	LH	2020-10-07	192.63	194.665	191.96	192.51	1019854	{}	\N	2020-10-12 15:31:33.705805
54631	LH	2020-10-08	193.62	194.763	191.3	194.26	653691	{}	\N	2020-10-12 15:31:33.710705
54632	LH	2020-10-09	195.49	196.86	194.33	195.76	642425	{}	\N	2020-10-12 15:31:33.715522
54633	LH	2020-10-12	196.61	197.05	195.47	195.47	166496	{}	\N	2020-10-12 15:31:33.720551
54634	LMB	2020-09-28	8.15	8.9	8.15	8.9	108801	{}	\N	2020-10-12 15:31:35.906283
54635	LMB	2020-09-29	9	9.5	8.9989	9.2	90534	{}	\N	2020-10-12 15:31:35.916338
54636	LMB	2020-09-30	9.18	11.01	8.97	10.7	259355	{}	\N	2020-10-12 15:31:35.925114
54637	LMB	2020-10-01	11	12.47	10.35	10.72	604911	{}	\N	2020-10-12 15:31:35.932048
54638	LMB	2020-10-02	10.28	10.8178	9.81	10.48	126599	{}	\N	2020-10-12 15:31:35.936753
54639	LMB	2020-10-05	10.5	10.9	9.27	10.04	247579	{}	\N	2020-10-12 15:31:35.941097
54640	LMB	2020-10-06	10.18	10.4	9.83	9.98	101985	{}	\N	2020-10-12 15:31:35.945266
54641	LMB	2020-10-07	10.1	10.12	9.6	9.6	111116	{}	\N	2020-10-12 15:31:35.94953
54642	LMB	2020-10-08	9.75	9.97	9.39	9.5	202850	{}	\N	2020-10-12 15:31:35.954754
54643	LMB	2020-10-09	9.5	10	9.5	9.5	214996	{}	\N	2020-10-12 15:31:35.959613
54644	LMB	2020-10-12	9.5	9.89	9.5	9.66	79641	{}	\N	2020-10-12 15:31:35.966443
54645	LODE	2020-09-28	1.14	1.15	1.1	1.14	248295	{}	\N	2020-10-12 15:31:37.189244
54646	LODE	2020-09-29	1.14	1.145	1.06	1.07	565208	{}	\N	2020-10-12 15:31:37.200593
54647	LODE	2020-09-30	1.06	1.1	1.03	1.08	553267	{}	\N	2020-10-12 15:31:37.208883
54648	LODE	2020-10-01	1.08	1.125	1.04	1.08	724250	{}	\N	2020-10-12 15:31:37.215563
54649	LODE	2020-10-02	1.04	1.08	1.02	1.08	274666	{}	\N	2020-10-12 15:31:37.221426
54650	LODE	2020-10-05	1.05	1.1	1.05	1.07	221500	{}	\N	2020-10-12 15:31:37.226035
54651	LODE	2020-10-06	1.11	1.11	1.03	1.04	542028	{}	\N	2020-10-12 15:31:37.231445
54652	LODE	2020-10-07	1.09	1.15	1.0418	1.05	876044	{}	\N	2020-10-12 15:31:37.236449
54653	LODE	2020-10-08	1.1	1.11	1.06	1.08	339642	{}	\N	2020-10-12 15:31:37.241463
54654	LODE	2020-10-09	1.12	1.13	1.08	1.1	591276	{}	\N	2020-10-12 15:31:37.247148
54655	LODE	2020-10-12	1.14	1.14	1.1	1.11	128206	{}	\N	2020-10-12 15:31:37.252052
54656	LRGF	2020-09-28	32.93	33.159	32.92	33.09	68965	{}	\N	2020-10-12 15:31:39.353906
54657	LRGF	2020-09-29	33.13	33.21	32.95	33.02	37673	{}	\N	2020-10-12 15:31:39.364268
54658	LRGF	2020-09-30	33.04	33.5	33.0281	33.2	58437	{}	\N	2020-10-12 15:31:39.371754
54659	LRGF	2020-10-01	33.37	33.52	33.27	33.43	56685	{}	\N	2020-10-12 15:31:39.378251
54660	LRGF	2020-10-02	32.89	33.44	32.89	33.29	55346	{}	\N	2020-10-12 15:31:39.383417
54661	LRGF	2020-10-05	33.54	34.04	33.54	34.01	41475	{}	\N	2020-10-12 15:31:39.38952
54662	LRGF	2020-10-06	34.07	34.28	33.62	33.6701	60938	{}	\N	2020-10-12 15:31:39.39475
54663	LRGF	2020-10-07	33.97	34.33	33.97	34.28	59832	{}	\N	2020-10-12 15:31:39.399452
54664	LRGF	2020-10-08	34.49	34.58	34.43	34.58	29895	{}	\N	2020-10-12 15:31:39.405495
54665	LRGF	2020-10-09	34.76	34.8753	34.69	34.75	81751	{}	\N	2020-10-12 15:31:39.410512
54666	LRGF	2020-10-12	35.03	35.18	34.9501	35.17	34502	{}	\N	2020-10-12 15:31:39.415651
54667	LSXMK	2020-09-28	33.35	33.92	33.05	33.48	919175	{}	\N	2020-10-12 15:31:40.511081
54668	LSXMK	2020-09-29	33.47	33.56	32.75	33.24	557263	{}	\N	2020-10-12 15:31:40.522052
54669	LSXMK	2020-09-30	33.27	33.77	32.94	33.08	912356	{}	\N	2020-10-12 15:31:40.533498
54670	LSXMK	2020-10-01	33.3	34.83	33.3	34.79	851325	{}	\N	2020-10-12 15:31:40.540645
54671	LSXMK	2020-10-02	33.7	34.92	33.7	34.24	607020	{}	\N	2020-10-12 15:31:40.546282
54672	LSXMK	2020-10-05	34.61	34.8199	33.81	34.44	769414	{}	\N	2020-10-12 15:31:40.551682
54673	LSXMK	2020-10-06	34.52	34.76	34.01	34.38	2040008	{}	\N	2020-10-12 15:31:40.559315
54674	LSXMK	2020-10-07	35.56	36.19	35.2481	35.495	745619	{}	\N	2020-10-12 15:31:40.565059
54675	LSXMK	2020-10-08	35.73	36.2	35.381	36.15	764797	{}	\N	2020-10-12 15:31:40.570503
54676	LSXMK	2020-10-09	36.36	36.55	35.08	35.14	747599	{}	\N	2020-10-12 15:31:40.576217
54677	LSXMK	2020-10-12	35.33	35.61	35.06	35.4	197718	{}	\N	2020-10-12 15:31:40.581044
54678	LCAHW	2020-09-28	5.21	5.24	4.7349	4.8	410865	{}	\N	2020-10-12 15:31:41.585957
54679	LCAHW	2020-09-29	4.93	4.93	4.52	4.62	296326	{}	\N	2020-10-12 15:31:41.597806
54680	LCAHW	2020-09-30	4.31	4.8355	4.26	4.39	291870	{}	\N	2020-10-12 15:31:41.605425
54681	LCAHW	2020-10-01	4.6	4.85	4.39	4.62	335956	{}	\N	2020-10-12 15:31:41.612787
54682	LCAHW	2020-10-02	4.25	4.7198	4.25	4.65	140079	{}	\N	2020-10-12 15:31:41.618861
54683	LCAHW	2020-10-05	4.8	4.8	4.11	4.4778	399781	{}	\N	2020-10-12 15:31:41.624495
54684	LCAHW	2020-10-06	4.4	4.4199	3.7	4.04	389019	{}	\N	2020-10-12 15:31:41.631052
54685	LCAHW	2020-10-07	3.96	4.27	3.81	4.2482	191478	{}	\N	2020-10-12 15:31:41.639624
54686	LCAHW	2020-10-08	4.26	4.3799	4.01	4.27	93833	{}	\N	2020-10-12 15:31:41.647427
54687	LCAHW	2020-10-09	4.3	4.3121	4.13	4.26	122580	{}	\N	2020-10-12 15:31:41.653461
54688	LCAHW	2020-10-12	4.25	4.2638	4.07	4.085	47861	{}	\N	2020-10-12 15:31:41.659427
54689	LDRS	2020-09-28	24.112	24.195	24.112	24.195	375	{}	\N	2020-10-12 15:31:42.623461
54690	LDRS	2020-09-29	24.112	24.16	24.112	24.16	1044	{}	\N	2020-10-12 15:31:42.634586
54691	LDRS	2020-09-30	24.16	24.16	24.16	24.16	410	{}	\N	2020-10-12 15:31:42.641834
54692	LDRS	2020-10-01	24.14	24.14	24.14	24.14	48	{}	\N	2020-10-12 15:31:42.648706
54693	LDRS	2020-10-02	24.1192	24.2	24.1192	24.18	4273	{}	\N	2020-10-12 15:31:42.654354
54694	LDRS	2020-10-05	24.25	24.65	24.25	24.63	4681	{}	\N	2020-10-12 15:31:42.659373
54695	LDRS	2020-10-06	24.83	24.93	24.5696	24.5696	2062	{}	\N	2020-10-12 15:31:42.664069
54696	LDRS	2020-10-07	24.9999	25.1254	24.9999	25.1254	1104	{}	\N	2020-10-12 15:31:42.668622
54697	LDRS	2020-10-08	25.2813	25.3399	25.2751	25.2751	4434	{}	\N	2020-10-12 15:31:42.673186
54698	LDRS	2020-10-09	25.56	25.57	25.47	25.57	4085	{}	\N	2020-10-12 15:31:42.677619
54699	LDRS	2020-10-12	25.86	25.86	25.7101	25.786	3358	{}	\N	2020-10-12 15:31:42.682395
54700	LEJU	2020-09-28	2.0087	2.1	1.98	2.09	30628	{}	\N	2020-10-12 15:31:44.162689
54701	LEJU	2020-09-29	2.01	2.05	1.95	1.97	148264	{}	\N	2020-10-12 15:31:44.174076
54702	LEJU	2020-09-30	1.96	2.03	1.96	1.975	68290	{}	\N	2020-10-12 15:31:44.181655
54703	LEJU	2020-10-01	2.0076	2.01	1.97	2.01	40370	{}	\N	2020-10-12 15:31:44.187853
54704	LEJU	2020-10-02	1.98	2.06	1.97	2.04	114598	{}	\N	2020-10-12 15:31:44.193745
54705	LEJU	2020-10-05	1.97	2.11	1.97	2.08	63254	{}	\N	2020-10-12 15:31:44.199029
54706	LEJU	2020-10-06	2.05	2.15	2.0294	2.06	56150	{}	\N	2020-10-12 15:31:44.203827
54707	LEJU	2020-10-07	2.0827	2.17	2.05	2.14	143177	{}	\N	2020-10-12 15:31:44.208957
54708	LEJU	2020-10-08	2.19	2.24	2.15	2.17	43900	{}	\N	2020-10-12 15:31:44.213828
54709	LEJU	2020-10-09	2.141	2.24	2.141	2.2	53272	{}	\N	2020-10-12 15:31:44.21867
54710	LEJU	2020-10-12	2.2	2.21	2.18	2.204	24235	{}	\N	2020-10-12 15:31:44.223763
54711	LFEQ	2020-09-28	31.4491	31.4491	31.4491	31.4491	6	{}	\N	2020-10-12 15:31:45.564467
54712	LFEQ	2020-09-29	31.5199	31.5199	31.33	31.38	3116	{}	\N	2020-10-12 15:31:45.575894
54713	LFEQ	2020-09-30	31.67	31.67	31.54	31.54	1702	{}	\N	2020-10-12 15:31:45.583581
54714	LFEQ	2020-10-01	31.7261	31.7261	31.7261	31.7261	62	{}	\N	2020-10-12 15:31:45.590475
54715	LFEQ	2020-10-02	31.45	31.59	31.45	31.59	1712	{}	\N	2020-10-12 15:31:45.597021
54716	LFEQ	2020-10-05	31.8963	31.998	31.87	31.998	786	{}	\N	2020-10-12 15:31:45.602384
54717	LFEQ	2020-10-06	31.97	32.115	31.56	31.56	16571	{}	\N	2020-10-12 15:31:45.607628
54718	LFEQ	2020-10-07	31.95	32.1283	31.95	32.1283	9856	{}	\N	2020-10-12 15:31:45.613369
54719	LFEQ	2020-10-08	32.3672	32.3672	32.3672	32.3672	43	{}	\N	2020-10-12 15:31:45.618868
54720	LFEQ	2020-10-09	32.5863	32.66	32.5	32.62	7897	{}	\N	2020-10-12 15:31:45.624341
54721	LFEQ	2020-10-12	32.8874	32.95	32.8874	32.95	1014	{}	\N	2020-10-12 15:31:45.629636
54722	LGL	2020-09-28	8.3501	8.99	8.3501	8.93	10999	{}	\N	2020-10-12 15:31:46.923757
54723	LGL	2020-09-29	8.765	8.84	8.685	8.8	2302	{}	\N	2020-10-12 15:31:46.935106
54724	LGL	2020-09-30	8.9885	8.9885	8.87	8.92	6253	{}	\N	2020-10-12 15:31:46.942562
54725	LGL	2020-10-01	8.6501	8.805	8.6501	8.7	4244	{}	\N	2020-10-12 15:31:46.948458
54726	LGL	2020-10-02	8.6	8.81	8.6	8.61	2146	{}	\N	2020-10-12 15:31:46.954166
54727	LGL	2020-10-05	8.61	8.79	8.61	8.67	6995	{}	\N	2020-10-12 15:31:46.959199
54728	LGL	2020-10-06	8.765	8.86	8.67	8.67	4250	{}	\N	2020-10-12 15:31:46.964161
54729	LGL	2020-10-07	8.76	8.85	8.65	8.7	13701	{}	\N	2020-10-12 15:31:46.968746
54730	LGL	2020-10-08	8.75	8.81	8.69	8.81	2801	{}	\N	2020-10-12 15:31:46.97433
54731	LGL	2020-10-09	8.82	9.2	8.82	9.04	23948	{}	\N	2020-10-12 15:31:46.979701
54732	LGL	2020-10-12	9.07	9.19	9.05	9.11	995	{}	\N	2020-10-12 15:31:46.984844
54733	LILA	2020-09-28	8.05	8.52	7.9076	8.33	288995	{}	\N	2020-10-12 15:31:48.128845
54734	LILA	2020-09-29	8.3	8.3673	8.1	8.35	135262	{}	\N	2020-10-12 15:31:48.138204
54735	LILA	2020-09-30	8.31	8.3914	8.11	8.25	230859	{}	\N	2020-10-12 15:31:48.144719
54736	LILA	2020-10-01	8.32	8.84	8.268	8.82	294909	{}	\N	2020-10-12 15:31:48.151039
54737	LILA	2020-10-02	8.79	8.92	8.59	8.92	336573	{}	\N	2020-10-12 15:31:48.156481
54738	LILA	2020-10-05	8.92	9.25	8.92	9.21	264146	{}	\N	2020-10-12 15:31:48.16092
54739	LILA	2020-10-06	9.25	9.38	8.77	8.81	197497	{}	\N	2020-10-12 15:31:48.165363
54740	LILA	2020-10-07	9.15	9.28	8.93	9.17	468111	{}	\N	2020-10-12 15:31:48.169593
54741	LILA	2020-10-08	9.33	9.76	9.25	9.75	529881	{}	\N	2020-10-12 15:31:48.173853
54742	LILA	2020-10-09	10.39	11.07	10.3	11.05	1112401	{}	\N	2020-10-12 15:31:48.178248
54743	LILA	2020-10-12	10.97	11.03	10.67	10.99	153637	{}	\N	2020-10-12 15:31:48.184186
54744	LITE	2020-09-28	75.15	75.61	73.92	75.14	1064900	{}	\N	2020-10-12 15:31:49.411933
54745	LITE	2020-09-29	75	75.88	74.33	74.61	758995	{}	\N	2020-10-12 15:31:49.423868
54746	LITE	2020-09-30	74.32	76.8	74.18	75.13	1263402	{}	\N	2020-10-12 15:31:49.43119
54747	LITE	2020-10-01	76.08	77.5	75.88	77.01	916356	{}	\N	2020-10-12 15:31:49.438266
54748	LITE	2020-10-02	75.54	77.99	75.175	77.37	1281742	{}	\N	2020-10-12 15:31:49.444914
54749	LITE	2020-10-05	78.12	80.76	77.75	80.68	1083606	{}	\N	2020-10-12 15:31:49.450409
54750	LITE	2020-10-06	81.06	82.81	79.88	80.45	1007614	{}	\N	2020-10-12 15:31:49.455552
54751	LITE	2020-10-07	81.1264	82.1299	79.92	80.49	943892	{}	\N	2020-10-12 15:31:49.461485
54752	LITE	2020-10-08	81.8239	83.88	81.61	82.05	717799	{}	\N	2020-10-12 15:31:49.467205
54753	LITE	2020-10-09	83.2	83.88	82.51	82.74	842523	{}	\N	2020-10-12 15:31:49.472947
54754	LITE	2020-10-12	83.8	85.19	82.63	82.8279	249478	{}	\N	2020-10-12 15:31:49.47861
54755	LND	2020-09-28	4.04	4.04	3.85	3.85	7357	{}	\N	2020-10-12 15:31:51.474365
54756	LND	2020-09-29	3.94	3.94	3.869	3.88	7536	{}	\N	2020-10-12 15:31:51.485231
54757	LND	2020-09-30	3.98	4.02	3.98	4	20152	{}	\N	2020-10-12 15:31:51.492514
54758	LND	2020-10-01	4	4	3.85	3.92	7608	{}	\N	2020-10-12 15:31:51.499282
54759	LND	2020-10-02	3.88	3.98	3.88	3.97	3718	{}	\N	2020-10-12 15:31:51.505432
54760	LND	2020-10-05	4.02	4.06	3.92	3.92	3334	{}	\N	2020-10-12 15:31:51.51059
54761	LND	2020-10-06	4.03	4.03	4	4.02	2916	{}	\N	2020-10-12 15:31:51.515455
54762	LND	2020-10-07	4.03	4.03	3.93	4	1784	{}	\N	2020-10-12 15:31:51.52046
54763	LND	2020-10-08	4.03	4.04	4.03	4.04	2572	{}	\N	2020-10-12 15:31:51.525791
54764	LND	2020-10-09	4.1	4.1	4.0399	4.0399	3730	{}	\N	2020-10-12 15:31:51.53086
54765	LND	2020-10-12	4.01	4.1	4.01	4.04	6858	{}	\N	2020-10-12 15:31:51.535562
54766	LOOP	2020-09-28	12.41	13.1773	12.3376	12.7	66146	{}	\N	2020-10-12 15:31:52.596442
54767	LOOP	2020-09-29	12.9	12.96	12.4201	12.85	41165	{}	\N	2020-10-12 15:31:52.606348
54768	LOOP	2020-09-30	12.86	13.69	12.73	13.66	107675	{}	\N	2020-10-12 15:31:52.613057
54769	LOOP	2020-10-01	13.74	13.7799	13.48	13.52	79524	{}	\N	2020-10-12 15:31:52.619434
54770	LOOP	2020-10-02	13.35	13.8765	13.28	13.28	36049	{}	\N	2020-10-12 15:31:52.625229
54771	LOOP	2020-10-05	13.41	13.895	13.33	13.37	50094	{}	\N	2020-10-12 15:31:52.63022
54772	LOOP	2020-10-06	13.43	13.55	13.04	13.04	44589	{}	\N	2020-10-12 15:31:52.634909
54773	LOOP	2020-10-07	13.04	13.7974	13.04	13.49	107627	{}	\N	2020-10-12 15:31:52.640044
54774	LOOP	2020-10-08	13.07	13.42	12.3201	12.4	112577	{}	\N	2020-10-12 15:31:52.64505
54775	LOOP	2020-10-09	12.37	12.6899	12.035	12.29	88352	{}	\N	2020-10-12 15:31:52.650054
54776	LOOP	2020-10-12	12.35	12.7	11.57	11.685	26727	{}	\N	2020-10-12 15:31:52.655525
54777	LOGI	2020-09-28	73.34	75.08	73.145	74.98	652346	{}	\N	2020-10-12 15:31:53.773318
54778	LOGI	2020-09-29	75.66	77.51	75.6	77.14	613139	{}	\N	2020-10-12 15:31:53.782062
54779	LOGI	2020-09-30	77.15	78.49	76.76	77.31	705856	{}	\N	2020-10-12 15:31:53.789026
54780	LOGI	2020-10-01	78.5	78.9	77.4	78.52	478862	{}	\N	2020-10-12 15:31:53.795335
54781	LOGI	2020-10-02	77.46	78.49	77.38	77.78	353404	{}	\N	2020-10-12 15:31:53.8014
54782	LOGI	2020-10-05	79.32	80.87	79.31	80.77	443267	{}	\N	2020-10-12 15:31:53.806712
54783	LOGI	2020-10-06	75.89	77.43	75.11	75.83	1615873	{}	\N	2020-10-12 15:31:53.812255
54784	LOGI	2020-10-07	76.61	76.7885	75.9	76.3	308603	{}	\N	2020-10-12 15:31:53.817629
54785	LOGI	2020-10-08	76.96	77.3917	76.7945	77.01	352833	{}	\N	2020-10-12 15:31:53.822749
54786	LOGI	2020-10-09	78.4	79.6584	78.25	79.61	310068	{}	\N	2020-10-12 15:31:53.827852
54787	LOGI	2020-10-12	80.28	80.7	80.0694	80.59	125427	{}	\N	2020-10-12 15:31:53.832948
54788	LPSN	2020-09-28	51.88	53.5	51.88	52.92	696642	{}	\N	2020-10-12 15:31:54.820593
54789	LPSN	2020-09-29	53.53	53.53	51.81	52.44	377394	{}	\N	2020-10-12 15:31:54.829892
54790	LPSN	2020-09-30	52.28	53.8499	51.19	51.99	869385	{}	\N	2020-10-12 15:31:54.836552
54791	LPSN	2020-10-01	53.56	53.98	51.25	53.53	771031	{}	\N	2020-10-12 15:31:54.842562
54792	LPSN	2020-10-02	51.08	54.09	51.08	52.91	476515	{}	\N	2020-10-12 15:31:54.848231
54793	LPSN	2020-10-05	53.63	54.88	53.07	53.68	434375	{}	\N	2020-10-12 15:31:54.853774
54794	LPSN	2020-10-06	54.19	56.42	54.09	55.62	1941184	{}	\N	2020-10-12 15:31:54.859002
54795	LPSN	2020-10-07	56.06	57.29	55.77	56.72	682295	{}	\N	2020-10-12 15:31:54.864097
54796	LPSN	2020-10-08	57.48	57.72	55.86	56.02	322232	{}	\N	2020-10-12 15:31:54.869626
54797	LPSN	2020-10-09	57	57.79	56.43	57.48	363960	{}	\N	2020-10-12 15:31:54.875389
54798	LPSN	2020-10-12	58.31	58.71	57.4	58.38	103097	{}	\N	2020-10-12 15:31:54.881481
54799	LRCX	2020-09-28	331.42	337.85	328.7	336.61	1684605	{}	\N	2020-10-12 15:31:56.021543
54800	LRCX	2020-09-29	333.9275	342.56	333.785	337.71	1709286	{}	\N	2020-10-12 15:31:56.032539
54801	LRCX	2020-09-30	335.24	338.23	330.2	331.75	1802014	{}	\N	2020-10-12 15:31:56.039385
54802	LRCX	2020-10-01	339.14	339.25	333.14	339.25	1427712	{}	\N	2020-10-12 15:31:56.045311
54803	LRCX	2020-10-02	329.36	335.8599	328.56	328.71	1078051	{}	\N	2020-10-12 15:31:56.050716
54804	LRCX	2020-10-05	333	343.05	332.61	342.23	1509951	{}	\N	2020-10-12 15:31:56.055871
54805	LRCX	2020-10-06	340.96	351.08	339.41	340.92	1138124	{}	\N	2020-10-12 15:31:56.061218
54806	LRCX	2020-10-07	347.4	351.5	343.51	345.77	1081513	{}	\N	2020-10-12 15:31:56.065563
54807	LRCX	2020-10-08	348.37	355.67	347	355.43	1148513	{}	\N	2020-10-12 15:31:56.070364
54808	LRCX	2020-10-09	360.9	364.66	359.17	362.71	1109751	{}	\N	2020-10-12 15:31:56.075297
54809	LRCX	2020-10-12	369.43	371.75	367.14	370.25	540735	{}	\N	2020-10-12 15:31:56.07982
54810	LSTR	2020-09-28	127.14	128.1	126.14	127.54	261739	{}	\N	2020-10-12 15:31:57.110812
54811	LSTR	2020-09-29	127.12	128.64	126.55	126.64	178484	{}	\N	2020-10-12 15:31:57.121291
54812	LSTR	2020-09-30	127	127.63	124.83	125.49	267619	{}	\N	2020-10-12 15:31:57.12894
54813	LSTR	2020-10-01	125.08	126.38	124.14	125.18	307367	{}	\N	2020-10-12 15:31:57.135894
54814	LSTR	2020-10-02	120.97	126.245	120.921	125.25	282820	{}	\N	2020-10-12 15:31:57.143072
54815	LSTR	2020-10-05	126.16	128.88	125.73	128.46	295998	{}	\N	2020-10-12 15:31:57.148806
54816	LSTR	2020-10-06	127.2	128.704	125.985	127.12	313385	{}	\N	2020-10-12 15:31:57.154419
54817	LSTR	2020-10-07	127.18	129.1	126.755	128.72	231427	{}	\N	2020-10-12 15:31:57.160046
54818	LSTR	2020-10-08	129.37	132.09	128.78	131.82	208379	{}	\N	2020-10-12 15:31:57.16752
54819	LSTR	2020-10-09	132.07	132.92	131.13	131.94	261588	{}	\N	2020-10-12 15:31:57.172796
54820	LSTR	2020-10-12	132.19	133.38	131.63	133.13	26050	{}	\N	2020-10-12 15:31:57.178481
54821	LEE	2020-09-28	0.82	0.8661	0.82	0.85	53968	{}	\N	2020-10-12 15:31:58.996814
54822	LEE	2020-09-29	0.83	0.8581	0.811	0.83	262134	{}	\N	2020-10-12 15:31:59.006363
54823	LEE	2020-09-30	0.8395	0.8639	0.8302	0.8388	272502	{}	\N	2020-10-12 15:31:59.014211
54824	LEE	2020-10-01	0.8684	0.8799	0.839	0.8538	99332	{}	\N	2020-10-12 15:31:59.02036
54825	LEE	2020-10-02	0.85	0.8989	0.85	0.8915	114371	{}	\N	2020-10-12 15:31:59.026668
54826	LEE	2020-10-05	0.872	0.9	0.85	0.88	249058	{}	\N	2020-10-12 15:31:59.032519
54827	LEE	2020-10-06	0.8802	0.9	0.88	0.8808	234771	{}	\N	2020-10-12 15:31:59.038223
54828	LEE	2020-10-07	0.87	0.9201	0.86	0.8993	326316	{}	\N	2020-10-12 15:31:59.043581
54829	LEE	2020-10-08	0.91	0.92	0.90011	0.91	192340	{}	\N	2020-10-12 15:31:59.049297
54830	LEE	2020-10-09	0.91	0.92	0.88	0.9001	49572	{}	\N	2020-10-12 15:31:59.054395
54831	LEE	2020-10-12	0.872	0.925	0.872	0.8801	56646	{}	\N	2020-10-12 15:31:59.060148
54832	LINC	2020-09-28	5.57	5.6599	5.3	5.39	132768	{}	\N	2020-10-12 15:32:00.731237
54833	LINC	2020-09-29	5.54	5.61	5.21	5.5201	144275	{}	\N	2020-10-12 15:32:00.738294
54834	LINC	2020-09-30	5.56	5.57	5.24	5.52	159956	{}	\N	2020-10-12 15:32:00.74376
54835	LINC	2020-10-01	5.55	5.75	5.43	5.73	198430	{}	\N	2020-10-12 15:32:00.75076
54836	LINC	2020-10-02	5.62	5.84	5.5	5.68	122100	{}	\N	2020-10-12 15:32:00.758048
54837	LINC	2020-10-05	5.69	5.73	5.42	5.49	151482	{}	\N	2020-10-12 15:32:00.765412
54838	LINC	2020-10-06	5.6	6	5.56	5.81	190407	{}	\N	2020-10-12 15:32:00.771033
54839	LINC	2020-10-07	5.9	5.9899	5.67	5.77	135967	{}	\N	2020-10-12 15:32:00.777251
54840	LINC	2020-10-08	5.79	5.96	5.61	5.66	76701	{}	\N	2020-10-12 15:32:00.783873
54841	LINC	2020-10-09	5.73	5.92	5.67	5.81	68810	{}	\N	2020-10-12 15:32:00.790901
54842	LINC	2020-10-12	5.79	5.79	5.3947	5.42	66497	{}	\N	2020-10-12 15:32:00.79742
54843	LIVN	2020-09-28	43.39	44.4	42.86	43.82	401081	{}	\N	2020-10-12 15:32:02.015697
54844	LIVN	2020-09-29	44.25	44.69	42.99	43.37	488895	{}	\N	2020-10-12 15:32:02.022127
54845	LIVN	2020-09-30	43.59	45.73	43.08	45.21	532334	{}	\N	2020-10-12 15:32:02.027606
54846	LIVN	2020-10-01	45.19	46.36	45.16	46.01	534918	{}	\N	2020-10-12 15:32:02.034744
54847	LIVN	2020-10-02	45.24	47	45.24	46.67	487375	{}	\N	2020-10-12 15:32:02.04069
54848	LIVN	2020-10-05	47.17	48.38	45.78	48.1	440995	{}	\N	2020-10-12 15:32:02.046665
54849	LIVN	2020-10-06	48.105	49.6	48.105	48.65	709950	{}	\N	2020-10-12 15:32:02.052838
54850	LIVN	2020-10-07	49.07	50.14	48.705	49.8	625430	{}	\N	2020-10-12 15:32:02.059666
54851	LIVN	2020-10-08	50.36	50.57	49.38	50.29	559295	{}	\N	2020-10-12 15:32:02.065344
54852	LIVN	2020-10-09	50.77	51.16	49.945	50.06	324987	{}	\N	2020-10-12 15:32:02.071505
54853	LIVN	2020-10-12	50.1	53.3	50.005	53.21	213038	{}	\N	2020-10-12 15:32:02.078637
54854	LMRKO	2020-09-28	24.6	24.8694	24.6	24.75	6454	{}	\N	2020-10-12 15:32:04.747922
54855	LMRKO	2020-09-29	24.8	24.86	24.55	24.6	3578	{}	\N	2020-10-12 15:32:04.755954
54856	LMRKO	2020-09-30	24.5829	24.65	24.4901	24.55	10711	{}	\N	2020-10-12 15:32:04.762421
54857	LMRKO	2020-10-01	24.5	24.5	24.4	24.4	7137	{}	\N	2020-10-12 15:32:04.769667
54858	LMRKO	2020-10-02	24.62	24.65	24.28	24.41	3679	{}	\N	2020-10-12 15:32:04.776483
54859	LMRKO	2020-10-05	24.44	24.6	24.31	24.525	3262	{}	\N	2020-10-12 15:32:04.784661
54860	LMRKO	2020-10-06	24.69	24.82	24.4476	24.82	13648	{}	\N	2020-10-12 15:32:04.79455
54861	LMRKO	2020-10-07	24.7	24.89	24.7	24.82	4396	{}	\N	2020-10-12 15:32:04.803643
54862	LMRKO	2020-10-08	24.89	24.95	24.832	24.88	10023	{}	\N	2020-10-12 15:32:04.809036
54863	LMRKO	2020-10-09	24.9976	25	24.99	25	1660	{}	\N	2020-10-12 15:32:04.817246
54864	LMRKO	2020-10-12	24.82	25	24.8	24.9503	2519	{}	\N	2020-10-12 15:32:04.822663
54865	LOB	2020-09-28	24.51	25.173	24.24	25.05	131423	{}	\N	2020-10-12 15:32:05.703608
54866	LOB	2020-09-29	25.66	25.94	25.1	25.79	187901	{}	\N	2020-10-12 15:32:05.71042
54867	LOB	2020-09-30	25.93	26.17	25.21	25.33	114347	{}	\N	2020-10-12 15:32:05.717661
54868	LOB	2020-10-01	25.38	26.04	24.95	26.01	166940	{}	\N	2020-10-12 15:32:05.724856
54869	LOB	2020-10-02	25.28	27.02	25.27	26.9	227521	{}	\N	2020-10-12 15:32:05.732333
54870	LOB	2020-10-05	28.13	29.66	28	29.57	405730	{}	\N	2020-10-12 15:32:05.73965
54871	LOB	2020-10-06	30	30.66	28.38	28.51	403870	{}	\N	2020-10-12 15:32:05.745942
54872	LOB	2020-10-07	29.21	30.88	29.21	30.04	312570	{}	\N	2020-10-12 15:32:05.752496
54873	LOB	2020-10-08	30.45	31.27	30.2401	30.89	219245	{}	\N	2020-10-12 15:32:05.758146
54874	LOB	2020-10-09	31.36	31.82	30.62	30.75	230115	{}	\N	2020-10-12 15:32:05.764801
54875	LOB	2020-10-12	30.99	31.21	30.8	30.95	57690	{}	\N	2020-10-12 15:32:05.771918
54876	LORL	2020-09-28	18.46	18.86	18.39	18.55	85685	{}	\N	2020-10-12 15:32:06.930169
54877	LORL	2020-09-29	18.44	18.63	18.13	18.53	92874	{}	\N	2020-10-12 15:32:06.937835
54878	LORL	2020-09-30	18.66	18.79	18.18	18.3	93124	{}	\N	2020-10-12 15:32:06.943965
54879	LORL	2020-10-01	18.42	18.42	17.685	18.4	116863	{}	\N	2020-10-12 15:32:06.951471
54880	LORL	2020-10-02	17.92	18.55	17.75	18.29	64341	{}	\N	2020-10-12 15:32:06.957321
54881	LORL	2020-10-05	18.5	19	18.34	18.94	85655	{}	\N	2020-10-12 15:32:06.964071
54882	LORL	2020-10-06	19.11	19.8	18.8672	18.97	94082	{}	\N	2020-10-12 15:32:06.969457
54883	LORL	2020-10-07	19.18	19.57	18.78	19.51	117809	{}	\N	2020-10-12 15:32:06.97496
54884	LORL	2020-10-08	19.8	20.3	19.54	20.23	111074	{}	\N	2020-10-12 15:32:06.980387
54885	LORL	2020-10-09	20.46	21	20.31	20.68	96687	{}	\N	2020-10-12 15:32:06.987075
54886	LORL	2020-10-12	20.79	21.46	20.64	21.17	41326	{}	\N	2020-10-12 15:32:06.992356
54887	LPG	2020-09-28	7.68	7.86	7.68	7.75	275513	{}	\N	2020-10-12 15:32:07.858666
54888	LPG	2020-09-29	7.75	7.78	7.5301	7.71	673076	{}	\N	2020-10-12 15:32:07.87125
54889	LPG	2020-09-30	7.77	8.19	7.77	8.01	1311515	{}	\N	2020-10-12 15:32:07.882876
54890	LPG	2020-10-01	8.09	8.19	7.93	8.18	579299	{}	\N	2020-10-12 15:32:07.892149
54891	LPG	2020-10-02	8.05	8.485	8.05	8.42	606667	{}	\N	2020-10-12 15:32:07.901928
54892	LPG	2020-10-05	8.52	8.84	8.47	8.68	678575	{}	\N	2020-10-12 15:32:07.909954
54893	LPG	2020-10-06	8.79	8.88	8.42	8.43	647593	{}	\N	2020-10-12 15:32:07.918906
54894	LPG	2020-10-07	8.44	8.52	8.15	8.32	734871	{}	\N	2020-10-12 15:32:07.927242
54895	LPG	2020-10-08	8.41	8.81	8.41	8.76	375742	{}	\N	2020-10-12 15:32:07.935284
54896	LPG	2020-10-09	8.83	8.88	8.61	8.77	553328	{}	\N	2020-10-12 15:32:07.94208
54897	LPG	2020-10-12	9.18	9.2	8.84	9.03	536503	{}	\N	2020-10-12 15:32:07.949845
54898	LQDH	2020-09-28	91.015	91.27	90.86	91.27	4342	{}	\N	2020-10-12 15:32:09.110988
54899	LQDH	2020-09-29	91.2812	91.33	91.16	91.16	5422	{}	\N	2020-10-12 15:32:09.120832
54900	LQDH	2020-09-30	91.2887	91.3	91.2	91.28	4506	{}	\N	2020-10-12 15:32:09.127633
54901	LQDH	2020-10-01	91.55	91.86	91.32	91.73	640128	{}	\N	2020-10-12 15:32:09.134583
54902	LQDH	2020-10-02	91.33	91.61	91.23	91.59	171840	{}	\N	2020-10-12 15:32:09.140531
54903	LQDH	2020-10-05	91.38	91.86	91.38	91.83	17469	{}	\N	2020-10-12 15:32:09.14708
54904	LQDH	2020-10-06	91.97	91.97	91.6644	91.72	61687	{}	\N	2020-10-12 15:32:09.153224
54905	LQDH	2020-10-07	91.72	91.93	91.14	91.64	333088	{}	\N	2020-10-12 15:32:09.158953
54906	LQDH	2020-10-08	92	92.25	91.39	91.95	350785	{}	\N	2020-10-12 15:32:09.166007
54907	LQDH	2020-10-09	91.9	92.2399	91.54	92.1825	49383	{}	\N	2020-10-12 15:32:09.172504
54908	LQDH	2020-10-12	92.49	92.6299	92.49	92.6299	4677	{}	\N	2020-10-12 15:32:09.177801
54909	LSI	2020-09-28	106.58	108.02	105.75	107.23	446024	{}	\N	2020-10-12 15:32:10.195998
54910	LSI	2020-09-29	106.93	107.76	105.21	106.39	626752	{}	\N	2020-10-12 15:32:10.203096
54911	LSI	2020-09-30	107.53	107.53	104.68	105.27	1165666	{}	\N	2020-10-12 15:32:10.209315
54912	LSI	2020-10-01	105.74	108.26	105.74	107.91	881090	{}	\N	2020-10-12 15:32:10.217592
54913	LSI	2020-10-02	106.49	111.72	106.03	111.26	958527	{}	\N	2020-10-12 15:32:10.225774
54914	LSI	2020-10-05	111.61	112.99	109.99	112.63	914116	{}	\N	2020-10-12 15:32:10.233445
54915	LSI	2020-10-06	115	115.02	112.18	113.67	1468477	{}	\N	2020-10-12 15:32:10.275865
54916	LSI	2020-10-07	113.99	115.17	113.22	114.18	638318	{}	\N	2020-10-12 15:32:10.282763
54917	LSI	2020-10-08	114.74	115.49	114.02	115.4	695113	{}	\N	2020-10-12 15:32:10.289446
54918	LSI	2020-10-09	114.84	114.84	112.75	113.68	492293	{}	\N	2020-10-12 15:32:10.295551
54919	LSI	2020-10-12	113.22	114	112.49	113.73	102127	{}	\N	2020-10-12 15:32:10.302041
54920	LTPZ	2020-09-28	87.92	88.02	87.725	87.91	74915	{}	\N	2020-10-12 15:32:11.241883
54921	LTPZ	2020-09-29	88.23	88.51	88.05	88.21	101190	{}	\N	2020-10-12 15:32:11.248001
54922	LTPZ	2020-09-30	88.02	88.19	87.3	87.41	67537	{}	\N	2020-10-12 15:32:11.253635
54923	LTPZ	2020-10-01	86.96	87.75	86.8033	87.49	79685	{}	\N	2020-10-12 15:32:11.259727
54924	LTPZ	2020-10-02	87.21	87.4	87.0766	87.27	37212	{}	\N	2020-10-12 15:32:11.266224
54925	LTPZ	2020-10-05	86.78	87	86.2	86.22	72040	{}	\N	2020-10-12 15:32:11.273383
54926	LTPZ	2020-10-06	86.21	86.54	85.56	86.23	82781	{}	\N	2020-10-12 15:32:11.280218
54927	LTPZ	2020-10-07	85.99	86.3514	85.5805	85.82	82184	{}	\N	2020-10-12 15:32:11.289099
54928	LTPZ	2020-10-08	86.37	86.79	86.15	86.78	50980	{}	\N	2020-10-12 15:32:11.294727
54929	LTPZ	2020-10-09	86.59	86.9183	86.28	86.82	46750	{}	\N	2020-10-12 15:32:11.300441
54930	LTPZ	2020-10-12	86.99	87.0687	86.875	87	53756	{}	\N	2020-10-12 15:32:11.30673
54931	LCUT	2020-09-28	9.14	9.1817	8.91	9.11	85903	{}	\N	2020-10-12 15:32:12.256357
54932	LCUT	2020-09-29	9.05	9.66	8.98	9.62	27078	{}	\N	2020-10-12 15:32:12.262045
54933	LCUT	2020-09-30	9.77	10	9.3323	9.55	9023	{}	\N	2020-10-12 15:32:12.268304
54934	LCUT	2020-10-01	9.735	9.9	9.33	9.57	10877	{}	\N	2020-10-12 15:32:12.275532
54935	LCUT	2020-10-02	9.38	9.73	9.38	9.73	19103	{}	\N	2020-10-12 15:32:12.284222
54936	LCUT	2020-10-05	9.74	10.45	9.72	10.31	38445	{}	\N	2020-10-12 15:32:12.290651
54937	LCUT	2020-10-06	10.26	10.73	10.26	10.38	18601	{}	\N	2020-10-12 15:32:12.296094
54938	LCUT	2020-10-07	10.52	10.84	10.48	10.72	23374	{}	\N	2020-10-12 15:32:12.304398
54939	LCUT	2020-10-08	10.79	10.79	10.55	10.61	19771	{}	\N	2020-10-12 15:32:12.310344
54940	LCUT	2020-10-09	10.55	10.74	9.91	10.01	29510	{}	\N	2020-10-12 15:32:12.321692
54941	LCUT	2020-10-12	10.12	10.6152	10.12	10.5	11888	{}	\N	2020-10-12 15:32:12.332162
54942	LEA	2020-09-28	107.27	109.76	106.48	107.35	472938	{}	\N	2020-10-12 15:32:13.264619
54943	LEA	2020-09-29	107.01	109.46	106.56	108.01	871939	{}	\N	2020-10-12 15:32:13.27092
54944	LEA	2020-09-30	108.49	111.21	108.31	109.05	948232	{}	\N	2020-10-12 15:32:13.275986
54945	LEA	2020-10-01	110.48	114.03	110.41	113.11	1127897	{}	\N	2020-10-12 15:32:13.281009
54946	LEA	2020-10-02	110.09	115.32	109.96	114.78	730505	{}	\N	2020-10-12 15:32:13.28869
54947	LEA	2020-10-05	116.96	120.6	115.94	119.91	762128	{}	\N	2020-10-12 15:32:13.294431
54948	LEA	2020-10-06	122.94	126.15	121.6	123.39	2030086	{}	\N	2020-10-12 15:32:13.299749
54949	LEA	2020-10-07	125.22	130.48	124.3	128.8	1225147	{}	\N	2020-10-12 15:32:13.305825
54950	LEA	2020-10-08	130.6	131.095	127.86	129.83	1310830	{}	\N	2020-10-12 15:32:13.311513
54951	LEA	2020-10-09	130.49	131.555	129	129.54	1517099	{}	\N	2020-10-12 15:32:13.317665
54952	LEA	2020-10-12	129.64	130.84	128.5	129.28	107310	{}	\N	2020-10-12 15:32:13.323924
54953	LGI	2020-09-28	15.71	15.845	15.7	15.76	29671	{}	\N	2020-10-12 15:32:15.894594
54954	LGI	2020-09-29	15.77	15.77	15.62	15.66	71808	{}	\N	2020-10-12 15:32:15.902597
54955	LGI	2020-09-30	15.61	15.79	15.52	15.52	119460	{}	\N	2020-10-12 15:32:15.909489
54956	LGI	2020-10-01	15.65	15.8	15.5	15.63	103139	{}	\N	2020-10-12 15:32:15.914992
54957	LGI	2020-10-02	15.4	15.57	15.25	15.47	96413	{}	\N	2020-10-12 15:32:15.920438
54958	LGI	2020-10-05	15.59	15.69	15.49	15.67	126807	{}	\N	2020-10-12 15:32:15.925847
54959	LGI	2020-10-06	15.65	16	15.65	15.93	85754	{}	\N	2020-10-12 15:32:15.930956
54960	LGI	2020-10-07	15.97	15.97	15.81	15.94	66110	{}	\N	2020-10-12 15:32:15.935967
54961	LGI	2020-10-08	15.98	16.1	15.91	15.98	73805	{}	\N	2020-10-12 15:32:15.941207
54962	LGI	2020-10-09	16.06	16.1496	15.94	16.06	54649	{}	\N	2020-10-12 15:32:15.946808
54963	LGI	2020-10-12	16.11	16.2	16.1	16.14	36540	{}	\N	2020-10-12 15:32:15.953233
54964	LILAK	2020-09-28	8	8.41	7.81	8.28	2731609	{}	\N	2020-10-12 15:32:18.179544
54965	LILAK	2020-09-29	8.22	8.51	8.02	8.25	1375002	{}	\N	2020-10-12 15:32:18.1857
54966	LILAK	2020-09-30	8.22	8.2747	8.02	8.14	3597113	{}	\N	2020-10-12 15:32:18.19506
54967	LILAK	2020-10-01	8.16	8.75	8.16	8.73	2685422	{}	\N	2020-10-12 15:32:18.203948
54968	LILAK	2020-10-02	8.51	8.86	8.3601	8.84	2290459	{}	\N	2020-10-12 15:32:18.211513
54969	LILAK	2020-10-05	8.92	9.155	8.84	9.14	1428154	{}	\N	2020-10-12 15:32:18.218704
54970	LILAK	2020-10-06	9.14	9.3	8.705	8.75	1530859	{}	\N	2020-10-12 15:32:18.223622
54971	LILAK	2020-10-07	8.78	9.17	8.78	9.03	1327724	{}	\N	2020-10-12 15:32:18.227836
54972	LILAK	2020-10-08	9.12	9.56	9.04	9.55	1276106	{}	\N	2020-10-12 15:32:18.23282
54973	LILAK	2020-10-09	10.06	10.99	10.05	10.94	5162404	{}	\N	2020-10-12 15:32:18.237587
54974	LILAK	2020-10-12	10.87	10.94	10.57	10.9	465386	{}	\N	2020-10-12 15:32:18.241528
54975	LJPC	2020-09-28	3.93	4.06	3.775	3.82	350418	{}	\N	2020-10-12 15:32:19.231399
54976	LJPC	2020-09-29	3.81	4.12	3.79	3.97	376705	{}	\N	2020-10-12 15:32:19.236965
54977	LJPC	2020-09-30	3.99	4.1	3.96	4.03	201533	{}	\N	2020-10-12 15:32:19.242672
54978	LJPC	2020-10-01	4.05	4.21	4.01	4.21	207494	{}	\N	2020-10-12 15:32:19.248697
54979	LJPC	2020-10-02	4.12	4.24	3.91	4.14	125359	{}	\N	2020-10-12 15:32:19.254473
54980	LJPC	2020-10-05	4.15	4.25	4	4.12	163178	{}	\N	2020-10-12 15:32:19.259621
54981	LJPC	2020-10-06	4.15	4.26	4.04	4.14	182163	{}	\N	2020-10-12 15:32:19.265579
54982	LJPC	2020-10-07	4.19	4.32	4.1301	4.29	172406	{}	\N	2020-10-12 15:32:19.270611
54983	LJPC	2020-10-08	4.32	4.415	4.25	4.28	133637	{}	\N	2020-10-12 15:32:19.275663
54984	LJPC	2020-10-09	4.249	4.28	4.02	4.09	290742	{}	\N	2020-10-12 15:32:19.281995
54985	LJPC	2020-10-12	4.13	4.26	4.0601	4.19	90801	{}	\N	2020-10-12 15:32:19.287827
54986	LMBS	2020-09-28	51.63	51.7	51.63	51.68	761850	{}	\N	2020-10-12 15:32:22.245977
54987	LMBS	2020-09-29	51.67	51.7	51.63	51.65	1036651	{}	\N	2020-10-12 15:32:22.252635
54988	LMBS	2020-09-30	51.67	51.69	51.57	51.68	1435403	{}	\N	2020-10-12 15:32:22.257584
54989	LMBS	2020-10-01	51.63	51.72	51.63	51.68	1213096	{}	\N	2020-10-12 15:32:22.262368
54990	LMBS	2020-10-02	51.69	51.75	51.69	51.73	625004	{}	\N	2020-10-12 15:32:22.268562
54991	LMBS	2020-10-05	51.74	51.75	51.69	51.72	877604	{}	\N	2020-10-12 15:32:22.27356
54992	LMBS	2020-10-06	51.69	51.75	51.68	51.73	583947	{}	\N	2020-10-12 15:32:22.279901
54993	LMBS	2020-10-07	51.71	51.72	51.65	51.695	735457	{}	\N	2020-10-12 15:32:22.286582
54994	LMBS	2020-10-08	51.69	51.71	51.61	51.69	714415	{}	\N	2020-10-12 15:32:22.291601
54995	LMBS	2020-10-09	51.61	51.6856	51.6	51.63	603569	{}	\N	2020-10-12 15:32:22.296913
54996	LMBS	2020-10-12	51.63	51.66	51.6266	51.6518	144045	{}	\N	2020-10-12 15:32:22.303043
54997	LNG	2020-09-28	50	50.15	49.33	49.33	820803	{}	\N	2020-10-12 15:32:23.548333
54998	LNG	2020-09-29	49.34	50.14	48.34	48.54	1103253	{}	\N	2020-10-12 15:32:23.554603
54999	LNG	2020-09-30	48.63	49.14	45.89	46.27	2346764	{}	\N	2020-10-12 15:32:23.560312
55000	LNG	2020-10-01	46	48.3	46	46.34	2176939	{}	\N	2020-10-12 15:32:23.565851
55001	LNG	2020-10-02	44.75	47.36	44.65	47.01	1317228	{}	\N	2020-10-12 15:32:23.572228
55002	LNG	2020-10-05	47.18	48.05	47.06	47.53	683033	{}	\N	2020-10-12 15:32:23.578326
55003	LNG	2020-10-06	47.68	48.5	46.43	46.6	993945	{}	\N	2020-10-12 15:32:23.585153
55004	LNG	2020-10-07	46.56	47.09	46.29	46.36	893281	{}	\N	2020-10-12 15:32:23.590834
55005	LNG	2020-10-08	46.69	48.6677	46.65	48.34	794972	{}	\N	2020-10-12 15:32:23.596254
55006	LNG	2020-10-09	48.51	48.83	47.08	48.29	1132485	{}	\N	2020-10-12 15:32:23.602
55007	LNG	2020-10-12	48.19	48.64	47.8	48.64	224344	{}	\N	2020-10-12 15:32:23.607338
55008	LONE	2020-09-28	0.2094	0.225	0.2056	0.2233	1006621	{}	\N	2020-10-12 15:32:24.830735
55009	LONE	2020-09-29	0.29	0.3579	0.2515	0.27	23836120	{}	\N	2020-10-12 15:32:24.840442
55010	LONE	2020-09-30	0.2305	0.2367	0.21471	0.229	3142935	{}	\N	2020-10-12 15:32:24.845792
55011	LONE	2020-10-01	0.2033	0.26	0.19	0.2445	20835236	{}	\N	2020-10-12 15:32:24.851523
55012	LONE	2020-10-02	0.5401	0.5935	0.31	0.3121	107667519	{}	\N	2020-10-12 15:32:24.857795
55013	LONE	2020-10-05	0.31	0.325	0.264	0.275	8449319	{}	\N	2020-10-12 15:32:24.864975
55014	LONE	2020-10-06	0.2784	0.32	0.2551	0.2811	6544359	{}	\N	2020-10-12 15:32:24.871826
55015	LONE	2020-10-07	0.273	0.28	0.258	0.2661	2720655	{}	\N	2020-10-12 15:32:24.877469
55016	LONE	2020-10-08	0.24	0.28	0.223	0.255	12945647	{}	\N	2020-10-12 15:32:24.88326
55017	LONE	2020-10-09	0.2662	0.275	0.24	0.25	8502372	{}	\N	2020-10-12 15:32:24.88935
55018	LOWC	2020-09-28	100.06	100.37	100	100.28	1310	{}	\N	2020-10-12 15:32:26.307749
55019	LOWC	2020-09-29	100.31	100.31	100.09	100.09	644	{}	\N	2020-10-12 15:32:26.313478
55020	LOWC	2020-09-30	100.56	100.56	100.56	100.56	685	{}	\N	2020-10-12 15:32:26.319074
55021	LOWC	2020-10-01	101.26	101.26	100.85	101.16	2422	{}	\N	2020-10-12 15:32:26.32513
55022	LOWC	2020-10-02	99.56	100.68	99.56	100.6231	1357	{}	\N	2020-10-12 15:32:26.331746
55023	LOWC	2020-10-05	101.4652	101.98	101.4652	101.98	493	{}	\N	2020-10-12 15:32:26.338986
55024	LOWC	2020-10-06	102.128	102.24	100.8163	100.94	1941	{}	\N	2020-10-12 15:32:26.345078
55025	LOWC	2020-10-07	102.23	102.41	101.994	102.3942	1106	{}	\N	2020-10-12 15:32:26.350063
55026	LOWC	2020-10-08	102.745	102.949	102.745	102.949	708	{}	\N	2020-10-12 15:32:26.355411
55027	LOWC	2020-10-09	103.7622	103.7622	103.7622	103.7622	545	{}	\N	2020-10-12 15:32:26.360864
55028	LPX	2020-09-28	30.49	31.47	30.49	31.27	1662482	{}	\N	2020-10-12 15:32:27.595458
55029	LPX	2020-09-29	31.56	31.7899	30.155	30.26	3819507	{}	\N	2020-10-12 15:32:27.60227
55030	LPX	2020-09-30	29.41	30.03	29.17	29.51	6257304	{}	\N	2020-10-12 15:32:27.607711
55031	LPX	2020-10-01	29.69	30.05	29.53	29.83	2268070	{}	\N	2020-10-12 15:32:27.614086
55032	LPX	2020-10-02	29.12	29.86	29.1	29.57	2729450	{}	\N	2020-10-12 15:32:27.621068
55033	LPX	2020-10-05	30.05	30.5	29.8906	30.29	2055198	{}	\N	2020-10-12 15:32:27.628764
55034	LPX	2020-10-06	30.65	30.9	29.63	29.7	2588616	{}	\N	2020-10-12 15:32:27.634591
55035	LPX	2020-10-07	30.2	31.8	30.03	31.64	3547923	{}	\N	2020-10-12 15:32:27.639834
55036	LPX	2020-10-08	32	32.48	31.67	31.9	2143310	{}	\N	2020-10-12 15:32:27.645254
55037	LPX	2020-10-09	32.23	32.47	31.45	31.69	1597694	{}	\N	2020-10-12 15:32:27.650678
55038	LPX	2020-10-12	31.9	32.18	31.74	32.075	415997	{}	\N	2020-10-12 15:32:27.656404
55039	LRN	2020-09-28	27.05	27.89	26.86	27.48	890859	{}	\N	2020-10-12 15:32:28.787001
55040	LRN	2020-09-29	27.33	27.97	26.26	27.08	2477717	{}	\N	2020-10-12 15:32:28.795719
55041	LRN	2020-09-30	26.86	27.09	25.7	26.34	3746713	{}	\N	2020-10-12 15:32:28.802058
55042	LRN	2020-10-01	26.47	27.27	25.56	27.05	3490661	{}	\N	2020-10-12 15:32:28.808637
55043	LRN	2020-10-02	26.34	26.84	25.61	26.74	3565021	{}	\N	2020-10-12 15:32:28.814418
55044	LRN	2020-10-05	27.4	29.07	26.9	28.94	5247000	{}	\N	2020-10-12 15:32:28.8198
55045	LRN	2020-10-06	29.17	29.895	28.22	29.04	3223583	{}	\N	2020-10-12 15:32:28.826378
55046	LRN	2020-10-07	29.28	30.25	29.1501	29.83	1994103	{}	\N	2020-10-12 15:32:28.83166
55047	LRN	2020-10-08	30.52	30.87	29.43	29.57	2117818	{}	\N	2020-10-12 15:32:28.83703
55048	LRN	2020-10-09	29.48	30.74	29.48	30.44	1424975	{}	\N	2020-10-12 15:32:28.843268
55049	LRN	2020-10-12	30.6	31.18	30.35	31.1336	438901	{}	\N	2020-10-12 15:32:28.847825
55050	LDL	2020-09-28	17.35	17.98	17.35	17.62	219492	{}	\N	2020-10-12 15:32:30.305339
55051	LDL	2020-09-29	17.8	17.8	16.6	16.91	160121	{}	\N	2020-10-12 15:32:30.313557
55052	LDL	2020-09-30	16.97	17.42	16.2	16.54	314798	{}	\N	2020-10-12 15:32:30.318575
55053	LDL	2020-10-01	16.58	16.92	16.1	16.88	136396	{}	\N	2020-10-12 15:32:30.32579
55054	LDL	2020-10-02	16.75	17.7535	16.75	17.44	239886	{}	\N	2020-10-12 15:32:30.334556
55055	LDL	2020-10-05	17.67	19.01	17.67	18.96	163157	{}	\N	2020-10-12 15:32:30.343144
55056	LDL	2020-10-06	19.36	19.606	18.02	18.26	227880	{}	\N	2020-10-12 15:32:30.350988
55057	LDL	2020-10-07	18.65	19.28	18.21	18.6	195202	{}	\N	2020-10-12 15:32:30.35839
55058	LDL	2020-10-08	18.89	19.17	18.38	18.96	140621	{}	\N	2020-10-12 15:32:30.365873
55059	LDL	2020-10-09	19.13	19.27	18.8	18.9	121033	{}	\N	2020-10-12 15:32:30.375795
55060	LDL	2020-10-12	18.89	18.89	17.87	17.91	34278	{}	\N	2020-10-12 15:32:30.38349
55061	LEAD	2020-09-28	44.15	44.15	43.7601	43.95	1491	{}	\N	2020-10-12 15:32:31.587624
55062	LEAD	2020-09-29	43.96	43.96	43.7101	43.95	917	{}	\N	2020-10-12 15:32:31.594722
55063	LEAD	2020-09-30	44.42	44.42	43.84	43.84	21800	{}	\N	2020-10-12 15:32:31.600022
55064	LEAD	2020-10-01	44.5632	44.6299	44.13	44.13	4163	{}	\N	2020-10-12 15:32:31.605206
55065	LEAD	2020-10-02	43.7488	44.2029	43.59	43.87	4642	{}	\N	2020-10-12 15:32:31.61232
55066	LEAD	2020-10-05	44.5498	44.7531	44.54	44.7531	2149	{}	\N	2020-10-12 15:32:31.617683
55067	LEAD	2020-10-06	44.7701	44.9001	44.2859	44.2859	2946	{}	\N	2020-10-12 15:32:31.624637
55068	LEAD	2020-10-07	44.98	45.0115	44.88	44.88	535	{}	\N	2020-10-12 15:32:31.632824
55069	LEAD	2020-10-08	45.1	45.1643	45.04	45.04	2596	{}	\N	2020-10-12 15:32:31.638516
55070	LEAD	2020-10-09	45.431	45.5899	45.27	45.38	6611	{}	\N	2020-10-12 15:32:31.646848
55071	LEAD	2020-10-12	45.75	45.889	45.75	45.8773	1886	{}	\N	2020-10-12 15:32:31.653246
55072	LFVN	2020-09-28	12.47	12.65	12.17	12.43	51380	{}	\N	2020-10-12 15:32:33.253532
55073	LFVN	2020-09-29	12.45	12.46	11.95	12.24	50383	{}	\N	2020-10-12 15:32:33.259199
55074	LFVN	2020-09-30	12.25	12.455	12.06	12.06	19918	{}	\N	2020-10-12 15:32:33.267584
55075	LFVN	2020-10-01	12.02	12.14	11.45	11.85	70516	{}	\N	2020-10-12 15:32:33.273732
55076	LFVN	2020-10-02	11.78	11.92	11.65	11.76	39204	{}	\N	2020-10-12 15:32:33.279896
55077	LFVN	2020-10-05	11.83	12.21	11.83	12.17	46611	{}	\N	2020-10-12 15:32:33.285717
55078	LFVN	2020-10-06	12.26	12.4461	12.15	12.16	49232	{}	\N	2020-10-12 15:32:33.291189
55079	LFVN	2020-10-07	12.16	12.55	12.155	12.48	34032	{}	\N	2020-10-12 15:32:33.29944
55080	LFVN	2020-10-08	12.44	12.95	12.3586	12.82	35206	{}	\N	2020-10-12 15:32:33.304581
55081	LFVN	2020-10-09	12.89	13.19	12.89	13.13	33885	{}	\N	2020-10-12 15:32:33.309843
55082	LFVN	2020-10-12	13.13	13.28	13.0813	13.2	10842	{}	\N	2020-10-12 15:32:33.317767
55083	LMT	2020-09-28	390	396.61	389.65	390.51	977287	{}	\N	2020-10-12 15:32:37.386662
55084	LMT	2020-09-29	389.61	393.15	387.51	388.74	2202926	{}	\N	2020-10-12 15:32:37.392295
55085	LMT	2020-09-30	389.47	389.76	380.51	383.28	2780003	{}	\N	2020-10-12 15:32:37.397168
55086	LMT	2020-10-01	384	390	379	380.89	2018416	{}	\N	2020-10-12 15:32:37.402602
55087	LMT	2020-10-02	376	383.4766	374.28	380.58	2230463	{}	\N	2020-10-12 15:32:37.408082
55088	LMT	2020-10-05	383.24	385.8891	378.86	383.96	1640165	{}	\N	2020-10-12 15:32:37.413756
55089	LMT	2020-10-06	386.03	386.11	376.02	376.46	1701682	{}	\N	2020-10-12 15:32:37.419879
55090	LMT	2020-10-07	378.6	383.84	378.56	382.94	1573490	{}	\N	2020-10-12 15:32:37.426335
55091	LMT	2020-10-08	383.9	389.19	382.2784	388.59	1369464	{}	\N	2020-10-12 15:32:37.434454
55092	LMT	2020-10-09	388.41	390.1	385.29	385.93	1281702	{}	\N	2020-10-12 15:32:37.442618
55093	LMT	2020-10-12	385.74	388.99	383.26	387.995	369868	{}	\N	2020-10-12 15:32:37.449612
55094	LNT	2020-09-28	51.66	51.9699	51.24	51.49	1460253	{}	\N	2020-10-12 15:32:40.61261
55095	LNT	2020-09-29	51.66	51.77	50.99	51.08	906782	{}	\N	2020-10-12 15:32:40.618699
55096	LNT	2020-09-30	51.56	51.87	51.24	51.65	1494929	{}	\N	2020-10-12 15:32:40.623974
55097	LNT	2020-10-01	51.66	52.53	51.57	52.33	1525599	{}	\N	2020-10-12 15:32:40.628791
55098	LNT	2020-10-02	52.06	53.47	52.02	53.09	1434480	{}	\N	2020-10-12 15:32:40.635322
55099	LNT	2020-10-05	53.15	53.52	52.53	53.31	1459111	{}	\N	2020-10-12 15:32:40.643625
55100	LNT	2020-10-06	53.32	54.8699	53.15	54.4	1173966	{}	\N	2020-10-12 15:32:40.650074
55101	LNT	2020-10-07	54.73	54.84	53.74	54.08	1044272	{}	\N	2020-10-12 15:32:40.655879
55102	LNT	2020-10-08	54.22	55.2	54.08	55.18	1383774	{}	\N	2020-10-12 15:32:40.660914
55103	LNT	2020-10-09	55.22	55.52	54.48	54.74	859302	{}	\N	2020-10-12 15:32:40.666372
55104	LNT	2020-10-12	54.63	55	54.62	54.91	427141	{}	\N	2020-10-12 15:32:40.672032
55105	LOMA	2020-09-28	4.2	4.3	4.1	4.12	319779	{}	\N	2020-10-12 15:32:42.678142
55106	LOMA	2020-09-29	4.04	4.15	3.96	4.1	592460	{}	\N	2020-10-12 15:32:42.683828
55107	LOMA	2020-09-30	4.18	4.33	4.16	4.31	674485	{}	\N	2020-10-12 15:32:42.689911
55108	LOMA	2020-10-01	4.36	4.42	4.26	4.35	394068	{}	\N	2020-10-12 15:32:42.695029
55109	LOMA	2020-10-02	4.24	4.37	4.17	4.31	330551	{}	\N	2020-10-12 15:32:42.701614
55110	LOMA	2020-10-05	4.38	4.4	4.2	4.34	389066	{}	\N	2020-10-12 15:32:42.708008
55111	LOMA	2020-10-06	4.33	4.45	4.24	4.3	375639	{}	\N	2020-10-12 15:32:42.713739
55112	LOMA	2020-10-07	4.4	4.44	4.295	4.36	581506	{}	\N	2020-10-12 15:32:42.71951
55113	LOMA	2020-10-08	4.41	4.61	4.371	4.58	639452	{}	\N	2020-10-12 15:32:42.726611
55114	LOMA	2020-10-09	4.3	4.405	4.15	4.38	398608	{}	\N	2020-10-12 15:32:42.73163
55115	LOMA	2020-10-12	4.36	4.43	4.26	4.33	118036	{}	\N	2020-10-12 15:32:42.737034
55116	LPCN	2020-09-28	1.39	1.435	1.38	1.4	1108870	{}	\N	2020-10-12 15:32:43.790683
55117	LPCN	2020-09-29	1.38	1.4299	1.36	1.38	1032193	{}	\N	2020-10-12 15:32:43.798084
55118	LPCN	2020-09-30	1.38	1.44	1.38	1.41	935896	{}	\N	2020-10-12 15:32:43.804413
55119	LPCN	2020-10-01	1.39	1.43	1.36	1.36	990838	{}	\N	2020-10-12 15:32:43.81466
55120	LPCN	2020-10-02	1.3	1.4	1.27	1.4	1034017	{}	\N	2020-10-12 15:32:43.823877
55121	LPCN	2020-10-05	1.41	1.42	1.36	1.4	739857	{}	\N	2020-10-12 15:32:43.833199
55122	LPCN	2020-10-06	1.37	1.42	1.36	1.38	930089	{}	\N	2020-10-12 15:32:43.843251
55123	LPCN	2020-10-07	1.38	1.41	1.32	1.38	964176	{}	\N	2020-10-12 15:32:43.852319
55124	LPCN	2020-10-08	1.4	1.57	1.39	1.56	2819593	{}	\N	2020-10-12 15:32:43.861953
55125	LPCN	2020-10-09	1.5	1.55	1.43	1.48	1372311	{}	\N	2020-10-12 15:32:43.867286
55126	LPCN	2020-10-12	1.48	1.4852	1.41	1.44	314655	{}	\N	2020-10-12 15:32:43.872513
55127	LPTH	2020-09-28	2.27	2.42	2.23	2.42	393917	{}	\N	2020-10-12 15:32:45.253288
55128	LPTH	2020-09-29	2.47	2.47	2.365	2.42	274677	{}	\N	2020-10-12 15:32:45.263608
55129	LPTH	2020-09-30	2.41	2.4533	2.38	2.41	159250	{}	\N	2020-10-12 15:32:45.270457
55130	LPTH	2020-10-01	2.42	2.43	2.34	2.36	122784	{}	\N	2020-10-12 15:32:45.279846
55131	LPTH	2020-10-02	2.26	2.39	2.2575	2.32	124550	{}	\N	2020-10-12 15:32:45.286842
55132	LPTH	2020-10-05	2.33	2.42	2.32	2.35	156270	{}	\N	2020-10-12 15:32:45.294149
55133	LPTH	2020-10-06	2.33	2.42	2.32	2.35	175272	{}	\N	2020-10-12 15:32:45.300064
55134	LPTH	2020-10-07	2.38	2.42	2.27	2.3	234383	{}	\N	2020-10-12 15:32:45.305605
55135	LPTH	2020-10-08	2.35	2.4099	2.28	2.38	155887	{}	\N	2020-10-12 15:32:45.313963
55136	LPTH	2020-10-09	2.45	2.555	2.42	2.5	426916	{}	\N	2020-10-12 15:32:45.321404
55137	LPTH	2020-10-12	2.58	2.59	2.41	2.51	94798	{}	\N	2020-10-12 15:32:45.328711
55138	LEO	2020-09-28	8.27	8.29	8.25	8.26	86291	{}	\N	2020-10-12 15:32:46.358697
55139	LEO	2020-09-29	8.26	8.27	8.23	8.24	196436	{}	\N	2020-10-12 15:32:46.368297
55140	LEO	2020-09-30	8.27	8.3	8.2524	8.28	263368	{}	\N	2020-10-12 15:32:46.37979
55141	LEO	2020-10-01	8.29	8.33	8.27	8.31	142408	{}	\N	2020-10-12 15:32:46.389428
55142	LEO	2020-10-02	8.28	8.35	8.28	8.32	155158	{}	\N	2020-10-12 15:32:46.400289
55143	LEO	2020-10-05	8.33	8.35	8.26	8.3	177068	{}	\N	2020-10-12 15:32:46.413577
55144	LEO	2020-10-06	8.3	8.33	8.28	8.3	118478	{}	\N	2020-10-12 15:32:46.425794
55145	LEO	2020-10-07	8.33	8.35	8.26	8.33	185488	{}	\N	2020-10-12 15:32:46.439345
55146	LEO	2020-10-08	8.34	8.35	8.25	8.29	167916	{}	\N	2020-10-12 15:32:46.452324
55147	LEO	2020-10-09	8.3	8.3	8.24	8.26	157260	{}	\N	2020-10-12 15:32:46.461836
55148	LEO	2020-10-12	8.28	8.2979	8.2751	8.28	23034	{}	\N	2020-10-12 15:32:46.470975
55149	LGC	2020-09-28	10.63	10.63	10.3818	10.4171	3040	{}	\N	2020-10-12 15:32:47.823104
55150	LGC	2020-09-29	10.45	10.45	10.36	10.38	78056	{}	\N	2020-10-12 15:32:47.82925
55151	LGC	2020-09-30	10.36	10.39	10.35	10.37	145568	{}	\N	2020-10-12 15:32:47.834821
55152	LGC	2020-10-01	10.37	10.4	10.36	10.3601	72712	{}	\N	2020-10-12 15:32:47.839846
55153	LGC	2020-10-02	10.6	10.6	10.38	10.4398	16778	{}	\N	2020-10-12 15:32:47.845646
55154	LGC	2020-10-05	10.57	10.57	10.41	10.44	28028	{}	\N	2020-10-12 15:32:47.852404
55155	LGC	2020-10-06	10.44	10.46	10.44	10.45	20683	{}	\N	2020-10-12 15:32:47.859548
55156	LGC	2020-10-07	10.27	10.45	10.27	10.41	51526	{}	\N	2020-10-12 15:32:47.866073
55157	LGC	2020-10-08	10.41	10.43	10.4	10.42	31480	{}	\N	2020-10-12 15:32:47.871741
55158	LGC	2020-10-09	10.4	10.48	10.4	10.44	13096	{}	\N	2020-10-12 15:32:47.8773
55159	LGC	2020-10-12	10.41	10.455	10.41	10.4115	4280	{}	\N	2020-10-12 15:32:47.883094
55160	LHCG	2020-09-28	207.005	207.74	204.49	207.45	133825	{}	\N	2020-10-12 15:32:49.05833
55161	LHCG	2020-09-29	207.44	211.82	206.62	207.7	141512	{}	\N	2020-10-12 15:32:49.064895
55162	LHCG	2020-09-30	208.64	217.02	208.64	212.56	347355	{}	\N	2020-10-12 15:32:49.069007
55163	LHCG	2020-10-01	212.17	214.56	208	214.02	150719	{}	\N	2020-10-12 15:32:49.073217
55164	LHCG	2020-10-02	210.79	216.39	210	214.62	126786	{}	\N	2020-10-12 15:32:49.077911
55165	LHCG	2020-10-05	215.53	218.39	214.72	218.02	148036	{}	\N	2020-10-12 15:32:49.082155
55166	LHCG	2020-10-06	218.53	221.29	214.88	217.62	150632	{}	\N	2020-10-12 15:32:49.086103
55167	LHCG	2020-10-07	219.88	221.72	219.05	220.79	116025	{}	\N	2020-10-12 15:32:49.090517
55168	LHCG	2020-10-08	222.66	222.94	218.74	221.06	103573	{}	\N	2020-10-12 15:32:49.094851
55169	LHCG	2020-10-09	222.88	226.54	221.14	225.66	103383	{}	\N	2020-10-12 15:32:49.10076
55170	LHCG	2020-10-12	227.01	228.13	225.2	228.08	10254	{}	\N	2020-10-12 15:32:49.107165
55171	LIQT	2020-09-28	7.7812	7.85	7.71	7.83	12045	{}	\N	2020-10-12 15:32:49.97697
55172	LIQT	2020-09-29	7.75	8.15	7.7043	8.08	75985	{}	\N	2020-10-12 15:32:49.984148
55173	LIQT	2020-09-30	7.98	8.69	7.98	8.45	106010	{}	\N	2020-10-12 15:32:49.990079
55174	LIQT	2020-10-01	8.41	8.65	8.41	8.59	28172	{}	\N	2020-10-12 15:32:49.995714
55175	LIQT	2020-10-02	8.62	8.87	8.5	8.8	49034	{}	\N	2020-10-12 15:32:50.002668
55176	LIQT	2020-10-05	8.8	9.1	8.8	8.95	46730	{}	\N	2020-10-12 15:32:50.007838
55177	LIQT	2020-10-06	9	9	8.45	8.54	46005	{}	\N	2020-10-12 15:32:50.013091
55178	LIQT	2020-10-07	8.5827	8.77	8.29	8.4	56328	{}	\N	2020-10-12 15:32:50.020638
55179	LIQT	2020-10-08	8.545	8.68	8.305	8.47	39351	{}	\N	2020-10-12 15:32:50.026861
55180	LIQT	2020-10-09	8.43	8.44	7.85	7.93	97417	{}	\N	2020-10-12 15:32:50.034898
55181	LIQT	2020-10-12	7.97	8.22	7.97	8.15	4589	{}	\N	2020-10-12 15:32:50.039864
55182	LKQ	2020-09-28	27.74	28.49	27.6	28.26	2035089	{}	\N	2020-10-12 15:32:51.668105
55183	LKQ	2020-09-29	28.071	28.26	27.62	27.65	1659388	{}	\N	2020-10-12 15:32:51.673785
55184	LKQ	2020-09-30	27.8	28.07	27.46	27.73	3306372	{}	\N	2020-10-12 15:32:51.681703
55185	LKQ	2020-10-01	28.085	28.34	27.58	28.01	2064373	{}	\N	2020-10-12 15:32:51.687558
55186	LKQ	2020-10-02	27.77	28.68	27.28	28.51	1545496	{}	\N	2020-10-12 15:32:51.693612
55187	LKQ	2020-10-05	28.79	29.63	28.59	29.53	1866583	{}	\N	2020-10-12 15:32:51.701388
55188	LKQ	2020-10-06	29.61	30.475	29.395	29.46	2254671	{}	\N	2020-10-12 15:32:51.707839
55189	LKQ	2020-10-07	29.82	30.65	29.82	30.32	1956196	{}	\N	2020-10-12 15:32:51.713095
55190	LKQ	2020-10-08	30.61	30.78	30.12	30.72	1511288	{}	\N	2020-10-12 15:32:51.719025
55191	LKQ	2020-10-09	31.22	31.25	30.5	30.52	1482873	{}	\N	2020-10-12 15:32:51.724309
55192	LKQ	2020-10-12	30.7	30.78	30.36	30.72	300712	{}	\N	2020-10-12 15:32:51.730964
55193	LMAT	2020-09-28	34.13	34.16	32.28	32.35	112343	{}	\N	2020-10-12 15:32:52.65145
55194	LMAT	2020-09-29	32.31	33.99	31.98	32.36	84780	{}	\N	2020-10-12 15:32:52.657557
55195	LMAT	2020-09-30	32.28	32.91	32.23	32.52	46504	{}	\N	2020-10-12 15:32:52.662363
55196	LMAT	2020-10-01	32.69	33.34	31.56	32.14	144265	{}	\N	2020-10-12 15:32:52.668701
55197	LMAT	2020-10-02	31.66	32.39	31.59	31.85	45107	{}	\N	2020-10-12 15:32:52.673837
55198	LMAT	2020-10-05	31.99	32.78	31.795	31.97	66031	{}	\N	2020-10-12 15:32:52.679956
55199	LMAT	2020-10-06	32.3	33.33	31.6	31.77	81293	{}	\N	2020-10-12 15:32:52.685764
55200	LMAT	2020-10-07	31.91	32.2	31.33	32.04	116145	{}	\N	2020-10-12 15:32:52.691653
55201	LMAT	2020-10-08	32.33	32.8	31.66	32.37	104567	{}	\N	2020-10-12 15:32:52.696865
55202	LMAT	2020-10-09	32.5	33.48	32.5	33.44	98967	{}	\N	2020-10-12 15:32:52.702652
55203	LMAT	2020-10-12	33.51	34.15	33.1437	33.99	8883	{}	\N	2020-10-12 15:32:52.707773
55204	LMRK	2020-09-28	9.19	9.6	9.19	9.37	63132	{}	\N	2020-10-12 15:32:54.291033
55205	LMRK	2020-09-29	9.35	9.35	9.05	9.26	41889	{}	\N	2020-10-12 15:32:54.297205
55206	LMRK	2020-09-30	9.33	9.41	9.0192	9.05	43783	{}	\N	2020-10-12 15:32:54.302107
55207	LMRK	2020-10-01	9.1	9.21	8.95	9.12	76825	{}	\N	2020-10-12 15:32:54.307105
55208	LMRK	2020-10-02	8.91	9.3021	8.91	9.19	45820	{}	\N	2020-10-12 15:32:54.31381
55209	LMRK	2020-10-05	9.5	9.6799	9.28	9.38	72375	{}	\N	2020-10-12 15:32:54.319944
55210	LMRK	2020-10-06	9.5	9.7839	9.38	9.73	57073	{}	\N	2020-10-12 15:32:54.324979
55211	LMRK	2020-10-07	9.75	9.79	9.42	9.43	57511	{}	\N	2020-10-12 15:32:54.329366
55212	LMRK	2020-10-08	9.52	10.19	9.52	10.15	107279	{}	\N	2020-10-12 15:32:54.333735
55213	LMRK	2020-10-09	10.2	10.274	10.11	10.2	56946	{}	\N	2020-10-12 15:32:54.338663
55214	LMRK	2020-10-12	10.28	10.2937	10.2	10.28	10030	{}	\N	2020-10-12 15:32:54.34314
55215	LNN	2020-09-28	96.68	98.01	96	96.71	64502	{}	\N	2020-10-12 15:32:55.212864
55216	LNN	2020-09-29	96.62	97.97	95.79	96.43	103586	{}	\N	2020-10-12 15:32:55.220092
55217	LNN	2020-09-30	96.5	98.18	95.9	96.68	137535	{}	\N	2020-10-12 15:32:55.226822
55218	LNN	2020-10-01	97.4	97.98	95.71	96.32	101489	{}	\N	2020-10-12 15:32:55.235593
55219	LNN	2020-10-02	94.96	98.5159	94.51	97.44	136726	{}	\N	2020-10-12 15:32:55.245498
55220	LNN	2020-10-05	98.13	99.98	97.28	98.64	59873	{}	\N	2020-10-12 15:32:55.254073
55221	LNN	2020-10-06	99.83	102.25	98.56	99.38	158567	{}	\N	2020-10-12 15:32:55.262256
55222	LNN	2020-10-07	100.26	103.38	99.76	102.75	172274	{}	\N	2020-10-12 15:32:55.268216
55223	LNN	2020-10-08	103.91	105.82	102.18	105.59	76700	{}	\N	2020-10-12 15:32:55.274408
55224	LNN	2020-10-09	106.56	107.12	105.84	106.45	82993	{}	\N	2020-10-12 15:32:55.283308
55225	LNN	2020-10-12	107.22	108.73	106.99	108.73	14540	{}	\N	2020-10-12 15:32:55.292861
55226	LSXMA	2020-09-28	33.43	34	33.2	33.59	553191	{}	\N	2020-10-12 15:32:57.664033
55227	LSXMA	2020-09-29	33.45	33.61	32.8	33.11	341423	{}	\N	2020-10-12 15:32:57.669949
55228	LSXMA	2020-09-30	33.17	33.74	32.97	33.17	547144	{}	\N	2020-10-12 15:32:57.675883
55229	LSXMA	2020-10-01	33.3	34.69	33.3	34.68	424978	{}	\N	2020-10-12 15:32:57.681712
55230	LSXMA	2020-10-02	34.03	34.74	33.89	34.035	581460	{}	\N	2020-10-12 15:32:57.688121
55231	LSXMA	2020-10-05	34.43	34.715	33.75	34.35	745134	{}	\N	2020-10-12 15:32:57.697811
55232	LSXMA	2020-10-06	34.345	34.83	33.87	34.58	879195	{}	\N	2020-10-12 15:32:57.704608
55233	LSXMA	2020-10-07	35.82	36.31	35.3	35.61	385252	{}	\N	2020-10-12 15:32:57.711239
55234	LSXMA	2020-10-08	35.95	36.23	35.4469	36.18	342051	{}	\N	2020-10-12 15:32:57.716488
55235	LSXMA	2020-10-09	36.59	36.6	35.23	35.31	189116	{}	\N	2020-10-12 15:32:57.721384
55236	LSXMA	2020-10-12	35.46	35.68	35.27	35.48	35566	{}	\N	2020-10-12 15:32:57.728735
55237	LTRX	2020-09-28	4.8	4.8408	4.75	4.82	116941	{}	\N	2020-10-12 15:32:59.803965
55238	LTRX	2020-09-29	4.83	5.03	4.8	5.01	165258	{}	\N	2020-10-12 15:32:59.811461
55239	LTRX	2020-09-30	5.04	5.05	4.76	4.81	95192	{}	\N	2020-10-12 15:32:59.818432
55240	LTRX	2020-10-01	4.86	4.99	4.815	4.93	65374	{}	\N	2020-10-12 15:32:59.824547
55241	LTRX	2020-10-02	4.9	5.15	4.84	5.02	132119	{}	\N	2020-10-12 15:32:59.831638
55242	LTRX	2020-10-05	5.09	5.0992	4.95	4.98	65220	{}	\N	2020-10-12 15:32:59.83776
55243	LTRX	2020-10-06	5.03	5.11	4.95	4.95	67518	{}	\N	2020-10-12 15:32:59.843785
55244	LTRX	2020-10-07	4.99	5.07	4.93	4.99	73820	{}	\N	2020-10-12 15:32:59.848955
55245	LTRX	2020-10-08	5.04	5.14	4.92	5	142425	{}	\N	2020-10-12 15:32:59.854175
55246	LTRX	2020-10-09	5	5.21	4.999	5.15	190882	{}	\N	2020-10-12 15:32:59.861666
55247	LTRX	2020-10-12	5.15	5.15	5.0203	5.03	45318	{}	\N	2020-10-12 15:32:59.868919
55248	MSI	2020-09-28	155.86	158.39	155.245	156.63	1162318	{}	\N	2020-10-12 15:33:02.235981
55249	MSI	2020-09-29	157.28	158.96	157.265	157.65	1432796	{}	\N	2020-10-12 15:33:02.243859
55250	MSI	2020-09-30	157.71	158.04	156.075	156.81	1595297	{}	\N	2020-10-12 15:33:02.250314
55251	MSI	2020-10-01	158	158.85	155.34	156.79	1377774	{}	\N	2020-10-12 15:33:02.255312
55252	MSI	2020-10-02	153.7	157.58	153.7	156.91	1156961	{}	\N	2020-10-12 15:33:02.260608
55253	MSI	2020-10-05	158	158.65	156.59	157.51	1379302	{}	\N	2020-10-12 15:33:02.26611
55254	MSI	2020-10-06	158.27	159.24	156.61	156.96	1496413	{}	\N	2020-10-12 15:33:02.271352
55255	MSI	2020-10-07	158.46	160.505	157.54	160.31	1656442	{}	\N	2020-10-12 15:33:02.276668
55256	MSI	2020-10-08	161.68	163.71	161.68	162.77	1368502	{}	\N	2020-10-12 15:33:02.284202
55257	MSI	2020-10-09	164.39	167.17	164	166.78	2459676	{}	\N	2020-10-12 15:33:02.290308
55258	MSI	2020-10-12	167.53	167.7	166.01	167.44	462688	{}	\N	2020-10-12 15:33:02.296877
55259	LFC	2020-09-28	11.25	11.26	11.2	11.25	287853	{}	\N	2020-10-12 15:33:03.913616
55260	LFC	2020-09-29	11.27	11.31	11.235	11.27	423624	{}	\N	2020-10-12 15:33:03.920772
55261	LFC	2020-09-30	11.3	11.375	11.27	11.32	599260	{}	\N	2020-10-12 15:33:03.92804
55262	LFC	2020-10-01	11.49	11.49	11.37	11.43	843363	{}	\N	2020-10-12 15:33:03.943041
55263	LFC	2020-10-02	11.31	11.48	11.31	11.38	588116	{}	\N	2020-10-12 15:33:03.953098
55264	LFC	2020-10-05	11.72	11.84	11.7	11.84	1422096	{}	\N	2020-10-12 15:33:03.960807
55265	LFC	2020-10-06	11.68	11.75	11.62	11.64	649697	{}	\N	2020-10-12 15:33:03.967102
55266	LFC	2020-10-07	11.64	11.65	11.54	11.6	438823	{}	\N	2020-10-12 15:33:03.97214
55267	LFC	2020-10-08	11.67	11.78	11.66	11.77	608994	{}	\N	2020-10-12 15:33:03.977272
55268	LFC	2020-10-09	11.69	11.79	11.66	11.74	1207100	{}	\N	2020-10-12 15:33:03.983
55269	LFC	2020-10-12	11.84	11.8899	11.8	11.8501	214818	{}	\N	2020-10-12 15:33:03.98899
55270	LGND	2020-09-28	98.43	99.13	95.1507	95.75	187911	{}	\N	2020-10-12 15:33:04.944814
55271	LGND	2020-09-29	97.55	98.45	94.815	95.8	247934	{}	\N	2020-10-12 15:33:04.952161
55272	LGND	2020-09-30	97.17	98.42	95.18	95.32	136470	{}	\N	2020-10-12 15:33:04.959465
55273	LGND	2020-10-01	96	98.4	93.84	98.2	176969	{}	\N	2020-10-12 15:33:04.970188
55274	LGND	2020-10-02	97.23	100.05	95.9	97.15	192319	{}	\N	2020-10-12 15:33:04.97774
55275	LGND	2020-10-05	97.83	101.31	97.07	100.26	287883	{}	\N	2020-10-12 15:33:04.984172
55276	LGND	2020-10-06	105.0955	106.875	100.3484	101.22	350120	{}	\N	2020-10-12 15:33:04.989155
55277	LGND	2020-10-07	101.39	104.5	101.39	104.11	190121	{}	\N	2020-10-12 15:33:04.994977
55278	LGND	2020-10-08	105.33	105.5	102.2558	104.04	190288	{}	\N	2020-10-12 15:33:05.000772
55279	LGND	2020-10-09	105	105.6	101.27	101.61	221253	{}	\N	2020-10-12 15:33:05.005941
55280	LGND	2020-10-12	101.9	103.41	100	102.41	57925	{}	\N	2020-10-12 15:33:05.011683
55281	LIVE	2020-09-28	8.93	9.06	8.93	9.0401	1567	{}	\N	2020-10-12 15:33:06.305058
55282	LIVE	2020-09-29	8.88	8.88	8.53	8.8303	3228	{}	\N	2020-10-12 15:33:06.311008
55283	LIVE	2020-09-30	8.86	8.99	8.82	8.94	2365	{}	\N	2020-10-12 15:33:06.316245
55284	LIVE	2020-10-01	8.6902	8.7	8.69	8.69	1551	{}	\N	2020-10-12 15:33:06.322737
55285	LIVE	2020-10-02	7.9287	9.8799	7.9287	9.6657	18900	{}	\N	2020-10-12 15:33:06.328865
55286	LIVE	2020-10-05	9.65	9.709	9.455	9.455	3869	{}	\N	2020-10-12 15:33:06.335421
55287	LIVE	2020-10-06	9.475	9.72	9.475	9.5	2001	{}	\N	2020-10-12 15:33:06.341185
55288	LIVE	2020-10-07	9.56	10.4699	9.5	9.5088	21370	{}	\N	2020-10-12 15:33:06.347312
55289	LIVE	2020-10-08	10.0532	10.195	10.0532	10.1	2149	{}	\N	2020-10-12 15:33:06.354673
55290	LIVE	2020-10-09	10.0999	10.0999	9.81	9.935	1182	{}	\N	2020-10-12 15:33:06.361139
55291	LIVE	2020-10-12	9.74	11.1	9.74	11.1	17783	{}	\N	2020-10-12 15:33:06.369373
55292	LLNW	2020-09-28	5.92	5.99	5.7	5.79	1664850	{}	\N	2020-10-12 15:33:09.402386
55293	LLNW	2020-09-29	5.71	6.08	5.7	5.92	2110389	{}	\N	2020-10-12 15:33:09.409258
55294	LLNW	2020-09-30	5.91	5.966	5.695	5.76	1990781	{}	\N	2020-10-12 15:33:09.414386
55295	LLNW	2020-10-01	5.8	5.84	5.69	5.8	1393472	{}	\N	2020-10-12 15:33:09.421175
55296	LLNW	2020-10-02	5.64	5.86	5.56	5.6	2061651	{}	\N	2020-10-12 15:33:09.429379
55297	LLNW	2020-10-05	5.66	5.7265	5.45	5.5	2851272	{}	\N	2020-10-12 15:33:09.436366
55298	LLNW	2020-10-06	5.64	5.81	5.6	5.655	2494235	{}	\N	2020-10-12 15:33:09.442274
55299	LLNW	2020-10-07	5.7	5.8899	5.7	5.86	1995487	{}	\N	2020-10-12 15:33:09.447967
55300	LLNW	2020-10-08	5.97	6.12	5.845	5.92	3298985	{}	\N	2020-10-12 15:33:09.454108
55301	LLNW	2020-10-09	6.04	6.36	5.96	6.32	3800038	{}	\N	2020-10-12 15:33:09.459591
55302	LLNW	2020-10-12	6.5	6.6	6.23	6.3161	2371601	{}	\N	2020-10-12 15:33:09.465751
55303	LMNR	2020-09-28	14.505	14.84	14.43	14.47	58012	{}	\N	2020-10-12 15:33:10.37135
55304	LMNR	2020-09-29	14.45	14.52	14.17	14.23	17479	{}	\N	2020-10-12 15:33:10.379133
55305	LMNR	2020-09-30	14.38	14.54	14.18	14.3	24128	{}	\N	2020-10-12 15:33:10.384137
55306	LMNR	2020-10-01	14.31	14.31	13.85	14.19	28840	{}	\N	2020-10-12 15:33:10.389756
55307	LMNR	2020-10-02	14.15	14.555	13.98	14.54	55815	{}	\N	2020-10-12 15:33:10.394975
55308	LMNR	2020-10-05	14.71	14.9	14.61	14.79	31965	{}	\N	2020-10-12 15:33:10.400102
55309	LMNR	2020-10-06	15.02	15.25	14.65	15.06	69307	{}	\N	2020-10-12 15:33:10.405385
55310	LMNR	2020-10-07	15.16	15.81	15.16	15.68	69157	{}	\N	2020-10-12 15:33:10.410258
55311	LMNR	2020-10-08	15.81	16.015	15.39	15.44	59287	{}	\N	2020-10-12 15:33:10.41528
55312	LMNR	2020-10-09	15.45	15.45	14.85	14.98	29145	{}	\N	2020-10-12 15:33:10.423679
55313	LMNR	2020-10-12	14.89	15.07	14.89	14.95	5477	{}	\N	2020-10-12 15:33:10.429899
55314	LNDC	2020-09-28	9.42	9.75	9.38	9.64	116895	{}	\N	2020-10-12 15:33:11.537886
55315	LNDC	2020-09-29	9.58	9.735	9.34	9.68	74147	{}	\N	2020-10-12 15:33:11.5449
55316	LNDC	2020-09-30	9.67	9.91	9.61	9.72	60049	{}	\N	2020-10-12 15:33:11.550065
55317	LNDC	2020-10-01	9.81	9.81	9.41	9.62	60670	{}	\N	2020-10-12 15:33:11.555343
55318	LNDC	2020-10-02	9.55	9.6	9.33	9.43	129192	{}	\N	2020-10-12 15:33:11.562101
55319	LNDC	2020-10-05	9.5	10.03	9.5	9.98	160220	{}	\N	2020-10-12 15:33:11.567838
55320	LNDC	2020-10-06	10.14	10.6891	10	10.52	291603	{}	\N	2020-10-12 15:33:11.573264
55321	LNDC	2020-10-07	9.3966	10.435	9.37	10.1	294166	{}	\N	2020-10-12 15:33:11.57884
55322	LNDC	2020-10-08	10.1	10.15	9.7	9.88	491625	{}	\N	2020-10-12 15:33:11.583693
55323	LNDC	2020-10-09	9.97	10.13	9.95	10.06	59802	{}	\N	2020-10-12 15:33:11.591697
55324	LNDC	2020-10-12	10.12	10.34	10.03	10.17	16584	{}	\N	2020-10-12 15:33:11.597379
55325	LOCO	2020-09-28	15.63	16.01	15.57	15.88	231011	{}	\N	2020-10-12 15:33:13.182895
55326	LOCO	2020-09-29	15.79	16.25	15.62	16.17	277881	{}	\N	2020-10-12 15:33:13.18843
55327	LOCO	2020-09-30	16.27	16.66	16.02	16.2	282391	{}	\N	2020-10-12 15:33:13.19478
55328	LOCO	2020-10-01	16.22	16.45	16.06	16.3	199797	{}	\N	2020-10-12 15:33:13.199113
55329	LOCO	2020-10-02	15.98	16.45	15.89	16.36	146715	{}	\N	2020-10-12 15:33:13.203282
55330	LOCO	2020-10-05	16.8925	16.8925	16.19	16.34	171558	{}	\N	2020-10-12 15:33:13.208654
55331	LOCO	2020-10-06	16.33	16.74	16.33	16.38	246236	{}	\N	2020-10-12 15:33:13.21318
55332	LOCO	2020-10-07	16.46	16.91	16.29	16.88	213228	{}	\N	2020-10-12 15:33:13.21825
55333	LOCO	2020-10-08	16.95	17.48	16.67	17.34	215651	{}	\N	2020-10-12 15:33:13.222439
55334	LOCO	2020-10-09	17.52	17.95	17.5	17.87	266705	{}	\N	2020-10-12 15:33:13.227212
55335	LOCO	2020-10-12	17.75	18.09	16.98	17.89	87244	{}	\N	2020-10-12 15:33:13.231918
55336	LOV	2020-09-28	4.83	4.83	4.47	4.6	39049	{}	\N	2020-10-12 15:33:14.469312
55337	LOV	2020-09-29	4.6	4.8463	4.6	4.84	59984	{}	\N	2020-10-12 15:33:14.476834
55338	LOV	2020-09-30	4.99	5.17	4.75	4.89	95463	{}	\N	2020-10-12 15:33:14.482222
55339	LOV	2020-10-01	4.87	4.99	4.3	4.58	28920	{}	\N	2020-10-12 15:33:14.488477
55340	LOV	2020-10-02	4.36	4.58	4.36	4.5	14546	{}	\N	2020-10-12 15:33:14.495608
55341	LOV	2020-10-05	4.68	4.79	4.6	4.62	44232	{}	\N	2020-10-12 15:33:14.501437
55342	LOV	2020-10-06	4.65	4.8	4.62	4.71	20801	{}	\N	2020-10-12 15:33:14.50795
55343	LOV	2020-10-07	4.75	4.7697	4.25	4.46	255319	{}	\N	2020-10-12 15:33:14.514573
55344	LOV	2020-10-08	4.54	4.69	4.5101	4.69	22793	{}	\N	2020-10-12 15:33:14.520719
55345	LOV	2020-10-09	4.8	4.8	4.45	4.59	19840	{}	\N	2020-10-12 15:33:14.52749
55346	LOV	2020-10-12	4.51	4.62	4.45	4.53	9349	{}	\N	2020-10-12 15:33:14.532956
55347	LPI	2020-09-28	9.62	10.36	9.35	10	518306	{}	\N	2020-10-12 15:33:15.403602
55348	LPI	2020-09-29	9.98	10.1	9.44	9.94	740479	{}	\N	2020-10-12 15:33:15.4113
55349	LPI	2020-09-30	9.94	10.31	9.63	9.8	954390	{}	\N	2020-10-12 15:33:15.416981
55350	LPI	2020-10-01	9.6	9.81	9.0601	9.16	1062793	{}	\N	2020-10-12 15:33:15.424058
55351	LPI	2020-10-02	8.63	9.58	8.59	9.24	952798	{}	\N	2020-10-12 15:33:15.432968
55352	LPI	2020-10-05	9.5	9.59	9.11	9.29	809543	{}	\N	2020-10-12 15:33:15.44132
55353	LPI	2020-10-06	9.52	9.7597	8.82	8.85	877129	{}	\N	2020-10-12 15:33:15.448227
55354	LPI	2020-10-07	9.02	9.2	8.7701	8.99	862859	{}	\N	2020-10-12 15:33:15.453487
55355	LPI	2020-10-08	9.22	9.98	9	9.8	1579694	{}	\N	2020-10-12 15:33:15.45896
55356	LPI	2020-10-09	9.88	10.2	9.48	9.49	953206	{}	\N	2020-10-12 15:33:15.465459
55357	LPI	2020-10-12	9.42	9.58	9.17	9.58	277192	{}	\N	2020-10-12 15:33:15.471246
55358	LQD	2020-09-28	134.52	134.92	134.39	134.9	8714457	{}	\N	2020-10-12 15:33:16.676245
55359	LQD	2020-09-29	135.06	135.18	134.71	134.73	8390202	{}	\N	2020-10-12 15:33:16.683894
55360	LQD	2020-09-30	134.65	134.785	134.4219	134.71	11496980	{}	\N	2020-10-12 15:33:16.689012
55361	LQD	2020-10-01	134.26	134.93	134.21	134.79	12785940	{}	\N	2020-10-12 15:33:16.695422
55362	LQD	2020-10-02	134.66	134.89	134.4	134.81	10743022	{}	\N	2020-10-12 15:33:16.701453
55363	LQD	2020-10-05	134.49	134.76	134.32	134.36	10089274	{}	\N	2020-10-12 15:33:16.707151
55364	LQD	2020-10-06	134.45	134.95	133.95	134.25	22179072	{}	\N	2020-10-12 15:33:16.713073
55365	LQD	2020-10-07	134.26	134.74	134.11	134.29	14724371	{}	\N	2020-10-12 15:33:16.720132
55366	LQD	2020-10-08	134.66	134.85	134.51	134.62	12118078	{}	\N	2020-10-12 15:33:16.726285
55367	LQD	2020-10-09	134.52	134.98	134.445	134.93	9994265	{}	\N	2020-10-12 15:33:16.732592
55368	LQD	2020-10-12	135.11	135.495	135.1	135.495	1831032	{}	\N	2020-10-12 15:33:16.738407
55369	LSCC	2020-09-28	28.46	28.895	28.42	28.75	667700	{}	\N	2020-10-12 15:33:17.796989
55370	LSCC	2020-09-29	28.75	29.2	28.6	28.64	599005	{}	\N	2020-10-12 15:33:17.804554
55371	LSCC	2020-09-30	28.83	29.27	28.535	28.96	1296436	{}	\N	2020-10-12 15:33:17.810823
55372	LSCC	2020-10-01	29.05	29.89	28.652	29.75	709301	{}	\N	2020-10-12 15:33:17.818348
55373	LSCC	2020-10-02	28.92	29.27	28.06	28.215	850307	{}	\N	2020-10-12 15:33:17.824814
55374	LSCC	2020-10-05	28.51	29.695	28.51	29.66	619930	{}	\N	2020-10-12 15:33:17.830945
55375	LSCC	2020-10-06	29.59	30.33	29.38	29.83	877126	{}	\N	2020-10-12 15:33:17.836931
55376	LSCC	2020-10-07	30.39	31.14	30.15	31.02	779652	{}	\N	2020-10-12 15:33:17.843382
55377	LSCC	2020-10-08	31.4	31.7219	31.04	31.44	1134146	{}	\N	2020-10-12 15:33:17.85015
55378	LSCC	2020-10-09	32.5	35.16	32.16	32.93	1861706	{}	\N	2020-10-12 15:33:17.855672
55379	LSCC	2020-10-12	33.43	33.77	33.1	33.62	503223	{}	\N	2020-10-12 15:33:17.861147
55380	LUB	2020-09-28	2.57	2.85	2.57	2.78	658608	{}	\N	2020-10-12 15:33:19.418817
55381	LUB	2020-09-29	2.74	2.94	2.73	2.91	638668	{}	\N	2020-10-12 15:33:19.425902
55382	LUB	2020-09-30	2.85	2.93	2.81	2.86	641411	{}	\N	2020-10-12 15:33:19.434696
55383	LUB	2020-10-01	2.78	2.92	2.76	2.88	952825	{}	\N	2020-10-12 15:33:19.442335
55384	LUB	2020-10-02	2.77	3.3	2.7001	3.22	1445981	{}	\N	2020-10-12 15:33:19.448776
55385	LUB	2020-10-05	3.15	3.18	2.82	2.86	1042408	{}	\N	2020-10-12 15:33:19.455301
55386	LUB	2020-10-06	2.89	2.93	2.82	2.83	511348	{}	\N	2020-10-12 15:33:19.461237
55387	LUB	2020-10-07	2.84	2.91	2.77	2.81	732918	{}	\N	2020-10-12 15:33:19.468063
55388	LUB	2020-10-08	2.76	2.85	2.71	2.75	197956	{}	\N	2020-10-12 15:33:19.475024
55389	LUB	2020-10-09	2.77	2.8	2.71	2.72	156380	{}	\N	2020-10-12 15:33:19.480588
55390	LUB	2020-10-12	2.7	2.8	2.69	2.7	234308	{}	\N	2020-10-12 15:33:19.487621
55391	LGIH	2020-09-28	110.28	116.25	109.7183	115.7	277735	{}	\N	2020-10-12 15:33:20.654962
55392	LGIH	2020-09-29	115.4	115.81	112.62	113.79	145598	{}	\N	2020-10-12 15:33:20.66194
55393	LGIH	2020-09-30	114.78	118.65	114.75	116.17	206804	{}	\N	2020-10-12 15:33:20.669332
55394	LGIH	2020-10-01	118	123.58	115.77	123.17	210917	{}	\N	2020-10-12 15:33:20.678852
55395	LGIH	2020-10-02	119.4	127.59	117.821	126.2	286744	{}	\N	2020-10-12 15:33:20.68911
55396	LGIH	2020-10-05	129.67	131.95	121.1514	126.76	380410	{}	\N	2020-10-12 15:33:20.697331
55397	LGIH	2020-10-06	128.66	130.44	121.35	122.58	263662	{}	\N	2020-10-12 15:33:20.704306
55398	LGIH	2020-10-07	123.85	126.6507	120.465	122.02	299905	{}	\N	2020-10-12 15:33:20.711144
55399	LGIH	2020-10-08	125	126.8	122.64	124.28	159145	{}	\N	2020-10-12 15:33:20.718635
55400	LGIH	2020-10-09	126.47	130.085	124.5	126.26	185082	{}	\N	2020-10-12 15:33:20.724688
55401	LGIH	2020-10-12	127.41	128.67	125.01	128.61	50851	{}	\N	2020-10-12 15:33:20.729794
55402	LIND	2020-09-28	9.0232	9.33	8.655	9.13	1360505	{}	\N	2020-10-12 15:33:21.880128
55403	LIND	2020-09-29	9.165	9.19	8.5	8.58	696569	{}	\N	2020-10-12 15:33:21.886178
55404	LIND	2020-09-30	8.86	9.14	8.41	8.51	430227	{}	\N	2020-10-12 15:33:21.892672
55405	LIND	2020-10-01	8.68	8.8	7.96	8.56	794248	{}	\N	2020-10-12 15:33:21.897188
55406	LIND	2020-10-02	8.29	8.61	8.21	8.52	359746	{}	\N	2020-10-12 15:33:21.903458
55407	LIND	2020-10-05	8.65	8.69	8.32	8.43	283394	{}	\N	2020-10-12 15:33:21.908656
55408	LIND	2020-10-06	8.53	8.92	8.44	8.65	780241	{}	\N	2020-10-12 15:33:21.913011
55409	LIND	2020-10-07	8.74	9.02	8.55	8.92	666582	{}	\N	2020-10-12 15:33:21.91812
55410	LIND	2020-10-08	9.02	9.12	8.37	8.61	476211	{}	\N	2020-10-12 15:33:21.924696
55411	LIND	2020-10-09	8.87	8.9027	8.39	8.81	534370	{}	\N	2020-10-12 15:33:21.929978
55412	LIND	2020-10-12	8.6955	8.7665	8.5	8.57	89555	{}	\N	2020-10-12 15:33:21.935023
55413	LKFN	2020-09-28	40.84	41.77	40.84	41.5	68664	{}	\N	2020-10-12 15:33:23.355945
55414	LKFN	2020-09-29	41.21	41.51	40.77	41.25	57365	{}	\N	2020-10-12 15:33:23.364135
55415	LKFN	2020-09-30	41.33	41.91	40.97	41.2	52678	{}	\N	2020-10-12 15:33:23.371867
55416	LKFN	2020-10-01	41.19	41.57	40.57	41.54	90933	{}	\N	2020-10-12 15:33:23.378612
55417	LKFN	2020-10-02	40.92	42.67	40.92	42.42	62566	{}	\N	2020-10-12 15:33:23.388979
55418	LKFN	2020-10-05	42.7	44.38	42.7	44.31	76643	{}	\N	2020-10-12 15:33:23.39671
55419	LKFN	2020-10-06	44.52	46	44.02	45.55	262820	{}	\N	2020-10-12 15:33:23.402938
55420	LKFN	2020-10-07	46.26	47.73	45.985	47.21	198445	{}	\N	2020-10-12 15:33:23.409308
55421	LKFN	2020-10-08	47.73	47.87	47.02	47.42	92525	{}	\N	2020-10-12 15:33:23.415054
55422	LKFN	2020-10-09	47.88	48.04	47.17	47.53	63182	{}	\N	2020-10-12 15:33:23.421957
55423	LKFN	2020-10-12	47.37	47.56	46.88	47.16	35044	{}	\N	2020-10-12 15:33:23.429658
55424	LLIT	2020-09-28	0.3797	0.383	0.36	0.383	477854	{}	\N	2020-10-12 15:33:24.555545
55425	LLIT	2020-09-29	0.3805	0.5	0.3781	0.4194	2242501	{}	\N	2020-10-12 15:33:24.562701
55426	LLIT	2020-09-30	0.41	0.44	0.3868	0.3995	295573	{}	\N	2020-10-12 15:33:24.567816
55427	LLIT	2020-10-01	0.4	0.40895	0.391	0.3911	128906	{}	\N	2020-10-12 15:33:24.574056
55428	LLIT	2020-10-02	0.4	0.4231	0.3701	0.3901	935055	{}	\N	2020-10-12 15:33:24.579804
55429	LLIT	2020-10-05	0.3881	0.395199	0.38	0.394	166908	{}	\N	2020-10-12 15:33:24.585088
55430	LLIT	2020-10-06	0.3885	0.3999	0.38	0.396	152685	{}	\N	2020-10-12 15:33:24.590786
55431	LLIT	2020-10-07	0.4158	0.42	0.39021	0.4047	123448	{}	\N	2020-10-12 15:33:24.595979
55432	LLIT	2020-10-08	0.397	0.41	0.3782	0.3995	261680	{}	\N	2020-10-12 15:33:24.60217
55433	LLIT	2020-10-09	0.385	0.427	0.38	0.40061	392944	{}	\N	2020-10-12 15:33:24.608205
55434	LLIT	2020-10-12	0.41	0.415	0.4007	0.4091	35398	{}	\N	2020-10-12 15:33:24.612728
55435	LMRKP	2020-09-28	25	25	24.76	24.953	3684	{}	\N	2020-10-12 15:33:27.519152
55436	LMRKP	2020-09-29	24.846	25	24.82	24.91	7593	{}	\N	2020-10-12 15:33:27.527397
55437	LMRKP	2020-09-30	24.68	24.68	24.4792	24.5	7374	{}	\N	2020-10-12 15:33:27.534273
55438	LMRKP	2020-10-01	24.48	24.54	24.2	24.2	7114	{}	\N	2020-10-12 15:33:27.541306
55439	LMRKP	2020-10-02	24.19	24.62	24.19	24.25	4066	{}	\N	2020-10-12 15:33:27.54809
55440	LMRKP	2020-10-05	24.4915	24.4915	24.15	24.31	5792	{}	\N	2020-10-12 15:33:27.554103
55441	LMRKP	2020-10-06	24.31	24.4	24.31	24.3604	635	{}	\N	2020-10-12 15:33:27.561726
55442	LMRKP	2020-10-07	24.4	24.88	24.27	24.27	4950	{}	\N	2020-10-12 15:33:27.567671
55443	LMRKP	2020-10-08	24.5	24.79	24.5	24.77	1769	{}	\N	2020-10-12 15:33:27.573062
55444	LMRKP	2020-10-09	24.53	24.8	24.53	24.8	7658	{}	\N	2020-10-12 15:33:27.578653
55445	LOAN	2020-09-28	4.15	4.25	4.15	4.24	15399	{}	\N	2020-10-12 15:33:28.686661
55446	LOAN	2020-09-29	4.25	4.36	4.25	4.2858	26759	{}	\N	2020-10-12 15:33:28.694677
55447	LOAN	2020-09-30	4.36	4.45	4.35	4.36	33426	{}	\N	2020-10-12 15:33:28.699713
55448	LOAN	2020-10-01	4.38	4.65	4.38	4.6	91976	{}	\N	2020-10-12 15:33:28.704362
55449	LOAN	2020-10-02	4.6	4.65	4.56	4.65	32150	{}	\N	2020-10-12 15:33:28.709747
55450	LOAN	2020-10-05	4.65	4.93	4.632	4.8	162938	{}	\N	2020-10-12 15:33:28.714602
55451	LOAN	2020-10-06	4.94	4.94	4.6967	4.7	40181	{}	\N	2020-10-12 15:33:28.719656
55452	LOAN	2020-10-07	4.77	4.7822	4.6288	4.64	97766	{}	\N	2020-10-12 15:33:28.724858
55453	LOAN	2020-10-08	4.68	4.68	4.4021	4.5	28782	{}	\N	2020-10-12 15:33:28.73031
55454	LOAN	2020-10-09	4.55	4.55	4.4201	4.475	22685	{}	\N	2020-10-12 15:33:28.735831
55455	LOAN	2020-10-12	4.41	4.435	4.2601	4.2999	35465	{}	\N	2020-10-12 15:33:28.740865
55456	LOPE	2020-09-28	84.08	84.55	83.33	83.76	217732	{}	\N	2020-10-12 15:33:29.71884
55457	LOPE	2020-09-29	83.56	83.56	80.39	81.6	196252	{}	\N	2020-10-12 15:33:29.725847
55458	LOPE	2020-09-30	81.75	82.27	79.585	79.94	228885	{}	\N	2020-10-12 15:33:29.732886
55459	LOPE	2020-10-01	79.7	80.32	78.61	80.2	439710	{}	\N	2020-10-12 15:33:29.738013
55460	LOPE	2020-10-02	78.875	80.64	78.3	79.27	235035	{}	\N	2020-10-12 15:33:29.743597
55461	LOPE	2020-10-05	79.41	81.85	79.41	81.19	446484	{}	\N	2020-10-12 15:33:29.750473
55462	LOPE	2020-10-06	81.26	86.24	81.26	84.37	305027	{}	\N	2020-10-12 15:33:29.756107
55463	LOPE	2020-10-07	85.16	85.6265	81.94	82.3	271902	{}	\N	2020-10-12 15:33:29.761842
55464	LOPE	2020-10-08	82.5	83.3	81.83	82.51	132766	{}	\N	2020-10-12 15:33:29.769154
55465	LOPE	2020-10-09	82.71	83.58	82.61	83.27	104619	{}	\N	2020-10-12 15:33:29.774938
55466	LOPE	2020-10-12	83.65	83.75	83.18	83.2201	32962	{}	\N	2020-10-12 15:33:29.781227
55467	LPL	2020-09-28	6.54	6.64	6.54	6.62	397678	{}	\N	2020-10-12 15:33:30.721761
55468	LPL	2020-09-29	6.67	6.6801	6.6	6.62	603982	{}	\N	2020-10-12 15:33:30.731747
55469	LPL	2020-09-30	6.62	6.695	6.51	6.53	722078	{}	\N	2020-10-12 15:33:30.75064
55470	LPL	2020-10-01	6.58	6.63	6.49	6.54	846139	{}	\N	2020-10-12 15:33:30.759441
55471	LPL	2020-10-02	6.39	6.54	6.365	6.47	989517	{}	\N	2020-10-12 15:33:30.778299
55472	LPL	2020-10-05	6.63	6.76	6.63	6.76	905349	{}	\N	2020-10-12 15:33:30.788065
55473	LPL	2020-10-06	6.62	6.66	6.39	6.51	4024616	{}	\N	2020-10-12 15:33:30.796413
55474	LPL	2020-10-07	6.54	6.61	6.53	6.59	2398932	{}	\N	2020-10-12 15:33:30.803959
55475	LPL	2020-10-08	6.81	6.88	6.77	6.78	1056545	{}	\N	2020-10-12 15:33:30.812021
55476	LPL	2020-10-09	6.85	6.905	6.81	6.89	1095545	{}	\N	2020-10-12 15:33:30.819242
55477	LPL	2020-10-12	6.91	6.94	6.84	6.91	156168	{}	\N	2020-10-12 15:33:30.825477
55478	LTL	2020-09-28	32.1911	32.1911	32.1911	32.1911	20	{}	\N	2020-10-12 15:33:33.093201
55479	LTL	2020-09-29	32.2519	32.2519	32.2519	32.2519	54	{}	\N	2020-10-12 15:33:33.104908
55480	LTL	2020-09-30	32.2611	32.2611	32.2611	32.2611	25	{}	\N	2020-10-12 15:33:33.115913
55481	LTL	2020-10-01	32.23	32.3117	32.2058	32.2058	317	{}	\N	2020-10-12 15:33:33.128315
55482	LTL	2020-10-02	32.1475	32.1475	32.1475	32.1475	58	{}	\N	2020-10-12 15:33:33.14061
55483	LTL	2020-10-05	32.7717	32.7717	32.7717	32.7717	57	{}	\N	2020-10-12 15:33:33.152945
55484	LTL	2020-10-06	32.6249	32.6249	32.6249	32.6249	195	{}	\N	2020-10-12 15:33:33.167108
55485	LTL	2020-10-08	33.9416	33.9416	33.9416	33.9416	0	{}	\N	2020-10-12 15:33:33.18971
55486	LTL	2020-10-09	34.3954	34.3954	34.3954	34.3954	0	{}	\N	2020-10-12 15:33:33.217874
55487	LUNA	2020-09-28	5.93	6.029	5.91	5.96	112370	{}	\N	2020-10-12 15:33:34.137712
55488	LUNA	2020-09-29	5.96	6.0401	5.86	5.98	85165	{}	\N	2020-10-12 15:33:34.146915
55489	LUNA	2020-09-30	5.96	6.09	5.91	5.97	167735	{}	\N	2020-10-12 15:33:34.154897
55490	LUNA	2020-10-01	6.03	6.15	5.95	6.1	157870	{}	\N	2020-10-12 15:33:34.162152
55491	LUNA	2020-10-02	5.92	6.16	5.92	6.14	128251	{}	\N	2020-10-12 15:33:34.168537
55492	LUNA	2020-10-05	6.14	6.42	6.14	6.4	180819	{}	\N	2020-10-12 15:33:34.174236
55493	LUNA	2020-10-06	6.45	6.5	6.32	6.4	136283	{}	\N	2020-10-12 15:33:34.18383
55494	LUNA	2020-10-07	6.42	6.46	6.24	6.32	98567	{}	\N	2020-10-12 15:33:34.193466
55495	LUNA	2020-10-08	6.36	6.43	6.29	6.35	93266	{}	\N	2020-10-12 15:33:34.203322
55496	LUNA	2020-10-09	6.39	6.59	6.2901	6.54	132301	{}	\N	2020-10-12 15:33:34.216386
55497	LUNA	2020-10-12	6.61	6.6609	6.48	6.55	64046	{}	\N	2020-10-12 15:33:34.224629
55498	LZB	2020-09-28	30.82	31.31	30.69	30.94	612874	{}	\N	2020-10-12 15:33:36.142454
55499	LZB	2020-09-29	30.81	31.05	30.46	30.66	450148	{}	\N	2020-10-12 15:33:36.149234
55500	LZB	2020-09-30	30.92	32	30.92	31.63	2279774	{}	\N	2020-10-12 15:33:36.154725
55501	LZB	2020-10-01	31.92	32.79	31.32	32.73	1628534	{}	\N	2020-10-12 15:33:36.162312
55502	LZB	2020-10-02	31.93	33.13	31.93	32.87	431805	{}	\N	2020-10-12 15:33:36.168815
55503	LZB	2020-10-05	33.16	33.32	32.58	33.11	495731	{}	\N	2020-10-12 15:33:36.174329
55504	LZB	2020-10-06	33.5	33.54	32.4401	32.62	1184741	{}	\N	2020-10-12 15:33:36.178957
55505	LZB	2020-10-07	33	33.74	32.75	33.67	705427	{}	\N	2020-10-12 15:33:36.184242
55506	LZB	2020-10-08	34.11	34.41	33.575	34.32	674075	{}	\N	2020-10-12 15:33:36.189779
55507	LZB	2020-10-09	34.5	34.68	33.7	34.2	580190	{}	\N	2020-10-12 15:33:36.195284
55508	LZB	2020-10-12	34.09	34.6199	34.0289	34.295	96783	{}	\N	2020-10-12 15:33:36.201242
55509	LIFE	2020-09-28	3.73	3.81	3.6291	3.66	46185	{}	\N	2020-10-12 15:33:37.166412
55510	LIFE	2020-09-29	3.68	3.6936	3.25	3.27	275979	{}	\N	2020-10-12 15:33:37.174082
55511	LIFE	2020-09-30	3.31	3.41	3.23	3.23	105619	{}	\N	2020-10-12 15:33:37.180332
55512	LIFE	2020-10-01	3.25	3.3399	3.23	3.28	42810	{}	\N	2020-10-12 15:33:37.186653
55513	LIFE	2020-10-02	3.15	3.25	3.08	3.1	83764	{}	\N	2020-10-12 15:33:37.193535
55514	LIFE	2020-10-05	3.11	3.33	3.11	3.3	35227	{}	\N	2020-10-12 15:33:37.198879
55515	LIFE	2020-10-06	3.36	3.42	3.275	3.28	34552	{}	\N	2020-10-12 15:33:37.205236
55516	LIFE	2020-10-07	3.27	3.477	3.215	3.3	65721	{}	\N	2020-10-12 15:33:37.211085
55517	LIFE	2020-10-08	3.37	3.55	3.367	3.49	120989	{}	\N	2020-10-12 15:33:37.216386
55518	LIFE	2020-10-09	3.55	3.66	3.37	3.46	110789	{}	\N	2020-10-12 15:33:37.222395
55519	LIFE	2020-10-12	3.48	3.59	3.45	3.54	15004	{}	\N	2020-10-12 15:33:37.227321
55520	LL	2020-09-28	21.48	21.85	21.115	21.76	370744	{}	\N	2020-10-12 15:33:38.159752
55521	LL	2020-09-29	21.78	21.94	21.32	21.63	553207	{}	\N	2020-10-12 15:33:38.174851
55522	LL	2020-09-30	21.56	22.625	21.56	22.05	1088340	{}	\N	2020-10-12 15:33:38.1831
55523	LL	2020-10-01	22.11	22.57	21.71	22.41	841178	{}	\N	2020-10-12 15:33:38.192639
55524	LL	2020-10-02	21.49	23.1	21.48	22.94	859848	{}	\N	2020-10-12 15:33:38.199151
55525	LL	2020-10-05	23	23.57	22.49	23.45	1105676	{}	\N	2020-10-12 15:33:38.208956
55526	LL	2020-10-06	23.72	24.93	23.5337	24.11	1995827	{}	\N	2020-10-12 15:33:38.216464
55527	LL	2020-10-07	24.475	25.21	24.365	24.93	1103121	{}	\N	2020-10-12 15:33:38.22441
55528	LL	2020-10-08	25.18	25.95	25	25.88	1166440	{}	\N	2020-10-12 15:33:38.23142
55529	LL	2020-10-09	26.13	27.99	26.05	26.96	2256254	{}	\N	2020-10-12 15:33:38.246679
55530	LL	2020-10-12	27.25	27.33	26.1701	26.69	406160	{}	\N	2020-10-12 15:33:38.300278
55531	LMFA	2020-09-28	0.6029	0.805	0.6004	0.7597	28430231	{}	\N	2020-10-12 15:33:39.272606
55532	LMFA	2020-09-29	1.03	1.23	0.8069	0.87	62317825	{}	\N	2020-10-12 15:33:39.280268
55533	LMFA	2020-09-30	0.7	0.75	0.65	0.651001	8381668	{}	\N	2020-10-12 15:33:39.28546
55534	LMFA	2020-10-01	0.62	0.647	0.605	0.6457	1795736	{}	\N	2020-10-12 15:33:39.294545
55535	LMFA	2020-10-02	0.62	0.699	0.6051	0.6501	1861882	{}	\N	2020-10-12 15:33:39.320796
55536	LMFA	2020-10-05	0.637	0.6393	0.5831	0.5988	1881877	{}	\N	2020-10-12 15:33:39.335508
55537	LMFA	2020-10-06	0.58	0.655	0.55	0.609	3979631	{}	\N	2020-10-12 15:33:39.345575
55538	LMFA	2020-10-07	0.609	0.6738	0.58	0.6294	2619509	{}	\N	2020-10-12 15:33:39.356838
55539	LMFA	2020-10-08	0.615	0.72	0.61	0.6257	4651678	{}	\N	2020-10-12 15:33:39.384101
55540	LMFA	2020-10-09	0.6129	0.6495	0.608	0.61	1126543	{}	\N	2020-10-12 15:33:39.43162
55541	LMFA	2020-10-12	0.6358	0.73	0.6325	0.6522	3346659	{}	\N	2020-10-12 15:33:39.466266
55542	LN	2020-09-28	50.4	51.08	50.4	50.75	13316	{}	\N	2020-10-12 15:33:42.374573
55543	LN	2020-09-29	50.78	50.91	50.54	50.91	6366	{}	\N	2020-10-12 15:33:42.380292
55544	LN	2020-09-30	50.83	50.91	50.58	50.8	10068	{}	\N	2020-10-12 15:33:42.385308
55545	LN	2020-10-01	50.83	50.89	50.5901	50.75	12198	{}	\N	2020-10-12 15:33:42.389942
55546	LN	2020-10-02	50.63	51.05	50.63	51.03	19856	{}	\N	2020-10-12 15:33:42.395455
55547	LN	2020-10-05	51.09	51.09	50.64	50.64	13360	{}	\N	2020-10-12 15:33:42.39998
55548	LN	2020-10-06	50.74	51.2	50.74	51.14	14135	{}	\N	2020-10-12 15:33:42.40421
55549	LN	2020-10-07	51.25	51.25	50.7139	51.03	11794	{}	\N	2020-10-12 15:33:42.409402
55550	LN	2020-10-08	51.42	51.42	50.8	51.01	28414	{}	\N	2020-10-12 15:33:42.415582
55551	LN	2020-10-09	51.05	51.17	50.87	51.17	4664	{}	\N	2020-10-12 15:33:42.420137
55552	LNTH	2020-09-28	12.36	12.64	12.345	12.43	423783	{}	\N	2020-10-12 15:33:43.353262
55553	LNTH	2020-09-29	12.46	12.52	12.12	12.35	298102	{}	\N	2020-10-12 15:33:43.35949
55554	LNTH	2020-09-30	12.43	12.8	12.43	12.69	265377	{}	\N	2020-10-12 15:33:43.366522
55555	LNTH	2020-10-01	12.67	13	12.67	12.93	355570	{}	\N	2020-10-12 15:33:43.373085
55556	LNTH	2020-10-02	12.775	12.96	12.36	12.89	407434	{}	\N	2020-10-12 15:33:43.381606
55557	LNTH	2020-10-05	13.08	13.63	13.08	13.6	462979	{}	\N	2020-10-12 15:33:43.387392
55558	LNTH	2020-10-06	13.75	14.09	13.61	13.68	572728	{}	\N	2020-10-12 15:33:43.394013
55559	LNTH	2020-10-07	13.87	14.2	13.79	13.86	699230	{}	\N	2020-10-12 15:33:43.400693
55560	LNTH	2020-10-08	14.11	14.45	13.81	13.84	454037	{}	\N	2020-10-12 15:33:43.406334
55561	LNTH	2020-10-09	13.92	14.12	13.3	13.42	236202	{}	\N	2020-10-12 15:33:43.414596
55562	LNTH	2020-10-12	13.41	13.41	13.2	13.24	90303	{}	\N	2020-10-12 15:33:43.42135
55563	LOW	2020-09-28	161.59	163.1	160.51	162.59	3026553	{}	\N	2020-10-12 15:33:44.350353
55564	LOW	2020-09-29	163.48	163.97	162.21	162.26	4327805	{}	\N	2020-10-12 15:33:44.35703
55565	LOW	2020-09-30	162.98	167.62	162.48	165.86	8268621	{}	\N	2020-10-12 15:33:44.36464
55566	LOW	2020-10-01	167.34	168.28	165.3698	167.17	4832229	{}	\N	2020-10-12 15:33:44.370383
55567	LOW	2020-10-02	164.47	167.46	164.05	166.49	3760024	{}	\N	2020-10-12 15:33:44.37709
55568	LOW	2020-10-05	168.11	168.97	167.01	168.54	4729386	{}	\N	2020-10-12 15:33:44.383814
55569	LOW	2020-10-06	168.48	168.98	164.42	164.99	4963484	{}	\N	2020-10-12 15:33:44.389559
55570	LOW	2020-10-07	167.72	169.81	166.635	168.47	4905640	{}	\N	2020-10-12 15:33:44.395652
55571	LOW	2020-10-08	169.49	171.61	168.2441	170.87	6137912	{}	\N	2020-10-12 15:33:44.402132
55572	LOW	2020-10-09	171.37	175	171.37	173.23	8662877	{}	\N	2020-10-12 15:33:44.412441
55573	LOW	2020-10-12	174.84	174.84	172.66	174.43	2736577	{}	\N	2020-10-12 15:33:44.421
55574	LPTX	2020-09-28	1.97	1.97	1.85	1.88	165295	{}	\N	2020-10-12 15:33:45.348275
55575	LPTX	2020-09-29	1.88	1.99	1.825	1.93	392495	{}	\N	2020-10-12 15:33:45.35374
55576	LPTX	2020-09-30	1.94	2.02	1.91	1.98	276928	{}	\N	2020-10-12 15:33:45.359013
55577	LPTX	2020-10-01	1.96	2.05	1.96	2	186257	{}	\N	2020-10-12 15:33:45.366802
55578	LPTX	2020-10-02	1.95	2.05	1.93	2	304249	{}	\N	2020-10-12 15:33:45.37318
55579	LPTX	2020-10-05	2.01	2.0599	1.98	2.02	254622	{}	\N	2020-10-12 15:33:45.380433
55580	LPTX	2020-10-06	2.0509	2.0548	1.98	2	262277	{}	\N	2020-10-12 15:33:45.386171
55581	LPTX	2020-10-07	2.02	2.07	1.986	2.05	240837	{}	\N	2020-10-12 15:33:45.391551
55582	LPTX	2020-10-08	2.06	2.0728	2.02	2.06	173106	{}	\N	2020-10-12 15:33:45.398222
55583	LPTX	2020-10-09	2.06	2.07	2.03	2.07	484260	{}	\N	2020-10-12 15:33:45.404042
55584	LPTX	2020-10-12	2.08	2.22	2.08	2.1989	364048	{}	\N	2020-10-12 15:33:45.412111
55585	LRGE	2020-09-28	45.42	45.6	45.37	45.6	4448	{}	\N	2020-10-12 15:33:46.904127
55586	LRGE	2020-09-29	45.605	45.71	45.35	45.5888	14498	{}	\N	2020-10-12 15:33:46.909455
55587	LRGE	2020-09-30	45.705	46.18	45.5465	45.7222	6512	{}	\N	2020-10-12 15:33:46.916825
55588	LRGE	2020-10-01	46.21	46.35	46.045	46.1726	4728	{}	\N	2020-10-12 15:33:46.922557
55589	LRGE	2020-10-02	45.845	45.8799	45.5035	45.5035	13063	{}	\N	2020-10-12 15:33:46.929194
55590	LRGE	2020-10-05	45.8937	46.2737	45.8937	46.2737	6187	{}	\N	2020-10-12 15:33:46.934579
55591	LRGE	2020-10-06	46.41	46.5	45.5944	45.73	5642	{}	\N	2020-10-12 15:33:46.938938
55592	LRGE	2020-10-07	46.27	46.68	46.2511	46.6335	11777	{}	\N	2020-10-12 15:33:46.943404
55593	LRGE	2020-10-08	46.62	46.77	46.62	46.7017	3594	{}	\N	2020-10-12 15:33:46.948333
55594	LRGE	2020-10-09	47.03	47.3599	47	47.2037	8806	{}	\N	2020-10-12 15:33:46.952881
55595	LRGE	2020-10-12	47.835	47.835	47.6452	47.6452	719	{}	\N	2020-10-12 15:33:46.960076
55596	LSXMB	2020-10-01	37.7	37.7	37.7	37.7	497	{}	\N	2020-10-12 15:33:48.298799
55597	LSXMB	2020-10-02	34.4501	34.4501	34.4501	34.4501	215	{}	\N	2020-10-12 15:33:48.307981
55598	LSXMB	2020-10-08	36.05	39.67	36.02	39.67	940	{}	\N	2020-10-12 15:33:48.316192
55599	LSXMB	2020-10-12	36.55	36.55	36.55	36.55	115	{}	\N	2020-10-12 15:33:48.325058
55600	LYV	2020-09-28	54.19	55.09	53.3	54.41	1534018	{}	\N	2020-10-12 15:33:50.21201
55601	LYV	2020-09-29	54.17	54.315	53	53.29	2914430	{}	\N	2020-10-12 15:33:50.218482
55602	LYV	2020-09-30	53.62	55.5	53.45	53.88	3174662	{}	\N	2020-10-12 15:33:50.225239
55603	LYV	2020-10-01	54.37	54.58	52.93	53.74	2464630	{}	\N	2020-10-12 15:33:50.230689
55604	LYV	2020-10-02	52.24	55.25	52.12	54.73	2715955	{}	\N	2020-10-12 15:33:50.2394
55605	LYV	2020-10-05	55.28	55.52	53.93	54.56	3536335	{}	\N	2020-10-12 15:33:50.2467
55606	LYV	2020-10-06	55.01	56.68	53.8	54.21	3770981	{}	\N	2020-10-12 15:33:50.256212
55607	LYV	2020-10-07	55.23	55.9676	54.43	54.82	2338632	{}	\N	2020-10-12 15:33:50.262284
55608	LYV	2020-10-08	55.25	56.3781	54.635	56.27	3055297	{}	\N	2020-10-12 15:33:50.268684
55609	LYV	2020-10-09	56.95	57.98	56.76	56.83	3043349	{}	\N	2020-10-12 15:33:50.274509
55610	LYV	2020-10-12	56.13	56.37	54.69	55.195	722603	{}	\N	2020-10-12 15:33:50.279808
55611	MAC	2020-09-28	6.92	7.26	6.9001	7.09	3382062	{}	\N	2020-10-12 15:33:51.604456
55612	MAC	2020-09-29	7.06	7.06	6.76	6.9	12148046	{}	\N	2020-10-12 15:33:51.613714
55613	MAC	2020-09-30	6.9	7.12	6.75	6.79	8216103	{}	\N	2020-10-12 15:33:51.624207
55614	MAC	2020-10-01	6.79	7.02	6.685	7.01	6862661	{}	\N	2020-10-12 15:33:51.638524
55615	MAC	2020-10-02	6.75	7.455	6.7401	7.41	7863339	{}	\N	2020-10-12 15:33:51.647315
55616	MAC	2020-10-05	7.52	7.58	7.3	7.47	4627077	{}	\N	2020-10-12 15:33:51.658204
55617	MAC	2020-10-06	7.6	7.8	7.36	7.44	7047872	{}	\N	2020-10-12 15:33:51.664476
55618	MAC	2020-10-07	7.5	7.71	7.5	7.62	6316927	{}	\N	2020-10-12 15:33:51.67096
55619	MAC	2020-10-08	7.74	7.81	7.57	7.74	5569973	{}	\N	2020-10-12 15:33:51.675633
55620	MAC	2020-10-09	7.8	7.84	7.48	7.52	5646609	{}	\N	2020-10-12 15:33:51.680134
55621	MAC	2020-10-12	7.48	7.585	7.347	7.5	1401275	{}	\N	2020-10-12 15:33:51.685092
55622	MARK	2020-09-28	1.15	1.2	1.13	1.15	2130154	{}	\N	2020-10-12 15:33:52.960365
55623	MARK	2020-09-29	1.13	1.17	1.12	1.12	1213231	{}	\N	2020-10-12 15:33:52.966047
55624	MARK	2020-09-30	1.15	1.24	1.1251	1.17	3140990	{}	\N	2020-10-12 15:33:52.971419
55625	MARK	2020-10-01	1.17	1.19	1.13	1.16	1280835	{}	\N	2020-10-12 15:33:52.976392
55626	MARK	2020-10-02	1.13	1.23	1.13	1.16	2933703	{}	\N	2020-10-12 15:33:52.981718
55627	MARK	2020-10-05	1.16	1.23	1.15	1.2	2294165	{}	\N	2020-10-12 15:33:52.988175
55628	MARK	2020-10-06	1.19	1.26	1.17	1.17	2697744	{}	\N	2020-10-12 15:33:52.994555
55629	MARK	2020-10-07	1.176	1.23	1.176	1.19	1527174	{}	\N	2020-10-12 15:33:52.999931
55630	MARK	2020-10-08	1.19	1.23	1.19	1.2	4656133	{}	\N	2020-10-12 15:33:53.009019
55631	MARK	2020-10-09	1.31	1.38	1.23	1.24	6473082	{}	\N	2020-10-12 15:33:53.016029
55632	MARK	2020-10-12	1.24	1.2799	1.24	1.245	691186	{}	\N	2020-10-12 15:33:53.024115
55633	LLY	2020-09-28	151	151.58	147.9	148.1	2441487	{}	\N	2020-10-12 15:33:54.160937
55634	LLY	2020-09-29	147.98	148.44	145.93	146.26	6333295	{}	\N	2020-10-12 15:33:54.168813
55635	LLY	2020-09-30	146.4	148.445	145.06	148.02	10143552	{}	\N	2020-10-12 15:33:54.17522
55636	LLY	2020-10-01	148.33	148.705	144.06	144.29	6883599	{}	\N	2020-10-12 15:33:54.179864
55637	LLY	2020-10-02	143.67	146.94	143.27	145.21	7192750	{}	\N	2020-10-12 15:33:54.185624
55638	LLY	2020-10-05	145.07	147.47	144.18	146.63	5536100	{}	\N	2020-10-12 15:33:54.192399
55639	LLY	2020-10-06	146.93	147.83	143.93	144.13	4388389	{}	\N	2020-10-12 15:33:54.197666
55640	LLY	2020-10-07	148.52	149.89	146.85	148.96	9114757	{}	\N	2020-10-12 15:33:54.203882
55641	LLY	2020-10-08	152.4	153.94	150.5	153.5	9450026	{}	\N	2020-10-12 15:33:54.210332
55642	LLY	2020-10-09	154.09	157.15	154.07	156.88	8952512	{}	\N	2020-10-12 15:33:54.216301
55643	LLY	2020-10-12	156.83	157.15	155.22	155.43	2453779	{}	\N	2020-10-12 15:33:54.222438
55644	LNGR	2020-09-28	25.4299	25.47	25.39	25.47	3764	{}	\N	2020-10-12 15:33:55.311458
55645	LNGR	2020-09-29	25.42	25.54	25.42	25.54	5751	{}	\N	2020-10-12 15:33:55.316874
55646	LNGR	2020-09-30	25.715	25.81	25.6633	25.6633	934	{}	\N	2020-10-12 15:33:55.322077
55647	LNGR	2020-10-01	25.76	25.845	25.675	25.73	1945	{}	\N	2020-10-12 15:33:55.327279
55648	LNGR	2020-10-02	25.588	25.6201	25.516	25.595	2531	{}	\N	2020-10-12 15:33:55.334051
55649	LNGR	2020-10-05	25.775	26.13	25.775	26.11	6675	{}	\N	2020-10-12 15:33:55.340845
55650	LNGR	2020-10-06	26.09	26.1	25.8393	25.98	2712	{}	\N	2020-10-12 15:33:55.346649
55651	LNGR	2020-10-07	25.98	26.29	25.98	26.29	11942	{}	\N	2020-10-12 15:33:55.351825
55652	LNGR	2020-10-08	26.28	26.3879	26.28	26.38	983	{}	\N	2020-10-12 15:33:55.357414
55653	LNGR	2020-10-09	26.538	26.59	26.4898	26.59	2802	{}	\N	2020-10-12 15:33:55.36264
55654	LNGR	2020-10-12	26.7099	26.7099	26.705	26.7052	2431	{}	\N	2020-10-12 15:33:55.368067
55655	LPLA	2020-09-28	76.72	78.12	76.245	77.15	358344	{}	\N	2020-10-12 15:33:57.584097
55656	LPLA	2020-09-29	77.01	77.03	75.42	76.04	363754	{}	\N	2020-10-12 15:33:57.592262
55657	LPLA	2020-09-30	75.97	77.7	75.97	76.67	1032295	{}	\N	2020-10-12 15:33:57.599339
55658	LPLA	2020-10-01	76.71	77.63	75.82	76.65	992343	{}	\N	2020-10-12 15:33:57.606704
55659	LPLA	2020-10-02	75.44	78.43	75.14	77.11	601460	{}	\N	2020-10-12 15:33:57.616434
55660	LPLA	2020-10-05	78.17	79.86	77.61	79.46	1099978	{}	\N	2020-10-12 15:33:57.625538
55661	LPLA	2020-10-06	80.42	82.4	79.27	79.74	581012	{}	\N	2020-10-12 15:33:57.635031
55662	LPLA	2020-10-07	79.96	81.725	79.1	79.93	558004	{}	\N	2020-10-12 15:33:57.643973
55663	LPLA	2020-10-08	80.35	82.07	80.03	82.05	353018	{}	\N	2020-10-12 15:33:57.652799
55664	LPLA	2020-10-09	82.56	83.55	82.5	82.94	271142	{}	\N	2020-10-12 15:33:57.65887
55665	LPLA	2020-10-12	82.99	84.06	82.32	83.59	32686	{}	\N	2020-10-12 15:33:57.664173
55666	LQDT	2020-09-28	6.81	7.35	6.81	7.255	206068	{}	\N	2020-10-12 15:33:58.552292
55667	LQDT	2020-09-29	7.5	8.07	7.45	7.725	266782	{}	\N	2020-10-12 15:33:58.559302
55668	LQDT	2020-09-30	7.7	7.81	7.37	7.46	172121	{}	\N	2020-10-12 15:33:58.565385
55669	LQDT	2020-10-01	7.39	7.56	7.25	7.4	141441	{}	\N	2020-10-12 15:33:58.571841
55670	LQDT	2020-10-02	7.25	7.74	7.2308	7.62	205746	{}	\N	2020-10-12 15:33:58.577916
55671	LQDT	2020-10-05	7.64	8.24	7.64	8.17	575535	{}	\N	2020-10-12 15:33:58.583699
55672	LQDT	2020-10-06	8.18	8.93	8.171	8.76	1148222	{}	\N	2020-10-12 15:33:58.58983
55673	LQDT	2020-10-07	9.11	9.96	9.02	9.83	757156	{}	\N	2020-10-12 15:33:58.597142
55674	LQDT	2020-10-08	10	10.43	9.96	10.21	468912	{}	\N	2020-10-12 15:33:58.602799
55675	LQDT	2020-10-09	10.58	11.185	10.46	11	776544	{}	\N	2020-10-12 15:33:58.608406
55676	LQDT	2020-10-12	11.2147	11.2147	10.65	10.79	204974	{}	\N	2020-10-12 15:33:58.614626
55677	LSST	2020-09-28	25.945	25.945	25.945	25.945	187	{}	\N	2020-10-12 15:33:59.566813
55678	LSST	2020-09-29	25.945	25.95	25.94	25.94	1931	{}	\N	2020-10-12 15:33:59.572463
55679	LSST	2020-09-30	25.911	25.911	25.895	25.895	8659	{}	\N	2020-10-12 15:33:59.580065
55680	LSST	2020-10-01	25.91	25.92	25.89	25.92	6712	{}	\N	2020-10-12 15:33:59.587445
55681	LSST	2020-10-02	25.92	25.94	25.905	25.94	3577	{}	\N	2020-10-12 15:33:59.598514
55682	LSST	2020-10-05	25.9399	25.9399	25.93	25.93	1284	{}	\N	2020-10-12 15:33:59.606954
55683	LSST	2020-10-06	25.93	25.97	25.93	25.94	1320	{}	\N	2020-10-12 15:33:59.615186
55684	LSST	2020-10-07	25.96	25.9899	25.96	25.9899	200	{}	\N	2020-10-12 15:33:59.622729
55685	LSST	2020-10-08	25.99	25.99	25.99	25.99	0	{}	\N	2020-10-12 15:33:59.631458
55686	LSST	2020-10-09	25.99	26	25.99	26	402	{}	\N	2020-10-12 15:33:59.638305
55687	LSST	2020-10-12	26	26.0022	26	26.0022	600	{}	\N	2020-10-12 15:33:59.645931
55688	LTRPB	2020-10-01	34.0001	37.1	34.0001	37.1	471	{}	\N	2020-10-12 15:34:01.850716
55689	LTRPB	2020-10-02	33.25	33.25	33.25	33.25	609	{}	\N	2020-10-12 15:34:01.85819
55690	LTRPB	2020-10-05	35.9889	36.33	35.9889	36	1400	{}	\N	2020-10-12 15:34:01.865182
55691	LTRPB	2020-10-06	34.35	36.3299	34.35	34.37	1442	{}	\N	2020-10-12 15:34:01.873113
55692	LTRPB	2020-10-07	34.69	35.9322	34.69	35.9322	724	{}	\N	2020-10-12 15:34:01.88037
55693	LTRPB	2020-10-08	34.5975	36.1487	34.5975	34.8515	1719	{}	\N	2020-10-12 15:34:01.889447
55694	LUV	2020-09-28	38.07	39.11	37.57	38.24	9402346	{}	\N	2020-10-12 15:34:02.756815
55695	LUV	2020-09-29	38.05	38.05	37.19	37.61	10241899	{}	\N	2020-10-12 15:34:02.766885
55696	LUV	2020-09-30	38.33	39.08	37.4	37.5	20020277	{}	\N	2020-10-12 15:34:02.772897
55697	LUV	2020-10-01	37.96	38.26	37.39	37.93	14949007	{}	\N	2020-10-12 15:34:02.778821
55698	LUV	2020-10-02	36.145	39.32	36.05	38.76	22727138	{}	\N	2020-10-12 15:34:02.785468
55699	LUV	2020-10-05	39.16	39.24	38.17	38.49	18529587	{}	\N	2020-10-12 15:34:02.790384
55700	LUV	2020-10-06	38.97	39.335	37.535	37.58	18515911	{}	\N	2020-10-12 15:34:02.794938
55701	LUV	2020-10-07	38	38.735	37.71	38.59	17099894	{}	\N	2020-10-12 15:34:02.801275
55702	LUV	2020-10-08	39	39.6	38.03	38.99	18155724	{}	\N	2020-10-12 15:34:02.80727
55703	LUV	2020-10-09	38.95	39.8	38.6	39.73	20192084	{}	\N	2020-10-12 15:34:02.812154
55704	LUV	2020-10-12	39.49	39.89	39.05	39.3784	5282823	{}	\N	2020-10-12 15:34:02.818092
55705	LYG	2020-09-28	1.3	1.33	1.3	1.31	9161634	{}	\N	2020-10-12 15:34:04.951486
55706	LYG	2020-09-29	1.3	1.32	1.28	1.29	14468518	{}	\N	2020-10-12 15:34:04.958401
55707	LYG	2020-09-30	1.31	1.33	1.3	1.3	11819307	{}	\N	2020-10-12 15:34:04.963412
55708	LYG	2020-10-01	1.31	1.32	1.29	1.31	8702376	{}	\N	2020-10-12 15:34:04.969721
55709	LYG	2020-10-02	1.31	1.37	1.31	1.35	12803872	{}	\N	2020-10-12 15:34:04.97563
55710	LYG	2020-10-05	1.34	1.37	1.33	1.35	12006892	{}	\N	2020-10-12 15:34:04.982857
55711	LYG	2020-10-06	1.39	1.4	1.36	1.38	11685523	{}	\N	2020-10-12 15:34:04.988445
55712	LYG	2020-10-07	1.4	1.41	1.37	1.39	7912209	{}	\N	2020-10-12 15:34:04.997916
55713	LYG	2020-10-08	1.41	1.44	1.4	1.43	13487226	{}	\N	2020-10-12 15:34:05.006213
55714	LYG	2020-10-09	1.44	1.45	1.41	1.43	10438675	{}	\N	2020-10-12 15:34:05.013536
55715	LYG	2020-10-12	1.44	1.44	1.42	1.425	4005880	{}	\N	2020-10-12 15:34:05.021054
55716	MAGS	2020-09-28	3.57	3.7496	3.53	3.72	6132	{}	\N	2020-10-12 15:34:05.963228
55717	MAGS	2020-09-29	3.7434	3.75	3.7283	3.73	2442	{}	\N	2020-10-12 15:34:05.973384
55718	MAGS	2020-09-30	3.65	3.75	3.63	3.73	29811	{}	\N	2020-10-12 15:34:05.981082
55719	MAGS	2020-10-01	3.76	3.8399	3.73	3.78	21115	{}	\N	2020-10-12 15:34:05.988985
55720	MAGS	2020-10-02	3.78	3.83	3.775	3.775	10733	{}	\N	2020-10-12 15:34:05.998663
55721	MAGS	2020-10-05	3.82	3.99	3.7889	3.88	71827	{}	\N	2020-10-12 15:34:06.005926
55722	MAGS	2020-10-06	3.9373	3.955	3.87	3.89	30621	{}	\N	2020-10-12 15:34:06.01195
55723	MAGS	2020-10-07	3.85	3.97	3.8	3.82	15573	{}	\N	2020-10-12 15:34:06.021226
55724	MAGS	2020-10-08	3.82	3.9099	3.77	3.799	28497	{}	\N	2020-10-12 15:34:06.027584
55725	MAGS	2020-10-09	3.81	3.88	3.81	3.88	13680	{}	\N	2020-10-12 15:34:06.032368
55726	MAGS	2020-10-12	3.85	3.85	3.8	3.83	4355	{}	\N	2020-10-12 15:34:06.038063
55727	MAS	2020-09-28	55.71	57.02	55.54	56.72	2425605	{}	\N	2020-10-12 15:34:07.300727
55728	MAS	2020-09-29	56.61	57.05	56.04	56.12	2941898	{}	\N	2020-10-12 15:34:07.307496
55729	MAS	2020-09-30	55.23	56.9	54.53	55.13	8852211	{}	\N	2020-10-12 15:34:07.314637
55730	MAS	2020-10-01	55.67	56.15	54.65	55.14	4648474	{}	\N	2020-10-12 15:34:07.321576
55731	MAS	2020-10-02	54.33	55.64	54.29	55.29	2743698	{}	\N	2020-10-12 15:34:07.327715
55732	MAS	2020-10-05	55.9	56.22	55.45	56.07	2749654	{}	\N	2020-10-12 15:34:07.333638
55733	MAS	2020-10-06	56.25	56.29	54.32	54.35	3990968	{}	\N	2020-10-12 15:34:07.340128
55734	MAS	2020-10-07	55.06	56.17	55	55.89	3448793	{}	\N	2020-10-12 15:34:07.346231
55735	MAS	2020-10-08	56.17	57.38	55.965	57.34	3463097	{}	\N	2020-10-12 15:34:07.353546
55736	MAS	2020-10-09	57.87	58.43	57.44	57.47	3489543	{}	\N	2020-10-12 15:34:07.361291
55737	MAS	2020-10-12	57.86	57.97	57.3	57.86	507000	{}	\N	2020-10-12 15:34:07.367464
55738	LTBR	2020-09-28	4.5	4.5339	3.86	4.05	200064	{}	\N	2020-10-12 15:34:08.253074
55739	LTBR	2020-09-29	4.11	5.05	4	4.19	393993	{}	\N	2020-10-12 15:34:08.259824
55740	LTBR	2020-09-30	4.2	4.3822	3.7601	4.17	165070	{}	\N	2020-10-12 15:34:08.266063
55741	LTBR	2020-10-01	4.15	4.15	3.9	4.01	44116	{}	\N	2020-10-12 15:34:08.273783
55742	LTBR	2020-10-02	3.9	4.16	3.8	4.07	23348	{}	\N	2020-10-12 15:34:08.280703
55743	LTBR	2020-10-05	4.2	4.27	3.8	4.06	183280	{}	\N	2020-10-12 15:34:08.287305
55744	LTBR	2020-10-06	3.91	4.89	3.7101	3.8561	3140038	{}	\N	2020-10-12 15:34:08.293275
55745	LTBR	2020-10-07	3.86	3.9507	3.3001	3.37	953454	{}	\N	2020-10-12 15:34:08.299436
55746	LTBR	2020-10-08	3.42	3.48	3.1	3.195	776750	{}	\N	2020-10-12 15:34:08.305915
55747	LTBR	2020-10-09	3.17	3.3	2.93	3.26	730855	{}	\N	2020-10-12 15:34:08.311576
55748	LTBR	2020-10-12	3.29	3.44	3.25	3.3901	233016	{}	\N	2020-10-12 15:34:08.318409
55749	LWAY	2020-09-28	5.24	5.51	5.15	5.38	82206	{}	\N	2020-10-12 15:34:10.756372
55750	LWAY	2020-09-29	5.38	5.5524	5.11	5.23	62884	{}	\N	2020-10-12 15:34:10.762671
55751	LWAY	2020-09-30	5.21	5.41	4.8469	5.03	107916	{}	\N	2020-10-12 15:34:10.769175
55752	LWAY	2020-10-01	5.18	5.3011	4.94	5.2	103116	{}	\N	2020-10-12 15:34:10.776775
55753	LWAY	2020-10-02	5.13	5.17	5.0157	5.05	19905	{}	\N	2020-10-12 15:34:10.782012
55754	LWAY	2020-10-05	5.02	5.14	4.65	4.75	179576	{}	\N	2020-10-12 15:34:10.787176
55755	LWAY	2020-10-06	4.81	4.98	4.75	4.82	89683	{}	\N	2020-10-12 15:34:10.793344
55756	LWAY	2020-10-07	4.97	5.2	4.8201	5.19	87954	{}	\N	2020-10-12 15:34:10.798233
55757	LWAY	2020-10-08	5.22	5.4	5.165	5.38	59666	{}	\N	2020-10-12 15:34:10.803449
55758	LWAY	2020-10-09	5.41	5.87	5.4	5.75	82975	{}	\N	2020-10-12 15:34:10.81076
55759	LWAY	2020-10-12	5.8	5.854	5.62	5.71	18742	{}	\N	2020-10-12 15:34:10.81614
55760	LYL	2020-09-28	0.95	0.95	0.9403	0.945	2769	{}	\N	2020-10-12 15:34:12.079836
55761	LYL	2020-09-29	0.9881	1.05	0.974	1	16234	{}	\N	2020-10-12 15:34:12.085261
55762	LYL	2020-09-30	1.035	1.035	1	1.02	6093	{}	\N	2020-10-12 15:34:12.091667
55763	LYL	2020-10-01	1.035	1.27	1.035	1.05	176749	{}	\N	2020-10-12 15:34:12.097632
55764	LYL	2020-10-02	1.08	1.15	1.01	1.01	8818	{}	\N	2020-10-12 15:34:12.1036
55765	LYL	2020-10-05	1.02	1.17	1.01	1.05	8326	{}	\N	2020-10-12 15:34:12.112626
55766	LYL	2020-10-06	1.0807	1.14	1	1.04	30391	{}	\N	2020-10-12 15:34:12.119115
55767	LYL	2020-10-07	1.1	1.18	1.0201	1.1	37400	{}	\N	2020-10-12 15:34:12.129377
55768	LYL	2020-10-08	1.1	1.15	1	1.09	31793	{}	\N	2020-10-12 15:34:12.136665
55769	LYL	2020-10-09	1.0727	1.08	1.07	1.0707	1853	{}	\N	2020-10-12 15:34:12.143863
55770	LYL	2020-10-12	1.06	1.08	1.05	1.0607	3348	{}	\N	2020-10-12 15:34:12.150301
55771	MACK	2020-09-28	3.86	3.9799	3.81	3.91	15896	{}	\N	2020-10-12 15:34:13.143603
55772	MACK	2020-09-29	3.9659	4.095	3.94	3.94	16932	{}	\N	2020-10-12 15:34:13.149917
55773	MACK	2020-09-30	4.06	4.06	3.92	4.01	11952	{}	\N	2020-10-12 15:34:13.155393
55774	MACK	2020-10-01	3.97	3.99	3.86	3.94	16971	{}	\N	2020-10-12 15:34:13.160846
55775	MACK	2020-10-02	3.84	4.04	3.7603	3.95	5391	{}	\N	2020-10-12 15:34:13.165923
55776	MACK	2020-10-05	4.05	4.05	3.91	3.95	16635	{}	\N	2020-10-12 15:34:13.170766
55777	MACK	2020-10-06	3.91	4.04	3.89	3.89	21902	{}	\N	2020-10-12 15:34:13.176143
55778	MACK	2020-10-07	3.9503	4.0236	3.91	3.95	21785	{}	\N	2020-10-12 15:34:13.181516
55779	MACK	2020-10-08	4	4.03	3.95	3.98	8067	{}	\N	2020-10-12 15:34:13.18619
55780	MACK	2020-10-09	4.03	4.07	3.97	3.97	8623	{}	\N	2020-10-12 15:34:13.191634
55781	MACK	2020-10-12	4.03	4.03	3.93	3.93	2341	{}	\N	2020-10-12 15:34:13.197866
55782	MANU	2020-09-28	14.96	15.06	14.76	14.76	90017	{}	\N	2020-10-12 15:34:14.149404
55783	MANU	2020-09-29	14.78	14.8	14.52	14.6	274936	{}	\N	2020-10-12 15:34:14.15492
55784	MANU	2020-09-30	14.62	14.76	14.5	14.54	227796	{}	\N	2020-10-12 15:34:14.161408
55785	MANU	2020-10-01	14.75	14.79	14.31	14.52	190903	{}	\N	2020-10-12 15:34:14.166361
55786	MANU	2020-10-02	14.34	14.77	14.28	14.57	312881	{}	\N	2020-10-12 15:34:14.171683
55787	MANU	2020-10-05	14.6	14.72	14.22	14.39	252771	{}	\N	2020-10-12 15:34:14.178444
55788	MANU	2020-10-06	14.36	14.45	14.15	14.22	256605	{}	\N	2020-10-12 15:34:14.185321
55789	MANU	2020-10-07	14.22	14.29	14.0788	14.15	219403	{}	\N	2020-10-12 15:34:14.191834
55790	MANU	2020-10-08	14.34	14.59	14.2	14.59	348987	{}	\N	2020-10-12 15:34:14.19946
55791	MANU	2020-10-09	14.6	14.7898	14.215	14.3	229825	{}	\N	2020-10-12 15:34:14.204824
55792	MANU	2020-10-12	14.3	14.5396	14.27	14.45	139848	{}	\N	2020-10-12 15:34:14.213243
55793	MAXR	2020-09-28	25	25.6378	23.95	25.41	1075728	{}	\N	2020-10-12 15:34:15.32214
55794	MAXR	2020-09-29	25.65	26.79	25.45	25.97	2257516	{}	\N	2020-10-12 15:34:15.328391
55795	MAXR	2020-09-30	25.8	26.02	24.73	24.94	2057545	{}	\N	2020-10-12 15:34:15.334195
55796	MAXR	2020-10-01	25.38	26.97	25.21	26.66	3029666	{}	\N	2020-10-12 15:34:15.340299
55797	MAXR	2020-10-02	25.3	26.36	25.1589	26.05	2468243	{}	\N	2020-10-12 15:34:15.347174
55798	MAXR	2020-10-05	26.24	26.83	25.87	26.36	1430978	{}	\N	2020-10-12 15:34:15.354957
55799	MAXR	2020-10-06	26.57	28.28	26.4	26.9	3566536	{}	\N	2020-10-12 15:34:15.36154
55800	MAXR	2020-10-07	27.7	28.96	27.55	28.69	2567185	{}	\N	2020-10-12 15:34:15.367067
55801	MAXR	2020-10-08	28.91	29.35	27.98	28.45	1694784	{}	\N	2020-10-12 15:34:15.37235
55802	MAXR	2020-10-09	28.6	29.83	28.325	28.78	2448956	{}	\N	2020-10-12 15:34:15.377743
55803	MAXR	2020-10-12	29.55	29.66	28.55	28.62	624308	{}	\N	2020-10-12 15:34:15.383524
55804	MBIN	2020-09-28	19.6	20.175	19.6	19.99	16286	{}	\N	2020-10-12 15:34:16.379016
55805	MBIN	2020-09-29	20.09	20.41	19.69	19.8	15180	{}	\N	2020-10-12 15:34:16.386603
55806	MBIN	2020-09-30	19.74	20.36	19.58	19.71	23897	{}	\N	2020-10-12 15:34:16.393213
55807	MBIN	2020-10-01	19.87	20.5402	19.52	19.97	45273	{}	\N	2020-10-12 15:34:16.401809
55808	MBIN	2020-10-02	19.71	20.269	19.61	20.07	48239	{}	\N	2020-10-12 15:34:16.414337
55809	MBIN	2020-10-05	20.3854	20.865	20.06	20.75	29880	{}	\N	2020-10-12 15:34:16.424557
55810	MBIN	2020-10-06	20.75	21.45	20.44	20.91	35398	{}	\N	2020-10-12 15:34:16.434622
55811	MBIN	2020-10-07	21.1	21.1	20.48	20.65	33809	{}	\N	2020-10-12 15:34:16.443471
55812	MBIN	2020-10-08	20.64	20.78	20.15	20.6	43294	{}	\N	2020-10-12 15:34:16.453585
55813	MBIN	2020-10-09	20.88	21.12	20.27	20.89	46334	{}	\N	2020-10-12 15:34:16.461037
55814	MBIN	2020-10-12	20.95	21.31	20.95	21.13	30040	{}	\N	2020-10-12 15:34:16.469259
55815	MCC	2020-09-28	17.42	18.49	17.42	18.19	4073	{}	\N	2020-10-12 15:34:17.379751
55816	MCC	2020-09-29	18.16	18.46	17.75	17.98	8998	{}	\N	2020-10-12 15:34:17.38703
55817	MCC	2020-09-30	17.82	18.32	17.82	17.83	6950	{}	\N	2020-10-12 15:34:17.392485
55818	MCC	2020-10-01	18.01	18.29	17.93	18.29	22124	{}	\N	2020-10-12 15:34:17.399276
55819	MCC	2020-10-02	18	18.25	17.5472	18.135	5208	{}	\N	2020-10-12 15:34:17.406544
55820	MCC	2020-10-05	18.08	18.85	17.7351	18.8	25990	{}	\N	2020-10-12 15:34:17.415358
55821	MCC	2020-10-06	18.92	18.92	18.2252	18.49	6370	{}	\N	2020-10-12 15:34:17.421255
55822	MCC	2020-10-07	18.61	18.65	18.3099	18.37	10386	{}	\N	2020-10-12 15:34:17.427023
55823	MCC	2020-10-08	18.6	18.6	18.37	18.37	5562	{}	\N	2020-10-12 15:34:17.433589
55824	MCC	2020-10-09	18.47	19.49	18.32	19.49	10128	{}	\N	2020-10-12 15:34:17.440134
55825	MCC	2020-10-12	22	22.37	21.39	21.39	22626	{}	\N	2020-10-12 15:34:17.446022
55826	MCK	2020-09-28	150.85	151.61	149.05	150.09	1157189	{}	\N	2020-10-12 15:34:18.368303
55827	MCK	2020-09-29	150.28	150.28	146.5	146.56	2950246	{}	\N	2020-10-12 15:34:18.374858
55828	MCK	2020-09-30	146.23	150.49	146.06	148.93	2711301	{}	\N	2020-10-12 15:34:18.380013
55829	MCK	2020-10-01	149.45	150.16	146.27	146.96	2644493	{}	\N	2020-10-12 15:34:18.38647
55830	MCK	2020-10-02	145.23	148.82	144.58	146.98	2617232	{}	\N	2020-10-12 15:34:18.393582
55831	MCK	2020-10-05	148	150.095	147.315	149.18	1909597	{}	\N	2020-10-12 15:34:18.400761
55832	MCK	2020-10-06	149.56	150.66	145.76	146.23	3056747	{}	\N	2020-10-12 15:34:18.410044
55833	MCK	2020-10-07	147.2	148.87	146.68	148.13	1714258	{}	\N	2020-10-12 15:34:18.420948
55834	MCK	2020-10-08	148.86	154.09	148.1301	153.99	2008641	{}	\N	2020-10-12 15:34:18.429235
55835	MCK	2020-10-09	154.46	154.56	150.82	151.29	2048992	{}	\N	2020-10-12 15:34:18.439767
55836	MCK	2020-10-12	151.92	152.45	150.46	151.52	317399	{}	\N	2020-10-12 15:34:18.455167
55837	MCS	2020-09-28	8.11	8.53	8.07	8.19	763513	{}	\N	2020-10-12 15:34:19.891268
55838	MCS	2020-09-29	8.1	8.18	7.66	7.83	1149202	{}	\N	2020-10-12 15:34:19.897475
55839	MCS	2020-09-30	7.89	8.17	7.7	7.73	1332341	{}	\N	2020-10-12 15:34:19.905001
55840	MCS	2020-10-01	7.77	7.91	7.42	7.78	1379979	{}	\N	2020-10-12 15:34:19.910606
55841	MCS	2020-10-02	7.44	8.28	7.42	8.06	1406761	{}	\N	2020-10-12 15:34:19.916728
55842	MCS	2020-10-05	8	8.0001	7.28	7.56	1572937	{}	\N	2020-10-12 15:34:19.924678
55843	MCS	2020-10-06	7.69	7.9	7.35	7.46	4571545	{}	\N	2020-10-12 15:34:19.931409
55844	MCS	2020-10-07	7.57	7.64	7.37	7.6	2120574	{}	\N	2020-10-12 15:34:19.940908
55845	MCS	2020-10-08	7.62	7.87	7.58	7.59	2467054	{}	\N	2020-10-12 15:34:19.948744
55846	MCS	2020-10-09	7.66	7.7417	7.44	7.64	1712965	{}	\N	2020-10-12 15:34:19.95753
55847	MCS	2020-10-12	7.69	7.705	7.3	7.47	840755	{}	\N	2020-10-12 15:34:19.965671
55848	MDGL	2020-09-28	118.88	122.16	117.67	121.6	159592	{}	\N	2020-10-12 15:34:22.062475
55849	MDGL	2020-09-29	121.55	121.94	118	119.22	140276	{}	\N	2020-10-12 15:34:22.070508
55850	MDGL	2020-09-30	118.94	120.18	113.76	118.73	131115	{}	\N	2020-10-12 15:34:22.076245
55851	MDGL	2020-10-01	119.72	120.05	116.78	118.19	103335	{}	\N	2020-10-12 15:34:22.081057
55852	MDGL	2020-10-02	119.9999	119.9999	115.36	116.52	106299	{}	\N	2020-10-12 15:34:22.086908
55853	MDGL	2020-10-05	118.07	119.85	116.5674	118.99	133618	{}	\N	2020-10-12 15:34:22.091716
55854	MDGL	2020-10-06	117.5	122.79	116.86	121.28	216930	{}	\N	2020-10-12 15:34:22.096416
55855	MDGL	2020-10-07	121.89	127.63	121.89	125.89	120524	{}	\N	2020-10-12 15:34:22.101298
55856	MDGL	2020-10-08	127.28	132.73	127.275	128.7	106437	{}	\N	2020-10-12 15:34:22.107856
55857	MDGL	2020-10-09	128.72	130.72	126.94	129.58	114479	{}	\N	2020-10-12 15:34:22.114181
55858	MDGL	2020-10-12	130.19	130.73	127.69	129.89	37743	{}	\N	2020-10-12 15:34:22.122083
55859	MELI	2020-09-28	1100.01	1107.86	1070	1093.43	460564	{}	\N	2020-10-12 15:34:24.822904
55860	MELI	2020-09-29	1095.43	1095.43	1068	1082.87	232489	{}	\N	2020-10-12 15:34:24.828219
55861	MELI	2020-09-30	1085	1098	1076.65	1081.89	296942	{}	\N	2020-10-12 15:34:24.833332
55862	MELI	2020-10-01	1097.99	1117.86	1095.91	1115.34	405766	{}	\N	2020-10-12 15:34:24.837885
55863	MELI	2020-10-02	1083.55	1105	1063.7	1079.19	355407	{}	\N	2020-10-12 15:34:24.843664
55864	MELI	2020-10-05	1085.78	1097.6125	1079.0834	1090.58	312974	{}	\N	2020-10-12 15:34:24.848451
55865	MELI	2020-10-06	1086.79	1116.65	1085.01	1091.39	378912	{}	\N	2020-10-12 15:34:24.853588
55866	MELI	2020-10-07	1105.7	1149	1094.86	1145	553247	{}	\N	2020-10-12 15:34:24.85933
55867	MELI	2020-10-08	1158.54	1173.43	1150.72	1159.62	421659	{}	\N	2020-10-12 15:34:24.86494
55868	MELI	2020-10-09	1165.13	1202.99	1163.94	1200.01	434415	{}	\N	2020-10-12 15:34:24.869439
55869	MELI	2020-10-12	1236.23	1236.23	1201.89	1222.02	161667	{}	\N	2020-10-12 15:34:24.874416
55870	LVHD	2020-09-28	29.44	29.7099	29.44	29.55	41876	{}	\N	2020-10-12 15:34:25.870905
55871	LVHD	2020-09-29	29.51	29.58	29.26	29.38	48147	{}	\N	2020-10-12 15:34:25.877324
55872	LVHD	2020-09-30	29.51	29.75	29.45	29.62	28408	{}	\N	2020-10-12 15:34:25.883358
55873	LVHD	2020-10-01	29.7	29.75	29.45	29.68	56738	{}	\N	2020-10-12 15:34:25.891369
55874	LVHD	2020-10-02	29.3	30.1	29.3	29.99	46212	{}	\N	2020-10-12 15:34:25.898331
55875	LVHD	2020-10-05	30.09	30.357	30.045	30.31	87438	{}	\N	2020-10-12 15:34:25.904158
55876	LVHD	2020-10-06	30.34	30.71	30.111	30.18	47747	{}	\N	2020-10-12 15:34:25.910458
55877	LVHD	2020-10-07	30.4	30.61	30.35	30.531	61449	{}	\N	2020-10-12 15:34:25.916508
55878	LVHD	2020-10-08	30.58	30.91	30.58	30.91	62685	{}	\N	2020-10-12 15:34:25.921099
55879	LVHD	2020-10-09	31.09	31.09	30.79	30.85	57325	{}	\N	2020-10-12 15:34:25.927268
55880	LVHD	2020-10-12	30.86	31.0483	30.85	31.04	7487	{}	\N	2020-10-12 15:34:25.934466
55881	LXU	2020-09-28	1.69	1.8	1.69	1.71	97519	{}	\N	2020-10-12 15:34:26.957423
55882	LXU	2020-09-29	1.73	1.73	1.62	1.63	284394	{}	\N	2020-10-12 15:34:26.96351
55883	LXU	2020-09-30	1.63	1.77	1.6	1.62	418399	{}	\N	2020-10-12 15:34:26.968389
55884	LXU	2020-10-01	1.63	1.71	1.56	1.6	516909	{}	\N	2020-10-12 15:34:26.973425
55885	LXU	2020-10-02	1.58	1.64	1.57	1.61	246692	{}	\N	2020-10-12 15:34:26.979661
55886	LXU	2020-10-05	1.61	1.7	1.61	1.62	191032	{}	\N	2020-10-12 15:34:26.985095
55887	LXU	2020-10-06	1.63	1.67	1.53	1.54	231339	{}	\N	2020-10-12 15:34:26.990055
55888	LXU	2020-10-07	1.55	1.66	1.55	1.65	172212	{}	\N	2020-10-12 15:34:26.996217
55889	LXU	2020-10-08	1.65	2.3	1.65	2.19	2691648	{}	\N	2020-10-12 15:34:27.001299
55890	LXU	2020-10-09	2.3	2.31	2.04	2.21	1424632	{}	\N	2020-10-12 15:34:27.005648
55891	LXU	2020-10-12	2.19	2.2	2.1	2.16	249290	{}	\N	2020-10-12 15:34:27.010918
55892	MARPS	2020-09-28	1.65	1.6699	1.6476	1.6476	2091	{}	\N	2020-10-12 15:34:28.462054
55893	MARPS	2020-09-29	1.85	1.93	1.69	1.88	79006	{}	\N	2020-10-12 15:34:28.468099
55894	MARPS	2020-09-30	1.88	1.9067	1.75	1.86	10755	{}	\N	2020-10-12 15:34:28.473134
55895	MARPS	2020-10-01	1.8201	1.8201	1.66	1.7771	34147	{}	\N	2020-10-12 15:34:28.479118
55896	MARPS	2020-10-02	1.7204	1.81	1.7	1.7	4182	{}	\N	2020-10-12 15:34:28.485616
55897	MARPS	2020-10-05	1.74	1.7645	1.7	1.7	4849	{}	\N	2020-10-12 15:34:28.491322
55898	MARPS	2020-10-06	1.6962	1.76	1.6962	1.7487	3188	{}	\N	2020-10-12 15:34:28.498441
55899	MARPS	2020-10-07	1.72	1.7793	1.65	1.7	5673	{}	\N	2020-10-12 15:34:28.503565
55900	MARPS	2020-10-08	1.61	1.85	1.61	1.78	82741	{}	\N	2020-10-12 15:34:28.509156
55901	MARPS	2020-10-09	1.77	5.65	1.7	3.75	7006357	{}	\N	2020-10-12 15:34:28.514546
55902	MARPS	2020-10-12	3.02	3.02	2.57	2.7314	158630	{}	\N	2020-10-12 15:34:28.519559
55903	MBSD	2020-09-28	24.1508	24.1699	24.145	24.15	7203	{}	\N	2020-10-12 15:34:30.52735
55904	MBSD	2020-09-29	24.1696	24.1696	24.14	24.14	4096	{}	\N	2020-10-12 15:34:30.533323
55905	MBSD	2020-09-30	24.1799	24.18	24.11	24.17	32032	{}	\N	2020-10-12 15:34:30.539433
55906	MBSD	2020-10-01	24.12	24.16	24.12	24.12	1581	{}	\N	2020-10-12 15:34:30.544117
55907	MBSD	2020-10-02	24.19	24.19	24.12	24.19	3605	{}	\N	2020-10-12 15:34:30.549489
55908	MBSD	2020-10-05	24.14	24.1415	24.13	24.1415	3709	{}	\N	2020-10-12 15:34:30.555365
55909	MBSD	2020-10-06	24.15	24.18	24.129	24.1497	5871	{}	\N	2020-10-12 15:34:30.560293
55910	MBSD	2020-10-07	24.1451	24.18	24.14	24.18	7725	{}	\N	2020-10-12 15:34:30.56619
55911	MBSD	2020-10-08	24.13	24.13	24.1	24.13	9647	{}	\N	2020-10-12 15:34:30.571765
55912	MBSD	2020-10-09	24.089	24.12	24.0601	24.1	82802	{}	\N	2020-10-12 15:34:30.576856
55913	MCB	2020-09-28	27	28.375	26.81	27.96	8542	{}	\N	2020-10-12 15:34:31.756846
55914	MCB	2020-09-29	27.725	28.23	27.43	28.23	24140	{}	\N	2020-10-12 15:34:31.763154
55915	MCB	2020-09-30	28.08	28.4	27.515	28	60972	{}	\N	2020-10-12 15:34:31.768521
55916	MCB	2020-10-01	28.01	28.49	27.9	28.27	140736	{}	\N	2020-10-12 15:34:31.774309
55917	MCB	2020-10-02	28.05	30.46	28.05	30.12	14580	{}	\N	2020-10-12 15:34:31.780241
55918	MCB	2020-10-05	30.005	31.21	29.63	30.9	21555	{}	\N	2020-10-12 15:34:31.786807
55919	MCB	2020-10-06	31.15	32.46	30.59	30.62	16114	{}	\N	2020-10-12 15:34:31.792706
55920	MCB	2020-10-07	31.2	31.73	31.15	31.5	20043	{}	\N	2020-10-12 15:34:31.798383
55921	MCB	2020-10-08	31.54	31.54	30.05	30.17	14599	{}	\N	2020-10-12 15:34:31.803346
55922	MCB	2020-10-09	30.42	31.12	30.23	30.85	17881	{}	\N	2020-10-12 15:34:31.808265
55923	MCB	2020-10-12	30.6316	30.6316	29.86	30.32	4348	{}	\N	2020-10-12 15:34:31.81351
55924	MCHP	2020-09-28	101.11	103.09	100.51	103.07	1941265	{}	\N	2020-10-12 15:34:33.015884
55925	MCHP	2020-09-29	102.74	104.04	102.01	102.74	1313416	{}	\N	2020-10-12 15:34:33.023874
55926	MCHP	2020-09-30	103.04	104.67	102.43	102.76	2221834	{}	\N	2020-10-12 15:34:33.030773
55927	MCHP	2020-10-01	106.08	108.01	105.01	107.05	2337524	{}	\N	2020-10-12 15:34:33.03673
55928	MCHP	2020-10-02	104.25	107.33	103.76	103.76	1536470	{}	\N	2020-10-12 15:34:33.043045
55929	MCHP	2020-10-05	104.74	107.41	104.74	107.31	1290197	{}	\N	2020-10-12 15:34:33.051743
55930	MCHP	2020-10-06	107.34	109.89	106.26	106.49	1166562	{}	\N	2020-10-12 15:34:33.065808
55931	MCHP	2020-10-07	108.38	109.18	107.73	108.79	1289544	{}	\N	2020-10-12 15:34:33.071579
55932	MCHP	2020-10-08	109.295	111.82	109.295	111.58	1751453	{}	\N	2020-10-12 15:34:33.076839
55933	MCHP	2020-10-09	114.24	116.56	113.22	113.98	1541448	{}	\N	2020-10-12 15:34:33.083807
55934	MCHP	2020-10-12	115.36	115.5	114.2	114.42	550315	{}	\N	2020-10-12 15:34:33.089638
55935	MCRO	2020-09-28	27.2846	27.2846	27.2846	27.2846	3	{}	\N	2020-10-12 15:34:34.185873
55936	MCRO	2020-09-29	27.3	27.3	27.3	27.3	654	{}	\N	2020-10-12 15:34:34.191717
55937	MCRO	2020-10-01	27.5	27.5	27.5	27.5	21	{}	\N	2020-10-12 15:34:34.196812
55938	MCRO	2020-10-02	27.35	27.3853	27.35	27.3853	200	{}	\N	2020-10-12 15:34:34.202882
55939	MCRO	2020-10-05	27.51	27.517	27.5043	27.517	1264	{}	\N	2020-10-12 15:34:34.208992
55940	MCRO	2020-10-07	27.62	27.62	27.62	27.62	125	{}	\N	2020-10-12 15:34:34.21518
55941	MCRO	2020-10-08	27.6696	27.6696	27.6696	27.6696	50	{}	\N	2020-10-12 15:34:34.220762
55942	MCRO	2020-10-09	27.7649	27.7649	27.7649	27.7649	94	{}	\N	2020-10-12 15:34:34.225757
55943	MDGS	2020-09-28	1.3	1.3401	1.27	1.29	124989	{}	\N	2020-10-12 15:34:35.110215
55944	MDGS	2020-09-29	1.27	1.35	1.27	1.33	186046	{}	\N	2020-10-12 15:34:35.120918
55945	MDGS	2020-09-30	1.3531	1.445	1.345	1.39	243178	{}	\N	2020-10-12 15:34:35.129852
55946	MDGS	2020-10-01	1.375	1.3884	1.34	1.34	12591	{}	\N	2020-10-12 15:34:35.135766
55947	MDGS	2020-10-02	1.37	1.37	1.29	1.29	71284	{}	\N	2020-10-12 15:34:35.140352
55948	MDGS	2020-10-05	1.28	1.38	1.28	1.31	61031	{}	\N	2020-10-12 15:34:35.145303
55949	MDGS	2020-10-06	1.4	1.4	1.32	1.34	370105	{}	\N	2020-10-12 15:34:35.149532
55950	MDGS	2020-10-07	1.41	1.5	1.399	1.49	401288	{}	\N	2020-10-12 15:34:35.153958
55951	MDGS	2020-10-08	1.54	1.59	1.5	1.52	166787	{}	\N	2020-10-12 15:34:35.158074
55952	MDGS	2020-10-09	2.05	2.28	1.76	1.92	30374822	{}	\N	2020-10-12 15:34:35.163472
55953	MDGS	2020-10-12	1.76	1.85	1.74	1.85	869406	{}	\N	2020-10-12 15:34:35.168497
55954	MDLZ	2020-09-28	56.96	57.37	56.5	57.01	5120193	{}	\N	2020-10-12 15:34:37.180109
55955	MDLZ	2020-09-29	57.07	57.09	56.3	56.48	6080073	{}	\N	2020-10-12 15:34:37.189773
55956	MDLZ	2020-09-30	56.59	57.73	56.42	57.45	5411985	{}	\N	2020-10-12 15:34:37.195793
55957	MDLZ	2020-10-01	57.63	57.96	57.16	57.91	5426217	{}	\N	2020-10-12 15:34:37.205899
55958	MDLZ	2020-10-02	57.44	57.78	56.565	56.86	5554593	{}	\N	2020-10-12 15:34:37.215932
55959	MDLZ	2020-10-05	56.95	57.14	56.7	57.08	4943859	{}	\N	2020-10-12 15:34:37.225854
55960	MDLZ	2020-10-06	57.07	57.2	55.92	56.09	4353213	{}	\N	2020-10-12 15:34:37.233485
55961	MDLZ	2020-10-07	56.15	56.42	55.6686	56.19	4591638	{}	\N	2020-10-12 15:34:37.240865
55962	MDLZ	2020-10-08	56.48	56.7465	56.23	56.51	3607273	{}	\N	2020-10-12 15:34:37.247898
55963	MDLZ	2020-10-09	56.92	57.48	56.675	57.3	4194181	{}	\N	2020-10-12 15:34:37.255936
55964	MDLZ	2020-10-12	57.63	58.82	57.62	58.645	2518951	{}	\N	2020-10-12 15:34:37.263042
55965	MDYV	2020-09-28	42.95	43.59	42.95	43.42	48462	{}	\N	2020-10-12 15:34:38.520625
55966	MDYV	2020-09-29	43.42	43.46	42.749	43.056	53113	{}	\N	2020-10-12 15:34:38.529781
55967	MDYV	2020-09-30	43.17	43.85	43.1359	43.39	56481	{}	\N	2020-10-12 15:34:38.544338
55968	MDYV	2020-10-01	43.5	43.915	43.3017	43.9	46076	{}	\N	2020-10-12 15:34:38.555953
55969	MDYV	2020-10-02	43.1756	44.78	43.1756	44.62	68868	{}	\N	2020-10-12 15:34:38.565016
55970	MDYV	2020-10-05	45	45.71	45	45.67	34988	{}	\N	2020-10-12 15:34:38.575984
55971	MDYV	2020-10-06	45.96	46.7361	45.49	45.51	60968	{}	\N	2020-10-12 15:34:38.58457
55972	MDYV	2020-10-07	46.02	46.4801	45.9001	46.3803	89378	{}	\N	2020-10-12 15:34:38.598856
55973	MDYV	2020-10-08	46.7	46.86	46.5	46.86	48059	{}	\N	2020-10-12 15:34:38.608107
55974	MDYV	2020-10-09	47.24	47.368	46.72	46.79	68115	{}	\N	2020-10-12 15:34:38.616093
55975	MDYV	2020-10-12	46.94	47.2	46.86	47.19	45224	{}	\N	2020-10-12 15:34:38.623779
55976	MEN	2020-09-28	11.25	11.31	11.23	11.23	21926	{}	\N	2020-10-12 15:34:39.531227
55977	MEN	2020-09-29	11.23	11.33	11.23	11.32	45836	{}	\N	2020-10-12 15:34:39.539771
55978	MEN	2020-09-30	11.3	11.33	11.28	11.3	104282	{}	\N	2020-10-12 15:34:39.5503
55979	MEN	2020-10-01	11.34	11.35	11.27	11.29	85102	{}	\N	2020-10-12 15:34:39.560489
55980	MEN	2020-10-02	11.36	11.43	11.27	11.4	128254	{}	\N	2020-10-12 15:34:39.57225
55981	MEN	2020-10-05	11.4	11.43	11.3	11.4	147757	{}	\N	2020-10-12 15:34:39.581215
55982	MEN	2020-10-06	11.33	11.455	11.31	11.41	85287	{}	\N	2020-10-12 15:34:39.593986
55983	MEN	2020-10-07	11.5	11.51	11.36	11.36	136370	{}	\N	2020-10-12 15:34:39.606161
55984	MEN	2020-10-08	11.42	11.42	11.33	11.42	128544	{}	\N	2020-10-12 15:34:39.615304
55985	MEN	2020-10-09	11.39	11.48	11.38	11.43	78216	{}	\N	2020-10-12 15:34:39.624172
55986	MEN	2020-10-12	11.52	11.65	11.46	11.47	46362	{}	\N	2020-10-12 15:34:39.632822
55987	LXFR	2020-09-28	12.46	12.89	12.42	12.8	111881	{}	\N	2020-10-12 15:34:40.600491
55988	LXFR	2020-09-29	12.75	13.11	12.695	13.07	311842	{}	\N	2020-10-12 15:34:40.610206
55989	LXFR	2020-09-30	13.06	13.2	12.42	12.55	178956	{}	\N	2020-10-12 15:34:40.618798
55990	LXFR	2020-10-01	12.65	12.745	12.24	12.59	167132	{}	\N	2020-10-12 15:34:40.628517
55991	LXFR	2020-10-02	12.39	12.95	12.31	12.78	78119	{}	\N	2020-10-12 15:34:40.637229
55992	LXFR	2020-10-05	12.78	13.05	12.78	13.03	51365	{}	\N	2020-10-12 15:34:40.645909
55993	LXFR	2020-10-06	13.22	13.3	12.94	13.04	176149	{}	\N	2020-10-12 15:34:40.653087
55994	LXFR	2020-10-07	13.22	13.5	13.06	13.15	227201	{}	\N	2020-10-12 15:34:40.659587
55995	LXFR	2020-10-08	13.31	13.405	13.16	13.32	144083	{}	\N	2020-10-12 15:34:40.665707
55996	LXFR	2020-10-09	13.52	13.79	13.28	13.73	216600	{}	\N	2020-10-12 15:34:40.673305
55997	LXFR	2020-10-12	13.82	13.82	13.61	13.61	9317	{}	\N	2020-10-12 15:34:40.679758
55998	M	2020-09-28	6.12	6.23	5.95	5.96	22299219	{}	\N	2020-10-12 15:34:41.62095
55999	M	2020-09-29	5.98	5.99	5.6	5.67	58452791	{}	\N	2020-10-12 15:34:41.626683
56000	M	2020-09-30	5.7	5.91	5.69	5.7	44425548	{}	\N	2020-10-12 15:34:41.631675
56001	M	2020-10-01	5.73	5.985	5.69	5.84	42671879	{}	\N	2020-10-12 15:34:41.639741
56002	M	2020-10-02	5.61	6.0382	5.57	6.01	43826445	{}	\N	2020-10-12 15:34:41.647066
56003	M	2020-10-05	6.07	6.2	6	6.06	29286739	{}	\N	2020-10-12 15:34:41.654909
56004	M	2020-10-06	6.14	6.18	5.77	5.81	44669402	{}	\N	2020-10-12 15:34:41.662541
56005	M	2020-10-07	5.93	6.09	5.86	5.96	35928569	{}	\N	2020-10-12 15:34:41.66982
56006	M	2020-10-08	6.02	6.32	5.98	6.29	42680861	{}	\N	2020-10-12 15:34:41.677718
56007	M	2020-10-09	6.4	6.45	6.14	6.23	38171911	{}	\N	2020-10-12 15:34:41.684581
56008	M	2020-10-12	6.33	6.6	6.278	6.37	25487128	{}	\N	2020-10-12 15:34:41.691344
56009	MAN	2020-09-28	72.57	73.54	72.3	72.63	710293	{}	\N	2020-10-12 15:34:45.876683
56010	MAN	2020-09-29	72.73	73.06	71.76	72.4	867262	{}	\N	2020-10-12 15:34:45.883037
56011	MAN	2020-09-30	72.6	74.26	72.54	73.33	992725	{}	\N	2020-10-12 15:34:45.887526
56012	MAN	2020-10-01	73.77	74.04	71.98	72.57	709023	{}	\N	2020-10-12 15:34:45.895306
56013	MAN	2020-10-02	71.71	75.38	71.23	75.09	844227	{}	\N	2020-10-12 15:34:45.901079
56014	MAN	2020-10-05	75.82	76.86	74.94	75.69	665480	{}	\N	2020-10-12 15:34:45.906812
56015	MAN	2020-10-06	76.17	76.74	74.08	74.29	761912	{}	\N	2020-10-12 15:34:45.912796
56016	MAN	2020-10-07	74.98	75.74	74.68	74.85	825552	{}	\N	2020-10-12 15:34:45.918063
56017	MAN	2020-10-08	75.45	75.98	74.12	75.29	614028	{}	\N	2020-10-12 15:34:45.922757
56018	MAN	2020-10-09	76.03	76.23	74.7	74.8	345594	{}	\N	2020-10-12 15:34:45.92722
56019	MAN	2020-10-12	74.97	76.52	74.83	76.36	81936	{}	\N	2020-10-12 15:34:45.932458
56020	MATW	2020-09-28	21.88	22.75	21.88	22.42	129960	{}	\N	2020-10-12 15:34:49.224012
56021	MATW	2020-09-29	22.29	22.4	21.7	22.14	131168	{}	\N	2020-10-12 15:34:49.232534
56022	MATW	2020-09-30	22.24	22.62	22.11	22.36	210992	{}	\N	2020-10-12 15:34:49.23822
56023	MATW	2020-10-01	22.2	22.7	22.06	22.42	89391	{}	\N	2020-10-12 15:34:49.244256
56024	MATW	2020-10-02	22.04	23.27	21.6832	23.21	124007	{}	\N	2020-10-12 15:34:49.251248
56025	MATW	2020-10-05	23.58	23.85	23.5	23.7	111941	{}	\N	2020-10-12 15:34:49.25642
56026	MATW	2020-10-06	23.94	24.37	23.44	23.71	347982	{}	\N	2020-10-12 15:34:49.261397
56027	MATW	2020-10-07	23.85	24.16	23.36	23.89	176116	{}	\N	2020-10-12 15:34:49.267366
56028	MATW	2020-10-08	24.08	24.59	23.91	24.57	146739	{}	\N	2020-10-12 15:34:49.273068
56029	MATW	2020-10-09	24.78	24.89	24.44	24.6	116715	{}	\N	2020-10-12 15:34:49.27821
56030	MATW	2020-10-12	24.53	24.55	24.355	24.48	11641	{}	\N	2020-10-12 15:34:49.284683
56031	MCHI	2020-09-28	72.44	72.58	72.05	72.49	2500940	{}	\N	2020-10-12 15:34:50.999731
56032	MCHI	2020-09-29	72.15	72.6	72.15	72.4	2231797	{}	\N	2020-10-12 15:34:51.006963
56033	MCHI	2020-09-30	73.24	74.38	73.18	74.17	2900556	{}	\N	2020-10-12 15:34:51.012262
56034	MCHI	2020-10-01	74.91	75.025	74.38	74.885	2582830	{}	\N	2020-10-12 15:34:51.020544
56035	MCHI	2020-10-02	73.59	74.75	73.55	73.99	2470716	{}	\N	2020-10-12 15:34:51.02865
56036	MCHI	2020-10-05	74.17	74.41	73.925	74.39	1601970	{}	\N	2020-10-12 15:34:51.040097
56037	MCHI	2020-10-06	74.82	75.67	74.67	75.04	2992230	{}	\N	2020-10-12 15:34:51.046181
56038	MCHI	2020-10-07	75.8	76.09	75.4	75.88	3593638	{}	\N	2020-10-12 15:34:51.053859
56039	MCHI	2020-10-08	76.05	76.45	75.85	76.36	3954370	{}	\N	2020-10-12 15:34:51.059906
56040	MCHI	2020-10-09	76.34	76.915	76.18	76.77	2576041	{}	\N	2020-10-12 15:34:51.065979
56041	MCHI	2020-10-12	77.95	78.3999	77.73	78.29	1197012	{}	\N	2020-10-12 15:34:51.072198
56042	MCV	2020-09-28	22.93	22.93	22.93	22.93	274	{}	\N	2020-10-12 15:34:52.050031
56043	MCV	2020-09-29	22.75	22.98	22.53	22.98	1848	{}	\N	2020-10-12 15:34:52.061253
56044	MCV	2020-09-30	23.06	23.36	22.21	23.27	3768	{}	\N	2020-10-12 15:34:52.06725
56045	MCV	2020-10-01	23	23	22.52	22.69	5858	{}	\N	2020-10-12 15:34:52.074713
56046	MCV	2020-10-02	22.65	22.8361	22.65	22.8361	1000	{}	\N	2020-10-12 15:34:52.079759
56047	MCV	2020-10-05	22.625	22.625	22.1	22.25	11842	{}	\N	2020-10-12 15:34:52.086027
56048	MCV	2020-10-06	22	22	21.14	21.82	60200	{}	\N	2020-10-12 15:34:52.092256
56049	MCV	2020-10-07	21.87	22.4	21.84	22.32	6298	{}	\N	2020-10-12 15:34:52.097164
56050	MCV	2020-10-08	22.25	22.99	21.91	22.99	6502	{}	\N	2020-10-12 15:34:52.105986
56051	MCV	2020-10-09	22.11	23.0688	21.87	22.88	5323	{}	\N	2020-10-12 15:34:52.11349
56052	MCV	2020-10-12	23.51	23.5324	23.5	23.5324	4744	{}	\N	2020-10-12 15:34:52.122729
56053	MDYG	2020-09-28	56.75	57.3428	56.75	57.22	47249	{}	\N	2020-10-12 15:34:53.370012
56054	MDYG	2020-09-29	57.2502	57.575	56.83	57.06	84604	{}	\N	2020-10-12 15:34:53.376909
56055	MDYG	2020-09-30	57.2	57.92	57.08	57.39	61306	{}	\N	2020-10-12 15:34:53.382197
56056	MDYG	2020-10-01	57.7	58.16	57.5382	58.16	267233	{}	\N	2020-10-12 15:34:53.388724
56057	MDYG	2020-10-02	57.04	58.6693	57.04	58.44	73507	{}	\N	2020-10-12 15:34:53.395965
56058	MDYG	2020-10-05	58.89	59.69	58.89	59.64	100399	{}	\N	2020-10-12 15:34:53.40333
56059	MDYG	2020-10-06	59.94	60.615	59.24	59.39	233263	{}	\N	2020-10-12 15:34:53.410387
56060	MDYG	2020-10-07	60.02	60.57	60.02	60.45	242263	{}	\N	2020-10-12 15:34:53.415534
56061	MDYG	2020-10-08	60.98	61	60.68	61	105780	{}	\N	2020-10-12 15:34:53.421227
56062	MDYG	2020-10-09	61.42	61.5538	61.13	61.38	108460	{}	\N	2020-10-12 15:34:53.427015
56063	MDYG	2020-10-12	61.71	61.73	61.38	61.7032	18380	{}	\N	2020-10-12 15:34:53.433094
56064	MEOH	2020-09-28	22.66	24.31	22.66	23.77	439778	{}	\N	2020-10-12 15:34:54.445197
56065	MEOH	2020-09-29	24.03	24.44	23.33	23.87	337651	{}	\N	2020-10-12 15:34:54.452251
56066	MEOH	2020-09-30	24.09	25.16	24.02	24.4	545586	{}	\N	2020-10-12 15:34:54.462939
56067	MEOH	2020-10-01	24.56	24.92	23.54	23.78	326871	{}	\N	2020-10-12 15:34:54.472076
56068	MEOH	2020-10-02	23.08	23.8358	22.9108	23.72	254217	{}	\N	2020-10-12 15:34:54.479521
56069	MEOH	2020-10-05	24.06	25.1	24.02	25.09	272374	{}	\N	2020-10-12 15:34:54.488623
56070	MEOH	2020-10-06	25.5	25.805	25.06	25.31	254602	{}	\N	2020-10-12 15:34:54.496964
56071	MEOH	2020-10-07	25.78	27.15	25.7	26.82	383581	{}	\N	2020-10-12 15:34:54.505576
56072	MEOH	2020-10-08	27.4	28.29	27.04	28.29	302206	{}	\N	2020-10-12 15:34:54.512868
56073	MEOH	2020-10-09	28.63	28.76	27.2	27.51	195044	{}	\N	2020-10-12 15:34:54.520582
56074	MEOH	2020-10-12	27.51	27.92	27.145	27.31	32059	{}	\N	2020-10-12 15:34:54.526945
56075	MFD	2020-09-28	7.71	7.71	7.6068	7.64	31597	{}	\N	2020-10-12 15:34:59.078214
56076	MFD	2020-09-29	7.6676	7.69	7.6	7.69	42674	{}	\N	2020-10-12 15:34:59.085227
56077	MFD	2020-09-30	7.77	7.77	7.645	7.68	71850	{}	\N	2020-10-12 15:34:59.092021
56078	MFD	2020-10-01	7.61	7.655	7.53	7.625	38698	{}	\N	2020-10-12 15:34:59.099757
56079	MFD	2020-10-02	7.5	7.74	7.5	7.62	36440	{}	\N	2020-10-12 15:34:59.106995
56080	MFD	2020-10-05	7.75	7.75	7.65	7.73	49928	{}	\N	2020-10-12 15:34:59.115713
56081	MFD	2020-10-06	7.69	7.86	7.69	7.7	76020	{}	\N	2020-10-12 15:34:59.121795
56082	MFD	2020-10-07	7.74	7.76	7.69	7.76	34120	{}	\N	2020-10-12 15:34:59.12768
56083	MFD	2020-10-08	7.77	7.85	7.769	7.82	35242	{}	\N	2020-10-12 15:34:59.132235
56084	MFD	2020-10-09	7.87	7.91	7.67	7.8	27502	{}	\N	2020-10-12 15:34:59.143072
56085	MFD	2020-10-12	7.8	7.8248	7.8	7.824	14472	{}	\N	2020-10-12 15:34:59.151383
56086	MFO	2020-09-28	23.95	24.25	23.8616	24.23	14532	{}	\N	2020-10-12 15:35:00.134811
56087	MFO	2020-09-29	24.035	24.09	24	24.0215	23924	{}	\N	2020-10-12 15:35:00.141214
56088	MFO	2020-09-30	24.0199	24.02	23.9391	24	22806	{}	\N	2020-10-12 15:35:00.148258
56089	MFO	2020-10-01	24	24.07	23.9	24.07	30756	{}	\N	2020-10-12 15:35:00.153846
56090	MFO	2020-10-02	23.97	23.97	23.9	23.92	36904	{}	\N	2020-10-12 15:35:00.160248
56091	MFO	2020-10-05	24.03	24.03	23.91	23.95	14138	{}	\N	2020-10-12 15:35:00.168428
56092	MFO	2020-10-06	24	24.08	23.91	23.91	66500	{}	\N	2020-10-12 15:35:00.176232
56093	MFO	2020-10-07	23.9683	24.175	23.93	24.16	14414	{}	\N	2020-10-12 15:35:00.183479
56094	MFO	2020-10-08	24.2063	24.33	24.165	24.33	28068	{}	\N	2020-10-12 15:35:00.189791
56095	MFO	2020-10-09	24.288	24.4	24.13	24.29	20960	{}	\N	2020-10-12 15:35:00.196452
56096	MFO	2020-10-12	24.29	24.35	24.29	24.32	12526	{}	\N	2020-10-12 15:35:00.201567
56097	LVS	2020-09-28	47.57	48.129	47.2	47.62	2931524	{}	\N	2020-10-12 15:35:02.916569
56098	LVS	2020-09-29	47.57	47.69	46.33	46.72	4872032	{}	\N	2020-10-12 15:35:02.922283
56099	LVS	2020-09-30	47.11	47.67	46.31	46.66	5974546	{}	\N	2020-10-12 15:35:02.926985
56100	LVS	2020-10-01	46.46	46.93	45.92	46.22	6122910	{}	\N	2020-10-12 15:35:02.933067
56101	LVS	2020-10-02	45.19	46.6048	44.77	46.15	8008567	{}	\N	2020-10-12 15:35:02.938574
56102	LVS	2020-10-05	46.15	46.28	44.95	46.22	14187170	{}	\N	2020-10-12 15:35:02.943641
56103	LVS	2020-10-06	46.64	48.05	46.29	46.42	9050250	{}	\N	2020-10-12 15:35:02.952012
56104	LVS	2020-10-07	46.96	47.1	46.11	46.18	7046461	{}	\N	2020-10-12 15:35:02.95787
56105	LVS	2020-10-08	46.4	46.66	45.93	46.11	8615874	{}	\N	2020-10-12 15:35:02.962806
56106	LVS	2020-10-09	46.18	46.3719	45.1	45.72	14873234	{}	\N	2020-10-12 15:35:02.968922
56107	LVS	2020-10-12	45.67	46.49	45.61	46.16	3200918	{}	\N	2020-10-12 15:35:02.973176
56108	LYB	2020-09-28	71.19	72.76	70.97	71.21	2025818	{}	\N	2020-10-12 15:35:04.408185
56109	LYB	2020-09-29	71.01	71.55	70.03	70.28	2495922	{}	\N	2020-10-12 15:35:04.415573
56110	LYB	2020-09-30	71.03	72.34	70.32	70.49	4911883	{}	\N	2020-10-12 15:35:04.421217
56111	LYB	2020-10-01	70.34	71.15	67.82	68.14	4118010	{}	\N	2020-10-12 15:35:04.425788
56112	LYB	2020-10-02	67.02	73.04	66.75	72.24	5915074	{}	\N	2020-10-12 15:35:04.430885
56113	LYB	2020-10-05	73.25	76.13	72.77	75.4	5429058	{}	\N	2020-10-12 15:35:04.436866
56114	LYB	2020-10-06	76.55	77.98	75.17	75.99	5690611	{}	\N	2020-10-12 15:35:04.44184
56115	LYB	2020-10-07	77.2	79.59	77.2	79.41	5504272	{}	\N	2020-10-12 15:35:04.446917
56116	LYB	2020-10-08	79.9	80.34	77.73	79.26	3653986	{}	\N	2020-10-12 15:35:04.45405
56117	LYB	2020-10-09	80.29	81.21	79.02	79.84	3218362	{}	\N	2020-10-12 15:35:04.460279
56118	LYB	2020-10-12	79.65	79.68	77.48	77.9747	1385619	{}	\N	2020-10-12 15:35:04.467211
56119	MAA	2020-09-28	116.83	118.17	116.15	117.14	416626	{}	\N	2020-10-12 15:35:06.156438
56120	MAA	2020-09-29	116.92	117.43	114.64	115.34	559827	{}	\N	2020-10-12 15:35:06.162446
56121	MAA	2020-09-30	116.07	116.63	114.44	115.95	1505987	{}	\N	2020-10-12 15:35:06.16775
56122	MAA	2020-10-01	116	120.33	115.57	120.29	1439272	{}	\N	2020-10-12 15:35:06.17546
56123	MAA	2020-10-02	118.81	124.33	117.42	123.71	1695565	{}	\N	2020-10-12 15:35:06.181568
56124	MAA	2020-10-05	124.56	124.88	122.4845	124.5	1133673	{}	\N	2020-10-12 15:35:06.188531
56125	MAA	2020-10-06	124.87	125.24	122.83	123.54	1212049	{}	\N	2020-10-12 15:35:06.193107
56126	MAA	2020-10-07	124.04	125.03	121.7736	122.95	995436	{}	\N	2020-10-12 15:35:06.198083
56127	MAA	2020-10-08	123.03	125.35	123.03	124.88	954924	{}	\N	2020-10-12 15:35:06.206974
56128	MAA	2020-10-09	125.64	125.64	123.01	123.66	701105	{}	\N	2020-10-12 15:35:06.212728
56129	MAA	2020-10-12	123.83	124.3	123.06	124.14	102338	{}	\N	2020-10-12 15:35:06.217215
56130	MARA	2020-09-28	2.05	2.15	1.98	2.02	3876429	{}	\N	2020-10-12 15:35:09.859215
56131	MARA	2020-09-29	1.98	2.08	1.95	2.02	2391747	{}	\N	2020-10-12 15:35:09.872371
56132	MARA	2020-09-30	2	2.0488	1.91	1.96	2016502	{}	\N	2020-10-12 15:35:09.88804
56133	MARA	2020-10-01	2.01	2.03	1.9	1.95	2494810	{}	\N	2020-10-12 15:35:09.896713
56134	MARA	2020-10-02	1.85	1.96	1.82	1.96	1832670	{}	\N	2020-10-12 15:35:09.904791
56135	MARA	2020-10-05	1.96	2.01	1.89	1.96	2893540	{}	\N	2020-10-12 15:35:09.915846
56136	MARA	2020-10-06	1.93	1.95	1.85	1.87	2214771	{}	\N	2020-10-12 15:35:09.92385
56137	MARA	2020-10-07	1.87	1.94	1.87	1.94	1731593	{}	\N	2020-10-12 15:35:09.931162
56138	MARA	2020-10-08	1.98	2.37	1.97	2.14	19093002	{}	\N	2020-10-12 15:35:09.938696
56139	MARA	2020-10-09	2.24	2.27	2.06	2.16	6088129	{}	\N	2020-10-12 15:35:09.945912
56140	MARA	2020-10-12	2.23	2.45	2.15	2.405	6943411	{}	\N	2020-10-12 15:35:09.952265
56141	MAT	2020-09-28	11.42	11.53	11.23	11.4	3428683	{}	\N	2020-10-12 15:35:11.258229
56142	MAT	2020-09-29	11.55	11.915	11.46	11.5	2982982	{}	\N	2020-10-12 15:35:11.263329
56143	MAT	2020-09-30	11.58	12.03	11.58	11.7	3559892	{}	\N	2020-10-12 15:35:11.268399
56144	MAT	2020-10-01	11.73	11.975	11.69	11.92	2173362	{}	\N	2020-10-12 15:35:11.272963
56145	MAT	2020-10-02	11.71	12.11	11.7	12.075	11635994	{}	\N	2020-10-12 15:35:11.278779
56146	MAT	2020-10-05	12.3	12.462	12.085	12.24	13389216	{}	\N	2020-10-12 15:35:11.285009
56147	MAT	2020-10-06	12.31	12.62	12.25	12.25	4826055	{}	\N	2020-10-12 15:35:11.29179
56148	MAT	2020-10-07	12.38	12.61	12.29	12.35	3214647	{}	\N	2020-10-12 15:35:11.297529
56149	MAT	2020-10-08	12.53	12.69	12.33	12.6	4632452	{}	\N	2020-10-12 15:35:11.302703
56150	MAT	2020-10-09	12.76	12.83	12.49	12.49	4876745	{}	\N	2020-10-12 15:35:11.30864
56151	MAT	2020-10-12	12.63	12.65	12.4	12.51	1091554	{}	\N	2020-10-12 15:35:11.314402
56152	MBUU	2020-09-28	48.53	49.6175	46.62	48.33	358636	{}	\N	2020-10-12 15:35:19.261574
56153	MBUU	2020-09-29	48.28	48.9	47.56	48.03	134115	{}	\N	2020-10-12 15:35:19.26838
56154	MBUU	2020-09-30	48.01	50.8973	47.8	49.56	730117	{}	\N	2020-10-12 15:35:19.274041
56155	MBUU	2020-10-01	49.0004	51.75	48.27	51.66	218327	{}	\N	2020-10-12 15:35:19.279392
56156	MBUU	2020-10-02	50.39	54.3	50.39	53.62	172374	{}	\N	2020-10-12 15:35:19.28483
56157	MBUU	2020-10-05	54	56.12	53.58	56	142852	{}	\N	2020-10-12 15:35:19.29171
56158	MBUU	2020-10-06	56.55	58.2	55.26	56.01	153244	{}	\N	2020-10-12 15:35:19.298173
56159	MBUU	2020-10-07	57.08	58.23	56.55	56.7	163186	{}	\N	2020-10-12 15:35:19.305905
56160	MBUU	2020-10-08	57.4	57.4	54.89	55.61	124758	{}	\N	2020-10-12 15:35:19.311916
56161	MBUU	2020-10-09	56.33	57.04	55.71	56.65	97028	{}	\N	2020-10-12 15:35:19.318664
56162	MBUU	2020-10-12	56.99	56.99	55.47	55.575	17419	{}	\N	2020-10-12 15:35:19.325092
56163	MCD	2020-09-28	220	221.98	219.02	220.26	2220422	{}	\N	2020-10-12 15:35:20.472242
56164	MCD	2020-09-29	220.42	220.955	218.4783	218.68	3513058	{}	\N	2020-10-12 15:35:20.479603
56165	MCD	2020-09-30	218.87	222.2	218.41	219.49	6013395	{}	\N	2020-10-12 15:35:20.485753
56166	MCD	2020-10-01	221.59	223	217.79	219.59	4707118	{}	\N	2020-10-12 15:35:20.49194
56167	MCD	2020-10-02	217.51	223.3499	217.1	222.67	7633475	{}	\N	2020-10-12 15:35:20.498258
56168	MCD	2020-10-05	224.86	226.5	224.01	226.07	5793168	{}	\N	2020-10-12 15:35:20.506367
56169	MCD	2020-10-06	226.83	228.64	223.6	224.09	4905843	{}	\N	2020-10-12 15:35:20.512798
56170	MCD	2020-10-07	225.58	228.41	225.125	226.48	3986701	{}	\N	2020-10-12 15:35:20.518865
56171	MCD	2020-10-08	226.09	227.08	224.29	225.8	6450270	{}	\N	2020-10-12 15:35:20.524824
56172	MCD	2020-10-09	226.94	227.22	223.12	224.83	6029908	{}	\N	2020-10-12 15:35:20.529963
56173	MCD	2020-10-12	224.81	225.49	223.12	225.35	1604275	{}	\N	2020-10-12 15:35:20.536654
56174	MCN	2020-09-28	5.81	5.87	5.794	5.82	122622	{}	\N	2020-10-12 15:35:21.937082
56175	MCN	2020-09-29	5.82	5.88	5.8001	5.88	172506	{}	\N	2020-10-12 15:35:21.94259
56176	MCN	2020-09-30	5.91	5.98	5.9	5.98	301812	{}	\N	2020-10-12 15:35:21.949901
56177	MCN	2020-10-01	5.96	6	5.93	5.94	62002	{}	\N	2020-10-12 15:35:21.960602
56178	MCN	2020-10-02	5.89	5.95	5.84	5.95	97938	{}	\N	2020-10-12 15:35:21.972193
56179	MCN	2020-10-05	5.95	6	5.95	5.95	71488	{}	\N	2020-10-12 15:35:21.980261
56180	MCN	2020-10-06	6	6.02	5.95	5.97	150874	{}	\N	2020-10-12 15:35:21.989357
56181	MCN	2020-10-07	6	6.06	5.97	6.03	131710	{}	\N	2020-10-12 15:35:21.995301
56182	MCN	2020-10-08	6.06	6.07	6.039	6.05	84474	{}	\N	2020-10-12 15:35:22.002248
56183	MCN	2020-10-09	6.08	6.1	6.05	6.08	70938	{}	\N	2020-10-12 15:35:22.009595
56184	MCN	2020-10-12	6.1	6.11	6.061	6.07	41212	{}	\N	2020-10-12 15:35:22.016059
56185	MET	2020-09-28	36.87	37.75	36.87	37.25	3886736	{}	\N	2020-10-12 15:35:27.60864
56186	MET	2020-09-29	37.11	37.32	36.46	36.77	7620486	{}	\N	2020-10-12 15:35:27.615937
56187	MET	2020-09-30	36.97	37.5377	36.74	37.17	11787425	{}	\N	2020-10-12 15:35:27.621054
56188	MET	2020-10-01	37.24	37.67	36.83	37.26	7598337	{}	\N	2020-10-12 15:35:27.627392
56189	MET	2020-10-02	36.65	37.95	36.57	37.63	9063753	{}	\N	2020-10-12 15:35:27.636558
56190	MET	2020-10-05	38.14	38.92	37.71	38.52	7969138	{}	\N	2020-10-12 15:35:27.64613
56191	MET	2020-10-06	39.01	39.75	38.26	38.37	11254781	{}	\N	2020-10-12 15:35:27.652562
56192	MET	2020-10-07	39.01	39.78	38.91	39.34	13399717	{}	\N	2020-10-12 15:35:27.657599
56193	MET	2020-10-08	39.49	40.07	39.415	39.85	9340431	{}	\N	2020-10-12 15:35:27.664577
56194	MET	2020-10-09	40.08	40.29	39.42	39.48	6868850	{}	\N	2020-10-12 15:35:27.670362
56195	MET	2020-10-12	39.63	40.29	39.52	40.24	2520747	{}	\N	2020-10-12 15:35:27.67744
56196	MFEM	2020-09-28	22.37	22.37	22.17	22.17	4911	{}	\N	2020-10-12 15:35:28.711758
56197	MFEM	2020-09-29	22.23	22.31	22.18	22.26	11072	{}	\N	2020-10-12 15:35:28.717962
56198	MFEM	2020-09-30	22.48	22.5735	22.43	22.45	6925	{}	\N	2020-10-12 15:35:28.72455
56199	MFEM	2020-10-01	22.58	22.62	22.48	22.5613	14531	{}	\N	2020-10-12 15:35:28.734846
56200	MFEM	2020-10-02	22.4695	22.4695	22.3224	22.3224	2183	{}	\N	2020-10-12 15:35:28.742785
56201	MFEM	2020-10-05	22.5	22.76	22.5	22.76	83039	{}	\N	2020-10-12 15:35:28.751604
56202	MFEM	2020-10-06	22.75	22.81	22.535	22.57	16759	{}	\N	2020-10-12 15:35:28.75762
56203	MFEM	2020-10-07	22.79	22.89	22.79	22.87	11662	{}	\N	2020-10-12 15:35:28.766546
56204	MFEM	2020-10-08	22.96	23.0541	22.95	23.0541	4642	{}	\N	2020-10-12 15:35:28.777121
56205	MFEM	2020-10-09	23.18	23.31	23.18	23.3	2006	{}	\N	2020-10-12 15:35:28.787189
56206	MFEM	2020-10-12	23.3	23.3575	23.3	23.3575	938	{}	\N	2020-10-12 15:35:28.793973
56207	LX	2020-09-28	6.5	6.59	6.2	6.25	1641549	{}	\N	2020-10-12 15:35:30.777924
56208	LX	2020-09-29	6.25	6.28	6.04	6.1	1377592	{}	\N	2020-10-12 15:35:30.787282
56209	LX	2020-09-30	6.14	6.9015	6.13	6.85	3004113	{}	\N	2020-10-12 15:35:30.796648
56210	LX	2020-10-01	6.76	6.84	6.465	6.54	1577370	{}	\N	2020-10-12 15:35:30.80685
56211	LX	2020-10-02	6.32	6.86	6.3	6.56	1299540	{}	\N	2020-10-12 15:35:30.816214
56212	LX	2020-10-05	6.56	6.74	6.4	6.68	1150180	{}	\N	2020-10-12 15:35:30.823294
56213	LX	2020-10-06	6.6	6.92	6.53	6.76	1006174	{}	\N	2020-10-12 15:35:30.829092
56214	LX	2020-10-07	6.8	6.89	6.63	6.74	810416	{}	\N	2020-10-12 15:35:30.836126
56215	LX	2020-10-08	6.84	6.89	6.66	6.78	597896	{}	\N	2020-10-12 15:35:30.842018
56216	LX	2020-10-09	6.75	6.98	6.6708	6.97	766789	{}	\N	2020-10-12 15:35:30.84865
56217	LX	2020-10-12	7.08	7.33	7	7.15	934645	{}	\N	2020-10-12 15:35:30.855816
56218	LYTS	2020-09-28	6.47	7.06	6.47	7.06	99250	{}	\N	2020-10-12 15:35:32.073
56219	LYTS	2020-09-29	7.07	7.12	6.88	6.99	126068	{}	\N	2020-10-12 15:35:32.084153
56220	LYTS	2020-09-30	6.98	7.26	6.68	6.75	264673	{}	\N	2020-10-12 15:35:32.093882
56221	LYTS	2020-10-01	6.8	6.83	6.54	6.79	61582	{}	\N	2020-10-12 15:35:32.101841
56222	LYTS	2020-10-02	6.62	6.8	6.55	6.75	33652	{}	\N	2020-10-12 15:35:32.109626
56223	LYTS	2020-10-05	6.87	7.19	6.87	7.15	96064	{}	\N	2020-10-12 15:35:32.117266
56224	LYTS	2020-10-06	7.055	7.15	7.01	7.12	89018	{}	\N	2020-10-12 15:35:32.12256
56225	LYTS	2020-10-07	7.12	7.46	7.12	7.4	80196	{}	\N	2020-10-12 15:35:32.130454
56226	LYTS	2020-10-08	7.49	7.525	7.29	7.42	76873	{}	\N	2020-10-12 15:35:32.144861
56227	LYTS	2020-10-09	7.45	7.51	7.31	7.48	43341	{}	\N	2020-10-12 15:35:32.152024
56228	LYTS	2020-10-12	7.57	7.58	7.42	7.575	19217	{}	\N	2020-10-12 15:35:32.156981
56229	MASI	2020-09-28	227.01	232.5	226.5101	230.76	359044	{}	\N	2020-10-12 15:35:35.112602
56230	MASI	2020-09-29	232	237.7	230.94	232.31	311065	{}	\N	2020-10-12 15:35:35.12801
56231	MASI	2020-09-30	232.78	238.715	231.645	236.05	268451	{}	\N	2020-10-12 15:35:35.136143
56232	MASI	2020-10-01	238	241.75	228.188	236.47	426878	{}	\N	2020-10-12 15:35:35.14215
56233	MASI	2020-10-02	233.37	237.97	232.19	234.25	263465	{}	\N	2020-10-12 15:35:35.152748
56234	MASI	2020-10-05	234.75	239.97	234.75	239.61	257779	{}	\N	2020-10-12 15:35:35.160612
56235	MASI	2020-10-06	238.99	243.87	238.13	240.13	318489	{}	\N	2020-10-12 15:35:35.168669
56236	MASI	2020-10-07	249	254	245.74	245.74	722909	{}	\N	2020-10-12 15:35:35.174555
56237	MASI	2020-10-08	246	247.5999	239.15	241.06	504802	{}	\N	2020-10-12 15:35:35.180353
56238	MASI	2020-10-09	242.88	244.43	241.36	242.43	224446	{}	\N	2020-10-12 15:35:35.188379
56239	MASI	2020-10-12	239.34	244.19	238.73	238.95	127299	{}	\N	2020-10-12 15:35:35.195374
56240	MBOT	2020-09-28	7.38	7.65	7.38	7.57	43496	{}	\N	2020-10-12 15:35:37.085019
56241	MBOT	2020-09-29	7.5	8.13	7.5	7.89	254542	{}	\N	2020-10-12 15:35:37.093602
56242	MBOT	2020-09-30	7.84	8.19	7.61	7.64	208873	{}	\N	2020-10-12 15:35:37.101912
56243	MBOT	2020-10-01	7.64	7.99	7.58	7.99	183279	{}	\N	2020-10-12 15:35:37.121143
56244	MBOT	2020-10-02	7.82	8.12	7.793	7.9	168383	{}	\N	2020-10-12 15:35:37.129224
56245	MBOT	2020-10-05	7.9	8.0666	7.54	7.67	133921	{}	\N	2020-10-12 15:35:37.139089
56246	MBOT	2020-10-06	7.66	8.12	7.6081	7.77	127173	{}	\N	2020-10-12 15:35:37.147887
56247	MBOT	2020-10-07	7.85	8.45	7.72	8.24	350803	{}	\N	2020-10-12 15:35:37.157658
56248	MBOT	2020-10-08	8.4	8.47	8.21	8.3	106304	{}	\N	2020-10-12 15:35:37.168005
56249	MBOT	2020-10-09	8.3	8.3	8.02	8.25	107736	{}	\N	2020-10-12 15:35:37.176124
56250	MBOT	2020-10-12	8.17	8.2101	7.85	7.92	68011	{}	\N	2020-10-12 15:35:37.184042
56251	MCI	2020-09-28	12.48	12.67	12.3122	12.38	33370	{}	\N	2020-10-12 15:35:38.432882
56252	MCI	2020-09-29	12.38	12.58	12.18	12.55	53198	{}	\N	2020-10-12 15:35:38.441699
56253	MCI	2020-09-30	12.535	12.5995	12.26	12.5	87072	{}	\N	2020-10-12 15:35:38.447254
56254	MCI	2020-10-01	12.5	12.5	12.2772	12.35	57024	{}	\N	2020-10-12 15:35:38.452849
56255	MCI	2020-10-02	12.19	12.3783	12.19	12.24	46076	{}	\N	2020-10-12 15:35:38.459594
56256	MCI	2020-10-05	12.17	12.31	12.17	12.2238	33490	{}	\N	2020-10-12 15:35:38.465095
56257	MCI	2020-10-06	12.2	12.37	12.18	12.25	77248	{}	\N	2020-10-12 15:35:38.470172
56258	MCI	2020-10-07	12.26	12.34	12.2	12.26	36612	{}	\N	2020-10-12 15:35:38.475836
56259	MCI	2020-10-08	12.37	12.5	12.35	12.4363	32046	{}	\N	2020-10-12 15:35:38.482295
56260	MCI	2020-10-09	12.41	12.4798	12.1731	12.47	73424	{}	\N	2020-10-12 15:35:38.488567
56261	MCI	2020-10-12	12.34	12.43	12.3318	12.43	6954	{}	\N	2020-10-12 15:35:38.495202
56262	MDC	2020-09-28	45.38	47.29	45.05	47.04	884864	{}	\N	2020-10-12 15:35:39.575457
56263	MDC	2020-09-29	46.95	47.04	46.04	46.39	748425	{}	\N	2020-10-12 15:35:39.581394
56264	MDC	2020-09-30	46.95	48.03	46.66	47.1	1007512	{}	\N	2020-10-12 15:35:39.586276
56265	MDC	2020-10-01	47.59	48.45	46.93	48.38	949818	{}	\N	2020-10-12 15:35:39.592051
56266	MDC	2020-10-02	46.78	49.47	46.78	49.03	605956	{}	\N	2020-10-12 15:35:39.596924
56267	MDC	2020-10-05	49.49	49.93	47.53	48.63	1038949	{}	\N	2020-10-12 15:35:39.602797
56268	MDC	2020-10-06	49.09	49.665	47.55	47.64	971114	{}	\N	2020-10-12 15:35:39.611823
56269	MDC	2020-10-07	48.44	48.91	46.98	48.2	1139729	{}	\N	2020-10-12 15:35:39.616179
56270	MDC	2020-10-08	50.13	50.98	49.4101	50.49	1064100	{}	\N	2020-10-12 15:35:39.620208
56271	MDC	2020-10-09	50.91	51.4075	50.1	50.65	597682	{}	\N	2020-10-12 15:35:39.625198
56272	MDC	2020-10-12	50.9	51.11	50.3	50.65	137570	{}	\N	2020-10-12 15:35:39.630123
56273	MDLY	2020-09-28	0.6	0.61	0.574101	0.6014	107561	{}	\N	2020-10-12 15:35:40.548238
56274	MDLY	2020-09-29	0.5843	0.62	0.5654	0.62	763870	{}	\N	2020-10-12 15:35:40.553978
56275	MDLY	2020-09-30	0.62	0.64	0.582	0.582	500758	{}	\N	2020-10-12 15:35:40.559964
56276	MDLY	2020-10-01	0.5872	0.604383	0.582	0.589	276539	{}	\N	2020-10-12 15:35:40.566068
56277	MDLY	2020-10-02	0.58	0.6301	0.58	0.6019	194158	{}	\N	2020-10-12 15:35:40.571738
56278	MDLY	2020-10-05	0.5801	0.601899	0.5801	0.5938	187822	{}	\N	2020-10-12 15:35:40.57793
56279	MDLY	2020-10-06	0.6195	0.6199	0.57	0.57	277356	{}	\N	2020-10-12 15:35:40.583237
56280	MDLY	2020-10-07	0.575	0.585	0.5603	0.5799	193898	{}	\N	2020-10-12 15:35:40.588778
56281	MDLY	2020-10-08	0.6	0.646	0.5649	0.62	1354594	{}	\N	2020-10-12 15:35:40.594723
56282	MDLY	2020-10-09	0.61	0.629	0.590201	0.5951	777041	{}	\N	2020-10-12 15:35:40.600685
56283	MDLY	2020-10-12	2.63	2.79	2.27	2.32	71344681	{}	\N	2020-10-12 15:35:40.608726
56284	MDRX	2020-09-28	8.14	8.395	8.13	8.21	1349308	{}	\N	2020-10-12 15:35:41.95405
56285	MDRX	2020-09-29	8.2	8.27	7.98	8.06	888930	{}	\N	2020-10-12 15:35:41.961212
56286	MDRX	2020-09-30	8.12	8.28	8.04	8.14	1042303	{}	\N	2020-10-12 15:35:41.966841
56287	MDRX	2020-10-01	8.17	8.3	7.89	8.17	830468	{}	\N	2020-10-12 15:35:41.972016
56288	MDRX	2020-10-02	7.97	8.32	7.97	8.26	513776	{}	\N	2020-10-12 15:35:41.979074
56289	MDRX	2020-10-05	8.34	8.53	8.27	8.29	1013004	{}	\N	2020-10-12 15:35:41.985601
56290	MDRX	2020-10-06	8.34	8.73	8.215	8.39	692649	{}	\N	2020-10-12 15:35:41.990878
56291	MDRX	2020-10-07	8.46	8.66	8.29	8.35	840556	{}	\N	2020-10-12 15:35:41.997074
56292	MDRX	2020-10-08	8.45	8.595	8.35	8.54	598804	{}	\N	2020-10-12 15:35:42.00212
56293	MDRX	2020-10-09	8.61	8.69	8.345	8.39	595036	{}	\N	2020-10-12 15:35:42.00709
56294	MDRX	2020-10-12	8.41	8.52	8.33	8.52	129746	{}	\N	2020-10-12 15:35:42.013051
56295	MEDP	2020-09-28	109.27	111.06	108.2901	109.91	312327	{}	\N	2020-10-12 15:35:43.140484
56296	MEDP	2020-09-29	110.08	112.04	109.895	111.33	225480	{}	\N	2020-10-12 15:35:43.147329
56297	MEDP	2020-09-30	112	113.61	111.36	111.75	359474	{}	\N	2020-10-12 15:35:43.153394
56298	MEDP	2020-10-01	112.19	114.32	111.27	114.16	212385	{}	\N	2020-10-12 15:35:43.158828
56299	MEDP	2020-10-02	117.75	118.14	114.5026	115.02	234177	{}	\N	2020-10-12 15:35:43.165248
56300	MEDP	2020-10-05	116.46	119.31	116.46	118.85	217897	{}	\N	2020-10-12 15:35:43.170465
56301	MEDP	2020-10-06	119.09	121.13	117.33	117.7	197444	{}	\N	2020-10-12 15:35:43.175385
56302	MEDP	2020-10-07	118.9	120.16	118.64	118.65	230458	{}	\N	2020-10-12 15:35:43.181166
56303	MEDP	2020-10-08	119.97	120.03	118	119.97	121499	{}	\N	2020-10-12 15:35:43.186065
56304	MEDP	2020-10-09	121.27	123.32	120.46	123.24	151012	{}	\N	2020-10-12 15:35:43.191044
56305	MEDP	2020-10-12	123.94	124.98	122.58	124.2841	23467	{}	\N	2020-10-12 15:35:43.197859
56306	MERC	2020-09-28	6.31	6.37	6.06	6.16	486271	{}	\N	2020-10-12 15:35:44.116624
56307	MERC	2020-09-29	6.2	6.385	6.12	6.35	452059	{}	\N	2020-10-12 15:35:44.125999
56308	MERC	2020-09-30	6.34	6.7	6.34	6.6	355953	{}	\N	2020-10-12 15:35:44.135964
56309	MERC	2020-10-01	6.62	6.62	6.4	6.46	221154	{}	\N	2020-10-12 15:35:44.142737
56310	MERC	2020-10-02	6.31	6.51	6.31	6.4	266271	{}	\N	2020-10-12 15:35:44.148839
56311	MERC	2020-10-05	6.41	6.68	6.34	6.57	312389	{}	\N	2020-10-12 15:35:44.15534
56312	MERC	2020-10-06	6.59	6.68	6.23	6.26	392028	{}	\N	2020-10-12 15:35:44.162218
56313	MERC	2020-10-07	6.3	6.48	6.26	6.44	150392	{}	\N	2020-10-12 15:35:44.167185
56314	MERC	2020-10-08	6.43	6.68	6.35	6.61	182813	{}	\N	2020-10-12 15:35:44.172246
56315	MERC	2020-10-09	6.65	6.7	6.45	6.52	108127	{}	\N	2020-10-12 15:35:44.177401
56316	MERC	2020-10-12	6.51	6.64	6.5	6.58	66066	{}	\N	2020-10-12 15:35:44.18312
56317	LXRX	2020-09-28	1.65	1.665	1.47	1.56	1137156	{}	\N	2020-10-12 15:35:46.395709
56318	LXRX	2020-09-29	1.55	1.55	1.45	1.46	1005685	{}	\N	2020-10-12 15:35:46.403178
56319	LXRX	2020-09-30	1.47	1.48	1.415	1.44	477450	{}	\N	2020-10-12 15:35:46.407384
56320	LXRX	2020-10-01	1.46	1.47	1.4	1.43	1213174	{}	\N	2020-10-12 15:35:46.412665
56321	LXRX	2020-10-02	1.43	1.46	1.37	1.46	577202	{}	\N	2020-10-12 15:35:46.418688
56322	LXRX	2020-10-05	1.46	1.48	1.405	1.415	1058114	{}	\N	2020-10-12 15:35:46.423256
56323	LXRX	2020-10-06	1.4356	1.5	1.42	1.44	1672507	{}	\N	2020-10-12 15:35:46.428285
56324	LXRX	2020-10-07	1.45	1.51	1.44	1.48	1444261	{}	\N	2020-10-12 15:35:46.433734
56325	LXRX	2020-10-08	1.5	1.58	1.5	1.56	549775	{}	\N	2020-10-12 15:35:46.439385
56326	LXRX	2020-10-09	1.5717	1.59	1.55	1.56	402954	{}	\N	2020-10-12 15:35:46.448246
56327	LXRX	2020-10-12	1.57	1.57	1.48	1.4888	216437	{}	\N	2020-10-12 15:35:46.453899
56328	MA	2020-09-28	336.61	340.57	335.81	338.39	3132010	{}	\N	2020-10-12 15:35:47.688988
56329	MA	2020-09-29	338.39	340.72	335.82	336.23	5487348	{}	\N	2020-10-12 15:35:47.694603
56330	MA	2020-09-30	336.7	343.65	336.42	338.17	6523495	{}	\N	2020-10-12 15:35:47.700343
56331	MA	2020-10-01	342.24	346.7	340.34	343.37	4845606	{}	\N	2020-10-12 15:35:47.706347
56332	MA	2020-10-02	338.27	342.61	336.01	338.83	4379047	{}	\N	2020-10-12 15:35:47.711694
56333	MA	2020-10-05	341.68	343.25	338.97	343.25	3954514	{}	\N	2020-10-12 15:35:47.718062
56334	MA	2020-10-06	342.06	346.36	336.22	337.43	4984730	{}	\N	2020-10-12 15:35:47.72359
56335	MA	2020-10-07	341.78	344.65	340.335	343.9	6143364	{}	\N	2020-10-12 15:35:47.72928
56336	MA	2020-10-08	345.01	347.69	344.24	345.15	4003483	{}	\N	2020-10-12 15:35:47.73574
56337	MA	2020-10-09	347.99	353.36	347	352.39	5832850	{}	\N	2020-10-12 15:35:47.741673
56338	MA	2020-10-12	355	355	350.82	354.315	1454583	{}	\N	2020-10-12 15:35:47.749538
56339	MANT	2020-09-28	70.31	70.635	69.69	70.54	184345	{}	\N	2020-10-12 15:35:48.641071
56340	MANT	2020-09-29	70.68	70.78	69.2	69.44	131169	{}	\N	2020-10-12 15:35:48.647225
56341	MANT	2020-09-30	69.41	69.91	67.7	68.88	163845	{}	\N	2020-10-12 15:35:48.653506
56342	MANT	2020-10-01	69.16	69.81	68.17	69.51	151358	{}	\N	2020-10-12 15:35:48.659956
56343	MANT	2020-10-02	68.25	70.3	68.05	70.08	137913	{}	\N	2020-10-12 15:35:48.66778
56344	MANT	2020-10-05	70.43	70.9799	69.83	70.27	112197	{}	\N	2020-10-12 15:35:48.674599
56345	MANT	2020-10-06	70.46	72.075	70.295	70.44	145948	{}	\N	2020-10-12 15:35:48.680724
56346	MANT	2020-10-07	71	72.8957	70.07	70.63	136086	{}	\N	2020-10-12 15:35:48.686872
56347	MANT	2020-10-08	71.09	71.9299	70.62	71.13	106964	{}	\N	2020-10-12 15:35:48.692782
56348	MANT	2020-10-09	71.46	72.1	70.83	71.09	132094	{}	\N	2020-10-12 15:35:48.70048
56349	MANT	2020-10-12	71.14	72.63	71.14	72.54	22053	{}	\N	2020-10-12 15:35:48.708405
56350	MATX	2020-09-28	38.52	39.75	38.52	39.59	181689	{}	\N	2020-10-12 15:35:49.64954
56351	MATX	2020-09-29	39.38	40.13	39.25	39.75	347266	{}	\N	2020-10-12 15:35:49.656383
56352	MATX	2020-09-30	40.08	41.12	39.62	40.09	424976	{}	\N	2020-10-12 15:35:49.662084
56353	MATX	2020-10-01	40.12	40.4352	39.658	39.88	384178	{}	\N	2020-10-12 15:35:49.669738
56354	MATX	2020-10-02	38.95	41.135	38.64	40.8	358208	{}	\N	2020-10-12 15:35:49.675905
56355	MATX	2020-10-05	41.03	42.57	40.8	42.22	340747	{}	\N	2020-10-12 15:35:49.682387
56356	MATX	2020-10-06	42.88	45.11	42.545	43.84	569058	{}	\N	2020-10-12 15:35:49.6881
56357	MATX	2020-10-07	44.3	45.2	44.01	44.66	494785	{}	\N	2020-10-12 15:35:49.693544
56358	MATX	2020-10-08	45.28	45.28	43.36	44.49	491242	{}	\N	2020-10-12 15:35:49.700779
56359	MATX	2020-10-09	49.31	52.78	49.31	51.37	924342	{}	\N	2020-10-12 15:35:49.706423
56360	MATX	2020-10-12	51.95	52.94	51.26	51.26	238212	{}	\N	2020-10-12 15:35:49.711954
56361	MBII	2020-09-28	1.22	1.22	1.17	1.18	125052	{}	\N	2020-10-12 15:35:50.610883
56362	MBII	2020-09-29	1.19	1.26	1.19	1.26	70356	{}	\N	2020-10-12 15:35:50.618459
56363	MBII	2020-09-30	1.26	1.26	1.22	1.22	88458	{}	\N	2020-10-12 15:35:50.624004
56364	MBII	2020-10-01	1.23	1.25	1.19	1.22	137234	{}	\N	2020-10-12 15:35:50.629657
56365	MBII	2020-10-02	1.2	1.25	1.2	1.23	101274	{}	\N	2020-10-12 15:35:50.636604
56366	MBII	2020-10-05	1.24	1.25	1.1759	1.19	152587	{}	\N	2020-10-12 15:35:50.642811
56367	MBII	2020-10-06	1.19	1.21	1.18	1.185	125190	{}	\N	2020-10-12 15:35:50.648569
56368	MBII	2020-10-07	1.2	1.2	1.19	1.2	83769	{}	\N	2020-10-12 15:35:50.654414
56369	MBII	2020-10-08	1.2	1.2	1.15	1.16	109384	{}	\N	2020-10-12 15:35:50.661131
56370	MBII	2020-10-09	1.17	1.17	1.12	1.15	92299	{}	\N	2020-10-12 15:35:50.667012
56371	MBII	2020-10-12	1.15	1.15	1.0716	1.0716	198248	{}	\N	2020-10-12 15:35:50.673181
56372	MCEP	2020-09-28	2.69	2.7291	2.61	2.65	50741	{}	\N	2020-10-12 15:35:52.294009
56373	MCEP	2020-09-29	2.95	3.37	2.603	2.65	3888786	{}	\N	2020-10-12 15:35:52.299312
56374	MCEP	2020-09-30	2.49	2.55	2.25	2.35	469589	{}	\N	2020-10-12 15:35:52.305306
56375	MCEP	2020-10-01	2.34	2.4	2.25	2.26	128087	{}	\N	2020-10-12 15:35:52.311022
56376	MCEP	2020-10-02	2.19	2.23	2.08	2.15	166006	{}	\N	2020-10-12 15:35:52.316336
56377	MCEP	2020-10-05	2.15	2.29	2.15	2.27	145122	{}	\N	2020-10-12 15:35:52.322924
56378	MCEP	2020-10-06	2.22	2.64	2.22	2.39	499565	{}	\N	2020-10-12 15:35:52.328883
56379	MCEP	2020-10-07	2.44	2.5	2.31	2.38	239207	{}	\N	2020-10-12 15:35:52.334981
56380	MCEP	2020-10-08	2.37	2.6299	2.26	2.45	1214626	{}	\N	2020-10-12 15:35:52.340957
56381	MCEP	2020-10-09	2.39	2.55	2.3	2.5025	473945	{}	\N	2020-10-12 15:35:52.34607
56382	MCEP	2020-10-12	2.52	2.52	2.41	2.4321	37276	{}	\N	2020-10-12 15:35:52.351377
56383	MCO	2020-09-28	283.66	286.5	282.655	285.16	553046	{}	\N	2020-10-12 15:35:53.290785
56384	MCO	2020-09-29	285.53	288.285	282.94	285.73	756784	{}	\N	2020-10-12 15:35:53.297671
56385	MCO	2020-09-30	286.88	292.63	286	289.85	1332437	{}	\N	2020-10-12 15:35:53.302594
56386	MCO	2020-10-01	294.08	298.18	290.43	293.5	1257912	{}	\N	2020-10-12 15:35:53.308221
56387	MCO	2020-10-02	289.88	296.4	287.68	293.96	884447	{}	\N	2020-10-12 15:35:53.313732
56388	MCO	2020-10-05	295.88	297.93	292.075	295.18	634344	{}	\N	2020-10-12 15:35:53.319091
56389	MCO	2020-10-06	296.35	296.77	286.02	287.94	1332596	{}	\N	2020-10-12 15:35:53.32486
56390	MCO	2020-10-07	288.86	291.22	287.19	288.25	932198	{}	\N	2020-10-12 15:35:53.329594
56391	MCO	2020-10-08	289.22	294.03	289.22	292.14	685869	{}	\N	2020-10-12 15:35:53.335631
56392	MCO	2020-10-09	293.79	295.19	292.65	294.73	702792	{}	\N	2020-10-12 15:35:53.341418
56393	MCO	2020-10-12	296.9	297.14	292.78	294.93	195749	{}	\N	2020-10-12 15:35:53.346201
56394	MCX	2020-09-28	24.5	24.69	24.5	24.6112	9275	{}	\N	2020-10-12 15:35:54.383667
56395	MCX	2020-09-29	24.61	24.67	24.6	24.63	9056	{}	\N	2020-10-12 15:35:54.389478
56396	MCX	2020-09-30	24.53	24.69	24.53	24.69	10394	{}	\N	2020-10-12 15:35:54.394929
56397	MCX	2020-10-01	24.625	24.74	24.55	24.67	14646	{}	\N	2020-10-12 15:35:54.400361
56398	MCX	2020-10-02	24.61	24.7	24.57	24.6593	8704	{}	\N	2020-10-12 15:35:54.407922
56399	MCX	2020-10-05	24.7	24.7	24.62	24.68	11066	{}	\N	2020-10-12 15:35:54.414753
56400	MCX	2020-10-06	24.7	24.9	24.66	24.89	40442	{}	\N	2020-10-12 15:35:54.420517
56401	MCX	2020-10-07	24.815	24.94	24.8064	24.94	8912	{}	\N	2020-10-12 15:35:54.426653
56402	MCX	2020-10-08	24.89	24.93	24.79	24.92	6312	{}	\N	2020-10-12 15:35:54.431721
56403	MCX	2020-10-09	24.91	24.92	24.89	24.89	4256	{}	\N	2020-10-12 15:35:54.436744
56404	MCX	2020-10-12	25.02	25.25	25.02	25.2	19376	{}	\N	2020-10-12 15:35:54.443008
56405	MDLX	2020-09-28	5.5	5.75	5.5	5.63	6471	{}	\N	2020-10-12 15:35:55.400158
56406	MDLX	2020-09-29	5.66	7	5.66	6.63	141730	{}	\N	2020-10-12 15:35:55.406839
56407	MDLX	2020-09-30	6.6	7.0554	6.6	6.81	23522	{}	\N	2020-10-12 15:35:55.412694
56408	MDLX	2020-10-01	6.45	6.94	6.415	6.6	11814	{}	\N	2020-10-12 15:35:55.41859
56409	MDLX	2020-10-02	6.5	6.5	5.6	6.31	57908	{}	\N	2020-10-12 15:35:55.425187
56410	MDLX	2020-10-05	6.27	7.16	6.27	6.915	51450	{}	\N	2020-10-12 15:35:55.431109
56411	MDLX	2020-10-06	6.955	6.98	6.77	6.77	6670	{}	\N	2020-10-12 15:35:55.436731
56412	MDLX	2020-10-07	6.89	7.05	6.8	6.8485	6946	{}	\N	2020-10-12 15:35:55.442612
56413	MDLX	2020-10-08	6.9644	7.93	6.84	7.93	100558	{}	\N	2020-10-12 15:35:55.448521
56414	MDLX	2020-10-09	7.89	8.7565	6.54	7.1946	117955	{}	\N	2020-10-12 15:35:55.45419
56415	MDLX	2020-10-12	9.7	9.7	9.7	9.7	6100	{}	\N	2020-10-12 15:35:55.461239
56416	MDWD	2020-09-28	3.71	3.765	3.6426	3.71	21162	{}	\N	2020-10-12 15:35:56.762756
56417	MDWD	2020-09-29	3.68	3.85	3.6211	3.84	43948	{}	\N	2020-10-12 15:35:56.768998
56418	MDWD	2020-09-30	3.785	3.8	3.71	3.72	69824	{}	\N	2020-10-12 15:35:56.774091
56419	MDWD	2020-10-01	3.76	3.8399	3.72	3.77	23399	{}	\N	2020-10-12 15:35:56.780535
56420	MDWD	2020-10-02	3.7	3.75	3.68	3.74	36179	{}	\N	2020-10-12 15:35:56.787411
56421	MDWD	2020-10-05	3.75	3.8	3.7	3.75	44718	{}	\N	2020-10-12 15:35:56.793467
56422	MDWD	2020-10-06	3.79	3.79	3.71	3.77	38564	{}	\N	2020-10-12 15:35:56.799252
56423	MDWD	2020-10-07	3.7339	3.79	3.71	3.74	22308	{}	\N	2020-10-12 15:35:56.804661
56424	MDWD	2020-10-08	3.77	3.8228	3.75	3.79	109304	{}	\N	2020-10-12 15:35:56.811047
56425	MDWD	2020-10-09	3.79	3.93	3.7	3.7401	120619	{}	\N	2020-10-12 15:35:56.816884
56426	MDWD	2020-10-12	3.76	3.76	3.65	3.66	23160	{}	\N	2020-10-12 15:35:56.823536
56427	MEI	2020-09-28	27.79	28.55	27.79	28.28	212941	{}	\N	2020-10-12 15:35:57.945937
56428	MEI	2020-09-29	28.47	28.79	28.21	28.61	358979	{}	\N	2020-10-12 15:35:57.952293
56429	MEI	2020-09-30	28.8	29.31	28.22	28.5	474667	{}	\N	2020-10-12 15:35:57.959047
56430	MEI	2020-10-01	28.65	28.92	28.13	28.42	435228	{}	\N	2020-10-12 15:35:57.96669
56431	MEI	2020-10-02	26.84	29.1	26.84	28.9	469858	{}	\N	2020-10-12 15:35:57.972602
56432	MEI	2020-10-05	29.09	29.55	29.07	29.16	353659	{}	\N	2020-10-12 15:35:57.978932
56433	MEI	2020-10-06	29.49	30.19	29.19	29.59	585717	{}	\N	2020-10-12 15:35:57.98466
56434	MEI	2020-10-07	29.85	30.14	29.49	29.49	549758	{}	\N	2020-10-12 15:35:57.990637
56435	MEI	2020-10-08	29.7	29.86	29.46	29.66	301671	{}	\N	2020-10-12 15:35:57.997028
56436	MEI	2020-10-09	29.93	30.22	29.7	29.84	288020	{}	\N	2020-10-12 15:35:58.004188
56437	MEI	2020-10-12	29.72	30.181	29.72	30.06	36365	{}	\N	2020-10-12 15:35:58.010251
56438	MFA	2020-09-28	2.69	2.8	2.68	2.78	5041434	{}	\N	2020-10-12 15:35:58.875157
56439	MFA	2020-09-29	2.7	2.74	2.63	2.7	8177090	{}	\N	2020-10-12 15:35:58.881754
56440	MFA	2020-09-30	2.67	2.725	2.64	2.68	8362045	{}	\N	2020-10-12 15:35:58.887767
56441	MFA	2020-10-01	2.67	2.8	2.67	2.79	13100842	{}	\N	2020-10-12 15:35:58.89344
56442	MFA	2020-10-02	2.65	2.81	2.63	2.81	10083990	{}	\N	2020-10-12 15:35:58.900123
56443	MFA	2020-10-05	2.77	2.82	2.77	2.8	7429158	{}	\N	2020-10-12 15:35:58.906173
56444	MFA	2020-10-06	2.81	2.84	2.74	2.76	9443836	{}	\N	2020-10-12 15:35:58.911837
56445	MFA	2020-10-07	2.76	2.8	2.74	2.79	7017316	{}	\N	2020-10-12 15:35:58.918785
56446	MFA	2020-10-08	2.79	2.89	2.7828	2.86	7698419	{}	\N	2020-10-12 15:35:58.924744
56447	MFA	2020-10-09	2.87	2.89	2.8116	2.87	5619339	{}	\N	2020-10-12 15:35:58.931074
56448	MFA	2020-10-12	2.85	2.89	2.83	2.88	1772296	{}	\N	2020-10-12 15:35:58.93687
56449	MFIN	2020-09-28	2.52	2.56	2.49	2.49	17287	{}	\N	2020-10-12 15:36:00.40448
56450	MFIN	2020-09-29	2.49	2.58	2.45	2.52	19669	{}	\N	2020-10-12 15:36:00.410655
56451	MFIN	2020-09-30	2.5353	2.5499	2.41	2.5	41598	{}	\N	2020-10-12 15:36:00.416189
56452	MFIN	2020-10-01	2.51	2.64	2.5	2.61	49702	{}	\N	2020-10-12 15:36:00.422831
56453	MFIN	2020-10-02	2.52	2.56	2.36	2.56	103185	{}	\N	2020-10-12 15:36:00.429741
56454	MFIN	2020-10-05	2.3601	2.6799	2.3601	2.59	49690	{}	\N	2020-10-12 15:36:00.435964
56455	MFIN	2020-10-06	2.59	2.62	2.52	2.55	16091	{}	\N	2020-10-12 15:36:00.442123
56456	MFIN	2020-10-07	2.5182	2.58	2.51	2.54	30506	{}	\N	2020-10-12 15:36:00.448634
56457	MFIN	2020-10-08	2.55	2.66	2.526	2.66	41236	{}	\N	2020-10-12 15:36:00.45447
56458	MFIN	2020-10-09	2.6	2.65	2.547	2.63	25818	{}	\N	2020-10-12 15:36:00.460571
56459	MFIN	2020-10-12	2.62	2.6499	2.6	2.6241	18004	{}	\N	2020-10-12 15:36:00.467568
56460	MGA	2020-09-28	45.55	46.25	45.26	45.97	932936	{}	\N	2020-10-12 15:36:01.343186
56461	MGA	2020-09-29	45.98	45.995	45.36	45.75	2243776	{}	\N	2020-10-12 15:36:01.349698
56462	MGA	2020-09-30	45.74	46.28	45.69	45.75	2166452	{}	\N	2020-10-12 15:36:01.35507
56463	MGA	2020-10-01	46.33	46.75	46.07	46.61	1610728	{}	\N	2020-10-12 15:36:01.360002
56464	MGA	2020-10-02	45.64	47.825	45.64	47.64	1717626	{}	\N	2020-10-12 15:36:01.3662
56465	MGA	2020-10-05	48.2	49.22	48.2	49.07	1359938	{}	\N	2020-10-12 15:36:01.373913
56466	MGA	2020-10-06	49.5	49.68	48.86	48.9	2424034	{}	\N	2020-10-12 15:36:01.38126
56467	MGA	2020-10-07	49.63	51.83	49.49	51.69	1618429	{}	\N	2020-10-12 15:36:01.386884
56468	MGA	2020-10-08	52	52.84	51.81	52.73	1913764	{}	\N	2020-10-12 15:36:01.39218
56469	MGA	2020-10-09	52.82	53.39	52.55	52.98	1678265	{}	\N	2020-10-12 15:36:01.398042
56470	MGA	2020-10-12	53.43	53.5	52.75	52.85	191999	{}	\N	2020-10-12 15:36:01.403844
56471	MGNX	2020-09-28	25.28	25.86	24.9	25.43	357258	{}	\N	2020-10-12 15:36:02.366425
56472	MGNX	2020-09-29	25.45	25.9	24.59	25.19	260247	{}	\N	2020-10-12 15:36:02.37463
56473	MGNX	2020-09-30	25.15	26.1	25.03	25.19	288014	{}	\N	2020-10-12 15:36:02.387453
56474	MGNX	2020-10-01	25.43	26.25	25.02	25.74	365418	{}	\N	2020-10-12 15:36:02.396017
56475	MGNX	2020-10-02	25.47	25.87	24.25	24.27	353757	{}	\N	2020-10-12 15:36:02.404619
56476	MGNX	2020-10-05	24.52	26.09	24.52	26	335132	{}	\N	2020-10-12 15:36:02.413091
56477	MGNX	2020-10-06	25.985	26.9	25.76	25.81	260013	{}	\N	2020-10-12 15:36:02.422019
56478	MGNX	2020-10-07	26.57	26.96	25.84	25.95	803119	{}	\N	2020-10-12 15:36:02.430218
56479	MGNX	2020-10-08	26.225	26.225	25.42	25.75	303560	{}	\N	2020-10-12 15:36:02.436406
56480	MGNX	2020-10-09	26.07	26.85	24.67	25.46	355866	{}	\N	2020-10-12 15:36:02.441704
56481	MGNX	2020-10-12	25.7	25.77	25.31	25.53	132394	{}	\N	2020-10-12 15:36:02.446832
56482	LXP	2020-09-28	10.61	10.8	10.55	10.75	4042093	{}	\N	2020-10-12 15:36:04.018632
56483	LXP	2020-09-29	10.67	10.67	10.4631	10.54	2936247	{}	\N	2020-10-12 15:36:04.02546
56484	LXP	2020-09-30	10.57	10.62	10.36	10.45	4026700	{}	\N	2020-10-12 15:36:04.032457
56485	LXP	2020-10-01	10.47	10.62	10.34	10.6	3989491	{}	\N	2020-10-12 15:36:04.037933
56486	LXP	2020-10-02	10.43	10.84	10.36	10.82	3742035	{}	\N	2020-10-12 15:36:04.049
56487	LXP	2020-10-05	10.9	10.95	10.72	10.84	2563986	{}	\N	2020-10-12 15:36:04.057257
56488	LXP	2020-10-06	10.9	11.02	10.69	10.73	5240555	{}	\N	2020-10-12 15:36:04.065193
56489	LXP	2020-10-07	10.81	10.86	10.71	10.72	4146593	{}	\N	2020-10-12 15:36:04.071855
56490	LXP	2020-10-08	10.81	11.025	10.81	10.92	1902500	{}	\N	2020-10-12 15:36:04.076201
56491	LXP	2020-10-09	11.02	11.08	10.92	10.95	2024807	{}	\N	2020-10-12 15:36:04.080316
56492	LXP	2020-10-12	10.98	11.06	10.91	11.05	563439	{}	\N	2020-10-12 15:36:04.086598
56493	MANH	2020-09-28	95.72	97.045	94.44	95.37	215600	{}	\N	2020-10-12 15:36:05.145394
56494	MANH	2020-09-29	95.31	96.95	95.13	95.3	186993	{}	\N	2020-10-12 15:36:05.150333
56495	MANH	2020-09-30	95.33	97.595	95	95.49	260478	{}	\N	2020-10-12 15:36:05.156175
56496	MANH	2020-10-01	95.8	98.15	95.8	97.43	211023	{}	\N	2020-10-12 15:36:05.160869
56497	MANH	2020-10-02	94.66	97.13	94.66	95.73	196990	{}	\N	2020-10-12 15:36:05.165763
56498	MANH	2020-10-05	96.85	97.63	95.71	96.85	493927	{}	\N	2020-10-12 15:36:05.171567
56499	MANH	2020-10-06	97.36	99.59	96.68	97.15	267614	{}	\N	2020-10-12 15:36:05.177533
56500	MANH	2020-10-07	97.75	99.99	97.045	99.54	210006	{}	\N	2020-10-12 15:36:05.182832
56501	MANH	2020-10-08	100.515	101.21	99.66	100.48	227606	{}	\N	2020-10-12 15:36:05.188039
56502	MANH	2020-10-09	101.21	102.31	99.2279	102	153280	{}	\N	2020-10-12 15:36:05.192825
56503	MANH	2020-10-12	103	103.86	102	103.015	40234	{}	\N	2020-10-12 15:36:05.197209
56504	MBB	2020-09-28	110.35	110.39	110.29	110.37	1108748	{}	\N	2020-10-12 15:36:06.174686
56505	MBB	2020-09-29	110.35	110.44	110.33	110.42	1495521	{}	\N	2020-10-12 15:36:06.18142
56506	MBB	2020-09-30	110.41	110.45	110.3901	110.4	1481960	{}	\N	2020-10-12 15:36:06.187474
56507	MBB	2020-10-01	110.2	110.21	110.1	110.2	2833096	{}	\N	2020-10-12 15:36:06.192764
56508	MBB	2020-10-02	110.32	110.32	110.19	110.26	2153419	{}	\N	2020-10-12 15:36:06.197809
56509	MBB	2020-10-05	110.2574	110.28	110.15	110.175	1367819	{}	\N	2020-10-12 15:36:06.203794
56510	MBB	2020-10-06	110.1909	110.3	110.18	110.24	1835084	{}	\N	2020-10-12 15:36:06.209075
56511	MBB	2020-10-07	110.3	110.35	110.28	110.28	11940762	{}	\N	2020-10-12 15:36:06.214395
56512	MBB	2020-10-08	110.2758	110.29	110.2	110.23	2338645	{}	\N	2020-10-12 15:36:06.220171
56513	MBB	2020-10-09	110.22	110.26	110.165	110.2	2788697	{}	\N	2020-10-12 15:36:06.226406
56514	MBB	2020-10-12	110.2	110.23	110.18	110.228	171957	{}	\N	2020-10-12 15:36:06.231285
56515	MBIO	2020-09-28	3.2333	3.2333	3.07	3.13	856689	{}	\N	2020-10-12 15:36:08.33766
56516	MBIO	2020-09-29	3.12	3.22	3.08	3.16	1009929	{}	\N	2020-10-12 15:36:08.345079
56517	MBIO	2020-09-30	3.1692	3.23	3.12	3.15	966952	{}	\N	2020-10-12 15:36:08.351152
56518	MBIO	2020-10-01	3.18	3.24	3.11	3.16	720084	{}	\N	2020-10-12 15:36:08.357035
56519	MBIO	2020-10-02	3.15	3.27	3.13	3.18	1047041	{}	\N	2020-10-12 15:36:08.362839
56520	MBIO	2020-10-05	3.22	3.29	3.16	3.27	1066003	{}	\N	2020-10-12 15:36:08.36785
56521	MBIO	2020-10-06	3.3	3.44	3.21	3.275	1831190	{}	\N	2020-10-12 15:36:08.373646
56522	MBIO	2020-10-07	3.34	3.4098	3.275	3.36	967989	{}	\N	2020-10-12 15:36:08.378665
56523	MBIO	2020-10-08	3.45	3.62	3.41	3.53	2236908	{}	\N	2020-10-12 15:36:08.384441
56524	MBIO	2020-10-09	3.52	3.5469	3.34	3.37	1458289	{}	\N	2020-10-12 15:36:08.391207
56525	MBIO	2020-10-12	3.38	3.39	3.15	3.22	984470	{}	\N	2020-10-12 15:36:08.39602
56526	MCEF	2020-09-28	18.97	19.0737	18.97	19.0737	318	{}	\N	2020-10-12 15:36:09.378466
56527	MCEF	2020-09-29	19.07	19.07	19.07	19.07	362	{}	\N	2020-10-12 15:36:09.385132
56528	MCEF	2020-10-01	19.1	19.1867	19.1	19.13	6408	{}	\N	2020-10-12 15:36:09.392288
56529	MCEF	2020-10-02	19.13	19.2	19.11	19.2	2630	{}	\N	2020-10-12 15:36:09.399316
56530	MCEF	2020-10-06	19.06	19.18	19.0501	19.18	6103	{}	\N	2020-10-12 15:36:09.404712
56531	MCEF	2020-10-08	19.24	19.24	19.19	19.19	1876	{}	\N	2020-10-12 15:36:09.412528
56532	MCEF	2020-10-09	19.17	19.255	19.17	19.255	128	{}	\N	2020-10-12 15:36:09.420855
56533	MCEF	2020-10-12	19.2823	19.2823	19.2823	19.2823	169	{}	\N	2020-10-12 15:36:09.431379
56534	MCR	2020-09-28	8.1	8.17	8.0951	8.11	67387	{}	\N	2020-10-12 15:36:10.341804
56535	MCR	2020-09-29	8.04	8.09	8.01	8.05	232996	{}	\N	2020-10-12 15:36:10.349233
56536	MCR	2020-09-30	8.14	8.14	8.01	8.01	236452	{}	\N	2020-10-12 15:36:10.356287
56537	MCR	2020-10-01	8.01	8.09	8.01	8.07	175608	{}	\N	2020-10-12 15:36:10.362619
56538	MCR	2020-10-02	8.11	8.2	8.0748	8.12	118712	{}	\N	2020-10-12 15:36:10.36869
56539	MCR	2020-10-05	8.15	8.1826	8.12	8.14	197721	{}	\N	2020-10-12 15:36:10.374327
56540	MCR	2020-10-06	8.12	8.18	8.1	8.13	220425	{}	\N	2020-10-12 15:36:10.380311
56541	MCR	2020-10-07	8.21	8.25	8.2	8.25	125915	{}	\N	2020-10-12 15:36:10.386366
56542	MCR	2020-10-08	8.22	8.3	8.22	8.23	101392	{}	\N	2020-10-12 15:36:10.39359
56543	MCR	2020-10-09	8.28	8.34	8.265	8.31	176314	{}	\N	2020-10-12 15:36:10.399541
56544	MCR	2020-10-12	8.3	8.36	8.3	8.34	28459	{}	\N	2020-10-12 15:36:10.405688
56545	MCY	2020-09-28	41.72	42.22	41.605	41.89	181799	{}	\N	2020-10-12 15:36:12.82188
56546	MCY	2020-09-29	41.9	41.9	41.11	41.37	305095	{}	\N	2020-10-12 15:36:12.829909
56547	MCY	2020-09-30	41.51	41.85	41.02	41.37	553270	{}	\N	2020-10-12 15:36:12.834898
56548	MCY	2020-10-01	41.44	41.63	41.02	41.23	269587	{}	\N	2020-10-12 15:36:12.840949
56549	MCY	2020-10-02	40.95	41.68	40.95	41.32	303938	{}	\N	2020-10-12 15:36:12.848179
56550	MCY	2020-10-05	41.69	42.18	41.53	41.61	689854	{}	\N	2020-10-12 15:36:12.85409
56551	MCY	2020-10-06	41.67	41.92	41.12	41.18	394976	{}	\N	2020-10-12 15:36:12.859004
56552	MCY	2020-10-07	41.28	41.62	40.63	40.82	330094	{}	\N	2020-10-12 15:36:12.86493
56553	MCY	2020-10-08	40.98	41.56	40.57	40.8	291903	{}	\N	2020-10-12 15:36:12.869954
56554	MCY	2020-10-09	41.02	41.02	40.27	40.8	317284	{}	\N	2020-10-12 15:36:12.875075
56555	MCY	2020-10-12	41	41.45	40.81	41.42	62495	{}	\N	2020-10-12 15:36:12.88049
56556	MED	2020-09-28	166.25	167.54	163.24	165.51	172661	{}	\N	2020-10-12 15:36:14.20506
56557	MED	2020-09-29	166.53	166.53	160.91	161.38	212636	{}	\N	2020-10-12 15:36:14.210302
56558	MED	2020-09-30	161.12	167.11	161.12	164.45	326294	{}	\N	2020-10-12 15:36:14.216157
56559	MED	2020-10-01	165	167.54	163.23	165.34	633184	{}	\N	2020-10-12 15:36:14.221032
56560	MED	2020-10-02	163.5	166.44	162.3555	164.55	450902	{}	\N	2020-10-12 15:36:14.229134
56561	MED	2020-10-05	165	167	159.4447	163.75	340026	{}	\N	2020-10-12 15:36:14.235452
56562	MED	2020-10-06	163.45	169.38	163.45	164.84	334221	{}	\N	2020-10-12 15:36:14.240394
56563	MED	2020-10-07	165.15	168	163.63	164.69	374649	{}	\N	2020-10-12 15:36:14.245642
56564	MED	2020-10-08	166.07	166.5	161.91	163.2	263806	{}	\N	2020-10-12 15:36:14.251099
56565	MED	2020-10-09	164.4	166.32	162.94	164.99	264454	{}	\N	2020-10-12 15:36:14.256055
56566	MED	2020-10-12	166	167.11	163.44	163.54	86506	{}	\N	2020-10-12 15:36:14.261155
56567	MEXX	2020-09-28	28.655	28.655	27.6	28.47	3503	{}	\N	2020-10-12 15:36:17.202616
56568	MEXX	2020-09-29	28.2	28.55	28.2	28.23	7419	{}	\N	2020-10-12 15:36:17.209152
56569	MEXX	2020-09-30	29.3	30.979	29.3	30.38	13558	{}	\N	2020-10-12 15:36:17.215269
56570	MEXX	2020-10-01	30.77	30.77	29.3725	29.3725	6180	{}	\N	2020-10-12 15:36:17.221593
56571	MEXX	2020-10-02	29	30.45	29	30.4	6413	{}	\N	2020-10-12 15:36:17.229137
56572	MEXX	2020-10-05	32.005	32.5	31.48	31.7413	3985	{}	\N	2020-10-12 15:36:17.237105
56573	MEXX	2020-10-06	32.01	32.07	30.7036	30.7036	4444	{}	\N	2020-10-12 15:36:17.242969
56574	MEXX	2020-10-07	32.36	32.8	32.27	32.6	7081	{}	\N	2020-10-12 15:36:17.249359
56575	MEXX	2020-10-08	33.19	35.7532	33.19	35.625	15234	{}	\N	2020-10-12 15:36:17.25626
56576	MEXX	2020-10-09	36.78	37.5497	36.0016	37.34	12946	{}	\N	2020-10-12 15:36:17.262056
56577	MEXX	2020-10-12	37.9	37.9	36.6939	37.6535	4762	{}	\N	2020-10-12 15:36:17.268273
56578	MFINL	2020-09-28	25.55	25.55	25.16	25.4	3113	{}	\N	2020-10-12 15:36:18.477325
56579	MFINL	2020-09-29	25.55	25.56	25.29	25.43	7636	{}	\N	2020-10-12 15:36:18.485575
56580	MFINL	2020-09-30	25.02	25.05	24.9158	24.9376	4452	{}	\N	2020-10-12 15:36:18.491646
56581	MFINL	2020-10-01	25	25.1	25	25.1	12843	{}	\N	2020-10-12 15:36:18.497857
56582	MFINL	2020-10-02	25.06	25.1	25.06	25.0805	483	{}	\N	2020-10-12 15:36:18.504117
56583	MFINL	2020-10-05	25.065	25.1	25.065	25.1	2373	{}	\N	2020-10-12 15:36:18.510417
56584	MFINL	2020-10-06	25.02	25.33	24.95	24.97	14195	{}	\N	2020-10-12 15:36:18.51632
56585	MFINL	2020-10-07	25	25.23	24.9999	25.212	5300	{}	\N	2020-10-12 15:36:18.523248
56586	MFINL	2020-10-08	25.1499	25.1499	25.1484	25.1484	301	{}	\N	2020-10-12 15:36:18.529738
56587	MFINL	2020-10-12	25.07	25.1088	25.07	25.07	1701	{}	\N	2020-10-12 15:36:18.53635
56588	MGEE	2020-09-28	63.08	63.69	62.22	63.15	101459	{}	\N	2020-10-12 15:36:19.424492
56589	MGEE	2020-09-29	63.24	63.4	62	62.49	63976	{}	\N	2020-10-12 15:36:19.430727
56590	MGEE	2020-09-30	63.17	63.37	62.29	62.66	97465	{}	\N	2020-10-12 15:36:19.436411
56591	MGEE	2020-10-01	62.65	63.44	62.24	63.31	79476	{}	\N	2020-10-12 15:36:19.443422
56592	MGEE	2020-10-02	62.62	64.51	62.55	63.92	80101	{}	\N	2020-10-12 15:36:19.450205
56593	MGEE	2020-10-05	63.91	64.19	63.2	64.11	74375	{}	\N	2020-10-12 15:36:19.455752
56594	MGEE	2020-10-06	64.07	66.09	63.725	64.8	160945	{}	\N	2020-10-12 15:36:19.461139
56595	MGEE	2020-10-07	65.25	65.48	64.495	65.15	89364	{}	\N	2020-10-12 15:36:19.466099
56596	MGEE	2020-10-08	65.64	66.81	65.55	66.51	68355	{}	\N	2020-10-12 15:36:19.471656
56597	MGEE	2020-10-09	66.21	66.73	65.68	66.31	55929	{}	\N	2020-10-12 15:36:19.478328
56598	MGEE	2020-10-12	66.21	67.17	65.91	66.88	8667	{}	\N	2020-10-12 15:36:19.483825
56599	MGPI	2020-09-28	37.9	39.07	37.52	38.18	99985	{}	\N	2020-10-12 15:36:20.442289
56600	MGPI	2020-09-29	38.19	39.855	37.98	39.44	112728	{}	\N	2020-10-12 15:36:20.449622
56601	MGPI	2020-09-30	39.52	40	39	39.74	114286	{}	\N	2020-10-12 15:36:20.455027
56602	MGPI	2020-10-01	39.5	40.37	39.23	40.31	118521	{}	\N	2020-10-12 15:36:20.460003
56603	MGPI	2020-10-02	40.15	42.54	39.77	42.15	161505	{}	\N	2020-10-12 15:36:20.465655
56604	MGPI	2020-10-05	42.09	42.66	41.39	42.05	123384	{}	\N	2020-10-12 15:36:20.470968
56605	MGPI	2020-10-06	42.21	43.16	41.89	42.11	144945	{}	\N	2020-10-12 15:36:20.476095
56606	MGPI	2020-10-07	42.22	42.55	40.48	40.58	108198	{}	\N	2020-10-12 15:36:20.481086
56607	MGPI	2020-10-08	40.75	41	39.54	40.01	88880	{}	\N	2020-10-12 15:36:20.487569
56608	MGPI	2020-10-09	40.49	41.11	40.05	40.16	68979	{}	\N	2020-10-12 15:36:20.492047
56609	MGPI	2020-10-12	40.26	41.13	40.26	40.58	20189	{}	\N	2020-10-12 15:36:20.49656
56610	QRHC	2020-09-28	1.82	1.87	1.82	1.87	711	{}	\N	2020-10-12 15:36:21.931418
56611	QRHC	2020-09-29	1.87	1.94	1.87	1.94	8331	{}	\N	2020-10-12 15:36:21.937899
56612	QRHC	2020-09-30	1.87	1.95	1.8311	1.9	11194	{}	\N	2020-10-12 15:36:21.944391
56613	QRHC	2020-10-01	1.9	1.93	1.9	1.93	3459	{}	\N	2020-10-12 15:36:21.950621
56614	QRHC	2020-10-02	1.9	1.93	1.9	1.93	1776	{}	\N	2020-10-12 15:36:21.957443
56615	QRHC	2020-10-05	1.95	2	1.9344	1.95	11489	{}	\N	2020-10-12 15:36:21.964122
56616	QRHC	2020-10-06	1.98	2.04	1.98	2.01	17233	{}	\N	2020-10-12 15:36:21.969753
56617	QRHC	2020-10-07	2.04	2.04	1.97	2.04	23420	{}	\N	2020-10-12 15:36:21.975634
56618	QRHC	2020-10-08	2.0185	2.04	2	2.03	5100	{}	\N	2020-10-12 15:36:21.981405
56619	QRHC	2020-10-09	2.0316	2.0316	1.93	1.96	18453	{}	\N	2020-10-12 15:36:21.988948
56620	QRHC	2020-10-12	1.97	2.09	1.9	2.065	85425	{}	\N	2020-10-12 15:36:21.994905
56621	MAYS	2020-10-01	21.893	22.6	21.5	22.6	2397	{}	\N	2020-10-12 15:36:22.921186
56622	MAYS	2020-10-06	22.42	22.42	21.39	21.39	646	{}	\N	2020-10-12 15:36:22.928133
56623	MBI	2020-09-28	6.25	6.36	6.08	6.34	495399	{}	\N	2020-10-12 15:36:23.86061
56624	MBI	2020-09-29	6.29	6.32	5.98	6.06	755409	{}	\N	2020-10-12 15:36:23.868463
56625	MBI	2020-09-30	6.07	6.21	6.04	6.06	868522	{}	\N	2020-10-12 15:36:23.874515
56626	MBI	2020-10-01	6.05	6.16	6.02	6.11	1068210	{}	\N	2020-10-12 15:36:23.879566
56627	MBI	2020-10-02	6.03	6.365	6.03	6.28	775962	{}	\N	2020-10-12 15:36:23.884399
56628	MBI	2020-10-05	6.34	6.555	6.33	6.48	579587	{}	\N	2020-10-12 15:36:23.88958
56629	MBI	2020-10-06	6.57	6.72	6.45	6.47	767824	{}	\N	2020-10-12 15:36:23.895963
56630	MBI	2020-10-07	6.52	6.59	6.4401	6.55	754880	{}	\N	2020-10-12 15:36:23.900617
56631	MBI	2020-10-08	6.63	6.72	6.5	6.64	706882	{}	\N	2020-10-12 15:36:23.905462
56632	MBI	2020-10-09	6.72	6.72	6.54	6.64	596387	{}	\N	2020-10-12 15:36:23.91039
56633	MBI	2020-10-12	6.64	6.765	6.48	6.75	205169	{}	\N	2020-10-12 15:36:23.914789
56634	MBWM	2020-09-28	17.75	18.69	17.65	18.22	29501	{}	\N	2020-10-12 15:36:24.770967
56635	MBWM	2020-09-29	18.03	18.15	17.56	17.8	27062	{}	\N	2020-10-12 15:36:24.777956
56636	MBWM	2020-09-30	17.9	18.43	17.83	18.02	28003	{}	\N	2020-10-12 15:36:24.783044
56637	MBWM	2020-10-01	18.1	18.79	17.85	18.59	46414	{}	\N	2020-10-12 15:36:24.788584
56638	MBWM	2020-10-02	18.25	19.85	18.25	19.6	33606	{}	\N	2020-10-12 15:36:24.794663
56639	MBWM	2020-10-05	20.08	20.1999	19.79	20.04	62605	{}	\N	2020-10-12 15:36:24.800196
56640	MBWM	2020-10-06	20.38	21.05	19.5738	20.3	55249	{}	\N	2020-10-12 15:36:24.805002
56641	MBWM	2020-10-07	20.54	20.85	20.4	20.75	31533	{}	\N	2020-10-12 15:36:24.810852
56642	MBWM	2020-10-08	20.8	21.26	20.63	21.09	21298	{}	\N	2020-10-12 15:36:24.817194
56643	MBWM	2020-10-09	21.29	21.31	20.77	20.9	21464	{}	\N	2020-10-12 15:36:24.822416
56644	MBWM	2020-10-12	20.96	21.14	20.91	21.14	3371	{}	\N	2020-10-12 15:36:24.827916
56645	MCF	2020-09-28	1.26	1.39	1.18	1.38	516156	{}	\N	2020-10-12 15:36:26.055551
56646	MCF	2020-09-29	1.38	1.47	1.38	1.47	322346	{}	\N	2020-10-12 15:36:26.061658
56647	MCF	2020-09-30	1.46	1.48	1.33	1.34	325290	{}	\N	2020-10-12 15:36:26.067185
56648	MCF	2020-10-01	1.35	1.35	1.2	1.27	353479	{}	\N	2020-10-12 15:36:26.072892
56649	MCF	2020-10-02	1.22	1.33	1.21	1.33	190721	{}	\N	2020-10-12 15:36:26.087673
56650	MCF	2020-10-05	1.33	1.35	1.26	1.34	289147	{}	\N	2020-10-12 15:36:26.096374
56651	MCF	2020-10-06	1.37	1.39	1.3	1.31	220628	{}	\N	2020-10-12 15:36:26.105626
56652	MCF	2020-10-07	1.31	1.4	1.29	1.39	242026	{}	\N	2020-10-12 15:36:26.118606
56653	MCF	2020-10-08	1.39	1.6	1.39	1.57	375396	{}	\N	2020-10-12 15:36:26.127353
56654	MCF	2020-10-09	1.58	1.58	1.41	1.45	271546	{}	\N	2020-10-12 15:36:26.13491
56655	MCF	2020-10-12	1.48	1.48	1.45	1.4567	49988	{}	\N	2020-10-12 15:36:26.140103
56656	MCRB	2020-09-28	26.4573	27.1	25.74	26.79	504700	{}	\N	2020-10-12 15:36:27.188367
56657	MCRB	2020-09-29	26.9	29.46	26.5	28.74	1009010	{}	\N	2020-10-12 15:36:27.194257
56658	MCRB	2020-09-30	28.98	29.24	27.935	28.31	647969	{}	\N	2020-10-12 15:36:27.201442
56659	MCRB	2020-10-01	28.4	28.4	26.64	27.7	627878	{}	\N	2020-10-12 15:36:27.207012
56660	MCRB	2020-10-02	27.04	28.58	26.91	27.69	418438	{}	\N	2020-10-12 15:36:27.212982
56661	MCRB	2020-10-05	27.68	29.69	27.68	29.65	768976	{}	\N	2020-10-12 15:36:27.218176
56662	MCRB	2020-10-06	30	30.88	27.9	28.22	644238	{}	\N	2020-10-12 15:36:27.22325
56663	MCRB	2020-10-07	28.52	30.6	28.36	30.38	674198	{}	\N	2020-10-12 15:36:27.228932
56664	MCRB	2020-10-08	31.15	32.44	30.43	31.62	896131	{}	\N	2020-10-12 15:36:27.234092
56665	MCRB	2020-10-09	32.2	32.25	31.3184	31.8	653380	{}	\N	2020-10-12 15:36:27.240792
56666	MCRB	2020-10-12	32	32.92	31.48	32.4	348251	{}	\N	2020-10-12 15:36:27.246908
56667	MD	2020-09-28	16.46	16.87	16.14	16.19	848529	{}	\N	2020-10-12 15:36:28.563044
56668	MD	2020-09-29	16.21	16.21	15.66	16.05	1303501	{}	\N	2020-10-12 15:36:28.570323
56669	MD	2020-09-30	16.01	16.76	16.01	16.28	1314330	{}	\N	2020-10-12 15:36:28.576252
56670	MD	2020-10-01	16.29	16.52	15.91	16.28	1309403	{}	\N	2020-10-12 15:36:28.582164
56671	MD	2020-10-02	15.85	16.785	15.75	16.65	890947	{}	\N	2020-10-12 15:36:28.58745
56672	MD	2020-10-05	16.89	17.34	16.89	17.27	1082887	{}	\N	2020-10-12 15:36:28.592203
56673	MD	2020-10-06	17.19	17.37	16.13	16.26	4046112	{}	\N	2020-10-12 15:36:28.598078
56674	MD	2020-10-07	16.41	16.84	16.28	16.42	2260527	{}	\N	2020-10-12 15:36:28.603182
56675	MD	2020-10-08	16.59	16.795	16.3	16.72	1101880	{}	\N	2020-10-12 15:36:28.607931
56676	MD	2020-10-09	16.93	17.105	16.23	16.56	871537	{}	\N	2020-10-12 15:36:28.614101
56677	MD	2020-10-12	16.57	16.64	16.22	16.48	266812	{}	\N	2020-10-12 15:36:28.620369
56678	MDU	2020-09-28	22.6	22.68	22.31	22.45	1018597	{}	\N	2020-10-12 15:36:29.846186
56679	MDU	2020-09-29	22.45	22.59	22.02	22.21	1709666	{}	\N	2020-10-12 15:36:29.853327
56680	MDU	2020-09-30	22.28	22.58	22.21	22.5	2738083	{}	\N	2020-10-12 15:36:29.859953
56681	MDU	2020-10-01	22.5	22.68	22.13	22.41	1572403	{}	\N	2020-10-12 15:36:29.867098
56682	MDU	2020-10-02	21.98	22.915	21.98	22.78	1525982	{}	\N	2020-10-12 15:36:29.873475
56683	MDU	2020-10-05	22.91	23.07	22.735	22.96	1212755	{}	\N	2020-10-12 15:36:29.879071
56684	MDU	2020-10-06	23.3	23.78	22.97	23.08	2225460	{}	\N	2020-10-12 15:36:29.885177
56685	MDU	2020-10-07	23.28	23.37	22.91	23.1	2019530	{}	\N	2020-10-12 15:36:29.890956
56686	MDU	2020-10-08	23.28	23.74	23.18	23.68	2489234	{}	\N	2020-10-12 15:36:29.896618
56687	MDU	2020-10-09	23.81	23.93	23.175	23.27	1572269	{}	\N	2020-10-12 15:36:29.903262
56688	MDU	2020-10-12	23.25	23.668	23.21	23.589	525316	{}	\N	2020-10-12 15:36:29.908867
56689	METC	2020-09-28	3.37	3.38	3.25	3.35	47676	{}	\N	2020-10-12 15:36:31.319203
56690	METC	2020-09-29	3.35	3.68	3.3	3.61	99478	{}	\N	2020-10-12 15:36:31.32621
56691	METC	2020-09-30	3.68	3.69	3.43	3.5	45682	{}	\N	2020-10-12 15:36:31.332367
56692	METC	2020-10-01	3.52	3.61	3.42	3.55	38702	{}	\N	2020-10-12 15:36:31.33803
56693	METC	2020-10-02	3.48	3.8	3.474	3.62	116619	{}	\N	2020-10-12 15:36:31.344053
56694	METC	2020-10-05	3.65	3.75	3.6101	3.69	70495	{}	\N	2020-10-12 15:36:31.349077
56695	METC	2020-10-06	3.71	3.98	3.65	3.75	119285	{}	\N	2020-10-12 15:36:31.354872
56696	METC	2020-10-07	3.81	3.81	3.67	3.68	89001	{}	\N	2020-10-12 15:36:31.360133
56697	METC	2020-10-08	3.8	3.92	3.71	3.71	66220	{}	\N	2020-10-12 15:36:31.3651
56698	METC	2020-10-09	3.8	3.8	3.475	3.51	73499	{}	\N	2020-10-12 15:36:31.373281
56699	METC	2020-10-12	3.71	3.71	3.495	3.58	21024	{}	\N	2020-10-12 15:36:31.378588
56700	MFM	2020-09-28	6.37	6.43	6.37	6.41	47971	{}	\N	2020-10-12 15:36:32.308429
56701	MFM	2020-09-29	6.4	6.47	6.4	6.46	91758	{}	\N	2020-10-12 15:36:32.314282
56702	MFM	2020-09-30	6.5	6.5	6.43	6.43	140700	{}	\N	2020-10-12 15:36:32.321398
56703	MFM	2020-10-01	6.44	6.5	6.42	6.49	138717	{}	\N	2020-10-12 15:36:32.327714
56704	MFM	2020-10-02	6.49	6.54	6.445	6.5	84318	{}	\N	2020-10-12 15:36:32.333822
56705	MFM	2020-10-05	6.53	6.53	6.47	6.49	91549	{}	\N	2020-10-12 15:36:32.339382
56706	MFM	2020-10-06	6.47	6.51	6.45	6.46	131766	{}	\N	2020-10-12 15:36:32.345462
56707	MFM	2020-10-07	6.5	6.54	6.42	6.44	135854	{}	\N	2020-10-12 15:36:32.351584
56708	MFM	2020-10-08	6.43	6.47	6.43	6.45	86948	{}	\N	2020-10-12 15:36:32.357281
56709	MFM	2020-10-09	6.45	6.4848	6.45	6.47	216474	{}	\N	2020-10-12 15:36:32.363607
56710	MFM	2020-10-12	6.53	6.65	6.51	6.57	162072	{}	\N	2020-10-12 15:36:32.370208
56711	MGIC	2020-09-28	12.45	12.925	12.45	12.8	57062	{}	\N	2020-10-12 15:36:33.739795
56712	MGIC	2020-09-29	12.76	12.97	12.63	12.89	24828	{}	\N	2020-10-12 15:36:33.746939
56713	MGIC	2020-09-30	13.27	13.33	13.07	13.13	36404	{}	\N	2020-10-12 15:36:33.752642
56714	MGIC	2020-10-01	13.66	13.7	13.3981	13.62	36610	{}	\N	2020-10-12 15:36:33.758167
56715	MGIC	2020-10-02	13.37	13.56	13.2101	13.45	34925	{}	\N	2020-10-12 15:36:33.764397
56716	MGIC	2020-10-05	13.68	14	13.6218	14	41665	{}	\N	2020-10-12 15:36:33.771566
56717	MGIC	2020-10-06	14.12	14.359	13.75	13.91	29173	{}	\N	2020-10-12 15:36:33.777427
56718	MGIC	2020-10-07	14.16	14.16	13.76	13.99	62641	{}	\N	2020-10-12 15:36:33.783061
56719	MGIC	2020-10-08	14.014	14.21	13.94	14.13	25053	{}	\N	2020-10-12 15:36:33.789672
56720	MGIC	2020-10-09	14.28	14.4	14.16	14.29	22865	{}	\N	2020-10-12 15:36:33.796472
56721	MGIC	2020-10-12	14.52	14.6	14.49	14.59	15972	{}	\N	2020-10-12 15:36:33.802032
56722	MHH	2020-09-28	18.1	18.99	18.1	18.74	16423	{}	\N	2020-10-12 15:36:34.748226
56723	MHH	2020-09-29	18.72	19.01	18.5	18.83	13334	{}	\N	2020-10-12 15:36:34.755359
56724	MHH	2020-09-30	18.66	18.79	18.01	18.01	10959	{}	\N	2020-10-12 15:36:34.761713
56725	MHH	2020-10-01	18.19	18.95	17.8392	18.17	11939	{}	\N	2020-10-12 15:36:34.767002
56726	MHH	2020-10-02	17.68	18.28	17.1547	17.77	19907	{}	\N	2020-10-12 15:36:34.773098
56727	MHH	2020-10-05	17.75	19	17.75	18.99	16524	{}	\N	2020-10-12 15:36:34.778553
56728	MHH	2020-10-06	19	19.895	19	19.57	17833	{}	\N	2020-10-12 15:36:34.78422
56729	MHH	2020-10-07	19.59	21.82	19.59	21.65	25073	{}	\N	2020-10-12 15:36:34.790473
56730	MHH	2020-10-08	22.14	22.98	21.325	21.45	22176	{}	\N	2020-10-12 15:36:34.795686
56731	MHH	2020-10-09	21.55	21.875	20.55	20.85	17251	{}	\N	2020-10-12 15:36:34.801022
56732	MHH	2020-10-12	21.01	21.01	20.5086	20.7567	4837	{}	\N	2020-10-12 15:36:34.806663
56733	MILN	2020-09-28	31.09	31.14	30.88	31.095	16837	{}	\N	2020-10-12 15:36:35.691556
56734	MILN	2020-09-29	31.08	31.17	30.93	31.055	31917	{}	\N	2020-10-12 15:36:35.697153
56735	MILN	2020-09-30	31.19	31.425	30.9	31.215	11824	{}	\N	2020-10-12 15:36:35.702258
56736	MILN	2020-10-01	31.53	31.96	31.53	31.96	15720	{}	\N	2020-10-12 15:36:35.708103
56737	MILN	2020-10-02	31.3	31.74	31.3	31.71	14876	{}	\N	2020-10-12 15:36:35.713487
56738	MILN	2020-10-05	31.97	32.2367	31.87	32.19	14780	{}	\N	2020-10-12 15:36:35.720176
56739	MILN	2020-10-06	32.4	32.4	31.57	31.6	34762	{}	\N	2020-10-12 15:36:35.725815
56740	MILN	2020-10-07	32.06	32.3	32	32.2594	23180	{}	\N	2020-10-12 15:36:35.730431
56741	MILN	2020-10-08	32.49	32.5	32.3	32.4985	11016	{}	\N	2020-10-12 15:36:35.735399
56742	MILN	2020-10-09	32.54	32.812	32.54	32.7584	24773	{}	\N	2020-10-12 15:36:35.740656
56743	MILN	2020-10-12	33.01	33.075	32.9003	33.03	14418	{}	\N	2020-10-12 15:36:35.745635
56744	MSN	2020-09-28	0.6803	0.7128	0.6803	0.69013	44077	{}	\N	2020-10-12 15:36:36.969868
56745	MSN	2020-09-29	0.69	0.7195	0.6665	0.683083	57635	{}	\N	2020-10-12 15:36:36.975714
56746	MSN	2020-09-30	0.7172	0.7172	0.6781	0.6799	24171	{}	\N	2020-10-12 15:36:36.979773
56747	MSN	2020-10-01	0.707	0.7139	0.6782	0.7049	27139	{}	\N	2020-10-12 15:36:36.983804
56748	MSN	2020-10-02	0.7041	0.7184	0.7002	0.7004	8657	{}	\N	2020-10-12 15:36:36.989776
56749	MSN	2020-10-05	0.7184	0.7184	0.7002	0.7002	11836	{}	\N	2020-10-12 15:36:36.99563
56750	MSN	2020-10-06	0.7003	0.720101	0.6655	0.7	18091	{}	\N	2020-10-12 15:36:37.000768
56751	MSN	2020-10-07	0.7	0.7294	0.7	0.7138	8847	{}	\N	2020-10-12 15:36:37.007439
56752	MSN	2020-10-08	0.7	0.7098	0.7	0.7095	6536	{}	\N	2020-10-12 15:36:37.01235
56753	MSN	2020-10-09	0.7095	0.7095	0.677	0.7	5880	{}	\N	2020-10-12 15:36:37.017067
56754	MSN	2020-10-12	0.7	0.88	0.6811	0.8216	774613	{}	\N	2020-10-12 15:36:37.021664
56755	MBT	2020-09-28	8.78	8.85	8.73	8.79	2056919	{}	\N	2020-10-12 15:36:38.048281
56756	MBT	2020-09-29	8.76	8.77	8.53	8.56	6284570	{}	\N	2020-10-12 15:36:38.05549
56757	MBT	2020-09-30	8.71	8.805	8.61	8.73	5465699	{}	\N	2020-10-12 15:36:38.067332
56758	MBT	2020-10-01	8.74	8.785	8.68	8.73	2742069	{}	\N	2020-10-12 15:36:38.075884
56759	MBT	2020-10-02	8.62	8.685	8.6	8.64	3608575	{}	\N	2020-10-12 15:36:38.083609
56760	MBT	2020-10-05	8.7	8.8	8.66	8.74	12825940	{}	\N	2020-10-12 15:36:38.097183
56761	MBT	2020-10-06	8.79	8.835	8.61	8.64	6936978	{}	\N	2020-10-12 15:36:38.104267
56762	MBT	2020-10-07	8.54	8.58	8.38	8.39	7182586	{}	\N	2020-10-12 15:36:38.111164
56763	MBT	2020-10-08	8.45	8.54	8.37	8.54	8810340	{}	\N	2020-10-12 15:36:38.115463
56764	MBT	2020-10-09	8.57	8.625	8.495	8.54	6092131	{}	\N	2020-10-12 15:36:38.119784
56765	MBT	2020-10-12	8.55	8.6	8.515	8.585	904877	{}	\N	2020-10-12 15:36:38.125364
56766	MCBC	2020-09-28	6.4	6.6689	6.391	6.65	64204	{}	\N	2020-10-12 15:36:39.251155
56767	MCBC	2020-09-29	6.55	6.568	6.43	6.54	79234	{}	\N	2020-10-12 15:36:39.258721
56768	MCBC	2020-09-30	6.5201	6.6399	6.46	6.53	59923	{}	\N	2020-10-12 15:36:39.265128
56769	MCBC	2020-10-01	6.53	6.65	6.452	6.62	67675	{}	\N	2020-10-12 15:36:39.271212
56770	MCBC	2020-10-02	6.47	6.85	6.47	6.78	81873	{}	\N	2020-10-12 15:36:39.278344
56771	MCBC	2020-10-05	6.9	7.08	6.82	7.06	45347	{}	\N	2020-10-12 15:36:39.284537
56772	MCBC	2020-10-06	7.16	7.39	7.07	7.12	50266	{}	\N	2020-10-12 15:36:39.28987
56773	MCBC	2020-10-07	7.19	7.32	7.15	7.28	46314	{}	\N	2020-10-12 15:36:39.297408
56774	MCBC	2020-10-08	7.2	7.37	7.11	7.29	51195	{}	\N	2020-10-12 15:36:39.303283
56775	MCBC	2020-10-09	7.28	7.35	7.24	7.33	22490	{}	\N	2020-10-12 15:36:39.309059
56776	MCBC	2020-10-12	7.29	7.32	7.231	7.315	4131	{}	\N	2020-10-12 15:36:39.314494
56777	MCHX	2020-09-28	2.09	2.1	2.06	2.1	234012	{}	\N	2020-10-12 15:36:46.717601
56778	MCHX	2020-09-29	2.1	2.1	2.09	2.095	63978	{}	\N	2020-10-12 15:36:46.723517
56779	MCHX	2020-09-30	2.1	2.12	2.0935	2.12	213259	{}	\N	2020-10-12 15:36:46.727899
56780	MCHX	2020-10-01	2.14	2.14	2.12	2.14	131486	{}	\N	2020-10-12 15:36:46.733153
56781	MCHX	2020-10-02	2.11	2.15	2.11	2.14	135334	{}	\N	2020-10-12 15:36:46.739103
56782	MCHX	2020-10-05	2.14	2.15	2.14	2.14	98290	{}	\N	2020-10-12 15:36:46.744692
56783	MCHX	2020-10-06	2.14	2.175	2.13	2.15	125359	{}	\N	2020-10-12 15:36:46.750884
56784	MCHX	2020-10-07	2.16	2.16	2.13	2.16	63049	{}	\N	2020-10-12 15:36:46.758605
56785	MCHX	2020-10-08	2.17	2.18	2.1	2.14	70237	{}	\N	2020-10-12 15:36:46.76542
56786	MCHX	2020-10-09	2.21	2.24	2.13	2.16	108466	{}	\N	2020-10-12 15:36:46.771865
56787	MCHX	2020-10-12	2.17	2.17	2.12	2.1327	27143	{}	\N	2020-10-12 15:36:46.77825
56788	MDCA	2020-09-28	1.78	1.8	1.75	1.78	56431	{}	\N	2020-10-12 15:36:48.386954
56789	MDCA	2020-09-29	1.75	1.7656	1.67	1.68	95719	{}	\N	2020-10-12 15:36:48.395331
56790	MDCA	2020-09-30	1.68	1.8082	1.6	1.6	877256	{}	\N	2020-10-12 15:36:48.401133
56791	MDCA	2020-10-01	1.61	1.75	1.6	1.7	300025	{}	\N	2020-10-12 15:36:48.407683
56792	MDCA	2020-10-02	1.66	1.75	1.64	1.75	43758	{}	\N	2020-10-12 15:36:48.41343
56793	MDCA	2020-10-05	1.75	1.85	1.75	1.8	57732	{}	\N	2020-10-12 15:36:48.420365
56794	MDCA	2020-10-06	1.95	2.17	1.9382	2.05	226808	{}	\N	2020-10-12 15:36:48.426385
56795	MDCA	2020-10-07	2.09	2.14	2.0211	2.14	135020	{}	\N	2020-10-12 15:36:48.433999
56796	MDCA	2020-10-08	2.18	2.23	2.05	2.08	129096	{}	\N	2020-10-12 15:36:48.440689
56797	MDCA	2020-10-09	2.07	2.1688	2.05	2.14	76910	{}	\N	2020-10-12 15:36:48.446386
56798	MDCA	2020-10-12	2.16	2.16	2.06	2.115	10917	{}	\N	2020-10-12 15:36:48.453061
56799	MDLQ	2020-09-28	6.25	6.42	6.25	6.42	3203	{}	\N	2020-10-12 15:36:49.438126
56800	MDLQ	2020-09-29	6.42	7.5	6.34	7.15	90504	{}	\N	2020-10-12 15:36:49.44428
56801	MDLQ	2020-09-30	7.35	7.7	7.2722	7.42	14964	{}	\N	2020-10-12 15:36:49.449319
56802	MDLQ	2020-10-01	7.37	7.78	7.2544	7.43	15460	{}	\N	2020-10-12 15:36:49.455092
56803	MDLQ	2020-10-02	7.43	7.43	6.9	7.185	29466	{}	\N	2020-10-12 15:36:49.461434
56804	MDLQ	2020-10-05	7.23	8	7.205	7.75	57948	{}	\N	2020-10-12 15:36:49.467864
56805	MDLQ	2020-10-06	7.99	7.99	7.65	7.92	26310	{}	\N	2020-10-12 15:36:49.474687
56806	MDLQ	2020-10-07	7.9067	7.99	7.8158	7.99	25090	{}	\N	2020-10-12 15:36:49.4806
56807	MDLQ	2020-10-08	7.98	8.612	7.92	8.612	77684	{}	\N	2020-10-12 15:36:49.48708
56808	MDLQ	2020-10-09	9	10.05	7.6	8.29	69986	{}	\N	2020-10-12 15:36:49.492985
56809	MDLQ	2020-10-12	10.64	10.64	10.5	10.57	20120	{}	\N	2020-10-12 15:36:49.49871
56810	MDT	2020-09-28	104.88	105.39	103.46	103.63	3521038	{}	\N	2020-10-12 15:36:51.396267
56811	MDT	2020-09-29	105.12	105.24	102.09	102.69	8392990	{}	\N	2020-10-12 15:36:51.402557
56812	MDT	2020-09-30	102.82	104.87	102.5	103.92	12991896	{}	\N	2020-10-12 15:36:51.409082
56813	MDT	2020-10-01	103.48	104.95	102.02	102.3	10246695	{}	\N	2020-10-12 15:36:51.41529
56814	MDT	2020-10-02	101	103.11	101	101.96	8265023	{}	\N	2020-10-12 15:36:51.421343
56815	MDT	2020-10-05	103.2	104.12	102.66	103.71	4901633	{}	\N	2020-10-12 15:36:51.427885
56816	MDT	2020-10-06	103.69	105.3342	103.07	103.38	5877377	{}	\N	2020-10-12 15:36:51.434808
56817	MDT	2020-10-07	104.33	106.31	103.92	105.66	6930222	{}	\N	2020-10-12 15:36:51.441289
56818	MDT	2020-10-08	110	110	107.73	107.8	11427945	{}	\N	2020-10-12 15:36:51.447072
56819	MDT	2020-10-09	108.5	109.06	107.97	108.68	5561492	{}	\N	2020-10-12 15:36:51.452964
56820	MDT	2020-10-12	109.63	109.94	108.1	109	1963973	{}	\N	2020-10-12 15:36:51.459445
56821	MESO	2020-09-28	18.75	19.45	18.2	19.44	1087180	{}	\N	2020-10-12 15:36:52.383965
56822	MESO	2020-09-29	19.39	19.44	18.6	19.19	612074	{}	\N	2020-10-12 15:36:52.390535
56823	MESO	2020-09-30	18.67	18.8	18.51	18.585	67348	{}	\N	2020-10-12 15:36:52.397071
56824	MESO	2020-10-02	11.9	13.7399	11.09	12.03	4576229	{}	\N	2020-10-12 15:36:52.403127
56825	MESO	2020-10-05	12.71	13.5	12.52	12.8	1594425	{}	\N	2020-10-12 15:36:52.410143
56826	MESO	2020-10-06	12.69	12.69	11.91	12.11	955959	{}	\N	2020-10-12 15:36:52.415721
56827	MESO	2020-10-07	12.15	12.22	11.93	12.09	645486	{}	\N	2020-10-12 15:36:52.421273
56828	MESO	2020-10-08	11.96	12.11	11.84	12.11	560447	{}	\N	2020-10-12 15:36:52.427428
56829	MESO	2020-10-09	12.06	12.3	11.99	12.11	556085	{}	\N	2020-10-12 15:36:52.433325
56830	MESO	2020-10-12	12.06	12.08	11.8385	11.93	298151	{}	\N	2020-10-12 15:36:52.438935
56831	MFGP	2020-09-28	3.36	3.445	3.3219	3.39	781239	{}	\N	2020-10-12 15:36:58.788399
56832	MFGP	2020-09-29	3.35	3.37	3.22	3.3	1874782	{}	\N	2020-10-12 15:36:58.794294
56833	MFGP	2020-09-30	3.26	3.33	3.21	3.22	1660077	{}	\N	2020-10-12 15:36:58.801355
56834	MFGP	2020-10-01	3.13	3.23	3.11	3.19	2543259	{}	\N	2020-10-12 15:36:58.806684
56835	MFGP	2020-10-02	3.11	3.255	3.11	3.21	2132004	{}	\N	2020-10-12 15:36:58.813502
56836	MFGP	2020-10-05	3.35	3.5	3.35	3.45	3640573	{}	\N	2020-10-12 15:36:58.819432
56837	MFGP	2020-10-06	3.46	3.57	3.445	3.45	3033876	{}	\N	2020-10-12 15:36:58.824522
56838	MFGP	2020-10-07	3.52	3.55	3.41	3.48	1927996	{}	\N	2020-10-12 15:36:58.829807
56839	MFGP	2020-10-08	3.53	3.63	3.515	3.55	1766746	{}	\N	2020-10-12 15:36:58.835981
56840	MFGP	2020-10-09	3.56	3.615	3.56	3.57	750318	{}	\N	2020-10-12 15:36:58.841836
56841	MFGP	2020-10-12	3.57	3.605	3.565	3.6	734291	{}	\N	2020-10-12 15:36:58.847873
56842	MFV	2020-09-28	5.33	5.33	5.27	5.28	8713	{}	\N	2020-10-12 15:37:00.256082
56843	MFV	2020-09-29	5.24	5.35	5.2	5.31	45084	{}	\N	2020-10-12 15:37:00.261833
56844	MFV	2020-09-30	5.46	5.46	5.31	5.3101	38898	{}	\N	2020-10-12 15:37:00.268788
56845	MFV	2020-10-01	5.24	5.33	5.24	5.3284	24978	{}	\N	2020-10-12 15:37:00.274687
56846	MFV	2020-10-02	5.235	5.33	5.235	5.29	3554	{}	\N	2020-10-12 15:37:00.280714
56847	MFV	2020-10-05	5.34	5.36	5.3301	5.3599	7056	{}	\N	2020-10-12 15:37:00.287301
56848	MFV	2020-10-06	5.37	5.37	5.28	5.28	58893	{}	\N	2020-10-12 15:37:00.293978
56849	MFV	2020-10-07	5.39	5.39	5.325	5.3316	21962	{}	\N	2020-10-12 15:37:00.299723
56850	MFV	2020-10-08	5.4	5.44	5.3501	5.38	28546	{}	\N	2020-10-12 15:37:00.306369
56851	MFV	2020-10-09	5.42	5.45	5.3801	5.44	16258	{}	\N	2020-10-12 15:37:00.312658
56852	MFV	2020-10-12	5.39	5.46	5.39	5.44	3120	{}	\N	2020-10-12 15:37:00.318657
56853	MGLN	2020-09-28	73.82	75.58	73.17	73.78	145316	{}	\N	2020-10-12 15:37:02.310571
56854	MGLN	2020-09-29	74.57	74.8	73.1	74.71	99498	{}	\N	2020-10-12 15:37:02.319494
56855	MGLN	2020-09-30	74.8	76.935	74.8	75.78	173543	{}	\N	2020-10-12 15:37:02.324079
56856	MGLN	2020-10-01	76.31	76.39	74.21	76.05	81680	{}	\N	2020-10-12 15:37:02.329332
56857	MGLN	2020-10-02	74.95	78.13	74.95	77.83	100953	{}	\N	2020-10-12 15:37:02.335647
56858	MGLN	2020-10-05	78	78.9	77.415	78.06	203948	{}	\N	2020-10-12 15:37:02.340803
56859	MGLN	2020-10-06	78.19	78.5	77.06	77.06	172482	{}	\N	2020-10-12 15:37:02.345868
56860	MGLN	2020-10-07	77.77	78.53	77.31	77.9	121866	{}	\N	2020-10-12 15:37:02.352488
56861	MGLN	2020-10-08	78.31	78.31	77.1	77.6	66845	{}	\N	2020-10-12 15:37:02.358889
56862	MGLN	2020-10-09	78.16	78.23	77.52	77.73	73782	{}	\N	2020-10-12 15:37:02.364206
56863	MGLN	2020-10-12	77.81	79.3	77.81	79.195	32371	{}	\N	2020-10-12 15:37:02.370199
56864	MHF	2020-09-28	7.37	7.45	7.3181	7.44	40595	{}	\N	2020-10-12 15:37:03.661207
56865	MHF	2020-09-29	7.47	7.47	7.42	7.42	47456	{}	\N	2020-10-12 15:37:03.666537
56866	MHF	2020-09-30	7.43	7.45	7.415	7.45	43762	{}	\N	2020-10-12 15:37:03.671968
56867	MHF	2020-10-01	7.49	7.49	7.45	7.458	23817	{}	\N	2020-10-12 15:37:03.676038
56868	MHF	2020-10-02	7.47	7.5	7.45	7.4746	60398	{}	\N	2020-10-12 15:37:03.680553
56869	MHF	2020-10-05	7.55	7.55	7.37	7.37	78922	{}	\N	2020-10-12 15:37:03.685917
56870	MHF	2020-10-06	7.32	7.41	7.29	7.29	107486	{}	\N	2020-10-12 15:37:03.690803
56871	MHF	2020-10-07	7.32	7.46	7.28	7.28	69936	{}	\N	2020-10-12 15:37:03.695633
56872	MHF	2020-10-08	7.32	7.35	7.26	7.29	77170	{}	\N	2020-10-12 15:37:03.70023
56873	MHF	2020-10-09	7.28	7.3203	7.27	7.27	52970	{}	\N	2020-10-12 15:37:03.705283
56874	MHF	2020-10-12	7.31	7.31	7.2968	7.2968	3200	{}	\N	2020-10-12 15:37:03.710265
56875	MHN	2020-09-28	13.64	13.72	13.56	13.56	42374	{}	\N	2020-10-12 15:37:04.681755
56876	MHN	2020-09-29	13.63	13.63	13.52	13.52	50334	{}	\N	2020-10-12 15:37:04.687607
56877	MHN	2020-09-30	13.54	13.544	13.45	13.46	54682	{}	\N	2020-10-12 15:37:04.692409
56878	MHN	2020-10-01	13.5	13.52	13.46	13.48	76392	{}	\N	2020-10-12 15:37:04.696765
56879	MHN	2020-10-02	13.43	13.58	13.43	13.57	33564	{}	\N	2020-10-12 15:37:04.70325
56880	MHN	2020-10-05	13.59	13.61	13.52	13.54	39306	{}	\N	2020-10-12 15:37:04.708006
56881	MHN	2020-10-06	13.57	13.59	13.48	13.5	58054	{}	\N	2020-10-12 15:37:04.712488
56882	MHN	2020-10-07	13.5	13.57	13.5	13.54	168880	{}	\N	2020-10-12 15:37:04.716904
56883	MHN	2020-10-08	13.54	13.57	13.49	13.57	110378	{}	\N	2020-10-12 15:37:04.721978
56884	MHN	2020-10-09	13.57	13.5867	13.5	13.51	87990	{}	\N	2020-10-12 15:37:04.726502
56885	MHN	2020-10-12	13.62	13.62	13.5138	13.5255	16374	{}	\N	2020-10-12 15:37:04.73111
56886	MBRX	2020-09-28	0.800999	0.800999	0.77	0.7998	267033	{}	\N	2020-10-12 15:37:05.986864
56887	MBRX	2020-09-29	0.8568	0.9	0.78	0.8	1426187	{}	\N	2020-10-12 15:37:05.994313
56888	MBRX	2020-09-30	0.79	0.82	0.79	0.8127	254071	{}	\N	2020-10-12 15:37:05.999633
56889	MBRX	2020-10-01	0.8401	0.867	0.82	0.83	985979	{}	\N	2020-10-12 15:37:06.004931
56890	MBRX	2020-10-02	0.8104	0.8197	0.791	0.8048	334066	{}	\N	2020-10-12 15:37:06.009971
56891	MBRX	2020-10-05	0.8104	0.819	0.8011	0.8104	250325	{}	\N	2020-10-12 15:37:06.014526
56892	MBRX	2020-10-06	0.88	0.89	0.8105	0.8149	930581	{}	\N	2020-10-12 15:37:06.01937
56893	MBRX	2020-10-07	0.8028	0.83	0.8	0.8258	357280	{}	\N	2020-10-12 15:37:06.024693
56894	MBRX	2020-10-08	0.82	0.8448	0.815	0.84	340174	{}	\N	2020-10-12 15:37:06.029941
56895	MBRX	2020-10-09	0.84	0.86	0.83	0.8335	280187	{}	\N	2020-10-12 15:37:06.035415
56896	MBRX	2020-10-12	0.8325	0.84	0.825	0.8321	85730	{}	\N	2020-10-12 15:37:06.040731
56897	MCFT	2020-09-28	17	17.25	16.42	17.1	362584	{}	\N	2020-10-12 15:37:08.126879
56898	MCFT	2020-09-29	17.09	17.6	16.97	17.01	131335	{}	\N	2020-10-12 15:37:08.132905
56899	MCFT	2020-09-30	17.05	17.85	17.05	17.49	271195	{}	\N	2020-10-12 15:37:08.138672
56900	MCFT	2020-10-01	17.69	18.52	17.4	18.37	221036	{}	\N	2020-10-12 15:37:08.14385
56901	MCFT	2020-10-02	18.01	19.06	17.9	18.9	290730	{}	\N	2020-10-12 15:37:08.149834
56902	MCFT	2020-10-05	19	19.92	19	19.78	213449	{}	\N	2020-10-12 15:37:08.154926
56903	MCFT	2020-10-06	19.5998	20.38	19.29	19.41	153664	{}	\N	2020-10-12 15:37:08.161264
56904	MCFT	2020-10-07	19.68	20.41	19.58	19.79	121328	{}	\N	2020-10-12 15:37:08.166889
56905	MCFT	2020-10-08	19.9	20.3	19.1	19.27	121797	{}	\N	2020-10-12 15:37:08.172107
56906	MCFT	2020-10-09	19.64	20.28	19.53	20	127671	{}	\N	2020-10-12 15:37:08.176815
56907	MCFT	2020-10-12	19.823	20.2	19.69	19.97	23111	{}	\N	2020-10-12 15:37:08.180898
56908	MCRI	2020-09-28	46.46	47.495	45.11	45.99	128586	{}	\N	2020-10-12 15:37:09.549686
56909	MCRI	2020-09-29	45.69	45.69	43.6	44.12	67526	{}	\N	2020-10-12 15:37:09.554763
56910	MCRI	2020-09-30	44.44	46	44.3	44.6	99558	{}	\N	2020-10-12 15:37:09.559845
56911	MCRI	2020-10-01	44.8	45.21	44.24	45	35254	{}	\N	2020-10-12 15:37:09.564307
56912	MCRI	2020-10-02	43.8	45.91	43.8	45.62	69458	{}	\N	2020-10-12 15:37:09.569633
56913	MCRI	2020-10-05	46.62	49.4899	45	45.84	55974	{}	\N	2020-10-12 15:37:09.575932
56914	MCRI	2020-10-06	46.985	47.34	45.42	45.62	76589	{}	\N	2020-10-12 15:37:09.581763
56915	MCRI	2020-10-07	46.29	46.98	45.5	46.43	56690	{}	\N	2020-10-12 15:37:09.586804
56916	MCRI	2020-10-08	46.96	46.96	45.57	46.34	46737	{}	\N	2020-10-12 15:37:09.592326
56917	MCRI	2020-10-09	46.5	47.0312	46.02	46.49	43713	{}	\N	2020-10-12 15:37:09.59715
56918	MCRI	2020-10-12	45.65	46.96	45.65	46.96	12702	{}	\N	2020-10-12 15:37:09.601702
56919	MDB	2020-09-28	233	236.47	225.25	230.95	877807	{}	\N	2020-10-12 15:37:10.608069
56920	MDB	2020-09-29	230.95	233.89	226.34	232.09	538355	{}	\N	2020-10-12 15:37:10.615972
56921	MDB	2020-09-30	230.31	236.5	228.12	231.51	530548	{}	\N	2020-10-12 15:37:10.62095
56922	MDB	2020-10-01	232.23	237.93	229.51	235.78	516427	{}	\N	2020-10-12 15:37:10.624936
56923	MDB	2020-10-02	224.47	237.8	224.23	234.32	693200	{}	\N	2020-10-12 15:37:10.630518
56924	MDB	2020-10-05	234.21	239.89	232.02	238.69	649265	{}	\N	2020-10-12 15:37:10.634562
56925	MDB	2020-10-06	236	246.72	235.97	246.41	1336632	{}	\N	2020-10-12 15:37:10.638549
56926	MDB	2020-10-07	248.96	259.627	248.96	253.11	1122285	{}	\N	2020-10-12 15:37:10.642727
56927	MDB	2020-10-08	255.01	257.49	248.81	252.86	603124	{}	\N	2020-10-12 15:37:10.646658
56928	MDB	2020-10-09	252.56	262.66	252.56	259.07	576161	{}	\N	2020-10-12 15:37:10.650621
56929	MDB	2020-10-12	265	268.05	261.66	264.765	175515	{}	\N	2020-10-12 15:37:10.655207
56930	MDP	2020-09-28	12.86	13.06	12.56	12.74	441016	{}	\N	2020-10-12 15:37:12.20896
56931	MDP	2020-09-29	12.68	12.94	12.22	12.48	1198908	{}	\N	2020-10-12 15:37:12.214858
56932	MDP	2020-09-30	12.59	13.25	12.59	13.12	1162837	{}	\N	2020-10-12 15:37:12.219223
56933	MDP	2020-10-01	13.02	13.28	12.6	12.89	1137954	{}	\N	2020-10-12 15:37:12.223533
56934	MDP	2020-10-02	12.55	13.18	12.47	13.15	712297	{}	\N	2020-10-12 15:37:12.228612
56935	MDP	2020-10-05	13.39	13.54	13.04	13.12	573052	{}	\N	2020-10-12 15:37:12.233704
56936	MDP	2020-10-06	13.33	13.35	12.8352	12.9	1495841	{}	\N	2020-10-12 15:37:12.238087
56937	MDP	2020-10-07	13.05	13.15	12.81	12.83	699199	{}	\N	2020-10-12 15:37:12.244376
56938	MDP	2020-10-08	13.01	13.34	12.94	13.16	761084	{}	\N	2020-10-12 15:37:12.249769
56939	MDP	2020-10-09	13.25	13.41	12.83	12.86	639320	{}	\N	2020-10-12 15:37:12.253784
56940	MDP	2020-10-12	12.72	12.73	12.39	12.55	180829	{}	\N	2020-10-12 15:37:12.258456
56941	MDY	2020-09-28	335.09	339.56	335.09	338.69	656765	{}	\N	2020-10-12 15:37:13.261229
56942	MDY	2020-09-29	338.49	339.73	334.6	336.66	544241	{}	\N	2020-10-12 15:37:13.266677
56943	MDY	2020-09-30	337.72	342.3	336.3	338.83	705694	{}	\N	2020-10-12 15:37:13.27092
56944	MDY	2020-10-01	340.6	343.51	339.18	343.51	1136008	{}	\N	2020-10-12 15:37:13.274804
56945	MDY	2020-10-02	337.71	348	336.6	346.52	1031813	{}	\N	2020-10-12 15:37:13.280053
56946	MDY	2020-10-05	349.54	354.69	349.46	354.68	798707	{}	\N	2020-10-12 15:37:13.286728
56947	MDY	2020-10-06	356.61	361.3	352.1	352.93	1133381	{}	\N	2020-10-12 15:37:13.293408
56948	MDY	2020-10-07	356.88	360.23	356.46	359.09	671395	{}	\N	2020-10-12 15:37:13.299695
56949	MDY	2020-10-08	362.29	362.96	360.16	362.88	713720	{}	\N	2020-10-12 15:37:13.304965
56950	MDY	2020-10-09	365.49	366.53	362.67	363.83	671662	{}	\N	2020-10-12 15:37:13.310565
56951	MDY	2020-10-12	365.32	366.595	363.83	366.17	262096	{}	\N	2020-10-12 15:37:13.316108
56952	MFG	2020-09-28	2.68	2.71	2.67	2.71	167209	{}	\N	2020-10-12 15:37:14.346928
56953	MFG	2020-09-29	2.62	2.62	2.58	2.58	406474	{}	\N	2020-10-12 15:37:14.356501
56954	MFG	2020-09-30	2.52	2.58	2.51	2.55	974542	{}	\N	2020-10-12 15:37:14.361779
56955	MFG	2020-10-01	2.54	2.6	2.53	2.53	636206	{}	\N	2020-10-12 15:37:14.367586
56956	MFG	2020-10-02	2.54	2.57	2.52	2.52	790913	{}	\N	2020-10-12 15:37:14.373872
56957	MFG	2020-10-05	2.57	2.58	2.55	2.55	790332	{}	\N	2020-10-12 15:37:14.379257
56958	MFG	2020-10-06	2.54	2.55	2.5	2.5	1044654	{}	\N	2020-10-12 15:37:14.384699
56959	MFG	2020-10-07	2.53	2.56	2.53	2.54	308070	{}	\N	2020-10-12 15:37:14.390005
56960	MFG	2020-10-08	2.55	2.565	2.525	2.54	397624	{}	\N	2020-10-12 15:37:14.39497
56961	MFG	2020-10-09	2.53	2.54	2.505	2.51	388592	{}	\N	2020-10-12 15:37:14.400384
56962	MFG	2020-10-12	2.53	2.53	2.5	2.515	129596	{}	\N	2020-10-12 15:37:14.405398
56963	MFUS	2020-09-28	29.8201	29.8201	29.719	29.719	1185	{}	\N	2020-10-12 15:37:16.28175
56964	MFUS	2020-09-29	29.6899	29.7	29.5644	29.5644	151894	{}	\N	2020-10-12 15:37:16.289086
56965	MFUS	2020-09-30	29.895	29.962	29.57	29.84	56063	{}	\N	2020-10-12 15:37:16.2954
56966	MFUS	2020-10-01	30.09	30.09	29.889	29.96	8716	{}	\N	2020-10-12 15:37:16.301671
56967	MFUS	2020-10-02	29.7887	29.99	29.7887	29.94	6425	{}	\N	2020-10-12 15:37:16.307412
56968	MFUS	2020-10-05	30.08	30.44	30.08	30.34	13139	{}	\N	2020-10-12 15:37:16.313091
56969	MFUS	2020-10-06	30.3965	30.5	30	30.03	1935	{}	\N	2020-10-12 15:37:16.319781
56970	MFUS	2020-10-07	30.41	30.59	30.375	30.54	9556	{}	\N	2020-10-12 15:37:16.326295
56971	MFUS	2020-10-08	30.57	30.7889	30.52	30.755	2569	{}	\N	2020-10-12 15:37:16.333841
56972	MFUS	2020-10-09	30.8901	31.02	30.8901	30.91	1318	{}	\N	2020-10-12 15:37:16.339786
56973	MFUS	2020-10-12	31.185	31.2602	31.185	31.2602	2338	{}	\N	2020-10-12 15:37:16.345758
56974	MGK	2020-09-28	184.61	184.8984	183.21	184.62	255690	{}	\N	2020-10-12 15:37:17.324673
56975	MGK	2020-09-29	184.62	185.1799	183.96	184.23	179272	{}	\N	2020-10-12 15:37:17.330797
56976	MGK	2020-09-30	184.31	187.53	184.31	185.4	237515	{}	\N	2020-10-12 15:37:17.338054
56977	MGK	2020-10-01	188	188.32	186.75	187.98	460931	{}	\N	2020-10-12 15:37:17.344656
56978	MGK	2020-10-02	183.74	186.5048	183.24	183.91	361400	{}	\N	2020-10-12 15:37:17.351649
56979	MGK	2020-10-05	185.19	187.7049	185.1	187.58	365843	{}	\N	2020-10-12 15:37:17.357397
56980	MGK	2020-10-06	187.28	188.04	183.68	184.08	217597	{}	\N	2020-10-12 15:37:17.36622
56981	MGK	2020-10-07	185.93	187.68	185.6966	187.34	271109	{}	\N	2020-10-12 15:37:17.372502
56982	MGK	2020-10-08	188.76	188.76	187.51	188.2	177015	{}	\N	2020-10-12 15:37:17.379488
56983	MGK	2020-10-09	189.17	191.27	189.14	191.27	224623	{}	\N	2020-10-12 15:37:17.385752
56984	MGK	2020-10-12	193.97	195.4696	193.185	195.3951	105046	{}	\N	2020-10-12 15:37:17.391425
56985	MGYR	2020-09-30	8.3416	8.3416	8.3416	8.3416	784	{}	\N	2020-10-12 15:37:18.593957
56986	MGYR	2020-10-05	8.0701	8.69	8.0701	8.38	1179	{}	\N	2020-10-12 15:37:18.600638
56987	MGYR	2020-10-06	8.36	8.36	8.36	8.36	216	{}	\N	2020-10-12 15:37:18.606371
56988	MHLA	2020-09-28	19.92	19.98	19.86	19.98	5537	{}	\N	2020-10-12 15:37:20.427868
56989	MHLA	2020-09-29	19.9	19.97	19.9	19.97	2782	{}	\N	2020-10-12 15:37:20.43421
56990	MHLA	2020-09-30	19.9372	19.965	19.85	19.85	6216	{}	\N	2020-10-12 15:37:20.440092
56991	MHLA	2020-10-01	19.9315	20	19.8887	19.993	10978	{}	\N	2020-10-12 15:37:20.4461
56992	MHLA	2020-10-02	19.8	20	19.8	20	2720	{}	\N	2020-10-12 15:37:20.453358
56993	MHLA	2020-10-05	20	20	20	20	7248	{}	\N	2020-10-12 15:37:20.461052
56994	MHLA	2020-10-06	20	20	19.95	19.96	10804	{}	\N	2020-10-12 15:37:20.467406
56995	MHLA	2020-10-07	20	20	19.92	19.92	7732	{}	\N	2020-10-12 15:37:20.474729
56996	MHLA	2020-10-08	19.88	19.93	19.85	19.93	10564	{}	\N	2020-10-12 15:37:20.481669
56997	MHLA	2020-10-09	19.87	19.97	19.8609	19.97	9430	{}	\N	2020-10-12 15:37:20.489565
56998	MHLA	2020-10-12	19.91	19.96	19.6229	19.96	5170	{}	\N	2020-10-12 15:37:20.497056
56999	MIME	2020-09-28	46.27	46.34	45.28	45.97	359980	{}	\N	2020-10-12 15:37:21.966119
57000	MIME	2020-09-29	46.06	46.47	45.64	45.98	413242	{}	\N	2020-10-12 15:37:21.972247
57001	MIME	2020-09-30	45.27	47.09	45.03	46.92	683679	{}	\N	2020-10-12 15:37:21.977818
57002	MIME	2020-10-01	47.36	47.7	46.77	47.21	508016	{}	\N	2020-10-12 15:37:21.98349
57003	MIME	2020-10-02	46.18	46.8	45.23	45.29	409855	{}	\N	2020-10-12 15:37:21.989436
57004	MIME	2020-10-05	45.725	46.42	45.27	46.31	711054	{}	\N	2020-10-12 15:37:21.994838
57005	MIME	2020-10-06	46.55	47.62	44.83	45.05	702812	{}	\N	2020-10-12 15:37:22.001861
57006	MIME	2020-10-07	45.42	46.95	45.42	46.83	347642	{}	\N	2020-10-12 15:37:22.010147
57007	MIME	2020-10-08	47.34	47.52	46.68	47.04	225106	{}	\N	2020-10-12 15:37:22.015546
57008	MIME	2020-10-09	47.3	48.01	47.3	47.92	285077	{}	\N	2020-10-12 15:37:22.021092
57009	MIME	2020-10-12	48.52	48.82	47.88	48.23	91164	{}	\N	2020-10-12 15:37:22.026791
57010	MG	2020-09-28	4.07	4.33	4.07	4.14	104075	{}	\N	2020-10-12 15:37:23.02645
57011	MG	2020-09-29	4.16	4.17	3.85	3.93	345934	{}	\N	2020-10-12 15:37:23.032061
57012	MG	2020-09-30	3.96	4.06	3.89	3.91	266007	{}	\N	2020-10-12 15:37:23.036731
57013	MG	2020-10-01	3.93	4.03	3.885	4.01	242965	{}	\N	2020-10-12 15:37:23.042711
57014	MG	2020-10-02	3.88	4.06	3.85	3.97	295982	{}	\N	2020-10-12 15:37:23.047589
57015	MG	2020-10-05	4.1	4.1388	4.01	4.03	331965	{}	\N	2020-10-12 15:37:23.052401
57016	MG	2020-10-06	4.06	4.09	3.91	3.93	177674	{}	\N	2020-10-12 15:37:23.058379
57017	MG	2020-10-07	3.93	4.014	3.8	3.96	354668	{}	\N	2020-10-12 15:37:23.064079
57018	MG	2020-10-08	4.03	4.17	3.93	4.15	268843	{}	\N	2020-10-12 15:37:23.069605
57019	MG	2020-10-09	4.18	4.1999	4.02	4.06	171617	{}	\N	2020-10-12 15:37:23.076194
57020	MG	2020-10-12	4.03	4.06	3.87	3.87	249724	{}	\N	2020-10-12 15:37:23.08371
57021	MGRC	2020-09-28	58.97	60.54	58.97	60.29	78871	{}	\N	2020-10-12 15:37:27.259393
57022	MGRC	2020-09-29	60.29	60.35	58.71	59.61	59337	{}	\N	2020-10-12 15:37:27.269419
57023	MGRC	2020-09-30	59.82	60.49	59.58	59.59	94897	{}	\N	2020-10-12 15:37:27.274638
57024	MGRC	2020-10-01	59.64	61.39	59.2	61.3	100980	{}	\N	2020-10-12 15:37:27.282008
57025	MGRC	2020-10-02	60.11	62.37	59.68	61.9	96671	{}	\N	2020-10-12 15:37:27.289091
57026	MGRC	2020-10-05	62.12	63.44	62.01	63.34	93862	{}	\N	2020-10-12 15:37:27.29667
57027	MGRC	2020-10-06	63.37	66.18	63.32	64.32	128845	{}	\N	2020-10-12 15:37:27.302406
57028	MGRC	2020-10-07	64.91	66.79	64.82	66.05	140665	{}	\N	2020-10-12 15:37:27.307623
57029	MGRC	2020-10-08	66.13	67.29	65.34	66.31	67675	{}	\N	2020-10-12 15:37:27.313646
57030	MGRC	2020-10-09	67.07	67.95	66.94	67.92	66408	{}	\N	2020-10-12 15:37:27.318617
57031	MGRC	2020-10-12	68.01	69.515	67.77	69	10628	{}	\N	2020-10-12 15:37:27.323539
57032	MHLD	2020-09-28	1.2	1.39	1.17	1.26	287416	{}	\N	2020-10-12 15:37:28.192649
57033	MHLD	2020-09-29	1.28	1.2893	1.21	1.24	23641	{}	\N	2020-10-12 15:37:28.198815
57034	MHLD	2020-09-30	1.25	1.25	1.21	1.22	39422	{}	\N	2020-10-12 15:37:28.203416
57035	MHLD	2020-10-01	1.23	1.28	1.23	1.25	65698	{}	\N	2020-10-12 15:37:28.20877
57036	MHLD	2020-10-02	1.25	1.28	1.24	1.24	34330	{}	\N	2020-10-12 15:37:28.215582
57037	MHLD	2020-10-05	1.24	1.27	1.24	1.25	27059	{}	\N	2020-10-12 15:37:28.220662
57038	MHLD	2020-10-06	1.26	1.29	1.25	1.26	38910	{}	\N	2020-10-12 15:37:28.225059
57039	MHLD	2020-10-07	1.26	1.28	1.24	1.25	46904	{}	\N	2020-10-12 15:37:28.230671
57040	MHLD	2020-10-08	1.2644	1.2644	1.24	1.25	66658	{}	\N	2020-10-12 15:37:28.234973
57041	MHLD	2020-10-09	1.25	1.25	1.22	1.22	17202	{}	\N	2020-10-12 15:37:28.239412
57042	MHLD	2020-10-12	1.22	1.24	1.22	1.24	8152	{}	\N	2020-10-12 15:37:28.245301
57043	MICT	2020-09-28	3.2173	3.71	3.21	3.57	362366	{}	\N	2020-10-12 15:37:29.111346
57044	MICT	2020-09-29	3.84	5.27	3.6401	4.05	3136737	{}	\N	2020-10-12 15:37:29.124629
57045	MICT	2020-09-30	3.71	4.1378	3.6004	3.87	676031	{}	\N	2020-10-12 15:37:29.135885
57046	MICT	2020-10-01	3.91	3.99	3.73	3.94	266949	{}	\N	2020-10-12 15:37:29.1446
57047	MICT	2020-10-02	3.84	4.69	3.41	3.59	1963738	{}	\N	2020-10-12 15:37:29.151025
57048	MICT	2020-10-05	3.65	3.73	3.47	3.48	281054	{}	\N	2020-10-12 15:37:29.156916
57049	MICT	2020-10-06	3.68	3.68	3.32	3.62	612404	{}	\N	2020-10-12 15:37:29.16399
57050	MICT	2020-10-07	3.55	3.68	3.46	3.63	497019	{}	\N	2020-10-12 15:37:29.169493
57051	MICT	2020-10-08	3.62	3.73	3.41	3.46	269172	{}	\N	2020-10-12 15:37:29.176493
57052	MICT	2020-10-09	3.53	3.65	3.45	3.51	298770	{}	\N	2020-10-12 15:37:29.183231
57053	MICT	2020-10-12	3.5502	3.5672	3.4	3.41	63592	{}	\N	2020-10-12 15:37:29.189087
57054	MKTX	2020-09-28	467.18	478.13	464.755	476.05	234149	{}	\N	2020-10-12 15:37:30.569359
57055	MKTX	2020-09-29	473.7	491.18	473.07	485.43	318279	{}	\N	2020-10-12 15:37:30.575721
57056	MKTX	2020-09-30	487.52	492.59	477.91	481.59	258090	{}	\N	2020-10-12 15:37:30.581265
57057	MKTX	2020-10-01	483.91	487.78	480.55	482.97	164386	{}	\N	2020-10-12 15:37:30.58693
57058	MKTX	2020-10-02	477.97	489.99	472.6012	482.52	161139	{}	\N	2020-10-12 15:37:30.593891
57059	MKTX	2020-10-05	488.87	498.07	486.72	497.28	198253	{}	\N	2020-10-12 15:37:30.601642
57060	MKTX	2020-10-06	499.57	501.41	492.415	498.58	182068	{}	\N	2020-10-12 15:37:30.607192
57061	MKTX	2020-10-07	505	512.935	504.8586	510.23	242682	{}	\N	2020-10-12 15:37:30.613031
57062	MKTX	2020-10-08	513.01	519.41	511.67	518.42	120793	{}	\N	2020-10-12 15:37:30.619419
57063	MKTX	2020-10-09	519.86	524.25	519.38	522.05	105477	{}	\N	2020-10-12 15:37:30.625139
57064	MKTX	2020-10-12	525.27	531.72	523.15	530.12	32603	{}	\N	2020-10-12 15:37:30.6311
57065	MLPA	2020-09-28	21.71	22.17	21.43	21.99	509844	{}	\N	2020-10-12 15:37:31.541433
57066	MLPA	2020-09-29	21.99	22.5	21.74	22.35	220222	{}	\N	2020-10-12 15:37:31.547068
57067	MLPA	2020-09-30	22.34	22.79	21.77	21.96	377757	{}	\N	2020-10-12 15:37:31.554139
57068	MLPA	2020-10-01	21.96	22.1	21.4	21.81	776843	{}	\N	2020-10-12 15:37:31.559069
57069	MLPA	2020-10-02	21.21	22.2	21.21	22.11	426547	{}	\N	2020-10-12 15:37:31.565035
57070	MLPA	2020-10-05	22.26	22.99	22.23	22.99	307807	{}	\N	2020-10-12 15:37:31.571109
57071	MLPA	2020-10-06	22.96	23.45	22.23	22.46	373516	{}	\N	2020-10-12 15:37:31.576908
57072	MLPA	2020-10-07	22.58	22.7145	22.37	22.53	294756	{}	\N	2020-10-12 15:37:31.582445
57073	MLPA	2020-10-08	22.75	23.95	22.68	23.92	292190	{}	\N	2020-10-12 15:37:31.588349
57074	MLPA	2020-10-09	24.23	24.24	23.61	23.96	214691	{}	\N	2020-10-12 15:37:31.594199
57075	MLPA	2020-10-12	23.96	23.96	23.63	23.84	77461	{}	\N	2020-10-12 15:37:31.600871
57076	MLPX	2020-09-28	22.47	22.839	22.3299	22.54	89610	{}	\N	2020-10-12 15:37:32.871058
57077	MLPX	2020-09-29	22.54	22.7299	22.31	22.56	107884	{}	\N	2020-10-12 15:37:32.877219
57078	MLPX	2020-09-30	22.64	22.87	21.98	22.1	198239	{}	\N	2020-10-12 15:37:32.882808
57079	MLPX	2020-10-01	22.03	22.18	21.66	21.775	148074	{}	\N	2020-10-12 15:37:32.889477
57080	MLPX	2020-10-02	21.45	22.18	21.39	22.12	62558	{}	\N	2020-10-12 15:37:32.894662
57081	MLPX	2020-10-05	22.39	22.78	22.26	22.77	78621	{}	\N	2020-10-12 15:37:32.900126
57082	MLPX	2020-10-06	23	23.18	22.14	22.32	85915	{}	\N	2020-10-12 15:37:32.905957
57083	MLPX	2020-10-07	22.38	22.64	22.23	22.55	75506	{}	\N	2020-10-12 15:37:32.911117
57084	MLPX	2020-10-08	22.63	23.74	22.63	23.64	250755	{}	\N	2020-10-12 15:37:32.916841
57085	MLPX	2020-10-09	24	24.01	23.35	23.49	110546	{}	\N	2020-10-12 15:37:32.922324
57086	MLPX	2020-10-12	23.43	23.48	23.31	23.465	54036	{}	\N	2020-10-12 15:37:32.927954
57087	MLVF	2020-09-28	11.86	12.1	11.86	11.91	5282	{}	\N	2020-10-12 15:37:33.911238
57088	MLVF	2020-09-29	11.94	11.97	11.88	11.88	610	{}	\N	2020-10-12 15:37:33.916913
57089	MLVF	2020-09-30	11.98	11.99	11.8	11.8	17580	{}	\N	2020-10-12 15:37:33.922718
57090	MLVF	2020-10-01	11.8	11.98	11.67	11.94	14228	{}	\N	2020-10-12 15:37:33.927776
57091	MLVF	2020-10-02	11.82	12.02	11.8001	12	10606	{}	\N	2020-10-12 15:37:33.93524
57092	MLVF	2020-10-05	12.15	12.33	12	12.03	7454	{}	\N	2020-10-12 15:37:33.941002
57093	MLVF	2020-10-06	12.19	12.35	12.088	12.2	9132	{}	\N	2020-10-12 15:37:33.946061
57094	MLVF	2020-10-07	12.1	12.2	12.1	12.2	587	{}	\N	2020-10-12 15:37:33.95046
57095	MLVF	2020-10-08	12.3	12.3	12.11	12.2	1896	{}	\N	2020-10-12 15:37:33.955615
57096	MLVF	2020-10-09	12.4	12.4	12.25	12.29	2765	{}	\N	2020-10-12 15:37:33.960475
57097	MLVF	2020-10-12	12.26	12.5	12.26	12.48	622	{}	\N	2020-10-12 15:37:33.9657
57098	MMI	2020-09-28	27.06	27.81	27.06	27.5	213083	{}	\N	2020-10-12 15:37:35.121841
57099	MMI	2020-09-29	27.5	27.52	26.87	27.47	299772	{}	\N	2020-10-12 15:37:35.126749
57100	MMI	2020-09-30	27.65	28.04	27.32	27.52	246954	{}	\N	2020-10-12 15:37:35.131568
57101	MMI	2020-10-01	27.76	28.06	27.33	28.06	135829	{}	\N	2020-10-12 15:37:35.137368
57102	MMI	2020-10-02	27.74	28.71	27.51	28.68	151366	{}	\N	2020-10-12 15:37:35.144637
57103	MMI	2020-10-05	28.75	28.8	28.45	28.76	166121	{}	\N	2020-10-12 15:37:35.151379
57104	MMI	2020-10-06	29.05	29.99	28.46	29.46	327931	{}	\N	2020-10-12 15:37:35.15763
57105	MMI	2020-10-07	29.69	29.91	29.15	29.49	231539	{}	\N	2020-10-12 15:37:35.163038
57106	MMI	2020-10-08	29.63	30.68	29.63	30.58	155146	{}	\N	2020-10-12 15:37:35.169405
57107	MMI	2020-10-09	30.85	30.88	30.09	30.33	210257	{}	\N	2020-10-12 15:37:35.17525
57108	MMI	2020-10-12	30.38	30.494	29.825	30.31	76802	{}	\N	2020-10-12 15:37:35.18113
57109	MMU	2020-09-28	12.12	12.14	12.0729	12.09	38056	{}	\N	2020-10-12 15:37:36.702895
57110	MMU	2020-09-29	12.11	12.1459	12.09	12.1	123160	{}	\N	2020-10-12 15:37:36.709256
57111	MMU	2020-09-30	12.1	12.19	12.09	12.19	119920	{}	\N	2020-10-12 15:37:36.714776
57112	MMU	2020-10-01	12.21	12.23	12.12	12.2	233126	{}	\N	2020-10-12 15:37:36.720821
57113	MMU	2020-10-02	12.15	12.23	12.15	12.2	118840	{}	\N	2020-10-12 15:37:36.727806
57114	MMU	2020-10-05	12.22	12.22	12.065	12.11	122414	{}	\N	2020-10-12 15:37:36.733242
57115	MMU	2020-10-06	12.14	12.15	12.1	12.14	110652	{}	\N	2020-10-12 15:37:36.73981
57116	MMU	2020-10-07	12.12	12.2	12.1	12.1	155724	{}	\N	2020-10-12 15:37:36.745644
57117	MMU	2020-10-08	12.09	12.16	12.09	12.16	119818	{}	\N	2020-10-12 15:37:36.751155
57118	MMU	2020-10-09	12.12	12.22	12.12	12.19	141495	{}	\N	2020-10-12 15:37:36.757329
57119	MMU	2020-10-12	12.19	12.2406	12.19	12.2406	39820	{}	\N	2020-10-12 15:37:36.763201
57120	MNKD	2020-09-28	1.87	1.935	1.86	1.93	2060905	{}	\N	2020-10-12 15:37:37.632621
57121	MNKD	2020-09-29	1.94	1.98	1.9	1.91	1491299	{}	\N	2020-10-12 15:37:37.639194
57122	MNKD	2020-09-30	1.95	1.95	1.88	1.88	1109156	{}	\N	2020-10-12 15:37:37.645948
57123	MNKD	2020-10-01	1.9	1.905	1.82	1.87	1498921	{}	\N	2020-10-12 15:37:37.651586
57124	MNKD	2020-10-02	1.81	1.88	1.8	1.805	2379573	{}	\N	2020-10-12 15:37:37.658728
57125	MNKD	2020-10-05	1.84	1.8799	1.78	1.81	1603270	{}	\N	2020-10-12 15:37:37.663937
57126	MNKD	2020-10-06	1.85	1.865	1.78	1.79	1419142	{}	\N	2020-10-12 15:37:37.669766
57127	MNKD	2020-10-07	1.82	1.89	1.805	1.89	1309503	{}	\N	2020-10-12 15:37:37.674916
57128	MNKD	2020-10-08	1.94	1.94	1.89	1.91	1453245	{}	\N	2020-10-12 15:37:37.680323
57129	MNKD	2020-10-09	1.92	1.96	1.88	1.89	1264700	{}	\N	2020-10-12 15:37:37.686864
57130	MNKD	2020-10-12	1.88	1.925	1.84	1.92	444793	{}	\N	2020-10-12 15:37:37.692618
57131	MO	2020-09-28	38.47	39.21	38.47	38.95	8082328	{}	\N	2020-10-12 15:37:38.638448
57132	MO	2020-09-29	38.94	38.95	38.33	38.36	14265258	{}	\N	2020-10-12 15:37:38.645035
57133	MO	2020-09-30	38.54	38.9	38.3332	38.64	15243268	{}	\N	2020-10-12 15:37:38.650698
57134	MO	2020-10-01	38.67	38.945	38.38	38.92	14695596	{}	\N	2020-10-12 15:37:38.655798
57135	MO	2020-10-02	38.4	39.22	38.35	38.97	10255625	{}	\N	2020-10-12 15:37:38.662215
57136	MO	2020-10-05	39.17	39.62	38.92	39.5	10125228	{}	\N	2020-10-12 15:37:38.667352
57137	MO	2020-10-06	39.65	40.22	39.4818	39.6	12895786	{}	\N	2020-10-12 15:37:38.67292
57138	MO	2020-10-07	39.81	40.835	39.81	40.53	14930547	{}	\N	2020-10-12 15:37:38.677631
57139	MO	2020-10-08	40.67	40.95	40	40.38	11496270	{}	\N	2020-10-12 15:37:38.681919
57140	MO	2020-10-09	40.83	41.26	40.585	40.69	22952485	{}	\N	2020-10-12 15:37:38.685978
57141	MO	2020-10-12	40.65	41.24	40.425	41.07	4753085	{}	\N	2020-10-12 15:37:38.691903
57142	MGI	2020-09-28	2.8	2.832	2.68	2.73	1163832	{}	\N	2020-10-12 15:37:40.273826
57143	MGI	2020-09-29	2.74	2.7882	2.7115	2.76	524288	{}	\N	2020-10-12 15:37:40.279964
57144	MGI	2020-09-30	2.77	2.83	2.74	2.825	878055	{}	\N	2020-10-12 15:37:40.285223
57145	MGI	2020-10-01	2.82	2.97	2.73	2.94	968567	{}	\N	2020-10-12 15:37:40.29092
57146	MGI	2020-10-02	2.85	2.93	2.83	2.89	406233	{}	\N	2020-10-12 15:37:40.297845
57147	MGI	2020-10-05	2.89	2.98	2.85	2.96	995419	{}	\N	2020-10-12 15:37:40.303148
57148	MGI	2020-10-06	3.11	3.21	2.99	3.17	3340314	{}	\N	2020-10-12 15:37:40.310377
57149	MGI	2020-10-07	3.17	3.1865	3.05	3.06	1100583	{}	\N	2020-10-12 15:37:40.317352
57150	MGI	2020-10-08	3.08	3.18	3.06	3.1	721296	{}	\N	2020-10-12 15:37:40.323398
57151	MGI	2020-10-09	3.11	3.27	3.08	3.15	1284534	{}	\N	2020-10-12 15:37:40.33003
57152	MGI	2020-10-12	3.16	3.21	3.1	3.12	477420	{}	\N	2020-10-12 15:37:40.335862
57153	MGV	2020-09-28	76.31	76.96	76.31	76.56	156578	{}	\N	2020-10-12 15:37:41.216557
57154	MGV	2020-09-29	76.63	76.63	75.75	75.9688	127933	{}	\N	2020-10-12 15:37:41.2221
57155	MGV	2020-09-30	76.5	77.27	76.25	76.7	221246	{}	\N	2020-10-12 15:37:41.227531
57156	MGV	2020-10-01	76.99	77.1	76.255	76.65	103575	{}	\N	2020-10-12 15:37:41.233829
57157	MGV	2020-10-02	75.57	77.08	75.57	76.63	95590	{}	\N	2020-10-12 15:37:41.239494
57158	MGV	2020-10-05	77.13	77.78	77.1	77.73	107560	{}	\N	2020-10-12 15:37:41.244937
57159	MGV	2020-10-06	77.93	78.48	76.9022	77.01	92651	{}	\N	2020-10-12 15:37:41.250355
57160	MGV	2020-10-07	77.9	78.5	77.71	78.33	79092	{}	\N	2020-10-12 15:37:41.255468
57161	MGV	2020-10-08	78.62	79.14	78.62	79.09	89812	{}	\N	2020-10-12 15:37:41.260971
57162	MGV	2020-10-09	79.53	79.7387	79.09	79.36	97546	{}	\N	2020-10-12 15:37:41.266401
57163	MGV	2020-10-12	79.51	79.99	79.43	79.9338	16444	{}	\N	2020-10-12 15:37:41.27185
57164	MIK	2020-09-28	9.62	9.725	9.23	9.58	6470867	{}	\N	2020-10-12 15:37:42.514692
57165	MIK	2020-09-29	9.56	9.62	9.245	9.37	4057846	{}	\N	2020-10-12 15:37:42.522604
57166	MIK	2020-09-30	9.39	10.12	9.37	9.655	8030401	{}	\N	2020-10-12 15:37:42.529363
57167	MIK	2020-10-01	9.72	10.74	9.72	10.46	6362579	{}	\N	2020-10-12 15:37:42.535256
57168	MIK	2020-10-02	9.89	9.97	9.43	9.71	12514142	{}	\N	2020-10-12 15:37:42.541655
57169	MIK	2020-10-05	9.84	10.1	9.41	10.01	7191194	{}	\N	2020-10-12 15:37:42.54881
57170	MIK	2020-10-06	10.05	10.26	9.75	9.78	4639332	{}	\N	2020-10-12 15:37:42.55426
57171	MIK	2020-10-07	9.92	10.43	9.9	10.16	4875158	{}	\N	2020-10-12 15:37:42.560809
57172	MIK	2020-10-08	10.28	10.4601	10.04	10.24	3657786	{}	\N	2020-10-12 15:37:42.567808
57173	MIK	2020-10-09	10.3	10.45	9.99	10.05	2874311	{}	\N	2020-10-12 15:37:42.573235
57174	MIK	2020-10-12	10.11	10.22	9.8	9.81	2110292	{}	\N	2020-10-12 15:37:42.579483
57175	MITT	2020-09-28	2.82	2.93	2.82	2.86	1521331	{}	\N	2020-10-12 15:37:43.662405
57176	MITT	2020-09-29	2.81	2.84	2.7186	2.8	1410701	{}	\N	2020-10-12 15:37:43.668474
57177	MITT	2020-09-30	2.78	2.8477	2.75	2.76	1148255	{}	\N	2020-10-12 15:37:43.673181
57178	MITT	2020-10-01	2.79	2.89	2.75	2.88	1096881	{}	\N	2020-10-12 15:37:43.678138
57179	MITT	2020-10-02	2.78	2.95	2.78	2.9	1304132	{}	\N	2020-10-12 15:37:43.684563
57180	MITT	2020-10-05	2.9	3.05	2.8943	2.95	1996257	{}	\N	2020-10-12 15:37:43.690358
57181	MITT	2020-10-06	2.95	3	2.83	2.85	2050656	{}	\N	2020-10-12 15:37:43.696649
57182	MITT	2020-10-07	2.85	2.8862	2.8	2.85	936669	{}	\N	2020-10-12 15:37:43.701815
57183	MITT	2020-10-08	2.84	2.88	2.775	2.81	2366649	{}	\N	2020-10-12 15:37:43.706914
57184	MITT	2020-10-09	2.84	2.85	2.79	2.81	1586915	{}	\N	2020-10-12 15:37:43.71215
57185	MITT	2020-10-12	2.79	2.815	2.76	2.78	387068	{}	\N	2020-10-12 15:37:43.717516
57186	MKSI	2020-09-28	109.69	110.87	108.59	110.49	298882	{}	\N	2020-10-12 15:37:44.961558
57187	MKSI	2020-09-29	110.01	112.69	109.15	110.74	251764	{}	\N	2020-10-12 15:37:44.967844
57188	MKSI	2020-09-30	109.38	111.7	108.39	109.23	331243	{}	\N	2020-10-12 15:37:44.973207
57189	MKSI	2020-10-01	110.41	112.08	110.05	111.48	302466	{}	\N	2020-10-12 15:37:44.979018
57190	MKSI	2020-10-02	108.76	110.41	108.46	108.46	279121	{}	\N	2020-10-12 15:37:44.985814
57191	MKSI	2020-10-05	109.35	111.75	109.35	111.32	532354	{}	\N	2020-10-12 15:37:44.992436
57192	MKSI	2020-10-06	111.59	114.25	110.795	111.24	345586	{}	\N	2020-10-12 15:37:44.99882
57193	MKSI	2020-10-07	113.04	115	112.55	114.61	452826	{}	\N	2020-10-12 15:37:45.004369
57194	MKSI	2020-10-08	115.57	116.84	114.44	116.7	203647	{}	\N	2020-10-12 15:37:45.009806
57195	MKSI	2020-10-09	118.27	119.65	117.695	118.88	353599	{}	\N	2020-10-12 15:37:45.016404
57196	MKSI	2020-10-12	120.6	120.64	118.54	118.54	55869	{}	\N	2020-10-12 15:37:45.022843
57197	MMC	2020-09-28	114.1	115.85	113.26	114.54	2039712	{}	\N	2020-10-12 15:37:47.060558
57198	MMC	2020-09-29	114.32	114.89	113.17	114.05	3271334	{}	\N	2020-10-12 15:37:47.068273
57199	MMC	2020-09-30	114.55	115.835	113.58	114.7	4893039	{}	\N	2020-10-12 15:37:47.073883
57200	MMC	2020-10-01	115.03	115.45	113.59	114.23	3879329	{}	\N	2020-10-12 15:37:47.079548
57201	MMC	2020-10-02	113.42	115.8	113.185	114.56	3203743	{}	\N	2020-10-12 15:37:47.085797
57202	MMC	2020-10-05	115.44	116.0939	114.68	115.66	2572703	{}	\N	2020-10-12 15:37:47.091686
57203	MMC	2020-10-06	116.08	117.14	114.46	114.88	4290026	{}	\N	2020-10-12 15:37:47.096885
57204	MMC	2020-10-07	115.59	116.2	114.255	114.69	3201437	{}	\N	2020-10-12 15:37:47.103225
57205	MMC	2020-10-08	114.61	115.6766	114.13	114.26	4270843	{}	\N	2020-10-12 15:37:47.119413
57206	MMC	2020-10-09	115.19	116.51	114.23	115.71	3178961	{}	\N	2020-10-12 15:37:47.128076
57207	MMC	2020-10-12	115.3	116.94	115.27	116.815	1147317	{}	\N	2020-10-12 15:37:47.136422
57208	MMLP	2020-09-28	1.2	1.25	1.17	1.19	270265	{}	\N	2020-10-12 15:37:48.272258
57209	MMLP	2020-09-29	1.2	1.32	1.19	1.23	241332	{}	\N	2020-10-12 15:37:48.27856
57210	MMLP	2020-09-30	1.28	1.29	1.17	1.17	102515	{}	\N	2020-10-12 15:37:48.285577
57211	MMLP	2020-10-01	1.17	1.23	1.14	1.15	132507	{}	\N	2020-10-12 15:37:48.291398
57212	MMLP	2020-10-02	1.03	1.4	1.03	1.34	629380	{}	\N	2020-10-12 15:37:48.297393
57213	MMLP	2020-10-05	1.29	1.35	1.2701	1.29	89492	{}	\N	2020-10-12 15:37:48.303878
57214	MMLP	2020-10-06	1.28	1.35	1.27	1.27	105411	{}	\N	2020-10-12 15:37:48.309591
57215	MMLP	2020-10-07	1.26	1.29	1.22	1.27	79765	{}	\N	2020-10-12 15:37:48.315041
57216	MMLP	2020-10-08	1.2756	1.38	1.25	1.36	189306	{}	\N	2020-10-12 15:37:48.322803
57217	MMLP	2020-10-09	1.36	1.76	1.31	1.37	2277199	{}	\N	2020-10-12 15:37:48.329184
57218	MMLP	2020-10-12	1.38	1.44	1.37	1.41	76747	{}	\N	2020-10-12 15:37:48.335064
57219	MN	2020-09-28	4.09	4.2105	4.09	4.15	31858	{}	\N	2020-10-12 15:37:49.617406
57220	MN	2020-09-29	4.1	4.2	4.08	4.17	60676	{}	\N	2020-10-12 15:37:49.624591
57221	MN	2020-09-30	4.21	4.39	4.21	4.27	90134	{}	\N	2020-10-12 15:37:49.630026
57222	MN	2020-10-01	4.35	4.35	4.19	4.28	61606	{}	\N	2020-10-12 15:37:49.635826
57223	MN	2020-10-02	4.2	4.39	4.0927	4.32	26796	{}	\N	2020-10-12 15:37:49.642749
57224	MN	2020-10-05	4.42	4.43	4.3	4.3	52962	{}	\N	2020-10-12 15:37:49.648059
57225	MN	2020-10-06	4.39	4.39	4.25	4.25	37132	{}	\N	2020-10-12 15:37:49.654397
57226	MN	2020-10-07	4.25	4.37	4.25	4.34	41222	{}	\N	2020-10-12 15:37:49.661169
57227	MN	2020-10-08	4.35	4.39	4.29	4.39	58426	{}	\N	2020-10-12 15:37:49.666919
57228	MN	2020-10-09	4.39	4.97	4.3	4.65	1275234	{}	\N	2020-10-12 15:37:49.673038
57229	MN	2020-10-12	4.72	4.88	3.8701	4.068	418436	{}	\N	2020-10-12 15:37:49.679033
57230	MNRO	2020-09-28	41.44	41.83	40.37	40.71	361650	{}	\N	2020-10-12 15:37:50.956586
57231	MNRO	2020-09-29	40.65	40.8	39.39	40.3	366869	{}	\N	2020-10-12 15:37:50.962925
57232	MNRO	2020-09-30	40.44	40.97	40.01	40.57	398341	{}	\N	2020-10-12 15:37:50.968688
57233	MNRO	2020-10-01	40.62	42.08	40.03	41.71	284596	{}	\N	2020-10-12 15:37:50.975172
57234	MNRO	2020-10-02	40.93	42.53	40.46	42.33	262734	{}	\N	2020-10-12 15:37:50.980965
57235	MNRO	2020-10-05	42.34	43.35	42.05	43.01	309426	{}	\N	2020-10-12 15:37:50.98889
57236	MNRO	2020-10-06	43.4	44.09	42.06	42.31	318316	{}	\N	2020-10-12 15:37:50.994933
57237	MNRO	2020-10-07	42.67	43.63	42.43	43.34	398527	{}	\N	2020-10-12 15:37:51.000397
57238	MNRO	2020-10-08	43.81	44.63	43.36	44.21	270357	{}	\N	2020-10-12 15:37:51.01038
57239	MNRO	2020-10-09	44.215	45.26	44.02	44.24	314309	{}	\N	2020-10-12 15:37:51.015831
57240	MNRO	2020-10-12	44.03	44.8224	44.03	44.655	57686	{}	\N	2020-10-12 15:37:51.022876
57241	MOMO	2020-09-28	14.23	14.35	13.66	13.96	4108597	{}	\N	2020-10-12 15:37:53.344551
57242	MOMO	2020-09-29	13.96	14.07	13.52	13.67	3406887	{}	\N	2020-10-12 15:37:53.350866
57243	MOMO	2020-09-30	13.52	13.98	13.33	13.76	7129934	{}	\N	2020-10-12 15:37:53.356021
57244	MOMO	2020-10-01	13.97	14.875	13.72	14.77	6646981	{}	\N	2020-10-12 15:37:53.362829
57245	MOMO	2020-10-02	14.27	14.93	14.225	14.83	3931283	{}	\N	2020-10-12 15:37:53.368682
57246	MOMO	2020-10-05	15.29	15.3	14.71	15	2854374	{}	\N	2020-10-12 15:37:53.37507
57247	MOMO	2020-10-06	15.02	15.3	14.76	14.87	3587000	{}	\N	2020-10-12 15:37:53.380059
57248	MOMO	2020-10-07	15.04	15.37	14.86	15.15	2778360	{}	\N	2020-10-12 15:37:53.385543
57249	MOMO	2020-10-08	15.32	15.39	14.95	15	2246662	{}	\N	2020-10-12 15:37:53.39123
57250	MOMO	2020-10-09	15.05	15.22	14.91	14.98	1618619	{}	\N	2020-10-12 15:37:53.396567
57251	MOMO	2020-10-12	15.04	15.24	14.95	15.21	984639	{}	\N	2020-10-12 15:37:53.402446
57252	MGEN	2020-09-28	0.84	0.84	0.79	0.8108	432403	{}	\N	2020-10-12 15:37:54.884778
57253	MGEN	2020-09-29	0.8173	0.82	0.762	0.7981	481616	{}	\N	2020-10-12 15:37:54.890032
57254	MGEN	2020-09-30	0.8	0.83	0.77	0.775	467796	{}	\N	2020-10-12 15:37:54.89555
57255	MGEN	2020-10-01	0.78	0.785	0.73	0.7559	737884	{}	\N	2020-10-12 15:37:54.900426
57256	MGEN	2020-10-02	0.75	0.75	0.72	0.7399	397441	{}	\N	2020-10-12 15:37:54.905945
57257	MGEN	2020-10-05	0.756	0.79	0.7415	0.76	2043354	{}	\N	2020-10-12 15:37:54.913579
57258	MGEN	2020-10-06	0.5799	0.63	0.56	0.5815	4223688	{}	\N	2020-10-12 15:37:54.918879
57259	MGEN	2020-10-07	0.6	0.603	0.5755	0.585	1718882	{}	\N	2020-10-12 15:37:54.923965
57260	MGEN	2020-10-08	0.595	0.608	0.5773	0.5981	1006303	{}	\N	2020-10-12 15:37:54.929769
57261	MGEN	2020-10-09	0.6	0.6078	0.59	0.5999	852437	{}	\N	2020-10-12 15:37:54.93436
57262	MGEN	2020-10-12	0.61	0.61	0.5903	0.5949	407989	{}	\N	2020-10-12 15:37:54.93926
57263	MGM	2020-09-28	22.61	22.86	22.08	22.63	9900996	{}	\N	2020-10-12 15:37:55.892067
57264	MGM	2020-09-29	22.58	22.58	21.36	21.72	25427227	{}	\N	2020-10-12 15:37:55.900079
57265	MGM	2020-09-30	21.9	22.55	21.5804	21.75	21054560	{}	\N	2020-10-12 15:37:55.905816
57266	MGM	2020-10-01	21.94	22.23	21.62	21.82	23016642	{}	\N	2020-10-12 15:37:55.911629
57267	MGM	2020-10-02	21	21.91	20.88	21.66	20408178	{}	\N	2020-10-12 15:37:55.916613
57268	MGM	2020-10-05	21.83	22.02	21.395	21.55	16549075	{}	\N	2020-10-12 15:37:55.921215
57269	MGM	2020-10-06	21.92	22.26	21.05	21.05	27239570	{}	\N	2020-10-12 15:37:55.925914
57270	MGM	2020-10-07	21.42	21.745	21.29	21.5	19165862	{}	\N	2020-10-12 15:37:55.931422
57271	MGM	2020-10-08	21.79	22.0288	21.43	21.93	20223353	{}	\N	2020-10-12 15:37:55.936517
57272	MGM	2020-10-09	21.87	22.08	21.32	21.36	18571461	{}	\N	2020-10-12 15:37:55.941356
57273	MGM	2020-10-12	21.28	21.4684	21.05	21.445	9118924	{}	\N	2020-10-12 15:37:55.946607
57274	MHE	2020-09-28	13.24	13.25	13.2	13.2	1219	{}	\N	2020-10-12 15:37:57.384315
57275	MHE	2020-09-29	13.25	13.25	13.06	13.06	2916	{}	\N	2020-10-12 15:37:57.390748
57276	MHE	2020-09-30	13.08	13.135	13.05	13.12	4402	{}	\N	2020-10-12 15:37:57.395575
57277	MHE	2020-10-01	13.3	13.3	13.05	13.05	4254	{}	\N	2020-10-12 15:37:57.40001
57278	MHE	2020-10-02	13.131	13.25	13.09	13.23	10642	{}	\N	2020-10-12 15:37:57.404362
57279	MHE	2020-10-05	13.155	13.2499	13.155	13.2252	3490	{}	\N	2020-10-12 15:37:57.408733
57280	MHE	2020-10-06	13.5799	13.5799	13.5799	13.5799	1486	{}	\N	2020-10-12 15:37:57.415046
57281	MHE	2020-10-07	13.41	13.4999	13.36	13.4325	7288	{}	\N	2020-10-12 15:37:57.419577
57282	MHE	2020-10-08	13.425	13.425	13.425	13.425	382	{}	\N	2020-10-12 15:37:57.424357
57283	MHNC	2020-09-28	21.45	22.12	21.45	21.99	8230	{}	\N	2020-10-12 15:37:58.340086
57284	MHNC	2020-09-29	22	22	21.82	21.95	8572	{}	\N	2020-10-12 15:37:58.34603
57285	MHNC	2020-09-30	21.9109	22.1063	21.82	22.04	15182	{}	\N	2020-10-12 15:37:58.351357
57286	MHNC	2020-10-01	21.99	22	21.9755	22	11274	{}	\N	2020-10-12 15:37:58.35647
57287	MHNC	2020-10-02	21.925	22.2	21.925	22.195	5934	{}	\N	2020-10-12 15:37:58.363329
57288	MHNC	2020-10-05	22.22	22.25	22	22.22	11342	{}	\N	2020-10-12 15:37:58.369587
57289	MHNC	2020-10-06	22.15	22.25	22.102	22.25	8288	{}	\N	2020-10-12 15:37:58.375111
57290	MHNC	2020-10-07	22.09	22.21	21.92	22.19	11030	{}	\N	2020-10-12 15:37:58.381045
57291	MHNC	2020-10-08	22.19	22.24	21.923	22.24	6223	{}	\N	2020-10-12 15:37:58.386097
57292	MHNC	2020-10-09	22.187	22.24	22.1	22.2	10606	{}	\N	2020-10-12 15:37:58.391264
57293	MHNC	2020-10-12	22.2399	22.2399	22.2399	22.2399	1000	{}	\N	2020-10-12 15:37:58.397264
57294	MIE	2020-09-28	1.51	1.57	1.51	1.55	255544	{}	\N	2020-10-12 15:37:59.32821
57295	MIE	2020-09-29	1.55	1.55	1.52	1.55	302834	{}	\N	2020-10-12 15:37:59.33523
57296	MIE	2020-09-30	1.59	1.59	1.5	1.51	349012	{}	\N	2020-10-12 15:37:59.339996
57297	MIE	2020-10-01	1.5	1.51	1.48	1.5	1679567	{}	\N	2020-10-12 15:37:59.344501
57298	MIE	2020-10-02	1.46	1.5	1.45	1.49	1499152	{}	\N	2020-10-12 15:37:59.351728
57299	MIE	2020-10-05	1.51	1.58	1.49	1.55	535362	{}	\N	2020-10-12 15:37:59.356384
57300	MIE	2020-10-06	1.56	1.66	1.535	1.55	1087253	{}	\N	2020-10-12 15:37:59.360735
57301	MIE	2020-10-07	1.54	1.62	1.54	1.6	257187	{}	\N	2020-10-12 15:37:59.365931
57302	MIE	2020-10-08	1.59	1.71	1.59	1.7	329664	{}	\N	2020-10-12 15:37:59.372141
57303	MIE	2020-10-09	1.72	1.735	1.67	1.73	423983	{}	\N	2020-10-12 15:37:59.380221
57304	MIE	2020-10-12	1.73	1.73	1.67	1.6799	245721	{}	\N	2020-10-12 15:37:59.388424
57305	MINDP	2020-09-28	17.57	18	17.454	18	4730	{}	\N	2020-10-12 15:38:00.46215
57306	MINDP	2020-09-29	17.6	17.9799	17.315	17.73	5788	{}	\N	2020-10-12 15:38:00.467937
57307	MINDP	2020-09-30	17.52	17.83	17.52	17.83	695	{}	\N	2020-10-12 15:38:00.472768
57308	MINDP	2020-10-01	17.71	17.85	17.419	17.77	6459	{}	\N	2020-10-12 15:38:00.477447
57309	MINDP	2020-10-02	17.8	17.8899	17.76	17.825	1602	{}	\N	2020-10-12 15:38:00.483564
57310	MINDP	2020-10-05	17.89	18.6	17.8	17.9	11451	{}	\N	2020-10-12 15:38:00.48907
57311	MINDP	2020-10-06	17.81	18.5885	17.81	17.97	8690	{}	\N	2020-10-12 15:38:00.494631
57312	MINDP	2020-10-07	18.19	19.1264	18.19	18.98	8036	{}	\N	2020-10-12 15:38:00.501985
57313	MINDP	2020-10-08	19.18	19.5	18.93	19.3	4539	{}	\N	2020-10-12 15:38:00.506797
57314	MINDP	2020-10-09	19.6934	19.951	19.35	19.951	2390	{}	\N	2020-10-12 15:38:00.511856
57315	MINDP	2020-10-12	19.9	19.9	19.9	19.9	190	{}	\N	2020-10-12 15:38:00.517376
57316	MIXT	2020-09-28	8.55	8.99	8.42	8.88	32794	{}	\N	2020-10-12 15:38:01.43623
57317	MIXT	2020-09-29	8.89	8.95	8.67	8.8	19900	{}	\N	2020-10-12 15:38:01.441845
57318	MIXT	2020-09-30	8.82	8.96	8.71	8.94	50258	{}	\N	2020-10-12 15:38:01.44721
57319	MIXT	2020-10-01	8.8997	9	8.74	8.74	44892	{}	\N	2020-10-12 15:38:01.45441
57320	MIXT	2020-10-02	8.57	8.825	8.57	8.62	23730	{}	\N	2020-10-12 15:38:01.460077
57321	MIXT	2020-10-05	8.57	8.805	8.56	8.71	15459	{}	\N	2020-10-12 15:38:01.466469
57322	MIXT	2020-10-06	8.75	8.765	8.475	8.5	73709	{}	\N	2020-10-12 15:38:01.476657
57323	MIXT	2020-10-07	8.54	8.8	8.52	8.7	100462	{}	\N	2020-10-12 15:38:01.486339
57324	MIXT	2020-10-08	8.8	8.85	8.7301	8.8	35766	{}	\N	2020-10-12 15:38:01.494712
57325	MIXT	2020-10-09	9.15	9.18	8.82	8.96	42128	{}	\N	2020-10-12 15:38:01.503116
57326	MIXT	2020-10-12	9.07	9.07	8.84	8.855	10762	{}	\N	2020-10-12 15:38:01.50873
57327	MLM	2020-09-28	229.29	234.29	228.58	231.88	529239	{}	\N	2020-10-12 15:38:02.4343
57328	MLM	2020-09-29	231.13	232.26	227.24	231.06	937301	{}	\N	2020-10-12 15:38:02.440413
57329	MLM	2020-09-30	232.72	238.6306	231.81	235.36	1301519	{}	\N	2020-10-12 15:38:02.44577
57330	MLM	2020-10-01	237.28	241.24	233.13	234.87	1288114	{}	\N	2020-10-12 15:38:02.453015
57331	MLM	2020-10-02	230.59	244.79	230.59	240.55	1597344	{}	\N	2020-10-12 15:38:02.460869
57332	MLM	2020-10-05	243.75	252.76	242.82	249.18	2179935	{}	\N	2020-10-12 15:38:02.46902
57333	MLM	2020-10-06	251.78	261.64	248.74	249.64	2221895	{}	\N	2020-10-12 15:38:02.474859
57334	MLM	2020-10-07	253.46	268.36	252.64	264.76	2737823	{}	\N	2020-10-12 15:38:02.480698
57335	MLM	2020-10-08	270	276.29	269.35	275.82	1892636	{}	\N	2020-10-12 15:38:02.487355
57336	MLM	2020-10-09	276.4	277.43	266.885	273.1	1948886	{}	\N	2020-10-12 15:38:02.49306
57337	MLM	2020-10-12	274.65	275.11	270.2777	271.94	285501	{}	\N	2020-10-12 15:38:02.50111
57338	MLPG	2020-09-28	9.439	9.47	9.3984	9.41	6499	{}	\N	2020-10-12 15:38:03.657336
57339	MLPG	2020-09-30	9.4937	9.4937	9.4937	9.4937	0	{}	\N	2020-10-12 15:38:03.663668
57340	MLPG	2020-10-01	9.44	9.5002	9.4137	9.5	2778	{}	\N	2020-10-12 15:38:03.67061
57341	MLPG	2020-10-02	9.53	9.5925	9.53	9.5925	463	{}	\N	2020-10-12 15:38:03.675769
57342	MLPG	2020-10-05	9.9917	9.9917	9.9917	9.9917	4	{}	\N	2020-10-12 15:38:03.681099
57343	MLPG	2020-10-06	10	10	9.8981	9.8981	502	{}	\N	2020-10-12 15:38:03.686896
57344	MLPG	2020-10-07	9.9333	9.9876	9.9333	9.9876	101	{}	\N	2020-10-12 15:38:03.692089
57345	MLPG	2020-10-08	10.25	10.75	9.98	10.75	5969	{}	\N	2020-10-12 15:38:03.697109
57346	MLPG	2020-10-09	10.24	10.33	10.2298	10.33	2073	{}	\N	2020-10-12 15:38:03.702865
57347	MMSI	2020-09-28	43.48	43.58	42.1736	43.25	436077	{}	\N	2020-10-12 15:38:05.337489
57348	MMSI	2020-09-29	42.71	44	42.71	43.07	242855	{}	\N	2020-10-12 15:38:05.343467
57349	MMSI	2020-09-30	43.1	44.13	43.1	43.5	243560	{}	\N	2020-10-12 15:38:05.348256
57350	MMSI	2020-10-01	43.86	44.34	42.61	42.82	353667	{}	\N	2020-10-12 15:38:05.353996
57351	MMSI	2020-10-02	41.97	43.15	41.565	42.16	333989	{}	\N	2020-10-12 15:38:05.36049
57352	MMSI	2020-10-05	42.43	44.01	42.43	43.97	242395	{}	\N	2020-10-12 15:38:05.365705
57353	MMSI	2020-10-06	44.21	46.27	43.52	45.7	398332	{}	\N	2020-10-12 15:38:05.372009
57354	MMSI	2020-10-07	45.82	47.07	45.315	46.98	393989	{}	\N	2020-10-12 15:38:05.377191
57355	MMSI	2020-10-08	47.32	47.98	46.48	47.05	311416	{}	\N	2020-10-12 15:38:05.382703
57356	MMSI	2020-10-09	47.62	47.98	46.2893	47.81	204244	{}	\N	2020-10-12 15:38:05.387897
57357	MMSI	2020-10-12	47.86	48.97	47.19	48.9	121514	{}	\N	2020-10-12 15:38:05.392252
57358	MNA	2020-09-28	33	33.27	33	33.23	66232	{}	\N	2020-10-12 15:38:06.32894
57359	MNA	2020-09-29	33.25	33.62	33.25	33.53	85062	{}	\N	2020-10-12 15:38:06.335183
57360	MNA	2020-09-30	33.54	33.69	33.51	33.64	66732	{}	\N	2020-10-12 15:38:06.342405
57361	MNA	2020-10-01	33.8	33.948	33.8	33.87	130270	{}	\N	2020-10-12 15:38:06.348607
57362	MNA	2020-10-02	33.63	33.94	33.63	33.69	60991	{}	\N	2020-10-12 15:38:06.354955
57363	MNA	2020-10-05	33.91	34.02	33.735	33.82	84291	{}	\N	2020-10-12 15:38:06.360829
57364	MNA	2020-10-06	33.74	33.785	33.6	33.66	71190	{}	\N	2020-10-12 15:38:06.367048
57365	MNA	2020-10-07	33.9	33.9344	33.76	33.81	126898	{}	\N	2020-10-12 15:38:06.373405
57366	MNA	2020-10-08	33.81	33.9	33.73	33.78	104117	{}	\N	2020-10-12 15:38:06.379798
57367	MNA	2020-10-09	33.89	34.068	33.89	33.94	58903	{}	\N	2020-10-12 15:38:06.386287
57368	MNA	2020-10-12	33.92	34.07	33.92	34.015	12096	{}	\N	2020-10-12 15:38:06.392572
57369	MNP	2020-09-28	14.23	14.25	14.19	14.21	24108	{}	\N	2020-10-12 15:38:07.327657
57370	MNP	2020-09-29	14.22	14.3972	14.22	14.374	72128	{}	\N	2020-10-12 15:38:07.333183
57371	MNP	2020-09-30	14.35	14.37	14.28	14.37	47356	{}	\N	2020-10-12 15:38:07.3391
57372	MNP	2020-10-01	14.36	14.38	14.31	14.33	30670	{}	\N	2020-10-12 15:38:07.343791
57373	MNP	2020-10-02	14.33	14.41	14.25	14.38	34844	{}	\N	2020-10-12 15:38:07.349857
57374	MNP	2020-10-05	14.35	14.4299	14.3	14.3	25528	{}	\N	2020-10-12 15:38:07.35482
57375	MNP	2020-10-06	14.28	14.365	14.27	14.29	18853	{}	\N	2020-10-12 15:38:07.359471
57376	MNP	2020-10-07	14.29	14.35	14.27	14.27	29584	{}	\N	2020-10-12 15:38:07.364285
57377	MNP	2020-10-08	14.27	14.365	14.27	14.31	31378	{}	\N	2020-10-12 15:38:07.369242
57378	MNP	2020-10-09	14.31	14.36	14.31	14.31	20608	{}	\N	2020-10-12 15:38:07.37538
57379	MNP	2020-10-12	14.27	14.32	14.26	14.32	3496	{}	\N	2020-10-12 15:38:07.381479
57380	MNTX	2020-09-28	4.45	4.45	4.12	4.12	61470	{}	\N	2020-10-12 15:38:08.657481
57381	MNTX	2020-09-29	4.1201	4.23	4.1201	4.2	9975	{}	\N	2020-10-12 15:38:08.664091
57382	MNTX	2020-09-30	4.1	4.2986	4.1	4.17	12051	{}	\N	2020-10-12 15:38:08.669431
57383	MNTX	2020-10-01	4	4.49	4	4.41	26592	{}	\N	2020-10-12 15:38:08.677303
57384	MNTX	2020-10-02	4.54	4.88	4.36	4.74	22758	{}	\N	2020-10-12 15:38:08.684765
57385	MNTX	2020-10-05	4.64	4.75	4.48	4.48	13405	{}	\N	2020-10-12 15:38:08.691408
57386	MNTX	2020-10-06	4.755	5.1	4.755	5	66123	{}	\N	2020-10-12 15:38:08.697445
57387	MNTX	2020-10-07	5.12	5.305	4.7801	4.89	20500	{}	\N	2020-10-12 15:38:08.703177
57388	MNTX	2020-10-08	4.975	4.975	4.66	4.77	4848	{}	\N	2020-10-12 15:38:08.710435
57389	MNTX	2020-10-09	4.87	4.87	4.68	4.81	18569	{}	\N	2020-10-12 15:38:08.717145
57390	MNTX	2020-10-12	4.88	4.97	4.76	4.76	1001	{}	\N	2020-10-12 15:38:08.722965
57391	MGF	2020-09-28	4.66	4.66	4.64	4.65	103132	{}	\N	2020-10-12 15:38:09.660176
57392	MGF	2020-09-29	4.64	4.65	4.63	4.64	195538	{}	\N	2020-10-12 15:38:09.666374
57393	MGF	2020-09-30	4.65	4.65	4.63	4.63	173046	{}	\N	2020-10-12 15:38:09.672507
57394	MGF	2020-10-01	4.63	4.65	4.63	4.64	235876	{}	\N	2020-10-12 15:38:09.679401
57395	MGF	2020-10-02	4.65	4.66	4.64	4.66	78178	{}	\N	2020-10-12 15:38:09.6856
57396	MGF	2020-10-05	4.69	4.69	4.64	4.65	256546	{}	\N	2020-10-12 15:38:09.690851
57397	MGF	2020-10-06	4.66	4.67	4.63	4.64	194580	{}	\N	2020-10-12 15:38:09.697376
57398	MGF	2020-10-07	4.64	4.65	4.64	4.645	63722	{}	\N	2020-10-12 15:38:09.703119
57399	MGF	2020-10-08	4.64	4.65	4.6	4.6	577868	{}	\N	2020-10-12 15:38:09.709627
57400	MGF	2020-10-09	4.6	4.66	4.6	4.66	328008	{}	\N	2020-10-12 15:38:09.71579
57401	MGF	2020-10-12	4.62	4.64	4.62	4.64	155788	{}	\N	2020-10-12 15:38:09.721317
57402	MHI	2020-09-28	11.63	11.66	11.53	11.56	23042	{}	\N	2020-10-12 15:38:10.709471
57403	MHI	2020-09-29	11.54	11.63	11.53	11.61	50226	{}	\N	2020-10-12 15:38:10.715127
57404	MHI	2020-09-30	11.64	11.66	11.56	11.58	83050	{}	\N	2020-10-12 15:38:10.720495
57405	MHI	2020-10-01	11.63	11.74	11.5845	11.73	163826	{}	\N	2020-10-12 15:38:10.72705
57406	MHI	2020-10-02	11.65	11.7	11.61	11.66	38006	{}	\N	2020-10-12 15:38:10.732655
57407	MHI	2020-10-05	11.73	11.75	11.57	11.58	73936	{}	\N	2020-10-12 15:38:10.73851
57408	MHI	2020-10-06	11.55	11.65	11.55	11.57	90380	{}	\N	2020-10-12 15:38:10.744574
57409	MHI	2020-10-07	11.65	11.75	11.57	11.61	92152	{}	\N	2020-10-12 15:38:10.750314
57410	MHI	2020-10-08	11.67	11.74	11.59	11.63	47204	{}	\N	2020-10-12 15:38:10.755361
57411	MHI	2020-10-09	11.65	11.74	11.63	11.65	74181	{}	\N	2020-10-12 15:38:10.761319
57412	MHI	2020-10-12	11.7	11.77	11.67	11.69	41992	{}	\N	2020-10-12 15:38:10.767091
57413	MIN	2020-09-28	3.76	3.78	3.75	3.76	369433	{}	\N	2020-10-12 15:38:11.711385
57414	MIN	2020-09-29	3.75	3.76	3.73	3.74	1485790	{}	\N	2020-10-12 15:38:11.718257
57415	MIN	2020-09-30	3.76	3.78	3.73	3.73	1102548	{}	\N	2020-10-12 15:38:11.722918
57416	MIN	2020-10-01	3.75	3.75	3.72	3.73	1265148	{}	\N	2020-10-12 15:38:11.728098
57417	MIN	2020-10-02	3.72	3.77	3.71	3.74	905638	{}	\N	2020-10-12 15:38:11.733215
57418	MIN	2020-10-05	3.76	3.81	3.75	3.78	890392	{}	\N	2020-10-12 15:38:11.738337
57419	MIN	2020-10-06	3.78	3.81	3.76	3.79	526848	{}	\N	2020-10-12 15:38:11.743797
57420	MIN	2020-10-07	3.8	3.83	3.78	3.8	322142	{}	\N	2020-10-12 15:38:11.749828
57421	MIN	2020-10-08	3.82	3.82	3.78	3.79	360614	{}	\N	2020-10-12 15:38:11.756687
57422	MIN	2020-10-09	3.81	3.84	3.8	3.82	567112	{}	\N	2020-10-12 15:38:11.764248
57423	MIN	2020-10-12	3.81	3.83	3.81	3.81	128528	{}	\N	2020-10-12 15:38:11.76992
57424	MKL	2020-09-28	965.97	996.95	956.01	978.37	53100	{}	\N	2020-10-12 15:38:13.152069
57425	MKL	2020-09-29	974.83	978.99	958.1	968.65	86207	{}	\N	2020-10-12 15:38:13.157215
57426	MKL	2020-09-30	971.18	988.45	962.02	973.7	108922	{}	\N	2020-10-12 15:38:13.162205
57427	MKL	2020-10-01	981.29	987.05	954.905	965.65	81179	{}	\N	2020-10-12 15:38:13.16721
57428	MKL	2020-10-02	946.57	985.3	944.97	976.69	61794	{}	\N	2020-10-12 15:38:13.172888
57429	MKL	2020-10-05	981.07	999.88	967.1	996.78	112855	{}	\N	2020-10-12 15:38:13.179361
57430	MKL	2020-10-06	997.94	997.94	976.1201	978.95	95713	{}	\N	2020-10-12 15:38:13.184972
57431	MKL	2020-10-07	989.38	999.11	974.74	981.95	109751	{}	\N	2020-10-12 15:38:13.190166
57432	MKL	2020-10-08	983.35	999.7	983.35	998	66201	{}	\N	2020-10-12 15:38:13.195256
57433	MKL	2020-10-09	1000	1004.89	985.47	985.47	73067	{}	\N	2020-10-12 15:38:13.200216
57434	MKL	2020-10-12	982.31	1009	982.31	1006.243	18390	{}	\N	2020-10-12 15:38:13.205033
57435	MLP	2020-09-28	11.0121	11.25	10.935	11.08	11909	{}	\N	2020-10-12 15:38:14.287166
57436	MLP	2020-09-29	11.15	11.255	10.88	11.1	18622	{}	\N	2020-10-12 15:38:14.293797
57437	MLP	2020-09-30	11.04	11.27	10.78	10.82	16709	{}	\N	2020-10-12 15:38:14.300144
57438	MLP	2020-10-01	10.92	11.14	10.69	11	46134	{}	\N	2020-10-12 15:38:14.307758
57439	MLP	2020-10-02	10.9135	10.96	10.67	10.72	14908	{}	\N	2020-10-12 15:38:14.315022
57440	MLP	2020-10-05	10.83	11.25	10.745	10.98	36092	{}	\N	2020-10-12 15:38:14.320585
57441	MLP	2020-10-06	11.26	11.51	10.9201	11.37	53315	{}	\N	2020-10-12 15:38:14.326642
57442	MLP	2020-10-07	11.75	11.915	11.33	11.62	31368	{}	\N	2020-10-12 15:38:14.332881
57443	MLP	2020-10-08	11.91	11.91	11.41	11.42	23540	{}	\N	2020-10-12 15:38:14.339087
57444	MLP	2020-10-09	11.4	11.57	11.396	11.49	13406	{}	\N	2020-10-12 15:38:14.345267
57445	MLP	2020-10-12	11.61	11.6724	11.5488	11.5845	4172	{}	\N	2020-10-12 15:38:14.352195
57446	MLPO	2020-09-28	6.19	6.19	6.19	6.19	8	{}	\N	2020-10-12 15:38:15.567202
57447	MLPO	2020-09-29	6.075	6.075	6.075	6.075	5	{}	\N	2020-10-12 15:38:15.573195
57448	MLPO	2020-10-05	6.39	7.5891	6.39	7.5891	4550	{}	\N	2020-10-12 15:38:15.578695
57449	MLPO	2020-10-06	7.65	7.65	7.65	7.65	219	{}	\N	2020-10-12 15:38:15.583955
57450	MLPO	2020-10-07	7.485	7.485	7.485	7.485	53	{}	\N	2020-10-12 15:38:15.589389
57451	MLPO	2020-10-08	7.695	7.695	7.695	7.695	222	{}	\N	2020-10-12 15:38:15.595442
57452	MLPO	2020-10-12	8.45	8.5	7.82	7.95	15228	{}	\N	2020-10-12 15:38:15.601008
57453	MMAC	2020-09-28	22.56	23	22.56	22.75	3440	{}	\N	2020-10-12 15:38:16.74524
57454	MMAC	2020-09-29	22.66	22.71	22.5	22.6	3019	{}	\N	2020-10-12 15:38:16.752642
57455	MMAC	2020-09-30	22.9824	22.9824	22.51	22.51	4702	{}	\N	2020-10-12 15:38:16.758997
57456	MMAC	2020-10-01	22.74	22.75	22.5	22.75	3183	{}	\N	2020-10-12 15:38:16.766382
57457	MMAC	2020-10-02	22.03	22.63	22.03	22.33	5077	{}	\N	2020-10-12 15:38:16.772262
57458	MMAC	2020-10-05	22.37	22.74	22.01	22.62	8663	{}	\N	2020-10-12 15:38:16.778084
57459	MMAC	2020-10-06	22.68	23.06	22.3328	22.53	18725	{}	\N	2020-10-12 15:38:16.784433
57460	MMAC	2020-10-07	22.78	23.84	22.78	23.84	13115	{}	\N	2020-10-12 15:38:16.789675
57461	MMAC	2020-10-08	23.9	23.9	23.0053	23.17	6716	{}	\N	2020-10-12 15:38:16.794885
57462	MMAC	2020-10-09	22.69	23.65	22.69	23.6	6896	{}	\N	2020-10-12 15:38:16.801134
57463	MMIT	2020-09-28	26.421	26.4399	26.4	26.425	13639	{}	\N	2020-10-12 15:38:20.999105
57464	MMIT	2020-09-29	26.47	26.47	26.4	26.4156	55769	{}	\N	2020-10-12 15:38:21.005266
57465	MMIT	2020-09-30	26.4099	26.4099	26.38	26.39	27747	{}	\N	2020-10-12 15:38:21.010392
57466	MMIT	2020-10-01	26.34	26.34	26.32	26.325	13281	{}	\N	2020-10-12 15:38:21.015498
57467	MMIT	2020-10-02	26.29	26.32	26.29	26.3198	19401	{}	\N	2020-10-12 15:38:21.022142
57468	MMIT	2020-10-05	26.3	26.32	26.3	26.31	30656	{}	\N	2020-10-12 15:38:21.028425
57469	MMIT	2020-10-06	26.3	26.3	26.26	26.2815	18012	{}	\N	2020-10-12 15:38:21.035005
57470	MMIT	2020-10-07	26.23	26.26	26.23	26.23	11714	{}	\N	2020-10-12 15:38:21.040638
57471	MMIT	2020-10-08	26.21	26.25	26.21	26.225	27135	{}	\N	2020-10-12 15:38:21.046168
57472	MMIT	2020-10-09	26.2	26.25	26.2	26.22	15034	{}	\N	2020-10-12 15:38:21.052398
57473	MMIT	2020-10-12	26.23	26.25	26.22	26.25	33726	{}	\N	2020-10-12 15:38:21.05898
57474	MMT	2020-09-28	5.74	5.76	5.7002	5.74	118929	{}	\N	2020-10-12 15:38:22.390968
57475	MMT	2020-09-29	5.74	5.74	5.68	5.71	147904	{}	\N	2020-10-12 15:38:22.398801
57476	MMT	2020-09-30	5.75	5.76	5.62	5.62	1196618	{}	\N	2020-10-12 15:38:22.404008
57477	MMT	2020-10-01	5.64	5.71	5.64	5.69	284135	{}	\N	2020-10-12 15:38:22.410188
57478	MMT	2020-10-02	5.68	5.78	5.68	5.77	192318	{}	\N	2020-10-12 15:38:22.416302
57479	MMT	2020-10-05	5.76	5.81	5.76	5.77	272304	{}	\N	2020-10-12 15:38:22.423039
57480	MMT	2020-10-06	5.78	5.82	5.77	5.77	145113	{}	\N	2020-10-12 15:38:22.42936
57481	MMT	2020-10-07	5.82	5.9	5.82	5.86	288172	{}	\N	2020-10-12 15:38:22.435433
57482	MMT	2020-10-08	5.87	5.92	5.8	5.8	186385	{}	\N	2020-10-12 15:38:22.442029
57483	MMT	2020-10-09	5.88	5.89	5.8502	5.88	120979	{}	\N	2020-10-12 15:38:22.447245
57484	MMT	2020-10-12	5.87	5.89	5.87	5.89	38234	{}	\N	2020-10-12 15:38:22.452335
57485	MOD	2020-09-28	6.09	6.39	6.09	6.29	350678	{}	\N	2020-10-12 15:38:24.086688
57486	MOD	2020-09-29	6.29	6.615	6.29	6.42	883208	{}	\N	2020-10-12 15:38:24.093222
57487	MOD	2020-09-30	6.39	6.47	6.18	6.25	864756	{}	\N	2020-10-12 15:38:24.099305
57488	MOD	2020-10-01	6.35	6.515	6.28	6.36	493273	{}	\N	2020-10-12 15:38:24.104786
57489	MOD	2020-10-02	6.25	6.79	6.225	6.62	766124	{}	\N	2020-10-12 15:38:24.110912
57490	MOD	2020-10-05	6.72	7.01	6.6859	7	527972	{}	\N	2020-10-12 15:38:24.115995
57491	MOD	2020-10-06	7.13	7.345	6.9	7.16	1106115	{}	\N	2020-10-12 15:38:24.120852
57492	MOD	2020-10-07	7.27	7.61	7.25	7.58	686907	{}	\N	2020-10-12 15:38:24.132896
57493	MOD	2020-10-08	7.72	7.77	7.09	7.3	802164	{}	\N	2020-10-12 15:38:24.14283
57494	MOD	2020-10-09	7.4	7.49	7.1	7.16	315347	{}	\N	2020-10-12 15:38:24.150852
57495	MOD	2020-10-12	7.1	7.25	6.87	7.12	233964	{}	\N	2020-10-12 15:38:24.158979
57496	MORN	2020-09-28	157.43	159.4	157.1	158.61	98264	{}	\N	2020-10-12 15:38:25.490774
57497	MORN	2020-09-29	159.33	160.88	157.96	159.3	75960	{}	\N	2020-10-12 15:38:25.499809
57498	MORN	2020-09-30	159.83	161.43	158.47	160.61	100729	{}	\N	2020-10-12 15:38:25.505438
57499	MORN	2020-10-01	160.94	163.58	159.23	163.19	146404	{}	\N	2020-10-12 15:38:25.512902
57500	MORN	2020-10-02	162	164.8	161.36	164.01	146646	{}	\N	2020-10-12 15:38:25.519094
57501	MORN	2020-10-05	164.99	164.99	161.53	163.77	143170	{}	\N	2020-10-12 15:38:25.526851
57502	MORN	2020-10-06	163.59	166.46	163.46	165.79	97411	{}	\N	2020-10-12 15:38:25.532042
57503	MORN	2020-10-07	165.83	170.11	165.63	170	123669	{}	\N	2020-10-12 15:38:25.5373
57504	MORN	2020-10-08	171.2	178.6	171.16	178.38	133632	{}	\N	2020-10-12 15:38:25.54348
57505	MORN	2020-10-09	177.5358	182.09	177.5358	180.3	81230	{}	\N	2020-10-12 15:38:25.549068
57506	MORN	2020-10-12	181.87	184.57	181.29	183.65	43059	{}	\N	2020-10-12 15:38:25.5556
57507	MGU	2020-09-28	17.4	17.44	17.21	17.42	26329	{}	\N	2020-10-12 15:38:26.496407
57508	MGU	2020-09-29	17.44	17.44	17.125	17.3	133209	{}	\N	2020-10-12 15:38:26.501773
57509	MGU	2020-09-30	17.26	17.55	17.26	17.55	101408	{}	\N	2020-10-12 15:38:26.50724
57510	MGU	2020-10-01	17.63	17.63	17.5035	17.58	58322	{}	\N	2020-10-12 15:38:26.514195
57511	MGU	2020-10-02	17.35	17.65	17.34	17.56	121605	{}	\N	2020-10-12 15:38:26.52088
57512	MGU	2020-10-05	17.69	17.7328	17.54	17.63	100260	{}	\N	2020-10-12 15:38:26.52952
57513	MGU	2020-10-06	17.76	17.8952	17.56	17.62	47205	{}	\N	2020-10-12 15:38:26.53742
57514	MGU	2020-10-07	17.76	17.96	17.69	17.875	46984	{}	\N	2020-10-12 15:38:26.542507
57515	MGU	2020-10-08	17.81	18.0901	17.81	18.03	51430	{}	\N	2020-10-12 15:38:26.549198
57516	MGU	2020-10-09	18.23	18.23	17.95	17.95	67893	{}	\N	2020-10-12 15:38:26.554832
57517	MGU	2020-10-12	18.14	18.2702	17.98	18.05	21022	{}	\N	2020-10-12 15:38:26.561958
57518	MHK	2020-09-28	97.28	99.6	95.72	98.42	583579	{}	\N	2020-10-12 15:38:27.63849
57519	MHK	2020-09-29	97.91	98.43	95.61	96.27	820888	{}	\N	2020-10-12 15:38:27.64479
57520	MHK	2020-09-30	95.65	98.745	95.58	97.59	1448329	{}	\N	2020-10-12 15:38:27.652254
57521	MHK	2020-10-01	99.24	100.175	96.79	98.4	983451	{}	\N	2020-10-12 15:38:27.65807
57522	MHK	2020-10-02	95.7	102.48	95.53	102.19	1006658	{}	\N	2020-10-12 15:38:27.665574
57523	MHK	2020-10-05	103.53	107.33	103.53	106.98	891014	{}	\N	2020-10-12 15:38:27.672567
57524	MHK	2020-10-06	107.04	109.16	104.32	104.91	988112	{}	\N	2020-10-12 15:38:27.678842
57525	MHK	2020-10-07	106.72	108.635	105.49	106.38	732434	{}	\N	2020-10-12 15:38:27.685636
57526	MHK	2020-10-08	107.63	109.58	107.25	109.01	827299	{}	\N	2020-10-12 15:38:27.692424
57527	MHK	2020-10-09	110	110	102.95	104.24	1442811	{}	\N	2020-10-12 15:38:27.698885
57528	MHK	2020-10-12	106.36	106.95	104	104.225	262290	{}	\N	2020-10-12 15:38:27.704674
57529	MIDU	2020-09-28	20.96	21.806	20.96	21.66	137884	{}	\N	2020-10-12 15:38:28.816214
57530	MIDU	2020-09-29	21.81	21.86	20.94	21.32	48875	{}	\N	2020-10-12 15:38:28.82203
57531	MIDU	2020-09-30	21.49	22.33	21.3199	21.7	123831	{}	\N	2020-10-12 15:38:28.827139
57532	MIDU	2020-10-01	21.98	22.5	21.87	22.4975	84282	{}	\N	2020-10-12 15:38:28.833262
57533	MIDU	2020-10-02	21.39	23.45	21.335	23.18	143680	{}	\N	2020-10-12 15:38:28.839189
57534	MIDU	2020-10-05	23.84	24.79	23.825	24.79	83017	{}	\N	2020-10-12 15:38:28.844044
57535	MIDU	2020-10-06	25.25	26.16	24.24	24.4	136338	{}	\N	2020-10-12 15:38:28.85014
57536	MIDU	2020-10-07	25.23	25.89	25.23	25.68	113652	{}	\N	2020-10-12 15:38:28.856556
57537	MIDU	2020-10-08	26.32	26.52	25.9601	26.52	92137	{}	\N	2020-10-12 15:38:28.861436
57538	MIDU	2020-10-09	27.09	27.28	26.5	26.72	76923	{}	\N	2020-10-12 15:38:28.867087
57539	MIDU	2020-10-12	27.02	27.31	26.81	27.23	17171	{}	\N	2020-10-12 15:38:28.87215
57540	MIND	2020-09-28	2.1	2.18	2.01	2.04	133578	{}	\N	2020-10-12 15:38:30.361559
57541	MIND	2020-09-29	2.01	2.12	2.01	2.09	129778	{}	\N	2020-10-12 15:38:30.366969
57542	MIND	2020-09-30	2.12	2.16	2.08	2.08	59178	{}	\N	2020-10-12 15:38:30.371998
57543	MIND	2020-10-01	2.09	2.2	2.09	2.17	77318	{}	\N	2020-10-12 15:38:30.376958
57544	MIND	2020-10-02	2.11	2.2	2.11	2.17	31790	{}	\N	2020-10-12 15:38:30.383113
57545	MIND	2020-10-05	2.17	2.19	2.1201	2.16	22538	{}	\N	2020-10-12 15:38:30.389112
57546	MIND	2020-10-06	2.19	2.36	2.16	2.35	111309	{}	\N	2020-10-12 15:38:30.394919
57547	MIND	2020-10-07	2.34	2.41	2.3151	2.37	31561	{}	\N	2020-10-12 15:38:30.400607
57548	MIND	2020-10-08	2.38	2.43	2.25	2.42	70388	{}	\N	2020-10-12 15:38:30.406759
57549	MIND	2020-10-09	2.45	2.48	2.4216	2.43	56194	{}	\N	2020-10-12 15:38:30.411931
57550	MIND	2020-10-12	2.45	2.45	2.4	2.4195	10555	{}	\N	2020-10-12 15:38:30.417558
57551	MKC	2020-09-28	194.74	196.56	193.78	195.18	1446170	{}	\N	2020-10-12 15:38:31.301507
57552	MKC	2020-09-29	192.8	193.16	183.22	189.89	4877838	{}	\N	2020-10-12 15:38:31.308216
57553	MKC	2020-09-30	191.38	196.17	191	194.1	1861644	{}	\N	2020-10-12 15:38:31.314021
57554	MKC	2020-10-01	194.74	195.455	191.13	193.2	1481533	{}	\N	2020-10-12 15:38:31.32196
57555	MKC	2020-10-02	190.77	193.91	190.15	192.84	987481	{}	\N	2020-10-12 15:38:31.328261
57556	MKC	2020-10-05	193.33	195.34	192.81	195.08	803597	{}	\N	2020-10-12 15:38:31.334627
57557	MKC	2020-10-06	195.77	195.77	191.24	191.64	1081743	{}	\N	2020-10-12 15:38:31.34002
57558	MKC	2020-10-07	192.66	195.04	192.12	194.18	981682	{}	\N	2020-10-12 15:38:31.344855
57559	MKC	2020-10-08	194.92	195.25	193.67	194.33	702471	{}	\N	2020-10-12 15:38:31.350363
57560	MKC	2020-10-09	195.79	197.03	195.27	196.11	690333	{}	\N	2020-10-12 15:38:31.356673
57561	MKC	2020-10-12	196.11	200.1	195.99	199.94	331725	{}	\N	2020-10-12 15:38:31.363338
57562	MLN	2020-09-28	21.44	21.45	21.4093	21.445	17767	{}	\N	2020-10-12 15:38:32.77038
57563	MLN	2020-09-29	21.46	21.47	21.4001	21.46	103844	{}	\N	2020-10-12 15:38:32.77891
57564	MLN	2020-09-30	21.46	21.46	21.36	21.39	19626	{}	\N	2020-10-12 15:38:32.785092
57565	MLN	2020-10-01	21.35	21.35	21.31	21.35	248872	{}	\N	2020-10-12 15:38:32.79019
57566	MLN	2020-10-02	21.35	21.35	21.3	21.32	112912	{}	\N	2020-10-12 15:38:32.796427
57567	MLN	2020-10-05	21.3	21.319	21.22	21.22	58663	{}	\N	2020-10-12 15:38:32.802426
57568	MLN	2020-10-06	21.29	21.3	21.21	21.22	69814	{}	\N	2020-10-12 15:38:32.808307
57569	MLN	2020-10-07	21.205	21.2499	21.13	21.14	38790	{}	\N	2020-10-12 15:38:32.813831
57570	MLN	2020-10-08	21.13	21.22	21.13	21.19	61805	{}	\N	2020-10-12 15:38:32.820116
57571	MLN	2020-10-09	21.21	21.21	21.15	21.16	37834	{}	\N	2020-10-12 15:38:32.826457
57572	MLN	2020-10-12	21.13	21.21	21.12	21.183	9443	{}	\N	2020-10-12 15:38:32.831974
57573	MLR	2020-09-28	30.9	31.81	30.85	31.58	85221	{}	\N	2020-10-12 15:38:34.267988
57574	MLR	2020-09-29	31.48	31.6	31.02	31.07	90407	{}	\N	2020-10-12 15:38:34.275001
57575	MLR	2020-09-30	31.13	31.5	30.465	30.57	117103	{}	\N	2020-10-12 15:38:34.281298
57576	MLR	2020-10-01	30.61	31.15	30.6	30.95	91281	{}	\N	2020-10-12 15:38:34.286615
57577	MLR	2020-10-02	30.51	31.076	30.49	30.79	136468	{}	\N	2020-10-12 15:38:34.292513
57578	MLR	2020-10-05	30.91	31.66	30.91	31.43	83047	{}	\N	2020-10-12 15:38:34.298469
57579	MLR	2020-10-06	31.7	32.4674	31.6	31.79	61520	{}	\N	2020-10-12 15:38:34.303741
57580	MLR	2020-10-07	32.11	32.34	31.52	31.71	85679	{}	\N	2020-10-12 15:38:34.30874
57581	MLR	2020-10-08	32	32.05	31.42	31.69	82118	{}	\N	2020-10-12 15:38:34.314211
57582	MLR	2020-10-09	31.88	31.9368	31.37	31.48	70489	{}	\N	2020-10-12 15:38:34.320849
57583	MLR	2020-10-12	31.42	31.8099	31.3	31.48	10732	{}	\N	2020-10-12 15:38:34.325231
57584	MMP	2020-09-28	34.11	35.45	34.02	35.01	1951585	{}	\N	2020-10-12 15:38:38.211578
57585	MMP	2020-09-29	34.95	35.591	34.565	34.9	2283320	{}	\N	2020-10-12 15:38:38.217989
57586	MMP	2020-09-30	35.01	35.52	34.105	34.2	3096179	{}	\N	2020-10-12 15:38:38.222611
57587	MMP	2020-10-01	34.15	34.63	33.37	33.63	2464606	{}	\N	2020-10-12 15:38:38.22713
57588	MMP	2020-10-02	33	34.84	32.61	34.67	2179335	{}	\N	2020-10-12 15:38:38.232832
57589	MMP	2020-10-05	35.18	35.82	34.93	35.7	1685632	{}	\N	2020-10-12 15:38:38.238649
57590	MMP	2020-10-06	36.17	36.38	35.04	35.22	3186227	{}	\N	2020-10-12 15:38:38.243684
57591	MMP	2020-10-07	35.22	35.84	35.11	35.4	1971683	{}	\N	2020-10-12 15:38:38.249257
57592	MMP	2020-10-08	35.52	36.97	35.46	36.93	2069875	{}	\N	2020-10-12 15:38:38.254225
57593	MMP	2020-10-09	37.09	37.3253	36.35	37.06	1761553	{}	\N	2020-10-12 15:38:38.259452
57594	MMP	2020-10-12	37	37.46	36.95	37.405	448923	{}	\N	2020-10-12 15:38:38.265379
57595	MNDO	2020-09-28	2.35	2.36	2.32	2.3437	11100	{}	\N	2020-10-12 15:38:39.822544
57596	MNDO	2020-09-29	2.3406	2.38	2.318	2.32	20102	{}	\N	2020-10-12 15:38:39.827644
57597	MNDO	2020-09-30	2.31	2.34	2.31	2.32	5770	{}	\N	2020-10-12 15:38:39.832798
57598	MNDO	2020-10-01	2.32	2.33	2.31	2.3198	10572	{}	\N	2020-10-12 15:38:39.838495
57599	MNDO	2020-10-02	2.31	2.33	2.31	2.33	22928	{}	\N	2020-10-12 15:38:39.843449
57600	MNDO	2020-10-05	2.33	2.36	2.33	2.3599	30822	{}	\N	2020-10-12 15:38:39.848306
57601	MNDO	2020-10-06	2.36	2.42	2.3598	2.4116	21265	{}	\N	2020-10-12 15:38:39.85324
57602	MNDO	2020-10-07	2.41	2.42	2.38	2.38	7827	{}	\N	2020-10-12 15:38:39.858477
57603	MNDO	2020-10-08	2.4	2.4099	2.36	2.375	5088	{}	\N	2020-10-12 15:38:39.86384
57604	MNDO	2020-10-09	2.38	2.39	2.34	2.34	24900	{}	\N	2020-10-12 15:38:39.868943
57605	MNDO	2020-10-12	2.3507	2.3699	2.35	2.3699	587	{}	\N	2020-10-12 15:38:39.873418
57606	MNST	2020-09-28	79.91	80.73	79.44	80.51	2139459	{}	\N	2020-10-12 15:38:41.109627
57607	MNST	2020-09-29	80.89	81.07	79.59	79.79	1381021	{}	\N	2020-10-12 15:38:41.124801
57608	MNST	2020-09-30	80.33	81.06	79.64	80.2	3176458	{}	\N	2020-10-12 15:38:41.131882
57609	MNST	2020-10-01	81.05	81.76	80.24	81.48	4551808	{}	\N	2020-10-12 15:38:41.140405
57610	MNST	2020-10-02	80.33	80.83	78.74	79.55	1804294	{}	\N	2020-10-12 15:38:41.147348
57611	MNST	2020-10-05	79.71	80.12	78.02	78.67	2209669	{}	\N	2020-10-12 15:38:41.153399
57612	MNST	2020-10-06	78.62	79.37	77.35	77.63	1656163	{}	\N	2020-10-12 15:38:41.159795
57613	MNST	2020-10-07	77.88	78.99	77.63	78.76	1607312	{}	\N	2020-10-12 15:38:41.164154
57614	MNST	2020-10-08	79.19	79.35	78.6	79.17	1404484	{}	\N	2020-10-12 15:38:41.169475
57615	MNST	2020-10-09	79.49	81.2	79.37	80.95	2352089	{}	\N	2020-10-12 15:38:41.174386
57616	MNST	2020-10-12	82.03	82.62	81.56	82.59	1006083	{}	\N	2020-10-12 15:38:41.17934
57617	MPA	2020-09-28	13.82	13.96	13.82	13.855	16604	{}	\N	2020-10-12 15:38:42.878211
57618	MPA	2020-09-29	13.93	13.9967	13.86	13.87	46064	{}	\N	2020-10-12 15:38:42.883731
57619	MPA	2020-09-30	13.84	13.9293	13.84	13.91	20975	{}	\N	2020-10-12 15:38:42.888811
57620	MPA	2020-10-01	14.01	14.01	13.9251	14.01	31953	{}	\N	2020-10-12 15:38:42.893172
57621	MPA	2020-10-02	14.03	14.13	13.98	14.03	33482	{}	\N	2020-10-12 15:38:42.898428
57622	MPA	2020-10-05	13.985	14.0092	13.82	13.89	72602	{}	\N	2020-10-12 15:38:42.906063
57623	MPA	2020-10-06	13.8704	13.9783	13.82	13.91	25888	{}	\N	2020-10-12 15:38:42.911252
57624	MPA	2020-10-07	13.83	13.94	13.79	13.82	44760	{}	\N	2020-10-12 15:38:42.915607
57625	MPA	2020-10-08	13.74	13.88	13.74	13.82	100218	{}	\N	2020-10-12 15:38:42.920737
57626	MPA	2020-10-09	13.8445	13.94	13.77	13.8402	74350	{}	\N	2020-10-12 15:38:42.925148
57627	MPA	2020-10-12	13.81	13.9	13.81	13.9	10430	{}	\N	2020-10-12 15:38:42.929393
57628	MPX	2020-09-28	15.7	16.0599	15.04	15.34	73974	{}	\N	2020-10-12 15:38:44.819126
57629	MPX	2020-09-29	15.23	15.6	15.22	15.38	128124	{}	\N	2020-10-12 15:38:44.826995
57630	MPX	2020-09-30	15.22	15.88	15.22	15.64	190226	{}	\N	2020-10-12 15:38:44.833609
57631	MPX	2020-10-01	15.99	17.12	15.78	17.02	208587	{}	\N	2020-10-12 15:38:44.839206
57632	MPX	2020-10-02	16.72	17.5	16.72	17.23	92567	{}	\N	2020-10-12 15:38:44.844634
57633	MPX	2020-10-05	17.47	17.9	17.33	17.43	143163	{}	\N	2020-10-12 15:38:44.850024
57634	MPX	2020-10-06	17.5	18.26	17.47	17.6	102879	{}	\N	2020-10-12 15:38:44.855565
57635	MPX	2020-10-07	17.84	18.355	16.93	17.47	234496	{}	\N	2020-10-12 15:38:44.862193
57636	MPX	2020-10-08	17.7	17.7	17.24	17.3	66075	{}	\N	2020-10-12 15:38:44.867902
57637	MPX	2020-10-09	17.06	17.7	16.91	17.22	104291	{}	\N	2020-10-12 15:38:44.873994
57638	MPX	2020-10-12	17.3	17.3	16.9337	17.01	8788	{}	\N	2020-10-12 15:38:44.880122
57639	MRK	2020-09-28	83.2	83.7	82.56	82.76	5984764	{}	\N	2020-10-12 15:38:45.817556
57640	MRK	2020-09-29	82.675	83.15	81.7	81.9	13093365	{}	\N	2020-10-12 15:38:45.826293
57641	MRK	2020-09-30	82.26	83.32	82.02	82.95	21916885	{}	\N	2020-10-12 15:38:45.832427
57642	MRK	2020-10-01	83.18	83.34	81.25	81.64	15172835	{}	\N	2020-10-12 15:38:45.837841
57643	MRK	2020-10-02	80.8	81.74	80.45	80.8	10151789	{}	\N	2020-10-12 15:38:45.844868
57644	MRK	2020-10-05	81.23	81.54	80.62	81.24	16406557	{}	\N	2020-10-12 15:38:45.851191
57645	MRK	2020-10-06	81.13	81.14	79.52	79.63	19502666	{}	\N	2020-10-12 15:38:45.857231
57646	MRK	2020-10-07	80.09	80.35	79.51	80.04	19143629	{}	\N	2020-10-12 15:38:45.863228
57647	MRK	2020-10-08	80.39	81.835	80.16	80.48	17669430	{}	\N	2020-10-12 15:38:45.869456
57648	MRK	2020-10-09	80.88	81.085	80.05	80.36	10831610	{}	\N	2020-10-12 15:38:45.875841
57649	MRK	2020-10-12	80.51	80.72	79.91	80.53	5214585	{}	\N	2020-10-12 15:38:45.881534
57650	MHO	2020-09-28	43.6	45.615	43.58	45.34	213611	{}	\N	2020-10-12 15:38:47.291878
57651	MHO	2020-09-29	45.42	45.42	43.73	44.17	649743	{}	\N	2020-10-12 15:38:47.29764
57652	MHO	2020-09-30	44.36	46.81	44.34	46.05	752482	{}	\N	2020-10-12 15:38:47.302629
57653	MHO	2020-10-01	46.35	46.74	44.36	46.08	502765	{}	\N	2020-10-12 15:38:47.308605
57654	MHO	2020-10-02	44.68	47.33	44.68	46.78	300937	{}	\N	2020-10-12 15:38:47.313749
57655	MHO	2020-10-05	47.39	48.1	44.79	45.3	607555	{}	\N	2020-10-12 15:38:47.319753
57656	MHO	2020-10-06	45.75	45.99	43.6065	43.88	465107	{}	\N	2020-10-12 15:38:47.32708
57657	MHO	2020-10-07	44.63	45.1	43.11	44.49	625073	{}	\N	2020-10-12 15:38:47.333998
57658	MHO	2020-10-08	45.39	46.9	45.13	46.75	397958	{}	\N	2020-10-12 15:38:47.339907
57659	MHO	2020-10-09	47.19	47.87	46.5	47.51	388392	{}	\N	2020-10-12 15:38:47.34638
57660	MHO	2020-10-12	47.98	48.83	47.25	48.22	159726	{}	\N	2020-10-12 15:38:47.351913
57661	MINT	2020-09-28	101.96	101.98	101.96	101.97	1394221	{}	\N	2020-10-12 15:38:48.647612
57662	MINT	2020-09-29	101.98	101.98	101.96	101.96	854654	{}	\N	2020-10-12 15:38:48.654145
57663	MINT	2020-09-30	101.97	101.98	101.95	101.96	873157	{}	\N	2020-10-12 15:38:48.660469
57664	MINT	2020-10-01	101.91	101.93	101.91	101.92	1030198	{}	\N	2020-10-12 15:38:48.667128
57665	MINT	2020-10-02	101.93	101.94	101.91	101.91	1036209	{}	\N	2020-10-12 15:38:48.67271
57666	MINT	2020-10-05	101.92	101.94	101.916	101.92	861917	{}	\N	2020-10-12 15:38:48.678881
57667	MINT	2020-10-06	101.9375	101.94	101.9	101.91	1460558	{}	\N	2020-10-12 15:38:48.684245
57668	MINT	2020-10-07	101.92	101.94	101.91	101.915	807121	{}	\N	2020-10-12 15:38:48.689746
57669	MINT	2020-10-08	101.94	101.94	101.91	101.93	833098	{}	\N	2020-10-12 15:38:48.695728
57670	MINT	2020-10-09	101.92	101.95	101.92	101.95	844329	{}	\N	2020-10-12 15:38:48.701227
57671	MINT	2020-10-12	101.92	101.94	101.92	101.935	260154	{}	\N	2020-10-12 15:38:48.706857
57672	MLHR	2020-09-28	30.78	31.1	30.01	30.34	768469	{}	\N	2020-10-12 15:38:49.871381
57673	MLHR	2020-09-29	30.21	30.535	29.45	30.43	864089	{}	\N	2020-10-12 15:38:49.878625
57674	MLHR	2020-09-30	30.46	31.54	29.99	30.16	720470	{}	\N	2020-10-12 15:38:49.883926
57675	MLHR	2020-10-01	30.375	32.105	30.375	31.9	1039290	{}	\N	2020-10-12 15:38:49.889117
57676	MLHR	2020-10-02	30.88	33.86	30.63	33.65	976846	{}	\N	2020-10-12 15:38:49.895461
57677	MLHR	2020-10-05	33.99	34.09	31.8	32.285	1314185	{}	\N	2020-10-12 15:38:49.900974
57678	MLHR	2020-10-06	32.39	33.195	31.88	31.94	558758	{}	\N	2020-10-12 15:38:49.906535
57679	MLHR	2020-10-07	32.32	33.32	32.02	32.27	651637	{}	\N	2020-10-12 15:38:49.913294
57680	MLHR	2020-10-08	32.965	33.16	31.3	31.7	935914	{}	\N	2020-10-12 15:38:49.918652
57681	MLHR	2020-10-09	32.3647	32.3647	31.31	31.75	448969	{}	\N	2020-10-12 15:38:49.924119
57682	MLHR	2020-10-12	31.81	32.3	31.5901	31.9	122298	{}	\N	2020-10-12 15:38:49.929827
57683	MLPI	2020-09-28	9.19	9.47	9.16	9.4	125470	{}	\N	2020-10-12 15:38:51.136957
57684	MLPI	2020-09-29	9.34	9.554	9.33	9.53	13025	{}	\N	2020-10-12 15:38:51.144291
57685	MLPI	2020-09-30	9.59	9.7	9.28	9.38	41337	{}	\N	2020-10-12 15:38:51.151185
57686	MLPI	2020-10-01	9.27	9.4	9.14	9.3	22484	{}	\N	2020-10-12 15:38:51.157641
57687	MLPI	2020-10-02	9.16	9.45	9.14	9.39	30923	{}	\N	2020-10-12 15:38:51.165627
57688	MLPI	2020-10-05	9.47	9.8	9.47	9.76	47261	{}	\N	2020-10-12 15:38:51.172926
57689	MLPI	2020-10-06	9.75	9.9599	9.51	9.56	98371	{}	\N	2020-10-12 15:38:51.178808
57690	MLPI	2020-10-07	9.57	9.62	9.53	9.6	23522	{}	\N	2020-10-12 15:38:51.184397
57691	MLPI	2020-10-08	9.81	10.19	9.7903	10.19	20139	{}	\N	2020-10-12 15:38:51.189929
57692	MLPI	2020-10-09	10.06	10.06	9.8398	9.86	24123	{}	\N	2020-10-12 15:38:51.195145
57693	MLPI	2020-10-12	9.8321	9.8321	9.815	9.8235	1186	{}	\N	2020-10-12 15:38:51.201298
57694	MMIN	2020-09-28	27.21	27.26	27.21	27.24	43932	{}	\N	2020-10-12 15:38:52.27209
57695	MMIN	2020-09-29	27.26	27.28	27.22	27.25	84338	{}	\N	2020-10-12 15:38:52.279043
57696	MMIN	2020-09-30	27.29	27.29	27.199	27.2	92652	{}	\N	2020-10-12 15:38:52.286089
57697	MMIN	2020-10-01	27.22	27.22	27.1533	27.17	81794	{}	\N	2020-10-12 15:38:52.29322
57698	MMIN	2020-10-02	27.165	27.165	27.1201	27.16	31049	{}	\N	2020-10-12 15:38:52.299912
57699	MMIN	2020-10-05	27.135	27.15	27.1	27.12	30449	{}	\N	2020-10-12 15:38:52.3061
57700	MMIN	2020-10-06	27.16	27.16	27.1	27.11	52960	{}	\N	2020-10-12 15:38:52.311475
57701	MMIN	2020-10-07	27.11	27.11	27.04	27.05	60085	{}	\N	2020-10-12 15:38:52.317479
57702	MMIN	2020-10-08	27.04	27.08	27.0216	27.05	73675	{}	\N	2020-10-12 15:38:52.323397
57703	MMIN	2020-10-09	27.12	27.12	27.03	27.09	100805	{}	\N	2020-10-12 15:38:52.329989
57704	MMIN	2020-10-12	27.01	27.0799	27.01	27.035	19695	{}	\N	2020-10-12 15:38:52.335466
57705	MMYT	2020-09-28	15.9	16.05	15.57	15.83	296537	{}	\N	2020-10-12 15:38:53.25249
57706	MMYT	2020-09-29	15.61	15.86	15.26	15.38	247159	{}	\N	2020-10-12 15:38:53.261245
57707	MMYT	2020-09-30	15.52	15.7092	15.26	15.36	170298	{}	\N	2020-10-12 15:38:53.267047
57708	MMYT	2020-10-01	15.48	16.05	15.48	15.99	271142	{}	\N	2020-10-12 15:38:53.272776
57709	MMYT	2020-10-02	15.65	15.88	15.405	15.79	94831	{}	\N	2020-10-12 15:38:53.278711
57710	MMYT	2020-10-05	16	16	15.35	15.45	197107	{}	\N	2020-10-12 15:38:53.285055
57711	MMYT	2020-10-06	15.655	16.54	15.605	16.19	357604	{}	\N	2020-10-12 15:38:53.29055
57712	MMYT	2020-10-07	16.2	16.73	16.16	16.43	208316	{}	\N	2020-10-12 15:38:53.296898
57713	MMYT	2020-10-08	17.23	18.73	16.775	18.29	521119	{}	\N	2020-10-12 15:38:53.303456
57714	MMYT	2020-10-09	18.36	18.36	17.5	17.78	180935	{}	\N	2020-10-12 15:38:53.308699
57715	MMYT	2020-10-12	17.77	18.03	17.07	17.14	51060	{}	\N	2020-10-12 15:38:53.314041
57716	MNOV	2020-09-28	4.93	5.04	4.75	4.96	486710	{}	\N	2020-10-12 15:38:54.275996
57717	MNOV	2020-09-29	4.99	5.11	4.8507	5.11	448197	{}	\N	2020-10-12 15:38:54.282051
57718	MNOV	2020-09-30	5.07	5.26	5.07	5.24	363376	{}	\N	2020-10-12 15:38:54.287791
57719	MNOV	2020-10-01	5.21	5.29	5.14	5.27	226866	{}	\N	2020-10-12 15:38:54.294244
57720	MNOV	2020-10-02	5.19	5.37	5.08	5.33	251232	{}	\N	2020-10-12 15:38:54.300464
57721	MNOV	2020-10-05	5.32	5.462	5.3	5.39	218755	{}	\N	2020-10-12 15:38:54.305427
57722	MNOV	2020-10-06	5.38	5.49	5.28	5.32	164576	{}	\N	2020-10-12 15:38:54.310409
57723	MNOV	2020-10-07	5.36	5.45	5.28	5.45	149488	{}	\N	2020-10-12 15:38:54.315807
57724	MNOV	2020-10-08	5.4253	5.48	5.33	5.44	104102	{}	\N	2020-10-12 15:38:54.320742
57725	MNOV	2020-10-09	5.4	5.49	5.31	5.46	96650	{}	\N	2020-10-12 15:38:54.326034
57726	MNOV	2020-10-12	5.47	5.48	5.31	5.38	63342	{}	\N	2020-10-12 15:38:54.331846
57727	MOAT	2020-09-28	54.38	54.94	54.38	54.74	514022	{}	\N	2020-10-12 15:38:55.340965
57728	MOAT	2020-09-29	54.805	54.85	54.2125	54.31	426146	{}	\N	2020-10-12 15:38:55.346593
57729	MOAT	2020-09-30	54.43	55.115	54.22	54.59	424357	{}	\N	2020-10-12 15:38:55.35237
57730	MOAT	2020-10-01	54.89	55.19	54.59	54.87	316102	{}	\N	2020-10-12 15:38:55.357602
57731	MOAT	2020-10-02	53.94	55.11	53.94	54.78	230642	{}	\N	2020-10-12 15:38:55.363119
57732	MOAT	2020-10-05	55.08	55.667	55.08	55.59	164959	{}	\N	2020-10-12 15:38:55.370414
57733	MOAT	2020-10-06	55.81	56.2789	54.98	55.09	225729	{}	\N	2020-10-12 15:38:55.375559
57734	MOAT	2020-10-07	55.67	56.2917	55.67	56.07	204403	{}	\N	2020-10-12 15:38:55.380608
57735	MOAT	2020-10-08	56.61	56.9301	56.51	56.93	176616	{}	\N	2020-10-12 15:38:55.386347
57736	MOAT	2020-10-09	57.14	57.539	57.04	57.25	167184	{}	\N	2020-10-12 15:38:55.391812
57737	MOAT	2020-10-12	57.58	57.77	57.41	57.715	195043	{}	\N	2020-10-12 15:38:55.397701
57738	MOO	2020-09-28	67.27	67.735	67.27	67.35	10546	{}	\N	2020-10-12 15:38:56.574746
57739	MOO	2020-09-29	67.36	67.5	66.9	67.01	32048	{}	\N	2020-10-12 15:38:56.581768
57740	MOO	2020-09-30	67.2	68.15	67.19	67.42	85916	{}	\N	2020-10-12 15:38:56.587094
57741	MOO	2020-10-01	67.36	67.52	66.6	66.86	105442	{}	\N	2020-10-12 15:38:56.591806
57742	MOO	2020-10-02	65.65	67.01	65.64	66.66	43133	{}	\N	2020-10-12 15:38:56.596933
57743	MOO	2020-10-05	67.3	68.13	67.3	68.13	43608	{}	\N	2020-10-12 15:38:56.602013
57744	MOO	2020-10-06	68.3	68.61	67.5	67.56	22520	{}	\N	2020-10-12 15:38:56.606649
57745	MOO	2020-10-07	67.95	68.4	67.95	68.21	57074	{}	\N	2020-10-12 15:38:56.61233
57746	MOO	2020-10-08	68.57	69.122	68.57	69.1	168087	{}	\N	2020-10-12 15:38:56.617174
57747	MOO	2020-10-09	69.46	69.9099	69.35	69.7	63206	{}	\N	2020-10-12 15:38:56.623847
57748	MOO	2020-10-12	69.84	70.2399	69.7	70.1899	27651	{}	\N	2020-10-12 15:38:56.630081
57749	MOSY	2020-09-28	1.49	1.5851	1.4521	1.48	49467	{}	\N	2020-10-12 15:39:00.318036
57750	MOSY	2020-09-29	1.46	1.6	1.46	1.55	73563	{}	\N	2020-10-12 15:39:00.324799
57751	MOSY	2020-09-30	1.55	1.5725	1.43	1.43	18808	{}	\N	2020-10-12 15:39:00.330505
57752	MOSY	2020-10-01	1.49	1.5699	1.46	1.53	29246	{}	\N	2020-10-12 15:39:00.336141
57753	MOSY	2020-10-02	1.52	1.57	1.48	1.57	24374	{}	\N	2020-10-12 15:39:00.345076
57754	MOSY	2020-10-05	1.62	1.68	1.59	1.59	47419	{}	\N	2020-10-12 15:39:00.350951
57755	MOSY	2020-10-06	1.53	1.6782	1.5	1.51	36140	{}	\N	2020-10-12 15:39:00.355503
57756	MOSY	2020-10-07	1.6499	1.74	1.56	1.62	49808	{}	\N	2020-10-12 15:39:00.361103
57757	MOSY	2020-10-08	1.65	1.66	1.59	1.6	19739	{}	\N	2020-10-12 15:39:00.366232
57758	MOSY	2020-10-09	1.6184	1.72	1.58	1.58	29020	{}	\N	2020-10-12 15:39:00.371399
57759	MOSY	2020-10-12	1.64	1.7	1.6	1.6	17103	{}	\N	2020-10-12 15:39:00.376992
57760	MPW	2020-09-28	17.41	17.64	17.22	17.6	3650602	{}	\N	2020-10-12 15:39:02.160067
57761	MPW	2020-09-29	17.6	17.61	17.275	17.59	7804360	{}	\N	2020-10-12 15:39:02.167868
57762	MPW	2020-09-30	17.69	17.895	17.425	17.63	10543685	{}	\N	2020-10-12 15:39:02.175474
57763	MPW	2020-10-01	17.73	18.19	17.58	18.18	7244467	{}	\N	2020-10-12 15:39:02.182587
57764	MPW	2020-10-02	17.9	18.62	17.71	18.56	8091073	{}	\N	2020-10-12 15:39:02.188545
57765	MPW	2020-10-05	18.68	18.75	18.22	18.7	6651287	{}	\N	2020-10-12 15:39:02.195087
57766	MPW	2020-10-06	18.75	18.98	18.472	18.63	8364318	{}	\N	2020-10-12 15:39:02.200978
57767	MPW	2020-10-07	18.76	18.76	18.26	18.38	5399583	{}	\N	2020-10-12 15:39:02.206667
57768	MPW	2020-10-08	18.4	18.71	18.35	18.7	4907847	{}	\N	2020-10-12 15:39:02.214046
57769	MPW	2020-10-09	18.91	18.94	18.5	18.59	6022900	{}	\N	2020-10-12 15:39:02.219584
57770	MPW	2020-10-12	18.66	18.77	18.54	18.755	1281831	{}	\N	2020-10-12 15:39:02.224771
57771	MRCC	2020-09-28	6.69	7.01	6.62	6.905	119995	{}	\N	2020-10-12 15:39:03.101407
57772	MRCC	2020-09-29	6.9	6.92	6.8	6.91	82102	{}	\N	2020-10-12 15:39:03.106889
57773	MRCC	2020-09-30	7.11	7.11	6.83	6.84	103362	{}	\N	2020-10-12 15:39:03.112446
57774	MRCC	2020-10-01	6.9	7.01	6.81	6.89	139873	{}	\N	2020-10-12 15:39:03.127712
57775	MRCC	2020-10-02	6.82	6.95	6.77	6.95	40890	{}	\N	2020-10-12 15:39:03.136655
57776	MRCC	2020-10-05	7	7	6.825	6.88	71368	{}	\N	2020-10-12 15:39:03.146597
57777	MRCC	2020-10-06	6.86	7.02	6.8	6.91	46024	{}	\N	2020-10-12 15:39:03.152709
57778	MRCC	2020-10-07	6.93	6.99	6.85	6.9	82698	{}	\N	2020-10-12 15:39:03.158273
57779	MRCC	2020-10-08	6.95	7	6.82	6.98	77217	{}	\N	2020-10-12 15:39:03.163596
57780	MRCC	2020-10-09	6.96	7.02	6.8681	6.98	56317	{}	\N	2020-10-12 15:39:03.168918
57781	MRCC	2020-10-12	6.98	6.98	6.9201	6.94	19424	{}	\N	2020-10-12 15:39:03.174684
57782	MLI	2020-09-28	27.06	27.92	27.06	27.53	235230	{}	\N	2020-10-12 15:39:04.950279
57783	MLI	2020-09-29	27.6	27.83	26.96	27.44	509875	{}	\N	2020-10-12 15:39:04.955839
57784	MLI	2020-09-30	27.56	27.84	26.97	27.06	567802	{}	\N	2020-10-12 15:39:04.961243
57785	MLI	2020-10-01	27.22	27.46	26.72	27.04	285702	{}	\N	2020-10-12 15:39:04.967118
57786	MLI	2020-10-02	26.45	27.95	26.45	27.73	370473	{}	\N	2020-10-12 15:39:04.972855
57787	MLI	2020-10-05	27.85	28.325	27.7	28.25	382757	{}	\N	2020-10-12 15:39:04.979254
57788	MLI	2020-10-06	28.67	29.36	28.24	28.62	626544	{}	\N	2020-10-12 15:39:04.98447
57789	MLI	2020-10-07	28.87	29.42	28.53	28.9	475191	{}	\N	2020-10-12 15:39:04.989879
57790	MLI	2020-10-08	29.4	29.47	28.91	29.22	255742	{}	\N	2020-10-12 15:39:04.995603
57791	MLI	2020-10-09	29.62	29.62	29.03	29.31	222286	{}	\N	2020-10-12 15:39:05.000786
57792	MLI	2020-10-12	29.33	29.85	29.14	29.79	71604	{}	\N	2020-10-12 15:39:05.006115
57793	MMD	2020-09-28	20.85	20.97	20.7248	20.92	23019	{}	\N	2020-10-12 15:39:06.313947
57794	MMD	2020-09-29	20.92	20.93	20.63	20.82	67722	{}	\N	2020-10-12 15:39:06.319585
57795	MMD	2020-09-30	20.92	20.92	20.69	20.71	92216	{}	\N	2020-10-12 15:39:06.324891
57796	MMD	2020-10-01	20.72	21	20.72	20.89	38704	{}	\N	2020-10-12 15:39:06.330798
57797	MMD	2020-10-02	20.8	20.943	20.68	20.85	47602	{}	\N	2020-10-12 15:39:06.337044
57798	MMD	2020-10-05	20.95	21.15	20.85	21.12	35718	{}	\N	2020-10-12 15:39:06.343641
57799	MMD	2020-10-06	21.12	21.19	20.99	21.11	60402	{}	\N	2020-10-12 15:39:06.351102
57800	MMD	2020-10-07	21.14	21.19	21	21.09	61996	{}	\N	2020-10-12 15:39:06.357161
57801	MMD	2020-10-08	21.02	21.09	20.95	20.95	51398	{}	\N	2020-10-12 15:39:06.363675
57802	MMD	2020-10-09	20.95	21.04	20.82	20.82	66776	{}	\N	2020-10-12 15:39:06.36903
57803	MMD	2020-10-12	20.76	20.8	20.64	20.7	36438	{}	\N	2020-10-12 15:39:06.37393
57804	MMM	2020-09-28	162	164.14	161.33	161.66	2127534	{}	\N	2020-10-12 15:39:07.345317
57805	MMM	2020-09-29	161.67	161.95	158.78	159.81	4080120	{}	\N	2020-10-12 15:39:07.353411
57806	MMM	2020-09-30	160.93	163.103	158.61	160.18	6120716	{}	\N	2020-10-12 15:39:07.359158
57807	MMM	2020-10-01	160.67	161.9	157.72	158.79	3775415	{}	\N	2020-10-12 15:39:07.364625
57808	MMM	2020-10-02	156.47	161.94	156.25	160.36	3453178	{}	\N	2020-10-12 15:39:07.369778
57809	MMM	2020-10-05	162.25	163.5	161.76	162.75	2776779	{}	\N	2020-10-12 15:39:07.374951
57810	MMM	2020-10-06	163.44	165.7	161.83	162.23	4006631	{}	\N	2020-10-12 15:39:07.380711
57811	MMM	2020-10-07	165.1	167.75	164.41	166.49	4377123	{}	\N	2020-10-12 15:39:07.386677
57812	MMM	2020-10-08	167.3	168.15	166.125	167.71	3821642	{}	\N	2020-10-12 15:39:07.391656
57813	MMM	2020-10-09	168.88	170.48	167.95	169.3	3685534	{}	\N	2020-10-12 15:39:07.397667
57814	MMM	2020-10-12	169.47	170.61	168.7	169.6683	815084	{}	\N	2020-10-12 15:39:07.404048
57815	MNTA	2020-09-28	52.41	52.47	52.39	52.46	1341837	{}	\N	2020-10-12 15:39:08.618189
57816	MNTA	2020-09-29	52.47	52.49	52.45	52.46	1170214	{}	\N	2020-10-12 15:39:08.624119
57817	MNTA	2020-09-30	52.46	52.53	52.46	52.48	7818386	{}	\N	2020-10-12 15:39:08.628449
57818	MOXC	2020-09-28	1.4	1.44	1.21	1.28	179672	{}	\N	2020-10-12 15:39:12.526858
57819	MOXC	2020-09-29	1.3	1.5	1.267	1.35	174856	{}	\N	2020-10-12 15:39:12.532161
57820	MOXC	2020-09-30	1.38	1.4786	1.27	1.35	269073	{}	\N	2020-10-12 15:39:12.537368
57821	MOXC	2020-10-01	1.35	1.42	1.311	1.36	105214	{}	\N	2020-10-12 15:39:12.542459
57822	MOXC	2020-10-02	1.28	1.35	1.27	1.3	50260	{}	\N	2020-10-12 15:39:12.547886
57823	MOXC	2020-10-05	1.4802	1.6	1.16	1.2	1535940	{}	\N	2020-10-12 15:39:12.555465
57824	MOXC	2020-10-06	1.24	1.25	1.15	1.2	345697	{}	\N	2020-10-12 15:39:12.560617
57825	MOXC	2020-10-07	1.2856	1.38	1.23	1.24	632821	{}	\N	2020-10-12 15:39:12.566347
57826	MOXC	2020-10-08	1.32	1.3241	1.2	1.2094	366906	{}	\N	2020-10-12 15:39:12.572271
57827	MOXC	2020-10-09	1.24	1.27	1.2	1.22	96702	{}	\N	2020-10-12 15:39:12.577503
57828	MOXC	2020-10-12	1.24	1.25	1.16	1.1701	81832	{}	\N	2020-10-12 15:39:12.58249
57829	MPWR	2020-09-28	272.09	275.34	269.29	274.88	335619	{}	\N	2020-10-12 15:39:13.572164
57830	MPWR	2020-09-29	273.28	279.22	273.28	276.61	178927	{}	\N	2020-10-12 15:39:13.578005
57831	MPWR	2020-09-30	275.46	282.88	275.46	279.61	244676	{}	\N	2020-10-12 15:39:13.58326
57832	MPWR	2020-10-01	282.78	285.33	278.17	280.34	336462	{}	\N	2020-10-12 15:39:13.589582
57833	MPWR	2020-10-02	274.73	277.445	271.73	272.37	242274	{}	\N	2020-10-12 15:39:13.596093
57834	MPWR	2020-10-05	274.84	282.755	273.685	282.5	365299	{}	\N	2020-10-12 15:39:13.60332
57835	MPWR	2020-10-06	282.01	292.1	282	286.51	336925	{}	\N	2020-10-12 15:39:13.609709
57836	MPWR	2020-10-07	292.82	298.83	289.48	298.32	346470	{}	\N	2020-10-12 15:39:13.615458
57837	MPWR	2020-10-08	299	303.69	297.33	303	200436	{}	\N	2020-10-12 15:39:13.622082
57838	MPWR	2020-10-09	305.7	317.97	305.7	315.46	389155	{}	\N	2020-10-12 15:39:13.627675
57839	MPWR	2020-10-12	319.64	322.7417	314.19	316.24	57332	{}	\N	2020-10-12 15:39:13.633123
57840	MRIN	2020-09-28	1.29	1.37	1.27	1.3695	33791	{}	\N	2020-10-12 15:39:15.144601
57841	MRIN	2020-09-29	1.3511	1.37	1.35	1.3501	6176	{}	\N	2020-10-12 15:39:15.151292
57842	MRIN	2020-09-30	1.37	1.37	1.33	1.33	8377	{}	\N	2020-10-12 15:39:15.158777
57843	MRIN	2020-10-01	1.33	1.345	1.33	1.33	11331	{}	\N	2020-10-12 15:39:15.164372
57844	MRIN	2020-10-02	1.299	1.3	1.18	1.27	105197	{}	\N	2020-10-12 15:39:15.169235
57845	MRIN	2020-10-05	1.2201	1.28	1.22	1.27	12155	{}	\N	2020-10-12 15:39:15.174501
57846	MRIN	2020-10-06	1.25	1.28	1.24	1.2699	23203	{}	\N	2020-10-12 15:39:15.179211
57847	MRIN	2020-10-07	1.26	1.37	1.2501	1.36	41990	{}	\N	2020-10-12 15:39:15.18375
57848	MRIN	2020-10-08	1.3899	1.39	1.3201	1.33	13356	{}	\N	2020-10-12 15:39:15.190704
57849	MRIN	2020-10-09	1.33	1.37	1.33	1.3484	4160	{}	\N	2020-10-12 15:39:15.195501
57850	MRIN	2020-10-12	1.355	1.355	1.29	1.3001	2557	{}	\N	2020-10-12 15:39:15.200069
57851	MRVL	2020-09-28	38.5424	40.04	38.5	39.89	6851195	{}	\N	2020-10-12 15:39:16.190451
57852	MRVL	2020-09-29	39.97	40.56	39.37	39.4	6390063	{}	\N	2020-10-12 15:39:16.196676
57853	MRVL	2020-09-30	39.15	39.98	38.78	39.7	5679567	{}	\N	2020-10-12 15:39:16.201581
57854	MRVL	2020-10-01	40.41	40.73	39.84	40.31	5701479	{}	\N	2020-10-12 15:39:16.207089
57855	MRVL	2020-10-02	39.49	40.44	39.3	39.77	5628972	{}	\N	2020-10-12 15:39:16.213603
57856	MRVL	2020-10-05	40.4865	41.83	40.03	41.52	9733501	{}	\N	2020-10-12 15:39:16.219085
57857	MRVL	2020-10-06	42.06	43.12	41.69	42.08	8677937	{}	\N	2020-10-12 15:39:16.225119
57858	MRVL	2020-10-07	43.0814	43.67	42.35	42.62	11178895	{}	\N	2020-10-12 15:39:16.232776
57859	MRVL	2020-10-08	43.18	43.72	42.5	42.75	10655356	{}	\N	2020-10-12 15:39:16.240101
57860	MRVL	2020-10-09	43.83	44.94	43.39	44.7	10453333	{}	\N	2020-10-12 15:39:16.246687
57861	MRVL	2020-10-12	44.9	45.04	43.3701	43.43	4165466	{}	\N	2020-10-12 15:39:16.251954
57862	MSFT	2020-09-28	210.95	212.57	208.06	209.44	30202694	{}	\N	2020-10-12 15:39:19.84794
57863	MSFT	2020-09-29	209.35	210.07	206.81	207.26	23466909	{}	\N	2020-10-12 15:39:19.854237
57864	MSFT	2020-09-30	207.73	211.97	206.54	210.33	31241581	{}	\N	2020-10-12 15:39:19.860574
57865	MSFT	2020-10-01	213.41	213.99	211.34	212.46	22079875	{}	\N	2020-10-12 15:39:19.86529
57866	MSFT	2020-10-02	208.01	210.99	205.54	206.19	30137965	{}	\N	2020-10-12 15:39:19.870258
57867	MSFT	2020-10-05	207.25	210.41	206.98	210.38	20857542	{}	\N	2020-10-12 15:39:19.87496
57868	MSFT	2020-10-06	208.82	210.18	204.82	205.91	26032261	{}	\N	2020-10-12 15:39:19.879934
57869	MSFT	2020-10-07	207.06	210	206.72	209.83	25176279	{}	\N	2020-10-12 15:39:19.88476
57870	MSFT	2020-10-08	210.51	211.19	208.32	210.58	19362239	{}	\N	2020-10-12 15:39:19.888964
57871	MSFT	2020-10-09	211.475	215.86	211.25	215.81	24864821	{}	\N	2020-10-12 15:39:19.893602
57872	MSFT	2020-10-12	218.79	220.26	216.81	220.22	11991560	{}	\N	2020-10-12 15:39:19.898613
57873	MINC	2020-09-28	48.6403	48.6403	48.5951	48.5951	678	{}	\N	2020-10-12 15:39:21.314868
57874	MINC	2020-09-29	48.7	48.7	48.49	48.4906	6283	{}	\N	2020-10-12 15:39:21.338319
57875	MINC	2020-09-30	48.53	48.6391	48.53	48.6391	2100	{}	\N	2020-10-12 15:39:21.346104
57876	MINC	2020-10-01	48.51	48.615	48.51	48.615	500	{}	\N	2020-10-12 15:39:21.355021
57877	MINC	2020-10-02	48.645	48.645	48.645	48.645	5	{}	\N	2020-10-12 15:39:21.362232
57878	MINC	2020-10-05	48.6	48.65	48.6	48.65	7686	{}	\N	2020-10-12 15:39:21.37198
57879	MINC	2020-10-06	48.48	48.64	48.48	48.64	3346	{}	\N	2020-10-12 15:39:21.38001
57880	MINC	2020-10-07	48.6	48.65	48.59	48.65	1549	{}	\N	2020-10-12 15:39:21.387882
57881	MINC	2020-10-08	48.6	48.6899	48.6	48.635	2801	{}	\N	2020-10-12 15:39:21.393632
57882	MINC	2020-10-09	48.7389	48.7389	48.685	48.685	992	{}	\N	2020-10-12 15:39:21.400342
57883	MINC	2020-10-12	48.66	48.66	48.65	48.66	583	{}	\N	2020-10-12 15:39:21.405382
57884	MIY	2020-09-28	14.025	14.1423	14	14	14765	{}	\N	2020-10-12 15:39:22.744159
57885	MIY	2020-09-29	14.07	14.07	13.97	13.97	60852	{}	\N	2020-10-12 15:39:22.751994
57886	MIY	2020-09-30	13.97	14.03	13.96	14.02	47592	{}	\N	2020-10-12 15:39:22.757799
57887	MIY	2020-10-01	14.11	14.14	14.04	14.1	61486	{}	\N	2020-10-12 15:39:22.763905
57888	MIY	2020-10-02	14.08	14.2	14.08	14.08	14230	{}	\N	2020-10-12 15:39:22.770146
57889	MIY	2020-10-05	14.06	14.17	14.04	14.09	33124	{}	\N	2020-10-12 15:39:22.777353
57890	MIY	2020-10-06	14.13	14.17	14.09	14.13	50869	{}	\N	2020-10-12 15:39:22.784275
57891	MIY	2020-10-07	14.19	14.24	14.1139	14.18	28036	{}	\N	2020-10-12 15:39:22.789818
57892	MIY	2020-10-08	14.22	14.24	14.17	14.22	28006	{}	\N	2020-10-12 15:39:22.796097
57893	MIY	2020-10-09	14.205	14.21	14.13	14.15	47058	{}	\N	2020-10-12 15:39:22.802057
57894	MIY	2020-10-12	14.155	14.1779	14.15	14.15	5604	{}	\N	2020-10-12 15:39:22.806841
57895	MLPE	2020-09-28	9.615	9.7225	9.615	9.7225	200	{}	\N	2020-10-12 15:39:24.930962
57896	MLPE	2020-09-29	9.9818	9.9818	9.9818	9.9818	0	{}	\N	2020-10-12 15:39:24.942345
57897	MLPE	2020-09-30	9.8248	9.8248	9.8248	9.8248	0	{}	\N	2020-10-12 15:39:24.951064
57898	MLPE	2020-10-01	9.7567	9.7567	9.7567	9.7567	2	{}	\N	2020-10-12 15:39:24.961435
57899	MLPE	2020-10-02	9.8994	9.8994	9.8994	9.8994	0	{}	\N	2020-10-12 15:39:24.967865
57900	MLPE	2020-10-05	10.2628	10.2628	10.2628	10.2628	20	{}	\N	2020-10-12 15:39:24.97312
57901	MLPE	2020-10-06	10.0915	10.0915	10.0915	10.0915	0	{}	\N	2020-10-12 15:39:24.979023
57902	MLPE	2020-10-07	10.14	10.1404	10.1	10.1	3235	{}	\N	2020-10-12 15:39:24.984868
57903	MLPE	2020-10-08	10.13	10.3302	10.13	10.3302	1182	{}	\N	2020-10-12 15:39:24.991239
57904	MLPE	2020-10-09	10.25	10.3193	10.24	10.3193	600	{}	\N	2020-10-12 15:39:24.996839
57905	MODN	2020-09-28	36.37	36.81	35.73	36.24	183015	{}	\N	2020-10-12 15:39:27.134205
57906	MODN	2020-09-29	36.2	36.4225	35.58	35.82	369417	{}	\N	2020-10-12 15:39:27.139887
57907	MODN	2020-09-30	35.72	36.01	35.175	35.28	431470	{}	\N	2020-10-12 15:39:27.144128
57908	MODN	2020-10-01	35.62	36.52	35.46	36.43	1090322	{}	\N	2020-10-12 15:39:27.148214
57909	MODN	2020-10-02	35.43	36.36	34.51	34.54	534889	{}	\N	2020-10-12 15:39:27.153043
57910	MODN	2020-10-05	34.58	35.66	34.505	35.49	318984	{}	\N	2020-10-12 15:39:27.169649
57911	MODN	2020-10-06	35.55	36.22	34.92	34.99	272540	{}	\N	2020-10-12 15:39:27.177162
57912	MODN	2020-10-07	35.14	35.7	34.71	35.6	675786	{}	\N	2020-10-12 15:39:27.185051
57913	MODN	2020-10-08	36.22	36.57	35.6	36.22	327685	{}	\N	2020-10-12 15:39:27.191719
57914	MODN	2020-10-09	34.21	37.31	33.05	35.61	1007973	{}	\N	2020-10-12 15:39:27.197189
57915	MODN	2020-10-12	36.11	36.11	35	35.66	125256	{}	\N	2020-10-12 15:39:27.202741
57916	MOS	2020-09-28	18.87	19.29	18.45	18.47	4084453	{}	\N	2020-10-12 15:39:28.32426
57917	MOS	2020-09-29	18.37	18.63	17.89	17.98	4865883	{}	\N	2020-10-12 15:39:28.330105
57918	MOS	2020-09-30	18.05	18.64	18	18.27	7503779	{}	\N	2020-10-12 15:39:28.335268
57919	MOS	2020-10-01	18.35	18.465	17.71	17.79	6865619	{}	\N	2020-10-12 15:39:28.341189
57920	MOS	2020-10-02	17.35	18.58	17.3	18.31	6976229	{}	\N	2020-10-12 15:39:28.347309
57921	MOS	2020-10-05	18.56	19.455	18.47	19.28	13871066	{}	\N	2020-10-12 15:39:28.352725
57922	MOS	2020-10-06	19.5	19.7299	18.77	18.83	6711228	{}	\N	2020-10-12 15:39:28.358857
57923	MOS	2020-10-07	19.19	19.87	19.19	19.74	12213053	{}	\N	2020-10-12 15:39:28.363592
57924	MOS	2020-10-08	19.83	20.2	19.29	19.72	7718250	{}	\N	2020-10-12 15:39:28.37028
57925	MOS	2020-10-09	19.92	20.07	19.39	19.71	5749725	{}	\N	2020-10-12 15:39:28.376791
57926	MOS	2020-10-12	19.74	19.75	19.27	19.345	1683584	{}	\N	2020-10-12 15:39:28.381915
57927	MPLX	2020-09-28	16.27	16.79	16.1394	16.65	1601955	{}	\N	2020-10-12 15:39:29.332199
57928	MPLX	2020-09-29	16.48	16.685	16.18	16.43	4358544	{}	\N	2020-10-12 15:39:29.338984
57929	MPLX	2020-09-30	16.41	16.66	15.59	15.74	7411766	{}	\N	2020-10-12 15:39:29.344237
57930	MPLX	2020-10-01	15.71	15.96	15.045	15.51	4891224	{}	\N	2020-10-12 15:39:29.349215
57931	MPLX	2020-10-02	15.08	15.75	15.08	15.56	3575920	{}	\N	2020-10-12 15:39:29.354911
57932	MPLX	2020-10-05	15.85	16.25	15.475	16.25	3915521	{}	\N	2020-10-12 15:39:29.360299
57933	MPLX	2020-10-06	16.46	16.58	15.73	15.95	3352666	{}	\N	2020-10-12 15:39:29.366545
57934	MPLX	2020-10-07	16.09	16.13	15.76	15.97	3358955	{}	\N	2020-10-12 15:39:29.372192
57935	MPLX	2020-10-08	16	17.2825	16	17.26	5121610	{}	\N	2020-10-12 15:39:29.376786
57936	MPLX	2020-10-09	17.45	17.68	17.22	17.26	5281554	{}	\N	2020-10-12 15:39:29.381638
57937	MPLX	2020-10-12	17.22	17.29	17.05	17.26	849321	{}	\N	2020-10-12 15:39:29.386616
57938	MRC	2020-09-28	4.61	4.855	4.5936	4.67	1054108	{}	\N	2020-10-12 15:39:30.295156
57939	MRC	2020-09-29	4.67	4.67	4.33	4.45	1440380	{}	\N	2020-10-12 15:39:30.300287
57940	MRC	2020-09-30	4.43	4.59	4.255	4.28	1569168	{}	\N	2020-10-12 15:39:30.305012
57941	MRC	2020-10-01	4.25	4.29	4.06	4.13	1394188	{}	\N	2020-10-12 15:39:30.310069
57942	MRC	2020-10-02	4	4.41	3.93	4.33	1482437	{}	\N	2020-10-12 15:39:30.315071
57943	MRC	2020-10-05	4.39	4.4399	4.23	4.38	1293334	{}	\N	2020-10-12 15:39:30.320622
57944	MRC	2020-10-06	4.44	4.615	4.36	4.39	3333004	{}	\N	2020-10-12 15:39:30.327508
57945	MRC	2020-10-07	4.45	4.52	4.34	4.49	1943474	{}	\N	2020-10-12 15:39:30.332731
57946	MRC	2020-10-08	4.61	4.67	4.525	4.64	1029207	{}	\N	2020-10-12 15:39:30.336935
57947	MRC	2020-10-09	4.65	4.715	4.48	4.68	1194831	{}	\N	2020-10-12 15:39:30.342665
57948	MRC	2020-10-12	4.66	4.75	4.5	4.55	344125	{}	\N	2020-10-12 15:39:30.348609
57949	MRNS	2020-09-28	11.44	11.73	10.94	11.26	1095749	{}	\N	2020-10-12 15:39:31.411783
57950	MRNS	2020-09-29	11.18	11.9064	11	11.62	749188	{}	\N	2020-10-12 15:39:31.418123
57951	MRNS	2020-09-30	12.25	13	11.94	12.85	1019873	{}	\N	2020-10-12 15:39:31.424511
57952	MRNS	2020-10-01	12.74	13.09	12.099	13.04	901592	{}	\N	2020-10-12 15:39:31.430397
57953	MRNS	2020-10-02	12.58	12.9799	11.85	12.07	1090754	{}	\N	2020-10-12 15:39:31.436266
57954	MRNS	2020-10-05	12.11	13.9	12.0885	13.86	1206963	{}	\N	2020-10-12 15:39:31.443012
57955	MRNS	2020-10-06	13.76	15.06	13.1701	14.15	1313381	{}	\N	2020-10-12 15:39:31.448253
57956	MRNS	2020-10-07	14.4	15.52	14.3	15.44	840669	{}	\N	2020-10-12 15:39:31.455142
57957	MRNS	2020-10-08	15.6436	16.1	14.5	15.03	916193	{}	\N	2020-10-12 15:39:31.461916
57958	MRNS	2020-10-09	15.05	15.34	14.41	14.53	787199	{}	\N	2020-10-12 15:39:31.467899
57959	MRNS	2020-10-12	14.53	15.15	14.53	15.02	237010	{}	\N	2020-10-12 15:39:31.473379
57960	MSA	2020-09-28	134.32	135.47	133.89	134.51	100956	{}	\N	2020-10-12 15:39:32.438215
57961	MSA	2020-09-29	134.55	135.2	132.38	132.66	226358	{}	\N	2020-10-12 15:39:32.444768
57962	MSA	2020-09-30	133.17	134.88	132.83	134.17	321729	{}	\N	2020-10-12 15:39:32.450285
57963	MSA	2020-10-01	134.42	134.69	132.91	133.45	194893	{}	\N	2020-10-12 15:39:32.457529
57964	MSA	2020-10-02	132.12	137.03	131.5	136.83	263945	{}	\N	2020-10-12 15:39:32.463322
57965	MSA	2020-10-05	137.38	140.61	137	140.12	210884	{}	\N	2020-10-12 15:39:32.469807
57966	MSA	2020-10-06	140.88	141.86	139.25	140.05	274550	{}	\N	2020-10-12 15:39:32.475978
57967	MSA	2020-10-07	140.64	142.04	139.7993	140.27	275352	{}	\N	2020-10-12 15:39:32.48096
57968	MSA	2020-10-08	141.02	142.405	139.57	141.93	166598	{}	\N	2020-10-12 15:39:32.486174
57969	MSA	2020-10-09	142	144.62	142	144.01	128479	{}	\N	2020-10-12 15:39:32.492445
57970	MSA	2020-10-12	144.61	145.8	144.28	145.17	94762	{}	\N	2020-10-12 15:39:32.498929
57971	MLCO	2020-09-28	16.63	16.82	16.42	16.59	1614867	{}	\N	2020-10-12 15:39:34.568381
57972	MLCO	2020-09-29	16.56	16.8	16.38	16.6	1967537	{}	\N	2020-10-12 15:39:34.572975
57973	MLCO	2020-09-30	16.6601	16.9338	16.56	16.65	1582865	{}	\N	2020-10-12 15:39:34.579449
57974	MLCO	2020-10-01	16.65	16.85	16.38	16.53	1165377	{}	\N	2020-10-12 15:39:34.585124
57975	MLCO	2020-10-02	16.26	16.8	16.24	16.78	3938812	{}	\N	2020-10-12 15:39:34.591476
57976	MLCO	2020-10-05	16.63	16.6818	15.51	16.13	5820903	{}	\N	2020-10-12 15:39:34.5987
57977	MLCO	2020-10-06	16.15	16.9	16.12	16.35	4296925	{}	\N	2020-10-12 15:39:34.603476
57978	MLCO	2020-10-07	16.45	16.52	16.16	16.17	1803077	{}	\N	2020-10-12 15:39:34.608051
57979	MLCO	2020-10-08	16.32	16.32	15.75	15.86	2854926	{}	\N	2020-10-12 15:39:34.613511
57980	MLCO	2020-10-09	15.98	16.04	15.32	15.44	3888351	{}	\N	2020-10-12 15:39:34.618545
57981	MLCO	2020-10-12	15.61	15.8502	15.34	15.57	1783962	{}	\N	2020-10-12 15:39:34.624131
57982	MLPY	2020-09-28	2.49	2.5999	2.49	2.5	10685	{}	\N	2020-10-12 15:39:36.627903
57983	MLPY	2020-09-29	2.58	2.61	2.5394	2.5394	4882	{}	\N	2020-10-12 15:39:36.634876
57984	MLPY	2020-09-30	2.62	2.6301	2.6001	2.6301	1898	{}	\N	2020-10-12 15:39:36.639198
57985	MLPY	2020-10-01	2.6036	2.6036	2.56	2.56	12978	{}	\N	2020-10-12 15:39:36.643301
57986	MLPY	2020-10-02	2.5101	2.58	2.5101	2.58	805	{}	\N	2020-10-12 15:39:36.648169
57987	MLPY	2020-10-05	2.6503	2.6503	2.6503	2.6503	137	{}	\N	2020-10-12 15:39:36.652366
57988	MLPY	2020-10-06	2.73	2.73	2.6643	2.6643	1308	{}	\N	2020-10-12 15:39:36.656757
57989	MLPY	2020-10-07	2.67	2.67	2.67	2.67	597	{}	\N	2020-10-12 15:39:36.661885
57990	MLPY	2020-10-08	2.7253	2.8203	2.7253	2.8203	2231	{}	\N	2020-10-12 15:39:36.666892
57991	MLPY	2020-10-09	2.75	2.75	2.6898	2.6898	5317	{}	\N	2020-10-12 15:39:36.671947
57992	MNR	2020-09-28	13.73	13.94	13.58	13.89	366998	{}	\N	2020-10-12 15:39:38.278756
57993	MNR	2020-09-29	13.9	13.9	13.6	13.82	564430	{}	\N	2020-10-12 15:39:38.284993
57994	MNR	2020-09-30	13.85	14.06	13.66	13.85	594655	{}	\N	2020-10-12 15:39:38.289809
57995	MNR	2020-10-01	13.93	14.05	13.8	14	864542	{}	\N	2020-10-12 15:39:38.294891
57996	MNR	2020-10-02	13.75	14.0701	13.64	14.01	945795	{}	\N	2020-10-12 15:39:38.301248
57997	MNR	2020-10-05	14.1	14.11	13.81	14.03	818815	{}	\N	2020-10-12 15:39:38.30711
57998	MNR	2020-10-06	14.15	14.15	13.92	13.97	1112801	{}	\N	2020-10-12 15:39:38.312202
57999	MNR	2020-10-07	14.07	14.14	13.81	13.98	1570306	{}	\N	2020-10-12 15:39:38.318232
58000	MNR	2020-10-08	14.13	14.28	13.99	14.12	830313	{}	\N	2020-10-12 15:39:38.323165
58001	MNR	2020-10-09	14.27	14.34	14.07	14.21	466236	{}	\N	2020-10-12 15:39:38.328019
58002	MNR	2020-10-12	14.15	14.69	14.0968	14.65	343906	{}	\N	2020-10-12 15:39:38.334902
58003	MOFG	2020-09-28	17.72	18.5	17.61	18.06	29344	{}	\N	2020-10-12 15:39:40.339435
58004	MOFG	2020-09-29	18.03	18.03	17.46	17.81	16055	{}	\N	2020-10-12 15:39:40.345123
58005	MOFG	2020-09-30	18.12	18.22	17.62	17.87	22142	{}	\N	2020-10-12 15:39:40.350994
58006	MOFG	2020-10-01	18.01	18.23	17.42	17.78	17720	{}	\N	2020-10-12 15:39:40.356396
58007	MOFG	2020-10-02	17.41	18.6346	17.41	18.55	29017	{}	\N	2020-10-12 15:39:40.36237
58008	MOFG	2020-10-05	19.17	19.38	18.93	19.38	31537	{}	\N	2020-10-12 15:39:40.368764
58009	MOFG	2020-10-06	19.5	20.5	19.37	20.03	53022	{}	\N	2020-10-12 15:39:40.374317
58010	MOFG	2020-10-07	20.19	20.76	20.08	20.59	35514	{}	\N	2020-10-12 15:39:40.380518
58011	MOFG	2020-10-08	20.76	20.94	20.46	20.86	29877	{}	\N	2020-10-12 15:39:40.387688
58012	MOFG	2020-10-09	20.99	20.99	20.38	20.61	23714	{}	\N	2020-10-12 15:39:40.393395
58013	MOFG	2020-10-12	20.24	20.655	20.24	20.655	950	{}	\N	2020-10-12 15:39:40.399128
58014	MORT	2020-09-28	13.97	14.3	13.97	14.2	66119	{}	\N	2020-10-12 15:39:41.355223
58015	MORT	2020-09-29	14.22	14.22	13.8137	13.92	49055	{}	\N	2020-10-12 15:39:41.36088
58016	MORT	2020-09-30	14.112	14.15	13.8537	13.92	37490	{}	\N	2020-10-12 15:39:41.365577
58017	MORT	2020-10-01	13.65	13.75	13.48	13.75	67903	{}	\N	2020-10-12 15:39:41.370816
58018	MORT	2020-10-02	13.4	13.98	13.4	13.95	70058	{}	\N	2020-10-12 15:39:41.377398
58019	MORT	2020-10-05	14.01	14.1	13.95	14.05	123613	{}	\N	2020-10-12 15:39:41.383171
58020	MORT	2020-10-06	14.09	14.29	13.92	13.92	27787	{}	\N	2020-10-12 15:39:41.389444
58021	MORT	2020-10-07	14.01	14.0494	13.84	14.0021	83549	{}	\N	2020-10-12 15:39:41.395093
58022	MORT	2020-10-08	14.09	14.38	14.07	14.38	89311	{}	\N	2020-10-12 15:39:41.402597
58023	MORT	2020-10-09	14.4	14.41	14.095	14.15	45707	{}	\N	2020-10-12 15:39:41.408258
58024	MORT	2020-10-12	14.15	14.225	14.14	14.21	34930	{}	\N	2020-10-12 15:39:41.413871
58025	MPC	2020-09-28	29.46	30.31	29.18	29.95	7572236	{}	\N	2020-10-12 15:39:42.607695
58026	MPC	2020-09-29	29.92	30.27	29.12	29.43	9788796	{}	\N	2020-10-12 15:39:42.614984
58027	MPC	2020-09-30	29.59	30.21	29.13	29.34	11582601	{}	\N	2020-10-12 15:39:42.620773
58028	MPC	2020-10-01	28.77	28.98	27.39	27.71	15424754	{}	\N	2020-10-12 15:39:42.626519
58029	MPC	2020-10-02	26.82	28.64	26.56	28.27	14469187	{}	\N	2020-10-12 15:39:42.633339
58030	MPC	2020-10-05	28.7	29.47	28.62	29.24	12980538	{}	\N	2020-10-12 15:39:42.639744
58031	MPC	2020-10-06	29.77	30.3	28.76	28.83	10547384	{}	\N	2020-10-12 15:39:42.645002
58032	MPC	2020-10-07	28.99	29.27	28.55	29.21	11027385	{}	\N	2020-10-12 15:39:42.650238
58033	MPC	2020-10-08	29.39	31.14	29.2819	31.12	13261686	{}	\N	2020-10-12 15:39:42.656435
58034	MPC	2020-10-09	31.35	31.39	30.07	30.12	14226930	{}	\N	2020-10-12 15:39:42.662096
58035	MPC	2020-10-12	29.97	30.17	29.62	30.12	2506612	{}	\N	2020-10-12 15:39:42.667588
58036	MRBK	2020-09-28	14.85	15.47	14.85	15.31	17220	{}	\N	2020-10-12 15:39:43.81238
58037	MRBK	2020-09-29	15.49	15.85	15.3925	15.5	14007	{}	\N	2020-10-12 15:39:43.818033
58038	MRBK	2020-09-30	15.64	16.31	15.64	16.13	12026	{}	\N	2020-10-12 15:39:43.823953
58039	MRBK	2020-10-01	16.29	17.205	16.2	16.74	23146	{}	\N	2020-10-12 15:39:43.829246
58040	MRBK	2020-10-02	16.45	17.02	16.44	16.44	7840	{}	\N	2020-10-12 15:39:43.835342
58041	MRBK	2020-10-05	16.56	17.71	16.56	17.45	12723	{}	\N	2020-10-12 15:39:43.841401
58042	MRBK	2020-10-06	17.62	19.78	17.62	18.35	24893	{}	\N	2020-10-12 15:39:43.84755
58043	MRBK	2020-10-07	18.66	19.4953	18.04	18.29	26022	{}	\N	2020-10-12 15:39:43.853337
58044	MRBK	2020-10-08	18.47	19.22	17.87	18.09	11569	{}	\N	2020-10-12 15:39:43.858659
58045	MRBK	2020-10-09	18.1499	18.15	17.91	18	7952	{}	\N	2020-10-12 15:39:43.863486
58046	MRBK	2020-10-12	18.23	18.23	17.179	17.46	3945	{}	\N	2020-10-12 15:39:43.868472
58047	MSD	2020-09-28	8.48	8.58	8.48	8.56	88819	{}	\N	2020-10-12 15:39:47.102904
58048	MSD	2020-09-29	8.45	8.5	8.43	8.43	401702	{}	\N	2020-10-12 15:39:47.110823
58049	MSD	2020-09-30	8.42	8.45	8.39	8.39	157220	{}	\N	2020-10-12 15:39:47.115911
58050	MSD	2020-10-01	8.43	8.48	8.39	8.4	100350	{}	\N	2020-10-12 15:39:47.121526
58051	MSD	2020-10-02	8.36	8.41	8.36	8.36	61654	{}	\N	2020-10-12 15:39:47.128177
58052	MSD	2020-10-05	8.39	8.42	8.35	8.39	74740	{}	\N	2020-10-12 15:39:47.133768
58053	MSD	2020-10-06	8.47	8.48	8.39	8.41	228242	{}	\N	2020-10-12 15:39:47.139325
58054	MSD	2020-10-07	8.43	8.47	8.43	8.44	141418	{}	\N	2020-10-12 15:39:47.146423
58055	MSD	2020-10-08	8.49	8.56	8.49	8.53	264169	{}	\N	2020-10-12 15:39:47.161368
58056	MSD	2020-10-09	8.58	8.58	8.52	8.55	137467	{}	\N	2020-10-12 15:39:47.168729
58057	MSD	2020-10-12	8.58	8.58	8.53	8.53	29128	{}	\N	2020-10-12 15:39:47.177985
58058	MSM	2020-09-28	62.36	63.52	62.36	63.02	320404	{}	\N	2020-10-12 15:39:48.663711
58059	MSM	2020-09-29	63.41	63.41	62.32	62.89	646163	{}	\N	2020-10-12 15:39:48.669292
58060	MSM	2020-09-30	63.25	63.99	62.72	63.28	839570	{}	\N	2020-10-12 15:39:48.673586
58061	MSM	2020-10-01	63.62	63.62	62.02	62.25	574157	{}	\N	2020-10-12 15:39:48.679407
58062	MSM	2020-10-02	61.29	63.6	61.29	63.09	600024	{}	\N	2020-10-12 15:39:48.68536
58063	MSM	2020-10-05	63.7	64.92	63.635	64.59	603072	{}	\N	2020-10-12 15:39:48.690667
58064	MSM	2020-10-06	64.7	65.1	63.46	63.46	811781	{}	\N	2020-10-12 15:39:48.696
58065	MSM	2020-10-07	63.93	65.39	63.83	65.31	1778992	{}	\N	2020-10-12 15:39:48.701058
58066	MSM	2020-10-08	65.7	65.7	63.97	65.1	870266	{}	\N	2020-10-12 15:39:48.706219
58067	MSM	2020-10-09	65.36	66.06	65.13	65.75	632604	{}	\N	2020-10-12 15:39:48.711613
58068	MSM	2020-10-12	66.18	67.89	66.18	67.82	260705	{}	\N	2020-10-12 15:39:48.71615
58069	MTBC	2020-09-28	8.57	8.74	8.57	8.74	67446	{}	\N	2020-10-12 15:39:49.737266
58070	MTBC	2020-09-29	8.73	9.01	8.65	8.92	62724	{}	\N	2020-10-12 15:39:49.744199
58071	MTBC	2020-09-30	8.92	9.36	8.81	8.8663	94483	{}	\N	2020-10-12 15:39:49.749637
58072	MTBC	2020-10-01	8.95	9.06	8.68	8.75	88903	{}	\N	2020-10-12 15:39:49.75619
58073	MTBC	2020-10-02	8.75	8.81	8.43	8.51	73953	{}	\N	2020-10-12 15:39:49.763457
58074	MTBC	2020-10-05	8.51	8.51	8.25	8.31	154694	{}	\N	2020-10-12 15:39:49.770002
58075	MTBC	2020-10-06	8.31	8.35	8.2	8.2	92755	{}	\N	2020-10-12 15:39:49.774871
58076	MTBC	2020-10-07	8.3	8.65	8.2	8.65	112731	{}	\N	2020-10-12 15:39:49.780775
58077	MTBC	2020-10-08	8.72	8.76	8.48	8.63	45037	{}	\N	2020-10-12 15:39:49.785775
58078	MTBC	2020-10-09	8.74	8.74	8.365	8.44	60629	{}	\N	2020-10-12 15:39:49.790756
58079	MTBC	2020-10-12	8.38	8.6888	8.32	8.585	56880	{}	\N	2020-10-12 15:39:49.797132
58080	MLPB	2020-09-28	9.18	9.44	9.18	9.38	428046	{}	\N	2020-10-12 15:39:51.139794
58081	MLPB	2020-09-29	9.33	9.57	9.33	9.52	14183	{}	\N	2020-10-12 15:39:51.145256
58082	MLPB	2020-09-30	9.6701	9.6788	9.28	9.35	37459	{}	\N	2020-10-12 15:39:51.150336
58083	MLPB	2020-10-01	9.34	9.3565	9.16	9.23	13882	{}	\N	2020-10-12 15:39:51.155076
58084	MLPB	2020-10-02	9.04	9.4191	9.04	9.4191	33726	{}	\N	2020-10-12 15:39:51.17113
58085	MLPB	2020-10-05	9.69	9.78	9.51	9.76	57908	{}	\N	2020-10-12 15:39:51.181264
58086	MLPB	2020-10-06	9.71	9.84	9.49	9.53	49842	{}	\N	2020-10-12 15:39:51.190965
58087	MLPB	2020-10-07	9.58	9.605	9.54	9.5901	21539	{}	\N	2020-10-12 15:39:51.198147
58088	MLPB	2020-10-08	9.39	10.19	9.39	10.19	28357	{}	\N	2020-10-12 15:39:51.204279
58089	MLPB	2020-10-09	9.88	9.9544	9.77	9.87	15680	{}	\N	2020-10-12 15:39:51.210368
58090	MLPB	2020-10-12	9.797	9.845	9.78	9.84	7756	{}	\N	2020-10-12 15:39:51.216398
58091	MMS	2020-09-28	68.51	69.27	68.51	68.75	256581	{}	\N	2020-10-12 15:39:52.78871
58092	MMS	2020-09-29	68.87	69.41	68.2	68.67	832888	{}	\N	2020-10-12 15:39:52.795522
58093	MMS	2020-09-30	69.03	69.4	68.1	68.41	725192	{}	\N	2020-10-12 15:39:52.80199
58094	MMS	2020-10-01	68.34	69.03	67.99	68.89	586285	{}	\N	2020-10-12 15:39:52.808852
58095	MMS	2020-10-02	67.89	68.845	67.65	67.84	491925	{}	\N	2020-10-12 15:39:52.815876
58096	MMS	2020-10-05	68.4	69.42	68	69.42	477000	{}	\N	2020-10-12 15:39:52.821576
58097	MMS	2020-10-06	69.04	70.21	67.85	68.4	750514	{}	\N	2020-10-12 15:39:52.827768
58098	MMS	2020-10-07	68.99	69.42	68.53	69.22	471705	{}	\N	2020-10-12 15:39:52.83448
58099	MMS	2020-10-08	69.9	70.57	69.32	69.85	452732	{}	\N	2020-10-12 15:39:52.840534
58100	MMS	2020-10-09	70.36	70.58	69.98	70.33	396855	{}	\N	2020-10-12 15:39:52.846986
58101	MMS	2020-10-12	70.6	71.25	70.48	71.145	71196	{}	\N	2020-10-12 15:39:52.853839
58102	MNK	2020-09-28	1.04	1.07	0.81	0.85	9484621	{}	\N	2020-10-12 15:39:54.825681
58103	MNK	2020-09-29	0.85	0.933	0.85	0.93	7504917	{}	\N	2020-10-12 15:39:54.830474
58104	MNK	2020-09-30	0.85	0.9733	0.85	0.9733	3969720	{}	\N	2020-10-12 15:39:54.835733
58105	MNK	2020-10-01	0.95	0.9999	0.9075	0.9599	4192900	{}	\N	2020-10-12 15:39:54.840194
58106	MNK	2020-10-02	0.89	0.9825	0.88	0.9773	6102052	{}	\N	2020-10-12 15:39:54.845588
58107	MNK	2020-10-05	0.96	0.96	0.9	0.9384	3864788	{}	\N	2020-10-12 15:39:54.85088
58108	MNK	2020-10-06	0.97	1.01	0.93	0.9518	5858149	{}	\N	2020-10-12 15:39:54.855728
58109	MNK	2020-10-07	0.94	1	0.94	0.97	3112743	{}	\N	2020-10-12 15:39:54.860386
58110	MNK	2020-10-08	0.977	1.11	0.97	1.09	7643774	{}	\N	2020-10-12 15:39:54.864827
58111	MNK	2020-10-09	1	1	0.75	0.75	33912985	{}	\N	2020-10-12 15:39:54.86998
58112	MOBL	2020-09-28	6.97	7.05	6.96	7.05	30839051	{}	\N	2020-10-12 15:39:56.061881
58113	MOBL	2020-09-29	7	7.06	6.99	7.05	10060737	{}	\N	2020-10-12 15:39:56.068736
58114	MOBL	2020-09-30	7.02	7.03	7.01	7.01	3787168	{}	\N	2020-10-12 15:39:56.074141
58115	MOBL	2020-10-01	7	7.04	7	7.03	1937497	{}	\N	2020-10-12 15:39:56.079932
58116	MOBL	2020-10-02	7.03	7.05	7.01	7.05	3246859	{}	\N	2020-10-12 15:39:56.085424
58117	MOBL	2020-10-05	7.04	7.1	7.02	7.08	5297766	{}	\N	2020-10-12 15:39:56.09162
58118	MOBL	2020-10-06	7.07	7.08	7.02	7.03	1753133	{}	\N	2020-10-12 15:39:56.097878
58119	MOBL	2020-10-07	7.03	7.04	7	7.02	1407936	{}	\N	2020-10-12 15:39:56.104813
58120	MOBL	2020-10-08	7.03	7.03	7.01	7.02	1203540	{}	\N	2020-10-12 15:39:56.109964
58121	MOBL	2020-10-09	7.02	7.03	7.01	7.03	1242122	{}	\N	2020-10-12 15:39:56.116027
58122	MOBL	2020-10-12	7.03	7.03	6.97	6.99	1799674	{}	\N	2020-10-12 15:39:56.122781
58123	MOTI	2020-09-28	29.23	29.35	29.23	29.35	3424	{}	\N	2020-10-12 15:39:57.048856
58124	MOTI	2020-09-29	29.1	29.24	29.1	29.21	6653	{}	\N	2020-10-12 15:39:57.054074
58125	MOTI	2020-09-30	29.14	29.42	29.14	29.33	6126	{}	\N	2020-10-12 15:39:57.058834
58126	MOTI	2020-10-01	29.4699	29.4699	29.341	29.36	14854	{}	\N	2020-10-12 15:39:57.065379
58127	MOTI	2020-10-02	29.36	29.4791	29.36	29.378	2290	{}	\N	2020-10-12 15:39:57.071717
58128	MOTI	2020-10-05	29.65	29.73	29.616	29.73	2570	{}	\N	2020-10-12 15:39:57.07715
58129	MOTI	2020-10-06	29.82	29.8263	29.6	29.8263	6242	{}	\N	2020-10-12 15:39:57.081358
58130	MOTI	2020-10-07	29.73	29.87	29.679	29.809	5331	{}	\N	2020-10-12 15:39:57.086229
58131	MOTI	2020-10-08	30.14	30.14	29.831	29.89	2532	{}	\N	2020-10-12 15:39:57.090767
58132	MOTI	2020-10-09	30.08	30.1097	30.072	30.1097	2014	{}	\N	2020-10-12 15:39:57.095401
58133	MOTI	2020-10-12	30.3483	30.3483	30.3483	30.3483	1029	{}	\N	2020-10-12 15:39:57.100075
58134	MRAM	2020-09-28	5.16	5.5	5.11	5.47	72022	{}	\N	2020-10-12 15:39:59.472201
58135	MRAM	2020-09-29	5.48	5.77	5.47	5.77	92838	{}	\N	2020-10-12 15:39:59.478398
58136	MRAM	2020-09-30	5.79	6.01	5.6	5.67	112203	{}	\N	2020-10-12 15:39:59.484679
58137	MRAM	2020-10-01	5.74	5.98	5.59	5.89	72941	{}	\N	2020-10-12 15:39:59.490537
58138	MRAM	2020-10-02	5.78	5.91	5.56	5.78	118490	{}	\N	2020-10-12 15:39:59.496298
58139	MRAM	2020-10-05	5.65	5.95	5.6134	5.89	96620	{}	\N	2020-10-12 15:39:59.502378
58140	MRAM	2020-10-06	5.8632	6.25	5.8632	5.92	124968	{}	\N	2020-10-12 15:39:59.508328
58141	MRAM	2020-10-07	6.1	6.2465	6.1	6.15	110915	{}	\N	2020-10-12 15:39:59.514779
58142	MRAM	2020-10-08	6.25	7.06	6.2341	6.85	257051	{}	\N	2020-10-12 15:39:59.520145
58143	MRAM	2020-10-09	7	7.0315	6.62	6.7	132972	{}	\N	2020-10-12 15:39:59.526017
58144	MRAM	2020-10-12	6.82	6.98	6.71	6.9	69171	{}	\N	2020-10-12 15:39:59.531165
58145	MRO	2020-09-28	4.21	4.37	4.13	4.29	25145894	{}	\N	2020-10-12 15:40:00.737952
58146	MRO	2020-09-29	4.26	4.27	4.04	4.16	47255858	{}	\N	2020-10-12 15:40:00.744105
58147	MRO	2020-09-30	4.15	4.21	4.06	4.09	48881807	{}	\N	2020-10-12 15:40:00.748628
58148	MRO	2020-10-01	4.05	4.08	3.93	3.95	49926359	{}	\N	2020-10-12 15:40:00.753286
58149	MRO	2020-10-02	3.82	4.13	3.79	4.12	67732053	{}	\N	2020-10-12 15:40:00.758252
58150	MRO	2020-10-05	4.21	4.24	4.09	4.23	52574623	{}	\N	2020-10-12 15:40:00.763642
58151	MRO	2020-10-06	4.29	4.38	4.03	4.04	57796046	{}	\N	2020-10-12 15:40:00.771708
58152	MRO	2020-10-07	4.1	4.15	4.03	4.13	43451228	{}	\N	2020-10-12 15:40:00.779735
58153	MRO	2020-10-08	4.11	4.38	4.06	4.38	52166976	{}	\N	2020-10-12 15:40:00.784414
58154	MRO	2020-10-09	4.41	4.475	4.2	4.26	44613683	{}	\N	2020-10-12 15:40:00.789222
58155	MRO	2020-10-12	4.24	4.32	4.18	4.295	25806472	{}	\N	2020-10-12 15:40:00.794454
58156	MS	2020-09-28	47.89	49.14	47.805	48.38	9385250	{}	\N	2020-10-12 15:40:02.161319
58157	MS	2020-09-29	48.07	48.26	47.19	47.24	22650839	{}	\N	2020-10-12 15:40:02.167001
58158	MS	2020-09-30	47.67	48.99	47.54	48.35	29341361	{}	\N	2020-10-12 15:40:02.171985
58159	MS	2020-10-01	48.75	49.07	46.9	47.26	49152673	{}	\N	2020-10-12 15:40:02.187242
58160	MS	2020-10-02	46.49	48.34	46.42	47.97	25100496	{}	\N	2020-10-12 15:40:02.195198
58161	MS	2020-10-05	48.43	48.89	47.78	48.6	36819014	{}	\N	2020-10-12 15:40:02.203103
58162	MS	2020-10-06	49.2	49.41	47.47	47.74	45438511	{}	\N	2020-10-12 15:40:02.210642
58163	MS	2020-10-07	48.38	49.17	48.24	48.71	24132506	{}	\N	2020-10-12 15:40:02.215396
58164	MS	2020-10-08	47.98	49.49	47.26	49	47946446	{}	\N	2020-10-12 15:40:02.220602
58165	MS	2020-10-09	49.44	49.73	48.73	48.83	21586710	{}	\N	2020-10-12 15:40:02.225904
58166	MS	2020-10-12	49.27	50.27	49.03	50.065	13900558	{}	\N	2020-10-12 15:40:02.230523
58167	MSTR	2020-09-28	148.45	153.41	148.45	151.46	140814	{}	\N	2020-10-12 15:40:04.176896
58168	MSTR	2020-09-29	150.67	151.1	148.08	149.69	60442	{}	\N	2020-10-12 15:40:04.183274
58169	MSTR	2020-09-30	150.22	153.07	150.03	150.56	79599	{}	\N	2020-10-12 15:40:04.188389
58170	MSTR	2020-10-01	150.66	152.71	148.65	148.96	66409	{}	\N	2020-10-12 15:40:04.194161
58171	MSTR	2020-10-02	146.32	149.95	145.16	146.46	107357	{}	\N	2020-10-12 15:40:04.200325
58172	MSTR	2020-10-05	147.73	149.74	146.9	148.21	48706	{}	\N	2020-10-12 15:40:04.205594
58173	MSTR	2020-10-06	149.32	154.3539	149.23	150.04	87270	{}	\N	2020-10-12 15:40:04.21176
58174	MSTR	2020-10-07	151.5	153.44	150.825	153.09	49451	{}	\N	2020-10-12 15:40:04.216933
58175	MSTR	2020-10-08	155.33	164.7899	155.33	163.94	169846	{}	\N	2020-10-12 15:40:04.221619
58176	MSTR	2020-10-09	164.59	165.53	160.65	164.73	98020	{}	\N	2020-10-12 15:40:04.227611
58177	MSTR	2020-10-12	168.02	169.4	164.3	165.805	37464	{}	\N	2020-10-12 15:40:04.233542
58178	MTCH	2020-09-28	107.15	109.84	106.775	108.59	2399183	{}	\N	2020-10-12 15:40:07.565111
58179	MTCH	2020-09-29	108.221	109.38	106.63	109.12	3420144	{}	\N	2020-10-12 15:40:07.571067
58180	MTCH	2020-09-30	108.43	111.79	108.43	110.65	2338484	{}	\N	2020-10-12 15:40:07.57717
58181	MTCH	2020-10-01	112.23	116.58	111.13	116.25	2634158	{}	\N	2020-10-12 15:40:07.583813
58182	MTCH	2020-10-02	113.13	115.9	111.63	111.94	1617450	{}	\N	2020-10-12 15:40:07.589286
58183	MTCH	2020-10-05	112.99	115.15	111.68	114.05	1710911	{}	\N	2020-10-12 15:40:07.594196
58184	MTCH	2020-10-06	113.94	115.82	112.79	114.21	1638711	{}	\N	2020-10-12 15:40:07.600279
58185	MTCH	2020-10-07	114.59	115.49	112.125	114.5	2241050	{}	\N	2020-10-12 15:40:07.605249
58186	MTCH	2020-10-08	115.99	117.49	113.1	113.45	2574339	{}	\N	2020-10-12 15:40:07.609937
58187	MTCH	2020-10-09	113.64	114.4	111.66	112.31	1445461	{}	\N	2020-10-12 15:40:07.616007
58188	MTCH	2020-10-12	112.5	113.9399	110.21	110.45	595933	{}	\N	2020-10-12 15:40:07.621941
58189	MPB	2020-09-28	17.69	18.04	17.46	17.52	4760	{}	\N	2020-10-12 15:40:10.336115
58190	MPB	2020-09-29	17.5	17.5	17.4	17.5	6902	{}	\N	2020-10-12 15:40:10.341836
58191	MPB	2020-09-30	17.5	17.8565	17.25	17.31	13890	{}	\N	2020-10-12 15:40:10.346722
58192	MPB	2020-10-01	17.48	17.57	17.25	17.3	6324	{}	\N	2020-10-12 15:40:10.352478
58193	MPB	2020-10-02	17.51	17.94	17.2	17.52	49359	{}	\N	2020-10-12 15:40:10.358774
58194	MPB	2020-10-05	17.61	18.22	17.61	18.12	7355	{}	\N	2020-10-12 15:40:10.36464
58195	MPB	2020-10-06	18.02	19.31	18.02	18.5	6385	{}	\N	2020-10-12 15:40:10.371661
58196	MPB	2020-10-07	18.665	19	18.2	19	7645	{}	\N	2020-10-12 15:40:10.376451
58197	MPB	2020-10-08	19.04	19.24	18.75	18.86	2682	{}	\N	2020-10-12 15:40:10.381257
58198	MPB	2020-10-09	19.01	19.01	18.58	18.69	2142	{}	\N	2020-10-12 15:40:10.38765
58199	MPB	2020-10-12	18.54	19.255	18.52	19.255	492	{}	\N	2020-10-12 15:40:10.393042
58200	MQY	2020-09-28	16.08	16.08	15.98	16.03	42066	{}	\N	2020-10-12 15:40:11.376635
58201	MQY	2020-09-29	16.13	16.18	16.03	16.17	116684	{}	\N	2020-10-12 15:40:11.382266
58202	MQY	2020-09-30	16.24	16.24	16.01	16.01	98988	{}	\N	2020-10-12 15:40:11.388379
58203	MQY	2020-10-01	16.15	16.56	16.1035	16.16	94508	{}	\N	2020-10-12 15:40:11.394118
58204	MQY	2020-10-02	16.2	16.32	16.11	16.26	73490	{}	\N	2020-10-12 15:40:11.401643
58205	MQY	2020-10-05	16.35	16.36	16.145	16.36	44521	{}	\N	2020-10-12 15:40:11.409182
58206	MQY	2020-10-06	16.44	16.455	16.26	16.34	50291	{}	\N	2020-10-12 15:40:11.415456
58207	MQY	2020-10-07	16.43	16.43	16.25	16.29	100412	{}	\N	2020-10-12 15:40:11.421564
58208	MQY	2020-10-08	16.35	16.35	16.25	16.25	109894	{}	\N	2020-10-12 15:40:11.427433
58209	MQY	2020-10-09	16.28	16.7299	16.28	16.34	74506	{}	\N	2020-10-12 15:40:11.432785
58210	MQY	2020-10-12	16.25	16.329	16.25	16.2852	62784	{}	\N	2020-10-12 15:40:11.439058
58211	MRLN	2020-09-28	6.83	7.24	6.83	7.17	31629	{}	\N	2020-10-12 15:40:12.458134
58212	MRLN	2020-09-29	7.25	7.2869	7.03	7.1	24122	{}	\N	2020-10-12 15:40:12.464306
58213	MRLN	2020-09-30	7.1611	7.17	6.97	7.05	34988	{}	\N	2020-10-12 15:40:12.470477
58214	MRLN	2020-10-01	7.14	7.19	7.0531	7.19	12465	{}	\N	2020-10-12 15:40:12.476185
58215	MRLN	2020-10-02	6.93	7.39	6.93	7.35	35880	{}	\N	2020-10-12 15:40:12.482263
58216	MRLN	2020-10-05	7.2	7.55	7.2	7.37	24287	{}	\N	2020-10-12 15:40:12.489019
58217	MRLN	2020-10-06	7.52	7.7	7.14	7.17	18756	{}	\N	2020-10-12 15:40:12.49548
58218	MRLN	2020-10-07	7.22	7.38	7.16	7.37	20226	{}	\N	2020-10-12 15:40:12.500413
58219	MRLN	2020-10-08	7.5097	7.8	7.35	7.74	26988	{}	\N	2020-10-12 15:40:12.507037
58220	MRLN	2020-10-09	7.8	7.94	7.6501	7.79	20271	{}	\N	2020-10-12 15:40:12.512734
58221	MRLN	2020-10-12	7.8	7.86	7.8	7.8	3655	{}	\N	2020-10-12 15:40:12.518101
58222	MRTX	2020-09-28	167.3	170.45	164.43	166.99	308732	{}	\N	2020-10-12 15:40:13.606515
58223	MRTX	2020-09-29	167.06	169.29	165.3	166.05	370312	{}	\N	2020-10-12 15:40:13.613396
58224	MRTX	2020-09-30	166.59	173.05	165.1108	166.05	571647	{}	\N	2020-10-12 15:40:13.619224
58225	MRTX	2020-10-01	170	171.63	166.51	169.89	511160	{}	\N	2020-10-12 15:40:13.625807
58226	MRTX	2020-10-02	167	170.4893	162.05	162.05	256455	{}	\N	2020-10-12 15:40:13.631459
58227	MRTX	2020-10-05	164.47	170.31	164.29	168.49	462506	{}	\N	2020-10-12 15:40:13.636734
58228	MRTX	2020-10-06	168.5	176.255	168.5	171.59	529322	{}	\N	2020-10-12 15:40:13.642452
58229	MRTX	2020-10-07	174.51	182.42	172.0022	180.17	442717	{}	\N	2020-10-12 15:40:13.647551
58230	MRTX	2020-10-08	182.44	187.26	179.75	184.67	352593	{}	\N	2020-10-12 15:40:13.65296
58231	MRTX	2020-10-09	178.66	185	178.43	183.3	314209	{}	\N	2020-10-12 15:40:13.659016
58232	MRTX	2020-10-12	184.46	186.3	182.53	183.975	56770	{}	\N	2020-10-12 15:40:13.665181
58233	MSGN	2020-09-28	10.02	10.28	9.9	9.93	497116	{}	\N	2020-10-12 15:40:15.631516
58234	MSGN	2020-09-29	9.92	9.92	9.63	9.75	551375	{}	\N	2020-10-12 15:40:15.636257
58235	MSGN	2020-09-30	9.72	9.96	9.53	9.57	601449	{}	\N	2020-10-12 15:40:15.641566
58236	MSGN	2020-10-01	9.57	9.63	8.83	9.15	2010734	{}	\N	2020-10-12 15:40:15.646499
58237	MSGN	2020-10-02	9.01	9.31	8.92	9.25	774940	{}	\N	2020-10-12 15:40:15.652309
58238	MSGN	2020-10-05	9.32	9.375	9.09	9.25	729822	{}	\N	2020-10-12 15:40:15.65894
58239	MSGN	2020-10-06	9.38	9.5	9.13	9.2	686482	{}	\N	2020-10-12 15:40:15.664874
58240	MSGN	2020-10-07	9.3	9.54	9.25	9.34	784054	{}	\N	2020-10-12 15:40:15.670669
58241	MSGN	2020-10-08	9.44	9.56	9.39	9.52	503188	{}	\N	2020-10-12 15:40:15.677587
58242	MSGN	2020-10-09	9.55	9.73	9.49	9.52	376066	{}	\N	2020-10-12 15:40:15.684616
58243	MSGN	2020-10-12	9.52	9.63	9.44	9.48	81558	{}	\N	2020-10-12 15:40:15.692484
58244	MTB	2020-09-28	93	95.375	92.03	94.22	834896	{}	\N	2020-10-12 15:40:17.371018
58245	MTB	2020-09-29	93.69	93.69	90.15	91.13	2095292	{}	\N	2020-10-12 15:40:17.377616
58246	MTB	2020-09-30	91.23	93.18	90.34	92.09	3028759	{}	\N	2020-10-12 15:40:17.382274
58247	MTB	2020-10-01	92.22	93.24	90.75	92.21	1600504	{}	\N	2020-10-12 15:40:17.387825
58248	MTB	2020-10-02	90.55	94.93	90.45	93.53	1924875	{}	\N	2020-10-12 15:40:17.394651
58249	MTB	2020-10-05	94.76	97.74	94.63	97.59	2353231	{}	\N	2020-10-12 15:40:17.400365
58250	MTB	2020-10-06	98.71	101.84	98.16	98.45	2625997	{}	\N	2020-10-12 15:40:17.405815
58251	MTB	2020-10-07	99.55	102.91	99.44	101.65	1929788	{}	\N	2020-10-12 15:40:17.412426
58252	MTB	2020-10-08	101.17	102.96	99.82	101.78	1632972	{}	\N	2020-10-12 15:40:17.41898
58253	MTB	2020-10-09	102.49	102.49	100.06	101.37	1094414	{}	\N	2020-10-12 15:40:17.425527
58254	MTB	2020-10-12	101	102.61	100.7865	102.495	265683	{}	\N	2020-10-12 15:40:17.431012
58255	MTH	2020-09-28	104.32	108.43	104	107.97	407467	{}	\N	2020-10-12 15:40:18.45565
58256	MTH	2020-09-29	107.97	109.1	106.29	106.63	599118	{}	\N	2020-10-12 15:40:18.463062
58257	MTH	2020-09-30	106.64	111.99	106.64	110.39	1291978	{}	\N	2020-10-12 15:40:18.468159
58258	MTH	2020-10-01	112.26	113.94	110.41	113.39	767534	{}	\N	2020-10-12 15:40:18.473374
58259	MTH	2020-10-02	110.73	116.31	110.45	115.01	617275	{}	\N	2020-10-12 15:40:18.479481
58260	MTH	2020-10-05	116.07	117.06	111.04	113.08	857268	{}	\N	2020-10-12 15:40:18.484424
58261	MTH	2020-10-06	114.24	115.1507	109	109.87	778933	{}	\N	2020-10-12 15:40:18.490626
58262	MTH	2020-10-07	112.49	113.305	107.2	109.23	855594	{}	\N	2020-10-12 15:40:18.49683
58263	MTH	2020-10-08	108.67	113.14	108.65	111.64	726861	{}	\N	2020-10-12 15:40:18.502229
58264	MTH	2020-10-09	113.6	114.39	111.37	112.72	700777	{}	\N	2020-10-12 15:40:18.508933
58265	MTH	2020-10-12	113.5	114.88	112.2268	114.52	117554	{}	\N	2020-10-12 15:40:18.514786
58266	MTL	2020-09-28	1.5	1.5	1.47	1.47	54446	{}	\N	2020-10-12 15:40:19.861356
58267	MTL	2020-09-29	1.49	1.52	1.46	1.47	296096	{}	\N	2020-10-12 15:40:19.874022
58268	MTL	2020-09-30	1.53	1.53	1.49	1.51	141052	{}	\N	2020-10-12 15:40:19.884196
58269	MTL	2020-10-01	1.51	1.54	1.49	1.5	84554	{}	\N	2020-10-12 15:40:19.892087
58270	MTL	2020-10-02	1.48	1.5	1.45	1.47	71660	{}	\N	2020-10-12 15:40:19.898743
58271	MTL	2020-10-05	1.47	1.5	1.46	1.46	68960	{}	\N	2020-10-12 15:40:19.904373
58272	MTL	2020-10-06	1.54	1.56	1.49	1.5	251394	{}	\N	2020-10-12 15:40:19.911197
58273	MTL	2020-10-07	1.48	1.51	1.48	1.5	87446	{}	\N	2020-10-12 15:40:19.920228
58274	MTL	2020-10-08	1.5	1.5277	1.46	1.46	140658	{}	\N	2020-10-12 15:40:19.926541
58275	MTL	2020-10-09	1.52	1.53	1.47	1.48	114502	{}	\N	2020-10-12 15:40:19.933207
58276	MTL	2020-10-12	1.48	1.52	1.48	1.5195	33088	{}	\N	2020-10-12 15:40:19.938443
58277	MTSC	2020-09-28	18.86	20.2799	18.79	19.88	149936	{}	\N	2020-10-12 15:40:21.564403
58278	MTSC	2020-09-29	19.73	19.83	18.65	19.1	128404	{}	\N	2020-10-12 15:40:21.571148
58279	MTSC	2020-09-30	19.19	19.65	18.86	19.11	114698	{}	\N	2020-10-12 15:40:21.576871
58280	MTSC	2020-10-01	18.91	19.38	18.84	19.29	93180	{}	\N	2020-10-12 15:40:21.584322
58281	MTSC	2020-10-02	18.65	20.15	18.51	19.99	154436	{}	\N	2020-10-12 15:40:21.592808
58282	MTSC	2020-10-05	20.46	20.52	19.69	20.4	105917	{}	\N	2020-10-12 15:40:21.600391
58283	MTSC	2020-10-06	20.76	21.5036	20	20.88	180091	{}	\N	2020-10-12 15:40:21.606262
58284	MTSC	2020-10-07	21.22	21.82	20.99	21.64	275245	{}	\N	2020-10-12 15:40:21.611637
58285	MTSC	2020-10-08	21.98	22.69	21.65	22.62	110354	{}	\N	2020-10-12 15:40:21.617842
58286	MTSC	2020-10-09	22.98	23.32	22.64	23.1	130114	{}	\N	2020-10-12 15:40:21.623729
58287	MTSC	2020-10-12	23.13	23.255	23.01	23.19	11394	{}	\N	2020-10-12 15:40:21.631292
58288	MTT	2020-09-28	20.19	20.31	20.1801	20.1895	10656	{}	\N	2020-10-12 15:40:22.62132
58289	MTT	2020-09-29	20.27	20.38	20.14	20.14	79052	{}	\N	2020-10-12 15:40:22.628304
58290	MTT	2020-09-30	20.15	20.3	20.15	20.24	65658	{}	\N	2020-10-12 15:40:22.640006
58291	MTT	2020-10-01	20.3	20.3	20.13	20.19	37494	{}	\N	2020-10-12 15:40:22.649825
58292	MTT	2020-10-02	20.24	20.3	20.18	20.2047	18840	{}	\N	2020-10-12 15:40:22.658394
58293	MTT	2020-10-05	20.26	20.26	20.18	20.18	16244	{}	\N	2020-10-12 15:40:22.670359
58294	MTT	2020-10-06	20.2	20.21	20.09	20.115	71974	{}	\N	2020-10-12 15:40:22.677711
58295	MTT	2020-10-07	20.13	20.26	20.13	20.15	11184	{}	\N	2020-10-12 15:40:22.68509
58296	MTT	2020-10-08	20.27	20.28	20.1849	20.1849	21652	{}	\N	2020-10-12 15:40:22.691388
58297	MTT	2020-10-09	20.25	20.28	20.13	20.1301	53908	{}	\N	2020-10-12 15:40:22.697138
58298	MTT	2020-10-12	20.15	20.181	20.15	20.16	17744	{}	\N	2020-10-12 15:40:22.703558
58299	MTZ	2020-09-28	42.91	43.76	42.7	43.13	481474	{}	\N	2020-10-12 15:40:24.033025
58300	MTZ	2020-09-29	42.95	43.69	42.25	43.3	1231515	{}	\N	2020-10-12 15:40:24.039053
58301	MTZ	2020-09-30	43.72	43.77	41.96	42.2	1157475	{}	\N	2020-10-12 15:40:24.043645
58302	MTZ	2020-10-01	42.16	42.69	41.33	41.5	1037623	{}	\N	2020-10-12 15:40:24.049119
58303	MTZ	2020-10-02	40.32	42.95	40.18	42.61	1262740	{}	\N	2020-10-12 15:40:24.055682
58304	MTZ	2020-10-05	43.07	45.73	43.07	43.9	1027025	{}	\N	2020-10-12 15:40:24.061205
58305	MTZ	2020-10-06	44.64	45.01	42.98	43.26	1652439	{}	\N	2020-10-12 15:40:24.067399
58306	MTZ	2020-10-07	43.7	45.94	43.7	45.74	1554320	{}	\N	2020-10-12 15:40:24.072143
58307	MTZ	2020-10-08	46.15	46.96	45.89	46.56	1358654	{}	\N	2020-10-12 15:40:24.076555
58308	MTZ	2020-10-09	46.94	47.1	46.31	46.75	2457612	{}	\N	2020-10-12 15:40:24.081553
58309	MTZ	2020-10-12	46.93	48.75	46.82	48.7	424985	{}	\N	2020-10-12 15:40:24.086905
58310	MRCY	2020-09-28	78.39	79.64	78.39	78.61	198524	{}	\N	2020-10-12 15:40:26.180463
58311	MRCY	2020-09-29	78.08	79.68	77.3	78.59	380833	{}	\N	2020-10-12 15:40:26.188762
58312	MRCY	2020-09-30	79.17	79.58	76.985	77.46	332637	{}	\N	2020-10-12 15:40:26.194345
58313	MRCY	2020-10-01	76.92	77.97	76.045	76.57	639971	{}	\N	2020-10-12 15:40:26.202184
58314	MRCY	2020-10-02	75	77.35	74.92	77.1	167406	{}	\N	2020-10-12 15:40:26.208203
58315	MRCY	2020-10-05	77.53	78.6	76.79	77.17	329799	{}	\N	2020-10-12 15:40:26.213787
58316	MRCY	2020-10-06	77.55	78.5325	76.41	76.8	203067	{}	\N	2020-10-12 15:40:26.221013
58317	MRCY	2020-10-07	77.33	78.11	75.53	75.62	241328	{}	\N	2020-10-12 15:40:26.225779
58318	MRCY	2020-10-08	76.27	77.09	74.99	76.71	120179	{}	\N	2020-10-12 15:40:26.230662
58319	MRCY	2020-10-09	77.23	78.81	77.23	78.51	245689	{}	\N	2020-10-12 15:40:26.23641
58320	MRCY	2020-10-12	78.82	79.34	78.63	78.9	29866	{}	\N	2020-10-12 15:40:26.241586
58321	MSB	2020-09-28	20.5	21.7494	20.5	21.66	63488	{}	\N	2020-10-12 15:40:28.689756
58322	MSB	2020-09-29	21.52	21.62	21.1209	21.62	71827	{}	\N	2020-10-12 15:40:28.695458
58323	MSB	2020-09-30	21.49	21.62	21.04	21.22	95036	{}	\N	2020-10-12 15:40:28.700278
58324	MSB	2020-10-01	21.24	21.44	20.76	21.1	65795	{}	\N	2020-10-12 15:40:28.705478
58325	MSB	2020-10-02	20.81	21.785	20.63	21.47	81280	{}	\N	2020-10-12 15:40:28.712286
58326	MSB	2020-10-05	21.59	22.21	21.59	22.2	133478	{}	\N	2020-10-12 15:40:28.718342
58327	MSB	2020-10-06	22.2	22.49	21.72	22.08	69036	{}	\N	2020-10-12 15:40:28.724599
58328	MSB	2020-10-07	22.08	22.44	22.08	22.42	40100	{}	\N	2020-10-12 15:40:28.729916
58329	MSB	2020-10-08	22.5	23.79	22.39	23.1	198060	{}	\N	2020-10-12 15:40:28.736611
58330	MSB	2020-10-09	23.1	23.45	22.43	22.51	73176	{}	\N	2020-10-12 15:40:28.743256
58331	MSB	2020-10-12	22.21	22.82	22.1323	22.82	24130	{}	\N	2020-10-12 15:40:28.749117
58332	MTBCP	2020-09-28	25.6	25.7	25.6	25.6612	21672	{}	\N	2020-10-12 15:40:30.908878
58333	MTBCP	2020-09-29	25.58	25.58	25.38	25.525	18444	{}	\N	2020-10-12 15:40:30.915435
58334	MTBCP	2020-09-30	25.5	25.6	25.4	25.505	19798	{}	\N	2020-10-12 15:40:30.921368
58335	MTBCP	2020-10-01	25.5333	25.55	25.35	25.48	21498	{}	\N	2020-10-12 15:40:30.930013
58336	MTBCP	2020-10-02	25.38	25.48	25.35	25.455	15160	{}	\N	2020-10-12 15:40:30.939264
58337	MTBCP	2020-10-05	25.5	25.5	25.4	25.47	25850	{}	\N	2020-10-12 15:40:30.947822
58338	MTBCP	2020-10-06	25.44	25.48	25.42	25.4612	8817	{}	\N	2020-10-12 15:40:30.953976
58339	MTBCP	2020-10-07	25.42	25.53	25.35	25.42	40050	{}	\N	2020-10-12 15:40:30.961186
58340	MTBCP	2020-10-08	25.48	25.4833	25.22	25.35	54804	{}	\N	2020-10-12 15:40:30.967253
58341	MTBCP	2020-10-09	25.29	25.3676	25.22	25.25	37591	{}	\N	2020-10-12 15:40:30.974273
58342	MTBCP	2020-10-12	25.16	25.29	25.16	25.24	13322	{}	\N	2020-10-12 15:40:30.980815
58343	MTG	2020-09-28	8.79	9.12	8.7	9.04	3611083	{}	\N	2020-10-12 15:40:32.337486
58344	MTG	2020-09-29	8.95	8.975	8.66	8.74	5594396	{}	\N	2020-10-12 15:40:32.34487
58345	MTG	2020-09-30	8.75	9.01	8.75	8.86	7156415	{}	\N	2020-10-12 15:40:32.351405
58346	MTG	2020-10-01	8.92	9.24	8.805	9.17	5272271	{}	\N	2020-10-12 15:40:32.360595
58347	MTG	2020-10-02	8.91	9.74	8.8503	9.66	9893833	{}	\N	2020-10-12 15:40:32.370074
58348	MTG	2020-10-05	9.74	9.96	9.725	9.85	5881813	{}	\N	2020-10-12 15:40:32.380082
58349	MTG	2020-10-06	9.99	10.335	9.885	9.93	8042250	{}	\N	2020-10-12 15:40:32.386408
58350	MTG	2020-10-07	10.33	10.83	10.275	10.45	13249369	{}	\N	2020-10-12 15:40:32.393844
58351	MTG	2020-10-08	10.55	10.65	10.225	10.62	10106274	{}	\N	2020-10-12 15:40:32.400244
58352	MTG	2020-10-09	10.75	10.78	10.46	10.51	10180829	{}	\N	2020-10-12 15:40:32.407113
58353	MTG	2020-10-12	10.61	10.78	10.54	10.765	1400507	{}	\N	2020-10-12 15:40:32.414655
58354	MTN	2020-09-28	228.14	230.94	221.09	222.63	871868	{}	\N	2020-10-12 15:40:33.541846
58355	MTN	2020-09-29	222.03	222.03	209.44	214.09	1107589	{}	\N	2020-10-12 15:40:33.550961
58356	MTN	2020-09-30	213.83	217.32	212.51	213.97	929636	{}	\N	2020-10-12 15:40:33.556993
58357	MTN	2020-10-01	216.11	217.96	212.74	215.1	892250	{}	\N	2020-10-12 15:40:33.565465
58358	MTN	2020-10-02	208.72	217.36	208.72	217.24	632162	{}	\N	2020-10-12 15:40:33.572081
58359	MTN	2020-10-05	218.92	223.46	217.68	222.43	581726	{}	\N	2020-10-12 15:40:33.57953
58360	MTN	2020-10-06	224	235.26	221.6	228.7	926525	{}	\N	2020-10-12 15:40:33.58539
58361	MTN	2020-10-07	230.95	233.43	228.655	233.42	667377	{}	\N	2020-10-12 15:40:33.591444
58362	MTN	2020-10-08	234.28	237.83	233.5768	237.04	521387	{}	\N	2020-10-12 15:40:33.597706
58363	MTN	2020-10-09	238.59	243.44	235.7259	241.64	788652	{}	\N	2020-10-12 15:40:33.604431
58364	MTN	2020-10-12	242.17	245.135	240.001	243.14	174919	{}	\N	2020-10-12 15:40:33.610969
58365	MTRX	2020-09-28	7.91	8.34	7.91	8.15	122479	{}	\N	2020-10-12 15:40:34.912427
58366	MTRX	2020-09-29	8.13	8.31	7.96	8.24	138911	{}	\N	2020-10-12 15:40:34.918183
58367	MTRX	2020-09-30	8.29	8.48	8.24	8.35	184090	{}	\N	2020-10-12 15:40:34.923553
58368	MTRX	2020-10-01	8.33	8.62	8.17	8.58	90987	{}	\N	2020-10-12 15:40:34.929679
58369	MTRX	2020-10-02	8.3	8.81	8.3	8.72	101830	{}	\N	2020-10-12 15:40:34.936102
58370	MTRX	2020-10-05	8.84	8.96	8.65	8.81	103556	{}	\N	2020-10-12 15:40:34.943392
58371	MTRX	2020-10-06	8.38	8.98	8.38	8.43	112126	{}	\N	2020-10-12 15:40:34.949542
58372	MTRX	2020-10-07	8.54	8.6	8.23	8.54	120527	{}	\N	2020-10-12 15:40:34.955681
58373	MTRX	2020-10-08	8.66	9.05	8.56	8.95	112412	{}	\N	2020-10-12 15:40:34.961746
58374	MTRX	2020-10-09	9.09	9.09	8.57	8.685	123478	{}	\N	2020-10-12 15:40:34.96763
58375	MTRX	2020-10-12	8.7	8.71	8.335	8.49	26644	{}	\N	2020-10-12 15:40:34.974462
58376	MTRN	2020-09-28	52.25	54.24	52.25	53.91	135195	{}	\N	2020-10-12 15:40:35.951005
58377	MTRN	2020-09-29	53.66	54.22	51.65	52.18	296975	{}	\N	2020-10-12 15:40:35.958241
58378	MTRN	2020-09-30	52.55	52.92	51.7	52.03	233164	{}	\N	2020-10-12 15:40:35.964274
58379	MTRN	2020-10-01	52.01	52.83	51.15	52.44	226607	{}	\N	2020-10-12 15:40:35.970559
58380	MTRN	2020-10-02	51.5	54.41	50.78	54.25	279043	{}	\N	2020-10-12 15:40:35.978429
58381	MTRN	2020-10-05	54.39	54.83	53.85	54.44	202387	{}	\N	2020-10-12 15:40:35.9853
58382	MTRN	2020-10-06	55.26	55.8	53.06	53.72	222657	{}	\N	2020-10-12 15:40:35.991735
58383	MTRN	2020-10-07	54.58	55.18	54.02	54.88	213325	{}	\N	2020-10-12 15:40:35.99794
58384	MTRN	2020-10-08	55.61	56.165	54.78	55.84	147399	{}	\N	2020-10-12 15:40:36.004342
58385	MTRN	2020-10-09	56.7	57.59	56.42	57.5	209999	{}	\N	2020-10-12 15:40:36.009922
58386	MTRN	2020-10-12	57.64	57.64	56.99	57.03	24405	{}	\N	2020-10-12 15:40:36.016135
58387	MTSL	2020-09-28	1.09	1.1499	1.07	1.145	35014	{}	\N	2020-10-12 15:40:36.923656
58388	MTSL	2020-09-29	1.14	1.415	1.13	1.28	981748	{}	\N	2020-10-12 15:40:36.930358
58389	MTSL	2020-09-30	1.29	1.33	1.17	1.2	69036	{}	\N	2020-10-12 15:40:36.935645
58390	MTSL	2020-10-01	1.16	1.23	1.16	1.22	43858	{}	\N	2020-10-12 15:40:36.94288
58391	MTSL	2020-10-02	1.17	1.2399	1.155	1.23	43357	{}	\N	2020-10-12 15:40:36.949566
58392	MTSL	2020-10-05	1.18	1.21	1.18	1.21	31505	{}	\N	2020-10-12 15:40:36.955367
58393	MTSL	2020-10-06	1.19	1.248	1.16	1.18	49634	{}	\N	2020-10-12 15:40:36.961058
58394	MTSL	2020-10-07	1.23	1.35	1.23	1.28	226382	{}	\N	2020-10-12 15:40:36.96675
58395	MTSL	2020-10-08	1.26	1.2607	1.15	1.22	70874	{}	\N	2020-10-12 15:40:36.972792
58396	MTSL	2020-10-09	1.2088	1.45	1.2088	1.32	326392	{}	\N	2020-10-12 15:40:36.978215
58397	MTSL	2020-10-12	1.2735	1.3306	1.27	1.3093	27095	{}	\N	2020-10-12 15:40:36.984832
58398	MTW	2020-09-28	8.28	8.59	8.2701	8.4	212680	{}	\N	2020-10-12 15:40:37.893183
58399	MTW	2020-09-29	8.39	8.43	8.12	8.33	409386	{}	\N	2020-10-12 15:40:37.905432
58400	MTW	2020-09-30	8.33	8.62	8.33	8.41	381262	{}	\N	2020-10-12 15:40:37.918049
58401	MTW	2020-10-01	8.46	8.6	8.34	8.48	327828	{}	\N	2020-10-12 15:40:37.926297
58402	MTW	2020-10-02	8.26	8.9	8.25	8.69	658495	{}	\N	2020-10-12 15:40:37.936007
58403	MTW	2020-10-05	8.8	9.39	8.8	9.19	758808	{}	\N	2020-10-12 15:40:37.944211
58404	MTW	2020-10-06	9.33	9.35	8.88	8.89	643432	{}	\N	2020-10-12 15:40:37.953124
58405	MTW	2020-10-07	9	9.15	8.95	8.99	822356	{}	\N	2020-10-12 15:40:37.959898
58406	MTW	2020-10-08	9.1	9.19	8.92	9.17	445860	{}	\N	2020-10-12 15:40:37.967145
58407	MTW	2020-10-09	9.23	9.38	9	9.05	401320	{}	\N	2020-10-12 15:40:37.97378
58408	MTW	2020-10-12	9.03	9.08	8.939	9.01	95119	{}	\N	2020-10-12 15:40:37.985461
58409	MU	2020-09-28	49.635	49.77	48.13	49.72	17767292	{}	\N	2020-10-12 15:40:39.141439
58410	MU	2020-09-29	49.71	51.38	49.59	50.7099	35097324	{}	\N	2020-10-12 15:40:39.147467
58411	MU	2020-09-30	48.95	49.27	46.83	46.96	53551244	{}	\N	2020-10-12 15:40:39.152981
58412	MU	2020-10-01	47.11	48.18	47.05	47.92	23287991	{}	\N	2020-10-12 15:40:39.160071
58413	MU	2020-10-02	46.83	47.52	46.5	46.55	18840791	{}	\N	2020-10-12 15:40:39.174997
58414	MU	2020-10-05	46.99	47.73	46.98	47.6	16629465	{}	\N	2020-10-12 15:40:39.183185
58415	MU	2020-10-06	47.55	48.83	47.215	47.32	16388937	{}	\N	2020-10-12 15:40:39.191507
58416	MU	2020-10-07	48.42	48.995	48.32	48.41	13298044	{}	\N	2020-10-12 15:40:39.199963
58417	MU	2020-10-08	48.88	49.9999	48.22	49.9	14322881	{}	\N	2020-10-12 15:40:39.205661
58418	MU	2020-10-09	50.24	50.6	49.31	49.89	13219837	{}	\N	2020-10-12 15:40:39.210052
58419	MU	2020-10-12	50.58	50.58	50.08	50.32	4959625	{}	\N	2020-10-12 15:40:39.214416
58420	MUB	2020-09-28	115.95	116.1	115.93	116.08	1103377	{}	\N	2020-10-12 15:40:40.238648
58421	MUB	2020-09-29	116.12	116.1426	116.07	116.11	1467127	{}	\N	2020-10-12 15:40:40.243988
58422	MUB	2020-09-30	116.02	116.06	115.9	115.93	1152308	{}	\N	2020-10-12 15:40:40.248689
58423	MUB	2020-10-01	115.71	115.72	115.58	115.71	1275299	{}	\N	2020-10-12 15:40:40.254686
58424	MUB	2020-10-02	115.64	115.8	115.5901	115.69	969693	{}	\N	2020-10-12 15:40:40.260315
58425	MUB	2020-10-05	115.65	115.65	115.47	115.54	1074641	{}	\N	2020-10-12 15:40:40.265347
58426	MUB	2020-10-06	115.45	115.55	115.42	115.48	1333458	{}	\N	2020-10-12 15:40:40.271267
58427	MUB	2020-10-07	115.39	115.39	115.24	115.29	864584	{}	\N	2020-10-12 15:40:40.276512
58428	MUB	2020-10-08	115.23	115.28	115.17	115.2	921043	{}	\N	2020-10-12 15:40:40.281446
58429	MUB	2020-10-09	115.19	115.29	115.15	115.26	858634	{}	\N	2020-10-12 15:40:40.287079
58430	MUB	2020-10-12	115.22	115.3	115.215	115.2886	241744	{}	\N	2020-10-12 15:40:40.292501
58431	MQT	2020-09-28	13	13.01	12.98	12.98	31504	{}	\N	2020-10-12 15:40:41.757356
58432	MQT	2020-09-29	12.97	13.06	12.97	13.06	64258	{}	\N	2020-10-12 15:40:41.769383
58433	MQT	2020-09-30	13.02	13.08	12.98	13.08	57890	{}	\N	2020-10-12 15:40:41.776137
58434	MQT	2020-10-01	13.17	13.19	13.0974	13.15	62294	{}	\N	2020-10-12 15:40:41.782747
58435	MQT	2020-10-02	13.16	13.17	13.07	13.16	49362	{}	\N	2020-10-12 15:40:41.789447
58436	MQT	2020-10-05	13.2	13.2	13.12	13.14	57990	{}	\N	2020-10-12 15:40:41.794793
58437	MQT	2020-10-06	13.19	13.2648	13.185	13.24	77678	{}	\N	2020-10-12 15:40:41.803565
58438	MQT	2020-10-07	13.37	13.4	13.225	13.32	229284	{}	\N	2020-10-12 15:40:41.809477
58439	MQT	2020-10-08	13.35	13.37	13.26	13.33	71034	{}	\N	2020-10-12 15:40:41.814772
58440	MQT	2020-10-09	13.4	13.4	13.2501	13.27	44848	{}	\N	2020-10-12 15:40:41.822208
58441	MQT	2020-10-12	13.39	13.4	13.3	13.35	59124	{}	\N	2020-10-12 15:40:41.828317
58442	MRTN	2020-09-28	16.44	16.88	16.26	16.79	461588	{}	\N	2020-10-12 15:40:43.10104
58443	MRTN	2020-09-29	16.69	16.81	16.305	16.64	259550	{}	\N	2020-10-12 15:40:43.107175
58444	MRTN	2020-09-30	16.73	16.73	16.2	16.32	308865	{}	\N	2020-10-12 15:40:43.112813
58445	MRTN	2020-10-01	16.39	16.525	16.17	16.43	355077	{}	\N	2020-10-12 15:40:43.118403
58446	MRTN	2020-10-02	16.31	17	16.26	16.77	345329	{}	\N	2020-10-12 15:40:43.125153
58447	MRTN	2020-10-05	16.81	17.1	16.4191	17.03	291011	{}	\N	2020-10-12 15:40:43.130127
58448	MRTN	2020-10-06	17.05	17.17	16.73	16.755	431636	{}	\N	2020-10-12 15:40:43.135119
58449	MRTN	2020-10-07	16.82	17.16	16.815	16.98	402627	{}	\N	2020-10-12 15:40:43.140972
58450	MRTN	2020-10-08	17.09	17.59	17.075	17.53	237384	{}	\N	2020-10-12 15:40:43.146002
58451	MRTN	2020-10-09	17.645	17.65	17.2	17.39	179781	{}	\N	2020-10-12 15:40:43.151528
58452	MRTN	2020-10-12	17.43	17.72	17.39	17.5	51858	{}	\N	2020-10-12 15:40:43.157555
58453	MSON	2020-09-28	11.84	12.23	11.75	11.87	38405	{}	\N	2020-10-12 15:40:44.049701
58454	MSON	2020-09-29	11.78	12.13	11.39	11.84	68801	{}	\N	2020-10-12 15:40:44.056885
58455	MSON	2020-09-30	12.03	12.17	11.6	11.73	46068	{}	\N	2020-10-12 15:40:44.062733
58456	MSON	2020-10-01	11.88	12.01	11.59	11.72	97467	{}	\N	2020-10-12 15:40:44.069678
58457	MSON	2020-10-02	11.54	12.31	11.25	11.41	35542	{}	\N	2020-10-12 15:40:44.075664
58458	MSON	2020-10-05	11.59	11.8092	11.33	11.57	21516	{}	\N	2020-10-12 15:40:44.080664
58459	MSON	2020-10-06	11.91	12.12	11.71	11.84	79173	{}	\N	2020-10-12 15:40:44.085585
58460	MSON	2020-10-07	11.85	12.76	11.64	12.64	67725	{}	\N	2020-10-12 15:40:44.092144
58461	MSON	2020-10-08	12.62	12.78	11.57	12.48	30312	{}	\N	2020-10-12 15:40:44.097393
58462	MSON	2020-10-09	12.58	12.765	12.32	12.67	31964	{}	\N	2020-10-12 15:40:44.103385
58463	MSON	2020-10-12	12.475	12.67	12.475	12.565	2067	{}	\N	2020-10-12 15:40:44.11157
58464	MTEX	2020-09-28	17.18	17.18	16.51	16.8	1087	{}	\N	2020-10-12 15:40:46.118164
58465	MTEX	2020-09-29	16.96	16.96	16.7536	16.7536	716	{}	\N	2020-10-12 15:40:46.12378
58466	MTEX	2020-10-01	16.81	17.1	16.76	16.76	676	{}	\N	2020-10-12 15:40:46.128862
58467	MTEX	2020-10-05	16.8	16.8	16.8	16.8	1025	{}	\N	2020-10-12 15:40:46.134409
58468	MTEX	2020-10-06	17.19	17.19	16.7	16.7	593	{}	\N	2020-10-12 15:40:46.14003
58469	MTEX	2020-10-09	16.9999	16.9999	16.8	16.8	679	{}	\N	2020-10-12 15:40:46.146159
58470	MTEX	2020-10-12	16.8	16.82	16.8	16.82	1333	{}	\N	2020-10-12 15:40:46.152318
58471	MTOR	2020-09-28	20.66	21.51	20.49	21.32	856448	{}	\N	2020-10-12 15:40:47.549004
58472	MTOR	2020-09-29	21.23	21.66	21.07	21.12	1063549	{}	\N	2020-10-12 15:40:47.554719
58473	MTOR	2020-09-30	21.18	21.7	20.69	20.94	1400024	{}	\N	2020-10-12 15:40:47.559344
58474	MTOR	2020-10-01	21.18	21.89	21.15	21.79	1070882	{}	\N	2020-10-12 15:40:47.563463
58475	MTOR	2020-10-02	21.14	22.76	21.12	22.62	1311428	{}	\N	2020-10-12 15:40:47.568408
58476	MTOR	2020-10-05	23.05	24.195	23.02	24.08	1346260	{}	\N	2020-10-12 15:40:47.574854
58477	MTOR	2020-10-06	24.45	24.92	23.85	23.86	1179270	{}	\N	2020-10-12 15:40:47.581624
58478	MTOR	2020-10-07	24.23	25.45	24.23	25.39	1379410	{}	\N	2020-10-12 15:40:47.587416
58479	MTOR	2020-10-08	25.71	26.18	24.96	25.39	1877524	{}	\N	2020-10-12 15:40:47.592263
58480	MTOR	2020-10-09	25.78	26.19	25.26	25.8	928255	{}	\N	2020-10-12 15:40:47.596595
58481	MTOR	2020-10-12	25.78	26.1	25.59	25.94	119812	{}	\N	2020-10-12 15:40:47.600392
58482	MTR	2020-09-28	3.71	4.06	3.58	3.79	16791	{}	\N	2020-10-12 15:40:48.851961
58483	MTR	2020-09-29	3.903	3.903	3.7101	3.8059	6084	{}	\N	2020-10-12 15:40:48.857338
58484	MTR	2020-09-30	3.89	3.92	3.7539	3.92	14630	{}	\N	2020-10-12 15:40:48.863147
58485	MTR	2020-10-01	4.15	4.15	3.66	3.9	13094	{}	\N	2020-10-12 15:40:48.86797
58486	MTR	2020-10-02	3.98	3.9997	3.6901	3.6901	4256	{}	\N	2020-10-12 15:40:48.87421
58487	MTR	2020-10-05	3.65	3.7844	3.63	3.63	7566	{}	\N	2020-10-12 15:40:48.881561
58488	MTR	2020-10-07	3.67	3.84	3.67	3.84	6404	{}	\N	2020-10-12 15:40:48.887474
58489	MTR	2020-10-08	3.85	4.14	3.78	3.94	40536	{}	\N	2020-10-12 15:40:48.892509
58490	MTR	2020-10-09	3.9188	4.5	3.88	3.88	69134	{}	\N	2020-10-12 15:40:48.897797
58491	MTR	2020-10-12	3.9	4.0315	3.8135	3.9663	23708	{}	\N	2020-10-12 15:40:48.902749
58492	MTX	2020-09-28	49.95	51.55	49.79	51.19	163460	{}	\N	2020-10-12 15:40:50.296403
58493	MTX	2020-09-29	51.33	51.53	50.5	51.16	306564	{}	\N	2020-10-12 15:40:50.301976
58494	MTX	2020-09-30	51.56	52.82	50.84	51.1	328478	{}	\N	2020-10-12 15:40:50.306394
58495	MTX	2020-10-01	51.49	51.895	50.93	51.54	290816	{}	\N	2020-10-12 15:40:50.310991
58496	MTX	2020-10-02	50.43	53.24	50.06	52.9	290884	{}	\N	2020-10-12 15:40:50.31721
58497	MTX	2020-10-05	53.39	54.86	53.39	54.79	322940	{}	\N	2020-10-12 15:40:50.324504
58498	MTX	2020-10-06	55.63	56.73	54.68	54.9	318431	{}	\N	2020-10-12 15:40:50.33133
58499	MTX	2020-10-07	55.71	57.93	55.68	57.48	511666	{}	\N	2020-10-12 15:40:50.336371
58500	MTX	2020-10-08	58.24	58.24	57.085	57.52	311842	{}	\N	2020-10-12 15:40:50.341208
58501	MTX	2020-10-09	58.01	58.29	57.4	57.67	267470	{}	\N	2020-10-12 15:40:50.346245
58502	MTX	2020-10-12	57.7	58.82	57.52	58.7	45559	{}	\N	2020-10-12 15:40:50.350865
58503	MUA	2020-09-28	13.77	13.87	13.77	13.87	24425	{}	\N	2020-10-12 15:40:51.462823
58504	MUA	2020-09-29	13.9	13.92	13.85	13.85	116450	{}	\N	2020-10-12 15:40:51.468655
58505	MUA	2020-09-30	13.86	13.91	13.76	13.76	81008	{}	\N	2020-10-12 15:40:51.474441
58506	MUA	2020-10-01	13.87	13.8999	13.788	13.85	119284	{}	\N	2020-10-12 15:40:51.479532
58507	MUA	2020-10-02	13.87	13.9	13.795	13.89	64348	{}	\N	2020-10-12 15:40:51.485429
58508	MUA	2020-10-05	13.86	13.9	13.79	13.88	65271	{}	\N	2020-10-12 15:40:51.491853
58509	MUA	2020-10-06	13.83	13.88	13.76	13.87	70254	{}	\N	2020-10-12 15:40:51.498157
58510	MUA	2020-10-07	13.95	13.97	13.8554	13.9	50012	{}	\N	2020-10-12 15:40:51.503157
58511	MUA	2020-10-08	14.02	14.05	13.95	13.96	89298	{}	\N	2020-10-12 15:40:51.508084
58512	MUA	2020-10-09	14.07	14.3145	13.95	14.13	137739	{}	\N	2020-10-12 15:40:51.513298
58513	MUA	2020-10-12	14.02	14.22	14.02	14.22	29112	{}	\N	2020-10-12 15:40:51.518633
58514	MUJ	2020-09-28	13.6	13.64	13.58	13.59	95536	{}	\N	2020-10-12 15:40:52.507221
58515	MUJ	2020-09-29	13.61	13.65	13.595	13.595	162504	{}	\N	2020-10-12 15:40:52.520325
58516	MUJ	2020-09-30	13.63	13.64	13.58	13.59	90198	{}	\N	2020-10-12 15:40:52.541811
58517	MUJ	2020-10-01	13.63	13.64	13.58	13.59	176772	{}	\N	2020-10-12 15:40:52.556318
58518	MUJ	2020-10-02	13.58	13.66	13.55	13.66	244364	{}	\N	2020-10-12 15:40:52.564032
58519	MUJ	2020-10-05	13.63	13.68	13.6	13.62	179898	{}	\N	2020-10-12 15:40:52.572073
58520	MUJ	2020-10-06	13.62	13.66	13.62	13.65	112165	{}	\N	2020-10-12 15:40:52.580412
58521	MUJ	2020-10-07	13.65	13.67	13.63	13.66	137826	{}	\N	2020-10-12 15:40:52.588543
58522	MUJ	2020-10-08	13.66	13.69	13.64	13.69	116000	{}	\N	2020-10-12 15:40:52.596389
58523	MUJ	2020-10-09	13.68	13.73	13.67	13.72	88620	{}	\N	2020-10-12 15:40:52.606144
58524	MUJ	2020-10-12	13.75	13.75	13.73	13.75	10538	{}	\N	2020-10-12 15:40:52.614831
58525	MUR	2020-09-28	8.97	9.4	8.87	9.2	4127486	{}	\N	2020-10-12 15:40:53.58072
58526	MUR	2020-09-29	9.12	9.12	8.7	8.82	7178978	{}	\N	2020-10-12 15:40:53.599037
58527	MUR	2020-09-30	8.89	9.295	8.79	8.92	10597169	{}	\N	2020-10-12 15:40:53.621637
58528	MUR	2020-10-01	8.71	8.83	8.34	8.43	8214672	{}	\N	2020-10-12 15:40:53.634726
58529	MUR	2020-10-02	8	8.795	7.96	8.57	11616524	{}	\N	2020-10-12 15:40:53.642533
58530	MUR	2020-10-05	8.79	8.9602	8.48	8.83	9323263	{}	\N	2020-10-12 15:40:53.651976
58531	MUR	2020-10-06	9.05	9.26	8.41	8.42	9047627	{}	\N	2020-10-12 15:40:53.658371
58532	MUR	2020-10-07	8.5	8.66	8.37	8.58	6405985	{}	\N	2020-10-12 15:40:53.664006
58533	MUR	2020-10-08	8.72	9.19	8.52	9.19	6939041	{}	\N	2020-10-12 15:40:53.669069
58534	MUR	2020-10-09	9.29	9.36	8.6	8.69	7997766	{}	\N	2020-10-12 15:40:53.675633
58535	MUR	2020-10-12	8.67	8.67	8.25	8.265	6477525	{}	\N	2020-10-12 15:40:53.679707
58536	MUS	2020-09-28	12.61	12.66	12.58	12.608	19483	{}	\N	2020-10-12 15:40:54.914076
58537	MUS	2020-09-29	12.65	12.67	12.6301	12.66	17858	{}	\N	2020-10-12 15:40:54.921687
58538	MUS	2020-09-30	12.63	12.88	12.6	12.88	32724	{}	\N	2020-10-12 15:40:54.926719
58539	MUS	2020-10-01	12.83	12.83	12.75	12.75	55134	{}	\N	2020-10-12 15:40:54.932346
58540	MUS	2020-10-02	12.69	12.83	12.69	12.83	47487	{}	\N	2020-10-12 15:40:54.939744
58541	MUS	2020-10-05	12.85	12.85	12.7873	12.85	37223	{}	\N	2020-10-12 15:40:54.945451
58542	MUS	2020-10-06	12.87	12.87	12.74	12.82	52255	{}	\N	2020-10-12 15:40:54.952314
58543	MUS	2020-10-07	12.92	12.92	12.79	12.8	98208	{}	\N	2020-10-12 15:40:54.957732
58544	MUS	2020-10-08	12.85	12.85	12.81	12.85	41792	{}	\N	2020-10-12 15:40:54.963282
58545	MUS	2020-10-09	12.83	12.8637	12.79	12.81	31344	{}	\N	2020-10-12 15:40:54.969545
58546	MUS	2020-10-12	12.92	12.99	12.9199	12.93	20622	{}	\N	2020-10-12 15:40:54.97457
58547	MRGR	2020-09-28	38.2207	38.2207	38.2207	38.2207	254	{}	\N	2020-10-12 15:40:56.455363
58548	MRGR	2020-09-29	38.3164	38.3164	38.3164	38.3164	102	{}	\N	2020-10-12 15:40:56.463692
58549	MRGR	2020-09-30	38.2811	38.2811	38.2811	38.2811	69	{}	\N	2020-10-12 15:40:56.470954
58550	MRGR	2020-10-01	38.3666	38.3666	38.3666	38.3666	145	{}	\N	2020-10-12 15:40:56.476837
58551	MRGR	2020-10-02	38.3	38.3	38.2824	38.2824	112	{}	\N	2020-10-12 15:40:56.483033
58552	MRGR	2020-10-05	38.36	38.5099	38.36	38.5099	4237	{}	\N	2020-10-12 15:40:56.490296
58553	MRGR	2020-10-06	38.5	38.5	38.4311	38.4311	200	{}	\N	2020-10-12 15:40:56.496492
58554	MRGR	2020-10-07	38.58	38.58	38.525	38.525	1258	{}	\N	2020-10-12 15:40:56.502483
58555	MRGR	2020-10-08	38.705	38.705	38.705	38.705	100	{}	\N	2020-10-12 15:40:56.508152
58556	MRGR	2020-10-09	38.56	38.6056	38.56	38.6056	201	{}	\N	2020-10-12 15:40:56.513528
58557	MSCI	2020-09-28	356.24	360.185	349.66	354.47	364207	{}	\N	2020-10-12 15:40:58.66092
58558	MSCI	2020-09-29	354.88	359.47	354.015	356.03	692728	{}	\N	2020-10-12 15:40:58.667045
58559	MSCI	2020-09-30	355.59	362.45	353.97	356.78	762667	{}	\N	2020-10-12 15:40:58.672397
58560	MSCI	2020-10-01	361.24	369	357.71	359.54	619026	{}	\N	2020-10-12 15:40:58.678246
58561	MSCI	2020-10-02	355.68	358.6231	347.41	348.16	552488	{}	\N	2020-10-12 15:40:58.684465
58562	MSCI	2020-10-05	350.86	353.1	348.02	351.31	725786	{}	\N	2020-10-12 15:40:58.690594
58563	MSCI	2020-10-06	355.49	355.49	345.2313	347.92	819561	{}	\N	2020-10-12 15:40:58.696608
58564	MSCI	2020-10-07	348.16	351.51	342.515	342.81	1276888	{}	\N	2020-10-12 15:40:58.704431
58565	MSCI	2020-10-08	343.97	349.06	342.61	346.4	957373	{}	\N	2020-10-12 15:40:58.711292
58566	MSCI	2020-10-09	349.27	354.9	347.23	352.33	571817	{}	\N	2020-10-12 15:40:58.716756
58567	MSCI	2020-10-12	354.76	355.86	350.23	353.21	155639	{}	\N	2020-10-12 15:40:58.723349
58568	MT	2020-09-28	13.17	13.475	12.98	13.37	6622877	{}	\N	2020-10-12 15:40:59.636593
58569	MT	2020-09-29	13.49	13.61	13.2317	13.34	9018915	{}	\N	2020-10-12 15:40:59.642607
58570	MT	2020-09-30	13.2	13.5	13.125	13.25	7742950	{}	\N	2020-10-12 15:40:59.647348
58571	MT	2020-10-01	13.62	13.7	13.45	13.62	5940494	{}	\N	2020-10-12 15:40:59.652217
58572	MT	2020-10-02	13.36	13.79	13.351	13.76	8542661	{}	\N	2020-10-12 15:40:59.658381
58573	MT	2020-10-05	13.77	14.085	13.77	14.03	6757352	{}	\N	2020-10-12 15:40:59.665765
58574	MT	2020-10-06	14.09	14.17	13.53	13.7	8022563	{}	\N	2020-10-12 15:40:59.672017
58575	MT	2020-10-07	14.13	14.54	14.11	14.4	5936078	{}	\N	2020-10-12 15:40:59.677725
58576	MT	2020-10-08	14.45	14.61	14.4	14.57	8297303	{}	\N	2020-10-12 15:40:59.68297
58577	MT	2020-10-09	14.61	14.6859	14.26	14.28	5908341	{}	\N	2020-10-12 15:40:59.688416
58578	MT	2020-10-12	14.16	14.23	14.115	14.19	1573021	{}	\N	2020-10-12 15:40:59.694041
58579	MTEM	2020-09-28	10.66	10.705	10.36	10.49	241934	{}	\N	2020-10-12 15:41:00.817654
58580	MTEM	2020-09-29	10.47	10.65	10.27	10.5	131975	{}	\N	2020-10-12 15:41:00.827873
58581	MTEM	2020-09-30	10.5	10.96	10.5	10.92	272436	{}	\N	2020-10-12 15:41:00.836437
58582	MTEM	2020-10-01	11.05	11.69	10.94	11.61	373102	{}	\N	2020-10-12 15:41:00.845015
58583	MTEM	2020-10-02	11.16	11.5488	10.91	10.96	177917	{}	\N	2020-10-12 15:41:00.85401
58584	MTEM	2020-10-05	11.09	11.575	10.87	11.1	109508	{}	\N	2020-10-12 15:41:00.862493
58585	MTEM	2020-10-06	11.32	11.49	10.97	11.27	112347	{}	\N	2020-10-12 15:41:00.873424
58586	MTEM	2020-10-07	11.38	11.465	11	11.26	184096	{}	\N	2020-10-12 15:41:00.883226
58587	MTEM	2020-10-08	11.49	11.71	11.045	11.58	154096	{}	\N	2020-10-12 15:41:00.892382
58588	MTEM	2020-10-09	11.44	12.42	11.37	12.24	163079	{}	\N	2020-10-12 15:41:00.901461
58589	MTEM	2020-10-12	12.27	12.27	11.73	12.08	76720	{}	\N	2020-10-12 15:41:00.910465
58590	MTLS	2020-09-28	35.46	37.43	35.19	36.92	109572	{}	\N	2020-10-12 15:41:03.815742
58591	MTLS	2020-09-29	36.19	38	35.67	37.71	92126	{}	\N	2020-10-12 15:41:03.822699
58592	MTLS	2020-09-30	38.19	38.4	36.7003	36.91	100429	{}	\N	2020-10-12 15:41:03.82831
58593	MTLS	2020-10-01	37.98	38.5681	37.3855	38.12	63921	{}	\N	2020-10-12 15:41:03.834884
58594	MTLS	2020-10-02	36.59	39.2	36	37.52	88590	{}	\N	2020-10-12 15:41:03.842517
58595	MTLS	2020-10-05	38.06	39.6999	37.95	39.54	85934	{}	\N	2020-10-12 15:41:03.851258
58596	MTLS	2020-10-06	39.3	41.73	39	40.25	128129	{}	\N	2020-10-12 15:41:03.858735
58597	MTLS	2020-10-07	40.51	42.8	40.51	42.35	130060	{}	\N	2020-10-12 15:41:03.866099
58598	MTLS	2020-10-08	43	43	41.81	42.36	126421	{}	\N	2020-10-12 15:41:03.872576
58599	MTLS	2020-10-09	42.74	45.49	42.51	44.9	275259	{}	\N	2020-10-12 15:41:03.880337
58600	MTLS	2020-10-12	45	47	44.68	46.02	121453	{}	\N	2020-10-12 15:41:03.891646
58601	MTP	2020-09-28	1.76	1.85	1.7143	1.75	95728	{}	\N	2020-10-12 15:41:04.961685
58602	MTP	2020-09-29	1.8	1.8633	1.77	1.81	75697	{}	\N	2020-10-12 15:41:04.969273
58603	MTP	2020-09-30	1.8999	1.8999	1.78	1.81	92705	{}	\N	2020-10-12 15:41:04.974765
58604	MTP	2020-10-01	1.85	1.85	1.7514	1.82	63265	{}	\N	2020-10-12 15:41:04.980512
58605	MTP	2020-10-02	1.76	1.81	1.76	1.78	36336	{}	\N	2020-10-12 15:41:04.986142
58606	MTP	2020-10-05	1.79	1.878	1.76	1.87	86946	{}	\N	2020-10-12 15:41:04.991639
58607	MTP	2020-10-06	1.87	1.91	1.82	1.82	82814	{}	\N	2020-10-12 15:41:04.997349
58608	MTP	2020-10-07	1.83	1.87	1.79	1.81	101416	{}	\N	2020-10-12 15:41:05.002023
58609	MTP	2020-10-08	1.84	1.95	1.82	1.85	242290	{}	\N	2020-10-12 15:41:05.008315
58610	MTP	2020-10-09	1.9	1.96	1.8718	1.92	98777	{}	\N	2020-10-12 15:41:05.015204
58611	MTP	2020-10-12	1.97	1.97	1.9	1.94	38325	{}	\N	2020-10-12 15:41:05.020217
58612	MUE	2020-09-28	12.55	12.64	12.55	12.63	30039	{}	\N	2020-10-12 15:41:06.760635
58613	MUE	2020-09-29	12.64	12.66	12.621	12.65	76876	{}	\N	2020-10-12 15:41:06.769719
58614	MUE	2020-09-30	12.65	12.74	12.63	12.74	85630	{}	\N	2020-10-12 15:41:06.777241
58615	MUE	2020-10-01	12.77	12.8	12.735	12.8	49114	{}	\N	2020-10-12 15:41:06.787692
58616	MUE	2020-10-02	12.76	12.87	12.76	12.86	49370	{}	\N	2020-10-12 15:41:06.796967
58617	MUE	2020-10-05	12.87	12.89	12.74	12.85	89592	{}	\N	2020-10-12 15:41:06.806017
58618	MUE	2020-10-06	12.87	12.88	12.77	12.83	85500	{}	\N	2020-10-12 15:41:06.814187
58619	MUE	2020-10-07	12.9	12.9	12.75	12.77	148378	{}	\N	2020-10-12 15:41:06.819735
58620	MUE	2020-10-08	12.8	12.8192	12.74	12.8	77108	{}	\N	2020-10-12 15:41:06.825345
58621	MUE	2020-10-09	12.8	12.8	12.64	12.69	96382	{}	\N	2020-10-12 15:41:06.83172
58622	MUE	2020-10-12	12.79	12.79	12.731	12.7408	20630	{}	\N	2020-10-12 15:41:06.837532
58623	MUH	2020-09-28	14.67	14.7	14.61	14.6699	10547	{}	\N	2020-10-12 15:41:12.061916
58624	MUH	2020-09-29	14.7	14.7	14.63	14.69	57978	{}	\N	2020-10-12 15:41:12.067917
58625	MUH	2020-09-30	14.66	14.68	14.6	14.67	20828	{}	\N	2020-10-12 15:41:12.073287
58626	MUH	2020-10-01	14.69	14.7	14.655	14.655	41766	{}	\N	2020-10-12 15:41:12.077588
58627	MUH	2020-10-02	14.66	14.7	14.51	14.7	22276	{}	\N	2020-10-12 15:41:12.08295
58628	MUH	2020-10-05	14.7	14.7	14.59	14.64	41322	{}	\N	2020-10-12 15:41:12.089285
58629	MUH	2020-10-06	14.64	14.64	14.47	14.56	51245	{}	\N	2020-10-12 15:41:12.094897
58630	MUH	2020-10-07	14.68	14.7	14.57	14.58	57816	{}	\N	2020-10-12 15:41:12.101934
58631	MUH	2020-10-08	14.7	14.7	14.57	14.63	27372	{}	\N	2020-10-12 15:41:12.108698
58632	MUH	2020-10-09	14.65	14.67	14.6	14.61	22680	{}	\N	2020-10-12 15:41:12.114873
58633	MUH	2020-10-12	14.7	14.7	14.6	14.6	13672	{}	\N	2020-10-12 15:41:12.120497
58634	MUI	2020-09-28	13.97	14.08	13.97	13.99	64502	{}	\N	2020-10-12 15:41:13.289176
58635	MUI	2020-09-29	14.05	14.08	13.99	14.07	55432	{}	\N	2020-10-12 15:41:13.294846
58636	MUI	2020-09-30	14.04	14.08	14.03	14.03	103390	{}	\N	2020-10-12 15:41:13.300649
58637	MUI	2020-10-01	14.13	14.25	14.06	14.24	252270	{}	\N	2020-10-12 15:41:13.306018
58638	MUI	2020-10-02	14.23	14.29	14.13	14.23	64046	{}	\N	2020-10-12 15:41:13.313382
58639	MUI	2020-10-05	14.23	14.27	14.18	14.19	106320	{}	\N	2020-10-12 15:41:13.318908
58640	MUI	2020-10-06	14.22	14.27	14.185	14.26	110629	{}	\N	2020-10-12 15:41:13.324695
58641	MUI	2020-10-07	14.28	14.35	14.2785	14.31	63398	{}	\N	2020-10-12 15:41:13.329764
58642	MUI	2020-10-08	14.28	14.35	14.2663	14.33	62686	{}	\N	2020-10-12 15:41:13.334585
58643	MUI	2020-10-09	14.31	14.3799	14.2885	14.32	51128	{}	\N	2020-10-12 15:41:13.33952
58644	MUI	2020-10-12	14.46	14.46	14.3001	14.36	32044	{}	\N	2020-10-12 15:41:13.34431
58645	MVBF	2020-09-28	15.56	16.14	15.51	15.51	6268	{}	\N	2020-10-12 15:41:14.31029
58646	MVBF	2020-09-29	15.77	15.77	15.51	15.56	7808	{}	\N	2020-10-12 15:41:14.317167
58647	MVBF	2020-09-30	15.75	15.995	15.64	15.97	11994	{}	\N	2020-10-12 15:41:14.321942
58648	MVBF	2020-10-01	15.99	16.7306	15.99	16.4	15302	{}	\N	2020-10-12 15:41:14.328486
58649	MVBF	2020-10-02	16.22	16.45	16.22	16.34	15918	{}	\N	2020-10-12 15:41:14.333521
58650	MVBF	2020-10-05	16.4	16.78	16.37	16.37	14824	{}	\N	2020-10-12 15:41:14.337737
58651	MVBF	2020-10-06	16.57	17.14	16.19	16.19	12506	{}	\N	2020-10-12 15:41:14.343313
58652	MVBF	2020-10-07	16.51	17.45	16.2393	17.15	30795	{}	\N	2020-10-12 15:41:14.348648
58653	MVBF	2020-10-08	16.85	17.87	16.85	17.59	20080	{}	\N	2020-10-12 15:41:14.353274
58654	MVBF	2020-10-09	17.5	17.5	17.08	17.08	22382	{}	\N	2020-10-12 15:41:14.358762
58655	MVBF	2020-10-12	17.25	17.36	17.25	17.36	1354	{}	\N	2020-10-12 15:41:14.363049
58656	MVC	2020-09-28	8	8	7.93	7.97	57487	{}	\N	2020-10-12 15:41:15.438818
58657	MVC	2020-09-29	7.91	7.97	7.86	7.86	15630	{}	\N	2020-10-12 15:41:15.444538
58658	MVC	2020-09-30	7.89	7.93	7.805	7.81	55468	{}	\N	2020-10-12 15:41:15.449206
58659	MVC	2020-10-01	7.77	7.98	7.77	7.98	26078	{}	\N	2020-10-12 15:41:15.454527
58660	MVC	2020-10-02	7.94	8	7.94	8	111023	{}	\N	2020-10-12 15:41:15.460474
58661	MVC	2020-10-05	8	8	7.89	7.91	26095	{}	\N	2020-10-12 15:41:15.465762
58662	MVC	2020-10-06	7.99	8.03	7.96	7.97	142044	{}	\N	2020-10-12 15:41:15.471864
58663	MVC	2020-10-07	7.98	8.025	7.93	7.97	18564	{}	\N	2020-10-12 15:41:15.478338
58664	MVC	2020-10-08	8	8.02	7.96	7.96	302884	{}	\N	2020-10-12 15:41:15.483443
58665	MVC	2020-10-09	8.05	8.05	7.99	8.02	66738	{}	\N	2020-10-12 15:41:15.488614
58666	MVC	2020-10-12	7.96	7.9696	7.94	7.9696	2146	{}	\N	2020-10-12 15:41:15.49401
58667	MVCD	2020-09-28	25.35	25.39	25.35	25.39	6014	{}	\N	2020-10-12 15:41:16.979903
58668	MVCD	2020-09-29	25.3	25.435	25.2797	25.34	13378	{}	\N	2020-10-12 15:41:16.985767
58669	MVCD	2020-09-30	25.09	25.09	25	25.0102	9646	{}	\N	2020-10-12 15:41:16.990968
58670	MVCD	2020-10-01	25.01	25.12	25.01	25.1	4088	{}	\N	2020-10-12 15:41:16.996787
58671	MVCD	2020-10-02	25.05	25.0767	25.05	25.0767	1666	{}	\N	2020-10-12 15:41:17.002209
58672	MVCD	2020-10-05	25.0628	25.0628	25.025	25.03	109972	{}	\N	2020-10-12 15:41:17.00889
58673	MVCD	2020-10-06	25.05	25.08	25.05	25.078	11994	{}	\N	2020-10-12 15:41:17.013645
58674	MVCD	2020-10-07	25.07	25.09	25.0501	25.0501	2128	{}	\N	2020-10-12 15:41:17.019577
58675	MVCD	2020-10-08	25.09	25.14	25.07	25.07	12148	{}	\N	2020-10-12 15:41:17.024461
58676	MVCD	2020-10-09	25.02	25.129	25.02	25.1	30210	{}	\N	2020-10-12 15:41:17.030311
58677	MVCD	2020-10-12	25.14	25.14	25.14	25.14	652	{}	\N	2020-10-12 15:41:17.035708
58678	MVF	2020-09-28	8.61	8.64	8.59	8.64	92221	{}	\N	2020-10-12 15:41:17.941846
58679	MVF	2020-09-29	8.63	8.68	8.62	8.62	184252	{}	\N	2020-10-12 15:41:17.948142
58680	MVF	2020-09-30	8.63	8.68	8.63	8.68	111970	{}	\N	2020-10-12 15:41:17.955601
58681	MVF	2020-10-01	8.68	8.68	8.66	8.68	240941	{}	\N	2020-10-12 15:41:17.962534
58682	MVF	2020-10-02	8.62	8.68	8.6186	8.66	130040	{}	\N	2020-10-12 15:41:17.968887
58683	MVF	2020-10-05	8.65	8.68	8.63	8.67	214582	{}	\N	2020-10-12 15:41:17.974988
58684	MVF	2020-10-06	8.67	8.69	8.63	8.68	96748	{}	\N	2020-10-12 15:41:17.983048
58685	MVF	2020-10-07	8.7	8.71	8.67	8.68	95078	{}	\N	2020-10-12 15:41:17.988843
58686	MVF	2020-10-08	8.72	8.72	8.66	8.7	108170	{}	\N	2020-10-12 15:41:17.99547
58687	MVF	2020-10-09	8.75	8.8	8.695	8.76	178594	{}	\N	2020-10-12 15:41:18.001237
58688	MVF	2020-10-12	8.82	8.82	8.73	8.7499	22972	{}	\N	2020-10-12 15:41:18.007
58689	KOS	2020-09-28	1.05	1.08	1	1.06	6884739	{}	\N	2020-10-12 15:41:18.999707
58690	KOS	2020-09-29	1.05	1.07	0.97	0.9973	17921690	{}	\N	2020-10-12 15:41:19.006322
58691	KOS	2020-09-30	1.01	1.03	0.9601	0.9756	18825864	{}	\N	2020-10-12 15:41:19.01397
58692	KOS	2020-10-01	0.98	1	0.955101	0.98	11191255	{}	\N	2020-10-12 15:41:19.020479
58693	KOS	2020-10-02	0.95	1.03	0.9	1.03	16207934	{}	\N	2020-10-12 15:41:19.026745
58694	KOS	2020-10-05	1.31	1.32	1.08	1.12	86825418	{}	\N	2020-10-12 15:41:19.032801
58695	KOS	2020-10-06	1.13	1.1392	1.01	1.04	20917674	{}	\N	2020-10-12 15:41:19.038994
58696	KOS	2020-10-07	1.07	1.08	1.04	1.08	11256766	{}	\N	2020-10-12 15:41:19.044964
58697	KOS	2020-10-08	1.09	1.13	1.06	1.13	12652045	{}	\N	2020-10-12 15:41:19.050178
58698	KOS	2020-10-09	1.14	1.16	1.1	1.13	9014992	{}	\N	2020-10-12 15:41:19.055663
58699	KOS	2020-10-12	1.11	1.12	1.07	1.0974	3449887	{}	\N	2020-10-12 15:41:19.063254
58700	KRC	2020-09-28	53.15	54.17	52.98	53.42	886875	{}	\N	2020-10-12 15:41:20.174714
58701	KRC	2020-09-29	52.64	52.81	51.04	51.76	1455870	{}	\N	2020-10-12 15:41:20.181519
58702	KRC	2020-09-30	52.11	53.06	51.56	51.96	2245771	{}	\N	2020-10-12 15:41:20.195547
58703	KRC	2020-10-01	52.03	52.74	51.81	52.73	2382307	{}	\N	2020-10-12 15:41:20.203679
58704	KRC	2020-10-02	51.81	54.09	51.8	53.92	1769930	{}	\N	2020-10-12 15:41:20.213756
58705	KRC	2020-10-05	54.45	54.675	53.35	53.89	1946501	{}	\N	2020-10-12 15:41:20.22961
58706	KRC	2020-10-06	54.02	54.4	53.34	53.81	1509938	{}	\N	2020-10-12 15:41:20.242993
58707	KRC	2020-10-07	54.17	54.24	52.83	53.21	1828230	{}	\N	2020-10-12 15:41:20.251076
58708	KRC	2020-10-08	53.68	54.41	53.52	54.27	2355064	{}	\N	2020-10-12 15:41:20.258989
58709	KRC	2020-10-09	54.79	54.855	53.29	53.94	1007290	{}	\N	2020-10-12 15:41:20.267351
58710	KRC	2020-10-12	53.94	54.18	53.42	54.1	160480	{}	\N	2020-10-12 15:41:20.27582
58711	KSA	2020-09-28	29.74	29.77	29.61	29.71	380091	{}	\N	2020-10-12 15:41:21.262828
58712	KSA	2020-09-29	29.55	29.55	29.29	29.39	422574	{}	\N	2020-10-12 15:41:21.269823
58713	KSA	2020-09-30	29.63	29.79	29.56	29.7	211476	{}	\N	2020-10-12 15:41:21.276127
58714	KSA	2020-10-01	29.49	29.5799	29.25	29.55	186378	{}	\N	2020-10-12 15:41:21.282839
58715	KSA	2020-10-02	29.23	29.48	29.165	29.26	301722	{}	\N	2020-10-12 15:41:21.289707
58716	KSA	2020-10-05	29.27	29.5	29.265	29.43	590735	{}	\N	2020-10-12 15:41:21.295827
58717	KSA	2020-10-06	29.75	30.04	29.75	29.91	410754	{}	\N	2020-10-12 15:41:21.301823
58718	KSA	2020-10-07	29.87	30.02	29.76	29.89	418602	{}	\N	2020-10-12 15:41:21.3085
58719	KSA	2020-10-08	30.09	30.28	30	30.26	249407	{}	\N	2020-10-12 15:41:21.314814
58720	KSA	2020-10-09	30.29	30.42	30.165	30.35	128738	{}	\N	2020-10-12 15:41:21.320047
58721	KSA	2020-10-12	30.5	30.65	30.44	30.54	263965	{}	\N	2020-10-12 15:41:21.325308
58722	KTN	2020-09-28	33.89	33.9067	33.82	33.8685	735	{}	\N	2020-10-12 15:41:22.236155
58723	KTN	2020-09-29	33.83	33.83	33.7	33.7	1200	{}	\N	2020-10-12 15:41:22.244645
58724	KTN	2020-09-30	33.8	33.8	33.7696	33.8	1700	{}	\N	2020-10-12 15:41:22.250478
58725	KTN	2020-10-01	33.86	33.86	33.68	33.8124	980	{}	\N	2020-10-12 15:41:22.256134
58726	KTN	2020-10-02	33.66	33.83	33.66	33.83	2194	{}	\N	2020-10-12 15:41:22.26212
58727	KTN	2020-10-05	33.7998	33.8	33.68	33.76	2454	{}	\N	2020-10-12 15:41:22.268952
58728	KTN	2020-10-06	33.8102	33.8999	33.72	33.8299	2598	{}	\N	2020-10-12 15:41:22.274594
58729	KTN	2020-10-07	33.76	33.76	33.76	33.76	800	{}	\N	2020-10-12 15:41:22.280797
58730	KTN	2020-10-08	33.75	33.8	33.75	33.77	1352	{}	\N	2020-10-12 15:41:22.286581
58731	KTN	2020-10-09	33.8329	33.8329	33.02	33.375	2000	{}	\N	2020-10-12 15:41:22.292543
58732	KTN	2020-10-12	32.83	33.39	32.83	33.25	10578	{}	\N	2020-10-12 15:41:22.297763
58733	KXI	2020-09-28	55.15	55.505	55.15	55.35	54629	{}	\N	2020-10-12 15:41:23.713703
58734	KXI	2020-09-29	55.29	55.46	54.99	55.15	25328	{}	\N	2020-10-12 15:41:23.719685
58735	KXI	2020-09-30	55.17	55.65	55.15	55.4	21457	{}	\N	2020-10-12 15:41:23.723878
58736	KXI	2020-10-01	55.59	55.72	55.275	55.63	64467	{}	\N	2020-10-12 15:41:23.72849
58737	KXI	2020-10-02	55.18	55.7	55.18	55.47	97503	{}	\N	2020-10-12 15:41:23.735976
58738	KXI	2020-10-05	55.69	55.95	55.69	55.94	11873	{}	\N	2020-10-12 15:41:23.74192
58739	KXI	2020-10-06	55.98	56.04	55.32	55.4	18581	{}	\N	2020-10-12 15:41:23.746534
58740	KXI	2020-10-07	55.73	55.99	55.67	55.88	22391	{}	\N	2020-10-12 15:41:23.75106
58741	KXI	2020-10-08	56	56.15	55.94	56.13	11356	{}	\N	2020-10-12 15:41:23.755477
58742	KXI	2020-10-09	56.38	56.7796	56.38	56.62	21187	{}	\N	2020-10-12 15:41:23.760152
58743	KXI	2020-10-12	56.74	57.185	56.74	57.185	6434	{}	\N	2020-10-12 15:41:23.764557
58744	LACQW	2020-09-28	0.4	0.4	0.36	0.3715	8680	{}	\N	2020-10-12 15:41:24.841821
58745	LACQW	2020-09-29	0.3901	0.425001	0.3901	0.41	12003	{}	\N	2020-10-12 15:41:24.847511
58746	LACQW	2020-09-30	0.4399	0.470101	0.4397	0.470101	8840	{}	\N	2020-10-12 15:41:24.853537
58747	LACQW	2020-10-01	0.49	0.555001	0.47	0.47	13584	{}	\N	2020-10-12 15:41:24.858632
58748	LACQW	2020-10-02	0.47	0.47	0.4699	0.4699	5400	{}	\N	2020-10-12 15:41:24.86415
58749	LACQW	2020-10-05	0.48	0.48	0.4	0.41	20062	{}	\N	2020-10-12 15:41:24.870313
58750	LACQW	2020-10-06	0.4101	0.4186	0.4	0.414057	65417	{}	\N	2020-10-12 15:41:24.875179
58751	LACQW	2020-10-07	0.4101	0.44	0.41	0.4293	7830	{}	\N	2020-10-12 15:41:24.879765
58752	LACQW	2020-10-08	0.45	0.475	0.39	0.44	120274	{}	\N	2020-10-12 15:41:24.884502
58753	LACQW	2020-10-09	0.4002	0.43	0.4001	0.4299	10008	{}	\N	2020-10-12 15:41:24.88896
58754	LACQW	2020-10-12	0.43	0.4401	0.43	0.4401	2900	{}	\N	2020-10-12 15:41:24.893753
58755	LANDP	2020-09-28	26.45	26.45	26.25	26.25	1410	{}	\N	2020-10-12 15:41:25.840496
58756	LANDP	2020-09-29	26.2	26.25	26.17	26.17	2496	{}	\N	2020-10-12 15:41:25.847313
58757	LANDP	2020-09-30	26.17	26.69	26.17	26.21	7955	{}	\N	2020-10-12 15:41:25.853091
58758	LANDP	2020-10-01	26.27	26.27	26.26	26.27	1185	{}	\N	2020-10-12 15:41:25.860703
58759	LANDP	2020-10-02	26.3	26.34	26.3	26.34	388	{}	\N	2020-10-12 15:41:25.866789
58760	LANDP	2020-10-05	26.48	26.5477	26.25	26.25	1899	{}	\N	2020-10-12 15:41:25.872944
58761	LANDP	2020-10-06	26.1931	26.2	26.05	26.1	5266	{}	\N	2020-10-12 15:41:25.879558
58762	LANDP	2020-10-08	26.2	26.44	25.91	25.91	9683	{}	\N	2020-10-12 15:41:25.886683
58763	LANDP	2020-10-09	26	26.42	25.99	26.0454	1518	{}	\N	2020-10-12 15:41:25.892179
58764	LANDP	2020-10-12	26.05	26.23	26.05	26.215	1051	{}	\N	2020-10-12 15:41:25.898335
58765	LBRDA	2020-09-28	139.99	143.19	139.99	142.61	84333	{}	\N	2020-10-12 15:41:27.130572
58766	LBRDA	2020-09-29	142.84	143.84	141.69	142.28	92551	{}	\N	2020-10-12 15:41:27.136226
58767	LBRDA	2020-09-30	142.63	143.34	141.25	141.85	219504	{}	\N	2020-10-12 15:41:27.142576
58768	LBRDA	2020-10-01	142.84	145.23	142.84	144.34	388873	{}	\N	2020-10-12 15:41:27.14886
58769	LBRDA	2020-10-02	142.8	143.73	140.77	142.17	279363	{}	\N	2020-10-12 15:41:27.155068
58770	LBRDA	2020-10-05	142.86	146.39	142.86	145.92	213752	{}	\N	2020-10-12 15:41:27.160558
58771	LBRDA	2020-10-06	146.305	146.305	142.56	143.56	145408	{}	\N	2020-10-12 15:41:27.166003
58772	LBRDA	2020-10-07	145.06	145.33	144.18	144.54	78522	{}	\N	2020-10-12 15:41:27.171921
58773	LBRDA	2020-10-08	145.78	146.55	144.52	146.06	69284	{}	\N	2020-10-12 15:41:27.176974
58774	LBRDA	2020-10-09	146.76	147.2666	146.2	146.22	35080	{}	\N	2020-10-12 15:41:27.182333
58775	LBRDA	2020-10-12	147.19	149.57	147.19	149.3	21598	{}	\N	2020-10-12 15:41:27.187969
58776	LD	2020-09-28	38.075	38.075	38.075	38.075	2	{}	\N	2020-10-12 15:41:28.371301
58777	LD	2020-09-29	38.1699	38.1699	38.1699	38.1699	100	{}	\N	2020-10-12 15:41:28.377811
58778	LD	2020-09-30	37.5464	37.5464	37.5464	37.5464	0	{}	\N	2020-10-12 15:41:28.383732
58779	LD	2020-10-01	36.965	37.12	36.93	36.93	201	{}	\N	2020-10-12 15:41:28.389805
58780	LD	2020-10-02	36.4048	36.4048	36.4048	36.4048	0	{}	\N	2020-10-12 15:41:28.395722
58781	LD	2020-10-05	36.075	36.075	36.075	36.075	3	{}	\N	2020-10-12 15:41:28.402841
58782	LD	2020-10-06	36.3502	36.4218	36.3502	36.4218	220	{}	\N	2020-10-12 15:41:28.410054
58783	LD	2020-10-07	36.99	36.99	36.99	36.99	0	{}	\N	2020-10-12 15:41:28.415692
58784	LD	2020-10-08	36.83	36.83	36.83	36.83	50	{}	\N	2020-10-12 15:41:28.421358
58785	LD	2020-10-09	37.425	37.425	37.425	37.425	0	{}	\N	2020-10-12 15:41:28.426381
58786	LECO	2020-09-28	90.24	92.1	90.08	91.42	300253	{}	\N	2020-10-12 15:41:29.371653
58787	LECO	2020-09-29	90.805	91.715	90.47	91.28	250095	{}	\N	2020-10-12 15:41:29.377311
58788	LECO	2020-09-30	91.84	93.02	91.38	92.04	434749	{}	\N	2020-10-12 15:41:29.382666
58789	LECO	2020-10-01	92.12	93.1	90.26	90.95	296962	{}	\N	2020-10-12 15:41:29.388041
58790	LECO	2020-10-02	89.33	92.89	89.33	91.88	254522	{}	\N	2020-10-12 15:41:29.393587
58791	LECO	2020-10-05	92.94	95.14	92.92	94.9	219013	{}	\N	2020-10-12 15:41:29.399717
58792	LECO	2020-10-06	95.39	97.015	94.61	94.81	268148	{}	\N	2020-10-12 15:41:29.407345
58793	LECO	2020-10-07	96.2	97.485	91.39	95.63	176988	{}	\N	2020-10-12 15:41:29.41367
58794	LECO	2020-10-08	96.71	96.82	93.9117	96.28	149358	{}	\N	2020-10-12 15:41:29.419746
58795	LECO	2020-10-09	96.98	97.055	95.56	95.98	160793	{}	\N	2020-10-12 15:41:29.425227
58796	LECO	2020-10-12	97.41	98.965	96.76	98.6	77586	{}	\N	2020-10-12 15:41:29.43096
58797	LII	2020-09-28	266.27	272.475	266.27	271.8	155341	{}	\N	2020-10-12 15:41:32.419294
58798	LII	2020-09-29	271.79	275.6	271.78	272.65	456278	{}	\N	2020-10-12 15:41:32.426195
58799	LII	2020-09-30	272.01	276.56	269.92	272.61	447196	{}	\N	2020-10-12 15:41:32.43182
58800	LII	2020-10-01	274.87	275.68	271.09	274.02	305656	{}	\N	2020-10-12 15:41:32.437772
58801	LII	2020-10-02	270.31	277.13	269.23	275.58	295226	{}	\N	2020-10-12 15:41:32.445961
58802	LII	2020-10-05	279.49	285.18	279.49	281.93	392823	{}	\N	2020-10-12 15:41:32.463027
58803	LII	2020-10-06	283.23	284.79	277.38	278.4	377015	{}	\N	2020-10-12 15:41:32.472784
58804	LII	2020-10-07	280.58	282.44	275.58	277.73	537100	{}	\N	2020-10-12 15:41:32.482139
58805	LII	2020-10-08	278.14	280.13	273.57	276.82	585036	{}	\N	2020-10-12 15:41:32.489578
58806	LII	2020-10-09	279.42	287.1	279.4	283.48	471922	{}	\N	2020-10-12 15:41:32.496919
58807	LII	2020-10-12	285.1	288.13	284.33	287.46	100836	{}	\N	2020-10-12 15:41:32.50402
58808	MRSN	2020-09-28	19.69	19.91	18.9301	19.59	579611	{}	\N	2020-10-12 15:41:33.583761
58809	MRSN	2020-09-29	19.33	19.33	18.23	18.78	742153	{}	\N	2020-10-12 15:41:33.59107
58810	MRSN	2020-09-30	18.79	19.44	18.11	18.62	1126270	{}	\N	2020-10-12 15:41:33.596085
58811	MRSN	2020-10-01	18.69	18.83	17.96	18.78	1210471	{}	\N	2020-10-12 15:41:33.60045
58812	MRSN	2020-10-02	18.42	18.68	17.21	17.74	867030	{}	\N	2020-10-12 15:41:33.604973
58813	MRSN	2020-10-05	17.8	18.49	17.63	18.28	616931	{}	\N	2020-10-12 15:41:33.609447
58814	MRSN	2020-10-06	18.24	19.02	18.07	18.67	591748	{}	\N	2020-10-12 15:41:33.614667
58815	MRSN	2020-10-07	18.91	19.57	18.6	19.42	886278	{}	\N	2020-10-12 15:41:33.619641
58816	MRSN	2020-10-08	19.8	19.8	18.83	19.04	312261	{}	\N	2020-10-12 15:41:33.624697
58817	MRSN	2020-10-09	19.25	19.52	18.86	19.47	397276	{}	\N	2020-10-12 15:41:33.630085
58818	MRSN	2020-10-12	19.69	19.89	19.26	19.44	75050	{}	\N	2020-10-12 15:41:33.634651
58819	MXE	2020-09-28	7.76	7.95	7.48	7.61	64882	{}	\N	2020-10-12 15:41:34.536971
58820	MXE	2020-09-29	7.66	7.66	7.6	7.63	7412	{}	\N	2020-10-12 15:41:34.545851
58821	MXE	2020-09-30	7.66	7.87	7.66	7.6999	30972	{}	\N	2020-10-12 15:41:34.551877
58822	MXE	2020-10-01	7.76	7.7601	7.67	7.7	62652	{}	\N	2020-10-12 15:41:34.559092
58823	MXE	2020-10-02	7.6	7.69	7.6	7.68	8638	{}	\N	2020-10-12 15:41:34.565252
58824	MXE	2020-10-05	7.75	7.815	7.75	7.75	16396	{}	\N	2020-10-12 15:41:34.57146
58825	MXE	2020-10-06	7.8	7.835	7.8	7.8	4417	{}	\N	2020-10-12 15:41:34.577013
58826	MXE	2020-10-07	7.9	7.98	7.9	7.98	35338	{}	\N	2020-10-12 15:41:34.583968
58827	MXE	2020-10-08	8.05	8.2	8.01	8.2	20958	{}	\N	2020-10-12 15:41:34.58979
58828	MXE	2020-10-09	8.21	8.287	8.21	8.2815	12500	{}	\N	2020-10-12 15:41:34.595741
58829	MXE	2020-10-12	8.38	8.38	8.38	8.38	2010	{}	\N	2020-10-12 15:41:34.601051
58830	MZA	2020-09-28	14.3	14.3458	14.3	14.3458	3959	{}	\N	2020-10-12 15:41:36.706826
58831	MZA	2020-09-29	14.34	14.4999	14.29	14.4999	11498	{}	\N	2020-10-12 15:41:36.715196
58832	MZA	2020-09-30	14.5	14.5	14.16	14.37	8176	{}	\N	2020-10-12 15:41:36.721167
58833	MZA	2020-10-01	14.51	14.51	14.255	14.38	3938	{}	\N	2020-10-12 15:41:36.726023
58834	MZA	2020-10-02	14.38	14.4993	14.32	14.44	7558	{}	\N	2020-10-12 15:41:36.73148
58835	MZA	2020-10-05	14.43	14.51	14.41	14.51	16889	{}	\N	2020-10-12 15:41:36.736701
58836	MZA	2020-10-06	14.4227	14.78	14.42	14.6	21104	{}	\N	2020-10-12 15:41:36.743525
58837	MZA	2020-10-07	14.78	14.78	14.549	14.58	2908	{}	\N	2020-10-12 15:41:36.750049
58838	MZA	2020-10-08	14.645	14.645	14.645	14.645	200	{}	\N	2020-10-12 15:41:36.755199
58839	MZA	2020-10-09	14.61	14.7	14.6	14.6	2712	{}	\N	2020-10-12 15:41:36.760747
58840	MZA	2020-10-12	14.65	14.65	14.6354	14.6354	1048	{}	\N	2020-10-12 15:41:36.766611
58841	NAII	2020-09-28	7.4101	8.06	7.4101	8.04	16493	{}	\N	2020-10-12 15:41:39.515442
58842	NAII	2020-09-29	7.49	7.98	7.49	7.5	7349	{}	\N	2020-10-12 15:41:39.522728
58843	NAII	2020-09-30	7.51	7.8467	7.39	7.56	16577	{}	\N	2020-10-12 15:41:39.528015
58844	NAII	2020-10-01	7.5149	7.5677	7.4	7.4	10790	{}	\N	2020-10-12 15:41:39.534415
58845	NAII	2020-10-02	7.69	7.96	7.54	7.87	9775	{}	\N	2020-10-12 15:41:39.540299
58846	NAII	2020-10-05	7.8243	7.87	7.68	7.68	5464	{}	\N	2020-10-12 15:41:39.546216
58847	NAII	2020-10-06	7.86	7.86	7.4456	7.61	13058	{}	\N	2020-10-12 15:41:39.551951
58848	NAII	2020-10-07	7.74	7.925	7.74	7.87	3915	{}	\N	2020-10-12 15:41:39.557055
58849	NAII	2020-10-08	7.52	7.91	7.52	7.86	5955	{}	\N	2020-10-12 15:41:39.562426
58850	NAII	2020-10-09	7.76	7.99	7.76	7.9	5945	{}	\N	2020-10-12 15:41:39.568418
58851	NBB	2020-09-28	22.14	22.3	22.14	22.3	23761	{}	\N	2020-10-12 15:41:41.211632
58852	NBB	2020-09-29	22.34	22.5	22.218	22.36	89100	{}	\N	2020-10-12 15:41:41.220885
58853	NBB	2020-09-30	22.4	22.47	22.36	22.4	94202	{}	\N	2020-10-12 15:41:41.228641
58854	NBB	2020-10-01	22.54	22.54	22.32	22.41	99144	{}	\N	2020-10-12 15:41:41.235168
58855	NBB	2020-10-02	22.48	22.51	22.35	22.43	67806	{}	\N	2020-10-12 15:41:41.24178
58856	NBB	2020-10-05	22.5	22.5	22.22	22.27	65194	{}	\N	2020-10-12 15:41:41.247583
58857	NBB	2020-10-06	22.23	22.288	22.01	22.15	94258	{}	\N	2020-10-12 15:41:41.255003
58858	NBB	2020-10-07	22.11	22.31	22.11	22.18	59294	{}	\N	2020-10-12 15:41:41.262654
58859	NBB	2020-10-08	22.16	22.29	22.05	22.17	74098	{}	\N	2020-10-12 15:41:41.271129
58860	NBB	2020-10-09	22.05	22.325	22	22.15	170844	{}	\N	2020-10-12 15:41:41.278534
58861	NBB	2020-10-12	22.06	22.12	22.06	22.0977	20274	{}	\N	2020-10-12 15:41:41.286177
58862	NBO	2020-09-28	11.76	11.76	11.69	11.73	27398	{}	\N	2020-10-12 15:41:42.154937
58863	NBO	2020-09-29	11.71	11.71	11.62	11.64	35816	{}	\N	2020-10-12 15:41:42.162747
58864	NBO	2020-09-30	11.63	11.655	11.63	11.64	4864	{}	\N	2020-10-12 15:41:42.172055
58865	NBO	2020-10-01	11.52	11.69	11.52	11.69	16533	{}	\N	2020-10-12 15:41:42.180897
58866	NBO	2020-10-02	11.696	11.7	11.66	11.7	2580	{}	\N	2020-10-12 15:41:42.189935
58867	NBO	2020-10-05	11.72	11.72	11.67	11.68	2263	{}	\N	2020-10-12 15:41:42.19842
58868	NBO	2020-10-06	11.88	11.88	11.61	11.68	12887	{}	\N	2020-10-12 15:41:42.206546
58869	NBO	2020-10-07	11.63	11.705	11.63	11.68	7340	{}	\N	2020-10-12 15:41:42.215077
58870	NBO	2020-10-08	12	12	11.71	11.75	21848	{}	\N	2020-10-12 15:41:42.223382
58871	NBO	2020-10-09	11.77	11.84	11.75	11.7648	2300	{}	\N	2020-10-12 15:41:42.229658
58872	NBO	2020-10-12	11.8	11.81	11.75	11.75	2671	{}	\N	2020-10-12 15:41:42.236028
58873	NDRA	2020-09-28	0.749899	0.76	0.7313	0.7415	74339	{}	\N	2020-10-12 15:41:43.795467
58874	NDRA	2020-09-29	0.7313	0.79	0.7313	0.7471	52390	{}	\N	2020-10-12 15:41:43.800875
58875	NDRA	2020-09-30	0.749	0.7499	0.7315	0.7315	34373	{}	\N	2020-10-12 15:41:43.805953
58876	NDRA	2020-10-01	0.7322	0.768899	0.7006	0.7199	81300	{}	\N	2020-10-12 15:41:43.810946
58877	NDRA	2020-10-02	0.7	0.74	0.651	0.7	86889	{}	\N	2020-10-12 15:41:43.816065
58878	NDRA	2020-10-05	0.7392	0.7392	0.667	0.7064	181820	{}	\N	2020-10-12 15:41:43.821252
58879	NDRA	2020-10-06	0.72	0.7999	0.709999	0.73	224451	{}	\N	2020-10-12 15:41:43.826832
58880	NDRA	2020-10-07	0.7422	0.77	0.6811	0.7038	114427	{}	\N	2020-10-12 15:41:43.832058
58881	NDRA	2020-10-08	0.74	0.74	0.7	0.7238	127578	{}	\N	2020-10-12 15:41:43.83753
58882	NDRA	2020-10-09	0.7209	0.732	0.702	0.7151	56702	{}	\N	2020-10-12 15:41:43.843043
58883	NDRA	2020-10-12	0.726	0.74	0.7	0.7067	84037	{}	\N	2020-10-12 15:41:43.84766
58884	NEON	2020-09-28	7.895	8.095	7.84	7.94	32134	{}	\N	2020-10-12 15:41:44.807794
58885	NEON	2020-09-29	7.98	7.98	7.62	7.66	61969	{}	\N	2020-10-12 15:41:44.813789
58886	NEON	2020-09-30	7.67	8.17	7.67	7.9	59195	{}	\N	2020-10-12 15:41:44.818968
58887	NEON	2020-10-01	8	8.14	7.51	7.84	156212	{}	\N	2020-10-12 15:41:44.824748
58888	NEON	2020-10-02	7.57	7.98	7.55	7.87	31337	{}	\N	2020-10-12 15:41:44.830766
58889	NEON	2020-10-05	7.97	8.05	7.51	7.66	69783	{}	\N	2020-10-12 15:41:44.83675
58890	NEON	2020-10-06	7.935	7.935	7.32	7.41	78757	{}	\N	2020-10-12 15:41:44.842615
58891	NEON	2020-10-07	7.56	8.04	7.56	7.9	70973	{}	\N	2020-10-12 15:41:44.84841
58892	NEON	2020-10-08	7.87	8.6999	7.87	8.59	92430	{}	\N	2020-10-12 15:41:44.853386
58893	NEON	2020-10-09	8.84	8.97	8	8.44	65579	{}	\N	2020-10-12 15:41:44.859156
58894	NEON	2020-10-12	8.5	8.8067	8.3	8.33	18976	{}	\N	2020-10-12 15:41:44.86441
58895	NEWR	2020-09-28	56.26	56.9899	55.345	56.32	346682	{}	\N	2020-10-12 15:41:45.861145
58896	NEWR	2020-09-29	56.23	57.02	55.65	56.11	719270	{}	\N	2020-10-12 15:41:45.867309
58897	NEWR	2020-09-30	56.1	57.2	55.84	56.36	692927	{}	\N	2020-10-12 15:41:45.873109
58898	NEWR	2020-10-01	57.15	58.16	56.99	57.35	1006388	{}	\N	2020-10-12 15:41:45.878875
58899	NEWR	2020-10-02	56.03	57.35	55.555	56.31	1076824	{}	\N	2020-10-12 15:41:45.886374
58900	NEWR	2020-10-05	56.41	57.66	56.03	57.48	1972348	{}	\N	2020-10-12 15:41:45.893008
58901	NEWR	2020-10-06	57.66	59.77	57.635	58.38	1110876	{}	\N	2020-10-12 15:41:45.898787
58902	NEWR	2020-10-07	59.96	62.18	59.63	61.6	1134779	{}	\N	2020-10-12 15:41:45.904215
58903	NEWR	2020-10-08	62.22	62.8	61.23	62.26	854874	{}	\N	2020-10-12 15:41:45.909786
58904	NEWR	2020-10-09	62.44	64.73	62.3736	64.39	809968	{}	\N	2020-10-12 15:41:45.914673
58905	NEWR	2020-10-12	65.15	65.65	63.84	64.09	184620	{}	\N	2020-10-12 15:41:45.919782
58906	MXL	2020-09-28	22.56	22.8	22.35	22.79	500497	{}	\N	2020-10-12 15:41:48.143673
58907	MXL	2020-09-29	22.75	23.255	22.51	22.97	805428	{}	\N	2020-10-12 15:41:48.148684
58908	MXL	2020-09-30	22.88	23.83	22.87	23.24	735000	{}	\N	2020-10-12 15:41:48.152967
58909	MXL	2020-10-01	23.54	24.04	23.2	23.92	1049252	{}	\N	2020-10-12 15:41:48.158435
58910	MXL	2020-10-02	23.15	23.52	22.745	22.75	709232	{}	\N	2020-10-12 15:41:48.164211
58911	MXL	2020-10-05	23	23.54	23	23.31	634996	{}	\N	2020-10-12 15:41:48.16995
58912	MXL	2020-10-06	23.53	24.665	23.333	23.83	1176050	{}	\N	2020-10-12 15:41:48.176003
58913	MXL	2020-10-07	24.09	24.3	23.61	24.05	918064	{}	\N	2020-10-12 15:41:48.182429
58914	MXL	2020-10-08	24.29	24.93	24.21	24.86	630766	{}	\N	2020-10-12 15:41:48.187292
58915	MXL	2020-10-09	25.09	25.88	25.09	25.86	716381	{}	\N	2020-10-12 15:41:48.191939
58916	MXL	2020-10-12	26.23	26.23	25.78	25.99	164883	{}	\N	2020-10-12 15:41:48.206501
58917	MYI	2020-09-28	13.31	13.401	13.2915	13.37	74003	{}	\N	2020-10-12 15:41:49.180846
58918	MYI	2020-09-29	13.37	13.41	13.355	13.4	141406	{}	\N	2020-10-12 15:41:49.185934
58919	MYI	2020-09-30	13.43	13.48	13.4	13.44	279876	{}	\N	2020-10-12 15:41:49.190027
58920	MYI	2020-10-01	13.48	13.51	13.44	13.46	221868	{}	\N	2020-10-12 15:41:49.194169
58921	MYI	2020-10-02	13.41	13.51	13.37	13.46	191892	{}	\N	2020-10-12 15:41:49.211056
58922	MYI	2020-10-05	13.48	13.49	13.36	13.39	187284	{}	\N	2020-10-12 15:41:49.219734
58923	MYI	2020-10-06	13.42	13.5	13.37	13.47	94042	{}	\N	2020-10-12 15:41:49.229632
58924	MYI	2020-10-07	13.5	13.56	13.47	13.47	197444	{}	\N	2020-10-12 15:41:49.237764
58925	MYI	2020-10-08	13.5	13.53	13.48	13.52	121872	{}	\N	2020-10-12 15:41:49.245139
58926	MYI	2020-10-09	13.51	13.5451	13.47	13.47	116076	{}	\N	2020-10-12 15:41:49.250488
58927	MYI	2020-10-12	13.46	13.499	13.46	13.4956	45422	{}	\N	2020-10-12 15:41:49.254789
58928	MYJ	2020-09-28	13.71	13.81	13.7	13.75	33280	{}	\N	2020-10-12 15:41:50.075006
58929	MYJ	2020-09-29	13.76	13.8	13.7575	13.76	23618	{}	\N	2020-10-12 15:41:50.081428
58930	MYJ	2020-09-30	13.8	13.8	13.74	13.79	88436	{}	\N	2020-10-12 15:41:50.086307
58931	MYJ	2020-10-01	13.81	13.81	13.74	13.79	73058	{}	\N	2020-10-12 15:41:50.091418
58932	MYJ	2020-10-02	13.8	13.83	13.79	13.8	120778	{}	\N	2020-10-12 15:41:50.098628
58933	MYJ	2020-10-05	13.9	13.9	13.74	13.78	208210	{}	\N	2020-10-12 15:41:50.103481
58934	MYJ	2020-10-06	13.78	13.81	13.73	13.73	93252	{}	\N	2020-10-12 15:41:50.107764
58935	MYJ	2020-10-07	13.73	13.82	13.72	13.73	60480	{}	\N	2020-10-12 15:41:50.11182
58936	MYJ	2020-10-08	13.74	13.81	13.72	13.73	50233	{}	\N	2020-10-12 15:41:50.116628
58937	MYJ	2020-10-09	13.72	13.78	13.7001	13.73	53633	{}	\N	2020-10-12 15:41:50.121214
58938	MYJ	2020-10-12	13.72	13.805	13.61	13.805	106186	{}	\N	2020-10-12 15:41:50.125648
58939	MYOK	2020-09-28	134.31	134.69	130.45	133.9	305759	{}	\N	2020-10-12 15:41:51.31361
58940	MYOK	2020-09-29	134	135.7824	131.85	134.13	281521	{}	\N	2020-10-12 15:41:51.319195
58941	MYOK	2020-09-30	135.32	139.22	133.56	136.33	445732	{}	\N	2020-10-12 15:41:51.323683
58942	MYOK	2020-10-01	137.8	140.77	136.83	140.11	359103	{}	\N	2020-10-12 15:41:51.328087
58943	MYOK	2020-10-02	137.2	140.9277	137.03	139.6	300147	{}	\N	2020-10-12 15:41:51.334104
58944	MYOK	2020-10-05	220.7	224	219.72	220.34	17602269	{}	\N	2020-10-12 15:41:51.338746
58945	MYOK	2020-10-06	220.31	221.49	220.25	220.96	9079095	{}	\N	2020-10-12 15:41:51.343251
58946	MYOK	2020-10-07	221.32	222	220.96	221.39	4533966	{}	\N	2020-10-12 15:41:51.349145
58947	MYOK	2020-10-08	221.5	221.73	220.5	220.67	3883771	{}	\N	2020-10-12 15:41:51.353309
58948	MYOK	2020-10-09	220.85	221.38	220.65	221.23	1755003	{}	\N	2020-10-12 15:41:51.358334
58949	MYOK	2020-10-12	221.25	221.5	220.96	220.98	739204	{}	\N	2020-10-12 15:41:51.362899
58950	MYSZ	2020-09-28	1.1	1.15	1	1.1	759798	{}	\N	2020-10-12 15:41:52.762627
58951	MYSZ	2020-09-29	1.13	1.2	1.01	1.04	845869	{}	\N	2020-10-12 15:41:52.769076
58952	MYSZ	2020-09-30	1	1.049	1	1.04	129147	{}	\N	2020-10-12 15:41:52.774562
58953	MYSZ	2020-10-01	1.03	1.06	0.9959	1.04	167631	{}	\N	2020-10-12 15:41:52.780298
58954	MYSZ	2020-10-02	1	1.04	0.99	1.03	77624	{}	\N	2020-10-12 15:41:52.786944
58955	MYSZ	2020-10-05	1.07	1.07	1.01	1.04	159469	{}	\N	2020-10-12 15:41:52.792209
58956	MYSZ	2020-10-06	1.03	1.04	1	1.02	64148	{}	\N	2020-10-12 15:41:52.797742
58957	MYSZ	2020-10-07	1.03	1.03	0.998	1.02	93169	{}	\N	2020-10-12 15:41:52.805306
58958	MYSZ	2020-10-08	1.02	1.05	1.01	1.05	102999	{}	\N	2020-10-12 15:41:52.810618
58959	MYSZ	2020-10-09	1.05	1.05	1.01	1.04	42255	{}	\N	2020-10-12 15:41:52.816925
58960	MYSZ	2020-10-12	1.04	1.11	1.02	1.06	154213	{}	\N	2020-10-12 15:41:52.822309
58961	NAC	2020-09-28	14.51	14.56	14.47	14.51	153761	{}	\N	2020-10-12 15:41:53.942409
58962	NAC	2020-09-29	14.51	14.545	14.46	14.54	217088	{}	\N	2020-10-12 15:41:53.947714
58963	NAC	2020-09-30	14.54	14.589	14.51	14.56	543588	{}	\N	2020-10-12 15:41:53.953595
58964	NAC	2020-10-01	14.6	14.6	14.5	14.56	260874	{}	\N	2020-10-12 15:41:53.958962
58965	NAC	2020-10-02	14.55	14.56	14.48	14.56	365428	{}	\N	2020-10-12 15:41:53.964905
58966	NAC	2020-10-05	14.56	14.58	14.45	14.58	324557	{}	\N	2020-10-12 15:41:53.97075
58967	NAC	2020-10-06	14.58	14.58	14.51	14.58	425212	{}	\N	2020-10-12 15:41:53.977382
58968	NAC	2020-10-07	14.6	14.65	14.51	14.59	442640	{}	\N	2020-10-12 15:41:53.982286
58969	NAC	2020-10-08	14.65	14.65	14.5696	14.65	220356	{}	\N	2020-10-12 15:41:53.988194
58970	NAC	2020-10-09	14.65	14.7	14.58	14.7	229309	{}	\N	2020-10-12 15:41:53.993475
58971	NAC	2020-10-12	14.7	14.7	14.6727	14.69	50334	{}	\N	2020-10-12 15:41:53.998212
58972	NAOV	2020-09-28	0.72	0.82	0.6604	0.759	1951792	{}	\N	2020-10-12 15:41:55.369476
58973	NAOV	2020-09-29	0.87	0.93	0.72	0.738	4015886	{}	\N	2020-10-12 15:41:55.375422
58974	NAOV	2020-09-30	0.76	0.7699	0.661	0.7089	903826	{}	\N	2020-10-12 15:41:55.381294
58975	NAOV	2020-10-01	0.71	0.72	0.6922	0.72	379088	{}	\N	2020-10-12 15:41:55.38795
58976	NAOV	2020-10-02	0.7175	0.73	0.65	0.7171	417054	{}	\N	2020-10-12 15:41:55.394221
58977	NAOV	2020-10-05	0.71	0.7131	0.6812	0.69	405812	{}	\N	2020-10-12 15:41:55.399462
58978	NAOV	2020-10-06	0.7	0.74	0.6767	0.685	476096	{}	\N	2020-10-12 15:41:55.4053
58979	NAOV	2020-10-07	0.68	0.7599	0.68	0.7324	602147	{}	\N	2020-10-12 15:41:55.411102
58980	NAOV	2020-10-08	0.754	0.77	0.6944	0.7006	687608	{}	\N	2020-10-12 15:41:55.416494
58981	NAOV	2020-10-09	0.74	0.74	0.6951	0.7101	692321	{}	\N	2020-10-12 15:41:55.422207
58982	NAOV	2020-10-12	0.7008	0.72	0.6922	0.703	148960	{}	\N	2020-10-12 15:41:55.427692
58983	NAVB	2020-09-28	2.81	2.92	2.76	2.91	118272	{}	\N	2020-10-12 15:41:57.309833
58984	NAVB	2020-09-29	2.86	2.938	2.81	2.86	75682	{}	\N	2020-10-12 15:41:57.317292
58985	NAVB	2020-09-30	2.41	2.7999	2.41	2.7	140877	{}	\N	2020-10-12 15:41:57.323387
58986	NAVB	2020-10-01	2.7	2.8	2.6201	2.64	110396	{}	\N	2020-10-12 15:41:57.329093
58987	NAVB	2020-10-02	2.7	2.7	2.62	2.7	48065	{}	\N	2020-10-12 15:41:57.335036
58988	NAVB	2020-10-05	2.7	2.8	2.68	2.8	75175	{}	\N	2020-10-12 15:41:57.342173
58989	NAVB	2020-10-06	2.66	2.78	2.66	2.72	75738	{}	\N	2020-10-12 15:41:57.349414
58990	NAVB	2020-10-07	2.78	2.83	2.72	2.76	83951	{}	\N	2020-10-12 15:41:57.35572
58991	NAVB	2020-10-08	2.82	2.9	2.796	2.84	90661	{}	\N	2020-10-12 15:41:57.361616
58992	NAVB	2020-10-09	2.91	2.92	2.77	2.78	90921	{}	\N	2020-10-12 15:41:57.367571
58993	NAVB	2020-10-12	2.77	2.81	2.77	2.8	9806	{}	\N	2020-10-12 15:41:57.373861
58994	NBLX	2020-09-28	6.77	7.27	6.77	6.98	360092	{}	\N	2020-10-12 15:41:58.600536
58995	NBLX	2020-09-29	6.96	7.375	6.79	7.29	198207	{}	\N	2020-10-12 15:41:58.606912
58996	NBLX	2020-09-30	7.31	7.61	7.03	7.32	523838	{}	\N	2020-10-12 15:41:58.611871
58997	NBLX	2020-10-01	7.4	7.57	7.03	7.46	280455	{}	\N	2020-10-12 15:41:58.617053
58998	NBLX	2020-10-02	7.19	7.72	7.1	7.56	216290	{}	\N	2020-10-12 15:41:58.621904
58999	NBLX	2020-10-05	7.75	8.45	7.71	8.33	458384	{}	\N	2020-10-12 15:41:58.628267
59000	NBLX	2020-10-06	8.47	8.6	8	8.28	474453	{}	\N	2020-10-12 15:41:58.633568
59001	NBLX	2020-10-07	8.4	8.634	8.26	8.52	247464	{}	\N	2020-10-12 15:41:58.638724
59002	NBLX	2020-10-08	8.63	9.2	8.55	8.79	385130	{}	\N	2020-10-12 15:41:58.644953
59003	NBLX	2020-10-09	8.91	9.11	8.26	8.32	651645	{}	\N	2020-10-12 15:41:58.650869
59004	NBLX	2020-10-12	8.33	8.53	8.06	8.315	70354	{}	\N	2020-10-12 15:41:58.656134
59005	NBIX	2020-09-28	98.62	98.62	96.2	97.4	517094	{}	\N	2020-10-12 15:41:59.883608
59006	NBIX	2020-09-29	97.39	98.7	96.66	97.7	726093	{}	\N	2020-10-12 15:41:59.89099
59007	NBIX	2020-09-30	97.52	98.94	95.42	96.16	790823	{}	\N	2020-10-12 15:41:59.897854
59008	NBIX	2020-10-01	96.61	97.7	94.41	97.46	1820697	{}	\N	2020-10-12 15:41:59.904929
59009	NBIX	2020-10-02	95.26	97.98	94.26	94.6	944656	{}	\N	2020-10-12 15:41:59.911616
59010	NBIX	2020-10-05	95.46	98.32	95.13	98.17	1038092	{}	\N	2020-10-12 15:41:59.917289
59011	NBIX	2020-10-06	98.54	99.8	96.66	97.03	664147	{}	\N	2020-10-12 15:41:59.923614
59012	NBIX	2020-10-07	99.54	102.48	99.54	101.86	1507455	{}	\N	2020-10-12 15:41:59.929018
59013	NBIX	2020-10-08	102.36	104.89	102.35	103.68	897525	{}	\N	2020-10-12 15:41:59.935678
59014	NBIX	2020-10-09	104.18	104.76	102.87	104.42	656215	{}	\N	2020-10-12 15:41:59.94205
59015	NBIX	2020-10-12	104.37	105.47	101.01	102.08	203230	{}	\N	2020-10-12 15:41:59.9477
59016	NC	2020-09-28	18.35	19.13	18.35	18.9	15795	{}	\N	2020-10-12 15:42:00.842975
59017	NC	2020-09-29	19.01	19.01	18.585	18.72	10512	{}	\N	2020-10-12 15:42:00.850593
59018	NC	2020-09-30	18.71	18.84	18.03	18.21	28337	{}	\N	2020-10-12 15:42:00.856762
59019	NC	2020-10-01	18.335	18.34	18.07	18.31	19104	{}	\N	2020-10-12 15:42:00.863038
59020	NC	2020-10-02	18.03	18.675	18.03	18.36	23594	{}	\N	2020-10-12 15:42:00.868515
59021	NC	2020-10-05	18.36	18.63	18.25	18.55	13948	{}	\N	2020-10-12 15:42:00.874254
59022	NC	2020-10-06	18.31	18.99	18.31	18.38	24450	{}	\N	2020-10-12 15:42:00.879963
59023	NC	2020-10-07	18.67	19.37	18.35	19.12	25210	{}	\N	2020-10-12 15:42:00.885698
59024	NC	2020-10-08	19.61	20.27	19.38	20.09	23330	{}	\N	2020-10-12 15:42:00.892979
59025	NC	2020-10-09	20.3	20.95	20.07	20.07	34315	{}	\N	2020-10-12 15:42:00.900164
59026	NC	2020-10-12	20.39	20.5099	20.11	20.265	3570	{}	\N	2020-10-12 15:42:00.906113
59027	NCBS	2020-09-28	55.51	56.435	55.27	55.9	21290	{}	\N	2020-10-12 15:42:02.467744
59028	NCBS	2020-09-29	55.56	55.7	54.56	55.38	13165	{}	\N	2020-10-12 15:42:02.474566
59029	NCBS	2020-09-30	55.4	55.54	54.52	54.61	22148	{}	\N	2020-10-12 15:42:02.481783
59030	NCBS	2020-10-01	54.76	55.93	54.34	55.85	46183	{}	\N	2020-10-12 15:42:02.488853
59031	NCBS	2020-10-02	54.005	57.61	54.005	57.3	18377	{}	\N	2020-10-12 15:42:02.496366
59032	NCBS	2020-10-05	58.08	59.34	57.95	59.11	14988	{}	\N	2020-10-12 15:42:02.502276
59033	NCBS	2020-10-06	59.94	60.72	58.1	58.44	27196	{}	\N	2020-10-12 15:42:02.508194
59034	NCBS	2020-10-07	59.22	60.2	59.15	59.42	29121	{}	\N	2020-10-12 15:42:02.514908
59035	NCBS	2020-10-08	59.37	61.2	59.37	60.32	42706	{}	\N	2020-10-12 15:42:02.520509
59036	NCBS	2020-10-09	60.33	60.74	59.154	60.36	32026	{}	\N	2020-10-12 15:42:02.527767
59037	NCBS	2020-10-12	59.98	60.72	59.7524	60.33	4024	{}	\N	2020-10-12 15:42:02.534725
59038	MYE	2020-09-28	13.18	13.67	13.18	13.35	89128	{}	\N	2020-10-12 15:42:04.632044
59039	MYE	2020-09-29	13.36	13.5	13.16	13.32	163491	{}	\N	2020-10-12 15:42:04.638891
59040	MYE	2020-09-30	13.25	13.58	13.2	13.23	279981	{}	\N	2020-10-12 15:42:04.645749
59041	MYE	2020-10-01	13.22	13.38	13.04	13.37	263454	{}	\N	2020-10-12 15:42:04.653496
59042	MYE	2020-10-02	13.15	13.87	13.15	13.65	183698	{}	\N	2020-10-12 15:42:04.658867
59043	MYE	2020-10-05	13.64	14	13.62	13.78	193318	{}	\N	2020-10-12 15:42:04.664552
59044	MYE	2020-10-06	13.96	14.31	13.83	13.94	559169	{}	\N	2020-10-12 15:42:04.670297
59045	MYE	2020-10-07	14.08	14.5	14.055	14.43	234638	{}	\N	2020-10-12 15:42:04.675297
59046	MYE	2020-10-08	14.6	14.7	14.29	14.57	135343	{}	\N	2020-10-12 15:42:04.681332
59047	MYE	2020-10-09	14.71	14.77	14.43	14.66	104677	{}	\N	2020-10-12 15:42:04.687297
59048	MYE	2020-10-12	14.65	14.83	14.51	14.8	21724	{}	\N	2020-10-12 15:42:04.693645
59049	MZZ	2020-09-28	10.7	10.7	10.52	10.55	17204	{}	\N	2020-10-12 15:42:07.87169
59050	MZZ	2020-09-29	10.52	10.71	10.52	10.62	12010	{}	\N	2020-10-12 15:42:07.877893
59051	MZZ	2020-09-30	10.34	10.5399	10.31	10.53	18033	{}	\N	2020-10-12 15:42:07.883951
59052	MZZ	2020-10-01	10.29	10.42	10.26	10.26	7352	{}	\N	2020-10-12 15:42:07.890265
59053	MZZ	2020-10-02	10.46	10.46	9.97	10.04	13086	{}	\N	2020-10-12 15:42:07.896876
59054	MZZ	2020-10-05	9.85	9.85	9.59	9.61	14950	{}	\N	2020-10-12 15:42:07.905141
59055	MZZ	2020-10-06	9.5	9.69	9.4	9.67	33839	{}	\N	2020-10-12 15:42:07.910818
59056	MZZ	2020-10-07	9.4107	9.4107	9.315	9.33	7891	{}	\N	2020-10-12 15:42:07.915951
59057	MZZ	2020-10-08	9.25	9.25	9.16	9.16	25426	{}	\N	2020-10-12 15:42:07.921924
59058	MZZ	2020-10-09	9.01	9.11	9	9.1	5547	{}	\N	2020-10-12 15:42:07.926971
59059	MZZ	2020-10-12	9.08	9.08	9	9	2798	{}	\N	2020-10-12 15:42:07.9315
59060	NATH	2020-09-28	53	53	52.52	52.83	3688	{}	\N	2020-10-12 15:42:09.242102
59061	NATH	2020-09-29	53.0333	53.0333	52.0275	52.505	2954	{}	\N	2020-10-12 15:42:09.248587
59062	NATH	2020-09-30	52.71	52.71	51.25	51.25	2289	{}	\N	2020-10-12 15:42:09.255426
59063	NATH	2020-10-01	51.8	52.5	50.85	51.46	6035	{}	\N	2020-10-12 15:42:09.262144
59064	NATH	2020-10-02	51	51.5	50.9001	51.2	3429	{}	\N	2020-10-12 15:42:09.269952
59065	NATH	2020-10-05	51	52.085	51	51.4	5828	{}	\N	2020-10-12 15:42:09.276051
59066	NATH	2020-10-06	51.5	51.5	50.55	50.55	6870	{}	\N	2020-10-12 15:42:09.282321
59067	NATH	2020-10-07	50.55	51.595	50.51	51.39	3656	{}	\N	2020-10-12 15:42:09.289508
59068	NATH	2020-10-08	51.23	51.9	51.23	51.28	2452	{}	\N	2020-10-12 15:42:09.295538
59069	NATH	2020-10-09	51.325	51.9	51.325	51.325	1365	{}	\N	2020-10-12 15:42:09.302292
59070	NATH	2020-10-12	51.07	51.275	51	51.275	3099	{}	\N	2020-10-12 15:42:09.308789
59071	NAVI	2020-09-28	8.08	8.375	8.05	8.25	2212329	{}	\N	2020-10-12 15:42:10.205331
59072	NAVI	2020-09-29	8.21	8.335	8.12	8.28	2052898	{}	\N	2020-10-12 15:42:10.217573
59073	NAVI	2020-09-30	8.36	8.77	8.34	8.45	2604210	{}	\N	2020-10-12 15:42:10.22816
59074	NAVI	2020-10-01	8.6136	8.6136	8.32	8.51	1665807	{}	\N	2020-10-12 15:42:10.23517
59075	NAVI	2020-10-02	8.34	8.72	8.23	8.67	2360418	{}	\N	2020-10-12 15:42:10.244322
59076	NAVI	2020-10-05	8.8	9.07	8.75	9.06	1442234	{}	\N	2020-10-12 15:42:10.25092
59077	NAVI	2020-10-06	9.09	9.255	8.835	8.88	1844496	{}	\N	2020-10-12 15:42:10.25738
59078	NAVI	2020-10-07	8.9864	9.18	8.89	9.01	1569739	{}	\N	2020-10-12 15:42:10.262714
59079	NAVI	2020-10-08	9.16	9.45	9.13	9.44	1330451	{}	\N	2020-10-12 15:42:10.268345
59080	NAVI	2020-10-09	9.5	9.585	9.3	9.5	1196823	{}	\N	2020-10-12 15:42:10.275074
59081	NAVI	2020-10-12	9.54	9.71	9.45	9.65	396545	{}	\N	2020-10-12 15:42:10.280453
59082	NBRV	2020-09-28	0.54	0.5437	0.52005	0.5221	868004	{}	\N	2020-10-12 15:42:11.541622
59083	NBRV	2020-09-29	0.5221	0.54	0.5155	0.54	1411088	{}	\N	2020-10-12 15:42:11.546913
59084	NBRV	2020-09-30	0.53	0.5449	0.53	0.533	1055096	{}	\N	2020-10-12 15:42:11.551869
59085	NBRV	2020-10-01	0.538	0.54	0.5175	0.53	1027000	{}	\N	2020-10-12 15:42:11.557691
59086	NBRV	2020-10-02	0.522	0.537	0.519	0.536	820354	{}	\N	2020-10-12 15:42:11.563879
59087	NBRV	2020-10-05	0.5385	0.5629	0.533	0.56	1409631	{}	\N	2020-10-12 15:42:11.571763
59088	NBRV	2020-10-06	0.56	0.565	0.54	0.5415	810825	{}	\N	2020-10-12 15:42:11.577831
59089	NBRV	2020-10-07	0.54	0.5757	0.54	0.57	1153904	{}	\N	2020-10-12 15:42:11.583783
59090	NBRV	2020-10-08	0.571	0.5949	0.57	0.585	1258071	{}	\N	2020-10-12 15:42:11.589418
59091	NBRV	2020-10-09	0.5936	0.616	0.5864	0.6	1300474	{}	\N	2020-10-12 15:42:11.59458
59092	NBRV	2020-10-12	0.617	0.6188	0.59	0.5957	517404	{}	\N	2020-10-12 15:42:11.599528
59093	NCSM	2020-09-28	0.573	0.6	0.5671	0.595	52586	{}	\N	2020-10-12 15:42:13.192409
59094	NCSM	2020-09-29	0.58	0.69	0.57	0.593	281244	{}	\N	2020-10-12 15:42:13.197608
59095	NCSM	2020-09-30	0.609	0.61	0.594	0.594	20165	{}	\N	2020-10-12 15:42:13.214214
59096	NCSM	2020-10-01	0.585	0.6	0.5575	0.57	22843	{}	\N	2020-10-12 15:42:13.223163
59097	NCSM	2020-10-02	0.579	0.598	0.56	0.561	38408	{}	\N	2020-10-12 15:42:13.232165
59098	NCSM	2020-10-05	0.59	0.595	0.56	0.56	21537	{}	\N	2020-10-12 15:42:13.241314
59099	NCSM	2020-10-06	0.58	0.597	0.58	0.58	55367	{}	\N	2020-10-12 15:42:13.247603
59100	NCSM	2020-10-07	0.589	0.622551	0.589	0.5987	68254	{}	\N	2020-10-12 15:42:13.253585
59101	NCSM	2020-10-08	0.592	0.74	0.592	0.6905	338167	{}	\N	2020-10-12 15:42:13.259618
59102	NCSM	2020-10-09	0.7195	0.74	0.65	0.6981	141551	{}	\N	2020-10-12 15:42:13.264972
59103	NCSM	2020-10-12	0.73	0.73	0.6814	0.7	65498	{}	\N	2020-10-12 15:42:13.270113
59104	NDRAW	2020-09-28	0.1	0.1	0.0802	0.0802	456	{}	\N	2020-10-12 15:42:14.547276
59105	NDRAW	2020-09-29	0.11	0.11	0.0975	0.0975	6500	{}	\N	2020-10-12 15:42:14.554244
59106	NDRAW	2020-10-01	0.1099	0.1099	0.0802	0.1009	1000	{}	\N	2020-10-12 15:42:14.560047
59107	NDRAW	2020-10-02	0.0804	0.1023	0.0711	0.1023	34322	{}	\N	2020-10-12 15:42:14.565442
59108	NDRAW	2020-10-05	0.1	0.1	0.09	0.09	5500	{}	\N	2020-10-12 15:42:14.571033
59109	NDRAW	2020-10-06	0.0914	0.0914	0.0715	0.0715	1735	{}	\N	2020-10-12 15:42:14.576657
59110	NDRAW	2020-10-07	0.0721	0.0721	0.0721	0.0721	500	{}	\N	2020-10-12 15:42:14.581718
59111	NDRAW	2020-10-08	0.0757	0.0757	0.0757	0.0757	6500	{}	\N	2020-10-12 15:42:14.586609
59112	NDRAW	2020-10-09	0.0969	0.0988	0.0969	0.0988	8710	{}	\N	2020-10-12 15:42:14.591934
59113	NDSN	2020-09-28	186.09	192.13	183.7389	189.8	172726	{}	\N	2020-10-12 15:42:15.69234
59114	NDSN	2020-09-29	189.77	192.34	189.21	191.42	196656	{}	\N	2020-10-12 15:42:15.698432
59115	NDSN	2020-09-30	192.79	195.26	190.535	191.82	217674	{}	\N	2020-10-12 15:42:15.704622
59116	NDSN	2020-10-01	191.59	193.43	191.11	191.72	143128	{}	\N	2020-10-12 15:42:15.711089
59117	NDSN	2020-10-02	188.71	191.7	187.19	190.48	118437	{}	\N	2020-10-12 15:42:15.717264
59118	NDSN	2020-10-05	192.13	196.48	192.13	196.26	126983	{}	\N	2020-10-12 15:42:15.722976
59119	NDSN	2020-10-06	197.48	202	195.84	197.74	193365	{}	\N	2020-10-12 15:42:15.729407
59120	NDSN	2020-10-07	200.38	202.5351	197.92	200.52	214451	{}	\N	2020-10-12 15:42:15.735237
59121	NDSN	2020-10-08	201.83	202.83	200.38	200.57	162299	{}	\N	2020-10-12 15:42:15.741005
59122	NDSN	2020-10-09	201.2	202.7875	200	201.19	119191	{}	\N	2020-10-12 15:42:15.748529
59123	NDSN	2020-10-12	202.48	204.455	201.7	203.8584	23431	{}	\N	2020-10-12 15:42:15.753692
59124	NEOG	2020-09-28	74.21	78.2	74.21	77.59	142498	{}	\N	2020-10-12 15:42:17.26666
59125	NEOG	2020-09-29	76.9	78.75	75.041	77.33	96449	{}	\N	2020-10-12 15:42:17.272431
59126	NEOG	2020-09-30	77.39	79.67	77.39	78.25	234848	{}	\N	2020-10-12 15:42:17.278664
59127	NEOG	2020-10-01	79.07	79.67	77.57	78.8	183237	{}	\N	2020-10-12 15:42:17.284696
59128	NEOG	2020-10-02	73.88	74.1	68.75	69.75	2081332	{}	\N	2020-10-12 15:42:17.291417
59129	NEOG	2020-10-05	70.44	72.97	68.56	69.58	1111044	{}	\N	2020-10-12 15:42:17.298083
59130	NEOG	2020-10-06	69.9198	69.94	66.22	68.3	11182426	{}	\N	2020-10-12 15:42:17.303503
59131	NEOG	2020-10-07	68.99	71.18	68.31	70.63	640686	{}	\N	2020-10-12 15:42:17.30868
59132	NEOG	2020-10-08	71.68	71.68	69.68	70.55	389359	{}	\N	2020-10-12 15:42:17.314898
59133	NEOG	2020-10-09	71.11	71.84	70.23	71.8	215062	{}	\N	2020-10-12 15:42:17.320383
59134	NEOG	2020-10-12	72.25	73.26	71.28	72.24	85497	{}	\N	2020-10-12 15:42:17.326631
59135	NEPT	2020-09-28	2.16	2.21	2.06	2.11	697390	{}	\N	2020-10-12 15:42:18.463249
59136	NEPT	2020-09-29	2.09	2.15	2.09	2.15	195869	{}	\N	2020-10-12 15:42:18.46944
59137	NEPT	2020-09-30	2.12	2.18	2.11	2.11	433966	{}	\N	2020-10-12 15:42:18.475062
59138	NEPT	2020-10-01	2.08	2.12	1.9301	1.95	1211898	{}	\N	2020-10-12 15:42:18.481294
59139	NEPT	2020-10-02	1.95	2.025	1.92	1.98	495303	{}	\N	2020-10-12 15:42:18.487194
59140	NEPT	2020-10-05	2.13	2.23	2.08	2.1	1840117	{}	\N	2020-10-12 15:42:18.492321
59141	NEPT	2020-10-06	2.12	2.23	2.08	2.2	654534	{}	\N	2020-10-12 15:42:18.499158
59142	NEPT	2020-10-07	2.09	2.34	2.09	2.28	777078	{}	\N	2020-10-12 15:42:18.504802
59143	NEPT	2020-10-08	2.29	2.38	2.29	2.34	430427	{}	\N	2020-10-12 15:42:18.50994
59144	NEPT	2020-10-09	2.35	2.49	2.35	2.47	322619	{}	\N	2020-10-12 15:42:18.515536
59145	NEPT	2020-10-12	2.5	2.55	2.4315	2.4801	99649	{}	\N	2020-10-12 15:42:18.520814
59146	NFG	2020-09-28	42.03	42.49	41.8	42.12	380672	{}	\N	2020-10-12 15:42:23.028256
59147	NFG	2020-09-29	41.67	41.7	40.39	41.15	812554	{}	\N	2020-10-12 15:42:23.035223
59148	NFG	2020-09-30	41.15	41.43	40.07	40.59	1224400	{}	\N	2020-10-12 15:42:23.040814
59149	NFG	2020-10-01	40.51	40.76	39.72	39.96	1201563	{}	\N	2020-10-12 15:42:23.046785
59150	NFG	2020-10-02	39.69	40.29	39.44	39.99	1158134	{}	\N	2020-10-12 15:42:23.052785
59151	NFG	2020-10-05	40.28	41.18	39.85	41.07	846804	{}	\N	2020-10-12 15:42:23.059018
59152	NFG	2020-10-06	41.19	41.725	40.9	41.07	831037	{}	\N	2020-10-12 15:42:23.06537
59153	NFG	2020-10-07	41.24	41.44	40.37	41.26	820082	{}	\N	2020-10-12 15:42:23.072376
59154	NFG	2020-10-08	41.57	42.04	41.13	41.97	637502	{}	\N	2020-10-12 15:42:23.078249
59155	NFG	2020-10-09	42.38	42.5	41.61	41.99	561555	{}	\N	2020-10-12 15:42:23.085569
59156	NFG	2020-10-12	42	42.31	41.71	42.25	196743	{}	\N	2020-10-12 15:42:23.090859
59157	NFLX	2020-09-28	489.1094	492	477.88	490.65	4590040	{}	\N	2020-10-12 15:42:24.050852
59158	NFLX	2020-09-29	489.51	496.2868	486.53	493.48	3449213	{}	\N	2020-10-12 15:42:24.056669
59159	NFLX	2020-09-30	492.74	504.63	489.2704	499.72	3841326	{}	\N	2020-10-12 15:42:24.060964
59160	NFLX	2020-10-01	506.03	529.55	503.6	527.51	8016405	{}	\N	2020-10-12 15:42:24.06533
59161	NFLX	2020-10-02	516.32	526.3714	502.7	503.06	5810870	{}	\N	2020-10-12 15:42:24.070572
59162	NFLX	2020-10-05	506.62	520.99	501.6997	520.65	4073305	{}	\N	2020-10-12 15:42:24.075909
59163	NFLX	2020-10-06	518.305	521.82	504.16	505.87	3967795	{}	\N	2020-10-12 15:42:24.080179
59164	NFLX	2020-10-07	518	536.49	515.815	534.66	7964657	{}	\N	2020-10-12 15:42:24.085388
59165	NFLX	2020-10-08	533.48	535	524.15	531.79	5367925	{}	\N	2020-10-12 15:42:24.090694
59166	NFLX	2020-10-09	537.8339	544.28	535	539.44	4693997	{}	\N	2020-10-12 15:42:24.095045
59167	NFLX	2020-10-12	548.81	548.81	538.24	542.815	1770948	{}	\N	2020-10-12 15:42:24.099948
59168	MYC	2020-09-28	13.87	13.92	13.84	13.88	51579	{}	\N	2020-10-12 15:42:25.505864
59169	MYC	2020-09-29	13.89	13.95	13.85	13.85	47562	{}	\N	2020-10-12 15:42:25.511806
59170	MYC	2020-09-30	13.86	13.87	13.77	13.85	215391	{}	\N	2020-10-12 15:42:25.516927
59171	MYC	2020-10-01	13.84	13.89	13.82	13.83	73753	{}	\N	2020-10-12 15:42:25.522875
59172	MYC	2020-10-02	13.83	13.85	13.78	13.85	32024	{}	\N	2020-10-12 15:42:25.529328
59173	MYC	2020-10-05	13.8	13.87	13.77	13.87	65441	{}	\N	2020-10-12 15:42:25.534993
59174	MYC	2020-10-06	13.79	13.85	13.79	13.85	34260	{}	\N	2020-10-12 15:42:25.541285
59175	MYC	2020-10-07	13.93	13.93	13.83	13.84	82606	{}	\N	2020-10-12 15:42:25.547332
59176	MYC	2020-10-08	13.93	14.02	13.85	13.98	73624	{}	\N	2020-10-12 15:42:25.55249
59177	MYC	2020-10-09	13.98	14.01	13.89	13.89	50914	{}	\N	2020-10-12 15:42:25.558665
59178	MYC	2020-10-12	14.02	14.02	13.9	13.97	5286	{}	\N	2020-10-12 15:42:25.563341
59179	MYN	2020-09-28	12.73	12.82	12.715	12.75	26009	{}	\N	2020-10-12 15:42:27.847209
59180	MYN	2020-09-29	12.8	12.8	12.685	12.705	162464	{}	\N	2020-10-12 15:42:27.851861
59181	MYN	2020-09-30	12.74	12.755	12.69	12.7	68988	{}	\N	2020-10-12 15:42:27.856439
59182	MYN	2020-10-01	12.71	12.74	12.64	12.7	153765	{}	\N	2020-10-12 15:42:27.861707
59183	MYN	2020-10-02	12.67	12.76	12.67	12.76	61603	{}	\N	2020-10-12 15:42:27.866958
59184	MYN	2020-10-05	12.8	12.8	12.72	12.77	62938	{}	\N	2020-10-12 15:42:27.873055
59185	MYN	2020-10-06	12.8	12.81	12.7001	12.76	45606	{}	\N	2020-10-12 15:42:27.880749
59186	MYN	2020-10-07	12.74	12.81	12.73	12.79	44312	{}	\N	2020-10-12 15:42:27.885626
59187	MYN	2020-10-08	12.87	12.89	12.79	12.86	150376	{}	\N	2020-10-12 15:42:27.891612
59188	MYN	2020-10-09	12.89	12.89	12.825	12.86	46996	{}	\N	2020-10-12 15:42:27.896984
59189	MYN	2020-10-12	12.83	12.86	12.78	12.8152	48236	{}	\N	2020-10-12 15:42:27.901648
59190	MYRG	2020-09-28	36.44	38.03	36.44	37.8	81585	{}	\N	2020-10-12 15:42:29.040353
59191	MYRG	2020-09-29	37.91	38.395	37.13	37.64	172429	{}	\N	2020-10-12 15:42:29.048597
59192	MYRG	2020-09-30	37.73	38.07	36.975	37.18	70470	{}	\N	2020-10-12 15:42:29.054956
59193	MYRG	2020-10-01	37.38	37.665	36.77	37.15	72243	{}	\N	2020-10-12 15:42:29.062029
59194	MYRG	2020-10-02	36.45	38.54	35.1906	38.19	62500	{}	\N	2020-10-12 15:42:29.067812
59195	MYRG	2020-10-05	38.42	40.31	38.42	40.1	71842	{}	\N	2020-10-12 15:42:29.073534
59196	MYRG	2020-10-06	40.45	41.84	40.06	40.53	95751	{}	\N	2020-10-12 15:42:29.080197
59197	MYRG	2020-10-07	41.29	42.48	40.71	42	84108	{}	\N	2020-10-12 15:42:29.086201
59198	MYRG	2020-10-08	42	42.96	41.47	42.31	50952	{}	\N	2020-10-12 15:42:29.092651
59199	MYRG	2020-10-09	42.5	43.1	42.23	43.09	76997	{}	\N	2020-10-12 15:42:29.09807
59200	MYRG	2020-10-12	42.99	43.73	42.84	43.635	21377	{}	\N	2020-10-12 15:42:29.103584
59201	NAD	2020-09-28	14.48	14.55	14.44	14.53	417859	{}	\N	2020-10-12 15:42:32.785949
59202	NAD	2020-09-29	14.53	14.571	14.51	14.55	636464	{}	\N	2020-10-12 15:42:32.792427
59203	NAD	2020-09-30	14.56	14.59	14.505	14.52	677787	{}	\N	2020-10-12 15:42:32.799032
59204	NAD	2020-10-01	14.56	14.575	14.5	14.53	839512	{}	\N	2020-10-12 15:42:32.805333
59205	NAD	2020-10-02	14.53	14.63	14.52	14.61	668909	{}	\N	2020-10-12 15:42:32.811418
59206	NAD	2020-10-05	14.58	14.6085	14.46	14.56	669432	{}	\N	2020-10-12 15:42:32.81752
59207	NAD	2020-10-06	14.55	14.58	14.5	14.54	702498	{}	\N	2020-10-12 15:42:32.822813
59208	NAD	2020-10-07	14.56	14.56	14.49	14.52	858746	{}	\N	2020-10-12 15:42:32.828916
59209	NAD	2020-10-08	14.54	14.55	14.49	14.51	375192	{}	\N	2020-10-12 15:42:32.834365
59210	NAD	2020-10-09	14.53	14.61	14.52	14.56	1130513	{}	\N	2020-10-12 15:42:32.840101
59211	NAD	2020-10-12	14.56	14.56	14.531	14.55	176617	{}	\N	2020-10-12 15:42:32.847614
59212	NAT	2020-09-28	3.57	3.58	3.42	3.46	1741904	{}	\N	2020-10-12 15:42:33.789708
59213	NAT	2020-09-29	3.46	3.515	3.375	3.46	6459843	{}	\N	2020-10-12 15:42:33.796155
59214	NAT	2020-09-30	3.45	3.585	3.43	3.49	4228900	{}	\N	2020-10-12 15:42:33.802171
59215	NAT	2020-10-01	3.48	3.57	3.45	3.53	3526378	{}	\N	2020-10-12 15:42:33.808684
59216	NAT	2020-10-02	3.43	3.6	3.42	3.54	3738255	{}	\N	2020-10-12 15:42:33.816031
59217	NAT	2020-10-05	3.56	3.72	3.5501	3.69	4170368	{}	\N	2020-10-12 15:42:33.823654
59218	NAT	2020-10-06	3.67	3.76	3.54	3.54	3763850	{}	\N	2020-10-12 15:42:33.830682
59219	NAT	2020-10-07	3.59	3.6	3.5	3.55	2768545	{}	\N	2020-10-12 15:42:33.836282
59220	NAT	2020-10-08	3.55	3.68	3.55	3.67	3035109	{}	\N	2020-10-12 15:42:33.842695
59221	NAT	2020-10-09	3.7	3.75	3.61	3.65	2943860	{}	\N	2020-10-12 15:42:33.850481
59222	NAT	2020-10-12	3.63	3.65	3.5	3.54	2154030	{}	\N	2020-10-12 15:42:33.85613
59223	NAZ	2020-09-28	14.6	14.74	14.6	14.64	10566	{}	\N	2020-10-12 15:42:36.087091
59224	NAZ	2020-09-29	14.67	14.69	14.62	14.663	23478	{}	\N	2020-10-12 15:42:36.09912
59225	NAZ	2020-09-30	14.61	14.69	14.61	14.68	6428	{}	\N	2020-10-12 15:42:36.104578
59226	NAZ	2020-10-01	14.7	14.7	14.665	14.7	11948	{}	\N	2020-10-12 15:42:36.11032
59227	NAZ	2020-10-02	14.7	14.7798	14.65	14.66	14740	{}	\N	2020-10-12 15:42:36.117297
59228	NAZ	2020-10-05	14.72	14.78	14.6874	14.78	45730	{}	\N	2020-10-12 15:42:36.123122
59229	NAZ	2020-10-06	14.75	14.8	14.705	14.75	78108	{}	\N	2020-10-12 15:42:36.1289
59230	NAZ	2020-10-07	14.79	14.79	14.64	14.78	46463	{}	\N	2020-10-12 15:42:36.136029
59231	NAZ	2020-10-08	14.77	14.85	14.75	14.85	52320	{}	\N	2020-10-12 15:42:36.142683
59232	NAZ	2020-10-09	14.85	14.88	14.81	14.87	51972	{}	\N	2020-10-12 15:42:36.149346
59233	NAZ	2020-10-12	14.81	14.866	14.81	14.866	3210	{}	\N	2020-10-12 15:42:36.155279
59234	NCLH	2020-09-28	16.92	16.95	16.01	16.68	29348490	{}	\N	2020-10-12 15:42:37.793204
59235	NCLH	2020-09-29	16.6	16.85	16.28	16.56	28599901	{}	\N	2020-10-12 15:42:37.799713
59236	NCLH	2020-09-30	17.09	18.23	16.9411	17.11	76982047	{}	\N	2020-10-12 15:42:37.806022
59237	NCLH	2020-10-01	17.4	17.405	16.74	17.34	44011017	{}	\N	2020-10-12 15:42:37.812212
59238	NCLH	2020-10-02	16.28	17.43	16.15	17.27	44541469	{}	\N	2020-10-12 15:42:37.819688
59239	NCLH	2020-10-05	17.06	17.36	16.69	16.99	30249926	{}	\N	2020-10-12 15:42:37.826386
59240	NCLH	2020-10-06	17.22	18.24	17.09	17.23	62315909	{}	\N	2020-10-12 15:42:37.831415
59241	NCLH	2020-10-07	17.75	18.16	17.46	18.01	33972014	{}	\N	2020-10-12 15:42:37.838202
59242	NCLH	2020-10-08	18.32	18.44	17.697	18.2	35164260	{}	\N	2020-10-12 15:42:37.843312
59243	NCLH	2020-10-09	18.3	18.91	17.98	18.73	47892735	{}	\N	2020-10-12 15:42:37.848841
59244	NCLH	2020-10-12	18.52	18.62	18.21	18.34	16048902	{}	\N	2020-10-12 15:42:37.857102
59245	NCTY	2020-09-28	0.66	0.735	0.65	0.704	2554481	{}	\N	2020-10-12 15:42:39.085309
59246	NCTY	2020-09-29	0.6644	0.6999	0.4567	0.52	8603037	{}	\N	2020-10-12 15:42:39.093515
59247	NCTY	2020-09-30	0.325	0.3347	0.3002	0.3168	36731673	{}	\N	2020-10-12 15:42:39.099086
59248	NCTY	2020-10-01	0.308	0.332	0.3	0.3111	13206547	{}	\N	2020-10-12 15:42:39.109396
59249	NCTY	2020-10-02	0.303	0.3275	0.303	0.32	9616104	{}	\N	2020-10-12 15:42:39.114576
59250	NCTY	2020-10-05	0.3199	0.328	0.31	0.3189	8600595	{}	\N	2020-10-12 15:42:39.12113
59251	NCTY	2020-10-06	0.3189	0.345	0.306	0.327	20411498	{}	\N	2020-10-12 15:42:39.126792
59252	NCTY	2020-10-07	0.3199	0.321999	0.3112	0.3205	6614517	{}	\N	2020-10-12 15:42:39.131668
59253	NCTY	2020-10-08	0.3209	0.3249	0.313	0.3249	4043001	{}	\N	2020-10-12 15:42:39.137872
59254	NCTY	2020-10-09	0.3199	0.3525	0.3141	0.329	16907852	{}	\N	2020-10-12 15:42:39.144199
59255	NCTY	2020-10-12	0.3134	0.3175	0.308	0.314	4186964	{}	\N	2020-10-12 15:42:39.149553
59256	NDLS	2020-09-28	7.12	7.215	6.94	7.19	150659	{}	\N	2020-10-12 15:42:40.019021
59257	NDLS	2020-09-29	7.1	7.15	6.94	7	148133	{}	\N	2020-10-12 15:42:40.027523
59258	NDLS	2020-09-30	7.05	7.3	6.82	6.87	129361	{}	\N	2020-10-12 15:42:40.032597
59259	NDLS	2020-10-01	6.9	7.06	6.66	7.06	395927	{}	\N	2020-10-12 15:42:40.039235
59260	NDLS	2020-10-02	7	7.44	6.76	7.39	276310	{}	\N	2020-10-12 15:42:40.044673
59261	NDLS	2020-10-05	7.46	7.54	7.255	7.48	218212	{}	\N	2020-10-12 15:42:40.049834
59262	NDLS	2020-10-06	7.62	7.75	7.4	7.46	260253	{}	\N	2020-10-12 15:42:40.055847
59263	NDLS	2020-10-07	7.6	7.83	7.28	7.71	316685	{}	\N	2020-10-12 15:42:40.061329
59264	NDLS	2020-10-08	7.8	7.84	7.58	7.77	166916	{}	\N	2020-10-12 15:42:40.066991
59265	NDLS	2020-10-09	7.88	8.32	7.8	8.31	283099	{}	\N	2020-10-12 15:42:40.072807
59266	NDLS	2020-10-12	8.37	8.458	8	8	85765	{}	\N	2020-10-12 15:42:40.079808
59267	NEA	2020-09-28	14.3	14.37	14.29	14.33	297647	{}	\N	2020-10-12 15:42:41.044104
59268	NEA	2020-09-29	14.35	14.39	14.32	14.33	738208	{}	\N	2020-10-12 15:42:41.052296
59269	NEA	2020-09-30	14.37	14.38	14.32	14.34	658438	{}	\N	2020-10-12 15:42:41.061373
59270	NEA	2020-10-01	14.39	14.39	14.31	14.32	1281924	{}	\N	2020-10-12 15:42:41.070851
59271	NEA	2020-10-02	14.31	14.395	14.3	14.39	721364	{}	\N	2020-10-12 15:42:41.081648
59272	NEA	2020-10-05	14.35	14.3714	14.3	14.34	838352	{}	\N	2020-10-12 15:42:41.09018
59273	NEA	2020-10-06	14.33	14.35	14.2	14.26	1221024	{}	\N	2020-10-12 15:42:41.096811
59274	NEA	2020-10-07	14.3	14.361	14.2482	14.27	883358	{}	\N	2020-10-12 15:42:41.103166
59275	NEA	2020-10-08	14.3	14.341	14.3	14.33	1043188	{}	\N	2020-10-12 15:42:41.11043
59276	NEA	2020-10-09	14.31	14.41	14.3	14.37	1061778	{}	\N	2020-10-12 15:42:41.116843
59277	NEA	2020-10-12	14.39	14.4	14.36	14.375	136754	{}	\N	2020-10-12 15:42:41.124643
59278	NEOS	2020-09-28	0.571	0.5889	0.5512	0.5569	257629	{}	\N	2020-10-12 15:42:43.919909
59279	NEOS	2020-09-29	0.5598	0.57	0.55	0.5656	103053	{}	\N	2020-10-12 15:42:43.927342
59280	NEOS	2020-09-30	0.58	0.5854	0.53	0.5303	335001	{}	\N	2020-10-12 15:42:43.934008
59281	NEOS	2020-10-01	0.54	0.56	0.4501	0.55515	871308	{}	\N	2020-10-12 15:42:43.940953
59282	NEOS	2020-10-02	0.5401	0.571099	0.5015	0.551	439215	{}	\N	2020-10-12 15:42:43.947197
59283	NEOS	2020-10-05	0.5512	0.5723	0.5201	0.5549	187790	{}	\N	2020-10-12 15:42:43.953772
59284	NEOS	2020-10-06	0.53	0.5565	0.5	0.5103	309570	{}	\N	2020-10-12 15:42:43.960608
59285	NEOS	2020-10-07	0.512	0.53	0.4975	0.5081	392475	{}	\N	2020-10-12 15:42:43.966415
59286	NEOS	2020-10-08	0.51	0.52	0.4957	0.5197	357744	{}	\N	2020-10-12 15:42:43.972398
59287	NEOS	2020-10-09	0.51	0.538	0.51	0.5281	380733	{}	\N	2020-10-12 15:42:43.979697
59288	NEOS	2020-10-12	0.5356	0.5397	0.52	0.535	59628	{}	\N	2020-10-12 15:42:43.98567
59289	NEP	2020-09-28	58.69	58.88	57.665	58.36	467166	{}	\N	2020-10-12 15:42:47.268261
59290	NEP	2020-09-29	58.3	59.37	57.87	59.03	777007	{}	\N	2020-10-12 15:42:47.274068
59291	NEP	2020-09-30	59.34	60.68	58.37	59.96	1137061	{}	\N	2020-10-12 15:42:47.281329
59292	NEP	2020-10-01	60.11	60.5498	58.47	59.79	1065368	{}	\N	2020-10-12 15:42:47.287329
59293	NEP	2020-10-02	59.01	60	58.68	58.99	706519	{}	\N	2020-10-12 15:42:47.294455
59294	NEP	2020-10-05	59.21	59.62	58.05	59.3	886665	{}	\N	2020-10-12 15:42:47.301289
59295	NEP	2020-10-06	59.48	63.79	59.36	62.75	1674163	{}	\N	2020-10-12 15:42:47.307173
59296	NEP	2020-10-07	64	66.005	63.0266	65.57	1269669	{}	\N	2020-10-12 15:42:47.315033
59297	NEP	2020-10-08	66.66	68.13	65.79	68.08	1277963	{}	\N	2020-10-12 15:42:47.320479
59298	NEP	2020-10-09	68.06	68.3877	66.8701	67.27	933823	{}	\N	2020-10-12 15:42:47.325583
59299	NEP	2020-10-12	67.75	67.805	65.7	65.77	240046	{}	\N	2020-10-12 15:42:47.332171
59300	NETE	2020-09-28	7	7.5	6.8	6.94	1117293	{}	\N	2020-10-12 15:42:48.519298
59301	NETE	2020-09-29	6.7512	7.0414	6.5399	6.88	463626	{}	\N	2020-10-12 15:42:48.52539
59302	NETE	2020-09-30	6.71	9.46	6.71	8.99	4699517	{}	\N	2020-10-12 15:42:48.533453
59303	NETE	2020-10-01	8.24	8.69	7.25	7.4401	1913247	{}	\N	2020-10-12 15:42:48.539311
59304	NETE	2020-10-02	6.98	7.75	6.85	7.695	1081663	{}	\N	2020-10-12 15:42:48.546165
59305	NETE	2020-10-05	7.81	8	7.26	7.71	711977	{}	\N	2020-10-12 15:42:48.553432
59306	NETE	2020-10-06	7.58	7.7	7.35	7.41	616532	{}	\N	2020-10-12 15:42:48.559301
59307	NETE	2020-10-07	7.27	7.67	7.27	7.45	386257	{}	\N	2020-10-12 15:42:48.566004
59308	NETE	2020-10-08	7.43	7.62	7.09	7.19	527075	{}	\N	2020-10-12 15:42:48.573104
59309	NETE	2020-10-09	7.33	7.55	7.04	7.33	407611	{}	\N	2020-10-12 15:42:48.579815
59310	NETE	2020-10-12	7.23	7.3999	6.91	7.0799	364342	{}	\N	2020-10-12 15:42:48.585922
59311	MYL	2020-09-28	14.64	14.84	14.57	14.64	4425384	{}	\N	2020-10-12 15:42:50.629574
59312	MYL	2020-09-29	14.63	14.9	14.54	14.57	3041544	{}	\N	2020-10-12 15:42:50.635559
59313	MYL	2020-09-30	14.6	14.99	14.59	14.83	3162100	{}	\N	2020-10-12 15:42:50.640499
59314	MYL	2020-10-01	14.84	15.25	14.705	14.8	4151669	{}	\N	2020-10-12 15:42:50.645347
59315	MYL	2020-10-02	14.64	15.16	14.59	14.89	3894545	{}	\N	2020-10-12 15:42:50.651115
59316	MYL	2020-10-05	15.06	15.33	14.97	15.09	6291551	{}	\N	2020-10-12 15:42:50.657297
59317	MYL	2020-10-06	15.17	15.3	14.825	14.87	3381938	{}	\N	2020-10-12 15:42:50.662527
59318	MYL	2020-10-07	14.96	15.2099	14.85	14.91	6105446	{}	\N	2020-10-12 15:42:50.668083
59319	MYL	2020-10-08	15.08	15.76	14.92	15.72	5518568	{}	\N	2020-10-12 15:42:50.673086
59320	MYL	2020-10-09	15.87	15.97	15.47	15.64	3308384	{}	\N	2020-10-12 15:42:50.677696
59321	MYL	2020-10-12	15.82	15.93	15.67	15.755	1002674	{}	\N	2020-10-12 15:42:50.68386
59322	MYY	2020-09-28	37.4539	37.4539	37.0787	37.18	7404	{}	\N	2020-10-12 15:42:51.640275
59323	MYY	2020-09-29	37.13	37.47	37.13	37.37	7571	{}	\N	2020-10-12 15:42:51.646951
59324	MYY	2020-09-30	37.0701	37.22	36.77	37.12	21667	{}	\N	2020-10-12 15:42:51.653375
59325	MYY	2020-10-01	36.95	37.015	36.6839	36.6839	13450	{}	\N	2020-10-12 15:42:51.659304
59326	MYY	2020-10-02	36.96	36.96	36.18	36.22	4465	{}	\N	2020-10-12 15:42:51.665367
59327	MYY	2020-10-05	35.771	35.771	35.45	35.45	10908	{}	\N	2020-10-12 15:42:51.67132
59328	MYY	2020-10-06	35.38	35.62	34.82	35.6	5110	{}	\N	2020-10-12 15:42:51.676517
59329	MYY	2020-10-07	35.11	35.11	34.91	34.96	4768	{}	\N	2020-10-12 15:42:51.681748
59330	MYY	2020-10-08	34.66	34.7139	34.6484	34.6484	3747	{}	\N	2020-10-12 15:42:51.687712
59331	MYY	2020-10-09	34.32	34.55	34.32	34.52	7778	{}	\N	2020-10-12 15:42:51.693197
59332	MYY	2020-10-12	34.52	34.5283	34.3	34.3041	853	{}	\N	2020-10-12 15:42:51.700922
59333	NAIL	2020-09-28	47.89	51.65	47.67	51.33	1076506	{}	\N	2020-10-12 15:42:52.76312
59334	NAIL	2020-09-29	51	51.8599	49.28	49.71	523413	{}	\N	2020-10-12 15:42:52.770365
59335	NAIL	2020-09-30	49.91	54.36	49.91	52.62	1014907	{}	\N	2020-10-12 15:42:52.776003
59336	NAIL	2020-10-01	54.2	55.6121	52.3	55.53	1026606	{}	\N	2020-10-12 15:42:52.784064
59337	NAIL	2020-10-02	51.6	57.31	51.31	56.3248	1206684	{}	\N	2020-10-12 15:42:52.790303
59338	NAIL	2020-10-05	57.55	59.05	53.7431	56.25	1372679	{}	\N	2020-10-12 15:42:52.796174
59339	NAIL	2020-10-06	56.56	57.6299	52.52	53.14	1079630	{}	\N	2020-10-12 15:42:52.80293
59340	NAIL	2020-10-07	55.12	56.71	53.56	55.07	651507	{}	\N	2020-10-12 15:42:52.809223
59341	NAIL	2020-10-08	56.5	59.52	56.3991	59.3	1019190	{}	\N	2020-10-12 15:42:52.816031
59342	NAIL	2020-10-09	60.49	61.89	59.5219	60.75	818676	{}	\N	2020-10-12 15:42:52.823253
59343	NAIL	2020-10-12	61.81	61.96	59.861	61.6196	246275	{}	\N	2020-10-12 15:42:52.829797
59344	NATR	2020-09-28	11.29	11.51	11	11.08	28022	{}	\N	2020-10-12 15:42:54.016196
59345	NATR	2020-09-29	11.07	11.42	10.8715	11.42	13516	{}	\N	2020-10-12 15:42:54.023487
59346	NATR	2020-09-30	11.36	11.8899	11.36	11.57	27170	{}	\N	2020-10-12 15:42:54.029703
59347	NATR	2020-10-01	11.94	12.05	11.6919	12.05	26597	{}	\N	2020-10-12 15:42:54.039092
59348	NATR	2020-10-02	11.99	12.04	11.786	11.99	8282	{}	\N	2020-10-12 15:42:54.048532
59349	NATR	2020-10-05	12.05	12.05	11.64	11.98	19893	{}	\N	2020-10-12 15:42:54.0551
59350	NATR	2020-10-06	12.14	12.38	11.9307	12.07	20447	{}	\N	2020-10-12 15:42:54.061127
59351	NATR	2020-10-07	12.2	12.2	12	12.125	16887	{}	\N	2020-10-12 15:42:54.066867
59352	NATR	2020-10-08	12.2	12.2	11.87	12.1	11572	{}	\N	2020-10-12 15:42:54.073793
59353	NATR	2020-10-09	12.2	12.3516	12.08	12.14	21541	{}	\N	2020-10-12 15:42:54.080108
59354	NATR	2020-10-12	12.2	12.36	12.2	12.36	1176	{}	\N	2020-10-12 15:42:54.086542
59355	NBHC	2020-09-28	25.57	26.47	25.57	26.27	118709	{}	\N	2020-10-12 15:42:54.992478
59356	NBHC	2020-09-29	26.28	26.41	25.7	26.17	182646	{}	\N	2020-10-12 15:42:54.998758
59357	NBHC	2020-09-30	26.17	26.67	26.07	26.25	233953	{}	\N	2020-10-12 15:42:55.006939
59358	NBHC	2020-10-01	26.08	26.42	25.71	26.36	235458	{}	\N	2020-10-12 15:42:55.023196
59359	NBHC	2020-10-02	26.05	27.08	26.05	26.89	241335	{}	\N	2020-10-12 15:42:55.032882
59360	NBHC	2020-10-05	27.42	28.06	27.38	28	181084	{}	\N	2020-10-12 15:42:55.043015
59361	NBHC	2020-10-06	28.52	29.775	28.19	28.86	504360	{}	\N	2020-10-12 15:42:55.051176
59362	NBHC	2020-10-07	29.17	29.6	28.57	28.91	257474	{}	\N	2020-10-12 15:42:55.059425
59363	NBHC	2020-10-08	29.3	29.8	29.15	29.72	196177	{}	\N	2020-10-12 15:42:55.065691
59364	NBHC	2020-10-09	29.82	30.27	29.66	29.81	134194	{}	\N	2020-10-12 15:42:55.074226
59365	NBHC	2020-10-12	29.57	30.12	29.57	30.01	23315	{}	\N	2020-10-12 15:42:55.083255
59366	NBTB	2020-09-28	26.93	29.62	26.93	27.17	117888	{}	\N	2020-10-12 15:42:56.349501
59367	NBTB	2020-09-29	26.96	27.07	26.43	26.77	85219	{}	\N	2020-10-12 15:42:56.357748
59368	NBTB	2020-09-30	26.9	27.43	26.64	26.82	155769	{}	\N	2020-10-12 15:42:56.364699
59369	NBTB	2020-10-01	27.06	27.06	26.29	26.73	176755	{}	\N	2020-10-12 15:42:56.371886
59370	NBTB	2020-10-02	26.41	27.41	26.4	27.27	154153	{}	\N	2020-10-12 15:42:56.37889
59371	NBTB	2020-10-05	27.35	28.08	27.35	27.8	122552	{}	\N	2020-10-12 15:42:56.384061
59372	NBTB	2020-10-06	28.12	28.85	27.94	28.19	237543	{}	\N	2020-10-12 15:42:56.390708
59373	NBTB	2020-10-07	28.46	28.93	27.915	28.03	259612	{}	\N	2020-10-12 15:42:56.396537
59374	NBTB	2020-10-08	28.34	28.87	28.24	28.8	112038	{}	\N	2020-10-12 15:42:56.403119
59375	NBTB	2020-10-09	28.96	29.06	28.43	28.51	86240	{}	\N	2020-10-12 15:42:56.410287
59376	NBTB	2020-10-12	28.45	28.95	28.3	28.92	13641	{}	\N	2020-10-12 15:42:56.415393
59377	NCMI	2020-09-28	2.66	2.72	2.58	2.61	285202	{}	\N	2020-10-12 15:42:58.085161
59378	NCMI	2020-09-29	2.631	2.655	2.5	2.53	454951	{}	\N	2020-10-12 15:42:58.091464
59379	NCMI	2020-09-30	2.52	2.73	2.5101	2.715	1254089	{}	\N	2020-10-12 15:42:58.096989
59380	NCMI	2020-10-01	2.75	2.7569	2.62	2.74	516790	{}	\N	2020-10-12 15:42:58.101955
59381	NCMI	2020-10-02	2.67	2.8532	2.6504	2.81	465916	{}	\N	2020-10-12 15:42:58.108753
59382	NCMI	2020-10-05	2.72	2.77	2.52	2.57	812245	{}	\N	2020-10-12 15:42:58.113853
59383	NCMI	2020-10-06	2.6287	2.66	2.58	2.58	428167	{}	\N	2020-10-12 15:42:58.120538
59384	NCMI	2020-10-07	2.68	2.72	2.55	2.59	711532	{}	\N	2020-10-12 15:42:58.131536
59385	NCMI	2020-10-08	2.62	2.69	2.6	2.68	367060	{}	\N	2020-10-12 15:42:58.139772
59386	NCMI	2020-10-09	2.64	2.74	2.55	2.55	510910	{}	\N	2020-10-12 15:42:58.147542
59387	NCMI	2020-10-12	2.55	2.5738	2.5	2.54	171863	{}	\N	2020-10-12 15:42:58.153014
59388	NDAQ	2020-09-28	125.86	126.33	123.98	125.37	804640	{}	\N	2020-10-12 15:42:59.177713
59389	NDAQ	2020-09-29	125.13	126.36	118.01	121.38	2611931	{}	\N	2020-10-12 15:42:59.249183
59390	NDAQ	2020-09-30	122.08	124.42	121.3	122.71	1859175	{}	\N	2020-10-12 15:42:59.268706
59391	NDAQ	2020-10-01	123.92	124.57	122.87	123.92	1040458	{}	\N	2020-10-12 15:42:59.280371
59392	NDAQ	2020-10-02	122.66	124.67	122.26	123.54	662316	{}	\N	2020-10-12 15:42:59.28953
59393	NDAQ	2020-10-05	124.19	124.99	122.7	124.87	635684	{}	\N	2020-10-12 15:42:59.29913
59394	NDAQ	2020-10-06	124.84	125.79	123.09	123.57	692603	{}	\N	2020-10-12 15:42:59.307238
59395	NDAQ	2020-10-07	123.38	124.8	122.18	122.98	830955	{}	\N	2020-10-12 15:42:59.315216
59396	NDAQ	2020-10-08	123.7352	124.89	123.32	124.6	616507	{}	\N	2020-10-12 15:42:59.326377
59397	NDAQ	2020-10-09	126.02	126.35	124.72	125.06	389387	{}	\N	2020-10-12 15:42:59.333259
59398	NDAQ	2020-10-12	125.81	126.96	125.25	126.825	115276	{}	\N	2020-10-12 15:42:59.341502
59399	NERV	2020-09-28	3.23	3.24	3.105	3.16	495097	{}	\N	2020-10-12 15:43:01.094603
59400	NERV	2020-09-29	3.18	3.26	3.13	3.21	390447	{}	\N	2020-10-12 15:43:01.100901
59401	NERV	2020-09-30	3.19	3.27	3.16	3.18	223744	{}	\N	2020-10-12 15:43:01.105827
59402	NERV	2020-10-01	3.21	3.26	3.175	3.23	300323	{}	\N	2020-10-12 15:43:01.111993
59403	NERV	2020-10-02	3.16	3.24	3.115	3.14	280806	{}	\N	2020-10-12 15:43:01.120727
59404	NERV	2020-10-05	3.15	3.4	3.15	3.39	449367	{}	\N	2020-10-12 15:43:01.163286
59405	NERV	2020-10-06	3.36	3.53	3.35	3.47	421274	{}	\N	2020-10-12 15:43:01.204708
59406	NERV	2020-10-07	3.52	3.62	3.45	3.56	569698	{}	\N	2020-10-12 15:43:01.224269
59407	NERV	2020-10-08	3.5601	3.5908	3.42	3.46	328600	{}	\N	2020-10-12 15:43:01.233466
59408	NERV	2020-10-09	3.48	3.58	3.45	3.49	291525	{}	\N	2020-10-12 15:43:01.252028
59409	NERV	2020-10-12	3.5	3.5	3.35	3.35	175341	{}	\N	2020-10-12 15:43:01.263479
59410	NEU	2020-09-28	349.61	353.54	344.015	345.73	25763	{}	\N	2020-10-12 15:43:02.158458
59411	NEU	2020-09-29	346.22	347.685	341.64	341.98	38301	{}	\N	2020-10-12 15:43:02.219756
59412	NEU	2020-09-30	343.94	346.05	337.87	342.32	85101	{}	\N	2020-10-12 15:43:02.236311
59413	NEU	2020-10-01	341.85	346.035	335.37	341.53	66023	{}	\N	2020-10-12 15:43:02.246377
59414	NEU	2020-10-02	337.68	348.1635	337.68	345.1	43351	{}	\N	2020-10-12 15:43:02.252308
59415	NEU	2020-10-05	345.21	353.44	345.21	353.37	57815	{}	\N	2020-10-12 15:43:02.258692
59416	NEU	2020-10-06	353.54	357.97	345.36	350.9	62424	{}	\N	2020-10-12 15:43:02.264969
59417	NEU	2020-10-07	351.99	356.71	349.06	352.23	106150	{}	\N	2020-10-12 15:43:02.270693
59418	NEU	2020-10-08	354.87	357.53	350.47	351.21	52316	{}	\N	2020-10-12 15:43:02.277469
59419	NEU	2020-10-09	354.04	354.04	350.45	353.17	30685	{}	\N	2020-10-12 15:43:02.283226
59420	NEU	2020-10-12	354.77	354.99	352.03	354.895	9547	{}	\N	2020-10-12 15:43:02.289243
59421	NEWA	2020-09-28	3.47	3.65	3.47	3.65	1390	{}	\N	2020-10-12 15:43:03.541456
59422	NEWA	2020-09-30	3.41	3.5954	3.31	3.4901	4244	{}	\N	2020-10-12 15:43:03.548231
59423	NEWA	2020-10-01	3.28	3.38	3.28	3.38	738	{}	\N	2020-10-12 15:43:03.553371
59424	NEWA	2020-10-02	3.42	3.4699	3.26	3.4699	2359	{}	\N	2020-10-12 15:43:03.558631
59425	NEWA	2020-10-05	3.45	3.499	3.45	3.4791	1567	{}	\N	2020-10-12 15:43:03.564993
59426	NEWA	2020-10-06	3.45	3.5	3.45	3.5	1855	{}	\N	2020-10-12 15:43:03.570419
59427	NEWA	2020-10-07	3.4973	3.5	3.46	3.46	4937	{}	\N	2020-10-12 15:43:03.575713
59428	NEWA	2020-10-08	3.5	3.54	3.4981	3.54	7065	{}	\N	2020-10-12 15:43:03.581446
59429	NEWA	2020-10-09	3.5	3.5	3.4825	3.49	2536	{}	\N	2020-10-12 15:43:03.585816
59430	NEWA	2020-10-12	3.45	3.49	3.45	3.475	1215	{}	\N	2020-10-12 15:43:03.591578
59431	NFBK	2020-09-28	9.07	9.28	9.07	9.19	140430	{}	\N	2020-10-12 15:43:04.519222
59432	NFBK	2020-09-29	9.15	9.18	8.9	9.1	103384	{}	\N	2020-10-12 15:43:04.52456
59433	NFBK	2020-09-30	9.13	9.28	8.97	9.12	201299	{}	\N	2020-10-12 15:43:04.530281
59434	NFBK	2020-10-01	9.64	9.7245	9.3	9.45	220871	{}	\N	2020-10-12 15:43:04.535183
59435	NFBK	2020-10-02	9.33	9.8	9.33	9.77	151645	{}	\N	2020-10-12 15:43:04.540045
59436	NFBK	2020-10-05	9.98	10.03	9.82	10.01	195522	{}	\N	2020-10-12 15:43:04.547203
59437	NFBK	2020-10-06	10.08	10.41	10	10.05	195183	{}	\N	2020-10-12 15:43:04.552241
59438	NFBK	2020-10-07	10.2	10.36	10.1	10.24	160447	{}	\N	2020-10-12 15:43:04.556465
59439	NFBK	2020-10-08	10.34	10.41	10.22	10.33	123936	{}	\N	2020-10-12 15:43:04.560478
59440	NFBK	2020-10-09	10.35	10.45	10.23	10.29	104658	{}	\N	2020-10-12 15:43:04.565779
59441	NFBK	2020-10-12	10.27	10.45	10.25	10.39	22924	{}	\N	2020-10-12 15:43:04.570111
59442	MYOV	2020-09-28	21.01	21.74	20.39	21.09	797451	{}	\N	2020-10-12 15:43:06.250167
59443	MYOV	2020-09-29	16.18	17.21	15.57	15.65	22578779	{}	\N	2020-10-12 15:43:06.256013
59444	MYOV	2020-09-30	16	16.4	14	14.05	12960336	{}	\N	2020-10-12 15:43:06.260389
59445	MYOV	2020-10-01	14.19	14.59	13.71	13.98	6729790	{}	\N	2020-10-12 15:43:06.266287
59446	MYOV	2020-10-02	13.6	14.36	13.42	14	4485299	{}	\N	2020-10-12 15:43:06.271303
59447	MYOV	2020-10-05	14.05	14.97	13.66	14.46	3640318	{}	\N	2020-10-12 15:43:06.276342
59448	MYOV	2020-10-06	14.5	15.27	14.33	14.62	2522674	{}	\N	2020-10-12 15:43:06.28222
59449	MYOV	2020-10-07	14.77	14.99	14.3	14.89	2225129	{}	\N	2020-10-12 15:43:06.287087
59450	MYOV	2020-10-08	15.05	15.63	14.923	15.18	2026311	{}	\N	2020-10-12 15:43:06.292422
59451	MYOV	2020-10-09	15.34	15.52	14.9	15.41	1790629	{}	\N	2020-10-12 15:43:06.299187
59452	MYOV	2020-10-12	15.45	15.82	15.26	15.64	790704	{}	\N	2020-10-12 15:43:06.304195
59453	NAK	2020-09-28	1	1.03	0.97	1.01	7148640	{}	\N	2020-10-12 15:43:07.219929
59454	NAK	2020-09-29	0.983	1.01	0.9747	0.9982	4330853	{}	\N	2020-10-12 15:43:07.241333
59455	NAK	2020-09-30	0.9856	0.9999	0.9728	0.9804	3992662	{}	\N	2020-10-12 15:43:07.252045
59456	NAK	2020-10-01	1.11	1.11	1.02	1.05	18764821	{}	\N	2020-10-12 15:43:07.259713
59457	NAK	2020-10-02	1	1.07	1	1.05	7886011	{}	\N	2020-10-12 15:43:07.267579
59458	NAK	2020-10-05	1.05	1.26	1.05	1.19	25593554	{}	\N	2020-10-12 15:43:07.272983
59459	NAK	2020-10-06	1.23	1.24	1.1	1.12	16017813	{}	\N	2020-10-12 15:43:07.278076
59460	NAK	2020-10-07	1.15	1.17	1.12	1.15	9607036	{}	\N	2020-10-12 15:43:07.284437
59461	NAK	2020-10-08	1.17	1.2	1.14	1.15	5638756	{}	\N	2020-10-12 15:43:07.290368
59462	NAK	2020-10-09	1.17	1.19	1.15	1.18	5789620	{}	\N	2020-10-12 15:43:07.303959
59463	NAK	2020-10-12	1.19	1.24	1.171	1.23	5313458	{}	\N	2020-10-12 15:43:07.3124
59464	NATI	2020-09-28	35.73	35.92	35.29	35.38	454567	{}	\N	2020-10-12 15:43:09.702089
59465	NATI	2020-09-29	35.33	35.91	35.22	35.59	505525	{}	\N	2020-10-12 15:43:09.713757
59466	NATI	2020-09-30	35.67	35.955	35.25	35.7	467693	{}	\N	2020-10-12 15:43:09.72038
59467	NATI	2020-10-01	35.8	36.27	35.5	35.96	406475	{}	\N	2020-10-12 15:43:09.726372
59468	NATI	2020-10-02	35.44	35.94	35.09	35.74	517741	{}	\N	2020-10-12 15:43:09.731336
59469	NATI	2020-10-05	35.82	36.54	35.76	36.19	2304680	{}	\N	2020-10-12 15:43:09.736487
59470	NATI	2020-10-06	36.48	36.88	35.753	36.04	669242	{}	\N	2020-10-12 15:43:09.741409
59471	NATI	2020-10-07	36.29	36.94	36.29	36.42	1061172	{}	\N	2020-10-12 15:43:09.745908
59472	NATI	2020-10-08	35.96	36.41	35.52	36.045	634617	{}	\N	2020-10-12 15:43:09.751517
59473	NATI	2020-10-09	36.32	36.89	36.21	36.4	849950	{}	\N	2020-10-12 15:43:09.758757
59474	NATI	2020-10-12	36.74	37.035	36.635	36.97	201167	{}	\N	2020-10-12 15:43:09.763371
59475	NBY	2020-09-28	0.721	0.74	0.72	0.7257	225128	{}	\N	2020-10-12 15:43:11.435543
59476	NBY	2020-09-29	0.71	0.805	0.71	0.7204	1500429	{}	\N	2020-10-12 15:43:11.443171
59477	NBY	2020-09-30	0.73	0.75	0.72	0.73	591604	{}	\N	2020-10-12 15:43:11.448505
59478	NBY	2020-10-01	0.7398	0.7398	0.72	0.7244	421415	{}	\N	2020-10-12 15:43:11.455192
59479	NBY	2020-10-02	0.824	0.87	0.7324	0.8045	12011623	{}	\N	2020-10-12 15:43:11.462482
59480	NBY	2020-10-05	0.77	0.79	0.753	0.7645	1832517	{}	\N	2020-10-12 15:43:11.468772
59481	NBY	2020-10-06	0.742	0.759	0.728	0.7378	862137	{}	\N	2020-10-12 15:43:11.475502
59482	NBY	2020-10-07	0.74	0.77	0.728	0.743049	663284	{}	\N	2020-10-12 15:43:11.480731
59483	NBY	2020-10-08	0.75	0.82	0.736	0.7601	1919568	{}	\N	2020-10-12 15:43:11.485886
59484	NBY	2020-10-09	0.7534	0.81	0.7412	0.805	2951904	{}	\N	2020-10-12 15:43:11.491389
59485	NBY	2020-10-12	0.79	0.84	0.7868	0.8	2324025	{}	\N	2020-10-12 15:43:11.497235
59486	NCR	2020-09-28	19.95	20.215	19.57	20.03	1106319	{}	\N	2020-10-12 15:43:12.608645
59487	NCR	2020-09-29	21.34	22.08	21.19	22.01	4934436	{}	\N	2020-10-12 15:43:12.614416
59488	NCR	2020-09-30	21.99	22.455	21.96	22.14	2833405	{}	\N	2020-10-12 15:43:12.61903
59489	NCR	2020-10-01	22.31	22.605	21.89	22.13	2597363	{}	\N	2020-10-12 15:43:12.625151
59490	NCR	2020-10-02	21.43	22.335	21.32	22.25	1777188	{}	\N	2020-10-12 15:43:12.631317
59491	NCR	2020-10-05	22.49	23.0753	22.34	22.87	5571569	{}	\N	2020-10-12 15:43:12.638312
59492	NCR	2020-10-06	23.11	23.42	22.62	22.65	3432757	{}	\N	2020-10-12 15:43:12.6442
59493	NCR	2020-10-07	23	23.36	22.79	23.19	3505666	{}	\N	2020-10-12 15:43:12.649468
59494	NCR	2020-10-08	23.49	24.27	23.412	24.21	2302556	{}	\N	2020-10-12 15:43:12.654324
59495	NCR	2020-10-09	24.32	24.5	23.69	23.81	2735486	{}	\N	2020-10-12 15:43:12.6603
59496	NCR	2020-10-12	23.85	23.91	23.22	23.42	435434	{}	\N	2020-10-12 15:43:12.665414
59497	NESR	2020-09-28	6.55	7.05	6.26	6.63	103041	{}	\N	2020-10-12 15:43:14.067774
59498	NESR	2020-09-29	6.61	6.61	6.325	6.42	55459	{}	\N	2020-10-12 15:43:14.075553
59499	NESR	2020-09-30	6.44	6.535	6.22	6.38	168527	{}	\N	2020-10-12 15:43:14.08374
59500	NESR	2020-10-01	6.37	6.375	6.1	6.2	150808	{}	\N	2020-10-12 15:43:14.093372
59501	NESR	2020-10-02	6.1	6.49	5.93	6.37	135463	{}	\N	2020-10-12 15:43:14.101324
59502	NESR	2020-10-05	6.4	6.71	6.2	6.64	65101	{}	\N	2020-10-12 15:43:14.1085
59503	NESR	2020-10-06	6.74	6.93	6.63	6.71	118811	{}	\N	2020-10-12 15:43:14.114657
59504	NESR	2020-10-07	6.705	6.95	6.66	6.94	50703	{}	\N	2020-10-12 15:43:14.120029
59505	NESR	2020-10-08	6.98	7.6	6.9	7.49	175087	{}	\N	2020-10-12 15:43:14.126284
59506	NESR	2020-10-09	7.57	7.57	7.18	7.29	45988	{}	\N	2020-10-12 15:43:14.131491
59507	NESR	2020-10-12	7.34	7.34	7.095	7.18	6638	{}	\N	2020-10-12 15:43:14.1381
59508	NEWT	2020-09-28	18.555	19.12	18.555	18.96	156400	{}	\N	2020-10-12 15:43:15.372214
59509	NEWT	2020-09-29	18.96	18.96	18.41	18.45	54823	{}	\N	2020-10-12 15:43:15.379307
59510	NEWT	2020-09-30	18.6	19.09	18.52	18.57	139141	{}	\N	2020-10-12 15:43:15.384572
59511	NEWT	2020-10-01	18.58	18.94	18.54	18.86	123407	{}	\N	2020-10-12 15:43:15.391222
59512	NEWT	2020-10-02	18.53	19.35	18.46	19.34	88420	{}	\N	2020-10-12 15:43:15.399023
59513	NEWT	2020-10-05	19.36	19.47	19.04	19.1	83958	{}	\N	2020-10-12 15:43:15.403609
59514	NEWT	2020-10-06	19.11	19.165	18.6101	18.72	58438	{}	\N	2020-10-12 15:43:15.409049
59515	NEWT	2020-10-07	18.9	19.22	18.71	18.98	75037	{}	\N	2020-10-12 15:43:15.413731
59516	NEWT	2020-10-08	19.06	19.22	18.65	18.79	73936	{}	\N	2020-10-12 15:43:15.418303
59517	NEWT	2020-10-09	18.78	18.97	18.69	18.75	66200	{}	\N	2020-10-12 15:43:15.424414
59518	NEWT	2020-10-12	18.75	18.86	18.3	18.3	57488	{}	\N	2020-10-12 15:43:15.429679
59519	NFRA	2020-09-28	50.01	50.13	49.91	49.99	62901	{}	\N	2020-10-12 15:43:16.91575
59520	NFRA	2020-09-29	50.08	50.17	49.775	49.89	57799	{}	\N	2020-10-12 15:43:16.92195
59521	NFRA	2020-09-30	50.06	50.27	49.72	49.86	36993	{}	\N	2020-10-12 15:43:16.928494
59522	NFRA	2020-10-01	50.06	50.19	49.8337	50.07	39708	{}	\N	2020-10-12 15:43:16.934669
59523	NFRA	2020-10-02	49.87	50.27	49.86	50.14	96256	{}	\N	2020-10-12 15:43:16.940443
59524	NFRA	2020-10-05	50.45	50.75	50.35	50.725	27856	{}	\N	2020-10-12 15:43:16.946897
59525	NFRA	2020-10-06	50.83	51.03	50.34	50.36	68882	{}	\N	2020-10-12 15:43:16.95158
59526	NFRA	2020-10-07	50.85	51.09	50.78	50.96	31449	{}	\N	2020-10-12 15:43:16.956046
59527	NFRA	2020-10-08	51.15	51.41	51.12	51.4	40711	{}	\N	2020-10-12 15:43:16.961622
59528	NFRA	2020-10-09	51.53	51.5634	51.32	51.43	71080	{}	\N	2020-10-12 15:43:16.967403
59529	NFRA	2020-10-12	51.65	51.75	51.54	51.73	8080	{}	\N	2020-10-12 15:43:16.97365
59530	NGVC	2020-09-28	9.74	9.77	9.36	9.59	123384	{}	\N	2020-10-12 15:43:18.469309
59531	NGVC	2020-09-29	9.43	9.8526	9.43	9.64	361294	{}	\N	2020-10-12 15:43:18.475962
59532	NGVC	2020-09-30	9.63	10.1	9.5605	9.86	309364	{}	\N	2020-10-12 15:43:18.48244
59533	NGVC	2020-10-01	9.94	10.37	9.8	10.2	360859	{}	\N	2020-10-12 15:43:18.487821
59534	NGVC	2020-10-02	10.1	10.4592	10.02	10.4	309548	{}	\N	2020-10-12 15:43:18.493213
59535	NGVC	2020-10-05	10.43	10.89	10.43	10.87	223523	{}	\N	2020-10-12 15:43:18.499407
59536	NGVC	2020-10-06	10.91	11.18	10.87	11.1	356361	{}	\N	2020-10-12 15:43:18.504672
59537	NGVC	2020-10-07	11.11	11.2061	10.67	10.74	257138	{}	\N	2020-10-12 15:43:18.51049
59538	NGVC	2020-10-08	10.79	10.8277	10.55	10.77	187388	{}	\N	2020-10-12 15:43:18.517447
59539	NGVC	2020-10-09	11.37	12	11.14	11.82	341822	{}	\N	2020-10-12 15:43:18.522886
59540	NGVC	2020-10-12	11.92	12.06	11.78	11.95	49564	{}	\N	2020-10-12 15:43:18.528143
59541	NGVT	2020-09-28	49.4	50.63	49.4	49.7	213743	{}	\N	2020-10-12 15:43:19.649972
59542	NGVT	2020-09-29	49.89	50.975	48.69	49.52	267769	{}	\N	2020-10-12 15:43:19.655188
59543	NGVT	2020-09-30	49.59	50.795	49.3	49.44	431933	{}	\N	2020-10-12 15:43:19.660157
59544	NGVT	2020-10-01	49.34	50.09	48.9	49.95	671078	{}	\N	2020-10-12 15:43:19.665803
59545	NGVT	2020-10-02	48.75	53.56	48.56	52.93	732384	{}	\N	2020-10-12 15:43:19.672102
59546	NGVT	2020-10-05	53.37	55.55	53.37	55.2	394559	{}	\N	2020-10-12 15:43:19.678472
59547	NGVT	2020-10-06	56.03	57.43	54.81	55.45	440004	{}	\N	2020-10-12 15:43:19.685304
59548	NGVT	2020-10-07	56.29	57.87	56.17	57.09	476764	{}	\N	2020-10-12 15:43:19.692294
59549	NGVT	2020-10-08	57.99	58.85	57.68	58.31	324564	{}	\N	2020-10-12 15:43:19.697835
59550	NGVT	2020-10-09	59.44	59.82	58.37	59.37	393769	{}	\N	2020-10-12 15:43:19.703431
59551	NGVT	2020-10-12	59.44	59.89	59.17	59.49	57883	{}	\N	2020-10-12 15:43:19.708667
59552	NHS	2020-09-28	11.13	11.17	11.09	11.13	39633	{}	\N	2020-10-12 15:43:20.591184
59553	NHS	2020-09-29	11.15	11.16	11.09	11.09	49442	{}	\N	2020-10-12 15:43:20.597657
59554	NHS	2020-09-30	11.13	11.27	11.03	11.09	295774	{}	\N	2020-10-12 15:43:20.603427
59555	NHS	2020-10-01	11.09	11.23	11.09	11.21	37513	{}	\N	2020-10-12 15:43:20.608018
59556	NHS	2020-10-02	11.19	11.23	11.1501	11.175	23346	{}	\N	2020-10-12 15:43:20.612623
59557	NHS	2020-10-05	11.18	11.27	11.07	11.27	47185	{}	\N	2020-10-12 15:43:20.6179
59558	NHS	2020-10-06	11.17	11.34	11.12	11.2	49905	{}	\N	2020-10-12 15:43:20.623059
59559	NHS	2020-10-07	11.25	11.33	11.25	11.27	16014	{}	\N	2020-10-12 15:43:20.628062
59560	NHS	2020-10-08	11.32	11.4	11.2747	11.34	25726	{}	\N	2020-10-12 15:43:20.633545
59561	NHS	2020-10-09	11.4	11.5	11.375	11.42	25032	{}	\N	2020-10-12 15:43:20.639882
59562	NHS	2020-10-12	11.5	11.522	11.4503	11.4503	14759	{}	\N	2020-10-12 15:43:20.64521
59563	NANR	2020-09-28	28.895	28.92	28.69	28.69	10956	{}	\N	2020-10-12 15:43:23.329081
59564	NANR	2020-09-29	28.63	28.68	28.25	28.4	9717	{}	\N	2020-10-12 15:43:23.335898
59565	NANR	2020-09-30	28.53	28.63	28.42	28.42	19800	{}	\N	2020-10-12 15:43:23.341618
59566	NANR	2020-10-01	28.215	28.2583	28.05	28.12	28448	{}	\N	2020-10-12 15:43:23.347547
59567	NANR	2020-10-02	27.57	28.354	27.57	28.2	19968	{}	\N	2020-10-12 15:43:23.353731
59568	NANR	2020-10-05	28.6	28.88	28.6	28.87	16253	{}	\N	2020-10-12 15:43:23.359735
59569	NANR	2020-10-06	28.82	29.1	28.21	28.2186	18814	{}	\N	2020-10-12 15:43:23.365351
59570	NANR	2020-10-07	28.52	28.7	28.45	28.62	98048	{}	\N	2020-10-12 15:43:23.371387
59571	NANR	2020-10-08	28.87	29.38	28.87	29.38	14311	{}	\N	2020-10-12 15:43:23.377295
59572	NANR	2020-10-09	29.5	29.64	29.37	29.58	16811	{}	\N	2020-10-12 15:43:23.382823
59573	NANR	2020-10-12	29.43	29.575	29.39	29.575	31211	{}	\N	2020-10-12 15:43:23.389192
59574	NBEV	2020-09-28	1.65	1.73	1.6427	1.69	958109	{}	\N	2020-10-12 15:43:24.639901
59575	NBEV	2020-09-29	1.73	1.83	1.69	1.72	2113119	{}	\N	2020-10-12 15:43:24.645285
59576	NBEV	2020-09-30	1.72	1.74	1.655	1.73	1285546	{}	\N	2020-10-12 15:43:24.652408
59577	NBEV	2020-10-01	1.73	1.75	1.6601	1.73	1116117	{}	\N	2020-10-12 15:43:24.657822
59578	NBEV	2020-10-02	1.73	1.8093	1.68	1.75	1644106	{}	\N	2020-10-12 15:43:24.662894
59579	NBEV	2020-10-05	1.72	1.84	1.72	1.82	1517918	{}	\N	2020-10-12 15:43:24.668738
59580	NBEV	2020-10-06	1.8033	1.88	1.74	1.76	1588376	{}	\N	2020-10-12 15:43:24.674596
59581	NBEV	2020-10-07	1.82	1.9	1.7939	1.89	1126343	{}	\N	2020-10-12 15:43:24.679125
59582	NBEV	2020-10-08	1.99	2	1.9	1.91	1484449	{}	\N	2020-10-12 15:43:24.686171
59583	NBEV	2020-10-09	2.03	2.25	1.98	2.15	4379374	{}	\N	2020-10-12 15:43:24.691265
59584	NBEV	2020-10-12	2.1	2.24	2.1	2.205	954964	{}	\N	2020-10-12 15:43:24.696204
59585	NBW	2020-09-28	13	13.07	13	13.03	16003	{}	\N	2020-10-12 15:43:25.833976
59586	NBW	2020-09-29	12.99	13.0045	12.965	12.9697	10267	{}	\N	2020-10-12 15:43:25.842412
59587	NBW	2020-09-30	13	13	12.94	12.94	4799	{}	\N	2020-10-12 15:43:25.847245
59588	NBW	2020-10-01	12.95	12.98	12.93	12.95	15221	{}	\N	2020-10-12 15:43:25.85553
59589	NBW	2020-10-02	13.01	13.01	12.92	12.98	7549	{}	\N	2020-10-12 15:43:25.862979
59590	NBW	2020-10-05	13.05	13.07	13.05	13.07	47184	{}	\N	2020-10-12 15:43:25.869036
59591	NBW	2020-10-06	13.07	13.1	13.07	13.09	4761	{}	\N	2020-10-12 15:43:25.874124
59592	NBW	2020-10-07	13.11	13.14	13.1	13.1	18024	{}	\N	2020-10-12 15:43:25.878041
59593	NBW	2020-10-08	13.13	13.15	13.13	13.15	3865	{}	\N	2020-10-12 15:43:25.882486
59594	NBW	2020-10-09	13.15	13.2	13.14	13.185	11044	{}	\N	2020-10-12 15:43:25.890479
59595	NBW	2020-10-12	13.24	13.24	13.15	13.15	1524	{}	\N	2020-10-12 15:43:25.897521
59596	NDP	2020-09-28	10.26	10.4	10.15	10.3	26240	{}	\N	2020-10-12 15:43:27.251547
59597	NDP	2020-09-29	10.27	10.27	9.92	10.09	27244	{}	\N	2020-10-12 15:43:27.260535
59598	NDP	2020-09-30	10.08	10.11	9.84	9.94	36860	{}	\N	2020-10-12 15:43:27.270521
59599	NDP	2020-10-01	9.9	9.9	9.58	9.71	39802	{}	\N	2020-10-12 15:43:27.283011
59600	NDP	2020-10-02	9.5	9.72	9.45	9.68	33918	{}	\N	2020-10-12 15:43:27.293046
59601	NDP	2020-10-05	9.8	10.15	9.775	10.15	36030	{}	\N	2020-10-12 15:43:27.301517
59602	NDP	2020-10-06	10.26	10.2711	10.03	10.03	74248	{}	\N	2020-10-12 15:43:27.3106
59603	NDP	2020-10-07	10.09	10.235	10.03	10.235	12794	{}	\N	2020-10-12 15:43:27.317477
59604	NDP	2020-10-08	10.37	10.71	10.3257	10.7	32902	{}	\N	2020-10-12 15:43:27.324668
59605	NDP	2020-10-09	10.76	10.76	10.54	10.54	17656	{}	\N	2020-10-12 15:43:27.331143
59606	NDP	2020-10-12	10.54	10.56	10.53	10.56	7182	{}	\N	2020-10-12 15:43:27.33704
59607	NEM	2020-09-28	62	62.83	61.305	62.13	5448553	{}	\N	2020-10-12 15:43:28.349632
59608	NEM	2020-09-29	62.5	63.45	62.39	62.77	8407946	{}	\N	2020-10-12 15:43:28.357789
59609	NEM	2020-09-30	62.49	63.9	61.85	63.45	14268467	{}	\N	2020-10-12 15:43:28.363639
59610	NEM	2020-10-01	64	64.1	62.66	62.83	12750239	{}	\N	2020-10-12 15:43:28.369634
59611	NEM	2020-10-02	62.64	63.4	62.15	62.29	9103552	{}	\N	2020-10-12 15:43:28.376989
59612	NEM	2020-10-05	62.3	63.95	62.27	63.13	8875056	{}	\N	2020-10-12 15:43:28.383518
59613	NEM	2020-10-06	63.3	63.73	60.77	60.82	13858519	{}	\N	2020-10-12 15:43:28.390721
59614	NEM	2020-10-07	61.36	61.78	60.89	61.17	8295331	{}	\N	2020-10-12 15:43:28.398897
59615	NEM	2020-10-08	61.47	62.16	61.21	61.72	7931208	{}	\N	2020-10-12 15:43:28.404793
59616	NEM	2020-10-09	62.72	62.92	61.85	62.82	9517795	{}	\N	2020-10-12 15:43:28.411302
59617	NEM	2020-10-12	62.71	63.3	62.5	62.9	3080940	{}	\N	2020-10-12 15:43:28.416522
59618	NES	2020-09-28	2.73	2.7776	2.64	2.65	5952	{}	\N	2020-10-12 15:43:29.275749
59619	NES	2020-09-29	2.66	2.7	2.4256	2.5	14734	{}	\N	2020-10-12 15:43:29.281936
59620	NES	2020-09-30	2.27	2.6	2.27	2.32	4609	{}	\N	2020-10-12 15:43:29.287678
59621	NES	2020-10-01	2.29	2.38	2.29	2.37	872	{}	\N	2020-10-12 15:43:29.295191
59622	NES	2020-10-02	2.3234	2.49	2.3234	2.49	5459	{}	\N	2020-10-12 15:43:29.301429
59623	NES	2020-10-05	2.37	2.54	2.37	2.47	6027	{}	\N	2020-10-12 15:43:29.310064
59624	NES	2020-10-06	2.426	2.45	2.29	2.31	6036	{}	\N	2020-10-12 15:43:29.318993
59625	NES	2020-10-07	2.23	2.353	2.17	2.17	1862	{}	\N	2020-10-12 15:43:29.32902
59626	NES	2020-10-08	2.195	2.33	2.19	2.3	6838	{}	\N	2020-10-12 15:43:29.335943
59627	NES	2020-10-09	2.4297	2.49	2.335	2.39	5275	{}	\N	2020-10-12 15:43:29.342709
59628	NES	2020-10-12	2.39	2.49	2.39	2.41	1927	{}	\N	2020-10-12 15:43:29.348911
59629	NEXT	2020-09-28	3.22	3.3	2.8	2.9	533115	{}	\N	2020-10-12 15:43:30.386787
59630	NEXT	2020-09-29	2.9	2.97	2.7	2.89	647259	{}	\N	2020-10-12 15:43:30.393933
59631	NEXT	2020-09-30	2.9	3.38	2.89	2.98	945459	{}	\N	2020-10-12 15:43:30.399755
59632	NEXT	2020-10-01	3.04	3.23	2.91	3.09	448910	{}	\N	2020-10-12 15:43:30.406232
59633	NEXT	2020-10-02	3	3.11	2.86	2.89	375407	{}	\N	2020-10-12 15:43:30.415392
59634	NEXT	2020-10-05	2.93	3.0443	2.85	2.92	306689	{}	\N	2020-10-12 15:43:30.424392
59635	NEXT	2020-10-06	2.993	3.21	2.88	2.95	454459	{}	\N	2020-10-12 15:43:30.431479
59636	NEXT	2020-10-07	3.05	3.09	2.83	2.88	315745	{}	\N	2020-10-12 15:43:30.438138
59637	NEXT	2020-10-08	2.92	2.99	2.8011	2.92	188187	{}	\N	2020-10-12 15:43:30.445201
59638	NEXT	2020-10-09	2.95	2.97	2.86	2.89	116566	{}	\N	2020-10-12 15:43:30.450873
59639	NEXT	2020-10-12	2.9	2.91	2.76	2.81	65874	{}	\N	2020-10-12 15:43:30.455867
59640	NGHCP	2020-09-28	25.5557	25.6	25.5494	25.592	6539	{}	\N	2020-10-12 15:43:32.207965
59641	NGHCP	2020-09-29	25.65	25.65	25.65	25.65	1571	{}	\N	2020-10-12 15:43:32.225952
59642	NGHCP	2020-09-30	25.22	25.22	25.16	25.22	8629	{}	\N	2020-10-12 15:43:32.233918
59643	NGHCP	2020-10-01	25.18	25.24	25.18	25.2399	6666	{}	\N	2020-10-12 15:43:32.242199
59644	NGHCP	2020-10-02	25.185	25.22	25.16	25.2104	2820	{}	\N	2020-10-12 15:43:32.248347
59645	NGHCP	2020-10-05	25.232	25.232	25.0875	25.2	2737	{}	\N	2020-10-12 15:43:32.253047
59646	NGHCP	2020-10-06	25.215	25.24	25.215	25.24	3774	{}	\N	2020-10-12 15:43:32.257952
59647	NGHCP	2020-10-07	25.24	25.48	25.21	25.445	4163	{}	\N	2020-10-12 15:43:32.263267
59648	NGHCP	2020-10-08	25.21	25.3281	25.21	25.26	1600	{}	\N	2020-10-12 15:43:32.267803
59649	NGHCP	2020-10-09	25.2124	25.2124	25.2124	25.2124	500	{}	\N	2020-10-12 15:43:32.272498
59650	NHI	2020-09-28	60.91	62.11	60.74	61.81	235299	{}	\N	2020-10-12 15:43:33.213633
59651	NHI	2020-09-29	60.99	61.005	59.11	60.59	464551	{}	\N	2020-10-12 15:43:33.23263
59652	NHI	2020-09-30	60.53	62.08	59.49	60.27	561786	{}	\N	2020-10-12 15:43:33.244452
59653	NHI	2020-10-01	60.3	61.88	59.68	61.78	462181	{}	\N	2020-10-12 15:43:33.263421
59654	NHI	2020-10-02	60.59	63.725	60.15	63.48	486347	{}	\N	2020-10-12 15:43:33.285774
59655	NHI	2020-10-05	63.79	64.255	62.11	63.33	354010	{}	\N	2020-10-12 15:43:33.305855
59656	NHI	2020-10-06	63.66	63.99	62.24	62.54	483650	{}	\N	2020-10-12 15:43:33.314768
59657	NHI	2020-10-07	62.91	63.23	61.7	61.73	459573	{}	\N	2020-10-12 15:43:33.323952
59658	NHI	2020-10-08	62.13	62.98	61.92	62.21	402487	{}	\N	2020-10-12 15:43:33.333915
59659	NHI	2020-10-09	62.79	62.79	61.02	61.13	363170	{}	\N	2020-10-12 15:43:33.341582
59660	NHI	2020-10-12	61.08	61.62	60.74	61.24	83509	{}	\N	2020-10-12 15:43:33.349039
59661	NHF	2020-09-28	8.76	8.98	8.76	8.82	149945	{}	\N	2020-10-12 15:43:34.337038
59662	NHF	2020-09-29	8.8	8.8275	8.5	8.61	580006	{}	\N	2020-10-12 15:43:34.342173
59663	NHF	2020-09-30	8.64	8.85	8.6	8.66	630846	{}	\N	2020-10-12 15:43:34.347166
59664	NHF	2020-10-01	8.82	8.8599	8.68	8.79	425814	{}	\N	2020-10-12 15:43:34.351867
59665	NHF	2020-10-02	8.67	8.88	8.65	8.73	348514	{}	\N	2020-10-12 15:43:34.358265
59666	NHF	2020-10-05	8.73	8.8049	8.65	8.69	325354	{}	\N	2020-10-12 15:43:34.364813
59667	NHF	2020-10-06	8.7	8.85	8.6365	8.72	375732	{}	\N	2020-10-12 15:43:34.373924
59668	NHF	2020-10-07	8.7	8.75	8.62	8.66	297677	{}	\N	2020-10-12 15:43:34.382956
59669	NHF	2020-10-08	8.7	8.7	8.54	8.57	399902	{}	\N	2020-10-12 15:43:34.388544
59670	NHF	2020-10-09	8.61	8.71	8.42	8.46	589600	{}	\N	2020-10-12 15:43:34.393203
59671	NHF	2020-10-12	8.48	8.52	8.46	8.49	134994	{}	\N	2020-10-12 15:43:34.399175
59672	NAKD	2020-09-28	0.117	0.124	0.1099	0.1211	28965556	{}	\N	2020-10-12 15:43:35.63282
59673	NAKD	2020-09-29	0.1194	0.124	0.113	0.1199	19893368	{}	\N	2020-10-12 15:43:35.637956
59674	NAKD	2020-09-30	0.113	0.1183	0.1081	0.1115	30850026	{}	\N	2020-10-12 15:43:35.642322
59675	NAKD	2020-10-01	0.1104	0.1425	0.1086	0.1384	107462983	{}	\N	2020-10-12 15:43:35.646682
59676	NAKD	2020-10-02	0.1182	0.13	0.1103	0.1163	54982732	{}	\N	2020-10-12 15:43:35.652641
59677	NAKD	2020-10-05	0.1164	0.1225	0.1101	0.1135	60183364	{}	\N	2020-10-12 15:43:35.65721
59678	NAKD	2020-10-06	0.125	0.128	0.116	0.12	105802474	{}	\N	2020-10-12 15:43:35.662613
59679	NAKD	2020-10-07	0.1275	0.135	0.1229	0.1261	65040578	{}	\N	2020-10-12 15:43:35.668609
59680	NAKD	2020-10-08	0.123	0.1279	0.1172	0.1241	65470249	{}	\N	2020-10-12 15:43:35.672767
59681	NAKD	2020-10-09	0.121	0.1218	0.115	0.12	32052303	{}	\N	2020-10-12 15:43:35.677149
59682	NAKD	2020-10-12	0.1199	0.1231	0.118	0.1205	22064354	{}	\N	2020-10-12 15:43:35.681568
59683	NBH	2020-09-28	15.28	15.6	15.28	15.47	22967	{}	\N	2020-10-12 15:43:37.114543
59684	NBH	2020-09-29	15.39	15.59	15.3	15.59	21749	{}	\N	2020-10-12 15:43:37.122553
59685	NBH	2020-09-30	15.5	15.63	15.34	15.4	31088	{}	\N	2020-10-12 15:43:37.127303
59686	NBH	2020-10-01	15.6	15.6	15.31	15.44	25684	{}	\N	2020-10-12 15:43:37.132569
59687	NBH	2020-10-02	15.6	15.6	15.45	15.49	11799	{}	\N	2020-10-12 15:43:37.138456
59688	NBH	2020-10-05	15.6	15.6	15.25	15.25	21693	{}	\N	2020-10-12 15:43:37.142842
59689	NBH	2020-10-06	15.25	15.55	15.19	15.2	31770	{}	\N	2020-10-12 15:43:37.147629
59690	NBH	2020-10-07	15.5	15.5	15.11	15.16	23639	{}	\N	2020-10-12 15:43:37.154282
59691	NBH	2020-10-08	15.27	15.43	15.06	15.09	19971	{}	\N	2020-10-12 15:43:37.158848
59692	NBH	2020-10-09	15.17	15.17	15.02	15.02	12187	{}	\N	2020-10-12 15:43:37.16542
59693	NBH	2020-10-12	15.17	15.17	15.02	15.07	5605	{}	\N	2020-10-12 15:43:37.172697
59694	NBN	2020-09-28	17.84	18.525	17.84	18.42	21920	{}	\N	2020-10-12 15:43:38.270635
59695	NBN	2020-09-29	18.46	18.87	17.69	18.86	25063	{}	\N	2020-10-12 15:43:38.27559
59696	NBN	2020-09-30	18.7	18.86	18.275	18.4	18782	{}	\N	2020-10-12 15:43:38.281437
59697	NBN	2020-10-01	18.6	18.675	18.08	18.36	15648	{}	\N	2020-10-12 15:43:38.288654
59698	NBN	2020-10-02	18.305	19.32	18.305	19.09	22351	{}	\N	2020-10-12 15:43:38.296715
59699	NBN	2020-10-05	19.09	19.745	19.09	19.63	12388	{}	\N	2020-10-12 15:43:38.30294
59700	NBN	2020-10-06	19.73	19.73	19	19.2	36183	{}	\N	2020-10-12 15:43:38.307972
59701	NBN	2020-10-07	18.855	19.46	18.855	19.45	17483	{}	\N	2020-10-12 15:43:38.312625
59702	NBN	2020-10-08	19.34	19.635	19.0249	19.49	18976	{}	\N	2020-10-12 15:43:38.317698
59703	NBN	2020-10-09	19.35	19.87	19.35	19.62	22814	{}	\N	2020-10-12 15:43:38.324002
59704	NBN	2020-10-12	19.47	19.63	19.47	19.565	13699	{}	\N	2020-10-12 15:43:38.330337
59705	NCA	2020-09-28	10.73	10.8	10.42	10.8	25522	{}	\N	2020-10-12 15:43:39.521788
59706	NCA	2020-09-29	10.81	10.81	10.7481	10.8	27312	{}	\N	2020-10-12 15:43:39.529548
59707	NCA	2020-09-30	10.81	10.81	10.77	10.78	79976	{}	\N	2020-10-12 15:43:39.537803
59708	NCA	2020-10-01	10.83	10.83	10.74	10.79	67316	{}	\N	2020-10-12 15:43:39.543851
59709	NCA	2020-10-02	10.76	10.8	10.71	10.74	53700	{}	\N	2020-10-12 15:43:39.549837
59710	NCA	2020-10-05	10.74	10.8	10.6	10.62	60110	{}	\N	2020-10-12 15:43:39.55623
59711	NCA	2020-10-06	10.62	10.66	10.55	10.62	63812	{}	\N	2020-10-12 15:43:39.561833
59712	NCA	2020-10-07	10.63	10.63	10.55	10.55	41652	{}	\N	2020-10-12 15:43:39.568145
59713	NCA	2020-10-08	10.6	10.63	10.53	10.58	22334	{}	\N	2020-10-12 15:43:39.575448
59714	NCA	2020-10-09	10.57	10.57	10.49	10.51	67244	{}	\N	2020-10-12 15:43:39.582808
59715	NCA	2020-10-12	10.56	10.56	10.5	10.5	13134	{}	\N	2020-10-12 15:43:39.589332
59716	NCV	2020-09-28	4.6	4.62	4.57	4.61	599204	{}	\N	2020-10-12 15:43:40.527135
59717	NCV	2020-09-29	4.59	4.608	4.56	4.6	1718816	{}	\N	2020-10-12 15:43:40.534718
59718	NCV	2020-09-30	4.59	4.63	4.58	4.58	4050922	{}	\N	2020-10-12 15:43:40.546902
59719	NCV	2020-10-01	4.62	4.66	4.6	4.64	2590941	{}	\N	2020-10-12 15:43:40.556652
59720	NCV	2020-10-02	4.58	4.64	4.57	4.62	1739819	{}	\N	2020-10-12 15:43:40.564536
59721	NCV	2020-10-05	4.63	4.69	4.63	4.65	3107262	{}	\N	2020-10-12 15:43:40.571229
59722	NCV	2020-10-06	4.69	4.75	4.67	4.69	3569571	{}	\N	2020-10-12 15:43:40.577036
59723	NCV	2020-10-07	4.71	4.86	4.71	4.84	1072122	{}	\N	2020-10-12 15:43:40.584466
59724	NCV	2020-10-08	4.85	4.89	4.82	4.83	1798203	{}	\N	2020-10-12 15:43:40.591059
59725	NCV	2020-10-09	4.81	4.84	4.78	4.8	1034370	{}	\N	2020-10-12 15:43:40.597143
59726	NCV	2020-10-12	4.85	4.85	4.81	4.8288	305368	{}	\N	2020-10-12 15:43:40.603609
59727	NEV	2020-09-28	14.87	14.87	14.74	14.78	16127	{}	\N	2020-10-12 15:43:41.947372
59728	NEV	2020-09-29	14.73	14.7754	14.66	14.71	96550	{}	\N	2020-10-12 15:43:41.954825
59729	NEV	2020-09-30	14.66	14.7096	14.64	14.65	91140	{}	\N	2020-10-12 15:43:41.961804
59730	NEV	2020-10-01	14.77	14.77	14.6	14.65	130834	{}	\N	2020-10-12 15:43:41.968529
59731	NEV	2020-10-02	14.58	14.69	14.58	14.65	71722	{}	\N	2020-10-12 15:43:41.978607
59732	NEV	2020-10-05	14.66	14.67	14.5	14.53	70210	{}	\N	2020-10-12 15:43:41.988842
59733	NEV	2020-10-06	14.62	14.65	14.54	14.64	84665	{}	\N	2020-10-12 15:43:41.99832
59734	NEV	2020-10-07	14.75	14.77	14.65	14.67	109352	{}	\N	2020-10-12 15:43:42.006476
59735	NEV	2020-10-08	14.74	14.98	14.68	14.86	56324	{}	\N	2020-10-12 15:43:42.012705
59736	NEV	2020-10-09	15.08	15.17	15.04	15.05	50232	{}	\N	2020-10-12 15:43:42.019226
59737	NEV	2020-10-12	15.11	15.11	15.01	15.01	14482	{}	\N	2020-10-12 15:43:42.027721
59738	NFLT	2020-09-28	24.61	24.63	24.6	24.63	3735	{}	\N	2020-10-12 15:43:43.138934
59739	NFLT	2020-09-29	24.65	24.65	24.57	24.5932	1830	{}	\N	2020-10-12 15:43:43.145666
59740	NFLT	2020-09-30	24.6093	24.6409	24.6093	24.6409	359	{}	\N	2020-10-12 15:43:43.150885
59741	NFLT	2020-10-01	24.66	24.66	24.64	24.64	10639	{}	\N	2020-10-12 15:43:43.156131
59742	NFLT	2020-10-02	24.6557	24.665	24.6515	24.665	503	{}	\N	2020-10-12 15:43:43.163695
59743	NFLT	2020-10-05	26.04	26.37	24.66	24.68	3868	{}	\N	2020-10-12 15:43:43.170089
59744	NFLT	2020-10-06	24.86	24.86	24.7097	24.7097	1205	{}	\N	2020-10-12 15:43:43.176242
59745	NFLT	2020-10-07	24.75	24.77	24.75	24.77	328	{}	\N	2020-10-12 15:43:43.181549
59746	NFLT	2020-10-08	24.8	24.83	24.8	24.8129	6191	{}	\N	2020-10-12 15:43:43.186735
59747	NFLT	2020-10-09	24.89	24.89	24.77	24.78	50160	{}	\N	2020-10-12 15:43:43.192877
59748	NG	2020-09-28	11.45	11.735	11.36	11.65	2139481	{}	\N	2020-10-12 15:43:44.443255
59749	NG	2020-09-29	11.79	12.1	11.665	11.81	2783865	{}	\N	2020-10-12 15:43:44.45017
59750	NG	2020-09-30	11.76	12.05	11.7	11.89	2386649	{}	\N	2020-10-12 15:43:44.45735
59751	NG	2020-10-01	11.7	11.92	11.25	11.5	3223305	{}	\N	2020-10-12 15:43:44.464722
59752	NG	2020-10-02	11.32	11.59	11.05	11.1	2392445	{}	\N	2020-10-12 15:43:44.471541
59753	NG	2020-10-05	11.1	11.37	11.04	11.14	1436984	{}	\N	2020-10-12 15:43:44.478359
59754	NG	2020-10-06	11.18	11.375	10.92	10.965	1966213	{}	\N	2020-10-12 15:43:44.483919
59755	NG	2020-10-07	11.2	11.2	10.93	10.96	1841600	{}	\N	2020-10-12 15:43:44.489487
59756	NG	2020-10-08	11.15	11.42	10.93	11.02	2437672	{}	\N	2020-10-12 15:43:44.495864
59757	NG	2020-10-09	11.28	11.46	11.12	11.4	1768246	{}	\N	2020-10-12 15:43:44.502672
59758	NG	2020-10-12	11.34	11.43	11.21	11.32	288465	{}	\N	2020-10-12 15:43:44.508769
59759	NGHCO	2020-09-28	25.59	25.59	25.52	25.58	19448	{}	\N	2020-10-12 15:43:45.429767
59760	NGHCO	2020-09-29	25.62	25.621	25.56	25.585	7984	{}	\N	2020-10-12 15:43:45.435777
59761	NGHCO	2020-09-30	25.19	25.2	25.12	25.2	15601	{}	\N	2020-10-12 15:43:45.441589
59762	NGHCO	2020-10-01	25.2	25.22	25.15	25.22	22734	{}	\N	2020-10-12 15:43:45.448627
59763	NGHCO	2020-10-02	25.2193	25.2193	25.11	25.141	9471	{}	\N	2020-10-12 15:43:45.455604
59764	NGHCO	2020-10-05	25.18	25.21	25.17	25.1894	7531	{}	\N	2020-10-12 15:43:45.464029
59765	NGHCO	2020-10-06	25.19	25.27	25.19	25.2	9659	{}	\N	2020-10-12 15:43:45.471433
59766	NGHCO	2020-10-07	25.21	25.24	25.2	25.2215	7764	{}	\N	2020-10-12 15:43:45.483241
59767	NGHCO	2020-10-08	25.2269	25.24	25.196	25.22	12423	{}	\N	2020-10-12 15:43:45.491816
59768	NGHCO	2020-10-09	25.2239	25.24	25.1925	25.2399	13830	{}	\N	2020-10-12 15:43:45.500463
59769	NGHCO	2020-10-12	25.22	25.2226	25.2194	25.2194	1827	{}	\N	2020-10-12 15:43:45.507932
59770	NH	2020-09-28	2.29	2.38	2.28	2.32	138849	{}	\N	2020-10-12 15:43:46.402572
59771	NH	2020-09-29	2.35	2.37	2.29	2.29	60596	{}	\N	2020-10-12 15:43:46.408569
59772	NH	2020-09-30	2.3	2.41	2.28	2.35	118469	{}	\N	2020-10-12 15:43:46.415992
59773	NH	2020-10-01	2.36	2.4	2.245	2.25	97464	{}	\N	2020-10-12 15:43:46.422966
59774	NH	2020-10-02	2.25	2.28	2.13	2.15	151083	{}	\N	2020-10-12 15:43:46.429751
59775	NH	2020-10-05	2.16	2.24	2.13	2.2	141836	{}	\N	2020-10-12 15:43:46.436248
59776	NH	2020-10-06	2.22	2.58	2.22	2.3	435144	{}	\N	2020-10-12 15:43:46.441965
59777	NH	2020-10-07	2.34	2.44	2.18	2.28	277352	{}	\N	2020-10-12 15:43:46.448544
59778	NH	2020-10-08	2.29	2.5	2.2701	2.39	256258	{}	\N	2020-10-12 15:43:46.455265
59779	NH	2020-10-09	2.43	2.48	2.325	2.43	153121	{}	\N	2020-10-12 15:43:46.462421
59780	NH	2020-10-12	2.46	2.49	2.36	2.36	53710	{}	\N	2020-10-12 15:43:46.467921
59781	NHLD	2020-09-28	2.28	2.28	2.075	2.1	28835	{}	\N	2020-10-12 15:43:47.443496
59782	NHLD	2020-09-29	2.1	2.11	2.085	2.11	22754	{}	\N	2020-10-12 15:43:47.450586
59783	NHLD	2020-09-30	2.1	2.21	2.09	2.12	19749	{}	\N	2020-10-12 15:43:47.455667
59784	NHLD	2020-10-01	2.165	2.23	2.165	2.23	272	{}	\N	2020-10-12 15:43:47.461037
59785	NHLD	2020-10-02	2.0101	2.26	2.0101	2.19	4054	{}	\N	2020-10-12 15:43:47.468449
59786	NHLD	2020-10-05	2.17	2.3	2.155	2.16	2308	{}	\N	2020-10-12 15:43:47.474712
59787	NHLD	2020-10-06	2.1617	2.33	2.1617	2.29	1869	{}	\N	2020-10-12 15:43:47.480343
59788	NHLD	2020-10-07	2.12	2.155	2.12	2.1508	1484	{}	\N	2020-10-12 15:43:47.485572
59789	NHLD	2020-10-08	2.21	2.21	2.21	2.21	272	{}	\N	2020-10-12 15:43:47.490383
59790	NHLD	2020-10-09	2.26	2.26	2.25	2.25	526	{}	\N	2020-10-12 15:43:47.49576
59791	NHLD	2020-10-12	2.125	2.125	2.125	2.125	318	{}	\N	2020-10-12 15:43:47.500256
59792	NWBI	2020-09-28	9.25	9.51	9.24	9.39	702122	{}	\N	2020-10-12 15:43:48.819425
59793	NWBI	2020-09-29	9.39	9.41	9.07	9.23	455057	{}	\N	2020-10-12 15:43:48.825437
59794	NWBI	2020-09-30	9.28	9.38	9.09	9.2	702116	{}	\N	2020-10-12 15:43:48.833657
59795	NWBI	2020-10-01	9.23	9.31	9.07	9.3	389772	{}	\N	2020-10-12 15:43:48.840236
59796	NWBI	2020-10-02	9.12	9.59	9.11	9.51	534694	{}	\N	2020-10-12 15:43:48.848835
59797	NWBI	2020-10-05	9.6	9.82	9.55	9.76	448108	{}	\N	2020-10-12 15:43:48.856929
59798	NWBI	2020-10-06	9.88	10.11	9.765	9.77	725398	{}	\N	2020-10-12 15:43:48.863524
59799	NWBI	2020-10-07	9.83	10.065	9.83	10.02	674247	{}	\N	2020-10-12 15:43:48.868792
59800	NWBI	2020-10-08	10.13	10.22	10.02	10.18	411914	{}	\N	2020-10-12 15:43:48.874851
59801	NWBI	2020-10-09	10.31	10.34	10.04	10.06	428154	{}	\N	2020-10-12 15:43:48.882157
59802	NWBI	2020-10-12	10.01	10.285	10.01	10.22	116142	{}	\N	2020-10-12 15:43:48.889073
59803	NHLDW	2020-10-02	0.0899	0.0899	0.0771	0.0833	10840	{}	\N	2020-10-12 15:43:50.597701
59804	NHLDW	2020-10-05	0.077	0.0995	0.077	0.0885	4259	{}	\N	2020-10-12 15:43:50.604996
59805	NHLDW	2020-10-06	0.0828	0.0828	0.0825	0.0825	879	{}	\N	2020-10-12 15:43:50.610376
59806	NHLDW	2020-10-08	0.083	0.083	0.083	0.083	152	{}	\N	2020-10-12 15:43:50.616917
59807	NHLDW	2020-10-09	0.0932	0.0982	0.0932	0.0982	3340	{}	\N	2020-10-12 15:43:50.623906
59808	NHLDW	2020-10-12	0.08	0.08	0.08	0.08	1122	{}	\N	2020-10-12 15:43:50.630243
59809	NIE	2020-09-28	24.19	24.32	24.18	24.19	78987	{}	\N	2020-10-12 15:43:51.538755
59810	NIE	2020-09-29	24.14	24.17	24.035	24.1	132510	{}	\N	2020-10-12 15:43:51.546281
59811	NIE	2020-09-30	24.11	24.48	24.0999	24.28	179316	{}	\N	2020-10-12 15:43:51.552116
59812	NIE	2020-10-01	24.38	24.56	24.35	24.4	159805	{}	\N	2020-10-12 15:43:51.557829
59813	NIE	2020-10-02	24.18	24.38	24.065	24.33	168106	{}	\N	2020-10-12 15:43:51.563708
59814	NIE	2020-10-05	24.39	24.66	24.36	24.65	169627	{}	\N	2020-10-12 15:43:51.569708
59815	NIE	2020-10-06	24.72	24.93	24.5	24.62	181258	{}	\N	2020-10-12 15:43:51.575115
59816	NIE	2020-10-07	24.76	25.133	24.74	25.08	217773	{}	\N	2020-10-12 15:43:51.580757
59817	NIE	2020-10-08	25.19	25.29	25	25.29	211884	{}	\N	2020-10-12 15:43:51.58812
59818	NIE	2020-10-09	25.4	25.47	25.27	25.39	110348	{}	\N	2020-10-12 15:43:51.594043
59819	NIE	2020-10-12	25.5	25.6	25.4365	25.5919	91336	{}	\N	2020-10-12 15:43:51.599494
59820	NK	2020-09-28	7.2928	7.2928	6.86	7.19	912095	{}	\N	2020-10-12 15:43:52.544595
59821	NK	2020-09-29	7.12	7.365	6.97	7.18	586719	{}	\N	2020-10-12 15:43:52.551163
59822	NK	2020-09-30	7.21	7.22	6.83	6.935	1352139	{}	\N	2020-10-12 15:43:52.55715
59823	NK	2020-10-01	6.95	7.16	6.84	7.06	687150	{}	\N	2020-10-12 15:43:52.56324
59824	NK	2020-10-02	6.92	7.1799	6.87	7.04	498043	{}	\N	2020-10-12 15:43:52.569838
59825	NK	2020-10-05	7.1	7.94	7.0936	7.89	1177563	{}	\N	2020-10-12 15:43:52.57528
59826	NK	2020-10-06	7.95	8.1	7.44	7.63	960295	{}	\N	2020-10-12 15:43:52.58067
59827	NK	2020-10-07	7.62	8.66	7.62	8.53	1562841	{}	\N	2020-10-12 15:43:52.586499
59828	NK	2020-10-08	8.93	8.95	8.16	8.26	1051534	{}	\N	2020-10-12 15:43:52.592923
59829	NK	2020-10-09	8.45	8.8	8.1	8.74	942076	{}	\N	2020-10-12 15:43:52.598355
59830	NK	2020-10-12	8.95	8.98	8.41	8.5161	337604	{}	\N	2020-10-12 15:43:52.603856
59831	NL	2020-09-28	4.15	4.35	4.15	4.32	16021	{}	\N	2020-10-12 15:43:53.467321
59832	NL	2020-09-29	4.2606	4.39	4.22	4.28	29179	{}	\N	2020-10-12 15:43:53.474344
59833	NL	2020-09-30	4.22	4.29	4.16	4.25	15761	{}	\N	2020-10-12 15:43:53.480326
59834	NL	2020-10-01	4.26	4.3	4.22	4.28	18873	{}	\N	2020-10-12 15:43:53.486417
59835	NL	2020-10-02	4.29	4.29	4.2	4.25	35286	{}	\N	2020-10-12 15:43:53.492416
59836	NL	2020-10-05	4.31	4.4	4.26	4.39	29317	{}	\N	2020-10-12 15:43:53.497734
59837	NL	2020-10-06	4.49	4.855	4.4886	4.59	165366	{}	\N	2020-10-12 15:43:53.502711
59838	NL	2020-10-07	4.74	5.12	4.74	4.95	154445	{}	\N	2020-10-12 15:43:53.508834
59839	NL	2020-10-08	5.01	5.4	4.9501	5.35	95618	{}	\N	2020-10-12 15:43:53.514413
59840	NL	2020-10-09	5.35	5.35	4.83	4.88	62408	{}	\N	2020-10-12 15:43:53.519781
59841	NL	2020-10-12	4.88	4.8801	4.82	4.845	3072	{}	\N	2020-10-12 15:43:53.524537
59842	NLY	2020-09-28	7.38	7.5	7.33	7.44	20472891	{}	\N	2020-10-12 15:43:54.388523
59843	NLY	2020-09-29	7.27	7.278	7.07	7.12	26072704	{}	\N	2020-10-12 15:43:54.394945
59844	NLY	2020-09-30	7.13	7.225	7.07	7.12	24669152	{}	\N	2020-10-12 15:43:54.400836
59845	NLY	2020-10-01	7.14	7.14	7.01	7.12	21626322	{}	\N	2020-10-12 15:43:54.407361
59846	NLY	2020-10-02	7	7.22	6.99	7.19	25285764	{}	\N	2020-10-12 15:43:54.413706
59847	NLY	2020-10-05	7.2	7.3	7.195	7.27	19998497	{}	\N	2020-10-12 15:43:54.420003
59848	NLY	2020-10-06	7.27	7.35	7.2	7.22	19865696	{}	\N	2020-10-12 15:43:54.427605
59849	NLY	2020-10-07	7.28	7.29	7.2	7.27	19887177	{}	\N	2020-10-12 15:43:54.432802
59850	NLY	2020-10-08	7.3	7.4	7.26	7.38	26975175	{}	\N	2020-10-12 15:43:54.439253
59851	NLY	2020-10-09	7.39	7.4	7.21	7.23	18793882	{}	\N	2020-10-12 15:43:54.445017
59852	NLY	2020-10-12	7.23	7.27	7.215	7.27	4914745	{}	\N	2020-10-12 15:43:54.450222
59853	NNN	2020-09-28	35.18	35.665	34.97	35.29	889006	{}	\N	2020-10-12 15:43:55.880211
59854	NNN	2020-09-29	35.15	35.29	34.29	34.7	1780659	{}	\N	2020-10-12 15:43:55.887734
59855	NNN	2020-09-30	35	35.36	34.09	34.51	2109199	{}	\N	2020-10-12 15:43:55.893768
59856	NNN	2020-10-01	34.52	35.61	34.52	35.59	1669888	{}	\N	2020-10-12 15:43:55.89968
59857	NNN	2020-10-02	34.88	36.82	34.64	36.69	2322344	{}	\N	2020-10-12 15:43:55.90604
59858	NNN	2020-10-05	36.95	37.06	35.98	36.67	1760976	{}	\N	2020-10-12 15:43:55.911378
59859	NNN	2020-10-06	36.98	37	36.05	36.28	1531980	{}	\N	2020-10-12 15:43:55.917359
59860	NNN	2020-10-07	36.42	36.5649	35.53	35.56	1206029	{}	\N	2020-10-12 15:43:55.92469
59861	NNN	2020-10-08	35.76	36.43	35.76	36.39	1519607	{}	\N	2020-10-12 15:43:55.930725
59862	NNN	2020-10-09	36.71	36.71	35.77	35.98	989610	{}	\N	2020-10-12 15:43:55.936026
59863	NNN	2020-10-12	35.92	36.12	35.5513	36.02	139427	{}	\N	2020-10-12 15:43:55.94184
59864	NODK	2020-09-28	16.1	17.09	15.65	16.47	9174	{}	\N	2020-10-12 15:43:57.414011
59865	NODK	2020-09-29	16.31	16.65	16.31	16.65	8871	{}	\N	2020-10-12 15:43:57.42058
59866	NODK	2020-09-30	16.7577	17.1	16.72	16.89	12691	{}	\N	2020-10-12 15:43:57.427555
59867	NODK	2020-10-01	16.9	16.92	16.355	16.43	22252	{}	\N	2020-10-12 15:43:57.432886
59868	NODK	2020-10-02	16.5	16.83	16.14	16.38	9993	{}	\N	2020-10-12 15:43:57.439084
59869	NODK	2020-10-05	15.995	16.72	15.86	16.72	10290	{}	\N	2020-10-12 15:43:57.445545
59870	NODK	2020-10-06	16.85	17.02	16.51	16.51	13407	{}	\N	2020-10-12 15:43:57.451163
59871	NODK	2020-10-07	16.925	16.98	16.7	16.98	18484	{}	\N	2020-10-12 15:43:57.457379
59872	NODK	2020-10-08	16.99	17	16.7	16.915	6367	{}	\N	2020-10-12 15:43:57.462668
59873	NODK	2020-10-09	16.44	17.1	16.42	17.1	18471	{}	\N	2020-10-12 15:43:57.468947
59874	NODK	2020-10-12	16.98	16.98	16.98	16.98	362	{}	\N	2020-10-12 15:43:57.475146
59875	NOW	2020-09-28	486.61	488.4	478.97	484.21	1283717	{}	\N	2020-10-12 15:43:58.874349
59876	NOW	2020-09-29	486.35	489.56	483.0952	487.98	1683455	{}	\N	2020-10-12 15:43:59.088221
59877	NOW	2020-09-30	485.67	493.31	481.15	485	2404085	{}	\N	2020-10-12 15:43:59.09496
59878	NOW	2020-10-01	491.57	495.13	487.4528	492.52	1763904	{}	\N	2020-10-12 15:43:59.099882
59879	NOW	2020-10-02	480.49	502.63	478.99	493.15	3137102	{}	\N	2020-10-12 15:43:59.104642
59880	NOW	2020-10-05	495.61	502.23	491.2702	501.79	2522966	{}	\N	2020-10-12 15:43:59.109905
59881	NOW	2020-10-06	500	502.5999	489.82	491.54	2107698	{}	\N	2020-10-12 15:43:59.114557
59882	NOW	2020-10-07	497.63	504.21	494	500	1723788	{}	\N	2020-10-12 15:43:59.11939
59883	NOW	2020-10-08	505	505	492	496.5	1681273	{}	\N	2020-10-12 15:43:59.123886
59884	NOW	2020-10-09	503.56	510.88	502.01	508.61	2282765	{}	\N	2020-10-12 15:43:59.129493
59885	NOW	2020-10-12	513.55	515.55	502	507.2855	780881	{}	\N	2020-10-12 15:43:59.135019
59886	NRIM	2020-09-28	24.57	25.5	24.325	25.22	59174	{}	\N	2020-10-12 15:44:00.406465
59887	NRIM	2020-09-29	25.35	25.35	24.55	25.3	31972	{}	\N	2020-10-12 15:44:00.413203
59888	NRIM	2020-09-30	25.15	25.82	25.15	25.49	17106	{}	\N	2020-10-12 15:44:00.418577
59889	NRIM	2020-10-01	25.64	25.64	24.92	25.48	39744	{}	\N	2020-10-12 15:44:00.423805
59890	NRIM	2020-10-02	25.07	26.655	25.01	26.58	26680	{}	\N	2020-10-12 15:44:00.431467
59891	NRIM	2020-10-05	27.24	27.51	26.59	27.4	27512	{}	\N	2020-10-12 15:44:00.437164
59892	NRIM	2020-10-06	27.6	29.1516	26.81	27.57	31076	{}	\N	2020-10-12 15:44:00.44379
59893	NRIM	2020-10-07	28.03	29.49	28.03	29.15	31123	{}	\N	2020-10-12 15:44:00.449953
59894	NRIM	2020-10-08	29.2	29.41	28.4	29.2	32422	{}	\N	2020-10-12 15:44:00.455475
59895	NRIM	2020-10-09	29.64	29.64	28.95	29.31	14004	{}	\N	2020-10-12 15:44:00.462657
59896	NRIM	2020-10-12	29.31	29.66	29.31	29.455	3984	{}	\N	2020-10-12 15:44:00.46861
59897	NSC	2020-09-28	221.91	222.82	219.8	221.02	1099960	{}	\N	2020-10-12 15:44:01.820046
59898	NSC	2020-09-29	220.5	221.49	215.8	215.81	3034745	{}	\N	2020-10-12 15:44:01.825572
59899	NSC	2020-09-30	217.53	219	212.85	213.99	3129546	{}	\N	2020-10-12 15:44:01.831575
59900	NSC	2020-10-01	215.73	215.84	210.5	211.85	2845709	{}	\N	2020-10-12 15:44:01.836519
59901	NSC	2020-10-02	208.98	216.01	208.17	212.66	4374457	{}	\N	2020-10-12 15:44:01.843262
59902	NSC	2020-10-05	215.51	216.59	214.29	215.16	2314239	{}	\N	2020-10-12 15:44:01.849543
59903	NSC	2020-10-06	216.91	218.26	213.23	213.52	1840251	{}	\N	2020-10-12 15:44:01.854472
59904	NSC	2020-10-07	214.39	218.47	214.39	216.86	1693406	{}	\N	2020-10-12 15:44:01.859435
59905	NSC	2020-10-08	219.57	220.72	213.56	216.39	3124681	{}	\N	2020-10-12 15:44:01.865248
59906	NSC	2020-10-09	219.5	219.6203	216.2401	218.39	1381468	{}	\N	2020-10-12 15:44:01.869869
59907	NSC	2020-10-12	219.85	221.38	218.14	218.6	1056553	{}	\N	2020-10-12 15:44:01.87485
59908	NTAP	2020-09-28	42.69	43.613	42.51	43.6	2217618	{}	\N	2020-10-12 15:44:02.756534
59909	NTAP	2020-09-29	43.71	44.27	43.495	43.99	1654496	{}	\N	2020-10-12 15:44:02.76313
59910	NTAP	2020-09-30	44.07	44.72	43.43	43.84	2351751	{}	\N	2020-10-12 15:44:02.768578
59911	NTAP	2020-10-01	44.37	44.37	42.62	42.89	1963736	{}	\N	2020-10-12 15:44:02.774815
59912	NTAP	2020-10-02	42.16	43.51	42.07	42.78	1563019	{}	\N	2020-10-12 15:44:02.783214
59913	NTAP	2020-10-05	43.38	43.65	43.072	43.35	2408315	{}	\N	2020-10-12 15:44:02.789462
59914	NTAP	2020-10-06	43.46	44.59	43.18	43.37	1353692	{}	\N	2020-10-12 15:44:02.79562
59915	NTAP	2020-10-07	43.89	44.74	43.72	44.59	1619345	{}	\N	2020-10-12 15:44:02.801916
59916	NTAP	2020-10-08	44.4243	46.29	44.4243	46.26	2919334	{}	\N	2020-10-12 15:44:02.80795
59917	NTAP	2020-10-09	46.49	46.69	45.52	45.87	1758440	{}	\N	2020-10-12 15:44:02.814974
59918	NTAP	2020-10-12	46.3687	46.3687	45.6071	46.15	397671	{}	\N	2020-10-12 15:44:02.821237
59919	NTLA	2020-09-28	21.67	21.78	20.94	21.22	467836	{}	\N	2020-10-12 15:44:04.303649
59920	NTLA	2020-09-29	21.15	22.4	21.08	21.61	649573	{}	\N	2020-10-12 15:44:04.310111
59921	NTLA	2020-09-30	21.62	22.09	19.75	19.88	638721	{}	\N	2020-10-12 15:44:04.317726
59922	NTLA	2020-10-01	20.08	20.29	19.51	20.09	532503	{}	\N	2020-10-12 15:44:04.324144
59923	NTLA	2020-10-02	19.281	20	18.81	18.83	451161	{}	\N	2020-10-12 15:44:04.33135
59924	NTLA	2020-10-05	19.26	20.93	19.26	20.77	620778	{}	\N	2020-10-12 15:44:04.337067
59925	NTLA	2020-10-06	20.81	21.6849	20.67	21.01	650627	{}	\N	2020-10-12 15:44:04.342698
59926	NTLA	2020-10-07	21.59	25.93	21.3595	23.78	2697090	{}	\N	2020-10-12 15:44:04.348861
59927	NTLA	2020-10-08	24.36	24.69	22.29	22.94	1727530	{}	\N	2020-10-12 15:44:04.354758
59928	NTLA	2020-10-09	23	26.7	22.96	26.31	1858411	{}	\N	2020-10-12 15:44:04.361771
59929	NTLA	2020-10-12	26.58	26.77	24.6782	25.05	883773	{}	\N	2020-10-12 15:44:04.367882
59930	NUEM	2020-09-28	27.7798	27.88	27.75	27.84	7305	{}	\N	2020-10-12 15:44:06.007468
59931	NUEM	2020-09-29	27.99	27.99	27.8965	27.8965	996	{}	\N	2020-10-12 15:44:06.013034
59932	NUEM	2020-09-30	28.06	28.3877	28.05	28.38	13051	{}	\N	2020-10-12 15:44:06.018978
59933	NUEM	2020-10-01	28.5632	28.67	28.54	28.6699	8349	{}	\N	2020-10-12 15:44:06.024912
59934	NUEM	2020-10-02	28.5365	28.5797	28.41	28.41	3017	{}	\N	2020-10-12 15:44:06.031167
59935	NUEM	2020-10-05	28.6062	28.815	28.6062	28.815	10720	{}	\N	2020-10-12 15:44:06.037014
59936	NUEM	2020-10-06	28.81	29.1	28.81	28.87	9134	{}	\N	2020-10-12 15:44:06.041626
59937	NUEM	2020-10-07	29.155	29.275	29.1514	29.275	2748	{}	\N	2020-10-12 15:44:06.046338
59938	NUEM	2020-10-08	29.32	29.5857	29.32	29.5857	1683	{}	\N	2020-10-12 15:44:06.051716
59939	NUEM	2020-10-09	29.86	29.86	29.7744	29.815	74959	{}	\N	2020-10-12 15:44:06.056789
59940	NUEM	2020-10-12	30.023	30.023	30.023	30.023	735	{}	\N	2020-10-12 15:44:06.061435
59941	NHA	2020-09-28	9.727	9.74	9.726	9.73	10400	{}	\N	2020-10-12 15:44:06.948182
59942	NHA	2020-09-29	9.74	9.74	9.68	9.6998	38568	{}	\N	2020-10-12 15:44:06.954349
59943	NHA	2020-09-30	9.69	9.71	9.69	9.71	812	{}	\N	2020-10-12 15:44:06.959646
59944	NHA	2020-10-01	9.74	9.74	9.72	9.73	3862	{}	\N	2020-10-12 15:44:06.964992
59945	NHA	2020-10-02	9.74	9.7434	9.715	9.74	38469	{}	\N	2020-10-12 15:44:06.971902
59946	NHA	2020-10-05	9.75	9.75	9.7	9.7	45462	{}	\N	2020-10-12 15:44:06.977574
59947	NHA	2020-10-06	9.72	9.7204	9.67	9.7	31400	{}	\N	2020-10-12 15:44:06.983408
59948	NHA	2020-10-07	9.7193	9.7193	9.7	9.7075	48106	{}	\N	2020-10-12 15:44:06.989683
59949	NHA	2020-10-09	9.74	9.74	9.7	9.7	502	{}	\N	2020-10-12 15:44:06.995168
59950	NHA	2020-10-12	9.74	9.74	9.74	9.74	700	{}	\N	2020-10-12 15:44:07.00057
59951	NICE	2020-09-28	227.98	228.55	224.25	224.4	186425	{}	\N	2020-10-12 15:44:08.315765
59952	NICE	2020-09-29	223.56	226.68	223.0791	225.98	232594	{}	\N	2020-10-12 15:44:08.323123
59953	NICE	2020-09-30	226.21	227.39	224.67	227.03	347202	{}	\N	2020-10-12 15:44:08.329336
59954	NICE	2020-10-01	228.18	230.93	227.03	230.7	511204	{}	\N	2020-10-12 15:44:08.337196
59955	NICE	2020-10-02	227.11	231	226.15	227.25	173666	{}	\N	2020-10-12 15:44:08.34396
59956	NICE	2020-10-05	229.64	231.7	229.3	231.62	93657	{}	\N	2020-10-12 15:44:08.350346
59957	NICE	2020-10-06	229.14	233.9	229.005	231.11	214403	{}	\N	2020-10-12 15:44:08.357731
59958	NICE	2020-10-07	233.87	234.492	231.23	231.8	107952	{}	\N	2020-10-12 15:44:08.364428
59959	NICE	2020-10-08	232.5	232.915	230.1	230.3	88627	{}	\N	2020-10-12 15:44:08.371043
59960	NICE	2020-10-09	233.16	235.99	232.05	235.37	176522	{}	\N	2020-10-12 15:44:08.377187
59961	NICE	2020-10-12	237.75	240.58	237.27	238.82	66558	{}	\N	2020-10-12 15:44:08.383605
59962	NJR	2020-09-28	26.82	27.021	26.55	26.66	494731	{}	\N	2020-10-12 15:44:09.639129
59963	NJR	2020-09-29	26.66	26.9072	26.18	26.66	845679	{}	\N	2020-10-12 15:44:09.645159
59964	NJR	2020-09-30	26.77	27.16	26.54	27.02	1191582	{}	\N	2020-10-12 15:44:09.650823
59965	NJR	2020-10-01	26.88	27.08	26.51	26.93	969398	{}	\N	2020-10-12 15:44:09.657962
59966	NJR	2020-10-02	26.74	27.7	26.43	27.56	940723	{}	\N	2020-10-12 15:44:09.663486
59967	NJR	2020-10-05	27.63	27.82	27.18	27.37	711933	{}	\N	2020-10-12 15:44:09.668701
59968	NJR	2020-10-06	27.62	28.74	27.38	28.07	987533	{}	\N	2020-10-12 15:44:09.674646
59969	NJR	2020-10-07	28.15	28.52	27.85	28.43	789315	{}	\N	2020-10-12 15:44:09.679731
59970	NJR	2020-10-08	28.45	29.22	28.45	29.14	762868	{}	\N	2020-10-12 15:44:09.684926
59971	NJR	2020-10-09	29.43	29.43	28.491	28.84	709836	{}	\N	2020-10-12 15:44:09.690788
59972	NJR	2020-10-12	28.74	29.05	28.5632	28.91	136761	{}	\N	2020-10-12 15:44:09.696805
59973	NMIH	2020-09-28	17.45	18.2	17.3	17.8	872401	{}	\N	2020-10-12 15:44:10.712735
59974	NMIH	2020-09-29	17.6025	17.92	17.28	17.54	709215	{}	\N	2020-10-12 15:44:10.720257
59975	NMIH	2020-09-30	17.69	18.18	17.61	17.8	792064	{}	\N	2020-10-12 15:44:10.72647
59976	NMIH	2020-10-01	17.9053	18.64	17.735	18.64	419201	{}	\N	2020-10-12 15:44:10.731633
59977	NMIH	2020-10-02	18.13	19.78	17.85	19.64	1029265	{}	\N	2020-10-12 15:44:10.738125
59978	NMIH	2020-10-05	20	20.8	19.86	20.77	984242	{}	\N	2020-10-12 15:44:10.74331
59979	NMIH	2020-10-06	21.08	22.34	20.91	21.51	1553362	{}	\N	2020-10-12 15:44:10.748054
59980	NMIH	2020-10-07	21.97	22.74	21.53	22.4	1308041	{}	\N	2020-10-12 15:44:10.752972
59981	NMIH	2020-10-08	22.7	23	22.14	22.97	488534	{}	\N	2020-10-12 15:44:10.758426
59982	NMIH	2020-10-09	23.36	23.58	22.61	22.75	636297	{}	\N	2020-10-12 15:44:10.763609
59983	NMIH	2020-10-12	23.07	23.23	22.67	23.145	103465	{}	\N	2020-10-12 15:44:10.768979
59984	NOAH	2020-09-28	25.6	26.1	25.36	26.07	196997	{}	\N	2020-10-12 15:44:11.881924
59985	NOAH	2020-09-29	26.03	26.15	25.83	26.12	252183	{}	\N	2020-10-12 15:44:11.886595
59986	NOAH	2020-09-30	26.35	27.52	26.06	26.07	596359	{}	\N	2020-10-12 15:44:11.891863
59987	NOAH	2020-10-01	26.26	26.85	26.105	26.78	351020	{}	\N	2020-10-12 15:44:11.895978
59988	NOAH	2020-10-02	26.37	27.32	26.37	27.02	183697	{}	\N	2020-10-12 15:44:11.901598
59989	NOAH	2020-10-05	27.15	27.59	27.11	27.29	319554	{}	\N	2020-10-12 15:44:11.907575
59990	NOAH	2020-10-06	27.46	27.79	27.14	27.29	195433	{}	\N	2020-10-12 15:44:11.912024
59991	NOAH	2020-10-07	27.43	27.7099	27.185	27.28	236155	{}	\N	2020-10-12 15:44:11.916687
59992	NOAH	2020-10-08	27.39	27.93	27.33	27.93	121960	{}	\N	2020-10-12 15:44:11.920828
59993	NOAH	2020-10-09	27.97	28.04	27.64	27.86	190813	{}	\N	2020-10-12 15:44:11.925524
59994	NOAH	2020-10-12	28.2	28.2	27.6	27.67	75407	{}	\N	2020-10-12 15:44:11.930169
59995	NOVT	2020-09-28	104.1077	106.68	103.89	106.39	96572	{}	\N	2020-10-12 15:44:12.81657
59996	NOVT	2020-09-29	107.97	109.28	106.75	107.13	84524	{}	\N	2020-10-12 15:44:12.824763
59997	NOVT	2020-09-30	106.99	108.5	105.22	105.34	98491	{}	\N	2020-10-12 15:44:12.830143
59998	NOVT	2020-10-01	105.96	109.71	105.96	109.21	97392	{}	\N	2020-10-12 15:44:12.836067
59999	NOVT	2020-10-02	106.92	108.83	106.48	107.4	59014	{}	\N	2020-10-12 15:44:12.842589
60000	NOVT	2020-10-05	108.34	111.045	108.34	110.8	52657	{}	\N	2020-10-12 15:44:12.849084
60001	NOVT	2020-10-06	111.66	113.25	109.31	109.38	80323	{}	\N	2020-10-12 15:44:12.854439
60002	NOVT	2020-10-07	110.72	112.67	110.56	111.83	69987	{}	\N	2020-10-12 15:44:12.861517
60003	NOVT	2020-10-08	112.88	114.39	112.875	113.3	63328	{}	\N	2020-10-12 15:44:12.866873
60004	NOVT	2020-10-09	114.3	116.34	114.3	115.37	55676	{}	\N	2020-10-12 15:44:12.871637
60005	NOVT	2020-10-12	116.28	116.54	113.67	114.99	12723	{}	\N	2020-10-12 15:44:12.877297
60006	NQP	2020-09-28	13.39	13.43	13.39	13.39	54988	{}	\N	2020-10-12 15:44:13.860128
60007	NQP	2020-09-29	13.41	13.47	13.34	13.41	106550	{}	\N	2020-10-12 15:44:13.866844
60008	NQP	2020-09-30	13.4	13.44	13.38	13.44	80984	{}	\N	2020-10-12 15:44:13.873231
60009	NQP	2020-10-01	13.49	13.49	13.4	13.47	108724	{}	\N	2020-10-12 15:44:13.879806
60010	NQP	2020-10-02	13.46	13.53	13.4246	13.53	62204	{}	\N	2020-10-12 15:44:13.885701
60011	NQP	2020-10-05	13.63	13.63	13.52	13.61	116778	{}	\N	2020-10-12 15:44:13.890361
60012	NQP	2020-10-06	13.61	13.6705	13.6072	13.65	104118	{}	\N	2020-10-12 15:44:13.896809
60013	NQP	2020-10-07	13.65	13.72	13.62	13.72	101576	{}	\N	2020-10-12 15:44:13.902863
60014	NQP	2020-10-08	13.69	13.69	13.62	13.62	73354	{}	\N	2020-10-12 15:44:13.908737
60015	NQP	2020-10-09	13.7	13.75	13.68	13.71	56386	{}	\N	2020-10-12 15:44:13.914159
60016	NQP	2020-10-12	13.82	13.82	13.73	13.7383	35748	{}	\N	2020-10-12 15:44:13.918855
60017	NS	2020-09-28	10.44	10.55	9.95	10.4	741166	{}	\N	2020-10-12 15:44:15.014655
60018	NS	2020-09-29	10.44	10.79	10.22	10.69	1826968	{}	\N	2020-10-12 15:44:15.020172
60019	NS	2020-09-30	10.59	10.92	10.4	10.62	2439635	{}	\N	2020-10-12 15:44:15.025239
60020	NS	2020-10-01	10.66	10.74	10.18	10.46	1120196	{}	\N	2020-10-12 15:44:15.03118
60021	NS	2020-10-02	10.12	10.65	10.12	10.55	614276	{}	\N	2020-10-12 15:44:15.036598
60022	NS	2020-10-05	10.69	11	10.6	10.98	880742	{}	\N	2020-10-12 15:44:15.041307
60023	NS	2020-10-06	11	11.4	10.72	10.97	1045295	{}	\N	2020-10-12 15:44:15.047114
60024	NS	2020-10-07	11.06	11.16	10.87	11.01	610419	{}	\N	2020-10-12 15:44:15.052128
60025	NS	2020-10-08	11.05	11.68	10.85	11.68	784637	{}	\N	2020-10-12 15:44:15.057083
60026	NS	2020-10-09	11.78	11.93	11.5	11.61	589484	{}	\N	2020-10-12 15:44:15.062982
60027	NS	2020-10-12	11.61	11.62	11.33	11.44	186049	{}	\N	2020-10-12 15:44:15.068426
60028	NTG	2020-09-28	14.34	14.6619	14.26	14.53	48004	{}	\N	2020-10-12 15:44:16.388342
60029	NTG	2020-09-29	14.5	14.89	14.46	14.74	67024	{}	\N	2020-10-12 15:44:16.395963
60030	NTG	2020-09-30	14.76	15.05	14.47	14.49	61403	{}	\N	2020-10-12 15:44:16.401242
60031	NTG	2020-10-01	14.49	14.54	14.2	14.3	50034	{}	\N	2020-10-12 15:44:16.407348
60032	NTG	2020-10-02	13.89	14.449	13.89	14.4	99226	{}	\N	2020-10-12 15:44:16.414939
60033	NTG	2020-10-05	14.62	15.29	14.62	15.2	93040	{}	\N	2020-10-12 15:44:16.421006
60034	NTG	2020-10-06	15.45	15.66	15.05	15.05	106466	{}	\N	2020-10-12 15:44:16.427058
60035	NTG	2020-10-07	15.13	15.481	15.1034	15.36	58443	{}	\N	2020-10-12 15:44:16.432971
60036	NTG	2020-10-08	15.5	16.22	15.496	16.15	132258	{}	\N	2020-10-12 15:44:16.438085
60037	NTG	2020-10-09	16.05	16.29	15.87	16	125727	{}	\N	2020-10-12 15:44:16.443037
60038	NTG	2020-10-12	15.89	15.89	15.73	15.86	38236	{}	\N	2020-10-12 15:44:16.45004
60039	NTRP	2020-09-28	1.13	1.13	1.0885	1.12	37070	{}	\N	2020-10-12 15:44:17.661514
60040	NTRP	2020-09-29	1.12	1.13	1.09	1.12	34290	{}	\N	2020-10-12 15:44:17.667608
60041	NTRP	2020-09-30	1.26	1.48	1.08	1.11	14607580	{}	\N	2020-10-12 15:44:17.673238
60042	NTRP	2020-10-01	1.12	1.14	1.0202	1.09	1777788	{}	\N	2020-10-12 15:44:17.678444
60043	NTRP	2020-10-02	1.02	1.13	1.02	1.06	616727	{}	\N	2020-10-12 15:44:17.68549
60044	NTRP	2020-10-05	1.06	1.1	1.06	1.06	214068	{}	\N	2020-10-12 15:44:17.691812
60045	NTRP	2020-10-06	1.07	1.11	1.06	1.08	345837	{}	\N	2020-10-12 15:44:17.698345
60046	NTRP	2020-10-07	1.0607	1.08	1.05	1.05	349984	{}	\N	2020-10-12 15:44:17.703504
60047	NTRP	2020-10-08	1.06	1.11	1.05	1.11	152267	{}	\N	2020-10-12 15:44:17.708944
60048	NTRP	2020-10-09	1.09	1.11	1.07	1.07	89568	{}	\N	2020-10-12 15:44:17.714922
60049	NTRP	2020-10-12	1.06	1.08	1.06	1.0657	34302	{}	\N	2020-10-12 15:44:17.720114
60050	NUAN	2020-09-28	33.54	33.65	33.0499	33.27	2435584	{}	\N	2020-10-12 15:44:18.621738
60051	NUAN	2020-09-29	33.24	33.64	32.98	33.07	3333475	{}	\N	2020-10-12 15:44:18.628199
60052	NUAN	2020-09-30	33.06	33.53	32.79	33.19	2954348	{}	\N	2020-10-12 15:44:18.634035
60053	NUAN	2020-10-01	33.5	34.145	33.33	33.82	3867034	{}	\N	2020-10-12 15:44:18.638793
60054	NUAN	2020-10-02	32.83	33.73	32.39	32.66	1862036	{}	\N	2020-10-12 15:44:18.643936
60055	NUAN	2020-10-05	33	33.6	33	33.45	1525511	{}	\N	2020-10-12 15:44:18.649764
60056	NUAN	2020-10-06	33.42	33.99	33.13	33.46	1290913	{}	\N	2020-10-12 15:44:18.65625
60057	NUAN	2020-10-07	33.73	34.44	33.68	34.02	1695649	{}	\N	2020-10-12 15:44:18.662224
60058	NUAN	2020-10-08	33.83	34.31	33.385	33.8	1868871	{}	\N	2020-10-12 15:44:18.668072
60059	NUAN	2020-10-09	34	34.665	33.91	34.33	1880787	{}	\N	2020-10-12 15:44:18.673209
60060	NUAN	2020-10-12	34.9	35.19	34.22	34.67	665244	{}	\N	2020-10-12 15:44:18.678648
60061	NUMV	2020-09-28	25.4	25.6701	25.4	25.56	21421	{}	\N	2020-10-12 15:44:19.874308
60062	NUMV	2020-09-29	25.51	25.51	25.25	25.36	32483	{}	\N	2020-10-12 15:44:19.880134
60063	NUMV	2020-09-30	25.61	25.73	25.34	25.5	19896	{}	\N	2020-10-12 15:44:19.885797
60064	NUMV	2020-10-01	25.735	25.735	25.44	25.63	22311	{}	\N	2020-10-12 15:44:19.890966
60065	NUMV	2020-10-02	25.41	25.9	25.41	25.9	9602	{}	\N	2020-10-12 15:44:19.896989
60066	NUMV	2020-10-05	26.08	26.33	26.08	26.3	8260	{}	\N	2020-10-12 15:44:19.902981
60067	NUMV	2020-10-06	26.5	26.67	26.13	26.13	6153	{}	\N	2020-10-12 15:44:19.909683
60068	NUMV	2020-10-07	26.45	26.555	26.39	26.555	10544	{}	\N	2020-10-12 15:44:19.91603
60069	NUMV	2020-10-08	26.64	26.92	26.64	26.91	25004	{}	\N	2020-10-12 15:44:19.92168
60070	NUMV	2020-10-09	27.17	27.21	26.93	26.94	85849	{}	\N	2020-10-12 15:44:19.927012
60071	NUMV	2020-10-12	27.12	27.12	26.9691	27.06	9457	{}	\N	2020-10-12 15:44:19.932556
60072	NVCN	2020-09-28	2.13	2.145	2.06	2.12	132687	{}	\N	2020-10-12 15:44:21.427741
60073	NVCN	2020-09-29	2.12	2.175	2.0804	2.12	135934	{}	\N	2020-10-12 15:44:21.435614
60074	NVCN	2020-09-30	2.11	2.2	2.11	2.14	108282	{}	\N	2020-10-12 15:44:21.441283
60075	NVCN	2020-10-01	2.14	2.22	2.14	2.2	116090	{}	\N	2020-10-12 15:44:21.446397
60076	NVCN	2020-10-02	2.15	2.25	2.09	2.14	261527	{}	\N	2020-10-12 15:44:21.453857
60077	NVCN	2020-10-05	2.19	2.23	2.11	2.12	139666	{}	\N	2020-10-12 15:44:21.460325
60078	NVCN	2020-10-06	2.2	2.22	2.14	2.17	105999	{}	\N	2020-10-12 15:44:21.465597
60079	NVCN	2020-10-07	2.19	2.2488	2.17	2.21	206916	{}	\N	2020-10-12 15:44:21.471301
60080	NVCN	2020-10-08	2.18	2.29	2.18	2.27	254171	{}	\N	2020-10-12 15:44:21.476572
60081	NVCN	2020-10-09	2.27	2.41	2.27	2.36	1133455	{}	\N	2020-10-12 15:44:21.481831
60082	NVCN	2020-10-12	2.39	2.41	2.35	2.39	345423	{}	\N	2020-10-12 15:44:21.48876
60083	NHTC	2020-09-28	5.8269	5.8269	5.63	5.71	5064	{}	\N	2020-10-12 15:44:22.708695
60084	NHTC	2020-09-29	5.77	5.8	5.56	5.65	8706	{}	\N	2020-10-12 15:44:22.71443
60085	NHTC	2020-09-30	5.73	5.8577	5.63	5.64	4894	{}	\N	2020-10-12 15:44:22.720133
60086	NHTC	2020-10-01	5.62	5.83	5.62	5.744	11560	{}	\N	2020-10-12 15:44:22.725772
60087	NHTC	2020-10-02	5.7	5.91	5.59	5.88	15422	{}	\N	2020-10-12 15:44:22.731551
60088	NHTC	2020-10-05	5.94	6.1	5.92	5.95	16271	{}	\N	2020-10-12 15:44:22.737623
60089	NHTC	2020-10-06	5.98	6.12	5.935	5.96	14971	{}	\N	2020-10-12 15:44:22.743694
60090	NHTC	2020-10-07	6.09	6.275	5.75	5.9	18246	{}	\N	2020-10-12 15:44:22.748807
60091	NHTC	2020-10-08	5.9	5.9101	5.76	5.8	19551	{}	\N	2020-10-12 15:44:22.754662
60092	NHTC	2020-10-09	5.73	6.2295	5.69	5.7	12749	{}	\N	2020-10-12 15:44:22.7597
60093	NHTC	2020-10-12	5.95	5.95	5.89	5.9	2476	{}	\N	2020-10-12 15:44:22.764729
60094	NIQ	2020-09-28	14.33	14.48	14.33	14.48	11201	{}	\N	2020-10-12 15:44:23.655142
60095	NIQ	2020-09-29	14.38	14.4	14.34	14.37	17332	{}	\N	2020-10-12 15:44:23.661708
60096	NIQ	2020-09-30	14.34	14.41	14.29	14.29	16178	{}	\N	2020-10-12 15:44:23.666962
60097	NIQ	2020-10-01	14.34	14.41	14.34	14.4099	4546	{}	\N	2020-10-12 15:44:23.673399
60098	NIQ	2020-10-02	14.37	14.47	14.3501	14.36	36326	{}	\N	2020-10-12 15:44:23.67899
60099	NIQ	2020-10-05	14.36	14.37	14.05	14.29	49953	{}	\N	2020-10-12 15:44:23.683731
60100	NIQ	2020-10-06	14.383	14.4665	14.332	14.36	70480	{}	\N	2020-10-12 15:44:23.689686
60101	NIQ	2020-10-07	14.37	14.38	14.26	14.331	68502	{}	\N	2020-10-12 15:44:23.694856
60102	NIQ	2020-10-08	14.3698	14.37	14.271	14.32	44808	{}	\N	2020-10-12 15:44:23.699629
60103	NIQ	2020-10-09	14.4558	14.4558	14.36	14.4	6097	{}	\N	2020-10-12 15:44:23.706453
60104	NIQ	2020-10-12	14.4033	14.455	14.35	14.35	22338	{}	\N	2020-10-12 15:44:23.71158
60105	NMR	2020-09-28	4.86	4.89	4.84	4.87	100882	{}	\N	2020-10-12 15:44:25.063122
60106	NMR	2020-09-29	4.8	4.81	4.73	4.73	302514	{}	\N	2020-10-12 15:44:25.068909
60107	NMR	2020-09-30	4.61	4.61	4.56	4.58	270138	{}	\N	2020-10-12 15:44:25.074847
60108	NMR	2020-10-01	4.58	4.6	4.56	4.57	511394	{}	\N	2020-10-12 15:44:25.0809
60109	NMR	2020-10-02	4.52	4.58	4.52	4.56	295716	{}	\N	2020-10-12 15:44:25.085689
60110	NMR	2020-10-05	4.64	4.68	4.63	4.65	343808	{}	\N	2020-10-12 15:44:25.091154
60111	NMR	2020-10-06	4.67	4.67	4.59	4.6	379580	{}	\N	2020-10-12 15:44:25.095969
60112	NMR	2020-10-07	4.67	4.69	4.65	4.69	163177	{}	\N	2020-10-12 15:44:25.100675
60113	NMR	2020-10-08	4.72	4.74	4.69	4.72	165829	{}	\N	2020-10-12 15:44:25.10638
60114	NMR	2020-10-09	4.62	4.63	4.57	4.59	328787	{}	\N	2020-10-12 15:44:25.111248
60115	NMR	2020-10-12	4.63	4.64	4.61	4.635	67098	{}	\N	2020-10-12 15:44:25.1162
60116	NNDM	2020-09-28	1.58	1.62	1.5	1.58	1280003	{}	\N	2020-10-12 15:44:26.082333
60117	NNDM	2020-09-29	1.75	2.23	1.65	2.13	14543553	{}	\N	2020-10-12 15:44:26.090749
60118	NNDM	2020-09-30	2.58	3.28	2.36	2.86	120566550	{}	\N	2020-10-12 15:44:26.097766
60119	NNDM	2020-10-01	2.395	2.47	2.14	2.4	19468344	{}	\N	2020-10-12 15:44:26.104587
60120	NNDM	2020-10-02	2.19	2.63	2.17	2.36	11501367	{}	\N	2020-10-12 15:44:26.112831
60121	NNDM	2020-10-05	2.4	2.47	2.27	2.35	5515491	{}	\N	2020-10-12 15:44:26.12083
60122	NNDM	2020-10-06	2.3	2.32	2.15	2.21	6434933	{}	\N	2020-10-12 15:44:26.128102
60123	NNDM	2020-10-07	2.22	2.5	2.19	2.36	8634848	{}	\N	2020-10-12 15:44:26.134911
60124	NNDM	2020-10-08	2.44	2.46	2.27	2.39	3687668	{}	\N	2020-10-12 15:44:26.14155
60125	NNDM	2020-10-09	2.39	2.63	2.33	2.62	8813264	{}	\N	2020-10-12 15:44:26.147849
60126	NNDM	2020-10-12	2.74	3	2.68	2.935	10508594	{}	\N	2020-10-12 15:44:26.153995
60127	NOVN	2020-09-28	0.495	0.51	0.485301	0.51	1901486	{}	\N	2020-10-12 15:44:27.033049
60128	NOVN	2020-09-29	0.507	0.5079	0.482	0.49	2633435	{}	\N	2020-10-12 15:44:27.04033
60129	NOVN	2020-09-30	0.489	0.4999	0.4755	0.4815	2179444	{}	\N	2020-10-12 15:44:27.046128
60130	NOVN	2020-10-01	0.4855	0.4899	0.452	0.4762	3450805	{}	\N	2020-10-12 15:44:27.051811
60131	NOVN	2020-10-02	0.45	0.4799	0.4444	0.4738	3522240	{}	\N	2020-10-12 15:44:27.058904
60132	NOVN	2020-10-05	0.46	0.4793	0.46	0.4693	2681136	{}	\N	2020-10-12 15:44:27.06497
60133	NOVN	2020-10-06	0.4684	0.4689	0.45	0.4501	3050080	{}	\N	2020-10-12 15:44:27.070779
60134	NOVN	2020-10-07	0.4525	0.4637	0.4326	0.4579	3846132	{}	\N	2020-10-12 15:44:27.077608
60135	NOVN	2020-10-08	0.4385	0.4599	0.4385	0.4471	2363822	{}	\N	2020-10-12 15:44:27.084391
60136	NOVN	2020-10-09	0.4495	0.4499	0.437	0.442	3746733	{}	\N	2020-10-12 15:44:27.090659
60137	NOVN	2020-10-12	0.4473	0.4598	0.44	0.4455	1181036	{}	\N	2020-10-12 15:44:27.09605
60138	NSA	2020-09-28	33.5	33.63	32.87	33.33	668420	{}	\N	2020-10-12 15:44:28.399064
60139	NSA	2020-09-29	33.18	33.56	32.92	33.24	1098971	{}	\N	2020-10-12 15:44:28.405443
60140	NSA	2020-09-30	33.5	33.62	32.485	32.71	1240864	{}	\N	2020-10-12 15:44:28.411727
60141	NSA	2020-10-01	32.77	33.56	32.63	33.45	1050113	{}	\N	2020-10-12 15:44:28.418681
60142	NSA	2020-10-02	33.11	34.44	32.83	34.3	2002187	{}	\N	2020-10-12 15:44:28.424949
60143	NSA	2020-10-05	34.2	34.89	33.55	34.78	666984	{}	\N	2020-10-12 15:44:28.431
60144	NSA	2020-10-06	35.09	35.12	33.93	34.08	1320304	{}	\N	2020-10-12 15:44:28.437401
60145	NSA	2020-10-07	34.22	34.57	34.03	34.35	864742	{}	\N	2020-10-12 15:44:28.443825
60146	NSA	2020-10-08	34.63	34.94	34.29	34.37	717080	{}	\N	2020-10-12 15:44:28.449452
60147	NSA	2020-10-09	34.62	34.81	33.86	34.24	1141020	{}	\N	2020-10-12 15:44:28.455013
60148	NSA	2020-10-12	34.14	34.51	33.82	34.47	110624	{}	\N	2020-10-12 15:44:28.463588
60149	NSYS	2020-09-28	4.53	4.66	4.51	4.572	7988	{}	\N	2020-10-12 15:44:29.409741
60150	NSYS	2020-09-29	4.55	4.74	4.55	4.7	9902	{}	\N	2020-10-12 15:44:29.416129
60151	NSYS	2020-09-30	4.6643	4.69	4.582	4.69	842	{}	\N	2020-10-12 15:44:29.422734
60152	NSYS	2020-10-01	4.5972	4.6	4.51	4.59	4509	{}	\N	2020-10-12 15:44:29.429058
60153	NSYS	2020-10-02	4.41	4.59	4.36	4.4471	12509	{}	\N	2020-10-12 15:44:29.435579
60154	NSYS	2020-10-05	4.387	4.42	4.3715	4.42	8414	{}	\N	2020-10-12 15:44:29.440911
60155	NSYS	2020-10-06	4.5	4.5	4.4102	4.46	4017	{}	\N	2020-10-12 15:44:29.447103
60156	NSYS	2020-10-07	4.5799	4.9997	4.515	4.8999	17302	{}	\N	2020-10-12 15:44:29.452878
60157	NSYS	2020-10-08	4.6872	5.24	4.6872	5.0011	9561	{}	\N	2020-10-12 15:44:29.460163
60158	NSYS	2020-10-09	4.67	5	4.67	4.7563	3030	{}	\N	2020-10-12 15:44:29.467519
60159	NSYS	2020-10-12	5.04	5.54	5	5.0751	15978	{}	\N	2020-10-12 15:44:29.473944
60160	NTIP	2020-09-28	2.55	2.9	2.525	2.66	62952	{}	\N	2020-10-12 15:44:30.740828
60161	NTIP	2020-09-29	2.62	3	2.62	2.6883	38134	{}	\N	2020-10-12 15:44:30.747901
60162	NTIP	2020-09-30	2.8	2.8	2.6	2.65	40475	{}	\N	2020-10-12 15:44:30.755023
60163	NTIP	2020-10-01	2.7699	2.84	2.7257	2.81	36376	{}	\N	2020-10-12 15:44:30.762323
60164	NTIP	2020-10-02	2.7793	2.8	2.71	2.74	8688	{}	\N	2020-10-12 15:44:30.768196
60165	NTIP	2020-10-05	2.7364	2.75	2.7201	2.74	28554	{}	\N	2020-10-12 15:44:30.774933
60166	NTIP	2020-10-06	2.73	2.85	2.7101	2.81	110286	{}	\N	2020-10-12 15:44:30.781215
60167	NTIP	2020-10-07	2.85	3.07	2.85	2.93	112259	{}	\N	2020-10-12 15:44:30.787535
60168	NTIP	2020-10-08	3.0042	3.1	2.83	2.9	51882	{}	\N	2020-10-12 15:44:30.794356
60169	NTIP	2020-10-09	3	3.0499	2.874	2.99	30000	{}	\N	2020-10-12 15:44:30.800349
60170	NTIP	2020-10-12	2.95	2.96	2.9101	2.925	12514	{}	\N	2020-10-12 15:44:30.806322
60171	NTWK	2020-09-28	2.8	2.905	2.74	2.87	48077	{}	\N	2020-10-12 15:44:32.140621
60172	NTWK	2020-09-29	2.83	2.9	2.7	2.9	18217	{}	\N	2020-10-12 15:44:32.147437
60173	NTWK	2020-09-30	2.9	2.95	2.8	2.93	23839	{}	\N	2020-10-12 15:44:32.153154
60174	NTWK	2020-10-01	2.9	2.93	2.855	2.9	15546	{}	\N	2020-10-12 15:44:32.159348
60175	NTWK	2020-10-02	2.83	2.95	2.815	2.95	29329	{}	\N	2020-10-12 15:44:32.165925
60176	NTWK	2020-10-05	2.93	3.14	2.9255	3.05	52995	{}	\N	2020-10-12 15:44:32.173058
60177	NTWK	2020-10-06	3.05	3.0501	2.95	2.96	20256	{}	\N	2020-10-12 15:44:32.180527
60178	NTWK	2020-10-07	2.977	3.04	2.885	3.02	27488	{}	\N	2020-10-12 15:44:32.186263
60179	NTWK	2020-10-08	3.058	3.065	2.86	2.91	23589	{}	\N	2020-10-12 15:44:32.192334
60180	NTWK	2020-10-09	2.8857	2.97	2.8341	2.92	18100	{}	\N	2020-10-12 15:44:32.198791
60181	NTWK	2020-10-12	2.91	3	2.91	2.995	8742	{}	\N	2020-10-12 15:44:32.204159
60182	NUDM	2020-09-28	25.62	25.6999	25.4801	25.62	24721	{}	\N	2020-10-12 15:44:33.330184
60183	NUDM	2020-09-29	25.65	25.72	25.53	25.56	8786	{}	\N	2020-10-12 15:44:33.337987
60184	NUDM	2020-09-30	25.58	25.74	25.57	25.57	3010	{}	\N	2020-10-12 15:44:33.343884
60185	NUDM	2020-10-01	25.63	25.7	25.5	25.6232	23167	{}	\N	2020-10-12 15:44:33.350947
60186	NUDM	2020-10-02	25.506	25.63	25.48	25.605	3265	{}	\N	2020-10-12 15:44:33.356969
60187	NUDM	2020-10-05	25.845	25.979	25.8341	25.955	29564	{}	\N	2020-10-12 15:44:33.362312
60188	NUDM	2020-10-06	25.97	26.0599	25.74	25.76	20449	{}	\N	2020-10-12 15:44:33.368103
60189	NUDM	2020-10-07	25.89	25.96	25.89	25.955	7084	{}	\N	2020-10-12 15:44:33.373098
60190	NUDM	2020-10-08	25.9832	26.11	25.9661	26.11	4187	{}	\N	2020-10-12 15:44:33.378958
60191	NUDM	2020-10-09	26.2699	26.2899	26.1301	26.2104	6369	{}	\N	2020-10-12 15:44:33.387996
60192	NUDM	2020-10-12	26.19	26.347	26.19	26.347	3417	{}	\N	2020-10-12 15:44:33.393801
60193	NWPX	2020-09-28	25.75	29.29	25.75	29.01	135416	{}	\N	2020-10-12 15:44:34.723749
60194	NWPX	2020-09-29	28.93	28.93	26.87	27.745	36969	{}	\N	2020-10-12 15:44:34.729611
60195	NWPX	2020-09-30	27.65	28.085	26.1	26.46	24408	{}	\N	2020-10-12 15:44:34.735611
60196	NWPX	2020-10-01	26.85	27.52	26.85	27.03	39469	{}	\N	2020-10-12 15:44:34.740775
60197	NWPX	2020-10-02	26.75	27.855	26.75	27.61	12691	{}	\N	2020-10-12 15:44:34.746246
60198	NWPX	2020-10-05	26.5	28.37	26.5	27.45	43413	{}	\N	2020-10-12 15:44:34.753143
60199	NWPX	2020-10-06	27.31	28.13	26.94	26.94	29910	{}	\N	2020-10-12 15:44:34.759365
60200	NWPX	2020-10-07	27.55	28.33	27.55	28.24	24756	{}	\N	2020-10-12 15:44:34.764449
60201	NWPX	2020-10-08	28.24	28.94	27.905	28.55	22189	{}	\N	2020-10-12 15:44:34.770032
60202	NWPX	2020-10-09	28.91	28.98	27.9	28.37	14323	{}	\N	2020-10-12 15:44:34.775098
60203	NWPX	2020-10-12	28.57	31.14	28.485	30.97	54643	{}	\N	2020-10-12 15:44:34.779821
60204	NLSN	2020-09-28	14.12	14.255	14.015	14.16	3063885	{}	\N	2020-10-12 15:44:35.734643
60205	NLSN	2020-09-29	14.12	14.275	13.92	14.01	3076541	{}	\N	2020-10-12 15:44:35.746329
60206	NLSN	2020-09-30	14.15	14.57	14.08	14.18	6574316	{}	\N	2020-10-12 15:44:35.751957
60207	NLSN	2020-10-01	14.26	14.47	14.02	14.18	4216816	{}	\N	2020-10-12 15:44:35.757956
60208	NLSN	2020-10-02	13.99	14.37	13.87	14.29	4885428	{}	\N	2020-10-12 15:44:35.763755
60209	NLSN	2020-10-05	14.44	14.54	14.29	14.51	4387831	{}	\N	2020-10-12 15:44:35.769416
60210	NLSN	2020-10-06	14.67	14.785	14.16	14.26	4911552	{}	\N	2020-10-12 15:44:35.774409
60211	NLSN	2020-10-07	14.22	14.5	14.21	14.41	5433054	{}	\N	2020-10-12 15:44:35.780049
60212	NLSN	2020-10-08	14.54	14.6	14.35	14.44	5479054	{}	\N	2020-10-12 15:44:35.786598
60213	NLSN	2020-10-09	14.58	14.597	13.66	13.75	12056933	{}	\N	2020-10-12 15:44:35.791686
60214	NLSN	2020-10-12	13.81	13.99	13.71	13.9	4292469	{}	\N	2020-10-12 15:44:35.796956
60215	NMRD	2020-09-28	3.78	4.25	3.65	3.8	179464	{}	\N	2020-10-12 15:44:36.936254
60216	NMRD	2020-09-29	3.67	3.73	3.465	3.63	80252	{}	\N	2020-10-12 15:44:36.942061
60217	NMRD	2020-09-30	3.58	3.68	3.5	3.57	15717	{}	\N	2020-10-12 15:44:36.94692
60218	NMRD	2020-10-01	3.64	3.66	3.4	3.47	125025	{}	\N	2020-10-12 15:44:36.952474
60219	NMRD	2020-10-02	3.41	3.76	3.41	3.55	34880	{}	\N	2020-10-12 15:44:36.957935
60220	NMRD	2020-10-05	3.61	3.84	3.57	3.66	28451	{}	\N	2020-10-12 15:44:36.965141
60221	NMRD	2020-10-06	3.78	3.85	3.71	3.71	10201	{}	\N	2020-10-12 15:44:36.97397
60222	NMRD	2020-10-07	3.72	4.17	3.72	3.8	78063	{}	\N	2020-10-12 15:44:36.981676
60223	NMRD	2020-10-08	3.8	3.98	3.78	3.78	38303	{}	\N	2020-10-12 15:44:36.992008
60224	NMRD	2020-10-09	3.76	3.96	3.63	3.69	120448	{}	\N	2020-10-12 15:44:36.99912
60225	NMRD	2020-10-12	3.67	3.86	3.67	3.86	828	{}	\N	2020-10-12 15:44:37.005544
60226	NNBR	2020-09-28	5.41	5.66	5.38	5.55	115477	{}	\N	2020-10-12 15:44:38.217214
60227	NNBR	2020-09-29	5.54	5.54	5.335	5.44	89237	{}	\N	2020-10-12 15:44:38.225921
60228	NNBR	2020-09-30	5.5	5.72	5.08	5.16	249005	{}	\N	2020-10-12 15:44:38.231492
60229	NNBR	2020-10-01	5.19	5.54	5.09	5.51	186992	{}	\N	2020-10-12 15:44:38.238111
60230	NNBR	2020-10-02	5.27	5.97	5.1	5.78	363457	{}	\N	2020-10-12 15:44:38.244026
60231	NNBR	2020-10-05	5.8895	6.09	5.86	6.03	219146	{}	\N	2020-10-12 15:44:38.249604
60232	NNBR	2020-10-06	6.15	6.25	5.8	5.83	351977	{}	\N	2020-10-12 15:44:38.263224
60233	NNBR	2020-10-07	5.9	6.5	5.9	6.3	320935	{}	\N	2020-10-12 15:44:38.27179
60234	NNBR	2020-10-08	6.44	6.68	6.3	6.5	241958	{}	\N	2020-10-12 15:44:38.279817
60235	NNBR	2020-10-09	6.54	6.6258	6.29	6.38	120548	{}	\N	2020-10-12 15:44:38.285172
60236	NNBR	2020-10-12	6.39	6.5	6.27	6.43	24020	{}	\N	2020-10-12 15:44:38.290436
60237	NOG	2020-09-28	5.34	5.45	5.16	5.42	456355	{}	\N	2020-10-12 15:44:40.063654
60238	NOG	2020-09-29	5.41	5.61	5.265	5.6	506116	{}	\N	2020-10-12 15:44:40.068634
60239	NOG	2020-09-30	5.53	5.74	5.4975	5.74	540324	{}	\N	2020-10-12 15:44:40.073896
60240	NOG	2020-10-01	5.66	5.68	5.3	5.41	557847	{}	\N	2020-10-12 15:44:40.078847
60241	NOG	2020-10-02	5.3	5.49	5.24	5.27	437247	{}	\N	2020-10-12 15:44:40.084009
60242	NOG	2020-10-05	5.45	5.45	5.14	5.21	622012	{}	\N	2020-10-12 15:44:40.090597
60243	NOG	2020-10-06	5.27	5.47	5.18	5.29	702890	{}	\N	2020-10-12 15:44:40.096438
60244	NOG	2020-10-07	5.3	5.44	5.2	5.37	567343	{}	\N	2020-10-12 15:44:40.101827
60245	NOG	2020-10-08	5.5	6.49	5.49	6.32	2217223	{}	\N	2020-10-12 15:44:40.107835
60246	NOG	2020-10-09	6.37	6.39	5.75	5.91	1141464	{}	\N	2020-10-12 15:44:40.112884
60247	NOG	2020-10-12	5.88	5.91	5.675	5.6801	329670	{}	\N	2020-10-12 15:44:40.117961
60248	NPN	2020-09-29	15.02	15.02	14.7617	14.7617	4220	{}	\N	2020-10-12 15:44:41.186016
60249	NPN	2020-10-01	15.05	15.05	15.05	15.05	778	{}	\N	2020-10-12 15:44:41.192237
60250	NPN	2020-10-02	15.05	15.05	14.8025	14.8025	1334	{}	\N	2020-10-12 15:44:41.197632
60251	NPN	2020-10-05	14.65	14.8207	14.42	14.8207	6304	{}	\N	2020-10-12 15:44:41.20299
60252	NPN	2020-10-06	14.84	14.84	14.51	14.5984	6522	{}	\N	2020-10-12 15:44:41.208925
60253	NPN	2020-10-07	14.58	14.58	14.58	14.58	1454	{}	\N	2020-10-12 15:44:41.214927
60254	NPN	2020-10-09	14.5	14.5	14.5	14.5	668	{}	\N	2020-10-12 15:44:41.222833
60255	NRT	2020-09-28	2.9	3.01	2.9	2.97	27554	{}	\N	2020-10-12 15:44:42.536233
60256	NRT	2020-09-29	2.86	2.99	2.86	2.95	24268	{}	\N	2020-10-12 15:44:42.54331
60257	NRT	2020-09-30	2.96	3	2.94	2.95	22724	{}	\N	2020-10-12 15:44:42.548447
60258	NRT	2020-10-01	2.9398	3.01	2.9201	3.01	33590	{}	\N	2020-10-12 15:44:42.553596
60259	NRT	2020-10-02	3.01	3.01	2.95	3.01	62174	{}	\N	2020-10-12 15:44:42.559931
60260	NRT	2020-10-05	3.01	3.0105	2.96	2.97	71512	{}	\N	2020-10-12 15:44:42.566681
60261	NRT	2020-10-06	2.985	3.14	2.98	3.13	69250	{}	\N	2020-10-12 15:44:42.571899
60262	NRT	2020-10-07	3.105	3.105	3	3.06	34146	{}	\N	2020-10-12 15:44:42.57734
60263	NRT	2020-10-08	3.09	3.18	3.0558	3.18	18950	{}	\N	2020-10-12 15:44:42.582544
60264	NRT	2020-10-09	3.12	3.29	3.11	3.19	97872	{}	\N	2020-10-12 15:44:42.587768
60265	NRT	2020-10-12	3.16	3.18	3.0962	3.1303	25692	{}	\N	2020-10-12 15:44:42.594001
60266	NSS	2020-09-28	19.75	20	19.75	19.8423	31200	{}	\N	2020-10-12 15:44:43.634722
60267	NSS	2020-09-29	19.77	19.9222	19.41	19.48	69256	{}	\N	2020-10-12 15:44:43.639911
60268	NSS	2020-09-30	19.12	19.42	19.12	19.14	170173	{}	\N	2020-10-12 15:44:43.648641
60269	NSS	2020-10-01	19.17	19.35	18.8	19.31	62967	{}	\N	2020-10-12 15:44:43.654181
60270	NSS	2020-10-02	18.93	19.58	18.93	19.48	41437	{}	\N	2020-10-12 15:44:43.660104
60271	NSS	2020-10-05	19.6	19.98	19.3399	19.55	64208	{}	\N	2020-10-12 15:44:43.666328
60272	NSS	2020-10-06	19.66	19.8186	19.45	19.67	48374	{}	\N	2020-10-12 15:44:43.671836
60273	NSS	2020-10-07	19.71	19.82	19.5	19.62	71230	{}	\N	2020-10-12 15:44:43.677929
60274	NSS	2020-10-08	19.5	19.92	19.5	19.92	64118	{}	\N	2020-10-12 15:44:43.682983
60275	NSS	2020-10-09	19.99	20.1699	19.76	19.96	78467	{}	\N	2020-10-12 15:44:43.688851
60276	NSS	2020-10-12	19.95	20.15	19.8066	20.15	50054	{}	\N	2020-10-12 15:44:43.69459
60277	NTES	2020-09-28	94.1747	94.1747	91.48	92.788	2485050	{}	\N	2020-10-12 15:44:45.098873
60278	NTES	2020-09-29	92.8	92.89	89.93	90.152	2733045	{}	\N	2020-10-12 15:44:45.103879
60279	NTES	2020-09-30	89.804	91.5164	88.825	90.934	2144480	{}	\N	2020-10-12 15:44:45.108308
60280	NTES	2020-10-01	91.81	93.209	90.964	92.718	1594195	{}	\N	2020-10-12 15:44:45.113677
60281	NTES	2020-10-02	91.28	92.09	88.12	89.1	1513694	{}	\N	2020-10-12 15:44:45.118559
60282	NTES	2020-10-05	89.8	90.95	89.16	90.62	1481516	{}	\N	2020-10-12 15:44:45.124301
60283	NTES	2020-10-06	90	91.1	89.25	89.39	1579742	{}	\N	2020-10-12 15:44:45.129992
60284	NTES	2020-10-07	89.6	89.99	86.1	86.94	3039381	{}	\N	2020-10-12 15:44:45.134853
60285	NTES	2020-10-08	86.5	88.94	85.04	88.42	2608365	{}	\N	2020-10-12 15:44:45.141022
60286	NTES	2020-10-09	89.3	91.09	89.03	90.6	1681468	{}	\N	2020-10-12 15:44:45.148668
60287	NTES	2020-10-12	91.79	92.88	90.85	91.31	734688	{}	\N	2020-10-12 15:44:45.153255
60288	NTRA	2020-09-28	70.16	70.97	68.56	70.05	870443	{}	\N	2020-10-12 15:44:46.042691
60289	NTRA	2020-09-29	69.59	73.99	69.44	72.67	776801	{}	\N	2020-10-12 15:44:46.049052
60290	NTRA	2020-09-30	72.46	73.92	71.12	72.24	604368	{}	\N	2020-10-12 15:44:46.05365
60291	NTRA	2020-10-01	73.26	73.59	71.81	72.89	454272	{}	\N	2020-10-12 15:44:46.059719
60292	NTRA	2020-10-02	72.13	73.26	70.145	70.78	541761	{}	\N	2020-10-12 15:44:46.06533
60293	NTRA	2020-10-05	71.24	72.29	69.43	70.72	712110	{}	\N	2020-10-12 15:44:46.070342
60294	NTRA	2020-10-06	70.99	73.58	69.92	71.41	940545	{}	\N	2020-10-12 15:44:46.074948
60295	NTRA	2020-10-07	71.92	74.47	70.35	72.56	844709	{}	\N	2020-10-12 15:44:46.079841
60296	NTRA	2020-10-08	74.26	74.43	70.8	71.4	643520	{}	\N	2020-10-12 15:44:46.084345
60297	NTRA	2020-10-09	71.49	74.17	70.54	73.58	420183	{}	\N	2020-10-12 15:44:46.08881
60298	NTRA	2020-10-12	74.5	75	73.4087	73.83	111578	{}	\N	2020-10-12 15:44:46.094607
60299	NULV	2020-09-28	28.8	29.17	28.8	29	60050	{}	\N	2020-10-12 15:44:47.231019
60300	NULV	2020-09-29	28.96	29.05	28.73	28.8863	40548	{}	\N	2020-10-12 15:44:47.236113
60301	NULV	2020-09-30	29.045	29.2599	28.87	29.12	35957	{}	\N	2020-10-12 15:44:47.240882
60302	NULV	2020-10-01	29.24	29.26	28.895	29.03	55684	{}	\N	2020-10-12 15:44:47.246979
60303	NULV	2020-10-02	28.74	29.2583	28.74	29.18	47713	{}	\N	2020-10-12 15:44:47.252443
60304	NULV	2020-10-05	29.41	29.58	29.38	29.57	45583	{}	\N	2020-10-12 15:44:47.257775
60305	NULV	2020-10-06	29.57	29.894	29.2878	29.33	70240	{}	\N	2020-10-12 15:44:47.263667
60306	NULV	2020-10-07	29.6384	29.88	29.62	29.84	45798	{}	\N	2020-10-12 15:44:47.278098
60307	NULV	2020-10-08	30.05	30.18	30.02	30.18	188985	{}	\N	2020-10-12 15:44:47.285552
60308	NULV	2020-10-09	30.39	30.39	30.11	30.22	251506	{}	\N	2020-10-12 15:44:47.291672
60309	NULV	2020-10-12	30.24	30.4398	30.24	30.42	15110	{}	\N	2020-10-12 15:44:47.296738
60310	NUSA	2020-09-28	25.565	25.566	25.53	25.5408	4471	{}	\N	2020-10-12 15:44:48.289389
60311	NUSA	2020-09-29	25.54	25.5699	25.53	25.5691	19646	{}	\N	2020-10-12 15:44:48.295628
60312	NUSA	2020-09-30	25.53	25.5544	25.53	25.53	3997	{}	\N	2020-10-12 15:44:48.300892
60313	NUSA	2020-10-01	25.56	25.56	25.49	25.49	18608	{}	\N	2020-10-12 15:44:48.306898
60314	NUSA	2020-10-02	25.59	25.59	25.4	25.54	13779	{}	\N	2020-10-12 15:44:48.312763
60315	NUSA	2020-10-05	25.57	25.57	25.51	25.53	2696	{}	\N	2020-10-12 15:44:48.319105
60316	NUSA	2020-10-06	25.5699	25.67	25.5134	25.57	35990	{}	\N	2020-10-12 15:44:48.325155
60317	NUSA	2020-10-07	25.535	25.559	25.485	25.51	8474	{}	\N	2020-10-12 15:44:48.331653
60318	NUSA	2020-10-08	25.48	25.5401	25.48	25.5067	16004	{}	\N	2020-10-12 15:44:48.337316
60319	NUSA	2020-10-09	25.49	25.53	25.49	25.49	1487	{}	\N	2020-10-12 15:44:48.342307
60320	NUSA	2020-10-12	25.58	25.58	25.58	25.58	640	{}	\N	2020-10-12 15:44:48.348177
60321	NVFY	2020-09-28	1.64	1.64	1.59	1.6213	4065	{}	\N	2020-10-12 15:44:49.256331
60322	NVFY	2020-09-29	1.64	1.65	1.5815	1.61	17257	{}	\N	2020-10-12 15:44:49.261983
60323	NVFY	2020-09-30	1.6117	1.68	1.61	1.65	18582	{}	\N	2020-10-12 15:44:49.266722
60324	NVFY	2020-10-01	1.62	1.67	1.61	1.62	11814	{}	\N	2020-10-12 15:44:49.283027
60325	NVFY	2020-10-02	1.6449	1.66	1.62	1.66	3534	{}	\N	2020-10-12 15:44:49.291433
60326	NVFY	2020-10-05	1.68	1.68	1.6	1.62	5168	{}	\N	2020-10-12 15:44:49.299302
60327	NVFY	2020-10-06	1.62	1.7	1.6	1.69	20486	{}	\N	2020-10-12 15:44:49.305549
60328	NVFY	2020-10-07	1.69	1.87	1.69	1.814	60899	{}	\N	2020-10-12 15:44:49.310639
60329	NVFY	2020-10-08	1.87	1.87	1.76	1.8	32032	{}	\N	2020-10-12 15:44:49.317711
60330	NVFY	2020-10-09	1.76	1.93	1.7	1.7	159512	{}	\N	2020-10-12 15:44:49.323032
60331	NVFY	2020-10-12	1.73	1.8266	1.73	1.8084	20596	{}	\N	2020-10-12 15:44:49.327902
60332	NWN	2020-09-28	44.3	44.87	43.89	43.94	169828	{}	\N	2020-10-12 15:44:50.629801
60333	NWN	2020-09-29	44.14	44.45	43.37	43.69	394735	{}	\N	2020-10-12 15:44:50.635589
60334	NWN	2020-09-30	44.22	45.48	44.09	45.39	723160	{}	\N	2020-10-12 15:44:50.640296
60335	NWN	2020-10-01	45.14	45.611	44.34	44.77	383959	{}	\N	2020-10-12 15:44:50.64644
60336	NWN	2020-10-02	44.32	45.76	43.96	45.48	339635	{}	\N	2020-10-12 15:44:50.653132
60337	NWN	2020-10-05	45.62	45.62	44.98	45.3	229072	{}	\N	2020-10-12 15:44:50.659609
60338	NWN	2020-10-06	45.81	47.33	45.35	46.48	463136	{}	\N	2020-10-12 15:44:50.664507
60339	NWN	2020-10-07	46.41	46.63	45.81	46.3	332471	{}	\N	2020-10-12 15:44:50.670002
60340	NWN	2020-10-08	46.62	47.09	46.47	46.89	265612	{}	\N	2020-10-12 15:44:50.674826
60341	NWN	2020-10-09	47.5	47.814	46.42	46.91	260520	{}	\N	2020-10-12 15:44:50.679756
60342	NWN	2020-10-12	46.56	47.08	46.475	46.85	58471	{}	\N	2020-10-12 15:44:50.685881
60343	NKX	2020-09-28	15.28	15.35	15.21	15.28	40397	{}	\N	2020-10-12 15:44:52.069742
60344	NKX	2020-09-29	15.33	15.34	15.2185	15.28	80984	{}	\N	2020-10-12 15:44:52.07434
60345	NKX	2020-09-30	15.27	15.2938	15.23	15.27	70806	{}	\N	2020-10-12 15:44:52.078671
60346	NKX	2020-10-01	15.34	15.34	15.24	15.29	98463	{}	\N	2020-10-12 15:44:52.08276
60347	NKX	2020-10-02	15.29	15.37	15.21	15.355	135864	{}	\N	2020-10-12 15:44:52.087454
60348	NKX	2020-10-05	15.35	15.35	15.27	15.33	69418	{}	\N	2020-10-12 15:44:52.092818
60349	NKX	2020-10-06	15.33	15.33	15.27	15.27	96136	{}	\N	2020-10-12 15:44:52.098364
60350	NKX	2020-10-07	15.37	15.39	15.27	15.32	77276	{}	\N	2020-10-12 15:44:52.103545
60351	NKX	2020-10-08	15.36	15.37	15.3	15.35	84284	{}	\N	2020-10-12 15:44:52.109132
60352	NKX	2020-10-09	15.37	15.38	15.26	15.31	78480	{}	\N	2020-10-12 15:44:52.113841
60353	NKX	2020-10-12	15.4	15.4	15.272	15.272	14876	{}	\N	2020-10-12 15:44:52.118903
60354	NMRK	2020-09-28	4.26	4.44	4.225	4.32	842791	{}	\N	2020-10-12 15:44:54.115598
60355	NMRK	2020-09-29	4.32	4.37	4.27	4.29	463149	{}	\N	2020-10-12 15:44:54.122036
60356	NMRK	2020-09-30	4.29	4.38	4.275	4.32	879039	{}	\N	2020-10-12 15:44:54.126778
60357	NMRK	2020-10-01	4.34	4.63	4.34	4.62	1397100	{}	\N	2020-10-12 15:44:54.131387
60358	NMRK	2020-10-02	4.37	4.725	4.361	4.65	1223831	{}	\N	2020-10-12 15:44:54.137349
60359	NMRK	2020-10-05	4.6	4.75	4.53	4.65	857986	{}	\N	2020-10-12 15:44:54.143311
60360	NMRK	2020-10-06	4.7	5.32	4.565	4.92	4457661	{}	\N	2020-10-12 15:44:54.148467
60361	NMRK	2020-10-07	4.94	5	4.76	4.79	1503830	{}	\N	2020-10-12 15:44:54.155433
60362	NMRK	2020-10-08	4.87	5.13	4.87	5	2082147	{}	\N	2020-10-12 15:44:54.160876
60363	NMRK	2020-10-09	5.08	5.145	4.98	4.99	1504994	{}	\N	2020-10-12 15:44:54.166017
60364	NMRK	2020-10-12	4.99	5.05	4.905	4.99	199453	{}	\N	2020-10-12 15:44:54.171279
60365	NNVC	2020-09-28	4.06	4.06	3.73	3.77	243630	{}	\N	2020-10-12 15:44:55.438504
60366	NNVC	2020-09-29	3.72	4.09	3.72	3.96	217893	{}	\N	2020-10-12 15:44:55.445117
60367	NNVC	2020-09-30	3.9463	3.98	3.815	3.84	136097	{}	\N	2020-10-12 15:44:55.449968
60368	NNVC	2020-10-01	3.82	3.92	3.75	3.81	199698	{}	\N	2020-10-12 15:44:55.454824
60369	NNVC	2020-10-02	3.8	3.95	3.75	3.83	182486	{}	\N	2020-10-12 15:44:55.461184
60370	NNVC	2020-10-05	3.9	3.91	3.82	3.82	106220	{}	\N	2020-10-12 15:44:55.466561
60371	NNVC	2020-10-06	3.87	3.9846	3.8	3.86	104082	{}	\N	2020-10-12 15:44:55.472418
60372	NNVC	2020-10-07	3.86	3.92	3.82	3.88	72497	{}	\N	2020-10-12 15:44:55.477747
60373	NNVC	2020-10-08	3.84	3.94	3.84	3.88	91091	{}	\N	2020-10-12 15:44:55.484905
60374	NNVC	2020-10-09	3.95	4.2	3.89	4.08	347325	{}	\N	2020-10-12 15:44:55.490078
60375	NNVC	2020-10-12	4.04	4.11	4.03	4.07	46037	{}	\N	2020-10-12 15:44:55.495247
60376	NPK	2020-09-28	85.11	85.43	83.4	83.4	13907	{}	\N	2020-10-12 15:44:56.411165
60377	NPK	2020-09-29	83.4	83.4	81.09	81.09	42500	{}	\N	2020-10-12 15:44:56.41647
60378	NPK	2020-09-30	81.69	82.8	81.1	81.86	65472	{}	\N	2020-10-12 15:44:56.42128
60379	NPK	2020-10-01	81.7	82.47	80.04	80.04	42106	{}	\N	2020-10-12 15:44:56.427056
60380	NPK	2020-10-02	79.49	81.76	79.49	80	44424	{}	\N	2020-10-12 15:44:56.432151
60381	NPK	2020-10-05	80.64	82.73	80.64	82.06	40236	{}	\N	2020-10-12 15:44:56.43784
60382	NPK	2020-10-06	82.66	84.48	82.22	82.53	61242	{}	\N	2020-10-12 15:44:56.444986
60383	NPK	2020-10-07	83.18	83.18	81.81	82.07	28776	{}	\N	2020-10-12 15:44:56.450078
60384	NPK	2020-10-08	82.74	84.3499	82.5274	83.68	28236	{}	\N	2020-10-12 15:44:56.454632
60385	NPK	2020-10-09	84.21	84.275	83.49	83.93	36302	{}	\N	2020-10-12 15:44:56.460146
60386	NPK	2020-10-12	83.88	84.66	83.88	84.66	7522	{}	\N	2020-10-12 15:44:56.465212
60387	NRP	2020-09-28	12.28	12.31	12.161	12.31	5817	{}	\N	2020-10-12 15:44:57.412833
60388	NRP	2020-09-29	12.25	12.25	11.68	11.84	18240	{}	\N	2020-10-12 15:44:57.41918
60389	NRP	2020-09-30	11.75	12	11.75	11.92	12229	{}	\N	2020-10-12 15:44:57.425406
60390	NRP	2020-10-01	11.93	11.99	11.54	11.98	20104	{}	\N	2020-10-12 15:44:57.433419
60391	NRP	2020-10-02	11.688	12.04	11.688	12.04	10228	{}	\N	2020-10-12 15:44:57.43995
60392	NRP	2020-10-05	12.05	12.05	11.86	12.02	16786	{}	\N	2020-10-12 15:44:57.446726
60393	NRP	2020-10-06	11.7201	12	11.7201	11.99	8502	{}	\N	2020-10-12 15:44:57.453794
60394	NRP	2020-10-07	11.77	11.955	11.7	11.7	7843	{}	\N	2020-10-12 15:44:57.45978
60395	NRP	2020-10-08	11.83	11.99	11.75	11.75	20462	{}	\N	2020-10-12 15:44:57.46514
60396	NRP	2020-10-09	11.7	11.77	11.61	11.75	13983	{}	\N	2020-10-12 15:44:57.470198
60397	NRP	2020-10-12	11.76	11.76	11.72	11.75	4154	{}	\N	2020-10-12 15:44:57.475848
60398	NSPR	2020-09-28	0.3128	0.315	0.3001	0.309667	496783	{}	\N	2020-10-12 15:44:58.436517
60399	NSPR	2020-09-29	0.31	0.3198	0.2999	0.3145	367837	{}	\N	2020-10-12 15:44:58.443272
60400	NSPR	2020-09-30	0.3112	0.3649	0.31	0.3173	1267189	{}	\N	2020-10-12 15:44:58.448887
60401	NSPR	2020-10-01	0.3154	0.345	0.31	0.3318	431928	{}	\N	2020-10-12 15:44:58.453211
60402	NSPR	2020-10-02	0.33	0.379	0.3203	0.3319	1832399	{}	\N	2020-10-12 15:44:58.458646
60403	NSPR	2020-10-05	0.3415	0.344	0.3244	0.335	469932	{}	\N	2020-10-12 15:44:58.463683
60404	NSPR	2020-10-06	0.33	0.35	0.325	0.3413	763567	{}	\N	2020-10-12 15:44:58.467744
60405	NSPR	2020-10-07	0.3313	0.34	0.3281	0.3332	286531	{}	\N	2020-10-12 15:44:58.472023
60406	NSPR	2020-10-08	0.335	0.3372	0.3206	0.3222	626188	{}	\N	2020-10-12 15:44:58.477878
60407	NSPR	2020-10-09	0.3294	0.3325	0.3202	0.3251	329320	{}	\N	2020-10-12 15:44:58.4829
60408	NSPR	2020-10-12	0.3325	0.3419	0.3174	0.3312	230317	{}	\N	2020-10-12 15:44:58.488541
60409	NTCT	2020-09-28	21.52	21.86	21.135	21.815	481524	{}	\N	2020-10-12 15:44:59.454997
60410	NTCT	2020-09-29	21.91	22.175	21.77	21.97	401658	{}	\N	2020-10-12 15:44:59.46204
60411	NTCT	2020-09-30	22.08	22.08	21.59	21.83	321385	{}	\N	2020-10-12 15:44:59.467821
60412	NTCT	2020-10-01	21.84	22.12	21.77	21.92	271012	{}	\N	2020-10-12 15:44:59.473034
60413	NTCT	2020-10-02	21.62	22.2	21.42	22.01	304607	{}	\N	2020-10-12 15:44:59.479208
60414	NTCT	2020-10-05	22.19	22.63	21.95	22.61	188803	{}	\N	2020-10-12 15:44:59.486723
60415	NTCT	2020-10-06	22.69	23.32	22.61	22.75	275087	{}	\N	2020-10-12 15:44:59.492368
60416	NTCT	2020-10-07	22.86	23.23	22.575	23.13	285769	{}	\N	2020-10-12 15:44:59.498348
60417	NTCT	2020-10-08	23.4	23.56	23.17	23.51	196127	{}	\N	2020-10-12 15:44:59.506432
60418	NTCT	2020-10-09	23.73	23.86	23.27	23.59	174658	{}	\N	2020-10-12 15:44:59.511673
60419	NTCT	2020-10-12	23.71	23.94	23.58	23.77	58988	{}	\N	2020-10-12 15:44:59.517142
60420	NTR	2020-09-28	40.29	40.79	39.86	39.89	842790	{}	\N	2020-10-12 15:45:00.785695
60421	NTR	2020-09-29	39.66	39.68	38.36	38.47	1817445	{}	\N	2020-10-12 15:45:00.790449
60422	NTR	2020-09-30	38.74	39.625	38.61	39.23	2248464	{}	\N	2020-10-12 15:45:00.795434
60423	NTR	2020-10-01	39.16	39.23	37.92	38.21	3137488	{}	\N	2020-10-12 15:45:00.800232
60424	NTR	2020-10-02	37.55	38.87	37.51	38.65	2140512	{}	\N	2020-10-12 15:45:00.805633
60425	NTR	2020-10-05	39.04	40.98	39.04	40.96	4856671	{}	\N	2020-10-12 15:45:00.811536
60426	NTR	2020-10-06	41.25	41.37	39.92	40.1	2259448	{}	\N	2020-10-12 15:45:00.816724
60427	NTR	2020-10-07	40.67	40.82	40.13	40.4	1774647	{}	\N	2020-10-12 15:45:00.823068
60428	NTR	2020-10-08	40.7	40.79	40.15	40.76	1731976	{}	\N	2020-10-12 15:45:00.828775
60429	NTR	2020-10-09	40.97	41.29	40.63	41.01	2100753	{}	\N	2020-10-12 15:45:00.833969
60430	NTR	2020-10-12	41.01	41.1	40.7005	41.0216	387177	{}	\N	2020-10-12 15:45:00.840069
60431	NTZ	2020-09-28	1.48	1.7235	1.48	1.6499	112460	{}	\N	2020-10-12 15:45:01.724002
60432	NTZ	2020-09-29	1.62	1.68	1.62	1.6652	158944	{}	\N	2020-10-12 15:45:01.73043
60433	NTZ	2020-09-30	1.7	1.74	1.63	1.7	583362	{}	\N	2020-10-12 15:45:01.73603
60434	NTZ	2020-10-01	1.7105	1.8896	1.7105	1.8501	82708	{}	\N	2020-10-12 15:45:01.743953
60435	NTZ	2020-10-02	1.81	1.94	1.805	1.9	175598	{}	\N	2020-10-12 15:45:01.751028
60436	NTZ	2020-10-05	1.96	2.29	1.9365	2.26	682356	{}	\N	2020-10-12 15:45:01.756733
60437	NTZ	2020-10-06	2.26	2.2899	2.08	2.25	405164	{}	\N	2020-10-12 15:45:01.76238
60438	NTZ	2020-10-07	2.15	2.8	2.15	2.79	884360	{}	\N	2020-10-12 15:45:01.768273
60439	NTZ	2020-10-08	2.75	3.0699	2.6	2.77	478663	{}	\N	2020-10-12 15:45:01.774011
60440	NTZ	2020-10-09	2.79	3.1899	2.49	3.1	570898	{}	\N	2020-10-12 15:45:01.779416
60441	NTZ	2020-10-12	3.1	3.39	3.1	3.275	178589	{}	\N	2020-10-12 15:45:01.78639
60442	NUMG	2020-09-28	42.13	42.21	41.89	42.08	22928	{}	\N	2020-10-12 15:45:04.136387
60443	NUMG	2020-09-29	42.29	42.34	42.06	42.12	14300	{}	\N	2020-10-12 15:45:04.143047
60444	NUMG	2020-09-30	42.15	42.605	41.97	42.25	16256	{}	\N	2020-10-12 15:45:04.149155
60445	NUMG	2020-10-01	42.5889	42.85	42.576	42.71	34289	{}	\N	2020-10-12 15:45:04.156097
60446	NUMG	2020-10-02	42.25	42.9299	42.25	42.74	19546	{}	\N	2020-10-12 15:45:04.163304
60447	NUMG	2020-10-05	43.17	43.47	43.06	43.43	12188	{}	\N	2020-10-12 15:45:04.170553
60448	NUMG	2020-10-06	43.45	43.9364	43.2	43.24	14696	{}	\N	2020-10-12 15:45:04.177235
60449	NUMG	2020-10-07	43.834	44.1999	43.834	44.17	19823	{}	\N	2020-10-12 15:45:04.183228
60450	NUMG	2020-10-08	44.31	44.4639	44.3	44.4639	15287	{}	\N	2020-10-12 15:45:04.188675
60451	NUMG	2020-10-09	44.85	45.06	44.8	44.85	60686	{}	\N	2020-10-12 15:45:04.194318
60452	NUMG	2020-10-12	45.3	45.33	45.128	45.33	4620	{}	\N	2020-10-12 15:45:04.200255
60453	NVAX	2020-09-28	116.77	117.88	106.3993	111.18	6143536	{}	\N	2020-10-12 15:45:05.426798
60454	NVAX	2020-09-29	107.7934	109.74	105.1	107.95	3218057	{}	\N	2020-10-12 15:45:05.434303
60455	NVAX	2020-09-30	107	116.25	107	108.35	4838166	{}	\N	2020-10-12 15:45:05.439403
60456	NVAX	2020-10-01	109.22	109.4544	103.1	104.9	3670461	{}	\N	2020-10-12 15:45:05.444856
60457	NVAX	2020-10-02	101.7	107.99	100.22	103.6	2677907	{}	\N	2020-10-12 15:45:05.451578
60458	NVAX	2020-10-05	105.13	110.7125	104.1	110.17	3217816	{}	\N	2020-10-12 15:45:05.460416
60459	NVAX	2020-10-06	109.2	113.08	106.22	107.41	2436133	{}	\N	2020-10-12 15:45:05.470552
60460	NVAX	2020-10-07	108.3	110.4	107.45	110	1833949	{}	\N	2020-10-12 15:45:05.476785
60461	NVAX	2020-10-08	110.95	114.35	109.2	111.02	2357360	{}	\N	2020-10-12 15:45:05.482047
60462	NVAX	2020-10-09	112.3	115.6	111.08	111.22	2426913	{}	\N	2020-10-12 15:45:05.488455
60463	NVAX	2020-10-12	111.25	112.702	110.5	111	921435	{}	\N	2020-10-12 15:45:05.493836
60464	NVUS	2020-09-28	0.98	1.02	0.952	0.9611	502859	{}	\N	2020-10-12 15:45:06.639996
60465	NVUS	2020-09-29	0.99	1.05	0.9668	0.9803	796869	{}	\N	2020-10-12 15:45:06.645949
60466	NVUS	2020-09-30	1.02	1.02	0.94	0.95	327447	{}	\N	2020-10-12 15:45:06.651003
60467	NVUS	2020-10-01	0.9428	0.98	0.9266	0.953	265701	{}	\N	2020-10-12 15:45:06.656328
60468	NVUS	2020-10-02	0.9359	0.938	0.812	0.9	1254067	{}	\N	2020-10-12 15:45:06.663667
60469	NVUS	2020-10-05	15	18.33	14	18.33	93273	{}	\N	2020-10-12 15:45:06.669549
60470	NVUS	2020-10-06	18.35	18.801	16.51	17.18	30883	{}	\N	2020-10-12 15:45:06.674783
60471	NVUS	2020-10-07	17.64	18	16	18	39521	{}	\N	2020-10-12 15:45:06.679537
60472	NVUS	2020-10-08	17.41	18	16.58	16.99	22656	{}	\N	2020-10-12 15:45:06.684329
60473	NVUS	2020-10-09	16.97	17.48	16.5	16.9	22803	{}	\N	2020-10-12 15:45:06.688535
60474	NVUS	2020-10-12	16.59	16.59	15.9	15.9	8912	{}	\N	2020-10-12 15:45:06.69299
60475	NML	2020-09-28	2.39	2.44	2.36	2.41	168642	{}	\N	2020-10-12 15:45:08.436897
60476	NML	2020-09-29	2.39	2.43	2.39	2.43	211286	{}	\N	2020-10-12 15:45:08.443135
60477	NML	2020-09-30	2.44	2.46	2.37	2.38	255638	{}	\N	2020-10-12 15:45:08.448627
60478	NML	2020-10-01	2.37	2.3846	2.33	2.34	200240	{}	\N	2020-10-12 15:45:08.455176
60479	NML	2020-10-02	2.3	2.38	2.3	2.35	320745	{}	\N	2020-10-12 15:45:08.46243
60480	NML	2020-10-05	2.42	2.5299	2.36	2.5	309845	{}	\N	2020-10-12 15:45:08.468783
60481	NML	2020-10-06	2.52	2.56	2.49	2.51	409021	{}	\N	2020-10-12 15:45:08.475037
60482	NML	2020-10-07	2.55	2.596	2.51	2.55	237402	{}	\N	2020-10-12 15:45:08.480844
60483	NML	2020-10-08	2.57	2.69	2.57	2.68	329201	{}	\N	2020-10-12 15:45:08.486453
60484	NML	2020-10-09	2.72	2.72	2.63	2.68	354196	{}	\N	2020-10-12 15:45:08.492607
60485	NML	2020-10-12	2.67	2.67	2.62	2.63	319766	{}	\N	2020-10-12 15:45:08.498131
60486	NNA	2020-09-28	4.17	4.17	4.0825	4.14	57777	{}	\N	2020-10-12 15:45:09.429935
60487	NNA	2020-09-29	4.14	4.19	4.1	4.17	116930	{}	\N	2020-10-12 15:45:09.435117
60488	NNA	2020-09-30	4.15	4.29	4.15	4.23	123345	{}	\N	2020-10-12 15:45:09.440301
60489	NNA	2020-10-01	4.29	4.29	4.18	4.29	98104	{}	\N	2020-10-12 15:45:09.445577
60490	NNA	2020-10-02	4.22	4.305	4.22	4.29	111166	{}	\N	2020-10-12 15:45:09.451477
60491	NNA	2020-10-05	4.33	4.4533	4.33	4.4	192312	{}	\N	2020-10-12 15:45:09.45781
60492	NNA	2020-10-06	4.5	4.52	4.2601	4.34	169684	{}	\N	2020-10-12 15:45:09.464486
60493	NNA	2020-10-07	4.43	4.43	4.31	4.38	125630	{}	\N	2020-10-12 15:45:09.469332
60494	NNA	2020-10-08	4.44	4.48	4.34	4.45	251497	{}	\N	2020-10-12 15:45:09.475209
60495	NNA	2020-10-09	4.45	4.58	4.43	4.52	136943	{}	\N	2020-10-12 15:45:09.480148
60496	NNA	2020-10-12	4.57	4.57	4.4804	4.5	58168	{}	\N	2020-10-12 15:45:09.485864
60497	NOC	2020-09-28	330.54	333.56	328.72	329.52	570086	{}	\N	2020-10-12 15:45:10.612604
60498	NOC	2020-09-29	328.78	330.42	324.43	325.28	980375	{}	\N	2020-10-12 15:45:10.618225
60499	NOC	2020-09-30	326.2	326.73	312.25	315.49	2334302	{}	\N	2020-10-12 15:45:10.624853
60500	NOC	2020-10-01	317.35	321.05	309.36	311.81	3303922	{}	\N	2020-10-12 15:45:10.629663
60501	NOC	2020-10-02	309.11	315.135	307.12	313.95	2749800	{}	\N	2020-10-12 15:45:10.634423
60502	NOC	2020-10-05	315.85	317.23	311.3	312.18	1735611	{}	\N	2020-10-12 15:45:10.639531
60503	NOC	2020-10-06	320.33	320.33	308.89	308.97	2257235	{}	\N	2020-10-12 15:45:10.644742
60504	NOC	2020-10-07	310.4	315.57	310.4	313.89	2057989	{}	\N	2020-10-12 15:45:10.649574
60505	NOC	2020-10-08	314.22	319.82	314.22	319.68	1189721	{}	\N	2020-10-12 15:45:10.654445
60506	NOC	2020-10-09	320	322.47	313.63	315.28	2336626	{}	\N	2020-10-12 15:45:10.660036
60507	NOC	2020-10-12	314.52	318.67	313.22	317.54	446191	{}	\N	2020-10-12 15:45:10.666224
60508	NOMD	2020-09-28	25.12	25.24	24.83	25.1	1156332	{}	\N	2020-10-12 15:45:11.843238
60509	NOMD	2020-09-29	25.4	25.63	25.2	25.3	2544814	{}	\N	2020-10-12 15:45:11.849403
60510	NOMD	2020-09-30	25.25	25.53	25.09	25.48	2063091	{}	\N	2020-10-12 15:45:11.855753
60511	NOMD	2020-10-01	25.61	25.72	25.25	25.63	1777667	{}	\N	2020-10-12 15:45:11.861227
60512	NOMD	2020-10-02	25.53	25.91	25.505	25.83	2163648	{}	\N	2020-10-12 15:45:11.867146
60513	NOMD	2020-10-05	25.86	26.12	25.68	26.05	2918870	{}	\N	2020-10-12 15:45:11.872564
60514	NOMD	2020-10-06	26.1	26.31	25.69	25.74	2520099	{}	\N	2020-10-12 15:45:11.87979
60515	NOMD	2020-10-07	25.74	25.81	25.42	25.72	2766747	{}	\N	2020-10-12 15:45:11.885382
60516	NOMD	2020-10-08	25.78	25.97	25.5	25.51	2326552	{}	\N	2020-10-12 15:45:11.891101
60517	NOMD	2020-10-09	25.68	25.89	25.43	25.58	1512546	{}	\N	2020-10-12 15:45:11.896918
60518	NOMD	2020-10-12	25.61	26.41	25.58	26.24	912865	{}	\N	2020-10-12 15:45:11.901978
60519	NPV	2020-09-28	15.14	15.17	15.03	15.11	21462	{}	\N	2020-10-12 15:45:13.858275
60520	NPV	2020-09-29	15.11	15.2361	15.06	15.06	21316	{}	\N	2020-10-12 15:45:13.86494
60521	NPV	2020-09-30	15.071	15.115	14.99	15.05	49918	{}	\N	2020-10-12 15:45:13.870364
60522	NPV	2020-10-01	15.19	15.316	15.05	15.14	54838	{}	\N	2020-10-12 15:45:13.877013
60523	NPV	2020-10-02	15.25	15.2876	14.85	14.91	34844	{}	\N	2020-10-12 15:45:13.884897
60524	NPV	2020-10-05	14.99	15.28	14.99	15.01	16179	{}	\N	2020-10-12 15:45:13.88991
60525	NPV	2020-10-06	15.1586	15.24	15.08	15.1	38980	{}	\N	2020-10-12 15:45:13.895689
60526	NPV	2020-10-07	15.1186	15.13	15.109	15.13	19960	{}	\N	2020-10-12 15:45:13.900276
60527	NPV	2020-10-08	15.05	15.13	15.05	15.13	52232	{}	\N	2020-10-12 15:45:13.905147
60528	NPV	2020-10-09	15.13	15.18	15.0572	15.18	33762	{}	\N	2020-10-12 15:45:13.910846
60529	NPV	2020-10-12	15.06	15.15	15.06	15.0646	20834	{}	\N	2020-10-12 15:45:13.915997
60530	NRO	2020-09-28	3.77	3.78	3.74	3.78	168518	{}	\N	2020-10-12 15:45:15.796537
60531	NRO	2020-09-29	3.81	3.8105	3.7	3.71	122252	{}	\N	2020-10-12 15:45:15.803034
60532	NRO	2020-09-30	3.76	3.79	3.7	3.71	266218	{}	\N	2020-10-12 15:45:15.807107
60533	NRO	2020-10-01	3.73	3.87	3.72	3.83	352975	{}	\N	2020-10-12 15:45:15.811064
60534	NRO	2020-10-02	3.83	3.92	3.8	3.89	438749	{}	\N	2020-10-12 15:45:15.815974
60535	NRO	2020-10-05	3.99	3.99	3.9	3.95	684898	{}	\N	2020-10-12 15:45:15.820271
60536	NRO	2020-10-06	3.98	4.01	3.93	3.96	286100	{}	\N	2020-10-12 15:45:15.824806
60537	NRO	2020-10-07	4.01	4.01	3.955	3.98	247380	{}	\N	2020-10-12 15:45:15.828995
60538	NRO	2020-10-08	4.01	4.06	4	4.05	190740	{}	\N	2020-10-12 15:45:15.834469
60539	NRO	2020-10-09	4.09	4.09	4.01	4.03	230388	{}	\N	2020-10-12 15:45:15.841393
60540	NRO	2020-10-12	4.04	4.0493	4.03	4.0399	69843	{}	\N	2020-10-12 15:45:15.848063
60541	NSL	2020-09-28	4.84	4.88	4.82	4.85	2161132	{}	\N	2020-10-12 15:45:18.199184
60542	NSL	2020-09-29	4.86	4.88	4.81	4.82	5493542	{}	\N	2020-10-12 15:45:18.205566
60543	NSL	2020-09-30	4.82	4.875	4.81	4.83	5455018	{}	\N	2020-10-12 15:45:18.211118
60544	NSL	2020-10-01	4.84	4.92	4.84	4.87	4537010	{}	\N	2020-10-12 15:45:18.218027
60545	NSL	2020-10-02	4.83	4.93	4.83	4.92	309242	{}	\N	2020-10-12 15:45:18.225454
60546	NSL	2020-10-05	4.91	4.96	4.91	4.94	4549722	{}	\N	2020-10-12 15:45:18.232088
60547	NSL	2020-10-06	4.93	4.94	4.92	4.92	230404	{}	\N	2020-10-12 15:45:18.238295
60548	NSL	2020-10-07	4.92	4.95	4.91	4.93	1171654	{}	\N	2020-10-12 15:45:18.243149
60549	NSL	2020-10-08	4.92	4.96	4.92	4.95	201876	{}	\N	2020-10-12 15:45:18.249197
60550	NSL	2020-10-09	4.95	4.98	4.94	4.94	158106	{}	\N	2020-10-12 15:45:18.254589
60551	NSL	2020-10-12	4.97	4.97	4.9458	4.96	69366	{}	\N	2020-10-12 15:45:18.260543
60552	NTP	2020-09-28	9.98	10	9.57	9.67	61090	{}	\N	2020-10-12 15:45:19.696828
60553	NTP	2020-09-29	9.65	9.692	9.33	9.37	58432	{}	\N	2020-10-12 15:45:19.703019
60554	NTP	2020-09-30	9.38	9.46	9.111	9.36	123353	{}	\N	2020-10-12 15:45:19.707598
60555	NTP	2020-10-01	9.29	9.37	9.12	9.23	82092	{}	\N	2020-10-12 15:45:19.712593
60556	NTP	2020-10-02	9.35	9.39	9.23	9.39	69985	{}	\N	2020-10-12 15:45:19.72077
60557	NTP	2020-10-05	9.21	9.285	6.78	6.9	956218	{}	\N	2020-10-12 15:45:19.727137
60558	NTP	2020-10-06	6.9	6.9402	5.915	6.01	851595	{}	\N	2020-10-12 15:45:19.732339
60559	NTP	2020-10-07	5.93	5.94	5.76	5.82	366897	{}	\N	2020-10-12 15:45:19.738078
60560	NTP	2020-10-08	5.74	6.085	5.5901	6.03	843025	{}	\N	2020-10-12 15:45:19.742908
60561	NTP	2020-10-09	5.93	5.975	5.75	5.8	400718	{}	\N	2020-10-12 15:45:19.747847
60562	NTP	2020-10-12	5.83	6.03	5.77	5.81	48120	{}	\N	2020-10-12 15:45:19.753374
60563	NUE	2020-09-28	46.25	46.94	45.62	46.27	1402327	{}	\N	2020-10-12 15:45:21.539089
60564	NUE	2020-09-29	45.61	45.79	44.561	44.74	2273552	{}	\N	2020-10-12 15:45:21.544489
60565	NUE	2020-09-30	45.12	45.75	44.58	44.86	2784361	{}	\N	2020-10-12 15:45:21.550429
60566	NUE	2020-10-01	45.02	45.55	44.7865	45.05	3537601	{}	\N	2020-10-12 15:45:21.556841
60567	NUE	2020-10-02	44.35	46.5779	44.2	46.04	4073860	{}	\N	2020-10-12 15:45:21.562351
60568	NUE	2020-10-05	46.56	47.65	46.26	47.28	4151354	{}	\N	2020-10-12 15:45:21.567359
60569	NUE	2020-10-06	47.54	48.23	46.65	46.93	4349472	{}	\N	2020-10-12 15:45:21.573211
60570	NUE	2020-10-07	47.67	49	47.41	48.65	3580192	{}	\N	2020-10-12 15:45:21.577982
60571	NUE	2020-10-08	48.93	49.49	48.68	49.44	3371465	{}	\N	2020-10-12 15:45:21.582927
60572	NUE	2020-10-09	49.87	49.9	48.51	48.86	3992430	{}	\N	2020-10-12 15:45:21.589017
60573	NUE	2020-10-12	48.81	49.33	48.74	49.09	813318	{}	\N	2020-10-12 15:45:21.594533
60574	NURO	2020-09-28	1.76	1.76	1.62	1.68	229531	{}	\N	2020-10-12 15:45:23.712326
60575	NURO	2020-09-29	1.71	1.74	1.67	1.73	130926	{}	\N	2020-10-12 15:45:23.718898
60576	NURO	2020-09-30	1.67	1.73	1.67	1.7	68395	{}	\N	2020-10-12 15:45:23.725782
60577	NURO	2020-10-01	1.7	1.77	1.6612	1.74	65033	{}	\N	2020-10-12 15:45:23.731164
60578	NURO	2020-10-02	1.66	1.68	1.6	1.66	105697	{}	\N	2020-10-12 15:45:23.737059
60579	NURO	2020-10-05	1.66	1.7299	1.6584	1.69	31374	{}	\N	2020-10-12 15:45:23.743789
60580	NURO	2020-10-06	1.7	1.77	1.67	1.72	63120	{}	\N	2020-10-12 15:45:23.749704
60581	NURO	2020-10-07	1.701	1.75	1.66	1.7	32385	{}	\N	2020-10-12 15:45:23.755801
60582	NURO	2020-10-08	1.7	1.78	1.7	1.74	97647	{}	\N	2020-10-12 15:45:23.761684
60583	NURO	2020-10-09	1.75	1.75	1.71	1.73	22013	{}	\N	2020-10-12 15:45:23.7668
60584	NURO	2020-10-12	1.73	1.7494	1.7101	1.7254	26249	{}	\N	2020-10-12 15:45:23.773002
60585	NVEC	2020-09-28	50.2	50.83	50.13	50.36	12585	{}	\N	2020-10-12 15:45:25.311578
60586	NVEC	2020-09-29	50.13	51.2	49.58	49.94	20344	{}	\N	2020-10-12 15:45:25.318534
60587	NVEC	2020-09-30	50.35	50.68	48.78	49.08	12237	{}	\N	2020-10-12 15:45:25.324504
60588	NVEC	2020-10-01	49.32	49.32	48.01	48.84	40308	{}	\N	2020-10-12 15:45:25.329969
60589	NVEC	2020-10-02	48.32	49.49	48.1944	48.99	14525	{}	\N	2020-10-12 15:45:25.336392
60590	NVEC	2020-10-05	49.24	50.57	48.75	50.32	25374	{}	\N	2020-10-12 15:45:25.343472
60591	NVEC	2020-10-06	50.48	52.33	50.48	51.44	15074	{}	\N	2020-10-12 15:45:25.349624
60592	NVEC	2020-10-07	51.5	52.49	50.61	50.9	21267	{}	\N	2020-10-12 15:45:25.354731
60593	NVEC	2020-10-08	51.44	51.44	50.06	50.2	21613	{}	\N	2020-10-12 15:45:25.360527
60594	NVEC	2020-10-09	50.44	50.545	49.91	49.97	11914	{}	\N	2020-10-12 15:45:25.365774
60595	NVEC	2020-10-12	50.43	50.5515	50	50.09	5721	{}	\N	2020-10-12 15:45:25.370684
60596	NJV	2020-09-28	13.9471	14.02	13.91	14.02	1229	{}	\N	2020-10-12 15:45:27.05805
60597	NJV	2020-09-29	14.01	14.01	13.93	13.93	3418	{}	\N	2020-10-12 15:45:27.064064
60598	NJV	2020-09-30	14.0199	14.02	13.8	13.8	38326	{}	\N	2020-10-12 15:45:27.070192
60599	NJV	2020-10-01	13.87	13.87	13.82	13.841	7438	{}	\N	2020-10-12 15:45:27.075453
60600	NJV	2020-10-02	13.8	13.8716	13.8	13.8716	6048	{}	\N	2020-10-12 15:45:27.080754
60601	NJV	2020-10-05	13.91	13.91	13.81	13.81	996	{}	\N	2020-10-12 15:45:27.086206
60602	NJV	2020-10-06	13.8201	13.828	13.81	13.81	12876	{}	\N	2020-10-12 15:45:27.092078
60603	NJV	2020-10-07	13.9	13.9	13.83	13.83	4260	{}	\N	2020-10-12 15:45:27.098348
60604	NJV	2020-10-08	13.9	13.9	13.8957	13.8957	12198	{}	\N	2020-10-12 15:45:27.103765
60605	NJV	2020-10-09	13.86	13.89	13.85	13.89	1006	{}	\N	2020-10-12 15:45:27.109525
60606	NLS	2020-09-28	17.52	17.6	16.81	17.51	2036537	{}	\N	2020-10-12 15:45:28.103999
60607	NLS	2020-09-29	17.5	18.25	17.46	18.15	4119422	{}	\N	2020-10-12 15:45:28.110294
60608	NLS	2020-09-30	17.98	18.26	17.05	17.16	4714014	{}	\N	2020-10-12 15:45:28.115373
60609	NLS	2020-10-01	17.46	17.67	17.01	17.61	2819690	{}	\N	2020-10-12 15:45:28.120457
60610	NLS	2020-10-02	16.99	17.79	16.73	17.44	2098711	{}	\N	2020-10-12 15:45:28.125948
60611	NLS	2020-10-05	17.71	19.8998	17.56	19.8	6523166	{}	\N	2020-10-12 15:45:28.131649
60612	NLS	2020-10-06	19.66	20.7	19.1	19.82	5001729	{}	\N	2020-10-12 15:45:28.138324
60613	NLS	2020-10-07	20.28	21.75	20.134	21.59	5017594	{}	\N	2020-10-12 15:45:28.144104
60614	NLS	2020-10-08	22.01	22.15	20.77	21.29	3604728	{}	\N	2020-10-12 15:45:28.149547
60615	NLS	2020-10-09	21.76	23.4	21.6	22.1	5880889	{}	\N	2020-10-12 15:45:28.154407
60616	NLS	2020-10-12	22.62	23	21.5	21.86	1391495	{}	\N	2020-10-12 15:45:28.160052
60617	NNI	2020-09-28	60.5	61.58	60.5	61.03	59474	{}	\N	2020-10-12 15:45:29.096978
60618	NNI	2020-09-29	61.3	61.45	59.956	60.68	63868	{}	\N	2020-10-12 15:45:29.10455
60619	NNI	2020-09-30	60.58	61.47	60.02	60.25	104052	{}	\N	2020-10-12 15:45:29.109549
60620	NNI	2020-10-01	60.3	61.03	59.29	60.35	149986	{}	\N	2020-10-12 15:45:29.11438
60621	NNI	2020-10-02	59.56	64.05	59.19	63.64	170749	{}	\N	2020-10-12 15:45:29.119263
60622	NNI	2020-10-05	63.83	65.18	63.83	65.04	149766	{}	\N	2020-10-12 15:45:29.12393
60623	NNI	2020-10-06	65.68	66.47	63.69	64.37	169763	{}	\N	2020-10-12 15:45:29.129146
60624	NNI	2020-10-07	65.03	66.02	64.46	65.69	141873	{}	\N	2020-10-12 15:45:29.134651
60625	NNI	2020-10-08	66.33	67.19	65.925	66.87	96554	{}	\N	2020-10-12 15:45:29.139526
60626	NNI	2020-10-09	67.07	67.824	66.17	66.29	58161	{}	\N	2020-10-12 15:45:29.146123
60627	NNI	2020-10-12	66.33	66.91	65.95	66.82	14686	{}	\N	2020-10-12 15:45:29.150785
60628	NOK	2020-09-28	3.85	3.94	3.83	3.9	28267206	{}	\N	2020-10-12 15:45:30.113049
60629	NOK	2020-09-29	3.96	4.04	3.93	3.95	50759208	{}	\N	2020-10-12 15:45:30.118782
60630	NOK	2020-09-30	3.945	3.98	3.9	3.91	49638520	{}	\N	2020-10-12 15:45:30.123757
60631	NOK	2020-10-01	3.91	3.915	3.84	3.87	33927256	{}	\N	2020-10-12 15:45:30.129354
60632	NOK	2020-10-02	3.8	3.86	3.8	3.82	38772386	{}	\N	2020-10-12 15:45:30.135181
60633	NOK	2020-10-05	3.95	3.96	3.88	3.93	31874009	{}	\N	2020-10-12 15:45:30.140021
60634	NOK	2020-10-06	3.98	4	3.87	3.91	47313227	{}	\N	2020-10-12 15:45:30.147417
60635	NOK	2020-10-07	3.9	3.92	3.86	3.9	30768812	{}	\N	2020-10-12 15:45:30.155492
60636	NOK	2020-10-08	3.9	3.98	3.89	3.95	39087272	{}	\N	2020-10-12 15:45:30.160756
60637	NOK	2020-10-09	4.07	4.13	4.02	4.03	70844168	{}	\N	2020-10-12 15:45:30.166124
60638	NOK	2020-10-12	4.05	4.06	4.01	4.03	21349751	{}	\N	2020-10-12 15:45:30.171322
60639	NP	2020-09-28	38.65	39.45	38.65	39.03	79445	{}	\N	2020-10-12 15:45:31.334039
60640	NP	2020-09-29	38.95	38.95	37.86	38.1	153353	{}	\N	2020-10-12 15:45:31.344676
60641	NP	2020-09-30	38.07	38.62	37.19	37.47	250575	{}	\N	2020-10-12 15:45:31.366012
60642	NP	2020-10-01	37.22	38.03	37.2	37.81	125508	{}	\N	2020-10-12 15:45:31.374086
60643	NP	2020-10-02	37.12	38.99	37.03	38.24	186635	{}	\N	2020-10-12 15:45:31.386339
60644	NP	2020-10-05	38.74	39.58	38.63	39.35	118731	{}	\N	2020-10-12 15:45:31.406685
60645	NP	2020-10-06	39.93	40.18	39.17	39.29	165579	{}	\N	2020-10-12 15:45:31.421149
60646	NP	2020-10-07	39.55	40.3	39.45	40.26	244121	{}	\N	2020-10-12 15:45:31.435941
60647	NP	2020-10-08	40.72	40.72	39.6689	40.44	107141	{}	\N	2020-10-12 15:45:31.443898
60648	NP	2020-10-09	40.77	41.07	39.36	39.72	125175	{}	\N	2020-10-12 15:45:31.451392
60649	NP	2020-10-12	39.7	40.16	39.65	39.78	27451	{}	\N	2020-10-12 15:45:31.458635
60650	NRG	2020-09-28	29.85	30.09	29.4801	29.52	2122193	{}	\N	2020-10-12 15:45:32.376579
60651	NRG	2020-09-29	29.76	30.065	29.43	29.7	4534255	{}	\N	2020-10-12 15:45:32.382852
60652	NRG	2020-09-30	29.95	30.92	29.81	30.74	5882108	{}	\N	2020-10-12 15:45:32.388258
60653	NRG	2020-10-01	30.79	30.93	30.235	30.63	4353349	{}	\N	2020-10-12 15:45:32.394164
60654	NRG	2020-10-02	30.25	31.17	29.95	31.06	3553046	{}	\N	2020-10-12 15:45:32.401229
60655	NRG	2020-10-05	31.15	31.585	30.884	31.27	3993440	{}	\N	2020-10-12 15:45:32.407498
60656	NRG	2020-10-06	31.31	31.55	30.9	31.1	2682768	{}	\N	2020-10-12 15:45:32.413443
60657	NRG	2020-10-07	31.13	31.53	31.04	31.39	3773541	{}	\N	2020-10-12 15:45:32.418732
60658	NRG	2020-10-08	31.48	32.41	31.44	32.35	3612764	{}	\N	2020-10-12 15:45:32.423497
60659	NRG	2020-10-09	32.53	32.67	32.06	32.27	5893490	{}	\N	2020-10-12 15:45:32.428794
60660	NRG	2020-10-12	32.35	32.71	32.04	32.68	1073646	{}	\N	2020-10-12 15:45:32.435281
60661	NSP	2020-09-28	65.95	66.79	65.47	65.97	221415	{}	\N	2020-10-12 15:45:33.567043
60662	NSP	2020-09-29	65.88	65.895	64.44	65.25	244239	{}	\N	2020-10-12 15:45:33.572991
60663	NSP	2020-09-30	65.83	66.66	65.05	65.49	452524	{}	\N	2020-10-12 15:45:33.578292
60664	NSP	2020-10-01	65.8	66.845	65.25	66.22	328237	{}	\N	2020-10-12 15:45:33.583722
60665	NSP	2020-10-02	64.98	67.84	64.53	67.54	371732	{}	\N	2020-10-12 15:45:33.590046
60666	NSP	2020-10-05	68.1	68.74	67.25	68.38	254048	{}	\N	2020-10-12 15:45:33.595298
60667	NSP	2020-10-06	69.16	70.68	68.5	68.55	345908	{}	\N	2020-10-12 15:45:33.601262
60668	NSP	2020-10-07	69.45	71.48	69.14	71.09	371537	{}	\N	2020-10-12 15:45:33.607418
60669	NSP	2020-10-08	71.1	72.93	70.75	72.8	412902	{}	\N	2020-10-12 15:45:33.613241
60670	NSP	2020-10-09	73.82	73.82	71.54	72.23	282431	{}	\N	2020-10-12 15:45:33.619351
60671	NSP	2020-10-12	72.31	73.11	72.15	72.71	42212	{}	\N	2020-10-12 15:45:33.624668
60672	NTEC	2020-09-28	0.25	0.259	0.2427	0.2468	1012957	{}	\N	2020-10-12 15:45:34.704242
60673	NTEC	2020-09-29	0.2592	0.26	0.245	0.247	1563870	{}	\N	2020-10-12 15:45:34.710013
60674	NTEC	2020-09-30	0.257	0.25845	0.2431	0.2516	1029712	{}	\N	2020-10-12 15:45:34.715771
60675	NTEC	2020-10-01	0.2489	0.2564	0.247	0.2519	815713	{}	\N	2020-10-12 15:45:34.722222
60676	NTEC	2020-10-02	0.245	0.2699	0.2401	0.259	2501901	{}	\N	2020-10-12 15:45:34.727818
60677	NTEC	2020-10-05	0.2586	0.2685	0.2501	0.2615	2045381	{}	\N	2020-10-12 15:45:34.733933
60678	NTEC	2020-10-06	0.267	0.267351	0.2505	0.2505	1150361	{}	\N	2020-10-12 15:45:34.740849
60679	NTEC	2020-10-07	0.258	0.258	0.245	0.2537	1168497	{}	\N	2020-10-12 15:45:34.746649
60680	NTEC	2020-10-08	0.2864	0.315	0.2729	0.2803	13078359	{}	\N	2020-10-12 15:45:34.75325
60681	NTEC	2020-10-09	0.289	0.29	0.263	0.269	3931891	{}	\N	2020-10-12 15:45:34.75955
60682	NTEC	2020-10-12	0.2646	0.2649	0.255	0.2619	1273313	{}	\N	2020-10-12 15:45:34.765982
60683	NTNX	2020-09-28	22.16	22.19	21.81	22.01	1330082	{}	\N	2020-10-12 15:45:36.434441
60684	NTNX	2020-09-29	22.08	22.36	21.8	22.13	1274486	{}	\N	2020-10-12 15:45:36.440884
60685	NTNX	2020-09-30	22.16	22.58	21.96	22.18	2251253	{}	\N	2020-10-12 15:45:36.446249
60686	NTNX	2020-10-01	22.54	23.13	22.44	22.99	2788504	{}	\N	2020-10-12 15:45:36.454866
60687	NTNX	2020-10-02	22.18	23.2799	22.11	22.25	1738722	{}	\N	2020-10-12 15:45:36.462519
60688	NTNX	2020-10-05	22.38	22.4	21.86	22.03	1661242	{}	\N	2020-10-12 15:45:36.470744
60689	NTNX	2020-10-06	22.01	22.52	21.39	21.46	2167159	{}	\N	2020-10-12 15:45:36.476407
60690	NTNX	2020-10-07	21.68	21.95	21.35	21.37	1538962	{}	\N	2020-10-12 15:45:36.482451
60691	NTNX	2020-10-08	21.65	23.11	21.6062	22.81	3185065	{}	\N	2020-10-12 15:45:36.489858
60692	NTNX	2020-10-09	23	23.4	22.885	23.12	1738074	{}	\N	2020-10-12 15:45:36.495052
60693	NTNX	2020-10-12	23.14	23.35	22.77	23.32	779822	{}	\N	2020-10-12 15:45:36.500705
60694	NUAG	2020-09-28	25.68	25.75	25.665	25.71	36857	{}	\N	2020-10-12 15:45:37.646232
60695	NUAG	2020-09-29	25.67	25.74	25.67	25.69	15896	{}	\N	2020-10-12 15:45:37.652699
60696	NUAG	2020-09-30	25.67	25.6915	25.67	25.683	5599	{}	\N	2020-10-12 15:45:37.657788
60697	NUAG	2020-10-01	25.56	25.6	25.54	25.58	20503	{}	\N	2020-10-12 15:45:37.662273
60698	NUAG	2020-10-02	25.611	25.63	25.59	25.59	6815	{}	\N	2020-10-12 15:45:37.666556
60699	NUAG	2020-10-05	25.59	25.59	25.49	25.52	15623	{}	\N	2020-10-12 15:45:37.672126
60700	NUAG	2020-10-06	25.52	25.57	25.49	25.53	19493	{}	\N	2020-10-12 15:45:37.677728
60701	NUAG	2020-10-07	25.5441	25.58	25.5441	25.56	5779	{}	\N	2020-10-12 15:45:37.682148
60702	NUAG	2020-10-08	25.5725	25.5725	25.53	25.55	8285	{}	\N	2020-10-12 15:45:37.687873
60703	NUAG	2020-10-09	25.565	25.58	25.52	25.55	21408	{}	\N	2020-10-12 15:45:37.692906
60704	NUAG	2020-10-12	25.6399	25.6399	25.6192	25.6192	936	{}	\N	2020-10-12 15:45:37.699345
60705	NUO	2020-09-28	15.11	15.15	15.0799	15.12	25561	{}	\N	2020-10-12 15:45:38.944879
60706	NUO	2020-09-29	15.16	15.21	15.02	15.05	47250	{}	\N	2020-10-12 15:45:38.952746
60707	NUO	2020-09-30	15.03	15.04	15	15.03	105583	{}	\N	2020-10-12 15:45:38.95814
60708	NUO	2020-10-01	15.12	15.14	15.016	15.12	23836	{}	\N	2020-10-12 15:45:38.962791
60709	NUO	2020-10-02	15.12	15.12	15	15.11	38886	{}	\N	2020-10-12 15:45:38.967971
60710	NUO	2020-10-05	15.12	15.16	15.08	15.13	52639	{}	\N	2020-10-12 15:45:38.973726
60711	NUO	2020-10-06	15.16	15.2	15.16	15.1618	3248	{}	\N	2020-10-12 15:45:38.980336
60712	NUO	2020-10-07	15.26	15.26	15.1422	15.19	3554	{}	\N	2020-10-12 15:45:38.985002
60713	NUO	2020-10-08	15.31	15.31	15.17	15.1898	15022	{}	\N	2020-10-12 15:45:38.991493
60714	NUO	2020-10-09	15.2	15.25	15.19	15.19	5998	{}	\N	2020-10-12 15:45:38.995836
60715	NUO	2020-10-12	15.2899	15.2899	15.1979	15.1979	4374	{}	\N	2020-10-12 15:45:39.000328
60716	NVDA	2020-09-28	521.61	522.07	510	521.4	14051767	{}	\N	2020-10-12 15:45:39.920063
60717	NVDA	2020-09-29	517.5	537.71	517.33	529.03	15328697	{}	\N	2020-10-12 15:45:39.926657
60718	NVDA	2020-09-30	526.3	544.9	526.01	541.22	16652598	{}	\N	2020-10-12 15:45:39.930662
60719	NVDA	2020-10-01	550.32	550.5095	538.4301	544.7	12384353	{}	\N	2020-10-12 15:45:39.93456
60720	NVDA	2020-10-02	528.83	540.78	522.04	522.49	13151340	{}	\N	2020-10-12 15:45:39.939626
60721	NVDA	2020-10-05	529.44	545.77	528.9106	545.7	11953503	{}	\N	2020-10-12 15:45:39.944371
60722	NVDA	2020-10-06	553.05	565.68	546.6	549.36	18644797	{}	\N	2020-10-12 15:45:39.9488
60723	NVDA	2020-10-07	560	564.77	554.54	558.56	10365764	{}	\N	2020-10-12 15:45:39.954673
60724	NVDA	2020-10-08	563.245	563.38	550.32	553.55	9393232	{}	\N	2020-10-12 15:45:39.959705
60725	NVDA	2020-10-09	550.68	552.97	546.43	550.51	8397738	{}	\N	2020-10-12 15:45:39.963922
60726	NVDA	2020-10-12	559.58	567.62	556.5	565.81	4738271	{}	\N	2020-10-12 15:45:39.968296
60727	NVO	2020-09-28	68.73	69.38	68.7	69.17	727664	{}	\N	2020-10-12 15:45:40.883215
60728	NVO	2020-09-29	69.61	70.46	69.5	70.22	2523636	{}	\N	2020-10-12 15:45:40.888864
60729	NVO	2020-09-30	69.5	69.68	69.06	69.43	2676724	{}	\N	2020-10-12 15:45:40.89432
60730	NVO	2020-10-01	70	70.02	69.07	69.25	1772926	{}	\N	2020-10-12 15:45:40.898891
60731	NVO	2020-10-02	68.65	68.985	68.35	68.66	1580805	{}	\N	2020-10-12 15:45:40.903435
60732	NVO	2020-10-05	69.35	70.24	69.29	70.12	1350358	{}	\N	2020-10-12 15:45:40.909549
60733	NVO	2020-10-06	69.63	69.75	69.08	69.16	1883947	{}	\N	2020-10-12 15:45:40.914524
60734	NVO	2020-10-07	69	69.16	68.34	68.68	1636548	{}	\N	2020-10-12 15:45:40.918902
60735	NVO	2020-10-08	68.9	71.81	68.81	71.24	4435418	{}	\N	2020-10-12 15:45:40.924183
60736	NVO	2020-10-09	71.6	72.31	71.39	72.07	2603149	{}	\N	2020-10-12 15:45:40.929057
60737	NVO	2020-10-12	73.55	73.75	73.04	73.59	1026332	{}	\N	2020-10-12 15:45:40.935023
60738	NWLI	2020-09-28	184.99	190.82	184.99	186.99	13895	{}	\N	2020-10-12 15:45:41.867252
60739	NWLI	2020-09-29	184.06	187.03	178	183.72	11908	{}	\N	2020-10-12 15:45:41.875887
60740	NWLI	2020-09-30	184.29	190.92	178.27	182.77	18691	{}	\N	2020-10-12 15:45:41.880903
60741	NWLI	2020-10-01	186	192.35	185.15	190	22004	{}	\N	2020-10-12 15:45:41.886153
60742	NWLI	2020-10-02	186.99	202.71	185.78	201.21	24727	{}	\N	2020-10-12 15:45:41.892555
60743	NWLI	2020-10-05	202.2175	204.3	195.5	196.77	14374	{}	\N	2020-10-12 15:45:41.897391
60744	NWLI	2020-10-06	196.7	198.34	189.47	189.47	9654	{}	\N	2020-10-12 15:45:41.902502
60745	NWLI	2020-10-07	192.58	198.32	192.1	197.04	24155	{}	\N	2020-10-12 15:45:41.908833
60746	NWLI	2020-10-08	197.86	200	193.96	198.01	10632	{}	\N	2020-10-12 15:45:41.914463
60747	NWLI	2020-10-09	201.0554	201.0554	197.57	198.6	11900	{}	\N	2020-10-12 15:45:41.919314
60748	NWLI	2020-10-12	201.03	203.39	199.7	200	4463	{}	\N	2020-10-12 15:45:41.925456
60749	NMI	2020-09-28	11.92	11.92	11.7707	11.88	8490	{}	\N	2020-10-12 15:45:43.245663
60750	NMI	2020-09-29	11.85	11.91	11.8	11.91	23412	{}	\N	2020-10-12 15:45:43.251609
60751	NMI	2020-09-30	11.91	11.91	11.7115	11.89	24768	{}	\N	2020-10-12 15:45:43.256812
60752	NMI	2020-10-01	11.88	11.88	11.74	11.74	19754	{}	\N	2020-10-12 15:45:43.262555
60753	NMI	2020-10-02	11.78	11.83	11.78	11.8299	9962	{}	\N	2020-10-12 15:45:43.268611
60754	NMI	2020-10-05	11.75	11.8219	11.73	11.73	11552	{}	\N	2020-10-12 15:45:43.274157
60755	NMI	2020-10-06	11.73	11.74	11.4	11.4	78502	{}	\N	2020-10-12 15:45:43.279796
60756	NMI	2020-10-07	11.48	11.48	11.29	11.29	17820	{}	\N	2020-10-12 15:45:43.293485
60757	NMI	2020-10-08	11.34	11.356	11.28	11.28	11030	{}	\N	2020-10-12 15:45:43.302121
60758	NMI	2020-10-09	11.32	11.37	11.27	11.27	26684	{}	\N	2020-10-12 15:45:43.308441
60759	NMI	2020-10-12	11.28	11.28	11.28	11.28	3696	{}	\N	2020-10-12 15:45:43.315604
60760	NOM	2020-09-28	14.9	14.9	14.715	14.715	340	{}	\N	2020-10-12 15:45:47.598316
60761	NOM	2020-09-29	14.89	14.9	14.49	14.49	14052	{}	\N	2020-10-12 15:45:47.603982
60762	NOM	2020-10-01	14.84	14.84	14.695	14.695	1690	{}	\N	2020-10-12 15:45:47.609764
60763	NOM	2020-10-06	14.67	14.67	14.67	14.67	856	{}	\N	2020-10-12 15:45:47.61602
60764	NOM	2020-10-08	14.623	14.623	14.47	14.565	14320	{}	\N	2020-10-12 15:45:47.622331
60765	NOM	2020-10-09	14.61	14.8	14.61	14.8	8808	{}	\N	2020-10-12 15:45:47.628177
60766	NPTN	2020-09-28	6.1	6.23	6.02	6.07	551908	{}	\N	2020-10-12 15:45:49.124441
60767	NPTN	2020-09-29	6.07	6.185	6.055	6.14	1070771	{}	\N	2020-10-12 15:45:49.132494
60768	NPTN	2020-09-30	6.18	6.215	6.075	6.09	950822	{}	\N	2020-10-12 15:45:49.137819
60769	NPTN	2020-10-01	6.13	6.17	6	6.08	1234129	{}	\N	2020-10-12 15:45:49.143281
60770	NPTN	2020-10-02	5.91	6.06	5.9	5.99	1001130	{}	\N	2020-10-12 15:45:49.149198
60771	NPTN	2020-10-05	6.25	6.63	6.24	6.58	3971879	{}	\N	2020-10-12 15:45:49.155838
60772	NPTN	2020-10-06	6.73	6.85	6.595	6.64	2472792	{}	\N	2020-10-12 15:45:49.161378
60773	NPTN	2020-10-07	6.75	6.76	6.535	6.61	1250213	{}	\N	2020-10-12 15:45:49.167887
60774	NPTN	2020-10-08	6.25	6.62	6.22	6.56	1480156	{}	\N	2020-10-12 15:45:49.173457
60775	NPTN	2020-10-09	6.66	6.84	6.585	6.77	1176822	{}	\N	2020-10-12 15:45:49.178901
60776	NPTN	2020-10-12	6.87	6.88	6.615	6.625	586797	{}	\N	2020-10-12 15:45:49.185004
60777	NRZ	2020-09-28	8.03	8.27	8.01	8.19	4564643	{}	\N	2020-10-12 15:45:50.268715
60778	NRZ	2020-09-29	8.19	8.19	7.8201	7.89	8237628	{}	\N	2020-10-12 15:45:50.275515
60779	NRZ	2020-09-30	7.92	8.12	7.86	7.95	7050461	{}	\N	2020-10-12 15:45:50.28129
60780	NRZ	2020-10-01	8	8.13	7.9022	8.12	7611791	{}	\N	2020-10-12 15:45:50.288034
60781	NRZ	2020-10-02	7.79	8.19	7.75	8.18	11496538	{}	\N	2020-10-12 15:45:50.295612
60782	NRZ	2020-10-05	8.13	8.35	8.13	8.34	8282746	{}	\N	2020-10-12 15:45:50.311584
60783	NRZ	2020-10-06	8.35	8.525	8.16	8.18	15981441	{}	\N	2020-10-12 15:45:50.319721
60784	NRZ	2020-10-07	8.3	8.35	8.11	8.19	12542838	{}	\N	2020-10-12 15:45:50.327301
60785	NRZ	2020-10-08	8.25	8.46	8.25	8.42	7194048	{}	\N	2020-10-12 15:45:50.333662
60786	NRZ	2020-10-09	8.42	8.5	8.21	8.29	6260581	{}	\N	2020-10-12 15:45:50.340474
60787	NRZ	2020-10-12	8.29	8.39	8.285	8.33	1563708	{}	\N	2020-10-12 15:45:50.346235
60788	NSSC	2020-09-28	22.84	24	22.78	23.65	68964	{}	\N	2020-10-12 15:45:51.536495
60789	NSSC	2020-09-29	23.53	23.84	23.12	23.6	96914	{}	\N	2020-10-12 15:45:51.544639
60790	NSSC	2020-09-30	23.49	24.29	23.225	23.52	109057	{}	\N	2020-10-12 15:45:51.55164
60791	NSSC	2020-10-01	23.59	24.4	23.42	24.17	109511	{}	\N	2020-10-12 15:45:51.557843
60792	NSSC	2020-10-02	23.76	24.05	22.9	23.91	154715	{}	\N	2020-10-12 15:45:51.563787
60793	NSSC	2020-10-05	23.97	24.22	23.42	23.81	75863	{}	\N	2020-10-12 15:45:51.57024
60794	NSSC	2020-10-06	23.57	24.43	22.9802	24.21	103272	{}	\N	2020-10-12 15:45:51.575785
60795	NSSC	2020-10-07	24.19	24.72	24	24.63	78738	{}	\N	2020-10-12 15:45:51.581148
60796	NSSC	2020-10-08	24.32	24.87	23.37	24.55	55752	{}	\N	2020-10-12 15:45:51.588466
60797	NSSC	2020-10-09	24.82	25.05	24.3125	24.85	36260	{}	\N	2020-10-12 15:45:51.593701
60798	NSSC	2020-10-12	24.99	25.5	24.57	24.8	17448	{}	\N	2020-10-12 15:45:51.599444
60799	NTIC	2020-09-28	8.31	8.49	8.255	8.36	17013	{}	\N	2020-10-12 15:45:52.573488
60800	NTIC	2020-09-29	8.46	8.725	8.41	8.45	8034	{}	\N	2020-10-12 15:45:52.580575
60801	NTIC	2020-09-30	8.36	8.4	8.3	8.3	2905	{}	\N	2020-10-12 15:45:52.58775
60802	NTIC	2020-10-01	8.35	8.72	8.05	8.25	9342	{}	\N	2020-10-12 15:45:52.59381
60803	NTIC	2020-10-02	8.1	8.62	8.1	8.28	3663	{}	\N	2020-10-12 15:45:52.5999
60804	NTIC	2020-10-05	8.3	8.645	8.2009	8.35	48832	{}	\N	2020-10-12 15:45:52.606688
60805	NTIC	2020-10-06	8.5	8.5	8.0101	8.25	6347	{}	\N	2020-10-12 15:45:52.612603
60806	NTIC	2020-10-07	8.32	8.8223	8.32	8.47	34232	{}	\N	2020-10-12 15:45:52.618732
60807	NTIC	2020-10-08	8.54	8.6599	8.4001	8.505	53752	{}	\N	2020-10-12 15:45:52.625722
60808	NTIC	2020-10-09	8.65	8.725	8.52	8.6	11883	{}	\N	2020-10-12 15:45:52.632135
60809	NTIC	2020-10-12	8.5	8.715	8.41	8.5	3924	{}	\N	2020-10-12 15:45:52.638315
60810	NTRS	2020-09-28	78.1	79.58	77.72	78.61	650708	{}	\N	2020-10-12 15:45:53.758045
60811	NTRS	2020-09-29	78.36	78.64	77.07	77.46	563252	{}	\N	2020-10-12 15:45:53.76342
60812	NTRS	2020-09-30	78.13	79.03	77.4	77.97	706621	{}	\N	2020-10-12 15:45:53.768459
60813	NTRS	2020-10-01	77.78	78.79	76.53	77.05	712402	{}	\N	2020-10-12 15:45:53.774774
60814	NTRS	2020-10-02	75.83	79.09	75.72	78.41	477164	{}	\N	2020-10-12 15:45:53.780611
60815	NTRS	2020-10-05	79.28	81.04	79.275	80.83	628487	{}	\N	2020-10-12 15:45:53.785926
60816	NTRS	2020-10-06	81.69	82.25	79.915	80.19	920780	{}	\N	2020-10-12 15:45:53.793486
60817	NTRS	2020-10-07	81.79	83.41	81.435	82.24	846103	{}	\N	2020-10-12 15:45:53.798891
60818	NTRS	2020-10-08	82.59	84.49	82.02	84.25	515106	{}	\N	2020-10-12 15:45:53.804376
60819	NTRS	2020-10-09	83.62	85.57	83.39	85.22	635695	{}	\N	2020-10-12 15:45:53.80954
60820	NTRS	2020-10-12	85.02	86.2	84.63	86.02	143350	{}	\N	2020-10-12 15:45:53.814648
60821	NULG	2020-09-28	50.76	50.76	50.3031	50.5448	30235	{}	\N	2020-10-12 15:45:54.820464
60822	NULG	2020-09-29	50.44	50.849	50.44	50.66	27043	{}	\N	2020-10-12 15:45:54.827486
60823	NULG	2020-09-30	50.57	51.355	50.57	50.92	16426	{}	\N	2020-10-12 15:45:54.834224
60824	NULG	2020-10-01	51.5	51.53	51.1923	51.33	44605	{}	\N	2020-10-12 15:45:54.840557
60825	NULG	2020-10-02	50.2	51.03	50.2	50.56	14288	{}	\N	2020-10-12 15:45:54.846202
60826	NULG	2020-10-05	50.77	51.39	50.77	51.375	22763	{}	\N	2020-10-12 15:45:54.85077
60827	NULG	2020-10-06	51.35	51.6227	50.7201	50.7432	41417	{}	\N	2020-10-12 15:45:54.856392
60828	NULG	2020-10-07	51.35	51.7	51.2099	51.62	28396	{}	\N	2020-10-12 15:45:54.861526
60829	NULG	2020-10-08	52.2	52.2	51.73	51.93	86696	{}	\N	2020-10-12 15:45:54.8665
60830	NULG	2020-10-09	52.2	52.71	52.2	52.68	110604	{}	\N	2020-10-12 15:45:54.871964
60831	NULG	2020-10-12	53.3	53.352	53	53.3401	6689	{}	\N	2020-10-12 15:45:54.879481
60832	NUS	2020-09-28	51.23	51.94	50.75	50.84	318821	{}	\N	2020-10-12 15:45:56.063462
60833	NUS	2020-09-29	51	51.005	48.75	48.91	802214	{}	\N	2020-10-12 15:45:56.06957
60834	NUS	2020-09-30	49.71	51.08	49.31	50.09	1084129	{}	\N	2020-10-12 15:45:56.075359
60835	NUS	2020-10-01	50.39	51.52	49.9	51.18	609127	{}	\N	2020-10-12 15:45:56.080531
60836	NUS	2020-10-02	55.52	57.01	53.5735	56.1	2722007	{}	\N	2020-10-12 15:45:56.086664
60837	NUS	2020-10-05	56	56.24	53.91	54.88	999475	{}	\N	2020-10-12 15:45:56.0929
60838	NUS	2020-10-06	54.75	55.84	54.32	54.61	643466	{}	\N	2020-10-12 15:45:56.099032
60839	NUS	2020-10-07	55.02	56.36	55	55.92	762894	{}	\N	2020-10-12 15:45:56.104199
60840	NUS	2020-10-08	55.94	56.42	54.56	55.14	498059	{}	\N	2020-10-12 15:45:56.111122
60841	NUS	2020-10-09	55.54	55.91	55.12	55.13	567797	{}	\N	2020-10-12 15:45:56.116433
60842	NUS	2020-10-12	55.58	55.795	54.845	55.23	126923	{}	\N	2020-10-12 15:45:56.121792
60843	NVCR	2020-09-28	107.08	109.17	106.36	108.32	770163	{}	\N	2020-10-12 15:45:59.079053
60844	NVCR	2020-09-29	108.72	110.79	108.21	109	623679	{}	\N	2020-10-12 15:45:59.086166
60845	NVCR	2020-09-30	109.4895	113.21	109.08	111.31	846262	{}	\N	2020-10-12 15:45:59.09403
60846	NVCR	2020-10-01	112.36	114.28	111.34	112.36	506723	{}	\N	2020-10-12 15:45:59.10119
60847	NVCR	2020-10-02	108.99	115.67	107.9	113.05	636581	{}	\N	2020-10-12 15:45:59.106954
60848	NVCR	2020-10-05	112.35	118.2	112.2	117.7	1140030	{}	\N	2020-10-12 15:45:59.113433
60849	NVCR	2020-10-06	118.2	121.25	116.59	119.35	831734	{}	\N	2020-10-12 15:45:59.118927
60850	NVCR	2020-10-07	123.26	129.41	123	127.02	1191445	{}	\N	2020-10-12 15:45:59.124057
60851	NVCR	2020-10-08	128.2	131.14	125.27	129.28	803846	{}	\N	2020-10-12 15:45:59.129919
60852	NVCR	2020-10-09	132.5	138.93	129.57	138.72	1346093	{}	\N	2020-10-12 15:45:59.135437
60853	NVCR	2020-10-12	140.21	140.5	131.81	134.245	492085	{}	\N	2020-10-12 15:45:59.140858
60854	NVR	2020-09-28	3971	4104.1	3971	4078.55	14253	{}	\N	2020-10-12 15:46:00.32001
60855	NVR	2020-09-29	4060.01	4065.66	3983.485	4001.56	21661	{}	\N	2020-10-12 15:46:00.329188
60856	NVR	2020-09-30	4035.17	4152.15	4018.995	4083.12	43746	{}	\N	2020-10-12 15:46:00.339702
60857	NVR	2020-10-01	4117.34	4240	4070.01	4228.31	37974	{}	\N	2020-10-12 15:46:00.347378
60858	NVR	2020-10-02	4188	4250	4120	4194.98	26823	{}	\N	2020-10-12 15:46:00.352742
60859	NVR	2020-10-05	4242.7	4265.2598	4133.015	4215.58	33916	{}	\N	2020-10-12 15:46:00.358119
60860	NVR	2020-10-06	4228.6	4268	4085.1	4145.26	47329	{}	\N	2020-10-12 15:46:00.364153
60861	NVR	2020-10-07	4188	4252.6987	4105.88	4208.82	34532	{}	\N	2020-10-12 15:46:00.369373
60862	NVR	2020-10-08	4243.81	4376.78	4208.82	4348.4	29747	{}	\N	2020-10-12 15:46:00.374913
60863	NVR	2020-10-09	4392.7	4428.1	4327.19	4399.01	35627	{}	\N	2020-10-12 15:46:00.381955
60864	NVR	2020-10-12	4410	4457.5	4375.0001	4439.03	6989	{}	\N	2020-10-12 15:46:00.387494
60865	NWS	2020-09-28	14.27	14.44	14.09	14.28	432214	{}	\N	2020-10-12 15:46:01.321663
60866	NWS	2020-09-29	14.08	14.14	13.69	13.77	282593	{}	\N	2020-10-12 15:46:01.329547
60867	NWS	2020-09-30	13.85	14.08	13.69	13.98	597403	{}	\N	2020-10-12 15:46:01.33634
60868	NWS	2020-10-01	13.97	14.15	13.55	14.15	377489	{}	\N	2020-10-12 15:46:01.345051
60869	NWS	2020-10-02	13.94	14.24	13.92	14.17	310256	{}	\N	2020-10-12 15:46:01.353678
60870	NWS	2020-10-05	14.21	14.39	14.09	14.32	466313	{}	\N	2020-10-12 15:46:01.359509
60871	NWS	2020-10-06	14.38	14.61	14.13	14.22	516032	{}	\N	2020-10-12 15:46:01.366327
60872	NWS	2020-10-07	14.33	14.42	14.14	14.16	331989	{}	\N	2020-10-12 15:46:01.37194
60873	NWS	2020-10-08	14.26	14.35	13.95	14.34	481665	{}	\N	2020-10-12 15:46:01.378925
60874	NWS	2020-10-09	14.33	14.41	14.135	14.155	378125	{}	\N	2020-10-12 15:46:01.385296
60875	NWS	2020-10-12	14.24	14.43	14.205	14.215	88962	{}	\N	2020-10-12 15:46:01.390698
60876	NXJ	2020-09-28	13.45	13.52	13.45	13.51	46791	{}	\N	2020-10-12 15:46:02.724299
60877	NXJ	2020-09-29	13.53	13.53	13.49	13.5	178036	{}	\N	2020-10-12 15:46:02.73381
60878	NXJ	2020-09-30	13.54	13.54	13.48	13.53	172705	{}	\N	2020-10-12 15:46:02.739262
60879	NXJ	2020-10-01	13.54	13.56	13.515	13.55	202766	{}	\N	2020-10-12 15:46:02.744557
60880	NXJ	2020-10-02	13.53	13.59	13.5	13.59	240264	{}	\N	2020-10-12 15:46:02.750769
60881	NXJ	2020-10-05	13.59	13.59	13.51	13.54	106474	{}	\N	2020-10-12 15:46:02.755247
60882	NXJ	2020-10-06	13.54	13.58	13.5127	13.52	83773	{}	\N	2020-10-12 15:46:02.759958
60883	NXJ	2020-10-07	13.57	13.58	13.52	13.56	99954	{}	\N	2020-10-12 15:46:02.765319
60884	NXJ	2020-10-08	13.58	13.59	13.55	13.57	37008	{}	\N	2020-10-12 15:46:02.770703
60885	NXJ	2020-10-09	13.58	13.63	13.58	13.61	39932	{}	\N	2020-10-12 15:46:02.775322
60886	NXJ	2020-10-12	13.68	13.68	13.65	13.67	34088	{}	\N	2020-10-12 15:46:02.779889
60887	NMFC	2020-09-28	9.52	9.93	9.52	9.78	552874	{}	\N	2020-10-12 15:46:04.10687
60888	NMFC	2020-09-29	9.85	9.85	9.46	9.57	343537	{}	\N	2020-10-12 15:46:04.112632
60889	NMFC	2020-09-30	9.84	9.8899	9.48	9.56	640332	{}	\N	2020-10-12 15:46:04.119147
60890	NMFC	2020-10-01	9.59	9.82	9.49	9.81	525535	{}	\N	2020-10-12 15:46:04.124614
60891	NMFC	2020-10-02	9.64	9.935	9.64	9.92	320249	{}	\N	2020-10-12 15:46:04.130334
60892	NMFC	2020-10-05	9.96	10.05	9.865	9.875	234859	{}	\N	2020-10-12 15:46:04.137494
60893	NMFC	2020-10-06	9.91	9.99	9.75	9.77	212535	{}	\N	2020-10-12 15:46:04.143232
60894	NMFC	2020-10-07	9.84	9.9	9.5899	9.75	239788	{}	\N	2020-10-12 15:46:04.150208
60895	NMFC	2020-10-08	9.75	9.9	9.65	9.86	356502	{}	\N	2020-10-12 15:46:04.157068
60896	NMFC	2020-10-09	9.89	9.92	9.79	9.84	253063	{}	\N	2020-10-12 15:46:04.162603
60897	NMFC	2020-10-12	9.93	9.93	9.78	9.78	137371	{}	\N	2020-10-12 15:46:04.169589
60898	NMZ	2020-09-28	13.35	13.43	13.3426	13.39	117497	{}	\N	2020-10-12 15:46:05.411655
60899	NMZ	2020-09-29	13.39	13.5	13.39	13.47	224768	{}	\N	2020-10-12 15:46:05.417729
60900	NMZ	2020-09-30	13.48	13.5	13.46	13.49	333420	{}	\N	2020-10-12 15:46:05.422788
60901	NMZ	2020-10-01	13.49	13.5	13.46	13.48	884992	{}	\N	2020-10-12 15:46:05.427783
60902	NMZ	2020-10-02	13.43	13.519	13.43	13.45	392858	{}	\N	2020-10-12 15:46:05.435596
60903	NMZ	2020-10-05	13.47	13.47	13.36	13.44	421566	{}	\N	2020-10-12 15:46:05.441495
60904	NMZ	2020-10-06	13.44	13.49	13.43	13.46	396225	{}	\N	2020-10-12 15:46:05.446692
60905	NMZ	2020-10-07	13.48	13.54	13.45	13.45	405646	{}	\N	2020-10-12 15:46:05.452719
60906	NMZ	2020-10-08	13.48	13.49	13.4	13.43	448476	{}	\N	2020-10-12 15:46:05.457903
60907	NMZ	2020-10-09	13.47	13.49	13.41	13.45	315104	{}	\N	2020-10-12 15:46:05.462822
60908	NMZ	2020-10-12	13.45	13.5	13.44	13.4447	148614	{}	\N	2020-10-12 15:46:05.468638
60909	NORW	2020-09-28	10.06	10.1	10.01	10.07	62821	{}	\N	2020-10-12 15:46:07.827918
60910	NORW	2020-09-29	10.1	10.15	10.007	10.08	28976	{}	\N	2020-10-12 15:46:07.833736
60911	NORW	2020-09-30	10.18	10.34	10.1201	10.34	25453	{}	\N	2020-10-12 15:46:07.838919
60912	NORW	2020-10-01	10.35	10.35	10.25	10.35	12168	{}	\N	2020-10-12 15:46:07.843967
60913	NORW	2020-10-02	10.2764	10.39	10.27	10.32	12954	{}	\N	2020-10-12 15:46:07.850211
60914	NORW	2020-10-05	10.51	10.705	10.51	10.67	46176	{}	\N	2020-10-12 15:46:07.857985
60915	NORW	2020-10-06	10.7038	10.76	10.411	10.411	15957	{}	\N	2020-10-12 15:46:07.86294
60916	NORW	2020-10-07	10.58	10.6	10.54	10.595	24907	{}	\N	2020-10-12 15:46:07.867938
60917	NORW	2020-10-08	10.67	10.75	10.67	10.7	11078	{}	\N	2020-10-12 15:46:07.873697
60918	NORW	2020-10-09	10.75	10.83	10.7171	10.82	7078	{}	\N	2020-10-12 15:46:07.878799
60919	NORW	2020-10-12	10.82	10.84	10.79	10.84	4818	{}	\N	2020-10-12 15:46:07.883661
60920	NR	2020-09-28	1.26	1.365	1.24	1.24	402675	{}	\N	2020-10-12 15:46:08.807282
60921	NR	2020-09-29	1.2	1.23	1.16	1.18	1626431	{}	\N	2020-10-12 15:46:08.814052
60922	NR	2020-09-30	1.19	1.24	1.05	1.05	1825388	{}	\N	2020-10-12 15:46:08.820037
60923	NR	2020-10-01	1.07	1.0885	0.9385	0.9385	4121439	{}	\N	2020-10-12 15:46:08.825922
60924	NR	2020-10-02	0.8601	0.8601	0.8	0.8	10606335	{}	\N	2020-10-12 15:46:08.831355
60925	NR	2020-10-05	0.84	0.9947	0.83	0.8671	14612098	{}	\N	2020-10-12 15:46:08.836732
60926	NR	2020-10-06	0.86	0.927	0.78	0.81	38949977	{}	\N	2020-10-12 15:46:08.842466
60927	NR	2020-10-07	0.8201	0.8826	0.8158	0.8611	7320736	{}	\N	2020-10-12 15:46:08.847443
60928	NR	2020-10-08	0.8702	0.9899	0.866	0.9662	6533958	{}	\N	2020-10-12 15:46:08.851736
60929	NR	2020-10-09	1	1.0199	0.8885	0.9063	2992022	{}	\N	2020-10-12 15:46:08.857396
60930	NR	2020-10-12	0.91	0.9101	0.8152	0.85	1988336	{}	\N	2020-10-12 15:46:08.862813
60931	NSEC	2020-09-28	11.8	11.8	11.75	11.75	578	{}	\N	2020-10-12 15:46:10.237948
60932	NSEC	2020-10-01	11.86	11.86	11.86	11.86	252	{}	\N	2020-10-12 15:46:10.243846
60933	NSEC	2020-10-02	11	11	11	11	1064	{}	\N	2020-10-12 15:46:10.249379
60934	NSEC	2020-10-05	10.89	10.9	10.89	10.9	2707	{}	\N	2020-10-12 15:46:10.254891
60935	NSEC	2020-10-09	10.95	11.86	10.95	11.86	1264	{}	\N	2020-10-12 15:46:10.261834
60936	NSTG	2020-09-28	44.66	45	43.68	44.36	286499	{}	\N	2020-10-12 15:46:11.395066
60937	NSTG	2020-09-29	44.66	46.19	44.66	45.69	417648	{}	\N	2020-10-12 15:46:11.403428
60938	NSTG	2020-09-30	45.84	46.23	43.985	44.7	279693	{}	\N	2020-10-12 15:46:11.410869
60939	NSTG	2020-10-01	44.11	45.11	43.65	45.01	274695	{}	\N	2020-10-12 15:46:11.417181
60940	NSTG	2020-10-02	44.24	45.17	43.88	43.98	230415	{}	\N	2020-10-12 15:46:11.424124
60941	NSTG	2020-10-05	43.99	45.17	43.81	44.23	286490	{}	\N	2020-10-12 15:46:11.430706
60942	NSTG	2020-10-06	44.33	44.98	43.5	43.71	410223	{}	\N	2020-10-12 15:46:11.437685
60943	NSTG	2020-10-07	41.7	45.235	41.03	41.96	1405944	{}	\N	2020-10-12 15:46:11.443794
60944	NSTG	2020-10-08	40.98	41.75	39.5	39.78	2844421	{}	\N	2020-10-12 15:46:11.449581
60945	NSTG	2020-10-09	40.79	41.63	39.8	40.665	2863292	{}	\N	2020-10-12 15:46:11.455153
60946	NSTG	2020-10-12	41.51	41.7	40.74	41.16	599990	{}	\N	2020-10-12 15:46:11.461075
60947	NTGR	2020-09-28	29.92	30.44	29.81	30.06	297069	{}	\N	2020-10-12 15:46:12.444862
60948	NTGR	2020-09-29	30.25	31.39	30.24	31.01	349507	{}	\N	2020-10-12 15:46:12.450781
60949	NTGR	2020-09-30	31	31.42	30.58	30.85	384881	{}	\N	2020-10-12 15:46:12.455788
60950	NTGR	2020-10-01	31.07	31.43	30.5	31.38	230847	{}	\N	2020-10-12 15:46:12.462228
60951	NTGR	2020-10-02	30.56	31.57	30.42	31.32	156926	{}	\N	2020-10-12 15:46:12.468331
60952	NTGR	2020-10-05	31.52	32.68	31.37	32.65	231279	{}	\N	2020-10-12 15:46:12.476253
60953	NTGR	2020-10-06	32.67	32.98	31.92	32.05	357737	{}	\N	2020-10-12 15:46:12.483949
60954	NTGR	2020-10-07	32.1	32.75	31.68	32.05	243425	{}	\N	2020-10-12 15:46:12.489265
60955	NTGR	2020-10-08	32.38	33.4899	32.38	33.17	240189	{}	\N	2020-10-12 15:46:12.495038
60956	NTGR	2020-10-09	33.49	34.21	33.34	34.21	228034	{}	\N	2020-10-12 15:46:12.499612
60957	NTGR	2020-10-12	34.44	34.44	33.89	33.89	48208	{}	\N	2020-10-12 15:46:12.503924
60958	NUBD	2020-09-28	27.01	27.01	26.9389	26.98	65217	{}	\N	2020-10-12 15:46:13.864392
60959	NUBD	2020-09-29	26.9648	27	26.95	26.96	27697	{}	\N	2020-10-12 15:46:13.870739
60960	NUBD	2020-09-30	27	27	26.9001	26.94	19538	{}	\N	2020-10-12 15:46:13.875797
60961	NUBD	2020-10-01	26.87	26.96	26.87	26.924	38909	{}	\N	2020-10-12 15:46:13.881836
60962	NUBD	2020-10-02	26.9	26.96	26.86	26.89	33109	{}	\N	2020-10-12 15:46:13.887315
60963	NUBD	2020-10-05	26.86	26.86	26.79	26.81	23939	{}	\N	2020-10-12 15:46:13.89382
60964	NUBD	2020-10-06	26.82	26.9	26.796	26.86	62634	{}	\N	2020-10-12 15:46:13.900067
60965	NUBD	2020-10-07	26.78	26.86	26.78	26.85	93258	{}	\N	2020-10-12 15:46:13.905174
60966	NUBD	2020-10-08	26.79	26.89	26.79	26.81	34445	{}	\N	2020-10-12 15:46:13.910337
60967	NUBD	2020-10-09	26.8196	26.84	26.7927	26.84	177528	{}	\N	2020-10-12 15:46:13.915891
60968	NUBD	2020-10-12	26.83	26.87	26.83	26.8598	4205	{}	\N	2020-10-12 15:46:13.920792
60969	NUM	2020-09-28	14.17	14.25	14.17	14.19	31364	{}	\N	2020-10-12 15:46:14.818706
60970	NUM	2020-09-29	14.17	14.2211	14.14	14.1825	42622	{}	\N	2020-10-12 15:46:14.824899
60971	NUM	2020-09-30	14.24	14.24	14.13	14.18	68356	{}	\N	2020-10-12 15:46:14.830866
60972	NUM	2020-10-01	14.31	14.31	14.17	14.19	51150	{}	\N	2020-10-12 15:46:14.836038
60973	NUM	2020-10-02	14.07	14.27	14.07	14.27	45632	{}	\N	2020-10-12 15:46:14.842278
60974	NUM	2020-10-05	14.27	14.285	14.16	14.21	111524	{}	\N	2020-10-12 15:46:14.848278
60975	NUM	2020-10-06	14.17	14.23	14.17	14.22	60348	{}	\N	2020-10-12 15:46:14.854422
60976	NUM	2020-10-07	14.2	14.23	14.19	14.21	68874	{}	\N	2020-10-12 15:46:14.859753
60977	NUM	2020-10-08	14.17	14.22	14.15	14.2	126426	{}	\N	2020-10-12 15:46:14.865803
60978	NUM	2020-10-09	14.21	14.25	14.18	14.2	96790	{}	\N	2020-10-12 15:46:14.871173
60979	NUM	2020-10-12	14.2	14.225	14.18	14.19	75568	{}	\N	2020-10-12 15:46:14.876448
60980	NUV	2020-09-28	10.58	10.62	10.535	10.58	212081	{}	\N	2020-10-12 15:46:16.082461
60981	NUV	2020-09-29	10.61	10.67	10.6	10.64	484730	{}	\N	2020-10-12 15:46:16.088927
60982	NUV	2020-09-30	10.64	10.66	10.605	10.66	674027	{}	\N	2020-10-12 15:46:16.094486
60983	NUV	2020-10-01	10.68	10.685	10.62	10.67	448970	{}	\N	2020-10-12 15:46:16.101161
60984	NUV	2020-10-02	10.66	10.71	10.64	10.7	469112	{}	\N	2020-10-12 15:46:16.108643
60985	NUV	2020-10-05	10.66	10.69	10.62	10.64	558704	{}	\N	2020-10-12 15:46:16.116165
60986	NUV	2020-10-06	10.64	10.72	10.61	10.69	394848	{}	\N	2020-10-12 15:46:16.121557
60987	NUV	2020-10-07	10.7	10.75	10.6796	10.74	417106	{}	\N	2020-10-12 15:46:16.127214
60988	NUV	2020-10-08	10.75	10.75	10.7055	10.73	392558	{}	\N	2020-10-12 15:46:16.133457
60989	NUV	2020-10-09	10.73	10.85	10.7	10.82	1198850	{}	\N	2020-10-12 15:46:16.139194
60990	NUV	2020-10-12	10.88	10.88	10.83	10.88	211932	{}	\N	2020-10-12 15:46:16.145331
60991	NVGS	2020-09-28	8.31	8.68	8.28	8.42	39714	{}	\N	2020-10-12 15:46:17.059091
60992	NVGS	2020-09-29	8.26	8.43	8.05	8.14	83500	{}	\N	2020-10-12 15:46:17.066229
60993	NVGS	2020-09-30	8.13	8.57	8.13	8.36	65021	{}	\N	2020-10-12 15:46:17.071658
60994	NVGS	2020-10-01	8.27	8.51	8.27	8.5	38969	{}	\N	2020-10-12 15:46:17.077028
60995	NVGS	2020-10-02	8.27	8.77	8.165	8.54	85823	{}	\N	2020-10-12 15:46:17.083701
60996	NVGS	2020-10-05	8.54	8.91	8.522	8.78	44985	{}	\N	2020-10-12 15:46:17.088665
60997	NVGS	2020-10-06	8.93	9.01	8.38	8.55	58108	{}	\N	2020-10-12 15:46:17.093805
60998	NVGS	2020-10-07	8.54	9.4	8.54	9.09	66880	{}	\N	2020-10-12 15:46:17.099732
60999	NVGS	2020-10-08	9.02	9.17	8.9565	9.06	45208	{}	\N	2020-10-12 15:46:17.10517
61000	NVGS	2020-10-09	9.06	9.17	8.81	8.96	61921	{}	\N	2020-10-12 15:46:17.111112
61001	NVGS	2020-10-12	8.88	8.93	8.6101	8.76	15110	{}	\N	2020-10-12 15:46:17.116952
61002	NVTA	2020-09-28	45.24	46.17	43.23	43.9	2675723	{}	\N	2020-10-12 15:46:18.298586
61003	NVTA	2020-09-29	44.35	45.94	44.04	44.29	4217621	{}	\N	2020-10-12 15:46:18.3109
61004	NVTA	2020-09-30	44.68	44.97	42.63	43.35	7255858	{}	\N	2020-10-12 15:46:18.320904
61005	NVTA	2020-10-01	43.98	44.63	43.38	44.07	3253407	{}	\N	2020-10-12 15:46:18.329886
61006	NVTA	2020-10-02	42.75	44.19	42.23	42.43	3712666	{}	\N	2020-10-12 15:46:18.338683
61007	NVTA	2020-10-05	43.75	45.6	43.47	45.4	4166973	{}	\N	2020-10-12 15:46:18.345612
61008	NVTA	2020-10-06	45.6	50.176	45.5	47.51	7361955	{}	\N	2020-10-12 15:46:18.351353
61009	NVTA	2020-10-07	48.82	51.44	48.35	50.08	6991564	{}	\N	2020-10-12 15:46:18.356411
61010	NVTA	2020-10-08	51.52	53.475	49.82	52.83	7296430	{}	\N	2020-10-12 15:46:18.362767
61011	NVTA	2020-10-09	52.93	54.49	52.16	53.31	5110268	{}	\N	2020-10-12 15:46:18.369096
61012	NVTA	2020-10-12	54.17	55.38	52.84	54.52	2165141	{}	\N	2020-10-12 15:46:18.374551
61013	NWSA	2020-09-28	14.35	14.545	14.17	14.34	2543559	{}	\N	2020-10-12 15:46:19.451865
61014	NWSA	2020-09-29	14.11	14.24	13.78	13.89	1573275	{}	\N	2020-10-12 15:46:19.458563
61015	NWSA	2020-09-30	13.97	14.15	13.895	14.02	2244361	{}	\N	2020-10-12 15:46:19.464124
61016	NWSA	2020-10-01	13.97	14.23	13.97	14.18	1435703	{}	\N	2020-10-12 15:46:19.470388
61017	NWSA	2020-10-02	14.01	14.315	13.89	14.215	1384809	{}	\N	2020-10-12 15:46:19.476688
61018	NWSA	2020-10-05	14.27	14.43	14.135	14.34	2567870	{}	\N	2020-10-12 15:46:19.483585
61019	NWSA	2020-10-06	14.41	14.7	14.13	14.31	1944224	{}	\N	2020-10-12 15:46:19.492105
61020	NWSA	2020-10-07	14.62	14.62	14.23	14.23	1843157	{}	\N	2020-10-12 15:46:19.497787
61021	NWSA	2020-10-08	14.27	14.445	14.035	14.44	2184827	{}	\N	2020-10-12 15:46:19.503703
61022	NWSA	2020-10-09	14.57	14.57	14.22	14.245	1756287	{}	\N	2020-10-12 15:46:19.509303
61023	NWSA	2020-10-12	14.36	14.54	14.29	14.29	346836	{}	\N	2020-10-12 15:46:19.51475
61024	NNY	2020-09-28	10.35	10.35	10.13	10.13	15120	{}	\N	2020-10-12 15:46:20.662681
61025	NNY	2020-09-29	10.18	10.18	10	10	31192	{}	\N	2020-10-12 15:46:20.667767
61026	NNY	2020-09-30	10.04	10.1	10.03	10.1	12880	{}	\N	2020-10-12 15:46:20.672741
61027	NNY	2020-10-01	10.2	10.2	9.88	9.95	84570	{}	\N	2020-10-12 15:46:20.677367
61028	NNY	2020-10-02	9.93	10.04	9.93	9.97	15780	{}	\N	2020-10-12 15:46:20.682541
61029	NNY	2020-10-05	10.2	10.2	10	10.0548	43234	{}	\N	2020-10-12 15:46:20.688382
61030	NNY	2020-10-06	10.05	10.1	10.05	10.05	10702	{}	\N	2020-10-12 15:46:20.695375
61031	NNY	2020-10-07	10.08	10.08	9.98	9.98	25556	{}	\N	2020-10-12 15:46:20.701736
61032	NNY	2020-10-08	10.01	10.13	10.01	10.0721	16256	{}	\N	2020-10-12 15:46:20.70703
61033	NNY	2020-10-09	10.07	10.1	9.96	9.96	23072	{}	\N	2020-10-12 15:46:20.711976
61034	NNY	2020-10-12	10	10	9.96	9.96	11842	{}	\N	2020-10-12 15:46:20.71669
61035	NOV	2020-09-28	9.82	10.1	9.71	9.8	6497353	{}	\N	2020-10-12 15:46:21.697713
61036	NOV	2020-09-29	9.68	9.705	9.05	9.21	15118309	{}	\N	2020-10-12 15:46:21.70405
61037	NOV	2020-09-30	9.32	9.515	8.96	9.06	13830216	{}	\N	2020-10-12 15:46:21.708489
61038	NOV	2020-10-01	8.89	8.935	8.52	8.66	15103199	{}	\N	2020-10-12 15:46:21.712845
61039	NOV	2020-10-02	8.19	8.895	8.13	8.7	11387430	{}	\N	2020-10-12 15:46:21.71761
61040	NOV	2020-10-05	8.89	9.07	8.835	8.96	7322218	{}	\N	2020-10-12 15:46:21.722878
61041	NOV	2020-10-06	9.17	9.3	8.62	8.67	10166572	{}	\N	2020-10-12 15:46:21.727337
61042	NOV	2020-10-07	8.71	8.85	8.59	8.77	6915966	{}	\N	2020-10-12 15:46:21.731749
61043	NOV	2020-10-08	8.87	9.26	8.81	9.25	9990883	{}	\N	2020-10-12 15:46:21.737916
61044	NOV	2020-10-09	9.34	9.38	8.98	9.05	6270469	{}	\N	2020-10-12 15:46:21.742452
61045	NOV	2020-10-12	9	9.08	8.75	8.835	2123511	{}	\N	2020-10-12 15:46:21.747024
61046	NPO	2020-09-28	57.33	58.8765	57.33	57.87	121032	{}	\N	2020-10-12 15:46:22.617598
61047	NPO	2020-09-29	57.75	57.79	56.12	57.15	116250	{}	\N	2020-10-12 15:46:22.62453
61048	NPO	2020-09-30	57.25	58.43	55.83	56.41	161575	{}	\N	2020-10-12 15:46:22.629842
61049	NPO	2020-10-01	56.75	57.06	55.77	56.82	147000	{}	\N	2020-10-12 15:46:22.634905
61050	NPO	2020-10-02	55.57	57.75	55.19	57.1	125793	{}	\N	2020-10-12 15:46:22.641263
61051	NPO	2020-10-05	57.59	59.4	56.96	59.31	335166	{}	\N	2020-10-12 15:46:22.645651
61052	NPO	2020-10-06	59.82	61.185	58.42	58.5	239449	{}	\N	2020-10-12 15:46:22.650352
61053	NPO	2020-10-07	59.23	59.76	58.27	58.66	247117	{}	\N	2020-10-12 15:46:22.65629
61054	NPO	2020-10-08	59.67	60.49	58.54	60.3	139322	{}	\N	2020-10-12 15:46:22.662126
61055	NPO	2020-10-09	61.06	61.29	59.71	59.96	90560	{}	\N	2020-10-12 15:46:22.667263
61056	NPO	2020-10-12	59.94	60.545	58.92	60.26	13840	{}	\N	2020-10-12 15:46:22.672656
61057	NRK	2020-09-28	12.87	12.92	12.87	12.91	127800	{}	\N	2020-10-12 15:46:23.825713
61058	NRK	2020-09-29	12.93	12.93	12.84	12.86	251297	{}	\N	2020-10-12 15:46:23.831366
61059	NRK	2020-09-30	12.84	12.89	12.84	12.86	173330	{}	\N	2020-10-12 15:46:23.837032
61060	NRK	2020-10-01	12.91	12.91	12.83	12.86	367688	{}	\N	2020-10-12 15:46:23.842388
61061	NRK	2020-10-02	12.81	12.88	12.77	12.88	244892	{}	\N	2020-10-12 15:46:23.847424
61062	NRK	2020-10-05	12.91	12.91	12.84	12.86	219990	{}	\N	2020-10-12 15:46:23.852766
61063	NRK	2020-10-06	12.86	12.88	12.81	12.82	131711	{}	\N	2020-10-12 15:46:23.857674
61064	NRK	2020-10-07	12.83	12.88	12.83	12.87	463790	{}	\N	2020-10-12 15:46:23.861954
61065	NRK	2020-10-08	12.89	12.94	12.86	12.94	547246	{}	\N	2020-10-12 15:46:23.86834
61066	NRK	2020-10-09	12.95	12.98	12.886	12.98	149786	{}	\N	2020-10-12 15:46:23.874226
61067	NRK	2020-10-12	12.98	12.9814	12.97	12.9747	93500	{}	\N	2020-10-12 15:46:23.878687
61068	NSIT	2020-09-28	56.09	57.23	55.85	56.44	226037	{}	\N	2020-10-12 15:46:24.951463
61069	NSIT	2020-09-29	56.78	57.03	55.64	56.39	157635	{}	\N	2020-10-12 15:46:24.957602
61070	NSIT	2020-09-30	56.54	57.71	56.28	56.58	310858	{}	\N	2020-10-12 15:46:24.962531
61071	NSIT	2020-10-01	57.7295	57.7295	56.42	57.41	136018	{}	\N	2020-10-12 15:46:24.967093
61072	NSIT	2020-10-02	56.28	57.71	56.03	57.27	157493	{}	\N	2020-10-12 15:46:24.972557
61073	NSIT	2020-10-05	58	59.2	57.84	58.77	150233	{}	\N	2020-10-12 15:46:24.978576
61074	NSIT	2020-10-06	59.27	60.45	58.625	58.64	200081	{}	\N	2020-10-12 15:46:24.983194
61075	NSIT	2020-10-07	59.17	60.25	58.95	60.1	178770	{}	\N	2020-10-12 15:46:24.989351
61076	NSIT	2020-10-08	60.87	61.38	60.64	60.8	230125	{}	\N	2020-10-12 15:46:24.994613
61077	NSIT	2020-10-09	61.1968	61.75	61.06	61.18	143012	{}	\N	2020-10-12 15:46:24.999646
61078	NSIT	2020-10-12	61.18	62.17	60.7101	61.975	26930	{}	\N	2020-10-12 15:46:25.004406
61079	NTB	2020-09-28	22.13	22.81	22.13	22.61	288306	{}	\N	2020-10-12 15:46:25.911772
61080	NTB	2020-09-29	22.62	22.62	21.95	21.95	770819	{}	\N	2020-10-12 15:46:25.917082
61081	NTB	2020-09-30	21.97	22.68	21.97	22.28	367189	{}	\N	2020-10-12 15:46:25.921746
61082	NTB	2020-10-01	22.26	22.82	22.05	22.79	388445	{}	\N	2020-10-12 15:46:25.926973
61083	NTB	2020-10-02	22.39	23.669	22.39	23.47	317202	{}	\N	2020-10-12 15:46:25.932463
61084	NTB	2020-10-05	23.86	24.61	23.73	24.5	318021	{}	\N	2020-10-12 15:46:25.938026
61085	NTB	2020-10-06	25.02	25.9899	24.8	24.81	404865	{}	\N	2020-10-12 15:46:25.94277
61086	NTB	2020-10-07	25.12	25.5899	24.72	24.89	294090	{}	\N	2020-10-12 15:46:25.948097
61087	NTB	2020-10-08	25.49	25.99	25.09	25.53	362087	{}	\N	2020-10-12 15:46:25.952821
61088	NTB	2020-10-09	25.83	25.83	25.09	25.15	228224	{}	\N	2020-10-12 15:46:25.957706
61089	NTB	2020-10-12	25.15	25.64	25.02	25.52	100379	{}	\N	2020-10-12 15:46:25.962374
61090	NTN	2020-09-28	1.69	1.9	1.69	1.77	301350	{}	\N	2020-10-12 15:46:27.333796
61091	NTN	2020-09-29	1.79	1.82	1.7	1.71	271529	{}	\N	2020-10-12 15:46:27.340818
61092	NTN	2020-09-30	1.74	1.81	1.7	1.7	200140	{}	\N	2020-10-12 15:46:27.349984
61093	NTN	2020-10-01	1.71	1.77	1.61	1.67	175104	{}	\N	2020-10-12 15:46:27.35662
61094	NTN	2020-10-02	1.62	1.77	1.62	1.71	163508	{}	\N	2020-10-12 15:46:27.36456
61095	NTN	2020-10-05	1.7	1.8	1.68	1.69	149488	{}	\N	2020-10-12 15:46:27.371201
61096	NTN	2020-10-06	1.75	1.76	1.68	1.7	116457	{}	\N	2020-10-12 15:46:27.377594
61097	NTN	2020-10-07	1.73	1.83	1.67	1.78	645174	{}	\N	2020-10-12 15:46:27.383379
61098	NTN	2020-10-08	1.85	1.925	1.78	1.79	549204	{}	\N	2020-10-12 15:46:27.38939
61099	NTN	2020-10-09	1.79	1.87	1.75	1.75	161499	{}	\N	2020-10-12 15:46:27.395669
61100	NTN	2020-10-12	1.8	1.8	1.75	1.78	29344	{}	\N	2020-10-12 15:46:27.400901
61101	NUGT	2020-09-28	83.86	84.77	81.42	83.59	2664353	{}	\N	2020-10-12 15:46:28.500459
61102	NUGT	2020-09-29	84.41	86.76	83.87	85.75	2664856	{}	\N	2020-10-12 15:46:28.509192
61103	NUGT	2020-09-30	84.48	86.32	82.13	84.79	3029091	{}	\N	2020-10-12 15:46:28.514626
61104	NUGT	2020-10-01	86.55	88.14	84.71	86.47	2874511	{}	\N	2020-10-12 15:46:28.51997
61105	NUGT	2020-10-02	84.43	86.48	83.04	83.83	2286055	{}	\N	2020-10-12 15:46:28.526355
61106	NUGT	2020-10-05	84.95	88.26	84.3541	86.56	2592652	{}	\N	2020-10-12 15:46:28.533149
61107	NUGT	2020-10-06	88.03	88.6	79.9102	80.15	3371868	{}	\N	2020-10-12 15:46:28.540083
61108	NUGT	2020-10-07	81.8	83.03	80.28	81.3	3297542	{}	\N	2020-10-12 15:46:28.546352
61109	NUGT	2020-10-08	82.74	84.55	81.62	83.6	2559184	{}	\N	2020-10-12 15:46:28.552177
61110	NUGT	2020-10-09	87.76	91.5	87.42	91.42	3377614	{}	\N	2020-10-12 15:46:28.557488
61111	NUGT	2020-10-12	91	91.5499	89.55	90.83	1327711	{}	\N	2020-10-12 15:46:28.563417
61112	NUSC	2020-09-28	29.34	29.59	29.2281	29.52	31196	{}	\N	2020-10-12 15:46:29.485453
61113	NUSC	2020-09-29	29.65	29.65	29.3101	29.49	30308	{}	\N	2020-10-12 15:46:29.491922
61114	NUSC	2020-09-30	29.69	29.935	29.45	29.65	40271	{}	\N	2020-10-12 15:46:29.497639
61115	NUSC	2020-10-01	29.78	30.06	29.6445	30.05	94557	{}	\N	2020-10-12 15:46:29.502772
61116	NUSC	2020-10-02	29.46	30.3699	29.46	30.28	51888	{}	\N	2020-10-12 15:46:29.508575
61117	NUSC	2020-10-05	30.65	31.06	30.65	31.02	27811	{}	\N	2020-10-12 15:46:29.514572
61118	NUSC	2020-10-06	31.14	31.76	30.96	31.03	68362	{}	\N	2020-10-12 15:46:29.519758
61119	NUSC	2020-10-07	31.55	31.69	31.45	31.64	59773	{}	\N	2020-10-12 15:46:29.525321
61120	NUSC	2020-10-08	31.87	32.06	31.8	32.05	88655	{}	\N	2020-10-12 15:46:29.532034
61121	NUSC	2020-10-09	32.38	32.38	32.11	32.26	90717	{}	\N	2020-10-12 15:46:29.538289
61122	NUSC	2020-10-12	32.59	32.59	32.295	32.41	15691	{}	\N	2020-10-12 15:46:29.543785
61123	NVEE	2020-09-28	51.61	53.51	51.61	53.01	45629	{}	\N	2020-10-12 15:46:30.447476
61124	NVEE	2020-09-29	52.6	54.21	52.6	53.51	43133	{}	\N	2020-10-12 15:46:30.453716
61125	NVEE	2020-09-30	53.99	54.95	52.21	52.77	67576	{}	\N	2020-10-12 15:46:30.45881
61126	NVEE	2020-10-01	52.43	53.14	52	52.97	40195	{}	\N	2020-10-12 15:46:30.465258
61127	NVEE	2020-10-02	51.71	55.14	51.71	54.82	39885	{}	\N	2020-10-12 15:46:30.471724
61128	NVEE	2020-10-05	55.38	56.4	55.03	55.73	57483	{}	\N	2020-10-12 15:46:30.478761
61129	NVEE	2020-10-06	56	58.11	55.86	56.555	69924	{}	\N	2020-10-12 15:46:30.486218
61130	NVEE	2020-10-07	57.21	59.16	57.02	57.71	53244	{}	\N	2020-10-12 15:46:30.49343
61131	NVEE	2020-10-08	58.54	59.38	58.0395	59.1	43006	{}	\N	2020-10-12 15:46:30.50049
61132	NVEE	2020-10-09	59.62	59.91	59	59.42	55720	{}	\N	2020-10-12 15:46:30.505659
61133	NVEE	2020-10-12	59.48	59.93	59	59.89	14510	{}	\N	2020-10-12 15:46:30.510762
61134	NVRO	2020-09-28	142.35	143.1	139.46	139.76	173610	{}	\N	2020-10-12 15:46:31.485429
61135	NVRO	2020-09-29	140.25	142.44	137.92	138.72	397877	{}	\N	2020-10-12 15:46:31.493505
61136	NVRO	2020-09-30	139.09	140.62	137.27	139.3	534182	{}	\N	2020-10-12 15:46:31.50021
61137	NVRO	2020-10-01	140.55	141.37	138.87	139.14	427531	{}	\N	2020-10-12 15:46:31.505885
61138	NVRO	2020-10-02	137.22	139.56	135.97	138.17	452691	{}	\N	2020-10-12 15:46:31.511709
61139	NVRO	2020-10-05	139.16	142.23	138.76	141.91	336893	{}	\N	2020-10-12 15:46:31.51717
61140	NVRO	2020-10-06	142.49	144.485	140.07	140.66	238157	{}	\N	2020-10-12 15:46:31.522383
61141	NVRO	2020-10-07	142.14	145.75	139.74	144.8	343753	{}	\N	2020-10-12 15:46:31.528593
61142	NVRO	2020-10-08	146	147.87	143.63	147.44	375204	{}	\N	2020-10-12 15:46:31.535552
61143	NVRO	2020-10-09	149.21	150.98	147.37	150.07	334456	{}	\N	2020-10-12 15:46:31.540916
61144	NVRO	2020-10-12	151.89	152.64	150.6	152.24	88327	{}	\N	2020-10-12 15:46:31.545958
61145	NWL	2020-09-28	16.98	17.3	16.82	17.22	1794029	{}	\N	2020-10-12 15:46:32.469924
61146	NWL	2020-09-29	17.09	17.25	16.975	17.12	1196319	{}	\N	2020-10-12 15:46:32.476103
61147	NWL	2020-09-30	17.22	17.46	17.11	17.16	1879628	{}	\N	2020-10-12 15:46:32.482198
61148	NWL	2020-10-01	17.31	17.35	16.87	17.01	2151356	{}	\N	2020-10-12 15:46:32.48761
61149	NWL	2020-10-02	16.72	17.52	16.66	17.38	1732786	{}	\N	2020-10-12 15:46:32.493136
61150	NWL	2020-10-05	17.57	17.67	17.44	17.6	1589532	{}	\N	2020-10-12 15:46:32.499108
61151	NWL	2020-10-06	17.68	17.86	17.29	17.31	1511914	{}	\N	2020-10-12 15:46:32.505531
61152	NWL	2020-10-07	17.47	17.83	17.47	17.7	1111996	{}	\N	2020-10-12 15:46:32.512187
61153	NWL	2020-10-08	17.8	18.11	17.76	18.1	1435902	{}	\N	2020-10-12 15:46:32.518249
61154	NWL	2020-10-09	18.23	18.27	17.97	18.01	1326726	{}	\N	2020-10-12 15:46:32.523768
61155	NWL	2020-10-12	18.08	18.13	17.965	18.0108	352381	{}	\N	2020-10-12 15:46:32.529042
61156	NUVA	2020-09-28	50.3	50.89	49.1	49.15	826523	{}	\N	2020-10-12 15:46:33.998199
61157	NUVA	2020-09-29	49.13	49.5	47	47.43	1114026	{}	\N	2020-10-12 15:46:34.004767
61158	NUVA	2020-09-30	47.68	50.3843	47.52	48.57	918691	{}	\N	2020-10-12 15:46:34.012463
61159	NUVA	2020-10-01	48.47	49.51	48.2	48.97	639721	{}	\N	2020-10-12 15:46:34.018792
61160	NUVA	2020-10-02	47.89	49.58	47.89	48.56	484115	{}	\N	2020-10-12 15:46:34.024975
61161	NUVA	2020-10-05	48.92	49.879	48.2	48.62	351369	{}	\N	2020-10-12 15:46:34.031893
61162	NUVA	2020-10-06	48.83	50.46	48.48	49.02	530170	{}	\N	2020-10-12 15:46:34.038417
61163	NUVA	2020-10-07	49.25	52.4	49.25	52.17	893234	{}	\N	2020-10-12 15:46:34.045187
61164	NUVA	2020-10-08	52.92	54.1	52.43	53.65	780299	{}	\N	2020-10-12 15:46:34.051609
61165	NUVA	2020-10-09	53.84	54.84	53.36	54.04	501277	{}	\N	2020-10-12 15:46:34.056912
61166	NUVA	2020-10-12	54.2	55.34	53.99	55.29	196194	{}	\N	2020-10-12 15:46:34.062951
61167	NVG	2020-09-28	15.48	15.55	15.48	15.52	321825	{}	\N	2020-10-12 15:46:35.165557
61168	NVG	2020-09-29	15.54	15.63	15.54	15.63	384254	{}	\N	2020-10-12 15:46:35.177903
61169	NVG	2020-09-30	15.65	15.66	15.57	15.66	532784	{}	\N	2020-10-12 15:46:35.18722
61170	NVG	2020-10-01	15.71	15.73	15.65	15.69	603300	{}	\N	2020-10-12 15:46:35.194429
61171	NVG	2020-10-02	15.67	15.77	15.6506	15.76	424915	{}	\N	2020-10-12 15:46:35.202525
61172	NVG	2020-10-05	15.76	15.7923	15.61	15.69	620420	{}	\N	2020-10-12 15:46:35.209726
61173	NVG	2020-10-06	15.72	15.82	15.69	15.75	608307	{}	\N	2020-10-12 15:46:35.219076
61174	NVG	2020-10-07	15.8	15.8068	15.71	15.74	460074	{}	\N	2020-10-12 15:46:35.226183
61175	NVG	2020-10-08	15.78	15.83	15.6	15.63	824372	{}	\N	2020-10-12 15:46:35.232728
61176	NVG	2020-10-09	15.66	15.8	15.63	15.77	408460	{}	\N	2020-10-12 15:46:35.238663
61177	NVG	2020-10-12	15.82	15.82	15.73	15.76	224412	{}	\N	2020-10-12 15:46:35.244096
61178	NVS	2020-09-28	87.36	87.74	86.99	87.05	1014181	{}	\N	2020-10-12 15:46:36.221368
61179	NVS	2020-09-29	88.1	88.1	86.68	87.04	7596652	{}	\N	2020-10-12 15:46:36.228438
61180	NVS	2020-09-30	87.29	87.47	86.48	86.96	5235876	{}	\N	2020-10-12 15:46:36.235584
61181	NVS	2020-10-01	88.3	88.32	87.04	87.3	3891423	{}	\N	2020-10-12 15:46:36.244403
61182	NVS	2020-10-02	86.88	87.81	86.76	87.52	3346455	{}	\N	2020-10-12 15:46:36.252533
61183	NVS	2020-10-05	88.2	88.49	87.97	88.27	4974079	{}	\N	2020-10-12 15:46:36.262448
61184	NVS	2020-10-06	87.86	87.92	86.3301	86.45	2774734	{}	\N	2020-10-12 15:46:36.269941
61185	NVS	2020-10-07	86.03	86.27	85.37	85.94	3091031	{}	\N	2020-10-12 15:46:36.277323
61186	NVS	2020-10-08	86.86	87.59	86.75	87.54	2412360	{}	\N	2020-10-12 15:46:36.2836
61187	NVS	2020-10-09	88.38	88.7	88.18	88.39	2833026	{}	\N	2020-10-12 15:46:36.291484
61188	NVS	2020-10-12	88.84	88.91	88.26	88.33	1002951	{}	\N	2020-10-12 15:46:36.308164
61189	NX	2020-09-28	18.19	18.78	18.19	18.66	168763	{}	\N	2020-10-12 15:46:37.303099
61190	NX	2020-09-29	18.6	19.07	18.5	18.97	455887	{}	\N	2020-10-12 15:46:37.31536
61191	NX	2020-09-30	19.05	19.2799	18.42	18.44	377170	{}	\N	2020-10-12 15:46:37.325828
61192	NX	2020-10-01	18.63	18.76	18.25	18.65	370857	{}	\N	2020-10-12 15:46:37.335589
61193	NX	2020-10-02	18.28	19.15	18.26	19.08	337767	{}	\N	2020-10-12 15:46:37.344769
61194	NX	2020-10-05	19.17	19.32	18.94	19.29	260393	{}	\N	2020-10-12 15:46:37.355054
61195	NX	2020-10-06	19.4	19.71	19.1	19.11	485559	{}	\N	2020-10-12 15:46:37.363207
61196	NX	2020-10-07	19.3	19.48	18.97	19.29	381741	{}	\N	2020-10-12 15:46:37.371347
61197	NX	2020-10-08	19.6	19.84	19.33	19.84	272740	{}	\N	2020-10-12 15:46:37.381143
61198	NX	2020-10-09	20	20.04	19.61	19.82	232217	{}	\N	2020-10-12 15:46:37.392459
61199	NX	2020-10-12	19.81	19.93	19.7	19.83	46912	{}	\N	2020-10-12 15:46:37.404992
61200	NXR	2020-09-28	16.45	16.51	16.34	16.38	14256	{}	\N	2020-10-12 15:46:39.116751
61201	NXR	2020-09-29	16.4	16.495	16.38	16.38	37368	{}	\N	2020-10-12 15:46:39.12348
61202	NXR	2020-09-30	16.49	16.49	16.29	16.4	36813	{}	\N	2020-10-12 15:46:39.129322
61203	NXR	2020-10-01	16.5	16.5	16.34	16.34	42356	{}	\N	2020-10-12 15:46:39.136122
61204	NXR	2020-10-02	16.38	16.4	16.33	16.38	17554	{}	\N	2020-10-12 15:46:39.143923
61205	NXR	2020-10-05	16.37	16.4	16.31	16.4	34056	{}	\N	2020-10-12 15:46:39.149465
61206	NXR	2020-10-06	16.4	16.4	16.362	16.4	19110	{}	\N	2020-10-12 15:46:39.15605
61207	NXR	2020-10-07	16.4	16.4	16.3	16.3425	21702	{}	\N	2020-10-12 15:46:39.162388
61208	NXR	2020-10-08	16.37	16.37	16.3	16.31	21810	{}	\N	2020-10-12 15:46:39.168431
61209	NXR	2020-10-09	16.4	16.4	16.3	16.3	10690	{}	\N	2020-10-12 15:46:39.175327
61210	NXR	2020-10-12	16.4	16.4	16.3431	16.35	4168	{}	\N	2020-10-12 15:46:39.182832
61211	NXTD	2020-09-28	0.3443	0.35	0.3342	0.3376	334021	{}	\N	2020-10-12 15:46:40.345156
61212	NXTD	2020-09-29	0.338	0.34	0.3343	0.34	138798	{}	\N	2020-10-12 15:46:40.360768
61213	NXTD	2020-09-30	0.338	0.355	0.3338	0.349799	515649	{}	\N	2020-10-12 15:46:40.371869
61214	NXTD	2020-10-01	0.3433	0.35	0.3159	0.3303	420870	{}	\N	2020-10-12 15:46:40.383553
61215	NXTD	2020-10-02	0.33	0.335	0.312	0.3192	308276	{}	\N	2020-10-12 15:46:40.393878
61216	NXTD	2020-10-05	0.32	0.3273	0.313	0.3231	400628	{}	\N	2020-10-12 15:46:40.404358
61217	NXTD	2020-10-06	0.3	0.319	0.29	0.3156	620973	{}	\N	2020-10-12 15:46:40.414779
61218	NXTD	2020-10-07	0.316	0.3189	0.3	0.3179	528631	{}	\N	2020-10-12 15:46:40.426931
61219	NXTD	2020-10-08	0.32	0.38	0.31	0.346	3528973	{}	\N	2020-10-12 15:46:40.43903
61220	NXTD	2020-10-09	0.34	0.3604	0.3251	0.326	771580	{}	\N	2020-10-12 15:46:40.449705
61221	NXTD	2020-10-12	0.3155	0.339	0.31	0.3352	330618	{}	\N	2020-10-12 15:46:40.460704
61222	NYCB	2020-09-28	8.37	8.605	8.3399	8.53	2220378	{}	\N	2020-10-12 15:46:42.568662
61223	NYCB	2020-09-29	8.46	8.54	8.27	8.36	4548552	{}	\N	2020-10-12 15:46:42.576189
61224	NYCB	2020-09-30	8.37	8.5	8.19	8.27	10140392	{}	\N	2020-10-12 15:46:42.58188
61225	NYCB	2020-10-01	8.18	8.32	8.13	8.22	8560975	{}	\N	2020-10-12 15:46:42.588019
61226	NYCB	2020-10-02	8.08	8.55	7.985	8.48	10110604	{}	\N	2020-10-12 15:46:42.596344
61227	NYCB	2020-10-05	8.59	8.76	8.56	8.71	5820627	{}	\N	2020-10-12 15:46:42.604136
61228	NYCB	2020-10-06	8.79	8.92	8.6301	8.66	6643602	{}	\N	2020-10-12 15:46:42.612965
61229	NYCB	2020-10-07	8.71	8.87	8.675	8.74	6415438	{}	\N	2020-10-12 15:46:42.621421
61230	NYCB	2020-10-08	8.75	8.865	8.66	8.7	6791838	{}	\N	2020-10-12 15:46:42.632587
61231	NYCB	2020-10-09	8.76	8.81	8.45	8.46	11728866	{}	\N	2020-10-12 15:46:42.646261
61232	NYCB	2020-10-12	8.45	8.465	8.33	8.45	3301781	{}	\N	2020-10-12 15:46:42.659626
61233	NYMX	2020-09-28	2.36	2.55	2.36	2.51	116505	{}	\N	2020-10-12 15:46:44.518615
61234	NYMX	2020-09-29	2.5	2.64	2.45	2.6	89694	{}	\N	2020-10-12 15:46:44.524375
61235	NYMX	2020-09-30	2.595	2.68	2.42	2.46	121572	{}	\N	2020-10-12 15:46:44.529706
61236	NYMX	2020-10-01	2.46	2.58	2.36	2.54	177350	{}	\N	2020-10-12 15:46:44.534554
61237	NYMX	2020-10-02	2.4	2.47	2.34	2.36	170056	{}	\N	2020-10-12 15:46:44.539219
61238	NYMX	2020-10-05	2.35	2.45	2.35	2.43	78953	{}	\N	2020-10-12 15:46:44.545354
61239	NYMX	2020-10-06	2.44	2.4698	2.36	2.36	51970	{}	\N	2020-10-12 15:46:44.550497
61240	NYMX	2020-10-07	2.36	2.46	2.351	2.39	70235	{}	\N	2020-10-12 15:46:44.557534
61241	NYMX	2020-10-08	2.35	2.35	2.32	2.32	145930	{}	\N	2020-10-12 15:46:44.56362
61242	NYMX	2020-10-09	2.35	2.36	2.17	2.2	162774	{}	\N	2020-10-12 15:46:44.570983
61243	NYMX	2020-10-12	2.23	2.3	2.15	2.1601	44114	{}	\N	2020-10-12 15:46:44.577517
61244	NUW	2020-09-28	15.79	16.31	15.784	16.1	31295	{}	\N	2020-10-12 15:46:45.495609
61245	NUW	2020-09-29	16.09	16.2999	15.88	16.05	37218	{}	\N	2020-10-12 15:46:45.502256
61246	NUW	2020-09-30	15.97	16.2	15.95	15.98	29674	{}	\N	2020-10-12 15:46:45.508658
61247	NUW	2020-10-01	16.18	16.18	16.03	16.11	37312	{}	\N	2020-10-12 15:46:45.518685
61248	NUW	2020-10-02	16.11	16.16	15.94	15.95	23292	{}	\N	2020-10-12 15:46:45.527373
61249	NUW	2020-10-05	16.077	16.15	15.9938	16.0554	29254	{}	\N	2020-10-12 15:46:45.535978
61250	NUW	2020-10-06	15.98	16.1	15.94	15.98	35732	{}	\N	2020-10-12 15:46:45.54181
61251	NUW	2020-10-07	16.01	16.15	15.94	16.14	25026	{}	\N	2020-10-12 15:46:45.548514
61252	NUW	2020-10-08	16.17	16.3499	16.1	16.1	32964	{}	\N	2020-10-12 15:46:45.554111
61253	NUW	2020-10-09	16.14	16.14	15.9	15.9	43056	{}	\N	2020-10-12 15:46:45.561188
61254	NUW	2020-10-12	16.07	16.07	15.974	15.974	3124	{}	\N	2020-10-12 15:46:45.567666
61255	NVIV	2020-09-28	1.4	1.4	1.32	1.35	30397	{}	\N	2020-10-12 15:46:47.090742
61256	NVIV	2020-09-29	1.37	1.39	1.35	1.385	40487	{}	\N	2020-10-12 15:46:47.096596
61257	NVIV	2020-09-30	1.38	1.44	1.3601	1.42	79027	{}	\N	2020-10-12 15:46:47.103228
61258	NVIV	2020-10-01	1.42	1.42	1.33	1.37	60054	{}	\N	2020-10-12 15:46:47.109043
61259	NVIV	2020-10-02	1.3	1.42	1.27	1.4	120787	{}	\N	2020-10-12 15:46:47.11449
61260	NVIV	2020-10-05	1.4	1.45	1.3701	1.43	33109	{}	\N	2020-10-12 15:46:47.119724
61261	NVIV	2020-10-06	1.4	1.48	1.4	1.46	30259	{}	\N	2020-10-12 15:46:47.124375
61262	NVIV	2020-10-07	1.39	1.45	1.3497	1.39	155125	{}	\N	2020-10-12 15:46:47.13086
61263	NVIV	2020-10-08	1.41	1.45	1.35	1.43	119843	{}	\N	2020-10-12 15:46:47.136698
61264	NVIV	2020-10-09	1.4	1.411	1.3549	1.38	78053	{}	\N	2020-10-12 15:46:47.142721
61265	NVIV	2020-10-12	1.4	1.4	1.35	1.38	22217	{}	\N	2020-10-12 15:46:47.148581
61266	NWE	2020-09-28	48.88	49.5	48.88	49.15	317481	{}	\N	2020-10-12 15:46:48.257656
61267	NWE	2020-09-29	49.31	49.44	48.07	48.41	509052	{}	\N	2020-10-12 15:46:48.264437
61268	NWE	2020-09-30	48.75	49.22	48.22	48.64	522006	{}	\N	2020-10-12 15:46:48.270917
61269	NWE	2020-10-01	48.85	49.07	48.36	49.01	198863	{}	\N	2020-10-12 15:46:48.27578
61270	NWE	2020-10-02	48.65	50.26	48.255	50.03	275822	{}	\N	2020-10-12 15:46:48.28119
61271	NWE	2020-10-05	50.39	50.95	49.99	50.87	278991	{}	\N	2020-10-12 15:46:48.287342
61272	NWE	2020-10-06	51.55	52.519	50.7252	51.38	244879	{}	\N	2020-10-12 15:46:48.292918
61273	NWE	2020-10-07	51.61	52.16	51.05	51.72	233844	{}	\N	2020-10-12 15:46:48.29882
61274	NWE	2020-10-08	51.85	52.32	51.31	52.1	229214	{}	\N	2020-10-12 15:46:48.305969
61275	NWE	2020-10-09	52.59	52.82	51.8701	52.16	191045	{}	\N	2020-10-12 15:46:48.316572
61276	NWE	2020-10-12	51.85	52.78	51.51	52.75	40245	{}	\N	2020-10-12 15:46:48.33726
61277	NXC	2020-09-28	15.98	16.2	15.94	15.9699	5970	{}	\N	2020-10-12 15:46:49.247147
61278	NXC	2020-09-29	16.06	16.11	16.06	16.0867	1786	{}	\N	2020-10-12 15:46:49.253198
61279	NXC	2020-09-30	16.01	16.01	15.96	16.01	3658	{}	\N	2020-10-12 15:46:49.258048
61280	NXC	2020-10-01	16.01	16.01	15.96	15.97	19062	{}	\N	2020-10-12 15:46:49.262927
61281	NXC	2020-10-02	15.9	16	15.9	16	22047	{}	\N	2020-10-12 15:46:49.270089
61282	NXC	2020-10-05	16	16	15.95	15.9593	16666	{}	\N	2020-10-12 15:46:49.276144
61283	NXC	2020-10-06	16	16	16	16	1048	{}	\N	2020-10-12 15:46:49.282026
61284	NXC	2020-10-07	15.99	16	15.97	15.97	10228	{}	\N	2020-10-12 15:46:49.287812
61285	NXC	2020-10-08	16	16	15.966	15.99	4182	{}	\N	2020-10-12 15:46:49.292149
61286	NXC	2020-10-09	16	16	15.98	15.99	12442	{}	\N	2020-10-12 15:46:49.296774
61287	NXC	2020-10-12	15.99	15.99	15.96	15.96	1344	{}	\N	2020-10-12 15:46:49.302732
61288	NXN	2020-09-28	13.06	13.11	12.81	13.08	14027	{}	\N	2020-10-12 15:46:50.648038
61289	NXN	2020-09-29	13.0499	13.1299	13.0499	13.07	3294	{}	\N	2020-10-12 15:46:50.654417
61290	NXN	2020-09-30	13.01	13.08	12.98	13.08	5110	{}	\N	2020-10-12 15:46:50.660979
61291	NXN	2020-10-01	13.23	13.24	12.98	13.24	5442	{}	\N	2020-10-12 15:46:50.667637
61292	NXN	2020-10-02	13.01	13.1512	13.01	13.1032	2610	{}	\N	2020-10-12 15:46:50.67513
61293	NXN	2020-10-05	13	13.1993	12.9914	13.1993	2386	{}	\N	2020-10-12 15:46:50.68134
61294	NXN	2020-10-06	13.04	13.2392	13.02	13.215	6970	{}	\N	2020-10-12 15:46:50.689707
61295	NXN	2020-10-07	13.14	13.14	13.03	13.1	5506	{}	\N	2020-10-12 15:46:50.696988
61296	NXN	2020-10-08	13.12	13.135	12.98	13.1	16322	{}	\N	2020-10-12 15:46:50.703003
61297	NXN	2020-10-09	13.05	13.0999	12.92	13.0999	7542	{}	\N	2020-10-12 15:46:50.70809
61298	NXN	2020-10-12	13.045	13.045	13.045	13.045	4000	{}	\N	2020-10-12 15:46:50.713029
61299	NXQ	2020-09-28	15.18	15.2	15.03	15.09	14003	{}	\N	2020-10-12 15:46:51.992321
61300	NXQ	2020-09-29	15.05	15.21	15	15.02	58970	{}	\N	2020-10-12 15:46:51.999094
61301	NXQ	2020-09-30	15.16	15.22	14.98	14.98	63177	{}	\N	2020-10-12 15:46:52.005714
61302	NXQ	2020-10-01	15.19	15.22	15.1	15.21	52448	{}	\N	2020-10-12 15:46:52.012633
61303	NXQ	2020-10-02	15.21	15.21	15.14	15.21	28480	{}	\N	2020-10-12 15:46:52.020352
61304	NXQ	2020-10-05	15.3	15.365	15.2101	15.22	84886	{}	\N	2020-10-12 15:46:52.02652
61305	NXQ	2020-10-06	15.33	15.36	15.2201	15.26	33151	{}	\N	2020-10-12 15:46:52.032213
61306	NXQ	2020-10-07	15.29	15.3	15.22	15.22	38750	{}	\N	2020-10-12 15:46:52.040433
61307	NXQ	2020-10-08	15.27	15.27	15.22	15.22	28428	{}	\N	2020-10-12 15:46:52.048702
61308	NXQ	2020-10-09	15.26	15.3899	15.15	15.15	60110	{}	\N	2020-10-12 15:46:52.057147
61309	NXQ	2020-10-12	15.12	15.3	15.12	15.16	8828	{}	\N	2020-10-12 15:46:52.06441
61310	NYF	2020-09-28	57.44	57.57	57.44	57.57	12782	{}	\N	2020-10-12 15:46:53.383791
61311	NYF	2020-09-29	57.56	57.56	57.44	57.54	11238	{}	\N	2020-10-12 15:46:53.390962
61312	NYF	2020-09-30	57.55	57.55	57.4453	57.4453	13374	{}	\N	2020-10-12 15:46:53.399622
61313	NYF	2020-10-01	57.33	57.36	57.33	57.36	6162	{}	\N	2020-10-12 15:46:53.409724
61314	NYF	2020-10-02	57.3604	57.3604	57.22	57.335	28237	{}	\N	2020-10-12 15:46:53.415591
61315	NYF	2020-10-05	57.26	57.33	57.26	57.28	20841	{}	\N	2020-10-12 15:46:53.421319
61316	NYF	2020-10-06	57.33	57.33	57.2	57.2	23754	{}	\N	2020-10-12 15:46:53.427067
61317	NYF	2020-10-07	57.21	57.24	57.06	57.06	20590	{}	\N	2020-10-12 15:46:53.433109
61318	NYF	2020-10-08	57.02	57.16	57.02	57.06	28915	{}	\N	2020-10-12 15:46:53.440775
61319	NYF	2020-10-09	57.12	57.139	57.06	57.0921	15470	{}	\N	2020-10-12 15:46:53.446723
61320	NYF	2020-10-12	57.05	57.0954	57.02	57.05	2493	{}	\N	2020-10-12 15:46:53.453535
61321	NYMT	2020-09-28	2.54	2.68	2.54	2.64	3719734	{}	\N	2020-10-12 15:46:54.483118
61322	NYMT	2020-09-29	2.63	2.63	2.51	2.59	2357323	{}	\N	2020-10-12 15:46:54.493434
61323	NYMT	2020-09-30	2.6	2.65	2.53	2.55	3627948	{}	\N	2020-10-12 15:46:54.501787
61324	NYMT	2020-10-01	2.59	2.59	2.51	2.57	2009415	{}	\N	2020-10-12 15:46:54.51301
61325	NYMT	2020-10-02	2.52	2.655	2.5	2.64	2316101	{}	\N	2020-10-12 15:46:54.52618
61326	NYMT	2020-10-05	2.65	2.6601	2.61	2.62	1934919	{}	\N	2020-10-12 15:46:54.534364
61327	NYMT	2020-10-06	2.64	2.68	2.59	2.59	3617750	{}	\N	2020-10-12 15:46:54.540809
61328	NYMT	2020-10-07	2.61	2.63	2.56	2.61	2038999	{}	\N	2020-10-12 15:46:54.546178
61329	NYMT	2020-10-08	2.62	2.72	2.61	2.71	4277956	{}	\N	2020-10-12 15:46:54.55295
61330	NYMT	2020-10-09	2.71	2.74	2.67	2.695	2227858	{}	\N	2020-10-12 15:46:54.56134
61331	NYMT	2020-10-12	2.68	2.69	2.6517	2.6619	504447	{}	\N	2020-10-12 15:46:54.567476
61332	NYT	2020-09-28	42.02	42.32	41.28	42.24	1485774	{}	\N	2020-10-12 15:46:55.770157
61333	NYT	2020-09-29	42.38	42.63	41.805	41.98	1511416	{}	\N	2020-10-12 15:46:55.778196
61334	NYT	2020-09-30	42.09	43.03	41.86	42.79	4940368	{}	\N	2020-10-12 15:46:55.784213
61335	NYT	2020-10-01	43.06	43.95	42.34	43.87	2644390	{}	\N	2020-10-12 15:46:55.791071
61336	NYT	2020-10-02	43.21	44.15	43.05	43.46	2342450	{}	\N	2020-10-12 15:46:55.799582
61337	NYT	2020-10-05	43.74	45.94	43.705	45.92	3549579	{}	\N	2020-10-12 15:46:55.807335
61338	NYT	2020-10-06	45.69	46.46	45.36	45.51	3219161	{}	\N	2020-10-12 15:46:55.812986
61339	NYT	2020-10-07	45.68	46.25	45.28	45.53	2789913	{}	\N	2020-10-12 15:46:55.820049
61340	NYT	2020-10-08	45.69	45.955	45.39	45.56	2205721	{}	\N	2020-10-12 15:46:55.826543
61341	NYT	2020-10-09	45.71	45.9	43.6356	43.83	4065623	{}	\N	2020-10-12 15:46:55.832543
61342	NYT	2020-10-12	43.91	44.31	42.85	42.93	695688	{}	\N	2020-10-12 15:46:55.838965
61343	NYV	2020-09-28	14.87	14.87	14.84	14.84	1802	{}	\N	2020-10-12 15:46:57.240244
61344	NYV	2020-09-29	14.83	14.83	14.7983	14.8273	10332	{}	\N	2020-10-12 15:46:57.247193
61345	NYV	2020-09-30	14.7894	14.8	14.78	14.8	1028	{}	\N	2020-10-12 15:46:57.253342
61346	NYV	2020-10-01	14.87	14.87	14.76	14.81	1964	{}	\N	2020-10-12 15:46:57.25994
61347	NYV	2020-10-02	14.7926	14.81	14.78	14.81	6206	{}	\N	2020-10-12 15:46:57.265546
61348	NYV	2020-10-05	14.69	14.79	14.69	14.78	2988	{}	\N	2020-10-12 15:46:57.273968
61349	NYV	2020-10-06	14.67	14.69	14.67	14.69	404	{}	\N	2020-10-12 15:46:57.283609
61350	NYV	2020-10-07	14.7576	14.77	14.75	14.77	8178	{}	\N	2020-10-12 15:46:57.291065
61351	NYV	2020-10-08	14.81	14.81	14.7	14.71	7288	{}	\N	2020-10-12 15:46:57.297845
61352	NYV	2020-10-09	14.76	14.82	14.76	14.8	6946	{}	\N	2020-10-12 15:46:57.304404
61353	NYV	2020-10-12	14.6676	14.6676	14.6676	14.6676	1000	{}	\N	2020-10-12 15:46:57.309845
61354	OBCI	2020-09-28	14.86	14.86	14.099	14.48	11333	{}	\N	2020-10-12 15:46:59.130982
61355	OBCI	2020-09-29	14.32	14.5	13.97	14.3	13940	{}	\N	2020-10-12 15:46:59.139837
61356	OBCI	2020-09-30	14.0559	14.4	14.0559	14.33	18580	{}	\N	2020-10-12 15:46:59.146099
61357	OBCI	2020-10-01	14.3	15.49	14.26	15.49	32505	{}	\N	2020-10-12 15:46:59.153732
61358	OBCI	2020-10-02	15.07	15.98	14.7	15.72	52230	{}	\N	2020-10-12 15:46:59.159716
61359	OBCI	2020-10-05	15.7	15.96	15.505	15.66	16458	{}	\N	2020-10-12 15:46:59.166242
61360	OBCI	2020-10-06	15.7	16.48	15.45	15.72	23512	{}	\N	2020-10-12 15:46:59.174593
61361	OBCI	2020-10-07	15.949	16.16	15.5701	15.85	8763	{}	\N	2020-10-12 15:46:59.181143
61362	OBCI	2020-10-08	16.08	16.1	14.8	14.9	19977	{}	\N	2020-10-12 15:46:59.186998
61363	OBCI	2020-10-09	15.06	15.15	14.76	15.15	9051	{}	\N	2020-10-12 15:46:59.194574
61364	OBCI	2020-10-12	15.23	15.31	15.1	15.28	2219	{}	\N	2020-10-12 15:46:59.200713
61365	NWHM	2020-09-28	4.76	5.17	4.64	5.12	155442	{}	\N	2020-10-12 15:47:00.788365
61366	NWHM	2020-09-29	5.15	5.18	4.84	5.11	195824	{}	\N	2020-10-12 15:47:00.795442
61367	NWHM	2020-09-30	5.1	5.5	5.01	5.44	255842	{}	\N	2020-10-12 15:47:00.802033
61368	NWHM	2020-10-01	5.52	5.57	5.17	5.46	164484	{}	\N	2020-10-12 15:47:00.809266
61369	NWHM	2020-10-02	5.3	5.57	5.3	5.48	115792	{}	\N	2020-10-12 15:47:00.817376
61370	NWHM	2020-10-05	5.5	5.7	5.47	5.57	472867	{}	\N	2020-10-12 15:47:00.822922
61371	NWHM	2020-10-06	5.58	5.69	5.5	5.58	197095	{}	\N	2020-10-12 15:47:00.829141
61372	NWHM	2020-10-07	5.64	5.75	5.54	5.61	118417	{}	\N	2020-10-12 15:47:00.838324
61373	NWHM	2020-10-08	5.62	5.71	5.56	5.68	55586	{}	\N	2020-10-12 15:47:00.84914
61374	NWHM	2020-10-09	5.67	5.78	5.56	5.69	78426	{}	\N	2020-10-12 15:47:00.857492
61375	NWHM	2020-10-12	5.72	5.76	5.54	5.66	50848	{}	\N	2020-10-12 15:47:00.864769
61376	NXE	2020-09-28	1.72	1.78	1.7	1.76	325928	{}	\N	2020-10-12 15:47:01.869742
61377	NXE	2020-09-29	1.78	1.78	1.71	1.73	127776	{}	\N	2020-10-12 15:47:01.877863
61378	NXE	2020-09-30	1.73	1.77	1.7	1.74	489667	{}	\N	2020-10-12 15:47:01.88273
61379	NXE	2020-10-01	1.74	1.7559	1.6936	1.71	448456	{}	\N	2020-10-12 15:47:01.854539
61380	NXE	2020-10-02	1.65	1.71	1.64	1.71	445651	{}	\N	2020-10-12 15:47:01.859299
61381	NXE	2020-10-05	1.73	1.74	1.69	1.74	287265	{}	\N	2020-10-12 15:47:01.865351
61382	NXE	2020-10-06	1.77	1.78	1.675	1.69	522387	{}	\N	2020-10-12 15:47:01.873133
61383	NXE	2020-10-07	1.7	1.775	1.69	1.76	267910	{}	\N	2020-10-12 15:47:01.879687
61384	NXE	2020-10-08	1.775	1.8	1.72	1.74	247436	{}	\N	2020-10-12 15:47:01.884978
61385	NXE	2020-10-09	1.74	1.78	1.71	1.73	249626	{}	\N	2020-10-12 15:47:01.889136
61386	NXE	2020-10-12	1.75	1.76	1.74	1.745	72121	{}	\N	2020-10-12 15:47:01.904693
61387	NXRT	2020-09-28	43.09	44.64	43.09	44.55	109883	{}	\N	2020-10-12 15:47:02.902674
61388	NXRT	2020-09-29	44.42	44.42	42.67	43.92	179172	{}	\N	2020-10-12 15:47:02.908715
61389	NXRT	2020-09-30	44.12	45.06	43.79	44.35	330528	{}	\N	2020-10-12 15:47:02.915014
61390	NXRT	2020-10-01	44.76	45.38	44.06	45	355047	{}	\N	2020-10-12 15:47:02.921692
61391	NXRT	2020-10-02	44.26	45.48	44	45.24	227986	{}	\N	2020-10-12 15:47:02.928209
61392	NXRT	2020-10-05	45.44	45.455	44.34	44.9	203081	{}	\N	2020-10-12 15:47:02.934211
61393	NXRT	2020-10-06	45.18	45.4	44.02	44.25	242983	{}	\N	2020-10-12 15:47:02.939307
61394	NXRT	2020-10-07	44.55	44.91	43.55	44.2	367075	{}	\N	2020-10-12 15:47:02.945176
61395	NXRT	2020-10-08	44.65	45.1203	44.48	44.86	205095	{}	\N	2020-10-12 15:47:02.95167
61396	NXRT	2020-10-09	45.27	45.52	44.27	44.64	186045	{}	\N	2020-10-12 15:47:02.956903
61397	NXRT	2020-10-12	44.49	44.76	43.88	44.57	27985	{}	\N	2020-10-12 15:47:02.96258
61398	NYMTN	2020-09-28	20.8	20.93	20.59	20.72	15520	{}	\N	2020-10-12 15:47:04.951132
61399	NYMTN	2020-09-29	20.8941	21.18	20.7	20.86	15493	{}	\N	2020-10-12 15:47:04.957292
61400	NYMTN	2020-09-30	20.6	20.75	20.3401	20.7	16293	{}	\N	2020-10-12 15:47:04.962625
61401	NYMTN	2020-10-01	20.55	20.69	20.26	20.42	26140	{}	\N	2020-10-12 15:47:04.968113
61402	NYMTN	2020-10-02	20.445	20.445	20	20.19	32280	{}	\N	2020-10-12 15:47:04.974806
61403	NYMTN	2020-10-05	20.26	20.26	20.08	20.14	5428	{}	\N	2020-10-12 15:47:04.980961
61404	NYMTN	2020-10-06	20.26	20.42	20.01	20.15	17374	{}	\N	2020-10-12 15:47:04.986418
61405	NYMTN	2020-10-07	20.37	20.405	20.2	20.21	16506	{}	\N	2020-10-12 15:47:04.991949
61406	NYMTN	2020-10-08	20.21	20.49	20.21	20.49	11494	{}	\N	2020-10-12 15:47:04.998707
61407	NYMTN	2020-10-09	20.44	20.4667	20.16	20.38	6535	{}	\N	2020-10-12 15:47:05.0042
61408	NYMTN	2020-10-12	20.38	20.38	20.18	20.2	3015	{}	\N	2020-10-12 15:47:05.008946
61409	O	2020-09-28	61.47	62.02	60.86	61.67	2279379	{}	\N	2020-10-12 15:47:05.988803
61410	O	2020-09-29	61.67	61.74	60.36	61.1	3415306	{}	\N	2020-10-12 15:47:05.996413
61411	O	2020-09-30	61.35	61.95	60.02	60.75	5764439	{}	\N	2020-10-12 15:47:06.004343
61412	O	2020-10-01	60.84	62.32	60.5	62.28	4171366	{}	\N	2020-10-12 15:47:06.014336
61413	O	2020-10-02	61.14	64.1	60.87	63.86	4930722	{}	\N	2020-10-12 15:47:06.02326
61414	O	2020-10-05	63.83	64.33	62.22	63.25	3615280	{}	\N	2020-10-12 15:47:06.031452
61415	O	2020-10-06	63.62	63.9	62.64	62.91	3780041	{}	\N	2020-10-12 15:47:06.038755
61416	O	2020-10-07	63.25	63.3899	62.4582	62.71	3210253	{}	\N	2020-10-12 15:47:06.044664
61417	O	2020-10-08	63.07	63.79	62.95	63.75	3665010	{}	\N	2020-10-12 15:47:06.050167
61418	O	2020-10-09	64.6	64.61	62.96	63.19	3598622	{}	\N	2020-10-12 15:47:06.055402
61419	O	2020-10-12	63.03	63.37	62.58	63.34	820298	{}	\N	2020-10-12 15:47:06.060042
61420	OBOR	2020-09-28	21.37	21.4175	21.37	21.4175	6568	{}	\N	2020-10-12 15:47:10.153053
61421	OBOR	2020-09-29	21.33	21.4798	21.33	21.4798	3544	{}	\N	2020-10-12 15:47:10.160601
61422	OBOR	2020-09-30	21.65	21.6561	21.65	21.6561	351	{}	\N	2020-10-12 15:47:10.167061
61423	OBOR	2020-10-01	21.8931	21.8931	21.8931	21.8931	3	{}	\N	2020-10-12 15:47:10.174654
61424	OBOR	2020-10-02	21.72	21.73	21.6223	21.6223	651	{}	\N	2020-10-12 15:47:10.180517
61425	OBOR	2020-10-05	21.83	21.9288	21.83	21.9288	600	{}	\N	2020-10-12 15:47:10.186637
61426	OBOR	2020-10-06	22.06	22.06	21.9386	21.9386	519	{}	\N	2020-10-12 15:47:10.195207
61427	OBOR	2020-10-07	21.7	22.0547	21.7	22.0547	1912	{}	\N	2020-10-12 15:47:10.2022
61428	OBOR	2020-10-08	22.2709	22.2709	22.2709	22.2709	1	{}	\N	2020-10-12 15:47:10.209962
61429	OBOR	2020-10-09	22.44	22.4603	22.44	22.4603	275	{}	\N	2020-10-12 15:47:10.216518
61430	OBOR	2020-10-12	22.6	22.6	22.6	22.6	251	{}	\N	2020-10-12 15:47:10.222619
61431	MYO	2020-09-28	4.56	4.65	4.46	4.52	118509	{}	\N	2020-10-12 15:47:12.868879
61432	MYO	2020-09-29	4.53	4.6806	4.45	4.515	134119	{}	\N	2020-10-12 15:47:12.875791
61433	MYO	2020-09-30	4.45	4.58	4.41	4.48	149597	{}	\N	2020-10-12 15:47:12.882017
61434	MYO	2020-10-01	4.42	4.6	4.383	4.54	71497	{}	\N	2020-10-12 15:47:12.88781
61435	MYO	2020-10-02	4.41	4.5639	4.4001	4.475	94663	{}	\N	2020-10-12 15:47:12.89439
61436	MYO	2020-10-05	4.5	4.6151	4.45	4.51	162427	{}	\N	2020-10-12 15:47:12.899683
61437	MYO	2020-10-06	4.5	4.649	4.35	4.48	169874	{}	\N	2020-10-12 15:47:12.905455
61438	MYO	2020-10-07	4.52	4.8	4.48	4.64	178062	{}	\N	2020-10-12 15:47:12.911482
61439	MYO	2020-10-08	4.64	4.78	4.64	4.73	114146	{}	\N	2020-10-12 15:47:12.918078
61440	MYO	2020-10-09	4.75	4.8203	4.635	4.65	47560	{}	\N	2020-10-12 15:47:12.92381
61441	MYO	2020-10-12	4.61	4.7	4.6	4.62	23308	{}	\N	2020-10-12 15:47:12.929747
61442	NAN	2020-09-28	13.68	13.69	13.62	13.65	23055	{}	\N	2020-10-12 15:47:13.804589
61443	NAN	2020-09-29	13.72	13.75	13.64	13.66	76980	{}	\N	2020-10-12 15:47:13.809883
61444	NAN	2020-09-30	13.72	13.72	13.64	13.67	56066	{}	\N	2020-10-12 15:47:13.815263
61445	NAN	2020-10-01	13.75	13.75	13.64	13.65	93380	{}	\N	2020-10-12 15:47:13.820614
61446	NAN	2020-10-02	13.68	13.7073	13.43	13.5	148474	{}	\N	2020-10-12 15:47:13.827238
61447	NAN	2020-10-05	13.42	13.48	13.42	13.45	127400	{}	\N	2020-10-12 15:47:13.833792
61448	NAN	2020-10-06	13.5	13.5308	13.45	13.45	57431	{}	\N	2020-10-12 15:47:13.84261
61449	NAN	2020-10-07	13.45	13.55	13.44	13.51	110836	{}	\N	2020-10-12 15:47:13.851481
61450	NAN	2020-10-08	13.58	13.63	13.54	13.63	116606	{}	\N	2020-10-12 15:47:13.860408
61451	NAN	2020-10-09	13.66	13.68	13.58	13.64	35518	{}	\N	2020-10-12 15:47:13.867607
61452	NAN	2020-10-12	13.73	13.73	13.6	13.65	24392	{}	\N	2020-10-12 15:47:13.874185
61453	NAV	2020-09-28	43.4	43.49	42.96	43	520200	{}	\N	2020-10-12 15:47:14.848635
61454	NAV	2020-09-29	43.03	43.23	42.745	42.95	2593514	{}	\N	2020-10-12 15:47:14.854912
61455	NAV	2020-09-30	43	43.96	43	43.54	1637160	{}	\N	2020-10-12 15:47:14.861367
61456	NAV	2020-10-01	43.89	44.13	43.21	43.89	2201624	{}	\N	2020-10-12 15:47:14.868615
61457	NAV	2020-10-02	43.3	44.27	43.3	44	1158552	{}	\N	2020-10-12 15:47:14.876985
61458	NAV	2020-10-05	44.84	45.25	43.97	43.99	4505083	{}	\N	2020-10-12 15:47:14.883274
61459	NAV	2020-10-06	44.41	44.75	43.81	43.86	1819663	{}	\N	2020-10-12 15:47:14.889545
61460	NAV	2020-10-07	44.3	44.82	44.17	44.7	1670882	{}	\N	2020-10-12 15:47:14.895024
61461	NAV	2020-10-08	44.64	44.85	44.32	44.76	698785	{}	\N	2020-10-12 15:47:14.90202
61462	NAV	2020-10-09	44.97	45.1	44.7	44.83	789445	{}	\N	2020-10-12 15:47:14.907882
61463	NAV	2020-10-12	44.78	44.88	44.41	44.42	309663	{}	\N	2020-10-12 15:47:14.914294
61464	NBR	2020-09-28	25	27.71	25	26.55	476649	{}	\N	2020-10-12 15:47:17.242156
61465	NBR	2020-09-29	26.62	26.62	23.945	25.14	926232	{}	\N	2020-10-12 15:47:17.249832
61466	NBR	2020-09-30	25.15	26.44	24.1676	24.44	619655	{}	\N	2020-10-12 15:47:17.254917
61467	NBR	2020-10-01	24.31	24.87	23.13	23.34	959541	{}	\N	2020-10-12 15:47:17.259749
61468	NBR	2020-10-02	21.81	24.95	21.66	24.08	732142	{}	\N	2020-10-12 15:47:17.265591
61469	NBR	2020-10-05	24.83	25.7	24.14	24.94	496653	{}	\N	2020-10-12 15:47:17.270414
61470	NBR	2020-10-06	25.44	26.6993	24.42	24.97	844945	{}	\N	2020-10-12 15:47:17.275303
61471	NBR	2020-10-07	24.72	25.59	24.11	25.31	544219	{}	\N	2020-10-12 15:47:17.280012
61472	NBR	2020-10-08	25.75	28.7458	25.75	28.3	1504441	{}	\N	2020-10-12 15:47:17.285697
61473	NBR	2020-10-09	28.97	29.3	26.9	28.29	985039	{}	\N	2020-10-12 15:47:17.290654
61474	NBR	2020-10-12	28.27	28.29	27.08	27.77	200214	{}	\N	2020-10-12 15:47:17.295886
61475	NCNA	2020-09-28	5.26	5.387	5.16	5.28	136754	{}	\N	2020-10-12 15:47:18.963872
61476	NCNA	2020-09-29	5.33	5.36	4.96	5.26	256827	{}	\N	2020-10-12 15:47:18.969589
61477	NCNA	2020-09-30	5.2	5.26	5.13	5.17	124595	{}	\N	2020-10-12 15:47:18.974339
61478	NCNA	2020-10-01	5.15	5.3	5.1301	5.15	197522	{}	\N	2020-10-12 15:47:18.979479
61479	NCNA	2020-10-02	5.05	5.15	4.91	5.03	180870	{}	\N	2020-10-12 15:47:18.985038
61480	NCNA	2020-10-05	5.14	5.22	5	5.15	176886	{}	\N	2020-10-12 15:47:18.990601
61481	NCNA	2020-10-06	5.15	5.28	5.08	5.15	240080	{}	\N	2020-10-12 15:47:18.99685
61482	NCNA	2020-10-07	5.13	5.29	5.04	5.17	210496	{}	\N	2020-10-12 15:47:19.007405
61483	NCNA	2020-10-08	5.19	5.25	5.112	5.17	52588	{}	\N	2020-10-12 15:47:19.017444
61484	NCNA	2020-10-09	5.15	5.24	5.03	5.09	116385	{}	\N	2020-10-12 15:47:19.024083
61485	NCNA	2020-10-12	5.04	5.16	4.94	5.0899	158319	{}	\N	2020-10-12 15:47:19.029678
61486	NCZ	2020-09-28	4.07	4.13	4.04	4.11	403121	{}	\N	2020-10-12 15:47:20.204359
61487	NCZ	2020-09-29	4.11	4.11	4.07	4.1	2734645	{}	\N	2020-10-12 15:47:20.210232
61488	NCZ	2020-09-30	4.11	4.13	4.09	4.11	549652	{}	\N	2020-10-12 15:47:20.215898
61489	NCZ	2020-10-01	4.14	4.17	4.12	4.16	868468	{}	\N	2020-10-12 15:47:20.222346
61490	NCZ	2020-10-02	4.12	4.15	4.115	4.14	656735	{}	\N	2020-10-12 15:47:20.22842
61491	NCZ	2020-10-05	4.16	4.2	4.15	4.18	467956	{}	\N	2020-10-12 15:47:20.233565
61492	NCZ	2020-10-06	4.19	4.2599	4.19	4.2	703284	{}	\N	2020-10-12 15:47:20.239169
61493	NCZ	2020-10-07	4.21	4.32	4.19	4.32	3428778	{}	\N	2020-10-12 15:47:20.244334
61494	NCZ	2020-10-08	4.35	4.37	4.27	4.29	7654200	{}	\N	2020-10-12 15:47:20.250188
61495	NCZ	2020-10-09	4.28	4.3	4.23	4.26	5279032	{}	\N	2020-10-12 15:47:20.256423
61496	NCZ	2020-10-12	4.27	4.3	4.26	4.29	320990	{}	\N	2020-10-12 15:47:20.261598
61497	NEN	2020-09-28	49.749	49.749	49.749	49.749	225	{}	\N	2020-10-12 15:47:21.807783
61498	NEN	2020-09-29	49.125	49.125	49.125	49.125	664	{}	\N	2020-10-12 15:47:21.814376
61499	NEN	2020-09-30	49	49	48.4899	49	1979	{}	\N	2020-10-12 15:47:21.819407
61500	NEN	2020-10-01	49.01	49.595	48.65	48.65	824	{}	\N	2020-10-12 15:47:21.825968
61501	NEN	2020-10-05	49.2	49.2	49.09	49.1	1316	{}	\N	2020-10-12 15:47:21.835997
61502	NEN	2020-10-06	49.1499	49.1499	49.1499	49.1499	171	{}	\N	2020-10-12 15:47:21.846644
61503	NEN	2020-10-08	48.8058	48.8058	48.8058	48.8058	1907	{}	\N	2020-10-12 15:47:21.85463
61504	NEN	2020-10-09	48.56	48.56	48.56	48.56	377	{}	\N	2020-10-12 15:47:21.861036
61505	NEN	2020-10-12	48.9479	48.9479	48.9479	48.9479	403	{}	\N	2020-10-12 15:47:21.865741
61506	NESRW	2020-09-28	0.701	0.743	0.701	0.73	5600	{}	\N	2020-10-12 15:47:23.444904
61507	NESRW	2020-09-29	0.6954	0.7168	0.69	0.7	20020	{}	\N	2020-10-12 15:47:23.451553
61508	NESRW	2020-09-30	0.694	0.7068	0.6895	0.7	29225	{}	\N	2020-10-12 15:47:23.459829
61509	NESRW	2020-10-01	0.803496	0.803496	0.803496	0.803496	300	{}	\N	2020-10-12 15:47:23.467883
61510	NESRW	2020-10-02	0.652	0.89005	0.59	0.89	101653	{}	\N	2020-10-12 15:47:23.474574
61511	NESRW	2020-10-05	0.7392	0.88	0.7392	0.88	1641	{}	\N	2020-10-12 15:47:23.479971
61512	NESRW	2020-10-07	0.8	0.9	0.8	0.9	11980	{}	\N	2020-10-12 15:47:23.484787
61513	NESRW	2020-10-08	0.95	0.95	0.95	0.95	8960	{}	\N	2020-10-12 15:47:23.49
61514	NEXA	2020-09-28	5.81	5.96	5.31	5.4	23913	{}	\N	2020-10-12 15:47:24.611277
61515	NEXA	2020-09-29	5.36	5.37	5.19	5.26	78154	{}	\N	2020-10-12 15:47:24.620999
61516	NEXA	2020-09-30	5.18	5.34	5.13	5.29	144798	{}	\N	2020-10-12 15:47:24.631149
61517	NEXA	2020-10-01	5.27	5.9784	5.22	5.82	325587	{}	\N	2020-10-12 15:47:24.638424
61518	NEXA	2020-10-02	5.55	6	5.55	5.94	245818	{}	\N	2020-10-12 15:47:24.645222
61519	NEXA	2020-10-05	6.11	6.53	6.09	6.49	70296	{}	\N	2020-10-12 15:47:24.650412
61520	NEXA	2020-10-06	6.51	6.78	5.82	5.91	87352	{}	\N	2020-10-12 15:47:24.655587
61521	NEXA	2020-10-07	5.81	5.81	5.5	5.59	146456	{}	\N	2020-10-12 15:47:24.661374
61522	NEXA	2020-10-08	5.6211	5.72	5.5	5.55	182492	{}	\N	2020-10-12 15:47:24.666433
61523	NEXA	2020-10-09	5.52	5.96	5.51	5.94	80852	{}	\N	2020-10-12 15:47:24.671144
61524	NEXA	2020-10-12	5.9	6.18	5.9	6.14	17612	{}	\N	2020-10-12 15:47:24.678376
61525	NFJ	2020-09-28	11.75	11.86	11.74	11.77	284717	{}	\N	2020-10-12 15:47:26.125984
61526	NFJ	2020-09-29	11.77	11.78	11.6618	11.72	442778	{}	\N	2020-10-12 15:47:26.132938
61527	NFJ	2020-09-30	11.69	11.89	11.69	11.8	1059901	{}	\N	2020-10-12 15:47:26.138366
61528	NFJ	2020-10-01	11.8	11.91	11.79	11.84	490082	{}	\N	2020-10-12 15:47:26.145036
61529	NFJ	2020-10-02	11.52	11.849	11.52	11.8	750080	{}	\N	2020-10-12 15:47:26.150856
61530	NFJ	2020-10-05	11.81	11.98	11.81	11.97	426238	{}	\N	2020-10-12 15:47:26.155756
61531	NFJ	2020-10-06	12	12.09	11.88	11.89	433210	{}	\N	2020-10-12 15:47:26.162262
61532	NFJ	2020-10-07	12.02	12.1351	11.9961	12.1	478780	{}	\N	2020-10-12 15:47:26.167385
61533	NFJ	2020-10-08	12.16	12.23	12.14	12.22	307375	{}	\N	2020-10-12 15:47:26.173636
61534	NFJ	2020-10-09	12.3	12.34	12.254	12.31	567093	{}	\N	2020-10-12 15:47:26.180629
61535	NFJ	2020-10-12	12.4	12.4	12.34	12.39	129811	{}	\N	2020-10-12 15:47:26.18586
61536	NGG	2020-09-28	55.04	55.2	54.56	54.56	322371	{}	\N	2020-10-12 15:47:27.797205
61537	NGG	2020-09-29	56.62	57.38	56.54	56.89	958026	{}	\N	2020-10-12 15:47:27.803344
61538	NGG	2020-09-30	57.8	57.94	57.34	57.79	1278365	{}	\N	2020-10-12 15:47:27.809162
61539	NGG	2020-10-01	58.22	58.93	57.69	58.28	1794699	{}	\N	2020-10-12 15:47:27.815434
61540	NGG	2020-10-02	59.13	59.96	59.11	59.92	1526678	{}	\N	2020-10-12 15:47:27.820759
61541	NGG	2020-10-05	59.33	60.12	59.11	60.04	693340	{}	\N	2020-10-12 15:47:27.826029
61542	NGG	2020-10-06	59.67	60.06	59.33	59.59	737076	{}	\N	2020-10-12 15:47:27.832038
61543	NGG	2020-10-07	59.78	60.735	59.6629	60.62	814324	{}	\N	2020-10-12 15:47:27.837352
61544	NGG	2020-10-08	60.69	61.22	60.62	61.02	571781	{}	\N	2020-10-12 15:47:27.842573
61545	NGG	2020-10-09	60.59	60.91	60.25	60.64	963238	{}	\N	2020-10-12 15:47:27.849558
61546	NGG	2020-10-12	61.07	61.67	60.96	61.64	145959	{}	\N	2020-10-12 15:47:27.855516
61547	NHC	2020-09-28	62.74	63.63	62.51	63.15	29184	{}	\N	2020-10-12 15:47:29.126088
61548	NHC	2020-09-29	62.56	62.56	60.7	61.8	29744	{}	\N	2020-10-12 15:47:29.134177
61549	NHC	2020-09-30	62.02	63.56	61.63	62.31	52718	{}	\N	2020-10-12 15:47:29.139747
61550	NHC	2020-10-01	60.37	62.35	60.245	61.24	43784	{}	\N	2020-10-12 15:47:29.146121
61551	NHC	2020-10-02	60.71	62.74	59.83	62.08	17332	{}	\N	2020-10-12 15:47:29.151869
61552	NHC	2020-10-05	62.54	64.03	62.255	64.02	23399	{}	\N	2020-10-12 15:47:29.157001
61553	NHC	2020-10-06	64.24	65.03	62.92	63.52	27213	{}	\N	2020-10-12 15:47:29.162776
61554	NHC	2020-10-07	64.2	64.9	63.57	64.36	40828	{}	\N	2020-10-12 15:47:29.167962
61555	NHC	2020-10-08	65.15	66.16	64.515	65.46	32244	{}	\N	2020-10-12 15:47:29.173382
61556	NHC	2020-10-09	66.2	66.59	64.91	65.37	20632	{}	\N	2020-10-12 15:47:29.181588
61557	NHC	2020-10-12	65.5	66.17	65.1	65.785	1629	{}	\N	2020-10-12 15:47:29.188105
61558	NICK	2020-09-28	7.685	7.685	7.38	7.52	4914	{}	\N	2020-10-12 15:47:31.074293
61559	NICK	2020-09-30	7.25	7.39	7.25	7.27	1605	{}	\N	2020-10-12 15:47:31.080348
61560	NICK	2020-10-01	7.36	7.37	7.35	7.35	1024	{}	\N	2020-10-12 15:47:31.086243
61561	NICK	2020-10-02	7.25	7.25	7.25	7.25	260	{}	\N	2020-10-12 15:47:31.091142
61562	NICK	2020-10-05	7.26	7.26	6.94	7.1	4178	{}	\N	2020-10-12 15:47:31.096562
61563	NICK	2020-10-06	7.1	7.1	7.1	7.1	225	{}	\N	2020-10-12 15:47:31.104354
61564	NICK	2020-10-08	7.25	7.25	7.25	7.25	425	{}	\N	2020-10-12 15:47:31.10986
61565	NICK	2020-10-09	7.25	7.25	7.25	7.25	223	{}	\N	2020-10-12 15:47:31.115675
61566	NKTR	2020-09-28	17.65	17.7	16.7	16.84	1551633	{}	\N	2020-10-12 15:47:33.055
61567	NKTR	2020-09-29	16.82	17.2648	16.595	16.89	1562863	{}	\N	2020-10-12 15:47:33.061244
61568	NKTR	2020-09-30	16.89	17	16.41	16.59	1404644	{}	\N	2020-10-12 15:47:33.069191
61569	NKTR	2020-10-01	16.56	16.75	16.46	16.7	1193659	{}	\N	2020-10-12 15:47:33.07523
61570	NKTR	2020-10-02	16.44	16.69	16.22	16.28	1021179	{}	\N	2020-10-12 15:47:33.080263
61571	NKTR	2020-10-05	16.53	17.3	16.5	17.28	1062483	{}	\N	2020-10-12 15:47:33.086015
61572	NKTR	2020-10-06	17.36	17.63	17.13	17.19	826751	{}	\N	2020-10-12 15:47:33.091537
61573	NKTR	2020-10-07	17.49	17.97	17.1	17.82	609258	{}	\N	2020-10-12 15:47:33.097749
61574	NKTR	2020-10-08	17.92	18.23	17.9	18.13	750137	{}	\N	2020-10-12 15:47:33.102918
61575	NKTR	2020-10-09	18.31	18.31	17.83	17.87	442856	{}	\N	2020-10-12 15:47:33.109751
61576	NKTR	2020-10-12	17.91	17.94	17.41	17.475	193627	{}	\N	2020-10-12 15:47:33.117772
61577	OCUL	2020-09-28	7.9638	8.0092	7.59	7.9	694769	{}	\N	2020-10-12 15:47:34.125739
61578	OCUL	2020-09-29	7.96	8.06	7.615	7.81	855230	{}	\N	2020-10-12 15:47:34.131111
61579	OCUL	2020-09-30	7.8	8.01	7.5	7.61	1389620	{}	\N	2020-10-12 15:47:34.136668
61580	OCUL	2020-10-01	7.73	8.05	7.7	7.91	611258	{}	\N	2020-10-12 15:47:34.142366
61581	OCUL	2020-10-02	7.67	8.18	7.67	8.04	362913	{}	\N	2020-10-12 15:47:34.14832
61582	OCUL	2020-10-05	8.2	8.51	8	8.42	571088	{}	\N	2020-10-12 15:47:34.154418
61583	OCUL	2020-10-06	8.46	8.77	8.36	8.41	443594	{}	\N	2020-10-12 15:47:34.159619
61584	OCUL	2020-10-07	9.4	10.95	9.2	10.82	7328888	{}	\N	2020-10-12 15:47:34.16467
61585	OCUL	2020-10-08	10.81	11.47	10	11.21	3254029	{}	\N	2020-10-12 15:47:34.170429
61586	OCUL	2020-10-09	11.44	12.07	10.985	11.27	1857044	{}	\N	2020-10-12 15:47:34.175783
61587	OCUL	2020-10-12	11.4	11.56	10.92	10.9644	564394	{}	\N	2020-10-12 15:47:34.182386
61588	OEF	2020-09-28	154.8	155.38	154.28	155.13	160422	{}	\N	2020-10-12 15:47:35.192305
61589	OEF	2020-09-29	155.06	155.1942	153.91	154.23	113048	{}	\N	2020-10-12 15:47:35.199183
61590	OEF	2020-09-30	154.59	157.15	154.59	155.73	143897	{}	\N	2020-10-12 15:47:35.205683
61591	OEF	2020-10-01	157.14	157.35	155.87	156.76	317928	{}	\N	2020-10-12 15:47:35.212272
61592	OEF	2020-10-02	153.98	155.748	153.81	154.44	124794	{}	\N	2020-10-12 15:47:35.219391
61593	OEF	2020-10-05	155.43	157.31	155.43	157.3	117136	{}	\N	2020-10-12 15:47:35.226818
61594	OEF	2020-10-06	157.22	157.995	154.47	154.79	183814	{}	\N	2020-10-12 15:47:35.234216
61595	OEF	2020-10-07	156.265	157.8377	156.16	157.52	476193	{}	\N	2020-10-12 15:47:35.240434
61596	OEF	2020-10-08	158.57	158.63	158.14	158.57	82009	{}	\N	2020-10-12 15:47:35.247548
61597	OEF	2020-10-09	159.4	160.35	159.24	160.35	142034	{}	\N	2020-10-12 15:47:35.253491
61598	OEF	2020-10-12	161.85	163.29	161.8	163.27	32001	{}	\N	2020-10-12 15:47:35.258481
61599	OFS	2020-09-28	4.48	4.81	4.4501	4.7	92288	{}	\N	2020-10-12 15:47:36.140461
61600	OFS	2020-09-29	4.63	4.74	4.6	4.65	37847	{}	\N	2020-10-12 15:47:36.146442
61601	OFS	2020-09-30	4.7	4.8	4.665	4.71	31098	{}	\N	2020-10-12 15:47:36.153565
61602	OFS	2020-10-01	4.73	4.76	4.645	4.71	24109	{}	\N	2020-10-12 15:47:36.159214
61603	OFS	2020-10-02	4.57	4.68	4.56	4.67	34131	{}	\N	2020-10-12 15:47:36.16473
61604	OFS	2020-10-05	4.67	4.69	4.5761	4.62	47950	{}	\N	2020-10-12 15:47:36.170079
61605	OFS	2020-10-06	4.645	4.69	4.5993	4.64	20312	{}	\N	2020-10-12 15:47:36.175585
61606	OFS	2020-10-07	4.67	4.7	4.56	4.66	35345	{}	\N	2020-10-12 15:47:36.181029
61607	OFS	2020-10-08	4.64	4.7	4.51	4.51	108425	{}	\N	2020-10-12 15:47:36.187336
61608	OFS	2020-10-09	4.58	4.59	4.4733	4.5	53755	{}	\N	2020-10-12 15:47:36.194765
61609	OFS	2020-10-12	4.48	4.57	4.4758	4.5175	9512	{}	\N	2020-10-12 15:47:36.201189
61610	OHI	2020-09-28	30.6	30.89	30.15	30.21	1199483	{}	\N	2020-10-12 15:47:37.121778
61611	OHI	2020-09-29	30.1	30.17	29.11	29.72	2598654	{}	\N	2020-10-12 15:47:37.130133
61612	OHI	2020-09-30	29.88	30.6	29.525	29.94	2667524	{}	\N	2020-10-12 15:47:37.139241
61613	OHI	2020-10-01	29.97	31.1	29.97	31.09	2197361	{}	\N	2020-10-12 15:47:37.153365
61614	OHI	2020-10-02	30.51	31.797	30.27	31.67	3269692	{}	\N	2020-10-12 15:47:37.160761
61615	OHI	2020-10-05	31.8	32.0452	31.03	31.64	2042184	{}	\N	2020-10-12 15:47:37.166134
61616	OHI	2020-10-06	31.75	32.28	31.29	31.64	2245037	{}	\N	2020-10-12 15:47:37.171401
61617	OHI	2020-10-07	31.8	31.89	31.21	31.22	1275505	{}	\N	2020-10-12 15:47:37.176775
61618	OHI	2020-10-08	31.42	31.91	31.25	31.68	2300683	{}	\N	2020-10-12 15:47:37.181886
61619	OHI	2020-10-09	31.84	31.93	30.91	31.05	2304438	{}	\N	2020-10-12 15:47:37.187976
61620	OHI	2020-10-12	31.12	31.58	30.9601	31.365	458330	{}	\N	2020-10-12 15:47:37.19474
61621	OILK	2020-09-28	36.54	37.12	36.54	36.96	24487	{}	\N	2020-10-12 15:47:38.177497
61622	OILK	2020-09-29	36.29	36.4	35.1	35.54	81327	{}	\N	2020-10-12 15:47:38.183402
61623	OILK	2020-09-30	35.965	36.7	35.91	36.4	19962	{}	\N	2020-10-12 15:47:38.1885
61624	OILK	2020-10-01	35.51	35.79	34.35	35.26	51590	{}	\N	2020-10-12 15:47:38.193583
61625	OILK	2020-10-02	33.485	34.48	33.46	33.789	80420	{}	\N	2020-10-12 15:47:38.199397
61626	OILK	2020-10-05	35.5	36.1499	35.24	35.9	97003	{}	\N	2020-10-12 15:47:38.204478
61627	OILK	2020-10-06	36.75	37.19	36.42	36.54	47556	{}	\N	2020-10-12 15:47:38.209939
61628	OILK	2020-10-07	36.32	36.49	35.82	36.49	18695	{}	\N	2020-10-12 15:47:38.215877
61629	OILK	2020-10-08	37.328	37.61	37.0404	37.59	62105	{}	\N	2020-10-12 15:47:38.220364
61630	OILK	2020-10-09	37.37	37.68	36.9	36.95	20095	{}	\N	2020-10-12 15:47:38.225503
61631	OILK	2020-10-12	36.6	36.6	35.76	35.8338	10151	{}	\N	2020-10-12 15:47:38.231314
61632	OMEX	2020-09-28	6.3748	6.469	6.24	6.45	32941	{}	\N	2020-10-12 15:47:39.118118
61633	OMEX	2020-09-29	6.4	6.45	6.09	6.43	30289	{}	\N	2020-10-12 15:47:39.127649
61634	OMEX	2020-09-30	6.45	6.84	6.35	6.8	35638	{}	\N	2020-10-12 15:47:39.135348
61635	OMEX	2020-10-01	6.78	6.825	6.6	6.68	8393	{}	\N	2020-10-12 15:47:39.141615
61636	OMEX	2020-10-02	6.6	7.1359	6.6	6.95	55350	{}	\N	2020-10-12 15:47:39.14828
61637	OMEX	2020-10-05	6.96	6.96	6.815	6.86	10453	{}	\N	2020-10-12 15:47:39.154492
61638	OMEX	2020-10-06	6.845	7.305	6.7619	6.94	47288	{}	\N	2020-10-12 15:47:39.160578
61639	OMEX	2020-10-07	7.2	7.2	6.75	6.8	24687	{}	\N	2020-10-12 15:47:39.165673
61640	OMEX	2020-10-08	6.79	6.79	6.12	6.595	42605	{}	\N	2020-10-12 15:47:39.170458
61641	OMEX	2020-10-09	6.75	6.75	6.55	6.7062	18167	{}	\N	2020-10-12 15:47:39.175309
61642	OMEX	2020-10-12	6.75	6.77	6.6659	6.6659	2077	{}	\N	2020-10-12 15:47:39.179583
61643	ONDK	2020-09-28	1.57	1.62	1.54	1.59	320732	{}	\N	2020-10-12 15:47:40.12696
61644	ONDK	2020-09-29	1.56	1.6	1.54	1.58	586325	{}	\N	2020-10-12 15:47:40.134676
61645	ONDK	2020-09-30	1.58	1.63	1.5799	1.6	933023	{}	\N	2020-10-12 15:47:40.141747
61646	ONDK	2020-10-01	1.61	1.65	1.57	1.64	469843	{}	\N	2020-10-12 15:47:40.148971
61647	ONDK	2020-10-02	1.59	1.7	1.57	1.69	2259151	{}	\N	2020-10-12 15:47:40.155611
61648	ONDK	2020-10-05	1.7	1.75	1.68	1.75	4278052	{}	\N	2020-10-12 15:47:40.163091
61649	ONDK	2020-10-06	1.72	1.8	1.68	1.69	899495	{}	\N	2020-10-12 15:47:40.168292
61650	ONDK	2020-10-07	1.75	1.759	1.68	1.73	399899	{}	\N	2020-10-12 15:47:40.173401
61651	ONDK	2020-10-08	1.75	1.86	1.71	1.85	1640455	{}	\N	2020-10-12 15:47:40.178666
61652	ONDK	2020-10-09	1.91	1.96	1.85	1.89	898258	{}	\N	2020-10-12 15:47:40.183962
61653	ONDK	2020-10-12	1.87	1.915	1.84	1.875	309869	{}	\N	2020-10-12 15:47:40.189236
61654	OPP	2020-09-28	13.93	13.99	13.93	13.965	122380	{}	\N	2020-10-12 15:47:41.619211
61655	OPP	2020-09-29	13.88	13.95	13.62	13.67	305982	{}	\N	2020-10-12 15:47:41.625083
61656	OPP	2020-09-30	13.74	13.87	13.72	13.72	259797	{}	\N	2020-10-12 15:47:41.630602
61657	OPP	2020-10-01	13.81	13.92	13.8	13.88	73284	{}	\N	2020-10-12 15:47:41.636151
61658	OPP	2020-10-02	13.85	13.941	13.8	13.92	125687	{}	\N	2020-10-12 15:47:41.64245
61659	OPP	2020-10-05	13.85	13.962	13.85	13.88	114628	{}	\N	2020-10-12 15:47:41.649592
61660	OPP	2020-10-06	13.86	14.1549	13.86	13.96	197887	{}	\N	2020-10-12 15:47:41.655527
61661	OPP	2020-10-07	13.96	14.14	13.96	14.04	59451	{}	\N	2020-10-12 15:47:41.661485
61662	OPP	2020-10-08	14.05	14.14	14.02	14.12	128602	{}	\N	2020-10-12 15:47:41.66706
61663	OPP	2020-10-09	14.08	14.15	14.08	14.11	144386	{}	\N	2020-10-12 15:47:41.672189
61664	OPP	2020-10-12	14.12	14.15	14.09	14.11	93978	{}	\N	2020-10-12 15:47:41.677922
61665	OSBC	2020-09-28	7.35	7.63	7.35	7.56	49500	{}	\N	2020-10-12 15:47:47.11831
61666	OSBC	2020-09-29	7.52	7.56	7.32	7.53	42712	{}	\N	2020-10-12 15:47:47.124071
61667	OSBC	2020-09-30	7.54	7.702	7.42	7.495	73774	{}	\N	2020-10-12 15:47:47.129302
61668	OSBC	2020-10-01	7.55	7.78	7.38	7.74	100785	{}	\N	2020-10-12 15:47:47.136166
61669	OSBC	2020-10-02	7.67	8.16	7.6	8.13	70307	{}	\N	2020-10-12 15:47:47.143255
61670	OSBC	2020-10-05	8.2	8.35	8.13	8.26	39749	{}	\N	2020-10-12 15:47:47.149477
61671	OSBC	2020-10-06	8.35	8.76	8.3	8.34	47598	{}	\N	2020-10-12 15:47:47.15709
61672	OSBC	2020-10-07	8.38	8.58	8.3	8.56	36152	{}	\N	2020-10-12 15:47:47.16299
61673	OSBC	2020-10-08	8.65	8.7	8.3	8.53	62074	{}	\N	2020-10-12 15:47:47.170064
61674	OSBC	2020-10-09	8.64	8.64	8.38	8.51	26761	{}	\N	2020-10-12 15:47:47.175398
61675	OSBC	2020-10-12	8.5799	8.5799	8.37	8.54	4190	{}	\N	2020-10-12 15:47:47.181904
61676	OSN	2020-09-28	4.34	4.38	4.3	4.34	10119	{}	\N	2020-10-12 15:47:48.081267
61677	OSN	2020-09-29	4.3793	4.46	4.3793	4.46	18554	{}	\N	2020-10-12 15:47:48.088036
61678	OSN	2020-09-30	4.42	4.59	4.41	4.59	20389	{}	\N	2020-10-12 15:47:48.093286
61679	OSN	2020-10-01	4.51	4.5899	4.41	4.58	17482	{}	\N	2020-10-12 15:47:48.09885
61680	OSN	2020-10-02	4.5	4.56	4.4235	4.43	6642	{}	\N	2020-10-12 15:47:48.106007
61681	OSN	2020-10-05	4.448	4.51	4.39	4.45	9761	{}	\N	2020-10-12 15:47:48.112101
61682	OSN	2020-10-06	4.52	4.53	4.43	4.48	19720	{}	\N	2020-10-12 15:47:48.120793
61683	OSN	2020-10-07	4.49	4.52	4.35	4.36	33495	{}	\N	2020-10-12 15:47:48.128788
61684	OSN	2020-10-08	4.37	4.38	4.27	4.2801	32834	{}	\N	2020-10-12 15:47:48.136404
61685	OSN	2020-10-09	4.26	4.4686	4.26	4.42	6825	{}	\N	2020-10-12 15:47:48.142542
61686	OSN	2020-10-12	4.42	4.42	4.365	4.365	1741	{}	\N	2020-10-12 15:47:48.148043
61687	OFED	2020-09-28	21.845	21.845	21.47	21.47	956	{}	\N	2020-10-12 15:47:49.246444
61688	OFED	2020-10-01	20.81	20.81	20.81	20.81	359	{}	\N	2020-10-12 15:47:49.252774
61689	OFED	2020-10-02	20.7	20.7	20.7	20.7	360	{}	\N	2020-10-12 15:47:49.257675
61690	OFED	2020-10-05	21.24	21.24	21.24	21.24	518	{}	\N	2020-10-12 15:47:49.26248
61691	OFED	2020-10-06	22	22.3	21.7089	22.3	3505	{}	\N	2020-10-12 15:47:49.267431
61692	OFED	2020-10-07	24.21	25.66	23.01	25.37	1499	{}	\N	2020-10-12 15:47:49.272505
61693	OFED	2020-10-08	23.5	25.5671	23.38	24.24	872	{}	\N	2020-10-12 15:47:49.27754
61694	OGS	2020-09-28	68.62	69.33	68.38	68.49	192803	{}	\N	2020-10-12 15:47:50.12605
61695	OGS	2020-09-29	68.79	68.9	67.7	68.44	320057	{}	\N	2020-10-12 15:47:50.132305
61696	OGS	2020-09-30	68.69	69.3	68.26	69.01	637817	{}	\N	2020-10-12 15:47:50.138164
61697	OGS	2020-10-01	68.85	69.17	68.21	68.88	563244	{}	\N	2020-10-12 15:47:50.143322
61698	OGS	2020-10-02	68.05	70.15	67.92	69.69	474489	{}	\N	2020-10-12 15:47:50.148101
61699	OGS	2020-10-05	69.93	70.4	69.61	70.29	426686	{}	\N	2020-10-12 15:47:50.154571
61700	OGS	2020-10-06	70.91	73.05	70.53	71.81	695996	{}	\N	2020-10-12 15:47:50.159955
61701	OGS	2020-10-07	72.09	72.25	71.1	72.15	498379	{}	\N	2020-10-12 15:47:50.165999
61702	OGS	2020-10-08	72.36	73.98	72.36	73.91	413094	{}	\N	2020-10-12 15:47:50.173055
61703	OGS	2020-10-09	74.37	74.54	73.01	73.72	399329	{}	\N	2020-10-12 15:47:50.178736
61704	OGS	2020-10-12	73.46	74.36	73.3904	74.27	73745	{}	\N	2020-10-12 15:47:50.185504
61705	OIA	2020-09-28	7.34	7.42	7.3	7.32	87168	{}	\N	2020-10-12 15:47:51.440864
61706	OIA	2020-09-29	7.32	7.37	7.319	7.3607	38656	{}	\N	2020-10-12 15:47:51.447489
61707	OIA	2020-09-30	7.43	7.43	7.3	7.3	159496	{}	\N	2020-10-12 15:47:51.461741
61708	OIA	2020-10-01	7.33	7.353	7.29	7.31	134744	{}	\N	2020-10-12 15:47:51.470403
61709	OIA	2020-10-02	7.29	7.33	7.29	7.33	62451	{}	\N	2020-10-12 15:47:51.47597
61710	OIA	2020-10-05	7.31	7.31	7.28	7.3	138181	{}	\N	2020-10-12 15:47:51.481059
61711	OIA	2020-10-06	7.26	7.31	7.26	7.3	72796	{}	\N	2020-10-12 15:47:51.487518
61712	OIA	2020-10-07	7.3	7.3771	7.3	7.3495	131952	{}	\N	2020-10-12 15:47:51.493761
61713	OIA	2020-10-08	7.36	7.39	7.33	7.38	74632	{}	\N	2020-10-12 15:47:51.500397
61714	OIA	2020-10-09	7.35	7.42	7.35	7.39	95856	{}	\N	2020-10-12 15:47:51.506484
61715	OIA	2020-10-12	7.39	7.44	7.39	7.4399	36792	{}	\N	2020-10-12 15:47:51.512372
61716	OIS	2020-09-28	2.8	3.13	2.78	3	1442178	{}	\N	2020-10-12 15:47:52.428946
61717	OIS	2020-09-29	3	3	2.7	2.83	1788844	{}	\N	2020-10-12 15:47:52.435014
61718	OIS	2020-09-30	2.86	2.9	2.71	2.73	1459547	{}	\N	2020-10-12 15:47:52.440447
61719	OIS	2020-10-01	2.69	2.71	2.5	2.55	4574311	{}	\N	2020-10-12 15:47:52.445616
61720	OIS	2020-10-02	2.49	2.83	2.42	2.69	2255819	{}	\N	2020-10-12 15:47:52.450708
61721	OIS	2020-10-05	2.88	2.97	2.75	2.82	1444335	{}	\N	2020-10-12 15:47:52.458048
61722	OIS	2020-10-06	2.9	3.03	2.73	2.81	1891908	{}	\N	2020-10-12 15:47:52.463823
61723	OIS	2020-10-07	2.83	2.95	2.695	2.8	1941378	{}	\N	2020-10-12 15:47:52.469806
61724	OIS	2020-10-08	2.87	2.97	2.84	2.94	1650141	{}	\N	2020-10-12 15:47:52.476076
61725	OIS	2020-10-09	2.96	2.99	2.78	2.86	1614890	{}	\N	2020-10-12 15:47:52.481804
61726	OIS	2020-10-12	2.83	2.92	2.78	2.8469	810653	{}	\N	2020-10-12 15:47:52.488045
61727	OLLI	2020-09-28	88.33	88.855	86.67	87.47	464286	{}	\N	2020-10-12 15:47:54.127397
61728	OLLI	2020-09-29	87.44	90.66	87.44	87.96	677867	{}	\N	2020-10-12 15:47:54.137056
61729	OLLI	2020-09-30	89.07	89.07	85.745	87.35	802718	{}	\N	2020-10-12 15:47:54.146774
61730	OLLI	2020-10-01	88	89.48	87.055	88.24	747715	{}	\N	2020-10-12 15:47:54.155279
61731	OLLI	2020-10-02	85.55	89.505	85.55	88.78	721279	{}	\N	2020-10-12 15:47:54.166798
61732	OLLI	2020-10-05	89.24	92.59	89.24	92.21	864392	{}	\N	2020-10-12 15:47:54.173337
61733	OLLI	2020-10-06	92.2	93.32	90.09	90.49	881116	{}	\N	2020-10-12 15:47:54.180791
61734	OLLI	2020-10-07	92.22	93.53	91.43	93.14	642757	{}	\N	2020-10-12 15:47:54.187002
61735	OLLI	2020-10-08	94.49	94.89	92.54	93.32	551509	{}	\N	2020-10-12 15:47:54.193281
61736	OLLI	2020-10-09	95	95.25	93.41	94.78	400122	{}	\N	2020-10-12 15:47:54.20077
61737	OLLI	2020-10-12	95.15	95.5547	92.25	92.9	195739	{}	\N	2020-10-12 15:47:54.208414
61738	OMCL	2020-09-28	71.8	72.79	71.34	71.65	441675	{}	\N	2020-10-12 15:47:55.630251
61739	OMCL	2020-09-29	71.73	72.905	71.105	72.5	367095	{}	\N	2020-10-12 15:47:55.635778
61740	OMCL	2020-09-30	72.77	75.52	72.35	74.66	416067	{}	\N	2020-10-12 15:47:55.641211
61741	OMCL	2020-10-01	76.26	76.26	74.26	75.11	392667	{}	\N	2020-10-12 15:47:55.64713
61742	OMCL	2020-10-02	73.46	75.12	72.8	73.85	304273	{}	\N	2020-10-12 15:47:55.653175
61743	OMCL	2020-10-05	74.41	75.89	69.015	74.5	217831	{}	\N	2020-10-12 15:47:55.659904
61744	OMCL	2020-10-06	75.2	77.63	74.46	76.49	373851	{}	\N	2020-10-12 15:47:55.667382
61745	OMCL	2020-10-07	77.24	78.23	76.33	77.68	305434	{}	\N	2020-10-12 15:47:55.672897
61746	OMCL	2020-10-08	78.85	79.84	78.25	79.7	294893	{}	\N	2020-10-12 15:47:55.678954
61747	OMCL	2020-10-09	80	80.16	79	79.54	138792	{}	\N	2020-10-12 15:47:55.684124
61748	OMCL	2020-10-12	80.03	81.23	79.555	81.03	35845	{}	\N	2020-10-12 15:47:55.688952
61749	OMF	2020-09-28	30.34	30.89	30.1	30.49	349156	{}	\N	2020-10-12 15:47:57.142431
61750	OMF	2020-09-29	30.34	30.65	29.87	30.47	622735	{}	\N	2020-10-12 15:47:57.150229
61751	OMF	2020-09-30	30.62	31.49	30.62	31.25	1270836	{}	\N	2020-10-12 15:47:57.15661
61752	OMF	2020-10-01	31.51	32.44	30.87	32.19	1529384	{}	\N	2020-10-12 15:47:57.164673
61753	OMF	2020-10-02	31.37	33.6	31.3	33.34	990461	{}	\N	2020-10-12 15:47:57.172392
61754	OMF	2020-10-05	33.62	34.51	33.58	34.39	2017274	{}	\N	2020-10-12 15:47:57.178116
61755	OMF	2020-10-06	34.99	35.8	33.88	34.35	2726294	{}	\N	2020-10-12 15:47:57.183918
61756	OMF	2020-10-07	35.06	35.76	34.82	35.46	1554953	{}	\N	2020-10-12 15:47:57.189105
61757	OMF	2020-10-08	35.7	36.48	35.54	36.14	864936	{}	\N	2020-10-12 15:47:57.194606
61758	OMF	2020-10-09	36.89	36.99	36.13	36.41	1142024	{}	\N	2020-10-12 15:47:57.200168
61759	OMF	2020-10-12	36.56	36.7436	36.265	36.33	259250	{}	\N	2020-10-12 15:47:57.205775
61760	OMI	2020-09-28	20.65	21.88	19.91	21.69	2634504	{}	\N	2020-10-12 15:47:58.7166
61761	OMI	2020-09-29	21.73	23.95	21.26	23.52	5896636	{}	\N	2020-10-12 15:47:58.722314
61762	OMI	2020-09-30	22.74	25.2	22.06	25.11	8678137	{}	\N	2020-10-12 15:47:58.727161
61763	OMI	2020-10-01	25.13	25.3	21.64	22.56	7788766	{}	\N	2020-10-12 15:47:58.732732
61764	OMI	2020-10-02	23.48	25.95	22.055	22.5	18161905	{}	\N	2020-10-12 15:47:58.738058
61765	OMI	2020-10-05	22.56	25.2	22.52	24.36	13114211	{}	\N	2020-10-12 15:47:58.744015
61766	OMI	2020-10-06	24.42	26.93	24.15	25.91	6079524	{}	\N	2020-10-12 15:47:58.75123
61767	OMI	2020-10-07	26.03	26.525	25.15	26.29	4505895	{}	\N	2020-10-12 15:47:58.756753
61768	OMI	2020-10-08	26.38	27.44	26.0601	27.3	3195050	{}	\N	2020-10-12 15:47:58.762193
61769	OMI	2020-10-09	27.36	27.75	25.99	26.36	2581086	{}	\N	2020-10-12 15:47:58.768222
61770	OMI	2020-10-12	26.69	26.88	26.19	26.33	652686	{}	\N	2020-10-12 15:47:58.773436
61771	ONEY	2020-09-28	60.76	61.2533	60.76	61.213	3731	{}	\N	2020-10-12 15:48:00.672143
61772	ONEY	2020-09-29	60.7453	60.7453	60.0486	60.21	3724	{}	\N	2020-10-12 15:48:00.677852
61773	ONEY	2020-09-30	60.56	60.6408	60.56	60.6408	1291	{}	\N	2020-10-12 15:48:00.683545
61774	ONEY	2020-10-01	60.5001	60.6299	60.5	60.6299	686	{}	\N	2020-10-12 15:48:00.688281
61775	ONEY	2020-10-02	61.738	61.738	61.738	61.738	81	{}	\N	2020-10-12 15:48:00.694533
61776	ONEY	2020-10-05	62.05	62.8556	62.05	62.8556	2220	{}	\N	2020-10-12 15:48:00.699896
61777	ONEY	2020-10-06	63.5213	64	62.39	62.39	3925	{}	\N	2020-10-12 15:48:00.704808
61778	ONEY	2020-10-07	63.4133	63.5866	63.4133	63.5866	636	{}	\N	2020-10-12 15:48:00.710069
61779	ONEY	2020-10-08	64.07	64.81	64.07	64.8099	2790	{}	\N	2020-10-12 15:48:00.71637
61780	ONEY	2020-10-09	65.3	65.3	64.4453	64.4453	2547	{}	\N	2020-10-12 15:48:00.721071
61781	ONEY	2020-10-12	64.82	64.85	64.6701	64.85	834	{}	\N	2020-10-12 15:48:00.726183
61782	ODT	2020-09-28	13.45	13.5845	12.56	13.14	568819	{}	\N	2020-10-12 15:48:03.993458
61783	ODT	2020-09-29	13.08	14.61	12.94	14.52	457561	{}	\N	2020-10-12 15:48:03.998591
61784	ODT	2020-09-30	14.39	14.59	13.36	13.43	336314	{}	\N	2020-10-12 15:48:04.004681
61785	ODT	2020-10-01	14.4	15.4	14.4	14.65	607955	{}	\N	2020-10-12 15:48:04.009422
61786	ODT	2020-10-02	14.17	15.03	14.0004	14.66	275418	{}	\N	2020-10-12 15:48:04.017399
61787	ODT	2020-10-05	14.88	16.25	14.66	16.11	538706	{}	\N	2020-10-12 15:48:04.026773
61788	ODT	2020-10-06	16.29	16.48	15.81	16.14	339612	{}	\N	2020-10-12 15:48:04.033083
61789	ODT	2020-10-07	16.36	16.68	15.85	16.24	337689	{}	\N	2020-10-12 15:48:05.045707
61790	ODT	2020-10-08	16.47	16.55	16.11	16.46	204084	{}	\N	2020-10-12 15:48:05.081039
61791	ODT	2020-10-09	16.6	16.6	15.8629	16.33	233274	{}	\N	2020-10-12 15:48:05.087344
61792	ODT	2020-10-12	16.5	16.62	16.01	16.6	43472	{}	\N	2020-10-12 15:48:05.093464
61793	OFIX	2020-09-28	29.14	30.56	29.14	30.5	93077	{}	\N	2020-10-12 15:48:06.555542
61794	OFIX	2020-09-29	31.0426	31.0426	30.32	30.9	143694	{}	\N	2020-10-12 15:48:06.562636
61795	OFIX	2020-09-30	30.91	31.6226	30.91	31.14	149168	{}	\N	2020-10-12 15:48:06.567747
61796	OFIX	2020-10-01	31.8614	31.8614	30.56	31.46	90148	{}	\N	2020-10-12 15:48:06.573778
61797	OFIX	2020-10-02	30.9	31.54	30.555	31.35	120351	{}	\N	2020-10-12 15:48:06.580938
61798	OFIX	2020-10-05	31.61	32.56	31.03	32.39	75821	{}	\N	2020-10-12 15:48:06.587786
61799	OFIX	2020-10-06	32.07	32.85	31.73	32.38	158659	{}	\N	2020-10-12 15:48:06.593119
61800	OFIX	2020-10-07	32.71	33.47	32.27	33.29	114102	{}	\N	2020-10-12 15:48:06.598985
61801	OFIX	2020-10-08	33.61	34.08	33.08	33.89	79855	{}	\N	2020-10-12 15:48:06.604786
61802	OFIX	2020-10-09	34.01	34.42	33.111	34.17	55583	{}	\N	2020-10-12 15:48:06.610666
61803	OFIX	2020-10-12	34.08	34.48	32.935	34.46	16873	{}	\N	2020-10-12 15:48:06.616191
61804	OI	2020-09-28	10.15	10.61	10.15	10.57	1113812	{}	\N	2020-10-12 15:48:08.198826
61805	OI	2020-09-29	10.62	10.7	10.421	10.58	1323298	{}	\N	2020-10-12 15:48:08.205459
61806	OI	2020-09-30	10.66	10.95	10.395	10.59	3725099	{}	\N	2020-10-12 15:48:08.211604
61807	OI	2020-10-01	10.69	10.95	10.54	10.82	2718652	{}	\N	2020-10-12 15:48:08.21741
61808	OI	2020-10-02	10.52	11.37	10.47	11.33	2765032	{}	\N	2020-10-12 15:48:08.223955
61809	OI	2020-10-05	11.53	11.96	11.5299	11.84	1986139	{}	\N	2020-10-12 15:48:08.231111
61810	OI	2020-10-06	11.98	12.13	11.33	11.33	1862200	{}	\N	2020-10-12 15:48:08.236459
61811	OI	2020-10-07	11.56	11.9	11.43	11.43	2517696	{}	\N	2020-10-12 15:48:08.242936
61812	OI	2020-10-08	11.58	12	11.57	11.99	2236891	{}	\N	2020-10-12 15:48:08.248484
61813	OI	2020-10-09	11.99	12.32	11.34	11.44	2278231	{}	\N	2020-10-12 15:48:08.253541
61814	OI	2020-10-12	11.48	11.62	11.32	11.4045	375487	{}	\N	2020-10-12 15:48:08.260789
61815	OKE	2020-09-28	25.48	26.2568	25.27	26.01	3966728	{}	\N	2020-10-12 15:48:09.193328
61816	OKE	2020-09-29	26.62	26.9	26.22	26.67	11523257	{}	\N	2020-10-12 15:48:09.199297
61817	OKE	2020-09-30	26.61	27.27	25.89	25.98	11502538	{}	\N	2020-10-12 15:48:09.204594
61818	OKE	2020-10-01	25.75	26.18	25.14	25.27	7362342	{}	\N	2020-10-12 15:48:09.211608
61819	OKE	2020-10-02	24.63	26.74	24.56	26.53	8553247	{}	\N	2020-10-12 15:48:09.217086
61820	OKE	2020-10-05	27.51	27.99	26.99	27.61	7726473	{}	\N	2020-10-12 15:48:09.222115
61821	OKE	2020-10-06	28.2	28.44	26.56	27.06	9277225	{}	\N	2020-10-12 15:48:09.229393
61822	OKE	2020-10-07	27.18	27.95	26.99	27.81	6722851	{}	\N	2020-10-12 15:48:09.234318
61823	OKE	2020-10-08	28.08	29.435	27.6796	29.41	9039568	{}	\N	2020-10-12 15:48:09.238707
61824	OKE	2020-10-09	29.7	29.77	28.9261	29.05	7319292	{}	\N	2020-10-12 15:48:09.244921
61825	OKE	2020-10-12	29.02	29.3	28.771	29.03	2208782	{}	\N	2020-10-12 15:48:09.249682
61826	OLN	2020-09-28	12.23	12.48	12.1	12.25	1991056	{}	\N	2020-10-12 15:48:10.648
61827	OLN	2020-09-29	12.22	12.49	12.13	12.21	2803914	{}	\N	2020-10-12 15:48:10.654693
61828	OLN	2020-09-30	12.28	12.745	12.28	12.38	5266324	{}	\N	2020-10-12 15:48:10.660474
61829	OLN	2020-10-01	12.39	12.66	12.345	12.48	3845439	{}	\N	2020-10-12 15:48:10.665083
61830	OLN	2020-10-02	12.15	13.01	12.14	12.93	4089703	{}	\N	2020-10-12 15:48:10.669683
61831	OLN	2020-10-05	13.16	13.585	13.085	13.51	5474232	{}	\N	2020-10-12 15:48:10.676286
61832	OLN	2020-10-06	13.71	14.83	13.63	14.3	11175536	{}	\N	2020-10-12 15:48:10.682592
61833	OLN	2020-10-07	14.56	14.76	14.34	14.65	6564647	{}	\N	2020-10-12 15:48:10.688603
61834	OLN	2020-10-08	14.87	15.8	14.8	15.74	7502207	{}	\N	2020-10-12 15:48:10.695018
61835	OLN	2020-10-09	15.85	16.05	15.36	15.37	5680175	{}	\N	2020-10-12 15:48:10.699787
61836	OLN	2020-10-12	15.6	15.61	15.31	15.555	1792154	{}	\N	2020-10-12 15:48:10.705007
61837	ONTXW	2020-09-28	0.0574	0.07	0.0571	0.0571	13072	{}	\N	2020-10-12 15:48:12.227672
61838	ONTXW	2020-09-29	0.0571	0.0638	0.0331	0.0638	16305	{}	\N	2020-10-12 15:48:12.234442
61839	ONTXW	2020-09-30	0.0599	0.07	0.055	0.055	14820	{}	\N	2020-10-12 15:48:12.239689
61840	ONTXW	2020-10-01	0.055	0.0553	0.045	0.05	16253	{}	\N	2020-10-12 15:48:12.246447
61841	ONTXW	2020-10-02	0.045	0.0653	0.045	0.0653	4455	{}	\N	2020-10-12 15:48:12.251932
61842	ONTXW	2020-10-05	0.067	0.069	0.0505	0.0533	21650	{}	\N	2020-10-12 15:48:12.257618
61843	ONTXW	2020-10-07	0.0505	0.061501	0.0505	0.0533	2550	{}	\N	2020-10-12 15:48:12.264664
61844	ONTXW	2020-10-08	0.059	0.059	0.054999	0.059	783	{}	\N	2020-10-12 15:48:12.270729
61845	ONTXW	2020-10-09	0.058	0.0685	0.0579	0.067399	11869	{}	\N	2020-10-12 15:48:12.276461
61846	ONTXW	2020-10-12	0.059	0.062751	0.059	0.062751	656	{}	\N	2020-10-12 15:48:12.282697
61847	OPOF	2020-09-28	15.55	15.58	15.0201	15.14	5945	{}	\N	2020-10-12 15:48:14.169463
61848	OPOF	2020-09-29	15.2358	15.2358	15.16	15.2285	1076	{}	\N	2020-10-12 15:48:14.175141
61849	OPOF	2020-09-30	15.5	15.65	15.15	15.15	1083	{}	\N	2020-10-12 15:48:14.181055
61850	OPOF	2020-10-02	15.54	15.61	15.5	15.55	5660	{}	\N	2020-10-12 15:48:14.187031
61851	OPOF	2020-10-05	15.4663	16	15.1783	15.62	1848	{}	\N	2020-10-12 15:48:14.191695
61852	OPOF	2020-10-06	15.9314	16	15.86	15.86	2586	{}	\N	2020-10-12 15:48:14.197168
61853	OPOF	2020-10-07	15.58	16.3	15.3	16.3	911	{}	\N	2020-10-12 15:48:14.202075
61854	OPOF	2020-10-08	15.76	15.76	15.71	15.71	753	{}	\N	2020-10-12 15:48:14.206978
61855	OPOF	2020-10-09	15.76	15.76	15.76	15.76	160	{}	\N	2020-10-12 15:48:14.212466
61856	OPTT	2020-09-28	1	1.07	0.9606	0.9975	2349783	{}	\N	2020-10-12 15:48:15.561485
61857	OPTT	2020-09-29	0.982	1.03	0.9501	1.01	1053517	{}	\N	2020-10-12 15:48:15.568598
61858	OPTT	2020-09-30	0.99	1.05	0.96401	1.01	1123143	{}	\N	2020-10-12 15:48:15.574323
61859	OPTT	2020-10-01	1.02	1.13	0.9935	1.04	3228966	{}	\N	2020-10-12 15:48:15.578804
61860	OPTT	2020-10-02	0.98	1.1	0.972	1.05	1988222	{}	\N	2020-10-12 15:48:15.585505
61861	OPTT	2020-10-05	1.05	1.35	1.05	1.31	27783378	{}	\N	2020-10-12 15:48:15.591607
61862	OPTT	2020-10-06	1.49	2.62	1.37	2.17	157391421	{}	\N	2020-10-12 15:48:15.598382
61863	OPTT	2020-10-07	3.18	3.72	2.68	2.98	169060784	{}	\N	2020-10-12 15:48:15.605029
61864	OPTT	2020-10-08	2.54	2.73	1.87	1.95	26272246	{}	\N	2020-10-12 15:48:15.610141
61865	OPTT	2020-10-09	2.1	2.88	1.9	2.55	46784219	{}	\N	2020-10-12 15:48:15.616234
61866	OPTT	2020-10-12	2.36	2.54	2.27	2.365	7315648	{}	\N	2020-10-12 15:48:15.621735
61867	ORCL	2020-09-28	60.03	60.54	59.47	59.58	9181941	{}	\N	2020-10-12 15:48:17.031709
61868	ORCL	2020-09-29	59.58	60.16	59.4	59.47	12123040	{}	\N	2020-10-12 15:48:17.038233
61869	ORCL	2020-09-30	59.52	60.39	59.37	59.7	20883715	{}	\N	2020-10-12 15:48:17.043335
61870	ORCL	2020-10-01	60.17	60.27	59.45	59.68	14218111	{}	\N	2020-10-12 15:48:17.048207
61871	ORCL	2020-10-02	58.71	59.52	58.51	58.83	13752869	{}	\N	2020-10-12 15:48:17.055375
61872	ORCL	2020-10-05	59.44	59.6664	59.09	59.56	13651962	{}	\N	2020-10-12 15:48:17.060778
61873	ORCL	2020-10-06	59.51	60.53	59.405	59.51	18270152	{}	\N	2020-10-12 15:48:17.06835
61874	ORCL	2020-10-07	59.77	60.895	59.635	60.59	16338811	{}	\N	2020-10-12 15:48:17.074081
61875	ORCL	2020-10-08	60.79	61.3	60.63	60.89	14094629	{}	\N	2020-10-12 15:48:17.079066
61876	ORCL	2020-10-09	61.15	61.38	60.86	61.15	13989284	{}	\N	2020-10-12 15:48:17.084528
61877	ORCL	2020-10-12	61.24	61.83	61.07	61.695	6841570	{}	\N	2020-10-12 15:48:17.090244
61878	OGCP	2020-09-28	6.16	6.31	6.16	6.18	2501	{}	\N	2020-10-12 15:48:18.862944
61879	OGCP	2020-09-29	6.02	6.02	6.0193	6.0193	214	{}	\N	2020-10-12 15:48:18.869462
61880	OGCP	2020-09-30	6.16	6.22	6.16	6.22	3681	{}	\N	2020-10-12 15:48:18.874667
61881	OGCP	2020-10-01	6.16	6.26	6.16	6.25	6544	{}	\N	2020-10-12 15:48:18.880802
61882	OGCP	2020-10-02	6.25	6.45	6.24	6.45	13873	{}	\N	2020-10-12 15:48:18.888251
61883	OGCP	2020-10-05	6.44	6.4722	6.4	6.47	10027	{}	\N	2020-10-12 15:48:18.894152
61884	OGCP	2020-10-06	6.52	6.54	6.52	6.53	2524	{}	\N	2020-10-12 15:48:18.900259
61885	OGCP	2020-10-07	6.5	6.55	6.5	6.55	256	{}	\N	2020-10-12 15:48:18.907656
61886	OGCP	2020-10-08	6.74	6.74	6.71	6.71	1658	{}	\N	2020-10-12 15:48:18.913494
61887	OGCP	2020-10-12	6.62	6.63	6.62	6.63	783	{}	\N	2020-10-12 15:48:18.919674
61888	OKTA	2020-09-28	216	216.45	209.6	213.33	896276	{}	\N	2020-10-12 15:48:20.532379
61889	OKTA	2020-09-29	212.18	216	211.77	213.56	623057	{}	\N	2020-10-12 15:48:20.538537
61890	OKTA	2020-09-30	212.5	215.77	210.04	213.85	721499	{}	\N	2020-10-12 15:48:20.544153
61891	OKTA	2020-10-01	216	220.8	213.62	219.3	907751	{}	\N	2020-10-12 15:48:20.549565
61892	OKTA	2020-10-02	212.97	220.71	211.8	219.06	1113633	{}	\N	2020-10-12 15:48:20.55804
61893	OKTA	2020-10-05	219.41	228	219	227.26	1501998	{}	\N	2020-10-12 15:48:20.577046
61894	OKTA	2020-10-06	226.5	235.38	226.355	232.49	1997390	{}	\N	2020-10-12 15:48:20.586651
61895	OKTA	2020-10-07	231.9	240.33	229.01	236.6	1820152	{}	\N	2020-10-12 15:48:20.594934
61896	OKTA	2020-10-08	236.6	239	232.6833	237.57	1582865	{}	\N	2020-10-12 15:48:20.602578
61897	OKTA	2020-10-09	236.98	240	234.55	236.45	1114026	{}	\N	2020-10-12 15:48:20.610732
61898	OKTA	2020-10-12	240	245.79	239	242.99	855679	{}	\N	2020-10-12 15:48:20.61859
61899	ONCS	2020-09-28	3.81	3.91	3.6	3.67	112336	{}	\N	2020-10-12 15:48:21.978544
61900	ONCS	2020-09-29	3.64	3.665	3.16	3.37	308274	{}	\N	2020-10-12 15:48:21.98621
61901	ONCS	2020-09-30	3.33	3.47	3.25	3.38	81142	{}	\N	2020-10-12 15:48:21.992037
61902	ONCS	2020-10-01	3.37	3.49	3.27	3.39	85036	{}	\N	2020-10-12 15:48:21.998127
61903	ONCS	2020-10-02	3.33	3.49	3.3	3.42	93748	{}	\N	2020-10-12 15:48:22.004793
61904	ONCS	2020-10-05	3.41	3.84	3.41	3.71	275172	{}	\N	2020-10-12 15:48:22.010649
61905	ONCS	2020-10-06	3.87	3.8761	3.525	3.56	166840	{}	\N	2020-10-12 15:48:22.016952
61906	ONCS	2020-10-07	3.54	3.85	3.54	3.71	196668	{}	\N	2020-10-12 15:48:22.023778
61907	ONCS	2020-10-08	3.73	3.85	3.62	3.7	111836	{}	\N	2020-10-12 15:48:22.029833
61908	ONCS	2020-10-09	3.7	3.81	3.56	3.56	113992	{}	\N	2020-10-12 15:48:22.036151
61909	ONCS	2020-10-12	3.6	3.72	3.53	3.7	63941	{}	\N	2020-10-12 15:48:22.042735
61910	ONTX	2020-09-28	0.285	0.285	0.271	0.28	6412116	{}	\N	2020-10-12 15:48:22.929633
61911	ONTX	2020-09-29	0.2844	0.2845	0.27	0.2759	6313484	{}	\N	2020-10-12 15:48:22.936633
61912	ONTX	2020-09-30	0.2721	0.28	0.265	0.28	5523701	{}	\N	2020-10-12 15:48:22.944832
61913	ONTX	2020-10-01	0.2741	0.275	0.265	0.2695	4971844	{}	\N	2020-10-12 15:48:22.950965
61914	ONTX	2020-10-02	0.2575	0.2755	0.255	0.2706	5039395	{}	\N	2020-10-12 15:48:22.95695
61915	ONTX	2020-10-05	0.27	0.2963	0.27	0.2801	7297498	{}	\N	2020-10-12 15:48:22.962721
61916	ONTX	2020-10-06	0.2826	0.2855	0.27	0.273	4829187	{}	\N	2020-10-12 15:48:22.968018
61917	ONTX	2020-10-07	0.273	0.2817	0.2725	0.278	3696436	{}	\N	2020-10-12 15:48:22.973711
61918	ONTX	2020-10-08	0.2765	0.2789	0.274	0.277	3928742	{}	\N	2020-10-12 15:48:22.979417
61919	ONTX	2020-10-09	0.2794	0.2908	0.2745	0.2875	7235745	{}	\N	2020-10-12 15:48:22.986598
61920	ONTX	2020-10-12	0.2823	0.29	0.275	0.2765	2407296	{}	\N	2020-10-12 15:48:22.993245
61921	OPK	2020-09-28	3.2	3.24	3.13	3.18	5993144	{}	\N	2020-10-12 15:48:24.238791
61922	OPK	2020-09-29	3.2434	3.51	3.21	3.43	11263252	{}	\N	2020-10-12 15:48:24.24816
61923	OPK	2020-09-30	3.56	3.74	3.535	3.69	13889820	{}	\N	2020-10-12 15:48:24.25396
61924	OPK	2020-10-01	3.75	3.89	3.7002	3.885	12810849	{}	\N	2020-10-12 15:48:24.260674
61925	OPK	2020-10-02	3.86	4.03	3.82	3.9	11299555	{}	\N	2020-10-12 15:48:24.26692
61926	OPK	2020-10-05	3.96	4.17	3.96	4.16	8112445	{}	\N	2020-10-12 15:48:24.272878
61927	OPK	2020-10-06	4.25	4.51	4.2	4.35	13118738	{}	\N	2020-10-12 15:48:24.279384
61928	OPK	2020-10-07	4.42	4.58	4.33	4.4	11633476	{}	\N	2020-10-12 15:48:24.284695
61929	OPK	2020-10-08	4.61	4.65	4.49	4.57	8329027	{}	\N	2020-10-12 15:48:24.290241
61930	OPK	2020-10-09	4.62	4.72	4.565	4.62	7013570	{}	\N	2020-10-12 15:48:24.296102
61931	OPK	2020-10-12	4.58	4.6	4.15	4.2017	8749199	{}	\N	2020-10-12 15:48:24.301777
61932	OPTN	2020-09-28	3.8	3.87	3.645	3.69	455090	{}	\N	2020-10-12 15:48:25.502381
61933	OPTN	2020-09-29	3.67	3.86	3.53	3.83	486904	{}	\N	2020-10-12 15:48:25.508596
61934	OPTN	2020-09-30	3.84	4.09	3.84	3.89	210298	{}	\N	2020-10-12 15:48:25.51543
61935	OPTN	2020-10-01	3.89	4	3.73	3.79	235291	{}	\N	2020-10-12 15:48:25.522393
61936	OPTN	2020-10-02	3.71	4	3.66	3.97	246284	{}	\N	2020-10-12 15:48:25.529544
61937	OPTN	2020-10-05	3.99	4.03	3.86	4	141500	{}	\N	2020-10-12 15:48:25.537614
61938	OPTN	2020-10-06	4.08	4.13	3.9381	3.96	222960	{}	\N	2020-10-12 15:48:25.544838
61939	OPTN	2020-10-07	4	4.09	3.96	3.995	253353	{}	\N	2020-10-12 15:48:25.552248
61940	OPTN	2020-10-08	4.09	4.2	4	4.02	256185	{}	\N	2020-10-12 15:48:25.559077
61941	OPTN	2020-10-09	4.09	4.11	3.96	4	244998	{}	\N	2020-10-12 15:48:25.56746
61942	OPTN	2020-10-12	4.01	4.02	3.84	3.94	65583	{}	\N	2020-10-12 15:48:25.574461
61943	ORBC	2020-09-28	3.46	3.6	3.46	3.55	203664	{}	\N	2020-10-12 15:48:26.894367
61944	ORBC	2020-09-29	3.55	3.5602	3.405	3.54	348680	{}	\N	2020-10-12 15:48:26.900069
61945	ORBC	2020-09-30	3.54	3.6266	3.39	3.4	491905	{}	\N	2020-10-12 15:48:26.906801
61946	ORBC	2020-10-01	3.4	3.6	3.38	3.59	323060	{}	\N	2020-10-12 15:48:26.912948
61947	ORBC	2020-10-02	3.51	3.65	3.48	3.59	407365	{}	\N	2020-10-12 15:48:26.919185
61948	ORBC	2020-10-05	3.6	3.73	3.56	3.71	331854	{}	\N	2020-10-12 15:48:26.924965
61949	ORBC	2020-10-06	3.75	3.86	3.68	3.68	267323	{}	\N	2020-10-12 15:48:26.931024
61950	ORBC	2020-10-07	3.73	3.795	3.71	3.75	327746	{}	\N	2020-10-12 15:48:26.936751
61951	ORBC	2020-10-08	3.82	3.885	3.76	3.83	209097	{}	\N	2020-10-12 15:48:26.942114
61952	ORBC	2020-10-09	3.86	3.9	3.83	3.85	176815	{}	\N	2020-10-12 15:48:26.949205
61953	ORBC	2020-10-12	3.85	3.88	3.775	3.79	113340	{}	\N	2020-10-12 15:48:26.954118
61954	OSK	2020-09-28	72.94	74.66	72.845	73.17	288746	{}	\N	2020-10-12 15:48:29.653477
61955	OSK	2020-09-29	73.35	73.74	72.36	72.79	463660	{}	\N	2020-10-12 15:48:29.65974
61956	OSK	2020-09-30	73.45	74.71	72.72	73.5	786575	{}	\N	2020-10-12 15:48:29.665007
61957	OSK	2020-10-01	73.91	74.16	72.42	73.25	854505	{}	\N	2020-10-12 15:48:29.669805
61958	OSK	2020-10-02	71.66	75.87	71.66	74.44	878033	{}	\N	2020-10-12 15:48:29.67531
61959	OSK	2020-10-05	75.4	77.595	75.18	77.1	697198	{}	\N	2020-10-12 15:48:29.681627
61960	OSK	2020-10-06	77.72	78.47	75.92	76.16	908745	{}	\N	2020-10-12 15:48:29.687102
61961	OSK	2020-10-07	77.42	78.87	77.16	78.17	621800	{}	\N	2020-10-12 15:48:29.691882
61962	OSK	2020-10-08	78.64	78.65	77.05	78.62	545099	{}	\N	2020-10-12 15:48:29.698377
61963	OSK	2020-10-09	79.44	79.8099	78.1856	78.37	453814	{}	\N	2020-10-12 15:48:29.703611
61964	OSK	2020-10-12	79.17	79.97	78.5	79.83	102453	{}	\N	2020-10-12 15:48:29.708412
61965	OTIC	2020-09-28	3.66	3.78	3.64	3.66	256573	{}	\N	2020-10-12 15:48:30.601087
61966	OTIC	2020-09-29	3.63	3.79	3.59	3.74	206594	{}	\N	2020-10-12 15:48:30.607191
61967	OTIC	2020-09-30	3.74	4.12	3.74	4.05	643138	{}	\N	2020-10-12 15:48:30.614242
61968	OTIC	2020-10-01	3.985	4.35	3.985	4.18	639798	{}	\N	2020-10-12 15:48:30.620739
61969	OTIC	2020-10-02	4.23	4.45	4.06	4.44	338954	{}	\N	2020-10-12 15:48:30.627624
61970	OTIC	2020-10-05	4.44	4.75	4.44	4.69	334713	{}	\N	2020-10-12 15:48:30.635186
61971	OTIC	2020-10-06	4.72	4.73	4.35	4.52	322118	{}	\N	2020-10-12 15:48:30.640437
61972	OTIC	2020-10-07	4.52	4.7	4.4252	4.6	146099	{}	\N	2020-10-12 15:48:30.647175
61973	OTIC	2020-10-08	4.66	4.66	4.19	4.22	263552	{}	\N	2020-10-12 15:48:30.655221
61974	OTIC	2020-10-09	4.11	4.32	4.11	4.23	186988	{}	\N	2020-10-12 15:48:30.663338
61975	OTIC	2020-10-12	4.26	4.36	4.18	4.36	40292	{}	\N	2020-10-12 15:48:30.67053
61976	OUNZ	2020-09-28	18.27	18.355	18.19	18.34	132732	{}	\N	2020-10-12 15:48:31.872301
61977	OUNZ	2020-09-29	18.41	18.52	18.39	18.51	326293	{}	\N	2020-10-12 15:48:31.882264
61978	OUNZ	2020-09-30	18.46	18.5499	18.3592	18.39	273493	{}	\N	2020-10-12 15:48:31.892734
61979	OUNZ	2020-10-01	18.56	18.65	18.49	18.56	342971	{}	\N	2020-10-12 15:48:31.904001
61980	OUNZ	2020-10-02	18.57	18.61	18.501	18.55	357671	{}	\N	2020-10-12 15:48:31.9143
61981	OUNZ	2020-10-05	18.6	18.71	18.585	18.64	334257	{}	\N	2020-10-12 15:48:31.923453
61982	OUNZ	2020-10-06	18.71	18.73	18.41	18.41	319030	{}	\N	2020-10-12 15:48:31.929841
61983	OUNZ	2020-10-07	18.4	18.42	18.335	18.41	241520	{}	\N	2020-10-12 15:48:31.936467
61984	OUNZ	2020-10-08	18.51	18.53	18.36	18.48	167835	{}	\N	2020-10-12 15:48:31.942054
61985	OUNZ	2020-10-09	18.72	18.82	18.69	18.8	179218	{}	\N	2020-10-12 15:48:31.947967
61986	OUNZ	2020-10-12	18.75	18.7799	18.725	18.75	43030	{}	\N	2020-10-12 15:48:31.954376
61987	OXBRW	2020-09-28	0.12	0.56	0.1056	0.1458	10171607	{}	\N	2020-10-12 15:48:33.060853
61988	OXBRW	2020-09-29	0.1378	0.1378	0.101	0.104	774348	{}	\N	2020-10-12 15:48:33.070143
61989	OXBRW	2020-09-30	0.1096	0.115	0.09	0.0975	291816	{}	\N	2020-10-12 15:48:33.078413
61990	OXBRW	2020-10-01	0.0989	0.147	0.0924	0.1103	1045622	{}	\N	2020-10-12 15:48:33.087904
61991	OXBRW	2020-10-02	0.109	0.112349	0.1	0.1054	127184	{}	\N	2020-10-12 15:48:33.097255
61992	OXBRW	2020-10-05	0.1005	0.11	0.1005	0.1087	41823	{}	\N	2020-10-12 15:48:33.106801
61993	OXBRW	2020-10-06	0.1151	0.1294	0.1025	0.111	294507	{}	\N	2020-10-12 15:48:33.114376
61994	OXBRW	2020-10-07	0.1008	0.1192	0.1008	0.1064	101260	{}	\N	2020-10-12 15:48:33.12251
61995	OXBRW	2020-10-08	0.1099	0.19	0.1062	0.1344	2492700	{}	\N	2020-10-12 15:48:33.129381
61996	OXBRW	2020-10-09	0.1279	0.128	0.1113	0.1249	515594	{}	\N	2020-10-12 15:48:33.135683
61997	OXBRW	2020-10-12	0.12	0.125	0.115	0.115001	68997	{}	\N	2020-10-12 15:48:33.141388
61998	OFG	2020-09-28	12.45	12.87	12.45	12.69	162451	{}	\N	2020-10-12 15:48:35.563284
61999	OFG	2020-09-29	12.63	12.63	12.14	12.46	310563	{}	\N	2020-10-12 15:48:35.574148
62000	OFG	2020-09-30	12.46	12.82	12.34	12.46	395330	{}	\N	2020-10-12 15:48:35.583785
62001	OFG	2020-10-01	12.56	12.67	12.24	12.59	373728	{}	\N	2020-10-12 15:48:35.596843
62002	OFG	2020-10-02	12.28	13.07	12.28	12.99	295418	{}	\N	2020-10-12 15:48:35.609223
62003	OFG	2020-10-05	13.25	13.67	13.23	13.65	312420	{}	\N	2020-10-12 15:48:35.616344
62004	OFG	2020-10-06	13.93	14.24	13.47	13.49	479583	{}	\N	2020-10-12 15:48:35.62678
62005	OFG	2020-10-07	13.71	13.95	13.32	13.76	513093	{}	\N	2020-10-12 15:48:35.635614
62006	OFG	2020-10-08	13.98	14.27	13.7	14.22	298619	{}	\N	2020-10-12 15:48:35.644294
62007	OFG	2020-10-09	14.4	14.46	13.9	13.94	407392	{}	\N	2020-10-12 15:48:35.656296
62008	OFG	2020-10-12	13.9	14.07	13.9	13.98	29870	{}	\N	2020-10-12 15:48:35.666537
62009	OIH	2020-09-28	99.86	103.16	99.86	101.16	321922	{}	\N	2020-10-12 15:48:36.985403
62010	OIH	2020-09-29	100.63	100.95	96.4	98.46	405834	{}	\N	2020-10-12 15:48:36.996766
62011	OIH	2020-09-30	98.94	101.33	96.9633	97.71	312875	{}	\N	2020-10-12 15:48:37.008062
62012	OIH	2020-10-01	96.72	96.99	93.43	94.21	517172	{}	\N	2020-10-12 15:48:37.017803
62013	OIH	2020-10-02	90.97	97.81	90.49	96.75	410417	{}	\N	2020-10-12 15:48:37.028448
62014	OIH	2020-10-05	98.39	100.31	97.91	100.13	318820	{}	\N	2020-10-12 15:48:37.038966
62015	OIH	2020-10-06	102.23	103.49	98.2	98.64	463155	{}	\N	2020-10-12 15:48:37.048042
62016	OIH	2020-10-07	98.65	100	97.2	99.63	362030	{}	\N	2020-10-12 15:48:37.05816
62017	OIH	2020-10-08	101.07	104.66	100.7	104.45	656306	{}	\N	2020-10-12 15:48:37.068331
62018	OIH	2020-10-09	105.47	105.9	101.87	102.81	405209	{}	\N	2020-10-12 15:48:37.081177
62019	OIH	2020-10-12	102.26	102.26	100.3	100.87	154880	{}	\N	2020-10-12 15:48:37.093426
62020	OLED	2020-09-28	174.12	175.43	172.01	174.6	278856	{}	\N	2020-10-12 15:48:38.844301
62021	OLED	2020-09-29	173.31	180.13	173.31	178.88	325803	{}	\N	2020-10-12 15:48:38.85773
62022	OLED	2020-09-30	178.99	182.845	178.635	180.74	315060	{}	\N	2020-10-12 15:48:38.868801
62023	OLED	2020-10-01	182.47	185.7	181.35	183.56	223130	{}	\N	2020-10-12 15:48:38.893667
62024	OLED	2020-10-02	178.59	183.902	176.602	179.43	179901	{}	\N	2020-10-12 15:48:38.902561
62025	OLED	2020-10-05	180.805	185.5	180.805	184.75	394964	{}	\N	2020-10-12 15:48:38.911964
62026	OLED	2020-10-06	185	189.79	184.32	186.03	246717	{}	\N	2020-10-12 15:48:38.925615
62027	OLED	2020-10-07	188.46	191.76	187.37	191.16	273869	{}	\N	2020-10-12 15:48:38.935574
62028	OLED	2020-10-08	192.66	198.23	192.66	197.24	334497	{}	\N	2020-10-12 15:48:38.943664
62029	OLED	2020-10-09	198.32	203.97	198.32	202.2	336319	{}	\N	2020-10-12 15:48:38.951359
62030	OLED	2020-10-12	203.72	203.72	198.91	198.985	128957	{}	\N	2020-10-12 15:48:38.959668
62031	OMFL	2020-09-28	30.89	31.28	30.81	31.04	31302	{}	\N	2020-10-12 15:48:39.929689
62032	OMFL	2020-09-29	31.01	31.01	30.52	30.67	24991	{}	\N	2020-10-12 15:48:39.93721
62033	OMFL	2020-09-30	30.86	31.252	30.74	30.92	853290	{}	\N	2020-10-12 15:48:39.945262
62034	OMFL	2020-10-01	30.98	31.19	30.74	31.01	42490	{}	\N	2020-10-12 15:48:39.954806
62035	OMFL	2020-10-02	30.42	31.62	30.37	31.55	37379	{}	\N	2020-10-12 15:48:39.965911
62036	OMFL	2020-10-05	31.88	32.27	31.88	32.26	35557	{}	\N	2020-10-12 15:48:39.977503
62037	OMFL	2020-10-06	32.46	32.87	31.92	31.97	61457	{}	\N	2020-10-12 15:48:39.985824
62038	OMFL	2020-10-07	32.55	32.645	32.3401	32.57	16924	{}	\N	2020-10-12 15:48:39.996206
62039	OMFL	2020-10-08	32.75	33.17	32.7165	33.17	21832	{}	\N	2020-10-12 15:48:40.007782
62040	OMFL	2020-10-09	33.0479	33.2834	32.989	33.03	37555	{}	\N	2020-10-12 15:48:40.019179
62041	OMFL	2020-10-12	33.16	33.17	33.085	33.17	3579	{}	\N	2020-10-12 15:48:40.032529
62042	ON	2020-09-28	20.97	21.75	20.81	21.725	4817827	{}	\N	2020-10-12 15:48:41.529247
62043	ON	2020-09-29	21.71	21.96	21.55	21.78	4510977	{}	\N	2020-10-12 15:48:41.536431
62044	ON	2020-09-30	21.72	22.065	21.615	21.69	3283384	{}	\N	2020-10-12 15:48:41.542998
62045	ON	2020-10-01	22.41	23.37	22.35	22.91	9857604	{}	\N	2020-10-12 15:48:41.549912
62046	ON	2020-10-02	21.95	22.93	21.91	22.44	6954943	{}	\N	2020-10-12 15:48:41.557007
62047	ON	2020-10-05	22.78	23.19	22.71	23.15	4884547	{}	\N	2020-10-12 15:48:41.56724
62048	ON	2020-10-06	23.18	23.49	22.67	22.89	4901980	{}	\N	2020-10-12 15:48:41.575687
62049	ON	2020-10-07	23.37	23.53	23.22	23.35	5809676	{}	\N	2020-10-12 15:48:41.585476
62050	ON	2020-10-08	25.0381	26.06	24.91	26.04	23146862	{}	\N	2020-10-12 15:48:41.59273
62051	ON	2020-10-09	26.1	26.97	25.51	26.35	14484246	{}	\N	2020-10-12 15:48:41.600019
62052	ON	2020-10-12	26.52	26.58	26.245	26.37	2228750	{}	\N	2020-10-12 15:48:41.607108
62053	OOMA	2020-09-28	13.11	13.2	12.96	13.13	88974	{}	\N	2020-10-12 15:48:43.172215
62054	OOMA	2020-09-29	13.13	13.3	13.08	13.25	144100	{}	\N	2020-10-12 15:48:43.184799
62055	OOMA	2020-09-30	13.24	13.39	12.93	13.05	210307	{}	\N	2020-10-12 15:48:43.192607
62056	OOMA	2020-10-01	13.28	13.455	12.98	13.4	350979	{}	\N	2020-10-12 15:48:43.199856
62057	OOMA	2020-10-02	13.14	13.305	12.89	13.06	420348	{}	\N	2020-10-12 15:48:43.206276
62058	OOMA	2020-10-05	13.12	13.32	13.12	13.26	218870	{}	\N	2020-10-12 15:48:43.213617
62059	OOMA	2020-10-06	13.41	13.72	13.35	13.43	367679	{}	\N	2020-10-12 15:48:43.221412
62060	OOMA	2020-10-07	13.6	13.7	13.4617	13.67	316538	{}	\N	2020-10-12 15:48:43.22872
62061	OOMA	2020-10-08	13.76	13.9	13.49	13.53	194065	{}	\N	2020-10-12 15:48:43.23469
62062	OOMA	2020-10-09	13.64	13.83	13.54	13.73	264105	{}	\N	2020-10-12 15:48:43.240561
62063	OOMA	2020-10-12	13.76	13.89	13.67	13.86	157183	{}	\N	2020-10-12 15:48:43.247377
62064	OR	2020-09-28	11.82	11.89	11.58	11.84	650865	{}	\N	2020-10-12 15:48:44.450937
62065	OR	2020-09-29	11.82	12.02	11.78	11.91	1021503	{}	\N	2020-10-12 15:48:44.457034
62066	OR	2020-09-30	11.8	11.9768	11.7	11.83	967266	{}	\N	2020-10-12 15:48:44.464411
62067	OR	2020-10-01	11.96	12.13	11.78	11.97	1028381	{}	\N	2020-10-12 15:48:44.471214
62068	OR	2020-10-02	11.83	11.99	11.7	11.76	1048340	{}	\N	2020-10-12 15:48:44.477288
62069	OR	2020-10-05	11.79	12.155	11.79	11.93	1066050	{}	\N	2020-10-12 15:48:44.484655
62070	OR	2020-10-06	12.3	12.84	12.04	12.05	2694999	{}	\N	2020-10-12 15:48:44.490489
62071	OR	2020-10-07	12.15	12.2	11.765	11.81	2646261	{}	\N	2020-10-12 15:48:44.495367
62072	OR	2020-10-08	11.9	12.015	11.76	11.97	1380694	{}	\N	2020-10-12 15:48:44.501443
62073	OR	2020-10-09	12.295	12.55	12.17	12.55	3593631	{}	\N	2020-10-12 15:48:44.508836
62074	OR	2020-10-12	12.52	12.533	12.28	12.3	597206	{}	\N	2020-10-12 15:48:44.515759
62075	OSG	2020-09-28	2.02	2.13	2.02	2.11	214650	{}	\N	2020-10-12 15:48:46.380824
62076	OSG	2020-09-29	2.19	2.19	2.09	2.11	250000	{}	\N	2020-10-12 15:48:46.391836
62077	OSG	2020-09-30	2.19	2.24	2.13	2.14	828096	{}	\N	2020-10-12 15:48:46.40087
62078	OSG	2020-10-01	2.14	2.14	2.02	2.1	437729	{}	\N	2020-10-12 15:48:46.410123
62079	OSG	2020-10-02	2.08	2.15	2.04	2.14	219789	{}	\N	2020-10-12 15:48:46.416656
62080	OSG	2020-10-05	2.17	2.26	2.14	2.15	400795	{}	\N	2020-10-12 15:48:46.423864
62081	OSG	2020-10-06	2.23	2.23	2.1	2.1	253725	{}	\N	2020-10-12 15:48:46.431983
62082	OSG	2020-10-07	2.12	2.13	2.07	2.11	314527	{}	\N	2020-10-12 15:48:46.440995
62083	OSG	2020-10-08	2.12	2.23	2.11	2.2	261026	{}	\N	2020-10-12 15:48:46.447118
62084	OSG	2020-10-09	2.22	2.24	2.14	2.14	223132	{}	\N	2020-10-12 15:48:46.453503
62085	OSG	2020-10-12	2.13	2.18	2.08	2.11	82542	{}	\N	2020-10-12 15:48:46.459604
62086	OSIS	2020-09-28	77.78	78.64	77.78	78.26	96030	{}	\N	2020-10-12 15:48:47.403077
62087	OSIS	2020-09-29	78.5	78.59	77.035	77.65	132758	{}	\N	2020-10-12 15:48:47.410724
62088	OSIS	2020-09-30	78	78.91	77.45	77.61	89345	{}	\N	2020-10-12 15:48:47.41667
62089	OSIS	2020-10-01	78.25	78.56	76.985	78.08	82901	{}	\N	2020-10-12 15:48:47.425754
62090	OSIS	2020-10-02	76.93	78.4175	76.93	78.04	61995	{}	\N	2020-10-12 15:48:47.435534
62091	OSIS	2020-10-05	78.78	79.22	78.28	79.06	167211	{}	\N	2020-10-12 15:48:47.444369
62092	OSIS	2020-10-06	79.4	80.54	78.86	79.33	87184	{}	\N	2020-10-12 15:48:47.451829
62093	OSIS	2020-10-07	79.78	80.36	79.39	79.78	91050	{}	\N	2020-10-12 15:48:47.458966
62094	OSIS	2020-10-08	80.13	81.083	80.13	80.93	77265	{}	\N	2020-10-12 15:48:47.464803
62095	OSIS	2020-10-09	81.21	81.61	80.14	80.53	123910	{}	\N	2020-10-12 15:48:47.4716
62096	OSIS	2020-10-12	80.67	81.57	80.1775	81.25	9954	{}	\N	2020-10-12 15:48:47.476738
62097	OFLX	2020-09-28	140.9	144.08	140.9	144	6778	{}	\N	2020-10-12 15:48:48.962915
62098	OFLX	2020-09-29	144.47	158.115	144	157.4	17174	{}	\N	2020-10-12 15:48:48.97261
62099	OFLX	2020-09-30	158.4	163	152.9301	156.72	42013	{}	\N	2020-10-12 15:48:48.981388
62100	OFLX	2020-10-01	154.9	162.72	154	160.1	24176	{}	\N	2020-10-12 15:48:48.990789
62101	OFLX	2020-10-02	157.86	162	156.485	160	11729	{}	\N	2020-10-12 15:48:49.000882
62102	OFLX	2020-10-05	160.24	165	158.6736	163.27	14972	{}	\N	2020-10-12 15:48:49.010539
62103	OFLX	2020-10-06	163.71	167.109	161.91	162.89	9465	{}	\N	2020-10-12 15:48:49.018949
62104	OFLX	2020-10-07	160	171.589	160	170.15	8425	{}	\N	2020-10-12 15:48:49.025958
62105	OFLX	2020-10-08	172	173.51	164.83	170.4	5238	{}	\N	2020-10-12 15:48:49.031832
62106	OFLX	2020-10-09	171.03	171.92	169.12	170.01	5232	{}	\N	2020-10-12 15:48:49.038649
62107	OFLX	2020-10-12	172	172.1	169.935	169.935	1398	{}	\N	2020-10-12 15:48:49.045672
62108	OIIM	2020-09-28	3.4	3.41	3.25	3.38	152750	{}	\N	2020-10-12 15:48:50.551397
62109	OIIM	2020-09-29	3.5	3.8494	3.45	3.66	391510	{}	\N	2020-10-12 15:48:50.559529
62110	OIIM	2020-09-30	3.55	3.78	3.45	3.54	108767	{}	\N	2020-10-12 15:48:50.56873
62111	OIIM	2020-10-01	3.6	3.6	3.4102	3.44	66419	{}	\N	2020-10-12 15:48:50.575993
62112	OIIM	2020-10-02	3.29	3.45	3.23	3.395	59595	{}	\N	2020-10-12 15:48:50.584623
62113	OIIM	2020-10-05	3.6	4.101	3.6	4.03	423476	{}	\N	2020-10-12 15:48:50.591551
62114	OIIM	2020-10-06	4.38	4.615	4.07	4.16	538894	{}	\N	2020-10-12 15:48:50.599651
62115	OIIM	2020-10-07	4.2	4.38	4.19	4.32	201189	{}	\N	2020-10-12 15:48:50.608937
62116	OIIM	2020-10-08	4.42	4.98	4.39	4.91	504433	{}	\N	2020-10-12 15:48:50.619447
62117	OIIM	2020-10-09	5.05	5.43	4.68	4.8	868398	{}	\N	2020-10-12 15:48:50.628996
62118	OIIM	2020-10-12	4.87	4.9	4.6	4.72	144892	{}	\N	2020-10-12 15:48:50.640348
62119	OLP	2020-09-28	16.46	17.31	16.46	17.06	56210	{}	\N	2020-10-12 15:48:52.213453
62120	OLP	2020-09-29	17.01	17.07	16.38	16.75	103190	{}	\N	2020-10-12 15:48:52.22246
62121	OLP	2020-09-30	16.9	17.2	16.19	16.36	137838	{}	\N	2020-10-12 15:48:52.235867
62122	OLP	2020-10-01	16.34	17.13	16.34	17.07	137198	{}	\N	2020-10-12 15:48:52.245302
62123	OLP	2020-10-02	16.74	17.45	16.4257	17.36	161116	{}	\N	2020-10-12 15:48:52.253767
62124	OLP	2020-10-05	17.5	17.66	17.1401	17.6	147783	{}	\N	2020-10-12 15:48:52.262048
62125	OLP	2020-10-06	17.72	18.15	17.48	17.56	116542	{}	\N	2020-10-12 15:48:52.274498
62126	OLP	2020-10-07	17.71	17.835	17.46	17.48	136535	{}	\N	2020-10-12 15:48:52.283177
62127	OLP	2020-10-08	17.6	17.81	17.45	17.51	78915	{}	\N	2020-10-12 15:48:52.291239
62128	OLP	2020-10-09	17.8	17.8	16.96	17.1	114189	{}	\N	2020-10-12 15:48:52.297628
62129	OLP	2020-10-12	17.05	17.35	16.77	17.123	60816	{}	\N	2020-10-12 15:48:52.306877
62130	ONEO	2020-09-28	73.03	73.199	72.973	72.973	1665	{}	\N	2020-10-12 15:48:53.880075
62131	ONEO	2020-09-29	72.76	72.949	72.535	72.94	3342	{}	\N	2020-10-12 15:48:53.887978
62132	ONEO	2020-09-30	73.0601	73.0601	73.03	73.03	3565	{}	\N	2020-10-12 15:48:53.896762
62133	ONEO	2020-10-01	73.6	73.6	73.4763	73.4763	487	{}	\N	2020-10-12 15:48:53.905173
62134	ONEO	2020-10-02	72.6	73.7	72.6	73.7	1022	{}	\N	2020-10-12 15:48:53.915022
62135	ONEO	2020-10-05	74.5218	75.1041	74.5218	75.1041	3066	{}	\N	2020-10-12 15:48:53.924057
62136	ONEO	2020-10-06	74.5686	74.5686	74.5686	74.5686	456	{}	\N	2020-10-12 15:48:53.931722
62137	ONEO	2020-10-07	75.919	75.919	75.84	75.84	3391	{}	\N	2020-10-12 15:48:53.937603
62138	ONEO	2020-10-08	76.54	76.6799	76.54	76.6799	10037	{}	\N	2020-10-12 15:48:53.945144
62139	ONEO	2020-10-09	77.25	77.2701	77.011	77.011	3931	{}	\N	2020-10-12 15:48:53.951355
62140	ONEO	2020-10-12	77.62	77.62	77.62	77.62	918	{}	\N	2020-10-12 15:48:53.958901
62141	OPHC	2020-09-28	2.56	3.38	2.42	2.8	642548	{}	\N	2020-10-12 15:48:55.989883
62142	OPHC	2020-09-29	2.6049	3.12	2.5	2.9	223144	{}	\N	2020-10-12 15:48:55.99688
62143	OPHC	2020-09-30	2.87	2.89	2.62	2.72	21206	{}	\N	2020-10-12 15:48:56.001949
62144	OPHC	2020-10-01	2.7	2.7	2.5	2.58	17958	{}	\N	2020-10-12 15:48:56.00742
62145	OPHC	2020-10-02	2.6	2.79	2.6	2.75	48485	{}	\N	2020-10-12 15:48:56.016218
62146	OPHC	2020-10-05	2.76	2.87	2.68	2.85	28863	{}	\N	2020-10-12 15:48:56.024036
62147	OPHC	2020-10-06	2.82	2.9465	2.81	2.84	17186	{}	\N	2020-10-12 15:48:56.033011
62148	OPHC	2020-10-07	2.85	3.14	2.77	2.81	104382	{}	\N	2020-10-12 15:48:56.039377
62149	OPHC	2020-10-08	2.89	3.13	2.8601	2.91	172303	{}	\N	2020-10-12 15:48:56.045383
62150	OPHC	2020-10-09	3.1011	3.1011	2.881	2.91	40667	{}	\N	2020-10-12 15:48:56.051335
62151	OPHC	2020-10-12	2.9	3.05	2.885	2.98	36621	{}	\N	2020-10-12 15:48:56.05717
62152	ORAN	2020-09-28	10.44	10.47	10.4	10.44	525174	{}	\N	2020-10-12 15:48:57.155412
62153	ORAN	2020-09-29	10.35	10.36	10.25	10.33	1108644	{}	\N	2020-10-12 15:48:57.161762
62154	ORAN	2020-09-30	10.41	10.47	10.36	10.38	1089946	{}	\N	2020-10-12 15:48:57.168621
62155	ORAN	2020-10-01	10.22	10.25	10.15	10.24	1198801	{}	\N	2020-10-12 15:48:57.175633
62156	ORAN	2020-10-02	10.22	10.38	10.2	10.38	994393	{}	\N	2020-10-12 15:48:57.182086
62157	ORAN	2020-10-05	10.68	10.8	10.6181	10.78	1017099	{}	\N	2020-10-12 15:48:57.188817
62158	ORAN	2020-10-06	10.93	10.93	10.69	10.7	1250583	{}	\N	2020-10-12 15:48:57.195045
62159	ORAN	2020-10-07	10.65	10.7	10.57	10.66	894709	{}	\N	2020-10-12 15:48:57.200807
62160	ORAN	2020-10-08	10.66	10.87	10.66	10.87	1038270	{}	\N	2020-10-12 15:48:57.2064
62161	ORAN	2020-10-09	11.07	11.29	10.98	11.23	2557332	{}	\N	2020-10-12 15:48:57.213245
62162	ORAN	2020-10-12	11.28	11.32	11.25	11.28	481512	{}	\N	2020-10-12 15:48:57.219558
62163	ORLY	2020-09-28	450.17	458.01	449.94	457.34	455672	{}	\N	2020-10-12 15:48:58.164815
62164	ORLY	2020-09-29	457.36	459.03	452.7	453.32	340615	{}	\N	2020-10-12 15:48:58.174475
62165	ORLY	2020-09-30	455.68	464.61	453.58	461.08	473525	{}	\N	2020-10-12 15:48:58.182796
62166	ORLY	2020-10-01	464.02	467.63	461.11	463.56	350784	{}	\N	2020-10-12 15:48:58.194741
62167	ORLY	2020-10-02	458.25	462	453.345	458.24	320764	{}	\N	2020-10-12 15:48:58.204174
62168	ORLY	2020-10-05	459.77	463.58	459.06	459.49	344149	{}	\N	2020-10-12 15:48:58.211373
62169	ORLY	2020-10-06	459.62	461.23	443.28	444.97	448347	{}	\N	2020-10-12 15:48:58.218337
62170	ORLY	2020-10-07	447.04	451.615	444.325	444.67	476250	{}	\N	2020-10-12 15:48:58.22517
62171	ORLY	2020-10-08	447.21	455.69	447.21	453.36	375950	{}	\N	2020-10-12 15:48:58.230978
62172	ORLY	2020-10-09	454.59	455.145	448.69	449.39	357759	{}	\N	2020-10-12 15:48:58.238717
62173	ORLY	2020-10-12	452.71	457.87	449.33	457.28	181550	{}	\N	2020-10-12 15:48:58.244932
62174	OSS	2020-09-28	2.18	2.25	2.11	2.21	140072	{}	\N	2020-10-12 15:48:59.164221
62175	OSS	2020-09-29	2.18	2.2285	2.08	2.13	121308	{}	\N	2020-10-12 15:48:59.171592
62176	OSS	2020-09-30	2.11	2.18	2.08	2.1	98649	{}	\N	2020-10-12 15:48:59.181521
62177	OSS	2020-10-01	2.11	2.15	2.08	2.14	73976	{}	\N	2020-10-12 15:48:59.192583
62178	OSS	2020-10-02	2.14	2.2	2.0915	2.13	98293	{}	\N	2020-10-12 15:48:59.200361
62179	OSS	2020-10-05	2.15	2.2	2.13	2.18	83928	{}	\N	2020-10-12 15:48:59.208793
62180	OSS	2020-10-06	2.18	2.3	2.18	2.2	161630	{}	\N	2020-10-12 15:48:59.214842
62181	OSS	2020-10-07	2.2201	2.26	2.18	2.18	51787	{}	\N	2020-10-12 15:48:59.221748
62182	OSS	2020-10-08	2.22	2.28	2.2	2.27	53086	{}	\N	2020-10-12 15:48:59.227152
62183	OSS	2020-10-09	2.24	2.35	2.23	2.32	92368	{}	\N	2020-10-12 15:48:59.233841
62184	OSS	2020-10-12	2.32	2.34	2.3	2.31	20555	{}	\N	2020-10-12 15:48:59.240984
62185	OSTK	2020-09-28	74.16	76.6899	72.95	74.62	3927364	{}	\N	2020-10-12 15:49:00.287308
62186	OSTK	2020-09-29	74.94	75.68	72.77	74.19	2211194	{}	\N	2020-10-12 15:49:00.293467
62187	OSTK	2020-09-30	74.22	75.46	71.83	72.65	3960029	{}	\N	2020-10-12 15:49:00.29794
62188	OSTK	2020-10-01	75.09	83.99	74.4	83.87	12042646	{}	\N	2020-10-12 15:49:00.304037
62189	OSTK	2020-10-02	78.94	81.99	76.8	77.93	7146337	{}	\N	2020-10-12 15:49:00.312084
62190	OSTK	2020-10-05	79.46	81.1	77.23	80.76	3842978	{}	\N	2020-10-12 15:49:00.322183
62191	OSTK	2020-10-06	81	83.5	77.1	78.21	4283613	{}	\N	2020-10-12 15:49:00.329769
62192	OSTK	2020-10-07	79.7	84.9	79.55	84.65	4305735	{}	\N	2020-10-12 15:49:00.336775
62193	OSTK	2020-10-08	85.67	87.76	81.03	82.23	4250294	{}	\N	2020-10-12 15:49:00.343031
62194	OSTK	2020-10-09	83.1	85.5	81.85	83.23	2664827	{}	\N	2020-10-12 15:49:00.34915
62195	OSTK	2020-10-12	84.26	84.7997	80.71	82.48	1501989	{}	\N	2020-10-12 15:49:00.358327
62196	OUSA	2020-09-28	36.06	36.314	36.06	36.13	91758	{}	\N	2020-10-12 15:49:01.471079
62197	OUSA	2020-09-29	36.15	36.19	35.87	35.96	33337	{}	\N	2020-10-12 15:49:01.478934
62198	OUSA	2020-09-30	36.08	36.5	36.05	36.26	45956	{}	\N	2020-10-12 15:49:01.484842
62199	OUSA	2020-10-01	36.46	36.46	36.05	36.25	95259	{}	\N	2020-10-12 15:49:01.490524
62200	OUSA	2020-10-02	35.75	36.1964	35.683	35.97	82990	{}	\N	2020-10-12 15:49:01.49594
62201	OUSA	2020-10-05	36.2089	36.49	36.12	36.48	281286	{}	\N	2020-10-12 15:49:01.501936
62202	OUSA	2020-10-06	36.58	36.68	36.015	36.09	157671	{}	\N	2020-10-12 15:49:01.509182
62203	OUSA	2020-10-07	36.38	36.7	36.34	36.655	147744	{}	\N	2020-10-12 15:49:01.515206
62204	OUSA	2020-10-08	36.8	36.88	36.67	36.76	49102	{}	\N	2020-10-12 15:49:01.521751
62205	OUSA	2020-10-09	36.97	37.15	36.915	37.04	58808	{}	\N	2020-10-12 15:49:01.527612
62206	OUSA	2020-10-12	37.25	37.44	37.2085	37.4301	37239	{}	\N	2020-10-12 15:49:01.532848
62207	OCX	2020-09-28	1.32	1.39	1.31	1.37	315354	{}	\N	2020-10-12 15:49:02.501364
62208	OCX	2020-09-29	1.38	1.44	1.35	1.42	265222	{}	\N	2020-10-12 15:49:02.507343
62209	OCX	2020-09-30	1.4	1.45	1.32	1.39	314338	{}	\N	2020-10-12 15:49:02.513038
62210	OCX	2020-10-01	1.55	1.56	1.34	1.38	1869034	{}	\N	2020-10-12 15:49:02.519224
62211	OCX	2020-10-02	1.33	1.39	1.32	1.38	318113	{}	\N	2020-10-12 15:49:02.528059
62212	OCX	2020-10-05	1.37	1.46	1.37	1.46	369333	{}	\N	2020-10-12 15:49:02.535014
62213	OCX	2020-10-06	1.46	1.5	1.4	1.4	350519	{}	\N	2020-10-12 15:49:02.540916
62214	OCX	2020-10-07	1.4	1.47	1.38	1.45	301189	{}	\N	2020-10-12 15:49:02.546997
62215	OCX	2020-10-08	1.47	1.54	1.449	1.49	320177	{}	\N	2020-10-12 15:49:02.554447
62216	OCX	2020-10-09	1.52	1.54	1.4601	1.49	243464	{}	\N	2020-10-12 15:49:02.561233
62217	OCX	2020-10-12	1.49	1.5227	1.44	1.47	71069	{}	\N	2020-10-12 15:49:02.567591
62218	OII	2020-09-28	3.66	3.82	3.65	3.68	1453521	{}	\N	2020-10-12 15:49:04.844022
62219	OII	2020-09-29	3.67	3.69	3.335	3.53	3199756	{}	\N	2020-10-12 15:49:04.849958
62220	OII	2020-09-30	3.59	3.75	3.51	3.52	3340922	{}	\N	2020-10-12 15:49:04.85843
62221	OII	2020-10-01	3.51	3.5599	3.36	3.48	2805885	{}	\N	2020-10-12 15:49:04.864267
62222	OII	2020-10-02	3.39	3.735	3.3058	3.65	3188976	{}	\N	2020-10-12 15:49:04.874255
62223	OII	2020-10-05	3.79	3.89	3.675	3.75	2547634	{}	\N	2020-10-12 15:49:04.881347
62224	OII	2020-10-06	3.85	3.96	3.62	3.66	2563922	{}	\N	2020-10-12 15:49:04.889844
62225	OII	2020-10-07	3.68	3.795	3.56	3.75	2682381	{}	\N	2020-10-12 15:49:04.896063
62226	OII	2020-10-08	3.8	4.06	3.7542	4.05	2227525	{}	\N	2020-10-12 15:49:04.906042
62227	OII	2020-10-09	4.1	4.19	3.97	4.03	1650500	{}	\N	2020-10-12 15:49:04.916013
62228	OII	2020-10-12	4.03	4.03	3.88	3.99	425957	{}	\N	2020-10-12 15:49:04.926105
62229	OMAB	2020-09-28	35.94	36.715	34.975	36.59	47183	{}	\N	2020-10-12 15:49:06.352083
62230	OMAB	2020-09-29	36.69	37.12	36.32	36.4	43578	{}	\N	2020-10-12 15:49:06.361556
62231	OMAB	2020-09-30	36.31	37.43	36.31	36.76	56647	{}	\N	2020-10-12 15:49:06.367798
62232	OMAB	2020-10-01	36.42	36.91	35.87	36.21	67331	{}	\N	2020-10-12 15:49:06.38756
62233	OMAB	2020-10-02	35.99	37.05	35.82	36.8	41829	{}	\N	2020-10-12 15:49:06.394522
62234	OMAB	2020-10-05	37.1	37.8	37.005	37.42	50748	{}	\N	2020-10-12 15:49:06.402889
62235	OMAB	2020-10-06	37.5	38.19	36.7	36.93	49186	{}	\N	2020-10-12 15:49:06.4111
62236	OMAB	2020-10-07	37.04	37.9	37.04	37.51	46598	{}	\N	2020-10-12 15:49:06.416276
62237	OMAB	2020-10-08	37.92	38.57	37.605	38.08	34766	{}	\N	2020-10-12 15:49:06.422046
62238	OMAB	2020-10-09	38.09	38.5	37.615	38.12	43826	{}	\N	2020-10-12 15:49:06.427556
62239	OMAB	2020-10-12	38.39	38.54	38.0408	38.5	19911	{}	\N	2020-10-12 15:49:06.432535
62240	ORN	2020-09-28	2.77	2.84	2.67	2.71	150106	{}	\N	2020-10-12 15:49:08.710774
62241	ORN	2020-09-29	2.7	2.73	2.61	2.68	130951	{}	\N	2020-10-12 15:49:08.746031
62242	ORN	2020-09-30	2.7	2.8282	2.7	2.75	346273	{}	\N	2020-10-12 15:49:08.779459
62243	ORN	2020-10-01	2.76	2.79	2.71	2.75	225387	{}	\N	2020-10-12 15:49:08.801146
62244	ORN	2020-10-02	2.65	2.88	2.65	2.75	136558	{}	\N	2020-10-12 15:49:08.813958
62245	ORN	2020-10-05	2.76	2.95	2.7401	2.91	222272	{}	\N	2020-10-12 15:49:08.824115
62246	ORN	2020-10-06	2.95	3.1037	2.9246	2.98	391370	{}	\N	2020-10-12 15:49:08.835468
62247	ORN	2020-10-07	3.04	3.2	2.9801	3.12	473643	{}	\N	2020-10-12 15:49:08.844187
62248	ORN	2020-10-08	3.14	3.26	3.1	3.21	520819	{}	\N	2020-10-12 15:49:08.852975
62249	ORN	2020-10-09	3.23	3.28	3.19	3.19	235545	{}	\N	2020-10-12 15:49:08.861562
62250	ORN	2020-10-12	3.24	3.24	3.06	3.08	102254	{}	\N	2020-10-12 15:49:08.872194
62251	OSUR	2020-09-28	11.28	11.71	11.25	11.445	1121599	{}	\N	2020-10-12 15:49:09.873338
62252	OSUR	2020-09-29	11.55	12.47	11.54	12.33	1423746	{}	\N	2020-10-12 15:49:09.886344
62253	OSUR	2020-09-30	12.25	12.4	11.95	12.165	1078462	{}	\N	2020-10-12 15:49:09.897407
62254	OSUR	2020-10-01	12.28	12.59	12.01	12.38	641496	{}	\N	2020-10-12 15:49:09.907888
62255	OSUR	2020-10-02	12.21	12.85	12.11	12.8	696782	{}	\N	2020-10-12 15:49:09.917511
62256	OSUR	2020-10-05	13.19	14.34	12.98	13.825	1883945	{}	\N	2020-10-12 15:49:09.925357
62257	OSUR	2020-10-06	14.28	14.4978	13.94	14.13	1731747	{}	\N	2020-10-12 15:49:09.932797
62258	OSUR	2020-10-07	14.39	14.72	14.24	14.48	1365436	{}	\N	2020-10-12 15:49:09.938987
62259	OSUR	2020-10-08	14.61	14.8895	14.39	14.8	1821441	{}	\N	2020-10-12 15:49:09.946729
62260	OSUR	2020-10-09	14.99	15.38	14.81	15.38	1715235	{}	\N	2020-10-12 15:49:09.952902
62261	OSUR	2020-10-12	15.4	15.45	15	15.22	309191	{}	\N	2020-10-12 15:49:09.961243
62262	OTEL	2020-09-28	11.45	11.48	11.35	11.48	16033	{}	\N	2020-10-12 15:49:12.432041
62263	OTEL	2020-09-29	11.44	11.45	11.44	11.4477	3341	{}	\N	2020-10-12 15:49:12.442211
62264	OTEL	2020-09-30	11.44	11.5017	11.44	11.46	3202	{}	\N	2020-10-12 15:49:12.451792
62265	OTEL	2020-10-01	11.46	11.46	11.44	11.44	5805	{}	\N	2020-10-12 15:49:12.462297
62266	OTEL	2020-10-02	11.45	11.45	11.4	11.4	14634	{}	\N	2020-10-12 15:49:12.470881
62267	OTEL	2020-10-05	11.41	11.43	11.41	11.41	1110	{}	\N	2020-10-12 15:49:12.480253
62268	OTEL	2020-10-06	11.42	11.46	11.41	11.41	3142	{}	\N	2020-10-12 15:49:12.490882
62269	OTEL	2020-10-07	11.46	11.46	11.46	11.46	139	{}	\N	2020-10-12 15:49:12.502543
62270	OTEL	2020-10-09	11.45	11.45	11.42	11.42	34740	{}	\N	2020-10-12 15:49:12.510399
62271	OTEL	2020-10-12	11.4407	11.4407	11.4407	11.4407	115	{}	\N	2020-10-12 15:49:12.518281
62272	OVBC	2020-09-28	21	21.5	21	21.4	2396	{}	\N	2020-10-12 15:49:13.688061
62273	OVBC	2020-09-29	21.25	21.25	21.25	21.25	400	{}	\N	2020-10-12 15:49:13.701096
62274	OVBC	2020-09-30	21.22	21.22	20.51	20.66	2290	{}	\N	2020-10-12 15:49:13.71513
62275	OVBC	2020-10-01	21.25	21.86	21.16	21.16	5544	{}	\N	2020-10-12 15:49:13.726218
62276	OVBC	2020-10-02	21.13	21.42	21.13	21.42	1639	{}	\N	2020-10-12 15:49:13.737349
62277	OVBC	2020-10-05	21.65	22.1	21.65	22.1	2034	{}	\N	2020-10-12 15:49:13.749178
62278	OVBC	2020-10-06	22.04	22.25	21.72	21.72	2251	{}	\N	2020-10-12 15:49:13.76035
62279	OVBC	2020-10-07	21.2579	22.48	21.2579	22.48	2495	{}	\N	2020-10-12 15:49:13.773791
62280	OVBC	2020-10-08	22.5	22.5	22	22.05	5548	{}	\N	2020-10-12 15:49:13.787802
62281	OVBC	2020-10-09	22.05	22.45	22.05	22.45	1974	{}	\N	2020-10-12 15:49:13.802084
62282	OVBC	2020-10-12	22.4	23.63	22.3	23.49	5288	{}	\N	2020-10-12 15:49:13.814984
62283	OUT	2020-09-28	15.31	15.45	14.82	15.01	1328569	{}	\N	2020-10-12 15:49:14.90177
62284	OUT	2020-09-29	14.95	14.95	14.2	14.59	3063765	{}	\N	2020-10-12 15:49:14.915814
62285	OUT	2020-09-30	14.63	15.08	14.41	14.55	4859220	{}	\N	2020-10-12 15:49:14.929218
62286	OUT	2020-10-01	14.74	15.33	14.74	15.22	5140360	{}	\N	2020-10-12 15:49:14.941771
62287	OUT	2020-10-02	14.72	15.49	14.72	15.35	1710863	{}	\N	2020-10-12 15:49:14.95923
62288	OUT	2020-10-05	15.58	15.64	15.21	15.37	2301048	{}	\N	2020-10-12 15:49:14.980138
62289	OUT	2020-10-06	15.49	15.74	15.1	15.16	2157612	{}	\N	2020-10-12 15:49:14.996851
62290	OUT	2020-10-07	15.33	15.39	14.84	14.86	2774260	{}	\N	2020-10-12 15:49:15.024214
62291	OUT	2020-10-08	15.03	15.16	14.76	15.15	1526663	{}	\N	2020-10-12 15:49:15.03392
62292	OUT	2020-10-09	15.34	15.35	14.94	15.12	1762052	{}	\N	2020-10-12 15:49:15.043393
62293	OUT	2020-10-12	15.16	15.37	15.03	15.11	825455	{}	\N	2020-10-12 15:49:15.052057
62294	OXLCM	2020-09-28	23.85	24.19	23.75	24	4054	{}	\N	2020-10-12 15:49:16.547057
62295	OXLCM	2020-09-29	24	24.1	24	24	1812	{}	\N	2020-10-12 15:49:16.555098
62296	OXLCM	2020-09-30	24	24.01	24	24.01	462	{}	\N	2020-10-12 15:49:16.561895
62297	OXLCM	2020-10-01	24	24.2	23.86	23.9	25800	{}	\N	2020-10-12 15:49:16.571997
62298	OXLCM	2020-10-02	23.89	24	23.89	24	2298	{}	\N	2020-10-12 15:49:16.579715
62299	OXLCM	2020-10-05	24.09	24.17	24.08	24.08	1602	{}	\N	2020-10-12 15:49:16.589759
62300	OXLCM	2020-10-06	24	24.2	24	24.2	2110	{}	\N	2020-10-12 15:49:16.598253
62301	OXLCM	2020-10-07	24.18	24.2	24.005	24.05	9983	{}	\N	2020-10-12 15:49:16.608989
62302	OXLCM	2020-10-08	24.11	24.17	24.0101	24.1104	2050	{}	\N	2020-10-12 15:49:16.6172
62303	OXLCM	2020-10-09	24.12	24.16	24	24.0297	107846	{}	\N	2020-10-12 15:49:16.625242
62304	OXLCM	2020-10-12	24.2	24.2	24.2	24.2	109	{}	\N	2020-10-12 15:49:16.634342
62305	OXLCO	2020-09-28	25.04	25.04	24.9	24.99	4076	{}	\N	2020-10-12 15:49:17.715487
62306	OXLCO	2020-09-29	25	25.06	24.955	25	2180	{}	\N	2020-10-12 15:49:17.726081
62307	OXLCO	2020-09-30	25.05	25.05	25	25.03	3668	{}	\N	2020-10-12 15:49:17.734089
62308	OXLCO	2020-10-01	25.06	25.07	24.96	25.05	3807	{}	\N	2020-10-12 15:49:17.744247
62309	OXLCO	2020-10-02	24.9	25.01	24.9	24.95	3518	{}	\N	2020-10-12 15:49:17.754184
62310	OXLCO	2020-10-05	24.95	25	24.95	25	2370	{}	\N	2020-10-12 15:49:17.761581
62311	OXLCO	2020-10-06	24.99	25.03	24.979	25	3746	{}	\N	2020-10-12 15:49:17.77202
62312	OXLCO	2020-10-07	25	25.06	25	25.04	1910	{}	\N	2020-10-12 15:49:17.78026
62313	OXLCO	2020-10-08	25.01	25.019	24.99	25.01	20529	{}	\N	2020-10-12 15:49:17.789748
62314	OXLCO	2020-10-09	25	25.0585	25	25.045	6037	{}	\N	2020-10-12 15:49:17.798474
62315	OXLCO	2020-10-12	25.05	25.0599	25.04	25.0599	1383	{}	\N	2020-10-12 15:49:17.807873
62316	OGEN	2020-09-28	0.5488	0.6	0.53	0.5563	4243746	{}	\N	2020-10-12 15:49:19.038556
62317	OGEN	2020-09-29	0.5505	0.615	0.5489	0.5807	1599725	{}	\N	2020-10-12 15:49:19.046739
62318	OGEN	2020-09-30	0.579	0.6098	0.57	0.575	796297	{}	\N	2020-10-12 15:49:19.053248
62319	OGEN	2020-10-01	0.6	0.6037	0.5626	0.57	692347	{}	\N	2020-10-12 15:49:19.062624
62320	OGEN	2020-10-02	0.55	0.62	0.55	0.599	2485708	{}	\N	2020-10-12 15:49:19.069977
62321	OGEN	2020-10-05	0.5871	0.62	0.5856	0.5978	910778	{}	\N	2020-10-12 15:49:19.078904
62322	OGEN	2020-10-06	0.7792	0.7792	0.5927	0.5992	18506458	{}	\N	2020-10-12 15:49:19.088249
62323	OGEN	2020-10-07	0.6172	0.619	0.581	0.5846	3472387	{}	\N	2020-10-12 15:49:19.096778
62324	OGEN	2020-10-08	0.58	0.584999	0.55	0.56	5000304	{}	\N	2020-10-12 15:49:19.104806
62325	OGEN	2020-10-09	0.56	0.57	0.542	0.5509	2595370	{}	\N	2020-10-12 15:49:19.115222
62326	OGEN	2020-10-12	0.562	0.562	0.540701	0.5513	805416	{}	\N	2020-10-12 15:49:19.123328
62327	OMC	2020-09-28	49.92	50.29	49.41	49.64	1672615	{}	\N	2020-10-12 15:49:20.598833
62328	OMC	2020-09-29	49.53	49.93	48.53	48.85	2876008	{}	\N	2020-10-12 15:49:20.605718
62329	OMC	2020-09-30	49.18	49.98	48.925	49.5	3561092	{}	\N	2020-10-12 15:49:20.610857
62330	OMC	2020-10-01	49.65	49.91	49.04	49.57	3112948	{}	\N	2020-10-12 15:49:20.616495
62331	OMC	2020-10-02	48.97	51.6	48.81	51.33	3210320	{}	\N	2020-10-12 15:49:20.622857
62332	OMC	2020-10-05	51.75	52.445	51.14	51.7	3404464	{}	\N	2020-10-12 15:49:20.629808
62333	OMC	2020-10-06	52.18	52.75	51.04	51.22	3337509	{}	\N	2020-10-12 15:49:20.635811
62334	OMC	2020-10-07	51.71	52.51	51.49	52.15	2965113	{}	\N	2020-10-12 15:49:20.642603
62335	OMC	2020-10-08	52.73	53.205	52.49	53.15	3851212	{}	\N	2020-10-12 15:49:20.648516
62336	OMC	2020-10-09	53.52	53.75	52.38	52.47	4178550	{}	\N	2020-10-12 15:49:20.654847
62337	OMC	2020-10-12	53.21	55	52.83	53.265	1626762	{}	\N	2020-10-12 15:49:20.660844
62338	ONB	2020-09-28	12.57	13.18	12.535	12.67	423572	{}	\N	2020-10-12 15:49:21.906538
62339	ONB	2020-09-29	12.61	12.64	12.25	12.46	272853	{}	\N	2020-10-12 15:49:21.915941
62340	ONB	2020-09-30	12.55	12.88	12.425	12.56	766023	{}	\N	2020-10-12 15:49:21.924391
62341	ONB	2020-10-01	12.47	12.66	12.32	12.6	655671	{}	\N	2020-10-12 15:49:21.930403
62342	ONB	2020-10-02	12.41	13	12.41	12.92	427089	{}	\N	2020-10-12 15:49:21.936762
62343	ONB	2020-10-05	13.06	13.37	12.81	13.32	453684	{}	\N	2020-10-12 15:49:21.942753
62344	ONB	2020-10-06	13.53	13.92	13.32	13.365	804877	{}	\N	2020-10-12 15:49:21.947818
62345	ONB	2020-10-07	13.58	13.74	13.465	13.61	739856	{}	\N	2020-10-12 15:49:21.95371
62346	ONB	2020-10-08	13.78	13.95	13.65	13.9	423985	{}	\N	2020-10-12 15:49:21.960136
62347	ONB	2020-10-09	13.93	14	13.68	13.71	366830	{}	\N	2020-10-12 15:49:21.965694
62348	ONB	2020-10-12	13.68	13.93	13.68	13.92	93664	{}	\N	2020-10-12 15:49:21.971003
62349	ONVO	2020-09-28	6.6	7.32	6.595	6.85	124750	{}	\N	2020-10-12 15:49:23.222386
62350	ONVO	2020-09-29	6.92	7.49	6.9	7.32	126923	{}	\N	2020-10-12 15:49:23.228435
62351	ONVO	2020-09-30	7.68	8.6631	7.45	7.91	662058	{}	\N	2020-10-12 15:49:23.232969
62352	ONVO	2020-10-01	7.9	8.4665	7.1	7.26	200537	{}	\N	2020-10-12 15:49:23.237641
62353	ONVO	2020-10-02	6.8601	7.33	6.86	7.19	63007	{}	\N	2020-10-12 15:49:23.242765
62354	ONVO	2020-10-05	7.24	7.65	7.24	7.44	75502	{}	\N	2020-10-12 15:49:23.249841
62355	ONVO	2020-10-06	7.45	7.77	7.24	7.51	85649	{}	\N	2020-10-12 15:49:23.259524
62356	ONVO	2020-10-07	7.67	8.5999	7.6001	8.3	102632	{}	\N	2020-10-12 15:49:23.263764
62357	ONVO	2020-10-08	8.51	9.42	8.51	8.73	157355	{}	\N	2020-10-12 15:49:23.268442
62358	ONVO	2020-10-09	8.89	9.2799	8.85	9.05	100955	{}	\N	2020-10-12 15:49:23.273467
62359	ONVO	2020-10-12	9.25	11.66	9.11	10.75	196508	{}	\N	2020-10-12 15:49:23.278063
62360	OPNT	2020-09-28	7.88	8.04	7.88	7.95	13021	{}	\N	2020-10-12 15:49:24.610608
62361	OPNT	2020-09-29	7.88	7.9488	7.56	7.6	16896	{}	\N	2020-10-12 15:49:24.616523
62362	OPNT	2020-09-30	7.62	7.75	7.4543	7.48	23365	{}	\N	2020-10-12 15:49:24.621416
62363	OPNT	2020-10-01	7.5	7.7	7.465	7.62	14402	{}	\N	2020-10-12 15:49:24.626853
62364	OPNT	2020-10-02	7.6	7.76	7.48	7.69	13919	{}	\N	2020-10-12 15:49:24.633264
62365	OPNT	2020-10-05	7.8	7.8023	7.45	7.61	10981	{}	\N	2020-10-12 15:49:24.639117
62366	OPNT	2020-10-06	7.61	7.76	7.4	7.42	20497	{}	\N	2020-10-12 15:49:24.645004
62367	OPNT	2020-10-07	7.59	8.14	7.42	7.88	20006	{}	\N	2020-10-12 15:49:24.649598
62368	OPNT	2020-10-08	7.91	8	7.56	7.7	17204	{}	\N	2020-10-12 15:49:24.654091
62369	OPNT	2020-10-09	7.7	7.915	7.6032	7.78	8643	{}	\N	2020-10-12 15:49:24.65921
62370	OPNT	2020-10-12	7.78	7.9	7.67	7.77	6087	{}	\N	2020-10-12 15:49:24.664894
62371	ORA	2020-09-28	57.96	58.705	57.42	58.36	270473	{}	\N	2020-10-12 15:49:25.600959
62372	ORA	2020-09-29	58.92	59.92	58.55	58.96	794771	{}	\N	2020-10-12 15:49:25.607092
62373	ORA	2020-09-30	58.64	59.54	57.4	59.11	602849	{}	\N	2020-10-12 15:49:25.612266
62374	ORA	2020-10-01	59.08	60.36	58	59.8	744620	{}	\N	2020-10-12 15:49:25.622861
62375	ORA	2020-10-02	59.03	60.82	58.78	60.63	578178	{}	\N	2020-10-12 15:49:25.629031
62376	ORA	2020-10-05	61.1	62.17	61.1	62	988783	{}	\N	2020-10-12 15:49:25.634161
62377	ORA	2020-10-06	62.26	65.28	62.26	64.13	1126403	{}	\N	2020-10-12 15:49:25.639207
62378	ORA	2020-10-07	64.91	69.91	64.91	68.78	1523180	{}	\N	2020-10-12 15:49:25.644783
62379	ORA	2020-10-08	69.12	71.305	69.11	70.53	1383071	{}	\N	2020-10-12 15:49:25.64961
62380	ORA	2020-10-09	70.51	73.93	70.43	72.47	1503020	{}	\N	2020-10-12 15:49:25.654556
62381	ORA	2020-10-12	71.24	74.94	71	74.89	716217	{}	\N	2020-10-12 15:49:25.65969
62382	ORI	2020-09-28	14.47	15.01	14.42	14.94	3057055	{}	\N	2020-10-12 15:49:27.738919
62383	ORI	2020-09-29	14.88	14.88	14.4	14.66	3464252	{}	\N	2020-10-12 15:49:27.746652
62384	ORI	2020-09-30	14.73	14.855	14.585	14.74	4963600	{}	\N	2020-10-12 15:49:27.751671
62385	ORI	2020-10-01	14.76	14.9	14.65	14.81	3227680	{}	\N	2020-10-12 15:49:27.756777
62386	ORI	2020-10-02	14.58	15.165	14.55	15.04	4894342	{}	\N	2020-10-12 15:49:27.764429
62387	ORI	2020-10-05	15.29	15.53	15.23	15.48	4773217	{}	\N	2020-10-12 15:49:27.770218
62388	ORI	2020-10-06	15.57	15.73	15.32	15.4	4586271	{}	\N	2020-10-12 15:49:27.775961
62389	ORI	2020-10-07	15.54	15.69	15.365	15.48	2683301	{}	\N	2020-10-12 15:49:27.783202
62390	ORI	2020-10-08	15.61	15.64	15.32	15.46	2422003	{}	\N	2020-10-12 15:49:27.789313
62391	ORI	2020-10-09	15.58	15.71	15.41	15.62	3583934	{}	\N	2020-10-12 15:49:27.794895
62392	ORI	2020-10-12	15.64	16.06	15.58	16.04	1345327	{}	\N	2020-10-12 15:49:27.799781
62393	ORRF	2020-09-28	12.89	13.32	12.89	12.9	13168	{}	\N	2020-10-12 15:49:28.902912
62394	ORRF	2020-09-29	12.76	12.88	12.76	12.85	9077	{}	\N	2020-10-12 15:49:28.909754
62395	ORRF	2020-09-30	13.0359	13.0359	12.74	12.8	20933	{}	\N	2020-10-12 15:49:28.918131
62396	ORRF	2020-10-01	12.84	13.14	12.84	12.9	13951	{}	\N	2020-10-12 15:49:28.925514
62397	ORRF	2020-10-02	12.9	12.99	12.825	12.95	21240	{}	\N	2020-10-12 15:49:28.933822
62398	ORRF	2020-10-05	12.96	13.7	12.96	13.54	21844	{}	\N	2020-10-12 15:49:28.940202
62399	ORRF	2020-10-06	13.45	13.5805	13.25	13.25	16137	{}	\N	2020-10-12 15:49:28.945766
62400	ORRF	2020-10-07	13.3	13.655	13.3	13.63	21984	{}	\N	2020-10-12 15:49:28.953816
62401	ORRF	2020-10-08	13.71	13.945	13.675	13.87	14594	{}	\N	2020-10-12 15:49:28.960805
62402	ORRF	2020-10-09	13.94	13.94	13.4565	13.65	17538	{}	\N	2020-10-12 15:49:28.973045
62403	ORRF	2020-10-12	13.35	13.49	13.35	13.49	519	{}	\N	2020-10-12 15:49:28.98238
62404	OTEX	2020-09-28	42.42	42.755	42.25	42.4	351382	{}	\N	2020-10-12 15:49:30.450411
62405	OTEX	2020-09-29	42.46	42.73	42.14	42.2	388392	{}	\N	2020-10-12 15:49:30.456436
62406	OTEX	2020-09-30	42.36	42.66	42.09	42.23	468294	{}	\N	2020-10-12 15:49:30.462997
62407	OTEX	2020-10-01	42.63	42.91	42.45	42.72	224893	{}	\N	2020-10-12 15:49:30.472188
62408	OTEX	2020-10-02	42.13	42.46	41.62	41.98	285246	{}	\N	2020-10-12 15:49:30.480981
62409	OTEX	2020-10-05	42.24	42.51	42.11	42.36	230662	{}	\N	2020-10-12 15:49:30.488655
62410	OTEX	2020-10-06	42.36	42.36	41.37	41.49	339063	{}	\N	2020-10-12 15:49:30.493892
62411	OTEX	2020-10-07	41.66	42.04	41.56	41.78	225422	{}	\N	2020-10-12 15:49:30.499552
62412	OTEX	2020-10-08	42	42.07	41.57	41.82	240336	{}	\N	2020-10-12 15:49:30.504246
62413	OTEX	2020-10-09	42.1	42.4	42.0001	42.31	205232	{}	\N	2020-10-12 15:49:30.509055
62414	OTEX	2020-10-12	42.5	42.77	42.36	42.72	65774	{}	\N	2020-10-12 15:49:30.514887
62415	OVID	2020-09-28	5.9	5.925	5.63	5.89	421858	{}	\N	2020-10-12 15:49:32.109437
62416	OVID	2020-09-29	5.88	6.04	5.73	5.98	328482	{}	\N	2020-10-12 15:49:32.116293
62417	OVID	2020-09-30	6.05	6.05	5.61	5.74	1594577	{}	\N	2020-10-12 15:49:32.122847
62418	OVID	2020-10-01	5.84	6.275	5.75	6.05	1051485	{}	\N	2020-10-12 15:49:32.127928
62419	OVID	2020-10-02	5.85	5.99	5.76	5.76	585530	{}	\N	2020-10-12 15:49:32.13449
62420	OVID	2020-10-05	5.89	6.1055	5.78	6.07	567211	{}	\N	2020-10-12 15:49:32.140642
62421	OVID	2020-10-06	6.1	6.24	5.98	5.99	328366	{}	\N	2020-10-12 15:49:32.147373
62422	OVID	2020-10-07	6.02	6.22	6.02	6.17	665312	{}	\N	2020-10-12 15:49:32.154457
62423	OVID	2020-10-08	6.15	6.26	5.81	5.83	590519	{}	\N	2020-10-12 15:49:32.159755
62424	OVID	2020-10-09	5.83	5.89	5.6	5.65	582906	{}	\N	2020-10-12 15:49:32.164968
62425	OVID	2020-10-12	5.74	5.78	5.5	5.59	382904	{}	\N	2020-10-12 15:49:32.171
62426	OVLY	2020-09-28	11.85	11.85	11.425	11.51	13476	{}	\N	2020-10-12 15:49:33.12659
62427	OVLY	2020-09-29	11.39	11.65	11.39	11.65	49627	{}	\N	2020-10-12 15:49:33.136246
62428	OVLY	2020-09-30	11.72	11.72	11.4	11.46	2760	{}	\N	2020-10-12 15:49:33.14386
62429	OVLY	2020-10-01	11.76	12.18	11.41	11.65	9127	{}	\N	2020-10-12 15:49:33.154341
62430	OVLY	2020-10-02	12.12	12.12	12.12	12.12	2231	{}	\N	2020-10-12 15:49:33.160421
62431	OVLY	2020-10-05	12.9999	13.36	12.805	13.3	6357	{}	\N	2020-10-12 15:49:33.16689
62432	OVLY	2020-10-06	13.45	14.34	13.45	13.61	6746	{}	\N	2020-10-12 15:49:33.175637
62433	OVLY	2020-10-07	13.9999	14.58	13.85	14.58	9379	{}	\N	2020-10-12 15:49:33.184051
62434	OVLY	2020-10-08	14.58	14.81	13.6286	14.19	15698	{}	\N	2020-10-12 15:49:33.191033
62435	OVLY	2020-10-09	14.31	14.8	14.31	14.79	2357	{}	\N	2020-10-12 15:49:33.196872
62436	OVLY	2020-10-12	14.73	14.73	14.68	14.68	365	{}	\N	2020-10-12 15:49:33.202963
62437	OXM	2020-09-28	38.83	40.57	38.83	40.07	139913	{}	\N	2020-10-12 15:49:35.119362
62438	OXM	2020-09-29	40.04	40.1	38.59	39.61	392683	{}	\N	2020-10-12 15:49:35.125616
62439	OXM	2020-09-30	39.57	41.13	39.57	40.36	330777	{}	\N	2020-10-12 15:49:35.130318
62440	OXM	2020-10-01	40.31	42.08	40.31	42.02	309294	{}	\N	2020-10-12 15:49:35.135457
62441	OXM	2020-10-02	40.66	43.22	40.66	42.88	297879	{}	\N	2020-10-12 15:49:35.147283
62442	OXM	2020-10-05	43.07	44.39	42.93	44.35	317488	{}	\N	2020-10-12 15:49:35.154647
62443	OXM	2020-10-06	44.89	45.82	43.75	43.93	244019	{}	\N	2020-10-12 15:49:35.159697
62444	OXM	2020-10-07	44.63	44.84	43.22	43.5	318603	{}	\N	2020-10-12 15:49:35.164414
62445	OXM	2020-10-08	44.03	45.58	43.705	45.5	209020	{}	\N	2020-10-12 15:49:35.169021
62446	OXM	2020-10-09	45.87	45.94	44.52	44.73	147681	{}	\N	2020-10-12 15:49:35.174424
62447	OXM	2020-10-12	44.66	45.19	44.2911	44.49	26653	{}	\N	2020-10-12 15:49:35.179874
62448	PAHC	2020-09-28	17.99	18.08	17.58	17.84	121184	{}	\N	2020-10-12 15:49:36.120139
62449	PAHC	2020-09-29	17.8	18.07	17.23	17.36	77886	{}	\N	2020-10-12 15:49:36.128108
62450	PAHC	2020-09-30	17.815	17.815	17.21	17.4	106531	{}	\N	2020-10-12 15:49:36.136593
62451	PAHC	2020-10-01	17.45	17.76	17.04	17.15	76015	{}	\N	2020-10-12 15:49:36.144782
62452	PAHC	2020-10-02	17	17.36	16.77	17.04	99578	{}	\N	2020-10-12 15:49:36.153798
62453	PAHC	2020-10-05	17.26	17.73	16.97	17.64	106800	{}	\N	2020-10-12 15:49:36.161067
62454	PAHC	2020-10-06	17.67	18.115	16.91	17.5	226569	{}	\N	2020-10-12 15:49:36.168099
62455	PAHC	2020-10-07	17.54	17.98	17.5	17.84	157310	{}	\N	2020-10-12 15:49:36.176112
62456	PAHC	2020-10-08	18.01	18.68	18.01	18.3	139481	{}	\N	2020-10-12 15:49:36.181407
62457	PAHC	2020-10-09	18.45	18.966	17.88	18.46	79597	{}	\N	2020-10-12 15:49:36.190907
62458	PAHC	2020-10-12	18.46	18.76	18.43	18.61	43123	{}	\N	2020-10-12 15:49:36.196814
62459	PAR	2020-09-28	44.15	45.3399	43.97	44.72	153030	{}	\N	2020-10-12 15:49:37.504764
62460	PAR	2020-09-29	44.51	44.97	43.57	44.28	329932	{}	\N	2020-10-12 15:49:37.511625
62461	PAR	2020-09-30	43.08	43.78	39.94	40.51	1366373	{}	\N	2020-10-12 15:49:37.518069
62462	PAR	2020-10-01	36.75	37.35	35.44	36.85	6991651	{}	\N	2020-10-12 15:49:37.525844
62463	PAR	2020-10-02	35.76	37.98	35.5014	37.58	1391667	{}	\N	2020-10-12 15:49:37.532947
62464	PAR	2020-10-05	38	38.3	37.3	37.88	912137	{}	\N	2020-10-12 15:49:37.540459
62465	PAR	2020-10-06	38.01	38.25	36.49	36.5	768093	{}	\N	2020-10-12 15:49:37.547467
62466	PAR	2020-10-07	37.36	38.03	37.03	37.62	657450	{}	\N	2020-10-12 15:49:37.553362
62467	PAR	2020-10-08	38	38	36.33	36.59	348802	{}	\N	2020-10-12 15:49:37.560287
62468	PAR	2020-10-09	37.12	37.15	36	36.13	480285	{}	\N	2020-10-12 15:49:37.566534
62469	PAR	2020-10-12	36.13	36.235	35.16	35.99	240544	{}	\N	2020-10-12 15:49:37.571736
62470	PBCTP	2020-09-28	26.57	28.13	26.52	27.44	23958	{}	\N	2020-10-12 15:49:38.490151
62471	PBCTP	2020-09-29	27.59	28.23	26.9252	28	13523	{}	\N	2020-10-12 15:49:38.496624
62472	PBCTP	2020-09-30	27.99	27.99	26.4	26.89	36038	{}	\N	2020-10-12 15:49:38.502046
62473	PBCTP	2020-10-01	27.155	27.18	26.46	26.88	25827	{}	\N	2020-10-12 15:49:38.508601
62474	PBCTP	2020-10-02	26.55	27.55	26.27	27.15	27419	{}	\N	2020-10-12 15:49:38.514983
62475	PBCTP	2020-10-05	27.25	27.25	26.82	26.85	13896	{}	\N	2020-10-12 15:49:38.521484
62476	PBCTP	2020-10-06	27.04	27.125	26.7972	26.96	16239	{}	\N	2020-10-12 15:49:38.530062
62477	PBCTP	2020-10-07	27.01	27.7059	27.01	27.44	22158	{}	\N	2020-10-12 15:49:38.536695
62478	PBCTP	2020-10-08	27.49	27.77	27.19	27.19	10923	{}	\N	2020-10-12 15:49:38.543412
62479	PBCTP	2020-10-09	27.55	27.84	26.98	27.35	10177	{}	\N	2020-10-12 15:49:38.552121
62480	PBCTP	2020-10-12	27.35	27.5257	26.82	27.5257	10256	{}	\N	2020-10-12 15:49:38.557388
62481	PBI	2020-09-28	5.2	5.65	5.16	5.52	2299191	{}	\N	2020-10-12 15:49:40.873716
62482	PBI	2020-09-29	5.5	5.5673	5.34	5.43	1747557	{}	\N	2020-10-12 15:49:40.880077
62483	PBI	2020-09-30	5.44	5.555	5.24	5.31	3132426	{}	\N	2020-10-12 15:49:40.884932
62484	PBI	2020-10-01	5.34	5.44	5.26	5.37	1782963	{}	\N	2020-10-12 15:49:40.889775
62485	PBI	2020-10-02	5.25	5.55	5.23	5.49	2114731	{}	\N	2020-10-12 15:49:40.89583
62486	PBI	2020-10-05	5.58	5.675	5.38	5.64	2706481	{}	\N	2020-10-12 15:49:40.902041
62487	PBI	2020-10-06	5.77	6.125	5.69	5.7	4995274	{}	\N	2020-10-12 15:49:40.907334
62488	PBI	2020-10-07	5.8	6.19	5.8	6.13	5145829	{}	\N	2020-10-12 15:49:40.91441
62489	PBI	2020-10-08	6.23	6.3098	6.015	6.13	2939517	{}	\N	2020-10-12 15:49:40.919094
62490	PBI	2020-10-09	6.18	6.29	6.01	6.06	2606921	{}	\N	2020-10-12 15:49:40.924109
62491	PBI	2020-10-12	6.07	6.1175	5.94	5.99	688542	{}	\N	2020-10-12 15:49:40.929208
62492	PBTP	2020-09-28	25.78	25.795	25.78	25.795	339	{}	\N	2020-10-12 15:49:41.814486
62493	PBTP	2020-09-29	25.8075	25.815	25.8075	25.815	434	{}	\N	2020-10-12 15:49:41.824335
62494	PBTP	2020-09-30	25.8199	25.8199	25.8	25.8	1693	{}	\N	2020-10-12 15:49:41.834679
62495	PBTP	2020-10-01	25.7874	25.81	25.7874	25.8	2804	{}	\N	2020-10-12 15:49:41.844
62496	PBTP	2020-10-02	25.8	25.8	25.785	25.785	1190	{}	\N	2020-10-12 15:49:41.854002
62497	PBTP	2020-10-05	25.8	25.8	25.7945	25.7945	480	{}	\N	2020-10-12 15:49:41.863342
62498	PBTP	2020-10-06	25.82	25.82	25.7901	25.791	4486	{}	\N	2020-10-12 15:49:41.870705
62499	PBTP	2020-10-07	25.81	25.81	25.79	25.8001	12761	{}	\N	2020-10-12 15:49:41.878091
62500	PBTP	2020-10-08	25.8317	25.84	25.8215	25.84	12185	{}	\N	2020-10-12 15:49:41.885401
62501	PBTP	2020-10-09	25.8401	25.85	25.8401	25.8458	3490	{}	\N	2020-10-12 15:49:41.89383
62502	PBTP	2020-10-12	25.85	25.85	25.85	25.85	777	{}	\N	2020-10-12 15:49:41.902697
62503	PCAR	2020-09-28	84.86	86.65	84.705	86.55	1531872	{}	\N	2020-10-12 15:49:42.992259
62504	PCAR	2020-09-29	86.26	87.2	85.78	86.15	1078880	{}	\N	2020-10-12 15:49:43.001095
62505	PCAR	2020-09-30	86.53	87.7	84.68	85.28	1901113	{}	\N	2020-10-12 15:49:43.011469
62506	PCAR	2020-10-01	86.01	86.22	83.56	84.49	1080109	{}	\N	2020-10-12 15:49:43.019758
62507	PCAR	2020-10-02	83.55	85.84	82.85	84.99	989961	{}	\N	2020-10-12 15:49:43.028889
62508	PCAR	2020-10-05	85.93	88.43	85.68	88.09	1170708	{}	\N	2020-10-12 15:49:43.036975
62509	PCAR	2020-10-06	88.08	89.85	87.45	87.68	1186846	{}	\N	2020-10-12 15:49:43.044681
62510	PCAR	2020-10-07	88.77	90	88.37	89.89	1028691	{}	\N	2020-10-12 15:49:43.052683
62511	PCAR	2020-10-08	89.61	89.9599	88	89.94	773362	{}	\N	2020-10-12 15:49:43.062482
62512	PCAR	2020-10-09	90	90.93	88.83	89.22	1794242	{}	\N	2020-10-12 15:49:43.073335
62513	PCAR	2020-10-12	90.22	90.265	89.13	90	348116	{}	\N	2020-10-12 15:49:43.084013
62514	PCN	2020-09-28	15.41	15.51	15.38	15.41	42568	{}	\N	2020-10-12 15:49:44.051671
62515	PCN	2020-09-29	15.5	15.5476	15.38	15.5	103250	{}	\N	2020-10-12 15:49:44.060729
62516	PCN	2020-09-30	15.46	15.65	15.46	15.61	120331	{}	\N	2020-10-12 15:49:44.069938
62517	PCN	2020-10-01	15.83	15.85	15.72	15.8	173201	{}	\N	2020-10-12 15:49:44.077336
62518	PCN	2020-10-02	15.69	15.93	15.69	15.78	180682	{}	\N	2020-10-12 15:49:44.084486
62519	PCN	2020-10-05	15.94	16.11	15.85	16.03	289202	{}	\N	2020-10-12 15:49:44.092021
62520	PCN	2020-10-06	16.09	16.0942	15.9	15.92	135332	{}	\N	2020-10-12 15:49:44.098461
62521	PCN	2020-10-07	16.05	16.1213	15.9745	16.1	117728	{}	\N	2020-10-12 15:49:44.108112
62522	PCN	2020-10-08	16.16	16.47	16.115	16.3	174618	{}	\N	2020-10-12 15:49:44.116356
62523	PCN	2020-10-09	16.16	16.27	16.07	16.14	130326	{}	\N	2020-10-12 15:49:44.123384
62524	PCN	2020-10-12	16.11	16.14	16.095	16.1265	36972	{}	\N	2020-10-12 15:49:44.132244
62525	PCY	2020-09-28	27.16	27.16	27.0367	27.07	847450	{}	\N	2020-10-12 15:49:45.019646
62526	PCY	2020-09-29	27.09	27.093	27	27.02	469522	{}	\N	2020-10-12 15:49:45.026084
62527	PCY	2020-09-30	27.01	27.16	27.0017	27.12	1899208	{}	\N	2020-10-12 15:49:45.035008
62528	PCY	2020-10-01	27.16	27.22	27.09	27.18	9958286	{}	\N	2020-10-12 15:49:45.04448
62529	PCY	2020-10-02	27.084	27.13	27.05	27.05	1069622	{}	\N	2020-10-12 15:49:45.053193
62530	PCY	2020-10-05	27.12	27.21	27.09	27.16	1064900	{}	\N	2020-10-12 15:49:45.059755
62531	PCY	2020-10-06	27.18	27.27	27.1415	27.17	1142788	{}	\N	2020-10-12 15:49:45.065351
62532	PCY	2020-10-07	27.15	27.3	27.15	27.3	923353	{}	\N	2020-10-12 15:49:45.070753
62533	PCY	2020-10-08	27.39	27.57	27.37	27.56	1577245	{}	\N	2020-10-12 15:49:45.077563
62534	PCY	2020-10-09	27.61	27.68	27.61	27.65	1943942	{}	\N	2020-10-12 15:49:45.083902
62535	PCY	2020-10-12	27.66	27.72	27.66	27.6733	286549	{}	\N	2020-10-12 15:49:45.089325
62536	PDLB	2020-09-28	8.79	8.93	8.73	8.73	8091	{}	\N	2020-10-12 15:49:46.933903
62537	PDLB	2020-09-29	8.77	8.95	8.77	8.94	3716	{}	\N	2020-10-12 15:49:46.94408
62538	PDLB	2020-09-30	8.82	8.82	8.82	8.82	1235	{}	\N	2020-10-12 15:49:46.952801
62539	PDLB	2020-10-01	8.88	9.18	8.59	8.78	10462	{}	\N	2020-10-12 15:49:46.961329
62540	PDLB	2020-10-02	8.7	9.58	8.7	9.52	10426	{}	\N	2020-10-12 15:49:46.969965
62541	PDLB	2020-10-05	9.58	9.87	9.58	9.82	6502	{}	\N	2020-10-12 15:49:46.978449
62542	PDLB	2020-10-06	9.18	9.25	9.09	9.09	3621	{}	\N	2020-10-12 15:49:46.987125
62543	PDLB	2020-10-07	9.385	9.92	9.34	9.92	4827	{}	\N	2020-10-12 15:49:46.995896
62544	PDLB	2020-10-08	9.88	10.04	9.35	9.35	7556	{}	\N	2020-10-12 15:49:47.004463
62545	PDLB	2020-10-09	9.44	9.44	9.44	9.44	898	{}	\N	2020-10-12 15:49:47.012214
62546	PEB	2020-09-28	12.44	12.72	12.32	12.59	1295024	{}	\N	2020-10-12 15:49:48.039627
62547	PEB	2020-09-29	13.16	13.16	12.04	12.43	2919800	{}	\N	2020-10-12 15:49:48.049593
62548	PEB	2020-09-30	12.52	12.93	12.295	12.53	4218115	{}	\N	2020-10-12 15:49:48.062785
62549	PEB	2020-10-01	12.62	13.11	12.55	13.09	2540198	{}	\N	2020-10-12 15:49:48.074259
62550	PEB	2020-10-02	12.63	13.42	12.54	13.36	2475836	{}	\N	2020-10-12 15:49:48.089894
62551	PEB	2020-10-05	13.47	13.47	12.85	13.11	2190133	{}	\N	2020-10-12 15:49:48.104097
62552	PEB	2020-10-06	13.3	13.67	13.11	13.35	3181091	{}	\N	2020-10-12 15:49:48.132398
62553	PEB	2020-10-07	13.49	13.85	13.36	13.77	2352295	{}	\N	2020-10-12 15:49:48.145133
62554	PEB	2020-10-08	13.95	14.05	13.6719	13.96	1222756	{}	\N	2020-10-12 15:49:48.159618
62555	PEB	2020-10-09	14	14.17	13.74	13.96	1834723	{}	\N	2020-10-12 15:49:48.172705
62556	PEB	2020-10-12	13.88	14.03	13.56	13.92	335588	{}	\N	2020-10-12 15:49:48.182705
62557	PEIX	2020-09-28	6.75	6.92	6.41	6.63	5180233	{}	\N	2020-10-12 15:49:49.41956
62558	PEIX	2020-09-29	6.68	8	6.6545	7.86	10792636	{}	\N	2020-10-12 15:49:49.429662
62559	PEIX	2020-09-30	7.92	7.95	7.22	7.31	7360369	{}	\N	2020-10-12 15:49:49.439243
62560	PEIX	2020-10-01	7.39	7.95	7.39	7.85	6476047	{}	\N	2020-10-12 15:49:49.448279
62561	PEIX	2020-10-02	7.49	7.82	7.23	7.47	6045199	{}	\N	2020-10-12 15:49:49.458474
62562	PEIX	2020-10-05	7.75	8.62	7.73	8.4	10981598	{}	\N	2020-10-12 15:49:49.464975
62563	PEIX	2020-10-06	8.47	9.02	8	8.49	9828957	{}	\N	2020-10-12 15:49:49.472381
62564	PEIX	2020-10-07	8.85	9.15	8.57	8.8	6068077	{}	\N	2020-10-12 15:49:49.47843
62565	PEIX	2020-10-08	8.7	9.19	8.68	8.91	4214162	{}	\N	2020-10-12 15:49:49.488799
62566	PEIX	2020-10-09	8.99	9.0814	8.65	8.83	3881077	{}	\N	2020-10-12 15:49:49.496944
62567	PEIX	2020-10-12	8.9	8.99	8.33	8.49	2167635	{}	\N	2020-10-12 15:49:49.504608
62568	PERI	2020-09-28	6.5	6.6699	6.45	6.51	66951	{}	\N	2020-10-12 15:49:50.530206
62569	PERI	2020-09-29	6.42	6.5	6.27	6.32	167280	{}	\N	2020-10-12 15:49:50.536165
62570	PERI	2020-09-30	6.805	7.07	6.7822	6.98	652902	{}	\N	2020-10-12 15:49:50.541847
62571	PERI	2020-10-01	7.15	7.24	6.8706	7.01	250878	{}	\N	2020-10-12 15:49:50.546858
62572	PERI	2020-10-02	6.84	6.9799	6.65	6.71	122281	{}	\N	2020-10-12 15:49:50.552847
62573	PERI	2020-10-05	6.72	6.87	6.72	6.82	55790	{}	\N	2020-10-12 15:49:50.558273
62574	PERI	2020-10-06	6.81	7	6.77	6.83	93492	{}	\N	2020-10-12 15:49:50.5634
62575	PERI	2020-10-07	7.5	7.95	7.46	7.64	779131	{}	\N	2020-10-12 15:49:50.569496
62576	PERI	2020-10-08	7.92	8.02	7.78	7.79	600180	{}	\N	2020-10-12 15:49:50.575182
62577	PERI	2020-10-09	7.85	7.99	7.73	7.86	255353	{}	\N	2020-10-12 15:49:50.580371
62578	PERI	2020-10-12	7.83	7.95	7.72	7.95	91089	{}	\N	2020-10-12 15:49:50.586094
62579	PAAS	2020-09-28	31.86	32.34	31.425	32.1	2338240	{}	\N	2020-10-12 15:49:51.775226
62580	PAAS	2020-09-29	32.5	32.81	32.045	32.36	2307049	{}	\N	2020-10-12 15:49:51.780113
62581	PAAS	2020-09-30	32.04	32.48	31.63	32.15	1864114	{}	\N	2020-10-12 15:49:51.786271
62582	PAAS	2020-10-01	32.98	34.04	32.66	33.81	2887625	{}	\N	2020-10-12 15:49:51.792807
62583	PAAS	2020-10-02	33.53	33.78	32.63	33.09	1930182	{}	\N	2020-10-12 15:49:51.798738
62584	PAAS	2020-10-05	33.25	34.16	33.24	33.57	1608609	{}	\N	2020-10-12 15:49:51.805045
62585	PAAS	2020-10-06	33.75	33.89	32.24	32.34	2185962	{}	\N	2020-10-12 15:49:51.811876
62586	PAAS	2020-10-07	32.4	32.83	31.54	31.63	3324064	{}	\N	2020-10-12 15:49:51.818045
62587	PAAS	2020-10-08	31.98	32.57	31.84	32.275	1829132	{}	\N	2020-10-12 15:49:51.824139
62588	PAAS	2020-10-09	33.1781	34.2	32.9	34.18	2297631	{}	\N	2020-10-12 15:49:51.829416
62589	PAAS	2020-10-12	34.2	34.2	33.5	34.1	621127	{}	\N	2020-10-12 15:49:51.834282
62590	PAI	2020-09-28	16.45	16.5561	16.37	16.4894	7950	{}	\N	2020-10-12 15:49:52.801861
62591	PAI	2020-09-29	16.38	16.6799	15.9	16.075	54078	{}	\N	2020-10-12 15:49:52.807422
62592	PAI	2020-09-30	15.99	16.1	15.95	16.1	32394	{}	\N	2020-10-12 15:49:52.812429
62593	PAI	2020-10-01	16.01	16.25	15.9549	16.02	35994	{}	\N	2020-10-12 15:49:52.81822
62594	PAI	2020-10-02	16	16.175	15.9	16.11	52090	{}	\N	2020-10-12 15:49:52.827664
62595	PAI	2020-10-05	16.11	16.5492	15.65	15.65	34099	{}	\N	2020-10-12 15:49:52.834247
62596	PAI	2020-10-06	15.94	16.04	15.46	15.71	32934	{}	\N	2020-10-12 15:49:52.839793
62597	PAI	2020-10-07	15.79	16.0298	15.55	15.56	42110	{}	\N	2020-10-12 15:49:52.846634
62598	PAI	2020-10-08	15.75	15.75	15.41	15.41	53980	{}	\N	2020-10-12 15:49:52.852294
62599	PAI	2020-10-09	15.52	15.5981	15.3	15.46	65474	{}	\N	2020-10-12 15:49:52.858936
62600	PAI	2020-10-12	15.45	15.48	15.3761	15.4	8234	{}	\N	2020-10-12 15:49:52.866723
62601	PAVM	2020-09-28	1.76	1.79	1.74	1.76	399175	{}	\N	2020-10-12 15:49:54.731199
62602	PAVM	2020-09-29	1.78	1.79	1.72	1.76	776859	{}	\N	2020-10-12 15:49:54.737773
62603	PAVM	2020-09-30	1.76	1.82	1.72	1.78	645919	{}	\N	2020-10-12 15:49:54.743985
62604	PAVM	2020-10-01	1.78	1.81	1.75	1.76	325104	{}	\N	2020-10-12 15:49:54.750276
62605	PAVM	2020-10-02	1.76	1.88	1.75	1.86	524331	{}	\N	2020-10-12 15:49:54.756149
62606	PAVM	2020-10-05	1.85	1.94	1.85	1.93	276226	{}	\N	2020-10-12 15:49:54.762252
62607	PAVM	2020-10-06	1.93	1.95	1.87	1.91	285919	{}	\N	2020-10-12 15:49:54.767938
62608	PAVM	2020-10-07	1.91	1.96	1.88	1.91	248911	{}	\N	2020-10-12 15:49:54.773327
62609	PAVM	2020-10-08	1.92	1.94	1.91	1.92	186189	{}	\N	2020-10-12 15:49:54.7793
62610	PAVM	2020-10-09	1.93	2.08	1.88	2.03	1217739	{}	\N	2020-10-12 15:49:54.784849
62611	PAVM	2020-10-12	2.09	2.1399	2.01	2.02	476665	{}	\N	2020-10-12 15:49:54.791975
62612	PBHC	2020-09-28	10.14	10.3	10.12	10.3	2334	{}	\N	2020-10-12 15:49:56.047299
62613	PBHC	2020-10-01	10.6501	10.6501	10.65	10.65	491	{}	\N	2020-10-12 15:49:56.053703
62614	PBHC	2020-10-05	10.69	10.94	10.69	10.92	785	{}	\N	2020-10-12 15:49:56.059018
62615	PBHC	2020-10-06	10.5	10.8	10.5	10.61	1336	{}	\N	2020-10-12 15:49:56.065098
62616	PBHC	2020-10-07	10.55	10.7	10.5	10.7	1012	{}	\N	2020-10-12 15:49:56.072121
62617	PBHC	2020-10-08	10.59	10.59	10.545	10.545	2727	{}	\N	2020-10-12 15:49:56.077761
62618	PBHC	2020-10-09	9.8	9.8	9.8	9.8	575	{}	\N	2020-10-12 15:49:56.083811
62619	PBHC	2020-10-12	10.1	10.1	10.1	10.1	446	{}	\N	2020-10-12 15:49:56.089885
62620	PBYI	2020-09-28	9.83	10.08	9.745	10.03	517099	{}	\N	2020-10-12 15:49:57.791217
62621	PBYI	2020-09-29	10.08	10.24	9.82	10.05	746179	{}	\N	2020-10-12 15:49:57.801778
62622	PBYI	2020-09-30	10.07	10.43	9.99	10.09	412450	{}	\N	2020-10-12 15:49:57.809991
62623	PBYI	2020-10-01	10.18	10.32	10.07	10.22	384407	{}	\N	2020-10-12 15:49:57.819414
62624	PBYI	2020-10-02	10.07	10.39	9.89	9.91	389108	{}	\N	2020-10-12 15:49:57.827389
62625	PBYI	2020-10-05	10.05	10.25	9.9	10.12	502799	{}	\N	2020-10-12 15:49:57.834257
62626	PBYI	2020-10-06	10.3	10.61	10.1	10.22	592556	{}	\N	2020-10-12 15:49:57.839193
62627	PBYI	2020-10-07	10.28	10.42	10.08	10.32	534775	{}	\N	2020-10-12 15:49:57.844442
62628	PBYI	2020-10-08	10.44	10.47	10.07	10.38	369460	{}	\N	2020-10-12 15:49:57.849796
62629	PBYI	2020-10-09	10.53	10.63	9.9	10.01	608639	{}	\N	2020-10-12 15:49:57.854504
62630	PBYI	2020-10-12	9.96	10.14	9.7	9.85	254392	{}	\N	2020-10-12 15:49:57.860339
62631	PCRX	2020-09-28	59.61	59.67	58.49	59.37	276463	{}	\N	2020-10-12 15:49:59.525298
62632	PCRX	2020-09-29	59.28	60.03	58.48	59.39	251211	{}	\N	2020-10-12 15:49:59.533986
62633	PCRX	2020-09-30	59.99	60.87	58.67	60.12	454494	{}	\N	2020-10-12 15:49:59.541484
62634	PCRX	2020-10-01	60.42	61.35	59.175	60.01	294471	{}	\N	2020-10-12 15:49:59.551993
62635	PCRX	2020-10-02	58.69	60.07	57.505	59.59	285765	{}	\N	2020-10-12 15:49:59.559573
62636	PCRX	2020-10-05	60	60.3017	59	60	332437	{}	\N	2020-10-12 15:49:59.566605
62637	PCRX	2020-10-06	60.32	60.8	58.84	59.19	269825	{}	\N	2020-10-12 15:49:59.574019
62638	PCRX	2020-10-07	59.34	59.34	56.9	57.11	640407	{}	\N	2020-10-12 15:49:59.581178
62639	PCRX	2020-10-08	57.9	58.93	56.82	58.12	502832	{}	\N	2020-10-12 15:49:59.587767
62640	PCRX	2020-10-09	58.99	59.85	57.61	57.97	291892	{}	\N	2020-10-12 15:49:59.593047
62641	PCRX	2020-10-12	57.84	58.38	56.85	58.08	64421	{}	\N	2020-10-12 15:49:59.599235
62642	PDFS	2020-09-28	19.21	19.755	19.19	19.66	94983	{}	\N	2020-10-12 15:50:00.719034
62643	PDFS	2020-09-29	19.75	20.02	19.4	19.42	127869	{}	\N	2020-10-12 15:50:00.72542
62644	PDFS	2020-09-30	19.29	19.54	18.62	18.71	270414	{}	\N	2020-10-12 15:50:00.731364
62645	PDFS	2020-10-01	18.8	19.8	18.61	19.76	136088	{}	\N	2020-10-12 15:50:00.738012
62646	PDFS	2020-10-02	19.24	19.58	18.65	18.66	135985	{}	\N	2020-10-12 15:50:00.746016
62647	PDFS	2020-10-05	18.79	19.53	18.79	19.49	226991	{}	\N	2020-10-12 15:50:00.753724
62648	PDFS	2020-10-06	19.61	20.57	19.33	20.04	151562	{}	\N	2020-10-12 15:50:00.759647
62649	PDFS	2020-10-07	20.26	20.59	19.87	19.92	211354	{}	\N	2020-10-12 15:50:00.765611
62650	PDFS	2020-10-08	20.11	20.59	19.67	20.01	76552	{}	\N	2020-10-12 15:50:00.771375
62651	PDFS	2020-10-09	20.13	20.79	19.93	20.71	63821	{}	\N	2020-10-12 15:50:00.777366
62652	PDFS	2020-10-12	20.87	21	20.81	20.88	16342	{}	\N	2020-10-12 15:50:00.782579
62653	PE	2020-09-28	9.26	9.89	9.18	9.69	10346986	{}	\N	2020-10-12 15:50:01.934793
62654	PE	2020-09-29	9.58	9.7	9.295	9.54	14367649	{}	\N	2020-10-12 15:50:01.941447
62655	PE	2020-09-30	9.59	9.59	9.25	9.36	19255281	{}	\N	2020-10-12 15:50:01.947561
62656	PE	2020-10-01	9.23	9.42	9	9.1	13359533	{}	\N	2020-10-12 15:50:01.955955
62657	PE	2020-10-02	8.73	9.5	8.73	9.41	18585258	{}	\N	2020-10-12 15:50:01.963206
62658	PE	2020-10-05	9.62	10	9.47	9.98	18755648	{}	\N	2020-10-12 15:50:01.970533
62659	PE	2020-10-06	10.25	10.42	9.61	9.62	18195298	{}	\N	2020-10-12 15:50:01.976848
62660	PE	2020-10-07	9.7	9.89	9.5648	9.86	11323742	{}	\N	2020-10-12 15:50:01.982492
62661	PE	2020-10-08	9.98	10.3	9.875	10.28	11050453	{}	\N	2020-10-12 15:50:01.989105
62662	PE	2020-10-09	10.45	10.45	10.01	10.21	12828880	{}	\N	2020-10-12 15:50:01.994824
62663	PE	2020-10-12	10.1	10.12	9.91	10.03	4910866	{}	\N	2020-10-12 15:50:02.001791
62664	PEJ	2020-09-28	32	32.41	32	32.36	52884	{}	\N	2020-10-12 15:50:03.496613
62665	PEJ	2020-09-29	32.43	32.465	31.885	32.08	211483	{}	\N	2020-10-12 15:50:03.504963
62666	PEJ	2020-09-30	32.24	32.6351	31.92	32.11	65042	{}	\N	2020-10-12 15:50:03.512451
62667	PEJ	2020-10-01	32.27	32.385	32	32.32	124960	{}	\N	2020-10-12 15:50:03.520038
62668	PEJ	2020-10-02	31.5801	32.43	31.56	32.35	61098	{}	\N	2020-10-12 15:50:03.526994
62669	PEJ	2020-10-05	32.5	32.6279	32.1	32.2752	66976	{}	\N	2020-10-12 15:50:03.53384
62670	PEJ	2020-10-06	32.37	32.836	31.99	31.99	163059	{}	\N	2020-10-12 15:50:03.540563
62671	PEJ	2020-10-07	32.3537	32.53	32.2	32.47	42606	{}	\N	2020-10-12 15:50:03.547503
62672	PEJ	2020-10-08	32.62	32.63	32.29	32.56	58137	{}	\N	2020-10-12 15:50:03.554858
62673	PEJ	2020-10-09	32.74	32.86	32.5049	32.59	201113	{}	\N	2020-10-12 15:50:03.561133
62674	PEJ	2020-10-12	32.75	32.79	32.59	32.7017	14448	{}	\N	2020-10-12 15:50:03.567097
62675	PEZ	2020-09-28	62.93	63.72	62.93	63.72	6628	{}	\N	2020-10-12 15:50:04.967747
62676	PEZ	2020-09-29	64.015	64.2775	63.41	63.54	3945	{}	\N	2020-10-12 15:50:04.976653
62677	PEZ	2020-09-30	63.76	64.369	63.6003	63.6003	3766	{}	\N	2020-10-12 15:50:04.98377
62678	PEZ	2020-10-01	64.08	65.7757	64.08	65.36	6835	{}	\N	2020-10-12 15:50:04.992398
62679	PEZ	2020-10-02	64.17	65.5	62.56	65.1735	3935	{}	\N	2020-10-12 15:50:04.998597
62680	PEZ	2020-10-05	66.0189	66.4106	65.34	66.35	11887	{}	\N	2020-10-12 15:50:05.004516
62681	PEZ	2020-10-06	66.18	66.7499	64.7453	64.82	4340	{}	\N	2020-10-12 15:50:05.011735
62682	PEZ	2020-10-07	65.78	66.5	65.78	66.3392	6974	{}	\N	2020-10-12 15:50:05.019244
62683	PEZ	2020-10-08	66.76	66.76	65.865	66.61	6183	{}	\N	2020-10-12 15:50:05.027113
62684	PEZ	2020-10-09	66.94	67.51	66.92	67.3607	3625	{}	\N	2020-10-12 15:50:05.032866
62685	PEZ	2020-10-12	67.62	68.71	67.62	68.44	1901	{}	\N	2020-10-12 15:50:05.039987
62686	PFG	2020-09-28	39.57	40.37	39.37	40.1	1571604	{}	\N	2020-10-12 15:50:05.947681
62687	PFG	2020-09-29	39.75	40.1424	39.18	40.06	1477222	{}	\N	2020-10-12 15:50:05.953972
62688	PFG	2020-09-30	40.26	40.87	39.835	40.27	1450082	{}	\N	2020-10-12 15:50:05.961106
62689	PFG	2020-10-01	40.37	40.81	39.85	40.36	1297337	{}	\N	2020-10-12 15:50:05.969546
62690	PFG	2020-10-02	39.72	41.34	39.72	41.05	764797	{}	\N	2020-10-12 15:50:05.977535
62691	PFG	2020-10-05	41.52	42.4	41.5	41.88	794351	{}	\N	2020-10-12 15:50:05.984578
62692	PFG	2020-10-06	42.25	43.15	41.59	41.73	1383872	{}	\N	2020-10-12 15:50:05.991712
62693	PFG	2020-10-07	42.17	43.19	42.17	43.09	1020782	{}	\N	2020-10-12 15:50:05.998592
62694	PFG	2020-10-08	43.43	43.75	42.95	43.45	884109	{}	\N	2020-10-12 15:50:06.004279
62695	PFG	2020-10-09	43.71	43.93	42.405	42.47	1236060	{}	\N	2020-10-12 15:50:06.012929
62696	PFG	2020-10-12	42.68	43	42.3	42.86	217562	{}	\N	2020-10-12 15:50:06.020185
62697	PFO	2020-09-28	12.01	12.01	11.8801	11.8911	6003	{}	\N	2020-10-12 15:50:07.267376
62698	PFO	2020-09-29	11.9	12.01	11.86	12.01	23008	{}	\N	2020-10-12 15:50:07.275485
62699	PFO	2020-09-30	12.11	12.11	11.88	11.9	53284	{}	\N	2020-10-12 15:50:07.281591
62700	PFO	2020-10-01	11.99	12.171	11.92	11.92	26872	{}	\N	2020-10-12 15:50:07.288081
62701	PFO	2020-10-02	11.9	12.0762	11.9	11.95	22106	{}	\N	2020-10-12 15:50:07.295331
62702	PFO	2020-10-05	12	12.119	11.9451	12.09	14726	{}	\N	2020-10-12 15:50:07.30269
62703	PFO	2020-10-06	11.97	12.12	11.94	12.06	35422	{}	\N	2020-10-12 15:50:07.309939
62704	PFO	2020-10-07	12.0714	12.095	12.02	12.095	23250	{}	\N	2020-10-12 15:50:07.316312
62705	PFO	2020-10-08	12.21	12.27	12.13	12.26	56940	{}	\N	2020-10-12 15:50:07.322454
62706	PFO	2020-10-09	12.22	12.22	11.91	11.94	46214	{}	\N	2020-10-12 15:50:07.329406
62707	PFO	2020-10-12	11.95	12.08	11.95	12.08	24354	{}	\N	2020-10-12 15:50:07.335608
62708	PAA	2020-09-28	5.72	5.98	5.66	5.89	3971418	{}	\N	2020-10-12 15:50:08.195429
62709	PAA	2020-09-29	5.85	6.02	5.725	5.94	10560621	{}	\N	2020-10-12 15:50:08.203482
62710	PAA	2020-09-30	5.95	6.08	5.81	5.98	19297149	{}	\N	2020-10-12 15:50:08.211114
62711	PAA	2020-10-01	5.94	6.05	5.812	6	10374947	{}	\N	2020-10-12 15:50:08.219409
62712	PAA	2020-10-02	5.8	6.02	5.77	5.94	7170899	{}	\N	2020-10-12 15:50:08.227737
62713	PAA	2020-10-05	6.05	6.24	5.93	6.23	8200164	{}	\N	2020-10-12 15:50:08.233741
62714	PAA	2020-10-06	6.3	6.46	5.77	5.79	19153443	{}	\N	2020-10-12 15:50:08.238698
62715	PAA	2020-10-07	5.82	5.89	5.615	5.76	16557794	{}	\N	2020-10-12 15:50:08.24404
62716	PAA	2020-10-08	5.81	6.4	5.76	6.37	13945585	{}	\N	2020-10-12 15:50:08.24959
62717	PAA	2020-10-09	6.45	6.6	6.315	6.39	10681506	{}	\N	2020-10-12 15:50:08.256431
62718	PAA	2020-10-12	6.4	6.43	6.28	6.415	2548256	{}	\N	2020-10-12 15:50:08.262507
62719	PAM	2020-09-28	10.56	10.77	10.2728	10.34	184884	{}	\N	2020-10-12 15:50:10.712732
62720	PAM	2020-09-29	10.315	10.36	10.02	10.13	334194	{}	\N	2020-10-12 15:50:10.721432
62721	PAM	2020-09-30	10.12	10.44	9.975	10.33	440566	{}	\N	2020-10-12 15:50:10.728638
62722	PAM	2020-10-01	10.35	10.71	10.35	10.58	235878	{}	\N	2020-10-12 15:50:10.735029
62723	PAM	2020-10-02	10.47	10.915	10.3301	10.78	344087	{}	\N	2020-10-12 15:50:10.74105
62724	PAM	2020-10-05	10.8	11.02	10.7	10.99	365487	{}	\N	2020-10-12 15:50:10.747376
62725	PAM	2020-10-06	11.13	11.2	10.23	10.37	439322	{}	\N	2020-10-12 15:50:10.753697
62726	PAM	2020-10-07	10.36	10.71	10.26	10.46	401947	{}	\N	2020-10-12 15:50:10.762448
62727	PAM	2020-10-08	10.48	10.74	10.25	10.65	340242	{}	\N	2020-10-12 15:50:10.771709
62728	PAM	2020-10-09	10.55	10.89	10.32	10.83	300374	{}	\N	2020-10-12 15:50:10.778097
62729	PAM	2020-10-12	10.84	10.84	10.65	10.71	25550	{}	\N	2020-10-12 15:50:10.784656
62730	PATI	2020-09-28	9.0261	9.0261	9.0261	9.0261	136	{}	\N	2020-10-12 15:50:12.080081
62731	PATI	2020-09-30	8.995	8.995	8.95	8.95	797	{}	\N	2020-10-12 15:50:12.088124
62732	PATI	2020-10-01	8.8431	8.8431	8.68	8.74	1046	{}	\N	2020-10-12 15:50:12.095032
62733	PATI	2020-10-02	8.7704	8.845	8.63	8.75	1052	{}	\N	2020-10-12 15:50:12.103999
62734	PATI	2020-10-05	8.8213	8.8213	8.6	8.6	654	{}	\N	2020-10-12 15:50:12.110514
62735	PATI	2020-10-06	8.56	8.65	8.56	8.65	691	{}	\N	2020-10-12 15:50:12.117421
62736	PATI	2020-10-07	8.7501	8.7501	8.75	8.75	480	{}	\N	2020-10-12 15:50:12.123997
62737	PATI	2020-10-08	8.75	8.7938	8.75	8.7938	580	{}	\N	2020-10-12 15:50:12.130791
62738	PATI	2020-10-12	8.8	8.8	8.8	8.8	143	{}	\N	2020-10-12 15:50:12.138622
62739	PBA	2020-09-28	22.16	22.23	21.65	21.78	1123897	{}	\N	2020-10-12 15:50:13.812705
62740	PBA	2020-09-29	21.75	21.94	21.37	21.56	2780526	{}	\N	2020-10-12 15:50:13.821656
62741	PBA	2020-09-30	21.59	21.88	21.17	21.23	2621188	{}	\N	2020-10-12 15:50:13.827538
62742	PBA	2020-10-01	21.36	21.67	21.12	21.4	2391469	{}	\N	2020-10-12 15:50:13.835151
62743	PBA	2020-10-02	20.91	21.63	20.77	21.44	2133496	{}	\N	2020-10-12 15:50:13.844048
62744	PBA	2020-10-05	21.79	22.06	21.48	21.85	1623226	{}	\N	2020-10-12 15:50:13.852434
62745	PBA	2020-10-06	22.23	22.34	21.08	21.28	1862236	{}	\N	2020-10-12 15:50:13.85878
62746	PBA	2020-10-07	21.37	21.53	21.08	21.36	1774804	{}	\N	2020-10-12 15:50:13.864974
62747	PBA	2020-10-08	21.6	22.6	21.51	22.6	1966825	{}	\N	2020-10-12 15:50:13.870636
62748	PBA	2020-10-09	22.76	22.89	21.96	22.25	1686620	{}	\N	2020-10-12 15:50:13.876525
62749	PBA	2020-10-12	22.12	22.3	22.0314	22.11	394763	{}	\N	2020-10-12 15:50:13.885716
62750	PBE	2020-09-28	55.97	56.25	55.8026	56.23	4519	{}	\N	2020-10-12 15:50:14.833306
62751	PBE	2020-09-29	55.61	56.98	55.61	56.8606	5618	{}	\N	2020-10-12 15:50:14.840045
62752	PBE	2020-09-30	56.96	57.1643	56.4	56.62	6397	{}	\N	2020-10-12 15:50:14.850169
62753	PBE	2020-10-01	56.79	57.56	56.79	57.49	3295	{}	\N	2020-10-12 15:50:14.858379
62754	PBE	2020-10-02	56.55	57.5593	56.43	56.43	7068	{}	\N	2020-10-12 15:50:14.864448
62755	PBE	2020-10-05	57.36	58.976	57.36	58.976	7191	{}	\N	2020-10-12 15:50:14.871941
62756	PBE	2020-10-06	59.14	59.486	58.52	58.54	4021	{}	\N	2020-10-12 15:50:14.878316
62757	PBE	2020-10-07	59.32	60.4667	59.32	60.2534	7494	{}	\N	2020-10-12 15:50:14.886282
62758	PBE	2020-10-08	60.95	61.1908	60.675	61.0966	4228	{}	\N	2020-10-12 15:50:14.894671
62759	PBE	2020-10-09	61.26	62.2848	61.2418	62.2848	5811	{}	\N	2020-10-12 15:50:14.90241
62760	PBE	2020-10-12	62	62.3	61.9309	62.015	2634	{}	\N	2020-10-12 15:50:14.909156
62761	PBND	2020-09-28	26.8213	26.8213	26.8009	26.8009	1632	{}	\N	2020-10-12 15:50:16.663057
62762	PBND	2020-09-29	26.8599	26.8599	26.825	26.825	213	{}	\N	2020-10-12 15:50:16.669897
62763	PBND	2020-09-30	26.84	26.84	26.7683	26.825	6655	{}	\N	2020-10-12 15:50:16.676863
62764	PBND	2020-10-01	26.8076	26.85	26.8076	26.85	4171	{}	\N	2020-10-12 15:50:16.684335
62765	PBND	2020-10-02	26.82	26.85	26.77	26.85	4890	{}	\N	2020-10-12 15:50:16.692385
62766	PBND	2020-10-05	26.76	26.76	26.7136	26.7136	835	{}	\N	2020-10-12 15:50:16.700128
62767	PBND	2020-10-06	26.72	26.77	26.6904	26.75	13501	{}	\N	2020-10-12 15:50:16.709334
62768	PBND	2020-10-07	26.7248	26.78	26.7052	26.723	3569	{}	\N	2020-10-12 15:50:16.717556
62769	PBND	2020-10-08	26.7261	26.76	26.7182	26.7375	3425	{}	\N	2020-10-12 15:50:16.726159
62770	PBND	2020-10-09	26.69	26.76	26.69	26.76	5888	{}	\N	2020-10-12 15:50:16.742058
62771	PBND	2020-10-12	26.76	26.7655	26.76	26.7655	701	{}	\N	2020-10-12 15:50:16.750714
62772	PBT	2020-09-28	2.57	2.7	2.53	2.67	69542	{}	\N	2020-10-12 15:50:17.649771
62773	PBT	2020-09-29	2.6	2.678	2.54	2.61	98503	{}	\N	2020-10-12 15:50:17.656983
62774	PBT	2020-09-30	2.57	2.65	2.48	2.48	178502	{}	\N	2020-10-12 15:50:17.66257
62775	PBT	2020-10-01	2.52	2.58	2.46	2.5	175965	{}	\N	2020-10-12 15:50:17.668057
62776	PBT	2020-10-02	2.47	2.55	2.47	2.52	92400	{}	\N	2020-10-12 15:50:17.67444
62777	PBT	2020-10-05	2.47	2.54	2.47	2.51	207000	{}	\N	2020-10-12 15:50:17.680402
62778	PBT	2020-10-06	2.54	2.605	2.511	2.59	160684	{}	\N	2020-10-12 15:50:17.687131
62779	PBT	2020-10-07	2.57	2.61	2.53	2.53	78667	{}	\N	2020-10-12 15:50:17.692882
62780	PBT	2020-10-08	2.51	2.58	2.49	2.49	216014	{}	\N	2020-10-12 15:50:17.700092
62781	PBT	2020-10-09	2.53	2.55	2.48	2.51	130350	{}	\N	2020-10-12 15:50:17.709562
62782	PBT	2020-10-12	2.48	2.55	2.48	2.49	70924	{}	\N	2020-10-12 15:50:17.715419
62783	PCM	2020-09-28	9.93	10.0229	9.866	10.01	27671	{}	\N	2020-10-12 15:50:18.808413
62784	PCM	2020-09-29	10.07	10.07	9.9387	10.07	45814	{}	\N	2020-10-12 15:50:18.815865
62785	PCM	2020-09-30	10.07	10.12	9.9799	10.12	28916	{}	\N	2020-10-12 15:50:18.821796
62786	PCM	2020-10-01	10.18	10.36	10.13	10.26	60102	{}	\N	2020-10-12 15:50:18.827365
62787	PCM	2020-10-02	10.16	10.33	10.04	10.28	39246	{}	\N	2020-10-12 15:50:18.833302
62788	PCM	2020-10-05	10.29	10.33	10.2436	10.32	110606	{}	\N	2020-10-12 15:50:18.838653
62789	PCM	2020-10-06	10.27	10.36	10.27	10.36	42834	{}	\N	2020-10-12 15:50:18.844133
62790	PCM	2020-10-07	10.3241	10.35	10.25	10.3	64845	{}	\N	2020-10-12 15:50:18.849509
62791	PCM	2020-10-08	10.3	10.33	10.07	10.18	79856	{}	\N	2020-10-12 15:50:18.856386
62792	PCM	2020-10-09	10.14	10.14	10	10.02	60424	{}	\N	2020-10-12 15:50:18.862111
62793	PCM	2020-10-12	9.97	10.01	9.955	10.01	19042	{}	\N	2020-10-12 15:50:18.867035
62794	PCYO	2020-09-28	9.25	9.309	9.18	9.18	69463	{}	\N	2020-10-12 15:50:20.12125
62795	PCYO	2020-09-29	9.19	9.2	8.96	9.01	92258	{}	\N	2020-10-12 15:50:20.128239
62796	PCYO	2020-09-30	9.03	9.09	8.96	9.01	83431	{}	\N	2020-10-12 15:50:20.135317
62797	PCYO	2020-10-01	9.01	9.26	8.815	9.18	98908	{}	\N	2020-10-12 15:50:20.145973
62798	PCYO	2020-10-02	9.14	9.14	8.96	9.05	34175	{}	\N	2020-10-12 15:50:20.153424
62799	PCYO	2020-10-05	9.13	9.22	9.04	9.22	93955	{}	\N	2020-10-12 15:50:20.159902
62800	PCYO	2020-10-06	9.48	9.555	9.12	9.27	39053	{}	\N	2020-10-12 15:50:20.165691
62801	PCYO	2020-10-07	9.41	9.52	9.235	9.45	38252	{}	\N	2020-10-12 15:50:20.171109
62802	PCYO	2020-10-08	9.53	9.69	9.43	9.63	27167	{}	\N	2020-10-12 15:50:20.177154
62803	PCYO	2020-10-09	9.58	9.95	9.58	9.88	36313	{}	\N	2020-10-12 15:50:20.182628
62804	PCYO	2020-10-12	9.99	10.17	9.95	10.12	15354	{}	\N	2020-10-12 15:50:20.188424
62805	PEG	2020-09-28	54.57	55.165	54.29	54.35	1622026	{}	\N	2020-10-12 15:50:21.177277
62806	PEG	2020-09-29	54.6	55.3531	54.35	54.8	3474526	{}	\N	2020-10-12 15:50:21.185897
62807	PEG	2020-09-30	55.18	55.395	54.53	54.91	5675922	{}	\N	2020-10-12 15:50:21.198017
62808	PEG	2020-10-01	54.96	55.37	54.47	55.07	4448470	{}	\N	2020-10-12 15:50:21.20958
62809	PEG	2020-10-02	54.59	56.41	54.18	55.95	7089016	{}	\N	2020-10-12 15:50:21.217918
62810	PEG	2020-10-05	56.05	56.9	55.56	56.32	3793235	{}	\N	2020-10-12 15:50:21.225056
62811	PEG	2020-10-06	56.39	56.945	55.56	55.9	4720163	{}	\N	2020-10-12 15:50:21.231323
62812	PEG	2020-10-07	56.38	57.34	56.38	57.03	4124854	{}	\N	2020-10-12 15:50:21.238094
62813	PEG	2020-10-08	57.13	58.49	57.12	58.4	2876799	{}	\N	2020-10-12 15:50:21.24453
62814	PEG	2020-10-09	58.64	58.69	57.35	57.56	4316281	{}	\N	2020-10-12 15:50:21.250774
62815	PEG	2020-10-12	57.52	57.84	57.24	57.765	922344	{}	\N	2020-10-12 15:50:21.257113
62816	PEO	2020-09-28	9.99	10.05	9.93	9.95	59380	{}	\N	2020-10-12 15:50:22.106227
62817	PEO	2020-09-29	9.98	9.98	9.66	9.72	122959	{}	\N	2020-10-12 15:50:22.11444
62818	PEO	2020-09-30	9.74	9.85	9.71	9.78	133704	{}	\N	2020-10-12 15:50:22.120847
62819	PEO	2020-10-01	9.68	9.71	9.56	9.62	508847	{}	\N	2020-10-12 15:50:22.128546
62820	PEO	2020-10-02	9.44	9.73	9.44	9.69	58496	{}	\N	2020-10-12 15:50:22.135966
62821	PEO	2020-10-05	9.75	9.88	9.75	9.85	134484	{}	\N	2020-10-12 15:50:22.142501
62822	PEO	2020-10-06	9.92	9.98	9.71	9.71	99103	{}	\N	2020-10-12 15:50:22.14913
62823	PEO	2020-10-07	9.77	9.91	9.76	9.91	85936	{}	\N	2020-10-12 15:50:22.154545
62824	PEO	2020-10-08	10	10.19	9.93	10.13	103518	{}	\N	2020-10-12 15:50:22.16036
62825	PEO	2020-10-09	10.16	10.2	10.0754	10.11	185323	{}	\N	2020-10-12 15:50:22.165756
62826	PEO	2020-10-12	10.1	10.16	10.05	10.11	46486	{}	\N	2020-10-12 15:50:22.172363
62827	PAC	2020-09-28	78.18	79.6874	77.1	79.37	124287	{}	\N	2020-10-12 15:50:23.34752
62828	PAC	2020-09-29	78.76	80.6	78.76	79.32	128571	{}	\N	2020-10-12 15:50:23.353925
62829	PAC	2020-09-30	79.39	83.54	79.39	80.64	270176	{}	\N	2020-10-12 15:50:23.360019
62830	PAC	2020-10-01	81.77	84.24	80.98	82.12	226473	{}	\N	2020-10-12 15:50:23.36536
62831	PAC	2020-10-02	80.78	83.805	80.59	82.72	125288	{}	\N	2020-10-12 15:50:23.371041
62832	PAC	2020-10-05	83.26	84.45	82.62	83.14	113999	{}	\N	2020-10-12 15:50:23.377345
62833	PAC	2020-10-06	84.51	85.35	83.055	83.56	107539	{}	\N	2020-10-12 15:50:23.382274
62834	PAC	2020-10-07	84.76	86.63	84.76	86.21	99525	{}	\N	2020-10-12 15:50:23.386638
62835	PAC	2020-10-08	87.03	88.44	86.39	88.09	116796	{}	\N	2020-10-12 15:50:23.39216
62836	PAC	2020-10-09	88.21	89	86.79	87.51	150203	{}	\N	2020-10-12 15:50:23.399454
62837	PAC	2020-10-12	87.26	89.13	87.26	88.985	40288	{}	\N	2020-10-12 15:50:23.40479
62838	PAGP	2020-09-28	5.92	6.195	5.85	6.11	3811902	{}	\N	2020-10-12 15:50:24.301621
62839	PAGP	2020-09-29	6.07	6.19	5.93	6.11	5748291	{}	\N	2020-10-12 15:50:24.307344
62840	PAGP	2020-09-30	6.14	6.3	5.965	6.09	6051199	{}	\N	2020-10-12 15:50:24.313241
62841	PAGP	2020-10-01	6.03	6.22	5.9801	6.16	6031257	{}	\N	2020-10-12 15:50:24.32042
62842	PAGP	2020-10-02	5.93	6.18	5.92	6.12	4524449	{}	\N	2020-10-12 15:50:24.327437
62843	PAGP	2020-10-05	6.23	6.42	6.11	6.4	4192593	{}	\N	2020-10-12 15:50:24.333625
62844	PAGP	2020-10-06	6.51	6.6	5.89	5.92	9235435	{}	\N	2020-10-12 15:50:24.338934
62845	PAGP	2020-10-07	5.98	6.0355	5.785	5.95	5690803	{}	\N	2020-10-12 15:50:24.345072
62846	PAGP	2020-10-08	5.99	6.66	5.96	6.64	9864849	{}	\N	2020-10-12 15:50:24.354585
62847	PAGP	2020-10-09	6.78	6.89	6.585	6.68	6804824	{}	\N	2020-10-12 15:50:24.364004
62848	PAGP	2020-10-12	6.74	6.8	6.56	6.71	1462323	{}	\N	2020-10-12 15:50:24.373602
62849	PATK	2020-09-28	55.99	58.2	55.17	56.39	179082	{}	\N	2020-10-12 15:50:25.487486
62850	PATK	2020-09-29	57.39	59.23	56.79	56.84	191991	{}	\N	2020-10-12 15:50:25.493667
62851	PATK	2020-09-30	56.83	59.07	56.83	57.7	129308	{}	\N	2020-10-12 15:50:25.499332
62852	PATK	2020-10-01	58.51	62.85	58.51	62.75	292582	{}	\N	2020-10-12 15:50:25.504201
62853	PATK	2020-10-02	60.68	64.01	60	63.48	209587	{}	\N	2020-10-12 15:50:25.510309
62854	PATK	2020-10-05	64.16	67.16	64.16	66.71	192736	{}	\N	2020-10-12 15:50:25.516274
62855	PATK	2020-10-06	66.74	68.5	65.3401	66.03	146967	{}	\N	2020-10-12 15:50:25.522328
62856	PATK	2020-10-07	65.97	68.29	65.6	66.76	144359	{}	\N	2020-10-12 15:50:25.529704
62857	PATK	2020-10-08	67.67	67.67	62.72	63.34	169740	{}	\N	2020-10-12 15:50:25.535545
62858	PATK	2020-10-09	63.87	64.89	63.395	64.1	134372	{}	\N	2020-10-12 15:50:25.540465
62859	PATK	2020-10-12	64.75	64.8373	63.9	64.28	19918	{}	\N	2020-10-12 15:50:25.545955
62860	PBB	2020-09-28	25.28	25.5	25.19	25.5	20602	{}	\N	2020-10-12 15:50:26.62333
62861	PBB	2020-09-29	25.49	25.49	25.42	25.49	20226	{}	\N	2020-10-12 15:50:26.628865
62862	PBB	2020-09-30	25.5	25.74	25.28	25.74	149244	{}	\N	2020-10-12 15:50:26.63461
62863	PBB	2020-10-01	25.66	25.77	25.62	25.77	49226	{}	\N	2020-10-12 15:50:26.639761
62864	PBB	2020-10-02	25.2274	25.92	25.2274	25.92	12112	{}	\N	2020-10-12 15:50:26.647907
62865	PBB	2020-10-05	25.88	25.92	25.5994	25.9	15654	{}	\N	2020-10-12 15:50:26.653889
62866	PBB	2020-10-06	25.63	25.99	25.5315	25.89	29950	{}	\N	2020-10-12 15:50:26.65895
62867	PBB	2020-10-07	25.57	25.73	25.23	25.54	35810	{}	\N	2020-10-12 15:50:26.664867
62868	PBB	2020-10-08	25.43	25.88	25.43	25.88	18470	{}	\N	2020-10-12 15:50:26.670351
62869	PBB	2020-10-09	25.77	25.85	25.535	25.75	10506	{}	\N	2020-10-12 15:50:26.67525
62870	PBB	2020-10-12	25.63	25.65	25.63	25.65	1532	{}	\N	2020-10-12 15:50:26.681804
62871	PBH	2020-09-28	34.64	35.3	34.56	35.22	461010	{}	\N	2020-10-12 15:50:27.991405
62872	PBH	2020-09-29	35.41	35.49	34.74	35.22	637685	{}	\N	2020-10-12 15:50:27.999593
62873	PBH	2020-09-30	35.47	36.52	35.45	36.42	868084	{}	\N	2020-10-12 15:50:28.008607
62874	PBH	2020-10-01	36.56	36.725	35.79	36.11	1003908	{}	\N	2020-10-12 15:50:28.016159
62875	PBH	2020-10-02	35.68	36.15	35.48	35.8	512912	{}	\N	2020-10-12 15:50:28.023165
62876	PBH	2020-10-05	36.08	36.56	36	36.45	471793	{}	\N	2020-10-12 15:50:28.028131
62877	PBH	2020-10-06	36.71	37.065	36.17	36.22	514101	{}	\N	2020-10-12 15:50:28.033529
62878	PBH	2020-10-07	36.34	36.63	35.86	36.29	587889	{}	\N	2020-10-12 15:50:28.039292
62879	PBH	2020-10-08	36.68	36.89	36.25	36.54	403127	{}	\N	2020-10-12 15:50:28.044548
62880	PBH	2020-10-09	36.91	37.17	36.475	37.16	418482	{}	\N	2020-10-12 15:50:28.050503
62881	PBH	2020-10-12	37.13	37.66	36.94	37.635	52511	{}	\N	2020-10-12 15:50:28.056523
62882	PBR	2020-09-28	7.56	7.62	7.19	7.2	25814291	{}	\N	2020-10-12 15:50:29.173713
62883	PBR	2020-09-29	7.13	7.22	6.95	6.99	49237174	{}	\N	2020-10-12 15:50:29.178572
62884	PBR	2020-09-30	7.1	7.19	7.025	7.12	52393296	{}	\N	2020-10-12 15:50:29.183903
62885	PBR	2020-10-01	6.95	7.115	6.83	7.07	65646879	{}	\N	2020-10-12 15:50:29.189513
62886	PBR	2020-10-02	6.9	7	6.795	6.84	53650983	{}	\N	2020-10-12 15:50:29.199096
62887	PBR	2020-10-05	6.96	7.2893	6.88	7.21	59908354	{}	\N	2020-10-12 15:50:29.204106
62888	PBR	2020-10-06	7.51	7.52	7.13	7.17	37090553	{}	\N	2020-10-12 15:50:29.209
62889	PBR	2020-10-07	7.14	7.2	6.97	7.11	29922980	{}	\N	2020-10-12 15:50:29.213956
62890	PBR	2020-10-08	7.13	7.39	7.11	7.38	36473287	{}	\N	2020-10-12 15:50:29.21943
62891	PBR	2020-10-09	7.35	7.3698	7.19	7.25	40515966	{}	\N	2020-10-12 15:50:29.224406
62892	PBR	2020-10-12	7.25	7.2901	7.16	7.205	5329107	{}	\N	2020-10-12 15:50:29.229478
62893	PCG	2020-09-28	9.96	9.99	9.195	9.45	47916213	{}	\N	2020-10-12 15:50:30.701657
62894	PCG	2020-09-29	9.45	9.54	9.33	9.39	44598407	{}	\N	2020-10-12 15:50:30.707347
62895	PCG	2020-09-30	9.45	9.63	9.345	9.39	36346425	{}	\N	2020-10-12 15:50:30.712687
62896	PCG	2020-10-01	9.5	9.83	9.45	9.83	38554420	{}	\N	2020-10-12 15:50:30.718364
62897	PCG	2020-10-02	9.69	10.05	9.62	9.9	52906321	{}	\N	2020-10-12 15:50:30.72376
62898	PCG	2020-10-05	9.93	10	9.85	9.94	51618336	{}	\N	2020-10-12 15:50:30.72944
62899	PCG	2020-10-06	9.95	10.44	9.91	10.15	78843133	{}	\N	2020-10-12 15:50:30.736643
62900	PCG	2020-10-07	10.27	10.37	10.165	10.26	25812507	{}	\N	2020-10-12 15:50:30.742258
62901	PCG	2020-10-08	10.31	10.61	10.28	10.57	35997895	{}	\N	2020-10-12 15:50:30.747047
62902	PCG	2020-10-09	10.56	10.79	10.44	10.76	32720647	{}	\N	2020-10-12 15:50:30.752506
62903	PCG	2020-10-12	10.13	10.425	10.11	10.41	22214823	{}	\N	2020-10-12 15:50:30.75739
62904	PCSB	2020-09-28	12.15	12.47	12.15	12.27	41880	{}	\N	2020-10-12 15:50:32.220057
62905	PCSB	2020-09-29	12.18	12.2	12.01	12.15	40737	{}	\N	2020-10-12 15:50:32.225855
62906	PCSB	2020-09-30	12.26	12.37	12	12.07	40101	{}	\N	2020-10-12 15:50:32.231503
62907	PCSB	2020-10-01	12.09	12.27	11.96	12.23	35449	{}	\N	2020-10-12 15:50:32.23822
62908	PCSB	2020-10-02	12.03	12.64	12.02	12.53	38063	{}	\N	2020-10-12 15:50:32.244377
62909	PCSB	2020-10-05	12.53	12.81	12.53	12.72	41736	{}	\N	2020-10-12 15:50:32.252232
62910	PCSB	2020-10-06	12.84	13.04	12.61	12.67	50276	{}	\N	2020-10-12 15:50:32.258163
62911	PCSB	2020-10-07	12.92	13.01	12.75	12.93	67020	{}	\N	2020-10-12 15:50:32.26362
62912	PCSB	2020-10-08	12.94	13.05	12.87	12.92	32789	{}	\N	2020-10-12 15:50:32.269766
62913	PCSB	2020-10-09	12.98	13.05	12.85	12.91	33589	{}	\N	2020-10-12 15:50:32.275939
62914	PCSB	2020-10-12	12.86	13.03	12.86	13.02	6435	{}	\N	2020-10-12 15:50:32.282624
62915	PDEX	2020-09-28	28.29	28.45	26.43	27.47	13663	{}	\N	2020-10-12 15:50:33.178225
62916	PDEX	2020-09-29	28.5999	28.5999	26.58	27.8	16855	{}	\N	2020-10-12 15:50:33.183022
62917	PDEX	2020-09-30	27.77	28.6	27.77	28.6	13609	{}	\N	2020-10-12 15:50:33.188392
62918	PDEX	2020-10-01	28.405	28.95	27.5652	28.2	45420	{}	\N	2020-10-12 15:50:33.193959
62919	PDEX	2020-10-02	28.5	29.5	28	29.5	15429	{}	\N	2020-10-12 15:50:33.199415
62920	PDEX	2020-10-05	29.58	29.58	27.95	28.5	19668	{}	\N	2020-10-12 15:50:33.207388
62921	PDEX	2020-10-06	28.88	29.55	28.1401	28.81	9590	{}	\N	2020-10-12 15:50:33.213681
62922	PDEX	2020-10-07	29.1	29.37	28.015	29.37	6226	{}	\N	2020-10-12 15:50:33.218948
62923	PDEX	2020-10-08	29.17	29.34	28.635	29.1	4874	{}	\N	2020-10-12 15:50:33.226646
62924	PDEX	2020-10-09	29.19	31.96	28.15	28.5	88481	{}	\N	2020-10-12 15:50:33.232631
62925	PDEX	2020-10-12	29.5	29.5	28.71	28.79	6640	{}	\N	2020-10-12 15:50:33.240155
62926	PDT	2020-09-28	12.35	12.48	12.333	12.3601	77895	{}	\N	2020-10-12 15:50:34.373116
62927	PDT	2020-09-29	12.42	12.42	12.2	12.28	199294	{}	\N	2020-10-12 15:50:34.37997
62928	PDT	2020-09-30	12.26	12.49	12.26	12.45	553220	{}	\N	2020-10-12 15:50:34.38505
62929	PDT	2020-10-01	12.5	12.56	12.4	12.51	181941	{}	\N	2020-10-12 15:50:34.390983
62930	PDT	2020-10-02	12.44	12.72	12.42	12.66	294177	{}	\N	2020-10-12 15:50:34.39576
62931	PDT	2020-10-05	12.71	12.86	12.68	12.82	118322	{}	\N	2020-10-12 15:50:34.410006
62932	PDT	2020-10-06	12.94	12.96	12.85	12.88	318000	{}	\N	2020-10-12 15:50:34.418559
62933	PDT	2020-10-07	13	13.15	12.962	13.14	259270	{}	\N	2020-10-12 15:50:34.427253
62934	PDT	2020-10-08	13.15	13.2	13.13	13.17	288879	{}	\N	2020-10-12 15:50:34.434504
62935	PDT	2020-10-09	13.17	13.17	13.02	13.07	267010	{}	\N	2020-10-12 15:50:34.441497
62936	PDT	2020-10-12	13.05	13.09	13.01	13.075	123918	{}	\N	2020-10-12 15:50:34.448082
62937	PENN	2020-09-28	70.89	73.2	68.2	72.92	12315457	{}	\N	2020-10-12 15:50:35.3945
62938	PENN	2020-09-29	72.71	72.84	67.77	69.41	12105147	{}	\N	2020-10-12 15:50:35.406495
62939	PENN	2020-09-30	71.0313	75.2425	70.8941	72.7	13893118	{}	\N	2020-10-12 15:50:35.417457
62940	PENN	2020-10-01	74.38	74.93	71.69	74.07	8758466	{}	\N	2020-10-12 15:50:35.426853
62941	PENN	2020-10-02	70.36	74	70.1	72.75	6827464	{}	\N	2020-10-12 15:50:35.435944
62942	PENN	2020-10-05	73.18	73.9299	69.16	70.49	7256422	{}	\N	2020-10-12 15:50:35.444958
62943	PENN	2020-10-06	70.97	71.49	67.28	67.94	7218281	{}	\N	2020-10-12 15:50:35.451075
62944	PENN	2020-10-07	67.88	69.89	67.53	68.45	4935120	{}	\N	2020-10-12 15:50:35.45794
62945	PENN	2020-10-08	69.49	70	64.51	65.69	8692359	{}	\N	2020-10-12 15:50:35.464204
62946	PENN	2020-10-09	66.5184	68.35	63.65	64.7	6546198	{}	\N	2020-10-12 15:50:35.471172
62947	PENN	2020-10-12	66.21	67.6042	65.1	65.72	2230884	{}	\N	2020-10-12 15:50:35.479284
62948	PFBI	2020-09-28	10.72	10.97	10.62	10.83	23453	{}	\N	2020-10-12 15:50:36.696487
62949	PFBI	2020-09-29	10.9	10.98	10.69	10.83	13455	{}	\N	2020-10-12 15:50:36.702315
62950	PFBI	2020-09-30	10.89	10.96	10.75	10.8	25150	{}	\N	2020-10-12 15:50:36.70864
62951	PFBI	2020-10-01	10.825	10.9377	10.54	10.68	29209	{}	\N	2020-10-12 15:50:36.713705
62952	PFBI	2020-10-02	10.5	10.99	10.1	10.85	13406	{}	\N	2020-10-12 15:50:36.718851
62953	PFBI	2020-10-05	10.75	11.305	10.75	11.26	39763	{}	\N	2020-10-12 15:50:36.725084
62954	PFBI	2020-10-06	11.37	12.04	11.01	11.52	25648	{}	\N	2020-10-12 15:50:36.7308
62955	PFBI	2020-10-07	11.5	11.93	11.4	11.82	23267	{}	\N	2020-10-12 15:50:36.735662
62956	PFBI	2020-10-08	11.9	12.39	11.7	12.19	12058	{}	\N	2020-10-12 15:50:36.741514
62957	PFBI	2020-10-09	12.32	12.35	11.85	12.05	27137	{}	\N	2020-10-12 15:50:36.748455
62958	PFBI	2020-10-12	12.06	12.1199	11.88	12.1199	2900	{}	\N	2020-10-12 15:50:36.753359
62959	PFL	2020-09-28	10.11	10.12	10.08	10.09	83629	{}	\N	2020-10-12 15:50:37.98964
62960	PFL	2020-09-29	10.13	10.15	10.08	10.08	177152	{}	\N	2020-10-12 15:50:37.996256
62961	PFL	2020-09-30	10.12	10.15	10.1	10.12	172606	{}	\N	2020-10-12 15:50:38.003083
62962	PFL	2020-10-01	10.21	10.2195	10.14	10.16	293330	{}	\N	2020-10-12 15:50:38.009536
62963	PFL	2020-10-02	10.12	10.2394	10.058	10.23	256589	{}	\N	2020-10-12 15:50:38.016236
62964	PFL	2020-10-05	10.25	10.33	10.25	10.3	419256	{}	\N	2020-10-12 15:50:38.021292
62965	PFL	2020-10-06	10.35	10.3604	10.29	10.29	522935	{}	\N	2020-10-12 15:50:38.030515
62966	PFL	2020-10-07	10.33	10.41	10.33	10.37	316808	{}	\N	2020-10-12 15:50:38.039656
62967	PFL	2020-10-08	10.41	10.52	10.38	10.5	397130	{}	\N	2020-10-12 15:50:38.050832
62968	PFL	2020-10-09	10.46	10.46	10.38	10.4	386157	{}	\N	2020-10-12 15:50:38.060104
62969	PFL	2020-10-12	10.39	10.4419	10.3299	10.35	122358	{}	\N	2020-10-12 15:50:38.067065
62970	PACW	2020-09-28	16.92	17.17	16.63	17.01	1030662	{}	\N	2020-10-12 15:50:39.000412
62971	PACW	2020-09-29	16.1484	16.979	16.1484	16.87	1121435	{}	\N	2020-10-12 15:50:39.006099
62972	PACW	2020-09-30	17.03	17.33	16.84	17.08	1365488	{}	\N	2020-10-12 15:50:39.012154
62973	PACW	2020-10-01	17.11	17.4	16.7999	17.005	1576364	{}	\N	2020-10-12 15:50:39.018605
62974	PACW	2020-10-02	16.65	17.47	16.65	17.4	1579468	{}	\N	2020-10-12 15:50:39.024552
62975	PACW	2020-10-05	17.64	18.33	17.585	18.17	1015441	{}	\N	2020-10-12 15:50:39.031032
62976	PACW	2020-10-06	18.47	18.97	17.92	18.04	1133204	{}	\N	2020-10-12 15:50:39.036535
62977	PACW	2020-10-07	18.38	18.99	18.27	18.92	1197637	{}	\N	2020-10-12 15:50:39.043771
62978	PACW	2020-10-08	19.07	19.5	18.91	19.45	1430109	{}	\N	2020-10-12 15:50:39.049793
62979	PACW	2020-10-09	19.45	19.67	19.08	19.13	1358512	{}	\N	2020-10-12 15:50:39.05522
62980	PACW	2020-10-12	19.14	19.51	19.02	19.48	488685	{}	\N	2020-10-12 15:50:39.060809
62981	PBIP	2020-09-28	10.25	10.77	10.25	10.49	4724	{}	\N	2020-10-12 15:50:41.170014
62982	PBIP	2020-09-29	10.59	10.9	10.535	10.81	9538	{}	\N	2020-10-12 15:50:41.175784
62983	PBIP	2020-09-30	10.92	10.97	10.41	10.54	18481	{}	\N	2020-10-12 15:50:41.181237
62984	PBIP	2020-10-01	10.8	10.9399	10.42	10.42	13725	{}	\N	2020-10-12 15:50:41.185615
62985	PBIP	2020-10-02	10.34	10.85	10.33	10.52	12513	{}	\N	2020-10-12 15:50:41.190479
62986	PBIP	2020-10-05	10.65	10.95	10.62	10.71	29942	{}	\N	2020-10-12 15:50:41.196042
62987	PBIP	2020-10-06	10.71	11.58	10.52	11.01	13793	{}	\N	2020-10-12 15:50:41.201231
62988	PBIP	2020-10-07	11.02	11.26	10.855	10.99	6162	{}	\N	2020-10-12 15:50:41.205768
62989	PBIP	2020-10-08	11.1	11.2	10.9	11.09	10047	{}	\N	2020-10-12 15:50:41.210493
62990	PBIP	2020-10-09	10.85	11.13	10.85	11.03	7049	{}	\N	2020-10-12 15:50:41.216866
62991	PBIP	2020-10-12	11.2	11.2	11.1843	11.1843	1253	{}	\N	2020-10-12 15:50:41.221298
62992	PBUS	2020-09-28	33.816	33.93	33.74	33.87	3773	{}	\N	2020-10-12 15:50:42.154457
62993	PBUS	2020-09-29	33.8699	33.8699	33.6519	33.6519	7181	{}	\N	2020-10-12 15:50:42.16236
62994	PBUS	2020-09-30	33.93	34.21	33.7825	33.9147	8085	{}	\N	2020-10-12 15:50:42.169534
62995	PBUS	2020-10-01	34.14	34.1591	34	34	1606	{}	\N	2020-10-12 15:50:42.175868
62996	PBUS	2020-10-02	33.9273	34.03	33.92	33.92	4930	{}	\N	2020-10-12 15:50:42.182101
62997	PBUS	2020-10-05	34.3	34.458	34.3	34.458	600	{}	\N	2020-10-12 15:50:42.187931
62998	PBUS	2020-10-06	34.5209	34.5509	34	34	2140	{}	\N	2020-10-12 15:50:42.193961
62999	PBUS	2020-10-07	34.4844	34.621	34.4524	34.621	3863	{}	\N	2020-10-12 15:50:42.200046
63000	PBUS	2020-10-08	34.7777	34.8693	34.7777	34.8693	1905	{}	\N	2020-10-12 15:50:42.205338
63001	PBUS	2020-10-09	36.17	36.17	35.05	35.47	6391	{}	\N	2020-10-12 15:50:42.212537
63002	PBUS	2020-10-12	35.6699	35.6699	35.6599	35.6599	1489	{}	\N	2020-10-12 15:50:42.21877
63003	PCF	2020-09-28	8.32	8.42	8.3	8.335	39562	{}	\N	2020-10-12 15:50:44.411306
63004	PCF	2020-09-29	8.325	8.345	8.31	8.33	20006	{}	\N	2020-10-12 15:50:44.424167
63005	PCF	2020-09-30	8.42	8.42	8.2939	8.33	44654	{}	\N	2020-10-12 15:50:44.432787
63006	PCF	2020-10-01	8.3198	8.32	8.3	8.32	46346	{}	\N	2020-10-12 15:50:44.440605
63007	PCF	2020-10-02	8.18	8.28	8.11	8.21	128196	{}	\N	2020-10-12 15:50:44.448535
63008	PCF	2020-10-05	8.21	8.23	8.19	8.23	58100	{}	\N	2020-10-12 15:50:44.456741
63009	PCF	2020-10-06	8.2998	8.32	8.24	8.24	63498	{}	\N	2020-10-12 15:50:44.462198
63010	PCF	2020-10-07	8.2839	8.3	8.2839	8.29	17610	{}	\N	2020-10-12 15:50:44.469084
63011	PCF	2020-10-08	8.32	8.325	8.2961	8.325	48612	{}	\N	2020-10-12 15:50:44.473809
63012	PCF	2020-10-09	8.31	8.36	8.31	8.35	27744	{}	\N	2020-10-12 15:50:44.478512
63013	PCF	2020-10-12	8.35	8.5	8.35	8.405	31148	{}	\N	2020-10-12 15:50:44.483188
63014	PCQ	2020-09-28	17.86	17.86	17.66	17.68	13738	{}	\N	2020-10-12 15:50:45.489711
63015	PCQ	2020-09-29	17.74	17.8546	17.6	17.61	25296	{}	\N	2020-10-12 15:50:45.495762
63016	PCQ	2020-09-30	17.65	17.7799	17.65	17.7	23540	{}	\N	2020-10-12 15:50:45.502659
63017	PCQ	2020-10-01	17.93	17.93	17.635	17.7526	70641	{}	\N	2020-10-12 15:50:45.507957
63018	PCQ	2020-10-02	17.6	17.92	17.6	17.8225	32614	{}	\N	2020-10-12 15:50:45.513191
63019	PCQ	2020-10-05	17.93	18	17.855	17.98	36898	{}	\N	2020-10-12 15:50:45.519558
63020	PCQ	2020-10-06	18	18.01	17.9355	18.01	34552	{}	\N	2020-10-12 15:50:45.52555
63021	PCQ	2020-10-07	18.05	18.2	18.021	18.025	41756	{}	\N	2020-10-12 15:50:45.531001
63022	PCQ	2020-10-08	17.88	18.15	17.87	17.9	25780	{}	\N	2020-10-12 15:50:45.536532
63023	PCQ	2020-10-09	17.74	18.09	17.74	17.84	36784	{}	\N	2020-10-12 15:50:45.542615
63024	PCQ	2020-10-12	18	18	17.99	17.99	3314	{}	\N	2020-10-12 15:50:45.54864
63025	PDCO	2020-09-28	23.88	24.43	23.69	24.27	713587	{}	\N	2020-10-12 15:50:46.51544
63026	PDCO	2020-09-29	24.24	24.27	23.52	23.57	548683	{}	\N	2020-10-12 15:50:46.521965
63027	PDCO	2020-09-30	23.55	24.36	23.53	24.105	950997	{}	\N	2020-10-12 15:50:46.527173
63028	PDCO	2020-10-01	24.23	24.63	23.77	24.13	1063431	{}	\N	2020-10-12 15:50:46.533764
63029	PDCO	2020-10-02	23.7	24.62	23.47	24.5	610094	{}	\N	2020-10-12 15:50:46.540341
63030	PDCO	2020-10-05	24.66	25.315	24.5	25.18	746250	{}	\N	2020-10-12 15:50:46.546227
63031	PDCO	2020-10-06	25.27	25.57	24.65	24.82	913541	{}	\N	2020-10-12 15:50:46.553658
63032	PDCO	2020-10-07	24.76	25.3	24.66	24.88	813745	{}	\N	2020-10-12 15:50:46.559469
63033	PDCO	2020-10-08	25.23	25.63	24.86	25.48	605601	{}	\N	2020-10-12 15:50:46.564664
63034	PDCO	2020-10-09	25.68	25.73	25.2	25.27	579093	{}	\N	2020-10-12 15:50:46.571411
63035	PDCO	2020-10-12	25.28	25.49	25.13	25.45	86623	{}	\N	2020-10-12 15:50:46.57713
63036	PDP	2020-09-28	75.87	76.2	75.578	76.14	344468	{}	\N	2020-10-12 15:50:47.551346
63037	PDP	2020-09-29	76.1	76.575	76.01	76.15	77209	{}	\N	2020-10-12 15:50:47.558328
63038	PDP	2020-09-30	76.15	77.12	75.9	76.45	123501	{}	\N	2020-10-12 15:50:47.563557
63039	PDP	2020-10-01	77.03	77.5875	76.85	77.49	450843	{}	\N	2020-10-12 15:50:47.569647
63040	PDP	2020-10-02	76.13	77.23	75.9528	76.54	74716	{}	\N	2020-10-12 15:50:47.57576
63041	PDP	2020-10-05	77.06	78.14	77	78.13	95692	{}	\N	2020-10-12 15:50:47.581031
63042	PDP	2020-10-06	78.14	78.91	77.24	77.43	129915	{}	\N	2020-10-12 15:50:47.586641
63043	PDP	2020-10-07	78.26	79.18	78.26	79.05	115420	{}	\N	2020-10-12 15:50:47.592342
63044	PDP	2020-10-08	79.56	79.67	78.91	79.18	95896	{}	\N	2020-10-12 15:50:47.597719
63045	PDP	2020-10-09	79.5	80.3	79.5	80.23	42928	{}	\N	2020-10-12 15:50:47.603426
63046	PDP	2020-10-12	80.94	81.0886	80.62	81.025	14841	{}	\N	2020-10-12 15:50:47.607773
63047	PEGA	2020-09-28	120.9	121.32	119.23	120.28	178281	{}	\N	2020-10-12 15:50:49.941478
63048	PEGA	2020-09-29	121.67	121.93	119.7573	121.66	260731	{}	\N	2020-10-12 15:50:49.946739
63049	PEGA	2020-09-30	122	123.9	120.28	121.04	291704	{}	\N	2020-10-12 15:50:49.951974
63050	PEGA	2020-10-01	122.5	124.835	121.6101	123.97	215713	{}	\N	2020-10-12 15:50:49.958352
63051	PEGA	2020-10-02	122.65	124.4	121.46	121.78	180834	{}	\N	2020-10-12 15:50:49.966215
63052	PEGA	2020-10-05	122.98	126.33	122.91	125.46	325418	{}	\N	2020-10-12 15:50:49.972439
63053	PEGA	2020-10-06	126.91	131.83	126.01	128.26	538968	{}	\N	2020-10-12 15:50:49.978447
63054	PEGA	2020-10-07	130	131.27	128.91	130.11	217681	{}	\N	2020-10-12 15:50:49.983985
63055	PEGA	2020-10-08	131	131.72	128.57	128.9	212602	{}	\N	2020-10-12 15:50:49.990085
63056	PEGA	2020-10-09	130	132.23	130	131.36	237045	{}	\N	2020-10-12 15:50:49.995644
63057	PEGA	2020-10-12	133	134.29	132.15	132.577	41897	{}	\N	2020-10-12 15:50:50.001073
63058	PETQ	2020-09-28	32.24	33.19	32.22	33.03	329357	{}	\N	2020-10-12 15:50:50.925174
63059	PETQ	2020-09-29	33.21	33.67	32.63	32.78	311125	{}	\N	2020-10-12 15:50:50.931298
63060	PETQ	2020-09-30	32.95	33.5	32.48	32.92	215039	{}	\N	2020-10-12 15:50:50.936752
63061	PETQ	2020-10-01	33.15	33.36	31.1804	32.48	231262	{}	\N	2020-10-12 15:50:50.943139
63062	PETQ	2020-10-02	31.83	33.36	31.77	32.98	366245	{}	\N	2020-10-12 15:50:50.950057
63063	PETQ	2020-10-05	33.42	33.75	32.79	33.71	143335	{}	\N	2020-10-12 15:50:50.95588
63064	PETQ	2020-10-06	34.09	34.93	33.56	33.74	263995	{}	\N	2020-10-12 15:50:50.963035
63065	PETQ	2020-10-07	33.82	34.92	33.82	34.54	308599	{}	\N	2020-10-12 15:50:50.969128
63066	PETQ	2020-10-08	34.99	34.99	33.4	33.47	216959	{}	\N	2020-10-12 15:50:50.975688
63067	PETQ	2020-10-09	33.58	34.52	32.6801	33.18	202675	{}	\N	2020-10-12 15:50:50.981346
63068	PETQ	2020-10-12	33.39	33.44	32.16	33.02	63281	{}	\N	2020-10-12 15:50:50.986514
63069	PFIN	2020-09-29	4.27	4.38	4.27	4.38	814	{}	\N	2020-10-12 15:50:53.128263
63070	PFIN	2020-09-30	4.315	4.43	4.2	4.38	4330	{}	\N	2020-10-12 15:50:53.134375
63071	PFIN	2020-10-01	4.4	4.45	4.4	4.4456	886	{}	\N	2020-10-12 15:50:53.141319
63072	PFIN	2020-10-02	4.3208	4.416	4.22	4.35	5090	{}	\N	2020-10-12 15:50:53.148418
63073	PFIN	2020-10-05	4.3099	4.3099	4.15	4.15	9923	{}	\N	2020-10-12 15:50:53.154806
63074	PFIN	2020-10-06	4.34	4.4399	4.16	4.395	19063	{}	\N	2020-10-12 15:50:53.160853
63075	PFIN	2020-10-07	4.47	4.47	4.2628	4.319	1947	{}	\N	2020-10-12 15:50:53.166701
63076	PFIN	2020-10-08	4.42	4.42	4.265	4.41	1423	{}	\N	2020-10-12 15:50:53.172302
63077	PFIN	2020-10-09	4.5	4.5	4.3384	4.3384	1286	{}	\N	2020-10-12 15:50:53.178646
63078	PFIN	2020-10-12	4.47	4.81	4.47	4.81	8380	{}	\N	2020-10-12 15:50:53.18654
63079	PG	2020-09-28	138.56	139.935	137.78	138.01	5814921	{}	\N	2020-10-12 15:50:54.916445
63080	PG	2020-09-29	138.39	138.42	136.66	137.26	8663797	{}	\N	2020-10-12 15:50:54.923299
63081	PG	2020-09-30	138.16	139.85	137.81	138.99	16834024	{}	\N	2020-10-12 15:50:54.92983
63082	PG	2020-10-01	139.58	139.99	138.21	139.24	11304831	{}	\N	2020-10-12 15:50:54.934906
63083	PG	2020-10-02	137.93	139.12	136.95	138.12	11720442	{}	\N	2020-10-12 15:50:54.940487
63084	PG	2020-10-05	138.72	139.65	138.3	139.39	12484732	{}	\N	2020-10-12 15:50:54.945878
63085	PG	2020-10-06	139.42	140.77	138.81	139.61	11482323	{}	\N	2020-10-12 15:50:54.951245
63086	PG	2020-10-07	140.08	141.39	139.37	140.7	9061958	{}	\N	2020-10-12 15:50:54.95663
63087	PG	2020-10-08	140.56	141.93	140.3635	141.65	9134510	{}	\N	2020-10-12 15:50:54.962799
63088	PG	2020-10-09	141.84	143.64	141.7	142.92	12779393	{}	\N	2020-10-12 15:50:54.968648
63089	PG	2020-10-12	143.33	145.28	143.33	145.19	4996065	{}	\N	2020-10-12 15:50:54.9738
63090	PHB	2020-09-28	18.44	18.525	18.44	18.52	314724	{}	\N	2020-10-12 15:50:57.012958
63091	PHB	2020-09-29	18.53	18.54	18.445	18.53	223782	{}	\N	2020-10-12 15:50:57.019134
63092	PHB	2020-09-30	18.5399	18.59	18.52	18.54	613396	{}	\N	2020-10-12 15:50:57.024513
63093	PHB	2020-10-01	18.58	18.64	18.57	18.64	399072	{}	\N	2020-10-12 15:50:57.030213
63094	PHB	2020-10-02	18.57	18.63	18.57	18.63	915404	{}	\N	2020-10-12 15:50:57.036512
63095	PHB	2020-10-05	18.67	18.73	18.63	18.72	752356	{}	\N	2020-10-12 15:50:57.041679
63096	PHB	2020-10-06	18.69	18.8	18.69	18.7	992765	{}	\N	2020-10-12 15:50:57.04713
63097	PHB	2020-10-07	18.78	18.78	18.7354	18.77	577794	{}	\N	2020-10-12 15:50:57.052645
63098	PHB	2020-10-08	18.78	18.83	18.77	18.79	140232	{}	\N	2020-10-12 15:50:57.06014
63099	PHB	2020-10-09	18.8	18.85	18.8	18.83	269744	{}	\N	2020-10-12 15:50:57.065695
63100	PHB	2020-10-12	18.87	18.928	18.87	18.9	27460	{}	\N	2020-10-12 15:50:57.070576
63101	PAGS	2020-09-28	38.28	38.74	37.49	37.73	837088	{}	\N	2020-10-12 15:50:59.007518
63102	PAGS	2020-09-29	37.8	37.92	37.11	37.54	1403884	{}	\N	2020-10-12 15:50:59.021226
63103	PAGS	2020-09-30	37.81	38.88	37.085	37.71	3312990	{}	\N	2020-10-12 15:50:59.029935
63104	PAGS	2020-10-01	38.4	39.3442	38.02	39.14	3868544	{}	\N	2020-10-12 15:50:59.040395
63105	PAGS	2020-10-02	38.03	39	37.8837	38.52	2625211	{}	\N	2020-10-12 15:50:59.047387
63106	PAGS	2020-10-05	38.34	38.94	37.73	38.78	1683081	{}	\N	2020-10-12 15:50:59.054931
63107	PAGS	2020-10-06	38.99	39.79	38.22	38.53	1464320	{}	\N	2020-10-12 15:50:59.061764
63108	PAGS	2020-10-07	39.18	40.09	38.39	40	2408297	{}	\N	2020-10-12 15:50:59.067454
63109	PAGS	2020-10-08	40.46	41	40	40.88	1782893	{}	\N	2020-10-12 15:50:59.072253
63110	PAGS	2020-10-09	41	42.82	40.57	41.85	1880679	{}	\N	2020-10-12 15:50:59.076869
63111	PAGS	2020-10-12	42.59	42.59	41.07	41.07	364539	{}	\N	2020-10-12 15:50:59.081384
63112	PANW	2020-09-28	244.42	247.55	244.01	245.94	943454	{}	\N	2020-10-12 15:51:00.132783
63113	PANW	2020-09-29	245.99	248.63	245.68	246.49	2350670	{}	\N	2020-10-12 15:51:00.140023
63114	PANW	2020-09-30	244.76	247.81	243.81	244.75	1316580	{}	\N	2020-10-12 15:51:00.145834
63115	PANW	2020-10-01	248.67	252.78	247.63	250.53	2205834	{}	\N	2020-10-12 15:51:00.151702
63116	PANW	2020-10-02	246.32	251.36	243.68	245.16	1390035	{}	\N	2020-10-12 15:51:00.157374
63117	PANW	2020-10-05	245.67	249.56	244.01	248.67	1658949	{}	\N	2020-10-12 15:51:00.161822
63118	PANW	2020-10-06	247.7	252.43	246.58	247.24	1113700	{}	\N	2020-10-12 15:51:00.166758
63119	PANW	2020-10-07	249.29	253.04	247.65	250.96	942268	{}	\N	2020-10-12 15:51:00.17359
63120	PANW	2020-10-08	252.24	253.32	248.5	249.51	1506054	{}	\N	2020-10-12 15:51:00.179691
63121	PANW	2020-10-09	252.24	254.07	250.2	252.52	1262777	{}	\N	2020-10-12 15:51:00.185236
63122	PANW	2020-10-12	255	255.9	252.82	253.53	304244	{}	\N	2020-10-12 15:51:00.189579
63123	PAYX	2020-09-28	78.77	79.83	78.77	79.58	1494898	{}	\N	2020-10-12 15:51:01.326932
63124	PAYX	2020-09-29	79.54	80.03	78.9	79.17	1575513	{}	\N	2020-10-12 15:51:01.332502
63125	PAYX	2020-09-30	79.2626	80.44	79.04	79.77	1821170	{}	\N	2020-10-12 15:51:01.337837
63126	PAYX	2020-10-01	80.11	80.95	78.7	79.35	1377834	{}	\N	2020-10-12 15:51:01.342948
63127	PAYX	2020-10-02	78.01	79.9899	78	79.42	1258110	{}	\N	2020-10-12 15:51:01.349623
63128	PAYX	2020-10-05	80.14	81.79	79.97	81.45	2516781	{}	\N	2020-10-12 15:51:01.356382
63129	PAYX	2020-10-06	80.68	82.8545	79.07	79.43	3298587	{}	\N	2020-10-12 15:51:01.361298
63130	PAYX	2020-10-07	80.425	81.16	80.17	80.52	1891302	{}	\N	2020-10-12 15:51:01.366432
63131	PAYX	2020-10-08	82	83.5	81.56	82.07	1786789	{}	\N	2020-10-12 15:51:01.372299
63132	PAYX	2020-10-09	82.54	82.98	81.58	82.4	1283437	{}	\N	2020-10-12 15:51:01.377473
63133	PAYX	2020-10-12	83.22	83.32	82.41	83.22	541543	{}	\N	2020-10-12 15:51:01.382507
63134	PBFX	2020-09-28	8.48	8.67	8.31	8.46	135898	{}	\N	2020-10-12 15:51:02.383885
63135	PBFX	2020-09-29	8.5	8.66	8.345	8.58	131896	{}	\N	2020-10-12 15:51:02.390387
63136	PBFX	2020-09-30	8.54	8.68	8.09	8.15	391086	{}	\N	2020-10-12 15:51:02.395571
63137	PBFX	2020-10-01	8.2	8.29	7.83	8.03	466769	{}	\N	2020-10-12 15:51:02.400936
63138	PBFX	2020-10-02	7.87	8.37	7.7	8.19	347411	{}	\N	2020-10-12 15:51:02.415068
63139	PBFX	2020-10-05	8.27	8.64	8.17	8.6	330344	{}	\N	2020-10-12 15:51:02.423766
63140	PBFX	2020-10-06	8.68	8.79	8.32	8.47	215989	{}	\N	2020-10-12 15:51:02.432398
63141	PBFX	2020-10-07	8.55	9.25	8.33	8.73	446160	{}	\N	2020-10-12 15:51:02.440129
63142	PBFX	2020-10-08	8.78	9.01	8.7102	8.91	161453	{}	\N	2020-10-12 15:51:02.445741
63143	PBFX	2020-10-09	8.97	9	8.745	8.92	297054	{}	\N	2020-10-12 15:51:02.45149
63144	PBFX	2020-10-12	8.92	8.92	8.65	8.715	91834	{}	\N	2020-10-12 15:51:02.458552
63145	PBPB	2020-09-28	3.67	3.91	3.645	3.81	108429	{}	\N	2020-10-12 15:51:03.656552
63146	PBPB	2020-09-29	3.79	3.84	3.615	3.79	135875	{}	\N	2020-10-12 15:51:03.665299
63147	PBPB	2020-09-30	3.79	3.92	3.72	3.8	95352	{}	\N	2020-10-12 15:51:03.670708
63148	PBPB	2020-10-01	3.84	3.85	3.65	3.67	112515	{}	\N	2020-10-12 15:51:03.677889
63149	PBPB	2020-10-02	3.6	3.82	3.5	3.82	139485	{}	\N	2020-10-12 15:51:03.683908
63150	PBPB	2020-10-05	3.84	3.8663	3.64	3.75	175776	{}	\N	2020-10-12 15:51:03.690837
63151	PBPB	2020-10-06	3.76	3.79	3.66	3.7	135466	{}	\N	2020-10-12 15:51:03.698985
63152	PBPB	2020-10-07	3.71	3.76	3.66	3.71	191562	{}	\N	2020-10-12 15:51:03.704521
63153	PBPB	2020-10-08	3.72	3.74	3.68	3.72	68968	{}	\N	2020-10-12 15:51:03.709916
63154	PBPB	2020-10-09	3.76	4.27	3.69	4.18	296845	{}	\N	2020-10-12 15:51:03.714747
63155	PBPB	2020-10-12	4.15	4.1989	3.99	4.01	55672	{}	\N	2020-10-12 15:51:03.720693
63156	PCEF	2020-09-28	20.16	20.31	20.16	20.22	84518	{}	\N	2020-10-12 15:51:05.138586
63157	PCEF	2020-09-29	20.26	20.27	20.1635	20.19	179290	{}	\N	2020-10-12 15:51:05.145659
63158	PCEF	2020-09-30	20.3335	20.4	20.24	20.24	132641	{}	\N	2020-10-12 15:51:05.152079
63159	PCEF	2020-10-01	20.29	20.41	20.29	20.36	112429	{}	\N	2020-10-12 15:51:05.158961
63160	PCEF	2020-10-02	20.19	20.35	20.14	20.31	76186	{}	\N	2020-10-12 15:51:05.165205
63161	PCEF	2020-10-05	20.38	20.5	20.38	20.5	51058	{}	\N	2020-10-12 15:51:05.170798
63162	PCEF	2020-10-06	20.51	20.58	20.415	20.45	86323	{}	\N	2020-10-12 15:51:05.1778
63163	PCEF	2020-10-07	20.52	20.5999	20.4	20.58	94051	{}	\N	2020-10-12 15:51:05.183332
63164	PCEF	2020-10-08	20.64	20.69	20.6234	20.67	179970	{}	\N	2020-10-12 15:51:05.189852
63165	PCEF	2020-10-09	20.7	20.74	20.66	20.7	80319	{}	\N	2020-10-12 15:51:05.196765
63166	PCEF	2020-10-12	20.72	20.7799	20.72	20.7799	36888	{}	\N	2020-10-12 15:51:05.202718
63167	PCOM	2020-09-28	9.5	9.77	9.5	9.65	8898	{}	\N	2020-10-12 15:51:06.324996
63168	PCOM	2020-09-29	9.59	9.79	9.57	9.79	11908	{}	\N	2020-10-12 15:51:06.329812
63169	PCOM	2020-09-30	9.25	9.83	9.25	9.68	21650	{}	\N	2020-10-12 15:51:06.334215
63170	PCOM	2020-10-01	9.715	9.91	9.66	9.91	12915	{}	\N	2020-10-12 15:51:06.338229
63171	PCOM	2020-10-02	9.755	9.85	9.6422	9.75	10626	{}	\N	2020-10-12 15:51:06.342803
63172	PCOM	2020-10-05	9.77	9.99	9.74	9.79	19005	{}	\N	2020-10-12 15:51:06.346697
63173	PCOM	2020-10-06	9.805	9.851	9.71	9.75	7276	{}	\N	2020-10-12 15:51:06.351154
63174	PCOM	2020-10-07	9.74	9.76	9.38	9.59	46235	{}	\N	2020-10-12 15:51:06.354876
63175	PCOM	2020-10-08	9.56	9.71	9.5	9.66	49580	{}	\N	2020-10-12 15:51:06.359982
63176	PCOM	2020-10-09	9.8	9.89	9.61	9.78	60319	{}	\N	2020-10-12 15:51:06.365544
63177	PCOM	2020-10-12	9.73	10.17	9.675	10.0901	10767	{}	\N	2020-10-12 15:51:06.369934
63178	PDCE	2020-09-28	12.1	12.48	11.71	12.1	1861610	{}	\N	2020-10-12 15:51:07.354575
63179	PDCE	2020-09-29	12	12.34	11.7	12.32	1525171	{}	\N	2020-10-12 15:51:07.362704
63180	PDCE	2020-09-30	12.45	12.75	12.235	12.395	2118332	{}	\N	2020-10-12 15:51:07.367155
63181	PDCE	2020-10-01	12.23	12.25	11.365	11.51	1877135	{}	\N	2020-10-12 15:51:07.3722
63182	PDCE	2020-10-02	11.03	11.84	10.86	11.7	1172712	{}	\N	2020-10-12 15:51:07.377948
63183	PDCE	2020-10-05	12	12.46	11.87	12.43	936511	{}	\N	2020-10-12 15:51:07.382755
63184	PDCE	2020-10-06	12.739	13.21	12.11	12.18	1497208	{}	\N	2020-10-12 15:51:07.387376
63185	PDCE	2020-10-07	12.27	12.64	11.99	12.395	981653	{}	\N	2020-10-12 15:51:07.392182
63186	PDCE	2020-10-08	12.54	13.06	12.265	12.9	1522795	{}	\N	2020-10-12 15:51:07.397653
63187	PDCE	2020-10-09	13.08	13.18	12.49	12.5	1293500	{}	\N	2020-10-12 15:51:07.402455
63188	PDCE	2020-10-12	12.49	12.5	12.02	12.46	310966	{}	\N	2020-10-12 15:51:07.406594
63189	PDS	2020-09-28	0.6399	0.6574	0.63	0.64	124978	{}	\N	2020-10-12 15:51:09.025918
63190	PDS	2020-09-29	0.63	0.64	0.623	0.6326	477024	{}	\N	2020-10-12 15:51:09.033594
63191	PDS	2020-09-30	0.6326	0.6608	0.6285	0.64	859157	{}	\N	2020-10-12 15:51:09.039328
63192	PDS	2020-10-01	0.63	0.6469	0.62	0.6268	568806	{}	\N	2020-10-12 15:51:09.045367
63193	PDS	2020-10-02	0.6112	0.6439	0.5808	0.6311	1090673	{}	\N	2020-10-12 15:51:09.050217
63194	PDS	2020-10-05	0.6357	0.686	0.6357	0.6691	658035	{}	\N	2020-10-12 15:51:09.054685
63195	PDS	2020-10-06	0.68	0.6923	0.6602	0.6901	902916	{}	\N	2020-10-12 15:51:09.058915
63196	PDS	2020-10-07	0.7	0.704	0.6626	0.6949	725456	{}	\N	2020-10-12 15:51:09.065239
63197	PDS	2020-10-08	0.69	0.7499	0.6858	0.727	909007	{}	\N	2020-10-12 15:51:09.071572
63198	PDS	2020-10-09	0.74	0.7539	0.714456	0.7456	539958	{}	\N	2020-10-12 15:51:09.081059
63199	PDS	2020-10-12	0.7489	0.75	0.73	0.7311	84822	{}	\N	2020-10-12 15:51:09.086436
63200	PEI	2020-09-28	0.5396	0.581	0.5396	0.5588	1804933	{}	\N	2020-10-12 15:51:10.027903
63201	PEI	2020-09-29	0.5442	0.56	0.5255	0.5365	2230709	{}	\N	2020-10-12 15:51:10.037573
63202	PEI	2020-09-30	0.5416	0.5968	0.54	0.5539	3134013	{}	\N	2020-10-12 15:51:10.049613
63203	PEI	2020-10-01	0.5684	0.5999	0.5511	0.5601	3399600	{}	\N	2020-10-12 15:51:10.058589
63204	PEI	2020-10-02	0.543	0.557	0.5311	0.5404	2238297	{}	\N	2020-10-12 15:51:10.068033
63205	PEI	2020-10-05	0.538	0.565	0.538	0.5535	1235090	{}	\N	2020-10-12 15:51:10.07573
63206	PEI	2020-10-06	0.56	0.6159	0.5555	0.5746	5451498	{}	\N	2020-10-12 15:51:10.081812
63207	PEI	2020-10-07	0.5885	0.6097	0.58	0.5893	2792588	{}	\N	2020-10-12 15:51:10.087276
63208	PEI	2020-10-08	0.629	0.63	0.6022	0.6142	2101024	{}	\N	2020-10-12 15:51:10.092248
63209	PEI	2020-10-09	0.6401	0.678099	0.615	0.62	2668657	{}	\N	2020-10-12 15:51:10.098485
63210	PEI	2020-10-12	0.6309	0.6499	0.6232	0.63	720665	{}	\N	2020-10-12 15:51:10.104131
63211	PFD	2020-09-28	16.35	16.5907	16.0275	16.0445	49274	{}	\N	2020-10-12 15:51:12.081668
63212	PFD	2020-09-29	16.19	16.255	16.105	16.25	29914	{}	\N	2020-10-12 15:51:12.087828
63213	PFD	2020-09-30	16.33	16.72	16.195	16.505	44422	{}	\N	2020-10-12 15:51:12.093618
63214	PFD	2020-10-01	16.4	16.9399	16.01	16.1	72452	{}	\N	2020-10-12 15:51:12.101503
63215	PFD	2020-10-02	16.43	16.6	16.13	16.6	70932	{}	\N	2020-10-12 15:51:12.110184
63216	PFD	2020-10-05	16.52	17	16.5	16.515	65656	{}	\N	2020-10-12 15:51:12.11875
63217	PFD	2020-10-06	16.78	16.78	16.5	16.616	59954	{}	\N	2020-10-12 15:51:12.124963
63218	PFD	2020-10-07	16.78	16.84	16.584	16.62	59076	{}	\N	2020-10-12 15:51:12.129863
63219	PFD	2020-10-08	16.8	17	16.72	16.9936	39660	{}	\N	2020-10-12 15:51:12.136387
63220	PFD	2020-10-09	16.88	17.19	16.88	16.9	63394	{}	\N	2020-10-12 15:51:12.140842
63221	PFD	2020-10-12	16.87	17.54	16.8284	17.54	62954	{}	\N	2020-10-12 15:51:12.14546
63222	PFLT	2020-09-28	8.34	8.71	8.3224	8.605	193100	{}	\N	2020-10-12 15:51:13.128385
63223	PFLT	2020-09-29	8.57	8.66	8.37	8.46	180624	{}	\N	2020-10-12 15:51:13.135027
63224	PFLT	2020-09-30	8.51	8.64	8.4173	8.44	165228	{}	\N	2020-10-12 15:51:13.139836
63225	PFLT	2020-10-01	8.52	8.56	8.45	8.54	132360	{}	\N	2020-10-12 15:51:13.144984
63226	PFLT	2020-10-02	8.35	8.7	8.34	8.63	110542	{}	\N	2020-10-12 15:51:13.150682
63227	PFLT	2020-10-05	8.7	8.77	8.64	8.67	93317	{}	\N	2020-10-12 15:51:13.156814
63228	PFLT	2020-10-06	8.73	8.79	8.61	8.67	112802	{}	\N	2020-10-12 15:51:13.162201
63229	PFLT	2020-10-07	8.72	8.78	8.6	8.68	101608	{}	\N	2020-10-12 15:51:13.168348
63230	PFLT	2020-10-08	8.75	8.79	8.72	8.77	71318	{}	\N	2020-10-12 15:51:13.173526
63231	PFLT	2020-10-09	8.77	8.81	8.69	8.7	114714	{}	\N	2020-10-12 15:51:13.178792
63232	PFLT	2020-10-12	8.71	8.7599	8.67	8.679	35503	{}	\N	2020-10-12 15:51:13.183726
63233	PFSI	2020-09-28	57.02	59.08	56.5	58.5	625255	{}	\N	2020-10-12 15:51:14.107732
63234	PFSI	2020-09-29	58.22	58.72	57.741	58.35	540651	{}	\N	2020-10-12 15:51:14.113603
63235	PFSI	2020-09-30	58.35	59.38	57.93	58.12	1377909	{}	\N	2020-10-12 15:51:14.119553
63236	PFSI	2020-10-01	58.86	59.35	56.81	58.73	1230460	{}	\N	2020-10-12 15:51:14.124533
63237	PFSI	2020-10-02	57.01	60.4	57	59.96	1054811	{}	\N	2020-10-12 15:51:14.130805
63238	PFSI	2020-10-05	60.61	62.03	59.76	60.4	1186163	{}	\N	2020-10-12 15:51:14.137316
63239	PFSI	2020-10-06	61.1	62.08	59.02	59.12	1239558	{}	\N	2020-10-12 15:51:14.142592
63240	PFSI	2020-10-07	59.96	60.8	58.34	60.04	1418419	{}	\N	2020-10-12 15:51:14.148182
63241	PFSI	2020-10-08	60.49	61.66	59.24	61.19	808703	{}	\N	2020-10-12 15:51:14.153631
63242	PFSI	2020-10-09	62.38	65.98	62.17	65.29	2359240	{}	\N	2020-10-12 15:51:14.15869
63243	PFSI	2020-10-12	65.57	65.98	64.03	65.49	449819	{}	\N	2020-10-12 15:51:14.164109
63244	PAK	2020-09-28	27	27.09	26.9201	26.9201	4404	{}	\N	2020-10-12 15:51:15.111807
63245	PAK	2020-09-29	26.82	27.16	26.82	27	2438	{}	\N	2020-10-12 15:51:15.11863
63246	PAK	2020-09-30	26.51	26.64	26.51	26.62	11158	{}	\N	2020-10-12 15:51:15.124377
63247	PAK	2020-10-01	26.5	26.8622	26.5	26.81	2921	{}	\N	2020-10-12 15:51:15.12984
63248	PAK	2020-10-02	26.57	26.61	26.49	26.5	2558	{}	\N	2020-10-12 15:51:15.136857
63249	PAK	2020-10-05	26	26	25.68	25.99	7165	{}	\N	2020-10-12 15:51:15.142391
63250	PAK	2020-10-06	26.05	26.19	25.9	26.14	7107	{}	\N	2020-10-12 15:51:15.148284
63251	PAK	2020-10-07	26.5	26.56	26.42	26.54	5330	{}	\N	2020-10-12 15:51:15.154687
63252	PAK	2020-10-08	26.81	26.92	26.722	26.92	3249	{}	\N	2020-10-12 15:51:15.162503
63253	PAK	2020-10-09	27	27.16	26.84	27.16	29124	{}	\N	2020-10-12 15:51:15.16936
63254	PAK	2020-10-12	27.13	27.15	27.1	27.15	3966	{}	\N	2020-10-12 15:51:15.174823
63255	PAVE	2020-09-28	16.85	17.06	16.85	16.95	72577	{}	\N	2020-10-12 15:51:16.577078
63256	PAVE	2020-09-29	16.9	16.99	16.7301	16.81	63115	{}	\N	2020-10-12 15:51:16.587606
63257	PAVE	2020-09-30	16.96	17.1333	16.73	16.8593	101486	{}	\N	2020-10-12 15:51:16.595062
63258	PAVE	2020-10-01	16.98	17.0191	16.7017	16.83	173023	{}	\N	2020-10-12 15:51:16.603795
63259	PAVE	2020-10-02	16.78	17.24	16.73	17.17	67492	{}	\N	2020-10-12 15:51:16.613913
63260	PAVE	2020-10-05	17.23	17.65	17.23	17.6	127656	{}	\N	2020-10-12 15:51:16.62277
63261	PAVE	2020-10-06	17.73	17.94	17.45	17.46	94411	{}	\N	2020-10-12 15:51:16.628937
63262	PAVE	2020-10-07	17.71	17.99	17.71	17.96	136436	{}	\N	2020-10-12 15:51:16.634981
63263	PAVE	2020-10-08	18.19	18.19	17.95	18.16	266148	{}	\N	2020-10-12 15:51:16.640878
63264	PAVE	2020-10-09	18.34	18.3499	18.1123	18.19	1086099	{}	\N	2020-10-12 15:51:16.646544
63265	PAVE	2020-10-12	18.41	18.41	18.24	18.37	158486	{}	\N	2020-10-12 15:51:16.65206
63266	PBCT	2020-09-28	10.1	10.315	10.04	10.23	2988764	{}	\N	2020-10-12 15:51:17.617762
63267	PBCT	2020-09-29	10.23	10.24	9.975	10.16	2794370	{}	\N	2020-10-12 15:51:17.625707
63268	PBCT	2020-09-30	10.2	10.43	10.18	10.31	3487540	{}	\N	2020-10-12 15:51:17.630521
63269	PBCT	2020-10-01	10.29	10.3953	10.08	10.16	2431213	{}	\N	2020-10-12 15:51:17.636004
63270	PBCT	2020-10-02	10.03	10.5856	9.98	10.53	3361745	{}	\N	2020-10-12 15:51:17.641862
63271	PBCT	2020-10-05	10.61	10.94	10.61	10.77	3399418	{}	\N	2020-10-12 15:51:17.646723
63272	PBCT	2020-10-06	10.92	11.2	10.71	10.78	4828172	{}	\N	2020-10-12 15:51:17.652316
63273	PBCT	2020-10-07	10.91	11.18	10.88	10.97	3264806	{}	\N	2020-10-12 15:51:17.657882
63274	PBCT	2020-10-08	11.03	11.1	10.91	11.02	2650209	{}	\N	2020-10-12 15:51:17.663883
63275	PBCT	2020-10-09	11.09	11.13	10.8	10.83	3001321	{}	\N	2020-10-12 15:51:17.669453
63276	PBCT	2020-10-12	10.82	10.95	10.76	10.915	951543	{}	\N	2020-10-12 15:51:17.675037
63277	PBS	2020-09-28	36.1	36.21	35.9934	36.21	3911	{}	\N	2020-10-12 15:51:20.520214
63278	PBS	2020-09-29	36.14	36.14	35.88	36.06	3636	{}	\N	2020-10-12 15:51:20.528567
63279	PBS	2020-09-30	36.45	36.45	36.061	36.25	1088	{}	\N	2020-10-12 15:51:20.534368
63280	PBS	2020-10-01	36.65	37.06	36.63	37.045	4327	{}	\N	2020-10-12 15:51:20.542434
63281	PBS	2020-10-02	36.6	36.91	36.6	36.75	13907	{}	\N	2020-10-12 15:51:20.547692
63282	PBS	2020-10-05	36.84	37.3762	36.84	37.3403	5159	{}	\N	2020-10-12 15:51:20.552229
63283	PBS	2020-10-06	37.69	37.69	36.701	36.701	4599	{}	\N	2020-10-12 15:51:20.558339
63284	PBS	2020-10-07	37.0763	37.185	37.0763	37.185	2206	{}	\N	2020-10-12 15:51:20.562952
63285	PBS	2020-10-08	37.1692	37.3	37.12	37.3	4873	{}	\N	2020-10-12 15:51:20.56734
63286	PBS	2020-10-09	37.38	37.51	37.3215	37.3215	8093	{}	\N	2020-10-12 15:51:20.572465
63287	PCH	2020-09-28	40.54	41.8	40.33	41.8	423477	{}	\N	2020-10-12 15:51:21.5033
63288	PCH	2020-09-29	41.58	42.09	41.23	41.8	570516	{}	\N	2020-10-12 15:51:21.510491
63289	PCH	2020-09-30	42.1	42.71	41.71	42.1	425181	{}	\N	2020-10-12 15:51:21.515633
63290	PCH	2020-10-01	43.49	43.73	42.38	43.295	663230	{}	\N	2020-10-12 15:51:21.521121
63291	PCH	2020-10-02	42.56	44.14	42.47	43.83	557218	{}	\N	2020-10-12 15:51:21.528168
63292	PCH	2020-10-05	44.19	45.59	43.66	45.53	435323	{}	\N	2020-10-12 15:51:21.534155
63293	PCH	2020-10-06	45.76	46.05	44.75	45.185	484305	{}	\N	2020-10-12 15:51:21.541681
63294	PCH	2020-10-07	45.71	46.46	45.24	45.53	613409	{}	\N	2020-10-12 15:51:21.548681
63295	PCH	2020-10-08	45.93	46.65	45.38	46.64	650382	{}	\N	2020-10-12 15:51:21.560825
63296	PCH	2020-10-09	46.75	46.75	45.98	46.61	405213	{}	\N	2020-10-12 15:51:21.569645
63297	PCH	2020-10-12	46.74	47.35	46.01	47.265	68717	{}	\N	2020-10-12 15:51:21.576234
63298	PDBC	2020-09-28	13.65	13.729	13.58	13.7	523426	{}	\N	2020-10-12 15:51:22.498779
63299	PDBC	2020-09-29	13.6	13.63	13.44	13.5	766443	{}	\N	2020-10-12 15:51:22.504403
63300	PDBC	2020-09-30	13.54	13.72	13.53	13.67	2944059	{}	\N	2020-10-12 15:51:22.510178
63301	PDBC	2020-10-01	13.59	13.59	13.375	13.49	1761159	{}	\N	2020-10-12 15:51:22.518709
63302	PDBC	2020-10-02	13.24	13.42	13.24	13.3	5736457	{}	\N	2020-10-12 15:51:22.531461
63303	PDBC	2020-10-05	13.54	13.71	13.54	13.67	1560560	{}	\N	2020-10-12 15:51:22.53976
63304	PDBC	2020-10-06	13.8	13.89	13.695	13.7	6588988	{}	\N	2020-10-12 15:51:22.548176
63305	PDBC	2020-10-07	13.75	13.795	13.67	13.75	4010697	{}	\N	2020-10-12 15:51:22.553463
63306	PDBC	2020-10-08	13.88	13.91	13.81	13.88	1202697	{}	\N	2020-10-12 15:51:22.559216
63307	PDBC	2020-10-09	13.93	14.0076	13.9	13.95	1328797	{}	\N	2020-10-12 15:51:22.564524
63308	PDBC	2020-10-12	13.91	13.9599	13.79	13.8	235724	{}	\N	2020-10-12 15:51:22.569818
63309	PEBK	2020-09-28	15.63	16.15	15.43	15.99	4528	{}	\N	2020-10-12 15:51:24.564059
63310	PEBK	2020-09-29	15.65	15.88	15.43	15.75	3091	{}	\N	2020-10-12 15:51:24.569606
63311	PEBK	2020-09-30	15.66	16.1	15.43	15.43	2939	{}	\N	2020-10-12 15:51:24.574665
63312	PEBK	2020-10-01	15.58	16.335	15.47	15.9	4759	{}	\N	2020-10-12 15:51:24.580912
63313	PEBK	2020-10-02	16.5	16.5	15.96	15.96	3517	{}	\N	2020-10-12 15:51:24.588458
63314	PEBK	2020-10-05	16.34	16.38	16.021	16.05	2694	{}	\N	2020-10-12 15:51:24.593761
63315	PEBK	2020-10-06	15.7005	16.5	15.7005	16.05	7736	{}	\N	2020-10-12 15:51:24.59975
63316	PEBK	2020-10-07	16.21	16.5	16.18	16.5	3446	{}	\N	2020-10-12 15:51:24.604951
63317	PEBK	2020-10-08	16.5	16.5	16.2	16.4	5186	{}	\N	2020-10-12 15:51:24.610722
63318	PEBK	2020-10-09	16.25	16.49	16.25	16.49	857	{}	\N	2020-10-12 15:51:24.616434
63319	PEBK	2020-10-12	16.49	16.95	16.49	16.9	3325	{}	\N	2020-10-12 15:51:24.621836
63320	PEN	2020-09-28	197.16	197.31	191.7729	194.46	404675	{}	\N	2020-10-12 15:51:25.888961
63321	PEN	2020-09-29	198.18	199.51	193.11	193.45	679445	{}	\N	2020-10-12 15:51:25.896182
63322	PEN	2020-09-30	193.81	198.16	192.86	194.38	830306	{}	\N	2020-10-12 15:51:25.903427
63323	PEN	2020-10-01	196.06	198.18	190.37	190.4	782932	{}	\N	2020-10-12 15:51:25.91004
63324	PEN	2020-10-02	188.01	189.73	183.55	186.92	842065	{}	\N	2020-10-12 15:51:25.916987
63325	PEN	2020-10-05	188.26	191.25	186.92	190	785285	{}	\N	2020-10-12 15:51:25.922565
63326	PEN	2020-10-06	191.24	192.985	187.85	188.98	575046	{}	\N	2020-10-12 15:51:25.92825
63327	PEN	2020-10-07	191.98	199.75	191.95	197.9	973066	{}	\N	2020-10-12 15:51:25.934341
63328	PEN	2020-10-08	199.9	208.22	197.65	207.98	800605	{}	\N	2020-10-12 15:51:25.940049
63329	PEN	2020-10-09	210.41	216.38	206.03	216.06	1000337	{}	\N	2020-10-12 15:51:25.946166
63330	PEN	2020-10-12	216.8	217.49	213.03	214	280494	{}	\N	2020-10-12 15:51:25.951929
63331	PFFD	2020-09-28	24.78	24.8899	24.7757	24.84	223675	{}	\N	2020-10-12 15:51:27.820595
63332	PFFD	2020-09-29	24.89	24.89	24.78	24.85	162547	{}	\N	2020-10-12 15:51:27.827181
63333	PFFD	2020-09-30	24.83	24.909	24.73	24.76	234705	{}	\N	2020-10-12 15:51:27.833024
63334	PFFD	2020-10-01	24.8	24.94	24.78	24.94	742956	{}	\N	2020-10-12 15:51:27.839035
63335	PFFD	2020-10-02	24.86	24.97	24.82	24.93	187029	{}	\N	2020-10-12 15:51:27.844909
63336	PFFD	2020-10-05	24.9	24.99	24.88	24.97	192292	{}	\N	2020-10-12 15:51:27.850956
63337	PFFD	2020-10-06	24.99	25.1	24.975	25.01	155544	{}	\N	2020-10-12 15:51:27.855991
63338	PFFD	2020-10-07	25.08	25.11	25.06	25.11	176047	{}	\N	2020-10-12 15:51:27.861215
63339	PFFD	2020-10-08	25.17	25.18	25.1201	25.15	303500	{}	\N	2020-10-12 15:51:27.866469
63340	PFFD	2020-10-09	25.23	25.23	25.1199	25.17	204740	{}	\N	2020-10-12 15:51:27.872638
63341	PFFD	2020-10-12	25.25	25.25	25.14	25.187	59581	{}	\N	2020-10-12 15:51:27.878659
63342	PFIS	2020-09-28	34.15	36.39	34.1	35.09	14417	{}	\N	2020-10-12 15:51:30.626606
63343	PFIS	2020-09-29	35.39	35.89	35	35.89	4186	{}	\N	2020-10-12 15:51:30.632496
63344	PFIS	2020-09-30	35.55	36.46	34.27	34.76	9254	{}	\N	2020-10-12 15:51:30.637654
63345	PFIS	2020-10-01	35.37	35.42	33.77	34.47	10786	{}	\N	2020-10-12 15:51:30.642645
63346	PFIS	2020-10-02	34.51	35.7	34.51	34.97	12081	{}	\N	2020-10-12 15:51:30.647684
63347	PFIS	2020-10-05	35.49	37.035	35.49	36.28	12171	{}	\N	2020-10-12 15:51:30.652949
63348	PFIS	2020-10-06	36.75	36.87	35.25	35.45	4023	{}	\N	2020-10-12 15:51:30.658059
63349	PFIS	2020-10-07	36.01	37.6	35.6	37.6	5431	{}	\N	2020-10-12 15:51:30.663284
63350	PFIS	2020-10-08	37.8	38.19	37.01	37.14	8385	{}	\N	2020-10-12 15:51:30.669274
63351	PFIS	2020-10-09	37.76	37.76	37.03	37.68	2226	{}	\N	2020-10-12 15:51:30.674962
63352	PFSW	2020-09-28	6.43	6.79	6.2701	6.62	38601	{}	\N	2020-10-12 15:51:31.721335
63353	PFSW	2020-09-29	6.63	6.635	6.265	6.39	232844	{}	\N	2020-10-12 15:51:31.73064
63354	PFSW	2020-09-30	6.53	6.99	6.37	6.69	83437	{}	\N	2020-10-12 15:51:31.736456
63355	PFSW	2020-10-01	6.74	6.78	6.41	6.42	145175	{}	\N	2020-10-12 15:51:31.74213
63356	PFSW	2020-10-02	6.25	6.59	6.09	6.35	70260	{}	\N	2020-10-12 15:51:31.747075
63357	PFSW	2020-10-05	6.49	6.58	6.29	6.55	181948	{}	\N	2020-10-12 15:51:31.753178
63358	PFSW	2020-10-06	6.59	7.19	6.525	7.1	91276	{}	\N	2020-10-12 15:51:31.758869
63359	PFSW	2020-10-07	7.17	7.58	7	7.5	73590	{}	\N	2020-10-12 15:51:31.765729
63360	PFSW	2020-10-08	7.64	7.65	7.15	7.51	60709	{}	\N	2020-10-12 15:51:31.77207
63361	PFSW	2020-10-09	7.6	8.23	7.39	8.16	95454	{}	\N	2020-10-12 15:51:31.778047
63362	PFSW	2020-10-12	8.27	8.31	7.9013	8.04	47269	{}	\N	2020-10-12 15:51:31.783596
63363	PALL	2020-09-28	210	213.81	210	213.81	41014	{}	\N	2020-10-12 15:51:32.702005
63364	PALL	2020-09-29	214.01	218.2	213.49	217.24	27462	{}	\N	2020-10-12 15:51:32.71071
63365	PALL	2020-09-30	218.5	219.995	215.57	216.51	30247	{}	\N	2020-10-12 15:51:32.718612
63366	PALL	2020-10-01	219.36	219.62	217.1701	218.65	23963	{}	\N	2020-10-12 15:51:32.727011
63367	PALL	2020-10-02	217.95	217.96	216.01	217	15945	{}	\N	2020-10-12 15:51:32.732731
63368	PALL	2020-10-05	219.6	223	219.6	221.56	25437	{}	\N	2020-10-12 15:51:32.738432
63369	PALL	2020-10-06	223.2	224	220	220	30326	{}	\N	2020-10-12 15:51:32.743856
63370	PALL	2020-10-07	221.85	222	220.708	221.49	6638	{}	\N	2020-10-12 15:51:32.749345
63371	PALL	2020-10-08	223.37	226	221.56	224.51	12615	{}	\N	2020-10-12 15:51:32.755503
63372	PALL	2020-10-09	228.21	232.4	228.21	230.46	33618	{}	\N	2020-10-12 15:51:32.761014
63373	PALL	2020-10-12	230.31	230.44	224.82	225.3653	18594	{}	\N	2020-10-12 15:51:32.767903
63374	PAVMW	2020-10-01	1.7	1.7675	1.7	1.7	3599	{}	\N	2020-10-12 15:51:33.81758
63375	PAVMW	2020-10-08	0.91	0.91	0.91	0.91	109	{}	\N	2020-10-12 15:51:33.824183
63376	PBJ	2020-09-28	32.54	32.71	32.54	32.65	8456	{}	\N	2020-10-12 15:51:34.728744
63377	PBJ	2020-09-29	32.55	32.58	32.3911	32.5	6426	{}	\N	2020-10-12 15:51:34.733694
63378	PBJ	2020-09-30	32.5	32.8	32.5	32.7162	2036	{}	\N	2020-10-12 15:51:34.738915
63379	PBJ	2020-10-01	32.92	33.0518	32.6083	33.0518	18751	{}	\N	2020-10-12 15:51:34.743337
63380	PBJ	2020-10-02	32.8313	33.0604	32.8313	33.0535	4866	{}	\N	2020-10-12 15:51:34.748251
63381	PBJ	2020-10-05	33.215	33.3676	33.165	33.3676	2424	{}	\N	2020-10-12 15:51:34.753634
63382	PBJ	2020-10-06	33.5	33.52	33.15	33.23	5281	{}	\N	2020-10-12 15:51:34.760052
63383	PBJ	2020-10-07	33.3741	33.5264	33.355	33.5264	3649	{}	\N	2020-10-12 15:51:34.765462
63384	PBJ	2020-10-08	33.74	33.75	33.55	33.64	269700	{}	\N	2020-10-12 15:51:34.770673
63385	PBJ	2020-10-09	33.94	34.31	33.9248	34.2	3109	{}	\N	2020-10-12 15:51:34.776128
63386	PBJ	2020-10-12	34.4299	34.6	34.3511	34.6	3739	{}	\N	2020-10-12 15:51:34.781064
63387	PCK	2020-09-28	9.33	9.33	9.18	9.21	24025	{}	\N	2020-10-12 15:51:36.531705
63388	PCK	2020-09-29	9.33	9.33	9.2384	9.2595	22990	{}	\N	2020-10-12 15:51:36.53816
63389	PCK	2020-09-30	9.3	9.32	9.265	9.32	49614	{}	\N	2020-10-12 15:51:36.544686
63390	PCK	2020-10-01	9.34	9.34	9.3	9.32	71822	{}	\N	2020-10-12 15:51:36.550494
63391	PCK	2020-10-02	9.3	9.35	9.26	9.33	88666	{}	\N	2020-10-12 15:51:36.558706
63392	PCK	2020-10-05	9.31	9.35	9.25	9.27	67894	{}	\N	2020-10-12 15:51:36.564761
63393	PCK	2020-10-06	9.25	9.3248	9.16	9.19	78622	{}	\N	2020-10-12 15:51:36.570986
63394	PCK	2020-10-07	9.22	9.302	9.22	9.3	36892	{}	\N	2020-10-12 15:51:36.577225
63395	PCK	2020-10-08	9.37	9.37	9.29	9.295	34560	{}	\N	2020-10-12 15:51:36.582986
63396	PCK	2020-10-09	9.37	9.39	9.22	9.2588	49428	{}	\N	2020-10-12 15:51:36.588959
63397	PCK	2020-10-12	9.25	9.33	9.245	9.25	23914	{}	\N	2020-10-12 15:51:36.595161
63398	PCTI	2020-09-28	5.63	5.89	5.6	5.83	57122	{}	\N	2020-10-12 15:51:37.629154
63399	PCTI	2020-09-29	5.86	5.93	5.69	5.85	47152	{}	\N	2020-10-12 15:51:37.636623
63400	PCTI	2020-09-30	5.85	5.855	5.61	5.66	43437	{}	\N	2020-10-12 15:51:37.6444
63401	PCTI	2020-10-01	5.7	5.78	5.63	5.71	59710	{}	\N	2020-10-12 15:51:37.654237
63402	PCTI	2020-10-02	5.61	5.74	5.59	5.62	33827	{}	\N	2020-10-12 15:51:37.662856
63403	PCTI	2020-10-05	5.69	5.77	5.24	5.36	307993	{}	\N	2020-10-12 15:51:37.669226
63404	PCTI	2020-10-06	5.37	6.03	5.37	5.68	320167	{}	\N	2020-10-12 15:51:37.67594
63405	PCTI	2020-10-07	5.75	5.94	5.75	5.9	79108	{}	\N	2020-10-12 15:51:37.683213
63406	PCTI	2020-10-08	5.95	5.98	5.81	5.88	65783	{}	\N	2020-10-12 15:51:37.689204
63407	PCTI	2020-10-09	5.89	5.9505	5.79	5.84	53134	{}	\N	2020-10-12 15:51:37.695151
63408	PCTI	2020-10-12	5.79	5.91	5.75	5.8547	24939	{}	\N	2020-10-12 15:51:37.700947
63409	PDM	2020-09-28	13.89	14.22	13.86	14.02	673255	{}	\N	2020-10-12 15:51:38.813038
63410	PDM	2020-09-29	13.95	13.95	13.48	13.76	1384713	{}	\N	2020-10-12 15:51:38.81956
63411	PDM	2020-09-30	13.86	14.01	13.5	13.57	3471850	{}	\N	2020-10-12 15:51:38.825558
63412	PDM	2020-10-01	13.6	13.87	13.45	13.86	1826018	{}	\N	2020-10-12 15:51:38.831676
63413	PDM	2020-10-02	13.61	14.47	13.6	14.47	1405081	{}	\N	2020-10-12 15:51:38.837439
63414	PDM	2020-10-05	14.53	14.64	14.16	14.36	1301320	{}	\N	2020-10-12 15:51:38.843599
63415	PDM	2020-10-06	14.49	14.61	14.16	14.22	1553731	{}	\N	2020-10-12 15:51:38.850757
63416	PDM	2020-10-07	14.34	14.34	13.955	14.03	1254727	{}	\N	2020-10-12 15:51:38.856696
63417	PDM	2020-10-08	14.15	14.36	14.13	14.3	1107132	{}	\N	2020-10-12 15:51:38.863333
63418	PDM	2020-10-09	14.44	14.44	14.06	14.08	655544	{}	\N	2020-10-12 15:51:38.869357
63419	PDM	2020-10-12	14.08	14.25	13.935	14.25	183224	{}	\N	2020-10-12 15:51:38.875774
63420	PED	2020-09-28	1.12	1.23	1.03	1.22	375033	{}	\N	2020-10-12 15:51:42.219351
63421	PED	2020-09-29	1.76	2.325	1.37	1.55	18490044	{}	\N	2020-10-12 15:51:42.22902
63422	PED	2020-09-30	1.26	1.51	1.26	1.49	1089963	{}	\N	2020-10-12 15:51:42.237512
63423	PED	2020-10-01	1.35	1.52	1.35	1.52	243428	{}	\N	2020-10-12 15:51:42.244195
63424	PED	2020-10-02	1.46	1.75	1.35	1.75	431268	{}	\N	2020-10-12 15:51:42.250788
63425	PED	2020-10-05	1.37	1.7	1.37	1.54	278402	{}	\N	2020-10-12 15:51:42.25538
63426	PED	2020-10-06	1.61	1.61	1.44	1.45	161238	{}	\N	2020-10-12 15:51:42.259751
63427	PED	2020-10-07	1.41	1.54	1.41	1.49	166066	{}	\N	2020-10-12 15:51:42.264932
63428	PED	2020-10-08	1.53	1.75	1.48	1.61	1064370	{}	\N	2020-10-12 15:51:42.270681
63429	PED	2020-10-09	1.61	1.66	1.42	1.45	634180	{}	\N	2020-10-12 15:51:42.276607
63430	PED	2020-10-12	1.46	1.4999	1.4	1.4154	55407	{}	\N	2020-10-12 15:51:42.283673
63431	PETS	2020-09-28	31.41	31.88	30.75	31.67	336678	{}	\N	2020-10-12 15:51:43.219036
63432	PETS	2020-09-29	31.59	32.225	31.14	31.38	300488	{}	\N	2020-10-12 15:51:43.224928
63433	PETS	2020-09-30	31.54	32.21	31.33	31.62	304753	{}	\N	2020-10-12 15:51:43.230383
63434	PETS	2020-10-01	31.855	31.855	30.38	30.52	475140	{}	\N	2020-10-12 15:51:43.236218
63435	PETS	2020-10-02	29.97	30.58	29.52	30.39	668423	{}	\N	2020-10-12 15:51:43.241575
63436	PETS	2020-10-05	30.7	31.63	30.32	31.56	465774	{}	\N	2020-10-12 15:51:43.246849
63437	PETS	2020-10-06	31.16	32.2515	31.04	31.19	385739	{}	\N	2020-10-12 15:51:43.252227
63438	PETS	2020-10-07	31.49	32.18	31.11	31.78	331649	{}	\N	2020-10-12 15:51:43.258632
63439	PETS	2020-10-08	32.02	32.72	31.835	32.71	521342	{}	\N	2020-10-12 15:51:43.266218
63440	PETS	2020-10-09	32.99	33.76	31.91	32.06	639782	{}	\N	2020-10-12 15:51:43.275438
63441	PETS	2020-10-12	32.36	33.24	31.91	32.85	191569	{}	\N	2020-10-12 15:51:43.283758
63442	PFFR	2020-09-28	22.1	22.15	21.9501	22.135	5546	{}	\N	2020-10-12 15:51:45.310009
63443	PFFR	2020-09-29	22.24	22.24	22.04	22.04	9765	{}	\N	2020-10-12 15:51:45.316144
63444	PFFR	2020-09-30	22.1864	22.1864	22.0169	22.0169	8029	{}	\N	2020-10-12 15:51:45.321807
63445	PFFR	2020-10-01	21.91	22.17	21.91	22.0709	5735	{}	\N	2020-10-12 15:51:45.327424
63446	PFFR	2020-10-02	22.04	22.125	22.01	22.1	13198	{}	\N	2020-10-12 15:51:45.333454
63447	PFFR	2020-10-05	22.1	22.2199	22.1	22.14	10872	{}	\N	2020-10-12 15:51:45.338725
63448	PFFR	2020-10-06	22.11	22.25	22.11	22.21	24380	{}	\N	2020-10-12 15:51:45.34573
63449	PFFR	2020-10-07	22.22	22.32	22.16	22.16	11531	{}	\N	2020-10-12 15:51:45.351659
63450	PFFR	2020-10-08	22.2325	22.34	22.2325	22.2997	5450	{}	\N	2020-10-12 15:51:45.357712
63451	PFFR	2020-10-09	22.3	22.38	22.25	22.3	8214	{}	\N	2020-10-12 15:51:45.363159
63452	PFFR	2020-10-12	22.35	22.35	22.27	22.27	4701	{}	\N	2020-10-12 15:51:45.368381
63453	PFNX	2020-09-28	12.69	12.84	12.69	12.8	1306275	{}	\N	2020-10-12 15:51:46.20901
63454	PFNX	2020-09-29	12.86	12.86	12.73	12.77	4075213	{}	\N	2020-10-12 15:51:46.214809
63455	PFNX	2020-09-30	12.74	12.94	12.74	12.76	1149818	{}	\N	2020-10-12 15:51:46.220595
63456	PGAL	2020-09-28	8.9882	9.05	8.9828	8.9828	1145	{}	\N	2020-10-12 15:51:47.412611
63457	PGAL	2020-09-29	8.96	9.01	8.9242	8.9242	2408	{}	\N	2020-10-12 15:51:47.419984
63458	PGAL	2020-09-30	8.96	8.975	8.91	8.975	34612	{}	\N	2020-10-12 15:51:47.425801
63459	PGAL	2020-10-01	8.98	9.18	8.93	9.03	241414	{}	\N	2020-10-12 15:51:47.432904
63460	PGAL	2020-10-02	9.03	9.04	8.92	8.93	7561	{}	\N	2020-10-12 15:51:47.446652
63461	PGAL	2020-10-05	9.07	9.18	9.07	9.13	6985	{}	\N	2020-10-12 15:51:47.456586
63462	PGAL	2020-10-06	9.2909	9.2909	9.17	9.18	3480	{}	\N	2020-10-12 15:51:47.465487
63463	PGAL	2020-10-07	9.18	9.265	9.18	9.26	5403	{}	\N	2020-10-12 15:51:47.474165
63464	PGAL	2020-10-08	9.255	9.325	9.25	9.32	3801	{}	\N	2020-10-12 15:51:47.479826
63465	PGAL	2020-10-09	9.2556	9.35	9.2556	9.28	4554	{}	\N	2020-10-12 15:51:47.488353
63466	PGAL	2020-10-12	9.31	9.31	9.31	9.31	404	{}	\N	2020-10-12 15:51:47.49478
63467	PGJ	2020-09-28	52.12	52.24	51.585	52.24	6248	{}	\N	2020-10-12 15:51:48.3794
63468	PGJ	2020-09-29	51.95	51.95	51.45	51.79	9907	{}	\N	2020-10-12 15:51:48.386139
63469	PGJ	2020-09-30	51.52	52.7792	51.52	52.68	8205	{}	\N	2020-10-12 15:51:48.392852
63470	PGJ	2020-10-01	52.97	53.37	52.69	53.32	7371	{}	\N	2020-10-12 15:51:48.399269
63471	PGJ	2020-10-02	52	52.99	52	52.45	18554	{}	\N	2020-10-12 15:51:48.406644
63472	PGJ	2020-10-05	52.87	53.13	52.71	53.13	6775	{}	\N	2020-10-12 15:51:48.4119
63473	PGJ	2020-10-06	53.08	54.06	53.02	53.68	8556	{}	\N	2020-10-12 15:51:48.416577
63474	PGJ	2020-10-07	54.3	54.475	54	54.45	7877	{}	\N	2020-10-12 15:51:48.422059
63475	PGJ	2020-10-08	54.56	54.56	54	54.33	13650	{}	\N	2020-10-12 15:51:48.427163
63476	PGJ	2020-10-09	54.15	55.1733	54.15	55.03	4882	{}	\N	2020-10-12 15:51:48.43268
63477	PGJ	2020-10-12	55.94	55.94	55.56	55.56	9194	{}	\N	2020-10-12 15:51:48.440255
63478	PARR	2020-09-28	6.71	6.7874	5.91	6.49	1159786	{}	\N	2020-10-12 15:51:49.990137
63479	PARR	2020-09-29	6.44	6.57	6.0901	6.47	1548377	{}	\N	2020-10-12 15:51:49.999385
63480	PARR	2020-09-30	6.51	6.92	6.44	6.77	1111341	{}	\N	2020-10-12 15:51:50.008506
63481	PARR	2020-10-01	6.67	6.89	6.59	6.87	757022	{}	\N	2020-10-12 15:51:50.015879
63482	PARR	2020-10-02	6.57	7.1	6.54	7.04	938204	{}	\N	2020-10-12 15:51:50.021881
63483	PARR	2020-10-05	7.17	7.29	7.01	7.15	731194	{}	\N	2020-10-12 15:51:50.02883
63484	PARR	2020-10-06	7.28	7.4	6.99	7.16	718637	{}	\N	2020-10-12 15:51:50.0339
63485	PARR	2020-10-07	7.24	7.31	6.97	7.29	704283	{}	\N	2020-10-12 15:51:50.039607
63486	PARR	2020-10-08	7.3	7.74	7.24	7.73	803699	{}	\N	2020-10-12 15:51:50.044686
63487	PARR	2020-10-09	7.69	7.77	7.29	7.49	689585	{}	\N	2020-10-12 15:51:50.049605
63488	PARR	2020-10-12	7.46	7.55	7.31	7.4	210205	{}	\N	2020-10-12 15:51:50.05503
63489	PB	2020-09-28	50.26	51.53	50.26	51.17	314740	{}	\N	2020-10-12 15:51:52.109096
63490	PB	2020-09-29	51.31	51.67	50.9	51.34	1078810	{}	\N	2020-10-12 15:51:52.115328
63491	PB	2020-09-30	52.11	52.6674	51.32	51.83	1181708	{}	\N	2020-10-12 15:51:52.120269
63492	PB	2020-10-01	51.79	52.42	50.94	51.56	906046	{}	\N	2020-10-12 15:51:52.125981
63493	PB	2020-10-02	50.82	53.36	50.43	52.98	886363	{}	\N	2020-10-12 15:51:52.131641
63494	PB	2020-10-05	53.5	55.28	53.37	54.79	1019791	{}	\N	2020-10-12 15:51:52.13623
63495	PB	2020-10-06	55.19	57.36	54.95	55.13	1164262	{}	\N	2020-10-12 15:51:52.141205
63496	PB	2020-10-07	55.64	56.96	55.62	56.14	841091	{}	\N	2020-10-12 15:51:52.147819
63497	PB	2020-10-08	56.72	57.22	56.27	56.59	752847	{}	\N	2020-10-12 15:51:52.15295
63498	PB	2020-10-09	57.15	57.3	55.8	55.94	537904	{}	\N	2020-10-12 15:51:52.158528
63499	PB	2020-10-12	55.88	56.71	55.88	56.5815	116986	{}	\N	2020-10-12 15:51:52.163605
63500	PBW	2020-09-28	57.87	59.7199	57.7999	59.68	301009	{}	\N	2020-10-12 15:51:53.733943
63501	PBW	2020-09-29	59.82	61.02	59.3	60.69	585202	{}	\N	2020-10-12 15:51:53.740776
63502	PBW	2020-09-30	61	62.01	60.41	60.78	408436	{}	\N	2020-10-12 15:51:53.748045
63503	PBW	2020-10-01	61.74	62.609	61.6	62.45	367910	{}	\N	2020-10-12 15:51:53.757896
63504	PBW	2020-10-02	60.67	63.54	60	62.54	306189	{}	\N	2020-10-12 15:51:53.766801
63505	PBW	2020-10-05	64.03	66	63.8701	65.98	454483	{}	\N	2020-10-12 15:51:53.772879
63506	PBW	2020-10-06	66.72	67.5	65.125	65.69	678093	{}	\N	2020-10-12 15:51:53.77826
63507	PBW	2020-10-07	67.18	69.56	67.18	69.31	720512	{}	\N	2020-10-12 15:51:53.784562
63508	PBW	2020-10-08	70.4	70.7299	68.34	68.85	569923	{}	\N	2020-10-12 15:51:53.789418
63509	PBW	2020-10-09	69.348	70.839	69.1	70.57	329056	{}	\N	2020-10-12 15:51:53.794633
63510	PBW	2020-10-12	71.61	71.61	69.2	69.24	273696	{}	\N	2020-10-12 15:51:53.800051
63511	PCTY	2020-09-28	153.57	159.66	153.57	158.55	353373	{}	\N	2020-10-12 15:51:56.364343
63512	PCTY	2020-09-29	159.13	163.35	158.66	162.33	316441	{}	\N	2020-10-12 15:51:56.370376
63513	PCTY	2020-09-30	161.96	167.59	159.23	161.42	574774	{}	\N	2020-10-12 15:51:56.375417
63514	PCTY	2020-10-01	162.68	165.31	161.6	162.46	431805	{}	\N	2020-10-12 15:51:56.380972
63515	PCTY	2020-10-02	167.52	167.52	162.05	164.57	506144	{}	\N	2020-10-12 15:51:56.386023
63516	PCTY	2020-10-05	166.06	173.3791	164.96	172.04	315173	{}	\N	2020-10-12 15:51:56.391094
63517	PCTY	2020-10-06	171.66	175.7	169.4	172.15	381248	{}	\N	2020-10-12 15:51:56.396676
63518	PCTY	2020-10-07	174.74	181.26	172.24	179	301377	{}	\N	2020-10-12 15:51:56.404
63519	PCTY	2020-10-08	180.84	181.71	178.42	180.65	296055	{}	\N	2020-10-12 15:51:56.410448
63520	PCTY	2020-10-09	182.9	184.69	180.2	183.94	259512	{}	\N	2020-10-12 15:51:56.416759
63521	PCTY	2020-10-12	185.63	187.57	182.4	184.67	64022	{}	\N	2020-10-12 15:51:56.422739
63522	PDI	2020-09-28	24.37	24.79	24.37	24.75	308203	{}	\N	2020-10-12 15:51:57.771745
63523	PDI	2020-09-29	24.66	25.11	24.66	25.03	655329	{}	\N	2020-10-12 15:51:57.78015
63524	PDI	2020-09-30	25.1	25.58	25.06	25.36	1742165	{}	\N	2020-10-12 15:51:57.786146
63525	PDI	2020-10-01	25.53	25.69	25.3	25.38	558973	{}	\N	2020-10-12 15:51:57.791773
63526	PDI	2020-10-02	25	25.63	24.96	25.47	692916	{}	\N	2020-10-12 15:51:57.798711
63527	PDI	2020-10-05	25.49	25.85	25.49	25.85	614862	{}	\N	2020-10-12 15:51:57.804725
63528	PDI	2020-10-06	24.39	24.85	24.39	24.68	6467036	{}	\N	2020-10-12 15:51:57.809986
63529	PDI	2020-10-07	24.76	24.841	24.65	24.75	2025888	{}	\N	2020-10-12 15:51:57.81697
63530	PDI	2020-10-08	24.85	24.85	24.74	24.79	1247152	{}	\N	2020-10-12 15:51:57.822949
63531	PDI	2020-10-09	24.66	24.81	24.61	24.69	1355520	{}	\N	2020-10-12 15:51:57.828446
63532	PDI	2020-10-12	24.75	24.89	24.7	24.865	677664	{}	\N	2020-10-12 15:51:57.834568
63533	PEP	2020-09-28	136.04	138.58	135.67	137.97	7309819	{}	\N	2020-10-12 15:52:01.080092
63534	PEP	2020-09-29	138.66	139.54	137.13	137.16	4230302	{}	\N	2020-10-12 15:52:01.086513
63535	PEP	2020-09-30	137.99	139.19	136.9	138.6	6921216	{}	\N	2020-10-12 15:52:01.091849
63536	PEP	2020-10-01	139.44	140.85	137.4866	140.8	6305198	{}	\N	2020-10-12 15:52:01.097534
63537	PEP	2020-10-02	139.28	139.91	136.2	138.06	4912656	{}	\N	2020-10-12 15:52:01.108399
63538	PEP	2020-10-05	138.11	138.38	136.98	137.93	4447424	{}	\N	2020-10-12 15:52:01.119044
63539	PEP	2020-10-06	138	138.82	135.15	135.7	4822354	{}	\N	2020-10-12 15:52:01.127769
63540	PEP	2020-10-07	135.94	137.52	135.76	137.01	4261964	{}	\N	2020-10-12 15:52:01.133201
63541	PEP	2020-10-08	137.28	138.13	136.76	137.85	2974258	{}	\N	2020-10-12 15:52:01.139622
63542	PEP	2020-10-09	138.7	139.26	137.895	138.44	3130725	{}	\N	2020-10-12 15:52:01.145163
63543	PEP	2020-10-12	141.12	142.64	139.78	142.63	2056410	{}	\N	2020-10-12 15:52:01.151734
63544	PEX	2020-09-28	26.55	26.94	26.55	26.93	1759	{}	\N	2020-10-12 15:52:02.35569
63545	PEX	2020-09-29	26.6033	26.6033	26.6033	26.6033	147	{}	\N	2020-10-12 15:52:02.361522
63546	PEX	2020-09-30	26.96	26.96	26.7789	26.7789	4186	{}	\N	2020-10-12 15:52:02.366682
63547	PEX	2020-10-01	26.78	27.15	26.78	27.15	959	{}	\N	2020-10-12 15:52:02.373529
63548	PEX	2020-10-02	26.9819	27.28	26.93	27.28	4206	{}	\N	2020-10-12 15:52:02.378901
63549	PEX	2020-10-05	27.56	27.56	27.56	27.56	482	{}	\N	2020-10-12 15:52:02.383661
63550	PEX	2020-10-06	27.63	27.63	27.3425	27.3425	323	{}	\N	2020-10-12 15:52:02.388468
63551	PEX	2020-10-07	27.61	27.61	27.43	27.55	1312	{}	\N	2020-10-12 15:52:02.394202
63552	PEX	2020-10-08	27.6785	27.75	27.675	27.675	530	{}	\N	2020-10-12 15:52:02.398611
63553	PEX	2020-10-09	27.99	27.99	27.73	27.8	18695	{}	\N	2020-10-12 15:52:02.404164
63554	PFH	2020-09-28	25.7	25.75	25.66	25.71	230734	{}	\N	2020-10-12 15:52:03.706959
63555	PFH	2020-09-29	25.79	25.79	25.6	25.6	443238	{}	\N	2020-10-12 15:52:03.713709
63556	PFH	2020-09-30	25.66	25.67	25.29	25.45	5530066	{}	\N	2020-10-12 15:52:03.718817
63557	PFH	2020-10-01	25.51	25.52	25.36	25.44	680205	{}	\N	2020-10-12 15:52:03.724532
63558	PFH	2020-10-02	25.39	25.48	25.375	25.48	357588	{}	\N	2020-10-12 15:52:03.730765
63559	PFH	2020-10-05	25.55	25.55	25.41	25.44	466434	{}	\N	2020-10-12 15:52:03.738989
63560	PFH	2020-10-06	25.46	25.49	25.33	25.34	354205	{}	\N	2020-10-12 15:52:03.746568
63561	PFH	2020-10-07	25.43	25.4509	25.35	25.35	304526	{}	\N	2020-10-12 15:52:03.755165
63562	PFH	2020-10-08	25.42	25.45	25.382	25.43	204152	{}	\N	2020-10-12 15:52:03.76408
63563	PFH	2020-10-09	25.44	25.44	25.36	25.4	267256	{}	\N	2020-10-12 15:52:03.771692
63564	PFH	2020-10-12	25.49	25.5	25.44	25.47	149141	{}	\N	2020-10-12 15:52:03.778275
63565	PFS	2020-09-28	12.34	12.57	12.29	12.42	318358	{}	\N	2020-10-12 15:52:04.762564
63566	PFS	2020-09-29	12.32	12.32	11.975	12.18	361868	{}	\N	2020-10-12 15:52:04.768751
63567	PFS	2020-09-30	12.17	12.43	12.13	12.2	620220	{}	\N	2020-10-12 15:52:04.776382
63568	PFS	2020-10-01	12.17	12.29	11.97	12.27	468898	{}	\N	2020-10-12 15:52:04.782537
63569	PFS	2020-10-02	12.11	12.85	12.1071	12.74	600882	{}	\N	2020-10-12 15:52:04.789169
63570	PFS	2020-10-05	12.78	13.1	12.78	13.03	417918	{}	\N	2020-10-12 15:52:04.795959
63571	PFS	2020-10-06	13.23	13.52	12.97	12.98	892252	{}	\N	2020-10-12 15:52:04.800934
63572	PFS	2020-10-07	13.15	13.33	12.9201	13.12	702479	{}	\N	2020-10-12 15:52:04.806811
63573	PFS	2020-10-08	13.22	13.48	13.09	13.41	465989	{}	\N	2020-10-12 15:52:04.812306
63574	PFS	2020-10-09	13.43	13.62	13.12	13.17	325726	{}	\N	2020-10-12 15:52:04.817169
63575	PFS	2020-10-12	13.05	13.31	13.04	13.24	76248	{}	\N	2020-10-12 15:52:04.821927
63576	PGM	2020-09-28	42.85	42.91	40.775	42.41	7853	{}	\N	2020-10-12 15:52:06.008246
63577	PGM	2020-09-29	43.22	43.22	41.61	41.8	2954	{}	\N	2020-10-12 15:52:06.014005
63578	PGM	2020-09-30	44.02	44.08	42.19	42.44	3408	{}	\N	2020-10-12 15:52:06.019956
63579	PGM	2020-10-01	44.57	44.57	42.4301	42.4301	2636	{}	\N	2020-10-12 15:52:06.02493
63580	PGM	2020-10-02	42.36	42.36	42.36	42.36	22	{}	\N	2020-10-12 15:52:06.029824
63581	PGM	2020-10-05	42.35	42.73	42.2	42.73	660	{}	\N	2020-10-12 15:52:06.034626
63582	PGM	2020-10-06	41.79	41.79	41.79	41.79	13	{}	\N	2020-10-12 15:52:06.039929
63583	PGM	2020-10-07	42.025	42.025	42.025	42.025	60	{}	\N	2020-10-12 15:52:06.046302
63584	PGM	2020-10-08	41.965	41.965	41.965	41.965	20	{}	\N	2020-10-12 15:52:06.051742
63585	PGM	2020-10-09	42.555	42.555	42.555	42.555	1	{}	\N	2020-10-12 15:52:06.059333
63586	PGM	2020-10-12	42.83	42.83	42.83	42.83	247	{}	\N	2020-10-12 15:52:06.065856
63587	PBP	2020-09-28	19.01	19.1748	18.94	19.03	28192	{}	\N	2020-10-12 15:52:07.86563
63588	PBP	2020-09-29	19.05	19.24	18.96	19.1409	19723	{}	\N	2020-10-12 15:52:07.870762
63589	PBP	2020-09-30	19.25	19.25	19.07	19.14	30877	{}	\N	2020-10-12 15:52:07.875058
63590	PBP	2020-10-01	19.14	19.2434	19.14	19.2	18035	{}	\N	2020-10-12 15:52:07.879227
63591	PBP	2020-10-02	19.14	19.19	19.07	19.19	13816	{}	\N	2020-10-12 15:52:07.88404
63592	PBP	2020-10-05	19.19	19.49	19.1724	19.39	48410	{}	\N	2020-10-12 15:52:07.88847
63593	PBP	2020-10-06	19.13	19.48	19.13	19.22	70916	{}	\N	2020-10-12 15:52:07.894387
63594	PBP	2020-10-07	19.25	19.42	19.16	19.26	413142	{}	\N	2020-10-12 15:52:07.900113
63595	PBP	2020-10-08	19.27	19.47	19.27	19.38	58705	{}	\N	2020-10-12 15:52:07.905135
63596	PBP	2020-10-09	19.38	19.56	19.38	19.56	135231	{}	\N	2020-10-12 15:52:07.911136
63597	PBP	2020-10-12	19.53	19.53	19.42	19.4413	6637	{}	\N	2020-10-12 15:52:07.916116
63598	PCI	2020-09-28	19.49	19.62	19.45	19.61	567804	{}	\N	2020-10-12 15:52:09.316359
63599	PCI	2020-09-29	19.67	19.92	19.5	19.79	1590321	{}	\N	2020-10-12 15:52:09.320962
63600	PCI	2020-09-30	19.76	20.25	19.76	20.08	3578884	{}	\N	2020-10-12 15:52:09.325574
63601	PCI	2020-10-01	20.11	20.25	20.04	20.08	1438808	{}	\N	2020-10-12 15:52:09.330653
63602	PCI	2020-10-02	19.9	20.24	19.79	20.15	1150994	{}	\N	2020-10-12 15:52:09.336208
63603	PCI	2020-10-05	20.23	20.46	20.17	20.2	1882440	{}	\N	2020-10-12 15:52:09.341368
63604	PCI	2020-10-06	20.2	20.23	20.02	20.08	1465622	{}	\N	2020-10-12 15:52:09.347863
63605	PCI	2020-10-07	20.03	20.14	19.95	20.01	1175262	{}	\N	2020-10-12 15:52:09.35191
63606	PCI	2020-10-08	20.06	20.1299	19.91	20.02	1100334	{}	\N	2020-10-12 15:52:09.355763
63607	PCI	2020-10-09	19.8	19.87	19.47	19.54	2246678	{}	\N	2020-10-12 15:52:09.360349
63608	PCI	2020-10-12	19.56	19.6154	19.4	19.41	858519	{}	\N	2020-10-12 15:52:09.364536
63609	PCYG	2020-09-28	5.24	5.24	4.82	5.23	56822	{}	\N	2020-10-12 15:52:10.57494
63610	PCYG	2020-09-29	4.96	5.0984	4.861	5.05	32752	{}	\N	2020-10-12 15:52:10.584186
63611	PCYG	2020-09-30	5.06	5.09	4.85	4.93	17171	{}	\N	2020-10-12 15:52:10.594469
63612	PCYG	2020-10-01	4.98	5.19	4.94	5.05	27150	{}	\N	2020-10-12 15:52:10.604375
63613	PCYG	2020-10-02	5.02	5.15	5.0001	5.01	12618	{}	\N	2020-10-12 15:52:10.614393
63614	PCYG	2020-10-05	5.0159	5.2	5.0159	5.18	22592	{}	\N	2020-10-12 15:52:10.62017
63615	PCYG	2020-10-06	5.12	5.2	4.92	4.92	30147	{}	\N	2020-10-12 15:52:10.626152
63616	PCYG	2020-10-07	5.14	5.14	5.02	5.07	36540	{}	\N	2020-10-12 15:52:10.631879
63617	PCYG	2020-10-08	5.16	5.18	5.04	5.09	19282	{}	\N	2020-10-12 15:52:10.637654
63618	PCYG	2020-10-09	5.13	5.25	5.08	5.19	30638	{}	\N	2020-10-12 15:52:10.643695
63619	PCYG	2020-10-12	5.16	5.2	5.12	5.2	16098	{}	\N	2020-10-12 15:52:10.650053
63620	PDLI	2020-09-28	3.28	3.28	3.18	3.28	443175	{}	\N	2020-10-12 15:52:11.86507
63621	PDLI	2020-09-29	3.28	3.36	3.25	3.33	585883	{}	\N	2020-10-12 15:52:11.871173
63622	PDLI	2020-09-30	3.34	3.36	3.14	3.15	1190795	{}	\N	2020-10-12 15:52:11.8768
63623	PDLI	2020-10-01	3.14	3.24	3.14	3.22	856802	{}	\N	2020-10-12 15:52:11.882401
63624	PDLI	2020-10-02	2.93	2.9799	2.59	2.61	1881532	{}	\N	2020-10-12 15:52:11.889446
63625	PDLI	2020-10-05	2.6	2.645	2.54	2.55	819039	{}	\N	2020-10-12 15:52:11.895069
63626	PDLI	2020-10-06	2.53	2.63	2.47	2.47	1199920	{}	\N	2020-10-12 15:52:11.900453
63627	PDLI	2020-10-07	2.5	2.5264	2.32	2.335	1686357	{}	\N	2020-10-12 15:52:11.905325
63628	PDLI	2020-10-08	2.34	2.435	2.321	2.41	1181884	{}	\N	2020-10-12 15:52:11.910838
63629	PDLI	2020-10-09	2.41	2.505	2.39	2.4	922757	{}	\N	2020-10-12 15:52:11.917374
63630	PDLI	2020-10-12	2.4	2.46	2.4	2.44	215227	{}	\N	2020-10-12 15:52:11.925883
63631	PEBO	2020-09-28	18.97	19.62	18.89	19.39	54457	{}	\N	2020-10-12 15:52:13.063739
63632	PEBO	2020-09-29	19.25	19.33	18.63	19.1	43271	{}	\N	2020-10-12 15:52:13.070705
63633	PEBO	2020-09-30	19.19	19.52	18.97	19.09	48135	{}	\N	2020-10-12 15:52:13.076799
63634	PEBO	2020-10-01	18.665	19.3	18.665	19.25	51246	{}	\N	2020-10-12 15:52:13.086163
63635	PEBO	2020-10-02	18.83	19.84	18.83	19.69	39355	{}	\N	2020-10-12 15:52:13.094949
63636	PEBO	2020-10-05	19.87	20.36	19.87	20.28	41306	{}	\N	2020-10-12 15:52:13.102549
63637	PEBO	2020-10-06	20.58	21.3236	20.19	20.35	70156	{}	\N	2020-10-12 15:52:13.109597
63638	PEBO	2020-10-07	20.58	21.07	20.37	20.96	43677	{}	\N	2020-10-12 15:52:13.115586
63639	PEBO	2020-10-08	21.22	21.8385	20.66	21.07	42792	{}	\N	2020-10-12 15:52:13.121757
63640	PEBO	2020-10-09	21.36	21.36	20.7	20.86	31596	{}	\N	2020-10-12 15:52:13.127494
63641	PEBO	2020-10-12	20.72	21.05	20.715	20.99	10238	{}	\N	2020-10-12 15:52:13.134042
63642	PESI	2020-09-28	6.8	7.02	6.8	6.96	3424	{}	\N	2020-10-12 15:52:14.498448
63643	PESI	2020-09-29	6.85	6.96	6.8	6.96	2491	{}	\N	2020-10-12 15:52:14.50609
63644	PESI	2020-09-30	6.98	7.125	6.98	7.05	3053	{}	\N	2020-10-12 15:52:14.510847
63645	PESI	2020-10-01	7	7	6.91	6.91	2004	{}	\N	2020-10-12 15:52:14.515375
63646	PESI	2020-10-02	6.8	6.99	6.7	6.95	6435	{}	\N	2020-10-12 15:52:14.521148
63647	PESI	2020-10-05	6.88	6.9	6.54	6.71	9330	{}	\N	2020-10-12 15:52:14.526279
63648	PESI	2020-10-06	6.75	7.1	6.7	7	106920	{}	\N	2020-10-12 15:52:14.531053
63649	PESI	2020-10-07	7.0142	7.1	7.0142	7.1	7313	{}	\N	2020-10-12 15:52:14.535889
63650	PESI	2020-10-08	7.1	7.1	6.88	6.95	6255	{}	\N	2020-10-12 15:52:14.540283
63651	PESI	2020-10-09	6.95	7.1	6.9001	7.1	11217	{}	\N	2020-10-12 15:52:14.545054
63652	PESI	2020-10-12	7	7.1152	6.965	6.975	3988	{}	\N	2020-10-12 15:52:14.550717
63653	PFBC	2020-09-28	32.11	33.06	32.02	32.84	47895	{}	\N	2020-10-12 15:52:15.874753
63654	PFBC	2020-09-29	32.65	32.67	31.79	32.46	33459	{}	\N	2020-10-12 15:52:15.8814
63655	PFBC	2020-09-30	32.62	33.34	31.89	32.12	68545	{}	\N	2020-10-12 15:52:15.88759
63656	PFBC	2020-10-01	32.04	32.24	31.49	32.13	48987	{}	\N	2020-10-12 15:52:15.894677
63657	PFBC	2020-10-02	31.43	33.16	31.43	32.9	49083	{}	\N	2020-10-12 15:52:15.901906
63658	PFBC	2020-10-05	33.11	33.88	33.11	33.84	52749	{}	\N	2020-10-12 15:52:15.907532
63659	PFBC	2020-10-06	33.68	34.85	33.385	33.54	66290	{}	\N	2020-10-12 15:52:15.912938
63660	PFBC	2020-10-07	33.79	34.44	33.255	34.14	93529	{}	\N	2020-10-12 15:52:15.918346
63661	PFBC	2020-10-08	34.48	35.07	33.94	34.81	40067	{}	\N	2020-10-12 15:52:15.925829
63662	PFBC	2020-10-09	35.05	35.73	34.3	34.38	47169	{}	\N	2020-10-12 15:52:15.93382
63663	PFBC	2020-10-12	34.4	34.485	34.01	34.29	6804	{}	\N	2020-10-12 15:52:15.940106
63664	PFGC	2020-09-28	34.8	35.39	34.6	34.79	580562	{}	\N	2020-10-12 15:52:17.342739
63665	PFGC	2020-09-29	35.08	35.23	34.13	34.52	2274400	{}	\N	2020-10-12 15:52:17.348336
63666	PFGC	2020-09-30	34.71	36.2562	34.32	34.62	2453049	{}	\N	2020-10-12 15:52:17.353241
63667	PFGC	2020-10-01	34.84	35.63	34.43	35.62	1626694	{}	\N	2020-10-12 15:52:17.358124
63668	PFGC	2020-10-02	34.65	36.26	34.34	35.99	1273856	{}	\N	2020-10-12 15:52:17.363534
63669	PFGC	2020-10-05	36.28	36.63	35.32	35.74	1157370	{}	\N	2020-10-12 15:52:17.36954
63670	PFGC	2020-10-06	36.4	37.69	36.01	36.01	2086186	{}	\N	2020-10-12 15:52:17.375216
63671	PFGC	2020-10-07	36.71	37.73	36.54	37.48	1679083	{}	\N	2020-10-12 15:52:17.380099
63672	PFGC	2020-10-08	38.07	38.72	37.55	38.09	1718081	{}	\N	2020-10-12 15:52:17.385462
63673	PFGC	2020-10-09	38.56	39.7	38.25	39.25	1178127	{}	\N	2020-10-12 15:52:17.391633
63674	PFGC	2020-10-12	39.06	39.39	38.49	39.16	212273	{}	\N	2020-10-12 15:52:17.396755
63675	PFM	2020-09-28	30.1	30.35	30.1	30.1798	24602	{}	\N	2020-10-12 15:52:18.531786
63676	PFM	2020-09-29	30.27	30.29	30.0005	30.06	17900	{}	\N	2020-10-12 15:52:18.538205
63677	PFM	2020-09-30	30.21	30.53	30.1	30.27	109125	{}	\N	2020-10-12 15:52:18.543285
63678	PFM	2020-10-01	30.4	30.47	30.2181	30.32	48018	{}	\N	2020-10-12 15:52:18.548431
63679	PFM	2020-10-02	29.97	30.4497	29.97	30.3	18289	{}	\N	2020-10-12 15:52:18.554894
63680	PFM	2020-10-05	30.42	30.7356	30.42	30.7	35982	{}	\N	2020-10-12 15:52:18.560644
63681	PFM	2020-10-06	30.77	31.0205	30.46	30.46	67193	{}	\N	2020-10-12 15:52:18.565747
63682	PFM	2020-10-07	30.72	31.02	30.72	30.95	242497	{}	\N	2020-10-12 15:52:18.571232
63683	PFM	2020-10-08	31.14	31.2328	31.1006	31.2328	9708	{}	\N	2020-10-12 15:52:18.576783
63684	PFM	2020-10-09	31.31	31.51	31.31	31.44	20012	{}	\N	2020-10-12 15:52:18.581833
63685	PFM	2020-10-12	31.58	31.7351	31.5692	31.7	4249	{}	\N	2020-10-12 15:52:18.587057
63686	PGC	2020-09-28	15.19	15.54	15.05	15.3	101964	{}	\N	2020-10-12 15:52:19.563371
63687	PGC	2020-09-29	15.17	15.35	14.74	15.32	72738	{}	\N	2020-10-12 15:52:19.568784
63688	PGC	2020-09-30	15.36	15.585	14.98	15.15	37146	{}	\N	2020-10-12 15:52:19.57467
63689	PGC	2020-10-01	15.16	15.59	15.08	15.45	45638	{}	\N	2020-10-12 15:52:19.579415
63690	PGC	2020-10-02	15.1	15.939	15.09	15.88	28873	{}	\N	2020-10-12 15:52:19.585164
63691	PGC	2020-10-05	16.15	16.46	15.59	16.39	36231	{}	\N	2020-10-12 15:52:19.590935
63692	PGC	2020-10-06	16.68	16.68	15.65	16.02	88802	{}	\N	2020-10-12 15:52:19.596718
63693	PGC	2020-10-07	16.29	16.49	16.05	16.39	120436	{}	\N	2020-10-12 15:52:19.601954
63694	PGC	2020-10-08	16.45	16.96	16.29	16.76	102451	{}	\N	2020-10-12 15:52:19.607564
63695	PGC	2020-10-09	16.87	16.88	16.21	16.4	98927	{}	\N	2020-10-12 15:52:19.612957
63696	PGC	2020-10-12	16.42	16.68	16.07	16.56	31514	{}	\N	2020-10-12 15:52:19.618188
63697	PGZ	2020-09-28	11	11.19	10.85	10.85	110400	{}	\N	2020-10-12 15:52:21.282106
63698	PGZ	2020-09-29	10.85	10.93	10.52	10.69	148694	{}	\N	2020-10-12 15:52:21.288259
63699	PGZ	2020-09-30	10.79	10.79	10.69	10.71	77792	{}	\N	2020-10-12 15:52:21.293743
63700	PGZ	2020-10-01	10.68	10.8	10.64	10.73	105444	{}	\N	2020-10-12 15:52:21.299349
63701	PGZ	2020-10-02	10.75	10.75	10.56	10.61	135098	{}	\N	2020-10-12 15:52:21.305914
63702	PGZ	2020-10-05	10.66	10.83	10.65	10.75	123468	{}	\N	2020-10-12 15:52:21.313256
63703	PGZ	2020-10-06	10.8	10.89	10.65	10.87	131876	{}	\N	2020-10-12 15:52:21.318668
63704	PGZ	2020-10-07	10.87	10.9305	10.87	10.9	44618	{}	\N	2020-10-12 15:52:21.324539
63705	PGZ	2020-10-08	10.95	11.08	10.9	11.08	48166	{}	\N	2020-10-12 15:52:21.330142
63706	PGZ	2020-10-09	11.11	11.15	11.02	11.12	64240	{}	\N	2020-10-12 15:52:21.336287
63707	PGZ	2020-10-12	11.1	11.22	11.0944	11.154	37838	{}	\N	2020-10-12 15:52:21.345649
63708	PFE	2020-09-28	36.16	36.575	36.16	36.39	14284247	{}	\N	2020-10-12 15:52:22.375559
63709	PFE	2020-09-29	36.43	36.54	36.035	36.17	24538009	{}	\N	2020-10-12 15:52:22.384216
63710	PFE	2020-09-30	36.27	36.959	36.25	36.7	40779633	{}	\N	2020-10-12 15:52:22.393526
63711	PFE	2020-10-01	36.91	37.09	36.1628	36.37	41457961	{}	\N	2020-10-12 15:52:22.40179
63712	PFE	2020-10-02	36.06	36.66	36.03	36.38	38990735	{}	\N	2020-10-12 15:52:22.407669
63713	PFE	2020-10-05	36.53	36.88	36.34	36.75	36440778	{}	\N	2020-10-12 15:52:22.413704
63714	PFE	2020-10-06	37	37.03	36.09	36.17	45576107	{}	\N	2020-10-12 15:52:22.419279
63715	PFE	2020-10-07	36.26	36.59	36.03	36.47	40637298	{}	\N	2020-10-12 15:52:22.42591
63716	PFE	2020-10-08	36.6	38.1	36.51	36.89	33635075	{}	\N	2020-10-12 15:52:22.431976
63717	PFE	2020-10-09	36.83	37.1174	36.59	36.79	43913111	{}	\N	2020-10-12 15:52:22.438512
63718	PFE	2020-10-12	36.7	37.18	36.465	36.94	16432749	{}	\N	2020-10-12 15:52:22.444093
63719	PFN	2020-09-28	9.03	9.1	9.03	9.06	262046	{}	\N	2020-10-12 15:52:23.425936
63720	PFN	2020-09-29	9.08	9.1	9.02	9.06	383008	{}	\N	2020-10-12 15:52:23.432956
63721	PFN	2020-09-30	9.1	9.12	9.08	9.08	389686	{}	\N	2020-10-12 15:52:23.438772
63722	PFN	2020-10-01	9.14	9.17	9.125	9.16	409042	{}	\N	2020-10-12 15:52:23.451691
63723	PFN	2020-10-02	8.97	9.19	8.97	9.18	822494	{}	\N	2020-10-12 15:52:23.460269
63724	PFN	2020-10-05	9.19	9.26	9.19	9.26	454854	{}	\N	2020-10-12 15:52:23.472193
63725	PFN	2020-10-06	9.29	9.33	9.26	9.31	664077	{}	\N	2020-10-12 15:52:23.481127
63726	PFN	2020-10-07	9.34	9.38	9.3278	9.38	508237	{}	\N	2020-10-12 15:52:23.487836
63727	PFN	2020-10-08	9.42	9.42	9.36	9.38	589752	{}	\N	2020-10-12 15:52:23.495765
63728	PFN	2020-10-09	9.34	9.35	9.28	9.32	512778	{}	\N	2020-10-12 15:52:23.503123
63729	PFN	2020-10-12	9.32	9.35	9.21	9.2563	191192	{}	\N	2020-10-12 15:52:23.512052
63730	PHD	2020-09-28	10.25	10.26	10.24	10.25	62629	{}	\N	2020-10-12 15:52:26.018575
63731	PHD	2020-09-29	10.22	10.23	10.17	10.21	137374	{}	\N	2020-10-12 15:52:26.026282
63732	PHD	2020-09-30	10.21	10.26	10.1	10.1	438926	{}	\N	2020-10-12 15:52:26.034233
63733	PHD	2020-10-01	10.13	10.16	10.1	10.11	274606	{}	\N	2020-10-12 15:52:26.042518
63734	PHD	2020-10-02	10.07	10.15	10.07	10.15	135651	{}	\N	2020-10-12 15:52:26.051574
63735	PHD	2020-10-05	10.14	10.2	10.13	10.2	142196	{}	\N	2020-10-12 15:52:26.057603
63736	PHD	2020-10-06	10.2	10.23	10.18	10.22	48885	{}	\N	2020-10-12 15:52:26.064321
63737	PHD	2020-10-07	10.23	10.28	10.213	10.27	64656	{}	\N	2020-10-12 15:52:26.07061
63738	PHD	2020-10-08	10.24	10.29	10.22	10.22	102770	{}	\N	2020-10-12 15:52:26.076429
63739	PHD	2020-10-09	10.25	10.25	10.18	10.18	64528	{}	\N	2020-10-12 15:52:26.083932
63740	PHD	2020-10-12	10.19	10.27	10.19	10.25	14974	{}	\N	2020-10-12 15:52:26.092649
63741	PHT	2020-09-28	7.96	8.03	7.9485	8.02	57390	{}	\N	2020-10-12 15:52:27.61276
63742	PHT	2020-09-29	7.99	8.03	7.97	8.02	209449	{}	\N	2020-10-12 15:52:27.619571
63743	PHT	2020-09-30	8.07	8.13	7.96	7.96	395376	{}	\N	2020-10-12 15:52:27.625154
63744	PHT	2020-10-01	8.04	8.09	8.0007	8.08	204655	{}	\N	2020-10-12 15:52:27.63108
63745	PHT	2020-10-02	8.05	8.14	8.03	8.14	225883	{}	\N	2020-10-12 15:52:27.638307
63746	PHT	2020-10-05	8.15	8.203	8.11	8.2	129348	{}	\N	2020-10-12 15:52:27.645529
63747	PHT	2020-10-06	8.2	8.28	8.18	8.23	186492	{}	\N	2020-10-12 15:52:27.651592
63748	PHT	2020-10-07	8.26	8.29	8.23	8.27	92171	{}	\N	2020-10-12 15:52:27.657494
63749	PHT	2020-10-08	8.29	8.37	8.24	8.29	146836	{}	\N	2020-10-12 15:52:27.663314
63750	PHT	2020-10-09	8.3	8.34	8.28	8.31	229053	{}	\N	2020-10-12 15:52:27.669595
63751	PHT	2020-10-12	8.3	8.36	8.2546	8.287	48940	{}	\N	2020-10-12 15:52:27.676153
63752	PJT	2020-09-28	58.53	59.89	58.53	59.44	63604	{}	\N	2020-10-12 15:52:29.072118
63753	PJT	2020-09-29	59.52	60.21	59.41	59.48	218704	{}	\N	2020-10-12 15:52:29.077994
63754	PJT	2020-09-30	59.62	61.6	59.62	60.61	229280	{}	\N	2020-10-12 15:52:29.084713
63755	PJT	2020-10-01	60.84	61.31	59.66	60.18	327448	{}	\N	2020-10-12 15:52:29.090037
63756	PJT	2020-10-02	59.39	61.705	59.39	60.96	250737	{}	\N	2020-10-12 15:52:29.096698
63757	PJT	2020-10-05	61.25	62.42	60.87	62.28	184036	{}	\N	2020-10-12 15:52:29.107524
63758	PJT	2020-10-06	62.36	64	61.5261	61.98	225133	{}	\N	2020-10-12 15:52:29.118133
63759	PJT	2020-10-07	63.25	64.08	62.74	63.7	260805	{}	\N	2020-10-12 15:52:29.126479
63760	PJT	2020-10-08	64	64.97	63.83	64.64	195106	{}	\N	2020-10-12 15:52:29.134161
63761	PJT	2020-10-09	65.78	67.34	65.11	67.25	276813	{}	\N	2020-10-12 15:52:29.140871
63762	PJT	2020-10-12	67.64	68.78	67.29	68.3611	57105	{}	\N	2020-10-12 15:52:29.1479
63763	PKX	2020-09-28	42.07	42.33	41.78	42.07	150627	{}	\N	2020-10-12 15:52:31.017165
63764	PKX	2020-09-29	42.09	42.22	41.69	41.72	185061	{}	\N	2020-10-12 15:52:31.024192
63765	PKX	2020-09-30	41.72	42.23	41.69	41.86	243857	{}	\N	2020-10-12 15:52:31.029612
63766	PKX	2020-10-01	41.95	42.52	41.95	42.22	131145	{}	\N	2020-10-12 15:52:31.034884
63767	PKX	2020-10-02	41.51	42.36	41.2552	42.17	216681	{}	\N	2020-10-12 15:52:31.04039
63768	PKX	2020-10-05	43.73	44.47	43.73	44.23	325676	{}	\N	2020-10-12 15:52:31.045301
63769	PKX	2020-10-06	43.65	43.82	42.42	42.75	368146	{}	\N	2020-10-12 15:52:31.049973
63770	PKX	2020-10-07	44.04	44.58	43.93	44.39	229775	{}	\N	2020-10-12 15:52:31.055655
63771	PKX	2020-10-08	44.91	44.96	44.32	44.65	306413	{}	\N	2020-10-12 15:52:31.061164
63772	PKX	2020-10-09	44.96	44.96	44.46	44.8	172506	{}	\N	2020-10-12 15:52:31.066965
63773	PKX	2020-10-12	44.95	44.99	44.54	44.56	74756	{}	\N	2020-10-12 15:52:31.072225
63774	PLSE	2020-09-28	11.82	11.98	11.7	11.72	43974	{}	\N	2020-10-12 15:52:32.123736
63775	PLSE	2020-09-29	11.8	12.26	11.57	12.23	59405	{}	\N	2020-10-12 15:52:32.129612
63776	PLSE	2020-09-30	12.24	12.25	11.63	11.79	35555	{}	\N	2020-10-12 15:52:32.134838
63777	PLSE	2020-10-01	11.86	12.24	11.4	11.66	73210	{}	\N	2020-10-12 15:52:32.139812
63778	PLSE	2020-10-02	11.54	14.51	11.4762	13.41	318286	{}	\N	2020-10-12 15:52:32.145438
63779	PLSE	2020-10-05	13.41	17.77	13.41	16.06	807938	{}	\N	2020-10-12 15:52:32.152625
63780	PLSE	2020-10-06	15.76	15.76	14.68	14.83	251153	{}	\N	2020-10-12 15:52:32.158933
63781	PLSE	2020-10-07	15.2	15.55	14.7979	15.43	89915	{}	\N	2020-10-12 15:52:32.164452
63782	PLSE	2020-10-08	15.47	16.14	15.31	15.66	86841	{}	\N	2020-10-12 15:52:32.170718
63783	PLSE	2020-10-09	15.69	15.9425	15.3	15.65	40018	{}	\N	2020-10-12 15:52:32.178608
63784	PLSE	2020-10-12	15.57	15.57	15.03	15.03	8428	{}	\N	2020-10-12 15:52:32.190785
63785	PMD	2020-09-28	4.35	4.465	4.35	4.37	14583	{}	\N	2020-10-12 15:52:33.627377
63786	PMD	2020-09-29	4.41	4.45	4.41	4.41	4144	{}	\N	2020-10-12 15:52:33.634005
63787	PMD	2020-09-30	4.41	4.49	4.41	4.41	6884	{}	\N	2020-10-12 15:52:33.640236
63788	PMD	2020-10-01	4.42	4.42	4.35	4.35	13944	{}	\N	2020-10-12 15:52:33.646176
63789	PMD	2020-10-02	4.375	4.4	4.31	4.3684	15427	{}	\N	2020-10-12 15:52:33.652252
63790	PMD	2020-10-05	4.35	4.56	4.35	4.47	20683	{}	\N	2020-10-12 15:52:33.659015
63791	PMD	2020-10-06	4.47	4.52	4.34	4.34	14348	{}	\N	2020-10-12 15:52:33.665135
63792	PMD	2020-10-07	4.47	4.47	4.3	4.32	23304	{}	\N	2020-10-12 15:52:33.67063
63793	PMD	2020-10-08	4.3	4.47	4.3	4.42	16609	{}	\N	2020-10-12 15:52:33.677587
63794	PMD	2020-10-09	4.41	4.4856	4.33	4.36	18351	{}	\N	2020-10-12 15:52:33.684966
63795	PMD	2020-10-12	4.4	4.4	4.25	4.31	25459	{}	\N	2020-10-12 15:52:33.694768
63796	PNR	2020-09-28	45.51	46.2955	45.415	45.85	546595	{}	\N	2020-10-12 15:52:35.361996
63797	PNR	2020-09-29	46	46.05	45.46	45.79	1135279	{}	\N	2020-10-12 15:52:35.372141
63798	PNR	2020-09-30	45.99	46.545	45.5	45.77	1670165	{}	\N	2020-10-12 15:52:35.379373
63799	PNR	2020-10-01	46.06	46.12	44.82	45.41	1649945	{}	\N	2020-10-12 15:52:35.385907
63800	PNR	2020-10-02	44.59	46.22	44.37	45.91	1682638	{}	\N	2020-10-12 15:52:35.393006
63801	PNR	2020-10-05	46.48	46.955	45.825	46.19	1324478	{}	\N	2020-10-12 15:52:35.399399
63802	PNR	2020-10-06	46.27	47.23	46	46.72	3448778	{}	\N	2020-10-12 15:52:35.40626
63803	PNR	2020-10-07	47.16	48.14	47.08	48.11	2772181	{}	\N	2020-10-12 15:52:35.413097
63804	PNR	2020-10-08	48.34	48.86	48.2	48.83	2738087	{}	\N	2020-10-12 15:52:35.421439
63805	PNR	2020-10-09	49.15	49.97	49	49.59	2164652	{}	\N	2020-10-12 15:52:35.429768
63806	PNR	2020-10-12	49.71	50.38	49.71	50.095	1016742	{}	\N	2020-10-12 15:52:35.436876
63807	POR	2020-09-28	35.72	35.8995	35.26	35.6	969974	{}	\N	2020-10-12 15:52:36.277571
63808	POR	2020-09-29	35.61	35.94	35	35.19	1250476	{}	\N	2020-10-12 15:52:36.284876
63809	POR	2020-09-30	35.51	35.695	35.19	35.5	1459280	{}	\N	2020-10-12 15:52:36.290646
63810	POR	2020-10-01	35.63	36.12	35.42	35.98	1491162	{}	\N	2020-10-12 15:52:36.296667
63811	POR	2020-10-02	35.6	36.84	35.54	36.62	979359	{}	\N	2020-10-12 15:52:36.302116
63812	POR	2020-10-05	36.76	36.83	36.17	36.8	973842	{}	\N	2020-10-12 15:52:36.30843
63813	POR	2020-10-06	36.91	37.3	36.37	36.5	2005406	{}	\N	2020-10-12 15:52:36.314379
63814	POR	2020-10-07	36.59	37.13	36.29	36.99	1284520	{}	\N	2020-10-12 15:52:36.320091
63815	POR	2020-10-08	37.15	37.76	37.0487	37.75	1129353	{}	\N	2020-10-12 15:52:36.327259
63816	POR	2020-10-09	37.9	38.19	37.17	37.69	1693323	{}	\N	2020-10-12 15:52:36.333276
63817	POR	2020-10-12	37.45	38.75	37.45	38.7	521827	{}	\N	2020-10-12 15:52:36.339055
63818	PPIH	2020-09-28	5.63	5.79	5.43	5.79	8917	{}	\N	2020-10-12 15:52:38.167383
63819	PPIH	2020-09-29	5.73	5.84	5.7	5.8	5966	{}	\N	2020-10-12 15:52:38.174326
63820	PPIH	2020-09-30	5.58	5.75	5.56	5.65	4144	{}	\N	2020-10-12 15:52:38.180455
63821	PPIH	2020-10-01	5.8	5.85	5.56	5.81	7738	{}	\N	2020-10-12 15:52:38.18516
63822	PPIH	2020-10-02	5.73	5.87	5.635	5.85	2205	{}	\N	2020-10-12 15:52:38.191616
63823	PPIH	2020-10-05	5.85	5.94	5.75	5.85	20965	{}	\N	2020-10-12 15:52:38.199932
63824	PPIH	2020-10-06	5.895	5.97	5.8	5.8	2005	{}	\N	2020-10-12 15:52:38.206169
63825	PPIH	2020-10-07	5.7901	5.85	5.79	5.85	7711	{}	\N	2020-10-12 15:52:38.214375
63826	PPIH	2020-10-08	5.8	5.88	5.73	5.88	8266	{}	\N	2020-10-12 15:52:38.21974
63827	PPIH	2020-10-09	5.9	5.9	5.9	5.9	1117	{}	\N	2020-10-12 15:52:38.227952
63828	PFIE	2020-09-28	0.6473	0.7	0.6473	0.7	58097	{}	\N	2020-10-12 15:52:39.350235
63829	PFIE	2020-09-29	0.7	0.75	0.68	0.745	51569	{}	\N	2020-10-12 15:52:39.356125
63830	PFIE	2020-09-30	0.74	0.7446	0.73	0.7398	36767	{}	\N	2020-10-12 15:52:39.361885
63831	PFIE	2020-10-01	0.73	0.73	0.72	0.73	551322	{}	\N	2020-10-12 15:52:39.368311
63832	PFIE	2020-10-02	0.7396	0.75	0.72	0.75	309666	{}	\N	2020-10-12 15:52:39.374813
63833	PFIE	2020-10-05	0.747	0.8	0.7313	0.7412	80612	{}	\N	2020-10-12 15:52:39.381443
63834	PFIE	2020-10-06	0.7321	0.75	0.73	0.7331	39079	{}	\N	2020-10-12 15:52:39.387458
63835	PFIE	2020-10-07	0.7405	0.76	0.74	0.740501	86334	{}	\N	2020-10-12 15:52:39.393244
63836	PFIE	2020-10-08	0.66	0.75	0.65	0.748	8429	{}	\N	2020-10-12 15:52:39.399256
63837	PFIE	2020-10-09	0.75	0.7942	0.7281	0.75	1551308	{}	\N	2020-10-12 15:52:39.40523
63838	PFIE	2020-10-12	0.76	0.78	0.736	0.736	9049	{}	\N	2020-10-12 15:52:39.413184
63839	PFMT	2020-09-28	0.65	1.19	0.65	1.02	2282669	{}	\N	2020-10-12 15:52:40.404266
63840	PFMT	2020-09-29	0.8564	0.9	0.7542	0.84	807009	{}	\N	2020-10-12 15:52:40.411027
63841	PFMT	2020-09-30	0.82	1.04	0.801	0.9385	241487	{}	\N	2020-10-12 15:52:40.41775
63842	PFMT	2020-10-01	0.9101	0.9799	0.9101	0.9575	67156	{}	\N	2020-10-12 15:52:40.423672
63843	PFMT	2020-10-02	1	1.1	0.961	1.1	109515	{}	\N	2020-10-12 15:52:40.430171
63844	PFMT	2020-10-05	1.1	1.26	1.1	1.21	294394	{}	\N	2020-10-12 15:52:40.436342
63845	PFMT	2020-10-06	1.26	1.68	1.22	1.35	635982	{}	\N	2020-10-12 15:52:40.442238
63846	PFMT	2020-10-07	1.35	1.4	1.21	1.34	221885	{}	\N	2020-10-12 15:52:40.447859
63847	PFMT	2020-10-08	1.35	1.3551	1.26	1.35	62575	{}	\N	2020-10-12 15:52:40.454481
63848	PFMT	2020-10-09	1.36	1.6727	1.3599	1.62	257333	{}	\N	2020-10-12 15:52:40.460811
63849	PFMT	2020-10-12	1.65	1.85	1.648	1.74	171953	{}	\N	2020-10-12 15:52:40.467101
63850	PFXF	2020-09-28	19.38	19.38	19.24	19.31	163728	{}	\N	2020-10-12 15:52:41.331339
63851	PFXF	2020-09-29	19.24	19.3	19.22	19.26	87221	{}	\N	2020-10-12 15:52:41.336965
63852	PFXF	2020-09-30	19.23	19.3099	19.17	19.18	148328	{}	\N	2020-10-12 15:52:41.342003
63853	PFXF	2020-10-01	19.09	19.27	19.09	19.27	79936	{}	\N	2020-10-12 15:52:41.348355
63854	PFXF	2020-10-02	19.18	19.33	19.17	19.32	138908	{}	\N	2020-10-12 15:52:41.354137
63855	PFXF	2020-10-05	19.31	19.47	19.3	19.43	158672	{}	\N	2020-10-12 15:52:41.35908
63856	PFXF	2020-10-06	19.4489	19.5763	19.4194	19.46	198755	{}	\N	2020-10-12 15:52:41.364619
63857	PFXF	2020-10-07	19.56	19.6075	19.52	19.6	94728	{}	\N	2020-10-12 15:52:41.369369
63858	PFXF	2020-10-08	19.6	19.71	19.5701	19.68	82597	{}	\N	2020-10-12 15:52:41.374507
63859	PFXF	2020-10-09	19.72	19.74	19.6541	19.72	139300	{}	\N	2020-10-12 15:52:41.379853
63860	PFXF	2020-10-12	19.74	19.74	19.69	19.731	52368	{}	\N	2020-10-12 15:52:41.385221
63861	PGP	2020-09-28	7.95	8.23	7.95	8.07	47897	{}	\N	2020-10-12 15:52:42.947145
63862	PGP	2020-09-29	8	8.1	7.97	8.07	159942	{}	\N	2020-10-12 15:52:42.953865
63863	PGP	2020-09-30	8.09	8.24	8.09	8.24	84938	{}	\N	2020-10-12 15:52:42.959664
63864	PGP	2020-10-01	8.34	8.3727	8.23	8.33	68852	{}	\N	2020-10-12 15:52:42.96547
63865	PGP	2020-10-02	8.26	8.39	8.151	8.39	145706	{}	\N	2020-10-12 15:52:42.971633
63866	PGP	2020-10-05	8.36	8.62	8.36	8.46	157525	{}	\N	2020-10-12 15:52:42.977178
63867	PGP	2020-10-06	8.45	8.5327	8.3013	8.39	108753	{}	\N	2020-10-12 15:52:42.982436
63868	PGP	2020-10-07	8.48	8.5284	8.3972	8.52	70076	{}	\N	2020-10-12 15:52:42.987611
63869	PGP	2020-10-08	8.51	8.58	8.46	8.58	85918	{}	\N	2020-10-12 15:52:42.993433
63870	PGP	2020-10-09	8.45	8.5	8.42	8.42	82942	{}	\N	2020-10-12 15:52:42.998959
63871	PGP	2020-10-12	8.5	8.51	8.4	8.4741	83330	{}	\N	2020-10-12 15:52:43.003942
63872	PHDG	2020-09-28	32.65	33.05	32.65	32.955	81448	{}	\N	2020-10-12 15:52:43.935619
63873	PHDG	2020-09-29	33.04	33.04	32.7001	32.74	46681	{}	\N	2020-10-12 15:52:43.94258
63874	PHDG	2020-09-30	32.79	33.26	32.76	32.99	31035	{}	\N	2020-10-12 15:52:43.948086
63875	PHDG	2020-10-01	33.1367	33.19	32.9416	33.17	131432	{}	\N	2020-10-12 15:52:43.954056
63876	PHDG	2020-10-02	32.89	33.2999	32.89	33.03	32707	{}	\N	2020-10-12 15:52:43.962188
63877	PHDG	2020-10-05	33.4	33.4299	33.25	33.42	56413	{}	\N	2020-10-12 15:52:43.968905
63878	PHDG	2020-10-06	33.31	33.353	32.89	32.89	27336	{}	\N	2020-10-12 15:52:43.974826
63879	PHDG	2020-10-07	32.98	33.3899	32.98	33.3288	421734	{}	\N	2020-10-12 15:52:43.980774
63880	PHDG	2020-10-08	33.48	33.53	33.35	33.36	81221	{}	\N	2020-10-12 15:52:43.986193
63881	PHDG	2020-10-09	33.5099	33.529	33.32	33.44	57225	{}	\N	2020-10-12 15:52:43.991518
63882	PHDG	2020-10-12	33.66	33.75	33.5161	33.75	22837	{}	\N	2020-10-12 15:52:43.996906
63883	PIO	2020-09-28	31.56	31.7037	31.471	31.48	5821	{}	\N	2020-10-12 15:52:44.949838
63884	PIO	2020-09-29	31.58	31.69	31.515	31.64	18294	{}	\N	2020-10-12 15:52:44.959408
63885	PIO	2020-09-30	31.64	31.91	31.64	31.72	7637	{}	\N	2020-10-12 15:52:44.969334
63886	PIO	2020-10-01	31.9	32	31.75	31.84	21102	{}	\N	2020-10-12 15:52:44.978781
63887	PIO	2020-10-02	31.53	31.98	31.53	31.89	9728	{}	\N	2020-10-12 15:52:44.986253
63888	PIO	2020-10-05	31.96	32.2475	31.96	32.2475	10963	{}	\N	2020-10-12 15:52:44.994207
63889	PIO	2020-10-06	32.24	32.38	31.99	31.99	4930	{}	\N	2020-10-12 15:52:45.004132
63890	PIO	2020-10-07	32.18	32.39	32.18	32.3138	16001	{}	\N	2020-10-12 15:52:45.013056
63891	PIO	2020-10-08	32.499	32.57	32.421	32.56	18453	{}	\N	2020-10-12 15:52:45.020491
63892	PIO	2020-10-09	32.67	32.9199	32.6623	32.78	25500	{}	\N	2020-10-12 15:52:45.027108
63893	PIO	2020-10-12	33.01	33.19	33.01	33.1733	5318	{}	\N	2020-10-12 15:52:45.033091
63894	PKW	2020-09-28	60.03	60.5439	60.03	60.37	25843	{}	\N	2020-10-12 15:52:46.393649
63895	PKW	2020-09-29	60.16	60.31	59.58	59.73	21826	{}	\N	2020-10-12 15:52:46.400429
63896	PKW	2020-09-30	59.8	60.83	59.8	60.18	28047	{}	\N	2020-10-12 15:52:46.407039
63897	PKW	2020-10-01	60.59	60.8	60.0541	60.37	29916	{}	\N	2020-10-12 15:52:46.41377
63898	PKW	2020-10-02	59.23	60.6246	59.23	60.31	54975	{}	\N	2020-10-12 15:52:46.424481
63899	PKW	2020-10-05	60.67	61.7001	60.67	61.67	29416	{}	\N	2020-10-12 15:52:46.431444
63900	PKW	2020-10-06	61.87	62.44	60.79	60.89	30632	{}	\N	2020-10-12 15:52:46.438898
63901	PKW	2020-10-07	61.47	62.3056	61.47	62.13	33479	{}	\N	2020-10-12 15:52:46.446528
63902	PKW	2020-10-08	62.2	62.68	62.2	62.67	30493	{}	\N	2020-10-12 15:52:46.453291
63903	PKW	2020-10-09	62.93	63.17	62.7511	62.96	17919	{}	\N	2020-10-12 15:52:46.459625
63904	PKW	2020-10-12	63.24	63.555	63.24	63.555	4126	{}	\N	2020-10-12 15:52:46.466071
63905	PLNT	2020-09-28	60.99	61.98	60.19	61.32	1545954	{}	\N	2020-10-12 15:52:47.378447
63906	PLNT	2020-09-29	61.44	61.47	58.86	58.91	2051076	{}	\N	2020-10-12 15:52:47.390418
63907	PLNT	2020-09-30	59.48	62.18	59.36	61.62	5169405	{}	\N	2020-10-12 15:52:47.399445
63908	PLNT	2020-10-01	62.73	64.85	62.27	63.53	4361415	{}	\N	2020-10-12 15:52:47.410617
63909	PLNT	2020-10-02	60.92	63.27	60.83	62.22	3814030	{}	\N	2020-10-12 15:52:47.437739
63910	PLNT	2020-10-05	62.8	63.04	61.65	62.48	2017314	{}	\N	2020-10-12 15:52:47.456066
63911	PLNT	2020-10-06	62.98	64.68	62.18	62.37	2874833	{}	\N	2020-10-12 15:52:47.46786
63912	PLNT	2020-10-07	63.58	65.62	62.28	65.42	2739064	{}	\N	2020-10-12 15:52:47.476578
63913	PLNT	2020-10-08	65.51	66.25	65.2975	65.88	1966658	{}	\N	2020-10-12 15:52:47.483644
63914	PLNT	2020-10-09	66	68.62	65.9	67.6	2920747	{}	\N	2020-10-12 15:52:47.489453
63915	PLNT	2020-10-12	68.48	68.48	66.86	67.055	757737	{}	\N	2020-10-12 15:52:47.494434
63916	PLX	2020-09-28	3.65	3.72	3.53	3.68	89677	{}	\N	2020-10-12 15:52:48.38868
63917	PLX	2020-09-29	3.8	4.05	3.8	3.86	424887	{}	\N	2020-10-12 15:52:48.398201
63918	PLX	2020-09-30	3.97	4	3.82	3.87	102148	{}	\N	2020-10-12 15:52:48.405413
63919	PLX	2020-10-01	3.99	3.997	3.82	3.86	83333	{}	\N	2020-10-12 15:52:48.412569
63920	PLX	2020-10-02	3.71	3.9532	3.68	3.725	105573	{}	\N	2020-10-12 15:52:48.418315
63921	PLX	2020-10-05	3.86	3.95	3.78	3.87	75321	{}	\N	2020-10-12 15:52:48.422783
63922	PLX	2020-10-06	3.88	3.9	3.71	3.75	79809	{}	\N	2020-10-12 15:52:48.4306
63923	PLX	2020-10-07	3.74	3.85	3.74	3.79	22295	{}	\N	2020-10-12 15:52:48.4362
63924	PLX	2020-10-08	3.79	3.84	3.65	3.79	44403	{}	\N	2020-10-12 15:52:48.442027
63925	PLX	2020-10-09	3.82	3.86	3.76	3.78	49188	{}	\N	2020-10-12 15:52:48.448042
63926	PLX	2020-10-12	3.8	3.8	3.7	3.8	29440	{}	\N	2020-10-12 15:52:48.454418
63927	PML	2020-09-28	13.24	13.45	13.24	13.37	128839	{}	\N	2020-10-12 15:52:49.410256
63928	PML	2020-09-29	13.36	13.5	13.36	13.47	308772	{}	\N	2020-10-12 15:52:49.418337
63929	PML	2020-09-30	13.43	13.68	13.43	13.5	496053	{}	\N	2020-10-12 15:52:49.426121
63930	PML	2020-10-01	13.63	13.66	13.55	13.61	189239	{}	\N	2020-10-12 15:52:49.433162
63931	PML	2020-10-02	13.55	13.75	13.49	13.69	184370	{}	\N	2020-10-12 15:52:49.439248
63932	PML	2020-10-05	13.65	13.74	13.63	13.67	153620	{}	\N	2020-10-12 15:52:49.447085
63933	PML	2020-10-06	13.68	13.71	13.6	13.6	64456	{}	\N	2020-10-12 15:52:49.455199
63934	PML	2020-10-07	13.58	13.75	13.55	13.58	179826	{}	\N	2020-10-12 15:52:49.461156
63935	PML	2020-10-08	13.61	13.7	13.57	13.62	204544	{}	\N	2020-10-12 15:52:49.466567
63936	PML	2020-10-09	13.6	13.64	13.54	13.59	106062	{}	\N	2020-10-12 15:52:49.471927
63937	PML	2020-10-12	13.6	13.6799	13.52	13.6	114767	{}	\N	2020-10-12 15:52:49.477202
63938	PNM	2020-09-28	41.43	41.9	41.1	41.45	354650	{}	\N	2020-10-12 15:52:50.423683
63939	PNM	2020-09-29	41.56	41.84	40.94	41.04	621632	{}	\N	2020-10-12 15:52:50.438966
63940	PNM	2020-09-30	41.37	41.93	40.87	41.33	694310	{}	\N	2020-10-12 15:52:50.447569
63941	PNM	2020-10-01	41.62	42.13	40.32	41.9	870691	{}	\N	2020-10-12 15:52:50.455025
63942	PNM	2020-10-02	41.45	43.01	41.39	42.86	567326	{}	\N	2020-10-12 15:52:50.461964
63943	PNM	2020-10-05	43.14	43.89	42.5201	43.86	637439	{}	\N	2020-10-12 15:52:50.468039
63944	PNM	2020-10-06	44.11	44.76	43.39	43.64	647045	{}	\N	2020-10-12 15:52:50.473967
63945	PNM	2020-10-07	44.05	44.2	43.715	44.03	695685	{}	\N	2020-10-12 15:52:50.479739
63946	PNM	2020-10-08	44.41	44.67	44.01	44.64	535891	{}	\N	2020-10-12 15:52:50.48535
63947	PNM	2020-10-09	45.05	45.05	43.89	44.52	477019	{}	\N	2020-10-12 15:52:50.491241
63948	PNM	2020-10-12	44.47	45.95	44.22	45.879	134797	{}	\N	2020-10-12 15:52:50.496914
63949	PGTI	2020-09-28	17.16	17.68	17.16	17.53	214798	{}	\N	2020-10-12 15:52:51.591499
63950	PGTI	2020-09-29	17.55	17.59	17.19	17.33	281573	{}	\N	2020-10-12 15:52:51.603866
63951	PGTI	2020-09-30	17.36	17.86	17.36	17.52	527463	{}	\N	2020-10-12 15:52:51.611277
63952	PGTI	2020-10-01	17.56	17.77	17.37	17.64	390380	{}	\N	2020-10-12 15:52:51.617958
63953	PGTI	2020-10-02	17.21	18.225	17.13	18.1	557108	{}	\N	2020-10-12 15:52:51.62487
63954	PGTI	2020-10-05	18.32	18.57	18.18	18.21	436470	{}	\N	2020-10-12 15:52:51.630133
63955	PGTI	2020-10-06	18.49	18.78	18.11	18.12	676742	{}	\N	2020-10-12 15:52:51.635448
63956	PGTI	2020-10-07	18.43	18.64	18.19	18.31	753275	{}	\N	2020-10-12 15:52:51.640659
63957	PGTI	2020-10-08	18.6	18.92	18.56	18.82	708236	{}	\N	2020-10-12 15:52:51.645912
63958	PGTI	2020-10-09	19	19.17	18.83	19.01	1084755	{}	\N	2020-10-12 15:52:51.65113
63959	PGTI	2020-10-12	19.13	19.29	18.91	19.15	274444	{}	\N	2020-10-12 15:52:51.657024
63960	PHI	2020-09-28	27.81	27.89	27.31	27.44	229319	{}	\N	2020-10-12 15:52:53.106987
63961	PHI	2020-09-29	27.36	27.68	27.17	27.4	274824	{}	\N	2020-10-12 15:52:53.114307
63962	PHI	2020-09-30	27.34	27.36	26.81	27.05	378650	{}	\N	2020-10-12 15:52:53.119916
63963	PHI	2020-10-01	27.03	27.27	26.86	27.17	248104	{}	\N	2020-10-12 15:52:53.126097
63964	PHI	2020-10-02	26.89	27.68	26.86	27.43	305861	{}	\N	2020-10-12 15:52:53.131962
63965	PHI	2020-10-05	27.59	27.98	27.52	27.86	200771	{}	\N	2020-10-12 15:52:53.1375
63966	PHI	2020-10-06	27.96	27.96	26.88	27.05	171087	{}	\N	2020-10-12 15:52:53.142477
63967	PHI	2020-10-07	26.95	26.95	26.17	26.51	273813	{}	\N	2020-10-12 15:52:53.147455
63968	PHI	2020-10-08	26.62	26.71	26.25	26.32	243918	{}	\N	2020-10-12 15:52:53.152419
63969	PHI	2020-10-09	26.43	26.8	26.29	26.75	275404	{}	\N	2020-10-12 15:52:53.158574
63970	PHI	2020-10-12	26.82	26.97	26.55	26.91	28172	{}	\N	2020-10-12 15:52:53.165767
63971	PI	2020-09-28	26.01	27	25.67	26.6	347833	{}	\N	2020-10-12 15:52:54.11458
63972	PI	2020-09-29	26.52	26.84	26.1	26.45	132454	{}	\N	2020-10-12 15:52:54.126984
63973	PI	2020-09-30	26.35	26.81	25.93	26.35	184158	{}	\N	2020-10-12 15:52:54.136434
63974	PI	2020-10-01	26.73	28.7899	26.73	28.65	273987	{}	\N	2020-10-12 15:52:54.142473
63975	PI	2020-10-02	27.85	29.16	27.8	28.5	199923	{}	\N	2020-10-12 15:52:54.147639
63976	PI	2020-10-05	28.98	29.18	28.57	28.98	140273	{}	\N	2020-10-12 15:52:54.152076
63977	PI	2020-10-06	28.99	30	28.89	28.96	222417	{}	\N	2020-10-12 15:52:54.156407
63978	PI	2020-10-07	29.5	30.13	29.36	29.72	117529	{}	\N	2020-10-12 15:52:54.161133
63979	PI	2020-10-08	30.2	30.33	29.6324	29.82	354370	{}	\N	2020-10-12 15:52:54.165322
63980	PI	2020-10-09	29.95	30.72	29.9	30.1	149546	{}	\N	2020-10-12 15:52:54.169454
63981	PI	2020-10-12	30.5	30.77	30.22	30.57	59926	{}	\N	2020-10-12 15:52:54.173504
63982	PIM	2020-09-28	4.01	4.04	4	4.04	132210	{}	\N	2020-10-12 15:52:55.0973
63983	PIM	2020-09-29	4.05	4.08	4.0314	4.06	137836	{}	\N	2020-10-12 15:52:55.111183
63984	PIM	2020-09-30	4.05	4.11	4.03	4.11	396787	{}	\N	2020-10-12 15:52:55.119592
63985	PIM	2020-10-01	4.12	4.12	4.09	4.12	462270	{}	\N	2020-10-12 15:52:55.126474
63986	PIM	2020-10-02	4.04	4.11	4.0301	4.11	302896	{}	\N	2020-10-12 15:52:55.133203
63987	PIM	2020-10-05	4.11	4.13	4.09	4.12	176482	{}	\N	2020-10-12 15:52:55.139322
63988	PIM	2020-10-06	4.13	4.14	4.1	4.14	217207	{}	\N	2020-10-12 15:52:55.144362
63989	PIM	2020-10-07	4.13	4.16	4.13	4.16	263622	{}	\N	2020-10-12 15:52:55.149641
63990	PIM	2020-10-08	4.16	4.17	4.14	4.16	196990	{}	\N	2020-10-12 15:52:55.154715
63991	PIM	2020-10-09	4.15	4.16	4.14	4.16	133394	{}	\N	2020-10-12 15:52:55.159959
63992	PIM	2020-10-12	4.15	4.17	4.15	4.15	58706	{}	\N	2020-10-12 15:52:55.166106
63993	PIZ	2020-09-28	31.005	31.032	30.83	30.83	18541	{}	\N	2020-10-12 15:52:56.531568
63994	PIZ	2020-09-29	31.11	31.25	31.03	31.163	10698	{}	\N	2020-10-12 15:52:56.557645
63995	PIZ	2020-09-30	31.09	31.21	30.923	31.09	16559	{}	\N	2020-10-12 15:52:56.567622
63996	PIZ	2020-10-01	31.31	31.69	31.2724	31.545	154082	{}	\N	2020-10-12 15:52:56.578562
63997	PIZ	2020-10-02	31.06	31.412	31.06	31.3	28414	{}	\N	2020-10-12 15:52:56.587565
63998	PIZ	2020-10-05	31.55	31.8222	31.55	31.8222	23217	{}	\N	2020-10-12 15:52:56.596008
63999	PIZ	2020-10-06	31.57	31.6	31.1416	31.18	45793	{}	\N	2020-10-12 15:52:56.603243
64000	PIZ	2020-10-07	31.44	31.57	31.4167	31.515	7412	{}	\N	2020-10-12 15:52:56.610282
64001	PIZ	2020-10-08	31.76	31.775	31.671	31.775	7270	{}	\N	2020-10-12 15:52:56.616623
64002	PIZ	2020-10-09	32.08	32.2487	32.075	32.18	19465	{}	\N	2020-10-12 15:52:56.622459
64003	PIZ	2020-10-12	32.39	32.441	32.3485	32.372	2828	{}	\N	2020-10-12 15:52:56.628269
64004	PKBK	2020-09-28	11.89	12.055	11.74	11.74	23870	{}	\N	2020-10-12 15:52:57.731371
64005	PKBK	2020-09-29	11.62	11.94	11.6	11.81	22175	{}	\N	2020-10-12 15:52:57.743561
64006	PKBK	2020-09-30	11.8	12.06	11.7978	11.94	10744	{}	\N	2020-10-12 15:52:57.754918
64007	PKBK	2020-10-01	11.92	12.06	11.73	11.87	17315	{}	\N	2020-10-12 15:52:57.76763
64008	PKBK	2020-10-02	11.69	12.38	11.69	12.25	17613	{}	\N	2020-10-12 15:52:57.778205
64009	PKBK	2020-10-05	12.29	12.57	11.9	12.24	42270	{}	\N	2020-10-12 15:52:57.789367
64010	PKBK	2020-10-06	12.37	13.1	12.2854	12.69	17968	{}	\N	2020-10-12 15:52:57.800266
64011	PKBK	2020-10-07	12.5476	13.17	12.5476	13.14	11879	{}	\N	2020-10-12 15:52:57.811135
64012	PKBK	2020-10-08	13.05	13.62	12.88	13.5	11931	{}	\N	2020-10-12 15:52:57.822059
64013	PKBK	2020-10-09	13.51	13.51	13.1	13.3	14969	{}	\N	2020-10-12 15:52:57.833505
64014	PKBK	2020-10-12	13.25	13.2886	13.25	13.275	1295	{}	\N	2020-10-12 15:52:57.841935
64015	PLAB	2020-09-28	9.95	10.17	9.9	10.08	467712	{}	\N	2020-10-12 15:52:59.142443
64016	PLAB	2020-09-29	10.08	10.24	10.05	10.11	398122	{}	\N	2020-10-12 15:52:59.150374
64017	PLAB	2020-09-30	10.1	10.19	9.94	9.96	449304	{}	\N	2020-10-12 15:52:59.15668
64018	PLAB	2020-10-01	10.05	10.23	10.05	10.19	506998	{}	\N	2020-10-12 15:52:59.163929
64019	PLAB	2020-10-02	9.87	10.23	9.75	10.08	529392	{}	\N	2020-10-12 15:52:59.170116
64020	PLAB	2020-10-05	10.16	10.29	10.04	10.27	386082	{}	\N	2020-10-12 15:52:59.177024
64021	PLAB	2020-10-06	10.24	10.55	10.22	10.24	456372	{}	\N	2020-10-12 15:52:59.184133
64022	PLAB	2020-10-07	10.32	10.53	10.32	10.46	347263	{}	\N	2020-10-12 15:52:59.191113
64023	PLAB	2020-10-08	10.53	10.67	10.53	10.63	290382	{}	\N	2020-10-12 15:52:59.197585
64024	PLAB	2020-10-09	10.64	10.85	10.64	10.78	290345	{}	\N	2020-10-12 15:52:59.203271
64025	PLAB	2020-10-12	10.9	10.9	10.66	10.675	44124	{}	\N	2020-10-12 15:52:59.209348
64026	PLPC	2020-09-28	49.85	54.95	49.85	50.85	11226	{}	\N	2020-10-12 15:53:00.408106
64027	PLPC	2020-09-29	51.16	51.2	49.2237	49.6	9259	{}	\N	2020-10-12 15:53:00.416181
64028	PLPC	2020-09-30	49.98	51.99	48.72	48.72	3202	{}	\N	2020-10-12 15:53:00.422984
64029	PLPC	2020-10-01	49.02	49.05	48.77	48.77	2609	{}	\N	2020-10-12 15:53:00.430553
64030	PLPC	2020-10-02	48.34	50	47.07	48.94	2581	{}	\N	2020-10-12 15:53:00.437105
64031	PLPC	2020-10-05	50.63	50.63	48.75	49.78	6336	{}	\N	2020-10-12 15:53:00.44455
64032	PLPC	2020-10-06	49.28	51.4497	49.28	49.49	4885	{}	\N	2020-10-12 15:53:00.451974
64033	PLPC	2020-10-07	49.4901	49.65	49.335	49.65	3292	{}	\N	2020-10-12 15:53:00.461221
64034	PLPC	2020-10-08	49.4575	49.5	49.15	49.15	3023	{}	\N	2020-10-12 15:53:00.467915
64035	PLPC	2020-10-09	49.2	49.5	49	49.28	4059	{}	\N	2020-10-12 15:53:00.475235
64036	PLPC	2020-10-12	49.28	49.9105	49.28	49.9105	1883	{}	\N	2020-10-12 15:53:00.484153
64037	PLXS	2020-09-28	70.21	71.94	69.8901	71.67	143592	{}	\N	2020-10-12 15:53:02.21641
64038	PLXS	2020-09-29	71.49	71.93	70.6	70.69	153895	{}	\N	2020-10-12 15:53:02.226086
64039	PLXS	2020-09-30	70.82	71.64	70.29	70.63	231479	{}	\N	2020-10-12 15:53:02.233808
64040	PLXS	2020-10-01	70.97	71.66	70.45	71.17	131504	{}	\N	2020-10-12 15:53:02.241677
64041	PLXS	2020-10-02	69.77	71.42	69.73	70.95	121451	{}	\N	2020-10-12 15:53:02.248648
64042	PLXS	2020-10-05	71.81	73.1	71.59	72.46	123805	{}	\N	2020-10-12 15:53:02.257474
64043	PLXS	2020-10-06	72.94	73.86	71.81	71.91	198987	{}	\N	2020-10-12 15:53:02.265933
64044	PLXS	2020-10-07	72.58	73.03	71.82	71.85	210486	{}	\N	2020-10-12 15:53:02.272361
64045	PLXS	2020-10-08	72.42	73.13	72	72.78	132577	{}	\N	2020-10-12 15:53:02.278558
64046	PLXS	2020-10-09	73.56	73.64	72.79	73.23	88264	{}	\N	2020-10-12 15:53:02.284391
64047	PLXS	2020-10-12	73.33	74.06	73.33	73.83	25327	{}	\N	2020-10-12 15:53:02.289569
64048	PMM	2020-09-28	7.73	7.81	7.73	7.81	52061	{}	\N	2020-10-12 15:53:03.584927
64049	PMM	2020-09-29	7.78	7.82	7.75	7.76	156712	{}	\N	2020-10-12 15:53:03.593213
64050	PMM	2020-09-30	7.76	7.77	7.66	7.66	207203	{}	\N	2020-10-12 15:53:03.599603
64051	PMM	2020-10-01	7.72	7.72	7.66	7.7	172248	{}	\N	2020-10-12 15:53:03.607816
64052	PMM	2020-10-02	7.65	7.73	7.65	7.65	150206	{}	\N	2020-10-12 15:53:03.614186
64053	PMM	2020-10-05	7.65	7.705	7.6	7.65	224332	{}	\N	2020-10-12 15:53:03.620013
64054	PMM	2020-10-06	7.66	7.73	7.66	7.71	143051	{}	\N	2020-10-12 15:53:03.626024
64055	PMM	2020-10-07	7.72	7.7456	7.68	7.68	212302	{}	\N	2020-10-12 15:53:03.632034
64056	PMM	2020-10-08	7.68	7.7191	7.63	7.64	186504	{}	\N	2020-10-12 15:53:03.638
64057	PMM	2020-10-09	7.66	7.69	7.64	7.66	94246	{}	\N	2020-10-12 15:53:03.64306
64058	PMM	2020-10-12	7.65	7.69	7.65	7.67	36180	{}	\N	2020-10-12 15:53:03.648809
64059	PNC	2020-09-28	106.82	109.15	106.55	108.13	1619246	{}	\N	2020-10-12 15:53:04.899403
64060	PNC	2020-09-29	107.5	107.76	105.16	106.93	3041994	{}	\N	2020-10-12 15:53:04.909174
64061	PNC	2020-09-30	107.55	110.28	107.355	109.91	4738874	{}	\N	2020-10-12 15:53:04.916674
64062	PNC	2020-10-01	110.05	110.9	108.37	109.43	3087166	{}	\N	2020-10-12 15:53:04.923416
64063	PNC	2020-10-02	108.09	113	107.79	112.16	4082214	{}	\N	2020-10-12 15:53:04.929176
64064	PNC	2020-10-05	113.47	114.97	112.5	114.8	3577558	{}	\N	2020-10-12 15:53:04.933653
64065	PNC	2020-10-06	115.99	117.155	112.59	113.17	3594838	{}	\N	2020-10-12 15:53:04.939572
64066	PNC	2020-10-07	114.34	117.0408	114.245	115.18	4638299	{}	\N	2020-10-12 15:53:04.944194
64067	PNC	2020-10-08	115.8	117.03	115.1	116.54	4937553	{}	\N	2020-10-12 15:53:04.949067
64068	PNC	2020-10-09	117.11	117.22	114.84	115.48	3762416	{}	\N	2020-10-12 15:53:04.953552
64069	PNC	2020-10-12	115.51	116.56	115.217	115.855	908572	{}	\N	2020-10-12 15:53:04.958214
64070	PNRG	2020-09-28	69.48	70.75	69	69	1510	{}	\N	2020-10-12 15:53:06.12479
64071	PNRG	2020-09-29	69.99	69.99	67.52	68.59	975	{}	\N	2020-10-12 15:53:06.134195
64072	PNRG	2020-09-30	69.44	69.59	69.04	69.04	627	{}	\N	2020-10-12 15:53:06.166334
64073	PNRG	2020-10-01	69.18	71.4	69.18	71.4	1215	{}	\N	2020-10-12 15:53:06.175945
64074	PNRG	2020-10-02	67.45	67.45	67.45	67.45	314	{}	\N	2020-10-12 15:53:06.183552
64075	PNRG	2020-10-05	70	70	67.45	68.85	1813	{}	\N	2020-10-12 15:53:06.195921
64076	PNRG	2020-10-06	66.09	66.09	66.09	66.09	287	{}	\N	2020-10-12 15:53:06.20453
64077	PNRG	2020-10-07	69.5	69.5	68.89	68.89	339	{}	\N	2020-10-12 15:53:06.212809
64078	PNRG	2020-10-08	68.35	71.8	67.99	70.01	7871	{}	\N	2020-10-12 15:53:06.222487
64079	PPT	2020-09-28	4.38	4.43	4.38	4.41	185882	{}	\N	2020-10-12 15:53:08.735733
64080	PPT	2020-09-29	4.42	4.45	4.41	4.44	1017108	{}	\N	2020-10-12 15:53:08.744197
64081	PPT	2020-09-30	4.42	4.45	4.41	4.45	778922	{}	\N	2020-10-12 15:53:08.753707
64082	PPT	2020-10-01	4.49	4.51	4.45	4.51	912134	{}	\N	2020-10-12 15:53:08.7626
64083	PPT	2020-10-02	4.49	4.5799	4.466	4.56	613468	{}	\N	2020-10-12 15:53:08.770763
64084	PPT	2020-10-05	4.57	4.59	4.5544	4.59	345592	{}	\N	2020-10-12 15:53:08.778565
64085	PPT	2020-10-06	4.57	4.62	4.56	4.58	495083	{}	\N	2020-10-12 15:53:08.786065
64086	PPT	2020-10-07	4.57	4.61	4.57	4.6	627702	{}	\N	2020-10-12 15:53:08.793413
64087	PPT	2020-10-08	4.59	4.65	4.59	4.63	395484	{}	\N	2020-10-12 15:53:08.802118
64088	PPT	2020-10-09	4.63	4.63	4.61	4.63	608420	{}	\N	2020-10-12 15:53:08.810954
64089	PPT	2020-10-12	4.58	4.6	4.56	4.5885	638628	{}	\N	2020-10-12 15:53:08.818543
64090	PHG	2020-09-28	46.34	46.61	46.32	46.49	453982	{}	\N	2020-10-12 15:53:09.999767
64091	PHG	2020-09-29	47.32	47.64	47.1	47.22	1184770	{}	\N	2020-10-12 15:53:10.008617
64092	PHG	2020-09-30	47.23	47.52	46.91	47.15	912342	{}	\N	2020-10-12 15:53:10.017579
64093	PHG	2020-10-01	47.59	47.66	47.1619	47.34	567156	{}	\N	2020-10-12 15:53:10.025917
64094	PHG	2020-10-02	46.78	47.44	46.6818	47.27	1501917	{}	\N	2020-10-12 15:53:10.032435
64095	PHG	2020-10-05	48.01	48.39	47.95	48.23	1273035	{}	\N	2020-10-12 15:53:10.039165
64096	PHG	2020-10-06	47.81	47.82	46.81	46.96	767068	{}	\N	2020-10-12 15:53:10.045521
64097	PHG	2020-10-07	46.96	47.32	46.81	47.23	525388	{}	\N	2020-10-12 15:53:10.052054
64098	PHG	2020-10-08	47.15	47.31	46.99	47.16	567650	{}	\N	2020-10-12 15:53:10.058574
64099	PHG	2020-10-09	47.32	47.96	47.29	47.84	531583	{}	\N	2020-10-12 15:53:10.064872
64100	PHG	2020-10-12	48.69	48.81	48.63	48.7	373076	{}	\N	2020-10-12 15:53:10.07101
64101	PID	2020-09-28	12.75	12.8693	12.74	12.8	153112	{}	\N	2020-10-12 15:53:10.989546
64102	PID	2020-09-29	12.79	12.83	12.63	12.72	144656	{}	\N	2020-10-12 15:53:10.99879
64103	PID	2020-09-30	12.75	12.8699	12.69	12.69	159001	{}	\N	2020-10-12 15:53:11.009077
64104	PID	2020-10-01	12.77	12.8168	12.6412	12.77	84239	{}	\N	2020-10-12 15:53:11.01725
64105	PID	2020-10-02	12.6	12.86	12.6	12.85	107916	{}	\N	2020-10-12 15:53:11.024601
64106	PID	2020-10-05	12.93	13.07	12.875	13.04	100299	{}	\N	2020-10-12 15:53:11.032564
64107	PID	2020-10-06	13.09	13.1241	12.92	12.95	91415	{}	\N	2020-10-12 15:53:11.042755
64108	PID	2020-10-07	13.03	13.14	13	13.1	102791	{}	\N	2020-10-12 15:53:11.051953
64109	PID	2020-10-08	13.16	13.37	13.16	13.37	97470	{}	\N	2020-10-12 15:53:11.059863
64110	PID	2020-10-09	13.44	13.465	13.315	13.35	177199	{}	\N	2020-10-12 15:53:11.06878
64111	PID	2020-10-12	13.36	13.39	13.36	13.371	55699	{}	\N	2020-10-12 15:53:11.078658
64112	PIRS	2020-09-28	2	2.17	1.96	2.14	360763	{}	\N	2020-10-12 15:53:12.292933
64113	PIRS	2020-09-29	2.15	2.15	2.01	2.06	532228	{}	\N	2020-10-12 15:53:12.30195
64114	PIRS	2020-09-30	2.03	2.15	2	2.07	389611	{}	\N	2020-10-12 15:53:12.30964
64115	PIRS	2020-10-01	2.09	2.1	1.9933	2.07	338425	{}	\N	2020-10-12 15:53:12.318332
64116	PIRS	2020-10-02	2	2.1	1.95	1.96	449326	{}	\N	2020-10-12 15:53:12.327556
64117	PIRS	2020-10-05	2.03	2.12	1.98	2.08	385467	{}	\N	2020-10-12 15:53:12.335926
64118	PIRS	2020-10-06	2.08	2.14	2.02	2.02	393906	{}	\N	2020-10-12 15:53:12.344786
64119	PIRS	2020-10-07	2.05	2.1	2	2.09	281863	{}	\N	2020-10-12 15:53:12.353505
64120	PIRS	2020-10-08	2.11	2.175	2.1	2.16	169391	{}	\N	2020-10-12 15:53:12.361963
64121	PIRS	2020-10-09	2.13	2.23	2.11	2.19	466868	{}	\N	2020-10-12 15:53:12.370655
64122	PIRS	2020-10-12	2.21	2.295	2.19	2.2881	141310	{}	\N	2020-10-12 15:53:12.378534
64123	PKB	2020-09-28	35.385	35.9291	35.385	35.85	18204	{}	\N	2020-10-12 15:53:13.996636
64124	PKB	2020-09-29	35.99	35.99	35.48	35.7	17280	{}	\N	2020-10-12 15:53:14.011806
64125	PKB	2020-09-30	36.12	36.56	35.925	36.24	43142	{}	\N	2020-10-12 15:53:14.02148
64126	PKB	2020-10-01	36.52	36.715	36.1404	36.63	14186	{}	\N	2020-10-12 15:53:14.028222
64127	PKB	2020-10-02	35.69	37.19	35.69	37.06	21491	{}	\N	2020-10-12 15:53:14.033427
64128	PKB	2020-10-05	37.54	37.94	37.42	37.89	20890	{}	\N	2020-10-12 15:53:14.038687
64129	PKB	2020-10-06	38	38.55	37.57	37.765	32586	{}	\N	2020-10-12 15:53:14.044111
64130	PKB	2020-10-07	38.34	38.81	38.22	38.4586	33145	{}	\N	2020-10-12 15:53:14.048617
64131	PKB	2020-10-08	38.92	39.54	38.92	39.48	63385	{}	\N	2020-10-12 15:53:14.053888
64132	PKB	2020-10-09	39.99	40	39.49	39.7	23394	{}	\N	2020-10-12 15:53:14.058992
64133	PKB	2020-10-12	39.8525	40.11	39.8314	40	3283	{}	\N	2020-10-12 15:53:14.064138
64134	PLAY	2020-09-28	16.21	16.63	15.52	16.37	4432883	{}	\N	2020-10-12 15:53:14.994059
64135	PLAY	2020-09-29	16.1658	16.1723	15.01	15.17	5834530	{}	\N	2020-10-12 15:53:15.000348
64136	PLAY	2020-09-30	15.41	16.04	15.14	15.16	4253962	{}	\N	2020-10-12 15:53:15.005543
64137	PLAY	2020-10-01	15.24	15.56	15.06	15.55	3109165	{}	\N	2020-10-12 15:53:15.012136
64138	PLAY	2020-10-02	14.76	15.82	14.55	15.71	3377593	{}	\N	2020-10-12 15:53:15.018795
64139	PLAY	2020-10-05	15.83	16.2	15.32	15.39	3271745	{}	\N	2020-10-12 15:53:15.024524
64140	PLAY	2020-10-06	15.64	16.6	15.26	15.46	5491273	{}	\N	2020-10-12 15:53:15.030265
64141	PLAY	2020-10-07	15.95	16.37	15.26	15.64	4825613	{}	\N	2020-10-12 15:53:15.036211
64142	PLAY	2020-10-08	15.94	16.135	15.44	16.04	3159040	{}	\N	2020-10-12 15:53:15.043403
64143	PLAY	2020-10-09	16.18	16.38	15.81	16.1	2178530	{}	\N	2020-10-12 15:53:15.048303
64144	PLAY	2020-10-12	16.07	16.1099	15.54	15.6032	1014796	{}	\N	2020-10-12 15:53:15.053838
64145	PLOW	2020-09-28	34.69	35.61	34.15	35.09	170883	{}	\N	2020-10-12 15:53:16.010995
64146	PLOW	2020-09-29	35.05	35.05	33.88	34.41	352464	{}	\N	2020-10-12 15:53:16.021485
64147	PLOW	2020-09-30	34.5	35.15	34.07	34.2	188267	{}	\N	2020-10-12 15:53:16.028621
64148	PLOW	2020-10-01	34.41	34.74	33.88	34.58	227394	{}	\N	2020-10-12 15:53:16.036008
64149	PLOW	2020-10-02	33.93	35.35	33.93	34.91	198226	{}	\N	2020-10-12 15:53:16.042151
64150	PLOW	2020-10-05	35.36	35.395	34.44	34.72	253286	{}	\N	2020-10-12 15:53:16.048172
64151	PLOW	2020-10-06	34.98	35.79	34.56	34.62	254038	{}	\N	2020-10-12 15:53:16.056645
64152	PLOW	2020-10-07	35.09	35.61	34.83	35.35	307711	{}	\N	2020-10-12 15:53:16.064479
64153	PLOW	2020-10-08	35.66	36.19	35.36	36.1	106485	{}	\N	2020-10-12 15:53:16.069887
64154	PLOW	2020-10-09	36.59	37.12	35.565	36.83	176146	{}	\N	2020-10-12 15:53:16.075187
64155	PLOW	2020-10-12	36.82	37.12	36.59	37.01	21450	{}	\N	2020-10-12 15:53:16.081424
64156	PLUS	2020-09-28	72.79	74.06	72.42	73.44	47538	{}	\N	2020-10-12 15:53:17.582513
64157	PLUS	2020-09-29	73.51	74.32	72.4704	73.83	41735	{}	\N	2020-10-12 15:53:17.593641
64158	PLUS	2020-09-30	73.89	75.485	73.01	73.2	51643	{}	\N	2020-10-12 15:53:17.602292
64159	PLUS	2020-10-01	73.13	73.51	72.3733	73.23	43463	{}	\N	2020-10-12 15:53:17.60955
64160	PLUS	2020-10-02	71.9	73.865	71.54	73.25	31726	{}	\N	2020-10-12 15:53:17.615672
64161	PLUS	2020-10-05	73.5	76.075	73.5	76.04	51208	{}	\N	2020-10-12 15:53:17.622163
64162	PLUS	2020-10-06	76.65	78.62	75.64	76.56	70765	{}	\N	2020-10-12 15:53:17.628791
64163	PLUS	2020-10-07	77.28	77.49	76.08	76.53	72106	{}	\N	2020-10-12 15:53:17.634669
64164	PLUS	2020-10-08	77.42	77.78	76.55	77.38	32141	{}	\N	2020-10-12 15:53:17.640839
64165	PLUS	2020-10-09	78.13	78.79	76.73	76.92	27653	{}	\N	2020-10-12 15:53:17.647381
64166	PLUS	2020-10-12	76.9	78.33	76.605	78.3	15872	{}	\N	2020-10-12 15:53:17.653616
64167	PM	2020-09-28	76.3	77.66	76.17	76.31	3820381	{}	\N	2020-10-12 15:53:18.893906
64168	PM	2020-09-29	75.78	76.12	74.3	74.34	7747967	{}	\N	2020-10-12 15:53:18.9031
64169	PM	2020-09-30	74.54	75.29	74.35	74.99	10403253	{}	\N	2020-10-12 15:53:18.910233
64170	PM	2020-10-01	74.7	74.97	73.51	74.14	7758543	{}	\N	2020-10-12 15:53:18.917407
64171	PM	2020-10-02	73.58	75.1265	73.45	74.46	5748756	{}	\N	2020-10-12 15:53:18.923914
64172	PM	2020-10-05	74.78	76.06	74.52	75.91	7353053	{}	\N	2020-10-12 15:53:18.933243
64173	PM	2020-10-06	76.23	76.74	75.31	75.5	6336606	{}	\N	2020-10-12 15:53:18.939705
64174	PM	2020-10-07	76.85	79.7	76.56	79.05	13097746	{}	\N	2020-10-12 15:53:18.946013
64175	PM	2020-10-08	78.93	79.3774	77.85	78.33	12764318	{}	\N	2020-10-12 15:53:18.95266
64176	PM	2020-10-09	79.06	79.75	78.68	78.88	5954288	{}	\N	2020-10-12 15:53:18.957516
64177	PM	2020-10-12	79.09	79.57	78.52	79.26	1939695	{}	\N	2020-10-12 15:53:18.96258
64178	POOL	2020-09-28	320.67	325.965	315.485	325.2	302973	{}	\N	2020-10-12 15:53:20.482764
64179	POOL	2020-09-29	325.59	330.45	324.76	328.54	310864	{}	\N	2020-10-12 15:53:20.494035
64180	POOL	2020-09-30	328.56	338.88	328.56	334.54	332488	{}	\N	2020-10-12 15:53:20.502394
64181	POOL	2020-10-01	338.28	338.57	325.93	328.28	376736	{}	\N	2020-10-12 15:53:20.508739
64182	POOL	2020-10-02	319.88	327.29	319.08	322.79	717103	{}	\N	2020-10-12 15:53:20.513834
64183	POOL	2020-10-05	323.01	329.425	321.3	327.75	674174	{}	\N	2020-10-12 15:53:20.521828
64184	POOL	2020-10-06	327.63	331.61	321.115	328.01	8118535	{}	\N	2020-10-12 15:53:20.526776
64185	POOL	2020-10-07	329.05	334	327.75	331.64	546788	{}	\N	2020-10-12 15:53:20.532015
64186	POOL	2020-10-08	333.01	342.8799	333.01	340.85	408662	{}	\N	2020-10-12 15:53:20.537221
64187	POOL	2020-10-09	347.4744	350.04	343.57	346.55	453631	{}	\N	2020-10-12 15:53:20.54215
64188	POOL	2020-10-12	349.99	352.54	347	351.46	82010	{}	\N	2020-10-12 15:53:20.5475
64189	PPG	2020-09-28	122.38	123.63	121.63	121.74	1064991	{}	\N	2020-10-12 15:53:21.718457
64190	PPG	2020-09-29	122.2	122.77	120.29	120.33	1268717	{}	\N	2020-10-12 15:53:21.725596
64191	PPG	2020-09-30	121.12	123.6	120.19	122.08	2721563	{}	\N	2020-10-12 15:53:21.732551
64192	PPG	2020-10-01	122.41	124.01	121.195	121.78	2712622	{}	\N	2020-10-12 15:53:21.738909
64193	PPG	2020-10-02	120.19	123.88	120	123.27	2565155	{}	\N	2020-10-12 15:53:21.744826
64194	PPG	2020-10-05	124.2	126.485	124.2	125.92	2839150	{}	\N	2020-10-12 15:53:21.751804
64195	PPG	2020-10-06	127	129.04	124.37	124.55	3244976	{}	\N	2020-10-12 15:53:21.758737
64196	PPG	2020-10-07	126.56	129.68	126.45	128.94	2639679	{}	\N	2020-10-12 15:53:21.764186
64197	PPG	2020-10-08	132.23	136.51	131.3	134.99	6270563	{}	\N	2020-10-12 15:53:21.769632
64198	PPG	2020-10-09	135.46	137	133.45	135.88	4048429	{}	\N	2020-10-12 15:53:21.775729
64199	PPG	2020-10-12	136.35	136.55	134.51	134.955	635101	{}	\N	2020-10-12 15:53:21.780959
64200	PIXY	2020-09-28	3.68	3.79	3.66	3.74	123376	{}	\N	2020-10-12 15:53:23.153209
64201	PIXY	2020-09-29	3.73	3.89	3.73	3.89	185422	{}	\N	2020-10-12 15:53:23.164982
64202	PIXY	2020-09-30	3.91	3.945	3.825	3.84	140002	{}	\N	2020-10-12 15:53:23.172628
64203	PIXY	2020-10-01	3.83	3.94	3.79	3.93	161705	{}	\N	2020-10-12 15:53:23.179649
64204	PIXY	2020-10-02	3.87	3.87	3.63	3.69	314434	{}	\N	2020-10-12 15:53:23.185446
64205	PIXY	2020-10-05	3.72	3.7293	3.6305	3.68	91159	{}	\N	2020-10-12 15:53:23.19048
64206	PIXY	2020-10-06	3.66	3.74	3.6	3.64	134638	{}	\N	2020-10-12 15:53:23.195714
64207	PIXY	2020-10-07	3.63	3.67	3.55	3.62	231858	{}	\N	2020-10-12 15:53:23.202162
64208	PIXY	2020-10-08	3.58	3.6	3.4	3.59	735563	{}	\N	2020-10-12 15:53:23.209231
64209	PIXY	2020-10-09	2.69	2.8	2.68	2.79	5946403	{}	\N	2020-10-12 15:53:23.216781
64210	PIXY	2020-10-12	2.84	2.85	2.68	2.6868	853283	{}	\N	2020-10-12 15:53:23.223956
64211	PLBC	2020-09-28	19.95	19.95	19.45	19.45	2553	{}	\N	2020-10-12 15:53:25.00063
64212	PLBC	2020-09-29	19.4	19.77	19.4	19.77	1330	{}	\N	2020-10-12 15:53:25.008526
64213	PLBC	2020-09-30	19.94	19.99	19.5001	19.68	3157	{}	\N	2020-10-12 15:53:25.015042
64214	PLBC	2020-10-01	19.5501	19.97	19.55	19.63	5684	{}	\N	2020-10-12 15:53:25.020435
64215	PLBC	2020-10-02	19.37	19.77	19.37	19.57	3494	{}	\N	2020-10-12 15:53:25.025607
64216	PLBC	2020-10-05	19.625	20	19.47	19.64	4284	{}	\N	2020-10-12 15:53:25.030965
64217	PLBC	2020-10-06	20	20.57	19.7992	20.28	11067	{}	\N	2020-10-12 15:53:25.03696
64218	PLBC	2020-10-07	19.71	20.88	19.71	20.81	3882	{}	\N	2020-10-12 15:53:25.042208
64219	PLBC	2020-10-08	20.57	20.6	20.195	20.43	2047	{}	\N	2020-10-12 15:53:25.047857
64220	PLBC	2020-10-09	20.35	20.43	19.62	20.39	2567	{}	\N	2020-10-12 15:53:25.053452
64221	PLT	2020-09-28	11.81	12.39	11.7524	12.27	591611	{}	\N	2020-10-12 15:53:26.423018
64222	PLT	2020-09-29	12.37	12.66	12.148	12.46	1157801	{}	\N	2020-10-12 15:53:26.432966
64223	PLT	2020-09-30	12.5	12.68	11.78	11.84	1585674	{}	\N	2020-10-12 15:53:26.440139
64224	PLT	2020-10-01	12.02	12.18	11.77	12.12	1206168	{}	\N	2020-10-12 15:53:26.445655
64225	PLT	2020-10-02	11.8	12.65	11.735	12.48	901988	{}	\N	2020-10-12 15:53:26.4514
64226	PLT	2020-10-05	12.65	13.12	12.5	13.08	1813797	{}	\N	2020-10-12 15:53:26.456335
64227	PLT	2020-10-06	13.23	13.6652	12.71	12.92	1404576	{}	\N	2020-10-12 15:53:26.461052
64228	PLT	2020-10-07	12.93	13.31	12.86	13.22	936094	{}	\N	2020-10-12 15:53:26.465857
64229	PLT	2020-10-08	13.5	13.7	13.19	13.61	760812	{}	\N	2020-10-12 15:53:26.470461
64230	PLT	2020-10-09	13.84	14.72	13.795	14.56	2017386	{}	\N	2020-10-12 15:53:26.475189
64231	PLT	2020-10-12	15.3	17.24	15.15	17.1	1895316	{}	\N	2020-10-12 15:53:26.480067
64232	PLYA	2020-09-28	4.14	4.3	4.1	4.28	734765	{}	\N	2020-10-12 15:53:27.456095
64233	PLYA	2020-09-29	4.25	4.3801	4.175	4.37	340628	{}	\N	2020-10-12 15:53:27.470103
64234	PLYA	2020-09-30	4.37	4.5	4.12	4.19	658864	{}	\N	2020-10-12 15:53:27.477291
64235	PLYA	2020-10-01	4.23	4.41	4.08	4.1	566749	{}	\N	2020-10-12 15:53:27.485141
64236	PLYA	2020-10-02	4.06	4.18	4	4.12	547584	{}	\N	2020-10-12 15:53:27.49354
64237	PLYA	2020-10-05	4.16	4.215	4.1	4.19	466970	{}	\N	2020-10-12 15:53:27.500981
64238	PLYA	2020-10-06	4.2	4.37	4.18	4.23	550329	{}	\N	2020-10-12 15:53:27.507427
64239	PLYA	2020-10-07	4.27	4.3	4.13	4.16	537434	{}	\N	2020-10-12 15:53:27.514047
64240	PLYA	2020-10-08	4.2	4.24	4.14	4.19	9296132	{}	\N	2020-10-12 15:53:27.522929
64241	PLYA	2020-10-09	4.25	4.32	4.19	4.23	306921	{}	\N	2020-10-12 15:53:27.528717
64242	PLYA	2020-10-12	4.2	4.202	4.11	4.155	303194	{}	\N	2020-10-12 15:53:27.535257
64243	PMF	2020-09-28	13.2	13.26	13.11	13.22	56212	{}	\N	2020-10-12 15:53:28.713263
64244	PMF	2020-09-29	13.23	13.33	13.22	13.33	205478	{}	\N	2020-10-12 15:53:28.724804
64245	PMF	2020-09-30	13.31	13.5	13.27	13.35	238488	{}	\N	2020-10-12 15:53:28.732944
64246	PMF	2020-10-01	13.43	13.4436	13.33	13.36	85966	{}	\N	2020-10-12 15:53:28.739835
64247	PMF	2020-10-02	13.32	13.5	13.32	13.48	120149	{}	\N	2020-10-12 15:53:28.745844
64248	PMF	2020-10-05	13.48	13.48	13.4	13.43	78028	{}	\N	2020-10-12 15:53:28.751174
64249	PMF	2020-10-06	13.46	13.55	13.43	13.47	77398	{}	\N	2020-10-12 15:53:28.756613
64250	PMF	2020-10-07	13.53	13.565	13.455	13.46	91457	{}	\N	2020-10-12 15:53:28.762529
64251	PMF	2020-10-08	13.44	13.53	13.41	13.45	50274	{}	\N	2020-10-12 15:53:28.767443
64252	PMF	2020-10-09	13.41	13.4916	13.38	13.39	102758	{}	\N	2020-10-12 15:53:28.772424
64253	PMF	2020-10-12	13.39	13.401	13.26	13.3	83942	{}	\N	2020-10-12 15:53:28.777739
64254	PNBK	2020-09-28	6.6	7.08	6.57	6.785	25214	{}	\N	2020-10-12 15:53:30.104195
64255	PNBK	2020-09-29	6.3	6.79	6.2637	6.79	3734	{}	\N	2020-10-12 15:53:30.114321
64256	PNBK	2020-09-30	6.675	6.99	6.48	6.92	2621	{}	\N	2020-10-12 15:53:30.120568
64257	PNBK	2020-10-01	6.92	7.5	6.695	7.5	13210	{}	\N	2020-10-12 15:53:30.127527
64258	PNBK	2020-10-02	6.98	7	6.7501	6.88	4725	{}	\N	2020-10-12 15:53:30.133569
64259	PNBK	2020-10-05	6.88	7.01	6.8311	6.905	1678	{}	\N	2020-10-12 15:53:30.139133
64260	PNBK	2020-10-06	7	7.0001	6.91	7	3513	{}	\N	2020-10-12 15:53:30.144297
64261	PNBK	2020-10-07	7.0497	7.95	6.4701	7.03	42560	{}	\N	2020-10-12 15:53:30.148602
64262	PNBK	2020-10-08	7	7.03	6.56	6.8673	4475	{}	\N	2020-10-12 15:53:30.152613
64263	PNBK	2020-10-09	6.73	6.76	6.66	6.7416	4129	{}	\N	2020-10-12 15:53:30.157152
64264	PNBK	2020-10-12	6.7717	6.81	6.7717	6.81	271	{}	\N	2020-10-12 15:53:30.162034
64265	PNW	2020-09-28	74.17	74.85	73.63	73.89	768472	{}	\N	2020-10-12 15:53:31.146588
64266	PNW	2020-09-29	73.82	74.65	73.38	73.45	1277037	{}	\N	2020-10-12 15:53:31.153932
64267	PNW	2020-09-30	74.16	75.2	73.76	74.55	2429786	{}	\N	2020-10-12 15:53:31.160019
64268	PNW	2020-10-01	74.97	75.86	74.35	75.32	1642406	{}	\N	2020-10-12 15:53:31.166542
64269	PNW	2020-10-02	74.37	78.1	74.18	77.86	2026025	{}	\N	2020-10-12 15:53:31.171516
64270	PNW	2020-10-05	77.92	80.34	77.53	79.64	2319342	{}	\N	2020-10-12 15:53:31.176473
64271	PNW	2020-10-06	80.01	82.54	79.39	81.12	2400603	{}	\N	2020-10-12 15:53:31.181475
64272	PNW	2020-10-07	81.67	83.4	81.18	82.95	2046922	{}	\N	2020-10-12 15:53:31.18613
64273	PNW	2020-10-08	82.88	84.11	82.85	83.54	1521292	{}	\N	2020-10-12 15:53:31.191648
64274	PNW	2020-10-09	84.07	84.24	82.12	82.56	1319239	{}	\N	2020-10-12 15:53:31.196271
64275	PNW	2020-10-12	81.65	81.9	81.07	81.835	533022	{}	\N	2020-10-12 15:53:31.201571
64276	POWI	2020-09-28	53.6	54.48	52.93	54.3	235113	{}	\N	2020-10-12 15:53:32.216147
64277	POWI	2020-09-29	54.3	55.88	53.97	55.39	303205	{}	\N	2020-10-12 15:53:32.226174
64278	POWI	2020-09-30	55.19	56.675	55.05	55.4	498799	{}	\N	2020-10-12 15:53:32.232554
64279	POWI	2020-10-01	56.07	56.28	54.8	55.17	479204	{}	\N	2020-10-12 15:53:32.238721
64280	POWI	2020-10-02	54.13	54.74	53.68	53.79	253959	{}	\N	2020-10-12 15:53:32.244298
64281	POWI	2020-10-05	54.37	56.1	53.91	56.04	313616	{}	\N	2020-10-12 15:53:32.248706
64282	POWI	2020-10-06	56.01	57.36	55.79	56.36	352768	{}	\N	2020-10-12 15:53:32.253093
64283	POWI	2020-10-07	56.99	57.2	55.85	56.57	331543	{}	\N	2020-10-12 15:53:32.257484
64284	POWI	2020-10-08	57.2	58.95	57.2	58.45	266721	{}	\N	2020-10-12 15:53:32.262155
64285	POWI	2020-10-09	59.28	61.12	58.67	60.57	394258	{}	\N	2020-10-12 15:53:32.266718
64286	POWI	2020-10-12	61.53	62.53	61.12	62.12	110496	{}	\N	2020-10-12 15:53:32.271353
64287	PPSI	2020-09-28	1.59	1.76	1.5	1.64	1869683	{}	\N	2020-10-12 15:53:33.287495
64288	PPSI	2020-09-29	1.59	1.59	1.46	1.56	573155	{}	\N	2020-10-12 15:53:33.295288
64289	PPSI	2020-09-30	1.52	1.66	1.5	1.54	335399	{}	\N	2020-10-12 15:53:33.302123
64290	PPSI	2020-10-01	1.52	1.6	1.45	1.45	231279	{}	\N	2020-10-12 15:53:33.308267
64291	PPSI	2020-10-02	1.4	1.58	1.33	1.48	365934	{}	\N	2020-10-12 15:53:33.315412
64292	PPSI	2020-10-05	1.48	1.755	1.465	1.57	945205	{}	\N	2020-10-12 15:53:33.32154
64293	PPSI	2020-10-06	3.32	9.43	2.9	4.8	324736038	{}	\N	2020-10-12 15:53:33.326901
64294	PPSI	2020-10-07	5.55	7.41	4.84	6.89	42880224	{}	\N	2020-10-12 15:53:33.33309
64295	PPSI	2020-10-08	5.82	6	5.14	5.15	6122972	{}	\N	2020-10-12 15:53:33.338845
64296	PPSI	2020-10-09	5.15	5.49	4.21	4.5	3586664	{}	\N	2020-10-12 15:53:33.345543
64297	PPSI	2020-10-12	4.42	4.74	4.39	4.6	673212	{}	\N	2020-10-12 15:53:33.35221
64298	PRI	2020-09-28	113.95	115.53	113.615	114.76	348666	{}	\N	2020-10-12 15:53:35.185699
64299	PRI	2020-09-29	114.55	114.72	111.85	112.94	306581	{}	\N	2020-10-12 15:53:35.193715
64300	PRI	2020-09-30	112.81	113.87	111.45	113.14	1026859	{}	\N	2020-10-12 15:53:35.201968
64301	PRI	2020-10-01	113.14	114.495	111.7	113.2	403887	{}	\N	2020-10-12 15:53:35.225227
64302	PRI	2020-10-02	111.51	114.04	111.13	112.67	485363	{}	\N	2020-10-12 15:53:35.247318
64303	PRI	2020-10-05	113.95	116.63	113.95	114.22	470240	{}	\N	2020-10-12 15:53:35.264247
64304	PRI	2020-10-06	115.05	115.115	111.36	112.67	804362	{}	\N	2020-10-12 15:53:35.278014
64305	PRI	2020-10-07	114.08	114.47	112.7	114.01	560535	{}	\N	2020-10-12 15:53:35.297292
64306	PRI	2020-10-08	115.11	116.27	114.19	115.13	580501	{}	\N	2020-10-12 15:53:35.315068
64307	PRI	2020-10-09	116.3	116.99	114.54	114.87	268209	{}	\N	2020-10-12 15:53:35.327144
64308	PRI	2020-10-12	114.65	116.235	114.65	116.04	61849	{}	\N	2020-10-12 15:53:35.337202
64309	PINC	2020-09-28	32.36	32.735	32.025	32.57	369002	{}	\N	2020-10-12 15:53:36.809635
64310	PINC	2020-09-29	32.6	32.83	32.11	32.21	514127	{}	\N	2020-10-12 15:53:36.825777
64311	PINC	2020-09-30	32.23	33.15	31.72	32.83	382368	{}	\N	2020-10-12 15:53:36.835555
64312	PINC	2020-10-01	32.75	33.05	31.84	32.39	352332	{}	\N	2020-10-12 15:53:36.844295
64313	PINC	2020-10-02	32.22	33.35	32.22	33.07	471209	{}	\N	2020-10-12 15:53:36.856968
64314	PINC	2020-10-05	33.38	34.35	33.28	34.25	526781	{}	\N	2020-10-12 15:53:36.864726
64315	PINC	2020-10-06	34.29	34.55	33.65	33.72	459812	{}	\N	2020-10-12 15:53:36.870954
64316	PINC	2020-10-07	33.88	34.2	33.03	33.12	469490	{}	\N	2020-10-12 15:53:36.876869
64317	PINC	2020-10-08	33.29	33.7709	33.08	33.31	426866	{}	\N	2020-10-12 15:53:36.885106
64318	PINC	2020-10-09	33.37	33.795	33.09	33.25	431392	{}	\N	2020-10-12 15:53:36.89234
64319	PINC	2020-10-12	33.29	33.3593	33.04	33.14	43695	{}	\N	2020-10-12 15:53:36.898153
64320	PK	2020-09-28	10.14	10.35	9.98	10.21	3238689	{}	\N	2020-10-12 15:53:37.855812
64321	PK	2020-09-29	10.56	10.56	9.95	10.13	7384828	{}	\N	2020-10-12 15:53:37.868977
64322	PK	2020-09-30	10.15	10.62	9.895	9.99	11096972	{}	\N	2020-10-12 15:53:37.878159
64323	PK	2020-10-01	10.14	10.39	9.96	10.39	8432863	{}	\N	2020-10-12 15:53:37.886425
64324	PK	2020-10-02	9.95	10.65	9.8501	10.59	8082048	{}	\N	2020-10-12 15:53:37.89319
64325	PK	2020-10-05	10.69	10.805	10.3	10.55	6478097	{}	\N	2020-10-12 15:53:37.899198
64326	PK	2020-10-06	10.71	10.84	10.4	10.53	6365488	{}	\N	2020-10-12 15:53:37.905257
64327	PK	2020-10-07	10.59	10.99	10.57	10.89	5145541	{}	\N	2020-10-12 15:53:37.912227
64328	PK	2020-10-08	10.94	11.09	10.7	11.08	5217466	{}	\N	2020-10-12 15:53:37.919512
64329	PK	2020-10-09	11.08	11.2	10.735	10.79	4791819	{}	\N	2020-10-12 15:53:37.926572
64330	PK	2020-10-12	10.69	10.785	10.5	10.76	1973045	{}	\N	2020-10-12 15:53:37.934197
64331	PKOH	2020-09-28	15.92	16.95	15.92	16.47	48607	{}	\N	2020-10-12 15:53:39.327852
64332	PKOH	2020-09-29	16.35	16.57	16.05	16.2	20959	{}	\N	2020-10-12 15:53:39.339551
64333	PKOH	2020-09-30	16.38	16.835	15.95	16.07	19491	{}	\N	2020-10-12 15:53:39.347868
64334	PKOH	2020-10-01	16.46	16.46	15.9	15.96	35916	{}	\N	2020-10-12 15:53:39.354478
64335	PKOH	2020-10-02	16.015	17.46	15.51	17.29	42132	{}	\N	2020-10-12 15:53:39.360326
64336	PKOH	2020-10-05	17.43	18.67	16.71	17.15	47114	{}	\N	2020-10-12 15:53:39.365389
64337	PKOH	2020-10-06	17.16	17.625	16.51	16.61	40476	{}	\N	2020-10-12 15:53:39.372932
64338	PKOH	2020-10-07	16.75	17.75	16.5	16.75	49161	{}	\N	2020-10-12 15:53:39.378486
64339	PKOH	2020-10-08	16.92	18.29	16.92	18.12	68352	{}	\N	2020-10-12 15:53:39.384251
64340	PKOH	2020-10-09	18.42	18.485	16.58	18.03	23892	{}	\N	2020-10-12 15:53:39.38924
64341	PKOH	2020-10-12	18.31	18.31	17.9	18.27	28404	{}	\N	2020-10-12 15:53:39.394074
64342	PLTM	2020-09-28	8.5	8.8	8.47	8.74	26415	{}	\N	2020-10-12 15:53:41.101882
64343	PLTM	2020-09-29	8.74	8.8423	8.625	8.76	30063	{}	\N	2020-10-12 15:53:41.109899
64344	PLTM	2020-09-30	8.9	8.95	8.76	8.855	16593	{}	\N	2020-10-12 15:53:41.117002
64345	PLTM	2020-10-01	8.95	8.95	8.77	8.94	13441	{}	\N	2020-10-12 15:53:41.12554
64346	PLTM	2020-10-02	8.83	8.83	8.69	8.7	16711	{}	\N	2020-10-12 15:53:41.170629
64347	PLTM	2020-10-05	8.78	8.92	8.76	8.872	22540	{}	\N	2020-10-12 15:53:41.175083
64348	PLTM	2020-10-06	8.7	8.89	8.48	8.48	47511	{}	\N	2020-10-12 15:53:41.180676
64349	PLTM	2020-10-07	8.5	8.5733	8.48	8.52	45546	{}	\N	2020-10-12 15:53:41.186461
64350	PLTM	2020-10-08	8.59	8.6	8.52	8.53	9162	{}	\N	2020-10-12 15:53:41.191995
64351	PLTM	2020-10-09	8.95	8.95	8.7415	8.81	19970	{}	\N	2020-10-12 15:53:41.197014
64352	PLTM	2020-10-12	8.81	8.81	8.6969	8.6969	4478	{}	\N	2020-10-12 15:53:41.202558
64353	PMBC	2020-09-28	3.66	3.82	3.66	3.7	11864	{}	\N	2020-10-12 15:53:42.56846
64354	PMBC	2020-09-29	3.6592	3.77	3.63	3.68	10243	{}	\N	2020-10-12 15:53:42.578763
64355	PMBC	2020-09-30	3.61	3.74	3.57	3.72	39974	{}	\N	2020-10-12 15:53:42.586393
64356	PMBC	2020-10-01	3.69	3.72	3.54	3.68	38860	{}	\N	2020-10-12 15:53:42.593108
64357	PMBC	2020-10-02	3.61	3.6999	3.51	3.64	25659	{}	\N	2020-10-12 15:53:42.598709
64358	PMBC	2020-10-05	3.6	3.67	3.5	3.6	17489	{}	\N	2020-10-12 15:53:42.603551
64359	PMBC	2020-10-06	3.6	3.81	3.6	3.72	18529	{}	\N	2020-10-12 15:53:42.608247
64360	PMBC	2020-10-07	3.67	3.76	3.66	3.73	21974	{}	\N	2020-10-12 15:53:42.613349
64361	PMBC	2020-10-08	3.67	3.72	3.6	3.72	43312	{}	\N	2020-10-12 15:53:42.617702
64362	PMBC	2020-10-09	3.68	3.78	3.595	3.68	123234	{}	\N	2020-10-12 15:53:42.622219
64363	PMBC	2020-10-12	3.6984	3.6984	3.68	3.68	5633	{}	\N	2020-10-12 15:53:42.626451
64364	PNF	2020-09-28	11.6	11.7	11.6	11.68	9582	{}	\N	2020-10-12 15:53:44.294565
64365	PNF	2020-09-29	11.7	11.7	11.58	11.59	15346	{}	\N	2020-10-12 15:53:44.303296
64366	PNF	2020-09-30	11.57	11.57	11.46	11.5	34684	{}	\N	2020-10-12 15:53:44.30972
64367	PNF	2020-10-01	11.5	11.5	11.4	11.42	33464	{}	\N	2020-10-12 15:53:44.314988
64368	PNF	2020-10-02	11.42	11.425	11.35	11.39	18508	{}	\N	2020-10-12 15:53:44.320649
64369	PNF	2020-10-05	11.44	11.46	11.33	11.46	20126	{}	\N	2020-10-12 15:53:44.325636
64370	PNF	2020-10-06	11.44	11.45	11.33	11.43	35000	{}	\N	2020-10-12 15:53:44.330479
64371	PNF	2020-10-07	11.38	11.49	11.38	11.48	39025	{}	\N	2020-10-12 15:53:44.335513
64372	PNF	2020-10-08	11.48	11.54	11.46	11.4832	41936	{}	\N	2020-10-12 15:53:44.340929
64373	PNF	2020-10-09	11.5	11.5	11.42	11.42	4840	{}	\N	2020-10-12 15:53:44.345887
64374	PNF	2020-10-12	11.52	11.52	11.4	11.5004	20726	{}	\N	2020-10-12 15:53:44.350708
64375	POLA	2020-09-28	3.13	3.76	2.85	3.65	4916754	{}	\N	2020-10-12 15:53:45.3292
64376	POLA	2020-09-29	3.326	4	3.25	3.39	1720374	{}	\N	2020-10-12 15:53:45.340477
64377	POLA	2020-09-30	3.61	3.75	3.1301	3.1601	1039298	{}	\N	2020-10-12 15:53:45.347568
64378	POLA	2020-10-01	3.24	3.3387	3.03	3.13	575533	{}	\N	2020-10-12 15:53:45.353772
64379	POLA	2020-10-02	3.04	3.57	3.01	3.38	986319	{}	\N	2020-10-12 15:53:45.359617
64380	POLA	2020-10-05	3.44	4.11	3.28	3.81	4748222	{}	\N	2020-10-12 15:53:45.364703
64381	POLA	2020-10-06	4.05	4.79	3.83	4.05	13409648	{}	\N	2020-10-12 15:53:45.369617
64382	POLA	2020-10-07	4.3	4.79	4.12	4.25	4140643	{}	\N	2020-10-12 15:53:45.374471
64383	POLA	2020-10-08	4.29	4.3899	3.82	3.93	1280106	{}	\N	2020-10-12 15:53:45.379572
64384	POLA	2020-10-09	3.9	4.0997	3.71	3.72	1047511	{}	\N	2020-10-12 15:53:45.384326
64385	POLA	2020-10-12	3.68	3.78	3.37	3.4999	472391	{}	\N	2020-10-12 15:53:45.389037
64386	PPC	2020-09-28	14.97	15.255	14.93	15.06	1001753	{}	\N	2020-10-12 15:53:46.342205
64387	PPC	2020-09-29	15.07	15.12	14.74	14.79	570182	{}	\N	2020-10-12 15:53:46.35272
64388	PPC	2020-09-30	14.91	15.13	14.82	14.965	581143	{}	\N	2020-10-12 15:53:46.360708
64389	PPC	2020-10-01	15.09	15.17	14.945	15.155	669472	{}	\N	2020-10-12 15:53:46.367117
64390	PPC	2020-10-02	14.94	15.57	14.71	15.53	834603	{}	\N	2020-10-12 15:53:46.374803
64391	PPC	2020-10-05	15.69	15.84	15.31	15.43	803476	{}	\N	2020-10-12 15:53:46.379426
64392	PPC	2020-10-06	15.55	15.87	15.445	15.53	826954	{}	\N	2020-10-12 15:53:46.384169
64393	PPC	2020-10-07	15.74	16.01	15.56	15.96	987866	{}	\N	2020-10-12 15:53:46.388748
64394	PPC	2020-10-08	16.01	16.13	15.69	15.72	638406	{}	\N	2020-10-12 15:53:46.393865
64395	PPC	2020-10-09	15.89	15.95	15.37	15.38	668815	{}	\N	2020-10-12 15:53:46.39881
64396	PPC	2020-10-12	15.41	15.825	15.2552	15.81	335415	{}	\N	2020-10-12 15:53:46.403563
64397	PRA	2020-09-28	15.2	15.77	15.185	15.48	424858	{}	\N	2020-10-12 15:53:47.435314
64398	PRA	2020-09-29	15.4	15.4	14.76	15.36	622895	{}	\N	2020-10-12 15:53:47.447833
64399	PRA	2020-09-30	15.49	15.76	15.3	15.64	811074	{}	\N	2020-10-12 15:53:47.455759
64400	PRA	2020-10-01	15.76	15.85	15.18	15.68	608998	{}	\N	2020-10-12 15:53:47.462318
64401	PRA	2020-10-02	15.32	15.95	15.31	15.78	523263	{}	\N	2020-10-12 15:53:47.468496
64402	PRA	2020-10-05	15.92	15.99	15.2	15.22	451332	{}	\N	2020-10-12 15:53:47.473796
64403	PRA	2020-10-06	15.43	15.6	14.89	14.97	665610	{}	\N	2020-10-12 15:53:47.479186
64404	PRA	2020-10-07	15.03	15.3	14.92	15.22	544036	{}	\N	2020-10-12 15:53:47.484617
64405	PRA	2020-10-08	15.4	15.78	15.14	15.67	555464	{}	\N	2020-10-12 15:53:47.489895
64406	PRA	2020-10-09	15.74	16.25	15.535	16.16	607169	{}	\N	2020-10-12 15:53:47.495273
64407	PRA	2020-10-12	16.09	16.49	15.97	16.39	213453	{}	\N	2020-10-12 15:53:47.499989
64408	PKI	2020-09-28	120.61	122.45	120.61	121.89	579243	{}	\N	2020-10-12 15:53:49.46991
64409	PKI	2020-09-29	121.89	124.659	121.89	123.26	2458346	{}	\N	2020-10-12 15:53:49.481025
64410	PKI	2020-09-30	123.65	126.1	123.65	125.51	2512266	{}	\N	2020-10-12 15:53:49.48896
64411	PKI	2020-10-01	127.5	128.98	123.66	124.86	2259674	{}	\N	2020-10-12 15:53:49.495513
64412	PKI	2020-10-02	123.18	125.14	122.41	124.56	1207174	{}	\N	2020-10-12 15:53:49.50074
64413	PKI	2020-10-05	125.62	129.8	125.15	129.71	1770629	{}	\N	2020-10-12 15:53:49.505469
64414	PKI	2020-10-06	129.57	129.85	124.82	126.55	2184225	{}	\N	2020-10-12 15:53:49.509835
64415	PKI	2020-10-07	127.3	129.29	127.3	128.24	1398237	{}	\N	2020-10-12 15:53:49.514312
64416	PKI	2020-10-08	128.51	129.1494	126.68	127.61	1135668	{}	\N	2020-10-12 15:53:49.518742
64417	PKI	2020-10-09	128.86	129.8	128	129.68	927588	{}	\N	2020-10-12 15:53:49.523633
64418	PKI	2020-10-12	134.05	134.39	130.4	131.97	1053533	{}	\N	2020-10-12 15:53:49.528711
64419	PLD	2020-09-28	99.13	100.2	98.37	100.03	2285200	{}	\N	2020-10-12 15:53:50.697287
64420	PLD	2020-09-29	99.58	100.45	99.0008	99.61	3484941	{}	\N	2020-10-12 15:53:50.710549
64421	PLD	2020-09-30	100.21	101.54	98.76	100.62	7046245	{}	\N	2020-10-12 15:53:50.718927
64422	PLD	2020-10-01	100.93	102.37	99.56	102.16	6064463	{}	\N	2020-10-12 15:53:50.726382
64423	PLD	2020-10-02	101	104.74	100.5	104.22	9075908	{}	\N	2020-10-12 15:53:50.733811
64424	PLD	2020-10-05	105	105.72	103.43	105.19	5376554	{}	\N	2020-10-12 15:53:50.73951
64425	PLD	2020-10-06	105.28	105.67	103.55	104.48	5113568	{}	\N	2020-10-12 15:53:50.744472
64426	PLD	2020-10-07	105.09	106.38	103.92	105.76	4002794	{}	\N	2020-10-12 15:53:50.749482
64427	PLD	2020-10-08	105.83	109.11	105.45	106.84	4694482	{}	\N	2020-10-12 15:53:50.754606
64428	PLD	2020-10-09	107.35	107.95	106.28	106.91	3747848	{}	\N	2020-10-12 15:53:50.759782
64429	PLD	2020-10-12	107.8	108.5	106.14	108.44	663683	{}	\N	2020-10-12 15:53:50.764945
64430	PLYM	2020-09-28	12.47	12.58	12.27	12.54	122077	{}	\N	2020-10-12 15:53:52.10137
64431	PLYM	2020-09-29	12.27	12.39	11.89	12.32	349102	{}	\N	2020-10-12 15:53:52.112915
64432	PLYM	2020-09-30	12.34	12.55	11.95	12.34	282026	{}	\N	2020-10-12 15:53:52.121553
64433	PLYM	2020-10-01	12.37	12.57	12.27	12.56	279530	{}	\N	2020-10-12 15:53:52.129343
64434	PLYM	2020-10-02	12.26	12.755	12.256	12.63	218435	{}	\N	2020-10-12 15:53:52.134944
64435	PLYM	2020-10-05	12.7	12.72	12.29	12.66	253570	{}	\N	2020-10-12 15:53:52.140864
64436	PLYM	2020-10-06	12.77	12.85	12.22	12.43	350883	{}	\N	2020-10-12 15:53:52.146266
64437	PLYM	2020-10-07	12.5	12.55	12.17	12.23	308751	{}	\N	2020-10-12 15:53:52.151378
64438	PLYM	2020-10-08	12.25	12.58	12.25	12.52	185036	{}	\N	2020-10-12 15:53:52.156143
64439	PLYM	2020-10-09	12.64	12.69	12.4608	12.57	167605	{}	\N	2020-10-12 15:53:52.161162
64440	PLYM	2020-10-12	12.52	12.55	12.285	12.39	52696	{}	\N	2020-10-12 15:53:52.16741
64441	PMX	2020-09-28	11.24	11.34	11.23	11.31	44406	{}	\N	2020-10-12 15:53:53.121948
64442	PMX	2020-09-29	11.28	11.36	11.23	11.27	122134	{}	\N	2020-10-12 15:53:53.129552
64443	PMX	2020-09-30	11.27	11.41	11.26	11.36	113239	{}	\N	2020-10-12 15:53:53.136399
64444	PMX	2020-10-01	11.41	11.44	11.3728	11.41	112479	{}	\N	2020-10-12 15:53:53.142518
64445	PMX	2020-10-02	11.38	11.69	11.3295	11.65	202160	{}	\N	2020-10-12 15:53:53.147774
64446	PMX	2020-10-05	11.66	11.67	11.56	11.57	91114	{}	\N	2020-10-12 15:53:53.152442
64447	PMX	2020-10-06	11.62	11.64	11.49	11.54	76286	{}	\N	2020-10-12 15:53:53.157578
64448	PMX	2020-10-07	11.64	11.64	11.45	11.49	91942	{}	\N	2020-10-12 15:53:53.162338
64449	PMX	2020-10-08	11.45	11.5672	11.45	11.48	135291	{}	\N	2020-10-12 15:53:53.167167
64450	PMX	2020-10-09	11.45	11.475	11.33	11.38	154210	{}	\N	2020-10-12 15:53:53.171604
64451	PMX	2020-10-12	11.38	11.43	11.3481	11.41	45812	{}	\N	2020-10-12 15:53:53.175758
64452	POST	2020-09-28	84.56	85.445	84.35	84.66	417546	{}	\N	2020-10-12 15:53:54.607822
64453	POST	2020-09-29	84.84	84.9	83.25	83.68	729686	{}	\N	2020-10-12 15:53:54.616732
64454	POST	2020-09-30	83.83	86.565	83.83	86	1506574	{}	\N	2020-10-12 15:53:54.623403
64455	POST	2020-10-01	86.3	88.22	86.205	87.2	1395994	{}	\N	2020-10-12 15:53:54.628944
64456	POST	2020-10-02	86.62	89.2	86	88.7	651335	{}	\N	2020-10-12 15:53:54.634826
64457	POST	2020-10-05	89.24	90.31	89.135	89.49	856901	{}	\N	2020-10-12 15:53:54.639575
64458	POST	2020-10-06	89.85	90.26	87.82	88.68	971586	{}	\N	2020-10-12 15:53:54.643991
64459	POST	2020-10-07	88.84	89.915	88.11	89.63	729851	{}	\N	2020-10-12 15:53:54.648398
64460	POST	2020-10-08	90.07	91.9	89.74	91.57	812826	{}	\N	2020-10-12 15:53:54.65285
64461	POST	2020-10-09	92.18	93.78	91.46	92.64	1090511	{}	\N	2020-10-12 15:53:54.657179
64462	POST	2020-10-12	92.8	93.25	92.1	93.08	124523	{}	\N	2020-10-12 15:53:54.662014
64463	PPL	2020-09-28	26.71	26.97	26.57	26.68	3716018	{}	\N	2020-10-12 15:53:55.984828
64464	PPL	2020-09-29	26.85	27.386	26.74	26.88	10495703	{}	\N	2020-10-12 15:53:55.992417
64465	PPL	2020-09-30	27.1	27.45	27.01	27.21	12653446	{}	\N	2020-10-12 15:53:55.997587
64466	PPL	2020-10-01	27.43	27.66	27.24	27.55	7856478	{}	\N	2020-10-12 15:53:56.003388
64467	PPL	2020-10-02	27.2	28.42	27.12	28.21	10099418	{}	\N	2020-10-12 15:53:56.009104
64468	PPL	2020-10-05	28.37	28.595	28.17	28.44	6694064	{}	\N	2020-10-12 15:53:56.015337
64469	PPL	2020-10-06	28.54	29.14	28.33	28.7	7643706	{}	\N	2020-10-12 15:53:56.021814
64470	PPL	2020-10-07	28.9	29.01	28.59	28.76	5405056	{}	\N	2020-10-12 15:53:56.027711
64471	PPL	2020-10-08	28.87	29.18	28.76	29.11	6268489	{}	\N	2020-10-12 15:53:56.032638
64472	PPL	2020-10-09	29.25	29.34	28.75	28.94	7011805	{}	\N	2020-10-12 15:53:56.037552
64473	PPL	2020-10-12	28.93	29.155	28.9	29.145	2267985	{}	\N	2020-10-12 15:53:56.04245
64474	PRCP	2020-09-28	6.84	6.85	6.79	6.82	3843828	{}	\N	2020-10-12 15:53:57.779329
64475	PRCP	2020-09-29	6.8	6.82	6.78	6.8	326882	{}	\N	2020-10-12 15:53:57.791883
64476	PRCP	2020-09-30	6.79	6.81	6.78	6.8	247597	{}	\N	2020-10-12 15:53:57.799566
64477	PRCP	2020-10-01	6.79	6.82	6.78	6.82	271862	{}	\N	2020-10-12 15:53:57.80598
64478	PRCP	2020-10-02	6.75	6.84	6.7	6.83	215903	{}	\N	2020-10-12 15:53:57.812455
64479	PRCP	2020-10-05	6.83	6.96	6.83	6.89	106192	{}	\N	2020-10-12 15:53:57.81718
64480	PRCP	2020-10-06	6.87	6.94	6.835	6.87	79341	{}	\N	2020-10-12 15:53:57.822027
64481	PRCP	2020-10-07	6.88	6.94	6.87	6.91	249733	{}	\N	2020-10-12 15:53:57.826941
64482	PRCP	2020-10-08	6.93	6.97	6.82	6.87	264623	{}	\N	2020-10-12 15:53:57.831506
64483	PRCP	2020-10-09	6.87	6.98	6.87	6.96	115696	{}	\N	2020-10-12 15:53:57.836398
64484	PRCP	2020-10-12	6.97	7	6.87	6.8897	45723	{}	\N	2020-10-12 15:53:57.841321
64485	PRPH	2020-09-28	3.94	4.1	3.61	3.81	94711	{}	\N	2020-10-12 15:53:59.684306
64486	PRPH	2020-09-29	4	4	3.6301	3.65	34786	{}	\N	2020-10-12 15:53:59.695913
64487	PRPH	2020-09-30	3.62	3.7332	3.46	3.46	22553	{}	\N	2020-10-12 15:53:59.704423
64488	PRPH	2020-10-01	3.802	4.0799	3.7	3.9	118293	{}	\N	2020-10-12 15:53:59.711952
64489	PRPH	2020-10-02	3.78	5.2	3.78	5.15	689379	{}	\N	2020-10-12 15:53:59.717831
64490	PRPH	2020-10-05	4.87	7.1	4.87	6.89	850724	{}	\N	2020-10-12 15:53:59.723112
64491	PRPH	2020-10-06	7.2535	7.49	5.53	5.93	631719	{}	\N	2020-10-12 15:53:59.728175
64492	PRPH	2020-10-07	6.41	6.41	6.11	6.24	109250	{}	\N	2020-10-12 15:53:59.733989
64493	PRPH	2020-10-08	6.74	6.74	5.85	6.27	78939	{}	\N	2020-10-12 15:53:59.73917
64494	PRPH	2020-10-09	6.17	6.7499	5.81	5.88	63928	{}	\N	2020-10-12 15:53:59.744195
64495	PRPH	2020-10-12	6.04	6.04	5.46	5.46	42760	{}	\N	2020-10-12 15:53:59.749617
64496	PRU	2020-09-28	63.5	64.87	63.29	64.03	2292320	{}	\N	2020-10-12 15:54:00.998069
64497	PRU	2020-09-29	63.76	64.25	62.375	63.09	4026763	{}	\N	2020-10-12 15:54:01.011489
64498	PRU	2020-09-30	63.2	64.1399	62.76	63.52	5792312	{}	\N	2020-10-12 15:54:01.02114
64499	PRU	2020-10-01	63.91	64.755	62.81	63.51	4581869	{}	\N	2020-10-12 15:54:01.026823
64500	PRU	2020-10-02	62.36	64.3	62.14	63.88	4120175	{}	\N	2020-10-12 15:54:01.03198
64501	PRU	2020-10-05	64.65	66.21	64.51	65.71	4800304	{}	\N	2020-10-12 15:54:01.03644
64502	PRU	2020-10-06	66.57	68.05	65.11	65.34	5970920	{}	\N	2020-10-12 15:54:01.041031
64503	PRU	2020-10-07	66.07	67.995	66.07	67.79	4403666	{}	\N	2020-10-12 15:54:01.045579
64504	PRU	2020-10-08	68.45	69.02	67.79	68.27	3051748	{}	\N	2020-10-12 15:54:01.050456
64505	PRU	2020-10-09	68.93	68.9852	67.07	67.88	2699815	{}	\N	2020-10-12 15:54:01.054749
64506	PRU	2020-10-12	68.38	69.03	67.8661	68.935	848053	{}	\N	2020-10-12 15:54:01.059462
64507	PIN	2020-09-28	19.44	19.5699	19.41	19.42	11604	{}	\N	2020-10-12 15:54:02.008856
64508	PIN	2020-09-29	19.45	19.46	19.38	19.43	6897	{}	\N	2020-10-12 15:54:02.019849
64509	PIN	2020-09-30	19.525	19.6599	19.51	19.53	6832	{}	\N	2020-10-12 15:54:02.026579
64510	PIN	2020-10-01	19.72	19.83	19.701	19.79	16178	{}	\N	2020-10-12 15:54:02.032358
64511	PIN	2020-10-02	19.58	19.7161	19.58	19.62	9141	{}	\N	2020-10-12 15:54:02.038079
64512	PIN	2020-10-05	19.92	20	19.92	19.9733	12714	{}	\N	2020-10-12 15:54:02.042886
64513	PIN	2020-10-06	20.16	20.17	20.03	20.05	8430	{}	\N	2020-10-12 15:54:02.047107
64514	PIN	2020-10-07	20.27	20.28	20.185	20.2175	12289	{}	\N	2020-10-12 15:54:02.051649
64515	PIN	2020-10-08	20.3	20.354	20.22	20.31	10346	{}	\N	2020-10-12 15:54:02.056039
64516	PIN	2020-10-09	20.4	20.5	20.32	20.45	11911	{}	\N	2020-10-12 15:54:02.06053
64517	PIN	2020-10-12	20.34	20.37	20.25	20.36	36685	{}	\N	2020-10-12 15:54:02.064796
64518	PJP	2020-09-28	63.24	63.4627	62.98	63.25	11006	{}	\N	2020-10-12 15:54:03.204396
64519	PJP	2020-09-29	63.08	63.48	62.875	63.07	10446	{}	\N	2020-10-12 15:54:03.21341
64520	PJP	2020-09-30	63.02	64.2388	63.02	63.41	9376	{}	\N	2020-10-12 15:54:03.220723
64521	PJP	2020-10-01	64.14	64.3	63.7	63.9465	10690	{}	\N	2020-10-12 15:54:03.228092
64522	PJP	2020-10-02	62.91	63.8	62.91	63.2	11605	{}	\N	2020-10-12 15:54:03.234474
64523	PJP	2020-10-05	63.72	64.965	63.72	64.93	8421	{}	\N	2020-10-12 15:54:03.238952
64524	PJP	2020-10-06	65.02	65.6027	64.47	64.47	8171	{}	\N	2020-10-12 15:54:03.243727
64525	PJP	2020-10-07	65.1006	65.5625	65.09	65.44	7881	{}	\N	2020-10-12 15:54:03.247864
64526	PJP	2020-10-08	65.9	66.74	65.9	66.56	21266	{}	\N	2020-10-12 15:54:03.251946
64527	PJP	2020-10-09	66.72	67.155	66.64	66.76	27249	{}	\N	2020-10-12 15:54:03.256908
64528	PJP	2020-10-12	66.88	66.9858	66.3965	66.775	4321	{}	\N	2020-10-12 15:54:03.261213
64529	PKG	2020-09-28	110.34	111.68	108.61	111.15	927490	{}	\N	2020-10-12 15:54:04.285612
64530	PKG	2020-09-29	111.39	111.455	108.35	108.75	1214643	{}	\N	2020-10-12 15:54:04.297017
64531	PKG	2020-09-30	109.36	110.56	107.96	109.05	1296758	{}	\N	2020-10-12 15:54:04.305187
64532	PKG	2020-10-01	109.76	110.56	106.94	107.65	1049434	{}	\N	2020-10-12 15:54:04.312578
64533	PKG	2020-10-02	106.34	109.65	106.0755	108.42	1202391	{}	\N	2020-10-12 15:54:04.318579
64534	PKG	2020-10-05	110.17	111.78	109.61	110.94	1564109	{}	\N	2020-10-12 15:54:04.324029
64535	PKG	2020-10-06	110.55	111.84	109.09	109.41	922563	{}	\N	2020-10-12 15:54:04.329731
64536	PKG	2020-10-07	110.54	112.89	110.54	111.97	731124	{}	\N	2020-10-12 15:54:04.337082
64537	PKG	2020-10-08	112.23	113.84	111.94	113.81	636677	{}	\N	2020-10-12 15:54:04.34259
64538	PKG	2020-10-09	114.56	115.51	112.74	112.9	961321	{}	\N	2020-10-12 15:54:04.34734
64539	PKG	2020-10-12	115.75	118.44	114.5	118.44	578683	{}	\N	2020-10-12 15:54:04.351767
64540	PLG	2020-09-28	2.19	2.28	2.04	2.09	3373286	{}	\N	2020-10-12 15:54:05.333218
64541	PLG	2020-09-29	2.04	2.13	1.98	2.06	666157	{}	\N	2020-10-12 15:54:05.340747
64542	PLG	2020-09-30	2.1	2.1	2	2.0001	622469	{}	\N	2020-10-12 15:54:05.347402
64543	PLG	2020-10-01	2.07	2.1161	2.01	2.06	372204	{}	\N	2020-10-12 15:54:05.352076
64544	PLG	2020-10-02	2.04	2.12	1.96	2.02	816202	{}	\N	2020-10-12 15:54:05.356843
64545	PLG	2020-10-05	2.02	2.2	2.02	2.11	3328701	{}	\N	2020-10-12 15:54:05.362378
64546	PLG	2020-10-06	2.26	2.35	2.1	2.13	3937626	{}	\N	2020-10-12 15:54:05.367382
64547	PLG	2020-10-07	2.23	2.34	2.16	2.25	1664808	{}	\N	2020-10-12 15:54:05.37234
64548	PLG	2020-10-08	2.31	2.5	2.17	2.2	2075363	{}	\N	2020-10-12 15:54:05.377911
64549	PLG	2020-10-09	2.24	2.3298	2.2	2.24	801707	{}	\N	2020-10-12 15:54:05.383401
64550	PLG	2020-10-12	2.2	2.2	2.11	2.1327	322074	{}	\N	2020-10-12 15:54:05.388689
64551	PLXP	2020-09-28	3.32	3.53	3.27	3.35	6510	{}	\N	2020-10-12 15:54:08.146609
64552	PLXP	2020-09-29	3.39	3.4438	3.15	3.16	3014	{}	\N	2020-10-12 15:54:08.158602
64553	PLXP	2020-09-30	3.4599	3.4599	3.23	3.23	15039	{}	\N	2020-10-12 15:54:08.167657
64554	PLXP	2020-10-01	3.29	3.4	3.15	3.29	34254	{}	\N	2020-10-12 15:54:08.173983
64555	PLXP	2020-10-02	3.18	3.34	3.18	3.25	3110	{}	\N	2020-10-12 15:54:08.179767
64556	PLXP	2020-10-05	3.28	3.28	3.22	3.22	7461	{}	\N	2020-10-12 15:54:08.185176
64557	PLXP	2020-10-06	3.21	3.3645	3.2	3.2	6866	{}	\N	2020-10-12 15:54:08.189976
64558	PLXP	2020-10-07	3.3047	3.32	3.175	3.175	7882	{}	\N	2020-10-12 15:54:08.194893
64559	PLXP	2020-10-08	3.26	3.47	3.13	3.19	15375	{}	\N	2020-10-12 15:54:08.199937
64560	PLXP	2020-10-09	3.19	3.35	3.19	3.29	16535	{}	\N	2020-10-12 15:54:08.204646
64561	PLXP	2020-10-12	3.23	3.295	3.23	3.25	2322	{}	\N	2020-10-12 15:54:08.209549
64562	PNNT	2020-09-28	3.19	3.35	3.19	3.25	293039	{}	\N	2020-10-12 15:54:09.64919
64563	PNNT	2020-09-29	3.25	3.3262	3.21	3.22	188124	{}	\N	2020-10-12 15:54:09.658056
64564	PNNT	2020-09-30	3.22	3.28	3.19	3.19	268118	{}	\N	2020-10-12 15:54:09.664596
64565	PNNT	2020-10-01	3.24	3.37	3.22	3.36	308688	{}	\N	2020-10-12 15:54:09.671157
64566	PNNT	2020-10-02	3.26	3.37	3.26	3.34	160501	{}	\N	2020-10-12 15:54:09.676716
64567	PNNT	2020-10-05	3.37	3.4	3.28	3.34	161977	{}	\N	2020-10-12 15:54:09.681966
64568	PNNT	2020-10-06	3.36	3.42	3.31	3.34	174279	{}	\N	2020-10-12 15:54:09.687478
64569	PNNT	2020-10-07	3.36	3.39	3.31	3.37	224712	{}	\N	2020-10-12 15:54:09.692447
64570	PNNT	2020-10-08	3.38	3.418	3.35	3.37	103510	{}	\N	2020-10-12 15:54:09.697623
64571	PNNT	2020-10-09	3.37	3.38	3.31	3.32	199192	{}	\N	2020-10-12 15:54:09.702702
64572	PNNT	2020-10-12	3.3	3.34	3.29	3.304	52096	{}	\N	2020-10-12 15:54:09.707781
64573	PPH	2020-09-28	62.09	62.15	61.815	61.87	9025	{}	\N	2020-10-12 15:54:12.321731
64574	PPH	2020-09-29	61.94	62.05	61.52	61.56	42411	{}	\N	2020-10-12 15:54:12.335818
64575	PPH	2020-09-30	61.56	62.439	61.56	62.18	32917	{}	\N	2020-10-12 15:54:12.343824
64576	PPH	2020-10-01	62.21	62.21	61.29	61.44	29865	{}	\N	2020-10-12 15:54:12.350139
64577	PPH	2020-10-02	60.63	61.58	60.63	61.21	22752	{}	\N	2020-10-12 15:54:12.355784
64578	PPH	2020-10-05	61.78	62.22	61.53	62.19	48798	{}	\N	2020-10-12 15:54:12.360594
64579	PPH	2020-10-06	62.19	62.19	61.07	61.17	37424	{}	\N	2020-10-12 15:54:12.365108
64580	PPH	2020-10-07	61.64	62.2546	61.4571	62.1	16278	{}	\N	2020-10-12 15:54:12.370089
64581	PPH	2020-10-08	62.63	63.6924	62.63	63.6	22588	{}	\N	2020-10-12 15:54:12.374619
64582	PPH	2020-10-09	64.01	64.23	63.7501	63.93	16162	{}	\N	2020-10-12 15:54:12.3797
64583	PPH	2020-10-12	64.31	64.31	63.81	64.1	4581	{}	\N	2020-10-12 15:54:12.384581
64584	PPX	2020-09-28	25.6	25.63	25.53	25.53	19693	{}	\N	2020-10-12 15:54:13.452814
64585	PPX	2020-09-29	25.6	25.6	25.45	25.51	53976	{}	\N	2020-10-12 15:54:13.461731
64586	PPX	2020-09-30	25.52	25.57	25.37	25.37	179020	{}	\N	2020-10-12 15:54:13.467949
64587	PPX	2020-10-01	25.46	25.6235	25.44	25.6	61340	{}	\N	2020-10-12 15:54:13.473776
64588	PPX	2020-10-02	25.5604	25.6	25.5462	25.57	42487	{}	\N	2020-10-12 15:54:13.479157
64589	PPX	2020-10-05	25.58	25.62	25.5	25.59	83952	{}	\N	2020-10-12 15:54:13.483581
64590	PPX	2020-10-06	25.6	25.605	25.53	25.59	48600	{}	\N	2020-10-12 15:54:13.487603
64591	PPX	2020-10-07	25.61	25.6282	25.59	25.6	46284	{}	\N	2020-10-12 15:54:13.491548
64592	PPX	2020-10-08	25.65	25.65	25.6	25.6	38352	{}	\N	2020-10-12 15:54:13.496111
64593	PPX	2020-10-09	25.6	25.65	25.5868	25.6	18806	{}	\N	2020-10-12 15:54:13.500971
64594	PPX	2020-10-12	25.66	25.66	25.58	25.6173	4504	{}	\N	2020-10-12 15:54:13.504735
64595	PRIM	2020-09-28	18.45	18.83	18.45	18.74	182960	{}	\N	2020-10-12 15:54:14.749561
64596	PRIM	2020-09-29	18.5	18.7	18.15	18.31	233686	{}	\N	2020-10-12 15:54:14.760425
64597	PRIM	2020-09-30	18.44	18.49	17.95	18.04	393915	{}	\N	2020-10-12 15:54:14.766992
64598	PRIM	2020-10-01	18.05	18.205	17.54	17.66	240324	{}	\N	2020-10-12 15:54:14.773954
64599	PRIM	2020-10-02	17.27	18.21	17.27	18.09	321687	{}	\N	2020-10-12 15:54:14.779
64600	PRIM	2020-10-05	18.35	18.96	18.27	18.82	171068	{}	\N	2020-10-12 15:54:14.783681
64601	PRIM	2020-10-06	18.97	19.44	18.674	18.88	235885	{}	\N	2020-10-12 15:54:14.788527
64602	PRIM	2020-10-07	19.08	19.68	18.875	19.53	369660	{}	\N	2020-10-12 15:54:14.792829
64603	PRIM	2020-10-08	19.75	19.86	19.57	19.78	147735	{}	\N	2020-10-12 15:54:14.797408
64604	PRIM	2020-10-09	19.92	20.24	19.62	19.8	211414	{}	\N	2020-10-12 15:54:14.801659
64605	PRIM	2020-10-12	19.89	20.17	19.735	20.01	76867	{}	\N	2020-10-12 15:54:14.80606
64606	PRPO	2020-09-28	2.34	2.53	2.31	2.37	1155084	{}	\N	2020-10-12 15:54:15.769156
64607	PRPO	2020-09-29	2.55	2.5767	2.39	2.48	882203	{}	\N	2020-10-12 15:54:15.782908
64608	PRPO	2020-09-30	2.48	2.51	2.36	2.45	509670	{}	\N	2020-10-12 15:54:15.790777
64609	PRPO	2020-10-01	2.42	2.45	2.34	2.45	230307	{}	\N	2020-10-12 15:54:15.797163
64610	PRPO	2020-10-02	2.44	2.73	2.3501	2.64	1184657	{}	\N	2020-10-12 15:54:15.803083
64611	PRPO	2020-10-05	2.65	2.65	2.44	2.54	556016	{}	\N	2020-10-12 15:54:15.808721
64612	PRPO	2020-10-06	2.72	3	2.3	2.37	2557334	{}	\N	2020-10-12 15:54:15.813767
64613	PRPO	2020-10-07	2.31	2.43	2.31	2.4	408725	{}	\N	2020-10-12 15:54:15.818531
64614	PRPO	2020-10-08	2.39	2.437	2.33	2.33	270477	{}	\N	2020-10-12 15:54:15.823726
64615	PRPO	2020-10-09	2.35	2.42	2.3	2.3	254186	{}	\N	2020-10-12 15:54:15.828994
64616	PRPO	2020-10-12	2.31	2.38	2.25	2.3	129773	{}	\N	2020-10-12 15:54:15.834183
64617	QADA	2020-09-28	41.72	42.81	41.72	42.64	36990	{}	\N	2020-10-12 15:54:16.928917
64618	QADA	2020-09-29	42.54	42.555	41.7	42.1	64688	{}	\N	2020-10-12 15:54:16.940822
64619	QADA	2020-09-30	42.31	43.1	42.17	42.2	60369	{}	\N	2020-10-12 15:54:16.948872
64620	QADA	2020-10-01	42.4	43.19	42.04	42.95	34306	{}	\N	2020-10-12 15:54:16.955347
64621	QADA	2020-10-02	42.19	42.99	41.99	42.12	21323	{}	\N	2020-10-12 15:54:16.961032
64622	QADA	2020-10-05	42.65	42.845	42.25	42.44	20137	{}	\N	2020-10-12 15:54:16.965992
64623	QADA	2020-10-06	42.8	43.83	42.73	43.095	34121	{}	\N	2020-10-12 15:54:16.970815
64624	QADA	2020-10-07	43.95	43.95	43.14	43.57	23229	{}	\N	2020-10-12 15:54:16.975324
64625	QADA	2020-10-08	43.99	44.58	43.23	44.46	29758	{}	\N	2020-10-12 15:54:16.979568
64626	QADA	2020-10-09	44.94	45.19	43.96	44.98	18914	{}	\N	2020-10-12 15:54:16.984376
64627	QADA	2020-10-12	45.25	45.25	44.65	45.03	5636	{}	\N	2020-10-12 15:54:16.988983
64628	PKE	2020-09-28	11.39	11.57	11.33	11.34	109301	{}	\N	2020-10-12 15:54:18.579965
64629	PKE	2020-09-29	11.25	11.37	11.13	11.18	147648	{}	\N	2020-10-12 15:54:18.591557
64630	PKE	2020-09-30	11.18	11.24	10.865	10.92	267025	{}	\N	2020-10-12 15:54:18.599407
64631	PKE	2020-10-01	10.87	11.07	10.85	11.04	116724	{}	\N	2020-10-12 15:54:18.605905
64632	PKE	2020-10-02	10.87	11.13	10.855	10.99	130875	{}	\N	2020-10-12 15:54:18.611566
64633	PKE	2020-10-05	11.06	11.41	11.06	11.35	93234	{}	\N	2020-10-12 15:54:18.616483
64634	PKE	2020-10-06	11.48	11.53	11.22	11.26	216122	{}	\N	2020-10-12 15:54:18.621095
64635	PKE	2020-10-07	11.32	11.38	11.06	11.1	163197	{}	\N	2020-10-12 15:54:18.626251
64636	PKE	2020-10-08	11.27	11.6	10.91	11.52	138448	{}	\N	2020-10-12 15:54:18.631716
64637	PKE	2020-10-09	11.6	11.675	11.3	11.37	227637	{}	\N	2020-10-12 15:54:18.636176
64638	PKE	2020-10-12	11.37	11.49	11.32	11.39	55994	{}	\N	2020-10-12 15:54:18.641036
64639	PLCE	2020-09-28	28.15	29.43	27.66	28.78	1251490	{}	\N	2020-10-12 15:54:20.254241
64640	PLCE	2020-09-29	28.58	28.6916	26.8275	27.36	964168	{}	\N	2020-10-12 15:54:20.267616
64641	PLCE	2020-09-30	27.72	28.83	27.385	28.35	833082	{}	\N	2020-10-12 15:54:20.276502
64642	PLCE	2020-10-01	28.53	30.55	28.53	29.7	1294353	{}	\N	2020-10-12 15:54:20.283245
64643	PLCE	2020-10-02	28.34	30.36	27.94	30.11	1071845	{}	\N	2020-10-12 15:54:20.289144
64644	PLCE	2020-10-05	30.51	31.24	29.365	31.02	954225	{}	\N	2020-10-12 15:54:20.294426
64645	PLCE	2020-10-06	30.49	30.745	28.96	29.04	1361939	{}	\N	2020-10-12 15:54:20.29956
64646	PLCE	2020-10-07	30.24	31.04	29.96	30.81	1240128	{}	\N	2020-10-12 15:54:20.304699
64647	PLCE	2020-10-08	31.49	32.55	30.45	30.49	852875	{}	\N	2020-10-12 15:54:20.309969
64648	PLCE	2020-10-09	30.77	30.92	28.14	28.42	1365687	{}	\N	2020-10-12 15:54:20.315956
64649	PLCE	2020-10-12	28.39	28.945	27.71	27.91	340212	{}	\N	2020-10-12 15:54:20.320827
64650	PLUG	2020-09-28	12.53	13.38	12.45	13.2	35175141	{}	\N	2020-10-12 15:54:21.631517
64651	PLUG	2020-09-29	13.2	13.59	12.88	13.16	21855684	{}	\N	2020-10-12 15:54:21.642666
64652	PLUG	2020-09-30	13.26	13.7	12.87	13.41	23730935	{}	\N	2020-10-12 15:54:21.649699
64653	PLUG	2020-10-01	13.77	14.03	13.51	13.995	25787308	{}	\N	2020-10-12 15:54:21.656175
64654	PLUG	2020-10-02	13.38	14.29	13.27	14.14	23720913	{}	\N	2020-10-12 15:54:21.661862
64655	PLUG	2020-10-05	14.66	15.96	14.66	15.74	42823022	{}	\N	2020-10-12 15:54:21.666769
64656	PLUG	2020-10-06	16.03	16.5	15.31	16.23	44033057	{}	\N	2020-10-12 15:54:21.671965
64657	PLUG	2020-10-07	16.53	18.26	16.515	18.225	52950880	{}	\N	2020-10-12 15:54:21.677221
64658	PLUG	2020-10-08	19.02	19.02	17.43	17.88	55418542	{}	\N	2020-10-12 15:54:21.68214
64659	PLUG	2020-10-09	17.8	18.89	17.7	18.43	34704344	{}	\N	2020-10-12 15:54:21.687402
64660	PLUG	2020-10-12	18.88	18.89	17.29	17.45	26190286	{}	\N	2020-10-12 15:54:21.692841
64661	PME	2020-09-28	1.43	1.64	1.43	1.63	189917	{}	\N	2020-10-12 15:54:23.272174
64662	PME	2020-09-29	1.61	1.665	1.5	1.632	220879	{}	\N	2020-10-12 15:54:23.285225
64663	PME	2020-09-30	1.62	1.62	1.55	1.57	42194	{}	\N	2020-10-12 15:54:23.29466
64664	PME	2020-10-01	1.55	1.619	1.51	1.52	43736	{}	\N	2020-10-12 15:54:23.301676
64665	PME	2020-10-02	1.45	1.58	1.4	1.52	100437	{}	\N	2020-10-12 15:54:23.307188
64666	PME	2020-10-05	1.52	1.61	1.52	1.56	72249	{}	\N	2020-10-12 15:54:23.312722
64667	PME	2020-10-06	1.55	1.63	1.5301	1.58	135158	{}	\N	2020-10-12 15:54:23.31828
64668	PME	2020-10-07	1.57	1.61	1.45	1.45	131614	{}	\N	2020-10-12 15:54:23.323466
64669	PME	2020-10-08	1.495	1.81	1.43	1.75	1113518	{}	\N	2020-10-12 15:54:23.328851
64670	PME	2020-10-09	1.75	1.85	1.7	1.7	130742	{}	\N	2020-10-12 15:54:23.333763
64671	PME	2020-10-12	1.7	1.85	1.7	1.8	18024	{}	\N	2020-10-12 15:54:23.338694
64672	PMT	2020-09-28	16.64	17.14	16.64	16.86	1488807	{}	\N	2020-10-12 15:54:24.261698
64673	PMT	2020-09-29	16.84	16.89	16.08	16.15	1095472	{}	\N	2020-10-12 15:54:24.273211
64674	PMT	2020-09-30	16.19	16.63	16.02	16.07	1652356	{}	\N	2020-10-12 15:54:24.280924
64675	PMT	2020-10-01	16.1	16.31	15.97	16.28	1296104	{}	\N	2020-10-12 15:54:24.286836
64676	PMT	2020-10-02	16	16.81	15.92	16.76	1642346	{}	\N	2020-10-12 15:54:24.292157
64677	PMT	2020-10-05	16.91	17.14	16.84	17.1	970559	{}	\N	2020-10-12 15:54:24.297007
64678	PMT	2020-10-06	17.28	17.55	17.09	17.11	1493076	{}	\N	2020-10-12 15:54:24.301247
64679	PMT	2020-10-07	17.14	17.39	16.7	16.86	1910658	{}	\N	2020-10-12 15:54:24.305797
64680	PMT	2020-10-08	17.05	17.45	16.9201	17.4	1143547	{}	\N	2020-10-12 15:54:24.31034
64681	PMT	2020-10-09	17.4	17.5689	17.17	17.33	1512851	{}	\N	2020-10-12 15:54:24.315462
64682	PMT	2020-10-12	17.38	17.46	17.22	17.31	474832	{}	\N	2020-10-12 15:54:24.32059
64683	PNI	2020-09-28	10.8	10.82	10.74	10.74	3924	{}	\N	2020-10-12 15:54:25.372585
64684	PNI	2020-09-29	10.8	10.8	10.62	10.62	47458	{}	\N	2020-10-12 15:54:25.383677
64685	PNI	2020-09-30	10.63	10.76	10.59	10.73	21175	{}	\N	2020-10-12 15:54:25.391207
64686	PNI	2020-10-01	10.8	10.8	10.6	10.66	31984	{}	\N	2020-10-12 15:54:25.398932
64687	PNI	2020-10-02	10.62	10.67	10.57	10.625	31190	{}	\N	2020-10-12 15:54:25.404767
64688	PNI	2020-10-05	10.65	10.67	10.52	10.53	36960	{}	\N	2020-10-12 15:54:25.410601
64689	PNI	2020-10-06	10.64	10.66	10.07	10.39	58884	{}	\N	2020-10-12 15:54:25.416868
64690	PNI	2020-10-07	10.37	10.55	10.37	10.4336	22366	{}	\N	2020-10-12 15:54:25.422917
64691	PNI	2020-10-08	10.48	10.48	10.42	10.46	22082	{}	\N	2020-10-12 15:54:25.429885
64692	PNI	2020-10-09	10.41	10.45	10.41	10.4401	13544	{}	\N	2020-10-12 15:54:25.435877
64693	PNI	2020-10-12	10.55	10.55	10.4	10.48	21712	{}	\N	2020-10-12 15:54:25.442236
64694	PODD	2020-09-28	227.92	228.19	220.98	227.38	326187	{}	\N	2020-10-12 15:54:26.53415
64695	PODD	2020-09-29	229.44	237	228.69	231.97	442541	{}	\N	2020-10-12 15:54:26.545906
64696	PODD	2020-09-30	232.23	242.125	231.49	236.59	468332	{}	\N	2020-10-12 15:54:26.554094
64697	PODD	2020-10-01	239.98	240.14	234.98	237.56	245997	{}	\N	2020-10-12 15:54:26.562194
64698	PODD	2020-10-02	233.47	238.06	228.38	231.76	203768	{}	\N	2020-10-12 15:54:26.568031
64699	PODD	2020-10-05	233.9	236.71	230.85	235.35	298551	{}	\N	2020-10-12 15:54:26.57354
64700	PODD	2020-10-06	237.61	238.09	228.64	230.74	407776	{}	\N	2020-10-12 15:54:26.578714
64701	PODD	2020-10-07	234.15	244.79	234.15	244.35	410447	{}	\N	2020-10-12 15:54:26.584234
64702	PODD	2020-10-08	245	247.89	244.25	247.71	265771	{}	\N	2020-10-12 15:54:26.589819
64703	PODD	2020-10-09	249.53	252.74	244.43	248.59	278384	{}	\N	2020-10-12 15:54:26.595021
64704	PODD	2020-10-12	252.72	252.72	246.5	247	53916	{}	\N	2020-10-12 15:54:26.600529
64705	PPA	2020-09-28	56.41	57.2154	56.41	56.6414	32942	{}	\N	2020-10-12 15:54:27.569123
64706	PPA	2020-09-29	56.3298	56.655	55.9905	56.15	23485	{}	\N	2020-10-12 15:54:27.580868
64707	PPA	2020-09-30	56.27	56.6979	55.3113	55.73	31461	{}	\N	2020-10-12 15:54:27.588179
64708	PPA	2020-10-01	56.02	56.69	55.5501	55.91	27035	{}	\N	2020-10-12 15:54:27.593936
64709	PPA	2020-10-02	54.91	56.59	54.72	56.41	44017	{}	\N	2020-10-12 15:54:27.599723
64710	PPA	2020-10-05	56.76	57.3033	56.76	57.01	36410	{}	\N	2020-10-12 15:54:27.604737
64711	PPA	2020-10-06	57.34	57.74	56.175	56.27	74180	{}	\N	2020-10-12 15:54:27.608646
64712	PPA	2020-10-07	56.78	57.39	56.78	57.24	54536	{}	\N	2020-10-12 15:54:27.612948
64713	PPA	2020-10-08	57.6	58.1	57.34	58.09	72625	{}	\N	2020-10-12 15:54:27.617462
64714	PPA	2020-10-09	58.42	58.51	57.9	57.9022	35433	{}	\N	2020-10-12 15:54:27.621809
64715	PPA	2020-10-12	57.99	58.345	57.91	58.25	16477	{}	\N	2020-10-12 15:54:27.625674
64716	PPLT	2020-09-28	81.48	83.4477	81.32	83.02	159553	{}	\N	2020-10-12 15:54:28.56603
64717	PPLT	2020-09-29	82.82	83.89	82.54	83.25	109038	{}	\N	2020-10-12 15:54:28.575857
64718	PPLT	2020-09-30	83.5	85.18	82.501	83.99	129621	{}	\N	2020-10-12 15:54:28.582803
64719	PPLT	2020-10-01	84.76	85.04	83.76	84.74	90232	{}	\N	2020-10-12 15:54:28.588821
64720	PPLT	2020-10-02	83.82	84.08	82.6	82.6	85131	{}	\N	2020-10-12 15:54:28.594166
64721	PPLT	2020-10-05	83.11	84.67	83.11	84.36	59203	{}	\N	2020-10-12 15:54:28.599462
64722	PPLT	2020-10-06	82.5482	82.96	80.32	80.73	320260	{}	\N	2020-10-12 15:54:28.60347
64723	PPLT	2020-10-07	81.25	81.58	80.4619	81.32	70735	{}	\N	2020-10-12 15:54:28.608234
64724	PPLT	2020-10-08	81.62	82	80.6216	81.24	106980	{}	\N	2020-10-12 15:54:28.612372
64725	PPLT	2020-10-09	83.8	83.8878	83.025	83.66	195164	{}	\N	2020-10-12 15:54:28.616866
64726	PPLT	2020-10-12	82.98	83.41	82.4537	82.8147	54357	{}	\N	2020-10-12 15:54:28.621173
64727	PRAA	2020-09-28	38.82	40.74	38.82	40.23	295138	{}	\N	2020-10-12 15:54:30.102529
64728	PRAA	2020-09-29	39.87	40.35	39.3	40.17	242988	{}	\N	2020-10-12 15:54:30.113054
64729	PRAA	2020-09-30	40.35	41.15	39.75	39.95	220334	{}	\N	2020-10-12 15:54:30.119803
64730	PRAA	2020-10-01	39.9	40.39	39.025	40.07	264606	{}	\N	2020-10-12 15:54:30.125644
64731	PRAA	2020-10-02	39.32	40.71	39.32	40.55	255780	{}	\N	2020-10-12 15:54:30.130963
64732	PRAA	2020-10-05	40.81	40.99	40.19	40.75	217935	{}	\N	2020-10-12 15:54:30.135638
64733	PRAA	2020-10-06	41.23	41.44	39.97	39.97	265451	{}	\N	2020-10-12 15:54:30.139791
64734	PRAA	2020-10-07	40.33	40.93	39.4	39.49	319401	{}	\N	2020-10-12 15:54:30.144121
64735	PRAA	2020-10-08	39.86	40.3	38.96	39.39	160272	{}	\N	2020-10-12 15:54:30.148271
64736	PRAA	2020-10-09	39.9	40.02	38.95	39.48	242360	{}	\N	2020-10-12 15:54:30.152621
64737	PRAA	2020-10-12	39.31	39.47	38.53	38.625	56402	{}	\N	2020-10-12 15:54:30.157072
64738	PRFT	2020-09-28	42.5	43.23	42.33	43.04	216496	{}	\N	2020-10-12 15:54:31.345949
64739	PRFT	2020-09-29	42.81	43.765	42.81	43.65	270888	{}	\N	2020-10-12 15:54:31.358827
64740	PRFT	2020-09-30	43.64	44.2	42.6	42.74	514378	{}	\N	2020-10-12 15:54:31.366911
64741	PRFT	2020-10-01	43.46	43.76	43.0156	43.63	240076	{}	\N	2020-10-12 15:54:31.372999
64742	PRFT	2020-10-02	42.66	43.69	42.66	43.23	238827	{}	\N	2020-10-12 15:54:31.378549
64743	PRFT	2020-10-05	43.84	44	43.39	43.88	117783	{}	\N	2020-10-12 15:54:31.384012
64744	PRFT	2020-10-06	44.21	44.65	43.45	43.55	181933	{}	\N	2020-10-12 15:54:31.389871
64745	PRFT	2020-10-07	43.92	44.57	43.54	44.4	208466	{}	\N	2020-10-12 15:54:31.395363
64746	PRFT	2020-10-08	45.06	45.31	44.5	44.64	119273	{}	\N	2020-10-12 15:54:31.400972
64747	PRFT	2020-10-09	45.22	45.44	44.835	45.1	150685	{}	\N	2020-10-12 15:54:31.406521
64748	PRFT	2020-10-12	45.41	45.49	44.89	44.89	21309	{}	\N	2020-10-12 15:54:31.412619
64749	PRK	2020-09-28	82.17	84.16	82.17	83.29	90813	{}	\N	2020-10-12 15:54:32.342823
64750	PRK	2020-09-29	83.3	83.3	81.3	82.42	44141	{}	\N	2020-10-12 15:54:32.354567
64751	PRK	2020-09-30	83	83.89	81.57	81.96	61345	{}	\N	2020-10-12 15:54:32.362342
64752	PRK	2020-10-01	81.91	83	81.21	81.79	61940	{}	\N	2020-10-12 15:54:32.370122
64753	PRK	2020-10-02	82.37	85.105	81.2512	84.29	40733	{}	\N	2020-10-12 15:54:32.376029
64754	PRK	2020-10-05	85.15	87.815	85.15	87.75	62574	{}	\N	2020-10-12 15:54:32.380749
64755	PRK	2020-10-06	88.82	91.9	87.86	88.89	89542	{}	\N	2020-10-12 15:54:32.385946
64756	PRK	2020-10-07	89.24	90.5687	88.315	90.41	63484	{}	\N	2020-10-12 15:54:32.391532
64757	PRK	2020-10-08	90.45	92.47	90.45	92.44	43054	{}	\N	2020-10-12 15:54:32.396365
64758	PRK	2020-10-09	93.1	93.1	90.1	90.25	71572	{}	\N	2020-10-12 15:54:32.401514
64759	PRK	2020-10-12	89.98	90.93	89.98	90.7	8937	{}	\N	2020-10-12 15:54:32.406403
64760	PKO	2020-09-28	22.34	22.54	22.3	22.33	99135	{}	\N	2020-10-12 15:54:33.718809
64761	PKO	2020-09-29	22.4	22.85	22.4	22.75	216059	{}	\N	2020-10-12 15:54:33.732494
64762	PKO	2020-09-30	22.86	23.1	22.8004	23.1	165466	{}	\N	2020-10-12 15:54:33.742604
64763	PKO	2020-10-01	23.23	23.36	23.08	23.13	186611	{}	\N	2020-10-12 15:54:33.750647
64764	PKO	2020-10-02	22.96	23.3493	22.96	23.32	145650	{}	\N	2020-10-12 15:54:33.757692
64765	PKO	2020-10-05	23.34	23.48	23.3322	23.41	188716	{}	\N	2020-10-12 15:54:33.764185
64766	PKO	2020-10-06	23.45	23.48	23.345	23.47	135394	{}	\N	2020-10-12 15:54:33.769776
64767	PKO	2020-10-07	23.51	23.6664	23.43	23.5	188042	{}	\N	2020-10-12 15:54:33.776308
64768	PKO	2020-10-08	23.46	23.64	23.43	23.57	136502	{}	\N	2020-10-12 15:54:33.782261
64769	PKO	2020-10-09	23.29	23.39	23.2785	23.3	129860	{}	\N	2020-10-12 15:54:33.787491
64770	PKO	2020-10-12	23.39	23.39	23.075	23.2734	71178	{}	\N	2020-10-12 15:54:33.793648
64771	PLM	2020-09-28	3.69	3.88	3.53	3.66	3873619	{}	\N	2020-10-12 15:54:34.719828
64772	PLM	2020-09-29	3.72	3.75	3.6	3.72	429576	{}	\N	2020-10-12 15:54:34.734478
64773	PLM	2020-09-30	3.67	3.7999	3.63	3.65	200147	{}	\N	2020-10-12 15:54:34.742547
64774	PLM	2020-10-01	3.77	3.87	3.68	3.86	660233	{}	\N	2020-10-12 15:54:34.75005
64775	PLM	2020-10-02	3.71	3.98	3.71	3.91	813611	{}	\N	2020-10-12 15:54:34.756933
64776	PLM	2020-10-05	3.92	4.43	3.91	4.42	2634012	{}	\N	2020-10-12 15:54:34.763285
64777	PLM	2020-10-06	4.63	4.8999	4.03	4.11	3608634	{}	\N	2020-10-12 15:54:34.7689
64778	PLM	2020-10-07	4.15	4.39	3.87	4.34	891845	{}	\N	2020-10-12 15:54:34.774902
64779	PLM	2020-10-08	4.5	4.5	4.25	4.3	414584	{}	\N	2020-10-12 15:54:34.78069
64780	PLM	2020-10-09	4.28	4.45	4.19	4.4	432829	{}	\N	2020-10-12 15:54:34.785996
64781	PLM	2020-10-12	4.47	4.48	4.26	4.2671	170784	{}	\N	2020-10-12 15:54:34.791624
64782	PLW	2020-09-28	39.47	39.4799	39.41	39.46	22755	{}	\N	2020-10-12 15:54:35.874421
64783	PLW	2020-09-29	39.48	39.53	39.4603	39.48	13845	{}	\N	2020-10-12 15:54:35.884193
64784	PLW	2020-09-30	39.35	39.38	39.18	39.29	149288	{}	\N	2020-10-12 15:54:35.890023
64785	PLW	2020-10-01	39.19	39.367	39.15	39.3	1178988	{}	\N	2020-10-12 15:54:35.895673
64786	PLW	2020-10-02	39.34	39.34	39.17	39.22	228222	{}	\N	2020-10-12 15:54:35.900446
64787	PLW	2020-10-05	39.01	39.01	38.8	38.82	80093	{}	\N	2020-10-12 15:54:35.904851
64788	PLW	2020-10-06	38.77	38.9919	38.6979	38.92	87248	{}	\N	2020-10-12 15:54:35.909775
64789	PLW	2020-10-07	38.81	38.88	38.7	38.74	24503	{}	\N	2020-10-12 15:54:35.914553
64790	PLW	2020-10-08	38.84	38.88	38.82	38.87	162288	{}	\N	2020-10-12 15:54:35.91856
64791	PLW	2020-10-09	38.78	38.8719	38.71	38.85	24700	{}	\N	2020-10-12 15:54:35.922815
64792	PLW	2020-10-12	38.92	38.92	38.8749	38.885	16678	{}	\N	2020-10-12 15:54:35.929245
64793	PMO	2020-09-28	13.09	13.09	12.97	13.03	37551	{}	\N	2020-10-12 15:54:37.552761
64794	PMO	2020-09-29	13.03	13.03	12.99	12.99	62140	{}	\N	2020-10-12 15:54:37.563815
64795	PMO	2020-09-30	12.99	13.02	12.97	12.97	110572	{}	\N	2020-10-12 15:54:37.571425
64796	PMO	2020-10-01	13.05	13.06	12.99	13.05	103738	{}	\N	2020-10-12 15:54:37.57716
64797	PMO	2020-10-02	13.04	13.05	12.99	13.05	94998	{}	\N	2020-10-12 15:54:37.582532
64798	PMO	2020-10-05	13.05	13.08	12.96	13.05	118331	{}	\N	2020-10-12 15:54:37.587989
64799	PMO	2020-10-06	13.04	13.13	13.04	13.13	107540	{}	\N	2020-10-12 15:54:37.592436
64800	PMO	2020-10-07	13.17	13.18	13.1351	13.14	142202	{}	\N	2020-10-12 15:54:37.597324
64801	PMO	2020-10-08	13.18	13.18	13.07	13.07	93358	{}	\N	2020-10-12 15:54:37.602468
64802	PMO	2020-10-09	13.11	13.15	13.09	13.12	124548	{}	\N	2020-10-12 15:54:37.607175
64803	PMO	2020-10-12	13.15	13.16	13.12	13.155	19520	{}	\N	2020-10-12 15:54:37.61215
64804	PNQI	2020-09-28	205.56	205.56	202.37	204.32	19311	{}	\N	2020-10-12 15:54:38.750052
64805	PNQI	2020-09-29	204.75	204.92	203.6692	204.21	14162	{}	\N	2020-10-12 15:54:38.759517
64806	PNQI	2020-09-30	205.4	208.58	205.1	206.31	23163	{}	\N	2020-10-12 15:54:38.767068
64807	PNQI	2020-10-01	208.05	210.771	208.05	210.69	14566	{}	\N	2020-10-12 15:54:38.773252
64808	PNQI	2020-10-02	205.65	209.75	204.89	206.47	17848	{}	\N	2020-10-12 15:54:38.779416
64809	PNQI	2020-10-05	207.53	209.619	207.53	209.56	9191	{}	\N	2020-10-12 15:54:38.784603
64810	PNQI	2020-10-06	209.87	211.06	206.5477	207.43	12857	{}	\N	2020-10-12 15:54:38.789384
64811	PNQI	2020-10-07	209.7325	211.627	209.05	211.253	17403	{}	\N	2020-10-12 15:54:38.794261
64812	PNQI	2020-10-08	212.06	213.0052	211.5626	212.4194	26820	{}	\N	2020-10-12 15:54:38.799414
64813	PNQI	2020-10-09	213.355	215.78	213.355	215.78	17091	{}	\N	2020-10-12 15:54:38.804361
64814	PNQI	2020-10-12	218.35	219.4993	217.63	219.45	16751	{}	\N	2020-10-12 15:54:38.809495
64815	PPBI	2020-09-28	19.9	20.58	19.9	20.31	282192	{}	\N	2020-10-12 15:54:40.077138
64816	PPBI	2020-09-29	20.12	20.43	19.8	20.24	342205	{}	\N	2020-10-12 15:54:40.091079
64817	PPBI	2020-09-30	20.47	20.71	19.89	20.14	449658	{}	\N	2020-10-12 15:54:40.097612
64818	PPBI	2020-10-01	20.09	20.77	19.79	20.73	457591	{}	\N	2020-10-12 15:54:40.10396
64819	PPBI	2020-10-02	20.32	21.64	20.3	21.56	290201	{}	\N	2020-10-12 15:54:40.109416
64820	PPBI	2020-10-05	21.85	22.71	21.67	22.65	286114	{}	\N	2020-10-12 15:54:40.114928
64821	PPBI	2020-10-06	23.06	23.86	22.61	22.71	686225	{}	\N	2020-10-12 15:54:40.120023
64822	PPBI	2020-10-07	23.05	23.67	22.88	23.55	495600	{}	\N	2020-10-12 15:54:40.124797
64823	PPBI	2020-10-08	23.91	24.03	23.47	24	264104	{}	\N	2020-10-12 15:54:40.129517
64824	PPBI	2020-10-09	24.35	24.38	23.785	24.1	339950	{}	\N	2020-10-12 15:54:40.134312
64825	PPBI	2020-10-12	24.06	24.41	23.83	24.315	94254	{}	\N	2020-10-12 15:54:40.138969
64826	PPR	2020-09-28	4.43	4.457	4.395	4.41	482951	{}	\N	2020-10-12 15:54:41.117757
64827	PPR	2020-09-29	4.4	4.42	4.4	4.41	1172620	{}	\N	2020-10-12 15:54:41.130442
64828	PPR	2020-09-30	4.4	4.43	4.38	4.41	6238983	{}	\N	2020-10-12 15:54:41.137777
64829	PPR	2020-10-01	4.41	4.43	4.4	4.4	311536	{}	\N	2020-10-12 15:54:41.144158
64830	PPR	2020-10-02	4.39	4.43	4.38	4.42	589182	{}	\N	2020-10-12 15:54:41.14988
64831	PPR	2020-10-05	4.41	4.44	4.41	4.41	388202	{}	\N	2020-10-12 15:54:41.154956
64832	PPR	2020-10-06	4.4	4.425	4.4	4.41	314416	{}	\N	2020-10-12 15:54:41.160374
64833	PPR	2020-10-07	4.42	4.43	4.4	4.41	365418	{}	\N	2020-10-12 15:54:41.166134
64834	PPR	2020-10-08	4.41	4.45	4.41	4.43	291249	{}	\N	2020-10-12 15:54:41.171388
64835	PPR	2020-10-09	4.41	4.43	4.41	4.41	269902	{}	\N	2020-10-12 15:54:41.176679
64836	PPR	2020-10-12	4.42	4.4299	4.42	4.425	93500	{}	\N	2020-10-12 15:54:41.182338
64837	PRAH	2020-09-28	98.5	99.74	96.86	99.15	142225	{}	\N	2020-10-12 15:54:42.125279
64838	PRAH	2020-09-29	99.55	100.275	98.085	98.28	157969	{}	\N	2020-10-12 15:54:42.136265
64839	PRAH	2020-09-30	98.27	101.965	97.72	101.44	382954	{}	\N	2020-10-12 15:54:42.143679
64840	PRAH	2020-10-01	101.28	105.5	100.2	105.11	366771	{}	\N	2020-10-12 15:54:42.150257
64841	PRAH	2020-10-02	103.99	106.66	101.95	105.83	472387	{}	\N	2020-10-12 15:54:42.156062
64842	PRAH	2020-10-05	106.62	109.19	105.1	109.08	464095	{}	\N	2020-10-12 15:54:42.162075
64843	PRAH	2020-10-06	109.07	109.47	106.47	108	285718	{}	\N	2020-10-12 15:54:42.168357
64844	PRAH	2020-10-07	108.69	109.78	107.715	109.285	374832	{}	\N	2020-10-12 15:54:42.173748
64845	PRAH	2020-10-08	109.8	110.52	108.955	109.76	151808	{}	\N	2020-10-12 15:54:42.179556
64846	PRAH	2020-10-09	110.7	112.05	109.18	111.5	371192	{}	\N	2020-10-12 15:54:42.185243
64847	PRAH	2020-10-12	112.22	112.28	110.58	111.435	68145	{}	\N	2020-10-12 15:54:42.190756
64848	PRGO	2020-09-28	45.79	46	45.47	45.63	449802	{}	\N	2020-10-12 15:54:43.40736
64849	PRGO	2020-09-29	45.32	46.11	45.27	45.67	1183811	{}	\N	2020-10-12 15:54:43.419069
64850	PRGO	2020-09-30	45.59	46.46	45.56	45.91	1456167	{}	\N	2020-10-12 15:54:43.426515
64851	PRGO	2020-10-01	45.99	46.11	45.03	45.28	1455129	{}	\N	2020-10-12 15:54:43.433268
64852	PRGO	2020-10-02	44.78	45.43	44.44	44.85	1238447	{}	\N	2020-10-12 15:54:43.439401
64853	PRGO	2020-10-05	45.17	45.81	44.85	45.3	2685422	{}	\N	2020-10-12 15:54:43.444397
64854	PRGO	2020-10-06	45.56	45.6399	44.28	44.4	2487195	{}	\N	2020-10-12 15:54:43.4497
64855	PRGO	2020-10-07	44.62	45.46	44.59	45.12	1185572	{}	\N	2020-10-12 15:54:43.455079
64856	PRGO	2020-10-08	45.49	45.96	45.3	45.8	1041778	{}	\N	2020-10-12 15:54:43.460394
64857	PRGO	2020-10-09	45.69	46.235	45.17	45.86	1528508	{}	\N	2020-10-12 15:54:43.465811
64858	PRGO	2020-10-12	46.04	46.77	45.86	46.5	279344	{}	\N	2020-10-12 15:54:43.471723
64859	PRQR	2020-09-28	5.06	5.06	4.83	4.93	88954	{}	\N	2020-10-12 15:54:44.647762
64860	PRQR	2020-09-29	5	5	4.82	4.82	73378	{}	\N	2020-10-12 15:54:44.659679
64861	PRQR	2020-09-30	4.88	4.9378	4.78	4.79	151570	{}	\N	2020-10-12 15:54:44.666503
64862	PRQR	2020-10-01	4.84	4.96	4.74	4.85	131013	{}	\N	2020-10-12 15:54:44.672982
64863	PRQR	2020-10-02	4.83	4.94	4.72	4.75	153815	{}	\N	2020-10-12 15:54:44.678547
64864	PRQR	2020-10-05	4.77	4.93	4.72	4.84	180718	{}	\N	2020-10-12 15:54:44.683673
64865	PRQR	2020-10-06	4.83	4.9	4.79	4.84	90114	{}	\N	2020-10-12 15:54:44.688652
64866	PRQR	2020-10-07	4.88	4.8999	4.8	4.85	81472	{}	\N	2020-10-12 15:54:44.693498
64867	PRQR	2020-10-08	4.89	4.9	4.79	4.84	60482	{}	\N	2020-10-12 15:54:44.698493
64868	PRQR	2020-10-09	4.84	4.9388	4.75	4.82	146850	{}	\N	2020-10-12 15:54:44.703862
64869	PRQR	2020-10-12	4.87	4.87	4.7	4.82	49991	{}	\N	2020-10-12 15:54:44.708882
64870	PSA	2020-09-28	220.83	223.101	219.79	221.13	588773	{}	\N	2020-10-12 15:54:45.751703
64871	PSA	2020-09-29	221.15	222.9	218.47	220.91	1090139	{}	\N	2020-10-12 15:54:45.764039
64872	PSA	2020-09-30	222.16	223.48	219.21	222.72	1661091	{}	\N	2020-10-12 15:54:45.770992
64873	PSA	2020-10-01	222.71	226.32	222.01	226.26	1644510	{}	\N	2020-10-12 15:54:45.777776
64874	PSA	2020-10-02	224.57	231.96	222.77	230.59	1590676	{}	\N	2020-10-12 15:54:45.783199
64875	PSA	2020-10-05	231.8	232.64	226.6	232.16	1160019	{}	\N	2020-10-12 15:54:45.7896
64876	PSA	2020-10-06	229.26	230.65	225.6797	228.29	1988360	{}	\N	2020-10-12 15:54:45.795433
64877	PSA	2020-10-07	229.09	231.53	228.07	230.04	1159908	{}	\N	2020-10-12 15:54:45.800678
64878	PSA	2020-10-08	230.84	233.1	230.31	232.02	1171736	{}	\N	2020-10-12 15:54:45.806198
64879	PSA	2020-10-09	232.45	234.25	230.66	234.01	837981	{}	\N	2020-10-12 15:54:45.811452
64880	PSA	2020-10-12	234.37	236.34	232.77	236.04	335037	{}	\N	2020-10-12 15:54:45.816348
64881	QDEL	2020-09-28	198.79	200.9	194.3292	198.64	691978	{}	\N	2020-10-12 15:54:46.716401
64882	QDEL	2020-09-29	200.33	219.1908	199.37	215	1395345	{}	\N	2020-10-12 15:54:46.726095
64883	QDEL	2020-09-30	213.06	220.94	211.48	219.38	1037245	{}	\N	2020-10-12 15:54:46.732844
64884	QDEL	2020-10-01	239	244.9	219.61	225.54	2270363	{}	\N	2020-10-12 15:54:46.738772
64885	QDEL	2020-10-02	221.85	238.906	221.07	235.34	1111020	{}	\N	2020-10-12 15:54:46.744246
64886	QDEL	2020-10-05	242.41	253.93	241.9287	251.68	1274456	{}	\N	2020-10-12 15:54:46.749199
64887	QDEL	2020-10-06	250.33	255.8	240.06	250	875413	{}	\N	2020-10-12 15:54:46.754315
64888	QDEL	2020-10-07	255.26	281.43	254.317	277.82	1293605	{}	\N	2020-10-12 15:54:46.760209
64889	QDEL	2020-10-08	282.507	282.507	269.28	276.78	958943	{}	\N	2020-10-12 15:54:46.76539
64890	QDEL	2020-10-09	278.65	283.515	275.8	278.05	696995	{}	\N	2020-10-12 15:54:46.770834
64891	QDEL	2020-10-12	282	287.3	270.69	274.465	551891	{}	\N	2020-10-12 15:54:46.776525
64892	PRGS	2020-09-28	36.26	37.425	36.03	37.28	498437	{}	\N	2020-10-12 15:54:49.425621
64893	PRGS	2020-09-29	36.9304	38.854	36.9304	38.75	932004	{}	\N	2020-10-12 15:54:49.43655
64894	PRGS	2020-09-30	34.65	37.325	34.5	36.68	1414429	{}	\N	2020-10-12 15:54:49.444335
64895	PRGS	2020-10-01	36.79	38.26	36.79	37.54	854505	{}	\N	2020-10-12 15:54:49.450532
64896	PRGS	2020-10-02	37.68	38.79	37.31	37.77	501397	{}	\N	2020-10-12 15:54:49.456157
64897	PRGS	2020-10-05	37.75	38.1024	37.6	37.87	426486	{}	\N	2020-10-12 15:54:49.460603
64898	PRGS	2020-10-06	38.29	39.535	38.225	38.78	474409	{}	\N	2020-10-12 15:54:49.465557
64899	PRGS	2020-10-07	39.145	39.28	38.73	38.89	423240	{}	\N	2020-10-12 15:54:49.470401
64900	PRGS	2020-10-08	39.2	39.7	39	39.51	427967	{}	\N	2020-10-12 15:54:49.475349
64901	PRGS	2020-10-09	39.66	40.28	39	40.22	360900	{}	\N	2020-10-12 15:54:49.479934
64902	PRGS	2020-10-12	40.38	40.92	40.38	40.77	89487	{}	\N	2020-10-12 15:54:49.485244
64903	PRNT	2020-09-28	22.9	23.8	22.9	23.4283	94521	{}	\N	2020-10-12 15:54:50.569582
64904	PRNT	2020-09-29	23.73	24	23.525	24	29879	{}	\N	2020-10-12 15:54:50.584464
64905	PRNT	2020-09-30	24.09	24.1566	23.7031	23.94	32598	{}	\N	2020-10-12 15:54:50.592936
64906	PRNT	2020-10-01	24.35	24.6599	24.15	24.362	28271	{}	\N	2020-10-12 15:54:50.599548
64907	PRNT	2020-10-02	23.4	24.195	23.29	24.12	19700	{}	\N	2020-10-12 15:54:50.605577
64908	PRNT	2020-10-05	24.4	24.6	24.2429	24.5761	20055	{}	\N	2020-10-12 15:54:50.61108
64909	PRNT	2020-10-06	24.6419	24.99	24.605	24.719	18483	{}	\N	2020-10-12 15:54:50.616941
64910	PRNT	2020-10-07	24.95	25.58	24.7313	25.4313	94414	{}	\N	2020-10-12 15:54:50.622549
64911	PRNT	2020-10-08	25.96	25.9691	25.78	25.8387	47025	{}	\N	2020-10-12 15:54:50.628086
64912	PRNT	2020-10-09	25.95	26.46	25.9	26.4	189483	{}	\N	2020-10-12 15:54:50.633236
64913	PRNT	2020-10-12	26.69	26.9741	26.69	26.86	34923	{}	\N	2020-10-12 15:54:50.63858
64914	PRTA	2020-09-28	10.23	10.23	9.93	10	145202	{}	\N	2020-10-12 15:54:52.283534
64915	PRTA	2020-09-29	9.99	10.175	9.67	9.93	155222	{}	\N	2020-10-12 15:54:52.295468
64916	PRTA	2020-09-30	10.04	10.15	9.89	9.99	248215	{}	\N	2020-10-12 15:54:52.302664
64917	PRTA	2020-10-01	10.04	10.42	9.92	10.31	245329	{}	\N	2020-10-12 15:54:52.309048
64918	PRTA	2020-10-02	10.12	10.48	10.04	10.12	248010	{}	\N	2020-10-12 15:54:52.315226
64919	PRTA	2020-10-05	10.25	10.67	9.86	10.64	165538	{}	\N	2020-10-12 15:54:52.320897
64920	PRTA	2020-10-06	10.67	11.08	10.48	10.93	278940	{}	\N	2020-10-12 15:54:52.326106
64921	PRTA	2020-10-07	10.97	11.13	10.75	11.07	112682	{}	\N	2020-10-12 15:54:52.331351
64922	PRTA	2020-10-08	11.21	11.26	10.9	11.04	139530	{}	\N	2020-10-12 15:54:52.336946
64923	PRTA	2020-10-09	11.12	11.18	10.84	11.065	102417	{}	\N	2020-10-12 15:54:52.343292
64924	PRTA	2020-10-12	11.12	11.29	10.93	11.28	33019	{}	\N	2020-10-12 15:54:52.349554
64925	PSCC	2020-09-28	75.0848	75.0848	74.6531	74.6531	1046	{}	\N	2020-10-12 15:54:53.458124
64926	PSCC	2020-09-29	73.38	73.7864	73.2368	73.6	2035	{}	\N	2020-10-12 15:54:53.46543
64927	PSCC	2020-10-01	73.2991	74.2639	73.2991	74.2639	2240	{}	\N	2020-10-12 15:54:53.471443
64928	PSCC	2020-10-02	73.5836	74.8459	73.5836	74.8459	1227	{}	\N	2020-10-12 15:54:53.477188
64929	PSCC	2020-10-05	75.196	75.3482	75.0125	75.3482	940	{}	\N	2020-10-12 15:54:53.482799
64930	PSCC	2020-10-06	76.355	76.355	76.355	76.355	335	{}	\N	2020-10-12 15:54:53.488733
64931	PSCC	2020-10-07	76.72	76.7499	76.4976	76.4976	1533	{}	\N	2020-10-12 15:54:53.495699
64932	PSCC	2020-10-08	76.38	77	76.38	76.895	1309	{}	\N	2020-10-12 15:54:53.500481
64933	PSCC	2020-10-09	76.94	77.8759	76.94	77.8759	693	{}	\N	2020-10-12 15:54:53.50526
64934	PSMG	2020-09-28	14.2	14.2	14.1845	14.1845	402	{}	\N	2020-10-12 15:54:56.143691
64935	PSMG	2020-09-29	14.15	14.15	14.1259	14.1259	769	{}	\N	2020-10-12 15:54:56.152063
64936	PSMG	2020-09-30	14.24	14.25	14.1358	14.18	1361	{}	\N	2020-10-12 15:54:56.158104
64937	PSMG	2020-10-01	14.24	14.24	14.2102	14.2102	3262	{}	\N	2020-10-12 15:54:56.163592
64938	PSMG	2020-10-02	14.23	14.28	14.23	14.27	2677	{}	\N	2020-10-12 15:54:56.169084
64939	PSMG	2020-10-05	14.435	14.4498	14.4302	14.4302	1367	{}	\N	2020-10-12 15:54:56.17395
64940	PSMG	2020-10-06	14.54	14.54	14.37	14.37	571	{}	\N	2020-10-12 15:54:56.178612
64941	PSMG	2020-10-07	14.5099	14.5099	14.5099	14.5099	99	{}	\N	2020-10-12 15:54:56.184103
64942	PSMG	2020-10-08	14.6361	14.6361	14.6361	14.6361	86	{}	\N	2020-10-12 15:54:56.188952
64943	PSMG	2020-10-09	14.75	14.75	14.68	14.695	1932	{}	\N	2020-10-12 15:54:56.19365
64944	PSTI	2020-09-28	9.66	9.97	9.4	9.49	149548	{}	\N	2020-10-12 15:54:57.594421
64945	PSTI	2020-09-29	9.6	10.48	9.43	10.12	299718	{}	\N	2020-10-12 15:54:57.604323
64946	PSTI	2020-09-30	10.15	10.55	10.0967	10.53	367623	{}	\N	2020-10-12 15:54:57.610566
64947	PSTI	2020-10-01	10.29	10.42	9.63	10.02	245996	{}	\N	2020-10-12 15:54:57.61672
64948	PSTI	2020-10-02	9.99	10.64	9.93	10.22	258098	{}	\N	2020-10-12 15:54:57.622606
64949	PSTI	2020-10-05	10.22	10.5622	10.1673	10.47	82253	{}	\N	2020-10-12 15:54:57.627147
64950	PSTI	2020-10-06	10.47	10.57	10.16	10.3	116951	{}	\N	2020-10-12 15:54:57.632258
64951	PSTI	2020-10-07	10.8	11.1	10.618	10.85	362637	{}	\N	2020-10-12 15:54:57.637136
64952	PSTI	2020-10-08	10.95	11.23	10.9	11.12	256645	{}	\N	2020-10-12 15:54:57.642087
64953	PSTI	2020-10-09	11.1183	11.4	10.95	11.13	143978	{}	\N	2020-10-12 15:54:57.646801
64954	PSTI	2020-10-12	10.98	11.11	10.71	10.99	69300	{}	\N	2020-10-12 15:54:57.65163
64955	PTI	2020-09-28	1.14	1.15	1.08	1.08	264519	{}	\N	2020-10-12 15:54:58.714469
64956	PTI	2020-09-29	1.09	1.13	1.07	1.11	324018	{}	\N	2020-10-12 15:54:58.728683
64957	PTI	2020-09-30	1.11	1.15	1.07	1.1	262495	{}	\N	2020-10-12 15:54:58.736207
64958	PTI	2020-10-01	1.08	1.12	1.0783	1.1	207736	{}	\N	2020-10-12 15:54:58.742567
64959	PTI	2020-10-02	1.08	1.125	1.05	1.07	293801	{}	\N	2020-10-12 15:54:58.748556
64960	PTI	2020-10-05	1.07	1.14	1.07	1.12	517945	{}	\N	2020-10-12 15:54:58.753699
64961	PTI	2020-10-06	1.12	1.17	1.09	1.09	378004	{}	\N	2020-10-12 15:54:58.75927
64962	PTI	2020-10-07	1.11	1.18	1.1	1.17	413112	{}	\N	2020-10-12 15:54:58.764695
64963	PTI	2020-10-08	1.18	1.2	1.1547	1.17	212588	{}	\N	2020-10-12 15:54:58.770066
64964	PTI	2020-10-09	1.16	1.19	1.15	1.15	105844	{}	\N	2020-10-12 15:54:58.775578
64965	PTI	2020-10-12	1.15	1.18	1.1202	1.16	97878	{}	\N	2020-10-12 15:54:58.780924
64966	PVI	2020-09-28	24.9	24.92	24.9	24.92	3908	{}	\N	2020-10-12 15:55:02.096173
64967	PVI	2020-09-29	24.9	24.924	24.9	24.924	6016	{}	\N	2020-10-12 15:55:02.106317
64968	PVI	2020-09-30	24.9	24.97	24.9	24.96	20221	{}	\N	2020-10-12 15:55:02.114354
64969	PVI	2020-10-01	24.92	24.92	24.92	24.92	729	{}	\N	2020-10-12 15:55:02.120951
64970	PVI	2020-10-02	24.929	24.929	24.905	24.91	1063	{}	\N	2020-10-12 15:55:02.12637
64971	PVI	2020-10-05	24.9102	24.95	24.9	24.92	11158	{}	\N	2020-10-12 15:55:02.132131
64972	PVI	2020-10-06	24.93	24.93	24.905	24.91	4503	{}	\N	2020-10-12 15:55:02.13783
64973	PVI	2020-10-07	24.93	24.93	24.9	24.903	4532	{}	\N	2020-10-12 15:55:02.143041
64974	PVI	2020-10-08	24.94	24.95	24.91	24.91	4070	{}	\N	2020-10-12 15:55:02.148497
64975	PVI	2020-10-09	24.919	24.92	24.9051	24.92	2095	{}	\N	2020-10-12 15:55:02.153423
64976	PRLB	2020-09-28	131	135.885	131	131.78	162348	{}	\N	2020-10-12 15:55:03.083285
64977	PRLB	2020-09-29	131.82	136.13	131.82	134.11	338605	{}	\N	2020-10-12 15:55:03.096573
64978	PRLB	2020-09-30	133.71	136.74	128.85	129.5	869362	{}	\N	2020-10-12 15:55:03.105334
64979	PRLB	2020-10-01	130.44	133.2774	130.44	132.9	514395	{}	\N	2020-10-12 15:55:03.111292
64980	PRLB	2020-10-02	129.83	133.23	128.73	131.45	481885	{}	\N	2020-10-12 15:55:03.117744
64981	PRLB	2020-10-05	132.16	136.13	132.16	135.74	249957	{}	\N	2020-10-12 15:55:03.122554
64982	PRLB	2020-10-06	136.44	143.11	135.57	138.03	434220	{}	\N	2020-10-12 15:55:03.127287
64983	PRLB	2020-10-07	139.49	148.14	139.29	145.05	577683	{}	\N	2020-10-12 15:55:03.132182
64984	PRLB	2020-10-08	147.28	147.75	142.69	147.1	394141	{}	\N	2020-10-12 15:55:03.137965
64985	PRLB	2020-10-09	149.16	153.595	147.98	150.77	391528	{}	\N	2020-10-12 15:55:03.143083
64986	PRLB	2020-10-12	151.6	152.7	147.43	148.17	147795	{}	\N	2020-10-12 15:55:03.148049
64987	PRSC	2020-09-28	86.74	89.155	86.49	89.11	68482	{}	\N	2020-10-12 15:55:04.912204
64988	PRSC	2020-09-29	90	91.98	87.77	89.71	117495	{}	\N	2020-10-12 15:55:04.927709
64989	PRSC	2020-09-30	89.49	93.56	89.03	92.91	126435	{}	\N	2020-10-12 15:55:04.935202
64990	PRSC	2020-10-01	93.92	95.12	92.39	94.05	250155	{}	\N	2020-10-12 15:55:04.942783
64991	PRSC	2020-10-02	92.81	96.17	92.81	94.96	205182	{}	\N	2020-10-12 15:55:04.949411
64992	PRSC	2020-10-05	94.96	95.61	91.4401	94.01	77400	{}	\N	2020-10-12 15:55:04.955292
64993	PRSC	2020-10-06	94.53	95.935	93.83	94.02	95219	{}	\N	2020-10-12 15:55:04.960648
64994	PRSC	2020-10-07	94.32	95.925	93.015	93.54	61643	{}	\N	2020-10-12 15:55:04.966113
64995	PRSC	2020-10-08	94.01	95.19	93.37	94.64	37770	{}	\N	2020-10-12 15:55:04.971653
64996	PRSC	2020-10-09	95.51	97.47	95.51	96.5	106071	{}	\N	2020-10-12 15:55:04.976683
64997	PRSC	2020-10-12	97.08	98.85	97.08	98.85	35709	{}	\N	2020-10-12 15:55:04.981653
64998	PSCF	2020-09-28	37.5547	37.9374	37.5547	37.77	4661	{}	\N	2020-10-12 15:55:06.00642
64999	PSCF	2020-09-29	37.2499	37.4516	37.11	37.4516	537	{}	\N	2020-10-12 15:55:06.017308
65000	PSCF	2020-09-30	37.78	38.0295	37.4103	37.61	2976	{}	\N	2020-10-12 15:55:06.026058
65001	PSCF	2020-10-01	38.1984	38.48	37.9153	38.48	6838	{}	\N	2020-10-12 15:55:06.03323
65002	PSCF	2020-10-02	38.26	39.4458	38.26	39.4458	967	{}	\N	2020-10-12 15:55:06.039242
65003	PSCF	2020-10-05	39.9933	40.125	39.892	40.125	654	{}	\N	2020-10-12 15:55:06.044598
65004	PSCF	2020-10-06	40.6645	40.6645	39.9552	39.9552	883	{}	\N	2020-10-12 15:55:06.049786
65005	PSCF	2020-10-07	40.6999	40.7099	40.08	40.105	4361	{}	\N	2020-10-12 15:55:06.054982
65006	PSCF	2020-10-08	40.54	41.0625	40.54	41.0625	1789	{}	\N	2020-10-12 15:55:06.060358
65007	PSCF	2020-10-09	41.17	41.17	40.76	40.76	1902	{}	\N	2020-10-12 15:55:06.065206
65008	PSCF	2020-10-12	40.8	41.0147	40.8	40.96	381	{}	\N	2020-10-12 15:55:06.070403
65009	PSL	2020-09-28	76.8	77.03	76.4545	76.59	25203	{}	\N	2020-10-12 15:55:07.078708
65010	PSL	2020-09-29	76.6	76.6092	75.72	75.73	6998	{}	\N	2020-10-12 15:55:07.08834
65011	PSL	2020-09-30	76.18	76.73	75.915	75.915	31078	{}	\N	2020-10-12 15:55:07.094612
65012	PSL	2020-10-01	76.45	77.14	76.02	77.08	4404	{}	\N	2020-10-12 15:55:07.099755
65013	PSL	2020-10-02	76.9142	77.2799	76.39	77.13	5320	{}	\N	2020-10-12 15:55:07.104541
65014	PSL	2020-10-05	77.83	77.9807	77.4418	77.9807	4127	{}	\N	2020-10-12 15:55:07.109308
65015	PSL	2020-10-06	78.46	78.81	77.5352	77.79	1225	{}	\N	2020-10-12 15:55:07.113578
65016	PSL	2020-10-07	79.11	79.2215	79.005	79.2215	1333	{}	\N	2020-10-12 15:55:07.117281
65017	PSL	2020-10-08	79.72	79.72	79.19	79.19	593	{}	\N	2020-10-12 15:55:07.121561
65018	PSL	2020-10-09	80.25	80.28	79.7651	80.28	4728	{}	\N	2020-10-12 15:55:07.126449
65019	PSL	2020-10-12	80.91	80.91	80.91	80.91	134	{}	\N	2020-10-12 15:55:07.13074
65020	PSO	2020-09-28	7.26	7.31	7.18	7.2	382602	{}	\N	2020-10-12 15:55:08.148354
65021	PSO	2020-09-29	7.16	7.16	7.1	7.12	530771	{}	\N	2020-10-12 15:55:08.157735
65022	PSO	2020-09-30	7.18	7.2	7.095	7.11	448120	{}	\N	2020-10-12 15:55:08.164347
65023	PSO	2020-10-01	7.09	7.12	7.02	7.1	510449	{}	\N	2020-10-12 15:55:08.170205
65024	PSO	2020-10-02	7.17	7.33	7.17	7.33	889498	{}	\N	2020-10-12 15:55:08.175569
65025	PSO	2020-10-05	7.21	7.25	7.19	7.24	513947	{}	\N	2020-10-12 15:55:08.180365
65026	PSO	2020-10-06	7.32	7.38	7.26	7.28	488546	{}	\N	2020-10-12 15:55:08.185003
65027	PSO	2020-10-07	7.17	7.245	7.17	7.21	361784	{}	\N	2020-10-12 15:55:08.189674
65028	PSO	2020-10-08	7.15	7.165	7.1196	7.14	235968	{}	\N	2020-10-12 15:55:08.194085
65029	PSO	2020-10-09	7.42	7.49	7.38	7.47	520117	{}	\N	2020-10-12 15:55:08.199135
65030	PSO	2020-10-12	7.44	7.505	7.415	7.475	160752	{}	\N	2020-10-12 15:55:08.204322
65031	PTC	2020-09-28	83.34	83.55	82.27	82.9	520042	{}	\N	2020-10-12 15:55:09.40733
65032	PTC	2020-09-29	83.14	84.51	82.3	82.35	585069	{}	\N	2020-10-12 15:55:09.418296
65033	PTC	2020-09-30	82.35	83.41	81.68	82.72	906392	{}	\N	2020-10-12 15:55:09.425628
65034	PTC	2020-10-01	83.66	84.52	83.205	84.13	830795	{}	\N	2020-10-12 15:55:09.431696
65035	PTC	2020-10-02	81.92	83.34	81.17	82.16	648890	{}	\N	2020-10-12 15:55:09.436956
65036	PTC	2020-10-05	83	83.72	82.08	82.72	792097	{}	\N	2020-10-12 15:55:09.441839
65037	PTC	2020-10-06	82.72	84.49	81.03	81.22	795325	{}	\N	2020-10-12 15:55:09.446223
65038	PTC	2020-10-07	82	83.36	81.12	83.21	1025280	{}	\N	2020-10-12 15:55:09.45071
65039	PTC	2020-10-08	84.05	85.21	83.105	84.06	651172	{}	\N	2020-10-12 15:55:09.455059
65040	PTC	2020-10-09	84.51	86.55	84.51	85.94	450124	{}	\N	2020-10-12 15:55:09.459789
65041	PTC	2020-10-12	86.86	87.53	85.91	87.075	141661	{}	\N	2020-10-12 15:55:09.464263
65042	PTY	2020-09-28	16.12	16.1962	15.9934	16.08	361593	{}	\N	2020-10-12 15:55:11.20138
65043	PTY	2020-09-29	16.09	16.11	16	16.08	618462	{}	\N	2020-10-12 15:55:11.213781
65044	PTY	2020-09-30	16.12	16.13	16.05	16.13	606160	{}	\N	2020-10-12 15:55:11.221482
65045	PTY	2020-10-01	16.24	16.24	16.1	16.18	791199	{}	\N	2020-10-12 15:55:11.2278
65046	PTY	2020-10-02	15.92	16.24	15.81	16.2	757854	{}	\N	2020-10-12 15:55:11.232749
65047	PTY	2020-10-05	16.52	16.5999	16.46	16.5	2755273	{}	\N	2020-10-12 15:55:11.237577
65048	PTY	2020-10-06	16.66	16.7399	16.5825	16.64	1739292	{}	\N	2020-10-12 15:55:11.241729
65049	PTY	2020-10-07	16.68	16.8	16.68	16.76	1677788	{}	\N	2020-10-12 15:55:11.246362
65050	PTY	2020-10-08	16.85	16.87	16.8	16.84	1095380	{}	\N	2020-10-12 15:55:11.250593
65051	PTY	2020-10-09	16.79	16.79	16.54	16.68	1050897	{}	\N	2020-10-12 15:55:11.25503
65052	PTY	2020-10-12	16.75	16.7599	16.6	16.6199	502923	{}	\N	2020-10-12 15:55:11.259375
65053	PVAC	2020-09-28	9.48	10.08	9.3901	9.92	568782	{}	\N	2020-10-12 15:55:12.255169
65054	PVAC	2020-09-29	9.73	10.23	9.58	10.16	488571	{}	\N	2020-10-12 15:55:12.265463
65055	PVAC	2020-09-30	10.1	10.3	9.75	9.85	440546	{}	\N	2020-10-12 15:55:12.272222
65056	PVAC	2020-10-01	9.89	9.9199	9.44	9.63	318563	{}	\N	2020-10-12 15:55:12.278229
65057	PVAC	2020-10-02	9.2	9.95	9.09	9.89	318726	{}	\N	2020-10-12 15:55:12.28308
65058	PVAC	2020-10-05	10.08	10.24	9.82	10.08	293151	{}	\N	2020-10-12 15:55:12.288191
65059	PVAC	2020-10-06	10.26	10.36	9.53	9.6	520379	{}	\N	2020-10-12 15:55:12.292904
65060	PVAC	2020-10-07	9.79	10.1	9.6401	9.99	379854	{}	\N	2020-10-12 15:55:12.298044
65061	PVAC	2020-10-08	10.1	10.39	9.831	10.32	520415	{}	\N	2020-10-12 15:55:12.303084
65062	PVAC	2020-10-09	10.5	10.66	9.9	10.04	292041	{}	\N	2020-10-12 15:55:12.309184
65063	PVAC	2020-10-12	10	10.15	9.75	10.085	107885	{}	\N	2020-10-12 15:55:12.314131
65064	PWV	2020-09-28	33.87	34.15	33.87	34.03	29805	{}	\N	2020-10-12 15:55:14.105239
65065	PWV	2020-09-29	33.9001	34.04	33.66	33.77	38282	{}	\N	2020-10-12 15:55:14.116366
65066	PWV	2020-09-30	33.9	34.32	33.8999	34.04	59140	{}	\N	2020-10-12 15:55:14.123785
65067	PWV	2020-10-01	34.11	34.22	33.769	33.98	33160	{}	\N	2020-10-12 15:55:14.13062
65068	PWV	2020-10-02	33.46	34.19	33.46	34.01	24308	{}	\N	2020-10-12 15:55:14.13633
65069	PWV	2020-10-05	34.17	34.5375	34.17	34.45	31608	{}	\N	2020-10-12 15:55:14.140972
65070	PWV	2020-10-06	34.56	34.775	34.1299	34.13	10348	{}	\N	2020-10-12 15:55:14.14536
65071	PWV	2020-10-07	34.7277	34.8583	34.5901	34.79	145871	{}	\N	2020-10-12 15:55:14.150538
65072	PWV	2020-10-08	34.98	35.1231	34.8801	35.07	41472	{}	\N	2020-10-12 15:55:14.154923
65073	PWV	2020-10-09	35.29	35.32	35.09	35.11	32060	{}	\N	2020-10-12 15:55:14.159477
65074	PWV	2020-10-12	35.34	35.47	35.3299	35.4533	13966	{}	\N	2020-10-12 15:55:14.164488
65075	QADB	2020-09-30	30.74	33.3	30.74	33.3	987	{}	\N	2020-10-12 15:55:17.002398
65076	QADB	2020-10-01	33.9962	33.9962	32	32.37	880	{}	\N	2020-10-12 15:55:17.011598
65077	QADB	2020-10-06	31.64	31.65	31.5	31.5	1423	{}	\N	2020-10-12 15:55:17.018975
65078	QADB	2020-10-07	31.5	31.5	31.5	31.5	1757	{}	\N	2020-10-12 15:55:17.025144
65079	QADB	2020-10-09	29.01	29.01	29.01	29.01	265	{}	\N	2020-10-12 15:55:17.030371
65080	QADB	2020-10-12	31.75	31.75	31.5	31.5	1180	{}	\N	2020-10-12 15:55:17.035613
65081	PRN	2020-09-28	75.99	76.6899	75.9694	76.35	11793	{}	\N	2020-10-12 15:55:19.003507
65082	PRN	2020-09-29	76.71	76.77	76.12	76.13	52809	{}	\N	2020-10-12 15:55:19.013799
65083	PRN	2020-09-30	76.26	76.9863	75.6448	76.31	4406	{}	\N	2020-10-12 15:55:19.020754
65084	PRN	2020-10-01	77.38	77.38	76.4979	76.7103	5420	{}	\N	2020-10-12 15:55:19.028444
65085	PRN	2020-10-02	75.86	77.18	75.78	76.635	4951	{}	\N	2020-10-12 15:55:19.034444
65086	PRN	2020-10-05	77.27	78.65	77.27	78.57	159491	{}	\N	2020-10-12 15:55:19.039777
65087	PRN	2020-10-06	78.63	80.0837	78.6099	78.6099	5231	{}	\N	2020-10-12 15:55:19.044469
65088	PRN	2020-10-07	79.79	80.5942	79.79	80.4701	16343	{}	\N	2020-10-12 15:55:19.048903
65089	PRN	2020-10-08	81.4599	81.4599	80.28	80.53	16012	{}	\N	2020-10-12 15:55:19.053878
65090	PRN	2020-10-09	81.42	81.99	81.42	81.8799	5067	{}	\N	2020-10-12 15:55:19.058743
65091	PRN	2020-10-12	83.44	83.44	81.41	81.5642	5303	{}	\N	2020-10-12 15:55:19.063394
65092	PRTS	2020-09-28	11.35	12.05	11.18	11.99	1575026	{}	\N	2020-10-12 15:55:20.733671
65093	PRTS	2020-09-29	12.1	12.27	11.24	11.375	1403980	{}	\N	2020-10-12 15:55:20.745315
65094	PRTS	2020-09-30	11.4	11.59	10.44	10.81	2428314	{}	\N	2020-10-12 15:55:20.75366
65095	PRTS	2020-10-01	10.94	11.11	10.31	10.6	1835206	{}	\N	2020-10-12 15:55:20.760488
65096	PRTS	2020-10-02	10.34	10.72	10.04	10.46	1103297	{}	\N	2020-10-12 15:55:20.766756
65097	PRTS	2020-10-05	10.56	11.45	10.56	11.2	1585385	{}	\N	2020-10-12 15:55:20.772676
65098	PRTS	2020-10-06	11.2	11.7	11.0141	11.1	914486	{}	\N	2020-10-12 15:55:20.779358
65099	PRTS	2020-10-07	11.21	11.67	11.2	11.38	720411	{}	\N	2020-10-12 15:55:20.784667
65100	PRTS	2020-10-08	11.58	11.59	10.98	11.11	556699	{}	\N	2020-10-12 15:55:20.790195
65101	PRTS	2020-10-09	11.17	11.285	10.78	10.91	650431	{}	\N	2020-10-12 15:55:20.79614
65102	PRTS	2020-10-12	11.07	12.78	11.0173	12.72	1275997	{}	\N	2020-10-12 15:55:20.801644
65103	PSC	2020-09-28	28.15	28.17	28.1007	28.1007	1635	{}	\N	2020-10-12 15:55:23.060564
65104	PSC	2020-09-29	27.84	27.84	27.84	27.84	669	{}	\N	2020-10-12 15:55:23.070235
65105	PSC	2020-09-30	28.28	28.29	28.0485	28.0485	1157	{}	\N	2020-10-12 15:55:23.078321
65106	PSC	2020-10-01	27.8219	28.2098	27.8219	28.2098	6101	{}	\N	2020-10-12 15:55:23.08586
65107	PSC	2020-10-02	28.59	28.59	28.59	28.59	712	{}	\N	2020-10-12 15:55:23.091373
65108	PSC	2020-10-05	29.07	29.14	29.07	29.14	2795	{}	\N	2020-10-12 15:55:23.095422
65109	PSC	2020-10-06	29.73	29.75	29.2746	29.2746	3378	{}	\N	2020-10-12 15:55:23.099667
65110	PSC	2020-10-07	29.58	29.8224	29.58	29.8224	1191	{}	\N	2020-10-12 15:55:23.104004
65111	PSC	2020-10-08	30.22	30.22	30.22	30.22	841	{}	\N	2020-10-12 15:55:23.108248
65112	PSC	2020-10-09	30.43	30.43	30.22	30.22	2720	{}	\N	2020-10-12 15:55:23.112504
65113	PSC	2020-10-12	30.14	30.19	30.14	30.19	621	{}	\N	2020-10-12 15:55:23.116654
65114	PSEC	2020-09-28	5.08	5.18	5.06	5.12	1928256	{}	\N	2020-10-12 15:55:24.227708
65115	PSEC	2020-09-29	5.11	5.11	5.01	5.02	1201863	{}	\N	2020-10-12 15:55:24.237791
65116	PSEC	2020-09-30	5.04	5.05	5.01	5.03	1443099	{}	\N	2020-10-12 15:55:24.245507
65117	PSEC	2020-10-01	5.04	5.09	5.03	5.08	897291	{}	\N	2020-10-12 15:55:24.251767
65118	PSEC	2020-10-02	5.02	5.13	5.01	5.1	1168882	{}	\N	2020-10-12 15:55:24.257555
65119	PSEC	2020-10-05	5.08	5.15	5.08	5.15	971759	{}	\N	2020-10-12 15:55:24.262881
65120	PSEC	2020-10-06	5.15	5.2	5.13	5.17	1658393	{}	\N	2020-10-12 15:55:24.267538
65121	PSEC	2020-10-07	5.17	5.2	5.16	5.17	910961	{}	\N	2020-10-12 15:55:24.272218
65122	PSEC	2020-10-08	5.18	5.2	5.1616	5.19	1100589	{}	\N	2020-10-12 15:55:24.277411
65123	PSEC	2020-10-09	5.19	5.2	5.16	5.17	795372	{}	\N	2020-10-12 15:55:24.282312
65124	PSEC	2020-10-12	5.16	5.17	5.14	5.145	443427	{}	\N	2020-10-12 15:55:24.286942
65125	PSMC	2020-09-28	12.9	12.91	12.87	12.91	5777	{}	\N	2020-10-12 15:55:26.72901
65126	PSMC	2020-09-29	12.921	12.921	12.9001	12.9001	761	{}	\N	2020-10-12 15:55:26.73852
65127	PSMC	2020-09-30	12.92	12.95	12.9148	12.9148	4943	{}	\N	2020-10-12 15:55:26.745562
65128	PSMC	2020-10-01	12.9373	12.96	12.9218	12.9218	3400	{}	\N	2020-10-12 15:55:26.752464
65129	PSMC	2020-10-02	12.78	12.9454	12.78	12.92	1370	{}	\N	2020-10-12 15:55:26.757966
65130	PSMC	2020-10-05	12.965	12.97	12.965	12.97	637	{}	\N	2020-10-12 15:55:26.763344
65131	PSMC	2020-10-06	13.0199	13.0199	12.945	12.945	4086	{}	\N	2020-10-12 15:55:26.768518
65132	PSMC	2020-10-07	12.9909	12.9909	12.9909	12.9909	101	{}	\N	2020-10-12 15:55:26.773654
65133	PSMC	2020-10-08	13.0302	13.0402	13.0302	13.0402	3525	{}	\N	2020-10-12 15:55:26.77888
65134	PSMC	2020-10-09	13.079	13.079	13.06	13.06	3889	{}	\N	2020-10-12 15:55:26.784164
65135	PSMC	2020-10-12	13.06	13.0999	13.0566	13.0999	2201	{}	\N	2020-10-12 15:55:26.78921
65136	PSTG	2020-09-28	15.12	15.345	15.03	15.22	3057756	{}	\N	2020-10-12 15:55:28.704066
65137	PSTG	2020-09-29	15.36	15.8499	15.31	15.68	7465976	{}	\N	2020-10-12 15:55:28.716716
65138	PSTG	2020-09-30	15.66	15.77	15.32	15.39	9446608	{}	\N	2020-10-12 15:55:28.723895
65139	PSTG	2020-10-01	15.75	15.85	15.525	15.79	6432108	{}	\N	2020-10-12 15:55:28.729999
65140	PSTG	2020-10-02	15.36	15.655	15.24	15.55	5169786	{}	\N	2020-10-12 15:55:28.735792
65141	PSTG	2020-10-05	15.74	16.405	15.66	16.25	16118290	{}	\N	2020-10-12 15:55:28.740867
65142	PSTG	2020-10-06	16.3	17.085	16.29	16.75	11204243	{}	\N	2020-10-12 15:55:28.74557
65143	PSTG	2020-10-07	17	17.4	16.97	17.24	11884402	{}	\N	2020-10-12 15:55:28.750716
65144	PSTG	2020-10-08	17.45	18.1	17.44	17.76	9934077	{}	\N	2020-10-12 15:55:28.756267
65145	PSTG	2020-10-09	17.9	18.05	17.505	17.57	7200499	{}	\N	2020-10-12 15:55:28.760609
65146	PSTG	2020-10-12	17.78	17.85	17.35	17.35	2724588	{}	\N	2020-10-12 15:55:28.765537
65147	PTGX	2020-09-28	20.2	20.28	19.33	20.04	202342	{}	\N	2020-10-12 15:55:32.868754
65148	PTGX	2020-09-29	20.11	21.34	20.11	20.25	346432	{}	\N	2020-10-12 15:55:32.879627
65149	PTGX	2020-09-30	20.33	20.6	19.39	19.55	388522	{}	\N	2020-10-12 15:55:32.887273
65150	PTGX	2020-10-01	19.87	21.31	19.65	21.03	479084	{}	\N	2020-10-12 15:55:32.894029
65151	PTGX	2020-10-02	20.38	21.31	19.86	20.51	180953	{}	\N	2020-10-12 15:55:32.899578
65152	PTGX	2020-10-05	20.4	21.25	20.4	21.09	351767	{}	\N	2020-10-12 15:55:32.905397
65153	PTGX	2020-10-06	21.26	21.9	20.89	21.03	588728	{}	\N	2020-10-12 15:55:32.911217
65154	PTGX	2020-10-07	21.2	21.7542	20.91	21.44	754077	{}	\N	2020-10-12 15:55:32.916539
65155	PTGX	2020-10-08	22.5	22.7965	20.9	21.1	505588	{}	\N	2020-10-12 15:55:32.921659
65156	PTGX	2020-10-09	21.33	21.9399	20.63	21.14	132449	{}	\N	2020-10-12 15:55:32.927314
65157	PTGX	2020-10-12	21.32	21.33	20.66	20.92	34559	{}	\N	2020-10-12 15:55:32.932555
65158	PTSI	2020-09-28	37.93	39.02	37.93	38.1	5991	{}	\N	2020-10-12 15:55:35.633141
65159	PTSI	2020-09-29	38.45	39.88	37.47	38.5	5851	{}	\N	2020-10-12 15:55:35.646007
65160	PTSI	2020-09-30	37.2096	37.92	37.2096	37.6	1993	{}	\N	2020-10-12 15:55:35.655572
65161	PTSI	2020-10-01	38.014	38.17	37.12	37.12	4965	{}	\N	2020-10-12 15:55:35.662455
65162	PTSI	2020-10-02	37.05	38.9	37.05	38.75	4303	{}	\N	2020-10-12 15:55:35.66912
65163	PTSI	2020-10-05	38.54	38.65	37.475	37.55	2861	{}	\N	2020-10-12 15:55:35.675688
65164	PTSI	2020-10-06	37.85	38.5	35.42	37.67	6921	{}	\N	2020-10-12 15:55:35.681839
65165	PTSI	2020-10-07	38.695	43.25	38.695	43.25	4331	{}	\N	2020-10-12 15:55:35.688136
65166	PTSI	2020-10-08	43	45	39.95	41.6	5444	{}	\N	2020-10-12 15:55:35.6945
65167	PTSI	2020-10-09	42	43.595	42	42	1773	{}	\N	2020-10-12 15:55:35.700615
65168	PTSI	2020-10-12	41.93	41.93	41.93	41.93	437	{}	\N	2020-10-12 15:55:35.706998
65169	PVBC	2020-09-28	7.69	7.89	7.69	7.74	22350	{}	\N	2020-10-12 15:55:37.513375
65170	PVBC	2020-09-29	7.7	7.8	7.65	7.76	16149	{}	\N	2020-10-12 15:55:37.526133
65171	PVBC	2020-09-30	7.805	7.85	7.75	7.79	12973	{}	\N	2020-10-12 15:55:37.533222
65172	PVBC	2020-10-01	7.82	7.99	7.79	7.96	20446	{}	\N	2020-10-12 15:55:37.539417
65173	PVBC	2020-10-02	7.9391	8.15	7.89	8.1	20554	{}	\N	2020-10-12 15:55:37.548234
65174	PVBC	2020-10-05	8.255	8.255	8.0463	8.14	19173	{}	\N	2020-10-12 15:55:37.554487
65175	PVBC	2020-10-06	8.04	8.18	7.91	7.92	37291	{}	\N	2020-10-12 15:55:37.559522
65176	PVBC	2020-10-07	8.09	8.2786	8.04	8.22	26687	{}	\N	2020-10-12 15:55:37.564435
65177	PVBC	2020-10-08	8.3	8.3	8.13	8.18	9712	{}	\N	2020-10-12 15:55:37.569147
65178	PVBC	2020-10-09	8.22	8.265	8.13	8.19	15051	{}	\N	2020-10-12 15:55:37.573952
65179	PVBC	2020-10-12	8.2399	8.32	8.19	8.3067	5520	{}	\N	2020-10-12 15:55:37.578871
65180	PWR	2020-09-28	52.49	53.1	51.92	52.89	955440	{}	\N	2020-10-12 15:55:40.993904
65181	PWR	2020-09-29	52.61	53.32	52.4501	52.74	1791311	{}	\N	2020-10-12 15:55:41.005222
65182	PWR	2020-09-30	52.7	54.12	52.39	52.86	3385761	{}	\N	2020-10-12 15:55:41.012271
65183	PWR	2020-10-01	53.42	53.97	52.6097	53.56	3512019	{}	\N	2020-10-12 15:55:41.018437
65184	PWR	2020-10-02	52.73	55.83	52.65	55.2	2909471	{}	\N	2020-10-12 15:55:41.024451
65185	PWR	2020-10-05	55.55	57.66	55.55	56.9	3435287	{}	\N	2020-10-12 15:55:41.029613
65186	PWR	2020-10-06	57.08	57.792	56.2407	56.74	3015738	{}	\N	2020-10-12 15:55:41.034834
65187	PWR	2020-10-07	57.33	58.89	57.3	58.36	1884726	{}	\N	2020-10-12 15:55:41.03981
65188	PWR	2020-10-08	58.56	59.25	57.81	59	2401776	{}	\N	2020-10-12 15:55:41.044642
65189	PWR	2020-10-09	59.2	59.36	58.18	59.23	2426471	{}	\N	2020-10-12 15:55:41.050008
65190	PWR	2020-10-12	60	61.485	59.77	61.085	1162583	{}	\N	2020-10-12 15:55:41.055566
65191	PXJ	2020-09-28	2.02	2.06	2.02	2.03	20002	{}	\N	2020-10-12 15:55:43.827548
65192	PXJ	2020-09-29	2	2.005	1.95	1.9842	15198	{}	\N	2020-10-12 15:55:43.840564
65193	PXJ	2020-09-30	2	2.04	1.9745	1.98	6317	{}	\N	2020-10-12 15:55:43.847529
65194	PXJ	2020-10-01	1.96	1.965	1.915	1.92	16567	{}	\N	2020-10-12 15:55:43.853711
65195	PXJ	2020-10-02	1.89	1.9711	1.875	1.96	2842	{}	\N	2020-10-12 15:55:43.858986
65196	PXJ	2020-10-05	1.99	2.03	1.99	2.03	31164	{}	\N	2020-10-12 15:55:43.864678
65197	PXJ	2020-10-06	2.05	2.07	1.99	1.99	21501	{}	\N	2020-10-12 15:55:43.870199
65198	PXJ	2020-10-07	1.9944	2.01	1.9677	2.01	17126	{}	\N	2020-10-12 15:55:43.87539
65199	PXJ	2020-10-08	2.02	2.1	2.02	2.1	10086	{}	\N	2020-10-12 15:55:43.881074
65200	PXJ	2020-10-09	2.09	2.09	2.06	2.07	33942	{}	\N	2020-10-12 15:55:43.886286
65201	PXJ	2020-10-12	2.03	2.035	2.03	2.035	5588	{}	\N	2020-10-12 15:55:43.891439
65202	PXS	2020-09-28	0.77	0.825	0.760001	0.795	41285	{}	\N	2020-10-12 15:55:52.1899
65203	PXS	2020-09-29	0.81	0.85	0.7976	0.816	37359	{}	\N	2020-10-12 15:55:52.200211
65204	PXS	2020-09-30	0.80271	0.85	0.7901	0.7901	29411	{}	\N	2020-10-12 15:55:52.207004
65205	PXS	2020-10-01	0.7902	0.95	0.7901	0.85	559032	{}	\N	2020-10-12 15:55:52.214924
65206	PXS	2020-10-02	0.84	0.855637	0.772	0.8175	135491	{}	\N	2020-10-12 15:55:52.221129
65207	PXS	2020-10-05	0.83	0.895	0.8201	0.89	281971	{}	\N	2020-10-12 15:55:52.227111
65208	PXS	2020-10-06	0.85	0.97	0.83	0.9005	306455	{}	\N	2020-10-12 15:55:52.232884
65209	PXS	2020-10-07	0.9099	0.9549	0.87	0.9098	169047	{}	\N	2020-10-12 15:55:52.238969
65210	PXS	2020-10-08	0.87	0.9399	0.820601	0.9399	400695	{}	\N	2020-10-12 15:55:52.24424
65211	PXS	2020-10-09	0.904	0.98	0.87	0.9425	405591	{}	\N	2020-10-12 15:55:52.248498
65212	PXS	2020-10-12	0.98	1.2	0.951	1.035	912553	{}	\N	2020-10-12 15:55:52.253687
65213	PRTK	2020-09-28	5.85	5.89	5.44	5.57	568382	{}	\N	2020-10-12 15:55:54.188282
65214	PRTK	2020-09-29	5.54	5.68	5.435	5.49	368529	{}	\N	2020-10-12 15:55:54.19827
65215	PRTK	2020-09-30	5.5	5.57	5.32	5.41	424262	{}	\N	2020-10-12 15:55:54.205489
65216	PRTK	2020-10-01	5.44	5.5	5.25	5.4	412402	{}	\N	2020-10-12 15:55:54.211399
65217	PRTK	2020-10-02	5.3	5.44	5.22	5.29	326785	{}	\N	2020-10-12 15:55:54.21652
65218	PRTK	2020-10-05	5.33	5.63	5.33	5.61	326511	{}	\N	2020-10-12 15:55:54.221967
65219	PRTK	2020-10-06	5.6	5.78	5.53	5.63	258637	{}	\N	2020-10-12 15:55:54.226518
65220	PRTK	2020-10-07	5.62	5.77	5.61	5.7	248184	{}	\N	2020-10-12 15:55:54.230946
65221	PRTK	2020-10-08	5.8	6.13	5.76	6.07	316669	{}	\N	2020-10-12 15:55:54.235313
65222	PRTK	2020-10-09	6.08	6.14	5.71	5.74	229514	{}	\N	2020-10-12 15:55:54.240148
65223	PRTK	2020-10-12	5.71	5.76	5.62	5.75	117335	{}	\N	2020-10-12 15:55:54.244447
65224	PSF	2020-09-28	26.99	27	26.52	26.7	13561	{}	\N	2020-10-12 15:55:55.568017
65225	PSF	2020-09-29	26.885	26.8936	26.565	26.7	28247	{}	\N	2020-10-12 15:55:55.578869
65226	PSF	2020-09-30	26.7	26.85	26.55	26.55	118394	{}	\N	2020-10-12 15:55:55.585413
65227	PSF	2020-10-01	26.65	26.73	26.52	26.52	17740	{}	\N	2020-10-12 15:55:55.592
65228	PSF	2020-10-02	26.12	26.41	26.02	26.02	53183	{}	\N	2020-10-12 15:55:55.597499
65229	PSF	2020-10-05	26.05	26.275	26.02	26.23	26214	{}	\N	2020-10-12 15:55:55.602317
65230	PSF	2020-10-06	26.25	26.64	26.1344	26.61	43003	{}	\N	2020-10-12 15:55:55.606871
65231	PSF	2020-10-07	26.53	26.73	26.4	26.6	29460	{}	\N	2020-10-12 15:55:55.611634
65232	PSF	2020-10-08	26.56	26.56	26.35	26.44	41040	{}	\N	2020-10-12 15:55:55.616195
65233	PSF	2020-10-09	26.52	26.617	25.9	26.18	226610	{}	\N	2020-10-12 15:55:55.620547
65234	PSF	2020-10-12	26.08	26.1951	25.91	26.1951	36529	{}	\N	2020-10-12 15:55:55.625273
65235	PSMT	2020-09-28	66.84	68.2	66.25	67.94	121901	{}	\N	2020-10-12 15:55:57.757055
65236	PSMT	2020-09-29	68.07	68.07	66.35	67.18	82612	{}	\N	2020-10-12 15:55:57.767161
65237	PSMT	2020-09-30	67.46	67.85	66.33	66.45	143919	{}	\N	2020-10-12 15:55:57.773414
65238	PSMT	2020-10-01	66.42	66.9395	64.945	66.54	97764	{}	\N	2020-10-12 15:55:57.779686
65239	PSMT	2020-10-02	65.58	67.21	65.58	66.81	84349	{}	\N	2020-10-12 15:55:57.785258
65240	PSMT	2020-10-05	67.31	68.32	66.94	68.12	119644	{}	\N	2020-10-12 15:55:57.790025
65241	PSMT	2020-10-06	68.64	70.23	68.1	68.705	147714	{}	\N	2020-10-12 15:55:57.794744
65242	PSMT	2020-10-07	68.7	69.38	67.405	68.08	116899	{}	\N	2020-10-12 15:55:57.799506
65243	PSMT	2020-10-08	68.5	69.87	68.05	69.7	91532	{}	\N	2020-10-12 15:55:57.804351
65244	PSMT	2020-10-09	69.95	70.41	68.59	69.29	103648	{}	\N	2020-10-12 15:55:57.809574
65245	PSMT	2020-10-12	69.63	70.08	68.87	69.82	25602	{}	\N	2020-10-12 15:55:57.814991
65246	PTEN	2020-09-28	2.96	3.105	2.96	3	3785033	{}	\N	2020-10-12 15:55:59.44297
65247	PTEN	2020-09-29	3	3.03	2.77	2.91	2052094	{}	\N	2020-10-12 15:55:59.456135
65248	PTEN	2020-09-30	2.94	3.05	2.83	2.85	2725040	{}	\N	2020-10-12 15:55:59.465586
65249	PTEN	2020-10-01	2.83	2.88	2.53	2.57	6488240	{}	\N	2020-10-12 15:55:59.472128
65250	PTEN	2020-10-02	2.42	2.73	2.41	2.57	5740762	{}	\N	2020-10-12 15:55:59.478587
65251	PTEN	2020-10-05	2.64	2.71	2.57	2.62	4019644	{}	\N	2020-10-12 15:55:59.484393
65252	PTEN	2020-10-06	2.66	2.87	2.66	2.69	4848148	{}	\N	2020-10-12 15:55:59.489815
65253	PTEN	2020-10-07	2.72	2.92	2.71	2.835	4084165	{}	\N	2020-10-12 15:55:59.495529
65254	PTEN	2020-10-08	2.93	3.07	2.87	3.04	3049391	{}	\N	2020-10-12 15:55:59.501373
65255	PTEN	2020-10-09	3.09	3.1168	2.88	2.96	2323684	{}	\N	2020-10-12 15:55:59.506716
65256	PTEN	2020-10-12	2.93	2.94	2.845	2.915	710375	{}	\N	2020-10-12 15:55:59.512296
65257	PTNQ	2020-09-28	47.72	47.8809	47.35	47.76	34303	{}	\N	2020-10-12 15:56:00.730613
65258	PTNQ	2020-09-29	47.77	47.94	47.65	47.65	33718	{}	\N	2020-10-12 15:56:00.74177
65259	PTNQ	2020-09-30	47.84	48.5815	47.825	48.08	64932	{}	\N	2020-10-12 15:56:00.750758
65260	PTNQ	2020-10-01	48.67	48.88	48.465	48.72	100223	{}	\N	2020-10-12 15:56:00.758421
65261	PTNQ	2020-10-02	47.74	48.33	47.425	47.43	78351	{}	\N	2020-10-12 15:56:00.763829
65262	PTNQ	2020-10-05	47.86	48.492	47.85	48.37	40662	{}	\N	2020-10-12 15:56:00.768123
65263	PTNQ	2020-10-06	48.39	48.55	47.49	47.6	74238	{}	\N	2020-10-12 15:56:00.772749
65264	PTNQ	2020-10-07	48.0097	48.5399	47.955	48.47	43870	{}	\N	2020-10-12 15:56:00.777584
65265	PTNQ	2020-10-08	48.73	48.748	48.55	48.6486	23656	{}	\N	2020-10-12 15:56:00.782122
65266	PTNQ	2020-10-09	48.89	49.3695	48.89	49.3695	29175	{}	\N	2020-10-12 15:56:00.786851
65267	PTNQ	2020-10-12	50.16	50.7321	50.16	50.65	11466	{}	\N	2020-10-12 15:56:00.791481
65268	PUMP	2020-09-28	4.06	4.245	3.99	4.08	1350574	{}	\N	2020-10-12 15:56:01.809724
65269	PUMP	2020-09-29	4.01	4.5	3.835	4.31	7790962	{}	\N	2020-10-12 15:56:01.821509
65270	PUMP	2020-09-30	4.31	4.41	4.01	4.06	2819527	{}	\N	2020-10-12 15:56:01.828638
65271	PUMP	2020-10-01	3.95	4.39	3.92	4.21	6064447	{}	\N	2020-10-12 15:56:01.834924
65272	PUMP	2020-10-02	4.02	4.15	3.77	4.05	2810448	{}	\N	2020-10-12 15:56:01.840582
65273	PUMP	2020-10-05	4.11	4.255	4.0831	4.17	1883852	{}	\N	2020-10-12 15:56:01.845802
65274	PUMP	2020-10-06	4.2	4.33	4.115	4.16	3661571	{}	\N	2020-10-12 15:56:01.850808
65275	PUMP	2020-10-07	4.27	4.35	4.11	4.26	3981813	{}	\N	2020-10-12 15:56:01.857344
65276	PUMP	2020-10-08	4.37	4.48	4.185	4.47	2580289	{}	\N	2020-10-12 15:56:01.862703
65277	PUMP	2020-10-09	4.53	4.54	4.365	4.5	1858179	{}	\N	2020-10-12 15:56:01.868166
65278	PUMP	2020-10-12	4.5	4.55	4.36	4.52	763389	{}	\N	2020-10-12 15:56:01.873947
65279	PWOD	2020-09-28	20.2	20.4	20.04	20.04	9242	{}	\N	2020-10-12 15:56:02.803133
65280	PWOD	2020-09-29	20.4155	20.4155	19.97	20.18	3403	{}	\N	2020-10-12 15:56:02.812312
65281	PWOD	2020-09-30	20.15	20.15	19.67	19.85	6570	{}	\N	2020-10-12 15:56:02.818587
65282	PWOD	2020-10-01	20.13	20.705	20	20.14	5687	{}	\N	2020-10-12 15:56:02.824676
65283	PWOD	2020-10-02	19.95	20.82	19.95	20.44	8975	{}	\N	2020-10-12 15:56:02.830477
65284	PWOD	2020-10-05	20.785	21.07	19.8794	21.07	5309	{}	\N	2020-10-12 15:56:02.83488
65285	PWOD	2020-10-06	21.36	21.88	20.97	20.97	3661	{}	\N	2020-10-12 15:56:02.839685
65286	PWOD	2020-10-07	20	21.63	20	21.63	7055	{}	\N	2020-10-12 15:56:02.844026
65287	PWOD	2020-10-08	21.71	21.98	21.5	21.73	4039	{}	\N	2020-10-12 15:56:02.848273
65288	PWOD	2020-10-09	20.8201	21.7	20.8201	21.69	9320	{}	\N	2020-10-12 15:56:02.853231
65289	PWOD	2020-10-12	21.42	21.42	21.34	21.4	905	{}	\N	2020-10-12 15:56:02.857887
65290	PYT	2020-09-30	23.12	23.12	23.12	23.12	2000	{}	\N	2020-10-12 15:56:05.759432
65291	PYT	2020-10-05	23.11	23.11	22.7	22.9	7550	{}	\N	2020-10-12 15:56:05.769447
65292	PYT	2020-10-06	22.65	22.71	22.65	22.71	600	{}	\N	2020-10-12 15:56:05.776321
65293	PYT	2020-10-07	22.71	22.71	22.71	22.71	5400	{}	\N	2020-10-12 15:56:05.782605
65294	PYT	2020-10-09	23.1	23.11	23.1	23.11	640	{}	\N	2020-10-12 15:56:05.789082
65295	QAT	2020-09-28	17.73	17.73	17.71	17.71	783	{}	\N	2020-10-12 15:56:07.007287
65296	QAT	2020-09-29	17.675	17.75	17.65	17.65	1598	{}	\N	2020-10-12 15:56:07.017862
65297	QAT	2020-09-30	17.89	17.9899	17.89	17.9	2408	{}	\N	2020-10-12 15:56:07.024617
65298	QAT	2020-10-01	17.945	17.945	17.8	17.8	3924	{}	\N	2020-10-12 15:56:07.030707
65299	QAT	2020-10-02	17.8	17.8	17.8	17.8	556	{}	\N	2020-10-12 15:56:07.035961
65300	QAT	2020-10-05	17.94	17.99	17.88	17.93	1951	{}	\N	2020-10-12 15:56:07.040625
65301	QAT	2020-10-06	18	18.17	18	18.12	1799	{}	\N	2020-10-12 15:56:07.044909
65302	QAT	2020-10-07	18	18.15	18	18.01	5820	{}	\N	2020-10-12 15:56:07.048929
65303	QAT	2020-10-08	18.1207	18.1207	18.02	18.04	14337	{}	\N	2020-10-12 15:56:07.053365
65304	QAT	2020-10-09	18.04	18.21	18.04	18.1767	2039	{}	\N	2020-10-12 15:56:07.057337
65305	QAT	2020-10-12	18.18	18.18	18.0958	18.0958	2312	{}	\N	2020-10-12 15:56:07.061339
65306	QDEF	2020-09-28	43.8642	44.115	43.8432	44	38043	{}	\N	2020-10-12 15:56:08.27647
65307	QDEF	2020-09-29	44.0327	44.0327	43.63	43.77	12591	{}	\N	2020-10-12 15:56:08.287825
65308	QDEF	2020-09-30	44.16	44.5074	43.875	44.21	22282	{}	\N	2020-10-12 15:56:08.296112
65309	QDEF	2020-10-01	44.38	44.51	44.1	44.36	12555	{}	\N	2020-10-12 15:56:08.30273
65310	QDEF	2020-10-02	43.69	44.295	43.69	44.29	15349	{}	\N	2020-10-12 15:56:08.308535
65311	QDEF	2020-10-05	44.36	44.8072	44.36	44.8013	11247	{}	\N	2020-10-12 15:56:08.313148
65312	QDEF	2020-10-06	44.84	45.13	44.36	44.37	8958	{}	\N	2020-10-12 15:56:08.317074
65313	QDEF	2020-10-07	44.75	45.03	44.75	44.99	30736	{}	\N	2020-10-12 15:56:08.320817
65314	QDEF	2020-10-08	45.17	45.33	45.155	45.3	13223	{}	\N	2020-10-12 15:56:08.325511
65315	QDEF	2020-10-09	45.5	45.6842	45.39	45.51	25902	{}	\N	2020-10-12 15:56:08.330141
65316	QDEF	2020-10-12	45.8	46.1699	45.8	46.15	13988	{}	\N	2020-10-12 15:56:08.33453
65317	PSCH	2020-09-28	129.5	130.8499	129.5	130.401	6055	{}	\N	2020-10-12 15:56:10.008531
65318	PSCH	2020-09-29	129.29	131.26	129.29	131.14	4017	{}	\N	2020-10-12 15:56:10.019151
65319	PSCH	2020-09-30	130.67	132.88	130.67	132.1	9747	{}	\N	2020-10-12 15:56:10.026705
65320	PSCH	2020-10-01	132.37	133.075	131.28	132.05	6734	{}	\N	2020-10-12 15:56:10.033441
65321	PSCH	2020-10-02	129.95	131.48	129.52	130.94	10047	{}	\N	2020-10-12 15:56:10.039567
65322	PSCH	2020-10-05	131.89	135.2099	131.89	134.9	8316	{}	\N	2020-10-12 15:56:10.044597
65323	PSCH	2020-10-06	135.33	138.2927	135.33	135.39	13787	{}	\N	2020-10-12 15:56:10.049803
65324	PSCH	2020-10-07	136.8501	137.6	136.45	137.5399	24337	{}	\N	2020-10-12 15:56:10.054156
65325	PSCH	2020-10-08	137.21	138.36	137.1501	137.98	7760	{}	\N	2020-10-12 15:56:10.058766
65326	PSCH	2020-10-09	138.18	139.03	138.18	139.03	9153	{}	\N	2020-10-12 15:56:10.063411
65327	PSCH	2020-10-12	138.79	139.7099	138.48	139.6287	2765	{}	\N	2020-10-12 15:56:10.067731
65328	PSLV	2020-09-28	8.32	8.39	8.22	8.39	2227999	{}	\N	2020-10-12 15:56:11.416919
65329	PSLV	2020-09-29	8.47	8.65	8.45	8.62	2573190	{}	\N	2020-10-12 15:56:11.425819
65330	PSLV	2020-09-30	8.5	8.5221	8.23	8.27	2335190	{}	\N	2020-10-12 15:56:11.432399
65331	PSLV	2020-10-01	8.43	8.51	8.3	8.4	3400377	{}	\N	2020-10-12 15:56:11.437869
65332	PSLV	2020-10-02	8.47	8.54	8.35	8.4	1714354	{}	\N	2020-10-12 15:56:11.442656
65333	PSLV	2020-10-05	8.45	8.6201	8.45	8.6	1215732	{}	\N	2020-10-12 15:56:11.447375
65334	PSLV	2020-10-06	8.61	8.61	8.205	8.22	2550286	{}	\N	2020-10-12 15:56:11.451935
65335	PSLV	2020-10-07	8.34	8.42	8.29	8.39	1449896	{}	\N	2020-10-12 15:56:11.456648
65336	PSLV	2020-10-08	8.5	8.5399	8.3501	8.42	1126773	{}	\N	2020-10-12 15:56:11.461429
65337	PSLV	2020-10-09	8.68	8.8	8.61	8.8	4452439	{}	\N	2020-10-12 15:56:11.4663
65338	PSLV	2020-10-12	8.82	8.82	8.69	8.795	945399	{}	\N	2020-10-12 15:56:11.470693
65339	PSXP	2020-09-28	22.29	22.74	21.85	22.61	330100	{}	\N	2020-10-12 15:56:12.543736
65340	PSXP	2020-09-29	22.61	23.47	22.14	23.35	1120081	{}	\N	2020-10-12 15:56:12.554451
65341	PSXP	2020-09-30	23.74	23.7797	22.86	23.04	2428155	{}	\N	2020-10-12 15:56:12.561917
65342	PSXP	2020-10-01	23.02	23.03	22.26	22.87	836902	{}	\N	2020-10-12 15:56:12.568378
65343	PSXP	2020-10-02	22	22.97	22	22.81	1100944	{}	\N	2020-10-12 15:56:12.573912
65344	PSXP	2020-10-05	23.11	23.89	22.6	23.76	758647	{}	\N	2020-10-12 15:56:12.578656
65345	PSXP	2020-10-06	24.01	24.3	22.85	23.12	1289481	{}	\N	2020-10-12 15:56:12.583323
65346	PSXP	2020-10-07	23.23	23.45	22.82	22.93	924194	{}	\N	2020-10-12 15:56:12.587819
65347	PSXP	2020-10-08	23.11	24.47	23.01	24.43	875915	{}	\N	2020-10-12 15:56:12.592655
65348	PSXP	2020-10-09	24.65	24.95	24	24.36	655218	{}	\N	2020-10-12 15:56:12.596809
65349	PSXP	2020-10-12	24.59	24.76	24.1	24.36	185296	{}	\N	2020-10-12 15:56:12.601516
65350	PWC	2020-09-28	93.392	93.392	93.392	93.392	329	{}	\N	2020-10-12 15:56:15.114853
65351	PWC	2020-09-29	93.13	93.3476	93.13	93.3476	1398	{}	\N	2020-10-12 15:56:15.1264
65352	PWC	2020-09-30	94.09	94.135	93.17	93.195	1321	{}	\N	2020-10-12 15:56:15.136927
65353	PWC	2020-10-01	93.88	94.07	93.7997	93.7997	616	{}	\N	2020-10-12 15:56:15.144888
65354	PWC	2020-10-02	93.0786	93.5069	93.0786	93.5069	957	{}	\N	2020-10-12 15:56:15.151551
65355	PWC	2020-10-05	94.2868	94.821	94.2868	94.821	464	{}	\N	2020-10-12 15:56:15.157831
65356	PWC	2020-10-06	95.0242	95.0242	93.9179	93.9179	878	{}	\N	2020-10-12 15:56:15.163841
65357	PWC	2020-10-07	94.83	95.4022	94.82	95.4022	1669	{}	\N	2020-10-12 15:56:15.169493
65358	PWC	2020-10-08	95.7	96.5177	95.7	96.5177	1067	{}	\N	2020-10-12 15:56:15.175005
65359	PWC	2020-10-09	97.5	97.51	97.4479	97.4479	452	{}	\N	2020-10-12 15:56:15.180797
65360	PWC	2020-10-12	97.88	97.89	97.88	97.89	420	{}	\N	2020-10-12 15:56:15.186495
65361	PXF	2020-09-28	35.52	35.71	35.52	35.64	37212	{}	\N	2020-10-12 15:56:16.723583
65362	PXF	2020-09-29	35.58	35.6325	35.36	35.5	88621	{}	\N	2020-10-12 15:56:16.734352
65363	PXF	2020-09-30	35.46	35.69	35.31	35.44	33813	{}	\N	2020-10-12 15:56:16.740941
65364	PXF	2020-10-01	35.53	35.54	35.35	35.51	50388	{}	\N	2020-10-12 15:56:16.747244
65365	PXF	2020-10-02	35.16	35.61	35.16	35.515	425697	{}	\N	2020-10-12 15:56:16.752916
65366	PXF	2020-10-05	35.94	36.22	35.94	36.21	79302	{}	\N	2020-10-12 15:56:16.758431
65367	PXF	2020-10-06	36.39	36.42	35.88	35.94	42709	{}	\N	2020-10-12 15:56:16.763482
65368	PXF	2020-10-07	36.26	36.4145	36.25	36.38	44513	{}	\N	2020-10-12 15:56:16.768343
65369	PXF	2020-10-08	36.54	36.6499	36.49	36.64	42145	{}	\N	2020-10-12 15:56:16.773416
65370	PXF	2020-10-09	36.79	36.828	36.69	36.77	27855	{}	\N	2020-10-12 15:56:16.778843
65371	PXF	2020-10-12	36.81	36.86	36.78	36.845	4003	{}	\N	2020-10-12 15:56:16.784129
65372	PZC	2020-09-28	10.8	10.8	10.57	10.6	26583	{}	\N	2020-10-12 15:56:18.128097
65373	PZC	2020-09-29	10.6	10.69	10.6	10.63	17786	{}	\N	2020-10-12 15:56:18.139686
65374	PZC	2020-09-30	10.53	10.75	10.5101	10.64	59444	{}	\N	2020-10-12 15:56:18.146619
65375	PZC	2020-10-01	10.64	10.76	10.64	10.67	40183	{}	\N	2020-10-12 15:56:18.153503
65376	PZC	2020-10-02	10.74	10.8	10.705	10.79	35246	{}	\N	2020-10-12 15:56:18.159267
65377	PZC	2020-10-05	10.8	10.82	10.74	10.74	36874	{}	\N	2020-10-12 15:56:18.164403
65378	PZC	2020-10-06	10.74	10.82	10.74	10.76	31680	{}	\N	2020-10-12 15:56:18.169495
65379	PZC	2020-10-07	10.77	10.81	10.755	10.81	19426	{}	\N	2020-10-12 15:56:18.17441
65380	PZC	2020-10-08	10.81	10.81	10.72	10.75	21344	{}	\N	2020-10-12 15:56:18.179858
65381	PZC	2020-10-09	10.66	10.8	10.66	10.73	20132	{}	\N	2020-10-12 15:56:18.186179
65382	PZC	2020-10-12	10.81	10.81	10.7	10.71	7170	{}	\N	2020-10-12 15:56:18.191015
65383	QAI	2020-09-28	30.93	30.93	30.82	30.85	53313	{}	\N	2020-10-12 15:56:19.595881
65384	QAI	2020-09-29	30.87	30.9934	30.83	30.86	678484	{}	\N	2020-10-12 15:56:19.60751
65385	QAI	2020-09-30	30.96	31	30.93	30.99	48538	{}	\N	2020-10-12 15:56:19.614475
65386	QAI	2020-10-01	31	31.09	31	31.08	75783	{}	\N	2020-10-12 15:56:19.620407
65387	QAI	2020-10-02	30.9	31.03	30.9	30.93	96703	{}	\N	2020-10-12 15:56:19.626719
65388	QAI	2020-10-05	30.99	31.0996	30.99	31.07	67177	{}	\N	2020-10-12 15:56:19.632538
65389	QAI	2020-10-06	31.04	31.1575	31.02	31.03	113657	{}	\N	2020-10-12 15:56:19.637868
65390	QAI	2020-10-07	31.02	31.18	31.02	31.15	47113	{}	\N	2020-10-12 15:56:19.642865
65391	QAI	2020-10-08	31.18	31.2	31.14	31.15	46884	{}	\N	2020-10-12 15:56:19.648804
65392	QAI	2020-10-09	31.25	31.2619	31.17	31.22	163796	{}	\N	2020-10-12 15:56:19.65473
65393	QAI	2020-10-12	31.29	31.395	31.2839	31.305	8687	{}	\N	2020-10-12 15:56:19.660232
65394	PRTY	2020-09-28	2.88	3.1	2.8311	2.97	5530329	{}	\N	2020-10-12 15:56:20.95115
65395	PRTY	2020-09-29	2.88	2.93	2.66	2.68	12942556	{}	\N	2020-10-12 15:56:20.961868
65396	PRTY	2020-09-30	2.74	2.79	2.55	2.6	12091183	{}	\N	2020-10-12 15:56:20.969772
65397	PRTY	2020-10-01	2.66	2.74	2.55	2.68	9413470	{}	\N	2020-10-12 15:56:20.976908
65398	PRTY	2020-10-02	2.53	2.7	2.46	2.7	9094463	{}	\N	2020-10-12 15:56:20.983671
65399	PRTY	2020-10-05	2.71	2.71	2.55	2.58	6533012	{}	\N	2020-10-12 15:56:20.989423
65400	PRTY	2020-10-06	2.62	2.7	2.51	2.52	8199962	{}	\N	2020-10-12 15:56:20.99482
65401	PRTY	2020-10-07	2.58	2.625	2.53	2.58	4665398	{}	\N	2020-10-12 15:56:21.000544
65402	PRTY	2020-10-08	2.61	2.61	2.51	2.54	6171847	{}	\N	2020-10-12 15:56:21.005576
65403	PRTY	2020-10-09	2.55	2.67	2.53	2.55	9494578	{}	\N	2020-10-12 15:56:21.010582
65404	PRTY	2020-10-12	2.6	2.825	2.555	2.7	6011004	{}	\N	2020-10-12 15:56:21.015834
65405	PSET	2020-09-29	42.405	42.405	42.3519	42.3519	163	{}	\N	2020-10-12 15:56:22.218893
65406	PSET	2020-09-30	42.56	42.58	42.56	42.58	427	{}	\N	2020-10-12 15:56:22.226697
65407	PSET	2020-10-05	42.98	43.1132	42.98	43.1132	565	{}	\N	2020-10-12 15:56:22.233098
65408	PSET	2020-10-06	43.26	43.27	42.8154	42.8154	531	{}	\N	2020-10-12 15:56:22.238789
65409	PSET	2020-10-08	43.6998	43.6998	43.6998	43.6998	563	{}	\N	2020-10-12 15:56:22.244445
65410	PSMM	2020-09-28	13.56	13.56	13.55	13.55	714	{}	\N	2020-10-12 15:56:23.220966
65411	PSMM	2020-09-29	13.538	13.538	13.5257	13.5257	337	{}	\N	2020-10-12 15:56:23.235754
65412	PSMM	2020-09-30	13.5486	13.5486	13.5486	13.5486	361	{}	\N	2020-10-12 15:56:23.244748
65413	PSMM	2020-10-01	13.531	13.57	13.531	13.57	1752	{}	\N	2020-10-12 15:56:23.25144
65414	PSMM	2020-10-02	13.53	13.5841	13.5	13.5841	2962	{}	\N	2020-10-12 15:56:23.257718
65415	PSMM	2020-10-05	13.6608	13.6608	13.6608	13.6608	101	{}	\N	2020-10-12 15:56:23.262629
65416	PSMM	2020-10-06	13.708	13.708	13.6259	13.6259	355	{}	\N	2020-10-12 15:56:23.26793
65417	PSMM	2020-10-07	13.6999	13.6999	13.6999	13.6999	30	{}	\N	2020-10-12 15:56:23.273124
65418	PSMM	2020-10-08	13.77	13.799	13.77	13.77	4887	{}	\N	2020-10-12 15:56:23.278399
65419	PSMM	2020-10-09	13.81	13.8399	13.78	13.8399	19041	{}	\N	2020-10-12 15:56:23.284049
65420	PSMM	2020-10-12	13.7801	13.8259	13.7801	13.8259	666	{}	\N	2020-10-12 15:56:23.289426
65421	PSX	2020-09-28	53.94	54.64	53.43	53.98	2212680	{}	\N	2020-10-12 15:56:24.228164
65422	PSX	2020-09-29	53.76	53.82	51.76	52.19	4889960	{}	\N	2020-10-12 15:56:24.242466
65423	PSX	2020-09-30	52.47	53.05	51.59	51.84	5155379	{}	\N	2020-10-12 15:56:24.251424
65424	PSX	2020-10-01	51.19	51.6	49.7457	50.28	7862735	{}	\N	2020-10-12 15:56:24.258238
65425	PSX	2020-10-02	48.96	51.7643	48.8205	51.06	6590145	{}	\N	2020-10-12 15:56:24.264238
65426	PSX	2020-10-05	51.84	53.125	51.72	52.55	5335080	{}	\N	2020-10-12 15:56:24.269885
65427	PSX	2020-10-06	53.49	53.49	50.935	51.06	5784630	{}	\N	2020-10-12 15:56:24.275
65428	PSX	2020-10-07	51.45	52.18	50.65	51.93	5185548	{}	\N	2020-10-12 15:56:24.280014
65429	PSX	2020-10-08	52.12	53.66	52.06	53.64	4789207	{}	\N	2020-10-12 15:56:24.285353
65430	PSX	2020-10-09	54.04	54.5	52.55	53.04	6313244	{}	\N	2020-10-12 15:56:24.29041
65431	PSX	2020-10-12	52.66	53.03	51.925	52.65	1572934	{}	\N	2020-10-12 15:56:24.295449
65432	PTMC	2020-09-28	26.95	27.34	26.95	27.26	35980	{}	\N	2020-10-12 15:56:26.110075
65433	PTMC	2020-09-29	27.22	27.34	26.98	27.11	31983	{}	\N	2020-10-12 15:56:26.120311
65434	PTMC	2020-09-30	27.32	27.55	27.0849	27.3	56932	{}	\N	2020-10-12 15:56:26.128084
65435	PTMC	2020-10-01	27.45	27.6249	27.31	27.6	127435	{}	\N	2020-10-12 15:56:26.134901
65436	PTMC	2020-10-02	27.21	27.9952	27.12	27.87	43731	{}	\N	2020-10-12 15:56:26.140718
65437	PTMC	2020-10-05	28.19	28.55	28.19	28.55	53533	{}	\N	2020-10-12 15:56:26.146282
65438	PTMC	2020-10-06	28.69	29.06	28.35	28.43	55308	{}	\N	2020-10-12 15:56:26.15161
65439	PTMC	2020-10-07	28.61	28.98	28.61	28.92	38789	{}	\N	2020-10-12 15:56:26.156694
65440	PTMC	2020-10-08	29.22	29.24	29.03	29.24	53494	{}	\N	2020-10-12 15:56:26.161472
65441	PTMC	2020-10-09	29.38	29.5999	29.251	29.3	68246	{}	\N	2020-10-12 15:56:26.166214
65442	PTMC	2020-10-12	29.45	29.5309	29.37	29.5	11183	{}	\N	2020-10-12 15:56:26.170737
65443	PUK	2020-09-28	29.61	29.78	29.33	29.4	256506	{}	\N	2020-10-12 15:56:27.159365
65444	PUK	2020-09-29	29.16	29.39	28.93	29.16	404715	{}	\N	2020-10-12 15:56:27.170471
65445	PUK	2020-09-30	28.71	28.95	28.38	28.59	444306	{}	\N	2020-10-12 15:56:27.177743
65446	PUK	2020-10-01	28.68	28.79	28.29	28.65	467968	{}	\N	2020-10-12 15:56:27.184346
65447	PUK	2020-10-02	28.52	29.5	28.5	29.24	369485	{}	\N	2020-10-12 15:56:27.19028
65448	PUK	2020-10-05	29.46	29.659	29.38	29.63	358509	{}	\N	2020-10-12 15:56:27.195289
65449	PUK	2020-10-06	29.82	30.095	29.31	29.36	360426	{}	\N	2020-10-12 15:56:27.200367
65450	PUK	2020-10-07	29.56	29.72	29.4534	29.69	280406	{}	\N	2020-10-12 15:56:27.205659
65451	PUK	2020-10-08	29.98	30.22	29.8	30.1	216389	{}	\N	2020-10-12 15:56:27.21119
65452	PUK	2020-10-09	30.45	30.46	30.11	30.35	236430	{}	\N	2020-10-12 15:56:27.216326
65453	PUK	2020-10-12	30.28	30.53	30.28	30.44	91075	{}	\N	2020-10-12 15:56:27.221519
65454	PW	2020-09-28	19.7899	20.6172	19.5	19.92	17832	{}	\N	2020-10-12 15:56:28.87384
65455	PW	2020-09-29	20.13	20.74	19.55	20.6	13702	{}	\N	2020-10-12 15:56:28.885382
65456	PW	2020-09-30	20.81	20.81	19.51	19.82	13666	{}	\N	2020-10-12 15:56:28.8925
65457	PW	2020-10-01	20.4999	20.6	19.19	19.5	26812	{}	\N	2020-10-12 15:56:28.898269
65458	PW	2020-10-02	19.22	20.55	18.27	20	45195	{}	\N	2020-10-12 15:56:28.903558
65459	PW	2020-10-05	20.49	20.49	19.6301	19.76	10690	{}	\N	2020-10-12 15:56:28.908054
65460	PW	2020-10-06	20.32	20.49	18.7	19.01	45728	{}	\N	2020-10-12 15:56:28.911706
65461	PW	2020-10-07	18.51	19.29	18.51	18.69	27802	{}	\N	2020-10-12 15:56:28.915873
65462	PW	2020-10-08	18.58	19.225	18.58	19.12	10978	{}	\N	2020-10-12 15:56:28.919874
65463	PW	2020-10-09	19.6091	19.6798	19.06	19.15	12029	{}	\N	2020-10-12 15:56:28.924111
65464	PW	2020-10-12	19.0707	19.255	18.45	18.45	7377	{}	\N	2020-10-12 15:56:28.928322
65465	PXD	2020-09-28	87.77	89.2025	86.73	88.31	1769866	{}	\N	2020-10-12 15:56:39.3225
65466	PXD	2020-09-29	87.56	87.73	83.9	86.4	2755982	{}	\N	2020-10-12 15:56:39.335986
65467	PXD	2020-09-30	87.09	87.45	85.43	85.99	3281354	{}	\N	2020-10-12 15:56:39.34675
65468	PXD	2020-10-01	84.83	86.22	83.99	84.57	2991301	{}	\N	2020-10-12 15:56:39.352566
65469	PXD	2020-10-02	82.06	87.555	81.86	86.54	3652208	{}	\N	2020-10-12 15:56:39.358918
65470	PXD	2020-10-05	88	89.39	86.42	89.32	1921245	{}	\N	2020-10-12 15:56:39.363883
65471	PXD	2020-10-06	90.98	92.25	87.75	88.03	2823762	{}	\N	2020-10-12 15:56:39.368629
65472	PXD	2020-10-07	88.01	89.68	87.22	88.94	2076329	{}	\N	2020-10-12 15:56:39.37277
65473	PXD	2020-10-08	89.71	90.17	88.32	89.48	2345391	{}	\N	2020-10-12 15:56:39.378629
65474	PXD	2020-10-09	90.07	90.55	87.63	88.07	2156230	{}	\N	2020-10-12 15:56:39.383695
65475	PXD	2020-10-12	88.29	88.61	86.56	88.2	532549	{}	\N	2020-10-12 15:56:39.388056
65476	PXQ	2020-09-28	67.82	67.82	67.77	67.77	562	{}	\N	2020-10-12 15:56:40.817539
65477	PXQ	2020-09-29	68.326	68.326	68.16	68.16	268	{}	\N	2020-10-12 15:56:40.826469
65478	PXQ	2020-09-30	68.3747	68.63	68	68	762	{}	\N	2020-10-12 15:56:40.832478
65479	PXQ	2020-10-01	68.95	69.32	68.95	69.32	908	{}	\N	2020-10-12 15:56:40.838198
65480	PXQ	2020-10-02	68.92	68.92	68.4383	68.4383	9562	{}	\N	2020-10-12 15:56:40.843864
65481	PXQ	2020-10-05	69.64	70.29	69.475	70.29	4504	{}	\N	2020-10-12 15:56:40.848655
65482	PXQ	2020-10-06	70.39	70.635	69.6301	69.6301	1655	{}	\N	2020-10-12 15:56:40.853397
65483	PXQ	2020-10-07	71.1401	71.1401	71.1401	71.1401	43	{}	\N	2020-10-12 15:56:40.858245
65484	PXQ	2020-10-08	71.3101	71.5	71.3101	71.4594	1730	{}	\N	2020-10-12 15:56:40.863658
65485	PXQ	2020-10-09	72.78	73.02	72.6593	72.6593	2010	{}	\N	2020-10-12 15:56:40.868829
65486	PYS	2020-09-28	16.8	16.9764	16.8	16.9764	940	{}	\N	2020-10-12 15:56:42.162248
65487	PYS	2020-09-29	17.655	17.655	17.655	17.655	572	{}	\N	2020-10-12 15:56:42.17418
65488	PYS	2020-09-30	17.27	17.27	17.27	17.27	612	{}	\N	2020-10-12 15:56:42.182484
65489	PYS	2020-10-01	17	17.01	17	17.01	1788	{}	\N	2020-10-12 15:56:42.189063
65490	PYS	2020-10-02	18.52	18.52	18.52	18.52	644	{}	\N	2020-10-12 15:56:42.194707
65491	PYS	2020-10-05	18.25	18.25	18.21	18.21	1700	{}	\N	2020-10-12 15:56:42.199847
65492	PYS	2020-10-06	17.735	17.735	17.735	17.735	510	{}	\N	2020-10-12 15:56:42.204364
65493	PYS	2020-10-07	17.51	17.51	17.4901	17.4901	1338	{}	\N	2020-10-12 15:56:42.208583
65494	PYS	2020-10-08	17.37	18.0365	17.2	18.0365	7174	{}	\N	2020-10-12 15:56:42.213061
65495	PYS	2020-10-09	18.16	18.5	18.0401	18.0401	5353	{}	\N	2020-10-12 15:56:42.217579
65496	PYS	2020-10-12	18.3	18.4258	17.77	17.77	7714	{}	\N	2020-10-12 15:56:42.222027
65497	PZZA	2020-09-28	82.33	84.3	82.03	83.42	423126	{}	\N	2020-10-12 15:56:43.618098
65498	PZZA	2020-09-29	84.28	86.37	83.65	85.77	630187	{}	\N	2020-10-12 15:56:43.629721
65499	PZZA	2020-09-30	86.28	86.3	81.65	82.28	897034	{}	\N	2020-10-12 15:56:43.638769
65500	PZZA	2020-10-01	83.26	85.25	82.24	83.33	746957	{}	\N	2020-10-12 15:56:43.644594
65501	PZZA	2020-10-02	81.5	83.92	81	83.27	535018	{}	\N	2020-10-12 15:56:43.650647
65502	PZZA	2020-10-05	83.37	84.42	81.47	83.14	511387	{}	\N	2020-10-12 15:56:43.655987
65503	PZZA	2020-10-06	83.41	84.8	82.2	82.36	446356	{}	\N	2020-10-12 15:56:43.660799
65504	PZZA	2020-10-07	82.81	84.19	81.74	82.15	654812	{}	\N	2020-10-12 15:56:43.66557
65505	PZZA	2020-10-08	81.45	81.85	79.71	80.85	1138113	{}	\N	2020-10-12 15:56:43.670198
65506	PZZA	2020-10-09	81.43	82.13	79.22	79.89	647550	{}	\N	2020-10-12 15:56:43.675509
65507	PZZA	2020-10-12	80.14	80.96	79.67	80.73	178732	{}	\N	2020-10-12 15:56:43.680017
65508	QGEN	2020-09-28	52.45	52.51	51.44	51.51	609519	{}	\N	2020-10-12 15:56:44.832187
65509	QGEN	2020-09-29	51.48	52.44	51.48	52.24	1021740	{}	\N	2020-10-12 15:56:44.8425
65510	QGEN	2020-09-30	51.61	52.33	51.42	52.26	2034858	{}	\N	2020-10-12 15:56:44.848748
65511	QGEN	2020-10-01	52.53	53.24	52.21	52.7	2041530	{}	\N	2020-10-12 15:56:44.856083
65512	QGEN	2020-10-02	51.63	52.49	51.6	52.32	803833	{}	\N	2020-10-12 15:56:44.863115
65513	QGEN	2020-10-05	52.67	53.37	52.61	53.2	1915685	{}	\N	2020-10-12 15:56:44.868898
65514	QGEN	2020-10-06	54.52	54.61	53.73	54.26	3462670	{}	\N	2020-10-12 15:56:44.87504
65515	QGEN	2020-10-07	53.99	54.75	53.88	54.38	2193266	{}	\N	2020-10-12 15:56:44.880662
65516	QGEN	2020-10-08	53.99	54.52	53.95	54.07	1541023	{}	\N	2020-10-12 15:56:44.885955
65517	QGEN	2020-10-09	54.25	54.85	54.25	54.74	1129553	{}	\N	2020-10-12 15:56:44.892095
65518	QGEN	2020-10-12	54.93	55.27	54.85	55	810170	{}	\N	2020-10-12 15:56:44.897763
65519	PSB	2020-09-28	119.62	123.27	119.62	123.17	111384	{}	\N	2020-10-12 15:56:46.332566
65520	PSB	2020-09-29	122.75	123.23	121.71	122.79	177815	{}	\N	2020-10-12 15:56:46.345956
65521	PSB	2020-09-30	123.06	124.22	120.83	122.39	212597	{}	\N	2020-10-12 15:56:46.353209
65522	PSB	2020-10-01	122.38	125.29	122.18	125.21	254238	{}	\N	2020-10-12 15:56:46.358685
65523	PSB	2020-10-02	124.05	127.12	121.89	126.84	232466	{}	\N	2020-10-12 15:56:46.364009
65524	PSB	2020-10-05	128.39	128.43	125.8403	127.1	124671	{}	\N	2020-10-12 15:56:46.36817
65525	PSB	2020-10-06	127.75	129.5	125.96	127.82	220227	{}	\N	2020-10-12 15:56:46.37239
65526	PSB	2020-10-07	128.46	129.23	126.73	127.1	142252	{}	\N	2020-10-12 15:56:46.376903
65527	PSB	2020-10-08	128.1	129.315	127	128.69	170381	{}	\N	2020-10-12 15:56:46.381525
65528	PSB	2020-10-09	129.76	129.76	127.56	128.48	122133	{}	\N	2020-10-12 15:56:46.385855
65529	PSB	2020-10-12	128.77	129.58	127.6	129.1	44827	{}	\N	2020-10-12 15:56:46.390518
65530	PSCT	2020-09-28	85.13	86.6574	85.13	86.38	6971	{}	\N	2020-10-12 15:56:47.385488
65531	PSCT	2020-09-29	86.28	86.98	86.28	86.98	5766	{}	\N	2020-10-12 15:56:47.396877
65532	PSCT	2020-09-30	87	87.71	86.0962	86.1167	7816	{}	\N	2020-10-12 15:56:47.403738
65533	PSCT	2020-10-01	86.91	88.03	86.35	88	4241	{}	\N	2020-10-12 15:56:47.410174
65534	PSCT	2020-10-02	85.77	87.56	85.77	87.525	6354	{}	\N	2020-10-12 15:56:47.415902
65535	PSCT	2020-10-05	87.95	89.25	87.95	89.1928	5777	{}	\N	2020-10-12 15:56:47.420816
65536	PSCT	2020-10-06	89.65	91.8	89.65	90.055	23434	{}	\N	2020-10-12 15:56:47.425695
65537	PSCT	2020-10-07	90.45	90.9403	90.25	90.87	2855	{}	\N	2020-10-12 15:56:47.430728
65538	PSCT	2020-10-08	91.955	92.29	91.831	92.29	5251	{}	\N	2020-10-12 15:56:47.436307
65539	PSCT	2020-10-09	93.08	94.15	93.08	94.0011	5282	{}	\N	2020-10-12 15:56:47.442313
65540	PSCT	2020-10-12	94.56	95.15	94.3	94.935	3478	{}	\N	2020-10-12 15:56:47.447571
65541	PSK	2020-09-28	43.66	43.6878	43.6	43.67	141616	{}	\N	2020-10-12 15:56:48.521944
65542	PSK	2020-09-29	43.67	43.67	43.51	43.55	116969	{}	\N	2020-10-12 15:56:48.535514
65543	PSK	2020-09-30	43.55	43.64	43.4	43.47	124065	{}	\N	2020-10-12 15:56:48.54516
65544	PSK	2020-10-01	43.29	43.56	43.24	43.56	113762	{}	\N	2020-10-12 15:56:48.552526
65545	PSK	2020-10-02	43.27	43.61	43.25	43.56	105256	{}	\N	2020-10-12 15:56:48.559277
65546	PSK	2020-10-05	43.72	43.74	43.62	43.73	85355	{}	\N	2020-10-12 15:56:48.564746
65547	PSK	2020-10-06	43.77	43.83	43.63	43.75	139040	{}	\N	2020-10-12 15:56:48.569937
65548	PSK	2020-10-07	43.9	43.9	43.7601	43.85	88284	{}	\N	2020-10-12 15:56:48.575976
65549	PSK	2020-10-08	43.94	43.94	43.84	43.88	81328	{}	\N	2020-10-12 15:56:48.581331
65550	PSK	2020-10-09	43.85	43.8999	43.71	43.84	84167	{}	\N	2020-10-12 15:56:48.58658
65551	PSK	2020-10-12	43.9	43.9	43.75	43.821	39319	{}	\N	2020-10-12 15:56:48.591384
65552	PST	2020-09-28	15.083	15.09	15.08	15.09	9748	{}	\N	2020-10-12 15:56:49.557186
65553	PST	2020-09-29	15.06	15.07	15.04	15.06	17273	{}	\N	2020-10-12 15:56:49.570642
65554	PST	2020-09-30	15.08	15.16	15.08	15.14	35099	{}	\N	2020-10-12 15:56:49.579472
65555	PST	2020-10-01	15.18	15.18	15.09	15.12	21350	{}	\N	2020-10-12 15:56:49.58604
65556	PST	2020-10-02	15.09	15.175	15.09	15.165	13849	{}	\N	2020-10-12 15:56:49.591839
65557	PST	2020-10-05	15.25	15.34	15.25	15.34	7216	{}	\N	2020-10-12 15:56:49.596853
65558	PST	2020-10-06	15.345	15.345	15.25	15.27	16994	{}	\N	2020-10-12 15:56:49.601977
65559	PST	2020-10-07	15.325	15.38	15.305	15.35	3788	{}	\N	2020-10-12 15:56:49.607464
65560	PST	2020-10-08	15.32	15.33	15.3	15.31	3269	{}	\N	2020-10-12 15:56:49.612716
65561	PST	2020-10-09	15.325	15.36	15.2863	15.32	4484	{}	\N	2020-10-12 15:56:49.617796
65562	PST	2020-10-12	15.31	15.3164	15.3	15.3164	3181	{}	\N	2020-10-12 15:56:49.622922
65563	PTEU	2020-09-28	22.4201	22.47	22.34	22.39	10573	{}	\N	2020-10-12 15:56:50.577312
65564	PTEU	2020-09-29	22.42	22.53	22.21	22.52	11295	{}	\N	2020-10-12 15:56:50.589927
65565	PTEU	2020-09-30	22.5	22.5	22.28	22.4	40051	{}	\N	2020-10-12 15:56:50.597072
65566	PTEU	2020-10-01	22.35	22.49	22.35	22.46	17871	{}	\N	2020-10-12 15:56:50.603009
65567	PTEU	2020-10-02	22.42	22.4799	22.42	22.46	8383	{}	\N	2020-10-12 15:56:50.608808
65568	PTEU	2020-10-05	22.475	22.476	22.4501	22.46	17610	{}	\N	2020-10-12 15:56:50.613461
65569	PTEU	2020-10-06	22.4642	22.47	22.46	22.46	16863	{}	\N	2020-10-12 15:56:50.617941
65570	PTEU	2020-10-07	22.46	22.535	22.31	22.43	27438	{}	\N	2020-10-12 15:56:50.622706
65571	PTEU	2020-10-08	22.455	22.4699	22.33	22.4235	5117	{}	\N	2020-10-12 15:56:50.627175
65572	PTEU	2020-10-09	22.5797	22.734	22.4601	22.734	14716	{}	\N	2020-10-12 15:56:50.63173
65573	PTEU	2020-10-12	22.875	22.875	22.59	22.7	10978	{}	\N	2020-10-12 15:56:50.636117
65574	PTN	2020-09-28	0.47	0.4945	0.461	0.4727	2292871	{}	\N	2020-10-12 15:56:51.570633
65575	PTN	2020-09-29	0.48	0.5019	0.46	0.4816	1778122	{}	\N	2020-10-12 15:56:51.582775
65576	PTN	2020-09-30	0.501	0.5245	0.47	0.4728	2309880	{}	\N	2020-10-12 15:56:51.591725
65577	PTN	2020-10-01	0.4866	0.4866	0.459	0.4713	1219246	{}	\N	2020-10-12 15:56:51.598704
65578	PTN	2020-10-02	0.4672	0.4672	0.455	0.4646	1286119	{}	\N	2020-10-12 15:56:51.604591
65579	PTN	2020-10-05	0.4611	0.48	0.456	0.4683	878711	{}	\N	2020-10-12 15:56:51.610122
65580	PTN	2020-10-06	0.4683	0.473299	0.4571	0.4571	851350	{}	\N	2020-10-12 15:56:51.615676
65581	PTN	2020-10-07	0.4626	0.469	0.4577	0.4584	1005306	{}	\N	2020-10-12 15:56:51.620738
65582	PTN	2020-10-08	0.46	0.4746	0.4584	0.4606	780343	{}	\N	2020-10-12 15:56:51.626072
65583	PTN	2020-10-09	0.47	0.47	0.455	0.461	924039	{}	\N	2020-10-12 15:56:51.63107
65584	PTN	2020-10-12	0.4661	0.4661	0.4521	0.4562	745945	{}	\N	2020-10-12 15:56:51.636095
65585	PULM	2020-09-28	1.1	1.14	1.07	1.13	315631	{}	\N	2020-10-12 15:56:52.544003
65586	PULM	2020-09-29	1.13	1.2	1.09	1.19	478828	{}	\N	2020-10-12 15:56:52.556137
65587	PULM	2020-09-30	1.19	1.22	1.18	1.19	397186	{}	\N	2020-10-12 15:56:52.565427
65588	PULM	2020-10-01	1.15	1.21	1.12	1.14	200308	{}	\N	2020-10-12 15:56:52.571601
65589	PULM	2020-10-02	1.12	1.17	1.1	1.15	162429	{}	\N	2020-10-12 15:56:52.577258
65590	PULM	2020-10-05	1.16	1.21	1.15	1.18	369328	{}	\N	2020-10-12 15:56:52.582135
65591	PULM	2020-10-06	1.19	1.19	1.11	1.13	304817	{}	\N	2020-10-12 15:56:52.587294
65592	PULM	2020-10-07	1.13	1.15	1.11	1.13	257385	{}	\N	2020-10-12 15:56:52.592438
65593	PULM	2020-10-08	1.13	1.1564	1.11	1.15	211572	{}	\N	2020-10-12 15:56:52.597264
65594	PULM	2020-10-09	1.14	1.2	1.13	1.19	334941	{}	\N	2020-10-12 15:56:52.602251
65595	PULM	2020-10-12	1.2	1.2	1.17	1.18	51001	{}	\N	2020-10-12 15:56:52.607126
65596	PVH	2020-09-28	63.02	63.34	61.69	61.89	1491186	{}	\N	2020-10-12 15:56:54.007794
65597	PVH	2020-09-29	61.53	61.69	58.59	58.69	2230058	{}	\N	2020-10-12 15:56:54.021922
65598	PVH	2020-09-30	58.98	61.15	58.98	59.64	3305005	{}	\N	2020-10-12 15:56:54.031296
65599	PVH	2020-10-01	60.1	61.41	59.56	60.59	2397146	{}	\N	2020-10-12 15:56:54.03915
65600	PVH	2020-10-02	58.57	62.24	58.3376	62.12	2136582	{}	\N	2020-10-12 15:56:54.045387
65601	PVH	2020-10-05	62.89	64.34	62.2	63.79	1871339	{}	\N	2020-10-12 15:56:54.050821
65602	PVH	2020-10-06	64.68	65.32	61.35	61.64	1725745	{}	\N	2020-10-12 15:56:54.056288
65603	PVH	2020-10-07	62.5	64.83	62.12	64.44	2383333	{}	\N	2020-10-12 15:56:54.061554
65604	PVH	2020-10-08	64.87	67.655	64.51	67.61	1921850	{}	\N	2020-10-12 15:56:54.067084
65605	PVH	2020-10-09	68.1	68.4806	65.54	66.13	2188005	{}	\N	2020-10-12 15:56:54.072369
65606	PVH	2020-10-12	66.5	66.5	65.2	65.72	552575	{}	\N	2020-10-12 15:56:54.077498
65607	PXE	2020-09-28	7.78	7.84	7.71	7.83	2815	{}	\N	2020-10-12 15:56:55.564352
65608	PXE	2020-09-29	7.78	7.78	7.48	7.48	4589	{}	\N	2020-10-12 15:56:55.578504
65609	PXE	2020-09-30	7.65	7.694	7.53	7.53	9871	{}	\N	2020-10-12 15:56:55.587178
65610	PXE	2020-10-01	7.43	7.44	7.245	7.26	17532	{}	\N	2020-10-12 15:56:55.593271
65611	PXE	2020-10-02	7.05	7.4441	7.05	7.41	4068	{}	\N	2020-10-12 15:56:55.59942
65612	PXE	2020-10-05	7.6073	7.77	7.55	7.77	11429	{}	\N	2020-10-12 15:56:55.604821
65613	PXE	2020-10-06	7.9756	7.9756	7.58	7.58	19715	{}	\N	2020-10-12 15:56:55.609976
65614	PXE	2020-10-07	7.65	7.8207	7.65	7.8207	6519	{}	\N	2020-10-12 15:56:55.615082
65615	PXE	2020-10-08	7.87	8.14	7.8588	8.14	11562	{}	\N	2020-10-12 15:56:55.620627
65616	PXE	2020-10-09	8.15	8.15	8	8.04	10786	{}	\N	2020-10-12 15:56:55.625957
65617	PXE	2020-10-12	8.03	8.03	7.91	7.91	1501	{}	\N	2020-10-12 15:56:55.631079
65618	PXLW	2020-09-28	2.09	2.11	2.045	2.05	191112	{}	\N	2020-10-12 15:56:56.551675
65619	PXLW	2020-09-29	2.06	2.125	2.04	2.09	264234	{}	\N	2020-10-12 15:56:56.563551
65620	PXLW	2020-09-30	2.09	2.1	2.03	2.05	141955	{}	\N	2020-10-12 15:56:56.572788
65621	PXLW	2020-10-01	2.07	2.11	2.03	2.09	183067	{}	\N	2020-10-12 15:56:56.578057
65622	PXLW	2020-10-02	2.03	2.06	2.01	2.01	187429	{}	\N	2020-10-12 15:56:56.583858
65623	PXLW	2020-10-05	2.02	2.1999	2.02	2.1	520207	{}	\N	2020-10-12 15:56:56.589439
65624	PXLW	2020-10-06	2.1	2.25	2.09	2.11	389582	{}	\N	2020-10-12 15:56:56.593809
65625	PXLW	2020-10-07	2.15	2.23	2.127	2.2	314544	{}	\N	2020-10-12 15:56:56.598078
65626	PXLW	2020-10-08	2.18	2.26	2.18	2.19	297432	{}	\N	2020-10-12 15:56:56.60266
65627	PXLW	2020-10-09	2.24	2.29	2.2	2.29	274039	{}	\N	2020-10-12 15:56:56.607516
65628	PXLW	2020-10-12	2.33	2.35	2.29	2.31	94452	{}	\N	2020-10-12 15:56:56.61173
65629	PY	2020-09-28	28.865	29.065	28.865	29.065	312	{}	\N	2020-10-12 15:56:57.557623
65630	PY	2020-09-29	28.615	28.6578	28.6058	28.6578	217	{}	\N	2020-10-12 15:56:57.568043
65631	PSCM	2020-09-29	39.7391	39.7391	39.6035	39.6035	1419	{}	\N	2020-10-12 15:56:58.726472
65632	PSCM	2020-09-30	39.39	39.415	39.15	39.415	684	{}	\N	2020-10-12 15:56:58.734794
65633	PSCM	2020-10-01	39.35	39.53	39.06	39.06	1414	{}	\N	2020-10-12 15:56:58.740925
65634	PSCM	2020-10-02	39.6534	40.5491	39.58	40.5491	571	{}	\N	2020-10-12 15:56:58.747865
65635	PSCM	2020-10-08	42.995	42.995	42.995	42.995	234	{}	\N	2020-10-12 15:56:58.753017
65636	PSCM	2020-10-09	43.53	43.57	43.53	43.56	1391	{}	\N	2020-10-12 15:56:58.757749
65637	PSI	2020-09-28	76.03	76.82	75.79	76.79	13413	{}	\N	2020-10-12 15:56:59.964991
65638	PSI	2020-09-29	76.57	77.9	76.57	77.48	21807	{}	\N	2020-10-12 15:56:59.977218
65639	PSI	2020-09-30	77.49	78.2635	77.3401	77.42	13972	{}	\N	2020-10-12 15:56:59.985417
65640	PSI	2020-10-01	78.77	79.71	78.44	79.51	15747	{}	\N	2020-10-12 15:56:59.991798
65641	PSI	2020-10-02	77.01	78.405	77.01	77.08	18678	{}	\N	2020-10-12 15:56:59.997863
65642	PSI	2020-10-05	77.89	79.63	77.89	79.63	21567	{}	\N	2020-10-12 15:57:00.003014
65643	PSI	2020-10-06	79.54	81.439	79.1801	79.39	23629	{}	\N	2020-10-12 15:57:00.008582
65644	PSI	2020-10-07	80.56	81.085	80.56	80.88	14761	{}	\N	2020-10-12 15:57:00.014286
65645	PSI	2020-10-08	81.6	82.2249	81.45	82.2249	14069	{}	\N	2020-10-12 15:57:00.019503
65646	PSI	2020-10-09	83.1	84.295	83.1	84.18	26812	{}	\N	2020-10-12 15:57:00.025
65647	PSI	2020-10-12	85	85.469	84.96	84.96	7466	{}	\N	2020-10-12 15:57:00.0305
65648	PSP	2020-09-28	10.86	10.96	10.86	10.94	250179	{}	\N	2020-10-12 15:57:00.919046
65649	PSP	2020-09-29	10.91	10.94	10.85	10.86	38325	{}	\N	2020-10-12 15:57:00.931884
65650	PSP	2020-09-30	10.88	10.9599	10.855	10.87	69083	{}	\N	2020-10-12 15:57:00.940928
65651	PSP	2020-10-01	10.94	11.04	10.92	11.04	146524	{}	\N	2020-10-12 15:57:00.946733
65652	PSP	2020-10-02	10.86	11.109	10.84	11.02	780295	{}	\N	2020-10-12 15:57:00.952103
65653	PSP	2020-10-05	11.14	11.25	11.14	11.23	369849	{}	\N	2020-10-12 15:57:00.957415
65654	PSP	2020-10-06	11.24	11.2599	11.07	11.11	96131	{}	\N	2020-10-12 15:57:00.962485
65655	PSP	2020-10-07	11.24	11.24	11.1816	11.2	27016	{}	\N	2020-10-12 15:57:00.96798
65656	PSP	2020-10-08	11.255	11.29	11.25	11.29	212754	{}	\N	2020-10-12 15:57:00.973186
65657	PSP	2020-10-09	11.37	11.39	11.33	11.38	26006	{}	\N	2020-10-12 15:57:00.978518
65658	PSP	2020-10-12	11.39	11.4293	11.37	11.4001	44366	{}	\N	2020-10-12 15:57:00.983865
65659	PTH	2020-09-28	135.5	135.5	132.62	133.88	18690	{}	\N	2020-10-12 15:57:01.906894
65660	PTH	2020-09-29	134.64	135.9644	133.955	135.21	19632	{}	\N	2020-10-12 15:57:01.918745
65661	PTH	2020-09-30	135.22	137.72	135.1999	135.92	21549	{}	\N	2020-10-12 15:57:01.926092
65662	PTH	2020-10-01	136.86	137.6332	135.8576	136.99	55224	{}	\N	2020-10-12 15:57:01.932738
65663	PTH	2020-10-02	134.81	136.4603	134.555	135.23	17815	{}	\N	2020-10-12 15:57:01.938414
65664	PTH	2020-10-05	137.73	141.48	137.73	141.47	125160	{}	\N	2020-10-12 15:57:01.943351
65665	PTH	2020-10-06	141.76	143.79	140.8136	141.59	34195	{}	\N	2020-10-12 15:57:01.948079
65666	PTH	2020-10-07	142.4	145.3799	142.4	144.4717	40672	{}	\N	2020-10-12 15:57:01.95281
65667	PTH	2020-10-08	146.38	146.38	144.64	145.29	27328	{}	\N	2020-10-12 15:57:01.958007
65668	PTH	2020-10-09	146.44	147.895	146.13	147.72	16869	{}	\N	2020-10-12 15:57:01.962833
65669	PTH	2020-10-12	149.58	149.58	147.6	148.349	13715	{}	\N	2020-10-12 15:57:01.967782
65670	PUI	2020-09-28	29.23	29.5101	29.23	29.3559	5038	{}	\N	2020-10-12 15:57:02.958606
65671	PUI	2020-09-29	29.24	29.395	29.06	29.2	9239	{}	\N	2020-10-12 15:57:02.967015
65672	PUI	2020-09-30	29.39	29.571	29.361	29.48	49487	{}	\N	2020-10-12 15:57:02.972626
65673	PUI	2020-10-01	29.47	29.65	29.41	29.61	47605	{}	\N	2020-10-12 15:57:02.978652
65674	PUI	2020-10-02	29.36	30.1005	29.36	30.04	7139	{}	\N	2020-10-12 15:57:02.985645
65675	PUI	2020-10-05	30.16	30.3948	29.9914	30.36	6039	{}	\N	2020-10-12 15:57:02.992236
65676	PUI	2020-10-06	30.43	30.7563	30.43	30.63	8511	{}	\N	2020-10-12 15:57:02.997649
65677	PUI	2020-10-07	30.621	30.7141	30.45	30.7	28152	{}	\N	2020-10-12 15:57:03.003475
65678	PUI	2020-10-08	30.84	31.2533	30.84	31.2533	3866	{}	\N	2020-10-12 15:57:03.009392
65679	PUI	2020-10-09	31.42	31.42	31.13	31.2735	1504	{}	\N	2020-10-12 15:57:03.015483
65680	PUI	2020-10-12	31.24	31.5535	31.24	31.5535	27953	{}	\N	2020-10-12 15:57:03.021324
65681	PWS	2020-09-28	25.21	25.307	25.16	25.307	7959	{}	\N	2020-10-12 15:57:04.680963
65682	PWS	2020-09-29	25.41	25.41	25.28	25.31	5068	{}	\N	2020-10-12 15:57:04.693789
65683	PWS	2020-09-30	25.4	25.66	25.26	25.26	14762	{}	\N	2020-10-12 15:57:04.701326
65684	PWS	2020-10-01	25.6	25.6499	25.525	25.621	4172	{}	\N	2020-10-12 15:57:04.708009
65685	PWS	2020-10-02	25.45	25.515	25.29	25.37	3596	{}	\N	2020-10-12 15:57:04.714619
65686	PWS	2020-10-05	25.43	25.9452	25.43	25.9452	5018	{}	\N	2020-10-12 15:57:04.720331
65687	PWS	2020-10-06	25.93	25.93	25.5947	25.5947	1222	{}	\N	2020-10-12 15:57:04.726639
65688	PWS	2020-10-07	25.941	26.2151	25.94	26.2151	3790	{}	\N	2020-10-12 15:57:04.732451
65689	PWS	2020-10-08	26.37	26.37	26.32	26.3289	1140	{}	\N	2020-10-12 15:57:04.738079
65690	PWS	2020-10-09	26.603	26.6684	26.521	26.6684	10822	{}	\N	2020-10-12 15:57:04.744163
65691	PWS	2020-10-12	26.91	27.045	26.901	27.045	1708	{}	\N	2020-10-12 15:57:04.749933
65692	PZG	2020-09-28	1.18	1.18	1.13	1.15	108628	{}	\N	2020-10-12 15:57:06.559903
65693	PZG	2020-09-29	1.15	1.175	1.15	1.16	76914	{}	\N	2020-10-12 15:57:06.571327
65694	PZG	2020-09-30	1.15	1.19	1.14	1.14	53085	{}	\N	2020-10-12 15:57:06.579038
65695	PZG	2020-10-01	1.15	1.208	1.13	1.1463	131562	{}	\N	2020-10-12 15:57:06.586789
65696	PZG	2020-10-02	1.18	1.18	1.14	1.15	86725	{}	\N	2020-10-12 15:57:06.592761
65697	PZG	2020-10-05	1.15	1.19	1.14	1.18	162651	{}	\N	2020-10-12 15:57:06.598118
65698	PZG	2020-10-06	1.17	1.25	1.1601	1.2	197476	{}	\N	2020-10-12 15:57:06.604201
65699	PZG	2020-10-07	1.22	1.22	1.17	1.22	69000	{}	\N	2020-10-12 15:57:06.609105
65700	PZG	2020-10-08	1.2	1.24	1.19	1.21	51539	{}	\N	2020-10-12 15:57:06.614825
65701	PZG	2020-10-09	1.21	1.28	1.21	1.26	209664	{}	\N	2020-10-12 15:57:06.620375
65702	PZG	2020-10-12	1.28	1.28	1.26	1.2774	48217	{}	\N	2020-10-12 15:57:06.625327
65703	PSJ	2020-09-28	122.84	123.8276	122.557	123.29	12856	{}	\N	2020-10-12 15:57:07.631821
65704	PSJ	2020-09-29	123.7	124.24	123.15	123.8	14543	{}	\N	2020-10-12 15:57:07.643222
65705	PSJ	2020-09-30	123.65	125.19	123.35	123.72	11007	{}	\N	2020-10-12 15:57:07.651876
65706	PSJ	2020-10-01	125.22	126.855	125.105	126.3013	11772	{}	\N	2020-10-12 15:57:07.658392
65707	PSJ	2020-10-02	123.12	126.105	123.12	123.87	9635	{}	\N	2020-10-12 15:57:07.664373
65708	PSJ	2020-10-05	125.05	126.4899	125.05	126.48	15240	{}	\N	2020-10-12 15:57:07.669851
65709	PSJ	2020-10-06	126.58	127.95	125.2	125.43	34434	{}	\N	2020-10-12 15:57:07.675161
65710	PSJ	2020-10-07	126.5472	127.7428	126.01	127.51	30623	{}	\N	2020-10-12 15:57:07.680691
65711	PSJ	2020-10-08	128.92	128.92	127.3662	127.67	11072	{}	\N	2020-10-12 15:57:07.686264
65712	PSJ	2020-10-09	128.33	129.74	128.33	129.69	22409	{}	\N	2020-10-12 15:57:07.691549
65713	PSJ	2020-10-12	131	131.564	130.63	131.41	5375	{}	\N	2020-10-12 15:57:07.697016
65714	PSR	2020-09-28	80.83	81.585	80.83	81.285	3715	{}	\N	2020-10-12 15:57:08.672111
65715	PSR	2020-09-29	81.0962	81.0962	80.14	80.8012	5090	{}	\N	2020-10-12 15:57:08.682761
65716	PSR	2020-09-30	81.1	81.3935	80.4719	80.5	27244	{}	\N	2020-10-12 15:57:08.690577
65717	PSR	2020-10-01	81.35	82.5101	81.099	82.5101	5261	{}	\N	2020-10-12 15:57:08.697491
65718	PSR	2020-10-02	81.33	84.1697	81.33	84.1697	2743	{}	\N	2020-10-12 15:57:08.703664
65719	PSR	2020-10-05	83.11	84.4999	83.11	84.4999	4358	{}	\N	2020-10-12 15:57:08.709208
65720	PSR	2020-10-06	84.77	85.1171	83.7	84.18	8495	{}	\N	2020-10-12 15:57:08.714591
65721	PSR	2020-10-07	84.61	84.61	83.8	84.42	2854	{}	\N	2020-10-12 15:57:08.719989
65722	PSR	2020-10-08	84.9	85.3265	84.9	85.3265	4114	{}	\N	2020-10-12 15:57:08.725681
65723	PSR	2020-10-09	86.12	86.12	84.75	84.8	5345	{}	\N	2020-10-12 15:57:08.731381
65724	PSR	2020-10-12	84.8971	85.294	84.8971	85.294	922	{}	\N	2020-10-12 15:57:08.736847
65725	PTCT	2020-09-28	46.89	48.04	46.54	47.69	448085	{}	\N	2020-10-12 15:57:10.077978
65726	PTCT	2020-09-29	47.78	48.28	47.24	47.27	243253	{}	\N	2020-10-12 15:57:10.090747
65727	PTCT	2020-09-30	47.19	47.88	46.26	46.75	251617	{}	\N	2020-10-12 15:57:10.100048
65728	PTCT	2020-10-01	47.425	49.14	47.05	48.93	396304	{}	\N	2020-10-12 15:57:10.107888
65729	PTCT	2020-10-02	48.08	49.66	47.1	47.96	332155	{}	\N	2020-10-12 15:57:10.113472
65730	PTCT	2020-10-05	48.68	50	48.68	49.97	557665	{}	\N	2020-10-12 15:57:10.11915
65731	PTCT	2020-10-06	49.95	50.67	49.35	49.64	504791	{}	\N	2020-10-12 15:57:10.124721
65732	PTCT	2020-10-07	52.61	54.13	51.85	53.61	1087346	{}	\N	2020-10-12 15:57:10.129914
65733	PTCT	2020-10-08	53.89	54.055	52.54	53.08	273862	{}	\N	2020-10-12 15:57:10.135194
65734	PTCT	2020-10-09	53.28	53.82	52.82	53.81	240222	{}	\N	2020-10-12 15:57:10.140898
65735	PTCT	2020-10-12	54	54.82	53.06	54.5	78065	{}	\N	2020-10-12 15:57:10.146023
65736	PVG	2020-09-28	12.71	12.81	12.405	12.56	1663915	{}	\N	2020-10-12 15:57:11.742421
65737	PVG	2020-09-29	12.64	12.85	12.54	12.83	2501632	{}	\N	2020-10-12 15:57:11.753524
65738	PVG	2020-09-30	12.82	12.93	12.685	12.84	1557330	{}	\N	2020-10-12 15:57:11.762243
65739	PVG	2020-10-01	13	13.05	12.815	12.9	1486403	{}	\N	2020-10-12 15:57:11.768892
65740	PVG	2020-10-02	12.82	13.04	12.77	12.82	1318579	{}	\N	2020-10-12 15:57:11.774051
65741	PVG	2020-10-05	12.86	13.13	12.82	12.95	1219005	{}	\N	2020-10-12 15:57:11.779136
65742	PVG	2020-10-06	12.98	13	12.27	12.28	2479833	{}	\N	2020-10-12 15:57:11.784539
65743	PVG	2020-10-07	12.48	12.595	12.325	12.4	3280978	{}	\N	2020-10-12 15:57:11.790115
65744	PVG	2020-10-08	12.56	12.72	12.41	12.61	2871826	{}	\N	2020-10-12 15:57:11.795683
65745	PVG	2020-10-09	12.89	13.55	12.86	13.5	2818637	{}	\N	2020-10-12 15:57:11.801551
65746	PVG	2020-10-12	13.5	13.555	13.29	13.5302	531387	{}	\N	2020-10-12 15:57:11.806752
65747	PWZ	2020-09-28	27.57	27.59	27.53	27.56	29417	{}	\N	2020-10-12 15:57:12.675773
65748	PWZ	2020-09-29	27.56	27.6	27.56	27.6	35618	{}	\N	2020-10-12 15:57:12.686865
65749	PWZ	2020-09-30	27.6	27.73	27.54	27.61	59505	{}	\N	2020-10-12 15:57:12.694655
65750	PWZ	2020-10-01	27.56	27.62	27.56	27.56	41970	{}	\N	2020-10-12 15:57:12.701558
65751	PWZ	2020-10-02	27.52	27.5899	27.52	27.57	30676	{}	\N	2020-10-12 15:57:12.707246
65752	PWZ	2020-10-05	27.55	27.61	27.48	27.49	59515	{}	\N	2020-10-12 15:57:12.712394
65753	PWZ	2020-10-06	27.475	27.48	27.42	27.44	42285	{}	\N	2020-10-12 15:57:12.717156
65754	PWZ	2020-10-07	27.39	27.48	27.39	27.41	36282	{}	\N	2020-10-12 15:57:12.722022
65755	PWZ	2020-10-08	27.4	27.434	27.4	27.41	37261	{}	\N	2020-10-12 15:57:12.726933
65756	PWZ	2020-10-09	27.44	27.45	27.37	27.42	37299	{}	\N	2020-10-12 15:57:12.731977
65757	PWZ	2020-10-12	27.45	27.45	27.41	27.41	9855	{}	\N	2020-10-12 15:57:12.737292
65758	PYPL	2020-09-28	190.9996	192.47	188.09	192.02	7404283	{}	\N	2020-10-12 15:57:14.29059
65759	PYPL	2020-09-29	192.64	195.78	191.11	194.5	6649338	{}	\N	2020-10-12 15:57:14.30076
65760	PYPL	2020-09-30	194.14	198.72	193.311	197.03	7790541	{}	\N	2020-10-12 15:57:14.308
65761	PYPL	2020-10-01	199.26	201.27	196.2205	196.94	7097187	{}	\N	2020-10-12 15:57:14.313815
65762	PYPL	2020-10-02	192.73	196.21	189.25	191.91	6305906	{}	\N	2020-10-12 15:57:14.318738
65763	PYPL	2020-10-05	194.06	196.91	192.16	196.85	4988661	{}	\N	2020-10-12 15:57:14.323601
65764	PYPL	2020-10-06	196.19	197.37	190.88	191.66	5701809	{}	\N	2020-10-12 15:57:14.327797
65765	PYPL	2020-10-07	193.91	195.71	193.33	194.61	4153055	{}	\N	2020-10-12 15:57:14.331913
65766	PYPL	2020-10-08	196.75	196.87	192.67	193.06	5568963	{}	\N	2020-10-12 15:57:14.336035
65767	PYPL	2020-10-09	194.36	197.35	192.6	197.27	7064374	{}	\N	2020-10-12 15:57:14.340396
65768	PYPL	2020-10-12	199.87	200.54	196.6323	199.78	3391077	{}	\N	2020-10-12 15:57:14.344838
65769	QABA	2020-09-28	31.28	32.1	31.28	31.94	38185	{}	\N	2020-10-12 15:57:15.237347
65770	QABA	2020-09-29	31.6	31.63	31.09	31.58	3577	{}	\N	2020-10-12 15:57:15.247208
65771	QABA	2020-09-30	31.78	32.1829	31.6216	31.74	28984	{}	\N	2020-10-12 15:57:15.255218
65772	QABA	2020-10-01	31.7	31.99	31.54	31.99	12642	{}	\N	2020-10-12 15:57:15.262509
65773	QABA	2020-10-02	31.74	33.0711	31.4	32.98	13640	{}	\N	2020-10-12 15:57:15.268819
65774	QABA	2020-10-05	33.74	34.14	33.384	34.14	17828	{}	\N	2020-10-12 15:57:15.273957
65775	QABA	2020-10-06	34.48	35.37	34.25	34.31	27129	{}	\N	2020-10-12 15:57:15.278864
65776	QABA	2020-10-07	34.74	35.16	34.66	35.0591	68179	{}	\N	2020-10-12 15:57:15.283795
65777	QABA	2020-10-08	35.26	35.6278	35.24	35.6278	3394	{}	\N	2020-10-12 15:57:15.28938
65778	QABA	2020-10-09	35.62	35.8	35.25	35.2601	4634	{}	\N	2020-10-12 15:57:15.294698
65779	QABA	2020-10-12	34.98	35.65	34.97	35.6099	10128	{}	\N	2020-10-12 15:57:15.299872
65780	QCRH	2020-09-28	26.83	27.715	26.8	27.42	35710	{}	\N	2020-10-12 15:57:16.533158
65781	QCRH	2020-09-29	27.24	28	26.64	27.15	32192	{}	\N	2020-10-12 15:57:16.543978
65782	QCRH	2020-09-30	27.25	27.7	27.13	27.41	47590	{}	\N	2020-10-12 15:57:16.551407
65783	QCRH	2020-10-01	27.54	27.68	27.06	27.49	46634	{}	\N	2020-10-12 15:57:16.557408
65784	QCRH	2020-10-02	27	28.45	27	28.21	27493	{}	\N	2020-10-12 15:57:16.562771
65785	QCRH	2020-10-05	28.56	29.33	27.945	28.78	31329	{}	\N	2020-10-12 15:57:16.567353
65786	QCRH	2020-10-06	29.24	30.8	27.2832	29.15	63834	{}	\N	2020-10-12 15:57:16.571665
65787	QCRH	2020-10-07	29.06	30.495	29.06	30.11	48452	{}	\N	2020-10-12 15:57:16.576062
65788	QCRH	2020-10-08	30.42	31	30.11	30.73	40900	{}	\N	2020-10-12 15:57:16.580309
65789	QCRH	2020-10-09	30.98	31.39	29.0621	30.56	30420	{}	\N	2020-10-12 15:57:16.584519
65790	QCRH	2020-10-12	30.42	30.64	30.29	30.51	4457	{}	\N	2020-10-12 15:57:16.588689
65791	QDF	2020-09-28	44.4	44.71	44.357	44.55	27916	{}	\N	2020-10-12 15:57:18.017816
65792	QDF	2020-09-29	44.58	44.67	44.22	44.34	47423	{}	\N	2020-10-12 15:57:18.029209
65793	QDF	2020-09-30	44.62	45.1	44.51	44.78	21420	{}	\N	2020-10-12 15:57:18.036546
65794	QDF	2020-10-01	44.95	44.95	44.591	44.82	42162	{}	\N	2020-10-12 15:57:18.043929
65795	QDF	2020-10-02	44.07	44.8299	44.02	44.57	19830	{}	\N	2020-10-12 15:57:18.050071
65796	QDF	2020-10-05	44.84	45.41	44.84	45.35	28810	{}	\N	2020-10-12 15:57:18.055073
65797	QDF	2020-10-06	45.45	45.6	44.81	44.94	45974	{}	\N	2020-10-12 15:57:18.059994
65798	QDF	2020-10-07	45.24	45.67	45.24	45.62	26881	{}	\N	2020-10-12 15:57:18.065493
65799	QDF	2020-10-08	45.92	46.06	45.85	46.06	16407	{}	\N	2020-10-12 15:57:18.070919
65800	QDF	2020-10-09	46.32	46.4	46.16	46.29	34862	{}	\N	2020-10-12 15:57:18.077585
65801	QDF	2020-10-12	46.6	46.86	46.57	46.86	2710	{}	\N	2020-10-12 15:57:18.083466
65802	PSQ	2020-09-28	16.64	16.8	16.6	16.6	8042092	{}	\N	2020-10-12 15:57:19.284286
65803	PSQ	2020-09-29	16.63	16.7	16.565	16.67	2686775	{}	\N	2020-10-12 15:57:19.296671
65804	PSQ	2020-09-30	16.67	16.67	16.36	16.55	5383001	{}	\N	2020-10-12 15:57:19.304278
65805	PSQ	2020-10-01	16.32	16.4391	16.25	16.28	4805851	{}	\N	2020-10-12 15:57:19.310974
65806	PSQ	2020-10-02	16.6539	16.8	16.44	16.76	7547934	{}	\N	2020-10-12 15:57:19.317406
65807	PSQ	2020-10-05	16.63	16.64	16.3701	16.39	3593369	{}	\N	2020-10-12 15:57:19.322649
65808	PSQ	2020-10-06	16.43	16.73	16.33	16.67	4968454	{}	\N	2020-10-12 15:57:19.327912
65809	PSQ	2020-10-07	16.51	16.55	16.34	16.37	2927649	{}	\N	2020-10-12 15:57:19.333539
65810	PSQ	2020-10-08	16.25	16.3698	16.24	16.29	2800213	{}	\N	2020-10-12 15:57:19.338171
65811	PSQ	2020-10-09	16.21	16.23	16.05	16.06	3078389	{}	\N	2020-10-12 15:57:19.342583
65812	PSQ	2020-10-12	15.77	15.84	15.61	15.6519	3726201	{}	\N	2020-10-12 15:57:19.346964
65813	PTF	2020-09-28	109.24	110.036	107.936	109.0774	18491	{}	\N	2020-10-12 15:57:20.729261
65814	PTF	2020-09-29	109.03	110.3	109.03	109.43	59198	{}	\N	2020-10-12 15:57:20.742955
65815	PTF	2020-09-30	108.94	111.19	108.9	110.04	17959	{}	\N	2020-10-12 15:57:20.749614
65816	PTF	2020-10-01	111.68	113.1899	111.2	113.01	20640	{}	\N	2020-10-12 15:57:20.756209
65817	PTF	2020-10-02	110.38	113.29	109.9444	111.4311	18735	{}	\N	2020-10-12 15:57:20.76235
65818	PTF	2020-10-05	112.28	114.08	112.28	114.05	15479	{}	\N	2020-10-12 15:57:20.767371
65819	PTF	2020-10-06	114	116.3586	113.3	113.98	29021	{}	\N	2020-10-12 15:57:20.771791
65820	PTF	2020-10-07	115.2	117.155	115.2	116.94	32666	{}	\N	2020-10-12 15:57:20.776642
65821	PTF	2020-10-08	116.8	117.0721	116.09	116.6	12164	{}	\N	2020-10-12 15:57:20.781387
65822	PTF	2020-10-09	117.54	118.77	117.263	118.77	19171	{}	\N	2020-10-12 15:57:20.786546
65823	PTF	2020-10-12	120.85	120.85	119.655	119.75	7424	{}	\N	2020-10-12 15:57:20.791637
65824	PTR	2020-09-28	29.54	29.96	29.42	29.76	288838	{}	\N	2020-10-12 15:57:22.452777
65825	PTR	2020-09-29	29.43	29.49	29.23	29.35	301813	{}	\N	2020-10-12 15:57:22.459947
65826	PTR	2020-09-30	29.48	29.75	29.31	29.44	219882	{}	\N	2020-10-12 15:57:22.465702
65827	PTR	2020-10-01	29.54	29.62	29.28	29.43	211051	{}	\N	2020-10-12 15:57:22.470883
65828	PTR	2020-10-02	28.97	29.45	28.97	29.14	232261	{}	\N	2020-10-12 15:57:22.475843
65829	PTR	2020-10-05	29.08	29.44	29.035	29.44	279821	{}	\N	2020-10-12 15:57:22.482724
65830	PTR	2020-10-06	29.32	29.58	29.17	29.19	261049	{}	\N	2020-10-12 15:57:22.488029
65831	PTR	2020-10-07	29.32	29.32	29.03	29.16	184007	{}	\N	2020-10-12 15:57:22.492848
65832	PTR	2020-10-08	29.34	29.7	29.2	29.64	252173	{}	\N	2020-10-12 15:57:22.498412
65833	PTR	2020-10-09	29.45	29.5	29.25	29.32	213651	{}	\N	2020-10-12 15:57:22.505246
65834	PTR	2020-10-12	29.57	29.62	29.38	29.4902	38862	{}	\N	2020-10-12 15:57:22.512273
65835	PUTW	2020-09-28	26.55	26.8386	26.52	26.69	8086	{}	\N	2020-10-12 15:57:23.563758
65836	PUTW	2020-09-29	26.7443	26.7443	26.61	26.71	20135	{}	\N	2020-10-12 15:57:23.575385
65837	PUTW	2020-09-30	26.72	26.87	26.72	26.76	8863	{}	\N	2020-10-12 15:57:23.583741
65838	PUTW	2020-10-01	26.84	26.96	26.78	26.87	12396	{}	\N	2020-10-12 15:57:23.590238
65839	PUTW	2020-10-02	26.8127	26.8401	26.71	26.71	7912	{}	\N	2020-10-12 15:57:23.596878
65840	PUTW	2020-10-05	26.82	27.02	26.8	26.92	25886	{}	\N	2020-10-12 15:57:23.602038
65841	PUTW	2020-10-06	27	27.06	26.8997	26.8997	5652	{}	\N	2020-10-12 15:57:23.607351
65842	PUTW	2020-10-07	27.01	27.0999	27	27.04	6076	{}	\N	2020-10-12 15:57:23.612934
65843	PUTW	2020-10-08	27.149	27.15	27.08	27.13	14689	{}	\N	2020-10-12 15:57:23.618132
65844	PUTW	2020-10-09	27.1493	27.1999	27.12	27.18	9028	{}	\N	2020-10-12 15:57:23.62307
65845	PUTW	2020-10-12	27.18	27.18	27.1501	27.18	3166	{}	\N	2020-10-12 15:57:23.628768
65846	PWB	2020-09-28	61.27	61.4199	61.01	61.31	145082	{}	\N	2020-10-12 15:57:24.610438
65847	PWB	2020-09-29	61.36	61.7	61.2901	61.34	21572	{}	\N	2020-10-12 15:57:24.62187
65848	PWB	2020-09-30	61.45	62.36	61.4342	61.9	14953	{}	\N	2020-10-12 15:57:24.630358
65849	PWB	2020-10-01	62.49	62.576	62.01	62.26	32333	{}	\N	2020-10-12 15:57:24.636764
65850	PWB	2020-10-02	61.6282	61.85	60.9891	61.21	30579	{}	\N	2020-10-12 15:57:24.642395
65851	PWB	2020-10-05	61.63	62.33	61.63	62.32	16829	{}	\N	2020-10-12 15:57:24.647563
65852	PWB	2020-10-06	62.59	62.5951	61.3733	61.4	57856	{}	\N	2020-10-12 15:57:24.653483
65853	PWB	2020-10-07	62.1628	62.4997	62.035	62.42	326015	{}	\N	2020-10-12 15:57:24.659041
65854	PWB	2020-10-08	62.81	62.81	62.53	62.72	36708	{}	\N	2020-10-12 15:57:24.664292
65855	PWB	2020-10-09	62.99	63.67	62.99	63.67	167208	{}	\N	2020-10-12 15:57:24.669853
65856	PWB	2020-10-12	64.29	64.5999	64.1	64.46	3775	{}	\N	2020-10-12 15:57:24.674997
65857	PXI	2020-09-28	13.02	13.33	13.02	13.33	40644	{}	\N	2020-10-12 15:57:26.278441
65858	PXI	2020-09-29	12.9687	12.9687	12.754	12.92	278919	{}	\N	2020-10-12 15:57:26.291004
65859	PXI	2020-09-30	13	13.11	12.74	12.76	24378	{}	\N	2020-10-12 15:57:26.298521
65860	PXI	2020-10-01	12.68	12.6915	12.35	12.38	26725	{}	\N	2020-10-12 15:57:26.30485
65861	PXI	2020-10-02	11.96	12.67	11.96	12.6301	13151	{}	\N	2020-10-12 15:57:26.309872
65862	PXI	2020-10-05	12.881	13.1141	12.7701	13.1141	666542	{}	\N	2020-10-12 15:57:26.314629
65863	PXI	2020-10-06	13.27	13.42	12.8	12.89	22942	{}	\N	2020-10-12 15:57:26.319382
65864	PXI	2020-10-07	12.91	13.0938	12.79	13.05	87357	{}	\N	2020-10-12 15:57:26.32378
65865	PXI	2020-10-08	13.2	13.5841	13.14	13.5841	122467	{}	\N	2020-10-12 15:57:26.328188
65866	PXI	2020-10-09	13.57	13.57	13.36	13.4299	11147	{}	\N	2020-10-12 15:57:26.332339
65867	PXI	2020-10-12	13.42	13.42	13.245	13.38	1564	{}	\N	2020-10-12 15:57:26.337542
65868	PZD	2020-09-28	56.85	57.904	56.85	57.63	8266	{}	\N	2020-10-12 15:57:27.742744
65869	PZD	2020-09-29	58.5469	58.7666	58.13	58.35	12808	{}	\N	2020-10-12 15:57:27.754642
65870	PZD	2020-09-30	58.45	59.179	58.2664	58.62	11984	{}	\N	2020-10-12 15:57:27.761198
65871	PZD	2020-10-01	59.23	59.953	59.23	59.74	26115	{}	\N	2020-10-12 15:57:27.767262
65872	PZD	2020-10-02	59.33	59.83	59.33	59.59	7731	{}	\N	2020-10-12 15:57:27.773013
65873	PZD	2020-10-05	60.06	61.444	60.06	61.22	21286	{}	\N	2020-10-12 15:57:27.77759
65874	PZD	2020-10-06	61.195	61.988	60.62	60.87	19280	{}	\N	2020-10-12 15:57:27.782309
65875	PZD	2020-10-07	61.2	62.4879	61.2	62.355	25198	{}	\N	2020-10-12 15:57:27.787818
65876	PZD	2020-10-08	62.75	62.75	61.97	62.19	28703	{}	\N	2020-10-12 15:57:27.793154
65877	PZD	2020-10-09	62.55	63.374	62.55	63.135	21039	{}	\N	2020-10-12 15:57:27.79831
65878	PZD	2020-10-12	63.42	63.42	63.2111	63.27	7907	{}	\N	2020-10-12 15:57:27.803555
65879	QCLN	2020-09-28	41.62	42.45	41.595	42.42	305750	{}	\N	2020-10-12 15:57:29.022002
65880	QCLN	2020-09-29	42.47	43.62	42.4001	43.49	252286	{}	\N	2020-10-12 15:57:29.034176
65881	QCLN	2020-09-30	43.75	44.829	43.75	44.35	362599	{}	\N	2020-10-12 15:57:29.042219
65882	QCLN	2020-10-01	45.17	45.6299	44.8	45.61	398396	{}	\N	2020-10-12 15:57:29.049362
65883	QCLN	2020-10-02	43.97	45.86	43.76	45.2	250629	{}	\N	2020-10-12 15:57:29.056246
65884	QCLN	2020-10-05	46	47.4	45.92	47.4	268117	{}	\N	2020-10-12 15:57:29.062242
65885	QCLN	2020-10-06	47.66	48.25	46.605	47.1	368531	{}	\N	2020-10-12 15:57:29.067529
65886	QCLN	2020-10-07	48.21	49.545	47.8937	49.44	559288	{}	\N	2020-10-12 15:57:29.072889
65887	QCLN	2020-10-08	50.38	50.38	48.595	48.92	386339	{}	\N	2020-10-12 15:57:29.078482
65888	QCLN	2020-10-09	49.35	50.255	49.1966	50.03	275604	{}	\N	2020-10-12 15:57:29.084183
65889	QCLN	2020-10-12	50.84	50.9	49.0701	49.1101	209560	{}	\N	2020-10-12 15:57:29.090218
65890	QTNT	2020-09-28	5.45	5.79	5.05	5.32	2686407	{}	\N	2020-10-12 15:57:30.103744
65891	QTNT	2020-09-29	5.3436	5.3527	5	5.21	1055661	{}	\N	2020-10-12 15:57:30.113765
65892	QTNT	2020-09-30	5.2	5.32	4.98	5.14	625172	{}	\N	2020-10-12 15:57:30.120914
65893	QTNT	2020-10-01	5.13	5.73	5.08	5.52	996863	{}	\N	2020-10-12 15:57:30.127237
65894	QTNT	2020-10-02	5.3	5.64	5.26	5.61	786188	{}	\N	2020-10-12 15:57:30.132921
65895	QTNT	2020-10-05	5.66	5.9499	5.415	5.5	1033720	{}	\N	2020-10-12 15:57:30.138893
65896	QTNT	2020-10-06	5.52	5.59	5.37	5.39	526329	{}	\N	2020-10-12 15:57:30.144789
65897	QTNT	2020-10-07	5.44	5.72	5.39	5.69	424537	{}	\N	2020-10-12 15:57:30.150259
65898	QTNT	2020-10-08	5.75	5.85	5.7	5.72	410298	{}	\N	2020-10-12 15:57:30.155841
65899	QTNT	2020-10-09	5.79	5.79	5.459	5.61	511977	{}	\N	2020-10-12 15:57:30.161338
65900	QTNT	2020-10-12	5.64	5.68	5.47	5.54	90722	{}	\N	2020-10-12 15:57:30.166878
65901	PYZ	2020-09-28	54.64	55.5477	54.64	55.5477	1027	{}	\N	2020-10-12 15:57:31.482942
65902	PYZ	2020-09-29	55.5815	55.5815	54.8001	54.925	1560	{}	\N	2020-10-12 15:57:31.494394
65903	PYZ	2020-09-30	55.94	55.94	55.4905	55.4905	5532	{}	\N	2020-10-12 15:57:31.501639
65904	PYZ	2020-10-01	55.96	55.96	55.17	55.21	3860	{}	\N	2020-10-12 15:57:31.508077
65905	PYZ	2020-10-02	54.77	56.22	54.77	56.22	2429	{}	\N	2020-10-12 15:57:31.513819
65906	PYZ	2020-10-05	58.0449	58.1253	57.62	58.089	209515	{}	\N	2020-10-12 15:57:31.518554
65907	PYZ	2020-10-06	58.5	59.127	57.2	57.2	11493	{}	\N	2020-10-12 15:57:31.523204
65908	PYZ	2020-10-07	58.06	59.3	57.72	59.2182	19452	{}	\N	2020-10-12 15:57:31.528098
65909	PYZ	2020-10-08	60.11	60.11	59.48	59.84	28759	{}	\N	2020-10-12 15:57:31.533487
65910	PYZ	2020-10-09	60.5	60.5	60.01	60.14	4346	{}	\N	2020-10-12 15:57:31.538879
65911	PYZ	2020-10-12	60.14	60.14	60.13	60.13	363	{}	\N	2020-10-12 15:57:31.544296
65912	PZN	2020-09-28	5.22	5.34	5.13	5.3	60491	{}	\N	2020-10-12 15:57:33.064954
65913	PZN	2020-09-29	5.19	5.46	5.09	5.42	82478	{}	\N	2020-10-12 15:57:33.076372
65914	PZN	2020-09-30	5.525	5.59	5.245	5.36	94352	{}	\N	2020-10-12 15:57:33.083063
65915	PZN	2020-10-01	5.4	5.4	5.25	5.28	38039	{}	\N	2020-10-12 15:57:33.089518
65916	PZN	2020-10-02	5.23	5.49	5.23	5.33	39427	{}	\N	2020-10-12 15:57:33.093978
65917	PZN	2020-10-05	5.41	5.4899	5.29	5.29	34782	{}	\N	2020-10-12 15:57:33.098272
65918	PZN	2020-10-06	5.32	5.39	5.18	5.23	47063	{}	\N	2020-10-12 15:57:33.102835
65919	PZN	2020-10-07	5.25	5.38	5.16	5.28	71411	{}	\N	2020-10-12 15:57:33.107393
65920	PZN	2020-10-08	5.29	5.81	5.29	5.77	78457	{}	\N	2020-10-12 15:57:33.112099
65921	PZN	2020-10-09	5.74	5.95	5.62	5.9	48828	{}	\N	2020-10-12 15:57:33.116859
65922	PZN	2020-10-12	5.81	5.93	5.81	5.93	6830	{}	\N	2020-10-12 15:57:33.121194
65923	QDYN	2020-09-28	44.91	44.91	44.8481	44.8481	217	{}	\N	2020-10-12 15:57:35.960972
65924	QDYN	2020-09-29	44.6014	44.6014	44.6014	44.6014	6	{}	\N	2020-10-12 15:57:35.968167
65925	QDYN	2020-09-30	45.08	45.0801	45.0303	45.0303	371	{}	\N	2020-10-12 15:57:35.974099
65926	QDYN	2020-10-01	45.25	45.25	45.0462	45.0462	360	{}	\N	2020-10-12 15:57:35.981014
65927	QDYN	2020-10-02	44.7431	45.07	44.7431	45.07	2670	{}	\N	2020-10-12 15:57:35.986675
65928	QDYN	2020-10-05	45.7966	45.7966	45.7966	45.7966	28	{}	\N	2020-10-12 15:57:35.992359
65929	QDYN	2020-10-07	46.1014	46.1014	46.1014	46.1014	50	{}	\N	2020-10-12 15:57:35.997698
65930	QDYN	2020-10-08	46.4219	46.4219	46.4219	46.4219	0	{}	\N	2020-10-12 15:57:36.002475
65931	QDYN	2020-10-09	46.64	46.7302	46.64	46.7302	296	{}	\N	2020-10-12 15:57:36.007636
65932	QLS	2020-09-28	23.8301	23.8599	23.8301	23.8599	451	{}	\N	2020-10-12 15:57:38.286204
65933	QLS	2020-09-29	23.8272	23.8272	23.8272	23.8272	333	{}	\N	2020-10-12 15:57:38.301011
65934	QLS	2020-09-30	23.85	23.88	23.8449	23.8449	1790	{}	\N	2020-10-12 15:57:38.308631
65935	QLS	2020-10-01	23.8801	23.93	23.8801	23.92	5157	{}	\N	2020-10-12 15:57:38.314962
65936	QLS	2020-10-02	23.9	23.9	23.8604	23.8604	858	{}	\N	2020-10-12 15:57:38.321194
65937	QLS	2020-10-05	23.9998	24.05	23.9998	24.0199	4753	{}	\N	2020-10-12 15:57:38.326993
65938	QLS	2020-10-06	24.08	24.08	23.9263	23.9263	11228	{}	\N	2020-10-12 15:57:38.331168
65939	QLS	2020-10-07	24.03	24.0731	24.02	24.0731	3791	{}	\N	2020-10-12 15:57:38.335481
65940	QLS	2020-10-08	24.1633	24.1633	24.155	24.16	600	{}	\N	2020-10-12 15:57:38.339454
65941	QLS	2020-10-09	24.23	24.2546	24.23	24.2546	503	{}	\N	2020-10-12 15:57:38.343332
65942	QLS	2020-10-12	24.31	24.31	24.31	24.31	354	{}	\N	2020-10-12 15:57:38.347529
65943	QLC	2020-09-28	36.4526	36.58	36.4526	36.54	6151	{}	\N	2020-10-12 15:57:39.444511
65944	QLC	2020-09-29	36.535	36.535	36.31	36.36	5292	{}	\N	2020-10-12 15:57:39.453267
65945	QLC	2020-09-30	36.719	36.84	36.68	36.68	4030	{}	\N	2020-10-12 15:57:39.459378
65946	QLC	2020-10-01	36.97	36.98	36.76	36.7803	4321	{}	\N	2020-10-12 15:57:39.464257
65947	QLC	2020-10-02	36.35	36.66	36.3239	36.55	5538	{}	\N	2020-10-12 15:57:39.469041
65948	QLC	2020-10-05	36.81	37.1348	36.81	37.1348	1483	{}	\N	2020-10-12 15:57:39.473101
65949	QLC	2020-10-06	37.1599	37.18	36.68	36.68	7631	{}	\N	2020-10-12 15:57:39.477056
65950	QLC	2020-10-07	37.1	37.2694	37.03	37.2694	4591	{}	\N	2020-10-12 15:57:39.481103
65951	QLC	2020-10-08	37.48	37.5362	37.4695	37.5362	4359	{}	\N	2020-10-12 15:57:39.486056
65952	QLC	2020-10-09	37.78	37.89	37.74	37.85	4301	{}	\N	2020-10-12 15:57:39.490168
65953	PZT	2020-09-28	25.645	25.71	25.58	25.58	2372	{}	\N	2020-10-12 15:57:40.41871
65954	PZT	2020-09-29	25.7	25.7	25.6301	25.6301	4927	{}	\N	2020-10-12 15:57:40.429795
65955	PZT	2020-09-30	25.73	25.73	25.52	25.52	14964	{}	\N	2020-10-12 15:57:40.436444
65956	PZT	2020-10-01	25.51	25.59	25.51	25.59	4876	{}	\N	2020-10-12 15:57:40.442151
65957	PZT	2020-10-02	25.54	25.6191	25.53	25.59	4997	{}	\N	2020-10-12 15:57:40.446785
65958	PZT	2020-10-05	25.56	25.61	25.515	25.575	11121	{}	\N	2020-10-12 15:57:40.4512
65959	PZT	2020-10-06	25.5411	25.58	25.48	25.53	11924	{}	\N	2020-10-12 15:57:40.455254
65960	PZT	2020-10-07	25.51	25.51	25.39	25.39	16914	{}	\N	2020-10-12 15:57:40.458964
65961	PZT	2020-10-08	25.448	25.4701	25.4	25.4686	6744	{}	\N	2020-10-12 15:57:40.462693
65962	PZT	2020-10-09	25.46	25.46	25.37	25.3814	4191	{}	\N	2020-10-12 15:57:40.466685
65963	PZT	2020-10-12	25.52	25.52	25.405	25.49	6506	{}	\N	2020-10-12 15:57:40.471451
65964	QCOM	2020-09-28	115.96	118.7391	114.92	118.47	8883985	{}	\N	2020-10-12 15:57:41.441983
65965	QCOM	2020-09-29	118.14	119.25	117.2	117.38	5583641	{}	\N	2020-10-12 15:57:41.455442
65966	QCOM	2020-09-30	117.5	119.255	116.9238	117.68	5685737	{}	\N	2020-10-12 15:57:41.463895
65967	QCOM	2020-10-01	119.93	120.205	117.7	119.52	5132802	{}	\N	2020-10-12 15:57:41.471086
65968	QCOM	2020-10-02	116.78	118.66	115.43	115.47	5561339	{}	\N	2020-10-12 15:57:41.47803
65969	QCOM	2020-10-05	116.9	120.64	116.73	120.52	5726132	{}	\N	2020-10-12 15:57:41.48428
65970	QCOM	2020-10-06	120.45	122.52	119.1	119.54	7328016	{}	\N	2020-10-12 15:57:41.490945
65971	QCOM	2020-10-07	121.235	123.36	120.65	123.03	7292236	{}	\N	2020-10-12 15:57:41.497287
65972	QCOM	2020-10-08	124.43	124.43	121.84	122.34	4734197	{}	\N	2020-10-12 15:57:41.502973
65973	QCOM	2020-10-09	123.93	125.89	122.79	124.87	11046639	{}	\N	2020-10-12 15:57:41.508978
65974	QCOM	2020-10-12	127.6996	127.6996	124.9523	126.39	3159417	{}	\N	2020-10-12 15:57:41.514927
65975	QID	2020-09-28	9.57	9.769	9.52	9.53	12747845	{}	\N	2020-10-12 15:57:42.985408
65976	QID	2020-09-29	9.55	9.6499	9.48	9.62	6227093	{}	\N	2020-10-12 15:57:42.994056
65977	QID	2020-09-30	9.6	9.61	9.25	9.47	10822663	{}	\N	2020-10-12 15:57:43.000965
65978	QID	2020-10-01	9.2001	9.34	9.13	9.17	10433723	{}	\N	2020-10-12 15:57:43.00637
65979	QID	2020-10-02	9.57	9.75	9.34	9.68	17285629	{}	\N	2020-10-12 15:57:43.011955
65980	QID	2020-10-05	9.54	9.545	9.25	9.26	7525289	{}	\N	2020-10-12 15:57:43.016531
65981	QID	2020-10-06	9.34	9.665	9.205	9.6	11454013	{}	\N	2020-10-12 15:57:43.030373
65982	QID	2020-10-07	9.41	9.455	9.21	9.25	6406263	{}	\N	2020-10-12 15:57:43.036035
65983	QID	2020-10-08	9.12	9.24	9.1	9.17	7556028	{}	\N	2020-10-12 15:57:43.040174
65984	QID	2020-10-09	9.07	9.0899	8.88	8.88	7967663	{}	\N	2020-10-12 15:57:43.044942
65985	QID	2020-10-12	8.59	8.655	8.4	8.44	3866554	{}	\N	2020-10-12 15:57:43.049581
65986	QLD	2020-09-28	90.7	91.11	88.945	91.05	1924955	{}	\N	2020-10-12 15:57:44.161372
65987	QLD	2020-09-29	90.81	91.43	89.9	90.22	1862227	{}	\N	2020-10-12 15:57:44.172682
65988	QLD	2020-09-30	90.34	93.62	90.24	91.65	2412080	{}	\N	2020-10-12 15:57:44.180504
65989	QLD	2020-10-01	94.11	94.92	92.85	94.43	2721691	{}	\N	2020-10-12 15:57:44.187439
65990	QLD	2020-10-02	90.25	92.75	88.51	89.11	3328486	{}	\N	2020-10-12 15:57:44.193592
65991	QLD	2020-10-05	90.39	93.08	90.35	92.97	2130621	{}	\N	2020-10-12 15:57:44.19905
65992	QLD	2020-10-06	92.33	93.56	88.92	89.52	2460473	{}	\N	2020-10-12 15:57:44.20494
65993	QLD	2020-10-07	91.31	93.17	90.9727	92.8	1199345	{}	\N	2020-10-12 15:57:44.21005
65994	QLD	2020-10-08	94.25	94.315	92.91	93.64	1536346	{}	\N	2020-10-12 15:57:44.215338
65995	QLD	2020-10-09	94.7	96.52	94.49	96.48	2023819	{}	\N	2020-10-12 15:57:44.220901
65996	QLD	2020-10-12	99.74	101.82	99	101.3383	1155513	{}	\N	2020-10-12 15:57:44.226391
65997	QLTA	2020-09-28	57.8	57.94	57.74	57.93	150044	{}	\N	2020-10-12 15:57:45.216578
65998	QLTA	2020-09-29	58.02	58.05	57.83	57.87	96152	{}	\N	2020-10-12 15:57:45.231405
65999	QLTA	2020-09-30	57.85	57.86	57.75	57.83	77887	{}	\N	2020-10-12 15:57:45.239322
66000	QLTA	2020-10-01	57.73	57.89	57.63	57.85	179651	{}	\N	2020-10-12 15:57:45.247307
66001	QLTA	2020-10-02	57.84	57.89	57.73	57.88	181687	{}	\N	2020-10-12 15:57:45.253821
66002	QLTA	2020-10-05	57.78	57.84	57.655	57.66	200372	{}	\N	2020-10-12 15:57:45.258733
66003	QLTA	2020-10-06	57.67	57.86	57.54	57.62	332371	{}	\N	2020-10-12 15:57:45.263599
66004	QLTA	2020-10-07	57.56	57.76	57.55	57.61	205548	{}	\N	2020-10-12 15:57:45.268874
66005	QLTA	2020-10-08	57.75	57.806	57.69	57.72	84946	{}	\N	2020-10-12 15:57:45.27404
66006	QLTA	2020-10-09	57.69	57.84	57.65	57.81	309897	{}	\N	2020-10-12 15:57:45.279319
66007	QLTA	2020-10-12	57.92	58.04	57.88	58.025	29439	{}	\N	2020-10-12 15:57:45.284764
66008	QQXT	2020-09-28	67.9401	68.3804	67.9401	68.23	1717	{}	\N	2020-10-12 15:57:46.270554
66009	QQXT	2020-09-29	68.2639	68.2639	67.9109	67.96	9562	{}	\N	2020-10-12 15:57:46.284854
66010	QQXT	2020-09-30	68.06	68.925	68.06	68.5713	8463	{}	\N	2020-10-12 15:57:46.293064
66011	QQXT	2020-10-01	69.05	69.27	68.74	69.27	17073	{}	\N	2020-10-12 15:57:46.299907
66012	QQXT	2020-10-02	68.27	68.27	67.555	68.0522	3805	{}	\N	2020-10-12 15:57:46.306188
66013	QQXT	2020-10-05	68.35	69.15	68.35	69.15	3575	{}	\N	2020-10-12 15:57:46.311838
66014	QQXT	2020-10-06	69.02	69.43	68.23	68.5	4791	{}	\N	2020-10-12 15:57:46.317427
66015	QQXT	2020-10-07	68.59	69.6	68.59	69.58	2616	{}	\N	2020-10-12 15:57:46.323372
66016	QQXT	2020-10-08	69.74	69.865	69.58	69.865	4053	{}	\N	2020-10-12 15:57:46.328682
66017	QQXT	2020-10-09	70.18	70.7662	70.18	70.65	4529	{}	\N	2020-10-12 15:57:46.334102
66018	QQXT	2020-10-12	71.14	71.93	71.1	71.86	22237	{}	\N	2020-10-12 15:57:46.339813
66019	QSR	2020-09-28	57.23	58.05	57.23	57.91	1784202	{}	\N	2020-10-12 15:57:47.812265
66020	QSR	2020-09-29	57.95	57.95	57.005	57.37	4306073	{}	\N	2020-10-12 15:57:47.826359
66021	QSR	2020-09-30	57.46	58	57.13	57.51	4645166	{}	\N	2020-10-12 15:57:47.833861
66022	QSR	2020-10-01	57.83	58.13	57.11	57.75	5910528	{}	\N	2020-10-12 15:57:47.840295
66023	QSR	2020-10-02	56.33	57.41	56.33	57.36	3481086	{}	\N	2020-10-12 15:57:47.845963
66024	QSR	2020-10-05	57.92	58.11	57.31	57.77	2274760	{}	\N	2020-10-12 15:57:47.850907
66025	QSR	2020-10-06	58	58.97	57.34	57.56	2120268	{}	\N	2020-10-12 15:57:47.855739
66026	QSR	2020-10-07	58.1	58.3699	57.64	58.1	1418338	{}	\N	2020-10-12 15:57:47.860881
66027	QSR	2020-10-08	58.72	60.16	58.2	59.96	2593558	{}	\N	2020-10-12 15:57:47.865416
66028	QSR	2020-10-09	60.2	60.41	59.55	59.6	1439545	{}	\N	2020-10-12 15:57:47.87034
66029	QSR	2020-10-12	59.88	59.96	59.15	59.47	295716	{}	\N	2020-10-12 15:57:47.874976
66030	QD	2020-09-28	1.34	1.34	1.24	1.24	1577487	{}	\N	2020-10-12 15:57:49.067879
66031	QD	2020-09-29	1.24	1.25	1.22	1.24	2703398	{}	\N	2020-10-12 15:57:49.079825
66032	QD	2020-09-30	1.25	1.3	1.22	1.24	2802881	{}	\N	2020-10-12 15:57:49.087956
66033	QD	2020-10-01	1.28	1.28	1.22	1.24	2566635	{}	\N	2020-10-12 15:57:49.094603
66034	QD	2020-10-02	1.21	1.32	1.21	1.31	3355465	{}	\N	2020-10-12 15:57:49.100893
66035	QD	2020-10-05	1.31	1.33	1.245	1.3	2942554	{}	\N	2020-10-12 15:57:49.10698
66036	QD	2020-10-06	1.31	1.43	1.28	1.33	8292481	{}	\N	2020-10-12 15:57:49.112366
66037	QD	2020-10-07	1.39	1.39	1.32	1.37	3695886	{}	\N	2020-10-12 15:57:49.117844
66038	QD	2020-10-08	1.4	1.415	1.34	1.34	3739208	{}	\N	2020-10-12 15:57:49.123454
66039	QD	2020-10-09	1.37	1.415	1.34	1.4	3147174	{}	\N	2020-10-12 15:57:49.129139
66040	QD	2020-10-12	1.41	1.43	1.36	1.365	1131035	{}	\N	2020-10-12 15:57:49.134523
66041	QIWI	2020-09-28	17.23	17.48	17.205	17.34	288706	{}	\N	2020-10-12 15:57:50.392737
66042	QIWI	2020-09-29	17.27	17.6	17.27	17.39	262160	{}	\N	2020-10-12 15:57:50.403991
66043	QIWI	2020-09-30	17.52	17.6	17.21	17.35	241271	{}	\N	2020-10-12 15:57:50.410537
66044	QIWI	2020-10-01	17.29	17.5	17.27	17.4	238679	{}	\N	2020-10-12 15:57:50.416887
66045	QIWI	2020-10-02	16.95	17.43	16.8505	17.32	138711	{}	\N	2020-10-12 15:57:50.422802
66046	QIWI	2020-10-05	17.3	17.47	17.2	17.31	90680	{}	\N	2020-10-12 15:57:50.427966
66047	QIWI	2020-10-06	17.34	17.56	17.1501	17.2	220125	{}	\N	2020-10-12 15:57:50.433488
66048	QIWI	2020-10-07	17.08	17.18	16.78	16.89	181156	{}	\N	2020-10-12 15:57:50.438159
66049	QIWI	2020-10-08	17.26	17.52	17.15	17.48	177757	{}	\N	2020-10-12 15:57:50.443381
66050	QIWI	2020-10-09	17.19	17.29	16.835	17.14	242979	{}	\N	2020-10-12 15:57:50.449331
66051	QIWI	2020-10-12	17.25	17.32	17.05	17.26	38558	{}	\N	2020-10-12 15:57:50.45472
66052	QLYS	2020-09-28	100.02	101.54	99.275	100.51	404520	{}	\N	2020-10-12 15:57:51.431513
66053	QLYS	2020-09-29	100.04	101.3	99.38	99.46	343575	{}	\N	2020-10-12 15:57:51.441304
66054	QLYS	2020-09-30	99.49	100.8	97.83	98.01	401134	{}	\N	2020-10-12 15:57:51.448246
66055	QLYS	2020-10-01	98.52	99.185	97.17	97.56	296413	{}	\N	2020-10-12 15:57:51.454177
66056	QLYS	2020-10-02	96.46	98.7	95.99	96.13	270894	{}	\N	2020-10-12 15:57:51.460348
66057	QLYS	2020-10-05	96.61	98.73	96.19	98.34	201942	{}	\N	2020-10-12 15:57:51.465238
66058	QLYS	2020-10-06	98.75	99.27	96.73	96.91	260683	{}	\N	2020-10-12 15:57:51.469999
66059	QLYS	2020-10-07	97.35	99.6	97.04	98.9	261610	{}	\N	2020-10-12 15:57:51.474677
66060	QLYS	2020-10-08	100.19	100.59	98.475	99.02	207388	{}	\N	2020-10-12 15:57:51.479347
66061	QLYS	2020-10-09	99.59	101	99	99.9	272527	{}	\N	2020-10-12 15:57:51.484327
66062	QLYS	2020-10-12	100.89	102.39	100.16	101.5	72067	{}	\N	2020-10-12 15:57:51.489115
66063	QMN	2020-09-28	26.695	26.695	26.695	26.695	86	{}	\N	2020-10-12 15:57:54.083022
66064	QMN	2020-09-30	26.77	26.77	26.68	26.745	60504	{}	\N	2020-10-12 15:57:54.091345
66065	QMN	2020-10-01	26.7827	26.8053	26.7827	26.8053	642	{}	\N	2020-10-12 15:57:54.097866
66066	QMN	2020-10-02	26.755	26.755	26.755	26.755	82	{}	\N	2020-10-12 15:57:54.103385
66067	QMN	2020-10-05	26.815	26.8259	26.81	26.8259	6059	{}	\N	2020-10-12 15:57:54.108907
66068	QMN	2020-10-06	26.84	26.84	26.8048	26.8048	52632	{}	\N	2020-10-12 15:57:54.114208
66069	QMN	2020-10-07	26.8499	26.8797	26.8499	26.85	1563	{}	\N	2020-10-12 15:57:54.118911
66070	QMN	2020-10-08	26.8943	26.9098	26.8943	26.8943	1559	{}	\N	2020-10-12 15:57:54.123807
66071	QMN	2020-10-09	26.9372	26.9372	26.9372	26.9372	437	{}	\N	2020-10-12 15:57:54.128955
66072	QQQX	2020-09-28	24.29	24.39	24.18	24.29	155018	{}	\N	2020-10-12 15:57:55.04777
66073	QQQX	2020-09-29	24.31	24.39	24.05	24.13	243843	{}	\N	2020-10-12 15:57:55.055598
66074	QQQX	2020-09-30	24.19	24.55	24.19	24.34	128794	{}	\N	2020-10-12 15:57:55.061256
66075	QQQX	2020-10-01	24.35	24.96	24.35	24.84	166563	{}	\N	2020-10-12 15:57:55.066839
66076	QQQX	2020-10-02	24.63	24.72	24.17	24.32	226595	{}	\N	2020-10-12 15:57:55.071759
66077	QQQX	2020-10-05	24.37	24.561	24.334	24.54	117481	{}	\N	2020-10-12 15:57:55.076786
66078	QQQX	2020-10-06	24.49	24.72	24.28	24.31	112625	{}	\N	2020-10-12 15:57:55.082155
66079	QQQX	2020-10-07	24.65	24.65	24.44	24.57	109988	{}	\N	2020-10-12 15:57:55.087282
66080	QQQX	2020-10-08	24.59	24.6202	24.33	24.41	245137	{}	\N	2020-10-12 15:57:55.092275
66081	QQQX	2020-10-09	24.46	24.69	24.46	24.58	120706	{}	\N	2020-10-12 15:57:55.097744
66082	QQQX	2020-10-12	24.68	24.93	24.68	24.91	67570	{}	\N	2020-10-12 15:57:55.102771
66083	QRVO	2020-09-28	128.04	128.72	126.58	128.43	1124971	{}	\N	2020-10-12 15:57:56.275528
66084	QRVO	2020-09-29	128.24	132.62	128.14	130.77	1072218	{}	\N	2020-10-12 15:57:56.288697
66085	QRVO	2020-09-30	130.53	131.27	128.02	129.01	1320389	{}	\N	2020-10-12 15:57:56.296051
66086	QRVO	2020-10-01	132.0845	135.21	131.3668	135.09	1566160	{}	\N	2020-10-12 15:57:56.30267
66087	QRVO	2020-10-02	131.5	133.48	130.5101	130.57	822152	{}	\N	2020-10-12 15:57:56.308466
66088	QRVO	2020-10-05	132.37	135.38	132.35	135.3	994707	{}	\N	2020-10-12 15:57:56.314141
66089	QRVO	2020-10-06	135.41	135.63	130.4882	131.08	1451720	{}	\N	2020-10-12 15:57:56.319829
66090	QRVO	2020-10-07	133.85	135.13	133.505	134.64	1146907	{}	\N	2020-10-12 15:57:56.324806
66091	QRVO	2020-10-08	136.1	137.34	134.3	135.41	1146715	{}	\N	2020-10-12 15:57:56.329486
66092	QRVO	2020-10-09	137.98	140.69	136.14	137.87	1296324	{}	\N	2020-10-12 15:57:56.335293
66093	QRVO	2020-10-12	139.4	139.995	136.769	137.79	356212	{}	\N	2020-10-12 15:57:56.340266
66094	QSY	2020-09-28	81.85	83.04	81.85	82.81	1483	{}	\N	2020-10-12 15:57:58.643248
66095	QSY	2020-09-30	83.2882	83.2882	82.242	82.37	1082	{}	\N	2020-10-12 15:57:58.652523
66096	QSY	2020-10-01	83.0847	83.2169	83.0224	83.0224	712	{}	\N	2020-10-12 15:57:58.659337
66097	QSY	2020-10-02	82.3535	83.4802	82.3535	83.4802	1228	{}	\N	2020-10-12 15:57:58.665626
66098	QSY	2020-10-05	84.8242	85.0497	84.765	85.0497	754	{}	\N	2020-10-12 15:57:58.671095
66099	QSY	2020-10-06	85.5	85.79	84.1788	84.1788	2355	{}	\N	2020-10-12 15:57:58.675961
66100	QSY	2020-10-07	85.9614	85.9614	85.9614	85.9614	74	{}	\N	2020-10-12 15:57:58.680686
66101	QSY	2020-10-08	87.1308	87.1308	87.1308	87.1308	9	{}	\N	2020-10-12 15:57:58.686262
66102	QSY	2020-10-09	87.4364	87.4364	87.4364	87.4364	115	{}	\N	2020-10-12 15:57:58.691711
66103	QTRX	2020-09-28	33.09	33.57	32.32	32.93	106539	{}	\N	2020-10-12 15:57:59.94789
66104	QTRX	2020-09-29	32.4877	34.7	32.4877	33.87	156654	{}	\N	2020-10-12 15:57:59.960274
66105	QTRX	2020-09-30	33.95	34.23	33.17	33.74	93939	{}	\N	2020-10-12 15:57:59.967086
66106	QTRX	2020-10-01	33.96	34.82	33.555	34.7	135048	{}	\N	2020-10-12 15:57:59.973053
66107	QTRX	2020-10-02	34.02	35.32	33.63	33.89	141488	{}	\N	2020-10-12 15:57:59.978878
66108	QTRX	2020-10-05	33.9	35.79	33.9	35.35	141111	{}	\N	2020-10-12 15:57:59.98402
66109	QTRX	2020-10-06	36.33	39.86	36	36.9	407175	{}	\N	2020-10-12 15:57:59.989298
66110	QTRX	2020-10-07	37.17	43.98	37.17	43.38	573765	{}	\N	2020-10-12 15:57:59.99511
66111	QTRX	2020-10-08	43.81	46	42.72	45.27	438294	{}	\N	2020-10-12 15:58:00.000153
66112	QTRX	2020-10-09	45.91	48.95	45.1001	46.15	378405	{}	\N	2020-10-12 15:58:00.005322
66113	QTRX	2020-10-12	46.15	47.98	45.06	45.1156	78910	{}	\N	2020-10-12 15:58:00.010763
66114	OMER	2020-09-28	10.22	10.24	9.72	9.95	1440114	{}	\N	2020-10-12 15:58:02.314733
66115	OMER	2020-09-29	9.85	9.98	9.56	9.57	989288	{}	\N	2020-10-12 15:58:02.327069
66116	OMER	2020-09-30	9.7	10.14	9.25	10.105	1439907	{}	\N	2020-10-12 15:58:02.335002
66117	OMER	2020-10-01	10.11	10.56	9.87	10.34	1246000	{}	\N	2020-10-12 15:58:02.341607
66118	OMER	2020-10-02	10.14	10.6	9.99	10.44	425092	{}	\N	2020-10-12 15:58:02.347167
66119	OMER	2020-10-05	10.55	10.94	10.12	10.9	705680	{}	\N	2020-10-12 15:58:02.352292
66120	OMER	2020-10-06	10.95	11.04	10.55	10.6	493662	{}	\N	2020-10-12 15:58:02.35689
66121	OMER	2020-10-07	10.64	11.06	10.56	10.92	712649	{}	\N	2020-10-12 15:58:02.361725
66122	OMER	2020-10-08	11.0064	11.49	11.0064	11.26	875517	{}	\N	2020-10-12 15:58:02.366663
66123	OMER	2020-10-09	11.38	11.49	10.87	11.08	631182	{}	\N	2020-10-12 15:58:02.37148
66124	OMER	2020-10-12	11.04	11.25	10.96	11.07	133401	{}	\N	2020-10-12 15:58:02.376119
66125	OMP	2020-09-28	7.52	7.94	7.51	7.86	71807	{}	\N	2020-10-12 15:58:03.379295
66126	OMP	2020-09-29	7.83	8.4	7.59	8.11	129447	{}	\N	2020-10-12 15:58:03.391784
66127	OMP	2020-09-30	7.62	7.96	5.5	6.2	1722114	{}	\N	2020-10-12 15:58:03.398872
66128	OMP	2020-10-01	6.87	7.19	6.55	7.17	383161	{}	\N	2020-10-12 15:58:03.405169
66129	OMP	2020-10-02	6.75	7.21	6.61	7.13	135985	{}	\N	2020-10-12 15:58:03.410768
66130	OMP	2020-10-05	7.2	7.45	7.021	7.37	127751	{}	\N	2020-10-12 15:58:03.41533
66131	OMP	2020-10-06	7.49	7.61	7.1501	7.3	108576	{}	\N	2020-10-12 15:58:03.419946
66132	OMP	2020-10-07	7.11	7.492	6.97	7.01	144718	{}	\N	2020-10-12 15:58:03.425037
66133	OMP	2020-10-08	7.09	7.46	7.0101	7.37	111354	{}	\N	2020-10-12 15:58:03.429525
66134	OMP	2020-10-09	7.29	7.34	7.11	7.2	58877	{}	\N	2020-10-12 15:58:03.434473
66135	OMP	2020-10-12	7.26	7.26	7.11	7.1844	24496	{}	\N	2020-10-12 15:58:03.43955
66136	OPGN	2020-09-28	2.161	2.28	2.09	2.2	3817970	{}	\N	2020-10-12 15:58:04.401464
66137	OPGN	2020-09-29	2.1137	2.15	2.06	2.12	713175	{}	\N	2020-10-12 15:58:04.413421
66138	OPGN	2020-09-30	2.16	2.19	2.09	2.12	1420035	{}	\N	2020-10-12 15:58:04.421857
66139	OPGN	2020-10-01	2.12	2.16	2.07	2.15	496190	{}	\N	2020-10-12 15:58:04.429382
66140	OPGN	2020-10-02	2.1	2.17	2.0901	2.15	494037	{}	\N	2020-10-12 15:58:04.4361
66141	OPGN	2020-10-05	2.17	2.23	2.1529	2.18	674552	{}	\N	2020-10-12 15:58:04.442214
66142	OPGN	2020-10-06	2.21	2.4	2.17	2.32	2114790	{}	\N	2020-10-12 15:58:04.447795
66143	OPGN	2020-10-07	2.3	2.4	2.24	2.3	1883113	{}	\N	2020-10-12 15:58:04.453228
66144	OPGN	2020-10-08	2.28	2.32	2.23	2.32	542653	{}	\N	2020-10-12 15:58:04.458955
66145	OPGN	2020-10-09	2.33	2.54	2.315	2.47	2504792	{}	\N	2020-10-12 15:58:04.464208
66146	OPGN	2020-10-12	2.48	2.48	2.37	2.46	372306	{}	\N	2020-10-12 15:58:04.469891
66147	OPY	2020-09-28	22.29	22.6	22.22	22.24	112197	{}	\N	2020-10-12 15:58:05.450014
66148	OPY	2020-09-29	22.27	22.36	21.91	22.31	181416	{}	\N	2020-10-12 15:58:05.464049
66149	OPY	2020-09-30	22.3	23	22.215	22.32	329241	{}	\N	2020-10-12 15:58:05.471855
66150	OPY	2020-10-01	22.31	22.6458	22.185	22.29	138413	{}	\N	2020-10-12 15:58:05.478669
66151	OPY	2020-10-02	21.89	22.67	21.89	22.45	99180	{}	\N	2020-10-12 15:58:05.484431
66152	OPY	2020-10-05	22.6	23.04	22.6	22.83	291257	{}	\N	2020-10-12 15:58:05.490012
66153	OPY	2020-10-06	23.14	23.87	23	23.3	465732	{}	\N	2020-10-12 15:58:05.495324
66154	OPY	2020-10-07	23.61	24.51	23.54	24.35	548824	{}	\N	2020-10-12 15:58:05.500345
66155	OPY	2020-10-08	24.57	24.95	24.18	24.55	497954	{}	\N	2020-10-12 15:58:05.505011
66156	OPY	2020-10-09	24.79	24.99	24.43	24.6	227901	{}	\N	2020-10-12 15:58:05.510154
66157	OPY	2020-10-12	24.69	25.625	24.69	25.29	270385	{}	\N	2020-10-12 15:58:05.515066
66158	ORC	2020-09-28	5.2	5.35	5.17	5.26	1772380	{}	\N	2020-10-12 15:58:06.474336
66159	ORC	2020-09-29	5.17	5.22	5.04	5.05	1832040	{}	\N	2020-10-12 15:58:06.484166
66160	ORC	2020-09-30	5.03	5.08	4.99	5.01	1568183	{}	\N	2020-10-12 15:58:06.490566
66161	ORC	2020-10-01	5	5.09	4.93	5.08	2389043	{}	\N	2020-10-12 15:58:06.497441
66162	ORC	2020-10-02	5	5.195	4.95	5.19	2412920	{}	\N	2020-10-12 15:58:06.502923
66163	ORC	2020-10-05	5.15	5.19	5.08	5.13	1088131	{}	\N	2020-10-12 15:58:06.507723
66164	ORC	2020-10-06	5.17	5.2201	5.09	5.09	1486591	{}	\N	2020-10-12 15:58:06.513418
66165	ORC	2020-10-07	5.12	5.143	5.06	5.11	1241169	{}	\N	2020-10-12 15:58:06.517557
66166	ORC	2020-10-08	5.14	5.27	5.14	5.26	2357406	{}	\N	2020-10-12 15:58:06.521719
66167	ORC	2020-10-09	5.28	5.29	5.1486	5.15	1362653	{}	\N	2020-10-12 15:58:06.526555
66168	ORC	2020-10-12	5.16	5.21	5.14	5.1839	392741	{}	\N	2020-10-12 15:58:06.531069
66169	ORMP	2020-09-28	2.85	2.91	2.57	2.68	370051	{}	\N	2020-10-12 15:58:07.76711
66170	ORMP	2020-09-29	2.77	2.77	2.54	2.58	272221	{}	\N	2020-10-12 15:58:07.777666
66171	ORMP	2020-09-30	2.58	2.68	2.58	2.61	90155	{}	\N	2020-10-12 15:58:07.7854
66172	ORMP	2020-10-01	2.61	2.66	2.4	2.54	300987	{}	\N	2020-10-12 15:58:07.792158
66173	ORMP	2020-10-02	2.5	2.623	2.5	2.58	60809	{}	\N	2020-10-12 15:58:07.797503
66174	ORMP	2020-10-05	2.64	2.7	2.613	2.64	41917	{}	\N	2020-10-12 15:58:07.803139
66175	ORMP	2020-10-06	2.64	2.71	2.52	2.57	37257	{}	\N	2020-10-12 15:58:07.808621
66176	ORMP	2020-10-07	2.61	2.7	2.55	2.59	49981	{}	\N	2020-10-12 15:58:07.813756
66177	ORMP	2020-10-08	2.71	2.76	2.63	2.69	58475	{}	\N	2020-10-12 15:58:07.819358
66178	ORMP	2020-10-09	2.7	2.82	2.69	2.7	80805	{}	\N	2020-10-12 15:58:07.824891
66179	ORMP	2020-10-12	2.74	2.74	2.66	2.71	14585	{}	\N	2020-10-12 15:58:07.830664
66180	PAG	2020-09-28	47.18	48.33	47.16	48.18	338107	{}	\N	2020-10-12 15:58:11.49694
66181	PAG	2020-09-29	48.26	48.38	47.35	47.63	465291	{}	\N	2020-10-12 15:58:11.509532
66182	PAG	2020-09-30	47.83	48.49	47.28	47.66	738032	{}	\N	2020-10-12 15:58:11.517722
66183	PAG	2020-10-01	48.07	48.52	46.97	47.97	1068160	{}	\N	2020-10-12 15:58:11.524737
66184	PAG	2020-10-02	46.99	49.94	46.99	49.67	556266	{}	\N	2020-10-12 15:58:11.530881
66185	PAG	2020-10-05	49.89	52.34	49.89	51.99	895946	{}	\N	2020-10-12 15:58:11.535754
66186	PAG	2020-10-06	53.14	54.86	52.85	53.55	1000691	{}	\N	2020-10-12 15:58:11.541223
66187	PAG	2020-10-07	54.49	55.74	54.49	54.75	1164555	{}	\N	2020-10-12 15:58:11.546621
66188	PAG	2020-10-08	55.14	55.7	54.325	55.62	759476	{}	\N	2020-10-12 15:58:11.551679
66189	PAG	2020-10-09	55.94	56.06	55.17	55.41	612068	{}	\N	2020-10-12 15:58:11.556999
66190	PAG	2020-10-12	55.58	56.11	55.3158	55.69	159541	{}	\N	2020-10-12 15:58:11.562189
66191	QVAL	2020-09-28	24.14	24.38	24.14	24.33	6203	{}	\N	2020-10-12 15:58:12.539495
66192	QVAL	2020-09-29	24.4	24.4	24.09	24.225	34240	{}	\N	2020-10-12 15:58:12.54933
66193	QVAL	2020-09-30	24.36	24.6537	24.31	24.3871	7755	{}	\N	2020-10-12 15:58:12.556308
66194	QVAL	2020-10-01	24.6	24.65	24.41	24.6	16608	{}	\N	2020-10-12 15:58:12.563006
66195	QVAL	2020-10-02	24.45	24.894	24.45	24.83	12988	{}	\N	2020-10-12 15:58:12.568395
66196	QVAL	2020-10-05	25.03	25.27	25.03	25.25	19485	{}	\N	2020-10-12 15:58:12.573299
66197	QVAL	2020-10-06	25.43	25.6127	25.095	25.095	10700	{}	\N	2020-10-12 15:58:12.57806
66198	QVAL	2020-10-07	25.37	25.67	25.37	25.665	13687	{}	\N	2020-10-12 15:58:12.582967
66199	QVAL	2020-10-08	25.82	25.89	25.74	25.88	24469	{}	\N	2020-10-12 15:58:12.587928
66200	QVAL	2020-10-09	26.14	26.14	25.87	25.88	18451	{}	\N	2020-10-12 15:58:12.592958
66201	QVAL	2020-10-12	25.93	25.9518	25.8996	25.9518	5140	{}	\N	2020-10-12 15:58:12.598061
66202	RA	2020-09-28	16.61	16.84	16.61	16.7	138843	{}	\N	2020-10-12 15:58:13.71655
66203	RA	2020-09-29	16.72	16.77	16.6	16.66	331740	{}	\N	2020-10-12 15:58:13.727082
66204	RA	2020-09-30	16.64	16.6942	16.5	16.52	730743	{}	\N	2020-10-12 15:58:13.734597
66205	RA	2020-10-01	16.57	16.6399	16.46	16.49	519653	{}	\N	2020-10-12 15:58:13.740746
66206	RA	2020-10-02	16.4	16.5	16.24	16.34	743674	{}	\N	2020-10-12 15:58:13.746357
66207	RA	2020-10-05	16.4	16.6003	16.4	16.58	539347	{}	\N	2020-10-12 15:58:13.751763
66208	RA	2020-10-06	16.58	16.8361	16.58	16.78	488305	{}	\N	2020-10-12 15:58:13.756564
66209	RA	2020-10-07	16.74	16.95	16.74	16.86	571877	{}	\N	2020-10-12 15:58:13.76145
66210	RA	2020-10-08	16.86	17	16.86	16.99	246495	{}	\N	2020-10-12 15:58:13.766781
66211	RA	2020-10-09	17	17.07	16.98	17.04	356928	{}	\N	2020-10-12 15:58:13.771649
66212	RA	2020-10-12	17.04	17.08	17.02	17.07	138422	{}	\N	2020-10-12 15:58:13.776807
66213	RAND	2020-09-28	10.99	11	10.99	10.9925	967	{}	\N	2020-10-12 15:58:15.512693
66214	RAND	2020-09-29	11.0107	11.0447	11	11	921	{}	\N	2020-10-12 15:58:15.521184
66215	RAND	2020-09-30	11.01	11.175	10.9455	11.1	2405	{}	\N	2020-10-12 15:58:15.527041
66216	RAND	2020-10-01	11.28	11.28	10.9	10.9	1398	{}	\N	2020-10-12 15:58:15.531921
66217	RAND	2020-10-02	11.1028	11.1028	11.1028	11.1028	195	{}	\N	2020-10-12 15:58:15.537715
66218	RAND	2020-10-05	11.33	11.33	11.15	11.15	362	{}	\N	2020-10-12 15:58:15.542377
66219	RAND	2020-10-06	11.0333	11.1555	11.0333	11.1555	1320	{}	\N	2020-10-12 15:58:15.546672
66220	RAND	2020-10-07	11.15	11.15	11.15	11.15	823	{}	\N	2020-10-12 15:58:15.551658
66221	RAND	2020-10-08	11.0302	11.18	11.0302	11.18	331	{}	\N	2020-10-12 15:58:15.556173
66222	RBC	2020-09-28	94.25	95.98	93.74	95.88	244721	{}	\N	2020-10-12 15:58:17.282983
66223	RBC	2020-09-29	95.75	97.23	95	95.13	416917	{}	\N	2020-10-12 15:58:17.298347
66224	RBC	2020-09-30	95.85	96.35	92.98	93.87	526080	{}	\N	2020-10-12 15:58:17.3061
66225	RBC	2020-10-01	94.26	94.67	92.52	93.58	470148	{}	\N	2020-10-12 15:58:17.312629
66226	RBC	2020-10-02	91.82	95.18	91.82	93.54	416222	{}	\N	2020-10-12 15:58:17.318245
66227	RBC	2020-10-05	95.14	97.375	95.14	97.33	377047	{}	\N	2020-10-12 15:58:17.323943
66228	RBC	2020-10-06	97.59	100.25	97.255	98.66	710857	{}	\N	2020-10-12 15:58:17.329589
66229	RBC	2020-10-07	99.68	101.55	98.94	99.76	334039	{}	\N	2020-10-12 15:58:17.335017
66230	RBC	2020-10-08	100.38	100.38	98.5	99.72	333882	{}	\N	2020-10-12 15:58:17.340447
66231	RBC	2020-10-09	100.96	101.805	99.87	99.95	292599	{}	\N	2020-10-12 15:58:17.345812
66232	RBC	2020-10-12	101.88	101.95	100.74	101.385	77352	{}	\N	2020-10-12 15:58:17.351343
66233	RCD	2020-09-28	102.68	103.2753	102.66	103.27	4499	{}	\N	2020-10-12 15:58:19.723143
66234	RCD	2020-09-29	102.025	102.2222	101.5874	102.1844	2700	{}	\N	2020-10-12 15:58:19.734963
66235	RCD	2020-09-30	103.1921	103.564	102.1733	102.1733	3714	{}	\N	2020-10-12 15:58:19.742227
66236	RCD	2020-10-01	103.9	104.39	103.08	104.17	171588	{}	\N	2020-10-12 15:58:19.748823
66237	RCD	2020-10-02	102.82	104.785	102.82	104.785	9462	{}	\N	2020-10-12 15:58:19.7544
66238	RCD	2020-10-05	105.63	106.1799	105.63	106.12	3717	{}	\N	2020-10-12 15:58:19.759233
66239	RCD	2020-10-06	106.9028	107.26	104.45	104.6	6617	{}	\N	2020-10-12 15:58:19.76613
66240	RCD	2020-10-07	106.04	107.08	106.04	107.07	2400	{}	\N	2020-10-12 15:58:19.771296
66241	RCD	2020-10-08	107.8052	108.229	107.25	108.229	11445	{}	\N	2020-10-12 15:58:19.776322
66242	RCD	2020-10-09	108.4165	109.02	108.25	108.45	12196	{}	\N	2020-10-12 15:58:19.781648
66243	RCD	2020-10-12	108.56	109.03	108.56	108.87	4598	{}	\N	2020-10-12 15:58:19.786603
66244	RCMT	2020-09-28	1.4	1.4	1.36	1.37	29184	{}	\N	2020-10-12 15:58:21.151677
66245	RCMT	2020-09-29	1.38	1.4	1.36	1.37	5640	{}	\N	2020-10-12 15:58:21.164257
66246	RCMT	2020-09-30	1.37	1.55	1.36	1.4099	365216	{}	\N	2020-10-12 15:58:21.170945
66247	RCMT	2020-10-01	1.5	1.5	1.29	1.32	252070	{}	\N	2020-10-12 15:58:21.176716
66248	RCMT	2020-10-02	1.33	1.34	1.27	1.34	112698	{}	\N	2020-10-12 15:58:21.181949
66249	RCMT	2020-10-05	1.31	1.3101	1.25	1.26	82224	{}	\N	2020-10-12 15:58:21.186222
66250	RCMT	2020-10-06	1.275	1.95	1.26	1.87	4105227	{}	\N	2020-10-12 15:58:21.190045
66251	RCMT	2020-10-07	1.46	1.52	1.35	1.48	1756746	{}	\N	2020-10-12 15:58:21.193565
66252	RCMT	2020-10-08	1.41	1.465	1.37	1.42	237099	{}	\N	2020-10-12 15:58:21.197531
66253	RCMT	2020-10-09	1.38	1.48	1.36	1.43	247645	{}	\N	2020-10-12 15:58:21.201853
66254	RCMT	2020-10-12	1.38	1.41	1.3602	1.385	118139	{}	\N	2020-10-12 15:58:21.20623
66255	RDIB	2020-09-28	24.02	25.8	22.364	23.6639	9701	{}	\N	2020-10-12 15:58:22.278054
66256	RDIB	2020-09-29	26.5	26.5	23.8	25.3272	4160	{}	\N	2020-10-12 15:58:22.288631
66257	RDIB	2020-09-30	24.23	25.04	23.9	24.3	1747	{}	\N	2020-10-12 15:58:22.295403
66258	RDIB	2020-10-01	25.56	25.56	24	24	1397	{}	\N	2020-10-12 15:58:22.30192
66259	RDIB	2020-10-02	22.54	24.79	22.54	24.5	1970	{}	\N	2020-10-12 15:58:22.307296
66260	RDIB	2020-10-05	24.49	25.4993	24.1	24.1	1805	{}	\N	2020-10-12 15:58:22.312377
66261	RDIB	2020-10-06	22.76	24	22.76	23.24	1815	{}	\N	2020-10-12 15:58:22.317029
66262	RDIB	2020-10-07	24	24	23.4153	23.4153	568	{}	\N	2020-10-12 15:58:22.321554
66263	RDIB	2020-10-08	23.6094	23.6094	23	23	808	{}	\N	2020-10-12 15:58:22.326408
66264	RDIB	2020-10-09	23.4999	23.4999	23.4999	23.4999	621	{}	\N	2020-10-12 15:58:22.331013
66265	RDIB	2020-10-12	20.8215	22.9	20.8215	22.9	498	{}	\N	2020-10-12 15:58:22.335857
66266	RE	2020-09-28	201.76	204.395	201.01	201.6	152420	{}	\N	2020-10-12 15:58:23.718955
66267	RE	2020-09-29	200.33	201.95	197.31	199.78	360080	{}	\N	2020-10-12 15:58:23.728152
66268	RE	2020-09-30	201.01	201.75	194.54	197.54	904104	{}	\N	2020-10-12 15:58:23.733701
66269	RE	2020-10-01	197.59	200.775	194.75	196.93	659127	{}	\N	2020-10-12 15:58:23.739988
66270	RE	2020-10-02	195.3	199.91	193.025	198.66	401446	{}	\N	2020-10-12 15:58:23.74553
66271	RE	2020-10-05	200.68	204.105	200.205	203.98	510288	{}	\N	2020-10-12 15:58:23.750092
66272	RE	2020-10-06	206.43	206.49	199.2	199.82	360871	{}	\N	2020-10-12 15:58:23.754632
66273	RE	2020-10-07	200.45	201.72	194.87	198.33	568051	{}	\N	2020-10-12 15:58:23.759345
66274	RE	2020-10-08	198.96	204.99	198.96	204.09	436033	{}	\N	2020-10-12 15:58:23.763623
66275	RE	2020-10-09	206.22	206.575	203.22	204.94	371454	{}	\N	2020-10-12 15:58:23.768095
66276	RE	2020-10-12	205.18	207.47	204.34	206.97	127849	{}	\N	2020-10-12 15:58:23.772841
66277	REGI	2020-09-28	46.16	47.85	45.3	46.83	960752	{}	\N	2020-10-12 15:58:24.928756
66278	REGI	2020-09-29	46.86	50.32	46.09	50.02	1852109	{}	\N	2020-10-12 15:58:24.93976
66279	REGI	2020-09-30	50.74	54.74	50.35	53.42	2068007	{}	\N	2020-10-12 15:58:24.948724
66280	REGI	2020-10-01	54.03	54.9898	52.73	53.93	1006672	{}	\N	2020-10-12 15:58:24.955648
66281	REGI	2020-10-02	52.35	56.2999	51.105	55.7	1195607	{}	\N	2020-10-12 15:58:24.961734
66282	REGI	2020-10-05	57.5	58.42	56.21	57.08	1205083	{}	\N	2020-10-12 15:58:24.96619
66283	REGI	2020-10-06	57.41	59.5063	55.15	56.41	1313464	{}	\N	2020-10-12 15:58:24.970949
66284	REGI	2020-10-07	57.44	61.2891	56.7485	59.73	1633209	{}	\N	2020-10-12 15:58:24.975757
66285	REGI	2020-10-08	60.85	62.29	59.25	60.16	1348315	{}	\N	2020-10-12 15:58:24.980515
66286	REGI	2020-10-09	60.7	63.78	60.25	63.27	1164920	{}	\N	2020-10-12 15:58:24.985078
66287	REGI	2020-10-12	65.21	65.654	62.12	62.33	978438	{}	\N	2020-10-12 15:58:24.989532
66288	REV	2020-09-28	6.72	6.7476	6.48	6.73	158231	{}	\N	2020-10-12 15:58:26.714206
66289	REV	2020-09-29	6.6	6.735	6.49	6.56	206358	{}	\N	2020-10-12 15:58:26.726135
66290	REV	2020-09-30	6.63	6.6351	6.21	6.32	834905	{}	\N	2020-10-12 15:58:26.734378
66291	REV	2020-10-01	6.33	6.79	6.33	6.77	313340	{}	\N	2020-10-12 15:58:26.74039
66292	REV	2020-10-02	6.6	6.85	6.45	6.62	243257	{}	\N	2020-10-12 15:58:26.746353
66293	REV	2020-10-05	6.68	6.78	6.6	6.66	143193	{}	\N	2020-10-12 15:58:26.751456
66294	REV	2020-10-06	6.82	6.94	6.43	6.46	288862	{}	\N	2020-10-12 15:58:26.756256
66295	REV	2020-10-07	6.52	6.7099	6.3	6.32	358172	{}	\N	2020-10-12 15:58:26.761675
66296	REV	2020-10-08	6.44	6.4999	6.26	6.38	417723	{}	\N	2020-10-12 15:58:26.767024
66297	REV	2020-10-09	6.42	6.42	6.14	6.15	566493	{}	\N	2020-10-12 15:58:26.772226
66298	REV	2020-10-12	6.15	6.43	6.15	6.36	187816	{}	\N	2020-10-12 15:58:26.778559
66299	RFCI	2020-09-28	26.13	26.1897	26.13	26.1897	767	{}	\N	2020-10-12 15:58:29.347399
66300	RFCI	2020-09-29	26.1977	26.2697	26.1977	26.245	2224	{}	\N	2020-10-12 15:58:29.358378
66301	RFCI	2020-09-30	26.165	26.165	26.04	26.12	5935	{}	\N	2020-10-12 15:58:29.366178
66302	RFCI	2020-10-01	26.19	26.21	26.14	26.14	10896	{}	\N	2020-10-12 15:58:29.373172
66303	RFCI	2020-10-02	26.2	26.2	26.165	26.165	382	{}	\N	2020-10-12 15:58:29.378501
66304	RFCI	2020-10-05	26.1232	26.1232	26.095	26.095	508	{}	\N	2020-10-12 15:58:29.383655
66305	RFCI	2020-10-06	26.12	26.13	26.12	26.13	211	{}	\N	2020-10-12 15:58:29.3889
66306	RFCI	2020-10-07	26.072	26.105	26.02	26.0625	8880	{}	\N	2020-10-12 15:58:29.394386
66307	RFCI	2020-10-08	26.0881	26.0881	25.99	26.06	15606	{}	\N	2020-10-12 15:58:29.399231
66308	RFCI	2020-10-09	26.16	26.17	26.07	26.07	8499	{}	\N	2020-10-12 15:58:29.404369
66309	RFCI	2020-10-12	26.1834	26.1834	26.1486	26.1486	502	{}	\N	2020-10-12 15:58:29.409117
66310	R	2020-09-28	41.86	42.3	41.49	41.91	420319	{}	\N	2020-10-12 15:58:30.986702
66311	R	2020-09-29	41.93	42.4	41.045	41.08	553579	{}	\N	2020-10-12 15:58:30.997928
66312	R	2020-09-30	41.12	42.615	41.12	42.24	1312563	{}	\N	2020-10-12 15:58:31.00518
66313	R	2020-10-01	42.35	42.97	41.76	42.14	714401	{}	\N	2020-10-12 15:58:31.011515
66314	R	2020-10-02	41.27	44.11	41.27	44.06	725049	{}	\N	2020-10-12 15:58:31.018132
66315	R	2020-10-05	44.5	45.99	44.33	45.93	736672	{}	\N	2020-10-12 15:58:31.022897
66316	R	2020-10-06	46.49	48.29	45.88	46.03	1528245	{}	\N	2020-10-12 15:58:31.027832
66317	R	2020-10-07	47.01	48.31	46.605	47.39	781254	{}	\N	2020-10-12 15:58:31.032876
66318	R	2020-10-08	48	48	46.75	47.25	728147	{}	\N	2020-10-12 15:58:31.037539
66319	R	2020-10-09	48.88	49.32	46.67	47.24	1365088	{}	\N	2020-10-12 15:58:31.042735
66320	R	2020-10-12	47.56	49.555	47.38	49	331997	{}	\N	2020-10-12 15:58:31.048136
66321	RAVE	2020-09-28	0.45	0.47	0.4173	0.46	335884	{}	\N	2020-10-12 15:58:32.212682
66322	RAVE	2020-09-29	0.475	0.52	0.43	0.475	599389	{}	\N	2020-10-12 15:58:32.225173
66323	RAVE	2020-09-30	0.4949	0.4987	0.4306	0.4436	186889	{}	\N	2020-10-12 15:58:32.23491
66324	RAVE	2020-10-01	0.51	0.51	0.44	0.49	306068	{}	\N	2020-10-12 15:58:32.240931
66325	RAVE	2020-10-02	0.4205	0.475	0.4205	0.472	96943	{}	\N	2020-10-12 15:58:32.246787
66326	RAVE	2020-10-05	0.4747	0.5	0.4504	0.472	178494	{}	\N	2020-10-12 15:58:32.251895
66327	RAVE	2020-10-06	0.4857	0.5679	0.470201	0.514	2222626	{}	\N	2020-10-12 15:58:32.256497
66328	RAVE	2020-10-07	1.981	2.36	0.92	1.17	68148804	{}	\N	2020-10-12 15:58:32.2614
66329	RAVE	2020-10-08	1.0305	1.1	0.9	0.9917	4671579	{}	\N	2020-10-12 15:58:32.26628
66330	RAVE	2020-10-09	0.897	0.9072	0.8023	0.8113	2089502	{}	\N	2020-10-12 15:58:32.271382
66331	RAVE	2020-10-12	0.7923	0.81	0.7613	0.800801	327447	{}	\N	2020-10-12 15:58:32.276218
66332	RBNC	2020-09-28	13.5	14.34	13.5	13.83	16945	{}	\N	2020-10-12 15:58:33.238875
66333	RBNC	2020-09-29	13.87	14.1	13.53	14.1	16869	{}	\N	2020-10-12 15:58:33.248099
66334	RBNC	2020-09-30	14.13	14.56	14.0754	14.5	50171	{}	\N	2020-10-12 15:58:33.256802
66335	RBNC	2020-10-01	14.52	14.6	14.28	14.52	84207	{}	\N	2020-10-12 15:58:33.264622
66336	RBNC	2020-10-02	14.26	14.99	14.26	14.85	33551	{}	\N	2020-10-12 15:58:33.271431
66337	RBNC	2020-10-05	14.85	15.04	14.75	14.82	19900	{}	\N	2020-10-12 15:58:33.277542
66338	RBNC	2020-10-06	14.79	15.42	14.66	14.73	38019	{}	\N	2020-10-12 15:58:33.283943
66339	RBNC	2020-10-07	14.775	15.25	14.66	15.25	41788	{}	\N	2020-10-12 15:58:33.28989
66340	RBNC	2020-10-08	15.33	15.5	15.25	15.37	23502	{}	\N	2020-10-12 15:58:33.295245
66341	RBNC	2020-10-09	14.105	15.45	14.105	15.37	22627	{}	\N	2020-10-12 15:58:33.300786
66342	RBNC	2020-10-12	15.25	15.36	15.25	15.36	1098	{}	\N	2020-10-12 15:58:33.306583
66343	RCKT	2020-09-28	23.17	23.17	22.035	22.51	183811	{}	\N	2020-10-12 15:58:34.480834
66344	RCKT	2020-09-29	22.47	23.195	22.25	23.09	226384	{}	\N	2020-10-12 15:58:34.493366
66345	RCKT	2020-09-30	23.24	23.4842	22.77	22.86	176752	{}	\N	2020-10-12 15:58:34.499507
66346	RCKT	2020-10-01	22.87	23.95	22.79	23.64	226243	{}	\N	2020-10-12 15:58:34.506353
66347	RCKT	2020-10-02	22.99	24.37	22.85	23.305	160406	{}	\N	2020-10-12 15:58:34.511784
66348	RCKT	2020-10-05	23.55	25.18	23.55	25.06	213492	{}	\N	2020-10-12 15:58:34.516703
66349	RCKT	2020-10-06	25.28	26.99	24.14	25.6	630016	{}	\N	2020-10-12 15:58:34.521635
66350	RCKT	2020-10-07	25.87	27.12	25.58	26.92	354232	{}	\N	2020-10-12 15:58:34.526414
66351	RCKT	2020-10-08	27.1	27.63	26.8	27.62	370317	{}	\N	2020-10-12 15:58:34.530931
66352	RCKT	2020-10-09	27.91	28.95	27.64	28.51	328715	{}	\N	2020-10-12 15:58:34.535537
66353	RCKT	2020-10-12	28.39	28.71	27.1591	28.535	53588	{}	\N	2020-10-12 15:58:34.540232
66354	RDIV	2020-09-28	26.94	27.4199	26.94	27.19	98262	{}	\N	2020-10-12 15:58:35.633352
66355	RDIV	2020-09-29	27.13	27.13	26.54	26.735	103189	{}	\N	2020-10-12 15:58:35.64657
66356	RDIV	2020-09-30	26.84	27.22	26.7476	26.89	124072	{}	\N	2020-10-12 15:58:35.653487
66357	RDIV	2020-10-01	26.87	27.01	26.3981	26.56	128291	{}	\N	2020-10-12 15:58:35.660488
66358	RDIV	2020-10-02	26.03	27.2063	26.03	27.07	75084	{}	\N	2020-10-12 15:58:35.666634
66359	RDIV	2020-10-05	27.31	27.88	27.31	27.85	178569	{}	\N	2020-10-12 15:58:35.672542
66360	RDIV	2020-10-06	28.02	28.51	27.5968	27.65	138093	{}	\N	2020-10-12 15:58:35.677947
66361	RDIV	2020-10-07	27.86	28.3115	27.86	28.22	102711	{}	\N	2020-10-12 15:58:35.68317
66362	RDIV	2020-10-08	28.44	28.9	28.43	28.88	121722	{}	\N	2020-10-12 15:58:35.688583
66363	RDIV	2020-10-09	29.05	29.06	28.6	28.61	120287	{}	\N	2020-10-12 15:58:35.694203
66364	RDIV	2020-10-12	28.65	28.8231	28.59	28.7978	25633	{}	\N	2020-10-12 15:58:35.69955
66365	REK	2020-09-28	12.45	12.45	12.291	12.3601	21659	{}	\N	2020-10-12 15:58:38.930786
66366	REK	2020-09-29	12.35	12.525	12.35	12.44	3685	{}	\N	2020-10-12 15:58:38.943477
66367	REK	2020-09-30	12.37	12.52	12.27	12.39	12054	{}	\N	2020-10-12 15:58:38.950787
66368	REK	2020-10-01	12.38	12.4108	12.15	12.155	23183	{}	\N	2020-10-12 15:58:38.95738
66369	REK	2020-10-02	12.31	12.35	11.95	11.965	21102	{}	\N	2020-10-12 15:58:38.963653
66370	REK	2020-10-05	11.94	12.105	11.91	11.92	253425	{}	\N	2020-10-12 15:58:38.968903
66371	REK	2020-10-06	11.85	12.05	11.81	12	16133	{}	\N	2020-10-12 15:58:38.974368
66372	REK	2020-10-07	11.9365	12	11.9	11.93	10181	{}	\N	2020-10-12 15:58:38.980061
66373	REK	2020-10-08	11.89	11.89	11.73	11.78	11256	{}	\N	2020-10-12 15:58:38.98624
66374	REK	2020-10-09	11.749	11.84	11.74	11.81	8971	{}	\N	2020-10-12 15:58:38.991609
66375	REK	2020-10-12	11.83	11.85	11.76	11.76	20751	{}	\N	2020-10-12 15:58:38.997114
66376	RES	2020-09-28	2.58	2.73	2.56	2.66	834910	{}	\N	2020-10-12 15:58:39.953692
66377	RES	2020-09-29	2.64	2.7	2.51	2.69	1493422	{}	\N	2020-10-12 15:58:39.963658
66378	RES	2020-09-30	2.69	2.77	2.59	2.64	1462616	{}	\N	2020-10-12 15:58:39.970419
66379	RES	2020-10-01	2.6	2.72	2.56	2.71	3483976	{}	\N	2020-10-12 15:58:39.975913
66380	RES	2020-10-02	2.6	2.84	2.6	2.78	1031353	{}	\N	2020-10-12 15:58:39.981453
66381	RES	2020-10-05	2.8	2.89	2.79	2.85	900775	{}	\N	2020-10-12 15:58:39.986371
66382	RES	2020-10-06	2.89	2.95	2.745	2.79	1088242	{}	\N	2020-10-12 15:58:39.990957
66383	RES	2020-10-07	2.81	2.855	2.75	2.82	880440	{}	\N	2020-10-12 15:58:39.995507
66384	RES	2020-10-08	2.87	2.94	2.86	2.88	1295595	{}	\N	2020-10-12 15:58:39.999952
66385	RES	2020-10-09	2.96	2.98	2.81	2.82	1437636	{}	\N	2020-10-12 15:58:40.004188
66386	RES	2020-10-12	2.8	2.825	2.65	2.73	869762	{}	\N	2020-10-12 15:58:40.008674
66387	REX	2020-09-28	62.06	64.97	61.73	64.36	24338	{}	\N	2020-10-12 15:58:41.003611
66388	REX	2020-09-29	63.24	66.4741	63.24	66.16	62115	{}	\N	2020-10-12 15:58:41.017297
66389	REX	2020-09-30	66.01	67.6	65.325	65.61	65154	{}	\N	2020-10-12 15:58:41.026141
66390	REX	2020-10-01	65.23	67.03	64.54	66.26	56874	{}	\N	2020-10-12 15:58:41.032436
66391	REX	2020-10-02	64.96	68.54	64.39	67.34	30029	{}	\N	2020-10-12 15:58:41.039167
66392	REX	2020-10-05	68.54	70.69	67.55	69.35	33600	{}	\N	2020-10-12 15:58:41.044891
66393	REX	2020-10-06	70.38	71.5	68.7	70.14	73679	{}	\N	2020-10-12 15:58:41.050037
66394	REX	2020-10-07	70.36	70.895	69.3	69.68	42290	{}	\N	2020-10-12 15:58:41.055487
66395	REX	2020-10-08	70.8	72.29	70	72.19	29135	{}	\N	2020-10-12 15:58:41.060897
66396	REX	2020-10-09	72.91	72.91	70.9688	72.69	34772	{}	\N	2020-10-12 15:58:41.066562
66397	REX	2020-10-12	71.98	71.98	71.98	71.98	2804	{}	\N	2020-10-12 15:58:41.072266
66398	RFUN	2020-09-28	25.055	25.055	25.055	25.055	34	{}	\N	2020-10-12 15:58:42.645717
66399	RFUN	2020-09-29	25.06	25.06	25.06	25.06	7	{}	\N	2020-10-12 15:58:42.655877
66400	RFUN	2020-09-30	25.05	25.1199	25.05	25.1199	636	{}	\N	2020-10-12 15:58:42.662586
66401	RFUN	2020-10-01	25.12	25.25	25.1	25.1	4574	{}	\N	2020-10-12 15:58:42.668792
66402	RFUN	2020-10-02	25.145	25.145	25.145	25.145	20	{}	\N	2020-10-12 15:58:42.674469
66403	RFUN	2020-10-05	25.18	25.28	25.18	25.2244	1832	{}	\N	2020-10-12 15:58:42.680352
66404	RFUN	2020-10-06	25.2412	25.2412	25.2412	25.2412	214	{}	\N	2020-10-12 15:58:42.685412
66405	RFUN	2020-10-07	25.25	25.39	25.25	25.39	3167	{}	\N	2020-10-12 15:58:42.690724
66406	RFUN	2020-10-08	25.315	25.3308	25.305	25.3308	756	{}	\N	2020-10-12 15:58:42.696599
66407	RFUN	2020-10-09	25.37	25.37	25.37	25.37	389	{}	\N	2020-10-12 15:58:42.701877
66408	RGT	2020-09-28	11.9	12.08	11.8268	12.08	8987	{}	\N	2020-10-12 15:58:43.656748
66409	RGT	2020-09-29	12.03	12.25	12.03	12.25	48552	{}	\N	2020-10-12 15:58:43.669352
66410	RGT	2020-09-30	12.2	12.28	12.16	12.28	15118	{}	\N	2020-10-12 15:58:43.675865
66411	RGT	2020-10-01	12.3	12.46	12.3	12.45	12984	{}	\N	2020-10-12 15:58:43.68219
66412	RGT	2020-10-02	12.33	12.48	12.33	12.48	5986	{}	\N	2020-10-12 15:58:43.688062
66413	RGT	2020-10-05	12.45	12.58	12.45	12.5	12808	{}	\N	2020-10-12 15:58:43.692942
66414	RGT	2020-10-06	12.58	12.64	12.22	12.27	32590	{}	\N	2020-10-12 15:58:43.698375
66415	RGT	2020-10-07	12.37	12.5673	12.3401	12.36	29068	{}	\N	2020-10-12 15:58:43.703239
66416	RGT	2020-10-08	12.4	12.5	12.235	12.5	39024	{}	\N	2020-10-12 15:58:43.707774
66417	RGT	2020-10-09	12.53	12.68	12.53	12.65	33470	{}	\N	2020-10-12 15:58:43.712698
66418	RGT	2020-10-12	13.26	13.6	13.25	13.53	191052	{}	\N	2020-10-12 15:58:43.717463
66419	RACE	2020-09-28	183.86	184.07	182.94	183.28	126711	{}	\N	2020-10-12 15:58:45.117226
66420	RACE	2020-09-29	183.63	184.27	182.1	182.35	244410	{}	\N	2020-10-12 15:58:45.127193
66421	RACE	2020-09-30	182.85	185.195	182.35	184.09	493428	{}	\N	2020-10-12 15:58:45.134954
66422	RACE	2020-10-01	184.49	185.08	182.92	184.6	357900	{}	\N	2020-10-12 15:58:45.140935
66423	RACE	2020-10-02	180.59	182.465	180.555	180.87	366257	{}	\N	2020-10-12 15:58:45.146553
66424	RACE	2020-10-05	181.25	182.7499	180.64	182.63	267739	{}	\N	2020-10-12 15:58:45.152057
66425	RACE	2020-10-06	181.29	181.49	176.03	176.49	682080	{}	\N	2020-10-12 15:58:45.156775
66426	RACE	2020-10-07	180.54	184.61	180.48	183.4	875113	{}	\N	2020-10-12 15:58:45.161536
66427	RACE	2020-10-08	182.28	182.83	180.97	181.32	436746	{}	\N	2020-10-12 15:58:45.166528
66428	RACE	2020-10-09	181.79	183.4	181.22	182.84	282837	{}	\N	2020-10-12 15:58:45.171343
66429	RACE	2020-10-12	184.74	185.59	184.205	184.26	156468	{}	\N	2020-10-12 15:58:45.17635
66430	RBCN	2020-09-28	8.641	8.641	8.51	8.51	1683	{}	\N	2020-10-12 15:58:46.702504
66431	RBCN	2020-09-29	8.53	8.53	8.53	8.53	315	{}	\N	2020-10-12 15:58:46.714543
66432	RBCN	2020-09-30	8.53	8.5321	8.5	8.5	5095	{}	\N	2020-10-12 15:58:46.722342
66433	RBCN	2020-10-01	8.51	8.66	8.5	8.66	2642	{}	\N	2020-10-12 15:58:46.727969
66434	RBCN	2020-10-02	8.69	8.69	8.645	8.65	1521	{}	\N	2020-10-12 15:58:46.733963
66435	RBCN	2020-10-05	8.69	8.7	8.53	8.6	1084	{}	\N	2020-10-12 15:58:46.73945
66436	RBCN	2020-10-06	8.68	8.7	8.54	8.54	1099	{}	\N	2020-10-12 15:58:46.744479
66437	RBCN	2020-10-07	8.574	8.59	8.55	8.59	1423	{}	\N	2020-10-12 15:58:46.74962
66438	RBCN	2020-10-08	8.601	8.6245	8.59	8.62	1080	{}	\N	2020-10-12 15:58:46.755031
66439	RBCN	2020-10-09	8.62	8.655	8.62	8.62	439	{}	\N	2020-10-12 15:58:46.760175
66440	RBCN	2020-10-12	8.67	8.67	8.52	8.52	1474	{}	\N	2020-10-12 15:58:46.765339
66441	RCII	2020-09-28	30.3	30.76	29.82	30.44	611290	{}	\N	2020-10-12 15:58:47.840151
66442	RCII	2020-09-29	30.34	30.69	29.61	30.25	447263	{}	\N	2020-10-12 15:58:47.850655
66443	RCII	2020-09-30	30.37	30.72	29.88	29.89	495543	{}	\N	2020-10-12 15:58:47.858039
66444	RCII	2020-10-01	30.02	30.485	29.47	30.27	477796	{}	\N	2020-10-12 15:58:47.863567
66445	RCII	2020-10-02	29.46	31.03	29.41	31	569741	{}	\N	2020-10-12 15:58:47.869324
66446	RCII	2020-10-05	31.28	33.03	31.28	32.83	662438	{}	\N	2020-10-12 15:58:47.87484
66447	RCII	2020-10-06	32.83	32.99	31.8	31.92	505938	{}	\N	2020-10-12 15:58:47.879626
66448	RCII	2020-10-07	32.21	32.75	32.2	32.53	481412	{}	\N	2020-10-12 15:58:47.88534
66449	RCII	2020-10-08	32.78	33.15	32.37	32.83	498158	{}	\N	2020-10-12 15:58:47.890398
66450	RCII	2020-10-09	32.99	34.3	32.655	34.24	567915	{}	\N	2020-10-12 15:58:47.895657
66451	RCII	2020-10-12	34.53	35	33.6394	34.5	247292	{}	\N	2020-10-12 15:58:47.901268
66452	RDHL	2020-09-28	10.1	10.16	9.92	9.98	130115	{}	\N	2020-10-12 15:58:50.085263
66453	RDHL	2020-09-29	9.98	10.34	9.93	10.23	107447	{}	\N	2020-10-12 15:58:50.098137
66454	RDHL	2020-09-30	10.22	10.6197	10.08	10.22	183327	{}	\N	2020-10-12 15:58:50.106763
66455	RDHL	2020-10-01	10.18	10.279	9.9442	10.19	135261	{}	\N	2020-10-12 15:58:50.113369
66456	RDHL	2020-10-02	10.07	10.27	9.997	10.17	99545	{}	\N	2020-10-12 15:58:50.119347
66457	RDHL	2020-10-05	10.17	10.535	10.17	10.29	186923	{}	\N	2020-10-12 15:58:50.12425
66458	RDHL	2020-10-06	10.3	10.3	9.36	9.45	333729	{}	\N	2020-10-12 15:58:50.129551
66459	RDHL	2020-10-07	9.51	9.94	9.4	9.57	411532	{}	\N	2020-10-12 15:58:50.134696
66460	RDHL	2020-10-08	9.63	10.055	9.54	9.71	219087	{}	\N	2020-10-12 15:58:50.139763
66461	RDHL	2020-10-09	9.86	10.2	9.8	9.93	240144	{}	\N	2020-10-12 15:58:50.145024
66462	RDHL	2020-10-12	10	10.27	9.7225	10.164	127349	{}	\N	2020-10-12 15:58:50.150518
66463	REGL	2020-09-28	52.92	53.32	52.81	53.09	53751	{}	\N	2020-10-12 15:58:52.858252
66464	REGL	2020-09-29	53.14	53.2199	52.4069	52.78	74161	{}	\N	2020-10-12 15:58:52.873091
66465	REGL	2020-09-30	53.01	53.4183	52.68	53.07	40652	{}	\N	2020-10-12 15:58:52.880671
66466	REGL	2020-10-01	53.15	53.3253	52.795	53.18	42082	{}	\N	2020-10-12 15:58:52.888685
66467	REGL	2020-10-02	52.69	54.1984	52.48	53.97	35331	{}	\N	2020-10-12 15:58:52.89522
66468	REGL	2020-10-05	54.33	54.99	54.33	54.99	43456	{}	\N	2020-10-12 15:58:52.900394
66469	REGL	2020-10-06	55.34	55.995	54.8101	54.9	30526	{}	\N	2020-10-12 15:58:52.905738
66470	REGL	2020-10-07	55.3395	55.5827	55.0509	55.47	41869	{}	\N	2020-10-12 15:58:52.911446
66471	REGL	2020-10-08	56.25	56.53	56.12	56.53	49454	{}	\N	2020-10-12 15:58:52.916743
66472	REGL	2020-10-09	56.77	56.96	56.38	56.415	64340	{}	\N	2020-10-12 15:58:52.922227
66473	REGL	2020-10-12	56.76	57.1432	56.565	57.1432	13642	{}	\N	2020-10-12 15:58:52.927319
66474	REMX	2020-09-28	37.4	37.72	37.0401	37.46	385842	{}	\N	2020-10-12 15:58:53.887271
66475	REMX	2020-09-29	37.6	37.8299	37.17	37.65	183613	{}	\N	2020-10-12 15:58:53.89499
66476	REMX	2020-09-30	37.65	38.24	37.4	37.71	457545	{}	\N	2020-10-12 15:58:53.901551
66477	REMX	2020-10-01	38.47	38.4999	38.06	38.15	262477	{}	\N	2020-10-12 15:58:53.907303
66478	REMX	2020-10-02	36.87	37.99	36.5	37.5	527333	{}	\N	2020-10-12 15:58:53.913264
66479	REMX	2020-10-05	38.48	39.59	38.47	39.45	401289	{}	\N	2020-10-12 15:58:53.920155
66480	REMX	2020-10-06	40.24	40.2499	39.18	39.36	236309	{}	\N	2020-10-12 15:58:53.927342
66481	REMX	2020-10-07	39.53	40.2	39.53	40.0482	201238	{}	\N	2020-10-12 15:58:53.934555
66482	REMX	2020-10-08	40.39	40.9	40.27	40.66	111244	{}	\N	2020-10-12 15:58:53.941612
66483	REMX	2020-10-09	41.94	42.1499	41.61	41.82	138821	{}	\N	2020-10-12 15:58:53.947648
66484	REMX	2020-10-12	42.24	42.36	41.76	41.81	37002	{}	\N	2020-10-12 15:58:53.952921
66485	RESN	2020-09-28	2.29	2.49	2.26	2.46	980788	{}	\N	2020-10-12 15:58:55.868
66486	RESN	2020-09-29	2.47	2.8	2.38	2.71	2793071	{}	\N	2020-10-12 15:58:55.881157
66487	RESN	2020-09-30	2.76	2.7601	2.37	2.38	1933520	{}	\N	2020-10-12 15:58:55.890372
66488	RESN	2020-10-01	2.49	2.5	2.3	2.48	973071	{}	\N	2020-10-12 15:58:55.898789
66489	RESN	2020-10-02	2.37	2.4399	2.305	2.36	666579	{}	\N	2020-10-12 15:58:55.906605
66490	RESN	2020-10-05	2.36	2.52	2.36	2.47	678055	{}	\N	2020-10-12 15:58:55.913701
66491	RESN	2020-10-06	2.5	2.72	2.4	2.42	3791997	{}	\N	2020-10-12 15:58:55.921706
66492	RESN	2020-10-07	2.61	2.64	2.46	2.57	1306481	{}	\N	2020-10-12 15:58:55.929414
66493	RESN	2020-10-08	2.6	2.69	2.53	2.59	987784	{}	\N	2020-10-12 15:58:55.936408
66494	RESN	2020-10-09	2.63	2.71	2.58	2.61	1077076	{}	\N	2020-10-12 15:58:55.94163
66495	RESN	2020-10-12	2.64	2.8	2.63	2.695	797023	{}	\N	2020-10-12 15:58:55.946661
66496	REXR	2020-09-28	45.35	46.41	45.09	46.41	830501	{}	\N	2020-10-12 15:58:57.026363
66497	REXR	2020-09-29	46.14	46.41	45.47	45.89	1238628	{}	\N	2020-10-12 15:58:57.037328
66498	REXR	2020-09-30	46.32	46.5	44.975	45.76	2007634	{}	\N	2020-10-12 15:58:57.045529
66499	REXR	2020-10-01	45.96	47.13	45.37	47.05	1675628	{}	\N	2020-10-12 15:58:57.051933
66500	REXR	2020-10-02	46.36	48.49	45.99	48.37	1654211	{}	\N	2020-10-12 15:58:57.058185
66501	REXR	2020-10-05	48.53	49.515	47.98	49.32	1722875	{}	\N	2020-10-12 15:58:57.063787
66502	REXR	2020-10-06	49.57	50.53	48.97	49.72	1678207	{}	\N	2020-10-12 15:58:57.06965
66503	REXR	2020-10-07	49.92	50.09	49.55	49.73	1309226	{}	\N	2020-10-12 15:58:57.075019
66504	REXR	2020-10-08	50.09	51	49.86	50.91	1220432	{}	\N	2020-10-12 15:58:57.080106
66505	REXR	2020-10-09	51.29	51.72	50.83	51.37	1051793	{}	\N	2020-10-12 15:58:57.085545
66506	REXR	2020-10-12	51.69	52.12	51.15	51.97	112686	{}	\N	2020-10-12 15:58:57.091154
66507	RFG	2020-09-28	165.075	167	165.075	166.88	3412	{}	\N	2020-10-12 15:58:59.244137
66508	RFG	2020-09-29	166.54	167.79	166.54	167.565	1253	{}	\N	2020-10-12 15:58:59.256267
66509	RFG	2020-09-30	168.86	169.685	168.4792	168.4792	1102	{}	\N	2020-10-12 15:58:59.264323
66510	RFG	2020-10-01	170.19	170.73	170.19	170.73	1887	{}	\N	2020-10-12 15:58:59.270733
66511	RFG	2020-10-02	166.61	172.4499	166.61	171.9	5122	{}	\N	2020-10-12 15:58:59.277578
66512	RFG	2020-10-05	172.69	175.92	172.69	175.92	16987	{}	\N	2020-10-12 15:58:59.283651
66513	RFG	2020-10-06	176.3785	178.27	176.3785	177.929	2103	{}	\N	2020-10-12 15:58:59.28892
66514	RFG	2020-10-07	178.4109	178.78	178.3972	178.78	3559	{}	\N	2020-10-12 15:58:59.294439
66515	RFG	2020-10-08	179.23	179.7365	178.88	179.665	6830	{}	\N	2020-10-12 15:58:59.299973
66516	RFG	2020-10-09	181.04	182.0899	181.04	181.6199	2639	{}	\N	2020-10-12 15:58:59.305187
66517	RFG	2020-10-12	183.34	183.34	182.0305	182.0305	1116	{}	\N	2020-10-12 15:58:59.310951
66518	RAD	2020-09-28	10.31	10.368	9.73	9.78	7564172	{}	\N	2020-10-12 15:59:01.211441
66519	RAD	2020-09-29	9.72	10.22	9.52	9.63	14001032	{}	\N	2020-10-12 15:59:01.222532
66520	RAD	2020-09-30	9.7	9.891	9.49	9.49	10824582	{}	\N	2020-10-12 15:59:01.229738
66521	RAD	2020-10-01	9.51	9.63	9.29	9.61	7702403	{}	\N	2020-10-12 15:59:01.235613
66522	RAD	2020-10-02	9.35	9.82	9.3	9.66	6049998	{}	\N	2020-10-12 15:59:01.240373
66523	RAD	2020-10-05	9.64	9.78	9.39	9.74	6442956	{}	\N	2020-10-12 15:59:01.246338
66524	RAD	2020-10-06	9.7	10.13	9.57	9.67	8148145	{}	\N	2020-10-12 15:59:01.250692
66525	RAD	2020-10-07	9.71	10.3898	9.63	10.31	11596754	{}	\N	2020-10-12 15:59:01.254771
66526	RAD	2020-10-08	10.41	10.6	9.83	10.28	7645244	{}	\N	2020-10-12 15:59:01.258873
66527	RAD	2020-10-09	10.3	10.48	10.04	10.07	4995273	{}	\N	2020-10-12 15:59:01.263404
66528	RAD	2020-10-12	10.08	10.19	9.91	10.105	1915946	{}	\N	2020-10-12 15:59:01.26813
66529	RADA	2020-09-28	6.2	6.2989	6.05	6.13	94237	{}	\N	2020-10-12 15:59:02.244272
66530	RADA	2020-09-29	6.15	6.16	5.92	6.09	191285	{}	\N	2020-10-12 15:59:02.253673
66531	RADA	2020-09-30	6.06	6.17	5.87	5.93	193805	{}	\N	2020-10-12 15:59:02.259641
66532	RADA	2020-10-01	5.93	6.33	5.93	6.25	246065	{}	\N	2020-10-12 15:59:02.265225
66533	RADA	2020-10-02	6.45	6.55	6.15	6.32	325787	{}	\N	2020-10-12 15:59:02.270144
66534	RADA	2020-10-05	6.67	6.72	6.48	6.54	296883	{}	\N	2020-10-12 15:59:02.274853
66535	RADA	2020-10-06	6.58	6.73	6.49	6.58	122609	{}	\N	2020-10-12 15:59:02.279224
66536	RADA	2020-10-07	6.75	6.8	6.56	6.72	152369	{}	\N	2020-10-12 15:59:02.283855
66537	RADA	2020-10-08	6.69	6.95	6.67	6.88	318249	{}	\N	2020-10-12 15:59:02.288136
66538	RADA	2020-10-09	6.9	6.94	6.6	6.65	147914	{}	\N	2020-10-12 15:59:02.292372
66539	RADA	2020-10-12	6.66	6.7	6.54	6.69	65378	{}	\N	2020-10-12 15:59:02.297242
66540	RAVI	2020-09-28	76.31	76.32	76.28	76.29	30757	{}	\N	2020-10-12 15:59:03.446392
66541	RAVI	2020-09-29	76.3192	76.34	76.31	76.33	19255	{}	\N	2020-10-12 15:59:03.458459
66542	RAVI	2020-09-30	76.3	76.34	76.3	76.31	16676	{}	\N	2020-10-12 15:59:03.466103
66543	RAVI	2020-10-01	76.3	76.3	76.27	76.28	11385	{}	\N	2020-10-12 15:59:03.472064
66544	RAVI	2020-10-02	76.24	76.29	76.24	76.275	19114	{}	\N	2020-10-12 15:59:03.477661
66545	RAVI	2020-10-05	76.255	76.28	76.2492	76.279	21083	{}	\N	2020-10-12 15:59:03.482693
66546	RAVI	2020-10-06	76.28	76.31	76.27	76.31	98259	{}	\N	2020-10-12 15:59:03.487715
66547	RAVI	2020-10-07	76.28	76.29	76.2565	76.28	20209	{}	\N	2020-10-12 15:59:03.492695
66548	RAVI	2020-10-08	76.27	76.3	76.27	76.285	5194	{}	\N	2020-10-12 15:59:03.497856
66549	RAVI	2020-10-09	76.285	76.32	76.285	76.31	46093	{}	\N	2020-10-12 15:59:03.503172
66550	RAVI	2020-10-12	76.29	76.29	76.2867	76.2867	3488	{}	\N	2020-10-12 15:59:03.508741
66551	RBA	2020-09-28	58.5	59.08	58.18	58.19	446398	{}	\N	2020-10-12 15:59:04.435955
66552	RBA	2020-09-29	58.22	59.2	57.91	59.14	614983	{}	\N	2020-10-12 15:59:04.447121
66553	RBA	2020-09-30	59.09	60.47	59.04	59.25	874988	{}	\N	2020-10-12 15:59:04.45503
66554	RBA	2020-10-01	59.73	60.685	59.58	60.61	798582	{}	\N	2020-10-12 15:59:04.461057
66555	RBA	2020-10-02	59.87	60	59.18	59.72	691838	{}	\N	2020-10-12 15:59:04.467449
66556	RBA	2020-10-05	60.28	61.48	59.81	61.43	499218	{}	\N	2020-10-12 15:59:04.472665
66557	RBA	2020-10-06	61.54	62.63	61.44	61.6	814569	{}	\N	2020-10-12 15:59:04.47776
66558	RBA	2020-10-07	61.93	63.62	61.93	63.43	937272	{}	\N	2020-10-12 15:59:04.484595
66559	RBA	2020-10-08	63.54	63.75	62.59	63.73	541710	{}	\N	2020-10-12 15:59:04.490705
66560	RBA	2020-10-09	64	65.48	63.9475	65.27	660326	{}	\N	2020-10-12 15:59:04.497542
66561	RBA	2020-10-12	65.38	65.9	65.305	65.66	111786	{}	\N	2020-10-12 15:59:04.503355
66562	RBIN	2020-09-28	25.0153	25.0153	25.0153	25.0153	0	{}	\N	2020-10-12 15:59:06.29374
66563	RBIN	2020-09-29	25.26	25.26	25.0392	25.0392	1200	{}	\N	2020-10-12 15:59:06.303484
66564	RBIN	2020-09-30	24.9464	24.9464	24.9464	24.9464	3	{}	\N	2020-10-12 15:59:06.310075
66565	RBIN	2020-10-01	25.0617	25.0617	25.0617	25.0617	20	{}	\N	2020-10-12 15:59:06.315883
66566	RBIN	2020-10-02	24.9533	24.9533	24.9533	24.9533	32	{}	\N	2020-10-12 15:59:06.321714
66567	RBIN	2020-10-05	25.2441	25.2441	25.2441	25.2441	0	{}	\N	2020-10-12 15:59:06.326641
66568	RBIN	2020-10-06	24.9246	24.9246	24.9246	24.9246	0	{}	\N	2020-10-12 15:59:06.331295
66569	RBIN	2020-10-07	25.0639	25.0639	25.0639	25.0639	10	{}	\N	2020-10-12 15:59:06.336942
66570	RBIN	2020-10-08	25.2072	25.2072	25.2072	25.2072	4	{}	\N	2020-10-12 15:59:06.341899
66571	RBIN	2020-10-09	25.2929	25.2929	25.2929	25.2929	1	{}	\N	2020-10-12 15:59:06.346712
66572	RCL	2020-09-28	66.13	66.83	63.32	66.09	5764583	{}	\N	2020-10-12 15:59:07.67513
66573	RCL	2020-09-29	65.86	66.23	63.51	64.4	8735136	{}	\N	2020-10-12 15:59:07.689334
66574	RCL	2020-09-30	66.47	68.21	64.18	64.73	15921242	{}	\N	2020-10-12 15:59:07.698322
66575	RCL	2020-10-01	65.15	65.92	63.57	65.55	10629598	{}	\N	2020-10-12 15:59:07.705818
66576	RCL	2020-10-02	62.5	65.76	62.3	65.26	9861230	{}	\N	2020-10-12 15:59:07.712296
66577	RCL	2020-10-05	65	66.1	64.5	65.25	7147112	{}	\N	2020-10-12 15:59:07.717698
66578	RCL	2020-10-06	66	69.91	65.77	66.18	14427754	{}	\N	2020-10-12 15:59:07.72329
66579	RCL	2020-10-07	67.47	68.64	66.1	68.02	9054643	{}	\N	2020-10-12 15:59:07.728414
66580	RCL	2020-10-08	68.81	69.36	67.25	69.29	9403510	{}	\N	2020-10-12 15:59:07.73385
66581	RCL	2020-10-09	69.4	72.11	68.8	71.09	12509202	{}	\N	2020-10-12 15:59:07.739307
66582	RCL	2020-10-12	70.36	70.73	69.18	69.75	4472550	{}	\N	2020-10-12 15:59:07.745043
66583	RCKY	2020-09-28	25.15	25.4709	24.86	25.19	26429	{}	\N	2020-10-12 15:59:08.665002
66584	RCKY	2020-09-29	25.642	25.642	24.25	24.51	36868	{}	\N	2020-10-12 15:59:08.679038
66585	RCKY	2020-09-30	24.73	25.255	24.71	24.83	35744	{}	\N	2020-10-12 15:59:08.686316
66586	RCKY	2020-10-01	25.09	25.16	24.78	25.02	31639	{}	\N	2020-10-12 15:59:08.692559
66587	RCKY	2020-10-02	24.94	25.91	24.94	25.5	14233	{}	\N	2020-10-12 15:59:08.698054
66588	RCKY	2020-10-05	25.57	25.94	25.16	25.73	22862	{}	\N	2020-10-12 15:59:08.703203
66589	RCKY	2020-10-06	25.89	26.25	25.29	25.51	15345	{}	\N	2020-10-12 15:59:08.708252
66590	RCKY	2020-10-07	25.81	26.95	25.8	26.75	33325	{}	\N	2020-10-12 15:59:08.713205
66591	RCKY	2020-10-08	26.75	26.75	25.46	26.4	30941	{}	\N	2020-10-12 15:59:08.71804
66592	RCKY	2020-10-09	26.41	26.54	26.02	26.26	10676	{}	\N	2020-10-12 15:59:08.723488
66593	RCKY	2020-10-12	26.23	26.65	26.09	26.39	7228	{}	\N	2020-10-12 15:59:08.72875
66594	RDCM	2020-09-28	9.13	10.0534	9.13	9.6846	43657	{}	\N	2020-10-12 15:59:10.696981
66595	RDCM	2020-09-29	9.793	9.793	9.4301	9.44	11646	{}	\N	2020-10-12 15:59:10.707422
66596	RDCM	2020-09-30	9.86	9.86	9.3	9.46	20961	{}	\N	2020-10-12 15:59:10.715313
66597	RDCM	2020-10-01	9.4	9.43	9.18	9.2226	32164	{}	\N	2020-10-12 15:59:10.722049
66598	RDCM	2020-10-02	9.3071	9.58	8.995	9.1	21375	{}	\N	2020-10-12 15:59:10.729111
66599	RDCM	2020-10-05	8.71	9.45	8.7	8.92	4554	{}	\N	2020-10-12 15:59:10.734182
66600	RDCM	2020-10-06	8.83	9	8.81	8.87	14969	{}	\N	2020-10-12 15:59:10.74006
66601	RDCM	2020-10-07	8.935	9.035	8.91	8.9895	5340	{}	\N	2020-10-12 15:59:10.745578
66602	RDCM	2020-10-08	8.99	9.0881	8.8945	8.99	25280	{}	\N	2020-10-12 15:59:10.750629
66603	RDCM	2020-10-09	8.9865	9.055	8.8759	8.9378	28888	{}	\N	2020-10-12 15:59:10.756406
66604	RDCM	2020-10-12	8.94	9.2	8.94	9	15147	{}	\N	2020-10-12 15:59:10.761841
66605	QYLD	2020-09-28	21.44	21.48	21.38	21.47	513089	{}	\N	2020-10-12 15:59:11.974694
66606	QYLD	2020-09-29	21.48	21.53	21.46	21.5	563155	{}	\N	2020-10-12 15:59:11.984242
66607	QYLD	2020-09-30	21.5	21.63	21.42	21.56	538697	{}	\N	2020-10-12 15:59:11.991615
66608	QYLD	2020-10-01	21.59	21.67	21.58	21.61	447687	{}	\N	2020-10-12 15:59:11.997529
66609	QYLD	2020-10-02	21.43	21.6	21.42	21.5	469558	{}	\N	2020-10-12 15:59:12.003023
66610	QYLD	2020-10-05	21.58	21.7	21.58	21.69	421927	{}	\N	2020-10-12 15:59:12.008345
66611	QYLD	2020-10-06	21.7	21.72	21.55	21.6	563102	{}	\N	2020-10-12 15:59:12.013431
66612	QYLD	2020-10-07	21.64	21.74	21.63	21.72	314963	{}	\N	2020-10-12 15:59:12.018494
66613	QYLD	2020-10-08	21.77	21.78	21.73	21.77	368535	{}	\N	2020-10-12 15:59:12.02369
66614	QYLD	2020-10-09	21.77	21.81	21.76	21.81	454098	{}	\N	2020-10-12 15:59:12.029109
66615	QYLD	2020-10-12	21.81	21.84	21.8	21.805	314353	{}	\N	2020-10-12 15:59:12.034263
66616	RARE	2020-09-28	82.9	83.21	81.045	82.5	185011	{}	\N	2020-10-12 15:59:13.28777
66617	RARE	2020-09-29	82.73	85.7299	82.3	85.1	294353	{}	\N	2020-10-12 15:59:13.300097
66618	RARE	2020-09-30	85.46	86.71	81.76	82.19	240535	{}	\N	2020-10-12 15:59:13.306737
66619	RARE	2020-10-01	82.5	85.72	82.43	85.71	382902	{}	\N	2020-10-12 15:59:13.312574
66620	RARE	2020-10-02	84.36	87.75	83.35	84.4	377610	{}	\N	2020-10-12 15:59:13.317449
66621	RARE	2020-10-05	84.54	89.67	84.54	89.25	357484	{}	\N	2020-10-12 15:59:13.322047
66622	RARE	2020-10-06	89.83	91.57	88.29	88.89	332039	{}	\N	2020-10-12 15:59:13.327075
66623	RARE	2020-10-07	89.94	94.86	88.91	94.2	369085	{}	\N	2020-10-12 15:59:13.331352
66624	RARE	2020-10-08	94.67	96.58	92.96	94.43	473959	{}	\N	2020-10-12 15:59:13.335494
66625	RARE	2020-10-09	95	96.5	93.15	94.45	231514	{}	\N	2020-10-12 15:59:13.339939
66626	RARE	2020-10-12	94.45	96.48	92.6	95.665	102412	{}	\N	2020-10-12 15:59:13.344816
66627	RBBN	2020-09-28	3.78	3.9	3.77	3.8	216562	{}	\N	2020-10-12 15:59:14.27364
66628	RBBN	2020-09-29	3.77	3.815	3.7	3.8	192992	{}	\N	2020-10-12 15:59:14.284636
66629	RBBN	2020-09-30	3.8	3.92	3.78	3.87	239403	{}	\N	2020-10-12 15:59:14.292384
66630	RBBN	2020-10-01	3.9	3.94	3.78	3.94	252402	{}	\N	2020-10-12 15:59:14.299005
66631	RBBN	2020-10-02	3.81	3.96	3.75	3.835	273803	{}	\N	2020-10-12 15:59:14.305102
66632	RBBN	2020-10-05	3.88	4.11	3.86	4.06	341984	{}	\N	2020-10-12 15:59:14.310448
66633	RBBN	2020-10-06	4.08	4.12	3.84	3.855	215240	{}	\N	2020-10-12 15:59:14.315662
66634	RBBN	2020-10-07	3.91	3.99	3.83	3.95	248782	{}	\N	2020-10-12 15:59:14.32049
66635	RBBN	2020-10-08	4.02	4.04	3.84	4	241653	{}	\N	2020-10-12 15:59:14.325774
66636	RBBN	2020-10-09	4.05	4.06	3.91	3.98	190104	{}	\N	2020-10-12 15:59:14.330911
66637	RBBN	2020-10-12	3.98	4.01	3.9	3.975	36897	{}	\N	2020-10-12 15:59:14.335685
66638	RCG	2020-09-28	1.3117	1.325	1.2918	1.325	6075	{}	\N	2020-10-12 15:59:15.220524
66639	RCG	2020-09-29	1.33	1.34	1.29	1.31	6994	{}	\N	2020-10-12 15:59:15.233183
66640	RCG	2020-09-30	1.315	1.315	1.3001	1.31	2209	{}	\N	2020-10-12 15:59:15.240575
66641	RCG	2020-10-01	1.31	1.31	1.3	1.305	4315	{}	\N	2020-10-12 15:59:15.246446
66642	RCG	2020-10-02	1.31	1.32	1.2906	1.31	8796	{}	\N	2020-10-12 15:59:15.252698
66643	RCG	2020-10-05	1.31	1.33	1.31	1.325	8103	{}	\N	2020-10-12 15:59:15.258236
66644	RCG	2020-10-06	1.32	1.3301	1.32	1.3301	7411	{}	\N	2020-10-12 15:59:15.263853
66645	RCG	2020-10-07	1.35	1.35	1.328	1.35	11218	{}	\N	2020-10-12 15:59:15.26899
66646	RCG	2020-10-08	1.35	1.36	1.3366	1.34	12587	{}	\N	2020-10-12 15:59:15.274034
66647	RCG	2020-10-09	1.33	1.3892	1.33	1.36	19921	{}	\N	2020-10-12 15:59:15.279567
66648	RCG	2020-10-12	1.4	1.4	1.35	1.35	1689	{}	\N	2020-10-12 15:59:15.284584
66649	RCS	2020-09-28	6.34	6.3796	6.3	6.31	141397	{}	\N	2020-10-12 15:59:16.545962
66650	RCS	2020-09-29	6.27	6.38	6.2512	6.38	377402	{}	\N	2020-10-12 15:59:16.557739
66651	RCS	2020-09-30	6.34	6.51	6.31	6.42	1183993	{}	\N	2020-10-12 15:59:16.566828
66652	RCS	2020-10-01	6.49	6.49	6.4117	6.44	437272	{}	\N	2020-10-12 15:59:16.574272
66653	RCS	2020-10-02	6.34	6.51	6.34	6.49	209495	{}	\N	2020-10-12 15:59:16.581131
66654	RCS	2020-10-05	6.49	6.53	6.445	6.47	246952	{}	\N	2020-10-12 15:59:16.587554
66655	RCS	2020-10-06	6.43	6.52	6.43	6.48	182598	{}	\N	2020-10-12 15:59:16.593911
66656	RCS	2020-10-07	6.5	6.52	6.44	6.48	190082	{}	\N	2020-10-12 15:59:16.600665
66657	RCS	2020-10-08	6.45	6.51	6.4	6.44	270370	{}	\N	2020-10-12 15:59:16.606578
66658	RCS	2020-10-09	6.41	6.4499	6.36	6.38	234548	{}	\N	2020-10-12 15:59:16.612572
66659	RCS	2020-10-12	6.37	6.39	6.36	6.3856	143410	{}	\N	2020-10-12 15:59:16.618833
66660	RDN	2020-09-28	14.66	15.185	14.55	15.01	1682136	{}	\N	2020-10-12 15:59:17.549602
66661	RDN	2020-09-29	14.98	14.98	14.41	14.59	1865658	{}	\N	2020-10-12 15:59:17.56086
66662	RDN	2020-09-30	14.69	15.035	14.51	14.61	2995108	{}	\N	2020-10-12 15:59:17.569125
66663	RDN	2020-10-01	14.66	15.01	14.525	14.94	2554795	{}	\N	2020-10-12 15:59:17.575023
66664	RDN	2020-10-02	14.55	15.89	14.49	15.7	5834270	{}	\N	2020-10-12 15:59:17.581446
66665	RDN	2020-10-05	16	16.25	15.81	16.22	2378011	{}	\N	2020-10-12 15:59:17.587498
66666	RDN	2020-10-06	16.58	17.24	16.35	16.79	6205067	{}	\N	2020-10-12 15:59:17.594502
66667	RDN	2020-10-07	17.35	18.135	17.34	17.57	7038959	{}	\N	2020-10-12 15:59:17.600068
66668	RDN	2020-10-08	17.84	17.98	17.375	17.64	3845162	{}	\N	2020-10-12 15:59:17.605713
66669	RDN	2020-10-09	17.82	18	17.38	17.44	2755109	{}	\N	2020-10-12 15:59:17.610466
66670	RDN	2020-10-12	17.69	18.165	17.55	18.13	1041782	{}	\N	2020-10-12 15:59:17.61564
66671	REDU	2020-09-28	5.87	6.21	5.87	5.99	38206	{}	\N	2020-10-12 15:59:18.998132
66672	REDU	2020-09-29	6.06	6.1689	5.93	6.16	15379	{}	\N	2020-10-12 15:59:19.008677
66673	REDU	2020-09-30	6.195	6.43	6.01	6.06	36895	{}	\N	2020-10-12 15:59:19.016387
66674	REDU	2020-10-01	6.05	6.21	6.05	6.12	16767	{}	\N	2020-10-12 15:59:19.022046
66675	REDU	2020-10-02	6.01	6.37	6.01	6.33	17899	{}	\N	2020-10-12 15:59:19.027448
66676	REDU	2020-10-05	5.945	6.33	5.945	6.25	27563	{}	\N	2020-10-12 15:59:19.032763
66677	REDU	2020-10-06	6.17	6.4599	6.01	6.3	30163	{}	\N	2020-10-12 15:59:19.037884
66678	REDU	2020-10-07	6.29	6.6	6.29	6.57	23974	{}	\N	2020-10-12 15:59:19.042806
66679	REDU	2020-10-08	6.52	6.6499	6.39	6.63	15798	{}	\N	2020-10-12 15:59:19.048194
66680	REDU	2020-10-09	6.6299	6.8663	6.5601	6.6	29369	{}	\N	2020-10-12 15:59:19.053117
66681	REDU	2020-10-12	6.71	6.78	6.5001	6.5001	16686	{}	\N	2020-10-12 15:59:19.058399
66682	REED	2020-09-28	0.9333	0.9679	0.922	0.9433	79909	{}	\N	2020-10-12 15:59:20.167118
66683	REED	2020-09-29	0.9556	1	0.9336	0.9535	184972	{}	\N	2020-10-12 15:59:20.178746
66684	REED	2020-09-30	0.9574	0.979	0.92	0.9285	225096	{}	\N	2020-10-12 15:59:20.186245
66685	REED	2020-10-01	0.93	0.97	0.92	0.9205	167279	{}	\N	2020-10-12 15:59:20.192239
66686	REED	2020-10-02	0.92	0.9575	0.92	0.921	96742	{}	\N	2020-10-12 15:59:20.197955
66687	REED	2020-10-05	0.9303	0.95	0.921	0.93	72811	{}	\N	2020-10-12 15:59:20.20355
66688	REED	2020-10-06	0.9331	0.95	0.9258	0.9258	152063	{}	\N	2020-10-12 15:59:20.20852
66689	REED	2020-10-07	0.95	0.96	0.93	0.9305	91318	{}	\N	2020-10-12 15:59:20.213666
66690	REED	2020-10-08	0.945	0.9694	0.93	0.9694	155876	{}	\N	2020-10-12 15:59:20.219506
66691	REED	2020-10-09	0.98	1.03	0.9713	1.01	229743	{}	\N	2020-10-12 15:59:20.224318
66692	REED	2020-10-12	1.03	1.03	1	1.0038	52679	{}	\N	2020-10-12 15:59:20.229348
66693	REI	2020-09-28	0.67	0.7	0.67	0.6847	346379	{}	\N	2020-10-12 15:59:21.239054
66694	REI	2020-09-29	0.68	0.6909	0.66	0.6872	285117	{}	\N	2020-10-12 15:59:21.250304
66695	REI	2020-09-30	0.69	0.7145	0.6759	0.68	453377	{}	\N	2020-10-12 15:59:21.257266
66696	REI	2020-10-01	0.6809	0.715	0.6701	0.6793	511970	{}	\N	2020-10-12 15:59:21.263065
66697	REI	2020-10-02	0.67	0.72	0.66	0.71	670079	{}	\N	2020-10-12 15:59:21.269175
66698	REI	2020-10-05	0.72	0.73	0.711	0.715	384439	{}	\N	2020-10-12 15:59:21.274114
66699	REI	2020-10-06	0.715	0.7351	0.6706	0.7017	729443	{}	\N	2020-10-12 15:59:21.279038
66700	REI	2020-10-07	0.705	0.737999	0.69	0.7302	706556	{}	\N	2020-10-12 15:59:21.284471
66701	REI	2020-10-08	0.73	0.94	0.71001	0.8399	5796697	{}	\N	2020-10-12 15:59:21.289575
66702	REI	2020-10-09	0.85	0.8799	0.8	0.8021	2054862	{}	\N	2020-10-12 15:59:21.294862
66703	REI	2020-10-12	0.83	0.83	0.76	0.76	311764	{}	\N	2020-10-12 15:59:21.300207
66704	REML	2020-09-28	3.9	4.09	3.849	4.05	843587	{}	\N	2020-10-12 15:59:22.372107
66705	REML	2020-09-29	4.0727	4.0878	3.83	3.99	535286	{}	\N	2020-10-12 15:59:22.383331
66706	REML	2020-09-30	3.91	4.0101	3.85	3.86	435220	{}	\N	2020-10-12 15:59:22.390425
66707	REML	2020-10-01	3.9	3.95	3.83	3.95	337396	{}	\N	2020-10-12 15:59:22.396399
66708	REML	2020-10-02	3.8	4.08	3.8	4.02	920983	{}	\N	2020-10-12 15:59:22.401856
66709	REML	2020-10-05	4.08	4.1399	4.04	4.095	629953	{}	\N	2020-10-12 15:59:22.406971
66710	REML	2020-10-06	4.12	4.24	4.03	4.08	946311	{}	\N	2020-10-12 15:59:22.41142
66711	REML	2020-10-07	4.14	4.1476	4.03	4.12	571036	{}	\N	2020-10-12 15:59:22.41601
66712	REML	2020-10-08	4.12	4.2978	4.1055	4.26	834827	{}	\N	2020-10-12 15:59:22.420794
66713	REML	2020-10-09	4.3	4.33	4.17	4.19	767389	{}	\N	2020-10-12 15:59:22.425509
66714	REML	2020-10-12	4.31	4.31	4.16	4.217	537235	{}	\N	2020-10-12 15:59:22.430018
66715	REPH	2020-09-28	2.13	2.24	2.08	2.16	148262	{}	\N	2020-10-12 15:59:24.707869
66716	REPH	2020-09-29	2.16	2.1655	2.03	2.1	245648	{}	\N	2020-10-12 15:59:24.718545
66717	REPH	2020-09-30	2.1	2.14	2.09	2.1	91339	{}	\N	2020-10-12 15:59:24.725494
66718	REPH	2020-10-01	2.11	2.1432	1.91	1.98	574041	{}	\N	2020-10-12 15:59:24.731802
66719	REPH	2020-10-02	1.99	2.1	1.9302	2.07	145770	{}	\N	2020-10-12 15:59:24.737441
66720	REPH	2020-10-05	2.08	2.13	2.05	2.11	145520	{}	\N	2020-10-12 15:59:24.742096
66721	REPH	2020-10-06	2.13	2.15	2.02	2.03	294176	{}	\N	2020-10-12 15:59:24.746316
66722	REPH	2020-10-07	2.03	2.18	2.0266	2.15	182627	{}	\N	2020-10-12 15:59:24.750356
66723	REPH	2020-10-08	2.2	2.37	2.16	2.34	197146	{}	\N	2020-10-12 15:59:24.754786
66724	REPH	2020-10-09	2.34	2.3599	2.28	2.3	106059	{}	\N	2020-10-12 15:59:24.759062
66725	REPH	2020-10-12	2.31	2.31	2.14	2.19	122993	{}	\N	2020-10-12 15:59:24.763358
66726	RALS	2020-09-28	29.8167	29.8167	29.8167	29.8167	155	{}	\N	2020-10-12 15:59:27.403034
66727	RALS	2020-09-29	29.7236	29.7236	29.7236	29.7236	33	{}	\N	2020-10-12 15:59:27.411195
66728	RALS	2020-09-30	29.6997	29.6997	29.6997	29.6997	41	{}	\N	2020-10-12 15:59:27.416484
66729	RALS	2020-10-01	29.6299	29.6299	29.585	29.585	520	{}	\N	2020-10-12 15:59:27.421676
66730	RALS	2020-10-02	29.8194	29.8194	29.8194	29.8194	3	{}	\N	2020-10-12 15:59:27.426544
66731	RALS	2020-10-05	29.8765	29.8765	29.8765	29.8765	0	{}	\N	2020-10-12 15:59:27.431641
66732	RALS	2020-10-06	29.95	29.9702	29.9468	29.9468	412	{}	\N	2020-10-12 15:59:27.436645
66733	RALS	2020-10-07	29.9056	29.9056	29.9056	29.9056	3368	{}	\N	2020-10-12 15:59:27.440996
66734	RALS	2020-10-08	30.05	30.05	30.05	30.05	0	{}	\N	2020-10-12 15:59:27.44539
66735	RALS	2020-10-09	29.8395	29.8395	29.8395	29.8395	0	{}	\N	2020-10-12 15:59:27.451332
66736	RBB	2020-09-28	11.16	11.66	11.16	11.43	31260	{}	\N	2020-10-12 15:59:28.494245
66737	RBB	2020-09-29	11.4	11.4986	11.12	11.41	26995	{}	\N	2020-10-12 15:59:28.504297
66738	RBB	2020-09-30	11.53	11.57	11.2956	11.34	28358	{}	\N	2020-10-12 15:59:28.511317
66739	RBB	2020-10-01	11.44	11.44	11.03	11.16	58163	{}	\N	2020-10-12 15:59:28.517073
66740	RBB	2020-10-02	11	11.73	11	11.66	22511	{}	\N	2020-10-12 15:59:28.52291
66741	RBB	2020-10-05	11.69	12.09	11.69	11.92	55976	{}	\N	2020-10-12 15:59:28.528049
66742	RBB	2020-10-06	12.1	12.28	11.51	11.86	53291	{}	\N	2020-10-12 15:59:28.532588
66743	RBB	2020-10-07	11.66	12.38	11.66	12.33	31228	{}	\N	2020-10-12 15:59:28.537032
66744	RBB	2020-10-08	12.45	12.8	11.1619	12.67	54390	{}	\N	2020-10-12 15:59:28.541874
66745	RBB	2020-10-09	12.24	12.79	11.8325	12.38	26296	{}	\N	2020-10-12 15:59:28.546598
66746	RBB	2020-10-12	12.43	12.55	12.32	12.5	16220	{}	\N	2020-10-12 15:59:28.551238
66747	RBUS	2020-09-28	32.16	32.2265	32.16	32.2265	201	{}	\N	2020-10-12 15:59:30.278745
66748	RBUS	2020-09-29	32.1131	32.1131	32.1131	32.1131	16	{}	\N	2020-10-12 15:59:30.288156
66749	RBUS	2020-09-30	32.3185	32.3185	32.3185	32.3185	11	{}	\N	2020-10-12 15:59:30.295116
66750	RBUS	2020-10-01	32.4054	32.4054	32.4054	32.4054	93	{}	\N	2020-10-12 15:59:30.301841
66751	RBUS	2020-10-02	32.39	32.39	32.39	32.39	460	{}	\N	2020-10-12 15:59:30.306908
66752	RBUS	2020-10-05	32.7479	32.7479	32.7479	32.7479	51	{}	\N	2020-10-12 15:59:30.311963
66753	RBUS	2020-10-06	32.795	32.82	32.5425	32.5425	220	{}	\N	2020-10-12 15:59:30.317272
66754	RBUS	2020-10-07	32.9926	32.9926	32.9926	32.9926	0	{}	\N	2020-10-12 15:59:30.324179
66755	RBUS	2020-10-08	33.172	33.172	33.172	33.172	0	{}	\N	2020-10-12 15:59:30.329481
66756	RBUS	2020-10-09	33.4382	33.4382	33.4382	33.4382	153	{}	\N	2020-10-12 15:59:30.334639
66757	RCM	2020-09-28	16.35	16.925	16.14	16.8	1279856	{}	\N	2020-10-12 15:59:32.318855
66758	RCM	2020-09-29	16.71	17.1	16.7	17.06	1043025	{}	\N	2020-10-12 15:59:32.331645
66759	RCM	2020-09-30	17.05	17.24	16.97	17.15	1121572	{}	\N	2020-10-12 15:59:32.338218
66760	RCM	2020-10-01	17.28	17.56	17.16	17.54	845345	{}	\N	2020-10-12 15:59:32.34388
66761	RCM	2020-10-02	17.12	17.62	16.89	17.55	1248479	{}	\N	2020-10-12 15:59:32.349172
66762	RCM	2020-10-05	17.71	17.83	17.56	17.74	614751	{}	\N	2020-10-12 15:59:32.353774
66763	RCM	2020-10-06	17.83	18.05	17.21	17.24	1432346	{}	\N	2020-10-12 15:59:32.358411
66764	RCM	2020-10-07	17.58	17.62	17.2	17.4	820722	{}	\N	2020-10-12 15:59:32.363417
66765	RCM	2020-10-08	17.52	17.65	17.39	17.64	384119	{}	\N	2020-10-12 15:59:32.367917
66766	RCM	2020-10-09	17.87	17.93	17.47	17.72	511023	{}	\N	2020-10-12 15:59:32.372056
66767	RCM	2020-10-12	17.76	17.93	17.58	17.67	104744	{}	\N	2020-10-12 15:59:32.376138
66768	RDFN	2020-09-28	48.47	48.87	47.47	48.7	819336	{}	\N	2020-10-12 15:59:34.051831
66769	RDFN	2020-09-29	48.77	49.81	48.7	49.74	789264	{}	\N	2020-10-12 15:59:34.064453
66770	RDFN	2020-09-30	49.66	51.22	49.45	49.93	1037561	{}	\N	2020-10-12 15:59:34.07261
66771	RDFN	2020-10-01	50.69	52.87	50.44	52.86	986271	{}	\N	2020-10-12 15:59:34.079552
66772	RDFN	2020-10-02	50.29	53.16	50.02	52.76	945584	{}	\N	2020-10-12 15:59:34.086124
66773	RDFN	2020-10-05	53.45	53.69	51.6701	53.56	794552	{}	\N	2020-10-12 15:59:34.091568
66774	RDFN	2020-10-06	53.88	54.77	52.5326	53.01	861334	{}	\N	2020-10-12 15:59:34.097076
66775	RDFN	2020-10-07	54.12	55.4	52.9	54.08	1035409	{}	\N	2020-10-12 15:59:34.102229
66776	RDFN	2020-10-08	54.55	54.89	53.15	53.67	678204	{}	\N	2020-10-12 15:59:34.10775
66777	RDFN	2020-10-09	54.4777	55.2	53.75	54.83	832304	{}	\N	2020-10-12 15:59:34.113227
66778	RDFN	2020-10-12	55.5	55.95	54.685	55.07	282588	{}	\N	2020-10-12 15:59:34.118927
66779	RDWR	2020-09-28	24.28	24.7	24.28	24.655	180996	{}	\N	2020-10-12 15:59:35.310544
66780	RDWR	2020-09-29	24.99	25.095	24.48	24.68	440762	{}	\N	2020-10-12 15:59:35.323573
66781	RDWR	2020-09-30	24.62	24.62	23.98	24.24	261461	{}	\N	2020-10-12 15:59:35.33086
66782	RDWR	2020-10-01	24.42	24.46	23.99	24.25	184667	{}	\N	2020-10-12 15:59:35.337514
66783	RDWR	2020-10-02	23.94	24.26	23.75	24.095	141418	{}	\N	2020-10-12 15:59:35.344119
66784	RDWR	2020-10-05	24.27	24.465	24.03	24.36	192758	{}	\N	2020-10-12 15:59:35.349656
66785	RDWR	2020-10-06	24.5	24.63	24.22	24.29	249553	{}	\N	2020-10-12 15:59:35.355029
66786	RDWR	2020-10-07	24.36	24.65	24.22	24.57	186576	{}	\N	2020-10-12 15:59:35.360233
66787	RDWR	2020-10-08	24.81	25	24.64	24.7	212057	{}	\N	2020-10-12 15:59:35.365896
66788	RDWR	2020-10-09	24.83	24.96	24.56	24.69	237033	{}	\N	2020-10-12 15:59:35.371238
66789	RDWR	2020-10-12	24.83	24.88	24.48	24.565	77671	{}	\N	2020-10-12 15:59:35.376237
66790	RELL	2020-09-28	4.26	4.4516	4.26	4.3487	16708	{}	\N	2020-10-12 15:59:36.735781
66791	RELL	2020-09-29	4.35	4.5	4.15	4.15	43332	{}	\N	2020-10-12 15:59:36.748139
66792	RELL	2020-09-30	4.1201	4.41	4.1201	4.17	16815	{}	\N	2020-10-12 15:59:36.755743
66793	RELL	2020-10-01	4.26	4.28	4.1306	4.1785	19715	{}	\N	2020-10-12 15:59:36.761624
66794	RELL	2020-10-02	4.1634	4.24	4.15	4.2399	14654	{}	\N	2020-10-12 15:59:36.767107
66795	RELL	2020-10-05	4.24	4.35	4.22	4.22	28373	{}	\N	2020-10-12 15:59:36.771888
66796	RELL	2020-10-06	4.3	4.325	4.23	4.26	9921	{}	\N	2020-10-12 15:59:36.776218
66797	RELL	2020-10-07	4.23	4.6	4.22	4.5	52734	{}	\N	2020-10-12 15:59:36.780647
66798	RELL	2020-10-08	4.4977	4.65	4.22	4.57	75623	{}	\N	2020-10-12 15:59:36.785177
66799	RELL	2020-10-09	4.491	4.52	4.405	4.449	23179	{}	\N	2020-10-12 15:59:36.78962
66800	RELL	2020-10-12	4.4602	4.51	4.3	4.41	11523	{}	\N	2020-10-12 15:59:36.794588
66801	REVG	2020-09-28	7.5	7.74	7.42	7.57	368820	{}	\N	2020-10-12 15:59:37.662443
66802	REVG	2020-09-29	7.52	7.87	7.47	7.76	622444	{}	\N	2020-10-12 15:59:37.672286
66803	REVG	2020-09-30	7.81	8.11	7.72	7.89	594862	{}	\N	2020-10-12 15:59:37.678255
66804	REVG	2020-10-01	7.98	8.1	7.75	8.05	471329	{}	\N	2020-10-12 15:59:37.684744
66805	REVG	2020-10-02	7.92	8.3	7.76	8.03	860888	{}	\N	2020-10-12 15:59:37.689952
66806	REVG	2020-10-05	8.18	8.41	8.01	8.38	586030	{}	\N	2020-10-12 15:59:37.694181
66807	REVG	2020-10-06	8.51	8.82	8.35	8.37	734272	{}	\N	2020-10-12 15:59:37.698782
66808	REVG	2020-10-07	8.49	8.86	8.4075	8.56	646304	{}	\N	2020-10-12 15:59:37.703602
66809	REVG	2020-10-08	8.65	8.81	8.35	8.66	718299	{}	\N	2020-10-12 15:59:37.707992
66810	REVG	2020-10-09	8.72	8.92	8.53	8.65	584269	{}	\N	2020-10-12 15:59:37.712087
66811	REVG	2020-10-12	8.63	8.81	8.58	8.77	137106	{}	\N	2020-10-12 15:59:37.716336
66812	SNPS	2020-09-28	210.51	212.84	209.26	212.64	655387	{}	\N	2020-10-12 15:59:38.831322
66813	SNPS	2020-09-29	213.12	215.4	211.8	212.32	598639	{}	\N	2020-10-12 15:59:38.842944
66814	SNPS	2020-09-30	211.21	216.86	210.8	213.98	672551	{}	\N	2020-10-12 15:59:38.850037
66815	SNPS	2020-10-01	216.37	219.69	216.01	217.09	676013	{}	\N	2020-10-12 15:59:38.85683
66816	SNPS	2020-10-02	212.86	216.01	210.331	212.05	695496	{}	\N	2020-10-12 15:59:38.863027
66817	SNPS	2020-10-05	213.12	216.66	213	216.64	524793	{}	\N	2020-10-12 15:59:38.868197
66818	SNPS	2020-10-06	215.6	218.66	213.08	214.34	658749	{}	\N	2020-10-12 15:59:38.87355
66819	SNPS	2020-10-07	215.71	220.39	214.78	219.53	481248	{}	\N	2020-10-12 15:59:38.878708
66820	SNPS	2020-10-08	221.43	221.6	218.45	219.03	364623	{}	\N	2020-10-12 15:59:38.883944
66821	SNPS	2020-10-09	220.99	224.5	220.86	222.82	541676	{}	\N	2020-10-12 15:59:38.889399
66822	SNPS	2020-10-12	225.65	229	223.58	228.75	224826	{}	\N	2020-10-12 15:59:38.894736
66823	RAIL	2020-09-28	2.25	2.37	2.235	2.31	100645	{}	\N	2020-10-12 15:59:40.25745
66824	RAIL	2020-09-29	2.33	2.44	2.3198	2.42	121154	{}	\N	2020-10-12 15:59:40.268512
66825	RAIL	2020-09-30	2.42	2.45	2.25	2.28	110664	{}	\N	2020-10-12 15:59:40.276051
66826	RAIL	2020-10-01	2.31	2.36	2.25	2.34	96434	{}	\N	2020-10-12 15:59:40.282042
66827	RAIL	2020-10-02	2.3	2.4397	2.3	2.38	97198	{}	\N	2020-10-12 15:59:40.287821
66828	RAIL	2020-10-05	2.43	2.5242	2.4	2.48	115328	{}	\N	2020-10-12 15:59:40.292802
66829	RAIL	2020-10-06	2.51	2.63	2.5089	2.54	144324	{}	\N	2020-10-12 15:59:40.297402
66830	RAIL	2020-10-07	2.56	2.72	2.56	2.66	198959	{}	\N	2020-10-12 15:59:40.302027
66831	RAIL	2020-10-08	2.7	2.85	2.68	2.78	240726	{}	\N	2020-10-12 15:59:40.306802
66832	RAIL	2020-10-09	2.8	2.8699	2.74	2.76	206813	{}	\N	2020-10-12 15:59:40.311625
66833	RAIL	2020-10-12	2.71	2.71	2.43	2.54	145148	{}	\N	2020-10-12 15:59:40.316303
66834	RAVN	2020-09-28	21.83	22.19	21.7	21.91	324668	{}	\N	2020-10-12 15:59:41.697239
66835	RAVN	2020-09-29	21.86	22.11	21.54	21.96	273080	{}	\N	2020-10-12 15:59:41.707148
66836	RAVN	2020-09-30	22.02	22.4175	21.41	21.52	208612	{}	\N	2020-10-12 15:59:41.714047
66837	RAVN	2020-10-01	21.53	21.945	20.83	21.27	206074	{}	\N	2020-10-12 15:59:41.720854
66838	RAVN	2020-10-02	20.84	21.92	20.84	21.67	156696	{}	\N	2020-10-12 15:59:41.72695
66839	RAVN	2020-10-05	21.88	22.095	21.56	21.74	109345	{}	\N	2020-10-12 15:59:41.732172
66840	RAVN	2020-10-06	21.86	22.54	21.51	22.03	234464	{}	\N	2020-10-12 15:59:41.737319
66841	RAVN	2020-10-07	22.1	22.571	21.98	22.22	244048	{}	\N	2020-10-12 15:59:41.743084
66842	RAVN	2020-10-08	22.52	22.85	22.3	22.6	136872	{}	\N	2020-10-12 15:59:41.747943
66843	RAVN	2020-10-09	22.93	22.98	22.42	22.69	138463	{}	\N	2020-10-12 15:59:41.752705
66844	RAVN	2020-10-12	22.67	23.1117	22.52	23.09	103897	{}	\N	2020-10-12 15:59:41.758221
66845	RBCAA	2020-09-28	28.16	28.919	28.16	28.45	16896	{}	\N	2020-10-12 15:59:43.425335
66846	RBCAA	2020-09-29	28.56	28.57	27.69	28.15	24077	{}	\N	2020-10-12 15:59:43.436715
66847	RBCAA	2020-09-30	28.54	28.6299	27.98	28.16	14531	{}	\N	2020-10-12 15:59:43.444159
66848	RBCAA	2020-10-01	28.31	28.63	27.22	27.98	23877	{}	\N	2020-10-12 15:59:43.449846
66849	RBCAA	2020-10-02	28.29	29.07	27.8	28.96	23290	{}	\N	2020-10-12 15:59:43.455031
66850	RBCAA	2020-10-05	29.2	30.38	29.03	30.38	16029	{}	\N	2020-10-12 15:59:43.460038
66851	RBCAA	2020-10-06	30.4	31.35	30.19	30.23	14726	{}	\N	2020-10-12 15:59:43.46453
66852	RBCAA	2020-10-07	30.39	30.99	30.26	30.76	14295	{}	\N	2020-10-12 15:59:43.468818
66853	RBCAA	2020-10-08	31.14	31.44	30.255	31.03	15879	{}	\N	2020-10-12 15:59:43.472764
66854	RBCAA	2020-10-09	31.25	31.46	30.7101	31.32	11486	{}	\N	2020-10-12 15:59:43.476881
66855	RBCAA	2020-10-12	31.07	31.55	30.9918	31.54	1646	{}	\N	2020-10-12 15:59:43.480737
66856	RCI	2020-09-28	39.24	39.63	38.99	39.63	347444	{}	\N	2020-10-12 15:59:44.462105
66857	RCI	2020-09-29	39.58	39.84	39.15	39.68	975595	{}	\N	2020-10-12 15:59:44.474675
66858	RCI	2020-09-30	39.87	39.92	39.51	39.65	645434	{}	\N	2020-10-12 15:59:44.482567
66859	RCI	2020-10-01	39.7	40.02	39.46	39.8	600215	{}	\N	2020-10-12 15:59:44.489216
66860	RCI	2020-10-02	39.27	40.21	39.2558	40.16	623042	{}	\N	2020-10-12 15:59:44.496141
66861	RCI	2020-10-05	40.46	40.51	39.73	40.29	679486	{}	\N	2020-10-12 15:59:44.501415
66862	RCI	2020-10-06	40.44	40.44	39.42	39.48	659845	{}	\N	2020-10-12 15:59:44.506681
66863	RCI	2020-10-07	39.5	40.27	39.5	40.2	580602	{}	\N	2020-10-12 15:59:44.511756
66864	RCI	2020-10-08	40.41	41.23	40.38	41.09	5246432	{}	\N	2020-10-12 15:59:44.51624
66865	RCI	2020-10-09	41.05	41.27	40.68	41	454596	{}	\N	2020-10-12 15:59:44.520714
66866	RCI	2020-10-12	41.08	41.29	40.81	41.27	26974	{}	\N	2020-10-12 15:59:44.525951
66867	RCON	2020-09-28	0.99	0.99	0.88299	0.968	32097	{}	\N	2020-10-12 15:59:45.479268
66868	RCON	2020-09-29	0.9508	1.28	0.9508	1.01	3751244	{}	\N	2020-10-12 15:59:45.491552
66869	RCON	2020-09-30	0.91	0.95	0.808	0.935	565928	{}	\N	2020-10-12 15:59:45.501117
66870	RCON	2020-10-01	0.92	0.935	0.862	0.888951	93534	{}	\N	2020-10-12 15:59:45.50746
66871	RCON	2020-10-02	0.889	0.8999	0.86	0.87505	92324	{}	\N	2020-10-12 15:59:45.514059
66872	RCON	2020-10-05	0.86	0.86	0.8102	0.849	327383	{}	\N	2020-10-12 15:59:45.519618
66873	RCON	2020-10-06	0.85	0.9282	0.84	0.88	126461	{}	\N	2020-10-12 15:59:45.524628
66874	RCON	2020-10-07	0.89	0.9	0.8744	0.895049	39072	{}	\N	2020-10-12 15:59:45.529822
66875	RCON	2020-10-08	0.8987	1.2	0.882101	1.09	1914011	{}	\N	2020-10-12 15:59:45.535
66876	RCON	2020-10-09	1.03	1.23	0.923	1.05	4390950	{}	\N	2020-10-12 15:59:45.540423
66877	RCON	2020-10-12	0.968	1.01	0.9204	0.9701	307579	{}	\N	2020-10-12 15:59:45.546145
66878	RDI	2020-09-28	3.68	3.6999	3.26	3.37	183807	{}	\N	2020-10-12 15:59:46.641759
66879	RDI	2020-09-29	3.32	3.32	3.08	3.2	144638	{}	\N	2020-10-12 15:59:46.654549
66880	RDI	2020-09-30	3.2	3.31	3.13	3.22	60452	{}	\N	2020-10-12 15:59:46.661617
66881	RDI	2020-10-01	3.24	3.295	3.22	3.26	24244	{}	\N	2020-10-12 15:59:46.668596
66882	RDI	2020-10-02	3.19	3.23	3.06	3.14	41619	{}	\N	2020-10-12 15:59:46.674225
66883	RDI	2020-10-05	3.15	3.15	3.01	3.03	73728	{}	\N	2020-10-12 15:59:46.679427
66884	RDI	2020-10-06	3.08	3.13	3.04	3.05	31402	{}	\N	2020-10-12 15:59:46.684413
66885	RDI	2020-10-07	3.05	3.075	3.01	3.04	81159	{}	\N	2020-10-12 15:59:46.68907
66886	RDI	2020-10-08	3.12	3.16	3.07	3.16	35086	{}	\N	2020-10-12 15:59:46.693693
66887	RDI	2020-10-09	3.0725	3.28	3	3.05	37783	{}	\N	2020-10-12 15:59:46.69835
66888	RDI	2020-10-12	3.04	3.11	3.02	3.0373	11368	{}	\N	2020-10-12 15:59:46.702991
66889	RDY	2020-09-28	69.6	69.6	68.53	69.01	263896	{}	\N	2020-10-12 15:59:48.561962
66890	RDY	2020-09-29	68.66	68.885	68.37	68.65	389748	{}	\N	2020-10-12 15:59:48.573584
66891	RDY	2020-09-30	69.77	70.24	69.32	69.56	784253	{}	\N	2020-10-12 15:59:48.58112
66892	RDY	2020-10-01	69.33	69.86	69.18	69.77	236597	{}	\N	2020-10-12 15:59:48.587521
66893	RDY	2020-10-02	69.06	69.8	68.56	69.31	152790	{}	\N	2020-10-12 15:59:48.593599
66894	RDY	2020-10-05	69.76	70.7504	69.76	70.46	578128	{}	\N	2020-10-12 15:59:48.5994
66895	RDY	2020-10-06	69.76	69.88	69.04	69.13	523945	{}	\N	2020-10-12 15:59:48.605067
66896	RDY	2020-10-07	69.06	69.11	68.57	68.65	613815	{}	\N	2020-10-12 15:59:48.610505
66897	RDY	2020-10-08	70.24	70.82	70.12	70.4	355584	{}	\N	2020-10-12 15:59:48.615974
66898	RDY	2020-10-09	70.06	70.32	69.7	70.03	234579	{}	\N	2020-10-12 15:59:48.621926
66899	RDY	2020-10-12	70.58	71.3	70.58	70.87	110764	{}	\N	2020-10-12 15:59:48.62676
66900	REFR	2020-09-28	2.53	2.7	2.48	2.65	145584	{}	\N	2020-10-12 15:59:49.661385
66901	REFR	2020-09-29	2.65	2.83	2.6345	2.81	129524	{}	\N	2020-10-12 15:59:49.674341
66902	REFR	2020-09-30	2.84	2.84	2.65	2.7	77309	{}	\N	2020-10-12 15:59:49.68318
66903	REFR	2020-10-01	2.68	2.81	2.62	2.78	86351	{}	\N	2020-10-12 15:59:49.690398
66904	REFR	2020-10-02	2.69	2.83	2.62	2.72	98832	{}	\N	2020-10-12 15:59:49.696756
66905	REFR	2020-10-05	2.76	2.8	2.63	2.68	86889	{}	\N	2020-10-12 15:59:49.702514
66906	REFR	2020-10-06	2.69	2.75	2.63	2.72	115659	{}	\N	2020-10-12 15:59:49.70826
66907	REFR	2020-10-07	2.72	2.79	2.67	2.72	84401	{}	\N	2020-10-12 15:59:49.714015
66908	REFR	2020-10-08	2.76	2.8	2.64	2.7	116485	{}	\N	2020-10-12 15:59:49.720125
66909	REFR	2020-10-09	2.69	2.94	2.67	2.9	168142	{}	\N	2020-10-12 15:59:49.725896
66910	REFR	2020-10-12	2.9	2.9	2.8	2.81	16786	{}	\N	2020-10-12 15:59:49.731823
66911	REET	2020-09-28	20.99	21.22	20.99	21.18	528921	{}	\N	2020-10-12 15:59:51.961674
66912	REET	2020-09-29	21.1	21.1	20.76	20.94	277470	{}	\N	2020-10-12 15:59:51.973277
66913	REET	2020-09-30	21	21.24	20.85	21.03	383083	{}	\N	2020-10-12 15:59:51.981355
66914	REET	2020-10-01	21.13	21.47	21.07	21.46	437254	{}	\N	2020-10-12 15:59:51.98816
66915	REET	2020-10-02	21.21	21.8	21.12	21.74	356029	{}	\N	2020-10-12 15:59:51.994273
66916	REET	2020-10-05	21.85	21.93	21.6	21.9	276101	{}	\N	2020-10-12 15:59:51.999598
66917	REET	2020-10-06	21.95	22.08	21.72	21.8	327623	{}	\N	2020-10-12 15:59:52.005162
66918	REET	2020-10-07	21.9	21.9	21.6911	21.78	1071001	{}	\N	2020-10-12 15:59:52.01016
66919	REET	2020-10-08	21.86	22.05	21.83	22.04	386349	{}	\N	2020-10-12 15:59:52.01548
66920	REET	2020-10-09	22.1439	22.1439	21.9	21.98	489012	{}	\N	2020-10-12 15:59:52.020946
66921	REET	2020-10-12	22.01	22.06	21.86	22.035	142172	{}	\N	2020-10-12 15:59:52.026119
66922	REGN	2020-09-28	572.83	574.615	559.06	571.92	622321	{}	\N	2020-10-12 15:59:53.159779
66923	REGN	2020-09-29	573.2	576	567.9186	573.61	505482	{}	\N	2020-10-12 15:59:53.171423
66924	REGN	2020-09-30	589.7668	589.7668	554	559.78	2155102	{}	\N	2020-10-12 15:59:53.179078
66925	REGN	2020-10-01	562.5	568.38	552.27	568.14	1058994	{}	\N	2020-10-12 15:59:53.18548
66926	REGN	2020-10-02	562	576	554.04	564.8	1225592	{}	\N	2020-10-12 15:59:53.191813
66927	REGN	2020-10-05	598.9	619.97	591.15	605.08	4871705	{}	\N	2020-10-12 15:59:53.19696
66928	REGN	2020-10-06	605.32	610.38	579.02	581.64	1694188	{}	\N	2020-10-12 15:59:53.202588
66929	REGN	2020-10-07	588.49	593.88	581.65	591.69	959342	{}	\N	2020-10-12 15:59:53.207799
66930	REGN	2020-10-08	614.645	615	598.098	599.88	2382388	{}	\N	2020-10-12 15:59:53.213117
66931	REGN	2020-10-09	601	604.44	593.505	602.07	1192920	{}	\N	2020-10-12 15:59:53.218618
66932	REGN	2020-10-12	602.2543	602.79	590.85	597.3477	448374	{}	\N	2020-10-12 15:59:53.223854
66933	REM	2020-09-28	25.65	26.445	25.65	26.24	974009	{}	\N	2020-10-12 15:59:54.504637
66934	REM	2020-09-29	26.32	26.32	25.53	25.79	428892	{}	\N	2020-10-12 15:59:54.515154
66935	REM	2020-09-30	25.75	26.2	25.6	25.76	389521	{}	\N	2020-10-12 15:59:54.522377
66936	REM	2020-10-01	25.83	26.04	25.5	26.04	893666	{}	\N	2020-10-12 15:59:54.528515
66937	REM	2020-10-02	25.53	26.4662	25.48	26.43	346435	{}	\N	2020-10-12 15:59:54.534
66938	REM	2020-10-05	26.43	26.68	26.38	26.59	402455	{}	\N	2020-10-12 15:59:54.539159
66939	REM	2020-10-06	26.74	27.06	26.32	26.34	623835	{}	\N	2020-10-12 15:59:54.543982
66940	REM	2020-10-07	26.66	26.662	26.2	26.48	340108	{}	\N	2020-10-12 15:59:54.548523
66941	REM	2020-10-08	26.52	27.22	26.52	27.2	512666	{}	\N	2020-10-12 15:59:54.553369
66942	REM	2020-10-09	27.34	27.34	26.68	26.77	365068	{}	\N	2020-10-12 15:59:54.558235
66943	REM	2020-10-12	26.73	26.9	26.69	26.79	206533	{}	\N	2020-10-12 15:59:54.563071
66944	REW	2020-09-28	19.58	19.8999	19.498	19.5457	33178	{}	\N	2020-10-12 15:59:57.311593
66945	REW	2020-09-29	19.43	19.688	19.4	19.51	14719	{}	\N	2020-10-12 15:59:57.324186
66946	REW	2020-09-30	19.4899	19.5	18.93	19.172	14065	{}	\N	2020-10-12 15:59:57.331832
66947	REW	2020-10-01	18.68	18.9522	18.66	18.7707	10288	{}	\N	2020-10-12 15:59:57.337999
66948	REW	2020-10-02	19.82	19.82	19.065	19.72	25373	{}	\N	2020-10-12 15:59:57.344379
66949	REW	2020-10-05	19.33	19.4	18.785	18.785	15395	{}	\N	2020-10-12 15:59:57.34982
66950	REW	2020-10-06	18.9	19.4083	18.82	19.355	9773	{}	\N	2020-10-12 15:59:57.354691
66951	REW	2020-10-07	18.91	19.061	18.65	18.6721	16522	{}	\N	2020-10-12 15:59:57.360073
66952	REW	2020-10-08	18.39	18.5036	18.351	18.5036	3747	{}	\N	2020-10-12 15:59:57.364764
66953	REW	2020-10-09	18.13	18.24	17.9152	17.9152	14118	{}	\N	2020-10-12 15:59:57.369317
66954	REW	2020-10-12	17.22	17.47	17.1538	17.1665	6453	{}	\N	2020-10-12 15:59:57.374526
66955	RFV	2020-09-28	54.9158	54.9158	54.8648	54.8648	724	{}	\N	2020-10-12 15:59:58.788253
66956	RFV	2020-09-29	53.7555	54.195	53.7555	53.99	2061	{}	\N	2020-10-12 15:59:58.79977
66957	RFV	2020-09-30	54.5	54.92	53.95	53.95	4663	{}	\N	2020-10-12 15:59:58.808299
66958	RFV	2020-10-01	54.665	55.26	54.425	55.165	24267	{}	\N	2020-10-12 15:59:58.814288
66959	RFV	2020-10-02	56.08	56.49	56.08	56.49	4595	{}	\N	2020-10-12 15:59:58.819168
66960	RFV	2020-10-05	57.635	58.0475	57.57	58.0475	32920	{}	\N	2020-10-12 15:59:58.824081
66961	RFV	2020-10-06	58.52	58.9617	57.5075	57.5075	4806	{}	\N	2020-10-12 15:59:58.828856
66962	RFV	2020-10-07	59.0883	59.27	59.0883	59.27	2157	{}	\N	2020-10-12 15:59:58.833651
66963	RFV	2020-10-08	59.7895	59.945	59.7895	59.91	1009	{}	\N	2020-10-12 15:59:58.838211
66964	RFV	2020-10-09	59.8013	60.17	59.77	59.77	1364	{}	\N	2020-10-12 15:59:58.843366
66965	RFV	2020-10-12	59.85	60.45	59.85	60.4	3476	{}	\N	2020-10-12 15:59:58.847877
66966	RGR	2020-09-28	61.95	62.35	60.4	62.08	268400	{}	\N	2020-10-12 15:59:59.806226
66967	RGR	2020-09-29	61.99	62.54	60.86	61.15	292230	{}	\N	2020-10-12 15:59:59.812479
66968	RGR	2020-09-30	60.75	62.54	60.7	61.16	565525	{}	\N	2020-10-12 15:59:59.818023
66969	RGR	2020-10-01	62.8	63.1	60.91	62.44	469108	{}	\N	2020-10-12 15:59:59.823378
66970	RGR	2020-10-02	61.5	63.75	61.5	62.71	447972	{}	\N	2020-10-12 15:59:59.828626
66971	RGR	2020-10-05	63.13	65.13	63	64.86	635208	{}	\N	2020-10-12 15:59:59.83423
66972	RGR	2020-10-06	65.32	66.2	63.41	64.5	741045	{}	\N	2020-10-12 15:59:59.839819
66973	RGR	2020-10-07	65	65.99	64.65	65.5	446062	{}	\N	2020-10-12 15:59:59.84682
66974	RGR	2020-10-08	66.11	66.55	64.15	65.05	475542	{}	\N	2020-10-12 15:59:59.852641
66975	RGR	2020-10-09	65.47	66.121	64.7501	65.59	407604	{}	\N	2020-10-12 15:59:59.859015
66976	RGR	2020-10-12	66.3	66.96	65.61	65.93	199279	{}	\N	2020-10-12 15:59:59.864171
66977	RICK	2020-09-28	18.35	19.85	18.35	19.41	93059	{}	\N	2020-10-12 16:00:01.045492
66978	RICK	2020-09-29	19.32	19.59	19.01	19.5	67033	{}	\N	2020-10-12 16:00:01.057044
66979	RICK	2020-09-30	19.75	20.53	19.7	20.4	87067	{}	\N	2020-10-12 16:00:01.064002
66980	RICK	2020-10-01	20.76	20.87	19.81	20.11	82069	{}	\N	2020-10-12 16:00:01.069911
66981	RICK	2020-10-02	19.41	20.64	18.66	20.5	63820	{}	\N	2020-10-12 16:00:01.07556
66982	RICK	2020-10-05	20.78	22.84	20.78	22.37	101113	{}	\N	2020-10-12 16:00:01.080902
66983	RICK	2020-10-06	21.815	23.46	21.815	22.57	98119	{}	\N	2020-10-12 16:00:01.085789
66984	RICK	2020-10-07	23.01	23.2999	22.59	23.13	52982	{}	\N	2020-10-12 16:00:01.05705
66985	RICK	2020-10-08	23.5	25.11	23.453	24.64	103564	{}	\N	2020-10-12 16:00:01.062261
66986	RICK	2020-10-09	24.82	25.4999	24.3	24.41	82497	{}	\N	2020-10-12 16:00:01.067039
66987	RICK	2020-10-12	24.24	24.66	23.425	24.63	31853	{}	\N	2020-10-12 16:00:01.071561
66988	RIO	2020-09-28	60.82	61.1	60.21	60.49	3116146	{}	\N	2020-10-12 16:00:03.591531
66989	RIO	2020-09-29	60.51	60.76	60.06	60.21	5122663	{}	\N	2020-10-12 16:00:03.604745
66990	RIO	2020-09-30	60.52	60.76	60.07	60.39	4396615	{}	\N	2020-10-12 16:00:03.612818
66991	RIO	2020-10-01	60.56	60.715	59.54	59.82	3300850	{}	\N	2020-10-12 16:00:03.619005
66992	RIO	2020-10-02	59.32	61.01	59.24	60.49	4894152	{}	\N	2020-10-12 16:00:03.624349
66993	RIO	2020-10-05	60.64	61.35	60.605	61.23	2578750	{}	\N	2020-10-12 16:00:03.629934
66994	RIO	2020-10-06	60.99	61.08	59.5	59.61	5635870	{}	\N	2020-10-12 16:00:03.634894
66995	RIO	2020-10-07	60.77	61.84	60.75	61.2	3916560	{}	\N	2020-10-12 16:00:03.639867
66996	RIO	2020-10-08	61.49	61.6	60.94	61.52	2734966	{}	\N	2020-10-12 16:00:03.644635
66997	RIO	2020-10-09	62.96	63.14	62.05	62.09	8035599	{}	\N	2020-10-12 16:00:03.649587
66998	RIO	2020-10-12	61.84	61.94	61.52	61.53	2794867	{}	\N	2020-10-12 16:00:03.654249
66999	RL	2020-09-28	72.59	72.72	70.29	70.44	1260715	{}	\N	2020-10-12 16:00:04.716635
67000	RL	2020-09-29	70.06	70.06	67.3	67.34	2432969	{}	\N	2020-10-12 16:00:04.728221
67001	RL	2020-09-30	67.65	68.72	67.11	67.97	3450762	{}	\N	2020-10-12 16:00:04.736353
67002	RL	2020-10-01	68.75	69.8	67.76	69.12	2435145	{}	\N	2020-10-12 16:00:04.742988
67003	RL	2020-10-02	67	70.17	66.85	69.93	3033564	{}	\N	2020-10-12 16:00:04.749565
67004	RL	2020-10-05	70.88	72.58	70.85	71.8	2381489	{}	\N	2020-10-12 16:00:04.755663
67005	RL	2020-10-06	72.86	72.97	69.12	69.28	2547852	{}	\N	2020-10-12 16:00:04.761534
67006	RL	2020-10-07	70	72.19	70	71.78	2886308	{}	\N	2020-10-12 16:00:04.767923
67007	RL	2020-10-08	72.53	74.38	72	74.33	2617114	{}	\N	2020-10-12 16:00:04.773648
67008	RL	2020-10-09	75	75.18	71.535	73.89	2830271	{}	\N	2020-10-12 16:00:04.778884
67009	RL	2020-10-12	72.96	75.22	72.62	74.36	934439	{}	\N	2020-10-12 16:00:04.784789
67010	RMBS	2020-09-28	13.47	13.8	13.405	13.75	410745	{}	\N	2020-10-12 16:00:07.197741
67011	RMBS	2020-09-29	13.67	13.94	13.635	13.74	346427	{}	\N	2020-10-12 16:00:07.211354
67012	RMBS	2020-09-30	13.76	13.91	13.53	13.69	597881	{}	\N	2020-10-12 16:00:07.218739
67013	RMBS	2020-10-01	13.82	13.9108	13.625	13.83	463645	{}	\N	2020-10-12 16:00:07.224299
67014	RMBS	2020-10-02	13.59	13.74	13.48	13.48	446834	{}	\N	2020-10-12 16:00:07.228984
67015	RMBS	2020-10-05	13.62	13.785	13.585	13.69	363241	{}	\N	2020-10-12 16:00:07.233644
67016	RMBS	2020-10-06	13.75	13.99	13.66	13.68	584908	{}	\N	2020-10-12 16:00:07.237759
67017	RMBS	2020-10-07	13.76	13.995	13.71	13.92	491575	{}	\N	2020-10-12 16:00:07.241838
67018	RMBS	2020-10-08	14.03	14.25	13.9794	14.16	498591	{}	\N	2020-10-12 16:00:07.246044
67019	RMBS	2020-10-09	14.32	14.49	14.22	14.45	445827	{}	\N	2020-10-12 16:00:07.250457
67020	RMBS	2020-10-12	14.5	14.575	14.385	14.415	127661	{}	\N	2020-10-12 16:00:07.254512
67021	RNEM	2020-09-28	41	41	40.641	40.68	2090	{}	\N	2020-10-12 16:00:08.644608
67022	RNEM	2020-09-29	40.5562	40.6156	40.5562	40.6156	350	{}	\N	2020-10-12 16:00:08.654099
67023	RNEM	2020-09-30	41.02	41.1799	41.0042	41.1201	2186	{}	\N	2020-10-12 16:00:08.660875
67024	RNEM	2020-10-01	41.3548	41.46	41.3548	41.401	1266	{}	\N	2020-10-12 16:00:08.666676
67025	RNEM	2020-10-07	41.54	41.73	41.54	41.73	1027	{}	\N	2020-10-12 16:00:08.671982
67026	RODM	2020-09-28	25.95	26.06	25.93	26.04	286647	{}	\N	2020-10-12 16:00:09.70618
67027	RODM	2020-09-29	26.035	26.09	25.92	26	338549	{}	\N	2020-10-12 16:00:09.715696
67028	RODM	2020-09-30	25.91	26.0517	25.83	25.88	650806	{}	\N	2020-10-12 16:00:09.722163
67029	RODM	2020-10-01	26.004	26.05	25.91	26.05	311406	{}	\N	2020-10-12 16:00:09.72779
67030	RODM	2020-10-02	25.77	26.04	25.77	25.99	354478	{}	\N	2020-10-12 16:00:09.733216
67031	RODM	2020-10-05	26.22	26.37	26.22	26.36	500995	{}	\N	2020-10-12 16:00:09.738795
67032	RODM	2020-10-06	26.35	26.4	26.0932	26.11	319101	{}	\N	2020-10-12 16:00:09.743671
67033	RODM	2020-10-07	26.27	26.3	26.1999	26.23	1532613	{}	\N	2020-10-12 16:00:09.748227
67034	RODM	2020-10-08	26.33	26.4	26.31	26.39	1097502	{}	\N	2020-10-12 16:00:09.75334
67035	RODM	2020-10-09	26.47	26.5699	26.4599	26.53	576081	{}	\N	2020-10-12 16:00:09.758112
67036	RODM	2020-10-12	26.59	26.66	26.5789	26.6299	132762	{}	\N	2020-10-12 16:00:09.762728
67037	ROP	2020-09-28	400.42	404.17	399.41	400.6	519558	{}	\N	2020-10-12 16:00:10.732132
67038	ROP	2020-09-29	401.62	404.29	398.2	401.36	685774	{}	\N	2020-10-12 16:00:10.745115
67039	ROP	2020-09-30	403.43	404.71	391.155	395.11	1293055	{}	\N	2020-10-12 16:00:10.752315
67040	ROP	2020-10-01	398.13	401.69	390.56	393.48	909376	{}	\N	2020-10-12 16:00:10.758439
67041	ROP	2020-10-02	390.03	398.685	389.7333	395.72	808587	{}	\N	2020-10-12 16:00:10.764104
67042	ROP	2020-10-05	399.39	401.615	397.06	401.2	599239	{}	\N	2020-10-12 16:00:10.769288
67043	ROP	2020-10-06	401.2	407	398.8	399.9	794244	{}	\N	2020-10-12 16:00:10.774118
67044	ROP	2020-10-07	402.12	407.8	402.12	405	850375	{}	\N	2020-10-12 16:00:10.778627
67045	ROP	2020-10-08	407.65	407.71	401.96	403.67	726974	{}	\N	2020-10-12 16:00:10.783135
67046	ROP	2020-10-09	406.57	417.87	405.23	414.39	803231	{}	\N	2020-10-12 16:00:10.787927
67047	ROP	2020-10-12	417.41	423.83	416	423.83	416983	{}	\N	2020-10-12 16:00:10.792636
67048	RPG	2020-09-28	140.56	141.6057	140.39	141.49	39570	{}	\N	2020-10-12 16:00:12.065741
67049	RPG	2020-09-29	141.39	142.18	141.12	141.3	53606	{}	\N	2020-10-12 16:00:12.076889
67050	RPG	2020-09-30	141.46	143	140.75	141.64	89839	{}	\N	2020-10-12 16:00:12.083904
67051	RPG	2020-10-01	142.81	143.52	142.43	143.2	42104	{}	\N	2020-10-12 16:00:12.09034
67052	RPG	2020-10-02	140.7467	142.37	140.7467	141.66	69195	{}	\N	2020-10-12 16:00:12.096472
67053	RPG	2020-10-05	142.8	144.41	142.77	144.31	23273	{}	\N	2020-10-12 16:00:12.101616
67054	RPG	2020-10-06	144.6	145.66	142.212	142.4	36737	{}	\N	2020-10-12 16:00:12.106994
67055	RPG	2020-10-07	144.08	145.37	144.08	145.08	33621	{}	\N	2020-10-12 16:00:12.111776
67056	RPG	2020-10-08	145.85	146.67	145.735	146.67	45385	{}	\N	2020-10-12 16:00:12.116394
67057	RPG	2020-10-09	147.48	148.44	147.48	148.02	36848	{}	\N	2020-10-12 16:00:12.12152
67058	RPG	2020-10-12	149.43	149.839	148.7016	149.7246	36502	{}	\N	2020-10-12 16:00:12.126312
67059	RFFC	2020-09-28	34.63	34.6981	34.63	34.6981	550	{}	\N	2020-10-12 16:00:13.221278
67060	RFFC	2020-09-29	34.87	34.87	34.66	34.72	2205	{}	\N	2020-10-12 16:00:13.234054
67061	RFFC	2020-09-30	35.05	35.21	34.75	34.95	9387	{}	\N	2020-10-12 16:00:13.241807
67062	RFFC	2020-10-01	35.21	35.3219	35.09	35.3219	7249	{}	\N	2020-10-12 16:00:13.248239
67063	RFFC	2020-10-02	35.11	35.26	35.11	35.26	4490	{}	\N	2020-10-12 16:00:13.254615
67064	RFFC	2020-10-05	35.55	35.88	35.5203	35.86	4226	{}	\N	2020-10-12 16:00:13.260126
67065	RFFC	2020-10-06	35.97	36.18	35.39	35.39	6825	{}	\N	2020-10-12 16:00:13.265163
67066	RFFC	2020-10-07	35.94	36.18	35.94	36.15	6910	{}	\N	2020-10-12 16:00:13.270804
67067	RFFC	2020-10-08	36.51	36.51	36.282	36.39	7370	{}	\N	2020-10-12 16:00:13.276453
67068	RFFC	2020-10-09	36.53	36.7865	36.53	36.74	9450	{}	\N	2020-10-12 16:00:13.281778
67069	RFFC	2020-10-12	36.96	37.19	36.94	37.145	14207	{}	\N	2020-10-12 16:00:13.286971
67070	RGCO	2020-09-28	23.33	23.495	23.16	23.37	6205	{}	\N	2020-10-12 16:00:15.029886
67071	RGCO	2020-09-29	23.507	23.75	23.19	23.7	17475	{}	\N	2020-10-12 16:00:15.042173
67072	RGCO	2020-09-30	23.6	23.84	23.44	23.45	3181	{}	\N	2020-10-12 16:00:15.048858
67073	RGCO	2020-10-01	23.45	23.69	23.35	23.47	7889	{}	\N	2020-10-12 16:00:15.054689
67074	RGCO	2020-10-02	23.39	23.68	23.39	23.47	4058	{}	\N	2020-10-12 16:00:15.060514
67075	RGCO	2020-10-05	23.52	23.93	23.43	23.63	11441	{}	\N	2020-10-12 16:00:15.065818
67076	RGCO	2020-10-06	23.68	23.85	23.27	23.27	6567	{}	\N	2020-10-12 16:00:15.071368
67077	RGCO	2020-10-07	23.812	23.92	23.72	23.72	6976	{}	\N	2020-10-12 16:00:15.076424
67078	RGCO	2020-10-08	23.89	24.28	23.74	23.86	7874	{}	\N	2020-10-12 16:00:15.081267
67079	RGCO	2020-10-09	23.95	24.25	23.815	24.14	6940	{}	\N	2020-10-12 16:00:15.085945
67080	RGCO	2020-10-12	23.95	23.95	23.8011	23.8821	4530	{}	\N	2020-10-12 16:00:15.090989
67081	RHP	2020-09-28	37.48	38.2824	36.65	37.6	376195	{}	\N	2020-10-12 16:00:16.791377
67082	RHP	2020-09-29	37.67	37.795	35.9236	36.37	677470	{}	\N	2020-10-12 16:00:16.803682
67083	RHP	2020-09-30	36.52	38.175	36.07	36.8	1112430	{}	\N	2020-10-12 16:00:16.810053
67084	RHP	2020-10-01	37.24	38.06	36.3	37.26	702182	{}	\N	2020-10-12 16:00:16.816623
67085	RHP	2020-10-02	35.75	37.98	35.41	37.81	495797	{}	\N	2020-10-12 16:00:16.822083
67086	RHP	2020-10-05	38.26	38.79	37.65	38.6	656590	{}	\N	2020-10-12 16:00:16.827188
67087	RHP	2020-10-06	38.82	39.725	38.17	38.75	824977	{}	\N	2020-10-12 16:00:16.831222
67088	RHP	2020-10-07	38.87	41.13	38.32	40.73	1193836	{}	\N	2020-10-12 16:00:16.835807
67089	RHP	2020-10-08	41.25	42.54	40.215	42.19	1581276	{}	\N	2020-10-12 16:00:16.840384
67090	RHP	2020-10-09	42.69	42.75	41.8001	42.55	718043	{}	\N	2020-10-12 16:00:16.844457
67091	RHP	2020-10-12	42.55	42.83	41.67	42.19	138127	{}	\N	2020-10-12 16:00:16.848891
67092	RILYZ	2020-09-28	24.49	24.71	24.37	24.45	32177	{}	\N	2020-10-12 16:00:18.139881
67093	RILYZ	2020-09-29	24.35	24.59	24.35	24.59	19996	{}	\N	2020-10-12 16:00:18.151904
67094	RILYZ	2020-09-30	24.65	24.72	24.48	24.72	23018	{}	\N	2020-10-12 16:00:18.158732
67095	RILYZ	2020-10-01	24.72	24.85	24.627	24.8	14976	{}	\N	2020-10-12 16:00:18.165095
67096	RILYZ	2020-10-02	24.8	24.8	24.635	24.75	5713	{}	\N	2020-10-12 16:00:18.170706
67097	RILYZ	2020-10-05	24.65	24.9521	24.65	24.94	10557	{}	\N	2020-10-12 16:00:18.175729
67098	RILYZ	2020-10-06	24.99	24.99	24.91	24.95	7180	{}	\N	2020-10-12 16:00:18.181637
67099	RILYZ	2020-10-07	24.9744	25.07	24.96	25.04	32190	{}	\N	2020-10-12 16:00:18.186466
67100	RILYZ	2020-10-08	25	25.066	25	25.05	11138	{}	\N	2020-10-12 16:00:18.191174
67101	RILYZ	2020-10-09	24.9601	25.035	24.95	24.9501	8318	{}	\N	2020-10-12 16:00:18.196248
67102	RILYZ	2020-10-12	25.07	25.07	24.95	24.99	4448	{}	\N	2020-10-12 16:00:18.201283
67103	RJF	2020-09-28	72.27	73.49	71.71	72.69	729135	{}	\N	2020-10-12 16:00:19.455158
67104	RJF	2020-09-29	72.49	72.96	71.78	72.3	1057992	{}	\N	2020-10-12 16:00:19.465325
67105	RJF	2020-09-30	72.14	73.56	71.99	72.76	1354849	{}	\N	2020-10-12 16:00:19.471186
67106	RJF	2020-10-01	73.44	74	71.44	72.69	1334140	{}	\N	2020-10-12 16:00:19.476916
67107	RJF	2020-10-02	71.52	75.18	71.28	74.52	1146249	{}	\N	2020-10-12 16:00:19.482281
67108	RJF	2020-10-05	75.4	76.55	75.35	76.16	1386387	{}	\N	2020-10-12 16:00:19.48729
67109	RJF	2020-10-06	76.77	77.08	74.65	75.09	969456	{}	\N	2020-10-12 16:00:19.491458
67110	RJF	2020-10-07	75.98	77.37	75.85	76.68	1177080	{}	\N	2020-10-12 16:00:19.495752
67111	RJF	2020-10-08	77.22	78.41	76.82	78.25	785091	{}	\N	2020-10-12 16:00:19.500188
67112	RJF	2020-10-09	79.17	79.2	77.54	78.27	728307	{}	\N	2020-10-12 16:00:19.504838
67113	RJF	2020-10-12	78.57	80.155	78.2	79.72	275391	{}	\N	2020-10-12 16:00:19.509275
67114	RLH	2020-09-28	2.18	2.28	2.18	2.23	28624	{}	\N	2020-10-12 16:00:20.742973
67115	RLH	2020-09-29	2.21	2.21	2.12	2.13	38010	{}	\N	2020-10-12 16:00:20.755217
67116	RLH	2020-09-30	2.11	2.17	2.11	2.15	33026	{}	\N	2020-10-12 16:00:20.761196
67117	RLH	2020-10-01	2.16	2.21	2.15	2.18	51794	{}	\N	2020-10-12 16:00:20.766951
67118	RLH	2020-10-02	2.155	2.215	2.09	2.16	85781	{}	\N	2020-10-12 16:00:20.772311
67119	RLH	2020-10-05	2.22	2.22	2.15	2.15	140503	{}	\N	2020-10-12 16:00:20.777162
67120	RLH	2020-10-06	2.17	2.25	2.11	2.12	95697	{}	\N	2020-10-12 16:00:20.78219
67121	RLH	2020-10-07	2.13	2.18	2.12	2.18	31766	{}	\N	2020-10-12 16:00:20.786645
67122	RLH	2020-10-08	2.17	2.22	2.1	2.13	92490	{}	\N	2020-10-12 16:00:20.79105
67123	RLH	2020-10-09	2.1378	2.19	2.01	2.08	198965	{}	\N	2020-10-12 16:00:20.796518
67124	RLH	2020-10-12	2.06	2.06	2	2.02	97749	{}	\N	2020-10-12 16:00:20.801424
67125	RMD	2020-09-28	171	174.2	169.85	172	577014	{}	\N	2020-10-12 16:00:21.740056
67126	RMD	2020-09-29	172	174.04	171.91	172.66	667772	{}	\N	2020-10-12 16:00:21.752518
67127	RMD	2020-09-30	173.05	174.83	170.6	171.43	1299057	{}	\N	2020-10-12 16:00:21.760329
67128	RMD	2020-10-01	172.84	174.43	168.59	169.51	966582	{}	\N	2020-10-12 16:00:21.76726
67129	RMD	2020-10-02	167.53	169.38	165.92	167.5	2234192	{}	\N	2020-10-12 16:00:21.773353
67130	RMD	2020-10-05	168.22	171.145	168.11	170.81	1300249	{}	\N	2020-10-12 16:00:21.778472
67131	RMD	2020-10-06	171.25	171.64	167.241	167.65	835016	{}	\N	2020-10-12 16:00:21.784356
67132	RMD	2020-10-07	169.25	171.55	168.92	170.7	835855	{}	\N	2020-10-12 16:00:21.78931
67133	RMD	2020-10-08	172.56	173.67	171.125	172.7	947398	{}	\N	2020-10-12 16:00:21.794081
67134	RMD	2020-10-09	173.5	176.72	173.27	176.12	882047	{}	\N	2020-10-12 16:00:21.799012
67135	RMD	2020-10-12	177.67	178.29	176.48	177	160487	{}	\N	2020-10-12 16:00:21.804061
67136	RNET	2020-09-28	4.2	4.37	3.94	4.03	128703	{}	\N	2020-10-12 16:00:23.335967
67137	RNET	2020-09-29	3.96	4.26	3.9331	4.23	339265	{}	\N	2020-10-12 16:00:23.346741
67138	RNET	2020-09-30	4.22	4.38	3.97	4.1	129366	{}	\N	2020-10-12 16:00:23.353325
67139	RNET	2020-10-01	4.13	4.34	4.13	4.24	81700	{}	\N	2020-10-12 16:00:23.359213
67140	RNET	2020-10-02	4.01	4.54	4.01	4.5	178565	{}	\N	2020-10-12 16:00:23.364742
67141	RNET	2020-10-05	4.7	4.83	4.42	4.7	262595	{}	\N	2020-10-12 16:00:23.370094
67142	RNET	2020-10-06	4.75	4.95	4.56	4.6	217084	{}	\N	2020-10-12 16:00:23.374598
67143	RNET	2020-10-07	4.65	4.795	4.3802	4.4	167760	{}	\N	2020-10-12 16:00:23.379055
67144	RNET	2020-10-08	4.55	4.55	4.318	4.41	87885	{}	\N	2020-10-12 16:00:23.383947
67145	RNET	2020-10-09	4.51	4.51	4.28	4.4	78962	{}	\N	2020-10-12 16:00:23.388608
67146	RNET	2020-10-12	4.4	4.4	4.1319	4.3	71855	{}	\N	2020-10-12 16:00:23.393499
67147	ROLL	2020-09-28	121	123.65	121	122.75	114003	{}	\N	2020-10-12 16:00:24.871172
67148	ROLL	2020-09-29	122.68	122.68	121.46	121.89	115642	{}	\N	2020-10-12 16:00:24.880944
67149	ROLL	2020-09-30	122.83	124.78	120.01	121.21	350845	{}	\N	2020-10-12 16:00:24.888007
67150	ROLL	2020-10-01	123.15	123.75	119.59	122.25	187651	{}	\N	2020-10-12 16:00:24.893374
67151	ROLL	2020-10-02	119.93	123.85	119.93	121.75	141222	{}	\N	2020-10-12 16:00:24.898175
67152	ROLL	2020-10-05	123.23	123.87	121.35	121.94	84506	{}	\N	2020-10-12 16:00:24.903148
67153	ROLL	2020-10-06	122.88	124.38	120.31	120.86	122242	{}	\N	2020-10-12 16:00:24.907775
67154	ROLL	2020-10-07	121.7	123.24	118.875	122.6	160507	{}	\N	2020-10-12 16:00:24.912106
67155	ROLL	2020-10-08	124.29	124.6	121.4801	122.97	108011	{}	\N	2020-10-12 16:00:24.916293
67156	ROLL	2020-10-09	124.28	127.46	122.64	125.87	106908	{}	\N	2020-10-12 16:00:24.920481
67157	ROLL	2020-10-12	125.79	128.045	124.72	128	34982	{}	\N	2020-10-12 16:00:24.924703
67158	ROST	2020-09-28	89.79	92.6736	88.82	92.26	1450139	{}	\N	2020-10-12 16:00:27.9324
67159	ROST	2020-09-29	91.85	92.14	90.2	91.03	1667214	{}	\N	2020-10-12 16:00:27.943095
67160	ROST	2020-09-30	91.55	94.63	91.12	93.32	1977813	{}	\N	2020-10-12 16:00:27.949679
67161	ROST	2020-10-01	94.11	96.27	93.95	94.83	1388731	{}	\N	2020-10-12 16:00:27.95502
67162	ROST	2020-10-02	93	96.72	92.23	96.22	1821067	{}	\N	2020-10-12 16:00:27.960498
67163	ROST	2020-10-05	96.61	97.295	95.905	97.08	1069658	{}	\N	2020-10-12 16:00:27.965288
67164	ROST	2020-10-06	97.41	97.5	94.215	94.64	1364512	{}	\N	2020-10-12 16:00:27.970718
67165	ROST	2020-10-07	95.92	97.645	95.54	97.14	1091998	{}	\N	2020-10-12 16:00:27.976925
67166	ROST	2020-10-08	97.27	98.03	96.2	97.86	893458	{}	\N	2020-10-12 16:00:27.982107
67167	ROST	2020-10-09	99.35	99.66	97.62	98.41	1495044	{}	\N	2020-10-12 16:00:27.987234
67168	ROST	2020-10-12	98.92	99.42	98.07	99.25	406325	{}	\N	2020-10-12 16:00:27.992343
67169	RPV	2020-09-28	48.85	49.59	48.85	49.25	60367	{}	\N	2020-10-12 16:00:28.924996
67170	RPV	2020-09-29	48.98	49.0399	48.1671	48.52	47336	{}	\N	2020-10-12 16:00:28.935277
67171	RPV	2020-09-30	48.75	49.62	48.75	49.02	118297	{}	\N	2020-10-12 16:00:28.942148
67172	RPV	2020-10-01	49.07	49.42	48.64	48.98	86093	{}	\N	2020-10-12 16:00:28.947998
67173	RPV	2020-10-02	47.99	49.87	47.99	49.62	82745	{}	\N	2020-10-12 16:00:28.95345
67174	RPV	2020-10-05	50.15	50.9	50.15	50.81	104899	{}	\N	2020-10-12 16:00:28.958648
67175	RPV	2020-10-06	51.11	51.64	50.06	50.14	128957	{}	\N	2020-10-12 16:00:28.963657
67176	RPV	2020-10-07	51.05	51.459	50.8746	51.3	81373	{}	\N	2020-10-12 16:00:28.968195
67177	RPV	2020-10-08	51.62	52.3	51.58	52.27	73319	{}	\N	2020-10-12 16:00:28.973091
67178	RPV	2020-10-09	52.6	52.72	51.96	52.02	85409	{}	\N	2020-10-12 16:00:28.978043
67179	RPV	2020-10-12	52.02	52.48	52.02	52.39	20245	{}	\N	2020-10-12 16:00:28.98315
67180	RUN	2020-09-28	67.89	72.2	66.78	71.98	5644815	{}	\N	2020-10-12 16:00:30.472345
67181	RUN	2020-09-29	72.89	75.2704	71.36	72.8	6071833	{}	\N	2020-10-12 16:00:30.48025
67182	RUN	2020-09-30	71.55	78.2199	70.53	77.07	8249595	{}	\N	2020-10-12 16:00:30.485807
67183	RUN	2020-10-01	79.415	82.42	77.77	79.97	14903298	{}	\N	2020-10-12 16:00:30.49135
67184	RUN	2020-10-02	77.51	80.34	75.84	76.45	8822792	{}	\N	2020-10-12 16:00:30.496024
67185	RUN	2020-10-05	78	79.86	76.17	78.62	7035542	{}	\N	2020-10-12 16:00:30.499945
67186	RUN	2020-10-06	77.845	80.2	74.86	78.57	7819736	{}	\N	2020-10-12 16:00:30.504416
67187	RUN	2020-10-07	80.29	82.37	77.5	78.05	13994230	{}	\N	2020-10-12 16:00:30.50921
67188	RUN	2020-10-08	77.13	77.4	68.8	70.54	10049365	{}	\N	2020-10-12 16:00:30.513742
67189	RUN	2020-10-09	71.35	74.49	70.21	71.43	9323539	{}	\N	2020-10-12 16:00:30.518224
67190	RUN	2020-10-12	72.1	72.1842	63.33	64.46	6185637	{}	\N	2020-10-12 16:00:30.522817
67191	RVNC	2020-09-28	26.36	26.94	25.77	25.99	505753	{}	\N	2020-10-12 16:00:32.418765
67192	RVNC	2020-09-29	25.85	26.37	25.32	25.71	420971	{}	\N	2020-10-12 16:00:32.433235
67193	RVNC	2020-09-30	25.8	26.09	25.05	25.14	625871	{}	\N	2020-10-12 16:00:32.442329
67194	RVNC	2020-10-01	25.24	25.75	24.79	25.3	476967	{}	\N	2020-10-12 16:00:32.449841
67195	RVNC	2020-10-02	24.55	25.93	24.25	24.73	702992	{}	\N	2020-10-12 16:00:32.455308
67196	RVNC	2020-10-05	25.11	25.8	24.61	25.66	786719	{}	\N	2020-10-12 16:00:32.460641
67197	RVNC	2020-10-06	25.75	26.48	25.39	25.51	570714	{}	\N	2020-10-12 16:00:32.466239
67198	RVNC	2020-10-07	25.92	27.46	25.63	27.13	693273	{}	\N	2020-10-12 16:00:32.4722
67199	RVNC	2020-10-08	27.37	27.89	26.45	27.75	506288	{}	\N	2020-10-12 16:00:32.47752
67200	RVNC	2020-10-09	27.9	28.8999	27	27.61	247911	{}	\N	2020-10-12 16:00:32.483051
67201	RVNC	2020-10-12	27.64	28.16	27.09	27.73	105588	{}	\N	2020-10-12 16:00:32.488042
67202	RFIL	2020-09-28	4.35	4.4993	4.34	4.48	16856	{}	\N	2020-10-12 16:00:33.52875
67203	RFIL	2020-09-29	4.45	4.52	4.3801	4.52	13911	{}	\N	2020-10-12 16:00:33.539955
67204	RFIL	2020-09-30	4.51	4.61	4.41	4.44	12106	{}	\N	2020-10-12 16:00:33.546535
67205	RFIL	2020-10-01	4.44	4.5	4.36	4.49	20392	{}	\N	2020-10-12 16:00:33.552529
67206	RFIL	2020-10-02	4.39	4.49	4.33	4.49	6072	{}	\N	2020-10-12 16:00:33.558234
67207	RFIL	2020-10-05	4.45	4.6	4.383	4.47	21733	{}	\N	2020-10-12 16:00:33.563388
67208	RFIL	2020-10-06	4.47	4.746	4.47	4.51	27934	{}	\N	2020-10-12 16:00:33.568372
67209	RFIL	2020-10-07	4.55	4.73	4.4676	4.47	17009	{}	\N	2020-10-12 16:00:33.573503
67210	RFIL	2020-10-08	4.53	4.5323	4.31	4.4	38466	{}	\N	2020-10-12 16:00:33.579009
67211	RFIL	2020-10-09	4.46	4.55	4.3984	4.5	16286	{}	\N	2020-10-12 16:00:33.584628
67212	RFIL	2020-10-12	4.55	4.55	4.4212	4.45	4011	{}	\N	2020-10-12 16:00:33.589894
67213	RGLD	2020-09-28	119.75	120.52	118.01	119.2	277792	{}	\N	2020-10-12 16:00:37.09959
67214	RGLD	2020-09-29	119.74	122.81	119.71	119.98	283057	{}	\N	2020-10-12 16:00:37.109257
67215	RGLD	2020-09-30	119.71	120.75	118.29	120.17	367878	{}	\N	2020-10-12 16:00:37.115811
67216	RGLD	2020-10-01	120.57	122.12	119.15	120.42	343061	{}	\N	2020-10-12 16:00:37.121293
67217	RGLD	2020-10-02	119.75	120.63	118.45	119.4	337945	{}	\N	2020-10-12 16:00:37.126478
67218	RGLD	2020-10-05	119.79	122.28	119.11	120.4	690043	{}	\N	2020-10-12 16:00:37.131899
67219	RGLD	2020-10-06	121.86	122.1	117.29	117.3	327682	{}	\N	2020-10-12 16:00:37.136214
67220	RGLD	2020-10-07	118.57	119.18	116.78	117.46	340274	{}	\N	2020-10-12 16:00:37.140342
67221	RGLD	2020-10-08	118.58	119.74	117.685	118.77	318781	{}	\N	2020-10-12 16:00:37.144528
67222	RGLD	2020-10-09	121.01	121.98	119.12	121.49	410345	{}	\N	2020-10-12 16:00:37.149047
67223	RGLD	2020-10-12	121.43	123.5	121	123.15	93194	{}	\N	2020-10-12 16:00:37.15389
67224	RHI	2020-09-28	53.27	53.65	53	53.24	523429	{}	\N	2020-10-12 16:00:38.247372
67225	RHI	2020-09-29	53.31	53.5	52.345	52.57	925968	{}	\N	2020-10-12 16:00:38.258408
67226	RHI	2020-09-30	52.67	53.97	52.2	52.94	2020954	{}	\N	2020-10-12 16:00:38.265295
67227	RHI	2020-10-01	53.14	53.68	51.925	52.54	1628105	{}	\N	2020-10-12 16:00:38.271304
67228	RHI	2020-10-02	51.87	54.81	51.55	54.57	1413223	{}	\N	2020-10-12 16:00:38.276307
67229	RHI	2020-10-05	55.32	55.89	54.95	55.6	1312231	{}	\N	2020-10-12 16:00:38.281317
67230	RHI	2020-10-06	55.67	56.56	54.96	55.07	1470562	{}	\N	2020-10-12 16:00:38.286256
67231	RHI	2020-10-07	55.75	57.15	55.7	57.07	1378342	{}	\N	2020-10-12 16:00:38.290655
67232	RHI	2020-10-08	57.31	57.78	57.17	57.74	1489598	{}	\N	2020-10-12 16:00:38.295105
67233	RHI	2020-10-09	58.27	58.81	57.88	58.14	1922987	{}	\N	2020-10-12 16:00:38.300162
67234	RHI	2020-10-12	58.73	58.93	58.45	58.7	346839	{}	\N	2020-10-12 16:00:38.304751
67235	RIGL	2020-09-28	2.32	2.33	2.27	2.31	2027658	{}	\N	2020-10-12 16:00:39.641252
67236	RIGL	2020-09-29	2.29	2.34	2.27	2.29	2174715	{}	\N	2020-10-12 16:00:39.651058
67237	RIGL	2020-09-30	2.28	2.46	2.28	2.4	2459529	{}	\N	2020-10-12 16:00:39.657952
67238	RIGL	2020-10-01	2.44	2.46	2.36	2.41	1541047	{}	\N	2020-10-12 16:00:39.663104
67239	RIGL	2020-10-02	2.36	2.42	2.34	2.35	1369942	{}	\N	2020-10-12 16:00:39.668615
67240	RIGL	2020-10-05	2.39	2.54	2.38	2.52	2601374	{}	\N	2020-10-12 16:00:39.673849
67241	RIGL	2020-10-06	2.55	2.641	2.49	2.52	2581096	{}	\N	2020-10-12 16:00:39.6781
67242	RIGL	2020-10-07	2.53	2.6	2.51	2.57	1681830	{}	\N	2020-10-12 16:00:39.682651
67243	RIGL	2020-10-08	2.57	2.6	2.51	2.59	1506584	{}	\N	2020-10-12 16:00:39.687422
67244	RIGL	2020-10-09	2.62	2.63	2.5	2.56	1589387	{}	\N	2020-10-12 16:00:39.69232
67245	RIGL	2020-10-12	2.53	2.57	2.48	2.485	505464	{}	\N	2020-10-12 16:00:39.696526
67246	RIOT	2020-09-28	2.79	2.81	2.66	2.74	3294263	{}	\N	2020-10-12 16:00:40.678492
67247	RIOT	2020-09-29	2.7	2.78	2.66	2.76	2028210	{}	\N	2020-10-12 16:00:40.690661
67248	RIOT	2020-09-30	2.73	2.7567	2.63	2.7	1956707	{}	\N	2020-10-12 16:00:40.697368
67249	RIOT	2020-10-01	2.71	2.74	2.62	2.65	1758019	{}	\N	2020-10-12 16:00:40.704012
67250	RIOT	2020-10-02	2.53	2.61	2.5	2.6	2141789	{}	\N	2020-10-12 16:00:40.710089
67251	RIOT	2020-10-05	2.63	2.68	2.57	2.66	2084702	{}	\N	2020-10-12 16:00:40.71538
67252	RIOT	2020-10-06	2.66	2.67	2.53	2.57	2463270	{}	\N	2020-10-12 16:00:40.72081
67253	RIOT	2020-10-07	2.57	2.66	2.5473	2.66	2196756	{}	\N	2020-10-12 16:00:40.7261
67254	RIOT	2020-10-08	2.77	3.0899	2.72	2.93	9975006	{}	\N	2020-10-12 16:00:40.731285
67255	RIOT	2020-10-09	3.15	3.15	2.96	3.095	6667174	{}	\N	2020-10-12 16:00:40.736251
67256	RIOT	2020-10-12	3.22	3.53	3.16	3.5114	6924075	{}	\N	2020-10-12 16:00:40.741287
67257	RKDA	2020-09-28	2.74	2.79	2.69	2.75	46640	{}	\N	2020-10-12 16:00:42.356082
67258	RKDA	2020-09-29	2.75	2.95	2.75	2.82	148135	{}	\N	2020-10-12 16:00:42.365952
67259	RKDA	2020-09-30	2.83	2.92	2.76	2.8	60139	{}	\N	2020-10-12 16:00:42.373276
67260	RKDA	2020-10-01	2.8	2.91	2.7984	2.88	59367	{}	\N	2020-10-12 16:00:42.37964
67261	RKDA	2020-10-02	2.76	2.889	2.76	2.87	31777	{}	\N	2020-10-12 16:00:42.385384
67262	RKDA	2020-10-05	2.86	3.3	2.82	3.2	849029	{}	\N	2020-10-12 16:00:42.39054
67263	RKDA	2020-10-06	3.2	3.205	2.9	3.01	285534	{}	\N	2020-10-12 16:00:42.395604
67264	RKDA	2020-10-07	3.05	3.42	3	3.42	420027	{}	\N	2020-10-12 16:00:42.400793
67265	RKDA	2020-10-08	3.4	3.42	3.13	3.17	151690	{}	\N	2020-10-12 16:00:42.40615
67266	RKDA	2020-10-09	3.24	3.36	3.13	3.23	165230	{}	\N	2020-10-12 16:00:42.411586
67267	RKDA	2020-10-12	3.2	3.3	3.2	3.3	13473	{}	\N	2020-10-12 16:00:42.416663
67268	RMBL	2020-09-28	28.06	28.38	26.97	27.69	99350	{}	\N	2020-10-12 16:00:45.004563
67269	RMBL	2020-09-29	27.92	28	25.3827	27.1	160511	{}	\N	2020-10-12 16:00:45.022433
67270	RMBL	2020-09-30	26.61	28	26.35	26.65	53835	{}	\N	2020-10-12 16:00:45.028582
67271	RMBL	2020-10-01	27.51	31.74	27.38	30.9	213222	{}	\N	2020-10-12 16:00:45.034736
67272	RMBL	2020-10-02	29	34.33	28.76	34.13	185194	{}	\N	2020-10-12 16:00:45.039877
67273	RMBL	2020-10-05	34	36	33.881	35.73	132064	{}	\N	2020-10-12 16:00:45.045332
67274	RMBL	2020-10-06	35.89	36.5	31.7539	33.15	118852	{}	\N	2020-10-12 16:00:45.049967
67275	RMBL	2020-10-07	34	36.25	33	36	88122	{}	\N	2020-10-12 16:00:45.054483
67276	RMBL	2020-10-08	36.15	36.8854	34.5101	35.22	50786	{}	\N	2020-10-12 16:00:45.059023
67277	RMBL	2020-10-09	36	36.39	32.8	34.22	71699	{}	\N	2020-10-12 16:00:45.063561
67278	RMBL	2020-10-12	35.01	36.02	33.7501	36.02	55787	{}	\N	2020-10-12 16:00:45.067912
67279	RNSC	2020-09-28	18.37	18.47	18.37	18.391	34602	{}	\N	2020-10-12 16:00:47.303486
67280	RNSC	2020-09-29	18.34	18.34	18.205	18.33	1640	{}	\N	2020-10-12 16:00:47.313104
67281	RNSC	2020-09-30	18.19	18.45	18.19	18.4049	4104	{}	\N	2020-10-12 16:00:47.319212
67282	RNSC	2020-10-01	18.31	18.454	18.31	18.454	5176	{}	\N	2020-10-12 16:00:47.324859
67283	RNSC	2020-10-02	18.08	18.7937	18.08	18.7937	1059	{}	\N	2020-10-12 16:00:47.32991
67284	RNSC	2020-10-05	18.8	19.16	18.8	19.16	780	{}	\N	2020-10-12 16:00:47.334412
67285	RNSC	2020-10-06	19.3863	19.3863	19.0698	19.0698	1122	{}	\N	2020-10-12 16:00:47.339006
67286	RNSC	2020-10-07	19.3002	19.395	19.2433	19.395	300	{}	\N	2020-10-12 16:00:47.343725
67287	RNSC	2020-10-08	19.5243	19.74	19.5243	19.74	1666	{}	\N	2020-10-12 16:00:47.348104
67288	RNSC	2020-10-09	19.77	19.77	19.66	19.73	687	{}	\N	2020-10-12 16:00:47.352638
67289	ROG	2020-09-28	97.87	100.17	97.21	99.1	96525	{}	\N	2020-10-12 16:00:48.550041
67290	ROG	2020-09-29	99.07	100.32	97.82	99.03	204839	{}	\N	2020-10-12 16:00:48.561258
67291	ROG	2020-09-30	99.28	100.56	97.64	98.06	180173	{}	\N	2020-10-12 16:00:48.56836
67292	ROG	2020-10-01	98.14	101.67	98.14	101.31	502292	{}	\N	2020-10-12 16:00:48.575118
67293	ROG	2020-10-02	99.07	101.7771	99.07	99.94	194581	{}	\N	2020-10-12 16:00:48.580646
67294	ROG	2020-10-05	100.83	102.315	99.89	101.41	159653	{}	\N	2020-10-12 16:00:48.585887
67295	ROG	2020-10-06	102.64	106.28	102.19	103.62	511734	{}	\N	2020-10-12 16:00:48.591133
67296	ROG	2020-10-07	105.81	106.67	103.61	103.75	292900	{}	\N	2020-10-12 16:00:48.596217
67297	ROG	2020-10-08	105.22	105.76	104.525	105.7	199300	{}	\N	2020-10-12 16:00:48.601417
67298	ROG	2020-10-09	107	110.71	107	110.32	473791	{}	\N	2020-10-12 16:00:48.606595
67299	ROG	2020-10-12	110.16	110.25	107.51	107.84	19266	{}	\N	2020-10-12 16:00:48.61151
67300	RPAI	2020-09-28	5.64	5.9	5.64	5.86	1485883	{}	\N	2020-10-12 16:00:50.422727
67301	RPAI	2020-09-29	5.85	5.85	5.53	5.66	2028624	{}	\N	2020-10-12 16:00:50.435171
67302	RPAI	2020-09-30	5.69	5.9	5.68	5.81	4089638	{}	\N	2020-10-12 16:00:50.442186
67303	RPAI	2020-10-01	5.82	6.07	5.78	6.02	3807687	{}	\N	2020-10-12 16:00:50.448368
67304	RPAI	2020-10-02	5.83	6.2	5.77	6.15	2998450	{}	\N	2020-10-12 16:00:50.454034
67305	RPAI	2020-10-05	6.22	6.27	6.01	6.12	2643084	{}	\N	2020-10-12 16:00:50.45931
67306	RPAI	2020-10-06	6.24	6.34	6.07	6.08	3138900	{}	\N	2020-10-12 16:00:50.464181
67307	RPAI	2020-10-07	6.15	6.19	5.98	6.01	3116289	{}	\N	2020-10-12 16:00:50.469092
67308	RPAI	2020-10-08	6.08	6.255	6	6.23	2273852	{}	\N	2020-10-12 16:00:50.473402
67309	RPAI	2020-10-09	6.28	6.33	5.9465	5.98	2179606	{}	\N	2020-10-12 16:00:50.478452
67310	RPAI	2020-10-12	5.97	6.08	5.83	6.055	616940	{}	\N	2020-10-12 16:00:50.483272
67311	RQI	2020-09-28	11.05	11.21	11.02	11.15	429694	{}	\N	2020-10-12 16:00:51.783629
67312	RQI	2020-09-29	11.16	11.16	10.9653	11.09	672919	{}	\N	2020-10-12 16:00:51.800506
67313	RQI	2020-09-30	11.14	11.261	11.01	11.11	693104	{}	\N	2020-10-12 16:00:51.815307
67314	RQI	2020-10-01	11.11	11.3499	11.11	11.34	687453	{}	\N	2020-10-12 16:00:51.824747
67315	RQI	2020-10-02	11.15	11.5	11.05	11.42	739391	{}	\N	2020-10-12 16:00:51.835691
67316	RQI	2020-10-05	11.51	11.535	11.3175	11.43	732529	{}	\N	2020-10-12 16:00:51.847015
67317	RQI	2020-10-06	11.47	11.5	11.34	11.4	646204	{}	\N	2020-10-12 16:00:51.857548
67318	RQI	2020-10-07	11.41	11.5	11.342	11.37	1168455	{}	\N	2020-10-12 16:00:51.866513
67319	RQI	2020-10-08	11.41	11.64	11.38	11.64	553579	{}	\N	2020-10-12 16:00:51.875255
67320	RQI	2020-10-09	11.67	11.74	11.64	11.68	545869	{}	\N	2020-10-12 16:00:51.884487
67321	RQI	2020-10-12	11.68	11.7	11.61	11.6732	366948	{}	\N	2020-10-12 16:00:51.892853
67322	SNV	2020-09-28	20.64	21.16	20.6	21.1	1074374	{}	\N	2020-10-12 16:00:52.911594
67323	SNV	2020-09-29	21.15	21.25	20.6	20.76	2327163	{}	\N	2020-10-12 16:00:52.921559
67324	SNV	2020-09-30	20.91	21.57	20.91	21.17	2404867	{}	\N	2020-10-12 16:00:52.928218
67325	SNV	2020-10-01	21.33	21.69	21.075	21.46	2057556	{}	\N	2020-10-12 16:00:52.93412
67326	SNV	2020-10-02	20.89	22.49	20.83	22.35	2588431	{}	\N	2020-10-12 16:00:52.939467
67327	SNV	2020-10-05	22.77	23.62	22.6	23.6	2461150	{}	\N	2020-10-12 16:00:52.944666
67328	SNV	2020-10-06	24	24.79	23.44	23.58	2767515	{}	\N	2020-10-12 16:00:52.949209
67329	SNV	2020-10-07	24.04	24.85	23.95	24.33	2601101	{}	\N	2020-10-12 16:00:52.953786
67330	SNV	2020-10-08	24.97	25.01	24.4	24.77	2151371	{}	\N	2020-10-12 16:00:52.958623
67331	SNV	2020-10-09	24.86	25.06	24.285	24.33	2048359	{}	\N	2020-10-12 16:00:52.963336
67332	SNV	2020-10-12	24.2	24.65	24.18	24.575	355884	{}	\N	2020-10-12 16:00:52.967668
67333	RFP	2020-09-28	4.33	4.46	4.26	4.36	251344	{}	\N	2020-10-12 16:00:55.108636
67334	RFP	2020-09-29	4.32	4.34	4.08	4.26	1039797	{}	\N	2020-10-12 16:00:55.124314
67335	RFP	2020-09-30	4.33	4.6976	4.33	4.48	1206875	{}	\N	2020-10-12 16:00:55.134231
67336	RFP	2020-10-01	4.54	4.79	4.46	4.58	924201	{}	\N	2020-10-12 16:00:55.142551
67337	RFP	2020-10-02	4.48	4.95	4.42	4.88	773851	{}	\N	2020-10-12 16:00:55.148804
67338	RFP	2020-10-05	4.95	5.11	4.9	5	568545	{}	\N	2020-10-12 16:00:55.155003
67339	RFP	2020-10-06	5.08	5.12	4.92	4.95	955165	{}	\N	2020-10-12 16:00:55.160961
67340	RFP	2020-10-07	5.06	5.19	5.0022	5.13	356284	{}	\N	2020-10-12 16:00:55.167336
67341	RFP	2020-10-08	5.15	5.29	5.07	5.27	441603	{}	\N	2020-10-12 16:00:55.172454
67342	RFP	2020-10-09	5.33	5.39	5.13	5.19	424739	{}	\N	2020-10-12 16:00:55.178086
67343	RFP	2020-10-12	5.14	5.23	5.01	5.12	156450	{}	\N	2020-10-12 16:00:55.18319
67344	RGNX	2020-09-28	28.14	28.21	26.84	27.36	394000	{}	\N	2020-10-12 16:00:56.639684
67345	RGNX	2020-09-29	27.65	28.23	27.256	27.91	273891	{}	\N	2020-10-12 16:00:56.651557
67346	RGNX	2020-09-30	28.05	28.55	27.02	27.52	300362	{}	\N	2020-10-12 16:00:56.658412
67347	RGNX	2020-10-01	27.4	27.92	26.855	27.53	300371	{}	\N	2020-10-12 16:00:56.664518
67348	RGNX	2020-10-02	27.09	27.89	26.52	26.52	209963	{}	\N	2020-10-12 16:00:56.671809
67349	RGNX	2020-10-05	26.84	28.55	26.69	28.13	532281	{}	\N	2020-10-12 16:00:56.677061
67350	RGNX	2020-10-06	28.15	28.37	27.51	27.58	305651	{}	\N	2020-10-12 16:00:56.682168
67351	RGNX	2020-10-07	27.52	28.2	27.33	28.11	293567	{}	\N	2020-10-12 16:00:56.686852
67352	RGNX	2020-10-08	28.59	29.12	27.98	29.05	691490	{}	\N	2020-10-12 16:00:56.691923
67353	RGNX	2020-10-09	29.095	29.8425	29.05	29.28	281889	{}	\N	2020-10-12 16:00:56.696985
67354	RGNX	2020-10-12	29.22	29.65	28.91	29.36	57055	{}	\N	2020-10-12 16:00:56.701973
67355	RILY	2020-09-28	24.77	25.5	24.77	25.2	69851	{}	\N	2020-10-12 16:00:59.880614
67356	RILY	2020-09-29	25.1	25.39	24.82	25.29	59558	{}	\N	2020-10-12 16:00:59.889468
67357	RILY	2020-09-30	25.41	25.59	24.75	25.06	78529	{}	\N	2020-10-12 16:00:59.895995
67358	RILY	2020-10-01	24.77	25.55	24.77	25.47	185016	{}	\N	2020-10-12 16:00:59.902033
67359	RILY	2020-10-02	25.26	26.15	25.26	25.8	52509	{}	\N	2020-10-12 16:00:59.907728
67360	RILY	2020-10-05	26.01	26.44	25.66	26.33	48644	{}	\N	2020-10-12 16:00:59.913226
67361	RILY	2020-10-06	26.61	27.13	25.95	26.38	66415	{}	\N	2020-10-12 16:00:59.918144
67362	RILY	2020-10-07	26.62	27.18	26.455	26.88	51179	{}	\N	2020-10-12 16:00:59.923326
67363	RILY	2020-10-08	27.14	27.39	26.89	26.97	77929	{}	\N	2020-10-12 16:00:59.928768
67364	RILY	2020-10-09	27.24	27.39	26.82	26.98	42721	{}	\N	2020-10-12 16:00:59.934249
67365	RILY	2020-10-12	26.91	27	26.85	26.93	7990	{}	\N	2020-10-12 16:00:59.939802
67366	RIV	2020-09-28	14.66	14.745	14.38	14.44	55762	{}	\N	2020-10-12 16:01:00.923938
67367	RIV	2020-09-29	14.33	14.49	13.98	14.1	248014	{}	\N	2020-10-12 16:01:00.935818
67368	RIV	2020-09-30	14.12	14.43	14.12	14.3336	205906	{}	\N	2020-10-12 16:01:00.943371
67369	RIV	2020-10-01	14.33	14.5627	14.31	14.39	112068	{}	\N	2020-10-12 16:01:00.950239
67370	RIV	2020-10-02	14.3124	14.33	14.15	14.263	161649	{}	\N	2020-10-12 16:01:00.956236
67371	RIV	2020-10-05	14.28	14.5504	14.28	14.41	101163	{}	\N	2020-10-12 16:01:00.961621
67372	RIV	2020-10-06	14.476	14.476	14.32	14.34	88253	{}	\N	2020-10-12 16:01:00.967538
67373	RIV	2020-10-07	14.34	14.5	14.34	14.5	109859	{}	\N	2020-10-12 16:01:00.973015
67374	RIV	2020-10-08	14.52	14.52	14.34	14.405	120173	{}	\N	2020-10-12 16:01:00.978235
67375	RIV	2020-10-09	14.5	14.52	14.453	14.48	67069	{}	\N	2020-10-12 16:01:00.983362
67376	RIV	2020-10-12	14.46	14.5	14.42	14.4322	77736	{}	\N	2020-10-12 16:01:00.988657
67377	RLGY	2020-09-28	9.3999	9.6706	9.3758	9.53	1142370	{}	\N	2020-10-12 16:01:02.224678
67378	RLGY	2020-09-29	9.55	9.96	9.425	9.5	2468328	{}	\N	2020-10-12 16:01:02.235975
67379	RLGY	2020-09-30	9.46	9.82	9.42	9.44	5032368	{}	\N	2020-10-12 16:01:02.243461
67380	RLGY	2020-10-01	9.55	10.1	9.47	10.08	2647517	{}	\N	2020-10-12 16:01:02.250248
67381	RLGY	2020-10-02	9.75	10.84	9.711	10.75	2535995	{}	\N	2020-10-12 16:01:02.256816
67382	RLGY	2020-10-05	10.89	11.15	10.72	10.75	2524812	{}	\N	2020-10-12 16:01:02.261834
67383	RLGY	2020-10-06	10.87	11.04	10.62	10.64	3637232	{}	\N	2020-10-12 16:01:02.267037
67384	RLGY	2020-10-07	10.94	11.53	10.9	11.09	5022031	{}	\N	2020-10-12 16:01:02.272313
67385	RLGY	2020-10-08	11.19	11.2999	10.69	10.82	2760984	{}	\N	2020-10-12 16:01:02.277358
67386	RLGY	2020-10-09	10.96	11.2	10.83	10.97	3046845	{}	\N	2020-10-12 16:01:02.282446
67387	RLGY	2020-10-12	11.02	11.125	10.75	11.05	705995	{}	\N	2020-10-12 16:01:02.28788
67388	RMCF	2020-09-28	2.75	2.79	2.68	2.7	25012	{}	\N	2020-10-12 16:01:03.219826
67389	RMCF	2020-09-29	2.67	2.71	2.65	2.67	50859	{}	\N	2020-10-12 16:01:03.232681
67390	RMCF	2020-09-30	2.72	2.76	2.68	2.68	27090	{}	\N	2020-10-12 16:01:03.240647
67391	RMCF	2020-10-01	2.73	2.73	2.6701	2.68	14712	{}	\N	2020-10-12 16:01:03.247499
67392	RMCF	2020-10-02	2.56	2.66	2.55	2.62	48763	{}	\N	2020-10-12 16:01:03.253976
67393	RMCF	2020-10-05	2.69	2.69	2.6	2.64	26756	{}	\N	2020-10-12 16:01:03.25934
67394	RMCF	2020-10-06	2.67	2.9443	2.65	2.84	101247	{}	\N	2020-10-12 16:01:03.265201
67395	RMCF	2020-10-07	2.85	2.94	2.8	2.8856	46041	{}	\N	2020-10-12 16:01:03.270797
67396	RMCF	2020-10-08	2.87	2.99	2.8	2.98	44089	{}	\N	2020-10-12 16:01:03.276289
67397	RMCF	2020-10-09	3	3	2.8	2.91	27821	{}	\N	2020-10-12 16:01:03.281921
67398	RMCF	2020-10-12	2.91	2.9509	2.7701	2.85	13556	{}	\N	2020-10-12 16:01:03.287276
67399	RRC	2020-09-28	7.22	7.385	6.96	7.3	5467955	{}	\N	2020-10-12 16:01:05.787577
67400	RRC	2020-09-29	7.22	7.25	6.7925	6.85	10310676	{}	\N	2020-10-12 16:01:05.798954
67401	RRC	2020-09-30	6.83	6.8799	6.51	6.62	10201002	{}	\N	2020-10-12 16:01:05.806831
67402	RRC	2020-10-01	6.56	6.745	6.31	6.49	7927725	{}	\N	2020-10-12 16:01:05.813382
67403	RRC	2020-10-02	6.29	6.64	6.225	6.5	8241181	{}	\N	2020-10-12 16:01:05.820305
67404	RRC	2020-10-05	6.66	7.11	6.66	7.06	8374183	{}	\N	2020-10-12 16:01:05.82606
67405	RRC	2020-10-06	7.16	7.541	6.99	7.03	13503263	{}	\N	2020-10-12 16:01:05.831592
67406	RRC	2020-10-07	7.2	7.95	7.2	7.88	11645308	{}	\N	2020-10-12 16:01:05.837237
67407	RRC	2020-10-08	7.9	8.1	7.725	8.09	10102995	{}	\N	2020-10-12 16:01:05.843059
67408	RRC	2020-10-09	8.33	8.52	7.93	8.06	8007980	{}	\N	2020-10-12 16:01:05.848404
67409	RRC	2020-10-12	8.3	8.375	8.04	8.36	4179568	{}	\N	2020-10-12 16:01:05.855299
67410	RSP	2020-09-28	107.75	108.81	107.63	108.44	1285341	{}	\N	2020-10-12 16:01:06.920526
67411	RSP	2020-09-29	108.42	108.46	107.03	107.57	1056621	{}	\N	2020-10-12 16:01:06.931733
67412	RSP	2020-09-30	107.9	109.2299	107.405	108.11	1756491	{}	\N	2020-10-12 16:01:06.938471
67413	RSP	2020-10-01	108.65	109.18	107.71	108.5	859783	{}	\N	2020-10-12 16:01:06.943974
67414	RSP	2020-10-02	106.77	109.4	106.7059	108.98	1471380	{}	\N	2020-10-12 16:01:06.9487
67415	RSP	2020-10-05	109.72	111.01	109.72	110.89	1061540	{}	\N	2020-10-12 16:01:06.954469
67416	RSP	2020-10-06	111.45	112.3899	109.57	109.75	2338115	{}	\N	2020-10-12 16:01:06.958602
67417	RSP	2020-10-07	110.84	112.06	110.84	111.75	2646276	{}	\N	2020-10-12 16:01:06.963208
67418	RSP	2020-10-08	112.41	113.42	112.28	113.4	4522711	{}	\N	2020-10-12 16:01:06.967547
67419	RSP	2020-10-09	114.02	114.31	113.23	113.57	1799556	{}	\N	2020-10-12 16:01:06.971744
67420	RSP	2020-10-12	114.08	114.46	113.743	114.33	520244	{}	\N	2020-10-12 16:01:06.976206
67421	RVNU	2020-09-28	28.52	28.52	28.4501	28.485	4139	{}	\N	2020-10-12 16:01:08.439325
67422	RVNU	2020-09-29	28.49	28.49	28.4181	28.47	26393	{}	\N	2020-10-12 16:01:08.450577
67423	RVNU	2020-09-30	28.47	28.5	28.3701	28.3701	10217	{}	\N	2020-10-12 16:01:08.458926
67424	RVNU	2020-10-01	28.3169	28.3662	28.3169	28.36	5714	{}	\N	2020-10-12 16:01:08.465248
67425	RVNU	2020-10-02	28.35	28.35	28.27	28.35	20197	{}	\N	2020-10-12 16:01:08.471243
67426	RVNU	2020-10-05	28.27	28.3	28.2	28.2	34301	{}	\N	2020-10-12 16:01:08.476203
67427	RVNU	2020-10-06	28.28	28.3	28.2301	28.2301	14959	{}	\N	2020-10-12 16:01:08.481367
67428	RVNU	2020-10-07	28.2099	28.21	28.126	28.126	15221	{}	\N	2020-10-12 16:01:08.486398
67429	RVNU	2020-10-08	28.07	28.1899	28.07	28.1399	7521	{}	\N	2020-10-12 16:01:08.491427
67430	RVNU	2020-10-09	28.22	28.22	28.09	28.2	23839	{}	\N	2020-10-12 16:01:08.496671
67431	RVNU	2020-10-12	28.22	28.22	28.2	28.2	10075	{}	\N	2020-10-12 16:01:08.50194
67432	RGA	2020-09-28	94.59	98.12	94.59	97.22	503005	{}	\N	2020-10-12 16:01:11.496628
67433	RGA	2020-09-29	96.71	97.005	93.565	95.28	798284	{}	\N	2020-10-12 16:01:11.507732
67434	RGA	2020-09-30	95.53	96.82	94.45	95.19	704658	{}	\N	2020-10-12 16:01:11.51385
67435	RGA	2020-10-01	95.74	97.12	94.935	96.87	630377	{}	\N	2020-10-12 16:01:11.518923
67436	RGA	2020-10-02	95.225	99.705	95.09	99.14	614811	{}	\N	2020-10-12 16:01:11.524201
67437	RGA	2020-10-05	100.91	102.57	100.3253	101.86	820144	{}	\N	2020-10-12 16:01:11.529059
67438	RGA	2020-10-06	102.95	103.51	99.3	99.87	712004	{}	\N	2020-10-12 16:01:11.533481
67439	RGA	2020-10-07	101.99	103.58	100.51	101.75	769806	{}	\N	2020-10-12 16:01:11.537361
67440	RGA	2020-10-08	103.36	105	102.3	104.45	621132	{}	\N	2020-10-12 16:01:11.541468
67441	RGA	2020-10-09	105.27	106.4	103.815	105.68	903212	{}	\N	2020-10-12 16:01:11.545419
67442	RGA	2020-10-12	105.83	108.58	105.39	108.185	247291	{}	\N	2020-10-12 16:01:11.549511
67443	RGS	2020-09-28	5.58	5.845	5.58	5.76	461906	{}	\N	2020-10-12 16:01:13.397565
67444	RGS	2020-09-29	5.74	6.01	5.51	6	1140361	{}	\N	2020-10-12 16:01:13.409398
67445	RGS	2020-09-30	6.05	6.34	6	6.14	1110645	{}	\N	2020-10-12 16:01:13.417002
67446	RGS	2020-10-01	6.14	6.25	5.97	6.23	776993	{}	\N	2020-10-12 16:01:13.423211
67447	RGS	2020-10-02	5.99	6.44	5.89	6.38	743346	{}	\N	2020-10-12 16:01:13.42899
67448	RGS	2020-10-05	6.38	6.59	6.32	6.53	698633	{}	\N	2020-10-12 16:01:13.43484
67449	RGS	2020-10-06	6.63	7.24	6.5835	6.72	1504461	{}	\N	2020-10-12 16:01:13.439691
67450	RGS	2020-10-07	6.72	7.04	6.68	6.96	971137	{}	\N	2020-10-12 16:01:13.444984
67451	RGS	2020-10-08	7.04	7.38	7.04	7.17	1024617	{}	\N	2020-10-12 16:01:13.450425
67452	RGS	2020-10-09	7.29	7.34	6.8201	7	1102481	{}	\N	2020-10-12 16:01:13.455263
67453	RGS	2020-10-12	6.96	7.33	6.92	7.11	383297	{}	\N	2020-10-12 16:01:13.460474
67454	RIG	2020-09-28	0.8425	0.9166	0.84	0.8756	22560316	{}	\N	2020-10-12 16:01:14.463087
67455	RIG	2020-09-29	0.882	0.8899	0.8015	0.8428	38593456	{}	\N	2020-10-12 16:01:14.47344
67456	RIG	2020-09-30	0.829	0.8644	0.8012	0.8069	35941946	{}	\N	2020-10-12 16:01:14.481186
67457	RIG	2020-10-01	0.8101	0.815	0.7725	0.7762	41710329	{}	\N	2020-10-12 16:01:14.488438
67458	RIG	2020-10-02	0.749	0.8144	0.7319	0.7976	44085594	{}	\N	2020-10-12 16:01:14.494343
67459	RIG	2020-10-05	0.8204	0.85	0.7808	0.7977	33564522	{}	\N	2020-10-12 16:01:14.499425
67460	RIG	2020-10-06	0.8236	0.835	0.786	0.788	24636659	{}	\N	2020-10-12 16:01:14.504651
67461	RIG	2020-10-07	0.791	0.833	0.7772	0.8122	27582808	{}	\N	2020-10-12 16:01:14.510408
67462	RIG	2020-10-08	0.8344	0.9	0.825	0.899	45879870	{}	\N	2020-10-12 16:01:14.515648
67463	RIG	2020-10-09	0.905	0.93	0.83	0.8638	32662022	{}	\N	2020-10-12 16:01:14.521204
67464	RIG	2020-10-12	0.84	0.848	0.8115	0.8172	12482206	{}	\N	2020-10-12 16:01:14.527519
67465	RISE	2020-09-28	20.07	20.11	20.02	20.11	121702	{}	\N	2020-10-12 16:01:16.934923
67466	RISE	2020-09-30	20.105	20.105	20.105	20.105	0	{}	\N	2020-10-12 16:01:16.943485
67467	RISE	2020-10-01	20.1299	20.135	20.1299	20.135	585	{}	\N	2020-10-12 16:01:16.94987
67468	RISE	2020-10-02	20.125	20.125	20.125	20.125	0	{}	\N	2020-10-12 16:01:16.955475
67469	RISE	2020-10-05	20.1853	20.1853	20.1853	20.1853	4	{}	\N	2020-10-12 16:01:16.960596
67470	RISE	2020-10-06	20.21	20.21	20.21	20.21	0	{}	\N	2020-10-12 16:01:16.965619
67471	RISE	2020-10-07	20.22	20.22	20.22	20.22	1	{}	\N	2020-10-12 16:01:16.970455
67472	RISE	2020-10-08	20.21	20.21	20.21	20.21	0	{}	\N	2020-10-12 16:01:16.975136
67473	RISE	2020-10-09	20.215	20.215	20.215	20.215	0	{}	\N	2020-10-12 16:01:16.979877
67474	RLI	2020-09-28	83.73	85.09	83.655	84.52	120945	{}	\N	2020-10-12 16:01:18.076934
67475	RLI	2020-09-29	84.47	84.97	83.33	84.22	176101	{}	\N	2020-10-12 16:01:18.090251
67476	RLI	2020-09-30	83.84	84.49	82.95	83.73	317374	{}	\N	2020-10-12 16:01:18.098505
67477	RLI	2020-10-01	83.44	84.46	82.8	84.33	292268	{}	\N	2020-10-12 16:01:18.104183
67478	RLI	2020-10-02	83.37	85.21	83.37	84.35	241101	{}	\N	2020-10-12 16:01:18.108882
67479	RLI	2020-10-05	84.92	85.95	84.485	85.58	216754	{}	\N	2020-10-12 16:01:18.113391
67480	RLI	2020-10-06	86.34	86.78	84.79	84.86	228536	{}	\N	2020-10-12 16:01:18.117747
67481	RLI	2020-10-07	85.41	85.93	84.23	84.99	215369	{}	\N	2020-10-12 16:01:18.122574
67482	RLI	2020-10-08	85.66	86.9	84.99	85.36	217334	{}	\N	2020-10-12 16:01:18.126876
67483	RLI	2020-10-09	85.34	86.68	85.29	85.72	197612	{}	\N	2020-10-12 16:01:18.131751
67484	RLI	2020-10-12	85.44	88.01	85.44	87.81	54733	{}	\N	2020-10-12 16:01:18.136312
67485	RMNI	2020-09-28	3.35	3.35	3.25	3.27	149947	{}	\N	2020-10-12 16:01:19.138831
67486	RMNI	2020-09-29	3.29	3.29	3.2	3.22	62504	{}	\N	2020-10-12 16:01:19.14985
67487	RMNI	2020-09-30	3.22	3.26	3.2	3.22	102570	{}	\N	2020-10-12 16:01:19.156945
67488	RMNI	2020-10-01	3.2	3.24	3.16	3.22	163373	{}	\N	2020-10-12 16:01:19.163302
67489	RMNI	2020-10-02	3.24	3.26	3.17	3.22	147892	{}	\N	2020-10-12 16:01:19.169421
67490	RMNI	2020-10-05	3.24	3.31	3.22	3.24	130748	{}	\N	2020-10-12 16:01:19.17458
67491	RMNI	2020-10-06	3.24	3.27	3.16	3.18	81969	{}	\N	2020-10-12 16:01:19.179624
67492	RMNI	2020-10-07	3.2	3.26	3.19	3.24	126707	{}	\N	2020-10-12 16:01:19.184958
67493	RMNI	2020-10-08	3.25	3.4	3.24	3.38	180056	{}	\N	2020-10-12 16:01:19.189955
67494	RMNI	2020-10-09	3.41	3.42	3.33	3.35	80941	{}	\N	2020-10-12 16:01:19.194913
67495	RMNI	2020-10-12	3.35	3.4	3.28	3.3119	36969	{}	\N	2020-10-12 16:01:19.200012
67496	RNMC	2020-09-28	19.27	19.74	19.27	19.621	43088	{}	\N	2020-10-12 16:01:20.729587
67497	RNMC	2020-09-29	19.33	19.4	19.32	19.4	993	{}	\N	2020-10-12 16:01:20.73853
67498	RNMC	2020-09-30	19.15	19.675	19.15	19.675	5634	{}	\N	2020-10-12 16:01:20.745275
67499	RNMC	2020-10-01	19.67	19.76	19.62	19.65	6546	{}	\N	2020-10-12 16:01:20.750954
67500	RNMC	2020-10-02	19.75	19.9279	19.75	19.9279	1284	{}	\N	2020-10-12 16:01:20.756028
67501	RNMC	2020-10-05	20.1769	20.3072	20.14	20.3072	990	{}	\N	2020-10-12 16:01:20.760616
67502	RNMC	2020-10-06	20.5288	20.5288	20.277	20.277	494	{}	\N	2020-10-12 16:01:20.765055
67503	RNMC	2020-10-07	20.5468	20.6397	20.5253	20.6397	477	{}	\N	2020-10-12 16:01:20.769517
67504	RNMC	2020-10-08	20.8929	20.95	20.865	20.9196	2347	{}	\N	2020-10-12 16:01:20.773867
67505	RNMC	2020-10-09	20.905	20.94	20.905	20.94	383	{}	\N	2020-10-12 16:01:20.778342
67506	RODI	2020-09-28	82.0689	82.0689	82.0689	82.0689	100	{}	\N	2020-10-12 16:01:23.186619
67507	RODI	2020-09-29	81.4781	81.4781	81.4781	81.4781	100	{}	\N	2020-10-12 16:01:23.193133
67508	RODI	2020-09-30	82.4975	82.4975	82.4975	82.4975	100	{}	\N	2020-10-12 16:01:23.19793
67509	RODI	2020-10-01	82.5439	82.5439	82.5439	82.5439	100	{}	\N	2020-10-12 16:01:23.202006
67510	RODI	2020-10-02	82.9308	82.9308	82.9308	82.9308	100	{}	\N	2020-10-12 16:01:23.20632
67511	RODI	2020-10-05	84.28	84.28	84.28	84.28	100	{}	\N	2020-10-12 16:01:23.210875
67512	RODI	2020-10-06	83.25	83.25	83.25	83.25	100	{}	\N	2020-10-12 16:01:23.215215
67513	RODI	2020-10-07	84.8659	84.8659	84.8659	84.8659	100	{}	\N	2020-10-12 16:01:23.219586
67514	RODI	2020-10-08	85.8217	85.8217	85.8217	85.8217	100	{}	\N	2020-10-12 16:01:23.224152
67515	RODI	2020-10-09	86.0041	86.0041	86.0041	86.0041	100	{}	\N	2020-10-12 16:01:23.229129
67516	ROOF	2020-09-28	17.41	17.9	17.41	17.85	16286	{}	\N	2020-10-12 16:01:24.574892
67517	ROOF	2020-09-29	17.63	17.6492	17.36	17.54	6171	{}	\N	2020-10-12 16:01:24.58523
67518	ROOF	2020-09-30	17.67	17.88	17.49	17.53	3521	{}	\N	2020-10-12 16:01:24.59322
67519	ROOF	2020-10-01	17.6499	17.8821	17.56	17.8821	9892	{}	\N	2020-10-12 16:01:24.600446
67520	ROOF	2020-10-02	17.69	18.32	17.69	18.32	5506	{}	\N	2020-10-12 16:01:24.607023
67521	ROOF	2020-10-05	18.34	18.34	18.07	18.23	6150	{}	\N	2020-10-12 16:01:24.613411
67522	ROOF	2020-10-06	18.33	18.56	18.1692	18.1692	11184	{}	\N	2020-10-12 16:01:24.619145
67523	ROOF	2020-10-07	18.34	18.34	18.08	18.23	8545	{}	\N	2020-10-12 16:01:24.624451
67524	ROOF	2020-10-08	18.29	18.61	18.29	18.61	3933	{}	\N	2020-10-12 16:01:24.629755
67525	ROOF	2020-10-09	18.66	18.66	18.42	18.42	8133	{}	\N	2020-10-12 16:01:24.634829
67526	ROOF	2020-10-12	18.488	18.488	18.488	18.488	291	{}	\N	2020-10-12 16:01:24.639898
67527	RPD	2020-09-28	61.48	61.84	60.695	61.59	334940	{}	\N	2020-10-12 16:01:25.603435
67528	RPD	2020-09-29	61.72	62.185	61.11	61.3	333527	{}	\N	2020-10-12 16:01:25.615412
67529	RPD	2020-09-30	61.03	62.92	60.6	61.24	596234	{}	\N	2020-10-12 16:01:25.622868
67530	RPD	2020-10-01	61.91	62.3	61.0238	61.13	484661	{}	\N	2020-10-12 16:01:25.628331
67531	RPD	2020-10-02	59.9	61.26	59.2	59.73	288801	{}	\N	2020-10-12 16:01:25.633176
67532	RPD	2020-10-05	60.33	61.67	59.7501	61.33	373956	{}	\N	2020-10-12 16:01:25.637621
67533	RPD	2020-10-06	61.74	63.35	61.3805	61.74	274043	{}	\N	2020-10-12 16:01:25.641885
67534	RPD	2020-10-07	61.73	64.46	61.73	64.04	434255	{}	\N	2020-10-12 16:01:25.646741
67535	RPD	2020-10-08	64.86	66.25	64.17	65	721085	{}	\N	2020-10-12 16:01:25.651397
67536	RPD	2020-10-09	65.05	66.74	65.01	65.72	434937	{}	\N	2020-10-12 16:01:25.655461
67537	RPD	2020-10-12	66.42	66.69	65.431	66.2546	152170	{}	\N	2020-10-12 16:01:25.659823
67538	RS	2020-09-28	104.76	105.925	103.57	104.27	191352	{}	\N	2020-10-12 16:01:26.610977
67539	RS	2020-09-29	103.83	104.445	102.09	102.13	426244	{}	\N	2020-10-12 16:01:26.621574
67540	RS	2020-09-30	102.06	103.67	101.39	102.04	747963	{}	\N	2020-10-12 16:01:26.628254
67541	RS	2020-10-01	102.51	105.36	102.37	104.75	844020	{}	\N	2020-10-12 16:01:26.633975
67542	RS	2020-10-02	102.88	107.05	102.88	105.98	504836	{}	\N	2020-10-12 16:01:26.639228
67543	RS	2020-10-05	107.12	109.25	107.02	108.33	515049	{}	\N	2020-10-12 16:01:26.643966
67544	RS	2020-10-06	109.08	109.8	106.445	106.53	634300	{}	\N	2020-10-12 16:01:26.648532
67545	RS	2020-10-07	108.3	109.98	106.53	109.46	508559	{}	\N	2020-10-12 16:01:26.653708
67546	RS	2020-10-08	110	112.01	109.59	112	407245	{}	\N	2020-10-12 16:01:26.659278
67547	RS	2020-10-09	113.05	113.58	111.46	111.75	375429	{}	\N	2020-10-12 16:01:26.664505
67548	RS	2020-10-12	112.13	113.68	111.59	113.29	75316	{}	\N	2020-10-12 16:01:26.669515
67549	RUSHB	2020-09-28	42.5	44.3	42.5	44.3	13227	{}	\N	2020-10-12 16:01:28.351385
67550	RUSHB	2020-09-29	44.58	44.64	43.66	44.23	10219	{}	\N	2020-10-12 16:01:28.360056
67551	RUSHB	2020-09-30	44.7	44.7	44.3	44.3	6969	{}	\N	2020-10-12 16:01:28.366817
67552	RUSHB	2020-10-01	44.5	44.71	44.27	44.4	5834	{}	\N	2020-10-12 16:01:28.372627
67553	RUSHB	2020-10-02	43.78	45.57	43	45.35	14103	{}	\N	2020-10-12 16:01:28.377963
67554	RUSHB	2020-10-05	45.2856	47.7	45.2856	47.7	7622	{}	\N	2020-10-12 16:01:28.383183
67555	RUSHB	2020-10-06	47.67	49.27	47.67	48.5	7398	{}	\N	2020-10-12 16:01:28.388658
67556	RUSHB	2020-10-07	49.96	50	48.9	49.85	15620	{}	\N	2020-10-12 16:01:28.394921
67557	RUSHB	2020-10-08	50	50.28	49.07	50.02	13636	{}	\N	2020-10-12 16:01:28.40064
67558	RUSHB	2020-10-09	50.48	50.48	48.7256	49.74	11417	{}	\N	2020-10-12 16:01:28.405292
67559	RGI	2020-09-28	134.9	136.945	134.9	135.93	17421	{}	\N	2020-10-12 16:01:29.468654
67560	RGI	2020-09-29	135.97	135.97	134.52	134.95	9618	{}	\N	2020-10-12 16:01:29.479543
67561	RGI	2020-09-30	136.6243	136.6243	134.0556	134.4692	15709	{}	\N	2020-10-12 16:01:29.487741
67562	RGI	2020-10-01	135.89	136.075	133.39	134.36	119550	{}	\N	2020-10-12 16:01:29.494016
67563	RGI	2020-10-02	131.66	136.6412	131.66	136.16	20445	{}	\N	2020-10-12 16:01:29.499229
67564	RGI	2020-10-05	137.95	138.63	137.881	138.44	9969	{}	\N	2020-10-12 16:01:29.504416
67565	RGI	2020-10-06	139.34	140.17	137.96	137.97	16457	{}	\N	2020-10-12 16:01:29.509421
67566	RGI	2020-10-07	139.227	140.18	139.0201	140.0199	25222	{}	\N	2020-10-12 16:01:29.514741
67567	RGI	2020-10-08	140.99	141.48	140	141.48	9620	{}	\N	2020-10-12 16:01:29.520204
67568	RGI	2020-10-09	142.64	142.91	141.7161	141.7913	8887	{}	\N	2020-10-12 16:01:29.525234
67569	RGI	2020-10-12	142.46	142.8101	142.12	142.73	4752	{}	\N	2020-10-12 16:01:29.530365
67570	RH	2020-09-28	363.61	363.9799	354.74	361.06	392145	{}	\N	2020-10-12 16:01:30.850876
67571	RH	2020-09-29	372.71	385.01	369.19	375.32	1484779	{}	\N	2020-10-12 16:01:30.863418
67572	RH	2020-09-30	376.5	388.3	376.5	382.62	1263477	{}	\N	2020-10-12 16:01:30.871313
67573	RH	2020-10-01	384.95	387.99	373.01	379.28	987648	{}	\N	2020-10-12 16:01:30.878576
67574	RH	2020-10-02	369.26	388.36	369.26	384.74	917924	{}	\N	2020-10-12 16:01:30.884523
67575	RH	2020-10-05	383.92	387.95	372.22	379.36	1234869	{}	\N	2020-10-12 16:01:30.889501
67576	RH	2020-10-06	379.83	380.66	361.79	361.98	1030918	{}	\N	2020-10-12 16:01:30.894323
67577	RH	2020-10-07	368.81	379.46	368.43	377.72	824302	{}	\N	2020-10-12 16:01:30.899291
67578	RH	2020-10-08	378.9	379.55	370.28	376.08	763077	{}	\N	2020-10-12 16:01:30.90453
67579	RH	2020-10-09	379.7	384.9	369.27	371.46	859984	{}	\N	2020-10-12 16:01:30.909529
67580	RH	2020-10-12	373.8	375	362.5201	366.3299	346168	{}	\N	2020-10-12 16:01:30.914702
67581	RJA	2020-09-28	5.4	5.4243	5.39	5.4	27322	{}	\N	2020-10-12 16:01:32.766023
67582	RJA	2020-09-29	5.36	5.4	5.36	5.38	26168	{}	\N	2020-10-12 16:01:32.781377
67583	RJA	2020-09-30	5.3999	5.53	5.3999	5.53	35182	{}	\N	2020-10-12 16:01:32.790004
67584	RJA	2020-10-01	5.59	5.59	5.498	5.5206	18194	{}	\N	2020-10-12 16:01:32.796086
67585	RJA	2020-10-02	5.441	5.5099	5.441	5.4866	11849	{}	\N	2020-10-12 16:01:32.801711
67586	RJA	2020-10-05	5.523	5.56	5.49	5.55	30975	{}	\N	2020-10-12 16:01:32.806235
67587	RJA	2020-10-06	5.545	5.61	5.52	5.54	78903	{}	\N	2020-10-12 16:01:32.810499
67588	RJA	2020-10-07	5.55	5.64	5.55	5.6	88213	{}	\N	2020-10-12 16:01:32.815629
67589	RJA	2020-10-08	5.64	5.64	5.58	5.62	47953	{}	\N	2020-10-12 16:01:32.82041
67590	RJA	2020-10-09	5.67	5.67	5.6	5.64	41160	{}	\N	2020-10-12 16:01:32.825
67591	RJA	2020-10-12	5.63	5.64	5.6	5.605	7762	{}	\N	2020-10-12 16:01:32.82959
67592	RMAX	2020-09-28	32.37	33.37	32.37	33.13	80337	{}	\N	2020-10-12 16:01:33.751955
67593	RMAX	2020-09-29	33.24	33.43	32.4	32.65	83828	{}	\N	2020-10-12 16:01:33.762232
67594	RMAX	2020-09-30	32.73	33.42	32.45	32.73	191079	{}	\N	2020-10-12 16:01:33.769656
67595	RMAX	2020-10-01	32.91	33.97	32.56	33.84	131730	{}	\N	2020-10-12 16:01:33.776953
67596	RMAX	2020-10-02	33.14	34.77	33.14	34.47	116630	{}	\N	2020-10-12 16:01:33.782927
67597	RMAX	2020-10-05	34.74	34.96	34.36	34.79	97887	{}	\N	2020-10-12 16:01:33.788683
67598	RMAX	2020-10-06	35.06	36.38	34.75	35.16	156193	{}	\N	2020-10-12 16:01:33.793523
67599	RMAX	2020-10-07	35.84	36.23	35.01	35.13	194312	{}	\N	2020-10-12 16:01:33.798875
67600	RMAX	2020-10-08	35.61	36.7	35.19	36.18	182143	{}	\N	2020-10-12 16:01:33.803876
67601	RMAX	2020-10-09	36.68	36.82	35.7	36.05	154730	{}	\N	2020-10-12 16:01:33.808659
67602	RMAX	2020-10-12	36.07	36.63	35.72	36.25	32933	{}	\N	2020-10-12 16:01:33.813273
67603	RMT	2020-09-28	7.62	7.6913	7.62	7.65	183623	{}	\N	2020-10-12 16:01:34.925643
67604	RMT	2020-09-29	7.65	7.73	7.62	7.66	162230	{}	\N	2020-10-12 16:01:34.937713
67605	RMT	2020-09-30	7.66	7.8	7.66	7.67	245206	{}	\N	2020-10-12 16:01:34.945726
67606	RMT	2020-10-01	7.72	7.77	7.63	7.76	245949	{}	\N	2020-10-12 16:01:34.952165
67607	RMT	2020-10-02	7.66	7.84	7.635	7.82	327313	{}	\N	2020-10-12 16:01:34.958273
67608	RMT	2020-10-05	7.92	8.06	7.83	8.05	339280	{}	\N	2020-10-12 16:01:34.963636
67609	RMT	2020-10-06	8.07	8.13	7.99	8.01	506962	{}	\N	2020-10-12 16:01:34.969225
67610	RMT	2020-10-07	8.11	8.2004	8.04	8.2	469107	{}	\N	2020-10-12 16:01:34.9745
67611	RMT	2020-10-08	8.23	8.29	8.23	8.27	372531	{}	\N	2020-10-12 16:01:34.978896
67612	RMT	2020-10-09	8.33	8.38	8.29	8.33	400924	{}	\N	2020-10-12 16:01:34.983911
67613	RMT	2020-10-12	8.37	8.4	8.34	8.39	68852	{}	\N	2020-10-12 16:01:34.988768
67614	RNR	2020-09-28	167.58	170.89	166.83	170.04	297203	{}	\N	2020-10-12 16:01:36.471821
67615	RNR	2020-09-29	170.47	170.965	167.23	168.5	327403	{}	\N	2020-10-12 16:01:36.48331
67616	RNR	2020-09-30	169.32	170.61	167.46	169.74	695045	{}	\N	2020-10-12 16:01:36.490661
67617	RNR	2020-10-01	169.93	173.6	168.83	170.13	474027	{}	\N	2020-10-12 16:01:36.496959
67618	RNR	2020-10-02	168.48	171.46	168.14	170.36	252472	{}	\N	2020-10-12 16:01:36.502548
67619	RNR	2020-10-05	171.79	174.74	170.68	174.14	344889	{}	\N	2020-10-12 16:01:36.507687
67620	RNR	2020-10-06	175.51	175.51	169.4	170.18	418721	{}	\N	2020-10-12 16:01:36.51275
67621	RNR	2020-10-07	171.39	171.39	166.42	169.74	588858	{}	\N	2020-10-12 16:01:36.518327
67622	RNR	2020-10-08	170.15	173.72	170.1	173.03	335305	{}	\N	2020-10-12 16:01:36.523353
67623	RNR	2020-10-09	174.77	175.3	172.445	173.19	385723	{}	\N	2020-10-12 16:01:36.528532
67624	RNR	2020-10-12	174.21	177.57	173.05	177.46	107381	{}	\N	2020-10-12 16:01:36.533393
67625	ROIC	2020-09-28	10.21	10.71	10.21	10.65	1401073	{}	\N	2020-10-12 16:01:38.066007
67626	ROIC	2020-09-29	10.64	10.64	10.025	10.22	1592565	{}	\N	2020-10-12 16:01:38.079021
67627	ROIC	2020-09-30	10.28	10.58	10.24	10.415	1687874	{}	\N	2020-10-12 16:01:38.087025
67628	ROIC	2020-10-01	10.46	10.77	10.26	10.76	1323283	{}	\N	2020-10-12 16:01:38.092737
67629	ROIC	2020-10-02	10.49	11.26	10	11.23	1440849	{}	\N	2020-10-12 16:01:38.097539
67630	ROIC	2020-10-05	11.43	11.45	10.8	11.22	662694	{}	\N	2020-10-12 16:01:38.101903
67631	ROIC	2020-10-06	11.38	11.53	11.16	11.22	1350366	{}	\N	2020-10-12 16:01:38.106315
67632	ROIC	2020-10-07	11.32	11.46	10.925	10.955	1130263	{}	\N	2020-10-12 16:01:38.110726
67633	ROIC	2020-10-08	11.08	11.16	10.88	10.95	1723550	{}	\N	2020-10-12 16:01:38.11472
67634	ROIC	2020-10-09	11.03	11.04	10.81	10.925	796349	{}	\N	2020-10-12 16:01:38.118729
67635	ROIC	2020-10-12	10.86	10.95	10.68	10.89	177083	{}	\N	2020-10-12 16:01:38.12332
67636	RUSHA	2020-09-28	49.05	50.66	49.05	50.34	107415	{}	\N	2020-10-12 16:01:41.762537
67637	RUSHA	2020-09-29	50.34	50.93	50.14	50.28	90476	{}	\N	2020-10-12 16:01:41.773943
67638	RUSHA	2020-09-30	50.29	51.41	50.22	50.54	132876	{}	\N	2020-10-12 16:01:41.781887
67639	RUSHA	2020-10-01	50.78	51.06	50.05	51.02	81096	{}	\N	2020-10-12 16:01:41.787586
67640	RUSHA	2020-10-02	50.24	52.43	50.24	52.24	105600	{}	\N	2020-10-12 16:01:41.79377
67641	RUSHA	2020-10-05	52.72	54.72	52.5	54.69	134708	{}	\N	2020-10-12 16:01:41.799048
67642	RUSHA	2020-10-06	54.91	56.54	53.05	55.4	211223	{}	\N	2020-10-12 16:01:41.80391
67643	RUSHA	2020-10-07	56.2	57.15	55.5	57	158317	{}	\N	2020-10-12 16:01:41.808389
67644	RUSHA	2020-10-08	57.75	57.75	55.98	56.73	125287	{}	\N	2020-10-12 16:01:41.813202
67645	RUSHA	2020-10-09	57.24	57.265	55.57	55.8	90184	{}	\N	2020-10-12 16:01:41.817859
67646	RUSHA	2020-10-12	55.76	56.43	55.635	56.2301	14600	{}	\N	2020-10-12 16:01:41.822379
67647	RVSB	2020-09-28	4.07	4.17	3.96	3.99	152431	{}	\N	2020-10-12 16:01:42.860997
67648	RVSB	2020-09-29	4.09	4.11	3.9172	4.1	41748	{}	\N	2020-10-12 16:01:42.874772
67649	RVSB	2020-09-30	4.09	4.18	4.09	4.15	41435	{}	\N	2020-10-12 16:01:42.881343
67650	RVSB	2020-10-01	4.14	4.22	4.14	4.21	31690	{}	\N	2020-10-12 16:01:42.886836
67651	RVSB	2020-10-02	4.09	4.5	4.09	4.46	54609	{}	\N	2020-10-12 16:01:42.892515
67652	RVSB	2020-10-05	4.46	4.83	4.44	4.77	43830	{}	\N	2020-10-12 16:01:42.897394
67653	RVSB	2020-10-06	4.74	5	4.7336	4.86	66783	{}	\N	2020-10-12 16:01:42.901643
67654	RVSB	2020-10-07	4.82	4.95	4.61	4.92	144612	{}	\N	2020-10-12 16:01:42.905931
67655	RVSB	2020-10-08	4.94	4.94	4.77	4.91	46176	{}	\N	2020-10-12 16:01:42.91044
67656	RVSB	2020-10-09	5.23	5.23	4.8588	5.13	67459	{}	\N	2020-10-12 16:01:42.914866
67657	RVSB	2020-10-12	5.1	5.1	4.895	5.1	1989	{}	\N	2020-10-12 16:01:42.919439
67658	RHE	2020-09-28	1.19	1.38	1.1861	1.26	53852	{}	\N	2020-10-12 16:01:43.880026
67659	RHE	2020-09-29	1.26	1.4	1.2261	1.36	67947	{}	\N	2020-10-12 16:01:43.892011
67660	RHE	2020-09-30	1.36	1.36	1.3	1.3	15602	{}	\N	2020-10-12 16:01:43.899889
67661	RHE	2020-10-01	1.34	1.38	1.27	1.32	13955	{}	\N	2020-10-12 16:01:43.906242
67662	RHE	2020-10-02	1.32	1.4133	1.32	1.36	46041	{}	\N	2020-10-12 16:01:43.912407
67663	RHE	2020-10-05	1.35	1.38	1.3	1.33	33702	{}	\N	2020-10-12 16:01:43.91787
67664	RHE	2020-10-06	1.37	1.5	1.3	1.3	128889	{}	\N	2020-10-12 16:01:43.922846
67665	RHE	2020-10-07	1.3	1.36	1.3	1.34	20460	{}	\N	2020-10-12 16:01:43.928147
67666	RHE	2020-10-08	1.46	1.54	1.36	1.38	50890	{}	\N	2020-10-12 16:01:43.933469
67667	RHE	2020-10-09	1.42	1.45	1.4101	1.415	12649	{}	\N	2020-10-12 16:01:43.938473
67668	RHE	2020-10-12	1.45	1.45	1.3999	1.3999	2948	{}	\N	2020-10-12 16:01:43.943508
67669	RINF	2020-09-28	25.56	25.7524	25.56	25.7524	2476	{}	\N	2020-10-12 16:01:45.088954
67670	RINF	2020-09-30	25.8453	25.8453	25.8453	25.8453	210	{}	\N	2020-10-12 16:01:45.102997
67671	RINF	2020-10-01	25.865	25.8996	25.865	25.8996	168	{}	\N	2020-10-12 16:01:45.111261
67672	RINF	2020-10-05	26.19	26.19	26.19	26.19	77	{}	\N	2020-10-12 16:01:45.118481
67673	RINF	2020-10-06	26.105	26.105	26.045	26.045	500	{}	\N	2020-10-12 16:01:45.123046
67674	RINF	2020-10-07	26.17	26.17	26.17	26.17	6775	{}	\N	2020-10-12 16:01:45.128142
67675	RINF	2020-10-08	26.3	26.3441	26.3	26.3441	508	{}	\N	2020-10-12 16:01:45.133215
67676	RINF	2020-10-09	26.3916	26.3916	26.3916	26.3916	28	{}	\N	2020-10-12 16:01:45.137514
67677	RINF	2020-10-12	26.3	26.3	26.27	26.28	11824	{}	\N	2020-10-12 16:01:45.14152
67678	RLGT	2020-09-28	5.1	5.29	5.05	5.21	194394	{}	\N	2020-10-12 16:01:46.135849
67679	RLGT	2020-09-29	5.4	5.79	5.27	5.27	375801	{}	\N	2020-10-12 16:01:46.144817
67680	RLGT	2020-09-30	5.28	5.41	5.11	5.14	258619	{}	\N	2020-10-12 16:01:46.151269
67681	RLGT	2020-10-01	5.16	5.25	5.11	5.16	226354	{}	\N	2020-10-12 16:01:46.156826
67682	RLGT	2020-10-02	5.1	5.19	4.99	5.05	192433	{}	\N	2020-10-12 16:01:46.162177
67683	RLGT	2020-10-05	5.15	5.2407	5.1229	5.19	176394	{}	\N	2020-10-12 16:01:46.167166
67684	RLGT	2020-10-06	5.25	5.29	5.16	5.17	154323	{}	\N	2020-10-12 16:01:46.171802
67685	RLGT	2020-10-07	5.26	5.3	5.2085	5.22	107059	{}	\N	2020-10-12 16:01:46.175898
67686	RLGT	2020-10-08	5.3	5.35	5.25	5.27	154504	{}	\N	2020-10-12 16:01:46.180768
67687	RLGT	2020-10-09	5.3	5.3	5.18	5.22	168052	{}	\N	2020-10-12 16:01:46.18571
67688	RLGT	2020-10-12	5.26	5.26	5.19	5.24	34536	{}	\N	2020-10-12 16:01:46.190215
67689	RMR	2020-09-28	27.75	29.0759	27.75	28.86	42052	{}	\N	2020-10-12 16:01:47.184936
67690	RMR	2020-09-29	28.82	28.85	27.59	28.03	34029	{}	\N	2020-10-12 16:01:47.197426
67691	RMR	2020-09-30	28.19	28.6	27.29	27.47	49877	{}	\N	2020-10-12 16:01:47.205849
67692	RMR	2020-10-01	28.455	28.46	27.4	28.33	53741	{}	\N	2020-10-12 16:01:47.21183
67693	RMR	2020-10-02	27.8	28.91	27.8	28.71	30633	{}	\N	2020-10-12 16:01:47.218084
67694	RMR	2020-10-05	28.95	28.95	28.21	28.42	40882	{}	\N	2020-10-12 16:01:47.223271
67695	RMR	2020-10-06	28.8	29.44	28	28.55	45223	{}	\N	2020-10-12 16:01:47.228928
67696	RMR	2020-10-07	28.62	29.08	28.51	28.84	36605	{}	\N	2020-10-12 16:01:47.234508
67697	RMR	2020-10-08	29.13	30.04	29.05	30.04	71642	{}	\N	2020-10-12 16:01:47.239702
67698	RMR	2020-10-09	30.55	30.7674	29.94	30.21	40879	{}	\N	2020-10-12 16:01:47.245153
67699	RMR	2020-10-12	30.1	30.1	29.465	29.67	8746	{}	\N	2020-10-12 16:01:47.250971
67700	RNG	2020-09-28	272.77	273.405	264.25	267.8	345785	{}	\N	2020-10-12 16:01:48.470459
67701	RNG	2020-09-29	270.69	272.625	266.62	270.76	1406114	{}	\N	2020-10-12 16:01:48.481455
67702	RNG	2020-09-30	268.07	275.89	266.19	274.61	1227826	{}	\N	2020-10-12 16:01:48.488545
67703	RNG	2020-10-01	276	280.86	270.54	280.48	1273273	{}	\N	2020-10-12 16:01:48.493923
67704	RNG	2020-10-02	276.32	288.7	272.5	275.11	1588262	{}	\N	2020-10-12 16:01:48.49909
67705	RNG	2020-10-05	274.64	276.94	269	274.75	1711880	{}	\N	2020-10-12 16:01:48.504252
67706	RNG	2020-10-06	277	286.76	275.96	281.02	1300839	{}	\N	2020-10-12 16:01:48.509229
67707	RNG	2020-10-07	283.49	292.92	282.95	289.4	1552538	{}	\N	2020-10-12 16:01:48.51468
67708	RNG	2020-10-08	289.4	291	278.83	281.33	1209445	{}	\N	2020-10-12 16:01:48.520641
67709	RNG	2020-10-09	284.25	291.6647	283	287.41	1257138	{}	\N	2020-10-12 16:01:48.525764
67710	RNG	2020-10-12	290	292.05	283.55	285.17	391542	{}	\N	2020-10-12 16:01:48.530943
67711	ROCK	2020-09-28	63.17	65.05	62.88	64.51	142538	{}	\N	2020-10-12 16:01:49.777645
67712	ROCK	2020-09-29	64.35	65.88	64.18	65.64	192031	{}	\N	2020-10-12 16:01:49.789699
67713	ROCK	2020-09-30	65.52	66.77	65.08	65.14	307730	{}	\N	2020-10-12 16:01:49.796464
67714	ROCK	2020-10-01	65.19	65.96	64.29	65.85	145733	{}	\N	2020-10-12 16:01:49.802919
67715	ROCK	2020-10-02	64.58	66.49	63.965	66.13	167194	{}	\N	2020-10-12 16:01:49.808743
67716	ROCK	2020-10-05	66.65	67.975	65.955	67.85	171915	{}	\N	2020-10-12 16:01:49.813834
67717	ROCK	2020-10-06	68.58	70.76	67.9	70.41	282183	{}	\N	2020-10-12 16:01:49.818791
67718	ROCK	2020-10-07	71.49	73.15	71.08	72.48	373473	{}	\N	2020-10-12 16:01:49.823827
67719	ROCK	2020-10-08	72.65	73.215	70.84	73.1	175749	{}	\N	2020-10-12 16:01:49.828569
67720	ROCK	2020-10-09	73.31	74	71.225	72.7	188626	{}	\N	2020-10-12 16:01:49.833428
67721	ROCK	2020-10-12	73.25	73.86	71.86	73	66865	{}	\N	2020-10-12 16:01:49.838871
67722	ROM	2020-09-28	56.39	56.5	55.22	56.16	120524	{}	\N	2020-10-12 16:01:50.703704
67723	ROM	2020-09-29	56.16	56.74	55.8	56.17	105515	{}	\N	2020-10-12 16:01:50.716406
67724	ROM	2020-09-30	56.02	58.1	56	56.91	396736	{}	\N	2020-10-12 16:01:50.723647
67725	ROM	2020-10-01	58.29	58.7699	57.8	58.44	112654	{}	\N	2020-10-12 16:01:50.729493
67726	ROM	2020-10-02	55.24	57.7054	55.24	55.59	180191	{}	\N	2020-10-12 16:01:50.734969
67727	ROM	2020-10-05	56.51	58.16	56.4704	58.16	86044	{}	\N	2020-10-12 16:01:50.740813
67728	ROM	2020-10-06	57.93	58.5	55.95	56.41	108922	{}	\N	2020-10-12 16:01:50.746229
67729	ROM	2020-10-07	57.54	58.479	57.1206	58.29	98392	{}	\N	2020-10-12 16:01:50.751834
67730	ROM	2020-10-08	59.23	59.29	58.5737	58.97	95730	{}	\N	2020-10-12 16:01:50.75698
67731	ROM	2020-10-09	59.49	60.85	59.49	60.82	417410	{}	\N	2020-10-12 16:01:50.762352
67732	ROM	2020-10-12	62.76	63.8999	62.26	63.71	232053	{}	\N	2020-10-12 16:01:50.767716
67733	ROUS	2020-09-28	31.625	31.736	31.59	31.6	10858	{}	\N	2020-10-12 16:01:51.914023
67734	ROUS	2020-09-29	31.8	31.82	31.5027	31.625	9591	{}	\N	2020-10-12 16:01:51.928064
67735	ROUS	2020-09-30	31.68	31.99	31.6172	31.76	14231	{}	\N	2020-10-12 16:01:51.935856
67736	ROUS	2020-10-01	31.8531	31.9332	31.73	31.79	33105	{}	\N	2020-10-12 16:01:51.942935
67737	ROUS	2020-10-02	31.5	31.8799	31.5	31.7371	18956	{}	\N	2020-10-12 16:01:51.948754
67738	ROUS	2020-10-05	31.75	32.2923	31.75	32.2923	5701	{}	\N	2020-10-12 16:01:51.95485
67739	ROUS	2020-10-06	32.39	32.547	31.92	31.92	7746	{}	\N	2020-10-12 16:01:51.960161
67740	ROUS	2020-10-07	32.15	32.5506	32.15	32.48	15840	{}	\N	2020-10-12 16:01:51.965552
67741	ROUS	2020-10-08	32.47	32.7993	32.47	32.74	7058	{}	\N	2020-10-12 16:01:51.971001
67742	ROUS	2020-10-09	33.02	33.0555	32.93	32.995	6200	{}	\N	2020-10-12 16:01:51.976754
67743	ROUS	2020-10-12	33.12	33.22	33.1163	33.22	897	{}	\N	2020-10-12 16:01:51.981851
67744	RPT	2020-09-28	5.5	5.845	5.48	5.73	534236	{}	\N	2020-10-12 16:01:52.95889
67745	RPT	2020-09-29	5.68	5.68	5.315	5.47	864040	{}	\N	2020-10-12 16:01:52.970066
67746	RPT	2020-09-30	5.5	5.655	5.375	5.44	1130279	{}	\N	2020-10-12 16:01:52.97726
67747	RPT	2020-10-01	5.52	5.69	5.455	5.67	600108	{}	\N	2020-10-12 16:01:52.983091
67748	RPT	2020-10-02	5.47	5.895	5.47	5.89	720630	{}	\N	2020-10-12 16:01:52.988782
67749	RPT	2020-10-05	5.95	5.97	5.65	5.86	903767	{}	\N	2020-10-12 16:01:52.993545
67750	RPT	2020-10-06	5.91	5.98	5.72	5.75	1018408	{}	\N	2020-10-12 16:01:52.997658
67751	RPT	2020-10-07	5.79	5.89	5.51	5.58	2146246	{}	\N	2020-10-12 16:01:53.001599
67752	RPT	2020-10-08	5.64	5.82	5.64	5.8	675181	{}	\N	2020-10-12 16:01:53.00737
67753	RPT	2020-10-09	5.81	5.8784	5.505	5.57	836089	{}	\N	2020-10-12 16:01:53.013174
67754	RPT	2020-10-12	5.57	5.62	5.45	5.56	268057	{}	\N	2020-10-12 16:01:53.01743
67755	RST	2020-09-28	29.93	30.05	29.9	29.9	495003	{}	\N	2020-10-12 16:01:54.887416
67756	RST	2020-09-29	29.95	29.95	29.89	29.9	785705	{}	\N	2020-10-12 16:01:54.898849
67757	RST	2020-09-30	29.91	29.995	29.89	29.98	1890733	{}	\N	2020-10-12 16:01:54.905974
67758	RST	2020-10-01	30.03	30.03	29.9	29.9	826379	{}	\N	2020-10-12 16:01:54.912614
67759	RST	2020-10-02	29.89	29.92	29.88	29.89	929479	{}	\N	2020-10-12 16:01:54.918585
67760	RST	2020-10-05	29.91	29.95	29.88	29.88	1104414	{}	\N	2020-10-12 16:01:54.924067
67761	RST	2020-10-06	29.9	29.94	29.89	29.9	713507	{}	\N	2020-10-12 16:01:54.929066
67762	RST	2020-10-07	29.92	29.94	29.9	29.93	633296	{}	\N	2020-10-12 16:01:54.934191
67763	RST	2020-10-08	29.94	29.97	29.93	29.95	462195	{}	\N	2020-10-12 16:01:54.939037
67764	RST	2020-10-09	29.94	29.96	29.94	29.95	612494	{}	\N	2020-10-12 16:01:54.943291
67765	RST	2020-10-12	29.96	30.01	29.93	29.945	206718	{}	\N	2020-10-12 16:01:54.947305
67766	RTRX	2020-09-28	18.37	18.37	17.75	18.08	244510	{}	\N	2020-10-12 16:01:55.918815
67767	RTRX	2020-09-29	18.05	19.02	18.03	18.62	303123	{}	\N	2020-10-12 16:01:55.930092
67768	RTRX	2020-09-30	18.67	18.93	18.37	18.46	283295	{}	\N	2020-10-12 16:01:55.938237
67769	RTRX	2020-10-01	18.5	19.24	18.325	19.19	255822	{}	\N	2020-10-12 16:01:55.945088
67770	RTRX	2020-10-02	18.9	19.3039	18.87	19.165	263102	{}	\N	2020-10-12 16:01:55.950923
67771	RTRX	2020-10-05	19.35	20.43	19.145	20.16	395126	{}	\N	2020-10-12 16:01:55.955634
67772	RTRX	2020-10-06	20.35	20.82	19.89	20.53	647933	{}	\N	2020-10-12 16:01:55.960376
67773	RTRX	2020-10-07	20.69	20.7	20.12	20.66	304977	{}	\N	2020-10-12 16:01:55.96518
67774	RTRX	2020-10-08	20.95	20.95	20.26	20.54	218469	{}	\N	2020-10-12 16:01:55.969436
67775	RTRX	2020-10-09	20.78	20.85	19.83	20.73	155380	{}	\N	2020-10-12 16:01:55.974431
67776	RTRX	2020-10-12	20.79	20.79	20.305	20.55	46109	{}	\N	2020-10-12 16:01:55.979689
67777	RGLS	2020-09-28	0.49	0.5398	0.49	0.5366	305352	{}	\N	2020-10-12 16:01:57.441805
67778	RGLS	2020-09-29	0.5473	0.58	0.5031	0.5483	360402	{}	\N	2020-10-12 16:01:57.453089
67779	RGLS	2020-09-30	0.5462	0.5828	0.514	0.514	292875	{}	\N	2020-10-12 16:01:57.459594
67780	RGLS	2020-10-01	0.55	0.5999	0.512	0.5448	466099	{}	\N	2020-10-12 16:01:57.469273
67781	RGLS	2020-10-02	0.5365	0.5739	0.520301	0.5739	191214	{}	\N	2020-10-12 16:01:57.473761
67782	RGLS	2020-10-05	0.58	0.58	0.5346	0.546	213761	{}	\N	2020-10-12 16:01:57.478532
67783	RGLS	2020-10-06	0.544	0.57	0.5357	0.5358	200873	{}	\N	2020-10-12 16:01:57.482655
67784	RGLS	2020-10-07	0.5364	0.5589	0.53	0.5452	91264	{}	\N	2020-10-12 16:01:57.487025
67785	RGLS	2020-10-08	0.528	0.5593	0.528	0.5378	141357	{}	\N	2020-10-12 16:01:57.49198
67786	RGLS	2020-10-09	0.552	0.56	0.536	0.5417	197039	{}	\N	2020-10-12 16:01:57.496744
67787	RGLS	2020-10-12	0.54	0.55	0.5301	0.5445	55812	{}	\N	2020-10-12 16:01:57.501022
67788	RILYL	2020-09-28	24.74	24.74	24.575	24.6	14182	{}	\N	2020-10-12 16:01:58.835015
67789	RILYL	2020-09-29	24.6	24.6	24.45	24.58	15333	{}	\N	2020-10-12 16:01:58.843365
67790	RILYL	2020-09-30	24.7	24.72	24.65	24.7	45479	{}	\N	2020-10-12 16:01:58.850943
67791	RILYL	2020-10-01	24.75	25.06	24.7291	24.95	58194	{}	\N	2020-10-12 16:01:58.856559
67792	RILYL	2020-10-02	24.9	24.95	24.8225	24.87	8175	{}	\N	2020-10-12 16:01:58.862569
67793	RILYL	2020-10-05	24.95	25.49	24.8701	25.1978	25976	{}	\N	2020-10-12 16:01:58.866941
67794	RILYL	2020-10-06	25.39	25.39	25	25.18	16679	{}	\N	2020-10-12 16:01:58.871676
67795	RILYL	2020-10-07	25.2	25.2	25.11	25.1196	10888	{}	\N	2020-10-12 16:01:58.87649
67796	RILYL	2020-10-08	25.25	25.3	25.2	25.29	10775	{}	\N	2020-10-12 16:01:58.881016
67797	RILYL	2020-10-09	25.31	25.31	25.2	25.29	9341	{}	\N	2020-10-12 16:01:58.885488
67798	RILYL	2020-10-12	25.25	25.26	25.17	25.17	2985	{}	\N	2020-10-12 16:01:58.890546
67799	RJN	2020-09-28	1.44	1.48	1.44	1.4721	2647	{}	\N	2020-10-12 16:02:01.481196
67800	RJN	2020-09-29	1.42	1.42	1.391	1.415	6146	{}	\N	2020-10-12 16:02:01.489906
67801	RJN	2020-09-30	1.41	1.43	1.41	1.43	9588	{}	\N	2020-10-12 16:02:01.496115
67802	RJN	2020-10-01	1.39	1.4	1.37	1.4	1220	{}	\N	2020-10-12 16:02:01.502433
67803	RJN	2020-10-02	1.35	1.35	1.3425	1.3425	459	{}	\N	2020-10-12 16:02:01.508237
67804	RJN	2020-10-05	1.36	1.425	1.36	1.425	8817	{}	\N	2020-10-12 16:02:01.513698
67805	RJN	2020-10-06	1.55	1.55	1.44	1.45	1552	{}	\N	2020-10-12 16:02:01.519587
67806	RJN	2020-10-07	1.435	1.445	1.435	1.445	187	{}	\N	2020-10-12 16:02:01.524884
67807	RJN	2020-10-08	1.4747	1.479	1.4747	1.479	500	{}	\N	2020-10-12 16:02:01.530364
67808	RJN	2020-10-09	1.4601	1.4899	1.45	1.456	8323	{}	\N	2020-10-12 16:02:01.536098
67809	RLJ	2020-09-28	8.7	9.05	8.605	8.96	1315114	{}	\N	2020-10-12 16:02:02.484113
67810	RLJ	2020-09-29	8.49	8.71	8.39	8.64	2484205	{}	\N	2020-10-12 16:02:02.496899
67811	RLJ	2020-09-30	8.71	9.12	8.54	8.66	1837537	{}	\N	2020-10-12 16:02:02.505576
67812	RLJ	2020-10-01	8.64	9.08	8.64	9.08	1861162	{}	\N	2020-10-12 16:02:02.512356
67813	RLJ	2020-10-02	8.5	9.065	8.32	9.03	1864276	{}	\N	2020-10-12 16:02:02.519013
67814	RLJ	2020-10-05	9.08	9.2	8.75	8.87	1521502	{}	\N	2020-10-12 16:02:02.524552
67815	RLJ	2020-10-06	9.04	9.225	8.755	8.88	1762205	{}	\N	2020-10-12 16:02:02.529999
67816	RLJ	2020-10-07	8.98	9.055	8.84	8.97	1947313	{}	\N	2020-10-12 16:02:02.535725
67817	RLJ	2020-10-08	9.14	9.16	8.885	9.15	2162297	{}	\N	2020-10-12 16:02:02.54107
67818	RLJ	2020-10-09	9.23	9.29	8.935	8.98	1346843	{}	\N	2020-10-12 16:02:02.546327
67819	RLJ	2020-10-12	8.98	8.98	8.76	8.77	590026	{}	\N	2020-10-12 16:02:02.551773
67820	RNDB	2020-09-28	12	12.3	12	12.18	1843	{}	\N	2020-10-12 16:02:05.411764
67821	RNDB	2020-09-29	12.3	12.855	12.3	12.59	3139	{}	\N	2020-10-12 16:02:05.421834
67822	RNDB	2020-09-30	12.79	13	12.05	12.43	17681	{}	\N	2020-10-12 16:02:05.428628
67823	RNDB	2020-10-01	12.39	12.39	12.39	12.39	484	{}	\N	2020-10-12 16:02:05.435813
67824	RNDB	2020-10-05	12.3	13	11.75	12.62	25450	{}	\N	2020-10-12 16:02:05.440937
67825	RNDB	2020-10-06	12.89	13.11	12.86	12.9	19853	{}	\N	2020-10-12 16:02:05.445776
67826	RNDB	2020-10-07	12.8114	12.93	12.8114	12.93	1169	{}	\N	2020-10-12 16:02:05.45136
67827	RNDB	2020-10-08	13.03	13.4	12.875	13.4	9220	{}	\N	2020-10-12 16:02:05.45696
67828	RNDB	2020-10-09	13.14	13.4	13.14	13.26	3083	{}	\N	2020-10-12 16:02:05.462105
67829	RNGR	2020-09-28	2.5	2.58	2.43	2.43	21563	{}	\N	2020-10-12 16:02:06.92073
67830	RNGR	2020-09-29	2.52	2.68	2.485	2.68	76518	{}	\N	2020-10-12 16:02:06.930466
67831	RNGR	2020-09-30	2.64	2.645	2.5	2.61	28152	{}	\N	2020-10-12 16:02:06.937342
67832	RNGR	2020-10-01	2.69	2.75	2.46	2.68	32322	{}	\N	2020-10-12 16:02:06.943491
67833	RNGR	2020-10-02	2.64	2.7	2.54	2.64	16472	{}	\N	2020-10-12 16:02:06.94843
67834	RNGR	2020-10-05	2.6	2.71	2.6	2.63	6632	{}	\N	2020-10-12 16:02:06.954045
67835	RNGR	2020-10-06	2.6	3	2.6	2.94	41462	{}	\N	2020-10-12 16:02:06.959471
67836	RNGR	2020-10-07	2.93	2.93	2.78	2.87	27456	{}	\N	2020-10-12 16:02:06.964547
67837	RNGR	2020-10-08	2.79	2.92	2.74	2.92	17888	{}	\N	2020-10-12 16:02:06.969575
67838	RNGR	2020-10-09	2.84	2.84	2.68	2.69	12898	{}	\N	2020-10-12 16:02:06.974787
67839	RNGR	2020-10-12	2.6	2.69	2.6	2.67	1326	{}	\N	2020-10-12 16:02:06.979524
67840	ROAM	2020-09-28	19.511	19.511	19.46	19.46	9839	{}	\N	2020-10-12 16:02:07.972722
67841	ROAM	2020-09-29	19.5089	19.52	19.48	19.52	3970	{}	\N	2020-10-12 16:02:07.982439
67842	ROAM	2020-09-30	19.66	19.82	19.62	19.62	101118	{}	\N	2020-10-12 16:02:07.98993
67843	ROAM	2020-10-01	19.78	19.87	19.741	19.84	5944	{}	\N	2020-10-12 16:02:07.995888
67844	ROAM	2020-10-02	19.39	19.81	19.39	19.74	6339	{}	\N	2020-10-12 16:02:08.000934
67845	ROAM	2020-10-05	19.89	19.91	19.82	19.91	6122	{}	\N	2020-10-12 16:02:08.005808
67846	ROAM	2020-10-06	19.9468	19.979	19.78	19.78	3402	{}	\N	2020-10-12 16:02:08.01062
67847	ROAM	2020-10-07	19.92	19.9683	19.92	19.95	3041	{}	\N	2020-10-12 16:02:08.015485
67848	ROAM	2020-10-08	20.01	20.12	20.01	20.12	16410	{}	\N	2020-10-12 16:02:08.020631
67849	ROAM	2020-10-09	20.28	20.32	20.27	20.305	6705	{}	\N	2020-10-12 16:02:08.02594
67850	ROAM	2020-10-12	20.285	20.285	20.285	20.285	175	{}	\N	2020-10-12 16:02:08.031035
67851	ROKU	2020-09-28	185	189.01	181.66	186.29	5601504	{}	\N	2020-10-12 16:02:08.98213
67852	ROKU	2020-09-29	186.2	190.34	183.22	187.87	6221310	{}	\N	2020-10-12 16:02:08.993125
67853	ROKU	2020-09-30	187.89	193.55	185.856	188.8	6780770	{}	\N	2020-10-12 16:02:08.999462
67854	ROKU	2020-10-01	192.83	200.47	192.7	200.34	9708313	{}	\N	2020-10-12 16:02:09.006064
67855	ROKU	2020-10-02	195.37	203.56	194.8	200	7326248	{}	\N	2020-10-12 16:02:09.012171
67856	ROKU	2020-10-05	201.39	204.94	197.9501	204.16	5164358	{}	\N	2020-10-12 16:02:09.01748
67857	ROKU	2020-10-06	203.69	208.96	202.66	204.83	5428030	{}	\N	2020-10-12 16:02:09.023462
67858	ROKU	2020-10-07	207.95	214.26	207.5	212.55	5149297	{}	\N	2020-10-12 16:02:09.028897
67859	ROKU	2020-10-08	221.96	228.33	219.33	223.84	12835815	{}	\N	2020-10-12 16:02:09.034227
67860	ROKU	2020-10-09	226.05	226.83	221.63	223.95	7276061	{}	\N	2020-10-12 16:02:09.039394
67861	ROKU	2020-10-12	226.2028	227	221.35	222.0799	2452676	{}	\N	2020-10-12 16:02:09.04485
67862	RPM	2020-09-28	81.94	83.25	81.94	82.49	385016	{}	\N	2020-10-12 16:02:10.257622
67863	RPM	2020-09-29	82.48	83	81.7	82.31	1014940	{}	\N	2020-10-12 16:02:10.267427
67864	RPM	2020-09-30	82.63	83.31	82.08	82.84	1286369	{}	\N	2020-10-12 16:02:10.274045
67865	RPM	2020-10-01	82.8	83.66	81.57	82.4	1192789	{}	\N	2020-10-12 16:02:10.279492
67866	RPM	2020-10-02	81.11	83.48	81.03	82.64	686857	{}	\N	2020-10-12 16:02:10.28456
67867	RPM	2020-10-05	83.52	85.33	83.347	84.42	984243	{}	\N	2020-10-12 16:02:10.289505
67868	RPM	2020-10-06	85.18	85.98	84.0836	84.32	1444467	{}	\N	2020-10-12 16:02:10.294169
67869	RPM	2020-10-07	86.87	87.45	83.7302	87.35	2297417	{}	\N	2020-10-12 16:02:10.298897
67870	RPM	2020-10-08	88.32	90.84	86.05	86.71	2132005	{}	\N	2020-10-12 16:02:10.303203
67871	RPM	2020-10-09	87.37	88.36	86.37	87.24	1468487	{}	\N	2020-10-12 16:02:10.308526
67872	RPM	2020-10-12	87.98	88.75	86.87	88.68	284159	{}	\N	2020-10-12 16:02:10.313573
67873	RRGB	2020-09-28	13.76	13.89	13.29	13.6	591095	{}	\N	2020-10-12 16:02:11.286016
67874	RRGB	2020-09-29	13.45	13.5	12.21	12.76	939318	{}	\N	2020-10-12 16:02:11.299437
67875	RRGB	2020-09-30	12.86	13.8	12.86	13.16	990331	{}	\N	2020-10-12 16:02:11.307129
67876	RRGB	2020-10-01	13.34	13.4424	13.02	13.39	529526	{}	\N	2020-10-12 16:02:11.315092
67877	RRGB	2020-10-02	12.7	13.27	12.55	13.11	1488657	{}	\N	2020-10-12 16:02:11.321775
67878	RRGB	2020-10-05	13.46	13.5	12.56	12.65	983632	{}	\N	2020-10-12 16:02:11.327566
67879	RRGB	2020-10-06	12.85	13.62	12.6007	12.64	821124	{}	\N	2020-10-12 16:02:11.332705
67880	RRGB	2020-10-07	13.02	13.79	12.8	13.58	624224	{}	\N	2020-10-12 16:02:11.337654
67881	RRGB	2020-10-08	13.66	13.92	13.2637	13.8	503870	{}	\N	2020-10-12 16:02:11.342806
67882	RRGB	2020-10-09	13.97	14.3	13.84	14.18	590989	{}	\N	2020-10-12 16:02:11.347842
67883	RRGB	2020-10-12	14.08	14.14	13.31	13.415	270537	{}	\N	2020-10-12 16:02:11.352587
67884	RUTH	2020-09-28	11.35	11.55	11.14	11.43	1124887	{}	\N	2020-10-12 16:02:12.679786
67885	RUTH	2020-09-29	11.35	11.35	10.7222	11.25	1214129	{}	\N	2020-10-12 16:02:12.691285
67886	RUTH	2020-09-30	11.38	11.81	10.99	11.06	1032802	{}	\N	2020-10-12 16:02:12.698986
67887	RUTH	2020-10-01	11.2	11.33	10.87	11.05	739411	{}	\N	2020-10-12 16:02:12.70536
67888	RUTH	2020-10-02	10.52	11.19	10.4872	11.08	753375	{}	\N	2020-10-12 16:02:12.711599
67889	RUTH	2020-10-05	11.27	11.44	10.71	10.71	631522	{}	\N	2020-10-12 16:02:12.71676
67890	RUTH	2020-10-06	10.84	11.33	10.72	10.9	1310386	{}	\N	2020-10-12 16:02:12.721585
67891	RUTH	2020-10-07	11.19	11.3	10.78	10.88	743574	{}	\N	2020-10-12 16:02:12.726857
67892	RUTH	2020-10-08	10.93	11.39	10.835	11.35	675973	{}	\N	2020-10-12 16:02:12.73194
67893	RUTH	2020-10-09	11.365	11.58	11.25	11.34	593052	{}	\N	2020-10-12 16:02:12.737022
67894	RUTH	2020-10-12	11.86	12.18	11.54	11.63	499713	{}	\N	2020-10-12 16:02:12.741961
67895	RWT	2020-09-28	7.56	7.835	7.56	7.69	1342510	{}	\N	2020-10-12 16:02:14.011799
67896	RWT	2020-09-29	7.73	7.73	7.26	7.4	2822635	{}	\N	2020-10-12 16:02:14.024883
67897	RWT	2020-09-30	7.43	7.72	7.43	7.52	2733502	{}	\N	2020-10-12 16:02:14.032604
67898	RWT	2020-10-01	7.65	7.72	7.49	7.71	1748513	{}	\N	2020-10-12 16:02:14.038997
67899	RWT	2020-10-02	7.47	7.8	7.47	7.79	2836976	{}	\N	2020-10-12 16:02:14.045453
67900	RWT	2020-10-05	7.84	7.88	7.64	7.81	2242088	{}	\N	2020-10-12 16:02:14.050941
67901	RWT	2020-10-06	7.88	8.14	7.8	7.81	2918613	{}	\N	2020-10-12 16:02:14.056307
67902	RWT	2020-10-07	7.9	7.94	7.705	7.86	2831015	{}	\N	2020-10-12 16:02:14.062259
67903	RWT	2020-10-08	7.9	8.17	7.885	8.15	3126264	{}	\N	2020-10-12 16:02:14.067821
67904	RWT	2020-10-09	8.18	8.18	7.85	7.92	1832925	{}	\N	2020-10-12 16:02:14.07254
67905	RWT	2020-10-12	7.89	7.995	7.86	7.93	557489	{}	\N	2020-10-12 16:02:14.077969
67906	RXN	2020-09-28	28.83	29.51	28.83	29.23	537872	{}	\N	2020-10-12 16:02:15.801758
67907	RXN	2020-09-29	29.33	30.16	29.2	29.75	1333884	{}	\N	2020-10-12 16:02:15.814472
67908	RXN	2020-09-30	29.78	30.39	29.6	29.84	1236745	{}	\N	2020-10-12 16:02:15.82293
67909	RXN	2020-10-01	30	30.29	29.315	29.71	1370557	{}	\N	2020-10-12 16:02:15.830493
67910	RXN	2020-10-02	29.22	30.5	29.04	30.16	786345	{}	\N	2020-10-12 16:02:15.836542
67911	RXN	2020-10-05	30.41	31.08	30.31	31.03	867727	{}	\N	2020-10-12 16:02:15.842158
67912	RXN	2020-10-06	31.25	31.86	31	31.07	1509350	{}	\N	2020-10-12 16:02:15.848178
67913	RXN	2020-10-07	31.52	32.12	31.47	31.91	1198611	{}	\N	2020-10-12 16:02:15.854141
67914	RXN	2020-10-08	32.16	32.38	31.39	31.82	935799	{}	\N	2020-10-12 16:02:15.859612
67915	RXN	2020-10-09	32.3	32.38	31.85	31.95	742522	{}	\N	2020-10-12 16:02:15.866043
67916	RXN	2020-10-12	32.19	32.79	32.165	32.58	355740	{}	\N	2020-10-12 16:02:15.872678
67917	RYN	2020-09-28	26.24	26.93	26.23	26.89	361742	{}	\N	2020-10-12 16:02:16.905211
67918	RYN	2020-09-29	26.76	26.94	26.49	26.58	599398	{}	\N	2020-10-12 16:02:16.914759
67919	RYN	2020-09-30	26.72	26.82	26.1	26.44	1398339	{}	\N	2020-10-12 16:02:16.922011
67920	RYN	2020-10-01	26.45	27	26.33	26.97	913965	{}	\N	2020-10-12 16:02:16.928385
67921	RYN	2020-10-02	26.49	27.34	26.38	27.16	1336643	{}	\N	2020-10-12 16:02:16.935002
67922	RYN	2020-10-05	27.22	27.75	26.82	27.7	1075101	{}	\N	2020-10-12 16:02:16.940416
67923	RYN	2020-10-06	27.89	28.13	27.43	27.85	1261625	{}	\N	2020-10-12 16:02:16.945686
67924	RYN	2020-10-07	28.09	28.25	27.915	28.15	926432	{}	\N	2020-10-12 16:02:16.95186
67925	RYN	2020-10-08	28.32	28.42	28.07	28.3	821328	{}	\N	2020-10-12 16:02:16.957335
67926	RYN	2020-10-09	28.39	28.73	28.21	28.56	677952	{}	\N	2020-10-12 16:02:16.962603
67927	RYN	2020-10-12	28.55	28.7	28.2001	28.57	105921	{}	\N	2020-10-12 16:02:16.968545
67928	RJZ	2020-09-28	9.13	9.15	9.13	9.15	218	{}	\N	2020-10-12 16:02:18.605677
67929	RJZ	2020-09-29	9.18	9.18	9.18	9.18	201	{}	\N	2020-10-12 16:02:18.613528
67930	RJZ	2020-09-30	9.2	9.2	9.115	9.115	467	{}	\N	2020-10-12 16:02:18.621241
67931	RJZ	2020-10-01	9.05	9.05	9.0367	9.0367	409	{}	\N	2020-10-12 16:02:18.627573
67932	RJZ	2020-10-02	9.04	9.09	9.04	9.09	833	{}	\N	2020-10-12 16:02:18.633546
67933	RJZ	2020-10-05	9.16	9.16	9.1455	9.1455	103	{}	\N	2020-10-12 16:02:18.638184
67934	RJZ	2020-10-06	9.0242	9.0242	9.0242	9.0242	4	{}	\N	2020-10-12 16:02:18.642719
67935	RJZ	2020-10-07	9.09	9.13	9.09	9.13	303	{}	\N	2020-10-12 16:02:18.648139
67936	RJZ	2020-10-08	9.1608	9.1608	9.1608	9.1608	90	{}	\N	2020-10-12 16:02:18.653157
67937	RJZ	2020-10-09	9.43	9.4599	9.395	9.4599	1631	{}	\N	2020-10-12 16:02:18.657616
67938	RLY	2020-09-28	21.859	21.9099	21.8336	21.885	3859	{}	\N	2020-10-12 16:02:20.593239
67939	RLY	2020-09-29	21.86	21.86	21.65	21.7129	1986	{}	\N	2020-10-12 16:02:20.606421
67940	RLY	2020-09-30	21.82	21.9199	21.775	21.775	6120	{}	\N	2020-10-12 16:02:20.613629
67941	RLY	2020-10-01	21.7245	21.765	21.705	21.7231	1740	{}	\N	2020-10-12 16:02:20.620009
67942	RLY	2020-10-02	21.6	21.825	21.6	21.805	5630	{}	\N	2020-10-12 16:02:20.626476
67943	RLY	2020-10-05	22.01	22.1011	22.01	22.1011	4272	{}	\N	2020-10-12 16:02:20.631758
67944	RLY	2020-10-06	22.17	22.2078	21.95	21.95	1347	{}	\N	2020-10-12 16:02:20.637724
67945	RLY	2020-10-07	22.0736	22.091	22.06	22.091	2771	{}	\N	2020-10-12 16:02:20.642724
67946	RLY	2020-10-08	22.2483	22.3066	22.216	22.3066	865	{}	\N	2020-10-12 16:02:20.647683
67947	RLY	2020-10-09	22.24	22.4387	22.24	22.38	5279	{}	\N	2020-10-12 16:02:20.652925
67948	RLY	2020-10-12	22.35	22.3714	22.35	22.3714	457	{}	\N	2020-10-12 16:02:20.659118
67949	RMTI	2020-09-28	1.09	1.09	1.03	1.04	2252271	{}	\N	2020-10-12 16:02:21.941931
67950	RMTI	2020-09-29	1.09	1.09	1.03	1.05	1541757	{}	\N	2020-10-12 16:02:21.951817
67951	RMTI	2020-09-30	1.05	1.08	1.04	1.07	1535781	{}	\N	2020-10-12 16:02:21.958889
67952	RMTI	2020-10-01	1.07	1.09	1.04	1.07	1719377	{}	\N	2020-10-12 16:02:21.964379
67953	RMTI	2020-10-02	1.05	1.08	1.04	1.06	1078649	{}	\N	2020-10-12 16:02:21.969613
67954	RMTI	2020-10-05	1.07	1.16	1.06	1.11	2279330	{}	\N	2020-10-12 16:02:21.974887
67955	RMTI	2020-10-06	1.11	1.12	1.07	1.08	1754676	{}	\N	2020-10-12 16:02:21.979738
67956	RMTI	2020-10-07	1.09	1.13	1.07	1.1	1608938	{}	\N	2020-10-12 16:02:21.984567
67957	RMTI	2020-10-08	1.1	1.12	1.09	1.11	1057218	{}	\N	2020-10-12 16:02:21.98998
67958	RMTI	2020-10-09	1.12	1.13	1.1	1.11	1076700	{}	\N	2020-10-12 16:02:21.995079
67959	RMTI	2020-10-12	1.11	1.12	1.08	1.085	533951	{}	\N	2020-10-12 16:02:21.99998
67960	RNP	2020-09-28	20.4	20.525	20.3257	20.42	178711	{}	\N	2020-10-12 16:02:23.042574
67961	RNP	2020-09-29	20.4	20.413	20.06	20.26	216404	{}	\N	2020-10-12 16:02:23.053705
67962	RNP	2020-09-30	20.38	20.4	20.09	20.19	263084	{}	\N	2020-10-12 16:02:23.060913
67963	RNP	2020-10-01	20.44	20.57	20.2449	20.55	356726	{}	\N	2020-10-12 16:02:23.066961
67964	RNP	2020-10-02	20.56	20.74	20.37	20.69	275116	{}	\N	2020-10-12 16:02:23.072845
67965	RNP	2020-10-05	20.89	20.9375	20.71	20.84	397156	{}	\N	2020-10-12 16:02:23.077729
67966	RNP	2020-10-06	20.86	21.03	20.7541	20.91	232358	{}	\N	2020-10-12 16:02:23.082385
67967	RNP	2020-10-07	21.04	21.14	20.8	20.87	346169	{}	\N	2020-10-12 16:02:23.087574
67968	RNP	2020-10-08	20.92	21.685	20.92	21.61	601053	{}	\N	2020-10-12 16:02:23.093391
67969	RNP	2020-10-09	21.68	21.87	21.56	21.8	242251	{}	\N	2020-10-12 16:02:23.098716
67970	RNP	2020-10-12	21.8	21.86	21.7038	21.8	91789	{}	\N	2020-10-12 16:02:23.103292
67971	RNWK	2020-09-28	1.21	1.24	1.2	1.23	31231	{}	\N	2020-10-12 16:02:25.170759
67972	RNWK	2020-09-29	1.21	1.25	1.2	1.21	56010	{}	\N	2020-10-12 16:02:25.184428
67973	RNWK	2020-09-30	1.21	1.25	1.205	1.22	44877	{}	\N	2020-10-12 16:02:25.191603
67974	RNWK	2020-10-01	1.23	1.2981	1.2161	1.275	181733	{}	\N	2020-10-12 16:02:25.197644
67975	RNWK	2020-10-02	1.26	1.305	1.2401	1.29	95306	{}	\N	2020-10-12 16:02:25.202864
67976	RNWK	2020-10-05	1.28	1.34	1.25	1.32	217981	{}	\N	2020-10-12 16:02:25.207959
67977	RNWK	2020-10-06	1.2901	1.62	1.29	1.49	2185835	{}	\N	2020-10-12 16:02:25.212446
67978	RNWK	2020-10-07	1.44	1.47	1.385	1.39	376894	{}	\N	2020-10-12 16:02:25.216966
67979	RNWK	2020-10-08	1.43	1.49	1.39	1.41	330304	{}	\N	2020-10-12 16:02:25.221611
67980	RNWK	2020-10-09	1.4	1.4299	1.36	1.37	178023	{}	\N	2020-10-12 16:02:25.226684
67981	RNWK	2020-10-12	1.38	1.445	1.38	1.41	137069	{}	\N	2020-10-12 16:02:25.231193
67982	ROL	2020-09-28	54.17	55.17	54.17	54.66	724663	{}	\N	2020-10-12 16:02:26.663644
67983	ROL	2020-09-29	54.65	55	54.48	54.5	1191760	{}	\N	2020-10-12 16:02:26.674194
67984	ROL	2020-09-30	54.75	54.89	53.87	54.19	2637847	{}	\N	2020-10-12 16:02:26.681435
67985	ROL	2020-10-01	54.63	54.75	53.52	53.81	1712426	{}	\N	2020-10-12 16:02:26.687132
67986	ROL	2020-10-02	53.41	55.05	53.32	54.47	1275312	{}	\N	2020-10-12 16:02:26.693168
67987	ROL	2020-10-05	54.57	55.79	54.54	55.68	3148022	{}	\N	2020-10-12 16:02:26.698204
67988	ROL	2020-10-06	55.66	56.97	55.01	56	2144389	{}	\N	2020-10-12 16:02:26.702743
67989	ROL	2020-10-07	56.43	57.35	56.32	57.11	2080097	{}	\N	2020-10-12 16:02:26.707071
67990	ROL	2020-10-08	57.23	57.32	56.55	57.02	964271	{}	\N	2020-10-12 16:02:26.711656
67991	ROL	2020-10-09	57.35	57.53	56.685	57.47	1163305	{}	\N	2020-10-12 16:02:26.716291
67992	ROL	2020-10-12	57.68	58.34	57.51	58.19	326367	{}	\N	2020-10-12 16:02:26.721282
67993	RP	2020-09-28	59.65	60.34	58.7	58.8	868703	{}	\N	2020-10-12 16:02:28.424402
67994	RP	2020-09-29	58.79	59.05	57.7	57.88	881003	{}	\N	2020-10-12 16:02:28.435984
67995	RP	2020-09-30	57.71	58.2857	57.24	57.64	1053435	{}	\N	2020-10-12 16:02:28.44259
67996	RP	2020-10-01	58.04	59.615	57.81	59.35	462166	{}	\N	2020-10-12 16:02:28.448772
67997	RP	2020-10-02	58.42	58.89	57.31	58.07	536847	{}	\N	2020-10-12 16:02:28.453972
67998	RP	2020-10-05	58.53	59.18	57.83	58.79	725541	{}	\N	2020-10-12 16:02:28.458569
67999	RP	2020-10-06	59.01	59.77	58.16	58.72	444777	{}	\N	2020-10-12 16:02:28.463615
68000	RP	2020-10-07	59.14	59.75	58.115	58.94	1799747	{}	\N	2020-10-12 16:02:28.468298
68001	RP	2020-10-08	59.32	59.98	58.255	58.39	444852	{}	\N	2020-10-12 16:02:28.472588
68002	RP	2020-10-09	59.15	59.75	58.375	59.42	574733	{}	\N	2020-10-12 16:02:28.478049
68003	RP	2020-10-12	59.98	60.52	59.37	59.52	60723	{}	\N	2020-10-12 16:02:28.484919
68004	RRR	2020-09-28	17.5	17.82	16.92	17.53	1337105	{}	\N	2020-10-12 16:02:30.763067
68005	RRR	2020-09-29	17.53	17.75	16.58	16.68	1274247	{}	\N	2020-10-12 16:02:30.775438
68006	RRR	2020-09-30	16.84	17.65	16.84	17.1	1334916	{}	\N	2020-10-12 16:02:30.783122
68007	RRR	2020-10-01	17.4	17.92	17.15	17.58	1026877	{}	\N	2020-10-12 16:02:30.789342
68008	RRR	2020-10-02	16.6	18.08	16.56	17.91	1105389	{}	\N	2020-10-12 16:02:30.7951
68009	RRR	2020-10-05	18	18.13	17.4339	17.92	847733	{}	\N	2020-10-12 16:02:30.800541
68010	RRR	2020-10-06	17.92	18.39	17.32	17.34	1085295	{}	\N	2020-10-12 16:02:30.805702
68011	RRR	2020-10-07	17.68	17.9	17.45	17.77	895336	{}	\N	2020-10-12 16:02:30.810635
68012	RRR	2020-10-08	18	18.34	17.6	17.9	705108	{}	\N	2020-10-12 16:02:30.815992
68013	RRR	2020-10-09	18.08	18.33	17.63	17.645	755505	{}	\N	2020-10-12 16:02:30.820928
68014	RRR	2020-10-12	17.7	18.1822	17.45	18.03	547757	{}	\N	2020-10-12 16:02:30.82586
68015	RTH	2020-09-28	147.32	147.97	146.47	147.7	9196	{}	\N	2020-10-12 16:02:32.010213
68016	RTH	2020-09-29	148.08	148.08	146.8934	146.99	8066	{}	\N	2020-10-12 16:02:32.024384
68017	RTH	2020-09-30	147.56	150.43	147.56	149.2	7760	{}	\N	2020-10-12 16:02:32.032243
68018	RTH	2020-10-01	150.09	151.1	149.57	150.65	8061	{}	\N	2020-10-12 16:02:32.039105
68019	RTH	2020-10-02	148.46	149.98	148.3372	149.75	4706	{}	\N	2020-10-12 16:02:32.04504
68020	RTH	2020-10-05	150.53	151.28	150.1606	151.28	11665	{}	\N	2020-10-12 16:02:32.050968
68021	RTH	2020-10-06	151.65	151.65	148.305	148.61	10360	{}	\N	2020-10-12 16:02:32.056202
68022	RTH	2020-10-07	149.95	151.5889	149.95	151.43	6596	{}	\N	2020-10-12 16:02:32.061001
68023	RTH	2020-10-08	152.77	152.77	151.8461	152.5089	7576	{}	\N	2020-10-12 16:02:32.067419
68024	RTH	2020-10-09	153.38	154.395	153.25	153.94	29710	{}	\N	2020-10-12 16:02:32.07252
68025	RTH	2020-10-12	155.57	156.77	155.15	156.72	4509	{}	\N	2020-10-12 16:02:32.077942
68026	RUSL	2020-09-28	15.41	15.45	15.13	15.31	57006	{}	\N	2020-10-12 16:02:33.788371
68027	RUSL	2020-09-29	15.15	15.32	14.89	15.15	115016	{}	\N	2020-10-12 16:02:33.799347
68028	RUSL	2020-09-30	15.41	15.64	15.36	15.51	59921	{}	\N	2020-10-12 16:02:33.808007
68029	RUSL	2020-10-01	15.55	15.56	15.285	15.42	53335	{}	\N	2020-10-12 16:02:33.816238
68030	RUSL	2020-10-02	15.0615	15.205	15.05	15.12	26845	{}	\N	2020-10-12 16:02:33.822526
68031	RUSL	2020-10-05	15.25	15.6	15.25	15.58	67382	{}	\N	2020-10-12 16:02:33.827448
68032	RUSL	2020-10-06	15.91	15.91	15.42	15.58	93975	{}	\N	2020-10-12 16:02:33.832457
68033	RUSL	2020-10-07	15.28	15.33	15.11	15.27	84082	{}	\N	2020-10-12 16:02:33.837629
68034	RUSL	2020-10-08	15.56	15.68	15.44	15.58	56376	{}	\N	2020-10-12 16:02:33.843564
68035	RUSL	2020-10-09	15.75	15.91	15.7011	15.83	57645	{}	\N	2020-10-12 16:02:33.848952
68036	RUSL	2020-10-12	15.9565	15.99	15.84	15.87	26228	{}	\N	2020-10-12 16:02:33.854472
68037	RVP	2020-09-28	6.52	6.52	6.11	6.28	244287	{}	\N	2020-10-12 16:02:34.819656
68038	RVP	2020-09-29	6.25	6.9	6.2	6.8	471835	{}	\N	2020-10-12 16:02:34.830433
68039	RVP	2020-09-30	6.85	6.85	6.45	6.66	327863	{}	\N	2020-10-12 16:02:34.838251
68040	RVP	2020-10-01	6.57	7	6.5028	6.78	244696	{}	\N	2020-10-12 16:02:34.846593
68041	RVP	2020-10-02	6.51	6.95	6.2843	6.7	347080	{}	\N	2020-10-12 16:02:34.852151
68042	RVP	2020-10-05	6.79	7.35	6.7665	7.17	381119	{}	\N	2020-10-12 16:02:34.857252
68043	RVP	2020-10-06	7.3	7.94	7.05	7.7	663880	{}	\N	2020-10-12 16:02:34.862217
68044	RVP	2020-10-07	7.66	8.16	7.57	7.93	461754	{}	\N	2020-10-12 16:02:34.867448
68045	RVP	2020-10-08	8.05	8.75	7.93	8.25	508666	{}	\N	2020-10-12 16:02:34.872362
68046	RVP	2020-10-09	8.49	8.61	8.02	8.6	380015	{}	\N	2020-10-12 16:02:34.877042
68047	RVP	2020-10-12	8.73	9.59	8.4563	9.0404	941686	{}	\N	2020-10-12 16:02:34.881502
68048	RWM	2020-09-28	34.29	34.3	33.79	33.91	1695191	{}	\N	2020-10-12 16:02:35.952266
68049	RWM	2020-09-29	33.94	34.33	33.8301	34.05	908777	{}	\N	2020-10-12 16:02:35.963984
68050	RWM	2020-09-30	33.91	34.22	33.45	33.94	1852944	{}	\N	2020-10-12 16:02:35.972472
68051	RWM	2020-10-01	33.66	33.99	33.39	33.4	980947	{}	\N	2020-10-12 16:02:35.979782
68052	RWM	2020-10-02	34.03	34.08	33.1	33.25	1645380	{}	\N	2020-10-12 16:02:35.985688
68053	RWM	2020-10-05	32.891	32.891	32.28	32.33	1402117	{}	\N	2020-10-12 16:02:35.991346
68054	RWM	2020-10-06	32.01	32.47	31.51	32.4	3532467	{}	\N	2020-10-12 16:02:35.996575
68055	RWM	2020-10-07	31.99	32.01	31.605	31.7	1336619	{}	\N	2020-10-12 16:02:36.002034
68056	RWM	2020-10-08	31.37	31.6	31.27	31.36	982733	{}	\N	2020-10-12 16:02:36.007368
68057	RWM	2020-10-09	31.06	31.35	31	31.18	1180836	{}	\N	2020-10-12 16:02:36.012516
68058	RWM	2020-10-12	31.03	31.185	30.91	31.01	464114	{}	\N	2020-10-12 16:02:36.017962
68059	RSG	2020-09-28	94.91	95.58	94.19	94.46	857759	{}	\N	2020-10-12 16:02:37.163861
68060	RSG	2020-09-29	94.56	94.96	93.78	93.92	1841595	{}	\N	2020-10-12 16:02:37.178407
68061	RSG	2020-09-30	93.7	94.11	92.79	93.35	2459375	{}	\N	2020-10-12 16:02:37.186106
68062	RSG	2020-10-01	93.67	93.67	92.18	92.97	2428410	{}	\N	2020-10-12 16:02:37.192912
68063	RSG	2020-10-02	91.35	93.88	91.35	93.32	3883281	{}	\N	2020-10-12 16:02:37.198493
68064	RSG	2020-10-05	93.8	94.46	93.17	93.53	2080795	{}	\N	2020-10-12 16:02:37.204946
68065	RSG	2020-10-06	94.13	94.38	92.26	92.48	1932900	{}	\N	2020-10-12 16:02:37.209509
68066	RSG	2020-10-07	93.1	93.39	92.48	92.92	2132945	{}	\N	2020-10-12 16:02:37.214336
68067	RSG	2020-10-08	93.36	94	92.87	93.5	1854192	{}	\N	2020-10-12 16:02:37.219433
68068	RSG	2020-10-09	93.8	93.95	93.12	93.14	2098450	{}	\N	2020-10-12 16:02:37.224312
68069	RSG	2020-10-12	93.43	94.81	93.29	94.65	1325128	{}	\N	2020-10-12 16:02:37.229127
68070	RTM	2020-09-28	116.51	117.3602	116.4367	116.4367	3100	{}	\N	2020-10-12 16:02:38.283594
68071	RTM	2020-09-29	116.4998	116.4998	115.218	115.874	1868	{}	\N	2020-10-12 16:02:38.296817
68072	RTM	2020-09-30	115.73	117.575	115.73	116.25	22339	{}	\N	2020-10-12 16:02:38.303907
68073	RTM	2020-10-01	117.495	117.576	114.875	114.9947	30061	{}	\N	2020-10-12 16:02:38.310831
68074	RTM	2020-10-02	115.19	117.44	115.09	116.7423	8804	{}	\N	2020-10-12 16:02:38.316501
68075	RTM	2020-10-05	117.92	120.21	117.92	119.92	3704	{}	\N	2020-10-12 16:02:38.321868
68076	RTM	2020-10-06	120.89	121.705	118.5756	118.5756	5896	{}	\N	2020-10-12 16:02:38.327004
68077	RTM	2020-10-07	120.86	122.05	120.86	121.86	21586	{}	\N	2020-10-12 16:02:38.331989
68078	RTM	2020-10-08	123.329	123.6766	122.63	123.6766	10808	{}	\N	2020-10-12 16:02:38.33673
68079	RTM	2020-10-09	124.9	125.3	123.82	124.2	11539	{}	\N	2020-10-12 16:02:38.342184
68080	RTM	2020-10-12	124.6934	124.76	124.46	124.67	2654	{}	\N	2020-10-12 16:02:38.348078
68081	RWK	2020-09-28	53.34	54.2923	53.34	54.18	11127	{}	\N	2020-10-12 16:02:40.897449
68082	RWK	2020-09-29	53.8	54.0099	53.325	53.6051	7260	{}	\N	2020-10-12 16:02:40.907945
68083	RWK	2020-09-30	53.62	54.5649	53.62	54.01	14554	{}	\N	2020-10-12 16:02:40.914698
68084	RWK	2020-10-01	53.98	54.7483	53.98	54.73	12029	{}	\N	2020-10-12 16:02:40.92059
68085	RWK	2020-10-02	53.7	55.935	53.7	55.86	7574	{}	\N	2020-10-12 16:02:40.92689
68086	RWK	2020-10-05	56.03	57.125	56.03	57.08	5652	{}	\N	2020-10-12 16:02:40.931652
68087	RWK	2020-10-06	57.23	58.27	56.57	56.74	5419	{}	\N	2020-10-12 16:02:40.936523
68088	RWK	2020-10-07	57.8653	58.22	57.78	58.02	9050	{}	\N	2020-10-12 16:02:40.941532
68089	RWK	2020-10-08	58.24	58.88	58.24	58.88	6815	{}	\N	2020-10-12 16:02:40.946537
68090	RWK	2020-10-09	59.08	59.08	58.6175	58.64	4847	{}	\N	2020-10-12 16:02:40.9514
68091	RWK	2020-10-12	59.15	59.31	58.905	59.1365	1683	{}	\N	2020-10-12 16:02:40.956342
68092	RYAAY	2020-09-28	82.72	82.83	81.2	81.41	993210	{}	\N	2020-10-12 16:02:42.182599
68093	RYAAY	2020-09-29	80.71	80.99	79.93	80.52	659919	{}	\N	2020-10-12 16:02:42.193325
68094	RYAAY	2020-09-30	80.42	83.37	80.33	81.76	846109	{}	\N	2020-10-12 16:02:42.200766
68095	RYAAY	2020-10-01	82.01	82.87	81.42	81.99	373644	{}	\N	2020-10-12 16:02:42.20786
68096	RYAAY	2020-10-02	80.64	82.42	80.44	81.59	372468	{}	\N	2020-10-12 16:02:42.214261
68097	RYAAY	2020-10-05	83.02	83.43	81.82	82.25	280560	{}	\N	2020-10-12 16:02:42.219573
68098	RYAAY	2020-10-06	85.48	86.95	84.73	85.48	532827	{}	\N	2020-10-12 16:02:42.225532
68099	RYAAY	2020-10-07	87.79	88.26	86.84	87.36	455810	{}	\N	2020-10-12 16:02:42.230596
68100	RYAAY	2020-10-08	89.16	89.84	88.265	88.59	282849	{}	\N	2020-10-12 16:02:42.235466
68101	RYAAY	2020-10-09	87.9	88.41	86.5	86.74	270808	{}	\N	2020-10-12 16:02:42.240371
68102	RYAAY	2020-10-12	87.29	87.97	86.975	87.5347	43773	{}	\N	2020-10-12 16:02:42.245464
68103	RZV	2020-09-28	50.84	51.5202	50.84	51.29	84299	{}	\N	2020-10-12 16:02:43.149515
68104	RZV	2020-09-29	50.97	50.97	50.47	50.47	7013	{}	\N	2020-10-12 16:02:43.159426
68105	RZV	2020-09-30	50.82	51.21	50.29	50.59	22214	{}	\N	2020-10-12 16:02:43.167264
68106	RZV	2020-10-01	50.6212	51.274	50.5339	51.15	6975	{}	\N	2020-10-12 16:02:43.173488
68107	RZV	2020-10-02	51	52.35	51	52.35	3641	{}	\N	2020-10-12 16:02:43.1799
68108	RZV	2020-10-05	53.7	53.98	53.3122	53.98	41823	{}	\N	2020-10-12 16:02:43.184695
68109	RZV	2020-10-06	54.62	55.1501	53.6435	53.6435	8223	{}	\N	2020-10-12 16:02:43.189155
68110	RZV	2020-10-07	54.34	55.33	54.34	55.31	2031	{}	\N	2020-10-12 16:02:43.194386
68111	RZV	2020-10-08	55.65	56.27	55.175	56.27	16749	{}	\N	2020-10-12 16:02:43.199262
68112	RZV	2020-10-09	56.92	57.0435	56.26	56.27	7586	{}	\N	2020-10-12 16:02:43.20382
68113	RZV	2020-10-12	56.4693	56.599	56.295	56.4845	2144	{}	\N	2020-10-12 16:02:43.209378
68114	SAFM	2020-09-28	119.31	121	118.71	119.97	130196	{}	\N	2020-10-12 16:02:44.087939
68115	SAFM	2020-09-29	119.85	122.485	119.01	119.24	117786	{}	\N	2020-10-12 16:02:44.098971
68116	SAFM	2020-09-30	119.85	121.355	117.63	117.97	184423	{}	\N	2020-10-12 16:02:44.10573
68117	SAFM	2020-10-01	118.03	119.65	116.65	119.5	172633	{}	\N	2020-10-12 16:02:44.111927
68118	SAFM	2020-10-02	118.04	121.43	117.43	120.77	154689	{}	\N	2020-10-12 16:02:44.116956
68119	SAFM	2020-10-05	121.32	122.8	119.99	121.81	179529	{}	\N	2020-10-12 16:02:44.121853
68120	SAFM	2020-10-06	122.49	125.565	120.66	123.71	208141	{}	\N	2020-10-12 16:02:44.126941
68121	SAFM	2020-10-07	123.05	125.62	122.19	123.83	171259	{}	\N	2020-10-12 16:02:44.13205
68122	SAFM	2020-10-08	124.98	126.13	121.505	122.24	106045	{}	\N	2020-10-12 16:02:44.137469
68123	SAFM	2020-10-09	122.08	124.7118	118	118.48	225740	{}	\N	2020-10-12 16:02:44.143307
68124	SAFM	2020-10-12	118.37	122.99	117.65	122.71	59002	{}	\N	2020-10-12 16:02:44.149304
68125	SAIL	2020-09-28	38.02	39.1699	37.671	38.49	794859	{}	\N	2020-10-12 16:02:45.443579
68126	SAIL	2020-09-29	38.7	39.04	38.06	38.44	1209941	{}	\N	2020-10-12 16:02:45.457358
68127	SAIL	2020-09-30	38.4	42.62	37.91	39.57	7742394	{}	\N	2020-10-12 16:02:45.464948
68128	SAIL	2020-10-01	44.19	46.19	44.15	45.58	21731502	{}	\N	2020-10-12 16:02:45.471389
68129	SAIL	2020-10-02	44.79	47.09	44.55	45.11	32389005	{}	\N	2020-10-12 16:02:45.477998
68130	SAIL	2020-10-05	45.11	46.3	44.6	45.49	5096098	{}	\N	2020-10-12 16:02:45.483532
68131	SAIL	2020-10-06	45.63	47.54	45.49	45.93	5076731	{}	\N	2020-10-12 16:02:45.488376
68132	SAIL	2020-10-07	46.49	47.18	46.26	46.26	2801166	{}	\N	2020-10-12 16:02:45.492959
68133	SAIL	2020-10-08	46.6	47	43.4	44.79	3139350	{}	\N	2020-10-12 16:02:45.498189
68134	SAIL	2020-10-09	45	45.38	44.29	44.6	2311466	{}	\N	2020-10-12 16:02:45.502934
68135	SAIL	2020-10-12	45.18	45.382	44.33	45.25	891326	{}	\N	2020-10-12 16:02:45.50769
68136	SANW	2020-09-28	2.53	2.58	2.5	2.5	17799	{}	\N	2020-10-12 16:02:47.139224
68137	SANW	2020-09-29	2.5	2.53	2.485	2.5	35879	{}	\N	2020-10-12 16:02:47.149371
68138	SANW	2020-09-30	2.5	2.53	2.38	2.48	63497	{}	\N	2020-10-12 16:02:47.157167
68139	SANW	2020-10-01	2.48	2.51	2.3579	2.41	38094	{}	\N	2020-10-12 16:02:47.163484
68140	SANW	2020-10-02	2.35	2.48	2.35	2.45	12218	{}	\N	2020-10-12 16:02:47.169593
68141	SANW	2020-10-05	2.44	2.49	2.41	2.41	14005	{}	\N	2020-10-12 16:02:47.1747
68142	SANW	2020-10-06	2.42	2.44	2.38	2.38	13579	{}	\N	2020-10-12 16:02:47.179893
68143	SANW	2020-10-07	2.38	2.42	2.345	2.39	20450	{}	\N	2020-10-12 16:02:47.185228
68144	SANW	2020-10-08	2.35	2.4199	2.28	2.35	42321	{}	\N	2020-10-12 16:02:47.19008
68145	SANW	2020-10-09	2.34	2.4	2.3	2.35	38141	{}	\N	2020-10-12 16:02:47.195191
68146	SANW	2020-10-12	2.39	2.42	2.37	2.4	18545	{}	\N	2020-10-12 16:02:47.200512
68147	SB	2020-09-28	1.02	1.06	1.01	1.05	164492	{}	\N	2020-10-12 16:02:48.281429
68148	SB	2020-09-29	1.04	1.06	1.04	1.05	97250	{}	\N	2020-10-12 16:02:48.294391
68149	SB	2020-09-30	1.03	1.07	1.03	1.03	84124	{}	\N	2020-10-12 16:02:48.301787
68150	SB	2020-10-01	1.04	1.12	1.04	1.1	322587	{}	\N	2020-10-12 16:02:48.307989
68151	SB	2020-10-02	1.11	1.11	1.05	1.05	350681	{}	\N	2020-10-12 16:02:48.314285
68152	SB	2020-10-05	1.07	1.09	1.04	1.04	930585	{}	\N	2020-10-12 16:02:48.319715
68153	SB	2020-10-06	1.07	1.09	1.05	1.06	204008	{}	\N	2020-10-12 16:02:48.324903
68154	SB	2020-10-07	1.05	1.08	1.04	1.07	124918	{}	\N	2020-10-12 16:02:48.33013
68155	SB	2020-10-08	1.08	1.08	1.04	1.07	174951	{}	\N	2020-10-12 16:02:48.335542
68156	SB	2020-10-09	1.07	1.09	1.06	1.08	145072	{}	\N	2020-10-12 16:02:48.340999
68157	SB	2020-10-12	1.06	1.08	1.05	1.06	53820	{}	\N	2020-10-12 16:02:48.346544
68158	SBLKZ	2020-09-28	25.2159	25.2159	25.2	25.2	1052	{}	\N	2020-10-12 16:02:49.511268
68159	SBLKZ	2020-09-29	25.16	25.16	25.1	25.1	860	{}	\N	2020-10-12 16:02:49.519031
68160	SBLKZ	2020-09-30	25.13	25.25	25.13	25.25	1837	{}	\N	2020-10-12 16:02:49.524453
68161	SBLKZ	2020-10-01	25.2352	25.2352	25.2352	25.2352	100	{}	\N	2020-10-12 16:02:49.530263
68162	SBLKZ	2020-10-02	25.37	25.37	25.24	25.24	275	{}	\N	2020-10-12 16:02:49.535898
68163	SBLKZ	2020-10-05	25.24	25.3634	25.2	25.2	2615	{}	\N	2020-10-12 16:02:49.541033
68164	SBLKZ	2020-10-06	25.3	25.4	25.3	25.4	854	{}	\N	2020-10-12 16:02:49.545705
68165	SBLKZ	2020-10-07	25.31	25.39	25.26	25.39	1701	{}	\N	2020-10-12 16:02:49.551075
68166	SBLKZ	2020-10-08	25.48	25.48	25.48	25.48	200	{}	\N	2020-10-12 16:02:49.556093
68167	SCA	2020-09-28	24.56	24.56	24.56	24.56	100	{}	\N	2020-10-12 16:02:50.725644
68168	SCA	2020-09-29	24.55	24.658	24.55	24.6425	7360	{}	\N	2020-10-12 16:02:50.734514
68169	SCA	2020-09-30	24.6	24.6	24.6	24.6	244	{}	\N	2020-10-12 16:02:50.741332
68170	SCA	2020-10-01	24.65	24.6712	24.65	24.6712	1660	{}	\N	2020-10-12 16:02:50.746916
68171	SCA	2020-10-02	24.65	24.65	24.5	24.6	2488	{}	\N	2020-10-12 16:02:50.752864
68172	SCA	2020-10-05	24.6255	24.6255	24.55	24.57	1668	{}	\N	2020-10-12 16:02:50.757735
68173	SCA	2020-10-06	24.57	24.8	24.5	24.75	50710	{}	\N	2020-10-12 16:02:50.763253
68174	SCA	2020-10-07	24.49	24.92	24.42	24.715	7602	{}	\N	2020-10-12 16:02:50.767917
68175	SCA	2020-10-08	24.6	24.6	24.59	24.59	600	{}	\N	2020-10-12 16:02:50.773225
68176	SCA	2020-10-09	24.62	24.92	24.51	24.92	6810	{}	\N	2020-10-12 16:02:50.778217
68177	SCHA	2020-09-28	67.75	68.6199	67.7237	68.41	309564	{}	\N	2020-10-12 16:02:52.001049
68178	SCHA	2020-09-29	68.37	68.6075	67.61	68.11	355417	{}	\N	2020-10-12 16:02:52.013943
68179	SCHA	2020-09-30	68.26	69.2	67.7801	68.32	360144	{}	\N	2020-10-12 16:02:52.022094
68180	SCHA	2020-10-01	68.88	69.38	68.38	69.32	224934	{}	\N	2020-10-12 16:02:52.028721
68181	SCHA	2020-10-02	67.84	70.09	67.78	69.78	1612736	{}	\N	2020-10-12 16:02:52.03494
68182	SCHA	2020-10-05	70.42	71.62	70.42	71.58	272261	{}	\N	2020-10-12 16:02:52.040726
68183	SCHA	2020-10-06	72.04	73.222	71.22	71.38	561680	{}	\N	2020-10-12 16:02:52.046193
68184	SCHA	2020-10-07	72.12	73.0067	72.12	72.83	294162	{}	\N	2020-10-12 16:02:52.051728
68185	SCHA	2020-10-08	73.43	73.74	73.0826	73.65	376702	{}	\N	2020-10-12 16:02:52.057546
68186	SCHA	2020-10-09	74.24	74.46	73.7	74.09	296716	{}	\N	2020-10-12 16:02:52.063228
68187	SCHA	2020-10-12	74.416	74.6439	74.09	74.46	58258	{}	\N	2020-10-12 16:02:52.068722
68188	RSXJ	2020-09-28	32.26	32.3839	32.2403	32.3839	1462	{}	\N	2020-10-12 16:02:54.082908
68189	RSXJ	2020-09-29	32.15	32.15	31.96	32.06	6034	{}	\N	2020-10-12 16:02:54.094065
68190	RSXJ	2020-09-30	32.72	32.72	32.47	32.47	1030	{}	\N	2020-10-12 16:02:54.101922
68191	RSXJ	2020-10-01	32.7	32.7	32.49	32.49	546	{}	\N	2020-10-12 16:02:54.109871
68192	RSXJ	2020-10-02	32.8227	32.8227	32.8227	32.8227	151	{}	\N	2020-10-12 16:02:54.115595
68193	RSXJ	2020-10-05	32.95	32.95	32.925	32.925	284	{}	\N	2020-10-12 16:02:54.122296
68194	RSXJ	2020-10-06	33.08	33.08	32.7	32.7	836	{}	\N	2020-10-12 16:02:54.12798
68195	RSXJ	2020-10-07	32.9299	32.9299	32.7982	32.7982	613	{}	\N	2020-10-12 16:02:54.133276
68196	RSXJ	2020-10-08	33.34	33.41	33.1226	33.1226	2757	{}	\N	2020-10-12 16:02:54.13938
68197	RSXJ	2020-10-09	32.99	33.3	32.99	33.3	2919	{}	\N	2020-10-12 16:02:54.144564
68198	RWX	2020-09-28	29.75	29.83	29.7	29.81	647873	{}	\N	2020-10-12 16:02:57.515766
68199	RWX	2020-09-29	29.58	29.625	29.43	29.54	218277	{}	\N	2020-10-12 16:02:57.532629
68200	RWX	2020-09-30	29.59	29.87	29.59	29.71	602712	{}	\N	2020-10-12 16:02:57.540251
68201	RWX	2020-10-01	29.96	30.07	29.85	30.03	243727	{}	\N	2020-10-12 16:02:57.546741
68202	RWX	2020-10-02	29.72	30.13	29.72	30.095	449791	{}	\N	2020-10-12 16:02:57.551783
68203	RWX	2020-10-05	30.34	30.46	30.29	30.46	475675	{}	\N	2020-10-12 16:02:57.557024
68204	RWX	2020-10-06	30.42	30.57	30.21	30.23	370781	{}	\N	2020-10-12 16:02:57.561915
68205	RWX	2020-10-07	30.34	30.38	30.21	30.265	220687	{}	\N	2020-10-12 16:02:57.566458
68206	RWX	2020-10-08	30.3	30.35	30.26	30.33	285700	{}	\N	2020-10-12 16:02:57.570995
68207	RWX	2020-10-09	30.33	30.44	30.29	30.4	220815	{}	\N	2020-10-12 16:02:57.575913
68208	RWX	2020-10-12	30.39	30.519	30.39	30.47	277885	{}	\N	2020-10-12 16:02:57.580512
68209	RYE	2020-09-28	23.06	23.54	23.06	23.25	43451	{}	\N	2020-10-12 16:02:58.530583
68210	RYE	2020-09-29	23.09	23.09	22.3557	22.68	16485	{}	\N	2020-10-12 16:02:58.541514
68211	RYE	2020-09-30	22.79	23.02	22.41	22.51	41598	{}	\N	2020-10-12 16:02:58.548088
68212	RYE	2020-10-01	22.25	22.29	21.66	21.75	27423	{}	\N	2020-10-12 16:02:58.554245
68213	RYE	2020-10-02	21.05	22.27	21.05	22.1645	7297	{}	\N	2020-10-12 16:02:58.560024
68214	RYE	2020-10-05	22.51	22.92	22.3926	22.92	1543009	{}	\N	2020-10-12 16:02:58.565334
68215	RYE	2020-10-06	23.31	23.45	22.43	22.43	14543	{}	\N	2020-10-12 16:02:58.570351
68216	RYE	2020-10-07	22.52	22.82	22.38	22.78	18282	{}	\N	2020-10-12 16:02:58.575799
68217	RYE	2020-10-08	22.88	23.71	22.88	23.71	31146	{}	\N	2020-10-12 16:02:58.581033
68218	RYE	2020-10-09	23.97	23.97	23.31	23.34	23743	{}	\N	2020-10-12 16:02:58.586124
68219	RYE	2020-10-12	23.28	23.28	22.94	23.2	5075	{}	\N	2020-10-12 16:02:58.591524
68220	RYTM	2020-09-28	22.91	22.98	21.42	22.41	343811	{}	\N	2020-10-12 16:02:59.562884
68221	RYTM	2020-09-29	22.52	22.84	21.82	22.27	138704	{}	\N	2020-10-12 16:02:59.574224
68222	RYTM	2020-09-30	22.18	22.26	21.49	21.67	172856	{}	\N	2020-10-12 16:02:59.581108
68223	RYTM	2020-10-01	21.34	23.03	21.33	22.65	356908	{}	\N	2020-10-12 16:02:59.586365
68224	RYTM	2020-10-02	22.17	22.85	21.31	21.37	158467	{}	\N	2020-10-12 16:02:59.591019
68225	RYTM	2020-10-05	21.46	22.44	21.46	22.28	218300	{}	\N	2020-10-12 16:02:59.596359
68226	RYTM	2020-10-06	22.49	23.36	22.3	22.36	216738	{}	\N	2020-10-12 16:02:59.601096
68227	RYTM	2020-10-07	22.57	23.38	22	23.35	219850	{}	\N	2020-10-12 16:02:59.60559
68228	RYTM	2020-10-08	24.07	26.49	23.545	24.54	651072	{}	\N	2020-10-12 16:02:59.610586
68229	RYTM	2020-10-09	24.71	25.24	24.05	24.62	200024	{}	\N	2020-10-12 16:02:59.615465
68230	RYTM	2020-10-12	24.52	24.7	24.09	24.48	46060	{}	\N	2020-10-12 16:02:59.620244
68231	SABR	2020-09-28	6.15	6.755	6.13	6.71	9878941	{}	\N	2020-10-12 16:03:00.533921
68232	SABR	2020-09-29	6.63	6.63	6.36	6.5	12530821	{}	\N	2020-10-12 16:03:00.539596
68233	SABR	2020-09-30	6.57	6.8	6.44	6.51	10571944	{}	\N	2020-10-12 16:03:00.544438
68234	SABR	2020-10-01	6.57	6.85	6.475	6.84	6718080	{}	\N	2020-10-12 16:03:00.549198
68235	SABR	2020-10-02	6.52	6.94	6.4602	6.915	8664817	{}	\N	2020-10-12 16:03:00.553992
68236	SABR	2020-10-05	6.98	7.02	6.695	6.78	5452269	{}	\N	2020-10-12 16:03:00.558814
68237	SABR	2020-10-06	6.91	7.01	6.66	6.68	4101017	{}	\N	2020-10-12 16:03:00.564598
68238	SABR	2020-10-07	6.8	6.89	6.65	6.88	3100810	{}	\N	2020-10-12 16:03:00.569724
68239	SABR	2020-10-08	6.93	7.12	6.9033	7.03	5756196	{}	\N	2020-10-12 16:03:00.574598
68240	SABR	2020-10-09	7.09	7.15	6.875	6.9	7883228	{}	\N	2020-10-12 16:03:00.580866
68241	SABR	2020-10-12	6.88	6.91	6.68	6.73	1109959	{}	\N	2020-10-12 16:03:00.585554
68242	SAH	2020-09-28	39.87	41	39.755	40.92	284221	{}	\N	2020-10-12 16:03:02.103454
68243	SAH	2020-09-29	41.08	41.08	39.51	40.25	843933	{}	\N	2020-10-12 16:03:02.114502
68244	SAH	2020-09-30	40.34	41.11	39.5	40.16	681893	{}	\N	2020-10-12 16:03:02.121423
68245	SAH	2020-10-01	40.64	41.65	39.87	41.56	470780	{}	\N	2020-10-12 16:03:02.126871
68246	SAH	2020-10-02	40.38	43.51	40.33	43.45	500932	{}	\N	2020-10-12 16:03:02.132152
68247	SAH	2020-10-05	43.93	45.81	43.93	45.43	485825	{}	\N	2020-10-12 16:03:02.136817
68248	SAH	2020-10-06	46.5	46.84	44.63	45.41	1159208	{}	\N	2020-10-12 16:03:02.142261
68249	SAH	2020-10-07	46.39	46.73	44.11	44.97	824020	{}	\N	2020-10-12 16:03:02.146767
68250	SAH	2020-10-08	45.5	45.57	41.1	41.93	1829381	{}	\N	2020-10-12 16:03:02.151254
68251	SAH	2020-10-09	42.46	42.96	41.2801	41.5	701153	{}	\N	2020-10-12 16:03:02.155255
68252	SAH	2020-10-12	41.6	42.02	40.88	41.28	380606	{}	\N	2020-10-12 16:03:02.1592
68253	SAMG	2020-09-28	11.0717	11.37	10.78	10.81	17825	{}	\N	2020-10-12 16:03:04.223927
68254	SAMG	2020-09-29	10.88	11.17	10.07	10.3	43676	{}	\N	2020-10-12 16:03:04.23885
68255	SAMG	2020-09-30	10.38	10.8475	9.41	10.46	128883	{}	\N	2020-10-12 16:03:04.246279
68256	SAMG	2020-10-01	10.47	10.79	10.32	10.47	18635	{}	\N	2020-10-12 16:03:04.252491
68257	SAMG	2020-10-02	10.45	10.74	10.36	10.38	15594	{}	\N	2020-10-12 16:03:04.257344
68258	SAMG	2020-10-05	10.56	11.14	10.495	11.14	16728	{}	\N	2020-10-12 16:03:04.262209
68259	SAMG	2020-10-06	11.14	11.48	11.07	11.27	12853	{}	\N	2020-10-12 16:03:04.267063
68260	SAMG	2020-10-07	11.17	12.13	11.09	11.8	91225	{}	\N	2020-10-12 16:03:04.272062
68261	SAMG	2020-10-08	11.9	12.05	11.79	11.97	15823	{}	\N	2020-10-12 16:03:04.276955
68262	SAMG	2020-10-09	12.1	12.4	11.6	11.98	18934	{}	\N	2020-10-12 16:03:04.282588
68263	SAMG	2020-10-12	11.98	12.18	11.98	12.07	3372	{}	\N	2020-10-12 16:03:04.287297
68264	SAP	2020-09-28	155.93	157.64	155.5	157.08	782170	{}	\N	2020-10-12 16:03:05.334009
68265	SAP	2020-09-29	156.4	157.52	156.13	156.57	985181	{}	\N	2020-10-12 16:03:05.346697
68266	SAP	2020-09-30	156.06	157.03	154.59	155.81	968755	{}	\N	2020-10-12 16:03:05.354375
68267	SAP	2020-10-01	156.39	157.42	155.42	156.82	910013	{}	\N	2020-10-12 16:03:05.361122
68268	SAP	2020-10-02	154.74	155.89	153.83	154.29	999365	{}	\N	2020-10-12 16:03:05.366968
68269	SAP	2020-10-05	154.79	156.75	154.73	156.6	574366	{}	\N	2020-10-12 16:03:05.372341
68270	SAP	2020-10-06	155.69	156.37	153.74	154.05	1278148	{}	\N	2020-10-12 16:03:05.376945
68271	SAP	2020-10-07	153.39	154.6	152.41	154.32	1730154	{}	\N	2020-10-12 16:03:05.381903
68272	SAP	2020-10-08	155.55	156.29	155.035	156.12	747751	{}	\N	2020-10-12 16:03:05.386879
68273	SAP	2020-10-09	156.69	157.71	156.6	157.14	550982	{}	\N	2020-10-12 16:03:05.391426
68274	SAP	2020-10-12	158.16	158.88	157.95	158.71	339947	{}	\N	2020-10-12 16:03:05.396112
68275	SBAC	2020-09-28	316.94	317.83	309.29	310.48	642475	{}	\N	2020-10-12 16:03:07.075971
68276	SBAC	2020-09-29	312	314.88	309.62	313.87	395160	{}	\N	2020-10-12 16:03:07.090675
68277	SBAC	2020-09-30	314.92	319.2199	313.05	318.48	607628	{}	\N	2020-10-12 16:03:07.098442
68278	SBAC	2020-10-01	320.45	322.09	313.32	315.18	856394	{}	\N	2020-10-12 16:03:07.105195
68279	SBAC	2020-10-02	314.05	320.59	311.14	318.12	644434	{}	\N	2020-10-12 16:03:07.110923
68280	SBAC	2020-10-05	317.71	322.01	312.87	321.93	563122	{}	\N	2020-10-12 16:03:07.116142
68281	SBAC	2020-10-06	322.57	322.57	313.68	315.04	456398	{}	\N	2020-10-12 16:03:07.12127
68282	SBAC	2020-10-07	315.98	315.98	309.71	313.31	359337	{}	\N	2020-10-12 16:03:07.126064
68283	SBAC	2020-10-08	320.58	320.58	313.14	315.32	502725	{}	\N	2020-10-12 16:03:07.130785
68284	SBAC	2020-10-09	315.48	316.29	312.23	314.63	413929	{}	\N	2020-10-12 16:03:07.13588
68285	SBAC	2020-10-12	313.89	315.79	313.07	313.45	125704	{}	\N	2020-10-12 16:03:07.140961
68286	SBFG	2020-09-28	12.44	12.84	12.205	12.69	10325	{}	\N	2020-10-12 16:03:08.958071
68287	SBFG	2020-09-29	12.57	13.5675	12.57	13.39	11518	{}	\N	2020-10-12 16:03:08.968414
68288	SBFG	2020-09-30	13.45	13.66	13.2	13.49	6887	{}	\N	2020-10-12 16:03:08.975185
68289	SBFG	2020-10-01	13.62	14.55	13.57	14.42	19760	{}	\N	2020-10-12 16:03:08.982196
68290	SBFG	2020-10-02	14.08	14.785	14.08	14.64	10815	{}	\N	2020-10-12 16:03:08.987725
68291	SBFG	2020-10-05	14.39	14.95	14.39	14.95	5592	{}	\N	2020-10-12 16:03:08.993166
68292	SBFG	2020-10-06	14.8	14.9165	14.61	14.61	8914	{}	\N	2020-10-12 16:03:08.998522
68293	SBFG	2020-10-07	14.62	15.24	14.3618	15.24	15369	{}	\N	2020-10-12 16:03:09.004117
68294	SBFG	2020-10-08	15.36	15.46	15	15.46	14814	{}	\N	2020-10-12 16:03:09.009357
68295	SBFG	2020-10-09	15.64	15.74	15.5	15.74	3136	{}	\N	2020-10-12 16:03:09.014947
68296	SBFG	2020-10-12	15.74	15.74	15.584	15.584	1499	{}	\N	2020-10-12 16:03:09.019673
68297	RVRS	2020-09-28	15.4276	15.4276	15.4276	15.4276	104	{}	\N	2020-10-12 16:03:13.048692
68298	RVRS	2020-09-29	15.32	15.336	15.2096	15.295	14704	{}	\N	2020-10-12 16:03:13.058026
68299	RVRS	2020-09-30	15.3625	15.3625	15.3625	15.3625	253	{}	\N	2020-10-12 16:03:13.063704
68300	RVRS	2020-10-01	15.3602	15.375	15.275	15.2869	3011	{}	\N	2020-10-12 16:03:13.068974
68301	RVRS	2020-10-02	15.24	15.6	15.2116	15.5701	3571	{}	\N	2020-10-12 16:03:13.073666
68302	RVRS	2020-10-05	15.8566	15.8566	15.8566	15.8566	104	{}	\N	2020-10-12 16:03:13.078876
68303	RVRS	2020-10-06	15.9906	16.045	15.722	15.722	5848	{}	\N	2020-10-12 16:03:13.084479
68304	RVRS	2020-10-07	16.0133	16.0133	16.0133	16.0133	101	{}	\N	2020-10-12 16:03:13.090217
68305	RVRS	2020-10-08	16.2912	16.2912	16.2912	16.2912	116	{}	\N	2020-10-12 16:03:13.095619
68306	RVRS	2020-10-09	16.2627	16.2627	16.2627	16.2627	124	{}	\N	2020-10-12 16:03:13.10032
68307	RXI	2020-09-28	134.525	134.99	134.4194	134.85	13635	{}	\N	2020-10-12 16:03:14.864989
68308	RXI	2020-09-29	134.32	134.88	134.24	134.4303	5540	{}	\N	2020-10-12 16:03:14.875362
68309	RXI	2020-09-30	134.28	135.89	134.21	134.92	22338	{}	\N	2020-10-12 16:03:14.882443
68310	RXI	2020-10-01	135.92	136.3399	135.31	136.04	6872	{}	\N	2020-10-12 16:03:14.888218
68311	RXI	2020-10-02	135.03	135.99	135.03	135.86	1715	{}	\N	2020-10-12 16:03:14.894049
68312	RXI	2020-10-05	136.79	137.625	136.79	137.625	4245	{}	\N	2020-10-12 16:03:14.898849
68313	RXI	2020-10-06	138.04	138.5446	135.83	135.9	15434	{}	\N	2020-10-12 16:03:14.903187
68314	RXI	2020-10-07	137.47	138.86	137.47	138.77	4757	{}	\N	2020-10-12 16:03:14.90785
68315	RXI	2020-10-08	138.754	139.187	138.5691	139.05	23837	{}	\N	2020-10-12 16:03:14.912154
68316	RXI	2020-10-09	139.81	140.56	139.72	140.56	3119	{}	\N	2020-10-12 16:03:14.916054
68317	RXI	2020-10-12	140.97	141.72	140.83	141.61	14615	{}	\N	2020-10-12 16:03:14.920104
68318	RYT	2020-09-28	209.45	210.44	208.4	210.44	41790	{}	\N	2020-10-12 16:03:15.90082
68319	RYT	2020-09-29	210.59	211.8554	210.3701	210.76	40097	{}	\N	2020-10-12 16:03:15.912345
68320	RYT	2020-09-30	211	212.65	209.625	210.67	53090	{}	\N	2020-10-12 16:03:15.919985
68321	RYT	2020-10-01	213.71	213.71	211.9022	212.85	23713	{}	\N	2020-10-12 16:03:15.925883
68322	RYT	2020-10-02	208.6483	211.07	208.4	209.5	34653	{}	\N	2020-10-12 16:03:15.931416
68323	RYT	2020-10-05	211.08	213.78	211.08	213.78	74017	{}	\N	2020-10-12 16:03:15.936849
68324	RYT	2020-10-06	214.08	216.685	212.05	212.15	20786	{}	\N	2020-10-12 16:03:15.941078
68325	RYT	2020-10-07	213.93	216.44	213.93	216.24	18248	{}	\N	2020-10-12 16:03:15.94625
68326	RYT	2020-10-08	217.79	218.8299	217.6867	218.69	32867	{}	\N	2020-10-12 16:03:15.95127
68327	RYT	2020-10-09	220.68	221.9359	220.68	221.45	46607	{}	\N	2020-10-12 16:03:15.956259
68328	RYT	2020-10-12	222.93	223.8658	222.39	223.4213	10032	{}	\N	2020-10-12 16:03:15.961315
68329	SACH	2020-09-28	3.94	4	3.91	3.99	85855	{}	\N	2020-10-12 16:03:17.16038
68330	SACH	2020-09-29	4	4	3.9	3.98	25413	{}	\N	2020-10-12 16:03:17.168176
68331	SACH	2020-09-30	3.94	4.0201	3.89	3.97	48983	{}	\N	2020-10-12 16:03:17.172975
68332	SACH	2020-10-01	3.98	4	3.91	3.95	18446	{}	\N	2020-10-12 16:03:17.178192
68333	SACH	2020-10-02	3.71	4.0001	3.71	4	73034	{}	\N	2020-10-12 16:03:17.183506
68334	SACH	2020-10-05	4.01	4.04	3.95	3.95	51526	{}	\N	2020-10-12 16:03:17.188345
68335	SACH	2020-10-06	3.89	4.08	3.88	4.01	29540	{}	\N	2020-10-12 16:03:17.192773
68336	SACH	2020-10-07	4.01	4.069	3.96	4	32825	{}	\N	2020-10-12 16:03:17.197786
68337	SACH	2020-10-08	4.06	4.06	3.99	4.02	20577	{}	\N	2020-10-12 16:03:17.202294
68338	SACH	2020-10-09	4.02	4.04	3.9602	4.01	48779	{}	\N	2020-10-12 16:03:17.206492
68339	SACH	2020-10-12	4.02	4.05	4.0102	4.05	15374	{}	\N	2020-10-12 16:03:17.211154
68340	SAIC	2020-09-28	79	79.97	78.64	79.87	511870	{}	\N	2020-10-12 16:03:18.18332
68341	SAIC	2020-09-29	79.72	80.2675	78.08	78.6	1338389	{}	\N	2020-10-12 16:03:18.195065
68342	SAIC	2020-09-30	78.79	79.29	77.94	78.42	1091130	{}	\N	2020-10-12 16:03:18.202853
68343	SAIC	2020-10-01	78.78	79.27	78.1	78.89	1144167	{}	\N	2020-10-12 16:03:18.208851
68344	SAIC	2020-10-02	77.45	79.84	77.19	79.22	664693	{}	\N	2020-10-12 16:03:18.215111
68345	SAIC	2020-10-05	79.81	80.605	79.26	79.9	1034823	{}	\N	2020-10-12 16:03:18.220254
68346	SAIC	2020-10-06	80.41	82.63	79.98	80.2	697451	{}	\N	2020-10-12 16:03:18.225531
68347	SAIC	2020-10-07	80.7	81.3	79.79	79.87	677677	{}	\N	2020-10-12 16:03:18.230545
68348	SAIC	2020-10-08	81.01	81.69	80.095	80.25	662694	{}	\N	2020-10-12 16:03:18.235778
68349	SAIC	2020-10-09	80.98	81.45	80.06	80.1	805693	{}	\N	2020-10-12 16:03:18.240939
68350	SAIC	2020-10-12	80.77	82.515	80.285	82.27	216563	{}	\N	2020-10-12 16:03:18.246975
68351	SAN	2020-09-28	1.89	1.935	1.88	1.91	8882126	{}	\N	2020-10-12 16:03:19.68962
68352	SAN	2020-09-29	1.89	1.9	1.84	1.84	17605715	{}	\N	2020-10-12 16:03:19.699649
68353	SAN	2020-09-30	1.87	1.9	1.85	1.85	13503849	{}	\N	2020-10-12 16:03:19.706269
68354	SAN	2020-10-01	1.86	1.8765	1.83	1.85	23605064	{}	\N	2020-10-12 16:03:19.712119
68355	SAN	2020-10-02	1.81	1.88	1.81	1.87	13350249	{}	\N	2020-10-12 16:03:19.718188
68356	SAN	2020-10-05	1.88	1.92	1.88	1.9	14066028	{}	\N	2020-10-12 16:03:19.72329
68357	SAN	2020-10-06	2.03	2.035	1.96	1.96	23423446	{}	\N	2020-10-12 16:03:19.735417
68358	SAN	2020-10-07	1.99	2.01	1.98	1.98	9487745	{}	\N	2020-10-12 16:03:19.740707
68359	SAN	2020-10-08	2.02	2.03	2	2.02	8570537	{}	\N	2020-10-12 16:03:19.745789
68360	SAN	2020-10-09	2.04	2.04	1.99	1.99	14888192	{}	\N	2020-10-12 16:03:19.751252
68361	SAN	2020-10-12	1.98	1.98	1.96	1.97	3986729	{}	\N	2020-10-12 16:03:19.756302
68362	SATS	2020-09-28	25.12	25.7999	25.12	25.5	168549	{}	\N	2020-10-12 16:03:21.460134
68363	SATS	2020-09-29	25.58	25.74	25.1	25.24	165357	{}	\N	2020-10-12 16:03:21.47151
68364	SATS	2020-09-30	25.32	25.71	24.65	24.89	278030	{}	\N	2020-10-12 16:03:21.478257
68365	SATS	2020-10-01	24.79	25.12	24.51	24.75	293898	{}	\N	2020-10-12 16:03:21.484042
68366	SATS	2020-10-02	24.19	24.46	23.93	24.38	278063	{}	\N	2020-10-12 16:03:21.490021
68367	SATS	2020-10-05	24.59	24.88	24.35	24.84	365289	{}	\N	2020-10-12 16:03:21.494709
68368	SATS	2020-10-06	24.785	25.5	24.57	24.59	237942	{}	\N	2020-10-12 16:03:21.499642
68369	SATS	2020-10-07	24.77	25.06	24.24	24.91	218793	{}	\N	2020-10-12 16:03:21.504478
68370	SATS	2020-10-08	25.23	25.66	24.76	25.57	341386	{}	\N	2020-10-12 16:03:21.509352
68371	SATS	2020-10-09	26.4	27.41	26.04	27.1	360291	{}	\N	2020-10-12 16:03:21.513861
68372	SATS	2020-10-12	27.06	27.315	26.66	26.7176	87187	{}	\N	2020-10-12 16:03:21.518795
68373	SC	2020-09-28	16.91	17.57	16.91	17.22	469576	{}	\N	2020-10-12 16:03:24.725532
68374	SC	2020-09-29	17.23	17.71	16.975	17.32	1027006	{}	\N	2020-10-12 16:03:24.738781
68375	SC	2020-09-30	17.51	18.27	17.51	18.19	2139807	{}	\N	2020-10-12 16:03:24.745894
68376	SC	2020-10-01	18.25	18.6	17.81	18.59	1040044	{}	\N	2020-10-12 16:03:24.751784
68377	SC	2020-10-02	18.31	19.42	18.15	19.26	1303540	{}	\N	2020-10-12 16:03:24.757513
68378	SC	2020-10-05	19.64	19.91	19.42	19.81	1294353	{}	\N	2020-10-12 16:03:24.762306
68379	SC	2020-10-06	20.34	20.44	19.23	19.24	1274979	{}	\N	2020-10-12 16:03:24.76721
68380	SC	2020-10-07	19.49	19.91	19.22	19.38	1423976	{}	\N	2020-10-12 16:03:24.773154
68381	SC	2020-10-08	19.64	19.81	19.44	19.72	1397755	{}	\N	2020-10-12 16:03:24.778703
68382	SC	2020-10-09	19.93	20.14	19.585	19.94	3990733	{}	\N	2020-10-12 16:03:24.783926
68383	SC	2020-10-12	20.05	20.11	19.8	20.105	461578	{}	\N	2020-10-12 16:03:24.789103
68384	SNX	2020-09-28	129.58	132.1367	129.33	131.5	448775	{}	\N	2020-10-12 16:03:26.096911
68385	SNX	2020-09-29	132.87	134.46	131.57	132.36	1867945	{}	\N	2020-10-12 16:03:26.108697
68386	SNX	2020-09-30	141.57	144	138.62	140.06	2786688	{}	\N	2020-10-12 16:03:26.116435
68387	SNX	2020-10-01	141.64	144.95	139.01	144.28	1099910	{}	\N	2020-10-12 16:03:26.123325
68388	SNX	2020-10-02	141.37	148.74	140.9109	147.64	1406206	{}	\N	2020-10-12 16:03:26.129195
68389	SNX	2020-10-05	149.12	151.03	148.29	150.6	1049091	{}	\N	2020-10-12 16:03:26.134093
68390	SNX	2020-10-06	151.45	154.48	149.13	151.92	686329	{}	\N	2020-10-12 16:03:26.139232
68391	SNX	2020-10-07	153.98	155.785	153.6279	154.79	857073	{}	\N	2020-10-12 16:03:26.144422
68392	SNX	2020-10-08	155.93	156.36	152.06	152.61	643172	{}	\N	2020-10-12 16:03:26.149673
68393	SNX	2020-10-09	153.56	154	149.72	150.26	602951	{}	\N	2020-10-12 16:03:26.154534
68394	SNX	2020-10-12	151.38	151.49	149.08	149.94	127134	{}	\N	2020-10-12 16:03:26.159703
68395	RY	2020-09-28	71.44	71.81	71.32	71.45	730376	{}	\N	2020-10-12 16:03:27.134474
68396	RY	2020-09-29	71.2	71.31	69.875	70.12	1206427	{}	\N	2020-10-12 16:03:27.148306
68397	RY	2020-09-30	70.38	70.75	69.97	70.14	1517386	{}	\N	2020-10-12 16:03:27.154803
68398	RY	2020-10-01	70.58	70.58	69.64	70.41	935164	{}	\N	2020-10-12 16:03:27.161772
68399	RY	2020-10-02	69.6	71.07	69.48	70.76	1408106	{}	\N	2020-10-12 16:03:27.166528
68400	RY	2020-10-05	71.41	72.2	70.9961	72	849554	{}	\N	2020-10-12 16:03:27.171661
68401	RY	2020-10-06	72.46	72.97	71.79	71.87	910235	{}	\N	2020-10-12 16:03:27.176271
68402	RY	2020-10-07	73.04	73.77	72.81	73.57	3805132	{}	\N	2020-10-12 16:03:27.180636
68403	RY	2020-10-08	73.92	74.277	73.72	74.02	2112281	{}	\N	2020-10-12 16:03:27.185187
68404	RY	2020-10-09	74.4	74.51	73.74	74.16	1544681	{}	\N	2020-10-12 16:03:27.189795
68405	RY	2020-10-12	74.44	74.495	74.15	74.41	256270	{}	\N	2020-10-12 16:03:27.19451
68406	RYJ	2020-09-28	39.45	39.9235	39.45	39.9235	4222	{}	\N	2020-10-12 16:03:28.249115
68407	RYJ	2020-09-29	39.83	39.83	39.35	39.7631	2896	{}	\N	2020-10-12 16:03:28.261991
68408	RYJ	2020-09-30	40.13	40.15	39.6101	39.814	3600	{}	\N	2020-10-12 16:03:28.269381
68409	RYJ	2020-10-01	40.09	40.2241	39.93	40.12	6188	{}	\N	2020-10-12 16:03:28.275469
68410	RYJ	2020-10-02	39.64	40.646	39.54	40.646	3530	{}	\N	2020-10-12 16:03:28.281386
68411	RYJ	2020-10-05	41.35	41.6296	41.28	41.6296	304	{}	\N	2020-10-12 16:03:28.287345
68412	RYJ	2020-10-06	41.95	41.97	41.3629	41.3629	2018	{}	\N	2020-10-12 16:03:28.292879
68413	RYJ	2020-10-07	42.05	42.3581	41.99	42.3092	2562	{}	\N	2020-10-12 16:03:28.29831
68414	RYJ	2020-10-08	42.36	42.855	42.36	42.85	9501	{}	\N	2020-10-12 16:03:28.303206
68415	RYJ	2020-10-09	42.93	42.93	42.77	42.77	612	{}	\N	2020-10-12 16:03:28.308156
68416	RYJ	2020-10-12	43.02	43.08	43.02	43.08	1290	{}	\N	2020-10-12 16:03:28.313318
68417	SAA	2020-09-28	64.538	64.538	64.0838	64.0838	1913	{}	\N	2020-10-12 16:03:30.591444
68418	SAA	2020-09-29	63.4468	63.4468	63.4468	63.4468	105	{}	\N	2020-10-12 16:03:30.602404
68419	SAA	2020-09-30	63.8126	63.8126	63.8126	63.8126	236	{}	\N	2020-10-12 16:03:30.609771
68420	SAA	2020-10-01	64.2197	65.3065	64.2197	65.3065	1191	{}	\N	2020-10-12 16:03:30.616786
68421	SAA	2020-10-02	67.1033	67.1033	67.1033	67.1033	61	{}	\N	2020-10-12 16:03:30.622651
68422	SAA	2020-10-05	69.36	69.931	69.36	69.931	205	{}	\N	2020-10-12 16:03:30.627795
68423	SAA	2020-10-06	73.51	73.51	70.0123	70.0123	414	{}	\N	2020-10-12 16:03:30.632982
68424	SAA	2020-10-07	71.9716	71.9716	71.9716	71.9716	445	{}	\N	2020-10-12 16:03:30.637752
68425	SAA	2020-10-08	72.94	73.9403	72.94	73.9403	317	{}	\N	2020-10-12 16:03:30.642941
68426	SAA	2020-10-09	74.14	74.9539	74.14	74.88	1812	{}	\N	2020-10-12 16:03:30.648291
68427	SAA	2020-10-12	75.33	75.33	75.33	75.33	120	{}	\N	2020-10-12 16:03:30.65355
68428	SAGE	2020-09-28	61.16	61.87	59.805	59.89	370280	{}	\N	2020-10-12 16:03:32.14824
68429	SAGE	2020-09-29	59.91	61.34	59.15	59.67	265003	{}	\N	2020-10-12 16:03:32.159556
68430	SAGE	2020-09-30	59.21	61.32	59.21	61.12	422349	{}	\N	2020-10-12 16:03:32.166147
68431	SAGE	2020-10-01	60.03	61.85	58.57	59.53	714791	{}	\N	2020-10-12 16:03:32.17167
68432	SAGE	2020-10-02	58.77	60.01	57.71	58.41	306868	{}	\N	2020-10-12 16:03:32.17685
68433	SAGE	2020-10-05	58.25	60.39	58.25	60.15	663758	{}	\N	2020-10-12 16:03:32.182163
68434	SAGE	2020-10-06	60.84	66.29	60.51	64.12	1459511	{}	\N	2020-10-12 16:03:32.186716
68435	SAGE	2020-10-07	64.54	66.34	62.78	65.54	691439	{}	\N	2020-10-12 16:03:32.191277
68436	SAGE	2020-10-08	66.38	68.06	65.48	67.53	664528	{}	\N	2020-10-12 16:03:32.19603
68437	SAGE	2020-10-09	67.47	67.57	64.2	64.41	472964	{}	\N	2020-10-12 16:03:32.200608
68438	SAGE	2020-10-12	64.84	64.91	62.23	62.68	207794	{}	\N	2020-10-12 16:03:32.205004
68439	SAM	2020-09-28	880.39	910.255	880.39	899.91	143008	{}	\N	2020-10-12 16:03:33.578173
68440	SAM	2020-09-29	895.71	903.4299	876.405	880.42	170037	{}	\N	2020-10-12 16:03:33.584045
68441	SAM	2020-09-30	885.34	887.92	877.535	883.36	217313	{}	\N	2020-10-12 16:03:33.588271
68442	SAM	2020-10-01	883	884.91	868.23	873.42	185119	{}	\N	2020-10-12 16:03:33.593003
68443	SAM	2020-10-02	865.85	882.305	865.85	880.1	184056	{}	\N	2020-10-12 16:03:33.598197
68444	SAM	2020-10-05	880.74	902.61	875.28	882.67	188972	{}	\N	2020-10-12 16:03:33.603532
68445	SAM	2020-10-06	884.92	905.29	884	895.6	170347	{}	\N	2020-10-12 16:03:33.607978
68446	SAM	2020-10-07	895.2	926.0321	895.2	914	217087	{}	\N	2020-10-12 16:03:33.612915
68447	SAM	2020-10-08	922.6	923.3244	909.63	920.01	142703	{}	\N	2020-10-12 16:03:33.617906
68448	SAM	2020-10-09	926.01	929.655	913.45	918.19	133418	{}	\N	2020-10-12 16:03:33.623028
68449	SAM	2020-10-12	918	928.99	918	928.99	28245	{}	\N	2020-10-12 16:03:33.628064
68450	SANM	2020-09-28	26.06	27.26	26.06	27.13	568423	{}	\N	2020-10-12 16:03:34.787274
68451	SANM	2020-09-29	26.6201	27.51	26.6201	27.16	544760	{}	\N	2020-10-12 16:03:34.797384
68452	SANM	2020-09-30	27.28	27.48	26.82	27.05	426211	{}	\N	2020-10-12 16:03:34.80421
68453	SANM	2020-10-01	27.13	27.51	26.7	26.94	221954	{}	\N	2020-10-12 16:03:34.809473
68454	SANM	2020-10-02	26.28	26.86	26.28	26.55	248464	{}	\N	2020-10-12 16:03:34.814482
68455	SANM	2020-10-05	26.85	27.33	26.79	27.12	243681	{}	\N	2020-10-12 16:03:34.819542
68456	SANM	2020-10-06	27.35	27.76	26.84	26.87	392825	{}	\N	2020-10-12 16:03:34.824013
68457	SANM	2020-10-07	27.13	27.45	26.92	27.33	344094	{}	\N	2020-10-12 16:03:34.828282
68458	SANM	2020-10-08	27.73	27.88	27.31	27.74	181461	{}	\N	2020-10-12 16:03:34.832387
68459	SANM	2020-10-09	28.04	28.23	27.6455	27.77	174805	{}	\N	2020-10-12 16:03:34.836991
68460	SANM	2020-10-12	27.95	28.08	27.765	28.01	46703	{}	\N	2020-10-12 16:03:34.841758
68461	SBGI	2020-09-28	20.29	20.7	20.18	20.28	520534	{}	\N	2020-10-12 16:03:36.733195
68462	SBGI	2020-09-29	20.25	20.49	19.425	20.07	496226	{}	\N	2020-10-12 16:03:36.753473
68463	SBGI	2020-09-30	19.78	20.08	19.16	19.23	1163617	{}	\N	2020-10-12 16:03:36.760045
68464	SBGI	2020-10-01	19.21	19.3799	18.06	18.52	1377840	{}	\N	2020-10-12 16:03:36.765674
68465	SBGI	2020-10-02	18	18.97	17.981	18.7	1095289	{}	\N	2020-10-12 16:03:36.77108
68466	SBGI	2020-10-05	18.97	19.255	18.38	18.69	839754	{}	\N	2020-10-12 16:03:36.775732
68467	SBGI	2020-10-06	18.96	19.3	18.5	18.52	838569	{}	\N	2020-10-12 16:03:36.781121
68468	SBGI	2020-10-07	18.71	18.965	18.52	18.68	563857	{}	\N	2020-10-12 16:03:36.785962
68469	SBGI	2020-10-08	18.88	19.44	18.8	19.35	6690763	{}	\N	2020-10-12 16:03:36.791335
68470	SBGI	2020-10-09	19.65	19.85	18.9	18.96	752451	{}	\N	2020-10-12 16:03:36.796263
68471	SBGI	2020-10-12	18.8	19.13	18.5	19.02	175940	{}	\N	2020-10-12 16:03:36.80142
68472	SBRA	2020-09-28	14.19	14.43	14.14	14.205	1591464	{}	\N	2020-10-12 16:03:38.537959
68473	SBRA	2020-09-29	14.21	14.21	13.57	13.92	948108	{}	\N	2020-10-12 16:03:38.551818
68474	SBRA	2020-09-30	13.99	14.405	13.5918	13.785	1607628	{}	\N	2020-10-12 16:03:38.559019
68475	SBRA	2020-10-01	13.87	14.5	13.79	14.41	1854092	{}	\N	2020-10-12 16:03:38.565181
68476	SBRA	2020-10-02	14.15	14.9	14	14.79	1868391	{}	\N	2020-10-12 16:03:38.570988
68477	SBRA	2020-10-05	14.97	15.08	14.4	14.69	1057087	{}	\N	2020-10-12 16:03:38.576376
68478	SBRA	2020-10-06	14.83	15.095	14.495	14.67	1363695	{}	\N	2020-10-12 16:03:38.581448
68479	SBRA	2020-10-07	14.76	14.99	14.585	14.62	1064784	{}	\N	2020-10-12 16:03:38.586728
68480	SBRA	2020-10-08	14.74	15.05	14.7	14.9	1345309	{}	\N	2020-10-12 16:03:38.591613
68481	SBRA	2020-10-09	15.06	15.17	14.5	14.54	797321	{}	\N	2020-10-12 16:03:38.59639
68482	SBRA	2020-10-12	14.51	14.795	14.38	14.72	325315	{}	\N	2020-10-12 16:03:38.601583
68483	SCHG	2020-09-28	114.51	114.51	113.4969	114.4	586891	{}	\N	2020-10-12 16:03:41.280222
68484	SCHG	2020-09-29	114.39	114.889	114.01	114.19	348369	{}	\N	2020-10-12 16:03:41.289697
68485	SCHG	2020-09-30	114.4	116.35	114.16	115.17	600050	{}	\N	2020-10-12 16:03:41.296313
68486	SCHG	2020-10-01	116.6248	116.965	115.995	116.8	504830	{}	\N	2020-10-12 16:03:41.301616
68487	SCHG	2020-10-02	114.3045	116.0001	113.86	114.35	317736	{}	\N	2020-10-12 16:03:41.306723
68488	SCHG	2020-10-05	115.3415	116.75	115.18	116.72	270591	{}	\N	2020-10-12 16:03:41.311547
68489	SCHG	2020-10-06	116.55	117.1095	114.282	114.66	573302	{}	\N	2020-10-12 16:03:41.315815
68490	SCHG	2020-10-07	115.82	117	115.765	116.8057	936902	{}	\N	2020-10-12 16:03:41.32023
68491	SCHG	2020-10-08	117.7	117.7	116.921	117.42	296047	{}	\N	2020-10-12 16:03:41.324592
68492	SCHG	2020-10-09	118.12	119.33	117.892	119.33	332255	{}	\N	2020-10-12 16:03:41.329253
68493	SCHG	2020-10-12	120.83	121.95	120.5	121.9	127027	{}	\N	2020-10-12 16:03:41.333732
68494	RYB	2020-09-28	2.98	3.09	2.92	2.94	10930	{}	\N	2020-10-12 16:03:42.306263
68495	RYB	2020-09-29	2.91	3.09	2.91	3.04	82136	{}	\N	2020-10-12 16:03:42.317649
68496	RYB	2020-09-30	3.1	3.1	2.99	3.04	12660	{}	\N	2020-10-12 16:03:42.324809
68497	RYB	2020-10-01	3.01	3.05	2.98	3	11826	{}	\N	2020-10-12 16:03:42.331195
68498	RYB	2020-10-02	2.86	2.94	2.79	2.92	13292	{}	\N	2020-10-12 16:03:42.336906
68499	RYB	2020-10-05	2.81	3.07	2.81	3.07	19606	{}	\N	2020-10-12 16:03:42.342218
68500	RYB	2020-10-06	2.99	3.0292	2.98	2.98	8126	{}	\N	2020-10-12 16:03:42.347371
68501	RYB	2020-10-07	3.0599	3.0599	2.93	2.93	15392	{}	\N	2020-10-12 16:03:42.3525
68502	RYB	2020-10-08	2.8973	2.9	2.85	2.9	54588	{}	\N	2020-10-12 16:03:42.357511
68503	RYB	2020-10-09	2.91	2.95	2.8801	2.8901	8004	{}	\N	2020-10-12 16:03:42.362523
68504	RYB	2020-10-12	2.8901	2.91	2.85	2.8501	20008	{}	\N	2020-10-12 16:03:42.366972
68505	RZB	2020-09-28	28.09	28.28	27.69	27.99	28540	{}	\N	2020-10-12 16:03:43.328676
68506	RZB	2020-09-29	28.15	28.15	27.4815	27.83	40941	{}	\N	2020-10-12 16:03:43.340601
68507	RZB	2020-09-30	27.84	27.93	27.1	27.4	137916	{}	\N	2020-10-12 16:03:43.347991
68508	RZB	2020-10-01	27.45	28	27.45	27.97	53338	{}	\N	2020-10-12 16:03:43.354299
68509	RZB	2020-10-02	27.71	27.99	27.71	27.94	28430	{}	\N	2020-10-12 16:03:43.36038
68510	RZB	2020-10-05	27.94	28.1	27.8441	27.96	48756	{}	\N	2020-10-12 16:03:43.365711
68511	RZB	2020-10-06	28.19	28.29	28.06	28.23	57686	{}	\N	2020-10-12 16:03:43.371157
68512	RZB	2020-10-07	28.29	28.5	27.46	28.31	73178	{}	\N	2020-10-12 16:03:43.37608
68513	RZB	2020-10-08	28.45	28.7399	28.25	28.39	24160	{}	\N	2020-10-12 16:03:43.381299
68514	RZB	2020-10-09	28.4	28.67	28.2091	28.5	23302	{}	\N	2020-10-12 16:03:43.385889
68515	RZB	2020-10-12	28.5	28.5	27.932	27.98	10688	{}	\N	2020-10-12 16:03:43.390563
68516	SALT	2020-09-28	13.2	13.6	12.75	13.37	209166	{}	\N	2020-10-12 16:03:45.909681
68517	SALT	2020-09-29	13.58	14.21	13.27	13.76	383193	{}	\N	2020-10-12 16:03:45.920523
68518	SALT	2020-09-30	13.95	14.35	13.85	14.16	298923	{}	\N	2020-10-12 16:03:45.927813
68519	SALT	2020-10-01	14.38	14.91	13.96	14.88	273558	{}	\N	2020-10-12 16:03:45.93376
68520	SALT	2020-10-02	14.47	15.53	14.2002	15.41	489902	{}	\N	2020-10-12 16:03:45.939352
68521	SALT	2020-10-05	15.7	16.3	15.63	15.79	318516	{}	\N	2020-10-12 16:03:45.944599
68522	SALT	2020-10-06	16.11	16.11	14.99	15.04	248745	{}	\N	2020-10-12 16:03:45.949595
68523	SALT	2020-10-07	15.09	15.38	14.56	14.81	222947	{}	\N	2020-10-12 16:03:45.955179
68524	SALT	2020-10-08	14.91	15.9	14.91	15.67	211883	{}	\N	2020-10-12 16:03:45.962375
68525	SALT	2020-10-09	15.89	15.89	15.0659	15.54	163437	{}	\N	2020-10-12 16:03:45.967282
68526	SALT	2020-10-12	15.47	15.47	14.22	14.22	130105	{}	\N	2020-10-12 16:03:45.971993
68527	SAVE	2020-09-28	16.68	17.045	16.28	16.53	7204022	{}	\N	2020-10-12 16:03:47.031327
68528	SAVE	2020-09-29	16.4	16.42	15.86	16.02	14421422	{}	\N	2020-10-12 16:03:47.042827
68529	SAVE	2020-09-30	16.18	16.84	16.06	16.1	15827834	{}	\N	2020-10-12 16:03:47.049638
68530	SAVE	2020-10-01	16.3	16.46	16.05	16.38	10106234	{}	\N	2020-10-12 16:03:47.055161
68531	SAVE	2020-10-02	15.5	16.82	15.46	16.71	27802950	{}	\N	2020-10-12 16:03:47.060214
68532	SAVE	2020-10-05	16.66	16.83	16.23	16.38	16206665	{}	\N	2020-10-12 16:03:47.065371
68533	SAVE	2020-10-06	16.6	16.82	15.75	15.75	22119101	{}	\N	2020-10-12 16:03:47.069521
68534	SAVE	2020-10-07	16.5	16.61	16.05	16.55	21121389	{}	\N	2020-10-12 16:03:47.073783
68535	SAVE	2020-10-08	17	17.35	16.58	17.04	21258684	{}	\N	2020-10-12 16:03:47.078099
68536	SAVE	2020-10-09	17	17.35	16.7	17.27	15497523	{}	\N	2020-10-12 16:03:47.083047
68537	SAVE	2020-10-12	17.03	17.16	16.77	16.948	5910978	{}	\N	2020-10-12 16:03:47.08758
68538	SBT	2020-09-28	3.08	3.305	3.08	3.15	61629	{}	\N	2020-10-12 16:03:48.671362
68539	SBT	2020-09-29	3.15	3.15	3.01	3.05	23117	{}	\N	2020-10-12 16:03:48.681505
68540	SBT	2020-09-30	3.06	3.09	2.98	3.01	55629	{}	\N	2020-10-12 16:03:48.687879
68541	SBT	2020-10-01	3.03	3.07	2.9972	3.03	22225	{}	\N	2020-10-12 16:03:48.693993
68542	SBT	2020-10-02	2.99	3.15	2.99	3.14	37365	{}	\N	2020-10-12 16:03:48.699511
68543	SBT	2020-10-05	3.2	3.22	3.11	3.18	45995	{}	\N	2020-10-12 16:03:48.70482
68544	SBT	2020-10-06	3.18	3.28	3.05	3.13	59196	{}	\N	2020-10-12 16:03:48.709699
68545	SBT	2020-10-07	3.28	3.5	3.22	3.46	132574	{}	\N	2020-10-12 16:03:48.714269
68546	SBT	2020-10-08	3.5	3.61	3.34	3.59	61345	{}	\N	2020-10-12 16:03:48.719295
68547	SBT	2020-10-09	3.64	3.71	3.5305	3.54	29184	{}	\N	2020-10-12 16:03:48.724355
68548	SBT	2020-10-12	3.38	3.565	3.38	3.55	7337	{}	\N	2020-10-12 16:03:48.730032
68549	SCD	2020-09-28	10.11	10.25	10.11	10.11	58749	{}	\N	2020-10-12 16:03:49.99253
68550	SCD	2020-09-29	10.16	10.176	10.08	10.12	172014	{}	\N	2020-10-12 16:03:50.002452
68551	SCD	2020-09-30	10.09	10.32	10.09	10.2	341399	{}	\N	2020-10-12 16:03:50.009069
68552	SCD	2020-10-01	10.31	10.38	10.13	10.19	401860	{}	\N	2020-10-12 16:03:50.014233
68553	SCD	2020-10-02	10.09	10.36	10.06	10.36	167660	{}	\N	2020-10-12 16:03:50.019752
68554	SCD	2020-10-05	10.38	10.48	10.36	10.46	72882	{}	\N	2020-10-12 16:03:50.026705
68555	SCD	2020-10-06	10.46	10.53	10.31	10.33	147440	{}	\N	2020-10-12 16:03:50.031084
68556	SCD	2020-10-07	10.38	10.51	10.2901	10.51	100284	{}	\N	2020-10-12 16:03:50.035821
68557	SCD	2020-10-08	10.57	10.61	10.5	10.61	86147	{}	\N	2020-10-12 16:03:50.040566
68558	SCD	2020-10-09	10.65	10.659	10.5	10.61	145795	{}	\N	2020-10-12 16:03:50.045875
68559	SCD	2020-10-12	10.61	10.69	10.5978	10.655	24820	{}	\N	2020-10-12 16:03:50.050931
68560	SCHK	2020-09-28	32.8	32.979	32.74	32.9	90716	{}	\N	2020-10-12 16:03:51.258743
68561	SCHK	2020-09-29	32.93	32.98	32.709	32.74	113930	{}	\N	2020-10-12 16:03:51.270896
68562	SCHK	2020-09-30	32.84	33.315	32.8202	32.98	131306	{}	\N	2020-10-12 16:03:51.281968
68563	SCHK	2020-10-01	33.27	33.36	33.0566	33.24	244054	{}	\N	2020-10-12 16:03:51.287771
68564	SCHK	2020-10-02	32.67	33.14	32.6513	32.94	163539	{}	\N	2020-10-12 16:03:51.29351
68565	SCHK	2020-10-05	33.3	33.56	33.18	33.56	76316	{}	\N	2020-10-12 16:03:51.298334
68566	SCHK	2020-10-06	33.59	33.8	33.06	33.16	238486	{}	\N	2020-10-12 16:03:51.303168
68567	SCHK	2020-10-07	33.47	33.7636	33.4635	33.68	1085440	{}	\N	2020-10-12 16:03:51.308113
68568	SCHK	2020-10-08	33.93	33.99	33.81	33.97	117962	{}	\N	2020-10-12 16:03:51.312709
68569	SCHK	2020-10-09	34.14	34.33	34.1	34.27	242103	{}	\N	2020-10-12 16:03:51.316803
68570	SCHK	2020-10-12	34.56	34.76	34.5	34.74	34804	{}	\N	2020-10-12 16:03:51.321364
68571	SCS	2020-09-28	10	10.14	9.79	9.9	1395798	{}	\N	2020-10-12 16:03:53.538343
68572	SCS	2020-09-29	9.88	10.1	9.72	10.04	2050651	{}	\N	2020-10-12 16:03:53.550726
68573	SCS	2020-09-30	10.02	10.3438	10.02	10.11	2693909	{}	\N	2020-10-12 16:03:53.557775
68574	SCS	2020-10-01	10.47	11.09	10.43	10.68	2566021	{}	\N	2020-10-12 16:03:53.564047
68575	SCS	2020-10-02	10.3	11.56	10.19	11.49	2469695	{}	\N	2020-10-12 16:03:53.569921
68576	SCS	2020-10-05	11.59	11.61	11.195	11.35	1745035	{}	\N	2020-10-12 16:03:53.57553
68577	SCS	2020-10-06	11.53	11.72	11.34	11.41	1687298	{}	\N	2020-10-12 16:03:53.580313
68578	SCS	2020-10-07	11.68	12.035	11.38	11.41	1622343	{}	\N	2020-10-12 16:03:53.585213
68579	SCS	2020-10-08	11.55	11.58	11.29	11.44	1092732	{}	\N	2020-10-12 16:03:53.589903
68580	SCS	2020-10-09	11.55	11.65	11.31	11.48	1005556	{}	\N	2020-10-12 16:03:53.594612
68581	SCS	2020-10-12	11.53	11.85	11.49	11.76	395421	{}	\N	2020-10-12 16:03:53.599337
68582	SCYX	2020-09-28	4.59	4.71	4.34	4.49	70551	{}	\N	2020-10-12 16:03:55.260608
68583	SCYX	2020-09-29	4.5	4.61	4.35	4.37	94532	{}	\N	2020-10-12 16:03:55.270561
68584	SCYX	2020-09-30	4.36	4.54	4.35	4.41	60718	{}	\N	2020-10-12 16:03:55.278303
68585	SCYX	2020-10-01	4.5	4.5	4.2009	4.28	103423	{}	\N	2020-10-12 16:03:55.284971
68586	SCYX	2020-10-02	4.28	4.72	4.23	4.72	96826	{}	\N	2020-10-12 16:03:55.29069
68587	SCYX	2020-10-05	4.72	4.8852	4.5274	4.68	45835	{}	\N	2020-10-12 16:03:55.296205
68588	SCYX	2020-10-06	4.7	4.75	4.51	4.51	54052	{}	\N	2020-10-12 16:03:55.301851
68589	SCYX	2020-10-07	4.64	4.663	4.51	4.59	33676	{}	\N	2020-10-12 16:03:55.30696
68590	SCYX	2020-10-08	4.6	6.3	4.6	6.15	356149	{}	\N	2020-10-12 16:03:55.311774
68591	SCYX	2020-10-09	6.01	6.07	5.4316	5.54	205167	{}	\N	2020-10-12 16:03:55.316981
68592	SCYX	2020-10-12	5.6	5.6	5.275	5.33	36867	{}	\N	2020-10-12 16:03:55.32163
68593	SDOG	2020-09-28	37.67	38.22	37.67	38	251930	{}	\N	2020-10-12 16:03:56.404062
68594	SDOG	2020-09-29	37.9848	38.03	37.47	37.6401	65692	{}	\N	2020-10-12 16:03:56.419448
68595	SDOG	2020-09-30	37.77	38.21	37.67	37.81	76951	{}	\N	2020-10-12 16:03:56.428603
68596	SDOG	2020-10-01	37.86	38.075	37.43	37.61	59552	{}	\N	2020-10-12 16:03:56.435478
68597	SDOG	2020-10-02	37.05	38.225	37.05	38.09	53933	{}	\N	2020-10-12 16:03:56.441862
68598	SDOG	2020-10-05	38.565	38.885	38.5017	38.82	54038	{}	\N	2020-10-12 16:03:56.447621
68599	SDOG	2020-10-06	38.95	39.42	38.5	38.57	78291	{}	\N	2020-10-12 16:03:56.4533
68600	SDOG	2020-10-07	38.82	39.46	38.82	39.395	67975	{}	\N	2020-10-12 16:03:56.458537
68601	SDOG	2020-10-08	39.56	40.06	39.56	40.06	63596	{}	\N	2020-10-12 16:03:56.464186
68602	SDOG	2020-10-09	40.29	40.35	39.9501	39.99	68860	{}	\N	2020-10-12 16:03:56.469785
68603	SDOG	2020-10-12	40.04	40.22	40.04	40.17	20136	{}	\N	2020-10-12 16:03:56.47598
68604	RYH	2020-09-28	231.57	234.04	231.57	232.53	6465	{}	\N	2020-10-12 16:03:57.958954
68605	RYH	2020-09-29	232.87	233.009	231.54	232.074	4981	{}	\N	2020-10-12 16:03:57.97074
68606	RYH	2020-09-30	232.95	236.55	232.95	235.5	4549	{}	\N	2020-10-12 16:03:57.978433
68607	RYH	2020-10-01	236.695	236.74	233.68	233.8168	6697	{}	\N	2020-10-12 16:03:57.984595
68608	RYH	2020-10-02	232.14	233.23	230.93	232.6801	4651	{}	\N	2020-10-12 16:03:57.990919
68609	RYH	2020-10-05	235.54	238.26	235.54	238.26	3669	{}	\N	2020-10-12 16:03:57.996438
68610	RYH	2020-10-06	239	239.4699	235.62	235.62	7832	{}	\N	2020-10-12 16:03:58.001777
68611	RYH	2020-10-07	238.3114	240.699	238.3114	240.6075	7966	{}	\N	2020-10-12 16:03:58.007104
68612	RYH	2020-10-08	241.65	242.135	241.3089	242.135	6206	{}	\N	2020-10-12 16:03:58.012495
68613	RYH	2020-10-09	243.48	244.295	243.3001	243.97	7805	{}	\N	2020-10-12 16:03:58.017301
68614	RYH	2020-10-12	245.3	246.219	245.3	245.9	3701	{}	\N	2020-10-12 16:03:58.022665
68615	SAND	2020-09-28	8.48	8.485	8.2	8.37	1217222	{}	\N	2020-10-12 16:03:59.860164
68616	SAND	2020-09-29	8.43	8.5691	8.37	8.47	2283610	{}	\N	2020-10-12 16:03:59.873443
68617	SAND	2020-09-30	8.39	8.55	8.29	8.44	1992114	{}	\N	2020-10-12 16:03:59.881205
68618	SAND	2020-10-01	8.55	8.661	8.45	8.54	1640805	{}	\N	2020-10-12 16:03:59.887599
68619	SAND	2020-10-02	8.45	8.52	8.335	8.45	1843297	{}	\N	2020-10-12 16:03:59.893261
68620	SAND	2020-10-05	8.48	8.745	8.48	8.57	1776539	{}	\N	2020-10-12 16:03:59.898407
68621	SAND	2020-10-06	8.63	8.63	8.21	8.21	2763534	{}	\N	2020-10-12 16:03:59.903658
68622	SAND	2020-10-07	8.34	8.3673	8.16	8.2	1949154	{}	\N	2020-10-12 16:03:59.908689
68623	SAND	2020-10-08	8.3	8.3744	8.195	8.27	2403374	{}	\N	2020-10-12 16:03:59.914338
68624	SAND	2020-10-09	8.46	8.75	8.43	8.73	3624031	{}	\N	2020-10-12 16:03:59.919391
68625	SAND	2020-10-12	8.74	8.74	8.59	8.65	764797	{}	\N	2020-10-12 16:03:59.925096
68626	SBI	2020-09-28	9.1	9.38	9.0646	9.23	21070	{}	\N	2020-10-12 16:04:02.188845
68627	SBI	2020-09-29	9.24	9.24	8.895	9.01	25628	{}	\N	2020-10-12 16:04:02.197705
68628	SBI	2020-09-30	8.95	9.09	8.95	9.06	10155	{}	\N	2020-10-12 16:04:02.204482
68629	SBI	2020-10-01	9.07	9.12	8.98	9.02	16892	{}	\N	2020-10-12 16:04:02.20998
68630	SBI	2020-10-02	9.09	9.09	8.94	8.9506	12187	{}	\N	2020-10-12 16:04:02.215646
68631	SBI	2020-10-05	8.96	8.995	8.94	8.9617	10868	{}	\N	2020-10-12 16:04:02.220839
68632	SBI	2020-10-06	8.92	8.9823	8.92	8.94	12095	{}	\N	2020-10-12 16:04:02.226305
68633	SBI	2020-10-07	8.95	8.97	8.91	8.97	12694	{}	\N	2020-10-12 16:04:02.231167
68634	SBI	2020-10-08	8.98	9.12	8.93	9.12	52488	{}	\N	2020-10-12 16:04:02.236273
68635	SBI	2020-10-09	9.08	9.13	9.06	9.13	13904	{}	\N	2020-10-12 16:04:02.241548
68636	SBI	2020-10-12	9.14	9.14	9.13	9.13	20202	{}	\N	2020-10-12 16:04:02.246272
68637	SBSI	2020-09-28	24.26	24.98	24.26	24.74	101642	{}	\N	2020-10-12 16:04:03.251151
68638	SBSI	2020-09-29	24.55	24.56	24.02	24.53	66572	{}	\N	2020-10-12 16:04:03.26301
68639	SBSI	2020-09-30	24.55	24.96	24.26	24.43	115615	{}	\N	2020-10-12 16:04:03.270664
68640	SBSI	2020-10-01	24.38	24.5499	23.96	24.48	72957	{}	\N	2020-10-12 16:04:03.276868
68641	SBSI	2020-10-02	24.06	25.08	23.9	24.99	64880	{}	\N	2020-10-12 16:04:03.283083
68642	SBSI	2020-10-05	25.07	25.72	25.04	25.63	86607	{}	\N	2020-10-12 16:04:03.288295
68643	SBSI	2020-10-06	25.95	26.49	25.69	25.69	116725	{}	\N	2020-10-12 16:04:03.293393
68644	SBSI	2020-10-07	25.83	26.25	25.6201	25.95	135234	{}	\N	2020-10-12 16:04:03.298573
68645	SBSI	2020-10-08	26.23	26.52	26	26.23	83162	{}	\N	2020-10-12 16:04:03.304515
68646	SBSI	2020-10-09	26.78	26.78	26.06	26.28	69253	{}	\N	2020-10-12 16:04:03.310226
68647	SBSI	2020-10-12	26.16	26.505	26.16	26.36	17432	{}	\N	2020-10-12 16:04:03.315689
68648	SCC	2020-09-28	15.0845	15.0845	15.0845	15.0845	60	{}	\N	2020-10-12 16:04:04.569165
68649	SCC	2020-09-29	14.66	15.3	14.66	15.215	5022	{}	\N	2020-10-12 16:04:04.57917
68650	SCC	2020-09-30	14.87	15.0245	14.87	15.0245	1137	{}	\N	2020-10-12 16:04:04.58537
68651	SCC	2020-10-01	14.577	14.577	14.577	14.577	401	{}	\N	2020-10-12 16:04:04.591249
68652	SCC	2020-10-02	14.9451	14.9451	14.9451	14.9451	998	{}	\N	2020-10-12 16:04:04.596778
68653	SCC	2020-10-05	14.74	14.74	14.4851	14.4851	1065	{}	\N	2020-10-12 16:04:04.602954
68654	SCC	2020-10-06	14.52	15.086	14.52	15.086	2035	{}	\N	2020-10-12 16:04:04.609404
68655	SCC	2020-10-07	14.4288	14.4288	14.4288	14.4288	34	{}	\N	2020-10-12 16:04:04.615308
68656	SCC	2020-10-08	14.27	14.27	14.2588	14.2588	1168	{}	\N	2020-10-12 16:04:04.620804
68657	SCC	2020-10-09	14.01	14.01	13.9483	13.9483	423	{}	\N	2020-10-12 16:04:04.625884
68658	SCHF	2020-09-28	31.52	31.59	31.465	31.59	2137492	{}	\N	2020-10-12 16:04:06.004395
68659	SCHF	2020-09-29	31.551	31.65	31.41	31.53	1858167	{}	\N	2020-10-12 16:04:06.01652
68660	SCHF	2020-09-30	31.45	31.62	31.305	31.4	3292187	{}	\N	2020-10-12 16:04:06.023668
68661	SCHF	2020-10-01	31.56	31.6	31.4014	31.57	3888249	{}	\N	2020-10-12 16:04:06.03
68662	SCHF	2020-10-02	31.19	31.595	31.175	31.53	2036627	{}	\N	2020-10-12 16:04:06.036105
68663	SCHF	2020-10-05	31.78	32.01	31.78	32	1380131	{}	\N	2020-10-12 16:04:06.041535
68664	SCHF	2020-10-06	32.03	32.0599	31.605	31.68	2291636	{}	\N	2020-10-12 16:04:06.046861
68665	SCHF	2020-10-07	31.91	32.02	31.86	31.99	3276131	{}	\N	2020-10-12 16:04:06.052157
68666	SCHF	2020-10-08	32.12	32.21	32.09	32.21	1948736	{}	\N	2020-10-12 16:04:06.057578
68667	SCHF	2020-10-09	32.35	32.44	32.3	32.39	1752576	{}	\N	2020-10-12 16:04:06.062835
68668	SCHF	2020-10-12	32.46	32.55	32.44	32.505	442753	{}	\N	2020-10-12 16:04:06.067771
68669	SCHP	2020-09-28	61.62	61.6959	61.58	61.67	1188072	{}	\N	2020-10-12 16:04:08.256541
68670	SCHP	2020-09-29	61.735	61.815	61.735	61.78	731592	{}	\N	2020-10-12 16:04:08.264306
68671	SCHP	2020-09-30	61.78	61.79	61.6	61.63	790700	{}	\N	2020-10-12 16:04:08.269211
68672	SCHP	2020-10-01	61.35	61.51	61.34	61.45	832650	{}	\N	2020-10-12 16:04:08.274649
68673	SCHP	2020-10-02	61.4328	61.4558	61.3743	61.44	488498	{}	\N	2020-10-12 16:04:08.281944
68674	SCHP	2020-10-05	61.42	61.42	61.25	61.27	619608	{}	\N	2020-10-12 16:04:08.288345
68675	SCHP	2020-10-06	61.28	61.36	61.17	61.29	1222786	{}	\N	2020-10-12 16:04:08.293391
68676	SCHP	2020-10-07	61.27	61.3	61.21	61.26	956924	{}	\N	2020-10-12 16:04:08.297863
68677	SCHP	2020-10-08	61.36	61.49	61.33	61.49	650520	{}	\N	2020-10-12 16:04:08.302954
68678	SCHP	2020-10-09	61.47	61.515	61.4	61.51	659480	{}	\N	2020-10-12 16:04:08.308419
68679	SCHP	2020-10-12	61.5	61.52	61.47	61.485	147728	{}	\N	2020-10-12 16:04:08.313602
68680	SCSC	2020-09-28	19.17	20.33	19.17	19.98	107753	{}	\N	2020-10-12 16:04:09.855434
68681	SCSC	2020-09-29	19.87	20.19	19.6	20.08	100115	{}	\N	2020-10-12 16:04:09.868288
68682	SCSC	2020-09-30	20.23	20.6	19.75	19.83	115365	{}	\N	2020-10-12 16:04:09.875978
68683	SCSC	2020-10-01	19.91	20.31	19.76	20.15	95100	{}	\N	2020-10-12 16:04:09.88367
68684	SCSC	2020-10-02	19.64	20.26	19.5	20.1	73812	{}	\N	2020-10-12 16:04:09.889868
68685	SCSC	2020-10-05	20.11	21	20.11	20.92	97863	{}	\N	2020-10-12 16:04:09.895205
68686	SCSC	2020-10-06	21.13	21.82	20.84	20.92	157490	{}	\N	2020-10-12 16:04:09.900434
68687	SCSC	2020-10-07	21.16	21.85	19.04	21.35	113912	{}	\N	2020-10-12 16:04:09.90551
68688	SCSC	2020-10-08	21.72	21.75	21.23	21.56	54897	{}	\N	2020-10-12 16:04:09.910509
68689	SCSC	2020-10-09	21.88	21.88	21.2	21.36	81440	{}	\N	2020-10-12 16:04:09.91563
68690	SCSC	2020-10-12	21.41	21.75	21.14	21.55	14490	{}	\N	2020-10-12 16:04:09.920479
68691	SD	2020-09-28	1.7242	1.75	1.64	1.64	125839	{}	\N	2020-10-12 16:04:11.765499
68692	SD	2020-09-29	1.66	1.73	1.61	1.67	346053	{}	\N	2020-10-12 16:04:11.77292
68693	SD	2020-09-30	1.67	1.72	1.62	1.65	309474	{}	\N	2020-10-12 16:04:11.778616
68694	SD	2020-10-01	1.64	1.64	1.55	1.59	296304	{}	\N	2020-10-12 16:04:11.783904
68695	SD	2020-10-02	1.55	1.63	1.5479	1.61	210734	{}	\N	2020-10-12 16:04:11.789491
68696	SD	2020-10-05	1.61	1.79	1.61	1.76	568454	{}	\N	2020-10-12 16:04:11.795211
68697	SD	2020-10-06	1.76	1.78	1.7	1.75	269962	{}	\N	2020-10-12 16:04:11.800099
68698	SD	2020-10-07	1.77	1.79	1.69	1.76	219376	{}	\N	2020-10-12 16:04:11.805032
68699	SD	2020-10-08	1.8	2.08	1.761	2	2910718	{}	\N	2020-10-12 16:04:11.809041
68700	SD	2020-10-09	2.11	2.12	1.88	1.9	4578197	{}	\N	2020-10-12 16:04:11.813303
68701	SD	2020-10-12	1.96	1.97	1.88	1.89	168194	{}	\N	2020-10-12 16:04:11.818022
68702	RYI	2020-09-28	5.79	5.87	5.63	5.76	171469	{}	\N	2020-10-12 16:04:12.927436
68703	RYI	2020-09-29	5.75	5.79	5.46	5.53	223108	{}	\N	2020-10-12 16:04:12.934357
68704	RYI	2020-09-30	5.55	5.81	5.52	5.73	531494	{}	\N	2020-10-12 16:04:12.940381
68705	RYI	2020-10-01	5.67	6.4	5.67	6.23	649221	{}	\N	2020-10-12 16:04:12.946687
68706	RYI	2020-10-02	6.02	6.5	6	6.39	319030	{}	\N	2020-10-12 16:04:12.95345
68707	RYI	2020-10-05	6.51	6.67	6.31	6.37	343394	{}	\N	2020-10-12 16:04:12.959595
68708	RYI	2020-10-06	6.39	6.79	6.3	6.63	1191475	{}	\N	2020-10-12 16:04:12.965178
68709	RYI	2020-10-07	6.7	7.09	6.68	7.08	476395	{}	\N	2020-10-12 16:04:12.970738
68710	RYI	2020-10-08	7	7.49	6.95	7.4	528180	{}	\N	2020-10-12 16:04:12.976406
68711	RYI	2020-10-09	7.42	7.5922	7.405	7.56	534318	{}	\N	2020-10-12 16:04:12.981838
68712	RYI	2020-10-12	7.69	7.73	7.41	7.68	96990	{}	\N	2020-10-12 16:04:12.987346
68713	SAIA	2020-09-28	124.48	127.845	124.48	127.09	166876	{}	\N	2020-10-12 16:04:14.613661
68714	SAIA	2020-09-29	127.27	129.84	124.92	126.43	104179	{}	\N	2020-10-12 16:04:14.622757
68715	SAIA	2020-09-30	126.43	129.17	124.22	126.14	263603	{}	\N	2020-10-12 16:04:14.631128
68716	SAIA	2020-10-01	125.145	128.8	124.23	127.83	240741	{}	\N	2020-10-12 16:04:14.637676
68717	SAIA	2020-10-02	125.51	132.475	125.51	129.2	137602	{}	\N	2020-10-12 16:04:14.644917
68718	SAIA	2020-10-05	129.79	135.46	129.79	134.13	191537	{}	\N	2020-10-12 16:04:14.650801
68719	SAIA	2020-10-06	134.3	138.64	134.3	136.22	232453	{}	\N	2020-10-12 16:04:14.656663
68720	SAIA	2020-10-07	137.64	142.46	137.42	137.97	260212	{}	\N	2020-10-12 16:04:14.661741
68721	SAIA	2020-10-08	139	143.24	137.67	142.27	166072	{}	\N	2020-10-12 16:04:14.667546
68722	SAIA	2020-10-09	143.42	144.64	140.96	143.78	165616	{}	\N	2020-10-12 16:04:14.673403
68723	SAIA	2020-10-12	144.14	151.36	143.85	145.21	49741	{}	\N	2020-10-12 16:04:14.678813
68724	SBCF	2020-09-28	17.75	18.31	17.41	18.06	200436	{}	\N	2020-10-12 16:04:15.926229
68725	SBCF	2020-09-29	18.01	18.35	17.52	17.99	168587	{}	\N	2020-10-12 16:04:15.93764
68726	SBCF	2020-09-30	17.85	18.35	17.85	18.03	227879	{}	\N	2020-10-12 16:04:15.944962
68727	SBCF	2020-10-01	18.09	18.25	17.62	18.01	207748	{}	\N	2020-10-12 16:04:15.950874
68728	SBCF	2020-10-02	17.71	18.56	17.71	18.46	280255	{}	\N	2020-10-12 16:04:15.956274
68729	SBCF	2020-10-05	18.67	19.33	18.49	19.29	275318	{}	\N	2020-10-12 16:04:15.961549
68730	SBCF	2020-10-06	19.67	20.34	19.44	19.54	234398	{}	\N	2020-10-12 16:04:15.966598
68731	SBCF	2020-10-07	19.86	20.15	19.63	19.7	279237	{}	\N	2020-10-12 16:04:15.971675
68732	SBCF	2020-10-08	19.98	20.11	19.65	19.92	290335	{}	\N	2020-10-12 16:04:15.976745
68733	SBCF	2020-10-09	20.24	20.33	19.89	20.27	180454	{}	\N	2020-10-12 16:04:15.981867
68734	SBCF	2020-10-12	20.2	20.41	20.12	20.31	38575	{}	\N	2020-10-12 16:04:15.986991
68735	SBNY	2020-09-28	86.19	87.01	85.37	85.54	329884	{}	\N	2020-10-12 16:04:17.029962
68736	SBNY	2020-09-29	84.39	84.55	81.15	82.49	723874	{}	\N	2020-10-12 16:04:17.041668
68737	SBNY	2020-09-30	82.87	84.61	82.25	82.99	394013	{}	\N	2020-10-12 16:04:17.048848
68738	SBNY	2020-10-01	82.97	83.83	81.26	81.91	395285	{}	\N	2020-10-12 16:04:17.054967
68739	SBNY	2020-10-02	80.47	84	80.11	83.02	444485	{}	\N	2020-10-12 16:04:17.060726
68740	SBNY	2020-10-05	84.1	86.2	83.75	85.8	630878	{}	\N	2020-10-12 16:04:17.065904
68741	SBNY	2020-10-06	87.2	88.24	84.63	85.1	524371	{}	\N	2020-10-12 16:04:17.070795
68742	SBNY	2020-10-07	85.59	87.26	85.24	86.05	460627	{}	\N	2020-10-12 16:04:17.075581
68743	SBNY	2020-10-08	88.49	88.51	85.93	86.9	567753	{}	\N	2020-10-12 16:04:17.080638
68744	SBNY	2020-10-09	88.58	88.7234	84.52	84.66	431243	{}	\N	2020-10-12 16:04:17.085256
68745	SBNY	2020-10-12	84.24	85.58	84.1801	85.535	88817	{}	\N	2020-10-12 16:04:17.089958
68746	SBS	2020-09-28	8.68	8.77	8.09	8.27	2024218	{}	\N	2020-10-12 16:04:18.350282
68747	SBS	2020-09-29	8.09	8.34	8.045	8.26	1780083	{}	\N	2020-10-12 16:04:18.360938
68748	SBS	2020-09-30	8.24	8.42	8.21	8.3	2037723	{}	\N	2020-10-12 16:04:18.367855
68749	SBS	2020-10-01	8.21	8.325	8.14	8.32	1773292	{}	\N	2020-10-12 16:04:18.374429
68750	SBS	2020-10-02	8.27	8.455	8	8.05	4076733	{}	\N	2020-10-12 16:04:18.379562
68751	SBS	2020-10-05	8.01	8.25	7.955	8.21	1638669	{}	\N	2020-10-12 16:04:18.384411
68752	SBS	2020-10-06	8.26	8.41	8.07	8.12	2513879	{}	\N	2020-10-12 16:04:18.389468
68753	SBS	2020-10-07	8.09	8.155	7.91	8.01	2097007	{}	\N	2020-10-12 16:04:18.393731
68754	SBS	2020-10-08	7.99	8.17	7.965	8.14	1482192	{}	\N	2020-10-12 16:04:18.398045
68755	SBS	2020-10-09	8.17	8.325	8.16	8.18	1124887	{}	\N	2020-10-12 16:04:18.402275
68756	SBS	2020-10-12	8.18	8.425	8.18	8.41	441670	{}	\N	2020-10-12 16:04:18.406455
68757	SCHB	2020-09-28	79.2	79.6485	79.02	79.52	514689	{}	\N	2020-10-12 16:04:20.78229
68758	SCHB	2020-09-29	79.49	79.63	78.92	79.11	794549	{}	\N	2020-10-12 16:04:20.793175
68759	SCHB	2020-09-30	79.32	80.42	79.1828	79.64	616836	{}	\N	2020-10-12 16:04:20.801207
68760	SCHB	2020-10-01	80.33	80.5413	79.84	80.36	480542	{}	\N	2020-10-12 16:04:20.807069
68761	SCHB	2020-10-02	78.88	80.1313	78.76	79.64	612002	{}	\N	2020-10-12 16:04:20.811731
68762	SCHB	2020-10-05	80.28	81.17	80.28	81.15	411217	{}	\N	2020-10-12 16:04:20.816259
68763	SCHB	2020-10-06	81.28	81.84	80.01	80.15	678535	{}	\N	2020-10-12 16:04:20.821016
68764	SCHB	2020-10-07	80.89	81.7122	80.89	81.56	306432	{}	\N	2020-10-12 16:04:20.825619
68765	SCHB	2020-10-08	82.09	82.28	81.85	82.28	439885	{}	\N	2020-10-12 16:04:20.830193
68766	SCHB	2020-10-09	82.73	83.1	82.5627	82.99	481020	{}	\N	2020-10-12 16:04:20.834858
68767	SCHB	2020-10-12	83.65	84.1199	83.4801	84.053	237282	{}	\N	2020-10-12 16:04:20.839804
68768	SCHO	2020-09-28	51.51	51.52	51.51	51.51	987296	{}	\N	2020-10-12 16:04:22.211836
68769	SCHO	2020-09-29	51.53	51.53	51.51	51.52	822951	{}	\N	2020-10-12 16:04:22.223573
68770	SCHO	2020-09-30	51.53	51.53	51.5	51.5	1635289	{}	\N	2020-10-12 16:04:22.231129
68771	SCHO	2020-10-01	51.4679	51.48	51.46	51.475	1122515	{}	\N	2020-10-12 16:04:22.236983
68772	SCHO	2020-10-02	51.48	51.48	51.46	51.4657	980053	{}	\N	2020-10-12 16:04:22.242785
68773	SCHO	2020-10-05	51.46	51.47	51.45	51.45	807683	{}	\N	2020-10-12 16:04:22.248582
68774	SCHO	2020-10-06	51.45	51.46	51.45	51.46	740440	{}	\N	2020-10-12 16:04:22.253304
68775	SCHO	2020-10-07	51.45	51.45	51.44	51.45	841063	{}	\N	2020-10-12 16:04:22.25821
68776	SCHO	2020-10-08	51.45	51.46	51.44	51.46	854529	{}	\N	2020-10-12 16:04:22.263205
68777	SCHO	2020-10-09	51.45	51.45	51.44	51.45	991889	{}	\N	2020-10-12 16:04:22.268198
68778	SCHO	2020-10-12	51.44	51.46	51.44	51.4503	242045	{}	\N	2020-10-12 16:04:22.273533
68779	SCHW	2020-09-28	35.5	36.31	35.43	36.2	5980753	{}	\N	2020-10-12 16:04:23.558578
68780	SCHW	2020-09-29	36.15	36.26	35.49	35.78	13896570	{}	\N	2020-10-12 16:04:23.572729
68781	SCHW	2020-09-30	36.08	36.68	35.955	36.23	13725612	{}	\N	2020-10-12 16:04:23.580956
68782	SCHW	2020-10-01	36.91	37.76	36.6	37.73	38707182	{}	\N	2020-10-12 16:04:23.587865
68783	SCHW	2020-10-02	37.17	38.475	37.07	37.67	43165123	{}	\N	2020-10-12 16:04:23.593992
68784	SCHW	2020-10-05	38.17	38.71	36.75	36.94	189410626	{}	\N	2020-10-12 16:04:23.599863
68785	SCHW	2020-10-06	37.14	37.46	36.07	36.75	39174001	{}	\N	2020-10-12 16:04:23.60487
68786	SCHW	2020-10-07	37.42	38.16	36.9	37.78	36099181	{}	\N	2020-10-12 16:04:23.609888
68787	SCHW	2020-10-08	37.83	37.95	37.44	37.74	21366899	{}	\N	2020-10-12 16:04:23.615219
68788	SCHW	2020-10-09	38.2	38.46	37.565	38.23	22791054	{}	\N	2020-10-12 16:04:23.620474
68789	SCHW	2020-10-12	38.25	38.38	37.9	38.175	4155513	{}	\N	2020-10-12 16:04:23.625524
68790	SCKT	2020-09-28	1.23	1.28	1.15	1.22	76527	{}	\N	2020-10-12 16:04:24.623733
68791	SCKT	2020-09-29	1.23	1.32	1.18	1.25	95673	{}	\N	2020-10-12 16:04:24.640136
68792	SCKT	2020-09-30	1.31	1.37	1.2	1.27	213502	{}	\N	2020-10-12 16:04:24.648009
68793	SCKT	2020-10-01	1.33	1.33	1.23	1.28	84302	{}	\N	2020-10-12 16:04:24.654466
68794	SCKT	2020-10-02	1.24	1.33	1.2311	1.27	75425	{}	\N	2020-10-12 16:04:24.660435
68795	SCKT	2020-10-05	1.28	1.32	1.28	1.3	23029	{}	\N	2020-10-12 16:04:24.666482
68796	SCKT	2020-10-06	1.32	1.3481	1.26	1.28	39327	{}	\N	2020-10-12 16:04:24.671523
68797	SCKT	2020-10-07	1.29	1.32	1.27	1.3	26404	{}	\N	2020-10-12 16:04:24.676621
68798	SCKT	2020-10-08	1.44	1.44	1.29	1.32	30444	{}	\N	2020-10-12 16:04:24.681885
68799	SCKT	2020-10-09	1.3	1.32	1.285	1.31	19934	{}	\N	2020-10-12 16:04:24.687119
68800	SCKT	2020-10-12	1.37	1.37	1.23	1.2699	30630	{}	\N	2020-10-12 16:04:24.691821
68801	SCWX	2020-09-28	11.87	12	11.64	11.81	86954	{}	\N	2020-10-12 16:04:25.790693
68802	SCWX	2020-09-29	11.79	12	11.6	11.65	51262	{}	\N	2020-10-12 16:04:25.805522
68803	SCWX	2020-09-30	11.67	11.86	11.34	11.39	74827	{}	\N	2020-10-12 16:04:25.812946
68804	SCWX	2020-10-01	11.59	11.93	11.47	11.76	100892	{}	\N	2020-10-12 16:04:25.819031
68805	SCWX	2020-10-02	11.49	11.89	11.4	11.5	64060	{}	\N	2020-10-12 16:04:25.824855
68806	SCWX	2020-10-05	11.61	11.78	11.43	11.54	77671	{}	\N	2020-10-12 16:04:25.829944
68807	SCWX	2020-10-06	11.58	11.925	11.25	11.27	164563	{}	\N	2020-10-12 16:04:25.834532
68808	SCWX	2020-10-07	11.475	11.955	11.475	11.69	107370	{}	\N	2020-10-12 16:04:25.838948
68809	SCWX	2020-10-08	11.7	11.92	11.41	11.56	126037	{}	\N	2020-10-12 16:04:25.843674
68810	SCWX	2020-10-09	11.69	11.89	11.57	11.78	56820	{}	\N	2020-10-12 16:04:25.850404
68811	SCWX	2020-10-12	11.99	12.045	11.88	11.89	16159	{}	\N	2020-10-12 16:04:25.855302
68812	RYF	2020-09-28	37.51	38.28	37.51	37.97	57892	{}	\N	2020-10-12 16:04:28.291546
68813	RYF	2020-09-29	37.89	37.97	37.31	37.62	14349	{}	\N	2020-10-12 16:04:28.3069
68814	RYF	2020-09-30	37.76	38.3834	37.74	38.1	27738	{}	\N	2020-10-12 16:04:28.314792
68815	RYF	2020-10-01	38.12	38.44	37.83	37.95	23763	{}	\N	2020-10-12 16:04:28.321879
68816	RYF	2020-10-02	37.885	38.75	37.885	38.62	6188	{}	\N	2020-10-12 16:04:28.327595
68817	RYF	2020-10-05	39.17	39.62	39.17	39.5	17854	{}	\N	2020-10-12 16:04:28.333202
68818	RYF	2020-10-06	39.772	40.259	39.07	39.07	27044	{}	\N	2020-10-12 16:04:28.33802
68819	RYF	2020-10-07	39.91	40.08	39.62	39.9699	14133	{}	\N	2020-10-12 16:04:28.343058
68820	RYF	2020-10-08	40.09	40.5927	40.09	40.51	17936	{}	\N	2020-10-12 16:04:28.348734
68821	RYF	2020-10-09	40.72	40.72	40.25	40.44	12331	{}	\N	2020-10-12 16:04:28.353529
68822	RYF	2020-10-12	40.6271	40.845	40.6271	40.845	2417	{}	\N	2020-10-12 16:04:28.358383
68823	RZA	2020-09-28	26.54	26.7	26.3	26.33	38695	{}	\N	2020-10-12 16:04:30.679135
68824	RZA	2020-09-29	26.41	26.5209	26.22	26.23	56889	{}	\N	2020-10-12 16:04:30.689496
68825	RZA	2020-09-30	26.25	26.338	26.11	26.11	160580	{}	\N	2020-10-12 16:04:30.696218
68826	RZA	2020-10-01	26.29	26.37	26.24	26.36	153962	{}	\N	2020-10-12 16:04:30.701918
68827	RZA	2020-10-02	26.23	26.48	26.23	26.44	52764	{}	\N	2020-10-12 16:04:30.707434
68828	RZA	2020-10-05	26.57	26.65	26.3401	26.41	70784	{}	\N	2020-10-12 16:04:30.713993
68829	RZA	2020-10-06	26.58	26.7418	26.57	26.68	186832	{}	\N	2020-10-12 16:04:30.718326
68830	RZA	2020-10-07	26.76	26.9	26.5751	26.83	74018	{}	\N	2020-10-12 16:04:30.722561
68831	RZA	2020-10-08	26.87	26.942	26.8	26.91	51363	{}	\N	2020-10-12 16:04:30.727075
68832	RZA	2020-10-09	26.99	26.99	26.7201	26.96	41740	{}	\N	2020-10-12 16:04:30.73151
68833	RZA	2020-10-12	26.9	26.96	26.64	26.96	19880	{}	\N	2020-10-12 16:04:30.735738
68834	SAFT	2020-09-28	68.44	70.56	68.44	69.78	74814	{}	\N	2020-10-12 16:04:33.092003
68835	SAFT	2020-09-29	69.48	70.21	69.005	70.06	50772	{}	\N	2020-10-12 16:04:33.09833
68836	SAFT	2020-09-30	69.99	70.56	68.44	69.09	115044	{}	\N	2020-10-12 16:04:33.103054
68837	SAFT	2020-10-01	68.85	68.97	67.03	67.68	62993	{}	\N	2020-10-12 16:04:33.107655
68838	SAFT	2020-10-02	66.71	68.675	66.71	67.88	60674	{}	\N	2020-10-12 16:04:33.111764
68839	SAFT	2020-10-05	68.57	69.13	68.08	68.4	45773	{}	\N	2020-10-12 16:04:33.116091
68840	SAFT	2020-10-06	68.92	68.92	67.1	67.28	93629	{}	\N	2020-10-12 16:04:33.120782
68841	SAFT	2020-10-07	67.79	68.3499	66.67	67.79	89124	{}	\N	2020-10-12 16:04:33.125462
68842	SAFT	2020-10-08	68.3	68.915	67.33	68	52688	{}	\N	2020-10-12 16:04:33.130179
68843	SAFT	2020-10-09	68.17	68.39	67.26	68.33	53249	{}	\N	2020-10-12 16:04:33.135041
68844	SAFT	2020-10-12	68.2	69.75	67.88	69.31	8028	{}	\N	2020-10-12 16:04:33.1398
68845	SBB	2020-09-28	29.71	29.71	29.215	29.215	6207	{}	\N	2020-10-12 16:04:34.065863
68846	SBB	2020-09-29	29.6792	29.6792	29.4533	29.4533	629	{}	\N	2020-10-12 16:04:34.076242
68847	SBB	2020-09-30	29.2	29.45	28.9593	29.395	3092	{}	\N	2020-10-12 16:04:34.082391
68848	SBB	2020-10-01	29.19	29.2882	29.021	29.021	726	{}	\N	2020-10-12 16:04:34.088499
68849	SBB	2020-10-02	28.68	28.77	28.61	28.61	16765	{}	\N	2020-10-12 16:04:34.093419
68850	SBB	2020-10-05	28.16	28.21	27.96	27.98	4997	{}	\N	2020-10-12 16:04:34.09815
68851	SBB	2020-10-06	27.73	28.01	27.3001	27.98	3039	{}	\N	2020-10-12 16:04:34.102814
68852	SBB	2020-10-07	27.705	27.779	27.575	27.58	2464	{}	\N	2020-10-12 16:04:34.107324
68853	SBB	2020-10-08	27.56	27.56	27.15	27.18	7578	{}	\N	2020-10-12 16:04:34.111704
68854	SBB	2020-10-09	26.93	27.085	26.93	27.05	6228	{}	\N	2020-10-12 16:04:34.11606
68855	SBB	2020-10-12	26.943	26.943	26.943	26.943	845	{}	\N	2020-10-12 16:04:34.120427
68856	SBIO	2020-09-28	41	41	40.5	40.86	12973	{}	\N	2020-10-12 16:04:35.554737
68857	SBIO	2020-09-29	40.69	41.313	40.69	41.16	9515	{}	\N	2020-10-12 16:04:35.567796
68858	SBIO	2020-09-30	41.08	41.734	41.04	41.13	12569	{}	\N	2020-10-12 16:04:35.574845
68859	SBIO	2020-10-01	41.42	41.83	41.11	41.83	16415	{}	\N	2020-10-12 16:04:35.581068
68860	SBIO	2020-10-02	40.94	41.7164	40.56	40.56	10645	{}	\N	2020-10-12 16:04:35.586554
68861	SBIO	2020-10-05	41.25	43.05	41.25	43.03	29375	{}	\N	2020-10-12 16:04:35.591582
68862	SBIO	2020-10-06	43.22	43.845	42.93	43.02	14797	{}	\N	2020-10-12 16:04:35.596146
68863	SBIO	2020-10-07	43.77	44.41	43.33	44.23	12391	{}	\N	2020-10-12 16:04:35.600493
68864	SBIO	2020-10-08	44.6	44.701	44.18	44.35	13270	{}	\N	2020-10-12 16:04:35.605028
68865	SBIO	2020-10-09	44.5	44.67	44.31	44.67	10166	{}	\N	2020-10-12 16:04:35.609297
68866	SBIO	2020-10-12	44.96	45.11	44.53	44.855	7260	{}	\N	2020-10-12 16:04:35.613748
68867	SBR	2020-09-28	29.39	30.21	29.39	30.19	23565	{}	\N	2020-10-12 16:04:36.612565
68868	SBR	2020-09-29	30.38	30.49	29.26	29.5	54890	{}	\N	2020-10-12 16:04:36.623228
68869	SBR	2020-09-30	29.75	30.15	28.7	29.1	62109	{}	\N	2020-10-12 16:04:36.62943
68870	SBR	2020-10-01	29.29	29.52	28.7001	29.47	54226	{}	\N	2020-10-12 16:04:36.635162
68871	SBR	2020-10-02	28.8101	29.53	28.8101	29.265	16590	{}	\N	2020-10-12 16:04:36.640803
68872	SBR	2020-10-05	29.33	30.39	29.16	30.3	43408	{}	\N	2020-10-12 16:04:36.645845
68873	SBR	2020-10-06	30.34	30.65	29.7164	29.81	38247	{}	\N	2020-10-12 16:04:36.650553
68874	SBR	2020-10-07	29.67	30.07	29.67	29.81	56620	{}	\N	2020-10-12 16:04:36.654914
68875	SBR	2020-10-08	29.9024	30.34	29.75	30.17	41368	{}	\N	2020-10-12 16:04:36.659069
68876	SBR	2020-10-09	30.4	30.48	29.89	29.95	20556	{}	\N	2020-10-12 16:04:36.663386
68877	SBR	2020-10-12	30.21	30.25	29.59	29.79	22932	{}	\N	2020-10-12 16:04:36.668231
68878	SCHL	2020-09-28	20.5	21.17	20.01	20.81	360114	{}	\N	2020-10-12 16:04:38.914316
68879	SCHL	2020-09-29	20.68	21.02	20.57	20.78	339189	{}	\N	2020-10-12 16:04:38.925674
68880	SCHL	2020-09-30	20.78	21.19	20.71	20.99	293179	{}	\N	2020-10-12 16:04:38.932145
68881	SCHL	2020-10-01	20.885	21.37	20.69	21.35	274188	{}	\N	2020-10-12 16:04:38.937847
68882	SCHL	2020-10-02	21	21.59	20.69	21.41	232378	{}	\N	2020-10-12 16:04:38.942755
68883	SCHL	2020-10-05	21.55	22.06	21.37	21.97	281270	{}	\N	2020-10-12 16:04:38.947659
68884	SCHL	2020-10-06	22.18	22.315	21.51	21.54	277281	{}	\N	2020-10-12 16:04:38.951898
68885	SCHL	2020-10-07	21.69	21.82	21.3	21.55	186996	{}	\N	2020-10-12 16:04:38.95586
68886	SCHL	2020-10-08	21.74	21.765	21.18	21.61	144419	{}	\N	2020-10-12 16:04:38.960308
68887	SCHL	2020-10-09	21.81	21.99	21.44	21.66	121661	{}	\N	2020-10-12 16:04:38.964554
68888	SCHL	2020-10-12	21.88	21.88	21.48	21.745	28501	{}	\N	2020-10-12 16:04:38.968847
68889	SCI	2020-09-28	42.53	43.31	42.44	43.07	1030743	{}	\N	2020-10-12 16:04:40.211755
68890	SCI	2020-09-29	43.03	43.09	41.715	41.98	2050274	{}	\N	2020-10-12 16:04:40.227556
68891	SCI	2020-09-30	42.15	42.4801	41.88	42.18	2347612	{}	\N	2020-10-12 16:04:40.234342
68892	SCI	2020-10-01	42.13	42.42	41.14	42.28	1564500	{}	\N	2020-10-12 16:04:40.240338
68893	SCI	2020-10-02	41.75	43.59	41.5	43.27	2009387	{}	\N	2020-10-12 16:04:40.245871
68894	SCI	2020-10-05	43.52	44.61	43.52	44.6	2382896	{}	\N	2020-10-12 16:04:40.250721
68895	SCI	2020-10-06	44.74	45.52	44.44	44.64	2028273	{}	\N	2020-10-12 16:04:40.255551
68896	SCI	2020-10-07	45.08	45.38	44.62	44.68	1566941	{}	\N	2020-10-12 16:04:40.260192
68897	SCI	2020-10-08	44.87	45.14	44.61	44.75	1358202	{}	\N	2020-10-12 16:04:40.265049
68898	SCI	2020-10-09	44.98	45.29	44.4	44.94	1105673	{}	\N	2020-10-12 16:04:40.269821
68899	SCI	2020-10-12	45.11	45.445	44.71	45.34	241269	{}	\N	2020-10-12 16:04:40.27458
68900	SCON	2020-09-28	2.25	2.28	2.13	2.18	322497	{}	\N	2020-10-12 16:04:41.399972
68901	SCON	2020-09-29	2.13	2.45	2.02	2.35	637721	{}	\N	2020-10-12 16:04:41.408617
68902	SCX	2020-09-28	3.03	3.18	3.03	3.1	37810	{}	\N	2020-10-12 16:04:43.563333
68903	SCX	2020-09-29	3.15	3.16	3.1	3.1101	90612	{}	\N	2020-10-12 16:04:43.573754
68904	SCX	2020-09-30	3.02	3.11	2.95	2.95	68524	{}	\N	2020-10-12 16:04:43.580556
68905	SCX	2020-10-01	2.95	2.98	2.72	2.76	110672	{}	\N	2020-10-12 16:04:43.58607
68906	SCX	2020-10-02	2.79	2.8599	2.51	2.74	195132	{}	\N	2020-10-12 16:04:43.591346
68907	SCX	2020-10-05	2.8	2.9799	2.69	2.91	48448	{}	\N	2020-10-12 16:04:43.59618
68908	SCX	2020-10-06	2.99	3.0998	2.97	2.99	26484	{}	\N	2020-10-12 16:04:43.600532
68909	SCX	2020-10-07	3.1	3.15	2.975	3.04	51700	{}	\N	2020-10-12 16:04:43.604829
68910	SCX	2020-10-08	3.0701	3.0701	3	3.06	29628	{}	\N	2020-10-12 16:04:43.609028
68911	SCX	2020-10-09	3.15	3.16	3.08	3.1256	66589	{}	\N	2020-10-12 16:04:43.613476
68912	SCX	2020-10-12	3.0701	3.132	3.0701	3.132	3980	{}	\N	2020-10-12 16:04:43.617781
68913	SAFE	2020-09-28	57.96	62.26	57.87	62.08	278272	{}	\N	2020-10-12 16:04:46.543042
68914	SAFE	2020-09-29	62.19	62.895	61.6136	62.2	557220	{}	\N	2020-10-12 16:04:46.556461
68915	SAFE	2020-09-30	61.97	63.21	61.13	62.1	382588	{}	\N	2020-10-12 16:04:46.563668
68916	SAFE	2020-10-01	62.77	63.39	61.73	63.18	433355	{}	\N	2020-10-12 16:04:46.569804
68917	SAFE	2020-10-02	62.16	65.22	61.82	64.93	348378	{}	\N	2020-10-12 16:04:46.575331
68918	SAFE	2020-10-05	64.77	65.6	63.98	65.35	267140	{}	\N	2020-10-12 16:04:46.580338
68919	SAFE	2020-10-06	65.59	67.24	64.26	65.6	319232	{}	\N	2020-10-12 16:04:46.584677
68920	SAFE	2020-10-07	65.82	65.82	62.1	62.55	332683	{}	\N	2020-10-12 16:04:46.589146
68921	SAFE	2020-10-08	62.72	64.47	61.87	64.12	181297	{}	\N	2020-10-12 16:04:46.593797
68922	SAFE	2020-10-09	64.87	64.87	62.99	64.46	84573	{}	\N	2020-10-12 16:04:46.59802
68923	SAFE	2020-10-12	64.87	65.21	64.32	65.0885	22831	{}	\N	2020-10-12 16:04:46.603587
68924	SALM	2020-09-28	0.95	1.02	0.92	0.95	1949288	{}	\N	2020-10-12 16:04:48.2216
68925	SALM	2020-09-29	0.93	1.05	0.9025	0.9905	3657123	{}	\N	2020-10-12 16:04:48.233384
68926	SALM	2020-09-30	0.96	0.9689	0.89	0.9157	1308899	{}	\N	2020-10-12 16:04:48.240657
68927	SALM	2020-10-01	0.9175	0.92	0.902	0.91	318229	{}	\N	2020-10-12 16:04:48.246587
68928	SALM	2020-10-02	0.9	0.937	0.8611	0.9249	720093	{}	\N	2020-10-12 16:04:48.25397
68929	SALM	2020-10-05	0.9234	0.9399	0.903	0.9235	445777	{}	\N	2020-10-12 16:04:48.259136
68930	SALM	2020-10-06	0.92	0.9798	0.91	0.9353	916173	{}	\N	2020-10-12 16:04:48.263864
68931	SALM	2020-10-07	0.93	1.19	0.93	1.18	5790674	{}	\N	2020-10-12 16:04:48.268349
68932	SALM	2020-10-08	1.1	1.25	1.02	1.05	4836059	{}	\N	2020-10-12 16:04:48.273197
68933	SALM	2020-10-09	1.01	1.0199	0.965	0.9812	915072	{}	\N	2020-10-12 16:04:48.278017
68934	SALM	2020-10-12	0.99	0.997699	0.95	0.9531	412882	{}	\N	2020-10-12 16:04:48.282859
68935	SASR	2020-09-28	22.63	23.38	22.63	23.19	132834	{}	\N	2020-10-12 16:04:49.319068
68936	SASR	2020-09-29	23.23	23.23	22.57	22.98	135231	{}	\N	2020-10-12 16:04:49.329209
68937	SASR	2020-09-30	23.22	23.5	22.915	23.08	154724	{}	\N	2020-10-12 16:04:49.335705
68938	SASR	2020-10-01	23.09	23.27	22.82	23.25	102849	{}	\N	2020-10-12 16:04:49.341485
68939	SASR	2020-10-02	22.82	24.13	22.72	24.01	109102	{}	\N	2020-10-12 16:04:49.34671
68940	SASR	2020-10-05	24.25	25.03	24.21	24.94	160508	{}	\N	2020-10-12 16:04:49.351328
68941	SASR	2020-10-06	25	25.97	24.91	25.03	290398	{}	\N	2020-10-12 16:04:49.356138
68942	SASR	2020-10-07	25.46	25.925	23.52	25.59	312684	{}	\N	2020-10-12 16:04:49.360469
68943	SASR	2020-10-08	25.97	25.99	25.29	25.83	107575	{}	\N	2020-10-12 16:04:49.364972
68944	SASR	2020-10-09	25.99	26.225	25.5	25.54	87060	{}	\N	2020-10-12 16:04:49.369239
68945	SASR	2020-10-12	25.5	25.92	25.315	25.835	22639	{}	\N	2020-10-12 16:04:49.373937
68946	SBH	2020-09-28	9.31	9.45	9.035	9.06	2535789	{}	\N	2020-10-12 16:04:51.003408
68947	SBH	2020-09-29	8.99	9.07	8.71	8.8	2740529	{}	\N	2020-10-12 16:04:51.014384
68948	SBH	2020-09-30	8.88	8.95	8.6	8.69	3737573	{}	\N	2020-10-12 16:04:51.020666
68949	SBH	2020-10-01	9.27	9.785	9.17	9.29	16661344	{}	\N	2020-10-12 16:04:51.026116
68950	SBH	2020-10-02	9.03	9.83	8.95	9.05	67886821	{}	\N	2020-10-12 16:04:51.031197
68951	SBH	2020-10-05	9.27	9.68	9.14	9.66	5859909	{}	\N	2020-10-12 16:04:51.036075
68952	SBH	2020-10-06	9.81	9.86	9.26	9.28	5203796	{}	\N	2020-10-12 16:04:51.040538
68953	SBH	2020-10-07	9.29	9.45	9.195	9.37	6652095	{}	\N	2020-10-12 16:04:51.045336
68954	SBH	2020-10-08	9.45	10.065	9.45	9.9	4800776	{}	\N	2020-10-12 16:04:51.050509
68955	SBH	2020-10-09	10.04	10.16	9.51	9.52	2643676	{}	\N	2020-10-12 16:04:51.055396
68956	SBH	2020-10-12	9.45	9.57	9.325	9.505	1288323	{}	\N	2020-10-12 16:04:51.060597
68957	SBUX	2020-09-28	85.52	86.24	84.99	86.07	5780433	{}	\N	2020-10-12 16:04:53.75332
68958	SBUX	2020-09-29	86.25	86.25	84.625	84.8	3970842	{}	\N	2020-10-12 16:04:53.767736
68959	SBUX	2020-09-30	86.63	87.16	85.47	85.92	9889996	{}	\N	2020-10-12 16:04:53.777545
68960	SBUX	2020-10-01	87.04	87.45	86.195	86.74	6932844	{}	\N	2020-10-12 16:04:53.78501
68961	SBUX	2020-10-02	85.15	87.13	85.09	86.57	5303438	{}	\N	2020-10-12 16:04:53.792688
68962	SBUX	2020-10-05	87.54	88.675	87.2	88.47	6878486	{}	\N	2020-10-12 16:04:53.798061
68963	SBUX	2020-10-06	88.65	89.28	86.8	87.01	7571816	{}	\N	2020-10-12 16:04:53.804285
68964	SBUX	2020-10-07	87.82	88.98	87.44	88.45	5019001	{}	\N	2020-10-12 16:04:53.809714
68965	SBUX	2020-10-08	88.46	89.61	88.41	89.53	4951839	{}	\N	2020-10-12 16:04:53.81567
68966	SBUX	2020-10-09	89.8	90.33	89.58	90.01	6082606	{}	\N	2020-10-12 16:04:53.820728
68967	SBUX	2020-10-12	90.4595	90.77	89.9301	90.67	2199382	{}	\N	2020-10-12 16:04:53.825682
68968	SCCO	2020-09-28	45.44	45.74	44.85	44.89	906471	{}	\N	2020-10-12 16:04:55.31179
68969	SCCO	2020-09-29	44.75	45.01	44.43	44.45	1958125	{}	\N	2020-10-12 16:04:55.323438
68970	SCCO	2020-09-30	44.83	45.41	44.61	45.27	2676709	{}	\N	2020-10-12 16:04:55.330152
68971	SCCO	2020-10-01	45.44	45.69	44.15	44.49	1695989	{}	\N	2020-10-12 16:04:55.335762
68972	SCCO	2020-10-02	43.65	46.26	43.63	45.63	1361859	{}	\N	2020-10-12 16:04:55.341726
68973	SCCO	2020-10-05	45.85	47.15	45.77	46.79	1546762	{}	\N	2020-10-12 16:04:55.34661
68974	SCCO	2020-10-06	46.85	46.94	45.57	45.61	1431371	{}	\N	2020-10-12 16:04:55.350828
68975	SCCO	2020-10-07	46.57	48.14	46.37	47.15	1587474	{}	\N	2020-10-12 16:04:55.354857
68976	SCCO	2020-10-08	47.51	47.79	46.865	47.43	887803	{}	\N	2020-10-12 16:04:55.358885
68977	SCCO	2020-10-09	48.26	48.48	47.77	48.08	1820480	{}	\N	2020-10-12 16:04:55.363133
68978	SCCO	2020-10-12	48	48.1	47.61	47.74	179515	{}	\N	2020-10-12 16:04:55.367645
68979	SCHH	2020-09-28	35.25	35.6906	35.24	35.54	497933	{}	\N	2020-10-12 16:04:56.404081
68980	SCHH	2020-09-29	35.53	35.55	34.9843	35.27	518999	{}	\N	2020-10-12 16:04:56.418357
68981	SCHH	2020-09-30	35.39	35.78	35.03	35.37	510469	{}	\N	2020-10-12 16:04:56.425774
68982	SCHH	2020-10-01	35.49	36.13	35.39	36.09	468528	{}	\N	2020-10-12 16:04:56.432117
68983	SCHH	2020-10-02	35.5465	36.85	35.47	36.7	309539	{}	\N	2020-10-12 16:04:56.437588
68984	SCHH	2020-10-05	36.89	36.98	36.23	36.93	370824	{}	\N	2020-10-12 16:04:56.442761
68985	SCHH	2020-10-06	37	37.2	36.55	36.68	873274	{}	\N	2020-10-12 16:04:56.447832
68986	SCHH	2020-10-07	36.86	36.94	36.54	36.7	497475	{}	\N	2020-10-12 16:04:56.452659
68987	SCHH	2020-10-08	36.88	37.405	36.81	37.28	406814	{}	\N	2020-10-12 16:04:56.45689
68988	SCHH	2020-10-09	37.47	37.51	37	37.17	505636	{}	\N	2020-10-12 16:04:56.461312
68989	SCHH	2020-10-12	37.18	37.325	36.92	37.24	175035	{}	\N	2020-10-12 16:04:56.465946
68990	SCPH	2020-09-28	7.19	7.54	7.176	7.43	36967	{}	\N	2020-10-12 16:04:57.897893
68991	SCPH	2020-09-29	7.43	7.6487	7.25	7.51	39616	{}	\N	2020-10-12 16:04:57.911396
68992	SCPH	2020-09-30	7.54	7.54	7.2773	7.45	49119	{}	\N	2020-10-12 16:04:57.918376
68993	SCPH	2020-10-01	7.43	7.6099	7.37	7.5	55347	{}	\N	2020-10-12 16:04:57.925354
68994	SCPH	2020-10-02	7.31	7.64	7.31	7.42	42857	{}	\N	2020-10-12 16:04:57.930379
68995	SCPH	2020-10-05	7.59	7.68	7.18	7.65	22443	{}	\N	2020-10-12 16:04:57.935479
68996	SCPH	2020-10-06	7.68	7.9399	7.6401	7.66	50098	{}	\N	2020-10-12 16:04:57.939743
68997	SCPH	2020-10-07	7.78	8.1	7.72	7.99	80582	{}	\N	2020-10-12 16:04:57.943937
68998	SCPH	2020-10-08	8	8.33	7.5275	8.24	66837	{}	\N	2020-10-12 16:04:57.948838
68999	SCPH	2020-10-09	8.3	8.35	7.98	8.13	39521	{}	\N	2020-10-12 16:04:57.952994
69000	SCPH	2020-10-12	8.21	8.33	8.0881	8.29	4570	{}	\N	2020-10-12 16:04:57.956948
69001	SCZ	2020-09-28	58.83	58.97	58.73	58.97	1199015	{}	\N	2020-10-12 16:04:58.945495
69002	SCZ	2020-09-29	58.92	59.12	58.72	58.91	743321	{}	\N	2020-10-12 16:04:58.955424
69003	SCZ	2020-09-30	58.88	59.369	58.8328	58.99	1786946	{}	\N	2020-10-12 16:04:58.962198
69004	SCZ	2020-10-01	59.3706	59.56	59.175	59.56	2443607	{}	\N	2020-10-12 16:04:58.967976
69005	SCZ	2020-10-02	58.73	59.41	58.73	59.31	2056189	{}	\N	2020-10-12 16:04:58.972956
69006	SCZ	2020-10-05	59.98	60.33	59.905	60.33	1578448	{}	\N	2020-10-12 16:04:58.977768
69007	SCZ	2020-10-06	60.42	60.55	59.73	59.86	1718081	{}	\N	2020-10-12 16:04:58.981983
69008	SCZ	2020-10-07	60.23	60.3299	60.04	60.22	1420271	{}	\N	2020-10-12 16:04:58.986485
69009	SCZ	2020-10-08	60.67	60.67	60.44	60.64	614435	{}	\N	2020-10-12 16:04:58.991407
69010	SCZ	2020-10-09	60.91	61.21	60.91	61.2	691441	{}	\N	2020-10-12 16:04:58.996027
69011	SCZ	2020-10-12	61.31	61.499	61.26	61.405	140255	{}	\N	2020-10-12 16:04:59.000914
69012	SDPI	2020-09-28	0.43	0.52	0.4079	0.4808	399142	{}	\N	2020-10-12 16:05:00.266219
69013	SDPI	2020-09-29	0.5282	0.5499	0.4141	0.4676	402772	{}	\N	2020-10-12 16:05:00.276043
69014	SDPI	2020-09-30	0.448	0.4753	0.3842	0.4109	238482	{}	\N	2020-10-12 16:05:00.282611
69015	SDPI	2020-10-01	0.435	0.435	0.3905	0.4002	59411	{}	\N	2020-10-12 16:05:00.288817
69016	SDPI	2020-10-02	0.391	0.4299	0.391	0.419	34145	{}	\N	2020-10-12 16:05:00.294595
69017	SDPI	2020-10-05	0.4238	0.44	0.388	0.395	91654	{}	\N	2020-10-12 16:05:00.299746
69018	SDPI	2020-10-06	0.3902	0.4147	0.334999	0.3621	116346	{}	\N	2020-10-12 16:05:00.304687
69019	SDPI	2020-10-07	0.371	0.386	0.3551	0.3632	76059	{}	\N	2020-10-12 16:05:00.309516
69020	SDPI	2020-10-08	0.3702	0.51	0.3631	0.4123	1298963	{}	\N	2020-10-12 16:05:00.314327
69021	SDPI	2020-10-09	0.73	0.92	0.532	0.6	15304183	{}	\N	2020-10-12 16:05:00.318911
69022	SDPI	2020-10-12	0.5579	0.57	0.5207	0.5491	562434	{}	\N	2020-10-12 16:05:00.323671
69023	SAR	2020-09-28	16.79	17.34	16.79	17.34	30865	{}	\N	2020-10-12 16:05:02.229201
69024	SAR	2020-09-29	17.35	17.35	17	17.06	26815	{}	\N	2020-10-12 16:05:02.241304
69025	SAR	2020-09-30	17.01	17.22	16.74	16.79	46596	{}	\N	2020-10-12 16:05:02.248644
69026	SAR	2020-10-01	16.61	17.02	16.42	17.02	78983	{}	\N	2020-10-12 16:05:02.255352
69027	SAR	2020-10-02	16.76	17.27	16.5371	17.27	47957	{}	\N	2020-10-12 16:05:02.261669
69028	SAR	2020-10-05	17.3	17.7799	17.3	17.37	73080	{}	\N	2020-10-12 16:05:02.266967
69029	SAR	2020-10-06	17.57	17.76	17.51	17.63	115641	{}	\N	2020-10-12 16:05:02.272753
69030	SAR	2020-10-07	17.7	17.94	17.6	17.7	198462	{}	\N	2020-10-12 16:05:02.278395
69031	SAR	2020-10-08	18.5	18.8188	18.3014	18.67	254531	{}	\N	2020-10-12 16:05:02.285899
69032	SAR	2020-10-09	18.95	19.3	18.8717	19.2	209030	{}	\N	2020-10-12 16:05:02.292071
69033	SAR	2020-10-12	19.39	20	19.23	19.76	59283	{}	\N	2020-10-12 16:05:02.298469
69034	SBBP	2020-09-28	2.21	2.23	2.11	2.15	380439	{}	\N	2020-10-12 16:05:03.423173
69035	SBBP	2020-09-29	2.1	2.13	2.09	2.11	360781	{}	\N	2020-10-12 16:05:03.439348
69036	SBBP	2020-09-30	2.14	2.15	2.09	2.1	345718	{}	\N	2020-10-12 16:05:03.446128
69037	SBBP	2020-10-01	2.13	2.14	2.015	2.09	512225	{}	\N	2020-10-12 16:05:03.451509
69038	SBBP	2020-10-02	2.09	2.12	2.04	2.06	458379	{}	\N	2020-10-12 16:05:03.456914
69039	SBBP	2020-10-05	2.1	2.2378	2.09	2.16	497091	{}	\N	2020-10-12 16:05:03.462543
69040	SBBP	2020-10-06	2.2	2.23	2.13	2.14	266240	{}	\N	2020-10-12 16:05:03.467602
69041	SBBP	2020-10-07	2.15	2.22	2.12	2.14	414149	{}	\N	2020-10-12 16:05:03.472237
69042	SBBP	2020-10-08	2.18	2.19	2.12	2.17	313105	{}	\N	2020-10-12 16:05:03.477096
69043	SBBP	2020-10-09	2.2	2.3	2.2	2.26	244565	{}	\N	2020-10-12 16:05:03.481735
69044	SBBP	2020-10-12	2.28	2.32	2.255	2.26	165380	{}	\N	2020-10-12 16:05:03.4865
69045	SBPH	2020-09-28	1.38	1.4	1.34	1.35	51058	{}	\N	2020-10-12 16:05:05.636968
69046	SBPH	2020-09-29	1.42	1.4408	1.375	1.42	61612	{}	\N	2020-10-12 16:05:05.648393
69047	SBPH	2020-09-30	1.4	1.421	1.34	1.34	23864	{}	\N	2020-10-12 16:05:05.656229
69048	SBPH	2020-10-01	1.35	1.42	1.31	1.31	68130	{}	\N	2020-10-12 16:05:05.663218
69049	SBPH	2020-10-02	1.31	1.35	1.28	1.33	28839	{}	\N	2020-10-12 16:05:05.669826
69050	SBPH	2020-10-05	1.33	1.35	1.31	1.32	35254	{}	\N	2020-10-12 16:05:05.674934
69051	SBPH	2020-10-06	1.32	1.37	1.31	1.34	30249	{}	\N	2020-10-12 16:05:05.680099
69052	SBPH	2020-10-07	1.35	1.37	1.33	1.34	20075	{}	\N	2020-10-12 16:05:05.68526
69053	SBPH	2020-10-08	1.33	1.39	1.3	1.38	33568	{}	\N	2020-10-12 16:05:05.690728
69054	SBPH	2020-10-09	1.38	1.38	1.34	1.36	15060	{}	\N	2020-10-12 16:05:05.696224
69055	SBPH	2020-10-12	1.38	1.44	1.38	1.43	7737	{}	\N	2020-10-12 16:05:05.701815
69056	SCHC	2020-09-28	32.39	32.49	32.3201	32.47	175466	{}	\N	2020-10-12 16:05:07.160677
69057	SCHC	2020-09-29	32.43	32.5588	32.335	32.48	163073	{}	\N	2020-10-12 16:05:07.170902
69058	SCHC	2020-09-30	32.44	32.73	32.44	32.53	391242	{}	\N	2020-10-12 16:05:07.176978
69059	SCHC	2020-10-01	32.73	32.88	32.589	32.85	442607	{}	\N	2020-10-12 16:05:07.182248
69060	SCHC	2020-10-02	32.39	32.76	32.39	32.65	260748	{}	\N	2020-10-12 16:05:07.186983
69061	SCHC	2020-10-05	32.99	33.25	32.99	33.23	216903	{}	\N	2020-10-12 16:05:07.191564
69062	SCHC	2020-10-06	33.32	33.42	32.9439	32.95	169840	{}	\N	2020-10-12 16:05:07.196871
69063	SCHC	2020-10-07	33.21	33.29	33.15	33.25	137060	{}	\N	2020-10-12 16:05:07.200754
69064	SCHC	2020-10-08	33.41	33.53	33.38	33.47	179004	{}	\N	2020-10-12 16:05:07.204817
69065	SCHC	2020-10-09	33.7	33.8699	33.7	33.81	235102	{}	\N	2020-10-12 16:05:07.209114
69066	SCHC	2020-10-12	33.88	33.98	33.8238	33.945	48339	{}	\N	2020-10-12 16:05:07.213066
69067	SCHR	2020-09-28	58.7	58.7299	58.7	58.72	311308	{}	\N	2020-10-12 16:05:08.843628
69068	SCHR	2020-09-29	58.745	58.77	58.73	58.75	272069	{}	\N	2020-10-12 16:05:08.854723
69069	SCHR	2020-09-30	58.73	58.73	58.64	58.68	389169	{}	\N	2020-10-12 16:05:08.862248
69070	SCHR	2020-10-01	58.56	58.655	58.53	58.64	370232	{}	\N	2020-10-12 16:05:08.86829
69071	SCHR	2020-10-02	58.66	58.66	58.57	58.59	1863008	{}	\N	2020-10-12 16:05:08.874119
69072	SCHR	2020-10-05	58.51	58.52	58.405	58.41	432367	{}	\N	2020-10-12 16:05:08.879208
69073	SCHR	2020-10-06	58.41	58.52	58.3963	58.47	1065930	{}	\N	2020-10-12 16:05:08.884819
69074	SCHR	2020-10-07	58.42	58.43	58.3501	58.38	386982	{}	\N	2020-10-12 16:05:08.889658
69075	SCHR	2020-10-08	58.42	58.45	58.4	58.445	319645	{}	\N	2020-10-12 16:05:08.894392
69076	SCHR	2020-10-09	58.42	58.46	58.37	58.42	403532	{}	\N	2020-10-12 16:05:08.899507
69077	SCHR	2020-10-12	58.43	58.44	58.4137	58.425	53418	{}	\N	2020-10-12 16:05:08.904254
69078	SCJ	2020-09-28	74.01	74.35	73.97	74.25	103990	{}	\N	2020-10-12 16:05:10.272903
69079	SCJ	2020-09-29	74.14	74.24	73.9463	73.9463	9044	{}	\N	2020-10-12 16:05:10.286365
69080	SCJ	2020-09-30	73.13	73.505	73.1	73.4	14774	{}	\N	2020-10-12 16:05:10.294249
69081	SCJ	2020-10-01	73.2	73.4139	73.18	73.4139	6449	{}	\N	2020-10-12 16:05:10.299943
69082	SCJ	2020-10-02	72.81	73.04	72.77	72.88	8854	{}	\N	2020-10-12 16:05:10.304329
69083	SCJ	2020-10-05	73.6699	73.85	73.5601	73.79	8033	{}	\N	2020-10-12 16:05:10.308878
69084	SCJ	2020-10-06	73.79	73.98	73.38	73.49	13664	{}	\N	2020-10-12 16:05:10.3129
69085	SCJ	2020-10-07	73.69	73.7017	73.64	73.7017	10147	{}	\N	2020-10-12 16:05:10.316646
69086	SCJ	2020-10-08	74.02	74.03	73.8587	73.8587	2154	{}	\N	2020-10-12 16:05:10.320549
69087	SCJ	2020-10-09	74.03	74.03	74.013	74.013	540	{}	\N	2020-10-12 16:05:10.324663
69088	SCJ	2020-10-12	74.025	74.025	73.94	74.0201	13993	{}	\N	2020-10-12 16:05:10.3286
69089	SCVL	2020-09-28	34.88	35.88	34.095	34.22	197437	{}	\N	2020-10-12 16:05:11.629926
69090	SCVL	2020-09-29	34.09	34.18	32.57	33.35	163684	{}	\N	2020-10-12 16:05:11.639871
69091	SCVL	2020-09-30	33.42	34.78	33.42	33.58	194500	{}	\N	2020-10-12 16:05:11.647219
69092	SCVL	2020-10-01	33.6	34.32	33.15	33.44	220286	{}	\N	2020-10-12 16:05:11.653157
69093	SCVL	2020-10-02	32.99	34.43	32.36	34.14	304082	{}	\N	2020-10-12 16:05:11.658863
69094	SCVL	2020-10-05	34.4	35.8	34.01	35.03	187309	{}	\N	2020-10-12 16:05:11.66398
69095	SCVL	2020-10-06	35.2	35.69	34.08	34.19	160535	{}	\N	2020-10-12 16:05:11.668786
69096	SCVL	2020-10-07	34.71	35.94	34.71	35.44	169613	{}	\N	2020-10-12 16:05:11.674051
69097	SCVL	2020-10-08	35.95	36.42	35.61	36.2	137674	{}	\N	2020-10-12 16:05:11.678518
69098	SCVL	2020-10-09	36.56	36.99	36.1	36.84	200635	{}	\N	2020-10-12 16:05:11.682986
69099	SCVL	2020-10-12	36.94	36.94	36.505	36.745	37651	{}	\N	2020-10-12 16:05:11.687557
69100	SDOW	2020-09-28	18.62	18.75	18.23	18.51	8170584	{}	\N	2020-10-12 16:05:12.710296
69101	SDOW	2020-09-29	18.55	19.02	18.48	18.81	5119085	{}	\N	2020-10-12 16:05:12.723653
69102	SDOW	2020-09-30	18.68	18.68	17.61	18.12	14049827	{}	\N	2020-10-12 16:05:12.731387
69103	SDOW	2020-10-01	17.78	18.3314	17.5916	18.02	7304867	{}	\N	2020-10-12 16:05:12.737743
69104	SDOW	2020-10-02	18.81	18.92	17.95	18.28	15927898	{}	\N	2020-10-12 16:05:12.743197
69105	SDOW	2020-10-05	17.89	17.91	17.34	17.37	6327136	{}	\N	2020-10-12 16:05:12.748346
69106	SDOW	2020-10-06	17.1558	18.1416	16.98	18.08	13660238	{}	\N	2020-10-12 16:05:12.753042
69107	SDOW	2020-10-07	17.58	17.58	16.9	17.03	8095773	{}	\N	2020-10-12 16:05:12.757666
69108	SDOW	2020-10-08	16.76	17.0899	16.72	16.8	6318963	{}	\N	2020-10-12 16:05:12.762338
69109	SDOW	2020-10-09	16.58	16.77	16.34	16.49	6412023	{}	\N	2020-10-12 16:05:12.767023
69110	SDOW	2020-10-12	16.31	16.365	16	16.1	2319090	{}	\N	2020-10-12 16:05:12.771562
69111	SDYL	2020-09-28	69.75	69.949	69.75	69.949	1517	{}	\N	2020-10-12 16:05:13.902905
69112	SDYL	2020-09-29	68.08	68.7694	68.08	68.7694	1176	{}	\N	2020-10-12 16:05:13.920229
69113	SDYL	2020-09-30	70.15	70.15	69.8329	69.8329	1726	{}	\N	2020-10-12 16:05:13.927872
69114	SDYL	2020-10-01	69.9	69.9	69.5407	69.5407	1500	{}	\N	2020-10-12 16:05:13.933976
69115	SDYL	2020-10-02	69.1	71.344	69.1	71.344	711	{}	\N	2020-10-12 16:05:13.939296
69116	SDYL	2020-10-05	72.4	73.4641	72.4	73.4641	1899	{}	\N	2020-10-12 16:05:13.944536
69117	SDYL	2020-10-06	73.0567	73.0567	73.0567	73.0567	574	{}	\N	2020-10-12 16:05:13.949222
69118	SDYL	2020-10-08	77.32	77.3412	77.32	77.3412	1446	{}	\N	2020-10-12 16:05:13.953767
69119	SDYL	2020-10-09	76.9898	77.38	76.9898	77.15	1049	{}	\N	2020-10-12 16:05:13.958289
69120	SDYL	2020-10-12	77.5	77.5	77.5	77.5	1509	{}	\N	2020-10-12 16:05:13.962828
69121	SOHOB	2020-09-28	5.2	5.59	5.2	5.505	9158	{}	\N	2020-10-12 16:05:15.15084
69122	SOHOB	2020-09-29	5.36	5.555	5.1	5.2797	4983	{}	\N	2020-10-12 16:05:15.159154
69123	SOHOB	2020-09-30	5.49	5.65	5.22	5.36	13677	{}	\N	2020-10-12 16:05:15.164914
69124	SOHOB	2020-10-01	5.16	5.5	5.14	5.49	4390	{}	\N	2020-10-12 16:05:15.169902
69125	SOHOB	2020-10-02	5.35	5.605	5.35	5.4	1872	{}	\N	2020-10-12 16:05:15.174727
69126	SOHOB	2020-10-05	5.7	5.7	5.16	5.53	689	{}	\N	2020-10-12 16:05:15.178969
69127	SOHOB	2020-10-06	5.45	5.45	5.1	5.1	2426	{}	\N	2020-10-12 16:05:15.183062
69128	SOHOB	2020-10-07	5.8299	5.8299	5.178	5.29	1616	{}	\N	2020-10-12 16:05:15.187112
69129	SOHOB	2020-10-08	5.57	5.57	5.2	5.21	3785	{}	\N	2020-10-12 16:05:15.191167
69130	SOHOB	2020-10-09	5.3531	5.71	5.26	5.45	3361	{}	\N	2020-10-12 16:05:15.19527
69131	SBOW	2020-09-28	4	4.11	4	4.02	82718	{}	\N	2020-10-12 16:05:16.177274
69132	SBOW	2020-09-29	3.99	4.1	3.88	4.05	83849	{}	\N	2020-10-12 16:05:16.187299
69133	SBOW	2020-09-30	4.03	4.19	3.9	3.93	88131	{}	\N	2020-10-12 16:05:16.194024
69134	SBOW	2020-10-01	3.86	3.97	3.83	3.86	143602	{}	\N	2020-10-12 16:05:16.199535
69135	SBOW	2020-10-02	3.82	4	3.8	3.91	120119	{}	\N	2020-10-12 16:05:16.20437
69136	SBOW	2020-10-05	4.01	4.19	3.94	4.08	153242	{}	\N	2020-10-12 16:05:16.209083
69137	SBOW	2020-10-06	4.13	4.18	3.94	4.04	216970	{}	\N	2020-10-12 16:05:16.213183
69138	SBOW	2020-10-07	3.98	4.19	3.95	4.11	138892	{}	\N	2020-10-12 16:05:16.217403
69139	SBOW	2020-10-08	4.11	4.31	4.07	4.3	154060	{}	\N	2020-10-12 16:05:16.221486
69140	SBOW	2020-10-09	4.33	4.45	4.17	4.36	208582	{}	\N	2020-10-12 16:05:16.225743
69141	SBOW	2020-10-12	4.26	4.4155	4.2	4.41	137384	{}	\N	2020-10-12 16:05:16.229889
69142	SCO	2020-09-28	16.71	16.91	16.31	16.51	2189364	{}	\N	2020-10-12 16:05:18.572681
69143	SCO	2020-09-29	16.89	17.96	16.87	17.54	2562933	{}	\N	2020-10-12 16:05:18.58535
69144	SCO	2020-09-30	17.22	17.3	16.55	16.77	3845076	{}	\N	2020-10-12 16:05:18.592277
69145	SCO	2020-10-01	17.369	18.4148	17.25	17.63	3903359	{}	\N	2020-10-12 16:05:18.598574
69146	SCO	2020-10-02	19.0954	19.25	18.35	18.95	3919632	{}	\N	2020-10-12 16:05:18.604573
69147	SCO	2020-10-05	17.36	17.55	16.64	16.94	5069316	{}	\N	2020-10-12 16:05:18.609844
69148	SCO	2020-10-06	16.15	16.5102	15.92	16.43	5520684	{}	\N	2020-10-12 16:05:18.615406
69149	SCO	2020-10-07	16.64	16.94	16.43	16.45	2816001	{}	\N	2020-10-12 16:05:18.620243
69150	SCO	2020-10-08	15.82	16.11	15.665	15.74	3783785	{}	\N	2020-10-12 16:05:18.625161
69151	SCO	2020-10-09	15.8299	16.2887	15.615	16.13	3012535	{}	\N	2020-10-12 16:05:18.630214
69152	SCO	2020-10-12	16.44	17.135	16.44	16.87	1618098	{}	\N	2020-10-12 16:05:18.635156
69153	SGH	2020-09-28	26.27	27.1	26.27	26.99	109793	{}	\N	2020-10-12 16:05:21.330118
69154	SGH	2020-09-29	26.93	27.72	26.825	27.51	150201	{}	\N	2020-10-12 16:05:21.342669
69155	SGH	2020-09-30	27.55	28.1699	26.93	27.34	165445	{}	\N	2020-10-12 16:05:21.350011
69156	SGH	2020-10-01	27.69	28.35	27.5539	28.3	614607	{}	\N	2020-10-12 16:05:21.356374
69157	SGH	2020-10-02	26.1	26.85	24.67	24.89	958803	{}	\N	2020-10-12 16:05:21.361687
69158	SGH	2020-10-05	24.93	25.2399	23.6	24.43	748107	{}	\N	2020-10-12 16:05:21.367045
69159	SGH	2020-10-06	24.47	25.26	24.31	24.39	340492	{}	\N	2020-10-12 16:05:21.371525
69160	SGH	2020-10-07	24.67	24.6983	23.76	23.89	405379	{}	\N	2020-10-12 16:05:21.376583
69161	SGH	2020-10-08	24.18	24.8	23.86	24.78	252509	{}	\N	2020-10-12 16:05:21.381069
69162	SGH	2020-10-09	25	25.3	24.2208	25.16	261988	{}	\N	2020-10-12 16:05:21.385028
69163	SGH	2020-10-12	25.09	25.57	25.04	25.36	84561	{}	\N	2020-10-12 16:05:21.389613
69164	SHE	2020-09-28	76.28	76.3201	76.13	76.27	2654	{}	\N	2020-10-12 16:05:22.377815
69165	SHE	2020-09-29	76.48	76.49	76.0001	76.06	5437	{}	\N	2020-10-12 16:05:22.389143
69166	SHE	2020-09-30	76.23	77.15	76.23	76.3791	2748	{}	\N	2020-10-12 16:05:22.395511
69167	SHE	2020-10-01	77.27	77.27	76.8391	76.94	6595	{}	\N	2020-10-12 16:05:22.401361
69168	SHE	2020-10-02	75.74	77.0025	75.74	76.935	4354	{}	\N	2020-10-12 16:05:22.407135
69169	SHE	2020-10-05	77.76	78.13	77.6901	78.13	2835	{}	\N	2020-10-12 16:05:22.412411
69170	SHE	2020-10-06	78.51	78.8661	77.2401	77.33	8281	{}	\N	2020-10-12 16:05:22.41722
69171	SHE	2020-10-07	78.479	78.96	78.46	78.86	5760	{}	\N	2020-10-12 16:05:22.422958
69172	SHE	2020-10-08	79.13	79.49	78.97	79.49	23877	{}	\N	2020-10-12 16:05:22.428271
69173	SHE	2020-10-09	79.97	80.34	79.9273	80.34	6956	{}	\N	2020-10-12 16:05:22.432723
69174	SHE	2020-10-12	80.56	80.89	80.56	80.87	3596	{}	\N	2020-10-12 16:05:22.437058
69175	SHIP	2020-09-28	0.45	0.4775	0.4401	0.4572	5792742	{}	\N	2020-10-12 16:05:23.39774
69176	SHIP	2020-09-29	0.4593	0.4675	0.435	0.4483	2559077	{}	\N	2020-10-12 16:05:23.410056
69177	SHIP	2020-09-30	0.4412	0.4694	0.435	0.4567	3209324	{}	\N	2020-10-12 16:05:23.417402
69178	SHIP	2020-10-01	0.461	0.4848	0.453	0.4781	5941924	{}	\N	2020-10-12 16:05:23.424037
69179	SHIP	2020-10-02	0.441	0.4728	0.4406	0.47	3240796	{}	\N	2020-10-12 16:05:23.429946
69180	SHIP	2020-10-05	0.4646	0.47	0.45	0.4671	2947957	{}	\N	2020-10-12 16:05:23.435298
69181	SHIP	2020-10-06	0.46	0.5099	0.46	0.4765	7355843	{}	\N	2020-10-12 16:05:23.440214
69182	SHIP	2020-10-07	0.495	0.52	0.485	0.5	5932724	{}	\N	2020-10-12 16:05:23.445083
69183	SHIP	2020-10-08	0.5107	0.5148	0.485	0.4901	3205676	{}	\N	2020-10-12 16:05:23.450256
69184	SHIP	2020-10-09	0.495	0.52	0.4875	0.5141	8077996	{}	\N	2020-10-12 16:05:23.455694
69185	SHIP	2020-10-12	0.52	0.53	0.5	0.5125	3127378	{}	\N	2020-10-12 16:05:23.46106
69186	SCHV	2020-09-28	51.92	52.41	51.92	52.18	436091	{}	\N	2020-10-12 16:05:25.795206
69187	SCHV	2020-09-29	52.18	52.26	51.65	51.81	1088799	{}	\N	2020-10-12 16:05:25.809358
69188	SCHV	2020-09-30	51.93	52.6337	51.85	52.19	763817	{}	\N	2020-10-12 16:05:25.81701
69189	SCHV	2020-10-01	52.4	52.59	51.96	52.28	294073	{}	\N	2020-10-12 16:05:25.823212
69190	SCHV	2020-10-02	51.59	52.5911	51.5508	52.36	283382	{}	\N	2020-10-12 16:05:25.828683
69191	SCHV	2020-10-05	52.65	53.19	52.65	53.16	371977	{}	\N	2020-10-12 16:05:25.834124
69192	SCHV	2020-10-06	53.28	53.72	52.57	52.66	634378	{}	\N	2020-10-12 16:05:25.839034
69193	SCHV	2020-10-07	53.2167	53.64	53.1801	53.5	1630949	{}	\N	2020-10-12 16:05:25.843951
69194	SCHV	2020-10-08	53.73	54.11	53.71	54.11	300291	{}	\N	2020-10-12 16:05:25.848825
69195	SCHV	2020-10-09	54.35	54.51	54.08	54.22	329038	{}	\N	2020-10-12 16:05:25.853377
69196	SCHV	2020-10-12	54.35	54.65	54.2901	54.59	102889	{}	\N	2020-10-12 16:05:25.857912
69197	SCM	2020-09-28	8.36	9	8.36	8.81	196505	{}	\N	2020-10-12 16:05:27.946058
69198	SCM	2020-09-29	8.89	8.89	8.5664	8.85	105897	{}	\N	2020-10-12 16:05:27.961405
69199	SCM	2020-09-30	8.97	9.02	8.7	8.7	347191	{}	\N	2020-10-12 16:05:27.96956
69200	SCM	2020-10-01	8.71	8.9	8.71	8.84	137016	{}	\N	2020-10-12 16:05:27.975944
69201	SCM	2020-10-02	8.74	8.92	8.74	8.88	117972	{}	\N	2020-10-12 16:05:27.981511
69202	SCM	2020-10-05	8.97	9.04	8.87	8.94	90618	{}	\N	2020-10-12 16:05:27.986871
69203	SCM	2020-10-06	8.95	9.03	8.85	8.9	101657	{}	\N	2020-10-12 16:05:27.99158
69204	SCM	2020-10-07	8.93	9.015	8.81	8.89	195124	{}	\N	2020-10-12 16:05:27.996385
69205	SCM	2020-10-08	8.95	9	8.9175	8.99	99984	{}	\N	2020-10-12 16:05:28.001314
69206	SCM	2020-10-09	9.02	9.0689	8.9044	8.95	122196	{}	\N	2020-10-12 16:05:28.006282
69207	SCM	2020-10-12	9	9.02	8.75	8.82	78488	{}	\N	2020-10-12 16:05:28.011565
69208	SDIV	2020-09-28	10.7	10.74	10.68	10.72	324015	{}	\N	2020-10-12 16:05:29.54801
69209	SDIV	2020-09-29	10.73	10.73	10.63	10.7	264128	{}	\N	2020-10-12 16:05:29.557735
69210	SDIV	2020-09-30	10.72	10.805	10.72	10.76	495623	{}	\N	2020-10-12 16:05:29.565236
69211	SDIV	2020-10-01	10.79	10.81	10.7153	10.81	409871	{}	\N	2020-10-12 16:05:29.571887
69212	SDIV	2020-10-02	10.7	10.87	10.7	10.85	331144	{}	\N	2020-10-12 16:05:29.578034
69213	SDIV	2020-10-05	10.82	10.85	10.8	10.84	213824	{}	\N	2020-10-12 16:05:29.583037
69214	SDIV	2020-10-06	10.9	10.9697	10.8	10.82	508716	{}	\N	2020-10-12 16:05:29.587999
69215	SDIV	2020-10-07	10.86	10.89	10.81	10.85	183337	{}	\N	2020-10-12 16:05:29.59287
69216	SDIV	2020-10-08	10.89	11.04	10.89	11.02	303892	{}	\N	2020-10-12 16:05:29.597851
69217	SDIV	2020-10-09	11.08	11.11	11.05	11.08	626502	{}	\N	2020-10-12 16:05:29.603225
69218	SDIV	2020-10-12	11.08	11.0992	11.05	11.0712	132866	{}	\N	2020-10-12 16:05:29.60851
69219	SDVY	2020-09-28	18.17	18.17	18.17	18.17	452	{}	\N	2020-10-12 16:05:31.770629
69220	SDVY	2020-09-29	18.13	18.2502	18.13	18.25	24569	{}	\N	2020-10-12 16:05:31.783962
69221	SDVY	2020-09-30	18.4184	18.4928	18.2838	18.2838	445	{}	\N	2020-10-12 16:05:31.791779
69222	SDVY	2020-10-02	18.4639	18.7322	18.4639	18.7322	3470	{}	\N	2020-10-12 16:05:31.797915
69223	SDVY	2020-10-05	19.019	19.1312	19.0171	19.1312	3101	{}	\N	2020-10-12 16:05:31.803909
69224	SDVY	2020-10-06	19.3899	19.39	18.99	19.05	28374	{}	\N	2020-10-12 16:05:31.808066
69225	SDVY	2020-10-08	19.48	19.6157	19.48	19.6157	1038	{}	\N	2020-10-12 16:05:31.812175
69226	SDVY	2020-10-09	19.5635	19.5945	19.5635	19.5945	168	{}	\N	2020-10-12 16:05:31.816669
69227	SDVY	2020-10-12	19.62	19.62	19.62	19.62	100	{}	\N	2020-10-12 16:05:31.821227
69228	SEF	2020-09-28	19.36	19.36	19.01	19.15	53328	{}	\N	2020-10-12 16:05:34.500943
69229	SEF	2020-09-29	19.23	19.37	19.179	19.3	15327	{}	\N	2020-10-12 16:05:34.510093
69230	SEF	2020-09-30	19.1451	19.3027	19.02	19.14	46274	{}	\N	2020-10-12 16:05:34.517192
69231	SEF	2020-10-01	18.9457	19.17	18.92	19	41555	{}	\N	2020-10-12 16:05:34.523783
69232	SEF	2020-10-02	19.33	19.33	18.7845	18.84	58817	{}	\N	2020-10-12 16:05:34.529104
69233	SEF	2020-10-05	18.6	18.77	18.565	18.58	36493	{}	\N	2020-10-12 16:05:34.534794
69234	SEF	2020-10-06	18.45	18.81	18.38	18.76	35881	{}	\N	2020-10-12 16:05:34.540208
69235	SEF	2020-10-07	18.499	18.62	18.45	18.495	23638	{}	\N	2020-10-12 16:05:34.545423
69236	SEF	2020-10-08	18.3873	18.4	18.26	18.27	26983	{}	\N	2020-10-12 16:05:34.551299
69237	SEF	2020-10-09	18.15	18.34	18.11	18.2423	32096	{}	\N	2020-10-12 16:05:34.557408
69238	SEF	2020-10-12	18.23	18.24	18.08	18.125	59100	{}	\N	2020-10-12 16:05:34.563899
69239	SFL	2020-09-28	7.61	7.71	7.51	7.55	476319	{}	\N	2020-10-12 16:05:37.540145
69240	SFL	2020-09-29	7.5	7.53	7.313	7.36	1246041	{}	\N	2020-10-12 16:05:37.550421
69241	SFL	2020-09-30	7.45	7.775	7.4	7.49	1574103	{}	\N	2020-10-12 16:05:37.557725
69242	SFL	2020-10-01	7.44	7.78	7.44	7.68	1800559	{}	\N	2020-10-12 16:05:37.563747
69243	SFL	2020-10-02	7.55	7.67	7.44	7.59	1425267	{}	\N	2020-10-12 16:05:37.569403
69244	SFL	2020-10-05	7.59	7.95	7.59	7.88	1122276	{}	\N	2020-10-12 16:05:37.575371
69245	SFL	2020-10-06	8	8.06	7.76	7.77	758014	{}	\N	2020-10-12 16:05:37.581597
69246	SFL	2020-10-07	7.86	7.86	7.64	7.74	810733	{}	\N	2020-10-12 16:05:37.586982
69247	SFL	2020-10-08	7.78	8.01	7.78	7.95	762314	{}	\N	2020-10-12 16:05:37.591991
69248	SFL	2020-10-09	8.02	8.06	7.75	7.84	670336	{}	\N	2020-10-12 16:05:37.59699
69249	SFL	2020-10-12	7.8	7.84	7.625	7.6264	293478	{}	\N	2020-10-12 16:05:37.601858
69250	SHM	2020-09-28	49.91	49.94	49.88	49.89	258531	{}	\N	2020-10-12 16:05:40.263626
69251	SHM	2020-09-29	49.88	49.93	49.88	49.91	198328	{}	\N	2020-10-12 16:05:40.272901
69252	SHM	2020-09-30	49.92	49.9337	49.86	49.87	337414	{}	\N	2020-10-12 16:05:40.278461
69253	SHM	2020-10-01	49.82	49.88	49.82	49.84	325431	{}	\N	2020-10-12 16:05:40.283293
69254	SHM	2020-10-02	49.85	49.86	49.79	49.81	512442	{}	\N	2020-10-12 16:05:40.287819
69255	SHM	2020-10-05	49.84	49.8599	49.79	49.81	295154	{}	\N	2020-10-12 16:05:40.292571
69256	SHM	2020-10-06	49.85	49.87	49.81	49.84	344764	{}	\N	2020-10-12 16:05:40.29718
69257	SHM	2020-10-07	49.834	49.85	49.81	49.82	314852	{}	\N	2020-10-12 16:05:40.301182
69258	SHM	2020-10-08	49.81	49.8431	49.8	49.825	402849	{}	\N	2020-10-12 16:05:40.305304
69259	SHM	2020-10-09	49.81	49.84	49.777	49.83	297538	{}	\N	2020-10-12 16:05:40.309313
69260	SHM	2020-10-12	49.8148	49.8348	49.78	49.805	43490	{}	\N	2020-10-12 16:05:40.313202
69261	SDEM	2020-09-28	9.8	9.82	9.7101	9.78	11976	{}	\N	2020-10-12 16:05:42.047089
69262	SDEM	2020-09-29	9.7345	9.79	9.71	9.77	16915	{}	\N	2020-10-12 16:05:42.057048
69263	SDEM	2020-09-30	9.69	9.83	9.69	9.76	8242	{}	\N	2020-10-12 16:05:42.063752
69264	SDEM	2020-10-01	9.9	9.9	9.83	9.88	4107	{}	\N	2020-10-12 16:05:42.069213
69265	SDEM	2020-10-02	9.81	9.85	9.8	9.81	12277	{}	\N	2020-10-12 16:05:42.073994
69266	SDEM	2020-10-05	9.75	9.77	9.7216	9.76	8557	{}	\N	2020-10-12 16:05:42.078974
69267	SDEM	2020-10-06	9.81	9.83	9.72	9.73	9080	{}	\N	2020-10-12 16:05:42.083346
69268	SDEM	2020-10-07	9.82	9.82	9.7001	9.71	14505	{}	\N	2020-10-12 16:05:42.087643
69269	SDEM	2020-10-08	9.71	9.86	9.71	9.86	6652	{}	\N	2020-10-12 16:05:42.091841
69270	SDEM	2020-10-09	9.87	9.94	9.87	9.92	6091	{}	\N	2020-10-12 16:05:42.096075
69271	SDEM	2020-10-12	9.89	9.949	9.89	9.93	4354	{}	\N	2020-10-12 16:05:42.100256
69272	SDY	2020-09-28	92.08	93.1366	92.08	92.63	757075	{}	\N	2020-10-12 16:05:43.801326
69273	SDY	2020-09-29	92.504	92.63	91.291	91.8	316004	{}	\N	2020-10-12 16:05:43.812175
69274	SDY	2020-09-30	92.08	93.17	91.6801	92.4	524022	{}	\N	2020-10-12 16:05:43.819342
69275	SDY	2020-10-01	92.64	93.07	91.755	92.42	560690	{}	\N	2020-10-12 16:05:43.826758
69276	SDY	2020-10-02	91.22	93.88	91.2	93.43	519512	{}	\N	2020-10-12 16:05:43.832684
69277	SDY	2020-10-05	94.06	95.11	94.06	94.99	525019	{}	\N	2020-10-12 16:05:43.837738
69278	SDY	2020-10-06	95.35	96.6299	94.44	94.57	791346	{}	\N	2020-10-12 16:05:43.842419
69279	SDY	2020-10-07	95.29	96.18	95.2	95.76	438177	{}	\N	2020-10-12 16:05:43.846825
69280	SDY	2020-10-08	96.97	97.72	96.91	97.69	541832	{}	\N	2020-10-12 16:05:43.851175
69281	SDY	2020-10-09	98.23	98.47	97.43	97.61	476567	{}	\N	2020-10-12 16:05:43.855986
69282	SDY	2020-10-12	97.83	98.47	97.66	98.31	242795	{}	\N	2020-10-12 16:05:43.86082
69283	SECO	2020-09-28	2.4	2.65	2.39	2.63	226648	{}	\N	2020-10-12 16:05:44.923414
69284	SECO	2020-09-29	2.61	2.61	2.47	2.51	144087	{}	\N	2020-10-12 16:05:44.935211
69285	SECO	2020-09-30	2.5364	2.67	2.5	2.67	101798	{}	\N	2020-10-12 16:05:44.942271
69286	SECO	2020-10-01	2.71	2.72	2.57	2.65	133504	{}	\N	2020-10-12 16:05:44.94899
69287	SECO	2020-10-02	2.6	2.7	2.6	2.68	45285	{}	\N	2020-10-12 16:05:44.954916
69288	SECO	2020-10-05	2.67	2.72	2.65	2.68	97111	{}	\N	2020-10-12 16:05:44.960229
69289	SECO	2020-10-06	2.67	2.75	2.66	2.7	96224	{}	\N	2020-10-12 16:05:44.965302
69290	SECO	2020-10-07	2.7	2.75	2.67	2.72	29861	{}	\N	2020-10-12 16:05:44.970226
69291	SECO	2020-10-08	2.72	2.8	2.72	2.78	70153	{}	\N	2020-10-12 16:05:44.975524
69292	SECO	2020-10-09	2.81	2.81	2.69	2.76	19226	{}	\N	2020-10-12 16:05:44.980513
69293	SECO	2020-10-12	2.75	2.77	2.7	2.75	35124	{}	\N	2020-10-12 16:05:44.985519
69294	SF	2020-09-28	50.7	51.67	50.7	50.93	251445	{}	\N	2020-10-12 16:05:48.304261
69295	SF	2020-09-29	50.92	50.95	49.58	50.13	531285	{}	\N	2020-10-12 16:05:48.319579
69296	SF	2020-09-30	50.56	51.82	50.04	50.56	621757	{}	\N	2020-10-12 16:05:48.325975
69297	SF	2020-10-01	51.39	51.62	50.31	51.44	822765	{}	\N	2020-10-12 16:05:48.331692
69298	SF	2020-10-02	50.33	53.75	50.2	53.33	734994	{}	\N	2020-10-12 16:05:48.3372
69299	SF	2020-10-05	53.92	54.865	53.92	54.52	638956	{}	\N	2020-10-12 16:05:48.341818
69300	SF	2020-10-06	55.39	55.97	53.96	54.04	820644	{}	\N	2020-10-12 16:05:48.345849
69301	SF	2020-10-07	54.7	55.74	54.62	55.35	634850	{}	\N	2020-10-12 16:05:48.349752
69302	SF	2020-10-08	56.12	56.83	55.77	56.52	599972	{}	\N	2020-10-12 16:05:48.353789
69303	SF	2020-10-09	57.26	57.59	56.24	56.45	552830	{}	\N	2020-10-12 16:05:48.358339
69304	SF	2020-10-12	56.64	58.07	56.42	57.72	157164	{}	\N	2020-10-12 16:05:48.362519
69305	SFIX	2020-09-28	26.91	27.57	26.54	27.5	2925352	{}	\N	2020-10-12 16:05:49.322332
69306	SFIX	2020-09-29	27.21	27.75	26.5	27.045	4053198	{}	\N	2020-10-12 16:05:49.332584
69307	SFIX	2020-09-30	27.53	28.65	26.96	27.13	2723897	{}	\N	2020-10-12 16:05:49.339124
69308	SFIX	2020-10-01	27.4	28.32	27.07	28.29	1710702	{}	\N	2020-10-12 16:05:49.344697
69309	SFIX	2020-10-02	27.3	27.77	26.94	27.57	2350767	{}	\N	2020-10-12 16:05:49.349966
69310	SFIX	2020-10-05	27.58	28.98	27.36	28.71	2303836	{}	\N	2020-10-12 16:05:49.357505
69311	SFIX	2020-10-06	29.03	29.29	27.92	28.19	1770607	{}	\N	2020-10-12 16:05:49.362225
69312	SFIX	2020-10-07	28.52	29.68	28.37	29.59	2023864	{}	\N	2020-10-12 16:05:49.367252
69313	SFIX	2020-10-08	29.9	31.95	29.65	31.21	4388648	{}	\N	2020-10-12 16:05:49.371851
69314	SFIX	2020-10-09	31.7413	32.3399	29	31.48	4833529	{}	\N	2020-10-12 16:05:49.376577
69315	SFIX	2020-10-12	31.73	32.87	31.5301	31.93	1080469	{}	\N	2020-10-12 16:05:49.38118
69316	SGRP	2020-09-28	0.6802	0.7525	0.6802	0.73	43023	{}	\N	2020-10-12 16:05:51.019442
69317	SGRP	2020-09-29	0.73	0.76	0.7005	0.7588	74414	{}	\N	2020-10-12 16:05:51.035085
69318	SGRP	2020-09-30	0.76	0.78	0.76	0.761	59719	{}	\N	2020-10-12 16:05:51.04322
69319	SGRP	2020-10-01	0.77	0.7899	0.7494	0.7749	10620	{}	\N	2020-10-12 16:05:51.050128
69320	SGRP	2020-10-02	0.71	0.799	0.7	0.7989	61851	{}	\N	2020-10-12 16:05:51.056578
69321	SGRP	2020-10-05	0.7749	0.81	0.7716	0.7986	61564	{}	\N	2020-10-12 16:05:51.062497
69322	SGRP	2020-10-06	0.81	0.8385	0.773	0.788	76918	{}	\N	2020-10-12 16:05:51.067746
69323	SGRP	2020-10-07	0.7869	0.8364	0.7801	0.819	71249	{}	\N	2020-10-12 16:05:51.072943
69324	SGRP	2020-10-08	0.8001	0.82	0.781	0.818	18075	{}	\N	2020-10-12 16:05:51.078115
69325	SGRP	2020-10-09	0.819899	0.819899	0.7831	0.8179	7659	{}	\N	2020-10-12 16:05:51.083602
69326	SGRP	2020-10-12	0.7872	0.815	0.7871	0.788	3469	{}	\N	2020-10-12 16:05:51.08896
69327	SHI	2020-09-28	17.62	17.9	17.62	17.77	50209	{}	\N	2020-10-12 16:05:52.728096
69328	SHI	2020-09-29	17.89	18.09	17.8583	18.01	90373	{}	\N	2020-10-12 16:05:52.74214
69329	SHI	2020-09-30	18.18	18.4196	18.18	18.36	124889	{}	\N	2020-10-12 16:05:52.750553
69330	SHI	2020-10-01	18.44	18.53	18.29	18.53	72166	{}	\N	2020-10-12 16:05:52.756792
69331	SHI	2020-10-02	18.32	18.54	18.31	18.31	58381	{}	\N	2020-10-12 16:05:52.763405
69332	SHI	2020-10-05	18.36	18.42	18.31	18.37	58788	{}	\N	2020-10-12 16:05:52.76918
69333	SHI	2020-10-06	18.5	18.97	18.5	18.77	142215	{}	\N	2020-10-12 16:05:52.774954
69334	SHI	2020-10-07	18.88	18.9	18.65	18.9	66648	{}	\N	2020-10-12 16:05:52.780342
69335	SHI	2020-10-08	18.98	18.99	18.71	18.99	66371	{}	\N	2020-10-12 16:05:52.785823
69336	SHI	2020-10-09	19.01	19.17	18.77	19.11	71144	{}	\N	2020-10-12 16:05:52.791344
69337	SHI	2020-10-12	19.82	19.94	19.62	19.79	26642	{}	\N	2020-10-12 16:05:52.796828
69338	SHYG	2020-09-28	43.74	43.84	43.6782	43.72	3783105	{}	\N	2020-10-12 16:05:54.915043
69339	SHYG	2020-09-29	43.78	43.835	43.65	43.78	1216667	{}	\N	2020-10-12 16:05:54.928679
69340	SHYG	2020-09-30	43.82	44.02	43.82	43.95	993135	{}	\N	2020-10-12 16:05:54.937572
69341	SHYG	2020-10-01	43.95	43.96	43.765	43.82	1069603	{}	\N	2020-10-12 16:05:54.944714
69342	SHYG	2020-10-02	43.68	43.86	43.68	43.81	1395065	{}	\N	2020-10-12 16:05:54.951019
69343	SHYG	2020-10-05	43.89	44.07	43.89	44.06	1454384	{}	\N	2020-10-12 16:05:54.95645
69344	SHYG	2020-10-06	44.12	44.22	43.95	43.98	1181349	{}	\N	2020-10-12 16:05:54.961621
69345	SHYG	2020-10-07	44.13	44.14	44.0524	44.1	2598121	{}	\N	2020-10-12 16:05:54.965941
69346	SHYG	2020-10-08	44.17	44.24	44.13	44.24	2346692	{}	\N	2020-10-12 16:05:54.970414
69347	SHYG	2020-10-09	44.26	44.35	44.2019	44.32	1585690	{}	\N	2020-10-12 16:05:54.974895
69348	SHYG	2020-10-12	44.399	44.48	44.375	44.465	259986	{}	\N	2020-10-12 16:05:54.979226
69349	SCL	2020-09-28	108.44	111.06	108.44	110.03	73373	{}	\N	2020-10-12 16:05:56.085971
69350	SCL	2020-09-29	109.86	110.46	108.22	108.51	144914	{}	\N	2020-10-12 16:05:56.101116
69351	SCL	2020-09-30	108.74	111.21	108.15	109	206309	{}	\N	2020-10-12 16:05:56.109435
69352	SCL	2020-10-01	108.95	109.28	107.01	107.66	142553	{}	\N	2020-10-12 16:05:56.115563
69353	SCL	2020-10-02	106.31	109.34	106.31	108.16	153257	{}	\N	2020-10-12 16:05:56.120966
69354	SCL	2020-10-05	108.97	112.205	108.97	110.96	132058	{}	\N	2020-10-12 16:05:56.126303
69355	SCL	2020-10-06	112.26	113.745	111.32	111.76	176338	{}	\N	2020-10-12 16:05:56.130817
69356	SCL	2020-10-07	112.87	113.5	111.03	112.36	199410	{}	\N	2020-10-12 16:05:56.135932
69357	SCL	2020-10-08	113.54	114.6	112.885	114.08	119574	{}	\N	2020-10-12 16:05:56.140835
69358	SCL	2020-10-09	115.15	116.11	114.25	115.45	108785	{}	\N	2020-10-12 16:05:56.145524
69359	SCL	2020-10-12	115.11	116.81	114.363	116.33	15547	{}	\N	2020-10-12 16:05:56.149968
69360	SDD	2020-09-28	9.31	9.31	8.98	9.06	14368	{}	\N	2020-10-12 16:05:57.08714
69361	SDD	2020-09-29	9.2325	9.31	9.083	9.16	6572	{}	\N	2020-10-12 16:05:57.097172
69362	SDD	2020-09-30	8.87	9.19	8.84	9.09	5433	{}	\N	2020-10-12 16:05:57.103769
69363	SDD	2020-10-01	8.98	9.11	8.85	8.85	3858	{}	\N	2020-10-12 16:05:57.10927
69364	SDD	2020-10-02	9.16	9.16	8.598	8.67	13689	{}	\N	2020-10-12 16:05:57.11434
69365	SDD	2020-10-05	8.41	8.42	8.2494	8.28	33423	{}	\N	2020-10-12 16:05:57.119443
69366	SDD	2020-10-06	8.17	8.29	7.88	8.27	17279	{}	\N	2020-10-12 16:05:57.124035
69367	SDD	2020-10-07	8.05	8.1505	7.99	8	7822	{}	\N	2020-10-12 16:05:57.128683
69368	SDD	2020-10-08	7.9	7.97	7.79	7.79	14281	{}	\N	2020-10-12 16:05:57.132964
69369	SDD	2020-10-09	7.64	7.7897	7.6	7.74	19415	{}	\N	2020-10-12 16:05:57.13794
69370	SDD	2020-10-12	7.7	7.705	7.64	7.705	5543	{}	\N	2020-10-12 16:05:57.143104
69371	SDS	2020-09-28	16.12	16.23	15.94	16.03	10453779	{}	\N	2020-10-12 16:05:58.233367
69372	SDS	2020-09-29	16.051	16.27	15.97	16.19	7093707	{}	\N	2020-10-12 16:05:58.246685
69373	SDS	2020-09-30	16.13	16.15	15.62	15.95	21090060	{}	\N	2020-10-12 16:05:58.255101
69374	SDS	2020-10-01	15.68	15.94	15.6	15.76	7394987	{}	\N	2020-10-12 16:05:58.261049
69375	SDS	2020-10-02	16.2499	16.2925	15.85	16.04	13776843	{}	\N	2020-10-12 16:05:58.267044
69376	SDS	2020-10-05	15.831	15.831	15.46	15.48	7181154	{}	\N	2020-10-12 16:05:58.272413
69377	SDS	2020-10-06	15.45	15.96	15.26	15.92	15990381	{}	\N	2020-10-12 16:05:58.277209
69378	SDS	2020-10-07	15.62	15.62	15.28	15.36	7178679	{}	\N	2020-10-12 16:05:58.281874
69379	SDS	2020-10-08	15.1785	15.26	15.08	15.1	7990947	{}	\N	2020-10-12 16:05:58.286376
69380	SDS	2020-10-09	14.94	14.99	14.7701	14.83	9300597	{}	\N	2020-10-12 16:05:58.29097
69381	SDS	2020-10-12	14.5804	14.63	14.36	14.4	3819510	{}	\N	2020-10-12 16:05:58.295409
69382	SEAC	2020-09-28	0.88	0.94	0.87	0.9265	176906	{}	\N	2020-10-12 16:06:00.126836
69383	SEAC	2020-09-29	0.939	0.9455	0.8901	0.8975	125488	{}	\N	2020-10-12 16:06:00.140045
69384	SEAC	2020-09-30	0.89	0.8945	0.87	0.8704	245808	{}	\N	2020-10-12 16:06:00.148099
69385	SEAC	2020-10-01	0.879	0.8899	0.7803	0.7901	925490	{}	\N	2020-10-12 16:06:00.154571
69386	SEAC	2020-10-02	0.76	0.849	0.76	0.8222	639064	{}	\N	2020-10-12 16:06:00.15987
69387	SEAC	2020-10-05	0.8362	0.87	0.82	0.8598	166131	{}	\N	2020-10-12 16:06:00.16528
69388	SEAC	2020-10-06	0.88	0.9311	0.85961	0.9178	194674	{}	\N	2020-10-12 16:06:00.170061
69389	SEAC	2020-10-07	0.93	0.94	0.8528	0.861	354915	{}	\N	2020-10-12 16:06:00.174761
69390	SEAC	2020-10-08	0.8594	0.921	0.8557	0.9109	317199	{}	\N	2020-10-12 16:06:00.179201
69391	SEAC	2020-10-09	0.91	0.94	0.8908	0.9062	306630	{}	\N	2020-10-12 16:06:00.183886
69392	SEAC	2020-10-12	0.8852	0.91	0.87	0.877199	29549	{}	\N	2020-10-12 16:06:00.188968
69393	SEE	2020-09-28	38.36	38.66	37.84	37.89	961205	{}	\N	2020-10-12 16:06:01.096842
69394	SEE	2020-09-29	37.85	37.99	37.245	37.83	2082872	{}	\N	2020-10-12 16:06:01.108848
69395	SEE	2020-09-30	38.07	39.175	37.83	38.81	4168700	{}	\N	2020-10-12 16:06:01.116477
69396	SEE	2020-10-01	39.1	40.03	38.83	39.93	3719393	{}	\N	2020-10-12 16:06:01.122654
69397	SEE	2020-10-02	39.2	40.69	39.2	40.35	2490316	{}	\N	2020-10-12 16:06:01.128477
69398	SEE	2020-10-05	40.72	41.74	40.675	41.04	2511833	{}	\N	2020-10-12 16:06:01.133215
69399	SEE	2020-10-06	41.37	42.47	41.13	41.8	2864204	{}	\N	2020-10-12 16:06:01.137843
69400	SEE	2020-10-07	42.12	43.02	41.66	41.96	3682611	{}	\N	2020-10-12 16:06:01.142485
69401	SEE	2020-10-08	42.08	42.92	41.92	42.87	1366060	{}	\N	2020-10-12 16:06:01.147225
69402	SEE	2020-10-09	43.25	43.37	42.465	42.68	1669253	{}	\N	2020-10-12 16:06:01.151944
69403	SEE	2020-10-12	42.98	42.98	42.18	42.47	1798060	{}	\N	2020-10-12 16:06:01.15658
69404	SENEB	2020-09-28	36.99	37.28	36.99	37.28	1494	{}	\N	2020-10-12 16:06:01.961172
69405	SENEB	2020-09-29	35.5	36	35.04	35.04	717	{}	\N	2020-10-12 16:06:01.969525
69406	SENEB	2020-10-05	37.36	37.36	37.36	37.36	269	{}	\N	2020-10-12 16:06:01.975841
69407	SFE	2020-09-28	5.27	5.53	5.27	5.34	66501	{}	\N	2020-10-12 16:06:03.869275
69408	SFE	2020-09-29	5.32	5.32	5.15	5.27	71669	{}	\N	2020-10-12 16:06:03.882413
69409	SFE	2020-09-30	5.3	5.6	5.3	5.48	199635	{}	\N	2020-10-12 16:06:03.889603
69410	SFE	2020-10-01	5.49	5.53	5.34	5.46	96532	{}	\N	2020-10-12 16:06:03.895712
69411	SFE	2020-10-02	5.36	5.62	5.33	5.55	67618	{}	\N	2020-10-12 16:06:03.900992
69412	SFE	2020-10-05	5.61	5.84	5.59	5.77	147165	{}	\N	2020-10-12 16:06:03.906257
69413	SFE	2020-10-06	5.85	5.85	5.62	5.65	351200	{}	\N	2020-10-12 16:06:03.910747
69414	SFE	2020-10-07	5.71	5.81	5.65	5.77	92815	{}	\N	2020-10-12 16:06:03.91521
69415	SFE	2020-10-08	5.77	5.97	5.745	5.94	85336	{}	\N	2020-10-12 16:06:03.919685
69416	SFE	2020-10-09	6	6.01	5.9	5.98	181223	{}	\N	2020-10-12 16:06:03.924311
69417	SFE	2020-10-12	6.04	6.06	5.9713	6.06	51167	{}	\N	2020-10-12 16:06:03.928854
69418	SGEN	2020-09-28	186.32	190.7	185.42	190.51	835087	{}	\N	2020-10-12 16:06:04.816886
69419	SGEN	2020-09-29	190.5	192.3	187	190.96	1234185	{}	\N	2020-10-12 16:06:04.82944
69420	SGEN	2020-09-30	192	198	191.32	195.69	1473335	{}	\N	2020-10-12 16:06:04.83837
69421	SGEN	2020-10-01	197.18	200.42	196.78	197.89	1168108	{}	\N	2020-10-12 16:06:04.844107
69422	SGEN	2020-10-02	194.87	199	191.57	193.08	723665	{}	\N	2020-10-12 16:06:04.849823
69423	SGEN	2020-10-05	194.43	204.3717	194.195	202.48	1067080	{}	\N	2020-10-12 16:06:04.854786
69424	SGEN	2020-10-06	201.41	203.73	199.17	199.61	723458	{}	\N	2020-10-12 16:06:04.858822
69425	SGEN	2020-10-07	201.91	202.84	200.63	201.03	445289	{}	\N	2020-10-12 16:06:04.862781
69426	SGEN	2020-10-08	201.83	201.83	197.26	198.73	796228	{}	\N	2020-10-12 16:06:04.866783
69427	SGEN	2020-10-09	199.62	202.23	198	200.78	647524	{}	\N	2020-10-12 16:06:04.870616
69428	SGEN	2020-10-12	204.11	204.83	202.03	203.56	216896	{}	\N	2020-10-12 16:06:04.874546
69429	SGRY	2020-09-28	19.8	20.385	19.51	19.89	348454	{}	\N	2020-10-12 16:06:06.191131
69430	SGRY	2020-09-29	19.92	20.27	19.54	19.63	333301	{}	\N	2020-10-12 16:06:06.205551
69431	SGRY	2020-09-30	19.68	23.07	19.68	21.9	971958	{}	\N	2020-10-12 16:06:06.213566
69432	SGRY	2020-10-01	21.9	22.78	21.131	22.72	490078	{}	\N	2020-10-12 16:06:06.220133
69433	SGRY	2020-10-02	22.06	24.03	21.8	23.35	321402	{}	\N	2020-10-12 16:06:06.226256
69434	SGRY	2020-10-05	24	24.93	23.5	24.14	399019	{}	\N	2020-10-12 16:06:06.231546
69435	SGRY	2020-10-06	24.48	24.99	23.25	23.38	331219	{}	\N	2020-10-12 16:06:06.236724
69436	SGRY	2020-10-07	23.76	24.2737	23.09	23.29	495847	{}	\N	2020-10-12 16:06:06.242172
69437	SGRY	2020-10-08	23.55	23.6	22.83	22.91	248686	{}	\N	2020-10-12 16:06:06.247163
69438	SGRY	2020-10-09	23.04	23.82	23	23.12	323457	{}	\N	2020-10-12 16:06:06.252288
69439	SGRY	2020-10-12	23.06	23.38	22.41	22.99	172637	{}	\N	2020-10-12 16:06:06.257591
69440	SHO	2020-09-28	7.92	8.14	7.83	8.04	1404548	{}	\N	2020-10-12 16:06:07.850064
69441	SHO	2020-09-29	8.33	8.33	7.835	7.96	3221353	{}	\N	2020-10-12 16:06:07.865575
69442	SHO	2020-09-30	7.98	8.17	7.81	7.94	6676869	{}	\N	2020-10-12 16:06:07.873604
69443	SHO	2020-10-01	8.02	8.09	7.83	8.08	5647410	{}	\N	2020-10-12 16:06:07.880758
69444	SHO	2020-10-02	7.82	8.29	7.78	8.25	3829509	{}	\N	2020-10-12 16:06:07.887209
69445	SHO	2020-10-05	8.34	8.37	8.06	8.18	3379430	{}	\N	2020-10-12 16:06:07.892594
69446	SHO	2020-10-06	8.26	8.37	8.115	8.15	5342814	{}	\N	2020-10-12 16:06:07.897855
69447	SHO	2020-10-07	8.21	8.28	8.14	8.23	3313929	{}	\N	2020-10-12 16:06:07.902938
69448	SHO	2020-10-08	8.35	8.37	8.12	8.27	4030636	{}	\N	2020-10-12 16:06:07.908925
69449	SHO	2020-10-09	8.31	8.44	8.135	8.19	4747949	{}	\N	2020-10-12 16:06:07.914399
69450	SHO	2020-10-12	8.15	8.18	8.055	8.11	1322204	{}	\N	2020-10-12 16:06:07.920037
69451	SIEB	2020-09-28	3.31	3.44	3.2841	3.39	17744	{}	\N	2020-10-12 16:06:09.092886
69452	SIEB	2020-09-29	3.3	3.42	3.2411	3.28	19035	{}	\N	2020-10-12 16:06:09.104558
69453	SIEB	2020-09-30	3.33	3.33	3.2	3.23	13190	{}	\N	2020-10-12 16:06:09.112037
69454	SIEB	2020-10-01	3.26	3.28	3.13	3.24	24359	{}	\N	2020-10-12 16:06:09.118609
69455	SIEB	2020-10-02	3.14	3.45	3.14	3.37	18603	{}	\N	2020-10-12 16:06:09.124798
69456	SIEB	2020-10-05	3.45	3.68	3.38	3.61	16946	{}	\N	2020-10-12 16:06:09.130584
69457	SIEB	2020-10-06	3.52	3.69	3.45	3.45	17409	{}	\N	2020-10-12 16:06:09.13596
69458	SIEB	2020-10-07	3.58	3.65	3.47	3.65	10280	{}	\N	2020-10-12 16:06:09.141052
69459	SIEB	2020-10-08	3.72	4.0999	3.64	3.91	21121	{}	\N	2020-10-12 16:06:09.146325
69460	SIEB	2020-10-09	4.02	4.03	3.88	3.99	7731	{}	\N	2020-10-12 16:06:09.151112
69461	SIEB	2020-10-12	3.99	4.09	3.99	4.09	2098	{}	\N	2020-10-12 16:06:09.155806
69462	SIX	2020-09-28	21.51	21.88	21	21.53	1353288	{}	\N	2020-10-12 16:06:10.995932
69463	SIX	2020-09-29	21.41	21.49	20.33	20.51	2532191	{}	\N	2020-10-12 16:06:11.008004
69464	SIX	2020-09-30	20.65	21.39	20.15	20.3	2863227	{}	\N	2020-10-12 16:06:11.014399
69465	SIX	2020-10-01	20.4	20.46	19.595	19.94	5125088	{}	\N	2020-10-12 16:06:11.020558
69466	SIX	2020-10-02	19.25	20.84	19.02	20.74	3240032	{}	\N	2020-10-12 16:06:11.026704
69467	SIX	2020-10-05	21	21.01	19.725	20.16	5258576	{}	\N	2020-10-12 16:06:11.032025
69468	SIX	2020-10-06	20.62	21.6	20.47	20.58	3432017	{}	\N	2020-10-12 16:06:11.037393
69469	SIX	2020-10-07	20.96	21.64	20.8501	21.45	3517434	{}	\N	2020-10-12 16:06:11.042405
69470	SIX	2020-10-08	21.71	21.74	21.24	21.66	2944711	{}	\N	2020-10-12 16:06:11.047287
69471	SIX	2020-10-09	21.67	22.1	21.4801	21.59	1984198	{}	\N	2020-10-12 16:06:11.05216
69472	SIX	2020-10-12	21.27	21.72	21.12	21.27	719250	{}	\N	2020-10-12 16:06:11.057248
69473	SJR	2020-09-28	18.3	18.44	18.24	18.42	461690	{}	\N	2020-10-12 16:06:12.062622
69474	SJR	2020-09-29	18.4	18.5	18.15	18.33	728452	{}	\N	2020-10-12 16:06:12.073809
69475	SJR	2020-09-30	18.35	18.4	18.15	18.25	864241	{}	\N	2020-10-12 16:06:12.081225
69476	SJR	2020-10-01	18.3	18.38	18.14	18.32	980376	{}	\N	2020-10-12 16:06:12.087533
69477	SJR	2020-10-02	18.12	18.42	18.08	18.36	759310	{}	\N	2020-10-12 16:06:12.09401
69478	SJR	2020-10-05	18.51	18.59	18.41	18.55	1319045	{}	\N	2020-10-12 16:06:12.099965
69479	SJR	2020-10-06	18.61	18.61	18.03	18.05	2021287	{}	\N	2020-10-12 16:06:12.105591
69480	SJR	2020-10-07	18.12	18.21	17.98	18.1	1004115	{}	\N	2020-10-12 16:06:12.111057
69481	SJR	2020-10-08	18.18	18.21	18.09	18.16	1133983	{}	\N	2020-10-12 16:06:12.116484
69482	SJR	2020-10-09	18.17	18.22	17.92	17.96	1045844	{}	\N	2020-10-12 16:06:12.121846
69483	SJR	2020-10-12	18.04	18.145	18.04	18.1	157147	{}	\N	2020-10-12 16:06:12.127663
69484	SKT	2020-09-28	5.95	6.22	5.9481	6.09	1601294	{}	\N	2020-10-12 16:06:13.12775
69485	SKT	2020-09-29	6.1	6.132	5.9014	6.08	2855212	{}	\N	2020-10-12 16:06:13.141858
69486	SKT	2020-09-30	6.12	6.305	6.01	6.03	3244653	{}	\N	2020-10-12 16:06:13.149562
69487	SKT	2020-10-01	6.03	6.28	6.03	6.26	2792649	{}	\N	2020-10-12 16:06:13.15613
69488	SKT	2020-10-02	6.04	6.47	5.97	6.44	3918292	{}	\N	2020-10-12 16:06:13.16198
69489	SKT	2020-10-05	6.5	6.63	6.33	6.46	2913528	{}	\N	2020-10-12 16:06:13.167506
69490	SKT	2020-10-06	6.5	6.71	6.41	6.42	3774847	{}	\N	2020-10-12 16:06:13.172519
69491	SKT	2020-10-07	6.45	6.6	6.35	6.37	2749364	{}	\N	2020-10-12 16:06:13.177374
69492	SKT	2020-10-08	6.49	6.82	6.4	6.78	4959149	{}	\N	2020-10-12 16:06:13.182419
69493	SKT	2020-10-09	6.82	6.89	6.58	6.6	3219016	{}	\N	2020-10-12 16:06:13.187263
69494	SKT	2020-10-12	6.55	6.77	6.5	6.64	1225725	{}	\N	2020-10-12 16:06:13.192296
69495	SEB	2020-09-28	2860	2885.99	2755.49	2840.2	611	{}	\N	2020-10-12 16:06:14.29744
69496	SEB	2020-09-29	2908.99	2908.99	2799	2809.48	776	{}	\N	2020-10-12 16:06:14.309289
69497	SEB	2020-09-30	2755.13	2849.97	2755.1201	2836.69	702	{}	\N	2020-10-12 16:06:14.316481
69498	SEB	2020-10-01	2838	2933.94	2838	2933.93	1210	{}	\N	2020-10-12 16:06:14.322658
69499	SEB	2020-10-02	2834	2989.99	2834	2950.39	710	{}	\N	2020-10-12 16:06:14.328304
69500	SEB	2020-10-05	2986.1595	2986.1595	2884.1594	2969.95	687	{}	\N	2020-10-12 16:06:14.33356
69501	SEB	2020-10-06	2984.97	2999.99	2935.705	2975	874	{}	\N	2020-10-12 16:06:14.338667
69502	SEB	2020-10-07	2999.99	3032.77	2944.8801	2954.24	2090	{}	\N	2020-10-12 16:06:14.343546
69503	SEB	2020-10-08	2997	3060	2960.3001	3060	523	{}	\N	2020-10-12 16:06:14.348631
69504	SEB	2020-10-09	3070	3195.51	3067	3195.51	1084	{}	\N	2020-10-12 16:06:14.353583
69505	SEB	2020-10-12	3155.6899	3240	3085.0001	3234.465	260	{}	\N	2020-10-12 16:06:14.358435
69506	SFBS	2020-09-28	33.49	34.21	33.49	33.99	128601	{}	\N	2020-10-12 16:06:15.958339
69507	SFBS	2020-09-29	33.82	33.98	33.259	33.74	95825	{}	\N	2020-10-12 16:06:15.972663
69508	SFBS	2020-09-30	33.87	34.51	33.66	34.03	182063	{}	\N	2020-10-12 16:06:15.979819
69509	SFBS	2020-10-01	34.12	34.81	33.8	34.81	202596	{}	\N	2020-10-12 16:06:15.985561
69510	SFBS	2020-10-02	34.21	35.9	34.21	35.63	116846	{}	\N	2020-10-12 16:06:15.990435
69511	SFBS	2020-10-05	35.85	37.227	35.85	37.15	163156	{}	\N	2020-10-12 16:06:15.996157
69512	SFBS	2020-10-06	37.62	38.66	37.34	37.4	173629	{}	\N	2020-10-12 16:06:16.000742
69513	SFBS	2020-10-07	37.9	38.65	37.72	38.01	186786	{}	\N	2020-10-12 16:06:16.005275
69514	SFBS	2020-10-08	38.35	38.75	37.97	38.51	95818	{}	\N	2020-10-12 16:06:16.009712
69515	SFBS	2020-10-09	38.93	38.99	38.01	38.07	77754	{}	\N	2020-10-12 16:06:16.014157
69516	SFBS	2020-10-12	38.05	38.76	37.874	38.52	22372	{}	\N	2020-10-12 16:06:16.018715
69517	SFST	2020-09-28	23.74	24.2	23.68	23.9	17865	{}	\N	2020-10-12 16:06:18.231782
69518	SFST	2020-09-29	23.62	23.93	23.57	23.86	9623	{}	\N	2020-10-12 16:06:18.242673
69519	SFST	2020-09-30	23.92	24.28	23.425	24.15	18655	{}	\N	2020-10-12 16:06:18.249886
69520	SFST	2020-10-01	24.07	24.26	23.92	24.15	17222	{}	\N	2020-10-12 16:06:18.256543
69521	SFST	2020-10-02	23.85	26.02	23.57	25.98	17983	{}	\N	2020-10-12 16:06:18.26219
69522	SFST	2020-10-05	26.25	26.91	26.25	26.76	15406	{}	\N	2020-10-12 16:06:18.267255
69523	SFST	2020-10-06	27.15	27.21	26.34	26.6	13159	{}	\N	2020-10-12 16:06:18.272423
69524	SFST	2020-10-07	26.925	27.65	26.925	27.6	30996	{}	\N	2020-10-12 16:06:18.277246
69525	SFST	2020-10-08	27.35	27.89	26.98	27.89	25008	{}	\N	2020-10-12 16:06:18.281939
69526	SFST	2020-10-09	28.06	28.06	27.36	27.47	3494	{}	\N	2020-10-12 16:06:18.286527
69527	SFST	2020-10-12	27.6	27.6	27.475	27.475	625	{}	\N	2020-10-12 16:06:18.299593
69528	SGLB	2020-09-28	2.13	2.34	2.05	2.29	575499	{}	\N	2020-10-12 16:06:20.232716
69529	SGLB	2020-09-29	2.57	2.64	2.23	2.37	2909826	{}	\N	2020-10-12 16:06:20.244412
69530	SGLB	2020-09-30	2.25	2.3999	2.17	2.24	510506	{}	\N	2020-10-12 16:06:20.251819
69531	SGLB	2020-10-01	2.28	2.32	2.2	2.21	109152	{}	\N	2020-10-12 16:06:20.258573
69532	SGLB	2020-10-02	2.15	2.27	2.06	2.16	158139	{}	\N	2020-10-12 16:06:20.264861
69533	SGLB	2020-10-05	2.18	2.21	2.1189	2.16	83807	{}	\N	2020-10-12 16:06:20.270391
69534	SGLB	2020-10-06	2.26	2.5	2.21	2.24	1530536	{}	\N	2020-10-12 16:06:20.275277
69535	SGLB	2020-10-07	2.26	2.64	2.17	2.58	1130502	{}	\N	2020-10-12 16:06:20.280204
69536	SGLB	2020-10-08	2.49	2.55	2.3964	2.48	346308	{}	\N	2020-10-12 16:06:20.28498
69537	SGLB	2020-10-09	2.52	2.54	2.4	2.44	128335	{}	\N	2020-10-12 16:06:20.289194
69538	SGLB	2020-10-12	2.45	2.48	2.37	2.42	26683	{}	\N	2020-10-12 16:06:20.293473
69539	SHEN	2020-09-28	43.84	44.655	43.78	44.22	159280	{}	\N	2020-10-12 16:06:21.261843
69540	SHEN	2020-09-29	44.1	44.78	43.83	44.49	189548	{}	\N	2020-10-12 16:06:21.27299
69541	SHEN	2020-09-30	44.77	45.1	44.04	44.435	256478	{}	\N	2020-10-12 16:06:21.279268
69542	SHEN	2020-10-01	44.8	44.84	43.22	44.57	130951	{}	\N	2020-10-12 16:06:21.284538
69543	SHEN	2020-10-02	44.03	44.97	43.66	44.42	149621	{}	\N	2020-10-12 16:06:21.28929
69544	SHEN	2020-10-05	44.72	45.4	44.56	45.21	113806	{}	\N	2020-10-12 16:06:21.29446
69545	SHEN	2020-10-06	45.24	46.18	45.02	45.07	191451	{}	\N	2020-10-12 16:06:21.298559
69546	SHEN	2020-10-07	45.22	45.55	44.16	45.07	176364	{}	\N	2020-10-12 16:06:21.30278
69547	SHEN	2020-10-08	45.58	46.065	45.11	45.93	107441	{}	\N	2020-10-12 16:06:21.306779
69548	SHEN	2020-10-09	46.42	47	46.04	46.99	120410	{}	\N	2020-10-12 16:06:21.310852
69549	SHEN	2020-10-12	46.945	47.463	46.94	47.26	28075	{}	\N	2020-10-12 16:06:21.316297
69550	SHOO	2020-09-28	20.31	20.49	19.58	19.61	540665	{}	\N	2020-10-12 16:06:22.340246
69551	SHOO	2020-09-29	19.52	19.605	18.47	19.12	939192	{}	\N	2020-10-12 16:06:22.353949
69552	SHOO	2020-09-30	19.13	19.69	19.1	19.5	768536	{}	\N	2020-10-12 16:06:22.361404
69553	SHOO	2020-10-01	19.54	20	19.3566	19.84	468721	{}	\N	2020-10-12 16:06:22.368336
69554	SHOO	2020-10-02	19.24	20.46	19.24	20.33	703697	{}	\N	2020-10-12 16:06:22.374066
69555	SHOO	2020-10-05	20.44	20.8	20.175	20.29	984806	{}	\N	2020-10-12 16:06:22.379527
69556	SHOO	2020-10-06	20.54	20.94	19.74	19.94	762505	{}	\N	2020-10-12 16:06:22.384262
69557	SHOO	2020-10-07	20.17	21.0098	20.17	20.9	1065467	{}	\N	2020-10-12 16:06:22.389051
69558	SHOO	2020-10-08	21.2	21.62	21.11	21.45	649635	{}	\N	2020-10-12 16:06:22.393672
69559	SHOO	2020-10-09	21.6	21.79	20.83	21	852873	{}	\N	2020-10-12 16:06:22.398886
69560	SHOO	2020-10-12	21.72	22.51	21.3	21.37	349190	{}	\N	2020-10-12 16:06:22.403999
69561	SID	2020-09-28	2.93	2.95	2.765	2.8	1531769	{}	\N	2020-10-12 16:06:24.304088
69562	SID	2020-09-29	2.79	2.825	2.7	2.7	4559481	{}	\N	2020-10-12 16:06:24.31544
69563	SID	2020-09-30	2.93	3.02	2.9	2.94	4010686	{}	\N	2020-10-12 16:06:24.322562
69564	SID	2020-10-01	2.95	2.955	2.87	2.91	3570264	{}	\N	2020-10-12 16:06:24.328416
69565	SID	2020-10-02	2.87	3	2.85	2.97	2891037	{}	\N	2020-10-12 16:06:24.33416
69566	SID	2020-10-05	3.01	3.19	3.01	3.18	3479831	{}	\N	2020-10-12 16:06:24.339223
69567	SID	2020-10-06	3.2	3.22	3.1	3.16	4115688	{}	\N	2020-10-12 16:06:24.347505
69568	SID	2020-10-07	3.21	3.28	3.19	3.24	3954195	{}	\N	2020-10-12 16:06:24.352271
69569	SID	2020-10-08	3.19	3.23	3.16	3.19	3788427	{}	\N	2020-10-12 16:06:24.357472
69570	SID	2020-10-09	3.27	3.3	3.17	3.22	3607180	{}	\N	2020-10-12 16:06:24.362481
69571	SID	2020-10-12	3.22	3.235	3.185	3.19	1269504	{}	\N	2020-10-12 16:06:24.367285
69572	SJI	2020-09-28	19.15	19.39	18.94	19.14	595827	{}	\N	2020-10-12 16:06:27.290541
69573	SJI	2020-09-29	19.09	19.34	18.91	19.17	1482628	{}	\N	2020-10-12 16:06:27.301847
69574	SJI	2020-09-30	19.26	19.5	18.94	19.27	2183234	{}	\N	2020-10-12 16:06:27.308726
69575	SJI	2020-10-01	19.17	19.3	18.86	19.02	1273241	{}	\N	2020-10-12 16:06:27.314805
69576	SJI	2020-10-02	18.82	19.8	18.7	19.71	1759939	{}	\N	2020-10-12 16:06:27.320616
69577	SJI	2020-10-05	19.59	19.94	19.56	19.69	2401952	{}	\N	2020-10-12 16:06:27.325751
69578	SJI	2020-10-06	19.81	20.78	19.75	20.42	2558354	{}	\N	2020-10-12 16:06:27.330479
69579	SJI	2020-10-07	20.45	20.57	20.01	20.24	2445968	{}	\N	2020-10-12 16:06:27.335147
69580	SJI	2020-10-08	20.45	20.75	20.36	20.61	1360612	{}	\N	2020-10-12 16:06:27.340164
69581	SJI	2020-10-09	20.79	20.79	20.25	20.33	990816	{}	\N	2020-10-12 16:06:27.344748
69582	SJI	2020-10-12	20.22	20.47	20.195	20.3	611252	{}	\N	2020-10-12 16:06:27.349109
69583	SKYS	2020-09-28	8.33	9.39	8.05	8.5	389526	{}	\N	2020-10-12 16:06:30.069278
69584	SKYS	2020-09-29	8.28	8.29	7.08	7.08	154410	{}	\N	2020-10-12 16:06:30.078725
69585	SKYS	2020-09-30	7.14	9.77	7.09	8.31	782444	{}	\N	2020-10-12 16:06:30.085372
69586	SKYS	2020-10-01	8.43	8.58	7.86	8.35	118842	{}	\N	2020-10-12 16:06:30.091403
69587	SKYS	2020-10-02	7.99	8.48	7.5	7.6	124154	{}	\N	2020-10-12 16:06:30.097382
69588	SKYS	2020-10-05	7.68	8.25	7.61	7.8	66627	{}	\N	2020-10-12 16:06:30.10249
69589	SKYS	2020-10-06	8	8.8499	7.95	8.63	116482	{}	\N	2020-10-12 16:06:30.107383
69590	SKYS	2020-10-07	9.27	10.2546	8.68	9.28	346849	{}	\N	2020-10-12 16:06:30.11212
69591	SKYS	2020-10-08	9.49	9.49	6.81	7.17	379767	{}	\N	2020-10-12 16:06:30.116838
69592	SLG	2020-09-28	47.72	49	47.3301	48.27	943523	{}	\N	2020-10-12 16:06:31.087247
69593	SLG	2020-09-29	47.78	47.965	45.57	46.53	2097068	{}	\N	2020-10-12 16:06:31.100028
69594	SLG	2020-09-30	47.09	47.73	45.77	46.37	2352277	{}	\N	2020-10-12 16:06:31.107588
69595	SLG	2020-10-01	46.45	47.6204	45.63	47.29	2417423	{}	\N	2020-10-12 16:06:31.114553
69596	SLG	2020-10-02	45.51	49.55	45.51	49.36	3137153	{}	\N	2020-10-12 16:06:31.120755
69597	SLG	2020-10-05	50.1	50.45	48.18	49.25	2068064	{}	\N	2020-10-12 16:06:31.12625
69598	SLG	2020-10-06	49.84	51.77	49.1901	49.92	3495028	{}	\N	2020-10-12 16:06:31.131996
69599	SLG	2020-10-07	50.16	51.6	49.76	50.73	2350757	{}	\N	2020-10-12 16:06:31.136915
69600	SLG	2020-10-08	50.84	53.2	50.78	52.99	2301283	{}	\N	2020-10-12 16:06:31.14179
69601	SLG	2020-10-09	53.42	53.42	50.95	51.44	2147119	{}	\N	2020-10-12 16:06:31.146757
69602	SLG	2020-10-12	51.34	51.3429	50.42	50.58	554999	{}	\N	2020-10-12 16:06:31.15169
69603	SLV	2020-09-28	21.7255	22.08	21.5	22.02	31481131	{}	\N	2020-10-12 16:06:32.11536
69604	SLV	2020-09-29	22.24	22.71	22.165	22.5	32933026	{}	\N	2020-10-12 16:06:32.128376
69605	SLV	2020-09-30	22.27	22.38	21.53	21.64	44398465	{}	\N	2020-10-12 16:06:32.136006
69606	SLV	2020-10-01	22.01	22.5	21.76	22.09	34818483	{}	\N	2020-10-12 16:06:32.142555
69607	SLV	2020-10-02	22.25	22.52	22.04	22.13	24868058	{}	\N	2020-10-12 16:06:32.148396
69608	SLV	2020-10-05	22.32	22.81	22.3	22.62	22327390	{}	\N	2020-10-12 16:06:32.153598
69609	SLV	2020-10-06	22.6301	22.73	21.6211	21.73	30272101	{}	\N	2020-10-12 16:06:32.158243
69610	SLV	2020-10-07	21.97	22.32	21.89	22.09	18297039	{}	\N	2020-10-12 16:06:32.163243
69611	SLV	2020-10-08	22.39	22.53	22.01	22.18	20146081	{}	\N	2020-10-12 16:06:32.167948
69612	SLV	2020-10-09	22.8	23.42	22.77	23.41	38638504	{}	\N	2020-10-12 16:06:32.173016
69613	SLV	2020-10-12	23.34	23.3881	23.06	23.31	12892909	{}	\N	2020-10-12 16:06:32.178077
69614	SMBK	2020-09-28	13.38	13.8	13.06	13.64	23541	{}	\N	2020-10-12 16:06:33.150526
69615	SMBK	2020-09-29	13.73	13.8	13.22	13.58	6806	{}	\N	2020-10-12 16:06:33.165028
69616	SMBK	2020-09-30	13.4	13.68	13.34	13.59	7727	{}	\N	2020-10-12 16:06:33.172782
69617	SMBK	2020-10-01	13.52	13.54	13.2	13.44	10207	{}	\N	2020-10-12 16:06:33.179354
69618	SMBK	2020-10-02	13.24	14.13	13.24	13.87	12042	{}	\N	2020-10-12 16:06:33.189954
69619	SMBK	2020-10-05	13.88	14.24	13.88	14.14	6935	{}	\N	2020-10-12 16:06:33.195722
69620	SMBK	2020-10-06	14.17	14.77	13.864	14.06	16156	{}	\N	2020-10-12 16:06:33.200944
69621	SMBK	2020-10-07	14.31	14.82	14.08	14.8	19858	{}	\N	2020-10-12 16:06:33.206103
69622	SMBK	2020-10-08	14.86	15.18	14.51	14.91	16608	{}	\N	2020-10-12 16:06:33.21121
69623	SMBK	2020-10-09	15.06	15.06	14.54	15	2552	{}	\N	2020-10-12 16:06:33.216466
69624	SMBK	2020-10-12	14.62	14.77	14.6	14.61	2817	{}	\N	2020-10-12 16:06:33.221632
69625	SEIC	2020-09-28	51.69	52.08	51.48	51.59	479570	{}	\N	2020-10-12 16:06:34.208728
69626	SEIC	2020-09-29	51.53	51.77	51.06	51.07	416739	{}	\N	2020-10-12 16:06:34.221426
69627	SEIC	2020-09-30	51.23	51.63	50.44	50.72	656923	{}	\N	2020-10-12 16:06:34.228583
69628	SEIC	2020-10-01	50.91	51.48	50.74	51.02	365718	{}	\N	2020-10-12 16:06:34.235124
69629	SEIC	2020-10-02	50.28	52	50.28	51.75	362425	{}	\N	2020-10-12 16:06:34.240662
69630	SEIC	2020-10-05	52	53.37	51.9125	53.16	687131	{}	\N	2020-10-12 16:06:34.246276
69631	SEIC	2020-10-06	53.55	54.46	53.08	53.32	658726	{}	\N	2020-10-12 16:06:34.251243
69632	SEIC	2020-10-07	53.55	54.115	52.76	52.94	428418	{}	\N	2020-10-12 16:06:34.256274
69633	SEIC	2020-10-08	53.71	55.205	53.71	55.01	712778	{}	\N	2020-10-12 16:06:34.260955
69634	SEIC	2020-10-09	55.15	55.48	54.96	55.2	416686	{}	\N	2020-10-12 16:06:34.265755
69635	SEIC	2020-10-12	55.56	56.4	55.56	56.03	92674	{}	\N	2020-10-12 16:06:34.270509
69636	SENS	2020-09-28	0.399	0.41	0.38	0.4041	1541139	{}	\N	2020-10-12 16:06:35.500456
69637	SENS	2020-09-29	0.4139	0.4139	0.3866	0.4021	1071620	{}	\N	2020-10-12 16:06:35.513577
69638	SENS	2020-09-30	0.4027	0.4027	0.3866	0.3873	1016026	{}	\N	2020-10-12 16:06:35.521356
69639	SENS	2020-10-01	0.398	0.3999	0.3875	0.3899	629870	{}	\N	2020-10-12 16:06:35.52786
69640	SENS	2020-10-02	0.3899	0.41	0.3861	0.3985	919741	{}	\N	2020-10-12 16:06:35.534513
69641	SENS	2020-10-05	0.4217	0.4298	0.4	0.4078	1956731	{}	\N	2020-10-12 16:06:35.540002
69642	SENS	2020-10-06	0.42	0.42	0.3916	0.4	1668151	{}	\N	2020-10-12 16:06:35.545519
69643	SENS	2020-10-07	0.4	0.4149	0.395	0.4	1268299	{}	\N	2020-10-12 16:06:35.551088
69644	SENS	2020-10-08	0.41	0.4139	0.3927	0.3961	927393	{}	\N	2020-10-12 16:06:35.556501
69645	SENS	2020-10-09	0.4	0.415	0.4	0.4005	1083753	{}	\N	2020-10-12 16:06:35.561651
69646	SENS	2020-10-12	0.409	0.4098	0.3921	0.401	504953	{}	\N	2020-10-12 16:06:35.566749
69647	SFNC	2020-09-28	15.47	15.99	15.46	15.9	482423	{}	\N	2020-10-12 16:06:36.787357
69648	SFNC	2020-09-29	15.85	15.865	15.32	15.6	454643	{}	\N	2020-10-12 16:06:36.80067
69649	SFNC	2020-09-30	15.73	16.14	15.6	15.855	641694	{}	\N	2020-10-12 16:06:36.808377
69650	SFNC	2020-10-01	15.78	15.93	15.4683	15.815	408911	{}	\N	2020-10-12 16:06:36.815191
69651	SFNC	2020-10-02	15.54	16.28	15.54	16.2	344894	{}	\N	2020-10-12 16:06:36.821337
69652	SFNC	2020-10-05	16.47	16.85	16.38	16.82	426322	{}	\N	2020-10-12 16:06:36.826644
69653	SFNC	2020-10-06	16.93	17.61	16.79	16.82	648911	{}	\N	2020-10-12 16:06:36.831755
69654	SFNC	2020-10-07	17.07	17.54	16.9	17.24	521559	{}	\N	2020-10-12 16:06:36.836977
69655	SFNC	2020-10-08	17.62	17.62	17.31	17.49	346715	{}	\N	2020-10-12 16:06:36.842063
69656	SFNC	2020-10-09	17.58	17.66	17.2832	17.32	348216	{}	\N	2020-10-12 16:06:36.846913
69657	SFNC	2020-10-12	17.31	17.71	17.3	17.62	140228	{}	\N	2020-10-12 16:06:36.852034
69658	SGDJ	2020-09-28	44.46	44.51	43.67	44.11	20674	{}	\N	2020-10-12 16:06:38.099662
69659	SGDJ	2020-09-29	44.405	44.9399	44.33	44.84	18932	{}	\N	2020-10-12 16:06:38.107476
69660	SGDJ	2020-09-30	44.63	44.99	44.2495	44.49	4668	{}	\N	2020-10-12 16:06:38.114745
69661	SGDJ	2020-10-01	45.17	45.9271	45.13	45.54	14251	{}	\N	2020-10-12 16:06:38.120392
69662	SGDJ	2020-10-02	45.36	45.62	44.89	45.02	10211	{}	\N	2020-10-12 16:06:38.125708
69663	SGDJ	2020-10-05	45.87	46.57	45.849	46.19	13876	{}	\N	2020-10-12 16:06:38.131205
69664	SGDJ	2020-10-06	46.45	46.45	45	45	15267	{}	\N	2020-10-12 16:06:38.136764
69665	SGDJ	2020-10-07	45.66	45.79	45.117	45.37	26244	{}	\N	2020-10-12 16:06:38.142211
69666	SGDJ	2020-10-08	45.58	46.01	45.33	46.01	5053	{}	\N	2020-10-12 16:06:38.147858
69667	SGDJ	2020-10-09	47.11	48.74	47.1	48.7	37029	{}	\N	2020-10-12 16:06:38.153237
69668	SGDJ	2020-10-12	48.59	48.64	48.35	48.5481	7201	{}	\N	2020-10-12 16:06:38.158678
69669	SGMS	2020-09-28	35.7	37.58	34.79	36.69	3450969	{}	\N	2020-10-12 16:06:40.238185
69670	SGMS	2020-09-29	36.76	36.76	34.38	34.96	3645225	{}	\N	2020-10-12 16:06:40.250809
69671	SGMS	2020-09-30	35.08	36	34.5	34.91	4728049	{}	\N	2020-10-12 16:06:40.258295
69672	SGMS	2020-10-01	34.92	36.48	34.36	35.97	3611438	{}	\N	2020-10-12 16:06:40.26457
69673	SGMS	2020-10-02	34.29	36.86	33.71	36.48	3925225	{}	\N	2020-10-12 16:06:40.271723
69674	SGMS	2020-10-05	36.89	37.2	34.2	36.45	3722105	{}	\N	2020-10-12 16:06:40.276681
69675	SGMS	2020-10-06	36.98	37.33	34.28	34.82	4694052	{}	\N	2020-10-12 16:06:40.281486
69676	SGMS	2020-10-07	35.79	35.93	34.23	34.41	6167017	{}	\N	2020-10-12 16:06:40.28639
69677	SGMS	2020-10-08	34.81	35.3	34.24	34.88	2922720	{}	\N	2020-10-12 16:06:40.29122
69678	SGMS	2020-10-09	34.67	34.99	32.88	33.57	2689621	{}	\N	2020-10-12 16:06:40.296141
69679	SGMS	2020-10-12	34.1	34.4	33.6	34.09	267616	{}	\N	2020-10-12 16:06:40.300857
69680	SGMO	2020-09-28	10.07	10.17	9.83	9.91	859794	{}	\N	2020-10-12 16:06:41.285126
69681	SGMO	2020-09-29	9.88	9.96	9.5	9.89	1338740	{}	\N	2020-10-12 16:06:41.296191
69682	SGMO	2020-09-30	9.92	10.03	9.41	9.45	1220988	{}	\N	2020-10-12 16:06:41.303
69683	SGMO	2020-10-01	9.605	9.66	9.23	9.36	1181811	{}	\N	2020-10-12 16:06:41.310582
69684	SGMO	2020-10-02	9.15	9.365	9.14	9.23	1508676	{}	\N	2020-10-12 16:06:41.316389
69685	SGMO	2020-10-05	9.39	9.82	9.39	9.78	995339	{}	\N	2020-10-12 16:06:41.321206
69686	SGMO	2020-10-06	9.8	9.92	9.65	9.7	1114610	{}	\N	2020-10-12 16:06:41.325824
69687	SGMO	2020-10-07	9.52	10.24	9.52	10.17	1549111	{}	\N	2020-10-12 16:06:41.330503
69688	SGMO	2020-10-08	10.4	10.435	9.825	9.89	1062159	{}	\N	2020-10-12 16:06:41.335245
69689	SGMO	2020-10-09	10.11	11.0148	10.01	10.83	2203281	{}	\N	2020-10-12 16:06:41.340073
69690	SGMO	2020-10-12	10.91	11.115	10.76	11.1	1026049	{}	\N	2020-10-12 16:06:41.344613
69691	SGU	2020-09-28	9.85	9.85	9.67	9.68	76447	{}	\N	2020-10-12 16:06:42.947708
69692	SGU	2020-09-29	9.66	9.835	9.66	9.8	80661	{}	\N	2020-10-12 16:06:42.96165
69693	SGU	2020-09-30	9.76	9.84	9.75	9.75	69068	{}	\N	2020-10-12 16:06:42.968624
69694	SGU	2020-10-01	9.66	9.77	9.63	9.64	151370	{}	\N	2020-10-12 16:06:42.999664
69695	SGU	2020-10-02	9.6	9.78	9.6	9.7	41791	{}	\N	2020-10-12 16:06:43.030467
69696	SGU	2020-10-05	9.74	9.7899	9.65	9.69	130300	{}	\N	2020-10-12 16:06:43.035429
69697	SGU	2020-10-06	9.64	9.7198	9.5	9.57	156423	{}	\N	2020-10-12 16:06:43.04035
69698	SGU	2020-10-07	9.53	9.63	9.44	9.44	144962	{}	\N	2020-10-12 16:06:43.045132
69699	SGU	2020-10-08	9.5	9.7	9.5	9.59	85081	{}	\N	2020-10-12 16:06:43.05012
69700	SGU	2020-10-09	9.62	9.6825	9.56	9.6	447056	{}	\N	2020-10-12 16:06:43.055024
69701	SGU	2020-10-12	9.6	9.77	9.6	9.67	75512	{}	\N	2020-10-12 16:06:43.059889
69702	SHBI	2020-09-28	10.25	10.68	10.06	10.54	43794	{}	\N	2020-10-12 16:06:44.978647
69703	SHBI	2020-09-29	10.33	10.74	10.3	10.63	13629	{}	\N	2020-10-12 16:06:44.990791
69704	SHBI	2020-09-30	10.71	11.3	10.71	10.98	24059	{}	\N	2020-10-12 16:06:44.997929
69705	SHBI	2020-10-01	10.8	11.04	10.8	10.95	27086	{}	\N	2020-10-12 16:06:45.004439
69706	SHBI	2020-10-02	10.83	11.24	10.83	11.2	34483	{}	\N	2020-10-12 16:06:45.010216
69707	SHBI	2020-10-05	11.08	11.4	11.0202	11.33	31645	{}	\N	2020-10-12 16:06:45.015429
69708	SHBI	2020-10-06	11.4	11.66	11.12	11.17	38159	{}	\N	2020-10-12 16:06:45.020235
69709	SHBI	2020-10-07	11.27	11.91	11.27	11.76	57723	{}	\N	2020-10-12 16:06:45.027891
69710	SHBI	2020-10-08	11.87	12.02	11.8	11.9	29468	{}	\N	2020-10-12 16:06:45.031804
69711	SHBI	2020-10-09	11.61	11.97	11.61	11.82	29438	{}	\N	2020-10-12 16:06:45.035808
69712	SHBI	2020-10-12	11.91	11.92	11.77	11.78	4208	{}	\N	2020-10-12 16:06:45.039988
69713	SHG	2020-09-28	23.47	23.7	23.47	23.51	168396	{}	\N	2020-10-12 16:06:46.539332
69714	SHG	2020-09-29	23.45	23.45	22.93	22.98	290229	{}	\N	2020-10-12 16:06:46.551968
69715	SHG	2020-09-30	22.96	23.3	22.81	22.97	261505	{}	\N	2020-10-12 16:06:46.559784
69716	SHG	2020-10-01	23.12	23.44	23.09	23.24	228290	{}	\N	2020-10-12 16:06:46.566555
69717	SHG	2020-10-02	22.88	23.39	22.88	23.2	212985	{}	\N	2020-10-12 16:06:46.572227
69718	SHG	2020-10-05	24.35	24.535	24.22	24.46	446395	{}	\N	2020-10-12 16:06:46.577593
69719	SHG	2020-10-06	24.34	24.4061	23.83	23.92	470962	{}	\N	2020-10-12 16:06:46.582578
69720	SHG	2020-10-07	24.36	24.65	24.36	24.57	271505	{}	\N	2020-10-12 16:06:46.587751
69721	SHG	2020-10-08	24.5	24.5683	24.35	24.46	129895	{}	\N	2020-10-12 16:06:46.592474
69722	SHG	2020-10-09	24.62	24.66	24.5	24.57	113125	{}	\N	2020-10-12 16:06:46.597232
69723	SHG	2020-10-12	24.67	24.81	24.58	24.7	49710	{}	\N	2020-10-12 16:06:46.601769
69724	SHW	2020-09-28	705.72	712.17	701.89	707.06	437078	{}	\N	2020-10-12 16:06:47.594176
69725	SHW	2020-09-29	710.47	715.34	705.63	705.67	568982	{}	\N	2020-10-12 16:06:47.606125
69726	SHW	2020-09-30	710	715	691.91	696.74	1569098	{}	\N	2020-10-12 16:06:47.6129
69727	SHW	2020-10-01	704.98	709.529	688.67	690.8	994493	{}	\N	2020-10-12 16:06:47.619004
69728	SHW	2020-10-02	686.71	696.895	680.767	688.98	775960	{}	\N	2020-10-12 16:06:47.624788
69729	SHW	2020-10-05	693.52	701.45	686.755	689.6	1289183	{}	\N	2020-10-12 16:06:47.630115
69730	SHW	2020-10-06	691.88	696.57	680.97	682.38	1008269	{}	\N	2020-10-12 16:06:47.635151
69731	SHW	2020-10-07	687.74	693.66	685.31	691.8	877484	{}	\N	2020-10-12 16:06:47.639805
69732	SHW	2020-10-08	694.62	697.96	685.47	688.28	967461	{}	\N	2020-10-12 16:06:47.644529
69733	SHW	2020-10-09	691.35	698.9395	690	698.4	885318	{}	\N	2020-10-12 16:06:47.649295
69734	SHW	2020-10-12	702	708.04	699.1945	705.76	214688	{}	\N	2020-10-12 16:06:47.654195
69735	SIG	2020-09-28	18.7992	19.435	18.41	19.39	1393607	{}	\N	2020-10-12 16:06:48.666876
69736	SIG	2020-09-29	19.31	19.56	17.91	17.95	2295774	{}	\N	2020-10-12 16:06:48.678443
69737	SIG	2020-09-30	18.13	19.04	18.06	18.7	2526987	{}	\N	2020-10-12 16:06:48.685502
69738	SIG	2020-10-01	18.83	19.34	18.36	19.11	2142556	{}	\N	2020-10-12 16:06:48.69214
69739	SIG	2020-10-02	18.3	19.66	18.25	19.51	1711719	{}	\N	2020-10-12 16:06:48.698374
69740	SIG	2020-10-05	19.78	20.3293	19.61	20.04	1878340	{}	\N	2020-10-12 16:06:48.704332
69741	SIG	2020-10-06	20.22	20.469	18.77	18.85	1963042	{}	\N	2020-10-12 16:06:48.709573
69742	SIG	2020-10-07	19.2	20.27	19.2	20.21	2556250	{}	\N	2020-10-12 16:06:48.715135
69743	SIG	2020-10-08	20.49	20.97	20.3566	20.91	1322983	{}	\N	2020-10-12 16:06:48.72001
69744	SIG	2020-10-09	21.01	21.59	20.82	20.83	1537961	{}	\N	2020-10-12 16:06:48.725144
69745	SIG	2020-10-12	20.83	21.287	20.59	20.65	568400	{}	\N	2020-10-12 16:06:48.730327
69746	SINO	2020-09-28	1.74	1.75	1.67	1.6992	16718	{}	\N	2020-10-12 16:06:49.73064
69747	SINO	2020-09-29	1.79	1.96	1.52	1.59	716813	{}	\N	2020-10-12 16:06:49.743637
69748	SINO	2020-09-30	1.7	1.7	1.3727	1.42	143563	{}	\N	2020-10-12 16:06:49.751798
69749	SINO	2020-10-01	1.41	1.55	1.41	1.48	105103	{}	\N	2020-10-12 16:06:49.757755
69750	SINO	2020-10-02	1.49	1.67	1.49	1.58	105932	{}	\N	2020-10-12 16:06:49.762638
69751	SINO	2020-10-05	1.66	1.7	1.5678	1.61	40830	{}	\N	2020-10-12 16:06:49.76776
69752	SINO	2020-10-06	1.61	1.67	1.5715	1.63	61873	{}	\N	2020-10-12 16:06:49.771938
69753	SINO	2020-10-07	1.61	1.78	1.579	1.65	105960	{}	\N	2020-10-12 16:06:49.776349
69754	SINO	2020-10-08	1.63	1.7	1.63	1.63	5801	{}	\N	2020-10-12 16:06:49.780484
69755	SINO	2020-10-09	1.63	1.6999	1.6	1.648	22586	{}	\N	2020-10-12 16:06:49.784905
69756	SINO	2020-10-12	1.63	1.84	1.63	1.68	150038	{}	\N	2020-10-12 16:06:49.78946
69757	SOHU	2020-09-28	17.48	18.555	17.26	18.04	512934	{}	\N	2020-10-12 16:06:51.212241
69758	SOHU	2020-09-29	20.53	21.5892	19.1	20.1	1872722	{}	\N	2020-10-12 16:06:51.223022
69759	SOHU	2020-09-30	19.87	20.08	19.25	19.86	643745	{}	\N	2020-10-12 16:06:51.230394
69760	SOHU	2020-10-01	20.08	20.74	19.66	20.28	644977	{}	\N	2020-10-12 16:06:51.237573
69761	SOHU	2020-10-02	20.04	20.62	19.8	19.93	388818	{}	\N	2020-10-12 16:06:51.243316
69762	SOHU	2020-10-05	19.93	20.69	19.6706	19.89	324143	{}	\N	2020-10-12 16:06:51.248217
69763	SOHU	2020-10-06	19.97	20.42	19.27	19.72	326870	{}	\N	2020-10-12 16:06:51.252962
69764	SOHU	2020-10-07	19.89	20.99	19.89	20.08	501726	{}	\N	2020-10-12 16:06:51.258065
69765	SOHU	2020-10-08	20.12	20.5	19.91	19.96	191115	{}	\N	2020-10-12 16:06:51.263056
69766	SOHU	2020-10-09	19.99	20.2096	19.56	19.62	307055	{}	\N	2020-10-12 16:06:51.268264
69767	SOHU	2020-10-12	19.92	20.12	19.69	19.94	255119	{}	\N	2020-10-12 16:06:51.27313
69768	SEM	2020-09-28	19.68	20.01	19.57	19.82	450877	{}	\N	2020-10-12 16:06:53.152182
69769	SEM	2020-09-29	20	20.18	19.54	19.87	1075633	{}	\N	2020-10-12 16:06:53.163014
69770	SEM	2020-09-30	20.07	20.88	20.06	20.82	1480395	{}	\N	2020-10-12 16:06:53.17091
69771	SEM	2020-10-01	21.01	21.27	20.53	20.99	1089117	{}	\N	2020-10-12 16:06:53.177352
69772	SEM	2020-10-02	20.57	21.43	20.05	21.26	1091828	{}	\N	2020-10-12 16:06:53.183072
69773	SEM	2020-10-05	21.46	21.83	21.31	21.78	831616	{}	\N	2020-10-12 16:06:53.188639
69774	SEM	2020-10-06	22.07	22.09	21.46	21.47	1721105	{}	\N	2020-10-12 16:06:53.194101
69775	SEM	2020-10-07	21.78	22.0937	21.57	21.8	1251851	{}	\N	2020-10-12 16:06:53.198974
69776	SEM	2020-10-08	22	22.35	21.79	22.23	711909	{}	\N	2020-10-12 16:06:53.203773
69777	SEM	2020-10-09	22.45	22.83	22.28	22.45	819595	{}	\N	2020-10-12 16:06:53.208517
69778	SEM	2020-10-12	22.43	22.43	21.69	21.96	156404	{}	\N	2020-10-12 16:06:53.213285
69779	SFBC	2020-09-28	28.9833	29.766	28.9833	29.51	4573	{}	\N	2020-10-12 16:06:54.114482
69780	SFBC	2020-09-29	29.63	29.63	29.63	29.63	162	{}	\N	2020-10-12 16:06:54.125687
69781	SFBC	2020-09-30	29.43	29.63	29.43	29.63	801	{}	\N	2020-10-12 16:06:54.13225
69782	SFBC	2020-10-02	29	29.89	29	29.57	3072	{}	\N	2020-10-12 16:06:54.13813
69783	SFBC	2020-10-05	29.465	29.95	29.425	29.6	9522	{}	\N	2020-10-12 16:06:54.144029
69784	SFBC	2020-10-06	29.6	29.6999	29.26	29.26	1819	{}	\N	2020-10-12 16:06:54.148699
69785	SFBC	2020-10-07	29.5	29.72	29.35	29.35	8027	{}	\N	2020-10-12 16:06:54.153218
69786	SFBC	2020-10-08	29.48	29.5	27.5025	28.78	8695	{}	\N	2020-10-12 16:06:54.157654
69787	SFBC	2020-10-09	28.65	29	28.02	28.07	3554	{}	\N	2020-10-12 16:06:54.1619
69788	SFBC	2020-10-12	28.0101	28.805	28.0101	28.13	1232	{}	\N	2020-10-12 16:06:54.166601
69789	SGA	2020-09-28	20.73	20.73	19.78	19.78	1824	{}	\N	2020-10-12 16:06:55.950008
69790	SGA	2020-09-29	19.94	19.94	19.94	19.94	425	{}	\N	2020-10-12 16:06:55.957972
69791	SGA	2020-09-30	19.87	20.91	19.87	19.88	2490	{}	\N	2020-10-12 16:06:55.963865
69792	SGA	2020-10-01	20	20	20	20	1879	{}	\N	2020-10-12 16:06:55.969302
69793	SGA	2020-10-02	19.89	20.45	19.89	20.45	2313	{}	\N	2020-10-12 16:06:55.973667
69794	SGA	2020-10-05	20.4	20.52	20.34	20.34	3209	{}	\N	2020-10-12 16:06:55.977577
69795	SGA	2020-10-06	20.34	20.75	20.27	20.31	3355	{}	\N	2020-10-12 16:06:55.981442
69796	SGA	2020-10-07	20.36	20.7	20.05	20.05	3888	{}	\N	2020-10-12 16:06:55.985462
69797	SGA	2020-10-08	20.11	20.11	19.9	19.9	2428	{}	\N	2020-10-12 16:06:55.989423
69798	SGA	2020-10-09	19.92	20	19.68	19.88	2741	{}	\N	2020-10-12 16:06:55.993305
69799	SHAK	2020-09-28	67.3	68.79	66.27	68.5	814089	{}	\N	2020-10-12 16:06:57.080028
69800	SHAK	2020-09-29	68.78	68.78	63.83	64.9	2841197	{}	\N	2020-10-12 16:06:57.09258
69801	SHAK	2020-09-30	65.27	67.09	63.49	64.48	2695404	{}	\N	2020-10-12 16:06:57.099712
69802	SHAK	2020-10-01	65.02	65.34	64.08	64.87	2136291	{}	\N	2020-10-12 16:06:57.105596
69803	SHAK	2020-10-02	64.21	68.08	63.77	66.77	2041047	{}	\N	2020-10-12 16:06:57.110761
69804	SHAK	2020-10-05	67.46	68	64.86	65.59	1767898	{}	\N	2020-10-12 16:06:57.115778
69805	SHAK	2020-10-06	66.03	68	65.28	65.54	1610670	{}	\N	2020-10-12 16:06:57.120054
69806	SHAK	2020-10-07	66.49	67.39	65.86	66.87	1272598	{}	\N	2020-10-12 16:06:57.124271
69807	SHAK	2020-10-08	67.5	71.09	66.7	70.89	2520774	{}	\N	2020-10-12 16:06:57.128647
69808	SHAK	2020-10-09	71.15	72.36	70.35	72.18	1303688	{}	\N	2020-10-12 16:06:57.132881
69809	SHAK	2020-10-12	71.19	72.29	70.89	71.3	434917	{}	\N	2020-10-12 16:06:57.136937
69810	SHIPW	2020-09-28	0.0166	0.0185	0.0166	0.0184	16886	{}	\N	2020-10-12 16:06:58.002487
69811	SHIPW	2020-09-29	0.016001	0.0184	0.016001	0.0183	13886	{}	\N	2020-10-12 16:06:58.016251
69812	SHIPW	2020-09-30	0.0163	0.019	0.0145	0.0163	96477	{}	\N	2020-10-12 16:06:58.023889
69813	SHIPW	2020-10-01	0.019	0.022	0.016	0.016	152243	{}	\N	2020-10-12 16:06:58.030493
69814	SHIPW	2020-10-02	0.02	0.0242	0.016	0.0242	46050	{}	\N	2020-10-12 16:06:58.036003
69815	SHIPW	2020-10-05	0.019501	0.0231	0.019501	0.021946	57883	{}	\N	2020-10-12 16:06:58.041811
69816	SHIPW	2020-10-06	0.02	0.02	0.0162	0.0173	35651	{}	\N	2020-10-12 16:06:58.047436
69817	SHIPW	2020-10-07	0.0242	0.03	0.016	0.0216	50878	{}	\N	2020-10-12 16:06:58.05546
69818	SHIPW	2020-10-08	0.03	0.0345	0.025	0.0258	294589	{}	\N	2020-10-12 16:06:58.060689
69819	SHIPW	2020-10-09	0.0345	0.0345	0.021	0.0274	113017	{}	\N	2020-10-12 16:06:58.065777
69820	SHIPW	2020-10-12	0.03	0.03	0.021	0.026299	7173	{}	\N	2020-10-12 16:06:58.070829
69821	SHYL	2020-09-28	45.79	45.88	45.76	45.88	3012	{}	\N	2020-10-12 16:06:59.71316
69822	SHYL	2020-09-29	45.78	46.02	45.7308	45.835	7622	{}	\N	2020-10-12 16:06:59.728328
69823	SHYL	2020-09-30	45.95	46.05	45.95	46.03	2652	{}	\N	2020-10-12 16:06:59.737407
69824	SHYL	2020-10-01	45.78	45.9199	45.77	45.8	4282	{}	\N	2020-10-12 16:06:59.744167
69825	SHYL	2020-10-02	45.84	45.8699	45.7972	45.7972	1723	{}	\N	2020-10-12 16:06:59.750235
69826	SHYL	2020-10-05	46.03	46.075	46.0116	46.075	4909	{}	\N	2020-10-12 16:06:59.755253
69827	SHYL	2020-10-06	46.17	46.19	45.995	45.995	1496	{}	\N	2020-10-12 16:06:59.760146
69828	SHYL	2020-10-07	46.22	46.25	46.1751	46.1751	5699	{}	\N	2020-10-12 16:06:59.764886
69829	SHYL	2020-10-08	46.28	46.29	46.2063	46.28	3322	{}	\N	2020-10-12 16:06:59.769566
69830	SHYL	2020-10-09	46.26	46.38	46.26	46.36	5777	{}	\N	2020-10-12 16:06:59.774381
69831	SHYL	2020-10-12	46.55	46.55	46.515	46.54	2342	{}	\N	2020-10-12 16:06:59.779457
69832	SINA	2020-09-28	42.45	42.9414	42.2	42.55	3403052	{}	\N	2020-10-12 16:07:01.819865
69833	SINA	2020-09-29	42.7	42.82	42.5	42.52	1425236	{}	\N	2020-10-12 16:07:01.830701
69834	SINA	2020-09-30	42.53	42.75	42.53	42.61	1035145	{}	\N	2020-10-12 16:07:01.837773
69835	SINA	2020-10-01	42.6	42.74	42.49	42.58	1175226	{}	\N	2020-10-12 16:07:01.843916
69836	SINA	2020-10-02	42.5	42.51	42.22	42.47	974484	{}	\N	2020-10-12 16:07:01.849323
69837	SINA	2020-10-05	42.4	42.51	42.4	42.5	1962183	{}	\N	2020-10-12 16:07:01.854046
69838	SINA	2020-10-06	42.5	42.78	42.47	42.68	763813	{}	\N	2020-10-12 16:07:01.858255
69839	SINA	2020-10-07	42.67	42.75	42.53	42.7	385562	{}	\N	2020-10-12 16:07:01.862515
69840	SINA	2020-10-08	42.7	42.7	42.54	42.62	465497	{}	\N	2020-10-12 16:07:01.86685
69841	SINA	2020-10-09	42.62	42.66	42.5	42.56	468631	{}	\N	2020-10-12 16:07:01.870929
69842	SINA	2020-10-12	42.65	42.85	42.585	42.62	269213	{}	\N	2020-10-12 16:07:01.875255
69843	SIJ	2020-09-28	7.64	7.64	6.891	6.97	26734	{}	\N	2020-10-12 16:07:02.809887
69844	SIJ	2020-09-29	7	7.05	6.96	7.01	11338	{}	\N	2020-10-12 16:07:02.820268
69845	SIJ	2020-09-30	6.9778	7.1009	6.89	7.1009	3448	{}	\N	2020-10-12 16:07:02.828022
69846	SIJ	2020-10-01	6.9	7.0999	6.9	7.09	2302	{}	\N	2020-10-12 16:07:02.834522
69847	SIJ	2020-10-02	7.29	7.4	6.9252	6.9252	4757	{}	\N	2020-10-12 16:07:02.840023
69848	SIJ	2020-10-05	6.8	6.801	6.7	6.71	5536	{}	\N	2020-10-12 16:07:02.84504
69849	SIJ	2020-10-06	6.64	6.8617	6.6	6.8617	5908	{}	\N	2020-10-12 16:07:02.849537
69850	SIJ	2020-10-07	6.6155	6.71	6.6051	6.6051	5141	{}	\N	2020-10-12 16:07:02.854011
69851	SIJ	2020-10-08	6.52	6.6	6.5	6.51	4652	{}	\N	2020-10-12 16:07:02.858393
69852	SIJ	2020-10-09	6.435	6.44	6.345	6.4198	2247	{}	\N	2020-10-12 16:07:02.862891
69853	SIJ	2020-10-12	6.38	6.38	6.3257	6.3257	4294	{}	\N	2020-10-12 16:07:02.868036
69854	SIZE	2020-09-28	93.8	94.25	93.53	93.83	45500	{}	\N	2020-10-12 16:07:04.076312
69855	SIZE	2020-09-29	93.91	94.04	93.135	93.39	26946	{}	\N	2020-10-12 16:07:04.088266
69856	SIZE	2020-09-30	93.95	94.67	93.33	93.77	56290	{}	\N	2020-10-12 16:07:04.096477
69857	SIZE	2020-10-01	94.34	94.76	93.84	94.45	75531	{}	\N	2020-10-12 16:07:04.104083
69858	SIZE	2020-10-02	93.84	95.2	93.72	94.83	245716	{}	\N	2020-10-12 16:07:04.110111
69859	SIZE	2020-10-05	95.45	96.57	95.45	96.53	34720	{}	\N	2020-10-12 16:07:04.116062
69860	SIZE	2020-10-06	96.89	97.81	95.6	95.66	34314	{}	\N	2020-10-12 16:07:04.12177
69861	SIZE	2020-10-07	96.99	97.5799	96.93	97.42	256305	{}	\N	2020-10-12 16:07:04.126667
69862	SIZE	2020-10-08	97.98	98.53	97.7701	98.53	105325	{}	\N	2020-10-12 16:07:04.131569
69863	SIZE	2020-10-09	99.11	99.25	98.67	98.88	59228	{}	\N	2020-10-12 16:07:04.136278
69864	SIZE	2020-10-12	99.11	99.625	99.0852	99.5167	8637	{}	\N	2020-10-12 16:07:04.141004
69865	SKF	2020-09-28	11.76	11.76	11.365	11.51	101013	{}	\N	2020-10-12 16:07:05.086249
69866	SKF	2020-09-29	11.6	11.8	11.52	11.71	67611	{}	\N	2020-10-12 16:07:05.098092
69867	SKF	2020-09-30	11.7	11.71	11.33	11.53	259894	{}	\N	2020-10-12 16:07:05.105263
69868	SKF	2020-10-01	11.36	11.53	11.2169	11.31	156569	{}	\N	2020-10-12 16:07:05.111266
69869	SKF	2020-10-02	11.67	11.68	11.05	11.13	314896	{}	\N	2020-10-12 16:07:05.117344
69870	SKF	2020-10-05	10.9225	11.05	10.815	10.84	146146	{}	\N	2020-10-12 16:07:05.122925
69871	SKF	2020-10-06	10.67	11.1	10.555	11.06	175394	{}	\N	2020-10-12 16:07:05.128399
69872	SKF	2020-10-07	10.87	10.895	10.68	10.76	146892	{}	\N	2020-10-12 16:07:05.13365
69873	SKF	2020-10-08	10.62	10.67	10.4301	10.48	104282	{}	\N	2020-10-12 16:07:05.138842
69874	SKF	2020-10-09	10.41	10.56	10.3349	10.43	165915	{}	\N	2020-10-12 16:07:05.143971
69875	SKF	2020-10-12	10.44	10.44	10.2543	10.31	37743	{}	\N	2020-10-12 16:07:05.149377
69876	SLCA	2020-09-28	2.96	3.245	2.96	3.09	1090303	{}	\N	2020-10-12 16:07:08.553673
69877	SLCA	2020-09-29	3.08	3.11	2.98	3.06	1458169	{}	\N	2020-10-12 16:07:08.564329
69878	SLCA	2020-09-30	3.05	3.16	2.96	3	1093423	{}	\N	2020-10-12 16:07:08.571923
69879	SLCA	2020-10-01	2.95	2.975	2.65	2.69	3495550	{}	\N	2020-10-12 16:07:08.578805
69880	SLCA	2020-10-02	2.54	2.795	2.445	2.77	1693811	{}	\N	2020-10-12 16:07:08.583374
69881	SLCA	2020-10-05	2.8	2.85	2.67	2.81	1639378	{}	\N	2020-10-12 16:07:08.587863
69882	SLCA	2020-10-06	2.89	3.08	2.795	2.85	2160925	{}	\N	2020-10-12 16:07:08.591804
69883	SLCA	2020-10-07	2.88	2.92	2.715	2.88	1205449	{}	\N	2020-10-12 16:07:08.595804
69884	SLCA	2020-10-08	2.86	3.03	2.8414	2.99	1355656	{}	\N	2020-10-12 16:07:08.599834
69885	SLCA	2020-10-09	3.08	3.14	2.9569	3.05	1535403	{}	\N	2020-10-12 16:07:08.604074
69886	SLCA	2020-10-12	3.05	3.05	2.88	2.895	482636	{}	\N	2020-10-12 16:07:08.607823
69887	SLVP	2020-09-28	15.48	16.29	15.101	15.35	122973	{}	\N	2020-10-12 16:07:09.597169
69888	SLVP	2020-09-29	15.5	15.7699	15.455	15.68	169236	{}	\N	2020-10-12 16:07:09.609024
69889	SLVP	2020-09-30	15.56	15.75	15.34	15.58	435718	{}	\N	2020-10-12 16:07:09.61566
69890	SLVP	2020-10-01	15.92	16.14	15.72	16.03	207494	{}	\N	2020-10-12 16:07:09.621339
69891	SLVP	2020-10-02	15.895	16	15.685	15.76	86128	{}	\N	2020-10-12 16:07:09.627162
69892	SLVP	2020-10-05	16.06	16.3009	15.99	16.2092	129275	{}	\N	2020-10-12 16:07:09.632948
69893	SLVP	2020-10-06	16.05	16.41	15.56	15.56	198890	{}	\N	2020-10-12 16:07:09.638089
69894	SLVP	2020-10-07	15.78	15.88	15.5973	15.67	69803	{}	\N	2020-10-12 16:07:09.643191
69895	SLVP	2020-10-08	15.85	16.045	15.85	16.01	63128	{}	\N	2020-10-12 16:07:09.64808
69896	SLVP	2020-10-09	16.49	16.99	16.385	16.96	264753	{}	\N	2020-10-12 16:07:09.652841
69897	SLVP	2020-10-12	17	17.0399	16.8	17.01	106377	{}	\N	2020-10-12 16:07:09.657666
69898	SO	2020-09-28	54.04	54.7	53.61	53.71	4565185	{}	\N	2020-10-12 16:07:10.666817
69899	SO	2020-09-29	53.95	54.18	53.22	53.7	5775834	{}	\N	2020-10-12 16:07:10.67726
69900	SO	2020-09-30	53.98	54.5	53.88	54.22	8898219	{}	\N	2020-10-12 16:07:10.684787
69901	SO	2020-10-01	54.3	54.735	53.975	54.62	10820819	{}	\N	2020-10-12 16:07:10.691086
69902	SO	2020-10-02	54.31	56.57	54.09	56.26	12777787	{}	\N	2020-10-12 16:07:10.696872
69903	SO	2020-10-05	56.32	57.3591	56.27	57.17	10427200	{}	\N	2020-10-12 16:07:10.701645
69904	SO	2020-10-06	57.04	58.83	56.92	57.83	9272615	{}	\N	2020-10-12 16:07:10.706584
69905	SO	2020-10-07	58.18	58.61	57.68	57.8	6401057	{}	\N	2020-10-12 16:07:10.711499
69906	SO	2020-10-08	58	58.955	57.9	58.71	5947480	{}	\N	2020-10-12 16:07:10.716076
69907	SO	2020-10-09	59	59.2	58.38	58.74	7618527	{}	\N	2020-10-12 16:07:10.720519
69908	SO	2020-10-12	58.69	59.1901	58.44	59.04	2877242	{}	\N	2020-10-12 16:07:10.724948
69909	SFM	2020-09-28	21.09	21.28	20.62	21.18	1995637	{}	\N	2020-10-12 16:07:12.854224
69910	SFM	2020-09-29	21.18	21.35	20.8001	21.12	2295449	{}	\N	2020-10-12 16:07:12.867804
69911	SFM	2020-09-30	21.26	21.27	20.745	20.93	2326438	{}	\N	2020-10-12 16:07:12.874779
69912	SFM	2020-10-01	20.96	21.23	20.67	21.16	1232813	{}	\N	2020-10-12 16:07:12.880265
69913	SFM	2020-10-02	20.78	21.035	20.49	20.69	1845993	{}	\N	2020-10-12 16:07:12.885881
69914	SFM	2020-10-05	20.86	21.38	20.815	21.34	1989346	{}	\N	2020-10-12 16:07:12.891749
69915	SFM	2020-10-06	21.4	21.7	21.06	21.09	1427774	{}	\N	2020-10-12 16:07:12.896638
69916	SFM	2020-10-07	21.11	21.47	21.11	21.17	1088420	{}	\N	2020-10-12 16:07:12.901291
69917	SFM	2020-10-08	21.36	21.36	20.935	21.29	1184936	{}	\N	2020-10-12 16:07:12.906146
69918	SFM	2020-10-09	21.35	21.86	21.35	21.79	1262542	{}	\N	2020-10-12 16:07:12.910858
69919	SFM	2020-10-12	21.87	22.09	21.8	21.93	476764	{}	\N	2020-10-12 16:07:12.915491
69920	SGBX	2020-09-28	2.1	2.34	2.01	2.18	2649045	{}	\N	2020-10-12 16:07:14.341538
69921	SGBX	2020-09-29	2.1	2.1362	1.86	1.94	1565963	{}	\N	2020-10-12 16:07:14.355023
69922	SGBX	2020-09-30	1.88	1.91	1.71	1.81	1217728	{}	\N	2020-10-12 16:07:14.361478
69923	SGBX	2020-10-01	1.79	2.02	1.79	2.01	639720	{}	\N	2020-10-12 16:07:14.367112
69924	SGBX	2020-10-02	1.9	2	1.88	1.9	366158	{}	\N	2020-10-12 16:07:14.372144
69925	SGBX	2020-10-05	1.95	1.99	1.86	1.86	385177	{}	\N	2020-10-12 16:07:14.377536
69926	SGBX	2020-10-06	1.87	2.14	1.87	1.9	1294767	{}	\N	2020-10-12 16:07:14.381561
69927	SGBX	2020-10-07	1.88	2.15	1.88	2.14	1017214	{}	\N	2020-10-12 16:07:14.385551
69928	SGBX	2020-10-08	2.14	2.22	2.05	2.18	509169	{}	\N	2020-10-12 16:07:14.389643
69929	SGBX	2020-10-09	2.12	2.19	2.05	2.06	247200	{}	\N	2020-10-12 16:07:14.39395
69930	SGBX	2020-10-12	2.09	2.1	2.02	2.0345	70552	{}	\N	2020-10-12 16:07:14.398283
69931	SGLBW	2020-09-28	0.1799	0.1999	0.1799	0.1999	1100	{}	\N	2020-10-12 16:07:15.833705
69932	SGLBW	2020-09-29	0.1998	0.1998	0.190001	0.190001	2150	{}	\N	2020-10-12 16:07:15.841893
69933	SGLBW	2020-09-30	0.18	0.1809	0.18	0.1801	3602	{}	\N	2020-10-12 16:07:15.847772
69934	SGLBW	2020-10-05	0.171	0.171	0.171	0.171	250	{}	\N	2020-10-12 16:07:15.853197
69935	SGLBW	2020-10-06	0.1701	0.1701	0.1701	0.1701	283	{}	\N	2020-10-12 16:07:15.85838
69936	SGLBW	2020-10-07	0.1401	0.1401	0.1401	0.1401	740	{}	\N	2020-10-12 16:07:15.862758
69937	SGOL	2020-09-28	18.01	18.105	17.9359	18.1	1202898	{}	\N	2020-10-12 16:07:18.547592
69938	SGOL	2020-09-29	18.16	18.26	18.13	18.25	1222479	{}	\N	2020-10-12 16:07:18.560165
69939	SGOL	2020-09-30	18.21	18.29	18.09	18.15	1248544	{}	\N	2020-10-12 16:07:18.568029
69940	SGOL	2020-10-01	18.3	18.3897	18.24	18.31	2211071	{}	\N	2020-10-12 16:07:18.574601
69941	SGOL	2020-10-02	18.31	18.36	18.25	18.29	4563855	{}	\N	2020-10-12 16:07:18.580757
69942	SGOL	2020-10-05	18.34	18.45	18.33	18.38	1323298	{}	\N	2020-10-12 16:07:18.585994
69943	SGOL	2020-10-06	18.45	18.47	18.14	18.16	5940797	{}	\N	2020-10-12 16:07:18.591433
69944	SGOL	2020-10-07	18.135	18.17	18.07	18.14	2384036	{}	\N	2020-10-12 16:07:18.596269
69945	SGOL	2020-10-08	18.25	18.2781	18.1	18.21	1319283	{}	\N	2020-10-12 16:07:18.600975
69946	SGOL	2020-10-09	18.45	18.56	18.43	18.54	1204257	{}	\N	2020-10-12 16:07:18.605915
69947	SGOL	2020-10-12	18.5	18.52	18.46	18.4899	510421	{}	\N	2020-10-12 16:07:18.61074
69948	SH	2020-09-28	20.45	20.5061	20.33	20.39	9862767	{}	\N	2020-10-12 16:07:19.802944
69949	SH	2020-09-29	20.4	20.535	20.3463	20.5	5304591	{}	\N	2020-10-12 16:07:19.814745
69950	SH	2020-09-30	20.44	20.4583	20.12	20.32	13113869	{}	\N	2020-10-12 16:07:19.821819
69951	SH	2020-10-01	20.16	20.33	20.108	20.21	8064156	{}	\N	2020-10-12 16:07:19.828072
69952	SH	2020-10-02	20.5272	20.555	20.27	20.4	9427934	{}	\N	2020-10-12 16:07:19.833599
69953	SH	2020-10-05	20.25	20.2519	20.02	20.03	7323182	{}	\N	2020-10-12 16:07:19.838956
69954	SH	2020-10-06	20.01	20.35	19.89	20.31	13737499	{}	\N	2020-10-12 16:07:19.843784
69955	SH	2020-10-07	20.13	20.13	19.91	19.96	5670072	{}	\N	2020-10-12 16:07:19.848531
69956	SH	2020-10-08	19.83	19.895	19.78	19.79	6113177	{}	\N	2020-10-12 16:07:19.853146
69957	SH	2020-10-09	19.68	19.72	19.58	19.59	7794673	{}	\N	2020-10-12 16:07:19.857841
69958	SH	2020-10-12	19.44	19.48	19.31	19.32792	3366730	{}	\N	2020-10-12 16:07:19.862561
69959	SHV	2020-09-28	110.7	110.71	110.7	110.7	1573632	{}	\N	2020-10-12 16:07:21.928763
69960	SHV	2020-09-29	110.7	110.72	110.7	110.71	2496153	{}	\N	2020-10-12 16:07:21.939823
69961	SHV	2020-09-30	110.7	110.71	110.7	110.7	1896253	{}	\N	2020-10-12 16:07:21.946972
69962	SHV	2020-10-01	110.69	110.71	110.69	110.7	1162413	{}	\N	2020-10-12 16:07:21.953185
69963	SHV	2020-10-02	110.69	110.7	110.69	110.69	2698377	{}	\N	2020-10-12 16:07:21.958463
69964	SHV	2020-10-05	110.69	110.7	110.69	110.69	1281405	{}	\N	2020-10-12 16:07:21.964763
69965	SHV	2020-10-06	110.69	110.7	110.68	110.7	2816921	{}	\N	2020-10-12 16:07:21.969192
69966	SHV	2020-10-07	110.69	110.7	110.69	110.69	1209442	{}	\N	2020-10-12 16:07:21.974143
69967	SHV	2020-10-08	110.6983	110.7	110.69	110.69	2594410	{}	\N	2020-10-12 16:07:21.978645
69968	SHV	2020-10-09	110.7	110.7	110.69	110.69	1010566	{}	\N	2020-10-12 16:07:21.983178
69969	SHV	2020-10-12	110.68	110.69	110.68	110.685	653405	{}	\N	2020-10-12 16:07:21.987999
69970	SILC	2020-09-28	32.51	32.51	31.7	32.38	10226	{}	\N	2020-10-12 16:07:23.936514
69971	SILC	2020-09-29	31.54	32.42	31.54	32.35	15276	{}	\N	2020-10-12 16:07:23.943472
69972	SILC	2020-09-30	32.49	32.61	29.24	32.39	14779	{}	\N	2020-10-12 16:07:23.949572
69973	SILC	2020-10-01	31.6144	32.25	31.6144	32	14113	{}	\N	2020-10-12 16:07:23.956139
69974	SILC	2020-10-02	31.02	31.775	31	31.4	15111	{}	\N	2020-10-12 16:07:23.962031
69975	SILC	2020-10-05	32.46	32.52	31.6	32.44	30241	{}	\N	2020-10-12 16:07:23.968009
69976	SILC	2020-10-06	32.12	32.5	31.8	32.4	14750	{}	\N	2020-10-12 16:07:23.973477
69977	SILC	2020-10-07	32.18	32.5867	32.04	32.21	8565	{}	\N	2020-10-12 16:07:23.978991
69978	SILC	2020-10-08	31.805	32.89	31.795	32.89	11344	{}	\N	2020-10-12 16:07:23.98489
69979	SILC	2020-10-09	32.88	33.4199	32.222	33.4	10045	{}	\N	2020-10-12 16:07:23.990699
69980	SILC	2020-10-12	33.73	33.73	32.93	32.93	1803	{}	\N	2020-10-12 16:07:23.996519
69981	SITE	2020-09-28	120.3	122.255	119.71	120.46	188883	{}	\N	2020-10-12 16:07:25.124697
69982	SITE	2020-09-29	121.62	122.75	120.23	121.43	244081	{}	\N	2020-10-12 16:07:25.141832
69983	SITE	2020-09-30	122.57	124.71	121.4783	121.95	454723	{}	\N	2020-10-12 16:07:25.14958
69984	SITE	2020-10-01	122.65	124.99	120.55	124.75	405659	{}	\N	2020-10-12 16:07:25.157222
69985	SITE	2020-10-02	122.57	127.175	121.58	126.32	372515	{}	\N	2020-10-12 16:07:25.163947
69986	SITE	2020-10-05	127.27	129.75	126.51	129.08	537585	{}	\N	2020-10-12 16:07:25.169837
69987	SITE	2020-10-06	130.08	130.83	126.585	126.81	551678	{}	\N	2020-10-12 16:07:25.175411
69988	SITE	2020-10-07	128.96	129.82	127.77	127.83	604678	{}	\N	2020-10-12 16:07:25.18123
69989	SITE	2020-10-08	129.07	132.045	126.47	129.72	459333	{}	\N	2020-10-12 16:07:25.186724
69990	SITE	2020-10-09	131	131.04	128.89	130.83	401374	{}	\N	2020-10-12 16:07:25.192493
69991	SITE	2020-10-12	131.5	135.1	131.5	134.275	101305	{}	\N	2020-10-12 16:07:25.197944
69992	SJNK	2020-09-28	25.73	25.805	25.71	25.78	5768888	{}	\N	2020-10-12 16:07:26.213541
69993	SJNK	2020-09-29	25.78	25.8052	25.7039	25.77	3834390	{}	\N	2020-10-12 16:07:26.224851
69994	SJNK	2020-09-30	25.8	25.9	25.79	25.87	8653530	{}	\N	2020-10-12 16:07:26.233075
69995	SJNK	2020-10-01	25.8	25.84	25.77	25.81	5600942	{}	\N	2020-10-12 16:07:26.239928
69996	SJNK	2020-10-02	25.71	25.83	25.71	25.8	4579161	{}	\N	2020-10-12 16:07:26.245967
69997	SJNK	2020-10-05	25.83	25.96	25.82	25.95	6291046	{}	\N	2020-10-12 16:07:26.251812
69998	SJNK	2020-10-06	25.97	26.03	25.88	25.89	4474437	{}	\N	2020-10-12 16:07:26.257293
69999	SJNK	2020-10-07	25.99	25.9971	25.94	25.96	3246557	{}	\N	2020-10-12 16:07:26.262827
70000	SJNK	2020-10-08	26.06	26.06	25.9956	26.02	2155088	{}	\N	2020-10-12 16:07:26.268195
70001	SJNK	2020-10-09	26.05	26.1	26.03	26.07	5036368	{}	\N	2020-10-12 16:07:26.274004
70002	SJNK	2020-10-12	26.15	26.19	26.14	26.185	698895	{}	\N	2020-10-12 16:07:26.280478
70003	SELB	2020-09-28	2.52	2.56	2.44	2.48	467499	{}	\N	2020-10-12 16:07:27.231083
70004	SELB	2020-09-29	2.5	2.57	2.4401	2.56	490534	{}	\N	2020-10-12 16:07:27.243168
70005	SELB	2020-09-30	2.55	2.6	2.44	2.48	7282601	{}	\N	2020-10-12 16:07:27.250963
70006	SELB	2020-10-01	1.56	1.82	1.48	1.64	13309933	{}	\N	2020-10-12 16:07:27.257271
70007	SELB	2020-10-02	1.55	1.72	1.4701	1.67	8142783	{}	\N	2020-10-12 16:07:27.262985
70008	SELB	2020-10-05	1.64	1.7	1.63	1.65	2439124	{}	\N	2020-10-12 16:07:27.268409
70009	SELB	2020-10-06	1.61	1.715	1.6	1.64	5078381	{}	\N	2020-10-12 16:07:27.272819
70010	SELB	2020-10-07	1.61	1.71	1.5	1.7	3685477	{}	\N	2020-10-12 16:07:27.277788
70011	SELB	2020-10-08	1.7	2	1.67	1.93	4174194	{}	\N	2020-10-12 16:07:27.282532
70012	SELB	2020-10-09	2.1	2.2897	1.8689	2.02	6099223	{}	\N	2020-10-12 16:07:27.286999
70013	SELB	2020-10-12	2.02	2.1989	2.01	2.0523	1236168	{}	\N	2020-10-12 16:07:27.291541
70014	SERV	2020-09-28	40.74	41.63	40.74	41.38	599406	{}	\N	2020-10-12 16:07:28.10399
70015	SERV	2020-09-29	41.36	41.48	40.29	40.42	1564356	{}	\N	2020-10-12 16:07:28.111822
70016	SERV	2020-09-30	40.44	40.8	39.65	39.88	1064891	{}	\N	2020-10-12 16:07:28.118369
70017	SERV	2020-10-01	40.09	40.89	39.85	40.48	1631131	{}	\N	2020-10-12 16:07:28.124014
70018	SERV	2020-10-02	40	41	39.97	40.91	797930	{}	\N	2020-10-12 16:07:28.1287
70019	SGDM	2020-09-28	33.48	33.649	32.71	33.05	25969	{}	\N	2020-10-12 16:07:29.761015
70020	SGDM	2020-09-29	33.33	33.815	33.3	33.645	21132	{}	\N	2020-10-12 16:07:29.772716
70021	SGDM	2020-09-30	33.5	33.795	33.03	33.51	19645	{}	\N	2020-10-12 16:07:29.780924
70022	SGDM	2020-10-01	33.9	34.13	33.5	33.85	22073	{}	\N	2020-10-12 16:07:29.78779
70023	SGDM	2020-10-02	33.52	33.795	33.28	33.28	31622	{}	\N	2020-10-12 16:07:29.793797
70024	SGDM	2020-10-05	33.58	34.23	33.35	33.86	47041	{}	\N	2020-10-12 16:07:29.799763
70025	SGDM	2020-10-06	34.04	34.12	32.4301	32.467	39079	{}	\N	2020-10-12 16:07:29.804958
70026	SGDM	2020-10-07	32.67	32.845	32.39	32.475	23515	{}	\N	2020-10-12 16:07:29.812862
70027	SGDM	2020-10-08	32.86	33.1839	32.722	33.1	19394	{}	\N	2020-10-12 16:07:29.817637
70028	SGDM	2020-10-09	33.77	34.72	33.77	34.72	38886	{}	\N	2020-10-12 16:07:29.822641
70029	SGDM	2020-10-12	34.74	34.81	34.3608	34.725	30158	{}	\N	2020-10-12 16:07:29.828117
70030	SGOC	2020-09-28	0.87	0.87	0.82	0.86	3252	{}	\N	2020-10-12 16:07:30.860543
70031	SGOC	2020-09-29	0.859899	0.86	0.82	0.85	19227	{}	\N	2020-10-12 16:07:30.87087
70032	SGOC	2020-09-30	0.8301	0.86	0.8301	0.86	16868	{}	\N	2020-10-12 16:07:30.878706
70033	SGOC	2020-10-01	0.85	0.85	0.849799	0.85	4052	{}	\N	2020-10-12 16:07:30.885498
70034	SGOC	2020-10-02	0.84	0.846492	0.82	0.84	4286	{}	\N	2020-10-12 16:07:30.891618
70035	SGOC	2020-10-05	0.83	0.84	0.82	0.84	20904	{}	\N	2020-10-12 16:07:30.896897
70036	SGOC	2020-10-06	0.85	0.8548	0.8	0.85	3821	{}	\N	2020-10-12 16:07:30.901825
70037	SGOC	2020-10-07	0.8	0.8442	0.8	0.84	1617	{}	\N	2020-10-12 16:07:30.906899
70038	SGOC	2020-10-08	0.858503	0.91875	0.801598	0.9	15789	{}	\N	2020-10-12 16:07:30.912034
70039	SGOC	2020-10-09	0.8515	0.8996	0.8031	0.8031	2454	{}	\N	2020-10-12 16:07:30.917181
70040	SGOC	2020-10-12	0.8849	0.8866	0.842	0.842	2148	{}	\N	2020-10-12 16:07:30.922319
70041	SGMA	2020-09-28	2.92	3.15	2.92	3.15	17662	{}	\N	2020-10-12 16:07:31.851221
70042	SGMA	2020-09-29	3.0833	3.15	3.0243	3.14	26517	{}	\N	2020-10-12 16:07:31.863088
70043	SGMA	2020-09-30	3.09	3.15	3.07	3.13	18806	{}	\N	2020-10-12 16:07:31.870483
70044	SGMA	2020-10-01	3.08	3.18	3.08	3.14	9888	{}	\N	2020-10-12 16:07:31.876717
70045	SGMA	2020-10-02	3	3.23	3	3.22	49478	{}	\N	2020-10-12 16:07:31.882981
70046	SGMA	2020-10-05	3.2064	3.63	3.2064	3.35	210509	{}	\N	2020-10-12 16:07:31.888853
70047	SGMA	2020-10-06	3.49	3.49	3.15	3.21	102755	{}	\N	2020-10-12 16:07:31.894005
70048	SGMA	2020-10-07	3.22	3.2617	3.1375	3.22	51173	{}	\N	2020-10-12 16:07:31.899463
70049	SGMA	2020-10-08	3.22	3.55	3.12	3.53	349503	{}	\N	2020-10-12 16:07:31.903737
70050	SGMA	2020-10-09	3.48	3.49	3.33	3.37	37293	{}	\N	2020-10-12 16:07:31.907868
70051	SGMA	2020-10-12	3.34	3.39	3.33	3.3822	18314	{}	\N	2020-10-12 16:07:31.91198
70052	SHLX	2020-09-28	9.3	9.535	9.19	9.49	681079	{}	\N	2020-10-12 16:07:33.277007
70053	SHLX	2020-09-29	9.51	9.77	9.4	9.67	2898757	{}	\N	2020-10-12 16:07:33.28782
70054	SHLX	2020-09-30	9.76	9.83	9.38	9.46	3809497	{}	\N	2020-10-12 16:07:33.295638
70055	SHLX	2020-10-01	9.38	9.465	9.21	9.22	2358945	{}	\N	2020-10-12 16:07:33.302345
70056	SHLX	2020-10-02	9.05	9.54	9.05	9.49	2048842	{}	\N	2020-10-12 16:07:33.308905
70057	SHLX	2020-10-05	9.63	9.98	9.41	9.52	2636905	{}	\N	2020-10-12 16:07:33.315001
70058	SHLX	2020-10-06	9.55	9.65	9.09	9.18	2762402	{}	\N	2020-10-12 16:07:33.320382
70059	SHLX	2020-10-07	9.24	9.25	9.13	9.15	3041473	{}	\N	2020-10-12 16:07:33.325745
70060	SHLX	2020-10-08	9.19	9.61	9.08	9.6	4038530	{}	\N	2020-10-12 16:07:33.331472
70061	SHLX	2020-10-09	9.63	9.76	9.45	9.6	2117233	{}	\N	2020-10-12 16:07:33.337759
70062	SHLX	2020-10-12	9.58	9.58	9.37	9.51	1000395	{}	\N	2020-10-12 16:07:33.343892
70063	SIF	2020-09-28	3.64	3.75	3.64	3.66	3439	{}	\N	2020-10-12 16:07:34.684479
70064	SIF	2020-09-29	3.64	3.64	3.64	3.64	1299	{}	\N	2020-10-12 16:07:34.697223
70065	SIF	2020-09-30	3.86	3.86	3.68	3.68	446	{}	\N	2020-10-12 16:07:34.704772
70066	SIF	2020-10-01	3.7001	3.87	3.5603	3.61	7412	{}	\N	2020-10-12 16:07:34.711075
70067	SIF	2020-10-02	3.55	3.61	3.55	3.61	869	{}	\N	2020-10-12 16:07:34.716533
70068	SIF	2020-10-05	3.705	3.705	3.56	3.56	888	{}	\N	2020-10-12 16:07:34.7221
70069	SIF	2020-10-06	3.8	3.8697	3.55	3.57	3550	{}	\N	2020-10-12 16:07:34.72684
70070	SIF	2020-10-07	3.8	3.8	3.6	3.6	4012	{}	\N	2020-10-12 16:07:34.731675
70071	SIF	2020-10-08	3.7	3.7818	3.7	3.71	5194	{}	\N	2020-10-12 16:07:34.736331
70072	SIF	2020-10-12	3.8	3.8	3.78	3.78	596	{}	\N	2020-10-12 16:07:34.741792
70073	SIM	2020-09-28	6.04	6.4999	6.04	6.4999	699	{}	\N	2020-10-12 16:07:36.928334
70074	SIM	2020-09-29	6.34	6.7	6.34	6.53	1012	{}	\N	2020-10-12 16:07:36.939682
70075	SIM	2020-09-30	6.09	7.02	6.01	7.02	1112	{}	\N	2020-10-12 16:07:36.948996
70076	SIM	2020-10-01	6.97	6.97	6.4	6.8	1190	{}	\N	2020-10-12 16:07:36.955591
70077	SIM	2020-10-05	6.42	6.99	6.27	6.94	5516	{}	\N	2020-10-12 16:07:36.961673
70078	SIM	2020-10-06	6.94	6.94	6.94	6.94	698	{}	\N	2020-10-12 16:07:36.967077
70079	SIM	2020-10-07	6.49	7.5	6.49	7.49	3357	{}	\N	2020-10-12 16:07:36.972163
70080	SIM	2020-10-09	7.5	7.5	6.99	7	2247	{}	\N	2020-10-12 16:07:36.977193
70081	SIM	2020-10-12	6.75	7.44	6.75	7.4	4582	{}	\N	2020-10-12 16:07:36.982462
70082	SJT	2020-09-28	2.58	2.64	2.58	2.62	11733	{}	\N	2020-10-12 16:07:38.986885
70083	SJT	2020-09-29	2.55	2.62	2.55	2.61	64256	{}	\N	2020-10-12 16:07:38.996673
70084	SJT	2020-09-30	2.57	2.62	2.51	2.51	72156	{}	\N	2020-10-12 16:07:39.003593
70085	SJT	2020-10-01	2.51	2.6	2.51	2.6	45056	{}	\N	2020-10-12 16:07:39.009573
70086	SJT	2020-10-02	2.51	2.59	2.51	2.58	23924	{}	\N	2020-10-12 16:07:39.0144
70087	SJT	2020-10-05	2.52	2.7	2.45	2.58	371512	{}	\N	2020-10-12 16:07:39.019222
70088	SJT	2020-10-06	2.58	2.69	2.46	2.49	203612	{}	\N	2020-10-12 16:07:39.023639
70089	SJT	2020-10-07	2.5	2.588	2.5	2.54	51528	{}	\N	2020-10-12 16:07:39.028076
70090	SJT	2020-10-08	2.52	2.58	2.51	2.55	143206	{}	\N	2020-10-12 16:07:39.032264
70091	SJT	2020-10-09	2.51	2.61	2.51	2.5698	178716	{}	\N	2020-10-12 16:07:39.036926
70092	SJT	2020-10-12	2.54	2.6391	2.54	2.5977	90870	{}	\N	2020-10-12 16:07:39.041198
70093	SKX	2020-09-28	30.82	30.95	29.63	29.67	2088413	{}	\N	2020-10-12 16:07:40.449037
70094	SKX	2020-09-29	29.51	29.62	28.25	28.61	4118037	{}	\N	2020-10-12 16:07:40.462003
70095	SKX	2020-09-30	28.55	30.38	28.55	30.22	5921353	{}	\N	2020-10-12 16:07:40.471368
70096	SKX	2020-10-01	30.95	31.93	30.16	31.53	4088806	{}	\N	2020-10-12 16:07:40.478429
70097	SKX	2020-10-02	30.4	32.355	30.31	32.25	2814076	{}	\N	2020-10-12 16:07:40.48393
70098	SKX	2020-10-05	32.66	32.93	32.15	32.32	2739483	{}	\N	2020-10-12 16:07:40.489117
70099	SKX	2020-10-06	32.64	33.19	31.785	31.89	2322573	{}	\N	2020-10-12 16:07:40.494676
70100	SKX	2020-10-07	32.46	33.33	32.16	33.11	2667474	{}	\N	2020-10-12 16:07:40.50002
70101	SKX	2020-10-08	33.34	33.97	33.295	33.57	2829027	{}	\N	2020-10-12 16:07:40.505984
70102	SKX	2020-10-09	33.7	33.84	32.84	32.98	2200404	{}	\N	2020-10-12 16:07:40.511727
70103	SKX	2020-10-12	33.18	33.34	32.68	32.95	1181863	{}	\N	2020-10-12 16:07:40.517194
70104	SOJC	2020-09-28	26.3	26.4	26.17	26.23	46430	{}	\N	2020-10-12 16:07:41.506234
70105	SOJC	2020-09-29	26.29	26.29	26.1	26.19	95746	{}	\N	2020-10-12 16:07:41.517147
70106	SOJC	2020-09-30	26.19	26.26	26.04	26.04	220726	{}	\N	2020-10-12 16:07:41.524718
70107	SOJC	2020-10-01	26.16	26.32	26.11	26.3	45296	{}	\N	2020-10-12 16:07:41.532265
70108	SOJC	2020-10-02	26.2	26.39	26.2	26.37	102494	{}	\N	2020-10-12 16:07:41.538705
70109	SOJC	2020-10-05	26.4	26.6	26.4	26.6	73456	{}	\N	2020-10-12 16:07:41.543963
70110	SOJC	2020-10-06	26.6	26.75	26.56	26.65	59604	{}	\N	2020-10-12 16:07:41.548803
70111	SOJC	2020-10-07	26.7	26.84	26.6614	26.84	85449	{}	\N	2020-10-12 16:07:41.55336
70112	SOJC	2020-10-08	26.86	26.9	26.7846	26.805	44772	{}	\N	2020-10-12 16:07:41.558138
70113	SOJC	2020-10-09	26.87	26.87	26.74	26.81	34230	{}	\N	2020-10-12 16:07:41.562946
70114	SOJC	2020-10-12	26.76	26.81	26.73	26.81	10962	{}	\N	2020-10-12 16:07:41.567926
70115	SENEA	2020-09-28	35.83	36.69	35.83	36.31	25934	{}	\N	2020-10-12 16:07:42.578318
70116	SENEA	2020-09-29	36.18	36.54	35.85	36.14	28869	{}	\N	2020-10-12 16:07:42.588978
70117	SENEA	2020-09-30	36.21	36.72	35.495	35.73	40198	{}	\N	2020-10-12 16:07:42.596341
70118	SENEA	2020-10-01	35.7	36.97	35.45	36.66	31271	{}	\N	2020-10-12 16:07:42.602616
70119	SENEA	2020-10-02	36.5	37.425	36.5	37.24	24955	{}	\N	2020-10-12 16:07:42.608355
70120	SENEA	2020-10-05	37.57	38.89	37.05	38.76	39755	{}	\N	2020-10-12 16:07:42.613374
70121	SENEA	2020-10-06	39.07	41.75	39.025	40.56	58668	{}	\N	2020-10-12 16:07:42.618129
70122	SENEA	2020-10-07	41.25	41.4746	37.8239	39.38	43696	{}	\N	2020-10-12 16:07:42.622859
70123	SENEA	2020-10-08	39.89	39.89	38.4	39.58	26225	{}	\N	2020-10-12 16:07:42.628463
70124	SENEA	2020-10-09	39.87	40.65	39.33	40.32	23262	{}	\N	2020-10-12 16:07:42.633382
70125	SENEA	2020-10-12	40.2	40.495	40.18	40.495	3188	{}	\N	2020-10-12 16:07:42.638232
70126	SFB	2020-09-28	26.35	26.88	26.3499	26.87	15804	{}	\N	2020-10-12 16:07:43.997884
70127	SFB	2020-09-29	26.81	27.1	26.63	27.1	47989	{}	\N	2020-10-12 16:07:44.009298
70128	SFB	2020-09-30	26.7	27.18	26.6429	27.18	88074	{}	\N	2020-10-12 16:07:44.017062
70129	SFB	2020-10-01	27.18	27.59	27.02	27.58	55664	{}	\N	2020-10-12 16:07:44.023574
70130	SFB	2020-10-02	27.31	27.59	27.2	27.54	21065	{}	\N	2020-10-12 16:07:44.029098
70131	SFB	2020-10-05	27.48	27.6	27.4	27.42	16097	{}	\N	2020-10-12 16:07:44.034526
70132	SFB	2020-10-06	27.58	27.84	27.51	27.64	16984	{}	\N	2020-10-12 16:07:44.039466
70133	SFB	2020-10-07	27.6	27.99	27.6	27.9	14508	{}	\N	2020-10-12 16:07:44.044228
70134	SFB	2020-10-08	27.8	27.99	27.738	27.79	21208	{}	\N	2020-10-12 16:07:44.049182
70135	SFB	2020-10-09	27.54	27.8	27.44	27.76	23036	{}	\N	2020-10-12 16:07:44.053955
70136	SFB	2020-10-12	27.68	27.79	27.65	27.79	8872	{}	\N	2020-10-12 16:07:44.058779
70137	SHAG	2020-09-28	51.54	51.56	51.5202	51.5599	9775	{}	\N	2020-10-12 16:07:44.920327
70138	SHAG	2020-09-29	51.5448	51.57	51.5448	51.5528	5188	{}	\N	2020-10-12 16:07:44.933475
70139	SHAG	2020-09-30	51.55	51.5699	51.52	51.545	10861	{}	\N	2020-10-12 16:07:44.941067
70140	SHAG	2020-10-01	51.5449	51.57	51.513	51.52	20115	{}	\N	2020-10-12 16:07:44.947874
70141	SHAG	2020-10-02	51.52	51.59	51.5116	51.59	16593	{}	\N	2020-10-12 16:07:44.953989
70142	SHAG	2020-10-05	51.5385	51.55	51.49	51.53	5231	{}	\N	2020-10-12 16:07:44.959629
70143	SHAG	2020-10-06	51.515	51.57	51.48	51.5293	12733	{}	\N	2020-10-12 16:07:44.964221
70144	SHAG	2020-10-07	51.52	51.58	51.495	51.54	28383	{}	\N	2020-10-12 16:07:44.968889
70145	SHAG	2020-10-08	51.48	51.56	51.48	51.5468	7651	{}	\N	2020-10-12 16:07:44.973738
70146	SHAG	2020-10-09	51.56	51.57	51.542	51.57	15735	{}	\N	2020-10-12 16:07:44.978566
70147	SHAG	2020-10-12	51.55	51.5599	51.516	51.5599	3247	{}	\N	2020-10-12 16:07:44.983404
70148	SHOP	2020-09-28	970.01	983.7199	951	966.5	1923565	{}	\N	2020-10-12 16:07:45.869155
70149	SHOP	2020-09-29	969.58	1030	961.4032	1025.7	6172270	{}	\N	2020-10-12 16:07:45.88234
70150	SHOP	2020-09-30	1037.69	1043	1014.35	1022.97	3843481	{}	\N	2020-10-12 16:07:45.891095
70151	SHOP	2020-10-01	1030.63	1063.41	1026.15	1054.43	3441944	{}	\N	2020-10-12 16:07:45.897768
70152	SHOP	2020-10-02	1020.5	1054	1017.01	1028.99	2549677	{}	\N	2020-10-12 16:07:45.903612
70153	SHOP	2020-10-05	1035.99	1065.89	1014.38	1065.53	3181962	{}	\N	2020-10-12 16:07:45.90962
70154	SHOP	2020-10-06	1068	1081.9999	1030.01	1041.25	2860972	{}	\N	2020-10-12 16:07:45.914765
70155	SHOP	2020-10-07	1059.18	1077.949	1048.16	1073.43	2504146	{}	\N	2020-10-12 16:07:45.919709
70156	SHOP	2020-10-08	1081.07	1086.77	1056.23	1069.21	2137732	{}	\N	2020-10-12 16:07:45.924924
70157	SHOP	2020-10-09	1072.58	1098.8773	1070	1096	2241150	{}	\N	2020-10-12 16:07:45.930177
70158	SHOP	2020-10-12	1105.13	1130	1085.3001	1095.183	1429516	{}	\N	2020-10-12 16:07:45.935183
70159	SIEN	2020-09-28	3.48	3.57	3.45	3.51	455842	{}	\N	2020-10-12 16:07:47.031725
70160	SIEN	2020-09-29	3.5	3.52	3.305	3.33	554246	{}	\N	2020-10-12 16:07:47.043793
70161	SIEN	2020-09-30	3.34	3.5101	3.3266	3.4	2102408	{}	\N	2020-10-12 16:07:47.05168
70162	SIEN	2020-10-01	3.47	3.57	3.37	3.41	592946	{}	\N	2020-10-12 16:07:47.058199
70163	SIEN	2020-10-02	3.32	3.6	3.3001	3.56	596013	{}	\N	2020-10-12 16:07:47.064092
70164	SIEN	2020-10-05	3.6	3.75	3.53	3.71	521219	{}	\N	2020-10-12 16:07:47.07001
70165	SIEN	2020-10-06	3.7	3.86	3.62	3.64	914386	{}	\N	2020-10-12 16:07:47.074998
70166	SIEN	2020-10-07	3.74	3.88	3.67	3.88	761370	{}	\N	2020-10-12 16:07:47.079916
70167	SIEN	2020-10-08	3.88	4.24	3.8	4.12	1180788	{}	\N	2020-10-12 16:07:47.084764
70168	SIEN	2020-10-09	4.15	4.64	4.15	4.48	1610506	{}	\N	2020-10-12 16:07:47.089845
70169	SIEN	2020-10-12	4.43	4.4922	4.22	4.28	342535	{}	\N	2020-10-12 16:07:47.09478
70170	SILJ	2020-09-28	13.73	13.83	13.42	13.59	922283	{}	\N	2020-10-12 16:07:48.384614
70171	SILJ	2020-09-29	13.82	14.06	13.69	13.96	1110274	{}	\N	2020-10-12 16:07:48.39551
70172	SILJ	2020-09-30	13.79	14.02	13.59	13.81	1106791	{}	\N	2020-10-12 16:07:48.403914
70173	SILJ	2020-10-01	14	14.25	13.85	14.1	577532	{}	\N	2020-10-12 16:07:48.41113
70174	SILJ	2020-10-02	13.94	14.11	13.76	13.9	557661	{}	\N	2020-10-12 16:07:48.417184
70175	SILJ	2020-10-05	14.02	14.38	14.02	14.3	563580	{}	\N	2020-10-12 16:07:48.423076
70176	SILJ	2020-10-06	14.29	14.4	13.5204	13.62	899280	{}	\N	2020-10-12 16:07:48.428233
70177	SILJ	2020-10-07	13.73	13.95	13.62	13.725	448770	{}	\N	2020-10-12 16:07:48.433259
70178	SILJ	2020-10-08	13.9	14.1	13.82	14.1	832005	{}	\N	2020-10-12 16:07:48.438154
70179	SILJ	2020-10-09	14.53	15.05	14.45	15.03	1803974	{}	\N	2020-10-12 16:07:48.442882
70180	SILJ	2020-10-12	15.07	15.1	14.8117	15.0782	619183	{}	\N	2020-10-12 16:07:48.447387
70181	SIVR	2020-09-28	22.56	22.9399	22.34	22.88	1079374	{}	\N	2020-10-12 16:07:49.342366
70182	SIVR	2020-09-29	23.1099	23.6	23.04	23.38	613799	{}	\N	2020-10-12 16:07:49.352282
70183	SIVR	2020-09-30	23.14	23.26	22.38	22.49	1093009	{}	\N	2020-10-12 16:07:49.359792
70184	SIVR	2020-10-01	22.87	23.37	22.63	22.94	774816	{}	\N	2020-10-12 16:07:49.366174
70185	SIVR	2020-10-02	23.12	23.39	22.91	22.97	926030	{}	\N	2020-10-12 16:07:49.37156
70186	SIVR	2020-10-05	23.18	23.71	23.18	23.54	666281	{}	\N	2020-10-12 16:07:49.376465
70187	SIVR	2020-10-06	23.53	23.62	22.49	22.59	1251216	{}	\N	2020-10-12 16:07:49.380848
70188	SIVR	2020-10-07	22.86	23.19	22.75	22.98	552387	{}	\N	2020-10-12 16:07:49.384938
70189	SIVR	2020-10-08	23.28	23.4	22.88	23.05	463454	{}	\N	2020-10-12 16:07:49.388894
70190	SIVR	2020-10-09	23.69	24.33	23.6638	24.32	927141	{}	\N	2020-10-12 16:07:49.39322
70191	SIVR	2020-10-12	24.25	24.3	23.97	24.185	294792	{}	\N	2020-10-12 16:07:49.398205
70192	SJM	2020-09-28	112.86	114.28	112.63	113.24	723941	{}	\N	2020-10-12 16:07:51.266252
70193	SJM	2020-09-29	113.04	113.71	112.34	113.01	767956	{}	\N	2020-10-12 16:07:51.278067
70194	SJM	2020-09-30	113.62	116.14	113.2	115.52	2110398	{}	\N	2020-10-12 16:07:51.28517
70195	SJM	2020-10-01	115.08	116.18	113.88	116	1339702	{}	\N	2020-10-12 16:07:51.291675
70196	SJM	2020-10-02	115.75	117.62	115.65	116.47	989330	{}	\N	2020-10-12 16:07:51.297039
70197	SJM	2020-10-05	116.32	117.68	115.47	117.38	1121437	{}	\N	2020-10-12 16:07:51.30197
70198	SJM	2020-10-06	117.38	118.75	116.785	117.5	1393602	{}	\N	2020-10-12 16:07:51.307187
70199	SJM	2020-10-07	117.37	118.18	116.31	117.45	1229597	{}	\N	2020-10-12 16:07:51.31207
70200	SJM	2020-10-08	118	118.19	116.48	117.13	1198798	{}	\N	2020-10-12 16:07:51.316953
70201	SJM	2020-10-09	117.39	118.75	116.62	118.68	1231859	{}	\N	2020-10-12 16:07:51.321774
70202	SJM	2020-10-12	119.12	120.59	119.12	120.07	365232	{}	\N	2020-10-12 16:07:51.326672
70203	SKY	2020-09-28	27.13	27.91	26.96	27.8	677945	{}	\N	2020-10-12 16:07:54.619159
70204	SKY	2020-09-29	27.89	28.0499	26.56	26.59	651267	{}	\N	2020-10-12 16:07:54.630549
70205	SKY	2020-09-30	26.68	27.34	26.36	26.77	570429	{}	\N	2020-10-12 16:07:54.638742
70206	SKY	2020-10-01	26.99	27.73	26.61	27.7	1420736	{}	\N	2020-10-12 16:07:54.645691
70207	SKY	2020-10-02	26.96	27.975	26.775	27.76	454991	{}	\N	2020-10-12 16:07:54.652148
70208	SKY	2020-10-05	27.95	28.71	27.45	28.64	663060	{}	\N	2020-10-12 16:07:54.657811
70209	SKY	2020-10-06	28.91	29.28	28.2	28.42	828599	{}	\N	2020-10-12 16:07:54.662859
70210	SKY	2020-10-07	28.93	29.06	27.71	28.68	623704	{}	\N	2020-10-12 16:07:54.668184
70211	SKY	2020-10-08	29.19	30.02	29.1	29.49	589930	{}	\N	2020-10-12 16:07:54.6729
70212	SKY	2020-10-09	29.75	29.87	28.87	29.54	433531	{}	\N	2020-10-12 16:07:54.677338
70213	SKY	2020-10-12	29.63	29.805	29.215	29.47	109800	{}	\N	2020-10-12 16:07:54.68205
70214	SLF	2020-09-28	40.78	41.28	40.78	41.12	348988	{}	\N	2020-10-12 16:07:55.807326
70215	SLF	2020-09-29	41.07	41.07	40.32	40.63	510609	{}	\N	2020-10-12 16:07:55.820391
70216	SLF	2020-09-30	40.88	41.3	40.61	40.73	790838	{}	\N	2020-10-12 16:07:55.828264
70217	SLF	2020-10-01	40.94	41.21	40.74	40.92	695136	{}	\N	2020-10-12 16:07:55.834751
70218	SLF	2020-10-02	40.44	41.32	40.44	41.21	480485	{}	\N	2020-10-12 16:07:55.840471
70219	SLF	2020-10-05	41.5	42	41.49	41.91	481192	{}	\N	2020-10-12 16:07:55.845858
70220	SLF	2020-10-06	42.16	42.27	41.485	41.75	798444	{}	\N	2020-10-12 16:07:55.850554
70221	SLF	2020-10-07	42.11	42.49	42	42.36	673875	{}	\N	2020-10-12 16:07:55.855116
70222	SLF	2020-10-08	42.56	42.89	42.22	42.31	880811	{}	\N	2020-10-12 16:07:55.859648
70223	SLF	2020-10-09	42.49	42.77	42.19	42.34	739923	{}	\N	2020-10-12 16:07:55.864088
70224	SLF	2020-10-12	42.45	42.73	42.4324	42.6	72848	{}	\N	2020-10-12 16:07:55.868713
70225	SLNO	2020-09-28	1.85	1.95	1.8	1.86	415304	{}	\N	2020-10-12 16:07:57.412771
70226	SLNO	2020-09-29	1.94	2.09	1.9	2.07	789592	{}	\N	2020-10-12 16:07:57.424051
70227	SLNO	2020-09-30	2.04	3	2	2.51	5620148	{}	\N	2020-10-12 16:07:57.43119
70228	SLNO	2020-10-01	2.2	2.29	2.05	2.225	1547556	{}	\N	2020-10-12 16:07:57.438168
70229	SLNO	2020-10-02	2.06	2.19	2.05	2.06	401766	{}	\N	2020-10-12 16:07:57.4441
70230	SLNO	2020-10-05	2.06	2.14	1.87	2.05	1417376	{}	\N	2020-10-12 16:07:57.449726
70231	SLNO	2020-10-06	1.95	2.03	1.94	1.97	460132	{}	\N	2020-10-12 16:07:57.454424
70232	SLNO	2020-10-07	1.98	2.12	1.98	2.1	400210	{}	\N	2020-10-12 16:07:57.459138
70233	SLNO	2020-10-08	2.12	2.1428	2.04	2.04	189447	{}	\N	2020-10-12 16:07:57.463908
70234	SLNO	2020-10-09	2.08	2.08	1.99	2.02	186034	{}	\N	2020-10-12 16:07:57.468658
70235	SLNO	2020-10-12	2.01	2.02	1.99	2.01	135085	{}	\N	2020-10-12 16:07:57.473378
70236	SLYG	2020-09-28	58.27	59.22	58.27	59	90576	{}	\N	2020-10-12 16:07:59.144552
70237	SLYG	2020-09-29	58.98	59.2647	58.431	58.93	74486	{}	\N	2020-10-12 16:07:59.156997
70238	SLYG	2020-09-30	59.1	59.941	58.61	59.03	77606	{}	\N	2020-10-12 16:07:59.164894
70239	SLYG	2020-10-01	59.33	59.87	58.8514	59.86	65994	{}	\N	2020-10-12 16:07:59.172514
70240	SLYG	2020-10-02	58.59	60.4199	58.59	60.18	85628	{}	\N	2020-10-12 16:07:59.178901
70241	SLYG	2020-10-05	60.56	61.51	60.56	61.4	159213	{}	\N	2020-10-12 16:07:59.185024
70242	SLYG	2020-10-06	61.85	62.82	61.3	61.35	151820	{}	\N	2020-10-12 16:07:59.191266
70243	SLYG	2020-10-07	62.13	62.4099	61.75	62.15	179039	{}	\N	2020-10-12 16:07:59.196925
70244	SLYG	2020-10-08	62.74	63	62.49	62.9	118455	{}	\N	2020-10-12 16:07:59.202918
70245	SLYG	2020-10-09	63.41	63.7	63.1101	63.46	99738	{}	\N	2020-10-12 16:07:59.20876
70246	SLYG	2020-10-12	63.73	64.01	63.51	63.9039	49871	{}	\N	2020-10-12 16:07:59.214897
70247	SMED	2020-09-28	6.38	6.98	6.38	6.83	298274	{}	\N	2020-10-12 16:08:00.152578
70248	SMED	2020-09-29	6.64	6.82	6.38	6.43	173043	{}	\N	2020-10-12 16:08:00.163715
70249	SMED	2020-09-30	6.4	6.42	6.2	6.27	274937	{}	\N	2020-10-12 16:08:00.171007
70250	SMED	2020-10-01	6.3	6.39	6.2156	6.3	224077	{}	\N	2020-10-12 16:08:00.177056
70251	SMED	2020-10-02	6.3	6.36	6.05	6.18	267831	{}	\N	2020-10-12 16:08:00.183209
70252	SMED	2020-10-05	6.24	6.4	6.05	6.35	342413	{}	\N	2020-10-12 16:08:00.188546
70253	SMED	2020-10-06	6.39	6.4	6.14	6.22	154726	{}	\N	2020-10-12 16:08:00.193323
70254	SMED	2020-10-07	6.32	6.42	6.22	6.35	124757	{}	\N	2020-10-12 16:08:00.198136
70255	SMED	2020-10-08	6.38	6.64	6.26	6.63	240477	{}	\N	2020-10-12 16:08:00.202983
70256	SMED	2020-10-09	6.73	6.7373	6.5336	6.69	109119	{}	\N	2020-10-12 16:08:00.207884
70257	SMED	2020-10-12	6.69	6.72	6.52	6.58	33966	{}	\N	2020-10-12 16:08:00.2127
70258	SIGI	2020-09-28	49.77	50.56	49.77	49.99	191515	{}	\N	2020-10-12 16:08:01.110817
70259	SIGI	2020-09-29	49.99	50.67	49.72	50.55	372937	{}	\N	2020-10-12 16:08:01.122253
70260	SIGI	2020-09-30	50.55	51.65	50.55	51.49	303633	{}	\N	2020-10-12 16:08:01.129979
70261	SIGI	2020-10-01	51.54	52.04	50.49	52.01	282468	{}	\N	2020-10-12 16:08:01.136328
70262	SIGI	2020-10-02	51.24	52.35	50.61	52.33	220273	{}	\N	2020-10-12 16:08:01.142025
70263	SIGI	2020-10-05	52.9	53.59	52.26	53.52	200008	{}	\N	2020-10-12 16:08:01.147498
70264	SIGI	2020-10-06	54.21	54.48	52.85	52.87	203451	{}	\N	2020-10-12 16:08:01.152279
70265	SIGI	2020-10-07	53.42	53.85	52.87	53.63	157493	{}	\N	2020-10-12 16:08:01.157429
70266	SIGI	2020-10-08	53.61	54.46	52.82	53.54	176798	{}	\N	2020-10-12 16:08:01.162947
70267	SIGI	2020-10-09	53.81	54.62	53.16	53.69	146366	{}	\N	2020-10-12 16:08:01.169064
70268	SIGI	2020-10-12	53.64	54.765	53.53	54.57	51221	{}	\N	2020-10-12 16:08:01.174888
70269	SIVB	2020-09-28	235.42	240.45	233.69	237.48	219698	{}	\N	2020-10-12 16:08:02.311014
70270	SIVB	2020-09-29	235.42	238.83	234.21	237.09	304026	{}	\N	2020-10-12 16:08:02.32178
70271	SIVB	2020-09-30	238.53	245.335	236.06	240.62	302156	{}	\N	2020-10-12 16:08:02.329621
70272	SIVB	2020-10-01	243.06	246.46	240.01	245.3	303332	{}	\N	2020-10-12 16:08:02.33629
70273	SIVB	2020-10-02	239.51	248.35	239.51	245.65	222290	{}	\N	2020-10-12 16:08:02.342207
70274	SIVB	2020-10-05	254.931	255.31	247.07	254.81	210037	{}	\N	2020-10-12 16:08:02.347124
70275	SIVB	2020-10-06	258.67	263.13	252.98	254.06	281699	{}	\N	2020-10-12 16:08:02.351571
70276	SIVB	2020-10-07	258.43	268.69	258.43	266.53	365468	{}	\N	2020-10-12 16:08:02.355742
70277	SIVB	2020-10-08	266.53	273.705	265.94	273.2	380659	{}	\N	2020-10-12 16:08:02.36023
70278	SIVB	2020-10-09	274.36	278.49	271.45	274.64	253271	{}	\N	2020-10-12 16:08:02.364216
70279	SIVB	2020-10-12	274.56	277.375	273.0903	274.54	51664	{}	\N	2020-10-12 16:08:02.368268
70280	SLCT	2020-09-28	7.25	7.49	7.25	7.26	20285	{}	\N	2020-10-12 16:08:04.05848
70281	SLCT	2020-09-29	7.3	7.3	7.05	7.16	5982	{}	\N	2020-10-12 16:08:04.069624
70282	SLCT	2020-09-30	7.2	7.2	7.11	7.19	11062	{}	\N	2020-10-12 16:08:04.077751
70283	SLCT	2020-10-01	7.15	7.45	7.15	7.39	143617	{}	\N	2020-10-12 16:08:04.084463
70284	SLCT	2020-10-02	7.23	7.46	7.23	7.45	17455	{}	\N	2020-10-12 16:08:04.090525
70285	SLCT	2020-10-05	7.36	7.75	7.2861	7.6	23308	{}	\N	2020-10-12 16:08:04.095877
70286	SLCT	2020-10-06	7.66	7.85	7.37	7.58	25167	{}	\N	2020-10-12 16:08:04.10075
70287	SLCT	2020-10-07	7.61	8	7.61	7.84	9922	{}	\N	2020-10-12 16:08:04.105492
70288	SLCT	2020-10-08	7.92	7.96	7.52	7.67	16475	{}	\N	2020-10-12 16:08:04.110122
70289	SLCT	2020-10-09	7.77	7.89	7.64	7.82	10104	{}	\N	2020-10-12 16:08:04.114815
70290	SLCT	2020-10-12	7.72	7.78	7.72	7.78	1153	{}	\N	2020-10-12 16:08:04.11963
70291	SLY	2020-09-28	60.25	61.282	60.25	61.04	48519	{}	\N	2020-10-12 16:08:06.163944
70292	SLY	2020-09-29	60.88	61.0897	60.1001	60.69	39955	{}	\N	2020-10-12 16:08:06.175068
70293	SLY	2020-09-30	60.85	61.77	60.5429	60.87	71742	{}	\N	2020-10-12 16:08:06.183054
70294	SLY	2020-10-01	61.25	61.71	60.78	61.7	120638	{}	\N	2020-10-12 16:08:06.190281
70295	SLY	2020-10-02	60.42	62.57	60.42	62.35	49941	{}	\N	2020-10-12 16:08:06.196368
70296	SLY	2020-10-05	63.02	63.79	63.02	63.79	58158	{}	\N	2020-10-12 16:08:06.201645
70297	SLY	2020-10-06	64.38	65.293	63.76	63.76	46705	{}	\N	2020-10-12 16:08:06.206627
70298	SLY	2020-10-07	64.5	64.9155	64.1939	64.69	38432	{}	\N	2020-10-12 16:08:06.211607
70299	SLY	2020-10-08	65.29	65.68	64.93	65.64	39496	{}	\N	2020-10-12 16:08:06.216546
70300	SLY	2020-10-09	66.09	66.39	65.63	65.92	284209	{}	\N	2020-10-12 16:08:06.221543
70301	SLY	2020-10-12	66.12	66.3799	65.8	66.1685	25294	{}	\N	2020-10-12 16:08:06.226912
70302	SMDV	2020-09-28	46.99	47.64	46.85	47.31	82000	{}	\N	2020-10-12 16:08:08.262171
70303	SMDV	2020-09-29	47.25	47.27	46.48	46.97	53077	{}	\N	2020-10-12 16:08:08.273615
70304	SMDV	2020-09-30	47.18	47.68	46.8394	47.1471	41853	{}	\N	2020-10-12 16:08:08.281776
70305	SMDV	2020-10-01	47.25	47.43	46.73	47.36	70173	{}	\N	2020-10-12 16:08:08.288575
70306	SMDV	2020-10-02	46.6	48.38	46.6	48.26	63327	{}	\N	2020-10-12 16:08:08.294679
70307	SMDV	2020-10-05	48.58	49.15	48.58	49.15	77728	{}	\N	2020-10-12 16:08:08.300575
70308	SMDV	2020-10-06	49.41	50.45	49.21	49.22	104885	{}	\N	2020-10-12 16:08:08.305818
70309	SMDV	2020-10-07	49.75	50.034	49.473	49.9	90588	{}	\N	2020-10-12 16:08:08.310639
70310	SMDV	2020-10-08	50.24	50.61	50.07	50.495	35591	{}	\N	2020-10-12 16:08:08.315543
70311	SMDV	2020-10-09	50.9	50.9	50.2	50.27	97111	{}	\N	2020-10-12 16:08:08.320553
70312	SMDV	2020-10-12	50.33	50.8797	50.17	50.74	21338	{}	\N	2020-10-12 16:08:08.325567
70313	SMDD	2020-09-28	13.33	13.33	12.78	12.87	39759	{}	\N	2020-10-12 16:08:10.470444
70314	SMDD	2020-09-29	12.91	13.31	12.82	13.08	48579	{}	\N	2020-10-12 16:08:10.482379
70315	SMDD	2020-09-30	12.94	13.03	12.465	12.85	20028	{}	\N	2020-10-12 16:08:10.490333
70316	SMDD	2020-10-01	12.595	12.73	12.35	12.36	57541	{}	\N	2020-10-12 16:08:10.497242
70317	SMDD	2020-10-02	12.98	12.98	11.85	11.98	126501	{}	\N	2020-10-12 16:08:10.503605
70318	SMDD	2020-10-05	11.66	11.66	11.16	11.16	41231	{}	\N	2020-10-12 16:08:10.509027
70319	SMDD	2020-10-06	10.94	11.39	10.545	11.33	67910	{}	\N	2020-10-12 16:08:10.513782
70320	SMDD	2020-10-07	10.94	10.97	10.65	10.71	49100	{}	\N	2020-10-12 16:08:10.518389
70321	SMDD	2020-10-08	10.46	10.63	10.39	10.39	41866	{}	\N	2020-10-12 16:08:10.52304
70322	SMDD	2020-10-09	10.19	10.429	10.08	10.31	26898	{}	\N	2020-10-12 16:08:10.527699
70323	SMDD	2020-10-12	10.14	10.3	10.1	10.125	8648	{}	\N	2020-10-12 16:08:10.532287
70324	SMHI	2020-09-28	2.095	2.2	2.04	2.11	145512	{}	\N	2020-10-12 16:08:11.597879
70325	SMHI	2020-09-29	2.11	2.11	2.02	2.07	152552	{}	\N	2020-10-12 16:08:11.610296
70326	SMHI	2020-09-30	2.05	2.13	2.03	2.03	169421	{}	\N	2020-10-12 16:08:11.61787
70327	SMHI	2020-10-01	2.04	2.09	1.9813	1.99	206540	{}	\N	2020-10-12 16:08:11.624454
70328	SMHI	2020-10-02	1.93	2.09	1.9199	2.05	121420	{}	\N	2020-10-12 16:08:11.631557
70329	SMHI	2020-10-05	2.05	2.255	2.05	2.18	97156	{}	\N	2020-10-12 16:08:11.636488
70330	SMHI	2020-10-06	2.19	2.45	2.13	2.21	403710	{}	\N	2020-10-12 16:08:11.641212
70331	SMHI	2020-10-07	2.2	2.25	2.14	2.19	219525	{}	\N	2020-10-12 16:08:11.646093
70332	SMHI	2020-10-08	2.19	2.27	2.15	2.24	172446	{}	\N	2020-10-12 16:08:11.653436
70333	SMHI	2020-10-09	2.24	2.48	2.24	2.31	296094	{}	\N	2020-10-12 16:08:11.658106
70334	SMHI	2020-10-12	2.34	2.34	2.09	2.17	145014	{}	\N	2020-10-12 16:08:11.662823
70335	SMMD	2020-09-28	44.79	44.8466	44.74	44.8466	3531	{}	\N	2020-10-12 16:08:13.601351
70336	SMMD	2020-09-29	45.08	45.08	44.29	44.63	10140	{}	\N	2020-10-12 16:08:13.613292
70337	SMMD	2020-09-30	44.985	45.2399	44.6	44.77	8907	{}	\N	2020-10-12 16:08:13.622126
70338	SMMD	2020-10-01	45.09	45.31	44.9659	45.141	6412	{}	\N	2020-10-12 16:08:13.629069
70339	SMMD	2020-10-02	44.72	45.7899	44.72	45.66	150904	{}	\N	2020-10-12 16:08:13.63477
70340	SMMD	2020-10-05	46.34	46.73	46.3305	46.73	6347	{}	\N	2020-10-12 16:08:13.64035
70341	SMMD	2020-10-06	47.05	47.624	46.5313	46.61	17718	{}	\N	2020-10-12 16:08:13.645392
70342	SMMD	2020-10-07	47.32	47.59	47.3038	47.59	9856	{}	\N	2020-10-12 16:08:13.650351
70343	SMMD	2020-10-08	48.08	48.09	47.74	48.07	19631	{}	\N	2020-10-12 16:08:13.65539
70344	SMMD	2020-10-09	48.5153	48.5153	48.165	48.313	4758	{}	\N	2020-10-12 16:08:13.660321
70345	SMMD	2020-10-12	48.75	48.75	48.32	48.625	2636	{}	\N	2020-10-12 16:08:13.664965
70346	SMPL	2020-09-28	22.11	22.37	21.93	22	484222	{}	\N	2020-10-12 16:08:15.483478
70347	SMPL	2020-09-29	22.06	22.06	21.385	21.75	434211	{}	\N	2020-10-12 16:08:15.494502
70348	SMPL	2020-09-30	21.65	22.43	21.65	22.05	529391	{}	\N	2020-10-12 16:08:15.503564
70349	SMPL	2020-10-01	22.17	22.34	21.57	22.16	661292	{}	\N	2020-10-12 16:08:15.50974
70350	SMPL	2020-10-02	21.69	22.8	21.64	22.65	416486	{}	\N	2020-10-12 16:08:15.515729
70351	SMPL	2020-10-05	22.78	22.855	22.17	22.54	464973	{}	\N	2020-10-12 16:08:15.520443
70352	SMPL	2020-10-06	22.72	22.776	21.73	21.89	504590	{}	\N	2020-10-12 16:08:15.524627
70353	SMPL	2020-10-07	22.03	22.38	21.74	21.9	537001	{}	\N	2020-10-12 16:08:15.528704
70354	SMPL	2020-10-08	22.18	22.36	21.78	21.92	564458	{}	\N	2020-10-12 16:08:15.532601
70355	SMPL	2020-10-09	21.99	22.19	21.65	21.79	679316	{}	\N	2020-10-12 16:08:15.536585
70356	SMPL	2020-10-12	21.8	22.32	21.695	22.2025	160007	{}	\N	2020-10-12 16:08:15.540558
70357	SIMO	2020-09-28	38.42	39.345	38.08	39.16	227286	{}	\N	2020-10-12 16:09:25.01955
70358	SIMO	2020-09-29	39.29	39.5	38.6	38.86	137525	{}	\N	2020-10-12 16:09:25.029589
70359	SIMO	2020-09-30	38.66	39.33	37.7	37.78	203883	{}	\N	2020-10-12 16:09:25.037203
70360	SIMO	2020-10-01	38.07	38.74	37.8	38.08	259022	{}	\N	2020-10-12 16:09:25.044423
70361	SIMO	2020-10-02	37.5	37.85	36.94	36.97	186611	{}	\N	2020-10-12 16:09:25.050337
70362	SIMO	2020-10-05	36.96	37.535	36.96	37.18	320843	{}	\N	2020-10-12 16:09:25.055893
70363	SIMO	2020-10-06	37.14	38.77	37.14	38.32	381615	{}	\N	2020-10-12 16:09:25.061204
70364	SIMO	2020-10-07	39.32	41.49	39.01	40.8	435987	{}	\N	2020-10-12 16:09:25.06615
70365	SIMO	2020-10-08	41.04	41.65	40.21	41.09	350306	{}	\N	2020-10-12 16:09:25.070919
70366	SIMO	2020-10-09	41.49	42.18	41	41.91	326287	{}	\N	2020-10-12 16:09:25.07584
70367	SIMO	2020-10-12	42.2	42.4	41.48	42.37	103187	{}	\N	2020-10-12 16:09:25.081077
70368	SJW	2020-09-28	61.15	61.95	60.9	61.23	63631	{}	\N	2020-10-12 16:09:29.355179
70369	SJW	2020-09-29	61.18	61.82	60.27	61.1	102846	{}	\N	2020-10-12 16:09:29.366393
70370	SJW	2020-09-30	61.08	61.48	60.34	60.86	203750	{}	\N	2020-10-12 16:09:29.375145
70371	SJW	2020-10-01	60.88	61.95	60.61	61.6	194802	{}	\N	2020-10-12 16:09:29.38146
70372	SJW	2020-10-02	60.71	62.42	60.62	61.56	177099	{}	\N	2020-10-12 16:09:29.387119
70373	SJW	2020-10-05	61.91	62	61.29	61.93	105471	{}	\N	2020-10-12 16:09:29.392351
70374	SJW	2020-10-06	62.4	62.4	59.72	60.11	410967	{}	\N	2020-10-12 16:09:29.397515
70375	SJW	2020-10-07	60.62	62.11	60.2	61.27	267306	{}	\N	2020-10-12 16:09:29.402036
70376	SJW	2020-10-08	61.81	61.81	60.8	61.27	173372	{}	\N	2020-10-12 16:09:29.407122
70377	SJW	2020-10-09	61.76	61.86	60.85	61.28	135245	{}	\N	2020-10-12 16:09:29.411933
70378	SJW	2020-10-12	61.15	62.77	61.15	62.77	81883	{}	\N	2020-10-12 16:09:29.416325
70379	SKYY	2020-09-28	78.22	78.44	77.45	78.14	312138	{}	\N	2020-10-12 16:09:31.989097
70380	SKYY	2020-09-29	78.12	78.72	78	78.32	245781	{}	\N	2020-10-12 16:09:31.999715
70381	SKYY	2020-09-30	78.17	79.356	78.025	78.49	278524	{}	\N	2020-10-12 16:09:32.007302
70382	SKYY	2020-10-01	79.08	79.6799	78.67	79.41	332979	{}	\N	2020-10-12 16:09:32.014009
70383	SKYY	2020-10-02	77.7	79.73	77.61	78.58	330223	{}	\N	2020-10-12 16:09:32.021183
70384	SKYY	2020-10-05	79.01	79.736	78.91	79.63	269828	{}	\N	2020-10-12 16:09:32.027785
70385	SKYY	2020-10-06	79.66	81.24	79.34	79.63	385874	{}	\N	2020-10-12 16:09:32.03595
70386	SKYY	2020-10-07	80.34	81.795	80.34	81.62	332397	{}	\N	2020-10-12 16:09:32.041879
70387	SKYY	2020-10-08	82.34	82.84	82.04	82.39	315909	{}	\N	2020-10-12 16:09:32.047325
70388	SKYY	2020-10-09	82.82	83.91	82.82	83.86	265757	{}	\N	2020-10-12 16:09:32.053283
70389	SKYY	2020-10-12	84.75	85.05	84.11	84.87	163053	{}	\N	2020-10-12 16:09:32.058513
70390	SLQD	2020-09-28	52.04	52.1	52.0361	52.06	181139	{}	\N	2020-10-12 16:09:34.949859
70391	SLQD	2020-09-29	52.06	52.12	52.06	52.11	277269	{}	\N	2020-10-12 16:09:34.957578
70392	SLQD	2020-09-30	52.1	52.12	52.07	52.12	299827	{}	\N	2020-10-12 16:09:34.964464
70393	SLQD	2020-10-01	52.02	52.04	52	52.02	155614	{}	\N	2020-10-12 16:09:34.970617
70394	SLQD	2020-10-02	52.02	52.04	52.015	52.03	136958	{}	\N	2020-10-12 16:09:34.976051
70395	SLQD	2020-10-05	52.05	52.08	52.03	52.04	327068	{}	\N	2020-10-12 16:09:34.981795
70396	SLQD	2020-10-06	52.03	52.069	52.02	52.05	257314	{}	\N	2020-10-12 16:09:34.988143
70397	SLQD	2020-10-07	52.01	52.07	52	52.05	196338	{}	\N	2020-10-12 16:09:34.993545
70398	SLQD	2020-10-08	52.05	52.08	52.03	52.04	202355	{}	\N	2020-10-12 16:09:34.99832
70399	SLQD	2020-10-09	52.08	52.1	52.06	52.09	281817	{}	\N	2020-10-12 16:09:35.004146
70400	SLQD	2020-10-12	52.08	52.11	52.065	52.1052	98076	{}	\N	2020-10-12 16:09:35.009443
70401	SMCP	2020-09-28	21.285	21.285	21.2576	21.2576	100	{}	\N	2020-10-12 16:09:36.767196
70402	SMCP	2020-09-30	21.27	21.27	21.2145	21.2145	282	{}	\N	2020-10-12 16:09:36.77863
70403	SMCP	2020-10-05	21.9249	21.99	21.9249	21.99	202	{}	\N	2020-10-12 16:09:36.787027
70404	SMCP	2020-10-07	22.02	22.2792	22.02	22.2792	1103	{}	\N	2020-10-12 16:09:36.793017
70405	SMCP	2020-10-08	22.4285	22.4632	22.4285	22.4632	200	{}	\N	2020-10-12 16:09:36.799064
70406	SMCP	2020-10-09	22.4602	22.4602	22.46	22.46	335	{}	\N	2020-10-12 16:09:36.803885
70407	SMCP	2020-10-12	22.605	22.605	22.605	22.605	200	{}	\N	2020-10-12 16:09:36.809327
70408	SMIT	2020-09-28	5.09	5.21	5.09	5.21	5871	{}	\N	2020-10-12 16:09:38.817617
70409	SMIT	2020-09-29	5.195	5.22	5.1798	5.1798	1186	{}	\N	2020-10-12 16:09:38.827789
70410	SMIT	2020-09-30	5.1791	5.19	5.15	5.1758	1960	{}	\N	2020-10-12 16:09:38.834983
70411	SMIT	2020-10-01	5.184	5.19	5.08	5.135	13417	{}	\N	2020-10-12 16:09:38.841075
70412	SMIT	2020-10-02	5.01	5.12	4.71	4.98	10919	{}	\N	2020-10-12 16:09:38.846269
70413	SMIT	2020-10-05	4.94	4.94	4.75	4.77	16360	{}	\N	2020-10-12 16:09:38.851591
70414	SMIT	2020-10-06	4.72	4.8541	4.72	4.8	8894	{}	\N	2020-10-12 16:09:38.856609
70415	SMIT	2020-10-07	4.92	4.92	4.78	4.8199	8357	{}	\N	2020-10-12 16:09:38.861107
70416	SMIT	2020-10-08	4.81	4.96	4.72	4.85	10606	{}	\N	2020-10-12 16:09:38.865962
70417	SMIT	2020-10-09	4.86	4.955	4.83	4.937	5153	{}	\N	2020-10-12 16:09:38.870721
70418	SMIT	2020-10-12	4.92	4.93	4.92	4.93	1951	{}	\N	2020-10-12 16:09:38.875282
70419	SMMU	2020-09-28	51.29	51.3	51.23	51.29	24315	{}	\N	2020-10-12 16:09:40.903112
70420	SMMU	2020-09-29	51.25	51.29	51.24	51.29	25412	{}	\N	2020-10-12 16:09:40.914354
70421	SMMU	2020-09-30	51.25	51.26	51.21	51.26	72594	{}	\N	2020-10-12 16:09:40.922395
70422	SMMU	2020-10-01	51.25	51.25	51.19	51.23	25200	{}	\N	2020-10-12 16:09:40.930239
70423	SMMU	2020-10-02	51.19	51.2293	51.16	51.16	121694	{}	\N	2020-10-12 16:09:40.936842
70424	SMMU	2020-10-05	51.16	51.22	51.16	51.16	37455	{}	\N	2020-10-12 16:09:40.942314
70425	SMMU	2020-10-06	51.17	51.2199	51.14	51.2	28163	{}	\N	2020-10-12 16:09:40.948036
70426	SMMU	2020-10-07	51.1999	51.2	51.131	51.19	51456	{}	\N	2020-10-12 16:09:40.95374
70427	SMMU	2020-10-08	51.175	51.2	51.14	51.2	6507	{}	\N	2020-10-12 16:09:40.959514
70428	SMMU	2020-10-09	51.2	51.2	51.1505	51.2	16821	{}	\N	2020-10-12 16:09:40.964311
70429	SMMU	2020-10-12	51.2	51.2099	51.1752	51.2	5937	{}	\N	2020-10-12 16:09:40.969493
70430	SNDX	2020-09-28	14.07	14.11	13.59	13.8	200609	{}	\N	2020-10-12 16:09:42.878728
70431	SNDX	2020-09-29	13.73	14.29	13.65	14.02	145563	{}	\N	2020-10-12 16:09:42.89062
70432	SNDX	2020-09-30	14.06	14.95	13.87	14.76	515131	{}	\N	2020-10-12 16:09:42.898266
70433	SNDX	2020-10-01	14.91	15.16	13.98	15.1	264569	{}	\N	2020-10-12 16:09:42.904456
70434	SNDX	2020-10-02	14.8	15.6	13.9233	15.44	341551	{}	\N	2020-10-12 16:09:42.910121
70435	SNDX	2020-10-05	15.53	17.165	15.53	17.04	493769	{}	\N	2020-10-12 16:09:42.91524
70436	SNDX	2020-10-06	17.06	19.06	16.105	16.74	536897	{}	\N	2020-10-12 16:09:42.919882
70437	SNDX	2020-10-07	16.93	17.8399	16.77	17	282043	{}	\N	2020-10-12 16:09:42.924859
70438	SNDX	2020-10-08	17.22	17.46	16.83	17.08	204317	{}	\N	2020-10-12 16:09:42.930021
70439	SNDX	2020-10-09	17	17.04	15.76	15.83	737902	{}	\N	2020-10-12 16:09:42.934573
70440	SNDX	2020-10-12	15.85	15.85	14.9	15.44	176564	{}	\N	2020-10-12 16:09:42.939356
70441	SIRI	2020-09-28	5.2	5.32	5.2	5.29	21770072	{}	\N	2020-10-12 16:09:44.303581
70442	SIRI	2020-09-29	5.3	5.315	5.19	5.25	17476307	{}	\N	2020-10-12 16:09:44.317063
70443	SIRI	2020-09-30	5.28	5.44	5.27	5.36	27806313	{}	\N	2020-10-12 16:09:44.326861
70444	SIRI	2020-10-01	5.4	5.66	5.38	5.65	43298879	{}	\N	2020-10-12 16:09:44.334031
70445	SIRI	2020-10-02	5.54	5.66	5.53	5.56	31052082	{}	\N	2020-10-12 16:09:44.339771
70446	SIRI	2020-10-05	5.59	5.655	5.52	5.61	21793603	{}	\N	2020-10-12 16:09:44.344986
70447	SIRI	2020-10-06	5.6	5.67	5.53	5.56	25638274	{}	\N	2020-10-12 16:09:44.350132
70448	SIRI	2020-10-07	5.77	5.89	5.75	5.815	32906657	{}	\N	2020-10-12 16:09:44.354884
70449	SIRI	2020-10-08	5.91	5.98	5.76	5.96	34007425	{}	\N	2020-10-12 16:09:44.359969
70450	SIRI	2020-10-09	5.97	6.03	5.84	5.84	20992708	{}	\N	2020-10-12 16:09:44.365103
70451	SIRI	2020-10-12	5.87	5.95	5.85	5.905	9186319	{}	\N	2020-10-12 16:09:44.371654
70452	SLAB	2020-09-28	98.15	99.05	97.78	98.37	205300	{}	\N	2020-10-12 16:09:47.1734
70453	SLAB	2020-09-29	98.34	99.01	97.59	98.07	128917	{}	\N	2020-10-12 16:09:47.185834
70454	SLAB	2020-09-30	97.75	99.1	97.265	97.85	172441	{}	\N	2020-10-12 16:09:47.193792
70455	SLAB	2020-10-01	98.6	100.24	97.755	99.07	211462	{}	\N	2020-10-12 16:09:47.200235
70456	SLAB	2020-10-02	96.39	98.09	96	96.13	211492	{}	\N	2020-10-12 16:09:47.205199
70457	SLAB	2020-10-05	97.07	98.59	96.1	98.51	184276	{}	\N	2020-10-12 16:09:47.210868
70458	SLAB	2020-10-06	98.74	101.27	96.94	97.2	428795	{}	\N	2020-10-12 16:09:47.216458
70459	SLAB	2020-10-07	96.91	99.34	96.91	98.99	393373	{}	\N	2020-10-12 16:09:47.221398
70460	SLAB	2020-10-08	100.52	101.9	99.98	101.39	154590	{}	\N	2020-10-12 16:09:47.228211
70461	SLAB	2020-10-09	102	106.72	102	106.65	311762	{}	\N	2020-10-12 16:09:47.235116
70462	SLAB	2020-10-12	107.44	107.79	106.1	106.215	31479	{}	\N	2020-10-12 16:09:47.242701
70463	SLGL	2020-09-28	7.3	7.4408	7.05	7.3062	6178	{}	\N	2020-10-12 16:09:48.656046
70464	SLGL	2020-09-29	7.24	7.3716	7.11	7.24	10659	{}	\N	2020-10-12 16:09:48.67783
70465	SLGL	2020-09-30	7.18	7.25	7.07	7.24	8807	{}	\N	2020-10-12 16:09:48.700154
70466	SLGL	2020-10-01	7.33	7.33	7.06	7.11	1123	{}	\N	2020-10-12 16:09:48.712376
70467	SLGL	2020-10-02	7.1	7.15	7.06	7.15	9032	{}	\N	2020-10-12 16:09:48.721291
70468	SLGL	2020-10-05	7.15	7.35	7.1	7.275	4479	{}	\N	2020-10-12 16:09:48.730607
70469	SLGL	2020-10-06	7.2767	7.4468	7.1	7.3	25508	{}	\N	2020-10-12 16:09:48.73958
70470	SLGL	2020-10-07	7.23	7.34	7.18	7.34	2795	{}	\N	2020-10-12 16:09:48.748402
70471	SLGL	2020-10-08	7.35	7.55	7.31	7.35	7286	{}	\N	2020-10-12 16:09:48.75712
70472	SLGL	2020-10-09	7.36	7.5449	7.2	7.52	12392	{}	\N	2020-10-12 16:09:48.764457
70473	SLGL	2020-10-12	7.44	7.44	7.44	7.44	1045	{}	\N	2020-10-12 16:09:48.771279
70474	SLP	2020-09-28	72.99	74.53	71.03	72.77	401839	{}	\N	2020-10-12 16:09:50.880081
70475	SLP	2020-09-29	72.5	76	71.46	74.8	292920	{}	\N	2020-10-12 16:09:50.892424
70476	SLP	2020-09-30	74.84	76.238	73.25	75.36	388132	{}	\N	2020-10-12 16:09:50.901702
70477	SLP	2020-10-01	76.13	76.44	72.51	73.1	359059	{}	\N	2020-10-12 16:09:50.90826
70478	SLP	2020-10-02	72.01	72.12	67.68	68	375344	{}	\N	2020-10-12 16:09:50.914894
70479	SLP	2020-10-05	68.61	69.28	67.41	68.77	342542	{}	\N	2020-10-12 16:09:50.920382
70480	SLP	2020-10-06	68.48	71.58	67.735	70.49	297369	{}	\N	2020-10-12 16:09:50.926209
70481	SLP	2020-10-07	70.69	72.71	70.69	71.54	162601	{}	\N	2020-10-12 16:09:50.932828
70482	SLP	2020-10-08	72.06	74.88	71.08	72.43	188005	{}	\N	2020-10-12 16:09:50.939254
70483	SLP	2020-10-09	72.57	74.965	72.57	74.59	172126	{}	\N	2020-10-12 16:09:50.945316
70484	SLP	2020-10-12	74.96	77.065	74.96	77.0567	83516	{}	\N	2020-10-12 16:09:50.950779
70485	SMB	2020-09-28	18.09	18.09	18.06	18.08	30836	{}	\N	2020-10-12 16:09:51.939237
70486	SMB	2020-09-29	18.085	18.11	18.06	18.09	65452	{}	\N	2020-10-12 16:09:51.949552
70487	SMB	2020-09-30	18.0792	18.0944	18.05	18.05	28626	{}	\N	2020-10-12 16:09:51.956217
70488	SMB	2020-10-01	18.07	18.07	18.02	18.04	24873	{}	\N	2020-10-12 16:09:51.962192
70489	SMB	2020-10-02	18	18.05	18	18.04	38182	{}	\N	2020-10-12 16:09:51.968542
70490	SMB	2020-10-05	18.01	18.05	18.01	18.01	19999	{}	\N	2020-10-12 16:09:51.973534
70491	SMB	2020-10-06	18.05	18.06	18.02	18.0251	17684	{}	\N	2020-10-12 16:09:51.978191
70492	SMB	2020-10-07	18.03	18.06	18	18.02	20424	{}	\N	2020-10-12 16:09:51.983838
70493	SMB	2020-10-08	18.04	18.07	18.01	18.05	35709	{}	\N	2020-10-12 16:09:51.988747
70494	SMB	2020-10-09	18.0331	18.0497	18.02	18.03	16396	{}	\N	2020-10-12 16:09:51.993883
70495	SMB	2020-10-12	17.99	18.04	17.99	18.035	39917	{}	\N	2020-10-12 16:09:51.998924
70496	SMEZ	2020-09-28	59.89	59.89	59.82	59.82	1118	{}	\N	2020-10-12 16:09:53.201173
70497	SMEZ	2020-09-29	59.8747	59.8747	59.8747	59.8747	46	{}	\N	2020-10-12 16:09:53.210971
70498	SMEZ	2020-09-30	60.0974	60.0974	60.0974	60.0974	0	{}	\N	2020-10-12 16:09:53.218189
70499	SMEZ	2020-10-02	60.3061	60.3061	60.3061	60.3061	25	{}	\N	2020-10-12 16:09:53.223861
70500	SMEZ	2020-10-05	61.17	61.53	61.17	61.53	913	{}	\N	2020-10-12 16:09:53.22913
70501	SMEZ	2020-10-06	62.0831	62.13	61.4529	61.4529	690	{}	\N	2020-10-12 16:09:53.233841
70502	SMEZ	2020-10-07	61.57	61.7649	61.57	61.7649	218	{}	\N	2020-10-12 16:09:53.238744
70503	SMEZ	2020-10-08	62.2534	62.2534	62.2534	62.2534	10	{}	\N	2020-10-12 16:09:53.243573
70504	SMEZ	2020-10-09	62.7051	62.7051	62.7051	62.7051	52	{}	\N	2020-10-12 16:09:53.248361
70505	SMMT	2020-09-28	3.42	3.48	3.3	3.47	24658	{}	\N	2020-10-12 16:09:54.200837
70506	SMMT	2020-09-29	3.5	3.57	3.25	3.495	91480	{}	\N	2020-10-12 16:09:54.210977
70507	SMMT	2020-09-30	3.27	3.5	3.25	3.44	32494	{}	\N	2020-10-12 16:09:54.218761
70508	SMMT	2020-10-01	3.5	3.5	3.2436	3.355	39354	{}	\N	2020-10-12 16:09:54.225089
70509	SMMT	2020-10-02	3.375	3.375	3.15	3.29	23061	{}	\N	2020-10-12 16:09:54.230695
70510	SMMT	2020-10-05	3.49	3.65	3.21	3.4	195253	{}	\N	2020-10-12 16:09:54.237231
70511	SMMT	2020-10-06	3.5	3.52	3.35	3.48	90860	{}	\N	2020-10-12 16:09:54.244315
70512	SMMT	2020-10-07	3.48	3.76	3.4308	3.53	132836	{}	\N	2020-10-12 16:09:54.25037
70513	SMMT	2020-10-08	3.61	3.7	3.52	3.525	80561	{}	\N	2020-10-12 16:09:54.25643
70514	SMMT	2020-10-09	3.6	3.65	3.5	3.6	108827	{}	\N	2020-10-12 16:09:54.26196
70515	SMMT	2020-10-12	3.67	3.735	3.62	3.63	16163	{}	\N	2020-10-12 16:09:54.269222
70516	SMTS	2020-09-28	1.43	1.48	1.43	1.48	36249	{}	\N	2020-10-12 16:09:55.232513
70517	SMTS	2020-09-29	1.477	1.4899	1.42	1.46	56625	{}	\N	2020-10-12 16:09:55.251802
70518	SMTS	2020-09-30	1.45	1.47	1.42	1.47	35759	{}	\N	2020-10-12 16:09:55.264115
70519	SMTS	2020-10-01	1.45	1.48	1.44	1.47	34620	{}	\N	2020-10-12 16:09:55.27541
70520	SMTS	2020-10-02	1.43	1.485	1.43	1.47	58211	{}	\N	2020-10-12 16:09:55.28627
70521	SMTS	2020-10-05	1.51	1.51	1.44	1.44	21042	{}	\N	2020-10-12 16:09:55.298649
70522	SMTS	2020-10-06	1.51	1.51	1.44	1.47	36367	{}	\N	2020-10-12 16:09:55.310114
70523	SMTS	2020-10-07	1.47	1.6599	1.47	1.51	211688	{}	\N	2020-10-12 16:09:55.327627
70524	SMTS	2020-10-08	1.5308	1.5308	1.47	1.485	15270	{}	\N	2020-10-12 16:09:55.337226
70525	SMTS	2020-10-09	1.53	1.53	1.5	1.5	23836	{}	\N	2020-10-12 16:09:55.349427
70526	SMTS	2020-10-12	1.49	1.51	1.4892	1.4997	6511	{}	\N	2020-10-12 16:09:55.358073
70527	SNDE	2020-09-28	2.4	2.4	2.19	2.3	179564	{}	\N	2020-10-12 16:09:57.0551
70528	SNDE	2020-09-29	2.48	2.5	2.2101	2.31	629636	{}	\N	2020-10-12 16:09:57.067172
70529	SNDE	2020-09-30	2.4	2.43	2.19	2.32	272616	{}	\N	2020-10-12 16:09:57.074334
70530	SNDE	2020-10-01	2.25	2.31	2.16	2.29	79970	{}	\N	2020-10-12 16:09:57.079978
70531	SNDE	2020-10-02	2.16	2.26	2.1	2.18	113262	{}	\N	2020-10-12 16:09:57.085046
70532	SNDE	2020-10-05	2.18	2.24	2.14	2.14	23503	{}	\N	2020-10-12 16:09:57.089663
70533	SNDE	2020-10-06	2.13	2.17	2.06	2.1	73969	{}	\N	2020-10-12 16:09:57.093773
70534	SNDE	2020-10-07	2.05	2.09	1.98	2.04	136624	{}	\N	2020-10-12 16:09:57.097709
70535	SNDE	2020-10-08	2.04	2.3	1.98	2.17	565257	{}	\N	2020-10-12 16:09:57.101511
70536	SNDE	2020-10-09	2.17	2.28	2.1	2.2	367105	{}	\N	2020-10-12 16:09:57.105594
70537	SNDE	2020-10-12	2.21	2.44	2.12	2.39	299737	{}	\N	2020-10-12 16:09:57.109574
70538	SOJB	2020-09-28	26.07	26.3027	26.07	26.25	59899	{}	\N	2020-10-12 16:09:58.132192
70539	SOJB	2020-09-29	25.99	25.99	25.85	25.88	125366	{}	\N	2020-10-12 16:09:58.141575
70540	SOJB	2020-09-30	25.81	25.9	25.65	25.65	222249	{}	\N	2020-10-12 16:09:58.148807
70541	SOJB	2020-10-01	25.71	25.99	25.7	25.98	137718	{}	\N	2020-10-12 16:09:58.155614
70542	SOJB	2020-10-02	25.89	25.96	25.81	25.95	88350	{}	\N	2020-10-12 16:09:58.162279
70543	SOJB	2020-10-05	26	26.05	25.96	25.99	78682	{}	\N	2020-10-12 16:09:58.168678
70544	SOJB	2020-10-06	26.01	26.21	26.01	26.21	91466	{}	\N	2020-10-12 16:09:58.174868
70545	SOJB	2020-10-07	26.126	26.34	26.126	26.34	107462	{}	\N	2020-10-12 16:09:58.179797
70546	SOJB	2020-10-08	26.25	26.33	26.22	26.23	63072	{}	\N	2020-10-12 16:09:58.184296
70547	SOJB	2020-10-09	26.29	26.29	26.16	26.28	107623	{}	\N	2020-10-12 16:09:58.189071
70548	SOJB	2020-10-12	26.19	26.31	26.16	26.3099	14766	{}	\N	2020-10-12 16:09:58.193621
70549	SKM	2020-09-28	22	22.28	22	22.21	186664	{}	\N	2020-10-12 16:09:59.996825
70550	SKM	2020-09-29	22.42	22.58	22.395	22.43	462170	{}	\N	2020-10-12 16:10:00.006394
70551	SKM	2020-09-30	22.44	22.55	22.3	22.42	784690	{}	\N	2020-10-12 16:10:00.014885
70552	SKM	2020-10-01	22.41	22.545	22.405	22.43	436411	{}	\N	2020-10-12 16:10:00.021376
70553	SKM	2020-10-02	22.29	22.47	22.29	22.43	347050	{}	\N	2020-10-12 16:10:00.027794
70554	SKM	2020-10-05	22.88	23.12	22.88	23.1	268706	{}	\N	2020-10-12 16:10:00.034812
70555	SKM	2020-10-06	23.06	23.12	22.68	22.73	426619	{}	\N	2020-10-12 16:10:00.040764
70556	SKM	2020-10-07	23.01	23.055	22.89	22.9	497029	{}	\N	2020-10-12 16:10:00.048559
70557	SKM	2020-10-08	23.04	23.04	22.89	22.93	655339	{}	\N	2020-10-12 16:10:00.055173
70558	SKM	2020-10-09	23.04	23.205	22.97	23.07	563815	{}	\N	2020-10-12 16:10:00.062309
70559	SKM	2020-10-12	23.08	23.33	23.06	23.26	254690	{}	\N	2020-10-12 16:10:00.068779
70560	SLB	2020-09-28	16.36	16.8486	16.26	16.45	11878134	{}	\N	2020-10-12 16:10:03.264639
70561	SLB	2020-09-29	16.35	16.4	15.47	15.6	31068978	{}	\N	2020-10-12 16:10:03.275298
70562	SLB	2020-09-30	15.81	16.02	15.4238	15.56	29040622	{}	\N	2020-10-12 16:10:03.282495
70563	SLB	2020-10-01	15.36	15.527	14.88	15.01	34569456	{}	\N	2020-10-12 16:10:03.289005
70564	SLB	2020-10-02	14.57	15.725	14.52	15.57	27780316	{}	\N	2020-10-12 16:10:03.2945
70565	SLB	2020-10-05	15.77	16.13	15.6301	16.07	24243710	{}	\N	2020-10-12 16:10:03.299573
70566	SLB	2020-10-06	16.44	16.56	15.64	15.67	25829248	{}	\N	2020-10-12 16:10:03.305017
70567	SLB	2020-10-07	15.72	16.03	15.58	15.95	20166597	{}	\N	2020-10-12 16:10:03.310223
70568	SLB	2020-10-08	16.12	16.44	15.94	16.43	21405180	{}	\N	2020-10-12 16:10:03.315245
70569	SLB	2020-10-09	16.58	16.6	16.05	16.23	22932803	{}	\N	2020-10-12 16:10:03.320509
70570	SLB	2020-10-12	16.03	16.2	15.76	15.8937	13145542	{}	\N	2020-10-12 16:10:03.325701
70571	SLGN	2020-09-28	36.87	36.97	36.145	36.79	341557	{}	\N	2020-10-12 16:10:04.25549
70572	SLGN	2020-09-29	36.86	36.9	36.16	36.71	226821	{}	\N	2020-10-12 16:10:04.267947
70573	SLGN	2020-09-30	36.68	37.14	36.2525	36.77	353805	{}	\N	2020-10-12 16:10:04.276884
70574	SLGN	2020-10-01	36.92	37.17	36.55	36.72	289792	{}	\N	2020-10-12 16:10:04.284989
70575	SLGN	2020-10-02	36.21	37.25	36.21	37.01	247478	{}	\N	2020-10-12 16:10:04.290799
70576	SLGN	2020-10-05	37.06	37.63	37.06	37.58	434301	{}	\N	2020-10-12 16:10:04.295769
70577	SLGN	2020-10-06	37.77	37.99	37.42	37.5	379572	{}	\N	2020-10-12 16:10:04.301348
70578	SLGN	2020-10-07	37.61	38.14	37.57	38.03	484392	{}	\N	2020-10-12 16:10:04.306621
70579	SLGN	2020-10-08	38.19	38.65	38.06	38.63	274045	{}	\N	2020-10-12 16:10:04.311894
70580	SLGN	2020-10-09	38.59	39.14	38.505	38.7	417465	{}	\N	2020-10-12 16:10:04.317033
70581	SLGN	2020-10-12	38.82	40.44	38.75	40.39	400464	{}	\N	2020-10-12 16:10:04.3222
70582	SM	2020-09-28	1.77	1.81	1.69	1.73	5651266	{}	\N	2020-10-12 16:10:05.88012
70583	SM	2020-09-29	1.7	1.73	1.56	1.62	11598513	{}	\N	2020-10-12 16:10:05.893543
70584	SM	2020-09-30	1.62	1.65	1.56	1.59	11816762	{}	\N	2020-10-12 16:10:05.902796
70585	SM	2020-10-01	1.55	1.58	1.48	1.51	10126735	{}	\N	2020-10-12 16:10:05.909609
70586	SM	2020-10-02	1.42	1.57	1.405	1.53	7239010	{}	\N	2020-10-12 16:10:05.915965
70587	SM	2020-10-05	1.6	1.63	1.51	1.56	7579970	{}	\N	2020-10-12 16:10:05.923247
70588	SM	2020-10-06	1.57	1.6401	1.48	1.49	10886116	{}	\N	2020-10-12 16:10:05.928189
70589	SM	2020-10-07	1.51	1.56	1.46	1.53	7532011	{}	\N	2020-10-12 16:10:05.933141
70590	SM	2020-10-08	1.56	1.87	1.54	1.85	18726961	{}	\N	2020-10-12 16:10:05.938262
70591	SM	2020-10-09	1.85	1.9086	1.69	1.71	12517702	{}	\N	2020-10-12 16:10:05.943279
70592	SM	2020-10-12	1.72	1.75	1.64	1.7	3583730	{}	\N	2020-10-12 16:10:05.948655
70593	SMG	2020-09-28	151.75	152.84	150.71	152.3	259088	{}	\N	2020-10-12 16:10:07.006491
70594	SMG	2020-09-29	153	153.9	150.56	152.31	753444	{}	\N	2020-10-12 16:10:07.017872
70595	SMG	2020-09-30	152.35	155.28	150.54	152.91	942184	{}	\N	2020-10-12 16:10:07.025782
70596	SMG	2020-10-01	153.24	154.055	149.19	150.48	484235	{}	\N	2020-10-12 16:10:07.032154
70597	SMG	2020-10-02	148.74	152.24	147.39	150.75	452285	{}	\N	2020-10-12 16:10:07.038124
70598	SMG	2020-10-05	151.37	154.53	151.04	153.07	485075	{}	\N	2020-10-12 16:10:07.043303
70599	SMG	2020-10-06	153	154.08	150.995	151.81	694230	{}	\N	2020-10-12 16:10:07.048787
70600	SMG	2020-10-07	153.49	156.08	153.2801	155.18	568771	{}	\N	2020-10-12 16:10:07.054774
70601	SMG	2020-10-08	156.06	157.76	154.7576	156.76	425824	{}	\N	2020-10-12 16:10:07.060457
70602	SMG	2020-10-09	157.63	165.5	157.63	164.05	835775	{}	\N	2020-10-12 16:10:07.065741
70603	SMG	2020-10-12	164.43	165.92	161.82	165.68	144963	{}	\N	2020-10-12 16:10:07.071283
70604	SMLV	2020-09-28	75.15	76.621	75.15	76.24	50031	{}	\N	2020-10-12 16:10:08.014491
70605	SMLV	2020-09-29	76.05	76.15	74.815	75.61	16676	{}	\N	2020-10-12 16:10:08.025816
70606	SMLV	2020-09-30	75.34	76.595	75.34	75.7	20056	{}	\N	2020-10-12 16:10:08.033176
70607	SMLV	2020-10-01	76.29	76.59	75.3124	76.59	14152	{}	\N	2020-10-12 16:10:08.040113
70608	SMLV	2020-10-02	75.17	77.7725	75.17	77.73	17339	{}	\N	2020-10-12 16:10:08.045339
70609	SMLV	2020-10-05	78.08	78.77	78.08	78.695	4035	{}	\N	2020-10-12 16:10:08.049859
70610	SMLV	2020-10-06	78.93	80.53	78.6541	78.6541	12682	{}	\N	2020-10-12 16:10:08.054369
70611	SMLV	2020-10-07	79.13	79.6899	78.9	79.61	8226	{}	\N	2020-10-12 16:10:08.059062
70612	SMLV	2020-10-08	80	80.98	79.955	80.87	11060	{}	\N	2020-10-12 16:10:08.063435
70613	SMLV	2020-10-09	81.32	81.32	80.38	80.6938	16414	{}	\N	2020-10-12 16:10:08.067624
70614	SMLV	2020-10-12	81	81.2003	80.8389	81.13	9021	{}	\N	2020-10-12 16:10:08.071661
70615	SMN	2020-09-28	14.71	14.71	14.13	14.4	5089	{}	\N	2020-10-12 16:10:09.425348
70616	SMN	2020-09-29	14.33	14.609	14.33	14.609	7013	{}	\N	2020-10-12 16:10:09.434803
70617	SMN	2020-09-30	14.56	14.56	14.081	14.28	4602	{}	\N	2020-10-12 16:10:09.441139
70618	SMN	2020-10-01	14.24	14.77	14.24	14.67	3581	{}	\N	2020-10-12 16:10:09.446431
70619	SMN	2020-10-02	14.441	14.47	14.441	14.4533	819	{}	\N	2020-10-12 16:10:09.452017
70620	SMN	2020-10-05	14.11	14.11	13.69	13.9	4886	{}	\N	2020-10-12 16:10:09.456955
70621	SMN	2020-10-06	13.615	14.1895	13.54	14.1895	5051	{}	\N	2020-10-12 16:10:09.461859
70622	SMN	2020-10-07	13.77	13.77	13.44	13.4487	2161	{}	\N	2020-10-12 16:10:09.466692
70623	SMN	2020-10-08	13.13	13.31	13.13	13.3099	9590	{}	\N	2020-10-12 16:10:09.471383
70624	SMN	2020-10-09	13.02	13.19	12.95	13	3466	{}	\N	2020-10-12 16:10:09.476157
70625	SMN	2020-10-12	13.03	13.03	13	13	1608	{}	\N	2020-10-12 16:10:09.481418
70626	SNA	2020-09-28	144.67	147.57	143.27	146.71	347277	{}	\N	2020-10-12 16:10:11.149085
70627	SNA	2020-09-29	147.21	147.7	144.41	145.4	472968	{}	\N	2020-10-12 16:10:11.161817
70628	SNA	2020-09-30	146.45	148.985	145.7	147.13	1172704	{}	\N	2020-10-12 16:10:11.169853
70629	SNA	2020-10-01	147.82	150	144.11	145.12	1031960	{}	\N	2020-10-12 16:10:11.176517
70630	SNA	2020-10-02	142.15	148.75	142.15	148.12	1130916	{}	\N	2020-10-12 16:10:11.182407
70631	SNA	2020-10-05	149.89	152.05	149.07	151.63	604917	{}	\N	2020-10-12 16:10:11.187729
70632	SNA	2020-10-06	152.58	161.47	151.97	156.05	1683688	{}	\N	2020-10-12 16:10:11.193382
70633	SNA	2020-10-07	157.99	161.31	157.56	158.4	793176	{}	\N	2020-10-12 16:10:11.198576
70634	SNA	2020-10-08	158.3	159.2	156.84	158.39	556356	{}	\N	2020-10-12 16:10:11.203329
70635	SNA	2020-10-09	159.15	160.23	156.78	157.21	531225	{}	\N	2020-10-12 16:10:11.208289
70636	SNA	2020-10-12	157.99	159.73	157.38	158.7201	233739	{}	\N	2020-10-12 16:10:11.213356
70637	SMTC	2020-09-28	52.21	53.25	51.83	53.19	303982	{}	\N	2020-10-12 16:10:13.805878
70638	SMTC	2020-09-29	53.18	53.95	51.5801	53.46	247019	{}	\N	2020-10-12 16:10:13.820151
70639	SMTC	2020-09-30	53.41	53.74	52.825	52.96	317004	{}	\N	2020-10-12 16:10:13.82836
70640	SMTC	2020-10-01	53.78	54.44	53.28	54.2	521987	{}	\N	2020-10-12 16:10:13.834719
70641	SMTC	2020-10-02	52.7	53.42	52.1	52.17	250930	{}	\N	2020-10-12 16:10:13.840491
70642	SMTC	2020-10-05	52.62	54.59	52.03	54.54	364125	{}	\N	2020-10-12 16:10:13.846239
70643	SMTC	2020-10-06	54.6	56.39	54.58	55.14	460531	{}	\N	2020-10-12 16:10:13.851438
70644	SMTC	2020-10-07	55.59	56.19	55.54	55.82	320129	{}	\N	2020-10-12 16:10:13.856373
70645	SMTC	2020-10-08	56.55	56.95	56.14	56.6	358144	{}	\N	2020-10-12 16:10:13.861204
70646	SMTC	2020-10-09	57.48	58.22	57.2	57.94	283214	{}	\N	2020-10-12 16:10:13.865973
70647	SMTC	2020-10-12	58.43	58.45	57.89	57.94	106328	{}	\N	2020-10-12 16:10:13.870743
70648	SNDR	2020-09-28	25.28	25.54	24.955	25.45	523613	{}	\N	2020-10-12 16:10:15.532481
70649	SNDR	2020-09-29	25.43	25.65	25.12	25.12	506977	{}	\N	2020-10-12 16:10:15.548473
70650	SNDR	2020-09-30	25.13	25.24	24.58	24.73	905983	{}	\N	2020-10-12 16:10:15.554846
70651	SNDR	2020-10-01	24.8	25.01	24.44	24.9	990499	{}	\N	2020-10-12 16:10:15.561108
70652	SNDR	2020-10-02	24.77	25.5	24.74	25.11	770789	{}	\N	2020-10-12 16:10:15.566622
70653	SNDR	2020-10-05	25.32	25.5	24.79	25.02	1331737	{}	\N	2020-10-12 16:10:15.571288
70654	SNDR	2020-10-06	24.86	25.07	24.66	24.84	1719816	{}	\N	2020-10-12 16:10:15.575843
70655	SNDR	2020-10-07	25.25	25.9	25.02	25.56	1048375	{}	\N	2020-10-12 16:10:15.580374
70656	SNDR	2020-10-08	25.74	25.87	25.4994	25.6	758680	{}	\N	2020-10-12 16:10:15.584773
70657	SNDR	2020-10-09	24.95	25.15	24.51	24.91	1067258	{}	\N	2020-10-12 16:10:15.589209
70658	SNDR	2020-10-12	25.11	25.37	24.83	24.86	327880	{}	\N	2020-10-12 16:10:15.594031
70659	SNE	2020-09-28	78.1	78.1	76.96	77.28	811837	{}	\N	2020-10-12 16:10:16.638126
70660	SNE	2020-09-29	77.43	77.82	77.075	77.48	889527	{}	\N	2020-10-12 16:10:16.650254
70661	SNE	2020-09-30	76.75	77.44	76.275	76.75	1380956	{}	\N	2020-10-12 16:10:16.657223
70662	SNE	2020-10-01	76.91	76.9161	76.16	76.45	1382045	{}	\N	2020-10-12 16:10:16.663431
70663	SNE	2020-10-02	74	74.95	73.6201	74	2712425	{}	\N	2020-10-12 16:10:16.669501
70664	SNE	2020-10-05	74.85	75.22	74.55	75.07	1707953	{}	\N	2020-10-12 16:10:16.674353
70665	SNE	2020-10-06	74.34	74.41	73.43	73.54	1661603	{}	\N	2020-10-12 16:10:16.679597
70666	SNE	2020-10-07	73.98	73.98	73.1	73.37	2479682	{}	\N	2020-10-12 16:10:16.684668
70667	SNE	2020-10-08	73.75	74.3	73.53	74.13	1486254	{}	\N	2020-10-12 16:10:16.689723
70668	SNE	2020-10-09	74.35	74.84	73.93	74.55	1511495	{}	\N	2020-10-12 16:10:16.694499
70669	SNE	2020-10-12	74.07	74.88	74.06	74.66	719079	{}	\N	2020-10-12 16:10:16.700036
70670	SNN	2020-09-28	38.17	38.42	38.02	38.16	182356	{}	\N	2020-10-12 16:10:18.091133
70671	SNN	2020-09-29	38.53	38.78	38.15	38.34	836623	{}	\N	2020-10-12 16:10:18.102632
70672	SNN	2020-09-30	39.14	39.65	38.86	39.1	769162	{}	\N	2020-10-12 16:10:18.11079
70673	SNN	2020-10-01	40.24	40.305	39.35	39.61	1146633	{}	\N	2020-10-12 16:10:18.11675
70674	SNN	2020-10-02	39.25	39.72	39.09	39.63	578741	{}	\N	2020-10-12 16:10:18.122706
70675	SNN	2020-10-05	39.99	40.12	39.62	39.85	646307	{}	\N	2020-10-12 16:10:18.128677
70676	SNN	2020-10-06	39.34	39.72	39.11	39.2	516512	{}	\N	2020-10-12 16:10:18.133938
70677	SNN	2020-10-07	38.85	39.59	38.85	39.56	566067	{}	\N	2020-10-12 16:10:18.138563
70678	SNN	2020-10-08	39.42	39.76	39.38	39.62	421222	{}	\N	2020-10-12 16:10:18.143438
70679	SNN	2020-10-09	39.64	40.04	39.605	39.83	718311	{}	\N	2020-10-12 16:10:18.148381
70680	SNN	2020-10-12	40.05	40.38	40.01	40.38	328721	{}	\N	2020-10-12 16:10:18.153143
70681	SLX	2020-09-28	31.12	31.4847	31.12	31.14	23034	{}	\N	2020-10-12 16:10:20.0169
70682	SLX	2020-09-29	31.2	31.22	30.67	30.71	28188	{}	\N	2020-10-12 16:10:20.031335
70683	SLX	2020-09-30	30.85	31.3145	30.731	30.9	38450	{}	\N	2020-10-12 16:10:20.04005
70684	SLX	2020-10-01	31.0138	31.2678	30.8814	31.15	25871	{}	\N	2020-10-12 16:10:20.04676
70685	SLX	2020-10-02	30.71	31.69	30.71	31.56	15553	{}	\N	2020-10-12 16:10:20.052961
70686	SLX	2020-10-05	31.75	32.62	31.75	32.56	168383	{}	\N	2020-10-12 16:10:20.057791
70687	SLX	2020-10-06	32.64	32.83	31.955	32.04	15969	{}	\N	2020-10-12 16:10:20.063349
70688	SLX	2020-10-07	32.5	32.97	32.4806	32.9	23045	{}	\N	2020-10-12 16:10:20.068653
70689	SLX	2020-10-08	33	33.28	32.9	33.26	13975	{}	\N	2020-10-12 16:10:20.073567
70690	SLX	2020-10-09	33.46	33.61	32.71	32.97	16503	{}	\N	2020-10-12 16:10:20.0784
70691	SLX	2020-10-12	32.97	32.97	32.7	32.8596	9914	{}	\N	2020-10-12 16:10:20.083121
70692	SMBC	2020-09-28	23.75	24.2	23.75	24.2	3841	{}	\N	2020-10-12 16:10:22.521327
70693	SMBC	2020-09-29	24.25	24.5	23.64	23.96	4093	{}	\N	2020-10-12 16:10:22.528499
70694	SMBC	2020-09-30	23.59	23.74	23.53	23.58	5001	{}	\N	2020-10-12 16:10:22.533925
70695	SMBC	2020-10-01	23.21	23.55	23.21	23.25	3120	{}	\N	2020-10-12 16:10:22.539708
70696	SMBC	2020-10-02	22.25	23.65	22.25	23.36	7113	{}	\N	2020-10-12 16:10:22.54542
70697	SMBC	2020-10-05	23.15	23.15	21.5	22.12	53131	{}	\N	2020-10-12 16:10:22.549755
70698	SMBC	2020-10-06	22.3	22.77	21.8	21.96	11109	{}	\N	2020-10-12 16:10:22.554522
70699	SMBC	2020-10-07	22.1442	23.0805	22.13	22.75	13292	{}	\N	2020-10-12 16:10:22.558974
70700	SMBC	2020-10-08	22.99	23.105	22.285	22.86	8540	{}	\N	2020-10-12 16:10:22.563794
70701	SMBC	2020-10-09	23.24	23.24	22.85	22.85	4019	{}	\N	2020-10-12 16:10:22.568462
70702	SMBC	2020-10-12	22.76	23	22.76	23	675	{}	\N	2020-10-12 16:10:22.573279
70703	SMH	2020-09-28	171.8572	173.68	170.46	173.54	1788762	{}	\N	2020-10-12 16:10:23.730579
70704	SMH	2020-09-29	173.13	175.57	173.13	174.46	3085612	{}	\N	2020-10-12 16:10:23.738306
70705	SMH	2020-09-30	174.13	176.19	173.4462	174.2	2705508	{}	\N	2020-10-12 16:10:23.744059
70706	SMH	2020-10-01	177.53	179.08	176.6473	178.5	2168633	{}	\N	2020-10-12 16:10:23.749744
70707	SMH	2020-10-02	174.53	176.95	173.08	173.15	2921077	{}	\N	2020-10-12 16:10:23.754895
70708	SMH	2020-10-05	174.79	179.05	174.65	179.05	1835797	{}	\N	2020-10-12 16:10:23.759699
70709	SMH	2020-10-06	178.62	181.61	176.78	177.48	4973648	{}	\N	2020-10-12 16:10:23.764513
70710	SMH	2020-10-07	180.16	181.77	180.12	181.28	1621759	{}	\N	2020-10-12 16:10:23.76933
70711	SMH	2020-10-08	183.24	183.74	182.36	183.5	1857143	{}	\N	2020-10-12 16:10:23.773995
70712	SMH	2020-10-09	186.27	187.1	185.4	186.65	2600826	{}	\N	2020-10-12 16:10:23.77883
70713	SMH	2020-10-12	189.24	189.687	188.11	189.11	869616	{}	\N	2020-10-12 16:10:23.78356
70714	SMM	2020-09-28	3.47	3.56	3.45	3.56	63305	{}	\N	2020-10-12 16:10:25.533405
70715	SMM	2020-09-29	3.54	3.56	3.51	3.56	159061	{}	\N	2020-10-12 16:10:25.540049
70716	SMM	2020-09-30	3.56	3.6	3.46	3.5	95016	{}	\N	2020-10-12 16:10:25.546178
70717	SMM	2020-10-01	3.53	3.53	3.42	3.48	122700	{}	\N	2020-10-12 16:10:25.551838
70718	SMM	2020-10-02	3.43	3.515	3.39	3.51	49550	{}	\N	2020-10-12 16:10:25.55765
70719	SMM	2020-10-05	3.61	3.78	3.5715	3.73	658348	{}	\N	2020-10-12 16:10:25.56286
70720	SMM	2020-10-06	3.77	3.84	3.63	3.67	329617	{}	\N	2020-10-12 16:10:25.568302
70721	SMM	2020-10-07	3.66	3.75	3.65	3.75	99826	{}	\N	2020-10-12 16:10:25.574059
70722	SMM	2020-10-08	3.8	3.95	3.8	3.95	194431	{}	\N	2020-10-12 16:10:25.579596
70723	SMM	2020-10-09	3.96	3.99	3.87	3.91	278962	{}	\N	2020-10-12 16:10:25.587079
70724	SMM	2020-10-12	3.96	3.98	3.88	3.8872	88788	{}	\N	2020-10-12 16:10:25.594256
70725	SMP	2020-09-28	43.19	44.53	43.075	43.97	93985	{}	\N	2020-10-12 16:10:27.486497
70726	SMP	2020-09-29	43.92	44.3	43.4	43.94	186702	{}	\N	2020-10-12 16:10:27.497374
70727	SMP	2020-09-30	44.19	45.31	44.14	44.65	256014	{}	\N	2020-10-12 16:10:27.506014
70728	SMP	2020-10-01	45.1	46.15	44.78	45.95	250108	{}	\N	2020-10-12 16:10:27.512334
70729	SMP	2020-10-02	45.33	47.09	45.06	46.77	217896	{}	\N	2020-10-12 16:10:27.518352
70730	SMP	2020-10-05	47.32	48.48	47.13	48.28	220696	{}	\N	2020-10-12 16:10:27.523327
70731	SMP	2020-10-06	48.72	49.95	48.36	48.78	234955	{}	\N	2020-10-12 16:10:27.528404
70732	SMP	2020-10-07	49.15	49.33	47.84	47.88	231331	{}	\N	2020-10-12 16:10:27.53305
70733	SMP	2020-10-08	48.44	48.79	48.03	48.33	200793	{}	\N	2020-10-12 16:10:27.538883
70734	SMP	2020-10-09	48.9	49.295	47.85	48.39	148152	{}	\N	2020-10-12 16:10:27.544726
70735	SMP	2020-10-12	48.39	49.14	48.33	48.67	26522	{}	\N	2020-10-12 16:10:27.549988
70736	SNCR	2020-09-28	3.2	3.2	3.07	3.09	301009	{}	\N	2020-10-12 16:10:29.249703
70737	SNCR	2020-09-29	3.07	3.12	2.99	3.1	306658	{}	\N	2020-10-12 16:10:29.26281
70738	SNCR	2020-09-30	3.1	3.115	3	3.01	400176	{}	\N	2020-10-12 16:10:29.270325
70739	SNCR	2020-10-01	3.02	3.1	3	3.06	279727	{}	\N	2020-10-12 16:10:29.276449
70740	SNCR	2020-10-02	2.98	3.27	2.98	3.19	482267	{}	\N	2020-10-12 16:10:29.282593
70741	SNCR	2020-10-05	3.26	3.3034	3.03	3.08	500343	{}	\N	2020-10-12 16:10:29.287783
70742	SNCR	2020-10-06	3.08	3.13	2.99	3.03	485393	{}	\N	2020-10-12 16:10:29.292806
70743	SNCR	2020-10-07	3.05	3.15	2.93	3.1	589583	{}	\N	2020-10-12 16:10:29.297895
70744	SNCR	2020-10-08	3.12	3.2	3.12	3.16	278453	{}	\N	2020-10-12 16:10:29.302942
70745	SNCR	2020-10-09	3.21	3.25	3.15	3.18	261723	{}	\N	2020-10-12 16:10:29.308641
70746	SNCR	2020-10-12	3.17	3.19	3.06	3.12	170174	{}	\N	2020-10-12 16:10:29.314959
70747	SND	2020-09-28	1.23	1.29	1.23	1.29	105980	{}	\N	2020-10-12 16:10:30.598403
70748	SND	2020-09-29	1.29	1.32	1.25	1.31	125659	{}	\N	2020-10-12 16:10:30.608512
70749	SND	2020-09-30	1.3	1.34	1.28	1.31	117217	{}	\N	2020-10-12 16:10:30.614912
70750	SND	2020-10-01	1.32	1.32	1.24	1.29	89322	{}	\N	2020-10-12 16:10:30.621381
70751	SND	2020-10-02	1.24	1.3	1.23	1.26	53774	{}	\N	2020-10-12 16:10:30.627387
70752	SND	2020-10-05	1.24	1.35	1.24	1.34	174864	{}	\N	2020-10-12 16:10:30.631943
70753	SND	2020-10-06	1.35	1.43	1.35	1.38	252748	{}	\N	2020-10-12 16:10:30.636526
70754	SND	2020-10-07	1.3638	1.43	1.3638	1.43	141340	{}	\N	2020-10-12 16:10:30.64137
70755	SND	2020-10-08	1.47	1.53	1.42	1.53	198857	{}	\N	2020-10-12 16:10:30.646056
70756	SND	2020-10-09	1.55	1.5599	1.4316	1.47	129966	{}	\N	2020-10-12 16:10:30.651435
70757	SND	2020-10-12	1.48	1.48	1.43	1.46	97635	{}	\N	2020-10-12 16:10:30.655773
70758	SNLN	2020-09-28	15.65	15.7902	15.65	15.67	2506	{}	\N	2020-10-12 16:10:33.039615
70759	SNLN	2020-09-29	15.65	15.73	15.65	15.73	2583	{}	\N	2020-10-12 16:10:33.05198
70760	SNLN	2020-09-30	15.7495	15.775	15.63	15.63	4788	{}	\N	2020-10-12 16:10:33.059739
70761	SNLN	2020-10-01	15.66	15.7328	15.66	15.66	3180	{}	\N	2020-10-12 16:10:33.065844
70762	SNLN	2020-10-02	15.7	15.75	15.61	15.61	3366	{}	\N	2020-10-12 16:10:33.071928
70763	SNLN	2020-10-05	15.71	15.75	15.67	15.74	13968	{}	\N	2020-10-12 16:10:33.076983
70764	SNLN	2020-10-06	15.68	15.73	15.67	15.73	5312	{}	\N	2020-10-12 16:10:33.082393
70765	SNLN	2020-10-07	15.76	15.84	15.71	15.79	8480	{}	\N	2020-10-12 16:10:33.088528
70766	SNLN	2020-10-08	15.84	15.84	15.73	15.73	10098	{}	\N	2020-10-12 16:10:33.093545
70767	SNLN	2020-10-09	15.77	15.83	15.73	15.83	8383	{}	\N	2020-10-12 16:10:33.098672
70768	SNLN	2020-10-12	15.88	15.88	15.88	15.88	183	{}	\N	2020-10-12 16:10:33.10325
70769	SLVO	2020-09-28	6.15	6.33	6.15	6.32	67458	{}	\N	2020-10-12 16:10:35.327391
70770	SLVO	2020-09-29	6.37	6.5	6.36	6.43	146153	{}	\N	2020-10-12 16:10:35.341817
70771	SLVO	2020-09-30	6.42	6.42	6.18	6.2418	88361	{}	\N	2020-10-12 16:10:35.349939
70772	SLVO	2020-10-01	6.36	6.42	6.2401	6.36	47691	{}	\N	2020-10-12 16:10:35.355833
70773	SLVO	2020-10-02	6.3	6.43	6.3	6.34	47399	{}	\N	2020-10-12 16:10:35.361467
70774	SLVO	2020-10-05	6.41	6.53	6.396	6.53	148278	{}	\N	2020-10-12 16:10:35.367007
70775	SLVO	2020-10-06	6.52	6.52	6.2	6.22	253137	{}	\N	2020-10-12 16:10:35.372371
70776	SLVO	2020-10-07	6.26	6.39	6.26	6.38	164062	{}	\N	2020-10-12 16:10:35.376922
70777	SLVO	2020-10-08	6.42	6.44	6.345	6.37	157870	{}	\N	2020-10-12 16:10:35.381615
70778	SLVO	2020-10-09	6.48	6.61	6.451	6.61	361967	{}	\N	2020-10-12 16:10:35.38644
70779	SLVO	2020-10-12	6.65	6.65	6.575	6.596	59498	{}	\N	2020-10-12 16:10:35.391175
70780	SMCI	2020-09-28	26.06	26.55	26.06	26.49	232078	{}	\N	2020-10-12 16:10:37.368458
70781	SMCI	2020-09-29	26.5	26.87	26.5	26.8	174500	{}	\N	2020-10-12 16:10:37.379925
70782	SMCI	2020-09-30	26.66	27.09	26.34	26.4	282910	{}	\N	2020-10-12 16:10:37.388052
70783	SMCI	2020-10-01	26.59	26.75	26.02	26.16	154598	{}	\N	2020-10-12 16:10:37.394048
70784	SMCI	2020-10-02	25.65	26.23	25.3101	25.83	121171	{}	\N	2020-10-12 16:10:37.400276
70785	SMCI	2020-10-05	26.1	26.86	26	26.76	231649	{}	\N	2020-10-12 16:10:37.405418
70786	SMCI	2020-10-06	26.95	27.13	26.38	26.51	183053	{}	\N	2020-10-12 16:10:37.410595
70787	SMCI	2020-10-07	26.68	27.25	26.68	27.03	161049	{}	\N	2020-10-12 16:10:37.41583
70788	SMCI	2020-10-08	27.27	27.66	27.27	27.61	129531	{}	\N	2020-10-12 16:10:37.421275
70789	SMCI	2020-10-09	27.87	27.94	27.63	27.71	96608	{}	\N	2020-10-12 16:10:37.426669
70790	SMCI	2020-10-12	27.89	28.19	27.71	27.955	83942	{}	\N	2020-10-12 16:10:37.431518
70791	SMMF	2020-09-28	15.12	15.59	15.12	15.15	6943	{}	\N	2020-10-12 16:10:39.371017
70792	SMMF	2020-09-29	15.27	15.29	14.85	15.04	16266	{}	\N	2020-10-12 16:10:39.377719
70793	SMMF	2020-09-30	15.18	15.18	14.71	14.81	17273	{}	\N	2020-10-12 16:10:39.383209
70794	SMMF	2020-10-01	14.82	15	14.57	14.83	20504	{}	\N	2020-10-12 16:10:39.38854
70795	SMMF	2020-10-02	14.5	15.1	14.5	14.925	18655	{}	\N	2020-10-12 16:10:39.394611
70796	SMMF	2020-10-05	15.29	15.4	14.96	15.4	13273	{}	\N	2020-10-12 16:10:39.401233
70797	SMMF	2020-10-06	15.58	16.32	15.5	15.6	16441	{}	\N	2020-10-12 16:10:39.407938
70798	SMMF	2020-10-07	15.873	16.3799	15.85	16.35	11447	{}	\N	2020-10-12 16:10:39.416295
70799	SMMF	2020-10-08	16.27	17.305	16.04	17.16	38946	{}	\N	2020-10-12 16:10:39.421429
70800	SMMF	2020-10-09	17.21	17.46	17.2	17.46	2592	{}	\N	2020-10-12 16:10:39.427997
70801	SMMF	2020-10-12	17.41	17.56	17.2	17.23	1656	{}	\N	2020-10-12 16:10:39.433071
70802	SNAP	2020-09-28	25.35	25.84	25.21	25.74	19971965	{}	\N	2020-10-12 16:10:40.435113
70803	SNAP	2020-09-29	25.83	26.36	25.7701	25.99	31285295	{}	\N	2020-10-12 16:10:40.447703
70804	SNAP	2020-09-30	25.87	26.47	25.82	26.11	25648241	{}	\N	2020-10-12 16:10:40.455329
70805	SNAP	2020-10-01	26.28	27.02	26.19	26.78	33437271	{}	\N	2020-10-12 16:10:40.461787
70806	SNAP	2020-10-02	26.23	26.9879	26.13	26.64	28684073	{}	\N	2020-10-12 16:10:40.467539
70807	SNAP	2020-10-05	26.95	27.64	26.91	27.54	28415729	{}	\N	2020-10-12 16:10:40.472756
70808	SNAP	2020-10-06	27.5	27.79	26.51	26.73	28764841	{}	\N	2020-10-12 16:10:40.477319
70809	SNAP	2020-10-07	27.1	27.13	26.62	26.67	17375885	{}	\N	2020-10-12 16:10:40.482063
70810	SNAP	2020-10-08	27	27.1348	26.81	27.05	19119294	{}	\N	2020-10-12 16:10:40.486885
70811	SNAP	2020-10-09	27.16	27.47	26.895	27	21400675	{}	\N	2020-10-12 16:10:40.491682
70812	SNAP	2020-10-12	27.36	27.67	26.99	27.1182	10554163	{}	\N	2020-10-12 16:10:40.496337
70813	SNBR	2020-09-28	48.73	48.91	47.33	48.34	192858	{}	\N	2020-10-12 16:10:41.461696
70814	SNBR	2020-09-29	47.76	48.88	47.56	47.8	215968	{}	\N	2020-10-12 16:10:41.471652
70815	SNBR	2020-09-30	47.91	49.18	47.85	48.91	276948	{}	\N	2020-10-12 16:10:41.478267
70816	SNBR	2020-10-01	49.83	49.935	48.23	49.46	229982	{}	\N	2020-10-12 16:10:41.484245
70817	SNBR	2020-10-02	48.15	52.2773	48.15	51.79	343616	{}	\N	2020-10-12 16:10:41.489342
70818	SNBR	2020-10-05	51.99	54.7941	51.48	54.43	579019	{}	\N	2020-10-12 16:10:41.494333
70819	SNBR	2020-10-06	55.22	55.595	52.63	53.18	345045	{}	\N	2020-10-12 16:10:41.498558
70820	SNBR	2020-10-07	54.44	54.44	51.81	53.02	372448	{}	\N	2020-10-12 16:10:41.503193
70821	SNBR	2020-10-08	53.42	54.58	52.26	54.19	301252	{}	\N	2020-10-12 16:10:41.507402
70822	SNBR	2020-10-09	55.11	56.59	54.51	55.5	349364	{}	\N	2020-10-12 16:10:41.511402
70823	SNBR	2020-10-12	55.67	57.08	55.55	56.97	161034	{}	\N	2020-10-12 16:10:41.515422
70824	SNGXW	2020-09-28	0.3501	0.4454	0.3501	0.4454	202	{}	\N	2020-10-12 16:10:43.066007
70825	SNGXW	2020-09-29	0.51	0.51	0.4	0.4979	1722	{}	\N	2020-10-12 16:10:43.074511
70826	SNGXW	2020-09-30	0.6	0.6	0.5982	0.5982	200	{}	\N	2020-10-12 16:10:43.08102
70827	SNGXW	2020-10-01	0.5997	0.63	0.562799	0.59	1905	{}	\N	2020-10-12 16:10:43.086453
70828	SNGXW	2020-10-02	0.4066	0.63	0.4066	0.63	213	{}	\N	2020-10-12 16:10:43.091872
70829	SNGXW	2020-10-06	0.5797	0.5797	0.5797	0.5797	208	{}	\N	2020-10-12 16:10:43.096089
70830	SNGXW	2020-10-07	0.549	0.59	0.54	0.59	450	{}	\N	2020-10-12 16:10:43.099972
70831	SNGX	2020-09-28	1.7	1.78	1.69	1.76	166785	{}	\N	2020-10-12 16:10:44.082057
70832	SNGX	2020-09-29	1.76	1.83	1.76	1.78	163972	{}	\N	2020-10-12 16:10:44.091891
70833	SNGX	2020-09-30	1.76	1.85	1.76	1.79	150518	{}	\N	2020-10-12 16:10:44.098828
70834	SNGX	2020-10-01	1.78	1.8401	1.77	1.8	112299	{}	\N	2020-10-12 16:10:44.104959
70835	SNGX	2020-10-02	1.76	1.7925	1.71	1.74	269539	{}	\N	2020-10-12 16:10:44.110463
70836	SNGX	2020-10-05	1.73	1.83	1.72	1.81	171368	{}	\N	2020-10-12 16:10:44.115702
70837	SNGX	2020-10-06	1.79	1.89	1.7701	1.79	179647	{}	\N	2020-10-12 16:10:44.120317
70838	SNGX	2020-10-07	1.8	1.85	1.79	1.83	125849	{}	\N	2020-10-12 16:10:44.124986
70839	SNGX	2020-10-08	1.86	1.9	1.83	1.86	172873	{}	\N	2020-10-12 16:10:44.129884
70840	SNGX	2020-10-09	1.85	1.89	1.75	1.79	259598	{}	\N	2020-10-12 16:10:44.134451
70841	SNGX	2020-10-12	1.86	1.88	1.8	1.8	71484	{}	\N	2020-10-12 16:10:44.138927
70842	SNMP	2020-09-28	0.2835	0.2866	0.2801	0.2812	98395	{}	\N	2020-10-12 16:10:46.678455
70843	SNMP	2020-09-29	0.289	0.35	0.289	0.29	669229	{}	\N	2020-10-12 16:10:46.691095
70844	SNMP	2020-09-30	0.271	0.3185	0.271	0.2875	50023	{}	\N	2020-10-12 16:10:46.70345
70845	SNMP	2020-10-01	0.2878	0.329999	0.2878	0.3277	59235	{}	\N	2020-10-12 16:10:46.709221
70846	SNMP	2020-10-02	0.3465	0.3465	0.2901	0.2901	62657	{}	\N	2020-10-12 16:10:46.714782
70847	SNMP	2020-10-05	0.2814	0.3049	0.28	0.29	34666	{}	\N	2020-10-12 16:10:46.719538
70848	SNMP	2020-10-06	0.29	0.33	0.285	0.285	77856	{}	\N	2020-10-12 16:10:46.724211
70849	SNMP	2020-10-07	0.2853	0.319155	0.2851	0.3013	39315	{}	\N	2020-10-12 16:10:46.728807
70850	SNMP	2020-10-08	0.292	0.55	0.29	0.37	4906343	{}	\N	2020-10-12 16:10:46.733456
70851	SNMP	2020-10-09	1.25	1.6	0.725	0.725	122082855	{}	\N	2020-10-12 16:10:46.737866
70852	SNMP	2020-10-12	0.72	0.85	0.705	0.769	3294328	{}	\N	2020-10-12 16:10:46.742481
70853	SNR	2020-09-28	4.03	4.2243	4.03	4.12	376930	{}	\N	2020-10-12 16:10:48.947573
70854	SNR	2020-09-29	4.12	4.15	3.97	4.02	570598	{}	\N	2020-10-12 16:10:48.957464
70855	SNR	2020-09-30	4.04	4.23	3.98	4	624486	{}	\N	2020-10-12 16:10:48.965112
70856	SNR	2020-10-01	4	4.17	4	4.16	731297	{}	\N	2020-10-12 16:10:48.97131
70857	SNR	2020-10-02	4.03	4.36	4.03	4.31	666687	{}	\N	2020-10-12 16:10:48.977391
70858	SNR	2020-10-05	4.33	4.38	4.12	4.19	652165	{}	\N	2020-10-12 16:10:48.982095
70859	SNR	2020-10-06	4.24	4.45	4.18	4.35	851133	{}	\N	2020-10-12 16:10:48.986567
70860	SNR	2020-10-07	4.42	4.6	4.365	4.47	1370293	{}	\N	2020-10-12 16:10:48.991214
70861	SNR	2020-10-08	4.51	4.745	4.45	4.72	1059388	{}	\N	2020-10-12 16:10:48.996638
70862	SNR	2020-10-09	4.8	4.8	4.455	4.46	890484	{}	\N	2020-10-12 16:10:49.002823
70863	SNR	2020-10-12	4.44	4.47	4.35	4.46	270500	{}	\N	2020-10-12 16:10:49.010008
70864	SLS	2020-09-28	2.53	2.53	2.4	2.4	78151	{}	\N	2020-10-12 16:10:50.778802
70865	SLS	2020-09-29	2.44	2.53	2.38	2.49	76389	{}	\N	2020-10-12 16:10:50.787267
70866	SLS	2020-09-30	2.46	2.7899	2.46	2.65	297940	{}	\N	2020-10-12 16:10:50.793636
70867	SLS	2020-10-01	2.66	2.66	2.51	2.61	113545	{}	\N	2020-10-12 16:10:50.799766
70868	SLS	2020-10-02	2.5	2.65	2.46	2.56	83804	{}	\N	2020-10-12 16:10:50.805377
70869	SLS	2020-10-05	2.6	2.69	2.55	2.65	54147	{}	\N	2020-10-12 16:10:50.810401
70870	SLS	2020-10-06	2.65	2.67	2.5633	2.59	49117	{}	\N	2020-10-12 16:10:50.816087
70871	SLS	2020-10-07	2.61	2.8	2.59	2.74	68385	{}	\N	2020-10-12 16:10:50.821491
70872	SLS	2020-10-08	2.74	2.8	2.63	2.64	85395	{}	\N	2020-10-12 16:10:50.826573
70873	SLS	2020-10-09	2.68	2.78	2.6	2.74	144719	{}	\N	2020-10-12 16:10:50.833164
70874	SLS	2020-10-12	2.8	2.8	2.58	2.5958	51780	{}	\N	2020-10-12 16:10:50.838434
70875	SLYV	2020-09-28	49.58	50.67	49.58	50.41	166199	{}	\N	2020-10-12 16:10:51.951381
70876	SLYV	2020-09-29	50.35	50.35	49.3513	49.94	205333	{}	\N	2020-10-12 16:10:51.961022
70877	SLYV	2020-09-30	50.06	50.97	49.71	50.1	164940	{}	\N	2020-10-12 16:10:51.968318
70878	SLYV	2020-10-01	50.7	50.84	49.9566	50.84	175410	{}	\N	2020-10-12 16:10:51.97465
70879	SLYV	2020-10-02	49.82	51.8399	49.82	51.63	294464	{}	\N	2020-10-12 16:10:51.980322
70880	SLYV	2020-10-05	52.22	52.95	52.2001	52.91	137721	{}	\N	2020-10-12 16:10:51.985249
70881	SLYV	2020-10-06	53.37	54.49	52.82	52.84	279219	{}	\N	2020-10-12 16:10:51.99023
70882	SLYV	2020-10-07	53.49	54.02	53.33	53.81	239319	{}	\N	2020-10-12 16:10:51.995189
70883	SLYV	2020-10-08	54.25	54.77	54.05	54.7	515160	{}	\N	2020-10-12 16:10:52.00039
70884	SLYV	2020-10-09	55.16	55.34	54.55	54.71	148650	{}	\N	2020-10-12 16:10:52.005554
70885	SLYV	2020-10-12	54.88	55.0623	54.5499	54.865	99314	{}	\N	2020-10-12 16:10:52.010703
70886	SMFG	2020-09-28	5.77	5.86	5.77	5.84	1105578	{}	\N	2020-10-12 16:10:52.989708
70887	SMFG	2020-09-29	5.75	5.76	5.7	5.73	2472110	{}	\N	2020-10-12 16:10:53.004273
70888	SMFG	2020-09-30	5.55	5.62	5.55	5.59	2336065	{}	\N	2020-10-12 16:10:53.013419
70889	SMFG	2020-10-01	5.59	5.62	5.57	5.61	2149351	{}	\N	2020-10-12 16:10:53.020678
70890	SMFG	2020-10-02	5.61	5.69	5.58	5.66	2549214	{}	\N	2020-10-12 16:10:53.026406
70891	SMFG	2020-10-05	5.69	5.74	5.68	5.74	2485368	{}	\N	2020-10-12 16:10:53.032044
70892	SMFG	2020-10-06	5.75	5.765	5.685	5.69	3089778	{}	\N	2020-10-12 16:10:53.038018
70893	SMFG	2020-10-07	5.74	5.76	5.71	5.73	2405466	{}	\N	2020-10-12 16:10:53.044054
70894	SMFG	2020-10-08	5.73	5.7366	5.7	5.7	2796487	{}	\N	2020-10-12 16:10:53.050246
70895	SMFG	2020-10-09	5.68	5.6999	5.65	5.65	2717561	{}	\N	2020-10-12 16:10:53.055607
70896	SMFG	2020-10-12	5.69	5.71	5.68	5.705	259603	{}	\N	2020-10-12 16:10:53.061428
70897	SMLP	2020-09-28	0.719	0.76	0.66	0.69	1161183	{}	\N	2020-10-12 16:10:54.832839
70898	SMLP	2020-09-29	0.69	0.7288	0.6275	0.6773	3064161	{}	\N	2020-10-12 16:10:54.843502
70899	SMLP	2020-09-30	0.62	0.69	0.62	0.6543	1195531	{}	\N	2020-10-12 16:10:54.850278
70900	SMLP	2020-10-01	0.643	0.675	0.605	0.6364	1027517	{}	\N	2020-10-12 16:10:54.856107
70901	SMLP	2020-10-02	0.6	0.63	0.6	0.6201	639239	{}	\N	2020-10-12 16:10:54.860997
70902	SMLP	2020-10-05	0.63	0.6592	0.6201	0.6387	457232	{}	\N	2020-10-12 16:10:54.86522
70903	SMLP	2020-10-06	0.65	0.6585	0.611	0.615	654627	{}	\N	2020-10-12 16:10:54.869778
70904	SMLP	2020-10-07	0.62	0.6457	0.6079	0.61	840068	{}	\N	2020-10-12 16:10:54.874902
70905	SMLP	2020-10-08	0.6243	0.9	0.6243	0.7801	22709658	{}	\N	2020-10-12 16:10:54.88101
70906	SMLP	2020-10-09	0.724	0.78	0.7	0.769799	3237139	{}	\N	2020-10-12 16:10:54.887156
70907	SMLP	2020-10-12	0.76	0.7771	0.71	0.7387	749336	{}	\N	2020-10-12 16:10:54.892618
70908	SMTX	2020-09-28	3.41	3.59	3.35	3.45	51510	{}	\N	2020-10-12 16:10:56.055385
70909	SMTX	2020-09-29	3.47	3.8099	3.47	3.57	17726	{}	\N	2020-10-12 16:10:56.066022
70910	SMTX	2020-09-30	3.58	3.66	3.53	3.62	24992	{}	\N	2020-10-12 16:10:56.072876
70911	SMTX	2020-10-01	3.6	3.64	3.54	3.575	76860	{}	\N	2020-10-12 16:10:56.079054
70912	SMTX	2020-10-02	3.52	3.67	3.51	3.59	58394	{}	\N	2020-10-12 16:10:56.084768
70913	SMTX	2020-10-05	3.62	3.8	3.6	3.67	171390	{}	\N	2020-10-12 16:10:56.089929
70914	SMTX	2020-10-06	3.73	3.78	3.63	3.72	133311	{}	\N	2020-10-12 16:10:56.09463
70915	SMTX	2020-10-07	3.87	3.9	3.78	3.88	108303	{}	\N	2020-10-12 16:10:56.099443
70916	SMTX	2020-10-08	3.88	3.8949	3.7	3.74	47685	{}	\N	2020-10-12 16:10:56.104126
70917	SMTX	2020-10-09	3.72	3.775	3.7	3.73	37202	{}	\N	2020-10-12 16:10:56.10865
70918	SMTX	2020-10-12	3.75	3.76	3.675	3.73	40734	{}	\N	2020-10-12 16:10:56.113113
70919	SNFCA	2020-09-28	6.25	6.5	6.25	6.45	14517	{}	\N	2020-10-12 16:11:00.010238
70920	SNFCA	2020-09-29	6.43	6.47	6.4	6.42	5326	{}	\N	2020-10-12 16:11:00.019547
70921	SNFCA	2020-09-30	6.42	6.48	6.3028	6.4	15739	{}	\N	2020-10-12 16:11:00.02723
70922	SNFCA	2020-10-01	6.41	6.5	6.37	6.42	14581	{}	\N	2020-10-12 16:11:00.03362
70923	SNFCA	2020-10-02	6.39	6.5	6.27	6.45	24907	{}	\N	2020-10-12 16:11:00.039966
70924	SNFCA	2020-10-05	6.59	6.59	6.44	6.46	63691	{}	\N	2020-10-12 16:11:00.04633
70925	SNFCA	2020-10-06	6.49	6.59	6.43	6.43	33758	{}	\N	2020-10-12 16:11:00.052269
70926	SNFCA	2020-10-07	6.51	6.58	6.51	6.56	20823	{}	\N	2020-10-12 16:11:00.058779
70927	SNFCA	2020-10-08	6.58	6.88	6.52	6.83	27465	{}	\N	2020-10-12 16:11:00.065136
70928	SNFCA	2020-10-09	6.85	7.22	6.85	7.2	33247	{}	\N	2020-10-12 16:11:00.069976
70929	SNFCA	2020-10-12	7.07	7.22	6.95	7.1027	6406	{}	\N	2020-10-12 16:11:00.074862
70930	SNES	2020-09-28	1.85	1.85	1.59	1.72	624222	{}	\N	2020-10-12 16:11:00.998241
70931	SNES	2020-09-29	1.78	1.84	1.6302	1.82	251062	{}	\N	2020-10-12 16:11:01.00966
70932	SNES	2020-09-30	1.85	1.9299	1.8	1.88	210498	{}	\N	2020-10-12 16:11:01.018104
70933	SNES	2020-10-01	1.88	1.9	1.75	1.76	79299	{}	\N	2020-10-12 16:11:01.024709
70934	SNES	2020-10-02	1.7	1.82	1.63	1.78	93631	{}	\N	2020-10-12 16:11:01.030748
70935	SNES	2020-10-05	1.78	1.8681	1.74	1.76	35869	{}	\N	2020-10-12 16:11:01.035693
70936	SNES	2020-10-06	1.76	2.13	1.76	2.12	392590	{}	\N	2020-10-12 16:11:01.040978
70937	SNES	2020-10-07	2.21	2.21	1.95	1.99	175679	{}	\N	2020-10-12 16:11:01.046429
70938	SNES	2020-10-08	2.04	2.06	1.8	2.0055	131722	{}	\N	2020-10-12 16:11:01.051772
70939	SNES	2020-10-09	2.02	2.02	1.91	1.92	53276	{}	\N	2020-10-12 16:11:01.056801
70940	SNES	2020-10-12	2.02	2.065	1.94	1.94	18827	{}	\N	2020-10-12 16:11:01.06187
70941	SNOA	2020-09-28	8.68	8.6999	8.52	8.53	16281	{}	\N	2020-10-12 16:11:02.409287
70942	SNOA	2020-09-29	8.42	8.5	8.2801	8.43	6886	{}	\N	2020-10-12 16:11:02.419107
70943	SNOA	2020-09-30	8.38	8.61	8.31	8.37	15057	{}	\N	2020-10-12 16:11:02.425561
70944	SNOA	2020-10-01	8.32	8.49	8.29	8.46	15247	{}	\N	2020-10-12 16:11:02.431589
70945	SNOA	2020-10-02	8.32	8.4599	8.02	8.35	7420	{}	\N	2020-10-12 16:11:02.436664
70946	SNOA	2020-10-05	8.41	8.41	8.0801	8.09	16738	{}	\N	2020-10-12 16:11:02.441466
70947	SNOA	2020-10-06	8.09	8.23	7.88	8.04	21421	{}	\N	2020-10-12 16:11:02.445977
70948	SNOA	2020-10-07	8.066	8.4999	8.065	8.4	15620	{}	\N	2020-10-12 16:11:02.45158
70949	SNOA	2020-10-08	8.115	8.49	8.115	8.36	23073	{}	\N	2020-10-12 16:11:02.455788
70950	SNOA	2020-10-09	8.38	8.4	8.0701	8.34	11562	{}	\N	2020-10-12 16:11:02.460276
70951	SNOA	2020-10-12	8.2901	8.43	8.06	8.1907	11789	{}	\N	2020-10-12 16:11:02.465078
70952	SNSS	2020-09-28	1.36	1.36	1.26	1.29	1036575	{}	\N	2020-10-12 16:11:05.339483
70953	SNSS	2020-09-29	1.3	1.33	1.26	1.32	518613	{}	\N	2020-10-12 16:11:05.348652
70954	SNSS	2020-09-30	1.32	1.325	1.25	1.25	459396	{}	\N	2020-10-12 16:11:05.354998
70955	SNSS	2020-10-01	1.26	1.3715	1.23	1.31	575041	{}	\N	2020-10-12 16:11:05.361951
70956	SNSS	2020-10-02	1.27	1.42	1.27	1.3	711333	{}	\N	2020-10-12 16:11:05.366715
70957	SNSS	2020-10-05	1.3	1.37	1.27	1.33	458116	{}	\N	2020-10-12 16:11:05.371924
70958	SNSS	2020-10-06	1.32	1.395	1.31	1.34	397943	{}	\N	2020-10-12 16:11:05.37631
70959	SNSS	2020-10-07	1.35	1.39	1.3308	1.38	276229	{}	\N	2020-10-12 16:11:05.381598
70960	SNSS	2020-10-08	1.38	1.46	1.37	1.38	787499	{}	\N	2020-10-12 16:11:05.386521
70961	SNSS	2020-10-09	1.44	1.44	1.36	1.41	337550	{}	\N	2020-10-12 16:11:05.391319
70962	SNSS	2020-10-12	1.44	1.45	1.4	1.43	301700	{}	\N	2020-10-12 16:11:05.396185
70963	SNY	2020-09-28	51.17	51.27	50.75	51	647634	{}	\N	2020-10-12 16:11:07.825003
70964	SNY	2020-09-29	50.73	51	45.69	50.5	615289	{}	\N	2020-10-12 16:11:07.837184
70965	SNY	2020-09-30	50.76	50.7854	49.9	50.17	977102	{}	\N	2020-10-12 16:11:07.84393
70966	SNY	2020-10-01	50	50.3141	49.83	49.96	1193990	{}	\N	2020-10-12 16:11:07.850025
70967	SNY	2020-10-02	49.69	50.71	49.67	50.48	524131	{}	\N	2020-10-12 16:11:07.85578
70968	SNY	2020-10-05	50.36	51.1	50.36	50.98	643966	{}	\N	2020-10-12 16:11:07.86101
70969	SNY	2020-10-06	50.48	50.5	49.64	49.73	2626456	{}	\N	2020-10-12 16:11:07.867614
70970	SNY	2020-10-07	49.67	49.6964	49.22	49.5	748641	{}	\N	2020-10-12 16:11:07.873026
70971	SNY	2020-10-08	49.9	50.39	49.83	50.36	868845	{}	\N	2020-10-12 16:11:07.877802
70972	SNY	2020-10-09	50.86	51.13	50.66	50.81	770529	{}	\N	2020-10-12 16:11:07.885239
70973	SNY	2020-10-12	51.29	51.2901	50.9	50.9269	266886	{}	\N	2020-10-12 16:11:07.891422
70974	SOCL	2020-09-28	48.55	48.55	48.0716	48.39	25312	{}	\N	2020-10-12 16:11:10.227323
70975	SOCL	2020-09-29	48.32	48.63	48.12	48.57	20593	{}	\N	2020-10-12 16:11:10.232472
70976	SOCL	2020-09-30	48.67	49.23	48.67	48.8	22694	{}	\N	2020-10-12 16:11:10.238008
70977	SOCL	2020-10-01	49.3	50.0958	49.3	49.95	17046	{}	\N	2020-10-12 16:11:10.242614
70978	SOCL	2020-10-02	48.94	49.83	48.94	49.16	22673	{}	\N	2020-10-12 16:11:10.248093
70979	SOCL	2020-10-05	49.49	50.2	49.49	50.2	54350	{}	\N	2020-10-12 16:11:10.253908
70980	SOCL	2020-10-06	50.19	50.44	49.39	49.66	32387	{}	\N	2020-10-12 16:11:10.259606
70981	SOCL	2020-10-07	50.05	50.05	49.64	49.8156	16557	{}	\N	2020-10-12 16:11:10.265243
70982	SOCL	2020-10-08	50.07	50.21	49.8	49.86	49895	{}	\N	2020-10-12 16:11:10.271185
70983	SOCL	2020-10-09	50.18	50.3594	50.08	50.2	19488	{}	\N	2020-10-12 16:11:10.276821
70984	SOCL	2020-10-12	50.62	50.745	50.49	50.6	10955	{}	\N	2020-10-12 16:11:10.282711
70985	RELX	2020-09-28	22.51	22.57	22.3364	22.41	448610	{}	\N	2020-10-12 16:11:12.078788
70986	RELX	2020-09-29	22.36	22.44	22.2	22.25	1009045	{}	\N	2020-10-12 16:11:12.091793
70987	RELX	2020-09-30	22.34	22.485	22.26	22.37	795384	{}	\N	2020-10-12 16:11:12.100841
70988	RELX	2020-10-01	22.18	22.405	21.98	22.29	2531070	{}	\N	2020-10-12 16:11:12.107196
70989	RELX	2020-10-02	22.3	22.675	22.29	22.6	1087861	{}	\N	2020-10-12 16:11:12.11267
70990	RELX	2020-10-05	22.81	22.93	22.77	22.89	857211	{}	\N	2020-10-12 16:11:12.117518
70991	RELX	2020-10-06	22.57	22.57	22.17	22.23	1151381	{}	\N	2020-10-12 16:11:12.122484
70992	RELX	2020-10-07	21.82	21.854	21.63	21.68	1680556	{}	\N	2020-10-12 16:11:12.127098
70993	RELX	2020-10-08	21.84	21.9	21.78	21.8	868012	{}	\N	2020-10-12 16:11:12.131764
70994	RELX	2020-10-09	21.84	22.14	21.83	22.07	1412659	{}	\N	2020-10-12 16:11:12.136287
70995	RELX	2020-10-12	22.2	22.265	22.15	22.16	542340	{}	\N	2020-10-12 16:11:12.141164
70996	RFAP	2020-09-29	50.31	50.31	50.31	50.31	426	{}	\N	2020-10-12 16:11:13.361118
70997	RFAP	2020-09-30	50.23	50.3102	50.23	50.25	1192	{}	\N	2020-10-12 16:11:13.366948
70998	RFAP	2020-10-02	49.875	50.0672	49.875	50.0672	621	{}	\N	2020-10-12 16:11:13.373046
70999	RFAP	2020-10-05	50.2299	50.4165	50.2299	50.4165	350	{}	\N	2020-10-12 16:11:13.378561
71000	RFAP	2020-10-08	51.02	51.02	51.02	51.02	130	{}	\N	2020-10-12 16:11:13.384335
71001	RFAP	2020-10-09	51.224	51.2849	51.224	51.2849	331	{}	\N	2020-10-12 16:11:13.391234
71002	RFAP	2020-10-12	51.35	51.42	51.35	51.42	382	{}	\N	2020-10-12 16:11:13.398056
71003	RFI	2020-09-28	12.275	12.36	12.15	12.15	94251	{}	\N	2020-10-12 16:11:16.94928
71004	RFI	2020-09-29	12.19	12.2136	11.96	11.99	181427	{}	\N	2020-10-12 16:11:16.965053
71005	RFI	2020-09-30	12.02	12.16	11.84	11.93	267908	{}	\N	2020-10-12 16:11:16.97339
71006	RFI	2020-10-01	12.02	12.14	11.97	12.14	144490	{}	\N	2020-10-12 16:11:16.979876
71007	RFI	2020-10-02	11.95	12.28	11.91	12.23	149376	{}	\N	2020-10-12 16:11:16.985547
71008	RFI	2020-10-05	12.35	12.39	12.1562	12.36	106652	{}	\N	2020-10-12 16:11:16.990611
71009	RFI	2020-10-06	12.4	12.56	12.3	12.37	224841	{}	\N	2020-10-12 16:11:16.995603
71010	RFI	2020-10-07	12.41	12.41	12.2	12.34	212168	{}	\N	2020-10-12 16:11:17.001212
71011	RFI	2020-10-08	12.42	12.59	12.3516	12.59	100376	{}	\N	2020-10-12 16:11:17.006444
71012	RFI	2020-10-09	12.61	12.61	12.49	12.5	128748	{}	\N	2020-10-12 16:11:17.011792
71013	RFI	2020-10-12	12.51	12.52	12.4139	12.49	97522	{}	\N	2020-10-12 16:11:17.01672
71014	RHS	2020-09-28	142.12	142.7101	141.6068	142.03	18535	{}	\N	2020-10-12 16:11:18.740616
71015	RHS	2020-09-29	141.68	142.299	140.93	141.13	10550	{}	\N	2020-10-12 16:11:18.753148
71016	RHS	2020-09-30	141.86	143.46	141.7422	142.9587	9231	{}	\N	2020-10-12 16:11:18.76066
71017	RHS	2020-10-01	143.29	143.4	142.02	143.11	10931	{}	\N	2020-10-12 16:11:18.766927
71018	RHS	2020-10-02	142.49	143.6147	142.46	143.1754	8883	{}	\N	2020-10-12 16:11:18.772654
71019	RHS	2020-10-05	143.33	144.2411	143.1584	144.17	7365	{}	\N	2020-10-12 16:11:18.777913
71020	RHS	2020-10-06	144.44	145.148	143.06	143.3	8766	{}	\N	2020-10-12 16:11:18.782454
71021	RHS	2020-10-07	144.05	145.4197	144.05	145.08	21324	{}	\N	2020-10-12 16:11:18.786941
71022	RHS	2020-10-08	145.59	146.01	145.4569	146.01	4597	{}	\N	2020-10-12 16:11:18.791301
71023	RHS	2020-10-09	146.68	147.19	146.3683	146.62	8136	{}	\N	2020-10-12 16:11:18.796488
71024	RHS	2020-10-12	147.195	148.1597	147.195	148.1597	1857	{}	\N	2020-10-12 16:11:18.800898
71025	RILYG	2020-09-28	24.38	24.49	24.3	24.42	6084	{}	\N	2020-10-12 16:11:20.817635
71026	RILYG	2020-09-29	24.4	24.5	24.35	24.5	6181	{}	\N	2020-10-12 16:11:20.827806
71027	RILYG	2020-09-30	24.49	24.6023	24.4	24.4	7092	{}	\N	2020-10-12 16:11:20.834683
71028	RILYG	2020-10-01	24.52	24.67	24.31	24.6	14998	{}	\N	2020-10-12 16:11:20.840959
71029	RILYG	2020-10-02	24.36	24.7	24.3	24.67	6512	{}	\N	2020-10-12 16:11:20.846973
71030	RILYG	2020-10-05	24.68	24.93	24.68	24.9201	3635	{}	\N	2020-10-12 16:11:20.851712
71031	RILYG	2020-10-06	24.9145	25	24.8577	24.99	4792	{}	\N	2020-10-12 16:11:20.856114
71032	RILYG	2020-10-07	25	25.03	24.8292	24.85	17129	{}	\N	2020-10-12 16:11:20.860797
71033	RILYG	2020-10-08	24.9501	24.99	24.85	24.924	11696	{}	\N	2020-10-12 16:11:20.865745
71034	RILYG	2020-10-09	24.93	24.95	24.59	24.78	13416	{}	\N	2020-10-12 16:11:20.870519
71035	RILYG	2020-10-12	24.78	24.87	24.73	24.87	1986	{}	\N	2020-10-12 16:11:20.875043
71036	RJI	2020-09-28	4.33	4.34	4.3	4.3392	65927	{}	\N	2020-10-12 16:11:22.086302
71037	RJI	2020-09-29	4.28	4.299	4.24	4.25	72170	{}	\N	2020-10-12 16:11:22.094635
71038	RJI	2020-09-30	4.2899	4.3256	4.2801	4.32	133930	{}	\N	2020-10-12 16:11:22.100075
71039	RJI	2020-10-01	4.2542	4.265	4.22	4.26	54461	{}	\N	2020-10-12 16:11:22.105884
71040	RJI	2020-10-02	4.17	4.2398	4.17	4.18	94235	{}	\N	2020-10-12 16:11:22.111318
71041	RJI	2020-10-05	4.27	4.32	4.27	4.3	58497	{}	\N	2020-10-12 16:11:22.116134
71042	RJI	2020-10-06	4.34	4.38	4.31	4.32	87724	{}	\N	2020-10-12 16:11:22.120752
71043	RJI	2020-10-07	4.355	4.355	4.32	4.33	73412	{}	\N	2020-10-12 16:11:22.125289
71044	RJI	2020-10-08	4.37	4.39	4.37	4.39	28290	{}	\N	2020-10-12 16:11:22.130038
71045	RJI	2020-10-09	4.41	4.4468	4.41	4.42	48989	{}	\N	2020-10-12 16:11:22.134695
71046	RJI	2020-10-12	4.41	4.41	4.36	4.381	14109	{}	\N	2020-10-12 16:11:22.139257
71047	RM	2020-09-28	16.47	17.05	16.47	16.8	31970	{}	\N	2020-10-12 16:11:23.4671
71048	RM	2020-09-29	16.77	16.99	16.47	16.57	41593	{}	\N	2020-10-12 16:11:23.48046
71049	RM	2020-09-30	16.72	17.05	16.355	16.66	70721	{}	\N	2020-10-12 16:11:23.48864
71050	RM	2020-10-01	16.75	17	16.48	16.94	64569	{}	\N	2020-10-12 16:11:23.495991
71051	RM	2020-10-02	16.57	17.41	16.45	17.35	55013	{}	\N	2020-10-12 16:11:23.501868
71052	RM	2020-10-05	17.62	18.19	17.5	18.18	76416	{}	\N	2020-10-12 16:11:23.506831
71053	RM	2020-10-06	18.25	18.91	18.25	18.3	69616	{}	\N	2020-10-12 16:11:23.511534
71054	RM	2020-10-07	18.46	19.24	18.25	19.02	98186	{}	\N	2020-10-12 16:11:23.516199
71055	RM	2020-10-08	19.21	19.81	19.21	19.59	49888	{}	\N	2020-10-12 16:11:23.52085
71056	RM	2020-10-09	19.76	20.55	19.74	20.47	73724	{}	\N	2020-10-12 16:11:23.525667
71057	RM	2020-10-12	20.45	20.45	20.1	20.12	5662	{}	\N	2020-10-12 16:11:23.530975
71058	RNDV	2020-09-28	21.529	21.58	21.485	21.58	9996	{}	\N	2020-10-12 16:11:24.830067
71059	RNDV	2020-09-30	21.34	21.46	21.34	21.3855	1181	{}	\N	2020-10-12 16:11:24.837059
71060	RNDV	2020-10-01	21.29	21.41	21.29	21.35	2728	{}	\N	2020-10-12 16:11:24.842413
71061	RNDV	2020-10-06	22.1199	22.1199	21.6203	21.6203	328	{}	\N	2020-10-12 16:11:24.84783
71062	RNDV	2020-10-08	22.32	22.3499	22.2355	22.3499	6520	{}	\N	2020-10-12 16:11:24.853177
71063	RNDV	2020-10-09	22.35	22.58	22.2547	22.2547	414	{}	\N	2020-10-12 16:11:24.858365
71064	RNST	2020-09-28	22.31	23.29	22.29	22.66	197532	{}	\N	2020-10-12 16:11:25.864143
71065	RNST	2020-09-29	22.81	22.87	22.27	22.815	162288	{}	\N	2020-10-12 16:11:25.875442
71066	RNST	2020-09-30	22.97	23.12	22.53	22.72	230809	{}	\N	2020-10-12 16:11:25.882127
71067	RNST	2020-10-01	22.64	23.06	22.42	23.01	281351	{}	\N	2020-10-12 16:11:25.887594
71068	RNST	2020-10-02	25.19	26.42	24.44	24.82	3803112	{}	\N	2020-10-12 16:11:25.892472
71069	RNST	2020-10-05	24.81	26.06	24.717	25.92	1082185	{}	\N	2020-10-12 16:11:25.896825
71070	RNST	2020-10-06	26.25	27.14	25.97	26.42	10890125	{}	\N	2020-10-12 16:11:25.901784
71071	RNST	2020-10-07	26.75	27.3	26.48	26.89	1231087	{}	\N	2020-10-12 16:11:25.906785
71072	RNST	2020-10-08	27.05	27.35	26.97	27.17	976633	{}	\N	2020-10-12 16:11:25.911026
71073	RNST	2020-10-09	27.33	27.49	26.69	26.7	440064	{}	\N	2020-10-12 16:11:25.915089
71074	RNST	2020-10-12	26.62	27.66	26.62	27.66	122328	{}	\N	2020-10-12 16:11:25.91914
71075	ROK	2020-09-28	219.12	221.74	218.37	219.99	460528	{}	\N	2020-10-12 16:11:27.043374
71076	ROK	2020-09-29	220.05	222.64	218.69	221.23	749150	{}	\N	2020-10-12 16:11:27.054289
71077	ROK	2020-09-30	222.58	225.78	219.55	220.68	1150690	{}	\N	2020-10-12 16:11:27.060818
71078	ROK	2020-10-01	222.67	224.36	216.36	219.52	1371329	{}	\N	2020-10-12 16:11:27.067597
71079	ROK	2020-10-02	215.72	225.49	215.51	223.43	960775	{}	\N	2020-10-12 16:11:27.074487
71080	ROK	2020-10-05	227.13	233.25	226.225	233.22	1436042	{}	\N	2020-10-12 16:11:27.080214
71081	ROK	2020-10-06	233.6	238.23	232.41	232.79	1012959	{}	\N	2020-10-12 16:11:27.085787
71082	ROK	2020-10-07	235.95	238.28	233.39	236.42	852840	{}	\N	2020-10-12 16:11:27.090286
71083	ROK	2020-10-08	237.43	241.35	235.4	241.12	1303759	{}	\N	2020-10-12 16:11:27.094852
71084	ROK	2020-10-09	243.92	246.46	242.6501	244.28	1502008	{}	\N	2020-10-12 16:11:27.099265
71085	ROK	2020-10-12	247.78	249	244.68	245.77	362985	{}	\N	2020-10-12 16:11:27.103665
71086	SOXX	2020-09-28	299.44	303.68	297.5	303.38	538014	{}	\N	2020-10-12 16:11:29.087637
71087	SOXX	2020-09-29	303.64	306.94	303.18	304.09	590401	{}	\N	2020-10-12 16:11:29.09613
71088	SOXX	2020-09-30	303.89	307.61	302.83	304.61	578305	{}	\N	2020-10-12 16:11:29.103902
71089	SOXX	2020-10-01	308.75	311.72	307.33	311.13	611696	{}	\N	2020-10-12 16:11:29.111147
71090	SOXX	2020-10-02	303.09	308.09	301.38	301.8	565432	{}	\N	2020-10-12 16:11:29.117737
71091	SOXX	2020-10-05	304.55	311.51	304.14	311.23	483370	{}	\N	2020-10-12 16:11:29.124148
71092	SOXX	2020-10-06	311.48	317.21	308.48	309.75	562736	{}	\N	2020-10-12 16:11:29.131005
71093	SOXX	2020-10-07	314.16	316.65	314.1062	315.68	385497	{}	\N	2020-10-12 16:11:29.136311
71094	SOXX	2020-10-08	318.57	320.24	317.57	319.59	326136	{}	\N	2020-10-12 16:11:29.141275
71095	SOXX	2020-10-09	324.21	326.12	322.98	325.21	626150	{}	\N	2020-10-12 16:11:29.146292
71096	SOXX	2020-10-12	329.51	329.97	327.87	329.12	285710	{}	\N	2020-10-12 16:11:29.151054
71097	SPGI	2020-09-28	357.21	359.68	355.12	355.55	755446	{}	\N	2020-10-12 16:11:31.08673
71098	SPGI	2020-09-29	355.94	360.32	354.55	356.98	1143369	{}	\N	2020-10-12 16:11:31.103069
71099	SPGI	2020-09-30	357	363.71	355.62	360.6	1977149	{}	\N	2020-10-12 16:11:31.110351
71100	SPGI	2020-10-01	364.89	368.1442	361.285	363.73	1256417	{}	\N	2020-10-12 16:11:31.115985
71101	SPGI	2020-10-02	359.87	365.5	357.0664	362.98	1390076	{}	\N	2020-10-12 16:11:31.121119
71102	SPGI	2020-10-05	365.7	365.98	361.98	364.97	1014180	{}	\N	2020-10-12 16:11:31.126242
71103	SPGI	2020-10-06	367.23	367.23	354.05	354.31	1504407	{}	\N	2020-10-12 16:11:31.130502
71104	SPGI	2020-10-07	355.92	358.83	351.88	354.9	1369212	{}	\N	2020-10-12 16:11:31.134819
71105	SPGI	2020-10-08	355.67	360.7764	355.08	357.2	1058199	{}	\N	2020-10-12 16:11:31.139077
71106	SPGI	2020-10-09	358.98	359.71	356.06	356.76	1564091	{}	\N	2020-10-12 16:11:31.143819
71107	SPGI	2020-10-12	361.64	364.09	358.18	360.82	479318	{}	\N	2020-10-12 16:11:31.148391
71108	SPIB	2020-09-28	36.73	36.77	36.67	36.77	1335538	{}	\N	2020-10-12 16:11:32.398929
71109	SPIB	2020-09-29	36.81	36.81	36.77	36.78	688162	{}	\N	2020-10-12 16:11:32.411143
71110	SPIB	2020-09-30	36.8	36.8	36.75	36.76	1316211	{}	\N	2020-10-12 16:11:32.418154
71111	SPIB	2020-10-01	36.72	36.76	36.69	36.75	1584544	{}	\N	2020-10-12 16:11:32.424371
71112	SPIB	2020-10-02	36.71	36.77	36.71	36.75	1798383	{}	\N	2020-10-12 16:11:32.430068
71113	SPIB	2020-10-05	36.77	36.87	36.71	36.71	754963	{}	\N	2020-10-12 16:11:32.435413
71114	SPIB	2020-10-06	36.7522	36.7963	36.71	36.72	1281891	{}	\N	2020-10-12 16:11:32.440098
71115	SPIB	2020-10-07	36.76	36.76	36.7	36.7	1269404	{}	\N	2020-10-12 16:11:32.44468
71116	SPIB	2020-10-08	36.7122	36.76	36.7122	36.75	1329400	{}	\N	2020-10-12 16:11:32.448973
71117	SPIB	2020-10-09	36.72	36.8	36.72	36.76	2359113	{}	\N	2020-10-12 16:11:32.453816
71118	SPIB	2020-10-12	36.78	36.855	36.78	36.855	145790	{}	\N	2020-10-12 16:11:32.458827
71119	SPLP	2020-09-28	6.3	6.3	6	6	457	{}	\N	2020-10-12 16:11:33.487739
71120	SPLP	2020-09-29	6	6.1799	6	6.1799	217242	{}	\N	2020-10-12 16:11:33.498717
71121	SPLP	2020-09-30	6.03	6.25	6.02	6.1739	14146	{}	\N	2020-10-12 16:11:33.505925
71122	SPLP	2020-10-01	6.68	7.2	6.17	6.71	372868	{}	\N	2020-10-12 16:11:33.51207
71123	SPLP	2020-10-02	6.785	7	6.67	6.67	22788	{}	\N	2020-10-12 16:11:33.51803
71124	SPLP	2020-10-05	6.67	7.05	6.6	6.6	27482	{}	\N	2020-10-12 16:11:33.52394
71125	SPLP	2020-10-06	6.516	6.67	6.5	6.5	7944	{}	\N	2020-10-12 16:11:33.528931
71126	SPLP	2020-10-07	6.42	6.4319	6.2	6.2	111242	{}	\N	2020-10-12 16:11:33.533886
71127	SPLP	2020-10-08	6.26	6.26	6	6	22078	{}	\N	2020-10-12 16:11:33.538838
71128	SPLP	2020-10-09	6.03	6.22	6	6	20922	{}	\N	2020-10-12 16:11:33.543278
71129	SPLP	2020-10-12	6	6.27	5.36	5.7943	43484	{}	\N	2020-10-12 16:11:33.548376
71130	SPNE	2020-09-28	14.43	14.88	14.205	14.77	96950	{}	\N	2020-10-12 16:11:35.807726
71131	SPNE	2020-09-29	14.69	14.7519	13.93	14.17	97240	{}	\N	2020-10-12 16:11:35.818167
71132	SPNE	2020-09-30	14.26	14.71	14.165	14.34	129806	{}	\N	2020-10-12 16:11:35.826204
71133	SPNE	2020-10-01	14.065	14.64	13.94	14.41	128517	{}	\N	2020-10-12 16:11:35.833174
71134	SPNE	2020-10-02	14.01	15	14.01	14.755	113850	{}	\N	2020-10-12 16:11:35.838682
71135	SPNE	2020-10-05	14.86	15.79	14.86	15.13	109904	{}	\N	2020-10-12 16:11:35.843094
71136	SPNE	2020-10-06	15.62	16.91	14.98	15.12	425440	{}	\N	2020-10-12 16:11:35.847483
71137	SPNE	2020-10-07	15.26	16.25	15.26	16.07	111748	{}	\N	2020-10-12 16:11:35.851968
71138	SPNE	2020-10-08	16	16	15.56	15.63	112186	{}	\N	2020-10-12 16:11:35.856511
71139	SPNE	2020-10-09	15.9	16.24	15.02	15.38	104305	{}	\N	2020-10-12 16:11:35.861029
71140	SPNE	2020-10-12	15.61	15.83	15.15	15.41	13820	{}	\N	2020-10-12 16:11:35.865049
71141	SPSB	2020-09-28	31.36	31.36	31.33	31.35	594806	{}	\N	2020-10-12 16:11:38.481827
71142	SPSB	2020-09-29	31.36	31.37	31.35	31.36	590884	{}	\N	2020-10-12 16:11:38.49615
71143	SPSB	2020-09-30	31.35	31.39	31.35	31.37	719724	{}	\N	2020-10-12 16:11:38.504771
71144	SPSB	2020-10-01	31.33	31.34	31.3	31.33	821114	{}	\N	2020-10-12 16:11:38.511133
71145	SPSB	2020-10-02	31.32	31.35	31.32	31.34	528732	{}	\N	2020-10-12 16:11:38.517196
71146	SPSB	2020-10-05	31.36	31.36	31.34	31.34	747498	{}	\N	2020-10-12 16:11:38.522145
71147	SPSB	2020-10-06	31.32	31.35	31.32	31.33	777258	{}	\N	2020-10-12 16:11:38.527053
71148	SPSB	2020-10-07	31.34	31.35	31.33	31.35	809003	{}	\N	2020-10-12 16:11:38.531937
71149	SPSB	2020-10-08	31.37	31.37	31.34	31.34	766513	{}	\N	2020-10-12 16:11:38.536764
71150	SPSB	2020-10-09	31.35	31.37	31.35	31.36	1662497	{}	\N	2020-10-12 16:11:38.541888
71151	SPSB	2020-10-12	31.38	31.39	31.37	31.385	178176	{}	\N	2020-10-12 16:11:38.546941
71152	SPTM	2020-09-28	40.65	40.88	40.54	40.81	281892	{}	\N	2020-10-12 16:11:40.419329
71153	SPTM	2020-09-29	40.7512	40.8599	40.4732	40.59	254002	{}	\N	2020-10-12 16:11:40.429735
71154	SPTM	2020-09-30	40.65	41.285	40.62	40.87	673013	{}	\N	2020-10-12 16:11:40.436865
71155	SPTM	2020-10-01	41.23	41.32	40.9	41.17	515313	{}	\N	2020-10-12 16:11:40.442922
71156	SPTM	2020-10-02	40.46	41.04	40.38	40.84	256331	{}	\N	2020-10-12 16:11:40.448742
71157	SPTM	2020-10-05	41.13	41.6	41.09	41.59	1319377	{}	\N	2020-10-12 16:11:40.453369
71158	SPTM	2020-10-06	41.61	41.905	40.96	41.04	419013	{}	\N	2020-10-12 16:11:40.457727
71159	SPTM	2020-10-07	41.4	41.8153	41.4	41.73	221575	{}	\N	2020-10-12 16:11:40.462134
71160	SPTM	2020-10-08	41.99	42.11	41.86	42.11	894939	{}	\N	2020-10-12 16:11:40.466309
71161	SPTM	2020-10-09	42.33	42.5222	42.23	42.44	275313	{}	\N	2020-10-12 16:11:40.470636
71162	SPTM	2020-10-12	42.78	43.07	42.73	43.05	213129	{}	\N	2020-10-12 16:11:40.475201
71163	SPXC	2020-09-28	44.49	45.56	43.95	45.45	376775	{}	\N	2020-10-12 16:11:41.441431
71164	SPXC	2020-09-29	45.62	46.11	45.22	45.87	613381	{}	\N	2020-10-12 16:11:41.455209
71165	SPXC	2020-09-30	45.92	46.85	45.64	46.38	727625	{}	\N	2020-10-12 16:11:41.463144
71166	SPXC	2020-10-01	46.75	47.38	45.65	47.13	1402998	{}	\N	2020-10-12 16:11:41.469878
71167	SPXC	2020-10-02	46.16	47.69	45.89	47.06	644436	{}	\N	2020-10-12 16:11:41.475398
71168	SPXC	2020-10-05	47.62	48.28	47.34	48.15	742887	{}	\N	2020-10-12 16:11:41.481025
71169	SPXC	2020-10-06	48.64	49.22	47.9	48.62	641055	{}	\N	2020-10-12 16:11:41.486334
71170	SPXC	2020-10-07	49.38	49.85	48.96	49.41	526837	{}	\N	2020-10-12 16:11:41.4917
71171	SPXC	2020-10-08	49.84	50.07	49.15	49.9	348095	{}	\N	2020-10-12 16:11:41.496126
71172	SPXC	2020-10-09	50.27	50.93	49.994	50.3	233473	{}	\N	2020-10-12 16:11:41.501515
71173	SPXC	2020-10-12	50.32	51.37	50.3	50.984	86673	{}	\N	2020-10-12 16:11:41.506857
71174	SPXT	2020-09-29	57.39	57.39	57.2769	57.2769	187	{}	\N	2020-10-12 16:11:42.766177
71175	SPXT	2020-09-30	57.45	57.7912	57.45	57.7912	1274	{}	\N	2020-10-12 16:11:42.775753
71176	SPXT	2020-10-02	57.7821	57.7821	57.7821	57.7821	337	{}	\N	2020-10-12 16:11:42.782183
71177	SPXT	2020-10-05	58.6379	58.6379	58.6379	58.6379	158	{}	\N	2020-10-12 16:11:42.78773
71178	SPXT	2020-10-06	58.5419	58.5419	57.9369	57.9369	283	{}	\N	2020-10-12 16:11:42.793004
71179	SPXT	2020-10-07	58.8988	58.8988	58.8988	58.8988	76	{}	\N	2020-10-12 16:11:42.797719
71180	SPXT	2020-10-08	59.11	59.4102	59.11	59.4102	165	{}	\N	2020-10-12 16:11:42.802468
71181	SPXT	2020-10-09	59.7402	59.7402	59.7402	59.7402	107	{}	\N	2020-10-12 16:11:42.807234
71182	SPXT	2020-10-12	60.42	60.4432	60.42	60.4432	305	{}	\N	2020-10-12 16:11:42.811825
71183	SON	2020-09-28	50.88	51.24	50.14	51.01	427799	{}	\N	2020-10-12 16:11:44.214342
71184	SON	2020-09-29	51.28	51.28	50.29	50.52	450894	{}	\N	2020-10-12 16:11:44.224761
71185	SON	2020-09-30	50.82	51.499	50.71	51.07	748289	{}	\N	2020-10-12 16:11:44.232335
71186	SON	2020-10-01	51.22	51.73	50.9	51.58	807589	{}	\N	2020-10-12 16:11:44.238836
71187	SON	2020-10-02	50.85	52.36	50.58	51.87	460078	{}	\N	2020-10-12 16:11:44.244473
71188	SON	2020-10-05	52.5	52.7856	51.995	52.53	634985	{}	\N	2020-10-12 16:11:44.249568
71189	SON	2020-10-06	53.07	53.34	52.16	52.23	612088	{}	\N	2020-10-12 16:11:44.254486
71190	SON	2020-10-07	52.74	53.25	52.57	52.92	567505	{}	\N	2020-10-12 16:11:44.259252
71191	SON	2020-10-08	53	53.215	52.54	53.03	675552	{}	\N	2020-10-12 16:11:44.263885
71192	SON	2020-10-09	53.63	54.21	53.01	53.15	821566	{}	\N	2020-10-12 16:11:44.268791
71193	SON	2020-10-12	53.44	54.58	53.32	54.415	139691	{}	\N	2020-10-12 16:11:44.273529
71194	SPE	2020-09-28	12.53	12.54	12.25	12.39	31522	{}	\N	2020-10-12 16:11:45.644603
71195	SPE	2020-09-29	12.28	12.4	12.27	12.4	13662	{}	\N	2020-10-12 16:11:45.657967
71196	SPE	2020-09-30	12.47	12.535	12.325	12.4081	19096	{}	\N	2020-10-12 16:11:45.666985
71197	SPE	2020-10-01	12.34	12.5	12.3	12.4	21562	{}	\N	2020-10-12 16:11:45.67425
71198	SPE	2020-10-02	12.29	12.47	12.17	12.28	25874	{}	\N	2020-10-12 16:11:45.680601
71199	SPE	2020-10-05	12.24	12.4399	12.24	12.375	12406	{}	\N	2020-10-12 16:11:45.68621
71200	SPE	2020-10-06	12.44	12.54	12.07	12.07	47848	{}	\N	2020-10-12 16:11:45.691491
71201	SPE	2020-10-07	12.21	12.5	12.21	12.37	11206	{}	\N	2020-10-12 16:11:45.696487
71202	SPE	2020-10-08	12.48	12.54	12.3435	12.54	42152	{}	\N	2020-10-12 16:11:45.701756
71203	SPE	2020-10-09	12.54	12.54	12.36	12.52	28676	{}	\N	2020-10-12 16:11:45.706589
71204	SPE	2020-10-12	12.55	12.55	12.435	12.4915	6856	{}	\N	2020-10-12 16:11:45.711664
71205	SPHD	2020-09-28	32.97	33.3395	32.96	33.16	941488	{}	\N	2020-10-12 16:11:46.629809
71206	SPHD	2020-09-29	33.13	33.15	32.59	32.77	761141	{}	\N	2020-10-12 16:11:46.639912
71207	SPHD	2020-09-30	32.95	33.3	32.78	32.99	1146583	{}	\N	2020-10-12 16:11:46.646552
71208	SPHD	2020-10-01	33.1	33.25	32.78	33.04	900011	{}	\N	2020-10-12 16:11:46.652318
71209	SPHD	2020-10-02	32.57	33.59	32.54	33.43	1328631	{}	\N	2020-10-12 16:11:46.65751
71210	SPHD	2020-10-05	33.63	33.95	33.57	33.89	780462	{}	\N	2020-10-12 16:11:46.662536
71211	SPHD	2020-10-06	34.03	34.49	33.69	33.73	1180767	{}	\N	2020-10-12 16:11:46.667027
71212	SPHD	2020-10-07	34.02	34.33	33.98	34.23	683294	{}	\N	2020-10-12 16:11:46.671483
71213	SPHD	2020-10-08	34.37	34.86	34.365	34.85	715903	{}	\N	2020-10-12 16:11:46.675888
71214	SPHD	2020-10-09	35.07	35.17	34.61	34.63	682509	{}	\N	2020-10-12 16:11:46.680247
71215	SPHD	2020-10-12	34.71	34.82	34.56	34.7601	292167	{}	\N	2020-10-12 16:11:46.684825
71216	SPLG	2020-09-28	39.15	39.3501	39.03	39.27	3029112	{}	\N	2020-10-12 16:11:47.767498
71217	SPLG	2020-09-29	39.25	39.33	38.9714	39.07	2063457	{}	\N	2020-10-12 16:11:47.779107
71218	SPLG	2020-09-30	39.15	39.7418	39.12	39.33	2906954	{}	\N	2020-10-12 16:11:47.786047
71219	SPLG	2020-10-01	39.68	39.805	39.37	39.59	2800294	{}	\N	2020-10-12 16:11:47.792308
71220	SPLG	2020-10-02	38.96	39.46	38.92	39.22	2535906	{}	\N	2020-10-12 16:11:47.797438
71221	SPLG	2020-10-05	39.48	39.94	39.48	39.92	2805386	{}	\N	2020-10-12 16:11:47.802717
71222	SPLG	2020-10-06	39.95	40.2	39.3	39.35	2357639	{}	\N	2020-10-12 16:11:47.807379
71223	SPLG	2020-10-07	39.72	40.14	39.72	40.03	2402618	{}	\N	2020-10-12 16:11:47.811835
71224	SPLG	2020-10-08	40.29	40.4	40.18	40.39	1465951	{}	\N	2020-10-12 16:11:47.816043
71225	SPLG	2020-10-09	40.62	40.81	40.53	40.74	2845996	{}	\N	2020-10-12 16:11:47.820063
71226	SPLG	2020-10-12	41.06	41.374	41.03	41.36	457101	{}	\N	2020-10-12 16:11:47.824243
71227	SPR	2020-09-28	20.18	20.81	19.8301	19.98	6446623	{}	\N	2020-10-12 16:11:50.141303
71228	SPR	2020-09-29	19.68	19.68	18.85	19	10158754	{}	\N	2020-10-12 16:11:50.155288
71229	SPR	2020-09-30	19	19.5	18.76	18.91	13511587	{}	\N	2020-10-12 16:11:50.163507
71230	SPR	2020-10-01	19.1	19.81	19.0601	19.62	8924612	{}	\N	2020-10-12 16:11:50.169921
71231	SPR	2020-10-02	18.89	19.8	18.78	19.7	7936327	{}	\N	2020-10-12 16:11:50.175474
71232	SPR	2020-10-05	19.9	20	19.03	19.05	9567524	{}	\N	2020-10-12 16:11:50.180654
71233	SPR	2020-10-06	19.42	20.08	18.48	18.5	13867424	{}	\N	2020-10-12 16:11:50.185292
71234	SPR	2020-10-07	18.98	19.34	18.7	19.21	8560455	{}	\N	2020-10-12 16:11:50.19017
71235	SPR	2020-10-08	19.39	20.05	19.28	19.82	9958118	{}	\N	2020-10-12 16:11:50.195023
71236	SPR	2020-10-09	19.99	20.07	19.4	19.42	6255939	{}	\N	2020-10-12 16:11:50.1999
71237	SPR	2020-10-12	19.45	19.63	19.01	19.52	2865436	{}	\N	2020-10-12 16:11:50.204621
71238	SPTL	2020-09-28	47.17	47.17	47.025	47.09	1517864	{}	\N	2020-10-12 16:11:51.748642
71239	SPTL	2020-09-29	47.13	47.25	47.075	47.14	332645	{}	\N	2020-10-12 16:11:51.756461
71240	SPTL	2020-09-30	46.89	46.9072	46.51	46.74	1317144	{}	\N	2020-10-12 16:11:51.764876
71241	SPTL	2020-10-01	46.47	46.835	46.3995	46.75	2203760	{}	\N	2020-10-12 16:11:51.77107
71242	SPTL	2020-10-02	46.8	46.8112	46.41	46.57	2075691	{}	\N	2020-10-12 16:11:51.77672
71243	SPTL	2020-10-05	46.08	46.08	45.68	45.7	1548058	{}	\N	2020-10-12 16:11:51.782246
71244	SPTL	2020-10-06	45.6	46.18	45.45	45.92	1101406	{}	\N	2020-10-12 16:11:51.787489
71245	SPTL	2020-10-07	45.67	45.915	45.495	45.6	849242	{}	\N	2020-10-12 16:11:51.792788
71246	SPTL	2020-10-08	45.78	45.87	45.72	45.86	615243	{}	\N	2020-10-12 16:11:51.797893
71247	SPTL	2020-10-09	45.67	45.8799	45.5	45.83	890831	{}	\N	2020-10-12 16:11:51.802696
71248	SPTL	2020-10-12	45.94	45.95	45.8315	45.8644	109205	{}	\N	2020-10-12 16:11:51.808378
71249	SPXS	2020-09-28	5.85	5.92	5.76	5.81	26825332	{}	\N	2020-10-12 16:11:52.933181
71250	SPXS	2020-09-29	5.81	5.94	5.78	5.91	30546950	{}	\N	2020-10-12 16:11:52.940081
71251	SPXS	2020-09-30	5.87	5.88	5.59	5.76	42434745	{}	\N	2020-10-12 16:11:52.947215
71252	SPXS	2020-10-01	5.62	5.76	5.57	5.66	37989284	{}	\N	2020-10-12 16:11:52.954288
71253	SPXS	2020-10-02	5.92	5.95	5.71	5.81	45960927	{}	\N	2020-10-12 16:11:52.961267
71254	SPXS	2020-10-05	5.7003	5.7012	5.5	5.5101	21615234	{}	\N	2020-10-12 16:11:52.967174
71255	SPXS	2020-10-06	5.5	5.77	5.39	5.75	33193076	{}	\N	2020-10-12 16:11:52.97284
71256	SPXS	2020-10-07	5.58	5.59	5.4	5.44	25226465	{}	\N	2020-10-12 16:11:52.978791
71257	SPXS	2020-10-08	5.33	5.39	5.29	5.29	17569416	{}	\N	2020-10-12 16:11:52.984693
71258	SPXS	2020-10-09	5.21	5.25	5.13	5.15	16858952	{}	\N	2020-10-12 16:11:52.991079
71259	SPXS	2020-10-12	5.03	5.06	4.92	4.925	12076694	{}	\N	2020-10-12 16:11:52.997097
71260	SPYX	2020-09-28	83.21	83.2201	82.67	83	30791	{}	\N	2020-10-12 16:11:55.135489
71261	SPYX	2020-09-29	83.0436	83.225	82.56	82.78	19419	{}	\N	2020-10-12 16:11:55.146348
71262	SPYX	2020-09-30	83.07	83.99	83.03	83.53	33390	{}	\N	2020-10-12 16:11:55.153936
71263	SPYX	2020-10-01	84.67	84.67	83.39	83.75	16955	{}	\N	2020-10-12 16:11:55.160884
71264	SPYX	2020-10-02	82.68	83.49	82.68	83.05	35781	{}	\N	2020-10-12 16:11:55.167059
71265	SPYX	2020-10-05	83.9	84.53	83.77	84.53	17708	{}	\N	2020-10-12 16:11:55.172209
71266	SPYX	2020-10-06	84.81	85.05	83.35	83.37	22754	{}	\N	2020-10-12 16:11:55.176599
71267	SPYX	2020-10-07	84.23	85.01	84.23	84.8974	10429	{}	\N	2020-10-12 16:11:55.181507
71268	SPYX	2020-10-08	85.2243	85.43	85.2	85.43	8778	{}	\N	2020-10-12 16:11:55.186403
71269	SPYX	2020-10-09	85.67	86.3	85.67	86.27	14175	{}	\N	2020-10-12 16:11:55.191203
71270	SPYX	2020-10-12	87.78	87.78	86.9134	87.2317	21210	{}	\N	2020-10-12 16:11:55.196095
71271	SOVB	2020-09-28	24.575	24.575	24.575	24.575	269	{}	\N	2020-10-12 16:11:56.798381
71272	SOVB	2020-09-29	24.525	24.525	24.525	24.525	304	{}	\N	2020-10-12 16:11:56.807234
71273	SOVB	2020-09-30	24.46	24.55	24.45	24.55	2521	{}	\N	2020-10-12 16:11:56.814216
71274	SOVB	2020-10-01	24.72	24.72	24.64	24.64	475	{}	\N	2020-10-12 16:11:56.81959
71275	SOVB	2020-10-02	24.315	24.315	24.315	24.315	282	{}	\N	2020-10-12 16:11:56.826218
71276	SOVB	2020-10-05	24.41	24.575	24.41	24.575	11943	{}	\N	2020-10-12 16:11:56.832035
71277	SOVB	2020-10-06	24.6	24.6	24.6	24.6	17	{}	\N	2020-10-12 16:11:56.837486
71278	SOVB	2020-10-07	24.74	24.749	24.575	24.575	1181	{}	\N	2020-10-12 16:11:56.844066
71279	SOVB	2020-10-08	24.33	24.33	24.33	24.33	20	{}	\N	2020-10-12 16:11:56.850055
71280	SOVB	2020-10-09	24.7	24.7	24.7	24.7	124	{}	\N	2020-10-12 16:11:56.856058
71281	SPB	2020-09-28	57.56	58.46	56.76	57.29	263721	{}	\N	2020-10-12 16:11:57.799265
71282	SPB	2020-09-29	56.94	57.56	56.02	56.49	401330	{}	\N	2020-10-12 16:11:57.80517
71283	SPB	2020-09-30	56.46	57.68	56.4	57.16	600418	{}	\N	2020-10-12 16:11:57.811092
71284	SPB	2020-10-01	57.5	58.825	57.39	58.64	556723	{}	\N	2020-10-12 16:11:57.81723
71285	SPB	2020-10-02	57.47	59.08	57.19	58.7	604527	{}	\N	2020-10-12 16:11:57.822252
71286	SPB	2020-10-05	59.62	60.05	59.1134	59.66	398795	{}	\N	2020-10-12 16:11:57.826787
71287	SPB	2020-10-06	59.66	61.05	59.43	59.93	618296	{}	\N	2020-10-12 16:11:57.832574
71288	SPB	2020-10-07	60.41	61.63	60.12	60.64	859862	{}	\N	2020-10-12 16:11:57.841163
71289	SPB	2020-10-08	61.04	61.94	60.32	61.67	390028	{}	\N	2020-10-12 16:11:57.850329
71290	SPB	2020-10-09	62.04	63.03	61.28	62.14	934611	{}	\N	2020-10-12 16:11:57.856196
71291	SPB	2020-10-12	62.17	63.46	62.17	63.42	119737	{}	\N	2020-10-12 16:11:57.861638
71292	SPEM	2020-09-28	36.13	36.22	35.91	36.05	593337	{}	\N	2020-10-12 16:11:58.795605
71293	SPEM	2020-09-29	35.92	36.12	35.92	35.99	430239	{}	\N	2020-10-12 16:11:58.802991
71294	SPEM	2020-09-30	36.24	36.65	36.24	36.56	1147088	{}	\N	2020-10-12 16:11:58.808258
71295	SPEM	2020-10-01	36.83	36.92	36.65	36.9	1471922	{}	\N	2020-10-12 16:11:58.813535
71296	SPEM	2020-10-02	36.36	36.7669	36.36	36.44	978638	{}	\N	2020-10-12 16:11:58.818572
71297	SPEM	2020-10-05	36.63	36.93	36.63	36.93	727975	{}	\N	2020-10-12 16:11:58.823048
71298	SPEM	2020-10-06	37.06	37.2972	36.92	37	929663	{}	\N	2020-10-12 16:11:58.82806
71299	SPEM	2020-10-07	37.3	37.4488	37.215	37.32	2424483	{}	\N	2020-10-12 16:11:58.832499
71300	SPEM	2020-10-08	37.46	37.6599	37.39	37.6	1875889	{}	\N	2020-10-12 16:11:58.837138
71301	SPEM	2020-10-09	37.71	37.975	37.64	37.9	834116	{}	\N	2020-10-12 16:11:58.841788
71302	SPEM	2020-10-12	38.1	38.26	38.05	38.19	265758	{}	\N	2020-10-12 16:11:58.846694
71303	SPKE	2020-09-28	8.4	8.49	8.32	8.4	93173	{}	\N	2020-10-12 16:12:00.428105
71304	SPKE	2020-09-29	8.43	8.49	8.32	8.38	66689	{}	\N	2020-10-12 16:12:00.436199
71305	SPKE	2020-09-30	8.43	8.53	8.21	8.32	66151	{}	\N	2020-10-12 16:12:00.443262
71306	SPKE	2020-10-01	8.36	8.5	8.29	8.47	69705	{}	\N	2020-10-12 16:12:00.449358
71307	SPKE	2020-10-02	8.32	8.55	8.2	8.5	54856	{}	\N	2020-10-12 16:12:00.454427
71308	SPKE	2020-10-05	8.59	8.86	8.51	8.8	65574	{}	\N	2020-10-12 16:12:00.458406
71309	SPKE	2020-10-06	8.85	9.05	8.67	8.75	85738	{}	\N	2020-10-12 16:12:00.462933
71310	SPKE	2020-10-07	8.78	8.92	8.68	8.84	114695	{}	\N	2020-10-12 16:12:00.467329
71311	SPKE	2020-10-08	8.87	9	8.82	8.83	99603	{}	\N	2020-10-12 16:12:00.47139
71312	SPKE	2020-10-09	8.88	8.98	8.84	8.93	70666	{}	\N	2020-10-12 16:12:00.475847
71313	SPKE	2020-10-12	8.9	9.19	8.9	9.17	33142	{}	\N	2020-10-12 16:12:00.480631
71314	SPLV	2020-09-28	53.3	53.74	53.26	53.5	2840370	{}	\N	2020-10-12 16:12:01.739485
71315	SPLV	2020-09-29	53.39	53.69	53.22	53.29	1444285	{}	\N	2020-10-12 16:12:01.750864
71316	SPLV	2020-09-30	53.48	54.02	53.305	53.64	2773402	{}	\N	2020-10-12 16:12:01.759056
71317	SPLV	2020-10-01	53.84	54.0001	53.51	53.78	1998842	{}	\N	2020-10-12 16:12:01.765451
71318	SPLV	2020-10-02	53.16	53.78	53.151	53.53	2955940	{}	\N	2020-10-12 16:12:01.771327
71319	SPLV	2020-10-05	53.71	54.24	53.6669	54.21	1620822	{}	\N	2020-10-12 16:12:01.776516
71320	SPLV	2020-10-06	54.24	54.54	53.69	53.79	5310745	{}	\N	2020-10-12 16:12:01.78078
71321	SPLV	2020-10-07	54.07	54.5901	54.05	54.44	1885759	{}	\N	2020-10-12 16:12:01.784766
71322	SPLV	2020-10-08	54.61	54.81	54.57	54.81	2244335	{}	\N	2020-10-12 16:12:01.789603
71323	SPLV	2020-10-09	54.99	55.38	54.92	55.25	1393774	{}	\N	2020-10-12 16:12:01.794839
71324	SPLV	2020-10-12	55.5	55.89	55.42	55.84	555080	{}	\N	2020-10-12 16:12:01.799226
71325	SPOK	2020-09-28	9.41	9.51	9.34	9.37	56218	{}	\N	2020-10-12 16:12:02.664067
71326	SPOK	2020-09-29	9.32	9.36	9.19	9.24	197331	{}	\N	2020-10-12 16:12:02.673915
71327	SPOK	2020-09-30	9.23	9.77	9.23	9.51	98805	{}	\N	2020-10-12 16:12:02.68068
71328	SPOK	2020-10-01	9.39	9.63	9.31	9.44	165084	{}	\N	2020-10-12 16:12:02.687898
71329	SPOK	2020-10-02	9.32	9.59	9.21	9.43	72914	{}	\N	2020-10-12 16:12:02.693813
71330	SPOK	2020-10-05	9.52	9.865	9.52	9.69	46612	{}	\N	2020-10-12 16:12:02.699634
71331	SPOK	2020-10-06	9.8	9.98	9.44	9.54	58386	{}	\N	2020-10-12 16:12:02.705862
71332	SPOK	2020-10-07	9.51	9.57	9.325	9.49	68208	{}	\N	2020-10-12 16:12:02.714051
71333	SPOK	2020-10-08	9.48	9.7006	9.48	9.53	34710	{}	\N	2020-10-12 16:12:02.71981
71334	SPOK	2020-10-09	9.58	9.74	9.51	9.63	27779	{}	\N	2020-10-12 16:12:02.725768
71335	SPOK	2020-10-12	9.62	9.64	9.52	9.59	5984	{}	\N	2020-10-12 16:12:02.731801
71336	SPSC	2020-09-28	76.73	78.47	76.69	78.29	208483	{}	\N	2020-10-12 16:12:04.280981
71337	SPSC	2020-09-29	78.63	79.48	77.545	77.89	156011	{}	\N	2020-10-12 16:12:04.29705
71338	SPSC	2020-09-30	77.77	78.91	77.31	77.87	233111	{}	\N	2020-10-12 16:12:04.312511
71339	SPSC	2020-10-01	78.54	80.495	78.1	80.46	175411	{}	\N	2020-10-12 16:12:04.320277
71340	SPSC	2020-10-02	79.1	80.49	79.1	79.72	203111	{}	\N	2020-10-12 16:12:04.328292
71341	SPSC	2020-10-05	80.27	81.36	79.485	81.34	158530	{}	\N	2020-10-12 16:12:04.336628
71342	SPSC	2020-10-06	81.88	83.815	81.21	82.19	176409	{}	\N	2020-10-12 16:12:04.344378
71343	SPSC	2020-10-07	83.18	84.74	82.02	84.64	251917	{}	\N	2020-10-12 16:12:04.352525
71344	SPSC	2020-10-08	85.11	85.57	83.39	83.98	121062	{}	\N	2020-10-12 16:12:04.361836
71345	SPSC	2020-10-09	84.23	85.89	84.23	85.88	89747	{}	\N	2020-10-12 16:12:04.369986
71346	SPSC	2020-10-12	86.77	87.08	86.14	86.96	25061	{}	\N	2020-10-12 16:12:04.376247
71347	SPUU	2020-09-28	63.99	64.59	63.8549	64.2593	4641	{}	\N	2020-10-12 16:12:05.293616
71348	SPUU	2020-09-29	64.24	64.24	63.6263	63.8546	2759	{}	\N	2020-10-12 16:12:05.30415
71349	SPUU	2020-09-30	64.4199	65.91	64.4199	65.027	6159	{}	\N	2020-10-12 16:12:05.312785
71350	SPUU	2020-10-01	65.66	66.0203	64.79	65.3921	6517	{}	\N	2020-10-12 16:12:05.322399
71351	SPUU	2020-10-02	63.31	64.975	62.9503	64.3849	8352	{}	\N	2020-10-12 16:12:05.331412
71352	SPUU	2020-10-05	65.335	66.49	65.335	66.4762	2014	{}	\N	2020-10-12 16:12:05.338867
71353	SPUU	2020-10-06	66.7499	67.33	64.7655	64.7655	8558	{}	\N	2020-10-12 16:12:05.344556
71354	SPUU	2020-10-07	66.17	67.05	66.16	67.025	11211	{}	\N	2020-10-12 16:12:05.350299
71355	SPUU	2020-10-08	67.69	67.9908	67.485	67.9908	20851	{}	\N	2020-10-12 16:12:05.355665
71356	SPUU	2020-10-09	69.325	69.325	69.05	69.124	3190	{}	\N	2020-10-12 16:12:05.361251
71357	SPUU	2020-10-12	70.3	71.3	70.22	71.22	3483	{}	\N	2020-10-12 16:12:05.367697
71358	SPXE	2020-09-28	72.39	73.04	72.39	73.04	2770	{}	\N	2020-10-12 16:12:06.295902
71359	SPXE	2020-09-29	72.8582	72.8582	72.7026	72.7026	124	{}	\N	2020-10-12 16:12:06.305931
71360	SPXE	2020-09-30	73.4206	73.4206	73.4206	73.4206	294	{}	\N	2020-10-12 16:12:06.311473
71361	SPXE	2020-10-01	73.71	73.89	73.51	73.51	1239	{}	\N	2020-10-12 16:12:06.317205
71362	SPXE	2020-10-02	72.9602	73.34	72.9602	73.34	516	{}	\N	2020-10-12 16:12:06.323328
71363	SPXE	2020-10-05	73.795	74.2588	73.795	74.2588	343	{}	\N	2020-10-12 16:12:06.329817
71364	SPXE	2020-10-06	74.3533	74.3533	73.3176	73.3176	167	{}	\N	2020-10-12 16:12:06.335291
71365	SPXE	2020-10-07	74.21	74.5825	74.21	74.5825	1354	{}	\N	2020-10-12 16:12:06.340391
71366	SPXE	2020-10-09	75.99	75.99	75.5799	75.7638	2444	{}	\N	2020-10-12 16:12:06.346186
71367	SPXE	2020-10-12	76.54	76.54	76.54	76.54	104	{}	\N	2020-10-12 16:12:06.351949
71368	SPXX	2020-09-28	13.22	13.29	13.2	13.25	90206	{}	\N	2020-10-12 16:12:07.505705
71369	SPXX	2020-09-29	13.29	13.29	13.1744	13.26	79475	{}	\N	2020-10-12 16:12:07.512941
71370	SPXX	2020-09-30	13.29	13.38	13.27	13.32	115191	{}	\N	2020-10-12 16:12:07.519701
71371	SPXX	2020-10-01	13.48	13.5	13.37	13.44	173442	{}	\N	2020-10-12 16:12:07.527129
71372	SPXX	2020-10-02	13.32	13.35	13.26	13.34	85445	{}	\N	2020-10-12 16:12:07.536709
71373	SPXX	2020-10-05	13.4	13.5	13.38	13.45	127928	{}	\N	2020-10-12 16:12:07.547501
71374	SPXX	2020-10-06	13.47	13.61	13.33	13.36	125808	{}	\N	2020-10-12 16:12:07.556815
71375	SPXX	2020-10-07	13.37	13.55	13.37	13.47	220146	{}	\N	2020-10-12 16:12:07.565438
71376	SPXX	2020-10-08	13.55	13.5558	13.48	13.51	51194	{}	\N	2020-10-12 16:12:07.572866
71377	SPXX	2020-10-09	13.55	13.6	13.53	13.57	78719	{}	\N	2020-10-12 16:12:07.578749
71378	SPXX	2020-10-12	13.65	13.76	13.61	13.7512	103650	{}	\N	2020-10-12 16:12:07.584932
71379	SOYB	2020-09-28	15.53	15.53	15.4	15.45	140381	{}	\N	2020-10-12 16:12:08.927684
71380	SOYB	2020-09-29	15.45	15.55	15.35	15.42	234424	{}	\N	2020-10-12 16:12:08.936391
71381	SOYB	2020-09-30	15.44	15.9941	15.38	15.84	432741	{}	\N	2020-10-12 16:12:08.943113
71382	SOYB	2020-10-01	15.9101	15.95	15.85	15.9	128043	{}	\N	2020-10-12 16:12:08.94934
71383	SOYB	2020-10-02	15.72	15.92	15.7	15.83	76506	{}	\N	2020-10-12 16:12:08.954489
71384	SOYB	2020-10-05	15.88	15.9	15.8076	15.87	300326	{}	\N	2020-10-12 16:12:08.959542
71385	SOYB	2020-10-06	15.97	16.18	15.97	16.02	491409	{}	\N	2020-10-12 16:12:08.964087
71386	SOYB	2020-10-07	16.18	16.2	16	16.01	159013	{}	\N	2020-10-12 16:12:08.968405
71387	SOYB	2020-10-08	16.14	16.165	15.91	15.96	205842	{}	\N	2020-10-12 16:12:08.973402
71388	SOYB	2020-10-09	16.09	16.3333	16.0201	16.27	247248	{}	\N	2020-10-12 16:12:08.978076
71389	SOYB	2020-10-12	16.2	16.2	15.94	15.97	748524	{}	\N	2020-10-12 16:12:08.982383
71390	SPCB	2020-09-28	0.7394	0.7837	0.7101	0.768	123278	{}	\N	2020-10-12 16:12:09.86664
71391	SPCB	2020-09-29	0.7669	0.7814	0.7201	0.766	119023	{}	\N	2020-10-12 16:12:09.878333
71392	SPCB	2020-09-30	0.74	0.767899	0.74	0.7612	58007	{}	\N	2020-10-12 16:12:09.885334
71393	SPCB	2020-10-01	0.755	0.78	0.75	0.77	57673	{}	\N	2020-10-12 16:12:09.891689
71394	SPCB	2020-10-02	0.77	0.85	0.7555	0.82	336548	{}	\N	2020-10-12 16:12:09.897104
71395	SPCB	2020-10-05	0.8295	0.8295	0.7708	0.7994	129400	{}	\N	2020-10-12 16:12:09.902127
71396	SPCB	2020-10-06	0.8	0.8079	0.7601	0.763	154200	{}	\N	2020-10-12 16:12:09.907362
71397	SPCB	2020-10-07	0.7698	0.819699	0.7601	0.8044	190486	{}	\N	2020-10-12 16:12:09.912431
71398	SPCB	2020-10-08	0.7647	0.8893	0.7647	0.86	529219	{}	\N	2020-10-12 16:12:09.918349
71399	SPCB	2020-10-09	0.9198	1.15	0.87	0.9039	5457507	{}	\N	2020-10-12 16:12:09.923067
71400	SPCB	2020-10-12	0.871	0.88	0.8125	0.83	366980	{}	\N	2020-10-12 16:12:09.927968
71401	SPFF	2020-09-28	11.14	11.15	11.1	11.12	47714	{}	\N	2020-10-12 16:12:11.202515
71402	SPFF	2020-09-29	11.13	11.1393	11.08	11.125	66394	{}	\N	2020-10-12 16:12:11.215621
71403	SPFF	2020-09-30	11.09	11.15	11.08	11.0999	114491	{}	\N	2020-10-12 16:12:11.22809
71404	SPFF	2020-10-01	11.08	11.16	11.08	11.16	51373	{}	\N	2020-10-12 16:12:11.235918
71405	SPFF	2020-10-02	11.1	11.18	11.1	11.16	42756	{}	\N	2020-10-12 16:12:11.241934
71406	SPFF	2020-10-05	11.13	11.185	11.13	11.14	25257	{}	\N	2020-10-12 16:12:11.248777
71407	SPFF	2020-10-06	11.15	11.23	11.13	11.15	230387	{}	\N	2020-10-12 16:12:11.254627
71408	SPFF	2020-10-07	11.16	11.25	11.16	11.23	58035	{}	\N	2020-10-12 16:12:11.259592
71409	SPFF	2020-10-08	11.22	11.31	11.2001	11.3	60815	{}	\N	2020-10-12 16:12:11.266012
71410	SPFF	2020-10-09	11.28	11.32	11.27	11.3	51453	{}	\N	2020-10-12 16:12:11.271062
71411	SPFF	2020-10-12	11.33	11.33	11.2884	11.3165	55622	{}	\N	2020-10-12 16:12:11.277263
71412	SPKEP	2020-09-28	22.28	22.3314	21.8656	22.3	15795	{}	\N	2020-10-12 16:12:12.562102
71413	SPKEP	2020-09-29	22.375	22.5	21.9	22.125	12252	{}	\N	2020-10-12 16:12:12.572219
71414	SPKEP	2020-09-30	22	22.34	21.7	22.0476	3870	{}	\N	2020-10-12 16:12:12.579008
71415	SPKEP	2020-10-01	21.99	22.02	21.8	22	10509	{}	\N	2020-10-12 16:12:12.586202
71416	SPKEP	2020-10-02	21.91	21.96	21.8	21.82	7528	{}	\N	2020-10-12 16:12:12.592525
71417	SPKEP	2020-10-05	21.986	22.3	21.8201	21.831	7898	{}	\N	2020-10-12 16:12:12.598127
71418	SPKEP	2020-10-06	21.8	21.8	21.55	21.8	7483	{}	\N	2020-10-12 16:12:12.603023
71419	SPKEP	2020-10-07	21.8	21.98	21.05	21.89	5162	{}	\N	2020-10-12 16:12:12.607448
71420	SPKEP	2020-10-08	21.99	22.15	21.99	22.15	4095	{}	\N	2020-10-12 16:12:12.612496
71421	SPKEP	2020-10-09	22.2098	22.3	22	22.2872	7311	{}	\N	2020-10-12 16:12:12.617912
71422	SPKEP	2020-10-12	22.3	22.4	22.3	22.35	1672	{}	\N	2020-10-12 16:12:12.622698
71423	SPMD	2020-09-28	32.27	32.6843	32.27	32.62	499340	{}	\N	2020-10-12 16:12:13.757045
71424	SPMD	2020-09-29	32.61	32.71	32.215	32.42	534708	{}	\N	2020-10-12 16:12:13.772326
71425	SPMD	2020-09-30	32.51	32.955	32.39	32.6	781162	{}	\N	2020-10-12 16:12:13.784785
71426	SPMD	2020-10-01	32.81	33.07	32.66	33.07	654282	{}	\N	2020-10-12 16:12:13.791312
71427	SPMD	2020-10-02	32.5	33.51	32.43	33.37	1041044	{}	\N	2020-10-12 16:12:13.796517
71428	SPMD	2020-10-05	33.69	34.14	33.69	34.12	439129	{}	\N	2020-10-12 16:12:13.802046
71429	SPMD	2020-10-06	34.3	34.77	33.89	33.96	969264	{}	\N	2020-10-12 16:12:13.807615
71430	SPMD	2020-10-07	34.36	34.67	34.34	34.58	864765	{}	\N	2020-10-12 16:12:13.813228
71431	SPMD	2020-10-08	34.87	34.94	34.66	34.91	497221	{}	\N	2020-10-12 16:12:13.818874
71432	SPMD	2020-10-09	35.18	35.28	34.91	35.03	631128	{}	\N	2020-10-12 16:12:13.82519
71433	SPMD	2020-10-12	35.19	35.2899	35.03	35.2199	142654	{}	\N	2020-10-12 16:12:13.832766
71434	SPPI	2020-09-28	4.34	4.375	4.19	4.24	2069464	{}	\N	2020-10-12 16:12:14.824501
71435	SPPI	2020-09-29	4.22	4.33	4.13	4.18	1992139	{}	\N	2020-10-12 16:12:14.836629
71436	SPPI	2020-09-30	4.17	4.24	4.01	4.08	2331562	{}	\N	2020-10-12 16:12:14.845199
71437	SPPI	2020-10-01	4.15	4.3	4.02	4.17	2140255	{}	\N	2020-10-12 16:12:14.852139
71438	SPPI	2020-10-02	4.04	4.16	3.91	3.94	2754864	{}	\N	2020-10-12 16:12:14.859367
71439	SPPI	2020-10-05	4.0308	4.32	4.02	4.19	2817322	{}	\N	2020-10-12 16:12:14.865263
71440	SPPI	2020-10-06	4.23	4.33	4.06	4.1	3069643	{}	\N	2020-10-12 16:12:14.872011
71441	SPPI	2020-10-07	4.11	4.33	4.11	4.27	1773394	{}	\N	2020-10-12 16:12:14.87936
71442	SPPI	2020-10-08	4.42	4.48	4.31	4.46	2571222	{}	\N	2020-10-12 16:12:14.887072
71443	SPPI	2020-10-09	4.57	4.58	4.38	4.51	2068469	{}	\N	2020-10-12 16:12:14.893529
71444	SPPI	2020-10-12	4.4035	4.535	4.37	4.4082	903847	{}	\N	2020-10-12 16:12:14.899254
71445	SPRT	2020-09-28	1.82	1.83	1.77	1.77	47135	{}	\N	2020-10-12 16:12:15.853499
71446	SPRT	2020-09-29	1.77	1.82	1.75	1.78	49237	{}	\N	2020-10-12 16:12:15.86428
71447	SPRT	2020-09-30	1.78	1.82	1.76	1.8	24287	{}	\N	2020-10-12 16:12:15.873099
71448	SPRT	2020-10-01	1.78	1.85	1.71	1.83	178794	{}	\N	2020-10-12 16:12:15.88091
71449	SPRT	2020-10-02	1.81	1.84	1.7401	1.83	34060	{}	\N	2020-10-12 16:12:15.886771
71450	SPRT	2020-10-05	1.83	1.85	1.75	1.83	87418	{}	\N	2020-10-12 16:12:15.892894
71451	SPRT	2020-10-06	1.83	1.84	1.77	1.82	107234	{}	\N	2020-10-12 16:12:15.898191
71452	SPRT	2020-10-07	1.795	1.82	1.67	1.74	177362	{}	\N	2020-10-12 16:12:15.903674
71453	SPRT	2020-10-08	1.76	1.84	1.76	1.79	59177	{}	\N	2020-10-12 16:12:15.90965
71454	SPRT	2020-10-09	1.795	1.8117	1.704	1.74	142541	{}	\N	2020-10-12 16:12:15.914948
71455	SPRT	2020-10-12	1.73	1.8	1.73	1.7667	7745	{}	\N	2020-10-12 16:12:15.922465
71456	SPTN	2020-09-28	16.14	16.4	15.75	16.25	644034	{}	\N	2020-10-12 16:12:16.849303
71457	SPTN	2020-09-29	16.2	16.53	15.75	16.395	535818	{}	\N	2020-10-12 16:12:16.861007
71458	SPTN	2020-09-30	16.36	16.5682	15.895	16.35	570686	{}	\N	2020-10-12 16:12:16.869546
71459	SPTN	2020-10-01	16.44	16.79	16.3038	16.72	279464	{}	\N	2020-10-12 16:12:16.876341
71460	SPTN	2020-10-02	16.51	16.62	16.143	16.45	301757	{}	\N	2020-10-12 16:12:16.882791
71461	SPTN	2020-10-05	16.62	16.87	16.57	16.69	280958	{}	\N	2020-10-12 16:12:16.888303
71462	SPTN	2020-10-06	16.8	17.22	16.735	16.84	366819	{}	\N	2020-10-12 16:12:16.894253
71463	SPTN	2020-10-07	16.85	17.01	16.585	16.84	328712	{}	\N	2020-10-12 16:12:16.900271
71464	SPTN	2020-10-08	16.99	17.45	16.91	17.02	283137	{}	\N	2020-10-12 16:12:16.905619
71465	SPTN	2020-10-09	19.2	23.45	19.11	21.49	6201499	{}	\N	2020-10-12 16:12:16.911636
71466	SPTN	2020-10-12	21.65	21.7554	19.715	19.91	843936	{}	\N	2020-10-12 16:12:16.917108
71467	SPWR	2020-09-28	10.91	12.34	10.91	11.7	13281236	{}	\N	2020-10-12 16:12:18.090632
71468	SPWR	2020-09-29	11.7404	12.91	11.51	11.96	14440218	{}	\N	2020-10-12 16:12:18.101663
71469	SPWR	2020-09-30	12.0973	13.39	12.0973	12.51	11275202	{}	\N	2020-10-12 16:12:18.109241
71470	SPWR	2020-10-01	12.85	13.54	12.4	13.36	9448933	{}	\N	2020-10-12 16:12:18.116418
71471	SPWR	2020-10-02	12.87	14.06	12.8	13.56	7140497	{}	\N	2020-10-12 16:12:18.122965
71472	SPWR	2020-10-05	13.97	15.94	13.92	15.56	14194024	{}	\N	2020-10-12 16:12:18.12924
71473	SPWR	2020-10-06	15.835	16.4	14.6801	15.61	14724195	{}	\N	2020-10-12 16:12:18.139354
71474	SPWR	2020-10-07	16.18	18.17	16.17	17.33	18627136	{}	\N	2020-10-12 16:12:18.14772
71475	SPWR	2020-10-08	18.1017	18.25	16.655	16.89	13143852	{}	\N	2020-10-12 16:12:18.153526
71476	SPWR	2020-10-09	16.91	17.95	16.53	17.43	9079237	{}	\N	2020-10-12 16:12:18.158515
71477	SPWR	2020-10-12	17.66	17.68	15.5	16.45	10159974	{}	\N	2020-10-12 16:12:18.163963
71478	SPXV	2020-09-29	69.577	69.577	69.577	69.577	8	{}	\N	2020-10-12 16:12:20.239727
71479	SPXV	2020-09-30	70.1524	70.1524	70.1524	70.1524	9	{}	\N	2020-10-12 16:12:20.248215
71480	SPXV	2020-10-01	70.3582	70.4392	70.3582	70.4392	127	{}	\N	2020-10-12 16:12:20.256406
71481	SPXV	2020-10-02	69.8925	69.8925	69.8925	69.8925	5	{}	\N	2020-10-12 16:12:20.265601
71482	SPXV	2020-10-05	70.81	71.0082	70.81	71.0082	103	{}	\N	2020-10-12 16:12:20.274657
71483	SPXV	2020-10-06	70.1017	70.1017	70.1017	70.1017	84	{}	\N	2020-10-12 16:12:20.280618
71484	SPXV	2020-10-07	71.2804	71.2804	71.2804	71.2804	12	{}	\N	2020-10-12 16:12:20.287076
71485	SPXV	2020-10-08	71.8579	71.8579	71.8579	71.8579	0	{}	\N	2020-10-12 16:12:20.292029
71486	SPXV	2020-10-09	72.4518	72.4518	72.4518	72.4518	5	{}	\N	2020-10-12 16:12:20.296753
71487	SQLV	2020-10-01	22.88	22.9087	22.88	22.9087	4491	{}	\N	2020-10-12 16:12:22.125637
71488	SQLV	2020-10-05	23.79	23.8899	23.79	23.8899	9379	{}	\N	2020-10-12 16:12:22.137425
71489	SQLV	2020-10-06	24.43	24.43	23.8737	23.8737	543	{}	\N	2020-10-12 16:12:22.146101
71490	SQLV	2020-10-08	24.46	24.79	24.45	24.79	1318	{}	\N	2020-10-12 16:12:22.156055
71491	SQLV	2020-10-09	24.96	24.96	24.68	24.86	2389	{}	\N	2020-10-12 16:12:22.166215
71492	SQLV	2020-10-12	24.78	24.78	24.78	24.78	100	{}	\N	2020-10-12 16:12:22.172575
71493	SPDW	2020-09-28	29.35	29.41	29.3	29.41	966597	{}	\N	2020-10-12 16:12:23.505637
71494	SPDW	2020-09-29	29.4	29.47	29.25	29.365	1806576	{}	\N	2020-10-12 16:12:23.517996
71495	SPDW	2020-09-30	29.29	29.46	29.1842	29.26	1694788	{}	\N	2020-10-12 16:12:23.529214
71496	SPDW	2020-10-01	29.43	29.47	29.29	29.44	1369081	{}	\N	2020-10-12 16:12:23.537659
71497	SPDW	2020-10-02	29.06	29.44	29.06	29.37	971913	{}	\N	2020-10-12 16:12:23.544088
71498	SPDW	2020-10-05	29.62	29.84	29.62	29.83	642515	{}	\N	2020-10-12 16:12:23.549504
71499	SPDW	2020-10-06	29.85	29.9	29.48	29.55	1570421	{}	\N	2020-10-12 16:12:23.554625
71500	SPDW	2020-10-07	29.76	29.87	29.71	29.84	2987418	{}	\N	2020-10-12 16:12:23.559822
71501	SPDW	2020-10-08	29.97	30.06	29.93	30.05	1089875	{}	\N	2020-10-12 16:12:23.565615
71502	SPDW	2020-10-09	30.18	30.27	30.145	30.24	890680	{}	\N	2020-10-12 16:12:23.570857
71503	SPDW	2020-10-12	30.305	30.3833	30.29	30.3573	193935	{}	\N	2020-10-12 16:12:23.575989
71504	SPH	2020-09-28	14.9962	15.165	14.62	15	388352	{}	\N	2020-10-12 16:12:25.419464
71505	SPH	2020-09-29	14.98	15.8253	14.9	15.74	1174847	{}	\N	2020-10-12 16:12:25.433346
71506	SPH	2020-09-30	15.63	16.32	15.63	16.28	1118802	{}	\N	2020-10-12 16:12:25.44167
71507	SPH	2020-10-01	16.28	16.33	15.7219	16.11	613624	{}	\N	2020-10-12 16:12:25.448251
71508	SPH	2020-10-02	15.69	16.395	15.56	16.23	600275	{}	\N	2020-10-12 16:12:25.454962
71509	SPH	2020-10-05	16.44	16.44	15.9	16.4	724646	{}	\N	2020-10-12 16:12:25.459999
71510	SPH	2020-10-06	16.33	17.22	16.284	16.91	1175357	{}	\N	2020-10-12 16:12:25.466228
71511	SPH	2020-10-07	17.04	17.93	17.03	17.58	1307238	{}	\N	2020-10-12 16:12:25.472497
71512	SPH	2020-10-08	17.68	18.04	17.44	17.85	782315	{}	\N	2020-10-12 16:12:25.478733
71513	SPH	2020-10-09	17.99	18.07	16.83	17.11	960884	{}	\N	2020-10-12 16:12:25.48519
71514	SPH	2020-10-12	16.95	17.04	16.67	16.83	303160	{}	\N	2020-10-12 16:12:25.490739
71515	SPI	2020-09-28	9.9	12	9.05	10.24	11037544	{}	\N	2020-10-12 16:12:27.484479
71516	SPI	2020-09-29	9.22	9.5	7.2	7.46	3757193	{}	\N	2020-10-12 16:12:27.495746
71517	SPI	2020-09-30	8	9.75	7.2	7.2	14112011	{}	\N	2020-10-12 16:12:27.504441
71518	SPI	2020-10-01	6.2	6.22	5.35	5.71	5268439	{}	\N	2020-10-12 16:12:27.511011
71519	SPI	2020-10-02	5.55	6.38	5.5	5.68	8825838	{}	\N	2020-10-12 16:12:27.51612
71520	SPI	2020-10-05	5.98	7.54	5.86	6.86	17902243	{}	\N	2020-10-12 16:12:27.521226
71521	SPI	2020-10-06	7	8.67	6.87	7.77	15724764	{}	\N	2020-10-12 16:12:27.527152
71522	SPI	2020-10-07	8.9	9.09	8	8.35	8074207	{}	\N	2020-10-12 16:12:27.532953
71523	SPI	2020-10-08	8.45	8.7	7.58	8.1	3764577	{}	\N	2020-10-12 16:12:27.53894
71524	SPI	2020-10-09	7.86	7.89	7.26	7.41	2516210	{}	\N	2020-10-12 16:12:27.544452
71525	SPI	2020-10-12	7.76	7.79	7.2125	7.24	1076847	{}	\N	2020-10-12 16:12:27.550019
71526	SPLK	2020-09-28	187.9	188.28	184.555	187.9	1215278	{}	\N	2020-10-12 16:12:28.783607
71527	SPLK	2020-09-29	188.2	189.7	185.45	187.18	1121199	{}	\N	2020-10-12 16:12:28.795335
71528	SPLK	2020-09-30	187.45	190	184.68	188.13	1559335	{}	\N	2020-10-12 16:12:28.802855
71529	SPLK	2020-10-01	189.72	194.84	189.54	192.56	1540873	{}	\N	2020-10-12 16:12:28.810769
71530	SPLK	2020-10-02	192.5	197.19	190.4974	194.8	1925529	{}	\N	2020-10-12 16:12:28.818469
71531	SPLK	2020-10-05	195	196	192.989	194.07	1262316	{}	\N	2020-10-12 16:12:28.825458
71532	SPLK	2020-10-06	195.44	202.44	194.5	198.19	1540243	{}	\N	2020-10-12 16:12:28.833696
71533	SPLK	2020-10-07	199.52	206.47	199.02	206.08	1545835	{}	\N	2020-10-12 16:12:28.840676
71534	SPLK	2020-10-08	206.8	210.6115	206.3	209.05	2039751	{}	\N	2020-10-12 16:12:28.847192
71535	SPLK	2020-10-09	209.63	212.9	208.43	212.43	1176570	{}	\N	2020-10-12 16:12:28.85318
71536	SPLK	2020-10-12	214.55	217.36	212.5	214.995	516056	{}	\N	2020-10-12 16:12:28.85967
71537	SPMV	2020-09-28	31.4569	31.4569	31.4569	31.4569	251	{}	\N	2020-10-12 16:12:31.092522
71538	SPMV	2020-09-29	31.3948	31.3948	31.3948	31.3948	139	{}	\N	2020-10-12 16:12:31.10075
71539	SPMV	2020-09-30	31.7095	31.7095	31.7095	31.7095	143	{}	\N	2020-10-12 16:12:31.107729
71540	SPMV	2020-10-01	31.7238	31.7238	31.7238	31.7238	101	{}	\N	2020-10-12 16:12:31.112638
71541	SPMV	2020-10-02	31.75	31.75	31.6176	31.6176	764	{}	\N	2020-10-12 16:12:31.117396
71542	SPMV	2020-10-05	31.9123	31.997	31.9123	31.997	223	{}	\N	2020-10-12 16:12:31.122041
71543	SPMV	2020-10-06	32	32	31.7346	31.7346	332	{}	\N	2020-10-12 16:12:31.126285
71544	SPMV	2020-10-07	32.1776	32.1776	32.1776	32.1776	131	{}	\N	2020-10-12 16:12:31.130665
71545	SPMV	2020-10-08	32.3799	32.3799	32.3799	32.3799	293	{}	\N	2020-10-12 16:12:31.13775
71546	SPMV	2020-10-09	32.64	32.64	32.6125	32.6125	1305	{}	\N	2020-10-12 16:12:31.144401
71547	SPPP	2020-09-28	15.26	15.84	15.26	15.8399	31284	{}	\N	2020-10-12 16:12:32.136678
71548	SPPP	2020-09-29	16.01	16.09	15.77	15.94	42335	{}	\N	2020-10-12 16:12:32.144594
71549	SPPP	2020-09-30	16.13	16.13	15.82	15.8387	31309	{}	\N	2020-10-12 16:12:32.150467
71550	SPPP	2020-10-01	16.32	16.45	16.08	16.1	42119	{}	\N	2020-10-12 16:12:32.156421
71551	SPPP	2020-10-02	15.95	16.0999	15.55	15.65	50296	{}	\N	2020-10-12 16:12:32.161543
71552	SPPP	2020-10-05	15.8	16.1104	15.8	16.05	15949	{}	\N	2020-10-12 16:12:32.166501
71553	SPPP	2020-10-06	16.01	16.18	15.65	15.67	35234	{}	\N	2020-10-12 16:12:32.172183
71554	SPPP	2020-10-07	16.2	16.2	15.78	15.87	19939	{}	\N	2020-10-12 16:12:32.176702
71555	SPPP	2020-10-08	16.2	16.35	16.0195	16.05	25216	{}	\N	2020-10-12 16:12:32.181174
71556	SPPP	2020-10-09	16.44	16.68	16.38	16.56	52357	{}	\N	2020-10-12 16:12:32.186636
71557	SPPP	2020-10-12	16.7	16.74	16.39	16.44	11758	{}	\N	2020-10-12 16:12:32.19209
71558	SPSM	2020-09-28	27.07	27.56	27.07	27.46	269533	{}	\N	2020-10-12 16:12:33.201416
71559	SPSM	2020-09-29	27.37	27.4739	27	27.27	964517	{}	\N	2020-10-12 16:12:33.211461
71560	SPSM	2020-09-30	27.31	27.8199	27.1691	27.38	453592	{}	\N	2020-10-12 16:12:33.21762
71561	SPSM	2020-10-01	27.57	27.77	27.2825	27.77	503235	{}	\N	2020-10-12 16:12:33.224085
71562	SPSM	2020-10-02	27.25	28.18	27.24	28.05	1353313	{}	\N	2020-10-12 16:12:33.229378
71563	SPSM	2020-10-05	28.29	28.7101	28.29	28.66	341891	{}	\N	2020-10-12 16:12:33.233589
71564	SPSM	2020-10-06	29	29.43	28.63	28.63	445322	{}	\N	2020-10-12 16:12:33.238585
71565	SPSM	2020-10-07	28.99	29.21	28.86	29.08	1098486	{}	\N	2020-10-12 16:12:33.243683
71566	SPSM	2020-10-08	29.28	29.5405	29.17	29.5	347275	{}	\N	2020-10-12 16:12:33.247951
71567	SPSM	2020-10-09	29.8	29.8446	29.507	29.66	3394247	{}	\N	2020-10-12 16:12:33.253409
71568	SPSM	2020-10-12	29.78	29.8799	29.63	29.7782	89996	{}	\N	2020-10-12 16:12:33.258551
71569	SPVU	2020-09-28	27.33	27.46	27.2346	27.2346	1227	{}	\N	2020-10-12 16:12:34.548314
71570	SPVU	2020-09-29	26.76	26.96	26.75	26.96	1549	{}	\N	2020-10-12 16:12:34.554489
71571	SPVU	2020-09-30	26.97	27.48	26.97	27.25	8568	{}	\N	2020-10-12 16:12:34.560202
71572	SPVU	2020-10-01	27.0668	27.14	27.0478	27.05	3105	{}	\N	2020-10-12 16:12:34.565584
71573	SPVU	2020-10-02	27.1723	27.53	27.1723	27.5	12930	{}	\N	2020-10-12 16:12:34.571662
71574	SPVU	2020-10-05	27.86	28.0496	27.77	27.99	7759	{}	\N	2020-10-12 16:12:34.577406
71575	SPVU	2020-10-06	28.22	28.5	27.67	27.7253	36905	{}	\N	2020-10-12 16:12:34.582699
71576	SPVU	2020-10-07	28.2673	28.36	28.1	28.3481	3241	{}	\N	2020-10-12 16:12:34.58838
71577	SPVU	2020-10-08	28.46	28.7999	28.46	28.7999	5021	{}	\N	2020-10-12 16:12:34.594558
71578	SPVU	2020-10-09	28.68	28.855	28.6104	28.6104	12898	{}	\N	2020-10-12 16:12:34.600127
71579	SPVU	2020-10-12	28.65	28.905	28.65	28.905	1956	{}	\N	2020-10-12 16:12:34.605845
71580	SPXL	2020-09-28	51.89	52.68	51.41	52.36	5811408	{}	\N	2020-10-12 16:12:36.481638
71581	SPXL	2020-09-29	52.2405	52.6	51.13	51.46	5890033	{}	\N	2020-10-12 16:12:36.49382
71582	SPXL	2020-09-30	51.85	54.2255	51.69	52.66	7975916	{}	\N	2020-10-12 16:12:36.501568
71583	SPXL	2020-10-01	53.9499	54.44	52.7	53.62	6666678	{}	\N	2020-10-12 16:12:36.510267
71584	SPXL	2020-10-02	51.11	53.11	50.85	52.18	9362579	{}	\N	2020-10-12 16:12:36.521179
71585	SPXL	2020-10-05	53.16	54.965	53.16	54.85	4681286	{}	\N	2020-10-12 16:12:36.529322
71586	SPXL	2020-10-06	54.98	56.04	52.28	52.56	8952890	{}	\N	2020-10-12 16:12:36.538439
71587	SPXL	2020-10-07	54.04	55.69	54.0299	55.23	5483383	{}	\N	2020-10-12 16:12:36.545175
71588	SPXL	2020-10-08	56.29	56.76	55.8294	56.69	4909299	{}	\N	2020-10-12 16:12:36.551301
71589	SPXL	2020-10-09	57.64	58.5	57.29	58.22	5088630	{}	\N	2020-10-12 16:12:36.556877
71590	SPXL	2020-10-12	59.65	60.9001	59.3771	60.78	2501154	{}	\N	2020-10-12 16:12:36.562644
71591	SPXU	2020-09-28	8.86	8.95	8.71	8.76	20376068	{}	\N	2020-10-12 16:12:37.511693
71592	SPXU	2020-09-29	8.7801	8.98	8.725	8.9	15126585	{}	\N	2020-10-12 16:12:37.523496
71593	SPXU	2020-09-30	8.85	8.8715	8.44	8.7	32206776	{}	\N	2020-10-12 16:12:37.531557
71594	SPXU	2020-10-01	8.48	8.7	8.4113	8.55	26529809	{}	\N	2020-10-12 16:12:37.537437
71595	SPXU	2020-10-02	8.95	8.99	8.63	8.79	41987401	{}	\N	2020-10-12 16:12:37.542423
71596	SPXU	2020-10-05	8.61	8.61	8.3	8.32	16392156	{}	\N	2020-10-12 16:12:37.547046
71597	SPXU	2020-10-06	8.32	8.71	8.14	8.67	34503760	{}	\N	2020-10-12 16:12:37.552
71598	SPXU	2020-10-07	8.43	8.43	8.15	8.23	18746466	{}	\N	2020-10-12 16:12:37.557102
71599	SPXU	2020-10-08	8.08	8.14	8	8.01	16807441	{}	\N	2020-10-12 16:12:37.561815
71600	SPXU	2020-10-09	7.88	7.93	7.75	7.78	16261064	{}	\N	2020-10-12 16:12:37.566396
71601	SPXU	2020-10-12	7.6	7.64	7.43	7.448	11207832	{}	\N	2020-10-12 16:12:37.571109
71602	SP	2020-09-28	18.6	18.94	18.56	18.75	94506	{}	\N	2020-10-12 16:12:38.482406
71603	SP	2020-09-29	18.57	18.61	17.47	17.98	105277	{}	\N	2020-10-12 16:12:38.493519
71604	SP	2020-09-30	17.46	18.2925	17.46	17.95	153104	{}	\N	2020-10-12 16:12:38.502091
71605	SP	2020-10-01	17.91	18.25	17.365	18.21	131825	{}	\N	2020-10-12 16:12:38.509041
71606	SP	2020-10-02	17.82	19.28	17.82	19.26	110894	{}	\N	2020-10-12 16:12:38.515354
71607	SP	2020-10-05	19.5	20.03	19.19	19.25	114668	{}	\N	2020-10-12 16:12:38.521729
71608	SP	2020-10-06	19.5	19.5	18.69	18.73	77349	{}	\N	2020-10-12 16:12:38.527926
71609	SP	2020-10-07	18.87	19.31	18.78	19	95249	{}	\N	2020-10-12 16:12:38.534017
71610	SP	2020-10-08	19.24	19.66	19.02	19.33	64838	{}	\N	2020-10-12 16:12:38.541218
71611	SP	2020-10-09	19.5	19.74	18.77	18.87	69703	{}	\N	2020-10-12 16:12:38.546871
71612	SP	2020-10-12	18.85	19.0628	18.45	18.75	9070	{}	\N	2020-10-12 16:12:38.552986
71613	SPDN	2020-09-28	20.94	20.97	20.82	20.85	111510	{}	\N	2020-10-12 16:12:39.911484
71614	SPDN	2020-09-29	20.88	21.01	20.83	20.96	49602	{}	\N	2020-10-12 16:12:39.92124
71615	SPDN	2020-09-30	20.91	20.92	20.6	20.81	170796	{}	\N	2020-10-12 16:12:39.92841
71616	SPDN	2020-10-01	20.64	20.8	20.585	20.69	85656	{}	\N	2020-10-12 16:12:39.935239
71617	SPDN	2020-10-02	21.02	21.03	20.7556	20.88	284272	{}	\N	2020-10-12 16:12:39.944941
71618	SPDN	2020-10-05	20.72	20.73	20.5	20.5	76028	{}	\N	2020-10-12 16:12:39.950393
71619	SPDN	2020-10-06	20.48	20.83	20.36	20.76	202249	{}	\N	2020-10-12 16:12:39.95653
71620	SPDN	2020-10-07	20.551	20.575	20.38	20.43	117262	{}	\N	2020-10-12 16:12:39.96176
71621	SPDN	2020-10-08	20.29	20.34	20.25	20.25	77166	{}	\N	2020-10-12 16:12:39.967609
71622	SPDN	2020-10-09	20.16	20.1882	20.05	20.08	74261	{}	\N	2020-10-12 16:12:39.973481
71623	SPDN	2020-10-12	19.93	19.94	19.76	19.7875	56782	{}	\N	2020-10-12 16:12:39.979609
71624	SPG	2020-09-28	65.8	66.535	64.8	65.79	3862032	{}	\N	2020-10-12 16:12:41.085091
71625	SPG	2020-09-29	65.33	65.34	62.9	63.99	7482517	{}	\N	2020-10-12 16:12:41.095476
71626	SPG	2020-09-30	64.15	66.12	63.7	64.68	8103083	{}	\N	2020-10-12 16:12:41.103799
71627	SPG	2020-10-01	64.84	66.56	64.25	66.53	6649186	{}	\N	2020-10-12 16:12:41.110531
71628	SPG	2020-10-02	64.11	68.5	64.05	68.2	10446775	{}	\N	2020-10-12 16:12:41.116333
71629	SPG	2020-10-05	68.71	69.33	67.42	68.76	7403520	{}	\N	2020-10-12 16:12:41.121124
71630	SPG	2020-10-06	69.75	72.63	69.01	69.98	12770233	{}	\N	2020-10-12 16:12:41.126612
71631	SPG	2020-10-07	70.32	72.16	69.8	71.03	8231291	{}	\N	2020-10-12 16:12:41.131964
71632	SPG	2020-10-08	70.83	71.23	69.4	70.07	7309024	{}	\N	2020-10-12 16:12:41.137068
71633	SPG	2020-10-09	70.14	70.3457	67.51	67.72	7524127	{}	\N	2020-10-12 16:12:41.141622
71634	SPG	2020-10-12	67.64	67.72	66.7	67.32	2838262	{}	\N	2020-10-12 16:12:41.146599
71635	SPLB	2020-09-28	31.8	31.93	31.77	31.91	273745	{}	\N	2020-10-12 16:12:42.498481
71636	SPLB	2020-09-29	32.02	32.02	31.85	31.88	206006	{}	\N	2020-10-12 16:12:42.51007
71637	SPLB	2020-09-30	31.8	31.85	31.7	31.78	179420	{}	\N	2020-10-12 16:12:42.517632
71638	SPLB	2020-10-01	31.63	31.87	31.63	31.87	632639	{}	\N	2020-10-12 16:12:42.523397
71639	SPLB	2020-10-02	31.82	31.86	31.7	31.83	97161	{}	\N	2020-10-12 16:12:42.528844
71640	SPLB	2020-10-05	31.69	31.77	31.58	31.59	567501	{}	\N	2020-10-12 16:12:42.534129
71641	SPLB	2020-10-06	31.675	31.86	31.53	31.58	499856	{}	\N	2020-10-12 16:12:42.539259
71642	SPLB	2020-10-07	31.61	31.81	31.54	31.58	1454465	{}	\N	2020-10-12 16:12:42.544553
71643	SPLB	2020-10-08	31.75	31.83	31.65	31.77	652326	{}	\N	2020-10-12 16:12:42.549333
71644	SPLB	2020-10-09	31.67	31.97	31.645	31.9	6016520	{}	\N	2020-10-12 16:12:42.55384
71645	SPLB	2020-10-12	31.96	32.05	31.92	32.045	56859	{}	\N	2020-10-12 16:12:42.558066
71646	SPNS	2020-09-28	29.82	30.1	29.5888	29.79	79577	{}	\N	2020-10-12 16:12:44.307372
71647	SPNS	2020-09-29	29.68	30.4	29.56	30.12	150934	{}	\N	2020-10-12 16:12:44.316751
71648	SPNS	2020-09-30	30.27	31.2	30.22	30.58	130956	{}	\N	2020-10-12 16:12:44.324173
71649	SPNS	2020-10-01	31.09	31.34	30.68	31.17	106879	{}	\N	2020-10-12 16:12:44.329811
71650	SPNS	2020-10-02	30.54	31.44	30.14	30.6	81731	{}	\N	2020-10-12 16:12:44.335
71651	SPNS	2020-10-05	30.82	31.56	30.68	31.43	70028	{}	\N	2020-10-12 16:12:44.339259
71652	SPNS	2020-10-06	31.1	31.14	30.12	30.31	108250	{}	\N	2020-10-12 16:12:44.343415
71653	SPNS	2020-10-07	30.46	30.945	30.35	30.78	118421	{}	\N	2020-10-12 16:12:44.347557
71654	SPNS	2020-10-08	30.87	31.41	30.6	31.36	80906	{}	\N	2020-10-12 16:12:44.352316
71655	SPNS	2020-10-09	31.67	32.42	31.55	31.93	124575	{}	\N	2020-10-12 16:12:44.356692
71656	SPNS	2020-10-12	32.48	32.75	32.34	32.61	16462	{}	\N	2020-10-12 16:12:44.36122
71657	SPRO	2020-09-28	9.7	9.7	9.25	9.3	194123	{}	\N	2020-10-12 16:12:47.148649
71658	SPRO	2020-09-29	10.2242	11.09	9.89	9.95	1348000	{}	\N	2020-10-12 16:12:47.155435
71659	SPRO	2020-09-30	10.2	11.38	10.2	11.16	591790	{}	\N	2020-10-12 16:12:47.161222
71660	SPRO	2020-10-01	11.37	11.937	11.03	11.65	273299	{}	\N	2020-10-12 16:12:47.166376
71661	SPRO	2020-10-02	11.34	12.4	11.27	12.06	307598	{}	\N	2020-10-12 16:12:47.171396
71662	SPRO	2020-10-05	12.27	12.92	12.15	12.46	402781	{}	\N	2020-10-12 16:12:47.175836
71663	SPRO	2020-10-06	12.46	12.67	12	12.26	319072	{}	\N	2020-10-12 16:12:47.180131
71664	SPRO	2020-10-07	12.3	13.15	12.18	12.91	383369	{}	\N	2020-10-12 16:12:47.184485
71665	SPRO	2020-10-08	13	13.48	12.7	13.35	259552	{}	\N	2020-10-12 16:12:47.188448
71666	SPRO	2020-10-09	13.45	13.7	13.21	13.31	239866	{}	\N	2020-10-12 16:12:47.193505
71667	SPRO	2020-10-12	13.42	13.8	13.05	13.66	98359	{}	\N	2020-10-12 16:12:47.198869
71668	SPTS	2020-09-28	30.7	30.71	30.7	30.71	1036107	{}	\N	2020-10-12 16:12:48.511836
71669	SPTS	2020-09-29	30.71	30.72	30.71	30.71	1241186	{}	\N	2020-10-12 16:12:48.525147
71670	SPTS	2020-09-30	30.72	30.72	30.7	30.7	875739	{}	\N	2020-10-12 16:12:48.534161
71671	SPTS	2020-10-01	30.7	30.71	30.7	30.71	2563659	{}	\N	2020-10-12 16:12:48.539772
71672	SPTS	2020-10-02	30.7	30.71	30.69	30.69	591669	{}	\N	2020-10-12 16:12:48.545117
71673	SPTS	2020-10-05	30.69	30.7	30.69	30.7	644841	{}	\N	2020-10-12 16:12:48.551125
71674	SPTS	2020-10-06	30.69	30.7	30.68	30.68	2070398	{}	\N	2020-10-12 16:12:48.558317
71675	SPTS	2020-10-07	30.68	30.69	30.68	30.68	2401042	{}	\N	2020-10-12 16:12:48.566332
71676	SPTS	2020-10-08	30.68	30.69	30.68	30.6811	1565436	{}	\N	2020-10-12 16:12:48.573129
71677	SPTS	2020-10-09	30.68	30.69	30.68	30.68	705966	{}	\N	2020-10-12 16:12:48.578043
71678	SPTS	2020-10-12	30.69	30.7	30.68	30.68	350356	{}	\N	2020-10-12 16:12:48.582355
71679	SPWH	2020-09-28	13.77	14.01	13.31	13.98	853681	{}	\N	2020-10-12 16:12:49.551398
71680	SPWH	2020-09-29	13.95	14.06	13.6434	13.83	615590	{}	\N	2020-10-12 16:12:49.563621
71681	SPWH	2020-09-30	14.04	14.7075	13.87	14.31	958789	{}	\N	2020-10-12 16:12:49.577874
71682	SPWH	2020-10-01	14.4	14.96	14.16	14.89	826499	{}	\N	2020-10-12 16:12:49.584722
71683	SPWH	2020-10-02	14.5	15.58	14.311	15.42	953074	{}	\N	2020-10-12 16:12:49.589288
71684	SPWH	2020-10-05	15.64	16.69	15.56	16.5	1356285	{}	\N	2020-10-12 16:12:49.593719
71685	SPWH	2020-10-06	16.52	16.75	15.93	16.39	1138675	{}	\N	2020-10-12 16:12:49.598728
71686	SPWH	2020-10-07	16.57	17.2	16.54	16.58	949017	{}	\N	2020-10-12 16:12:49.603939
71687	SPWH	2020-10-08	16.8	16.8	15.89	16.28	805506	{}	\N	2020-10-12 16:12:49.608472
71688	SPWH	2020-10-09	16.31	16.9	16.3	16.66	1093027	{}	\N	2020-10-12 16:12:49.613435
71689	SPWH	2020-10-12	17.05	17.14	16.3634	16.48	438951	{}	\N	2020-10-12 16:12:49.617875
71690	SPXN	2020-09-28	71.82	71.9954	71.82	71.9954	301	{}	\N	2020-10-12 16:12:51.14778
71691	SPXN	2020-09-29	71.7992	71.7992	71.7992	71.7992	78	{}	\N	2020-10-12 16:12:51.157845
71692	SPXN	2020-09-30	72.59	72.59	72.4795	72.4795	624	{}	\N	2020-10-12 16:12:51.165891
71693	SPXN	2020-10-01	72.87	72.87	72.3901	72.7447	627	{}	\N	2020-10-12 16:12:51.175719
71694	SPXN	2020-10-02	71.9212	71.9212	71.9212	71.9212	2	{}	\N	2020-10-12 16:12:51.181567
71695	SPXN	2020-10-05	73.1597	73.1597	73.1597	73.1597	2	{}	\N	2020-10-12 16:12:51.186424
71696	SPXN	2020-10-06	73.05	73.05	72.1707	72.1707	708	{}	\N	2020-10-12 16:12:51.191661
71697	SPXN	2020-10-07	73.4554	73.4554	73.4554	73.4554	15	{}	\N	2020-10-12 16:12:51.196208
71698	SPXN	2020-10-08	73.9417	73.9417	73.9417	73.9417	66	{}	\N	2020-10-12 16:12:51.201425
71699	SPXN	2020-10-09	74.68	74.71	74.605	74.655	568	{}	\N	2020-10-12 16:12:51.206315
71700	SPYG	2020-09-28	49.82	49.97	49.5681	49.94	1858514	{}	\N	2020-10-12 16:12:53.911933
71701	SPYG	2020-09-29	49.93	50.08	49.66	49.77	1505667	{}	\N	2020-10-12 16:12:53.919025
71702	SPYG	2020-09-30	49.8	50.61	49.8	50.11	1987281	{}	\N	2020-10-12 16:12:53.924998
71703	SPYG	2020-10-01	50.61	50.75	50.2774	50.57	2816375	{}	\N	2020-10-12 16:12:53.932731
71704	SPYG	2020-10-02	49.63	50.23	49.49	49.75	2772299	{}	\N	2020-10-12 16:12:53.940864
71705	SPYG	2020-10-05	50.07	50.71	50.06	50.67	1588461	{}	\N	2020-10-12 16:12:53.948304
71706	SPYG	2020-10-06	50.62	50.9071	49.7375	49.8	2642443	{}	\N	2020-10-12 16:12:53.95352
71707	SPYG	2020-10-07	50.34	50.82	50.31	50.73	2552066	{}	\N	2020-10-12 16:12:53.958433
71708	SPYG	2020-10-08	51.07	51.07	50.825	51.05	1172163	{}	\N	2020-10-12 16:12:53.964147
71709	SPYG	2020-10-09	51.31	51.7299	51.2745	51.71	956293	{}	\N	2020-10-12 16:12:53.969185
71710	SPYG	2020-10-12	52.35	52.789	52.19	52.74	355587	{}	\N	2020-10-12 16:12:53.974621
71711	SR	2020-09-28	53.06	53.46	52.42	52.89	271126	{}	\N	2020-10-12 16:12:55.082927
71712	SR	2020-09-29	52.67	53.24	52.4	52.92	481177	{}	\N	2020-10-12 16:12:55.095065
71713	SR	2020-09-30	54.1	55.22	52.47	53.2	725586	{}	\N	2020-10-12 16:12:55.103423
71714	SR	2020-10-01	52.8	53.11	52.28	52.56	668135	{}	\N	2020-10-12 16:12:55.109376
71715	SR	2020-10-02	52.28	53.52	51.815	53.39	695805	{}	\N	2020-10-12 16:12:55.115492
71716	SR	2020-10-05	53.59	53.97	52.78	53.53	627137	{}	\N	2020-10-12 16:12:55.120404
71717	SR	2020-10-06	53.92	55.22	53.44	54.08	775882	{}	\N	2020-10-12 16:12:55.125548
71718	SR	2020-10-07	54.18	54.96	53.64	54.87	638945	{}	\N	2020-10-12 16:12:55.130922
71719	SR	2020-10-08	55.33	55.86	54.75	55.8	585809	{}	\N	2020-10-12 16:12:55.135762
71720	SR	2020-10-09	55.83	56.11	55.54	55.77	398121	{}	\N	2020-10-12 16:12:55.140938
71721	SR	2020-10-12	55.51	56.3	55.44	55.88	153541	{}	\N	2020-10-12 16:12:55.146237
71722	SRG	2020-09-28	12.95	14.0499	12.94	13.66	670522	{}	\N	2020-10-12 16:12:56.096813
71723	SRG	2020-09-29	13.65	13.65	12.784	13.1	787951	{}	\N	2020-10-12 16:12:56.111835
71724	SRG	2020-09-30	13.13	14.21	13.13	13.45	1042572	{}	\N	2020-10-12 16:12:56.125038
71725	SRG	2020-10-01	13.51	13.98	13.41	13.86	1284813	{}	\N	2020-10-12 16:12:56.13376
71726	SRG	2020-10-02	13.25	14.18	13.11	14.07	648341	{}	\N	2020-10-12 16:12:56.140568
71727	SRG	2020-10-05	14.2	14.3888	13.48	13.79	698498	{}	\N	2020-10-12 16:12:56.148252
71728	SRG	2020-10-06	14.08	14.49	13.7101	13.81	768466	{}	\N	2020-10-12 16:12:56.155736
71729	SRG	2020-10-07	14	14.09	13.77	13.79	523236	{}	\N	2020-10-12 16:12:56.162091
71730	SRG	2020-10-08	13.87	14.45	13.83	14.4	569207	{}	\N	2020-10-12 16:12:56.168239
71731	SRG	2020-10-09	14.5	14.58	14.04	14.1	562991	{}	\N	2020-10-12 16:12:56.17372
71732	SRG	2020-10-12	14	14.2293	13.67	13.8	355271	{}	\N	2020-10-12 16:12:56.1791
71733	SRTS	2020-09-28	2.5431	2.7	2.5	2.53	20497	{}	\N	2020-10-12 16:12:57.256399
71734	SRTS	2020-09-29	2.575	2.595	2.55	2.595	1158	{}	\N	2020-10-12 16:12:57.266831
71735	SRTS	2020-09-30	2.55	2.68	2.38	2.52	42775	{}	\N	2020-10-12 16:12:57.273661
71736	SRTS	2020-10-01	2.51	2.69	2.49	2.54	14256	{}	\N	2020-10-12 16:12:57.279985
71737	SRTS	2020-10-02	2.56	2.56	2.43	2.495	46188	{}	\N	2020-10-12 16:12:57.285621
71738	SRTS	2020-10-05	2.48	2.75	2.46	2.49	30228	{}	\N	2020-10-12 16:12:57.291349
71739	SRTS	2020-10-06	2.695	2.695	2.52	2.52	15172	{}	\N	2020-10-12 16:12:57.296207
71740	SRTS	2020-10-07	2.46	2.54	2.445	2.5	38323	{}	\N	2020-10-12 16:12:57.301157
71741	SRTS	2020-10-08	2.5	2.65	2.5	2.63	12824	{}	\N	2020-10-12 16:12:57.305417
71742	SRTS	2020-10-09	2.63	2.81	2.63	2.81	7239	{}	\N	2020-10-12 16:12:57.309875
71743	SRTS	2020-10-12	2.81	2.81	2.7739	2.7739	285	{}	\N	2020-10-12 16:12:57.31391
71744	SSKN	2020-09-28	1.42	1.44	1.3801	1.44	8809	{}	\N	2020-10-12 16:12:58.29489
71745	SSKN	2020-09-29	1.3469	1.45	1.3469	1.37	20835	{}	\N	2020-10-12 16:12:58.304824
71746	SSKN	2020-09-30	1.38	1.4151	1.32	1.35	26241	{}	\N	2020-10-12 16:12:58.312224
71747	SSKN	2020-10-01	1.3323	1.34	1.2766	1.31	35845	{}	\N	2020-10-12 16:12:58.318891
71748	SSKN	2020-10-02	1.31	1.33	1.27	1.32	15792	{}	\N	2020-10-12 16:12:58.324405
71749	SSKN	2020-10-05	1.29	1.42	1.26	1.38	48783	{}	\N	2020-10-12 16:12:58.329987
71750	SSKN	2020-10-06	1.37	1.46	1.37	1.4	50149	{}	\N	2020-10-12 16:12:58.335053
71751	SSKN	2020-10-07	1.44	1.45	1.3915	1.45	8319	{}	\N	2020-10-12 16:12:58.340009
71752	SSKN	2020-10-08	1.44	1.44	1.4	1.44	10806	{}	\N	2020-10-12 16:12:58.345057
71753	SSKN	2020-10-09	1.43	1.45	1.4	1.41	23659	{}	\N	2020-10-12 16:12:58.349776
71754	SSKN	2020-10-12	1.4	1.4	1.34	1.34	8912	{}	\N	2020-10-12 16:12:58.354755
71755	STAY	2020-09-28	12.07	12.525	12.07	12.31	1745678	{}	\N	2020-10-12 16:12:59.671922
71756	STAY	2020-09-29	12.91	12.91	11.92	12.04	1455588	{}	\N	2020-10-12 16:12:59.683093
71757	STAY	2020-09-30	12.32	12.64	11.83	11.95	2218626	{}	\N	2020-10-12 16:12:59.69347
71758	STAY	2020-10-01	11.95	12.08	11.65	11.97	2081643	{}	\N	2020-10-12 16:12:59.70283
71759	STAY	2020-10-02	11.65	12.09	11.6	12.07	1326239	{}	\N	2020-10-12 16:12:59.708931
71760	STAY	2020-10-05	12.08	12.22	11.95	12.2	1085154	{}	\N	2020-10-12 16:12:59.714601
71761	STAY	2020-10-06	12.26	12.38	12.0219	12.07	1018676	{}	\N	2020-10-12 16:12:59.719783
71762	STAY	2020-10-07	12.16	12.22	12.02	12.13	935305	{}	\N	2020-10-12 16:12:59.725571
71763	STAY	2020-10-08	12.21	12.27	12.01	12.11	1386464	{}	\N	2020-10-12 16:12:59.730921
71764	STAY	2020-10-09	12.2	12.25	11.915	12.14	976629	{}	\N	2020-10-12 16:12:59.736401
71765	STAY	2020-10-12	12.14	12.17	11.98	12.17	234754	{}	\N	2020-10-12 16:12:59.742521
71766	STKS	2020-09-28	1.97	2.19	1.93	2.06	31696	{}	\N	2020-10-12 16:13:03.102028
71767	STKS	2020-09-29	2.07	2.11	2.03	2.04	23634	{}	\N	2020-10-12 16:13:03.1122
71768	STKS	2020-09-30	2.02	2.09	1.98	2	14599	{}	\N	2020-10-12 16:13:03.120297
71769	STKS	2020-10-01	2	2.0999	2	2.0499	11905	{}	\N	2020-10-12 16:13:03.126851
71770	STKS	2020-10-02	2	2.29	2	2.25	101635	{}	\N	2020-10-12 16:13:03.132678
71771	STKS	2020-10-05	2.25	2.4699	2.23	2.28	82909	{}	\N	2020-10-12 16:13:03.138849
71772	STKS	2020-10-06	2.33	2.44	2.29	2.31	70390	{}	\N	2020-10-12 16:13:03.144364
71773	STKS	2020-10-07	2.38	2.38	2.2	2.33	33655	{}	\N	2020-10-12 16:13:03.151622
71774	STKS	2020-10-08	2.4	2.575	2.32	2.5	96630	{}	\N	2020-10-12 16:13:03.157415
71775	STKS	2020-10-09	2.56	2.6546	2.47	2.53	78799	{}	\N	2020-10-12 16:13:03.162485
71776	STKS	2020-10-12	2.58	2.7	2.33	2.57	17458	{}	\N	2020-10-12 16:13:03.168454
71777	STOR	2020-09-28	27.43	28.04	27.35	27.94	1998492	{}	\N	2020-10-12 16:13:05.823425
71778	STOR	2020-09-29	27.51	27.65	26.77	27.23	4180949	{}	\N	2020-10-12 16:13:05.836428
71779	STOR	2020-09-30	27.64	27.95	27.1	27.43	5593135	{}	\N	2020-10-12 16:13:05.843591
71780	STOR	2020-10-01	27.59	28.42	27.3	28.42	5899433	{}	\N	2020-10-12 16:13:05.850168
71781	STOR	2020-10-02	27.59	29.0101	27.44	28.85	4885139	{}	\N	2020-10-12 16:13:05.85623
71782	STOR	2020-10-05	29.1	29.24	28.39	28.87	5586464	{}	\N	2020-10-12 16:13:05.861583
71783	STOR	2020-10-06	29	29.17	28.34	28.47	5315684	{}	\N	2020-10-12 16:13:05.866413
71784	STOR	2020-10-07	28.61	28.67	27.86	27.86	4911629	{}	\N	2020-10-12 16:13:05.871917
71785	STOR	2020-10-08	28.1	28.46	27.9	28.41	3972999	{}	\N	2020-10-12 16:13:05.877856
71786	STOR	2020-10-09	28.65	28.65	27.9001	27.98	4414101	{}	\N	2020-10-12 16:13:05.882951
71787	STOR	2020-10-12	28.07	28.071	27.66	27.945	906726	{}	\N	2020-10-12 16:13:05.888613
71788	STX	2020-09-28	48.56	49.8	48.31	49.66	3427794	{}	\N	2020-10-12 16:13:06.852256
71789	STX	2020-09-29	49.61	50.7	49.48	50.07	3475199	{}	\N	2020-10-12 16:13:06.86364
71790	STX	2020-09-30	49.97	50.07	49.07	49.27	2791046	{}	\N	2020-10-12 16:13:06.871236
71791	STX	2020-10-01	49.82	49.82	48.83	49.06	1834148	{}	\N	2020-10-12 16:13:06.877912
71792	STX	2020-10-02	48.41	49.16	48.08	48.74	1647639	{}	\N	2020-10-12 16:13:06.883927
71793	STX	2020-10-05	49.18	49.48	48.74	49.11	1922706	{}	\N	2020-10-12 16:13:06.888864
71794	STX	2020-10-06	49.35	50.41	49.045	49.6	2540887	{}	\N	2020-10-12 16:13:06.893916
71795	STX	2020-10-07	49.7	50.7872	49.65	50.45	2541694	{}	\N	2020-10-12 16:13:06.899344
71796	STX	2020-10-08	50.81	51.38	50.52	51.33	2393918	{}	\N	2020-10-12 16:13:06.90624
71797	STX	2020-10-09	51.58	51.83	50.735	50.97	2374318	{}	\N	2020-10-12 16:13:06.912947
71798	STX	2020-10-12	50.75	51.34	50.735	51.12	928942	{}	\N	2020-10-12 16:13:06.920283
71799	SUNW	2020-09-28	2.1	3.44	1.88	2.97	77995641	{}	\N	2020-10-12 16:13:08.11289
71800	SUNW	2020-09-29	2.67	2.76	2.27	2.48	14004601	{}	\N	2020-10-12 16:13:08.121934
71801	SUNW	2020-09-30	2.675	2.85	2.45	2.54	10150765	{}	\N	2020-10-12 16:13:08.129002
71802	SUNW	2020-10-01	2.26	2.44	2.22	2.33	6064972	{}	\N	2020-10-12 16:13:08.135867
71803	SUNW	2020-10-02	2.125	2.49	2.09	2.35	6336053	{}	\N	2020-10-12 16:13:08.140902
71804	SUNW	2020-10-05	2.29	2.8	2.28	2.59	13675006	{}	\N	2020-10-12 16:13:08.145565
71805	SUNW	2020-10-06	2.6	3.4	2.51	2.8	30574295	{}	\N	2020-10-12 16:13:08.152388
71806	SUNW	2020-10-07	4.59	4.85	3.73	4.06	89341770	{}	\N	2020-10-12 16:13:08.157651
71807	SUNW	2020-10-08	3.75	3.94	3.3	3.58	10583041	{}	\N	2020-10-12 16:13:08.162276
71808	SUNW	2020-10-09	3.48	3.84	3.4	3.52	8516427	{}	\N	2020-10-12 16:13:08.166851
71809	SUNW	2020-10-12	3.4	3.44	3.2	3.255	2545691	{}	\N	2020-10-12 16:13:08.1714
71810	SVVC	2020-09-28	4.3015	4.34	4.19	4.33	8750	{}	\N	2020-10-12 16:13:09.113247
71811	SVVC	2020-09-29	4.28	4.3	4.12	4.13	9220	{}	\N	2020-10-12 16:13:09.121127
71812	SVVC	2020-09-30	4.13	4.245	4.12	4.13	12350	{}	\N	2020-10-12 16:13:09.127831
71813	SVVC	2020-10-01	4.09	4.17	4	4.03	74417	{}	\N	2020-10-12 16:13:09.134405
71814	SVVC	2020-10-02	4.1708	4.2153	4.01	4.01	11647	{}	\N	2020-10-12 16:13:09.140245
71815	SVVC	2020-10-05	4.1078	4.1078	3.9	3.98	71030	{}	\N	2020-10-12 16:13:09.145382
71816	SVVC	2020-10-06	3.95	4.175	3.95	3.98	19325	{}	\N	2020-10-12 16:13:09.151621
71817	SVVC	2020-10-07	4.01	4.08	3.97	3.97	7906	{}	\N	2020-10-12 16:13:09.158222
71818	SVVC	2020-10-08	4.05	4.05	3.94	4	9542	{}	\N	2020-10-12 16:13:09.164408
71819	SVVC	2020-10-09	3.94	3.96	3.86	3.92	20704	{}	\N	2020-10-12 16:13:09.171223
71820	SVVC	2020-10-12	3.89	3.99	3.89	3.901	14655	{}	\N	2020-10-12 16:13:09.177269
71821	SWZ	2020-09-28	8.25	8.25	8.1482	8.22	27227	{}	\N	2020-10-12 16:13:10.158515
71822	SWZ	2020-09-29	8.24	8.24	8.11	8.19	124052	{}	\N	2020-10-12 16:13:10.170312
71823	SWZ	2020-09-30	8.3	8.31	8.21	8.28	47806	{}	\N	2020-10-12 16:13:10.178264
71824	SWZ	2020-10-01	8.34	8.51	8.3	8.33	110820	{}	\N	2020-10-12 16:13:10.185427
71825	SWZ	2020-10-02	8.37	8.37	8.29	8.32	39292	{}	\N	2020-10-12 16:13:10.191658
71826	SWZ	2020-10-05	8.3	8.4007	8.3	8.39	17740	{}	\N	2020-10-12 16:13:10.197238
71827	SWZ	2020-10-06	8.36	8.445	8.2426	8.25	32258	{}	\N	2020-10-12 16:13:10.202291
71828	SWZ	2020-10-07	8.28	8.31	8.22	8.22	25371	{}	\N	2020-10-12 16:13:10.20807
71829	SWZ	2020-10-08	8.24	8.3599	8.1	8.19	111894	{}	\N	2020-10-12 16:13:10.213259
71830	SWZ	2020-10-09	8.26	8.3574	8.18	8.19	50571	{}	\N	2020-10-12 16:13:10.21944
71831	SWZ	2020-10-12	8.26	8.27	8.26	8.26	27908	{}	\N	2020-10-12 16:13:10.224705
71832	SYE	2020-09-28	87.49	87.5471	87.4659	87.5471	868	{}	\N	2020-10-12 16:13:13.167539
71833	SYE	2020-09-29	87.34	87.34	87.045	87.15	2620	{}	\N	2020-10-12 16:13:13.175368
71834	SYE	2020-09-30	88.22	88.531	88.0952	88.0952	978	{}	\N	2020-10-12 16:13:13.181572
71835	SYE	2020-10-01	88.08	88.1157	88.08	88.1157	268	{}	\N	2020-10-12 16:13:13.186809
71836	SYE	2020-10-02	87.246	87.246	87.246	87.246	38	{}	\N	2020-10-12 16:13:13.191987
71837	SYE	2020-10-05	88.461	88.7304	88.3995	88.7304	278	{}	\N	2020-10-12 16:13:13.196692
71838	SYE	2020-10-06	88.85	88.85	87.3979	87.3979	431	{}	\N	2020-10-12 16:13:13.201015
71839	SYE	2020-10-07	88.7	89.0221	88.7	89.0221	926	{}	\N	2020-10-12 16:13:13.205431
71840	SYE	2020-10-08	89.3	89.488	89.3	89.488	243	{}	\N	2020-10-12 16:13:13.210018
71841	SYE	2020-10-09	90.1435	90.1435	90.1435	90.1435	157	{}	\N	2020-10-12 16:13:13.214461
71842	SYN	2020-09-28	0.4736	0.48	0.451	0.4771	133322	{}	\N	2020-10-12 16:13:14.196061
71843	SYN	2020-09-29	0.4828	0.4897	0.466	0.484899	146508	{}	\N	2020-10-12 16:13:14.20773
71844	SYN	2020-09-30	0.489	0.489	0.461	0.4747	136218	{}	\N	2020-10-12 16:13:14.214553
71845	SYN	2020-10-01	0.47	0.47	0.43	0.4629	180480	{}	\N	2020-10-12 16:13:14.220387
71846	SYN	2020-10-02	0.261	0.3295	0.261	0.317	2919226	{}	\N	2020-10-12 16:13:14.22709
71847	SYN	2020-10-05	0.3203	0.38	0.3203	0.37	1262101	{}	\N	2020-10-12 16:13:14.232581
71848	SYN	2020-10-06	0.41	0.4297	0.3616	0.3659	1010005	{}	\N	2020-10-12 16:13:14.238345
71849	SYN	2020-10-07	0.35	0.379299	0.3359	0.35	579543	{}	\N	2020-10-12 16:13:14.243545
71850	SYN	2020-10-08	0.33	0.3489	0.3226	0.3431	596769	{}	\N	2020-10-12 16:13:14.249017
71851	SYN	2020-10-09	0.35	0.35	0.3161	0.328	662660	{}	\N	2020-10-12 16:13:14.253852
71852	SYN	2020-10-12	0.3218	0.339	0.317	0.3206	175128	{}	\N	2020-10-12 16:13:14.259092
71853	SYX	2020-09-28	22.56	23.46	22.56	23.35	78463	{}	\N	2020-10-12 16:13:16.15864
71854	SYX	2020-09-29	23.47	23.64	23.09	23.45	135054	{}	\N	2020-10-12 16:13:16.166244
71855	SYX	2020-09-30	23.66	24.64	23.66	23.94	167440	{}	\N	2020-10-12 16:13:16.172611
71856	SYX	2020-10-01	24.16	24.16	23.31	23.59	84069	{}	\N	2020-10-12 16:13:16.180847
71857	SYX	2020-10-02	22.71	23.41	22.55	22.97	112959	{}	\N	2020-10-12 16:13:16.188416
71858	SYX	2020-10-05	23.17	23.51	23.17	23.44	78398	{}	\N	2020-10-12 16:13:16.19554
71859	SYX	2020-10-06	23.62	23.62	22.87	22.94	57050	{}	\N	2020-10-12 16:13:16.201835
71860	SYX	2020-10-07	23.04	23.45	22.92	23.32	154967	{}	\N	2020-10-12 16:13:16.207285
71861	SYX	2020-10-08	23.52	24.54	23.52	24.28	119485	{}	\N	2020-10-12 16:13:16.21292
71862	SYX	2020-10-09	24.45	24.45	23.7	23.9	61244	{}	\N	2020-10-12 16:13:16.218547
71863	SYX	2020-10-12	24.06	24.36	23.9	24.18	18868	{}	\N	2020-10-12 16:13:16.224529
71864	SRCE	2020-09-28	30.33	31.11	30.3	30.89	65796	{}	\N	2020-10-12 16:13:18.344776
71865	SRCE	2020-09-29	30.69	31.05	30.35	30.85	67532	{}	\N	2020-10-12 16:13:18.353978
71866	SRCE	2020-09-30	31.06	31.36	30.55	30.84	75156	{}	\N	2020-10-12 16:13:18.378482
71867	SRCE	2020-10-01	30.91	31.24	30.325	31.24	65485	{}	\N	2020-10-12 16:13:18.389215
71868	SRCE	2020-10-02	30.65	32.148	30.65	31.83	49521	{}	\N	2020-10-12 16:13:18.404058
71869	SRCE	2020-10-05	32.2	32.85	31.63	32.68	68310	{}	\N	2020-10-12 16:13:18.412778
71870	SRCE	2020-10-06	33.01	34.01	32.285	32.8	49834	{}	\N	2020-10-12 16:13:18.421894
71871	SRCE	2020-10-07	33.19	33.73	32.135	33.48	69467	{}	\N	2020-10-12 16:13:18.430366
71872	SRCE	2020-10-08	33.86	33.875	33.25	33.59	50017	{}	\N	2020-10-12 16:13:18.437487
71873	SRCE	2020-10-09	33.93	33.93	33.15	33.16	54787	{}	\N	2020-10-12 16:13:18.445098
71874	SRCE	2020-10-12	33.22	33.66	32.94	33.45	7324	{}	\N	2020-10-12 16:13:18.451475
71875	SRLP	2020-09-28	15.11	15.1625	14.995	15.08	10484	{}	\N	2020-10-12 16:13:19.510443
71876	SRLP	2020-09-29	15.13	15.21	15.1	15.15	16838	{}	\N	2020-10-12 16:13:19.519968
71877	SRLP	2020-09-30	15.17	15.3753	15.13	15.13	23996	{}	\N	2020-10-12 16:13:19.529938
71878	SRLP	2020-10-01	15.22	15.235	15.017	15.03	20159	{}	\N	2020-10-12 16:13:19.538813
71879	SRLP	2020-10-02	14.93	15.13	14.93	15.02	18224	{}	\N	2020-10-12 16:13:19.545141
71880	SRLP	2020-10-05	15.02	15.329	15.02	15.27	32850	{}	\N	2020-10-12 16:13:19.551557
71881	SRLP	2020-10-06	15.37	15.62	15.34	15.62	20150	{}	\N	2020-10-12 16:13:19.559603
71882	SRLP	2020-10-07	15.61	15.61	15.34	15.36	21580	{}	\N	2020-10-12 16:13:19.566706
71883	SRLP	2020-10-08	15.5	15.685	15.37	15.4	56754	{}	\N	2020-10-12 16:13:19.573285
71884	SRLP	2020-10-09	15.5	15.615	15.3477	15.52	35674	{}	\N	2020-10-12 16:13:19.580578
71885	SRLP	2020-10-12	15.56	15.649	15.46	15.46	9488	{}	\N	2020-10-12 16:13:19.586369
71886	SSB	2020-09-28	47.62	48.97	47.525	48.03	263399	{}	\N	2020-10-12 16:13:21.865101
71887	SSB	2020-09-29	47.72	47.93	45.96	47.51	178120	{}	\N	2020-10-12 16:13:21.880707
71888	SSB	2020-09-30	47.92	49.02	47.92	48.15	306634	{}	\N	2020-10-12 16:13:21.889704
71889	SSB	2020-10-01	48.06	48.36	46.88	48.15	321573	{}	\N	2020-10-12 16:13:21.896164
71890	SSB	2020-10-02	47.3	49.6	47.21	49.28	185196	{}	\N	2020-10-12 16:13:21.901366
71891	SSB	2020-10-05	49.98	51.55	49.69	51.37	265812	{}	\N	2020-10-12 16:13:21.906604
71892	SSB	2020-10-06	52.3	55	51.91	52.99	532229	{}	\N	2020-10-12 16:13:21.910941
71893	SSB	2020-10-07	53.79	55.51	53.52	55.42	330457	{}	\N	2020-10-12 16:13:21.915452
71894	SSB	2020-10-08	56.26	56.415	55.02	55.39	251355	{}	\N	2020-10-12 16:13:21.919979
71895	SSB	2020-10-09	55.82	55.82	54.005	54.58	319709	{}	\N	2020-10-12 16:13:21.92445
71896	SSB	2020-10-12	54.53	55.62	54.23	55.36	77084	{}	\N	2020-10-12 16:13:21.92882
71897	SSTK	2020-09-28	52.39	52.57	51.47	52.04	193962	{}	\N	2020-10-12 16:13:22.865658
71898	SSTK	2020-09-29	53.82	54.35	52.63	53.4	643415	{}	\N	2020-10-12 16:13:22.884416
71899	SSTK	2020-09-30	53.25	53.25	51.62	52.04	544728	{}	\N	2020-10-12 16:13:22.898807
71900	SSTK	2020-10-01	52.29	53.61	51.99	53.21	629365	{}	\N	2020-10-12 16:13:22.913003
71901	SSTK	2020-10-02	52.08	53.21	51.4118	52.41	716905	{}	\N	2020-10-12 16:13:22.92576
71902	SSTK	2020-10-05	53.02	53.84	52.05	53.48	401033	{}	\N	2020-10-12 16:13:22.933593
71903	SSTK	2020-10-06	53.39	56.4	52.94	55.4	1434281	{}	\N	2020-10-12 16:13:22.946207
71904	SSTK	2020-10-07	55.66	57.93	55.66	57.84	930359	{}	\N	2020-10-12 16:13:22.960735
71905	SSTK	2020-10-08	58	58.43	56.615	56.93	563130	{}	\N	2020-10-12 16:13:22.967841
71906	SSTK	2020-10-09	57.44	58.45	57.11	58.26	399170	{}	\N	2020-10-12 16:13:22.974522
71907	SSTK	2020-10-12	58.51	59.01	57.91	58.8	157284	{}	\N	2020-10-12 16:13:22.987993
71908	STAG	2020-09-28	30.53	31.13	30.2804	30.98	630676	{}	\N	2020-10-12 16:13:24.03695
71909	STAG	2020-09-29	30.73	30.84	30.25	30.59	939232	{}	\N	2020-10-12 16:13:24.051018
71910	STAG	2020-09-30	30.83	31.09	30.13	30.49	1372149	{}	\N	2020-10-12 16:13:24.059162
71911	STAG	2020-10-01	30.57	31.1	30.2	31.09	1679749	{}	\N	2020-10-12 16:13:24.065985
71912	STAG	2020-10-02	30.61	31.64	30.45	31.4	1217761	{}	\N	2020-10-12 16:13:24.072696
71913	STAG	2020-10-05	31.59	32.01	31.09	31.94	3004420	{}	\N	2020-10-12 16:13:24.07794
71914	STAG	2020-10-06	32.25	32.4	31.58	31.71	2023057	{}	\N	2020-10-12 16:13:24.083222
71915	STAG	2020-10-07	31.91	32.47	31.69	32.36	1508164	{}	\N	2020-10-12 16:13:24.088702
71916	STAG	2020-10-08	32.53	33	32.38	32.98	1951838	{}	\N	2020-10-12 16:13:24.094037
71917	STAG	2020-10-09	33.15	33.18	32.54	32.6	1634312	{}	\N	2020-10-12 16:13:24.099106
71918	STAG	2020-10-12	32.6	33.37	32.5289	33.0978	422572	{}	\N	2020-10-12 16:13:24.103846
71919	STNG	2020-09-28	11.22	11.25	10.81	11.02	1250131	{}	\N	2020-10-12 16:13:25.096804
71920	STNG	2020-09-29	10.96	11.08	10.65	10.95	2556343	{}	\N	2020-10-12 16:13:25.109796
71921	STNG	2020-09-30	10.95	11.35	10.93	11.07	2316720	{}	\N	2020-10-12 16:13:25.117922
71922	STNG	2020-10-01	11.1	11.42	10.82	11.24	1949710	{}	\N	2020-10-12 16:13:25.123833
71923	STNG	2020-10-02	10.9	11.36	10.7589	11.19	2889867	{}	\N	2020-10-12 16:13:25.130185
71924	STNG	2020-10-05	11.3	11.61	11.21	11.3	2231717	{}	\N	2020-10-12 16:13:25.134734
71925	STNG	2020-10-06	11.44	11.77	11.18	11.18	3040587	{}	\N	2020-10-12 16:13:25.139634
71926	STNG	2020-10-07	11.19	11.2799	10.905	10.92	2687378	{}	\N	2020-10-12 16:13:25.144912
71927	STNG	2020-10-08	10.99	11.67	10.96	11.54	2372641	{}	\N	2020-10-12 16:13:25.14979
71928	STNG	2020-10-09	11.6	11.72	11.285	11.34	1427205	{}	\N	2020-10-12 16:13:25.154824
71929	STNG	2020-10-12	11.18	11.3	10.605	10.87	1554777	{}	\N	2020-10-12 16:13:25.1604
71930	STT	2020-09-28	59.73	61.2	59.5	60.49	2104693	{}	\N	2020-10-12 16:13:27.860029
71931	STT	2020-09-29	60.28	60.59	59.26	59.89	2432006	{}	\N	2020-10-12 16:13:27.872054
71932	STT	2020-09-30	59.53	60.34	58.725	59.33	5297980	{}	\N	2020-10-12 16:13:27.881776
71933	STT	2020-10-01	59.76	60.06	58.05	58.75	4621018	{}	\N	2020-10-12 16:13:27.888835
71934	STT	2020-10-02	57.84	59.88	57.6	59.68	3578429	{}	\N	2020-10-12 16:13:27.894984
71935	STT	2020-10-05	60.1	61.67	60.095	61.39	2620563	{}	\N	2020-10-12 16:13:27.900565
71936	STT	2020-10-06	61.99	62.96	60.72	61.02	3098258	{}	\N	2020-10-12 16:13:27.905921
71937	STT	2020-10-07	61.46	63.72	61.46	63.41	3658649	{}	\N	2020-10-12 16:13:27.911519
71938	STT	2020-10-08	63.83	65.05	63.47	65	3367865	{}	\N	2020-10-12 16:13:27.917084
71939	STT	2020-10-09	66.05	67.31	65.64	66.43	6170976	{}	\N	2020-10-12 16:13:27.923043
71940	STT	2020-10-12	66.29	66.58	65.73	66.545	1282911	{}	\N	2020-10-12 16:13:27.928582
71941	SUNS	2020-09-28	12.81	13.3899	12.81	13.03	71805	{}	\N	2020-10-12 16:13:29.316705
71942	SUNS	2020-09-29	13.1	13.2	12.63	12.67	38608	{}	\N	2020-10-12 16:13:29.324591
71943	SUNS	2020-09-30	12.69	12.78	12.6	12.6	55142	{}	\N	2020-10-12 16:13:29.330981
71944	SUNS	2020-10-01	12.95	13.11	12.75	13.11	125308	{}	\N	2020-10-12 16:13:29.337727
71945	SUNS	2020-10-02	13.11	13.59	13.05	13.54	134766	{}	\N	2020-10-12 16:13:29.343894
71946	SUNS	2020-10-05	13.56	13.7	13.45	13.5	77883	{}	\N	2020-10-12 16:13:29.349475
71947	SUNS	2020-10-06	13.6	13.613	13.31	13.4	34364	{}	\N	2020-10-12 16:13:29.354931
71948	SUNS	2020-10-07	13.45	13.55	13.35	13.41	41547	{}	\N	2020-10-12 16:13:29.360452
71949	SUNS	2020-10-08	13.47	13.7	13.43	13.5	24640	{}	\N	2020-10-12 16:13:29.366962
71950	SUNS	2020-10-09	13.48	13.48	13.37	13.41	31270	{}	\N	2020-10-12 16:13:29.371933
71951	SUNS	2020-10-12	13.36	13.489	13.35	13.4	25364	{}	\N	2020-10-12 16:13:29.377485
71952	SVM	2020-09-28	7.28	7.33	7.055	7.18	964840	{}	\N	2020-10-12 16:13:30.989043
71953	SVM	2020-09-29	7.27	7.48	7.23	7.38	1054705	{}	\N	2020-10-12 16:13:31.002355
71954	SVM	2020-09-30	7.28	7.39	7.13	7.24	1007333	{}	\N	2020-10-12 16:13:31.01346
71955	SVM	2020-10-01	7.35	7.505	7.24	7.45	812665	{}	\N	2020-10-12 16:13:31.021204
71956	SVM	2020-10-02	7.31	7.45	7.23	7.26	919916	{}	\N	2020-10-12 16:13:31.029939
71957	SVM	2020-10-05	7.29	7.66	7.29	7.54	1184097	{}	\N	2020-10-12 16:13:31.038075
71958	SVM	2020-10-06	7.56	7.61	7.13	7.14	989860	{}	\N	2020-10-12 16:13:31.046004
71959	SVM	2020-10-07	7.22	7.28	7.08	7.14	998476	{}	\N	2020-10-12 16:13:31.052706
71960	SVM	2020-10-08	7.24	7.3499	7.16	7.3	846519	{}	\N	2020-10-12 16:13:31.059806
71961	SVM	2020-10-09	7.38	7.78	7.38	7.76	1288230	{}	\N	2020-10-12 16:13:31.067307
71962	SVM	2020-10-12	7.76	7.855	7.64	7.76	364124	{}	\N	2020-10-12 16:13:31.074994
71963	SWIR	2020-09-28	9.95	10.58	9.9	10.58	159452	{}	\N	2020-10-12 16:13:32.85292
71964	SWIR	2020-09-29	10.53	10.865	10.52	10.81	184916	{}	\N	2020-10-12 16:13:32.865298
71965	SWIR	2020-09-30	10.75	11.25	10.75	11.16	283722	{}	\N	2020-10-12 16:13:32.873234
71966	SWIR	2020-10-01	11.22	11.34	11.06	11.19	209130	{}	\N	2020-10-12 16:13:32.879598
71967	SWIR	2020-10-02	10.94	10.94	10.56	10.66	200301	{}	\N	2020-10-12 16:13:32.885442
71968	SWIR	2020-10-05	10.8	11.07	10.8	11.07	91919	{}	\N	2020-10-12 16:13:32.890879
71969	SWIR	2020-10-06	11.08	11.26	11.04	11.08	121857	{}	\N	2020-10-12 16:13:32.895807
71970	SWIR	2020-10-07	11.1	11.39	11.1	11.35	109915	{}	\N	2020-10-12 16:13:32.900744
71971	SWIR	2020-10-08	11.45	11.85	11.35	11.77	124267	{}	\N	2020-10-12 16:13:32.906142
71972	SWIR	2020-10-09	11.85	11.99	11.55	11.7	122929	{}	\N	2020-10-12 16:13:32.911762
71973	SWIR	2020-10-12	11.7	11.74	11.455	11.4643	30556	{}	\N	2020-10-12 16:13:32.917027
71974	SYNA	2020-09-28	80.11	81.18	79.38	81.06	335875	{}	\N	2020-10-12 16:13:37.086808
71975	SYNA	2020-09-29	81.5	82.58	80.615	80.83	200298	{}	\N	2020-10-12 16:13:37.099016
71976	SYNA	2020-09-30	81	82.16	80.14	80.42	333418	{}	\N	2020-10-12 16:13:37.107919
71977	SYNA	2020-10-01	81.42	82.93	79.595	80.66	357586	{}	\N	2020-10-12 16:13:37.114667
71978	SYNA	2020-10-02	78.24	79.89	76.87	78.18	400403	{}	\N	2020-10-12 16:13:37.121059
71979	SYNA	2020-10-05	79.09	81.93	79.01	80.98	275922	{}	\N	2020-10-12 16:13:37.126877
71980	SYNA	2020-10-06	81.38	82.62	80.06	80.22	304348	{}	\N	2020-10-12 16:13:37.132469
71981	SYNA	2020-10-07	81.49	82.845	81.1	81.66	303871	{}	\N	2020-10-12 16:13:37.13834
71982	SYNA	2020-10-08	82.42	82.42	80.92	81.49	223263	{}	\N	2020-10-12 16:13:37.143693
71983	SYNA	2020-10-09	82.45	85.255	82.33	84.44	562056	{}	\N	2020-10-12 16:13:37.149228
71984	SYNA	2020-10-12	84.72	85.96	83.29	85.395	236841	{}	\N	2020-10-12 16:13:37.154865
71985	TA	2020-09-28	19.05	19.46	18.95	19.01	175254	{}	\N	2020-10-12 16:13:38.465978
71986	TA	2020-09-29	19.01	19.48	18.9	19.06	205235	{}	\N	2020-10-12 16:13:38.477937
71987	TA	2020-09-30	19.1	19.92	19.1	19.54	107409	{}	\N	2020-10-12 16:13:38.484992
71988	TA	2020-10-01	19.54	20.82	19.4	20.74	111415	{}	\N	2020-10-12 16:13:38.491545
71989	TA	2020-10-02	20.54	21.11	20.07	21.07	149584	{}	\N	2020-10-12 16:13:38.497173
71990	TA	2020-10-05	21.46	21.74	21.1	21.57	134229	{}	\N	2020-10-12 16:13:38.502383
71991	TA	2020-10-06	21.68	21.83	20.95	21.23	257486	{}	\N	2020-10-12 16:13:38.507363
71992	TA	2020-10-07	21.59	23.4799	21.2402	23.1	195054	{}	\N	2020-10-12 16:13:38.512224
71993	TA	2020-10-08	23.31	23.9	22.405	23.71	329596	{}	\N	2020-10-12 16:13:38.517115
71994	TA	2020-10-09	23.94	24.86	23.37	23.92	241694	{}	\N	2020-10-12 16:13:38.522046
71995	TA	2020-10-12	23.9	24.1903	22.57	22.59	61476	{}	\N	2020-10-12 16:13:38.526592
71996	TBB	2020-09-28	26.74	26.7799	26.65	26.68	92013	{}	\N	2020-10-12 16:13:40.512457
71997	TBB	2020-09-29	26.68	26.68	26.53	26.6	140633	{}	\N	2020-10-12 16:13:40.524085
71998	TBB	2020-09-30	26.56	26.65	26.43	26.43	582275	{}	\N	2020-10-12 16:13:40.533809
71999	TBB	2020-10-01	26.52	26.76	26.46	26.69	154798	{}	\N	2020-10-12 16:13:40.540015
72000	TBB	2020-10-02	26.49	26.79	26.49	26.77	152956	{}	\N	2020-10-12 16:13:40.546158
72001	TBB	2020-10-05	26.78	27.06	26.78	27	180064	{}	\N	2020-10-12 16:13:40.551637
72002	TBB	2020-10-06	27	27.17	26.9101	27.16	130066	{}	\N	2020-10-12 16:13:40.557099
72003	TBB	2020-10-07	27.04	27.25	27.04	27.25	176525	{}	\N	2020-10-12 16:13:40.562718
72004	TBB	2020-10-08	27.25	27.3	27.16	27.26	158642	{}	\N	2020-10-12 16:13:40.567749
72005	TBB	2020-10-09	27.29	27.3	27.03	27.14	144146	{}	\N	2020-10-12 16:13:40.573366
72006	TBB	2020-10-12	27.03	27.09	26.93	27.06	124900	{}	\N	2020-10-12 16:13:40.578845
72007	SRS	2020-09-28	14.95	14.95	14.55	14.68	153096	{}	\N	2020-10-12 16:13:42.766578
72008	SRS	2020-09-29	14.69	15.1	14.67	14.885	221279	{}	\N	2020-10-12 16:13:42.774007
72009	SRS	2020-09-30	14.77	15.07	14.48	14.78	145836	{}	\N	2020-10-12 16:13:42.78004
72010	SRS	2020-10-01	14.73	14.79	14.21	14.21	159254	{}	\N	2020-10-12 16:13:42.786083
72011	SRS	2020-10-02	14.69	14.7	13.69	13.77	207670	{}	\N	2020-10-12 16:13:42.792271
72012	SRS	2020-10-05	13.65	14.09	13.56	13.59	95692	{}	\N	2020-10-12 16:13:42.797833
72013	SRS	2020-10-06	13.55	13.89	13.42	13.8	161002	{}	\N	2020-10-12 16:13:42.803056
72014	SRS	2020-10-07	13.6	13.86	13.6	13.74	84395	{}	\N	2020-10-12 16:13:42.808557
72015	SRS	2020-10-08	13.61	13.61	13.21	13.31	123115	{}	\N	2020-10-12 16:13:42.813478
72016	SRS	2020-10-09	13.19	13.49	13.165	13.42	150537	{}	\N	2020-10-12 16:13:42.818072
72017	SRS	2020-10-12	13.35	13.52	13.29	13.31	63877	{}	\N	2020-10-12 16:13:42.822991
72018	STL	2020-09-28	10.6049	10.795	10.56	10.62	2817914	{}	\N	2020-10-12 16:13:46.400489
72019	STL	2020-09-29	10.56	10.6	10.115	10.33	5908570	{}	\N	2020-10-12 16:13:46.413804
72020	STL	2020-09-30	10.41	10.7501	10.33	10.52	4181746	{}	\N	2020-10-12 16:13:46.419931
72021	STL	2020-10-01	10.67	10.77	10.44	10.69	6039533	{}	\N	2020-10-12 16:13:46.425899
72022	STL	2020-10-02	10.4	11.19	10.33	11.16	4217742	{}	\N	2020-10-12 16:13:46.431644
72023	STL	2020-10-05	11.4	11.82	11.4	11.76	4249621	{}	\N	2020-10-12 16:13:46.436743
72024	STL	2020-10-06	11.99	12.26	11.59	11.65	6085242	{}	\N	2020-10-12 16:13:46.441338
72025	STL	2020-10-07	11.89	12.41	11.89	12.06	6768565	{}	\N	2020-10-12 16:13:46.44595
72026	STL	2020-10-08	12.27	12.48	12.12	12.35	5487170	{}	\N	2020-10-12 16:13:46.450094
72027	STL	2020-10-09	12.5	12.53	12.07	12.07	3084913	{}	\N	2020-10-12 16:13:46.453957
72028	STL	2020-10-12	12.04	12.31	12	12.25	1598105	{}	\N	2020-10-12 16:13:46.457981
72029	STOT	2020-09-28	49.725	49.735	49.72	49.735	2570	{}	\N	2020-10-12 16:13:47.475031
72030	STOT	2020-09-29	49.73	49.8	49.73	49.7612	11383	{}	\N	2020-10-12 16:13:47.483997
72031	STOT	2020-09-30	49.769	49.84	49.75	49.75	4029	{}	\N	2020-10-12 16:13:47.491084
72032	STOT	2020-10-01	49.74	49.74	49.69	49.7091	14534	{}	\N	2020-10-12 16:13:47.496808
72033	STOT	2020-10-02	49.71	49.7398	49.7	49.7219	2742	{}	\N	2020-10-12 16:13:47.502786
72034	STOT	2020-10-05	49.715	49.72	49.7002	49.7042	3415	{}	\N	2020-10-12 16:13:47.507929
72035	STOT	2020-10-06	49.72	49.76	49.7	49.7	63937	{}	\N	2020-10-12 16:13:47.512813
72036	STOT	2020-10-07	49.7272	49.78	49.72	49.77	155631	{}	\N	2020-10-12 16:13:47.518089
72037	STOT	2020-10-08	49.79	49.79	49.735	49.735	5225	{}	\N	2020-10-12 16:13:47.523194
72038	STOT	2020-10-09	49.78	49.83	49.775	49.79	21959	{}	\N	2020-10-12 16:13:47.527988
72039	STOT	2020-10-12	49.81	49.81	49.78	49.78	2849	{}	\N	2020-10-12 16:13:47.532898
72040	STWD	2020-09-28	15.64	16.145	15.64	15.95	2795075	{}	\N	2020-10-12 16:13:48.461175
72041	STWD	2020-09-29	15.45	15.5254	14.925	15.1	4244580	{}	\N	2020-10-12 16:13:48.477407
72042	STWD	2020-09-30	15.17	15.35	14.9734	15.09	3771333	{}	\N	2020-10-12 16:13:48.486665
72043	STWD	2020-10-01	15.2	15.28	14.93	15.22	3489136	{}	\N	2020-10-12 16:13:48.493835
72044	STWD	2020-10-02	14.9	15.555	14.79	15.53	6491716	{}	\N	2020-10-12 16:13:48.500737
72045	STWD	2020-10-05	15.68	15.78	15.43	15.55	4255302	{}	\N	2020-10-12 16:13:48.505915
72046	STWD	2020-10-06	15.65	15.81	15.281	15.33	4492799	{}	\N	2020-10-12 16:13:48.511348
72047	STWD	2020-10-07	15.42	15.57	15.22	15.37	3455388	{}	\N	2020-10-12 16:13:48.516518
72048	STWD	2020-10-08	15.48	15.93	15.45	15.93	3908808	{}	\N	2020-10-12 16:13:48.522137
72049	STWD	2020-10-09	16	16.08	15.5543	15.63	2563885	{}	\N	2020-10-12 16:13:48.52747
72050	STWD	2020-10-12	15.59	15.77	15.54	15.73	1172713	{}	\N	2020-10-12 16:13:48.532785
72051	SVBI	2020-09-28	6.1	6.14	6.09	6.09	2600	{}	\N	2020-10-12 16:13:49.732124
72052	SVBI	2020-09-29	6.01	6.18	6.01	6.12	6493	{}	\N	2020-10-12 16:13:49.739922
72053	SVBI	2020-09-30	6.13	6.49	6.13	6.49	15137	{}	\N	2020-10-12 16:13:49.745468
72054	SVBI	2020-10-01	6.39	6.48	6.36	6.4119	2852	{}	\N	2020-10-12 16:13:49.75072
72055	SVBI	2020-10-05	6.3	6.47	6.3	6.35	6629	{}	\N	2020-10-12 16:13:49.755663
72056	SVBI	2020-10-06	6.25	6.25	6.25	6.25	1120	{}	\N	2020-10-12 16:13:49.760214
72057	SVBI	2020-10-07	6.29	6.49	6.28	6.49	2741	{}	\N	2020-10-12 16:13:49.764628
72058	SVBI	2020-10-08	6.35	6.49	6.31	6.47	1062	{}	\N	2020-10-12 16:13:49.768826
72059	SVBI	2020-10-09	6.3	6.48	6.3	6.48	1350	{}	\N	2020-10-12 16:13:49.772918
72060	SWK	2020-09-28	162.53	165.4181	161.79	164.07	1079565	{}	\N	2020-10-12 16:13:50.700483
72061	SWK	2020-09-29	164.2	164.525	160.94	161.48	1834396	{}	\N	2020-10-12 16:13:50.708094
72062	SWK	2020-09-30	162.23	165.08	161.02	162.2	1760493	{}	\N	2020-10-12 16:13:50.713916
72063	SWK	2020-10-01	163.58	165.42	161.2	162.63	1502186	{}	\N	2020-10-12 16:13:50.720199
72064	SWK	2020-10-02	160.03	167.46	158.92	164.81	1385423	{}	\N	2020-10-12 16:13:50.725918
72065	SWK	2020-10-05	166.99	169.77	164.67	169.31	1464198	{}	\N	2020-10-12 16:13:50.731073
72066	SWK	2020-10-06	169.8	170.74	166.74	166.86	2093747	{}	\N	2020-10-12 16:13:50.735932
72067	SWK	2020-10-07	169.68	172.325	169.215	172	1609597	{}	\N	2020-10-12 16:13:50.740909
72068	SWK	2020-10-08	173	174.52	171.93	173.59	1473673	{}	\N	2020-10-12 16:13:50.746048
72069	SWK	2020-10-09	175	179.43	174.45	176.66	3314553	{}	\N	2020-10-12 16:13:50.750946
72070	SWK	2020-10-12	176.56	179.5	176.56	177.57	954927	{}	\N	2020-10-12 16:13:50.755865
72071	SYKE	2020-09-28	33.72	34.205	33.64	34	219498	{}	\N	2020-10-12 16:13:54.011415
72072	SYKE	2020-09-29	34.04	34.39	33.02	34.37	169269	{}	\N	2020-10-12 16:13:54.022297
72073	SYKE	2020-09-30	34.48	34.9	33.93	34.21	281345	{}	\N	2020-10-12 16:13:54.031671
72074	SYKE	2020-10-01	34.34	34.78	33.915	34.75	188119	{}	\N	2020-10-12 16:13:54.039526
72075	SYKE	2020-10-02	34.09	35.24	33.81	35.01	203727	{}	\N	2020-10-12 16:13:54.04452
72076	SYKE	2020-10-05	35.33	35.52	34.49	35.1	151533	{}	\N	2020-10-12 16:13:54.049744
72077	SYKE	2020-10-06	35.37	35.58	34.72	34.85	223205	{}	\N	2020-10-12 16:13:54.055077
72078	SYKE	2020-10-07	35.2	35.3	34.74	35.18	240452	{}	\N	2020-10-12 16:13:54.059943
72079	SYKE	2020-10-08	34	35.86	34	35.62	156970	{}	\N	2020-10-12 16:13:54.064634
72080	SYKE	2020-10-09	36.07	36.23	35.57	35.92	154479	{}	\N	2020-10-12 16:13:54.069643
72081	SYKE	2020-10-12	36.18	36.24	35.0001	35.95	26203	{}	\N	2020-10-12 16:13:54.074606
72082	SYNL	2020-09-28	5.71	5.85	5.71	5.82	1604	{}	\N	2020-10-12 16:13:55.844911
72083	SYNL	2020-09-29	5.8339	5.895	5.79	5.79	1860	{}	\N	2020-10-12 16:13:55.855725
72084	SYNL	2020-09-30	5.815	5.815	5.52	5.52	16522	{}	\N	2020-10-12 16:13:55.863142
72085	SYNL	2020-10-01	5.745	5.745	5.45	5.47	7571	{}	\N	2020-10-12 16:13:55.868284
72086	SYNL	2020-10-02	5.435	5.82	5.435	5.67	12921	{}	\N	2020-10-12 16:13:55.874344
72087	SYNL	2020-10-05	5.555	5.6	4.66	4.69	158672	{}	\N	2020-10-12 16:13:55.878889
72088	SYNL	2020-10-06	4.92	4.9498	3.81	3.98	331165	{}	\N	2020-10-12 16:13:55.88318
72089	SYNL	2020-10-07	4.01	4.2	4.0005	4.12	63004	{}	\N	2020-10-12 16:13:55.887353
72090	SYNL	2020-10-08	4.1	4.4	4.0601	4.4	65598	{}	\N	2020-10-12 16:13:55.89247
72091	SYNL	2020-10-09	4.41	4.85	4.41	4.755	84413	{}	\N	2020-10-12 16:13:55.897132
72092	SYNL	2020-10-12	4.84	4.84	4.55	4.6	14590	{}	\N	2020-10-12 16:13:55.901139
72093	SRCL	2020-09-28	62.92	63.94	62.92	63.61	314247	{}	\N	2020-10-12 16:13:56.837485
72094	SRCL	2020-09-29	63.24	64.09	62.2398	62.87	291695	{}	\N	2020-10-12 16:13:56.846788
72095	SRCL	2020-09-30	63.27	63.9227	60.11	63.06	320164	{}	\N	2020-10-12 16:13:56.853071
72096	SRCL	2020-10-01	63.18	63.92	62.8	63.34	348663	{}	\N	2020-10-12 16:13:56.860783
72097	SRCL	2020-10-02	62.02	64.6	61.94	64.06	234857	{}	\N	2020-10-12 16:13:56.867619
72098	SRCL	2020-10-05	64.82	65.09	63.74	64.26	492074	{}	\N	2020-10-12 16:13:56.873582
72099	SRCL	2020-10-06	64.44	64.99	62.555	62.91	535882	{}	\N	2020-10-12 16:13:56.878296
72100	SRCL	2020-10-07	63.36	64.255	62.71	63.5	458267	{}	\N	2020-10-12 16:13:56.883065
72101	SRCL	2020-10-08	63.93	64.54	63.45	64.49	276946	{}	\N	2020-10-12 16:13:56.887848
72102	SRCL	2020-10-09	64.97	65.76	64.44	65.44	258457	{}	\N	2020-10-12 16:13:56.892916
72103	SRCL	2020-10-12	65.35	66.97	65.2	66.75	70112	{}	\N	2020-10-12 16:13:56.897594
72104	SSG	2020-09-28	21.45	21.56	20.82	20.83	29318	{}	\N	2020-10-12 16:13:57.859211
72105	SSG	2020-09-29	20.83	20.83	20.335	20.56	15340	{}	\N	2020-10-12 16:13:57.871262
72106	SSG	2020-09-30	20.7499	20.7499	19.9998	20.49	20970	{}	\N	2020-10-12 16:13:57.8796
72107	SSG	2020-10-01	19.78	19.99	19.6171	19.65	47735	{}	\N	2020-10-12 16:13:57.888033
72108	SSG	2020-10-02	20.72	20.9	20.07	20.9	57684	{}	\N	2020-10-12 16:13:57.894758
72109	SSG	2020-10-05	20.2693	20.2693	19.4701	19.4701	14902	{}	\N	2020-10-12 16:13:57.900869
72110	SSG	2020-10-06	19.36	19.73	18.8156	19.685	32000	{}	\N	2020-10-12 16:13:57.907308
72111	SSG	2020-10-07	18.9199	18.9599	18.7484	18.7484	8903	{}	\N	2020-10-12 16:13:57.912739
72112	SSG	2020-10-08	18.5	18.7	18.43	18.581	33777	{}	\N	2020-10-12 16:13:57.917557
72113	SSG	2020-10-09	18.219	18.2569	18.1387	18.2569	16494	{}	\N	2020-10-12 16:13:57.923032
72114	SSG	2020-10-12	17.86	17.86	17.62	17.63	27410	{}	\N	2020-10-12 16:13:57.928769
72115	SSRM	2020-09-28	18.71	18.93	18.07	18.16	1943027	{}	\N	2020-10-12 16:13:58.924187
72116	SSRM	2020-09-29	18.21	18.57	17.96	18.56	2395314	{}	\N	2020-10-12 16:13:58.933257
72117	SSRM	2020-09-30	18.44	19.03	18.25	18.67	2353675	{}	\N	2020-10-12 16:13:58.939605
72118	SSRM	2020-10-01	18.85	19.105	18.7	18.88	1562203	{}	\N	2020-10-12 16:13:58.945566
72119	SSRM	2020-10-02	18.67	18.79	18.26	18.31	1186603	{}	\N	2020-10-12 16:13:58.951856
72120	SSRM	2020-10-05	18.47	18.9252	18.39	18.68	1284405	{}	\N	2020-10-12 16:13:58.955934
72121	SSRM	2020-10-06	18.79	18.805	17.63	17.7	2154283	{}	\N	2020-10-12 16:13:58.959874
72122	SSRM	2020-10-07	18	18.02	17.6501	17.84	1145171	{}	\N	2020-10-12 16:13:58.964081
72123	SSRM	2020-10-08	18.04	18.65	18.04	18.37	1765875	{}	\N	2020-10-12 16:13:58.968222
72124	SSRM	2020-10-09	18.8	19.32	18.76	19.3	1993475	{}	\N	2020-10-12 16:13:58.972258
72125	SSRM	2020-10-12	19.21	19.7	19.05	19.6951	533444	{}	\N	2020-10-12 16:13:58.976817
72126	STAF	2020-09-28	0.6797	0.73	0.5701	0.64	2253127	{}	\N	2020-10-12 16:13:59.908292
72127	STAF	2020-09-29	0.62	0.67	0.561	0.597899	242537	{}	\N	2020-10-12 16:13:59.919785
72128	STAF	2020-09-30	0.575	0.6599	0.561	0.6056	260662	{}	\N	2020-10-12 16:13:59.927364
72129	STAF	2020-10-01	0.5612	0.65	0.5611	0.6343	679394	{}	\N	2020-10-12 16:13:59.933895
72130	STAF	2020-10-02	0.6	0.63	0.5712	0.6219	166586	{}	\N	2020-10-12 16:13:59.941022
72131	STAF	2020-10-05	0.573	0.6294	0.573	0.6017	48031	{}	\N	2020-10-12 16:13:59.948167
72132	STAF	2020-10-06	0.6008	0.62	0.6008	0.6134	52223	{}	\N	2020-10-12 16:13:59.955275
72133	STAF	2020-10-07	0.6101	0.6214	0.5812	0.62	15386	{}	\N	2020-10-12 16:13:59.961476
72134	STAF	2020-10-08	0.6137	0.629	0.60001	0.629	32688	{}	\N	2020-10-12 16:13:59.967404
72135	STAF	2020-10-09	0.615	0.64	0.615	0.62	45617	{}	\N	2020-10-12 16:13:59.973107
72136	STAF	2020-10-12	0.625	0.63	0.6	0.628	23081	{}	\N	2020-10-12 16:13:59.979177
72137	STFC	2020-09-28	13.76	14.15	13.63	13.91	20630	{}	\N	2020-10-12 16:14:01.074554
72138	STFC	2020-09-29	13.96	13.96	13.5	13.86	24095	{}	\N	2020-10-12 16:14:01.082763
72139	STFC	2020-09-30	13.98	14	13.51	13.76	36795	{}	\N	2020-10-12 16:14:01.089426
72140	STFC	2020-10-01	13.8	13.9	13.52	13.66	22248	{}	\N	2020-10-12 16:14:01.095793
72141	STFC	2020-10-02	13.4	13.62	13.37	13.43	42558	{}	\N	2020-10-12 16:14:01.102064
72142	STFC	2020-10-05	13.6	14.2	13.6	13.8	51656	{}	\N	2020-10-12 16:14:01.107807
72143	STFC	2020-10-06	13.87	14.0546	13.6	13.63	36411	{}	\N	2020-10-12 16:14:01.113351
72144	STFC	2020-10-07	13.7817	13.7817	13.58	13.59	36598	{}	\N	2020-10-12 16:14:01.119179
72145	STFC	2020-10-08	13.8	13.81	13.535	13.55	40354	{}	\N	2020-10-12 16:14:01.126417
72146	STFC	2020-10-09	13.75	14.03	13.67	13.72	40196	{}	\N	2020-10-12 16:14:01.13353
72147	STFC	2020-10-12	14.01	14.04	13.68	13.77	19131	{}	\N	2020-10-12 16:14:01.140212
72148	STON	2020-09-28	0.9988	1.04	0.9589	1	255345	{}	\N	2020-10-12 16:14:03.459416
72149	STON	2020-09-29	0.9908	1.04	0.98	0.99	185865	{}	\N	2020-10-12 16:14:03.476611
72150	STON	2020-09-30	0.97	1	0.9126	0.965	312544	{}	\N	2020-10-12 16:14:03.487135
72151	STON	2020-10-01	0.97	0.9701	0.9301	0.95	215713	{}	\N	2020-10-12 16:14:03.492882
72152	STON	2020-10-02	0.93	0.97	0.93	0.97	150242	{}	\N	2020-10-12 16:14:03.498547
72153	STON	2020-10-05	0.97	1.02	0.97	0.9863	116894	{}	\N	2020-10-12 16:14:03.503724
72154	STON	2020-10-06	0.9972	1.03	0.9722	0.9998	233365	{}	\N	2020-10-12 16:14:03.508833
72155	STON	2020-10-07	1	1.03	0.97	0.9714	111632	{}	\N	2020-10-12 16:14:03.514325
72156	STON	2020-10-08	0.97	1	0.97	0.9847	62832	{}	\N	2020-10-12 16:14:03.519552
72157	STON	2020-10-09	0.977	1.02	0.97	0.9912	167796	{}	\N	2020-10-12 16:14:03.524519
72158	STON	2020-10-12	1	1	0.97	0.9701	29990	{}	\N	2020-10-12 16:14:03.530259
72159	STRT	2020-09-28	19.7	20.085	19.7	20.08	1916	{}	\N	2020-10-12 16:14:04.68245
72160	STRT	2020-09-30	19.63	19.9	19.5	19.9	878	{}	\N	2020-10-12 16:14:04.694321
72161	STRT	2020-10-01	19.975	20.127	19.95	19.95	4509	{}	\N	2020-10-12 16:14:04.705225
72162	STRT	2020-10-02	19.675	20.0763	19.6668	20.02	2767	{}	\N	2020-10-12 16:14:04.713922
72163	STRT	2020-10-05	19.99	21.25	19.99	21.09	6130	{}	\N	2020-10-12 16:14:04.721157
72164	STRT	2020-10-06	21.14	21.6	20.71	21.2	10715	{}	\N	2020-10-12 16:14:04.72788
72165	STRT	2020-10-07	21.2	21.945	21.2	21.74	6685	{}	\N	2020-10-12 16:14:04.733862
72166	STRT	2020-10-08	21.95	22	21.23	22	14248	{}	\N	2020-10-12 16:14:04.739166
72167	STRT	2020-10-09	22.25	22.3	22	22.175	3836	{}	\N	2020-10-12 16:14:04.743985
72168	STRT	2020-10-12	22.12	22.12	22.12	22.12	566	{}	\N	2020-10-12 16:14:04.750009
72169	SUB	2020-09-28	108	108.11	108	108.05	138595	{}	\N	2020-10-12 16:14:05.742448
72170	SUB	2020-09-29	108	108.1	107.96	108.04	129438	{}	\N	2020-10-12 16:14:05.749948
72171	SUB	2020-09-30	108.07	108.15	107.98	108.1	187671	{}	\N	2020-10-12 16:14:05.755386
72172	SUB	2020-10-01	107.9806	108.1	107.855	107.9	219575	{}	\N	2020-10-12 16:14:05.760612
72173	SUB	2020-10-02	107.93	107.93	107.82	107.92	168573	{}	\N	2020-10-12 16:14:05.766132
72174	SUB	2020-10-05	107.87	107.97	107.82	107.84	191859	{}	\N	2020-10-12 16:14:05.771451
72175	SUB	2020-10-06	107.85	107.96	107.82	107.94	168390	{}	\N	2020-10-12 16:14:05.776712
72176	SUB	2020-10-07	107.92	107.96	107.81	107.94	149905	{}	\N	2020-10-12 16:14:05.782704
72177	SUB	2020-10-08	107.89	108	107.8666	107.985	277752	{}	\N	2020-10-12 16:14:05.790323
72178	SUB	2020-10-09	107.96	107.97	107.875	107.96	174598	{}	\N	2020-10-12 16:14:05.797405
72179	SUB	2020-10-12	107.85	107.9302	107.85	107.9001	28485	{}	\N	2020-10-12 16:14:05.804943
72180	SUPN	2020-09-28	20.56	21.4958	20.51	20.78	341999	{}	\N	2020-10-12 16:14:08.60081
72181	SUPN	2020-09-29	20.71	20.94	20.31	20.86	530576	{}	\N	2020-10-12 16:14:08.608324
72182	SUPN	2020-09-30	20.97	21.17	20.55	20.84	393180	{}	\N	2020-10-12 16:14:08.615459
72183	SUPN	2020-10-01	20.86	21.6	20.82	21.46	491667	{}	\N	2020-10-12 16:14:08.621299
72184	SUPN	2020-10-02	21.18	21.4	20.54	20.86	365094	{}	\N	2020-10-12 16:14:08.627401
72185	SUPN	2020-10-05	21.02	21.55	20.89	21.2	299906	{}	\N	2020-10-12 16:14:08.631974
72186	SUPN	2020-10-06	21.33	21.6658	20.59	20.66	420776	{}	\N	2020-10-12 16:14:08.637804
72187	SUPN	2020-10-07	20.76	21.19	20.58	20.875	557083	{}	\N	2020-10-12 16:14:08.643694
72188	SUPN	2020-10-08	21.03	21.63	20.94	21.1	505260	{}	\N	2020-10-12 16:14:08.648454
72189	SUPN	2020-10-09	21.22	21.86	20.9	21.43	277149	{}	\N	2020-10-12 16:14:08.654155
72190	SUPN	2020-10-12	21.54	21.55	21.0101	21.17	96786	{}	\N	2020-10-12 16:14:08.659478
72191	SWM	2020-09-28	29.03	29.28	28.71	28.79	237148	{}	\N	2020-10-12 16:14:09.898503
72192	SWM	2020-09-29	28.83	30.01	28.74	29.72	711906	{}	\N	2020-10-12 16:14:09.909412
72193	SWM	2020-09-30	29.75	30.53	29.75	30.39	599664	{}	\N	2020-10-12 16:14:09.917693
72194	SWM	2020-10-01	30.58	31.09	30.47	31.02	518596	{}	\N	2020-10-12 16:14:09.924072
72195	SWM	2020-10-02	30.53	31.9	30.53	31.63	394550	{}	\N	2020-10-12 16:14:09.929641
72196	SWM	2020-10-05	31.9	32.42	31.83	32.38	273514	{}	\N	2020-10-12 16:14:09.934457
72197	SWM	2020-10-06	32.53	32.99	32.19	32.22	391616	{}	\N	2020-10-12 16:14:09.939103
72198	SWM	2020-10-07	32.57	33.21	32.06	32.73	396602	{}	\N	2020-10-12 16:14:09.943831
72199	SWM	2020-10-08	33	33.49	32.795	33.24	395567	{}	\N	2020-10-12 16:14:09.94999
72200	SWM	2020-10-09	33.1	33.55	32.225	32.26	348733	{}	\N	2020-10-12 16:14:09.955455
72201	SWM	2020-10-12	32.44	33	32.425	32.765	61850	{}	\N	2020-10-12 16:14:09.960451
72202	SXI	2020-09-28	59.78	61.76	59.78	59.78	51732	{}	\N	2020-10-12 16:14:11.832336
72203	SXI	2020-09-29	59.44	59.82	58.45	59.17	118264	{}	\N	2020-10-12 16:14:11.844208
72204	SXI	2020-09-30	59.39	60.75	58.61	59.2	120565	{}	\N	2020-10-12 16:14:11.85572
72205	SXI	2020-10-01	59.49	60.3	58.42	60.13	123712	{}	\N	2020-10-12 16:14:11.863935
72206	SXI	2020-10-02	59.11	62.43	59.11	62.08	92616	{}	\N	2020-10-12 16:14:11.871312
72207	SXI	2020-10-05	62.55	63.72	62.1801	63.61	105517	{}	\N	2020-10-12 16:14:11.878004
72208	SXI	2020-10-06	64	65.62	63.41	63.68	152733	{}	\N	2020-10-12 16:14:11.884776
72209	SXI	2020-10-07	64.56	65.05	63.07	63.61	108900	{}	\N	2020-10-12 16:14:11.890812
72210	SXI	2020-10-08	64.47	65.39	63.37	64.87	102006	{}	\N	2020-10-12 16:14:11.898538
72211	SXI	2020-10-09	65.7	65.7	63.91	64.65	59202	{}	\N	2020-10-12 16:14:11.906272
72212	SXI	2020-10-12	64.7	65.1699	63.87	64.23	17723	{}	\N	2020-10-12 16:14:11.912923
72213	SRT	2020-09-28	5.48	5.67	5.48	5.62	14094	{}	\N	2020-10-12 16:14:16.291589
72214	SRT	2020-09-29	5.57	5.63	5.44	5.45	17628	{}	\N	2020-10-12 16:14:16.301886
72215	SRT	2020-09-30	5.41	5.53	5.21	5.25	26423	{}	\N	2020-10-12 16:14:16.309697
72216	SRT	2020-10-01	5.23	5.35	5.18	5.23	65919	{}	\N	2020-10-12 16:14:16.316201
72217	SRT	2020-10-02	5.09	5.35	5.06	5.22	49934	{}	\N	2020-10-12 16:14:16.322166
72218	SRT	2020-10-05	5.27	5.33	5.095	5.13	60004	{}	\N	2020-10-12 16:14:16.327123
72219	SRT	2020-10-06	5.18	5.4	5.14	5.14	37358	{}	\N	2020-10-12 16:14:16.331822
72220	SRT	2020-10-07	5.15	5.42	5.12	5.28	55473	{}	\N	2020-10-12 16:14:16.336275
72221	SRT	2020-10-08	5.39	5.39	4.95	4.98	106327	{}	\N	2020-10-12 16:14:16.34105
72222	SRT	2020-10-09	4.99	5.02	4.94	4.97	97728	{}	\N	2020-10-12 16:14:16.346503
72223	SRT	2020-10-12	4.95	4.95	4.76	4.795	54048	{}	\N	2020-10-12 16:14:16.351711
72224	SSNT	2020-09-28	2.56	2.8299	2.46	2.56	199469	{}	\N	2020-10-12 16:14:19.583252
72225	SSNT	2020-09-29	2.5	2.8395	2.47	2.61	151737	{}	\N	2020-10-12 16:14:19.594759
72226	SSNT	2020-09-30	2.6	2.7142	2.51	2.65	98502	{}	\N	2020-10-12 16:14:19.602876
72227	SSNT	2020-10-01	2.72	2.72	2.51	2.63	89828	{}	\N	2020-10-12 16:14:19.6114
72228	SSNT	2020-10-02	2.58	2.58	2.46	2.5	93610	{}	\N	2020-10-12 16:14:19.618157
72229	SSNT	2020-10-05	2.52	2.57	2.47	2.47	48335	{}	\N	2020-10-12 16:14:19.624199
72230	SSNT	2020-10-06	2.47	2.63	2.4127	2.51	80848	{}	\N	2020-10-12 16:14:19.631032
72231	SSNT	2020-10-07	2.52	2.64	2.52	2.55	53288	{}	\N	2020-10-12 16:14:19.638121
72232	SSNT	2020-10-08	2.56	2.68	2.55	2.6243	21887	{}	\N	2020-10-12 16:14:19.643791
72233	SSNT	2020-10-09	2.66	3.045	2.6202	2.88	322621	{}	\N	2020-10-12 16:14:19.649766
72234	SSNT	2020-10-12	2.97	3.4	2.9	3.1629	405210	{}	\N	2020-10-12 16:14:19.656159
72235	SSYS	2020-09-28	12.27	12.5	12.11	12.14	558994	{}	\N	2020-10-12 16:14:23.09774
72236	SSYS	2020-09-29	12.16	12.65	12.08	12.63	1069905	{}	\N	2020-10-12 16:14:23.106265
72237	SSYS	2020-09-30	12.65	12.95	12.38	12.47	882608	{}	\N	2020-10-12 16:14:23.115902
72238	SSYS	2020-10-01	12.51	12.75	12.4158	12.51	817322	{}	\N	2020-10-12 16:14:23.122917
72239	SSYS	2020-10-02	12.14	12.48	12.12	12.36	1385471	{}	\N	2020-10-12 16:14:23.130163
72240	SSYS	2020-10-05	12.5	13.14	12.49	12.87	616473	{}	\N	2020-10-12 16:14:23.137613
72241	SSYS	2020-10-06	12.92	13.71	12.91	13.38	1017942	{}	\N	2020-10-12 16:14:23.1445
72242	SSYS	2020-10-07	13.48	14.37	13.34	14.13	1892597	{}	\N	2020-10-12 16:14:23.151962
72243	SSYS	2020-10-08	14.33	14.5	13.93	13.99	908735	{}	\N	2020-10-12 16:14:23.160553
72244	SSYS	2020-10-09	14.4205	15.24	14.4	14.86	1449540	{}	\N	2020-10-12 16:14:23.167597
72245	SSYS	2020-10-12	15.12	15.28	14.51	14.79	558090	{}	\N	2020-10-12 16:14:23.173246
72246	STLD	2020-09-28	29.59	30.39	29.37	29.98	2311806	{}	\N	2020-10-12 16:14:25.481274
72247	STLD	2020-09-29	29.46	29.695	28.53	28.66	1478137	{}	\N	2020-10-12 16:14:25.494308
72248	STLD	2020-09-30	28.87	29.14	28.36	28.63	1810585	{}	\N	2020-10-12 16:14:25.503056
72249	STLD	2020-10-01	28.68	29.31	28.68	29.09	1331829	{}	\N	2020-10-12 16:14:25.509685
72250	STLD	2020-10-02	28.42	30	28.3105	29.7	1675607	{}	\N	2020-10-12 16:14:25.515619
72251	STLD	2020-10-05	30	31.17	29.7	30.99	1626023	{}	\N	2020-10-12 16:14:25.521377
72252	STLD	2020-10-06	31.49	31.695	30.63	30.87	1943617	{}	\N	2020-10-12 16:14:25.5268
72253	STLD	2020-10-07	31.42	32.15	31.3	31.94	1461440	{}	\N	2020-10-12 16:14:25.532157
72254	STLD	2020-10-08	32.2	32.94	31.9	32.84	2365477	{}	\N	2020-10-12 16:14:25.537378
72255	STLD	2020-10-09	33.14	33.27	32.05	32.58	2143158	{}	\N	2020-10-12 16:14:25.542272
72256	STLD	2020-10-12	32.58	33.37	32.48	33.185	670524	{}	\N	2020-10-12 16:14:25.547352
72257	STRM	2020-09-28	1.43	1.5276	1.38	1.51	35111	{}	\N	2020-10-12 16:14:27.794536
72258	STRM	2020-09-29	1.5	1.55	1.5	1.52	19300	{}	\N	2020-10-12 16:14:27.80576
72259	STRM	2020-09-30	1.52	1.6	1.52	1.56	27753	{}	\N	2020-10-12 16:14:27.813389
72260	STRM	2020-10-01	1.5922	1.5922	1.51	1.54	7817	{}	\N	2020-10-12 16:14:27.82041
72261	STRM	2020-10-02	1.59	1.63	1.44	1.445	46725	{}	\N	2020-10-12 16:14:27.826346
72262	STRM	2020-10-05	1.52	1.54	1.41	1.43	44352	{}	\N	2020-10-12 16:14:27.831346
72263	STRM	2020-10-06	1.5	1.5399	1.42	1.42	20746	{}	\N	2020-10-12 16:14:27.836925
72264	STRM	2020-10-07	1.46	1.46	1.43	1.44	7599	{}	\N	2020-10-12 16:14:27.842646
72265	STRM	2020-10-08	1.44	1.4462	1.28	1.36	74493	{}	\N	2020-10-12 16:14:27.847905
72266	STRM	2020-10-09	1.36	1.4099	1.36	1.39	9509	{}	\N	2020-10-12 16:14:27.853233
72267	STRM	2020-10-12	1.37	1.42	1.37	1.4156	12134	{}	\N	2020-10-12 16:14:27.858782
72268	SUMR	2020-09-28	17.45	17.5	17.015	17.1584	4239	{}	\N	2020-10-12 16:14:29.816843
72269	SUMR	2020-09-29	17.39	17.39	16.29	16.675	7097	{}	\N	2020-10-12 16:14:29.828932
72270	SUMR	2020-09-30	16.87	17.29	16.87	17.29	3399	{}	\N	2020-10-12 16:14:29.837622
72271	SUMR	2020-10-01	17.2	17.2	17	17.2	1838	{}	\N	2020-10-12 16:14:29.84595
72272	SUMR	2020-10-02	17.4	17.4	16.954	17.34	6848	{}	\N	2020-10-12 16:14:29.852047
72273	SUMR	2020-10-05	17.34	17.9898	17.01	17.25	2866	{}	\N	2020-10-12 16:14:29.85715
72274	SUMR	2020-10-06	17.2663	17.2663	17.07	17.07	905	{}	\N	2020-10-12 16:14:29.862053
72275	SUMR	2020-10-07	17.39	17.48	17.2114	17.48	2833	{}	\N	2020-10-12 16:14:29.866829
72276	SUMR	2020-10-08	17.4	17.49	17.22	17.22	4816	{}	\N	2020-10-12 16:14:29.871843
72277	SUMR	2020-10-09	17.175	17.175	16.95	17.11	2341	{}	\N	2020-10-12 16:14:29.876558
72278	SUMR	2020-10-12	16.95	17.07	16.25	16.4	7500	{}	\N	2020-10-12 16:14:29.881242
72279	SVRA	2020-09-28	1.21	1.22	1.16	1.18	104433	{}	\N	2020-10-12 16:14:31.072164
72280	SVRA	2020-09-29	1.17	1.2	1.09	1.09	240364	{}	\N	2020-10-12 16:14:31.081273
72281	SVRA	2020-09-30	1.12	1.13	1.09	1.09	120040	{}	\N	2020-10-12 16:14:31.088825
72282	SVRA	2020-10-01	1.08	1.11	1.08	1.08	117943	{}	\N	2020-10-12 16:14:31.095491
72283	SVRA	2020-10-02	1.09	1.12	1.08	1.09	150792	{}	\N	2020-10-12 16:14:31.101469
72284	SVRA	2020-10-05	1.11	1.14	1.09	1.095	225051	{}	\N	2020-10-12 16:14:31.10679
72285	SVRA	2020-10-06	1.11	1.13	1.08	1.1	430392	{}	\N	2020-10-12 16:14:31.11151
72286	SVRA	2020-10-07	1.1	1.22	1.1	1.21	259969	{}	\N	2020-10-12 16:14:31.115872
72287	SVRA	2020-10-08	1.19	1.28	1.19	1.25	185551	{}	\N	2020-10-12 16:14:31.120269
72288	SVRA	2020-10-09	1.29	1.3337	1.22	1.27	152507	{}	\N	2020-10-12 16:14:31.124638
72289	SVRA	2020-10-12	1.27	1.29	1.27	1.28	34315	{}	\N	2020-10-12 16:14:31.128967
72290	SYK	2020-09-28	209.02	210.15	207.01	207.25	1192352	{}	\N	2020-10-12 16:14:32.799706
72291	SYK	2020-09-29	207.99	208.86	203.45	204.74	1562661	{}	\N	2020-10-12 16:14:32.81191
72292	SYK	2020-09-30	205.22	210.67	204.78	208.37	2193977	{}	\N	2020-10-12 16:14:32.819596
72293	SYK	2020-10-01	209.84	210.24	206.33	208.67	1988488	{}	\N	2020-10-12 16:14:32.826441
72294	SYK	2020-10-02	205.85	210.93	204.45	208.3	1870272	{}	\N	2020-10-12 16:14:32.832663
72295	SYK	2020-10-05	210.54	214.1	210.37	214.06	2014006	{}	\N	2020-10-12 16:14:32.838261
72296	SYK	2020-10-06	213.8	214.81	210.02	210.59	1580017	{}	\N	2020-10-12 16:14:32.844122
72297	SYK	2020-10-07	213.43	218.12	213.18	217.4	2015830	{}	\N	2020-10-12 16:14:32.849587
72298	SYK	2020-10-08	218	220.1	216.81	219.09	2204514	{}	\N	2020-10-12 16:14:32.855949
72299	SYK	2020-10-09	220	224.73	219.62	222.93	2444550	{}	\N	2020-10-12 16:14:32.862015
72300	SYK	2020-10-12	225	226	223.6	224.76	1054603	{}	\N	2020-10-12 16:14:32.867527
72301	SYPR	2020-09-28	1.16	1.23	1.11	1.19	1477922	{}	\N	2020-10-12 16:14:37.722621
72302	SYPR	2020-09-29	1.14	1.18	1.12	1.14	383499	{}	\N	2020-10-12 16:14:37.731777
72303	SYPR	2020-09-30	1.18	1.18	1.07	1.11	259465	{}	\N	2020-10-12 16:14:37.738425
72304	SYPR	2020-10-01	1.07	1.1099	1.06	1.08	210237	{}	\N	2020-10-12 16:14:37.743498
72305	SYPR	2020-10-02	1.06	1.12	1.03	1.09	140064	{}	\N	2020-10-12 16:14:37.749165
72306	SYPR	2020-10-05	1.07	1.13	1.06	1.06	133981	{}	\N	2020-10-12 16:14:37.755108
72307	SYPR	2020-10-06	1.06	1.12	1.06	1.08	187775	{}	\N	2020-10-12 16:14:37.760113
72308	SYPR	2020-10-07	1.09	1.21	1.08	1.18	298931	{}	\N	2020-10-12 16:14:37.765471
72309	SYPR	2020-10-08	1.17	1.19	1.11	1.16	209731	{}	\N	2020-10-12 16:14:37.770834
72310	SYPR	2020-10-09	1.16	1.25	1.16	1.22	192908	{}	\N	2020-10-12 16:14:37.776595
72311	SYPR	2020-10-12	1.2	1.25	1.18	1.24	162052	{}	\N	2020-10-12 16:14:37.782328
72312	TACT	2020-09-28	5	5.02	4.9334	4.95	2697	{}	\N	2020-10-12 16:14:38.949379
72313	TACT	2020-09-29	4.9	5.03	4.8899	5.03	3179	{}	\N	2020-10-12 16:14:38.96387
72314	TACT	2020-09-30	5.115	5.2	5	5	2726	{}	\N	2020-10-12 16:14:38.973024
72315	TACT	2020-10-01	4.95	5.21	4.93	5.08	10886	{}	\N	2020-10-12 16:14:38.979582
72316	TACT	2020-10-02	5	5.31	4.99	5.11	5024	{}	\N	2020-10-12 16:14:38.98575
72317	TACT	2020-10-05	5.29	5.29	5.1	5.1	4266	{}	\N	2020-10-12 16:14:38.991046
72318	TACT	2020-10-06	5.44	7.41	5.32	6.75	655796	{}	\N	2020-10-12 16:14:38.996425
72319	TACT	2020-10-07	6.75	7.17	6.61	7.11	90357	{}	\N	2020-10-12 16:14:39.001757
72320	TACT	2020-10-08	7.2	7.49	7.1264	7.48	100820	{}	\N	2020-10-12 16:14:39.007212
72321	TACT	2020-10-09	7.24	7.84	6.91	7.62	56381	{}	\N	2020-10-12 16:14:39.01251
72322	TACT	2020-10-12	7.67	7.67	7.37	7.45	12199	{}	\N	2020-10-12 16:14:39.017959
72323	TAST	2020-09-28	6.39	6.8	6.39	6.75	513854	{}	\N	2020-10-12 16:14:40.168221
72324	TAST	2020-09-29	6.65	6.81	6.53	6.69	325761	{}	\N	2020-10-12 16:14:40.179707
72325	TAST	2020-09-30	6.73	6.9039	6.34	6.45	297750	{}	\N	2020-10-12 16:14:40.187208
72326	TAST	2020-10-01	6.46	6.67	6.41	6.62	223641	{}	\N	2020-10-12 16:14:40.193908
72327	TAST	2020-10-02	6.46	6.83	6.42	6.81	247477	{}	\N	2020-10-12 16:14:40.199977
72328	TAST	2020-10-05	6.79	7	6.78	6.94	334899	{}	\N	2020-10-12 16:14:40.204895
72329	TAST	2020-10-06	6.98	7.14	6.73	6.73	409652	{}	\N	2020-10-12 16:14:40.210108
72330	TAST	2020-10-07	6.83	6.96	6.79	6.91	212084	{}	\N	2020-10-12 16:14:40.215014
72331	TAST	2020-10-08	6.97	7.08	6.915	7	296232	{}	\N	2020-10-12 16:14:40.219889
72332	TAST	2020-10-09	7.11	7.11	6.84	6.85	161617	{}	\N	2020-10-12 16:14:40.224888
72333	TAST	2020-10-12	7.32	7.57	7.21	7.315	403848	{}	\N	2020-10-12 16:14:40.229855
72334	SRNE	2020-09-28	10.18	10.489	9.84	10.15	22754782	{}	\N	2020-10-12 16:14:41.723845
72335	SRNE	2020-09-29	10.71	12.18	10.45	11.6	76329388	{}	\N	2020-10-12 16:14:41.73299
72336	SRNE	2020-09-30	11.6	11.91	10.82	11.15	32313372	{}	\N	2020-10-12 16:14:41.739154
72337	SRNE	2020-10-01	10.55	10.94	10.34	10.69	16928635	{}	\N	2020-10-12 16:14:41.745929
72338	SRNE	2020-10-02	10.6	10.89	10.12	10.38	17370165	{}	\N	2020-10-12 16:14:41.751431
72339	SRNE	2020-10-05	10.55	11.16	10.35	10.87	16489889	{}	\N	2020-10-12 16:14:41.756467
72340	SRNE	2020-10-06	10.8	11.19	10.55	10.61	12691265	{}	\N	2020-10-12 16:14:41.76099
72341	SRNE	2020-10-07	10.8	11.65	10.71	11.58	24087929	{}	\N	2020-10-12 16:14:41.76536
72342	SRNE	2020-10-08	11.63	11.69	11.04	11.175	18429146	{}	\N	2020-10-12 16:14:41.769423
72343	SRNE	2020-10-09	11.03	11.475	10.83	11.25	12123146	{}	\N	2020-10-12 16:14:41.773861
72344	SRNE	2020-10-12	11.28	11.37	11.03	11.33	6331131	{}	\N	2020-10-12 16:14:41.778125
72345	SRV	2020-09-28	15.5	15.99	15.4999	15.84	8068	{}	\N	2020-10-12 16:14:43.811899
72346	SRV	2020-09-29	15.74	16.04	15.6614	15.68	15764	{}	\N	2020-10-12 16:14:43.821548
72347	SRV	2020-09-30	15.7	16.01	15.51	15.51	22482	{}	\N	2020-10-12 16:14:43.828974
72348	SRV	2020-10-01	15.46	15.6	15.45	15.5111	24596	{}	\N	2020-10-12 16:14:43.835575
72349	SRV	2020-10-02	15.35	15.74	15.35	15.73	23682	{}	\N	2020-10-12 16:14:43.842501
72350	SRV	2020-10-05	15.84	16.48	15.84	16.16	16296	{}	\N	2020-10-12 16:14:43.849873
72351	SRV	2020-10-06	16.29	16.89	16.29	16.52	28035	{}	\N	2020-10-12 16:14:43.856939
72352	SRV	2020-10-07	16.54	16.86	16.51	16.7535	20628	{}	\N	2020-10-12 16:14:43.86329
72353	SRV	2020-10-08	16.85	17.59	16.85	17.49	40910	{}	\N	2020-10-12 16:14:43.869577
72354	SRV	2020-10-09	17.56	17.62	17.38	17.39	18441	{}	\N	2020-10-12 16:14:43.877303
72355	SRV	2020-10-12	17.32	17.47	17.31	17.345	5552	{}	\N	2020-10-12 16:14:43.884029
72356	SSO	2020-09-28	72.79	73.53	72.33	73.22	2040292	{}	\N	2020-10-12 16:14:45.659074
72357	SSO	2020-09-29	73.11	73.45	72.07	72.43	1712440	{}	\N	2020-10-12 16:14:45.670409
72358	SSO	2020-09-30	72.74	74.98	72.6499	73.47	5442857	{}	\N	2020-10-12 16:14:45.678372
72359	SSO	2020-10-01	74.7	75.13	73.56	74.4	2087259	{}	\N	2020-10-12 16:14:45.684624
72360	SSO	2020-10-02	72.09	73.92	71.84	73	2537225	{}	\N	2020-10-12 16:14:45.690162
72361	SSO	2020-10-05	73.98	75.67	73.98	75.59	1630127	{}	\N	2020-10-12 16:14:45.694879
72362	SSO	2020-10-06	75.68	76.6559	73.21	73.43	3751124	{}	\N	2020-10-12 16:14:45.699337
72363	SSO	2020-10-07	74.83	76.37	74.83	75.99	1768491	{}	\N	2020-10-12 16:14:45.70421
72364	SSO	2020-10-08	76.91	77.365	76.49	77.32	1525818	{}	\N	2020-10-12 16:14:45.709282
72365	SSO	2020-10-09	78.14	78.94	77.8486	78.73	2241747	{}	\N	2020-10-12 16:14:45.714976
72366	SSO	2020-10-12	79.98	81.1068	79.74	81.07	820742	{}	\N	2020-10-12 16:14:45.719569
72367	ST	2020-09-28	42.57	43.38	42.36	42.89	625603	{}	\N	2020-10-12 16:14:46.916235
72368	ST	2020-09-29	42.92	43.25	42.48	42.69	1648685	{}	\N	2020-10-12 16:14:46.922998
72369	ST	2020-09-30	42.92	43.68	42.885	43.14	1957009	{}	\N	2020-10-12 16:14:46.927926
72370	ST	2020-10-01	43.86	44.47	43.08	43.97	2272909	{}	\N	2020-10-12 16:14:46.932444
72371	ST	2020-10-02	42.79	44.44	42.79	43.6	1892775	{}	\N	2020-10-12 16:14:46.937229
72372	ST	2020-10-05	46.34	46.57	45.33	45.91	3715349	{}	\N	2020-10-12 16:14:46.942769
72373	ST	2020-10-06	46.51	46.69	45.4	45.49	2232359	{}	\N	2020-10-12 16:14:46.948002
72374	ST	2020-10-07	46.18	46.74	45.77	46.26	1456135	{}	\N	2020-10-12 16:14:46.952226
72375	ST	2020-10-08	46.29	47.17	46.26	46.95	1690532	{}	\N	2020-10-12 16:14:46.95706
72376	ST	2020-10-09	47.57	48.16	47.07	47.74	1391334	{}	\N	2020-10-12 16:14:46.961499
72377	ST	2020-10-12	48.13	48.17	47.43	47.945	647253	{}	\N	2020-10-12 16:14:46.965951
72378	STK	2020-09-28	21.69	21.8	21.545	21.7	33634	{}	\N	2020-10-12 16:14:49.411979
72379	STK	2020-09-29	21.62	21.84	21.62	21.7	70969	{}	\N	2020-10-12 16:14:49.420464
72380	STK	2020-09-30	21.8	21.9299	21.67	21.89	102860	{}	\N	2020-10-12 16:14:49.434275
72381	STK	2020-10-01	21.99	22.1	21.93	22.04	100814	{}	\N	2020-10-12 16:14:49.444176
72382	STK	2020-10-02	21.51	21.79	21.31	21.69	100662	{}	\N	2020-10-12 16:14:49.453991
72383	STK	2020-10-05	21.66	22.082	21.66	22.03	164312	{}	\N	2020-10-12 16:14:49.460859
72384	STK	2020-10-06	22.03	22.35	21.8881	22.04	126566	{}	\N	2020-10-12 16:14:49.46836
72385	STK	2020-10-07	22.33	22.55	22.33	22.49	124722	{}	\N	2020-10-12 16:14:49.477114
72386	STK	2020-10-08	22.58	22.77	22.5671	22.71	96836	{}	\N	2020-10-12 16:14:49.483996
72387	STK	2020-10-09	22.75	23.1	22.74	23.02	142508	{}	\N	2020-10-12 16:14:49.491747
72388	STK	2020-10-12	23.02	23.305	23.02	23.3	63437	{}	\N	2020-10-12 16:14:49.503223
72389	STRL	2020-09-28	13.97	14.53	13.97	14.5	126314	{}	\N	2020-10-12 16:14:51.660088
72390	STRL	2020-09-29	14.61	14.61	14.03	14.57	176020	{}	\N	2020-10-12 16:14:51.67066
72391	STRL	2020-09-30	14.64	14.74	14.01	14.16	102932	{}	\N	2020-10-12 16:14:51.677371
72392	STRL	2020-10-01	14.18	14.2933	13.69	13.98	179422	{}	\N	2020-10-12 16:14:51.684094
72393	STRL	2020-10-02	13.67	14.15	13.67	14.03	96007	{}	\N	2020-10-12 16:14:51.689643
72394	STRL	2020-10-05	14.29	14.66	14.23	14.43	131715	{}	\N	2020-10-12 16:14:51.695505
72395	STRL	2020-10-06	14.55	14.67	14.1	14.38	277728	{}	\N	2020-10-12 16:14:51.700271
72396	STRL	2020-10-07	14.51	15.2	14.3772	14.97	193244	{}	\N	2020-10-12 16:14:51.70534
72397	STRL	2020-10-08	15.19	15.43	14.99	15.22	86898	{}	\N	2020-10-12 16:14:51.710944
72398	STRL	2020-10-09	15.31	15.415	14.98	14.99	99900	{}	\N	2020-10-12 16:14:51.715903
72399	STRL	2020-10-12	14.94	15.22	14.8	15.13	25901	{}	\N	2020-10-12 16:14:51.720834
72400	SUP	2020-09-28	1.26	1.29	1.24	1.24	102190	{}	\N	2020-10-12 16:14:53.573841
72401	SUP	2020-09-29	1.35	1.36	1.23	1.27	302360	{}	\N	2020-10-12 16:14:53.585586
72402	SUP	2020-09-30	1.23	1.29	1.22	1.25	178026	{}	\N	2020-10-12 16:14:53.59256
72403	SUP	2020-10-01	1.27	1.27	1.23	1.26	190721	{}	\N	2020-10-12 16:14:53.598441
72404	SUP	2020-10-02	1.23	1.27	1.23	1.26	221494	{}	\N	2020-10-12 16:14:53.603636
72405	SUP	2020-10-05	1.27	1.29	1.23	1.24	390677	{}	\N	2020-10-12 16:14:53.60885
72406	SUP	2020-10-06	1.25	1.26	1.23	1.23	271594	{}	\N	2020-10-12 16:14:53.61436
72407	SUP	2020-10-07	1.23	1.28	1.21	1.21	407813	{}	\N	2020-10-12 16:14:53.620219
72408	SUP	2020-10-08	1.2214	1.28	1.2214	1.26	332180	{}	\N	2020-10-12 16:14:53.625705
72409	SUP	2020-10-09	1.27	1.27	1.24	1.24	190087	{}	\N	2020-10-12 16:14:53.631744
72410	SUP	2020-10-12	1.27	1.27	1.24	1.24	116738	{}	\N	2020-10-12 16:14:53.638117
72411	SVT	2020-09-28	7.76	7.9038	7.76	7.89	3883	{}	\N	2020-10-12 16:14:55.876504
72412	SVT	2020-09-29	8	8.0241	7.8	7.8	5370	{}	\N	2020-10-12 16:14:55.888142
72413	SVT	2020-09-30	7.7575	7.98	7.75	7.79	4084	{}	\N	2020-10-12 16:14:55.897489
72414	SVT	2020-10-01	8	8	7.75	7.895	4668	{}	\N	2020-10-12 16:14:55.903534
72415	SVT	2020-10-02	7.94	7.94	7.9	7.9	545	{}	\N	2020-10-12 16:14:55.909165
72416	SVT	2020-10-05	7.8	7.9	7.8	7.9	5592	{}	\N	2020-10-12 16:14:55.914803
72417	SVT	2020-10-06	8	8.22	7.92	7.92	4693	{}	\N	2020-10-12 16:14:55.920735
72418	SVT	2020-10-08	8.1584	8.725	8.06	8.4755	6757	{}	\N	2020-10-12 16:14:55.925983
72419	SVT	2020-10-09	8.52	8.6264	8.5	8.5001	3314	{}	\N	2020-10-12 16:14:55.931591
72420	SVT	2020-10-12	8.625	8.7499	8.625	8.7499	395	{}	\N	2020-10-12 16:14:55.937406
72421	SXC	2020-09-28	3.43	3.55	3.4	3.5	468708	{}	\N	2020-10-12 16:14:57.195002
72422	SXC	2020-09-29	3.52	3.531	3.395	3.49	827276	{}	\N	2020-10-12 16:14:57.206211
72423	SXC	2020-09-30	3.47	3.57	3.41	3.42	703081	{}	\N	2020-10-12 16:14:57.213835
72424	SXC	2020-10-01	3.41	3.45	3.29	3.36	685594	{}	\N	2020-10-12 16:14:57.222844
72425	SXC	2020-10-02	3.3	3.52	3.27	3.49	973690	{}	\N	2020-10-12 16:14:57.230707
72426	SXC	2020-10-05	3.52	3.69	3.515	3.63	889313	{}	\N	2020-10-12 16:14:57.236393
72427	SXC	2020-10-06	3.71	3.72	3.44	3.45	1068982	{}	\N	2020-10-12 16:14:57.24155
72428	SXC	2020-10-07	3.5	3.57	3.44	3.53	1090506	{}	\N	2020-10-12 16:14:57.246048
72429	SXC	2020-10-08	3.58	3.725	3.56	3.71	704110	{}	\N	2020-10-12 16:14:57.250549
72430	SXC	2020-10-09	3.83	3.83	3.56	3.6	953673	{}	\N	2020-10-12 16:14:57.25485
72431	SXC	2020-10-12	3.64	3.76	3.57	3.685	350926	{}	\N	2020-10-12 16:14:57.259807
72432	SYG	2020-09-28	107.12	107.12	106.92	106.93	582	{}	\N	2020-10-12 16:14:58.824375
72433	SYG	2020-09-29	106.8398	106.8398	106.8398	106.8398	246	{}	\N	2020-10-12 16:14:58.8368
72434	SYG	2020-09-30	107.5	107.8494	107.5	107.8494	625	{}	\N	2020-10-12 16:14:58.844406
72435	SYG	2020-10-01	108.676	108.676	108.676	108.676	32	{}	\N	2020-10-12 16:14:58.851451
72436	SYG	2020-10-02	106.8306	106.8306	106.8306	106.8306	243	{}	\N	2020-10-12 16:14:58.857273
72437	SYG	2020-10-05	107.89	108.8924	107.89	108.8924	808	{}	\N	2020-10-12 16:14:58.863647
72438	SYG	2020-10-06	108.65	108.65	107.152	107.152	431	{}	\N	2020-10-12 16:14:58.871806
72439	SYG	2020-10-07	109.086	109.086	109.086	109.086	227	{}	\N	2020-10-12 16:14:58.880115
72440	SYG	2020-10-08	109.2145	109.2145	109.2145	109.2145	4	{}	\N	2020-10-12 16:14:58.886251
72441	SYG	2020-10-09	110.6171	110.6171	110.5961	110.5961	125	{}	\N	2020-10-12 16:14:58.891776
72442	SYG	2020-10-12	112.0717	112.0717	112.0717	112.0717	807	{}	\N	2020-10-12 16:14:58.896497
72443	SYNH	2020-09-28	52.61	54.48	52.47	53.39	833117	{}	\N	2020-10-12 16:15:01.930572
72444	SYNH	2020-09-29	53.67	54.09	52.04	52.97	624025	{}	\N	2020-10-12 16:15:01.941422
72445	SYNH	2020-09-30	53.07	53.995	52.79	53.16	799732	{}	\N	2020-10-12 16:15:01.949138
72446	SYNH	2020-10-01	55.0757	56.46	54.05	56.155	1390046	{}	\N	2020-10-12 16:15:01.957111
72447	SYNH	2020-10-02	54.94	56.26	54.94	55.86	752528	{}	\N	2020-10-12 16:15:01.962184
72448	SYNH	2020-10-05	56.36	56.98	56.06	56.71	642114	{}	\N	2020-10-12 16:15:01.968754
72449	SYNH	2020-10-06	57.8	57.89	56.13	56.61	539332	{}	\N	2020-10-12 16:15:01.973855
72450	SYNH	2020-10-07	57.27	58.92	56.89	57.95	835012	{}	\N	2020-10-12 16:15:01.978756
72451	SYNH	2020-10-08	58.44	58.54	57.485	58.19	591370	{}	\N	2020-10-12 16:15:01.984188
72452	SYNH	2020-10-09	58.935	58.935	58.13	58.84	381887	{}	\N	2020-10-12 16:15:01.990205
72453	SYNH	2020-10-12	59.05	59.35	58.46	59.08	204496	{}	\N	2020-10-12 16:15:01.99509
72454	SZC	2020-09-28	27.27	27.52	27.04	27.4	8452	{}	\N	2020-10-12 16:15:02.927596
72455	SZC	2020-09-29	27.4	27.6	27.28	27.51	19146	{}	\N	2020-10-12 16:15:02.936453
72456	SZC	2020-09-30	27.71	27.71	26.83	26.83	25082	{}	\N	2020-10-12 16:15:02.942852
72457	SZC	2020-10-01	27.02	27.41	26.88	26.91	31190	{}	\N	2020-10-12 16:15:02.948434
72458	SZC	2020-10-02	26.72	27.3	26.72	27.22	19518	{}	\N	2020-10-12 16:15:02.955515
72459	SZC	2020-10-05	27.42	28.24	27.42	28.24	35986	{}	\N	2020-10-12 16:15:02.961281
72460	SZC	2020-10-06	28.32	28.92	28.245	28.6	43820	{}	\N	2020-10-12 16:15:02.966467
72461	SZC	2020-10-07	28.62	28.932	28.62	28.9106	31782	{}	\N	2020-10-12 16:15:02.972715
72462	SZC	2020-10-08	28.98	29.85	28.98	29.85	17572	{}	\N	2020-10-12 16:15:02.977606
72463	SZC	2020-10-09	30	30	29.7	29.85	8750	{}	\N	2020-10-12 16:15:02.981956
72464	SZC	2020-10-12	30.05	30.05	29.75	29.75	4336	{}	\N	2020-10-12 16:15:02.987745
72465	TACO	2020-09-28	8.04	8.14	7.93	8.09	201871	{}	\N	2020-10-12 16:15:04.027175
72466	TACO	2020-09-29	8.16	8.558	7.84	8.21	1372987	{}	\N	2020-10-12 16:15:04.034131
72467	TACO	2020-09-30	8.3013	8.69	8.09	8.2	517256	{}	\N	2020-10-12 16:15:04.039385
72468	TACO	2020-10-01	8.18	8.2443	7.84	7.88	413883	{}	\N	2020-10-12 16:15:04.045843
72469	TACO	2020-10-02	7.72	8.28	7.62	8.22	394610	{}	\N	2020-10-12 16:15:04.05133
72470	TACO	2020-10-05	8.22	8.45	8.05	8.37	341398	{}	\N	2020-10-12 16:15:04.056805
72471	TACO	2020-10-06	8.46	8.95	8.36	8.56	470844	{}	\N	2020-10-12 16:15:04.062906
72472	TACO	2020-10-07	8.85	9.63	8.73	9.3	733098	{}	\N	2020-10-12 16:15:04.067982
72473	TACO	2020-10-08	9.55	9.56	9.12	9.29	405825	{}	\N	2020-10-12 16:15:04.073358
72474	TACO	2020-10-09	9.28	9.49	9.25	9.42	446158	{}	\N	2020-10-12 16:15:04.07951
72475	TACO	2020-10-12	9.46	9.73	9.385	9.66	285539	{}	\N	2020-10-12 16:15:04.08483
72476	SRI	2020-09-28	17.82	18.75	17.8	18.34	99068	{}	\N	2020-10-12 16:15:05.20537
72477	SRI	2020-09-29	18.4	18.4	17.82	18.22	346551	{}	\N	2020-10-12 16:15:05.212363
72478	SRI	2020-09-30	18.21	18.89	18.21	18.37	420209	{}	\N	2020-10-12 16:15:05.217554
72479	SRI	2020-10-01	18.62	19.06	18.46	19.03	198674	{}	\N	2020-10-12 16:15:05.224022
72480	SRI	2020-10-02	18.9	19.52	18.82	19.41	199035	{}	\N	2020-10-12 16:15:05.232496
72481	SRI	2020-10-05	19.7	20.82	19.51	20.81	237838	{}	\N	2020-10-12 16:15:05.239118
72482	SRI	2020-10-06	21.11	22.19	20.7425	21.36	379210	{}	\N	2020-10-12 16:15:05.245735
72483	SRI	2020-10-07	21.82	22.65	21.81	22.54	291516	{}	\N	2020-10-12 16:15:05.250344
72484	SRI	2020-10-08	22.82	23.03	22.425	22.91	249341	{}	\N	2020-10-12 16:15:05.255025
72485	SRI	2020-10-09	23.09	23.33	22.73	23.03	167096	{}	\N	2020-10-12 16:15:05.260011
72486	SRI	2020-10-12	22.99	23.49	22.98	23.21	30615	{}	\N	2020-10-12 16:15:05.265157
72487	SRTY	2020-09-28	8.87	8.9	8.48	8.54	5278739	{}	\N	2020-10-12 16:15:06.902985
72488	SRTY	2020-09-29	8.58	8.9	8.5178	8.67	4718843	{}	\N	2020-10-12 16:15:06.914579
72489	SRTY	2020-09-30	8.6	8.82	8.22	8.59	6693608	{}	\N	2020-10-12 16:15:06.924265
72490	SRTY	2020-10-01	8.41	8.645	8.18	8.18	5116169	{}	\N	2020-10-12 16:15:06.933965
72491	SRTY	2020-10-02	8.64	8.69	7.964	8.07	6924119	{}	\N	2020-10-12 16:15:06.939753
72492	SRTY	2020-10-05	7.81	7.8101	7.37	7.4	5956814	{}	\N	2020-10-12 16:15:06.945456
72493	SRTY	2020-10-06	7.19	7.51	6.85	7.45	12806072	{}	\N	2020-10-12 16:15:06.952784
72494	SRTY	2020-10-07	7.17	7.19	6.9	6.97	5791078	{}	\N	2020-10-12 16:15:06.963823
72495	SRTY	2020-10-08	6.75	6.91	6.69	6.74	4775963	{}	\N	2020-10-12 16:15:06.972897
72496	SRTY	2020-10-09	6.55	6.75	6.51	6.62	5543651	{}	\N	2020-10-12 16:15:06.978365
72497	SRTY	2020-10-12	6.54	6.6492	6.46	6.52	1634688	{}	\N	2020-10-12 16:15:06.984676
72498	SSP	2020-09-28	11.82	12.33	10.76	11.355	1172906	{}	\N	2020-10-12 16:15:09.376282
72499	SSP	2020-09-29	11.18	11.36	10.51	11.27	850190	{}	\N	2020-10-12 16:15:09.387073
72500	SSP	2020-09-30	11.11	11.975	11.11	11.44	813053	{}	\N	2020-10-12 16:15:09.394798
72501	SSP	2020-10-01	11.5	11.68	11.25	11.63	399790	{}	\N	2020-10-12 16:15:09.401137
72502	SSP	2020-10-02	11.25	11.85	11.25	11.77	288028	{}	\N	2020-10-12 16:15:09.406745
72503	SSP	2020-10-05	11.75	11.99	11.61	11.66	297469	{}	\N	2020-10-12 16:15:09.411635
72504	SSP	2020-10-06	11.87	12.04	11.19	11.25	544708	{}	\N	2020-10-12 16:15:09.416452
72505	SSP	2020-10-07	11.38	11.495	10.89	10.9	595807	{}	\N	2020-10-12 16:15:09.423055
72506	SSP	2020-10-08	11.2588	11.49	10.89	10.95	542036	{}	\N	2020-10-12 16:15:09.428141
72507	SSP	2020-10-09	11.04	11.565	10.82	11	559339	{}	\N	2020-10-12 16:15:09.433068
72508	SSP	2020-10-12	11	11.125	10.65	10.74	180575	{}	\N	2020-10-12 16:15:09.437688
72509	STAA	2020-09-28	56	56.56	55.3	56.36	645210	{}	\N	2020-10-12 16:15:10.415011
72510	STAA	2020-09-29	56.53	56.99	56.08	56.36	325002	{}	\N	2020-10-12 16:15:10.427787
72511	STAA	2020-09-30	56.22	57.4	55.5001	56.56	383291	{}	\N	2020-10-12 16:15:10.442221
72512	STAA	2020-10-01	56.87	56.8701	56	56.33	394338	{}	\N	2020-10-12 16:15:10.447746
72513	STAA	2020-10-02	55.23	56.64	55.01	56.43	482143	{}	\N	2020-10-12 16:15:10.452947
72514	STAA	2020-10-05	56.84	57.5	56.35	56.88	469738	{}	\N	2020-10-12 16:15:10.458742
72515	STAA	2020-10-06	56.83	57.66	55.81	55.93	292887	{}	\N	2020-10-12 16:15:10.464192
72516	STAA	2020-10-07	57.07	57.25	56.326	56.99	397735	{}	\N	2020-10-12 16:15:10.469428
72517	STAA	2020-10-08	57.32	57.88	56.5	57.53	274797	{}	\N	2020-10-12 16:15:10.474774
72518	STAA	2020-10-09	58	58.6334	56.96	58.07	338898	{}	\N	2020-10-12 16:15:10.480518
72519	STAA	2020-10-12	58.82	59.83	58.27	58.58	182432	{}	\N	2020-10-12 16:15:10.485582
72520	STE	2020-09-28	172.72	175.37	172.47	172.96	445568	{}	\N	2020-10-12 16:15:12.273614
72521	STE	2020-09-29	173.47	174.73	172.18	172.82	816969	{}	\N	2020-10-12 16:15:12.284775
72522	STE	2020-09-30	173	177.75	173	176.19	1030388	{}	\N	2020-10-12 16:15:12.29274
72523	STE	2020-10-01	177.11	178.755	173.215	174.47	912353	{}	\N	2020-10-12 16:15:12.303535
72524	STE	2020-10-02	172.04	174.18	171.35	172.67	957532	{}	\N	2020-10-12 16:15:12.311251
72525	STE	2020-10-05	173.43	175.02	173.19	174.72	850332	{}	\N	2020-10-12 16:15:12.318168
72526	STE	2020-10-06	176.19	184.99	176.03	181.01	1378264	{}	\N	2020-10-12 16:15:12.323857
72527	STE	2020-10-07	182.68	186.02	181.3801	185.29	946453	{}	\N	2020-10-12 16:15:12.329902
72528	STE	2020-10-08	187	187	185.92	186.75	607564	{}	\N	2020-10-12 16:15:12.336171
72529	STE	2020-10-09	187.57	189.345	186.6	188.45	1092961	{}	\N	2020-10-12 16:15:12.342204
72530	STE	2020-10-12	189.2	191.5	188.435	191.22	132646	{}	\N	2020-10-12 16:15:12.348805
72531	STRS	2020-09-28	21.16	21.31	20.46	20.63	11380	{}	\N	2020-10-12 16:15:14.347125
72532	STRS	2020-09-29	20.75	21.7	20.5	21.7	3285	{}	\N	2020-10-12 16:15:14.359253
72533	STRS	2020-09-30	21.57	21.87	21.25	21.56	18305	{}	\N	2020-10-12 16:15:14.366134
72534	STRS	2020-10-01	21.9	22.4	21.7	22.03	6250	{}	\N	2020-10-12 16:15:14.372965
72535	STRS	2020-10-02	21.75	21.85	21.375	21.85	3161	{}	\N	2020-10-12 16:15:14.378469
72536	STRS	2020-10-05	21.76	22.5	21.76	22.05	6877	{}	\N	2020-10-12 16:15:14.382971
72537	STRS	2020-10-06	22.4	22.5999	21.595	22.36	11540	{}	\N	2020-10-12 16:15:14.387808
72538	STRS	2020-10-07	22.9	23.78	22.9	23.78	4613	{}	\N	2020-10-12 16:15:14.392544
72539	STRS	2020-10-08	23.76	24.78	23.76	24.665	2745	{}	\N	2020-10-12 16:15:14.396775
72540	STRS	2020-10-09	24.19	24.305	24.15	24.19	2372	{}	\N	2020-10-12 16:15:14.401156
72541	STRS	2020-10-12	23.51	23.75	23.5	23.75	1883	{}	\N	2020-10-12 16:15:14.405974
72542	SU	2020-09-28	12.7	13.09	12.67	12.91	7227232	{}	\N	2020-10-12 16:15:16.09036
72543	SU	2020-09-29	13	13.01	12.255	12.31	17109837	{}	\N	2020-10-12 16:15:16.10326
72544	SU	2020-09-30	12.34	12.47	12.2	12.23	16045498	{}	\N	2020-10-12 16:15:16.111062
72545	SU	2020-10-01	12.16	12.19	11.58	11.68	25654317	{}	\N	2020-10-12 16:15:16.118589
72546	SU	2020-10-02	11.29	12.02	11.11	12.01	28497863	{}	\N	2020-10-12 16:15:16.126759
72547	SU	2020-10-05	12.25	12.38	11.875	12.1	26408291	{}	\N	2020-10-12 16:15:16.132931
72548	SU	2020-10-06	12.36	12.475	11.925	11.98	20084283	{}	\N	2020-10-12 16:15:16.139935
72549	SU	2020-10-07	11.96	12.285	11.905	12.28	17961661	{}	\N	2020-10-12 16:15:16.145425
72550	SU	2020-10-08	12.44	12.99	12.4	12.92	15156115	{}	\N	2020-10-12 16:15:16.150736
72551	SU	2020-10-09	13.15	13.23	12.645	12.86	13936499	{}	\N	2020-10-12 16:15:16.156607
72552	SU	2020-10-12	12.76	12.8	12.56	12.65	2975791	{}	\N	2020-10-12 16:15:16.161901
72553	SUPV	2020-09-28	1.84	1.89	1.72	1.73	460235	{}	\N	2020-10-12 16:15:17.75052
72554	SUPV	2020-09-29	1.72	1.84	1.7	1.78	847666	{}	\N	2020-10-12 16:15:17.761804
72555	SUPV	2020-09-30	1.77	1.835	1.77	1.81	283908	{}	\N	2020-10-12 16:15:17.769444
72556	SUPV	2020-10-01	1.79	1.88	1.79	1.86	317764	{}	\N	2020-10-12 16:15:17.775656
72557	SUPV	2020-10-02	1.77	1.87	1.72	1.79	554495	{}	\N	2020-10-12 16:15:17.781403
72558	SUPV	2020-10-05	1.77	1.835	1.755	1.78	373822	{}	\N	2020-10-12 16:15:17.78597
72559	SUPV	2020-10-06	1.76	1.805	1.69	1.7	1618673	{}	\N	2020-10-12 16:15:17.79123
72560	SUPV	2020-10-07	1.7	1.76	1.68	1.68	1015261	{}	\N	2020-10-12 16:15:17.796406
72561	SUPV	2020-10-08	1.7	1.74	1.6866	1.71	778619	{}	\N	2020-10-12 16:15:17.800728
72562	SUPV	2020-10-09	1.74	1.7513	1.69	1.72	701865	{}	\N	2020-10-12 16:15:17.805547
72563	SUPV	2020-10-12	1.74	1.74	1.67	1.67	189354	{}	\N	2020-10-12 16:15:17.810211
72564	SWN	2020-09-28	2.47	2.55	2.405	2.5	15532147	{}	\N	2020-10-12 16:15:19.882817
72565	SWN	2020-09-29	2.49	2.495	2.25	2.36	42460070	{}	\N	2020-10-12 16:15:19.891443
72566	SWN	2020-09-30	2.32	2.39	2.31	2.35	27216949	{}	\N	2020-10-12 16:15:19.897775
72567	SWN	2020-10-01	2.33	2.33	2.18	2.27	40907551	{}	\N	2020-10-12 16:15:19.90426
72568	SWN	2020-10-02	2.22	2.38	2.19	2.36	22050011	{}	\N	2020-10-12 16:15:19.910345
72569	SWN	2020-10-05	2.39	2.575	2.39	2.55	26351711	{}	\N	2020-10-12 16:15:19.916655
72570	SWN	2020-10-06	2.58	2.68	2.523	2.56	28868543	{}	\N	2020-10-12 16:15:19.924467
72571	SWN	2020-10-07	2.59	2.82	2.59	2.8	34996034	{}	\N	2020-10-12 16:15:19.934767
72572	SWN	2020-10-08	2.83	2.94	2.75	2.89	27286286	{}	\N	2020-10-12 16:15:19.943475
72573	SWN	2020-10-09	2.99	3.1	2.88	2.92	51127629	{}	\N	2020-10-12 16:15:19.949653
72574	SWN	2020-10-12	2.97	3.01	2.84	3.005	16131380	{}	\N	2020-10-12 16:15:19.954804
72575	SYBT	2020-09-28	34.46	35.33	34.46	35.02	36906	{}	\N	2020-10-12 16:15:21.688335
72576	SYBT	2020-09-29	34.97	36.0516	34.0582	34.54	37161	{}	\N	2020-10-12 16:15:21.702952
72577	SYBT	2020-09-30	34.27	34.88	33.79	34.04	73055	{}	\N	2020-10-12 16:15:21.715296
72578	SYBT	2020-10-01	34.41	35.038	33.8	34.165	87005	{}	\N	2020-10-12 16:15:21.721124
72579	SYBT	2020-10-02	33.29	35.1747	33.29	34.9	77214	{}	\N	2020-10-12 16:15:21.726645
72580	SYBT	2020-10-05	35.29	36.68	35.29	36.61	51310	{}	\N	2020-10-12 16:15:21.731825
72581	SYBT	2020-10-06	37.11	37.96	36.23	36.72	75960	{}	\N	2020-10-12 16:15:21.738844
72582	SYBT	2020-10-07	37.15	38.2	36.83	38.02	60181	{}	\N	2020-10-12 16:15:21.744375
72583	SYBT	2020-10-08	38.41	38.41	37.6	37.96	41130	{}	\N	2020-10-12 16:15:21.749526
72584	SYBT	2020-10-09	38.48	38.48	37.63	37.64	25457	{}	\N	2020-10-12 16:15:21.7548
72585	SYBT	2020-10-12	37.51	38.09	37.415	38.015	4378	{}	\N	2020-10-12 16:15:21.762138
72586	SYNC	2020-09-28	1.57	1.6	1.5	1.6	204346	{}	\N	2020-10-12 16:15:22.711684
72587	SYNC	2020-09-29	1.51	1.6001	1.51	1.57	169298	{}	\N	2020-10-12 16:15:22.71866
72588	SYNC	2020-09-30	1.55	1.59	1.5	1.51	85684	{}	\N	2020-10-12 16:15:22.725407
72589	SYNC	2020-10-01	1.53	1.57	1.52	1.52	54682	{}	\N	2020-10-12 16:15:22.730702
72590	SYNC	2020-10-02	1.5	1.58	1.49	1.56	104268	{}	\N	2020-10-12 16:15:22.735641
72591	SYNC	2020-10-05	1.565	1.58	1.52	1.52	41640	{}	\N	2020-10-12 16:15:22.740687
72592	SYNC	2020-10-06	1.53	1.55	1.49	1.49	62231	{}	\N	2020-10-12 16:15:22.745346
72593	SYNC	2020-10-07	1.46	1.51	1.4408	1.5	37143	{}	\N	2020-10-12 16:15:22.749702
72594	SYNC	2020-10-08	1.5	1.6	1.45	1.59	90636	{}	\N	2020-10-12 16:15:22.754559
72595	SYNC	2020-10-09	1.59	1.6	1.51	1.54	126393	{}	\N	2020-10-12 16:15:22.759213
72596	SYNC	2020-10-12	1.54	1.54	1.51	1.51	5230	{}	\N	2020-10-12 16:15:22.764031
72597	SSL	2020-09-28	7.92	7.98	7.7801	7.8	936612	{}	\N	2020-10-12 16:15:26.198688
72598	SSL	2020-09-29	7.62	7.72	7.51	7.59	1184398	{}	\N	2020-10-12 16:15:26.213471
72599	SSL	2020-09-30	7.61	7.75	7.54	7.6	1243118	{}	\N	2020-10-12 16:15:26.223348
72600	SSL	2020-10-01	7.8	7.83	7.63	7.69	1389685	{}	\N	2020-10-12 16:15:26.230369
72601	SSL	2020-10-02	7.14	7.83	7.11	7.67	3509930	{}	\N	2020-10-12 16:15:26.235571
72602	SSL	2020-10-05	7.52	7.57	7.4	7.42	2350433	{}	\N	2020-10-12 16:15:26.24103
72603	SSL	2020-10-06	7.53	7.6	7.22	7.23	2251594	{}	\N	2020-10-12 16:15:26.246858
72604	SSL	2020-10-07	7.37	7.48	7.3444	7.44	1749271	{}	\N	2020-10-12 16:15:26.251945
72605	SSL	2020-10-08	7.52	7.68	7.5	7.66	1548824	{}	\N	2020-10-12 16:15:26.255806
72606	SSL	2020-10-09	7.66	7.66	7.46	7.51	1401140	{}	\N	2020-10-12 16:15:26.259442
72607	SSL	2020-10-12	7.4	7.4173	7.27	7.29	737921	{}	\N	2020-10-12 16:15:26.263812
72608	STC	2020-09-28	42.71	43.36	42.52	43.28	235508	{}	\N	2020-10-12 16:15:27.741168
72609	STC	2020-09-29	43.19	44	42.61	43.79	787033	{}	\N	2020-10-12 16:15:27.751288
72610	STC	2020-09-30	43.87	44.2896	43.46	43.73	561960	{}	\N	2020-10-12 16:15:27.759441
72611	STC	2020-10-01	43.9	44.44	43.5	44.42	323642	{}	\N	2020-10-12 16:15:27.767804
72612	STC	2020-10-02	43.67	46.38	43.52	46.19	462667	{}	\N	2020-10-12 16:15:27.77526
72613	STC	2020-10-05	46.78	47.55	46.6401	47.48	466061	{}	\N	2020-10-12 16:15:27.78088
72614	STC	2020-10-06	48.08	48.92	46.95	47.6	1192655	{}	\N	2020-10-12 16:15:27.786573
72615	STC	2020-10-07	48.11	49.48	47.48	47.53	832435	{}	\N	2020-10-12 16:15:27.792848
72616	STC	2020-10-08	48.05	48.29	46.24	46.31	402602	{}	\N	2020-10-12 16:15:27.798613
72617	STC	2020-10-09	46.29	47.13	45.92	45.98	436018	{}	\N	2020-10-12 16:15:27.804103
72618	STC	2020-10-12	45.99	46.165	45.64	46.09	95618	{}	\N	2020-10-12 16:15:27.810372
72619	STMP	2020-09-28	240.93	245.8	235.84	245.23	191377	{}	\N	2020-10-12 16:15:28.955927
72620	STMP	2020-09-29	245	247.6034	239.985	242.67	188773	{}	\N	2020-10-12 16:15:28.967409
72621	STMP	2020-09-30	242.08	245.76	237.225	240.95	277920	{}	\N	2020-10-12 16:15:28.975581
72622	STMP	2020-10-01	242.51	255.63	242.28	253.2	466149	{}	\N	2020-10-12 16:15:28.982279
72623	STMP	2020-10-02	248.13	256.93	248.13	254.64	313390	{}	\N	2020-10-12 16:15:28.988774
72624	STMP	2020-10-05	256	266.425	256	265.52	203734	{}	\N	2020-10-12 16:15:28.994047
72625	STMP	2020-10-06	263.85	273.5	260.86	265.53	323667	{}	\N	2020-10-12 16:15:28.999623
72626	STMP	2020-10-07	267.76	275.39	267.65	269.05	222085	{}	\N	2020-10-12 16:15:29.004832
72627	STMP	2020-10-08	272	274.9	258.505	262.76	203020	{}	\N	2020-10-12 16:15:29.009864
72628	STMP	2020-10-09	265.97	267	260.7001	263.53	177589	{}	\N	2020-10-12 16:15:29.014974
72629	STMP	2020-10-12	266	275.5	265.5	274.7	139418	{}	\N	2020-10-12 16:15:29.01986
72630	STPZ	2020-09-28	53.83	53.8929	53.83	53.87	13648	{}	\N	2020-10-12 16:15:30.040063
72631	STPZ	2020-09-29	53.88	53.94	53.88	53.93	63570	{}	\N	2020-10-12 16:15:30.054848
72632	STPZ	2020-09-30	53.94	53.96	53.89	53.9	74218	{}	\N	2020-10-12 16:15:30.06423
72633	STPZ	2020-10-01	53.62	53.68	53.6	53.66	10051	{}	\N	2020-10-12 16:15:30.071033
72634	STPZ	2020-10-02	53.64	53.6434	53.58	53.63	86739	{}	\N	2020-10-12 16:15:30.076945
72635	STPZ	2020-10-05	53.655	53.67	53.62	53.63	23052	{}	\N	2020-10-12 16:15:30.081984
72636	STPZ	2020-10-06	53.69	53.69	53.63	53.64	24200	{}	\N	2020-10-12 16:15:30.087042
72637	STPZ	2020-10-07	53.66	53.7	53.6379	53.7	19343	{}	\N	2020-10-12 16:15:30.091566
72638	STPZ	2020-10-08	53.75	53.77	53.7128	53.77	11478	{}	\N	2020-10-12 16:15:30.095936
72639	STPZ	2020-10-09	53.76	53.777	53.7329	53.77	67608	{}	\N	2020-10-12 16:15:30.100638
72640	STPZ	2020-10-12	53.7	53.85	53.7	53.73	6190	{}	\N	2020-10-12 16:15:30.105452
72641	SUN	2020-09-28	24.73	24.96	24.33	24.7	320039	{}	\N	2020-10-12 16:15:32.370863
72642	SUN	2020-09-29	24.54	24.98	24.43	24.86	873017	{}	\N	2020-10-12 16:15:32.38481
72643	SUN	2020-09-30	25	25.16	24.1602	24.39	595119	{}	\N	2020-10-12 16:15:32.394271
72644	SUN	2020-10-01	24.2	24.71	24.07	24.44	641430	{}	\N	2020-10-12 16:15:32.401144
72645	SUN	2020-10-02	24.05	24.73	24.05	24.51	515212	{}	\N	2020-10-12 16:15:32.406675
72646	SUN	2020-10-05	24.6	25.1699	24.359	25.07	303196	{}	\N	2020-10-12 16:15:32.411614
72647	SUN	2020-10-06	25.22	25.33	24.49	24.67	450149	{}	\N	2020-10-12 16:15:32.416872
72648	SUN	2020-10-07	24.74	24.96	24.47	24.73	366169	{}	\N	2020-10-12 16:15:32.422393
72649	SUN	2020-10-08	24.98	25.38	24.76	25.36	441968	{}	\N	2020-10-12 16:15:32.427123
72650	SUN	2020-10-09	25.46	25.46	25.08	25.25	279086	{}	\N	2020-10-12 16:15:32.432189
72651	SUN	2020-10-12	25.25	25.25	24.92	24.94	172550	{}	\N	2020-10-12 16:15:32.437575
72652	SUSC	2020-09-28	27.73	27.775	27.7	27.76	192935	{}	\N	2020-10-12 16:15:34.195784
72653	SUSC	2020-09-29	27.83	27.835	27.77	27.77	145516	{}	\N	2020-10-12 16:15:34.208483
72654	SUSC	2020-09-30	27.75	27.75	27.72	27.725	345353	{}	\N	2020-10-12 16:15:34.216727
72655	SUSC	2020-10-01	27.65	27.74	27.63	27.73	521592	{}	\N	2020-10-12 16:15:34.223345
72656	SUSC	2020-10-02	27.78	27.78	27.685	27.77	411212	{}	\N	2020-10-12 16:15:34.229447
72657	SUSC	2020-10-05	27.71	27.7499	27.664	27.675	147858	{}	\N	2020-10-12 16:15:34.235347
72658	SUSC	2020-10-06	27.71	27.78	27.64	27.67	282774	{}	\N	2020-10-12 16:15:34.241347
72659	SUSC	2020-10-07	27.67	27.7	27.61	27.64	1578220	{}	\N	2020-10-12 16:15:34.247303
72660	SUSC	2020-10-08	27.71	27.72	27.64	27.685	183462	{}	\N	2020-10-12 16:15:34.252643
72661	SUSC	2020-10-09	27.68	27.73	27.6514	27.71	256702	{}	\N	2020-10-12 16:15:34.258132
72662	SUSC	2020-10-12	27.81	27.84	27.76	27.81	39309	{}	\N	2020-10-12 16:15:34.263194
72663	SWKS	2020-09-28	140.63	144.01	139.02	143.97	1974436	{}	\N	2020-10-12 16:15:37.922249
72664	SWKS	2020-09-29	145.2	146.4752	143.71	144.35	1398248	{}	\N	2020-10-12 16:15:37.933394
72665	SWKS	2020-09-30	143.52	147.13	143.36	145.5	1771500	{}	\N	2020-10-12 16:15:37.940288
72666	SWKS	2020-10-01	147.315	152.48	147.14	152.1	1874332	{}	\N	2020-10-12 16:15:37.947989
72667	SWKS	2020-10-02	148.0969	149.616	146.8	146.83	1497495	{}	\N	2020-10-12 16:15:37.958828
72668	SWKS	2020-10-05	148.1	151.76	148.1	151.62	1320758	{}	\N	2020-10-12 16:15:37.968235
72669	SWKS	2020-10-06	150.58	151.7	146.54	147.27	1502182	{}	\N	2020-10-12 16:15:37.974025
72670	SWKS	2020-10-07	149.95	151.03	149.0298	149.86	1292601	{}	\N	2020-10-12 16:15:37.979958
72671	SWKS	2020-10-08	151	151.66	149.81	151.64	1988038	{}	\N	2020-10-12 16:15:37.986304
72672	SWKS	2020-10-09	153.3	154.84	152.48	154.28	1867576	{}	\N	2020-10-12 16:15:37.991836
72673	SWKS	2020-10-12	156.48	157.19	155.255	156.9	1148824	{}	\N	2020-10-12 16:15:37.9985
72674	SXT	2020-09-28	56.01	58	56	57.58	530550	{}	\N	2020-10-12 16:15:38.966558
72675	SXT	2020-09-29	57.58	58.61	57.17	58.19	1056035	{}	\N	2020-10-12 16:15:38.976985
72676	SXT	2020-09-30	58.65	59.6	57.51	57.74	735746	{}	\N	2020-10-12 16:15:38.984357
72677	SXT	2020-10-01	58.14	58.54	57.73	58.47	513737	{}	\N	2020-10-12 16:15:38.990951
72678	SXT	2020-10-02	57.8	60.155	57.8	59.89	911877	{}	\N	2020-10-12 16:15:38.997074
72679	SXT	2020-10-05	60.32	61.99	60.238	61.83	1367688	{}	\N	2020-10-12 16:15:39.003373
72680	SXT	2020-10-06	62.48	62.48	60.6	60.71	726914	{}	\N	2020-10-12 16:15:39.009183
72681	SXT	2020-10-07	61.59	62.32	61.185	61.56	379171	{}	\N	2020-10-12 16:15:39.014413
72682	SXT	2020-10-08	62.17	62.48	61.38	62.02	370924	{}	\N	2020-10-12 16:15:39.019585
72683	SXT	2020-10-09	62.34	62.83	61.88	62.64	296586	{}	\N	2020-10-12 16:15:39.024488
72684	SXT	2020-10-12	62.74	64.12	62.56	63.85	86034	{}	\N	2020-10-12 16:15:39.029641
72685	SYLD	2020-09-28	34.82	35.17	34.7117	35.0642	7274	{}	\N	2020-10-12 16:15:40.226796
72686	SYLD	2020-09-29	34.5034	34.58	34.42	34.42	4756	{}	\N	2020-10-12 16:15:40.237503
72687	SYLD	2020-09-30	34.92	34.99	34.41	34.57	11037	{}	\N	2020-10-12 16:15:40.24466
72688	SYLD	2020-10-01	34.79	34.85	34.42	34.5259	3368	{}	\N	2020-10-12 16:15:40.250333
72689	SYLD	2020-10-02	34.8932	35.1645	34.8932	35.1645	1157	{}	\N	2020-10-12 16:15:40.256847
72690	SYLD	2020-10-05	35.54	36.0091	35.54	36.0091	28794	{}	\N	2020-10-12 16:15:40.262297
72691	SYLD	2020-10-06	36.35	36.5856	35.6493	35.69	10371	{}	\N	2020-10-12 16:15:40.26735
72692	SYLD	2020-10-07	36.378	36.4401	36.21	36.4401	5175	{}	\N	2020-10-12 16:15:40.274025
72693	SYLD	2020-10-08	36.57	37.2566	36.57	37.2566	9809	{}	\N	2020-10-12 16:15:40.280228
72694	SYLD	2020-10-09	37.23	37.2871	36.98	36.98	6935	{}	\N	2020-10-12 16:15:40.285304
72695	SYLD	2020-10-12	37	37.26	37	37.23	4776	{}	\N	2020-10-12 16:15:40.290866
72696	T	2020-09-28	28.32	28.65	28.24	28.38	30255376	{}	\N	2020-10-12 16:15:41.591834
72697	T	2020-09-29	28.36	28.405	28.13	28.3	50759180	{}	\N	2020-10-12 16:15:41.603386
72698	T	2020-09-30	28.38	28.67	28.285	28.51	77514129	{}	\N	2020-10-12 16:15:41.612483
72699	T	2020-10-01	28.52	28.64	28.34	28.49	53826931	{}	\N	2020-10-12 16:15:41.619118
72700	T	2020-10-02	28.27	28.89	28.21	28.68	55204092	{}	\N	2020-10-12 16:15:41.625575
72701	T	2020-10-05	28.49	28.75	28.36	28.67	68038525	{}	\N	2020-10-12 16:15:41.630636
72702	T	2020-10-06	28.745	29.16	28.66	28.71	76107665	{}	\N	2020-10-12 16:15:41.63547
72703	T	2020-10-07	28.92	28.975	28.69	28.8	79321081	{}	\N	2020-10-12 16:15:41.641133
72704	T	2020-10-08	28.32	28.63	28.24	28.61	64786592	{}	\N	2020-10-12 16:15:41.647105
72705	T	2020-10-09	28.63	28.69	28.23	28.32	63649653	{}	\N	2020-10-12 16:15:41.65251
72706	T	2020-10-12	28.3	28.3	28.05	28.073	50947654	{}	\N	2020-10-12 16:15:41.657821
72707	TAIL	2020-09-28	21.68	21.79	21.65	21.78	598900	{}	\N	2020-10-12 16:15:43.511215
72708	TAIL	2020-09-29	21.74	21.82	21.73	21.74	184580	{}	\N	2020-10-12 16:15:43.522373
72709	TAIL	2020-09-30	21.77	21.77	21.5601	21.6	275741	{}	\N	2020-10-12 16:15:43.531492
72710	TAIL	2020-10-01	21.52	21.71	21.511	21.685	565125	{}	\N	2020-10-12 16:15:43.539754
72711	TAIL	2020-10-02	21.89	21.89	21.6541	21.66	565135	{}	\N	2020-10-12 16:15:43.54612
72712	TAIL	2020-10-05	21.5	21.6	21.39	21.39	654371	{}	\N	2020-10-12 16:15:43.55256
72713	TAIL	2020-10-06	21.39	21.59	21.39	21.57	368876	{}	\N	2020-10-12 16:15:43.557793
72714	TAIL	2020-10-07	21.5	21.5	21.31	21.41	488124	{}	\N	2020-10-12 16:15:43.562864
72715	TAIL	2020-10-08	21.33	21.41	21.0469	21.345	218535	{}	\N	2020-10-12 16:15:43.569122
72716	TAIL	2020-10-09	21.27	21.34	21.25	21.26	182219	{}	\N	2020-10-12 16:15:43.573995
72717	TAIL	2020-10-12	21.21	21.27	21.18	21.2	66460	{}	\N	2020-10-12 16:15:43.578876
72718	SRPT	2020-09-28	146.02	150	140.53	143.2	512808	{}	\N	2020-10-12 16:15:44.907592
72719	SRPT	2020-09-29	143.15	143.76	140.02	141.28	541391	{}	\N	2020-10-12 16:15:44.918208
72720	SRPT	2020-09-30	141.65	144.8	139.52	140.43	381279	{}	\N	2020-10-12 16:15:44.927242
72721	SRPT	2020-10-01	140.38	142.65	139.13	141.55	436439	{}	\N	2020-10-12 16:15:44.934654
72722	SRPT	2020-10-02	138.85	140.77	134.61	134.79	649050	{}	\N	2020-10-12 16:15:44.940422
72723	SRPT	2020-10-05	136.05	140.755	136.02	138.8	595830	{}	\N	2020-10-12 16:15:44.945934
72724	SRPT	2020-10-06	139	140.43	137.9	139.5	430465	{}	\N	2020-10-12 16:15:44.951963
72725	SRPT	2020-10-07	140	146.78	139.8316	145.86	1133844	{}	\N	2020-10-12 16:15:44.957642
72726	SRPT	2020-10-08	146.63	146.96	144.26	144.7	436152	{}	\N	2020-10-12 16:15:44.96382
72727	SRPT	2020-10-09	144.22	145.99	142.48	143.58	313482	{}	\N	2020-10-12 16:15:44.969822
72728	SRPT	2020-10-12	145.22	146.508	142.73	144.09	118249	{}	\N	2020-10-12 16:15:44.975751
72729	SSTI	2020-09-28	30.29	31.09	30.125	30.43	60874	{}	\N	2020-10-12 16:15:45.951215
72730	SSTI	2020-09-29	30.54	31.315	30.54	31.03	61397	{}	\N	2020-10-12 16:15:45.96325
72731	SSTI	2020-09-30	30.97	31.82	30.78	31.04	60958	{}	\N	2020-10-12 16:15:45.973205
72732	SSTI	2020-10-01	31.25	31.76	30.3201	31.28	40827	{}	\N	2020-10-12 16:15:45.980427
72733	SSTI	2020-10-02	30.56	31.88	29.8	30.08	47158	{}	\N	2020-10-12 16:15:45.986693
72734	SSTI	2020-10-05	30.2	31.4899	29.6831	29.97	76617	{}	\N	2020-10-12 16:15:45.992972
72735	SSTI	2020-10-06	30.2	31.1599	29.92	30.08	26210	{}	\N	2020-10-12 16:15:45.998746
72736	SSTI	2020-10-07	30.36	31.03	30.015	30.63	18544	{}	\N	2020-10-12 16:15:46.004721
72737	SSTI	2020-10-08	30.87	31.25	30.05	30.3	21515	{}	\N	2020-10-12 16:15:46.011121
72738	SSTI	2020-10-09	30.615	31.07	30.06	30.87	18243	{}	\N	2020-10-12 16:15:46.016236
72739	SSTI	2020-10-12	30.89	32.25	30.89	32.2462	16429	{}	\N	2020-10-12 16:15:46.022139
72740	STAR	2020-09-28	11.71	12.19	11.71	12.06	447368	{}	\N	2020-10-12 16:15:48.524742
72741	STAR	2020-09-29	12.1	12.12	11.72	11.91	467325	{}	\N	2020-10-12 16:15:48.535012
72742	STAR	2020-09-30	11.84	12.2	11.74	11.81	838360	{}	\N	2020-10-12 16:15:48.541463
72743	STAR	2020-10-01	11.84	12.005	11.67	11.9	638512	{}	\N	2020-10-12 16:15:48.548427
72744	STAR	2020-10-02	11.7	12.12	11.65	12.03	1003519	{}	\N	2020-10-12 16:15:48.553641
72745	STAR	2020-10-05	12.13	12.14	11.83	12.01	1430054	{}	\N	2020-10-12 16:15:48.558506
72746	STAR	2020-10-06	12.1	12.26	11.85	11.86	1672024	{}	\N	2020-10-12 16:15:48.563313
72747	STAR	2020-10-07	11.96	11.98	11.645	11.74	1028019	{}	\N	2020-10-12 16:15:48.568306
72748	STAR	2020-10-08	11.85	12.135	11.78	12.11	417845	{}	\N	2020-10-12 16:15:48.573966
72749	STAR	2020-10-09	12.22	12.22	11.94	12.01	438766	{}	\N	2020-10-12 16:15:48.57865
72750	STAR	2020-10-12	12.05	12.11	11.915	12.03	136780	{}	\N	2020-10-12 16:15:48.583433
72751	STM	2020-09-28	29.96	30.28	29.76	30.27	1683331	{}	\N	2020-10-12 16:15:49.914954
72752	STM	2020-09-29	30.38	30.83	30.33	30.6	2658270	{}	\N	2020-10-12 16:15:49.92894
72753	STM	2020-09-30	30.42	31.01	30.37	30.69	2601809	{}	\N	2020-10-12 16:15:49.936495
72754	STM	2020-10-01	32.91	33.3	32.6715	33.21	9968841	{}	\N	2020-10-12 16:15:49.942869
72755	STM	2020-10-02	32.34	32.67	32.21	32.23	3998360	{}	\N	2020-10-12 16:15:49.948469
72756	STM	2020-10-05	32.77	33.32	32.72	33.25	3498506	{}	\N	2020-10-12 16:15:49.95373
72757	STM	2020-10-06	32.91	33.32	32.475	32.56	4929804	{}	\N	2020-10-12 16:15:49.958796
72758	STM	2020-10-07	33.15	33.42	32.99	33.32	3836940	{}	\N	2020-10-12 16:15:49.963523
72759	STM	2020-10-08	33.38	33.65	33.325	33.53	3664521	{}	\N	2020-10-12 16:15:49.968646
72760	STM	2020-10-09	34.29	34.95	34.29	34.9	7939576	{}	\N	2020-10-12 16:15:49.973717
72761	STM	2020-10-12	35.16	35.34	34.89	34.985	5427562	{}	\N	2020-10-12 16:15:49.978429
72762	STRA	2020-09-28	95.78	96.44	94.28	94.88	172864	{}	\N	2020-10-12 16:15:51.925565
72763	STRA	2020-09-29	94.57	95.23	90.825	94.14	234056	{}	\N	2020-10-12 16:15:51.934627
72764	STRA	2020-09-30	94.56	95.2	90.7	91.47	174586	{}	\N	2020-10-12 16:15:51.940706
72765	STRA	2020-10-01	92.16	92.64	90.255	91.49	240768	{}	\N	2020-10-12 16:15:51.946925
72766	STRA	2020-10-02	90.21	91.45	89.29	91.11	129107	{}	\N	2020-10-12 16:15:51.952921
72767	STRA	2020-10-05	91.32	92.01	90.12	91.96	160522	{}	\N	2020-10-12 16:15:51.958326
72768	STRA	2020-10-06	92.4	97.98	92.4	93.65	219861	{}	\N	2020-10-12 16:15:51.962855
72769	STRA	2020-10-07	94.52	95.93	93.65	93.99	136020	{}	\N	2020-10-12 16:15:51.968663
72770	STRA	2020-10-08	94.64	95.96	93.95	95.51	88964	{}	\N	2020-10-12 16:15:51.973141
72771	STRA	2020-10-09	96.26	97.575	95.08	96.43	102066	{}	\N	2020-10-12 16:15:51.979261
72772	STRA	2020-10-12	96.71	97.63	96.08	97.28	25413	{}	\N	2020-10-12 16:15:51.985365
72773	SUM	2020-09-28	16.34	17.14	16.34	16.83	1687320	{}	\N	2020-10-12 16:15:52.937805
72774	SUM	2020-09-29	16.73	16.73	16.22	16.27	2224662	{}	\N	2020-10-12 16:15:52.950937
72775	SUM	2020-09-30	16.39	16.88	16.39	16.54	2290672	{}	\N	2020-10-12 16:15:52.959709
72776	SUM	2020-10-01	16.54	17.07	16.54	16.76	2123228	{}	\N	2020-10-12 16:15:52.965334
72777	SUM	2020-10-02	16.18	17.47	16.12	17.27	2624290	{}	\N	2020-10-12 16:15:52.971842
72778	SUM	2020-10-05	17.6	18.3	17.6	18.13	3876680	{}	\N	2020-10-12 16:15:52.977602
72779	SUM	2020-10-06	18.37	18.89	17.77	17.83	2383699	{}	\N	2020-10-12 16:15:52.982749
72780	SUM	2020-10-07	18.24	18.78	18.15	18.58	1973870	{}	\N	2020-10-12 16:15:52.988757
72781	SUM	2020-10-08	18.99	19.88	18.9	19.75	3666251	{}	\N	2020-10-12 16:15:52.995168
72782	SUM	2020-10-09	19.84	19.99	18.68	19.17	3792805	{}	\N	2020-10-12 16:15:53.000255
72783	SUM	2020-10-12	19.31	19.8	19.3	19.6	1084094	{}	\N	2020-10-12 16:15:53.005914
72784	SWCH	2020-09-28	15.47	15.515	15.235	15.48	1375384	{}	\N	2020-10-12 16:15:54.608459
72785	SWCH	2020-09-29	15.58	16.05	15.54	16.01	5096811	{}	\N	2020-10-12 16:15:54.621134
72786	SWCH	2020-09-30	16.02	16.02	15.495	15.61	3268266	{}	\N	2020-10-12 16:15:54.630236
72787	SWCH	2020-10-01	15.71	15.875	15.46	15.5	2736233	{}	\N	2020-10-12 16:15:54.636704
72788	SWCH	2020-10-02	15.26	15.71	15.2154	15.6	3165408	{}	\N	2020-10-12 16:15:54.642408
72789	SWCH	2020-10-05	15.73	16.06	15.44	15.98	6142648	{}	\N	2020-10-12 16:15:54.647271
72790	SWCH	2020-10-06	16.07	16.19	15.69	15.76	3055721	{}	\N	2020-10-12 16:15:54.651666
72791	SWCH	2020-10-07	15.85	16.025	15.83	15.99	1930857	{}	\N	2020-10-12 16:15:54.656311
72792	SWCH	2020-10-08	16.2	16.32	16.09	16.22	2431129	{}	\N	2020-10-12 16:15:54.660798
72793	SWCH	2020-10-09	16.33	16.44	16.09	16.3	1396627	{}	\N	2020-10-12 16:15:54.665332
72794	SWCH	2020-10-12	16.44	16.54	16.21	16.26	785042	{}	\N	2020-10-12 16:15:54.669578
72795	SYF	2020-09-28	25.85	26.26	25.65	25.79	3243613	{}	\N	2020-10-12 16:15:57.691188
72796	SYF	2020-09-29	25.84	25.97	25.275	25.55	6695093	{}	\N	2020-10-12 16:15:57.704354
72797	SYF	2020-09-30	25.68	26.615	25.55	26.17	12860025	{}	\N	2020-10-12 16:15:57.712972
72798	SYF	2020-10-01	26.45	26.7	26.1121	26.65	9614395	{}	\N	2020-10-12 16:15:57.719482
72799	SYF	2020-10-02	26.13	27.6	25.88	27.46	6887457	{}	\N	2020-10-12 16:15:57.724843
72800	SYF	2020-10-05	27.88	28.48	27.83	28.38	8373246	{}	\N	2020-10-12 16:15:57.729983
72801	SYF	2020-10-06	28.85	29.03	27.17	27.5	11198875	{}	\N	2020-10-12 16:15:57.73585
72802	SYF	2020-10-07	27.95	28.71	27.89	28.3	7338124	{}	\N	2020-10-12 16:15:57.741364
72803	SYF	2020-10-08	28.57	29.195	28.39	29.15	6392425	{}	\N	2020-10-12 16:15:57.746468
72804	SYF	2020-10-09	29.24	29.49	28.66	28.84	12942377	{}	\N	2020-10-12 16:15:57.751817
72805	SYF	2020-10-12	29.18	29.18	28.7514	28.8	3136243	{}	\N	2020-10-12 16:15:57.757752
72806	SYV	2020-09-28	61.3	61.3562	61.2222	61.2222	1563	{}	\N	2020-10-12 16:15:59.925557
72807	SYV	2020-09-29	60.52	60.6908	60.52	60.6908	635	{}	\N	2020-10-12 16:15:59.934158
72808	SYV	2020-09-30	60.59	61.3328	60.59	61.26	891	{}	\N	2020-10-12 16:15:59.941344
72809	SYV	2020-10-01	61.26	61.39	60.97	61.04	3955	{}	\N	2020-10-12 16:15:59.948237
72810	SYV	2020-10-02	60.88	61.46	60.835	61.255	4314	{}	\N	2020-10-12 16:15:59.953338
72811	SYV	2020-10-05	61.94	62.1386	61.7	62.1386	2054	{}	\N	2020-10-12 16:15:59.958549
72812	SYV	2020-10-06	62.4902	62.65	61.58	61.58	3287	{}	\N	2020-10-12 16:15:59.964252
72813	SYV	2020-10-07	62.5164	62.5164	62.486	62.486	403	{}	\N	2020-10-12 16:15:59.971541
72814	SYV	2020-10-08	62.92	63.2238	62.91	63.2238	2037	{}	\N	2020-10-12 16:15:59.97837
72815	SYV	2020-10-09	63.725	63.725	63.362	63.362	429	{}	\N	2020-10-12 16:15:59.983481
72816	TANH	2020-09-28	2.34	2.42	2.28	2.39	460692	{}	\N	2020-10-12 16:16:03.254577
72817	TANH	2020-09-29	2.4	2.45	2.34	2.44	850230	{}	\N	2020-10-12 16:16:03.264356
72818	TANH	2020-09-30	2.45	2.53	2.42	2.49	2525113	{}	\N	2020-10-12 16:16:03.271309
72819	TANH	2020-10-01	2.44	2.47	2.22	2.33	1859454	{}	\N	2020-10-12 16:16:03.2785
72820	TANH	2020-10-02	2.24	2.3285	2.24	2.31	458121	{}	\N	2020-10-12 16:16:03.285002
72821	TANH	2020-10-05	2.37	2.5	2.31	2.47	1385640	{}	\N	2020-10-12 16:16:03.290038
72822	TANH	2020-10-06	2.47	2.51	2.41	2.49	758135	{}	\N	2020-10-12 16:16:03.294213
72823	TANH	2020-10-07	2.46	2.7	2.445	2.7	976967	{}	\N	2020-10-12 16:16:03.299448
72824	TANH	2020-10-08	2.64	2.75	2.58	2.71	972012	{}	\N	2020-10-12 16:16:03.303566
72825	TANH	2020-10-09	2.69	2.73	2.62	2.64	1095035	{}	\N	2020-10-12 16:16:03.307793
72826	TANH	2020-10-12	2.57	2.59	2.35	2.46	520527	{}	\N	2020-10-12 16:16:03.313181
72827	TBBK	2020-09-28	8.45	8.68	8.45	8.61	150905	{}	\N	2020-10-12 16:16:05.356781
72828	TBBK	2020-09-29	8.53	8.7	8.37	8.655	202277	{}	\N	2020-10-12 16:16:05.362668
72829	TBBK	2020-09-30	8.73	8.85	8.395	8.64	166307	{}	\N	2020-10-12 16:16:05.366661
72830	TBBK	2020-10-01	8.63	8.81	8.49	8.73	164717	{}	\N	2020-10-12 16:16:05.370826
72831	TBBK	2020-10-02	8.55	9.05	8.55	9.01	114928	{}	\N	2020-10-12 16:16:05.374711
72832	TBBK	2020-10-05	9.16	9.44	8.8364	9.395	175844	{}	\N	2020-10-12 16:16:05.378377
72833	TBBK	2020-10-06	9.55	9.63	9.26	9.3	269564	{}	\N	2020-10-12 16:16:05.382106
72834	TBBK	2020-10-07	9.42	9.63	9.24	9.3	273878	{}	\N	2020-10-12 16:16:05.385809
72835	TBBK	2020-10-08	9.44	9.64	9.44	9.52	233534	{}	\N	2020-10-12 16:16:05.389719
72836	TBBK	2020-10-09	9.64	9.77	9.39	9.46	160849	{}	\N	2020-10-12 16:16:05.393477
72837	TBBK	2020-10-12	9.43	9.5	9.37	9.425	60057	{}	\N	2020-10-12 16:16:05.397313
72838	SYBX	2020-09-28	2.09	2.16	2.04	2.04	92624	{}	\N	2020-10-12 16:16:06.436839
72839	SYBX	2020-09-29	2.06	2.06	1.92	2.01	167326	{}	\N	2020-10-12 16:16:06.446528
72840	SYBX	2020-09-30	2	2.05	1.95	2.02	78495	{}	\N	2020-10-12 16:16:06.454111
72841	SYBX	2020-10-01	2.02	2.07	1.98	2.01	106992	{}	\N	2020-10-12 16:16:06.460588
72842	SYBX	2020-10-02	1.95	2.06	1.95	2.01	78300	{}	\N	2020-10-12 16:16:06.466551
72843	SYBX	2020-10-05	2.01	2.13	2	2.06	169593	{}	\N	2020-10-12 16:16:06.472111
72844	SYBX	2020-10-06	2.09	2.12	2.01	2.04	83167	{}	\N	2020-10-12 16:16:06.477583
72845	SYBX	2020-10-07	2.05	2.11	2	2.11	90564	{}	\N	2020-10-12 16:16:06.482261
72846	SYBX	2020-10-08	2.1	2.3	2.04	2.15	407253	{}	\N	2020-10-12 16:16:06.48679
72847	SYBX	2020-10-09	2.13	2.19	2.09	2.14	109048	{}	\N	2020-10-12 16:16:06.491203
72848	SYBX	2020-10-12	2.15	2.175	2.12	2.15	47605	{}	\N	2020-10-12 16:16:06.496211
72849	SYY	2020-09-28	63.08	63.48	61.77	61.85	2135978	{}	\N	2020-10-12 16:16:09.103668
72850	SYY	2020-09-29	61.65	61.99	60.81	61.17	4263737	{}	\N	2020-10-12 16:16:09.115768
72851	SYY	2020-09-30	61.55	63.0838	61.55	62.22	4511862	{}	\N	2020-10-12 16:16:09.12332
72852	SYY	2020-10-01	62.32	63.03	61.48	62.62	3956263	{}	\N	2020-10-12 16:16:09.130177
72853	SYY	2020-10-02	61.38	63.41	60.95	63.17	3250171	{}	\N	2020-10-12 16:16:09.13666
72854	SYY	2020-10-05	63.64	63.94	62.36	63.54	3419230	{}	\N	2020-10-12 16:16:09.141684
72855	SYY	2020-10-06	64.07	65.73	63.83	64.67	7244654	{}	\N	2020-10-12 16:16:09.147074
72856	SYY	2020-10-07	65.3	66.47	65.3	66.28	3892774	{}	\N	2020-10-12 16:16:09.152074
72857	SYY	2020-10-08	66.64	67.87	66.44	67.84	4677891	{}	\N	2020-10-12 16:16:09.156726
72858	SYY	2020-10-09	68	69.27	67.65	67.98	5144744	{}	\N	2020-10-12 16:16:09.162434
72859	SYY	2020-10-12	67.87	68.04	67.157	67.97	1541448	{}	\N	2020-10-12 16:16:09.167717
72860	TANNI	2020-09-28	25.84	25.98	25.84	25.98	4770	{}	\N	2020-10-12 16:16:10.21337
72861	TANNI	2020-09-29	25.895	26.08	25.85	25.95	18203	{}	\N	2020-10-12 16:16:10.221625
72862	TANNI	2020-09-30	25.84	25.84	25.5115	25.76	2252	{}	\N	2020-10-12 16:16:10.227857
72863	TANNI	2020-10-01	25.74	25.905	25.67	25.85	3176	{}	\N	2020-10-12 16:16:10.233844
72864	TANNI	2020-10-02	25.38	25.62	25.35	25.49	11935	{}	\N	2020-10-12 16:16:10.239183
72865	TANNI	2020-10-05	25.49	25.75	25.49	25.75	2688	{}	\N	2020-10-12 16:16:10.244859
72866	TANNI	2020-10-06	25.9	27	25.885	26.28	24617	{}	\N	2020-10-12 16:16:10.249671
72867	TANNI	2020-10-07	26.28	26.3	26.25	26.25	1956	{}	\N	2020-10-12 16:16:10.255541
72868	TANNI	2020-10-08	26.29	27	26.14	26.34	15511	{}	\N	2020-10-12 16:16:10.260856
72869	TANNI	2020-10-09	26.26	26.5505	26.1217	26.5505	2582	{}	\N	2020-10-12 16:16:10.265775
72870	TANNI	2020-10-12	26.28	26.43	26.28	26.42	2442	{}	\N	2020-10-12 16:16:10.271372
72871	TBK	2020-09-28	29.76	31.25	29.13	31.04	228050	{}	\N	2020-10-12 16:16:11.298028
72872	TBK	2020-09-29	30.89	30.94	30.05	30.62	71519	{}	\N	2020-10-12 16:16:11.305637
72873	TBK	2020-09-30	31.01	31.8	30.68	31.14	121329	{}	\N	2020-10-12 16:16:11.312784
72874	TBK	2020-10-01	32.04	33.39	31.81	32.87	195360	{}	\N	2020-10-12 16:16:11.320103
72875	TBK	2020-10-02	32.8	34.42	32.61	34.19	173485	{}	\N	2020-10-12 16:16:11.326723
72876	TBK	2020-10-05	34.82	35.56	33.73	35.5	128848	{}	\N	2020-10-12 16:16:11.335131
72877	TBK	2020-10-06	35.94	37.4	35	35.64	190283	{}	\N	2020-10-12 16:16:11.343518
72878	TBK	2020-10-07	36.37	37.85	35.99	37.65	168829	{}	\N	2020-10-12 16:16:11.350215
72879	TBK	2020-10-08	38	39.93	38	38.81	265057	{}	\N	2020-10-12 16:16:11.355278
72880	TBK	2020-10-09	39.37	39.37	38.37	39.28	105951	{}	\N	2020-10-12 16:16:11.360503
72881	TBK	2020-10-12	39.33	40	39.1347	39.38	23614	{}	\N	2020-10-12 16:16:11.365662
72882	TCBK	2020-09-28	24.2	24.88	24.2	24.64	51742	{}	\N	2020-10-12 16:16:12.698008
72883	TCBK	2020-09-29	24.55	24.65	24.03	24.57	53751	{}	\N	2020-10-12 16:16:12.711914
72884	TCBK	2020-09-30	24.66	24.96	24.27	24.49	66096	{}	\N	2020-10-12 16:16:12.720355
72885	TCBK	2020-10-01	24.41	24.67	23.98	24.58	61132	{}	\N	2020-10-12 16:16:12.728584
72886	TCBK	2020-10-02	24.36	25.35	24.11	25.11	40070	{}	\N	2020-10-12 16:16:12.735305
72887	TCBK	2020-10-05	25.47	26.295	25.47	25.94	86133	{}	\N	2020-10-12 16:16:12.743683
72888	TCBK	2020-10-06	26.35	27.195	25.98	26.47	125843	{}	\N	2020-10-12 16:16:12.751905
72889	TCBK	2020-10-07	26.77	27.26	25.7	27.1	108775	{}	\N	2020-10-12 16:16:12.75898
72890	TCBK	2020-10-08	27.44	27.66	26.54	27.49	80249	{}	\N	2020-10-12 16:16:12.766394
72891	TCBK	2020-10-09	27.84	27.97	27.09	27.28	62287	{}	\N	2020-10-12 16:16:12.774603
72892	TCBK	2020-10-12	27.905	27.905	27.11	27.405	28543	{}	\N	2020-10-12 16:16:12.788733
72893	TDJ	2020-09-28	25.43	25.83	25.4	25.805	28618	{}	\N	2020-10-12 16:16:14.686178
72894	TDJ	2020-09-29	25.71	25.89	25.65	25.85	35706	{}	\N	2020-10-12 16:16:14.700594
72895	TDJ	2020-09-30	25.85	25.88	25.58	25.59	205007	{}	\N	2020-10-12 16:16:14.711838
72896	TDJ	2020-10-01	25.65	25.85	25.59	25.82	19360	{}	\N	2020-10-12 16:16:14.722898
72897	TDJ	2020-10-02	25.5	25.6	25.4	25.51	34281	{}	\N	2020-10-12 16:16:14.734833
72898	TDJ	2020-10-05	25.67	25.67	25.55	25.59	13200	{}	\N	2020-10-12 16:16:14.74535
72899	TDJ	2020-10-06	25.68	25.75	25.55	25.73	25743	{}	\N	2020-10-12 16:16:14.754369
72900	TDJ	2020-10-07	25.83	25.85	25.68	25.76	15538	{}	\N	2020-10-12 16:16:14.762567
72901	TDJ	2020-10-08	25.85	25.85	25.8	25.81	11796	{}	\N	2020-10-12 16:16:14.772445
72902	TDJ	2020-10-09	25.82	25.84	25.59	25.84	23096	{}	\N	2020-10-12 16:16:14.785772
72903	TDJ	2020-10-12	25.86	25.86	25.67	25.77	6730	{}	\N	2020-10-12 16:16:14.794163
72904	TECH	2020-09-28	244	247.23	242.0301	243.74	103093	{}	\N	2020-10-12 16:16:15.80841
72905	TECH	2020-09-29	244.09	249.59	242.9	246.76	165715	{}	\N	2020-10-12 16:16:15.822775
72906	TECH	2020-09-30	249.25	249.58	245.4	247.73	229359	{}	\N	2020-10-12 16:16:15.830744
72907	TECH	2020-10-01	247.59	251.99	246.71	248.71	197911	{}	\N	2020-10-12 16:16:15.838179
72908	TECH	2020-10-02	246	248.77	245.83	246.9	148461	{}	\N	2020-10-12 16:16:15.844246
72909	TECH	2020-10-05	247.87	252.47	247.87	252.17	213535	{}	\N	2020-10-12 16:16:15.850382
72910	TECH	2020-10-06	253	258.62	252.125	256.19	147806	{}	\N	2020-10-12 16:16:15.859752
72911	TECH	2020-10-07	257.44	264.885	256.06	263.94	175097	{}	\N	2020-10-12 16:16:15.869384
72912	TECH	2020-10-08	263.63	264.72	260.18	261.9	151314	{}	\N	2020-10-12 16:16:15.879558
72913	TECH	2020-10-09	264	265.57	261.04	265.08	140450	{}	\N	2020-10-12 16:16:15.889768
72914	TECH	2020-10-12	267.53	268.97	264.16	267.34	12897	{}	\N	2020-10-12 16:16:15.898545
72915	TELL	2020-09-28	0.7309	0.7699	0.717	0.7312	5430481	{}	\N	2020-10-12 16:16:17.264668
72916	TELL	2020-09-29	0.7462	0.77	0.7114	0.7429	4772500	{}	\N	2020-10-12 16:16:17.277071
72917	TELL	2020-09-30	0.753	0.8198	0.75	0.7968	6077219	{}	\N	2020-10-12 16:16:17.284167
72918	TELL	2020-10-01	0.8	0.8106	0.751	0.7581	4135151	{}	\N	2020-10-12 16:16:17.29109
72919	TELL	2020-10-02	0.75	0.8	0.7005	0.7469	5039039	{}	\N	2020-10-12 16:16:17.296749
72920	TELL	2020-10-05	0.76	0.79	0.7307	0.7896	3620998	{}	\N	2020-10-12 16:16:17.303686
72921	TELL	2020-10-06	0.79	0.87	0.785	0.786	5881813	{}	\N	2020-10-12 16:16:17.310577
72922	TELL	2020-10-07	0.8	0.85	0.8	0.8469	3891181	{}	\N	2020-10-12 16:16:17.315567
72923	TELL	2020-10-08	0.85	0.96	0.837	0.9553	8367925	{}	\N	2020-10-12 16:16:17.321082
72924	TELL	2020-10-09	0.9749	0.99	0.87	0.9188	8209392	{}	\N	2020-10-12 16:16:17.32719
72925	TELL	2020-10-12	0.91	0.9481	0.865	0.9198	2365925	{}	\N	2020-10-12 16:16:17.333043
72926	TFI	2020-09-28	52	52.05	52	52.02	194516	{}	\N	2020-10-12 16:16:18.840847
72927	TFI	2020-09-29	52	52.09	52	52	252821	{}	\N	2020-10-12 16:16:18.853369
72928	TFI	2020-09-30	52.02	52.045	51.9286	51.93	255271	{}	\N	2020-10-12 16:16:18.86154
72929	TFI	2020-10-01	51.82	51.88	51.79	51.83	250136	{}	\N	2020-10-12 16:16:18.867124
72930	TFI	2020-10-02	51.85	51.8799	51.78	51.8	270203	{}	\N	2020-10-12 16:16:18.872491
72931	TFI	2020-10-05	51.82	51.82	51.73	51.75	295359	{}	\N	2020-10-12 16:16:18.877621
72932	TFI	2020-10-06	51.78	51.78	51.67	51.7	249065	{}	\N	2020-10-12 16:16:18.882226
72933	TFI	2020-10-07	51.64	51.6992	51.56	51.59	593932	{}	\N	2020-10-12 16:16:18.886536
72934	TFI	2020-10-08	51.55	51.675	51.54	51.59	249095	{}	\N	2020-10-12 16:16:18.890458
72935	TFI	2020-10-09	51.63	51.64	51.56	51.6	271125	{}	\N	2020-10-12 16:16:18.895414
72936	TFI	2020-10-12	51.58	51.6332	51.58	51.605	59955	{}	\N	2020-10-12 16:16:18.899288
72937	TGLS	2020-09-28	5.01	5.27	5.01	5.27	16109	{}	\N	2020-10-12 16:16:20.161513
72938	TGLS	2020-09-29	5.165	5.42	5.0917	5.15	41465	{}	\N	2020-10-12 16:16:20.17561
72939	TGLS	2020-09-30	5.15	5.447	5.15	5.3	27727	{}	\N	2020-10-12 16:16:20.185252
72940	TGLS	2020-10-01	5.34	5.34	5.07	5.25	45023	{}	\N	2020-10-12 16:16:20.191616
72941	TGLS	2020-10-02	5.23	5.25	5.01	5.25	136884	{}	\N	2020-10-12 16:16:20.198425
72942	TGLS	2020-10-05	5.3	5.375	5.15	5.16	9972	{}	\N	2020-10-12 16:16:20.203675
72943	TGLS	2020-10-06	5.16	5.32	5.15	5.24	42038	{}	\N	2020-10-12 16:16:20.208632
72944	TGLS	2020-10-07	5.2744	5.53	5.2744	5.51	32640	{}	\N	2020-10-12 16:16:20.213667
72945	TGLS	2020-10-08	5.41	5.54	5.21	5.52	45183	{}	\N	2020-10-12 16:16:20.218573
72946	TGLS	2020-10-09	5.5273	5.6299	5.46	5.54	55874	{}	\N	2020-10-12 16:16:20.224015
72947	TGLS	2020-10-12	5.6	5.6	5.34	5.34	16642	{}	\N	2020-10-12 16:16:20.230187
72948	THFF	2020-09-28	31	31.96	31	31.85	59259	{}	\N	2020-10-12 16:16:21.115426
72949	THFF	2020-09-29	31.65	31.65	30.99	31.55	46685	{}	\N	2020-10-12 16:16:21.128537
72950	THFF	2020-09-30	31.65	32	31.21	31.4	59792	{}	\N	2020-10-12 16:16:21.13723
72951	THFF	2020-10-01	31.33	31.97	31.07	31.88	28123	{}	\N	2020-10-12 16:16:21.143482
72952	THFF	2020-10-02	31.46	32.82	31.33	32.71	40788	{}	\N	2020-10-12 16:16:21.148947
72953	THFF	2020-10-05	32.84	33.37	32.675	33.29	40426	{}	\N	2020-10-12 16:16:21.15708
72954	THFF	2020-10-06	33.63	34.51	33.0488	33.59	33584	{}	\N	2020-10-12 16:16:21.165974
72955	THFF	2020-10-07	33.76	34.37	33.09	34.21	35680	{}	\N	2020-10-12 16:16:21.172149
72956	THFF	2020-10-08	34.45	34.63	33.25	34.42	36291	{}	\N	2020-10-12 16:16:21.177746
72957	THFF	2020-10-09	32.96	34.77	32.96	34.08	18580	{}	\N	2020-10-12 16:16:21.183084
72958	THFF	2020-10-12	33.98	34.1714	33.89	34.01	6880	{}	\N	2020-10-12 16:16:21.188663
72959	THW	2020-09-28	14.24	14.25	14.05	14.05	120226	{}	\N	2020-10-12 16:16:22.167449
72960	THW	2020-09-29	13.99	14.15	13.965	14.12	181131	{}	\N	2020-10-12 16:16:22.175366
72961	THW	2020-09-30	14.22	14.39	14.1475	14.33	211828	{}	\N	2020-10-12 16:16:22.181725
72962	THW	2020-10-01	14.42	14.43	14.174	14.2	174460	{}	\N	2020-10-12 16:16:22.187922
72963	THW	2020-10-02	14.08	14.27	13.97	14.2	193045	{}	\N	2020-10-12 16:16:22.193186
72964	THW	2020-10-05	14.27	14.46	14.27	14.42	176108	{}	\N	2020-10-12 16:16:22.197809
72965	THW	2020-10-06	14.44	14.5	14.22	14.23	230783	{}	\N	2020-10-12 16:16:22.20627
72966	THW	2020-10-07	14.27	14.49	14.27	14.46	107595	{}	\N	2020-10-12 16:16:22.211819
72967	THW	2020-10-08	14.5	14.58	14.48	14.54	165475	{}	\N	2020-10-12 16:16:22.216585
72968	THW	2020-10-09	14.6	14.699	14.54	14.62	162452	{}	\N	2020-10-12 16:16:22.221582
72969	THW	2020-10-12	14.66	14.73	14.65	14.7	81608	{}	\N	2020-10-12 16:16:22.227608
72970	TARO	2020-09-28	55	55	54.22	54.41	11173	{}	\N	2020-10-12 16:16:24.461654
72971	TARO	2020-09-29	54.17	54.93	53.9462	54.53	31385	{}	\N	2020-10-12 16:16:24.472295
72972	TARO	2020-09-30	54.78	54.98	54.38	54.85	38676	{}	\N	2020-10-12 16:16:24.482034
72973	TARO	2020-10-01	54.64	54.98	54.4	54.86	67778	{}	\N	2020-10-12 16:16:24.490171
72974	TARO	2020-10-02	54.55	54.85	54.14	54.4	25398	{}	\N	2020-10-12 16:16:24.495738
72975	TARO	2020-10-05	54.83	55	54.644	55	66020	{}	\N	2020-10-12 16:16:24.500888
72976	TARO	2020-10-06	54.95	55	54.425	54.89	35886	{}	\N	2020-10-12 16:16:24.505269
72977	TARO	2020-10-07	54.98	56.74	54.7631	55.68	262499	{}	\N	2020-10-12 16:16:24.510131
72978	TARO	2020-10-08	55.89	58.92	55.89	58.33	50340	{}	\N	2020-10-12 16:16:24.514808
72979	TARO	2020-10-09	58.33	58.78	57.67	58	16094	{}	\N	2020-10-12 16:16:24.519858
72980	TARO	2020-10-12	58.4	60.17	58.24	60.17	14384	{}	\N	2020-10-12 16:16:24.525445
72981	TBI	2020-09-28	15.46	15.92	15.46	15.72	309424	{}	\N	2020-10-12 16:16:26.641888
72982	TBI	2020-09-29	15.74	15.82	15.44	15.59	1097436	{}	\N	2020-10-12 16:16:26.65417
72983	TBI	2020-09-30	15.77	16.25	15.43	15.49	557220	{}	\N	2020-10-12 16:16:26.662525
72984	TBI	2020-10-01	15.51	15.77	15.165	15.28	417135	{}	\N	2020-10-12 16:16:26.668742
72985	TBI	2020-10-02	15.1	15.74	15.1	15.65	324867	{}	\N	2020-10-12 16:16:26.674795
72986	TBI	2020-10-05	15.8	16.23	15.8	16.15	278340	{}	\N	2020-10-12 16:16:26.679844
72987	TBI	2020-10-06	16.33	16.76	16.33	16.46	485177	{}	\N	2020-10-12 16:16:26.684959
72988	TBI	2020-10-07	16.58	16.75	16.51	16.64	277900	{}	\N	2020-10-12 16:16:26.689953
72989	TBI	2020-10-08	16.84	17.08	16.77	16.98	216686	{}	\N	2020-10-12 16:16:26.695707
72990	TBI	2020-10-09	17.1	17.29	16.96	17.08	296925	{}	\N	2020-10-12 16:16:26.70089
72991	TBI	2020-10-12	17.13	17.44	17.13	17.39	53187	{}	\N	2020-10-12 16:16:26.705651
72992	TCBIL	2020-09-28	25.53	25.85	25.49	25.85	2806	{}	\N	2020-10-12 16:16:28.220319
72993	TCBIL	2020-09-29	25.75	25.85	25.75	25.83	1680	{}	\N	2020-10-12 16:16:28.235647
72994	TCBIL	2020-09-30	25.6726	25.74	25.57	25.68	2944	{}	\N	2020-10-12 16:16:28.245654
72995	TCBIL	2020-10-01	25.74	25.85	25.45	25.85	3451	{}	\N	2020-10-12 16:16:28.253911
72996	TCBIL	2020-10-02	25.5131	25.9	25.2801	25.87	3283	{}	\N	2020-10-12 16:16:28.259841
72997	TCBIL	2020-10-05	26.05	26.12	26.05	26.11	1028	{}	\N	2020-10-12 16:16:28.265309
72998	TCBIL	2020-10-06	25.96	26.015	25.765	26	3658	{}	\N	2020-10-12 16:16:28.271328
72999	TCBIL	2020-10-07	25.93	26.13	25.93	26.13	1623	{}	\N	2020-10-12 16:16:28.277404
73000	TCBIL	2020-10-08	26.22	26.22	26	26.22	2355	{}	\N	2020-10-12 16:16:28.28274
73001	TCBIL	2020-10-09	25.91	26.02	25.63	26.02	1956	{}	\N	2020-10-12 16:16:28.287921
73002	TCBIL	2020-10-12	26.21	26.21	26.21	26.21	1428	{}	\N	2020-10-12 16:16:28.293232
73003	TCO	2020-09-28	33.9	34.31	33.6	33.96	477016	{}	\N	2020-10-12 16:16:29.277839
73004	TCO	2020-09-29	33.7	33.82	33.11	33.36	901457	{}	\N	2020-10-12 16:16:29.287623
73005	TCO	2020-09-30	33.32	34.01	33.15	33.29	1136542	{}	\N	2020-10-12 16:16:29.294526
73006	TCO	2020-10-01	33.36	34.49	32.96	34.48	1100336	{}	\N	2020-10-12 16:16:29.299985
73007	TCO	2020-10-02	33.98	35.19	33.98	35.13	1232542	{}	\N	2020-10-12 16:16:29.30748
73008	TCO	2020-10-05	35.44	35.5	34.59	35.4	2074707	{}	\N	2020-10-12 16:16:29.314167
73009	TCO	2020-10-06	35.5	35.7	35.06	35.34	947112	{}	\N	2020-10-12 16:16:29.320146
73010	TCO	2020-10-07	35.51	36.07	35.375	35.91	1682042	{}	\N	2020-10-12 16:16:29.326676
73011	TCO	2020-10-08	36.1	36.1	35.725	36.03	703677	{}	\N	2020-10-12 16:16:29.333768
73012	TCO	2020-10-09	36.25	36.44	35.94	36.35	1136347	{}	\N	2020-10-12 16:16:29.339667
73013	TCO	2020-10-12	36.24	36.24	35.43	35.94	299418	{}	\N	2020-10-12 16:16:29.344861
73014	TD	2020-09-28	46.52	46.9164	46.43	46.63	1219608	{}	\N	2020-10-12 16:16:30.546646
73015	TD	2020-09-29	46.45	46.49	45.615	45.96	2431396	{}	\N	2020-10-12 16:16:30.560882
73016	TD	2020-09-30	46.09	46.77	46.09	46.24	2595087	{}	\N	2020-10-12 16:16:30.566669
73017	TD	2020-10-01	46.49	46.59	46.03	46.54	1990074	{}	\N	2020-10-12 16:16:30.572235
73018	TD	2020-10-02	45.92	47.15	45.83	46.94	2407911	{}	\N	2020-10-12 16:16:30.577144
73019	TD	2020-10-05	47.32	48	47.17	47.87	2354078	{}	\N	2020-10-12 16:16:30.582105
73020	TD	2020-10-06	48.23	48.3	47.33	47.57	8850950	{}	\N	2020-10-12 16:16:30.58727
73021	TD	2020-10-07	47.91	48.05	47.6164	47.74	12033513	{}	\N	2020-10-12 16:16:30.592233
73022	TD	2020-10-08	47.5	47.7	47.21	47.49	9642287	{}	\N	2020-10-12 16:16:30.597139
73023	TD	2020-10-09	47.44	47.56	46.8	46.88	4205600	{}	\N	2020-10-12 16:16:30.60207
73024	TD	2020-10-12	46.74	46.99	46.68	46.755	916681	{}	\N	2020-10-12 16:16:30.607405
73025	TDTF	2020-09-28	27.47	27.54	27.47	27.54	33809	{}	\N	2020-10-12 16:16:31.823384
73026	TDTF	2020-09-29	27.57	27.585	27.5561	27.57	18250	{}	\N	2020-10-12 16:16:31.836134
73027	TDTF	2020-09-30	27.55	27.58	27.52	27.525	65125	{}	\N	2020-10-12 16:16:31.844564
73028	TDTF	2020-10-01	27.46	27.5	27.45	27.48	51633	{}	\N	2020-10-12 16:16:31.851551
73029	TDTF	2020-10-02	27.47	27.49	27.45	27.45	26605	{}	\N	2020-10-12 16:16:31.857129
73030	TDTF	2020-10-05	27.4501	27.465	27.42	27.429	38462	{}	\N	2020-10-12 16:16:31.862188
73031	TDTF	2020-10-06	27.44	27.455	27.4	27.44	28002	{}	\N	2020-10-12 16:16:31.86799
73032	TDTF	2020-10-07	27.4276	27.44	27.42	27.44	36357	{}	\N	2020-10-12 16:16:31.874564
73033	TDTF	2020-10-08	27.48	27.53	27.4678	27.49	31008	{}	\N	2020-10-12 16:16:31.879958
73034	TDTF	2020-10-09	27.51	27.5399	27.4982	27.51	60396	{}	\N	2020-10-12 16:16:31.885053
73035	TDTF	2020-10-12	27.48	27.53	27.48	27.52	9249	{}	\N	2020-10-12 16:16:31.890256
73036	TEI	2020-09-28	7.37	7.5	7.37	7.48	228685	{}	\N	2020-10-12 16:16:32.910442
73037	TEI	2020-09-29	7.43	7.49	7.43	7.49	208141	{}	\N	2020-10-12 16:16:32.919998
73038	TEI	2020-09-30	7.49	7.5	7.41	7.41	441537	{}	\N	2020-10-12 16:16:32.927448
73039	TEI	2020-10-01	7.46	7.51	7.45	7.45	608582	{}	\N	2020-10-12 16:16:32.935292
73040	TEI	2020-10-02	7.44	7.452	7.33	7.39	800553	{}	\N	2020-10-12 16:16:32.941588
73041	TEI	2020-10-05	7.39	7.43	7.39	7.39	250586	{}	\N	2020-10-12 16:16:32.947064
73042	TEI	2020-10-06	7.4	7.473	7.4	7.4	234459	{}	\N	2020-10-12 16:16:32.952621
73043	TEI	2020-10-07	7.4	7.45	7.36	7.36	319746	{}	\N	2020-10-12 16:16:32.957772
73044	TEI	2020-10-08	7.38	7.44	7.38	7.43	415008	{}	\N	2020-10-12 16:16:32.963006
73045	TEI	2020-10-09	7.42	7.4507	7.37	7.39	204673	{}	\N	2020-10-12 16:16:32.968524
73046	TEI	2020-10-12	7.4	7.4311	7.34	7.39	134410	{}	\N	2020-10-12 16:16:32.973835
73047	TEUM	2020-09-28	0.6208	0.6598	0.6206	0.6501	296068	{}	\N	2020-10-12 16:16:35.144256
73048	TEUM	2020-09-29	0.65	0.7598	0.65	0.7315	4132765	{}	\N	2020-10-12 16:16:35.156395
73049	TEUM	2020-09-30	0.69	0.71	0.661	0.6768	1468240	{}	\N	2020-10-12 16:16:35.164789
73050	TEUM	2020-10-01	0.66	0.7199	0.66	0.7056	930669	{}	\N	2020-10-12 16:16:35.173402
73051	TEUM	2020-10-02	0.7056	0.753	0.688	0.74	929637	{}	\N	2020-10-12 16:16:35.179186
73052	TEUM	2020-10-05	0.755	0.7956	0.73	0.7878	1478709	{}	\N	2020-10-12 16:16:35.184644
73053	TEUM	2020-10-06	0.79	0.875	0.7774	0.831	2433428	{}	\N	2020-10-12 16:16:35.190121
73054	TEUM	2020-10-07	0.8479	0.85	0.8	0.8337	904561	{}	\N	2020-10-12 16:16:35.195508
73055	TEUM	2020-10-08	0.8339	0.849	0.8111	0.8357	611063	{}	\N	2020-10-12 16:16:35.200843
73056	TEUM	2020-10-09	0.81	0.8393	0.8	0.8177	465151	{}	\N	2020-10-12 16:16:35.206259
73057	TEUM	2020-10-12	0.819	0.819	0.78	0.78	381394	{}	\N	2020-10-12 16:16:35.21339
73058	TGA	2020-09-28	0.4	0.405	0.382	0.3834	120631	{}	\N	2020-10-12 16:16:36.131893
73059	TGA	2020-09-29	0.38	0.42	0.38	0.40935	305506	{}	\N	2020-10-12 16:16:36.144064
73060	TGA	2020-09-30	0.3825	0.42	0.3825	0.41	187339	{}	\N	2020-10-12 16:16:36.154243
73061	TGA	2020-10-01	0.41	0.41	0.3837	0.4	124580	{}	\N	2020-10-12 16:16:36.161158
73062	TGA	2020-10-02	0.3838	0.3956	0.383	0.395	159439	{}	\N	2020-10-12 16:16:36.166305
73063	TGA	2020-10-05	0.398	0.4148	0.39	0.4001	131390	{}	\N	2020-10-12 16:16:36.171697
73064	TGA	2020-10-06	0.3999	0.4182	0.3851	0.398	248337	{}	\N	2020-10-12 16:16:36.176818
73065	TGA	2020-10-07	0.3979	0.414	0.394	0.414	128979	{}	\N	2020-10-12 16:16:36.181916
73066	TGA	2020-10-08	0.419	0.58	0.4001	0.52	1294584	{}	\N	2020-10-12 16:16:36.187247
73067	TGA	2020-10-09	0.52	0.52	0.44	0.48	438318	{}	\N	2020-10-12 16:16:36.192948
73068	TGA	2020-10-12	0.475	0.475	0.44	0.4599	98438	{}	\N	2020-10-12 16:16:36.200042
73069	TGNA	2020-09-28	11.91	12.08	11.87	11.94	907351	{}	\N	2020-10-12 16:16:38.484359
73070	TGNA	2020-09-29	11.92	12.06	11.7001	11.78	1418361	{}	\N	2020-10-12 16:16:38.492338
73071	TGNA	2020-09-30	11.84	12.01	11.645	11.75	3298960	{}	\N	2020-10-12 16:16:38.498038
73072	TGNA	2020-10-01	11.74	11.89	11.33	11.47	2903069	{}	\N	2020-10-12 16:16:38.50439
73073	TGNA	2020-10-02	11.32	11.73	11.26	11.48	2829478	{}	\N	2020-10-12 16:16:38.510153
73074	TGNA	2020-10-05	11.61	11.82	11.6052	11.79	2201548	{}	\N	2020-10-12 16:16:38.515811
73075	TGNA	2020-10-06	11.86	12.09	11.73	11.81	3650675	{}	\N	2020-10-12 16:16:38.521643
73076	TGNA	2020-10-07	11.89	12.225	11.89	12.13	3856205	{}	\N	2020-10-12 16:16:38.527257
73077	TGNA	2020-10-08	12.28	12.35	12.16	12.34	2244979	{}	\N	2020-10-12 16:16:38.533419
73078	TGNA	2020-10-09	12.43	12.55	12.07	12.09	1646662	{}	\N	2020-10-12 16:16:38.540259
73079	TGNA	2020-10-12	12.01	12.11	11.92	12.1	533219	{}	\N	2020-10-12 16:16:38.545037
73080	THG	2020-09-28	92	94.08	91.925	92.79	210101	{}	\N	2020-10-12 16:16:39.537696
73081	THG	2020-09-29	92.37	92.63	90.96	92	356286	{}	\N	2020-10-12 16:16:39.549076
73082	THG	2020-09-30	92.57	93.8	92	93.18	500207	{}	\N	2020-10-12 16:16:39.556394
73083	THG	2020-10-01	93.45	94.43	92.655	93.58	471537	{}	\N	2020-10-12 16:16:39.564089
73084	THG	2020-10-02	92.02	95.09	92.02	93.64	470783	{}	\N	2020-10-12 16:16:39.570336
73085	THG	2020-10-05	94.8	96.554	94.8	95.86	450019	{}	\N	2020-10-12 16:16:39.575255
73086	THG	2020-10-06	96.78	96.93	94.87	95.17	435392	{}	\N	2020-10-12 16:16:39.580012
73087	THG	2020-10-07	95.68	97.075	95.65	96.22	360163	{}	\N	2020-10-12 16:16:39.584592
73088	THG	2020-10-08	97.01	97.7	95.98	96.54	247522	{}	\N	2020-10-12 16:16:39.589113
73089	THG	2020-10-09	96.92	96.96	95.82	95.94	230022	{}	\N	2020-10-12 16:16:39.593628
73090	THG	2020-10-12	95.93	98.4	95.785	97.885	91783	{}	\N	2020-10-12 16:16:39.598593
73091	TIPT	2020-09-28	4.86	5.1	4.86	4.99	56054	{}	\N	2020-10-12 16:16:41.092718
73092	TIPT	2020-09-29	4.94	4.95	4.865	4.94	21562	{}	\N	2020-10-12 16:16:41.105643
73093	TIPT	2020-09-30	4.92	5	4.92	4.95	29950	{}	\N	2020-10-12 16:16:41.112449
73094	TIPT	2020-10-01	4.97	4.9798	4.84	4.89	36061	{}	\N	2020-10-12 16:16:41.118249
73095	TIPT	2020-10-02	4.86	4.95	4.83	4.91	31125	{}	\N	2020-10-12 16:16:41.123572
73096	TIPT	2020-10-05	4.9	5	4.9	5	63738	{}	\N	2020-10-12 16:16:41.128293
73097	TIPT	2020-10-06	4.99	5.1	4.97	4.97	58506	{}	\N	2020-10-12 16:16:41.132447
73098	TIPT	2020-10-07	4.98	5.15	4.98	5.1	71545	{}	\N	2020-10-12 16:16:41.136985
73099	TIPT	2020-10-08	5.1	5.24	5.1	5.18	82553	{}	\N	2020-10-12 16:16:41.141106
73100	TIPT	2020-10-09	5.19	5.34	5.16	5.18	68913	{}	\N	2020-10-12 16:16:41.146108
73101	TIPT	2020-10-12	5.18	5.2	5.16	5.17	12150	{}	\N	2020-10-12 16:16:41.151637
73102	TAPR	2020-09-28	87.0615	87.0615	87.0615	87.0615	149	{}	\N	2020-10-12 16:16:43.299629
73103	TAPR	2020-09-29	89.94	89.94	89.94	89.94	102	{}	\N	2020-10-12 16:16:43.307067
73104	TAPR	2020-09-30	90.105	90.105	90.105	90.105	110	{}	\N	2020-10-12 16:16:43.313253
73105	TAPR	2020-10-01	92.875	92.875	92.875	92.875	107	{}	\N	2020-10-12 16:16:43.318541
73106	TAPR	2020-10-02	98.06	98.06	98.06	98.06	100	{}	\N	2020-10-12 16:16:43.324067
73107	TAPR	2020-10-05	97.01	97.01	87.855	87.855	318	{}	\N	2020-10-12 16:16:43.329746
73108	TAPR	2020-10-06	88.475	88.475	88.475	88.475	101	{}	\N	2020-10-12 16:16:43.335293
73109	TAPR	2020-10-07	88.87	88.87	88.87	88.87	102	{}	\N	2020-10-12 16:16:43.340296
73110	TAPR	2020-10-08	89.125	89.125	89.125	89.125	100	{}	\N	2020-10-12 16:16:43.346331
73111	TAPR	2020-10-09	89.29	89.29	89.29	89.29	207	{}	\N	2020-10-12 16:16:43.35204
73112	TBF	2020-09-28	15.22	15.27	15.22	15.25	2886480	{}	\N	2020-10-12 16:16:44.730861
73113	TBF	2020-09-29	15.22	15.2587	15.19	15.24	129071	{}	\N	2020-10-12 16:16:44.747039
73114	TBF	2020-09-30	15.3	15.44	15.3	15.36	2994884	{}	\N	2020-10-12 16:16:44.753746
73115	TBF	2020-10-01	15.44	15.46	15.32	15.34	2913175	{}	\N	2020-10-12 16:16:44.760507
73116	TBF	2020-10-02	15.33	15.46	15.33	15.4	198306	{}	\N	2020-10-12 16:16:44.766718
73117	TBF	2020-10-05	15.57	15.71	15.57	15.71	484757	{}	\N	2020-10-12 16:16:44.770967
73118	TBF	2020-10-06	15.74	15.78	15.55	15.63	668058	{}	\N	2020-10-12 16:16:44.775005
73119	TBF	2020-10-07	15.72	15.7799	15.625	15.75	429923	{}	\N	2020-10-12 16:16:44.779291
73120	TBF	2020-10-08	15.67	15.7	15.645	15.65	380908	{}	\N	2020-10-12 16:16:44.784659
73121	TBF	2020-10-09	15.72	15.77	15.641	15.67	369900	{}	\N	2020-10-12 16:16:44.788922
73122	TBF	2020-10-12	15.63	15.66	15.6186	15.65	107909	{}	\N	2020-10-12 16:16:44.792923
73123	TBX	2020-09-28	23.6401	23.65	23.64	23.645	2974	{}	\N	2020-10-12 16:16:47.13859
73124	TBX	2020-09-29	23.62	23.65	23.61	23.63	324059	{}	\N	2020-10-12 16:16:47.150439
73125	TBX	2020-09-30	23.69	23.71	23.685	23.685	6620	{}	\N	2020-10-12 16:16:47.1583
73126	TBX	2020-10-01	23.71	23.71	23.67	23.67	26873	{}	\N	2020-10-12 16:16:47.164963
73127	TBX	2020-10-02	23.65	23.71	23.65	23.71	15075	{}	\N	2020-10-12 16:16:47.171057
73128	TBX	2020-10-05	23.75	23.84	23.75	23.84	16234	{}	\N	2020-10-12 16:16:47.176352
73129	TBX	2020-10-06	23.83	23.84	23.75	23.7899	14117	{}	\N	2020-10-12 16:16:47.182335
73130	TBX	2020-10-07	23.83	23.855	23.8207	23.855	5092	{}	\N	2020-10-12 16:16:47.188114
73131	TBX	2020-10-08	23.83	23.83	23.815	23.815	15417	{}	\N	2020-10-12 16:16:47.193641
73132	TBX	2020-10-09	23.81	23.8699	23.81	23.835	4007	{}	\N	2020-10-12 16:16:47.199121
73133	TBX	2020-10-12	23.81	23.8309	23.81	23.8309	1773	{}	\N	2020-10-12 16:16:47.204345
73134	TCF	2020-09-28	23.37	23.96	22.86	23.73	560594	{}	\N	2020-10-12 16:16:49.979713
73135	TCF	2020-09-29	23.64	23.72	22.67	23.28	615693	{}	\N	2020-10-12 16:16:50.024205
73136	TCF	2020-09-30	23.38	24.07	23.1	23.36	487481	{}	\N	2020-10-12 16:16:50.035472
73137	TCF	2020-10-01	23.63	23.97	23.21	23.81	577173	{}	\N	2020-10-12 16:16:50.056796
73138	TCF	2020-10-02	23.01	24.73	23.01	24.53	578141	{}	\N	2020-10-12 16:16:50.065862
73139	TCF	2020-10-05	25.608	25.62	24.68	25.56	858829	{}	\N	2020-10-12 16:16:50.074565
73140	TCF	2020-10-06	25.93	26.99	25.55	25.68	780860	{}	\N	2020-10-12 16:16:50.083342
73141	TCF	2020-10-07	25.74	26.98	25.74	26.31	1007715	{}	\N	2020-10-12 16:16:50.091609
73142	TCF	2020-10-08	26.65	27.47	26.39	27.32	803088	{}	\N	2020-10-12 16:16:50.099494
73143	TCF	2020-10-09	27.67	27.87	26.975	27.29	758271	{}	\N	2020-10-12 16:16:50.108571
73144	TCF	2020-10-12	27.29	27.72	27.15	27.41	158798	{}	\N	2020-10-12 16:16:50.11626
73145	TDC	2020-09-28	23.21	23.5	23	23.2	770728	{}	\N	2020-10-12 16:16:52.372173
73146	TDC	2020-09-29	23.21	23.29	22.77	22.81	1116161	{}	\N	2020-10-12 16:16:52.384381
73147	TDC	2020-09-30	22.8	23.21	22.53	22.7	1658633	{}	\N	2020-10-12 16:16:52.392049
73148	TDC	2020-10-01	22.66	22.89	22.14	22.23	1802795	{}	\N	2020-10-12 16:16:52.397855
73149	TDC	2020-10-02	21.7	22.44	21.68	21.7	1565638	{}	\N	2020-10-12 16:16:52.403126
73150	TDC	2020-10-05	21.91	21.96	21.32	21.5	2675260	{}	\N	2020-10-12 16:16:52.407748
73151	TDC	2020-10-06	21.66	22.53	21.54	21.96	2101410	{}	\N	2020-10-12 16:16:52.413157
73152	TDC	2020-10-07	22.06	22.25	21.86	21.94	1084437	{}	\N	2020-10-12 16:16:52.419736
73153	TDC	2020-10-08	22.07	22.52	22.07	22.38	1319760	{}	\N	2020-10-12 16:16:52.424816
73154	TDC	2020-10-09	22.63	22.65	21.9	22.01	1235970	{}	\N	2020-10-12 16:16:52.398219
73155	TDC	2020-10-12	22.12	22.35	21.94	22.15	326194	{}	\N	2020-10-12 16:16:52.40461
73156	TDTT	2020-09-28	25.8	25.855	25.8	25.845	154712	{}	\N	2020-10-12 16:16:53.436145
73157	TDTT	2020-09-29	25.87	25.88	25.85	25.88	39863	{}	\N	2020-10-12 16:16:53.446799
73158	TDTT	2020-09-30	25.8816	25.89	25.84	25.84	87184	{}	\N	2020-10-12 16:16:53.455227
73159	TDTT	2020-10-01	25.8	25.83	25.79	25.8	60922	{}	\N	2020-10-12 16:16:53.463755
73160	TDTT	2020-10-02	25.8	25.81	25.78	25.79	103006	{}	\N	2020-10-12 16:16:53.469451
73161	TDTT	2020-10-05	25.819	25.819	25.79	25.79	102112	{}	\N	2020-10-12 16:16:53.47507
73162	TDTT	2020-10-06	25.8	25.81	25.781	25.79	48597	{}	\N	2020-10-12 16:16:53.480727
73163	TDTT	2020-10-07	25.81	25.82	25.79	25.8098	81901	{}	\N	2020-10-12 16:16:53.486265
73164	TDTT	2020-10-08	25.837	25.86	25.83	25.86	75205	{}	\N	2020-10-12 16:16:53.49143
73165	TDTT	2020-10-09	25.87	25.87	25.84	25.86	61569	{}	\N	2020-10-12 16:16:53.499077
73166	TDTT	2020-10-12	25.85	25.865	25.85	25.85	3201	{}	\N	2020-10-12 16:16:53.504767
73167	TECL	2020-09-28	300	301.5	292.135	301.5	502413	{}	\N	2020-10-12 16:16:55.487801
73168	TECL	2020-09-29	300.4	303.71	296.74	298.39	325397	{}	\N	2020-10-12 16:16:55.497065
73169	TECL	2020-09-30	298.63	314.97	297.75	305.83	517692	{}	\N	2020-10-12 16:16:55.504471
73170	TECL	2020-10-01	318.15	319.693	309.7	315.53	482138	{}	\N	2020-10-12 16:16:55.513101
73171	TECL	2020-10-02	292.76	307.28	287.92	291.24	844337	{}	\N	2020-10-12 16:16:55.519771
73172	TECL	2020-10-05	297.75	311	296.99	310.93	349060	{}	\N	2020-10-12 16:16:55.526293
73173	TECL	2020-10-06	308.23	317.1	293	296.25	564832	{}	\N	2020-10-12 16:16:55.533033
73174	TECL	2020-10-07	305.29	314	305	312.09	270092	{}	\N	2020-10-12 16:16:55.539124
73175	TECL	2020-10-08	319.93	320.36	313.81	316.81	295775	{}	\N	2020-10-12 16:16:55.547874
73176	TECL	2020-10-09	322.01	331.95	321.5	331.95	323687	{}	\N	2020-10-12 16:16:55.554476
73177	TECL	2020-10-12	347.44	355.63	341.21	355.25	238605	{}	\N	2020-10-12 16:16:55.560209
73178	TEVA	2020-09-28	8.8	8.88	8.7	8.8	4411963	{}	\N	2020-10-12 16:16:56.768994
73179	TEVA	2020-09-29	8.79	8.885	8.705	8.85	10098051	{}	\N	2020-10-12 16:16:56.782201
73180	TEVA	2020-09-30	8.94	9.1975	8.89	9.01	18107153	{}	\N	2020-10-12 16:16:56.792144
73181	TEVA	2020-10-01	9.06	9.27	8.95	9.07	11257213	{}	\N	2020-10-12 16:16:56.798633
73182	TEVA	2020-10-02	8.83	9.07	8.811	9.02	8741611	{}	\N	2020-10-12 16:16:56.804874
73183	TEVA	2020-10-05	9.12	9.345	9.035	9.3	12521053	{}	\N	2020-10-12 16:16:56.811027
73184	TEVA	2020-10-06	9.34	9.49	9.14	9.2	12134166	{}	\N	2020-10-12 16:16:56.81615
73185	TEVA	2020-10-07	9.35	9.575	9.31	9.55	9371196	{}	\N	2020-10-12 16:16:56.82125
73186	TEVA	2020-10-08	9.7	10.005	9.66	9.94	15729641	{}	\N	2020-10-12 16:16:56.826441
73187	TEVA	2020-10-09	10.06	10.09	9.8	9.83	10117426	{}	\N	2020-10-12 16:16:56.832611
73188	TEVA	2020-10-12	9.81	9.81	9.48	9.615	8358683	{}	\N	2020-10-12 16:16:56.838199
73189	TFX	2020-09-28	341.93	345.95	340.16	341.89	160114	{}	\N	2020-10-12 16:16:57.813936
73190	TFX	2020-09-29	343.3	345.63	335.33	335.54	370703	{}	\N	2020-10-12 16:16:57.826987
73191	TFX	2020-09-30	336.62	344.54	336.62	340.42	335830	{}	\N	2020-10-12 16:16:57.835608
73192	TFX	2020-10-01	343.76	347.94	334.81	337.24	438451	{}	\N	2020-10-12 16:16:57.842623
73193	TFX	2020-10-02	334.61	337.87	330.42	335.9	336337	{}	\N	2020-10-12 16:16:57.848958
73194	TFX	2020-10-05	340.17	342.34	337.57	339.3	270308	{}	\N	2020-10-12 16:16:57.854518
73195	TFX	2020-10-06	339.03	342.74	335.64	336.37	353193	{}	\N	2020-10-12 16:16:57.860091
73196	TFX	2020-10-07	337.22	350.46	337.22	348.03	531555	{}	\N	2020-10-12 16:16:57.865067
73197	TFX	2020-10-08	350	355.23	348.79	354.34	393427	{}	\N	2020-10-12 16:16:57.870019
73198	TFX	2020-10-09	358.43	365.83	354.97	363.51	434293	{}	\N	2020-10-12 16:16:57.875481
73199	TFX	2020-10-12	367.4	367.4	360.31	362.99	66154	{}	\N	2020-10-12 16:16:57.88035
73200	TGS	2020-09-28	4.19	4.33	4.13	4.17	244689	{}	\N	2020-10-12 16:16:59.087705
73201	TGS	2020-09-29	4.19	4.2	4.05	4.05	424086	{}	\N	2020-10-12 16:16:59.101027
73202	TGS	2020-09-30	4.05	4.21	4.05	4.16	570143	{}	\N	2020-10-12 16:16:59.108852
73203	TGS	2020-10-01	4.14	4.33	4.08	4.28	562348	{}	\N	2020-10-12 16:16:59.115428
73204	TGS	2020-10-02	4.23	4.31	4.18	4.27	389081	{}	\N	2020-10-12 16:16:59.121544
73205	TGS	2020-10-05	4.28	4.39	4.22	4.36	231484	{}	\N	2020-10-12 16:16:59.126768
73206	TGS	2020-10-06	4.39	4.53	4.222	4.23	291812	{}	\N	2020-10-12 16:16:59.131987
73207	TGS	2020-10-07	4.23	4.32	4.08	4.23	568772	{}	\N	2020-10-12 16:16:59.136855
73208	TGS	2020-10-08	4.22	4.35	4.2033	4.33	336408	{}	\N	2020-10-12 16:16:59.141761
73209	TGS	2020-10-09	4.31	4.4	4.28	4.34	511011	{}	\N	2020-10-12 16:16:59.146586
73210	TGS	2020-10-12	4.33	4.43	4.23	4.43	284740	{}	\N	2020-10-12 16:16:59.15152
73211	TAP	2020-09-28	33.25	33.66	32.85	33.58	1799748	{}	\N	2020-10-12 16:17:00.432966
73212	TAP	2020-09-29	34.75	35.25	33.53	33.53	7632247	{}	\N	2020-10-12 16:17:00.446367
73213	TAP	2020-09-30	33.65	33.87	33.16	33.56	4837274	{}	\N	2020-10-12 16:17:00.454627
73214	TAP	2020-10-01	33.5	33.61	32.63	32.78	6038081	{}	\N	2020-10-12 16:17:00.46187
73215	TAP	2020-10-02	32.49	34.25	32.35	34.04	5516828	{}	\N	2020-10-12 16:17:00.468323
73216	TAP	2020-10-05	34.25	34.98	33.82	34.95	5242640	{}	\N	2020-10-12 16:17:00.473986
73217	TAP	2020-10-06	35.33	35.97	34.88	34.92	4550544	{}	\N	2020-10-12 16:17:00.479336
73218	TAP	2020-10-07	35.63	36.54	35.51	35.68	5823472	{}	\N	2020-10-12 16:17:00.485396
73219	TAP	2020-10-08	35.81	36.91	35.58	36.8	4529099	{}	\N	2020-10-12 16:17:00.49107
73220	TAP	2020-10-09	36.8	37.06	36.01	36.11	3276525	{}	\N	2020-10-12 16:17:00.496954
73221	TAP	2020-10-12	36.01	36.24	35.9101	36.09	1091920	{}	\N	2020-10-12 16:17:00.502155
73222	TBPH	2020-09-28	16.02	16.3498	15.46	15.56	297283	{}	\N	2020-10-12 16:17:01.772945
73223	TBPH	2020-09-29	15.49	15.53	14.95	15.07	553894	{}	\N	2020-10-12 16:17:01.783284
73224	TBPH	2020-09-30	15.48	15.55	14.62	14.785	900589	{}	\N	2020-10-12 16:17:01.790382
73225	TBPH	2020-10-01	14.73	15.42	14.48	14.89	416905	{}	\N	2020-10-12 16:17:01.796345
73226	TBPH	2020-10-02	14.57	15.18	14.57	14.98	223262	{}	\N	2020-10-12 16:17:01.801604
73227	TBPH	2020-10-05	15.13	15.515	14.885	15.33	235468	{}	\N	2020-10-12 16:17:01.80592
73228	TBPH	2020-10-06	15.45	15.6	14.88	14.9	178265	{}	\N	2020-10-12 16:17:01.809893
73229	TBPH	2020-10-07	14.96	16.78	14.52	16.39	697380	{}	\N	2020-10-12 16:17:01.814428
73230	TBPH	2020-10-08	16.61	17.92	15.95	17.32	524993	{}	\N	2020-10-12 16:17:01.818546
73231	TBPH	2020-10-09	17.41	18.37	17.05	18	445112	{}	\N	2020-10-12 16:17:01.822826
73232	TBPH	2020-10-12	17.98	18.56	16.18	17.845	66675	{}	\N	2020-10-12 16:17:01.826983
73233	TCFC	2020-09-28	20.7894	21.0325	20.67	21	2631	{}	\N	2020-10-12 16:17:02.827942
73234	TCFC	2020-09-29	21	21	21	21	2206	{}	\N	2020-10-12 16:17:02.841984
73235	TCFC	2020-09-30	21.26	21.47	20.69	21.35	1681	{}	\N	2020-10-12 16:17:02.850633
73236	TCFC	2020-10-01	20.93	21.95	20.92	21.28	5086	{}	\N	2020-10-12 16:17:02.856851
73237	TCFC	2020-10-02	21.21	21.55	20.6429	21.18	7460	{}	\N	2020-10-12 16:17:02.862005
73238	TCFC	2020-10-05	21.15	21.415	21	21.15	11294	{}	\N	2020-10-12 16:17:02.866814
73239	TCFC	2020-10-06	21.14	22.15	21.14	21.2	7689	{}	\N	2020-10-12 16:17:02.871832
73240	TCFC	2020-10-07	21.54	21.55	21	21.25	17360	{}	\N	2020-10-12 16:17:02.876557
73241	TCFC	2020-10-08	21.42	22.02	21.4	22.02	6331	{}	\N	2020-10-12 16:17:02.882481
73242	TCFC	2020-10-09	21.62	22.16	21.12	22.16	5652	{}	\N	2020-10-12 16:17:02.887307
73243	TCFC	2020-10-12	22	22.1	22	22.1	1038	{}	\N	2020-10-12 16:17:02.892208
73244	TCP	2020-09-28	26.11	26.675	25.14	26.07	255467	{}	\N	2020-10-12 16:17:03.886589
73245	TCP	2020-09-29	26.04	27.06	25.725	26.79	593765	{}	\N	2020-10-12 16:17:03.89804
73246	TCP	2020-09-30	26.73	27.12	25.43	25.58	1144725	{}	\N	2020-10-12 16:17:03.906561
73247	TCP	2020-10-01	25.42	25.59	24.71	25.24	590931	{}	\N	2020-10-12 16:17:03.912948
73248	TCP	2020-10-02	24.7	26.22	24.4	25.9	978696	{}	\N	2020-10-12 16:17:03.919216
73249	TCP	2020-10-05	27.19	29.27	27.01	29.02	3013107	{}	\N	2020-10-12 16:17:03.92429
73250	TCP	2020-10-06	29.04	29.57	28	28.41	2187746	{}	\N	2020-10-12 16:17:03.929421
73251	TCP	2020-10-07	28.4	29.14	28.4	29.02	1105696	{}	\N	2020-10-12 16:17:03.934525
73252	TCP	2020-10-08	29.17	30.31	29.099	30.25	767223	{}	\N	2020-10-12 16:17:03.939928
73253	TCP	2020-10-09	30.31	30.67	30.175	30.38	777129	{}	\N	2020-10-12 16:17:03.945265
73254	TCP	2020-10-12	30.5	30.5	29.93	29.9788	117701	{}	\N	2020-10-12 16:17:03.950581
73255	TDA	2020-09-28	25.245	25.77	25.2437	25.77	39256	{}	\N	2020-10-12 16:17:04.911572
73256	TDA	2020-09-29	25.57	25.69	25.45	25.69	43588	{}	\N	2020-10-12 16:17:04.923954
73257	TDA	2020-09-30	25.65	25.65	25.38	25.6	140814	{}	\N	2020-10-12 16:17:04.93162
73258	TDA	2020-10-01	25.06	25.58	25.06	25.52	35836	{}	\N	2020-10-12 16:17:04.937813
73259	TDA	2020-10-02	25.45	25.55	25.45	25.54	29896	{}	\N	2020-10-12 16:17:04.943077
73260	TDA	2020-10-05	25.54	25.54	25.28	25.3	22454	{}	\N	2020-10-12 16:17:04.948512
73261	TDA	2020-10-06	25.37	25.52	25.31	25.48	11770	{}	\N	2020-10-12 16:17:04.953418
73262	TDA	2020-10-07	25.5	25.5	25.37	25.39	8676	{}	\N	2020-10-12 16:17:04.95784
73263	TDA	2020-10-08	25.39	25.49	25.39	25.49	16090	{}	\N	2020-10-12 16:17:04.962591
73264	TDA	2020-10-09	25.44	25.44	25.22	25.37	9364	{}	\N	2020-10-12 16:17:04.967265
73265	TDA	2020-10-12	25.25	25.39	25.25	25.39	11226	{}	\N	2020-10-12 16:17:04.972007
73266	TDS	2020-09-28	18.84	19.04	18.8	19.01	638768	{}	\N	2020-10-12 16:17:07.014656
73267	TDS	2020-09-29	19.06	19.06	18.73	18.79	1337767	{}	\N	2020-10-12 16:17:07.024819
73268	TDS	2020-09-30	18.82	18.87	18.29	18.44	1605702	{}	\N	2020-10-12 16:17:07.031155
73269	TDS	2020-10-01	18.42	18.52	18.07	18.38	1127257	{}	\N	2020-10-12 16:17:07.036287
73270	TDS	2020-10-02	18.09	18.48	18.04	18.22	1502760	{}	\N	2020-10-12 16:17:07.041272
73271	TDS	2020-10-05	18.43	18.6	17.97	18.44	1474778	{}	\N	2020-10-12 16:17:07.046462
73272	TDS	2020-10-06	18.59	18.67	18.18	18.25	1363451	{}	\N	2020-10-12 16:17:07.050679
73273	TDS	2020-10-07	18.33	18.55	18.1	18.5	926953	{}	\N	2020-10-12 16:17:07.055876
73274	TDS	2020-10-08	18.65	19.03	18.52	18.93	1292730	{}	\N	2020-10-12 16:17:07.059712
73275	TDS	2020-10-09	19.01	19.35	18.84	19.05	1664775	{}	\N	2020-10-12 16:17:07.063588
73276	TDS	2020-10-12	19.12	19.19	18.96	19.0799	211978	{}	\N	2020-10-12 16:17:07.0681
73277	TERM	2020-09-28	15.4114	15.4114	15.4114	15.4114	4	{}	\N	2020-10-12 16:17:10.123643
73278	TERM	2020-09-29	15.35	15.35	15.3236	15.3236	402	{}	\N	2020-10-12 16:17:10.133021
73279	TERM	2020-09-30	15.4568	15.4568	15.4568	15.4568	65	{}	\N	2020-10-12 16:17:10.139599
73280	TERM	2020-10-01	15.451	15.451	15.41	15.41	1722	{}	\N	2020-10-12 16:17:10.145731
73281	TERM	2020-10-02	15.493	15.493	15.493	15.493	38	{}	\N	2020-10-12 16:17:10.15233
73282	TERM	2020-10-05	15.67	15.7372	15.65	15.7372	4902	{}	\N	2020-10-12 16:17:10.157682
73283	TERM	2020-10-06	15.83	15.85	15.5874	15.5874	5780	{}	\N	2020-10-12 16:17:10.162951
73284	TERM	2020-10-07	15.8947	15.8947	15.8947	15.8947	51	{}	\N	2020-10-12 16:17:10.168346
73285	TERM	2020-10-08	15.91	16.09	15.91	16.08	2683	{}	\N	2020-10-12 16:17:10.173975
73286	TERM	2020-10-09	16.1393	16.1393	16.09	16.09	6232	{}	\N	2020-10-12 16:17:10.179607
73287	TG	2020-09-28	14.8	15.37	14.8	15.07	242753	{}	\N	2020-10-12 16:17:12.340354
73288	TG	2020-09-29	15.05	15.09	14.76	15	204677	{}	\N	2020-10-12 16:17:12.354572
73289	TG	2020-09-30	15.09	15.32	14.8	14.87	315453	{}	\N	2020-10-12 16:17:12.363526
73290	TG	2020-10-01	14.85	15.15	14.65	15.05	321573	{}	\N	2020-10-12 16:17:12.370206
73291	TG	2020-10-02	14.76	15.63	14.76	15.4	138246	{}	\N	2020-10-12 16:17:12.376184
73292	TG	2020-10-05	15.49	15.93	15.49	15.89	186075	{}	\N	2020-10-12 16:17:12.38112
73293	TG	2020-10-06	16.07	16.39	15.69	15.69	197356	{}	\N	2020-10-12 16:17:12.386302
73294	TG	2020-10-07	15.85	15.93	15.54	15.84	227968	{}	\N	2020-10-12 16:17:12.391377
73295	TG	2020-10-08	15.89	16.015	15.74	15.87	142392	{}	\N	2020-10-12 16:17:12.396554
73296	TG	2020-10-09	16.05	16.06	15.65	15.81	144225	{}	\N	2020-10-12 16:17:12.401716
73297	TG	2020-10-12	15.81	15.81	15.65	15.77	29082	{}	\N	2020-10-12 16:17:12.406587
73298	TGP	2020-09-28	10.5	10.56	10.27	10.35	258476	{}	\N	2020-10-12 16:17:13.47429
73299	TGP	2020-09-29	10.43	10.45	10.295	10.38	711461	{}	\N	2020-10-12 16:17:13.485075
73300	TGP	2020-09-30	10.35	10.665	10.35	10.5	558462	{}	\N	2020-10-12 16:17:13.493582
73301	TGP	2020-10-01	10.4	10.585	10.2573	10.39	355205	{}	\N	2020-10-12 16:17:13.500025
73302	TGP	2020-10-02	10.29	10.57	10.27	10.56	226824	{}	\N	2020-10-12 16:17:13.505685
73303	TGP	2020-10-05	10.55	10.77	10.52	10.67	282752	{}	\N	2020-10-12 16:17:13.510775
73304	TGP	2020-10-06	10.67	10.83	10.44	10.58	326726	{}	\N	2020-10-12 16:17:13.515487
73305	TGP	2020-10-07	10.6	10.71	10.35	10.5	439859	{}	\N	2020-10-12 16:17:13.520202
73306	TGP	2020-10-08	10.56	11.04	10.51	11.04	532143	{}	\N	2020-10-12 16:17:13.524932
73307	TGP	2020-10-09	11.12	11.19	10.798	10.86	297352	{}	\N	2020-10-12 16:17:13.529324
73308	TGP	2020-10-12	10.98	10.98	10.56	10.845	146368	{}	\N	2020-10-12 16:17:13.533978
73309	THM	2020-09-28	1.37	1.37	1.3	1.31	189391	{}	\N	2020-10-12 16:17:14.477669
73310	THM	2020-09-29	1.3	1.37	1.3	1.36	162298	{}	\N	2020-10-12 16:17:14.48849
73311	THM	2020-09-30	1.37	1.38	1.3	1.33	156140	{}	\N	2020-10-12 16:17:14.496433
73312	THM	2020-10-01	1.33	1.3399	1.27	1.28	194297	{}	\N	2020-10-12 16:17:14.502588
73313	THM	2020-10-02	1.27	1.3	1.245	1.27	301005	{}	\N	2020-10-12 16:17:14.50851
73314	THM	2020-10-05	1.36	1.42	1.27	1.34	592570	{}	\N	2020-10-12 16:17:14.513556
73315	THM	2020-10-06	1.4	1.4899	1.34	1.36	705610	{}	\N	2020-10-12 16:17:14.518406
73316	THM	2020-10-07	1.36	1.37	1.31	1.34	161885	{}	\N	2020-10-12 16:17:14.523215
73317	THM	2020-10-08	1.34	1.37	1.32	1.33	253222	{}	\N	2020-10-12 16:17:14.52997
73318	THM	2020-10-09	1.39	1.43	1.36	1.41	516044	{}	\N	2020-10-12 16:17:14.535418
73319	THM	2020-10-12	1.44	1.448	1.37	1.39	155162	{}	\N	2020-10-12 16:17:14.540943
73320	TK	2020-09-28	2.2124	2.22	2.09	2.18	564655	{}	\N	2020-10-12 16:17:15.457804
73321	TK	2020-09-29	2.2	2.2	2.12	2.19	946656	{}	\N	2020-10-12 16:17:15.469821
73322	TK	2020-09-30	2.2	2.31	2.2	2.23	854588	{}	\N	2020-10-12 16:17:15.47744
73323	TK	2020-10-01	2.24	2.31	2.21	2.3	771116	{}	\N	2020-10-12 16:17:15.484048
73324	TK	2020-10-02	2.28	2.3	2.215	2.28	644435	{}	\N	2020-10-12 16:17:15.489998
73325	TK	2020-10-05	2.28	2.3499	2.24	2.32	482156	{}	\N	2020-10-12 16:17:15.495521
73326	TK	2020-10-06	2.31	2.3801	2.25	2.28	637452	{}	\N	2020-10-12 16:17:15.501583
73327	TK	2020-10-07	2.29	2.295	2.18	2.23	1033128	{}	\N	2020-10-12 16:17:15.507743
73328	TK	2020-10-08	2.25	2.375	2.25	2.37	1053040	{}	\N	2020-10-12 16:17:15.513388
73329	TK	2020-10-09	2.37	2.39	2.25	2.25	912422	{}	\N	2020-10-12 16:17:15.519101
73330	TK	2020-10-12	2.24	2.3	2.16	2.1801	409095	{}	\N	2020-10-12 16:17:15.524958
73331	TLI	2020-09-28	9.22	9.24	9.22	9.23	14992	{}	\N	2020-10-12 16:17:17.222357
73332	TLI	2020-09-29	9.23	9.26	9.23	9.24	17564	{}	\N	2020-10-12 16:17:17.233528
73333	TLI	2020-09-30	9.24	9.28	9.24	9.26	23706	{}	\N	2020-10-12 16:17:17.242099
73334	TLI	2020-10-01	9.27	9.29	9.25	9.27	36170	{}	\N	2020-10-12 16:17:17.24906
73335	TLI	2020-10-02	9.27	9.3	9.24	9.27	57130	{}	\N	2020-10-12 16:17:17.254833
73336	TLI	2020-10-05	9.27	9.3	9.27	9.3	32570	{}	\N	2020-10-12 16:17:17.25961
73337	TLI	2020-10-06	9.28	9.31	9.28	9.29	49518	{}	\N	2020-10-12 16:17:17.264891
73338	TLI	2020-10-07	9.28	9.305	9.27	9.27	80008	{}	\N	2020-10-12 16:17:17.27018
73339	TLI	2020-10-08	9.28	9.3199	9.27	9.27	59250	{}	\N	2020-10-12 16:17:17.275434
73340	TLI	2020-10-09	9.29	9.29	9.27	9.27	35608	{}	\N	2020-10-12 16:17:17.280445
73341	TLI	2020-10-12	9.27	9.3	9.27	9.27	56332	{}	\N	2020-10-12 16:17:17.285501
73342	TM	2020-09-28	135.04	136.31	135.04	136.01	219490	{}	\N	2020-10-12 16:17:19.863436
73343	TM	2020-09-29	135.07	135.75	134.185	134.25	274727	{}	\N	2020-10-12 16:17:19.871585
73344	TM	2020-09-30	132.65	133.26	132.12	132.44	315438	{}	\N	2020-10-12 16:17:19.878074
73345	TM	2020-10-01	132.63	132.83	132.02	132.43	249801	{}	\N	2020-10-12 16:17:19.884769
73346	TM	2020-10-02	130.98	132.13	130.8828	131.73	193101	{}	\N	2020-10-12 16:17:19.890855
73347	TM	2020-10-05	131.48	132.18	131.25	132.1	477051	{}	\N	2020-10-12 16:17:19.895807
73348	TM	2020-10-06	131.925	132.63	131.16	131.26	503323	{}	\N	2020-10-12 16:17:19.901398
73349	TM	2020-10-07	132.68	132.68	132.01	132.43	229420	{}	\N	2020-10-12 16:17:19.906017
73350	TM	2020-10-08	132.93	133.14	132.47	132.95	195813	{}	\N	2020-10-12 16:17:19.910474
73351	TM	2020-10-09	132.22	132.36	131.9716	132.08	214709	{}	\N	2020-10-12 16:17:19.915346
73352	TM	2020-10-12	132	132.22	131.62	131.78	123183	{}	\N	2020-10-12 16:17:19.920138
73353	TAT	2020-09-28	0.22	0.2389	0.2132	0.217	1047247	{}	\N	2020-10-12 16:17:20.982889
73354	TAT	2020-09-29	0.236	0.3125	0.22	0.273	19500475	{}	\N	2020-10-12 16:17:20.995872
73355	TAT	2020-09-30	0.26	0.26	0.22	0.2324	5668082	{}	\N	2020-10-12 16:17:21.004702
73356	TAT	2020-10-01	0.2301	0.252	0.2255	0.2366	2724660	{}	\N	2020-10-12 16:17:21.011244
73357	TAT	2020-10-02	0.244	0.267	0.235	0.251	4452640	{}	\N	2020-10-12 16:17:21.017056
73358	TAT	2020-10-05	0.2491	0.26	0.24	0.2451	1620939	{}	\N	2020-10-12 16:17:21.022197
73359	TAT	2020-10-06	0.25	0.28	0.241	0.2543	7891637	{}	\N	2020-10-12 16:17:21.026992
73360	TAT	2020-10-07	0.245	0.268	0.24	0.26	3972079	{}	\N	2020-10-12 16:17:21.031754
73361	TAT	2020-10-08	0.23	0.3444	0.23	0.3299	16046666	{}	\N	2020-10-12 16:17:21.037139
73362	TAT	2020-10-09	0.34	0.3499	0.26	0.2929	13504914	{}	\N	2020-10-12 16:17:21.042928
73363	TAT	2020-10-12	0.2835	0.2835	0.261	0.2616	1284198	{}	\N	2020-10-12 16:17:21.048408
73364	TCBI	2020-09-28	30.34	30.84	29.89	30.775	496483	{}	\N	2020-10-12 16:17:23.308132
73365	TCBI	2020-09-29	30.74	30.88	29.87	30.61	205531	{}	\N	2020-10-12 16:17:23.320101
73366	TCBI	2020-09-30	31.02	31.92	30.675	31.13	253116	{}	\N	2020-10-12 16:17:23.327476
73367	TCBI	2020-10-01	31.08	31.78	30.985	31.68	237657	{}	\N	2020-10-12 16:17:23.334367
73368	TCBI	2020-10-02	30.78	33.37	30.76	33.15	271223	{}	\N	2020-10-12 16:17:23.339981
73369	TCBI	2020-10-05	33.71	35.741	33.7	35.58	381651	{}	\N	2020-10-12 16:17:23.34538
73370	TCBI	2020-10-06	36.03	37.71	35.34	35.5	396567	{}	\N	2020-10-12 16:17:23.350722
73371	TCBI	2020-10-07	36.13	38.01	36.13	36.98	448912	{}	\N	2020-10-12 16:17:23.355475
73372	TCBI	2020-10-08	36.03	37.82	35.93	37.75	249217	{}	\N	2020-10-12 16:17:23.360828
73373	TCBI	2020-10-09	38.38	38.71	36.445	36.52	277530	{}	\N	2020-10-12 16:17:23.366206
73374	TCBI	2020-10-12	36.58	37.14	35.735	36.92	27287	{}	\N	2020-10-12 16:17:23.371524
73375	TCMD	2020-09-28	34.15	35.215	33.86	34.93	216609	{}	\N	2020-10-12 16:17:27.441996
73376	TCMD	2020-09-29	35.08	36.18	34.51	35.75	186440	{}	\N	2020-10-12 16:17:27.451762
73377	TCMD	2020-09-30	35.54	37.14	35.49	36.59	216205	{}	\N	2020-10-12 16:17:27.459142
73378	TCMD	2020-10-01	37.12	37.25	36.26	36.91	181356	{}	\N	2020-10-12 16:17:27.466254
73379	TCMD	2020-10-02	36.24	37.45	36	37.19	161983	{}	\N	2020-10-12 16:17:27.471999
73380	TCMD	2020-10-05	37.54	38.21	37.3	37.94	139011	{}	\N	2020-10-12 16:17:27.476882
73381	TCMD	2020-10-06	38.49	40.13	38.07	38.69	148233	{}	\N	2020-10-12 16:17:27.482166
73382	TCMD	2020-10-07	39.33	39.7688	38.65	39.34	179078	{}	\N	2020-10-12 16:17:27.48731
73383	TCMD	2020-10-08	39.94	40.27	39.38	39.53	107271	{}	\N	2020-10-12 16:17:27.492331
73384	TCMD	2020-10-09	39.43	39.97	35.21	35.64	306800	{}	\N	2020-10-12 16:17:27.497505
73385	TCMD	2020-10-12	35.33	36.75	34.76	36.45	63169	{}	\N	2020-10-12 16:17:27.502589
73386	TCTL	2020-09-28	26.909	26.984	26.909	26.984	3826	{}	\N	2020-10-12 16:17:35.644141
73387	TCTL	2020-09-29	26.97	26.97	26.86	26.86	952	{}	\N	2020-10-12 16:17:35.651457
73388	TCTL	2020-09-30	26.95	27.06	26.95	27.015	48699	{}	\N	2020-10-12 16:17:35.656616
73389	TCTL	2020-10-01	27.1018	27.1193	27.1018	27.1193	293	{}	\N	2020-10-12 16:17:35.661529
73390	TCTL	2020-10-02	26.98	27.0498	26.81	27.0498	12017	{}	\N	2020-10-12 16:17:35.667017
73391	TCTL	2020-10-05	27.3394	27.3394	27.3394	27.3394	183	{}	\N	2020-10-12 16:17:35.673081
73392	TCTL	2020-10-06	27.355	27.355	27.1736	27.1736	569	{}	\N	2020-10-12 16:17:35.678803
73393	TCTL	2020-10-07	27.4408	27.4408	27.4408	27.4408	100	{}	\N	2020-10-12 16:17:35.684539
73394	TCTL	2020-10-08	27.5892	27.6047	27.58	27.6047	2138	{}	\N	2020-10-12 16:17:35.690417
73395	TCTL	2020-10-09	27.68	27.7518	27.67	27.7518	18850	{}	\N	2020-10-12 16:17:35.695976
73396	TCTL	2020-10-12	27.9771	27.9771	27.9771	27.9771	1479	{}	\N	2020-10-12 16:17:35.702271
73397	TDIV	2020-09-28	43.58	43.854	43.43	43.85	98629	{}	\N	2020-10-12 16:17:38.609378
73398	TDIV	2020-09-29	43.82	43.91	43.5138	43.64	94092	{}	\N	2020-10-12 16:17:38.625657
73399	TDIV	2020-09-30	43.71	44.21	43.65	43.78	84460	{}	\N	2020-10-12 16:17:38.634246
73400	TDIV	2020-10-01	44.24	44.351	43.95	44.14	56091	{}	\N	2020-10-12 16:17:38.639857
73401	TDIV	2020-10-02	43.25	43.819	43.226	43.4	361043	{}	\N	2020-10-12 16:17:38.644596
73402	TDIV	2020-10-05	43.73	44.27	43.73	44.25	52121	{}	\N	2020-10-12 16:17:38.649428
73403	TDIV	2020-10-06	44.3	44.76	43.7885	43.91	48139	{}	\N	2020-10-12 16:17:38.654154
73404	TDIV	2020-10-07	44.24	44.6604	44.24	44.59	48178	{}	\N	2020-10-12 16:17:38.658751
73405	TDIV	2020-10-08	45.1505	45.2771	45	45.2	56415	{}	\N	2020-10-12 16:17:38.663108
73406	TDIV	2020-10-09	45.45	45.59	45.3551	45.49	54833	{}	\N	2020-10-12 16:17:38.667613
73407	TDIV	2020-10-12	45.99	46.0799	45.7596	46.0799	19132	{}	\N	2020-10-12 16:17:38.672185
73408	TEAM	2020-09-28	182.02	184.035	178.7	181	2108828	{}	\N	2020-10-12 16:17:40.580305
73409	TEAM	2020-09-29	181.63	183.38	179.56	182.35	903554	{}	\N	2020-10-12 16:17:40.592354
73410	TEAM	2020-09-30	180.54	185.15	179.21	181.79	2646692	{}	\N	2020-10-12 16:17:40.600694
73411	TEAM	2020-10-01	183.22	188.0601	180.57	186.01	963259	{}	\N	2020-10-12 16:17:40.607253
73412	TEAM	2020-10-02	182.31	189	182.31	184.79	1665147	{}	\N	2020-10-12 16:17:40.613217
73413	TEAM	2020-10-05	186	191.63	185.27	186.85	1428690	{}	\N	2020-10-12 16:17:40.619315
73414	TEAM	2020-10-06	186.79	194.61	186.59	189.88	1358810	{}	\N	2020-10-12 16:17:40.625119
73415	TEAM	2020-10-07	191.21	195.5	190.055	193.04	843306	{}	\N	2020-10-12 16:17:40.630605
73416	TEAM	2020-10-08	194.52	194.754	188.25	188.78	1224215	{}	\N	2020-10-12 16:17:40.636645
73417	TEAM	2020-10-09	190.35	199.87	190	198.85	1336299	{}	\N	2020-10-12 16:17:40.64195
73418	TEAM	2020-10-12	199.95	199.95	192.21	193.39	1083045	{}	\N	2020-10-12 16:17:40.647171
73419	TEDU	2020-09-28	1.67	2.06	1.67	1.81	2040591	{}	\N	2020-10-12 16:17:41.628389
73420	TEDU	2020-09-29	1.74	1.83	1.7	1.72	190896	{}	\N	2020-10-12 16:17:41.640756
73421	TEDU	2020-09-30	1.69	1.81	1.69	1.76	100472	{}	\N	2020-10-12 16:17:41.648217
73422	TEDU	2020-10-01	1.77	1.84	1.75	1.7893	96521	{}	\N	2020-10-12 16:17:41.655223
73423	TEDU	2020-10-02	1.74	1.79	1.6	1.65	276188	{}	\N	2020-10-12 16:17:41.662453
73424	TEDU	2020-10-05	1.66	1.75	1.66	1.7	53492	{}	\N	2020-10-12 16:17:41.667322
73425	TEDU	2020-10-06	1.69	1.77	1.69	1.69	70957	{}	\N	2020-10-12 16:17:41.672816
73426	TEDU	2020-10-07	1.71	1.77	1.71	1.73	76235	{}	\N	2020-10-12 16:17:41.678328
73427	TEDU	2020-10-08	1.74	1.77	1.73	1.75	47216	{}	\N	2020-10-12 16:17:41.683676
73428	TEDU	2020-10-09	1.75	1.78	1.72	1.76	28337	{}	\N	2020-10-12 16:17:41.688958
73429	TEDU	2020-10-12	1.775	1.82	1.77	1.79	106757	{}	\N	2020-10-12 16:17:41.693964
73430	TFSL	2020-09-28	14.56	14.97	14.55	14.91	218110	{}	\N	2020-10-12 16:17:44.810812
73431	TFSL	2020-09-29	14.98	14.98	14.51	14.74	157032	{}	\N	2020-10-12 16:17:44.82466
73432	TFSL	2020-09-30	14.63	14.86	14.59	14.69	206590	{}	\N	2020-10-12 16:17:44.83287
73433	TFSL	2020-10-01	14.69	14.9	14.57	14.88	191788	{}	\N	2020-10-12 16:17:44.838979
73434	TFSL	2020-10-02	14.77	15.635	14.76	15.59	289506	{}	\N	2020-10-12 16:17:44.844417
73435	TFSL	2020-10-05	15.77	15.77	15.47	15.62	193911	{}	\N	2020-10-12 16:17:44.849233
73436	TFSL	2020-10-06	15.6	15.81	15.44	15.56	237305	{}	\N	2020-10-12 16:17:44.854218
73437	TFSL	2020-10-07	15.81	16.05	15.63	15.95	274377	{}	\N	2020-10-12 16:17:44.859308
73438	TFSL	2020-10-08	16.05	16.35	15.97	16.33	156695	{}	\N	2020-10-12 16:17:44.864016
73439	TFSL	2020-10-09	16.4	16.44	16.15	16.36	228484	{}	\N	2020-10-12 16:17:44.868534
73440	TFSL	2020-10-12	16.23	16.35	16.03	16.03	98831	{}	\N	2020-10-12 16:17:44.872888
73441	TGI	2020-09-28	7.12	7.41	6.98	7.01	2335979	{}	\N	2020-10-12 16:17:46.071549
73442	TGI	2020-09-29	6.99	6.99	6.58	6.63	3123584	{}	\N	2020-10-12 16:17:46.079582
73443	TGI	2020-09-30	6.73	6.97	6.46	6.51	3431513	{}	\N	2020-10-12 16:17:46.08588
73444	TGI	2020-10-01	6.65	7.39	6.62	7.2	5732018	{}	\N	2020-10-12 16:17:46.091978
73445	TGI	2020-10-02	6.85	7.27	6.81	7.22	4868194	{}	\N	2020-10-12 16:17:46.097489
73446	TGI	2020-10-05	7.39	7.6	7.09	7.09	3372339	{}	\N	2020-10-12 16:17:46.102996
73447	TGI	2020-10-06	7.29	7.3771	6.82	6.82	4254929	{}	\N	2020-10-12 16:17:46.108365
73448	TGI	2020-10-07	6.96	7.305	6.86	7.12	3561807	{}	\N	2020-10-12 16:17:46.113642
73449	TGI	2020-10-08	7.33	7.47	7.05	7.35	5117540	{}	\N	2020-10-12 16:17:46.119162
73450	TGI	2020-10-09	7.41	7.53	7.12	7.2	2736163	{}	\N	2020-10-12 16:17:46.124527
73451	TGI	2020-10-12	7.19	7.227	7.04	7.11	849750	{}	\N	2020-10-12 16:17:46.129643
73452	THRM	2020-09-28	40.16	41.29	40.16	41.05	166672	{}	\N	2020-10-12 16:17:47.192663
73453	THRM	2020-09-29	40.96	41.89	40.59	40.955	221349	{}	\N	2020-10-12 16:17:47.206913
73454	THRM	2020-09-30	40.93	41.4365	40.51	40.9	212402	{}	\N	2020-10-12 16:17:47.216245
73455	THRM	2020-10-01	41.15	41.87	41.01	41.87	168648	{}	\N	2020-10-12 16:17:47.222199
73456	THRM	2020-10-02	41.4	43.27	41.4	42.97	156589	{}	\N	2020-10-12 16:17:47.227316
73457	THRM	2020-10-05	43.37	44.64	43.37	44.59	114251	{}	\N	2020-10-12 16:17:47.232333
73458	THRM	2020-10-06	45.07	47.49	45.07	45.96	244225	{}	\N	2020-10-12 16:17:47.23674
73459	THRM	2020-10-07	46.39	47.45	45.795	46.49	181119	{}	\N	2020-10-12 16:17:47.241571
73460	THRM	2020-10-08	47.05	47.05	46.08	46.64	112455	{}	\N	2020-10-12 16:17:47.246395
73461	THRM	2020-10-09	46.97	47.48	46.65	46.84	87563	{}	\N	2020-10-12 16:17:47.251167
73462	THRM	2020-10-12	46.9	47.18	46.61	47.14	20492	{}	\N	2020-10-12 16:17:47.255897
73463	TITN	2020-09-28	12.8	13.28	12.8	13.09	87613	{}	\N	2020-10-12 16:17:48.268125
73464	TITN	2020-09-29	13.02	13.05	12.75	12.9	138673	{}	\N	2020-10-12 16:17:48.282524
73465	TITN	2020-09-30	12.94	13.46	12.94	13.23	148004	{}	\N	2020-10-12 16:17:48.290077
73466	TITN	2020-10-01	13.24	13.79	13.04	13.68	331873	{}	\N	2020-10-12 16:17:48.297176
73467	TITN	2020-10-02	13.26	14.39	13.12	14.31	261312	{}	\N	2020-10-12 16:17:48.302895
73468	TITN	2020-10-05	14.52	15.04	14.44	14.95	284667	{}	\N	2020-10-12 16:17:48.308368
73469	TITN	2020-10-06	15	15.38	14.5	14.57	257171	{}	\N	2020-10-12 16:17:48.313583
73470	TITN	2020-10-07	14.75	15.32	14.44	15.08	181365	{}	\N	2020-10-12 16:17:48.318634
73471	TITN	2020-10-08	15.26	15.325	14.86	15.24	85073	{}	\N	2020-10-12 16:17:48.32478
73472	TITN	2020-10-09	15.36	15.36	14.97	15.22	104071	{}	\N	2020-10-12 16:17:48.33072
73473	TITN	2020-10-12	15.21	15.44	15.03	15.165	32790	{}	\N	2020-10-12 16:17:48.336235
73474	TAYD	2020-09-28	9.27	9.3124	9.27	9.3124	959	{}	\N	2020-10-12 16:17:49.318139
73475	TAYD	2020-09-29	9.28	9.4	9.049	9.1	3932	{}	\N	2020-10-12 16:17:49.330498
73476	TAYD	2020-09-30	9.49	9.5	9.49	9.5	711	{}	\N	2020-10-12 16:17:49.338998
73477	TAYD	2020-10-01	9.425	9.425	9.21	9.21	1314	{}	\N	2020-10-12 16:17:49.345989
73478	TAYD	2020-10-02	9.12	9.465	9.12	9.45	1861	{}	\N	2020-10-12 16:17:49.352314
73479	TAYD	2020-10-05	9.4392	9.48	9.228	9.25	1470	{}	\N	2020-10-12 16:17:49.358566
73480	TAYD	2020-10-06	9.22	9.22	9.22	9.22	277	{}	\N	2020-10-12 16:17:49.364475
73481	TAYD	2020-10-07	9.4245	9.4668	9.22	9.22	1905	{}	\N	2020-10-12 16:17:49.370075
73482	TAYD	2020-10-08	9.39	9.39	9.2	9.2	445	{}	\N	2020-10-12 16:17:49.375683
73483	TAYD	2020-10-09	9.3	9.4923	9.03	9.4923	3864	{}	\N	2020-10-12 16:17:49.381371
73484	TAYD	2020-10-12	9.4	9.4	9.1	9.2275	368	{}	\N	2020-10-12 16:17:49.38688
73485	TCCO	2020-09-28	3.1999	3.3629	3.1999	3.31	8373	{}	\N	2020-10-12 16:17:50.464745
73486	TCCO	2020-09-29	3.38	3.5313	3.255	3.375	39766	{}	\N	2020-10-12 16:17:50.47612
73487	TCCO	2020-09-30	3.39	3.448	3.3	3.41	9538	{}	\N	2020-10-12 16:17:50.483322
73488	TCCO	2020-10-01	3.335	3.3706	3.32	3.36	5888	{}	\N	2020-10-12 16:17:50.489582
73489	TCCO	2020-10-02	3.37	3.44	3.34	3.42	6303	{}	\N	2020-10-12 16:17:50.495787
73490	TCCO	2020-10-05	3.45	3.55	3.36	3.36	12438	{}	\N	2020-10-12 16:17:50.50101
73491	TCCO	2020-10-06	3.43	3.51	3.3427	3.37	11064	{}	\N	2020-10-12 16:17:50.505986
73492	TCCO	2020-10-07	3.416	3.4893	3.3927	3.41	6480	{}	\N	2020-10-12 16:17:50.511601
73493	TCCO	2020-10-08	3.47	3.55	3.4498	3.46	9202	{}	\N	2020-10-12 16:17:50.517017
73494	TCCO	2020-10-09	3.45	3.5	3.4353	3.45	4682	{}	\N	2020-10-12 16:17:50.522168
73495	TCCO	2020-10-12	3.36	3.547	3.36	3.547	4036	{}	\N	2020-10-12 16:17:50.527263
73496	TCS	2020-09-28	6.19	6.55	6.14	6.17	583131	{}	\N	2020-10-12 16:17:54.443792
73497	TCS	2020-09-29	6.27	7.09	6.27	6.49	2756896	{}	\N	2020-10-12 16:17:54.454215
73498	TCS	2020-09-30	6.47	6.66	6.08	6.21	2484178	{}	\N	2020-10-12 16:17:54.462361
73499	TCS	2020-10-01	6.32	7.82	6.32	7.79	7899561	{}	\N	2020-10-12 16:17:54.468681
73500	TCS	2020-10-02	7.43	8.15	7.31	7.93	3439784	{}	\N	2020-10-12 16:17:54.474862
73501	TCS	2020-10-05	8.12	8.72	8.0201	8.7	4995052	{}	\N	2020-10-12 16:17:54.480074
73502	TCS	2020-10-06	8.77	9.32	8.65	9	3555539	{}	\N	2020-10-12 16:17:54.485092
73503	TCS	2020-10-07	9.16	9.72	8.26	8.8	3442984	{}	\N	2020-10-12 16:17:54.490851
73504	TCS	2020-10-08	8.77	9.32	8.2	8.57	2893501	{}	\N	2020-10-12 16:17:54.496104
73505	TCS	2020-10-09	8.55	8.84	8.3838	8.55	1117432	{}	\N	2020-10-12 16:17:54.502215
73506	TCS	2020-10-12	8.63	8.74	8.42	8.59	354285	{}	\N	2020-10-12 16:17:54.508271
73507	TDG	2020-09-28	502.34	511.44	498.36	498.6	398598	{}	\N	2020-10-12 16:17:55.53156
73508	TDG	2020-09-29	501.28	501.28	488.5	489.08	1157893	{}	\N	2020-10-12 16:17:55.541416
73509	TDG	2020-09-30	489	494.8	471.22	475.12	1381909	{}	\N	2020-10-12 16:17:55.550088
73510	TDG	2020-10-01	481.53	492.47	475.165	481.34	1136369	{}	\N	2020-10-12 16:17:55.557691
73511	TDG	2020-10-02	468.56	489.06	468.56	485.58	861333	{}	\N	2020-10-12 16:17:55.564911
73512	TDG	2020-10-05	490	494.65	486.49	487	731444	{}	\N	2020-10-12 16:17:55.572734
73513	TDG	2020-10-06	493.06	503.005	480.41	480.54	866665	{}	\N	2020-10-12 16:17:55.579619
73514	TDG	2020-10-07	488.81	496.61	487.5	488.5	654294	{}	\N	2020-10-12 16:17:55.585673
73515	TDG	2020-10-08	492.09	494.01	483.195	490.97	518371	{}	\N	2020-10-12 16:17:55.591019
73516	TDG	2020-10-09	493.91	494.33	484.21	485	381795	{}	\N	2020-10-12 16:17:55.597189
73517	TDG	2020-10-12	486.78	488.09	482.45	486.65	165914	{}	\N	2020-10-12 16:17:55.60316
73518	TEF	2020-09-28	3.49	3.53	3.47	3.52	1835406	{}	\N	2020-10-12 16:17:57.017847
73519	TEF	2020-09-29	3.45	3.46	3.4	3.41	4641451	{}	\N	2020-10-12 16:17:57.028127
73520	TEF	2020-09-30	3.47	3.515	3.44	3.44	2951438	{}	\N	2020-10-12 16:17:57.034222
73521	TEF	2020-10-01	3.44	3.49	3.425	3.45	5661444	{}	\N	2020-10-12 16:17:57.040253
73522	TEF	2020-10-02	3.4	3.48	3.385	3.48	4639471	{}	\N	2020-10-12 16:17:57.04522
73523	TEF	2020-10-05	3.67	3.79	3.66	3.77	5369691	{}	\N	2020-10-12 16:17:57.05009
73524	TEF	2020-10-06	3.88	3.92	3.83	3.83	9354462	{}	\N	2020-10-12 16:17:57.054592
73525	TEF	2020-10-07	3.8	3.81	3.73	3.75	3861707	{}	\N	2020-10-12 16:17:57.058825
73526	TEF	2020-10-08	3.83	3.88	3.825	3.88	2026726	{}	\N	2020-10-12 16:17:57.063172
73527	TEF	2020-10-09	3.85	3.86	3.79	3.86	5378350	{}	\N	2020-10-12 16:17:57.067622
73528	TEF	2020-10-12	3.75	3.76	3.715	3.76	2210933	{}	\N	2020-10-12 16:17:57.071444
73529	TFLO	2020-09-28	50.3	50.3	50.2901	50.2983	14650	{}	\N	2020-10-12 16:17:58.583593
73530	TFLO	2020-09-29	50.2996	50.3	50.29	50.29	31947	{}	\N	2020-10-12 16:17:58.595326
73531	TFLO	2020-09-30	50.3	50.306	50.29	50.29	37885	{}	\N	2020-10-12 16:17:58.601893
73532	TFLO	2020-10-01	50.2909	50.3	50.28	50.29	47147	{}	\N	2020-10-12 16:17:58.607875
73533	TFLO	2020-10-02	50.29	50.3	50.29	50.295	17527	{}	\N	2020-10-12 16:17:58.613687
73534	TFLO	2020-10-05	50.28	50.3	50.28	50.29	50107	{}	\N	2020-10-12 16:17:58.618638
73535	TFLO	2020-10-06	50.2938	50.3	50.27	50.29	166075	{}	\N	2020-10-12 16:17:58.623607
73536	TFLO	2020-10-07	50.28	50.3	50.27	50.3	27602	{}	\N	2020-10-12 16:17:58.628305
73537	TFLO	2020-10-08	50.28	50.3	50.28	50.29	26772	{}	\N	2020-10-12 16:17:58.632922
73538	TFLO	2020-10-09	50.28	50.2999	50.28	50.295	25564	{}	\N	2020-10-12 16:17:58.637654
73539	TFLO	2020-10-12	50.28	50.3	50.28	50.29	12813	{}	\N	2020-10-12 16:17:58.642528
73540	TGT	2020-09-28	154.95	157.71	154.65	156.46	3571373	{}	\N	2020-10-12 16:17:59.625402
73541	TGT	2020-09-29	156.75	158.43	155.87	156.94	5558682	{}	\N	2020-10-12 16:17:59.635407
73542	TGT	2020-09-30	157.45	159.15	156.5044	157.42	5624922	{}	\N	2020-10-12 16:17:59.642688
73543	TGT	2020-10-01	158.41	160.89	158.1	158.6	7489837	{}	\N	2020-10-12 16:17:59.649639
73544	TGT	2020-10-02	156.97	159.99	156.16	159.1	3853470	{}	\N	2020-10-12 16:17:59.655683
73545	TGT	2020-10-05	159.99	162.31	159.99	161.49	4560521	{}	\N	2020-10-12 16:17:59.660817
73546	TGT	2020-10-06	161.47	161.98	157.61	158.41	5801093	{}	\N	2020-10-12 16:17:59.665662
73547	TGT	2020-10-07	160.06	161.61	159.61	160.68	5682805	{}	\N	2020-10-12 16:17:59.670663
73548	TGT	2020-10-08	162.6	163.32	161.39	161.86	4351625	{}	\N	2020-10-12 16:17:59.67549
73549	TGT	2020-10-09	162.2	164.81	162.1	163.71	5116845	{}	\N	2020-10-12 16:17:59.680611
73550	TGT	2020-10-12	164.3	164.88	163.62	164.76	1775675	{}	\N	2020-10-12 16:17:59.685534
73551	TIPZ	2020-09-28	64.89	64.95	64.85	64.93	10531	{}	\N	2020-10-12 16:18:02.687936
73552	TIPZ	2020-09-29	65.03	65.12	65.0201	65.06	29114	{}	\N	2020-10-12 16:18:02.700153
73553	TIPZ	2020-09-30	64.99	65.0299	64.865	64.865	7913	{}	\N	2020-10-12 16:18:02.707157
73554	TIPZ	2020-10-01	64.57	64.73	64.57	64.715	3562	{}	\N	2020-10-12 16:18:02.712951
73555	TIPZ	2020-10-02	64.67	64.68	64.61	64.65	7677	{}	\N	2020-10-12 16:18:02.71836
73556	TIPZ	2020-10-05	64.575	64.6	64.42	64.42	7776	{}	\N	2020-10-12 16:18:02.723471
73557	TIPZ	2020-10-06	64.48	64.52	64.35	64.47	7627	{}	\N	2020-10-12 16:18:02.727715
73558	TIPZ	2020-10-07	64.42	64.4832	64.3767	64.42	12360	{}	\N	2020-10-12 16:18:02.732101
73559	TIPZ	2020-10-08	64.53	64.68	64.525	64.68	5304	{}	\N	2020-10-12 16:18:02.736668
73560	TIPZ	2020-10-09	64.615	64.71	64.5966	64.62	3388	{}	\N	2020-10-12 16:18:02.741403
73561	TIPZ	2020-10-12	64.73	64.7883	64.72	64.7883	2719	{}	\N	2020-10-12 16:18:02.74601
73562	TJX	2020-09-28	54.06	55.88	54.01	55.6	7673495	{}	\N	2020-10-12 16:18:03.883375
73563	TJX	2020-09-29	55.01	55.19	53.62	54.16	12938428	{}	\N	2020-10-12 16:18:03.893677
73564	TJX	2020-09-30	54.62	56.575	54.5799	55.65	12666899	{}	\N	2020-10-12 16:18:03.899764
73565	TJX	2020-10-01	56.02	57.385	55.89	57.08	11829829	{}	\N	2020-10-12 16:18:03.905375
73566	TJX	2020-10-02	55.62	57.81	55.58	57.63	8683363	{}	\N	2020-10-12 16:18:03.910232
73567	TJX	2020-10-05	58.04	58.46	57.07	57.52	7016368	{}	\N	2020-10-12 16:18:03.914642
73568	TJX	2020-10-06	57.8	58.46	56.5	56.61	10846906	{}	\N	2020-10-12 16:18:03.918338
73569	TJX	2020-10-07	57.48	58.83	57.36	58.36	8803834	{}	\N	2020-10-12 16:18:03.921914
73570	TJX	2020-10-08	58.25	58.6	57.625	58.3	8980277	{}	\N	2020-10-12 16:18:03.9261
73571	TJX	2020-10-09	58.85	59.16	57.9	58.19	11253364	{}	\N	2020-10-12 16:18:03.930173
73572	TJX	2020-10-12	58.34	58.48	57.83	58.15	2579481	{}	\N	2020-10-12 16:18:03.934301
73573	TLGT	2020-09-28	0.703	0.798	0.702	0.7511	223633	{}	\N	2020-10-12 16:18:04.807248
73574	TLGT	2020-09-29	0.82	0.8422	0.7	0.757	1791535	{}	\N	2020-10-12 16:18:04.821384
73575	TLGT	2020-09-30	0.7002	0.74	0.7002	0.7201	361507	{}	\N	2020-10-12 16:18:04.830161
73576	TLGT	2020-10-01	0.712	0.768	0.7	0.71	263317	{}	\N	2020-10-12 16:18:04.836763
73577	TLGT	2020-10-02	0.71	0.73	0.7031	0.73	293152	{}	\N	2020-10-12 16:18:04.842715
73578	TLGT	2020-10-05	0.74	0.7463	0.709	0.719	201631	{}	\N	2020-10-12 16:18:04.84838
73579	TLGT	2020-10-06	0.7301	0.7327	0.6801	0.7014	282509	{}	\N	2020-10-12 16:18:04.853467
73580	TLGT	2020-10-07	0.72	0.7456	0.6777	0.71	533358	{}	\N	2020-10-12 16:18:04.858686
73581	TLGT	2020-10-08	0.728	0.7455	0.69	0.7	1337321	{}	\N	2020-10-12 16:18:04.863903
73582	TLGT	2020-10-09	0.69	0.7089	0.6501	0.6597	855889	{}	\N	2020-10-12 16:18:04.868702
73583	TLGT	2020-10-12	0.689	0.689	0.6531	0.6831	293901	{}	\N	2020-10-12 16:18:04.873864
73584	YELP	2020-09-28	20.27	20.53	19.97	20.13	765355	{}	\N	2020-10-12 16:18:07.198662
73585	YELP	2020-09-29	20.26	20.64	19.73	20.07	1605460	{}	\N	2020-10-12 16:18:07.207606
73586	YELP	2020-09-30	20.15	20.83	19.93	20.09	3250136	{}	\N	2020-10-12 16:18:07.21472
73587	YELP	2020-10-01	20.18	20.82	19.83	20.81	2277341	{}	\N	2020-10-12 16:18:07.221102
73588	YELP	2020-10-02	20.2	21.0725	19.83	20.87	3818692	{}	\N	2020-10-12 16:18:07.22693
73589	YELP	2020-10-05	21.01	21.9	20.959	21.31	5501065	{}	\N	2020-10-12 16:18:07.232595
73590	YELP	2020-10-06	21.47	21.56	20.57	20.66	2209452	{}	\N	2020-10-12 16:18:07.23777
73591	YELP	2020-10-07	20.89	21.42	20.83	21.2	1563999	{}	\N	2020-10-12 16:18:07.243075
73592	YELP	2020-10-08	21.46	21.52	20.97	21.07	1084349	{}	\N	2020-10-12 16:18:07.248336
73593	YELP	2020-10-09	21.35	21.35	20.6	20.72	1528875	{}	\N	2020-10-12 16:18:07.253204
73594	YELP	2020-10-12	20.87	21.24	20.87	21.06	467973	{}	\N	2020-10-12 16:18:07.257723
73595	TBT	2020-09-28	15.4	15.5	15.4	15.45	529797	{}	\N	2020-10-12 16:18:08.175614
73596	TBT	2020-09-29	15.43	15.4585	15.335	15.41	533328	{}	\N	2020-10-12 16:18:08.18437
73597	TBT	2020-09-30	15.57	15.845	15.57	15.7	1829724	{}	\N	2020-10-12 16:18:08.193232
73598	TBT	2020-10-01	15.82	15.89	15.59	15.66	1159014	{}	\N	2020-10-12 16:18:08.201432
73599	TBT	2020-10-02	15.6	15.88	15.6	15.77	891991	{}	\N	2020-10-12 16:18:08.208561
73600	TBT	2020-10-05	16.11	16.41	16.11	16.39	1968225	{}	\N	2020-10-12 16:18:08.217006
73601	TBT	2020-10-06	16.45	16.57	16.03	16.22	3079707	{}	\N	2020-10-12 16:18:08.224619
73602	TBT	2020-10-07	16.39	16.535	16.23	16.47	1508908	{}	\N	2020-10-12 16:18:08.231559
73603	TBT	2020-10-08	16.32	16.37	16.257	16.27	1370519	{}	\N	2020-10-12 16:18:08.23873
73604	TBT	2020-10-09	16.4	16.545	16.255	16.3	1058024	{}	\N	2020-10-12 16:18:08.245819
73605	TBT	2020-10-12	16.2	16.3	16.2	16.295	641781	{}	\N	2020-10-12 16:18:08.253897
73606	TCI	2020-09-28	23.23	23.23	23.23	23.23	627	{}	\N	2020-10-12 16:18:10.692326
73607	TCI	2020-09-29	24.4058	25.25	24.25	24.76	14712	{}	\N	2020-10-12 16:18:10.701203
73608	TCI	2020-09-30	24.9	25.5673	24.78	24.93	4116	{}	\N	2020-10-12 16:18:10.706935
73609	TCI	2020-10-01	25.18	25.18	24.4	24.4	2192	{}	\N	2020-10-12 16:18:10.711729
73610	TCI	2020-10-02	24.75	25.01	24.75	25.01	1096	{}	\N	2020-10-12 16:18:10.716644
73611	TCI	2020-10-05	25.2814	25.45	24.43	24.43	4544	{}	\N	2020-10-12 16:18:10.720885
73612	TCI	2020-10-06	23.76	23.76	23.76	23.76	1012	{}	\N	2020-10-12 16:18:10.724939
73613	TCI	2020-10-07	24.35	25.14	24.35	25.14	1970	{}	\N	2020-10-12 16:18:10.729344
73614	TCI	2020-10-08	24.8245	24.8245	23.88	23.88	1540	{}	\N	2020-10-12 16:18:10.733715
73615	TDE	2020-09-28	25.35	25.68	25.33	25.63	32621	{}	\N	2020-10-12 16:18:11.675851
73616	TDE	2020-09-29	25.63	25.71	25.56	25.66	40056	{}	\N	2020-10-12 16:18:11.686177
73617	TDE	2020-09-30	25.69	25.71	25.51	25.51	144534	{}	\N	2020-10-12 16:18:11.694021
73618	TDE	2020-10-01	25.53	25.68	25.53	25.67	21184	{}	\N	2020-10-12 16:18:11.700557
73619	TDE	2020-10-02	25.45	25.5554	25.41	25.44	15840	{}	\N	2020-10-12 16:18:11.706267
73620	TDE	2020-10-05	25.4402	25.64	25.4402	25.58	10572	{}	\N	2020-10-12 16:18:11.711179
73621	TDE	2020-10-06	25.46	25.65	25.45	25.65	26604	{}	\N	2020-10-12 16:18:11.715935
73622	TDE	2020-10-07	25.65	25.72	25.5909	25.72	14264	{}	\N	2020-10-12 16:18:11.720721
73623	TDE	2020-10-08	25.725	25.725	25.57	25.63	13602	{}	\N	2020-10-12 16:18:11.72585
73624	TDE	2020-10-09	25.63	25.71	25.6	25.71	14128	{}	\N	2020-10-12 16:18:11.73087
73625	TDE	2020-10-12	25.62	25.69	25.5722	25.69	8560	{}	\N	2020-10-12 16:18:11.736507
73626	TDW	2020-09-28	6.35	6.91	6.35	6.87	111815	{}	\N	2020-10-12 16:18:13.936783
73627	TDW	2020-09-29	6.87	6.87	6.4	6.81	665260	{}	\N	2020-10-12 16:18:13.950345
73628	TDW	2020-09-30	6.81	7.14	6.65	6.71	248066	{}	\N	2020-10-12 16:18:13.958664
73629	TDW	2020-10-01	6.6	6.76	6.44	6.6	322653	{}	\N	2020-10-12 16:18:13.964694
73630	TDW	2020-10-02	6.38	6.5026	6.26	6.41	264531	{}	\N	2020-10-12 16:18:13.970692
73631	TDW	2020-10-05	6.45	6.615	6.34	6.4	223505	{}	\N	2020-10-12 16:18:13.975877
73632	TDW	2020-10-06	6.52	6.71	6.2	6.2	212373	{}	\N	2020-10-12 16:18:13.980928
73633	TDW	2020-10-07	6.29	6.58	6.15	6.4	281368	{}	\N	2020-10-12 16:18:13.986011
73634	TDW	2020-10-08	6.46	6.61	6.39	6.58	325411	{}	\N	2020-10-12 16:18:13.990903
73635	TDW	2020-10-09	6.66	6.78	6.39	6.5	577636	{}	\N	2020-10-12 16:18:13.996003
73636	TDW	2020-10-12	6.41	6.41	6.21	6.2946	40312	{}	\N	2020-10-12 16:18:14.001298
73637	TEN	2020-09-28	6.55	6.92	6.55	6.83	856577	{}	\N	2020-10-12 16:18:15.654426
73638	TEN	2020-09-29	6.8	7	6.72	6.91	1180716	{}	\N	2020-10-12 16:18:15.665937
73639	TEN	2020-09-30	6.96	7.23	6.87	6.94	1521302	{}	\N	2020-10-12 16:18:15.674398
73640	TEN	2020-10-01	7.07	7.24	6.85	7.2	1444372	{}	\N	2020-10-12 16:18:15.680517
73641	TEN	2020-10-02	6.94	7.618	6.89	7.55	1607522	{}	\N	2020-10-12 16:18:15.686478
73642	TEN	2020-10-05	7.72	8.39	7.72	8.33	3459490	{}	\N	2020-10-12 16:18:15.691769
73643	TEN	2020-10-06	8.49	8.86	8.2603	8.36	2647133	{}	\N	2020-10-12 16:18:15.696794
73644	TEN	2020-10-07	8.54	9	8.5	8.95	2302778	{}	\N	2020-10-12 16:18:15.702134
73645	TEN	2020-10-08	9.1	9.38	8.89	9.03	2259724	{}	\N	2020-10-12 16:18:15.707509
73646	TEN	2020-10-09	9.11	9.3299	8.81	8.82	1413273	{}	\N	2020-10-12 16:18:15.712217
73647	TEN	2020-10-12	8.8	9.01	8.67	8.7684	712036	{}	\N	2020-10-12 16:18:15.716993
73648	THD	2020-09-28	63.73	64.1	63.26	63.78	66998	{}	\N	2020-10-12 16:18:17.681442
73649	THD	2020-09-29	63.8	63.81	63.23	63.73	42305	{}	\N	2020-10-12 16:18:17.692401
73650	THD	2020-09-30	62.35	63.34	62.35	62.74	176988	{}	\N	2020-10-12 16:18:17.69963
73651	THD	2020-10-01	63	63.335	62.5	63.19	110594	{}	\N	2020-10-12 16:18:17.706394
73652	THD	2020-10-02	62.35	63.13	62.3	62.45	46088	{}	\N	2020-10-12 16:18:17.712331
73653	THD	2020-10-05	63.24	63.95	63.24	63.67	210456	{}	\N	2020-10-12 16:18:17.717496
73654	THD	2020-10-06	64	64.665	63.83	64.04	176356	{}	\N	2020-10-12 16:18:17.722787
73655	THD	2020-10-07	64.67	65.24	64.58	64.93	102439	{}	\N	2020-10-12 16:18:17.727809
73656	THD	2020-10-08	65.39	65.925	65.23	65.65	44977	{}	\N	2020-10-12 16:18:17.732706
73657	THD	2020-10-09	65.34	65.9866	64.98	65.35	58447	{}	\N	2020-10-12 16:18:17.737774
73658	THD	2020-10-12	65.18	65.5	65.07	65.355	29752	{}	\N	2020-10-12 16:18:17.743192
73659	THS	2020-09-28	39.75	40.64	39.75	40.38	434642	{}	\N	2020-10-12 16:18:23.132368
73660	THS	2020-09-29	40.59	40.635	39.6	40.19	1043710	{}	\N	2020-10-12 16:18:23.14349
73661	THS	2020-09-30	40.5	40.67	40.31	40.53	1434575	{}	\N	2020-10-12 16:18:23.151066
73662	THS	2020-10-01	40.6	40.9	39.79	40.9	1213781	{}	\N	2020-10-12 16:18:23.157488
73663	THS	2020-10-02	40.66	41.84	40.66	41.53	1188768	{}	\N	2020-10-12 16:18:23.163534
73664	THS	2020-10-05	41.45	41.87	40.94	41.19	931558	{}	\N	2020-10-12 16:18:23.16845
73665	THS	2020-10-06	41.23	41.645	40.83	41.19	692163	{}	\N	2020-10-12 16:18:23.173469
73666	THS	2020-10-07	41.45	41.95	41.06	41.42	892464	{}	\N	2020-10-12 16:18:23.178806
73667	THS	2020-10-08	41.73	41.77	40.95	41.13	593843	{}	\N	2020-10-12 16:18:23.183574
73668	THS	2020-10-09	41.03	41.2	40.2	40.3	646348	{}	\N	2020-10-12 16:18:23.188287
73669	THS	2020-10-12	40.19	41.4	40.19	41.06	164177	{}	\N	2020-10-12 16:18:23.193194
73670	TILE	2020-09-28	6.26	6.54	6.26	6.38	633155	{}	\N	2020-10-12 16:18:26.626659
73671	TILE	2020-09-29	6.37	6.37	6.1094	6.18	422665	{}	\N	2020-10-12 16:18:26.638144
73672	TILE	2020-09-30	6.23	6.4	6.09	6.12	320513	{}	\N	2020-10-12 16:18:26.646957
73673	TILE	2020-10-01	6.12	6.32	6.08	6.3	423503	{}	\N	2020-10-12 16:18:26.653534
73674	TILE	2020-10-02	6.12	6.66	6.12	6.6	396982	{}	\N	2020-10-12 16:18:26.659553
73675	TILE	2020-10-05	6.7	6.9	6.7	6.78	333524	{}	\N	2020-10-12 16:18:26.664832
73676	TILE	2020-10-06	6.82	7.015	6.68	6.73	447171	{}	\N	2020-10-12 16:18:26.669852
73677	TILE	2020-10-07	6.78	6.83	6.54	6.71	479079	{}	\N	2020-10-12 16:18:26.674653
73678	TILE	2020-10-08	6.84	7.09	6.81	6.96	348305	{}	\N	2020-10-12 16:18:26.680091
73679	TILE	2020-10-09	7.03	7.0799	6.81	6.9	340599	{}	\N	2020-10-12 16:18:26.685556
73680	TILE	2020-10-12	6.88	7.02	6.815	6.94	64133	{}	\N	2020-10-12 16:18:26.690811
73681	TKR	2020-09-28	54.73	55.3	54.31	54.81	446002	{}	\N	2020-10-12 16:18:34.042727
73682	TKR	2020-09-29	54.63	55.21	54.21	54.54	536438	{}	\N	2020-10-12 16:18:34.052045
73683	TKR	2020-09-30	54.99	55.64	53.69	54.22	1048835	{}	\N	2020-10-12 16:18:34.058339
73684	TKR	2020-10-01	54.68	55.1	53.3	53.88	1132621	{}	\N	2020-10-12 16:18:34.064605
73685	TKR	2020-10-02	52.71	55.78	52.71	55.25	607762	{}	\N	2020-10-12 16:18:34.069743
73686	TKR	2020-10-05	55.92	57.78	55.92	57.65	945286	{}	\N	2020-10-12 16:18:34.075457
73687	TKR	2020-10-06	57.98	60.05	57.86	58	1393376	{}	\N	2020-10-12 16:18:34.080148
73688	TKR	2020-10-07	58.9	59.89	58.49	59.43	886628	{}	\N	2020-10-12 16:18:34.085142
73689	TKR	2020-10-08	59.94	60.12	59.2	59.81	754784	{}	\N	2020-10-12 16:18:34.089493
73690	TKR	2020-10-09	60.31	61.27	60.2	60.55	787614	{}	\N	2020-10-12 16:18:34.094408
73691	TKR	2020-10-12	60.96	61.32	60.13	60.432	130201	{}	\N	2020-10-12 16:18:34.099452
73692	TLND	2020-09-28	40.94	40.94	38.58	38.75	192852	{}	\N	2020-10-12 16:18:41.21875
73693	TLND	2020-09-29	38.67	39	38	38.33	275797	{}	\N	2020-10-12 16:18:41.228043
73694	TLND	2020-09-30	38.46	39.43	38.36	39.04	349089	{}	\N	2020-10-12 16:18:41.23456
73695	TLND	2020-10-01	39.82	40.96	39.65	40.95	226297	{}	\N	2020-10-12 16:18:41.24072
73696	TLND	2020-10-02	40.07	40.706	39.12	39.28	96095	{}	\N	2020-10-12 16:18:41.246086
73697	TLND	2020-10-05	39.3	39.95	38.6	39.3	70857	{}	\N	2020-10-12 16:18:41.251763
73698	TLND	2020-10-06	39.31	40.172	39.31	39.89	115578	{}	\N	2020-10-12 16:18:41.256255
73699	TLND	2020-10-07	40.1	40.43	39.77	40.15	171888	{}	\N	2020-10-12 16:18:41.260376
73700	TLND	2020-10-08	40.15	40.6	39.9391	40.13	122482	{}	\N	2020-10-12 16:18:41.26467
73701	TLND	2020-10-09	40.62	41.93	40.15	41.72	192633	{}	\N	2020-10-12 16:18:41.268924
73702	TLND	2020-10-12	41.78	42.65	41.78	42.14	84195	{}	\N	2020-10-12 16:18:41.273038
73703	AAL	2020-09-28	12.74	13.15	12.49	12.76	63528456	{}	\N	2020-10-12 16:18:42.600373
73704	AAL	2020-09-29	12.59	12.66	12.11	12.25	46379632	{}	\N	2020-10-12 16:18:42.611163
73705	AAL	2020-09-30	12.37	12.915	12.22	12.29	63938071	{}	\N	2020-10-12 16:18:42.617882
73706	AAL	2020-10-01	12.45	12.77	12.35	12.58	48404835	{}	\N	2020-10-12 16:18:42.623726
73707	AAL	2020-10-02	12.05	13.35	12	13	112056604	{}	\N	2020-10-12 16:18:42.628757
73708	AAL	2020-10-05	13.09	13.2	12.75	13.12	54026828	{}	\N	2020-10-12 16:18:42.633214
73709	AAL	2020-10-06	13.28	13.44	12.46	12.53	82861490	{}	\N	2020-10-12 16:18:42.637504
73710	AAL	2020-10-07	12.98	13.18	12.81	13.07	62391133	{}	\N	2020-10-12 16:18:42.642876
73711	AAL	2020-10-08	13.33	13.68	12.725	13.155	89358047	{}	\N	2020-10-12 16:18:42.646897
73712	AAL	2020-10-09	13.24	13.49	12.95	13.2	66130028	{}	\N	2020-10-12 16:18:42.651209
73713	AAL	2020-10-12	13.14	13.17	12.96	13.035	23995201	{}	\N	2020-10-12 16:18:42.655451
73714	TCX	2020-09-28	68.12	68.61	67.53	68.04	16727	{}	\N	2020-10-12 16:18:47.306803
73715	TCX	2020-09-29	68.825	69.85	68.48	69.54	24748	{}	\N	2020-10-12 16:18:47.317549
73716	TCX	2020-09-30	69.56	69.8	68.44	68.9	29624	{}	\N	2020-10-12 16:18:47.325176
73717	TCX	2020-10-01	69.1	71.58	69.1	71.15	37571	{}	\N	2020-10-12 16:18:47.332664
73718	TCX	2020-10-02	70.51	71.9199	70.5	71.11	103842	{}	\N	2020-10-12 16:18:47.339405
73719	TCX	2020-10-05	71.25	73.22	71.25	72.7	39626	{}	\N	2020-10-12 16:18:47.345149
73720	TCX	2020-10-06	73.26	74.01	71.87	72.35	37877	{}	\N	2020-10-12 16:18:47.350684
73721	TCX	2020-10-07	72.78	73.25	72.06	72.64	33902	{}	\N	2020-10-12 16:18:47.356514
73722	TCX	2020-10-08	73.15	74.3	72.97	74.03	25798	{}	\N	2020-10-12 16:18:47.362985
73723	TCX	2020-10-09	74.64	75	72.7	73.85	25846	{}	\N	2020-10-12 16:18:47.369365
73724	TCX	2020-10-12	73.83	74.15	73.22	73.22	4237	{}	\N	2020-10-12 16:18:47.374753
73725	TDOC	2020-09-28	222.03	222.03	212.01	215.09	2322980	{}	\N	2020-10-12 16:18:51.317847
73726	TDOC	2020-09-29	216.5	221.9354	213.01	217.34	4306820	{}	\N	2020-10-12 16:18:51.331466
73727	TDOC	2020-09-30	217.27	224.9399	214.39	219.24	5317514	{}	\N	2020-10-12 16:18:51.340927
73728	TDOC	2020-10-01	222.86	223.74	218	220.68	3659243	{}	\N	2020-10-12 16:18:51.347886
73729	TDOC	2020-10-02	215	227.9499	213.6201	221.6	4689152	{}	\N	2020-10-12 16:18:51.353722
73730	TDOC	2020-10-05	223.39	227.48	220.1418	225.21	3370290	{}	\N	2020-10-12 16:18:51.359336
73731	TDOC	2020-10-06	223.29	225.7979	217	217.72	3545023	{}	\N	2020-10-12 16:18:51.365115
73732	TDOC	2020-10-07	222.15	230.8699	221.1897	222.85	4560230	{}	\N	2020-10-12 16:18:51.370043
73733	TDOC	2020-10-08	225.28	225.8	219.64	221.48	2560267	{}	\N	2020-10-12 16:18:51.375589
73734	TDOC	2020-10-09	223.88	224.78	217.87	218.11	3196106	{}	\N	2020-10-12 16:18:51.380786
73735	TDOC	2020-10-12	221.9	224.74	218.25	222.745	1637987	{}	\N	2020-10-12 16:18:51.385977
73736	TER	2020-09-28	78.66	79.49	77.57	79.2	1774089	{}	\N	2020-10-12 16:18:52.635674
73737	TER	2020-09-29	79.52	80.67	79.485	79.86	1126783	{}	\N	2020-10-12 16:18:52.646526
73738	TER	2020-09-30	79.33	80.395	78.81	79.46	1709991	{}	\N	2020-10-12 16:18:52.653873
73739	TER	2020-10-01	80.84	81.65	80.37	81.3	1136073	{}	\N	2020-10-12 16:18:52.660431
73740	TER	2020-10-02	78.75	79.89	78.3115	78.62	962734	{}	\N	2020-10-12 16:18:52.665712
73741	TER	2020-10-05	79.5	81.31	79.48	81.23	1275924	{}	\N	2020-10-12 16:18:52.670651
73742	TER	2020-10-06	81.27	83.14	80.57	81.11	1097450	{}	\N	2020-10-12 16:18:52.676172
73743	TER	2020-10-07	82.35	83.32	81.85	82.29	1195708	{}	\N	2020-10-12 16:18:52.681567
73744	TER	2020-10-08	82.6	83.3772	82.165	82.48	1122857	{}	\N	2020-10-12 16:18:52.686901
73745	TER	2020-10-09	85.36	86.835	84.61	86.1	1745599	{}	\N	2020-10-12 16:18:52.692117
73746	TER	2020-10-12	87.26	87.53	85.79	86.225	879353	{}	\N	2020-10-12 16:18:52.69701
73747	TGB	2020-09-28	1.09	1.09	1.02	1.05	1114001	{}	\N	2020-10-12 16:18:53.925678
73748	TGB	2020-09-29	1.06	1.07	1	1.03	857580	{}	\N	2020-10-12 16:18:53.938591
73749	TGB	2020-09-30	1.05	1.1	1.022	1.06	1176192	{}	\N	2020-10-12 16:18:53.946604
73750	TGB	2020-10-01	1.05	1.08	0.9965	1.02	1306901	{}	\N	2020-10-12 16:18:53.953383
73751	TGB	2020-10-02	0.98	1.04	0.96	1.02	761226	{}	\N	2020-10-12 16:18:53.959158
73752	TGB	2020-10-05	1.02	1.09	1.01	1.08	575394	{}	\N	2020-10-12 16:18:53.964366
73753	TGB	2020-10-06	1.07	1.0817	1.03	1.04	776595	{}	\N	2020-10-12 16:18:53.969344
73754	TGB	2020-10-07	1.07	1.15	1.035	1.07	1892839	{}	\N	2020-10-12 16:18:53.97471
73755	TGB	2020-10-08	1.04	1.11	1.0304	1.07	683290	{}	\N	2020-10-12 16:18:53.979531
73756	TGB	2020-10-09	1.11	1.11	1.045	1.1	618593	{}	\N	2020-10-12 16:18:53.984198
73757	TGB	2020-10-12	1.1	1.1	1.05	1.0878	262696	{}	\N	2020-10-12 16:18:53.989129
73758	TGTX	2020-09-28	24.75	25.785	24.26	25.53	1390309	{}	\N	2020-10-12 16:18:55.021311
73759	TGTX	2020-09-29	25.7	26.37	25.22	26.17	1630786	{}	\N	2020-10-12 16:18:55.032974
73760	TGTX	2020-09-30	26.16	27.277	26.16	26.76	2362331	{}	\N	2020-10-12 16:18:55.040585
73761	TGTX	2020-10-01	27.07	27.25	26.46	27.09	1706901	{}	\N	2020-10-12 16:18:55.047949
73762	TGTX	2020-10-02	26.35	27.005	26.0127	26.12	1064428	{}	\N	2020-10-12 16:18:55.053827
73763	TGTX	2020-10-05	26.35	27.63	26.14	27.56	1667201	{}	\N	2020-10-12 16:18:55.059103
73764	TGTX	2020-10-06	27.92	29.53	27.92	28.69	2168894	{}	\N	2020-10-12 16:18:55.064273
73765	TGTX	2020-10-07	29.39	30.96	29.146	30.58	1936942	{}	\N	2020-10-12 16:18:55.069147
73766	TGTX	2020-10-08	31	31.1	30.18	30.38	1581239	{}	\N	2020-10-12 16:18:55.07407
73767	TGTX	2020-10-09	31.06	31.667	30.75	31	1775818	{}	\N	2020-10-12 16:18:55.079702
73768	TGTX	2020-10-12	31.68	31.85	30.42	30.535	509793	{}	\N	2020-10-12 16:18:55.084538
73769	THO	2020-09-28	97.16	98.3	90.79	94.65	2775048	{}	\N	2020-10-12 16:18:56.206716
73770	THO	2020-09-29	95.84	98.56	94.11	94.26	2765776	{}	\N	2020-10-12 16:18:56.221567
73771	THO	2020-09-30	94.28	98.44	94.28	95.26	3904506	{}	\N	2020-10-12 16:18:56.230317
73772	THO	2020-10-01	95.94	99.15	95.935	98.61	2367921	{}	\N	2020-10-12 16:18:56.237142
73773	THO	2020-10-02	95.82	102.63	95.34	99.92	2279038	{}	\N	2020-10-12 16:18:56.243856
73774	THO	2020-10-05	100.72	103.665	100.7	103.22	1601373	{}	\N	2020-10-12 16:18:56.249772
73775	THO	2020-10-06	103.84	107.585	101.175	101.97	2137431	{}	\N	2020-10-12 16:18:56.255343
73776	THO	2020-10-07	103.96	104.845	100.78	103.45	1468634	{}	\N	2020-10-12 16:18:56.260974
73777	THO	2020-10-08	104.12	104.12	96.99	97.7	1960282	{}	\N	2020-10-12 16:18:56.266316
73778	THO	2020-10-09	97.8	98.61	94.88	96.59	2071853	{}	\N	2020-10-12 16:18:56.272056
73779	THO	2020-10-12	97.9	97.9	94.645	95.5122	362739	{}	\N	2020-10-12 16:18:56.27751
73780	TIPX	2020-09-28	20.72	20.745	20.71	20.7328	51822	{}	\N	2020-10-12 16:18:57.410229
73781	TIPX	2020-09-29	20.76	20.775	20.75	20.76	20313	{}	\N	2020-10-12 16:18:57.420916
73782	TIPX	2020-09-30	20.78	20.78	20.73	20.73	977705	{}	\N	2020-10-12 16:18:57.428058
73783	TIPX	2020-10-01	20.65	20.67	20.635	20.66	189680	{}	\N	2020-10-12 16:18:57.435024
73784	TIPX	2020-10-02	20.64	20.6561	20.6301	20.645	37566	{}	\N	2020-10-12 16:18:57.440424
73785	TIPX	2020-10-05	20.64	20.66	20.6216	20.64	109421	{}	\N	2020-10-12 16:18:57.44504
73786	TIPX	2020-10-06	20.64	20.66	20.6242	20.64	51035	{}	\N	2020-10-12 16:18:57.449838
73787	TIPX	2020-10-07	20.64	20.6501	20.62	20.6466	89568	{}	\N	2020-10-12 16:18:57.454264
73788	TIPX	2020-10-08	20.665	20.7	20.665	20.695	20692	{}	\N	2020-10-12 16:18:57.458788
73789	TIPX	2020-10-09	20.7	20.7093	20.681	20.69	88907	{}	\N	2020-10-12 16:18:57.463205
73790	TIPX	2020-10-12	20.69	20.7	20.68	20.69	162222	{}	\N	2020-10-12 16:18:57.467678
73791	TLDH	2020-09-28	25.61	25.61	25.585	25.585	693	{}	\N	2020-10-12 16:19:00.455889
73792	TLDH	2020-09-29	25.48	25.48	25.4347	25.4347	1152	{}	\N	2020-10-12 16:19:00.464124
73793	TLDH	2020-09-30	25.5028	25.5028	25.4236	25.4236	1951	{}	\N	2020-10-12 16:19:00.470618
73794	TLDH	2020-10-01	25.4594	25.4594	25.4594	25.4594	21	{}	\N	2020-10-12 16:19:00.476077
73795	TLDH	2020-10-02	25.4177	25.4177	25.4177	25.4177	4	{}	\N	2020-10-12 16:19:00.481746
73796	TLDH	2020-10-05	25.6981	25.7848	25.6981	25.7848	4472	{}	\N	2020-10-12 16:19:00.486865
73797	TLDH	2020-10-06	25.88	25.915	25.6997	25.6997	2070	{}	\N	2020-10-12 16:19:00.49188
73798	TLDH	2020-10-07	25.9278	25.9278	25.9278	25.9278	0	{}	\N	2020-10-12 16:19:00.497642
73799	TLDH	2020-10-08	26.0676	26.0676	26.0676	26.0676	0	{}	\N	2020-10-12 16:19:00.503196
73800	TLDH	2020-10-09	26.04	26.04	25.91	26.035	7622	{}	\N	2020-10-12 16:19:00.508593
73801	TMFC	2020-09-28	31.86	31.86	31.49	31.655	63362	{}	\N	2020-10-12 16:19:03.627815
73802	TMFC	2020-09-29	31.81	31.82	31.54	31.62	27073	{}	\N	2020-10-12 16:19:03.640534
73803	TMFC	2020-09-30	31.62	32.2	31.606	31.8895	39881	{}	\N	2020-10-12 16:19:03.650809
73804	TMFC	2020-10-01	32.1	32.3563	32.02	32.21	68234	{}	\N	2020-10-12 16:19:03.65845
73805	TMFC	2020-10-02	31.49	32.017	31.4101	31.6	68445	{}	\N	2020-10-12 16:19:03.664342
73806	TMFC	2020-10-05	31.9	32.2	31.86	32.2	72762	{}	\N	2020-10-12 16:19:03.670956
73807	TMFC	2020-10-06	32.21	32.21	31.5442	31.6	53403	{}	\N	2020-10-12 16:19:03.676695
73808	TMFC	2020-10-07	31.82	32.25	31.82	32.23	37564	{}	\N	2020-10-12 16:19:03.68209
73809	TMFC	2020-10-08	32.51	32.51	32.2	32.37	21200	{}	\N	2020-10-12 16:19:03.687813
73810	TMFC	2020-10-09	32.58	32.84	32.475	32.84	39309	{}	\N	2020-10-12 16:19:03.693753
73811	TMFC	2020-10-12	33.32	33.58	33.11	33.58	45862	{}	\N	2020-10-12 16:19:03.699189
73812	TNDM	2020-09-28	109.82	112.165	109.25	110.49	569267	{}	\N	2020-10-12 16:19:04.732805
73813	TNDM	2020-09-29	111.18	114.78	110.85	112.61	642934	{}	\N	2020-10-12 16:19:04.742954
73814	TNDM	2020-09-30	112.49	115.2	112.36	113.5	540542	{}	\N	2020-10-12 16:19:04.750537
73815	TNDM	2020-10-01	114.81	115.29	112.68	113.64	445077	{}	\N	2020-10-12 16:19:04.757732
73816	TNDM	2020-10-02	111.29	113.9	109.4	111.05	562178	{}	\N	2020-10-12 16:19:04.763332
73817	TNDM	2020-10-05	111.43	114.11	111.31	113.7	570846	{}	\N	2020-10-12 16:19:04.769043
73818	TNDM	2020-10-06	114.05	114.75	111.4736	111.96	568185	{}	\N	2020-10-12 16:19:04.774929
73819	TNDM	2020-10-07	112.42	118.45	112.42	116.71	673377	{}	\N	2020-10-12 16:19:04.780748
73820	TNDM	2020-10-08	118.3	118.8	114.14	114.45	544146	{}	\N	2020-10-12 16:19:04.786697
73821	TNDM	2020-10-09	114.6	116.23	114.6	115.52	437006	{}	\N	2020-10-12 16:19:04.792498
73822	TNDM	2020-10-12	117.02	117.499	115.52	116.83	140158	{}	\N	2020-10-12 16:19:04.798378
73823	TOT	2020-09-28	33.52	33.86	33.43	33.79	3361830	{}	\N	2020-10-12 16:19:05.884292
73824	TOT	2020-09-29	33.54	33.64	33.005	33.33	10176446	{}	\N	2020-10-12 16:19:05.896416
73825	TOT	2020-09-30	34.38	34.9189	34.11	34.3	11392121	{}	\N	2020-10-12 16:19:05.903903
73826	TOT	2020-10-01	33.66	33.725	33.24	33.58	8496124	{}	\N	2020-10-12 16:19:05.909908
73827	TOT	2020-10-02	32.59	33.57	32.5502	33.33	6378489	{}	\N	2020-10-12 16:19:05.915235
73828	TOT	2020-10-05	33.94	34.38	33.635	34.36	5895275	{}	\N	2020-10-12 16:19:05.920109
73829	TOT	2020-10-06	35.22	35.33	34.23	34.27	4409101	{}	\N	2020-10-12 16:19:05.925176
73830	TOT	2020-10-07	34.4	34.62	34.02	34.57	2719021	{}	\N	2020-10-12 16:19:05.929843
73831	TOT	2020-10-08	34.77	35.1	34.63	34.93	4339612	{}	\N	2020-10-12 16:19:05.934328
73832	TOT	2020-10-09	35.57	35.6	35.03	35.25	21023837	{}	\N	2020-10-12 16:19:05.938769
73833	TOT	2020-10-12	35.24	35.24	34.88	34.95	1347083	{}	\N	2020-10-12 16:19:05.94301
73834	TCON	2020-09-28	5.1704	5.64	5.13	5.34	1155270	{}	\N	2020-10-12 16:19:07.762398
73835	TCON	2020-09-29	5.25	5.39	4.95	5.07	694242	{}	\N	2020-10-12 16:19:07.773327
73836	TCON	2020-09-30	4.99	5.271	4.9	5.17	443605	{}	\N	2020-10-12 16:19:07.780914
73837	TCON	2020-10-01	5.13	5.2	4.7	4.79	626670	{}	\N	2020-10-12 16:19:07.787389
73838	TCON	2020-10-02	4.7	4.89	4.45	4.65	390637	{}	\N	2020-10-12 16:19:07.793614
73839	TCON	2020-10-05	4.65	5.24	4.4	5.095	680785	{}	\N	2020-10-12 16:19:07.798937
73840	TCON	2020-10-06	5.12	5.1425	4.75	4.87	424009	{}	\N	2020-10-12 16:19:07.803966
73841	TCON	2020-10-07	4.93	5.38	4.85	5.1	491206	{}	\N	2020-10-12 16:19:07.809825
73842	TCON	2020-10-08	5.1	5.1893	4.9	5.03	277158	{}	\N	2020-10-12 16:19:07.815124
73843	TCON	2020-10-09	5	5.09	4.89	4.925	247722	{}	\N	2020-10-12 16:19:07.820221
73844	TCON	2020-10-12	5	5.37	4.89	5.29	405716	{}	\N	2020-10-12 16:19:07.825246
73845	TDI	2020-09-28	25.86	26.24	25.65	26.19	8750	{}	\N	2020-10-12 16:19:08.818217
73846	TDI	2020-09-29	26.04	26.4	26.04	26.35	18830	{}	\N	2020-10-12 16:19:08.832936
73847	TDI	2020-09-30	26.3236	26.4	26.06	26.06	5436	{}	\N	2020-10-12 16:19:08.841772
73848	TDI	2020-10-01	26.2	26.47	26.172	26.46	16150	{}	\N	2020-10-12 16:19:08.850062
73849	TDI	2020-10-02	26.3	26.63	25.96	26.63	6826	{}	\N	2020-10-12 16:19:08.856469
73850	TDI	2020-10-05	26.57	26.67	26.2611	26.54	10704	{}	\N	2020-10-12 16:19:08.862491
73851	TDI	2020-10-06	26.3	26.525	26.21	26.48	20036	{}	\N	2020-10-12 16:19:08.868246
73852	TDI	2020-10-07	26.46	26.58	26.38	26.485	8826	{}	\N	2020-10-12 16:19:08.873669
73853	TDI	2020-10-08	26.47	26.78	26.47	26.76	14422	{}	\N	2020-10-12 16:19:08.879916
73854	TDI	2020-10-09	26.76	26.87	26.57	26.8	10172	{}	\N	2020-10-12 16:19:08.885758
73855	TDI	2020-10-12	26.71	26.775	26.71	26.775	4386	{}	\N	2020-10-12 16:19:08.892033
73856	TECK	2020-09-28	14.56	14.64	13.88	13.97	3843752	{}	\N	2020-10-12 16:19:09.861916
73857	TECK	2020-09-29	13.95	13.95	13.62	13.84	6820393	{}	\N	2020-10-12 16:19:09.873882
73858	TECK	2020-09-30	13.97	14.1	13.86	13.92	4688215	{}	\N	2020-10-12 16:19:09.881295
73859	TECK	2020-10-01	14.02	14.28	13.84	14.06	10671990	{}	\N	2020-10-12 16:19:09.888024
73860	TECK	2020-10-02	13.66	14.53	13.6	14.26	8040710	{}	\N	2020-10-12 16:19:09.894195
73861	TECK	2020-10-05	14.38	14.44	13.74	13.93	11669543	{}	\N	2020-10-12 16:19:09.89978
73862	TECK	2020-10-06	14.05	14.14	13.09	13.23	13474452	{}	\N	2020-10-12 16:19:09.904921
73863	TECK	2020-10-07	13.5	13.6	13.18	13.48	22154789	{}	\N	2020-10-12 16:19:09.910171
73864	TECK	2020-10-08	13.85	13.96	13.55	13.91	11742238	{}	\N	2020-10-12 16:19:09.915235
73865	TECK	2020-10-09	14.02	14.18	13.77	13.9	11065911	{}	\N	2020-10-12 16:19:09.921387
73866	TECK	2020-10-12	14	14.015	13.26	13.455	4629971	{}	\N	2020-10-12 16:19:09.926344
73867	TENX	2020-09-28	1.68	1.71	1.5492	1.56	188212	{}	\N	2020-10-12 16:19:11.284424
73868	TENX	2020-09-29	1.56	1.6202	1.52	1.55	107244	{}	\N	2020-10-12 16:19:11.2987
73869	TENX	2020-09-30	1.54	1.58	1.53	1.54	89187	{}	\N	2020-10-12 16:19:11.308236
73870	TENX	2020-10-01	1.54	1.564	1.42	1.43	169313	{}	\N	2020-10-12 16:19:11.31561
73871	TENX	2020-10-02	1.38	1.43	1.2387	1.37	266490	{}	\N	2020-10-12 16:19:11.321739
73872	TENX	2020-10-05	1.38	1.4099	1.2	1.25	204878	{}	\N	2020-10-12 16:19:11.327077
73873	TENX	2020-10-06	1.285	1.29	1.23	1.24	107593	{}	\N	2020-10-12 16:19:11.333096
73874	TENX	2020-10-07	1.24	1.3334	1.21	1.22	131768	{}	\N	2020-10-12 16:19:11.33857
73875	TENX	2020-10-08	1.27	1.34	1.25	1.34	98496	{}	\N	2020-10-12 16:19:11.343871
73876	TENX	2020-10-09	1.34	1.41	1.28	1.28	194760	{}	\N	2020-10-12 16:19:11.349132
73877	TENX	2020-10-12	1.26	1.34	1.25	1.275	31474	{}	\N	2020-10-12 16:19:11.355257
73878	TEX	2020-09-28	18.52	19.58	18.52	19.04	635544	{}	\N	2020-10-12 16:19:13.937427
73879	TEX	2020-09-29	19.14	19.55	18.91	19.36	897364	{}	\N	2020-10-12 16:19:13.945385
73880	TEX	2020-09-30	19.49	20.02	19.17	19.36	1111670	{}	\N	2020-10-12 16:19:13.95203
73881	TEX	2020-10-01	19.95	20.65	19.82	20.33	1745971	{}	\N	2020-10-12 16:19:13.958332
73882	TEX	2020-10-02	19.77	21.83	19.68	21.59	1649559	{}	\N	2020-10-12 16:19:13.963867
73883	TEX	2020-10-05	22.21	24.3	22.21	24.04	3138761	{}	\N	2020-10-12 16:19:13.969249
73884	TEX	2020-10-06	24.46	24.47	22.95	23.23	1911411	{}	\N	2020-10-12 16:19:13.974062
73885	TEX	2020-10-07	23.94	24.71	23.93	24.36	1721397	{}	\N	2020-10-12 16:19:13.978643
73886	TEX	2020-10-08	24.61	24.94	24.2	24.93	1380065	{}	\N	2020-10-12 16:19:13.983296
73887	TEX	2020-10-09	24.95	25.23	24.07	24.48	1672445	{}	\N	2020-10-12 16:19:13.988257
73888	TEX	2020-10-12	24.58	25.09	24.4307	24.915	393350	{}	\N	2020-10-12 16:19:13.993039
73889	TGH	2020-09-28	13.8	14.13	13.8	13.99	413817	{}	\N	2020-10-12 16:19:15.256013
73890	TGH	2020-09-29	14.05	14.17	13.7	14.02	440491	{}	\N	2020-10-12 16:19:15.269896
73891	TGH	2020-09-30	14.11	14.35	14.11	14.16	520782	{}	\N	2020-10-12 16:19:15.277893
73892	TGH	2020-10-01	14.1	14.25	13.97	14.03	359757	{}	\N	2020-10-12 16:19:15.28479
73893	TGH	2020-10-02	13.77	15.1	13.77	14.79	736790	{}	\N	2020-10-12 16:19:15.290608
73894	TGH	2020-10-05	14.78	15.35	14.49	15.25	923438	{}	\N	2020-10-12 16:19:15.295679
73895	TGH	2020-10-06	15.17	15.34	14.9	14.92	712757	{}	\N	2020-10-12 16:19:15.300792
73896	TGH	2020-10-07	15.17	15.35	14.94	14.98	454854	{}	\N	2020-10-12 16:19:15.305723
73897	TGH	2020-10-08	15.25	15.25	14.86	14.9	342706	{}	\N	2020-10-12 16:19:15.310634
73898	TGH	2020-10-09	14.98	15.0648	14.69	14.78	386479	{}	\N	2020-10-12 16:19:15.315489
73899	TGH	2020-10-12	14.71	14.77	14.5	14.6	230425	{}	\N	2020-10-12 16:19:15.32041
73900	THQ	2020-09-28	17.87	17.89	17.67	17.71	105139	{}	\N	2020-10-12 16:19:16.271383
73901	THQ	2020-09-29	17.65	17.83	17.5733	17.75	186013	{}	\N	2020-10-12 16:19:16.286294
73902	THQ	2020-09-30	17.93	18.19	17.885	18.09	411112	{}	\N	2020-10-12 16:19:16.295094
73903	THQ	2020-10-01	18.25	18.3678	18.05	18.09	258036	{}	\N	2020-10-12 16:19:16.301811
73904	THQ	2020-10-02	17.85	18.27	17.85	18	205945	{}	\N	2020-10-12 16:19:16.307778
73905	THQ	2020-10-05	18.02	18.53	18.02	18.48	207442	{}	\N	2020-10-12 16:19:16.31297
73906	THQ	2020-10-06	18.55	18.63	18.4	18.45	310302	{}	\N	2020-10-12 16:19:16.318227
73907	THQ	2020-10-07	18.47	18.75	18.41	18.71	242199	{}	\N	2020-10-12 16:19:16.323957
73908	THQ	2020-10-08	18.83	18.99	18.78	18.81	177884	{}	\N	2020-10-12 16:19:16.329095
73909	THQ	2020-10-09	18.96	19.0596	18.8944	18.96	237506	{}	\N	2020-10-12 16:19:16.334142
73910	THQ	2020-10-12	18.96	19.1	18.9	19.0501	131328	{}	\N	2020-10-12 16:19:16.339703
73911	TISI	2020-09-28	5.36	5.63	5.36	5.41	272810	{}	\N	2020-10-12 16:19:17.71308
73912	TISI	2020-09-29	5.33	5.39	5.14	5.31	352593	{}	\N	2020-10-12 16:19:17.725966
73913	TISI	2020-09-30	5.35	5.56	5.3	5.5	747968	{}	\N	2020-10-12 16:19:17.735131
73914	TISI	2020-10-01	5.53	5.59	5.34	5.46	293785	{}	\N	2020-10-12 16:19:17.741479
73915	TISI	2020-10-02	5.27	5.68	5.16	5.58	463837	{}	\N	2020-10-12 16:19:17.74727
73916	TISI	2020-10-05	5.64	5.76	5.51	5.68	363867	{}	\N	2020-10-12 16:19:17.753047
73917	TISI	2020-10-06	5.8	5.93	5.57	5.8	660479	{}	\N	2020-10-12 16:19:17.758322
73918	TISI	2020-10-07	5.89	5.9224	5.59	5.79	552542	{}	\N	2020-10-12 16:19:17.763668
73919	TISI	2020-10-08	5.88	6	5.76	5.92	326786	{}	\N	2020-10-12 16:19:17.769152
73920	TISI	2020-10-09	6	6	5.84	5.91	260738	{}	\N	2020-10-12 16:19:17.774618
73921	TISI	2020-10-12	5.93	6.08	5.86	5.995	102020	{}	\N	2020-10-12 16:19:17.780304
73922	TLTD	2020-09-28	56.16	56.2816	56.07	56.21	34447	{}	\N	2020-10-12 16:19:19.283072
73923	TLTD	2020-09-29	56.2	56.28	55.95	56.03	10437	{}	\N	2020-10-12 16:19:19.293718
73924	TLTD	2020-09-30	56.06	56.31	55.97	55.97	9774	{}	\N	2020-10-12 16:19:19.301121
73925	TLTD	2020-10-01	56.17	56.21	55.94	56.2	8701	{}	\N	2020-10-12 16:19:19.309043
73926	TLTD	2020-10-02	55.67	56.22	55.67	56.09	29942	{}	\N	2020-10-12 16:19:19.315464
73927	TLTD	2020-10-05	56.55	57.03	56.55	57	9558	{}	\N	2020-10-12 16:19:19.321136
73928	TLTD	2020-10-06	57.21	57.36	56.67	56.8	9260	{}	\N	2020-10-12 16:19:19.326864
73929	TLTD	2020-10-07	57.07	57.2578	57.03	57.17	7709	{}	\N	2020-10-12 16:19:19.332313
73930	TLTD	2020-10-08	57.49	57.66	57.47	57.66	6190	{}	\N	2020-10-12 16:19:19.337829
73931	TLTD	2020-10-09	57.9353	57.97	57.78	57.97	4364	{}	\N	2020-10-12 16:19:19.343346
73932	TLTD	2020-10-12	57.97	58.041	57.97	58.01	3555	{}	\N	2020-10-12 16:19:19.348709
73933	TMP	2020-09-28	55.81	57.26	55.81	56.73	41502	{}	\N	2020-10-12 16:19:20.319133
73934	TMP	2020-09-29	56.66	56.66	55.17	56.42	25370	{}	\N	2020-10-12 16:19:20.325773
73935	TMP	2020-09-30	56.81	58.3199	56.76	56.81	72922	{}	\N	2020-10-12 16:19:20.330874
73936	TMP	2020-10-01	57.23	57.41	55.75	56.09	52891	{}	\N	2020-10-12 16:19:20.336504
73937	TMP	2020-10-02	55.6	57.69	55.6	57.35	35200	{}	\N	2020-10-12 16:19:20.342036
73938	TMP	2020-10-05	57.51	59.39	57.51	58.9	32292	{}	\N	2020-10-12 16:19:20.347513
73939	TMP	2020-10-06	59.43	60.19	58.46	58.59	59574	{}	\N	2020-10-12 16:19:20.354185
73940	TMP	2020-10-07	59.49	59.49	58.13	58.26	64876	{}	\N	2020-10-12 16:19:20.362841
73941	TMP	2020-10-08	58.54	59.49	57.935	58.66	38991	{}	\N	2020-10-12 16:19:20.370999
73942	TMP	2020-10-09	58.73	58.89	57.7	57.92	36894	{}	\N	2020-10-12 16:19:20.378475
73943	TMP	2020-10-12	57.76	58.25	57.72	57.8664	9277	{}	\N	2020-10-12 16:19:20.386225
73944	TNET	2020-09-28	59.57	61.24	59.29	60.28	333820	{}	\N	2020-10-12 16:19:21.633311
73945	TNET	2020-09-29	60.28	60.39	58.44	58.98	425490	{}	\N	2020-10-12 16:19:21.649982
73946	TNET	2020-09-30	59.06	60.62	58.89	59.32	452708	{}	\N	2020-10-12 16:19:21.657431
73947	TNET	2020-10-01	59.49	60.43	58.84	59.65	459237	{}	\N	2020-10-12 16:19:21.662482
73948	TNET	2020-10-02	58.66	61.18	58.66	60.61	329280	{}	\N	2020-10-12 16:19:21.667567
73949	TNET	2020-10-05	61.02	62.1	61.02	61.98	309064	{}	\N	2020-10-12 16:19:21.671903
73950	TNET	2020-10-06	62.7	63.75	61.84	61.97	318642	{}	\N	2020-10-12 16:19:21.676709
73951	TNET	2020-10-07	62.29	64.6	62.29	64.06	564398	{}	\N	2020-10-12 16:19:21.680968
73952	TNET	2020-10-08	64.61	65.75	64.13	65.38	376721	{}	\N	2020-10-12 16:19:21.685497
73953	TNET	2020-10-09	66.13	66.33	65.04	65.76	274921	{}	\N	2020-10-12 16:19:21.69034
73954	TNET	2020-10-12	65.95	67.43	65.88	66.97	99906	{}	\N	2020-10-12 16:19:21.695196
73955	TOPS	2020-09-28	1.07	1.12	1.04	1.06	2717351	{}	\N	2020-10-12 16:19:23.599514
73956	TOPS	2020-09-29	1.08	1.19	1.05	1.08	4659455	{}	\N	2020-10-12 16:19:23.611135
73957	TOPS	2020-09-30	1.09	1.1	1.06	1.08	1553150	{}	\N	2020-10-12 16:19:23.617942
73958	TOPS	2020-10-01	1.1	1.17	1.06	1.14	2572086	{}	\N	2020-10-12 16:19:23.624324
73959	TOPS	2020-10-02	1.1	1.15	1.07	1.13	1240267	{}	\N	2020-10-12 16:19:23.629621
73960	TOPS	2020-10-05	1.15	1.18	1.11	1.15	2152454	{}	\N	2020-10-12 16:19:23.634116
73961	TOPS	2020-10-06	1.14	1.26	1.1256	1.18	4714617	{}	\N	2020-10-12 16:19:23.638288
73962	TOPS	2020-10-07	1.21	1.29	1.18	1.22	3417624	{}	\N	2020-10-12 16:19:23.642345
73963	TOPS	2020-10-08	1.24	1.25	1.16	1.19	2521452	{}	\N	2020-10-12 16:19:23.646386
73964	TOPS	2020-10-09	1.2	1.28	1.18	1.25	3027521	{}	\N	2020-10-12 16:19:23.650512
73965	TOPS	2020-10-12	1.3	1.31	1.21	1.2379	2338105	{}	\N	2020-10-12 16:19:23.654996
73966	TCPC	2020-09-28	9.85	10.27	9.85	10.2	457254	{}	\N	2020-10-12 16:19:25.122272
73967	TCPC	2020-09-29	10.18	10.22	9.77	9.81	314666	{}	\N	2020-10-12 16:19:25.132493
73968	TCPC	2020-09-30	10.03	10.2	9.8	9.81	385489	{}	\N	2020-10-12 16:19:25.139909
73969	TCPC	2020-10-01	9.81	9.91	9.75	9.91	168991	{}	\N	2020-10-12 16:19:25.14593
73970	TCPC	2020-10-02	9.8	10.1	9.7988	10	152736	{}	\N	2020-10-12 16:19:25.151288
73971	TCPC	2020-10-05	10.05	10.15	9.92	9.98	139874	{}	\N	2020-10-12 16:19:25.156403
73972	TCPC	2020-10-06	10.03	10.18	9.9	9.9	154659	{}	\N	2020-10-12 16:19:25.161355
73973	TCPC	2020-10-07	10.02	10.0667	9.8701	9.91	199944	{}	\N	2020-10-12 16:19:25.166063
73974	TCPC	2020-10-08	9.92	9.99	9.825	9.89	269749	{}	\N	2020-10-12 16:19:25.170792
73975	TCPC	2020-10-09	9.91	9.96	9.87	9.89	169157	{}	\N	2020-10-12 16:19:25.175407
73976	TCPC	2020-10-12	9.87	9.89	9.65	9.67	114389	{}	\N	2020-10-12 16:19:25.180037
73977	TDF	2020-09-28	21.65	21.78	21.59	21.76	39324	{}	\N	2020-10-12 16:19:27.419695
73978	TDF	2020-09-29	21.76	21.76	21.6659	21.75	18632	{}	\N	2020-10-12 16:19:27.430324
73979	TDF	2020-09-30	21.75	21.94	21.6884	21.93	75554	{}	\N	2020-10-12 16:19:27.438582
73980	TDF	2020-10-01	22.07	22.26	21.99	22	80313	{}	\N	2020-10-12 16:19:27.445729
73981	TDF	2020-10-02	21.75	21.91	21.74	21.82	38461	{}	\N	2020-10-12 16:19:27.452095
73982	TDF	2020-10-05	21.87	21.95	21.7	21.87	45098	{}	\N	2020-10-12 16:19:27.457288
73983	TDF	2020-10-06	21.79	22.22	21.74	22.13	287192	{}	\N	2020-10-12 16:19:27.46235
73984	TDF	2020-10-07	22.3	22.5	22.165	22.39	61640	{}	\N	2020-10-12 16:19:27.467375
73985	TDF	2020-10-08	22.49	22.57	22.3	22.51	41210	{}	\N	2020-10-12 16:19:27.473066
73986	TDF	2020-10-09	22.46	22.66	22.46	22.66	25524	{}	\N	2020-10-12 16:19:27.478061
73987	TDF	2020-10-12	22.88	23.4356	22.88	23.3498	58604	{}	\N	2020-10-12 16:19:27.483304
73988	TDY	2020-09-28	317.1	320.14	315.11	315.39	181529	{}	\N	2020-10-12 16:19:28.87948
73989	TDY	2020-09-29	313.93	315.06	310.38	311.94	264480	{}	\N	2020-10-12 16:19:28.891319
73990	TDY	2020-09-30	313.11	315.42	307.61	310.21	571658	{}	\N	2020-10-12 16:19:28.900613
73991	TDY	2020-10-01	312.66	314.8	307.22	307.75	326974	{}	\N	2020-10-12 16:19:28.910304
73992	TDY	2020-10-02	303.19	312.79	301.76	309.81	225678	{}	\N	2020-10-12 16:19:28.918886
73993	TDY	2020-10-05	313.44	317.62	313.36	315.52	263329	{}	\N	2020-10-12 16:19:28.927395
73994	TDY	2020-10-06	318.36	322.28	313.105	314.87	417767	{}	\N	2020-10-12 16:19:28.956187
73995	TDY	2020-10-07	317.23	319.86	315.3	317.87	368510	{}	\N	2020-10-12 16:19:28.969377
73996	TDY	2020-10-08	317.71	325.72	317.66	325.5	215903	{}	\N	2020-10-12 16:19:28.979239
73997	TDY	2020-10-09	328.61	328.91	324.5	325.14	217396	{}	\N	2020-10-12 16:19:28.987203
73998	TDY	2020-10-12	326.32	328.54	325.34	327.375	57510	{}	\N	2020-10-12 16:19:28.994855
73999	TEO	2020-09-28	6.83	6.86	6.61	6.65	193012	{}	\N	2020-10-12 16:19:31.160919
74000	TEO	2020-09-29	6.55	6.7101	6.41	6.47	529953	{}	\N	2020-10-12 16:19:31.17528
74001	TEO	2020-09-30	6.46	6.6552	6.45	6.54	389935	{}	\N	2020-10-12 16:19:31.183204
74002	TEO	2020-10-01	6.54	6.64	6.45	6.61	269954	{}	\N	2020-10-12 16:19:31.189431
74003	TEO	2020-10-02	6.51	6.6999	6.4801	6.67	132773	{}	\N	2020-10-12 16:19:31.194854
74004	TEO	2020-10-05	6.7	6.91	6.63	6.83	348895	{}	\N	2020-10-12 16:19:31.200035
74005	TEO	2020-10-06	6.83	7.09	6.79	7.04	434402	{}	\N	2020-10-12 16:19:31.204973
74006	TEO	2020-10-07	7.08	7.16	6.92	7.03	274805	{}	\N	2020-10-12 16:19:31.210043
74007	TEO	2020-10-08	7.15	7.4	7.15	7.34	469274	{}	\N	2020-10-12 16:19:31.215025
74008	TEO	2020-10-09	7.4	7.4	7.15	7.36	403344	{}	\N	2020-10-12 16:19:31.219943
74009	TEO	2020-10-12	7.3	7.31	7.16	7.2626	59512	{}	\N	2020-10-12 16:19:31.224774
74010	TGC	2020-09-28	0.752	0.793	0.752	0.79	18852	{}	\N	2020-10-12 16:19:33.535217
74011	TGC	2020-09-29	0.81	1.06	0.79	0.87	262664	{}	\N	2020-10-12 16:19:33.546807
74012	TGC	2020-09-30	0.872	0.8745	0.8	0.8224	82013	{}	\N	2020-10-12 16:19:33.554268
74013	TGC	2020-10-01	0.8325	0.8325	0.76	0.7602	82447	{}	\N	2020-10-12 16:19:33.559931
74014	TGC	2020-10-02	0.842	0.852	0.722	0.7836	35696	{}	\N	2020-10-12 16:19:33.565342
74015	TGC	2020-10-05	0.81	0.86	0.81	0.816	47832	{}	\N	2020-10-12 16:19:33.5702
74016	TGC	2020-10-06	0.896	0.9193	0.872	0.872	11579	{}	\N	2020-10-12 16:19:33.574815
74017	TGC	2020-10-07	0.8784	0.9954	0.8637	0.9	50820	{}	\N	2020-10-12 16:19:33.579269
74018	TGC	2020-10-08	0.869	1.0871	0.869	0.9889	115109	{}	\N	2020-10-12 16:19:33.583848
74019	TGC	2020-10-09	0.93	0.9823	0.87	0.8911	94197	{}	\N	2020-10-12 16:19:33.588328
74020	TGC	2020-10-12	0.88	0.8925	0.8202	0.8889	55351	{}	\N	2020-10-12 16:19:33.59265
74021	THR	2020-09-28	10.8	11.24	10.8	11.11	162477	{}	\N	2020-10-12 16:19:35.840007
74022	THR	2020-09-29	11.04	11.25	10.9	11.14	459749	{}	\N	2020-10-12 16:19:35.849367
74023	THR	2020-09-30	11.18	11.5018	11.17	11.23	724179	{}	\N	2020-10-12 16:19:35.856975
74024	THR	2020-10-01	11.2	11.44	11	11.42	378784	{}	\N	2020-10-12 16:19:35.862438
74025	THR	2020-10-02	11.19	11.64	11.04	11.5	221512	{}	\N	2020-10-12 16:19:35.867556
74026	THR	2020-10-05	11.62	11.9	11.59	11.67	362215	{}	\N	2020-10-12 16:19:35.872376
74027	THR	2020-10-06	11.86	12.09	11.55	11.59	473877	{}	\N	2020-10-12 16:19:35.87692
74028	THR	2020-10-07	11.67	12.045	11.66	11.78	528031	{}	\N	2020-10-12 16:19:35.881261
74029	THR	2020-10-08	11.89	12.21	11.82	11.96	298658	{}	\N	2020-10-12 16:19:35.885339
74030	THR	2020-10-09	12.05	12.3	11.82	11.95	534108	{}	\N	2020-10-12 16:19:35.889778
74031	THR	2020-10-12	11.93	12.2	11.86	12	158298	{}	\N	2020-10-12 16:19:35.895129
74032	TIF	2020-09-28	116	116.83	115.83	116.7	756791	{}	\N	2020-10-12 16:19:38.18863
74033	TIF	2020-09-29	116.75	116.8	115.59	116.44	1817671	{}	\N	2020-10-12 16:19:38.197831
74034	TIF	2020-09-30	116.34	116.65	115.75	115.85	2374967	{}	\N	2020-10-12 16:19:38.20333
74035	TIF	2020-10-01	116.2	116.31	115.75	116.03	2284622	{}	\N	2020-10-12 16:19:38.208368
74036	TIF	2020-10-02	115.28	116.26	115.28	116.11	1746481	{}	\N	2020-10-12 16:19:38.213567
74037	TIF	2020-10-05	116.38	117.17	116.11	116.99	2988195	{}	\N	2020-10-12 16:19:38.218788
74038	TIF	2020-10-06	117.33	117.45	116.52	117	1928499	{}	\N	2020-10-12 16:19:38.22316
74039	TIF	2020-10-07	117.11	117.7	116.87	117.7	1312029	{}	\N	2020-10-12 16:19:38.227264
74040	TIF	2020-10-08	117.7	118	117.44	117.97	838735	{}	\N	2020-10-12 16:19:38.231557
74041	TIF	2020-10-09	118.34	118.45	117.63	118.41	819373	{}	\N	2020-10-12 16:19:38.23696
74042	TIF	2020-10-12	118.75	118.75	118.0178	118.21	438529	{}	\N	2020-10-12 16:19:38.24138
74043	TKC	2020-09-28	4.88	4.88	4.76	4.77	320011	{}	\N	2020-10-12 16:19:40.486197
74044	TKC	2020-09-29	4.79	4.86	4.633	4.65	536138	{}	\N	2020-10-12 16:19:40.496577
74045	TKC	2020-09-30	4.8	4.86	4.77	4.79	849028	{}	\N	2020-10-12 16:19:40.50404
74046	TKC	2020-10-01	4.89	4.891	4.82	4.83	562882	{}	\N	2020-10-12 16:19:40.510166
74047	TKC	2020-10-02	4.82	4.915	4.8	4.82	505946	{}	\N	2020-10-12 16:19:40.515344
74048	TKC	2020-10-05	4.97	5.04	4.9	5.01	592026	{}	\N	2020-10-12 16:19:40.520043
74049	TKC	2020-10-06	5.01	5.01	4.9	4.9	648219	{}	\N	2020-10-12 16:19:40.524521
74050	TKC	2020-10-07	4.93	4.99	4.9	4.91	341047	{}	\N	2020-10-12 16:19:40.528905
74051	TKC	2020-10-08	4.9	5.045	4.9	5.02	476172	{}	\N	2020-10-12 16:19:40.532935
74052	TKC	2020-10-09	5.02	5.02	4.96	4.97	233425	{}	\N	2020-10-12 16:19:40.537894
74053	TKC	2020-10-12	5.05	5.14	5.05	5.13	209013	{}	\N	2020-10-12 16:19:40.54216
74054	TLK	2020-09-28	18.2	18.3	18.04	18.2	231044	{}	\N	2020-10-12 16:19:42.507592
74055	TLK	2020-09-29	18.05	18.0524	17.76	17.79	636115	{}	\N	2020-10-12 16:19:42.533667
74056	TLK	2020-09-30	17.52	17.66	17.33	17.37	805245	{}	\N	2020-10-12 16:19:42.540588
74057	TLK	2020-10-01	18.24	18.53	18.21	18.45	665609	{}	\N	2020-10-12 16:19:42.547814
74058	TLK	2020-10-02	18.09	18.29	18	18.15	281807	{}	\N	2020-10-12 16:19:42.554615
74059	TLK	2020-10-05	18.205	18.48	18.11	18.36	522786	{}	\N	2020-10-12 16:19:42.562106
74060	TLK	2020-10-06	18.54	18.54	18.21	18.21	1098665	{}	\N	2020-10-12 16:19:42.575385
74061	TLK	2020-10-07	18.28	18.49	18.12	18.34	693431	{}	\N	2020-10-12 16:19:42.580694
74062	TLK	2020-10-08	18.48	18.6925	18.36	18.59	727409	{}	\N	2020-10-12 16:19:42.58626
74063	TLK	2020-10-09	18.82	18.82	18.6	18.78	751433	{}	\N	2020-10-12 16:19:42.592597
74064	TLK	2020-10-12	18.8	18.8	18.58	18.65	252170	{}	\N	2020-10-12 16:19:42.597889
74065	TMST	2020-09-28	3.61	3.79	3.55	3.72	280358	{}	\N	2020-10-12 16:19:45.82051
74066	TMST	2020-09-29	3.7	3.7	3.48	3.53	1041059	{}	\N	2020-10-12 16:19:45.831162
74067	TMST	2020-09-30	3.57	3.68	3.51	3.55	509539	{}	\N	2020-10-12 16:19:45.838171
74068	TMST	2020-10-01	3.56	3.72	3.49	3.68	430535	{}	\N	2020-10-12 16:19:45.844973
74069	TMST	2020-10-02	3.56	3.83	3.53	3.76	356808	{}	\N	2020-10-12 16:19:45.851136
74070	TMST	2020-10-05	3.81	4.02	3.81	3.83	458444	{}	\N	2020-10-12 16:19:45.856191
74071	TMST	2020-10-06	3.93	4.015	3.72	3.72	394867	{}	\N	2020-10-12 16:19:45.861522
74072	TMST	2020-10-07	3.79	4.07	3.79	3.93	621808	{}	\N	2020-10-12 16:19:45.867018
74073	TMST	2020-10-08	3.99	4.075	3.92	3.96	400507	{}	\N	2020-10-12 16:19:45.871882
74074	TMST	2020-10-09	3.93	4.08	3.83	3.89	317732	{}	\N	2020-10-12 16:19:45.876585
74075	TMST	2020-10-12	3.85	3.87	3.775	3.8	93284	{}	\N	2020-10-12 16:19:45.881783
74076	TNXP	2020-09-28	0.8699	0.87	0.85	0.8511	4378572	{}	\N	2020-10-12 16:19:47.241315
74077	TNXP	2020-09-29	0.93	0.9384	0.8802	0.9	8025557	{}	\N	2020-10-12 16:19:47.256009
74078	TNXP	2020-09-30	0.8158	0.8498	0.799	0.84	8205729	{}	\N	2020-10-12 16:19:47.264475
74079	TNXP	2020-10-01	0.825	0.83	0.8	0.8058	3252241	{}	\N	2020-10-12 16:19:47.27119
74080	TNXP	2020-10-02	0.8	0.815	0.78	0.8	3865210	{}	\N	2020-10-12 16:19:47.277552
74081	TNXP	2020-10-05	0.7873	0.8099	0.77	0.7901	4729206	{}	\N	2020-10-12 16:19:47.282823
74082	TNXP	2020-10-06	0.7877	0.7998	0.765	0.7671	4128213	{}	\N	2020-10-12 16:19:47.288012
74083	TNXP	2020-10-07	0.7682	0.77	0.7201	0.7579	6111818	{}	\N	2020-10-12 16:19:47.293215
74084	TNXP	2020-10-08	0.755	0.7887	0.7428	0.7801	5361816	{}	\N	2020-10-12 16:19:47.298338
74085	TNXP	2020-10-09	0.768	0.7747	0.75	0.7679	3699887	{}	\N	2020-10-12 16:19:47.30346
74086	TNXP	2020-10-12	0.77	0.774899	0.7501	0.7606	1248732	{}	\N	2020-10-12 16:19:47.308655
74087	TOUR	2020-09-28	0.92	0.9397	0.91	0.91	26018	{}	\N	2020-10-12 16:19:48.38093
74088	TOUR	2020-09-29	0.9114	0.9465	0.911	0.9277	45217	{}	\N	2020-10-12 16:19:48.393543
74089	TOUR	2020-09-30	1.1618	1.25	1.08	1.18	6867842	{}	\N	2020-10-12 16:19:48.403304
74090	TOUR	2020-10-01	1.13	1.1507	1.06	1.1	633257	{}	\N	2020-10-12 16:19:48.409659
74091	TOUR	2020-10-02	1	1.09	0.98	1.08	378210	{}	\N	2020-10-12 16:19:48.415567
74092	TOUR	2020-10-05	1.1	1.17	1.06	1.15	203213	{}	\N	2020-10-12 16:19:48.421646
74093	TOUR	2020-10-06	1.18	1.19	1.12	1.13	210097	{}	\N	2020-10-12 16:19:48.426678
74094	TOUR	2020-10-07	1.14	1.16	1.1	1.13	144003	{}	\N	2020-10-12 16:19:48.432226
74095	TOUR	2020-10-08	1.13	1.18	1.105	1.12	129741	{}	\N	2020-10-12 16:19:48.437335
74096	TOUR	2020-10-09	1.13	1.15	1.1	1.11	109710	{}	\N	2020-10-12 16:19:48.4422
74097	TOUR	2020-10-12	1.12	1.14	1.11	1.13	75331	{}	\N	2020-10-12 16:19:48.447236
74098	TMHC	2020-09-28	24.1	25.24	23.78	25.13	644103	{}	\N	2020-10-12 16:19:52.574186
74099	TMHC	2020-09-29	25.13	25.28	24.14	24.31	1531483	{}	\N	2020-10-12 16:19:52.585396
74100	TMHC	2020-09-30	24.56	25.15	24.38	24.59	3263890	{}	\N	2020-10-12 16:19:52.591987
74101	TMHC	2020-10-01	24.86	25.63	24.6471	25.61	2262763	{}	\N	2020-10-12 16:19:52.597944
74102	TMHC	2020-10-02	24.8	26.12	24.491	25.92	1530952	{}	\N	2020-10-12 16:19:52.603586
74103	TMHC	2020-10-05	26.14	26.74	25.18	25.81	1992308	{}	\N	2020-10-12 16:19:52.609216
74104	TMHC	2020-10-06	26.22	26.25	24.81	25.02	2112830	{}	\N	2020-10-12 16:19:52.614328
74105	TMHC	2020-10-07	25.5	26	24.915	25.59	2291224	{}	\N	2020-10-12 16:19:52.619243
74106	TMHC	2020-10-08	26.35	26.81	25.94	26.48	1705256	{}	\N	2020-10-12 16:19:52.624538
74107	TMHC	2020-10-09	26.85	27.58	26.78	27.37	2151514	{}	\N	2020-10-12 16:19:52.630028
74108	TMHC	2020-10-12	27.45	27.59	27	27.3345	397628	{}	\N	2020-10-12 16:19:52.634703
74109	TNAV	2020-09-28	3.7	3.78	3.63	3.735	112146	{}	\N	2020-10-12 16:19:53.928714
74110	TNAV	2020-09-29	3.76	3.76	3.58	3.61	133538	{}	\N	2020-10-12 16:19:53.93992
74111	TNAV	2020-09-30	3.6	3.65	3.5	3.6	275831	{}	\N	2020-10-12 16:19:53.947761
74112	TNAV	2020-10-01	3.6	3.71	3.59	3.67	98606	{}	\N	2020-10-12 16:19:53.953964
74113	TNAV	2020-10-02	4.1642	4.44	4.03	4.35	844611	{}	\N	2020-10-12 16:19:53.959721
74114	TNAV	2020-10-05	4.35	4.38	4.24	4.29	659916	{}	\N	2020-10-12 16:19:53.964588
74115	TNAV	2020-10-06	4.29	4.33	4.24	4.29	568527	{}	\N	2020-10-12 16:19:53.969411
74116	TNAV	2020-10-07	4.32	4.32	4.21	4.24	506832	{}	\N	2020-10-12 16:19:53.974136
74117	TNAV	2020-10-08	4.25	4.26	4.18	4.23	250218	{}	\N	2020-10-12 16:19:53.978831
74118	TNAV	2020-10-09	4.24	4.2519	4.15	4.205	341295	{}	\N	2020-10-12 16:19:53.983473
74119	TNAV	2020-10-12	4.18	4.215	4.13	4.185	64018	{}	\N	2020-10-12 16:19:53.988195
74120	TOLZ	2020-09-28	38.62	38.62	38.3397	38.4299	6910	{}	\N	2020-10-12 16:19:57.229105
74121	TOLZ	2020-09-29	38.45	38.47	38.2341	38.3999	19483	{}	\N	2020-10-12 16:19:57.238884
74122	TOLZ	2020-09-30	38.37	38.66	38.165	38.18	5808	{}	\N	2020-10-12 16:19:57.245974
74123	TOLZ	2020-10-01	38.29	38.41	38.1895	38.1895	26628	{}	\N	2020-10-12 16:19:57.251974
74124	TOLZ	2020-10-02	37.79	38.69	37.79	38.645	22975	{}	\N	2020-10-12 16:19:57.257254
74125	TOLZ	2020-10-05	39.06	39.17	38.55	39.17	80237	{}	\N	2020-10-12 16:19:57.262517
74126	TOLZ	2020-10-06	39.03	39.25	38.7988	38.7988	13321	{}	\N	2020-10-12 16:19:57.266762
74127	TOLZ	2020-10-07	39.24	39.24	38.8297	39.05	16565	{}	\N	2020-10-12 16:19:57.271197
74128	TOLZ	2020-10-08	39.16	39.7	39.16	39.68	12240	{}	\N	2020-10-12 16:19:57.277051
74129	TOLZ	2020-10-09	39.95	39.95	39.47	39.56	19363	{}	\N	2020-10-12 16:19:57.281518
74130	TOLZ	2020-10-12	39.75	39.838	39.657	39.78	3003	{}	\N	2020-10-12 16:19:57.28644
74131	TPB	2020-09-28	28.87	29.08	28.09	28.38	99962	{}	\N	2020-10-12 16:19:58.579402
74132	TPB	2020-09-29	28.39	28.65	27.94	28.02	213900	{}	\N	2020-10-12 16:19:58.592021
74133	TPB	2020-09-30	27.89	28.22	27.84	27.9	415103	{}	\N	2020-10-12 16:19:58.599986
74134	TPB	2020-10-01	27.97	28	26.1801	26.41	692526	{}	\N	2020-10-12 16:19:58.606778
74135	TPB	2020-10-02	26.09	27.455	25.99	27.16	203673	{}	\N	2020-10-12 16:19:58.613261
74136	TPB	2020-10-05	27.96	30.4	27.67	30.35	692045	{}	\N	2020-10-12 16:19:58.618582
74137	TPB	2020-10-06	30.53	30.84	29.37	29.59	224491	{}	\N	2020-10-12 16:19:58.623904
74138	TPB	2020-10-07	29.92	30.77	29.77	30.54	163709	{}	\N	2020-10-12 16:19:58.629718
74139	TPB	2020-10-08	30.79	30.86	29.37	29.76	343209	{}	\N	2020-10-12 16:19:58.635319
74140	TPB	2020-10-09	30.05	30.8	29.93	30.2	145321	{}	\N	2020-10-12 16:19:58.641426
74141	TPB	2020-10-12	30.2	30.46	29.75	29.81	37290	{}	\N	2020-10-12 16:19:58.647135
74142	TPR	2020-09-28	16.69	16.95	16.49	16.58	5657843	{}	\N	2020-10-12 16:20:02.062527
74143	TPR	2020-09-29	16.51	16.52	15.48	15.5	11673761	{}	\N	2020-10-12 16:20:02.076074
74144	TPR	2020-09-30	15.55	15.91	15.45	15.63	10497565	{}	\N	2020-10-12 16:20:02.086013
74145	TPR	2020-10-01	15.71	16.19	15.65	15.9	9824371	{}	\N	2020-10-12 16:20:02.092521
74146	TPR	2020-10-02	15.23	16.29	15.23	16.15	8612730	{}	\N	2020-10-12 16:20:02.098308
74147	TPR	2020-10-05	16.39	16.98	16.39	16.97	12642947	{}	\N	2020-10-12 16:20:02.103674
74148	TPR	2020-10-06	17.31	17.82	16.815	16.83	14683692	{}	\N	2020-10-12 16:20:02.109104
74149	TPR	2020-10-07	17.27	18.015	17.27	17.87	10112786	{}	\N	2020-10-12 16:20:02.114847
74150	TPR	2020-10-08	18.17	18.75	18	18.67	12842537	{}	\N	2020-10-12 16:20:02.121157
74151	TPR	2020-10-09	18.97	19.28	18.62	18.82	10638601	{}	\N	2020-10-12 16:20:02.126539
74152	TPR	2020-10-12	18.9	19.23	18.8	19.185	3787679	{}	\N	2020-10-12 16:20:02.132067
74153	TREE	2020-09-28	307.7	310	300.15	301.12	89566	{}	\N	2020-10-12 16:20:03.041923
74154	TREE	2020-09-29	301.14	308.59	300.45	305.57	66595	{}	\N	2020-10-12 16:20:03.052996
74155	TREE	2020-09-30	305.09	312.425	303.3	306.89	126472	{}	\N	2020-10-12 16:20:03.059221
74156	TREE	2020-10-01	311	314.53	306.28	314.03	100149	{}	\N	2020-10-12 16:20:03.06576
74157	TREE	2020-10-02	306.32	313.765	306.32	310	66639	{}	\N	2020-10-12 16:20:03.072395
74158	TREE	2020-10-05	312.03	316.24	307.31	310.84	65454	{}	\N	2020-10-12 16:20:03.077156
74159	TREE	2020-10-06	312.9	324.13	312.5726	314.29	98380	{}	\N	2020-10-12 16:20:03.082129
74160	TREE	2020-10-07	316.16	321.02	314.75	320.81	46984	{}	\N	2020-10-12 16:20:03.087246
74161	TREE	2020-10-08	322	329.64	319.02	327.75	70699	{}	\N	2020-10-12 16:20:03.092231
74162	TREE	2020-10-09	328.5	334.63	326.04	326.53	72425	{}	\N	2020-10-12 16:20:03.097
74163	TREE	2020-10-12	328.75	335.37	325.845	326.29	21301	{}	\N	2020-10-12 16:20:03.102202
74164	TRNO	2020-09-28	55.1	55.66	54.36	55.62	385582	{}	\N	2020-10-12 16:20:05.398215
74165	TRNO	2020-09-29	55.32	55.58	54.635	54.94	588646	{}	\N	2020-10-12 16:20:05.406276
74166	TRNO	2020-09-30	55.28	55.47	54.0801	54.76	768498	{}	\N	2020-10-12 16:20:05.41302
74167	TRNO	2020-10-01	54.5	56.08	54.46	56.06	441250	{}	\N	2020-10-12 16:20:05.420206
74168	TRNO	2020-10-02	55.35	56.97	54.97	56.57	490229	{}	\N	2020-10-12 16:20:05.425707
74169	TRNO	2020-10-05	56.73	57.73	56.37	57.64	492595	{}	\N	2020-10-12 16:20:05.430152
74170	TRNO	2020-10-06	57.9	59.2	57.495	58.04	611912	{}	\N	2020-10-12 16:20:05.434722
74171	TRNO	2020-10-07	58.34	58.89	57.97	58.42	537721	{}	\N	2020-10-12 16:20:05.439256
74172	TRNO	2020-10-08	58.77	59.8	58.66	59.61	494578	{}	\N	2020-10-12 16:20:05.444568
74173	TRNO	2020-10-09	60.1	60.74	59.445	60.32	486095	{}	\N	2020-10-12 16:20:05.449515
74174	TRNO	2020-10-12	60.38	61.4	60.12	61.19	91861	{}	\N	2020-10-12 16:20:05.453556
74175	TRT	2020-09-28	3.45	3.45	3.4	3.4	2962	{}	\N	2020-10-12 16:20:08.960796
74176	TRT	2020-09-29	3.42	3.51	3.4	3.4	4567	{}	\N	2020-10-12 16:20:08.972041
74177	TRT	2020-09-30	3.42	3.42	3.42	3.42	248	{}	\N	2020-10-12 16:20:08.979126
74178	TRT	2020-10-01	3.455	3.455	3.4	3.4	564	{}	\N	2020-10-12 16:20:08.985612
74179	TRT	2020-10-02	3.5	3.5	3.43	3.43	1118	{}	\N	2020-10-12 16:20:08.991192
74180	TRT	2020-10-05	3.44	3.445	3.31	3.38	11780	{}	\N	2020-10-12 16:20:08.996105
74181	TRT	2020-10-06	3.45	3.475	3.4	3.4	1348	{}	\N	2020-10-12 16:20:09.00146
74182	TRT	2020-10-07	3.48	3.68	3.41	3.46	13631	{}	\N	2020-10-12 16:20:09.006109
74183	TRT	2020-10-08	3.95	4.275	3.6	3.81	182429	{}	\N	2020-10-12 16:20:09.011734
74184	TRT	2020-10-09	3.8	3.8073	3.65	3.65	17496	{}	\N	2020-10-12 16:20:09.018513
74185	TRT	2020-10-12	3.66	3.87	3.65	3.8	19405	{}	\N	2020-10-12 16:20:09.023898
74186	TRX	2020-09-28	0.69	0.7	0.6792	0.689	97237	{}	\N	2020-10-12 16:20:10.15524
74187	TRX	2020-09-29	0.7	0.7146	0.689	0.7146	118989	{}	\N	2020-10-12 16:20:10.165601
74188	TRX	2020-09-30	0.74	0.74	0.6922	0.696	123102	{}	\N	2020-10-12 16:20:10.1719
74189	TRX	2020-10-01	0.695	0.72	0.695	0.71	212460	{}	\N	2020-10-12 16:20:10.179894
74190	TRX	2020-10-02	0.69	0.714	0.6741	0.6741	175994	{}	\N	2020-10-12 16:20:10.185176
74191	TRX	2020-10-05	0.69	0.7097	0.6775	0.7	311959	{}	\N	2020-10-12 16:20:10.189352
74192	TRX	2020-10-06	0.705	0.705	0.68	0.689	246404	{}	\N	2020-10-12 16:20:10.193871
74193	TRX	2020-10-07	0.696	0.7	0.6751	0.7	144867	{}	\N	2020-10-12 16:20:10.198188
74194	TRX	2020-10-08	0.71	0.71	0.68	0.69	218744	{}	\N	2020-10-12 16:20:10.202137
74195	TRX	2020-10-09	0.7	0.774	0.68	0.7475	553789	{}	\N	2020-10-12 16:20:10.206342
74196	TRX	2020-10-12	0.75	0.771	0.7437	0.7534	196692	{}	\N	2020-10-12 16:20:10.210235
74197	TKAT	2020-09-28	0.788501	0.856	0.785	0.8225	1930	{}	\N	2020-10-12 16:20:12.311114
74198	TKAT	2020-09-29	0.82	0.8225	0.8	0.8	7558	{}	\N	2020-10-12 16:20:12.320311
74199	TKAT	2020-09-30	0.85	0.8756	0.85	0.8756	1180	{}	\N	2020-10-12 16:20:12.327126
74200	TKAT	2020-10-01	0.94	1.1701	0.920101	1.02	289915	{}	\N	2020-10-12 16:20:12.333989
74201	TKAT	2020-10-02	1.09	1.09	0.9656	1.0324	10603	{}	\N	2020-10-12 16:20:12.339265
74202	TKAT	2020-10-05	1.07	1.07	0.98	0.98	2159	{}	\N	2020-10-12 16:20:12.344848
74203	TKAT	2020-10-06	0.98	1.0484	0.98	1.04	3104	{}	\N	2020-10-12 16:20:12.349254
74204	TKAT	2020-10-07	1.04	1.1	1.04	1.0501	5078	{}	\N	2020-10-12 16:20:12.354148
74205	TKAT	2020-10-08	1.03	1.05	1.0141	1.04	4043	{}	\N	2020-10-12 16:20:12.359688
74206	TKAT	2020-10-09	1.04	1.05	0.94	1	7976	{}	\N	2020-10-12 16:20:12.364483
74207	TMQ	2020-09-28	1.8	1.8451	1.79	1.79	130154	{}	\N	2020-10-12 16:20:16.827021
74208	TMQ	2020-09-29	1.78	1.83	1.78	1.825	114837	{}	\N	2020-10-12 16:20:16.839216
74209	TMQ	2020-09-30	1.83	1.84	1.78	1.79	93003	{}	\N	2020-10-12 16:20:16.846982
74210	TMQ	2020-10-01	1.8	1.8	1.69	1.73	201810	{}	\N	2020-10-12 16:20:16.853933
74211	TMQ	2020-10-02	1.72	1.77	1.71	1.74	81466	{}	\N	2020-10-12 16:20:16.860537
74212	TMQ	2020-10-05	1.75	1.8	1.74	1.78	120550	{}	\N	2020-10-12 16:20:16.865203
74213	TMQ	2020-10-06	1.8	1.83	1.74	1.75	148132	{}	\N	2020-10-12 16:20:16.86949
74214	TMQ	2020-10-07	1.8	1.83	1.73	1.74	114504	{}	\N	2020-10-12 16:20:16.873687
74215	TMQ	2020-10-08	1.75	1.79	1.73	1.75	97546	{}	\N	2020-10-12 16:20:16.877989
74216	TMQ	2020-10-09	1.78	1.8	1.77	1.8	107655	{}	\N	2020-10-12 16:20:16.882627
74217	TMQ	2020-10-12	1.8	1.8	1.7401	1.76	128645	{}	\N	2020-10-12 16:20:16.887184
74218	TNP	2020-09-28	7.35	7.63	7.09	7.53	123942	{}	\N	2020-10-12 16:20:18.409598
74219	TNP	2020-09-29	7.55	7.5518	7.17	7.42	191857	{}	\N	2020-10-12 16:20:18.425728
74220	TNP	2020-09-30	7.42	7.99	7.4037	7.92	248688	{}	\N	2020-10-12 16:20:18.435073
74221	TNP	2020-10-01	7.88	7.96	7.6	7.91	265891	{}	\N	2020-10-12 16:20:18.440675
74222	TNP	2020-10-02	7.73	7.885	7.55	7.79	139885	{}	\N	2020-10-12 16:20:18.445993
74223	TNP	2020-10-05	7.79	8.44	7.775	8.22	218918	{}	\N	2020-10-12 16:20:18.451033
74224	TNP	2020-10-06	8.3	8.5931	7.93	7.94	212039	{}	\N	2020-10-12 16:20:18.455953
74225	TNP	2020-10-07	8.08	8.08	7.745	7.83	141572	{}	\N	2020-10-12 16:20:18.461641
74226	TNP	2020-10-08	7.87	8.62	7.87	8.53	199326	{}	\N	2020-10-12 16:20:18.466433
74227	TNP	2020-10-09	8.56	8.62	8.1	8.1	107422	{}	\N	2020-10-12 16:20:18.470651
74228	TNP	2020-10-12	8.14	8.32	7.91	8.32	129872	{}	\N	2020-10-12 16:20:18.47492
74229	TPHS	2020-09-28	1.3	1.345	1.27	1.29	15207	{}	\N	2020-10-12 16:20:20.736561
74230	TPHS	2020-09-29	1.3	1.54	1.27	1.4199	334312	{}	\N	2020-10-12 16:20:20.745238
74231	TPHS	2020-09-30	1.49	1.49	1.41	1.41	64317	{}	\N	2020-10-12 16:20:20.75186
74232	TPHS	2020-10-01	1.46	1.46	1.35	1.4	21754	{}	\N	2020-10-12 16:20:20.757472
74233	TPHS	2020-10-02	1.35	1.42	1.35	1.39	8816	{}	\N	2020-10-12 16:20:20.763216
74234	TPHS	2020-10-05	1.4	1.4	1.3	1.3	18942	{}	\N	2020-10-12 16:20:20.768269
74235	TPHS	2020-10-06	1.3	1.37	1.29	1.3	12777	{}	\N	2020-10-12 16:20:20.773326
74236	TPHS	2020-10-07	1.32	1.36	1.32	1.34	7678	{}	\N	2020-10-12 16:20:20.777768
74237	TPHS	2020-10-08	1.29	1.3253	1.28	1.3	11515	{}	\N	2020-10-12 16:20:20.783155
74238	TPHS	2020-10-09	1.3	1.39	1.27	1.3	32880	{}	\N	2020-10-12 16:20:20.78868
74239	TPHS	2020-10-12	1.27	1.29	1.26	1.28	9112	{}	\N	2020-10-12 16:20:20.793139
74240	TPYP	2020-09-28	15.29	15.38	15.15	15.3	96582	{}	\N	2020-10-12 16:20:21.87081
74241	TPYP	2020-09-29	15.32	15.37	15.13	15.26	121784	{}	\N	2020-10-12 16:20:21.883755
74242	TPYP	2020-09-30	15.16	15.38	14.96	14.98	111335	{}	\N	2020-10-12 16:20:21.892159
74243	TPYP	2020-10-01	14.97	15.03	14.7839	14.86	62056	{}	\N	2020-10-12 16:20:21.8984
74244	TPYP	2020-10-02	14.71	15.08	14.61	15.04	100895	{}	\N	2020-10-12 16:20:21.904359
74245	TPYP	2020-10-05	15.11	15.34	15.09	15.31	46141	{}	\N	2020-10-12 16:20:21.909361
74246	TPYP	2020-10-06	15.52	15.539	15.14	15.2	39936	{}	\N	2020-10-12 16:20:21.914217
74247	TPYP	2020-10-07	15.25	15.3543	15.14	15.27	305405	{}	\N	2020-10-12 16:20:21.919648
74248	TPYP	2020-10-08	15.34	15.91	15.34	15.91	78580	{}	\N	2020-10-12 16:20:21.926231
74249	TPYP	2020-10-09	16.07	16.07	15.71	15.76	118600	{}	\N	2020-10-12 16:20:21.930664
74250	TPYP	2020-10-12	15.84	15.84	15.7	15.82	40079	{}	\N	2020-10-12 16:20:21.935651
74251	TRIB	2020-09-28	2.05	2.15	2.0366	2.09	132481	{}	\N	2020-10-12 16:20:23.33811
74252	TRIB	2020-09-29	2.08	2.177	2.05	2.06	222127	{}	\N	2020-10-12 16:20:23.350051
74253	TRIB	2020-09-30	2.08	2.14	1.96	2.02	131488	{}	\N	2020-10-12 16:20:23.359121
74254	TRIB	2020-10-01	2.03	2.0625	1.96	1.98	219853	{}	\N	2020-10-12 16:20:23.364983
74255	TRIB	2020-10-02	1.91	2.04	1.82	1.95	203114	{}	\N	2020-10-12 16:20:23.371315
74256	TRIB	2020-10-05	1.97	2.09	1.96	2.06	181567	{}	\N	2020-10-12 16:20:23.375876
74257	TRIB	2020-10-06	2.07	2.0937	2.0215	2.05	207503	{}	\N	2020-10-12 16:20:23.380247
74258	TRIB	2020-10-07	2.05	2.18	2.05	2.14	190840	{}	\N	2020-10-12 16:20:23.385028
74259	TRIB	2020-10-08	2.14	2.2	2.1	2.18	117822	{}	\N	2020-10-12 16:20:23.389985
74260	TRIB	2020-10-09	2.25	2.25	2.19	2.22	139570	{}	\N	2020-10-12 16:20:23.396066
74261	TRIB	2020-10-12	2.23	2.32	2.19	2.28	106861	{}	\N	2020-10-12 16:20:23.400936
74262	TRN	2020-09-28	19.65	20.04	19.65	19.76	1254477	{}	\N	2020-10-12 16:20:26.54352
74263	TRN	2020-09-29	19.85	19.94	19.505	19.51	1574525	{}	\N	2020-10-12 16:20:26.558389
74264	TRN	2020-09-30	19.7	19.9	19.42	19.5	2525188	{}	\N	2020-10-12 16:20:26.570135
74265	TRN	2020-10-01	19.5	19.72	19.23	19.47	1570239	{}	\N	2020-10-12 16:20:26.576789
74266	TRN	2020-10-02	19.15	19.67	19.1	19.36	2487485	{}	\N	2020-10-12 16:20:26.583536
74267	TRN	2020-10-05	19.7	20.36	19.555	20.29	2322552	{}	\N	2020-10-12 16:20:26.588928
74268	TRN	2020-10-06	20.55	20.66	19.905	20.04	1912811	{}	\N	2020-10-12 16:20:26.594324
74269	TRN	2020-10-07	20.2	20.655	20.2	20.44	1221007	{}	\N	2020-10-12 16:20:26.599485
74270	TRN	2020-10-08	20.58	20.7	20.22	20.68	874407	{}	\N	2020-10-12 16:20:26.604363
74271	TRN	2020-10-09	20.93	21	20.39	20.41	1393414	{}	\N	2020-10-12 16:20:26.61024
74272	TRN	2020-10-12	20.45	20.83	20.45	20.66	387610	{}	\N	2020-10-12 16:20:26.615499
74273	TRP	2020-09-28	45	45.29	44.13	44.23	2149534	{}	\N	2020-10-12 16:20:27.792361
74274	TRP	2020-09-29	43.54	43.78	42.81	43.26	2570272	{}	\N	2020-10-12 16:20:27.805732
74275	TRP	2020-09-30	43.42	43.64	41.98	42.02	3604862	{}	\N	2020-10-12 16:20:27.814935
74276	TRP	2020-10-01	41.89	42.2	41.37	41.68	3474343	{}	\N	2020-10-12 16:20:27.821112
74277	TRP	2020-10-02	41.3	42.05	41.05	42.01	3230713	{}	\N	2020-10-12 16:20:27.827019
74278	TRP	2020-10-05	42.36	42.36	41.36	42.15	3630613	{}	\N	2020-10-12 16:20:27.831858
74279	TRP	2020-10-06	42.66	42.67	40.92	41.37	4743780	{}	\N	2020-10-12 16:20:27.836875
74280	TRP	2020-10-07	41.3	42.05	41.16	41.96	4102667	{}	\N	2020-10-12 16:20:27.843266
74281	TRP	2020-10-08	42.32	43.94	42.32	43.91	2858093	{}	\N	2020-10-12 16:20:27.848074
74282	TRP	2020-10-09	44.08	44.32	43.61	43.78	3469273	{}	\N	2020-10-12 16:20:27.853233
74283	TRP	2020-10-12	43.82	43.845	43.03	43.04	1015105	{}	\N	2020-10-12 16:20:27.858626
74284	TS	2020-09-28	9.85	10.01	9.81	9.86	1915519	{}	\N	2020-10-12 16:20:29.065741
74285	TS	2020-09-29	9.93	9.94	9.54	9.72	3197045	{}	\N	2020-10-12 16:20:29.075506
74286	TS	2020-09-30	9.96	10.08	9.81	9.85	5475926	{}	\N	2020-10-12 16:20:29.082526
74287	TS	2020-10-01	9.78	9.85	9.6	9.75	5889590	{}	\N	2020-10-12 16:20:29.089461
74288	TS	2020-10-02	9.57	10.065	9.54	9.91	4245365	{}	\N	2020-10-12 16:20:29.094649
74289	TS	2020-10-05	10.06	10.15	9.98	10.1	2324273	{}	\N	2020-10-12 16:20:29.098969
74290	TS	2020-10-06	10.34	10.41	10.045	10.12	6349569	{}	\N	2020-10-12 16:20:29.10346
74291	TS	2020-10-07	10.15	10.22	10.025	10.19	4078575	{}	\N	2020-10-12 16:20:29.107907
74292	TS	2020-10-08	10.34	10.53	10.315	10.51	2362599	{}	\N	2020-10-12 16:20:29.112817
74293	TS	2020-10-09	10.6	10.66	10.35	10.44	2815023	{}	\N	2020-10-12 16:20:29.118023
74294	TS	2020-10-12	10.35	10.37	10.21	10.215	939630	{}	\N	2020-10-12 16:20:29.122503
74295	TSLX	2020-09-28	17.01	17.3384	16.94	17.19	264877	{}	\N	2020-10-12 16:20:31.410083
74296	TSLX	2020-09-29	17.16	17.32	17.015	17.03	389310	{}	\N	2020-10-12 16:20:31.422724
74297	TSLX	2020-09-30	16.97	17.2744	16.97	17.21	640351	{}	\N	2020-10-12 16:20:31.430729
74298	TSLX	2020-10-01	17.21	17.35	17.15	17.32	476680	{}	\N	2020-10-12 16:20:31.437004
74299	TSLX	2020-10-02	17.17	17.3818	17.06	17.35	335038	{}	\N	2020-10-12 16:20:31.442618
74300	TSLX	2020-10-05	17.3	17.465	17.2	17.25	423110	{}	\N	2020-10-12 16:20:31.448185
74301	TSLX	2020-10-06	17.34	17.37	17.03	17.03	620316	{}	\N	2020-10-12 16:20:31.453749
74302	TSLX	2020-10-07	17.1	17.25	17.03	17.16	351269	{}	\N	2020-10-12 16:20:31.45975
74303	TSLX	2020-10-08	17.12	17.28	17.06	17.23	289139	{}	\N	2020-10-12 16:20:31.465479
74304	TSLX	2020-10-09	17.29	17.39	17.15	17.29	405765	{}	\N	2020-10-12 16:20:31.470758
74305	TSLX	2020-10-12	17.35	17.3518	17.2004	17.29	230220	{}	\N	2020-10-12 16:20:31.476041
74306	DSS	2020-09-28	4.48	4.71	4.2	4.63	104132	{}	\N	2020-10-12 16:20:34.167114
74307	DSS	2020-09-29	4.63	4.72	4.46	4.63	55626	{}	\N	2020-10-12 16:20:34.17957
74308	DSS	2020-09-30	4.65	4.66	4.46	4.56	71316	{}	\N	2020-10-12 16:20:34.187916
74309	DSS	2020-10-01	4.85	4.95	4.61	4.67	120855	{}	\N	2020-10-12 16:20:34.195009
74310	DSS	2020-10-02	4.5	4.69	4.417	4.48	97909	{}	\N	2020-10-12 16:20:34.201584
74311	DSS	2020-10-05	4.46	4.63	4.46	4.55	59101	{}	\N	2020-10-12 16:20:34.206864
74312	DSS	2020-10-06	4.52	4.61	4.3	4.42	110558	{}	\N	2020-10-12 16:20:34.212118
74313	DSS	2020-10-07	4.41	4.97	4.4	4.94	736944	{}	\N	2020-10-12 16:20:34.217652
74314	DSS	2020-10-08	4.9	4.9025	4.445	4.57	232574	{}	\N	2020-10-12 16:20:34.223023
74315	DSS	2020-10-09	4.6	4.7	4.54	4.56	109530	{}	\N	2020-10-12 16:20:34.228133
74316	DSS	2020-10-12	4.5	4.55	4.35	4.38	76130	{}	\N	2020-10-12 16:20:34.233422
74317	TNK	2020-09-28	10.75	10.88	10.41	10.72	389319	{}	\N	2020-10-12 16:20:35.57779
74318	TNK	2020-09-29	10.73	10.9868	10.52	10.84	823758	{}	\N	2020-10-12 16:20:35.588693
74319	TNK	2020-09-30	10.94	11.3	10.82	10.84	921838	{}	\N	2020-10-12 16:20:35.596223
74320	TNK	2020-10-01	10.8	11.04	10.63	11.03	787599	{}	\N	2020-10-12 16:20:35.602706
74321	TNK	2020-10-02	10.79	11.3	10.51	11.24	1143392	{}	\N	2020-10-12 16:20:35.608878
74322	TNK	2020-10-05	11.25	11.74	11.25	11.56	702538	{}	\N	2020-10-12 16:20:35.613967
74323	TNK	2020-10-06	11.59	11.77	11.13	11.25	659623	{}	\N	2020-10-12 16:20:35.620053
74324	TNK	2020-10-07	11.43	11.46	11.04	11.14	739378	{}	\N	2020-10-12 16:20:35.62453
74325	TNK	2020-10-08	11.14	11.52	11.09	11.46	752690	{}	\N	2020-10-12 16:20:35.62888
74326	TNK	2020-10-09	11.33	11.52	11.22	11.33	846083	{}	\N	2020-10-12 16:20:35.633234
74327	TNK	2020-10-12	11.33	11.33	10.69	10.85	445194	{}	\N	2020-10-12 16:20:35.637774
74328	TPL	2020-09-28	459.2155	464	454.6	461.48	13836	{}	\N	2020-10-12 16:20:37.861475
74329	TPL	2020-09-29	454.96	454.96	439.99	452.06	39042	{}	\N	2020-10-12 16:20:37.871366
74330	TPL	2020-09-30	452.06	462.98	451.56	451.56	42663	{}	\N	2020-10-12 16:20:37.878054
74331	TPL	2020-10-01	450.58	460	440.2201	450.2	60913	{}	\N	2020-10-12 16:20:37.885299
74332	TPL	2020-10-02	440	453.7119	437.01	448.1	36733	{}	\N	2020-10-12 16:20:37.890998
74333	TPL	2020-10-05	451	459.64	450.47	455.11	25678	{}	\N	2020-10-12 16:20:37.896563
74334	TPL	2020-10-06	461.98	475.03	457.2	460.22	28188	{}	\N	2020-10-12 16:20:37.902073
74335	TPL	2020-10-07	458.96	477.62	458.7079	472.98	31242	{}	\N	2020-10-12 16:20:37.907996
74336	TPL	2020-10-08	478.55	484.73	470.25	484.73	29921	{}	\N	2020-10-12 16:20:37.913988
74337	TPL	2020-10-09	484.73	485	462.84	466.6	41013	{}	\N	2020-10-12 16:20:37.919804
74338	TPL	2020-10-12	468	471.653	462	465	14450	{}	\N	2020-10-12 16:20:37.925192
74339	TRC	2020-09-28	13.93	14.5699	13.93	14.41	78246	{}	\N	2020-10-12 16:20:39.315635
74340	TRC	2020-09-29	14.39	14.43	14.15	14.29	77990	{}	\N	2020-10-12 16:20:39.325494
74341	TRC	2020-09-30	14.31	14.58	14.07	14.15	148513	{}	\N	2020-10-12 16:20:39.332343
74342	TRC	2020-10-01	14.11	14.31	13.93	14.18	178548	{}	\N	2020-10-12 16:20:39.33871
74343	TRC	2020-10-02	13.98	14.25	13.89	14.18	91362	{}	\N	2020-10-12 16:20:39.344446
74344	TRC	2020-10-05	14.25	14.51	14.25	14.47	89696	{}	\N	2020-10-12 16:20:39.349405
74345	TRC	2020-10-06	14.62	15	14.42	14.49	153064	{}	\N	2020-10-12 16:20:39.354624
74346	TRC	2020-10-07	14.68	15.13	14.6	14.89	156021	{}	\N	2020-10-12 16:20:39.359574
74347	TRC	2020-10-08	15.09	15.25	14.87	15.19	137363	{}	\N	2020-10-12 16:20:39.3648
74348	TRC	2020-10-09	15.35	15.84	15.0701	15.75	277999	{}	\N	2020-10-12 16:20:39.369413
74349	TRC	2020-10-12	15.71	15.82	15.6119	15.75	41618	{}	\N	2020-10-12 16:20:39.373951
74350	TRI	2020-09-28	79.21	80.17	79.12	80.15	528313	{}	\N	2020-10-12 16:20:42.107065
74351	TRI	2020-09-29	80	80.58	79.63	79.88	893688	{}	\N	2020-10-12 16:20:42.117734
74352	TRI	2020-09-30	79.82	80.43	79.44	79.77	1007623	{}	\N	2020-10-12 16:20:42.125691
74353	TRI	2020-10-01	80.17	80.92	79.9	80.54	797602	{}	\N	2020-10-12 16:20:42.131316
74354	TRI	2020-10-02	80.07	80.65	79.3239	80.12	811338	{}	\N	2020-10-12 16:20:42.136719
74355	TRI	2020-10-05	80.36	81.19	80.22	81.1	437874	{}	\N	2020-10-12 16:20:42.141768
74356	TRI	2020-10-06	81.16	81.33	80.2921	80.41	453296	{}	\N	2020-10-12 16:20:42.146518
74357	TRI	2020-10-07	80.89	81.21	80.47	81.19	464553	{}	\N	2020-10-12 16:20:42.151297
74358	TRI	2020-10-08	81.44	81.74	81.3	81.63	907719	{}	\N	2020-10-12 16:20:42.155634
74359	TRI	2020-10-09	81.99	82.7	81.75	82.36	830881	{}	\N	2020-10-12 16:20:42.159776
74360	TRI	2020-10-12	82.82	83.13	82.45	83.13	103164	{}	\N	2020-10-12 16:20:42.163706
74361	TRMT	2020-09-28	2.81	3.5	2.79	3.01	92275	{}	\N	2020-10-12 16:20:43.447786
74362	TRMT	2020-09-29	3.11	3.11	2.9	2.91	11996	{}	\N	2020-10-12 16:20:43.457412
74363	TRMT	2020-09-30	2.9	3.05	2.9	2.92	7064	{}	\N	2020-10-12 16:20:43.463326
74364	TRMT	2020-10-01	2.92	2.99	2.915	2.93	2420	{}	\N	2020-10-12 16:20:43.469569
74365	TRMT	2020-10-02	2.8	2.95	2.8	2.81	20702	{}	\N	2020-10-12 16:20:43.475096
74366	TRMT	2020-10-05	2.88	2.92	2.81	2.82	19063	{}	\N	2020-10-12 16:20:43.48027
74367	TRMT	2020-10-06	2.86	2.91	2.8	2.82	15131	{}	\N	2020-10-12 16:20:43.48472
74368	TRMT	2020-10-07	2.84	2.89	2.83	2.87	14202	{}	\N	2020-10-12 16:20:43.488766
74369	TRMT	2020-10-08	2.92	2.99	2.876	2.9	16416	{}	\N	2020-10-12 16:20:43.492887
74370	TRMT	2020-10-09	2.87	2.98	2.87	2.88	21146	{}	\N	2020-10-12 16:20:43.49708
74371	TRMT	2020-10-12	2.9	2.97	2.8944	2.97	5522	{}	\N	2020-10-12 16:20:43.501696
74372	TRS	2020-09-28	23.07	23.49	22.85	23.14	102614	{}	\N	2020-10-12 16:20:44.750083
74373	TRS	2020-09-29	23.15	23.2	22.57	22.77	168000	{}	\N	2020-10-12 16:20:44.758291
74374	TRS	2020-09-30	22.82	23.085	22.56	22.8	248183	{}	\N	2020-10-12 16:20:44.763686
74375	TRS	2020-10-01	22.82	23.06	22.67	23.05	155112	{}	\N	2020-10-12 16:20:44.770074
74376	TRS	2020-10-02	22.61	23.61	22.61	23.47	136260	{}	\N	2020-10-12 16:20:44.776245
74377	TRS	2020-10-05	23.67	23.991	23.48	23.74	101334	{}	\N	2020-10-12 16:20:44.781063
74378	TRS	2020-10-06	23.88	24.08	23.44	23.53	100537	{}	\N	2020-10-12 16:20:44.786145
74379	TRS	2020-10-07	23.7	24.18	23.59	24.08	145361	{}	\N	2020-10-12 16:20:44.791296
74380	TRS	2020-10-08	24.29	24.74	24.085	24.5	77233	{}	\N	2020-10-12 16:20:44.796524
74381	TRS	2020-10-09	24.7	24.93	24.59	24.73	64336	{}	\N	2020-10-12 16:20:44.801207
74382	TRS	2020-10-12	24.73	25.14	24.51	24.87	19911	{}	\N	2020-10-12 16:20:44.805579
74383	TRVN	2020-09-28	2.93	2.95	2.75	2.82	7300088	{}	\N	2020-10-12 16:20:46.48871
74384	TRVN	2020-09-29	2.8	3.02	2.77	2.98	8794004	{}	\N	2020-10-12 16:20:46.497315
74385	TRVN	2020-09-30	2.96	3.03	2.87	2.99	5189289	{}	\N	2020-10-12 16:20:46.504097
74386	TRVN	2020-10-01	2.95	3.03	2.8683	2.96	4238224	{}	\N	2020-10-12 16:20:46.510429
74387	TRVN	2020-10-02	2.79	2.95	2.75	2.9	5412129	{}	\N	2020-10-12 16:20:46.516408
74388	TRVN	2020-10-05	2.93	3.1	2.9	3.08	6941429	{}	\N	2020-10-12 16:20:46.52049
74389	TRVN	2020-10-06	3.08	3.305	3.04	3.14	10606882	{}	\N	2020-10-12 16:20:46.524409
74390	TRVN	2020-10-07	3.17	3.4	3.14	3.19	9095800	{}	\N	2020-10-12 16:20:46.529
74391	TRVN	2020-10-08	3.23	3.3	3.21	3.26	4106227	{}	\N	2020-10-12 16:20:46.533689
74392	TRVN	2020-10-09	3.27	3.2867	3.11	3.15	5984914	{}	\N	2020-10-12 16:20:46.538409
74393	TRVN	2020-10-12	3.13	3.16	2.96	2.995	4830993	{}	\N	2020-10-12 16:20:46.543909
74394	TSLA	2020-09-28	424.62	428.08	415.55	421.2	49401383	{}	\N	2020-10-12 16:20:47.599692
74395	TSLA	2020-09-29	416	428.5	411.6	419.07	50180166	{}	\N	2020-10-12 16:20:47.611089
74396	TSLA	2020-09-30	421.32	433.93	420.47	429.01	47940745	{}	\N	2020-10-12 16:20:47.618112
74397	TSLA	2020-10-01	440.76	448.88	434.42	448.16	50282175	{}	\N	2020-10-12 16:20:47.624138
74398	TSLA	2020-10-02	421.39	439.13	415	415.09	70864144	{}	\N	2020-10-12 16:20:47.632543
74399	TSLA	2020-10-05	423.35	433.64	419.33	425.68	44812583	{}	\N	2020-10-12 16:20:47.636901
74400	TSLA	2020-10-06	423.79	428.7799	406.05	413.98	48793289	{}	\N	2020-10-12 16:20:47.641651
74401	TSLA	2020-10-07	419.87	429.9	413.845	425.3	43343782	{}	\N	2020-10-12 16:20:47.646399
74402	TSLA	2020-10-08	438.44	439	425.3	425.92	40780983	{}	\N	2020-10-12 16:20:47.650941
74403	TSLA	2020-10-09	430.13	434.5899	426.4601	434	29387763	{}	\N	2020-10-12 16:20:47.655592
74404	TSLA	2020-10-12	442	445.85	438.58	443.13	19055920	{}	\N	2020-10-12 16:20:47.660398
74405	TSRI	2020-09-28	5.04	5.08	4.71	5.06	764	{}	\N	2020-10-12 16:20:50.816842
74406	TSRI	2020-09-29	5.05	5.06	5.037	5.06	1728	{}	\N	2020-10-12 16:20:50.827571
74407	TSRI	2020-09-30	5.0899	5.192	4.97	5.192	4016	{}	\N	2020-10-12 16:20:50.835386
74408	TSRI	2020-10-01	5.1	5.2	5.03	5.2	6163	{}	\N	2020-10-12 16:20:50.842
74409	TSRI	2020-10-02	5.4041	5.5436	4.86	4.86	12959	{}	\N	2020-10-12 16:20:50.848052
74410	TSRI	2020-10-05	5.68	5.68	5.1001	5.35	1756	{}	\N	2020-10-12 16:20:50.853189
74411	TSRI	2020-10-06	5.01	5.3033	5.01	5.1	756	{}	\N	2020-10-12 16:20:50.85795
74412	TSRI	2020-10-07	5.13	5.13	5.1	5.13	1756	{}	\N	2020-10-12 16:20:50.863265
74413	TSRI	2020-10-08	5.6999	5.7	5.3301	5.385	1585	{}	\N	2020-10-12 16:20:50.868562
74414	TSRI	2020-10-09	4.96	5.109	4.96	5.1	4418	{}	\N	2020-10-12 16:20:50.872861
74415	TSRI	2020-10-12	5.26	5.3	4.91	4.91	2101	{}	\N	2020-10-12 16:20:50.877509
74416	TTEK	2020-09-28	92.76	94.71	92.63	94.63	258661	{}	\N	2020-10-12 16:20:51.80432
74417	TTEK	2020-09-29	94.57	96.08	93.98	94.97	308541	{}	\N	2020-10-12 16:20:51.813905
74418	TTEK	2020-09-30	95.01	96.42	94.295	95.5	343197	{}	\N	2020-10-12 16:20:51.821828
74419	TTEK	2020-10-01	95.78	97.46	95.47	97.16	361040	{}	\N	2020-10-12 16:20:51.827468
74420	TTEK	2020-10-02	95.58	99.76	95.3	98.58	336823	{}	\N	2020-10-12 16:20:51.83316
74421	TTEK	2020-10-05	99.31	102.53	99.14	101.1	295849	{}	\N	2020-10-12 16:20:51.83865
74422	TTEK	2020-10-06	102.425	103.93	100.26	100.45	384858	{}	\N	2020-10-12 16:20:51.843435
74423	TTEK	2020-10-07	101.82	102.25	100.29	101.44	293245	{}	\N	2020-10-12 16:20:51.848289
74424	TTEK	2020-10-08	102.72	103.19	101.01	102.14	286306	{}	\N	2020-10-12 16:20:51.853396
74425	TTEK	2020-10-09	103.02	103.63	101.46	102.56	227487	{}	\N	2020-10-12 16:20:51.858862
74426	TTEK	2020-10-12	104.14	107.81	104.14	107.335	132491	{}	\N	2020-10-12 16:20:51.863799
74427	TTMI	2020-09-28	11.27	11.56	11.19	11.44	1604357	{}	\N	2020-10-12 16:20:53.081512
74428	TTMI	2020-09-29	11.76	11.8	11.36	11.48	1817764	{}	\N	2020-10-12 16:20:53.097356
74429	TTMI	2020-09-30	11.5	11.695	11.32	11.41	1742531	{}	\N	2020-10-12 16:20:53.106101
74430	TTMI	2020-10-01	11.51	11.625	11.38	11.57	778681	{}	\N	2020-10-12 16:20:53.11213
74431	TTMI	2020-10-02	11.33	11.69	11.2	11.45	932465	{}	\N	2020-10-12 16:20:53.118552
74432	TTMI	2020-10-05	11.56	11.7	11.515	11.595	978825	{}	\N	2020-10-12 16:20:53.1236
74433	TTMI	2020-10-06	11.69	12.23	11.58	11.865	3126238	{}	\N	2020-10-12 16:20:53.128172
74434	TTMI	2020-10-07	11.97	12.09	11.87	11.965	1337607	{}	\N	2020-10-12 16:20:53.13333
74435	TTMI	2020-10-08	12.11	12.31	12.07	12.18	1152127	{}	\N	2020-10-12 16:20:53.13766
74436	TTMI	2020-10-09	12.29	12.53	12.28	12.51	1400463	{}	\N	2020-10-12 16:20:53.142685
74437	TTMI	2020-10-12	12.56	12.67	12.45	12.6285	677493	{}	\N	2020-10-12 16:20:53.147783
74438	TLT	2020-09-28	164.87	164.9	164.37	164.65	4480713	{}	\N	2020-10-12 16:20:54.399675
74439	TLT	2020-09-29	164.8	165.24	164.56	164.84	10316403	{}	\N	2020-10-12 16:20:54.411154
74440	TLT	2020-09-30	164	164	162.55	163.26	13116496	{}	\N	2020-10-12 16:20:54.418537
74441	TLT	2020-10-01	162.51	163.7401	162.1	163.36	11444969	{}	\N	2020-10-12 16:20:54.424898
74442	TLT	2020-10-02	163.61	163.67	162.18	162.75	8680146	{}	\N	2020-10-12 16:20:54.430115
74443	TLT	2020-10-05	160.95	160.95	159.52	159.57	14802894	{}	\N	2020-10-12 16:20:54.435957
74444	TLT	2020-10-06	159.33	161.405	158.68	160.43	22387796	{}	\N	2020-10-12 16:20:54.440825
74445	TLT	2020-10-07	159.56	160.417	158.86	159.26	9591867	{}	\N	2020-10-12 16:20:54.448468
74446	TLT	2020-10-08	159.97	160.25	159.655	160.12	8603193	{}	\N	2020-10-12 16:20:54.453349
74447	TLT	2020-10-09	159.52	160.27	158.81	160.09	8522830	{}	\N	2020-10-12 16:20:54.458027
74448	TLT	2020-10-12	160.43	160.54	160.11	160.195	1719943	{}	\N	2020-10-12 16:20:54.462259
74449	TMUS	2020-09-28	112.87	114.48	112.485	114.24	5708453	{}	\N	2020-10-12 16:20:56.163888
74450	TMUS	2020-09-29	114.35	114.84	113.29	113.52	3136522	{}	\N	2020-10-12 16:20:56.1784
74451	TMUS	2020-09-30	113.36	115.09	112.01	114.36	5061523	{}	\N	2020-10-12 16:20:56.18723
74452	TMUS	2020-10-01	115	116.43	113.89	115.73	4109986	{}	\N	2020-10-12 16:20:56.19353
74453	TMUS	2020-10-02	114.41	115.7	113	113.41	3810223	{}	\N	2020-10-12 16:20:56.199196
74454	TMUS	2020-10-05	113.46	114.99	113.42	114.8	2605278	{}	\N	2020-10-12 16:20:56.204492
74455	TMUS	2020-10-06	113.61	114.47	111.8	112.45	3187143	{}	\N	2020-10-12 16:20:56.209679
74456	TMUS	2020-10-07	113.02	114.5199	112.62	114.22	3315696	{}	\N	2020-10-12 16:20:56.214692
74457	TMUS	2020-10-08	114.89	115.59	114.2801	115.49	2233846	{}	\N	2020-10-12 16:20:56.219798
74458	TMUS	2020-10-09	115.5833	117.52	115.52	117.23	3296740	{}	\N	2020-10-12 16:20:56.225754
74459	TMUS	2020-10-12	118.06	118.87	117.4	118.62	1905460	{}	\N	2020-10-12 16:20:56.23121
74460	TOWN	2020-09-28	16.11	16.625	16.11	16.55	237328	{}	\N	2020-10-12 16:20:58.119849
74461	TOWN	2020-09-29	16.37	16.37	15.91	16.26	139780	{}	\N	2020-10-12 16:20:58.131811
74462	TOWN	2020-09-30	16.36	16.655	16.18	16.4	184373	{}	\N	2020-10-12 16:20:58.139224
74463	TOWN	2020-10-01	16.4	16.57	16.11	16.54	225616	{}	\N	2020-10-12 16:20:58.144831
74464	TOWN	2020-10-02	16.18	17.3	16.1	17.21	258985	{}	\N	2020-10-12 16:20:58.149995
74465	TOWN	2020-10-05	17.4	17.69	17.29	17.66	240187	{}	\N	2020-10-12 16:20:58.155112
74466	TOWN	2020-10-06	17.91	18.44	17.74	17.81	351883	{}	\N	2020-10-12 16:20:58.159691
74467	TOWN	2020-10-07	18.02	18.5	17.95	18.19	211469	{}	\N	2020-10-12 16:20:58.163821
74468	TOWN	2020-10-08	18.28	18.7	18.08	18.55	243174	{}	\N	2020-10-12 16:20:58.168
74469	TOWN	2020-10-09	18.81	19.38	18.2	18.27	241394	{}	\N	2020-10-12 16:20:58.171965
74470	TOWN	2020-10-12	18.2	18.69	18.05	18.61	613049	{}	\N	2020-10-12 16:20:58.175966
74471	TPVY	2020-09-28	25.36	25.36	25.04	25.195	2756	{}	\N	2020-10-12 16:20:59.506327
74472	TPVY	2020-09-29	25.1685	25.2	25.09	25.2	12440	{}	\N	2020-10-12 16:20:59.513104
74473	TPVY	2020-09-30	24.99	25.065	24.9	25.01	15540	{}	\N	2020-10-12 16:20:59.519446
74474	TPVY	2020-10-01	25.0947	25.0947	24.75	24.9716	5548	{}	\N	2020-10-12 16:20:59.524534
74475	TPVY	2020-10-02	24.975	25	24.95	24.95	2356	{}	\N	2020-10-12 16:20:59.528952
74476	TPVY	2020-10-05	24.9915	25	24.85	24.9001	6746	{}	\N	2020-10-12 16:20:59.53355
74477	TPVY	2020-10-06	24.99	25.02	24.949	24.97	18954	{}	\N	2020-10-12 16:20:59.537685
74478	TPVY	2020-10-07	25.05	25.05	24.93	24.9595	17020	{}	\N	2020-10-12 16:20:59.541715
74479	TPVY	2020-10-08	25.01	25.01	24.92	25	27800	{}	\N	2020-10-12 16:20:59.545872
74480	TPVY	2020-10-09	24.976	25	24.9	24.97	5716	{}	\N	2020-10-12 16:20:59.549976
74481	TRIP	2020-09-28	19.6	19.95	19.53	19.82	1674095	{}	\N	2020-10-12 16:21:00.928559
74482	TRIP	2020-09-29	19.82	20.03	19.315	19.58	1916981	{}	\N	2020-10-12 16:21:00.94014
74483	TRIP	2020-09-30	19.75	20.18	19.36	19.59	2074053	{}	\N	2020-10-12 16:21:00.9476
74484	TRIP	2020-10-01	19.69	20.09	19.42	19.87	1354133	{}	\N	2020-10-12 16:21:00.953953
74485	TRIP	2020-10-02	19.22	19.81	18.86	19.35	2117810	{}	\N	2020-10-12 16:21:00.959874
74486	TRIP	2020-10-05	19.56	19.67	18.895	19.3	2024320	{}	\N	2020-10-12 16:21:00.965468
74487	TRIP	2020-10-06	19.37	20.76	19.3	19.7	2789669	{}	\N	2020-10-12 16:21:00.970126
74488	TRIP	2020-10-07	19.91	20.36	19.69	20.17	1633270	{}	\N	2020-10-12 16:21:00.974767
74489	TRIP	2020-10-08	20.6	20.65	19.99	20.22	1384739	{}	\N	2020-10-12 16:21:00.979599
74490	TRIP	2020-10-09	20.12	20.77	19.95	20.37	1579692	{}	\N	2020-10-12 16:21:00.98433
74491	TRIP	2020-10-12	20.55	20.87	20.27	20.82	472726	{}	\N	2020-10-12 16:21:00.988924
74492	TROX	2020-09-28	7.97	8.185	7.87	7.94	2203186	{}	\N	2020-10-12 16:21:02.584689
74493	TROX	2020-09-29	7.88	8.11	7.725	7.81	992937	{}	\N	2020-10-12 16:21:02.597322
74494	TROX	2020-09-30	7.77	8.09	7.77	7.87	1505407	{}	\N	2020-10-12 16:21:02.605352
74495	TROX	2020-10-01	7.93	7.998	7.67	7.91	1506204	{}	\N	2020-10-12 16:21:02.611832
74496	TROX	2020-10-02	7.7	8.3087	7.7	8.25	1740372	{}	\N	2020-10-12 16:21:02.618535
74497	TROX	2020-10-05	8.4	8.51	8.35	8.43	1469710	{}	\N	2020-10-12 16:21:02.623641
74498	TROX	2020-10-06	8.6	8.68	8.36	8.39	1591060	{}	\N	2020-10-12 16:21:02.62862
74499	TROX	2020-10-07	8.63	9.005	8.535	8.93	2191146	{}	\N	2020-10-12 16:21:02.633808
74500	TROX	2020-10-08	9.07	9.24	8.9969	9.17	2123803	{}	\N	2020-10-12 16:21:02.638792
74501	TROX	2020-10-09	9.28	9.4	9.02	9.06	1153061	{}	\N	2020-10-12 16:21:02.643769
74502	TROX	2020-10-12	9.05	9.24	8.99	9.2	330858	{}	\N	2020-10-12 16:21:02.649099
74503	TRUE	2020-09-28	4.97	4.98	4.78	4.85	523627	{}	\N	2020-10-12 16:21:04.224864
74504	TRUE	2020-09-29	4.86	5.03	4.85	4.98	860921	{}	\N	2020-10-12 16:21:04.237161
74505	TRUE	2020-09-30	5.01	5.32	4.97	5	1119081	{}	\N	2020-10-12 16:21:04.245447
74506	TRUE	2020-10-01	5	5.48	5	5.14	1149287	{}	\N	2020-10-12 16:21:04.252691
74507	TRUE	2020-10-02	5.01	5.175	4.9	5.08	894552	{}	\N	2020-10-12 16:21:04.258808
74508	TRUE	2020-10-05	5.1	5.25	5.08	5.17	681668	{}	\N	2020-10-12 16:21:04.264368
74509	TRUE	2020-10-06	5.22	5.33	4.99	5	908860	{}	\N	2020-10-12 16:21:04.269197
74510	TRUE	2020-10-07	5.03	5.2	5.03	5.17	553202	{}	\N	2020-10-12 16:21:04.275584
74511	TRUE	2020-10-08	5.26	5.59	5.26	5.4	1071280	{}	\N	2020-10-12 16:21:04.281006
74512	TRUE	2020-10-09	5.43	5.91	5.17	5.23	2923089	{}	\N	2020-10-12 16:21:04.286062
74513	TRUE	2020-10-12	5.3	5.37	5.1711	5.1719	254317	{}	\N	2020-10-12 16:21:04.291286
74514	TSCO	2020-09-28	140.6	142.03	139.43	142.02	937602	{}	\N	2020-10-12 16:21:06.233855
74515	TSCO	2020-09-29	141.9	142.915	140.82	140.98	919422	{}	\N	2020-10-12 16:21:06.247681
74516	TSCO	2020-09-30	141.63	144.83	141.08	143.34	1500241	{}	\N	2020-10-12 16:21:06.255689
74517	TSCO	2020-10-01	143.71	144.81	141.52	142.05	1343041	{}	\N	2020-10-12 16:21:06.261903
74518	TSCO	2020-10-02	140.63	142.335	140.12	140.62	889074	{}	\N	2020-10-12 16:21:06.267397
74519	TSCO	2020-10-05	141.5	144.52	141.18	144.4	964189	{}	\N	2020-10-12 16:21:06.272471
74520	TSCO	2020-10-06	144.31	144.97	141.11	141.77	950399	{}	\N	2020-10-12 16:21:06.276952
74521	TSCO	2020-10-07	142.22	147.395	142.22	145.79	1282906	{}	\N	2020-10-12 16:21:06.28114
74522	TSCO	2020-10-08	146.13	149.6999	145.2	149.22	1027447	{}	\N	2020-10-12 16:21:06.286031
74523	TSCO	2020-10-09	150.09	152.19	148.6619	149.93	895566	{}	\N	2020-10-12 16:21:06.290104
74524	TSCO	2020-10-12	151.5	152.9	150.22	151.26	625938	{}	\N	2020-10-12 16:21:06.294533
74525	TSN	2020-09-28	60.05	60.82	59.5889	59.76	2361853	{}	\N	2020-10-12 16:21:07.676064
74526	TSN	2020-09-29	59.75	59.97	58.5901	58.96	2313297	{}	\N	2020-10-12 16:21:07.689043
74527	TSN	2020-09-30	58.99	60.09	58.99	59.48	3125574	{}	\N	2020-10-12 16:21:07.703758
74528	TSN	2020-10-01	59.41	59.82	58.84	59.22	2529871	{}	\N	2020-10-12 16:21:07.708879
74529	TSN	2020-10-02	58.68	60.545	58.59	59.31	4469232	{}	\N	2020-10-12 16:21:07.713661
74530	TSN	2020-10-05	59.99	60.3	58.72	59	7865509	{}	\N	2020-10-12 16:21:07.719093
74531	TSN	2020-10-06	59.23	59.89	58.18	58.61	3961132	{}	\N	2020-10-12 16:21:07.724121
74532	TSN	2020-10-07	59.1	59.25	58.1538	58.77	3340182	{}	\N	2020-10-12 16:21:07.729026
74533	TSN	2020-10-08	59	59.535	58.69	58.99	4181071	{}	\N	2020-10-12 16:21:07.733974
74534	TSN	2020-10-09	59.21	59.48	57.92	57.98	4878523	{}	\N	2020-10-12 16:21:07.738938
74535	TSN	2020-10-12	58.14	59.61	57.19	59.57	1877726	{}	\N	2020-10-12 16:21:07.743881
74536	TTAI	2020-09-28	27.42	27.4396	27.42	27.4396	363	{}	\N	2020-10-12 16:21:09.543362
74537	TTAI	2020-09-29	27.5475	27.5475	27.5475	27.5475	103	{}	\N	2020-10-12 16:21:09.55518
74538	TTAI	2020-09-30	27.6198	27.6198	27.6198	27.6198	100	{}	\N	2020-10-12 16:21:09.566483
74539	TTAI	2020-10-01	27.755	27.755	27.755	27.755	101	{}	\N	2020-10-12 16:21:09.579945
74540	TTAI	2020-10-02	27.6684	27.6684	27.6684	27.6684	101	{}	\N	2020-10-12 16:21:09.587555
74541	TTAI	2020-10-05	27.97	28.0992	27.97	28.0992	600	{}	\N	2020-10-12 16:21:09.598998
74542	TTAI	2020-10-06	27.7564	27.7564	27.7564	27.7564	102	{}	\N	2020-10-12 16:21:09.614814
74543	TTAI	2020-10-07	27.97	28.0924	27.97	28.0924	209	{}	\N	2020-10-12 16:21:09.626644
74544	TTAI	2020-10-08	28.25	28.2891	28.25	28.2891	300	{}	\N	2020-10-12 16:21:09.633542
74545	TTAI	2020-10-09	28.4727	28.4727	28.4727	28.4727	103	{}	\N	2020-10-12 16:21:09.64114
74546	TTNP	2020-09-28	0.1785	0.1785	0.1675	0.1695	4431055	{}	\N	2020-10-12 16:21:11.946162
74547	TTNP	2020-09-29	0.172	0.172	0.1645	0.167	3750406	{}	\N	2020-10-12 16:21:11.959586
74548	TTNP	2020-09-30	0.17	0.17	0.162	0.1654	3328084	{}	\N	2020-10-12 16:21:11.977119
74549	TTNP	2020-10-01	0.167	0.169	0.16	0.1645	3762378	{}	\N	2020-10-12 16:21:11.988749
74550	TTNP	2020-10-02	0.161	0.171	0.1523	0.1594	3789084	{}	\N	2020-10-12 16:21:12.000883
74551	TTNP	2020-10-05	0.16	0.1601	0.155	0.1554	4956234	{}	\N	2020-10-12 16:21:12.011084
74552	TTNP	2020-10-06	0.1569	0.1593	0.153	0.1559	4828216	{}	\N	2020-10-12 16:21:12.018619
74553	TTNP	2020-10-07	0.1573	0.163	0.155	0.1573	6514586	{}	\N	2020-10-12 16:21:12.027155
74554	TTNP	2020-10-08	0.1581	0.15845	0.1537	0.1541	3882555	{}	\N	2020-10-12 16:21:12.032797
74555	TTNP	2020-10-09	0.1589	0.159	0.155	0.1564	4355334	{}	\N	2020-10-12 16:21:12.038406
74556	TTNP	2020-10-12	0.1599	0.1599	0.153	0.1544	2867954	{}	\N	2020-10-12 16:21:12.043565
74557	TUR	2020-09-28	19.6	19.6	19.34	19.36	189054	{}	\N	2020-10-12 16:21:12.992405
74558	TUR	2020-09-29	19.4	19.55	19.121	19.35	155862	{}	\N	2020-10-12 16:21:13.002798
74559	TUR	2020-09-30	19.67	19.98	19.63	19.76	432125	{}	\N	2020-10-12 16:21:13.013375
74560	TUR	2020-10-01	19.99	20.04	19.72	19.9	197641	{}	\N	2020-10-12 16:21:13.0206
74561	TUR	2020-10-02	19.74	19.91	19.68	19.75	274505	{}	\N	2020-10-12 16:21:13.025766
74562	TUR	2020-10-05	19.97	20.17	19.97	20.08	136620	{}	\N	2020-10-12 16:21:13.030231
74563	TUR	2020-10-06	20.1	20.16	19.7	19.79	241108	{}	\N	2020-10-12 16:21:13.034557
74564	TUR	2020-10-07	19.75	19.75	19.59	19.62	208293	{}	\N	2020-10-12 16:21:13.039182
74565	TUR	2020-10-08	19.36	19.57	19.32	19.5	282405	{}	\N	2020-10-12 16:21:13.044155
74566	TUR	2020-10-09	19.64	20.02	19.64	19.9	149668	{}	\N	2020-10-12 16:21:13.048868
74567	TUR	2020-10-12	20.05	20.08	19.9433	20.06	121645	{}	\N	2020-10-12 16:21:13.05339
74568	TMO	2020-09-28	426.51	431.12	424.04	428.39	1088360	{}	\N	2020-10-12 16:21:14.806909
74569	TMO	2020-09-29	430.77	436.3	430	434.34	1822692	{}	\N	2020-10-12 16:21:14.818827
74570	TMO	2020-09-30	436.6	444.8	435.07	441.52	3035529	{}	\N	2020-10-12 16:21:14.828481
74571	TMO	2020-10-01	445.97	452.685	436.11	438.2	2741795	{}	\N	2020-10-12 16:21:14.835041
74572	TMO	2020-10-02	434.21	441.8	433.01	435.06	1843078	{}	\N	2020-10-12 16:21:14.841014
74573	TMO	2020-10-05	440	451.75	440	450.67	2310671	{}	\N	2020-10-12 16:21:14.84674
74574	TMO	2020-10-06	451.86	452	445.2701	445.89	2364530	{}	\N	2020-10-12 16:21:14.851382
74575	TMO	2020-10-07	447.89	455.9535	447.89	452.36	1898427	{}	\N	2020-10-12 16:21:14.856138
74576	TMO	2020-10-08	454.76	454.76	447.17	453.54	1672902	{}	\N	2020-10-12 16:21:14.86172
74577	TMO	2020-10-09	457.39	465.4	455.29	464.7	2118146	{}	\N	2020-10-12 16:21:14.866641
74578	TMO	2020-10-12	465.78	471.21	462.51	469.8	843354	{}	\N	2020-10-12 16:21:14.871589
74579	TNC	2020-09-28	60.37	62.14	60.37	61.33	88719	{}	\N	2020-10-12 16:21:16.649469
74580	TNC	2020-09-29	61	61.5482	59.41	60.08	258276	{}	\N	2020-10-12 16:21:16.659612
74581	TNC	2020-09-30	59.89	61.255	59.82	60.36	212873	{}	\N	2020-10-12 16:21:16.666465
74582	TNC	2020-10-01	60.26	60.92	58.77	59.27	174386	{}	\N	2020-10-12 16:21:16.673072
74583	TNC	2020-10-02	58.31	59.86	57.991	59.1	241014	{}	\N	2020-10-12 16:21:16.679309
74584	TNC	2020-10-05	59.86	60.75	59.58	60.18	232986	{}	\N	2020-10-12 16:21:16.68454
74585	TNC	2020-10-06	60.18	61.49	59.98	60.04	367147	{}	\N	2020-10-12 16:21:16.689216
74586	TNC	2020-10-07	60.31	62.51	60.12	62.37	373078	{}	\N	2020-10-12 16:21:16.694171
74587	TNC	2020-10-08	63.12	63.6	62.25	62.64	98241	{}	\N	2020-10-12 16:21:16.698835
74588	TNC	2020-10-09	63.43	63.83	62.48	63.54	123441	{}	\N	2020-10-12 16:21:16.70394
74589	TNC	2020-10-12	63.58	64.49	63.46	64.49	27709	{}	\N	2020-10-12 16:21:16.709027
74590	TPC	2020-09-28	11.48	12.07	11.48	11.71	312156	{}	\N	2020-10-12 16:21:19.771715
74591	TPC	2020-09-29	11.65	11.65	11.05	11.33	688570	{}	\N	2020-10-12 16:21:19.782796
74592	TPC	2020-09-30	11.42	11.8	11.07	11.13	1112130	{}	\N	2020-10-12 16:21:19.789839
74593	TPC	2020-10-01	11.21	11.3332	10.79	11.28	702357	{}	\N	2020-10-12 16:21:19.795936
74594	TPC	2020-10-02	11.04	12.11	11.02	12.05	1114019	{}	\N	2020-10-12 16:21:19.804437
74595	TPC	2020-10-05	12.25	12.74	12.25	12.66	504528	{}	\N	2020-10-12 16:21:19.808994
74596	TPC	2020-10-06	12.69	13.26	12.64	12.81	975077	{}	\N	2020-10-12 16:21:19.813761
74597	TPC	2020-10-07	12.86	13.88	12.82	13.61	1078066	{}	\N	2020-10-12 16:21:19.819312
74598	TPC	2020-10-08	13.83	14.56	13.6	14.53	663381	{}	\N	2020-10-12 16:21:19.826728
74599	TPC	2020-10-09	14.6	14.69	14.1564	14.39	536944	{}	\N	2020-10-12 16:21:19.832801
74600	TPC	2020-10-12	14.4	14.86	14.22	14.64	225658	{}	\N	2020-10-12 16:21:19.838376
74601	TPVG	2020-09-28	11.4	11.66	11.39	11.45	123926	{}	\N	2020-10-12 16:21:22.935621
74602	TPVG	2020-09-29	11.46	11.5044	11.22	11.24	139962	{}	\N	2020-10-12 16:21:22.947435
74603	TPVG	2020-09-30	11.22	11.46	11.02	11.04	252578	{}	\N	2020-10-12 16:21:22.955284
74604	TPVG	2020-10-01	11.29	11.5	11.11	11.43	444585	{}	\N	2020-10-12 16:21:22.961615
74605	TPVG	2020-10-02	11.43	11.84	11.33	11.83	356570	{}	\N	2020-10-12 16:21:22.967287
74606	TPVG	2020-10-05	11.98	12	11.72	11.81	346452	{}	\N	2020-10-12 16:21:22.971825
74607	TPVG	2020-10-06	11.8	11.97	11.59	11.61	243465	{}	\N	2020-10-12 16:21:22.976124
74608	TPVG	2020-10-07	11.69	11.8	11.48	11.54	189344	{}	\N	2020-10-12 16:21:22.980485
74609	TPVG	2020-10-08	11.7	11.7	11.5256	11.62	175238	{}	\N	2020-10-12 16:21:22.984987
74610	TPVG	2020-10-09	11.65	11.65	11.3499	11.44	205422	{}	\N	2020-10-12 16:21:22.989387
74611	TPVG	2020-10-12	11.47	11.4723	11.05	11.101	199878	{}	\N	2020-10-12 16:21:22.994227
74612	TREC	2020-09-28	6.09	6.265	6.06	6.21	26431	{}	\N	2020-10-12 16:21:25.106395
74613	TREC	2020-09-29	6.2	6.2	5.97	6.19	82635	{}	\N	2020-10-12 16:21:25.116349
74614	TREC	2020-09-30	6.2	6.28	6.1	6.14	123042	{}	\N	2020-10-12 16:21:25.123591
74615	TREC	2020-10-01	6.14	6.7	6.07	6.62	276330	{}	\N	2020-10-12 16:21:25.129596
74616	TREC	2020-10-02	6.47	6.61	6.38	6.59	54294	{}	\N	2020-10-12 16:21:25.135308
74617	TREC	2020-10-05	6.57	6.7	6.565	6.61	70254	{}	\N	2020-10-12 16:21:25.14066
74618	TREC	2020-10-06	6.72	6.85	6.65	6.65	93324	{}	\N	2020-10-12 16:21:25.145517
74619	TREC	2020-10-07	6.74	6.86	6.56	6.84	68530	{}	\N	2020-10-12 16:21:25.150585
74620	TREC	2020-10-08	6.95	6.95	6.74	6.93	49489	{}	\N	2020-10-12 16:21:25.15569
74621	TREC	2020-10-09	6.95	6.9527	6.85	6.91	31986	{}	\N	2020-10-12 16:21:25.16026
74622	TREC	2020-10-12	6.54	7.23	6.52	6.87	419432	{}	\N	2020-10-12 16:21:25.165524
74623	TRMD	2020-09-28	6.825	6.8262	6.52	6.8	20947	{}	\N	2020-10-12 16:21:26.097275
74624	TRMD	2020-09-29	6.77	6.825	6.55	6.75	13301	{}	\N	2020-10-12 16:21:26.108333
74625	TRMD	2020-09-30	6.79	6.9	6.765	6.82	10692	{}	\N	2020-10-12 16:21:26.11703
74626	TRMD	2020-10-01	6.83	6.8364	6.5941	6.8	12868	{}	\N	2020-10-12 16:21:26.123354
74627	TRMD	2020-10-02	6.76	6.85	6.7061	6.75	10919	{}	\N	2020-10-12 16:21:26.129211
74628	TRMD	2020-10-05	6.98	7.07	6.98	7	11555	{}	\N	2020-10-12 16:21:26.134995
74629	TRMD	2020-10-06	7.11	7.2237	7.01	7.01	11622	{}	\N	2020-10-12 16:21:26.140482
74630	TRMD	2020-10-07	7.05	7.31	7.0183	7.3	16903	{}	\N	2020-10-12 16:21:26.145929
74631	TRMD	2020-10-08	7.21	7.38	7.07	7.25	20364	{}	\N	2020-10-12 16:21:26.151827
74632	TRMD	2020-10-09	7.36	7.36	7.1106	7.29	12018	{}	\N	2020-10-12 16:21:26.157082
74633	TRMD	2020-10-12	7.29	7.29	7.1101	7.1101	6407	{}	\N	2020-10-12 16:21:26.162336
74634	TRUP	2020-09-28	73.85	74	68.065	72.5	463422	{}	\N	2020-10-12 16:21:29.016183
74635	TRUP	2020-09-29	73.25	79.2639	72.419	78.68	644211	{}	\N	2020-10-12 16:21:29.02819
74636	TRUP	2020-09-30	78.5	80.1477	77.36	78.9	758805	{}	\N	2020-10-12 16:21:29.034411
74637	TRUP	2020-10-01	79.69	81.26	78.74	81.16	346346	{}	\N	2020-10-12 16:21:29.040566
74638	TRUP	2020-10-02	79.67	84.0649	79.1778	83.37	541855	{}	\N	2020-10-12 16:21:29.045146
74639	TRUP	2020-10-05	85.31	88.1	84.33	87.24	503986	{}	\N	2020-10-12 16:21:29.049925
74640	TRUP	2020-10-06	87.6	89.96	85.05	86.37	559357	{}	\N	2020-10-12 16:21:29.05467
74641	TRUP	2020-10-07	87.72	88.99	86.0375	86.87	479551	{}	\N	2020-10-12 16:21:29.059265
74642	TRUP	2020-10-08	87.84	89	85.6	87.2	650180	{}	\N	2020-10-12 16:21:29.064665
74643	TRUP	2020-10-09	88.13	90.9568	88.13	89.98	475805	{}	\N	2020-10-12 16:21:29.069192
74644	TRUP	2020-10-12	91.17	92	89.67	90.03	101885	{}	\N	2020-10-12 16:21:29.074403
74645	TSM	2020-09-28	80.31	80.37	79.47	79.77	5982739	{}	\N	2020-10-12 16:21:30.693204
74646	TSM	2020-09-29	79.7	81.24	79.31	80.51	12273835	{}	\N	2020-10-12 16:21:30.706116
74647	TSM	2020-09-30	80.59	81.935	80.57	81.07	11862358	{}	\N	2020-10-12 16:21:30.71431
74648	TSM	2020-10-01	82.06	82.72	81.74	82.62	12304446	{}	\N	2020-10-12 16:21:30.720968
74649	TSM	2020-10-02	80.71	82.195	80.61	80.8	11694642	{}	\N	2020-10-12 16:21:30.727191
74650	TSM	2020-10-05	81.55	84.63	81.55	84.54	20452646	{}	\N	2020-10-12 16:21:30.732404
74651	TSM	2020-10-06	83.77	84.45	82.765	83.12	15657781	{}	\N	2020-10-12 16:21:30.73789
74652	TSM	2020-10-07	84.78	87	84.78	86.92	22007705	{}	\N	2020-10-12 16:21:30.743409
74653	TSM	2020-10-08	88.86	88.86	86.52	87.8	22278405	{}	\N	2020-10-12 16:21:30.748713
74654	TSM	2020-10-09	88.34	88.8	87.64	88.78	15448453	{}	\N	2020-10-12 16:21:30.754142
74655	TSM	2020-10-12	89.12	90.91	88.68	90.39	12234676	{}	\N	2020-10-12 16:21:30.760002
74656	TTD	2020-09-28	480.04	488.65	477	487.63	566908	{}	\N	2020-10-12 16:21:32.109704
74657	TTD	2020-09-29	488.52	506.75	488.38	498.3	1215406	{}	\N	2020-10-12 16:21:32.119175
74658	TTD	2020-09-30	496.61	520.385	495.01	518.78	1070576	{}	\N	2020-10-12 16:21:32.126184
74659	TTD	2020-10-01	529.99	556.7447	526.01	555.17	1646135	{}	\N	2020-10-12 16:21:32.132668
74660	TTD	2020-10-02	535.9502	569.93	532.34	556.48	1320905	{}	\N	2020-10-12 16:21:32.137825
74661	TTD	2020-10-05	566.63	575.49	562.27	573.91	769285	{}	\N	2020-10-12 16:21:32.143242
74662	TTD	2020-10-06	573	597.5	572.75	585.25	1133944	{}	\N	2020-10-12 16:21:32.147671
74663	TTD	2020-10-07	594.99	608.3999	586.29	593.43	1196836	{}	\N	2020-10-12 16:21:32.151621
74664	TTD	2020-10-08	603.49	607.7	592.17	605.55	699547	{}	\N	2020-10-12 16:21:32.156086
74665	TTD	2020-10-09	604.01	620.8899	602.1	613.11	895244	{}	\N	2020-10-12 16:21:32.16016
74666	TTD	2020-10-12	621.99	624.9	610.04	616.6	378347	{}	\N	2020-10-12 16:21:32.16433
74667	TUSA	2020-09-30	33.1629	33.1629	33.1629	33.1629	310	{}	\N	2020-10-12 16:21:34.520824
74668	TUSA	2020-10-01	33.38	33.5484	33.38	33.5484	383	{}	\N	2020-10-12 16:21:34.530008
74669	TUSA	2020-10-05	34.35	34.35	34.35	34.35	277	{}	\N	2020-10-12 16:21:34.536304
74670	TUSA	2020-10-06	34.3284	34.3284	34.1978	34.1978	570	{}	\N	2020-10-12 16:21:34.543243
74671	TUSA	2020-10-08	35.1639	35.2169	35.1639	35.2169	461	{}	\N	2020-10-12 16:21:34.548854
74672	TVTY	2020-09-28	13.86	14.5	13.86	14.03	422922	{}	\N	2020-10-12 16:21:35.723218
74673	TVTY	2020-09-29	14.01	14.33	13.74	14	501735	{}	\N	2020-10-12 16:21:35.731805
74674	TVTY	2020-09-30	14.09	14.43	13.92	14.02	977916	{}	\N	2020-10-12 16:21:35.737475
74675	TVTY	2020-10-01	14.26	14.62	14.14	14.61	427971	{}	\N	2020-10-12 16:21:35.742132
74676	TVTY	2020-10-02	14.17	14.92	13.6	14.64	283798	{}	\N	2020-10-12 16:21:35.747558
74677	TVTY	2020-10-05	14.67	15.37	14.67	15.3	397669	{}	\N	2020-10-12 16:21:35.752234
74678	TVTY	2020-10-06	15.47	15.57	14.71	14.87	619102	{}	\N	2020-10-12 16:21:35.756338
74679	TVTY	2020-10-07	15.28	15.74	15.13	15.58	727758	{}	\N	2020-10-12 16:21:35.760723
74680	TVTY	2020-10-08	15.77	15.9	15.37	15.48	225820	{}	\N	2020-10-12 16:21:35.764767
74681	TVTY	2020-10-09	15.62	15.65	15.15	15.3	239870	{}	\N	2020-10-12 16:21:35.768841
74682	TVTY	2020-10-12	15.29	15.595	15.02	15.53	88336	{}	\N	2020-10-12 16:21:35.773486
74683	TOL	2020-09-28	48.32	49.695	48	49.35	1569499	{}	\N	2020-10-12 16:21:38.706625
74684	TOL	2020-09-29	49.5	49.56	46.99	47.12	5504469	{}	\N	2020-10-12 16:21:38.723152
74685	TOL	2020-09-30	47.53	49.21	47.51	48.66	4554160	{}	\N	2020-10-12 16:21:38.73135
74686	TOL	2020-10-01	48.83	49.48	48.2	49.48	3861565	{}	\N	2020-10-12 16:21:38.737134
74687	TOL	2020-10-02	48.33	50.16	48.16	49.79	2658722	{}	\N	2020-10-12 16:21:38.742405
74688	TOL	2020-10-05	50.2	50.415	47.91	49.08	3935641	{}	\N	2020-10-12 16:21:38.747094
74689	TOL	2020-10-06	49.31	49.31	47.01	47.32	4298816	{}	\N	2020-10-12 16:21:38.751736
74690	TOL	2020-10-07	47.9	48.665	46.86	47.54	3000596	{}	\N	2020-10-12 16:21:38.756406
74691	TOL	2020-10-08	48.22	49.26	48.0184	49.23	3674217	{}	\N	2020-10-12 16:21:38.760967
74692	TOL	2020-10-09	49.5	50.35	49.37	49.83	1931189	{}	\N	2020-10-12 16:21:38.765932
74693	TOL	2020-10-12	50	50.1	49.2	49.9	476894	{}	\N	2020-10-12 16:21:38.770582
74694	TPRE	2020-09-28	6.87	7.07	6.87	6.97	225284	{}	\N	2020-10-12 16:21:40.371088
74695	TPRE	2020-09-29	6.94	6.96	6.8	6.91	448623	{}	\N	2020-10-12 16:21:40.378848
74696	TPRE	2020-09-30	6.92	7.01	6.83	6.95	1227840	{}	\N	2020-10-12 16:21:40.385077
74697	TPRE	2020-10-01	6.94	7.06	6.895	6.99	556984	{}	\N	2020-10-12 16:21:40.391058
74698	TPRE	2020-10-02	6.83	7.23	6.77	7.16	844824	{}	\N	2020-10-12 16:21:40.397674
74699	TPRE	2020-10-05	7.24	7.39	7.24	7.36	551788	{}	\N	2020-10-12 16:21:40.402386
74700	TPRE	2020-10-06	7.49	7.59	7.29	7.31	858938	{}	\N	2020-10-12 16:21:40.411136
74701	TPRE	2020-10-07	7.36	7.49	7.22	7.39	924897	{}	\N	2020-10-12 16:21:40.416084
74702	TPRE	2020-10-08	7.48	7.78	7.44	7.74	794238	{}	\N	2020-10-12 16:21:40.420724
74703	TPRE	2020-10-09	7.8	7.89	7.57	7.69	663875	{}	\N	2020-10-12 16:21:40.424827
74704	TPRE	2020-10-12	7.7	7.82	7.64	7.77	115451	{}	\N	2020-10-12 16:21:40.428726
74705	TRCH	2020-09-28	0.236	0.26	0.23	0.2568	942327	{}	\N	2020-10-12 16:21:41.523383
74706	TRCH	2020-09-29	0.260994	0.285	0.2536	0.2636	1299559	{}	\N	2020-10-12 16:21:41.535615
74707	TRCH	2020-09-30	0.2566	0.2687	0.2441	0.26	425808	{}	\N	2020-10-12 16:21:41.543784
74708	TRCH	2020-10-01	0.2522	0.2687	0.25	0.265	228132	{}	\N	2020-10-12 16:21:41.550227
74709	TRCH	2020-10-02	0.2687	0.2687	0.2517	0.264	1329741	{}	\N	2020-10-12 16:21:41.555607
74710	TRCH	2020-10-05	0.26	0.2782	0.26	0.2651	537957	{}	\N	2020-10-12 16:21:41.560136
74711	TRCH	2020-10-06	0.2942	0.2942	0.2549	0.2549	388540	{}	\N	2020-10-12 16:21:41.564672
74712	TRCH	2020-10-07	0.2559	0.2649	0.249	0.2567	253187	{}	\N	2020-10-12 16:21:41.569728
74713	TRCH	2020-10-08	0.268	0.269	0.2502	0.2638	577712	{}	\N	2020-10-12 16:21:41.574128
74714	TRCH	2020-10-09	0.2645	0.335	0.2645	0.325	9032071	{}	\N	2020-10-12 16:21:41.578374
74715	TRCH	2020-10-12	0.336	0.395	0.333	0.3751	4706699	{}	\N	2020-10-12 16:21:41.582948
74716	TRST	2020-09-28	5.21	5.38	5.19	5.33	474099	{}	\N	2020-10-12 16:21:42.972011
74717	TRST	2020-09-29	5.31	5.33	5.2	5.28	388648	{}	\N	2020-10-12 16:21:42.983043
74718	TRST	2020-09-30	5.3	5.38	5.22	5.22	421085	{}	\N	2020-10-12 16:21:42.99104
74719	TRST	2020-10-01	5.27	5.32	5.185	5.3	387446	{}	\N	2020-10-12 16:21:42.997075
74720	TRST	2020-10-02	5.2	5.43	5.2	5.37	396148	{}	\N	2020-10-12 16:21:43.003132
74721	TRST	2020-10-05	5.43	5.465	5.39	5.42	260287	{}	\N	2020-10-12 16:21:43.008198
74722	TRST	2020-10-06	5.49	5.73	5.45	5.58	825786	{}	\N	2020-10-12 16:21:43.013073
74723	TRST	2020-10-07	5.63	5.74	5.54	5.64	423122	{}	\N	2020-10-12 16:21:43.017988
74724	TRST	2020-10-08	5.69	5.7134	5.59	5.63	267561	{}	\N	2020-10-12 16:21:43.023055
74725	TRST	2020-10-09	5.68	5.72	5.6	5.64	295766	{}	\N	2020-10-12 16:21:43.02784
74726	TRST	2020-10-12	5.63	5.69	5.61	5.63	63103	{}	\N	2020-10-12 16:21:43.032131
74727	TRVG	2020-09-28	1.54	1.61	1.54	1.59	156744	{}	\N	2020-10-12 16:21:44.296548
74728	TRVG	2020-09-29	1.58	1.6	1.52	1.57	124551	{}	\N	2020-10-12 16:21:44.306163
74729	TRVG	2020-09-30	1.59	1.61	1.52	1.55	491618	{}	\N	2020-10-12 16:21:44.312858
74730	TRVG	2020-10-01	1.57	1.57	1.45	1.48	663575	{}	\N	2020-10-12 16:21:44.318399
74731	TRVG	2020-10-02	1.45	1.49	1.4	1.42	398084	{}	\N	2020-10-12 16:21:44.324575
74732	TRVG	2020-10-05	1.48	1.48	1.3701	1.41	588496	{}	\N	2020-10-12 16:21:44.329177
74733	TRVG	2020-10-06	1.41	1.475	1.35	1.4	516990	{}	\N	2020-10-12 16:21:44.333746
74734	TRVG	2020-10-07	1.4	1.45	1.3866	1.41	241308	{}	\N	2020-10-12 16:21:44.338715
74735	TRVG	2020-10-08	1.42	1.48	1.42	1.47	213769	{}	\N	2020-10-12 16:21:44.34328
74736	TRVG	2020-10-09	1.46	1.5	1.43	1.45	217001	{}	\N	2020-10-12 16:21:44.348657
74737	TRVG	2020-10-12	1.45	1.49	1.445	1.47	77150	{}	\N	2020-10-12 16:21:44.354174
74738	TSI	2020-09-28	5.74	5.745	5.68	5.69	153496	{}	\N	2020-10-12 16:21:45.295628
74739	TSI	2020-09-29	5.68	5.705	5.55	5.59	290550	{}	\N	2020-10-12 16:21:45.307043
74740	TSI	2020-09-30	5.58	5.67	5.58	5.62	236884	{}	\N	2020-10-12 16:21:45.314374
74741	TSI	2020-10-01	5.58	5.63	5.58	5.6	308500	{}	\N	2020-10-12 16:21:45.320772
74742	TSI	2020-10-02	5.56	5.64	5.56	5.56	132166	{}	\N	2020-10-12 16:21:45.326211
74743	TSI	2020-10-05	5.56	5.6	5.48	5.56	298732	{}	\N	2020-10-12 16:21:45.331383
74744	TSI	2020-10-06	5.53	5.56	5.48	5.52	186735	{}	\N	2020-10-12 16:21:45.336239
74745	TSI	2020-10-07	5.52	5.55	5.44	5.49	422872	{}	\N	2020-10-12 16:21:45.34123
74746	TSI	2020-10-08	5.55	5.58	5.5291	5.57	253106	{}	\N	2020-10-12 16:21:45.346136
74747	TSI	2020-10-09	5.67	5.68	5.61	5.64	240914	{}	\N	2020-10-12 16:21:45.35067
74748	TSI	2020-10-12	5.67	5.75	5.6684	5.69	111916	{}	\N	2020-10-12 16:21:45.355872
74749	TTM	2020-09-28	9.03	9.09	8.95	8.98	1266349	{}	\N	2020-10-12 16:21:47.149044
74750	TTM	2020-09-29	8.92	8.935	8.8	8.87	1316389	{}	\N	2020-10-12 16:21:47.161929
74751	TTM	2020-09-30	9.07	9.23	9.03	9.09	2222655	{}	\N	2020-10-12 16:21:47.170211
74752	TTM	2020-10-01	9.13	9.16	8.9801	9.15	1431460	{}	\N	2020-10-12 16:21:47.176978
74753	TTM	2020-10-02	8.96	9.088	8.93	9.07	1587439	{}	\N	2020-10-12 16:21:47.182828
74754	TTM	2020-10-05	9.11	9.255	9.1	9.2	1422639	{}	\N	2020-10-12 16:21:47.188248
74755	TTM	2020-10-06	9.83	9.96	9.63	9.7	4329965	{}	\N	2020-10-12 16:21:47.193793
74756	TTM	2020-10-07	9.63	9.79	9.6	9.73	2613355	{}	\N	2020-10-12 16:21:47.198791
74757	TTM	2020-10-08	9.65	9.67	9.59	9.67	2257617	{}	\N	2020-10-12 16:21:47.203648
74758	TTM	2020-10-09	9.55	9.6	9.45	9.58	2010634	{}	\N	2020-10-12 16:21:47.209031
74759	TTM	2020-10-12	9.33	9.4	9.29	9.35	960610	{}	\N	2020-10-12 16:21:47.214869
74760	TVC	2020-09-28	26.47	26.9	26.3	26.75	17269	{}	\N	2020-10-12 16:21:49.264212
74761	TVC	2020-09-29	26.72	26.79	26.51	26.6694	23982	{}	\N	2020-10-12 16:21:49.274995
74762	TVC	2020-09-30	26.5216	27	26.39	26.96	145649	{}	\N	2020-10-12 16:21:49.282243
74763	TVC	2020-10-01	26.87	27.115	26.15	26.96	75030	{}	\N	2020-10-12 16:21:49.289062
74764	TVC	2020-10-02	26.96	27	26.8205	26.96	20126	{}	\N	2020-10-12 16:21:49.295203
74765	TVC	2020-10-05	26.9009	26.9601	26.64	26.8862	17186	{}	\N	2020-10-12 16:21:49.301033
74766	TVC	2020-10-06	26.8	27.1275	26.3837	27.03	39496	{}	\N	2020-10-12 16:21:49.306081
74767	TVC	2020-10-07	27.04	27.08	26.56	26.87	16658	{}	\N	2020-10-12 16:21:49.311244
74768	TVC	2020-10-08	26.98	27.25	26.73	27	17746	{}	\N	2020-10-12 16:21:49.315899
74769	TVC	2020-10-09	27.02	27.02	26.867	27	8054	{}	\N	2020-10-12 16:21:49.320625
74770	TVC	2020-10-12	26.92	26.99	26.92	26.99	7594	{}	\N	2020-10-12 16:21:49.325689
74771	TWTR	2020-09-28	44.4	44.42	43.61	44.15	11644322	{}	\N	2020-10-12 16:21:50.623286
74772	TWTR	2020-09-29	44.35	45.51	44.29	44.74	27131910	{}	\N	2020-10-12 16:21:50.635918
74773	TWTR	2020-09-30	44.62	45.09	43.95	44.5	25807940	{}	\N	2020-10-12 16:21:50.647006
74774	TWTR	2020-10-01	45.07	46.72	44.98	46.7	35886354	{}	\N	2020-10-12 16:21:50.654371
74775	TWTR	2020-10-02	45.54	46.885	45.43	46.12	25989850	{}	\N	2020-10-12 16:21:50.660124
74776	TWTR	2020-10-05	46.46	47.495	46.12	47.31	22351994	{}	\N	2020-10-12 16:21:50.666032
74777	TWTR	2020-10-06	47.27	47.87	45.03	45.6	30758001	{}	\N	2020-10-12 16:21:50.672182
74778	TWTR	2020-10-07	46.07	46.4	45.74	45.87	37442282	{}	\N	2020-10-12 16:21:50.67827
74779	TWTR	2020-10-08	46.44	46.62	45.78	46.01	22674162	{}	\N	2020-10-12 16:21:50.684253
74780	TWTR	2020-10-09	46.4	46.4073	45.66	45.9	16338236	{}	\N	2020-10-12 16:21:50.69167
74781	TWTR	2020-10-12	47.61	48.65	47.41	47.98	25701391	{}	\N	2020-10-12 16:21:50.697465
74782	TYHT	2020-09-28	2.93	2.93	2.5801	2.7945	40997	{}	\N	2020-10-12 16:21:52.282998
74783	TYHT	2020-09-29	2.61	2.76	2.61	2.66	27604	{}	\N	2020-10-12 16:21:52.293388
74784	TYHT	2020-09-30	2.62	2.7834	2.6102	2.68	39101	{}	\N	2020-10-12 16:21:52.30125
74785	TYHT	2020-10-01	2.7	2.7	2.45	2.6	34056	{}	\N	2020-10-12 16:21:52.306473
74786	TYHT	2020-10-02	2.51	2.73	2.51	2.71	9488	{}	\N	2020-10-12 16:21:52.311436
74787	TYHT	2020-10-05	2.6301	2.7	2.6	2.61	9630	{}	\N	2020-10-12 16:21:52.316899
74788	TYHT	2020-10-06	2.73	2.74	2.6	2.66	12811	{}	\N	2020-10-12 16:21:52.322051
74789	TYHT	2020-10-07	2.6371	2.78	2.5801	2.69	12664	{}	\N	2020-10-12 16:21:52.32727
74790	TYHT	2020-10-08	2.7142	2.905	2.69	2.87	27859	{}	\N	2020-10-12 16:21:52.332264
74791	TYHT	2020-10-09	2.885	2.9799	2.82	2.83	12237	{}	\N	2020-10-12 16:21:52.337783
74792	TYHT	2020-10-12	2.91	3.23	2.91	3.05	131849	{}	\N	2020-10-12 16:21:52.342657
74793	UAG	2020-09-28	15.385	15.385	15.385	15.385	0	{}	\N	2020-10-12 16:21:53.781975
74794	UAG	2020-09-29	16	16	15.4783	15.4783	157	{}	\N	2020-10-12 16:21:53.792032
74795	UAG	2020-09-30	15.8091	15.8091	15.8091	15.8091	2	{}	\N	2020-10-12 16:21:53.800783
74796	UAG	2020-10-01	15.8774	15.8774	15.8774	15.8774	78	{}	\N	2020-10-12 16:21:53.807088
74797	UAG	2020-10-02	15.8482	15.8482	15.8482	15.8482	450	{}	\N	2020-10-12 16:21:53.812301
74798	UAG	2020-10-05	15.9	15.9	15.9	15.9	9	{}	\N	2020-10-12 16:21:53.817678
74799	UAG	2020-10-06	16.176	16.176	16.176	16.176	50	{}	\N	2020-10-12 16:21:53.82273
74800	UAG	2020-10-07	16.4009	16.4295	16.2749	16.2749	326	{}	\N	2020-10-12 16:21:53.827673
74801	UAG	2020-10-08	16.2964	16.2964	16.2964	16.2964	1	{}	\N	2020-10-12 16:21:53.832986
74802	UAG	2020-10-09	16.3791	16.3791	16.3791	16.3791	1	{}	\N	2020-10-12 16:21:53.838259
74803	TOK	2020-09-28	74.28	74.41	74.28	74.34	1505	{}	\N	2020-10-12 16:21:54.814814
74804	TOK	2020-09-29	73.9901	74.2157	73.9901	74.2157	579	{}	\N	2020-10-12 16:21:54.826157
74805	TOK	2020-09-30	74.13	74.59	74.13	74.59	1327	{}	\N	2020-10-12 16:21:54.833755
74806	TOK	2020-10-01	75	75.04	75	75.04	1101	{}	\N	2020-10-12 16:21:54.840986
74807	TOK	2020-10-02	73.96	74.641	73.96	74.505	11599	{}	\N	2020-10-12 16:21:54.846924
74808	TOK	2020-10-05	75.26	75.5974	75.26	75.5974	577	{}	\N	2020-10-12 16:21:54.85236
74809	TOK	2020-10-06	75.83	75.83	74.8414	74.8414	599	{}	\N	2020-10-12 16:21:54.857694
74810	TOK	2020-10-07	75.33	75.78	75.32	75.69	6026	{}	\N	2020-10-12 16:21:54.862374
74811	TOK	2020-10-08	76.13	76.4606	76.13	76.4606	870	{}	\N	2020-10-12 16:21:54.867457
74812	TOK	2020-10-09	76.91	76.96	76.86	76.94	3732	{}	\N	2020-10-12 16:21:54.872233
74813	TOK	2020-10-12	77.5	78	77.49	77.895	961	{}	\N	2020-10-12 16:21:54.876941
74814	TOTL	2020-09-28	49.41	49.41	49.3	49.3	321264	{}	\N	2020-10-12 16:21:55.866498
74815	TOTL	2020-09-29	49.38	49.4	49.33	49.35	264089	{}	\N	2020-10-12 16:21:55.87758
74816	TOTL	2020-09-30	49.4	49.45	49.31	49.31	431778	{}	\N	2020-10-12 16:21:55.885406
74817	TOTL	2020-10-01	49.29	49.37	49.2336	49.35	394937	{}	\N	2020-10-12 16:21:55.891476
74818	TOTL	2020-10-02	49.37	49.3955	49.2508	49.27	435265	{}	\N	2020-10-12 16:21:55.897483
74819	TOTL	2020-10-05	49.26	49.31	49.2195	49.27	546827	{}	\N	2020-10-12 16:21:55.902414
74820	TOTL	2020-10-06	49.2	49.4	49.19	49.32	673318	{}	\N	2020-10-12 16:21:55.907354
74821	TOTL	2020-10-07	49.34	49.346	49.2401	49.34	255844	{}	\N	2020-10-12 16:21:55.911831
74822	TOTL	2020-10-08	49.25	49.32	49.22	49.3	230526	{}	\N	2020-10-12 16:21:55.916874
74823	TOTL	2020-10-09	49.27	49.34	49.24	49.31	239713	{}	\N	2020-10-12 16:21:55.921893
74824	TOTL	2020-10-12	49.26	49.35	49.26	49.3213	88741	{}	\N	2020-10-12 16:21:55.926766
74825	TPIC	2020-09-28	27.07	28.035	27.0101	27.76	290062	{}	\N	2020-10-12 16:21:56.936827
74826	TPIC	2020-09-29	27.78	28.38	27.73	28.15	445834	{}	\N	2020-10-12 16:21:56.948013
74827	TPIC	2020-09-30	28.38	29.41	28.22	28.96	438769	{}	\N	2020-10-12 16:21:56.956147
74828	TPIC	2020-10-01	29.3	30.36	29.105	29.68	558301	{}	\N	2020-10-12 16:21:56.962705
74829	TPIC	2020-10-02	29.35	30.06	29.01	29.18	667606	{}	\N	2020-10-12 16:21:56.968711
74830	TPIC	2020-10-05	29.58	31.09	29.31	31.08	500221	{}	\N	2020-10-12 16:21:56.973828
74831	TPIC	2020-10-06	31.2	33	30.4	31.36	774120	{}	\N	2020-10-12 16:21:56.979049
74832	TPIC	2020-10-07	32.04	35.49	31.74	35.06	1257597	{}	\N	2020-10-12 16:21:56.98471
74833	TPIC	2020-10-08	35.54	36.29	33.31	34.06	818080	{}	\N	2020-10-12 16:21:56.990119
74834	TPIC	2020-10-09	34.79	35.63	34.16	35.53	680090	{}	\N	2020-10-12 16:21:56.995429
74835	TPIC	2020-10-12	36.59	36.59	34.17	34.71	321263	{}	\N	2020-10-12 16:21:57.001018
74836	TPZ	2020-09-28	8.44	8.5	8.36	8.47	68233	{}	\N	2020-10-12 16:21:59.263493
74837	TPZ	2020-09-29	8.4375	8.44	8.36	8.44	60098	{}	\N	2020-10-12 16:21:59.273821
74838	TPZ	2020-09-30	8.49	8.52	8.4234	8.52	76248	{}	\N	2020-10-12 16:21:59.280797
74839	TPZ	2020-10-01	8.49	8.49	8.4	8.43	41136	{}	\N	2020-10-12 16:21:59.288
74840	TPZ	2020-10-02	8.31	8.39	8.11	8.35	70192	{}	\N	2020-10-12 16:21:59.294144
74841	TPZ	2020-10-05	8.36	8.65	8.36	8.53	121268	{}	\N	2020-10-12 16:21:59.299319
74842	TPZ	2020-10-06	8.56	8.59	8.44	8.52	73004	{}	\N	2020-10-12 16:21:59.305241
74843	TPZ	2020-10-07	8.53	8.64	8.53	8.64	47654	{}	\N	2020-10-12 16:21:59.310684
74844	TPZ	2020-10-08	8.62	8.88	8.62	8.88	46002	{}	\N	2020-10-12 16:21:59.316214
74845	TPZ	2020-10-09	8.82	8.869	8.775	8.85	38736	{}	\N	2020-10-12 16:21:59.321913
74846	TPZ	2020-10-12	8.82	8.82	8.7397	8.74	4984	{}	\N	2020-10-12 16:21:59.327501
74847	TRHC	2020-09-28	41	41.05	40.03	40.14	236300	{}	\N	2020-10-12 16:22:00.23489
74848	TRHC	2020-09-29	40	40.94	39.94	40.4	164407	{}	\N	2020-10-12 16:22:00.248873
74849	TRHC	2020-09-30	40.69	41.4	40.13	40.77	201305	{}	\N	2020-10-12 16:22:00.257586
74850	TRHC	2020-10-01	40.77	41.93	40.59	41.14	156465	{}	\N	2020-10-12 16:22:00.263457
74851	TRHC	2020-10-02	40.25	43.71	40.07	43.345	604740	{}	\N	2020-10-12 16:22:00.269288
74852	TRHC	2020-10-05	43.75	43.75	41.48	42.46	297614	{}	\N	2020-10-12 16:22:00.27485
74853	TRHC	2020-10-06	42.65	44.67	42.2	42.84	502453	{}	\N	2020-10-12 16:22:00.279885
74854	TRHC	2020-10-07	43.33	44.2	42.42	43.52	431856	{}	\N	2020-10-12 16:22:00.285973
74855	TRHC	2020-10-08	43.94	44.65	43.405	43.45	164311	{}	\N	2020-10-12 16:22:00.291354
74856	TRHC	2020-10-09	43.79	44.01	42.05	42.77	260877	{}	\N	2020-10-12 16:22:00.296613
74857	TRHC	2020-10-12	43.5	44.06	42.59	43.46	35328	{}	\N	2020-10-12 16:22:00.301839
74858	TRMK	2020-09-28	21.29	21.65	21.26	21.42	178142	{}	\N	2020-10-12 16:22:01.350324
74859	TRMK	2020-09-29	21.45	21.45	20.5225	21.31	203220	{}	\N	2020-10-12 16:22:01.359814
74860	TRMK	2020-09-30	21.24	21.71	21.17	21.41	187331	{}	\N	2020-10-12 16:22:01.365467
74861	TRMK	2020-10-01	21.26	21.55	21.05	21.44	224429	{}	\N	2020-10-12 16:22:01.371382
74862	TRMK	2020-10-02	21.13	22.21	21.13	22.1	177479	{}	\N	2020-10-12 16:22:01.377206
74863	TRMK	2020-10-05	22.37	22.85	22.31	22.77	170053	{}	\N	2020-10-12 16:22:01.382834
74864	TRMK	2020-10-06	23.09	23.63	22.66	22.71	298374	{}	\N	2020-10-12 16:22:01.388053
74865	TRMK	2020-10-07	22.93	23.5	22.33	23.15	186169	{}	\N	2020-10-12 16:22:01.393317
74866	TRMK	2020-10-08	23.32	23.53	23.12	23.35	155683	{}	\N	2020-10-12 16:22:01.398116
74867	TRMK	2020-10-09	23.53	23.53	22.97	23.06	170256	{}	\N	2020-10-12 16:22:01.40278
74868	TRMK	2020-10-12	22.95	23.65	22.94	23.485	50403	{}	\N	2020-10-12 16:22:01.407714
74869	TRQ	2020-09-28	0.83	0.849	0.827	0.835	1091711	{}	\N	2020-10-12 16:22:03.626999
74870	TRQ	2020-09-29	0.83	0.8446	0.8251	0.83	2191158	{}	\N	2020-10-12 16:22:03.639141
74871	TRQ	2020-09-30	0.8292	0.845	0.8256	0.8434	3058502	{}	\N	2020-10-12 16:22:03.648299
74872	TRQ	2020-10-01	0.839	0.853399	0.832	0.8495	2678023	{}	\N	2020-10-12 16:22:03.656905
74873	TRQ	2020-10-02	0.8209	0.8399	0.7975	0.8065	8378669	{}	\N	2020-10-12 16:22:03.662901
74874	TRQ	2020-10-05	0.81	0.8167	0.7968	0.8016	4102432	{}	\N	2020-10-12 16:22:03.668341
74875	TRQ	2020-10-06	0.8014	0.818	0.7553	0.7622	8490594	{}	\N	2020-10-12 16:22:03.674198
74876	TRQ	2020-10-07	0.7624	0.8	0.7575	0.775	5088515	{}	\N	2020-10-12 16:22:03.67943
74877	TRQ	2020-10-08	0.775	0.795	0.775	0.7822	2897610	{}	\N	2020-10-12 16:22:03.684261
74878	TRQ	2020-10-09	0.79	0.7989	0.7791	0.7792	4636678	{}	\N	2020-10-12 16:22:03.689155
74879	TRQ	2020-10-12	0.78	0.785	0.769	0.7759	1951198	{}	\N	2020-10-12 16:22:03.694408
74880	TRV	2020-09-28	109.73	111.9	109.3	110.22	1639012	{}	\N	2020-10-12 16:22:05.685939
74881	TRV	2020-09-29	109.97	110.25	106.55	107.44	2570444	{}	\N	2020-10-12 16:22:05.700277
74882	TRV	2020-09-30	107.81	109.13	107.22	108.19	3180078	{}	\N	2020-10-12 16:22:05.709413
74883	TRV	2020-10-01	108.18	109	106.26	107.81	2440278	{}	\N	2020-10-12 16:22:05.71613
74884	TRV	2020-10-02	106.69	109.49	106.46	108.58	2296584	{}	\N	2020-10-12 16:22:05.722398
74885	TRV	2020-10-05	110.55	112.03	109.55	111.91	2577537	{}	\N	2020-10-12 16:22:05.729242
74886	TRV	2020-10-06	112.55	113.31	110.79	111.53	2913220	{}	\N	2020-10-12 16:22:05.734347
74887	TRV	2020-10-07	112.08	114.34	112.08	113.8	2547068	{}	\N	2020-10-12 16:22:05.739753
74888	TRV	2020-10-08	114.85	116.95	114.35	115.62	1410873	{}	\N	2020-10-12 16:22:05.745275
74889	TRV	2020-10-09	116.44	116.98	115.23	115.83	1751148	{}	\N	2020-10-12 16:22:05.750551
74890	TRV	2020-10-12	114.35	116.92	114.055	116.08	878108	{}	\N	2020-10-12 16:22:05.756321
74891	TSC	2020-09-28	13.21	13.725	13.12	13.57	74561	{}	\N	2020-10-12 16:22:08.986375
74892	TSC	2020-09-29	13.5	13.5	12.91	13.215	100565	{}	\N	2020-10-12 16:22:08.995217
74893	TSC	2020-09-30	13.23	13.6818	13.05	13.24	97470	{}	\N	2020-10-12 16:22:09.002573
74894	TSC	2020-10-01	13.25	13.625	13.05	13.41	50008	{}	\N	2020-10-12 16:22:09.009455
74895	TSC	2020-10-02	13.07	14.08	12.99	13.92	97422	{}	\N	2020-10-12 16:22:09.014987
74896	TSC	2020-10-05	14.15	14.395	14.09	14.36	79707	{}	\N	2020-10-12 16:22:09.019967
74897	TSC	2020-10-06	14.61	14.78	14.04	14.07	131693	{}	\N	2020-10-12 16:22:09.025396
74898	TSC	2020-10-07	14.34	14.69	14.22	14.67	70732	{}	\N	2020-10-12 16:22:09.03078
74899	TSC	2020-10-08	14.72	14.94	14.501	14.83	50826	{}	\N	2020-10-12 16:22:09.035581
74900	TSC	2020-10-09	15.03	15.03	14.39	14.44	49292	{}	\N	2020-10-12 16:22:09.04022
74901	TSC	2020-10-12	14.12	14.38	13.38	14.23	18697	{}	\N	2020-10-12 16:22:09.044946
74902	TTAC	2020-09-28	40.9203	41.17	40.9134	41.1	4945	{}	\N	2020-10-12 16:22:10.689341
74903	TTAC	2020-09-29	41.0761	41.105	40.8873	40.921	6314	{}	\N	2020-10-12 16:22:10.699259
74904	TTAC	2020-09-30	41.3638	41.49	40.95	41.15	54284	{}	\N	2020-10-12 16:22:10.706501
74905	TTAC	2020-10-01	41.42	41.6337	41.38	41.54	9144	{}	\N	2020-10-12 16:22:10.712875
74906	TTAC	2020-10-02	41.3476	41.4656	41.22	41.22	10466	{}	\N	2020-10-12 16:22:10.718209
74907	TTAC	2020-10-05	41.66	42.0019	41.66	41.96	5456	{}	\N	2020-10-12 16:22:10.723283
74908	TTAC	2020-10-06	42.11	42.2824	41.38	41.49	11433	{}	\N	2020-10-12 16:22:10.7283
74909	TTAC	2020-10-07	42.0199	42.26	42.0199	42.2001	10887	{}	\N	2020-10-12 16:22:10.734016
74910	TTAC	2020-10-08	42.46	42.5336	42.4186	42.4577	10450	{}	\N	2020-10-12 16:22:10.739065
74911	TTAC	2020-10-09	42.759	42.9626	42.759	42.91	9350	{}	\N	2020-10-12 16:22:10.743986
74912	TTAC	2020-10-12	43.1801	43.4738	43.1801	43.4738	4212	{}	\N	2020-10-12 16:22:10.74935
74913	TV	2020-09-28	6.18	6.26	6.11	6.14	1266517	{}	\N	2020-10-12 16:22:13.436051
74914	TV	2020-09-29	6.13	6.195	6.06	6.11	2198127	{}	\N	2020-10-12 16:22:13.444541
74915	TV	2020-09-30	6.18	6.23	6.12	6.18	4736326	{}	\N	2020-10-12 16:22:13.450718
74916	TV	2020-10-01	6.23	6.36	6.16	6.36	3137579	{}	\N	2020-10-12 16:22:13.456387
74917	TV	2020-10-02	6.23	6.41	6.18	6.32	995428	{}	\N	2020-10-12 16:22:13.46127
74918	TV	2020-10-05	6.38	6.56	6.37	6.52	2216460	{}	\N	2020-10-12 16:22:13.465772
74919	TV	2020-10-06	6.57	6.585	6.27	6.31	1972312	{}	\N	2020-10-12 16:22:13.470591
74920	TV	2020-10-07	6.34	6.7	6.34	6.67	5168084	{}	\N	2020-10-12 16:22:13.475419
74921	TV	2020-10-08	6.73	7.175	6.7	7.15	2585007	{}	\N	2020-10-12 16:22:13.480051
74922	TV	2020-10-09	7.25	7.465	7.14	7.24	2133573	{}	\N	2020-10-12 16:22:13.484736
74923	TV	2020-10-12	7.28	7.29	7.131	7.22	412246	{}	\N	2020-10-12 16:22:13.490135
74924	DTF	2020-09-28	14.4328	14.48	14.4328	14.48	2187	{}	\N	2020-10-12 16:22:14.603958
74925	DTF	2020-09-29	14.47	14.47	14.47	14.47	828	{}	\N	2020-10-12 16:22:14.614548
74926	DTF	2020-09-30	14.54	14.54	14.4	14.44	6289	{}	\N	2020-10-12 16:22:14.621489
74927	DTF	2020-10-01	14.42	14.42	14.38	14.4001	4416	{}	\N	2020-10-12 16:22:14.627431
74928	DTF	2020-10-02	14.374	14.455	14.374	14.455	24704	{}	\N	2020-10-12 16:22:14.632705
74929	DTF	2020-10-05	14.5	14.5	14.4	14.43	14360	{}	\N	2020-10-12 16:22:14.638008
74930	DTF	2020-10-06	14.425	14.49	14.42	14.43	21874	{}	\N	2020-10-12 16:22:14.642525
74931	DTF	2020-10-07	14.44	14.45	14.44	14.45	7346	{}	\N	2020-10-12 16:22:14.647008
74932	DTF	2020-10-08	14.4201	14.455	14.4	14.4	10594	{}	\N	2020-10-12 16:22:14.651589
74933	DTF	2020-10-09	14.475	14.5499	14.475	14.5499	820	{}	\N	2020-10-12 16:22:14.656411
74934	DTF	2020-10-12	14.47	14.47	14.43	14.43	1652	{}	\N	2020-10-12 16:22:14.661011
74935	TREX	2020-09-28	69.7	71.52	69.43	71.06	445003	{}	\N	2020-10-12 16:22:16.302423
74936	TREX	2020-09-29	71.21	72.14	70.6	70.79	888315	{}	\N	2020-10-12 16:22:16.31379
74937	TREX	2020-09-30	71.28	72.77	70.29	71.6	1490564	{}	\N	2020-10-12 16:22:16.319926
74938	TREX	2020-10-01	72.47	73.62	71.96	73.45	951670	{}	\N	2020-10-12 16:22:16.325614
74939	TREX	2020-10-02	71.72	74.66	71.51	73.73	1087822	{}	\N	2020-10-12 16:22:16.331098
74940	TREX	2020-10-05	74.49	76.83	74.15	76.58	1720039	{}	\N	2020-10-12 16:22:16.33624
74941	TREX	2020-10-06	76.73	78.61	75.65	76.05	1414013	{}	\N	2020-10-12 16:22:16.34118
74942	TREX	2020-10-07	77.49	78	74.83	75.65	1336367	{}	\N	2020-10-12 16:22:16.345987
74943	TREX	2020-10-08	76.54	77.255	75.69	76.33	673460	{}	\N	2020-10-12 16:22:16.350824
74944	TREX	2020-10-09	77.57	79.19	77	78.22	708672	{}	\N	2020-10-12 16:22:16.355563
74945	TREX	2020-10-12	79.23	79.34	78.345	79.045	318351	{}	\N	2020-10-12 16:22:16.360251
74946	TRNS	2020-09-28	29.63	30.25	29.63	29.63	10356	{}	\N	2020-10-12 16:22:17.541044
74947	TRNS	2020-09-29	29.3961	29.555	29.3	29.3	6509	{}	\N	2020-10-12 16:22:17.552214
74948	TRNS	2020-09-30	29.4362	29.47	29.1	29.3	48207	{}	\N	2020-10-12 16:22:17.559606
74949	TRNS	2020-10-01	29.4846	29.8985	29.05	29.39	21987	{}	\N	2020-10-12 16:22:17.565753
74950	TRNS	2020-10-02	29.05	29.49	29.05	29.06	7956	{}	\N	2020-10-12 16:22:17.57148
74951	TRNS	2020-10-05	29.11	29.535	29.1	29.31	8255	{}	\N	2020-10-12 16:22:17.576923
74952	TRNS	2020-10-06	29.49	29.625	29.05	29.05	12105	{}	\N	2020-10-12 16:22:17.581923
74953	TRNS	2020-10-07	29.05	29.35	28.7	28.9	25233	{}	\N	2020-10-12 16:22:17.586581
74954	TRNS	2020-10-08	28.98	29.325	28.76	28.78	10819	{}	\N	2020-10-12 16:22:17.591812
74955	TRNS	2020-10-09	29.06	29.385	28.5101	29.09	7354	{}	\N	2020-10-12 16:22:17.596768
74956	TRNS	2020-10-12	29.14	29.15	29.14	29.15	606	{}	\N	2020-10-12 16:22:17.601403
74957	TRU	2020-09-28	82.46	84.92	82.24	83.74	1995393	{}	\N	2020-10-12 16:22:18.608019
74958	TRU	2020-09-29	84.24	86.23	83.82	84.62	2929011	{}	\N	2020-10-12 16:22:18.618368
74959	TRU	2020-09-30	85.1	85.17	83.35	84.13	3425436	{}	\N	2020-10-12 16:22:18.625484
74960	TRU	2020-10-01	85.14	86.76	84.785	86.42	2334623	{}	\N	2020-10-12 16:22:18.631493
74961	TRU	2020-10-02	84.77	86.29	83.93	85.8	1685418	{}	\N	2020-10-12 16:22:18.636805
74962	TRU	2020-10-05	86.14	87.8	86.03	87.35	1768925	{}	\N	2020-10-12 16:22:18.642199
74963	TRU	2020-10-06	87.87	89.195	87.02	87.24	2413041	{}	\N	2020-10-12 16:22:18.646523
74964	TRU	2020-10-07	87.77	88.18	86.92	87.73	1782208	{}	\N	2020-10-12 16:22:18.650853
74965	TRU	2020-10-08	89.45	89.45	87.97	88.54	1064189	{}	\N	2020-10-12 16:22:18.655442
74966	TRU	2020-10-09	88.93	90.13	88.59	89.22	912271	{}	\N	2020-10-12 16:22:18.660174
74967	TRU	2020-10-12	90.21	90.48	89.33	89.7	425768	{}	\N	2020-10-12 16:22:18.664453
74968	TSE	2020-09-28	25.44	26.87	25.44	26.51	457510	{}	\N	2020-10-12 16:22:20.392696
74969	TSE	2020-09-29	26.33	26.67	25.63	25.79	546059	{}	\N	2020-10-12 16:22:20.402845
74970	TSE	2020-09-30	25.85	26.96	25.43	25.64	740241	{}	\N	2020-10-12 16:22:20.40937
74971	TSE	2020-10-01	25.62	25.9	25.03	25.5	712742	{}	\N	2020-10-12 16:22:20.415204
74972	TSE	2020-10-02	25	27.07	24.92	26.6	744089	{}	\N	2020-10-12 16:22:20.420013
74973	TSE	2020-10-05	26.97	28.15	26.83	27.52	704569	{}	\N	2020-10-12 16:22:20.424702
74974	TSE	2020-10-06	29.53	30.16	27.565	27.78	1469569	{}	\N	2020-10-12 16:22:20.429345
74975	TSE	2020-10-07	27.75	29.3329	27.75	29	2788272	{}	\N	2020-10-12 16:22:20.434292
74976	TSE	2020-10-08	29.2	29.72	29.06	29.44	808198	{}	\N	2020-10-12 16:22:20.439042
74977	TSE	2020-10-09	29.73	30.24	29.22	29.43	835072	{}	\N	2020-10-12 16:22:20.444325
74978	TSE	2020-10-12	30.86	30.86	29.65	30	672294	{}	\N	2020-10-12 16:22:20.449218
74979	TSQ	2020-09-28	4.685	4.7606	4.64	4.75	17168	{}	\N	2020-10-12 16:22:21.511449
74980	TSQ	2020-09-29	4.7	4.71	4.65	4.7	18580	{}	\N	2020-10-12 16:22:21.523271
74981	TSQ	2020-09-30	4.68	4.7199	4.64	4.66	45470	{}	\N	2020-10-12 16:22:21.531295
74982	TSQ	2020-10-01	4.62	4.73	4.62	4.67	21952	{}	\N	2020-10-12 16:22:21.537205
74983	TSQ	2020-10-02	4.66	4.84	4.6	4.82	33646	{}	\N	2020-10-12 16:22:21.542771
74984	TSQ	2020-10-05	4.69	4.84	4.615	4.8	29027	{}	\N	2020-10-12 16:22:21.547761
74985	TSQ	2020-10-06	4.8	5.02	4.7383	5	41789	{}	\N	2020-10-12 16:22:21.552515
74986	TSQ	2020-10-07	5.05	5.06	4.855	5.03	33924	{}	\N	2020-10-12 16:22:21.557386
74987	TSQ	2020-10-08	5.08	5.08	5	5.01	10044	{}	\N	2020-10-12 16:22:21.562643
74988	TSQ	2020-10-09	5	5.0288	4.88	4.89	24972	{}	\N	2020-10-12 16:22:21.567378
74989	TSQ	2020-10-12	4.92	4.92	4.85	4.85	3550	{}	\N	2020-10-12 16:22:21.572018
74990	TTEC	2020-09-28	53.14	54.36	53.14	54.01	115730	{}	\N	2020-10-12 16:22:22.879724
74991	TTEC	2020-09-29	54.06	55.56	53.4801	55.33	96114	{}	\N	2020-10-12 16:22:22.892339
74992	TTEC	2020-09-30	55.61	56.65	53.94	54.55	139998	{}	\N	2020-10-12 16:22:22.898478
74993	TTEC	2020-10-01	54.42	55.5	53.66	55.37	154947	{}	\N	2020-10-12 16:22:22.903891
74994	TTEC	2020-10-02	54.22	56.6	53.47	56.25	117457	{}	\N	2020-10-12 16:22:22.909166
74995	TTEC	2020-10-05	56.72	57.55	55.77	56.99	102282	{}	\N	2020-10-12 16:22:22.914131
74996	TTEC	2020-10-06	57.22	57.98	56.06	56.35	105775	{}	\N	2020-10-12 16:22:22.918549
74997	TTEC	2020-10-07	56.92	57.5599	56.04	56.89	119294	{}	\N	2020-10-12 16:22:22.923116
74998	TTEC	2020-10-08	57.39	58.5	56.62	58.3	94553	{}	\N	2020-10-12 16:22:22.928462
74999	TTEC	2020-10-09	58.8	59.41	58.48	58.69	70018	{}	\N	2020-10-12 16:22:22.933201
75000	TTEC	2020-10-12	58.84	58.84	57.89	58	25815	{}	\N	2020-10-12 16:22:22.937496
75001	TTOO	2020-09-28	1.25	1.3	1.2145	1.29	2873830	{}	\N	2020-10-12 16:22:24.073263
75002	TTOO	2020-09-29	1.28	1.3	1.26	1.29	2002567	{}	\N	2020-10-12 16:22:24.086405
75003	TTOO	2020-09-30	1.3833	1.43	1.29	1.36	5929289	{}	\N	2020-10-12 16:22:24.094021
75004	TTOO	2020-10-01	1.35	1.35	1.29	1.3	1675381	{}	\N	2020-10-12 16:22:24.10077
75005	TTOO	2020-10-02	1.22	1.34	1.22	1.34	2364608	{}	\N	2020-10-12 16:22:24.10611
75006	TTOO	2020-10-05	1.31	1.35	1.29	1.33	1735607	{}	\N	2020-10-12 16:22:24.111364
75007	TTOO	2020-10-06	1.32	1.3619	1.26	1.29	2807374	{}	\N	2020-10-12 16:22:24.116076
75008	TTOO	2020-10-07	1.2953	1.32	1.28	1.3	2086413	{}	\N	2020-10-12 16:22:24.120927
75009	TTOO	2020-10-08	1.36	1.37	1.32	1.37	3000605	{}	\N	2020-10-12 16:22:24.125549
75010	TTOO	2020-10-09	1.38	1.42	1.36	1.37	1899308	{}	\N	2020-10-12 16:22:24.130561
75011	TTOO	2020-10-12	1.38	1.38	1.35	1.3675	609016	{}	\N	2020-10-12 16:22:24.135204
75012	TUP	2020-09-28	23.15	23.26	20.75	21.26	2565009	{}	\N	2020-10-12 16:22:25.105461
75013	TUP	2020-09-29	21.42	21.88	19.44	20.11	3925590	{}	\N	2020-10-12 16:22:25.117721
75014	TUP	2020-09-30	20	21.25	19.85	20.16	3814748	{}	\N	2020-10-12 16:22:25.124877
75015	TUP	2020-10-01	20.27	21.4133	19.6	21.19	3663619	{}	\N	2020-10-12 16:22:25.131554
75016	TUP	2020-10-02	20.06	21.33	19.77	21.05	2663450	{}	\N	2020-10-12 16:22:25.137531
75017	TUP	2020-10-05	21.54	22.35	19.97	21.37	6631255	{}	\N	2020-10-12 16:22:25.142361
75018	TUP	2020-10-06	21.68	22.93	21.13	21.59	3764998	{}	\N	2020-10-12 16:22:25.148133
75019	TUP	2020-10-07	22.26	23.83	22.25	23.51	2536436	{}	\N	2020-10-12 16:22:25.153335
75020	TUP	2020-10-08	24	24.35	22.56	22.73	1835762	{}	\N	2020-10-12 16:22:25.158205
75021	TUP	2020-10-09	22.76	23.456	22.12	22.17	1933650	{}	\N	2020-10-12 16:22:25.163576
75022	TUP	2020-10-12	22.22	22.665	21.42	21.85	1229873	{}	\N	2020-10-12 16:22:25.168765
75023	TWO	2020-09-28	5.21	5.37	5.19	5.3	3691818	{}	\N	2020-10-12 16:22:28.638069
75024	TWO	2020-09-29	5.3	5.33	5.1	5.2	7060237	{}	\N	2020-10-12 16:22:28.648512
75025	TWO	2020-09-30	5.19	5.23	5.05	5.09	8222629	{}	\N	2020-10-12 16:22:28.65571
75026	TWO	2020-10-01	5.1	5.17	5.03	5.15	4875085	{}	\N	2020-10-12 16:22:28.661557
75027	TWO	2020-10-02	4.98	5.245	4.9436	5.22	6444921	{}	\N	2020-10-12 16:22:28.666878
75028	TWO	2020-10-05	5.24	5.29	5.2	5.27	4268186	{}	\N	2020-10-12 16:22:28.671729
75029	TWO	2020-10-06	5.29	5.37	5.19	5.19	5174818	{}	\N	2020-10-12 16:22:28.676072
75030	TWO	2020-10-07	5.23	5.28	5.165	5.26	4574215	{}	\N	2020-10-12 16:22:28.680282
75031	TWO	2020-10-08	5.32	5.45	5.29	5.44	5714692	{}	\N	2020-10-12 16:22:28.684787
75032	TWO	2020-10-09	5.42	5.45	5.35	5.41	6063941	{}	\N	2020-10-12 16:22:28.689795
75033	TWO	2020-10-12	5.39	5.4304	5.315	5.365	1687703	{}	\N	2020-10-12 16:22:28.694211
75034	TYD	2020-09-28	68.35	68.5	67.85	67.85	7573	{}	\N	2020-10-12 16:22:30.09855
75035	TYD	2020-09-29	68.49	68.72	68.3714	68.3916	2322	{}	\N	2020-10-12 16:22:30.109661
75036	TYD	2020-09-30	68.42	68.5	67.74	68.14	6848	{}	\N	2020-10-12 16:22:30.116807
75037	TYD	2020-10-01	67.44	68.12	67.44	68.12	1815	{}	\N	2020-10-12 16:22:30.123535
75038	TYD	2020-10-02	68.4519	68.4519	67.91	67.91	1030	{}	\N	2020-10-12 16:22:30.128735
75039	TYD	2020-10-05	67.72	67.72	66.6339	66.6339	3182	{}	\N	2020-10-12 16:22:30.134648
75040	TYD	2020-10-06	66.6	67.3846	66.5289	67.3846	4267	{}	\N	2020-10-12 16:22:30.140022
75041	TYD	2020-10-07	66.3706	66.6605	66.3706	66.626	1168	{}	\N	2020-10-12 16:22:30.145355
75042	TYD	2020-10-08	66.3601	66.767	66.3601	66.767	1739	{}	\N	2020-10-12 16:22:30.149995
75043	TYD	2020-10-09	66.564	66.92	66.25	66.25	3726	{}	\N	2020-10-12 16:22:30.155337
75044	TYD	2020-10-12	66.8397	66.875	66.6445	66.67	1312	{}	\N	2020-10-12 16:22:30.159974
75045	UAE	2020-09-28	10.67	10.8	10.663	10.8	5712	{}	\N	2020-10-12 16:22:31.063138
75046	UAE	2020-09-29	10.6381	10.6381	10.6052	10.6052	450	{}	\N	2020-10-12 16:22:31.074
75047	UAE	2020-09-30	10.76	10.8	10.69	10.75	5663	{}	\N	2020-10-12 16:22:31.082075
75048	UAE	2020-10-01	10.64	10.6503	10.625	10.6503	512	{}	\N	2020-10-12 16:22:31.090256
75049	UAE	2020-10-02	10.64	10.67	10.41	10.48	3864644	{}	\N	2020-10-12 16:22:31.096269
75050	UAE	2020-10-05	10.48	10.6597	10.48	10.65	2415	{}	\N	2020-10-12 16:22:31.10138
75051	UAE	2020-10-06	10.65	10.65	10.43	10.43	662244	{}	\N	2020-10-12 16:22:31.106611
75052	UAE	2020-10-07	10.5	10.78	10.485	10.68	420641	{}	\N	2020-10-12 16:22:31.111959
75053	UAE	2020-10-08	10.72	10.83	10.7002	10.83	275884	{}	\N	2020-10-12 16:22:31.117419
75054	UAE	2020-10-09	10.54	10.64	10.54	10.62	4124	{}	\N	2020-10-12 16:22:31.122875
75055	UAE	2020-10-12	10.52	10.67	10.52	10.66	5117	{}	\N	2020-10-12 16:22:31.12838
75056	TR	2020-09-28	30.7	30.86	30.24	30.48	169753	{}	\N	2020-10-12 16:22:34.313757
75057	TR	2020-09-29	30.53	30.78	30.3408	30.63	269827	{}	\N	2020-10-12 16:22:34.326165
75058	TR	2020-09-30	30.89	31.02	30.56	30.9	421170	{}	\N	2020-10-12 16:22:34.332936
75059	TR	2020-10-01	31.01	31.17	30.6	31.13	347059	{}	\N	2020-10-12 16:22:34.338896
75060	TR	2020-10-02	31.08	31.435	30.96	31.33	276675	{}	\N	2020-10-12 16:22:34.344767
75061	TR	2020-10-05	31.43	31.47	31.13	31.32	204686	{}	\N	2020-10-12 16:22:34.349721
75062	TR	2020-10-06	31.39	31.5	31.04	31.23	254597	{}	\N	2020-10-12 16:22:34.355186
75063	TR	2020-10-07	31.34	31.5	31.03	31.46	282501	{}	\N	2020-10-12 16:22:34.360192
75064	TR	2020-10-08	31.55	31.9	31.47	31.64	424799	{}	\N	2020-10-12 16:22:34.364972
75065	TR	2020-10-09	31.68	32.11	31.6001	31.96	229954	{}	\N	2020-10-12 16:22:34.370056
75066	TR	2020-10-12	31.86	32.37	31.86	32.19	89086	{}	\N	2020-10-12 16:22:34.375192
75067	TRIL	2020-09-28	13.84	13.9	13.17	13.65	1953194	{}	\N	2020-10-12 16:22:35.342248
75068	TRIL	2020-09-29	13.68	14.0931	13.28	13.55	979627	{}	\N	2020-10-12 16:22:35.354805
75069	TRIL	2020-09-30	13.56	14.3	13.41	14.21	1466759	{}	\N	2020-10-12 16:22:35.364145
75070	TRIL	2020-10-01	14.7721	15.07	14.16	14.57	3010603	{}	\N	2020-10-12 16:22:35.370572
75071	TRIL	2020-10-02	14.1	14.33	13.4527	13.65	1701114	{}	\N	2020-10-12 16:22:35.376174
75072	TRIL	2020-10-05	13.86	14.75	13.85	14.69	1919288	{}	\N	2020-10-12 16:22:35.38059
75073	TRIL	2020-10-06	14.7	15.55	14.267	15.2	1794247	{}	\N	2020-10-12 16:22:35.385913
75074	TRIL	2020-10-07	15.35	15.39	14.9	15.18	928875	{}	\N	2020-10-12 16:22:35.39181
75075	TRIL	2020-10-08	15.36	15.69	14.92	15.15	1347093	{}	\N	2020-10-12 16:22:35.397261
75076	TRIL	2020-10-09	15.15	15.68	15.11	15.37	999150	{}	\N	2020-10-12 16:22:35.401463
75077	TRIL	2020-10-12	15.92	16.15	15.56	15.625	930173	{}	\N	2020-10-12 16:22:35.405698
75078	TROW	2020-09-28	127.26	129.08	126.915	128.55	1085386	{}	\N	2020-10-12 16:22:38.545096
75079	TROW	2020-09-29	127.63	129.365	126.99	127.49	1038669	{}	\N	2020-10-12 16:22:38.556744
75080	TROW	2020-09-30	129.1	130.02	127.11	128.22	1532073	{}	\N	2020-10-12 16:22:38.564356
75081	TROW	2020-10-01	128.4	129.26	127.45	128.04	963258	{}	\N	2020-10-12 16:22:38.570633
75082	TROW	2020-10-02	126.83	130.08	125.905	129.09	1247767	{}	\N	2020-10-12 16:22:38.577523
75083	TROW	2020-10-05	130.34	133.64	130.27	133.51	963545	{}	\N	2020-10-12 16:22:38.582429
75084	TROW	2020-10-06	134.15	137.1027	132.74	133.87	877937	{}	\N	2020-10-12 16:22:38.587278
75085	TROW	2020-10-07	135.04	137.685	135.04	137.23	813112	{}	\N	2020-10-12 16:22:38.592051
75086	TROW	2020-10-08	138.84	140.85	137.24	140.83	875283	{}	\N	2020-10-12 16:22:38.59752
75087	TROW	2020-10-09	141.09	142.51	140.65	142.21	895808	{}	\N	2020-10-12 16:22:38.602687
75088	TROW	2020-10-12	142.6	144.475	142.21	143.34	613099	{}	\N	2020-10-12 16:22:38.607679
75089	TRTX	2020-09-28	8.36	8.74	8.3	8.64	331760	{}	\N	2020-10-12 16:22:40.496284
75090	TRTX	2020-09-29	8.57	8.62	8.37	8.53	520258	{}	\N	2020-10-12 16:22:40.507996
75091	TRTX	2020-09-30	8.41	8.63	8.34	8.46	1074624	{}	\N	2020-10-12 16:22:40.513454
75092	TRTX	2020-10-01	8.6	8.6	8.27	8.51	674440	{}	\N	2020-10-12 16:22:40.519324
75093	TRTX	2020-10-02	8.32	8.72	8.271	8.69	547191	{}	\N	2020-10-12 16:22:40.52378
75094	TRTX	2020-10-05	8.72	8.83	8.6348	8.76	953288	{}	\N	2020-10-12 16:22:40.528332
75095	TRTX	2020-10-06	8.86	9.14	8.62	8.64	661697	{}	\N	2020-10-12 16:22:40.533469
75096	TRTX	2020-10-07	8.8	8.8	8.45	8.62	510050	{}	\N	2020-10-12 16:22:40.538452
75097	TRTX	2020-10-08	8.68	8.94	8.655	8.92	404507	{}	\N	2020-10-12 16:22:40.543019
75098	TRTX	2020-10-09	9.04	9.04	8.6	8.61	347096	{}	\N	2020-10-12 16:22:40.547746
75099	TRTX	2020-10-12	8.6	8.71	8.54	8.59	181517	{}	\N	2020-10-12 16:22:40.552712
75100	TTI	2020-09-28	0.52	0.525	0.4963	0.5011	1207643	{}	\N	2020-10-12 16:22:46.354451
75101	TTI	2020-09-29	0.52	0.5298	0.49	0.5151	3729228	{}	\N	2020-10-12 16:22:46.366066
75102	TTI	2020-09-30	0.495	0.5207	0.4925	0.5108	3955377	{}	\N	2020-10-12 16:22:46.373439
75103	TTI	2020-10-01	0.5	0.5194	0.495	0.5022	1899244	{}	\N	2020-10-12 16:22:46.379582
75104	TTI	2020-10-02	0.475	0.525	0.473	0.5213	2730915	{}	\N	2020-10-12 16:22:46.385609
75105	TTI	2020-10-05	0.5222	0.55	0.5214	0.5402	2099018	{}	\N	2020-10-12 16:22:46.390924
75106	TTI	2020-10-06	0.54	0.54	0.5066	0.5066	1987504	{}	\N	2020-10-12 16:22:46.395844
75107	TTI	2020-10-07	0.506	0.525	0.496	0.5188	1988161	{}	\N	2020-10-12 16:22:46.400905
75108	TTI	2020-10-08	0.51	0.5479	0.51	0.5368	1838471	{}	\N	2020-10-12 16:22:46.406009
75109	TTI	2020-10-09	0.5369	0.55	0.5192	0.521	2015132	{}	\N	2020-10-12 16:22:46.411102
75110	TTI	2020-10-12	0.52	0.5298	0.505	0.5137	859594	{}	\N	2020-10-12 16:22:46.415988
75111	TU	2020-09-28	17.88	17.97	17.815	17.93	838942	{}	\N	2020-10-12 16:22:49.568582
75112	TU	2020-09-29	17.88	17.96	17.73	17.79	1991641	{}	\N	2020-10-12 16:22:49.580006
75113	TU	2020-09-30	17.83	17.83	17.57	17.61	2536838	{}	\N	2020-10-12 16:22:49.587738
75114	TU	2020-10-01	17.62	17.76	17.56	17.62	2573042	{}	\N	2020-10-12 16:22:49.594264
75115	TU	2020-10-02	17.62	17.83	17.54	17.77	1973692	{}	\N	2020-10-12 16:22:49.599904
75116	TU	2020-10-05	17.91	18	17.725	17.98	1648568	{}	\N	2020-10-12 16:22:49.604755
75117	TU	2020-10-06	18	18.07	17.83	17.98	2448580	{}	\N	2020-10-12 16:22:49.60956
75118	TU	2020-10-07	18.05	18.1	17.92	18.06	2400379	{}	\N	2020-10-12 16:22:49.6142
75119	TU	2020-10-08	18.15	18.39	18.09	18.38	1605203	{}	\N	2020-10-12 16:22:49.618907
75120	TU	2020-10-09	18.41	18.5	18.27	18.37	1128742	{}	\N	2020-10-12 16:22:49.623738
75121	TU	2020-10-12	18.45	18.5032	18.37	18.4414	214497	{}	\N	2020-10-12 16:22:49.628374
75122	TWIN	2020-09-28	5.01	5.35	5.01	5.33	48305	{}	\N	2020-10-12 16:22:50.836799
75123	TWIN	2020-09-29	5.17	5.24	4.87	5.23	65075	{}	\N	2020-10-12 16:22:50.846799
75124	TWIN	2020-09-30	5.16	5.25	5.01	5.06	51557	{}	\N	2020-10-12 16:22:50.853551
75125	TWIN	2020-10-01	5.06	5.19	4.96	5.18	45967	{}	\N	2020-10-12 16:22:50.859054
75126	TWIN	2020-10-02	4.99	5.14	4.96	5.09	21118	{}	\N	2020-10-12 16:22:50.864359
75127	TWIN	2020-10-05	5.15	5.84	5.15	5.71	51325	{}	\N	2020-10-12 16:22:50.868577
75128	TWIN	2020-10-06	5.61	5.81	5.35	5.4	15331	{}	\N	2020-10-12 16:22:50.873139
75129	TWIN	2020-10-07	5.43	5.64	5.21	5.51	21675	{}	\N	2020-10-12 16:22:50.877889
75130	TWIN	2020-10-08	5.425	5.99	5.425	5.92	39571	{}	\N	2020-10-12 16:22:50.882356
75131	TWIN	2020-10-09	5.87	6.12	5.59	6.07	26265	{}	\N	2020-10-12 16:22:50.886644
75132	TWIN	2020-10-12	5.915	6.02	5.88	5.91	8071	{}	\N	2020-10-12 16:22:50.891189
75133	TXMD	2020-09-28	1.44	1.49	1.42	1.475	2237807	{}	\N	2020-10-12 16:22:53.270135
75134	TXMD	2020-09-29	1.45	1.62	1.4401	1.61	3907465	{}	\N	2020-10-12 16:22:53.281736
75135	TXMD	2020-09-30	1.6	1.68	1.55	1.58	3057990	{}	\N	2020-10-12 16:22:53.288539
75136	TXMD	2020-10-01	1.62	1.6775	1.58	1.65	2688798	{}	\N	2020-10-12 16:22:53.293873
75137	TXMD	2020-10-02	1.52	1.63	1.51	1.59	1801650	{}	\N	2020-10-12 16:22:53.299386
75138	TXMD	2020-10-05	1.59	1.66	1.59	1.62	2336524	{}	\N	2020-10-12 16:22:53.304188
75139	TXMD	2020-10-06	1.6	1.7	1.59	1.6	2752202	{}	\N	2020-10-12 16:22:53.308379
75140	TXMD	2020-10-07	1.64	1.7	1.62	1.68	2389753	{}	\N	2020-10-12 16:22:53.31284
75141	TXMD	2020-10-08	1.69	1.81	1.69	1.76	3104618	{}	\N	2020-10-12 16:22:53.317085
75142	TXMD	2020-10-09	1.825	1.85	1.63	1.65	4802943	{}	\N	2020-10-12 16:22:53.321323
75143	TXMD	2020-10-12	1.67	1.7	1.545	1.64	1391376	{}	\N	2020-10-12 16:22:53.325533
75144	TYL	2020-09-28	346.29	350.79	340.84	346.69	239734	{}	\N	2020-10-12 16:22:54.942604
75145	TYL	2020-09-29	347.8	348.79	345.65	346.52	343235	{}	\N	2020-10-12 16:22:54.950054
75146	TYL	2020-09-30	348.37	352.5	344.6	348.56	693522	{}	\N	2020-10-12 16:22:54.956427
75147	TYL	2020-10-01	352.57	358.435	349.34	357.26	402518	{}	\N	2020-10-12 16:22:54.961828
75148	TYL	2020-10-02	352.37	354.44	346.45	350.72	374112	{}	\N	2020-10-12 16:22:54.967719
75149	TYL	2020-10-05	351.76	357.53	349.23	357.17	353368	{}	\N	2020-10-12 16:22:54.972639
75150	TYL	2020-10-06	355.31	369.78	355.19	363.94	416259	{}	\N	2020-10-12 16:22:54.978197
75151	TYL	2020-10-07	366.23	376.03	362.73	375.76	435974	{}	\N	2020-10-12 16:22:54.983568
75152	TYL	2020-10-08	378.42	379.275	373.01	378.43	285478	{}	\N	2020-10-12 16:22:54.988976
75153	TYL	2020-10-09	380	390.2	378.04	389.28	632322	{}	\N	2020-10-12 16:22:54.994195
75154	TYL	2020-10-12	392.93	395	388.29	390.22	105899	{}	\N	2020-10-12 16:22:54.999082
75155	UAA	2020-09-28	11.37	11.53	11.25	11.43	6612534	{}	\N	2020-10-12 16:22:56.98683
75156	UAA	2020-09-29	11.37	11.37	10.89	11.08	9897932	{}	\N	2020-10-12 16:22:57.002871
75157	UAA	2020-09-30	11.17	11.47	11.13	11.23	9580528	{}	\N	2020-10-12 16:22:57.011977
75158	UAA	2020-10-01	11.37	11.935	11.34	11.86	14607436	{}	\N	2020-10-12 16:22:57.018241
75159	UAA	2020-10-02	11.49	12.1	11.43	12.06	9249919	{}	\N	2020-10-12 16:22:57.024374
75160	UAA	2020-10-05	12.25	12.31	12.07	12.16	8526767	{}	\N	2020-10-12 16:22:57.02941
75161	UAA	2020-10-06	12.2	12.295	11.64	11.69	8969028	{}	\N	2020-10-12 16:22:57.034587
75162	UAA	2020-10-07	11.88	12.23	11.85	12.17	7722095	{}	\N	2020-10-12 16:22:57.039544
75163	UAA	2020-10-08	12.25	12.65	12.22	12.6	12732985	{}	\N	2020-10-12 16:22:57.044741
75164	UAA	2020-10-09	12.66	12.708	12.29	12.55	10720337	{}	\N	2020-10-12 16:22:57.04987
75165	UAA	2020-10-12	12.59	12.64	12.445	12.53	3641648	{}	\N	2020-10-12 16:22:57.054803
75166	UBA	2020-09-28	9.09	9.38	9.09	9.22	387473	{}	\N	2020-10-12 16:22:59.496829
75167	UBA	2020-09-29	9.22	9.24	8.89	9.09	712359	{}	\N	2020-10-12 16:22:59.510396
75168	UBA	2020-09-30	9.22	9.45	9.12	9.2	749135	{}	\N	2020-10-12 16:22:59.517713
75169	UBA	2020-10-01	9.14	9.42	9.13	9.41	674714	{}	\N	2020-10-12 16:22:59.52433
75170	UBA	2020-10-02	9.17	9.58	9.17	9.52	564669	{}	\N	2020-10-12 16:22:59.530315
75171	UBA	2020-10-05	9.67	9.74	9.35	9.56	710471	{}	\N	2020-10-12 16:22:59.535534
75172	UBA	2020-10-06	9.61	9.79	9.46	9.6	751310	{}	\N	2020-10-12 16:22:59.540813
75173	UBA	2020-10-07	9.63	9.73	9.47	9.53	745088	{}	\N	2020-10-12 16:22:59.546117
75174	UBA	2020-10-08	9.62	9.89	9.62	9.73	405551	{}	\N	2020-10-12 16:22:59.551083
75175	UBA	2020-10-09	9.96	9.96	9.6	9.7	555578	{}	\N	2020-10-12 16:22:59.556525
75176	UBA	2020-10-12	9.95	10.0225	9.62	9.9099	657247	{}	\N	2020-10-12 16:22:59.561722
75177	UBP	2020-09-28	8.6455	8.9062	8.48	8.72	5855	{}	\N	2020-10-12 16:23:00.539967
75178	UBP	2020-09-29	8.53	8.6	8.26	8.49	16808	{}	\N	2020-10-12 16:23:00.551481
75179	UBP	2020-09-30	8.48	8.77	8.285	8.43	16004	{}	\N	2020-10-12 16:23:00.558887
75180	UBP	2020-10-01	8.37	8.56	8.36	8.5	37750	{}	\N	2020-10-12 16:23:00.566287
75181	UBP	2020-10-02	8.35	8.81	8.35	8.81	3310	{}	\N	2020-10-12 16:23:00.572117
75182	UBP	2020-10-05	8.57	8.8747	8.57	8.63	6758	{}	\N	2020-10-12 16:23:00.577498
75183	UBP	2020-10-06	8.7277	8.8601	8.66	8.66	4554	{}	\N	2020-10-12 16:23:00.582331
75184	UBP	2020-10-07	8.9	8.9001	8.7	8.7	8756	{}	\N	2020-10-12 16:23:00.587382
75185	UBP	2020-10-08	8.9599	8.96	8.84	8.8825	7368	{}	\N	2020-10-12 16:23:00.592118
75186	UBP	2020-10-09	9.02	9.11	9	9.11	5014	{}	\N	2020-10-12 16:23:00.597103
75187	UBP	2020-10-12	8.9421	9.1727	8.9421	9.1727	2588	{}	\N	2020-10-12 16:23:00.602118
75188	UCI	2020-09-28	12.92	13.45	12.92	13.45	10731	{}	\N	2020-10-12 16:23:01.809699
75189	UCI	2020-09-29	13.405	13.405	13.405	13.405	3	{}	\N	2020-10-12 16:23:01.821116
75190	UCI	2020-09-30	13.49	13.49	13.49	13.49	150	{}	\N	2020-10-12 16:23:01.827703
75191	UCI	2020-10-01	12.98	13.38	12.98	13.38	851	{}	\N	2020-10-12 16:23:01.833177
75192	UCI	2020-10-02	13.71	13.72	13.3	13.3	1614	{}	\N	2020-10-12 16:23:01.838359
75193	UCI	2020-10-05	13.244	13.37	13.244	13.37	517	{}	\N	2020-10-12 16:23:01.842914
75194	UCI	2020-10-06	13.17	13.17	13.17	13.17	2	{}	\N	2020-10-12 16:23:01.847883
75195	UCI	2020-10-07	13.49	13.49	13.49	13.49	450	{}	\N	2020-10-12 16:23:01.852439
75196	UCI	2020-10-08	13.58	13.58	13.58	13.58	470	{}	\N	2020-10-12 16:23:01.857229
75197	UCI	2020-10-09	13.665	13.665	13.665	13.665	170	{}	\N	2020-10-12 16:23:01.862315
75198	UCI	2020-10-12	13.74	13.749	13.7205	13.749	473	{}	\N	2020-10-12 16:23:01.867174
75199	TRGP	2020-09-28	14.48	14.81	14.255	14.65	1921341	{}	\N	2020-10-12 16:23:02.862169
75200	TRGP	2020-09-29	14.6	14.81	14.2983	14.62	5582239	{}	\N	2020-10-12 16:23:02.873777
75201	TRGP	2020-09-30	14.71	14.87	13.99	14.03	6891201	{}	\N	2020-10-12 16:23:02.879755
75202	TRGP	2020-10-01	13.86	14.05	13.335	13.58	6639722	{}	\N	2020-10-12 16:23:02.88505
75203	TRGP	2020-10-02	13.3	14.53	13.22	14.31	6734888	{}	\N	2020-10-12 16:23:02.890775
75204	TRGP	2020-10-05	15.38	15.95	15.19	15.85	8780064	{}	\N	2020-10-12 16:23:02.895596
75205	TRGP	2020-10-06	16.35	16.46	15.28	15.55	7802825	{}	\N	2020-10-12 16:23:02.90048
75206	TRGP	2020-10-07	15.56	16.37	15.56	16.17	6204559	{}	\N	2020-10-12 16:23:02.905111
75207	TRGP	2020-10-08	16.39	17.16	16.05	17.1	4853693	{}	\N	2020-10-12 16:23:02.909687
75208	TRGP	2020-10-09	17.35	17.54	16.7	16.77	5804186	{}	\N	2020-10-12 16:23:02.914252
75209	TRGP	2020-10-12	16.68	16.96	16.53	16.7981	2201010	{}	\N	2020-10-12 16:23:02.918797
75210	TRMB	2020-09-28	48.8	49.19	48.55	48.7	1012825	{}	\N	2020-10-12 16:23:03.867918
75211	TRMB	2020-09-29	48.62	49.21	48.22	48.92	982333	{}	\N	2020-10-12 16:23:03.880003
75212	TRMB	2020-09-30	49	49.85	48.49	48.7	1113204	{}	\N	2020-10-12 16:23:03.887796
75213	TRMB	2020-10-01	48.76	49.68	48.63	48.87	844551	{}	\N	2020-10-12 16:23:03.894016
75214	TRMB	2020-10-02	47.97	49.62	47.51	49.33	625164	{}	\N	2020-10-12 16:23:03.900126
75215	TRMB	2020-10-05	49.71	51.13	49.69	50.72	891284	{}	\N	2020-10-12 16:23:03.910727
75216	TRMB	2020-10-06	50.72	50.88	48.88	48.93	1172187	{}	\N	2020-10-12 16:23:03.916527
75217	TRMB	2020-10-07	49.72	50.67	49.51	50.51	1530556	{}	\N	2020-10-12 16:23:03.921655
75218	TRMB	2020-10-08	50.92	51.59	50.83	51.1	788661	{}	\N	2020-10-12 16:23:03.926882
75219	TRMB	2020-10-09	51.52	52.09	51.27	51.38	770953	{}	\N	2020-10-12 16:23:03.931779
75220	TRMB	2020-10-12	51.62	52.37	51.4	52.21	233676	{}	\N	2020-10-12 16:23:03.936852
75221	TRTN	2020-09-28	39.92	40.39	39.75	39.94	386001	{}	\N	2020-10-12 16:23:05.069254
75222	TRTN	2020-09-29	39.94	40.02	39.1	39.78	478705	{}	\N	2020-10-12 16:23:05.083242
75223	TRTN	2020-09-30	40.11	40.81	39.82	40.67	1194998	{}	\N	2020-10-12 16:23:05.092012
75224	TRTN	2020-10-01	37.74	38.2	37.25	37.8	6891327	{}	\N	2020-10-12 16:23:05.098847
75225	TRTN	2020-10-02	37.23	38	37.215	37.7	3789521	{}	\N	2020-10-12 16:23:05.104785
75226	TRTN	2020-10-05	37.97	38.25	37.87	38.19	8429520	{}	\N	2020-10-12 16:23:05.109975
75227	TRTN	2020-10-06	38.39	38.75	38.01	38.22	2829856	{}	\N	2020-10-12 16:23:05.115091
75228	TRTN	2020-10-07	38.47	39.01	37.67	38.84	3484675	{}	\N	2020-10-12 16:23:05.119799
75229	TRTN	2020-10-08	39.26	39.29	37.83	38.24	941220	{}	\N	2020-10-12 16:23:05.12411
75230	TRTN	2020-10-09	38.69	38.69	37.6	38.19	952736	{}	\N	2020-10-12 16:23:05.128734
75231	TRTN	2020-10-12	38.47	38.5	37.94	38.01	193761	{}	\N	2020-10-12 16:23:05.133588
75232	TSEM	2020-09-28	18.24	18.64	18.1391	18.5	169101	{}	\N	2020-10-12 16:23:06.142247
75233	TSEM	2020-09-29	18.36	18.64	18.29	18.42	171377	{}	\N	2020-10-12 16:23:06.154125
75234	TSEM	2020-09-30	18.45	18.62	18.1	18.22	265957	{}	\N	2020-10-12 16:23:06.161066
75235	TSEM	2020-10-01	18.39	18.74	18.39	18.63	125350	{}	\N	2020-10-12 16:23:06.167734
75236	TSEM	2020-10-02	18.25	18.57	18.21	18.3	146385	{}	\N	2020-10-12 16:23:06.173921
75237	TSEM	2020-10-05	18.46	18.79	18.46	18.745	131017	{}	\N	2020-10-12 16:23:06.179
75238	TSEM	2020-10-06	18.75	18.83	18.42	18.51	144829	{}	\N	2020-10-12 16:23:06.184354
75239	TSEM	2020-10-07	18.69	18.94	18.69	18.88	174731	{}	\N	2020-10-12 16:23:06.189367
75240	TSEM	2020-10-08	19.15	19.71	19.04	19.61	272198	{}	\N	2020-10-12 16:23:06.194403
75241	TSEM	2020-10-09	19.94	20.2	19.71	19.9	200409	{}	\N	2020-10-12 16:23:06.19931
75242	TSEM	2020-10-12	20.09	20.2	19.85	20.065	130014	{}	\N	2020-10-12 16:23:06.204357
75243	TSU	2020-09-28	12.28	12.28	11.5281	11.62	1088856	{}	\N	2020-10-12 16:23:07.583009
75244	TSU	2020-09-29	11.53	11.76	11.38	11.6	2365176	{}	\N	2020-10-12 16:23:07.595749
75245	TSU	2020-09-30	11.63	11.74	11.49	11.53	1477804	{}	\N	2020-10-12 16:23:07.602496
75246	TSU	2020-10-01	11.41	11.61	11.3	11.53	1141509	{}	\N	2020-10-12 16:23:07.607903
75247	TSU	2020-10-02	11.46	11.66	11.26	11.43	3430849	{}	\N	2020-10-12 16:23:07.612926
75248	TSU	2020-10-05	11.5	11.87	11.45	11.81	1327300	{}	\N	2020-10-12 16:23:07.617883
75249	TSU	2020-10-06	11.97	12.13	11.71	11.78	2027430	{}	\N	2020-10-12 16:23:07.622
75250	TSU	2020-10-07	11.67	11.81	11.42	11.68	2278611	{}	\N	2020-10-12 16:23:07.626247
75251	TSU	2020-10-08	11.57	12.18	11.55	12.12	3707098	{}	\N	2020-10-12 16:23:07.630434
75252	TSU	2020-10-09	12.01	12.43	11.94	12.24	2125973	{}	\N	2020-10-12 16:23:07.634505
75253	TSU	2020-10-12	12.33	12.3699	12.125	12.255	137735	{}	\N	2020-10-12 16:23:07.638664
75254	TTP	2020-09-28	12.15	12.35	12.03	12.32	8583	{}	\N	2020-10-12 16:23:08.931838
75255	TTP	2020-09-29	12.31	12.31	12.16	12.22	18728	{}	\N	2020-10-12 16:23:08.943158
75256	TTP	2020-09-30	12.24	12.395	11.94	11.94	17618	{}	\N	2020-10-12 16:23:08.950049
75257	TTP	2020-10-01	11.9	11.9	11.65	11.69	25758	{}	\N	2020-10-12 16:23:08.956364
75258	TTP	2020-10-02	11.39	11.8	11.12	11.765	44948	{}	\N	2020-10-12 16:23:08.962113
75259	TTP	2020-10-05	11.91	12.61	11.91	12.26	32918	{}	\N	2020-10-12 16:23:08.967723
75260	TTP	2020-10-06	12.46	12.5999	12.16	12.17	60834	{}	\N	2020-10-12 16:23:08.97272
75261	TTP	2020-10-07	12.19	12.53	12.0101	12.36	19032	{}	\N	2020-10-12 16:23:08.977644
75262	TTP	2020-10-08	12.51	13.06	12.47	13.05	20756	{}	\N	2020-10-12 16:23:08.982607
75263	TTP	2020-10-09	13.1	13.18	12.95	13.01	33270	{}	\N	2020-10-12 16:23:08.98768
75264	TTP	2020-10-12	13	13.038	13	13.0243	2584	{}	\N	2020-10-12 16:23:08.992871
75265	TUSK	2020-09-28	1.65	1.7462	1.57	1.57	66969	{}	\N	2020-10-12 16:23:09.96452
75266	TUSK	2020-09-29	1.58	1.6	1.44	1.54	108946	{}	\N	2020-10-12 16:23:09.976069
75267	TUSK	2020-09-30	1.55	1.63	1.53	1.6	82507	{}	\N	2020-10-12 16:23:09.982959
75268	TUSK	2020-10-01	1.57	1.61	1.52	1.58	37820	{}	\N	2020-10-12 16:23:09.989077
75269	TUSK	2020-10-02	1.52	1.63	1.51	1.59	59162	{}	\N	2020-10-12 16:23:09.994636
75270	TUSK	2020-10-05	1.6	1.73	1.6	1.68	127096	{}	\N	2020-10-12 16:23:09.999703
75271	TUSK	2020-10-06	1.76	1.9	1.7	1.7	97361	{}	\N	2020-10-12 16:23:10.004487
75272	TUSK	2020-10-07	1.73	1.92	1.7	1.86	73912	{}	\N	2020-10-12 16:23:10.009373
75273	TUSK	2020-10-08	1.87	1.94	1.7803	1.87	54857	{}	\N	2020-10-12 16:23:10.014319
75274	TUSK	2020-10-09	1.87	1.89	1.83	1.86	33680	{}	\N	2020-10-12 16:23:10.019373
75275	TUSK	2020-10-12	1.85	1.91	1.77	1.785	41164	{}	\N	2020-10-12 16:23:10.024205
75276	TWI	2020-09-28	2.83	3.08	2.8209	3	342444	{}	\N	2020-10-12 16:23:11.583902
75277	TWI	2020-09-29	3	3.04	2.7899	2.8	1423145	{}	\N	2020-10-12 16:23:11.595702
75278	TWI	2020-09-30	2.83	2.97	2.81	2.89	823221	{}	\N	2020-10-12 16:23:11.602666
75279	TWI	2020-10-01	2.66	2.83	2.66	2.83	1000762	{}	\N	2020-10-12 16:23:11.608634
75280	TWI	2020-10-02	2.71	2.94	2.54	2.85	994049	{}	\N	2020-10-12 16:23:11.614702
75281	TWI	2020-10-05	2.85	3.02	2.8301	2.88	530496	{}	\N	2020-10-12 16:23:11.619898
75282	TWI	2020-10-06	2.94	3	2.83	2.83	631104	{}	\N	2020-10-12 16:23:11.625235
75283	TWI	2020-10-07	2.91	2.9978	2.88	2.93	594647	{}	\N	2020-10-12 16:23:11.630687
75284	TWI	2020-10-08	3	3.25	2.989	3.24	1044212	{}	\N	2020-10-12 16:23:11.635847
75285	TWI	2020-10-09	3.28	3.36	3.04	3.12	759116	{}	\N	2020-10-12 16:23:11.641143
75286	TWI	2020-10-12	3.16	3.25	3.15	3.1802	164732	{}	\N	2020-10-12 16:23:11.646215
75287	TWOU	2020-09-28	35.66	36.38	34.78	35.7	1021354	{}	\N	2020-10-12 16:23:14.009016
75288	TWOU	2020-09-29	35.68	36.09	34.9	35.15	659493	{}	\N	2020-10-12 16:23:14.020941
75289	TWOU	2020-09-30	35.08	35.47	33.525	33.86	939851	{}	\N	2020-10-12 16:23:14.028526
75290	TWOU	2020-10-01	34.2	34.92	33.17	34.84	964125	{}	\N	2020-10-12 16:23:14.034611
75291	TWOU	2020-10-02	33.65	35	33.33	34.28	693139	{}	\N	2020-10-12 16:23:14.040072
75292	TWOU	2020-10-05	34.31	35.21	33.82	35.13	1465289	{}	\N	2020-10-12 16:23:14.04566
75293	TWOU	2020-10-06	34.9	36.88	34.9	36.03	1071586	{}	\N	2020-10-12 16:23:14.051418
75294	TWOU	2020-10-07	36.41	36.99	36.19	36.45	772802	{}	\N	2020-10-12 16:23:14.056748
75295	TWOU	2020-10-08	36.82	37.14	36.2	36.37	670347	{}	\N	2020-10-12 16:23:14.062255
75296	TWOU	2020-10-09	36.63	38.23	36.63	37.65	879653	{}	\N	2020-10-12 16:23:14.067197
75297	TWOU	2020-10-12	38.02	38.8	37.2411	37.67	301695	{}	\N	2020-10-12 16:23:14.07167
75298	TZOO	2020-09-28	6.32	6.56	6.32	6.46	87601	{}	\N	2020-10-12 16:23:16.316236
75299	TZOO	2020-09-29	6.45	6.58	6.38	6.54	52526	{}	\N	2020-10-12 16:23:16.328451
75300	TZOO	2020-09-30	6.6	6.73	6.31	6.43	64659	{}	\N	2020-10-12 16:23:16.335977
75301	TZOO	2020-10-01	6.5	6.6	6.36	6.55	54383	{}	\N	2020-10-12 16:23:16.342575
75302	TZOO	2020-10-02	6.25	6.63	6.211	6.49	38930	{}	\N	2020-10-12 16:23:16.348632
75303	TZOO	2020-10-05	6.62	6.62	6.37	6.43	49645	{}	\N	2020-10-12 16:23:16.353702
75304	TZOO	2020-10-06	6.51	6.62	6.25	6.5	33971	{}	\N	2020-10-12 16:23:16.358369
75305	TZOO	2020-10-07	6.6	6.68	6.4	6.58	79818	{}	\N	2020-10-12 16:23:16.363655
75306	TZOO	2020-10-08	6.65	6.86	6.54	6.83	52475	{}	\N	2020-10-12 16:23:16.368638
75307	TZOO	2020-10-09	6.88	6.88	6.67	6.77	19188	{}	\N	2020-10-12 16:23:16.373683
75308	TZOO	2020-10-12	6.74	6.77	6.6	6.705	26934	{}	\N	2020-10-12 16:23:16.378468
75309	UAUD	2020-09-28	15.25	15.3034	15.17	15.3034	448	{}	\N	2020-10-12 16:23:17.956004
75310	UAUD	2020-09-29	15.73	15.8037	15.61	15.8037	435	{}	\N	2020-10-12 16:23:17.964685
75311	UAUD	2020-09-30	16	16.185	16	16.1254	767	{}	\N	2020-10-12 16:23:17.971156
75312	UAUD	2020-10-01	16.23	16.37	16.23	16.36	2013	{}	\N	2020-10-12 16:23:17.977768
75313	UAUD	2020-10-02	16.0581	16.0581	16.0581	16.0581	12	{}	\N	2020-10-12 16:23:17.982705
75314	UAUD	2020-10-05	16.25	16.2841	16.17	16.2841	5761	{}	\N	2020-10-12 16:23:17.988004
75315	UAUD	2020-10-06	15.695	15.695	15.695	15.695	323	{}	\N	2020-10-12 16:23:17.993491
75316	UAUD	2020-10-07	15.86	15.86	15.8343	15.8343	567	{}	\N	2020-10-12 16:23:17.998909
75317	UAUD	2020-10-08	15.96	16.1047	15.955	16.1047	1644	{}	\N	2020-10-12 16:23:18.003674
75318	UAUD	2020-10-09	16.615	16.7529	16.615	16.7529	1895	{}	\N	2020-10-12 16:23:18.008608
75319	UBSI	2020-09-28	21.41	22.02	21.35	21.7	472402	{}	\N	2020-10-12 16:23:20.111719
75320	UBSI	2020-09-29	21.67	21.7881	20.77	21.195	543041	{}	\N	2020-10-12 16:23:20.118735
75321	UBSI	2020-09-30	21.26	21.68	21.16	21.47	621929	{}	\N	2020-10-12 16:23:20.125321
75322	UBSI	2020-10-01	21.57	22.23	21.19	22.2	1014265	{}	\N	2020-10-12 16:23:20.132013
75323	UBSI	2020-10-02	22.39	23.7	22.1275	23.5	998544	{}	\N	2020-10-12 16:23:20.137997
75324	UBSI	2020-10-05	23.73	24.3	23.675	24.23	542017	{}	\N	2020-10-12 16:23:20.143635
75325	UBSI	2020-10-06	24.58	25.15	24.02	24.12	733705	{}	\N	2020-10-12 16:23:20.149081
75326	UBSI	2020-10-07	24.31	25.09	24.31	24.75	549002	{}	\N	2020-10-12 16:23:20.15424
75327	UBSI	2020-10-08	25.01	25.01	24.59	24.81	562896	{}	\N	2020-10-12 16:23:20.160131
75328	UBSI	2020-10-09	25.09	25.28	24.26	24.56	464452	{}	\N	2020-10-12 16:23:20.166419
75329	UBSI	2020-10-12	24.46	24.91	24.38	24.67	120082	{}	\N	2020-10-12 16:23:20.172367
75330	UCO	2020-09-28	28.76	29.44	28.38	29.11	3315194	{}	\N	2020-10-12 16:23:21.939859
75331	UCO	2020-09-29	28.4325	28.47	26.515	27.25	6570782	{}	\N	2020-10-12 16:23:21.949932
75332	UCO	2020-09-30	27.78	28.84	27.66	28.45	4425360	{}	\N	2020-10-12 16:23:21.957015
75333	UCO	2020-10-01	27.44	27.6499	25.63	27	8530997	{}	\N	2020-10-12 16:23:21.96267
75334	UCO	2020-10-02	24.76	25.92	24.51	24.95	9893811	{}	\N	2020-10-12 16:23:21.968176
75335	UCO	2020-10-05	27.05	28	26.8	27.62	8266564	{}	\N	2020-10-12 16:23:21.972889
75336	UCO	2020-10-06	28.89	29.28	28.18	28.36	5803944	{}	\N	2020-10-12 16:23:21.977748
75337	UCO	2020-10-07	27.96	28.33	27.44	28.29	4520754	{}	\N	2020-10-12 16:23:21.982061
75338	UCO	2020-10-08	29.39	29.647	28.8901	29.55	5447218	{}	\N	2020-10-12 16:23:21.985907
75339	UCO	2020-10-09	29.33	29.74	28.5	28.74	3999466	{}	\N	2020-10-12 16:23:21.989742
75340	UCO	2020-10-12	28.21	28.22	27	27.595	2865157	{}	\N	2020-10-12 16:23:21.993803
75341	TWN	2020-09-28	22.95	23.5	22.95	23.5	359	{}	\N	2020-10-12 16:23:25.57923
75342	TWN	2020-09-29	23.34	23.4899	23.11	23.4	15199	{}	\N	2020-10-12 16:23:25.588206
75343	TWN	2020-09-30	23.4	23.4	22.9801	23.03	17644	{}	\N	2020-10-12 16:23:25.595037
75344	TWN	2020-10-01	23.72	23.74	23.4	23.535	20200	{}	\N	2020-10-12 16:23:25.601517
75345	TWN	2020-10-02	23.29	23.4545	23.2249	23.293	10030	{}	\N	2020-10-12 16:23:25.607362
75346	TWN	2020-10-05	23.75	23.75	23.35	23.4454	15686	{}	\N	2020-10-12 16:23:25.613066
75347	TWN	2020-10-06	23.51	24.3699	23.51	23.7593	4956	{}	\N	2020-10-12 16:23:25.618665
75348	TWN	2020-10-07	23.89	24.4192	23.89	24.31	10092	{}	\N	2020-10-12 16:23:25.625088
75349	TWN	2020-10-08	24.49	24.4999	24.29	24.29	8616	{}	\N	2020-10-12 16:23:25.630968
75350	TWN	2020-10-09	24.57	24.6699	24.57	24.652	3408	{}	\N	2020-10-12 16:23:25.637511
75351	TWN	2020-10-12	24.62	24.68	24.59	24.68	5548	{}	\N	2020-10-12 16:23:25.643308
75352	TYME	2020-09-28	0.93	0.986	0.93	0.9596	337916	{}	\N	2020-10-12 16:23:26.621875
75353	TYME	2020-09-29	0.971	0.98	0.9333	0.9421	331960	{}	\N	2020-10-12 16:23:26.635427
75354	TYME	2020-09-30	0.95	0.987099	0.95	0.98	346246	{}	\N	2020-10-12 16:23:26.643286
75355	TYME	2020-10-01	0.97	1.01	0.95	0.9953	302934	{}	\N	2020-10-12 16:23:26.65009
75356	TYME	2020-10-02	0.9899	0.99	0.9677	0.9719	189498	{}	\N	2020-10-12 16:23:26.656594
75357	TYME	2020-10-05	0.9672	1.01	0.9672	0.9909	258795	{}	\N	2020-10-12 16:23:26.661797
75358	TYME	2020-10-06	1	1.01	0.97	0.983	382082	{}	\N	2020-10-12 16:23:26.667248
75359	TYME	2020-10-07	0.98	1.01	0.97	0.9845	356941	{}	\N	2020-10-12 16:23:26.672516
75360	TYME	2020-10-08	0.9982	1.01	0.9705	0.9829	485912	{}	\N	2020-10-12 16:23:26.677974
75361	TYME	2020-10-09	1.01	1.01	0.986049	1.01	342452	{}	\N	2020-10-12 16:23:26.682892
75362	TYME	2020-10-12	1	1.01	0.997	1	70746	{}	\N	2020-10-12 16:23:26.687697
75363	UDOW	2020-09-28	75.86	77.4099	75.47	76.44	1628349	{}	\N	2020-10-12 16:23:28.982973
75364	UDOW	2020-09-29	76.2	76.465	74.26	75.13	2233593	{}	\N	2020-10-12 16:23:28.993605
75365	UDOW	2020-09-30	75.6526	79.915	75.6526	77.81	2749644	{}	\N	2020-10-12 16:23:29.000357
75366	UDOW	2020-10-01	79.32	80.15	76.951	78.3	2119915	{}	\N	2020-10-12 16:23:29.006456
75367	UDOW	2020-10-02	74.88	78.55	74.4115	77.13	2272460	{}	\N	2020-10-12 16:23:29.012595
75368	UDOW	2020-10-05	78.81	81.12	78.73	80.97	953909	{}	\N	2020-10-12 16:23:29.018199
75369	UDOW	2020-10-06	81.94	82.7796	77.35	77.67	2449380	{}	\N	2020-10-12 16:23:29.023509
75370	UDOW	2020-10-07	79.87	82.75	79.859	82.06	1269435	{}	\N	2020-10-12 16:23:29.028988
75371	UDOW	2020-10-08	83.55	83.63	81.9101	83.33	1287255	{}	\N	2020-10-12 16:23:29.034531
75372	UDOW	2020-10-09	84.44	85.52	83.4201	84.79	1748040	{}	\N	2020-10-12 16:23:29.040096
75373	UDOW	2020-10-12	85.69	87.311	85.38	87.235	608293	{}	\N	2020-10-12 16:23:29.045951
75374	UFCS	2020-09-28	19.98	20.91	19.98	20.68	91351	{}	\N	2020-10-12 16:23:31.03086
75375	UFCS	2020-09-29	20.56	20.56	19.89	20.36	53898	{}	\N	2020-10-12 16:23:31.042101
75376	UFCS	2020-09-30	20.36	20.61	20.05	20.32	94531	{}	\N	2020-10-12 16:23:31.049956
75377	UFCS	2020-10-01	20.23	20.56	20.01	20.25	60954	{}	\N	2020-10-12 16:23:31.056758
75378	UFCS	2020-10-02	19.91	20.625	19.91	20.32	72493	{}	\N	2020-10-12 16:23:31.062674
75379	UFCS	2020-10-05	20.5	21.07	20.02	20.77	59394	{}	\N	2020-10-12 16:23:31.068189
75380	UFCS	2020-10-06	20.71	21.32	20.505	20.52	86861	{}	\N	2020-10-12 16:23:31.074031
75381	UFCS	2020-10-07	20.48	20.85	20.3	20.79	69678	{}	\N	2020-10-12 16:23:31.079367
75382	UFCS	2020-10-08	21.03	21.56	20.79	21.41	58008	{}	\N	2020-10-12 16:23:31.084914
75383	UFCS	2020-10-09	21.68	21.68	21.19	21.26	46932	{}	\N	2020-10-12 16:23:31.08991
75384	UFCS	2020-10-12	21.4	22.08	21.4	21.915	24564	{}	\N	2020-10-12 16:23:31.094621
75385	UGE	2020-09-28	61.03	61.18	60.62	60.62	2470	{}	\N	2020-10-12 16:23:32.314552
75386	UGE	2020-09-29	60.78	60.81	60.1552	60.1552	3769	{}	\N	2020-10-12 16:23:32.3246
75387	UGE	2020-09-30	61.44	61.66	61.2	61.2	1309	{}	\N	2020-10-12 16:23:32.331092
75388	UGE	2020-10-01	61.88	62.9423	61.88	62.9423	628	{}	\N	2020-10-12 16:23:32.337071
75389	UGE	2020-10-02	61.77	62	61.6289	61.6289	686	{}	\N	2020-10-12 16:23:32.342731
75390	UGE	2020-10-05	62.67	62.8999	62.67	62.8999	1384	{}	\N	2020-10-12 16:23:32.348087
75391	UGE	2020-10-06	62.24	62.95	62.24	62.95	3222	{}	\N	2020-10-12 16:23:32.352891
75392	UGE	2020-10-07	63.7453	63.7453	63.7453	63.7453	63	{}	\N	2020-10-12 16:23:32.356801
75393	UGE	2020-10-08	64.47	64.47	64.4193	64.4193	951	{}	\N	2020-10-12 16:23:32.36093
75394	UGE	2020-10-09	65.51	65.79	65.51	65.67	3519	{}	\N	2020-10-12 16:23:32.365054
75395	UGE	2020-10-12	67.4	67.41	67.3999	67.41	877	{}	\N	2020-10-12 16:23:32.36936
75396	UITB	2020-09-28	54.18	54.2099	54.18	54.2099	8644	{}	\N	2020-10-12 16:23:35.260605
75397	UITB	2020-09-29	54.24	54.24	54.22	54.22	12901	{}	\N	2020-10-12 16:23:35.270642
75398	UITB	2020-09-30	54.16	54.18	54.11	54.13	28942	{}	\N	2020-10-12 16:23:35.277085
75399	UITB	2020-10-01	54.14	54.2417	54.14	54.22	567742	{}	\N	2020-10-12 16:23:35.282467
75400	UITB	2020-10-02	54.21	54.2334	54.2	54.225	6659	{}	\N	2020-10-12 16:23:35.288303
75401	UITB	2020-10-05	54.15	54.15	54.11	54.11	24365	{}	\N	2020-10-12 16:23:35.292942
75402	UITB	2020-10-06	54.12	54.1915	53.93	53.97	19238	{}	\N	2020-10-12 16:23:35.297651
75403	UITB	2020-10-07	54.04	54.0746	54	54.02	11616	{}	\N	2020-10-12 16:23:35.30232
75404	UITB	2020-10-08	54.09	54.36	54.0701	54.14	471047	{}	\N	2020-10-12 16:23:35.307598
75405	UITB	2020-10-09	54.13	54.13	54.04	54.11	6895	{}	\N	2020-10-12 16:23:35.313433
75406	UITB	2020-10-12	54.13	54.1698	54.13	54.15	5955	{}	\N	2020-10-12 16:23:35.319321
75407	ULVM	2020-09-28	48.07	48.48	48.07	48.32	13081	{}	\N	2020-10-12 16:23:37.057602
75408	ULVM	2020-09-29	48.3	48.46	48.08	48.3	6212	{}	\N	2020-10-12 16:23:37.068687
75409	ULVM	2020-09-30	48.43	48.7124	48.16	48.44	14934	{}	\N	2020-10-12 16:23:37.076957
75410	ULVM	2020-10-01	48.54	48.59	48.3825	48.59	2105	{}	\N	2020-10-12 16:23:37.08412
75411	ULVM	2020-10-02	48.51	48.79	48.2	48.63	6035	{}	\N	2020-10-12 16:23:37.090847
75412	ULVM	2020-10-05	49.35	49.5343	49.24	49.5343	6606	{}	\N	2020-10-12 16:23:37.096352
75413	ULVM	2020-10-06	49.68	49.8603	49	49.08	5960	{}	\N	2020-10-12 16:23:37.102052
75414	ULVM	2020-10-07	49.39	49.8142	49.39	49.76	3528	{}	\N	2020-10-12 16:23:37.107872
75415	ULVM	2020-10-08	50.12	50.2971	50.0901	50.2971	2346	{}	\N	2020-10-12 16:23:37.113055
75416	ULVM	2020-10-09	50.61	50.68	50.4009	50.47	3866	{}	\N	2020-10-12 16:23:37.118658
75417	ULVM	2020-10-12	50.7	50.93	50.68	50.92	2644	{}	\N	2020-10-12 16:23:37.124179
75418	UNH	2020-09-28	306.39	310.01	302.83	303.23	2593807	{}	\N	2020-10-12 16:23:38.547079
75419	UNH	2020-09-29	303.67	305.5	300.68	304.15	3607690	{}	\N	2020-10-12 16:23:38.560655
75420	UNH	2020-09-30	304.02	314.52	303.99	311.77	7003360	{}	\N	2020-10-12 16:23:38.567678
75421	UNH	2020-10-01	312.91	315.17	309.15	313.07	5309155	{}	\N	2020-10-12 16:23:38.574506
75422	UNH	2020-10-02	312.1	317.39	308.36	311.98	4556225	{}	\N	2020-10-12 16:23:38.580447
75423	UNH	2020-10-05	315.07	318.42	313.12	318.15	3276399	{}	\N	2020-10-12 16:23:38.585656
75424	UNH	2020-10-06	318.95	320.3	313.49	314.45	4750726	{}	\N	2020-10-12 16:23:38.59287
75425	UNH	2020-10-07	315.82	324.74	315.82	323.17	4888133	{}	\N	2020-10-12 16:23:38.597916
75426	UNH	2020-10-08	323.43	325.35	320.99	322.41	4036684	{}	\N	2020-10-12 16:23:38.603648
75427	UNH	2020-10-09	324.66	328.17	323.92	327.84	3973225	{}	\N	2020-10-12 16:23:38.609299
75428	UNH	2020-10-12	329.12	333.5619	329.01	332.17	2240361	{}	\N	2020-10-12 16:23:38.614629
75429	TTWO	2020-09-28	167.37	167.9498	164.82	167.46	932241	{}	\N	2020-10-12 16:23:40.225041
75430	TTWO	2020-09-29	166.6	167.35	163.905	164.34	1169737	{}	\N	2020-10-12 16:23:40.237941
75431	TTWO	2020-09-30	164.79	166.25	162.81	165.22	897181	{}	\N	2020-10-12 16:23:40.245998
75432	TTWO	2020-10-01	166.48	168.03	165.68	166.11	998037	{}	\N	2020-10-12 16:23:40.252861
75433	TTWO	2020-10-02	162.62	164.195	157.82	160.15	1183185	{}	\N	2020-10-12 16:23:40.259382
75434	TTWO	2020-10-05	161.927	164.96	161.18	164.76	843646	{}	\N	2020-10-12 16:23:40.264864
75435	TTWO	2020-10-06	164.9	167	161.4001	162.2	1030602	{}	\N	2020-10-12 16:23:40.270063
75436	TTWO	2020-10-07	162.56	163.53	159.485	160.96	969251	{}	\N	2020-10-12 16:23:40.275308
75437	TTWO	2020-10-08	162.46	162.915	159.07	160.42	894682	{}	\N	2020-10-12 16:23:40.280407
75438	TTWO	2020-10-09	161.13	162.7	160.76	161.64	600973	{}	\N	2020-10-12 16:23:40.28601
75439	TTWO	2020-10-12	164.94	166.23	163.93	165.65	583700	{}	\N	2020-10-12 16:23:40.291983
75440	TVE	2020-09-28	26.5	26.99	26.3968	26.8901	13692	{}	\N	2020-10-12 16:23:41.406595
75441	TVE	2020-09-29	26.87	26.97	26.708	26.97	18640	{}	\N	2020-10-12 16:23:41.418902
75442	TVE	2020-09-30	26.72	27.73	26.72	27.53	108496	{}	\N	2020-10-12 16:23:41.426693
75443	TVE	2020-10-01	27.9	27.9	27.16	27.54	102323	{}	\N	2020-10-12 16:23:41.433264
75444	TVE	2020-10-02	27.77	27.8	27.023	27.26	26551	{}	\N	2020-10-12 16:23:41.438942
75445	TVE	2020-10-05	26.5481	27.01	26.5481	26.9762	16444	{}	\N	2020-10-12 16:23:41.444314
75446	TVE	2020-10-06	26.98	27.4998	26.98	27.41	19196	{}	\N	2020-10-12 16:23:41.44915
75447	TVE	2020-10-07	27.2	27.2	26.98	27.09	21148	{}	\N	2020-10-12 16:23:41.454072
75448	TVE	2020-10-08	27.2	27.56	27.19	27.34	22466	{}	\N	2020-10-12 16:23:41.459891
75449	TVE	2020-10-09	27.39	27.39	27.2	27.22	12768	{}	\N	2020-10-12 16:23:41.464755
75450	TVE	2020-10-12	27.22	27.32	27.2	27.31	10158	{}	\N	2020-10-12 16:23:41.469564
75451	TWNK	2020-09-28	12.56	12.7194	12.4	12.53	561995	{}	\N	2020-10-12 16:23:42.987596
75452	TWNK	2020-09-29	12.53	12.56	12.285	12.32	665374	{}	\N	2020-10-12 16:23:42.998678
75453	TWNK	2020-09-30	12.36	12.51	12.26	12.33	911891	{}	\N	2020-10-12 16:23:43.005533
75454	TWNK	2020-10-01	12.3	12.37	11.965	12.33	3042959	{}	\N	2020-10-12 16:23:43.01169
75455	TWNK	2020-10-02	12.22	12.5	12.11	12.39	1025199	{}	\N	2020-10-12 16:23:43.017425
75456	TWNK	2020-10-05	12.44	12.55	12.24	12.47	807348	{}	\N	2020-10-12 16:23:43.022544
75457	TWNK	2020-10-06	12.53	12.645	12.39	12.51	969940	{}	\N	2020-10-12 16:23:43.027265
75458	TWNK	2020-10-07	12.53	12.69	12.47	12.6	689261	{}	\N	2020-10-12 16:23:43.032417
75459	TWNK	2020-10-08	12.425	12.67	12.395	12.54	1012585	{}	\N	2020-10-12 16:23:43.037499
75460	TWNK	2020-10-09	12.54	12.82	12.48	12.75	1389929	{}	\N	2020-10-12 16:23:43.043065
75461	TWNK	2020-10-12	12.77	12.98	12.7	12.97	377958	{}	\N	2020-10-12 16:23:43.048246
75462	TZA	2020-09-28	17.52	17.57	16.74	16.86	15554662	{}	\N	2020-10-12 16:23:45.043729
75463	TZA	2020-09-29	16.94	17.575	16.82	17.12	12307418	{}	\N	2020-10-12 16:23:45.052586
75464	TZA	2020-09-30	16.97	17.415	16.225	16.96	18373692	{}	\N	2020-10-12 16:23:45.059311
75465	TZA	2020-10-01	16.64	17.0701	16.15	16.16	15549456	{}	\N	2020-10-12 16:23:45.065175
75466	TZA	2020-10-02	17.05	17.1701	15.72	15.93	22275231	{}	\N	2020-10-12 16:23:45.07061
75467	TZA	2020-10-05	15.42	15.43	14.555	14.62	18658465	{}	\N	2020-10-12 16:23:45.077381
75468	TZA	2020-10-06	14.23	14.83	13.5201	14.74	31874018	{}	\N	2020-10-12 16:23:45.082426
75469	TZA	2020-10-07	14.19	14.2195	13.64	13.77	18547182	{}	\N	2020-10-12 16:23:45.086955
75470	TZA	2020-10-08	13.34	13.64	13.21	13.31	14865689	{}	\N	2020-10-12 16:23:45.091927
75471	TZA	2020-10-09	12.94	13.3299	12.86	13.11	14601319	{}	\N	2020-10-12 16:23:45.097236
75472	TZA	2020-10-12	12.94	13.1299	12.76	12.9047	6036506	{}	\N	2020-10-12 16:23:45.101981
75473	UCC	2020-09-28	70.06	71.2	70	71.06	8700	{}	\N	2020-10-12 16:23:47.133315
75474	UCC	2020-09-29	71.01	71.01	70.01	70.01	1256	{}	\N	2020-10-12 16:23:47.144045
75475	UCC	2020-09-30	70.55	71.7501	70.5402	71.19	1997	{}	\N	2020-10-12 16:23:47.151891
75476	UCC	2020-10-01	72.19	73.0623	72.19	73.0623	567	{}	\N	2020-10-12 16:23:47.157815
75477	UCC	2020-10-02	70.38	71.85	70.0002	71.59	11706	{}	\N	2020-10-12 16:23:47.163436
75478	UCC	2020-10-05	71.7	73.45	71.7	73.45	1202	{}	\N	2020-10-12 16:23:47.16841
75479	UCC	2020-10-06	73.45	73.75	70.9577	70.9577	3749	{}	\N	2020-10-12 16:23:47.17305
75480	UCC	2020-10-07	73.7646	73.7646	73.7646	73.7646	143	{}	\N	2020-10-12 16:23:47.177838
75481	UCC	2020-10-08	73.83	74.1308	73.83	74.1308	736	{}	\N	2020-10-12 16:23:47.182918
75482	UCC	2020-10-09	75.99	75.99	75.863	75.863	2240	{}	\N	2020-10-12 16:23:47.188037
75483	UCC	2020-10-12	77.49	78.55	76.27	78.44	3297	{}	\N	2020-10-12 16:23:47.192668
75484	UE	2020-09-28	9.8	10.19	9.66	10.15	945094	{}	\N	2020-10-12 16:23:48.382828
75485	UE	2020-09-29	10.15	10.15	9.54	9.78	1936266	{}	\N	2020-10-12 16:23:48.393789
75486	UE	2020-09-30	9.85	10.08	9.58	9.72	1847945	{}	\N	2020-10-12 16:23:48.402225
75487	UE	2020-10-01	9.69	10.2	9.69	10.19	2131160	{}	\N	2020-10-12 16:23:48.408839
75488	UE	2020-10-02	9.93	10.56	9.92	10.5	1334294	{}	\N	2020-10-12 16:23:48.415405
75489	UE	2020-10-05	10.59	10.7	10.26	10.53	1454144	{}	\N	2020-10-12 16:23:48.420898
75490	UE	2020-10-06	10.66	10.87	10.4	10.45	1238176	{}	\N	2020-10-12 16:23:48.425871
75491	UE	2020-10-07	10.51	10.67	10.43	10.47	1496394	{}	\N	2020-10-12 16:23:48.430797
75492	UE	2020-10-08	10.6	10.73	10.43	10.66	937799	{}	\N	2020-10-12 16:23:48.435971
75493	UE	2020-10-09	10.78	10.78	10.5	10.65	1035245	{}	\N	2020-10-12 16:23:48.440494
75494	UE	2020-10-12	10.59	10.76	10.44	10.65	326404	{}	\N	2020-10-12 16:23:48.445259
75495	UGI	2020-09-28	32.89	33.195	32.65	32.72	781124	{}	\N	2020-10-12 16:23:49.637852
75496	UGI	2020-09-29	32.81	33.12	32.43	32.49	1470877	{}	\N	2020-10-12 16:23:49.647008
75497	UGI	2020-09-30	32.7	33.1	32.63	32.98	2173962	{}	\N	2020-10-12 16:23:49.65368
75498	UGI	2020-10-01	33.02	33.245	32.395	32.64	1478412	{}	\N	2020-10-12 16:23:49.658864
75499	UGI	2020-10-02	32.07	33.48	31.97	33.39	1447328	{}	\N	2020-10-12 16:23:49.664545
75500	UGI	2020-10-05	33.79	33.98	33.45	33.5	1503223	{}	\N	2020-10-12 16:23:49.669215
75501	UGI	2020-10-06	33.77	34.8	33.64	34.07	2042951	{}	\N	2020-10-12 16:23:49.67387
75502	UGI	2020-10-07	34.25	34.45	33.59	34.1	1422565	{}	\N	2020-10-12 16:23:49.678329
75503	UGI	2020-10-08	34.31	35.19	34.12	35.18	1570516	{}	\N	2020-10-12 16:23:49.682957
75504	UGI	2020-10-09	35.34	35.53	34.53	34.84	1611243	{}	\N	2020-10-12 16:23:49.687826
75505	UGI	2020-10-12	34.77	35.04	34.58	34.86	368086	{}	\N	2020-10-12 16:23:49.69235
75506	UL	2020-09-28	61.89	62.25	61.66	61.73	861027	{}	\N	2020-10-12 16:23:50.605026
75507	UL	2020-09-29	61.77	62.04	61.58	61.91	1816657	{}	\N	2020-10-12 16:23:50.613602
75508	UL	2020-09-30	62.02	62.18	61.41	61.68	1294845	{}	\N	2020-10-12 16:23:50.620357
75509	UL	2020-10-01	61.87	62.2939	61.42	61.92	2079902	{}	\N	2020-10-12 16:23:50.626099
75510	UL	2020-10-02	62.11	62.91	62.02	62.69	1933150	{}	\N	2020-10-12 16:23:50.631678
75511	UL	2020-10-05	62.84	63.02	62.68	62.94	1180211	{}	\N	2020-10-12 16:23:50.636773
75512	UL	2020-10-06	62.52	62.56	61.55	61.73	1374355	{}	\N	2020-10-12 16:23:50.641523
75513	UL	2020-10-07	61.76	62.135	61.73	62.04	810787	{}	\N	2020-10-12 16:23:50.646197
75514	UL	2020-10-08	62.08	62.405	62.02	62.22	1293823	{}	\N	2020-10-12 16:23:50.651349
75515	UL	2020-10-09	62.69	63.32	62.6	63.12	2667669	{}	\N	2020-10-12 16:23:50.656077
75516	UL	2020-10-12	63.04	63.5	63.01	63.19	1292662	{}	\N	2020-10-12 16:23:50.661391
75517	UNF	2020-09-28	188.35	191.79	187.04	190.97	51977	{}	\N	2020-10-12 16:23:51.759219
75518	UNF	2020-09-29	190.66	191.46	189.49	190.74	89002	{}	\N	2020-10-12 16:23:51.769785
75519	UNF	2020-09-30	192.17	192.77	188.41	189.37	150486	{}	\N	2020-10-12 16:23:51.776743
75520	UNF	2020-10-01	189.23	192.27	188.22	189.48	130130	{}	\N	2020-10-12 16:23:51.782757
75521	UNF	2020-10-02	186.7	188.91	186.19	187.57	114076	{}	\N	2020-10-12 16:23:51.789414
75522	UNF	2020-10-05	188.54	191.05	187.19	189.32	93556	{}	\N	2020-10-12 16:23:51.794556
75523	UNF	2020-10-06	190.6	192.58	187.77	188.46	112316	{}	\N	2020-10-12 16:23:51.800041
75524	UNF	2020-10-07	189.46	191.44	187.51	190.45	123873	{}	\N	2020-10-12 16:23:51.805376
75525	UNF	2020-10-08	192.56	194.48	190.47	192.81	83219	{}	\N	2020-10-12 16:23:51.810606
75526	UNF	2020-10-09	194.61	197.38	193.32	196.9	116081	{}	\N	2020-10-12 16:23:51.815413
75527	UNF	2020-10-12	196.88	201.45	195.975	200.955	20856	{}	\N	2020-10-12 16:23:51.820598
75528	UPW	2020-09-28	49.29	50.0651	49.17	49.17	2955	{}	\N	2020-10-12 16:23:53.001928
75529	UPW	2020-09-29	49.3079	49.3079	49.3079	49.3079	146	{}	\N	2020-10-12 16:23:53.013174
75530	UPW	2020-09-30	49.44	50.2652	49.44	50.2652	373	{}	\N	2020-10-12 16:23:53.020689
75531	UPW	2020-10-01	50.9	50.9874	50.9	50.9874	264	{}	\N	2020-10-12 16:23:53.026517
75532	UPW	2020-10-02	50.2376	52.3366	50.2376	52.3366	2269	{}	\N	2020-10-12 16:23:53.03192
75533	UPW	2020-10-05	52.4727	53.3864	52.4727	53.3864	3404	{}	\N	2020-10-12 16:23:53.037344
75534	UPW	2020-10-06	53.545	55.7099	53.34	54.2902	3976	{}	\N	2020-10-12 16:23:53.042629
75535	UPW	2020-10-07	55	55.2124	55	55.2124	507	{}	\N	2020-10-12 16:23:53.048293
75536	UPW	2020-10-08	56.44	57.1298	56.09	57.1298	4533	{}	\N	2020-10-12 16:23:53.054363
75537	UPW	2020-10-09	56.61	57.24	56.45	56.76	3816	{}	\N	2020-10-12 16:23:53.060082
75538	UPW	2020-10-12	57.35	57.55	57.35	57.55	336	{}	\N	2020-10-12 16:23:53.065527
75539	URTH	2020-09-28	99.28	99.72	99.04	99.54	164567	{}	\N	2020-10-12 16:23:54.039606
75540	URTH	2020-09-29	99.57	99.7299	98.995	99.21	24531	{}	\N	2020-10-12 16:23:54.049974
75541	URTH	2020-09-30	99.17	100.27	99.17	99.58	22161	{}	\N	2020-10-12 16:23:54.057234
75542	URTH	2020-10-01	100.18	100.35	99.7	100.13	58184	{}	\N	2020-10-12 16:23:54.063249
75543	URTH	2020-10-02	98.59	99.89	98.59	99.4	40843	{}	\N	2020-10-12 16:23:54.068849
75544	URTH	2020-10-05	100.22	101.2	100.22	101.18	71067	{}	\N	2020-10-12 16:23:54.074168
75545	URTH	2020-10-06	101.22	101.698	99.78	99.97	43222	{}	\N	2020-10-12 16:23:54.078638
75546	URTH	2020-10-07	100.75	101.56	100.75	101.36	54496	{}	\N	2020-10-12 16:23:54.083194
75547	URTH	2020-10-08	101.87	102.2	101.77	102.11	40117	{}	\N	2020-10-12 16:23:54.088022
75548	URTH	2020-10-09	102.61	103.0001	102.53	102.9	29530	{}	\N	2020-10-12 16:23:54.093335
75549	URTH	2020-10-12	103.59	104.0999	103.55	104.0201	9451	{}	\N	2020-10-12 16:23:54.098984
75550	USAU	2020-09-28	9.35	9.77	9.35	9.54	25265	{}	\N	2020-10-12 16:23:55.070668
75551	USAU	2020-09-29	9.47	9.7469	9.18	9.27	38453	{}	\N	2020-10-12 16:23:55.082857
75552	USAU	2020-09-30	9.21	9.5029	9.14	9.2	21384	{}	\N	2020-10-12 16:23:55.090649
75553	USAU	2020-10-01	9.19	9.23	8.75	8.84	54299	{}	\N	2020-10-12 16:23:55.096598
75554	USAU	2020-10-02	8.72	9.05	8.65	8.8	32731	{}	\N	2020-10-12 16:23:55.102723
75555	USAU	2020-10-05	8.71	8.94	8.54	8.55	36092	{}	\N	2020-10-12 16:23:55.108166
75556	USAU	2020-10-06	8.6	8.85	8.6	8.76	28683	{}	\N	2020-10-12 16:23:55.113275
75557	USAU	2020-10-07	8.94	9.055	8.73	8.8	14106	{}	\N	2020-10-12 16:23:55.118564
75558	USAU	2020-10-08	8.9	9.0912	8.83	9.05	14134	{}	\N	2020-10-12 16:23:55.123926
75559	USAU	2020-10-09	9.07	9.75	9.06	9.61	48551	{}	\N	2020-10-12 16:23:55.129299
75560	USAU	2020-10-12	9.58	9.61	8.82	9.0597	42077	{}	\N	2020-10-12 16:23:55.134673
75561	TWLO	2020-09-28	247.1	249.0481	241.26	244.52	1665512	{}	\N	2020-10-12 16:23:56.488256
75562	TWLO	2020-09-29	245	251.755	243	248.8	2841608	{}	\N	2020-10-12 16:23:56.501964
75563	TWLO	2020-09-30	246.41	250.74	244.4454	247.09	3420516	{}	\N	2020-10-12 16:23:56.510699
75564	TWLO	2020-10-01	249.95	258	247.5	256.96	4447816	{}	\N	2020-10-12 16:23:56.517104
75565	TWLO	2020-10-02	276.95	295.52	272.5	290.36	27575510	{}	\N	2020-10-12 16:23:56.524079
75566	TWLO	2020-10-05	290	290.9	282.31	283	7812091	{}	\N	2020-10-12 16:23:56.53005
75567	TWLO	2020-10-06	283.58	298.12	282.31	294.51	11314737	{}	\N	2020-10-12 16:23:56.535413
75568	TWLO	2020-10-07	298	307.61	296.6	298.18	7058316	{}	\N	2020-10-12 16:23:56.540881
75569	TWLO	2020-10-08	303.05	306.64	297	299.1	3651930	{}	\N	2020-10-12 16:23:56.546423
75570	TWLO	2020-10-09	302	310.64	300.565	306.24	3868893	{}	\N	2020-10-12 16:23:56.551686
75571	TWLO	2020-10-12	322.68	328	311.5079	324.4894	7497439	{}	\N	2020-10-12 16:23:56.557345
75572	TXRH	2020-09-28	61.12	62.03	60.175	62.01	556278	{}	\N	2020-10-12 16:23:57.504364
75573	TXRH	2020-09-29	62.02	62.03	59.6	60.32	512948	{}	\N	2020-10-12 16:23:57.517038
75574	TXRH	2020-09-30	60.66	62.26	60.465	60.79	935610	{}	\N	2020-10-12 16:23:57.526385
75575	TXRH	2020-10-01	61.39	64.03	61.11	63.99	896686	{}	\N	2020-10-12 16:23:57.532533
75576	TXRH	2020-10-02	62.45	64.77	61.96	64.22	775573	{}	\N	2020-10-12 16:23:57.538428
75577	TXRH	2020-10-05	64.68	66.19	63.82	65.8	739965	{}	\N	2020-10-12 16:23:57.543491
75578	TXRH	2020-10-06	66.48	68.9	65.63	66.44	1315240	{}	\N	2020-10-12 16:23:57.54824
75579	TXRH	2020-10-07	67.51	68.76	66.8	68.65	924698	{}	\N	2020-10-12 16:23:57.55274
75580	TXRH	2020-10-08	69	70.09	68.56	69.76	778893	{}	\N	2020-10-12 16:23:57.557574
75581	TXRH	2020-10-09	70.17	70.644	69.36	70.34	638067	{}	\N	2020-10-12 16:23:57.562574
75582	TXRH	2020-10-12	70.75	73.16	70.52	71.85	677850	{}	\N	2020-10-12 16:23:57.56722
75583	UA	2020-09-28	9.99	10.13	9.9142	10.04	2765377	{}	\N	2020-10-12 16:23:59.559861
75584	UA	2020-09-29	9.97	9.99	9.62	9.77	5685501	{}	\N	2020-10-12 16:23:59.572744
75585	UA	2020-09-30	9.8	10.09	9.795	9.84	5388758	{}	\N	2020-10-12 16:23:59.581117
75586	UA	2020-10-01	10	10.46	9.93	10.35	7097612	{}	\N	2020-10-12 16:23:59.587253
75587	UA	2020-10-02	10.01	10.575	9.96	10.54	5463946	{}	\N	2020-10-12 16:23:59.593803
75588	UA	2020-10-05	10.7	10.76	10.52	10.59	4695456	{}	\N	2020-10-12 16:23:59.599372
75589	UA	2020-10-06	10.62	10.73	10.175	10.2	5446389	{}	\N	2020-10-12 16:23:59.604254
75590	UA	2020-10-07	10.37	10.68	10.34	10.61	4584747	{}	\N	2020-10-12 16:23:59.6095
75591	UA	2020-10-08	10.69	11.01	10.66	10.96	5827679	{}	\N	2020-10-12 16:23:59.614846
75592	UA	2020-10-09	11	11.09	10.735	10.97	5919745	{}	\N	2020-10-12 16:23:59.619785
75593	UA	2020-10-12	11.02	11.03	10.84	10.9399	2385430	{}	\N	2020-10-12 16:23:59.625092
75594	UBCP	2020-09-28	12.45	12.6	12.45	12.46	1370	{}	\N	2020-10-12 16:24:01.778854
75595	UBCP	2020-09-29	12.6	12.8655	12.445	12.445	6873	{}	\N	2020-10-12 16:24:01.79026
75596	UBCP	2020-09-30	12.16	12.56	12.16	12.56	3607	{}	\N	2020-10-12 16:24:01.797107
75597	UBCP	2020-10-01	12.56	12.6	12.07	12.26	12120	{}	\N	2020-10-12 16:24:01.802723
75598	UBCP	2020-10-02	12.38	12.7	12.38	12.605	4950	{}	\N	2020-10-12 16:24:01.80834
75599	UBCP	2020-10-05	12.65	12.75	12.25	12.36	4947	{}	\N	2020-10-12 16:24:01.813531
75600	UBCP	2020-10-06	12.475	13.32	12.475	12.88	3365	{}	\N	2020-10-12 16:24:01.817864
75601	UBCP	2020-10-07	12.8908	13.17	12.3401	12.3401	7953	{}	\N	2020-10-12 16:24:01.822327
75602	UBCP	2020-10-08	12.32	12.6	12.32	12.4	1467	{}	\N	2020-10-12 16:24:01.82734
75603	UBCP	2020-10-09	12.15	12.15	12.15	12.15	401	{}	\N	2020-10-12 16:24:01.832343
75604	UBCP	2020-10-12	12.4	12.95	12.3743	12.8709	2628	{}	\N	2020-10-12 16:24:01.837377
75605	UEIC	2020-09-28	36.48	38.07	36.48	37.67	51337	{}	\N	2020-10-12 16:24:03.770155
75606	UEIC	2020-09-29	37.53	37.8	36.895	37.61	46721	{}	\N	2020-10-12 16:24:03.780336
75607	UEIC	2020-09-30	37.78	38.37	37.31	37.74	88653	{}	\N	2020-10-12 16:24:03.787941
75608	UEIC	2020-10-01	37.76	38.35	37.2	38.06	53571	{}	\N	2020-10-12 16:24:03.794205
75609	UEIC	2020-10-02	37.41	37.69	36.58	37.52	68441	{}	\N	2020-10-12 16:24:03.800816
75610	UEIC	2020-10-05	37.78	37.99	37.01	37.97	55805	{}	\N	2020-10-12 16:24:03.805954
75611	UEIC	2020-10-06	37.88	39.11	37.21	38.05	90222	{}	\N	2020-10-12 16:24:03.811161
75612	UEIC	2020-10-07	38.18	38.99	38.05	38.78	53317	{}	\N	2020-10-12 16:24:03.816497
75613	UEIC	2020-10-08	39.43	39.44	38.75	39.29	34793	{}	\N	2020-10-12 16:24:03.821659
75614	UEIC	2020-10-09	39.77	40.665	39.11	39.38	38178	{}	\N	2020-10-12 16:24:03.826594
75615	UEIC	2020-10-12	39.43	39.93	38.77	39.61	41402	{}	\N	2020-10-12 16:24:03.832107
75616	UFPT	2020-09-28	39.88	40.83	38.795	40.27	16853	{}	\N	2020-10-12 16:24:04.764868
75617	UFPT	2020-09-29	40.72	41.98	40.32	41.3	24301	{}	\N	2020-10-12 16:24:04.776226
75618	UFPT	2020-09-30	41.8	42.52	41.41	41.42	27146	{}	\N	2020-10-12 16:24:04.783746
75619	UFPT	2020-10-01	41.035	41.52	39.02	40.56	19626	{}	\N	2020-10-12 16:24:04.789895
75620	UFPT	2020-10-02	39.73	40.5	39	40.16	14626	{}	\N	2020-10-12 16:24:04.795408
75621	UFPT	2020-10-05	40.41	40.96	39.95	40.09	7372	{}	\N	2020-10-12 16:24:04.800623
75622	UFPT	2020-10-06	40.49	41.67	39.56	40.24	14957	{}	\N	2020-10-12 16:24:04.80589
75623	UFPT	2020-10-07	40.33	41.16	39.81	41.08	12538	{}	\N	2020-10-12 16:24:04.810489
75624	UFPT	2020-10-08	41	41.49	40.54	41.03	6794	{}	\N	2020-10-12 16:24:04.815795
75625	UFPT	2020-10-09	41.47	42.73	41.065	42.73	44638	{}	\N	2020-10-12 16:24:04.820573
75626	UFPT	2020-10-12	42.57	43	41.9	41.9	4440	{}	\N	2020-10-12 16:24:04.824769
75627	UGP	2020-09-28	3.78	3.81	3.52	3.53	1204232	{}	\N	2020-10-12 16:24:06.95182
75628	UGP	2020-09-29	3.48	3.57	3.45	3.46	1607403	{}	\N	2020-10-12 16:24:06.963708
75629	UGP	2020-09-30	3.44	3.49	3.39	3.42	1802867	{}	\N	2020-10-12 16:24:06.971503
75630	UGP	2020-10-01	3.36	3.475	3.31	3.42	2003353	{}	\N	2020-10-12 16:24:06.977469
75631	UGP	2020-10-02	3.39	3.435	3.305	3.36	1484626	{}	\N	2020-10-12 16:24:06.983012
75632	UGP	2020-10-05	3.33	3.46	3.295	3.44	1796571	{}	\N	2020-10-12 16:24:06.988071
75633	UGP	2020-10-06	3.5	3.56	3.36	3.39	2325420	{}	\N	2020-10-12 16:24:06.992652
75634	UGP	2020-10-07	3.39	3.415	3.28	3.28	1601490	{}	\N	2020-10-12 16:24:06.997285
75635	UGP	2020-10-08	3.27	3.41	3.265	3.38	1484576	{}	\N	2020-10-12 16:24:07.002194
75636	UGP	2020-10-09	3.42	3.48	3.34	3.39	3397529	{}	\N	2020-10-12 16:24:07.006549
75637	UGP	2020-10-12	3.44	3.53	3.39	3.47	359592	{}	\N	2020-10-12 16:24:07.010673
75638	UJB	2020-09-28	67.6652	67.6652	67.6652	67.6652	337	{}	\N	2020-10-12 16:24:08.164526
75639	UJB	2020-09-29	67.665	67.665	67.665	67.665	20	{}	\N	2020-10-12 16:24:08.174766
75640	UJB	2020-09-30	68.25	68.455	68.25	68.455	1356	{}	\N	2020-10-12 16:24:08.183019
75641	UJB	2020-10-01	68.7599	68.7599	68.7599	68.7599	720	{}	\N	2020-10-12 16:24:08.190935
75642	UJB	2020-10-02	68.8016	68.8016	68.8016	68.8016	382	{}	\N	2020-10-12 16:24:08.197971
75643	UJB	2020-10-05	69.09	69.7139	69	69.7139	1546	{}	\N	2020-10-12 16:24:08.203969
75644	UJB	2020-10-06	70.32	70.32	69.39	69.39	1495	{}	\N	2020-10-12 16:24:08.209678
75645	UJB	2020-10-07	69.855	69.855	69.855	69.855	1061	{}	\N	2020-10-12 16:24:08.21524
75646	UJB	2020-10-09	70.45	70.45	70.45	70.45	315	{}	\N	2020-10-12 16:24:08.221353
75647	ULH	2020-09-28	20.33	20.97	20.33	20.86	33108	{}	\N	2020-10-12 16:24:09.141623
75648	ULH	2020-09-29	20.86	21.04	20.68	20.79	19265	{}	\N	2020-10-12 16:24:09.15192
75649	ULH	2020-09-30	20.93	21.64	20.51	20.86	63715	{}	\N	2020-10-12 16:24:09.159795
75650	ULH	2020-10-01	21	21.27	20.78	21.08	34596	{}	\N	2020-10-12 16:24:09.165936
75651	ULH	2020-10-02	20.74	21.3	20.6	21.23	20301	{}	\N	2020-10-12 16:24:09.172295
75652	ULH	2020-10-05	21.25	21.94	20.69	21.87	62139	{}	\N	2020-10-12 16:24:09.177515
75653	ULH	2020-10-06	21.91	22.06	21.2089	21.55	21605	{}	\N	2020-10-12 16:24:09.182956
75654	ULH	2020-10-07	21.98	22.02	21.47	21.93	16874	{}	\N	2020-10-12 16:24:09.188561
75655	ULH	2020-10-08	22.05	22.42	21.63	22.35	29423	{}	\N	2020-10-12 16:24:09.194255
75656	ULH	2020-10-09	22.43	22.44	22.01	22.27	14145	{}	\N	2020-10-12 16:24:09.199366
75657	ULH	2020-10-12	22.38	23.94	22.38	23.7	68182	{}	\N	2020-10-12 16:24:09.204696
75658	UMDD	2020-09-28	52.64	54.595	52.64	54.545	10130	{}	\N	2020-10-12 16:24:10.242422
75659	UMDD	2020-09-29	54.2	54.83	52.73	52.93	2670	{}	\N	2020-10-12 16:24:10.254703
75660	UMDD	2020-09-30	55	56.03	54.485	54.618	11295	{}	\N	2020-10-12 16:24:10.262674
75661	UMDD	2020-10-01	56.291	56.58	54.7089	56.58	6997	{}	\N	2020-10-12 16:24:10.269166
75662	UMDD	2020-10-02	53.57	58.62	53.57	58.39	24927	{}	\N	2020-10-12 16:24:10.27548
75663	UMDD	2020-10-05	59.99	62.34	59.99	62.34	16780	{}	\N	2020-10-12 16:24:10.281064
75664	UMDD	2020-10-06	63.35	65.5	61.02	61.32	36357	{}	\N	2020-10-12 16:24:10.286463
75665	UMDD	2020-10-07	63.4	65	63.4	64.73	16533	{}	\N	2020-10-12 16:24:10.29175
75666	UMDD	2020-10-08	66.54	66.54	65.6817	66.5103	4688	{}	\N	2020-10-12 16:24:10.297258
75667	UMDD	2020-10-09	68.51	68.51	67.0526	67.0526	16990	{}	\N	2020-10-12 16:24:10.302913
75668	UMDD	2020-10-12	67.83	68.49	67.24	68.23	4985	{}	\N	2020-10-12 16:24:10.309105
75669	UNIT	2020-09-28	9.94	10.0356	9.87	9.94	2047700	{}	\N	2020-10-12 16:24:12.618552
75670	UNIT	2020-09-29	9.94	10.33	9.93	10.17	1632631	{}	\N	2020-10-12 16:24:12.632155
75671	UNIT	2020-09-30	10.24	10.64	10.22	10.535	3623009	{}	\N	2020-10-12 16:24:12.63966
75672	UNIT	2020-10-01	10.66	10.69	10.32	10.55	1265001	{}	\N	2020-10-12 16:24:12.647034
75673	UNIT	2020-10-02	10.31	10.85	10.3	10.58	3831279	{}	\N	2020-10-12 16:24:12.654528
75674	UNIT	2020-10-05	10.66	10.72	10.01	10.33	1783824	{}	\N	2020-10-12 16:24:12.662638
75675	UNIT	2020-10-06	10.4	10.73	10.315	10.395	2202064	{}	\N	2020-10-12 16:24:12.668733
75676	UNIT	2020-10-07	10.44	10.59	10.39	10.41	6371745	{}	\N	2020-10-12 16:24:12.674752
75677	UNIT	2020-10-08	10.55	10.7	10.4	10.46	2653917	{}	\N	2020-10-12 16:24:12.68026
75678	UNIT	2020-10-09	10.5	10.58	10.35	10.49	1380611	{}	\N	2020-10-12 16:24:12.685353
75679	UNIT	2020-10-12	10.47	10.53	10.44	10.46	277457	{}	\N	2020-10-12 16:24:12.690851
75680	TWM	2020-09-28	9.21	9.23	8.951	8.99	2154624	{}	\N	2020-10-12 16:24:14.795882
75681	TWM	2020-09-29	9.01	9.2438	8.9758	9.09	2233269	{}	\N	2020-10-12 16:24:14.806653
75682	TWM	2020-09-30	9.01	9.19	8.77	9.02	3087404	{}	\N	2020-10-12 16:24:14.813493
75683	TWM	2020-10-01	8.92	9.07	8.74	8.75	1606477	{}	\N	2020-10-12 16:24:14.819057
75684	TWM	2020-10-02	9.06	9.11	8.59	8.66	3627225	{}	\N	2020-10-12 16:24:14.824506
75685	TWM	2020-10-05	8.46	8.471	8.16	8.18	1790359	{}	\N	2020-10-12 16:24:14.829897
75686	TWM	2020-10-06	8.04	8.2501	7.78	8.24	3850287	{}	\N	2020-10-12 16:24:14.835265
75687	TWM	2020-10-07	8.03	8.035	7.82	7.86	1460156	{}	\N	2020-10-12 16:24:14.841212
75688	TWM	2020-10-08	7.7	7.82	7.66	7.69	1495581	{}	\N	2020-10-12 16:24:14.847201
75689	TWM	2020-10-09	7.56	7.699	7.525	7.61	1818989	{}	\N	2020-10-12 16:24:14.852606
75690	TWM	2020-10-12	7.54	7.62	7.48	7.535	485561	{}	\N	2020-10-12 16:24:14.858233
75691	TXT	2020-09-28	36.07	37.165	36.07	36.81	941071	{}	\N	2020-10-12 16:24:16.29897
75692	TXT	2020-09-29	36.57	37.09	36.42	36.77	1923025	{}	\N	2020-10-12 16:24:16.310779
75693	TXT	2020-09-30	36.86	37.4	35.83	36.09	2197081	{}	\N	2020-10-12 16:24:16.317882
75694	TXT	2020-10-01	35.63	36.55	34.78	35.11	3014730	{}	\N	2020-10-12 16:24:16.323957
75695	TXT	2020-10-02	34.29	36.44	34.1783	35.72	3162072	{}	\N	2020-10-12 16:24:16.33026
75696	TXT	2020-10-05	36.26	36.73	36.04	36.4	1322579	{}	\N	2020-10-12 16:24:16.335683
75697	TXT	2020-10-06	36.8	37.075	35.65	35.72	1906156	{}	\N	2020-10-12 16:24:16.340196
75698	TXT	2020-10-07	36.21	36.59	35.92	36.44	2750876	{}	\N	2020-10-12 16:24:16.345522
75699	TXT	2020-10-08	36.77	37.24	36.26	37.02	2018146	{}	\N	2020-10-12 16:24:16.350313
75700	TXT	2020-10-09	37.49	37.66	36.69	36.76	1735281	{}	\N	2020-10-12 16:24:16.354736
75701	TXT	2020-10-12	36.66	36.73	36.3262	36.48	935667	{}	\N	2020-10-12 16:24:16.359347
75702	UAL	2020-09-28	35.32	36.9	34.6622	35.94	34729335	{}	\N	2020-10-12 16:24:17.474414
75703	UAL	2020-09-29	35.55	35.61	34.24	34.51	24901833	{}	\N	2020-10-12 16:24:17.48391
75704	UAL	2020-09-30	34.95	36.22	34.54	34.75	29177202	{}	\N	2020-10-12 16:24:17.49041
75705	UAL	2020-10-01	35.22	35.49	34.5665	35.18	18908867	{}	\N	2020-10-12 16:24:17.496705
75706	UAL	2020-10-02	33.36	36.77	33.27	36.01	35073412	{}	\N	2020-10-12 16:24:17.502116
75707	UAL	2020-10-05	36.43	36.5798	35.4201	36.2	17967975	{}	\N	2020-10-12 16:24:17.506783
75708	UAL	2020-10-06	36.74	37.09	34.84	34.88	28395156	{}	\N	2020-10-12 16:24:17.511606
75709	UAL	2020-10-07	36.51	37.22	35.88	36.38	27454170	{}	\N	2020-10-12 16:24:17.516494
75710	UAL	2020-10-08	37.18	37.69	35.85	37	28526527	{}	\N	2020-10-12 16:24:17.521244
75711	UAL	2020-10-09	37.0535	37.8	36.27	37.12	25347288	{}	\N	2020-10-12 16:24:17.525661
75712	UAL	2020-10-12	36.7	36.99	36.26	36.5401	11458287	{}	\N	2020-10-12 16:24:17.530659
75713	UBFO	2020-09-28	6.06	6.18	6.06	6.11	9476	{}	\N	2020-10-12 16:24:18.500203
75714	UBFO	2020-09-29	6.03	6.17	6.01	6.13	7690	{}	\N	2020-10-12 16:24:18.510557
75715	UBFO	2020-09-30	6.1	6.15	6.08	6.11	14934	{}	\N	2020-10-12 16:24:18.518292
75716	UBFO	2020-10-01	6.1	6.2	6.1	6.17	20381	{}	\N	2020-10-12 16:24:18.524724
75717	UBFO	2020-10-02	6.14	6.37	6.14	6.25	23765	{}	\N	2020-10-12 16:24:18.531094
75718	UBFO	2020-10-05	6.18	6.34	6.13	6.3	40812	{}	\N	2020-10-12 16:24:18.536121
75719	UBFO	2020-10-06	6.31	6.37	6.13	6.14	25458	{}	\N	2020-10-12 16:24:18.54077
75720	UBFO	2020-10-07	6.13	6.33	6.01	6.28	34594	{}	\N	2020-10-12 16:24:18.54565
75721	UBFO	2020-10-08	6.29	6.57	6.28	6.46	29089	{}	\N	2020-10-12 16:24:18.550305
75722	UBFO	2020-10-09	6.3643	6.65	6.1	6.62	20307	{}	\N	2020-10-12 16:24:18.555441
75723	UBFO	2020-10-12	6.58	6.58	6.5	6.5	527	{}	\N	2020-10-12 16:24:18.560059
75724	UBS	2020-09-28	10.94	11.12	10.91	11.07	3005033	{}	\N	2020-10-12 16:24:19.560239
75725	UBS	2020-09-29	11	11.05	10.915	11.01	5326800	{}	\N	2020-10-12 16:24:19.571634
75726	UBS	2020-09-30	11.13	11.28	11.09	11.15	6570860	{}	\N	2020-10-12 16:24:19.578275
75727	UBS	2020-10-01	11.23	11.255	11.1015	11.22	8828681	{}	\N	2020-10-12 16:24:19.584336
75728	UBS	2020-10-02	11.1	11.38	11.07	11.34	6473299	{}	\N	2020-10-12 16:24:19.589898
75729	UBS	2020-10-05	11.53	11.6	11.49	11.57	4826021	{}	\N	2020-10-12 16:24:19.595581
75730	UBS	2020-10-06	11.8	11.82	11.57	11.58	6702858	{}	\N	2020-10-12 16:24:19.600992
75731	UBS	2020-10-07	11.69	11.79	11.6601	11.75	4863553	{}	\N	2020-10-12 16:24:19.606403
75732	UBS	2020-10-08	11.81	11.82	11.7	11.78	4864437	{}	\N	2020-10-12 16:24:19.61149
75733	UBS	2020-10-09	11.8	11.8	11.5964	11.67	3990106	{}	\N	2020-10-12 16:24:19.616776
75734	UBS	2020-10-12	11.71	11.78	11.69	11.73	1470155	{}	\N	2020-10-12 16:24:19.621609
75735	UFAB	2020-09-28	3.49	3.49	3.2	3.35	23868	{}	\N	2020-10-12 16:24:21.109864
75736	UFAB	2020-09-29	3.3005	3.5	3.3005	3.4974	7192	{}	\N	2020-10-12 16:24:21.117487
75737	UFAB	2020-09-30	3.5	3.5	3.37	3.37	2998	{}	\N	2020-10-12 16:24:21.123346
75738	UFAB	2020-10-01	3.37	3.53	3.358	3.5	24697	{}	\N	2020-10-12 16:24:21.128625
75739	UFAB	2020-10-02	3.35	3.5	3.35	3.5	17585	{}	\N	2020-10-12 16:24:21.133767
75740	UFAB	2020-10-05	3.5	3.5	3.37	3.37	2905	{}	\N	2020-10-12 16:24:21.139359
75741	UFAB	2020-10-06	3.4	3.5	3.39	3.43	4085	{}	\N	2020-10-12 16:24:21.144579
75742	UFAB	2020-10-07	3.5	3.56	3.4369	3.5	14540	{}	\N	2020-10-12 16:24:21.150252
75743	UFAB	2020-10-08	3.51	3.63	3.51	3.63	11737	{}	\N	2020-10-12 16:24:21.15507
75744	UFAB	2020-10-09	3.55	4	3.55	3.94	35128	{}	\N	2020-10-12 16:24:21.159916
75745	UFAB	2020-10-12	3.83	3.99	3.83	3.9424	2185	{}	\N	2020-10-12 16:24:21.164709
75746	UGA	2020-09-28	20.26	20.65	20.1	20.56	48742	{}	\N	2020-10-12 16:24:22.085319
75747	UGA	2020-09-29	20.25	20.28	19.6	19.7653	43139	{}	\N	2020-10-12 16:24:22.095409
75748	UGA	2020-09-30	19.7	20.03	19.67	19.92	51067	{}	\N	2020-10-12 16:24:22.103621
75749	UGA	2020-10-01	19.56	19.68	19.05	19.52	153060	{}	\N	2020-10-12 16:24:22.109436
75750	UGA	2020-10-02	18.7	19.22	18.58	18.9947	154877	{}	\N	2020-10-12 16:24:22.114945
75751	UGA	2020-10-05	20.14	20.59	20.1129	20.29	195352	{}	\N	2020-10-12 16:24:22.120753
75752	UGA	2020-10-06	20.93	21.04	20.5818	20.65	131630	{}	\N	2020-10-12 16:24:22.126175
75753	UGA	2020-10-07	20.36	20.45	20.03	20.38	55344	{}	\N	2020-10-12 16:24:22.131337
75754	UGA	2020-10-08	20.81	20.98	20.75	20.9	131698	{}	\N	2020-10-12 16:24:22.136852
75755	UGA	2020-10-09	20.44	20.72	20.3204	20.36	97619	{}	\N	2020-10-12 16:24:22.142034
75756	UGA	2020-10-12	20.27	20.27	19.82	20.045	102731	{}	\N	2020-10-12 16:24:22.146763
75757	UHS	2020-09-28	106.72	108.95	103.81	104.41	1122568	{}	\N	2020-10-12 16:24:24.26372
75758	UHS	2020-09-29	103.21	104	99.96	102.53	1461004	{}	\N	2020-10-12 16:24:24.277833
75759	UHS	2020-09-30	103.47	108.61	103.325	107.02	1487797	{}	\N	2020-10-12 16:24:24.28523
75760	UHS	2020-10-01	107.12	108.1	104.57	106.2	1232831	{}	\N	2020-10-12 16:24:24.290822
75761	UHS	2020-10-02	104.09	109.97	103.36	109.15	933179	{}	\N	2020-10-12 16:24:24.295697
75762	UHS	2020-10-05	110.04	111.88	109.175	111.54	1100128	{}	\N	2020-10-12 16:24:24.300267
75763	UHS	2020-10-06	112.64	113.78	110.24	110.64	1146081	{}	\N	2020-10-12 16:24:24.304805
75764	UHS	2020-10-07	111.37	113.82	108.64	109.5	1399092	{}	\N	2020-10-12 16:24:24.308844
75765	UHS	2020-10-08	110.43	113.22	110.12	112.89	712845	{}	\N	2020-10-12 16:24:24.312672
75766	UHS	2020-10-09	120	120.01	111.51	111.73	1134637	{}	\N	2020-10-12 16:24:24.316815
75767	UHS	2020-10-12	111.24	111.78	109.5	109.76	340516	{}	\N	2020-10-12 16:24:24.321068
75768	UMBF	2020-09-28	48.38	49.72	47.98	49.54	180742	{}	\N	2020-10-12 16:24:26.624722
75769	UMBF	2020-09-29	49.26	50.175	48.24	48.68	138625	{}	\N	2020-10-12 16:24:26.636301
75770	UMBF	2020-09-30	49.17	50.18	48.68	49.01	211662	{}	\N	2020-10-12 16:24:26.644524
75771	UMBF	2020-10-01	48.83	49.65	48.58	49.21	208317	{}	\N	2020-10-12 16:24:26.650639
75772	UMBF	2020-10-02	48.21	51.48	47.85	51.24	184882	{}	\N	2020-10-12 16:24:26.656772
75773	UMBF	2020-10-05	51.8	53.23	51.755	53.14	146967	{}	\N	2020-10-12 16:24:26.662175
75774	UMBF	2020-10-06	53.94	56.3	53.15	54.05	267642	{}	\N	2020-10-12 16:24:26.668256
75775	UMBF	2020-10-07	54.54	56.02	54	55.64	188032	{}	\N	2020-10-12 16:24:26.673878
75776	UMBF	2020-10-08	56.47	57.06	55.79	56.48	106532	{}	\N	2020-10-12 16:24:26.678623
75777	UMBF	2020-10-09	57.06	57.18	56.065	56.34	86460	{}	\N	2020-10-12 16:24:26.682878
75778	UMBF	2020-10-12	56.07	57.1	55.9645	56.78	27516	{}	\N	2020-10-12 16:24:26.687585
75779	UNG	2020-09-28	12.23	12.63	12.16	12.48	4296127	{}	\N	2020-10-12 16:24:27.928229
75780	UNG	2020-09-29	11.75	11.83	11.26	11.29	8604413	{}	\N	2020-10-12 16:24:27.937807
75781	UNG	2020-09-30	11.4	11.575	11.15	11.41	3265368	{}	\N	2020-10-12 16:24:27.945323
75782	UNG	2020-10-01	11.38	11.415	10.9773	11.23	3363638	{}	\N	2020-10-12 16:24:27.950898
75783	UNG	2020-10-02	11.08	11.46	10.93	11.07	3308554	{}	\N	2020-10-12 16:24:27.956732
75784	UNG	2020-10-05	11.9	12.24	11.73	11.87	6332371	{}	\N	2020-10-12 16:24:27.961883
75785	UNG	2020-10-06	11.6	11.85	11.25	11.34	4812380	{}	\N	2020-10-12 16:24:27.966702
75786	UNG	2020-10-07	11.95	12.01	11.41	11.7	5413522	{}	\N	2020-10-12 16:24:27.971382
75787	UNG	2020-10-08	11.5	11.9003	11.39	11.86	5316298	{}	\N	2020-10-12 16:24:27.977219
75788	UNG	2020-10-09	12.48	12.67	12.24	12.38	4639380	{}	\N	2020-10-12 16:24:27.981668
75789	UNG	2020-10-12	13.09	13.15	12.82	12.9	2197249	{}	\N	2020-10-12 16:24:27.986144
75790	TY	2020-09-28	25.79	25.9455	25.79	25.83	89474	{}	\N	2020-10-12 16:24:29.812793
75791	TY	2020-09-29	25.87	25.893	25.58	25.82	125800	{}	\N	2020-10-12 16:24:29.819196
75792	TY	2020-09-30	25.68	26.14	25.68	25.93	200013	{}	\N	2020-10-12 16:24:29.826523
75793	TY	2020-10-01	26.08	26.17	25.97	26.07	112235	{}	\N	2020-10-12 16:24:29.833677
75794	TY	2020-10-02	25.81	26.08	25.81	26.08	131710	{}	\N	2020-10-12 16:24:29.84032
75795	TY	2020-10-05	26.17	26.53	26.1104	26.5	180292	{}	\N	2020-10-12 16:24:29.847081
75796	TY	2020-10-06	26.44	26.65	26.2739	26.36	189918	{}	\N	2020-10-12 16:24:29.852824
75797	TY	2020-10-07	26.48	26.85	26.48	26.74	154512	{}	\N	2020-10-12 16:24:29.858913
75798	TY	2020-10-08	26.86	26.99	26.62	26.99	140938	{}	\N	2020-10-12 16:24:29.865093
75799	TY	2020-10-09	27.14	27.26	27.03	27.21	167978	{}	\N	2020-10-12 16:24:29.873286
75800	TY	2020-10-12	27.21	27.33	27.0305	27.3013	41266	{}	\N	2020-10-12 16:24:29.880065
75801	UAMY	2020-09-28	0.315	0.32	0.28	0.2899	331089	{}	\N	2020-10-12 16:24:32.560426
75802	UAMY	2020-09-29	0.2899	0.3	0.2288	0.2438	2041520	{}	\N	2020-10-12 16:24:32.572491
75803	UAMY	2020-09-30	0.265	0.295	0.255	0.29	1170041	{}	\N	2020-10-12 16:24:32.57984
75804	UAMY	2020-10-01	0.2899	0.2899	0.2589	0.2669	514499	{}	\N	2020-10-12 16:24:32.586222
75805	UAMY	2020-10-02	0.25	0.28	0.2195	0.26	567390	{}	\N	2020-10-12 16:24:32.592653
75806	UAMY	2020-10-05	0.26	0.4322	0.2596	0.3645	6566050	{}	\N	2020-10-12 16:24:32.598691
75807	UAMY	2020-10-06	0.3518	0.38	0.31	0.3241	4609210	{}	\N	2020-10-12 16:24:32.603718
75808	UAMY	2020-10-07	0.3501	0.7344	0.35	0.58	54174717	{}	\N	2020-10-12 16:24:32.608773
75809	UAMY	2020-10-08	0.46	0.63	0.45	0.6079	13840246	{}	\N	2020-10-12 16:24:32.614146
75810	UAMY	2020-10-09	0.546	0.5501	0.401	0.42	4999862	{}	\N	2020-10-12 16:24:32.619436
75811	UAMY	2020-10-12	0.4	0.41248	0.365	0.3731	1763233	{}	\N	2020-10-12 16:24:32.624586
75812	UBOH	2020-09-28	19.05	21	18.76	21	6324	{}	\N	2020-10-12 16:24:34.758506
75813	UBOH	2020-09-29	20.72	22	20.36	22	3441	{}	\N	2020-10-12 16:24:34.772616
75814	UBOH	2020-09-30	21.75	21.75	21.49	21.5	903	{}	\N	2020-10-12 16:24:34.780806
75815	UBOH	2020-10-02	20.53	21.4	20.5	21.4	507	{}	\N	2020-10-12 16:24:34.78814
75816	UBOH	2020-10-05	20.7	20.7	20.7	20.7	215	{}	\N	2020-10-12 16:24:34.793964
75817	UBOH	2020-10-06	20.49	20.49	20.49	20.49	324	{}	\N	2020-10-12 16:24:34.799533
75818	UBOH	2020-10-07	20.84	20.84	19.51	19.55	3949	{}	\N	2020-10-12 16:24:34.805031
75819	UBOH	2020-10-08	19.35	19.61	18.3201	18.3742	2824	{}	\N	2020-10-12 16:24:34.810485
75820	UBOH	2020-10-09	18.82	20.14	18.82	19.55	3374	{}	\N	2020-10-12 16:24:34.815896
75821	UCHF	2020-09-28	22.8293	22.8293	22.8293	22.8293	33	{}	\N	2020-10-12 16:24:35.742062
75822	UCHF	2020-09-29	23.13	23.32	23.1277	23.3065	3087	{}	\N	2020-10-12 16:24:35.752131
75823	UCHF	2020-09-30	23.1385	23.5301	23.1	23.21	7321	{}	\N	2020-10-12 16:24:35.759856
75824	UCHF	2020-10-01	23.44	23.44	23.3958	23.3958	2734	{}	\N	2020-10-12 16:24:35.765587
75825	UCHF	2020-10-02	23.12	23.22	23.12	23.185	1586	{}	\N	2020-10-12 16:24:35.771423
75826	UCHF	2020-10-05	23.6902	23.6902	23.645	23.645	712	{}	\N	2020-10-12 16:24:35.77631
75827	UCHF	2020-10-06	23.67	23.67	23.5594	23.5594	800	{}	\N	2020-10-12 16:24:35.78158
75828	UCHF	2020-10-07	22.18	23.6	21.51	23.54	2138	{}	\N	2020-10-12 16:24:35.786773
75829	UCHF	2020-10-08	23.33	23.54	23.33	23.54	2810	{}	\N	2020-10-12 16:24:35.791484
75830	UCHF	2020-10-09	24.2467	24.2467	24.2467	24.2467	41	{}	\N	2020-10-12 16:24:35.795888
75831	UCHF	2020-10-12	24.345	24.345	24.345	24.345	243	{}	\N	2020-10-12 16:24:35.800812
75832	UDR	2020-09-28	33.01	33.38	32.59	33	1518423	{}	\N	2020-10-12 16:24:37.935806
75833	UDR	2020-09-29	32.86	33.05	32.13	32.32	2825716	{}	\N	2020-10-12 16:24:37.948639
75834	UDR	2020-09-30	32.61	32.845	32.18	32.61	6154949	{}	\N	2020-10-12 16:24:37.956008
75835	UDR	2020-10-01	32.61	33.81	32.56	33.8	5403208	{}	\N	2020-10-12 16:24:37.962011
75836	UDR	2020-10-02	33.34	34.9	32.86	34.74	4984620	{}	\N	2020-10-12 16:24:37.969314
75837	UDR	2020-10-05	34.97	35.13	34.195	35.11	4078085	{}	\N	2020-10-12 16:24:37.974433
75838	UDR	2020-10-06	35.28	35.67	34.78	35.06	4063524	{}	\N	2020-10-12 16:24:37.979371
75839	UDR	2020-10-07	35.17	35.51	34.63	35.21	5144667	{}	\N	2020-10-12 16:24:37.98465
75840	UDR	2020-10-08	35.01	35.79	34.89	35.74	5295834	{}	\N	2020-10-12 16:24:37.990404
75841	UDR	2020-10-09	35.97	35.97	35.23	35.47	2580989	{}	\N	2020-10-12 16:24:37.995552
75842	UDR	2020-10-12	35.43	35.47	34.97	35.11	933437	{}	\N	2020-10-12 16:24:38.001343
75843	UFI	2020-09-28	12.13	12.94	12.13	12.57	88836	{}	\N	2020-10-12 16:24:39.973506
75844	UFI	2020-09-29	12.56	12.88	12.26	12.59	442000	{}	\N	2020-10-12 16:24:39.984046
75845	UFI	2020-09-30	12.68	13.16	12.68	12.84	205149	{}	\N	2020-10-12 16:24:39.992503
75846	UFI	2020-10-01	12.73	13.345	12.73	13.27	177913	{}	\N	2020-10-12 16:24:39.998813
75847	UFI	2020-10-02	12.97	13.6005	12.97	13.51	148244	{}	\N	2020-10-12 16:24:40.005508
75848	UFI	2020-10-05	13.63	13.95	13.46	13.67	114888	{}	\N	2020-10-12 16:24:40.010679
75849	UFI	2020-10-06	13.8	14.11	13.52	13.58	519760	{}	\N	2020-10-12 16:24:40.01656
75850	UFI	2020-10-07	13.78	14.8	13.78	14.64	495020	{}	\N	2020-10-12 16:24:40.021975
75851	UFI	2020-10-08	14.77	14.94	14.11	14.36	124610	{}	\N	2020-10-12 16:24:40.02755
75852	UFI	2020-10-09	14.6	14.62	14.11	14.2	90431	{}	\N	2020-10-12 16:24:40.032836
75853	UFI	2020-10-12	14.12	14.21	13.9	14.085	40841	{}	\N	2020-10-12 16:24:40.038312
75854	UHAL	2020-09-28	356.12	364.33	354.53	356.98	34804	{}	\N	2020-10-12 16:24:41.437339
75855	UHAL	2020-09-29	358	361.56	351.66	351.98	29522	{}	\N	2020-10-12 16:24:41.451113
75856	UHAL	2020-09-30	353.98	364.26	350.32	355.98	35946	{}	\N	2020-10-12 16:24:41.460259
75857	UHAL	2020-10-01	356.14	358.06	352.88	355.33	32992	{}	\N	2020-10-12 16:24:41.468116
75858	UHAL	2020-10-02	349.99	362.45	349.99	361.95	33873	{}	\N	2020-10-12 16:24:41.475215
75859	UHAL	2020-10-05	362.09	368.75	359.75	361.62	31670	{}	\N	2020-10-12 16:24:41.480605
75860	UHAL	2020-10-06	361.71	368.1	355.02	363.65	31689	{}	\N	2020-10-12 16:24:41.485487
75861	UHAL	2020-10-07	362.75	373	362.75	363.9	47138	{}	\N	2020-10-12 16:24:41.490733
75862	UHAL	2020-10-08	364.02	365.27	360	362.13	41536	{}	\N	2020-10-12 16:24:41.49544
75863	UHAL	2020-10-09	363.29	365.11	358.57	360.16	35409	{}	\N	2020-10-12 16:24:41.499774
75864	UHAL	2020-10-12	362.46	363.39	360.7	361.19	4188	{}	\N	2020-10-12 16:24:41.504397
75865	ULBI	2020-09-28	5.86	6.41	5.86	6.35	31459	{}	\N	2020-10-12 16:24:42.857578
75866	ULBI	2020-09-29	6.4073	6.43	6	6.18	18952	{}	\N	2020-10-12 16:24:42.869053
75867	ULBI	2020-09-30	6.2	6.2	5.9	5.9	31085	{}	\N	2020-10-12 16:24:42.876255
75868	ULBI	2020-10-01	5.995	6.095	5.88	5.97	54310	{}	\N	2020-10-12 16:24:42.882569
75869	ULBI	2020-10-02	5.92	6.105	5.86	5.89	34837	{}	\N	2020-10-12 16:24:42.888388
75870	ULBI	2020-10-05	5.9	6.18	5.9	6.05	19166	{}	\N	2020-10-12 16:24:42.893797
75871	ULBI	2020-10-06	6.155	6.32	6.13	6.17	15244	{}	\N	2020-10-12 16:24:42.898503
75872	ULBI	2020-10-07	6.27	6.295	6.04	6.22	16798	{}	\N	2020-10-12 16:24:42.903269
75873	ULBI	2020-10-08	6.32	6.39	6.22	6.31	9078	{}	\N	2020-10-12 16:24:42.908329
75874	ULBI	2020-10-09	6.28	6.3537	6.1403	6.26	4427	{}	\N	2020-10-12 16:24:42.913319
75875	ULBI	2020-10-12	6.2465	6.41	6.2465	6.3735	2144	{}	\N	2020-10-12 16:24:42.918422
75876	UNB	2020-09-28	20.715	21.22	20.06	20.06	979	{}	\N	2020-10-12 16:24:43.877646
75877	UNB	2020-09-29	20.48	20.48	20.108	20.25	1814	{}	\N	2020-10-12 16:24:43.888119
75878	UNB	2020-09-30	20.45	20.9999	20.28	20.28	2760	{}	\N	2020-10-12 16:24:43.896017
75879	UNB	2020-10-01	20.5	20.6959	20.35	20.36	1095	{}	\N	2020-10-12 16:24:43.901934
75880	UNB	2020-10-02	20.55	21.21	20.55	21.21	6433	{}	\N	2020-10-12 16:24:43.90778
75881	UNB	2020-10-05	21.49	24.0799	21.48	22.62	13505	{}	\N	2020-10-12 16:24:43.912921
75882	UNB	2020-10-06	22.159	22.99	22.159	22.57	2355	{}	\N	2020-10-12 16:24:43.918603
75883	UNB	2020-10-07	22.49	23.35	22.3887	23.19	6121	{}	\N	2020-10-12 16:24:43.923614
75884	UNB	2020-10-08	23.2555	23.2555	22.57	23	5161	{}	\N	2020-10-12 16:24:43.928639
75885	UNB	2020-10-09	23.5	23.5	22.41	23	4867	{}	\N	2020-10-12 16:24:43.933534
75886	UNB	2020-10-12	23.23	23.49	23.145	23.49	1314	{}	\N	2020-10-12 16:24:43.93816
75887	UONE	2020-09-28	6.46	7.25	5.3	6.11	24044577	{}	\N	2020-10-12 16:24:44.893099
75888	UONE	2020-09-29	5.35	7.8	5.22	6.42	14199647	{}	\N	2020-10-12 16:24:44.904658
75889	UONE	2020-09-30	5.66	6.19	5.56	5.73	2932281	{}	\N	2020-10-12 16:24:44.912174
75890	UONE	2020-10-01	5.63	5.6608	4.85	4.89	1983054	{}	\N	2020-10-12 16:24:44.918608
75891	UONE	2020-10-02	4.65	5.28	4.55	5.15	1343559	{}	\N	2020-10-12 16:24:44.924882
75892	UONE	2020-10-05	5.04	5.3795	4.8501	5.05	755621	{}	\N	2020-10-12 16:24:44.930371
75893	UONE	2020-10-06	5	5.65	4.96	5.21	1945773	{}	\N	2020-10-12 16:24:44.935584
75894	UONE	2020-10-07	5.3	6.28	5.15	5.77	4286854	{}	\N	2020-10-12 16:24:44.940888
75895	UONE	2020-10-08	6	6.85	5.76	5.78	5484887	{}	\N	2020-10-12 16:24:44.946323
75896	UONE	2020-10-09	5.7	5.95	5.3801	5.49	1276735	{}	\N	2020-10-12 16:24:44.951436
75897	UONE	2020-10-12	5.5	5.58	5.17	5.25	420135	{}	\N	2020-10-12 16:24:44.956429
75898	URA	2020-09-28	11.2	11.35	11.05	11.14	119424	{}	\N	2020-10-12 16:24:46.798775
75899	URA	2020-09-29	11.19	11.26	11.1	11.19	94082	{}	\N	2020-10-12 16:24:46.811771
75900	URA	2020-09-30	11.17	11.24	11.1	11.13	156020	{}	\N	2020-10-12 16:24:46.818895
75901	URA	2020-10-01	11.06	11.1409	10.7	10.79	879280	{}	\N	2020-10-12 16:24:46.825344
75902	URA	2020-10-02	10.845	10.9	10.76	10.86	55097	{}	\N	2020-10-12 16:24:46.830656
75903	URA	2020-10-05	10.92	11.08	10.92	11	170859	{}	\N	2020-10-12 16:24:46.835746
75904	URA	2020-10-06	11.05	11.1	10.825	10.83	125780	{}	\N	2020-10-12 16:24:46.840944
75905	URA	2020-10-07	10.95	11.06	10.86	11	118230	{}	\N	2020-10-12 16:24:46.846545
75906	URA	2020-10-08	11.05	11.05	10.94	10.97	111212	{}	\N	2020-10-12 16:24:46.851971
75907	URA	2020-10-09	11	11.06	10.93	11.03	34015	{}	\N	2020-10-12 16:24:46.858633
75908	URA	2020-10-12	11.01	11.05	10.94	10.97	65026	{}	\N	2020-10-12 16:24:46.86471
75909	USA	2020-09-28	6	6	5.94	5.97	834516	{}	\N	2020-10-12 16:24:48.155554
75910	USA	2020-09-29	5.99	6	5.94	5.98	1111240	{}	\N	2020-10-12 16:24:48.163984
75911	USA	2020-09-30	5.99	6.07	5.98	6.01	1361200	{}	\N	2020-10-12 16:24:48.170045
75912	USA	2020-10-01	6.05	6.1	6.02	6.03	1078506	{}	\N	2020-10-12 16:24:48.175526
75913	USA	2020-10-02	5.97	6.03	5.9	5.98	1511386	{}	\N	2020-10-12 16:24:48.181241
75914	USA	2020-10-05	6.05	6.1	6.04	6.1	1452738	{}	\N	2020-10-12 16:24:48.186164
75915	USA	2020-10-06	6.09	6.145	6.02	6.03	1720941	{}	\N	2020-10-12 16:24:48.190937
75916	USA	2020-10-07	6.04	6.12	6.04	6.1	1220862	{}	\N	2020-10-12 16:24:48.196003
75917	USA	2020-10-08	6.15	6.19	6.12	6.17	1502229	{}	\N	2020-10-12 16:24:48.201013
75918	USA	2020-10-09	6.17	6.24	6.16	6.16	1705247	{}	\N	2020-10-12 16:24:48.206055
75919	USA	2020-10-12	6.21	6.24	6.2012	6.235	825404	{}	\N	2020-10-12 16:24:48.211415
75920	USCR	2020-09-28	27.44	28.88	27.415	28.3	304767	{}	\N	2020-10-12 16:24:49.092903
75921	USCR	2020-09-29	28.11	28.77	27.49	28.18	175426	{}	\N	2020-10-12 16:24:49.105997
75922	USCR	2020-09-30	28.26	29.99	27.7741	29.04	399937	{}	\N	2020-10-12 16:24:49.112357
75923	USCR	2020-10-01	29.13	29.815	28.47	28.88	208330	{}	\N	2020-10-12 16:24:49.120331
75924	USCR	2020-10-02	27.92	30.05	27.825	29.61	257342	{}	\N	2020-10-12 16:24:49.125695
75925	USCR	2020-10-05	30.23	31.8	29.93	31.59	362530	{}	\N	2020-10-12 16:24:49.130602
75926	USCR	2020-10-06	32.13	32.61	30.5	30.56	354184	{}	\N	2020-10-12 16:24:49.135189
75927	USCR	2020-10-07	30.92	33.08	30.5	32.38	337414	{}	\N	2020-10-12 16:24:49.139564
75928	USCR	2020-10-08	33.11	34.9588	33.1049	34.16	335930	{}	\N	2020-10-12 16:24:49.143778
75929	USCR	2020-10-09	34.4	34.69	33.05	33.47	264570	{}	\N	2020-10-12 16:24:49.149318
75930	USCR	2020-10-12	33.7	33.8647	33.1	33.425	78837	{}	\N	2020-10-12 16:24:49.154353
75931	UAN	2020-09-28	0.77	0.83	0.77	0.8222	141109	{}	\N	2020-10-12 16:24:51.079925
75932	UAN	2020-09-29	0.8	0.83	0.79	0.8093	182546	{}	\N	2020-10-12 16:24:51.091642
75933	UAN	2020-09-30	0.79	0.8679	0.79	0.8679	482168	{}	\N	2020-10-12 16:24:51.099018
75934	UAN	2020-10-01	0.86	0.86	0.7945	0.81055	253808	{}	\N	2020-10-12 16:24:51.105531
75935	UAN	2020-10-02	0.84	0.84	0.8	0.82	84857	{}	\N	2020-10-12 16:24:51.111077
75936	UAN	2020-10-05	0.85	0.85	0.7891	0.8	161590	{}	\N	2020-10-12 16:24:51.11658
75937	UAN	2020-10-06	0.79	0.82	0.78	0.8055	231188	{}	\N	2020-10-12 16:24:51.121871
75938	UAN	2020-10-07	0.8	0.8025	0.7863	0.8	186860	{}	\N	2020-10-12 16:24:51.126641
75939	UAN	2020-10-08	0.79	0.82	0.78	0.8151	280085	{}	\N	2020-10-12 16:24:51.13195
75940	UAN	2020-10-09	0.8	0.8285	0.7976	0.8136	176932	{}	\N	2020-10-12 16:24:51.137249
75941	UAN	2020-10-12	0.79	0.815	0.78	0.781	153934	{}	\N	2020-10-12 16:24:51.142497
75942	UCBI	2020-09-28	16.69	17.1	16.51	16.95	565030	{}	\N	2020-10-12 16:24:53.32929
75943	UCBI	2020-09-29	16.86	16.86	16.33	16.65	371311	{}	\N	2020-10-12 16:24:53.339831
75944	UCBI	2020-09-30	16.81	17.13	16.7	16.93	576085	{}	\N	2020-10-12 16:24:53.346785
75945	UCBI	2020-10-01	16.88	17.16	16.69	17.03	377172	{}	\N	2020-10-12 16:24:53.353019
75946	UCBI	2020-10-02	16.72	17.49	16.72	17.42	442253	{}	\N	2020-10-12 16:24:53.358745
75947	UCBI	2020-10-05	17.73	18.14	17.5	18.13	426668	{}	\N	2020-10-12 16:24:53.363705
75948	UCBI	2020-10-06	18.45	18.88	18.1272	18.15	554169	{}	\N	2020-10-12 16:24:53.369293
75949	UCBI	2020-10-07	18.32	18.8	18.28	18.61	502416	{}	\N	2020-10-12 16:24:53.375139
75950	UCBI	2020-10-08	18.85	19.08	18.69	19.01	476185	{}	\N	2020-10-12 16:24:53.380935
75951	UCBI	2020-10-09	19.08	19.17	18.31	19.015	433694	{}	\N	2020-10-12 16:24:53.387027
75952	UCBI	2020-10-12	18.99	19.1	18.74	18.99	91560	{}	\N	2020-10-12 16:24:53.392673
75953	UDN	2020-09-28	20.81	20.8397	20.79	20.83	119248	{}	\N	2020-10-12 16:24:54.744964
75954	UDN	2020-09-29	20.85	20.91	20.85	20.8801	119578	{}	\N	2020-10-12 16:24:54.756257
75955	UDN	2020-09-30	20.8531	20.94	20.81	20.8899	380971	{}	\N	2020-10-12 16:24:54.763156
75956	UDN	2020-10-01	20.9	20.94	20.8802	20.93	110237	{}	\N	2020-10-12 16:24:54.769689
75957	UDN	2020-10-02	20.88	20.92	20.88	20.9	73981	{}	\N	2020-10-12 16:24:54.775665
75958	UDN	2020-10-05	20.97	21.01	20.97	20.99	65648	{}	\N	2020-10-12 16:24:54.780263
75959	UDN	2020-10-06	21.04	21.04	20.91	20.93	98117	{}	\N	2020-10-12 16:24:54.785421
75960	UDN	2020-10-07	20.94	20.965	20.93	20.93	75261	{}	\N	2020-10-12 16:24:54.790802
75961	UDN	2020-10-08	20.94	20.9699	20.9	20.95	116600	{}	\N	2020-10-12 16:24:54.795869
75962	UDN	2020-10-09	21.04	21.1	21.04	21.1	48357	{}	\N	2020-10-12 16:24:54.800819
75963	UDN	2020-10-12	21.09	21.1	21.07	21.07	53600	{}	\N	2020-10-12 16:24:54.805909
75964	UFPI	2020-09-28	54.92	56.69	54.845	56.38	251816	{}	\N	2020-10-12 16:24:57.758826
75965	UFPI	2020-09-29	56.34	56.78	55.86	56.2	200426	{}	\N	2020-10-12 16:24:57.769162
75966	UFPI	2020-09-30	56.58	57.62	56.18	56.51	316099	{}	\N	2020-10-12 16:24:57.777119
75967	UFPI	2020-10-01	56.89	57.27	55.915	57.27	260785	{}	\N	2020-10-12 16:24:57.784704
75968	UFPI	2020-10-02	56.09	58.01	56.09	57.22	255513	{}	\N	2020-10-12 16:24:57.790621
75969	UFPI	2020-10-05	57.7	58.78	57.7	57.95	243956	{}	\N	2020-10-12 16:24:57.795927
75970	UFPI	2020-10-06	58.54	59.015	57.82	58.06	455739	{}	\N	2020-10-12 16:24:57.800978
75971	UFPI	2020-10-07	58.99	59.535	58.115	58.85	273670	{}	\N	2020-10-12 16:24:57.806598
75972	UFPI	2020-10-08	59.52	59.855	58.84	59.6	172258	{}	\N	2020-10-12 16:24:57.812512
75973	UFPI	2020-10-09	60	60.445	59.02	59.24	215676	{}	\N	2020-10-12 16:24:57.818708
75974	UFPI	2020-10-12	59.63	60.25	59.46	59.75	33454	{}	\N	2020-10-12 16:24:57.824214
75975	UIHC	2020-09-28	6.33	6.64	6.25	6.49	85978	{}	\N	2020-10-12 16:24:59.912558
75976	UIHC	2020-09-29	6.45	6.45	6.12	6.24	57232	{}	\N	2020-10-12 16:24:59.923039
75977	UIHC	2020-09-30	6.21	6.37	6.01	6.06	76380	{}	\N	2020-10-12 16:24:59.930047
75978	UIHC	2020-10-01	6.1	6.24	5.96	6.12	75176	{}	\N	2020-10-12 16:24:59.935792
75979	UIHC	2020-10-02	6	6.36	5.98	6.24	88095	{}	\N	2020-10-12 16:24:59.941283
75980	UIHC	2020-10-05	6.23	6.7	6.21	6.45	97128	{}	\N	2020-10-12 16:24:59.946501
75981	UIHC	2020-10-06	6.54	6.54	6.15	6.17	93809	{}	\N	2020-10-12 16:24:59.951122
75982	UIHC	2020-10-07	6.18	6.26	5.79	5.95	95521	{}	\N	2020-10-12 16:24:59.955976
75983	UIHC	2020-10-08	6.04	6.06	5.88	5.88	67388	{}	\N	2020-10-12 16:24:59.961423
75984	UIHC	2020-10-09	6.265	6.265	5.9	5.9	43012	{}	\N	2020-10-12 16:24:59.966468
75985	UIHC	2020-10-12	5.89	6.08	5.869	5.955	25958	{}	\N	2020-10-12 16:24:59.971429
75986	ULST	2020-09-28	40.42	40.69	40.41	40.49	356921	{}	\N	2020-10-12 16:25:00.979007
75987	ULST	2020-09-29	40.41	40.45	40.41	40.44	30447	{}	\N	2020-10-12 16:25:00.98993
75988	ULST	2020-09-30	40.42	40.45	40.42	40.43	50025	{}	\N	2020-10-12 16:25:00.997912
75989	ULST	2020-10-01	40.39	40.42	40.35	40.36	77815	{}	\N	2020-10-12 16:25:01.004529
75990	ULST	2020-10-02	40.42	40.42	40.388	40.392	32924	{}	\N	2020-10-12 16:25:01.011346
75991	ULST	2020-10-05	40.39	40.42	40.39	40.405	21660	{}	\N	2020-10-12 16:25:01.017444
75992	ULST	2020-10-06	40.39	40.45	40.39	40.42	355139	{}	\N	2020-10-12 16:25:01.02326
75993	ULST	2020-10-07	40.45	40.45	40.4082	40.44	57484	{}	\N	2020-10-12 16:25:01.031197
75994	ULST	2020-10-08	40.43	40.44	40.41	40.44	40551	{}	\N	2020-10-12 16:25:01.035638
75995	ULST	2020-10-09	40.43	40.47	40.42	40.44	42743	{}	\N	2020-10-12 16:25:01.040619
75996	ULST	2020-10-12	40.42	40.43	40.415	40.4284	14690	{}	\N	2020-10-12 16:25:01.045667
75997	UMPQ	2020-09-28	10.8	11.1	10.61	10.95	1961622	{}	\N	2020-10-12 16:25:02.093507
75998	UMPQ	2020-09-29	10.94	10.96	10.39	10.64	2435985	{}	\N	2020-10-12 16:25:02.105246
75999	UMPQ	2020-09-30	10.68	10.79	10.54	10.62	2015285	{}	\N	2020-10-12 16:25:02.11311
76000	UMPQ	2020-10-01	10.65	10.99	10.62	10.985	2010927	{}	\N	2020-10-12 16:25:02.118933
76001	UMPQ	2020-10-02	10.66	11.41	10.631	11.36	1484799	{}	\N	2020-10-12 16:25:02.123862
76002	UMPQ	2020-10-05	11.48	12.02	11.44	12	2195897	{}	\N	2020-10-12 16:25:02.129142
76003	UMPQ	2020-10-06	12.17	12.415	11.93	12	2177415	{}	\N	2020-10-12 16:25:02.133729
76004	UMPQ	2020-10-07	12.16	12.51	12.0501	12.46	1727666	{}	\N	2020-10-12 16:25:02.137946
76005	UMPQ	2020-10-08	12.81	12.92	12.5401	12.7	1021995	{}	\N	2020-10-12 16:25:02.142182
76006	UMPQ	2020-10-09	12.73	12.87	12.54	12.58	1538188	{}	\N	2020-10-12 16:25:02.146615
76007	UMPQ	2020-10-12	12.65	12.75	12.59	12.65	248557	{}	\N	2020-10-12 16:25:02.150723
76008	UNP	2020-09-28	201.59	202.2	199.13	199.45	2026704	{}	\N	2020-10-12 16:25:03.25857
76009	UNP	2020-09-29	199.59	200.38	196.65	197.24	4625683	{}	\N	2020-10-12 16:25:03.271387
76010	UNP	2020-09-30	198.13	202.2	196.315	196.87	7269379	{}	\N	2020-10-12 16:25:03.27775
76011	UNP	2020-10-01	198	198	192.1	194.23	6053839	{}	\N	2020-10-12 16:25:03.28363
76012	UNP	2020-10-02	191.03	198.59	190.87	196.35	6285833	{}	\N	2020-10-12 16:25:03.288721
76013	UNP	2020-10-05	198.2	202.25	197.45	201.32	6743032	{}	\N	2020-10-12 16:25:03.293508
76014	UNP	2020-10-06	202	204	199.29	199.41	6634116	{}	\N	2020-10-12 16:25:03.298138
76015	UNP	2020-10-07	201.08	205.58	201	204.62	4218764	{}	\N	2020-10-12 16:25:03.302572
76016	UNP	2020-10-08	204.89	205.5	201.56	203.78	3426858	{}	\N	2020-10-12 16:25:03.306698
76017	UNP	2020-10-09	204.64	206.48	204.55	205.78	3396543	{}	\N	2020-10-12 16:25:03.311152
76018	UNP	2020-10-12	208.68	208.819	205.455	206.16	1794046	{}	\N	2020-10-12 16:25:03.315699
76019	UPS	2020-09-28	169.91	171.98	168.49	168.9	4391065	{}	\N	2020-10-12 16:25:04.558337
76020	UPS	2020-09-29	168.81	170.465	167.115	168.5	6268854	{}	\N	2020-10-12 16:25:04.572001
76021	UPS	2020-09-30	169.29	169.67	165.5	166.63	10870190	{}	\N	2020-10-12 16:25:04.578827
76022	UPS	2020-10-01	169.04	170.8	166.88	167.65	9699499	{}	\N	2020-10-12 16:25:04.5849
76023	UPS	2020-10-02	166.66	168.5	165.75	166.96	8773893	{}	\N	2020-10-12 16:25:04.589857
76024	UPS	2020-10-05	168.16	169.96	167.44	169.16	5499912	{}	\N	2020-10-12 16:25:04.59503
76025	UPS	2020-10-06	169.28	172.05	168.72	169.35	7266513	{}	\N	2020-10-12 16:25:04.599464
76026	UPS	2020-10-07	170.61	175.44	170.305	174.99	7377318	{}	\N	2020-10-12 16:25:04.603642
76027	UPS	2020-10-08	175.55	175.9755	173.45	174.21	5573937	{}	\N	2020-10-12 16:25:04.607851
76028	UPS	2020-10-09	174.51	175.38	172.86	174.69	5704965	{}	\N	2020-10-12 16:25:04.612546
76029	UPS	2020-10-12	174.86	176.25	174.2101	175.77	1901944	{}	\N	2020-10-12 16:25:04.6172
76030	URGN	2020-09-28	19.51	19.61	18.36	18.81	169962	{}	\N	2020-10-12 16:25:05.899684
76031	URGN	2020-09-29	19.45	19.45	18.6	19.32	214014	{}	\N	2020-10-12 16:25:05.912035
76032	URGN	2020-09-30	19.38	20.1	19.16	19.29	171545	{}	\N	2020-10-12 16:25:05.920845
76033	URGN	2020-10-01	19.24	19.86	19.065	19.66	149294	{}	\N	2020-10-12 16:25:05.926854
76034	URGN	2020-10-02	19.22	19.565	19.1	19.23	89415	{}	\N	2020-10-12 16:25:05.93327
76035	URGN	2020-10-05	19.44	20.18	19.36	20.18	56081	{}	\N	2020-10-12 16:25:05.938628
76036	URGN	2020-10-06	20.33	20.82	20.22	20.33	48865	{}	\N	2020-10-12 16:25:05.943657
76037	URGN	2020-10-07	20.32	22.105	20.32	21.64	143811	{}	\N	2020-10-12 16:25:05.950561
76038	URGN	2020-10-08	21.8	22.2	21.47	21.88	156118	{}	\N	2020-10-12 16:25:05.956687
76039	URGN	2020-10-09	21.465	22.33	21.465	22.16	108934	{}	\N	2020-10-12 16:25:05.963786
76040	URGN	2020-10-12	22.17	22.65	21.48	22.375	20426	{}	\N	2020-10-12 16:25:05.970388
76041	USAP	2020-09-28	5.44	6.015	5.4	5.76	15776	{}	\N	2020-10-12 16:25:07.280904
76042	USAP	2020-09-29	5.805	6.005	5.61	5.64	5511	{}	\N	2020-10-12 16:25:07.294623
76043	USAP	2020-09-30	5.66	6	5.45	5.49	28816	{}	\N	2020-10-12 16:25:07.301859
76044	USAP	2020-10-01	5.86	5.86	5.47	5.51	18524	{}	\N	2020-10-12 16:25:07.307943
76045	USAP	2020-10-02	5.49	5.96	5.2	5.69	31737	{}	\N	2020-10-12 16:25:07.313634
76046	USAP	2020-10-05	5.865	6.05	5.48	5.48	28275	{}	\N	2020-10-12 16:25:07.319606
76047	USAP	2020-10-06	5.6	5.89	5.5	5.5	32659	{}	\N	2020-10-12 16:25:07.324262
76048	USAP	2020-10-07	5.56	6.045	5.485	5.6	27530	{}	\N	2020-10-12 16:25:07.329479
76049	USAP	2020-10-08	5.68	5.94	5.63	5.85	30886	{}	\N	2020-10-12 16:25:07.334174
76050	USAP	2020-10-09	5.89	5.9	5.745	5.8	7037	{}	\N	2020-10-12 16:25:07.338822
76051	USAP	2020-10-12	5.97	5.97	5.7571	5.7571	10205	{}	\N	2020-10-12 16:25:07.34316
76052	UBR	2020-09-28	22.05	22.05	19.9	19.93	4526	{}	\N	2020-10-12 16:25:08.273814
76053	UBR	2020-09-29	19.84	20.42	19.424	19.49	8343	{}	\N	2020-10-12 16:25:08.284591
76054	UBR	2020-09-30	19.96	20.245	19.94	20.245	2534	{}	\N	2020-10-12 16:25:08.292842
76055	UBR	2020-10-01	19.805	20.1541	19.572	20.1541	2990	{}	\N	2020-10-12 16:25:08.299163
76056	UBR	2020-10-02	20.575	20.575	19.42	19.42	4707	{}	\N	2020-10-12 16:25:08.305236
76057	UBR	2020-10-05	19.99	21.1181	19.99	21.1181	3288	{}	\N	2020-10-12 16:25:08.309941
76058	UBR	2020-10-06	21.62	22.045	20.5	20.85	3094	{}	\N	2020-10-12 16:25:08.314573
76059	UBR	2020-10-07	20.65	20.6899	20.5054	20.5054	2136	{}	\N	2020-10-12 16:25:08.319118
76060	UBR	2020-10-08	20.89	21.6	20.89	21.6	8434	{}	\N	2020-10-12 16:25:08.323489
76061	UBR	2020-10-09	21.8	22.5	21.8	22.089	5187	{}	\N	2020-10-12 16:25:08.327872
76062	UBR	2020-10-12	22.1708	22.355	22.1708	22.34	2493	{}	\N	2020-10-12 16:25:08.332263
76063	UEC	2020-09-28	1.03	1.05	1.01	1.01	926266	{}	\N	2020-10-12 16:25:09.96961
76064	UEC	2020-09-29	1.04	1.05	1.02	1.04	575425	{}	\N	2020-10-12 16:25:09.982152
76065	UEC	2020-09-30	1.02	1.05	0.9968	0.9968	1005824	{}	\N	2020-10-12 16:25:09.990742
76066	UEC	2020-10-01	1.01	1.02	0.9747	1	1448575	{}	\N	2020-10-12 16:25:09.997478
76067	UEC	2020-10-02	1	1.005	0.98	0.98	1042788	{}	\N	2020-10-12 16:25:10.003866
76068	UEC	2020-10-05	1	1.02	0.97	1	788935	{}	\N	2020-10-12 16:25:10.009844
76069	UEC	2020-10-06	1	1.09	0.99	0.9926	2937964	{}	\N	2020-10-12 16:25:10.015024
76070	UEC	2020-10-07	1	1.13	1	1.07	4729745	{}	\N	2020-10-12 16:25:10.020941
76071	UEC	2020-10-08	1.05	1.08	1.04	1.06	1162852	{}	\N	2020-10-12 16:25:10.026363
76072	UEC	2020-10-09	1.05	1.06	1.02	1.03	964687	{}	\N	2020-10-12 16:25:10.031332
76073	UEC	2020-10-12	1.02	1.03	1.01	1.014	594715	{}	\N	2020-10-12 16:25:10.036898
76074	UGBP	2020-09-28	16.1562	16.1562	16.1562	16.1562	20	{}	\N	2020-10-12 16:25:10.97416
76075	UGBP	2020-09-29	16.27	16.4104	16.14	16.26	4231	{}	\N	2020-10-12 16:25:10.985269
76076	UGBP	2020-09-30	16.2423	16.5457	16.2423	16.5457	798	{}	\N	2020-10-12 16:25:10.993397
76077	UGBP	2020-10-01	16.4	16.4063	16.4	16.4063	133	{}	\N	2020-10-12 16:25:10.999983
76078	UGBP	2020-10-02	16.68	16.68	16.6533	16.67	2635	{}	\N	2020-10-12 16:25:11.006426
76079	UGBP	2020-10-05	16.84	16.91	16.84	16.91	3298	{}	\N	2020-10-12 16:25:11.011876
76080	UGBP	2020-10-06	16.5129	16.5129	16.5129	16.5129	21	{}	\N	2020-10-12 16:25:11.01822
76081	UGBP	2020-10-07	16.51	16.5552	16.51	16.53	1853	{}	\N	2020-10-12 16:25:11.024121
76082	UGBP	2020-10-08	16.6	16.64	16.6	16.64	436	{}	\N	2020-10-12 16:25:11.029801
76083	UGBP	2020-10-09	17.1101	17.1798	17.11	17.1798	574	{}	\N	2020-10-12 16:25:11.035515
76084	UGBP	2020-10-12	17.18	17.28	17.18	17.28	2423	{}	\N	2020-10-12 16:25:11.041608
76085	UIS	2020-09-28	10.47	10.85	10.36	10.75	535297	{}	\N	2020-10-12 16:25:11.978912
76086	UIS	2020-09-29	10.79	10.795	10.365	10.6	912218	{}	\N	2020-10-12 16:25:11.989736
76087	UIS	2020-09-30	10.7	10.86	10.46	10.67	1309216	{}	\N	2020-10-12 16:25:11.997072
76088	UIS	2020-10-01	10.78	10.92	10.59	10.68	711215	{}	\N	2020-10-12 16:25:12.003294
76089	UIS	2020-10-02	10.4	10.8	10.35	10.68	594058	{}	\N	2020-10-12 16:25:12.009673
76090	UIS	2020-10-05	10.82	11.01	10.7	10.85	662084	{}	\N	2020-10-12 16:25:12.015289
76091	UIS	2020-10-06	10.96	11.15	10.77	10.77	781652	{}	\N	2020-10-12 16:25:12.021416
76092	UIS	2020-10-07	10.89	10.99	10.52	10.52	1061648	{}	\N	2020-10-12 16:25:12.027322
76093	UIS	2020-10-08	10.7	10.99	10.56	10.64	799736	{}	\N	2020-10-12 16:25:12.032903
76094	UIS	2020-10-09	10.76	10.89	10.61	10.69	489180	{}	\N	2020-10-12 16:25:12.039051
76095	UIS	2020-10-12	10.69	10.83	10.6101	10.69	209011	{}	\N	2020-10-12 16:25:12.044852
76096	ULTA	2020-09-28	221.69	227.93	219.07	225.74	739417	{}	\N	2020-10-12 16:25:13.34499
76097	ULTA	2020-09-29	224.96	225.925	218.48	222.88	659572	{}	\N	2020-10-12 16:25:13.357875
76098	ULTA	2020-09-30	223.43	229.13	222.16	223.98	730653	{}	\N	2020-10-12 16:25:13.365865
76099	ULTA	2020-10-01	226.96	229.51	224.54	228.63	679844	{}	\N	2020-10-12 16:25:13.373523
76100	ULTA	2020-10-02	223.94	230.98	222.975	228.68	911912	{}	\N	2020-10-12 16:25:13.380286
76101	ULTA	2020-10-05	230.48	234.71	227.71	233.78	756818	{}	\N	2020-10-12 16:25:13.385767
76102	ULTA	2020-10-06	234.95	235.92	225.02	225.58	860188	{}	\N	2020-10-12 16:25:13.391568
76103	ULTA	2020-10-07	229.1705	232.32	226.49	229.49	894680	{}	\N	2020-10-12 16:25:13.396756
76104	ULTA	2020-10-08	232.56	238.2704	231.67	237.42	666467	{}	\N	2020-10-12 16:25:13.4018
76105	ULTA	2020-10-09	239.34	240.64	232.78	233.55	688073	{}	\N	2020-10-12 16:25:13.406809
76106	ULTA	2020-10-12	235.45	242.78	234.06	240.53	498348	{}	\N	2020-10-12 16:25:13.411896
76107	UNAM	2020-09-28	6	6	6	6	378	{}	\N	2020-10-12 16:25:16.790132
76108	UNAM	2020-10-01	5.2	5.91	5.2	5.91	969	{}	\N	2020-10-12 16:25:16.799415
76109	UNAM	2020-10-02	5.4001	5.4001	5.4001	5.4001	385	{}	\N	2020-10-12 16:25:16.806766
76110	UNAM	2020-10-07	5.6348	5.6348	5.6348	5.6348	1118	{}	\N	2020-10-12 16:25:16.812088
76111	UNAM	2020-10-09	5.53	5.53	5.5	5.5	600	{}	\N	2020-10-12 16:25:16.81716
76112	UNTY	2020-09-28	11.1	11.42	11.02	11.14	26610	{}	\N	2020-10-12 16:25:19.566828
76113	UNTY	2020-09-29	11.04	11.31	10.56	11.27	11038	{}	\N	2020-10-12 16:25:19.577465
76114	UNTY	2020-09-30	11.08	11.61	11.08	11.58	5223	{}	\N	2020-10-12 16:25:19.584839
76115	UNTY	2020-10-01	11.65	11.9109	11.65	11.8	4756	{}	\N	2020-10-12 16:25:19.59049
76116	UNTY	2020-10-02	11.55	12.2806	11.4069	12.07	15114	{}	\N	2020-10-12 16:25:19.596356
76117	UNTY	2020-10-05	12.21	12.51	12.08	12.51	13489	{}	\N	2020-10-12 16:25:19.601413
76118	UNTY	2020-10-06	12.6	12.7737	11.96	11.96	14503	{}	\N	2020-10-12 16:25:19.60635
76119	UNTY	2020-10-07	12	12.8	12	12.74	30588	{}	\N	2020-10-12 16:25:19.611206
76120	UNTY	2020-10-08	12.9	13.18	12.54	13.12	16936	{}	\N	2020-10-12 16:25:19.616379
76121	UNTY	2020-10-09	13.24	13.24	12.81	12.89	2794	{}	\N	2020-10-12 16:25:19.621471
76122	UNTY	2020-10-12	12.85	13.215	12.85	13.215	1790	{}	\N	2020-10-12 16:25:19.626098
76123	URI	2020-09-28	172.13	178.6406	170.99	175.73	755494	{}	\N	2020-10-12 16:25:20.578133
76124	URI	2020-09-29	174.89	176.62	171.5	174.21	1180127	{}	\N	2020-10-12 16:25:20.590743
76125	URI	2020-09-30	175.56	179.45	171.75	174.5	1590792	{}	\N	2020-10-12 16:25:20.597622
76126	URI	2020-10-01	176.58	176.58	171.88	175.35	1741452	{}	\N	2020-10-12 16:25:20.603934
76127	URI	2020-10-02	171.75	186.24	171.75	185.02	2227680	{}	\N	2020-10-12 16:25:20.609974
76128	URI	2020-10-05	187.2	198.62	186.6	197.7	3086702	{}	\N	2020-10-12 16:25:20.614903
76129	URI	2020-10-06	197.97	200.65	193.47	193.87	2283556	{}	\N	2020-10-12 16:25:20.619896
76130	URI	2020-10-07	197.82	201.51	196.08	198.96	1499790	{}	\N	2020-10-12 16:25:20.624462
76131	URI	2020-10-08	201	201.9977	197.04	199.49	1182630	{}	\N	2020-10-12 16:25:20.62907
76132	URI	2020-10-09	201.15	203.57	195.93	196.94	1615889	{}	\N	2020-10-12 16:25:20.633812
76133	URI	2020-10-12	197.54	200.36	194.18	200.215	524042	{}	\N	2020-10-12 16:25:20.639154
76134	USEQ	2020-09-28	26.4703	26.4703	26.4703	26.4703	164	{}	\N	2020-10-12 16:25:22.072906
76135	USEQ	2020-09-29	26.322	26.322	26.322	26.322	105	{}	\N	2020-10-12 16:25:22.081513
76136	USEQ	2020-09-30	26.4879	26.4879	26.4879	26.4879	103	{}	\N	2020-10-12 16:25:22.088455
76137	USEQ	2020-10-01	26.6047	26.6047	26.6047	26.6047	167	{}	\N	2020-10-12 16:25:22.094275
76138	USEQ	2020-10-02	26.8234	26.8234	26.8234	26.8234	101	{}	\N	2020-10-12 16:25:22.099156
76139	USEQ	2020-10-05	27.08	27.2826	27.08	27.2826	349	{}	\N	2020-10-12 16:25:22.103737
76140	USEQ	2020-10-06	27.1193	27.1193	27.1193	27.1193	100	{}	\N	2020-10-12 16:25:22.108259
76141	USEQ	2020-10-07	27.5471	27.5471	27.5471	27.5471	140	{}	\N	2020-10-12 16:25:22.112957
76142	USEQ	2020-10-08	27.8631	27.8631	27.8631	27.8631	100	{}	\N	2020-10-12 16:25:22.118073
76143	USEQ	2020-10-09	27.9204	27.9204	27.9204	27.9204	100	{}	\N	2020-10-12 16:25:22.123563
76144	USMF	2020-09-28	31.105	31.25	31.105	31.25	13024	{}	\N	2020-10-12 16:25:23.274189
76145	USMF	2020-09-29	31.22	31.23	31.138	31.23	3118	{}	\N	2020-10-12 16:25:23.285646
76146	USMF	2020-09-30	31.35	31.48	31.1182	31.29	3184	{}	\N	2020-10-12 16:25:23.292084
76147	USMF	2020-10-01	31.52	31.56	31.37	31.49	12064	{}	\N	2020-10-12 16:25:23.297573
76148	USMF	2020-10-02	31.06	31.456	31.06	31.44	12129	{}	\N	2020-10-12 16:25:23.303011
76149	USMF	2020-10-05	31.67	31.94	31.67	31.94	9856	{}	\N	2020-10-12 16:25:23.308051
76150	USMF	2020-10-06	31.9784	32.23	31.72	31.72	34182	{}	\N	2020-10-12 16:25:23.312557
76151	USMF	2020-10-07	31.94	32.2304	31.94	32.2304	3556	{}	\N	2020-10-12 16:25:23.317789
76152	USMF	2020-10-08	32.35	32.3807	32.27	32.3807	17437	{}	\N	2020-10-12 16:25:23.322502
76153	USMF	2020-10-09	32.5392	32.6899	32.5392	32.62	13514	{}	\N	2020-10-12 16:25:23.327044
76154	USMF	2020-10-12	32.8083	32.8786	32.8083	32.8786	827	{}	\N	2020-10-12 16:25:23.331649
76155	USPH	2020-09-28	85.24	87.37	85.24	85.89	41370	{}	\N	2020-10-12 16:25:26.933166
76156	USPH	2020-09-29	85.57	86.45	83.61	85.7	122784	{}	\N	2020-10-12 16:25:26.944805
76157	USPH	2020-09-30	86.04	88.31	86.04	86.88	190259	{}	\N	2020-10-12 16:25:26.952019
76158	USPH	2020-10-01	86.99	88.94	86.37	86.76	168525	{}	\N	2020-10-12 16:25:26.958484
76159	USPH	2020-10-02	84.59	87.36	84.59	86.15	111229	{}	\N	2020-10-12 16:25:26.963882
76160	USPH	2020-10-05	87.08	88.99	87.08	87.44	179047	{}	\N	2020-10-12 16:25:26.9693
76161	USPH	2020-10-06	88.87	89.67	86.4	86.55	176631	{}	\N	2020-10-12 16:25:26.974848
76162	USPH	2020-10-07	87.66	87.66	84.25	85.36	226126	{}	\N	2020-10-12 16:25:26.979991
76163	USPH	2020-10-08	86.42	90	86.42	89.62	142369	{}	\N	2020-10-12 16:25:26.984912
76164	USPH	2020-10-09	90.8	90.92	87.82	87.98	81238	{}	\N	2020-10-12 16:25:26.990247
76165	USPH	2020-10-12	88.13	88.13	86.91	87.03	17082	{}	\N	2020-10-12 16:25:26.995558
76166	UBT	2020-09-28	67.57	67.595	67.115	67.32	25346	{}	\N	2020-10-12 16:25:28.556993
76167	UBT	2020-09-29	67.54	67.7821	67.385	67.4591	7501	{}	\N	2020-10-12 16:25:28.568381
76168	UBT	2020-09-30	66.83	66.83	65.7995	66.32	201479	{}	\N	2020-10-12 16:25:28.575054
76169	UBT	2020-10-01	65.72	66.74	65.47	66.506	83307	{}	\N	2020-10-12 16:25:28.580618
76170	UBT	2020-10-02	66.67	66.69	65.61	65.96	64874	{}	\N	2020-10-12 16:25:28.586116
76171	UBT	2020-10-05	64.48	64.48	63.37	63.37	26584	{}	\N	2020-10-12 16:25:28.593766
76172	UBT	2020-10-06	63.2	64.47	62.69	64.07	19528	{}	\N	2020-10-12 16:25:28.598339
76173	UBT	2020-10-07	63.3	63.9322	62.845	63.04	30065	{}	\N	2020-10-12 16:25:28.602875
76174	UBT	2020-10-08	63.7	63.87	63.48	63.794	31591	{}	\N	2020-10-12 16:25:28.607664
76175	UBT	2020-10-09	63.3	63.75	62.76	63.71	32360	{}	\N	2020-10-12 16:25:28.612606
76176	UBT	2020-10-12	64.06	64.06	63.78	63.81	4377	{}	\N	2020-10-12 16:25:28.617481
76177	UCTT	2020-09-28	21.06	21.97	21.06	21.65	415471	{}	\N	2020-10-12 16:25:30.930758
76178	UCTT	2020-09-29	21.63	22.35	21.63	22.15	506039	{}	\N	2020-10-12 16:25:30.940332
76179	UCTT	2020-09-30	22.14	22.46	21.35	21.46	532519	{}	\N	2020-10-12 16:25:30.948004
76180	UCTT	2020-10-01	21.77	22.67	21.77	22.43	391160	{}	\N	2020-10-12 16:25:30.955255
76181	UCTT	2020-10-02	21.74	22.25	21.38	21.82	317845	{}	\N	2020-10-12 16:25:30.961573
76182	UCTT	2020-10-05	22.16	22.77	22.16	22.77	258190	{}	\N	2020-10-12 16:25:30.967357
76183	UCTT	2020-10-06	22.69	23.71	22.69	22.84	404129	{}	\N	2020-10-12 16:25:30.972878
76184	UCTT	2020-10-07	23.53	23.53	22.72	22.95	405840	{}	\N	2020-10-12 16:25:30.978622
76185	UCTT	2020-10-08	23.4	23.59	23.1	23.56	446791	{}	\N	2020-10-12 16:25:30.98432
76186	UCTT	2020-10-09	23.8	24.63	23.65	24.34	355708	{}	\N	2020-10-12 16:25:30.990115
76187	UCTT	2020-10-12	24.63	24.84	24.39	24.49	108139	{}	\N	2020-10-12 16:25:30.995719
76188	UGL	2020-09-28	67.5	68.31	66.96	68.14	321456	{}	\N	2020-10-12 16:25:32.676821
76189	UGL	2020-09-29	68.6	69.51	68.4335	69.34	170632	{}	\N	2020-10-12 16:25:32.687572
76190	UGL	2020-09-30	69.02	69.7988	68.23	68.58	186731	{}	\N	2020-10-12 16:25:32.69757
76191	UGL	2020-10-01	69.77	70.45	69.37	69.92	171200	{}	\N	2020-10-12 16:25:32.703316
76192	UGL	2020-10-02	69.83	70.26	69.4	69.7	109470	{}	\N	2020-10-12 16:25:32.709092
76193	UGL	2020-10-05	70.02	70.93	69.97	70.4	125271	{}	\N	2020-10-12 16:25:32.714153
76194	UGL	2020-10-06	70.92	71.15	68.48	68.58	126097	{}	\N	2020-10-12 16:25:32.719382
76195	UGL	2020-10-07	68.4	68.64	67.86	68.45	129614	{}	\N	2020-10-12 16:25:32.724957
76196	UGL	2020-10-08	69.28	69.51	68.13	69.05	63223	{}	\N	2020-10-12 16:25:32.730189
76197	UGL	2020-10-09	70.86	71.72	70.7568	71.64	212583	{}	\N	2020-10-12 16:25:32.735133
76198	UGL	2020-10-12	71.23	71.41	70.91	71.2899	61393	{}	\N	2020-10-12 16:25:32.741233
76199	UIVM	2020-09-28	40.64	40.7	40.55	40.6845	69270	{}	\N	2020-10-12 16:25:33.716882
76200	UIVM	2020-09-29	40.72	40.79	40.601	40.69	5086	{}	\N	2020-10-12 16:25:33.724628
76201	UIVM	2020-09-30	40.59	40.73	40.38	40.38	7098	{}	\N	2020-10-12 16:25:33.731231
76202	UIVM	2020-10-01	40.62	40.74	40.54	40.66	6988	{}	\N	2020-10-12 16:25:33.736989
76203	UIVM	2020-10-02	40.6001	40.71	40.47	40.68	7133	{}	\N	2020-10-12 16:25:33.742652
76204	UIVM	2020-10-05	41.12	41.41	41.08	41.29	173394	{}	\N	2020-10-12 16:25:33.747591
76205	UIVM	2020-10-06	41.26	41.3411	40.9309	40.9309	6924	{}	\N	2020-10-12 16:25:33.752597
76206	UIVM	2020-10-07	41.07	41.16	41	41.1406	10261	{}	\N	2020-10-12 16:25:33.757333
76207	UIVM	2020-10-08	41.32	41.3729	41.31	41.3729	75063	{}	\N	2020-10-12 16:25:33.762354
76208	UIVM	2020-10-09	41.59	41.6801	41.5801	41.63	3775	{}	\N	2020-10-12 16:25:33.767565
76209	UIVM	2020-10-12	41.72	41.83	41.72	41.83	3603	{}	\N	2020-10-12 16:25:33.772324
76210	UMC	2020-09-28	4.78	4.97	4.73	4.96	4311972	{}	\N	2020-10-12 16:25:35.561015
76211	UMC	2020-09-29	4.73	4.78	4.58	4.73	7100444	{}	\N	2020-10-12 16:25:35.573699
76212	UMC	2020-09-30	4.84	4.86	4.775	4.79	4328778	{}	\N	2020-10-12 16:25:35.582326
76213	UMC	2020-10-01	4.89	4.95	4.86	4.94	2800518	{}	\N	2020-10-12 16:25:35.589162
76214	UMC	2020-10-02	4.81	4.91	4.81	4.81	4279177	{}	\N	2020-10-12 16:25:35.595502
76215	UMC	2020-10-05	4.92	5.075	4.88	5.04	4745801	{}	\N	2020-10-12 16:25:35.601319
76216	UMC	2020-10-06	4.92	4.985	4.84	4.87	6101544	{}	\N	2020-10-12 16:25:35.606693
76217	UMC	2020-10-07	5.03	5.455	5.03	5.23	11414690	{}	\N	2020-10-12 16:25:35.612551
76218	UMC	2020-10-08	5.61	5.91	5.23	5.41	15124950	{}	\N	2020-10-12 16:25:35.618559
76219	UMC	2020-10-09	5.5	5.7	5.45	5.7	8885018	{}	\N	2020-10-12 16:25:35.624251
76220	UMC	2020-10-12	5.76	5.79	5.52	5.585	5177683	{}	\N	2020-10-12 16:25:35.630023
76221	UNFI	2020-09-28	18.51	19.43	18	19.18	4491903	{}	\N	2020-10-12 16:25:36.597727
76222	UNFI	2020-09-29	17.34	17.9	16.38	16.46	13237289	{}	\N	2020-10-12 16:25:36.611818
76223	UNFI	2020-09-30	16.51	16.69	14.8	14.87	11417247	{}	\N	2020-10-12 16:25:36.619053
76224	UNFI	2020-10-01	15.05	16	15	15.84	4784163	{}	\N	2020-10-12 16:25:36.624918
76225	UNFI	2020-10-02	15.69	16.33	15.24	15.92	3323846	{}	\N	2020-10-12 16:25:36.630817
76226	UNFI	2020-10-05	16.15	17.14	16.06	16.88	3965434	{}	\N	2020-10-12 16:25:36.636246
76227	UNFI	2020-10-06	16.94	18	16.77	16.79	4402614	{}	\N	2020-10-12 16:25:36.641156
76228	UNFI	2020-10-07	16.99	17.56	16.88	17.41	2722559	{}	\N	2020-10-12 16:25:36.646465
76229	UNFI	2020-10-08	17.51	17.99	16.62	17.19	2772544	{}	\N	2020-10-12 16:25:36.651763
76230	UNFI	2020-10-09	16.9	17.84	16.89	17.06	3944624	{}	\N	2020-10-12 16:25:36.656813
76231	UNFI	2020-10-12	17.21	17.49	16.7866	17.35	1119297	{}	\N	2020-10-12 16:25:36.662191
76232	UPRO	2020-09-28	55.1	55.93	54.59	55.6	4427343	{}	\N	2020-10-12 16:25:38.61268
76233	UPRO	2020-09-29	55.49	55.849	54.3	54.69	3554533	{}	\N	2020-10-12 16:25:38.625194
76234	UPRO	2020-09-30	55.05	57.59	54.93	55.91	6762882	{}	\N	2020-10-12 16:25:38.633259
76235	UPRO	2020-10-01	57.31	57.82	55.961	56.95	4003434	{}	\N	2020-10-12 16:25:38.64014
76236	UPRO	2020-10-02	54.28	56.408	54.0202	55.38	5542107	{}	\N	2020-10-12 16:25:38.646468
76237	UPRO	2020-10-05	56.47	58.38	56.44	58.26	4039186	{}	\N	2020-10-12 16:25:38.651818
76238	UPRO	2020-10-06	58.3761	59.52	55.53	55.82	6897348	{}	\N	2020-10-12 16:25:38.657679
76239	UPRO	2020-10-07	57.3999	59.16	57.3866	58.72	3490368	{}	\N	2020-10-12 16:25:38.662677
76240	UPRO	2020-10-08	59.81	60.301	59.3	60.23	3896093	{}	\N	2020-10-12 16:25:38.667996
76241	UPRO	2020-10-09	61.22	62.15	60.87	61.87	4528529	{}	\N	2020-10-12 16:25:38.672931
76242	UPRO	2020-10-12	63.44	64.965	63.09	64.955	2574551	{}	\N	2020-10-12 16:25:38.677676
76243	USFR	2020-09-28	25.12	25.12	25.11	25.11	500452	{}	\N	2020-10-12 16:25:40.87362
76244	USFR	2020-09-29	25.12	25.12	25.11	25.11	318136	{}	\N	2020-10-12 16:25:40.883739
76245	USFR	2020-09-30	25.12	25.12	25.11	25.11	469781	{}	\N	2020-10-12 16:25:40.890606
76246	USFR	2020-10-01	25.12	25.13	25.11	25.13	363273	{}	\N	2020-10-12 16:25:40.89615
76247	USFR	2020-10-02	25.12	25.12	25.11	25.11	276038	{}	\N	2020-10-12 16:25:40.901973
76248	USFR	2020-10-05	25.11	25.12	25.11	25.12	1474701	{}	\N	2020-10-12 16:25:40.908217
76249	USFR	2020-10-06	25.11	25.12	25.11	25.11	339059	{}	\N	2020-10-12 16:25:40.913048
76250	USFR	2020-10-07	25.11	25.12	25.1	25.11	282641	{}	\N	2020-10-12 16:25:40.924188
76251	USFR	2020-10-08	25.12	25.12	25.11	25.11	330202	{}	\N	2020-10-12 16:25:40.928666
76252	USFR	2020-10-09	25.11	25.12	25.11	25.12	293958	{}	\N	2020-10-12 16:25:40.933379
76253	USFR	2020-10-12	25.11	25.12	25.11	25.1119	78202	{}	\N	2020-10-12 16:25:40.937949
76254	UJPY	2020-09-28	24.5398	24.5398	24.5398	24.5398	63	{}	\N	2020-10-12 16:25:46.715514
76255	UJPY	2020-09-29	24.3739	24.3739	24.3739	24.3739	10	{}	\N	2020-10-12 16:25:46.724718
76256	UJPY	2020-09-30	24.3	24.5564	24.29	24.5564	4357	{}	\N	2020-10-12 16:25:46.731183
76257	UJPY	2020-10-01	24.4751	24.4751	24.4751	24.4751	25	{}	\N	2020-10-12 16:25:46.737101
76258	UJPY	2020-10-02	24.6144	24.6144	24.6144	24.6144	16	{}	\N	2020-10-12 16:25:46.742596
76259	UJPY	2020-10-05	24.44	24.4566	24.2522	24.2522	371	{}	\N	2020-10-12 16:25:46.747669
76260	UJPY	2020-10-06	24.4563	24.4563	24.4563	24.4563	60	{}	\N	2020-10-12 16:25:46.752811
76261	UJPY	2020-10-07	24.0095	24.0095	24.0095	24.0095	6	{}	\N	2020-10-12 16:25:46.758005
76262	UJPY	2020-10-08	24.0334	24.0334	24.0334	24.0334	140	{}	\N	2020-10-12 16:25:46.763313
76263	UJPY	2020-10-09	24.245	24.3893	24.245	24.3893	350	{}	\N	2020-10-12 16:25:46.768721
76264	UJPY	2020-10-12	24.64	24.6601	24.64	24.6601	2191	{}	\N	2020-10-12 16:25:46.774146
76265	UMH	2020-09-28	13.71	13.99	13.45	13.95	369966	{}	\N	2020-10-12 16:25:50.562068
76266	UMH	2020-09-29	13.87	13.94	13.56	13.69	554234	{}	\N	2020-10-12 16:25:50.574559
76267	UMH	2020-09-30	13.77	14	13.42	13.54	240666	{}	\N	2020-10-12 16:25:50.582624
76268	UMH	2020-10-01	14.06	14.22	13.79	14.17	292849	{}	\N	2020-10-12 16:25:50.588645
76269	UMH	2020-10-02	13.92	14.45	13.92	14.41	233076	{}	\N	2020-10-12 16:25:50.594467
76270	UMH	2020-10-05	14.45	14.5	14.16	14.38	204809	{}	\N	2020-10-12 16:25:50.599557
76271	UMH	2020-10-06	14.46	14.72	14.25	14.46	321196	{}	\N	2020-10-12 16:25:50.604789
76272	UMH	2020-10-07	14.46	14.64	14.27	14.54	199337	{}	\N	2020-10-12 16:25:50.609894
76273	UMH	2020-10-08	14.99	15	14.59	14.65	296959	{}	\N	2020-10-12 16:25:50.615866
76274	UMH	2020-10-09	14.78	14.78	14.4	14.46	117479	{}	\N	2020-10-12 16:25:50.620795
76275	UMH	2020-10-12	14.44	14.49	14.29	14.44	36372	{}	\N	2020-10-12 16:25:50.626041
76276	UNL	2020-09-28	8.7	8.82	8.64	8.81	16697	{}	\N	2020-10-12 16:25:53.935642
76277	UNL	2020-09-29	8.52	8.6084	8.5	8.5001	19941	{}	\N	2020-10-12 16:25:53.94734
76278	UNL	2020-09-30	8.58	8.59	8.485	8.59	3444	{}	\N	2020-10-12 16:25:53.954627
76279	UNL	2020-10-01	8.59	8.59	8.35	8.35	5276	{}	\N	2020-10-12 16:25:53.96078
76280	UNL	2020-10-02	8.28	8.41	8.27	8.325	15549	{}	\N	2020-10-12 16:25:53.966221
76281	UNL	2020-10-05	8.545	8.6899	8.545	8.68	10177	{}	\N	2020-10-12 16:25:53.971531
76282	UNL	2020-10-06	8.63	8.68	8.46	8.4758	5758	{}	\N	2020-10-12 16:25:53.976139
76283	UNL	2020-10-07	8.85	8.85	8.56	8.61	3035	{}	\N	2020-10-12 16:25:53.981062
76284	UNL	2020-10-08	8.61	8.63	8.55	8.63	5406	{}	\N	2020-10-12 16:25:53.985812
76285	UNL	2020-10-09	8.72	8.8	8.7107	8.8	8064	{}	\N	2020-10-12 16:25:53.990834
76286	UNL	2020-10-12	8.92	8.94	8.9	8.93	3146	{}	\N	2020-10-12 16:25:53.995495
76287	URBN	2020-09-28	20.78	22.26	20.725	22.09	1765262	{}	\N	2020-10-12 16:25:56.387814
76288	URBN	2020-09-29	21.99	22.21	20.705	20.89	1391556	{}	\N	2020-10-12 16:25:56.398426
76289	URBN	2020-09-30	21.05	21.62	20.59	20.81	1839307	{}	\N	2020-10-12 16:25:56.405777
76290	URBN	2020-10-01	20.89	22.03	20.89	21.7	1477921	{}	\N	2020-10-12 16:25:56.412538
76291	URBN	2020-10-02	20.93	22.78	20.75	22.67	1797966	{}	\N	2020-10-12 16:25:56.418791
76292	URBN	2020-10-05	22.94	23.6	22.86	23.59	1834444	{}	\N	2020-10-12 16:25:56.424515
76293	URBN	2020-10-06	23.87	24.11	22.71	22.82	1890832	{}	\N	2020-10-12 16:25:56.429889
76294	URBN	2020-10-07	23.72	24.62	23.625	24.21	1826296	{}	\N	2020-10-12 16:25:56.435379
76295	URBN	2020-10-08	24.25	24.5	23.35	23.47	2050538	{}	\N	2020-10-12 16:25:56.440894
76296	URBN	2020-10-09	23.89	24.15	23.51	23.595	892548	{}	\N	2020-10-12 16:25:56.446293
76297	URBN	2020-10-12	23.56	24.23	23.2331	23.675	282574	{}	\N	2020-10-12 16:25:56.451949
76298	USAK	2020-09-28	9.88	10.28	9.88	10.1	46934	{}	\N	2020-10-12 16:25:59.50415
76299	USAK	2020-09-29	9.98	10.18	9.58	9.94	45131	{}	\N	2020-10-12 16:25:59.51533
76300	USAK	2020-09-30	9.92	10.01	9.25	9.45	63250	{}	\N	2020-10-12 16:25:59.522618
76301	USAK	2020-10-01	9.48	9.71	9.1801	9.71	37050	{}	\N	2020-10-12 16:25:59.52844
76302	USAK	2020-10-02	9.57	9.77	9.31	9.58	17683	{}	\N	2020-10-12 16:25:59.53437
76303	USAK	2020-10-05	9.58	9.98	9.2301	9.82	22611	{}	\N	2020-10-12 16:25:59.539947
76304	USAK	2020-10-06	9.82	10.465	9.82	9.95	31992	{}	\N	2020-10-12 16:25:59.545161
76305	USAK	2020-10-07	10.1224	10.29	10.0199	10.22	42930	{}	\N	2020-10-12 16:25:59.549905
76306	USAK	2020-10-08	10.31	10.33	9.99	10.18	24527	{}	\N	2020-10-12 16:25:59.555007
76307	USAK	2020-10-09	10.13	10.48	9.3836	10.3	55839	{}	\N	2020-10-12 16:25:59.560303
76308	USAK	2020-10-12	10.495	10.62	10.475	10.58	11938	{}	\N	2020-10-12 16:25:59.565313
76309	USEG	2020-09-28	4.2845	4.34	4.24	4.32	16322	{}	\N	2020-10-12 16:26:00.884502
76310	USEG	2020-09-29	17.23	18.57	6.5501	6.77	15715426	{}	\N	2020-10-12 16:26:00.895509
76311	USEG	2020-09-30	6.275	6.4283	4.91	5	715785	{}	\N	2020-10-12 16:26:00.902817
76312	USEG	2020-10-01	5.07	5.59	5	5.32	443420	{}	\N	2020-10-12 16:26:00.908465
76313	USEG	2020-10-02	4.89	5.3	4.89	5.21	211163	{}	\N	2020-10-12 16:26:00.914075
76314	USEG	2020-10-05	5.17	5.5	4.92	5.26	386099	{}	\N	2020-10-12 16:26:00.919091
76315	USEG	2020-10-06	5.2	6.25	5.03	5.6	1478046	{}	\N	2020-10-12 16:26:00.923345
76316	USEG	2020-10-07	5.53	5.58	5.05	5.22	363081	{}	\N	2020-10-12 16:26:00.92756
76317	USEG	2020-10-08	5.16	6.9	5.16	5.73	2819778	{}	\N	2020-10-12 16:26:00.931995
76318	USEG	2020-10-09	5.3	5.7	5.2	5.36	401180	{}	\N	2020-10-12 16:26:00.935981
76319	USEG	2020-10-12	5.3	5.48	5.11	5.235	88745	{}	\N	2020-10-12 16:26:00.939873
76320	USM	2020-09-28	29.76	30.19	29.76	30.11	117006	{}	\N	2020-10-12 16:26:02.877215
76321	USM	2020-09-29	30.19	30.29	29.85	29.87	233940	{}	\N	2020-10-12 16:26:02.890585
76322	USM	2020-09-30	29.6	30	29.405	29.53	347266	{}	\N	2020-10-12 16:26:02.899715
76323	USM	2020-10-01	29.55	29.73	28.95	29.56	236864	{}	\N	2020-10-12 16:26:02.906272
76324	USM	2020-10-02	29.14	29.7	29.02	29.54	215631	{}	\N	2020-10-12 16:26:02.911905
76325	USM	2020-10-05	29.7	29.92	29.1	29.65	263839	{}	\N	2020-10-12 16:26:02.917123
76326	USM	2020-10-06	29.73	30.06	29.38	29.57	273397	{}	\N	2020-10-12 16:26:02.924898
76327	USM	2020-10-07	29.72	30.24	29.32	30.16	343813	{}	\N	2020-10-12 16:26:02.929627
76328	USM	2020-10-08	30.28	30.81	30.06	30.63	268958	{}	\N	2020-10-12 16:26:02.934461
76329	USM	2020-10-09	30.67	31.47	30.67	31.17	175205	{}	\N	2020-10-12 16:26:02.939314
76330	USM	2020-10-12	31.07	31.31	30.73	30.93	76338	{}	\N	2020-10-12 16:26:02.944173
76331	UTES	2020-09-28	37.59	37.94	37.59	37.8501	5733	{}	\N	2020-10-12 16:26:04.059777
76332	UTES	2020-09-29	37.75	37.8	37.6743	37.6743	2649	{}	\N	2020-10-12 16:26:04.072295
76333	UTES	2020-09-30	37.77	37.77	37.5493	37.5493	1068	{}	\N	2020-10-12 16:26:04.080355
76334	UTES	2020-10-01	38.1171	38.1171	37.8104	37.8104	668	{}	\N	2020-10-12 16:26:04.08709
76335	UTES	2020-10-02	37.61	38.33	37.61	38.2	1553	{}	\N	2020-10-12 16:26:04.093283
76336	UTES	2020-10-05	38.2904	38.6016	38.2904	38.6016	1533	{}	\N	2020-10-12 16:26:04.098889
76337	UTES	2020-10-06	39.0301	39.256	39.03	39.256	1331	{}	\N	2020-10-12 16:26:04.103866
76338	UTES	2020-10-07	39.32	39.52	39.16	39.52	2609	{}	\N	2020-10-12 16:26:04.108889
76339	UTES	2020-10-08	39.945	40.1672	39.9	40.1672	1717	{}	\N	2020-10-12 16:26:04.113455
76340	UTES	2020-10-09	40.2	40.2	40.075	40.08	3712	{}	\N	2020-10-12 16:26:04.118709
76341	UTES	2020-10-12	40.14	40.14	40.14	40.14	333	{}	\N	2020-10-12 16:26:04.12356
76342	UG	2020-09-28	14.95	15.18	14.95	15.05	11324	{}	\N	2020-10-12 16:26:05.406139
76343	UG	2020-09-29	15.05	15.09	14.975	15.05	1413	{}	\N	2020-10-12 16:26:05.417385
76344	UG	2020-09-30	15	15.26	14.57	15.155	29762	{}	\N	2020-10-12 16:26:05.424973
76345	UG	2020-10-01	15.15	15.15	14.67	15	13006	{}	\N	2020-10-12 16:26:05.430568
76346	UG	2020-10-02	14.72	15.28	14.72	15.1	1054	{}	\N	2020-10-12 16:26:05.436123
76347	UG	2020-10-05	15.3	15.3	15.06	15.06	1002	{}	\N	2020-10-12 16:26:05.44084
76348	UG	2020-10-06	15.1636	15.4	15.1636	15.23	2085	{}	\N	2020-10-12 16:26:05.446157
76349	UG	2020-10-07	15.5	15.8	15.5	15.7	4132	{}	\N	2020-10-12 16:26:05.451195
76350	UG	2020-10-08	16.07	16.35	16.05	16.154	1682	{}	\N	2020-10-12 16:26:05.456154
76351	UG	2020-10-09	16.35	16.47	16.01	16.07	3059	{}	\N	2020-10-12 16:26:05.461232
76352	UG	2020-10-12	16	16.0672	16	16.0672	417	{}	\N	2020-10-12 16:26:05.466332
76353	UHT	2020-09-28	55.29	57.88	55.29	57.42	50846	{}	\N	2020-10-12 16:26:07.221163
76354	UHT	2020-09-29	57.1	57.25	54.81	56.14	66239	{}	\N	2020-10-12 16:26:07.236973
76355	UHT	2020-09-30	56.83	57.73	56	56.99	147282	{}	\N	2020-10-12 16:26:07.244214
76356	UHT	2020-10-01	56.81	58.38	56.33	58.29	125606	{}	\N	2020-10-12 16:26:07.250043
76357	UHT	2020-10-02	57.23	60.6	56.75	59.74	88981	{}	\N	2020-10-12 16:26:07.255929
76358	UHT	2020-10-05	59.74	59.87	57.49	58.39	85244	{}	\N	2020-10-12 16:26:07.261275
76359	UHT	2020-10-06	58.9	60.18	57.56	57.93	111703	{}	\N	2020-10-12 16:26:07.26679
76360	UHT	2020-10-07	58.36	58.858	55.86	57.29	141576	{}	\N	2020-10-12 16:26:07.271992
76361	UHT	2020-10-08	57.34	59.59	57.34	58.91	64679	{}	\N	2020-10-12 16:26:07.27719
76362	UHT	2020-10-09	59.27	59.54	57.57	57.75	54254	{}	\N	2020-10-12 16:26:07.282254
76363	UHT	2020-10-12	57.75	58.6	56.8012	58.01	27620	{}	\N	2020-10-12 16:26:07.287684
76364	UN	2020-09-28	60.6	60.99	60.54	60.58	1364291	{}	\N	2020-10-12 16:26:09.609743
76365	UN	2020-09-29	60.76	60.85	60.36	60.62	2336141	{}	\N	2020-10-12 16:26:09.618442
76366	UN	2020-09-30	60.8	60.98	60.16	60.4	2446316	{}	\N	2020-10-12 16:26:09.638355
76367	UN	2020-10-01	60.61	60.865	60.2	60.73	2535485	{}	\N	2020-10-12 16:26:09.650867
76368	UN	2020-10-02	60.85	61.54	60.79	61.32	2266023	{}	\N	2020-10-12 16:26:09.660094
76369	UN	2020-10-05	61.61	61.86	61.47	61.74	2025154	{}	\N	2020-10-12 16:26:09.667435
76370	UN	2020-10-06	61.56	61.57	60.47	60.61	2433406	{}	\N	2020-10-12 16:26:09.674723
76371	UN	2020-10-07	60.75	61.01	60.665	60.89	1802264	{}	\N	2020-10-12 16:26:09.68399
76372	UN	2020-10-08	60.87	61.16	60.76	60.91	2036403	{}	\N	2020-10-12 16:26:09.690776
76373	UN	2020-10-09	61.99	62.49	61.87	62.43	3827080	{}	\N	2020-10-12 16:26:09.697883
76374	UN	2020-10-12	62.2	62.52	62.08	62.21	1367074	{}	\N	2020-10-12 16:26:09.704803
76375	UNM	2020-09-28	16.89	17.4	16.75	17.08	1671643	{}	\N	2020-10-12 16:26:13.044468
76376	UNM	2020-09-29	17.07	17.11	16.47	16.69	3312015	{}	\N	2020-10-12 16:26:13.05726
76377	UNM	2020-09-30	16.78	17	16.61	16.83	4598515	{}	\N	2020-10-12 16:26:13.065748
76378	UNM	2020-10-01	17.01	17.15	16.66	17	3408716	{}	\N	2020-10-12 16:26:13.072236
76379	UNM	2020-10-02	16.63	17.795	16.6	17.68	4327860	{}	\N	2020-10-12 16:26:13.07856
76380	UNM	2020-10-05	18	18.55	17.965	18.46	4898787	{}	\N	2020-10-12 16:26:13.084148
76381	UNM	2020-10-06	18.73	19.2	18.1171	18.21	5975890	{}	\N	2020-10-12 16:26:13.089115
76382	UNM	2020-10-07	18.5	19.08	18.5	18.9	3562403	{}	\N	2020-10-12 16:26:13.094723
76383	UNM	2020-10-08	19.13	19.255	18.785	19	3547232	{}	\N	2020-10-12 16:26:13.100513
76384	UNM	2020-10-09	19.15	19.19	18.69	18.73	3529741	{}	\N	2020-10-12 16:26:13.105659
76385	UNM	2020-10-12	18.83	19.175	18.75	19.06	1120100	{}	\N	2020-10-12 16:26:13.110983
76386	UPLD	2020-09-28	38.03	38.59	37.795	38.35	147777	{}	\N	2020-10-12 16:26:15.858557
76387	UPLD	2020-09-29	38.45	38.5577	37.51	37.57	131629	{}	\N	2020-10-12 16:26:15.870932
76388	UPLD	2020-09-30	37.59	38.5899	37.24	37.7	316888	{}	\N	2020-10-12 16:26:15.87827
76389	UPLD	2020-10-01	37.96	39.3125	37.82	39.26	273222	{}	\N	2020-10-12 16:26:15.88451
76390	UPLD	2020-10-02	38.21	40.24	38.21	39.85	226455	{}	\N	2020-10-12 16:26:15.890493
76391	UPLD	2020-10-05	40.24	41.63	39.93	41.44	237509	{}	\N	2020-10-12 16:26:15.895857
76392	UPLD	2020-10-06	41.52	42.2	40.83	41.06	172541	{}	\N	2020-10-12 16:26:15.90044
76393	UPLD	2020-10-07	41.31	43.1	41.03	42.62	284623	{}	\N	2020-10-12 16:26:15.905273
76394	UPLD	2020-10-08	43.36	43.61	42.55	43.33	151405	{}	\N	2020-10-12 16:26:15.909776
76395	UPLD	2020-10-09	43.77	44	43.04	43.4	259058	{}	\N	2020-10-12 16:26:15.915805
76396	UPLD	2020-10-12	43.92	43.92	42.81	43.85	53568	{}	\N	2020-10-12 16:26:15.920577
76397	URG	2020-09-28	0.4928	0.5227	0.4928	0.509	194810	{}	\N	2020-10-12 16:26:17.187483
76398	URG	2020-09-29	0.5091	0.5149	0.4818	0.5	338635	{}	\N	2020-10-12 16:26:17.198629
76399	URG	2020-09-30	0.5	0.51	0.48	0.48	237055	{}	\N	2020-10-12 16:26:17.206402
76400	URG	2020-10-01	0.4805	0.51	0.4805	0.4866	157321	{}	\N	2020-10-12 16:26:17.213695
76401	URG	2020-10-02	0.48	0.4866	0.45	0.4591	1162013	{}	\N	2020-10-12 16:26:17.219478
76402	URG	2020-10-05	0.47	0.4747	0.45	0.455361	736551	{}	\N	2020-10-12 16:26:17.224471
76403	URG	2020-10-06	0.46	0.4887	0.459986	0.4661	427036	{}	\N	2020-10-12 16:26:17.229501
76404	URG	2020-10-07	0.47	0.499	0.47	0.4959	324608	{}	\N	2020-10-12 16:26:17.234787
76405	URG	2020-10-08	0.5	0.502	0.4739	0.4785	453265	{}	\N	2020-10-12 16:26:17.240094
76406	URG	2020-10-09	0.49	0.508	0.4707	0.4751	257217	{}	\N	2020-10-12 16:26:17.244936
76407	URG	2020-10-12	0.4785	0.48	0.4601	0.478099	190530	{}	\N	2020-10-12 16:26:17.249817
76408	USAI	2020-09-28	15.1586	15.3652	15.1586	15.3652	927	{}	\N	2020-10-12 16:26:19.323242
76409	USAI	2020-09-29	15.2394	15.2529	15.125	15.211	2992	{}	\N	2020-10-12 16:26:19.333431
76410	USAI	2020-09-30	15.387	15.387	14.9499	14.9499	2292	{}	\N	2020-10-12 16:26:19.340127
76411	USAI	2020-10-01	14.7644	14.7644	14.7644	14.7644	162	{}	\N	2020-10-12 16:26:19.345743
76412	USAI	2020-10-02	14.617	14.9953	14.59	14.9953	2787	{}	\N	2020-10-12 16:26:19.351329
76413	USAI	2020-10-05	15.23	15.4158	15.23	15.4158	3237	{}	\N	2020-10-12 16:26:19.356111
76414	USAI	2020-10-06	15.4661	15.5154	15.1205	15.1205	4981	{}	\N	2020-10-12 16:26:19.36099
76415	USAI	2020-10-07	15.168	15.2925	15.168	15.2925	704	{}	\N	2020-10-12 16:26:19.365986
76416	USAI	2020-10-08	15.4165	16.0141	15.4165	16.0141	702	{}	\N	2020-10-12 16:26:19.370779
76417	USAI	2020-10-09	15.88	16.02	15.88	15.91	420	{}	\N	2020-10-12 16:26:19.375841
76418	USAI	2020-10-12	15.9	15.9	15.58	15.84	15964	{}	\N	2020-10-12 16:26:19.380108
76419	USDU	2020-09-28	26.85	26.899	26.7	26.77	33762	{}	\N	2020-10-12 16:26:20.638747
76420	USDU	2020-09-29	26.69	26.8	26.62	26.62	13016	{}	\N	2020-10-12 16:26:20.648489
76421	USDU	2020-09-30	26.71	26.72	26.64	26.64	159136	{}	\N	2020-10-12 16:26:20.65661
76422	USDU	2020-10-01	26.59	26.65	26.49	26.49	88443	{}	\N	2020-10-12 16:26:20.662823
76423	USDU	2020-10-02	26.59	26.65	26.5	26.51	129254	{}	\N	2020-10-12 16:26:20.669957
76424	USDU	2020-10-05	26.42	26.53	26.42	26.45	40194	{}	\N	2020-10-12 16:26:20.67579
76425	USDU	2020-10-06	26.42	26.63	26.42	26.63	6851	{}	\N	2020-10-12 16:26:20.681071
76426	USDU	2020-10-07	26.49	26.58	26.35	26.42	21127	{}	\N	2020-10-12 16:26:20.686483
76427	USDU	2020-10-08	26.46	26.51	26.4596	26.51	3200	{}	\N	2020-10-12 16:26:20.692161
76428	USDU	2020-10-09	26.34	26.3439	26.19	26.19	16911	{}	\N	2020-10-12 16:26:20.697638
76429	USDU	2020-10-12	26.27	26.27	26.17	26.188	6745	{}	\N	2020-10-12 16:26:20.703833
76430	USMC	2020-09-28	32.585	32.7385	32.585	32.69	5630	{}	\N	2020-10-12 16:26:21.88004
76431	USMC	2020-09-29	32.72	32.72	32.56	32.64	7070	{}	\N	2020-10-12 16:26:21.888034
76432	USMC	2020-09-30	32.665	33.12	32.61	33.01	166905	{}	\N	2020-10-12 16:26:21.894406
76433	USMC	2020-10-01	33	33.07	32.72	32.88	26581	{}	\N	2020-10-12 16:26:21.900192
76434	USMC	2020-10-02	32.6643	32.6643	32.4472	32.4472	10069	{}	\N	2020-10-12 16:26:21.905755
76435	USMC	2020-10-05	32.6899	32.94	32.65	32.93	29344	{}	\N	2020-10-12 16:26:21.910926
76436	USMC	2020-10-06	33.02	33.02	32.535	32.535	12829	{}	\N	2020-10-12 16:26:21.915836
76437	USMC	2020-10-07	32.77	33.0966	32.77	33.0846	9392	{}	\N	2020-10-12 16:26:21.921157
76438	USMC	2020-10-08	33.25	33.26	33.15	33.22	5647	{}	\N	2020-10-12 16:26:21.926409
76439	USMC	2020-10-09	33.425	33.57	33.4	33.49	6597	{}	\N	2020-10-12 16:26:21.93137
76440	USMC	2020-10-12	33.77	33.9574	33.75	33.9574	1990	{}	\N	2020-10-12 16:26:21.936357
76441	USV	2020-09-28	28.99	28.99	28.99	28.99	138	{}	\N	2020-10-12 16:26:25.431248
76442	USV	2020-09-29	29.85	29.85	29.85	29.85	44	{}	\N	2020-10-12 16:26:25.44038
76443	USV	2020-09-30	29.3215	29.3215	29.3215	29.3215	115	{}	\N	2020-10-12 16:26:25.44726
76444	USV	2020-10-01	29.59	29.59	29.59	29.59	177	{}	\N	2020-10-12 16:26:25.453311
76445	USV	2020-10-02	29.745	29.7529	29.745	29.7529	121	{}	\N	2020-10-12 16:26:25.459861
76446	USV	2020-10-05	29.46	30.4659	29.4	30.4659	985	{}	\N	2020-10-12 16:26:25.464712
76447	USV	2020-10-06	31.26	31.26	29.5907	29.5907	425	{}	\N	2020-10-12 16:26:25.469596
76448	USV	2020-10-07	28.5	30.128	28.5	30.128	891	{}	\N	2020-10-12 16:26:25.474695
76449	USV	2020-10-08	29.6574	29.6574	29.6574	29.6574	0	{}	\N	2020-10-12 16:26:25.480159
76450	USV	2020-10-09	30.3	31.3322	30.3	31.3322	2717	{}	\N	2020-10-12 16:26:25.485611
76451	UWM	2020-09-28	51.88	53.3	51.809	53.1	329372	{}	\N	2020-10-12 16:26:26.44399
76452	UWM	2020-09-29	52.94	53.19	51.65	52.54	177944	{}	\N	2020-10-12 16:26:26.452858
76453	UWM	2020-09-30	52.91	54.4	51.97	52.9	366204	{}	\N	2020-10-12 16:26:26.46028
76454	UWM	2020-10-01	53.6	54.58	52.67	54.52	438718	{}	\N	2020-10-12 16:26:26.46632
76455	UWM	2020-10-02	52.56	55.53	52.3	55.06	388976	{}	\N	2020-10-12 16:26:26.471854
76456	UWM	2020-10-05	56.23	58.1999	56.23	58.08	210167	{}	\N	2020-10-12 16:26:26.47757
76457	UWM	2020-10-06	59.16	60.9695	57.55	57.71	616248	{}	\N	2020-10-12 16:26:26.483195
76458	UWM	2020-10-07	59.22	60.63	59.16	60.29	371407	{}	\N	2020-10-12 16:26:26.48918
76459	UWM	2020-10-08	61.52	61.92	60.67	61.65	313248	{}	\N	2020-10-12 16:26:26.4948
76460	UWM	2020-10-09	62.8	63	61.58	62.22	374943	{}	\N	2020-10-12 16:26:26.500312
76461	UWM	2020-10-12	62.8	63.32	62.25	62.91	222630	{}	\N	2020-10-12 16:26:26.505341
76462	UPV	2020-09-28	41	41	40.78	40.84	19583	{}	\N	2020-10-12 16:26:27.986709
76463	UPV	2020-09-29	41.0186	41.0186	41.0186	41.0186	960	{}	\N	2020-10-12 16:26:27.995619
76464	UPV	2020-09-30	40.7635	40.7635	40.7635	40.7635	352	{}	\N	2020-10-12 16:26:28.002101
76465	UPV	2020-10-01	41.1591	41.1591	41.1591	41.1591	9640	{}	\N	2020-10-12 16:26:28.007748
76466	UPV	2020-10-02	41.285	41.285	41.285	41.285	3064	{}	\N	2020-10-12 16:26:28.013766
76467	UPV	2020-10-05	42.7503	42.7503	42.7503	42.7503	1278	{}	\N	2020-10-12 16:26:28.019933
76468	UPV	2020-10-06	42.82	42.82	41.42	41.63	6909	{}	\N	2020-10-12 16:26:28.024573
76469	UPV	2020-10-07	42.05	42.2945	42.05	42.2945	12835	{}	\N	2020-10-12 16:26:28.02946
76470	UPV	2020-10-08	42.9474	42.9474	42.9474	42.9474	226	{}	\N	2020-10-12 16:26:28.034139
76471	UPV	2020-10-09	43.79	43.89	43.79	43.89	1167	{}	\N	2020-10-12 16:26:28.03868
76472	USAC	2020-09-28	10.37	10.5599	10.15	10.17	183920	{}	\N	2020-10-12 16:26:29.000571
76473	USAC	2020-09-29	10.15	10.3	9.97	10.18	298008	{}	\N	2020-10-12 16:26:29.011168
76474	USAC	2020-09-30	10.11	10.46	9.98	10.01	323296	{}	\N	2020-10-12 16:26:29.019322
76475	USAC	2020-10-01	10.08	10.26	9.88	10.23	321713	{}	\N	2020-10-12 16:26:29.024953
76476	USAC	2020-10-02	9.96	10.47	9.95	10.44	322615	{}	\N	2020-10-12 16:26:29.030621
76477	USAC	2020-10-05	10.45	10.66	10.33	10.37	373520	{}	\N	2020-10-12 16:26:29.036101
76478	USAC	2020-10-06	10.42	10.68	10.3901	10.5	362305	{}	\N	2020-10-12 16:26:29.041366
76479	USAC	2020-10-07	10.59	10.64	10.44	10.5	222782	{}	\N	2020-10-12 16:26:29.046841
76480	USAC	2020-10-08	10.66	10.98	10.55	10.98	458532	{}	\N	2020-10-12 16:26:29.052348
76481	USAC	2020-10-09	10.87	10.9855	10.71	10.73	277168	{}	\N	2020-10-12 16:26:29.057435
76482	USAC	2020-10-12	10.68	10.83	10.6001	10.694	121514	{}	\N	2020-10-12 16:26:29.062838
76483	USCI	2020-09-28	29.5	29.59	29.46	29.57	6933	{}	\N	2020-10-12 16:26:30.015034
76484	USCI	2020-09-29	29.33	29.57	29.33	29.54	10769	{}	\N	2020-10-12 16:26:30.021259
76485	USCI	2020-09-30	29.56	29.8	29.5199	29.7	33104	{}	\N	2020-10-12 16:26:30.026732
76486	USCI	2020-10-01	29.63	29.63	29.4	29.45	11956	{}	\N	2020-10-12 16:26:30.031575
76487	USCI	2020-10-02	29.22	29.49	29.22	29.38	22309	{}	\N	2020-10-12 16:26:30.036476
76488	USCI	2020-10-05	29.4	29.73	29.4	29.68	12799	{}	\N	2020-10-12 16:26:30.041216
76489	USCI	2020-10-06	29.925	29.94	29.62	29.62	29495	{}	\N	2020-10-12 16:26:30.046269
76490	USCI	2020-10-07	29.89	29.96	29.885	29.92	7218	{}	\N	2020-10-12 16:26:30.051353
76491	USCI	2020-10-08	29.88	30.1491	29.88	30	10415	{}	\N	2020-10-12 16:26:30.055994
76492	USCI	2020-10-09	30.19	30.333	30.19	30.27	15358	{}	\N	2020-10-12 16:26:30.060839
76493	USCI	2020-10-12	30.19	30.22	30.1008	30.16	2338	{}	\N	2020-10-12 16:26:30.067057
76494	USNA	2020-09-28	75.25	75.25	74.03	74.24	84905	{}	\N	2020-10-12 16:26:31.360566
76495	USNA	2020-09-29	74.56	74.56	72.025	72.39	306688	{}	\N	2020-10-12 16:26:31.374122
76496	USNA	2020-09-30	72.24	74.83	72.24	73.65	314142	{}	\N	2020-10-12 16:26:31.381261
76497	USNA	2020-10-01	73.8	75.3	73.38	74.89	201729	{}	\N	2020-10-12 16:26:31.38796
76498	USNA	2020-10-02	74.37	76.22	73.39	75.09	182508	{}	\N	2020-10-12 16:26:31.394663
76499	USNA	2020-10-05	75.43	76.2053	74.26	74.78	137836	{}	\N	2020-10-12 16:26:31.40024
76500	USNA	2020-10-06	75.2	76.26	74.79	75.06	170884	{}	\N	2020-10-12 16:26:31.405365
76501	USNA	2020-10-07	75.4	76.23	74.33	75.09	214944	{}	\N	2020-10-12 16:26:31.410444
76502	USNA	2020-10-08	75.38	76.84	75.29	76.47	149307	{}	\N	2020-10-12 16:26:31.416136
76503	USNA	2020-10-09	77	78.2	76.5	77.96	206295	{}	\N	2020-10-12 16:26:31.421821
76504	USNA	2020-10-12	77.92	78.41	77.24	78.32	45438	{}	\N	2020-10-12 16:26:31.427344
76505	USVM	2020-09-28	47.98	48.621	47.98	48.48	6276	{}	\N	2020-10-12 16:26:33.038144
76506	USVM	2020-09-29	48.68	48.68	48.25	48.66	7134	{}	\N	2020-10-12 16:26:33.051917
76507	USVM	2020-09-30	48.73	49.2033	48.66	48.6736	5122	{}	\N	2020-10-12 16:26:33.063478
76508	USVM	2020-10-01	48.94	49.2901	48.7	49.2901	2234	{}	\N	2020-10-12 16:26:33.069289
76509	USVM	2020-10-02	48.53	49.81	48.53	49.7	11148	{}	\N	2020-10-12 16:26:33.075387
76510	USVM	2020-10-05	50.12	51.0086	50.12	51.0086	46811	{}	\N	2020-10-12 16:26:33.080658
76511	USVM	2020-10-06	51.39	52.05	50.78	51.1483	15489	{}	\N	2020-10-12 16:26:33.085949
76512	USVM	2020-10-07	51.5	51.6844	51.4557	51.6844	6762	{}	\N	2020-10-12 16:26:33.091194
76513	USVM	2020-10-08	52.13	52.2199	51.93	52.2199	49169	{}	\N	2020-10-12 16:26:33.096435
76514	USVM	2020-10-09	52.5314	52.89	52.45	52.76	8879	{}	\N	2020-10-12 16:26:33.101598
76515	USVM	2020-10-12	52.93	53.0299	52.6503	52.9715	4383	{}	\N	2020-10-12 16:26:33.107124
76516	VBR	2020-09-28	109.82	111.619	109.82	111.21	345324	{}	\N	2020-10-12 16:26:36.30341
76517	VBR	2020-09-29	111.11	111.11	109.13	110.03	479161	{}	\N	2020-10-12 16:26:36.316773
76518	VBR	2020-09-30	110.53	111.8764	109.75	110.63	557902	{}	\N	2020-10-12 16:26:36.323632
76519	VBR	2020-10-01	111.12	111.95	110.46	111.89	451512	{}	\N	2020-10-12 16:26:36.329268
76520	VBR	2020-10-02	109.95	114.0794	109.7424	113.56	334014	{}	\N	2020-10-12 16:26:36.334293
76521	VBR	2020-10-05	114.75	116.24	114.6595	116.19	437175	{}	\N	2020-10-12 16:26:36.33909
76522	VBR	2020-10-06	117.01	118.77	115.49	115.69	564728	{}	\N	2020-10-12 16:26:36.343361
76523	VBR	2020-10-07	116.99	118.06	116.98	117.68	383351	{}	\N	2020-10-12 16:26:36.347781
76524	VBR	2020-10-08	118.81	119.67	118.32	119.62	401494	{}	\N	2020-10-12 16:26:36.353385
76525	VBR	2020-10-09	120.53	120.98	119.18	119.58	333450	{}	\N	2020-10-12 16:26:36.357613
76526	VBR	2020-10-12	120	120.47	119.58	120.31	126847	{}	\N	2020-10-12 16:26:36.361868
76527	VCV	2020-09-28	12.58	12.59	12.45	12.51	67152	{}	\N	2020-10-12 16:26:37.988278
76528	VCV	2020-09-29	12.55	12.55	12.5	12.52	56032	{}	\N	2020-10-12 16:26:38.000031
76529	VCV	2020-09-30	12.58	12.59	12.45	12.49	183936	{}	\N	2020-10-12 16:26:38.007338
76530	VCV	2020-10-01	12.55	12.56	12.5	12.53	175268	{}	\N	2020-10-12 16:26:38.012589
76531	VCV	2020-10-02	12.54	12.54	12.45	12.46	192646	{}	\N	2020-10-12 16:26:38.017698
76532	VCV	2020-10-05	12.5	12.5	12.35	12.38	206380	{}	\N	2020-10-12 16:26:38.022495
76533	VCV	2020-10-06	12.45	12.45	12.35	12.38	108182	{}	\N	2020-10-12 16:26:38.026759
76534	VCV	2020-10-07	12.41	12.47	12.38	12.45	197340	{}	\N	2020-10-12 16:26:38.031103
76535	VCV	2020-10-08	12.51	12.56	12.49	12.53	116758	{}	\N	2020-10-12 16:26:38.035551
76536	VCV	2020-10-09	12.59	12.59	12.525	12.59	61130	{}	\N	2020-10-12 16:26:38.039743
76537	VCV	2020-10-12	12.67	12.67	12.59	12.59	20522	{}	\N	2020-10-12 16:26:38.044225
76538	VEC	2020-09-28	39.63	40.02	39.365	39.43	39224	{}	\N	2020-10-12 16:26:39.044107
76539	VEC	2020-09-29	39.13	39.4	38.37	39.07	240958	{}	\N	2020-10-12 16:26:39.058241
76540	VEC	2020-09-30	38.98	39.55	37.77	38	237684	{}	\N	2020-10-12 16:26:39.065832
76541	VEC	2020-10-01	38.15	38.53	37.41	37.61	112872	{}	\N	2020-10-12 16:26:39.071958
76542	VEC	2020-10-02	36.83	38.71	36.83	38.27	158533	{}	\N	2020-10-12 16:26:39.077321
76543	VEC	2020-10-05	38.45	39.28	38.45	38.59	113227	{}	\N	2020-10-12 16:26:39.082585
76544	VEC	2020-10-06	39.22	39.89	38.4	38.46	172209	{}	\N	2020-10-12 16:26:39.087259
76545	VEC	2020-10-07	39.06	39.37	38.5	38.86	112082	{}	\N	2020-10-12 16:26:39.091757
76546	VEC	2020-10-08	39.3	39.65	38.84	39.17	122354	{}	\N	2020-10-12 16:26:39.096569
76547	VEC	2020-10-09	39.48	40.1	39.08	39.62	70173	{}	\N	2020-10-12 16:26:39.101141
76548	VEC	2020-10-12	39.53	40.39	39.46	40.34	48060	{}	\N	2020-10-12 16:26:39.105964
76549	VET	2020-09-28	2.44	2.58	2.43	2.54	1547852	{}	\N	2020-10-12 16:26:40.354829
76550	VET	2020-09-29	2.52	2.52	2.33	2.38	2752561	{}	\N	2020-10-12 16:26:40.3683
76551	VET	2020-09-30	2.4	2.46	2.33	2.34	2130106	{}	\N	2020-10-12 16:26:40.375665
76552	VET	2020-10-01	2.33	2.35	2.2286	2.25	2841637	{}	\N	2020-10-12 16:26:40.381744
76553	VET	2020-10-02	2.16	2.42	2.1307	2.38	2905730	{}	\N	2020-10-12 16:26:40.387533
76554	VET	2020-10-05	2.45	2.54	2.375	2.52	2381254	{}	\N	2020-10-12 16:26:40.39262
76555	VET	2020-10-06	2.64	2.6986	2.46	2.48	3723495	{}	\N	2020-10-12 16:26:40.397787
76556	VET	2020-10-07	2.49	2.64	2.48	2.59	2922018	{}	\N	2020-10-12 16:26:40.402995
76557	VET	2020-10-08	2.61	2.85	2.61	2.84	2462244	{}	\N	2020-10-12 16:26:40.408129
76558	VET	2020-10-09	2.86	2.91	2.72	2.84	2804990	{}	\N	2020-10-12 16:26:40.41345
76559	VET	2020-10-12	2.82	2.82	2.75	2.79	367129	{}	\N	2020-10-12 16:26:40.418853
76560	ECPG	2020-09-28	37.9	40.76	37.7	39.44	489213	{}	\N	2020-10-12 16:26:42.644307
76561	ECPG	2020-09-29	39.14	39.15	37.39	38.66	241474	{}	\N	2020-10-12 16:26:42.657396
76562	ECPG	2020-09-30	39.07	39.98	38.19	38.59	356862	{}	\N	2020-10-12 16:26:42.665151
76563	ECPG	2020-10-01	38.49	39.3	37.5618	39.26	432586	{}	\N	2020-10-12 16:26:42.671698
76564	ECPG	2020-10-02	38.41	39.68	38.39	39.04	266980	{}	\N	2020-10-12 16:26:42.677805
76565	ECPG	2020-10-05	39.46	40.08	39.05	39.75	249319	{}	\N	2020-10-12 16:26:42.682854
76566	ECPG	2020-10-06	40.24	40.33	38.25	38.27	246389	{}	\N	2020-10-12 16:26:42.687699
76567	ECPG	2020-10-07	38.61	39.21	37.295	37.54	376588	{}	\N	2020-10-12 16:26:42.692539
76568	ECPG	2020-10-08	38.19	38.58	37.35	38.43	288935	{}	\N	2020-10-12 16:26:42.697368
76569	ECPG	2020-10-09	38.81	39.7	38.12	38.43	215676	{}	\N	2020-10-12 16:26:42.703989
76570	ECPG	2020-10-12	38.33	38.335	37.47	37.82	68650	{}	\N	2020-10-12 16:26:42.709148
76571	USAS	2020-09-28	2.63	2.67	2.56	2.6	447574	{}	\N	2020-10-12 16:26:45.279922
76572	USAS	2020-09-29	2.6	2.7	2.6	2.67	507229	{}	\N	2020-10-12 16:26:45.293472
76573	USAS	2020-09-30	2.65	2.6784	2.57	2.64	626936	{}	\N	2020-10-12 16:26:45.30264
76574	USAS	2020-10-01	2.69	2.69	2.6	2.61	487835	{}	\N	2020-10-12 16:26:45.308811
76575	USAS	2020-10-02	2.58	2.62	2.53	2.57	746325	{}	\N	2020-10-12 16:26:45.314325
76576	USAS	2020-10-05	2.6	2.6327	2.54	2.6	671231	{}	\N	2020-10-12 16:26:45.31952
76577	USAS	2020-10-06	2.61	2.625	2.44	2.44	841925	{}	\N	2020-10-12 16:26:45.324214
76578	USAS	2020-10-07	2.46	2.57	2.46	2.56	516118	{}	\N	2020-10-12 16:26:45.329093
76579	USAS	2020-10-08	2.54	2.58	2.5	2.55	397576	{}	\N	2020-10-12 16:26:45.33386
76580	USAS	2020-10-09	2.6	2.815	2.6	2.8	1084252	{}	\N	2020-10-12 16:26:45.338536
76581	USAS	2020-10-12	2.82	2.835	2.705	2.75	474489	{}	\N	2020-10-12 16:26:45.343237
76582	USD	2020-09-28	73.44	74.89	71.88	74.86	49877	{}	\N	2020-10-12 16:26:48.33371
76583	USD	2020-09-29	74.88	76.6977	74.76	75.52	33883	{}	\N	2020-10-12 16:26:48.343817
76584	USD	2020-09-30	75.18	77.79	75.18	76.14	28159	{}	\N	2020-10-12 16:26:48.349911
76585	USD	2020-10-01	78.58	79.44	77.5	79.08	44303	{}	\N	2020-10-12 16:26:48.355213
76586	USD	2020-10-02	75.15	77.66	74.15	74.15	57245	{}	\N	2020-10-12 16:26:48.360292
76587	USD	2020-10-05	75.8	79.08	75.8	79.05	26686	{}	\N	2020-10-12 16:26:48.365158
76588	USD	2020-10-06	79.3	82.38	78.12	78.73	44102	{}	\N	2020-10-12 16:26:48.369691
76589	USD	2020-10-07	81.521	82.05	80.82	81.78	28575	{}	\N	2020-10-12 16:26:48.374138
76590	USD	2020-10-08	83.04	83.441	82.22	83.02	30540	{}	\N	2020-10-12 16:26:48.378161
76591	USD	2020-10-09	84.61	85	83.66	84.54	38956	{}	\N	2020-10-12 16:26:48.382432
76592	USD	2020-10-12	86.5	87.19	85.9738	87.19	42165	{}	\N	2020-10-12 16:26:48.386461
76593	USLM	2020-09-28	89.25	91.1	88.95	89.7	11700	{}	\N	2020-10-12 16:26:51.786706
76594	USLM	2020-09-29	90.1	90.14	89	90.06	3032	{}	\N	2020-10-12 16:26:51.795258
76595	USLM	2020-09-30	90.94	92.81	90.1	90.1	4553	{}	\N	2020-10-12 16:26:51.801177
76596	USLM	2020-10-01	91.01	91.01	89.9	89.9	3395	{}	\N	2020-10-12 16:26:51.807393
76597	USLM	2020-10-02	90.98	92.41	90.2	90.6	4021	{}	\N	2020-10-12 16:26:51.812402
76598	USLM	2020-10-05	92.94	92.94	91.23	91.23	1147	{}	\N	2020-10-12 16:26:51.816594
76599	USLM	2020-10-06	91.34	91.5	91.32	91.32	1666	{}	\N	2020-10-12 16:26:51.82082
76600	USLM	2020-10-07	93.01	93.01	93.01	93.01	1622	{}	\N	2020-10-12 16:26:51.825265
76601	USLM	2020-10-08	92.56	96.47	92.56	94.59	2871	{}	\N	2020-10-12 16:26:51.829659
76602	USLM	2020-10-09	95.12	95.12	95.12	95.12	1128	{}	\N	2020-10-12 16:26:51.83412
76603	USOI	2020-09-28	4.34	4.3956	4.31	4.3824	189778	{}	\N	2020-10-12 16:26:52.906658
76604	USOI	2020-09-29	4.36	4.3699	4.21	4.26	129342	{}	\N	2020-10-12 16:26:52.912857
76605	USOI	2020-09-30	4.25	4.3389	4.25	4.3	126788	{}	\N	2020-10-12 16:26:52.91765
76606	USOI	2020-10-01	4.28	4.28	4.1401	4.23	258537	{}	\N	2020-10-12 16:26:52.921962
76607	USOI	2020-10-02	4.15	4.16	4.02	4.09	625536	{}	\N	2020-10-12 16:26:52.926117
76608	USOI	2020-10-05	4.24	4.34	4.2	4.31	452697	{}	\N	2020-10-12 16:26:52.930191
76609	USOI	2020-10-06	4.32	4.41	4.3135	4.32	711958	{}	\N	2020-10-12 16:26:52.935018
76610	USOI	2020-10-07	4.32	4.33	4.25	4.31	343231	{}	\N	2020-10-12 16:26:52.940311
76611	USOI	2020-10-08	4.41	4.4355	4.36	4.41	230996	{}	\N	2020-10-12 16:26:52.945115
76612	USOI	2020-10-09	4.44	4.44	4.35	4.37	289519	{}	\N	2020-10-12 16:26:52.949862
76613	USOI	2020-10-12	4.35	4.35	4.261	4.3	73391	{}	\N	2020-10-12 16:26:52.954724
76614	UTG	2020-09-28	31.33	31.35	31.045	31.1	145591	{}	\N	2020-10-12 16:26:54.238045
76615	UTG	2020-09-29	31.15	31.167	30.9805	31.07	92396	{}	\N	2020-10-12 16:26:54.251333
76616	UTG	2020-09-30	31.05	31.47	31.05	31.29	192568	{}	\N	2020-10-12 16:26:54.258923
76617	UTG	2020-10-01	31.2	31.48	31.15	31.4	146863	{}	\N	2020-10-12 16:26:54.26536
76618	UTG	2020-10-02	31.15	31.64	31.15	31.54	134589	{}	\N	2020-10-12 16:26:54.270686
76619	UTG	2020-10-05	31.55	31.8	31.55	31.75	156410	{}	\N	2020-10-12 16:26:54.275391
76620	UTG	2020-10-06	32.19	32.23	31.81	32.03	168550	{}	\N	2020-10-12 16:26:54.279567
76621	UTG	2020-10-07	32.12	32.3	31.95	32.17	182328	{}	\N	2020-10-12 16:26:54.284121
76622	UTG	2020-10-08	32.27	32.54	32.2215	32.51	132957	{}	\N	2020-10-12 16:26:54.288676
76623	UTG	2020-10-09	32.6	32.65	32.3	32.3	126673	{}	\N	2020-10-12 16:26:54.292833
76624	UTG	2020-10-12	32.32	32.75	32.32	32.3911	69137	{}	\N	2020-10-12 16:26:54.297197
76625	UUU	2020-09-28	1.75	1.87	1.745	1.8	27165	{}	\N	2020-10-12 16:26:55.804298
76626	UUU	2020-09-29	1.83	2.01	1.761	1.97	98064	{}	\N	2020-10-12 16:26:55.816478
76627	UUU	2020-09-30	2.07	2.35	1.86	1.9	306977	{}	\N	2020-10-12 16:26:55.823815
76628	UUU	2020-10-01	1.85	2.4	1.85	2.17	171061	{}	\N	2020-10-12 16:26:55.830459
76629	UUU	2020-10-02	2.08	2.35	1.94	2.3	89579	{}	\N	2020-10-12 16:26:55.835787
76630	UUU	2020-10-05	2.38	2.41	2.17	2.24	46101	{}	\N	2020-10-12 16:26:55.840853
76631	UUU	2020-10-06	2.26	2.26	2.1	2.11	28670	{}	\N	2020-10-12 16:26:55.845758
76632	UUU	2020-10-07	2.19	2.4	2.16	2.3	74782	{}	\N	2020-10-12 16:26:55.850294
76633	UUU	2020-10-08	2.26	2.55	2.23	2.51	135169	{}	\N	2020-10-12 16:26:55.854746
76634	UUU	2020-10-09	2.31	2.3565	2.17	2.325	51250	{}	\N	2020-10-12 16:26:55.859053
76635	UUU	2020-10-12	2.25	2.4317	2.23	2.28	14302	{}	\N	2020-10-12 16:26:55.863428
76636	UYG	2020-09-28	30.39	31.35	30.39	31.05	39915	{}	\N	2020-10-12 16:26:57.282719
76637	UYG	2020-09-29	30.86	30.93	30.22	30.4	15376	{}	\N	2020-10-12 16:26:57.293842
76638	UYG	2020-09-30	30.65	31.43	30.58	30.91	39551	{}	\N	2020-10-12 16:26:57.300783
76639	UYG	2020-10-01	31.05	31.7599	30.88	31.49	32566	{}	\N	2020-10-12 16:26:57.306608
76640	UYG	2020-10-02	30.451	32.24	30.451	31.91	71010	{}	\N	2020-10-12 16:26:57.311485
76641	UYG	2020-10-05	32.57	32.89	32.29	32.84	25428	{}	\N	2020-10-12 16:26:57.31648
76642	UYG	2020-10-06	33.22	33.71	32.08	32.24	34796	{}	\N	2020-10-12 16:26:57.320599
76643	UYG	2020-10-07	32.86	33.26	32.64	33.05	51692	{}	\N	2020-10-12 16:26:57.324641
76644	UYG	2020-10-08	33.45	34.035	33.43	33.95	33606	{}	\N	2020-10-12 16:26:57.328766
76645	UYG	2020-10-09	34.23	34.37	33.64	34.03	74665	{}	\N	2020-10-12 16:26:57.332898
76646	UYG	2020-10-12	34.05	34.61	34	34.4252	32075	{}	\N	2020-10-12 16:26:57.337209
76647	VALE	2020-09-28	10.88	10.92	10.45	10.49	38090613	{}	\N	2020-10-12 16:26:58.394697
76648	VALE	2020-09-29	10.47	10.715	10.4	10.41	56857113	{}	\N	2020-10-12 16:26:58.404935
76649	VALE	2020-09-30	10.555	10.73	10.465	10.58	53792645	{}	\N	2020-10-12 16:26:58.41079
76650	VALE	2020-10-01	10.58	10.615	10.29	10.41	42728792	{}	\N	2020-10-12 16:26:58.415584
76651	VALE	2020-10-02	10.33	10.535	10.31	10.39	48339421	{}	\N	2020-10-12 16:26:58.420271
76652	VALE	2020-10-05	10.57	10.785	10.53	10.75	47321292	{}	\N	2020-10-12 16:26:58.424934
76653	VALE	2020-10-06	10.87	10.93	10.425	10.5	48952056	{}	\N	2020-10-12 16:26:58.429159
76654	VALE	2020-10-07	10.68	10.88	10.58	10.75	60719327	{}	\N	2020-10-12 16:26:58.433116
76655	VALE	2020-10-08	10.75	11.06	10.71	11.05	57195048	{}	\N	2020-10-12 16:26:58.437087
76656	VALE	2020-10-09	11.16	11.205	11.05	11.19	52231642	{}	\N	2020-10-12 16:26:58.441146
76657	VALE	2020-10-12	11.27	11.29	11.18	11.275	12414236	{}	\N	2020-10-12 16:26:58.445088
76658	VCLT	2020-09-28	106.63	107.42	106.63	107.42	592871	{}	\N	2020-10-12 16:27:00.409774
76659	VCLT	2020-09-29	107.53	107.6789	106.93	107.11	411009	{}	\N	2020-10-12 16:27:00.420229
76660	VCLT	2020-09-30	106.9	107.135	106.5101	106.88	410062	{}	\N	2020-10-12 16:27:00.427283
76661	VCLT	2020-10-01	106.6	107.28	106.5	107.01	400346	{}	\N	2020-10-12 16:27:00.433173
76662	VCLT	2020-10-02	106.88	107.16	106.43	107.05	309429	{}	\N	2020-10-12 16:27:00.437878
76663	VCLT	2020-10-05	106.56	106.93	106.16	106.31	582184	{}	\N	2020-10-12 16:27:00.443001
76664	VCLT	2020-10-06	106.47	107.24	106	106.5	1881860	{}	\N	2020-10-12 16:27:00.447641
76665	VCLT	2020-10-07	106.24	106.69	105.7908	106.12	804292	{}	\N	2020-10-12 16:27:00.452505
76666	VCLT	2020-10-08	106.73	106.88	106.34	106.47	567591	{}	\N	2020-10-12 16:27:00.457107
76667	VCLT	2020-10-09	106.4	107.02	106.22	106.96	1351145	{}	\N	2020-10-12 16:27:00.46134
76668	VCLT	2020-10-12	107.37	107.74	107.23	107.735	145999	{}	\N	2020-10-12 16:27:00.465566
76669	USB	2020-09-28	35.88	36.4645	35.67	36.1	5515066	{}	\N	2020-10-12 16:27:03.628754
76670	USB	2020-09-29	35.5	35.58	34.72	35.3	10357810	{}	\N	2020-10-12 16:27:03.639726
76671	USB	2020-09-30	35.45	36.17	35.34	35.85	18767865	{}	\N	2020-10-12 16:27:03.647961
76672	USB	2020-10-01	36.03	36.09	35.4392	36	8647588	{}	\N	2020-10-12 16:27:03.654785
76673	USB	2020-10-02	35.37	36.95	35.25	36.77	10168700	{}	\N	2020-10-12 16:27:03.661513
76674	USB	2020-10-05	37.32	37.915	37.11	37.73	10974527	{}	\N	2020-10-12 16:27:03.666632
76675	USB	2020-10-06	38.15	38.795	37.27	37.42	11782739	{}	\N	2020-10-12 16:27:03.672065
76676	USB	2020-10-07	38.05	39.06	37.99	38.79	14553319	{}	\N	2020-10-12 16:27:03.677568
76677	USB	2020-10-08	39	39.66	38.79	39.5	13572763	{}	\N	2020-10-12 16:27:03.682924
76678	USB	2020-10-09	39.7	39.76	38.58	39.17	12707848	{}	\N	2020-10-12 16:27:03.688311
76679	USB	2020-10-12	39.03	39.6936	38.79	39.4218	6212034	{}	\N	2020-10-12 16:27:03.693731
76680	UTF	2020-09-28	21.99	22.305	21.99	22.22	185670	{}	\N	2020-10-12 16:27:05.137533
76681	UTF	2020-09-29	22.15	22.2726	21.92	22.06	374292	{}	\N	2020-10-12 16:27:05.152006
76682	UTF	2020-09-30	22.11	22.5	22.0701	22.23	541269	{}	\N	2020-10-12 16:27:05.160403
76683	UTF	2020-10-01	22.23	22.7	22.23	22.44	400854	{}	\N	2020-10-12 16:27:05.168582
76684	UTF	2020-10-02	22.14	22.49	22.09	22.28	290153	{}	\N	2020-10-12 16:27:05.174919
76685	UTF	2020-10-05	22.23	22.63	22.23	22.55	333933	{}	\N	2020-10-12 16:27:05.180772
76686	UTF	2020-10-06	22.55	23.05	22.55	22.82	375412	{}	\N	2020-10-12 16:27:05.185984
76687	UTF	2020-10-07	22.97	23.25	22.95	23.15	383238	{}	\N	2020-10-12 16:27:05.191269
76688	UTF	2020-10-08	23.24	23.42	23.2037	23.34	301630	{}	\N	2020-10-12 16:27:05.196461
76689	UTF	2020-10-09	23.5	23.5	23.34	23.34	352399	{}	\N	2020-10-12 16:27:05.201905
76690	UTF	2020-10-12	23.34	23.6425	23.34	23.5898	215894	{}	\N	2020-10-12 16:27:05.207617
76691	UUUU	2020-09-28	1.66	1.73	1.61	1.72	1011836	{}	\N	2020-10-12 16:27:07.289809
76692	UUUU	2020-09-29	1.74	1.75	1.68	1.74	867903	{}	\N	2020-10-12 16:27:07.304469
76693	UUUU	2020-09-30	1.73	1.73	1.65	1.68	670115	{}	\N	2020-10-12 16:27:07.313378
76694	UUUU	2020-10-01	1.72	1.76	1.65	1.74	1995432	{}	\N	2020-10-12 16:27:07.320511
76695	UUUU	2020-10-02	1.66	1.69	1.62	1.64	1106368	{}	\N	2020-10-12 16:27:07.326995
76696	UUUU	2020-10-05	1.67	1.71	1.6221	1.7	1013605	{}	\N	2020-10-12 16:27:07.332807
76697	UUUU	2020-10-06	1.72	1.79	1.65	1.67	2406355	{}	\N	2020-10-12 16:27:07.338379
76698	UUUU	2020-10-07	1.7	1.77	1.665	1.75	1594217	{}	\N	2020-10-12 16:27:07.343959
76699	UUUU	2020-10-08	1.76	1.79	1.7	1.7	1113680	{}	\N	2020-10-12 16:27:07.349747
76700	UUUU	2020-10-09	1.73	1.74	1.68	1.7	777732	{}	\N	2020-10-12 16:27:07.355406
76701	UUUU	2020-10-12	1.71	1.7168	1.6259	1.6621	575025	{}	\N	2020-10-12 16:27:07.360979
76702	UZA	2020-09-28	25.26	25.6964	25.26	25.69	49658	{}	\N	2020-10-12 16:27:08.29148
76703	UZA	2020-09-29	25.69	25.855	25.64	25.855	36018	{}	\N	2020-10-12 16:27:08.302932
76704	UZA	2020-09-30	25.88	26.33	25.6512	26.27	206564	{}	\N	2020-10-12 16:27:08.310264
76705	UZA	2020-10-01	26.19	26.27	25.92	26.2	92486	{}	\N	2020-10-12 16:27:08.316525
76706	UZA	2020-10-02	26.01	26.2	25.88	26.14	42480	{}	\N	2020-10-12 16:27:08.322586
76707	UZA	2020-10-05	26.15	26.24	26.01	26.06	58524	{}	\N	2020-10-12 16:27:08.327667
76708	UZA	2020-10-06	26.109	26.205	25.91	26.04	47856	{}	\N	2020-10-12 16:27:08.332672
76709	UZA	2020-10-07	26.1	26.25	26.0977	26.25	46842	{}	\N	2020-10-12 16:27:08.337771
76710	UZA	2020-10-08	26.14	26.25	26.03	26.23	27278	{}	\N	2020-10-12 16:27:08.343048
76711	UZA	2020-10-09	26.13	26.25	25.67	25.9	37758	{}	\N	2020-10-12 16:27:08.348525
76712	UZA	2020-10-12	25.9	26.12	25.9	26.07	5686	{}	\N	2020-10-12 16:27:08.353737
76713	VB	2020-09-28	152.75	154.46	152.73	154.01	385136	{}	\N	2020-10-12 16:27:09.74277
76714	VB	2020-09-29	154	154.376	152.27	153.2	478908	{}	\N	2020-10-12 16:27:09.752982
76715	VB	2020-09-30	153.68	155.49	152.6301	153.8	810667	{}	\N	2020-10-12 16:27:09.759307
76716	VB	2020-10-01	154.74	156	153.955	156	1065589	{}	\N	2020-10-12 16:27:09.765031
76717	VB	2020-10-02	153.13	157.35	153.05	156.78	663894	{}	\N	2020-10-12 16:27:09.770224
76718	VB	2020-10-05	158.18	160.45	158.09	160.31	743012	{}	\N	2020-10-12 16:27:09.775466
76719	VB	2020-10-06	161.28	163.6802	159.58	159.93	763218	{}	\N	2020-10-12 16:27:09.78061
76720	VB	2020-10-07	161.63	163.45	161.59	163.03	813344	{}	\N	2020-10-12 16:27:09.78566
76721	VB	2020-10-08	164.36	165.01	163.7031	165.01	756883	{}	\N	2020-10-12 16:27:09.791169
76722	VB	2020-10-09	166.24	166.485	165.05	165.78	993612	{}	\N	2020-10-12 16:27:09.797079
76723	VB	2020-10-12	166.53	167.06	165.838	166.7319	162785	{}	\N	2020-10-12 16:27:09.8038
76724	VCF	2020-09-28	13.8261	13.8901	13.8261	13.8901	1453	{}	\N	2020-10-12 16:27:11.12329
76725	VCF	2020-09-29	13.885	13.885	13.87	13.8771	3285	{}	\N	2020-10-12 16:27:11.134925
76726	VCF	2020-09-30	13.75	13.9	13.75	13.9	1720	{}	\N	2020-10-12 16:27:11.141959
76727	VCF	2020-10-01	13.8999	13.91	13.86	13.86	4433	{}	\N	2020-10-12 16:27:11.148043
76728	VCF	2020-10-02	13.77	13.8999	13.77	13.8401	2620	{}	\N	2020-10-12 16:27:11.153926
76729	VCF	2020-10-05	13.81	13.8862	13.77	13.79	5306	{}	\N	2020-10-12 16:27:11.159167
76730	VCF	2020-10-06	14.3146	14.3146	13.8	13.8	3499	{}	\N	2020-10-12 16:27:11.1646
76731	VCF	2020-10-07	13.8764	13.8764	13.8764	13.8764	1519	{}	\N	2020-10-12 16:27:11.169661
76732	VCF	2020-10-08	13.88	13.91	13.76	13.87	12142	{}	\N	2020-10-12 16:27:11.17492
76733	VCF	2020-10-09	13.79	13.85	13.79	13.85	2635	{}	\N	2020-10-12 16:27:11.180274
76734	VCF	2020-10-12	13.91	13.91	13.91	13.91	253	{}	\N	2020-10-12 16:27:11.185278
76735	VEA	2020-09-28	40.97	41.07	40.9	41.06	5853909	{}	\N	2020-10-12 16:27:12.195289
76736	VEA	2020-09-29	41.03	41.15	40.835	41	6725999	{}	\N	2020-10-12 16:27:12.208971
76737	VEA	2020-09-30	40.91	41.16	40.76	40.86	11467287	{}	\N	2020-10-12 16:27:12.217838
76738	VEA	2020-10-01	41.1	41.16	40.9	41.14	7698423	{}	\N	2020-10-12 16:27:12.224568
76739	VEA	2020-10-02	40.63	41.135	40.6	41.05	10338433	{}	\N	2020-10-12 16:27:12.230569
76740	VEA	2020-10-05	41.41	41.7	41.39	41.69	5002455	{}	\N	2020-10-12 16:27:12.235878
76741	VEA	2020-10-06	41.7	41.77	41.17	41.26	7551290	{}	\N	2020-10-12 16:27:12.240948
76742	VEA	2020-10-07	41.61	41.71	41.5	41.65	7726469	{}	\N	2020-10-12 16:27:12.24612
76743	VEA	2020-10-08	41.85	41.97	41.79	41.96	9923562	{}	\N	2020-10-12 16:27:12.251255
76744	VEA	2020-10-09	42.16	42.2799	42.08	42.24	5339119	{}	\N	2020-10-12 16:27:12.256296
76745	VEA	2020-10-12	42.32	42.44	42.28	42.431	1661207	{}	\N	2020-10-12 16:27:12.26137
76746	VEON	2020-09-28	1.29	1.3	1.27	1.27	3172934	{}	\N	2020-10-12 16:27:13.276185
76747	VEON	2020-09-29	1.29	1.29	1.25	1.25	2086734	{}	\N	2020-10-12 16:27:13.286324
76748	VEON	2020-09-30	1.26	1.28	1.25	1.26	2046851	{}	\N	2020-10-12 16:27:13.292929
76749	VEON	2020-10-01	1.26	1.275	1.24	1.25	3242408	{}	\N	2020-10-12 16:27:13.298947
76750	VEON	2020-10-02	1.25	1.26	1.24	1.25	2041045	{}	\N	2020-10-12 16:27:13.30485
76751	VEON	2020-10-05	1.26	1.26	1.25	1.26	2316654	{}	\N	2020-10-12 16:27:13.310062
76752	VEON	2020-10-06	1.27	1.34	1.26	1.28	4795221	{}	\N	2020-10-12 16:27:13.314959
76753	VEON	2020-10-07	1.29	1.31	1.28	1.3	2402829	{}	\N	2020-10-12 16:27:13.319813
76754	VEON	2020-10-08	1.3	1.32	1.29	1.29	2083284	{}	\N	2020-10-12 16:27:13.324911
76755	VEON	2020-10-09	1.3	1.36	1.3	1.36	4682524	{}	\N	2020-10-12 16:27:13.32976
76756	VEON	2020-10-12	1.36	1.38	1.35	1.38	1034727	{}	\N	2020-10-12 16:27:13.334767
76757	VFH	2020-09-28	57.81	58.86	57.7	58.46	742205	{}	\N	2020-10-12 16:27:15.110338
76758	VFH	2020-09-29	58.36	58.39	57.43	57.82	296642	{}	\N	2020-10-12 16:27:15.121001
76759	VFH	2020-09-30	58	59.01	57.99	58.5	328401	{}	\N	2020-10-12 16:27:15.127425
76760	VFH	2020-10-01	58.71	59.19	58.24	58.76	599180	{}	\N	2020-10-12 16:27:15.133507
76761	VFH	2020-10-02	57.77	59.6567	57.77	59.31	611108	{}	\N	2020-10-12 16:27:15.138538
76762	VFH	2020-10-05	59.83	60.6	59.83	60.43	413388	{}	\N	2020-10-12 16:27:15.143398
76763	VFH	2020-10-06	60.9	61.52	59.781	59.87	764109	{}	\N	2020-10-12 16:27:15.148248
76764	VFH	2020-10-07	60.55	61.25	60.47	60.85	981419	{}	\N	2020-10-12 16:27:15.152698
76765	VFH	2020-10-08	61.21	61.95	61.18	61.86	1291332	{}	\N	2020-10-12 16:27:15.157284
76766	VFH	2020-10-09	62.13	62.3	61.47	61.81	481271	{}	\N	2020-10-12 16:27:15.164811
76767	VFH	2020-10-12	61.89	62.5118	61.81	62.345	178073	{}	\N	2020-10-12 16:27:15.168761
76768	VGSH	2020-09-28	62.09	62.11	62.09	62.1	1648782	{}	\N	2020-10-12 16:27:17.077419
76769	VGSH	2020-09-29	62.09	62.11	62.09	62.11	731503	{}	\N	2020-10-12 16:27:17.090455
76770	VGSH	2020-09-30	62.1	62.11	62.09	62.09	1457959	{}	\N	2020-10-12 16:27:17.097785
76771	VGSH	2020-10-01	62.05	62.06	62.04	62.06	1097692	{}	\N	2020-10-12 16:27:17.103904
76772	VGSH	2020-10-02	62.07	62.07	62.04	62.05	2069928	{}	\N	2020-10-12 16:27:17.109127
76773	VGSH	2020-10-05	62.05	62.05	62.02	62.02	1496029	{}	\N	2020-10-12 16:27:17.113852
76774	VGSH	2020-10-06	62.03	62.04	62.02	62.04	965541	{}	\N	2020-10-12 16:27:17.118232
76775	VGSH	2020-10-07	62.03	62.03	62.02	62.03	1048539	{}	\N	2020-10-12 16:27:17.122259
76776	VGSH	2020-10-08	62.03	62.05	62.03	62.05	1253429	{}	\N	2020-10-12 16:27:17.126445
76777	VGSH	2020-10-09	62.04	62.04	62.03	62.04	807236	{}	\N	2020-10-12 16:27:17.130468
76778	VGSH	2020-10-12	62.04	62.04	62.03	62.03	345308	{}	\N	2020-10-12 16:27:17.134572
76779	UTL	2020-09-28	38.1	39.51	38.1	38.83	59874	{}	\N	2020-10-12 16:27:19.091803
76780	UTL	2020-09-29	38.72	38.86	37.9656	38.47	69441	{}	\N	2020-10-12 16:27:19.104457
76781	UTL	2020-09-30	38.6	38.93	38.36	38.64	100207	{}	\N	2020-10-12 16:27:19.111584
76782	UTL	2020-10-01	38.46	39.29	38.38	39.26	138966	{}	\N	2020-10-12 16:27:19.11758
76783	UTL	2020-10-02	38.74	39.84	38.74	39.46	85380	{}	\N	2020-10-12 16:27:19.122458
76784	UTL	2020-10-05	39.42	40.4799	39.26	39.82	91871	{}	\N	2020-10-12 16:27:19.127555
76785	UTL	2020-10-06	40	40.16	39.3	39.76	165763	{}	\N	2020-10-12 16:27:19.131528
76786	UTL	2020-10-07	39.7	39.87	39.2	39.6	113258	{}	\N	2020-10-12 16:27:19.135449
76787	UTL	2020-10-08	39.75	40.75	39.61	40.38	110314	{}	\N	2020-10-12 16:27:19.139405
76788	UTL	2020-10-09	40.8	41.31	40.63	41.08	85640	{}	\N	2020-10-12 16:27:19.143883
76789	UTL	2020-10-12	41.22	41.46	40.86	41.175	25597	{}	\N	2020-10-12 16:27:19.14817
76790	UVXY	2020-09-28	20.51	20.89	20.42	20.65	18564619	{}	\N	2020-10-12 16:27:20.986689
76791	UVXY	2020-09-29	20.7	20.71	19.57	20.03	30008227	{}	\N	2020-10-12 16:27:20.999661
76792	UVXY	2020-09-30	19.6299	20.28	19.25	19.92	31093307	{}	\N	2020-10-12 16:27:21.006627
76793	UVXY	2020-10-01	19.64	20.57	19.62	20.11	23211501	{}	\N	2020-10-12 16:27:21.012337
76794	UVXY	2020-10-02	21.6499	21.76	20.4501	21.03	42691998	{}	\N	2020-10-12 16:27:21.017321
76795	UVXY	2020-10-05	20.72	21.04	19.8	20.03	20615622	{}	\N	2020-10-12 16:27:21.022576
76796	UVXY	2020-10-06	19.77	20.73	19.225	20.41	40328669	{}	\N	2020-10-12 16:27:21.027014
76797	UVXY	2020-10-07	19.85	19.99	19.26	19.46	24438123	{}	\N	2020-10-12 16:27:21.031779
76798	UVXY	2020-10-08	19.13	19.29	18.24	18.33	33240206	{}	\N	2020-10-12 16:27:21.036816
76799	UVXY	2020-10-09	17.44	17.49	16.86	16.89	33148272	{}	\N	2020-10-12 16:27:21.041347
76800	UVXY	2020-10-12	16.42	16.8599	16.33	16.72	11212209	{}	\N	2020-10-12 16:27:21.045876
76801	VALQ	2020-09-28	37.59	37.69	37.55	37.6	2511	{}	\N	2020-10-12 16:27:21.965976
76802	VALQ	2020-09-29	37.632	37.632	37.3371	37.5192	834	{}	\N	2020-10-12 16:27:21.976635
76803	VALQ	2020-09-30	37.42	37.965	37.42	37.51	9886	{}	\N	2020-10-12 16:27:21.983329
76804	VALQ	2020-10-01	37.7098	37.7634	37.6931	37.6931	1291	{}	\N	2020-10-12 16:27:21.989116
76805	VALQ	2020-10-02	37.5698	38.06	37.56	38.01	24353	{}	\N	2020-10-12 16:27:21.994582
76806	VALQ	2020-10-05	38.2717	38.56	38.265	38.49	27048	{}	\N	2020-10-12 16:27:21.999745
76807	VALQ	2020-10-06	38.76	39	38.18	38.41	27414	{}	\N	2020-10-12 16:27:22.00448
76808	VALQ	2020-10-07	38.66	38.8	38.6176	38.8	5367	{}	\N	2020-10-12 16:27:22.008949
76809	VALQ	2020-10-08	39	39.125	39	39.1199	6783	{}	\N	2020-10-12 16:27:22.013538
76810	VALQ	2020-10-09	39.46	39.46	39.165	39.165	3495	{}	\N	2020-10-12 16:27:22.018036
76811	VALQ	2020-10-12	39.48	39.49	39.48	39.49	1130	{}	\N	2020-10-12 16:27:22.022999
76812	VBLT	2020-09-28	1.17	1.1789	1.11	1.12	213948	{}	\N	2020-10-12 16:27:24.729479
76813	VBLT	2020-09-29	1.1	1.2	1.1	1.18	134770	{}	\N	2020-10-12 16:27:24.738891
76814	VBLT	2020-09-30	1.18	1.22	1.17	1.22	95628	{}	\N	2020-10-12 16:27:24.745841
76815	VBLT	2020-10-01	1.21	1.21	1.12	1.16	60712	{}	\N	2020-10-12 16:27:24.752162
76816	VBLT	2020-10-02	1.13	1.17	1.1243	1.15	70483	{}	\N	2020-10-12 16:27:24.75765
76817	VBLT	2020-10-05	1.17	1.18	1.14	1.16	92623	{}	\N	2020-10-12 16:27:24.762406
76818	VBLT	2020-10-06	1.16	1.19	1.15	1.17	44417	{}	\N	2020-10-12 16:27:24.767187
76819	VBLT	2020-10-07	1.16	1.19	1.15	1.18	85844	{}	\N	2020-10-12 16:27:24.772013
76820	VBLT	2020-10-08	1.17	1.22	1.15	1.19	194402	{}	\N	2020-10-12 16:27:24.776812
76821	VBLT	2020-10-09	1.2	1.22	1.18	1.22	89654	{}	\N	2020-10-12 16:27:24.781187
76822	VBLT	2020-10-12	1.25	1.27	1.22	1.22	140874	{}	\N	2020-10-12 16:27:24.785562
76823	VCSH	2020-09-28	82.73	82.81	82.71	82.81	1872775	{}	\N	2020-10-12 16:27:25.95146
76824	VCSH	2020-09-29	82.83	82.86	82.77	82.84	1537664	{}	\N	2020-10-12 16:27:25.964423
76825	VCSH	2020-09-30	82.83	82.86	82.81	82.85	2095964	{}	\N	2020-10-12 16:27:25.971627
76826	VCSH	2020-10-01	82.68	82.8	82.66	82.77	3519484	{}	\N	2020-10-12 16:27:25.977778
76827	VCSH	2020-10-02	82.76	82.81	82.72	82.79	5739956	{}	\N	2020-10-12 16:27:25.983327
76828	VCSH	2020-10-05	82.8	82.82	82.76	82.78	2563955	{}	\N	2020-10-12 16:27:25.987799
76829	VCSH	2020-10-06	82.78	82.83	82.74	82.78	2914328	{}	\N	2020-10-12 16:27:25.991844
76830	VCSH	2020-10-07	82.73	82.8	82.7243	82.78	2320000	{}	\N	2020-10-12 16:27:25.996478
76831	VCSH	2020-10-08	82.8	82.83	82.78	82.83	2264906	{}	\N	2020-10-12 16:27:26.001782
76832	VCSH	2020-10-09	82.84	82.87	82.82	82.87	1996333	{}	\N	2020-10-12 16:27:26.006203
76833	VCSH	2020-10-12	82.92	82.93	82.9	82.925	548939	{}	\N	2020-10-12 16:27:26.01072
76834	VEDL	2020-09-28	7.5	7.51	7.385	7.47	829972	{}	\N	2020-10-12 16:27:27.236919
76835	VEDL	2020-09-29	7.45	7.5177	7.43	7.48	1735094	{}	\N	2020-10-12 16:27:27.24665
76836	VEDL	2020-09-30	7.45	7.45	7.355	7.39	1724403	{}	\N	2020-10-12 16:27:27.253522
76837	VEDL	2020-10-01	7.44	7.475	7.35	7.4	1149080	{}	\N	2020-10-12 16:27:27.259315
76838	VEDL	2020-10-02	7.24	7.355	7.19	7.33	1226529	{}	\N	2020-10-12 16:27:27.264932
76839	VEDL	2020-10-05	7.41	7.46	7.37	7.43	730704	{}	\N	2020-10-12 16:27:27.270061
76840	VEDL	2020-10-06	7.5	7.5	7.33	7.34	948963	{}	\N	2020-10-12 16:27:27.274533
76841	VEDL	2020-10-07	6.69	6.87	6.56	6.72	2597718	{}	\N	2020-10-12 16:27:27.278983
76842	VEDL	2020-10-08	6.64	6.66	6.45	6.46	3992692	{}	\N	2020-10-12 16:27:27.28328
76843	VEDL	2020-10-09	6.56	6.66	5.8001	5.96	9474435	{}	\N	2020-10-12 16:27:27.287672
76844	VEDL	2020-10-12	5.47	5.49	5.2538	5.32	3086660	{}	\N	2020-10-12 16:27:27.291967
76845	VERI	2020-09-28	8.55	9.05	8.37	9.02	412716	{}	\N	2020-10-12 16:27:28.597164
76846	VERI	2020-09-29	8.98	9.38	8.75	9.37	466480	{}	\N	2020-10-12 16:27:28.609441
76847	VERI	2020-09-30	9.42	9.62	8.98	9.16	528285	{}	\N	2020-10-12 16:27:28.616082
76848	VERI	2020-10-01	9.23	9.32	8.9013	9.05	447323	{}	\N	2020-10-12 16:27:28.622169
76849	VERI	2020-10-02	8.75	9.22	8.5	8.85	276111	{}	\N	2020-10-12 16:27:28.627608
76850	VERI	2020-10-05	8.98	9.49	8.91	9.44	409467	{}	\N	2020-10-12 16:27:28.635945
76851	VERI	2020-10-06	9.52	10	9.05	9.11	616563	{}	\N	2020-10-12 16:27:28.641247
76852	VERI	2020-10-07	9.25	10.488	9.2	10.305	747471	{}	\N	2020-10-12 16:27:28.646401
76853	VERI	2020-10-08	10.5	11.64	10.49	11.1	1152199	{}	\N	2020-10-12 16:27:28.651528
76854	VERI	2020-10-09	11.52	11.6	10.5	11.09	727118	{}	\N	2020-10-12 16:27:28.656094
76855	VERI	2020-10-12	11.32	11.49	10.37	10.6	420251	{}	\N	2020-10-12 16:27:28.660526
76856	VG	2020-09-28	10.27	10.36	10.115	10.25	1647231	{}	\N	2020-10-12 16:27:30.059444
76857	VG	2020-09-29	10.25	10.33	9.99	10.24	2190168	{}	\N	2020-10-12 16:27:30.074946
76858	VG	2020-09-30	10.27	10.4	10.13	10.23	3343719	{}	\N	2020-10-12 16:27:30.082517
76859	VG	2020-10-01	10.27	10.36	10.065	10.13	1342634	{}	\N	2020-10-12 16:27:30.088288
76860	VG	2020-10-02	10.01	10.2401	9.93	10.15	1460052	{}	\N	2020-10-12 16:27:30.093573
76861	VG	2020-10-05	10.22	10.2502	10.07	10.25	969087	{}	\N	2020-10-12 16:27:30.098829
76862	VG	2020-10-06	10.28	10.44	10.2	10.24	1817102	{}	\N	2020-10-12 16:27:30.103765
76863	VG	2020-10-07	10.27	10.31	10.13	10.21	2150641	{}	\N	2020-10-12 16:27:30.108916
76864	VG	2020-10-08	10.35	10.53	10.22	10.43	1758537	{}	\N	2020-10-12 16:27:30.113749
76865	VG	2020-10-09	10.54	10.65	10.48	10.64	1208932	{}	\N	2020-10-12 16:27:30.118701
76866	VG	2020-10-12	10.66	10.895	10.64	10.845	588957	{}	\N	2020-10-12 16:27:30.123826
76867	VHC	2020-09-28	4.98	5.185	4.9707	5.16	297556	{}	\N	2020-10-12 16:27:31.519064
76868	VHC	2020-09-29	5.18	5.255	5.11	5.24	376674	{}	\N	2020-10-12 16:27:31.533716
76869	VHC	2020-09-30	5.22	5.3683	5.21	5.27	455578	{}	\N	2020-10-12 16:27:31.541982
76870	VHC	2020-10-01	5.3	5.56	5.27	5.51	826236	{}	\N	2020-10-12 16:27:31.548636
76871	VHC	2020-10-02	5.35	5.39	5.13	5.19	646182	{}	\N	2020-10-12 16:27:31.554181
76872	VHC	2020-10-05	5.27	5.54	5.26	5.53	547202	{}	\N	2020-10-12 16:27:31.559693
76873	VHC	2020-10-06	5.57	5.68	5.45	5.48	561400	{}	\N	2020-10-12 16:27:31.565992
76874	VHC	2020-10-07	5.59	5.7	5.52	5.69	730791	{}	\N	2020-10-12 16:27:31.571184
76875	VHC	2020-10-08	5.78	5.87	5.68	5.76	574534	{}	\N	2020-10-12 16:27:31.576638
76876	VHC	2020-10-09	5.86	6	5.8	5.99	492544	{}	\N	2020-10-12 16:27:31.581756
76877	VHC	2020-10-12	6	6.03	5.82	5.85	278770	{}	\N	2020-10-12 16:27:31.586927
76878	VICR	2020-09-28	76.75	77.99	75.63	75.85	150264	{}	\N	2020-10-12 16:27:32.818435
76879	VICR	2020-09-29	75.67	78.09	75.67	77.87	182926	{}	\N	2020-10-12 16:27:32.833102
76880	VICR	2020-09-30	77.98	79.51	76.586	77.73	191896	{}	\N	2020-10-12 16:27:32.840944
76881	VICR	2020-10-01	78.72	80.05	78.02	78.87	143973	{}	\N	2020-10-12 16:27:32.847828
76882	VICR	2020-10-02	76.67	78.83	76.13	77.39	127701	{}	\N	2020-10-12 16:27:32.854005
76883	VICR	2020-10-05	77.89	82.36	77.39	81.84	136600	{}	\N	2020-10-12 16:27:32.859852
76884	VICR	2020-10-06	82.24	83.35	80.7646	81.5	136964	{}	\N	2020-10-12 16:27:32.864959
76885	VICR	2020-10-07	82.9	83.78	81.29	82.73	106464	{}	\N	2020-10-12 16:27:32.869815
76886	VICR	2020-10-08	83.68	84.18	82.745	83.56	106936	{}	\N	2020-10-12 16:27:32.87467
76887	VICR	2020-10-09	84.56	87.39	84	86.67	126543	{}	\N	2020-10-12 16:27:32.879589
76888	VICR	2020-10-12	87.97	87.97	85.295	85.665	24902	{}	\N	2020-10-12 16:27:32.884597
76889	VIVE	2020-09-28	0.52	0.5303	0.502	0.529	310766	{}	\N	2020-10-12 16:27:34.076087
76890	VIVE	2020-09-29	0.54	0.54	0.5	0.5243	559932	{}	\N	2020-10-12 16:27:34.087642
76891	VIVE	2020-09-30	0.53	0.5334	0.5051	0.5225	807141	{}	\N	2020-10-12 16:27:34.095401
76892	VIVE	2020-10-01	0.525	0.533	0.515	0.5285	320245	{}	\N	2020-10-12 16:27:34.10107
76893	VIVE	2020-10-02	0.5	0.535	0.5	0.53	341024	{}	\N	2020-10-12 16:27:34.106668
76894	VIVE	2020-10-05	0.5295	0.55	0.52	0.5294	519666	{}	\N	2020-10-12 16:27:34.111864
76895	VIVE	2020-10-06	0.5202	0.534799	0.5201	0.5252	322529	{}	\N	2020-10-12 16:27:34.116715
76896	VIVE	2020-10-07	0.54	0.555	0.5201	0.53	264384	{}	\N	2020-10-12 16:27:34.12147
76897	VIVE	2020-10-08	0.53	0.54	0.52	0.5375	293380	{}	\N	2020-10-12 16:27:34.125864
76898	VIVE	2020-10-09	0.5475	0.5485	0.53	0.545	415666	{}	\N	2020-10-12 16:27:34.13033
76899	VIVE	2020-10-12	0.5499	0.55	0.5254	0.54021	210666	{}	\N	2020-10-12 16:27:34.134609
76900	USRT	2020-09-28	43.69	44.37	43.69	44.19	274523	{}	\N	2020-10-12 16:27:35.196413
76901	USRT	2020-09-29	44.15	44.15	43.2825	43.7	132217	{}	\N	2020-10-12 16:27:35.207981
76902	USRT	2020-09-30	43.89	44.34	43.31	43.74	229983	{}	\N	2020-10-12 16:27:35.214959
76903	USRT	2020-10-01	43.93	44.9	43.815	44.88	132076	{}	\N	2020-10-12 16:27:35.22114
76904	USRT	2020-10-02	44.22	45.9283	44.17	45.74	93046	{}	\N	2020-10-12 16:27:35.226036
76905	USRT	2020-10-05	45.86	46.19	45.17	46.01	105385	{}	\N	2020-10-12 16:27:35.231028
76906	USRT	2020-10-06	46.1	46.501	45.61	45.83	82375	{}	\N	2020-10-12 16:27:35.235229
76907	USRT	2020-10-07	46.11	46.11	45.616	45.86	77090	{}	\N	2020-10-12 16:27:35.239432
76908	USRT	2020-10-08	46.01	46.8	46.01	46.69	1030576	{}	\N	2020-10-12 16:27:35.243692
76909	USRT	2020-10-09	46.9	47	46.29	46.46	128306	{}	\N	2020-10-12 16:27:35.247903
76910	USRT	2020-10-12	46.56	46.645	46.1637	46.51	36891	{}	\N	2020-10-12 16:27:35.252379
76911	UTHR	2020-09-28	101.46	102.5	100.24	101.51	195949	{}	\N	2020-10-12 16:27:36.645161
76912	UTHR	2020-09-29	101.82	101.82	100.09	101.04	177556	{}	\N	2020-10-12 16:27:36.658041
76913	UTHR	2020-09-30	101.37	102.851	100.11	101	468850	{}	\N	2020-10-12 16:27:36.665957
76914	UTHR	2020-10-01	101.03	103.59	100.05	103.17	259749	{}	\N	2020-10-12 16:27:36.672785
76915	UTHR	2020-10-02	102.35	104.78	101.09	101.87	273346	{}	\N	2020-10-12 16:27:36.678603
76916	UTHR	2020-10-05	102.83	105.85	102.83	105.69	423511	{}	\N	2020-10-12 16:27:36.684159
76917	UTHR	2020-10-06	105.87	106.6	102.725	103.07	271035	{}	\N	2020-10-12 16:27:36.689133
76918	UTHR	2020-10-07	103.59	104.315	103.06	103.87	235779	{}	\N	2020-10-12 16:27:36.695514
76919	UTHR	2020-10-08	104.18	105.15	103.65	104.89	202445	{}	\N	2020-10-12 16:27:36.700014
76920	UTHR	2020-10-09	105.01	105.435	102.91	104.01	205205	{}	\N	2020-10-12 16:27:36.704083
76921	UTHR	2020-10-12	103.73	104.5	102.25	103.7	103811	{}	\N	2020-10-12 16:27:36.708467
76922	UUP	2020-09-28	25.45	25.5	25.43	25.47	764459	{}	\N	2020-10-12 16:27:38.588665
76923	UUP	2020-09-29	25.4	25.415	25.35	25.35	1249300	{}	\N	2020-10-12 16:27:38.600229
76924	UUP	2020-09-30	25.4	25.4499	25.3	25.35	1245861	{}	\N	2020-10-12 16:27:38.607778
76925	UUP	2020-10-01	25.33	25.38	25.31	25.31	919677	{}	\N	2020-10-12 16:27:38.613449
76926	UUP	2020-10-02	25.37	25.38	25.33	25.37	1339487	{}	\N	2020-10-12 16:27:38.618791
76927	UUP	2020-10-05	25.27	25.27	25.22	25.26	1059214	{}	\N	2020-10-12 16:27:38.623666
76928	UUP	2020-10-06	25.22	25.3499	25.22	25.33	2288801	{}	\N	2020-10-12 16:27:38.628592
76929	UUP	2020-10-07	25.31	25.32	25.28	25.3	1164894	{}	\N	2020-10-12 16:27:38.633382
76930	UUP	2020-10-08	25.3	25.35	25.28	25.29	866725	{}	\N	2020-10-12 16:27:38.637995
76931	UUP	2020-10-09	25.17	25.2	25.12	25.14	2000415	{}	\N	2020-10-12 16:27:38.6426
76932	UUP	2020-10-12	25.13	25.14	25.11	25.13	392827	{}	\N	2020-10-12 16:27:38.647049
76933	UXI	2020-09-28	75.86	75.99	75.15	75.15	627	{}	\N	2020-10-12 16:27:39.911716
76934	UXI	2020-09-29	74.9289	74.9289	74.9289	74.9289	58	{}	\N	2020-10-12 16:27:39.925725
76935	UXI	2020-09-30	74.8	74.8	74.68	74.68	783	{}	\N	2020-10-12 16:27:39.933911
76936	UXI	2020-10-01	76.03	76.59	74.11	74.11	4167	{}	\N	2020-10-12 16:27:39.940937
76937	UXI	2020-10-02	75.6	75.6	75.13	75.13	288	{}	\N	2020-10-12 16:27:39.946588
76938	UXI	2020-10-05	77.16	77.9285	77.16	77.9285	447	{}	\N	2020-10-12 16:27:39.95199
76939	UXI	2020-10-06	78.1596	78.1596	76.4093	76.4093	816	{}	\N	2020-10-12 16:27:39.956658
76940	UXI	2020-10-07	78.67	79.407	78.67	79.407	621	{}	\N	2020-10-12 16:27:39.961665
76941	UXI	2020-10-08	79.4	80.702	79.4	80.702	1369	{}	\N	2020-10-12 16:27:39.966425
76942	UXI	2020-10-09	81.76	82	81.7109	81.7109	2488	{}	\N	2020-10-12 16:27:39.971031
76943	UXI	2020-10-12	82.23	82.74	82.23	82.74	1003	{}	\N	2020-10-12 16:27:39.975706
76944	VAC	2020-09-28	93.2	94.71	91.19	94.19	332037	{}	\N	2020-10-12 16:27:42.011028
76945	VAC	2020-09-29	94.53	94.53	89.73	90.95	599182	{}	\N	2020-10-12 16:27:42.028994
76946	VAC	2020-09-30	91.68	94.12	90.23	90.81	713389	{}	\N	2020-10-12 16:27:42.035451
76947	VAC	2020-10-01	91.29	93.85	89.98	93.73	546368	{}	\N	2020-10-12 16:27:42.040931
76948	VAC	2020-10-02	90.74	94.54	90.74	93.85	408507	{}	\N	2020-10-12 16:27:42.047367
76949	VAC	2020-10-05	95.24	95.92	94.05	95.48	390582	{}	\N	2020-10-12 16:27:42.051833
76950	VAC	2020-10-06	97.19	100.33	96.3602	97.21	777063	{}	\N	2020-10-12 16:27:42.056132
76951	VAC	2020-10-07	99.51	100.28	98.11	98.72	664482	{}	\N	2020-10-12 16:27:42.061054
76952	VAC	2020-10-08	100.21	100.33	96.41	98.95	641012	{}	\N	2020-10-12 16:27:42.065903
76953	VAC	2020-10-09	100.02	100.12	97.93	98.12	345019	{}	\N	2020-10-12 16:27:42.070836
76954	VAC	2020-10-12	97.92	98.64	97.2	97.94	112493	{}	\N	2020-10-12 16:27:42.075635
76955	VBF	2020-09-28	20.08	20.17	20.01	20.09	32886	{}	\N	2020-10-12 16:27:43.423007
76956	VBF	2020-09-29	20.15	20.15	20.0368	20.12	38914	{}	\N	2020-10-12 16:27:43.434808
76957	VBF	2020-09-30	20.2	20.2859	19.99	20.01	81674	{}	\N	2020-10-12 16:27:43.442218
76958	VBF	2020-10-01	20.1	20.19	20	20	55400	{}	\N	2020-10-12 16:27:43.448355
76959	VBF	2020-10-02	20.02	20.4	20	20.15	34472	{}	\N	2020-10-12 16:27:43.45372
76960	VBF	2020-10-05	20.15	20.15	20.09	20.1	21342	{}	\N	2020-10-12 16:27:43.459191
76961	VBF	2020-10-06	20.09	20.1295	19.89	19.97	45710	{}	\N	2020-10-12 16:27:43.463669
76962	VBF	2020-10-07	20.08	20.08	19.92	19.92	54976	{}	\N	2020-10-12 16:27:43.468251
76963	VBF	2020-10-08	19.98	20	19.9	19.9	66554	{}	\N	2020-10-12 16:27:43.472845
76964	VBF	2020-10-09	20.06	20.37	19.87	20.08	168920	{}	\N	2020-10-12 16:27:43.477265
76965	VBF	2020-10-12	20.1	20.2315	20.07	20.14	14424	{}	\N	2020-10-12 16:27:43.481734
76966	VCIT	2020-09-28	95.64	95.9	95.6	95.9	2478926	{}	\N	2020-10-12 16:27:45.116474
76967	VCIT	2020-09-29	95.888	96.03	95.87	95.87	2065117	{}	\N	2020-10-12 16:27:45.127482
76968	VCIT	2020-09-30	95.88	95.91	95.76	95.8	3113175	{}	\N	2020-10-12 16:27:45.134904
76969	VCIT	2020-10-01	95.57	95.86	95.57	95.85	2063712	{}	\N	2020-10-12 16:27:45.141206
76970	VCIT	2020-10-02	95.82	95.9	95.67	95.855	2925875	{}	\N	2020-10-12 16:27:45.146769
76971	VCIT	2020-10-05	95.76	95.84	95.6	95.62	2593004	{}	\N	2020-10-12 16:27:45.151804
76972	VCIT	2020-10-06	95.739	95.91	95.59	95.68	3251647	{}	\N	2020-10-12 16:27:45.156631
76973	VCIT	2020-10-07	95.62	95.8	95.61	95.63	2558547	{}	\N	2020-10-12 16:27:45.161338
76974	VCIT	2020-10-08	95.7208	95.88	95.7208	95.84	3003875	{}	\N	2020-10-12 16:27:45.166
76975	VCIT	2020-10-09	95.83	95.98	95.8	95.94	2853058	{}	\N	2020-10-12 16:27:45.170479
76976	VCIT	2020-10-12	96.09	96.219	96	96.219	749785	{}	\N	2020-10-12 16:27:45.174945
76977	VFC	2020-09-28	71.81	72.39	71.05	71.19	1295819	{}	\N	2020-10-12 16:27:46.59343
76978	VFC	2020-09-29	70.88	71.1	68.85	69.66	3740417	{}	\N	2020-10-12 16:27:46.605029
76979	VFC	2020-09-30	70.56	71.29	69.65	70.25	3775100	{}	\N	2020-10-12 16:27:46.612394
76980	VFC	2020-10-01	70.82	71.83	70.435	71.58	3436222	{}	\N	2020-10-12 16:27:46.62033
76981	VFC	2020-10-02	69.73	72.82	69.69	72.53	2458781	{}	\N	2020-10-12 16:27:46.62647
76982	VFC	2020-10-05	73.44	75.04	73.29	74.91	3285590	{}	\N	2020-10-12 16:27:46.63186
76983	VFC	2020-10-06	75	77.38	74.53	74.66	4183649	{}	\N	2020-10-12 16:27:46.637369
76984	VFC	2020-10-07	75.82	77.13	75.82	76.96	3688362	{}	\N	2020-10-12 16:27:46.642952
76985	VFC	2020-10-08	77.54	78.46	77.05	78.33	2731170	{}	\N	2020-10-12 16:27:46.648177
76986	VFC	2020-10-09	77.66	78.06	76.42	77.61	4281523	{}	\N	2020-10-12 16:27:46.653223
76987	VFC	2020-10-12	77.96	78.4	77.35	78.35	1459360	{}	\N	2020-10-12 16:27:46.658567
76988	VGM	2020-09-28	12.42	12.5	12.39	12.4	128230	{}	\N	2020-10-12 16:27:49.690263
76989	VGM	2020-09-29	12.41	12.42	12.4	12.41	63920	{}	\N	2020-10-12 16:27:49.702099
76990	VGM	2020-09-30	12.45	12.45	12.39	12.42	168689	{}	\N	2020-10-12 16:27:49.710424
76991	VGM	2020-10-01	12.44	12.5	12.4265	12.5	138218	{}	\N	2020-10-12 16:27:49.717585
76992	VGM	2020-10-02	12.49	12.54	12.45	12.54	126346	{}	\N	2020-10-12 16:27:49.723669
76993	VGM	2020-10-05	12.56	12.56	12.4584	12.47	97828	{}	\N	2020-10-12 16:27:49.73069
76994	VGM	2020-10-06	12.47	12.5	12.45	12.45	117220	{}	\N	2020-10-12 16:27:49.738072
76995	VGM	2020-10-07	12.47	12.54	12.43	12.51	161238	{}	\N	2020-10-12 16:27:49.74465
76996	VGM	2020-10-08	12.53	12.56	12.51	12.54	121412	{}	\N	2020-10-12 16:27:49.750416
76997	VGM	2020-10-09	12.55	12.6138	12.55	12.57	122259	{}	\N	2020-10-12 16:27:49.755118
76998	VGM	2020-10-12	12.62	12.62	12.5915	12.6199	135264	{}	\N	2020-10-12 16:27:49.759789
76999	VIG	2020-09-28	128.55	129.38	128.3	128.83	1393836	{}	\N	2020-10-12 16:27:51.570832
77000	VIG	2020-09-29	128.39	128.665	127.409	127.7	1155855	{}	\N	2020-10-12 16:27:51.581061
77001	VIG	2020-09-30	128.03	129.83	127.8674	128.66	1019433	{}	\N	2020-10-12 16:27:51.588194
77002	VIG	2020-10-01	129.54	130.03	128.548	129.16	1350165	{}	\N	2020-10-12 16:27:51.594599
77003	VIG	2020-10-02	127.69	129.375	127.41	128.55	1482428	{}	\N	2020-10-12 16:27:51.599888
77004	VIG	2020-10-05	129.4	130.57	129.38	130.46	907517	{}	\N	2020-10-12 16:27:51.604587
77005	VIG	2020-10-06	130.72	131.57	128.93	129.14	1508707	{}	\N	2020-10-12 16:27:51.609526
77006	VIG	2020-10-07	130.18	131.54	130.18	131.22	915765	{}	\N	2020-10-12 16:27:51.614104
77007	VIG	2020-10-08	131.93	132.2	131.5	132.19	965872	{}	\N	2020-10-12 16:27:51.618696
77008	VIG	2020-10-09	132.93	133.73	132.74	133.37	1073271	{}	\N	2020-10-12 16:27:51.623147
77009	VIG	2020-10-12	134	134.88	133.8574	134.88	570426	{}	\N	2020-10-12 16:27:51.628072
77010	XLF	2020-09-28	23.82	24.22	23.82	24.05	56028470	{}	\N	2020-10-12 16:27:53.066424
77011	XLF	2020-09-29	23.98	24.015	23.62	23.75	51661212	{}	\N	2020-10-12 16:27:53.076604
77012	XLF	2020-09-30	23.85	24.27	23.84	24.07	65301333	{}	\N	2020-10-12 16:27:53.08331
77013	XLF	2020-10-01	24.17	24.33	23.92	24.11	59043246	{}	\N	2020-10-12 16:27:53.089013
77014	XLF	2020-10-02	23.79	24.44	23.73	24.3	55380013	{}	\N	2020-10-12 16:27:53.093878
77015	XLF	2020-10-05	24.53	24.78	24.465	24.7	55589598	{}	\N	2020-10-12 16:27:53.0983
77016	XLF	2020-10-06	24.91	25.08	24.36	24.45	78358535	{}	\N	2020-10-12 16:27:53.102598
77017	XLF	2020-10-07	24.68	24.98	24.68	24.86	50498587	{}	\N	2020-10-12 16:27:53.106771
77018	XLF	2020-10-08	25	25.26	24.94	25.2	57123299	{}	\N	2020-10-12 16:27:53.111515
77019	XLF	2020-10-09	25.34	25.37	25.0317	25.23	45410341	{}	\N	2020-10-12 16:27:53.116307
77020	XLF	2020-10-12	25.21	25.48	25.205	25.41	15594151	{}	\N	2020-10-12 16:27:53.121339
77021	F	2020-09-28	6.59	6.77	6.58	6.69	48257560	{}	\N	2020-10-12 16:27:54.427705
77022	F	2020-09-29	6.69	6.695	6.54	6.6	76308074	{}	\N	2020-10-12 16:27:54.441769
77023	F	2020-09-30	6.61	6.75	6.59	6.66	106437054	{}	\N	2020-10-12 16:27:54.45016
77024	F	2020-10-01	6.71	6.77	6.63	6.75	109427898	{}	\N	2020-10-12 16:27:54.455744
77025	F	2020-10-02	6.62	6.93	6.6	6.89	113879914	{}	\N	2020-10-12 16:27:54.461071
77026	F	2020-10-05	6.95	7.05	6.945	7.02	83950165	{}	\N	2020-10-12 16:27:54.465692
77027	F	2020-10-06	7.06	7.25	6.96	6.98	150080723	{}	\N	2020-10-12 16:27:54.469583
77028	F	2020-10-07	7.11	7.25	7.06	7.23	100483349	{}	\N	2020-10-12 16:27:54.473632
77029	F	2020-10-08	7.29	7.35	7.2	7.35	106872850	{}	\N	2020-10-12 16:27:54.477603
77030	F	2020-10-09	7.36	7.44	7.23	7.25	98899349	{}	\N	2020-10-12 16:27:54.48144
77031	F	2020-10-12	7.36	7.865	7.33	7.705	126107156	{}	\N	2020-10-12 16:27:54.485486
77032	UTMD	2020-09-28	81.81	82.99	81.03	81.03	9298	{}	\N	2020-10-12 16:27:55.579082
77033	UTMD	2020-09-29	80.95	80.95	80.95	80.95	1169	{}	\N	2020-10-12 16:27:55.590326
77034	UTMD	2020-09-30	82.395	82.395	79.05	79.87	13205	{}	\N	2020-10-12 16:27:55.598035
77035	UTMD	2020-10-01	80.4	81.015	79.35	79.35	6201	{}	\N	2020-10-12 16:27:55.604062
77036	UTMD	2020-10-02	78.9	80.7642	78.9	79.83	8812	{}	\N	2020-10-12 16:27:55.610241
77037	UTMD	2020-10-05	81.5814	82.44	79.85	80.88	16094	{}	\N	2020-10-12 16:27:55.615399
77038	UTMD	2020-10-06	81.8	83.855	81.6	82.52	12903	{}	\N	2020-10-12 16:27:55.620511
77039	UTMD	2020-10-07	83.2	83.24	81.85	82.8	15002	{}	\N	2020-10-12 16:27:55.625148
77040	UTMD	2020-10-08	82.46	83.58	81.73	81.73	9013	{}	\N	2020-10-12 16:27:55.630052
77041	UTMD	2020-10-09	82.6	83.23	81.6125	83.2	7667	{}	\N	2020-10-12 16:27:55.634878
77042	UTMD	2020-10-12	83.72	83.72	83.72	83.72	320	{}	\N	2020-10-12 16:27:55.640306
77043	UVV	2020-09-28	41.5	42.5	41.5	41.72	145400	{}	\N	2020-10-12 16:27:56.988556
77044	UVV	2020-09-29	41.8	41.8	40.87	41.53	267173	{}	\N	2020-10-12 16:27:56.99902
77045	UVV	2020-09-30	41.56	42.15	41.47	41.88	344972	{}	\N	2020-10-12 16:27:57.005904
77046	UVV	2020-10-01	41.9	42.1	41.3	41.8	244713	{}	\N	2020-10-12 16:27:57.011941
77047	UVV	2020-10-02	41.34	42.35	41.34	42.2	292286	{}	\N	2020-10-12 16:27:57.01726
77048	UVV	2020-10-05	42.5	42.98	42.23	42.91	257399	{}	\N	2020-10-12 16:27:57.022092
77049	UVV	2020-10-06	43.43	44.09	43.175	43.64	523481	{}	\N	2020-10-12 16:27:57.026563
77050	UVV	2020-10-07	43.75	44.69	43.7122	44.43	302687	{}	\N	2020-10-12 16:27:57.031161
77051	UVV	2020-10-08	43.88	44	43.115	43.49	265874	{}	\N	2020-10-12 16:27:57.035752
77052	UVV	2020-10-09	43.67	44.28	43.23	43.32	210334	{}	\N	2020-10-12 16:27:57.040244
77053	UVV	2020-10-12	43.04	43.74	43.04	43.63	54742	{}	\N	2020-10-12 16:27:57.044752
77054	VALU	2020-09-28	24.27	24.8692	23.8	24.02	1406	{}	\N	2020-10-12 16:27:58.01237
77055	VALU	2020-09-29	24.287	24.8799	24.287	24.74	585	{}	\N	2020-10-12 16:27:58.023648
77056	VALU	2020-09-30	24.7	24.7	24.7	24.7	487	{}	\N	2020-10-12 16:27:58.030273
77057	VALU	2020-10-01	24.19	24.19	24.19	24.19	676	{}	\N	2020-10-12 16:27:58.036189
77058	VALU	2020-10-02	24.245	24.88	23.95	24.88	625	{}	\N	2020-10-12 16:27:58.041533
77059	VALU	2020-10-05	25.19	25.22	25.12	25.12	2427	{}	\N	2020-10-12 16:27:58.047047
77060	VALU	2020-10-06	25.105	25.25	25.105	25.25	982	{}	\N	2020-10-12 16:27:58.051973
77061	VALU	2020-10-07	25.33	25.33	25.27	25.27	1161	{}	\N	2020-10-12 16:27:58.056619
77062	VALU	2020-10-08	25.49	28.58	25.49	27.1	12689	{}	\N	2020-10-12 16:27:58.060918
77063	VALU	2020-10-09	27.23	27.44	26.0033	26.98	6641	{}	\N	2020-10-12 16:27:58.064945
77064	VALU	2020-10-12	26.43	28.11	26.43	27.62	3719	{}	\N	2020-10-12 16:27:58.06921
77065	VBIV	2020-09-28	3.12	3.13	2.855	2.96	6908006	{}	\N	2020-10-12 16:27:59.842283
77066	VBIV	2020-09-29	2.96	3.02	2.8823	2.98	5449674	{}	\N	2020-10-12 16:27:59.856265
77067	VBIV	2020-09-30	2.93	3	2.81	2.86	6425765	{}	\N	2020-10-12 16:27:59.864196
77068	VBIV	2020-10-01	2.89	2.9	2.66	2.73	7832035	{}	\N	2020-10-12 16:27:59.870596
77069	VBIV	2020-10-02	2.74	2.8594	2.63	2.72	4554936	{}	\N	2020-10-12 16:27:59.876195
77070	VBIV	2020-10-05	2.76	2.95	2.74	2.93	4600388	{}	\N	2020-10-12 16:27:59.882219
77071	VBIV	2020-10-06	2.91	2.96	2.79	2.83	4138210	{}	\N	2020-10-12 16:27:59.887313
77072	VBIV	2020-10-07	2.87	3.1	2.82	3.03	6135612	{}	\N	2020-10-12 16:27:59.892584
77073	VBIV	2020-10-08	3.12	3.12	2.88	2.93	4269398	{}	\N	2020-10-12 16:27:59.897384
77074	VBIV	2020-10-09	2.95	2.99	2.8686	2.95	3514677	{}	\N	2020-10-12 16:27:59.902792
77075	VBIV	2020-10-12	2.95	2.97	2.82	2.8589	1775263	{}	\N	2020-10-12 16:27:59.907788
77076	VCEL	2020-09-28	18.46	18.46	17.88	18.08	340867	{}	\N	2020-10-12 16:28:01.540985
77077	VCEL	2020-09-29	18.14	18.56	17.87	18.19	464866	{}	\N	2020-10-12 16:28:01.554538
77078	VCEL	2020-09-30	18.28	18.88	18.26	18.53	307646	{}	\N	2020-10-12 16:28:01.561842
77079	VCEL	2020-10-01	18.53	19.58	18.48	19.58	681298	{}	\N	2020-10-12 16:28:01.567991
77080	VCEL	2020-10-02	19.2	19.6	18.85	19.46	523915	{}	\N	2020-10-12 16:28:01.573892
77081	VCEL	2020-10-05	19.47	20.74	19.47	20.71	643511	{}	\N	2020-10-12 16:28:01.579247
77082	VCEL	2020-10-06	20.94	21.37	20.3001	20.44	952644	{}	\N	2020-10-12 16:28:01.584051
77083	VCEL	2020-10-07	20.69	21.38	20.69	21.16	466742	{}	\N	2020-10-12 16:28:01.589015
77084	VCEL	2020-10-08	21.46	21.68	21.125	21.42	569184	{}	\N	2020-10-12 16:28:01.593883
77085	VCEL	2020-10-09	21.57	22.06	21.4715	21.82	457107	{}	\N	2020-10-12 16:28:01.598703
77086	VCEL	2020-10-12	22.06	22.26	21.71	21.945	118725	{}	\N	2020-10-12 16:28:01.60403
77087	VCR	2020-09-28	233.48	235.175	232.315	234.85	70633	{}	\N	2020-10-12 16:28:03.231587
77088	VCR	2020-09-29	234.73	234.73	232.615	232.84	206709	{}	\N	2020-10-12 16:28:03.243105
77089	VCR	2020-09-30	233.25	237.2793	233.25	234.62	159987	{}	\N	2020-10-12 16:28:03.249751
77090	VCR	2020-10-01	237.38	239.64	236.3	239.64	104063	{}	\N	2020-10-12 16:28:03.255908
77091	VCR	2020-10-02	234.12	238.415	233.16	237.3	111911	{}	\N	2020-10-12 16:28:03.261482
77092	VCR	2020-10-05	239.29	241.25	239	241.15	81566	{}	\N	2020-10-12 16:28:03.267094
77093	VCR	2020-10-06	241.67	242.45	235.995	236.74	568403	{}	\N	2020-10-12 16:28:03.271995
77094	VCR	2020-10-07	239.47	242.98	239.47	242.48	98476	{}	\N	2020-10-12 16:28:03.276949
77095	VCR	2020-10-08	244.61	244.61	242.355	243.95	52351	{}	\N	2020-10-12 16:28:03.281775
77096	VCR	2020-10-09	245.63	247.2165	245.1571	246.83	80953	{}	\N	2020-10-12 16:28:03.286889
77097	VCR	2020-10-12	248.98	250.76	248.29	250.7112	43503	{}	\N	2020-10-12 16:28:03.29173
77098	VER	2020-09-28	6.51	6.64	6.45	6.58	4939765	{}	\N	2020-10-12 16:28:04.734459
77099	VER	2020-09-29	6.45	6.555	6.35	6.46	14740032	{}	\N	2020-10-12 16:28:04.746325
77100	VER	2020-09-30	6.51	6.61	6.41	6.5	19642626	{}	\N	2020-10-12 16:28:04.75187
77101	VER	2020-10-01	6.51	6.75	6.49	6.75	22581936	{}	\N	2020-10-12 16:28:04.758361
77102	VER	2020-10-02	6.61	6.99	6.575	6.94	19601797	{}	\N	2020-10-12 16:28:04.764091
77103	VER	2020-10-05	7.01	7.065	6.85	7.03	12684182	{}	\N	2020-10-12 16:28:04.769188
77104	VER	2020-10-06	7.07	7.14	6.885	6.99	17282876	{}	\N	2020-10-12 16:28:04.773732
77105	VER	2020-10-07	7.01	7.04	6.8299	6.89	9602047	{}	\N	2020-10-12 16:28:04.778076
77106	VER	2020-10-08	6.94	7.03	6.85	7.01	15440330	{}	\N	2020-10-12 16:28:04.782359
77107	VER	2020-10-09	7.04	7.08	6.83	6.86	9552450	{}	\N	2020-10-12 16:28:04.786819
77108	VER	2020-10-12	6.86	6.9	6.76	6.83	3489383	{}	\N	2020-10-12 16:28:04.790521
77109	VGI	2020-09-28	10.84	10.94	10.84	10.9	50465	{}	\N	2020-10-12 16:28:06.944902
77110	VGI	2020-09-29	10.87	10.96	10.82	10.82	80114	{}	\N	2020-10-12 16:28:06.958218
77111	VGI	2020-09-30	10.81	10.8799	10.75	10.78	127104	{}	\N	2020-10-12 16:28:06.966336
77112	VGI	2020-10-01	10.8	10.87	10.8	10.81	62784	{}	\N	2020-10-12 16:28:06.973255
77113	VGI	2020-10-02	10.62	10.905	10.41	10.8	81494	{}	\N	2020-10-12 16:28:06.979867
77114	VGI	2020-10-05	10.86	10.98	10.82	10.98	54214	{}	\N	2020-10-12 16:28:06.985359
77115	VGI	2020-10-06	10.99	11.2	10.98	11.07	86059	{}	\N	2020-10-12 16:28:06.990106
77116	VGI	2020-10-07	11.25	11.72	11.09	11.1	111314	{}	\N	2020-10-12 16:28:06.994941
77117	VGI	2020-10-08	11.27	11.27	11.18	11.21	84392	{}	\N	2020-10-12 16:28:06.999631
77118	VGI	2020-10-09	11.19	11.19	11.0182	11.11	87006	{}	\N	2020-10-12 16:28:07.004308
77119	VGI	2020-10-12	11.08	11.1678	11.06	11.08	21474	{}	\N	2020-10-12 16:28:07.008867
77120	VHT	2020-09-28	201.3	202.4681	200.6218	200.99	134184	{}	\N	2020-10-12 16:28:08.527124
77121	VHT	2020-09-29	201.57	202.5567	200.321	200.92	92626	{}	\N	2020-10-12 16:28:08.540998
77122	VHT	2020-09-30	201.57	205.5493	201.57	204	125636	{}	\N	2020-10-12 16:28:08.549102
77123	VHT	2020-10-01	205.16	205.515	202.69	203.45	153932	{}	\N	2020-10-12 16:28:08.556382
77124	VHT	2020-10-02	201	203.45	200.6	201.68	89479	{}	\N	2020-10-12 16:28:08.562883
77125	VHT	2020-10-05	203.38	206.365	203.38	206.35	163803	{}	\N	2020-10-12 16:28:08.5684
77126	VHT	2020-10-06	206.75	207.4153	203.96	204.02	197738	{}	\N	2020-10-12 16:28:08.573975
77127	VHT	2020-10-07	205.56	208.6999	205.56	208.25	144706	{}	\N	2020-10-12 16:28:08.579599
77128	VHT	2020-10-08	209.14	209.71	208.68	209.28	1006236	{}	\N	2020-10-12 16:28:08.585009
77129	VHT	2020-10-09	210.26	211.62	209.94	211.04	162507	{}	\N	2020-10-12 16:28:08.590081
77130	VHT	2020-10-12	212.23	213.08	211.47	212.825	82078	{}	\N	2020-10-12 16:28:08.595388
77131	VKQ	2020-09-28	11.93	11.99	11.92	11.94	76823	{}	\N	2020-10-12 16:28:10.932654
77132	VKQ	2020-09-29	11.95	11.99	11.95	11.98	62544	{}	\N	2020-10-12 16:28:10.944396
77133	VKQ	2020-09-30	12	12.02	11.95	12.01	199362	{}	\N	2020-10-12 16:28:10.951348
77134	VKQ	2020-10-01	12.02	12.07	11.96	12.05	248030	{}	\N	2020-10-12 16:28:10.957797
77135	VKQ	2020-10-02	12.03	12.08	11.98	12.08	212492	{}	\N	2020-10-12 16:28:10.963847
77136	VKQ	2020-10-05	12.05	12.08	12.03	12.03	225590	{}	\N	2020-10-12 16:28:10.969051
77137	VKQ	2020-10-06	12.04	12.076	12.02	12.07	115906	{}	\N	2020-10-12 16:28:10.974403
77138	VKQ	2020-10-07	12.11	12.11	12.04	12.1	336938	{}	\N	2020-10-12 16:28:10.979716
77139	VKQ	2020-10-08	12.08	12.14	12.08	12.11	92344	{}	\N	2020-10-12 16:28:10.985109
77140	VKQ	2020-10-09	12.11	12.26	12.11	12.18	214952	{}	\N	2020-10-12 16:28:10.990232
77141	VKQ	2020-10-12	12.18	12.217	12.12	12.2	87958	{}	\N	2020-10-12 16:28:10.995297
77142	VMI	2020-09-28	119.88	122.84	119.88	122.32	147726	{}	\N	2020-10-12 16:28:12.025376
77143	VMI	2020-09-29	122.14	122.83	120.33	122.75	249500	{}	\N	2020-10-12 16:28:12.037201
77144	VMI	2020-09-30	123.36	125.65	123.35	124.18	334351	{}	\N	2020-10-12 16:28:12.045163
77145	VMI	2020-10-01	124.54	125.205	122.14	122.75	173307	{}	\N	2020-10-12 16:28:12.051829
77146	VMI	2020-10-02	121.03	125.9	120.258	124.99	184816	{}	\N	2020-10-12 16:28:12.057828
77147	VMI	2020-10-05	126.23	128.28	125.3001	127.56	230183	{}	\N	2020-10-12 16:28:12.063096
77148	VMI	2020-10-06	128.42	132.93	126.45	131.14	384388	{}	\N	2020-10-12 16:28:12.068239
77149	VMI	2020-10-07	133.35	138.41	132.19	137.5	366218	{}	\N	2020-10-12 16:28:12.073425
77150	VMI	2020-10-08	138.1	140.4	137.34	138.21	297394	{}	\N	2020-10-12 16:28:12.078724
77151	VMI	2020-10-09	138.36	141.6476	137.53	140.61	313497	{}	\N	2020-10-12 16:28:12.083719
77152	VMI	2020-10-12	140.16	142.49	140.16	140.905	39649	{}	\N	2020-10-12 16:28:12.08901
77153	UTSI	2020-09-28	1.07	1.1199	1.02	1.08	48105	{}	\N	2020-10-12 16:28:13.314877
77154	UTSI	2020-09-29	1.11	1.1399	1.04	1.08	61278	{}	\N	2020-10-12 16:28:13.328227
77155	UTSI	2020-09-30	1.07	1.105	1.05	1.075	26600	{}	\N	2020-10-12 16:28:13.335236
77156	UTSI	2020-10-01	1.08	1.12	1.05	1.07	27030	{}	\N	2020-10-12 16:28:13.34105
77157	UTSI	2020-10-02	1.03	1.09	1.03	1.07	22583	{}	\N	2020-10-12 16:28:13.345877
77158	UTSI	2020-10-05	1.07	1.1	1	1.05	54511	{}	\N	2020-10-12 16:28:13.350931
77159	UTSI	2020-10-06	1.02	1.09	1.01	1.09	57650	{}	\N	2020-10-12 16:28:13.355882
77160	UTSI	2020-10-07	1.0771	1.11	1.0547	1.11	29762	{}	\N	2020-10-12 16:28:13.360075
77161	UTSI	2020-10-08	1.12	1.15	1.085	1.09	54162	{}	\N	2020-10-12 16:28:13.364197
77162	UTSI	2020-10-09	1.09	1.13	1.09	1.1079	9732	{}	\N	2020-10-12 16:28:13.368603
77163	UTSI	2020-10-12	1.08	1.12	1.08	1.115	10939	{}	\N	2020-10-12 16:28:13.372741
77164	UVSP	2020-09-28	14.31	14.87	14.31	14.7	68220	{}	\N	2020-10-12 16:28:14.330662
77165	UVSP	2020-09-29	14.66	14.66	14.05	14.41	68703	{}	\N	2020-10-12 16:28:14.344274
77166	UVSP	2020-09-30	14.51	14.615	14.29	14.37	84290	{}	\N	2020-10-12 16:28:14.352116
77167	UVSP	2020-10-01	14.515	14.515	14.13	14.4	92270	{}	\N	2020-10-12 16:28:14.358961
77168	UVSP	2020-10-02	14.14	14.87	14.1	14.78	59349	{}	\N	2020-10-12 16:28:14.364978
77169	UVSP	2020-10-05	14.93	15.49	14.645	15.47	89115	{}	\N	2020-10-12 16:28:14.370758
77170	UVSP	2020-10-06	15.71	16.26	15.34	15.6	90721	{}	\N	2020-10-12 16:28:14.376275
77171	UVSP	2020-10-07	15.81	15.94	15.62	15.85	76431	{}	\N	2020-10-12 16:28:14.381781
77172	UVSP	2020-10-08	16.04	16.11	15.61	15.87	74869	{}	\N	2020-10-12 16:28:14.387428
77173	UVSP	2020-10-09	16.08	16.08	15.65	15.66	49113	{}	\N	2020-10-12 16:28:14.393418
77174	UVSP	2020-10-12	15.6	15.83	15.57	15.74	11670	{}	\N	2020-10-12 16:28:14.39904
77175	UYM	2020-09-28	54.91	56.151	54.91	55.24	9945	{}	\N	2020-10-12 16:28:15.70079
77176	UYM	2020-09-29	54.49	54.78	54.3992	54.3992	2549	{}	\N	2020-10-12 16:28:15.713851
77177	UYM	2020-09-30	55.38	56.34	54.995	54.995	4266	{}	\N	2020-10-12 16:28:15.721986
77178	UYM	2020-10-01	55.785	55.785	53.9521	53.9521	3929	{}	\N	2020-10-12 16:28:15.730755
77179	UYM	2020-10-02	53.67	55.54	53.67	55.54	3256	{}	\N	2020-10-12 16:28:15.736191
77180	UYM	2020-10-05	57.315	57.315	57	57.1045	3603	{}	\N	2020-10-12 16:28:15.740989
77181	UYM	2020-10-06	57.87	58.5	55.605	55.755	4236	{}	\N	2020-10-12 16:28:15.74588
77182	UYM	2020-10-07	57.94	58.8196	57.94	58.68	3133	{}	\N	2020-10-12 16:28:15.750422
77183	UYM	2020-10-08	59.65	60.185	59.38	59.4475	4121	{}	\N	2020-10-12 16:28:15.755096
77184	UYM	2020-10-09	61.07	61.07	59.92	60.5	8225	{}	\N	2020-10-12 16:28:15.759707
77185	UYM	2020-10-12	61.21	61.21	60.5	60.83	1946	{}	\N	2020-10-12 16:28:15.764118
77186	VAR	2020-09-28	172.18	172.24	171.77	171.95	846110	{}	\N	2020-10-12 16:28:17.179065
77187	VAR	2020-09-29	172.25	172.25	171.75	172	1568409	{}	\N	2020-10-12 16:28:17.189308
77188	VAR	2020-09-30	172.13	172.21	171.9	172	3027648	{}	\N	2020-10-12 16:28:17.196152
77189	VAR	2020-10-01	172	172.65	170.95	171.63	4213097	{}	\N	2020-10-12 16:28:17.202364
77190	VAR	2020-10-02	171.21	172.1	171.21	171.93	2537244	{}	\N	2020-10-12 16:28:17.20811
77191	VAR	2020-10-05	172.29	172.29	171.75	171.84	2739986	{}	\N	2020-10-12 16:28:17.212781
77192	VAR	2020-10-06	171.95	172.06	171.5	171.68	2424420	{}	\N	2020-10-12 16:28:17.217593
77193	VAR	2020-10-07	171.91	172.01	171.86	171.95	1793608	{}	\N	2020-10-12 16:28:17.221884
77194	VAR	2020-10-08	171.99	172.025	171.63	171.83	2243959	{}	\N	2020-10-12 16:28:17.225949
77195	VAR	2020-10-09	171.98	171.98	171.76	171.9	1825447	{}	\N	2020-10-12 16:28:17.229702
77196	VAR	2020-10-12	171.67	172	171.67	171.92	1102118	{}	\N	2020-10-12 16:28:17.233315
77197	VBK	2020-09-28	213.04	214.92	212.86	214.36	255026	{}	\N	2020-10-12 16:28:18.927626
77198	VBK	2020-09-29	214.42	216.11	213.83	214.4	142150	{}	\N	2020-10-12 16:28:18.94204
77199	VBK	2020-09-30	214.89	217.08	213.36	214.93	131080	{}	\N	2020-10-12 16:28:18.94797
77200	VBK	2020-10-01	216.65	218.56	215.72	218.31	166125	{}	\N	2020-10-12 16:28:18.952621
77201	VBK	2020-10-02	213.5749	218.0964	213.03	217.17	231919	{}	\N	2020-10-12 16:28:18.957186
77202	VBK	2020-10-05	219.1249	222.4	219.1249	222.34	133050	{}	\N	2020-10-12 16:28:18.961641
77203	VBK	2020-10-06	223.62	226.23	221.37	222.05	195592	{}	\N	2020-10-12 16:28:18.966271
77204	VBK	2020-10-07	224.53	227.16	224.38	226.78	194242	{}	\N	2020-10-12 16:28:18.970798
77205	VBK	2020-10-08	228.63	228.8799	227.4778	228.41	183388	{}	\N	2020-10-12 16:28:18.975302
77206	VBK	2020-10-09	230.11	231.342	229.9	230.91	158689	{}	\N	2020-10-12 16:28:18.979994
77207	VBK	2020-10-12	232.67	232.7602	231.365	232.2192	63864	{}	\N	2020-10-12 16:28:18.984663
77208	VCRA	2020-09-28	28.28	28.98	28.11	28.72	256173	{}	\N	2020-10-12 16:28:20.644073
77209	VCRA	2020-09-29	28.69	29.06	28.3	28.67	652178	{}	\N	2020-10-12 16:28:20.655002
77210	VCRA	2020-09-30	28.74	29.36	28.63	29.08	509379	{}	\N	2020-10-12 16:28:20.661231
77211	VCRA	2020-10-01	29.28	29.46	28.705	29.43	683705	{}	\N	2020-10-12 16:28:20.666705
77212	VCRA	2020-10-02	29.02	29.8	28.68	29.4	426853	{}	\N	2020-10-12 16:28:20.67252
77213	VCRA	2020-10-05	29.51	30.25	29.39	30.18	354276	{}	\N	2020-10-12 16:28:20.67765
77214	VCRA	2020-10-06	30.4	31.045	30.035	30.42	735653	{}	\N	2020-10-12 16:28:20.682727
77215	VCRA	2020-10-07	30.69	31.56	30.69	31.09	419744	{}	\N	2020-10-12 16:28:20.688189
77216	VCRA	2020-10-08	31.44	31.69	31.15	31.59	250240	{}	\N	2020-10-12 16:28:20.693103
77217	VCRA	2020-10-09	31.98	32.41	31.49	31.94	370219	{}	\N	2020-10-12 16:28:20.697851
77218	VCRA	2020-10-12	31.97	32.62	31.45	32.025	149295	{}	\N	2020-10-12 16:28:20.702744
77219	VEGA	2020-09-28	33.8501	34.0031	33.8501	34.0031	305	{}	\N	2020-10-12 16:28:21.85096
77220	VEGA	2020-09-29	34	34	33.52	33.52	786	{}	\N	2020-10-12 16:28:21.859501
77221	VEGA	2020-09-30	33.91	34.1599	33.9	34.1599	1328	{}	\N	2020-10-12 16:28:21.865956
77222	VEGA	2020-10-01	34.21	34.21	34.21	34.21	200	{}	\N	2020-10-12 16:28:21.871756
77223	VEGA	2020-10-02	33.92	34.1064	33.91	34.1064	2300	{}	\N	2020-10-12 16:28:21.876807
77224	VEGA	2020-10-05	32.02	34.3772	31.67	34.3772	656	{}	\N	2020-10-12 16:28:21.881127
77225	VEGA	2020-10-06	34.4363	34.4363	34.1327	34.1327	124	{}	\N	2020-10-12 16:28:21.885945
77226	VEGA	2020-10-07	34.4001	34.4787	34.4001	34.4787	206	{}	\N	2020-10-12 16:28:21.890102
77227	VEGA	2020-10-08	34.6697	34.6697	34.6697	34.6697	60	{}	\N	2020-10-12 16:28:21.89412
77228	VEGA	2020-10-09	34.8432	34.8432	34.8432	34.8432	120	{}	\N	2020-10-12 16:28:21.898593
77229	VFL	2020-09-28	12.3949	12.3949	12.35	12.355	3241	{}	\N	2020-10-12 16:28:22.87783
77230	VFL	2020-09-29	12.39	12.397	12.3142	12.36	14048	{}	\N	2020-10-12 16:28:22.891433
77231	VFL	2020-09-30	12.32	12.3394	12.32	12.3394	816	{}	\N	2020-10-12 16:28:22.899056
77232	VFL	2020-10-01	12.38	12.38	12.35	12.37	8897	{}	\N	2020-10-12 16:28:22.90547
77233	VFL	2020-10-02	12.39	12.4	12.336	12.4	3767	{}	\N	2020-10-12 16:28:22.910923
77234	VFL	2020-10-05	12.38	12.39	12.35	12.35	3399	{}	\N	2020-10-12 16:28:22.916404
77235	VFL	2020-10-06	12.27	12.27	12.2601	12.2601	2752	{}	\N	2020-10-12 16:28:22.92113
77236	VFL	2020-10-07	12.3	12.3242	12.27	12.28	7505	{}	\N	2020-10-12 16:28:22.925876
77237	VFL	2020-10-08	12.305	12.36	12.25	12.3599	4454	{}	\N	2020-10-12 16:28:22.930527
77238	VFL	2020-10-09	12.4	12.4	12.31	12.345	6917	{}	\N	2020-10-12 16:28:22.934989
77239	VFL	2020-10-12	12.35	12.3984	12.35	12.357	5617	{}	\N	2020-10-12 16:28:22.939719
77240	VGT	2020-09-28	309.145	310	306.85	310	697112	{}	\N	2020-10-12 16:28:23.980129
77241	VGT	2020-09-29	309.87	311.15	308.74	309.15	556811	{}	\N	2020-10-12 16:28:23.990867
77242	VGT	2020-09-30	309.23	314.58	308.6912	311.45	827762	{}	\N	2020-10-12 16:28:23.99777
77243	VGT	2020-10-01	315.59	316.18	313.43	315.33	718183	{}	\N	2020-10-12 16:28:24.003373
77244	VGT	2020-10-02	307.96	313.13	306.85	308.07	1064152	{}	\N	2020-10-12 16:28:24.008347
77245	VGT	2020-10-05	310.41	315.07	310.41	315.07	741308	{}	\N	2020-10-12 16:28:24.013424
77246	VGT	2020-10-06	314.41	317.7395	309.43	310.73	692736	{}	\N	2020-10-12 16:28:24.017794
77247	VGT	2020-10-07	313.93	317.185	313.93	316.61	540672	{}	\N	2020-10-12 16:28:24.022136
77248	VGT	2020-10-08	319.31	319.69	317.145	318	1201745	{}	\N	2020-10-12 16:28:24.02727
77249	VGT	2020-10-09	319.78	323.37	319.6315	323.37	453401	{}	\N	2020-10-12 16:28:24.031405
77250	VGT	2020-10-12	328.33	331	326.22	331	407373	{}	\N	2020-10-12 16:28:24.035683
77251	VIGI	2020-09-28	72.01	72.05	71.753	72	131906	{}	\N	2020-10-12 16:28:26.420075
77252	VIGI	2020-09-29	72.15	72.3562	71.92	72.14	144826	{}	\N	2020-10-12 16:28:26.432655
77253	VIGI	2020-09-30	72.13	72.45	71.882	72.01	143131	{}	\N	2020-10-12 16:28:26.440789
77254	VIGI	2020-10-01	72.76	72.8994	72.3969	72.86	132412	{}	\N	2020-10-12 16:28:26.447494
77255	VIGI	2020-10-02	72.03	72.735	72	72.53	149504	{}	\N	2020-10-12 16:28:26.454622
77256	VIGI	2020-10-05	73.02	73.52	73.02	73.52	103715	{}	\N	2020-10-12 16:28:26.460351
77257	VIGI	2020-10-06	73.42	73.4955	72.61	72.76	93411	{}	\N	2020-10-12 16:28:26.46559
77258	VIGI	2020-10-07	73.19	73.43	73.03	73.26	138702	{}	\N	2020-10-12 16:28:26.470981
77259	VIGI	2020-10-08	73.54	73.71	73.445	73.68	150377	{}	\N	2020-10-12 16:28:26.476437
77260	VIGI	2020-10-09	74.19	74.629	74.11	74.6	187586	{}	\N	2020-10-12 16:28:26.481978
77261	VIGI	2020-10-12	74.725	75	74.725	75	28451	{}	\N	2020-10-12 16:28:26.487984
77262	VIV	2020-09-28	8.17	8.18	7.71	7.75	1198625	{}	\N	2020-10-12 16:28:29.233387
77263	VIV	2020-09-29	7.64	7.78	7.6	7.64	3008601	{}	\N	2020-10-12 16:28:29.24619
77264	VIV	2020-09-30	7.68	7.81	7.67	7.67	1778846	{}	\N	2020-10-12 16:28:29.253772
77265	VIV	2020-10-01	7.63	7.67	7.53	7.62	2362443	{}	\N	2020-10-12 16:28:29.260315
77266	VIV	2020-10-02	7.63	7.78	7.575	7.61	2038965	{}	\N	2020-10-12 16:28:29.266996
77267	VIV	2020-10-05	7.62	7.81	7.58	7.74	1922528	{}	\N	2020-10-12 16:28:29.272562
77268	VIV	2020-10-06	7.84	8.02	7.74	7.77	2298745	{}	\N	2020-10-12 16:28:29.277987
77269	VIV	2020-10-07	7.81	7.84	7.65	7.74	3166731	{}	\N	2020-10-12 16:28:29.283316
77270	VIV	2020-10-08	7.7	7.95	7.7	7.89	3056400	{}	\N	2020-10-12 16:28:29.288622
77271	VIV	2020-10-09	7.94	8.065	7.89	7.94	2432690	{}	\N	2020-10-12 16:28:29.293722
77272	VIV	2020-10-12	7.97	7.97	7.91	7.94	451927	{}	\N	2020-10-12 16:28:29.29923
77273	UVE	2020-09-28	14.38	14.63	14.33	14.41	237112	{}	\N	2020-10-12 16:28:30.659503
77274	UVE	2020-09-29	14.45	14.45	13.65	13.69	412808	{}	\N	2020-10-12 16:28:30.671668
77275	UVE	2020-09-30	13.74	14.12	13.66	13.84	629414	{}	\N	2020-10-12 16:28:30.678889
77276	UVE	2020-10-01	13.84	13.945	13.55	13.85	445662	{}	\N	2020-10-12 16:28:30.685177
77277	UVE	2020-10-02	13.65	14.24	13.65	14.06	405334	{}	\N	2020-10-12 16:28:30.693134
77278	UVE	2020-10-05	14.17	14.38	14.06	14.34	266965	{}	\N	2020-10-12 16:28:30.69813
77279	UVE	2020-10-06	14.51	14.63	14.21	14.26	384149	{}	\N	2020-10-12 16:28:30.702647
77280	UVE	2020-10-07	14.32	14.37	13.95	14.21	397479	{}	\N	2020-10-12 16:28:30.707597
77281	UVE	2020-10-08	14.34	14.405	14.11	14.29	257843	{}	\N	2020-10-12 16:28:30.712602
77282	UVE	2020-10-09	14.29	14.39	14.11	14.2	210322	{}	\N	2020-10-12 16:28:30.717972
77283	UVE	2020-10-12	14.18	14.45	14.05	14.37	142654	{}	\N	2020-10-12 16:28:30.722795
77284	UZC	2020-09-28	25.4	25.52	25.4	25.5	56117	{}	\N	2020-10-12 16:28:32.765644
77285	UZC	2020-09-29	25.53	25.59	25.51	25.59	41612	{}	\N	2020-10-12 16:28:32.777903
77286	UZC	2020-09-30	25.62	25.64	25.59	25.64	144478	{}	\N	2020-10-12 16:28:32.785209
77287	UZC	2020-10-01	25.51	25.74	25.51	25.72	40894	{}	\N	2020-10-12 16:28:32.791439
77288	UZC	2020-10-02	25.7	25.71	25.61	25.69	36844	{}	\N	2020-10-12 16:28:32.797869
77289	UZC	2020-10-05	25.69	25.72	25.52	25.52	30872	{}	\N	2020-10-12 16:28:32.80319
77290	UZC	2020-10-06	25.52	25.73	25.52	25.7	39118	{}	\N	2020-10-12 16:28:32.808524
77291	UZC	2020-10-07	25.68	25.75	25.68	25.74	32316	{}	\N	2020-10-12 16:28:32.813935
77292	UZC	2020-10-08	25.71	25.79	25.71	25.79	33210	{}	\N	2020-10-12 16:28:32.818945
77293	UZC	2020-10-09	25.75	25.8	25.6509	25.8	29988	{}	\N	2020-10-12 16:28:32.824156
77294	UZC	2020-10-12	25.72	25.79	25.7	25.79	4516	{}	\N	2020-10-12 16:28:32.829285
77295	VBND	2020-09-28	51.7	51.91	51.7	51.79	9207	{}	\N	2020-10-12 16:28:35.305111
77296	VBND	2020-09-29	51.87	51.91	51.74	51.9	5818	{}	\N	2020-10-12 16:28:35.316654
77297	VBND	2020-09-30	51.76	51.8201	51.71	51.8201	7561	{}	\N	2020-10-12 16:28:35.324189
77298	VBND	2020-10-01	51.6236	51.925	51.6236	51.8376	19710	{}	\N	2020-10-12 16:28:35.330732
77299	VBND	2020-10-02	51.79	51.91	51.74	51.88	12510	{}	\N	2020-10-12 16:28:35.336665
77300	VBND	2020-10-05	51.57	51.83	51.56	51.6205	10467	{}	\N	2020-10-12 16:28:35.342054
77301	VBND	2020-10-06	51.715	51.715	51.4694	51.49	7645	{}	\N	2020-10-12 16:28:35.347784
77302	VBND	2020-10-07	51.6438	51.75	51.4502	51.6199	4205	{}	\N	2020-10-12 16:28:35.353915
77303	VBND	2020-10-08	51.66	51.89	51.55	51.88	16639	{}	\N	2020-10-12 16:28:35.360018
77304	VBND	2020-10-09	51.725	51.8299	51.5961	51.7606	7389	{}	\N	2020-10-12 16:28:35.365522
77305	VBND	2020-10-12	51.77	51.78	51.73	51.763	946	{}	\N	2020-10-12 16:28:35.371301
77306	VDE	2020-09-28	41.31	42.04	41.265	41.57	590734	{}	\N	2020-10-12 16:28:36.427451
77307	VDE	2020-09-29	41.48	41.535	40.05	40.53	630818	{}	\N	2020-10-12 16:28:36.438065
77308	VDE	2020-09-30	40.73	41.19	40.12	40.31	590312	{}	\N	2020-10-12 16:28:36.444785
77309	VDE	2020-10-01	39.9	40.06	38.89	39.14	1261948	{}	\N	2020-10-12 16:28:36.450137
77310	VDE	2020-10-02	38.05	39.8869	38.03	39.6	880774	{}	\N	2020-10-12 16:28:36.454956
77311	VDE	2020-10-05	40.14	40.81	39.875	40.78	865185	{}	\N	2020-10-12 16:28:36.459776
77312	VDE	2020-10-06	41.34	41.665	40.07	40.1	865136	{}	\N	2020-10-12 16:28:36.46435
77313	VDE	2020-10-07	40.34	40.89	39.9601	40.74	585381	{}	\N	2020-10-12 16:28:36.468796
77314	VDE	2020-10-08	41.14	42.4	40.96	42.36	767811	{}	\N	2020-10-12 16:28:36.473315
77315	VDE	2020-10-09	42.75	42.9	41.624	41.68	1182157	{}	\N	2020-10-12 16:28:36.477567
77316	VDE	2020-10-12	41.61	41.7	41.17	41.58	382704	{}	\N	2020-10-12 16:28:36.482066
77317	VEGI	2020-09-28	28.73	28.8028	28.6419	28.6419	6454	{}	\N	2020-10-12 16:28:37.717797
77318	VEGI	2020-09-29	28.63	28.63	28.39	28.59	6756	{}	\N	2020-10-12 16:28:37.727638
77319	VEGI	2020-09-30	28.78	28.96	28.68	28.7117	1653	{}	\N	2020-10-12 16:28:37.734304
77320	VEGI	2020-10-01	28.67	28.75	28.56	28.6536	3083	{}	\N	2020-10-12 16:28:37.740426
77321	VEGI	2020-10-02	28.71	28.94	28.54	28.9	4496	{}	\N	2020-10-12 16:28:37.745938
77322	VEGI	2020-10-05	29.35	29.4054	29.302	29.4054	2401	{}	\N	2020-10-12 16:28:37.750565
77323	VEGI	2020-10-06	29.56	29.76	29.32	29.32	4318	{}	\N	2020-10-12 16:28:37.75566
77324	VEGI	2020-10-07	29.37	29.8094	29.37	29.67	8852	{}	\N	2020-10-12 16:28:37.760553
77325	VEGI	2020-10-08	29.95	30.08	29.95	30.08	4892	{}	\N	2020-10-12 16:28:37.765018
77326	VEGI	2020-10-09	30.23	30.3099	30.13	30.2	4954	{}	\N	2020-10-12 16:28:37.769417
77327	VEGI	2020-10-12	30.21	30.31	30.21	30.31	695	{}	\N	2020-10-12 16:28:37.774077
77328	VHI	2020-09-28	14	14.22	13.7	13.86	11740	{}	\N	2020-10-12 16:28:40.946247
77329	VHI	2020-09-29	13.86	14.13	13.67	13.68	18058	{}	\N	2020-10-12 16:28:40.957232
77330	VHI	2020-09-30	13.8	13.965	13.16	13.16	23392	{}	\N	2020-10-12 16:28:40.964617
77331	VHI	2020-10-01	13.41	13.67	13.21	13.59	23830	{}	\N	2020-10-12 16:28:40.970508
77332	VHI	2020-10-02	13.34	14.38	13.34	14.12	14832	{}	\N	2020-10-12 16:28:40.975745
77333	VHI	2020-10-05	14.18	14.215	13.53	13.835	16488	{}	\N	2020-10-12 16:28:40.980864
77334	VHI	2020-10-06	13.56	14.38	13.56	13.84	9289	{}	\N	2020-10-12 16:28:40.985107
77335	VHI	2020-10-07	14.0458	14.4	13.95	14.1	21816	{}	\N	2020-10-12 16:28:40.989238
77336	VHI	2020-10-08	14.2512	14.75	14.2512	14.61	15226	{}	\N	2020-10-12 16:28:40.993577
77337	VHI	2020-10-09	14.73	14.8	14.23	14.57	16841	{}	\N	2020-10-12 16:28:40.998544
77338	VHI	2020-10-12	14.51	14.51	14.16	14.33	7696	{}	\N	2020-10-12 16:28:41.003322
77339	VIOO	2020-09-28	127.95	130.09	127.95	129.66	23962	{}	\N	2020-10-12 16:28:41.960763
77340	VIOO	2020-09-29	129.46	129.62	127.6919	128.88	21720	{}	\N	2020-10-12 16:28:41.975203
77341	VIOO	2020-09-30	129.1	131.32	128.38	129.14	54312	{}	\N	2020-10-12 16:28:41.98248
77342	VIOO	2020-10-01	130.03	131.065	129.11	131.065	19012	{}	\N	2020-10-12 16:28:41.988662
77343	VIOO	2020-10-02	128.22	132.88	128.13	132.59	27882	{}	\N	2020-10-12 16:28:41.993818
77344	VIOO	2020-10-05	133.74	135.4594	133.74	135.44	43577	{}	\N	2020-10-12 16:28:41.999002
77345	VIOO	2020-10-06	136.7	138.91	135.26	135.389	54978	{}	\N	2020-10-12 16:28:42.003049
77346	VIOO	2020-10-07	137.04	137.88	136.3348	137.36	36625	{}	\N	2020-10-12 16:28:42.008106
77347	VIOO	2020-10-08	138.7	139.5099	137.835	139.46	22949	{}	\N	2020-10-12 16:28:42.012771
77348	VIOO	2020-10-09	140.68	140.7484	139.465	139.9968	15926	{}	\N	2020-10-12 16:28:42.017153
77349	VIOO	2020-10-12	140.51	141.02	139.9791	140.6353	17398	{}	\N	2020-10-12 16:28:42.021435
77350	VIXY	2020-09-28	20.73	21.015	20.7073	20.85	2215655	{}	\N	2020-10-12 16:28:43.628151
77351	VIXY	2020-09-29	20.91	20.91	20.13	20.44	2757423	{}	\N	2020-10-12 16:28:43.6416
77352	VIXY	2020-09-30	20.17	20.62	19.91	20.41	3778297	{}	\N	2020-10-12 16:28:43.64882
77353	VIXY	2020-10-01	20.19	20.81	20.17	20.51	3052957	{}	\N	2020-10-12 16:28:43.65475
77354	VIXY	2020-10-02	21.53	21.62	20.74	21.09	4979225	{}	\N	2020-10-12 16:28:43.660111
77355	VIXY	2020-10-05	20.92	21.14	20.315	20.47	2564816	{}	\N	2020-10-12 16:28:43.665369
77356	VIXY	2020-10-06	20.29	20.93	19.915	20.73	5404810	{}	\N	2020-10-12 16:28:43.669858
77357	VIXY	2020-10-07	20.33	20.44	19.94	20.08	2438211	{}	\N	2020-10-12 16:28:43.67543
77358	VIXY	2020-10-08	19.82	19.96	19.235	19.29	3293807	{}	\N	2020-10-12 16:28:43.680221
77359	VIXY	2020-10-09	18.68	18.7	18.2601	18.27	3870707	{}	\N	2020-10-12 16:28:43.68547
77360	VIXY	2020-10-12	17.95	18.26	17.89	18.15	1459252	{}	\N	2020-10-12 16:28:43.691294
77361	VMC	2020-09-28	131.74	135.71	131.74	133.72	716608	{}	\N	2020-10-12 16:28:45.421576
77362	VMC	2020-09-29	132.85	133.595	131.18	132.7	1082004	{}	\N	2020-10-12 16:28:45.433937
77363	VMC	2020-09-30	133.36	136.44	133.235	135.54	1923790	{}	\N	2020-10-12 16:28:45.440555
77364	VMC	2020-10-01	136	138.81	133.83	134.38	1927951	{}	\N	2020-10-12 16:28:45.446973
77365	VMC	2020-10-02	132.25	138.05	132.25	136.8	2274581	{}	\N	2020-10-12 16:28:45.45317
77366	VMC	2020-10-05	138	143.63	137.98	141.55	3016021	{}	\N	2020-10-12 16:28:45.458365
77367	VMC	2020-10-06	142.11	147.15	140.62	141.31	3123865	{}	\N	2020-10-12 16:28:45.463454
77368	VMC	2020-10-07	143.81	147.75	143.33	146.48	1841703	{}	\N	2020-10-12 16:28:45.46827
77369	VMC	2020-10-08	149.99	153.88	148.66	153.09	3439251	{}	\N	2020-10-12 16:28:45.473095
77370	VMC	2020-10-09	153.13	153.515	149.44	150.78	1944294	{}	\N	2020-10-12 16:28:45.47796
77371	VMC	2020-10-12	151.27	152.19	148.18	151.06	654427	{}	\N	2020-10-12 16:28:45.482804
77372	VNDA	2020-09-28	9.43	9.5	9.17	9.33	684158	{}	\N	2020-10-12 16:28:48.108294
77373	VNDA	2020-09-29	9.34	9.56	9.23	9.505	766965	{}	\N	2020-10-12 16:28:48.119786
77374	VNDA	2020-09-30	9.55	9.785	9.5	9.66	541154	{}	\N	2020-10-12 16:28:48.127253
77375	VNDA	2020-10-01	9.73	9.94	9.71	9.88	418141	{}	\N	2020-10-12 16:28:48.133568
77376	VNDA	2020-10-02	9.81	9.95	9.62	9.77	560947	{}	\N	2020-10-12 16:28:48.139408
77377	VNDA	2020-10-05	9.87	10.218	9.87	10.08	353165	{}	\N	2020-10-12 16:28:48.144736
77378	VNDA	2020-10-06	10.07	10.29	9.99	10.07	669492	{}	\N	2020-10-12 16:28:48.151387
77379	VNDA	2020-10-07	10.38	10.63	10.04	10.36	839241	{}	\N	2020-10-12 16:28:48.156811
77380	VNDA	2020-10-08	10.4	10.71	10.3	10.67	431451	{}	\N	2020-10-12 16:28:48.161921
77381	VNDA	2020-10-09	10.78	11.05	10.7	10.97	485691	{}	\N	2020-10-12 16:28:48.166946
77382	VNDA	2020-10-12	11.07	11.21	10.97	11.17	225745	{}	\N	2020-10-12 16:28:48.172443
77383	XYL	2020-09-28	84.21	84.97	83.66	83.86	1025809	{}	\N	2020-10-12 16:28:50.228162
77384	XYL	2020-09-29	84.39	84.39	83.3	83.77	1831966	{}	\N	2020-10-12 16:28:50.241359
77385	XYL	2020-09-30	83.88	85.16	83.42	84.12	1663132	{}	\N	2020-10-12 16:28:50.24952
77386	XYL	2020-10-01	84.5	84.98	81.9092	82.56	1385021	{}	\N	2020-10-12 16:28:50.255624
77387	XYL	2020-10-02	80.95	85.02	80.95	84.43	1223614	{}	\N	2020-10-12 16:28:50.260854
77388	XYL	2020-10-05	85.13	86.52	85	86.32	834546	{}	\N	2020-10-12 16:28:50.265862
77389	XYL	2020-10-06	87.24	88.6	85.92	86.51	1172938	{}	\N	2020-10-12 16:28:50.27022
77390	XYL	2020-10-07	87.25	88.82	87.132	88.2	1209720	{}	\N	2020-10-12 16:28:50.274567
77391	XYL	2020-10-08	89.07	89.865	88.055	89.83	1146894	{}	\N	2020-10-12 16:28:50.27908
77392	XYL	2020-10-09	90.55	91.27	89.81	90.59	1428061	{}	\N	2020-10-12 16:28:50.283595
77393	XYL	2020-10-12	90.99	93.145	90.815	92.24	756924	{}	\N	2020-10-12 16:28:50.288508
77394	VBFC	2020-10-02	29.5	29.62	29.5	29.5	1632	{}	\N	2020-10-12 16:28:51.889152
77395	VBFC	2020-10-05	32.93	33	29.66	29.66	2004	{}	\N	2020-10-12 16:28:51.900494
77396	VBFC	2020-10-08	30.8159	30.8159	30.8159	30.8159	137	{}	\N	2020-10-12 16:28:51.906924
77397	VBFC	2020-10-12	30.06	30.06	30.06	30.06	465	{}	\N	2020-10-12 16:28:51.912525
77398	VC	2020-09-28	66.06	69.96	66.06	68.51	556987	{}	\N	2020-10-12 16:28:52.848148
77399	VC	2020-09-29	68.18	69.81	67.78	67.9	324343	{}	\N	2020-10-12 16:28:52.85951
77400	VC	2020-09-30	68.31	70.69	68.31	69.22	494873	{}	\N	2020-10-12 16:28:52.866766
77401	VC	2020-10-01	71.005	71.35	69.95	70.99	409767	{}	\N	2020-10-12 16:28:52.872479
77402	VC	2020-10-02	69.02	71.565	68.97	70.56	523378	{}	\N	2020-10-12 16:28:52.877928
77403	VC	2020-10-05	71.83	73.58	71.2	73.31	197131	{}	\N	2020-10-12 16:28:52.883085
77404	VC	2020-10-06	72.755	77.95	72.755	75.62	454550	{}	\N	2020-10-12 16:28:52.887626
77405	VC	2020-10-07	76.87	81.45	76.87	80.53	346641	{}	\N	2020-10-12 16:28:52.892229
77406	VC	2020-10-08	81.69	82.23	80.665	81.54	242293	{}	\N	2020-10-12 16:28:52.896794
77407	VC	2020-10-09	81.95	83.44	81.67	82.65	334375	{}	\N	2020-10-12 16:28:52.901124
77408	VC	2020-10-12	82.69	84.27	82.47	83.65	41726	{}	\N	2020-10-12 16:28:52.905409
77409	VDC	2020-09-28	162.08	163.5145	161.935	162.62	102635	{}	\N	2020-10-12 16:28:55.35691
77410	VDC	2020-09-29	162.79	163.03	161.26	161.6	138342	{}	\N	2020-10-12 16:28:55.364268
77411	VDC	2020-09-30	162.3	164.506	162.3	163.36	76937	{}	\N	2020-10-12 16:28:55.370323
77412	VDC	2020-10-01	164.24	164.7397	163.0536	164.39	111314	{}	\N	2020-10-12 16:28:55.376142
77413	VDC	2020-10-02	162.64	164.33	162.3301	163.72	98441	{}	\N	2020-10-12 16:28:55.38128
77414	VDC	2020-10-05	164.54	165	164	164.9	62490	{}	\N	2020-10-12 16:28:55.386508
77415	VDC	2020-10-06	165.31	166.105	163.55	163.86	68158	{}	\N	2020-10-12 16:28:55.391755
77416	VDC	2020-10-07	164.92	166.41	164.7527	165.74	79031	{}	\N	2020-10-12 16:28:55.396934
77417	VDC	2020-10-08	166.48	166.99	165.9289	166.86	188245	{}	\N	2020-10-12 16:28:55.40206
77418	VDC	2020-10-09	167.45	168.89	167.45	168.12	108024	{}	\N	2020-10-12 16:28:55.407369
77419	VDC	2020-10-12	168.71	170.865	168.71	170.7713	29832	{}	\N	2020-10-12 16:28:55.41277
77420	VEEV	2020-09-28	278.3	287.1799	277.42	281.92	1272806	{}	\N	2020-10-12 16:28:56.333492
77421	VEEV	2020-09-29	282.92	285.98	280.36	280.99	1156536	{}	\N	2020-10-12 16:28:56.347124
77422	VEEV	2020-09-30	280.41	285.45	277.15	281.19	1509775	{}	\N	2020-10-12 16:28:56.35547
77423	VEEV	2020-10-01	283.65	285.38	276.28	277.04	1835280	{}	\N	2020-10-12 16:28:56.361804
77424	VEEV	2020-10-02	266.2	280	266	275.56	1696657	{}	\N	2020-10-12 16:28:56.367595
77425	VEEV	2020-10-05	275.14	279.77	273.88	278.45	1464061	{}	\N	2020-10-12 16:28:56.373466
77426	VEEV	2020-10-06	279.72	291.42	278.11	285.88	2285221	{}	\N	2020-10-12 16:28:56.379884
77427	VEEV	2020-10-07	289	290.82	285.57	287.41	1101536	{}	\N	2020-10-12 16:28:56.386405
77428	VEEV	2020-10-08	290	290.9	284.03	290.75	1133588	{}	\N	2020-10-12 16:28:56.39218
77429	VEEV	2020-10-09	292.68	297.06	292.33	294.12	1048512	{}	\N	2020-10-12 16:28:56.398228
77430	VEEV	2020-10-12	298.75	308.2809	298.66	306.5681	1428174	{}	\N	2020-10-12 16:28:56.404007
77431	VEU	2020-09-28	50.42	50.5172	50.29	50.46	1687887	{}	\N	2020-10-12 16:28:57.55383
77432	VEU	2020-09-29	50.41	50.54	50.21	50.42	2466355	{}	\N	2020-10-12 16:28:57.566277
77433	VEU	2020-09-30	50.4	50.74	50.31	50.45	2755692	{}	\N	2020-10-12 16:28:57.573051
77434	VEU	2020-10-01	50.77	50.83	50.51	50.82	1626770	{}	\N	2020-10-12 16:28:57.579309
77435	VEU	2020-10-02	50.17	50.75	50.13	50.59	1843736	{}	\N	2020-10-12 16:28:57.584331
77436	VEU	2020-10-05	50.96	51.36	50.94	51.36	1369909	{}	\N	2020-10-12 16:28:57.589977
77437	VEU	2020-10-06	51.41	51.5177	50.8306	50.94	3016648	{}	\N	2020-10-12 16:28:57.594803
77438	VEU	2020-10-07	51.39	51.55	51.2965	51.47	1785378	{}	\N	2020-10-12 16:28:57.59973
77439	VEU	2020-10-08	51.66	51.84	51.61	51.84	1373664	{}	\N	2020-10-12 16:28:57.60455
77440	VEU	2020-10-09	52.01	52.2323	51.95	52.19	2269523	{}	\N	2020-10-12 16:28:57.609159
77441	VEU	2020-10-12	52.35	52.5	52.28	52.5	488755	{}	\N	2020-10-12 16:28:57.614107
77442	VGR	2020-09-28	9.8	10.01	9.8	9.92	600574	{}	\N	2020-10-12 16:28:59.616549
77443	VGR	2020-09-29	9.92	9.96	9.605	9.69	1149115	{}	\N	2020-10-12 16:28:59.630159
77444	VGR	2020-09-30	9.67	9.915	9.59	9.69	1579623	{}	\N	2020-10-12 16:28:59.638693
77445	VGR	2020-10-01	9.7	9.81	9.54	9.8	1018270	{}	\N	2020-10-12 16:28:59.650916
77446	VGR	2020-10-02	9.68	10.05	9.62	10.01	1246998	{}	\N	2020-10-12 16:28:59.659167
77447	VGR	2020-10-05	10.12	10.19	9.9401	10.1	1225862	{}	\N	2020-10-12 16:28:59.664217
77448	VGR	2020-10-06	10.25	10.56	10.18	10.46	1836260	{}	\N	2020-10-12 16:28:59.669282
77449	VGR	2020-10-07	10.53	10.775	10.45	10.7	1442820	{}	\N	2020-10-12 16:28:59.674341
77450	VGR	2020-10-08	10.79	10.91	10.66	10.83	868540	{}	\N	2020-10-12 16:28:59.679341
77451	VGR	2020-10-09	10.9	11.085	10.875	10.89	934483	{}	\N	2020-10-12 16:28:59.684615
77452	VGR	2020-10-12	10.85	11.005	10.76	10.91	300308	{}	\N	2020-10-12 16:28:59.689575
77453	VIAV	2020-09-28	11.74	11.975	11.6	11.905	1918364	{}	\N	2020-10-12 16:29:00.639053
77454	VIAV	2020-09-29	11.93	12.05	11.75	11.82	1071007	{}	\N	2020-10-12 16:29:00.653253
77455	VIAV	2020-09-30	11.79	11.93	11.605	11.73	1250052	{}	\N	2020-10-12 16:29:00.661185
77456	VIAV	2020-10-01	11.79	11.91	11.58	11.91	2658682	{}	\N	2020-10-12 16:29:00.667852
77457	VIAV	2020-10-02	11.62	11.93	11.56	11.81	1739728	{}	\N	2020-10-12 16:29:00.674391
77458	VIAV	2020-10-05	11.88	12.42	11.88	12.4	4589282	{}	\N	2020-10-12 16:29:00.680021
77459	VIAV	2020-10-06	12.49	12.65	12.31	12.34	2456630	{}	\N	2020-10-12 16:29:00.68528
77460	VIAV	2020-10-07	12.43	12.59	12.345	12.48	1948104	{}	\N	2020-10-12 16:29:00.69033
77461	VIAV	2020-10-08	12.64	12.99	12.58	12.89	3182648	{}	\N	2020-10-12 16:29:00.695552
77462	VIAV	2020-10-09	12.96	13	12.66	12.68	2128297	{}	\N	2020-10-12 16:29:00.700777
77463	VIAV	2020-10-12	12.79	12.79	12.56	12.61	558604	{}	\N	2020-10-12 16:29:00.706029
77464	VIRC	2020-09-28	2.07	2.1	2.05	2.0904	20058	{}	\N	2020-10-12 16:29:01.725257
77465	VIRC	2020-09-29	2.0999	2.1171	2.08	2.1171	4078	{}	\N	2020-10-12 16:29:01.734808
77466	VIRC	2020-09-30	2.14	2.16	2.05	2.15	17348	{}	\N	2020-10-12 16:29:01.74229
77467	VIRC	2020-10-01	2.135	2.34	2.1005	2.1701	15498	{}	\N	2020-10-12 16:29:01.748955
77468	VIRC	2020-10-02	2.1011	2.5	2.1011	2.3	209459	{}	\N	2020-10-12 16:29:01.754835
77469	VIRC	2020-10-05	2.2999	2.36	2.21	2.3	23457	{}	\N	2020-10-12 16:29:01.760389
77470	VIRC	2020-10-06	2.3257	2.3257	2.05	2.1871	12994	{}	\N	2020-10-12 16:29:01.76572
77471	VIRC	2020-10-07	2.1639	2.2	2.14	2.2	3650	{}	\N	2020-10-12 16:29:01.770768
77472	VIRC	2020-10-08	2.18	2.2	2.18	2.2	1998	{}	\N	2020-10-12 16:29:01.776094
77473	VIRC	2020-10-09	2.17	2.21	2.17	2.2	6606	{}	\N	2020-10-12 16:29:01.781647
77474	VIRC	2020-10-12	2.23	2.23	2.22	2.2217	1810	{}	\N	2020-10-12 16:29:01.786349
77475	VIXM	2020-09-28	41.19	41.475	41.0815	41.2	40747	{}	\N	2020-10-12 16:29:03.543167
77476	VIXM	2020-09-29	41.12	41.17	40.565	40.83	29649	{}	\N	2020-10-12 16:29:03.555039
77477	VIXM	2020-09-30	40.72	40.8934	40.56	40.68	29426	{}	\N	2020-10-12 16:29:03.561813
77478	VIXM	2020-10-01	40.71	41.42	40.6401	41.22	25990	{}	\N	2020-10-12 16:29:03.567488
77479	VIXM	2020-10-02	42.12	42.12	41.41	41.73	140897	{}	\N	2020-10-12 16:29:03.572596
77480	VIXM	2020-10-05	41.6	41.69	40.84	40.84	37655	{}	\N	2020-10-12 16:29:03.577462
77481	VIXM	2020-10-06	40.78	41.37	40.4149	40.88	57680	{}	\N	2020-10-12 16:29:03.581761
77482	VIXM	2020-10-07	40.85	40.85	40.4	40.71	34785	{}	\N	2020-10-12 16:29:03.585802
77483	VIXM	2020-10-08	40.4	40.74	40.2	40.24	25083	{}	\N	2020-10-12 16:29:03.590303
77484	VIXM	2020-10-09	39.97	39.97	38.8	39.14	50521	{}	\N	2020-10-12 16:29:03.594639
77485	VIXM	2020-10-12	38.37	38.88	38.37	38.7112	14613	{}	\N	2020-10-12 16:29:03.599
77486	VLT	2020-09-28	12.32	12.3868	12.28	12.29	55462	{}	\N	2020-10-12 16:29:04.680313
77487	VLT	2020-09-29	12.33	12.33	12.2	12.32	88542	{}	\N	2020-10-12 16:29:04.692345
77488	VLT	2020-09-30	12.33	12.43	12.22	12.26	364048	{}	\N	2020-10-12 16:29:04.704249
77489	VLT	2020-10-01	12.33	12.4	12.33	12.38	40510	{}	\N	2020-10-12 16:29:04.712381
77490	VLT	2020-10-02	12.36	12.39	12.0646	12.37	84794	{}	\N	2020-10-12 16:29:04.71862
77491	VLT	2020-10-05	12.31	12.48	12.31	12.43	42140	{}	\N	2020-10-12 16:29:04.723775
77492	VLT	2020-10-06	12.42	12.52	12.42	12.47	52203	{}	\N	2020-10-12 16:29:04.728543
77493	VLT	2020-10-07	12.5	12.53	12.45	12.46	39753	{}	\N	2020-10-12 16:29:04.733363
77494	VLT	2020-10-08	12.45	12.55	12.45	12.54	41258	{}	\N	2020-10-12 16:29:04.738178
77495	VLT	2020-10-09	12.55	12.58	12.51	12.53	60971	{}	\N	2020-10-12 16:29:04.742781
77496	VLT	2020-10-12	12.5	12.62	12.5	12.6147	27104	{}	\N	2020-10-12 16:29:04.747502
77497	VMO	2020-09-28	12.14	12.2	12.14	12.16	145115	{}	\N	2020-10-12 16:29:07.386133
77498	VMO	2020-09-29	12.16	12.19	12.149	12.17	197636	{}	\N	2020-10-12 16:29:07.397079
77499	VMO	2020-09-30	12.22	12.2227	12.16	12.22	220464	{}	\N	2020-10-12 16:29:07.403849
77500	VMO	2020-10-01	12.22	12.32	12.22	12.3	192904	{}	\N	2020-10-12 16:29:07.409948
77501	VMO	2020-10-02	12.26	12.38	12.26	12.36	154076	{}	\N	2020-10-12 16:29:07.415574
77502	VMO	2020-10-05	12.37	12.37	12.29	12.31	116988	{}	\N	2020-10-12 16:29:07.420732
77503	VMO	2020-10-06	12.3	12.38	12.3	12.37	140887	{}	\N	2020-10-12 16:29:07.425335
77504	VMO	2020-10-07	12.4	12.42	12.3395	12.37	170624	{}	\N	2020-10-12 16:29:07.429714
77505	VMO	2020-10-08	12.36	12.42	12.36	12.41	119992	{}	\N	2020-10-12 16:29:07.434184
77506	VMO	2020-10-09	12.4	12.47	12.39	12.39	102606	{}	\N	2020-10-12 16:29:07.438593
77507	VMO	2020-10-12	12.39	12.45	12.39	12.4	75704	{}	\N	2020-10-12 16:29:07.443263
77508	VBTX	2020-09-28	16.47	17.08	16.47	16.94	177423	{}	\N	2020-10-12 16:29:10.475816
77509	VBTX	2020-09-29	16.85	17.03	16.67	16.92	186239	{}	\N	2020-10-12 16:29:10.486015
77510	VBTX	2020-09-30	17.04	17.455	16.8	17.03	227802	{}	\N	2020-10-12 16:29:10.492935
77511	VBTX	2020-10-01	16.99	17.43	16.9061	17.43	281909	{}	\N	2020-10-12 16:29:10.49869
77512	VBTX	2020-10-02	17.06	18.125	17.06	18	234850	{}	\N	2020-10-12 16:29:10.504283
77513	VBTX	2020-10-05	18.3	18.83	18.285	18.8	173570	{}	\N	2020-10-12 16:29:10.509297
77514	VBTX	2020-10-06	19.14	19.78	18.94	19.12	377847	{}	\N	2020-10-12 16:29:10.51377
77515	VBTX	2020-10-07	19.38	19.85	19.3	19.55	333446	{}	\N	2020-10-12 16:29:10.518345
77516	VBTX	2020-10-08	19.8	20.26	19.61	20.22	263839	{}	\N	2020-10-12 16:29:10.522652
77517	VBTX	2020-10-09	20.46	20.7244	19.81	19.87	224596	{}	\N	2020-10-12 16:29:10.527061
77518	VBTX	2020-10-12	19.78	20.28	19.77	20.23	40206	{}	\N	2020-10-12 16:29:10.531264
77519	VCYT	2020-09-28	31.28	31.99	30.68	31.7	242166	{}	\N	2020-10-12 16:29:12.196486
77520	VCYT	2020-09-29	31.57	33.2	31.1	32.79	250684	{}	\N	2020-10-12 16:29:12.208484
77521	VCYT	2020-09-30	33.13	33.55	32.19	32.49	341597	{}	\N	2020-10-12 16:29:12.216131
77522	VCYT	2020-10-01	32.8	33.45	32.15	33.43	266362	{}	\N	2020-10-12 16:29:12.222449
77523	VCYT	2020-10-02	32.88	33.8	32.19	32.5	172979	{}	\N	2020-10-12 16:29:12.227795
77524	VCYT	2020-10-05	32.94	34.13	32.82	34.13	239898	{}	\N	2020-10-12 16:29:12.233428
77525	VCYT	2020-10-06	34.4	35.87	33.9504	35.13	465601	{}	\N	2020-10-12 16:29:12.237685
77526	VCYT	2020-10-07	35.56	36.63	35.42	36.3	440102	{}	\N	2020-10-12 16:29:12.242605
77527	VCYT	2020-10-08	37.86	38.75	36.7211	38.64	724887	{}	\N	2020-10-12 16:29:12.247201
77528	VCYT	2020-10-09	39.21	41.65	38.4	41.46	471544	{}	\N	2020-10-12 16:29:12.252048
77529	VCYT	2020-10-12	41.91	42.95	41.42	41.78	221888	{}	\N	2020-10-12 16:29:12.256559
77530	VERU	2020-09-28	2.62	2.68	2.613	2.64	260581	{}	\N	2020-10-12 16:29:13.784757
77531	VERU	2020-09-29	2.72	2.74	2.58	2.59	346483	{}	\N	2020-10-12 16:29:13.79902
77532	VERU	2020-09-30	2.59	2.665	2.58	2.62	253158	{}	\N	2020-10-12 16:29:13.806525
77533	VERU	2020-10-01	2.64	2.64	2.55	2.57	297799	{}	\N	2020-10-12 16:29:13.812909
77534	VERU	2020-10-02	2.51	2.56	2.4891	2.5	250248	{}	\N	2020-10-12 16:29:13.818324
77535	VERU	2020-10-05	2.5	2.58	2.5	2.57	262517	{}	\N	2020-10-12 16:29:13.823174
77536	VERU	2020-10-06	2.58	2.62	2.555	2.56	311110	{}	\N	2020-10-12 16:29:13.827737
77537	VERU	2020-10-07	2.57	2.66	2.54	2.65	471941	{}	\N	2020-10-12 16:29:13.832161
77538	VERU	2020-10-08	2.68	2.74	2.663	2.7	293279	{}	\N	2020-10-12 16:29:13.836466
77539	VERU	2020-10-09	2.72	2.7299	2.68	2.71	133154	{}	\N	2020-10-12 16:29:13.840755
77540	VERU	2020-10-12	2.7	2.72	2.66	2.6711	76736	{}	\N	2020-10-12 16:29:13.845011
77541	VGLT	2020-09-28	100.51	100.58	100.28	100.41	609928	{}	\N	2020-10-12 16:29:17.328233
77542	VGLT	2020-09-29	100.54	100.77	100.39	100.5	186612	{}	\N	2020-10-12 16:29:17.3334
77543	VGLT	2020-09-30	100.03	100.03	99.165	99.66	780433	{}	\N	2020-10-12 16:29:17.337789
77544	VGLT	2020-10-01	99.09	99.87	98.9	99.7	300448	{}	\N	2020-10-12 16:29:17.342222
77545	VGLT	2020-10-02	99.81	99.81	98.96	99.29	905250	{}	\N	2020-10-12 16:29:17.34654
77546	VGLT	2020-10-05	98.23	98.23	97.39	97.43	690736	{}	\N	2020-10-12 16:29:17.351027
77547	VGLT	2020-10-06	97.27	98.42	96.91	97.96	221200	{}	\N	2020-10-12 16:29:17.35571
77548	VGLT	2020-10-07	97.42	97.89	97	97.22	230194	{}	\N	2020-10-12 16:29:17.364097
77549	VGLT	2020-10-08	97.64	97.79	97.48	97.75	184321	{}	\N	2020-10-12 16:29:17.371035
77550	VGLT	2020-10-09	97.35	97.81	97	97.71	329765	{}	\N	2020-10-12 16:29:17.378245
77551	VGLT	2020-10-12	97.89	97.97	97.728	97.77	54926	{}	\N	2020-10-12 16:29:17.384276
77552	VICI	2020-09-28	24.22	24.34	23.84	24.19	3348998	{}	\N	2020-10-12 16:29:18.441787
77553	VICI	2020-09-29	23.75	23.77	23.21	23.39	16626610	{}	\N	2020-10-12 16:29:18.452276
77554	VICI	2020-09-30	23.55	23.94	23.2	23.37	11207648	{}	\N	2020-10-12 16:29:18.459684
77555	VICI	2020-10-01	23.63	23.75	23.26	23.72	21066090	{}	\N	2020-10-12 16:29:18.465779
77556	VICI	2020-10-02	23.23	24.16	23.12	24.06	10482357	{}	\N	2020-10-12 16:29:18.470888
77557	VICI	2020-10-05	24.33	24.42	23.78	24.25	4744035	{}	\N	2020-10-12 16:29:18.476236
77558	VICI	2020-10-06	24.36	24.43	23.66	23.85	12397781	{}	\N	2020-10-12 16:29:18.481405
77559	VICI	2020-10-07	24	24.1	23.68	23.72	7706569	{}	\N	2020-10-12 16:29:18.486522
77560	VICI	2020-10-08	23.89	24.18	23.76	24.02	7037459	{}	\N	2020-10-12 16:29:18.491619
77561	VICI	2020-10-09	24.25	24.38	23.56	23.64	13585593	{}	\N	2020-10-12 16:29:18.49659
77562	VICI	2020-10-12	23.83	23.8574	23.59	23.855	1364344	{}	\N	2020-10-12 16:29:18.501731
77563	VIOV	2020-09-28	105.31	107.37	105.31	106.9	20779	{}	\N	2020-10-12 16:29:19.551216
77564	VIOV	2020-09-29	106.86	106.86	104.65	105.89	14136	{}	\N	2020-10-12 16:29:19.563577
77565	VIOV	2020-09-30	106.21	107.835	105.61	106.2	38248	{}	\N	2020-10-12 16:29:19.571289
77566	VIOV	2020-10-01	107.22	107.74	106.195	107.74	20096	{}	\N	2020-10-12 16:29:19.578078
77567	VIOV	2020-10-02	105.43	109.94	105.43	109.7	37915	{}	\N	2020-10-12 16:29:19.583799
77568	VIOV	2020-10-05	111.55	112.2377	111.12	112.21	18392	{}	\N	2020-10-12 16:29:19.589013
77569	VIOV	2020-10-06	113.3	115.44	111.98	111.98	49101	{}	\N	2020-10-12 16:29:19.593873
77570	VIOV	2020-10-07	113.64	114.52	113.195	114.17	16794	{}	\N	2020-10-12 16:29:19.598933
77571	VIOV	2020-10-08	115.24	116.07	114.709	116	16742	{}	\N	2020-10-12 16:29:19.604027
77572	VIOV	2020-10-09	117.1	117.1	115.65	116	28324	{}	\N	2020-10-12 16:29:19.60956
77573	VIOV	2020-10-12	116.32	116.71	115.86	116.32	9344	{}	\N	2020-10-12 16:29:19.615126
77574	VIVO	2020-09-28	16.6	17	16.44	16.61	353817	{}	\N	2020-10-12 16:29:20.815336
77575	VIVO	2020-09-29	16.6	17.34	16.6	17.24	432228	{}	\N	2020-10-12 16:29:20.826665
77576	VIVO	2020-09-30	17.22	17.53	16.68	16.98	710525	{}	\N	2020-10-12 16:29:20.833708
77577	VIVO	2020-10-01	17.2	17.82	17.06	17.17	407176	{}	\N	2020-10-12 16:29:20.839767
77578	VIVO	2020-10-02	16.705	17.3	16.61	17.16	310848	{}	\N	2020-10-12 16:29:20.84523
77579	VIVO	2020-10-05	17.47	18.3	17.39	18.1	359332	{}	\N	2020-10-12 16:29:20.84979
77580	VIVO	2020-10-06	18.12	18.7	18.08	18.19	465728	{}	\N	2020-10-12 16:29:20.853514
77581	VIVO	2020-10-07	18.37	19.19	18.31	18.63	618634	{}	\N	2020-10-12 16:29:20.85728
77582	VIVO	2020-10-08	18.78	19.0721	18.34	18.58	287193	{}	\N	2020-10-12 16:29:20.861167
77583	VIVO	2020-10-09	18.79	18.95	18.38	18.84	527805	{}	\N	2020-10-12 16:29:20.864975
77584	VIVO	2020-10-12	18.87	19.18	18.6	19.12	278671	{}	\N	2020-10-12 16:29:20.869364
77585	VNM	2020-09-28	14.69	14.8	14.6067	14.8	126384	{}	\N	2020-10-12 16:29:25.000534
77586	VNM	2020-09-29	14.77	14.77	14.63	14.66	179173	{}	\N	2020-10-12 16:29:25.015219
77587	VNM	2020-09-30	14.65	14.88	14.65	14.85	96020	{}	\N	2020-10-12 16:29:25.023699
77588	VNM	2020-10-01	14.87	14.99	14.85	14.95	159413	{}	\N	2020-10-12 16:29:25.029939
77589	VNM	2020-10-02	14.69	14.94	14.68	14.85	214197	{}	\N	2020-10-12 16:29:25.03582
77590	VNM	2020-10-05	14.85	14.98	14.85	14.98	181183	{}	\N	2020-10-12 16:29:25.041004
77591	VNM	2020-10-06	14.99	15	14.74	14.82	122056	{}	\N	2020-10-12 16:29:25.045849
77592	VNM	2020-10-07	15	15.1045	14.97	15.1	134648	{}	\N	2020-10-12 16:29:25.05079
77593	VNM	2020-10-08	15.06	15.1	15.02	15.08	51556	{}	\N	2020-10-12 16:29:25.055554
77594	VNM	2020-10-09	15.095	15.2	15.08	15.15	232983	{}	\N	2020-10-12 16:29:25.06052
77595	VNM	2020-10-12	15.15	15.2	15.125	15.199	39911	{}	\N	2020-10-12 16:29:25.065425
77596	VOC	2020-09-28	1.3	1.37	1.3	1.36	23897	{}	\N	2020-10-12 16:29:26.657551
77597	VOC	2020-09-29	1.37	2.37	1.37	1.64	13634147	{}	\N	2020-10-12 16:29:26.669572
77598	VOC	2020-09-30	1.53	1.63	1.42	1.59	619339	{}	\N	2020-10-12 16:29:26.677391
77599	VOC	2020-10-01	1.52	1.59	1.44	1.54	129778	{}	\N	2020-10-12 16:29:26.684036
77600	VOC	2020-10-02	1.54	1.58	1.48	1.56	104185	{}	\N	2020-10-12 16:29:26.69035
77601	VOC	2020-10-05	1.5	1.59	1.4271	1.47	185918	{}	\N	2020-10-12 16:29:26.69553
77602	VOC	2020-10-06	1.47	1.48	1.36	1.46	142395	{}	\N	2020-10-12 16:29:26.701073
77603	VOC	2020-10-07	1.42	1.46	1.32	1.36	155386	{}	\N	2020-10-12 16:29:26.706263
77604	VOC	2020-10-08	1.39	1.8188	1.38	1.6395	1157902	{}	\N	2020-10-12 16:29:26.711563
77605	VOC	2020-10-09	1.64	1.75	1.49	1.61	458698	{}	\N	2020-10-12 16:29:26.71673
77606	VOC	2020-10-12	1.51	1.59	1.46	1.5199	223216	{}	\N	2020-10-12 16:29:26.7218
77607	VOO	2020-09-28	307.42	309.02	306.43	308.36	2512483	{}	\N	2020-10-12 16:29:28.468254
77608	VOO	2020-09-29	306.84	307.51	304.65	305.3	2336779	{}	\N	2020-10-12 16:29:28.482689
77609	VOO	2020-09-30	306.04	310.7499	305.85	307.65	4307431	{}	\N	2020-10-12 16:29:28.490544
77610	VOO	2020-10-01	310.16	311.11	307.76	309.6	3032641	{}	\N	2020-10-12 16:29:28.4966
77611	VOO	2020-10-02	304.72	308.58	304.215	306.72	3199915	{}	\N	2020-10-12 16:29:28.502363
77612	VOO	2020-10-05	308.69	312.29	308.69	312.08	2377734	{}	\N	2020-10-12 16:29:28.507543
77613	VOO	2020-10-06	312.25	314.32	307.1806	307.72	3368393	{}	\N	2020-10-12 16:29:28.512619
77614	VOO	2020-10-07	310.6072	313.81	310.6	313	1711957	{}	\N	2020-10-12 16:29:28.517318
77615	VOO	2020-10-08	314.96	315.85	314.05	315.77	1739792	{}	\N	2020-10-12 16:29:28.521997
77616	VOO	2020-10-09	317.47	319.0999	316.85	318.58	2773984	{}	\N	2020-10-12 16:29:28.526984
77617	VOO	2020-10-12	321.14	324.01	320.66	323.935	893420	{}	\N	2020-10-12 16:29:28.531762
77618	VPG	2020-09-28	24.47	25.33	24.47	24.97	32447	{}	\N	2020-10-12 16:29:30.187796
77619	VPG	2020-09-29	25.08	25.28	24.7	25.13	58812	{}	\N	2020-10-12 16:29:30.201404
77620	VPG	2020-09-30	25.12	25.68	25.12	25.32	124645	{}	\N	2020-10-12 16:29:30.208874
77621	VPG	2020-10-01	25.33	26	25.33	25.76	109576	{}	\N	2020-10-12 16:29:30.215253
77622	VPG	2020-10-02	25.41	25.98	25.305	25.86	65396	{}	\N	2020-10-12 16:29:30.220689
77623	VPG	2020-10-05	26.06	26.46	26.06	26.29	63744	{}	\N	2020-10-12 16:29:30.226671
77624	VPG	2020-10-06	26.4	26.69	26	26.12	82727	{}	\N	2020-10-12 16:29:30.231139
77625	VPG	2020-10-07	26.34	27.3068	26.34	27.13	115880	{}	\N	2020-10-12 16:29:30.235592
77626	VPG	2020-10-08	27.28	27.49	27.05	27.28	69078	{}	\N	2020-10-12 16:29:30.240287
77627	VPG	2020-10-09	27.6	27.84	27.01	27.27	35258	{}	\N	2020-10-12 16:29:30.244843
77628	VPG	2020-10-12	27.26	27.32	27.11	27.2	26718	{}	\N	2020-10-12 16:29:30.249646
77629	VGZ	2020-09-28	1.08	1.1	1.08	1.08	222868	{}	\N	2020-10-12 16:29:31.180567
77630	VGZ	2020-09-29	1.09	1.11	1.08	1.09	194239	{}	\N	2020-10-12 16:29:31.191009
77631	VGZ	2020-09-30	1.09	1.11	1.07	1.09	134319	{}	\N	2020-10-12 16:29:31.197829
77632	VGZ	2020-10-01	1.1	1.15	1.09	1.11	140097	{}	\N	2020-10-12 16:29:31.204122
77633	VGZ	2020-10-02	1.09	1.11	1.03	1.06	348861	{}	\N	2020-10-12 16:29:31.209669
77634	VGZ	2020-10-05	1.07	1.09	1.03	1.08	154027	{}	\N	2020-10-12 16:29:31.215208
77635	VGZ	2020-10-06	1.08	1.08	1.06	1.0681	217570	{}	\N	2020-10-12 16:29:31.219925
77636	VGZ	2020-10-07	1.1	1.1	1.04	1.07	163412	{}	\N	2020-10-12 16:29:31.224532
77637	VGZ	2020-10-08	1.07	1.09	1.04	1.04	226560	{}	\N	2020-10-12 16:29:31.229369
77638	VGZ	2020-10-09	1.04	1.15	1.04	1.12	361452	{}	\N	2020-10-12 16:29:31.234085
77639	VGZ	2020-10-12	1.1	1.14	1.1	1.13	105313	{}	\N	2020-10-12 16:29:31.238996
77640	VIDI	2020-09-28	20.77	20.82	20.7574	20.79	22153	{}	\N	2020-10-12 16:29:33.905883
77641	VIDI	2020-09-29	20.75	20.8399	20.7138	20.8	11886	{}	\N	2020-10-12 16:29:33.91991
77642	VIDI	2020-09-30	20.81	20.95	20.81	20.9	34413	{}	\N	2020-10-12 16:29:33.927893
77643	VIDI	2020-10-01	20.94	21.08	20.9319	21.08	150890	{}	\N	2020-10-12 16:29:33.934294
77644	VIDI	2020-10-02	20.77	20.965	20.77	20.92	36393	{}	\N	2020-10-12 16:29:33.939534
77645	VIDI	2020-10-05	21.19	21.28	21.17	21.28	29648	{}	\N	2020-10-12 16:29:33.944446
77646	VIDI	2020-10-06	21.25	21.39	21.16	21.16	27646	{}	\N	2020-10-12 16:29:33.94901
77647	VIDI	2020-10-07	21.32	21.4	21.2658	21.4	24236	{}	\N	2020-10-12 16:29:33.953262
77648	VIDI	2020-10-08	21.47	21.58	21.4603	21.56	28545	{}	\N	2020-10-12 16:29:33.957768
77649	VIDI	2020-10-09	21.5301	21.69	21.5301	21.69	21328	{}	\N	2020-10-12 16:29:33.962353
77650	VIDI	2020-10-12	21.69	21.69	21.5501	21.63	2449	{}	\N	2020-10-12 16:29:33.967331
77651	VIPS	2020-09-28	16.57	16.59	15.63	15.9	16954705	{}	\N	2020-10-12 16:29:34.968303
77652	VIPS	2020-09-29	15.94	16.09	15.29	15.41	25233903	{}	\N	2020-10-12 16:29:34.979034
77653	VIPS	2020-09-30	15.35	15.95	15.29	15.64	17847011	{}	\N	2020-10-12 16:29:34.98599
77654	VIPS	2020-10-01	15.87	15.955	15.54	15.85	9014872	{}	\N	2020-10-12 16:29:34.992356
77655	VIPS	2020-10-02	15.5	15.5	14.94	15.35	11321549	{}	\N	2020-10-12 16:29:34.997942
77656	VIPS	2020-10-05	15.39	15.43	14.865	14.98	22444132	{}	\N	2020-10-12 16:29:35.003146
77657	VIPS	2020-10-06	15.05	15.335	14.89	15.07	14019369	{}	\N	2020-10-12 16:29:35.007834
77658	VIPS	2020-10-07	15.27	15.42	15.05	15.29	8142879	{}	\N	2020-10-12 16:29:35.012587
77659	VIPS	2020-10-08	15.27	15.4855	15.21	15.25	5860272	{}	\N	2020-10-12 16:29:35.017482
77660	VIPS	2020-10-09	15.34	15.65	15.14	15.6	8828768	{}	\N	2020-10-12 16:29:35.022001
77661	VIPS	2020-10-12	15.86	16.86	15.63	16.645	13433674	{}	\N	2020-10-12 16:29:35.027144
77662	VKTX	2020-09-28	6.14	6.16	5.94	5.99	998005	{}	\N	2020-10-12 16:29:36.147619
77663	VKTX	2020-09-29	5.96	6.005	5.855	5.93	936150	{}	\N	2020-10-12 16:29:36.161017
77664	VKTX	2020-09-30	5.93	5.99	5.8	5.82	1177828	{}	\N	2020-10-12 16:29:36.168308
77665	VKTX	2020-10-01	5.82	5.91	5.7	5.72	1164025	{}	\N	2020-10-12 16:29:36.17429
77666	VKTX	2020-10-02	5.51	5.71	5.43	5.55	1596336	{}	\N	2020-10-12 16:29:36.179253
77667	VKTX	2020-10-05	5.56	5.91	5.56	5.91	1129250	{}	\N	2020-10-12 16:29:36.184335
77668	VKTX	2020-10-06	5.96	6.03	5.8127	5.86	1104300	{}	\N	2020-10-12 16:29:36.188706
77669	VKTX	2020-10-07	5.9	6.17	5.8899	6.11	1607046	{}	\N	2020-10-12 16:29:36.192924
77670	VKTX	2020-10-08	6.23	6.3062	6.06	6.12	877032	{}	\N	2020-10-12 16:29:36.197216
77671	VKTX	2020-10-09	6.18	6.2	6.07	6.12	971956	{}	\N	2020-10-12 16:29:36.201297
77672	VKTX	2020-10-12	6.09	6.1337	5.975	6.03	306002	{}	\N	2020-10-12 16:29:36.205581
77673	VNQ	2020-09-28	78.79	79.67	78.7	79.29	3488964	{}	\N	2020-10-12 16:29:38.621268
77674	VNQ	2020-09-29	79.28	79.33	78.08	78.72	3676694	{}	\N	2020-10-12 16:29:38.634692
77675	VNQ	2020-09-30	79.12	79.88	78.15	78.96	4356189	{}	\N	2020-10-12 16:29:38.642414
77676	VNQ	2020-10-01	79.28	80.615	78.93	80.54	6382647	{}	\N	2020-10-12 16:29:38.648834
77677	VNQ	2020-10-02	79.46	82.19	79.17	81.93	3290043	{}	\N	2020-10-12 16:29:38.654821
77678	VNQ	2020-10-05	82.23	82.66	80.83	82.4	2779477	{}	\N	2020-10-12 16:29:38.659863
77679	VNQ	2020-10-06	82.61	83.0738	81.54	81.86	5819331	{}	\N	2020-10-12 16:29:38.665513
77680	VNQ	2020-10-07	82.37	82.4	81.6	81.98	2362755	{}	\N	2020-10-12 16:29:38.670503
77681	VNQ	2020-10-08	82.42	83.58	82.28	83.35	2268522	{}	\N	2020-10-12 16:29:38.675009
77682	VNQ	2020-10-09	83.86	83.875	82.73	83.02	6556122	{}	\N	2020-10-12 16:29:38.679496
77683	VNQ	2020-10-12	83.12	83.42	82.55	83.175	1279466	{}	\N	2020-10-12 16:29:38.684429
77684	VONV	2020-09-28	103.26	104.33	103.26	103.92	315166	{}	\N	2020-10-12 16:29:40.073588
77685	VONV	2020-09-29	103.94	103.94	102.66	103.08	181353	{}	\N	2020-10-12 16:29:40.086835
77686	VONV	2020-09-30	103.26	104.7216	103.23	103.81	88869	{}	\N	2020-10-12 16:29:40.094443
77687	VONV	2020-10-01	104.3	104.64	103.4	104.09	90961	{}	\N	2020-10-12 16:29:40.100736
77688	VONV	2020-10-02	102.39	104.89	102.39	104.29	45906	{}	\N	2020-10-12 16:29:40.105929
77689	VONV	2020-10-05	105.01	106.1	105.01	106.01	700305	{}	\N	2020-10-12 16:29:40.111032
77690	VONV	2020-10-06	106.33	107.2677	104.91	104.95	131525	{}	\N	2020-10-12 16:29:40.115477
77691	VONV	2020-10-07	106.02	106.99	106.02	106.73	67586	{}	\N	2020-10-12 16:29:40.119884
77692	VONV	2020-10-08	107.34	108.07	107.29	108.07	41933	{}	\N	2020-10-12 16:29:40.124546
77693	VONV	2020-10-09	108.67	108.82	108.03	108.35	63466	{}	\N	2020-10-12 16:29:40.128893
77694	VONV	2020-10-12	108.67	109.17	108.5	109.11	26821	{}	\N	2020-10-12 16:29:40.133648
77695	VPV	2020-09-28	12.23	12.37	12.2	12.27	68385	{}	\N	2020-10-12 16:29:41.090008
77696	VPV	2020-09-29	12.24	12.2896	12.16	12.27	68982	{}	\N	2020-10-12 16:29:41.104385
77697	VPV	2020-09-30	12.35	12.35	12.23	12.26	50852	{}	\N	2020-10-12 16:29:41.112101
77698	VPV	2020-10-01	12.26	12.33	12.25	12.28	58566	{}	\N	2020-10-12 16:29:41.118959
77699	VPV	2020-10-02	12.21	12.31	12.21	12.3	44288	{}	\N	2020-10-12 16:29:41.124555
77700	VPV	2020-10-05	12.37	12.37	12.26	12.3	57052	{}	\N	2020-10-12 16:29:41.129723
77701	VPV	2020-10-06	12.28	12.29	12.19	12.19	104669	{}	\N	2020-10-12 16:29:41.134446
77702	VPV	2020-10-07	12.19	12.255	12.16	12.16	66881	{}	\N	2020-10-12 16:29:41.139079
77703	VPV	2020-10-08	12.16	12.2119	12.1	12.16	73100	{}	\N	2020-10-12 16:29:41.144408
77704	VPV	2020-10-09	12.17	12.25	12.17	12.2	79456	{}	\N	2020-10-12 16:29:41.149187
77705	VPV	2020-10-12	12.2	12.22	12.2	12.22	11094	{}	\N	2020-10-12 16:29:41.153997
77706	VRP	2020-09-28	24.41	24.66	24.38	24.46	208233	{}	\N	2020-10-12 16:29:43.41251
77707	VRP	2020-09-29	24.46	24.6	24.45	24.57	321616	{}	\N	2020-10-12 16:29:43.423016
77708	VRP	2020-09-30	24.65	24.689	24.5301	24.62	242731	{}	\N	2020-10-12 16:29:43.429945
77709	VRP	2020-10-01	24.61	24.7899	24.517	24.66	164924	{}	\N	2020-10-12 16:29:43.435868
77710	VRP	2020-10-02	24.54	24.73	24.54	24.68	141697	{}	\N	2020-10-12 16:29:43.440875
77711	VRP	2020-10-05	24.65	24.81	24.64	24.75	123194	{}	\N	2020-10-12 16:29:43.445804
77712	VRP	2020-10-06	24.8	24.85	24.6843	24.74	138498	{}	\N	2020-10-12 16:29:43.450846
77713	VRP	2020-10-07	24.72	24.88	24.72	24.86	114066	{}	\N	2020-10-12 16:29:43.454883
77714	VRP	2020-10-08	24.83	24.9	24.71	24.9	255382	{}	\N	2020-10-12 16:29:43.45928
77715	VRP	2020-10-09	24.85	24.8799	24.7001	24.83	121547	{}	\N	2020-10-12 16:29:43.464505
77716	VRP	2020-10-12	24.79	24.869	24.79	24.8423	63482	{}	\N	2020-10-12 16:29:43.468913
77717	VSEC	2020-09-28	30.07	31.75	30.07	31.56	33084	{}	\N	2020-10-12 16:29:44.809295
77718	VSEC	2020-09-29	31.5	31.74	30.8	31.19	27118	{}	\N	2020-10-12 16:29:44.821471
77719	VSEC	2020-09-30	31.645	31.68	30.33	30.64	12957	{}	\N	2020-10-12 16:29:44.829586
77720	VSEC	2020-10-01	30.79	31.3	30.19	30.75	62672	{}	\N	2020-10-12 16:29:44.836234
77721	VSEC	2020-10-02	30.05	31.9	29.6461	31.3	38510	{}	\N	2020-10-12 16:29:44.842167
77722	VSEC	2020-10-05	31.4	31.55	30.9127	31.23	15523	{}	\N	2020-10-12 16:29:44.847551
77723	VSEC	2020-10-06	31.5	31.99	30.5841	30.93	72471	{}	\N	2020-10-12 16:29:44.852592
77724	VSEC	2020-10-07	31.37	31.49	30.8	31.24	14342	{}	\N	2020-10-12 16:29:44.857696
77725	VSEC	2020-10-08	31.6	31.74	31.02	31.34	20625	{}	\N	2020-10-12 16:29:44.863086
77726	VSEC	2020-10-09	31.26	31.4199	30.65	30.65	5721	{}	\N	2020-10-12 16:29:44.868276
77727	VSEC	2020-10-12	30.55	31.1999	30.55	31.095	3220	{}	\N	2020-10-12 16:29:44.873291
77728	VIOG	2020-09-28	149.89	152.42	149.89	151.93	8293	{}	\N	2020-10-12 16:29:46.598537
77729	VIOG	2020-09-29	152.16	152.33	150.759	151.68	13505	{}	\N	2020-10-12 16:29:46.613894
77730	VIOG	2020-09-30	151.85	153.61	151.2815	152.04	18666	{}	\N	2020-10-12 16:29:46.621643
77731	VIOG	2020-10-01	152.89	154.09	152.42	154.09	7667	{}	\N	2020-10-12 16:29:46.628483
77732	VIOG	2020-10-02	150.98	155.5	150.98	155.33	13725	{}	\N	2020-10-12 16:29:46.63441
77733	VIOG	2020-10-05	157.52	158.255	156.7912	158.255	12038	{}	\N	2020-10-12 16:29:46.639615
77734	VIOG	2020-10-06	159.5	161.54	158.15	158.15	18194	{}	\N	2020-10-12 16:29:46.64465
77735	VIOG	2020-10-07	160.4	160.62	159.2	160.06	12999	{}	\N	2020-10-12 16:29:46.649391
77736	VIOG	2020-10-08	161.43	162.11	161.2104	161.91	10238	{}	\N	2020-10-12 16:29:46.654239
77737	VIOG	2020-10-09	163.18	163.865	162.56	163.51	15156	{}	\N	2020-10-12 16:29:46.659217
77738	VIOG	2020-10-12	164.29	164.82	163.61	164.4	10146	{}	\N	2020-10-12 16:29:46.664255
77739	VKI	2020-09-28	10.71	10.77	10.71	10.735	38313	{}	\N	2020-10-12 16:29:50.331882
77740	VKI	2020-09-29	10.75	10.78	10.69	10.75	81478	{}	\N	2020-10-12 16:29:50.343006
77741	VKI	2020-09-30	10.85	10.85	10.74	10.81	88113	{}	\N	2020-10-12 16:29:50.351348
77742	VKI	2020-10-01	10.81	10.87	10.81	10.855	47011	{}	\N	2020-10-12 16:29:50.358578
77743	VKI	2020-10-02	10.89	10.91	10.88	10.9	24198	{}	\N	2020-10-12 16:29:50.364863
77744	VKI	2020-10-05	10.87	10.92	10.81	10.85	29340	{}	\N	2020-10-12 16:29:50.370892
77745	VKI	2020-10-06	10.85	10.8777	10.8	10.8	53045	{}	\N	2020-10-12 16:29:50.37674
77746	VKI	2020-10-07	10.8	10.92	10.76	10.77	72860	{}	\N	2020-10-12 16:29:50.382241
77747	VKI	2020-10-08	10.87	10.87	10.78	10.78	67114	{}	\N	2020-10-12 16:29:50.387539
77748	VKI	2020-10-09	10.8	10.8898	10.8	10.84	43438	{}	\N	2020-10-12 16:29:50.392732
77749	VKI	2020-10-12	10.81	10.8811	10.81	10.84	26927	{}	\N	2020-10-12 16:29:50.397726
77750	VLY	2020-09-28	6.86	6.97	6.82	6.91	2068281	{}	\N	2020-10-12 16:29:51.688959
77751	VLY	2020-09-29	6.89	6.89	6.73	6.83	1124805	{}	\N	2020-10-12 16:29:51.700347
77752	VLY	2020-09-30	6.85	6.98	6.76	6.85	1737762	{}	\N	2020-10-12 16:29:51.675872
77753	VLY	2020-10-01	6.89	6.95	6.77	6.9	1996528	{}	\N	2020-10-12 16:29:51.686147
77754	VLY	2020-10-02	6.69	7.125	6.69	7.09	1498879	{}	\N	2020-10-12 16:29:51.692567
77755	VLY	2020-10-05	7.15	7.4	7.14	7.39	1673588	{}	\N	2020-10-12 16:29:51.699012
77756	VLY	2020-10-06	7.51	7.775	7.38	7.41	2657471	{}	\N	2020-10-12 16:29:51.705934
77757	VLY	2020-10-07	7.53	7.74	7.47	7.58	1728930	{}	\N	2020-10-12 16:29:51.711929
77758	VLY	2020-10-08	7.65	7.71	7.575	7.7	1249028	{}	\N	2020-10-12 16:29:51.717857
77759	VLY	2020-10-09	7.73	7.8	7.55	7.58	1225280	{}	\N	2020-10-12 16:29:51.7238
77760	VLY	2020-10-12	7.61	7.69	7.55	7.65	359891	{}	\N	2020-10-12 16:29:51.729556
77761	VOD	2020-09-28	13.44	13.57	13.43	13.48	4310534	{}	\N	2020-10-12 16:29:53.009707
77762	VOD	2020-09-29	13.37	13.41	13.2418	13.37	4503817	{}	\N	2020-10-12 16:29:53.026032
77763	VOD	2020-09-30	13.39	13.525	13.37	13.42	3873940	{}	\N	2020-10-12 16:29:53.034297
77764	VOD	2020-10-01	13.27	13.32	13.14	13.25	3484164	{}	\N	2020-10-12 16:29:53.041014
77765	VOD	2020-10-02	13.16	13.52	13.1518	13.46	2943194	{}	\N	2020-10-12 16:29:53.046721
77766	VOD	2020-10-05	14.04	14.2	13.95	14.19	6333225	{}	\N	2020-10-12 16:29:53.051894
77767	VOD	2020-10-06	14.49	14.51	14.2	14.23	4279239	{}	\N	2020-10-12 16:29:53.056511
77768	VOD	2020-10-07	14.3	14.36	14.12	14.22	2196963	{}	\N	2020-10-12 16:29:53.060925
77769	VOD	2020-10-08	14.51	14.63	14.44	14.57	3093673	{}	\N	2020-10-12 16:29:53.065638
77770	VOD	2020-10-09	14.6	14.68	14.44	14.63	2301662	{}	\N	2020-10-12 16:29:53.070105
77771	VOD	2020-10-12	14.73	14.7879	14.655	14.7739	1068527	{}	\N	2020-10-12 16:29:53.0748
77772	VOXX	2020-09-28	7.71	8.04	7.66	7.95	98927	{}	\N	2020-10-12 16:29:54.946561
77773	VOXX	2020-09-29	7.8	7.95	7.63	7.79	175773	{}	\N	2020-10-12 16:29:54.959506
77774	VOXX	2020-09-30	7.92	8.165	7.58	7.69	110816	{}	\N	2020-10-12 16:29:54.966713
77775	VOXX	2020-10-01	7.72	8.11	7.63	7.86	122463	{}	\N	2020-10-12 16:29:54.972505
77776	VOXX	2020-10-02	7.7	8.06	7.7	7.88	125289	{}	\N	2020-10-12 16:29:54.978484
77777	VOXX	2020-10-05	8.09	9.18	7.95	8.91	351034	{}	\N	2020-10-12 16:29:54.983563
77778	VOXX	2020-10-06	9.03	9.24	8.5001	8.71	202843	{}	\N	2020-10-12 16:29:54.988593
77779	VOXX	2020-10-07	8.78	9.67	8.78	9.01	394142	{}	\N	2020-10-12 16:29:54.993611
77780	VOXX	2020-10-08	9.24	9.3527	8.65	8.73	205345	{}	\N	2020-10-12 16:29:54.998689
77781	VOXX	2020-10-09	8.8	9.15	8.7	8.93	130707	{}	\N	2020-10-12 16:29:55.003942
77782	VOXX	2020-10-12	8.99	9.08	8.75	8.81	38448	{}	\N	2020-10-12 16:29:55.008757
77783	VRAY	2020-09-28	3.29	3.36	3.22	3.26	632715	{}	\N	2020-10-12 16:29:56.71698
77784	VRAY	2020-09-29	3.26	3.34	3.21	3.27	603913	{}	\N	2020-10-12 16:29:56.727668
77785	VRAY	2020-09-30	3.28	3.715	3.23	3.5	3155802	{}	\N	2020-10-12 16:29:56.734522
77786	VRAY	2020-10-01	3.47	3.52	3.35	3.47	840539	{}	\N	2020-10-12 16:29:56.74047
77787	VRAY	2020-10-02	3.37	3.455	3.34	3.41	553451	{}	\N	2020-10-12 16:29:56.746269
77788	VRAY	2020-10-05	3.48	3.49	3.185	3.25	1163083	{}	\N	2020-10-12 16:29:56.751721
77789	VRAY	2020-10-06	3.21	3.3484	3.1216	3.15	907373	{}	\N	2020-10-12 16:29:56.756989
77790	VRAY	2020-10-07	3.19	3.38	3.15	3.25	1024311	{}	\N	2020-10-12 16:29:56.762104
77791	VRAY	2020-10-08	3.23	3.26	3.16	3.16	531822	{}	\N	2020-10-12 16:29:56.767572
77792	VRAY	2020-10-09	3.17	3.26	3.06	3.16	600506	{}	\N	2020-10-12 16:29:56.772682
77793	VRAY	2020-10-12	3.17	3.21	3.05	3.07	142860	{}	\N	2020-10-12 16:29:56.778072
77794	VSLR	2020-09-28	37.45	39.56	36.64	39.45	3589811	{}	\N	2020-10-12 16:29:58.933139
77795	VSLR	2020-09-29	39.75	41.21	39.13	39.97	13488005	{}	\N	2020-10-12 16:29:58.940819
77796	VSLR	2020-09-30	39.34	42.99	38.7201	42.35	8717024	{}	\N	2020-10-12 16:29:58.946375
77797	VSLR	2020-10-01	43.55	45.25	42.75	43.88	11486645	{}	\N	2020-10-12 16:29:58.951674
77798	VSLR	2020-10-02	42.43	44.15	41.73	42.02	11894310	{}	\N	2020-10-12 16:29:58.955745
77799	VSLR	2020-10-05	42.98	43.91	41.91	43.12	12374122	{}	\N	2020-10-12 16:29:58.959657
77800	VSLR	2020-10-06	43	44.1053	41.21	43.2	6067955	{}	\N	2020-10-12 16:29:58.966508
77801	VSLR	2020-10-07	44.13	45.3	42.61	43.08	20330494	{}	\N	2020-10-12 16:29:58.970886
77802	VTA	2020-09-28	9.21	9.275	9.21	9.27	177718	{}	\N	2020-10-12 16:30:00.194492
77803	VTA	2020-09-29	9.25	9.25	9.16	9.22	505040	{}	\N	2020-10-12 16:30:00.20575
77804	VTA	2020-09-30	9.24	9.25	9.1725	9.21	573999	{}	\N	2020-10-12 16:30:00.212834
77805	VTA	2020-10-01	9.2	9.24	9.18	9.21	614566	{}	\N	2020-10-12 16:30:00.219174
77806	VTA	2020-10-02	9.19	9.22	9.15	9.2	835885	{}	\N	2020-10-12 16:30:00.22528
77807	VTA	2020-10-05	9.2	9.28	9.195	9.26	342272	{}	\N	2020-10-12 16:30:00.230515
77808	VTA	2020-10-06	9.26	9.315	9.25	9.27	361664	{}	\N	2020-10-12 16:30:00.23561
77809	VTA	2020-10-07	9.32	9.43	9.31	9.37	946493	{}	\N	2020-10-12 16:30:00.240776
77810	VTA	2020-10-08	9.38	9.435	9.38	9.43	658754	{}	\N	2020-10-12 16:30:00.245823
77811	VTA	2020-10-09	9.44	9.5	9.4	9.5	678129	{}	\N	2020-10-12 16:30:00.251083
77812	VTA	2020-10-12	9.5	9.545	9.465	9.545	124674	{}	\N	2020-10-12 16:30:00.256182
77813	VTNR	2020-09-28	0.4724	0.544	0.4724	0.519	264981	{}	\N	2020-10-12 16:30:01.747161
77814	VTNR	2020-09-29	0.5192	0.55	0.506101	0.5399	98744	{}	\N	2020-10-12 16:30:01.760599
77815	VTNR	2020-09-30	0.51	0.5389	0.48001	0.5	171577	{}	\N	2020-10-12 16:30:01.769031
77816	VTNR	2020-10-01	0.495	0.5197	0.4877	0.5	99417	{}	\N	2020-10-12 16:30:01.775589
77817	VTNR	2020-10-02	0.5185	0.5199	0.4801	0.5061	80563	{}	\N	2020-10-12 16:30:01.781518
77818	VTNR	2020-10-05	0.4922	0.52	0.49	0.4951	60367	{}	\N	2020-10-12 16:30:01.787033
77819	VTNR	2020-10-06	0.4902	0.5193	0.49	0.5	60776	{}	\N	2020-10-12 16:30:01.792493
77820	VTNR	2020-10-07	0.51	0.52	0.48	0.4887	148950	{}	\N	2020-10-12 16:30:01.797846
77821	VTNR	2020-10-08	0.49	0.749	0.49	0.568	4458982	{}	\N	2020-10-12 16:30:01.803208
77822	VTNR	2020-10-09	0.646	0.66	0.5061	0.537	3618276	{}	\N	2020-10-12 16:30:01.808429
77823	VTNR	2020-10-12	0.4936	0.4999	0.48	0.48	365967	{}	\N	2020-10-12 16:30:01.813555
77824	VUZI	2020-09-28	4.2	4.275	4.12	4.21	902253	{}	\N	2020-10-12 16:30:03.274802
77825	VUZI	2020-09-29	4.2589	4.53	4.21	4.44	1972667	{}	\N	2020-10-12 16:30:03.289834
77826	VUZI	2020-09-30	4.45	4.6878	4.4	4.56	2100627	{}	\N	2020-10-12 16:30:03.297476
77827	VUZI	2020-10-01	4.58	4.74	4.56	4.66	1015777	{}	\N	2020-10-12 16:30:03.303983
77828	VUZI	2020-10-02	4.32	4.61	4.26	4.44	1309622	{}	\N	2020-10-12 16:30:03.309844
77829	VUZI	2020-10-05	4.45	4.63	4.4	4.52	778435	{}	\N	2020-10-12 16:30:03.315772
77830	VUZI	2020-10-06	4.54	4.83	4.35	4.38	2162648	{}	\N	2020-10-12 16:30:03.320664
77831	VUZI	2020-10-07	4.47	4.55	4.401	4.46	804892	{}	\N	2020-10-12 16:30:03.325832
77832	VUZI	2020-10-08	4.53	4.57	4.38	4.42	735122	{}	\N	2020-10-12 16:30:03.330785
77833	VUZI	2020-10-09	4.5	4.6	4.43	4.56	1116714	{}	\N	2020-10-12 16:30:03.335823
77834	VUZI	2020-10-12	4.69	5.17	4.67	5.04	5072136	{}	\N	2020-10-12 16:30:03.340807
77835	YEXT	2020-09-28	15.68	15.73	15.28	15.41	622580	{}	\N	2020-10-12 16:30:06.380045
77836	YEXT	2020-09-29	15.48	15.74	15.32	15.41	1198448	{}	\N	2020-10-12 16:30:06.394877
77837	YEXT	2020-09-30	15.29	15.615	15.07	15.18	1674103	{}	\N	2020-10-12 16:30:06.402981
77838	YEXT	2020-10-01	15.26	15.51	15.1	15.26	1761130	{}	\N	2020-10-12 16:30:06.409566
77839	YEXT	2020-10-02	14.86	15.8	14.63	15.65	1804614	{}	\N	2020-10-12 16:30:06.414966
77840	YEXT	2020-10-05	15.76	15.76	15.22	15.37	1450929	{}	\N	2020-10-12 16:30:06.419699
77841	YEXT	2020-10-06	15.37	15.8273	15.27	15.46	1467648	{}	\N	2020-10-12 16:30:06.424587
77842	YEXT	2020-10-07	15.6	16.04	15.6	15.9	1666721	{}	\N	2020-10-12 16:30:06.429978
77843	YEXT	2020-10-08	16.1	16.2	15.82	16	1976542	{}	\N	2020-10-12 16:30:06.435023
77844	YEXT	2020-10-09	16.14	16.27	16.02	16.18	1166684	{}	\N	2020-10-12 16:30:06.440146
77845	YEXT	2020-10-12	16.35	16.76	16.2	16.45	375733	{}	\N	2020-10-12 16:30:06.445128
77846	VLGEA	2020-09-28	24.5	25.025	24.34	24.93	28147	{}	\N	2020-10-12 16:30:07.667852
77847	VLGEA	2020-09-29	25.04	25.04	24.61	24.91	16721	{}	\N	2020-10-12 16:30:07.678023
77848	VLGEA	2020-09-30	24.82	24.95	24.53	24.61	17514	{}	\N	2020-10-12 16:30:07.685443
77849	VLGEA	2020-10-01	24.75	24.75	24.15	24.37	19498	{}	\N	2020-10-12 16:30:07.690903
77850	VLGEA	2020-10-02	24.28	24.815	24.215	24.48	14650	{}	\N	2020-10-12 16:30:07.695945
77851	VLGEA	2020-10-05	24.52	25	24.52	24.95	28380	{}	\N	2020-10-12 16:30:07.70303
77852	VLGEA	2020-10-06	25	25.07	24.66	25	25738	{}	\N	2020-10-12 16:30:07.707498
77853	VLGEA	2020-10-07	24.97	25.07	24.32	24.32	46974	{}	\N	2020-10-12 16:30:07.711987
77854	VLGEA	2020-10-08	24.45	24.89	24.3401	24.67	37317	{}	\N	2020-10-12 16:30:07.716547
77855	VLGEA	2020-10-09	24.76	25.24	24.7	25.24	17129	{}	\N	2020-10-12 16:30:07.721195
77856	VLGEA	2020-10-12	25.16	25.64	25	25.4724	8452	{}	\N	2020-10-12 16:30:07.725638
77857	VLUE	2020-09-28	73.34	74.17	73.34	73.89	672996	{}	\N	2020-10-12 16:30:09.104555
77858	VLUE	2020-09-29	73.96	74.06	73.08	73.37	486776	{}	\N	2020-10-12 16:30:09.115816
77859	VLUE	2020-09-30	73.67	74.54	73.32	73.78	1260655	{}	\N	2020-10-12 16:30:09.12303
77860	VLUE	2020-10-01	74.07	74.38	73.58	74.04	478773	{}	\N	2020-10-12 16:30:09.129478
77861	VLUE	2020-10-02	72.75	74.64	72.75	74.19	668066	{}	\N	2020-10-12 16:30:09.135213
77862	VLUE	2020-10-05	74.6	75.64	74.6	75.51	411311	{}	\N	2020-10-12 16:30:09.140455
77863	VLUE	2020-10-06	75.75	76.745	74.8	74.91	780273	{}	\N	2020-10-12 16:30:09.14539
77864	VLUE	2020-10-07	75.77	76.57	75.77	76.38	1184662	{}	\N	2020-10-12 16:30:09.150314
77865	VLUE	2020-10-08	77	77.67	76.89	77.67	541758	{}	\N	2020-10-12 16:30:09.155379
77866	VLUE	2020-10-09	78.03	78.29	77.2	77.28	595593	{}	\N	2020-10-12 16:30:09.160414
77867	VLUE	2020-10-12	77.61	77.95	77.435	77.91	152587	{}	\N	2020-10-12 16:30:09.165803
77868	VNET	2020-09-28	21.69	22.01	21.205	22.01	1821369	{}	\N	2020-10-12 16:30:10.334608
77869	VNET	2020-09-29	21.71	22.78	21.38	22.77	862036	{}	\N	2020-10-12 16:30:10.346162
77870	VNET	2020-09-30	22.63	23.72	22.54	23.16	1171804	{}	\N	2020-10-12 16:30:10.353289
77871	VNET	2020-10-01	23.51	24.59	23.235	23.83	1571319	{}	\N	2020-10-12 16:30:10.360464
77872	VNET	2020-10-02	23.05	23.69	22.51	22.84	550157	{}	\N	2020-10-12 16:30:10.367583
77873	VNET	2020-10-05	22.95	23.04	22.32	22.37	935896	{}	\N	2020-10-12 16:30:10.372799
77874	VNET	2020-10-06	22.41	23.08	21.855	22.56	1027471	{}	\N	2020-10-12 16:30:10.378046
77875	VNET	2020-10-07	22.98	23.75	22.91	23.45	712338	{}	\N	2020-10-12 16:30:10.383124
77876	VNET	2020-10-08	23.36	23.67	22.22	22.6	670984	{}	\N	2020-10-12 16:30:10.387905
77877	VNET	2020-10-09	22.99	23.19	22.53	23.03	402192	{}	\N	2020-10-12 16:30:10.392456
77878	VNET	2020-10-12	23.41	23.82	23.18	23.51	244646	{}	\N	2020-10-12 16:30:10.397038
77879	VNTR	2020-09-28	1.85	2	1.85	1.91	154889	{}	\N	2020-10-12 16:30:11.773805
77880	VNTR	2020-09-29	1.91	1.91	1.83	1.87	157113	{}	\N	2020-10-12 16:30:11.784586
77881	VNTR	2020-09-30	1.87	1.945	1.87	1.93	380383	{}	\N	2020-10-12 16:30:11.791593
77882	VNTR	2020-10-01	1.94	2	1.89	1.99	211242	{}	\N	2020-10-12 16:30:11.797306
77883	VNTR	2020-10-02	1.95	2.11	1.92	2.1	223403	{}	\N	2020-10-12 16:30:11.802866
77884	VNTR	2020-10-05	2.1	2.17	1.98	2.06	176418	{}	\N	2020-10-12 16:30:11.808661
77885	VNTR	2020-10-06	2.05	2.1	1.98	1.99	112711	{}	\N	2020-10-12 16:30:11.813674
77886	VNTR	2020-10-07	2.01	2.09	2	2.08	229686	{}	\N	2020-10-12 16:30:11.818895
77887	VNTR	2020-10-08	2.09	2.2	2.04	2.08	340368	{}	\N	2020-10-12 16:30:11.822879
77888	VNTR	2020-10-09	2.08	2.1	2.05	2.07	84068	{}	\N	2020-10-12 16:30:11.827424
77889	VNTR	2020-10-12	2.06	2.128	2.05	2.05	71100	{}	\N	2020-10-12 16:30:11.831742
77890	VOX	2020-09-28	101.62	101.7899	100.931	101.59	118588	{}	\N	2020-10-12 16:30:12.829402
77891	VOX	2020-09-29	101.74	102.0999	101.1412	101.78	113630	{}	\N	2020-10-12 16:30:12.839575
77892	VOX	2020-09-30	101.85	102.91	101.32	101.87	111493	{}	\N	2020-10-12 16:30:12.84597
77893	VOX	2020-10-01	102.69	103.645	102.69	103.56	100206	{}	\N	2020-10-12 16:30:12.851666
77894	VOX	2020-10-02	101.56	102.95	101.45	101.81	179047	{}	\N	2020-10-12 16:30:12.857028
77895	VOX	2020-10-05	102.34	103.46	102.11	103.41	119586	{}	\N	2020-10-12 16:30:12.862308
77896	VOX	2020-10-06	103.21	103.6536	101.4	101.59	187082	{}	\N	2020-10-12 16:30:12.866626
77897	VOX	2020-10-07	102.34	102.65	101.51	102.43	95113	{}	\N	2020-10-12 16:30:12.871198
77898	VOX	2020-10-08	103	103.65	102.96	103.54	479672	{}	\N	2020-10-12 16:30:12.875602
77899	VOX	2020-10-09	104.21	104.35	103.87	104.2	130555	{}	\N	2020-10-12 16:30:12.880088
77900	VOX	2020-10-12	105.44	106.43	105.06	106.43	67933	{}	\N	2020-10-12 16:30:12.884522
77901	VRTV	2020-09-28	13.28	13.775	13.24	13.45	51632	{}	\N	2020-10-12 16:30:15.180214
77902	VRTV	2020-09-29	13.22	13.57	12.73	12.78	112047	{}	\N	2020-10-12 16:30:15.190383
77903	VRTV	2020-09-30	12.73	13.3	12.61	12.66	141771	{}	\N	2020-10-12 16:30:15.196774
77904	VRTV	2020-10-01	12.63	13.11	12.3	13.01	129487	{}	\N	2020-10-12 16:30:15.202104
77905	VRTV	2020-10-02	12.85	13.57	12.59	13.45	100837	{}	\N	2020-10-12 16:30:15.207173
77906	VRTV	2020-10-05	13.8	14.76	13.8	14.57	167877	{}	\N	2020-10-12 16:30:15.212076
77907	VRTV	2020-10-06	14.97	15.565	14.51	14.55	142341	{}	\N	2020-10-12 16:30:15.216456
77908	VRTV	2020-10-07	14.95	15.4	14.73	15.33	186221	{}	\N	2020-10-12 16:30:15.220758
77909	VRTV	2020-10-08	15.6	15.63	14.91	15.53	86289	{}	\N	2020-10-12 16:30:15.224781
77910	VRTV	2020-10-09	15.76	16.27	15.49	15.73	102966	{}	\N	2020-10-12 16:30:15.228735
77911	VRTV	2020-10-12	15.88	16.46	15.86	16.17	44376	{}	\N	2020-10-12 16:30:15.232767
77912	VSMV	2020-09-28	32.2546	32.28	32.1174	32.1174	202296	{}	\N	2020-10-12 16:30:16.620912
77913	VSMV	2020-09-29	31.9569	32.094	31.9004	32.094	1452	{}	\N	2020-10-12 16:30:16.630363
77914	VSMV	2020-09-30	32.2058	32.3318	32.0252	32.14	2924	{}	\N	2020-10-12 16:30:16.636326
77915	VSMV	2020-10-01	32.39	32.39	32.1	32.23	120011	{}	\N	2020-10-12 16:30:16.641892
77916	VSMV	2020-10-02	31.96	32.23	31.96	32.23	4409	{}	\N	2020-10-12 16:30:16.647161
77917	VSMV	2020-10-05	32.39	32.4963	32.39	32.4963	3903	{}	\N	2020-10-12 16:30:16.652309
77918	VSMV	2020-10-06	32.63	32.74	32.2149	32.2149	8583	{}	\N	2020-10-12 16:30:16.65772
77919	VSMV	2020-10-07	32.4787	32.61	32.4194	32.61	3271	{}	\N	2020-10-12 16:30:16.662396
77920	VSMV	2020-10-08	32.75	32.82	32.65	32.758	121686	{}	\N	2020-10-12 16:30:16.667049
77921	VSMV	2020-10-09	32.86	32.98	32.86	32.95	907	{}	\N	2020-10-12 16:30:16.671732
77922	VSMV	2020-10-12	33.155	33.27	33.155	33.27	1621	{}	\N	2020-10-12 16:30:16.676604
77923	VTIP	2020-09-28	50.96	51.03	50.958	51.02	1462151	{}	\N	2020-10-12 16:30:18.098812
77924	VTIP	2020-09-29	51.01	51.06	51.01	51.05	1122205	{}	\N	2020-10-12 16:30:18.109488
77925	VTIP	2020-09-30	51.07	51.08	51.02	51.02	783120	{}	\N	2020-10-12 16:30:18.116643
77926	VTIP	2020-10-01	50.75	50.77	50.72	50.74	1604861	{}	\N	2020-10-12 16:30:18.122795
77927	VTIP	2020-10-02	50.7	50.75	50.7	50.72	922471	{}	\N	2020-10-12 16:30:18.128617
77928	VTIP	2020-10-05	50.78	50.78	50.73	50.73	1313672	{}	\N	2020-10-12 16:30:18.133832
77929	VTIP	2020-10-06	50.77	50.79	50.73	50.76	1343677	{}	\N	2020-10-12 16:30:18.138714
77930	VTIP	2020-10-07	50.77	50.78	50.74	50.77	747812	{}	\N	2020-10-12 16:30:18.143634
77931	VTIP	2020-10-08	50.8193	50.85	50.79	50.85	846473	{}	\N	2020-10-12 16:30:18.148437
77932	VTIP	2020-10-09	50.84	50.86	50.82	50.85	987612	{}	\N	2020-10-12 16:30:18.15415
77933	VTIP	2020-10-12	50.84	50.85	50.82	50.8359	493117	{}	\N	2020-10-12 16:30:18.160434
77934	VTWV	2020-09-28	86.44	87.839	86.44	87.7	2406	{}	\N	2020-10-12 16:30:20.033218
77935	VTWV	2020-09-29	87.44	87.44	85.8	86.8695	17262	{}	\N	2020-10-12 16:30:20.040032
77936	VTWV	2020-09-30	86.94	88.437	86.69	87.18	11751	{}	\N	2020-10-12 16:30:20.046193
77937	VTWV	2020-10-01	88.149	88.5	87.0006	88.38	12072	{}	\N	2020-10-12 16:30:20.052986
77938	VTWV	2020-10-02	86.24	90.13	86.24	90.02	11347	{}	\N	2020-10-12 16:30:20.058482
77939	VTWV	2020-10-05	90.82	92.19	90.82	92.19	38147	{}	\N	2020-10-12 16:30:20.063617
77940	VTWV	2020-10-06	92.99	94.21	91.8	91.8	18063	{}	\N	2020-10-12 16:30:20.068413
77941	VTWV	2020-10-07	92.96	93.75	92.96	93.68	10377	{}	\N	2020-10-12 16:30:20.0739
77942	VTWV	2020-10-08	94.65	95.15	94.287	95.15	20616	{}	\N	2020-10-12 16:30:20.078858
77943	VTWV	2020-10-09	96.15	96.15	94.975	95.14	10814	{}	\N	2020-10-12 16:30:20.083869
77944	VTWV	2020-10-12	95.28	95.8498	95.16	95.575	3960	{}	\N	2020-10-12 16:30:20.089423
77945	VVV	2020-09-28	19.5	19.61	19.29	19.36	1690335	{}	\N	2020-10-12 16:30:21.115744
77946	VVV	2020-09-29	19.39	19.55	19.125	19.14	1852894	{}	\N	2020-10-12 16:30:21.127408
77947	VVV	2020-09-30	19.15	19.3	18.86	19.04	4471435	{}	\N	2020-10-12 16:30:21.134838
77948	VVV	2020-10-01	19.19	19.37	18.89	19.06	2904148	{}	\N	2020-10-12 16:30:21.141363
77949	VVV	2020-10-02	18.8	19.72	18.73	19.56	2245683	{}	\N	2020-10-12 16:30:21.146776
77950	VVV	2020-10-05	19.82	20	19.665	19.76	3519515	{}	\N	2020-10-12 16:30:21.1517
77951	VVV	2020-10-06	20	20.07	19.49	19.49	1594858	{}	\N	2020-10-12 16:30:21.155856
77952	VVV	2020-10-07	19.69	20.03	19.65	19.71	5446367	{}	\N	2020-10-12 16:30:21.160272
77953	VVV	2020-10-08	19.87	20.07	19.48	19.7	3131942	{}	\N	2020-10-12 16:30:21.164681
77954	VVV	2020-10-09	19.64	20.31	19.64	20.02	4697400	{}	\N	2020-10-12 16:30:21.169231
77955	VVV	2020-10-12	20.07	20.085	19.67	19.75	651717	{}	\N	2020-10-12 16:30:21.173622
77956	VLO	2020-09-28	44.82	45.57	44.63	44.82	3333267	{}	\N	2020-10-12 16:30:25.074826
77957	VLO	2020-09-29	44.91	44.99	43.105	43.41	8761185	{}	\N	2020-10-12 16:30:25.087649
77958	VLO	2020-09-30	43.57	44.4	42.93	43.32	9257821	{}	\N	2020-10-12 16:30:25.095056
77959	VLO	2020-10-01	41.62	42	39.72	40.28	21237139	{}	\N	2020-10-12 16:30:25.10183
77960	VLO	2020-10-02	39.24	41.24	38.87	40.73	13976210	{}	\N	2020-10-12 16:30:25.108009
77961	VLO	2020-10-05	41.45	43.12	41.25	43.07	10149910	{}	\N	2020-10-12 16:30:25.113677
77962	VLO	2020-10-06	43.87	44.32	42.09	42.33	8674175	{}	\N	2020-10-12 16:30:25.11913
77963	VLO	2020-10-07	42.5	42.86	41.86	42.35	8450273	{}	\N	2020-10-12 16:30:25.124315
77964	VLO	2020-10-08	42.64	44.38	42.26	44.36	6609061	{}	\N	2020-10-12 16:30:25.129999
77965	VLO	2020-10-09	44.76	44.885	43.15	43.64	11152392	{}	\N	2020-10-12 16:30:25.135294
77966	VLO	2020-10-12	43.26	43.49	42.59	42.71	3571142	{}	\N	2020-10-12 16:30:25.141004
77967	VMBS	2020-09-28	54.25	54.25	54.2	54.22	1220857	{}	\N	2020-10-12 16:30:26.187934
77968	VMBS	2020-09-29	54.23	54.26	54.22	54.25	850991	{}	\N	2020-10-12 16:30:26.20078
77969	VMBS	2020-09-30	54.25	54.28	54.22	54.27	1036661	{}	\N	2020-10-12 16:30:26.208421
77970	VMBS	2020-10-01	54.15	54.19	54.11	54.17	2033584	{}	\N	2020-10-12 16:30:26.214984
77971	VMBS	2020-10-02	54.27	54.27	54.2	54.25	765408	{}	\N	2020-10-12 16:30:26.22127
77972	VMBS	2020-10-05	54.23	54.23	54.17	54.17	988923	{}	\N	2020-10-12 16:30:26.226834
77973	VMBS	2020-10-06	54.2	54.245	54.18	54.24	1392846	{}	\N	2020-10-12 16:30:26.231804
77974	VMBS	2020-10-07	54.21	54.23	54.18	54.23	930302	{}	\N	2020-10-12 16:30:26.23718
77975	VMBS	2020-10-08	54.24	54.24	54.17	54.17	1469802	{}	\N	2020-10-12 16:30:26.243072
77976	VMBS	2020-10-09	54.05	54.1	54.03	54.08	1848150	{}	\N	2020-10-12 16:30:26.24825
77977	VMBS	2020-10-12	54.07	54.14	54.06	54.1399	489720	{}	\N	2020-10-12 16:30:26.253774
77978	VMW	2020-09-28	143	144.19	142.49	142.99	952764	{}	\N	2020-10-12 16:30:27.305648
77979	VMW	2020-09-29	143.63	148.45	143.366	145.74	3931088	{}	\N	2020-10-12 16:30:27.320376
77980	VMW	2020-09-30	144.59	146.8	142.76	143.67	1713479	{}	\N	2020-10-12 16:30:27.328577
77981	VMW	2020-10-01	145	146.4182	144.2	144.7	1204905	{}	\N	2020-10-12 16:30:27.335338
77982	VMW	2020-10-02	141.16	145.58	141	142.95	1613006	{}	\N	2020-10-12 16:30:27.341138
77983	VMW	2020-10-05	144.01	146	143.66	145.06	1478607	{}	\N	2020-10-12 16:30:27.346619
77984	VMW	2020-10-06	145	148.6	144.8	147	2674206	{}	\N	2020-10-12 16:30:27.351263
77985	VMW	2020-10-07	148.29	149.12	146.95	147.75	1894116	{}	\N	2020-10-12 16:30:27.356056
77986	VMW	2020-10-08	148.95	151.3	148.01	150.25	2117872	{}	\N	2020-10-12 16:30:27.360753
77987	VMW	2020-10-09	151.5	152.98	150.34	151.48	1701340	{}	\N	2020-10-12 16:30:27.365606
77988	VMW	2020-10-12	151.65	152.97	151.44	152.175	607087	{}	\N	2020-10-12 16:30:27.370219
77989	VOOG	2020-09-28	207.39	208.02	206.375	207.96	103271	{}	\N	2020-10-12 16:30:29.753631
77990	VOOG	2020-09-29	207.76	208.4599	206.9	207.2	86309	{}	\N	2020-10-12 16:30:29.76289
77991	VOOG	2020-09-30	207.44	210.68	207.44	208.43	116573	{}	\N	2020-10-12 16:30:29.770103
77992	VOOG	2020-10-01	210.75	211.25	209.41	210.47	142005	{}	\N	2020-10-12 16:30:29.776252
77993	VOOG	2020-10-02	206.51	209.06	206.18	206.92	241906	{}	\N	2020-10-12 16:30:29.782081
77994	VOOG	2020-10-05	208.43	211.04	208.43	211.04	71940	{}	\N	2020-10-12 16:30:29.786919
77995	VOOG	2020-10-06	210.74	211.92	207.022	207.48	92587	{}	\N	2020-10-12 16:30:29.791588
77996	VOOG	2020-10-07	209.55	211.5488	209.55	211.18	54880	{}	\N	2020-10-12 16:30:29.796168
77997	VOOG	2020-10-08	212.56	212.56	211.65	212.51	71203	{}	\N	2020-10-12 16:30:29.800922
77998	VOOG	2020-10-09	213.55	215.38	213.55	215.38	113861	{}	\N	2020-10-12 16:30:29.806599
77999	VOOG	2020-10-12	217.81	220.4	217.3592	220.4	76189	{}	\N	2020-10-12 16:30:29.810996
78000	VPU	2020-09-28	128.27	129.15	127.73	127.92	179428	{}	\N	2020-10-12 16:30:31.003539
78001	VPU	2020-09-29	128.28	129.11	127.3017	127.83	129019	{}	\N	2020-10-12 16:30:31.021996
78002	VPU	2020-09-30	128.86	129.669	128.08	129.1	176905	{}	\N	2020-10-12 16:30:31.028031
78003	VPU	2020-10-01	129.44	130.57	129.18	130.28	135544	{}	\N	2020-10-12 16:30:31.033826
78004	VPU	2020-10-02	129.13	132.45	128.8	131.85	191308	{}	\N	2020-10-12 16:30:31.038902
78005	VPU	2020-10-05	132.13	133.71	131.425	133.379	168326	{}	\N	2020-10-12 16:30:31.043805
78006	VPU	2020-10-06	133.53	136.1099	133.04	134.5	174271	{}	\N	2020-10-12 16:30:31.048748
78007	VPU	2020-10-07	135.06	135.83	134.405	135.4	197840	{}	\N	2020-10-12 16:30:31.053747
78008	VPU	2020-10-08	136	138	136	137.82	318380	{}	\N	2020-10-12 16:30:31.058803
78009	VPU	2020-10-09	138.54	138.54	136.89	137.79	160432	{}	\N	2020-10-12 16:30:31.06356
78010	VPU	2020-10-12	137.65	139.13	137.491	139.0293	273975	{}	\N	2020-10-12 16:30:31.068436
78011	VRS	2020-09-28	8.07	8.21	7.88	8.13	454251	{}	\N	2020-10-12 16:30:32.002249
78012	VRS	2020-09-29	8.23	8.51	7.81	8.03	1010067	{}	\N	2020-10-12 16:30:32.015169
78013	VRS	2020-09-30	8	8.12	7.825	7.89	779781	{}	\N	2020-10-12 16:30:32.023196
78014	VRS	2020-10-01	7.8	7.835	7.44	7.76	1148032	{}	\N	2020-10-12 16:30:32.030246
78015	VRS	2020-10-02	7.56	7.875	7.53	7.79	704188	{}	\N	2020-10-12 16:30:32.03673
78016	VRS	2020-10-05	8	8.25	7.93	8.09	773150	{}	\N	2020-10-12 16:30:32.042336
78017	VRS	2020-10-06	8.2	8.25	7.93	7.98	651319	{}	\N	2020-10-12 16:30:32.048001
78018	VRS	2020-10-07	8.08	8.152	7.93	8.11	575595	{}	\N	2020-10-12 16:30:32.053609
78019	VRS	2020-10-08	8.11	8.22	8.04	8.19	451614	{}	\N	2020-10-12 16:30:32.059415
78020	VRS	2020-10-09	8.3	8.39	8.1	8.15	565832	{}	\N	2020-10-12 16:30:32.065061
78021	VRS	2020-10-12	8.22	8.26	7.92	8.05	264657	{}	\N	2020-10-12 16:30:32.07076
78022	VSH	2020-09-28	15.35	15.9	15.32	15.89	864921	{}	\N	2020-10-12 16:30:33.087496
78023	VSH	2020-09-29	15.81	16	15.66	15.79	1151604	{}	\N	2020-10-12 16:30:33.097811
78024	VSH	2020-09-30	15.93	16.06	15.42	15.57	2547256	{}	\N	2020-10-12 16:30:33.10486
78025	VSH	2020-10-01	15.8	16.03	15.67	15.82	1574551	{}	\N	2020-10-12 16:30:33.110939
78026	VSH	2020-10-02	15.46	15.88	15.46	15.63	1614353	{}	\N	2020-10-12 16:30:33.116947
78027	VSH	2020-10-05	15.74	16.44	15.74	16.41	2220958	{}	\N	2020-10-12 16:30:33.122243
78028	VSH	2020-10-06	16.51	16.68	16.13	16.19	1600396	{}	\N	2020-10-12 16:30:33.127386
78029	VSH	2020-10-07	17	17.34	16.85	17.27	4114303	{}	\N	2020-10-12 16:30:33.132447
78030	VSH	2020-10-08	17.48	17.74	17.24	17.74	2244555	{}	\N	2020-10-12 16:30:33.137754
78031	VSH	2020-10-09	18.01	18.09	17.64	17.7	2067998	{}	\N	2020-10-12 16:30:33.142844
78032	VSH	2020-10-12	17.8	18.125	17.8	18.005	855650	{}	\N	2020-10-12 16:30:33.148106
78033	VTEB	2020-09-28	54.44	54.4925	54.44	54.46	740827	{}	\N	2020-10-12 16:30:34.278237
78034	VTEB	2020-09-29	54.46	54.51	54.46	54.48	1468483	{}	\N	2020-10-12 16:30:34.292362
78035	VTEB	2020-09-30	54.48	54.48	54.4	54.44	869715	{}	\N	2020-10-12 16:30:34.300025
78036	VTEB	2020-10-01	54.33	54.337	54.29	54.33	2172997	{}	\N	2020-10-12 16:30:34.306583
78037	VTEB	2020-10-02	54.3	54.32	54.27	54.27	729563	{}	\N	2020-10-12 16:30:34.312333
78038	VTEB	2020-10-05	54.27	54.28	54.23	54.25	808069	{}	\N	2020-10-12 16:30:34.317813
78039	VTEB	2020-10-06	54.25	54.25	54.19	54.21	1300688	{}	\N	2020-10-12 16:30:34.322849
78040	VTEB	2020-10-07	54.19	54.19	54.1	54.1	877224	{}	\N	2020-10-12 16:30:34.327865
78041	VTEB	2020-10-08	54.1	54.14	54.09	54.11	1038440	{}	\N	2020-10-12 16:30:34.333143
78042	VTEB	2020-10-09	54.12	54.16	54.1	54.16	1100582	{}	\N	2020-10-12 16:30:34.338367
78043	VTEB	2020-10-12	54.15	54.15	54.12	54.13	295363	{}	\N	2020-10-12 16:30:34.343641
78044	VTWG	2020-09-28	159.15	161.11	159.15	160.69	15197	{}	\N	2020-10-12 16:30:36.148919
78045	VTWG	2020-09-29	160.43	161.654	160.125	161.02	4109	{}	\N	2020-10-12 16:30:36.162951
78046	VTWG	2020-09-30	162.22	163.2206	160.2525	161.32	6799	{}	\N	2020-10-12 16:30:36.170084
78047	VTWG	2020-10-01	162.15	163.49	161.78	163.49	4561	{}	\N	2020-10-12 16:30:36.176435
78048	VTWG	2020-10-02	160.16	163.599	160.16	163.599	7064	{}	\N	2020-10-12 16:30:36.182906
78049	VTWG	2020-10-05	165.19	167.815	165.19	167.815	10799	{}	\N	2020-10-12 16:30:36.18863
78050	VTWG	2020-10-06	169.09	170.94	167.66	167.88	61036	{}	\N	2020-10-12 16:30:36.193786
78051	VTWG	2020-10-07	169.93	171.76	169.9	171.475	16277	{}	\N	2020-10-12 16:30:36.199056
78052	VTWG	2020-10-08	173.4	173.4	171.9414	172.6	5308	{}	\N	2020-10-12 16:30:36.204279
78053	VTWG	2020-10-09	174.22	174.5225	173.3655	174.13	8013	{}	\N	2020-10-12 16:30:36.209647
78054	VTWG	2020-10-12	175.83	175.83	174.1745	175.1	6099	{}	\N	2020-10-12 16:30:36.214999
78055	VVR	2020-09-28	3.63	3.65	3.62	3.62	1219683	{}	\N	2020-10-12 16:30:37.108676
78056	VVR	2020-09-29	3.63	3.64	3.61	3.63	2792148	{}	\N	2020-10-12 16:30:37.124618
78057	VVR	2020-09-30	3.65	3.67	3.63	3.66	6866948	{}	\N	2020-10-12 16:30:37.131657
78058	VVR	2020-10-01	3.66	3.68	3.63	3.63	995023	{}	\N	2020-10-12 16:30:37.138456
78059	VVR	2020-10-02	3.62	3.68	3.62	3.67	590938	{}	\N	2020-10-12 16:30:37.146533
78060	VVR	2020-10-05	3.67	3.7	3.66	3.685	769856	{}	\N	2020-10-12 16:30:37.151283
78061	VVR	2020-10-06	3.68	3.69	3.64	3.64	1659129	{}	\N	2020-10-12 16:30:37.156136
78062	VVR	2020-10-07	3.65	3.6699	3.615	3.63	2978364	{}	\N	2020-10-12 16:30:37.160955
78063	VVR	2020-10-08	3.64	3.66	3.64	3.66	542922	{}	\N	2020-10-12 16:30:37.165556
78064	VVR	2020-10-09	3.66	3.7	3.65	3.69	1178234	{}	\N	2020-10-12 16:30:37.170054
78065	VVR	2020-10-12	3.68	3.7	3.68	3.695	111418	{}	\N	2020-10-12 16:30:37.174834
78066	VXX	2020-09-28	25.33	25.65	25.26	25.36	18242603	{}	\N	2020-10-12 16:30:38.132575
78067	VXX	2020-09-29	25.47	25.5	24.555	24.94	22680161	{}	\N	2020-10-12 16:30:38.145023
78068	VXX	2020-09-30	24.61	25.18	24.29	24.9	25498149	{}	\N	2020-10-12 16:30:38.152471
78069	VXX	2020-10-01	24.61	25.4	24.61	25.01	23858946	{}	\N	2020-10-12 16:30:38.158335
78070	VXX	2020-10-02	26.31	26.38	25.29	25.78	29692208	{}	\N	2020-10-12 16:30:38.16355
78071	VXX	2020-10-05	25.53	25.8	24.775	24.97	20566495	{}	\N	2020-10-12 16:30:38.16829
78072	VXX	2020-10-06	24.76	25.54	24.2923	25.32	34514263	{}	\N	2020-10-12 16:30:38.172792
78073	VXX	2020-10-07	24.83	24.93	24.31	24.49	19220612	{}	\N	2020-10-12 16:30:38.178077
78074	VXX	2020-10-08	24.21	24.35	23.46	23.51	24854196	{}	\N	2020-10-12 16:30:38.182298
78075	VXX	2020-10-09	22.81	22.82	22.28	22.29	28818464	{}	\N	2020-10-12 16:30:38.186534
78076	VXX	2020-10-12	21.9	22.29	21.82	22.11	13643863	{}	\N	2020-10-12 16:30:38.190611
78077	VMM	2020-09-28	13.31	13.34	13.1	13.31	8391	{}	\N	2020-10-12 16:30:40.743869
78078	VMM	2020-09-29	13.18	13.195	12.94	12.995	26060	{}	\N	2020-10-12 16:30:40.756603
78079	VMM	2020-09-30	13.11	13.11	12.9722	13	4503	{}	\N	2020-10-12 16:30:40.762775
78080	VMM	2020-10-01	12.94	13.2773	12.9	13.2773	8604	{}	\N	2020-10-12 16:30:40.768678
78081	VMM	2020-10-02	13.15	13.25	13	13.24	27224	{}	\N	2020-10-12 16:30:40.774047
78082	VMM	2020-10-05	13.1691	13.1691	13.07	13.11	14246	{}	\N	2020-10-12 16:30:40.779298
78083	VMM	2020-10-06	13.11	13.11	12.95	12.95	2134	{}	\N	2020-10-12 16:30:40.784057
78084	VMM	2020-10-07	12.97	13.06	12.97	13.01	10127	{}	\N	2020-10-12 16:30:40.789431
78085	VMM	2020-10-08	13.07	13.07	13.03	13.0602	1345	{}	\N	2020-10-12 16:30:40.79432
78086	VMM	2020-10-09	13.019	13.019	12.98	12.98	2051	{}	\N	2020-10-12 16:30:40.799325
78087	VONE	2020-09-28	153.61	154.633	153.61	154.5	32030	{}	\N	2020-10-12 16:30:42.093897
78088	VONE	2020-09-29	154.34	154.65	153.42	153.74	81945	{}	\N	2020-10-12 16:30:42.1056
78089	VONE	2020-09-30	153.91	156.27	153.91	154.8	39725	{}	\N	2020-10-12 16:30:42.113706
78090	VONE	2020-10-01	156.485	156.485	155.15	156.05	26221	{}	\N	2020-10-12 16:30:42.12005
78091	VONE	2020-10-02	153.24	155.425	153	154.74	26127	{}	\N	2020-10-12 16:30:42.125726
78092	VONE	2020-10-05	155.79	157.5	155.79	157.44	41283	{}	\N	2020-10-12 16:30:42.130626
78093	VONE	2020-10-06	157.6	158.58	155.161	155.36	25114	{}	\N	2020-10-12 16:30:42.135192
78094	VONE	2020-10-07	157.02	158.42	157.02	158.115	23319	{}	\N	2020-10-12 16:30:42.139687
78095	VONE	2020-10-08	159.13	159.44	158.64	159.44	35536	{}	\N	2020-10-12 16:30:42.143668
78096	VONE	2020-10-09	160.39	161.07	160.0697	160.91	63084	{}	\N	2020-10-12 16:30:42.147623
78097	VONE	2020-10-12	162.31	163.47	161.9	163.47	11123	{}	\N	2020-10-12 16:30:42.151888
78098	VRIG	2020-09-28	24.84	24.85	24.82	24.84	67355	{}	\N	2020-10-12 16:30:43.913893
78099	VRIG	2020-09-29	24.845	24.8499	24.82	24.84	82122	{}	\N	2020-10-12 16:30:43.925535
78100	VRIG	2020-09-30	24.86	24.86	24.83	24.85	64728	{}	\N	2020-10-12 16:30:43.932912
78101	VRIG	2020-10-01	24.84	24.87	24.83	24.87	135545	{}	\N	2020-10-12 16:30:43.939901
78102	VRIG	2020-10-02	24.82	24.87	24.82	24.86	60193	{}	\N	2020-10-12 16:30:43.946507
78103	VRIG	2020-10-05	24.85	24.8799	24.84	24.87	85044	{}	\N	2020-10-12 16:30:43.95195
78104	VRIG	2020-10-06	24.85	24.87	24.84	24.855	35590	{}	\N	2020-10-12 16:30:43.95742
78105	VRIG	2020-10-07	24.86	24.87	24.84	24.86	75159	{}	\N	2020-10-12 16:30:43.962853
78106	VRIG	2020-10-08	24.84	24.87	24.84	24.87	49719	{}	\N	2020-10-12 16:30:43.968192
78107	VRIG	2020-10-09	24.86	24.89	24.85	24.87	66949	{}	\N	2020-10-12 16:30:43.973624
78108	VRIG	2020-10-12	24.88	24.8847	24.87	24.88	38364	{}	\N	2020-10-12 16:30:43.978915
78109	VRTU	2020-09-28	48.9	49.47	48.6	48.62	431598	{}	\N	2020-10-12 16:30:45.123814
78110	VRTU	2020-09-29	48.7	48.91	48.5	48.55	267274	{}	\N	2020-10-12 16:30:45.134254
78111	VRTU	2020-09-30	48.5	49.35	48.5	49.16	804194	{}	\N	2020-10-12 16:30:45.141568
78112	VRTU	2020-10-01	50.04	50.04	49.1	49.36	315792	{}	\N	2020-10-12 16:30:45.147801
78113	VRTU	2020-10-02	48.82	49.82	48.46	49.25	345959	{}	\N	2020-10-12 16:30:45.153675
78114	VRTU	2020-10-05	49.57	49.78	49.25	49.46	487775	{}	\N	2020-10-12 16:30:45.159094
78115	VRTU	2020-10-06	49.45	49.97	49.41	49.77	385873	{}	\N	2020-10-12 16:30:45.163941
78116	VRTU	2020-10-07	50.37	50.84	50.05	50.26	1591888	{}	\N	2020-10-12 16:30:45.16893
78117	VRTU	2020-10-08	50.48	50.48	50.095	50.17	331890	{}	\N	2020-10-12 16:30:45.173494
78118	VRTU	2020-10-09	50.37	50.51	50.22	50.32	457709	{}	\N	2020-10-12 16:30:45.178174
78119	VRTU	2020-10-12	50.43	50.43	50.155	50.26	132670	{}	\N	2020-10-12 16:30:45.183216
78120	VT	2020-09-28	80.39	80.64	80.17	80.55	869233	{}	\N	2020-10-12 16:30:46.42223
78121	VT	2020-09-29	80.47	80.66	80.06	80.26	660943	{}	\N	2020-10-12 16:30:46.43623
78122	VT	2020-09-30	80.31	81.26	80.28	80.64	2858150	{}	\N	2020-10-12 16:30:46.443816
78123	VT	2020-10-01	81.26	81.41	80.83	81.29	1006684	{}	\N	2020-10-12 16:30:46.449642
78124	VT	2020-10-02	80.08	81.14	80.04	80.77	1512432	{}	\N	2020-10-12 16:30:46.454537
78125	VT	2020-10-05	81.31	82.14	81.31	82.1	677791	{}	\N	2020-10-12 16:30:46.459717
78126	VT	2020-10-06	82.24	82.64	81.13	81.26	1635685	{}	\N	2020-10-12 16:30:46.463975
78127	VT	2020-10-07	82.02	82.57	81.99	82.41	1343476	{}	\N	2020-10-12 16:30:46.468237
78128	VT	2020-10-08	82.87	83.1	82.68	83.09	804612	{}	\N	2020-10-12 16:30:46.473024
78129	VT	2020-10-09	83.54	83.81	83.365	83.72	1621571	{}	\N	2020-10-12 16:30:46.477341
78130	VT	2020-10-12	84.18	84.73	84.095	84.71	597705	{}	\N	2020-10-12 16:30:46.481644
78131	VTR	2020-09-28	42.74	43.11	42.4	42.88	2972024	{}	\N	2020-10-12 16:30:47.520286
78132	VTR	2020-09-29	42.48	42.72	41.44	42.58	3026183	{}	\N	2020-10-12 16:30:47.533838
78133	VTR	2020-09-30	42.58	43.425	41.48	41.96	4808406	{}	\N	2020-10-12 16:30:47.542267
78134	VTR	2020-10-01	42.13	43.55	41.89	43.49	4368705	{}	\N	2020-10-12 16:30:47.549434
78135	VTR	2020-10-02	42.88	45.01	42.63	44.61	5739607	{}	\N	2020-10-12 16:30:47.555988
78136	VTR	2020-10-05	44.91	45.43	43.46	44.11	3720992	{}	\N	2020-10-12 16:30:47.561916
78137	VTR	2020-10-06	44.42	45.11	43.64	44.33	4959323	{}	\N	2020-10-12 16:30:47.567531
78138	VTR	2020-10-07	44.62	44.71	43.52	43.98	4600628	{}	\N	2020-10-12 16:30:47.573009
78139	VTR	2020-10-08	44.29	45.06	43.99	45.05	3771990	{}	\N	2020-10-12 16:30:47.578473
78140	VTR	2020-10-09	45.44	45.45	44.16	44.67	4352773	{}	\N	2020-10-12 16:30:47.583878
78141	VTR	2020-10-12	44.67	44.7	43.85	44.215	550827	{}	\N	2020-10-12 16:30:47.589124
78142	VXUS	2020-09-28	52.18	52.2734	52.0535	52.19	2108575	{}	\N	2020-10-12 16:30:50.328142
78143	VXUS	2020-09-29	52.11	52.295	51.95	52.12	2469994	{}	\N	2020-10-12 16:30:50.340554
78144	VXUS	2020-09-30	52.13	52.495	52.04	52.17	3019341	{}	\N	2020-10-12 16:30:50.347996
78145	VXUS	2020-10-01	52.54	52.6	52.265	52.59	2441322	{}	\N	2020-10-12 16:30:50.353808
78146	VXUS	2020-10-02	51.95	52.53	51.89	52.36	2329786	{}	\N	2020-10-12 16:30:50.358778
78147	VXUS	2020-10-05	52.75	53.13	52.7	53.13	1695987	{}	\N	2020-10-12 16:30:50.363893
78148	VXUS	2020-10-06	53.22	53.34	52.62	52.75	2860889	{}	\N	2020-10-12 16:30:50.369005
78149	VXUS	2020-10-07	53.2	53.35	53.07	53.28	2287404	{}	\N	2020-10-12 16:30:50.374257
78150	VXUS	2020-10-08	53.51	53.65	53.42	53.65	2072362	{}	\N	2020-10-12 16:30:50.379451
78151	VXUS	2020-10-09	53.9	54.08	53.79	54.02	1750144	{}	\N	2020-10-12 16:30:50.384238
78152	VXUS	2020-10-12	54.2	54.38	54.1201	54.37	913454	{}	\N	2020-10-12 16:30:50.389091
78153	WAB	2020-09-28	61.89	63.54	61.805	62.96	857556	{}	\N	2020-10-12 16:30:53.923592
78154	WAB	2020-09-29	62.72	63.115	61.795	62.47	1122862	{}	\N	2020-10-12 16:30:53.937836
78155	WAB	2020-09-30	62.9	63.765	61.5	61.88	2189758	{}	\N	2020-10-12 16:30:53.94659
78156	WAB	2020-10-01	62.04	62.55	60.22	60.73	3158799	{}	\N	2020-10-12 16:30:53.953405
78157	WAB	2020-10-02	59.33	63.38	59.33	62.72	1586459	{}	\N	2020-10-12 16:30:53.959618
78158	WAB	2020-10-05	63.43	64.57	63.34	64.43	1352145	{}	\N	2020-10-12 16:30:53.965096
78159	WAB	2020-10-06	65	66.05	63.63	63.73	1715991	{}	\N	2020-10-12 16:30:53.970544
78160	WAB	2020-10-07	64.31	65.27	63.9	64.62	1486413	{}	\N	2020-10-12 16:30:53.976032
78161	WAB	2020-10-08	65.06	65.37	63.74	64.84	1496538	{}	\N	2020-10-12 16:30:53.981112
78162	WAB	2020-10-09	65.58	65.77	63.87	64.51	1921456	{}	\N	2020-10-12 16:30:53.986436
78163	WAB	2020-10-12	64.78	64.98	63.11	63.94	1231201	{}	\N	2020-10-12 16:30:53.992077
78164	WB	2020-09-28	34.33	37.4	34.2	34.96	2534069	{}	\N	2020-10-12 16:30:55.439706
78165	WB	2020-09-29	34.96	35.1	33.03	33.65	1456840	{}	\N	2020-10-12 16:30:55.448354
78166	WB	2020-09-30	33.84	37.15	33.78	36.43	4173599	{}	\N	2020-10-12 16:30:55.455352
78167	WB	2020-10-01	36.37	37.76	35.92	37	1417251	{}	\N	2020-10-12 16:30:55.463063
78168	WB	2020-10-02	36.26	39.05	36.0007	38.55	1940286	{}	\N	2020-10-12 16:30:55.469859
78169	WB	2020-10-05	38.82	41.9537	38.56	40.3	3647492	{}	\N	2020-10-12 16:30:55.475962
78170	WB	2020-10-06	40.35	41.51	40.1	40.79	1354819	{}	\N	2020-10-12 16:30:55.481907
78171	WB	2020-10-07	40.71	41.67	40.71	41.51	1144660	{}	\N	2020-10-12 16:30:55.487706
78172	WB	2020-10-08	41.73	42.45	41.18	41.4	595963	{}	\N	2020-10-12 16:30:55.492462
78173	WB	2020-10-09	41.15	41.8	41.13	41.23	529158	{}	\N	2020-10-12 16:30:55.49721
78174	WB	2020-10-12	41.8766	41.99	40.92	41	448199	{}	\N	2020-10-12 16:30:55.502097
78175	YGYI	2020-09-28	0.598	0.61	0.4589	0.488	687247	{}	\N	2020-10-12 16:30:57.082932
78176	YGYI	2020-09-29	0.521	0.521	0.45	0.4702	223724	{}	\N	2020-10-12 16:30:57.093036
78177	YGYI	2020-09-30	0.475	0.733	0.4702	0.6003	848461	{}	\N	2020-10-12 16:30:57.099173
78178	YGYI	2020-10-01	0.61	0.65	0.56	0.5852	390077	{}	\N	2020-10-12 16:30:57.104381
78179	YGYI	2020-10-02	0.5362	0.6	0.51	0.597	193263	{}	\N	2020-10-12 16:30:57.109493
78180	YGYI	2020-10-05	0.525	0.59	0.5051	0.5451	221368	{}	\N	2020-10-12 16:30:57.114161
78181	YGYI	2020-10-06	0.5561	0.58	0.53	0.5401	58419	{}	\N	2020-10-12 16:30:57.118167
78182	YGYI	2020-10-07	0.5238	0.5639	0.51	0.53	45585	{}	\N	2020-10-12 16:30:57.122119
78183	YGYI	2020-10-08	0.5393	0.57	0.5105	0.54	58265	{}	\N	2020-10-12 16:30:57.126082
78184	YGYI	2020-10-09	0.55	0.65	0.55	0.5899	203441	{}	\N	2020-10-12 16:30:57.130169
78185	YGYI	2020-10-12	0.6	0.625	0.56	0.59	79159	{}	\N	2020-10-12 16:30:57.134103
78186	VNCE	2020-09-28	5.67	5.67	5.5	5.6	5588	{}	\N	2020-10-12 16:30:58.197575
78187	VNCE	2020-09-29	5.54	5.55	5.3	5.46	27534	{}	\N	2020-10-12 16:30:58.208525
78188	VNCE	2020-09-30	5.43	5.5666	5.3	5.41	13908	{}	\N	2020-10-12 16:30:58.215224
78189	VNCE	2020-10-01	5.34	6.19	5.225	5.95	39771	{}	\N	2020-10-12 16:30:58.221118
78190	VNCE	2020-10-02	5.84	5.99	5.35	5.58	27674	{}	\N	2020-10-12 16:30:58.226514
78191	VNCE	2020-10-05	5.6	5.82	5.6	5.66	6716	{}	\N	2020-10-12 16:30:58.231165
78192	VNCE	2020-10-06	5.87	5.87	5.58	5.62	12070	{}	\N	2020-10-12 16:30:58.235493
78193	VNCE	2020-10-07	5.735	5.735	5.26	5.46	35058	{}	\N	2020-10-12 16:30:58.239534
78194	VNCE	2020-10-08	5.52	5.68	5.46	5.6	11980	{}	\N	2020-10-12 16:30:58.244176
78195	VNCE	2020-10-09	5.48	5.61	5.31	5.4	16818	{}	\N	2020-10-12 16:30:58.248039
78196	VNCE	2020-10-12	5.42	5.5299	5.2448	5.28	9702	{}	\N	2020-10-12 16:30:58.252162
78197	VNRX	2020-09-28	3.13	3.33	3.04	3.32	326859	{}	\N	2020-10-12 16:30:59.40113
78198	VNRX	2020-09-29	3.3	3.3	3.16	3.23	126371	{}	\N	2020-10-12 16:30:59.414584
78199	VNRX	2020-09-30	3.22	3.35	3.2	3.21	78996	{}	\N	2020-10-12 16:30:59.422199
78200	VNRX	2020-10-01	3.18	3.25	3.15	3.17	115754	{}	\N	2020-10-12 16:30:59.428862
78201	VNRX	2020-10-02	3.13	3.23	3.12	3.19	74777	{}	\N	2020-10-12 16:30:59.434677
78202	VNRX	2020-10-05	3.21	3.4	3.21	3.39	192179	{}	\N	2020-10-12 16:30:59.440093
78203	VNRX	2020-10-06	3.37	3.54	3.36	3.41	276697	{}	\N	2020-10-12 16:30:59.445521
78204	VNRX	2020-10-07	3.45	3.5	3.37	3.48	101603	{}	\N	2020-10-12 16:30:59.450664
78205	VNRX	2020-10-08	3.47	3.55	3.42	3.48	109993	{}	\N	2020-10-12 16:30:59.455738
78206	VNRX	2020-10-09	3.46	3.49	3.44	3.45	56782	{}	\N	2020-10-12 16:30:59.461739
78207	VNRX	2020-10-12	3.64	3.75	3.55	3.59	232621	{}	\N	2020-10-12 16:30:59.46873
78208	VOYA	2020-09-28	47.38	48.37	47.09	47.98	1114362	{}	\N	2020-10-12 16:31:05.714922
78209	VOYA	2020-09-29	47.97	48.2	47.15	47.6	1481588	{}	\N	2020-10-12 16:31:05.726052
78210	VOYA	2020-09-30	47.72	48.33	47.35	47.93	3027986	{}	\N	2020-10-12 16:31:05.73259
78211	VOYA	2020-10-01	48.76	49.46	47.55	47.92	3092602	{}	\N	2020-10-12 16:31:05.737959
78212	VOYA	2020-10-02	47.16	48.43	47.01	48.14	3358621	{}	\N	2020-10-12 16:31:05.742397
78213	VOYA	2020-10-05	48.79	49.48	48.605	49.29	2702509	{}	\N	2020-10-12 16:31:05.747307
78214	VOYA	2020-10-06	49.79	51.24	49.29	49.75	2680368	{}	\N	2020-10-12 16:31:05.751654
78215	VOYA	2020-10-07	50.3	50.86	48.865	49.31	5018497	{}	\N	2020-10-12 16:31:05.756215
78216	VOYA	2020-10-08	49.7	50.51	49.54	50.47	2200478	{}	\N	2020-10-12 16:31:05.761228
78217	VOYA	2020-10-09	50.66	51.01	49.16	49.22	3355986	{}	\N	2020-10-12 16:31:05.765142
78218	VOYA	2020-10-12	49.72	50.53	49.27	49.97	1302408	{}	\N	2020-10-12 16:31:05.769117
78219	VRNA	2020-09-28	6.5	6.6899	6.39	6.39	10706	{}	\N	2020-10-12 16:31:06.974703
78220	VRNA	2020-09-29	6.39	6.62	6.39	6.61	14025	{}	\N	2020-10-12 16:31:06.97998
78221	VRNA	2020-09-30	6.615	6.66	6.2	6.24	27627	{}	\N	2020-10-12 16:31:06.98629
78222	VRNA	2020-10-01	6.3	6.45	6.0778	6.32	64805	{}	\N	2020-10-12 16:31:06.992994
78223	VRNA	2020-10-02	6.2	6.2475	6.04	6.08	33568	{}	\N	2020-10-12 16:31:07.001337
78224	VRNA	2020-10-05	6.02	6.2	6	6.12	42954	{}	\N	2020-10-12 16:31:07.007272
78225	VRNA	2020-10-06	6.175	6.21	6.02	6.1	21437	{}	\N	2020-10-12 16:31:07.014427
78226	VRNA	2020-10-07	6.46	6.46	6.08	6.14	31601	{}	\N	2020-10-12 16:31:07.021069
78227	VRNA	2020-10-08	6.16	6.16	6	6.09	48747	{}	\N	2020-10-12 16:31:07.029764
78228	VRNA	2020-10-09	6.04	6.08	5.91	6.05	30055	{}	\N	2020-10-12 16:31:07.037494
78229	VRNA	2020-10-12	5.9	5.9	5.64	5.87	52635	{}	\N	2020-10-12 16:31:07.046871
78230	VRSN	2020-09-28	207.93	207.97	202.71	204.72	564384	{}	\N	2020-10-12 16:31:08.426363
78231	VRSN	2020-09-29	200.63	206.2	200.63	204.91	493532	{}	\N	2020-10-12 16:31:08.439402
78232	VRSN	2020-09-30	205.03	207.36	203.6	204.85	607916	{}	\N	2020-10-12 16:31:08.447012
78233	VRSN	2020-10-01	206.52	208.19	205.92	207.22	401800	{}	\N	2020-10-12 16:31:08.453323
78234	VRSN	2020-10-02	203.45	205.03	200.85	202.16	386272	{}	\N	2020-10-12 16:31:08.458766
78235	VRSN	2020-10-05	202.42	204.045	201.05	203.45	421997	{}	\N	2020-10-12 16:31:08.464923
78236	VRSN	2020-10-06	203.3	206.41	201.17	202.09	565688	{}	\N	2020-10-12 16:31:08.470184
78237	VRSN	2020-10-07	202.28	204.65	201.23	203.82	367768	{}	\N	2020-10-12 16:31:08.475362
78238	VRSN	2020-10-08	204.45	206.43	204.18	205.47	320053	{}	\N	2020-10-12 16:31:08.480585
78239	VRSN	2020-10-09	207.03	210.07	206.72	209.56	381911	{}	\N	2020-10-12 16:31:08.486389
78240	VRSN	2020-10-12	212.22	215	210.53	215	151950	{}	\N	2020-10-12 16:31:08.492091
78241	VSTM	2020-09-28	1.28	1.3	1.25	1.27	1114887	{}	\N	2020-10-12 16:31:10.637468
78242	VSTM	2020-09-29	1.27	1.278	1.21	1.23	1432736	{}	\N	2020-10-12 16:31:10.644779
78243	VSTM	2020-09-30	1.23	1.27	1.19	1.21	3154917	{}	\N	2020-10-12 16:31:10.651963
78244	VSTM	2020-10-01	1.24	1.27	1.2119	1.26	4680563	{}	\N	2020-10-12 16:31:10.660467
78245	VSTM	2020-10-02	1.24	1.28	1.23	1.27	3742221	{}	\N	2020-10-12 16:31:10.668432
78246	VSTM	2020-10-05	1.27	1.35	1.26	1.33	2382214	{}	\N	2020-10-12 16:31:10.676316
78247	VSTM	2020-10-06	1.32	1.3435	1.26	1.27	1495778	{}	\N	2020-10-12 16:31:10.684174
78248	VSTM	2020-10-07	1.3	1.41	1.26	1.36	8502491	{}	\N	2020-10-12 16:31:10.691647
78249	VSTM	2020-10-08	1.38	1.39	1.34	1.34	1230726	{}	\N	2020-10-12 16:31:10.699671
78250	VSTM	2020-10-09	1.37	1.41	1.34	1.4	1654587	{}	\N	2020-10-12 16:31:10.708409
78251	VSTM	2020-10-12	1.42	1.42	1.35	1.355	759904	{}	\N	2020-10-12 16:31:10.717165
78252	VUG	2020-09-28	226.36	226.8399	224.96	226.63	851467	{}	\N	2020-10-12 16:31:12.632218
78253	VUG	2020-09-29	226.6	227.43	225.875	226.16	930072	{}	\N	2020-10-12 16:31:12.648101
78254	VUG	2020-09-30	226.56	230.06	226.28	227.61	822922	{}	\N	2020-10-12 16:31:12.658385
78255	VUG	2020-10-01	230.48	231.04	229.22	230.64	691247	{}	\N	2020-10-12 16:31:12.665833
78256	VUG	2020-10-02	226.11	229.1358	225.2	226.3	884641	{}	\N	2020-10-12 16:31:12.671334
78257	VUG	2020-10-05	227.71	230.78	227.71	230.59	620107	{}	\N	2020-10-12 16:31:12.677171
78258	VUG	2020-10-06	230.43	231.6	226.08	226.74	664825	{}	\N	2020-10-12 16:31:12.682186
78259	VUG	2020-10-07	228.91	231.1888	228.81	230.56	741579	{}	\N	2020-10-12 16:31:12.689828
78260	VUG	2020-10-08	232.45	232.5997	231.16	231.95	552459	{}	\N	2020-10-12 16:31:12.696337
78261	VUG	2020-10-09	233.12	235.43	233.01	235.43	1014259	{}	\N	2020-10-12 16:31:12.70206
78262	VUG	2020-10-12	238.61	241.23	237.56	241.165	385813	{}	\N	2020-10-12 16:31:12.708471
78263	VZ	2020-09-28	59.56	59.795	59.183	59.36	12127670	{}	\N	2020-10-12 16:31:13.807937
78264	VZ	2020-09-29	59.28	59.49	59.1	59.24	19676222	{}	\N	2020-10-12 16:31:13.816688
78265	VZ	2020-09-30	59.46	59.705	59.045	59.49	31251765	{}	\N	2020-10-12 16:31:13.822621
78266	VZ	2020-10-01	59.4	59.6	59.03	59.45	28795850	{}	\N	2020-10-12 16:31:13.830409
78267	VZ	2020-10-02	59.29	59.545	59.11	59.24	20667770	{}	\N	2020-10-12 16:31:13.837649
78268	VZ	2020-10-05	59.63	59.84	59.26	59.66	20697860	{}	\N	2020-10-12 16:31:13.843423
78269	VZ	2020-10-06	59.76	59.97	59.33	59.46	23279441	{}	\N	2020-10-12 16:31:13.849256
78270	VZ	2020-10-07	59.62	59.78	59.2	59.61	27187008	{}	\N	2020-10-12 16:31:13.856608
78271	VZ	2020-10-08	59.12	59.73	58.97	59.19	28914627	{}	\N	2020-10-12 16:31:13.86414
78272	VZ	2020-10-09	59.25	59.54	59.08	59.33	19297976	{}	\N	2020-10-12 16:31:13.87314
78273	VZ	2020-10-12	59.33	59.61	59.15	59.5191	15356815	{}	\N	2020-10-12 16:31:13.879462
78274	WBT	2020-09-28	6.29	6.41	6.13	6.29	2045424	{}	\N	2020-10-12 16:31:16.375459
78275	WBT	2020-09-29	6.27	6.27	5.915	5.95	4127800	{}	\N	2020-10-12 16:31:16.384765
78276	WBT	2020-09-30	5.99	6.26	5.985	6.16	2424106	{}	\N	2020-10-12 16:31:16.394544
78277	WBT	2020-10-01	6.17	6.28	5.99	6.12	3217534	{}	\N	2020-10-12 16:31:16.401208
78278	WBT	2020-10-02	5.9	6.52	5.85	6.49	3181893	{}	\N	2020-10-12 16:31:16.406283
78279	WBT	2020-10-05	6.61	6.8	6.49	6.77	3469419	{}	\N	2020-10-12 16:31:16.411316
78280	WBT	2020-10-06	6.92	7.01	6.63	6.7	4028931	{}	\N	2020-10-12 16:31:16.416226
78281	WBT	2020-10-07	6.8	7.09	6.8	7.01	1581127	{}	\N	2020-10-12 16:31:16.421451
78282	WBT	2020-10-08	7.14	7.3	6.985	7.08	7541378	{}	\N	2020-10-12 16:31:16.427396
78283	WBT	2020-10-09	7.14	7.22	6.89	7.04	2240247	{}	\N	2020-10-12 16:31:16.432241
78284	WBT	2020-10-12	7.02	7.19	6.94	7.12	537727	{}	\N	2020-10-12 16:31:16.438928
78285	VO	2020-09-28	175.17	176.53	174.69	176.23	623068	{}	\N	2020-10-12 16:31:17.412997
78286	VO	2020-09-29	176.07	176.59	174.765	175.22	757885	{}	\N	2020-10-12 16:31:17.420208
78287	VO	2020-09-30	175.81	177.7564	174.92	176.26	787824	{}	\N	2020-10-12 16:31:17.425919
78288	VO	2020-10-01	177.35	178.19	176.59	177.87	1783782	{}	\N	2020-10-12 16:31:17.43166
78289	VO	2020-10-02	174.84	178.86	174.76	178.13	769264	{}	\N	2020-10-12 16:31:17.440959
78290	VO	2020-10-05	179.45	181.3686	179.375	181.1	752156	{}	\N	2020-10-12 16:31:17.449249
78291	VO	2020-10-06	181.86	183.5	179.31	179.53	671852	{}	\N	2020-10-12 16:31:17.455998
78292	VO	2020-10-07	181.37	183.11	181.1	182.5	688682	{}	\N	2020-10-12 16:31:17.462353
78293	VO	2020-10-08	183.78	184.8	183.4501	184.8	493211	{}	\N	2020-10-12 16:31:17.46882
78294	VO	2020-10-09	186.08	186.4	185.11	185.57	491239	{}	\N	2020-10-12 16:31:17.476325
78295	VO	2020-10-12	186.62	187.3	185.98	187.2788	168271	{}	\N	2020-10-12 16:31:17.483048
78296	VOOV	2020-09-28	108.3	109.25	108.3	108.68	29855	{}	\N	2020-10-12 16:31:19.83424
78297	VOOV	2020-09-29	108.69	108.69	107.5756	107.85	28957	{}	\N	2020-10-12 16:31:19.844299
78298	VOOV	2020-09-30	108.25	109.81	108.25	108.71	42830	{}	\N	2020-10-12 16:31:19.857684
78299	VOOV	2020-10-01	109.31	109.6127	108.29	108.91	35161	{}	\N	2020-10-12 16:31:19.863502
78300	VOOV	2020-10-02	107.35	109.755	107.35	109.2605	33512	{}	\N	2020-10-12 16:31:19.868512
78301	VOOV	2020-10-05	109.95	110.9585	109.95	110.79	27074	{}	\N	2020-10-12 16:31:19.873869
78302	VOOV	2020-10-06	111.25	111.99	109.75	109.83	54706	{}	\N	2020-10-12 16:31:19.879857
78303	VOOV	2020-10-07	110.66	111.92	110.66	111.53	39858	{}	\N	2020-10-12 16:31:19.884914
78304	VOOV	2020-10-08	112.19	112.965	112.19	112.91	52250	{}	\N	2020-10-12 16:31:19.889895
78305	VOOV	2020-10-09	113.55	113.7999	112.88	113.24	70955	{}	\N	2020-10-12 16:31:19.895687
78306	VOOV	2020-10-12	113.46	114.1	113.35	114.1	50773	{}	\N	2020-10-12 16:31:19.900596
78307	VRA	2020-09-28	6.55	6.68	6.465	6.51	340159	{}	\N	2020-10-12 16:31:20.943486
78308	VRA	2020-09-29	6.55	6.55	6.05	6.11	501904	{}	\N	2020-10-12 16:31:20.950495
78309	VRA	2020-09-30	6.17	6.5	6.05	6.11	470328	{}	\N	2020-10-12 16:31:20.955798
78310	VRA	2020-10-01	6.13	6.63	6.12	6.57	534104	{}	\N	2020-10-12 16:31:20.962345
78311	VRA	2020-10-02	6.32	6.96	6.32	6.89	357926	{}	\N	2020-10-12 16:31:20.97015
78312	VRA	2020-10-05	6.97	6.99	6.775	6.94	457576	{}	\N	2020-10-12 16:31:20.979012
78313	VRA	2020-10-06	6.98	7.08	6.64	6.69	493386	{}	\N	2020-10-12 16:31:20.985751
78314	VRA	2020-10-07	6.72	7.09	6.72	6.93	365780	{}	\N	2020-10-12 16:31:20.993792
78315	VRA	2020-10-08	6.99	7.19	6.88	7.02	244454	{}	\N	2020-10-12 16:31:21.002221
78316	VRA	2020-10-09	7.11	7.28	6.96	7.14	279290	{}	\N	2020-10-12 16:31:21.009091
78317	VRA	2020-10-12	7.18	7.34	7.1	7.2	95424	{}	\N	2020-10-12 16:31:21.015013
78318	VRSK	2020-09-28	183.33	185.08	182.96	184.52	599005	{}	\N	2020-10-12 16:31:22.29538
78319	VRSK	2020-09-29	185.02	188.03	184.36	184.38	907859	{}	\N	2020-10-12 16:31:22.307555
78320	VRSK	2020-09-30	184.7	186.7	183.185	185.31	961506	{}	\N	2020-10-12 16:31:22.316945
78321	VRSK	2020-10-01	185.73	188.73	184.925	185.64	766714	{}	\N	2020-10-12 16:31:22.323589
78322	VRSK	2020-10-02	182.64	184.74	180.725	182.1	570222	{}	\N	2020-10-12 16:31:22.329589
78323	VRSK	2020-10-05	183.08	183.565	180.52	181.79	337805	{}	\N	2020-10-12 16:31:22.335401
78324	VRSK	2020-10-06	181.58	183.02	179.5	179.79	485230	{}	\N	2020-10-12 16:31:22.341116
78325	VRSK	2020-10-07	181.71	183.22	180.81	183.03	536475	{}	\N	2020-10-12 16:31:22.345544
78326	VRSK	2020-10-08	184.42	185.225	182.3	184.9	397980	{}	\N	2020-10-12 16:31:22.350839
78327	VRSK	2020-10-09	185.94	188.03	184.88	187.18	543970	{}	\N	2020-10-12 16:31:22.356982
78328	VRSK	2020-10-12	189.59	192.43	188.79	192.28	369713	{}	\N	2020-10-12 16:31:22.363584
78329	VSDA	2020-09-28	36.05	36.32	36.05	36.12	6917	{}	\N	2020-10-12 16:31:24.118487
78330	VSDA	2020-09-29	36.1638	36.18	35.868	36.0422	13099	{}	\N	2020-10-12 16:31:24.128795
78331	VSDA	2020-09-30	36.06	36.502	36.06	36.19	28653	{}	\N	2020-10-12 16:31:24.135955
78332	VSDA	2020-10-01	36.4	36.459	36.1707	36.21	16625	{}	\N	2020-10-12 16:31:24.142786
78333	VSDA	2020-10-02	35.84	36.47	35.84	36.31	111885	{}	\N	2020-10-12 16:31:24.149291
78334	VSDA	2020-10-05	36.615	36.831	36.615	36.79	14875	{}	\N	2020-10-12 16:31:24.154932
78335	VSDA	2020-10-06	36.9	37.27	36.59	36.59	16090	{}	\N	2020-10-12 16:31:24.160271
78336	VSDA	2020-10-07	36.95	37.16	36.88	37.1	9025	{}	\N	2020-10-12 16:31:24.166618
78337	VSDA	2020-10-08	37.31	37.3902	37.205	37.35	30473	{}	\N	2020-10-12 16:31:24.17276
78338	VSDA	2020-10-09	37.6534	37.714	37.529	37.6597	8336	{}	\N	2020-10-12 16:31:24.178454
78339	VSDA	2020-10-12	37.74	37.9783	37.74	37.9783	4816	{}	\N	2020-10-12 16:31:24.183014
78340	VTHR	2020-09-28	151.69	152.14	151.0483	152.14	6287	{}	\N	2020-10-12 16:31:25.542306
78341	VTHR	2020-09-29	152.25	152.25	151.13	151.53	1940	{}	\N	2020-10-12 16:31:25.550954
78342	VTHR	2020-09-30	152.07	153.75	152.07	152.24	11139	{}	\N	2020-10-12 16:31:25.556198
78343	VTHR	2020-10-01	153.63	153.63	152.775	153.1101	19209	{}	\N	2020-10-12 16:31:25.561525
78344	VTHR	2020-10-02	150.46	152.9129	150.46	152.745	10314	{}	\N	2020-10-12 16:31:25.566343
78345	VTHR	2020-10-05	153.25	155.12	153	155.12	6122	{}	\N	2020-10-12 16:31:25.57152
78346	VTHR	2020-10-06	155.5	155.83	153.38	153.48	36635	{}	\N	2020-10-12 16:31:25.576836
78347	VTHR	2020-10-07	154.45	156.25	154.0035	156.03	18404	{}	\N	2020-10-12 16:31:25.582619
78348	VTHR	2020-10-08	156.92	157.37	156.81	157.35	103830	{}	\N	2020-10-12 16:31:25.587493
78349	VTHR	2020-10-09	158	158.91	157.9805	158.56	2979	{}	\N	2020-10-12 16:31:25.592192
78350	VTHR	2020-10-12	160.6	160.68	159.78	160.68	10934	{}	\N	2020-10-12 16:31:25.597251
78351	WAL	2020-09-28	32.14	32.785	31.9	32.55	333676	{}	\N	2020-10-12 16:31:27.945021
78352	WAL	2020-09-29	32.4	32.4	31.345	31.78	1259130	{}	\N	2020-10-12 16:31:27.958517
78353	WAL	2020-09-30	32.07	32.53	31.43	31.62	2078423	{}	\N	2020-10-12 16:31:27.966004
78354	WAL	2020-10-01	31.77	31.99	31.23	31.92	1676219	{}	\N	2020-10-12 16:31:27.972131
78355	WAL	2020-10-02	31.29	33.18	31.1946	33.05	1273786	{}	\N	2020-10-12 16:31:27.977712
78356	WAL	2020-10-05	33.58	34.58	33.3701	34.47	1366254	{}	\N	2020-10-12 16:31:27.983354
78357	WAL	2020-10-06	34.89	36.24	34.84	34.89	1751123	{}	\N	2020-10-12 16:31:27.988462
78358	WAL	2020-10-07	35.36	36.86	35.36	36.71	2645644	{}	\N	2020-10-12 16:31:27.993811
78359	WAL	2020-10-08	37.01	37.62	36.81	37.25	1301791	{}	\N	2020-10-12 16:31:28.000224
78360	WAL	2020-10-09	37.64	38.16	36.995	37.55	2355158	{}	\N	2020-10-12 16:31:28.005622
78361	WAL	2020-10-12	37.48	37.76	37.45	37.65	344591	{}	\N	2020-10-12 16:31:28.011311
78362	VPL	2020-09-28	68.23	68.49	68.14	68.46	628028	{}	\N	2020-10-12 16:31:29.498588
78363	VPL	2020-09-29	68.4	68.51	68.13	68.25	439948	{}	\N	2020-10-12 16:31:29.513035
78364	VPL	2020-09-30	67.85	68.37	67.661	68.05	340611	{}	\N	2020-10-12 16:31:29.522319
78365	VPL	2020-10-01	68.27	68.38	68.05	68.33	338662	{}	\N	2020-10-12 16:31:29.529267
78366	VPL	2020-10-02	67.45	68.105	67.43	67.91	397461	{}	\N	2020-10-12 16:31:29.535055
78367	VPL	2020-10-05	68.47	68.84	68.47	68.77	263567	{}	\N	2020-10-12 16:31:29.540875
78368	VPL	2020-10-06	68.83	69.01	68.26	68.44	348638	{}	\N	2020-10-12 16:31:29.546476
78369	VPL	2020-10-07	68.96	69.2	68.75	69.13	235874	{}	\N	2020-10-12 16:31:29.551698
78370	VPL	2020-10-08	69.34	69.42	69.2	69.36	238842	{}	\N	2020-10-12 16:31:29.557037
78371	VPL	2020-10-09	69.39	69.61	69.37	69.53	337008	{}	\N	2020-10-12 16:31:29.561975
78372	VPL	2020-10-12	69.6	69.84	69.41	69.83	147851	{}	\N	2020-10-12 16:31:29.567504
78373	VRNT	2020-09-28	47.57	47.84	46.93	47.52	366297	{}	\N	2020-10-12 16:31:30.545607
78374	VRNT	2020-09-29	47.4	47.475	46.91	47.18	259743	{}	\N	2020-10-12 16:31:30.556531
78375	VRNT	2020-09-30	47.17	48.3	46.88	48.18	539189	{}	\N	2020-10-12 16:31:30.563809
78376	VRNT	2020-10-01	48.5	49.58	48.09	49.53	468787	{}	\N	2020-10-12 16:31:30.570796
78377	VRNT	2020-10-02	48.6	49.6	48.25	49.07	394670	{}	\N	2020-10-12 16:31:30.576813
78378	VRNT	2020-10-05	49.23	49.7	48.99	49.61	218217	{}	\N	2020-10-12 16:31:30.581322
78379	VRNT	2020-10-06	49.84	50.96	49.74	50.07	358414	{}	\N	2020-10-12 16:31:30.586121
78380	VRNT	2020-10-07	50.84	51.73	50	51.56	407719	{}	\N	2020-10-12 16:31:30.590593
78381	VRNT	2020-10-08	51.99	52.79	51.36	52.77	437979	{}	\N	2020-10-12 16:31:30.595048
78382	VRNT	2020-10-09	52.8	53.38	52.68	53.21	344958	{}	\N	2020-10-12 16:31:30.599613
78383	VRNT	2020-10-12	53.39	54.205	53.09	53.35	168180	{}	\N	2020-10-12 16:31:30.604196
78384	VRTS	2020-09-28	136.87	140.08	135	138.49	37920	{}	\N	2020-10-12 16:31:33.569624
78385	VRTS	2020-09-29	138.41	139.18	135.48	137.14	32156	{}	\N	2020-10-12 16:31:33.577
78386	VRTS	2020-09-30	137.95	140.94	137.69	138.65	37492	{}	\N	2020-10-12 16:31:33.582786
78387	VRTS	2020-10-01	139.2	139.78	137.19	138.3	30411	{}	\N	2020-10-12 16:31:33.588479
78388	VRTS	2020-10-02	135.89	143.14	135.22	142.41	49290	{}	\N	2020-10-12 16:31:33.594068
78389	VRTS	2020-10-05	144.35	148.96	143.64	148.46	56226	{}	\N	2020-10-12 16:31:33.59928
78390	VRTS	2020-10-06	150.68	150.68	145.18	145.68	69124	{}	\N	2020-10-12 16:31:33.604201
78391	VRTS	2020-10-07	147.1	148.95	143.69	144.2	84486	{}	\N	2020-10-12 16:31:33.608788
78392	VRTS	2020-10-08	145.8	154.64	145.8	154.27	66289	{}	\N	2020-10-12 16:31:33.613242
78393	VRTS	2020-10-09	156.66	157.6	154.28	155.61	49211	{}	\N	2020-10-12 16:31:33.617869
78394	VRTS	2020-10-12	156.85	158.04	155.02	156.455	17602	{}	\N	2020-10-12 16:31:33.623177
78395	VTC	2020-09-28	92.03	92.33	92.03	92.3	43718	{}	\N	2020-10-12 16:31:35.89224
78396	VTC	2020-09-29	92.31	92.46	92.25	92.27	35897	{}	\N	2020-10-12 16:31:35.903237
78397	VTC	2020-09-30	92.23	92.25	92.05	92.19	32650	{}	\N	2020-10-12 16:31:35.911374
78398	VTC	2020-10-01	92.01	92.25	91.9066	92.25	43249	{}	\N	2020-10-12 16:31:35.91909
78399	VTC	2020-10-02	92.07	92.25	91.95	92.16	69922	{}	\N	2020-10-12 16:31:35.925912
78400	VTC	2020-10-05	92.04	92.14	91.9	91.91	106881	{}	\N	2020-10-12 16:31:35.931402
78401	VTC	2020-10-06	91.9	92.29	91.85	91.88	45306	{}	\N	2020-10-12 16:31:35.936318
78402	VTC	2020-10-07	91.93	92.06	91.78	91.86	64261	{}	\N	2020-10-12 16:31:35.942268
78403	VTC	2020-10-08	92.01	92.129	91.98	92.08	25293	{}	\N	2020-10-12 16:31:35.94828
78404	VTC	2020-10-09	92.07	92.2499	91.97	92.19	81493	{}	\N	2020-10-12 16:31:35.953747
78405	VTC	2020-10-12	92.41	92.5722	92.395	92.56	11865	{}	\N	2020-10-12 16:31:35.959468
78406	VTWO	2020-09-28	119.93	121.34	119.77	121.31	76471	{}	\N	2020-10-12 16:31:37.526259
78407	VTWO	2020-09-29	121.09	121.27	119.57	120.67	152096	{}	\N	2020-10-12 16:31:37.536414
78408	VTWO	2020-09-30	121.07	122.5613	120.145	120.94	62804	{}	\N	2020-10-12 16:31:37.543164
78409	VTWO	2020-10-01	121.85	122.9	120.82	122.89	334876	{}	\N	2020-10-12 16:31:37.548926
78410	VTWO	2020-10-02	120.4	123.9025	120.4	123.53	138313	{}	\N	2020-10-12 16:31:37.553648
78411	VTWO	2020-10-05	124.86	127	124.86	126.89	249040	{}	\N	2020-10-12 16:31:37.558744
78412	VTWO	2020-10-06	127.83	129.94	126.26	126.38	146707	{}	\N	2020-10-12 16:31:37.564013
78413	VTWO	2020-10-07	128.15	129.6	128.02	129.31	167924	{}	\N	2020-10-12 16:31:37.569912
78414	VTWO	2020-10-08	130.47	130.94	129.77	130.69	108406	{}	\N	2020-10-12 16:31:37.576363
78415	VTWO	2020-10-09	131.84	132.05	130.76	131.39	63027	{}	\N	2020-10-12 16:31:37.582826
78416	VTWO	2020-10-12	131.71	132.4248	131.2504	132.1	36429	{}	\N	2020-10-12 16:31:37.588147
78417	VYMI	2020-09-28	52.22	52.41	52.137	52.25	103584	{}	\N	2020-10-12 16:31:41.564915
78418	VYMI	2020-09-29	52.13	52.2	51.8499	52.09	53801	{}	\N	2020-10-12 16:31:41.575328
78419	VYMI	2020-09-30	52.01	52.3154	51.93	52.06	76498	{}	\N	2020-10-12 16:31:41.582854
78420	VYMI	2020-10-01	52.21	52.25	51.9	52.14	76817	{}	\N	2020-10-12 16:31:41.589154
78421	VYMI	2020-10-02	51.64	52.215	51.64	52.13	106207	{}	\N	2020-10-12 16:31:41.595155
78422	VYMI	2020-10-05	52.47	53	52.47	53	145450	{}	\N	2020-10-12 16:31:41.600898
78423	VYMI	2020-10-06	53.19	53.22	52.55	52.56	101988	{}	\N	2020-10-12 16:31:41.606856
78424	VYMI	2020-10-07	52.92	53.15	52.8401	53.02	207448	{}	\N	2020-10-12 16:31:41.612868
78425	VYMI	2020-10-08	53.31	53.5	53.22	53.5	95840	{}	\N	2020-10-12 16:31:41.619305
78426	VYMI	2020-10-09	53.69	53.84	53.55	53.69	134451	{}	\N	2020-10-12 16:31:41.626087
78427	VYMI	2020-10-12	53.84	53.9499	53.69	53.923	57743	{}	\N	2020-10-12 16:31:41.633516
78428	WALA	2020-09-28	25.5264	25.925	25.5264	25.88	10966	{}	\N	2020-10-12 16:31:42.539927
78429	WALA	2020-09-29	25.88	25.88	25.7	25.72	12140	{}	\N	2020-10-12 16:31:42.550602
78430	WALA	2020-09-30	25.75	26	25.62	26	76726	{}	\N	2020-10-12 16:31:42.558981
78431	WALA	2020-10-01	26.14	26.14	25.7	26.03	31340	{}	\N	2020-10-12 16:31:42.565402
78432	WALA	2020-10-02	25.7	26.05	25.7	26.005	10132	{}	\N	2020-10-12 16:31:42.571976
78433	WALA	2020-10-05	25.975	26.1074	25.97	25.97	9794	{}	\N	2020-10-12 16:31:42.577263
78434	WALA	2020-10-06	26.15	26.26	25.96	26.03	15446	{}	\N	2020-10-12 16:31:42.581915
78435	WALA	2020-10-07	26	26.1	25.82	25.99	10868	{}	\N	2020-10-12 16:31:42.587002
78436	WALA	2020-10-08	26.0569	26.0569	25.952	26	8132	{}	\N	2020-10-12 16:31:42.592224
78437	WALA	2020-10-09	25.992	26.11	25.955	26.11	2264	{}	\N	2020-10-12 16:31:42.597072
78438	WALA	2020-10-12	26.11	26.11	26	26	2902	{}	\N	2020-10-12 16:31:42.601441
78439	VREX	2020-09-28	11.865	12.69	11.865	12.3	335501	{}	\N	2020-10-12 16:31:44.778175
78440	VREX	2020-09-29	12.06	12.54	12	12.34	871732	{}	\N	2020-10-12 16:31:44.78918
78441	VREX	2020-09-30	12.37	12.88	12.11	12.72	300149	{}	\N	2020-10-12 16:31:44.796444
78442	VREX	2020-10-01	12.4	12.84	12.27	12.44	221555	{}	\N	2020-10-12 16:31:44.802857
78443	VREX	2020-10-02	12.24	12.59	11.96	12.37	206288	{}	\N	2020-10-12 16:31:44.808486
78444	VREX	2020-10-05	12.61	12.77	12.49	12.64	231913	{}	\N	2020-10-12 16:31:44.814209
78445	VREX	2020-10-06	12.79	12.84	12.41	12.51	390895	{}	\N	2020-10-12 16:31:44.819045
78446	VREX	2020-10-07	12.67	13.05	12.347	13	413391	{}	\N	2020-10-12 16:31:44.823815
78447	VREX	2020-10-08	13.16	13.3441	12.98	13.14	288423	{}	\N	2020-10-12 16:31:44.828669
78448	VREX	2020-10-09	13.17	13.59	12.95	13.52	333221	{}	\N	2020-10-12 16:31:44.834484
78449	VREX	2020-10-12	13.56	13.56	13.161	13.5	62501	{}	\N	2020-10-12 16:31:44.839293
78450	VRTX	2020-09-28	269.9	270.585	264.23	267.73	1058816	{}	\N	2020-10-12 16:31:47.090121
78451	VRTX	2020-09-29	267.82	271.68	266.38	270.31	862669	{}	\N	2020-10-12 16:31:47.100214
78452	VRTX	2020-09-30	271.33	274.66	269.465	272.12	1319281	{}	\N	2020-10-12 16:31:47.107231
78453	VRTX	2020-10-01	272.9	275.4	270.65	273.51	891553	{}	\N	2020-10-12 16:31:47.11311
78454	VRTX	2020-10-02	269.34	272.79	260.13	260.8	1272966	{}	\N	2020-10-12 16:31:47.118891
78455	VRTX	2020-10-05	262.86	269.07	261.7	268.98	1284412	{}	\N	2020-10-12 16:31:47.123925
78456	VRTX	2020-10-06	269.89	270.5	260.34	260.98	984635	{}	\N	2020-10-12 16:31:47.128383
78457	VRTX	2020-10-07	263.18	266.5	262.1	265.82	745737	{}	\N	2020-10-12 16:31:47.132614
78458	VRTX	2020-10-08	267.16	268.92	264.91	268.53	767924	{}	\N	2020-10-12 16:31:47.137291
78459	VRTX	2020-10-09	268.865	270.075	266.2401	269.95	862806	{}	\N	2020-10-12 16:31:47.141532
78460	VRTX	2020-10-12	273.14	273.22	269.575	272.8	551470	{}	\N	2020-10-12 16:31:47.145997
78461	VTGN	2020-09-28	0.63	0.661	0.6212	0.6322	691403	{}	\N	2020-10-12 16:31:48.897939
78462	VTGN	2020-09-29	0.655	0.6755	0.628	0.6637	800568	{}	\N	2020-10-12 16:31:48.909201
78463	VTGN	2020-09-30	0.67	0.75	0.6601	0.697	1344524	{}	\N	2020-10-12 16:31:48.916785
78464	VTGN	2020-10-01	0.681	0.7881	0.67	0.71	1235873	{}	\N	2020-10-12 16:31:48.922921
78465	VTGN	2020-10-02	0.73	0.75	0.6852	0.732	634261	{}	\N	2020-10-12 16:31:48.928776
78466	VTGN	2020-10-05	0.75	0.75	0.689	0.7115	597696	{}	\N	2020-10-12 16:31:48.933572
78467	VTGN	2020-10-06	0.71	0.75	0.6899	0.7	1003680	{}	\N	2020-10-12 16:31:48.938369
78468	VTGN	2020-10-07	0.74	0.745	0.693	0.7036	412715	{}	\N	2020-10-12 16:31:48.943448
78469	VTGN	2020-10-08	0.71	0.75	0.7003	0.74	735982	{}	\N	2020-10-12 16:31:48.948317
78470	VTGN	2020-10-09	0.76	0.7604	0.725	0.745	439779	{}	\N	2020-10-12 16:31:48.953126
78471	VTGN	2020-10-12	0.78	0.78	0.7456	0.7506	392681	{}	\N	2020-10-12 16:31:48.957983
78472	VTV	2020-09-28	104.02	104.97	103.99	104.45	3053577	{}	\N	2020-10-12 16:31:50.673953
78473	VTV	2020-09-29	104.45	104.54	103.2	103.57	1932294	{}	\N	2020-10-12 16:31:50.684529
78474	VTV	2020-09-30	103.94	105.43	103.89	104.51	3010895	{}	\N	2020-10-12 16:31:50.691101
78475	VTV	2020-10-01	104.95	105.3	103.9301	104.58	1798374	{}	\N	2020-10-12 16:31:50.696904
78476	VTV	2020-10-02	103.11	105.341	103.11	104.68	1762120	{}	\N	2020-10-12 16:31:50.702606
78477	VTV	2020-10-05	105.44	106.435	105.3768	106.33	1715409	{}	\N	2020-10-12 16:31:50.707997
78478	VTV	2020-10-06	106.64	107.47	105.16	105.33	1898274	{}	\N	2020-10-12 16:31:50.71233
78479	VTV	2020-10-07	106.19	107.37	106.17	106.99	1334656	{}	\N	2020-10-12 16:31:50.716597
78480	VTV	2020-10-08	107.57	108.35	107.53	108.32	1278422	{}	\N	2020-10-12 16:31:50.721083
78481	VTV	2020-10-09	108.93	109.12	108.2	108.51	1114244	{}	\N	2020-10-12 16:31:50.725305
78482	VTV	2020-10-12	108.81	109.44	108.6	109.43	523585	{}	\N	2020-10-12 16:31:50.730183
78483	VVPR	2020-09-28	8.61	8.85	7.93	8.1	457195	{}	\N	2020-10-12 16:31:52.89049
78484	VVPR	2020-09-29	8.1	9.3728	8.08	8.37	982433	{}	\N	2020-10-12 16:31:52.902048
78485	VVPR	2020-09-30	8.42	9.6868	8.42	9.16	1578569	{}	\N	2020-10-12 16:31:52.909902
78486	VVPR	2020-10-01	9.52	11.15	9.5	10.63	2349563	{}	\N	2020-10-12 16:31:52.91766
78487	VVPR	2020-10-02	10.01	12.9799	9.65	12.27	3054139	{}	\N	2020-10-12 16:31:52.923505
78488	VVPR	2020-10-05	12.5	12.89	10.83	11.49	1907551	{}	\N	2020-10-12 16:31:52.929361
78489	VVPR	2020-10-06	11.06	11.64	9.61	9.8	1265739	{}	\N	2020-10-12 16:31:52.936542
78490	VVPR	2020-10-07	10.4	11.49	10.15	10.5	883156	{}	\N	2020-10-12 16:31:52.943869
78491	VVPR	2020-10-08	10.6	10.75	9.97	10.09	444715	{}	\N	2020-10-12 16:31:52.950755
78492	VVPR	2020-10-09	15.84	24.33	14.1	16.28	113008411	{}	\N	2020-10-12 16:31:52.957836
78493	VVPR	2020-10-12	15	16	13.85	14.3794	3943727	{}	\N	2020-10-12 16:31:52.964165
78494	WATT	2020-09-28	2.95	3.08	2.87	3.02	794428	{}	\N	2020-10-12 16:31:54.271795
78495	WATT	2020-09-29	3.03	3.28	2.97	3.07	1979667	{}	\N	2020-10-12 16:31:54.283453
78496	WATT	2020-09-30	3.24	3.38	2.85	2.95	5200872	{}	\N	2020-10-12 16:31:54.291945
78497	WATT	2020-10-01	2.88	2.945	2.83	2.88	733981	{}	\N	2020-10-12 16:31:54.298506
78498	WATT	2020-10-02	2.8	2.87	2.76	2.77	640291	{}	\N	2020-10-12 16:31:54.304772
78499	WATT	2020-10-05	2.77	2.89	2.77	2.84	489639	{}	\N	2020-10-12 16:31:54.310545
78500	WATT	2020-10-06	2.81	2.9	2.78	2.78	608809	{}	\N	2020-10-12 16:31:54.316066
78501	WATT	2020-10-07	2.85	2.865	2.82	2.825	270702	{}	\N	2020-10-12 16:31:54.321658
78502	WATT	2020-10-08	2.87	2.91	2.82	2.88	385311	{}	\N	2020-10-12 16:31:54.327052
78503	WATT	2020-10-09	2.87	2.91	2.8406	2.87	257883	{}	\N	2020-10-12 16:31:54.331914
78504	WATT	2020-10-12	2.87	2.9	2.81	2.8211	138887	{}	\N	2020-10-12 16:31:54.337447
78505	WBIE	2020-09-28	26.69	27.2099	26.69	27.2099	1158	{}	\N	2020-10-12 16:31:55.340402
78506	WBIE	2020-09-29	27.1648	27.199	27.06	27.19	4950	{}	\N	2020-10-12 16:31:55.350905
78507	WBIE	2020-09-30	27.25	27.38	27.245	27.38	5230	{}	\N	2020-10-12 16:31:55.359663
78508	WBIE	2020-10-01	27.355	27.459	27.355	27.36	3380	{}	\N	2020-10-12 16:31:55.367296
78509	WBIE	2020-10-02	27.08	27.34	27.06	27.284	3182	{}	\N	2020-10-12 16:31:55.372866
78510	WBIE	2020-10-05	27.46	27.5573	27.43	27.5573	4845	{}	\N	2020-10-12 16:31:55.385998
78511	WBIE	2020-10-06	27.64	27.74	27.2828	27.2828	10831	{}	\N	2020-10-12 16:31:55.394163
78512	WBIE	2020-10-07	27.52	27.64	27.49	27.64	9395	{}	\N	2020-10-12 16:31:55.401944
78513	WBIE	2020-10-08	27.7739	27.8326	27.7739	27.8326	1238	{}	\N	2020-10-12 16:31:55.409946
78514	WBIE	2020-10-09	28.06	28.06	28.04	28.06	1034	{}	\N	2020-10-12 16:31:55.418411
78515	WBIE	2020-10-12	28.22	28.28	28.22	28.28	599	{}	\N	2020-10-12 16:31:55.423918
78516	WBK	2020-09-28	12.28	12.36	12.23	12.3	301777	{}	\N	2020-10-12 16:31:56.815969
78517	WBK	2020-09-29	12.19	12.2581	11.97	12.04	415604	{}	\N	2020-10-12 16:31:56.831128
78518	WBK	2020-09-30	12.03	12.17	12.005	12.04	400025	{}	\N	2020-10-12 16:31:56.840133
78519	WBK	2020-10-01	12.11	12.14	11.98	12.09	271463	{}	\N	2020-10-12 16:31:56.84783
78520	WBK	2020-10-02	11.81	12.09	11.8	12.02	467589	{}	\N	2020-10-12 16:31:56.855135
78521	WBK	2020-10-05	12.29	12.448	12.29	12.42	349884	{}	\N	2020-10-12 16:31:56.861105
78522	WBK	2020-10-06	12.53	12.585	12.32	12.33	455332	{}	\N	2020-10-12 16:31:56.866433
78523	WBK	2020-10-07	12.74	12.785	12.685	12.75	299303	{}	\N	2020-10-12 16:31:56.872726
78524	WBK	2020-10-08	12.88	12.92	12.83	12.91	290036	{}	\N	2020-10-12 16:31:56.877682
78525	WBK	2020-10-09	13.04	13.07	12.92	13.06	380536	{}	\N	2020-10-12 16:31:56.882923
78526	WBK	2020-10-12	13.2	13.325	13.2	13.315	233038	{}	\N	2020-10-12 16:31:56.888302
78527	VSAT	2020-09-28	33.76	34.72	33.72	34.19	357553	{}	\N	2020-10-12 16:31:59.328755
78528	VSAT	2020-09-29	34.17	34.66	33.8	34.24	417749	{}	\N	2020-10-12 16:31:59.338321
78529	VSAT	2020-09-30	34.49	34.98	33.92	34.39	868357	{}	\N	2020-10-12 16:31:59.345057
78530	VSAT	2020-10-01	34.4	35.22	34	34.35	378132	{}	\N	2020-10-12 16:31:59.351733
78531	VSAT	2020-10-02	33.59	34.67	33.56	34.48	255511	{}	\N	2020-10-12 16:31:59.357285
78532	VSAT	2020-10-05	34.97	35.37	34	34.37	178444	{}	\N	2020-10-12 16:31:59.362628
78533	VSAT	2020-10-06	34.53	35.34	34.01	34.1	309833	{}	\N	2020-10-12 16:31:59.367579
78534	VSAT	2020-10-07	34.15	35.4	33.96	34.13	419128	{}	\N	2020-10-12 16:31:59.372643
78535	VSAT	2020-10-08	34.43	35.29	34.42	35.15	434373	{}	\N	2020-10-12 16:31:59.37748
78536	VSAT	2020-10-09	35.51	35.95	34.9737	35.62	355593	{}	\N	2020-10-12 16:31:59.382926
78537	VSAT	2020-10-12	35.57	36.55	35.28	36.105	488006	{}	\N	2020-10-12 16:31:59.387992
78538	VSTO	2020-09-28	19.85	19.94	18.59	19.2	1567616	{}	\N	2020-10-12 16:32:00.571461
78539	VSTO	2020-09-29	19.1	19.6999	19.03	19.1	1123344	{}	\N	2020-10-12 16:32:00.582323
78540	VSTO	2020-09-30	19.1	20.9	19.1	20.18	5718361	{}	\N	2020-10-12 16:32:00.590618
78541	VSTO	2020-10-01	20.29	21.3	19.51	20.99	2927696	{}	\N	2020-10-12 16:32:00.598345
78542	VSTO	2020-10-02	20.3	21.23	20.08	20.69	2782146	{}	\N	2020-10-12 16:32:00.604595
78543	VSTO	2020-10-05	20.97	22.33	20.91	22.25	2433111	{}	\N	2020-10-12 16:32:00.610986
78544	VSTO	2020-10-06	22.75	23.11	21.83	21.97	3541886	{}	\N	2020-10-12 16:32:00.616474
78545	VSTO	2020-10-07	22.1	23.13	22.1	23	2612516	{}	\N	2020-10-12 16:32:00.621647
78546	VSTO	2020-10-08	23.32	23.5	21.4	21.88	2955798	{}	\N	2020-10-12 16:32:00.626601
78547	VSTO	2020-10-09	21.97	22.08	21.11	21.45	2204330	{}	\N	2020-10-12 16:32:00.631522
78548	VSTO	2020-10-12	21.51	21.86	20.5	20.52	1091665	{}	\N	2020-10-12 16:32:00.638325
78549	VTN	2020-09-28	12.32	12.405	12.2904	12.36	46024	{}	\N	2020-10-12 16:32:02.39538
78550	VTN	2020-09-29	12.37	12.3823	12.31	12.31	29254	{}	\N	2020-10-12 16:32:02.40319
78551	VTN	2020-09-30	12.38	12.385	12.33	12.38	38138	{}	\N	2020-10-12 16:32:02.408997
78552	VTN	2020-10-01	12.42	12.46	12.35	12.43	61668	{}	\N	2020-10-12 16:32:02.414352
78553	VTN	2020-10-02	12.32	12.43	12.32	12.4167	18517	{}	\N	2020-10-12 16:32:02.419432
78554	VTN	2020-10-05	12.43	12.43	12.36	12.36	13756	{}	\N	2020-10-12 16:32:02.42399
78555	VTN	2020-10-06	12.42	12.42	12.3	12.32	25462	{}	\N	2020-10-12 16:32:02.428392
78556	VTN	2020-10-07	12.29	12.32	12.21	12.25	125102	{}	\N	2020-10-12 16:32:02.432624
78557	VTN	2020-10-08	12.35	12.35	12.229	12.27	86902	{}	\N	2020-10-12 16:32:02.436957
78558	VTN	2020-10-09	12.32	12.32	12.25	12.28	28924	{}	\N	2020-10-12 16:32:02.441596
78559	VTN	2020-10-12	12.32	12.32	12.27	12.27	7340	{}	\N	2020-10-12 16:32:02.445997
78560	VVI	2020-09-28	19.55	21.58	19.55	21.35	491092	{}	\N	2020-10-12 16:32:05.290766
78561	VVI	2020-09-29	21.38	21.38	20.16	20.74	428002	{}	\N	2020-10-12 16:32:05.304404
78562	VVI	2020-09-30	20.85	21.78	20.51	20.83	306392	{}	\N	2020-10-12 16:32:05.312322
78563	VVI	2020-10-01	20.92	21.68	20.5	21.66	386668	{}	\N	2020-10-12 16:32:05.31858
78564	VVI	2020-10-02	20.8	22.69	20.7126	22.5	532843	{}	\N	2020-10-12 16:32:05.323598
78565	VVI	2020-10-05	22.72	22.9432	22.19	22.53	328458	{}	\N	2020-10-12 16:32:05.328482
78566	VVI	2020-10-06	22.92	23.4705	22.36	22.67	344964	{}	\N	2020-10-12 16:32:05.332507
78567	VVI	2020-10-07	23.22	23.255	22.56	23	391680	{}	\N	2020-10-12 16:32:05.336925
78568	VVI	2020-10-08	23.35	23.88	23.09	23.49	225536	{}	\N	2020-10-12 16:32:05.344314
78569	VVI	2020-10-09	23.73	23.8	22.8	23.24	241192	{}	\N	2020-10-12 16:32:05.348438
78570	VVI	2020-10-12	23.25	23.25	22.645	22.825	87098	{}	\N	2020-10-12 16:32:05.352682
78571	VYM	2020-09-28	80.57	81.37	80.551	80.94	1402599	{}	\N	2020-10-12 16:32:06.327724
78572	VYM	2020-09-29	80.96	81.06	80	80.31	1038530	{}	\N	2020-10-12 16:32:06.338295
78573	VYM	2020-09-30	80.52	81.619	80.41	80.93	1216332	{}	\N	2020-10-12 16:32:06.344841
78574	VYM	2020-10-01	81.23	81.5	80.42	80.9	1328907	{}	\N	2020-10-12 16:32:06.351062
78575	VYM	2020-10-02	79.87	81.44	79.811	81	1701098	{}	\N	2020-10-12 16:32:06.356457
78576	VYM	2020-10-05	81.55	82.41	81.51	82.32	1417177	{}	\N	2020-10-12 16:32:06.360716
78577	VYM	2020-10-06	82.61	83.39	81.59	81.72	1823022	{}	\N	2020-10-12 16:32:06.36508
78578	VYM	2020-10-07	82.36	83.23	82.345	83.03	1118698	{}	\N	2020-10-12 16:32:06.369185
78579	VYM	2020-10-08	83.42	84.07	83.305	84.04	1383415	{}	\N	2020-10-12 16:32:06.373231
78580	VYM	2020-10-09	84.5	84.64	83.93	84.09	1166021	{}	\N	2020-10-12 16:32:06.377201
78581	VYM	2020-10-12	84.24	84.875	84.14	84.84	615226	{}	\N	2020-10-12 16:32:06.38122
78582	WBIL	2020-09-28	25.85	26.3535	25.85	26.3535	3707	{}	\N	2020-10-12 16:32:09.779483
78583	WBIL	2020-09-29	26.29	26.355	26.2703	26.2703	6023	{}	\N	2020-10-12 16:32:09.791207
78584	WBIL	2020-09-30	26.37	26.5183	26.3087	26.3087	8256	{}	\N	2020-10-12 16:32:09.798729
78585	WBIL	2020-10-01	26.451	26.5734	26.45	26.4929	4228	{}	\N	2020-10-12 16:32:09.805066
78586	WBIL	2020-10-02	26.28	26.45	26.28	26.38	3036	{}	\N	2020-10-12 16:32:09.810942
78587	WBIL	2020-10-05	26.5599	26.6423	26.53	26.6423	4490	{}	\N	2020-10-12 16:32:09.816147
78588	WBIL	2020-10-06	26.671	26.71	26.3861	26.3861	6557	{}	\N	2020-10-12 16:32:09.821137
78589	WBIL	2020-10-07	26.65	26.6844	26.56	26.6844	13082	{}	\N	2020-10-12 16:32:09.826451
78590	WBIL	2020-10-08	26.8839	26.8967	26.84	26.8967	1678	{}	\N	2020-10-12 16:32:09.83148
78591	WBIL	2020-10-09	27.0701	27.0701	27.0666	27.0666	1807	{}	\N	2020-10-12 16:32:09.836422
78592	WBIL	2020-10-12	27.24	27.24	27.24	27.24	266	{}	\N	2020-10-12 16:32:09.841572
78593	WDR	2020-09-28	14.43	15.025	14.43	14.95	870088	{}	\N	2020-10-12 16:32:10.916919
78594	WDR	2020-09-29	14.86	14.9	14.53	14.67	1388599	{}	\N	2020-10-12 16:32:10.929623
78595	WDR	2020-09-30	14.73	15.04	14.73	14.85	1714156	{}	\N	2020-10-12 16:32:10.939416
78596	WDR	2020-10-01	14.85	14.925	14.4	14.55	1370684	{}	\N	2020-10-12 16:32:10.946726
78597	WDR	2020-10-02	14.37	15.2	14.365	15.05	2074025	{}	\N	2020-10-12 16:32:10.953398
78598	WDR	2020-10-05	15.2	15.44	15.185	15.4	1042174	{}	\N	2020-10-12 16:32:10.960314
78599	WDR	2020-10-06	15.6	15.8	15.37	15.45	1960708	{}	\N	2020-10-12 16:32:10.966618
78600	WDR	2020-10-07	15.58	16.02	15.56	15.97	1579369	{}	\N	2020-10-12 16:32:10.974091
78601	WDR	2020-10-08	16.16	17.03	16.16	16.93	3970353	{}	\N	2020-10-12 16:32:10.981628
78602	WDR	2020-10-09	17.03	17.14	16.78	17	2403417	{}	\N	2020-10-12 16:32:10.989273
78603	WDR	2020-10-12	17.03	17.23	16.57	16.63	750073	{}	\N	2020-10-12 16:32:10.995277
78604	WEX	2020-09-28	143.24	145.25	142.59	143.78	903011	{}	\N	2020-10-12 16:32:12.760651
78605	WEX	2020-09-29	142.95	143.155	140.7001	141.24	988865	{}	\N	2020-10-12 16:32:12.767316
78606	WEX	2020-09-30	140.65	144.2377	137.05	138.97	1127549	{}	\N	2020-10-12 16:32:12.773288
78607	WEX	2020-10-01	139.59	142.84	137.58	140.25	867141	{}	\N	2020-10-12 16:32:12.780057
78608	WEX	2020-10-02	136.38	142.88	136.12	142.58	752191	{}	\N	2020-10-12 16:32:12.786571
78609	WEX	2020-10-05	144.67	148.145	144.67	147.2	753877	{}	\N	2020-10-12 16:32:12.797156
78610	WEX	2020-10-06	148.07	151.11	145.06	145.44	645539	{}	\N	2020-10-12 16:32:12.806456
78611	WEX	2020-10-07	147.7	148.87	144.93	144.93	498879	{}	\N	2020-10-12 16:32:12.814599
78612	WEX	2020-10-08	146.88	148.96	145.995	147	647601	{}	\N	2020-10-12 16:32:12.825262
78613	WEX	2020-10-09	148.77	152.55	148.07	150.21	888592	{}	\N	2020-10-12 16:32:12.836972
78614	WEX	2020-10-12	170	170	157.19	160.7	1382956	{}	\N	2020-10-12 16:32:12.845431
78615	WHG	2020-09-28	11.59	11.84	11.53	11.63	26677	{}	\N	2020-10-12 16:32:16.117808
78616	WHG	2020-09-29	11.6	11.67	11.09	11.2	43980	{}	\N	2020-10-12 16:32:16.129136
78617	WHG	2020-09-30	11.56	11.67	11.05	11.14	69771	{}	\N	2020-10-12 16:32:16.1369
78618	WHG	2020-10-01	11.14	11.56	11.09	11.15	66675	{}	\N	2020-10-12 16:32:16.142779
78619	WHG	2020-10-02	11.09	11.68	11.09	11.56	49123	{}	\N	2020-10-12 16:32:16.14877
78620	WHG	2020-10-05	11.74	12.09	11.6653	11.74	63218	{}	\N	2020-10-12 16:32:16.153621
78621	WHG	2020-10-06	11.76	11.945	11.09	11.3	61895	{}	\N	2020-10-12 16:32:16.158366
78622	WHG	2020-10-07	11.4	11.68	11.33	11.5	60874	{}	\N	2020-10-12 16:32:16.163242
78623	WHG	2020-10-08	11.7	12.095	11.41	11.98	56708	{}	\N	2020-10-12 16:32:16.168625
78624	WHG	2020-10-09	11.98	12.03	11.51	11.69	65140	{}	\N	2020-10-12 16:32:16.173335
78625	WHG	2020-10-12	11.78	11.83	11.55	11.83	6550	{}	\N	2020-10-12 16:32:16.178075
78626	WINA	2020-09-28	165.89	169.535	164.14	168.42	12129	{}	\N	2020-10-12 16:32:17.897554
78627	WINA	2020-09-29	167.5	169.7	166.335	169.51	10029	{}	\N	2020-10-12 16:32:17.906562
78628	WINA	2020-09-30	169.95	172.7	166.42	172.18	23674	{}	\N	2020-10-12 16:32:17.912406
78629	WINA	2020-10-01	171.56	173.97	170.03	172.11	46218	{}	\N	2020-10-12 16:32:17.91925
78630	WINA	2020-10-02	170.38	176.59	162.075	176.38	23464	{}	\N	2020-10-12 16:32:17.924737
78631	WINA	2020-10-05	178	183.25	178	181.49	35563	{}	\N	2020-10-12 16:32:17.929541
78632	WINA	2020-10-06	180	182.105	179.26	180.97	31629	{}	\N	2020-10-12 16:32:17.934769
78633	WINA	2020-10-07	181.115	182.44	180.32	182.09	17950	{}	\N	2020-10-12 16:32:17.939711
78634	WINA	2020-10-08	182	182.99	181.14	181.5	28693	{}	\N	2020-10-12 16:32:17.944585
78635	WINA	2020-10-09	182	183.77	176.74	181.01	12518	{}	\N	2020-10-12 16:32:17.949406
78636	WINA	2020-10-12	180.5	180.62	180.5	180.62	1768	{}	\N	2020-10-12 16:32:17.954602
78637	VTVT	2020-09-28	1.77	1.77	1.72	1.77	168961	{}	\N	2020-10-12 16:32:19.177474
78638	VTVT	2020-09-29	1.74	1.8	1.73	1.8	237526	{}	\N	2020-10-12 16:32:19.187396
78639	VTVT	2020-09-30	1.8	1.8	1.74	1.77	141459	{}	\N	2020-10-12 16:32:19.193771
78640	VTVT	2020-10-01	1.78	1.785	1.72	1.76	118247	{}	\N	2020-10-12 16:32:19.199936
78641	VTVT	2020-10-02	1.73	1.78	1.7	1.75	176570	{}	\N	2020-10-12 16:32:19.20567
78642	VTVT	2020-10-05	1.8	1.84	1.76	1.82	178616	{}	\N	2020-10-12 16:32:19.210716
78643	VTVT	2020-10-06	1.83	1.88	1.79	1.8	253805	{}	\N	2020-10-12 16:32:19.215618
78644	VTVT	2020-10-07	1.82	1.88	1.8	1.87	100128	{}	\N	2020-10-12 16:32:19.220792
78645	VTVT	2020-10-08	1.88	1.89	1.82	1.86	73628	{}	\N	2020-10-12 16:32:19.22586
78646	VTVT	2020-10-09	1.87	1.8825	1.83	1.85	67920	{}	\N	2020-10-12 16:32:19.231224
78647	VTVT	2020-10-12	1.84	1.8413	1.77	1.8	121758	{}	\N	2020-10-12 16:32:19.23625
78648	VXZ	2020-09-28	34.38	34.52	34.3	34.45	9233	{}	\N	2020-10-12 16:32:22.069691
78649	VXZ	2020-09-29	34.32	34.32	33.87	34.035	8085	{}	\N	2020-10-12 16:32:22.081142
78650	VXZ	2020-09-30	33.8901	34.1666	33.8901	34.1666	3463	{}	\N	2020-10-12 16:32:22.0948
78651	VXZ	2020-10-01	34.02	34.59	33.97	34.59	6734	{}	\N	2020-10-12 16:32:22.100049
78652	VXZ	2020-10-02	35.22	35.22	34.65	34.97	8469	{}	\N	2020-10-12 16:32:22.10481
78653	VXZ	2020-10-05	34.96	34.96	34.12	34.13	12726	{}	\N	2020-10-12 16:32:22.109223
78654	VXZ	2020-10-06	34.03	34.5315	33.77	34.12	14955	{}	\N	2020-10-12 16:32:22.114108
78655	VXZ	2020-10-07	34.15	34.15	33.73	33.89	12822	{}	\N	2020-10-12 16:32:22.118808
78656	VXZ	2020-10-08	33.87	34.0999	33.54	33.55	3820	{}	\N	2020-10-12 16:32:22.124309
78657	VXZ	2020-10-09	33.08	33.08	32.47	32.61	12410	{}	\N	2020-10-12 16:32:22.129565
78658	VXZ	2020-10-12	32.18	32.415	32.1024	32.365	7832	{}	\N	2020-10-12 16:32:22.134218
78659	WABC	2020-09-28	53.46	54.54	53.46	54.43	108618	{}	\N	2020-10-12 16:32:23.932736
78660	WABC	2020-09-29	54.19	54.82	53.665	54.8	145067	{}	\N	2020-10-12 16:32:23.945442
78661	WABC	2020-09-30	54.98	55.4	53.9	54.35	125482	{}	\N	2020-10-12 16:32:23.961118
78662	WABC	2020-10-01	54.21	54.215	53	53.77	84389	{}	\N	2020-10-12 16:32:23.968953
78663	WABC	2020-10-02	53.11	54.88	53.11	54.42	71439	{}	\N	2020-10-12 16:32:23.978094
78664	WABC	2020-10-05	55.07	56.13	54.89	55.5	65737	{}	\N	2020-10-12 16:32:23.993028
78665	WABC	2020-10-06	56.03	56.2	55.18	55.2	398865	{}	\N	2020-10-12 16:32:24.002119
78666	WABC	2020-10-07	55.61	56.06	55.06	55.69	118301	{}	\N	2020-10-12 16:32:24.009867
78667	WABC	2020-10-08	55.98	56.47	55.35	56.34	118973	{}	\N	2020-10-12 16:32:24.016226
78668	WABC	2020-10-09	56.38	57.21	56.345	56.78	66877	{}	\N	2020-10-12 16:32:24.02202
78669	WABC	2020-10-12	56.78	57.41	56.31	57.04	14984	{}	\N	2020-10-12 16:32:24.027506
78670	WBA	2020-09-28	35.89	36.515	35.83	36.07	5711322	{}	\N	2020-10-12 16:32:24.933557
78671	WBA	2020-09-29	35.8	36.27	35.35	35.38	4936640	{}	\N	2020-10-12 16:32:24.942077
78672	WBA	2020-09-30	35.55	36.295	35.55	35.92	6422733	{}	\N	2020-10-12 16:32:24.947343
78673	WBA	2020-10-01	35.84	36.12	35.54	35.87	4593810	{}	\N	2020-10-12 16:32:24.956309
78674	WBA	2020-10-02	35.32	35.96	35.2801	35.77	5272339	{}	\N	2020-10-12 16:32:24.9657
78675	WBA	2020-10-05	36.02	36.67	35.95	36.62	4649505	{}	\N	2020-10-12 16:32:24.974447
78676	WBA	2020-10-06	36.6	37.03	35.81	35.95	4958207	{}	\N	2020-10-12 16:32:24.98021
78677	WBA	2020-10-07	36.1769	36.75	36.02	36.49	3593127	{}	\N	2020-10-12 16:32:24.986747
78678	WBA	2020-10-08	36.65	37.09	36.59	37.07	3593741	{}	\N	2020-10-12 16:32:24.993447
78679	WBA	2020-10-09	37.29	37.59	36.28	36.34	7339747	{}	\N	2020-10-12 16:32:24.998545
78680	WBA	2020-10-12	36.62	37.55	36.42	37.51	3619058	{}	\N	2020-10-12 16:32:25.003889
78681	WDC	2020-09-28	38.8466	39.24	38.3975	38.62	5113175	{}	\N	2020-10-12 16:32:26.905098
78682	WDC	2020-09-29	38.45	40.05	38.45	39.32	6544768	{}	\N	2020-10-12 16:32:26.915574
78683	WDC	2020-09-30	37.78	38.0151	36.47	36.55	13990805	{}	\N	2020-10-12 16:32:26.922851
78684	WDC	2020-10-01	36.81	37.026	36.09	36.33	7711425	{}	\N	2020-10-12 16:32:26.930622
78685	WDC	2020-10-02	35.5	36.58	35.29	35.96	5173563	{}	\N	2020-10-12 16:32:26.935826
78686	WDC	2020-10-05	36.23	36.94	36.2	36.57	5098415	{}	\N	2020-10-12 16:32:26.940893
78687	WDC	2020-10-06	36.63	37.12	36.17	36.26	4527794	{}	\N	2020-10-12 16:32:26.946753
78688	WDC	2020-10-07	36.99	37.815	36.71	37.55	5165670	{}	\N	2020-10-12 16:32:26.951838
78689	WDC	2020-10-08	37.975	38.82	37.89	38.7	4614621	{}	\N	2020-10-12 16:32:26.957144
78690	WDC	2020-10-09	39.34	39.55	37.97	38.12	4622956	{}	\N	2020-10-12 16:32:26.963319
78691	WDC	2020-10-12	38.08	38.135	37.6	37.98	2574441	{}	\N	2020-10-12 16:32:26.968346
78692	WHF	2020-09-28	9.55	9.995	9.4577	9.91	87707	{}	\N	2020-10-12 16:32:29.01185
78693	WHF	2020-09-29	9.86	9.86	9.46	9.6	53315	{}	\N	2020-10-12 16:32:29.019828
78694	WHF	2020-09-30	10.21	10.21	9.7401	9.79	81281	{}	\N	2020-10-12 16:32:29.026024
78695	WHF	2020-10-01	9.82	9.9	9.63	9.9	45375	{}	\N	2020-10-12 16:32:29.032734
78696	WHF	2020-10-02	9.85	10.47	9.83	10.39	130057	{}	\N	2020-10-12 16:32:29.039771
78697	WHF	2020-10-05	10.43	10.4699	10.24	10.28	64145	{}	\N	2020-10-12 16:32:29.048845
78698	WHF	2020-10-06	10.3	10.42	10.2501	10.31	40098	{}	\N	2020-10-12 16:32:29.057524
78699	WHF	2020-10-07	10.33	10.5	10.31	10.41	35230	{}	\N	2020-10-12 16:32:29.066903
78700	WHF	2020-10-08	10.43	10.55	10.405	10.48	33961	{}	\N	2020-10-12 16:32:29.075031
78701	WHF	2020-10-09	10.59	10.75	10.521	10.7	61832	{}	\N	2020-10-12 16:32:29.083714
78702	WHF	2020-10-12	10.76	10.78	10.57	10.6783	35787	{}	\N	2020-10-12 16:32:29.091182
78703	WING	2020-09-28	137.14	139.02	135.65	138.87	410660	{}	\N	2020-10-12 16:32:31.846326
78704	WING	2020-09-29	138.57	139.47	136.585	137.19	250710	{}	\N	2020-10-12 16:32:31.854381
78705	WING	2020-09-30	138.22	139.27	135.2	136.65	397888	{}	\N	2020-10-12 16:32:31.860547
78706	WING	2020-10-01	137.37	140.85	137.02	140.34	411808	{}	\N	2020-10-12 16:32:31.866698
78707	WING	2020-10-02	136.62	143.43	135.84	142.82	381059	{}	\N	2020-10-12 16:32:31.871687
78708	WING	2020-10-05	143.58	145	139.55	142.28	388901	{}	\N	2020-10-12 16:32:31.876527
78709	WING	2020-10-06	139.89	140.37	132.26	132.74	1054831	{}	\N	2020-10-12 16:32:31.881715
78710	WING	2020-10-07	133.73	135.9	129.74	131.25	846308	{}	\N	2020-10-12 16:32:31.886841
78711	WING	2020-10-08	132	132.4073	127.6079	130.34	807558	{}	\N	2020-10-12 16:32:31.891521
78712	WING	2020-10-09	131.96	132.45	129.51	131.26	588604	{}	\N	2020-10-12 16:32:31.895738
78713	WING	2020-10-12	131.53	132.75	129	129.27	158773	{}	\N	2020-10-12 16:32:31.900245
78714	WLDR	2020-09-28	20.16	20.3017	20.11	20.2105	5425	{}	\N	2020-10-12 16:32:33.288798
78715	WLDR	2020-09-29	20.095	20.095	20.095	20.095	267	{}	\N	2020-10-12 16:32:33.295633
78716	WLDR	2020-09-30	20.2999	20.4278	20.27	20.4278	4837	{}	\N	2020-10-12 16:32:33.302227
78717	WLDR	2020-10-01	20.36	20.36	20.36	20.36	197	{}	\N	2020-10-12 16:32:33.308634
78718	WLDR	2020-10-02	19.66	20.3589	19.66	20.3589	1348	{}	\N	2020-10-12 16:32:33.31565
78719	WLDR	2020-10-05	20.548	20.65	20.548	20.65	622	{}	\N	2020-10-12 16:32:33.32169
78720	WLDR	2020-10-06	20.37	20.37	20.3633	20.3633	947	{}	\N	2020-10-12 16:32:33.328718
78721	WLDR	2020-10-07	20.67	20.6799	20.6321	20.6321	802	{}	\N	2020-10-12 16:32:33.336293
78722	WLDR	2020-10-08	20.78	20.915	20.78	20.915	543	{}	\N	2020-10-12 16:32:33.344625
78723	WLDR	2020-10-09	21.05	21.17	21.05	21.17	3692	{}	\N	2020-10-12 16:32:33.350281
78724	WNS	2020-09-28	64.21	64.655	63.58	63.79	68582	{}	\N	2020-10-12 16:32:36.246857
78725	WNS	2020-09-29	63.79	64.39	63.11	64.17	123513	{}	\N	2020-10-12 16:32:36.259495
78726	WNS	2020-09-30	64.53	65.09	63.68	63.96	199382	{}	\N	2020-10-12 16:32:36.268102
78727	WNS	2020-10-01	64.48	65.12	63.93	64.4	138836	{}	\N	2020-10-12 16:32:36.274932
78728	WNS	2020-10-02	63.06	64.27	62.79	63.77	201173	{}	\N	2020-10-12 16:32:36.281306
78729	WNS	2020-10-05	64.27	65.17	63.19	64.62	157202	{}	\N	2020-10-12 16:32:36.286551
78730	WNS	2020-10-06	65.13	65.86	64.81	65	199347	{}	\N	2020-10-12 16:32:36.292864
78731	WNS	2020-10-07	65.9	66.1	65	65.71	120277	{}	\N	2020-10-12 16:32:36.298264
78732	WNS	2020-10-08	65.84	67	65.22	65.9	221656	{}	\N	2020-10-12 16:32:36.304057
78733	WNS	2020-10-09	66	67.34	65.72	67.23	213110	{}	\N	2020-10-12 16:32:36.309552
78734	WNS	2020-10-12	67.5	67.74	66.5212	67.55	74426	{}	\N	2020-10-12 16:32:36.315205
78735	YIN	2020-09-28	7.19	7.2	7.19	7.19	3274	{}	\N	2020-10-12 16:32:37.507396
78736	YIN	2020-09-29	7.19	7.195	7.19	7.19	10324	{}	\N	2020-10-12 16:32:37.517871
78737	YIN	2020-09-30	7.19	7.2	7.19	7.2	9309	{}	\N	2020-10-12 16:32:37.524538
78738	YIN	2020-10-01	7.19	7.21	7.19	7.19	10911	{}	\N	2020-10-12 16:32:37.531721
78739	YIN	2020-10-02	7.19	7.2	7.19	7.19	8322	{}	\N	2020-10-12 16:32:37.536823
78740	YIN	2020-10-05	7.19	7.215	7.19	7.19	12557	{}	\N	2020-10-12 16:32:37.541924
78741	YIN	2020-10-06	7.19	7.2	7.19	7.19	8751	{}	\N	2020-10-12 16:32:37.546852
78742	YIN	2020-10-07	7.19	7.1962	7.19	7.19	5227	{}	\N	2020-10-12 16:32:37.551994
78743	YIN	2020-10-08	7.19	7.2	7.19	7.2	6039	{}	\N	2020-10-12 16:32:37.557034
78744	YIN	2020-10-09	7.2	7.23	7.19	7.23	8313	{}	\N	2020-10-12 16:32:37.561882
78745	YIN	2020-10-12	7.19	7.2	7.19	7.19	5321	{}	\N	2020-10-12 16:32:37.566865
78746	VYGR	2020-09-28	11.64	11.658	10.96	11.17	191729	{}	\N	2020-10-12 16:32:38.718669
78747	VYGR	2020-09-29	11.15	11.35	10.71	11.075	190407	{}	\N	2020-10-12 16:32:38.730455
78748	VYGR	2020-09-30	11.11	11.22	10.505	10.67	121811	{}	\N	2020-10-12 16:32:38.738913
78749	VYGR	2020-10-01	10.74	10.86	10.54	10.8	171896	{}	\N	2020-10-12 16:32:38.744472
78750	VYGR	2020-10-02	10.43	10.7591	10.375	10.51	125531	{}	\N	2020-10-12 16:32:38.750209
78751	VYGR	2020-10-05	10.61	11.435	10.58	11.34	205545	{}	\N	2020-10-12 16:32:38.755415
78752	VYGR	2020-10-06	11.23	11.36	10.9	10.96	134372	{}	\N	2020-10-12 16:32:38.760174
78753	VYGR	2020-10-07	11.01	11.495	10.96	11.42	137443	{}	\N	2020-10-12 16:32:38.764742
78754	VYGR	2020-10-08	11.6	11.98	11.45	11.9	155130	{}	\N	2020-10-12 16:32:38.769573
78755	VYGR	2020-10-09	11.9	11.93	11.52	11.58	122317	{}	\N	2020-10-12 16:32:38.774292
78756	VYGR	2020-10-12	11.55	11.96	11.31	11.68	60333	{}	\N	2020-10-12 16:32:38.77898
78757	WAFD	2020-09-28	20.85	21.27	20.84	21.08	290719	{}	\N	2020-10-12 16:32:40.105577
78758	WAFD	2020-09-29	20.94	21.25	20.25	20.73	421236	{}	\N	2020-10-12 16:32:40.117961
78759	WAFD	2020-09-30	20.9	21.16	20.63	20.86	342353	{}	\N	2020-10-12 16:32:40.125931
78760	WAFD	2020-10-01	21.9339	21.9339	20.52	20.9	400883	{}	\N	2020-10-12 16:32:40.132411
78761	WAFD	2020-10-02	20.45	21.64	20.45	21.49	299303	{}	\N	2020-10-12 16:32:40.138568
78762	WAFD	2020-10-05	21.75	22.09	21.56	22.07	250176	{}	\N	2020-10-12 16:32:40.143719
78763	WAFD	2020-10-06	22.32	23.1	22.13	22.4	348440	{}	\N	2020-10-12 16:32:40.148877
78764	WAFD	2020-10-07	22.73	23.06	22.64	22.86	344734	{}	\N	2020-10-12 16:32:40.15376
78765	WAFD	2020-10-08	22.97	23.35	22.73	23.08	228117	{}	\N	2020-10-12 16:32:40.159595
78766	WAFD	2020-10-09	23.32	23.42	22.85	23.21	269656	{}	\N	2020-10-12 16:32:40.164678
78767	WAFD	2020-10-12	23	23.7	22.98	23.64	117496	{}	\N	2020-10-12 16:32:40.169612
78768	WBAI	2020-09-29	2.96	3.0999	2.92	3	67226	{}	\N	2020-10-12 16:32:41.548631
78769	WBAI	2020-09-30	3	3.09	2.945	3	46484	{}	\N	2020-10-12 16:32:41.558812
78770	WBAI	2020-10-01	2.96	3	2.9454	2.95	24184	{}	\N	2020-10-12 16:32:41.565951
78771	WBAI	2020-10-02	2.82	2.89	2.82	2.84	13745	{}	\N	2020-10-12 16:32:41.572478
78772	WBAI	2020-10-05	2.9101	2.9101	2.84	2.9	4366	{}	\N	2020-10-12 16:32:41.578562
78773	WBAI	2020-10-06	2.96	3	2.9	2.935	20379	{}	\N	2020-10-12 16:32:41.583703
78774	WBAI	2020-10-07	2.98	3.06	2.98	3	9984	{}	\N	2020-10-12 16:32:41.58775
78775	WBAI	2020-10-08	3	3.06	3	3.06	3668	{}	\N	2020-10-12 16:32:41.59216
78776	WBAI	2020-10-09	3.06	3.06	2.9299	2.98	41064	{}	\N	2020-10-12 16:32:41.596542
78777	WBAI	2020-10-12	3	3.01	2.88	2.8988	30916	{}	\N	2020-10-12 16:32:41.600651
78778	WBII	2020-09-28	24.08	24.12	24.075	24.12	4389	{}	\N	2020-10-12 16:32:42.708637
78779	WBII	2020-09-29	24.07	24.11	24.07	24.1	9495	{}	\N	2020-10-12 16:32:42.725137
78780	WBII	2020-09-30	23.97	23.97	23.87	23.925	12988	{}	\N	2020-10-12 16:32:42.732995
78781	WBII	2020-10-01	23.92	23.9734	23.92	23.9734	5894	{}	\N	2020-10-12 16:32:42.738982
78782	WBII	2020-10-02	23.86	23.8853	23.83	23.8853	8892	{}	\N	2020-10-12 16:32:42.745098
78783	WBII	2020-10-05	23.64	23.65	23.5623	23.5623	4812	{}	\N	2020-10-12 16:32:42.753425
78784	WBII	2020-10-06	23.77	23.83	23.71	23.71	13712	{}	\N	2020-10-12 16:32:42.758455
78785	WBII	2020-10-07	23.76	23.82	23.75	23.81	41316	{}	\N	2020-10-12 16:32:42.763167
78786	WBII	2020-10-08	23.8299	23.83	23.79	23.82	3826	{}	\N	2020-10-12 16:32:42.767556
78787	WBII	2020-10-09	23.8601	23.88	23.85	23.86	2389	{}	\N	2020-10-12 16:32:42.771917
78788	WBII	2020-10-12	23.97	23.98	23.97	23.98	680	{}	\N	2020-10-12 16:32:42.776232
78789	WFHY	2020-09-28	49.71	49.835	49.6064	49.7701	12017	{}	\N	2020-10-12 16:32:43.802195
78790	WFHY	2020-09-29	49.74	49.82	49.5636	49.81	7690	{}	\N	2020-10-12 16:32:43.816888
78791	WFHY	2020-09-30	49.8681	50.11	49.8453	50.055	11810	{}	\N	2020-10-12 16:32:43.824042
78792	WFHY	2020-10-01	50.0384	50.24	50.0384	50.22	11749	{}	\N	2020-10-12 16:32:43.830147
78793	WFHY	2020-10-02	49.95	50.12	49.92	50.12	12354	{}	\N	2020-10-12 16:32:43.835797
78794	WFHY	2020-10-05	50.08	50.43	50.08	50.42	8315	{}	\N	2020-10-12 16:32:43.841401
78795	WFHY	2020-10-06	50.45	50.64	50.42	50.42	15284	{}	\N	2020-10-12 16:32:43.845745
78796	WFHY	2020-10-07	50.38	50.66	50.38	50.58	12565	{}	\N	2020-10-12 16:32:43.850427
78797	WFHY	2020-10-08	50.58	50.64	50.5657	50.61	18969	{}	\N	2020-10-12 16:32:43.855491
78798	WFHY	2020-10-09	50.75	50.94	50.67	50.74	19505	{}	\N	2020-10-12 16:32:43.860844
78799	WFHY	2020-10-12	50.94	51	50.8413	50.995	13421	{}	\N	2020-10-12 16:32:43.866164
78800	WHLRD	2020-09-28	15	15	15	15	1905	{}	\N	2020-10-12 16:32:45.32361
78801	WHLRD	2020-09-29	15.2	15.4	14.9275	15.4	10723	{}	\N	2020-10-12 16:32:45.331719
78802	WHLRD	2020-09-30	15.25	15.25	15.1575	15.1755	3387	{}	\N	2020-10-12 16:32:45.337939
78803	WHLRD	2020-10-02	15.4001	15.45	15.287	15.39	3714	{}	\N	2020-10-12 16:32:45.342951
78804	WHLRD	2020-10-07	15.24	15.25	15.12	15.12	1010	{}	\N	2020-10-12 16:32:45.348739
78805	WIP	2020-09-28	53.99	53.99	53.85	53.85	9181	{}	\N	2020-10-12 16:32:46.293706
78806	WIP	2020-09-29	54.05	54.28	54.02	54.19	23485	{}	\N	2020-10-12 16:32:46.300509
78807	WIP	2020-09-30	54.2	54.46	54.19	54.235	26580	{}	\N	2020-10-12 16:32:46.306509
78808	WIP	2020-10-01	54.22	54.34	53.94	54.15	26280	{}	\N	2020-10-12 16:32:46.317341
78809	WIP	2020-10-02	54.15	54.43	54.15	54.285	15123	{}	\N	2020-10-12 16:32:46.325908
78810	WIP	2020-10-05	54.39	54.48	54.37	54.37	17762	{}	\N	2020-10-12 16:32:46.332687
78811	WIP	2020-10-06	54.29	54.44	54.0101	54.06	9872	{}	\N	2020-10-12 16:32:46.340184
78812	WIP	2020-10-07	54.26	54.26	54	54.01	13207	{}	\N	2020-10-12 16:32:46.345852
78813	WIP	2020-10-08	53.98	54.2041	53.71	53.98	29054	{}	\N	2020-10-12 16:32:46.351831
78814	WIP	2020-10-09	54.38	54.68	53.99	54.54	25524	{}	\N	2020-10-12 16:32:46.357773
78815	WIP	2020-10-12	54.51	54.7	54.51	54.68	8254	{}	\N	2020-10-12 16:32:46.362974
78816	WLFC	2020-09-28	19.09	19.09	18.57	18.63	9976	{}	\N	2020-10-12 16:32:49.69694
78817	WLFC	2020-09-29	18.44	18.95	18.44	18.95	1340	{}	\N	2020-10-12 16:32:49.709568
78818	WLFC	2020-09-30	18.552	19.0054	18.45	18.45	21887	{}	\N	2020-10-12 16:32:49.723433
78819	WLFC	2020-10-01	18.36	18.99	18.36	18.61	10433	{}	\N	2020-10-12 16:32:49.731981
78820	WLFC	2020-10-02	18.28	19.9899	18.15	18.67	21948	{}	\N	2020-10-12 16:32:49.737811
78821	WLFC	2020-10-05	18.38	19.03	18.38	18.67	15958	{}	\N	2020-10-12 16:32:49.745323
78822	WLFC	2020-10-06	18.82	19.25	18.67	19.07	6677	{}	\N	2020-10-12 16:32:49.749895
78823	WLFC	2020-10-07	19.1	19.49	18.83	19.2	7913	{}	\N	2020-10-12 16:32:49.75472
78824	WLFC	2020-10-08	19.25	20.49	19.25	20.32	11723	{}	\N	2020-10-12 16:32:49.759051
78825	WLFC	2020-10-09	20.31	20.49	20.0483	20.49	6104	{}	\N	2020-10-12 16:32:49.763101
78826	WLFC	2020-10-12	20.5	20.7	20.1	20.1	6907	{}	\N	2020-10-12 16:32:49.767548
78827	WMC	2020-09-28	2.1	2.19	2.1	2.12	1060289	{}	\N	2020-10-12 16:32:53.445365
78828	WMC	2020-09-29	2.11	2.18	2.03	2.07	1623566	{}	\N	2020-10-12 16:32:53.45918
78829	WMC	2020-09-30	2.07	2.079	2.01	2.04	3526582	{}	\N	2020-10-12 16:32:53.467323
78830	WMC	2020-10-01	2.03	2.03	1.99	2	3185139	{}	\N	2020-10-12 16:32:53.473839
78831	WMC	2020-10-02	1.95	2.12	1.9408	2.05	2595483	{}	\N	2020-10-12 16:32:53.479873
78832	WMC	2020-10-05	2.07	2.12	2.05	2.09	1246547	{}	\N	2020-10-12 16:32:53.485261
78833	WMC	2020-10-06	2.1	2.21	2.095	2.11	2476331	{}	\N	2020-10-12 16:32:53.491041
78834	WMC	2020-10-07	2.15	2.16	2.12	2.13	1367161	{}	\N	2020-10-12 16:32:53.496595
78835	WMC	2020-10-08	2.15	2.29	2.14	2.23	1797810	{}	\N	2020-10-12 16:32:53.502086
78836	WMC	2020-10-09	2.23	2.25	2.1711	2.18	838227	{}	\N	2020-10-12 16:32:53.507575
78837	WMC	2020-10-12	2.19	2.27	2.16	2.23	537702	{}	\N	2020-10-12 16:32:53.513046
78838	WOOD	2020-09-28	64.91	65.12	64.75	65.0065	18189	{}	\N	2020-10-12 16:32:54.518181
78839	WOOD	2020-09-29	65.16	65.315	64.7116	65	15098	{}	\N	2020-10-12 16:32:54.528882
78840	WOOD	2020-09-30	64.64	65.0872	64.235	64.65	15141	{}	\N	2020-10-12 16:32:54.536252
78841	WOOD	2020-10-01	65.235	65.63	64.935	65.63	23377	{}	\N	2020-10-12 16:32:54.543283
78842	WOOD	2020-10-02	64.59	66.1	64.59	66	18805	{}	\N	2020-10-12 16:32:54.549504
78843	WOOD	2020-10-05	66.43	67.31	66.43	67.31	17492	{}	\N	2020-10-12 16:32:54.55526
78844	WOOD	2020-10-06	67.45	67.45	66.1101	66.35	19952	{}	\N	2020-10-12 16:32:54.56055
78845	WOOD	2020-10-07	66.95	67.65	66.95	67.48	10264	{}	\N	2020-10-12 16:32:54.565221
78846	WOOD	2020-10-08	67.31	67.55	66.8801	67.45	12993	{}	\N	2020-10-12 16:32:54.571086
78847	WOOD	2020-10-09	67.91	68.134	67.515	67.55	13218	{}	\N	2020-10-12 16:32:54.575759
78848	WOOD	2020-10-12	67.87	68.43	67.87	68.37	4223	{}	\N	2020-10-12 16:32:54.581
78849	WASH	2020-09-28	31.52	32.05	31.3864	31.49	56739	{}	\N	2020-10-12 16:32:56.446386
78850	WASH	2020-09-29	31.49	31.6399	30.9	31.45	89338	{}	\N	2020-10-12 16:32:56.457956
78851	WASH	2020-09-30	31.08	31.5	30.5	30.66	79019	{}	\N	2020-10-12 16:32:56.464728
78852	WASH	2020-10-01	30.51	30.951	30.01	30.74	61708	{}	\N	2020-10-12 16:32:56.470628
78853	WASH	2020-10-02	30.01	31.53	30.01	31.48	60152	{}	\N	2020-10-12 16:32:56.476276
78854	WASH	2020-10-05	31.76	32.46	31.76	32.42	54853	{}	\N	2020-10-12 16:32:56.481849
78855	WASH	2020-10-06	32.65	33.605	32.56	32.63	51188	{}	\N	2020-10-12 16:32:56.486379
78856	WASH	2020-10-07	31.99	33.86	31.99	33.68	69641	{}	\N	2020-10-12 16:32:56.491113
78857	WASH	2020-10-08	34.12	34.38	33.7	34.26	68273	{}	\N	2020-10-12 16:32:56.495939
78858	WASH	2020-10-09	34.39	34.39	33.88	33.98	68097	{}	\N	2020-10-12 16:32:56.500422
78859	WASH	2020-10-12	33.92	34.39	33.92	34.06	13889	{}	\N	2020-10-12 16:32:56.505268
78860	WBS	2020-09-28	25.36	26.03	25.36	25.62	528520	{}	\N	2020-10-12 16:32:58.135399
78861	WBS	2020-09-29	25.61	25.85	25.2	25.64	1796714	{}	\N	2020-10-12 16:32:58.14865
78862	WBS	2020-09-30	25.79	26.83	25.79	26.41	2277693	{}	\N	2020-10-12 16:32:58.158213
78863	WBS	2020-10-01	26.47	26.84	25.95	26.39	949337	{}	\N	2020-10-12 16:32:58.165262
78864	WBS	2020-10-02	25.92	27.4	25.5901	27.34	683560	{}	\N	2020-10-12 16:32:58.171388
78865	WBS	2020-10-05	27.71	28.85	27.52	28.76	1075500	{}	\N	2020-10-12 16:32:58.177162
78866	WBS	2020-10-06	29.21	30.11	28.51	28.63	1020632	{}	\N	2020-10-12 16:32:58.182711
78867	WBS	2020-10-07	29.03	29.91	28.74	29.33	1188231	{}	\N	2020-10-12 16:32:58.18846
78868	WBS	2020-10-08	29.77	30.17	29.4428	30.07	710777	{}	\N	2020-10-12 16:32:58.193653
78869	WBS	2020-10-09	30.37	30.51	29.71	30.15	644450	{}	\N	2020-10-12 16:32:58.198736
78870	WBS	2020-10-12	29.96	30.48	29.96	30.29	135340	{}	\N	2020-10-12 16:32:58.203904
78871	WILC	2020-09-28	18.99	18.99	18.99	18.99	201	{}	\N	2020-10-12 16:32:59.893553
78872	WILC	2020-09-29	19.06	19.4899	19.0002	19.06	3814	{}	\N	2020-10-12 16:32:59.904901
78873	WILC	2020-09-30	19.49	19.73	19.0101	19.73	2956	{}	\N	2020-10-12 16:32:59.913015
78874	WILC	2020-10-01	20	20	19.3763	19.3763	2048	{}	\N	2020-10-12 16:32:59.918884
78875	WILC	2020-10-02	18.9577	19.45	18.9577	19.45	1417	{}	\N	2020-10-12 16:32:59.923801
78876	WILC	2020-10-05	19.95	19.95	19.3284	19.71	3214	{}	\N	2020-10-12 16:32:59.928604
78877	WILC	2020-10-06	19.71	19.71	19.01	19.69	1623	{}	\N	2020-10-12 16:32:59.932779
78878	WILC	2020-10-07	19.6	19.6	19.46	19.51	2258	{}	\N	2020-10-12 16:32:59.936967
78879	WILC	2020-10-08	19.5	19.5	18.9	19.4	1309	{}	\N	2020-10-12 16:32:59.941082
78880	WILC	2020-10-09	19.49	19.5	19.0783	19.08	1597	{}	\N	2020-10-12 16:32:59.945214
78881	WILC	2020-10-12	19.37	19.37	19.11	19.11	957	{}	\N	2020-10-12 16:32:59.949321
78882	WK	2020-09-28	56.9	58.32	55.9901	57.53	148120	{}	\N	2020-10-12 16:33:00.948924
78883	WK	2020-09-29	57.53	58.27	56.67	56.93	321175	{}	\N	2020-10-12 16:33:00.960671
78884	WK	2020-09-30	56.92	57.59	55.505	55.76	655066	{}	\N	2020-10-12 16:33:00.968237
78885	WK	2020-10-01	56.29	56.765	55.53	56.65	411455	{}	\N	2020-10-12 16:33:00.974462
78886	WK	2020-10-02	55.51	56.9	55.09	55.52	295079	{}	\N	2020-10-12 16:33:00.981186
78887	WK	2020-10-05	55.85	56.73	55.64	56.51	260128	{}	\N	2020-10-12 16:33:00.986012
78888	WK	2020-10-06	56.69	58.72	56.69	57.57	386516	{}	\N	2020-10-12 16:33:00.990786
78889	WK	2020-10-07	57.93	59.61	57.93	59.11	373124	{}	\N	2020-10-12 16:33:00.995518
78890	WK	2020-10-08	59.78	59.78	58.81	59.33	321413	{}	\N	2020-10-12 16:33:01.000232
78891	WK	2020-10-09	59.88	60.92	59.3	60.45	336419	{}	\N	2020-10-12 16:33:01.004846
78892	WK	2020-10-12	61	61.25	60.07	61.185	141502	{}	\N	2020-10-12 16:33:01.009686
78893	WLL	2020-09-28	17.76	19.34	17.76	18.62	525368	{}	\N	2020-10-12 16:33:02.300869
78894	WLL	2020-09-29	18.4	18.54	17.235	17.47	1675685	{}	\N	2020-10-12 16:33:02.315881
78895	WLL	2020-09-30	17.5	18.18	17.16	17.29	955257	{}	\N	2020-10-12 16:33:02.324104
78896	WLL	2020-10-01	17.18	17.38	16.09	16.17	1682699	{}	\N	2020-10-12 16:33:02.330995
78897	WLL	2020-10-02	15.48	17.19	15.48	16.89	1274854	{}	\N	2020-10-12 16:33:02.337778
78898	WLL	2020-10-05	17.14	17.74	17.06	17.62	952479	{}	\N	2020-10-12 16:33:02.343859
78899	WLL	2020-10-06	17.99	18.79	17.23	17.25	1538995	{}	\N	2020-10-12 16:33:02.348846
78900	WLL	2020-10-07	17.5	18	17.11	17.86	667464	{}	\N	2020-10-12 16:33:02.353793
78901	WLL	2020-10-08	17.91	19.17	17.78	19.13	1053264	{}	\N	2020-10-12 16:33:02.358764
78902	WLL	2020-10-09	19.31	19.49	18.02	18.71	802645	{}	\N	2020-10-12 16:33:02.363725
78903	WLL	2020-10-12	18.43	18.62	18.2	18.345	222298	{}	\N	2020-10-12 16:33:02.369185
78904	WMT	2020-09-28	137.16	138.05	136.72	137.25	7403354	{}	\N	2020-10-12 16:33:04.277602
78905	WMT	2020-09-29	137.25	138.14	136.39	137.14	20906831	{}	\N	2020-10-12 16:33:04.291012
78906	WMT	2020-09-30	137.39	141.74	137.2014	139.91	22711523	{}	\N	2020-10-12 16:33:04.299872
78907	WMT	2020-10-01	140.8	144.3799	140.26	143.08	28915923	{}	\N	2020-10-12 16:33:04.306045
78908	WMT	2020-10-02	142.34	143.1	140.1297	140.5	16043140	{}	\N	2020-10-12 16:33:04.311834
78909	WMT	2020-10-05	141.29	142.1899	141.065	141.8	9538589	{}	\N	2020-10-12 16:33:04.316879
78910	WMT	2020-10-06	141.87	142.9399	140.25	140.63	13908055	{}	\N	2020-10-12 16:33:04.321995
78911	WMT	2020-10-07	141.26	141.55	140.25	140.89	11327542	{}	\N	2020-10-12 16:33:04.32689
78912	WMT	2020-10-08	141.51	142.09	141.12	141.36	9894672	{}	\N	2020-10-12 16:33:04.331819
78913	WMT	2020-10-09	141.66	143.34	141.56	142.78	9670954	{}	\N	2020-10-12 16:33:04.336857
78914	WMT	2020-10-12	143.02	145.39	143.02	145.255	6454736	{}	\N	2020-10-12 16:33:04.341887
78915	WPM	2020-09-28	49.06	49.13	48.03	48.29	2552576	{}	\N	2020-10-12 16:33:05.96122
78916	WPM	2020-09-29	49.01	49.77	48.67	49.05	3035645	{}	\N	2020-10-12 16:33:05.972027
78917	WPM	2020-09-30	48.6	49.38	47.951	49.07	2401144	{}	\N	2020-10-12 16:33:05.979392
78918	WPM	2020-10-01	49.86	50.83	49.16	50.18	3469267	{}	\N	2020-10-12 16:33:05.986041
78919	WPM	2020-10-02	50.06	50.31	48.84	48.97	2248846	{}	\N	2020-10-12 16:33:05.991737
78920	WPM	2020-10-05	49.34	50.42	49.32	49.83	1988543	{}	\N	2020-10-12 16:33:05.996882
78921	WPM	2020-10-06	49.83	50.12	47.28	47.34	2666434	{}	\N	2020-10-12 16:33:06.002161
78922	WPM	2020-10-07	47.8	48.33	47.44	47.79	2448118	{}	\N	2020-10-12 16:33:06.007124
78923	WPM	2020-10-08	48.14	48.84	47.82	48.08	2239677	{}	\N	2020-10-12 16:33:06.012017
78924	WPM	2020-10-09	49.16	51.05	49.13	51.03	4742755	{}	\N	2020-10-12 16:33:06.01709
78925	WPM	2020-10-12	51.03	51.1	50.2	50.92	864512	{}	\N	2020-10-12 16:33:06.02215
78926	WRN	2020-09-28	1.22	1.25	1.18	1.22	212660	{}	\N	2020-10-12 16:33:07.223265
78927	WRN	2020-09-29	1.22	1.25	1.2	1.25	133067	{}	\N	2020-10-12 16:33:07.234659
78928	WRN	2020-09-30	1.23	1.2429	1.2061	1.21	133892	{}	\N	2020-10-12 16:33:07.241716
78929	WRN	2020-10-01	1.21	1.21	1.17	1.18	179677	{}	\N	2020-10-12 16:33:07.248773
78930	WRN	2020-10-02	1.15	1.1899	1.1	1.1	243613	{}	\N	2020-10-12 16:33:07.254754
78931	WRN	2020-10-05	1.13	1.17	1.1	1.12	274447	{}	\N	2020-10-12 16:33:07.259798
78932	WRN	2020-10-06	1.12	1.17	1.07	1.09	195074	{}	\N	2020-10-12 16:33:07.264308
78933	WRN	2020-10-07	1.12	1.17	1.09	1.15	207376	{}	\N	2020-10-12 16:33:07.269246
78934	WRN	2020-10-08	1.15	1.17	1.13	1.155	100751	{}	\N	2020-10-12 16:33:07.273854
78935	WRN	2020-10-09	1.17	1.2498	1.165	1.24	279169	{}	\N	2020-10-12 16:33:07.278153
78936	WRN	2020-10-12	1.27	1.32	1.255	1.32	272269	{}	\N	2020-10-12 16:33:07.283388
78937	WSR	2020-09-28	5.97	6.26	5.93	6.19	352739	{}	\N	2020-10-12 16:33:08.179306
78938	WSR	2020-09-29	6.16	6.1784	5.88	6	515748	{}	\N	2020-10-12 16:33:08.190031
78939	WSR	2020-09-30	6	6.18	5.93	6	554657	{}	\N	2020-10-12 16:33:08.196724
78940	WSR	2020-10-01	6.06	6.22	5.95	6.21	508766	{}	\N	2020-10-12 16:33:08.202208
78941	WSR	2020-10-02	5.95	6.4	5.95	6.38	746446	{}	\N	2020-10-12 16:33:08.206924
78942	WSR	2020-10-05	6.31	6.43	6.21	6.32	423481	{}	\N	2020-10-12 16:33:08.211392
78943	WSR	2020-10-06	6.5	6.63	6.3101	6.34	593976	{}	\N	2020-10-12 16:33:08.215376
78944	WSR	2020-10-07	6.39	6.41	6.16	6.26	455792	{}	\N	2020-10-12 16:33:08.219757
78945	WSR	2020-10-08	6.31	6.44	6.2676	6.37	329110	{}	\N	2020-10-12 16:33:08.224095
78946	WSR	2020-10-09	6.42	6.42	6.18	6.24	356343	{}	\N	2020-10-12 16:33:08.228391
78947	WSR	2020-10-12	6.21	6.32	6.14	6.24	171426	{}	\N	2020-10-12 16:33:08.232666
78948	WTM	2020-09-28	768.24	787.8716	768.24	775.35	28435	{}	\N	2020-10-12 16:33:09.361842
78949	WTM	2020-09-29	768.87	795.23	765.49	775.02	59696	{}	\N	2020-10-12 16:33:09.370309
78950	WTM	2020-09-30	782.11	782.45	773.1666	779	26985	{}	\N	2020-10-12 16:33:09.37622
78951	WTM	2020-10-01	780.6	798.34	778.11	785.87	59584	{}	\N	2020-10-12 16:33:09.38193
78952	WTM	2020-10-02	783	805.9908	783	790.26	40923	{}	\N	2020-10-12 16:33:09.386882
78953	WTM	2020-10-05	795.33	808.71	795.33	804.03	43498	{}	\N	2020-10-12 16:33:09.391722
78954	WTM	2020-10-06	805.77	850	805.77	842.1	51562	{}	\N	2020-10-12 16:33:09.396035
78955	WTM	2020-10-07	849.53	868.75	847.79	858.2	48251	{}	\N	2020-10-12 16:33:09.399995
78956	WTM	2020-10-08	875.285	876.72	862.21	862.21	20614	{}	\N	2020-10-12 16:33:09.404498
78957	WTM	2020-10-09	873	878.05	873	875.14	21140	{}	\N	2020-10-12 16:33:09.408667
78958	WTM	2020-10-12	878.58	906.12	875.4	906	13276	{}	\N	2020-10-12 16:33:09.412767
78959	WETF	2020-09-28	3.2	3.39	3.19	3.24	719454	{}	\N	2020-10-12 16:33:11.997944
78960	WETF	2020-09-29	3.22	3.28	3.195	3.24	597967	{}	\N	2020-10-12 16:33:12.010015
78961	WETF	2020-09-30	3.28	3.33	3.18	3.2	819810	{}	\N	2020-10-12 16:33:12.016934
78962	WETF	2020-10-01	3.18	3.26	3.18	3.25	513651	{}	\N	2020-10-12 16:33:12.023143
78963	WETF	2020-10-02	3.18	3.39	3.15	3.31	2183745	{}	\N	2020-10-12 16:33:12.028416
78964	WETF	2020-10-05	3.33	3.535	3.33	3.46	624721	{}	\N	2020-10-12 16:33:12.033587
78965	WETF	2020-10-06	3.5	3.75	3.42	3.66	1392477	{}	\N	2020-10-12 16:33:12.038169
78966	WETF	2020-10-07	3.73	3.8	3.605	3.74	1000436	{}	\N	2020-10-12 16:33:12.045277
78967	WETF	2020-10-08	3.81	4.02	3.72	3.99	1102975	{}	\N	2020-10-12 16:33:12.052654
78968	WETF	2020-10-09	3.83	3.98	3.83	3.97	624977	{}	\N	2020-10-12 16:33:12.060213
78969	WETF	2020-10-12	3.97	4.085	3.88	3.98	408682	{}	\N	2020-10-12 16:33:12.066176
78970	WIFI	2020-09-28	10.46	10.699	9.63	9.78	725609	{}	\N	2020-10-12 16:33:14.219964
78971	WIFI	2020-09-29	9.81	10.05	9.62	9.88	398489	{}	\N	2020-10-12 16:33:14.229257
78972	WIFI	2020-09-30	9.92	10.58	9.89	10.195	1002482	{}	\N	2020-10-12 16:33:14.236529
78973	WIFI	2020-10-01	10.19	10.37	9.99	10.2	373228	{}	\N	2020-10-12 16:33:14.243527
78974	WIFI	2020-10-02	9.95	10.42	9.735	10.235	417475	{}	\N	2020-10-12 16:33:14.249053
78975	WIFI	2020-10-05	10.34	10.4799	9.95	10.21	255718	{}	\N	2020-10-12 16:33:14.253812
78976	WIFI	2020-10-06	10.24	10.49	9.95	9.96	314420	{}	\N	2020-10-12 16:33:14.258271
78977	WIFI	2020-10-07	9.97	10.2	9.865	10.12	281478	{}	\N	2020-10-12 16:33:14.262847
78978	WIFI	2020-10-08	10.24	10.34	10.0869	10.17	208747	{}	\N	2020-10-12 16:33:14.267047
78979	WIFI	2020-10-09	10.27	10.36	10.02	10.3	288662	{}	\N	2020-10-12 16:33:14.271513
78980	WIFI	2020-10-12	10.38	10.62	10.31	10.47	182396	{}	\N	2020-10-12 16:33:14.276339
78981	WIT	2020-09-28	4.69	4.72	4.63	4.67	1542902	{}	\N	2020-10-12 16:33:15.875035
78982	WIT	2020-09-29	4.64	4.69	4.63	4.64	3524680	{}	\N	2020-10-12 16:33:15.886576
78983	WIT	2020-09-30	4.72	4.765	4.7	4.7	4091045	{}	\N	2020-10-12 16:33:15.894153
78984	WIT	2020-10-01	4.73	4.73	4.645	4.68	2727896	{}	\N	2020-10-12 16:33:15.90134
78985	WIT	2020-10-02	4.65	4.66	4.62	4.63	2891250	{}	\N	2020-10-12 16:33:15.906975
78986	WIT	2020-10-05	4.94	4.96	4.88	4.9	6178301	{}	\N	2020-10-12 16:33:15.912289
78987	WIT	2020-10-06	4.87	4.89	4.74	4.76	5803990	{}	\N	2020-10-12 16:33:15.917238
78988	WIT	2020-10-07	4.93	5.055	4.87	4.97	7528386	{}	\N	2020-10-12 16:33:15.922104
78989	WIT	2020-10-08	5.28	5.32	5.175	5.23	5450526	{}	\N	2020-10-12 16:33:15.927099
78990	WIT	2020-10-09	5.42	5.4978	5.39	5.4	6523788	{}	\N	2020-10-12 16:33:15.932303
78991	WIT	2020-10-12	5.45	5.46	5.39	5.4	1828539	{}	\N	2020-10-12 16:33:15.937706
78992	WLKP	2020-09-28	19.62	19.9	19.132	19.33	46191	{}	\N	2020-10-12 16:33:17.161376
78993	WLKP	2020-09-29	19.35	19.35	19	19.14	35115	{}	\N	2020-10-12 16:33:17.171684
78994	WLKP	2020-09-30	19.18	19.28	18.74	18.75	83912	{}	\N	2020-10-12 16:33:17.179074
78995	WLKP	2020-10-01	18.86	18.97	18.47	18.54	85624	{}	\N	2020-10-12 16:33:17.185646
78996	WLKP	2020-10-02	18.08	19.64	18.05	19.53	163606	{}	\N	2020-10-12 16:33:17.191178
78997	WLKP	2020-10-05	19.49	19.97	19.48	19.61	135802	{}	\N	2020-10-12 16:33:17.196237
78998	WLKP	2020-10-06	19.805	19.97	19.48	19.65	75908	{}	\N	2020-10-12 16:33:17.201222
78999	WLKP	2020-10-07	19.66	19.97	19.47	19.77	375826	{}	\N	2020-10-12 16:33:17.206092
79000	WLKP	2020-10-08	19.8	20.1499	19.74	19.88	99366	{}	\N	2020-10-12 16:33:17.210835
79001	WLKP	2020-10-09	19.9	19.9	19.71	19.71	61588	{}	\N	2020-10-12 16:33:17.215629
79002	WLKP	2020-10-12	19.66	20.14	19.5947	20.07	28616	{}	\N	2020-10-12 16:33:17.220611
79003	WOW	2020-09-28	5.25	5.31	5.17	5.21	161499	{}	\N	2020-10-12 16:33:18.591603
79004	WOW	2020-09-29	5.21	5.25	5.04	5.12	270059	{}	\N	2020-10-12 16:33:18.60422
79005	WOW	2020-09-30	5.14	5.29	5.1	5.19	361446	{}	\N	2020-10-12 16:33:18.612259
79006	WOW	2020-10-01	5.25	5.27	5.165	5.23	266301	{}	\N	2020-10-12 16:33:18.618748
79007	WOW	2020-10-02	5.12	5.355	5.07	5.3	351658	{}	\N	2020-10-12 16:33:18.628262
79008	WOW	2020-10-05	5.33	5.48	5.28	5.45	347731	{}	\N	2020-10-12 16:33:18.633895
79009	WOW	2020-10-06	5.56	5.56	5.19	5.21	389597	{}	\N	2020-10-12 16:33:18.6391
79010	WOW	2020-10-07	5.25	5.375	5.19	5.29	429993	{}	\N	2020-10-12 16:33:18.644331
79011	WOW	2020-10-08	5.32	5.33	5.12	5.31	277851	{}	\N	2020-10-12 16:33:18.649893
79012	WOW	2020-10-09	5.35	5.39	5.25	5.27	242174	{}	\N	2020-10-12 16:33:18.655309
79013	WOW	2020-10-12	5.27	5.4	5.24	5.395	83227	{}	\N	2020-10-12 16:33:18.660865
79014	WRLD	2020-09-28	96.63	100.78	96.63	99.97	66606	{}	\N	2020-10-12 16:33:22.324951
79015	WRLD	2020-09-29	99.77	103.21	99.73	101.61	97240	{}	\N	2020-10-12 16:33:22.334167
79016	WRLD	2020-09-30	102.13	106.935	102.13	105.55	90815	{}	\N	2020-10-12 16:33:22.340682
79017	WRLD	2020-10-01	105.27	109.33	104.49	109.33	65292	{}	\N	2020-10-12 16:33:22.346961
79018	WRLD	2020-10-02	106.58	110.2	106.58	108.98	113172	{}	\N	2020-10-12 16:33:22.352728
79019	WRLD	2020-10-05	110.7	111.36	109.02	109.37	57853	{}	\N	2020-10-12 16:33:22.35735
79020	WRLD	2020-10-06	110.44	110.88	106.37	107.65	100923	{}	\N	2020-10-12 16:33:22.362223
79021	WRLD	2020-10-07	108.91	109.44	104.17	106.01	53212	{}	\N	2020-10-12 16:33:22.367028
79022	WRLD	2020-10-08	107.15	108.8427	105.82	107.34	42769	{}	\N	2020-10-12 16:33:22.371226
79023	WRLD	2020-10-09	108.83	109.88	105.13	105.13	40612	{}	\N	2020-10-12 16:33:22.375375
79024	WRLD	2020-10-12	105.69	107.075	103.67	104.205	18404	{}	\N	2020-10-12 16:33:22.380152
79025	WSTL	2020-09-28	1.15	1.28	1.14	1.27	371369	{}	\N	2020-10-12 16:33:23.646409
79026	WSTL	2020-09-29	1.29	1.35	1.2	1.24	585641	{}	\N	2020-10-12 16:33:23.658354
79027	WSTL	2020-09-30	1.3	1.33	1.16	1.28	745077	{}	\N	2020-10-12 16:33:23.66631
79028	WSTL	2020-10-01	1.25	1.32	1.2	1.23	342940	{}	\N	2020-10-12 16:33:23.673165
79029	WSTL	2020-10-02	1.25	1.25	1.0199	1.12	273716	{}	\N	2020-10-12 16:33:23.679335
79030	WSTL	2020-10-05	1.16	1.1877	0.89	0.9196	178338	{}	\N	2020-10-12 16:33:23.685004
79031	WSTL	2020-10-06	0.921	0.9841	0.8352	0.9227	135636	{}	\N	2020-10-12 16:33:23.690153
79032	WSTL	2020-10-07	0.92	0.92	0.8612	0.8922	328833	{}	\N	2020-10-12 16:33:23.695127
79033	WSTL	2020-10-08	0.85	0.909003	0.833	0.885	314327	{}	\N	2020-10-12 16:33:23.700648
79034	WSTL	2020-10-09	0.885	0.885	0.77	0.7841	332297	{}	\N	2020-10-12 16:33:23.705498
79035	WSTL	2020-10-12	0.77	0.8383	0.76	0.8159	35055	{}	\N	2020-10-12 16:33:23.710903
79036	WTT	2020-09-28	1.42	1.4201	1.3601	1.4	10118	{}	\N	2020-10-12 16:33:24.740482
79037	WTT	2020-09-29	1.4	1.4	1.35	1.39	10541	{}	\N	2020-10-12 16:33:24.751346
79038	WTT	2020-09-30	1.38	1.407	1.38	1.38	6209	{}	\N	2020-10-12 16:33:24.758835
79039	WTT	2020-10-01	1.43	1.43	1.39	1.4	24501	{}	\N	2020-10-12 16:33:24.765507
79040	WTT	2020-10-02	1.29	1.4227	1.29	1.35	60098	{}	\N	2020-10-12 16:33:24.771942
79041	WTT	2020-10-05	1.35	1.43	1.35	1.42	15487	{}	\N	2020-10-12 16:33:24.777231
79042	WTT	2020-10-06	1.39	1.48	1.39	1.42	18577	{}	\N	2020-10-12 16:33:24.782182
79043	WTT	2020-10-07	1.44	1.5	1.43	1.49	25417	{}	\N	2020-10-12 16:33:24.787444
79044	WTT	2020-10-08	1.51	1.51	1.4638	1.5	8644	{}	\N	2020-10-12 16:33:24.792584
79045	WTT	2020-10-09	1.5	1.51	1.4826	1.5	10984	{}	\N	2020-10-12 16:33:24.797583
79046	WTT	2020-10-12	1.52	1.52	1.5	1.5132	3045	{}	\N	2020-10-12 16:33:24.802687
79047	WWR	2020-09-28	1.6165	2.23	1.52	1.97	82728729	{}	\N	2020-10-12 16:33:27.970757
79048	WWR	2020-09-29	2.59	2.65	2.06	2.3	30707238	{}	\N	2020-10-12 16:33:27.981583
79049	WWR	2020-09-30	2.04	2.53	2.0011	2.51	5985410	{}	\N	2020-10-12 16:33:27.988109
79050	WWR	2020-10-01	3.32	3.34	2.65	2.96	29848199	{}	\N	2020-10-12 16:33:27.993955
79051	WWR	2020-10-02	2.7	5.62	2.67	4.42	102959327	{}	\N	2020-10-12 16:33:27.999183
79052	WWR	2020-10-05	4.82	9.2	4.6	8.28	202614809	{}	\N	2020-10-12 16:33:28.004168
79053	WWR	2020-10-06	12.5	14.5	9.3	11.8	129469394	{}	\N	2020-10-12 16:33:28.009196
79054	WWR	2020-10-07	12.17	12.88	10.13	11.72	36222379	{}	\N	2020-10-12 16:33:28.013475
79055	WWR	2020-10-08	10.65	11.25	7.61	7.8	25022951	{}	\N	2020-10-12 16:33:28.017967
79056	WWR	2020-10-09	9	9.12	7.07	8	29010226	{}	\N	2020-10-12 16:33:28.022629
79057	WWR	2020-10-12	8	8.1	7.5	7.81	5283358	{}	\N	2020-10-12 16:33:28.027497
79058	XBIT	2020-09-28	19.01	19.43	18.75	19.39	70599	{}	\N	2020-10-12 16:33:28.966796
79059	XBIT	2020-09-29	19.17	19.32	18.5907	18.96	65447	{}	\N	2020-10-12 16:33:28.976795
79060	XBIT	2020-09-30	19.02	19.42	18.72	19.09	75838	{}	\N	2020-10-12 16:33:28.983481
79061	XBIT	2020-10-01	19.2	19.283	18.7	18.84	60389	{}	\N	2020-10-12 16:33:28.989909
79062	XBIT	2020-10-02	18.43	18.74	18.11	18.21	75021	{}	\N	2020-10-12 16:33:28.995315
79063	XBIT	2020-10-05	18.28	19.85	18.28	19.8	121942	{}	\N	2020-10-12 16:33:29.00014
79064	XBIT	2020-10-06	20	20	19.2666	19.36	45482	{}	\N	2020-10-12 16:33:29.004839
79065	XBIT	2020-10-07	19.37	20.1	19.3357	19.65	80313	{}	\N	2020-10-12 16:33:29.009536
79066	XBIT	2020-10-08	19.95	19.95	19.1	19.36	65699	{}	\N	2020-10-12 16:33:29.014926
79067	XBIT	2020-10-09	19.51	19.8999	19.37	19.74	70553	{}	\N	2020-10-12 16:33:29.020084
79068	XBIT	2020-10-12	19.72	19.935	19.21	19.44	21051	{}	\N	2020-10-12 16:33:29.025755
79069	WFIG	2020-09-28	54.82	54.9075	54.77	54.9075	2423	{}	\N	2020-10-12 16:33:29.973064
79070	WFIG	2020-09-29	54.99	55.09	54.8957	54.8957	8562	{}	\N	2020-10-12 16:33:29.985573
79071	WFIG	2020-09-30	54.83	54.83	54.7	54.7455	5081	{}	\N	2020-10-12 16:33:29.993213
79072	WFIG	2020-10-01	54.815	54.9745	54.8105	54.9745	571	{}	\N	2020-10-12 16:33:29.999388
79073	WFIG	2020-10-02	54.89	54.9576	54.82	54.9576	4808	{}	\N	2020-10-12 16:33:30.005537
79074	WFIG	2020-10-05	54.86	54.91	54.7515	54.7515	3989	{}	\N	2020-10-12 16:33:30.011127
79075	WFIG	2020-10-06	54.92	55.01	54.775	54.9	8946	{}	\N	2020-10-12 16:33:30.016239
79076	WFIG	2020-10-07	54.93	54.94	54.705	54.705	4205	{}	\N	2020-10-12 16:33:30.021378
79077	WFIG	2020-10-08	54.85	54.85	54.77	54.81	11739	{}	\N	2020-10-12 16:33:30.026667
79078	WFIG	2020-10-09	54.88	54.9397	54.88	54.9397	1124	{}	\N	2020-10-12 16:33:30.031698
79079	WFIG	2020-10-12	55.25	55.25	55.25	55.25	30788	{}	\N	2020-10-12 16:33:30.036481
79080	WHR	2020-09-28	180.91	185.31	179.546	185	730590	{}	\N	2020-10-12 16:33:31.755598
79081	WHR	2020-09-29	184.09	184.78	180.68	180.75	906545	{}	\N	2020-10-12 16:33:31.769283
79082	WHR	2020-09-30	183.9	186.85	181.8049	183.89	1740209	{}	\N	2020-10-12 16:33:31.777851
79083	WHR	2020-10-01	185.16	185.535	181.4668	183.19	1439067	{}	\N	2020-10-12 16:33:31.784827
79084	WHR	2020-10-02	179.6	185.46	179.32	184.21	893811	{}	\N	2020-10-12 16:33:31.79181
79085	WHR	2020-10-05	186	193.85	185.175	192.99	2190748	{}	\N	2020-10-12 16:33:31.797666
79086	WHR	2020-10-06	193.72	195.07	189.72	190.62	1767733	{}	\N	2020-10-12 16:33:31.802995
79087	WHR	2020-10-07	193.33	197.47	193.03	196.15	1329584	{}	\N	2020-10-12 16:33:31.808326
79088	WHR	2020-10-08	197.09	200.4095	196.92	200.19	1666230	{}	\N	2020-10-12 16:33:31.813971
79089	WHR	2020-10-09	201.84	205.2	200.58	201.4	1664993	{}	\N	2020-10-12 16:33:31.819163
79090	WHR	2020-10-12	202.73	203.24	199.7	200.74	518146	{}	\N	2020-10-12 16:33:31.82464
79091	WIX	2020-09-28	253.99	257.76	250.54	254.67	307073	{}	\N	2020-10-12 16:33:33.663352
79092	WIX	2020-09-29	253.38	260	253	255.5	335752	{}	\N	2020-10-12 16:33:33.672932
79093	WIX	2020-09-30	254	260.705	252.55	254.85	447476	{}	\N	2020-10-12 16:33:33.680272
79094	WIX	2020-10-01	259	270	256.72	269.45	536104	{}	\N	2020-10-12 16:33:33.688579
79095	WIX	2020-10-02	260.02	270	250.35	253.57	695538	{}	\N	2020-10-12 16:33:33.694374
79096	WIX	2020-10-05	258.18	269.16	253.95	267.91	668962	{}	\N	2020-10-12 16:33:33.700083
79097	WIX	2020-10-06	267	274.5292	262.6	265.38	517625	{}	\N	2020-10-12 16:33:33.705246
79098	WIX	2020-10-07	268.6	282.91	268	281.81	598387	{}	\N	2020-10-12 16:33:33.710833
79099	WIX	2020-10-08	287.76	291.96	276.51	279.67	726801	{}	\N	2020-10-12 16:33:33.717541
79100	WIX	2020-10-09	283.9	293.125	281.02	291.82	544973	{}	\N	2020-10-12 16:33:33.724118
79101	WIX	2020-10-12	296	296.02	286.93	289.98	238158	{}	\N	2020-10-12 16:33:33.729651
79102	WMB	2020-09-28	20.19	20.64	20.025	20.4	7836223	{}	\N	2020-10-12 16:33:35.022262
79103	WMB	2020-09-29	20.45	20.63	20.09	20.26	14131452	{}	\N	2020-10-12 16:33:35.034822
79104	WMB	2020-09-30	20.37	20.5	19.63	19.65	22765208	{}	\N	2020-10-12 16:33:35.042496
79105	WMB	2020-10-01	19.46	19.605	18.655	18.78	28652220	{}	\N	2020-10-12 16:33:35.048182
79106	WMB	2020-10-02	18.44	19.27	18.31	19.09	23223801	{}	\N	2020-10-12 16:33:35.0537
79107	WMB	2020-10-05	19.32	19.69	19.175	19.54	12231156	{}	\N	2020-10-12 16:33:35.058792
79108	WMB	2020-10-06	19.79	19.88	19.28	19.33	22323420	{}	\N	2020-10-12 16:33:35.063492
79109	WMB	2020-10-07	19.46	19.58	19.14	19.39	14554459	{}	\N	2020-10-12 16:33:35.06794
79110	WMB	2020-10-08	19.47	20.01	19.29	19.99	13319887	{}	\N	2020-10-12 16:33:35.072502
79111	WMB	2020-10-09	20.05	20.11	19.34	19.42	19805788	{}	\N	2020-10-12 16:33:35.077251
79112	WMB	2020-10-12	19.54	19.56	19.26	19.415	10039578	{}	\N	2020-10-12 16:33:35.082009
79113	WPRT	2020-09-28	1.63	1.76	1.56	1.72	2737686	{}	\N	2020-10-12 16:33:36.521995
79114	WPRT	2020-09-29	1.76	1.78	1.65	1.7	1480906	{}	\N	2020-10-12 16:33:36.536115
79115	WPRT	2020-09-30	1.68	1.75	1.57	1.63	1794377	{}	\N	2020-10-12 16:33:36.544214
79116	WPRT	2020-10-01	1.62	1.71	1.57	1.7	1655688	{}	\N	2020-10-12 16:33:36.551732
79117	WPRT	2020-10-02	1.61	1.75	1.61	1.75	1687851	{}	\N	2020-10-12 16:33:36.5582
79118	WPRT	2020-10-05	1.75	1.895	1.7301	1.85	2593983	{}	\N	2020-10-12 16:33:36.564284
79119	WPRT	2020-10-06	1.86	1.96	1.83	1.89	2513056	{}	\N	2020-10-12 16:33:36.570002
79120	WPRT	2020-10-07	1.9	2.15	1.89	2.12	4838268	{}	\N	2020-10-12 16:33:36.575559
79121	WPRT	2020-10-08	2.14	2.25	2.12	2.19	3089576	{}	\N	2020-10-12 16:33:36.581102
79122	WPRT	2020-10-09	2.18	2.305	2.06	2.08	3643746	{}	\N	2020-10-12 16:33:36.586333
79123	WPRT	2020-10-12	1.92	2.07	1.92	1.99	1070199	{}	\N	2020-10-12 16:33:36.592259
79124	WST	2020-09-28	271.97	276.3	270	272.39	382231	{}	\N	2020-10-12 16:33:40.241786
79125	WST	2020-09-29	273.98	277.14	273.49	273.65	490921	{}	\N	2020-10-12 16:33:40.253601
79126	WST	2020-09-30	273.78	277.8	272.57	274.9	929958	{}	\N	2020-10-12 16:33:40.261367
79127	WST	2020-10-01	278.76	280.5	274.55	275.79	798218	{}	\N	2020-10-12 16:33:40.267908
79128	WST	2020-10-02	273.84	276.43	270.52	273.85	560723	{}	\N	2020-10-12 16:33:40.274137
79129	WST	2020-10-05	275.8	288.08	275.25	287.64	1744052	{}	\N	2020-10-12 16:33:40.279003
79130	WST	2020-10-06	287.35	295.39	286.68	290.33	1383879	{}	\N	2020-10-12 16:33:40.284095
79131	WST	2020-10-07	291.95	295.1	291.63	292.71	898946	{}	\N	2020-10-12 16:33:40.28859
79132	WST	2020-10-08	293.58	294.88	289.57	291.26	743907	{}	\N	2020-10-12 16:33:40.29323
79133	WST	2020-10-09	292.81	298.71	292.42	298.28	596925	{}	\N	2020-10-12 16:33:40.298094
79134	WST	2020-10-12	300	301.8	298	301.35	278369	{}	\N	2020-10-12 16:33:40.303029
79135	WTMF	2020-09-28	34.84	34.93	34.8	34.93	12086	{}	\N	2020-10-12 16:33:44.10304
79136	WTMF	2020-09-29	34.96	34.96	34.78	34.8562	13053	{}	\N	2020-10-12 16:33:44.114024
79137	WTMF	2020-09-30	34.72	35.1	34.72	34.99	10375	{}	\N	2020-10-12 16:33:44.121156
79138	WTMF	2020-10-01	34.86	34.8856	34.775	34.82	4657	{}	\N	2020-10-12 16:33:44.127065
79139	WTMF	2020-10-02	34.6676	34.8462	34.6676	34.75	32265	{}	\N	2020-10-12 16:33:44.132758
79140	WTMF	2020-10-05	34.72	35.09	34.72	35.01	8960	{}	\N	2020-10-12 16:33:44.137894
79141	WTMF	2020-10-06	35.15	35.1856	35.02	35.03	6966	{}	\N	2020-10-12 16:33:44.143019
79142	WTMF	2020-10-07	35.205	35.24	35.18	35.24	8173	{}	\N	2020-10-12 16:33:44.148108
79143	WTMF	2020-10-08	35.3523	35.37	35.16	35.36	10344	{}	\N	2020-10-12 16:33:44.153012
79144	WTMF	2020-10-09	35.3766	35.52	35.36	35.4	16248	{}	\N	2020-10-12 16:33:44.157823
79145	WTMF	2020-10-12	35.35	35.372	35.27	35.27	1876	{}	\N	2020-10-12 16:33:44.163058
79146	WU	2020-09-28	21.84	22.08	21.7	21.99	3089792	{}	\N	2020-10-12 16:33:46.259675
79147	WU	2020-09-29	22	22.17	21.725	21.78	4711645	{}	\N	2020-10-12 16:33:46.270577
79148	WU	2020-09-30	21.8	22.02	21.29	21.43	8833254	{}	\N	2020-10-12 16:33:46.278128
79149	WU	2020-10-01	21.48	21.75	21.12	21.48	8211378	{}	\N	2020-10-12 16:33:46.285475
79150	WU	2020-10-02	21.2	21.64	21.05	21.4	5826353	{}	\N	2020-10-12 16:33:46.292425
79151	WU	2020-10-05	21.28	21.69	21.22	21.48	6623239	{}	\N	2020-10-12 16:33:46.297483
79152	WU	2020-10-06	21.59	22.13	21.48	21.73	9180350	{}	\N	2020-10-12 16:33:46.30316
79153	WU	2020-10-07	21.89	22.24	21.77	22.02	6832737	{}	\N	2020-10-12 16:33:46.308755
79154	WU	2020-10-08	22.14	22.5	22.115	22.32	4598305	{}	\N	2020-10-12 16:33:46.314058
79155	WU	2020-10-09	22.5	22.65	22.18	22.23	6439549	{}	\N	2020-10-12 16:33:46.318997
79156	WU	2020-10-12	22.29	22.38	21.99	22.23	1909107	{}	\N	2020-10-12 16:33:46.324466
79157	WEYS	2020-09-28	15.67	15.8299	15.56	15.67	8115	{}	\N	2020-10-12 16:33:47.272773
79158	WEYS	2020-09-29	16.162	16.162	15.35	15.65	11771	{}	\N	2020-10-12 16:33:47.284701
79159	WEYS	2020-09-30	15.69	17.2599	15.69	16.17	9608	{}	\N	2020-10-12 16:33:47.292917
79160	WEYS	2020-10-01	16.21	16.3	15.7	15.72	21193	{}	\N	2020-10-12 16:33:47.299838
79161	WEYS	2020-10-02	16.0387	16.395	15.82	16.05	8353	{}	\N	2020-10-12 16:33:47.305854
79162	WEYS	2020-10-05	16.05	16.79	16.05	16.6	10167	{}	\N	2020-10-12 16:33:47.311191
79163	WEYS	2020-10-06	16.71	17.29	16.58	16.58	7749	{}	\N	2020-10-12 16:33:47.316482
79164	WEYS	2020-10-07	17.085	17.38	16.57	17.38	23317	{}	\N	2020-10-12 16:33:47.321305
79165	WEYS	2020-10-08	17.34	17.665	17.25	17.25	9910	{}	\N	2020-10-12 16:33:47.32641
79166	WEYS	2020-10-09	17.55	17.89	17.29	17.29	9321	{}	\N	2020-10-12 16:33:47.331097
79167	WEYS	2020-10-12	17.29	17.4351	17.29	17.41	872	{}	\N	2020-10-12 16:33:47.335962
79168	WHLM	2020-09-28	3.25	3.7	3.25	3.45	21529	{}	\N	2020-10-12 16:33:48.277933
79169	WHLM	2020-09-29	3.97	4.1603	3.13	3.4067	134341	{}	\N	2020-10-12 16:33:48.288335
79170	WHLM	2020-09-30	3.305	3.305	3.13	3.13	29427	{}	\N	2020-10-12 16:33:48.295935
79171	WHLM	2020-10-01	3.14	3.23	3.08	3.21	5895	{}	\N	2020-10-12 16:33:48.302528
79172	WHLM	2020-10-02	3.04	3.57	3.04	3.3	45561	{}	\N	2020-10-12 16:33:48.308916
79173	WHLM	2020-10-05	3.39	3.592	3.15	3.17	18919	{}	\N	2020-10-12 16:33:48.313918
79174	WHLM	2020-10-06	3.17	3.33	3.05	3.08	24481	{}	\N	2020-10-12 16:33:48.318548
79175	WHLM	2020-10-07	3.14	3.17	2.875	3	16448	{}	\N	2020-10-12 16:33:48.323277
79176	WHLM	2020-10-08	3	3.3	2.95	2.96	10320	{}	\N	2020-10-12 16:33:48.328483
79177	WHLM	2020-10-09	3.15	3.15	3	3.011	6306	{}	\N	2020-10-12 16:33:48.333124
79178	WHLM	2020-10-12	3.13	3.18	3.0977	3.11	8087	{}	\N	2020-10-12 16:33:48.338002
79179	WKHS	2020-09-28	26.14	28.62	25	28.13	41052045	{}	\N	2020-10-12 16:33:51.070884
79180	WKHS	2020-09-29	27.64	27.98	26.3	27.1	24256072	{}	\N	2020-10-12 16:33:51.084914
79181	WKHS	2020-09-30	26.59	26.97	25.08	25.28	23373306	{}	\N	2020-10-12 16:33:51.094534
79182	WKHS	2020-10-01	25.6	26.97	24.41	25.08	22587231	{}	\N	2020-10-12 16:33:51.103327
79183	WKHS	2020-10-02	22.9	24.88	22.59	23.62	15418586	{}	\N	2020-10-12 16:33:51.112737
79184	WKHS	2020-10-05	23.9	24.55	23.33	24.42	12804863	{}	\N	2020-10-12 16:33:51.120298
79185	WKHS	2020-10-06	24.44	25.41	23.63	23.93	13993576	{}	\N	2020-10-12 16:33:51.131746
79186	WKHS	2020-10-07	24.35	24.4196	23.66	24.2	9203139	{}	\N	2020-10-12 16:33:51.168253
79187	WKHS	2020-10-08	22.8	25.14	22.72	23.77	24530317	{}	\N	2020-10-12 16:33:51.176565
79188	WKHS	2020-10-09	23.8099	27.11	23.62	26.77	34809741	{}	\N	2020-10-12 16:33:51.183331
79189	WKHS	2020-10-12	27.975	30.59	27.96	28.4299	55211269	{}	\N	2020-10-12 16:33:51.189269
79190	WM	2020-09-28	114.6	115.21	113.88	114.04	1287947	{}	\N	2020-10-12 16:33:53.007285
79191	WM	2020-09-29	114.18	114.6532	113.33	113.41	2094753	{}	\N	2020-10-12 16:33:53.019884
79192	WM	2020-09-30	113.8	114.32	112.54	113.17	4096198	{}	\N	2020-10-12 16:33:53.027226
79193	WM	2020-10-01	113.96	113.99	112.0258	112.89	3016731	{}	\N	2020-10-12 16:33:53.034192
79194	WM	2020-10-02	111.66	114.56	111.58	113.89	2167636	{}	\N	2020-10-12 16:33:53.039898
79195	WM	2020-10-05	114.83	115.06	113.84	114.24	2456243	{}	\N	2020-10-12 16:33:53.045446
79196	WM	2020-10-06	114.81	115.05	112.9	113.25	2695366	{}	\N	2020-10-12 16:33:53.050512
79197	WM	2020-10-07	113.85	114.4	113.33	113.88	2182592	{}	\N	2020-10-12 16:33:53.055465
79198	WM	2020-10-08	114.33	114.89	113.91	114.7	1862896	{}	\N	2020-10-12 16:33:53.061423
79199	WM	2020-10-09	115.13	115.7	114.68	115.35	2185877	{}	\N	2020-10-12 16:33:53.066161
79200	WM	2020-10-12	115.83	116.92	115.68	116.83	905779	{}	\N	2020-10-12 16:33:53.071548
79201	WNC	2020-09-28	12.1	12.71	12.1	12.4	478145	{}	\N	2020-10-12 16:33:54.119278
79202	WNC	2020-09-29	12.44	12.53	12.02	12.1	530933	{}	\N	2020-10-12 16:33:54.130956
79203	WNC	2020-09-30	12.14	12.39	11.82	11.96	778708	{}	\N	2020-10-12 16:33:54.140421
79204	WNC	2020-10-01	11.95	12.16	11.82	11.95	646364	{}	\N	2020-10-12 16:33:54.149787
79205	WNC	2020-10-02	11.6	12.41	11.57	12.25	1217645	{}	\N	2020-10-12 16:33:54.156474
79206	WNC	2020-10-05	12.43	13.08	12.27	12.97	1053192	{}	\N	2020-10-12 16:33:54.163095
79207	WNC	2020-10-06	13.32	13.9	13.17	13.52	1524786	{}	\N	2020-10-12 16:33:54.168915
79208	WNC	2020-10-07	13.77	13.82	13.47	13.73	850297	{}	\N	2020-10-12 16:33:54.174809
79209	WNC	2020-10-08	13.91	14.11	13.66	14.02	679899	{}	\N	2020-10-12 16:33:54.179702
79210	WNC	2020-10-09	14.17	14.2	13.73	13.76	526693	{}	\N	2020-10-12 16:33:54.185263
79211	WNC	2020-10-12	13.76	14	13.74	13.77	163065	{}	\N	2020-10-12 16:33:54.190556
79212	WPX	2020-09-28	4.91	5.255	4.61	5.17	68049462	{}	\N	2020-10-12 16:33:55.577475
79213	WPX	2020-09-29	5.07	5.1	4.67	4.99	53732470	{}	\N	2020-10-12 16:33:55.586149
79214	WPX	2020-09-30	4.98	5.105	4.86	4.9	25922551	{}	\N	2020-10-12 16:33:55.592901
79215	WPX	2020-10-01	4.87	4.95	4.63	4.65	23711930	{}	\N	2020-10-12 16:33:55.599424
79216	WPX	2020-10-02	4.42	5.06	4.42	4.93	33766904	{}	\N	2020-10-12 16:33:55.60645
79217	WPX	2020-10-05	5.05	5.28	4.92	5.24	34905292	{}	\N	2020-10-12 16:33:55.612627
79218	WPX	2020-10-06	5.37	5.485	4.96	4.99	14371538	{}	\N	2020-10-12 16:33:55.619005
79219	WPX	2020-10-07	5.04	5.15	4.94	5.08	14000419	{}	\N	2020-10-12 16:33:55.624966
79220	WPX	2020-10-08	5.15	5.3	5.07	5.3	10477796	{}	\N	2020-10-12 16:33:55.632831
79221	WPX	2020-10-09	5.38	5.3999	5.13	5.2	12002265	{}	\N	2020-10-12 16:33:55.640307
79222	WPX	2020-10-12	5.17	5.19	5.035	5.0584	3928320	{}	\N	2020-10-12 16:33:55.65011
79223	WSFS	2020-09-28	26.09	26.71	26.09	26.48	167343	{}	\N	2020-10-12 16:33:57.05868
79224	WSFS	2020-09-29	26.42	26.58	25.61	26.56	201485	{}	\N	2020-10-12 16:33:57.06793
79225	WSFS	2020-09-30	26.85	27.33	26.68	26.97	308026	{}	\N	2020-10-12 16:33:57.073674
79226	WSFS	2020-10-01	27.05	27.27	26.48	27.23	204071	{}	\N	2020-10-12 16:33:57.079246
79227	WSFS	2020-10-02	26.83	28.37	26.58	28.25	251094	{}	\N	2020-10-12 16:33:57.084178
79228	WSFS	2020-10-05	28.63	29.71	28.48	29.49	273760	{}	\N	2020-10-12 16:33:57.089004
79229	WSFS	2020-10-06	29.98	30.96	29.31	29.55	250779	{}	\N	2020-10-12 16:33:57.094119
79230	WSFS	2020-10-07	29.7	30.65	29.7	30.48	331032	{}	\N	2020-10-12 16:33:57.098858
79231	WSFS	2020-10-08	30.85	31.26	30.3	31.19	167137	{}	\N	2020-10-12 16:33:57.104377
79232	WSFS	2020-10-09	31.6	31.6	30.57	30.65	221881	{}	\N	2020-10-12 16:33:57.11102
79233	WSFS	2020-10-12	30.77	31.05	30.49	30.95	59018	{}	\N	2020-10-12 16:33:57.119193
79234	WTS	2020-09-28	98.47	99.13	98.02	98.19	240298	{}	\N	2020-10-12 16:34:03.225459
79235	WTS	2020-09-29	98.75	99.065	98.08	98.84	317030	{}	\N	2020-10-12 16:34:03.236032
79236	WTS	2020-09-30	99.15	101.245	98.94	100.15	508397	{}	\N	2020-10-12 16:34:03.244389
79237	WTS	2020-10-01	100.78	101.255	100	101	312808	{}	\N	2020-10-12 16:34:03.250786
79238	WTS	2020-10-02	99.27	102.56	99.27	101.78	159513	{}	\N	2020-10-12 16:34:03.256138
79239	WTS	2020-10-05	102.74	103.98	102.74	103.95	255676	{}	\N	2020-10-12 16:34:03.262275
79240	WTS	2020-10-06	104.59	105.32	103.45	103.7	310301	{}	\N	2020-10-12 16:34:03.267395
79241	WTS	2020-10-07	104.53	106.3	104.415	105.93	416674	{}	\N	2020-10-12 16:34:03.27242
79242	WTS	2020-10-08	106.66	107.13	105.455	106.27	246846	{}	\N	2020-10-12 16:34:03.281093
79243	WTS	2020-10-09	107.24	108.2036	107.025	107.45	217367	{}	\N	2020-10-12 16:34:03.290066
79244	WTS	2020-10-12	107.94	110.82	107.94	110.33	123689	{}	\N	2020-10-12 16:34:03.297075
79245	WVVI	2020-09-28	6.195	6.4	6.18	6.3398	4465	{}	\N	2020-10-12 16:34:04.52641
79246	WVVI	2020-09-29	6.39	6.41	6.25	6.3328	4771	{}	\N	2020-10-12 16:34:04.538194
79247	WVVI	2020-09-30	6.3	6.34	6.15	6.15	4428	{}	\N	2020-10-12 16:34:04.544415
79248	WVVI	2020-10-01	6.15	6.305	6.1	6.2117	4869	{}	\N	2020-10-12 16:34:04.550176
79249	WVVI	2020-10-02	6.27	6.315	6.13	6.315	7214	{}	\N	2020-10-12 16:34:04.554912
79250	WVVI	2020-10-05	6.35	6.45	6.138	6.28	7529	{}	\N	2020-10-12 16:34:04.559242
79251	WVVI	2020-10-06	6.4	6.47	6.3001	6.31	2670	{}	\N	2020-10-12 16:34:04.563706
79252	WVVI	2020-10-07	6.41	6.41	6.3399	6.3399	478	{}	\N	2020-10-12 16:34:04.568016
79253	WVVI	2020-10-08	6.3	6.395	6.13	6.165	5918	{}	\N	2020-10-12 16:34:04.572018
79254	WVVI	2020-10-09	6.1872	6.39	6.1872	6.3571	2495	{}	\N	2020-10-12 16:34:04.578224
79255	WVVI	2020-10-12	6.28	6.28	6	6.1044	28051	{}	\N	2020-10-12 16:34:04.584292
79256	WYY	2020-09-28	0.485	0.5249	0.485	0.51	187970	{}	\N	2020-10-12 16:34:05.589754
79257	WYY	2020-09-29	0.5218	0.5352	0.4922	0.5227	413218	{}	\N	2020-10-12 16:34:05.600988
79258	WYY	2020-09-30	0.5169	0.534	0.4998	0.4999	415934	{}	\N	2020-10-12 16:34:05.609981
79259	WYY	2020-10-01	0.5217	0.5217	0.486	0.49	364801	{}	\N	2020-10-12 16:34:05.61739
79260	WYY	2020-10-02	0.486	0.49	0.48	0.4863	274794	{}	\N	2020-10-12 16:34:05.623682
79261	WYY	2020-10-05	0.49	0.499	0.46	0.4889	463682	{}	\N	2020-10-12 16:34:05.629447
79262	WYY	2020-10-06	0.4798	0.51	0.475	0.475	207595	{}	\N	2020-10-12 16:34:05.635094
79263	WYY	2020-10-07	0.49	0.49	0.48	0.4839	126005	{}	\N	2020-10-12 16:34:05.640157
79264	WYY	2020-10-08	0.48	0.5098	0.48	0.5066	418709	{}	\N	2020-10-12 16:34:05.645465
79265	WYY	2020-10-09	0.474	0.514	0.47	0.505	550375	{}	\N	2020-10-12 16:34:05.650818
79266	WYY	2020-10-12	0.51	0.5299	0.5	0.519799	244669	{}	\N	2020-10-12 16:34:05.656309
79267	XEC	2020-09-28	24.48	25.42	23.76	25.01	2223368	{}	\N	2020-10-12 16:34:10.105416
79268	XEC	2020-09-29	24.7	25.12	23.93	24.7	4182535	{}	\N	2020-10-12 16:34:10.112389
79269	XEC	2020-09-30	24.85	24.94	24.08	24.33	3242128	{}	\N	2020-10-12 16:34:10.119675
79270	XEC	2020-10-01	23.89	24.15	23.22	23.38	3275960	{}	\N	2020-10-12 16:34:10.128436
79271	XEC	2020-10-02	22.43	24.24	22.39	24.08	3114048	{}	\N	2020-10-12 16:34:10.136903
79272	XEC	2020-10-05	24.59	25.15	24.24	25.12	1868564	{}	\N	2020-10-12 16:34:10.146078
79273	XEC	2020-10-06	25.72	26.11	23.55	23.58	2555911	{}	\N	2020-10-12 16:34:10.153269
79274	XEC	2020-10-07	23.67	24.405	23.4	24.21	2195072	{}	\N	2020-10-12 16:34:10.158813
79275	XEC	2020-10-08	24.46	25.83	24.37	25.77	1787022	{}	\N	2020-10-12 16:34:10.163936
79276	XEC	2020-10-09	26.01	26.21	24.8	24.91	2484030	{}	\N	2020-10-12 16:34:10.169139
79277	XEC	2020-10-12	24.72	24.7886	24.25	24.58	282511	{}	\N	2020-10-12 16:34:10.174111
79278	WF	2020-09-28	21.84	22.05	21.76	22.05	21403	{}	\N	2020-10-12 16:34:11.465294
79279	WF	2020-09-29	22.07	22.07	21.75	21.81	50036	{}	\N	2020-10-12 16:34:11.478213
79280	WF	2020-09-30	21.9	22.3	21.8959	22.08	107815	{}	\N	2020-10-12 16:34:11.485642
79281	WF	2020-10-01	22.06	22.2	22.02	22.11	29274	{}	\N	2020-10-12 16:34:11.491885
79282	WF	2020-10-02	21.72	22.56	21.72	22.56	48913	{}	\N	2020-10-12 16:34:11.4977
79283	WF	2020-10-05	22.65	22.68	22.3875	22.53	74625	{}	\N	2020-10-12 16:34:11.503273
79284	WF	2020-10-06	22.21	22.22	21.66	21.66	53048	{}	\N	2020-10-12 16:34:11.508696
79285	WF	2020-10-07	22.2	22.65	22.2	22.65	75684	{}	\N	2020-10-12 16:34:11.516354
79286	WF	2020-10-08	22.78	22.86	22.54	22.86	22698	{}	\N	2020-10-12 16:34:11.524836
79287	WF	2020-10-09	22.96	22.99	22.75	22.99	57346	{}	\N	2020-10-12 16:34:11.53203
79288	WF	2020-10-12	23.17	23.17	22.62	22.89	26633	{}	\N	2020-10-12 16:34:11.539889
79289	WHLR	2020-09-28	2.93	3.05	2.9131	3.01	49613	{}	\N	2020-10-12 16:34:12.639287
79290	WHLR	2020-09-29	3.025	3.03	2.967	2.98	15423	{}	\N	2020-10-12 16:34:12.647371
79291	WHLR	2020-09-30	2.96	3.1	2.95	3.04	22796	{}	\N	2020-10-12 16:34:12.655708
79292	WHLR	2020-10-01	3.02	3.08	2.948	3.08	19957	{}	\N	2020-10-12 16:34:12.663659
79293	WHLR	2020-10-02	3.08	3.1	3.01	3.1	21086	{}	\N	2020-10-12 16:34:12.671324
79294	WHLR	2020-10-05	3.14	3.1847	3.02	3.03	12872	{}	\N	2020-10-12 16:34:12.678526
79295	WHLR	2020-10-06	3.01	3.11	2.93	2.96	6320	{}	\N	2020-10-12 16:34:12.684517
79296	WHLR	2020-10-07	3.1	3.1	2.86	2.91	10659	{}	\N	2020-10-12 16:34:12.69168
79297	WHLR	2020-10-08	2.88	2.91	2.75	2.79	21955	{}	\N	2020-10-12 16:34:12.698532
79298	WHLR	2020-10-09	2.77	2.85	2.65	2.7	20522	{}	\N	2020-10-12 16:34:12.704696
79299	WHLR	2020-10-12	2.7	2.72	2.6673	2.67	4697	{}	\N	2020-10-12 16:34:12.711822
79300	WIL	2020-09-28	70.9534	70.9534	70.9534	70.9534	100	{}	\N	2020-10-12 16:34:14.581191
79301	WIL	2020-09-29	70.8878	70.8878	70.8878	70.8878	100	{}	\N	2020-10-12 16:34:14.589988
79302	WIL	2020-09-30	71.4758	71.4758	71.4758	71.4758	103	{}	\N	2020-10-12 16:34:14.59578
79303	WIL	2020-10-01	71.5492	71.5492	71.5492	71.5492	102	{}	\N	2020-10-12 16:34:14.601099
79304	WIL	2020-10-02	71.4829	71.4829	71.4829	71.4829	101	{}	\N	2020-10-12 16:34:14.606134
79305	WIL	2020-10-05	71.3301	72.3363	71.3301	72.3363	1160	{}	\N	2020-10-12 16:34:14.610791
79306	WIL	2020-10-06	71.8149	71.8149	71.8149	71.8149	131	{}	\N	2020-10-12 16:34:14.615226
79307	WIL	2020-10-07	72.7455	72.7455	72.7455	72.7455	100	{}	\N	2020-10-12 16:34:14.619453
79308	WIL	2020-10-08	73.5317	73.5317	73.5317	73.5317	144	{}	\N	2020-10-12 16:34:14.624005
79309	WIL	2020-10-09	73.9496	73.9496	73.9496	73.9496	202	{}	\N	2020-10-12 16:34:14.629125
79310	WINS	2020-09-28	19.02	19.02	19.02	19.02	760	{}	\N	2020-10-12 16:34:16.590436
79311	WINS	2020-09-29	18.5	19.5	18.5	18.7	6809	{}	\N	2020-10-12 16:34:16.601924
79312	WINS	2020-09-30	18.7496	18.7496	18.7496	18.7496	2013	{}	\N	2020-10-12 16:34:16.609685
79313	WINS	2020-10-01	19	20	19	20	1431	{}	\N	2020-10-12 16:34:16.616289
79314	WINS	2020-10-02	19	19.8	18.6	18.72	3635	{}	\N	2020-10-12 16:34:16.622126
79315	WINS	2020-10-05	19	19.8774	18.57	19	8685	{}	\N	2020-10-12 16:34:16.6272
79316	WINS	2020-10-06	19.4498	19.5	18.6	18.68	5228	{}	\N	2020-10-12 16:34:16.63325
79317	WINS	2020-10-07	19	19	18.52	18.88	3312	{}	\N	2020-10-12 16:34:16.638722
79318	WINS	2020-10-08	18.85	18.85	18.85	18.85	1361	{}	\N	2020-10-12 16:34:16.644853
79319	WINS	2020-10-09	18.7301	18.7301	18.62	18.62	1150	{}	\N	2020-10-12 16:34:16.649945
79320	WINS	2020-10-12	18.973	19	18.973	19	2828	{}	\N	2020-10-12 16:34:16.655811
79321	WMGI	2020-09-28	30.46	30.58	30.41	30.57	1104617	{}	\N	2020-10-12 16:34:20.446828
79322	WMGI	2020-09-29	30.59	30.63	30.45	30.56	1440965	{}	\N	2020-10-12 16:34:20.455547
79323	WMGI	2020-09-30	30.55	30.62	30.52	30.54	1329172	{}	\N	2020-10-12 16:34:20.461584
79324	WMGI	2020-10-01	30.58	30.715	30.54	30.63	1538060	{}	\N	2020-10-12 16:34:20.466315
79325	WMGI	2020-10-02	30.51	30.67	30.51	30.59	1124595	{}	\N	2020-10-12 16:34:20.471203
79326	WMGI	2020-10-05	30.61	30.61	30.475	30.52	1024620	{}	\N	2020-10-12 16:34:20.475831
79327	WMGI	2020-10-06	30.52	30.59	30.52	30.54	1200860	{}	\N	2020-10-12 16:34:20.480612
79328	WMGI	2020-10-07	30.55	30.64	30.55	30.56	1043872	{}	\N	2020-10-12 16:34:20.485126
79329	WMGI	2020-10-08	30.59	30.64	30.53	30.56	1849551	{}	\N	2020-10-12 16:34:20.489412
79330	WMGI	2020-10-09	30.56	30.61	30.56	30.57	1532543	{}	\N	2020-10-12 16:34:20.493959
79331	WMGI	2020-10-12	30.59	30.67	30.59	30.63	204095	{}	\N	2020-10-12 16:34:20.498593
79332	WNEB	2020-09-28	5.24	5.7974	5.24	5.73	50242	{}	\N	2020-10-12 16:34:21.495645
79333	WNEB	2020-09-29	5.575	5.64	5.5	5.6	23796	{}	\N	2020-10-12 16:34:21.501394
79334	WNEB	2020-09-30	5.56	5.71	5.56	5.63	28936	{}	\N	2020-10-12 16:34:21.506239
79335	WNEB	2020-10-01	5.58	5.72	5.58	5.66	37894	{}	\N	2020-10-12 16:34:21.511268
79336	WNEB	2020-10-02	5.62	5.905	5.61	5.79	60705	{}	\N	2020-10-12 16:34:21.515763
79337	WNEB	2020-10-05	6	6.37	5.98	6.34	38452	{}	\N	2020-10-12 16:34:21.524205
79338	WNEB	2020-10-06	6.44	6.9	6.36	6.8	122795	{}	\N	2020-10-12 16:34:21.529926
79339	WNEB	2020-10-07	6.98	7.18	6.95	7.04	58078	{}	\N	2020-10-12 16:34:21.534595
79340	WNEB	2020-10-08	7.12	7.12	6.81	6.96	58366	{}	\N	2020-10-12 16:34:21.53888
79341	WNEB	2020-10-09	6.9	6.96	6.62	6.9	28773	{}	\N	2020-10-12 16:34:21.54379
79342	WNEB	2020-10-12	6.9	6.945	6.85	6.91	2840	{}	\N	2020-10-12 16:34:21.548292
79343	WPP	2020-09-28	39.16	39.43	38.89	39.14	177828	{}	\N	2020-10-12 16:34:22.568957
79344	WPP	2020-09-29	38.95	39.01	38.4991	38.7	240655	{}	\N	2020-10-12 16:34:22.575753
79345	WPP	2020-09-30	39.04	39.61	39.04	39.24	359013	{}	\N	2020-10-12 16:34:22.580522
79346	WPP	2020-10-01	39.34	39.7	38.94	39.69	411340	{}	\N	2020-10-12 16:34:22.584526
79347	WPP	2020-10-02	39.26	40.76	39.21	40.68	274747	{}	\N	2020-10-12 16:34:22.589067
79348	WPP	2020-10-05	41.73	42.31	41.7	42.29	347770	{}	\N	2020-10-12 16:34:22.595168
79349	WPP	2020-10-06	42.74	42.79	41.76	41.88	265566	{}	\N	2020-10-12 16:34:22.599499
79350	WPP	2020-10-07	42.38	42.55	42.16	42.47	323984	{}	\N	2020-10-12 16:34:22.604545
79351	WPP	2020-10-08	42.81	42.95	42.53	42.8	283162	{}	\N	2020-10-12 16:34:22.609289
79352	WPP	2020-10-09	43.61	43.68	43.21	43.56	243292	{}	\N	2020-10-12 16:34:22.614221
79353	WPP	2020-10-12	43.04	43.19	42.86	43.0001	104858	{}	\N	2020-10-12 16:34:22.619053
79354	WRI	2020-09-28	16.89	17.22	16.76	17.1	666203	{}	\N	2020-10-12 16:34:24.41979
79355	WRI	2020-09-29	17.01	17.01	16.3	16.9	1644611	{}	\N	2020-10-12 16:34:24.42659
79356	WRI	2020-09-30	17.07	17.375	16.65	16.96	1854950	{}	\N	2020-10-12 16:34:24.432067
79357	WRI	2020-10-01	16.96	17.7	16.87	17.69	1495980	{}	\N	2020-10-12 16:34:24.437373
79358	WRI	2020-10-02	17.24	17.98	17.08	17.93	1417972	{}	\N	2020-10-12 16:34:24.443204
79359	WRI	2020-10-05	18.08	18.23	17.73	18.15	1512317	{}	\N	2020-10-12 16:34:24.44842
79360	WRI	2020-10-06	18.33	18.6	18.02	18.19	2277893	{}	\N	2020-10-12 16:34:24.453305
79361	WRI	2020-10-07	18.15	18.15	17.49	17.49	2023596	{}	\N	2020-10-12 16:34:24.459048
79362	WRI	2020-10-08	17.67	17.95	17.49	17.87	1629286	{}	\N	2020-10-12 16:34:24.465975
79363	WRI	2020-10-09	17.95	18	17.27	17.3	2025817	{}	\N	2020-10-12 16:34:24.47315
79364	WRI	2020-10-12	17.3	17.41	17.06	17.25	554832	{}	\N	2020-10-12 16:34:24.478358
79365	WSBF	2020-09-28	15.03	15.48	15.03	15.29	96487	{}	\N	2020-10-12 16:34:25.950278
79366	WSBF	2020-09-29	15.26	15.38	15.15	15.33	66437	{}	\N	2020-10-12 16:34:25.958166
79367	WSBF	2020-09-30	15.38	15.59	15.32	15.49	149760	{}	\N	2020-10-12 16:34:25.964842
79368	WSBF	2020-10-01	15.49	15.72	15.39	15.68	75406	{}	\N	2020-10-12 16:34:25.970563
79369	WSBF	2020-10-02	15.49	16.04	15.49	15.93	49882	{}	\N	2020-10-12 16:34:25.975621
79370	WSBF	2020-10-05	16.01	16.25	15.92	16.22	67043	{}	\N	2020-10-12 16:34:25.980608
79371	WSBF	2020-10-06	16.25	16.845	16.01	16.01	97328	{}	\N	2020-10-12 16:34:25.985893
79372	WSBF	2020-10-07	16.21	16.39	16.12	16.31	71355	{}	\N	2020-10-12 16:34:25.991325
79373	WSBF	2020-10-08	16.34	16.55	16.33	16.43	59130	{}	\N	2020-10-12 16:34:25.996614
79374	WSBF	2020-10-09	16.6	16.62	16.46	16.54	49181	{}	\N	2020-10-12 16:34:26.001848
79375	WSBF	2020-10-12	16.51	16.5881	16.45	16.53	7842	{}	\N	2020-10-12 16:34:26.007253
79376	WTBA	2020-09-28	16.22	16.44	16.2	16.28	32103	{}	\N	2020-10-12 16:34:27.345909
79377	WTBA	2020-09-29	16.12	16.2699	15.76	16	18142	{}	\N	2020-10-12 16:34:27.352495
79378	WTBA	2020-09-30	15.95	16.12	15.71	15.84	30381	{}	\N	2020-10-12 16:34:27.357952
79379	WTBA	2020-10-01	15.86	15.97	15.53	15.89	24520	{}	\N	2020-10-12 16:34:27.362568
79380	WTBA	2020-10-02	15.61	16.57	15.61	16.28	23876	{}	\N	2020-10-12 16:34:27.367125
79381	WTBA	2020-10-05	16.485	16.67	16.465	16.61	35736	{}	\N	2020-10-12 16:34:27.371603
79382	WTBA	2020-10-06	16.74	17.1	16.3125	16.44	48281	{}	\N	2020-10-12 16:34:27.376335
79383	WTBA	2020-10-07	16.8	17.05	16.71	16.97	26531	{}	\N	2020-10-12 16:34:27.381785
79384	WTBA	2020-10-08	17	17.54	16.45	17.37	26807	{}	\N	2020-10-12 16:34:27.387049
79385	WTBA	2020-10-09	17.13	17.298	17.02	17.17	9556	{}	\N	2020-10-12 16:34:27.391726
79386	WTBA	2020-10-12	17.03	17.3	16.92	17.3	4824	{}	\N	2020-10-12 16:34:27.396241
79387	WVVIP	2020-09-28	5.5827	5.5827	5.5	5.5273	1155	{}	\N	2020-10-12 16:34:30.097287
79388	WVVIP	2020-09-29	5.275	5.4	5.2454	5.28	2044	{}	\N	2020-10-12 16:34:30.107449
79389	WVVIP	2020-09-30	5.45	5.7002	5.13	5.22	3217	{}	\N	2020-10-12 16:34:30.114856
79390	WVVIP	2020-10-01	5.22	5.55	5.22	5.2971	1852	{}	\N	2020-10-12 16:34:30.121472
79391	WVVIP	2020-10-02	5	5.3841	5	5.3841	718	{}	\N	2020-10-12 16:34:30.127172
79392	WVVIP	2020-10-08	5.6	5.6	5.5	5.5	440	{}	\N	2020-10-12 16:34:30.132211
79393	WVVIP	2020-10-12	6.05	6.05	5.711	5.711	447	{}	\N	2020-10-12 16:34:30.137324
79394	WGO	2020-09-28	51.91	51.91	49.15	49.96	948117	{}	\N	2020-10-12 16:34:31.141808
79395	WGO	2020-09-29	50.04	51.53	50	50.09	1089407	{}	\N	2020-10-12 16:34:31.152872
79396	WGO	2020-09-30	50.17	52.86	50.17	51.67	2597846	{}	\N	2020-10-12 16:34:31.159652
79397	WGO	2020-10-01	52.22	53.1	51.225	52.97	1787210	{}	\N	2020-10-12 16:34:31.165866
79398	WGO	2020-10-02	51.5	55.34	51.26	53.95	1496883	{}	\N	2020-10-12 16:34:31.171808
79399	WGO	2020-10-05	54.44	56.42	54.07	56.17	922601	{}	\N	2020-10-12 16:34:31.17722
79400	WGO	2020-10-06	56.55	58.34	55.58	55.87	1046908	{}	\N	2020-10-12 16:34:31.182207
79401	WGO	2020-10-07	56.7	57.4832	54.59	55.72	1301951	{}	\N	2020-10-12 16:34:31.187106
79402	WGO	2020-10-08	56.34	56.5	52.72	52.82	1352085	{}	\N	2020-10-12 16:34:31.19229
79403	WGO	2020-10-09	53.28	54.69	52.97	53.61	1393602	{}	\N	2020-10-12 16:34:31.197229
79404	WGO	2020-10-12	53.75	54.0496	52.0312	52.5582	350602	{}	\N	2020-10-12 16:34:31.202279
79405	WIA	2020-09-28	12.53	12.54	12.31	12.36	575284	{}	\N	2020-10-12 16:34:33.160225
79406	WIA	2020-09-29	12.34	12.3791	12.08	12.09	1081740	{}	\N	2020-10-12 16:34:33.173264
79407	WIA	2020-09-30	12.11	12.2101	12.1	12.18	938754	{}	\N	2020-10-12 16:34:33.181236
79408	WIA	2020-10-01	12.24	12.25	12.14	12.21	608568	{}	\N	2020-10-12 16:34:33.187713
79409	WIA	2020-10-02	12.14	12.17	12.07	12.12	222893	{}	\N	2020-10-12 16:34:33.19348
79410	WIA	2020-10-05	12.11	12.14	12.04	12.07	398698	{}	\N	2020-10-12 16:34:33.19846
79411	WIA	2020-10-06	12.07	12.17	12.07	12.16	51825	{}	\N	2020-10-12 16:34:33.203708
79412	WIA	2020-10-07	12.16	12.1662	12.11	12.12	146068	{}	\N	2020-10-12 16:34:33.208895
79413	WIA	2020-10-08	12.15	12.29	12.135	12.26	82198	{}	\N	2020-10-12 16:34:33.213615
79414	WIA	2020-10-09	12.2	12.353	12.2	12.32	82560	{}	\N	2020-10-12 16:34:33.2184
79415	WIA	2020-10-12	12.34	12.37	12.27	12.29	21246	{}	\N	2020-10-12 16:34:33.222927
79416	WIRE	2020-09-28	46.37	47.74	46.37	47.13	88412	{}	\N	2020-10-12 16:34:36.309546
79417	WIRE	2020-09-29	47.03	47.63	46.17	46.55	85462	{}	\N	2020-10-12 16:34:36.31948
79418	WIRE	2020-09-30	46.55	47.62	46.14	46.42	115038	{}	\N	2020-10-12 16:34:36.325907
79419	WIRE	2020-10-01	46.54	47.26	46.3	47.02	79786	{}	\N	2020-10-12 16:34:36.332266
79420	WIRE	2020-10-02	46.32	47.84	46.135	47.43	134778	{}	\N	2020-10-12 16:34:36.33721
79421	WIRE	2020-10-05	47.83	48.87	47.83	48.3	75537	{}	\N	2020-10-12 16:34:36.341537
79422	WIRE	2020-10-06	48.64	48.64	47.36	47.74	273936	{}	\N	2020-10-12 16:34:36.345779
79423	WIRE	2020-10-07	48.23	49.09	47.63	48.13	103849	{}	\N	2020-10-12 16:34:36.350475
79424	WIRE	2020-10-08	48.51	49.47	48.51	49.28	98077	{}	\N	2020-10-12 16:34:36.354432
79425	WIRE	2020-10-09	49.72	50.06	49.18	49.49	74525	{}	\N	2020-10-12 16:34:36.358865
79426	WIRE	2020-10-12	49.34	50.27	48.96	50.0325	15017	{}	\N	2020-10-12 16:34:36.363253
79427	WLK	2020-09-28	61.81	63.185	61.22	62.59	194482	{}	\N	2020-10-12 16:34:38.74933
79428	WLK	2020-09-29	62.38	62.75	61.5075	62.09	400348	{}	\N	2020-10-12 16:34:38.762505
79429	WLK	2020-09-30	62.93	65.02	62.93	63.22	704832	{}	\N	2020-10-12 16:34:38.770722
79430	WLK	2020-10-01	63.66	64.045	61.65	62.25	601742	{}	\N	2020-10-12 16:34:38.777471
79431	WLK	2020-10-02	60.88	64.73	60.84	63.98	530914	{}	\N	2020-10-12 16:34:38.78371
79432	WLK	2020-10-05	65.23	67.25	65.16	67.16	668773	{}	\N	2020-10-12 16:34:38.788578
79433	WLK	2020-10-06	67.92	68.24	65.86	66.01	581731	{}	\N	2020-10-12 16:34:38.79471
79434	WLK	2020-10-07	67.16	68.46	66.7584	67.56	448113	{}	\N	2020-10-12 16:34:38.799464
79435	WLK	2020-10-08	67.9	68.87	67.3	67.93	457790	{}	\N	2020-10-12 16:34:38.804199
79436	WLK	2020-10-09	69.72	70.21	68.46	69.08	772957	{}	\N	2020-10-12 16:34:38.809078
79437	WLK	2020-10-12	69.45	69.7	68.58	69.14	194630	{}	\N	2020-10-12 16:34:38.813917
79438	WMS	2020-09-28	58.93	61.38	58.93	60.37	495903	{}	\N	2020-10-12 16:34:40.390662
79439	WMS	2020-09-29	60.32	61.5	60.04	60.86	546070	{}	\N	2020-10-12 16:34:40.401557
79440	WMS	2020-09-30	61.17	63.36	61.17	62.44	869992	{}	\N	2020-10-12 16:34:40.408783
79441	WMS	2020-10-01	63	64.58	62.81	64.52	1024456	{}	\N	2020-10-12 16:34:40.4146
79442	WMS	2020-10-02	62.99	64.74	62.66	63.02	1185763	{}	\N	2020-10-12 16:34:40.418484
79443	WMS	2020-10-05	63.93	65.12	63.84	65	557781	{}	\N	2020-10-12 16:34:40.422687
79444	WMS	2020-10-06	65.36	67.4	65.26	65.67	746343	{}	\N	2020-10-12 16:34:40.427162
79445	WMS	2020-10-07	67.46	67.95	66.385	67.25	1127772	{}	\N	2020-10-12 16:34:40.431782
79446	WMS	2020-10-08	67.96	68.71	67.27	67.91	492927	{}	\N	2020-10-12 16:34:40.43609
79447	WMS	2020-10-09	68.34	69.93	67.995	69.47	1095900	{}	\N	2020-10-12 16:34:40.440318
79448	WMS	2020-10-12	70.72	71.65	70.02	71.51	347039	{}	\N	2020-10-12 16:34:40.444876
79449	WPC	2020-09-28	67.87	68.08	66.5	66.6	1093231	{}	\N	2020-10-12 16:34:41.520426
79450	WPC	2020-09-29	65.51	65.95	64.35	65.13	1098651	{}	\N	2020-10-12 16:34:41.531184
79451	WPC	2020-09-30	65.66	66.04	64.19	65.16	1730080	{}	\N	2020-10-12 16:34:41.538822
79452	WPC	2020-10-01	65.36	67.06	65.3	67.05	1892286	{}	\N	2020-10-12 16:34:41.545123
79453	WPC	2020-10-02	65.44	67.83	65.26	67.47	1980042	{}	\N	2020-10-12 16:34:41.550702
79454	WPC	2020-10-05	67.75	68.1	66.58	67.47	1165052	{}	\N	2020-10-12 16:34:41.555513
79455	WPC	2020-10-06	67.69	68.76	67.03	67.64	1277043	{}	\N	2020-10-12 16:34:41.559878
79456	WPC	2020-10-07	68.02	68.365	66.78	67.66	1263150	{}	\N	2020-10-12 16:34:41.564294
79457	WPC	2020-10-08	68	69.19	67.84	68.88	1101685	{}	\N	2020-10-12 16:34:41.568594
79458	WPC	2020-10-09	69.16	69.2298	67.72	68	918287	{}	\N	2020-10-12 16:34:41.573102
79459	WPC	2020-10-12	68.03	68.41	67.04	68.23	460962	{}	\N	2020-10-12 16:34:41.577151
79460	WSTG	2020-09-28	23.8	24.25	23.15	23.15	9804	{}	\N	2020-10-12 16:34:43.450806
79461	WSTG	2020-09-29	23.86	23.86	23.273	23.38	2666	{}	\N	2020-10-12 16:34:43.462949
79462	WSTG	2020-09-30	23.13	23.57	22.5095	23.05	21288	{}	\N	2020-10-12 16:34:43.475153
79463	WSTG	2020-10-01	23.625	23.93	23.02	23.63	11954	{}	\N	2020-10-12 16:34:43.482953
79464	WSTG	2020-10-02	23.25	24.285	22.965	23.48	15462	{}	\N	2020-10-12 16:34:43.489525
79465	WSTG	2020-10-05	23.61	23.9957	23.0319	23.6	11249	{}	\N	2020-10-12 16:34:43.494673
79466	WSTG	2020-10-06	25.3304	25.5	23.26	23.8	18426	{}	\N	2020-10-12 16:34:43.500413
79467	WSTG	2020-10-07	23.63	23.9	23.48	23.64	17889	{}	\N	2020-10-12 16:34:43.506578
79468	WSTG	2020-10-08	23.57	23.57	22.49	23	13135	{}	\N	2020-10-12 16:34:43.511285
79469	WSTG	2020-10-09	22.9	23.2375	22.55	23.2375	37958	{}	\N	2020-10-12 16:34:43.516291
79470	WSTG	2020-10-12	23.15	23.68	22.485	22.75	8913	{}	\N	2020-10-12 16:34:43.522139
79471	XBIO	2020-09-28	0.91	0.93	0.86	0.8996	71813	{}	\N	2020-10-12 16:34:45.117815
79472	XBIO	2020-09-29	0.8726	0.904801	0.84	0.8497	125120	{}	\N	2020-10-12 16:34:45.128845
79473	XBIO	2020-09-30	0.9	0.91	0.8701	0.905	115103	{}	\N	2020-10-12 16:34:45.136545
79474	XBIO	2020-10-01	0.8844	0.93	0.87	0.89	53957	{}	\N	2020-10-12 16:34:45.143888
79475	XBIO	2020-10-02	0.8807	0.9	0.85	0.87	95815	{}	\N	2020-10-12 16:34:45.150092
79476	XBIO	2020-10-05	0.8694	0.9	0.856	0.867	61837	{}	\N	2020-10-12 16:34:45.156597
79477	XBIO	2020-10-06	0.9	0.9054	0.87	0.8717	97764	{}	\N	2020-10-12 16:34:45.163045
79478	XBIO	2020-10-07	0.89	0.91	0.86	0.8886	146747	{}	\N	2020-10-12 16:34:45.168229
79479	XBIO	2020-10-08	0.887	0.9729	0.8722	0.97	211067	{}	\N	2020-10-12 16:34:45.173384
79480	XBIO	2020-10-09	0.97	0.98	0.9401	0.96	75147	{}	\N	2020-10-12 16:34:45.177822
79481	XBIO	2020-10-12	0.9618	0.98	0.95	0.969799	18026	{}	\N	2020-10-12 16:34:45.182214
79482	XFLT	2020-09-28	5.91	6.04	5.91	5.98	39605	{}	\N	2020-10-12 16:34:48.987954
79483	XFLT	2020-09-29	6.03	6.03	5.98	5.985	49488	{}	\N	2020-10-12 16:34:49.000169
79484	XFLT	2020-09-30	6	6.06	5.93	5.99	159788	{}	\N	2020-10-12 16:34:49.008615
79485	XFLT	2020-10-01	6.02	6.05	6.01	6.04	88030	{}	\N	2020-10-12 16:34:49.015447
79486	XFLT	2020-10-02	6.1	6.38	6.03	6.26	503602	{}	\N	2020-10-12 16:34:49.021519
79487	XFLT	2020-10-05	6.33	7.01	6.33	6.57	308096	{}	\N	2020-10-12 16:34:49.026823
79488	XFLT	2020-10-06	6.6	6.8976	6.6	6.63	141653	{}	\N	2020-10-12 16:34:49.031792
79489	XFLT	2020-10-07	6.83	6.88	6.67	6.74	173014	{}	\N	2020-10-12 16:34:49.036695
79490	XFLT	2020-10-08	6.7	6.94	6.7	6.8641	144428	{}	\N	2020-10-12 16:34:49.041556
79491	XFLT	2020-10-09	6.82	6.8732	6.67	6.67	148876	{}	\N	2020-10-12 16:34:49.046245
79492	XFLT	2020-10-12	6.65	6.896	6.65	6.85	108426	{}	\N	2020-10-12 16:34:49.051256
79493	XLB	2020-09-28	63.35	63.99	63.09	63.47	4284384	{}	\N	2020-10-12 16:34:51.388259
79494	XLB	2020-09-29	63.51	63.71	62.83	63	5618213	{}	\N	2020-10-12 16:34:51.399687
79495	XLB	2020-09-30	63.28	64.16	63.14	63.64	6051130	{}	\N	2020-10-12 16:34:51.40766
79496	XLB	2020-10-01	63.94	64.24	62.53	62.74	14409238	{}	\N	2020-10-12 16:34:51.413857
79497	XLB	2020-10-02	62.03	63.79	61.8402	63.3	13761230	{}	\N	2020-10-12 16:34:51.419532
79498	XLB	2020-10-05	63.81	64.94	63.79	64.51	9191894	{}	\N	2020-10-12 16:34:51.391792
79499	XLB	2020-10-06	64.83	65.365	63.58	63.7	15498166	{}	\N	2020-10-12 16:34:51.397425
79500	XLB	2020-10-07	64.51	65.51	64.41	65.34	7080944	{}	\N	2020-10-12 16:34:51.402923
79501	XLB	2020-10-08	65.84	66.25	65.66	66	6150730	{}	\N	2020-10-12 16:34:51.409541
79502	XLB	2020-10-09	66.55	66.8	66.1	66.51	4578111	{}	\N	2020-10-12 16:34:51.414897
79503	XLB	2020-10-12	66.89	66.97	66.39	66.91	1769915	{}	\N	2020-10-12 16:34:51.420229
79504	WHLRP	2020-09-28	10.23	10.99	10.1	10.861	2330	{}	\N	2020-10-12 16:34:55.421402
79505	WHLRP	2020-09-29	10.12	10.49	9.71	10.437	3097	{}	\N	2020-10-12 16:34:55.429333
79506	WHLRP	2020-09-30	10.29	10.43	9.44	9.548	4282	{}	\N	2020-10-12 16:34:55.436537
79507	WHLRP	2020-10-01	9.98	10.23	9.98	10.22	1914	{}	\N	2020-10-12 16:34:55.443242
79508	WHLRP	2020-10-02	10.2199	10.89	9.5168	10.84	4726	{}	\N	2020-10-12 16:34:55.449246
79509	WHLRP	2020-10-06	10.8149	10.98	10.6098	10.98	520	{}	\N	2020-10-12 16:34:55.454996
79510	WHLRP	2020-10-07	10.822	10.99	10.822	10.99	972	{}	\N	2020-10-12 16:34:55.461759
79511	WHLRP	2020-10-09	10.1497	10.9899	10.1497	10.9899	302	{}	\N	2020-10-12 16:34:55.467689
79512	WIW	2020-09-28	11.33	11.33	11.21	11.22	164152	{}	\N	2020-10-12 16:34:56.654706
79513	WIW	2020-09-29	11.22	11.25	11.18	11.18	289488	{}	\N	2020-10-12 16:34:56.667551
79514	WIW	2020-09-30	11.2	11.29	11.2	11.24	503730	{}	\N	2020-10-12 16:34:56.675262
79515	WIW	2020-10-01	11.27	11.325	11.231	11.26	1162104	{}	\N	2020-10-12 16:34:56.681935
79516	WIW	2020-10-02	11.24	11.27	11.2	11.2	212687	{}	\N	2020-10-12 16:34:56.68714
79517	WIW	2020-10-05	11.22	11.27	11.21	11.25	162888	{}	\N	2020-10-12 16:34:56.69162
79518	WIW	2020-10-06	11.28	11.3399	11.26	11.3	183259	{}	\N	2020-10-12 16:34:56.695832
79519	WIW	2020-10-07	11.29	11.34	11.2866	11.32	249126	{}	\N	2020-10-12 16:34:56.700038
79520	WIW	2020-10-08	11.34	11.4	11.34	11.39	136310	{}	\N	2020-10-12 16:34:56.705341
79521	WIW	2020-10-09	11.41	11.47	11.37	11.47	132243	{}	\N	2020-10-12 16:34:56.709878
79522	WIW	2020-10-12	11.48	11.49	11.45	11.4501	65192	{}	\N	2020-10-12 16:34:56.714081
79523	WLTW	2020-09-28	201.18	208.52	201.17	206.85	1127040	{}	\N	2020-10-12 16:34:58.146241
79524	WLTW	2020-09-29	204.83	208.6	204.83	208.09	667570	{}	\N	2020-10-12 16:34:58.155292
79525	WLTW	2020-09-30	211.33	211.5	207.42	208.82	852271	{}	\N	2020-10-12 16:34:58.161729
79526	WLTW	2020-10-01	209.03	210.84	207.47	210.08	671418	{}	\N	2020-10-12 16:34:58.1673
79527	WLTW	2020-10-02	207.5	211.99	207.46	209.61	471483	{}	\N	2020-10-12 16:34:58.172688
79528	WLTW	2020-10-05	210.81	212.245	208.91	210.14	1379190	{}	\N	2020-10-12 16:34:58.177239
79529	WLTW	2020-10-06	211.34	213.54	208.25	209.05	489453	{}	\N	2020-10-12 16:34:58.181623
79530	WLTW	2020-10-07	210.47	212.77	210.15	211.12	867286	{}	\N	2020-10-12 16:34:58.185951
79531	WLTW	2020-10-08	211.85	216.15	210.78	211.07	382979	{}	\N	2020-10-12 16:34:58.190406
79532	WLTW	2020-10-09	211.64	213.73	211.28	211.44	669127	{}	\N	2020-10-12 16:34:58.195104
79533	WLTW	2020-10-12	211.16	217.96	211.16	217.74	205163	{}	\N	2020-10-12 16:34:58.200221
79534	WPG	2020-09-28	0.61	0.6898	0.6001	0.6752	4387559	{}	\N	2020-10-12 16:35:01.960504
79535	WPG	2020-09-29	0.66	0.675	0.611	0.6282	3571983	{}	\N	2020-10-12 16:35:01.971087
79536	WPG	2020-09-30	0.63	0.6517	0.6165	0.6474	5015846	{}	\N	2020-10-12 16:35:01.978131
79537	WPG	2020-10-01	0.6321	0.6385	0.615	0.626	4043515	{}	\N	2020-10-12 16:35:01.984785
79538	WPG	2020-10-02	0.6175	0.6269	0.6	0.6166	4327090	{}	\N	2020-10-12 16:35:01.990303
79539	WPG	2020-10-05	0.62	0.6249	0.606	0.616	2468335	{}	\N	2020-10-12 16:35:01.995443
79540	WPG	2020-10-06	0.62	0.6277	0.608	0.61	2637951	{}	\N	2020-10-12 16:35:02.000063
79541	WPG	2020-10-07	0.6058	0.62	0.5935	0.603	7236916	{}	\N	2020-10-12 16:35:02.004742
79542	WPG	2020-10-08	0.6089	0.6089	0.59	0.595	5952444	{}	\N	2020-10-12 16:35:02.010066
79543	WPG	2020-10-09	0.595	0.599599	0.59	0.595	3369426	{}	\N	2020-10-12 16:35:02.014866
79544	WPG	2020-10-12	0.59	0.5949	0.585	0.585	2259460	{}	\N	2020-10-12 16:35:02.019905
79545	WRE	2020-09-28	19.71	20.52	19.71	20.41	578455	{}	\N	2020-10-12 16:35:03.494689
79546	WRE	2020-09-29	20.3	20.35	19.81	20.11	514496	{}	\N	2020-10-12 16:35:03.50461
79547	WRE	2020-09-30	20.17	20.54	19.89	20.13	1167978	{}	\N	2020-10-12 16:35:03.51193
79548	WRE	2020-10-01	20.13	20.69	20.04	20.67	928230	{}	\N	2020-10-12 16:35:03.518577
79549	WRE	2020-10-02	20.35	21.13	20.21	21.09	511678	{}	\N	2020-10-12 16:35:03.524266
79550	WRE	2020-10-05	21.28	21.36	20.9	21.11	525465	{}	\N	2020-10-12 16:35:03.529047
79551	WRE	2020-10-06	21.29	21.685	20.91	21.21	778291	{}	\N	2020-10-12 16:35:03.533597
79552	WRE	2020-10-07	21.32	21.32	20.74	21.01	632514	{}	\N	2020-10-12 16:35:03.53795
79553	WRE	2020-10-08	21.15	21.45	21.03	21.28	428756	{}	\N	2020-10-12 16:35:03.542604
79554	WRE	2020-10-09	21.44	21.5	21.08	21.31	444722	{}	\N	2020-10-12 16:35:03.546743
79555	WRE	2020-10-12	21.06	21.35	21.01	21.16	115572	{}	\N	2020-10-12 16:35:03.550751
79556	WSC	2020-09-28	16.68	16.765	16.365	16.53	829820	{}	\N	2020-10-12 16:35:05.245138
79557	WSC	2020-09-29	16.47	16.52	16.06	16.49	767571	{}	\N	2020-10-12 16:35:05.25595
79558	WSC	2020-09-30	16.59	16.945	16.5	16.68	1103879	{}	\N	2020-10-12 16:35:05.263946
79559	WSC	2020-10-01	16.86	17.185	16.43	17.16	1375519	{}	\N	2020-10-12 16:35:05.270912
79560	WSC	2020-10-02	16.82	17.56	16.74	17.47	970635	{}	\N	2020-10-12 16:35:05.277256
79561	WSC	2020-10-05	17.64	17.94	17.4	17.42	1244493	{}	\N	2020-10-12 16:35:05.282227
79562	WSC	2020-10-06	17.59	17.99	17.42	17.43	1957645	{}	\N	2020-10-12 16:35:05.287298
79563	WSC	2020-10-07	17.65	17.95	17.65	17.83	1500287	{}	\N	2020-10-12 16:35:05.292307
79564	WSC	2020-10-08	17.99	17.99	17.43	17.63	807904	{}	\N	2020-10-12 16:35:05.297244
79565	WSC	2020-10-09	17.84	17.965	17.54	17.72	751767	{}	\N	2020-10-12 16:35:05.302007
79566	WSC	2020-10-12	17.81	18.26	17.56	18.04	772491	{}	\N	2020-10-12 16:35:05.306693
79567	WTFC	2020-09-28	39.35	40.9	39.35	40.57	358904	{}	\N	2020-10-12 16:35:06.612502
79568	WTFC	2020-09-29	40.16	40.95	39.25	40.33	365296	{}	\N	2020-10-12 16:35:06.622515
79569	WTFC	2020-09-30	40.5	41.17	39.6746	40.05	290733	{}	\N	2020-10-12 16:35:06.630574
79570	WTFC	2020-10-01	39.8	40.99	39.63	40.99	233602	{}	\N	2020-10-12 16:35:06.636448
79571	WTFC	2020-10-02	40.14	42.99	40.14	42.84	263918	{}	\N	2020-10-12 16:35:06.642468
79572	WTFC	2020-10-05	43.54	45.52	42.71	45.39	344484	{}	\N	2020-10-12 16:35:06.647157
79573	WTFC	2020-10-06	46.38	47.19	45.26	45.34	798224	{}	\N	2020-10-12 16:35:06.652317
79574	WTFC	2020-10-07	46.1	47.81	45.75	46.93	398496	{}	\N	2020-10-12 16:35:06.656683
79575	WTFC	2020-10-08	47.69	48.77	47.26	48.73	351276	{}	\N	2020-10-12 16:35:06.661081
79576	WTFC	2020-10-09	49.2	49.22	47.76	48.31	273090	{}	\N	2020-10-12 16:35:06.665852
79577	WTFC	2020-10-12	48.04	48.73	47.2675	48.5	64036	{}	\N	2020-10-12 16:35:06.67056
79578	WVFC	2020-09-28	13.2	13.38	13.12	13.15	2942	{}	\N	2020-10-12 16:35:08.537141
79579	WVFC	2020-09-29	13.2384	13.265	13.1	13.1	3958	{}	\N	2020-10-12 16:35:08.545492
79580	WVFC	2020-09-30	13.38	13.43	13.3163	13.3163	818	{}	\N	2020-10-12 16:35:08.551367
79581	WVFC	2020-10-01	13.3196	13.3196	13.18	13.197	1824	{}	\N	2020-10-12 16:35:08.556858
79582	WVFC	2020-10-05	13.38	13.38	13.38	13.38	142	{}	\N	2020-10-12 16:35:08.562123
79583	WVFC	2020-10-06	13.38	13.38	13.38	13.38	150	{}	\N	2020-10-12 16:35:08.566821
79584	WVFC	2020-10-07	13.22	13.22	13.22	13.22	271	{}	\N	2020-10-12 16:35:08.571385
79585	WVFC	2020-10-09	13.28	13.28	13.28	13.28	312	{}	\N	2020-10-12 16:35:08.575596
79586	XELA	2020-09-28	0.3749	0.3849	0.37	0.37	1895866	{}	\N	2020-10-12 16:35:09.623472
79587	XELA	2020-09-29	0.375	0.38	0.3653	0.3703	1025929	{}	\N	2020-10-12 16:35:09.633327
79588	XELA	2020-09-30	0.375	0.41	0.371601	0.3847	3274866	{}	\N	2020-10-12 16:35:09.639738
79589	XELA	2020-10-01	0.39	0.4	0.38	0.3853	1389057	{}	\N	2020-10-12 16:35:09.646904
79590	XELA	2020-10-02	0.38	0.3853	0.361	0.3807	1067694	{}	\N	2020-10-12 16:35:09.652283
79591	XELA	2020-10-05	0.3889	0.3889	0.3771	0.3811	1088836	{}	\N	2020-10-12 16:35:09.657561
79592	XELA	2020-10-06	0.38	0.3999	0.3645	0.3666	2266199	{}	\N	2020-10-12 16:35:09.662618
79593	XELA	2020-10-07	0.37	0.3866	0.36885	0.376	1341324	{}	\N	2020-10-12 16:35:09.670623
79594	XELA	2020-10-08	0.38	0.389	0.3761	0.3815	1389242	{}	\N	2020-10-12 16:35:09.67567
79595	XELA	2020-10-09	0.3961	0.427	0.3901	0.4171	6129627	{}	\N	2020-10-12 16:35:09.680566
79596	XELA	2020-10-12	0.42	0.428	0.4	0.404	1618063	{}	\N	2020-10-12 16:35:09.685416
79597	XHE	2020-09-28	95.5	96.25	95.5	95.62	16200	{}	\N	2020-10-12 16:35:11.023663
79598	XHE	2020-09-29	95.45	97.14	95.45	96.2	20953	{}	\N	2020-10-12 16:35:11.032891
79599	XHE	2020-09-30	96.26	98.25	96.26	97.36	16789	{}	\N	2020-10-12 16:35:11.039019
79600	XHE	2020-10-01	97.88	98.3642	96.48	96.81	65349	{}	\N	2020-10-12 16:35:11.045657
79601	XHE	2020-10-02	95.1	96.41	95.0376	96.18	17541	{}	\N	2020-10-12 16:35:11.05085
79602	XHE	2020-10-05	96.86	98.45	96.86	98.34	33367	{}	\N	2020-10-12 16:35:11.055657
79603	XHE	2020-10-06	98.89	100.48	98.46	98.5	28005	{}	\N	2020-10-12 16:35:11.060653
79604	XHE	2020-10-07	99.27	101.57	99.27	101.42	32349	{}	\N	2020-10-12 16:35:11.065551
79605	XHE	2020-10-08	102.19	102.19	101.2718	101.77	44932	{}	\N	2020-10-12 16:35:11.070526
79606	XHE	2020-10-09	102.82	102.82	101.8501	102.74	24727	{}	\N	2020-10-12 16:35:11.075552
79607	XHE	2020-10-12	103.24	103.62	102.53	103.335	6278	{}	\N	2020-10-12 16:35:11.080556
79608	XITK	2020-09-28	169.45	169.45	166.72	168.24	18779	{}	\N	2020-10-12 16:35:12.103438
79609	XITK	2020-09-29	168.81	169.38	168	168.61	16337	{}	\N	2020-10-12 16:35:12.115288
79610	XITK	2020-09-30	167.98	170.67	167.98	168.9	14301	{}	\N	2020-10-12 16:35:12.123384
79611	XITK	2020-10-01	171.65	174.2789	171.12	174.0608	9874	{}	\N	2020-10-12 16:35:12.130437
79612	XITK	2020-10-02	169.67	175	169.54	172.36	23589	{}	\N	2020-10-12 16:35:12.136315
79613	XITK	2020-10-05	174.53	175.9	174.09	175.85	21082	{}	\N	2020-10-12 16:35:12.141675
79614	XITK	2020-10-06	177	179.499	175.6535	176.2015	18162	{}	\N	2020-10-12 16:35:12.147409
79615	XITK	2020-10-07	178.98	179.82	177.94	179.47	23467	{}	\N	2020-10-12 16:35:12.152404
79616	XITK	2020-10-08	181.04	181.04	179.0342	179.22	11827	{}	\N	2020-10-12 16:35:12.157425
79617	XITK	2020-10-09	181.42	182.71	181.42	182.47	14590	{}	\N	2020-10-12 16:35:12.162425
79618	XITK	2020-10-12	185.66	185.66	183.5	184.3645	12076	{}	\N	2020-10-12 16:35:12.167379
79619	XLI	2020-09-28	77.45	78.49	77.45	77.87	6973768	{}	\N	2020-10-12 16:35:13.208249
79620	XLI	2020-09-29	77.8	78.05	76.96	77.18	10340301	{}	\N	2020-10-12 16:35:13.218911
79621	XLI	2020-09-30	77.48	78.39	76.41	76.98	11709385	{}	\N	2020-10-12 16:35:13.226404
79622	XLI	2020-10-01	77.52	77.87	76.21	76.75	13027107	{}	\N	2020-10-12 16:35:13.232591
79623	XLI	2020-10-02	75.5	78.055	75.47	77.61	16453913	{}	\N	2020-10-12 16:35:13.238055
79624	XLI	2020-10-05	78.27	79.025	78.27	78.86	8475785	{}	\N	2020-10-12 16:35:13.243167
79625	XLI	2020-10-06	79.19	79.715	77.82	77.9	10870361	{}	\N	2020-10-12 16:35:13.247907
79626	XLI	2020-10-07	78.82	79.855	78.79	79.62	9535203	{}	\N	2020-10-12 16:35:13.252607
79627	XLI	2020-10-08	80.15	80.415	79.44	80.41	16254171	{}	\N	2020-10-12 16:35:13.257285
79628	XLI	2020-10-09	80.91	81.26	80.46	80.71	11151237	{}	\N	2020-10-12 16:35:13.262304
79629	XLI	2020-10-12	81.07	81.35	80.83	81.31	3326244	{}	\N	2020-10-12 16:35:13.26742
79630	WMK	2020-09-28	47.92	48.495	47.59	48.28	49633	{}	\N	2020-10-12 16:35:14.598446
79631	WMK	2020-09-29	48.34	48.69	47.64	48.48	109954	{}	\N	2020-10-12 16:35:14.611074
79632	WMK	2020-09-30	48.75	48.88	47.96	48	253522	{}	\N	2020-10-12 16:35:14.618654
79633	WMK	2020-10-01	47.77	48.9	47.31	48.62	248506	{}	\N	2020-10-12 16:35:14.625423
79634	WMK	2020-10-02	48.33	49.4	48.12	49.16	162210	{}	\N	2020-10-12 16:35:14.631182
79635	WMK	2020-10-05	49.1	49.89	49.075	49.67	109221	{}	\N	2020-10-12 16:35:14.636287
79636	WMK	2020-10-06	49.98	50.27	48.94	49.31	141084	{}	\N	2020-10-12 16:35:14.641251
79637	WMK	2020-10-07	49.76	49.98	49.27	49.39	144619	{}	\N	2020-10-12 16:35:14.64739
79638	WMK	2020-10-08	49.82	50.13	49.45	49.6	245842	{}	\N	2020-10-12 16:35:14.654956
79639	WMK	2020-10-09	49.92	50.97	49.8	50.8	162048	{}	\N	2020-10-12 16:35:14.66
79640	WMK	2020-10-12	50.8	51.21	50.8	51.05	23526	{}	\N	2020-10-12 16:35:14.66527
79641	WOR	2020-09-28	40.19	41.65	39.775	40.95	433331	{}	\N	2020-10-12 16:35:16.781489
79642	WOR	2020-09-29	41.03	41.77	40.6801	41.14	629333	{}	\N	2020-10-12 16:35:16.793402
79643	WOR	2020-09-30	41.48	42.3	40.41	40.78	644094	{}	\N	2020-10-12 16:35:16.801099
79644	WOR	2020-10-01	40.77	42.63	40.76	42.59	662296	{}	\N	2020-10-12 16:35:16.807855
79645	WOR	2020-10-02	41.74	44.27	41.5416	43.71	661145	{}	\N	2020-10-12 16:35:16.813443
79646	WOR	2020-10-05	44.04	45.32	44.04	45.14	753087	{}	\N	2020-10-12 16:35:16.818723
79647	WOR	2020-10-06	45.11	46.56	44.685	44.8	1060239	{}	\N	2020-10-12 16:35:16.823456
79648	WOR	2020-10-07	45.71	47.18	45.6	46.47	671631	{}	\N	2020-10-12 16:35:16.828733
79649	WOR	2020-10-08	47.17	47.92	46.555	47.86	490439	{}	\N	2020-10-12 16:35:16.833478
79650	WOR	2020-10-09	48.28	48.5	47.42	47.83	537042	{}	\N	2020-10-12 16:35:16.837937
79651	WOR	2020-10-12	47.92	48.458	47.65	48.14	115446	{}	\N	2020-10-12 16:35:16.842715
79652	WSBC	2020-09-28	20.84	21.66	20.84	21.43	211101	{}	\N	2020-10-12 16:35:18.346257
79653	WSBC	2020-09-29	21.49	21.49	20.77	21.19	143964	{}	\N	2020-10-12 16:35:18.357838
79654	WSBC	2020-09-30	21.32	21.66	21.16	21.36	175544	{}	\N	2020-10-12 16:35:18.36526
79655	WSBC	2020-10-01	21.6	21.9217	21.14	21.7	262760	{}	\N	2020-10-12 16:35:18.371863
79656	WSBC	2020-10-02	21.29	22.44	21.1018	22.34	180016	{}	\N	2020-10-12 16:35:18.377933
79657	WSBC	2020-10-05	22.58	23.37	22.58	23.34	198769	{}	\N	2020-10-12 16:35:18.382518
79658	WSBC	2020-10-06	23.71	24.41	23.37	23.41	285483	{}	\N	2020-10-12 16:35:18.386906
79659	WSBC	2020-10-07	24.34	24.34	23.54	23.93	191877	{}	\N	2020-10-12 16:35:18.391358
79660	WSBC	2020-10-08	24.09	24.33	23.84	24.23	144262	{}	\N	2020-10-12 16:35:18.396587
79661	WSBC	2020-10-09	23.895	24.44	23.72	23.82	149639	{}	\N	2020-10-12 16:35:18.401938
79662	WSBC	2020-10-12	23.72	24.11	23.61	24	40414	{}	\N	2020-10-12 16:35:18.406588
79663	WTFCM	2020-09-28	25.88	25.99	25.81	25.93	7442	{}	\N	2020-10-12 16:35:19.40532
79664	WTFCM	2020-09-29	25.9125	26.05	25.895	26.05	4491	{}	\N	2020-10-12 16:35:19.41602
79665	WTFCM	2020-09-30	25.8	25.89	25.52	25.65	17532	{}	\N	2020-10-12 16:35:19.423137
79666	WTFCM	2020-10-01	25.62	25.76	25.52	25.69	18558	{}	\N	2020-10-12 16:35:19.429702
79667	WTFCM	2020-10-02	25.685	25.75	25.52	25.65	12757	{}	\N	2020-10-12 16:35:19.437462
79668	WTFCM	2020-10-05	25.6096	25.67	25.58	25.65	14875	{}	\N	2020-10-12 16:35:19.443098
79669	WTFCM	2020-10-06	25.68	25.7596	25.27	25.6	22042	{}	\N	2020-10-12 16:35:19.447969
79670	WTFCM	2020-10-07	25.71	25.8	25.62	25.65	8414	{}	\N	2020-10-12 16:35:19.452924
79671	WTFCM	2020-10-08	25.65	25.8	25.53	25.65	12669	{}	\N	2020-10-12 16:35:19.458279
79672	WTFCM	2020-10-09	25.65	25.717	25.59	25.65	10653	{}	\N	2020-10-12 16:35:19.463301
79673	WTFCM	2020-10-12	25.65	25.79	25.38	25.7	7877	{}	\N	2020-10-12 16:35:19.468404
79674	WVE	2020-09-28	9.36	9.65	8.6	9.34	2155447	{}	\N	2020-10-12 16:35:21.972443
79675	WVE	2020-09-29	9.28	9.47	9.02	9.19	1027779	{}	\N	2020-10-12 16:35:21.983144
79676	WVE	2020-09-30	9.19	9.56	8.17	8.49	2883439	{}	\N	2020-10-12 16:35:21.989832
79677	WVE	2020-10-01	8.49	8.68	8.05	8.18	2238935	{}	\N	2020-10-12 16:35:21.996951
79678	WVE	2020-10-02	8.01	8.19	7.9	8.04	991145	{}	\N	2020-10-12 16:35:22.003208
79679	WVE	2020-10-05	8.16	8.7201	7.95	8.51	1765821	{}	\N	2020-10-12 16:35:22.008483
79680	WVE	2020-10-06	8.69	9.1436	8.61	8.72	1670323	{}	\N	2020-10-12 16:35:22.013665
79681	WVE	2020-10-07	8.79	9.08	8.64	8.97	626794	{}	\N	2020-10-12 16:35:22.018871
79682	WVE	2020-10-08	9.1	9.18	8.87	9.02	476113	{}	\N	2020-10-12 16:35:22.023973
79683	WVE	2020-10-09	9.06	9.06	8.36	8.44	811096	{}	\N	2020-10-12 16:35:22.029129
79684	WVE	2020-10-12	8.49	9.01	8.4735	8.73	470640	{}	\N	2020-10-12 16:35:22.034307
79685	WY	2020-09-28	28.01	28.41	27.61	28.31	3483564	{}	\N	2020-10-12 16:35:23.27234
79686	WY	2020-09-29	28.41	28.605	27.85	28.05	5891500	{}	\N	2020-10-12 16:35:23.285753
79687	WY	2020-09-30	28.21	28.815	28.12	28.52	9493410	{}	\N	2020-10-12 16:35:23.293247
79688	WY	2020-10-01	28.68	28.95	28.245	28.79	8166084	{}	\N	2020-10-12 16:35:23.299903
79689	WY	2020-10-02	28.09	29.27	27.93	29.11	9471353	{}	\N	2020-10-12 16:35:23.306114
79690	WY	2020-10-05	29.21	29.84	29.11	29.73	6423727	{}	\N	2020-10-12 16:35:23.311248
79691	WY	2020-10-06	30	30	28.93	28.99	8883342	{}	\N	2020-10-12 16:35:23.315923
79692	WY	2020-10-07	29.39	29.75	29.36	29.52	5929856	{}	\N	2020-10-12 16:35:23.321125
79693	WY	2020-10-08	29.71	30.28	29.55	30.26	6172257	{}	\N	2020-10-12 16:35:23.325796
79694	WY	2020-10-09	30.51	30.54	29.165	29.32	8295326	{}	\N	2020-10-12 16:35:23.33064
79695	WY	2020-10-12	29.42	29.67	29.205	29.555	1439589	{}	\N	2020-10-12 16:35:23.335326
79696	XEL	2020-09-28	68.86	69.26	68.37	68.85	2002524	{}	\N	2020-10-12 16:35:24.411509
79697	XEL	2020-09-29	69.12	69.52	68.4	68.645	1994835	{}	\N	2020-10-12 16:35:24.420332
79698	XEL	2020-09-30	69.04	69.595	68.71	69.01	3080084	{}	\N	2020-10-12 16:35:24.427348
79699	XEL	2020-10-01	69.41	70.41	69.24	70.38	2120104	{}	\N	2020-10-12 16:35:24.433637
79700	XEL	2020-10-02	69.89	70.89	69.48	70.48	2156521	{}	\N	2020-10-12 16:35:24.438895
79701	XEL	2020-10-05	70.44	71.55	69.6	71.47	1540977	{}	\N	2020-10-12 16:35:24.443902
79702	XEL	2020-10-06	71.13	72.95	70.97	72.45	2433300	{}	\N	2020-10-12 16:35:24.44861
79703	XEL	2020-10-07	72.61	72.81	71.27	72.09	1679666	{}	\N	2020-10-12 16:35:24.453272
79704	XEL	2020-10-08	72.43	72.73	72.05	72.51	1891076	{}	\N	2020-10-12 16:35:24.457877
79705	XEL	2020-10-09	72.99	73.08	72.2342	72.71	1430978	{}	\N	2020-10-12 16:35:24.462505
79706	XEL	2020-10-12	73.17	74.08	72.88	73.94	739898	{}	\N	2020-10-12 16:35:24.46707
79707	XHS	2020-09-28	75	75.9201	75	75.67	5681	{}	\N	2020-10-12 16:35:25.746233
79708	XHS	2020-09-29	75.125	75.86	75.125	75.745	9464	{}	\N	2020-10-12 16:35:25.758966
79709	XHS	2020-09-30	76.75	77.85	76.75	77.4	22172	{}	\N	2020-10-12 16:35:25.767062
79710	XHS	2020-10-01	77.95	77.95	76.355	76.96	6658	{}	\N	2020-10-12 16:35:25.774277
79711	XHS	2020-10-02	76.655	77.875	76.6	77.45	5481	{}	\N	2020-10-12 16:35:25.780519
79712	XHS	2020-10-05	78.45	79.6465	78.45	79.6465	3236	{}	\N	2020-10-12 16:35:25.785582
79713	XHS	2020-10-06	80.04	80.65	79.1067	79.1067	3127	{}	\N	2020-10-12 16:35:25.790766
79714	XHS	2020-10-07	80.0356	80.1698	79.92	80.1698	4099	{}	\N	2020-10-12 16:35:25.79604
79715	XHS	2020-10-08	80.34	80.99	80.34	80.9	1857	{}	\N	2020-10-12 16:35:25.800842
79716	XHS	2020-10-09	82	82	80.82	80.82	2566	{}	\N	2020-10-12 16:35:25.805813
79717	XHS	2020-10-12	81.1	81.282	81.1	81.282	1326	{}	\N	2020-10-12 16:35:25.81076
79718	XLNX	2020-09-28	101.64	103.91	101.4	103.8	1351406	{}	\N	2020-10-12 16:35:26.815381
79719	XLNX	2020-09-29	103.19	104.56	102.97	103.63	889557	{}	\N	2020-10-12 16:35:26.82946
79720	XLNX	2020-09-30	103.36	105.85	103.28	104.24	1886141	{}	\N	2020-10-12 16:35:26.837271
79721	XLNX	2020-10-01	105.84	107.665	105.37	105.91	1612842	{}	\N	2020-10-12 16:35:26.844017
79722	XLNX	2020-10-02	103.14	104.3	101.96	102.59	1673019	{}	\N	2020-10-12 16:35:26.850346
79723	XLNX	2020-10-05	103.335	104.87	103.17	104.74	961699	{}	\N	2020-10-12 16:35:26.856678
79724	XLNX	2020-10-06	104.57	105.73	103.2	103.94	1328191	{}	\N	2020-10-12 16:35:26.86151
79725	XLNX	2020-10-07	104.82	106.29	104.3	105.83	1346998	{}	\N	2020-10-12 16:35:26.866666
79726	XLNX	2020-10-08	106.4	106.63	105.59	105.99	1071798	{}	\N	2020-10-12 16:35:26.873044
79727	XLNX	2020-10-09	122.56	123.78	116.52	120.94	24328692	{}	\N	2020-10-12 16:35:26.877928
79728	XLNX	2020-10-12	117.25	120	117.25	119.27	2887997	{}	\N	2020-10-12 16:35:26.882743
79729	XLY	2020-09-28	145.81	147.27	145.22	146.93	2451376	{}	\N	2020-10-12 16:35:28.071307
79730	XLY	2020-09-29	146.86	146.959	145.69	145.79	2866447	{}	\N	2020-10-12 16:35:28.083139
79731	XLY	2020-09-30	146.23	148.72	146.05	146.98	4232202	{}	\N	2020-10-12 16:35:28.090998
79732	XLY	2020-10-01	148.43	149.03	147.49	148.9	3553776	{}	\N	2020-10-12 16:35:28.098011
79733	XLY	2020-10-02	146.26	148.77	146.06	148.27	3575567	{}	\N	2020-10-12 16:35:28.103685
79734	XLY	2020-10-05	149.06	150.395	148.93	150.23	2038435	{}	\N	2020-10-12 16:35:28.10873
79735	XLY	2020-10-06	150.37	151.05	147.21	147.45	3394330	{}	\N	2020-10-12 16:35:28.114127
79736	XLY	2020-10-07	149.24	151.18	149.24	150.82	2337791	{}	\N	2020-10-12 16:35:28.119474
79737	XLY	2020-10-08	151.8	151.91	150.695	151.81	2511218	{}	\N	2020-10-12 16:35:28.124533
79738	XLY	2020-10-09	152.6	153.75	152.41	153.39	3228577	{}	\N	2020-10-12 16:35:28.130327
79739	XLY	2020-10-12	154.46	155.95	154.24	155.82	1295811	{}	\N	2020-10-12 16:35:28.136108
79740	WWW	2020-09-28	26.38	26.62	26.04	26.08	513021	{}	\N	2020-10-12 16:35:32.034975
79741	WWW	2020-09-29	25.92	26.075	25.24	25.68	1065272	{}	\N	2020-10-12 16:35:32.04498
79742	WWW	2020-09-30	25.97	26.53	25.66	25.84	1547581	{}	\N	2020-10-12 16:35:32.052532
79743	WWW	2020-10-01	25.79	26.88	25.79	26.64	976752	{}	\N	2020-10-12 16:35:32.059858
79744	WWW	2020-10-02	25.88	27.39	25.67	27.15	959906	{}	\N	2020-10-12 16:35:32.065816
79745	WWW	2020-10-05	27.41	27.83	27.03	27.67	1061760	{}	\N	2020-10-12 16:35:32.071262
79746	WWW	2020-10-06	28	28.19	26.795	26.86	1226934	{}	\N	2020-10-12 16:35:32.0763
79747	WWW	2020-10-07	27.33	27.85	27.28	27.71	987142	{}	\N	2020-10-12 16:35:32.081179
79748	WWW	2020-10-08	27.96	28.205	27.41	27.63	711462	{}	\N	2020-10-12 16:35:32.086414
79749	WWW	2020-10-09	28.05	28.3	26.99	27.24	1042847	{}	\N	2020-10-12 16:35:32.091376
79750	WWW	2020-10-12	27.26	28.05	27.03	27.83	283004	{}	\N	2020-10-12 16:35:32.09677
79751	XCEM	2020-09-28	24.1599	24.1599	23.855	23.855	1513	{}	\N	2020-10-12 16:35:34.609531
79752	XCEM	2020-09-29	23.89	24	23.75	24	2579	{}	\N	2020-10-12 16:35:34.618919
79753	XCEM	2020-09-30	24.0857	24.0857	24.0857	24.0857	41	{}	\N	2020-10-12 16:35:34.624889
79754	XCEM	2020-10-01	24.1999	24.33	24.1527	24.33	584	{}	\N	2020-10-12 16:35:34.630506
79755	XCEM	2020-10-02	24.19	24.2	24.05	24.05	2870	{}	\N	2020-10-12 16:35:34.636077
79756	XCEM	2020-10-05	24.5641	24.62	24.38	24.62	1344	{}	\N	2020-10-12 16:35:34.640665
79757	XCEM	2020-10-06	24.6499	24.6499	24.34	24.34	1033	{}	\N	2020-10-12 16:35:34.645384
79758	XCEM	2020-10-07	24.756	24.756	24.49	24.49	1131	{}	\N	2020-10-12 16:35:34.650304
79759	XCEM	2020-10-08	24.8262	24.83	24.8262	24.83	305	{}	\N	2020-10-12 16:35:34.655004
79760	XCEM	2020-10-09	25.245	25.25	25.1741	25.25	962	{}	\N	2020-10-12 16:35:34.659631
79761	XCEM	2020-10-12	25	25.24	25	25.24	2630	{}	\N	2020-10-12 16:35:34.664622
79762	XLE	2020-09-28	30.725	31.23	30.685	30.89	26413018	{}	\N	2020-10-12 16:35:35.670323
79763	XLE	2020-09-29	30.8	30.87	29.74	30.04	23754108	{}	\N	2020-10-12 16:35:35.684727
79764	XLE	2020-09-30	30.23	30.58	29.81	29.95	24221526	{}	\N	2020-10-12 16:35:35.693205
79765	XLE	2020-10-01	29.62	29.745	28.85	29.03	27942259	{}	\N	2020-10-12 16:35:35.699675
79766	XLE	2020-10-02	28.25	29.5599	28.2	29.31	28469762	{}	\N	2020-10-12 16:35:35.705658
79767	XLE	2020-10-05	29.57	30.21	29.495	30.19	30320504	{}	\N	2020-10-12 16:35:35.71071
79768	XLE	2020-10-06	30.63	30.8	29.66	29.74	30654148	{}	\N	2020-10-12 16:35:35.715203
79769	XLE	2020-10-07	29.91	30.24	29.565	30.16	20907683	{}	\N	2020-10-12 16:35:35.722886
79770	XLE	2020-10-08	30.41	31.31	30.265	31.29	30384697	{}	\N	2020-10-12 16:35:35.728134
79771	XLE	2020-10-09	31.57	31.65	30.735	30.79	25019348	{}	\N	2020-10-12 16:35:35.732726
79772	XLE	2020-10-12	30.69	30.81	30.37	30.7	8805172	{}	\N	2020-10-12 16:35:35.737272
79773	XLV	2020-09-28	104.01	104.755	103.75	103.95	6073960	{}	\N	2020-10-12 16:35:37.005949
79774	XLV	2020-09-29	104.13	104.66	103.41	103.82	5211569	{}	\N	2020-10-12 16:35:37.017339
79775	XLV	2020-09-30	104.14	106.33	104.12	105.48	7230019	{}	\N	2020-10-12 16:35:37.025063
79776	XLV	2020-10-01	105.97	106.4	104.67	105.12	8362672	{}	\N	2020-10-12 16:35:37.031846
79777	XLV	2020-10-02	104.12	105.21	103.71	104.16	7858542	{}	\N	2020-10-12 16:35:37.0378
79778	XLV	2020-10-05	104.91	106.41	104.88	106.3	6441442	{}	\N	2020-10-12 16:35:37.042775
79779	XLV	2020-10-06	106.51	106.66	104.785	104.89	6623614	{}	\N	2020-10-12 16:35:37.047451
79780	XLV	2020-10-07	105.62	107.2	105.61	106.87	8008276	{}	\N	2020-10-12 16:35:37.052216
79781	XLV	2020-10-08	107.49	107.74	107.16	107.39	5900595	{}	\N	2020-10-12 16:35:37.056914
79782	XLV	2020-10-09	107.99	108.68	107.71	108.36	4936863	{}	\N	2020-10-12 16:35:37.061011
79783	XLV	2020-10-12	108.85	109.485	108.49	109.46	2491374	{}	\N	2020-10-12 16:35:37.06528
79784	XOP	2020-09-28	43.3	44.3	42.71	43.76	7363050	{}	\N	2020-10-12 16:35:38.12664
79785	XOP	2020-09-29	43.54	43.5777	41.57	42.45	7000633	{}	\N	2020-10-12 16:35:38.139847
79786	XOP	2020-09-30	42.45	43.05	41.82	42.07	6600803	{}	\N	2020-10-12 16:35:38.148024
79787	XOP	2020-10-01	41.56	41.81	40.43	40.67	8217117	{}	\N	2020-10-12 16:35:38.15575
79788	XOP	2020-10-02	39.25	42.045	39.15	41.62	6328289	{}	\N	2020-10-12 16:35:38.162814
79789	XOP	2020-10-05	42.45	43.55	42.1	43.49	5868621	{}	\N	2020-10-12 16:35:38.168167
79790	XOP	2020-10-06	44.26	44.852	42.35	42.46	7445728	{}	\N	2020-10-12 16:35:38.173669
79791	XOP	2020-10-07	42.87	43.8	42.6	43.66	4649054	{}	\N	2020-10-12 16:35:38.179048
79792	XOP	2020-10-08	43.97	45.59	43.72	45.55	6049546	{}	\N	2020-10-12 16:35:38.184927
79793	XOP	2020-10-09	45.91	46.31	44.44	44.85	5507414	{}	\N	2020-10-12 16:35:38.190349
79794	XOP	2020-10-12	44.65	44.86	43.89	44.66	1754988	{}	\N	2020-10-12 16:35:38.195344
79795	XONE	2020-09-28	11.76	12.14	11.35	11.62	329550	{}	\N	2020-10-12 16:35:39.541258
79796	XONE	2020-09-29	11.52	12.61	11.52	12.41	449456	{}	\N	2020-10-12 16:35:39.553219
79797	XONE	2020-09-30	12.42	12.61	11.8	12.22	320198	{}	\N	2020-10-12 16:35:39.561115
79798	XONE	2020-10-01	12.3	12.44	11.89	12.11	249088	{}	\N	2020-10-12 16:35:39.567972
79799	XONE	2020-10-02	11.27	11.96	11.25	11.73	231018	{}	\N	2020-10-12 16:35:39.573541
79800	XONE	2020-10-05	11.83	12.11	11.8	11.99	160396	{}	\N	2020-10-12 16:35:39.578749
79801	XONE	2020-10-06	11.98	12.1392	11.6	11.64	181621	{}	\N	2020-10-12 16:35:39.583762
79802	XONE	2020-10-07	11.8	13.4	11.8	13.2	841262	{}	\N	2020-10-12 16:35:39.588648
79803	XONE	2020-10-08	13.44	13.96	12.66	12.97	405518	{}	\N	2020-10-12 16:35:39.593568
79804	XONE	2020-10-09	13	13.56	12.95	13.43	331506	{}	\N	2020-10-12 16:35:39.598612
79805	XONE	2020-10-12	13.61	14.69	13.61	14.36	337435	{}	\N	2020-10-12 16:35:39.603438
79806	XRX	2020-09-28	18.36	18.745	18.328	18.58	2031356	{}	\N	2020-10-12 16:35:40.898114
79807	XRX	2020-09-29	18.25	18.64	18.2345	18.46	3393761	{}	\N	2020-10-12 16:35:40.908334
79808	XRX	2020-09-30	18.5	18.9211	18.5	18.77	5345878	{}	\N	2020-10-12 16:35:40.915737
79809	XRX	2020-10-01	18.9	19.19	18.57	18.67	4572819	{}	\N	2020-10-12 16:35:40.922988
79810	XRX	2020-10-02	18.2	19.155	18.18	19.05	5194275	{}	\N	2020-10-12 16:35:40.92881
79811	XRX	2020-10-05	19.34	19.79	19.26	19.64	8431523	{}	\N	2020-10-12 16:35:40.933758
79812	XRX	2020-10-06	19.72	20.48	19.64	19.71	6136675	{}	\N	2020-10-12 16:35:40.938519
79813	XRX	2020-10-07	19.98	20.385	19.87	20.23	4744819	{}	\N	2020-10-12 16:35:40.943149
79814	XRX	2020-10-08	20.43	20.62	19.975	20.16	4835305	{}	\N	2020-10-12 16:35:40.947958
79815	XRX	2020-10-09	20.38	20.5	19.86	19.88	4427402	{}	\N	2020-10-12 16:35:40.952858
79816	XRX	2020-10-12	20.05	20.21	19.79	20.105	1581678	{}	\N	2020-10-12 16:35:40.95768
79817	WSO	2020-09-28	231.03	235.105	231.03	233.49	106538	{}	\N	2020-10-12 16:35:42.965709
79818	WSO	2020-09-29	234.32	235.11	228.88	232.38	560543	{}	\N	2020-10-12 16:35:42.978505
79819	WSO	2020-09-30	233.48	236.28	231.3034	232.89	366532	{}	\N	2020-10-12 16:35:42.986502
79820	WSO	2020-10-01	234.3	234.31	231.04	232.08	226608	{}	\N	2020-10-12 16:35:42.993678
79821	WSO	2020-10-02	229.9	232.92	229.9	230.97	265824	{}	\N	2020-10-12 16:35:42.999146
79822	WSO	2020-10-05	232.23	235.75	231.945	233.72	308388	{}	\N	2020-10-12 16:35:43.004214
79823	WSO	2020-10-06	233.66	236.82	231	232.41	330299	{}	\N	2020-10-12 16:35:43.008691
79824	WSO	2020-10-07	234.52	238.35	232.43	236.81	432145	{}	\N	2020-10-12 16:35:43.013018
79825	WSO	2020-10-08	238.32	238.75	235.22	236.48	203260	{}	\N	2020-10-12 16:35:43.017228
79826	WSO	2020-10-09	236.7	239.1	236.31	237.15	260523	{}	\N	2020-10-12 16:35:43.021489
79827	WSO	2020-10-12	238.15	240.03	237.36	239.94	72377	{}	\N	2020-10-12 16:35:43.025469
79828	WTI	2020-09-28	1.77	1.83	1.735	1.81	2554568	{}	\N	2020-10-12 16:35:44.651654
79829	WTI	2020-09-29	1.81	1.83	1.75	1.83	3998245	{}	\N	2020-10-12 16:35:44.663735
79830	WTI	2020-09-30	1.82	1.88	1.79	1.8	3661521	{}	\N	2020-10-12 16:35:44.671333
79831	WTI	2020-10-01	1.75	1.82	1.73	1.81	4782673	{}	\N	2020-10-12 16:35:44.677565
79832	WTI	2020-10-02	1.74	1.85	1.74	1.84	4577626	{}	\N	2020-10-12 16:35:44.684346
79833	WTI	2020-10-05	1.9	1.93	1.84	1.87	3484230	{}	\N	2020-10-12 16:35:44.689272
79834	WTI	2020-10-06	1.87	1.8998	1.72	1.72	11086505	{}	\N	2020-10-12 16:35:44.694062
79835	WTI	2020-10-07	1.75	1.765	1.69	1.74	5773823	{}	\N	2020-10-12 16:35:44.698978
79836	WTI	2020-10-08	1.75	1.82	1.72	1.81	5315250	{}	\N	2020-10-12 16:35:44.70363
79837	WTI	2020-10-09	1.8	1.85	1.76	1.81	3157329	{}	\N	2020-10-12 16:35:44.708604
79838	WTI	2020-10-12	1.77	1.81	1.76	1.775	1646307	{}	\N	2020-10-12 16:35:44.713186
79839	WTTR	2020-09-28	3.85	4.19	3.84	4.12	229587	{}	\N	2020-10-12 16:35:46.003254
79840	WTTR	2020-09-29	4.07	4.14	3.93	4.11	806424	{}	\N	2020-10-12 16:35:46.014124
79841	WTTR	2020-09-30	4.16	4.23	3.69	3.84	499049	{}	\N	2020-10-12 16:35:46.02104
79842	WTTR	2020-10-01	3.83	3.97	3.71	3.91	572252	{}	\N	2020-10-12 16:35:46.02738
79843	WTTR	2020-10-02	3.72	3.93	3.3863	3.9	1272079	{}	\N	2020-10-12 16:35:46.033294
79844	WTTR	2020-10-05	3.92	4.09	3.92	4.09	345759	{}	\N	2020-10-12 16:35:46.039243
79845	WTTR	2020-10-06	4.15	4.22	4.01	4.04	618376	{}	\N	2020-10-12 16:35:46.044632
79846	WTTR	2020-10-07	3.98	4.1	3.71	3.94	941886	{}	\N	2020-10-12 16:35:46.050414
79847	WTTR	2020-10-08	4.07	4.07	3.86	3.96	711434	{}	\N	2020-10-12 16:35:46.056212
79848	WTTR	2020-10-09	4.02	4.1	3.77	3.78	564357	{}	\N	2020-10-12 16:35:46.062259
79849	WTTR	2020-10-12	3.78	3.89	3.75	3.82	204742	{}	\N	2020-10-12 16:35:46.068226
79850	X	2020-09-28	7.35	7.67	7.25	7.58	11909546	{}	\N	2020-10-12 16:35:49.113515
79851	X	2020-09-29	7.57	7.67	7.31	7.36	16032834	{}	\N	2020-10-12 16:35:49.127374
79852	X	2020-09-30	7.37	7.675	7.32	7.34	17878056	{}	\N	2020-10-12 16:35:49.13619
79853	X	2020-10-01	7.41	7.59	7.33	7.48	19086474	{}	\N	2020-10-12 16:35:49.142793
79854	X	2020-10-02	7.28	7.73	7.25	7.65	18233946	{}	\N	2020-10-12 16:35:49.149311
79855	X	2020-10-05	7.76	8.05	7.72	7.83	14805329	{}	\N	2020-10-12 16:35:49.155021
79856	X	2020-10-06	7.98	8.055	7.59	7.67	13920843	{}	\N	2020-10-12 16:35:49.160295
79857	X	2020-10-07	7.85	8.51	7.83	8.47	29064613	{}	\N	2020-10-12 16:35:49.165598
79858	X	2020-10-08	8.55	8.76	8.41	8.53	18972753	{}	\N	2020-10-12 16:35:49.17093
79859	X	2020-10-09	8.66	8.7781	8.185	8.36	20460555	{}	\N	2020-10-12 16:35:49.176022
79860	X	2020-10-12	8.33	8.44	8.23	8.315	5731695	{}	\N	2020-10-12 16:35:49.181113
79861	XELB	2020-09-28	0.67	0.7374	0.6699	0.7128	40390	{}	\N	2020-10-12 16:35:51.539045
79862	XELB	2020-09-29	0.71	0.763844	0.709851	0.76	29569	{}	\N	2020-10-12 16:35:51.553254
79863	XELB	2020-09-30	0.77	0.783	0.72	0.7661	58646	{}	\N	2020-10-12 16:35:51.561183
79864	XELB	2020-10-01	0.7554	0.77	0.75	0.7601	14055	{}	\N	2020-10-12 16:35:51.567495
79865	XELB	2020-10-02	0.75	0.79	0.690184	0.7839	15325	{}	\N	2020-10-12 16:35:51.57353
79866	XELB	2020-10-05	0.7604	0.786344	0.7383	0.7405	21646	{}	\N	2020-10-12 16:35:51.579108
79867	XELB	2020-10-06	0.7383	0.78	0.716951	0.74	22388	{}	\N	2020-10-12 16:35:51.584641
79868	XELB	2020-10-07	0.728499	0.8099	0.7201	0.7501	75736	{}	\N	2020-10-12 16:35:51.589911
79869	XELB	2020-10-08	0.78	0.79	0.732	0.768	42367	{}	\N	2020-10-12 16:35:51.595755
79870	XELB	2020-10-09	0.78	0.7888	0.7415	0.7888	77083	{}	\N	2020-10-12 16:35:51.600921
79871	XELB	2020-10-12	0.79	0.9	0.75	0.85	156638	{}	\N	2020-10-12 16:35:51.606297
79872	XLRE	2020-09-28	35.12	35.42	35.06	35.27	2597936	{}	\N	2020-10-12 16:35:53.944848
79873	XLRE	2020-09-29	35.22	35.32	34.88	35.09	2043786	{}	\N	2020-10-12 16:35:53.958475
79874	XLRE	2020-09-30	35.26	35.6	34.92	35.27	4052541	{}	\N	2020-10-12 16:35:53.966156
79875	XLRE	2020-10-01	35.4	35.86	35.17	35.85	5014147	{}	\N	2020-10-12 16:35:53.973062
79876	XLRE	2020-10-02	35.31	36.54	35.29	36.43	3873469	{}	\N	2020-10-12 16:35:53.979072
79877	XLRE	2020-10-05	36.53	36.68	35.885	36.63	2060434	{}	\N	2020-10-12 16:35:53.984539
79878	XLRE	2020-10-06	36.73	36.83	36.19	36.32	3548700	{}	\N	2020-10-12 16:35:53.990216
79879	XLRE	2020-10-07	36.56	36.655	36.29	36.42	1967142	{}	\N	2020-10-12 16:35:53.995905
79880	XLRE	2020-10-08	36.65	37.21	36.6	37.03	2247598	{}	\N	2020-10-12 16:35:54.001595
79881	XLRE	2020-10-09	37.24	37.25	36.7887	36.93	2110308	{}	\N	2020-10-12 16:35:54.008535
79882	XLRE	2020-10-12	37	37.1	36.76	37.01	1463622	{}	\N	2020-10-12 16:35:54.014299
79883	XME	2020-09-28	23.55	24.1	23.38	23.8	2603267	{}	\N	2020-10-12 16:35:55.303168
79884	XME	2020-09-29	23.84	23.95	23.1	23.33	1665983	{}	\N	2020-10-12 16:35:55.315576
79885	XME	2020-09-30	23.35	23.76	23.09	23.24	1241913	{}	\N	2020-10-12 16:35:55.323694
79886	XME	2020-10-01	23.33	23.605	23.095	23.5	2038873	{}	\N	2020-10-12 16:35:55.3297
79887	XME	2020-10-02	23.04	24.1189	22.95	24.01	1987216	{}	\N	2020-10-12 16:35:55.335352
79888	XME	2020-10-05	24.2	24.77	24.2	24.62	1757029	{}	\N	2020-10-12 16:35:55.34017
79889	XME	2020-10-06	24.88	24.99	23.96	23.97	2288227	{}	\N	2020-10-12 16:35:55.345194
79890	XME	2020-10-07	24.37	24.89	24.3605	24.77	1462203	{}	\N	2020-10-12 16:35:55.350427
79891	XME	2020-10-08	25	25.28	24.91	25.23	1241329	{}	\N	2020-10-12 16:35:55.357057
79892	XME	2020-10-09	25.63	25.72	24.95	25.36	1442776	{}	\N	2020-10-12 16:35:55.362055
79893	XME	2020-10-12	25.36	25.495	25.16	25.33	535323	{}	\N	2020-10-12 16:35:55.366742
79894	XNCR	2020-09-28	38.18	38.54	37.59	38.285	199373	{}	\N	2020-10-12 16:35:58.274132
79895	XNCR	2020-09-29	38.35	39.26	38	39.16	176245	{}	\N	2020-10-12 16:35:58.283775
79896	XNCR	2020-09-30	39.37	39.6	38.29	38.79	409889	{}	\N	2020-10-12 16:35:58.289774
79897	XNCR	2020-10-01	39.14	40.11	38.88	40.06	269312	{}	\N	2020-10-12 16:35:58.296459
79898	XNCR	2020-10-02	39.26	39.26	36.62	36.63	415689	{}	\N	2020-10-12 16:35:58.301666
79899	XNCR	2020-10-05	37.44	38.75	36.7	38.7	378368	{}	\N	2020-10-12 16:35:58.306184
79900	XNCR	2020-10-06	39	41.14	38.88	40.15	354462	{}	\N	2020-10-12 16:35:58.310856
79901	XNCR	2020-10-07	40.33	41.943	40.11	41.56	333719	{}	\N	2020-10-12 16:35:58.315678
79902	XNCR	2020-10-08	41.8	41.96	41.015	41.21	564553	{}	\N	2020-10-12 16:35:58.319983
79903	XNCR	2020-10-09	41.6	41.97	41.065	41.95	165587	{}	\N	2020-10-12 16:35:58.324634
79904	XNCR	2020-10-12	42.37	42.51	41.34	41.99	43009	{}	\N	2020-10-12 16:35:58.329758
79905	XOMA	2020-09-28	18.9	19.13	18.66	18.94	10119	{}	\N	2020-10-12 16:35:59.635244
79906	XOMA	2020-09-29	18.835	18.94	18.04	18.72	16657	{}	\N	2020-10-12 16:35:59.645647
79907	XOMA	2020-09-30	18.64	19.39	18.506	18.84	17318	{}	\N	2020-10-12 16:35:59.652131
79908	XOMA	2020-10-01	19.01	19.68	18.58	19.34	14730	{}	\N	2020-10-12 16:35:59.659635
79909	XOMA	2020-10-02	18.89	19.88	18.82	19.66	42343	{}	\N	2020-10-12 16:35:59.665012
79910	XOMA	2020-10-05	19.79	20	18.96	19.38	11103	{}	\N	2020-10-12 16:35:59.669755
79911	XOMA	2020-10-06	19.255	19.97	19.18	19.26	16906	{}	\N	2020-10-12 16:35:59.674576
79912	XOMA	2020-10-07	19.44	20.94	18.91	20.42	41789	{}	\N	2020-10-12 16:35:59.679155
79913	XOMA	2020-10-08	20.42	20.42	19.26	20.23	18643	{}	\N	2020-10-12 16:35:59.683686
79914	XOMA	2020-10-09	20.43	20.43	19.55	19.77	10155	{}	\N	2020-10-12 16:35:59.688008
79915	XOMA	2020-10-12	19.6844	20.32	19.6844	19.98	8997	{}	\N	2020-10-12 16:35:59.692553
79916	XPL	2020-09-28	0.405	0.407399	0.381	0.4	46278	{}	\N	2020-10-12 16:36:00.685878
79917	XPL	2020-09-29	0.4	0.4001	0.3901	0.3918	78821	{}	\N	2020-10-12 16:36:00.697844
79918	XPL	2020-09-30	0.4	0.4258	0.4	0.4	160125	{}	\N	2020-10-12 16:36:00.70585
79919	XPL	2020-10-01	0.4084	0.416	0.38221	0.38619	82458	{}	\N	2020-10-12 16:36:00.712361
79920	XPL	2020-10-02	0.38	0.4088	0.3664	0.403	102609	{}	\N	2020-10-12 16:36:00.7223
79921	XPL	2020-10-05	0.4209	0.462	0.395	0.4489	419210	{}	\N	2020-10-12 16:36:00.727727
79922	XPL	2020-10-06	0.46	0.46	0.42	0.4375	347646	{}	\N	2020-10-12 16:36:00.732954
79923	XPL	2020-10-07	0.42	0.8799	0.42	0.6172	40838831	{}	\N	2020-10-12 16:36:00.738793
79924	XPL	2020-10-08	0.55	0.63	0.46	0.4719	7240259	{}	\N	2020-10-12 16:36:00.744965
79925	XPL	2020-10-09	0.494	0.5	0.416	0.438	1850543	{}	\N	2020-10-12 16:36:00.750603
79926	XPL	2020-10-12	0.45	0.4559	0.425	0.4372	1297707	{}	\N	2020-10-12 16:36:00.755981
79927	XRT	2020-09-28	49.52	50.15	49.48	50.08	3510093	{}	\N	2020-10-12 16:36:02.34211
79928	XRT	2020-09-29	50.04	50.07	49.08	49.31	2109117	{}	\N	2020-10-12 16:36:02.350167
79929	XRT	2020-09-30	49.5	50.19	49.29	49.66	2960872	{}	\N	2020-10-12 16:36:02.356072
79930	XRT	2020-10-01	49.96	51.08	49.91	51.01	4893557	{}	\N	2020-10-12 16:36:02.361735
79931	XRT	2020-10-02	49.84	51.455	49.6602	51.29	4543869	{}	\N	2020-10-12 16:36:02.367021
79932	XRT	2020-10-05	51.63	52.83	51.63	52.77	4927326	{}	\N	2020-10-12 16:36:02.3718
79933	XRT	2020-10-06	53.11	53.325	51.54	51.76	5535018	{}	\N	2020-10-12 16:36:02.376319
79934	XRT	2020-10-07	52.5	53.335	52.48	53.18	3139805	{}	\N	2020-10-12 16:36:02.381367
79935	XRT	2020-10-08	53.52	54.02	53.02	54.01	2102397	{}	\N	2020-10-12 16:36:02.385986
79936	XRT	2020-10-09	54.32	54.6099	53.95	54.04	3068356	{}	\N	2020-10-12 16:36:02.390897
79937	XRT	2020-10-12	54.26	54.56	54.1	54.23	763738	{}	\N	2020-10-12 16:36:02.395559
79938	CSD	2020-09-28	42.74	43.6442	42.74	43.6442	681	{}	\N	2020-10-12 16:36:03.58394
79939	CSD	2020-09-29	43.6588	43.6588	43.05	43.2673	3092	{}	\N	2020-10-12 16:36:03.595417
79940	CSD	2020-09-30	43.7401	43.94	43.4232	43.4232	2897	{}	\N	2020-10-12 16:36:03.602027
79941	CSD	2020-10-01	43.6	43.91	43.49	43.5	3079	{}	\N	2020-10-12 16:36:03.607646
79942	CSD	2020-10-02	42.7362	44.3139	42.7362	44.29	7090	{}	\N	2020-10-12 16:36:03.612844
79943	CSD	2020-10-05	44.25	45.0483	44.25	45.0483	1835	{}	\N	2020-10-12 16:36:03.618185
79944	CSD	2020-10-06	45.31	45.8111	44.7504	44.7504	1260	{}	\N	2020-10-12 16:36:03.622718
79945	CSD	2020-10-07	45.45	45.5528	45.45	45.5528	553	{}	\N	2020-10-12 16:36:03.6276
79946	CSD	2020-10-08	45.95	46.0895	45.75	46.0895	3410	{}	\N	2020-10-12 16:36:03.631995
79947	CSD	2020-10-09	46.16	46.4128	45.9	46.16	2477	{}	\N	2020-10-12 16:36:03.635929
79948	CSD	2020-10-12	46.285	46.285	46.285	46.285	172	{}	\N	2020-10-12 16:36:03.640624
79949	XHB	2020-09-28	52.67	53.77	52.66	53.63	2084983	{}	\N	2020-10-12 16:36:04.843186
79950	XHB	2020-09-29	53.73	53.89	53.07	53.22	2740900	{}	\N	2020-10-12 16:36:04.853363
79951	XHB	2020-09-30	53.2	54.53	53.2	53.88	3833367	{}	\N	2020-10-12 16:36:04.860331
79952	XHB	2020-10-01	54.37	54.82	53.87	54.69	3445578	{}	\N	2020-10-12 16:36:04.866383
79953	XHB	2020-10-02	53.66	55.44	53.5	55.1	3497896	{}	\N	2020-10-12 16:36:04.87136
79954	XHB	2020-10-05	55.58	56.04	55.145	55.85	2701265	{}	\N	2020-10-12 16:36:04.876248
79955	XHB	2020-10-06	56	56.19	54.69	54.8	3879628	{}	\N	2020-10-12 16:36:04.880965
79956	XHB	2020-10-07	55.5	56.06	55.27	55.51	1725847	{}	\N	2020-10-12 16:36:04.885152
79957	XHB	2020-10-08	56.1	56.65	55.93	56.63	2683852	{}	\N	2020-10-12 16:36:04.889137
79958	XHB	2020-10-09	57.2	57.44	56.87	56.94	2997124	{}	\N	2020-10-12 16:36:04.892986
79959	XHB	2020-10-12	57.3	57.455	56.97	57.23	997388	{}	\N	2020-10-12 16:36:04.896866
79960	XLP	2020-09-28	63.56	64.11	63.53	63.75	5455579	{}	\N	2020-10-12 16:36:07.621147
79961	XLP	2020-09-29	63.78	63.9266	63.21	63.33	7684333	{}	\N	2020-10-12 16:36:07.631777
79962	XLP	2020-09-30	63.59	64.5	63.56	64.1	7869509	{}	\N	2020-10-12 16:36:07.638739
79963	XLP	2020-10-01	64.47	64.61	63.91	64.46	8269015	{}	\N	2020-10-12 16:36:07.64441
79964	XLP	2020-10-02	63.82	64.39	63.605	64.1	6428738	{}	\N	2020-10-12 16:36:07.649723
79965	XLP	2020-10-05	64.25	64.615	64.15	64.53	4203562	{}	\N	2020-10-12 16:36:07.654153
79966	XLP	2020-10-06	64.68	64.93	63.92	64.11	5456485	{}	\N	2020-10-12 16:36:07.658342
79967	XLP	2020-10-07	64.48	65.07	64.35	64.8	7393675	{}	\N	2020-10-12 16:36:07.662839
79968	XLP	2020-10-08	65.09	65.29	64.82	65.23	6560855	{}	\N	2020-10-12 16:36:07.667459
79969	XLP	2020-10-09	65.49	66.03	65.41	65.74	6626193	{}	\N	2020-10-12 16:36:07.672199
79970	XLP	2020-10-12	66.02	66.845	65.98	66.8	4434749	{}	\N	2020-10-12 16:36:07.676877
79971	XNET	2020-09-28	3.52	3.53	3.4269	3.5	241499	{}	\N	2020-10-12 16:36:09.130003
79972	XNET	2020-09-29	3.47	3.54	3.38	3.43	213483	{}	\N	2020-10-12 16:36:09.140136
79973	XNET	2020-09-30	3.46	3.475	3.38	3.44	144347	{}	\N	2020-10-12 16:36:09.146961
79974	XNET	2020-10-01	3.48	3.54	3.43	3.49	217574	{}	\N	2020-10-12 16:36:09.153039
79975	XNET	2020-10-02	3.42	3.46	3.35	3.38	99307	{}	\N	2020-10-12 16:36:09.158445
79976	XNET	2020-10-05	3.35	3.45	3.32	3.36	83192	{}	\N	2020-10-12 16:36:09.16317
79977	XNET	2020-10-06	3.37	3.48	3.37	3.42	143773	{}	\N	2020-10-12 16:36:09.167453
79978	XNET	2020-10-07	3.46	3.47	3.35	3.43	125801	{}	\N	2020-10-12 16:36:09.172211
79979	XNET	2020-10-08	3.23	3.33	3.16	3.16	821796	{}	\N	2020-10-12 16:36:09.176442
79980	XNET	2020-10-09	3.19	3.225	2.9	3.07	1011475	{}	\N	2020-10-12 16:36:09.180784
79981	XNET	2020-10-12	3.06	3.17	2.83	2.8582	407775	{}	\N	2020-10-12 16:36:09.184802
79982	XPH	2020-09-28	43.28	43.28	42.78	43.03	52292	{}	\N	2020-10-12 16:36:12.112042
79983	XPH	2020-09-29	42.97	43.2	42.865	42.94	30610	{}	\N	2020-10-12 16:36:12.125222
79984	XPH	2020-09-30	43.03	43.92	43.03	43.48	74637	{}	\N	2020-10-12 16:36:12.133391
79985	XPH	2020-10-01	43.69	43.93	43.51	43.68	81401	{}	\N	2020-10-12 16:36:12.139696
79986	XPH	2020-10-02	43.04	43.715	43.04	43.42	22534	{}	\N	2020-10-12 16:36:12.145146
79987	XPH	2020-10-05	44.62	45.59	44.62	45.56	64778	{}	\N	2020-10-12 16:36:12.150417
79988	XPH	2020-10-06	45.75	46.05	45.2301	45.33	43222	{}	\N	2020-10-12 16:36:12.154798
79989	XPH	2020-10-07	45.57	46.3893	45.4	46.2	44477	{}	\N	2020-10-12 16:36:12.159599
79990	XPH	2020-10-08	46.4	47.32	46.4	47.21	45818	{}	\N	2020-10-12 16:36:12.164089
79991	XPH	2020-10-09	47.54	47.69	47.345	47.48	37509	{}	\N	2020-10-12 16:36:12.168513
79992	XPH	2020-10-12	47.74	47.74	47.18	47.48	12225	{}	\N	2020-10-12 16:36:12.172973
79993	XRLV	2020-09-28	37.91	38.53	37.91	38.39	5935	{}	\N	2020-10-12 16:36:13.333217
79994	XRLV	2020-09-29	38.5247	38.5247	38.29	38.34	13525	{}	\N	2020-10-12 16:36:13.34834
79995	XRLV	2020-09-30	38.5449	38.7	38.27	38.47	4450	{}	\N	2020-10-12 16:36:13.356463
79996	XRLV	2020-10-01	38.46	38.46	38.305	38.42	1092	{}	\N	2020-10-12 16:36:13.363919
79997	XRLV	2020-10-02	38.3561	38.3561	38.2547	38.2547	772	{}	\N	2020-10-12 16:36:13.369815
79998	XRLV	2020-10-05	38.51	38.85	38.51	38.85	3179	{}	\N	2020-10-12 16:36:13.375158
79999	XRLV	2020-10-06	38.985	39.0182	38.985	39.0182	1341	{}	\N	2020-10-12 16:36:13.380166
80000	XRLV	2020-10-07	38.92	39.1116	38.92	39.0603	2733	{}	\N	2020-10-12 16:36:13.384933
80001	XRLV	2020-10-08	39.2951	39.3364	39.19	39.3364	1014	{}	\N	2020-10-12 16:36:13.389949
80002	XRLV	2020-10-09	39.5333	39.66	39.462	39.64	5016	{}	\N	2020-10-12 16:36:13.394961
80003	XRLV	2020-10-12	39.981	40.009	39.981	40.009	485	{}	\N	2020-10-12 16:36:13.399925
80004	XSHD	2020-09-28	15.62	15.849	15.62	15.7427	6234	{}	\N	2020-10-12 16:36:14.440027
80005	XSHD	2020-09-29	15.745	15.745	15.515	15.66	3306	{}	\N	2020-10-12 16:36:14.454907
80006	XSHD	2020-09-30	15.79	15.91	15.67	15.67	5697	{}	\N	2020-10-12 16:36:14.462644
80007	XSHD	2020-10-01	15.71	15.7407	15.6001	15.7407	12198	{}	\N	2020-10-12 16:36:14.468781
80008	XSHD	2020-10-02	15.46	16.025	15.46	16.025	9178	{}	\N	2020-10-12 16:36:14.474194
80009	XSHD	2020-10-05	16.202	16.2961	16.175	16.2961	9533	{}	\N	2020-10-12 16:36:14.479417
80010	XSHD	2020-10-06	16.4422	16.66	16.326	16.326	16660	{}	\N	2020-10-12 16:36:14.484218
80011	XSHD	2020-10-07	16.482	16.5359	16.375	16.5359	12176	{}	\N	2020-10-12 16:36:14.48882
80012	XSHD	2020-10-08	16.7098	16.7452	16.64	16.7452	4517	{}	\N	2020-10-12 16:36:14.493566
80013	XSHD	2020-10-09	16.76	17	16.76	16.822	89046	{}	\N	2020-10-12 16:36:14.498384
80014	XSHD	2020-10-12	16.8324	16.91	16.8324	16.91	1223	{}	\N	2020-10-12 16:36:14.503206
80015	XSOE	2020-09-28	33.22	33.23	32.99	33.13	642781	{}	\N	2020-10-12 16:36:16.14189
80016	XSOE	2020-09-29	33.1	33.22	33.0565	33.14	529925	{}	\N	2020-10-12 16:36:16.153805
80017	XSOE	2020-09-30	33.36	33.8	33.36	33.67	286898	{}	\N	2020-10-12 16:36:16.161585
80018	XSOE	2020-10-01	34.03	34.06	33.8	34.04	171157	{}	\N	2020-10-12 16:36:16.167685
80019	XSOE	2020-10-02	33.51	33.9399	33.51	33.67	449994	{}	\N	2020-10-12 16:36:16.172979
80020	XSOE	2020-10-05	33.9	34.13	33.87	34.13	352572	{}	\N	2020-10-12 16:36:16.177919
80021	XSOE	2020-10-06	34.25	34.46	34.05	34.15	1226140	{}	\N	2020-10-12 16:36:16.182459
80022	XSOE	2020-10-07	34.53	34.67	34.43	34.59	3222722	{}	\N	2020-10-12 16:36:16.187287
80023	XSOE	2020-10-08	34.72	34.88	34.62	34.8	2132255	{}	\N	2020-10-12 16:36:16.192457
80024	XSOE	2020-10-09	34.97	35.22	34.92	35.16	1498978	{}	\N	2020-10-12 16:36:16.197202
80025	XSOE	2020-10-12	35.32	35.51	35.26	35.5	627096	{}	\N	2020-10-12 16:36:16.201404
80026	XENE	2020-09-28	11	11.16	10.86	11.02	55555	{}	\N	2020-10-12 16:36:20.158805
80027	XENE	2020-09-29	10.91	11.14	10.65	10.83	51764	{}	\N	2020-10-12 16:36:20.172393
80028	XENE	2020-09-30	10.78	11.375	10.75	11.07	104614	{}	\N	2020-10-12 16:36:20.181537
80029	XENE	2020-10-01	11.17	11.39	10.87	11.19	133842	{}	\N	2020-10-12 16:36:20.188435
80030	XENE	2020-10-02	11.37	11.88	10.77	10.92	132272	{}	\N	2020-10-12 16:36:20.194783
80031	XENE	2020-10-05	11	12	11	12	144726	{}	\N	2020-10-12 16:36:20.200255
80032	XENE	2020-10-06	11.99	12	11.19	11.78	67256	{}	\N	2020-10-12 16:36:20.205602
80033	XENE	2020-10-07	11.9	12.35	11.8	12.21	70551	{}	\N	2020-10-12 16:36:20.211109
80034	XENE	2020-10-08	12.29	12.64	11.94	12.18	70478	{}	\N	2020-10-12 16:36:20.216609
80035	XENE	2020-10-09	12.27	12.27	11.615	11.85	97518	{}	\N	2020-10-12 16:36:20.221634
80036	XENE	2020-10-12	11.81	11.8517	11.65	11.727	12965	{}	\N	2020-10-12 16:36:20.226815
80037	XPO	2020-09-28	85.97	87.91	85.5	86.27	697104	{}	\N	2020-10-12 16:36:23.186851
80038	XPO	2020-09-29	86.56	87.5	85.463	85.93	1295641	{}	\N	2020-10-12 16:36:23.198236
80039	XPO	2020-09-30	85.93	87.96	83.9885	84.66	2098064	{}	\N	2020-10-12 16:36:23.20592
80040	XPO	2020-10-01	85.46	86.3	83.95	85.18	1583823	{}	\N	2020-10-12 16:36:23.212734
80041	XPO	2020-10-02	83.47	87.22	83.33	86.26	2215819	{}	\N	2020-10-12 16:36:23.218828
80042	XPO	2020-10-05	89.56	95.99	89.45	93.48	5030363	{}	\N	2020-10-12 16:36:23.224156
80043	XPO	2020-10-06	94.75	96.98	93.135	95.26	4279806	{}	\N	2020-10-12 16:36:23.228968
80044	XPO	2020-10-07	96.61	99.43	91.04	95.15	5716886	{}	\N	2020-10-12 16:36:23.233774
80045	XPO	2020-10-08	96.38	98.19	94.53	95.9	1500594	{}	\N	2020-10-12 16:36:23.238643
80046	XPO	2020-10-09	96.55	97.82	94.4	95.27	2179658	{}	\N	2020-10-12 16:36:23.243367
80047	XPO	2020-10-12	95.57	96.27	94.34	95.415	519352	{}	\N	2020-10-12 16:36:23.248025
80048	XPP	2020-09-28	56.11	56.235	55.935	56.235	8710	{}	\N	2020-10-12 16:36:25.939954
80049	XPP	2020-09-29	56.06	56.06	55.75	55.77	968	{}	\N	2020-10-12 16:36:25.950683
80050	XPP	2020-09-30	57.08	58.15	57.06	57.95	12209	{}	\N	2020-10-12 16:36:25.957508
80051	XPP	2020-10-01	59.12	59.3753	58.67	59.3753	5342	{}	\N	2020-10-12 16:36:25.96382
80052	XPP	2020-10-02	58.4	59.0501	58.1252	58.1252	3279	{}	\N	2020-10-12 16:36:25.970157
80053	XPP	2020-10-05	57.9101	58.54	57.91	58.54	7751	{}	\N	2020-10-12 16:36:25.975785
80054	XPP	2020-10-06	59.44	59.97	59.44	59.97	2170	{}	\N	2020-10-12 16:36:25.981254
80055	XPP	2020-10-07	60.36	60.78	60.28	60.67	4852	{}	\N	2020-10-12 16:36:25.986816
80056	XPP	2020-10-08	60.7	61.2602	60.7	61.2602	2540	{}	\N	2020-10-12 16:36:25.992181
80057	XPP	2020-10-09	60.84	61.64	60.83	61.51	9410	{}	\N	2020-10-12 16:36:25.99736
80058	XPP	2020-10-12	64.38	64.76	64.14	64.76	10829	{}	\N	2020-10-12 16:36:26.002613
80059	XSHQ	2020-09-28	25.4816	25.4816	25.4816	25.4816	107	{}	\N	2020-10-12 16:36:28.649357
80060	XSHQ	2020-09-29	25.3696	25.4376	25.3696	25.4376	464	{}	\N	2020-10-12 16:36:28.659648
80061	XSHQ	2020-09-30	25.4139	25.4833	25.4139	25.4833	263	{}	\N	2020-10-12 16:36:28.66715
80062	XSHQ	2020-10-01	25.73	25.7891	25.5	25.7891	2907	{}	\N	2020-10-12 16:36:28.673269
80063	XSHQ	2020-10-02	25.79	26.048	25.79	26.048	1800	{}	\N	2020-10-12 16:36:28.67917
80064	XSHQ	2020-10-05	26.5662	26.5662	26.5662	26.5662	106	{}	\N	2020-10-12 16:36:28.68434
80065	XSHQ	2020-10-06	26.6916	26.6916	26.6916	26.6916	0	{}	\N	2020-10-12 16:36:28.689375
80066	XSHQ	2020-10-07	26.982	26.982	26.982	26.982	0	{}	\N	2020-10-12 16:36:28.695154
80067	XSHQ	2020-10-08	27.3119	27.32	27.3119	27.32	7289	{}	\N	2020-10-12 16:36:28.699842
80068	XSHQ	2020-10-09	27.52	27.52	27.4043	27.4043	1156	{}	\N	2020-10-12 16:36:28.704809
80069	XSHQ	2020-10-12	27.6604	27.6604	27.6604	27.6604	273	{}	\N	2020-10-12 16:36:28.714199
80070	XSW	2020-09-28	119.23	120.25	118.93	119.83	19218	{}	\N	2020-10-12 16:36:30.097595
80071	XSW	2020-09-29	120.11	120.5	119.36	119.74	22879	{}	\N	2020-10-12 16:36:30.107329
80072	XSW	2020-09-30	119.56	121.0499	118.7	119.31	20473	{}	\N	2020-10-12 16:36:30.11432
80073	XSW	2020-10-01	120.92	121.95	120.23	121.72	35509	{}	\N	2020-10-12 16:36:30.121105
80074	XSW	2020-10-02	119.31	121.27	118.85	120.11	38991	{}	\N	2020-10-12 16:36:30.126887
80075	XSW	2020-10-05	121	122.24	121	122.24	33815	{}	\N	2020-10-12 16:36:30.132354
80076	XSW	2020-10-06	123.2466	125.159	122.3701	122.3701	37568	{}	\N	2020-10-12 16:36:30.137475
80077	XSW	2020-10-07	123.61	124.7396	123.43	124.66	17779	{}	\N	2020-10-12 16:36:30.142897
80078	XSW	2020-10-08	125.52	125.62	125.0501	125.52	12393	{}	\N	2020-10-12 16:36:30.148007
80079	XSW	2020-10-09	126.52	127.74	126.52	127.64	9833	{}	\N	2020-10-12 16:36:30.153031
80080	XSW	2020-10-12	128.755	129	127.83	128.96	7726	{}	\N	2020-10-12 16:36:30.157909
80081	XTLB	2020-09-28	1.7812	1.88	1.7812	1.85	1642	{}	\N	2020-10-12 16:36:34.539494
80082	XTLB	2020-09-29	1.89	1.8998	1.89	1.89	2380	{}	\N	2020-10-12 16:36:34.551237
80083	XTLB	2020-09-30	1.8999	1.8999	1.8999	1.8999	972	{}	\N	2020-10-12 16:36:34.558982
80084	XTLB	2020-10-01	1.855	1.855	1.8	1.85	5733	{}	\N	2020-10-12 16:36:34.56583
80085	XTLB	2020-10-02	1.88	1.88	1.81	1.83	7145	{}	\N	2020-10-12 16:36:34.571953
80086	XTLB	2020-10-05	1.88	2.025	1.8602	1.95	5790	{}	\N	2020-10-12 16:36:34.577254
80087	XTLB	2020-10-06	1.93	1.97	1.9239	1.97	7377	{}	\N	2020-10-12 16:36:34.582578
80088	XTLB	2020-10-07	1.9901	2.04	1.99	2	1763	{}	\N	2020-10-12 16:36:34.587706
80089	XTLB	2020-10-08	2.06	2.1	2.06	2.06	3717	{}	\N	2020-10-12 16:36:34.592494
80090	XTLB	2020-10-09	2	2.269	1.955	2.15	21073	{}	\N	2020-10-12 16:36:34.597253
80091	XTLB	2020-10-12	2.185	2.2999	2.185	2.26	2009	{}	\N	2020-10-12 16:36:34.601669
80092	XXII	2020-09-28	0.63	0.6441	0.60445	0.613	624843	{}	\N	2020-10-12 16:36:35.695218
80093	XXII	2020-09-29	0.617299	0.6629	0.6007	0.65	1252865	{}	\N	2020-10-12 16:36:35.707972
80094	XXII	2020-09-30	0.65	0.6637	0.6376	0.6408	467784	{}	\N	2020-10-12 16:36:35.716238
80095	XXII	2020-10-01	0.6408	0.6696	0.63	0.6557	888924	{}	\N	2020-10-12 16:36:35.722239
80096	XXII	2020-10-02	0.625	0.72	0.625	0.7062	1426669	{}	\N	2020-10-12 16:36:35.731253
80097	XXII	2020-10-05	0.73	0.74	0.68	0.7	1048642	{}	\N	2020-10-12 16:36:35.736126
80098	XXII	2020-10-06	0.7007	0.729	0.6999	0.7074	937183	{}	\N	2020-10-12 16:36:35.741019
80099	XXII	2020-10-07	0.72	0.745	0.707	0.7304	661011	{}	\N	2020-10-12 16:36:35.745204
80100	XXII	2020-10-08	0.74	0.745	0.7156	0.74	788598	{}	\N	2020-10-12 16:36:35.749318
80101	XXII	2020-10-09	0.75	0.788	0.74	0.7671	936096	{}	\N	2020-10-12 16:36:35.753274
80102	XXII	2020-10-12	0.78	0.7995	0.7663	0.7802	585363	{}	\N	2020-10-12 16:36:35.757514
80103	XIN	2020-09-28	1.74	1.8898	1.74	1.87	103247	{}	\N	2020-10-12 16:36:37.569175
80104	XIN	2020-09-29	1.88	1.92	1.83	1.83	222062	{}	\N	2020-10-12 16:36:37.580815
80105	XIN	2020-09-30	1.84	1.91	1.84	1.9	174394	{}	\N	2020-10-12 16:36:37.587613
80106	XIN	2020-10-01	1.89	1.92	1.85	1.855	136580	{}	\N	2020-10-12 16:36:37.593222
80107	XIN	2020-10-02	1.8	1.9299	1.8	1.89	115790	{}	\N	2020-10-12 16:36:37.602133
80108	XIN	2020-10-05	1.85	1.95	1.85	1.91	116140	{}	\N	2020-10-12 16:36:37.606763
80109	XIN	2020-10-06	1.93	1.99	1.93	1.97	113328	{}	\N	2020-10-12 16:36:37.611707
80110	XIN	2020-10-07	1.96	2.12	1.96	2.11	204384	{}	\N	2020-10-12 16:36:37.616742
80111	XIN	2020-10-08	2.13	2.2	2.1	2.18	154582	{}	\N	2020-10-12 16:36:37.621276
80112	XIN	2020-10-09	2.14	2.19	2.0605	2.08	183540	{}	\N	2020-10-12 16:36:37.625312
80113	XIN	2020-10-12	2.06	2.17	2.06	2.143	52582	{}	\N	2020-10-12 16:36:37.629366
80114	XLK	2020-09-28	115.83	116.13	114.87	116.12	7923876	{}	\N	2020-10-12 16:36:39.952501
80115	XLK	2020-09-29	115.97	116.46	115.48	115.73	11944593	{}	\N	2020-10-12 16:36:39.964892
80116	XLK	2020-09-30	115.67	117.89	115.62	116.7	10576932	{}	\N	2020-10-12 16:36:39.972855
80117	XLK	2020-10-01	118.27	118.48	117.18	117.93	12255336	{}	\N	2020-10-12 16:36:39.979358
80118	XLK	2020-10-02	115.03	116.94	114.48	114.94	16886397	{}	\N	2020-10-12 16:36:39.985404
80119	XLK	2020-10-05	115.76	117.53	115.69	117.46	5731479	{}	\N	2020-10-12 16:36:39.990558
80120	XLK	2020-10-06	117.18	118.29	115.23	115.64	12186387	{}	\N	2020-10-12 16:36:39.995339
80121	XLK	2020-10-07	116.85	117.99	116.755	117.76	5978194	{}	\N	2020-10-12 16:36:40.000087
80122	XLK	2020-10-08	118.79	118.79	117.97	118.37	4960763	{}	\N	2020-10-12 16:36:40.005258
80123	XLK	2020-10-09	118.93	120.21	118.9	120.18	6120745	{}	\N	2020-10-12 16:36:40.010396
80124	XLK	2020-10-12	121.95	123.495	121.34	123.43	4535266	{}	\N	2020-10-12 16:36:40.015366
80125	XMPT	2020-09-28	26.89	26.94	26.79	26.8	17248	{}	\N	2020-10-12 16:36:43.881945
80126	XMPT	2020-09-29	26.73	26.89	26.73	26.89	7509	{}	\N	2020-10-12 16:36:43.89343
80127	XMPT	2020-09-30	27	27	26.86	26.87	8770	{}	\N	2020-10-12 16:36:43.901893
80128	XMPT	2020-10-01	26.8534	26.91	26.845	26.91	12907	{}	\N	2020-10-12 16:36:43.908596
80129	XMPT	2020-10-02	26.9	26.97	26.87	26.95	13776	{}	\N	2020-10-12 16:36:43.914993
80130	XMPT	2020-10-05	26.975	26.975	26.8485	26.9	7311	{}	\N	2020-10-12 16:36:43.920265
80131	XMPT	2020-10-06	26.93	26.98	26.8701	26.9	16012	{}	\N	2020-10-12 16:36:43.926047
80132	XMPT	2020-10-07	26.96	27.0199	26.92	26.97	29862	{}	\N	2020-10-12 16:36:43.931482
80133	XMPT	2020-10-08	27.07	27.07	26.89	26.89	13779	{}	\N	2020-10-12 16:36:43.937105
80134	XMPT	2020-10-09	26.92	26.9999	26.9181	26.92	9137	{}	\N	2020-10-12 16:36:43.942888
80135	XMPT	2020-10-12	27.015	27.015	26.96	26.9999	5141	{}	\N	2020-10-12 16:36:43.948067
80136	XOM	2020-09-28	35.14	35.9	35.095	35.31	24031304	{}	\N	2020-10-12 16:36:47.665017
80137	XOM	2020-09-29	35	35.11	33.99	34.28	41632974	{}	\N	2020-10-12 16:36:47.675867
80138	XOM	2020-09-30	34.64	34.92	34.13	34.33	43239140	{}	\N	2020-10-12 16:36:47.683497
80139	XOM	2020-10-01	33.79	34.1	32.94	33.13	74252287	{}	\N	2020-10-12 16:36:47.690048
80140	XOM	2020-10-02	32.25	33.35	32.21	32.98	65528717	{}	\N	2020-10-12 16:36:47.696522
80141	XOM	2020-10-05	33.32	33.75	33.17	33.74	45980005	{}	\N	2020-10-12 16:36:47.701349
80142	XOM	2020-10-06	34.12	34.42	33.31	33.39	51301203	{}	\N	2020-10-12 16:36:47.706627
80143	XOM	2020-10-07	33.64	33.74	33.21	33.5	47609766	{}	\N	2020-10-12 16:36:47.712034
80144	XOM	2020-10-08	33.88	35.34	33.76	35.26	69409361	{}	\N	2020-10-12 16:36:47.717145
80145	XOM	2020-10-09	35.68	35.95	34.65	34.74	58436190	{}	\N	2020-10-12 16:36:47.721876
80146	XOM	2020-10-12	34.54	34.72	34.23	34.5901	17317998	{}	\N	2020-10-12 16:36:47.726563
80147	XRAY	2020-09-28	43.78	44.28	43.2	43.31	1268054	{}	\N	2020-10-12 16:36:48.670472
80148	XRAY	2020-09-29	43.4	43.67	42.74	42.75	1022704	{}	\N	2020-10-12 16:36:48.68119
80149	XRAY	2020-09-30	42.76	44.15	42.501	43.73	1120298	{}	\N	2020-10-12 16:36:48.688066
80150	XRAY	2020-10-01	44	44.6	42.74	43.51	646747	{}	\N	2020-10-12 16:36:48.694248
80151	XRAY	2020-10-02	42.79	44.12	42.67	43.66	626022	{}	\N	2020-10-12 16:36:48.700507
80152	XRAY	2020-10-05	43.94	44.92	43.94	44.81	500063	{}	\N	2020-10-12 16:36:48.705639
80153	XRAY	2020-10-06	44.91	45.28	43.96	44.11	565827	{}	\N	2020-10-12 16:36:48.710502
80154	XRAY	2020-10-07	44.81	45.18	44.23	45.05	550067	{}	\N	2020-10-12 16:36:48.715682
80155	XRAY	2020-10-08	45.31	46.15	44.775	46.06	610813	{}	\N	2020-10-12 16:36:48.720506
80156	XRAY	2020-10-09	46.3	46.35	45.36	45.5	921938	{}	\N	2020-10-12 16:36:48.72525
80157	XRAY	2020-10-12	45.72	45.92	45.35	45.85	135130	{}	\N	2020-10-12 16:36:48.733974
80158	XSLV	2020-09-28	32.99	33.4286	32.99	33.3	624690	{}	\N	2020-10-12 16:36:49.999416
80159	XSLV	2020-09-29	33.23	33.45	32.94	33.18	413443	{}	\N	2020-10-12 16:36:50.01113
80160	XSLV	2020-09-30	33.27	33.6467	32.95	33.2	517196	{}	\N	2020-10-12 16:36:50.018505
80161	XSLV	2020-10-01	33.27	33.46	32.95	33.4	560777	{}	\N	2020-10-12 16:36:50.024727
80162	XSLV	2020-10-02	32.86	33.71	32.855	33.59	532935	{}	\N	2020-10-12 16:36:50.030474
80163	XSLV	2020-10-05	33.77	34.2575	33.77	34.21	327667	{}	\N	2020-10-12 16:36:50.035415
80164	XSLV	2020-10-06	34.41	35.0185	34.28	34.4	584952	{}	\N	2020-10-12 16:36:50.040164
80165	XSLV	2020-10-07	34.59	34.8	34.42	34.62	589927	{}	\N	2020-10-12 16:36:50.044825
80166	XSLV	2020-10-08	34.86	35.19	34.83	35.13	250410	{}	\N	2020-10-12 16:36:50.049137
80167	XSLV	2020-10-09	35.36	35.4551	35.14	35.26	388944	{}	\N	2020-10-12 16:36:50.0535
80168	XSLV	2020-10-12	35.3	35.63	35.2869	35.59	150661	{}	\N	2020-10-12 16:36:50.058009
80169	XTNT	2020-09-28	0.798	0.8	0.74	0.799	41615	{}	\N	2020-10-12 16:36:51.023207
80170	XTNT	2020-09-29	0.76001	0.81	0.76001	0.79	41413	{}	\N	2020-10-12 16:36:51.03082
80171	XTNT	2020-09-30	0.8489	0.8489	0.800701	0.8237	110143	{}	\N	2020-10-12 16:36:51.036996
80172	XTNT	2020-10-01	0.8652	0.8818	0.8144	0.8203	2294758	{}	\N	2020-10-12 16:36:51.042058
80173	XTNT	2020-10-02	2.28	2.5	1.34	1.57	112439492	{}	\N	2020-10-12 16:36:51.047509
80174	XTNT	2020-10-05	1.37	1.52	1.2406	1.52	6731826	{}	\N	2020-10-12 16:36:51.052348
80175	XTNT	2020-10-06	1.95	2.28	1.5	1.53	17975719	{}	\N	2020-10-12 16:36:51.057585
80176	XTNT	2020-10-07	1.42	1.51	1.26	1.3	1897882	{}	\N	2020-10-12 16:36:51.062906
80177	XTNT	2020-10-08	1.27	1.34	1.25	1.31	676167	{}	\N	2020-10-12 16:36:51.068014
80178	XTNT	2020-10-09	1.3	1.41	1.3	1.33	699278	{}	\N	2020-10-12 16:36:51.072836
80179	XTNT	2020-10-12	1.34	1.45	1.34	1.41	219336	{}	\N	2020-10-12 16:36:51.078119
80180	XTL	2020-09-28	67.32	67.6	67.32	67.6	1698	{}	\N	2020-10-12 16:36:53.767277
80181	XTL	2020-09-29	67.17	68.2201	67.17	68.2201	15077	{}	\N	2020-10-12 16:36:53.77592
80182	XTL	2020-09-30	67.7927	67.93	67.7886	67.93	1853	{}	\N	2020-10-12 16:36:53.783048
80183	XTL	2020-10-01	67.65	67.8144	67.59	67.8144	20924	{}	\N	2020-10-12 16:36:53.790191
80184	XTL	2020-10-02	67.262	68.0001	67.262	67.92	5995	{}	\N	2020-10-12 16:36:53.795793
80185	XTL	2020-10-05	68.17	69.3559	68.17	69.3559	2375	{}	\N	2020-10-12 16:36:53.801443
80186	XTL	2020-10-06	69.8	70.24	69.038	69.038	11431	{}	\N	2020-10-12 16:36:53.806809
80187	XTL	2020-10-07	69.33	69.84	69.33	69.84	1708	{}	\N	2020-10-12 16:36:53.812183
80188	XTL	2020-10-08	70.68	70.8868	70.68	70.8868	1108	{}	\N	2020-10-12 16:36:53.818042
80189	XTL	2020-10-09	71.81	72.2199	71.81	72.13	5315	{}	\N	2020-10-12 16:36:53.823168
80190	XTL	2020-10-12	72.605	72.69	72.605	72.69	414	{}	\N	2020-10-12 16:36:53.82801
80191	XVZ	2020-09-28	40.96	41.165	40.96	41.165	1691	{}	\N	2020-10-12 16:36:55.170754
80192	XVZ	2020-09-29	40.98	41.0641	40.81	41.0641	2027	{}	\N	2020-10-12 16:36:55.182308
80193	XVZ	2020-09-30	40.8401	41.4318	40.8401	41.4318	5557	{}	\N	2020-10-12 16:36:55.190555
80194	XVZ	2020-10-01	41.37	41.37	41.2495	41.3001	1330	{}	\N	2020-10-12 16:36:55.197305
80195	XVZ	2020-10-02	43.37	43.37	41.2102	41.2102	14404	{}	\N	2020-10-12 16:36:55.203539
80196	XVZ	2020-10-05	41.57	41.57	41.1201	41.5699	1827	{}	\N	2020-10-12 16:36:55.208751
80197	XVZ	2020-10-06	41.33	41.64	41.1915	41.1915	6305	{}	\N	2020-10-12 16:36:55.213689
80198	XVZ	2020-10-07	41	41.48	41	41.47	2057	{}	\N	2020-10-12 16:36:55.21864
80199	XVZ	2020-10-08	41.15	41.4	41.15	41.4	3207	{}	\N	2020-10-12 16:36:55.223623
80200	XVZ	2020-10-09	40.52	41.18	40.4352	40.81	2624	{}	\N	2020-10-12 16:36:55.22907
80201	XVZ	2020-10-12	40.62	40.7281	40.53	40.7281	1028	{}	\N	2020-10-12 16:36:55.234132
80202	YCS	2020-09-29	71.22	71.225	71.2027	71.2027	644	{}	\N	2020-10-12 16:36:58.174925
80203	YCS	2020-09-30	71	71	70.91	70.91	2350	{}	\N	2020-10-12 16:36:58.186272
80204	YCS	2020-10-01	71.27	71.27	70.915	70.915	8222	{}	\N	2020-10-12 16:36:58.193392
80205	YCS	2020-10-02	70.18	70.841	70.18	70.841	621	{}	\N	2020-10-12 16:36:58.199917
80206	YCS	2020-10-05	71.03	71.3554	71.03	71.3554	438	{}	\N	2020-10-12 16:36:58.205584
80207	YCS	2020-10-06	71.19	71.2	71.0575	71.0575	29456	{}	\N	2020-10-12 16:36:58.210247
80208	YCS	2020-10-07	71.62	71.7072	71.62	71.7072	749	{}	\N	2020-10-12 16:36:58.214899
80209	YCS	2020-10-08	71.6588	71.6588	71.6588	71.6588	509	{}	\N	2020-10-12 16:36:58.220205
80210	YCS	2020-10-09	71.285	71.285	71.1318	71.1318	2453	{}	\N	2020-10-12 16:36:58.225209
80211	YCS	2020-10-12	70.69	70.711	70.675	70.711	1816	{}	\N	2020-10-12 16:36:58.230237
80212	XLRN	2020-09-28	109.08	109.46	107.12	108.99	232890	{}	\N	2020-10-12 16:37:01.518134
80213	XLRN	2020-09-29	109.15	113.71	108.17	113.11	438024	{}	\N	2020-10-12 16:37:01.529722
80214	XLRN	2020-09-30	113.73	114.87	111.52	112.53	399562	{}	\N	2020-10-12 16:37:01.537439
80215	XLRN	2020-10-01	113.6	114.71	110.82	114.53	368352	{}	\N	2020-10-12 16:37:01.544221
80216	XLRN	2020-10-02	113.24	113.87	105.75	106.48	364966	{}	\N	2020-10-12 16:37:01.550576
80217	XLRN	2020-10-05	107.61	110.69	106.47	110.11	374409	{}	\N	2020-10-12 16:37:01.555322
80218	XLRN	2020-10-06	110.41	112.09	108.69	109.45	422152	{}	\N	2020-10-12 16:37:01.560088
80219	XLRN	2020-10-07	110.32	113.53	109.06	112.82	161961	{}	\N	2020-10-12 16:37:01.564672
80220	XLRN	2020-10-08	113.3	114.72	112.62	114.19	183920	{}	\N	2020-10-12 16:37:01.569275
80221	XLRN	2020-10-09	114.72	119.83	114.31	118.98	292255	{}	\N	2020-10-12 16:37:01.573918
80222	XLRN	2020-10-12	119.99	120.81	117.27	119.35	94675	{}	\N	2020-10-12 16:37:01.578416
80223	XT	2020-09-28	48.23	48.41	48.127	48.39	127913	{}	\N	2020-10-12 16:37:04.161116
80224	XT	2020-09-29	48.45	48.72	48.37	48.45	137277	{}	\N	2020-10-12 16:37:04.172692
80225	XT	2020-09-30	48.47	49	48.42	48.58	151559	{}	\N	2020-10-12 16:37:04.180182
80226	XT	2020-10-01	48.99	49.21	48.85	49.1	189759	{}	\N	2020-10-12 16:37:04.186267
80227	XT	2020-10-02	48.42	48.94	48.42	48.6	121235	{}	\N	2020-10-12 16:37:04.191642
80228	XT	2020-10-05	49.03	49.6799	49.03	49.66	293583	{}	\N	2020-10-12 16:37:04.196645
80229	XT	2020-10-06	49.75	50.01	49.09	49.17	130393	{}	\N	2020-10-12 16:37:04.20095
80230	XT	2020-10-07	49.66	50.03	49.62	49.91	450775	{}	\N	2020-10-12 16:37:04.205275
80231	XT	2020-10-08	50.36	50.55	50.335	50.51	487145	{}	\N	2020-10-12 16:37:04.209872
80232	XT	2020-10-09	50.79	51.06	50.78	50.97	210136	{}	\N	2020-10-12 16:37:04.214198
80233	XT	2020-10-12	51.4	51.55	51.28	51.5427	74385	{}	\N	2020-10-12 16:37:04.218531
80234	XTN	2020-09-28	58.77	59.66	58.75	59.281	42423	{}	\N	2020-10-12 16:37:07.57919
80235	XTN	2020-09-29	59.26	59.26	58.22	58.33	152984	{}	\N	2020-10-12 16:37:07.588359
80236	XTN	2020-09-30	58.85	59.43	57.95	58.1515	26196	{}	\N	2020-10-12 16:37:07.595035
80237	XTN	2020-10-01	58.46	58.718	58.125	58.43	25459	{}	\N	2020-10-12 16:37:07.601322
80238	XTN	2020-10-02	57.01	59.55	57.01	59.3752	29583	{}	\N	2020-10-12 16:37:07.607088
80239	XTN	2020-10-05	59.79	60.25	59.62	60.08	14604	{}	\N	2020-10-12 16:37:07.612183
80240	XTN	2020-10-06	60.34	60.915	59.28	59.28	48151	{}	\N	2020-10-12 16:37:07.616993
80241	XTN	2020-10-07	60.17	60.96	60.17	60.8063	17179	{}	\N	2020-10-12 16:37:07.621788
80242	XTN	2020-10-08	61.38	61.7	61	61.5456	41463	{}	\N	2020-10-12 16:37:07.626537
80243	XTN	2020-10-09	61.77	62.13	61.377	61.84	23182	{}	\N	2020-10-12 16:37:07.631443
80244	XTN	2020-10-12	62.25	62.25	61.77	61.9278	19845	{}	\N	2020-10-12 16:37:07.636173
80245	Y	2020-09-28	513.57	523.24	513.57	517.86	48763	{}	\N	2020-10-12 16:37:12.09456
80246	Y	2020-09-29	515.77	522.6	510.24	520.28	123824	{}	\N	2020-10-12 16:37:12.103573
80247	Y	2020-09-30	520.31	527.59	514.43	520.45	127283	{}	\N	2020-10-12 16:37:12.110008
80248	Y	2020-10-01	519.27	527.05	515.58	525.74	103051	{}	\N	2020-10-12 16:37:12.115864
80249	Y	2020-10-02	515.94	535.65	515.94	530.24	77229	{}	\N	2020-10-12 16:37:12.120805
80250	Y	2020-10-05	537.3	544.62	537.3	543.2	79947	{}	\N	2020-10-12 16:37:12.125719
80251	Y	2020-10-06	544.34	550.92	538.16	538.93	106009	{}	\N	2020-10-12 16:37:12.1304
80252	Y	2020-10-07	544.66	544.98	537.865	539.32	102469	{}	\N	2020-10-12 16:37:12.134971
80253	Y	2020-10-08	539.81	558.82	535.72	553.87	59050	{}	\N	2020-10-12 16:37:12.139547
80254	Y	2020-10-09	553.98	564.1	552.26	558.42	85951	{}	\N	2020-10-12 16:37:12.144406
80255	Y	2020-10-12	555.03	568.53	555.03	566.09	18907	{}	\N	2020-10-12 16:37:12.149842
80256	YCL	2020-09-28	57.64	57.676	57.64	57.676	1091	{}	\N	2020-10-12 16:37:13.634031
80257	YCL	2020-09-29	57.4793	57.4793	57.4793	57.4793	182	{}	\N	2020-10-12 16:37:13.644481
80258	YCL	2020-09-30	57.4508	57.75	57.4508	57.75	2202	{}	\N	2020-10-12 16:37:13.652469
80259	YCL	2020-10-01	57.47	57.59	57.47	57.59	6956	{}	\N	2020-10-12 16:37:13.658738
80260	YCL	2020-10-02	57.7617	57.7617	57.7617	57.7617	7	{}	\N	2020-10-12 16:37:13.664357
80261	YCL	2020-10-05	57.3359	57.3359	57.3359	57.3359	113	{}	\N	2020-10-12 16:37:13.670255
80262	YCL	2020-10-06	57.581	57.581	57.581	57.581	112	{}	\N	2020-10-12 16:37:13.675832
80263	YCL	2020-10-07	57.09	57.0902	57.0499	57.0499	653	{}	\N	2020-10-12 16:37:13.68434
80264	YCL	2020-10-08	57.0819	57.0819	57.0819	57.0819	54	{}	\N	2020-10-12 16:37:13.691911
80265	YCL	2020-10-09	57.4999	57.4999	57.4999	57.4999	0	{}	\N	2020-10-12 16:37:13.697507
80266	SPY	2020-09-28	333.22	334.96	332.15	334.19	63883419	{}	\N	2020-10-12 16:37:15.439342
80267	SPY	2020-09-29	333.97	334.77	331.6209	332.37	50472963	{}	\N	2020-10-12 16:37:15.449102
80268	SPY	2020-09-30	333.09	338.29	332.88	334.89	103402933	{}	\N	2020-10-12 16:37:15.455195
80269	SPY	2020-10-01	337.69	338.74	335.01	337.04	88066495	{}	\N	2020-10-12 16:37:15.460756
80270	SPY	2020-10-02	331.7	337.0126	331.19	333.84	85702649	{}	\N	2020-10-12 16:37:15.466142
80271	SPY	2020-10-05	336.06	339.96	336.01	339.76	44517218	{}	\N	2020-10-12 16:37:15.470719
80272	SPY	2020-10-06	339.92	342.17	334.38	334.93	83825650	{}	\N	2020-10-12 16:37:15.475528
80273	SPY	2020-10-07	338.12	341.63	338.09	340.76	55345401	{}	\N	2020-10-12 16:37:15.480327
80274	SPY	2020-10-08	342.85	343.85	341.86	343.78	44155427	{}	\N	2020-10-12 16:37:15.485354
80275	SPY	2020-10-09	345.56	347.35	344.89	346.85	56054642	{}	\N	2020-10-12 16:37:15.489738
80276	SPY	2020-10-12	349.59	353.03	349.06	352.85	24162093	{}	\N	2020-10-12 16:37:15.493939
80277	SQNS	2020-09-28	5.9	5.95	5.7901	5.9	109878	{}	\N	2020-10-12 16:37:19.117773
80278	SQNS	2020-09-29	5.85	5.94	5.77	5.77	164540	{}	\N	2020-10-12 16:37:19.129707
80279	SQNS	2020-09-30	5.84	6.09	5.8156	5.97	188051	{}	\N	2020-10-12 16:37:19.137202
80280	SQNS	2020-10-01	6	6.053	5.9483	6	101739	{}	\N	2020-10-12 16:37:19.14391
80281	SQNS	2020-10-02	5.81	6	5.79	5.79	122366	{}	\N	2020-10-12 16:37:19.149457
80282	SQNS	2020-10-05	5.97	6	5.81	5.91	208006	{}	\N	2020-10-12 16:37:19.154395
80283	SQNS	2020-10-06	5.84	5.98	5.8	5.9	128421	{}	\N	2020-10-12 16:37:19.158816
80284	SQNS	2020-10-07	5.87	6	5.67	5.71	277146	{}	\N	2020-10-12 16:37:19.163137
80285	SQNS	2020-10-08	5.71	5.8	5.61	5.65	256920	{}	\N	2020-10-12 16:37:19.167611
80286	SQNS	2020-10-09	5.61	5.7	5.45	5.5	283878	{}	\N	2020-10-12 16:37:19.171887
80287	SQNS	2020-10-12	5.54	5.57	5.49	5.53	99094	{}	\N	2020-10-12 16:37:19.176331
80288	SRRA	2020-09-28	10.95	11.38	10.81	10.91	3780	{}	\N	2020-10-12 16:37:20.169398
80289	SRRA	2020-09-29	10.74	10.8102	10.74	10.795	2985	{}	\N	2020-10-12 16:37:20.1816
80290	SRRA	2020-09-30	11.3	11.3	10.58	10.67	4997	{}	\N	2020-10-12 16:37:20.189533
80291	SRRA	2020-10-01	10.6498	11.02	10.6498	11.02	1113	{}	\N	2020-10-12 16:37:20.196123
80292	SRRA	2020-10-02	10.65	10.9	10.6	10.6	6264	{}	\N	2020-10-12 16:37:20.201833
80293	SRRA	2020-10-05	10.875	11.25	10.52	10.8	11246	{}	\N	2020-10-12 16:37:20.206458
80294	SRRA	2020-10-06	10.6957	10.85	10.51	10.53	5427	{}	\N	2020-10-12 16:37:20.212085
80295	SRRA	2020-10-07	10.65	10.84	10.55	10.55	3133	{}	\N	2020-10-12 16:37:20.217395
80296	SRRA	2020-10-08	10.745	10.81	10.63	10.72	1728	{}	\N	2020-10-12 16:37:20.222519
80297	SRRA	2020-10-09	10.76	12.8337	10.71	11.75	604810	{}	\N	2020-10-12 16:37:20.227542
80298	SRRA	2020-10-12	12.19	12.19	11.753	11.855	9041	{}	\N	2020-10-12 16:37:20.233167
80299	SSBI	2020-09-28	10.9	10.9612	10.8999	10.9612	8709	{}	\N	2020-10-12 16:37:21.100372
80300	SSBI	2020-09-29	10.75	10.76	10.75	10.75	2802	{}	\N	2020-10-12 16:37:21.111693
80301	SSBI	2020-09-30	10.6227	11.5	10.51	11.15	10302	{}	\N	2020-10-12 16:37:21.119691
80302	SSBI	2020-10-01	11.12	11.208	10.59	10.59	9537	{}	\N	2020-10-12 16:37:21.126269
80303	SSBI	2020-10-05	10.65	11.12	10.63	10.79	2339	{}	\N	2020-10-12 16:37:21.133261
80304	SSBI	2020-10-06	10.79	11.34	10.79	10.97	9091	{}	\N	2020-10-12 16:37:21.137937
80305	SSBI	2020-10-07	11.1727	11.1727	11.1727	11.1727	361	{}	\N	2020-10-12 16:37:21.142526
80306	SSBI	2020-10-08	11.06	11.77	11.06	11.77	5745	{}	\N	2020-10-12 16:37:21.147046
80307	SSBI	2020-10-09	11.7	12.32	11.7	12.32	696	{}	\N	2020-10-12 16:37:21.151762
80308	SSBI	2020-10-12	12.38	12.5	12.2	12.31	5152	{}	\N	2020-10-12 16:37:21.156365
80309	SSNC	2020-09-28	61.58	62.15	61.26	61.42	735934	{}	\N	2020-10-12 16:37:22.109563
80310	SSNC	2020-09-29	61.38	61.6	60.43	60.5	654297	{}	\N	2020-10-12 16:37:22.119833
80311	SSNC	2020-09-30	60.64	61.35	60.1	60.52	762163	{}	\N	2020-10-12 16:37:22.126644
80312	SSNC	2020-10-01	61	61.85	61	61.42	1067237	{}	\N	2020-10-12 16:37:22.132445
80313	SSNC	2020-10-02	60.33	61.84	60.33	61.61	870639	{}	\N	2020-10-12 16:37:22.137829
80314	SSNC	2020-10-05	62.37	63.28	62.21	63.25	1242430	{}	\N	2020-10-12 16:37:22.14258
80315	SSNC	2020-10-06	63.29	63.885	62.24	62.52	1290491	{}	\N	2020-10-12 16:37:22.147163
80316	SSNC	2020-10-07	62.94	63.9	62.59	63.02	991996	{}	\N	2020-10-12 16:37:22.151814
80317	SSNC	2020-10-08	63.38	63.51	62.92	63.32	1202362	{}	\N	2020-10-12 16:37:22.156397
80318	SSNC	2020-10-09	63.87	64.63	63.59	64.09	857447	{}	\N	2020-10-12 16:37:22.16081
80319	SSNC	2020-10-12	64.24	64.83	64.24	64.67	174731	{}	\N	2020-10-12 16:37:22.165339
80320	SSY	2020-09-28	1.02	1.02	0.9021	0.92	45187	{}	\N	2020-10-12 16:37:24.413646
80321	SSY	2020-09-29	0.9398	0.98	0.87	0.97	18021	{}	\N	2020-10-12 16:37:24.424381
80322	SSY	2020-09-30	0.9305	0.98	0.8891	0.95	4387	{}	\N	2020-10-12 16:37:24.43174
80323	SSY	2020-10-01	0.98	0.9859	0.96	0.9859	20375	{}	\N	2020-10-12 16:37:24.439354
80324	SSY	2020-10-02	0.9859	0.99	0.9413	0.9601	6835	{}	\N	2020-10-12 16:37:24.444817
80325	SSY	2020-10-05	0.951	0.9998	0.951	0.98	31445	{}	\N	2020-10-12 16:37:24.45007
80326	SSY	2020-10-06	0.9951	1.04	0.942	0.9702	22071	{}	\N	2020-10-12 16:37:24.455313
80327	SSY	2020-10-07	0.9819	1.04	0.977	1.03	3260	{}	\N	2020-10-12 16:37:24.461072
80328	SSY	2020-10-08	1.04	1.04	1.0084	1.0201	8423	{}	\N	2020-10-12 16:37:24.466701
80329	SSY	2020-10-09	1.05	1.09	1.01	1.0401	17532	{}	\N	2020-10-12 16:37:24.472014
80330	SSY	2020-10-12	1.07	1.08	1.06	1.07	6488	{}	\N	2020-10-12 16:37:24.477173
80331	STBA	2020-09-28	17.58	18.3	17.495	18.01	161733	{}	\N	2020-10-12 16:37:25.606588
80332	STBA	2020-09-29	17.86	17.99	17.475	17.79	107010	{}	\N	2020-10-12 16:37:25.620129
80333	STBA	2020-09-30	17.95	18.18	17.55	17.69	224651	{}	\N	2020-10-12 16:37:25.628093
80334	STBA	2020-10-01	17.64	17.81	17.32	17.81	171305	{}	\N	2020-10-12 16:37:25.635807
80335	STBA	2020-10-02	17.41	18.47	17.315	18.36	207559	{}	\N	2020-10-12 16:37:25.641516
80336	STBA	2020-10-05	18.68	19.31	18.4	19.19	219025	{}	\N	2020-10-12 16:37:25.646917
80337	STBA	2020-10-06	19.48	20.04	19.17	19.24	225928	{}	\N	2020-10-12 16:37:25.652326
80338	STBA	2020-10-07	19.34	19.74	19.22	19.58	184355	{}	\N	2020-10-12 16:37:25.657512
80339	STBA	2020-10-08	19.85	20	19.57	19.94	148517	{}	\N	2020-10-12 16:37:25.662538
80340	STBA	2020-10-09	20.19	20.19	19.72	19.8	124221	{}	\N	2020-10-12 16:37:25.667575
80341	STBA	2020-10-12	19.76	20.19	19.63	19.97	24900	{}	\N	2020-10-12 16:37:25.672732
80342	STKL	2020-09-28	7.48	7.6025	7.4	7.56	731084	{}	\N	2020-10-12 16:37:28.263635
80343	STKL	2020-09-29	7.52	7.54	7.3	7.51	381749	{}	\N	2020-10-12 16:37:28.275593
80344	STKL	2020-09-30	7.51	7.91	7.38	7.42	816596	{}	\N	2020-10-12 16:37:28.285221
80345	STKL	2020-10-01	7.53	7.78	7.4924	7.78	472531	{}	\N	2020-10-12 16:37:28.291588
80346	STKL	2020-10-02	7.55	7.76	7.5	7.6	297604	{}	\N	2020-10-12 16:37:28.296786
80347	STKL	2020-10-05	7.64	7.68	7.5	7.68	279030	{}	\N	2020-10-12 16:37:28.300923
80348	STKL	2020-10-06	7.66	7.82	7.5735	7.65	277666	{}	\N	2020-10-12 16:37:28.305412
80349	STKL	2020-10-07	7.72	8.04	7.68	7.81	473787	{}	\N	2020-10-12 16:37:28.309391
80350	STKL	2020-10-08	7.89	7.95	7.76	7.8	285382	{}	\N	2020-10-12 16:37:28.313731
80351	STKL	2020-10-09	7.9	7.94	7.69	7.93	255940	{}	\N	2020-10-12 16:37:28.318148
80352	STKL	2020-10-12	8	8.05	7.82	7.83	201247	{}	\N	2020-10-12 16:37:28.322474
80353	STN	2020-09-28	30.1	30.65	30.1	30.54	97789	{}	\N	2020-10-12 16:37:30.544824
80354	STN	2020-09-29	30.44	31.06	30.42	30.94	191766	{}	\N	2020-10-12 16:37:30.555531
80355	STN	2020-09-30	30.84	30.98	30.31	30.34	163179	{}	\N	2020-10-12 16:37:30.562372
80356	STN	2020-10-01	30.62	30.62	30.23	30.42	187724	{}	\N	2020-10-12 16:37:30.568335
80357	STN	2020-10-02	30.09	30.13	29.45	29.84	220201	{}	\N	2020-10-12 16:37:30.573961
80358	STN	2020-10-05	30.03	30.33	30.01	30.2	190189	{}	\N	2020-10-12 16:37:30.579424
80359	STN	2020-10-06	30.17	30.42	29.775	29.99	125757	{}	\N	2020-10-12 16:37:30.584728
80360	STN	2020-10-07	30.07	30.3	29.85	30.22	93278	{}	\N	2020-10-12 16:37:30.589513
80361	STN	2020-10-08	30.57	30.57	30.17	30.34	92411	{}	\N	2020-10-12 16:37:30.594288
80362	STN	2020-10-09	30.39	30.78	30.39	30.65	151714	{}	\N	2020-10-12 16:37:30.598885
80363	STN	2020-10-12	30.86	31	30.73	30.74	18422	{}	\N	2020-10-12 16:37:30.603557
80364	STZ	2020-09-28	184.69	186.05	183.88	185.45	1590965	{}	\N	2020-10-12 16:37:32.264156
80365	STZ	2020-09-29	186.76	190.16	186.03	187.52	1968305	{}	\N	2020-10-12 16:37:32.275491
80366	STZ	2020-09-30	187.97	190.57	187.56	189.51	2520675	{}	\N	2020-10-12 16:37:32.281142
80367	STZ	2020-10-01	191	191	183.41	184.61	3748681	{}	\N	2020-10-12 16:37:32.287311
80368	STZ	2020-10-02	184.36	189.2962	183.2989	185.6	2583136	{}	\N	2020-10-12 16:37:32.292947
80369	STZ	2020-10-05	185.4	186.045	182.82	183.6	1947639	{}	\N	2020-10-12 16:37:32.298318
80370	STZ	2020-10-06	183.5	184.87	180.31	180.58	1643990	{}	\N	2020-10-12 16:37:32.303837
80371	STZ	2020-10-07	182.02	183.98	181.12	181.48	1674130	{}	\N	2020-10-12 16:37:32.309689
80372	STZ	2020-10-08	181.58	185.23	180.7	185.15	2147537	{}	\N	2020-10-12 16:37:32.315444
80373	STZ	2020-10-09	186.24	187.44	184.835	186.65	1675059	{}	\N	2020-10-12 16:37:32.3212
80374	STZ	2020-10-12	187.26	189.56	187.17	188.21	494657	{}	\N	2020-10-12 16:37:32.326323
80375	SUSA	2020-09-28	145.87	146.6281	145.731	146.24	33051	{}	\N	2020-10-12 16:37:33.342981
80376	SUSA	2020-09-29	146.28	146.62	145.615	145.98	38016	{}	\N	2020-10-12 16:37:33.354736
80377	SUSA	2020-09-30	146.04	148.35	146.04	147.09	93283	{}	\N	2020-10-12 16:37:33.362883
80378	SUSA	2020-10-01	148.2	148.385	147.0599	147.84	44365	{}	\N	2020-10-12 16:37:33.369293
80379	SUSA	2020-10-02	145.6	147.31	145.6	146.6384	31262	{}	\N	2020-10-12 16:37:33.375081
80380	SUSA	2020-10-05	147.58	149.29	147.49	149.14	36659	{}	\N	2020-10-12 16:37:33.380536
80381	SUSA	2020-10-06	149.62	150.5368	147.38	147.47	116320	{}	\N	2020-10-12 16:37:33.385743
80382	SUSA	2020-10-07	148.95	150.39	148.95	150.01	186892	{}	\N	2020-10-12 16:37:33.391194
80383	SUSA	2020-10-08	150.89	151.42	150.8181	151.38	68221	{}	\N	2020-10-12 16:37:33.396659
80384	SUSA	2020-10-09	152.39	153.03	151.97	152.85	158420	{}	\N	2020-10-12 16:37:33.401918
80385	SUSA	2020-10-12	154.05	155.07	153.73	155.0408	15095	{}	\N	2020-10-12 16:37:33.407251
80386	XHR	2020-09-28	8.72	9.13	8.72	9.01	715456	{}	\N	2020-10-12 16:37:34.355101
80387	XHR	2020-09-29	9.2	9.22	8.76	8.84	1825521	{}	\N	2020-10-12 16:37:34.367587
80388	XHR	2020-09-30	8.88	9.2	8.705	8.78	1894004	{}	\N	2020-10-12 16:37:34.37567
80389	XHR	2020-10-01	8.77	9.09	8.73	9.07	858010	{}	\N	2020-10-12 16:37:34.382825
80390	XHR	2020-10-02	8.8	9.13	8.6401	9.07	947153	{}	\N	2020-10-12 16:37:34.388882
80391	XHR	2020-10-05	9.15	9.25	8.85	9.02	569155	{}	\N	2020-10-12 16:37:34.394298
80392	XHR	2020-10-06	9.15	9.64	9	9.25	1797358	{}	\N	2020-10-12 16:37:34.399851
80393	XHR	2020-10-07	9.37	9.37	8.98	9.28	1894111	{}	\N	2020-10-12 16:37:34.405293
80394	XHR	2020-10-08	9.41	9.73	9.2901	9.72	812473	{}	\N	2020-10-12 16:37:34.410959
80395	XHR	2020-10-09	9.81	9.83	9.43	9.56	969057	{}	\N	2020-10-12 16:37:34.41758
80396	XHR	2020-10-12	9.48	9.575	9.28	9.51	227698	{}	\N	2020-10-12 16:37:34.424104
80397	AADR	2020-09-28	54.9878	55.11	54.8011	54.81	7288	{}	\N	2020-10-12 16:37:35.552415
80398	AADR	2020-09-29	55.08	55.4599	55.0604	55.24	4088	{}	\N	2020-10-12 16:37:35.566932
80399	AADR	2020-09-30	55.46	55.6838	55.15	55.25	6185	{}	\N	2020-10-12 16:37:35.575036
80400	AADR	2020-10-01	56.13	56.34	55.885	56.3199	6111	{}	\N	2020-10-12 16:37:35.58168
80401	AADR	2020-10-02	55.4601	55.752	55.4213	55.61	4026	{}	\N	2020-10-12 16:37:35.587764
80402	AADR	2020-10-05	56.2	56.325	56.1601	56.325	1128	{}	\N	2020-10-12 16:37:35.592843
80403	AADR	2020-10-06	56.0954	56.0954	55.56	55.9799	983	{}	\N	2020-10-12 16:37:35.598066
80404	AADR	2020-10-07	56.1249	56.355	56.1243	56.355	2718	{}	\N	2020-10-12 16:37:35.603221
80405	AADR	2020-10-08	56.5696	56.9499	56.5696	56.9499	1936	{}	\N	2020-10-12 16:37:35.608673
80406	AADR	2020-10-09	57.5043	58	57.5043	57.99	4021	{}	\N	2020-10-12 16:37:35.613795
80407	AADR	2020-10-12	58.37	58.7499	58.335	58.7499	1320	{}	\N	2020-10-12 16:37:35.619122
80408	AAXN	2020-09-28	88.09	89.52	87.55	87.91	495280	{}	\N	2020-10-12 16:37:36.556708
80409	AAXN	2020-09-29	87.92	89.15	87.1	88.62	381758	{}	\N	2020-10-12 16:37:36.568877
80410	AAXN	2020-09-30	89.36	91.8325	89.17	90.7	867658	{}	\N	2020-10-12 16:37:36.575566
80411	AAXN	2020-10-01	91.84	96.09	91.305	95.64	814015	{}	\N	2020-10-12 16:37:36.581436
80412	AAXN	2020-10-02	93.5	94.49	90.87	91.55	462677	{}	\N	2020-10-12 16:37:36.587032
80413	AAXN	2020-10-05	92.22	93.45	90.1706	91.84	429795	{}	\N	2020-10-12 16:37:36.592307
80414	AAXN	2020-10-06	92.2	94.47	91.46	93.22	434807	{}	\N	2020-10-12 16:37:36.596908
80415	AAXN	2020-10-07	94.54	96.355	93.91	96.07	388844	{}	\N	2020-10-12 16:37:36.601172
80416	AAXN	2020-10-08	96.85	97.38	95.54	97.07	375683	{}	\N	2020-10-12 16:37:36.605715
80417	AAXN	2020-10-09	97.57	99.66	97.56	98.99	429262	{}	\N	2020-10-12 16:37:36.610569
80418	AAXN	2020-10-12	99.72	99.9	98.13	99.07	121878	{}	\N	2020-10-12 16:37:36.615127
80419	ADXS	2020-09-28	0.38	0.4041	0.371501	0.392	1383124	{}	\N	2020-10-12 16:37:37.73277
80420	ADXS	2020-09-29	0.4	0.4507	0.3977	0.421	6250022	{}	\N	2020-10-12 16:37:37.741555
80421	ADXS	2020-09-30	0.434	0.45	0.4023	0.4122	1391508	{}	\N	2020-10-12 16:37:37.747728
80422	ADXS	2020-10-01	0.415	0.415	0.395	0.3955	891828	{}	\N	2020-10-12 16:37:37.753924
80423	ADXS	2020-10-02	0.3874	0.4128	0.3874	0.4094	1042872	{}	\N	2020-10-12 16:37:37.759654
80424	ADXS	2020-10-05	0.4215	0.4215	0.395	0.3997	958375	{}	\N	2020-10-12 16:37:37.764618
80425	ADXS	2020-10-06	0.4	0.41	0.391	0.394	687964	{}	\N	2020-10-12 16:37:37.769379
80426	ADXS	2020-10-07	0.4001	0.4066	0.396	0.398	445929	{}	\N	2020-10-12 16:37:37.773763
80427	ADXS	2020-10-08	0.398	0.41	0.3925	0.3952	891674	{}	\N	2020-10-12 16:37:37.77806
80428	ADXS	2020-10-09	0.3948	0.419	0.393	0.4101	810547	{}	\N	2020-10-12 16:37:37.782825
80429	ADXS	2020-10-12	0.4288	0.4288	0.4025	0.406	641444	{}	\N	2020-10-12 16:37:37.78739
80430	AEO	2020-09-28	14.57	15.015	14.46	14.84	4670561	{}	\N	2020-10-12 16:37:39.45051
80431	AEO	2020-09-29	14.82	14.82	13.9	13.99	9708112	{}	\N	2020-10-12 16:37:39.46239
80432	AEO	2020-09-30	14.14	14.87	14.13	14.81	20065013	{}	\N	2020-10-12 16:37:39.470051
80433	AEO	2020-10-01	14.9293	15.22	14.66	14.92	13242150	{}	\N	2020-10-12 16:37:39.476405
80434	AEO	2020-10-02	14.7	15.67	14.42	15.38	11203907	{}	\N	2020-10-12 16:37:39.482353
80435	AEO	2020-10-05	15.81	16.25	15.26	15.46	10787159	{}	\N	2020-10-12 16:37:39.487047
80436	AEO	2020-10-06	15.57	15.68	14.84	14.86	6599085	{}	\N	2020-10-12 16:37:39.492176
80437	AEO	2020-10-07	15.48	16.04	15.39	15.76	16769141	{}	\N	2020-10-12 16:37:39.496809
80438	AEO	2020-10-08	15.91	16	15.34	15.46	6609643	{}	\N	2020-10-12 16:37:39.501315
80439	AEO	2020-10-09	15.52	15.7	15.225	15.31	6048177	{}	\N	2020-10-12 16:37:39.506019
80440	AEO	2020-10-12	15.36	15.64	15.12	15.135	2633472	{}	\N	2020-10-12 16:37:39.510743
80441	ALTS	2020-09-28	34.8381	34.8381	34.8381	34.8381	132	{}	\N	2020-10-12 16:37:41.497772
80442	ALTS	2020-09-29	34.7601	34.8332	34.76	34.8332	361	{}	\N	2020-10-12 16:37:41.509644
80443	ALTS	2020-09-30	34.9005	34.9005	34.9005	34.9005	161	{}	\N	2020-10-12 16:37:41.5171
80444	ALTS	2020-10-01	34.8853	34.8853	34.8853	34.8853	300	{}	\N	2020-10-12 16:37:41.523252
80445	ALTS	2020-10-02	34.79	34.8623	34.79	34.8623	303	{}	\N	2020-10-12 16:37:41.528991
80446	ALTS	2020-10-05	35.0731	35.0731	35.0731	35.0731	144	{}	\N	2020-10-12 16:37:41.534042
80447	ALTS	2020-10-06	35.1047	35.1047	35.1047	35.1047	171	{}	\N	2020-10-12 16:37:41.538457
80448	ALTS	2020-10-07	34.94	35.1949	34.94	35.1949	748	{}	\N	2020-10-12 16:37:41.542671
80449	ALTS	2020-10-08	35.2868	35.2868	35.2868	35.2868	310	{}	\N	2020-10-12 16:37:41.547448
80450	ALTS	2020-10-09	35.3058	35.3058	35.3058	35.3058	157	{}	\N	2020-10-12 16:37:41.552199
80451	ALTS	2020-10-12	35.33	35.33	35.33	35.33	132	{}	\N	2020-10-12 16:37:41.556767
80452	AMOT	2020-09-28	40	41.66	40	41.08	22557	{}	\N	2020-10-12 16:37:43.748671
80453	AMOT	2020-09-29	41.07	41.32	40.3	40.9	11068	{}	\N	2020-10-12 16:37:43.761546
80454	AMOT	2020-09-30	41.3	42.1	40.55	41.28	63441	{}	\N	2020-10-12 16:37:43.767513
80455	AMOT	2020-10-01	42.075	42.5	41.35	42.3	24161	{}	\N	2020-10-12 16:37:43.772434
80456	AMOT	2020-10-02	41.495	43.23	41.495	41.66	19148	{}	\N	2020-10-12 16:37:43.776353
80457	AMOT	2020-10-05	41.87	44.52	41.66	43.65	17577	{}	\N	2020-10-12 16:37:43.780128
80458	AMOT	2020-10-06	44.25	44.97	43.57	43.92	13520	{}	\N	2020-10-12 16:37:43.783975
80459	AMOT	2020-10-07	43.87	44.55	42.9023	44.14	18338	{}	\N	2020-10-12 16:37:43.788087
80460	AMOT	2020-10-08	41.6194	44.27	40.41	44	14456	{}	\N	2020-10-12 16:37:43.792089
80461	AMOT	2020-10-09	44.41	44.41	43.26	43.88	5778	{}	\N	2020-10-12 16:37:43.795889
80462	AMOT	2020-10-12	43.505	44.08	43.0937	43.79	7579	{}	\N	2020-10-12 16:37:43.800117
80463	YORW	2020-09-28	42.13	43.71	42.13	43.39	37817	{}	\N	2020-10-12 16:37:44.7097
80464	YORW	2020-09-29	43.24	43.305	42.41	42.95	26867	{}	\N	2020-10-12 16:37:44.719525
80465	YORW	2020-09-30	43.04	43.04	42.03	42.27	26370	{}	\N	2020-10-12 16:37:44.726649
80466	YORW	2020-10-01	42.5	42.89	42.03	42.6	32499	{}	\N	2020-10-12 16:37:44.732424
80467	YORW	2020-10-02	42.12	43.31	42.0814	42.68	19173	{}	\N	2020-10-12 16:37:44.742984
80468	YORW	2020-10-05	42.8	43.1	42.49	42.94	22175	{}	\N	2020-10-12 16:37:44.749634
80469	YORW	2020-10-06	42.93	43.885	42.27	42.54	19779	{}	\N	2020-10-12 16:37:44.75548
80470	YORW	2020-10-07	43.29	43.47	42.66	43.21	30240	{}	\N	2020-10-12 16:37:44.760169
80471	YORW	2020-10-08	43.56	44.01	43.245	43.66	20113	{}	\N	2020-10-12 16:37:44.764738
80472	YORW	2020-10-09	43.635	44.05	43.25	43.58	15202	{}	\N	2020-10-12 16:37:44.769541
80473	YORW	2020-10-12	43.77	44.76	43.76	44.38	8853	{}	\N	2020-10-12 16:37:44.774699
80474	YUMC	2020-09-28	51.87	52.05	51.52	52	1419728	{}	\N	2020-10-12 16:37:49.038118
80475	YUMC	2020-09-29	52.05	52.05	51.465	51.56	4723219	{}	\N	2020-10-12 16:37:49.048722
80476	YUMC	2020-09-30	51.94	53.13	51.84	52.95	4249646	{}	\N	2020-10-12 16:37:49.056665
80477	YUMC	2020-10-01	53.5	53.6	52.905	53	2087680	{}	\N	2020-10-12 16:37:49.062811
80478	YUMC	2020-10-02	52.22	53.43	52.1938	52.98	1413288	{}	\N	2020-10-12 16:37:49.067936
80479	YUMC	2020-10-05	53.17	53.72	52.79	53.55	3004966	{}	\N	2020-10-12 16:37:49.072768
80480	YUMC	2020-10-06	53.47	54.23	53.14	53.31	4832454	{}	\N	2020-10-12 16:37:49.077338
80481	YUMC	2020-10-07	53.8	53.915	52.65	53.37	2946516	{}	\N	2020-10-12 16:37:49.081552
80482	YUMC	2020-10-08	53.38	53.9	53.01	53.72	2482170	{}	\N	2020-10-12 16:37:49.085828
80483	YUMC	2020-10-09	53.96	54.7	53.5	53.99	2365797	{}	\N	2020-10-12 16:37:49.089851
80484	YUMC	2020-10-12	54.28	54.28	53.5	53.95	1002959	{}	\N	2020-10-12 16:37:49.09409
80485	ZAGG	2020-09-28	3.1	3.13	2.88	2.95	633801	{}	\N	2020-10-12 16:37:49.960756
80486	ZAGG	2020-09-29	2.95	3.26	2.8931	2.98	1891242	{}	\N	2020-10-12 16:37:49.975831
80487	ZAGG	2020-09-30	2.91	2.9426	2.8	2.8	835667	{}	\N	2020-10-12 16:37:49.984055
80488	ZAGG	2020-10-01	2.81	2.9	2.72	2.85	905305	{}	\N	2020-10-12 16:37:49.990831
80489	ZAGG	2020-10-02	2.8	3.13	2.76	3.06	1484909	{}	\N	2020-10-12 16:37:49.99679
80490	ZAGG	2020-10-05	3.1	3.12	2.84	2.99	809673	{}	\N	2020-10-12 16:37:50.001868
80491	ZAGG	2020-10-06	3.01	3.0581	2.95	2.96	363010	{}	\N	2020-10-12 16:37:50.007393
80492	ZAGG	2020-10-07	2.97	3.04	2.935	3.04	276379	{}	\N	2020-10-12 16:37:50.013315
80493	ZAGG	2020-10-08	3.1	3.14	3.06	3.11	411654	{}	\N	2020-10-12 16:37:50.018178
80494	ZAGG	2020-10-09	3.13	3.3	3.09	3.27	1487597	{}	\N	2020-10-12 16:37:50.022881
80495	ZAGG	2020-10-12	3.32	3.32	3.07	3.115	536633	{}	\N	2020-10-12 16:37:50.027619
80496	ZEN	2020-09-28	100.19	103.01	100	101.91	1578127	{}	\N	2020-10-12 16:37:51.242427
80497	ZEN	2020-09-29	102	102.98	100.71	101.8	1948579	{}	\N	2020-10-12 16:37:51.253996
80498	ZEN	2020-09-30	101.79	104.07	101.7	102.92	3384412	{}	\N	2020-10-12 16:37:51.261749
80499	ZEN	2020-10-01	104	105.62	102.23	104.36	3599154	{}	\N	2020-10-12 16:37:51.267812
80500	ZEN	2020-10-02	101.74	105.22	101.17	103.44	3365560	{}	\N	2020-10-12 16:37:51.273521
80501	ZEN	2020-10-05	104	105.91	103.5	105.35	1978553	{}	\N	2020-10-12 16:37:51.278643
80502	ZEN	2020-10-06	105.35	107.9899	104.61	105.84	2391868	{}	\N	2020-10-12 16:37:51.283773
80503	ZEN	2020-10-07	107.16	109.72	107.16	108.85	2220526	{}	\N	2020-10-12 16:37:51.289023
80504	ZEN	2020-10-08	109.58	109.8	107.11	107.83	2556977	{}	\N	2020-10-12 16:37:51.294059
80505	ZEN	2020-10-09	108.5	111.48	108.28	110.86	1862564	{}	\N	2020-10-12 16:37:51.299432
80506	ZEN	2020-10-12	112.92	113.63	110.72	112.055	702238	{}	\N	2020-10-12 16:37:51.304676
80507	ZLAB	2020-09-28	81.58	83.93	79.645	82.72	2573135	{}	\N	2020-10-12 16:37:54.717554
80508	ZLAB	2020-09-29	83.31	83.63	81.2	82.59	489461	{}	\N	2020-10-12 16:37:54.728749
80509	ZLAB	2020-09-30	82.35	83.29	81.7	83.17	264746	{}	\N	2020-10-12 16:37:54.735727
80510	ZLAB	2020-10-01	84.26	87.38	83.4749	87.38	376719	{}	\N	2020-10-12 16:37:54.741819
80511	ZLAB	2020-10-02	86.29	87.9994	83.81	84.54	194436	{}	\N	2020-10-12 16:37:54.747326
80512	ZLAB	2020-10-05	84.55	86.2676	83.07	84.61	416183	{}	\N	2020-10-12 16:37:54.7523
80513	ZLAB	2020-10-06	84.9	88.68	84.61	85.08	207824	{}	\N	2020-10-12 16:37:54.757844
80514	ZLAB	2020-10-07	86.24	90.7	85.99	90.53	315920	{}	\N	2020-10-12 16:37:54.762744
80515	ZLAB	2020-10-08	91.34	93.32	90.09	91.8	270732	{}	\N	2020-10-12 16:37:54.767751
80516	ZLAB	2020-10-09	92	93.9999	91.03	93	260132	{}	\N	2020-10-12 16:37:54.772925
80517	ZLAB	2020-10-12	94.11	94.6	92.03	92.3	89952	{}	\N	2020-10-12 16:37:54.77794
80518	ZUMZ	2020-09-28	27.71	27.81	27.05	27.67	284655	{}	\N	2020-10-12 16:38:44.151597
80519	ZUMZ	2020-09-29	27.63	27.77	26.91	27.65	317884	{}	\N	2020-10-12 16:38:44.161194
80520	ZUMZ	2020-09-30	27.87	28.41	27.6	27.82	333864	{}	\N	2020-10-12 16:38:44.167429
80521	ZUMZ	2020-10-01	27.97	28.49	27.31	27.77	246012	{}	\N	2020-10-12 16:38:44.173646
80522	ZUMZ	2020-10-02	27.02	28.37	27	28.17	314333	{}	\N	2020-10-12 16:38:44.180007
80523	ZUMZ	2020-10-05	28.39	29.28	27.81	28.8	341893	{}	\N	2020-10-12 16:38:44.185638
80524	ZUMZ	2020-10-06	28.88	29.83	28.44	28.51	321156	{}	\N	2020-10-12 16:38:44.19081
80525	ZUMZ	2020-10-07	29.07	30.15	29.02	30.08	330870	{}	\N	2020-10-12 16:38:44.196233
80526	ZUMZ	2020-10-08	30.5	31.32	30.2	31.02	227146	{}	\N	2020-10-12 16:38:44.20156
80527	ZUMZ	2020-10-09	31.1	31.85	31.08	31.29	282901	{}	\N	2020-10-12 16:38:44.206983
80528	ZUMZ	2020-10-12	31.01	31.72	31.01	31.44	83725	{}	\N	2020-10-12 16:38:44.2128
80529	AAOI	2020-09-28	10.95	11.18	10.66	10.95	454250	{}	\N	2020-10-12 16:38:47.972213
80530	AAOI	2020-09-29	10.94	11.36	10.8559	11.24	380011	{}	\N	2020-10-12 16:38:47.98602
80531	AAOI	2020-09-30	11.23	11.6	11.1	11.25	440100	{}	\N	2020-10-12 16:38:47.994188
80532	AAOI	2020-10-01	11.4	11.4066	11.0374	11.22	250747	{}	\N	2020-10-12 16:38:48.000469
80533	AAOI	2020-10-02	10.8	11.36	10.55	11.33	393982	{}	\N	2020-10-12 16:38:48.006213
80534	AAOI	2020-10-05	11.49	11.62	11.17	11.49	352903	{}	\N	2020-10-12 16:38:48.011536
80535	AAOI	2020-10-06	11.49	11.85	11.01	11.2	443164	{}	\N	2020-10-12 16:38:48.01628
80536	AAOI	2020-10-07	11.28	11.61	11.26	11.48	368585	{}	\N	2020-10-12 16:38:48.021396
80537	AAOI	2020-10-08	11.51	11.85	11.3	11.59	365568	{}	\N	2020-10-12 16:38:48.02744
80538	AAOI	2020-10-09	11.79	11.94	11.46	11.83	338195	{}	\N	2020-10-12 16:38:48.032349
80539	AAOI	2020-10-12	11.79	11.84	11.37	11.45	189013	{}	\N	2020-10-12 16:38:48.037523
80540	AAT	2020-09-28	23.71	24.63	23.71	24.48	199867	{}	\N	2020-10-12 16:38:49.031617
80541	AAT	2020-09-29	24.43	24.43	23.53	23.98	438589	{}	\N	2020-10-12 16:38:49.042487
80542	AAT	2020-09-30	24.27	24.54	23.65	24.09	620003	{}	\N	2020-10-12 16:38:49.049404
80543	AAT	2020-10-01	24.06	24.81	24.06	24.8	333193	{}	\N	2020-10-12 16:38:49.055314
80544	AAT	2020-10-02	24.36	25.51	24.3	25.45	293250	{}	\N	2020-10-12 16:38:49.060464
80545	AAT	2020-10-05	25.65	25.95	24.99	25.63	450277	{}	\N	2020-10-12 16:38:49.065469
80546	AAT	2020-10-06	25.94	26.26	25.37	25.41	819549	{}	\N	2020-10-12 16:38:49.069998
80547	AAT	2020-10-07	25.59	25.62	24.64	24.71	521654	{}	\N	2020-10-12 16:38:49.075207
80548	AAT	2020-10-08	24.92	25.8	24.87	25.78	379785	{}	\N	2020-10-12 16:38:49.080531
80549	AAT	2020-10-09	25.86	26.41	25.5	26.17	563604	{}	\N	2020-10-12 16:38:49.084857
80550	AAT	2020-10-12	25.95	26.41	25.72	26.06	93547	{}	\N	2020-10-12 16:38:49.089547
80551	ACWF	2020-09-28	29.54	29.64	29.54	29.62	4088	{}	\N	2020-10-12 16:38:52.118518
80552	ACWF	2020-09-29	29.59	29.64	29.5645	29.61	7385	{}	\N	2020-10-12 16:38:52.128065
80553	ACWF	2020-09-30	29.59	29.79	29.58	29.68	66156	{}	\N	2020-10-12 16:38:52.135468
80554	ACWF	2020-10-01	29.82	29.8682	29.73	29.8645	4989	{}	\N	2020-10-12 16:38:52.141643
80555	ACWF	2020-10-02	29.42	29.8155	29.42	29.7718	5928	{}	\N	2020-10-12 16:38:52.147125
80556	ACWF	2020-10-05	29.97	30.32	29.97	30.32	11325	{}	\N	2020-10-12 16:38:52.15198
80557	ACWF	2020-10-06	30.32	30.379	29.9824	29.9824	1514	{}	\N	2020-10-12 16:38:52.157424
80558	ACWF	2020-10-07	30.3115	30.45	30.3	30.45	4469	{}	\N	2020-10-12 16:38:52.163187
80559	ACWF	2020-10-08	30.63	30.71	30.55	30.65	7563	{}	\N	2020-10-12 16:38:52.168726
80560	ACWF	2020-10-09	30.8055	30.965	30.8055	30.9	3781	{}	\N	2020-10-12 16:38:52.174341
80561	ACWF	2020-10-12	31.09	31.1203	31.09	31.1101	2678	{}	\N	2020-10-12 16:38:52.179685
80562	AMBA	2020-09-28	50.63	52.25	50.22	51.97	385588	{}	\N	2020-10-12 16:38:54.6854
80563	AMBA	2020-09-29	52	53.25	51.29	52.02	473077	{}	\N	2020-10-12 16:38:54.695155
80564	AMBA	2020-09-30	51.77	52.83	51.15	52.18	427704	{}	\N	2020-10-12 16:38:54.70444
80565	AMBA	2020-10-01	52.69	54.1452	52.5	53.63	404160	{}	\N	2020-10-12 16:38:54.714157
80566	AMBA	2020-10-02	51.86	53.15	51.72	51.92	441298	{}	\N	2020-10-12 16:38:54.720183
80567	AMBA	2020-10-05	52.52	53.55	52.52	53.4	272590	{}	\N	2020-10-12 16:38:54.727105
80568	AMBA	2020-10-06	53.5	54.8	53.0669	53.29	281658	{}	\N	2020-10-12 16:38:54.733143
80569	AMBA	2020-10-07	53.9	54.49	53.34	53.72	249192	{}	\N	2020-10-12 16:38:54.73834
80570	AMBA	2020-10-08	55.59	57.32	54.69	56.53	700856	{}	\N	2020-10-12 16:38:54.743674
80571	AMBA	2020-10-09	56.57	58.79	56.57	58.15	638231	{}	\N	2020-10-12 16:38:54.749009
80572	AMBA	2020-10-12	58.86	58.86	57.06	57.43	217177	{}	\N	2020-10-12 16:38:54.753829
80573	YTEN	2020-09-28	7.39	8.16	7.18	8.13	259136	{}	\N	2020-10-12 16:38:56.090449
80574	YTEN	2020-09-29	8.02	8.5	7.5601	8.25	205626	{}	\N	2020-10-12 16:38:56.101786
80575	YTEN	2020-09-30	8.21	8.42	7.73	7.77	144438	{}	\N	2020-10-12 16:38:56.109405
80576	YTEN	2020-10-01	7.8	8.35	7.67	7.97	86767	{}	\N	2020-10-12 16:38:56.115475
80577	YTEN	2020-10-02	7.77	7.88	7.26	7.41	181883	{}	\N	2020-10-12 16:38:56.121302
80578	YTEN	2020-10-05	7.46	7.5766	7.44	7.5	62586	{}	\N	2020-10-12 16:38:56.126036
80579	YTEN	2020-10-06	7.5	7.85	7.45	7.5	183725	{}	\N	2020-10-12 16:38:56.131136
80580	YTEN	2020-10-07	7.5	7.825	7.5	7.6	235447	{}	\N	2020-10-12 16:38:56.136159
80581	YTEN	2020-10-08	7.6	7.6	7.39	7.39	76693	{}	\N	2020-10-12 16:38:56.141015
80582	YTEN	2020-10-09	7.33	7.3607	6.85	7.08	136745	{}	\N	2020-10-12 16:38:56.145939
80583	YTEN	2020-10-12	7.06	7.239	7	7.0708	13981	{}	\N	2020-10-12 16:38:56.150956
80584	YXI	2020-09-28	16.96	16.98	16.9275	16.9275	545	{}	\N	2020-10-12 16:38:59.552513
80585	YXI	2020-09-29	17.0356	17.0356	16.965	16.994	2212	{}	\N	2020-10-12 16:38:59.565985
80586	YXI	2020-09-30	16.79	16.79	16.6348	16.6348	3305	{}	\N	2020-10-12 16:38:59.573209
80587	YXI	2020-10-01	16.48	16.48	16.45	16.47	819	{}	\N	2020-10-12 16:38:59.579331
80588	YXI	2020-10-02	16.46	16.6241	16.46	16.6241	4487	{}	\N	2020-10-12 16:38:59.585519
80589	YXI	2020-10-05	16.65	16.65	16.62	16.63	3396	{}	\N	2020-10-12 16:38:59.590734
80590	YXI	2020-10-06	16.45	16.46	16.37	16.45	1460	{}	\N	2020-10-12 16:38:59.596206
80591	YXI	2020-10-07	16.29	16.32	16.26	16.26	1604	{}	\N	2020-10-12 16:38:59.602243
80592	YXI	2020-10-08	16.23	16.23	16.1884	16.1884	340	{}	\N	2020-10-12 16:38:59.607492
80593	YXI	2020-10-09	16.18	16.18	16.12	16.15	1283	{}	\N	2020-10-12 16:38:59.612642
80594	YXI	2020-10-12	15.75	15.8	15.71	15.71	2310	{}	\N	2020-10-12 16:38:59.61885
80595	ZGNX	2020-09-28	17.77	18.06	17.31	18	2832028	{}	\N	2020-10-12 16:39:03.254675
80596	ZGNX	2020-09-29	17.98	18.44	17.48	17.95	1609961	{}	\N	2020-10-12 16:39:03.269988
80597	ZGNX	2020-09-30	18.02	18.4	17.81	17.93	1392929	{}	\N	2020-10-12 16:39:03.278345
80598	ZGNX	2020-10-01	17.92	18.355	17.6	17.92	1068998	{}	\N	2020-10-12 16:39:03.284927
80599	ZGNX	2020-10-02	17.54	17.98	17.42	17.58	858380	{}	\N	2020-10-12 16:39:03.290876
80600	ZGNX	2020-10-05	17.79	18.37	17.77	18.055	1320334	{}	\N	2020-10-12 16:39:03.296214
80601	ZGNX	2020-10-06	18.22	18.39	17.79	17.93	1510684	{}	\N	2020-10-12 16:39:03.300901
80602	ZGNX	2020-10-07	18	18.42	17.62	18.23	1258939	{}	\N	2020-10-12 16:39:03.305687
80603	ZGNX	2020-10-08	18.54	18.65	18.16	18.45	808220	{}	\N	2020-10-12 16:39:03.310707
80604	ZGNX	2020-10-09	18.42	18.595	18.15	18.31	1029167	{}	\N	2020-10-12 16:39:03.317075
80605	ZGNX	2020-10-12	18.46	18.53	17.97	18.41	301670	{}	\N	2020-10-12 16:39:03.322866
80606	ZIXI	2020-09-28	5.78	6.02	5.67	5.93	589873	{}	\N	2020-10-12 16:39:07.247994
80607	ZIXI	2020-09-29	5.92	5.92	5.77	5.85	606570	{}	\N	2020-10-12 16:39:07.259528
80608	ZIXI	2020-09-30	5.88	6.0329	5.79	5.84	285799	{}	\N	2020-10-12 16:39:07.267631
80609	ZIXI	2020-10-01	5.9	5.99	5.88	5.96	247412	{}	\N	2020-10-12 16:39:07.274432
80610	ZIXI	2020-10-02	5.78	5.92	5.76	5.83	428726	{}	\N	2020-10-12 16:39:07.280452
80611	ZIXI	2020-10-05	5.87	6.07	5.87	6.05	286183	{}	\N	2020-10-12 16:39:07.285207
80612	ZIXI	2020-10-06	6.08	6.295	6.05	6.09	438119	{}	\N	2020-10-12 16:39:07.290103
80613	ZIXI	2020-10-07	6.1	6.31	6.02	6.29	523171	{}	\N	2020-10-12 16:39:07.295037
80614	ZIXI	2020-10-08	6.37	6.4848	6.27	6.34	395697	{}	\N	2020-10-12 16:39:07.300103
80615	ZIXI	2020-10-09	6.36	6.685	6.35	6.54	524247	{}	\N	2020-10-12 16:39:07.305446
80616	ZIXI	2020-10-12	6.56	6.7671	6.53	6.56	215310	{}	\N	2020-10-12 16:39:07.310195
80617	ZTO	2020-09-28	31	32.21	30.92	31.97	5594832	{}	\N	2020-10-12 16:39:09.25491
80618	ZTO	2020-09-29	30.97	31.16	29.55	29.59	18395838	{}	\N	2020-10-12 16:39:09.265481
80619	ZTO	2020-09-30	29.27	30.18	29.2	29.92	6223826	{}	\N	2020-10-12 16:39:09.272893
80620	ZTO	2020-10-01	30.01	30.25	29.215	29.48	2789154	{}	\N	2020-10-12 16:39:09.27923
80621	ZTO	2020-10-02	29	29.56	28.79	29.08	2534419	{}	\N	2020-10-12 16:39:09.284732
80622	ZTO	2020-10-05	29.29	29.29	28.67	29.08	6028738	{}	\N	2020-10-12 16:39:09.289505
80623	ZTO	2020-10-06	29.24	29.55	29.02	29.25	3107889	{}	\N	2020-10-12 16:39:09.294921
80624	ZTO	2020-10-07	29.46	29.46	28.6	28.94	5149605	{}	\N	2020-10-12 16:39:09.298761
80625	ZTO	2020-10-08	29.16	29.5	28.89	28.92	3126126	{}	\N	2020-10-12 16:39:09.302677
80626	ZTO	2020-10-09	28.87	29.98	28.87	29.91	3637935	{}	\N	2020-10-12 16:39:09.30697
80627	ZTO	2020-10-12	30.3	30.42	30.05	30.19	1073701	{}	\N	2020-10-12 16:39:09.311144
80628	AAMC	2020-09-28	21.1554	21.185	20.375	20.375	2557	{}	\N	2020-10-12 16:39:10.712362
80629	AAMC	2020-09-29	20.8	20.8	18.9	19.55	6048	{}	\N	2020-10-12 16:39:10.722913
80630	AAMC	2020-09-30	19.5	22.3	19.5	20.74	5406	{}	\N	2020-10-12 16:39:10.729658
80631	AAMC	2020-10-01	20.74	21.05	20.13	21	3382	{}	\N	2020-10-12 16:39:10.735904
80632	AAMC	2020-10-02	20.95	20.95	20.95	20.95	540	{}	\N	2020-10-12 16:39:10.741369
80633	AAMC	2020-10-05	21	21	19.6	20.19	5543	{}	\N	2020-10-12 16:39:10.749714
80634	AAMC	2020-10-06	19.487	20.78	19.44	20.5	3182	{}	\N	2020-10-12 16:39:10.754083
80635	AAMC	2020-10-07	20.84	20.87	20.55	20.55	1006	{}	\N	2020-10-12 16:39:10.758617
80636	AAMC	2020-10-08	20.5501	20.5501	19.48	20.4	1299	{}	\N	2020-10-12 16:39:10.763672
80637	AAMC	2020-10-12	20	21.8	20	21.8	1423	{}	\N	2020-10-12 16:39:10.768233
80638	ACGLP	2020-09-28	25.51	25.689	25.36	25.56	41158	{}	\N	2020-10-12 16:39:12.141974
80639	ACGLP	2020-09-29	25.56	25.6	25.52	25.59	16908	{}	\N	2020-10-12 16:39:12.149837
80640	ACGLP	2020-09-30	25.525	25.6	25.24	25.33	27030	{}	\N	2020-10-12 16:39:12.158019
80641	ACGLP	2020-10-01	25.35	25.6	25.35	25.6	22370	{}	\N	2020-10-12 16:39:12.163616
80642	ACGLP	2020-10-02	25.5	25.66	25.35	25.55	24825	{}	\N	2020-10-12 16:39:12.168277
80643	ACGLP	2020-10-05	25.46	25.6	25.46	25.49	23398	{}	\N	2020-10-12 16:39:12.172833
80644	ACGLP	2020-10-06	25.4886	25.6	25.45	25.6	30874	{}	\N	2020-10-12 16:39:12.177891
80645	ACGLP	2020-10-07	25.6	25.62	25.53	25.53	43014	{}	\N	2020-10-12 16:39:12.182854
80646	ACGLP	2020-10-08	25.51	25.65	25.51	25.65	24479	{}	\N	2020-10-12 16:39:12.187554
80647	ACGLP	2020-10-09	25.58	25.68	25.56	25.68	13782	{}	\N	2020-10-12 16:39:12.19245
80648	ACGLP	2020-10-12	25.6	25.64	25.58	25.6275	13047	{}	\N	2020-10-12 16:39:12.197141
80649	ADC	2020-09-28	63.61	64.51	63.135	64.33	420112	{}	\N	2020-10-12 16:39:13.203911
80650	ADC	2020-09-29	64.34	64.34	62.535	63.16	664450	{}	\N	2020-10-12 16:39:13.217849
80651	ADC	2020-09-30	64.13	65.01	63.02	63.64	957058	{}	\N	2020-10-12 16:39:13.226841
80652	ADC	2020-10-01	63.78	64.72	63.26	64.51	676877	{}	\N	2020-10-12 16:39:13.234106
80653	ADC	2020-10-02	63.4	65.46	63.13	65.22	400970	{}	\N	2020-10-12 16:39:13.240223
80654	ADC	2020-10-05	65.64	65.917	64.13	64.44	703810	{}	\N	2020-10-12 16:39:13.24585
80655	ADC	2020-10-06	65.01	66.12	64.16	65.38	666997	{}	\N	2020-10-12 16:39:13.251549
80656	ADC	2020-10-07	65.76	65.95	64.73	64.75	721236	{}	\N	2020-10-12 16:39:13.257026
80657	ADC	2020-10-08	65.3	66.8	65.13	66.58	771706	{}	\N	2020-10-12 16:39:13.262451
80658	ADC	2020-10-09	66.79	67.44	66.595	67	606145	{}	\N	2020-10-12 16:39:13.267811
80659	ADC	2020-10-12	67.33	67.33	66.55	66.915	111716	{}	\N	2020-10-12 16:39:13.272773
80660	ALTY	2020-09-28	10.47	10.6	10.47	10.56	6075	{}	\N	2020-10-12 16:39:14.427574
80661	ALTY	2020-09-29	10.75	10.75	10.45	10.54	36774	{}	\N	2020-10-12 16:39:14.439141
80662	ALTY	2020-09-30	10.62	10.62	10.53	10.57	20805	{}	\N	2020-10-12 16:39:14.446627
80663	ALTY	2020-10-01	10.59	10.6316	10.52	10.6316	6776	{}	\N	2020-10-12 16:39:14.45251
80664	ALTY	2020-10-02	10.5	10.72	10.5	10.72	5015	{}	\N	2020-10-12 16:39:14.45828
80665	ALTY	2020-10-05	10.69	10.69	10.64	10.6614	6857	{}	\N	2020-10-12 16:39:14.462986
80666	ALTY	2020-10-06	10.7	10.77	10.665	10.68	16883	{}	\N	2020-10-12 16:39:14.467093
80667	ALTY	2020-10-07	10.69	10.7098	10.64	10.6602	14048	{}	\N	2020-10-12 16:39:14.471068
80668	ALTY	2020-10-08	10.77	10.845	10.77	10.845	5988	{}	\N	2020-10-12 16:39:14.474833
80669	ALTY	2020-10-09	10.86	10.877	10.77	10.78	19241	{}	\N	2020-10-12 16:39:14.47878
80670	ALTY	2020-10-12	10.84	10.84	10.825	10.825	583	{}	\N	2020-10-12 16:39:14.482739
80671	APVO	2020-09-28	7.5	7.75	7.29	7.5	31991	{}	\N	2020-10-12 16:39:16.253407
80672	APVO	2020-09-29	7.54	7.92	7.44	7.85	30485	{}	\N	2020-10-12 16:39:16.265733
80673	APVO	2020-09-30	7.84	8.1	7.5	7.6	73762	{}	\N	2020-10-12 16:39:16.2733
80674	APVO	2020-10-01	7.56	7.69	7.42	7.62	13128	{}	\N	2020-10-12 16:39:16.279004
80675	APVO	2020-10-02	7.59	7.75	7.5	7.66	7007	{}	\N	2020-10-12 16:39:16.284324
80676	APVO	2020-10-05	7.66	7.95	7.66	7.73	16633	{}	\N	2020-10-12 16:39:16.288866
80677	APVO	2020-10-06	7.99	7.99	7.5	7.5	11605	{}	\N	2020-10-12 16:39:16.2926
80678	APVO	2020-10-07	7.29	7.6844	7.26	7.34	33569	{}	\N	2020-10-12 16:39:16.296247
80679	APVO	2020-10-08	7.45	7.45	7.05	7.05	35871	{}	\N	2020-10-12 16:39:16.300236
80680	APVO	2020-10-09	7.2396	7.43	7.1	7.17	24825	{}	\N	2020-10-12 16:39:16.304096
80681	APVO	2020-10-12	7.13	7.4519	7.13	7.28	7956	{}	\N	2020-10-12 16:39:16.307763
80682	ASHS	2020-09-28	33.14	33.2	33.12	33.17	2255	{}	\N	2020-10-12 16:39:17.229045
80683	ASHS	2020-09-29	33.11	33.5	33.11	33.48	3056	{}	\N	2020-10-12 16:39:17.239064
80684	ASHS	2020-09-30	33.34	33.58	33.3151	33.493	8516	{}	\N	2020-10-12 16:39:17.24532
80685	ASHS	2020-10-01	33.875	33.9267	33.87	33.9267	2908	{}	\N	2020-10-12 16:39:17.251152
80686	ASHS	2020-10-02	33.605	33.61	33.4941	33.4941	375	{}	\N	2020-10-12 16:39:17.25662
80687	ASHS	2020-10-05	33.875	33.9099	33.78	33.9099	5250	{}	\N	2020-10-12 16:39:17.2607
80688	ASHS	2020-10-06	33.98	34.14	33.815	33.975	4015	{}	\N	2020-10-12 16:39:17.264795
80689	ASHS	2020-10-07	34.19	34.1997	34.05	34.1997	4689	{}	\N	2020-10-12 16:39:17.268795
80690	ASHS	2020-10-08	34.2	34.3013	34.1986	34.3013	1751	{}	\N	2020-10-12 16:39:17.272762
80691	ASHS	2020-10-09	34.75	34.87	34.7301	34.87	9204	{}	\N	2020-10-12 16:39:17.276726
80692	ASHS	2020-10-12	35.51	35.5873	35.35	35.4	5585	{}	\N	2020-10-12 16:39:17.280783
80693	AXDX	2020-09-28	11.64	11.9893	10.86	10.91	177636	{}	\N	2020-10-12 16:39:19.455729
80694	AXDX	2020-09-29	10.98	11.27	10.54	10.81	139535	{}	\N	2020-10-12 16:39:19.470108
80695	AXDX	2020-09-30	10.77	11.15	10.55	10.66	111613	{}	\N	2020-10-12 16:39:19.478553
80696	AXDX	2020-10-01	10.79	10.79	10.39	10.45	116250	{}	\N	2020-10-12 16:39:19.485579
80697	AXDX	2020-10-02	10.39	10.86	10.12	10.68	161117	{}	\N	2020-10-12 16:39:19.491093
80698	AXDX	2020-10-05	10.74	10.895	10.66	10.69	91131	{}	\N	2020-10-12 16:39:19.496315
80699	AXDX	2020-10-06	10.68	11.22	10.55	10.82	278693	{}	\N	2020-10-12 16:39:19.500627
80700	AXDX	2020-10-07	10.75	11.51	10.7	11.19	301451	{}	\N	2020-10-12 16:39:19.50508
80701	AXDX	2020-10-08	11.2	11.77	10.64	11.06	225648	{}	\N	2020-10-12 16:39:19.509714
80702	AXDX	2020-10-09	11.01	11.11	10.815	11.06	204498	{}	\N	2020-10-12 16:39:19.514255
80703	AXDX	2020-10-12	11.55	11.55	10.9	11.19	82456	{}	\N	2020-10-12 16:39:19.518742
80704	BFIT	2020-09-28	21.97	22	21.9	21.975	3504	{}	\N	2020-10-12 16:39:21.45369
80705	BFIT	2020-09-29	22.07	22.07	21.8824	21.8824	5150	{}	\N	2020-10-12 16:39:21.468116
80706	BFIT	2020-09-30	21.9499	21.9499	21.9499	21.9499	1413	{}	\N	2020-10-12 16:39:21.476344
80707	BFIT	2020-10-01	22.14	22.21	22.13	22.21	3552	{}	\N	2020-10-12 16:39:21.482956
80708	BFIT	2020-10-02	22	22.3603	22	22.3603	4615	{}	\N	2020-10-12 16:39:21.489592
80709	BFIT	2020-10-05	22.71	23	22.61	23	7056	{}	\N	2020-10-12 16:39:21.495181
80710	BFIT	2020-10-06	22.74	22.7663	22.6027	22.6027	3311	{}	\N	2020-10-12 16:39:21.499869
80711	BFIT	2020-10-07	22.64	22.81	22.64	22.7	5096	{}	\N	2020-10-12 16:39:21.504975
80712	BFIT	2020-10-08	22.95	22.95	22.72	22.75	25262	{}	\N	2020-10-12 16:39:21.509735
80713	BFIT	2020-10-09	22.935	22.98	22.9	22.98	2334	{}	\N	2020-10-12 16:39:21.514634
80714	BFIT	2020-10-12	23.18	23.18	23.0557	23.0557	7867	{}	\N	2020-10-12 16:39:21.519608
80715	YRD	2020-09-28	2.77	2.8798	2.77	2.79	43444	{}	\N	2020-10-12 16:39:22.780734
80716	YRD	2020-09-29	2.79	3.09	2.77	3.08	292957	{}	\N	2020-10-12 16:39:22.792713
80717	YRD	2020-09-30	2.99	3.09	2.94	2.94	109003	{}	\N	2020-10-12 16:39:22.801768
80718	YRD	2020-10-01	2.95	3.05	2.9159	3.04	37234	{}	\N	2020-10-12 16:39:22.808252
80719	YRD	2020-10-02	2.97	3.04	2.96	3.04	74492	{}	\N	2020-10-12 16:39:22.813957
80720	YRD	2020-10-05	3.08	3.2399	3.0501	3.15	121454	{}	\N	2020-10-12 16:39:22.818841
80721	YRD	2020-10-06	3.15	3.27	3.0801	3.1	111739	{}	\N	2020-10-12 16:39:22.823744
80722	YRD	2020-10-07	3.16	3.1829	3.1	3.11	34238	{}	\N	2020-10-12 16:39:22.828374
80723	YRD	2020-10-08	3.16	3.16	3.03	3.1	43292	{}	\N	2020-10-12 16:39:22.833578
80724	YRD	2020-10-09	3.09	3.16	3.09	3.11	54878	{}	\N	2020-10-12 16:39:22.838706
80725	YRD	2020-10-12	3.11	3.14	3.06	3.13	36132	{}	\N	2020-10-12 16:39:22.843472
80726	ZDGE	2020-09-28	1.47	1.51	1.415	1.51	9537	{}	\N	2020-10-12 16:39:25.737961
80727	ZDGE	2020-09-29	1.4798	1.48	1.43	1.45	5910	{}	\N	2020-10-12 16:39:25.748278
80728	ZDGE	2020-09-30	1.45	1.47	1.4104	1.46	10668	{}	\N	2020-10-12 16:39:25.755383
80729	ZDGE	2020-10-01	1.421	1.56	1.421	1.56	36372	{}	\N	2020-10-12 16:39:25.761874
80730	ZDGE	2020-10-02	1.56	1.56	1.45	1.46	30082	{}	\N	2020-10-12 16:39:25.767383
80731	ZDGE	2020-10-05	1.45	1.52	1.45	1.48	28303	{}	\N	2020-10-12 16:39:25.77262
80732	ZDGE	2020-10-06	1.4627	1.58	1.44	1.55	34808	{}	\N	2020-10-12 16:39:25.777898
80733	ZDGE	2020-10-07	1.5527	1.5527	1.45	1.46	28088	{}	\N	2020-10-12 16:39:25.782933
80734	ZDGE	2020-10-08	1.4802	1.4802	1.45	1.46	6785	{}	\N	2020-10-12 16:39:25.787972
80735	ZDGE	2020-10-09	1.4593	1.57	1.4308	1.57	9134	{}	\N	2020-10-12 16:39:25.79345
80736	ZDGE	2020-10-12	1.57	1.6465	1.533	1.5402	27861	{}	\N	2020-10-12 16:39:25.798952
80737	ZSAN	2020-09-28	1.72	1.7495	1.63	1.66	5043894	{}	\N	2020-10-12 16:39:28.256331
80738	ZSAN	2020-09-29	1.66	1.67	1.58	1.6	3156168	{}	\N	2020-10-12 16:39:28.268426
80739	ZSAN	2020-09-30	1.62	1.65	1.58	1.62	7262765	{}	\N	2020-10-12 16:39:28.277012
80740	ZSAN	2020-10-01	0.76	0.7843	0.6152	0.7	48780600	{}	\N	2020-10-12 16:39:28.28448
80741	ZSAN	2020-10-02	0.6651	0.6999	0.652	0.6601	7649641	{}	\N	2020-10-12 16:39:28.289891
80742	ZSAN	2020-10-05	0.6943	0.6943	0.651	0.6727	5838265	{}	\N	2020-10-12 16:39:28.294432
80743	ZSAN	2020-10-06	0.68	0.7882	0.677	0.7439	10050707	{}	\N	2020-10-12 16:39:28.299074
80744	ZSAN	2020-10-07	0.77	0.78	0.71	0.7186	5078511	{}	\N	2020-10-12 16:39:28.303604
80745	ZSAN	2020-10-08	0.71	0.73	0.682	0.724	3443352	{}	\N	2020-10-12 16:39:28.308106
80746	ZSAN	2020-10-09	0.71	0.7239	0.698	0.7038	2205125	{}	\N	2020-10-12 16:39:28.312676
80747	ZSAN	2020-10-12	0.6908	0.7	0.63	0.6761	4175669	{}	\N	2020-10-12 16:39:28.317674
80748	A	2020-09-28	98.64	99.73	98.12	99.51	1341679	{}	\N	2020-10-12 16:39:30.933899
80749	A	2020-09-29	99.79	101.33	99.59	100.22	1268715	{}	\N	2020-10-12 16:39:30.944882
80750	A	2020-09-30	100.54	101.89	100.31	100.94	2430706	{}	\N	2020-10-12 16:39:30.953143
80751	A	2020-10-01	101.77	102.41	100.54	101.22	1284752	{}	\N	2020-10-12 16:39:30.959905
80752	A	2020-10-02	100.21	101.275	99.8127	100.01	1252537	{}	\N	2020-10-12 16:39:30.966756
80753	A	2020-10-05	100.99	103.46	100.96	103.12	1585990	{}	\N	2020-10-12 16:39:30.972612
80754	A	2020-10-06	103.12	103.33	101.48	101.63	1914824	{}	\N	2020-10-12 16:39:30.978621
80755	A	2020-10-07	102.47	104.75	102.42	103.88	2490579	{}	\N	2020-10-12 16:39:30.984176
80756	A	2020-10-08	104.2	104.39	103.13	104.16	1529817	{}	\N	2020-10-12 16:39:30.98991
80757	A	2020-10-09	104.83	106.1	104.55	105.76	1332770	{}	\N	2020-10-12 16:39:30.995833
80758	A	2020-10-12	106.45	107	105.79	106.09	903067	{}	\N	2020-10-12 16:39:31.001932
80759	ABB	2020-09-28	25.21	25.47	25.2	25.39	1352839	{}	\N	2020-10-12 16:39:31.946546
80760	ABB	2020-09-29	25.65	25.76	25.52	25.61	4110099	{}	\N	2020-10-12 16:39:31.95613
80761	ABB	2020-09-30	25.52	25.695	25.325	25.45	2531566	{}	\N	2020-10-12 16:39:31.962716
80762	ABB	2020-10-01	25.77	25.82	25.52	25.64	4419765	{}	\N	2020-10-12 16:39:31.969566
80763	ABB	2020-10-02	25.39	26	25.37	25.89	7658684	{}	\N	2020-10-12 16:39:31.975035
80764	ABB	2020-10-05	26.06	26.395	26.06	26.37	4181483	{}	\N	2020-10-12 16:39:31.980115
80765	ABB	2020-10-06	26.3	26.36	25.9	25.91	2867537	{}	\N	2020-10-12 16:39:31.984949
80766	ABB	2020-10-07	26.22	26.355	26.17	26.3	2533278	{}	\N	2020-10-12 16:39:31.98951
80767	ABB	2020-10-08	26.53	26.57	26.4	26.54	1833336	{}	\N	2020-10-12 16:39:31.993633
80768	ABB	2020-10-09	26.68	26.945	26.61	26.85	2075554	{}	\N	2020-10-12 16:39:31.998024
80769	ABB	2020-10-12	26.9	26.975	26.865	26.965	786691	{}	\N	2020-10-12 16:39:32.003145
80770	ACH	2020-09-28	5.11	5.15	5.11	5.15	78364	{}	\N	2020-10-12 16:39:33.633055
80771	ACH	2020-09-29	5.17	5.17	5.13	5.16	75274	{}	\N	2020-10-12 16:39:33.64228
80772	ACH	2020-09-30	5.14	5.2	5.131	5.2	62888	{}	\N	2020-10-12 16:39:33.648461
80773	ACH	2020-10-01	5.19	5.26	5.17	5.21	26571	{}	\N	2020-10-12 16:39:33.655556
80774	ACH	2020-10-02	5.16	5.28	5.16	5.22	182104	{}	\N	2020-10-12 16:39:33.660931
80775	ACH	2020-10-05	5.21	5.3	5.18	5.3	79822	{}	\N	2020-10-12 16:39:33.665187
80776	ACH	2020-10-06	5.43	5.49	5.4	5.44	71216	{}	\N	2020-10-12 16:39:33.669598
80777	ACH	2020-10-07	5.45	5.45	5.4	5.4	54916	{}	\N	2020-10-12 16:39:33.673886
80778	ACH	2020-10-08	5.5	5.57	5.49	5.57	69410	{}	\N	2020-10-12 16:39:33.677904
80779	ACH	2020-10-09	5.5	5.53	5.43	5.5	97591	{}	\N	2020-10-12 16:39:33.682094
80780	ACH	2020-10-12	5.55	5.5922	5.52	5.58	39056	{}	\N	2020-10-12 16:39:33.68754
80781	ACNB	2020-09-28	20.5	21.24	20.41	20.86	9703	{}	\N	2020-10-12 16:39:35.609502
80782	ACNB	2020-09-29	21.3238	21.5	20.78	21.24	11536	{}	\N	2020-10-12 16:39:35.621728
80783	ACNB	2020-09-30	21.36	21.8899	20.8	20.8	5343	{}	\N	2020-10-12 16:39:35.630058
80784	ACNB	2020-10-01	20.86	21.59	20.47	20.47	10418	{}	\N	2020-10-12 16:39:35.636182
80785	ACNB	2020-10-02	20.05	20.96	20.05	20.96	6820	{}	\N	2020-10-12 16:39:35.641203
80786	ACNB	2020-10-05	21	21.5	20.79	21.46	10996	{}	\N	2020-10-12 16:39:35.645827
80787	ACNB	2020-10-06	21.61	21.9	21.03	21.03	15381	{}	\N	2020-10-12 16:39:35.649921
80788	ACNB	2020-10-07	21.33	21.85	21.27	21.85	8961	{}	\N	2020-10-12 16:39:35.654572
80789	ACNB	2020-10-08	21.6	21.81	21.1596	21.65	5504	{}	\N	2020-10-12 16:39:35.658828
80790	ACNB	2020-10-09	21.6787	21.89	21.65	21.89	2869	{}	\N	2020-10-12 16:39:35.662846
80791	ACNB	2020-10-12	21.82	22.06	21.705	22.06	6136	{}	\N	2020-10-12 16:39:35.666931
80792	AIRR	2020-09-28	25.5	26.0339	25.5	25.96	17621	{}	\N	2020-10-12 16:39:37.066541
80793	AIRR	2020-09-29	25.55	25.8232	25.55	25.79	59159	{}	\N	2020-10-12 16:39:37.080141
80794	AIRR	2020-09-30	25.89	26.1325	25.53	25.64	7564	{}	\N	2020-10-12 16:39:37.089133
80795	AIRR	2020-10-01	25.68	25.89	25.55	25.88	6433	{}	\N	2020-10-12 16:39:37.095895
80796	AIRR	2020-10-02	25.4	26.3194	25.29	26.3	9874	{}	\N	2020-10-12 16:39:37.101871
80797	AIRR	2020-10-05	26.51	26.93	26.51	26.93	3660	{}	\N	2020-10-12 16:39:37.107353
80798	AIRR	2020-10-06	27.09	27.68	26.94	26.94	40088	{}	\N	2020-10-12 16:39:37.112982
80799	AIRR	2020-10-07	27.5105	27.633	27.43	27.633	19778	{}	\N	2020-10-12 16:39:37.118531
80800	AIRR	2020-10-08	27.76	27.93	27.6158	27.93	77388	{}	\N	2020-10-12 16:39:37.123546
80801	AIRR	2020-10-09	28.14	28.173	27.905	28.02	7260	{}	\N	2020-10-12 16:39:37.128634
80802	AIRR	2020-10-12	28.04	28.31	28.04	28.31	5286	{}	\N	2020-10-12 16:39:37.133917
80803	AMP	2020-09-28	149.67	152.8	147.9	151.48	647364	{}	\N	2020-10-12 16:39:38.126565
80804	AMP	2020-09-29	150.25	151.535	147	150.38	913893	{}	\N	2020-10-12 16:39:38.137786
80805	AMP	2020-09-30	151.63	156.38	150.84	154.11	1725141	{}	\N	2020-10-12 16:39:38.146613
80806	AMP	2020-10-01	155.18	156.91	153.17	155.31	1028699	{}	\N	2020-10-12 16:39:38.152941
80807	AMP	2020-10-02	152.37	160.69	152.02	158.85	965350	{}	\N	2020-10-12 16:39:38.158637
80808	AMP	2020-10-05	161.04	163.42	160.87	162.66	1303569	{}	\N	2020-10-12 16:39:38.163561
80809	AMP	2020-10-06	164.06	165.56	159.435	159.8	1096774	{}	\N	2020-10-12 16:39:38.168774
80810	AMP	2020-10-07	162.62	165.67	161.81	164.89	1252987	{}	\N	2020-10-12 16:39:38.174059
80811	AMP	2020-10-08	166.58	169.5	165.4048	169.31	899735	{}	\N	2020-10-12 16:39:38.179116
80812	AMP	2020-10-09	170	171.67	169.13	170.34	978186	{}	\N	2020-10-12 16:39:38.184201
80813	AMP	2020-10-12	169.75	172.31	169.75	171.945	249011	{}	\N	2020-10-12 16:39:38.189572
80814	APD	2020-09-28	293.74	295.81	292.13	292.67	700388	{}	\N	2020-10-12 16:39:39.156859
80815	APD	2020-09-29	294.63	296.78	291.81	293.86	1206653	{}	\N	2020-10-12 16:39:39.168702
80816	APD	2020-09-30	293.67	299.32	293.06	297.86	2196614	{}	\N	2020-10-12 16:39:39.177106
80817	APD	2020-10-01	299.06	301.74	293.57	294.67	2482139	{}	\N	2020-10-12 16:39:39.183753
80818	APD	2020-10-02	291.32	298.47	290.2	296.04	1905414	{}	\N	2020-10-12 16:39:39.189967
80819	APD	2020-10-05	298.29	304.67	296.19	300.32	2084865	{}	\N	2020-10-12 16:39:39.19572
80820	APD	2020-10-06	301.53	304.24	295.92	296.2	2813345	{}	\N	2020-10-12 16:39:39.201294
80821	APD	2020-10-07	300.7	304.73	299.6909	302.11	1215429	{}	\N	2020-10-12 16:39:39.206972
80822	APD	2020-10-08	303.02	304.68	299.09	299.66	1467209	{}	\N	2020-10-12 16:39:39.212471
80823	APD	2020-10-09	302.29	303.46	299.23	300.6	1895349	{}	\N	2020-10-12 16:39:39.217999
80824	APD	2020-10-12	303.88	304.86	301.88	304.01	491998	{}	\N	2020-10-12 16:39:39.223273
80825	AQMS	2020-09-28	0.8946	0.9472	0.8801	0.9303	137899	{}	\N	2020-10-12 16:39:42.82647
80826	AQMS	2020-09-29	0.96	0.99	0.92	0.9246	286384	{}	\N	2020-10-12 16:39:42.838226
80827	AQMS	2020-09-30	0.9401	0.940946	0.905	0.911	276093	{}	\N	2020-10-12 16:39:42.845122
80828	AQMS	2020-10-01	0.9253	0.9299	0.89	0.9094	115025	{}	\N	2020-10-12 16:39:42.851416
80829	AQMS	2020-10-02	0.899	0.9	0.821	0.8948	135415	{}	\N	2020-10-12 16:39:42.857289
80830	AQMS	2020-10-05	0.8881	0.95	0.8742	0.94	223691	{}	\N	2020-10-12 16:39:42.862767
80831	AQMS	2020-10-06	0.93	1.6	0.875101	1.08	14007239	{}	\N	2020-10-12 16:39:42.867916
80832	AQMS	2020-10-07	1.12	1.27	1.09	1.15	4710346	{}	\N	2020-10-12 16:39:42.872581
80833	AQMS	2020-10-08	1.15	1.16	1	1.03	1624096	{}	\N	2020-10-12 16:39:42.877128
80834	AQMS	2020-10-09	0.9976	1.01	0.895	0.96	1296365	{}	\N	2020-10-12 16:39:42.88211
80835	AQMS	2020-10-12	0.93	0.94	0.89	0.926	678120	{}	\N	2020-10-12 16:39:42.887259
80836	YUM	2020-09-28	91.88	92.45	91.39	91.88	1755550	{}	\N	2020-10-12 16:39:43.80189
80837	YUM	2020-09-29	91.58	91.88	90.24	90.71	3170795	{}	\N	2020-10-12 16:39:43.811252
80838	YUM	2020-09-30	91.12	92.525	90.76	91.3	3297316	{}	\N	2020-10-12 16:39:43.81825
80839	YUM	2020-10-01	92.15	92.51	91.21	91.82	2574218	{}	\N	2020-10-12 16:39:43.827012
80840	YUM	2020-10-02	90.62	95.31	90.33	95.17	4276479	{}	\N	2020-10-12 16:39:43.833273
80841	YUM	2020-10-05	95.68	95.78	93.35	94.09	2776879	{}	\N	2020-10-12 16:39:43.8398
80842	YUM	2020-10-06	94.56	94.83	91.8284	91.92	2451528	{}	\N	2020-10-12 16:39:43.845617
80843	YUM	2020-10-07	92.8	94.47	92.71	93.88	3019510	{}	\N	2020-10-12 16:39:43.851369
80844	YUM	2020-10-08	94.19	95.43	93.89	95.41	2497058	{}	\N	2020-10-12 16:39:43.857511
80845	YUM	2020-10-09	95.63	96.08	94.82	95.49	2293376	{}	\N	2020-10-12 16:39:43.86401
80846	YUM	2020-10-12	95.51	96.13	94.85	95.99	533106	{}	\N	2020-10-12 16:39:43.870628
80847	Z	2020-09-28	102.85	103.12	100.07	101.16	2765507	{}	\N	2020-10-12 16:39:45.044839
80848	Z	2020-09-29	101.26	102.81	100.19	101.42	1747487	{}	\N	2020-10-12 16:39:45.056682
80849	Z	2020-09-30	101.48	104.05	100.92	101.59	3391077	{}	\N	2020-10-12 16:39:45.06364
80850	Z	2020-10-01	103.5241	108.22	102.8	107.53	3502393	{}	\N	2020-10-12 16:39:45.070487
80851	Z	2020-10-02	103.92	109.67	103.61	108.18	2193439	{}	\N	2020-10-12 16:39:45.07603
80852	Z	2020-10-05	109.81	112.49	109.38	110.11	2413837	{}	\N	2020-10-12 16:39:45.081167
80853	Z	2020-10-06	109.53	110.94	104.25	104.99	3214955	{}	\N	2020-10-12 16:39:45.086118
80854	Z	2020-10-07	107.71	108.26	104.42	105.51	3544821	{}	\N	2020-10-12 16:39:45.091239
80855	Z	2020-10-08	106.62	106.78	104.05	105.04	2336003	{}	\N	2020-10-12 16:39:45.096059
80856	Z	2020-10-09	105.25	106.5	103.52	104.26	1859940	{}	\N	2020-10-12 16:39:45.100298
80857	Z	2020-10-12	106.48	106.48	102.23	102.72	876655	{}	\N	2020-10-12 16:39:45.104761
80858	ZEUS	2020-09-28	11.62	11.86	11.42	11.79	50809	{}	\N	2020-10-12 16:39:47.043741
80859	ZEUS	2020-09-29	11.65	11.89	10.87	10.99	39465	{}	\N	2020-10-12 16:39:47.057238
80860	ZEUS	2020-09-30	11.2	11.6025	11.06	11.36	73418	{}	\N	2020-10-12 16:39:47.065606
80861	ZEUS	2020-10-01	11.34	11.76	11.07	11.63	25966	{}	\N	2020-10-12 16:39:47.071151
80862	ZEUS	2020-10-02	11.42	11.995	11.42	11.83	37706	{}	\N	2020-10-12 16:39:47.077167
80863	ZEUS	2020-10-05	11.82	12.2	11.78	11.86	64960	{}	\N	2020-10-12 16:39:47.082186
80864	ZEUS	2020-10-06	11.94	12.1	10.84	10.89	38881	{}	\N	2020-10-12 16:39:47.087852
80865	ZEUS	2020-10-07	11.16	13.23	11.16	12.86	142296	{}	\N	2020-10-12 16:39:47.092858
80866	ZEUS	2020-10-08	13.15	13.15	12	12.54	63499	{}	\N	2020-10-12 16:39:47.097522
80867	ZEUS	2020-10-09	12.68	12.78	12.24	12.28	34985	{}	\N	2020-10-12 16:39:47.102008
80868	ZEUS	2020-10-12	12.41	12.48	12.325	12.48	6125	{}	\N	2020-10-12 16:39:47.10638
80869	ZM	2020-09-28	502.41	519.77	479.57	487.66	17591352	{}	\N	2020-10-12 16:39:49.772271
80870	ZM	2020-09-29	488.13	489.8	463	465.5	15237612	{}	\N	2020-10-12 16:39:49.78429
80871	ZM	2020-09-30	464.21	481.5552	462	470.11	8960013	{}	\N	2020-10-12 16:39:49.791054
80872	ZM	2020-10-01	477	483.73	465	482.99	7811947	{}	\N	2020-10-12 16:39:49.797369
80873	ZM	2020-10-02	485.005	507.4899	481	482.23	12744453	{}	\N	2020-10-12 16:39:49.803372
80874	ZM	2020-10-05	493.97	496.9	475.26	485.37	7572810	{}	\N	2020-10-12 16:39:49.80801
80875	ZM	2020-10-06	480.51	487.8999	473.2229	478.25	6033608	{}	\N	2020-10-12 16:39:49.812343
80876	ZM	2020-10-07	482.5	487.5	477.11	480.61	4407075	{}	\N	2020-10-12 16:39:49.816973
80877	ZM	2020-10-08	484	489.49	475.58	478.55	4221097	{}	\N	2020-10-12 16:39:49.822436
80878	ZM	2020-10-09	487.99	498.9	484.08	492.41	8801755	{}	\N	2020-10-12 16:39:49.830655
80879	ZM	2020-10-12	501.48	506.11	490.4779	496.2	4164783	{}	\N	2020-10-12 16:39:49.835771
80880	ZSL	2020-09-28	9.82	10.01	9.485	9.54	1928516	{}	\N	2020-10-12 16:39:50.822965
80881	ZSL	2020-09-29	9.36	9.4	8.925	9.08	1689195	{}	\N	2020-10-12 16:39:50.835668
80882	ZSL	2020-09-30	9.29	9.91	9.19	9.81	3874960	{}	\N	2020-10-12 16:39:50.842694
80883	ZSL	2020-10-01	9.45	9.68	9.03	9.35	3874533	{}	\N	2020-10-12 16:39:50.848733
80884	ZSL	2020-10-02	9.24	9.4	9	9.33	2055204	{}	\N	2020-10-12 16:39:50.85546
80885	ZSL	2020-10-05	9.18	9.18	8.735	8.885	1624675	{}	\N	2020-10-12 16:39:50.860799
80886	ZSL	2020-10-06	8.9	9.73	8.83	9.66	2589357	{}	\N	2020-10-12 16:39:50.865853
80887	ZSL	2020-10-07	9.44	9.5	9.135	9.32	1563885	{}	\N	2020-10-12 16:39:50.871095
80888	ZSL	2020-10-08	9.09	9.39	8.94	9.24	907259	{}	\N	2020-10-12 16:39:50.876489
80889	ZSL	2020-10-09	8.7	8.72	8.23	8.23	3173562	{}	\N	2020-10-12 16:39:50.881794
80890	ZSL	2020-10-12	8.29	8.47	8.19	8.2001	652151	{}	\N	2020-10-12 16:39:50.88683
80891	ZYME	2020-09-28	45.17	46.1	44.14	44.98	297094	{}	\N	2020-10-12 16:39:51.800949
80892	ZYME	2020-09-29	44.91	45.94	44.533	45.08	512728	{}	\N	2020-10-12 16:39:51.812256
80893	ZYME	2020-09-30	45.09	46.76	44.7	46.58	870185	{}	\N	2020-10-12 16:39:51.820302
80894	ZYME	2020-10-01	46.97	47	44.17	46.09	2032234	{}	\N	2020-10-12 16:39:51.827351
80895	ZYME	2020-10-02	45.24	45.85	42.63	43.34	625912	{}	\N	2020-10-12 16:39:51.833235
80896	ZYME	2020-10-05	43.6	44.84	43.43	44.66	272322	{}	\N	2020-10-12 16:39:51.838675
80897	ZYME	2020-10-06	44.73	45.02	43.39	43.57	265375	{}	\N	2020-10-12 16:39:51.844125
80898	ZYME	2020-10-07	43.99	45.68	43.74	45.06	454623	{}	\N	2020-10-12 16:39:51.849392
80899	ZYME	2020-10-08	45.27	45.97	44.6	45.61	288996	{}	\N	2020-10-12 16:39:51.854609
80900	ZYME	2020-10-09	45.53	47.12	45.33	47.1	847243	{}	\N	2020-10-12 16:39:51.859921
80901	ZYME	2020-10-12	47.07	47.45	46.1601	46.53	84024	{}	\N	2020-10-12 16:39:51.865558
80902	AAN	2020-09-28	56.84	57.88	56.1	57.84	440083	{}	\N	2020-10-12 16:39:52.84416
80903	AAN	2020-09-29	58	58	56.16	57.28	641722	{}	\N	2020-10-12 16:39:52.854852
80904	AAN	2020-09-30	57.58	59.08	55.84	56.65	1204466	{}	\N	2020-10-12 16:39:52.862198
80905	AAN	2020-10-01	56.77	57.33	54.95	56.18	1185511	{}	\N	2020-10-12 16:39:52.868295
80906	AAN	2020-10-02	54.99	57.42	54.79	57.1	692613	{}	\N	2020-10-12 16:39:52.873451
80907	AAN	2020-10-05	57.99	59.165	57.3	58.99	856475	{}	\N	2020-10-12 16:39:52.878281
80908	AAN	2020-10-06	59.87	60.13	57.11	57.56	1012107	{}	\N	2020-10-12 16:39:52.882667
80909	AAN	2020-10-07	58.26	59.95	58.1912	59.06	861027	{}	\N	2020-10-12 16:39:52.88722
80910	AAN	2020-10-08	59.82	60.11	58.77	59.73	800390	{}	\N	2020-10-12 16:39:52.892192
80911	AAN	2020-10-09	60.65	61.19	58.55	58.67	970667	{}	\N	2020-10-12 16:39:52.89752
80912	AAN	2020-10-12	58.61	59.45	58.42	58.795	262457	{}	\N	2020-10-12 16:39:52.902404
80913	AAU	2020-09-28	0.88	0.894399	0.8425	0.852	505914	{}	\N	2020-10-12 16:39:53.857425
80914	AAU	2020-09-29	0.87	0.8899	0.82	0.8205	368624	{}	\N	2020-10-12 16:39:53.870974
80915	AAU	2020-09-30	0.83	0.8374	0.82	0.8252	247669	{}	\N	2020-10-12 16:39:53.879199
80916	AAU	2020-10-01	0.83	0.843	0.7717	0.8067	625412	{}	\N	2020-10-12 16:39:53.886084
80917	AAU	2020-10-02	0.81	0.82	0.781	0.8114	261041	{}	\N	2020-10-12 16:39:53.891723
80918	AAU	2020-10-05	0.81	0.829	0.79	0.7957	269474	{}	\N	2020-10-12 16:39:53.897167
80919	AAU	2020-10-06	0.81	0.8182	0.771	0.7835	355398	{}	\N	2020-10-12 16:39:53.901832
80920	AAU	2020-10-07	0.8	0.81	0.77	0.7886	285410	{}	\N	2020-10-12 16:39:53.906681
80921	AAU	2020-10-08	0.77	0.795	0.75	0.777	467826	{}	\N	2020-10-12 16:39:53.91121
80922	AAU	2020-10-09	0.78	0.909	0.78	0.845	1194661	{}	\N	2020-10-12 16:39:53.915685
80923	AAU	2020-10-12	0.8873	0.899	0.822	0.8279	208308	{}	\N	2020-10-12 16:39:53.920691
80924	ACET	2020-09-28	11.24	11.24	10.53	10.57	243470	{}	\N	2020-10-12 16:39:54.838066
80925	ACET	2020-09-29	10.585	12.71	10.585	12.53	155427	{}	\N	2020-10-12 16:39:54.849724
80926	ACET	2020-09-30	12.82	12.82	11.7	11.89	84963	{}	\N	2020-10-12 16:39:54.857401
80927	ACET	2020-10-01	12	12.5	11.89	12.21	44831	{}	\N	2020-10-12 16:39:54.866117
80928	ACET	2020-10-02	11.68	12.31	11.57	11.89	71408	{}	\N	2020-10-12 16:39:54.871722
80929	ACET	2020-10-05	12.08	12.3299	11.92	12.03	32020	{}	\N	2020-10-12 16:39:54.876327
80930	ACET	2020-10-06	12.28	12.44	12	12.14	29573	{}	\N	2020-10-12 16:39:54.881109
80931	ACET	2020-10-07	12.22	12.77	11.9629	12.66	31585	{}	\N	2020-10-12 16:39:54.885848
80932	ACET	2020-10-08	12.92	12.98	12.37	12.64	21385	{}	\N	2020-10-12 16:39:54.890727
80933	ACET	2020-10-09	12.93	13.9599	12.92	13.72	58771	{}	\N	2020-10-12 16:39:54.896292
80934	ACET	2020-10-12	13.72	14.3499	13.6017	14.15	36225	{}	\N	2020-10-12 16:39:54.903315
80935	ADMP	2020-09-28	0.75	0.85	0.742	0.7785	6117242	{}	\N	2020-10-12 16:39:56.627638
80936	ADMP	2020-09-29	0.7909	0.819	0.76015	0.8048	2300725	{}	\N	2020-10-12 16:39:56.639015
80937	ADMP	2020-09-30	0.8073	0.8197	0.775	0.7834	1800290	{}	\N	2020-10-12 16:39:56.645838
80938	ADMP	2020-10-01	0.7889	0.8046	0.76	0.7681	1600444	{}	\N	2020-10-12 16:39:56.651841
80939	ADMP	2020-10-02	0.7034	0.767	0.7034	0.73	2718861	{}	\N	2020-10-12 16:39:56.657142
80940	ADMP	2020-10-05	0.74	0.7697	0.73	0.7317	1722202	{}	\N	2020-10-12 16:39:56.662083
80941	ADMP	2020-10-06	0.7397	0.7797	0.7311	0.77	3781739	{}	\N	2020-10-12 16:39:56.666267
80942	ADMP	2020-10-07	0.7866	0.8497	0.771	0.83	4587102	{}	\N	2020-10-12 16:39:56.670613
80943	ADMP	2020-10-08	0.86	0.9199	0.835	0.8876	8208910	{}	\N	2020-10-12 16:39:56.67516
80944	ADMP	2020-10-09	0.9399	1.03	0.8603	0.9845	15731487	{}	\N	2020-10-12 16:39:56.679271
80945	ADMP	2020-10-12	1.01	1.06	0.98	1.0201	9320547	{}	\N	2020-10-12 16:39:56.683332
80946	AEB	2020-09-28	24.8551	25.05	24.85	25.05	13845	{}	\N	2020-10-12 16:39:57.638939
80947	AEB	2020-09-29	25.05	25.06	24.95	25.0036	27698	{}	\N	2020-10-12 16:39:57.650269
80948	AEB	2020-09-30	24.9646	25.0303	24.67	24.67	123162	{}	\N	2020-10-12 16:39:57.657505
80949	AEB	2020-10-01	24.6	24.87	24.55	24.8216	30784	{}	\N	2020-10-12 16:39:57.663441
80950	AEB	2020-10-02	24.65	24.94	24.65	24.89	56215	{}	\N	2020-10-12 16:39:57.668037
80951	AEB	2020-10-05	24.94	25	24.88	24.98	34868	{}	\N	2020-10-12 16:39:57.672521
80952	AEB	2020-10-06	24.9	25.17	24.9	25.09	41241	{}	\N	2020-10-12 16:39:57.676492
80953	AEB	2020-10-07	25.07	25.14	25	25.08	25538	{}	\N	2020-10-12 16:39:57.680294
80954	AEB	2020-10-08	25.01	25.06	24.75	25	49480	{}	\N	2020-10-12 16:39:57.683816
80955	AEB	2020-10-09	24.9495	25.06	24.9495	25.05	13220	{}	\N	2020-10-12 16:39:57.687539
80956	AEB	2020-10-12	25.05	25.09	25.0445	25.054	16050	{}	\N	2020-10-12 16:39:57.691197
80957	AEHR	2020-09-28	1.52	1.555	1.39	1.42	273339	{}	\N	2020-10-12 16:40:00.191033
80958	AEHR	2020-09-29	1.46	1.4988	1.39	1.4	77521	{}	\N	2020-10-12 16:40:00.203337
80959	AEHR	2020-09-30	1.41	1.438	1.39	1.39	108741	{}	\N	2020-10-12 16:40:00.21104
80960	AEHR	2020-10-01	1.3942	1.4012	1.375	1.4	58980	{}	\N	2020-10-12 16:40:00.216952
80961	AEHR	2020-10-02	1.35	1.3898	1.32	1.36	207931	{}	\N	2020-10-12 16:40:00.222227
80962	AEHR	2020-10-05	1.3671	1.39	1.32	1.32	95246	{}	\N	2020-10-12 16:40:00.226956
80963	AEHR	2020-10-06	1.33	1.4	1.33	1.34	153129	{}	\N	2020-10-12 16:40:00.232182
80964	AEHR	2020-10-07	1.32	1.44	1.32	1.415	222340	{}	\N	2020-10-12 16:40:00.236803
80965	AEHR	2020-10-08	1.42	1.43	1.34	1.41	325189	{}	\N	2020-10-12 16:40:00.241352
80966	AEHR	2020-10-09	1.39	1.48	1.385	1.46	110885	{}	\N	2020-10-12 16:40:00.24627
80967	AEHR	2020-10-12	1.46	1.465	1.37	1.41	143499	{}	\N	2020-10-12 16:40:00.251049
80968	AMNB	2020-09-28	20.8093	21.38	20.57	21.09	7457	{}	\N	2020-10-12 16:40:02.828675
80969	AMNB	2020-09-29	20.89	21.035	19.7501	20.7	8108	{}	\N	2020-10-12 16:40:02.842039
80970	AMNB	2020-09-30	21	21.35	20.92	20.92	4945	{}	\N	2020-10-12 16:40:02.849776
80971	AMNB	2020-10-01	21.11	21.435	20.54	20.69	7652	{}	\N	2020-10-12 16:40:02.856291
80972	AMNB	2020-10-02	20.51	21.65	20.51	21.52	10310	{}	\N	2020-10-12 16:40:02.862131
80973	AMNB	2020-10-05	21	22.57	21	21.95	9829	{}	\N	2020-10-12 16:40:02.867411
80974	AMNB	2020-10-06	22.02	22.71	22.02	22.2	20634	{}	\N	2020-10-12 16:40:02.872118
80975	AMNB	2020-10-07	22.28	22.5	22.155	22.5	18193	{}	\N	2020-10-12 16:40:02.876731
80976	AMNB	2020-10-08	22.0683	22.7	22.0574	22.33	17039	{}	\N	2020-10-12 16:40:02.88673
80977	AMNB	2020-10-09	22.21	22.6	22.11	22.26	16142	{}	\N	2020-10-12 16:40:02.905152
80978	AMNB	2020-10-12	22.1814	22.37	22.15	22.235	5385	{}	\N	2020-10-12 16:40:03.002462
80979	BBU	2020-09-28	31.48	31.62	31.03	31.26	42734	{}	\N	2020-10-12 16:40:04.39784
80980	BBU	2020-09-29	31.36	31.51	29.8701	30.46	102740	{}	\N	2020-10-12 16:40:04.409494
80981	BBU	2020-09-30	30.08	31.2	30.08	30.13	257711	{}	\N	2020-10-12 16:40:04.416993
80982	BBU	2020-10-01	30.47	31.9	30.285	31.53	144124	{}	\N	2020-10-12 16:40:04.424024
80983	BBU	2020-10-02	30.98	31.39	30.85	31.15	70725	{}	\N	2020-10-12 16:40:04.429196
80984	BBU	2020-10-05	31.22	31.45	30.94	31.17	148688	{}	\N	2020-10-12 16:40:04.435117
80985	BBU	2020-10-06	31.1	31.14	30.26	30.69	105345	{}	\N	2020-10-12 16:40:04.44142
80986	BBU	2020-10-07	30.64	32.4	30.64	32.12	80886	{}	\N	2020-10-12 16:40:04.446361
80987	BBU	2020-10-08	31.9	33.81	31.9	33.64	43292	{}	\N	2020-10-12 16:40:04.452275
80988	BBU	2020-10-09	33.68	33.68	32.0201	32.75	77544	{}	\N	2020-10-12 16:40:04.458309
80989	BBU	2020-10-12	33.18	33.18	32.8	32.91	10650	{}	\N	2020-10-12 16:40:04.46401
80990	YTRA	2020-09-28	0.7099	0.75	0.6906	0.7241	359256	{}	\N	2020-10-12 16:40:05.906642
80991	YTRA	2020-09-29	0.7201	0.7599	0.72	0.7472	266684	{}	\N	2020-10-12 16:40:05.918754
80992	YTRA	2020-09-30	0.7473	0.76	0.7301	0.75	199647	{}	\N	2020-10-12 16:40:05.926304
80993	YTRA	2020-10-01	0.75	0.7599	0.7433	0.7438	171113	{}	\N	2020-10-12 16:40:05.932766
80994	YTRA	2020-10-02	0.7438	0.765	0.72	0.754	349843	{}	\N	2020-10-12 16:40:05.93818
80995	YTRA	2020-10-05	0.75	0.77	0.730101	0.7611	249227	{}	\N	2020-10-12 16:40:05.942767
80996	YTRA	2020-10-06	0.77	0.81	0.76	0.763	531698	{}	\N	2020-10-12 16:40:05.947704
80997	YTRA	2020-10-07	0.7401	0.8	0.74	0.7822	252805	{}	\N	2020-10-12 16:40:05.952743
80998	YTRA	2020-10-08	0.78	0.79	0.765	0.7843	157147	{}	\N	2020-10-12 16:40:05.958261
80999	YTRA	2020-10-09	0.795	0.8407	0.77	0.7951	1083094	{}	\N	2020-10-12 16:40:05.963001
81000	YTRA	2020-10-12	0.7944	0.81	0.7831	0.80011	113386	{}	\N	2020-10-12 16:40:05.968067
81001	ZEAL	2020-09-28	36.1	36.19	36.02	36.19	604	{}	\N	2020-10-12 16:40:07.626263
81002	ZEAL	2020-09-29	36.54	38	36.54	37.1	13289	{}	\N	2020-10-12 16:40:07.638702
81003	ZEAL	2020-09-30	37.96	38.41	37.95	38.41	13280	{}	\N	2020-10-12 16:40:07.645996
81004	ZEAL	2020-10-01	38.82	39.14	38.82	39.13	3432	{}	\N	2020-10-12 16:40:07.653308
81005	ZEAL	2020-10-02	40.34	42	40.34	41.1499	9505	{}	\N	2020-10-12 16:40:07.659497
81006	ZEAL	2020-10-05	42	44.42	42	44	17541	{}	\N	2020-10-12 16:40:07.667936
81007	ZEAL	2020-10-06	42.87	43.08	42.45	42.61	8689	{}	\N	2020-10-12 16:40:07.672702
81008	ZEAL	2020-10-07	42.175	42.35	42.175	42.35	1801	{}	\N	2020-10-12 16:40:07.677947
81009	ZEAL	2020-10-08	41.75	42.04	41.49	41.6454	7260	{}	\N	2020-10-12 16:40:07.683735
81010	ZEAL	2020-10-09	42.38	42.38	42.12	42.2	1980	{}	\N	2020-10-12 16:40:07.690085
81011	ZEAL	2020-10-12	42.63	42.82	42.63	42.68	7399	{}	\N	2020-10-12 16:40:07.695511
81012	ZIOP	2020-09-28	2.49	2.55	2.42	2.49	857893	{}	\N	2020-10-12 16:40:09.050602
81013	ZIOP	2020-09-29	2.5	2.575	2.48	2.57	1035429	{}	\N	2020-10-12 16:40:09.061818
81014	ZIOP	2020-09-30	2.54	2.67	2.52	2.52	1285820	{}	\N	2020-10-12 16:40:09.06907
81015	ZIOP	2020-10-01	2.55	2.74	2.55	2.62	1628416	{}	\N	2020-10-12 16:40:09.074931
81016	ZIOP	2020-10-02	2.58	2.705	2.56	2.63	1152681	{}	\N	2020-10-12 16:40:09.082634
81017	ZIOP	2020-10-05	2.65	2.74	2.65	2.69	1107919	{}	\N	2020-10-12 16:40:09.086983
81018	ZIOP	2020-10-06	2.69	2.72	2.61	2.61	984475	{}	\N	2020-10-12 16:40:09.091249
81019	ZIOP	2020-10-07	2.67	2.77	2.65	2.73	1074180	{}	\N	2020-10-12 16:40:09.095362
81020	ZIOP	2020-10-08	2.76	2.82	2.7	2.76	853372	{}	\N	2020-10-12 16:40:09.099517
81021	ZIOP	2020-10-09	2.77	2.86	2.76	2.81	1181852	{}	\N	2020-10-12 16:40:09.10393
81022	ZIOP	2020-10-12	2.81	2.86	2.75	2.815	525566	{}	\N	2020-10-12 16:40:09.108373
81023	AA	2020-09-28	11.83	12.33	11.82	12.24	4990986	{}	\N	2020-10-12 16:40:10.107007
81024	AA	2020-09-29	12.1799	12.1799	11.55	11.67	9817256	{}	\N	2020-10-12 16:40:10.115312
81025	AA	2020-09-30	11.68	12.1	11.56	11.63	10304979	{}	\N	2020-10-12 16:40:10.121272
81026	AA	2020-10-01	11.69	11.88	11.29	11.39	11625670	{}	\N	2020-10-12 16:40:10.126626
81027	AA	2020-10-02	11.02	11.95	10.98	11.82	10428780	{}	\N	2020-10-12 16:40:10.132246
81028	AA	2020-10-05	11.95	12.17	11.835	11.93	10811708	{}	\N	2020-10-12 16:40:10.137111
81029	AA	2020-10-06	12.11	12.47	11.66	11.67	12868439	{}	\N	2020-10-12 16:40:10.141413
81030	AA	2020-10-07	11.96	12.3	11.96	12.24	7519443	{}	\N	2020-10-12 16:40:10.145203
81031	AA	2020-10-08	12.45	12.74	12.38	12.73	9688444	{}	\N	2020-10-12 16:40:10.14974
81032	AA	2020-10-09	12.94	12.97	12.25	12.52	16223456	{}	\N	2020-10-12 16:40:10.154303
81033	AA	2020-10-12	12.59	12.65	12.3494	12.495	4080012	{}	\N	2020-10-12 16:40:10.158391
81034	AB	2020-09-28	27.48	27.8	26.845	27.45	524359	{}	\N	2020-10-12 16:40:12.204738
81035	AB	2020-09-29	27.45	27.45	26.75	27.03	596344	{}	\N	2020-10-12 16:40:12.215016
81036	AB	2020-09-30	26.98	27.57	26.98	27.04	711292	{}	\N	2020-10-12 16:40:12.222583
81037	AB	2020-10-01	27.04	27.16	26.52	26.82	894122	{}	\N	2020-10-12 16:40:12.229366
81038	AB	2020-10-02	26.5	27.25	26.5	26.96	497612	{}	\N	2020-10-12 16:40:12.234896
81039	AB	2020-10-05	26.99	27.42	26.95	27.34	745332	{}	\N	2020-10-12 16:40:12.240294
81040	AB	2020-10-06	27.4	28.4	27.27	27.62	965159	{}	\N	2020-10-12 16:40:12.245099
81041	AB	2020-10-07	27.96	28.32	27.68	28.1	637073	{}	\N	2020-10-12 16:40:12.249794
81042	AB	2020-10-08	28.7	29.84	28.5	29.74	1983813	{}	\N	2020-10-12 16:40:12.254903
81043	AB	2020-10-09	29.8	30.37	29.4332	30.26	997769	{}	\N	2020-10-12 16:40:12.259558
81044	AB	2020-10-12	32.98	33.5	31.07	32.36	2444324	{}	\N	2020-10-12 16:40:12.264368
81045	ACCO	2020-09-28	5.98	6.16	5.98	6.09	460292	{}	\N	2020-10-12 16:40:16.072871
81046	ACCO	2020-09-29	6.15	6.17	5.84	5.96	1092159	{}	\N	2020-10-12 16:40:16.085161
81047	ACCO	2020-09-30	5.92	6.08	5.785	5.8	917230	{}	\N	2020-10-12 16:40:16.092867
81048	ACCO	2020-10-01	5.75	5.93	5.745	5.78	650706	{}	\N	2020-10-12 16:40:16.098805
81049	ACCO	2020-10-02	5.67	6	5.67	5.94	622156	{}	\N	2020-10-12 16:40:16.103535
81050	ACCO	2020-10-05	5.96	6.21	5.92	6.17	877846	{}	\N	2020-10-12 16:40:16.108307
81051	ACCO	2020-10-06	6.25	6.345	6.1	6.11	984887	{}	\N	2020-10-12 16:40:16.11329
81052	ACCO	2020-10-07	6.16	6.37	6.11	6.29	964581	{}	\N	2020-10-12 16:40:16.117705
81053	ACCO	2020-10-08	6.38	6.5	6.295	6.44	748869	{}	\N	2020-10-12 16:40:16.122282
81054	ACCO	2020-10-09	6.5	6.6	6.4	6.49	731633	{}	\N	2020-10-12 16:40:16.127275
81055	ACCO	2020-10-12	6.5	6.59	6.38	6.54	142875	{}	\N	2020-10-12 16:40:16.131947
81056	AIRT	2020-09-28	9.65	9.9175	9.65	9.9175	297	{}	\N	2020-10-12 16:40:17.473031
81057	AIRT	2020-09-29	9.69	9.69	9.69	9.69	166	{}	\N	2020-10-12 16:40:17.484831
81058	AIRT	2020-09-30	9.825	9.825	9.65	9.735	690	{}	\N	2020-10-12 16:40:17.492673
81059	AIRT	2020-10-01	9.71	9.71	8.95	9.2546	3727	{}	\N	2020-10-12 16:40:17.500123
81060	AIRT	2020-10-02	9	9.7	9	9.7	572	{}	\N	2020-10-12 16:40:17.506662
81061	AIRT	2020-10-05	9.7	9.7923	9.7	9.7923	575	{}	\N	2020-10-12 16:40:17.512088
81062	AIRT	2020-10-06	9.5	9.6354	9.5	9.5	1607	{}	\N	2020-10-12 16:40:17.517221
81063	AIRT	2020-10-07	9.625	9.625	9.625	9.625	357	{}	\N	2020-10-12 16:40:17.522423
81064	AIRT	2020-10-08	9.6475	9.66	9.6475	9.65	1324	{}	\N	2020-10-12 16:40:17.527707
81065	AIRT	2020-10-09	9.86	11	9.26	9.729	12478	{}	\N	2020-10-12 16:40:17.532758
81066	AIRT	2020-10-12	9.26	9.26	9.26	9.26	396	{}	\N	2020-10-12 16:40:17.537591
81067	AKAM	2020-09-28	112.01	112.12	109.18	110.97	1547448	{}	\N	2020-10-12 16:40:21.063337
81068	AKAM	2020-09-29	110.64	111.96	110.5	111.21	768619	{}	\N	2020-10-12 16:40:21.074595
81069	AKAM	2020-09-30	111.54	112.21	110.15	110.54	990285	{}	\N	2020-10-12 16:40:21.082061
81070	AKAM	2020-10-01	110.95	111.44	108.91	109.93	1341957	{}	\N	2020-10-12 16:40:21.087779
81071	AKAM	2020-10-02	108.62	110	108.12	108.61	832101	{}	\N	2020-10-12 16:40:21.092902
81072	AKAM	2020-10-05	109.12	109.24	107.5	109.1	1212897	{}	\N	2020-10-12 16:40:21.098392
81073	AKAM	2020-10-06	108.98	110.29	108.03	108.16	658506	{}	\N	2020-10-12 16:40:21.103486
81074	AKAM	2020-10-07	108.2	109.87	108.2	109.68	995240	{}	\N	2020-10-12 16:40:21.107239
81075	AKAM	2020-10-08	110.15	111.66	110.15	111.08	721153	{}	\N	2020-10-12 16:40:21.11131
81076	AKAM	2020-10-09	111.42	112.87	111.305	112.53	631744	{}	\N	2020-10-12 16:40:21.115802
81077	AKAM	2020-10-12	114	114.75	113.43	114	343187	{}	\N	2020-10-12 16:40:21.120024
81078	AOK	2020-09-28	37.11	37.21	37.1072	37.19	133250	{}	\N	2020-10-12 16:40:24.814976
81079	AOK	2020-09-29	37.22	37.225	37.1404	37.18	246877	{}	\N	2020-10-12 16:40:24.827572
81080	AOK	2020-09-30	37.18	37.2838	37.15	37.17	71611	{}	\N	2020-10-12 16:40:24.835737
81081	AOK	2020-10-01	37.16	37.33	37.16	37.29	70635	{}	\N	2020-10-12 16:40:24.841898
81082	AOK	2020-10-02	37.12	37.17	37.03	37.17	186092	{}	\N	2020-10-12 16:40:24.847414
81083	AOK	2020-10-05	37.22	37.33	37.17	37.33	133035	{}	\N	2020-10-12 16:40:24.852517
81084	AOK	2020-10-06	37.33	37.41	37.17	37.2	93781	{}	\N	2020-10-12 16:40:24.858049
81085	AOK	2020-10-07	37.2	37.37	37.2	37.35	76746	{}	\N	2020-10-12 16:40:24.863205
81086	AOK	2020-10-08	37.45	37.51	37.3922	37.49	74152	{}	\N	2020-10-12 16:40:24.868426
81087	AOK	2020-10-09	37.56	37.59	37.49	37.57	122479	{}	\N	2020-10-12 16:40:24.872771
81088	AOK	2020-10-12	37.63	37.747	37.63	37.747	60049	{}	\N	2020-10-12 16:40:24.877101
81089	ASET	2020-09-28	26.63	26.7	26.57	26.64	5032	{}	\N	2020-10-12 16:40:27.488616
81090	ASET	2020-09-29	26.62	26.6204	26.46	26.46	1219	{}	\N	2020-10-12 16:40:27.496841
81091	ASET	2020-09-30	26.48	26.48	26.48	26.48	161	{}	\N	2020-10-12 16:40:27.50372
81092	ASET	2020-10-01	26.66	26.66	26.66	26.66	220	{}	\N	2020-10-12 16:40:27.509873
81093	ASET	2020-10-02	26.73	26.9373	26.73	26.9373	592	{}	\N	2020-10-12 16:40:27.515604
81094	ASET	2020-10-05	27.12	27.24	27.06	27.22	2191	{}	\N	2020-10-12 16:40:27.520977
81095	ASET	2020-10-06	27.27	27.27	26.97	26.99	1256	{}	\N	2020-10-12 16:40:27.52645
81096	ASET	2020-10-07	27.18	27.18	27.15	27.15	281	{}	\N	2020-10-12 16:40:27.531787
81097	ASET	2020-10-08	27.46	27.46	27.43	27.43	200	{}	\N	2020-10-12 16:40:27.537245
81098	ASET	2020-10-09	27.45	27.45	27.44	27.44	494	{}	\N	2020-10-12 16:40:27.542645
81099	BANC	2020-09-28	9.94	10.26	9.94	10.02	155290	{}	\N	2020-10-12 16:40:29.373566
81100	BANC	2020-09-29	10.03	10.19	9.89	10.15	475082	{}	\N	2020-10-12 16:40:29.385557
81101	BANC	2020-09-30	10.21	10.43	10.02	10.12	381972	{}	\N	2020-10-12 16:40:29.393782
81102	BANC	2020-10-01	10.02	10.3	9.96	10.29	368232	{}	\N	2020-10-12 16:40:29.400452
81103	BANC	2020-10-02	9.98	10.7	9.98	10.61	305496	{}	\N	2020-10-12 16:40:29.406395
81104	BANC	2020-10-05	10.83	11.12	10.77	11.07	294071	{}	\N	2020-10-12 16:40:29.411668
81105	BANC	2020-10-06	11.31	11.54	10.78	10.78	490643	{}	\N	2020-10-12 16:40:29.41688
81106	BANC	2020-10-07	10.95	11.23	10.95	11.16	538748	{}	\N	2020-10-12 16:40:29.422155
81107	BANC	2020-10-08	11.37	11.77	11.078	11.6	304842	{}	\N	2020-10-12 16:40:29.42752
81108	BANC	2020-10-09	11.84	11.85	11.415	11.43	273593	{}	\N	2020-10-12 16:40:29.432696
81109	BANC	2020-10-12	11.36	11.52	11.3	11.42	37627	{}	\N	2020-10-12 16:40:29.437735
81110	BCD	2020-09-28	23.06	23.0649	22.917	23.0649	924	{}	\N	2020-10-12 16:40:30.579444
81111	BCD	2020-09-29	22.9199	22.9199	22.8686	22.8686	298	{}	\N	2020-10-12 16:40:30.58874
81112	BCD	2020-09-30	22.88	23.22	22.88	23.18	2539	{}	\N	2020-10-12 16:40:30.596248
81113	BCD	2020-10-01	22.93	22.93	22.87	22.87	454	{}	\N	2020-10-12 16:40:30.603306
81114	BCD	2020-10-02	22.66	22.8826	22.66	22.875	1943	{}	\N	2020-10-12 16:40:30.609232
81115	BCD	2020-10-05	23.0952	23.0952	23.0952	23.0952	164	{}	\N	2020-10-12 16:40:30.615004
81116	BCD	2020-10-06	23.2802	23.2802	23.1102	23.1102	268	{}	\N	2020-10-12 16:40:30.620051
81117	BCD	2020-10-07	23.38	23.39	23.3331	23.39	1616	{}	\N	2020-10-12 16:40:30.625782
81118	BCD	2020-10-08	23.74	23.74	23.45	23.45	3024	{}	\N	2020-10-12 16:40:30.631207
81119	BCD	2020-10-09	23.64	23.755	23.64	23.755	694	{}	\N	2020-10-12 16:40:30.63618
81120	ZBH	2020-09-28	137.19	138.015	135.94	136.31	985154	{}	\N	2020-10-12 16:40:36.316747
81121	ZBH	2020-09-29	136.81	136.93	131.79	133.4	1651367	{}	\N	2020-10-12 16:40:36.327273
81122	ZBH	2020-09-30	133.8	137.93	133.59	136.14	1954350	{}	\N	2020-10-12 16:40:36.334602
81123	ZBH	2020-10-01	137.14	138.9	135.905	137.33	2179096	{}	\N	2020-10-12 16:40:36.340692
81124	ZBH	2020-10-02	134.43	139	134.26	137.69	1933975	{}	\N	2020-10-12 16:40:36.346021
81125	ZBH	2020-10-05	139.27	141.74	138.5731	141.59	1202523	{}	\N	2020-10-12 16:40:36.350913
81126	ZBH	2020-10-06	141.68	144.37	140.88	141.08	1470830	{}	\N	2020-10-12 16:40:36.355238
81127	ZBH	2020-10-07	141.95	145.56	141.95	144.83	1423995	{}	\N	2020-10-12 16:40:36.359626
81128	ZBH	2020-10-08	146.43	147.62	145.36	146.66	1598471	{}	\N	2020-10-12 16:40:36.364223
81129	ZBH	2020-10-09	147.94	149.8	147.19	147.99	2067834	{}	\N	2020-10-12 16:40:36.368798
81130	ZBH	2020-10-12	150.09	150.09	147.84	148.82	930636	{}	\N	2020-10-12 16:40:36.373374
81131	ZG	2020-09-28	102.64	102.81	99.9	100.98	403317	{}	\N	2020-10-12 16:40:38.342264
81132	ZG	2020-09-29	100.44	102.6209	100.06	101.39	464440	{}	\N	2020-10-12 16:40:38.356186
81133	ZG	2020-09-30	101.08	103.92	100.63	101.54	650347	{}	\N	2020-10-12 16:40:38.364489
81134	ZG	2020-10-01	104.17	108.07	102.7016	107.56	1140369	{}	\N	2020-10-12 16:40:38.371429
81135	ZG	2020-10-02	103.93	109.4	103.57	108	692781	{}	\N	2020-10-12 16:40:38.377492
81136	ZG	2020-10-05	109.49	112.2074	109.48	110.26	675424	{}	\N	2020-10-12 16:40:38.38275
81137	ZG	2020-10-06	110.1	110.69	104.22	105.28	700599	{}	\N	2020-10-12 16:40:38.387432
81138	ZG	2020-10-07	107.59	108.4999	104.6	105.79	860091	{}	\N	2020-10-12 16:40:38.394164
81139	ZG	2020-10-08	106.31	106.8	104.33	105.09	439636	{}	\N	2020-10-12 16:40:38.398469
81140	ZG	2020-10-09	105.42	106.31	103.78	104.64	756002	{}	\N	2020-10-12 16:40:38.403864
81141	ZG	2020-10-12	106.3	106.62	102.42	102.89	237061	{}	\N	2020-10-12 16:40:38.40849
81142	AAME	2020-09-28	2.189	2.2	2.07	2.07	5841	{}	\N	2020-10-12 16:40:39.399801
81143	AAME	2020-09-29	1.98	2.03	1.98	1.98	2053	{}	\N	2020-10-12 16:40:39.40923
81144	AAME	2020-09-30	1.99	2	1.88	2	2887	{}	\N	2020-10-12 16:40:39.414989
81145	AAME	2020-10-01	1.91	2.17	1.9	2.0367	883	{}	\N	2020-10-12 16:40:39.42136
81146	AAME	2020-10-02	1.94	2.01	1.94	1.96	2643	{}	\N	2020-10-12 16:40:39.427118
81147	AAME	2020-10-05	2.02	2.02	2.02	2.02	613	{}	\N	2020-10-12 16:40:39.431759
81148	AAME	2020-10-06	1.97	1.97	1.97	1.97	192	{}	\N	2020-10-12 16:40:39.437423
81149	AAME	2020-10-07	1.97	2	1.97	2	811	{}	\N	2020-10-12 16:40:39.44214
81150	AAME	2020-10-08	2	2	2	2	312	{}	\N	2020-10-12 16:40:39.446482
81151	AAME	2020-10-12	2.05	2.05	2.05	2.05	684	{}	\N	2020-10-12 16:40:39.451462
81152	AAON	2020-09-28	57.875	58.95	57.875	58.4	147278	{}	\N	2020-10-12 16:40:41.785598
81153	AAON	2020-09-29	58.34	59.76	58.34	59.59	169547	{}	\N	2020-10-12 16:40:41.796524
81154	AAON	2020-09-30	59.76	61.24	59.76	60.25	274444	{}	\N	2020-10-12 16:40:41.804071
81155	AAON	2020-10-01	60.845	61.17	59.3706	60.54	195760	{}	\N	2020-10-12 16:40:41.810831
81156	AAON	2020-10-02	59.59	61.19	59.47	60.93	131483	{}	\N	2020-10-12 16:40:41.816646
81157	AAON	2020-10-05	61.04	62.38	61.04	62.05	153619	{}	\N	2020-10-12 16:40:41.82151
81158	AAON	2020-10-06	62.26	63.74	61.6134	62.88	220729	{}	\N	2020-10-12 16:40:41.826359
81159	AAON	2020-10-07	63.11	64.29	62.61	62.85	224380	{}	\N	2020-10-12 16:40:41.831279
81160	AAON	2020-10-08	63.26	63.49	62.12	62.77	119426	{}	\N	2020-10-12 16:40:41.836277
81161	AAON	2020-10-09	63.19	64.32	62.87	63.56	129897	{}	\N	2020-10-12 16:40:41.841315
81162	AAON	2020-10-12	63.57	65.11	63.55	64.65	41717	{}	\N	2020-10-12 16:40:41.846373
81163	ABEV	2020-09-28	2.33	2.35	2.21	2.23	43512818	{}	\N	2020-10-12 16:40:42.772591
81164	ABEV	2020-09-29	2.22	2.28	2.19	2.19	66171084	{}	\N	2020-10-12 16:40:42.786314
81165	ABEV	2020-09-30	2.23	2.28	2.22	2.26	45625195	{}	\N	2020-10-12 16:40:42.795354
81166	ABEV	2020-10-01	2.23	2.275	2.215	2.26	35538085	{}	\N	2020-10-12 16:40:42.801657
81167	ABEV	2020-10-02	2.25	2.3	2.22	2.24	36037665	{}	\N	2020-10-12 16:40:42.807554
81168	ABEV	2020-10-05	2.24	2.32	2.24	2.32	32309124	{}	\N	2020-10-12 16:40:42.812864
81169	ABEV	2020-10-06	2.39	2.51	2.385	2.42	60032639	{}	\N	2020-10-12 16:40:42.817462
81170	ABEV	2020-10-07	2.42	2.5	2.415	2.43	58179198	{}	\N	2020-10-12 16:40:42.822317
81171	ABEV	2020-10-08	2.42	2.49	2.4	2.46	39090823	{}	\N	2020-10-12 16:40:42.826854
81172	ABEV	2020-10-09	2.48	2.54	2.45	2.47	43028992	{}	\N	2020-10-12 16:40:42.831243
81173	ABEV	2020-10-12	2.48	2.51	2.45	2.51	6128955	{}	\N	2020-10-12 16:40:42.835676
81174	AGD	2020-09-28	9.05	9.1	8.98	9.1	63337	{}	\N	2020-10-12 16:40:43.89519
81175	AGD	2020-09-29	9.1	9.1	8.98	9.06	89546	{}	\N	2020-10-12 16:40:43.905176
81176	AGD	2020-09-30	9.16	9.21	9.0295	9.21	147714	{}	\N	2020-10-12 16:40:43.913092
81177	AGD	2020-10-01	9.27	9.27	9.12	9.17	111880	{}	\N	2020-10-12 16:40:43.919487
81178	AGD	2020-10-02	9.05	9.11	9.03	9.1	82882	{}	\N	2020-10-12 16:40:43.924758
81179	AGD	2020-10-05	9.09	9.23	9.09	9.18	65262	{}	\N	2020-10-12 16:40:43.929866
81180	AGD	2020-10-06	9.22	9.31	9.17	9.21	136120	{}	\N	2020-10-12 16:40:43.934329
81181	AGD	2020-10-07	9.27	9.295	9.2029	9.25	76444	{}	\N	2020-10-12 16:40:43.939026
81182	AGD	2020-10-08	9.3	9.335	9.23	9.3	37202	{}	\N	2020-10-12 16:40:43.943663
81183	AGD	2020-10-09	9.29	9.34	9.28	9.31	73018	{}	\N	2020-10-12 16:40:43.948181
81184	AGD	2020-10-12	9.35	9.44	9.34	9.4372	69011	{}	\N	2020-10-12 16:40:43.952429
81185	AINV	2020-09-28	8.33	8.665	8.29	8.55	608895	{}	\N	2020-10-12 16:40:45.115231
81186	AINV	2020-09-29	8.51	8.52	8.25	8.27	391743	{}	\N	2020-10-12 16:40:45.126389
81187	AINV	2020-09-30	8.35	8.43	8.26	8.27	387807	{}	\N	2020-10-12 16:40:45.134625
81188	AINV	2020-10-01	8.31	8.4	8.22	8.37	314524	{}	\N	2020-10-12 16:40:45.14113
81189	AINV	2020-10-02	8.21	8.65	8.21	8.62	348127	{}	\N	2020-10-12 16:40:45.14705
81190	AINV	2020-10-05	8.65	8.89	8.6113	8.78	374339	{}	\N	2020-10-12 16:40:45.151751
81191	AINV	2020-10-06	8.82	8.87	8.57	8.61	269675	{}	\N	2020-10-12 16:40:45.156275
81192	AINV	2020-10-07	8.76	8.83	8.51	8.57	455211	{}	\N	2020-10-12 16:40:45.160865
81193	AINV	2020-10-08	8.58	8.73	8.575	8.73	411940	{}	\N	2020-10-12 16:40:45.165149
81194	AINV	2020-10-09	8.75	8.75	8.42	8.43	317533	{}	\N	2020-10-12 16:40:45.169567
81195	AINV	2020-10-12	8.44	8.45	8.32	8.32	166724	{}	\N	2020-10-12 16:40:45.174163
81196	AIV	2020-09-28	34.5	35.13	34.3	34.54	1046016	{}	\N	2020-10-12 16:40:46.421403
81197	AIV	2020-09-29	34.37	34.63	33.14	33.36	2029590	{}	\N	2020-10-12 16:40:46.43475
81198	AIV	2020-09-30	33.53	33.97	33.4	33.72	2393912	{}	\N	2020-10-12 16:40:46.442394
81199	AIV	2020-10-01	33.88	35.21	33.75	35.19	1813800	{}	\N	2020-10-12 16:40:46.448908
81200	AIV	2020-10-02	34.65	36.485	34.37	36.25	2459807	{}	\N	2020-10-12 16:40:46.454464
81201	AIV	2020-10-05	36.47	36.63	35.5	36.5	4102659	{}	\N	2020-10-12 16:40:46.459348
81202	AIV	2020-10-06	36.88	37.2	36.24	36.75	2637217	{}	\N	2020-10-12 16:40:46.464213
81203	AIV	2020-10-07	36.95	36.95	36.2	36.29	4111139	{}	\N	2020-10-12 16:40:46.469186
81204	AIV	2020-10-08	36.32	36.92	36.17	36.74	3615327	{}	\N	2020-10-12 16:40:46.474003
81205	AIV	2020-10-09	36.95	36.95	36.21	36.76	1670828	{}	\N	2020-10-12 16:40:46.478778
81206	AIV	2020-10-12	36.63	36.85	36.3	36.55	447326	{}	\N	2020-10-12 16:40:46.483609
81207	ALLT	2020-09-28	8.74	9	8.74	8.96	108247	{}	\N	2020-10-12 16:40:47.427786
81208	ALLT	2020-09-29	8.91	9.31	8.91	9.25	115265	{}	\N	2020-10-12 16:40:47.43954
81209	ALLT	2020-09-30	9.25	9.25	8.96	9.12	123576	{}	\N	2020-10-12 16:40:47.44871
81210	ALLT	2020-10-01	9.14	9.2	8.96	9.16	118311	{}	\N	2020-10-12 16:40:47.455636
81211	ALLT	2020-10-02	9.025	9.08	8.88	8.91	176670	{}	\N	2020-10-12 16:40:47.461603
81212	ALLT	2020-10-05	9.06	9.52	9.02	9.49	162581	{}	\N	2020-10-12 16:40:47.466676
81213	ALLT	2020-10-06	9.49	9.67	9.21	9.25	188952	{}	\N	2020-10-12 16:40:47.47185
81214	ALLT	2020-10-07	9.32	9.47	9.31	9.41	121387	{}	\N	2020-10-12 16:40:47.477271
81215	ALLT	2020-10-08	9.62	9.7133	9.44	9.54	95347	{}	\N	2020-10-12 16:40:47.48232
81216	ALLT	2020-10-09	9.65	9.86	9.56	9.84	86876	{}	\N	2020-10-12 16:40:47.487199
81217	ALLT	2020-10-12	10.05	10.29	10.05	10.09	70344	{}	\N	2020-10-12 16:40:47.492712
81218	AMRH	2020-09-28	1.08	1.1699	1.07	1.11	79256	{}	\N	2020-10-12 16:40:48.411496
81219	AMRH	2020-09-29	1.06	1.18	1.06	1.13	62947	{}	\N	2020-10-12 16:40:48.421807
81220	AMRH	2020-09-30	1.16	1.4748	1.13	1.38	4946090	{}	\N	2020-10-12 16:40:48.430242
81221	AMRH	2020-10-01	1.64	1.65	1.25	1.32	3588804	{}	\N	2020-10-12 16:40:48.436727
81222	AMRH	2020-10-02	1.2	1.31	1.2	1.31	438220	{}	\N	2020-10-12 16:40:48.442943
81223	AMRH	2020-10-05	1.26	1.2824	1.12	1.22	377598	{}	\N	2020-10-12 16:40:48.449105
81224	AMRH	2020-10-06	1.19	1.3501	1.19	1.19	304608	{}	\N	2020-10-12 16:40:48.453591
81225	AMRH	2020-10-07	1.22	1.344	1.19	1.3	334486	{}	\N	2020-10-12 16:40:48.458021
81226	AMRH	2020-10-08	1.3	1.35	1.21	1.35	251937	{}	\N	2020-10-12 16:40:48.463256
81227	AMRH	2020-10-09	1.4	1.44	1.3	1.35	313230	{}	\N	2020-10-12 16:40:48.468261
81228	AMRH	2020-10-12	1.38	1.4	1.3	1.32	89874	{}	\N	2020-10-12 16:40:48.472784
81229	YY	2020-09-28	77.57	78.9	76.3	78	617151	{}	\N	2020-10-12 16:40:49.382228
81230	YY	2020-09-29	78.77	78.77	76.77	77.9	296619	{}	\N	2020-10-12 16:40:49.39411
81231	YY	2020-09-30	76.9	81.35	76.54	80.67	689834	{}	\N	2020-10-12 16:40:49.401311
81232	YY	2020-10-01	81.4	83.25	81.235	83.12	354725	{}	\N	2020-10-12 16:40:49.406972
81233	YY	2020-10-02	81	82.68	80.86	81.53	281446	{}	\N	2020-10-12 16:40:49.412149
81234	YY	2020-10-05	82.97	82.97	81.0923	81.99	439906	{}	\N	2020-10-12 16:40:49.417007
81235	YY	2020-10-06	82.5	83.75	81.59	82.22	360180	{}	\N	2020-10-12 16:40:49.421519
81236	YY	2020-10-07	82.53	83	81.72	82.78	429113	{}	\N	2020-10-12 16:40:49.425554
81237	YY	2020-10-08	83.54	84.42	80.65	81.32	559803	{}	\N	2020-10-12 16:40:49.43015
81238	YY	2020-10-09	81.63	83.38	81.41	82.67	328966	{}	\N	2020-10-12 16:40:49.435532
81239	YY	2020-10-12	83.99	85.5	80.86	83.02	556288	{}	\N	2020-10-12 16:40:49.439967
81240	ZBRA	2020-09-28	261.06	264.025	259.05	260.56	266073	{}	\N	2020-10-12 16:40:53.151
81241	ZBRA	2020-09-29	262.21	263.09	257.63	258.92	188108	{}	\N	2020-10-12 16:40:53.163694
81242	ZBRA	2020-09-30	258.07	259.385	250.45	252.46	580106	{}	\N	2020-10-12 16:40:53.172014
81243	ZBRA	2020-10-01	256.81	259.15	254.34	258.48	447171	{}	\N	2020-10-12 16:40:53.178421
81244	ZBRA	2020-10-02	252.52	259.21	252.52	258.81	340001	{}	\N	2020-10-12 16:40:53.184752
81245	ZBRA	2020-10-05	261.09	268.47	260.4	268.01	369281	{}	\N	2020-10-12 16:40:53.189877
81246	ZBRA	2020-10-06	267.14	273.49	267.14	269.97	355411	{}	\N	2020-10-12 16:40:53.19519
81247	ZBRA	2020-10-07	274.01	278.305	272.83	276.58	270753	{}	\N	2020-10-12 16:40:53.200899
81248	ZBRA	2020-10-08	277.73	281.4	277.73	279.175	218698	{}	\N	2020-10-12 16:40:53.206222
81249	ZBRA	2020-10-09	281.63	284.25	280.99	284	212198	{}	\N	2020-10-12 16:40:53.211615
81250	ZBRA	2020-10-12	287.24	289.3	286.245	286.8	92400	{}	\N	2020-10-12 16:40:53.216965
81251	ZNH	2020-09-28	26.81	27.475	26.81	27.2	27544	{}	\N	2020-10-12 16:40:58.16329
81252	ZNH	2020-09-29	26.85	26.85	26.65	26.73	33916	{}	\N	2020-10-12 16:40:58.174906
81253	ZNH	2020-09-30	27.09	27.4699	27.05	27.19	61067	{}	\N	2020-10-12 16:40:58.182399
81254	ZNH	2020-10-01	27.5	27.69	27.3301	27.57	31660	{}	\N	2020-10-12 16:40:58.188816
81255	ZNH	2020-10-02	26.95	27.55	26.83	27.22	36581	{}	\N	2020-10-12 16:40:58.194935
81256	ZNH	2020-10-05	26.99	27.15	26.875	27.07	30376	{}	\N	2020-10-12 16:40:58.20006
81257	ZNH	2020-10-06	27.54	28.1265	27.54	27.62	47104	{}	\N	2020-10-12 16:40:58.205219
81258	ZNH	2020-10-07	27.69	27.96	27.48	27.6	44184	{}	\N	2020-10-12 16:40:58.2102
81259	ZNH	2020-10-08	28.15	28.48	28.13	28.38	34026	{}	\N	2020-10-12 16:40:58.215387
81260	ZNH	2020-10-09	27.78	28.11	27.68	27.95	24498	{}	\N	2020-10-12 16:40:58.220643
81261	ZNH	2020-10-12	27.31	27.6	27.121	27.465	22247	{}	\N	2020-10-12 16:40:58.225993
81262	ZTR	2020-09-28	7.89	7.945	7.85	7.86	123910	{}	\N	2020-10-12 16:40:59.812099
81263	ZTR	2020-09-29	7.85	7.88	7.81	7.82	199242	{}	\N	2020-10-12 16:40:59.823536
81264	ZTR	2020-09-30	7.81	7.89	7.81	7.83	113386	{}	\N	2020-10-12 16:40:59.831454
81265	ZTR	2020-10-01	7.82	7.89	7.82	7.84	161324	{}	\N	2020-10-12 16:40:59.837659
81266	ZTR	2020-10-02	7.81	7.88	7.79	7.86	200440	{}	\N	2020-10-12 16:40:59.843619
81267	ZTR	2020-10-05	7.86	7.95	7.86	7.94	208248	{}	\N	2020-10-12 16:40:59.848328
81268	ZTR	2020-10-06	7.97	8.0884	7.95	7.99	277090	{}	\N	2020-10-12 16:40:59.852641
81269	ZTR	2020-10-07	8.1	8.1	8.03	8.04	254260	{}	\N	2020-10-12 16:40:59.857388
81270	ZTR	2020-10-08	8.1	8.1	8.02	8.04	495338	{}	\N	2020-10-12 16:40:59.861822
81271	ZTR	2020-10-09	7.98	8	7.91	7.91	444444	{}	\N	2020-10-12 16:40:59.866254
81272	ZTR	2020-10-12	7.91	7.98	7.91	7.97	194786	{}	\N	2020-10-12 16:40:59.870818
81273	AAXJ	2020-09-28	76.02	76.14	75.69	76.06	349811	{}	\N	2020-10-12 16:41:01.618184
81274	AAXJ	2020-09-29	75.83	76.22	75.75	75.96	345613	{}	\N	2020-10-12 16:41:01.630063
81275	AAXJ	2020-09-30	76.5	77.39	76.49	77.17	759100	{}	\N	2020-10-12 16:41:01.637114
81276	AAXJ	2020-10-01	77.94	78.16	77.49	78.05	1167004	{}	\N	2020-10-12 16:41:01.643387
81277	AAXJ	2020-10-02	76.84	77.82	76.84	77.19	689892	{}	\N	2020-10-12 16:41:01.648886
81278	AAXJ	2020-10-05	77.73	78.06	77.65	78.06	494954	{}	\N	2020-10-12 16:41:01.653887
81279	AAXJ	2020-10-06	78.23	78.81	78.07	78.25	1273501	{}	\N	2020-10-12 16:41:01.659357
81280	AAXJ	2020-10-07	79.05	79.42	78.87	79.29	1479277	{}	\N	2020-10-12 16:41:01.664562
81281	AAXJ	2020-10-08	79.5241	79.77	79.33	79.65	1041992	{}	\N	2020-10-12 16:41:01.669352
81282	AAXJ	2020-10-09	79.73	80.3	79.615	80.12	656020	{}	\N	2020-10-12 16:41:01.674487
81283	AAXJ	2020-10-12	80.57	80.985	80.43	80.955	290168	{}	\N	2020-10-12 16:41:01.680233
81284	AEL	2020-09-28	20.56	21.63	20.56	21.18	560386	{}	\N	2020-10-12 16:41:02.795626
81285	AEL	2020-09-29	21.51	21.99	21.24	21.89	1222709	{}	\N	2020-10-12 16:41:02.807694
81286	AEL	2020-09-30	21.88	22.2995	21.69	21.99	1239464	{}	\N	2020-10-12 16:41:02.816531
81287	AEL	2020-10-01	33.99	34.25	29.6	31.68	16278787	{}	\N	2020-10-12 16:41:02.8238
81288	AEL	2020-10-02	31.28	32	30.63	31.75	5738627	{}	\N	2020-10-12 16:41:02.830182
81289	AEL	2020-10-05	31.85	32.11	31.11	31.52	3892159	{}	\N	2020-10-12 16:41:02.835463
81290	AEL	2020-10-06	31.84	32.56	30.8	31.11	2759461	{}	\N	2020-10-12 16:41:02.84118
81291	AEL	2020-10-07	31.98	32.02	30.54	31.01	2703709	{}	\N	2020-10-12 16:41:02.846356
81292	AEL	2020-10-08	31.19	31.44	30.69	30.96	1465029	{}	\N	2020-10-12 16:41:02.85131
81293	AEL	2020-10-09	31.35	31.66	31.04	31.17	1551390	{}	\N	2020-10-12 16:41:02.856287
81294	AEL	2020-10-12	31.3	31.76	31.3	31.54	577857	{}	\N	2020-10-12 16:41:02.861602
81295	AM	2020-09-28	5.39	5.55	5.3	5.5	3690516	{}	\N	2020-10-12 16:41:04.293021
81296	AM	2020-09-29	5.52	5.58	5.39	5.5	6429582	{}	\N	2020-10-12 16:41:04.304558
81297	AM	2020-09-30	5.48	5.6	5.34	5.37	6056440	{}	\N	2020-10-12 16:41:04.312578
81298	AM	2020-10-01	5.38	5.41	5.13	5.19	9299887	{}	\N	2020-10-12 16:41:04.319841
81299	AM	2020-10-02	5.09	5.38	5.09	5.33	6579951	{}	\N	2020-10-12 16:41:04.326668
81300	AM	2020-10-05	5.42	5.65	5.41	5.62	5943355	{}	\N	2020-10-12 16:41:04.331586
81301	AM	2020-10-06	5.7	5.795	5.5	5.53	7617953	{}	\N	2020-10-12 16:41:04.336142
81302	AM	2020-10-07	5.59	5.8	5.53	5.72	8952206	{}	\N	2020-10-12 16:41:04.340488
81303	AM	2020-10-08	5.78	6.0285	5.725	5.95	8656511	{}	\N	2020-10-12 16:41:04.344858
81304	AM	2020-10-09	6.06	6.17	5.9	5.93	4987099	{}	\N	2020-10-12 16:41:04.349295
81305	AM	2020-10-12	6	6.04	5.95	6.01	2286688	{}	\N	2020-10-12 16:41:04.353644
81306	ANAT	2020-09-28	68.56	69.839	68.3101	69.05	37875	{}	\N	2020-10-12 16:41:06.585252
81307	ANAT	2020-09-29	69.25	69.25	67.33	68.15	30510	{}	\N	2020-10-12 16:41:06.595561
81308	ANAT	2020-09-30	68.15	68.4	67.21	67.53	51259	{}	\N	2020-10-12 16:41:06.603034
81309	ANAT	2020-10-01	67.8	68.39	66.5	68.27	57772	{}	\N	2020-10-12 16:41:06.609309
81310	ANAT	2020-10-02	67.54	71.11	67.36	70.44	36268	{}	\N	2020-10-12 16:41:06.615659
81311	ANAT	2020-10-05	70.75	72.98	70.65	72.4	43131	{}	\N	2020-10-12 16:41:06.620888
81312	ANAT	2020-10-06	73.04	73.7366	70.61	71.1	93375	{}	\N	2020-10-12 16:41:06.625737
81313	ANAT	2020-10-07	71.39	73.35	71.14	73.22	72540	{}	\N	2020-10-12 16:41:06.631109
81314	ANAT	2020-10-08	73.56	74.94	73.03	74.38	35558	{}	\N	2020-10-12 16:41:06.636449
81315	ANAT	2020-10-09	74.42	74.42	73.1	73.11	29090	{}	\N	2020-10-12 16:41:06.641668
81316	ANAT	2020-10-12	73	75.4705	73	73.86	4544	{}	\N	2020-10-12 16:41:06.647371
81317	APTV	2020-09-28	88.41	90.88	88.35	90.03	1241270	{}	\N	2020-10-12 16:41:09.030882
81318	APTV	2020-09-29	89.61	92.29	89.52	91.92	3784166	{}	\N	2020-10-12 16:41:09.042464
81319	APTV	2020-09-30	92.18	93.28	90.9	91.68	4381775	{}	\N	2020-10-12 16:41:09.049579
81320	APTV	2020-10-01	92.75	94.9653	91.68	93.96	3978988	{}	\N	2020-10-12 16:41:09.055452
81321	APTV	2020-10-02	91.26	94.495	91.24	93.56	2027063	{}	\N	2020-10-12 16:41:09.060648
81322	APTV	2020-10-05	95	97.6261	94.83	97.2	2187251	{}	\N	2020-10-12 16:41:09.065773
81323	APTV	2020-10-06	99.49	99.54	94.06	94.48	5165561	{}	\N	2020-10-12 16:41:09.070827
81324	APTV	2020-10-07	96.14	99.48	95.93	99.19	3119702	{}	\N	2020-10-12 16:41:09.075606
81325	APTV	2020-10-08	100	100.82	97.54	97.97	3353946	{}	\N	2020-10-12 16:41:09.080286
81326	APTV	2020-10-09	99.29	99.29	97.21	98.06	2835866	{}	\N	2020-10-12 16:41:09.085465
81327	APTV	2020-10-12	98.71	99.5	98.06	98.78	1470581	{}	\N	2020-10-12 16:41:09.090271
81328	ARDC	2020-09-28	12.69	12.705	12.645	12.66	129286	{}	\N	2020-10-12 16:41:10.856674
81329	ARDC	2020-09-29	12.66	12.755	12.66	12.7	361336	{}	\N	2020-10-12 16:41:10.8693
81330	ARDC	2020-09-30	12.77	12.77	12.63	12.68	1021512	{}	\N	2020-10-12 16:41:10.876667
81331	ARDC	2020-10-01	12.71	12.8	12.71	12.71	230188	{}	\N	2020-10-12 16:41:10.883712
81332	ARDC	2020-10-02	12.69	12.76	12.65	12.74	94157	{}	\N	2020-10-12 16:41:10.89064
81333	ARDC	2020-10-05	12.75	12.84	12.74	12.84	160673	{}	\N	2020-10-12 16:41:10.896038
81334	ARDC	2020-10-06	12.84	12.96	12.8246	12.91	177557	{}	\N	2020-10-12 16:41:10.901432
81335	ARDC	2020-10-07	12.92	12.97	12.84	12.85	214126	{}	\N	2020-10-12 16:41:10.906749
81336	ARDC	2020-10-08	12.88	12.95	12.75	12.76	286359	{}	\N	2020-10-12 16:41:10.912027
81337	ARDC	2020-10-09	12.81	12.87	12.8	12.86	275258	{}	\N	2020-10-12 16:41:10.917532
81338	ARDC	2020-10-12	12.89	12.95	12.88	12.9136	105743	{}	\N	2020-10-12 16:41:10.922683
81339	ATR	2020-09-28	113.17	114.22	112.04	112.36	230068	{}	\N	2020-10-12 16:41:15.488532
81340	ATR	2020-09-29	112.06	113.35	111.745	112.62	467118	{}	\N	2020-10-12 16:41:15.500233
81341	ATR	2020-09-30	113.27	113.935	112.5	113.2	394337	{}	\N	2020-10-12 16:41:15.507078
81342	ATR	2020-10-01	113.45	114.97	113.14	113.7	322441	{}	\N	2020-10-12 16:41:15.512814
81343	ATR	2020-10-02	112.31	114.175	112.31	112.85	323761	{}	\N	2020-10-12 16:41:15.518344
81344	ATR	2020-10-05	113.36	114.93	113.21	114.55	292404	{}	\N	2020-10-12 16:41:15.523295
81345	ATR	2020-10-06	115.01	115.9	113.92	114.56	447854	{}	\N	2020-10-12 16:41:15.528005
81346	ATR	2020-10-07	115.09	116.17	114.5	114.98	340910	{}	\N	2020-10-12 16:41:15.532521
81347	ATR	2020-10-08	115.8	116.79	115.52	116.52	272822	{}	\N	2020-10-12 16:41:15.53733
81348	ATR	2020-10-09	118.12	118.39	116.185	116.81	295053	{}	\N	2020-10-12 16:41:15.542881
81349	ATR	2020-10-12	117.05	118.825	116.7	118.5667	59570	{}	\N	2020-10-12 16:41:15.548403
81350	BBC	2020-09-28	39.53	39.53	38.38	38.67	16742	{}	\N	2020-10-12 16:41:16.628777
81351	BBC	2020-09-29	38.57	39.0632	38.41	38.95	8073	{}	\N	2020-10-12 16:41:16.640532
81352	BBC	2020-09-30	39.36	39.44	38.75	38.981	6634	{}	\N	2020-10-12 16:41:16.648285
81353	BBC	2020-10-01	38.87	39.45	38.78	39.45	11225	{}	\N	2020-10-12 16:41:16.654048
81354	BBC	2020-10-02	38.83	39.37	38.49	38.54	8377	{}	\N	2020-10-12 16:41:16.659912
81355	BBC	2020-10-05	39.32	41.165	39.32	41.09	16857	{}	\N	2020-10-12 16:41:16.664708
81356	BBC	2020-10-06	41.28	41.8599	40.8	41.2028	15739	{}	\N	2020-10-12 16:41:16.669397
81357	BBC	2020-10-07	41.29	42.4	41.29	42.33	10678	{}	\N	2020-10-12 16:41:16.67425
81358	BBC	2020-10-08	42.25	42.74	42.19	42.34	15910	{}	\N	2020-10-12 16:41:16.678993
81359	BBC	2020-10-09	42.88	42.88	42.29	42.74	13271	{}	\N	2020-10-12 16:41:16.683707
81360	BBC	2020-10-12	42.68	42.86	42.53	42.805	9252	{}	\N	2020-10-12 16:41:16.689609
81361	FBNC	2020-09-28	20.73	21.53	20.73	21.3	197475	{}	\N	2020-10-12 16:41:19.163228
81362	FBNC	2020-09-29	21.11	21.11	20.3149	20.96	140142	{}	\N	2020-10-12 16:41:19.174677
81363	FBNC	2020-09-30	21.12	21.41	20.76	20.93	124920	{}	\N	2020-10-12 16:41:19.18159
81364	FBNC	2020-10-01	20.83	21.13	20.59	21.1	109163	{}	\N	2020-10-12 16:41:19.187792
81365	FBNC	2020-10-02	20.44	21.82	20.44	21.76	113762	{}	\N	2020-10-12 16:41:19.194081
81366	FBNC	2020-10-05	21.96	22.35	21.67	22.15	120515	{}	\N	2020-10-12 16:41:19.19912
81367	FBNC	2020-10-06	22.49	23.11	21.785	22.23	97676	{}	\N	2020-10-12 16:41:19.204017
81368	FBNC	2020-10-07	22.37	22.93	22.33	22.85	175038	{}	\N	2020-10-12 16:41:19.209177
81369	FBNC	2020-10-08	22.9	23.34	22.77	23.07	107426	{}	\N	2020-10-12 16:41:19.214461
81370	FBNC	2020-10-09	23.36	23.36	22.61	22.81	67756	{}	\N	2020-10-12 16:41:19.219646
81371	FBNC	2020-10-12	22.8	23	22.12	22.87	18104	{}	\N	2020-10-12 16:41:19.225203
81372	ZION	2020-09-28	28.99	29.64	28.83	29.38	1445547	{}	\N	2020-10-12 16:41:20.702921
81373	ZION	2020-09-29	29.2202	29.56	28.54	28.97	1386014	{}	\N	2020-10-12 16:41:20.71647
81374	ZION	2020-09-30	29.08	29.57	28.87	29.22	1840457	{}	\N	2020-10-12 16:41:20.724226
81375	ZION	2020-10-01	29.22	29.57	28.79	29.12	1472618	{}	\N	2020-10-12 16:41:20.736015
81376	ZION	2020-10-02	28.76	29.96	28.57	29.85	3087429	{}	\N	2020-10-12 16:41:20.741413
81377	ZION	2020-10-05	30.39	31.49	30.39	31.28	1510320	{}	\N	2020-10-12 16:41:20.746792
81378	ZION	2020-10-06	31.66	32.34	30.81	30.9	1548125	{}	\N	2020-10-12 16:41:20.752387
81379	ZION	2020-10-07	31.27	32.26	31.27	32.15	1278141	{}	\N	2020-10-12 16:41:20.757305
81380	ZION	2020-10-08	32.23	32.33	31.63	32.14	980020	{}	\N	2020-10-12 16:41:20.762428
81381	ZION	2020-10-09	32.32	32.5	31.47	31.575	874408	{}	\N	2020-10-12 16:41:20.767283
81382	ZION	2020-10-12	31.54	31.79	31.42	31.63	281628	{}	\N	2020-10-12 16:41:20.771484
81383	ZMLP	2020-09-28	48.82	50.1993	48.82	50.1993	11678	{}	\N	2020-10-12 16:41:23.032439
81384	ZMLP	2020-09-29	49.84	50.9544	49.71	50.9544	9555	{}	\N	2020-10-12 16:41:23.043701
81385	ZMLP	2020-09-30	50.95	51.47	50.39	50.4172	4341	{}	\N	2020-10-12 16:41:23.051475
81386	ZMLP	2020-10-01	50	50.53	49.68	50.04	152640	{}	\N	2020-10-12 16:41:23.057221
81387	ZMLP	2020-10-02	48.65	50.7993	48.65	50.645	3071	{}	\N	2020-10-12 16:41:23.063168
81388	ZMLP	2020-10-05	49.86	52.32	49.86	52.32	22316	{}	\N	2020-10-12 16:41:23.067948
81389	ZMLP	2020-10-06	51.98	52.5	51.39	51.7	8262	{}	\N	2020-10-12 16:41:23.072736
81390	ZMLP	2020-10-07	51.33	51.99	51.33	51.76	22191	{}	\N	2020-10-12 16:41:23.080447
81391	ZMLP	2020-10-08	50.9	53.585	50.9	53.585	10048	{}	\N	2020-10-12 16:41:23.085576
81392	ZMLP	2020-10-09	53.73	53.82	53.24	53.5	17348	{}	\N	2020-10-12 16:41:23.091494
81393	ZMLP	2020-10-12	53.5	53.5	53.24	53.32	11940	{}	\N	2020-10-12 16:41:23.097017
81394	ZTS	2020-09-28	162.16	164.3424	161.16	161.32	2313888	{}	\N	2020-10-12 16:41:24.052606
81395	ZTS	2020-09-29	161.62	163.41	161.4419	162.48	2083426	{}	\N	2020-10-12 16:41:24.063911
81396	ZTS	2020-09-30	162.92	166.79	162.75	165.37	3676600	{}	\N	2020-10-12 16:41:24.071054
81397	ZTS	2020-10-01	166.23	166.78	163.1001	164.03	2121119	{}	\N	2020-10-12 16:41:24.07682
81398	ZTS	2020-10-02	162.38	164.71	162	163.55	2022966	{}	\N	2020-10-12 16:41:24.082726
81399	ZTS	2020-10-05	164.11	165.07	162.53	163.45	2498153	{}	\N	2020-10-12 16:41:24.087877
81400	ZTS	2020-10-06	164.5	164.5	160.84	161.52	2315988	{}	\N	2020-10-12 16:41:24.092386
81401	ZTS	2020-10-07	161.7	163.49	158.95	159.91	7147222	{}	\N	2020-10-12 16:41:24.09704
81402	ZTS	2020-10-08	161.93	163.555	159.89	163.17	7326641	{}	\N	2020-10-12 16:41:24.101794
81403	ZTS	2020-10-09	163.98	165.64	162.86	165.43	3230866	{}	\N	2020-10-12 16:41:24.10638
81404	ZTS	2020-10-12	167.08	168.84	166.19	168.69	1403775	{}	\N	2020-10-12 16:41:24.110752
81405	AAWW	2020-09-28	60.65	63.04	60.55	62.36	396155	{}	\N	2020-10-12 16:41:27.189817
81406	AAWW	2020-09-29	62.02	63.31	61.01	61.28	479129	{}	\N	2020-10-12 16:41:27.200227
81407	AAWW	2020-09-30	61.43	62.3049	60.41	60.9	475194	{}	\N	2020-10-12 16:41:27.207566
81408	AAWW	2020-10-01	61.15	63	61.15	62.8	460396	{}	\N	2020-10-12 16:41:27.213232
81409	AAWW	2020-10-02	61.49	64.2	61.05	63.89	275752	{}	\N	2020-10-12 16:41:27.218674
81410	AAWW	2020-10-05	64.44	66.35	63.7922	65.4	794222	{}	\N	2020-10-12 16:41:27.22378
81411	AAWW	2020-10-06	65.99	66.73	63.56	63.96	514300	{}	\N	2020-10-12 16:41:27.228476
81412	AAWW	2020-10-07	65.245	67.16	65.07	67.02	597981	{}	\N	2020-10-12 16:41:27.232858
81413	AAWW	2020-10-08	67	67.63	66.415	67.43	558460	{}	\N	2020-10-12 16:41:27.23755
81414	AAWW	2020-10-09	68.12	68.63	66.665	67.73	393332	{}	\N	2020-10-12 16:41:27.241832
81415	AAWW	2020-10-12	68.23	69.08	67.4425	67.635	115471	{}	\N	2020-10-12 16:41:27.246393
81416	AKTX	2020-09-28	1.69	1.73	1.66	1.7	37144	{}	\N	2020-10-12 16:41:29.120656
81417	AKTX	2020-09-29	1.65	1.69	1.64	1.68	38036	{}	\N	2020-10-12 16:41:29.131753
81418	AKTX	2020-09-30	1.68	1.75	1.68	1.74	85059	{}	\N	2020-10-12 16:41:29.139449
81419	AKTX	2020-10-01	1.7	1.76	1.64	1.75	226307	{}	\N	2020-10-12 16:41:29.144708
81420	AKTX	2020-10-02	1.69	1.75	1.64	1.73	109635	{}	\N	2020-10-12 16:41:29.149876
81421	AKTX	2020-10-05	1.71	1.75	1.705	1.73	63271	{}	\N	2020-10-12 16:41:29.154878
81422	AKTX	2020-10-06	1.74	1.955	1.7022	1.85	493842	{}	\N	2020-10-12 16:41:29.159586
81423	AKTX	2020-10-07	1.85	1.92	1.78	1.88	150039	{}	\N	2020-10-12 16:41:29.164094
81424	AKTX	2020-10-08	1.85	1.925	1.77	1.77	200339	{}	\N	2020-10-12 16:41:29.168691
81425	AKTX	2020-10-09	1.85	1.94	1.84	1.89	224232	{}	\N	2020-10-12 16:41:29.173342
81426	AKTX	2020-10-12	1.92	1.93	1.82	1.8586	79359	{}	\N	2020-10-12 16:41:29.17873
81427	AOD	2020-09-28	7.85	7.92	7.85	7.87	269795	{}	\N	2020-10-12 16:41:30.428726
81428	AOD	2020-09-29	7.9	7.9	7.8242	7.9	396218	{}	\N	2020-10-12 16:41:30.439733
81429	AOD	2020-09-30	7.94	7.97	7.9001	7.94	681573	{}	\N	2020-10-12 16:41:30.447029
81430	AOD	2020-10-01	7.96	7.97	7.89	7.89	553986	{}	\N	2020-10-12 16:41:30.452972
81431	AOD	2020-10-02	7.84	7.9	7.83	7.89	409407	{}	\N	2020-10-12 16:41:30.458838
81432	AOD	2020-10-05	7.93	8.02	7.9201	8.02	249871	{}	\N	2020-10-12 16:41:30.464003
81433	AOD	2020-10-06	8.04	8.085	7.95	7.99	530613	{}	\N	2020-10-12 16:41:30.469261
81434	AOD	2020-10-07	8.05	8.085	8.025	8.07	420188	{}	\N	2020-10-12 16:41:30.474904
81435	AOD	2020-10-08	8.1	8.12	8.07	8.1	402656	{}	\N	2020-10-12 16:41:30.480251
81436	AOD	2020-10-09	8.15	8.15	8.09	8.1	469675	{}	\N	2020-10-12 16:41:30.485383
81437	AOD	2020-10-12	8.14	8.18	8.13	8.18	215974	{}	\N	2020-10-12 16:41:30.490799
81438	AWI	2020-09-28	69.08	71.15	69.08	70	390939	{}	\N	2020-10-12 16:41:32.25446
81439	AWI	2020-09-29	69.9	70.72	69.28	70.3	890873	{}	\N	2020-10-12 16:41:32.266964
81440	AWI	2020-09-30	69.55	70.37	68.225	68.81	1237242	{}	\N	2020-10-12 16:41:32.274841
81441	AWI	2020-10-01	69.39	71.065	68.68	70.93	895495	{}	\N	2020-10-12 16:41:32.281399
81442	AWI	2020-10-02	69.78	71.16	69.39	70.47	637994	{}	\N	2020-10-12 16:41:32.28774
81443	AWI	2020-10-05	71.13	72.11	70.79	70.93	835961	{}	\N	2020-10-12 16:41:32.293225
81444	AWI	2020-10-06	71.21	71.685	69.88	70.26	951788	{}	\N	2020-10-12 16:41:32.298615
81445	AWI	2020-10-07	70.81	71.08	69.11	69.31	768602	{}	\N	2020-10-12 16:41:32.303586
81446	AWI	2020-10-08	69.82	70.76	69.75	70.16	593012	{}	\N	2020-10-12 16:41:32.309191
81447	AWI	2020-10-09	70.45	71.07	69.82	69.85	269421	{}	\N	2020-10-12 16:41:32.314527
81448	AWI	2020-10-12	70.02	70.66	69.89	70.235	122517	{}	\N	2020-10-12 16:41:32.320075
81449	BBH	2020-09-28	161.06	161.06	158.91	160.35	7979	{}	\N	2020-10-12 16:41:34.042787
81450	BBH	2020-09-29	159.71	160.815	159.6801	160.55	6079	{}	\N	2020-10-12 16:41:34.054262
81451	BBH	2020-09-30	161.9218	163.04	161.06	162.08	14243	{}	\N	2020-10-12 16:41:34.060958
81452	BBH	2020-10-01	163.37	163.37	161.8066	163.03	7939	{}	\N	2020-10-12 16:41:34.066713
81453	BBH	2020-10-02	160.54	162.92	159.39	159.73	13425	{}	\N	2020-10-12 16:41:34.071971
81454	BBH	2020-10-05	161.89	165.669	161.82	165.53	26854	{}	\N	2020-10-12 16:41:34.076923
81455	BBH	2020-10-06	165.77	166.32	163.21	163.42	15899	{}	\N	2020-10-12 16:41:34.081423
81456	BBH	2020-10-07	163.78	167.36	163.78	167	8632	{}	\N	2020-10-12 16:41:34.085723
81457	BBH	2020-10-08	166.94	166.94	165.5	166.17	24876	{}	\N	2020-10-12 16:41:34.08984
81458	BBH	2020-10-09	166.2	167.1159	166.05	167.02	9090	{}	\N	2020-10-12 16:41:34.094192
81459	BBH	2020-10-12	167.85	168.56	167.14	168.56	7492	{}	\N	2020-10-12 16:41:34.098693
81460	CHAD	2020-09-28	21.33	21.4384	21.33	21.345	19810	{}	\N	2020-10-12 16:41:37.68596
81461	CHAD	2020-09-29	21.36	21.3634	21.1892	21.23	14352	{}	\N	2020-10-12 16:41:37.694956
81462	CHAD	2020-09-30	21.3173	21.3296	21.16	21.221	8892	{}	\N	2020-10-12 16:41:37.700843
81463	CHAD	2020-10-01	20.87	20.91	20.84	20.85	3981	{}	\N	2020-10-12 16:41:37.705691
81464	CHAD	2020-10-02	21.18	21.1801	20.9901	21.0735	8317	{}	\N	2020-10-12 16:41:37.710358
81465	CHAD	2020-10-05	20.8919	20.91	20.79	20.79	17713	{}	\N	2020-10-12 16:41:37.716723
81466	CHAD	2020-10-06	20.7	20.85	20.7	20.79	21064	{}	\N	2020-10-12 16:41:37.722817
81467	CHAD	2020-10-07	20.635	20.66	20.57	20.59	10960	{}	\N	2020-10-12 16:41:37.728514
81468	CHAD	2020-10-08	20.6	20.6298	20.5403	20.58	25538	{}	\N	2020-10-12 16:41:37.733208
81469	CHAD	2020-10-09	20.41	20.4423	20.38	20.41	19583	{}	\N	2020-10-12 16:41:37.737737
81470	CHAD	2020-10-12	20.045	20.045	19.99	20.01	13450	{}	\N	2020-10-12 16:41:37.741743
81471	ACC	2020-09-28	34.7	35.37	34.7	35.04	1072299	{}	\N	2020-10-12 16:41:40.301449
81472	ACC	2020-09-29	34.91	35.04	34.19	34.71	1554177	{}	\N	2020-10-12 16:41:40.312016
81473	ACC	2020-09-30	34.95	35.23	34.51	34.92	4099002	{}	\N	2020-10-12 16:41:40.318447
81474	ACC	2020-10-01	34.94	36.72	34.94	36.67	1812093	{}	\N	2020-10-12 16:41:40.325157
81475	ACC	2020-10-02	35.98	37.88	35.75	37.76	1543613	{}	\N	2020-10-12 16:41:40.33156
81476	ACC	2020-10-05	37.48	38.5	36.76	37.15	1856886	{}	\N	2020-10-12 16:41:40.337371
81477	ACC	2020-10-06	37.26	37.45	36.26	36.43	2419434	{}	\N	2020-10-12 16:41:40.342344
81478	ACC	2020-10-07	36.75	37.12	36.0583	36.62	2312430	{}	\N	2020-10-12 16:41:40.347453
81479	ACC	2020-10-08	36.74	37.48	36.74	37.34	1120614	{}	\N	2020-10-12 16:41:40.353079
81480	ACC	2020-10-09	37.5	37.69	36.93	37.07	1122193	{}	\N	2020-10-12 16:41:40.359014
81481	ACC	2020-10-12	37	37.13	36.61	36.9701	459762	{}	\N	2020-10-12 16:41:40.366177
81482	AKCA	2020-09-28	18.18	18.22	18.115	18.13	222810	{}	\N	2020-10-12 16:41:42.703042
81483	AKCA	2020-09-29	18.14	18.15	18.13	18.14	246105	{}	\N	2020-10-12 16:41:42.711727
81484	AKCA	2020-09-30	18.14	18.155	18.12	18.14	701837	{}	\N	2020-10-12 16:41:42.718718
81485	AKCA	2020-10-01	18.13	18.15	18.11	18.14	446523	{}	\N	2020-10-12 16:41:42.727671
81486	AKCA	2020-10-02	18.08	18.17	18.08	18.13	279280	{}	\N	2020-10-12 16:41:42.733624
81487	AKCA	2020-10-05	18.13	18.16	18.08	18.14	1023980	{}	\N	2020-10-12 16:41:42.738708
81488	AKCA	2020-10-06	18.1	18.14	18.1	18.13	191432	{}	\N	2020-10-12 16:41:42.743698
81489	AKCA	2020-10-07	18.14	18.15	18.13	18.13	785755	{}	\N	2020-10-12 16:41:42.74837
81490	AKCA	2020-10-08	18.15	18.15	18.13	18.14	223842	{}	\N	2020-10-12 16:41:42.753571
81491	AKCA	2020-10-09	18.15	18.17	18.13	18.17	720711	{}	\N	2020-10-12 16:41:42.758669
81492	AL	2020-09-28	29.04	30.4	28.59	29.85	1131884	{}	\N	2020-10-12 16:41:46.213821
81493	AL	2020-09-29	29.63	29.755	28.85	29.27	1267173	{}	\N	2020-10-12 16:41:46.223575
81494	AL	2020-09-30	29.47	30.11	28.87	29.42	2321912	{}	\N	2020-10-12 16:41:46.230393
81495	AL	2020-10-01	29.55	30.99	29.38	30.93	1793309	{}	\N	2020-10-12 16:41:46.236891
81496	AL	2020-10-02	29.52	31.415	29.24	31.05	1375295	{}	\N	2020-10-12 16:41:46.241967
81497	AL	2020-10-05	31.62	32.03	31.25	31.49	1396027	{}	\N	2020-10-12 16:41:46.246882
81498	AL	2020-10-06	32.1473	32.34	29.72	29.94	1369529	{}	\N	2020-10-12 16:41:46.251479
81499	AL	2020-10-07	30.45	30.92	30.15	30.64	1386673	{}	\N	2020-10-12 16:41:46.256596
81500	AL	2020-10-08	30.92	31.42	30.26	30.73	2826576	{}	\N	2020-10-12 16:41:46.261777
81501	AL	2020-10-09	30.8	31.57	29.94	31.19	3066718	{}	\N	2020-10-12 16:41:46.266667
81502	AL	2020-10-12	31.07	31.59	30.8	30.88	658782	{}	\N	2020-10-12 16:41:46.271513
81503	AMRC	2020-09-28	32.05	32.81	31.84	32.71	225890	{}	\N	2020-10-12 16:41:48.353815
81504	AMRC	2020-09-29	32.65	33.71	32.5	33.46	520556	{}	\N	2020-10-12 16:41:48.363004
81505	AMRC	2020-09-30	33.53	34.095	32.75	33.4	627458	{}	\N	2020-10-12 16:41:48.369268
81506	AMRC	2020-10-01	33.82	33.93	33	33.49	348312	{}	\N	2020-10-12 16:41:48.375269
81507	AMRC	2020-10-02	32.54	34.17	32.42	33.64	394286	{}	\N	2020-10-12 16:41:48.380373
81508	AMRC	2020-10-05	33.74	35.89	33.59	35.53	400195	{}	\N	2020-10-12 16:41:48.385087
81509	AMRC	2020-10-06	35.6	36.945	34.62	36.13	407333	{}	\N	2020-10-12 16:41:48.390512
81510	AMRC	2020-10-07	37	39.08	36.64	37.24	1263734	{}	\N	2020-10-12 16:41:48.395303
81511	AMRC	2020-10-08	37.75	38	36.39	36.83	527546	{}	\N	2020-10-12 16:41:48.399571
81512	AMRC	2020-10-09	37.06	37.93	36.84	37.78	393820	{}	\N	2020-10-12 16:41:48.404061
81513	AMRC	2020-10-12	37.78	38.31	37.41	38	182253	{}	\N	2020-10-12 16:41:48.408819
81514	ARCM	2020-09-28	100.565	100.68	100.565	100.68	722	{}	\N	2020-10-12 16:41:50.302435
81515	ARCM	2020-09-29	100.51	100.51	100.51	100.51	100	{}	\N	2020-10-12 16:41:50.311521
81516	ARCM	2020-09-30	100.6	100.6	100.6	100.6	100	{}	\N	2020-10-12 16:41:50.318613
81517	ARCM	2020-10-01	100.48	100.48	100.48	100.48	100	{}	\N	2020-10-12 16:41:50.32456
81518	ARCM	2020-10-02	100.475	100.475	100.475	100.475	168	{}	\N	2020-10-12 16:41:50.329495
81519	ARCM	2020-10-05	100.485	100.485	100.485	100.485	100	{}	\N	2020-10-12 16:41:50.334868
81520	ARCM	2020-10-06	100.4202	100.45	100.4202	100.45	257	{}	\N	2020-10-12 16:41:50.339691
81521	ARCM	2020-10-07	100.45	100.45	100.45	100.45	101	{}	\N	2020-10-12 16:41:50.344707
81522	ARCM	2020-10-08	100.44	100.44	100.44	100.44	101	{}	\N	2020-10-12 16:41:50.350214
81523	ARCM	2020-10-09	100.44	100.44	100.44	100.44	100	{}	\N	2020-10-12 16:41:50.356001
81524	ASG	2020-09-28	7.22	7.23	7.08	7.14	206769	{}	\N	2020-10-12 16:41:51.303449
81525	ASG	2020-09-29	7.14	7.14	7.06	7.09	321358	{}	\N	2020-10-12 16:41:51.315198
81526	ASG	2020-09-30	7.1	7.175	7.0884	7.11	429056	{}	\N	2020-10-12 16:41:51.322983
81527	ASG	2020-10-01	7.15	7.2133	7.07	7.19	453383	{}	\N	2020-10-12 16:41:51.329599
81528	ASG	2020-10-02	7.08	7.16	7.03	7.14	386267	{}	\N	2020-10-12 16:41:51.33494
81529	ASG	2020-10-05	7.16	7.206	7.07	7.2	399078	{}	\N	2020-10-12 16:41:51.340601
81530	ASG	2020-10-06	7.22	7.31	7.19	7.22	324966	{}	\N	2020-10-12 16:41:51.346279
81531	ASG	2020-10-07	7.25	7.33	7.19	7.3	281720	{}	\N	2020-10-12 16:41:51.352352
81532	ASG	2020-10-08	7.38	7.57	7.3711	7.55	507747	{}	\N	2020-10-12 16:41:51.358126
81533	ASG	2020-10-09	7.58	7.62	7.56	7.56	289120	{}	\N	2020-10-12 16:41:51.364207
81534	ASG	2020-10-12	7.66	7.72	7.5	7.705	382957	{}	\N	2020-10-12 16:41:51.370428
81535	BHLB	2020-09-28	9.52	9.955	9.52	9.84	557711	{}	\N	2020-10-12 16:41:52.819538
81536	BHLB	2020-09-29	9.79	9.92	9.65	9.82	671457	{}	\N	2020-10-12 16:41:52.83224
81537	BHLB	2020-09-30	9.88	10.38	9.86	10.11	1385971	{}	\N	2020-10-12 16:41:52.839834
81538	BHLB	2020-10-01	10.18	10.32	9.8	10.23	1254107	{}	\N	2020-10-12 16:41:52.84625
81539	BHLB	2020-10-02	9.98	10.65	9.86	10.65	1286160	{}	\N	2020-10-12 16:41:52.852423
81540	BHLB	2020-10-05	10.69	10.87	10.57	10.66	938781	{}	\N	2020-10-12 16:41:52.85736
81541	BHLB	2020-10-06	10.86	11.14	10.54	10.59	1391096	{}	\N	2020-10-12 16:41:52.862549
81542	BHLB	2020-10-07	10.65	11.03	10.6457	10.92	1130139	{}	\N	2020-10-12 16:41:52.867702
81543	BHLB	2020-10-08	11.08	11.3	10.9648	11.22	607538	{}	\N	2020-10-12 16:41:52.87281
81544	BHLB	2020-10-09	11.35	11.42	10.96	10.98	507177	{}	\N	2020-10-12 16:41:52.877688
81545	BHLB	2020-10-12	10.92	11.17	10.83	11.07	188160	{}	\N	2020-10-12 16:41:52.882833
81546	BKEP	2020-09-28	1.48	1.5	1.4501	1.47	93987	{}	\N	2020-10-12 16:41:53.960308
81547	BKEP	2020-09-29	1.49	1.65	1.4792	1.59	738702	{}	\N	2020-10-12 16:41:53.972875
81548	BKEP	2020-09-30	1.57	1.58	1.46	1.54	287478	{}	\N	2020-10-12 16:41:53.97955
81549	BKEP	2020-10-01	1.52	1.61	1.2	1.39	511960	{}	\N	2020-10-12 16:41:53.985096
81550	BKEP	2020-10-02	1.39	1.51	1.3867	1.44	109510	{}	\N	2020-10-12 16:41:53.990025
81551	BKEP	2020-10-05	1.44	1.49	1.42	1.45	106858	{}	\N	2020-10-12 16:41:53.994784
81552	BKEP	2020-10-06	1.44	1.48	1.374	1.42	103335	{}	\N	2020-10-12 16:41:54.000235
81553	BKEP	2020-10-07	1.47	1.47	1.4329	1.46	75721	{}	\N	2020-10-12 16:41:54.005039
81554	BKEP	2020-10-08	1.48	1.55	1.43	1.53	117484	{}	\N	2020-10-12 16:41:54.009838
81555	BKEP	2020-10-09	1.55	1.55	1.4	1.44	215128	{}	\N	2020-10-12 16:41:54.014872
81556	BKEP	2020-10-12	1.4	1.44	1.37	1.39	158430	{}	\N	2020-10-12 16:41:54.020011
81557	BSJN	2020-09-28	24.44	24.5	24.41	24.47	93887	{}	\N	2020-10-12 16:41:55.023114
81558	BSJN	2020-09-29	24.47	24.4808	24.41	24.47	127534	{}	\N	2020-10-12 16:41:55.035671
81559	BSJN	2020-09-30	24.47	24.6	24.47	24.53	112966	{}	\N	2020-10-12 16:41:55.04272
81560	BSJN	2020-10-01	24.57	24.63	24.57	24.6	112837	{}	\N	2020-10-12 16:41:55.04874
81561	BSJN	2020-10-02	24.55	24.63	24.55	24.61	70829	{}	\N	2020-10-12 16:41:55.054286
81562	BSJN	2020-10-05	24.66	24.76	24.66	24.74	102828	{}	\N	2020-10-12 16:41:55.059013
81563	BSJN	2020-10-06	24.76	24.86	24.7	24.71	157826	{}	\N	2020-10-12 16:41:55.063815
81564	BSJN	2020-10-07	24.77	24.804	24.7353	24.76	100019	{}	\N	2020-10-12 16:41:55.068706
81565	BSJN	2020-10-08	24.8	24.87	24.8	24.83	127247	{}	\N	2020-10-12 16:41:55.07384
81566	BSJN	2020-10-09	24.87	24.97	24.824	24.84	162126	{}	\N	2020-10-12 16:41:55.078884
81567	BSJN	2020-10-12	24.96	25	24.9079	25	107501	{}	\N	2020-10-12 16:41:55.083746
81568	CDK	2020-09-28	44.21	44.61	43.58	43.78	871333	{}	\N	2020-10-12 16:41:57.022094
81569	CDK	2020-09-29	43.72	43.84	43.43	43.59	562792	{}	\N	2020-10-12 16:41:57.030796
81570	CDK	2020-09-30	43.58	44.21	43.4	43.59	796801	{}	\N	2020-10-12 16:41:57.037631
81571	CDK	2020-10-01	43.64	44.3	43.065	44.03	599521	{}	\N	2020-10-12 16:41:57.044334
81572	CDK	2020-10-02	43.31	44.3199	43.165	44.05	407765	{}	\N	2020-10-12 16:41:57.049556
81573	CDK	2020-10-05	44.44	44.71	42.22	44.28	820749	{}	\N	2020-10-12 16:41:57.054564
81574	CDK	2020-10-06	44.37	45.51	44.2	44.63	602916	{}	\N	2020-10-12 16:41:57.059537
81575	CDK	2020-10-07	44.76	45.02	43.52	43.54	477308	{}	\N	2020-10-12 16:41:57.065033
81576	CDK	2020-10-08	43.83	43.86	42.54	43.02	662075	{}	\N	2020-10-12 16:41:57.070346
81577	CDK	2020-10-09	43.82	43.8895	43.093	43.67	506163	{}	\N	2020-10-12 16:41:57.075634
81578	CDK	2020-10-12	43.93	44.18	43.43	44.055	98383	{}	\N	2020-10-12 16:41:57.080684
81579	CIDM	2020-09-28	0.58	0.59	0.5587	0.5649	1219616	{}	\N	2020-10-12 16:41:58.405422
81580	CIDM	2020-09-29	0.561	0.571419	0.55	0.563	1094328	{}	\N	2020-10-12 16:41:58.420006
81581	CIDM	2020-09-30	0.5617	0.582	0.5564	0.5651	1347301	{}	\N	2020-10-12 16:41:58.426193
81582	CIDM	2020-10-01	0.57	0.5942	0.56	0.5748	2210706	{}	\N	2020-10-12 16:41:58.432006
81583	CIDM	2020-10-02	0.55	0.5722	0.5458	0.5699	1005863	{}	\N	2020-10-12 16:41:58.437819
81584	CIDM	2020-10-05	0.5779	0.5779	0.56	0.5715	712582	{}	\N	2020-10-12 16:41:58.442946
81585	CIDM	2020-10-06	0.5612	0.584	0.56	0.5607	1107090	{}	\N	2020-10-12 16:41:58.447578
81586	CIDM	2020-10-07	0.579	0.579	0.5515	0.566	1203253	{}	\N	2020-10-12 16:41:58.452368
81587	CIDM	2020-10-08	0.5691	0.5818	0.56	0.5721	1029722	{}	\N	2020-10-12 16:41:58.457198
81588	CIDM	2020-10-09	0.585	0.585	0.565	0.5709	1128107	{}	\N	2020-10-12 16:41:58.461887
81589	CIDM	2020-10-12	0.575	0.585	0.5727	0.5837	596546	{}	\N	2020-10-12 16:41:58.466827
81590	CLIR	2020-09-28	2.25	2.2583	2.16	2.22	74382	{}	\N	2020-10-12 16:42:02.099427
81591	CLIR	2020-09-29	2.19	2.2701	2.14	2.25	45650	{}	\N	2020-10-12 16:42:02.116792
81592	CLIR	2020-09-30	2.2001	2.29	2.2	2.27	63658	{}	\N	2020-10-12 16:42:02.124138
81593	CLIR	2020-10-01	2.25	2.53	2.25	2.5	200072	{}	\N	2020-10-12 16:42:02.131077
81594	CLIR	2020-10-02	2.4	2.55	2.3524	2.48	100861	{}	\N	2020-10-12 16:42:02.137374
81595	CLIR	2020-10-05	2.46	2.54	2.4	2.45	50700	{}	\N	2020-10-12 16:42:02.142973
81596	CLIR	2020-10-06	2.49	2.53	2.45	2.45	96309	{}	\N	2020-10-12 16:42:02.148491
81597	CLIR	2020-10-07	2.46	2.73	2.46	2.61	255035	{}	\N	2020-10-12 16:42:02.153949
81598	CLIR	2020-10-08	2.64	2.92	2.48	2.59	1110917	{}	\N	2020-10-12 16:42:02.160507
81599	CLIR	2020-10-09	2.59	2.89	2.5201	2.87	684033	{}	\N	2020-10-12 16:42:02.16595
81600	CLIR	2020-10-12	2.9	2.99	2.6	2.65	319307	{}	\N	2020-10-12 16:42:02.171594
81601	AMED	2020-09-28	234.92	236.01	232.19	232.91	131730	{}	\N	2020-10-12 16:42:04.355979
81602	AMED	2020-09-29	232.48	235.315	230.28	232.04	162794	{}	\N	2020-10-12 16:42:04.367086
81603	AMED	2020-09-30	230.79	239.68	230.79	236.22	316229	{}	\N	2020-10-12 16:42:04.37393
81604	AMED	2020-10-01	235.84	238.144	229.97	235.61	178839	{}	\N	2020-10-12 16:42:04.379955
81605	AMED	2020-10-02	229.335	237.19	229.33	233.53	174832	{}	\N	2020-10-12 16:42:04.385676
81606	AMED	2020-10-05	233.85	240.36	233.82	240	148926	{}	\N	2020-10-12 16:42:04.391083
81607	AMED	2020-10-06	239.42	242.695	236.585	242	200036	{}	\N	2020-10-12 16:42:04.396174
81608	AMED	2020-10-07	243.02	245.88	242.0234	242.61	190320	{}	\N	2020-10-12 16:42:04.40116
81609	AMED	2020-10-08	243.24	244.0259	239.49	242.72	133134	{}	\N	2020-10-12 16:42:04.406105
81610	AMED	2020-10-09	243.34	247.96	243.08	247.83	99255	{}	\N	2020-10-12 16:42:04.410765
81611	AMED	2020-10-12	248.48	250.52	246.52	248.95	21861	{}	\N	2020-10-12 16:42:04.415775
81612	AOA	2020-09-28	57.84	58	57.68	57.88	39098	{}	\N	2020-10-12 16:42:06.329008
81613	AOA	2020-09-29	57.94	58.02	57.6272	57.75	42298	{}	\N	2020-10-12 16:42:06.340286
81614	AOA	2020-09-30	57.74	58.2959	57.6901	58	35538	{}	\N	2020-10-12 16:42:06.347564
81615	AOA	2020-10-01	58.18	58.3268	58.0856	58.25	88481	{}	\N	2020-10-12 16:42:06.35423
81616	AOA	2020-10-02	57.42	58	57.344	57.8	76290	{}	\N	2020-10-12 16:42:06.359886
81617	AOA	2020-10-05	58.17	58.63	58.17	58.63	40635	{}	\N	2020-10-12 16:42:06.365552
81618	AOA	2020-10-06	58.68	58.89	58.06	58.14	36516	{}	\N	2020-10-12 16:42:06.369926
81619	AOA	2020-10-07	58.51	58.8536	58.51	58.74	28073	{}	\N	2020-10-12 16:42:06.374049
81620	AOA	2020-10-08	59.02	59.19	58.9654	59.13	41494	{}	\N	2020-10-12 16:42:06.378105
81621	AOA	2020-10-09	59.36	59.6	59.32	59.58	49415	{}	\N	2020-10-12 16:42:06.382654
81622	AOA	2020-10-12	59.84	60.11	59.74	60.11	100235	{}	\N	2020-10-12 16:42:06.386947
81623	ARCB	2020-09-28	30.51	31.845	30.47	31.39	195688	{}	\N	2020-10-12 16:42:07.318006
81624	ARCB	2020-09-29	31.52	32.91	31.2	32.24	388897	{}	\N	2020-10-12 16:42:07.329239
81625	ARCB	2020-09-30	32.29	32.62	30.8	31.06	234077	{}	\N	2020-10-12 16:42:07.335767
81626	ARCB	2020-10-01	31.02	31.18	29.43	31.02	306377	{}	\N	2020-10-12 16:42:07.341122
81627	ARCB	2020-10-02	30.46	31.95	29.33	31.65	105146	{}	\N	2020-10-12 16:42:07.346641
81628	ARCB	2020-10-05	32.1	33.47	32.1	33.35	143614	{}	\N	2020-10-12 16:42:07.351435
81629	ARCB	2020-10-06	33.37	34.05	32.41	33.5	267754	{}	\N	2020-10-12 16:42:07.356189
81630	ARCB	2020-10-07	33.9	34.95	33.51	34.41	279280	{}	\N	2020-10-12 16:42:07.361042
81631	ARCB	2020-10-08	34.69	36.01	34.4	35.79	194304	{}	\N	2020-10-12 16:42:07.365707
81632	ARCB	2020-10-09	35.8	36.15	34.94	35	162250	{}	\N	2020-10-12 16:42:07.370525
81633	ARCB	2020-10-12	35.43	35.53	34.89	35.09	31115	{}	\N	2020-10-12 16:42:07.375763
81634	ASTC	2020-09-28	1.57	1.7498	1.57	1.6801	185155	{}	\N	2020-10-12 16:42:08.848941
81635	ASTC	2020-09-29	1.6973	1.8687	1.65	1.84	434472	{}	\N	2020-10-12 16:42:08.859077
81636	ASTC	2020-09-30	1.77	1.83	1.6801	1.7	96440	{}	\N	2020-10-12 16:42:08.866277
81637	ASTC	2020-10-01	1.73	1.7599	1.69	1.72	71254	{}	\N	2020-10-12 16:42:08.872043
81638	ASTC	2020-10-02	1.67	1.75	1.58	1.72	139370	{}	\N	2020-10-12 16:42:08.877244
81639	ASTC	2020-10-05	1.73	1.76	1.65	1.72	46600	{}	\N	2020-10-12 16:42:08.881698
81640	ASTC	2020-10-06	1.71	1.76	1.6561	1.74	33020	{}	\N	2020-10-12 16:42:08.885914
81641	ASTC	2020-10-07	1.72	1.78	1.69	1.77	40742	{}	\N	2020-10-12 16:42:08.890111
81642	ASTC	2020-10-08	1.74	1.78	1.66	1.76	78795	{}	\N	2020-10-12 16:42:08.894712
81643	ASTC	2020-10-09	1.75	1.77	1.7	1.75	102571	{}	\N	2020-10-12 16:42:08.902159
81644	ASTC	2020-10-12	1.72	1.78	1.7	1.76	28061	{}	\N	2020-10-12 16:42:08.906778
81645	BEDU	2020-09-28	6.89	6.96	6.83	6.89	9117	{}	\N	2020-10-12 16:42:10.406002
81646	BEDU	2020-09-29	6.81	7.01	6.75	6.77	25547	{}	\N	2020-10-12 16:42:10.417108
81647	BEDU	2020-09-30	6.78	6.91	6.18	6.39	144428	{}	\N	2020-10-12 16:42:10.423952
81648	BEDU	2020-10-01	6.37	6.64	6.331	6.5924	161542	{}	\N	2020-10-12 16:42:10.430129
81649	BEDU	2020-10-02	6.5468	6.68	6.43	6.58	67992	{}	\N	2020-10-12 16:42:10.435759
81650	BEDU	2020-10-05	6.68	6.7099	6.57	6.57	58236	{}	\N	2020-10-12 16:42:10.440656
81651	BEDU	2020-10-06	6.67	6.67	6.51	6.51	96087	{}	\N	2020-10-12 16:42:10.445365
81652	BEDU	2020-10-07	6.52	6.7	6.03	6.03	140012	{}	\N	2020-10-12 16:42:10.44989
81653	BEDU	2020-10-08	6.2	6.59	6.2	6.525	61514	{}	\N	2020-10-12 16:42:10.454667
81654	BEDU	2020-10-09	6.43	6.6286	6.31	6.49	126732	{}	\N	2020-10-12 16:42:10.459057
81655	BEDU	2020-10-12	6.61	6.625	6.36	6.45	31399	{}	\N	2020-10-12 16:42:10.463525
81656	BIBL	2020-09-28	33.0699	33.2	33.0037	33.05	31427	{}	\N	2020-10-12 16:42:12.121256
81657	BIBL	2020-09-29	33.1199	33.145	32.9242	33.01	10336	{}	\N	2020-10-12 16:42:12.13105
81658	BIBL	2020-09-30	33.3757	33.43	33.04	33.28	13124	{}	\N	2020-10-12 16:42:12.138005
81659	BIBL	2020-10-01	33.4293	33.4293	33.15	33.37	88301	{}	\N	2020-10-12 16:42:12.143239
81660	BIBL	2020-10-02	33.109	33.4699	33.109	33.4699	12483	{}	\N	2020-10-12 16:42:12.148816
81661	BIBL	2020-10-05	33.81	34	33.69	34	71519	{}	\N	2020-10-12 16:42:12.153246
81662	BIBL	2020-10-06	34.05	34.37	33.68	33.76	26040	{}	\N	2020-10-12 16:42:12.157768
81663	BIBL	2020-10-07	34.115	34.3499	34.115	34.27	16652	{}	\N	2020-10-12 16:42:12.162362
81664	BIBL	2020-10-08	34.39	34.522	34.3501	34.52	6462	{}	\N	2020-10-12 16:42:12.167026
81665	BIBL	2020-10-09	34.88	34.88	34.64	34.75	18976	{}	\N	2020-10-12 16:42:12.171466
81666	BIBL	2020-10-12	35.049	35.0899	35.0324	35.085	9029	{}	\N	2020-10-12 16:42:12.176263
81667	BRKR	2020-09-28	39.7731	40.03	39.32	39.49	441517	{}	\N	2020-10-12 16:42:15.438096
81668	BRKR	2020-09-29	39.56	40.475	39.56	39.9	662847	{}	\N	2020-10-12 16:42:15.451853
81669	BRKR	2020-09-30	40.14	40.56	38.13	39.75	404483	{}	\N	2020-10-12 16:42:15.458245
81670	BRKR	2020-10-01	39.55	40.49	39.06	39.15	404391	{}	\N	2020-10-12 16:42:15.464828
81671	BRKR	2020-10-02	38.66	39.61	38.66	38.78	349229	{}	\N	2020-10-12 16:42:15.471174
81672	BRKR	2020-10-05	39.15	39.86	39.15	39.48	544073	{}	\N	2020-10-12 16:42:15.476153
81673	BRKR	2020-10-06	39.52	40.31	39.41	39.75	954783	{}	\N	2020-10-12 16:42:15.481933
81674	BRKR	2020-10-07	40.08	40.83	40.08	40.73	491297	{}	\N	2020-10-12 16:42:15.486973
81675	BRKR	2020-10-08	40.92	42.19	40.635	41.64	655749	{}	\N	2020-10-12 16:42:15.492007
81676	BRKR	2020-10-09	42.02	42.27	41.53	42.14	486691	{}	\N	2020-10-12 16:42:15.496587
81677	BRKR	2020-10-12	42.27	43.47	42.26	43.34	247698	{}	\N	2020-10-12 16:42:15.501491
81678	BRZU	2020-09-28	72.85	73.34	64.195	65.39	588729	{}	\N	2020-10-12 16:42:16.513977
81679	BRZU	2020-09-29	64.54	67.2694	63.64	63.89	532441	{}	\N	2020-10-12 16:42:16.527387
81680	BRZU	2020-09-30	64.7	67.17	64.7	65.98	250340	{}	\N	2020-10-12 16:42:16.534961
81681	BRZU	2020-10-01	65.06	66.34	63.46	66.17	354111	{}	\N	2020-10-12 16:42:16.540923
81682	BRZU	2020-10-02	65.17	67.84	63.64	63.67	305008	{}	\N	2020-10-12 16:42:16.546862
81683	BRZU	2020-10-05	65.16	69.77	64.6	69.18	330460	{}	\N	2020-10-12 16:42:16.55203
81684	BRZU	2020-10-06	71.06	72.85	66.68	67.57	448504	{}	\N	2020-10-12 16:42:16.556756
81685	BRZU	2020-10-07	68.57	69.18	65.88	67.15	262089	{}	\N	2020-10-12 16:42:16.561656
81686	BRZU	2020-10-08	67.31	71.45	66.9	70.9	286384	{}	\N	2020-10-12 16:42:16.566252
81687	BRZU	2020-10-09	71.84	74.13	71.0484	72.24	280612	{}	\N	2020-10-12 16:42:16.571353
81688	BRZU	2020-10-12	73.41	73.99	72.36	73.89	69902	{}	\N	2020-10-12 16:42:16.576238
81689	BSJM	2020-09-28	22.74	22.798	22.74	22.775	115133	{}	\N	2020-10-12 16:42:17.633598
81690	BSJM	2020-09-29	22.79	22.81	22.75	22.81	112254	{}	\N	2020-10-12 16:42:17.645206
81691	BSJM	2020-09-30	22.81	22.9	22.8	22.86	160667	{}	\N	2020-10-12 16:42:17.651706
81692	BSJM	2020-10-01	22.9	22.93	22.84	22.91	122123	{}	\N	2020-10-12 16:42:17.658081
81693	BSJM	2020-10-02	22.91	22.95	22.855	22.93	104740	{}	\N	2020-10-12 16:42:17.663596
81694	BSJM	2020-10-05	22.97	23.03	22.92	23	98544	{}	\N	2020-10-12 16:42:17.668813
81695	BSJM	2020-10-06	23.02	23.07	22.98	22.99	166581	{}	\N	2020-10-12 16:42:17.673703
81696	BSJM	2020-10-07	23.01	23.06	22.9746	23.03	166612	{}	\N	2020-10-12 16:42:17.67871
81697	BSJM	2020-10-08	23.05	23.1	23.02	23.07	165473	{}	\N	2020-10-12 16:42:17.683578
81698	BSJM	2020-10-09	23.08	23.12	23.04	23.07	172592	{}	\N	2020-10-12 16:42:17.688873
81699	BSJM	2020-10-12	23.09	23.1537	23.0862	23.145	73202	{}	\N	2020-10-12 16:42:17.693941
81700	CAAP	2020-09-28	2.2	2.28	2.2	2.23	37053	{}	\N	2020-10-12 16:42:19.425178
81701	CAAP	2020-09-29	2.26	2.29	2.2	2.28	161834	{}	\N	2020-10-12 16:42:19.439853
81702	CAAP	2020-09-30	2.2801	2.3	2.2284	2.23	118671	{}	\N	2020-10-12 16:42:19.446643
81703	CAAP	2020-10-01	2.23	2.28	2.18	2.24	108640	{}	\N	2020-10-12 16:42:19.453018
81704	CAAP	2020-10-02	2.22	2.22	2.12	2.15	366428	{}	\N	2020-10-12 16:42:19.459023
81705	CAAP	2020-10-05	2.23	2.27	2.15	2.17	140020	{}	\N	2020-10-12 16:42:19.464383
81706	CAAP	2020-10-06	2.2248	2.25	2.07	2.1	108275	{}	\N	2020-10-12 16:42:19.469379
81707	CAAP	2020-10-07	2.15	2.18	2.02	2.05	336856	{}	\N	2020-10-12 16:42:19.474015
81708	CAAP	2020-10-08	2.07	2.07	1.78	1.92	1406524	{}	\N	2020-10-12 16:42:19.478518
81709	CAAP	2020-10-09	1.93	1.98	1.8	1.84	1205251	{}	\N	2020-10-12 16:42:19.483053
81710	CAAP	2020-10-12	1.86	1.9155	1.79	1.855	254758	{}	\N	2020-10-12 16:42:19.487676
81711	CASY	2020-09-28	174.99	176.72	174.83	175.82	160715	{}	\N	2020-10-12 16:42:20.541694
81712	CASY	2020-09-29	175.44	178	174.56	177.31	217931	{}	\N	2020-10-12 16:42:20.554606
81713	CASY	2020-09-30	177.39	179.57	176.64	177.65	235456	{}	\N	2020-10-12 16:42:20.561288
81714	CASY	2020-10-01	177.005	180.98	177.005	180.2	171784	{}	\N	2020-10-12 16:42:20.567047
81715	CASY	2020-10-02	178.61	181.83	177.87	181.05	175638	{}	\N	2020-10-12 16:42:20.572526
81716	CASY	2020-10-05	181.78	184.84	181.78	183.81	162465	{}	\N	2020-10-12 16:42:20.57715
81717	CASY	2020-10-06	185	186.58	181.93	182.65	187094	{}	\N	2020-10-12 16:42:20.581916
81718	CASY	2020-10-07	184.12	185.24	180.75	180.99	193172	{}	\N	2020-10-12 16:42:20.587569
81719	CASY	2020-10-08	181.13	183.61	180.75	182.77	142232	{}	\N	2020-10-12 16:42:20.593149
81720	CASY	2020-10-09	183.96	184.77	182.605	183.31	112511	{}	\N	2020-10-12 16:42:20.598462
81721	CASY	2020-10-12	184	184.96	183.33	184.575	25634	{}	\N	2020-10-12 16:42:20.603802
81722	ASEA	2020-09-28	11.8	11.8	11.7117	11.76	8193	{}	\N	2020-10-12 16:42:22.705165
81723	ASEA	2020-09-29	11.76	11.76	11.699	11.73	10067	{}	\N	2020-10-12 16:42:22.720423
81724	ASEA	2020-09-30	11.59	11.74	11.59	11.7	27129	{}	\N	2020-10-12 16:42:22.727539
81725	ASEA	2020-10-01	11.75	11.8461	11.75	11.83	8070	{}	\N	2020-10-12 16:42:22.733719
81726	ASEA	2020-10-02	11.8	11.84	11.71	11.8	5084	{}	\N	2020-10-12 16:42:22.739399
81727	ASEA	2020-10-05	11.97	11.97	11.905	11.945	6635	{}	\N	2020-10-12 16:42:22.744424
81728	ASEA	2020-10-06	12.0101	12.0101	11.9585	11.9585	1161	{}	\N	2020-10-12 16:42:22.748934
81729	ASEA	2020-10-07	11.98	11.998	11.93	11.998	5095	{}	\N	2020-10-12 16:42:22.753625
81730	ASEA	2020-10-08	12.07	12.1099	12.04	12.1099	1280	{}	\N	2020-10-12 16:42:22.758573
81731	ASEA	2020-10-09	12.205	12.2299	12.1848	12.1848	720	{}	\N	2020-10-12 16:42:22.763364
81732	ASEA	2020-10-12	12.14	12.18	12.1224	12.18	5926	{}	\N	2020-10-12 16:42:22.768585
81733	BC	2020-09-28	59.94	60.77	58.77	59.35	679327	{}	\N	2020-10-12 16:42:24.065125
81734	BC	2020-09-29	59.72	60.13	58.08	58.57	1327401	{}	\N	2020-10-12 16:42:24.079302
81735	BC	2020-09-30	58.77	60.37	58.37	58.91	2360292	{}	\N	2020-10-12 16:42:24.08668
81736	BC	2020-10-01	59.53	60.43	58.735	60.29	1331877	{}	\N	2020-10-12 16:42:24.093079
81737	BC	2020-10-02	59	63.69	58.98	62.13	1463227	{}	\N	2020-10-12 16:42:24.098889
81738	BC	2020-10-05	62.86	64.89	62.85	64.36	1091122	{}	\N	2020-10-12 16:42:24.105004
81739	BC	2020-10-06	64.68	65.58	62.65	62.87	1595737	{}	\N	2020-10-12 16:42:24.110075
81740	BC	2020-10-07	63.75	65.71	63.49	64.7	983997	{}	\N	2020-10-12 16:42:24.114921
81741	BC	2020-10-08	65.33	65.96	62.59	63.38	1302816	{}	\N	2020-10-12 16:42:24.120391
81742	BC	2020-10-09	63.93	64.83	62.63	62.87	1228660	{}	\N	2020-10-12 16:42:24.125639
81743	BC	2020-10-12	63.6	63.69	62.2	62.87	234840	{}	\N	2020-10-12 16:42:24.131274
81744	BGI	2020-09-28	0.59	0.605	0.5807	0.5979	56424	{}	\N	2020-10-12 16:42:25.921678
81745	BGI	2020-09-29	0.62	0.6592	0.5831	0.645	62977	{}	\N	2020-10-12 16:42:25.934628
81746	BGI	2020-09-30	0.66	0.68	0.641	0.676	27862	{}	\N	2020-10-12 16:42:25.941742
81747	BGI	2020-10-01	0.6251	0.67	0.6251	0.6696	9479	{}	\N	2020-10-12 16:42:25.947614
81748	BGI	2020-10-02	0.6275	0.67	0.61	0.66999	23752	{}	\N	2020-10-12 16:42:25.953903
81749	BGI	2020-10-05	0.6392	0.6789	0.6382	0.66	15536	{}	\N	2020-10-12 16:42:25.958711
81750	BGI	2020-10-06	0.670499	0.68	0.649999	0.649999	23896	{}	\N	2020-10-12 16:42:25.96357
81751	BGI	2020-10-07	0.64	0.68	0.64	0.6701	5272	{}	\N	2020-10-12 16:42:25.967928
81752	BGI	2020-10-08	0.68	0.745001	0.6714	0.71	157010	{}	\N	2020-10-12 16:42:25.972393
81753	BGI	2020-10-09	0.6887	0.74	0.6603	0.74	196705	{}	\N	2020-10-12 16:42:25.976705
81754	BGI	2020-10-12	0.73	0.88	0.7067	0.78	518484	{}	\N	2020-10-12 16:42:25.980961
81755	BOTZ	2020-09-28	27.4	27.56	27.3501	27.56	822047	{}	\N	2020-10-12 16:42:30.255151
81756	BOTZ	2020-09-29	27.79	28.07	27.79	27.9	1560083	{}	\N	2020-10-12 16:42:30.266489
81757	BOTZ	2020-09-30	27.86	28.1	27.8	27.87	852427	{}	\N	2020-10-12 16:42:30.27359
81758	BOTZ	2020-10-01	28.13	28.29	28.03	28.24	838553	{}	\N	2020-10-12 16:42:30.280063
81759	BOTZ	2020-10-02	27.68	28.05	27.63	27.98	1047583	{}	\N	2020-10-12 16:42:30.286343
81760	BOTZ	2020-10-05	27.98	28.28	27.98	28.27	1077659	{}	\N	2020-10-12 16:42:30.291159
81761	BOTZ	2020-10-06	28.32	28.59	28.08	28.19	1114331	{}	\N	2020-10-12 16:42:30.296293
81762	BOTZ	2020-10-07	28.45	28.685	28.45	28.66	1712690	{}	\N	2020-10-12 16:42:30.301816
81763	BOTZ	2020-10-08	28.888	28.9	28.7	28.75	690221	{}	\N	2020-10-12 16:42:30.307363
81764	BOTZ	2020-10-09	28.83	29.06	28.83	29.03	978592	{}	\N	2020-10-12 16:42:30.312882
81765	BOTZ	2020-10-12	29.23	29.25	29.1	29.2481	537354	{}	\N	2020-10-12 16:42:30.31804
81766	BSJL	2020-09-28	22.8507	22.91	22.8439	22.87	129379	{}	\N	2020-10-12 16:42:32.039533
81767	BSJL	2020-09-29	22.9	22.92	22.8304	22.92	181592	{}	\N	2020-10-12 16:42:32.049129
81768	BSJL	2020-09-30	22.92	22.9761	22.88	22.94	120005	{}	\N	2020-10-12 16:42:32.055672
81769	BSJL	2020-10-01	22.97	22.99	22.9305	22.96	198909	{}	\N	2020-10-12 16:42:32.061258
81770	BSJL	2020-10-02	22.93	22.99	22.91	22.97	92717	{}	\N	2020-10-12 16:42:32.067204
81771	BSJL	2020-10-05	22.99	23.01	22.98	23	73295	{}	\N	2020-10-12 16:42:32.072608
81772	BSJL	2020-10-06	23.01	23.06	22.99	22.99	204500	{}	\N	2020-10-12 16:42:32.078474
81773	BSJL	2020-10-07	23.02	23.0277	22.96	23.02	379985	{}	\N	2020-10-12 16:42:32.084779
81774	BSJL	2020-10-08	23.04	23.07	23.01	23.07	352545	{}	\N	2020-10-12 16:42:32.092583
81775	BSJL	2020-10-09	23.05	23.11	23.035	23.07	359028	{}	\N	2020-10-12 16:42:32.097532
81776	BSJL	2020-10-12	23.09	23.14	23.09	23.135	105528	{}	\N	2020-10-12 16:42:32.102664
81777	BTA	2020-09-28	11.86	11.9	11.81	11.88	26774	{}	\N	2020-10-12 16:42:33.505425
81778	BTA	2020-09-29	11.85	11.95	11.84	11.86	51626	{}	\N	2020-10-12 16:42:33.516359
81779	BTA	2020-09-30	11.86	11.89	11.8	11.8	46226	{}	\N	2020-10-12 16:42:33.523577
81780	BTA	2020-10-01	11.87	11.87	11.82	11.87	56834	{}	\N	2020-10-12 16:42:33.529348
81781	BTA	2020-10-02	11.83	11.9	11.82	11.9	73407	{}	\N	2020-10-12 16:42:33.534451
81782	BTA	2020-10-05	11.86	11.95	11.84	11.9	80500	{}	\N	2020-10-12 16:42:33.538605
81783	BTA	2020-10-06	11.95	11.95	11.87	11.92	61176	{}	\N	2020-10-12 16:42:33.542607
81784	BTA	2020-10-07	12	12.07	11.93	12.01	92037	{}	\N	2020-10-12 16:42:33.546783
81785	BTA	2020-10-08	12.01	12.02	11.985	12	36602	{}	\N	2020-10-12 16:42:33.551123
81786	BTA	2020-10-09	12	12.03	11.97	11.98	27992	{}	\N	2020-10-12 16:42:33.555381
81787	BTA	2020-10-12	12.03	12.06	12.02	12.02	10164	{}	\N	2020-10-12 16:42:33.559453
81788	BYLD	2020-09-28	25.71	25.7599	25.71	25.7291	16177	{}	\N	2020-10-12 16:42:35.958999
81789	BYLD	2020-09-29	25.72	25.77	25.711	25.7439	12926	{}	\N	2020-10-12 16:42:35.971735
81790	BYLD	2020-09-30	25.7	25.78	25.7	25.73	15245	{}	\N	2020-10-12 16:42:35.979096
81791	BYLD	2020-10-01	25.72	25.8	25.72	25.78	26081	{}	\N	2020-10-12 16:42:35.985276
81792	BYLD	2020-10-02	25.68	25.72	25.67	25.72	18266	{}	\N	2020-10-12 16:42:35.991323
81793	BYLD	2020-10-05	25.75	25.75	25.6867	25.71	28550	{}	\N	2020-10-12 16:42:35.99629
81794	BYLD	2020-10-06	25.7188	25.75	25.6901	25.73	24827	{}	\N	2020-10-12 16:42:36.00134
81795	BYLD	2020-10-07	25.7118	25.74	25.68	25.69	363220	{}	\N	2020-10-12 16:42:36.006426
81796	BYLD	2020-10-08	25.77	25.78	25.73	25.78	13620	{}	\N	2020-10-12 16:42:36.012008
81797	BYLD	2020-10-09	25.78	25.8	25.7546	25.8	28973	{}	\N	2020-10-12 16:42:36.017493
81798	BYLD	2020-10-12	25.7935	25.82	25.78	25.805	5922	{}	\N	2020-10-12 16:42:36.023116
81799	CCR	2020-09-28	3.36	3.4313	3.36	3.37	7298	{}	\N	2020-10-12 16:42:37.589832
81800	CCR	2020-09-29	3.37	3.395	3.31	3.33	69446	{}	\N	2020-10-12 16:42:37.602873
81801	CCR	2020-09-30	3.31	3.405	3.3	3.33	39944	{}	\N	2020-10-12 16:42:37.612986
81802	CCR	2020-10-01	3.35	3.35	3.2	3.2	106766	{}	\N	2020-10-12 16:42:37.6202
81803	CCR	2020-10-02	3.2	3.35	3.134	3.24	61050	{}	\N	2020-10-12 16:42:37.625727
81804	CCR	2020-10-05	3.2	3.265	3.17	3.17	25338	{}	\N	2020-10-12 16:42:37.631016
81805	CCR	2020-10-06	3.13	3.195	2.96	3.05	151202	{}	\N	2020-10-12 16:42:37.636272
81806	CCR	2020-10-07	3.03	3.05	2.91	3.04	120776	{}	\N	2020-10-12 16:42:37.642118
81807	CCR	2020-10-08	3.01	3.1199	3	3.03	40866	{}	\N	2020-10-12 16:42:37.647105
81808	CCR	2020-10-09	3.07	3.17	3.004	3.02	44604	{}	\N	2020-10-12 16:42:37.652438
81809	CCR	2020-10-12	3.17	3.34	3.17	3.215	45570	{}	\N	2020-10-12 16:42:37.657828
81810	CHGG	2020-09-28	70.24	71.9639	70	70.79	1641984	{}	\N	2020-10-12 16:42:39.759481
81811	CHGG	2020-09-29	70.86	72.15	70.2348	71.69	1992264	{}	\N	2020-10-12 16:42:39.770689
81812	CHGG	2020-09-30	71.47	72.56	70.45	71.44	2722125	{}	\N	2020-10-12 16:42:39.777465
81813	CHGG	2020-10-01	72.44	74.71	72	74.51	3046009	{}	\N	2020-10-12 16:42:39.782917
81814	CHGG	2020-10-02	73.05	74.48	72.22	72.57	2606622	{}	\N	2020-10-12 16:42:39.78775
81815	CHGG	2020-10-05	72.77	75.14	71.54	74.76	3928159	{}	\N	2020-10-12 16:42:39.792006
81816	CHGG	2020-10-06	75.26	78	75.02	76.17	3524086	{}	\N	2020-10-12 16:42:39.796115
81817	CHGG	2020-10-07	77.01	80.04	76.99	79.72	3790717	{}	\N	2020-10-12 16:42:39.800271
81818	CHGG	2020-10-08	80.04	81.17	78.79	79.94	2951843	{}	\N	2020-10-12 16:42:39.805097
81819	CHGG	2020-10-09	80.1	82.25	79.91	81.38	1920271	{}	\N	2020-10-12 16:42:39.809685
81820	CHGG	2020-10-12	82.42	82.89	81.23	82.12	875801	{}	\N	2020-10-12 16:42:39.814095
81821	CINF	2020-09-28	78.32	80.055	77.56	79.02	758446	{}	\N	2020-10-12 16:42:40.736068
81822	CINF	2020-09-29	78.24	78.73	76.68	77.89	511854	{}	\N	2020-10-12 16:42:40.745401
81823	CINF	2020-09-30	78.52	79.37	77.33	77.97	680393	{}	\N	2020-10-12 16:42:40.752322
81824	CINF	2020-10-01	77.93	78.77	76.05	76.49	627766	{}	\N	2020-10-12 16:42:40.758275
81825	CINF	2020-10-02	75.13	78.21	75.13	77.81	399325	{}	\N	2020-10-12 16:42:40.763153
81826	CINF	2020-10-05	80.99	82.79	80.36	82.03	815443	{}	\N	2020-10-12 16:42:40.767788
81827	CINF	2020-10-06	82.49	82.71	79.6001	79.91	594277	{}	\N	2020-10-12 16:42:40.772514
81828	CINF	2020-10-07	80.34	81.21	78.43	79.44	672072	{}	\N	2020-10-12 16:42:40.777276
81829	CINF	2020-10-08	80.02	81.38	79.32	80.64	404080	{}	\N	2020-10-12 16:42:40.781817
81830	CINF	2020-10-09	81.36	81.63	79.325	79.69	564634	{}	\N	2020-10-12 16:42:40.786151
81831	CINF	2020-10-12	80.1	82.15	79.825	81.14	167020	{}	\N	2020-10-12 16:42:40.790531
81832	CLNE	2020-09-28	2.45	2.5	2.425	2.46	964878	{}	\N	2020-10-12 16:42:42.199443
81833	CLNE	2020-09-29	2.46	2.615	2.4349	2.59	1463923	{}	\N	2020-10-12 16:42:42.209215
81834	CLNE	2020-09-30	2.59	2.61	2.46	2.48	1121305	{}	\N	2020-10-12 16:42:42.216293
81835	CLNE	2020-10-01	2.48	2.51	2.44	2.48	931517	{}	\N	2020-10-12 16:42:42.221165
81836	CLNE	2020-10-02	2.41	2.52	2.41	2.49	699645	{}	\N	2020-10-12 16:42:42.226099
81837	CLNE	2020-10-05	2.52	2.59	2.5	2.57	891743	{}	\N	2020-10-12 16:42:42.23054
81838	CLNE	2020-10-06	2.6	2.6484	2.52	2.53	1024024	{}	\N	2020-10-12 16:42:42.235121
81839	CLNE	2020-10-07	2.54	2.62	2.54	2.6	1002630	{}	\N	2020-10-12 16:42:42.239668
81840	CLNE	2020-10-08	2.62	2.68	2.61	2.65	905096	{}	\N	2020-10-12 16:42:42.244482
81841	CLNE	2020-10-09	2.67	2.71	2.63	2.66	683010	{}	\N	2020-10-12 16:42:42.249274
81842	CLNE	2020-10-12	2.71	2.71	2.6	2.6134	320429	{}	\N	2020-10-12 16:42:42.254256
81843	CMCO	2020-09-28	33.46	34.1799	33.29	33.52	59555	{}	\N	2020-10-12 16:42:45.974677
81844	CMCO	2020-09-29	33.61	33.66	33.1	33.53	56061	{}	\N	2020-10-12 16:42:45.986174
81845	CMCO	2020-09-30	33.71	34.13	32.86	33.1	52256	{}	\N	2020-10-12 16:42:45.994004
81846	CMCO	2020-10-01	33.86	33.86	32.77	33.45	52223	{}	\N	2020-10-12 16:42:46.000555
81847	CMCO	2020-10-02	32.83	33.98	32.71	33.55	76915	{}	\N	2020-10-12 16:42:46.006332
81848	CMCO	2020-10-05	34.3	35.05	33.745	34.47	88626	{}	\N	2020-10-12 16:42:46.011473
81849	CMCO	2020-10-06	34.88	35.92	33.91	34.52	51885	{}	\N	2020-10-12 16:42:46.01696
81850	CMCO	2020-10-07	35.23	36.6484	34.89	36.31	117545	{}	\N	2020-10-12 16:42:46.02231
81851	CMCO	2020-10-08	36.86	37.26	36.19	36.44	104010	{}	\N	2020-10-12 16:42:46.027616
81852	CMCO	2020-10-09	36.5	36.98	36.01	36.7	101769	{}	\N	2020-10-12 16:42:46.03283
81853	CMCO	2020-10-12	36.55	37.6899	36.55	36.92	16279	{}	\N	2020-10-12 16:42:46.038066
81854	COMM	2020-09-28	8.69	8.92	8.64	8.67	1449156	{}	\N	2020-10-12 16:42:47.441159
81855	COMM	2020-09-29	8.6	8.89	8.6	8.82	3038624	{}	\N	2020-10-12 16:42:47.457923
81856	COMM	2020-09-30	8.94	9.2	8.77	9	2882248	{}	\N	2020-10-12 16:42:47.465356
81857	COMM	2020-10-01	9.36	9.455	8.96	9.03	4332739	{}	\N	2020-10-12 16:42:47.470993
81858	COMM	2020-10-02	8.76	9.39	8.63	9.22	2779503	{}	\N	2020-10-12 16:42:47.476118
81859	COMM	2020-10-05	9.38	9.63	9.17	9.57	2628910	{}	\N	2020-10-12 16:42:47.481217
81860	COMM	2020-10-06	9.62	9.92	9.45	9.49	3306210	{}	\N	2020-10-12 16:42:47.485287
81861	COMM	2020-10-07	9.6	9.765	9.54	9.74	2555432	{}	\N	2020-10-12 16:42:47.489222
81862	COMM	2020-10-08	9.88	9.94	9.78	9.9	1805267	{}	\N	2020-10-12 16:42:47.493176
81863	COMM	2020-10-09	10.04	10.09	9.54	9.63	1839128	{}	\N	2020-10-12 16:42:47.497754
81864	COMM	2020-10-12	9.77	9.77	9.2848	9.3018	1142046	{}	\N	2020-10-12 16:42:47.50235
81865	ARKG	2020-09-28	63.72	63.8197	62.29	63.15	754649	{}	\N	2020-10-12 16:42:49.989988
81866	ARKG	2020-09-29	63.24	64.65	62.91	64.24	1004919	{}	\N	2020-10-12 16:42:50.000294
81867	ARKG	2020-09-30	64.62	64.93	63.2	63.71	753494	{}	\N	2020-10-12 16:42:50.007274
81868	ARKG	2020-10-01	64.55	64.899	63.9227	64.76	664066	{}	\N	2020-10-12 16:42:50.012954
81869	ARKG	2020-10-02	63.18	64.58	63.12	63.39	606902	{}	\N	2020-10-12 16:42:50.018736
81870	ARKG	2020-10-05	64.77	67.32	64.36	67.25	1374616	{}	\N	2020-10-12 16:42:50.023517
81871	ARKG	2020-10-06	67.48	68.82	66.7857	67.28	1348446	{}	\N	2020-10-12 16:42:50.028362
81872	ARKG	2020-10-07	68.2	70.65	68.02	70.18	1394436	{}	\N	2020-10-12 16:42:50.032844
81873	ARKG	2020-10-08	72.24	72.355	71.22	71.75	1894078	{}	\N	2020-10-12 16:42:50.037745
81874	ARKG	2020-10-09	72.68	73.95	72.29	73.81	1607757	{}	\N	2020-10-12 16:42:50.042395
81875	ARKG	2020-10-12	75.34	75.34	73.31	73.83	1176815	{}	\N	2020-10-12 16:42:50.047142
81876	AUBN	2020-09-28	37.43	38.25	36.81	37.06	6351	{}	\N	2020-10-12 16:42:51.341502
81877	AUBN	2020-09-29	36.09	37.96	36.02	36.7	1520	{}	\N	2020-10-12 16:42:51.351728
81878	AUBN	2020-09-30	36.83	36.83	36.11	36.26	1884	{}	\N	2020-10-12 16:42:51.358725
81879	AUBN	2020-10-01	36.64	39.1	36.64	38.44	4413	{}	\N	2020-10-12 16:42:51.364423
81880	AUBN	2020-10-02	38.27	38.27	36.7	36.75	2629	{}	\N	2020-10-12 16:42:51.370718
81881	AUBN	2020-10-05	37	38.5	36.925	37.55	5243	{}	\N	2020-10-12 16:42:51.375976
81882	AUBN	2020-10-06	37.85	38.365	36.83	36.83	3134	{}	\N	2020-10-12 16:42:51.381202
81883	AUBN	2020-10-07	37.29	37.9999	36.5	37.24	3482	{}	\N	2020-10-12 16:42:51.387823
81884	AUBN	2020-10-08	37	37	37	37	762	{}	\N	2020-10-12 16:42:51.393499
81885	AUBN	2020-10-09	37	37	36	37	1876	{}	\N	2020-10-12 16:42:51.398456
81886	AUBN	2020-10-12	37.9965	37.9965	36.7	36.7	1386	{}	\N	2020-10-12 16:42:51.404254
81887	AXP	2020-09-28	99	99.725	98.63	98.82	3906898	{}	\N	2020-10-12 16:42:52.641774
81888	AXP	2020-09-29	98.74	99.35	97.23	97.75	6294004	{}	\N	2020-10-12 16:42:52.651175
81889	AXP	2020-09-30	98.23	101.52	98.23	100.25	9697764	{}	\N	2020-10-12 16:42:52.657731
81890	AXP	2020-10-01	100.33	101.48	99.37	101.33	7549769	{}	\N	2020-10-12 16:42:52.66314
81891	AXP	2020-10-02	99.78	103.14	98.63	101.61	6833918	{}	\N	2020-10-12 16:42:52.668496
81892	AXP	2020-10-05	103.33	104.08	102.27	103.89	5449161	{}	\N	2020-10-12 16:42:52.673664
81893	AXP	2020-10-06	105.01	105.29	101.32	101.76	6552349	{}	\N	2020-10-12 16:42:52.678835
81894	AXP	2020-10-07	102.81	104.1546	102.39	103.38	5880494	{}	\N	2020-10-12 16:42:52.68299
81895	AXP	2020-10-08	104.33	106.14	104.06	106.06	5695242	{}	\N	2020-10-12 16:42:52.687781
81896	AXP	2020-10-09	105.99	107.21	105.235	106.34	7011959	{}	\N	2020-10-12 16:42:52.692749
81897	AXP	2020-10-12	106.46	107.01	106	106.73	2578218	{}	\N	2020-10-12 16:42:52.697375
81898	BBL	2020-09-28	44.01	44.04	43.515	43.61	1417549	{}	\N	2020-10-12 16:42:53.833915
81899	BBL	2020-09-29	43.1	43.215	42.52	42.61	2434749	{}	\N	2020-10-12 16:42:53.84627
81900	BBL	2020-09-30	42.95	43.005	42.44	42.66	2791078	{}	\N	2020-10-12 16:42:53.853629
81901	BBL	2020-10-01	42.84	42.91	42.14	42.35	1763101	{}	\N	2020-10-12 16:42:53.866926
81902	BBL	2020-10-02	41.81	42.95	41.75	42.62	2750237	{}	\N	2020-10-12 16:42:53.871553
81903	BBL	2020-10-05	42.81	43.21	42.78	42.92	3635276	{}	\N	2020-10-12 16:42:53.876094
81904	BBL	2020-10-06	42.7	42.79	41.39	41.47	4497922	{}	\N	2020-10-12 16:42:53.880358
81905	BBL	2020-10-07	42.25	43.11	42.23	42.76	3381310	{}	\N	2020-10-12 16:42:53.884934
81906	BBL	2020-10-08	42.77	43.01	42.52	43.01	2685572	{}	\N	2020-10-12 16:42:53.889627
81907	BBL	2020-10-09	43.59	43.69	43.18	43.25	1847033	{}	\N	2020-10-12 16:42:53.89452
81908	BBL	2020-10-12	43.25	43.28	42.94	43.06	1209652	{}	\N	2020-10-12 16:42:53.898911
81909	CEPU	2020-09-28	2.24	2.24	2.095	2.1	42780	{}	\N	2020-10-12 16:42:55.32815
81910	CEPU	2020-09-29	2.07	2.166	2.07	2.15	107627	{}	\N	2020-10-12 16:42:55.339579
81911	CEPU	2020-09-30	2.15	2.25	2.12	2.24	272762	{}	\N	2020-10-12 16:42:55.34777
81912	CEPU	2020-10-01	2.24	2.29	2.16	2.24	180506	{}	\N	2020-10-12 16:42:55.355596
81913	CEPU	2020-10-02	2.16	2.25	2.1	2.15	289416	{}	\N	2020-10-12 16:42:55.362068
81914	CEPU	2020-10-05	2.23	2.23	2.14	2.14	108770	{}	\N	2020-10-12 16:42:55.367222
81915	CEPU	2020-10-06	2.16	2.2	2.12	2.12	130453	{}	\N	2020-10-12 16:42:55.372333
81916	CEPU	2020-10-07	2.11	2.2772	2.1	2.2	150346	{}	\N	2020-10-12 16:42:55.377996
81917	CEPU	2020-10-08	2.21	2.3	2.21	2.26	54740	{}	\N	2020-10-12 16:42:55.38309
81918	CEPU	2020-10-09	2.26	2.298	2.21	2.21	41342	{}	\N	2020-10-12 16:42:55.388361
81919	CEPU	2020-10-12	2.26	2.28	2.22	2.23	40386	{}	\N	2020-10-12 16:42:55.394601
81920	CEV	2020-09-28	13.29	13.432	13.25	13.31	11270	{}	\N	2020-10-12 16:42:57.585137
81921	CEV	2020-09-29	13.3	13.41	13.3	13.4	7151	{}	\N	2020-10-12 16:42:57.597062
81922	CEV	2020-09-30	13.33	13.4296	13.26	13.3	7022	{}	\N	2020-10-12 16:42:57.604188
81923	CEV	2020-10-01	13.32	13.385	13.32	13.3683	5386	{}	\N	2020-10-12 16:42:57.610863
81924	CEV	2020-10-02	13.38	13.52	13.315	13.42	12324	{}	\N	2020-10-12 16:42:57.61694
81925	CEV	2020-10-05	13.34	13.44	13.26	13.44	8221	{}	\N	2020-10-12 16:42:57.62215
81926	CEV	2020-10-06	13.44	13.48	13.31	13.37	5811	{}	\N	2020-10-12 16:42:57.627691
81927	CEV	2020-10-07	13.93	13.93	13.43	13.46	11300	{}	\N	2020-10-12 16:42:57.632594
81928	CEV	2020-10-08	13.49	13.515	13.48	13.48	28307	{}	\N	2020-10-12 16:42:57.638508
81929	CEV	2020-10-09	13.5	13.5	13.39	13.39	6092	{}	\N	2020-10-12 16:42:57.644661
81930	CEV	2020-10-12	13.32	13.38	13.31	13.34	4949	{}	\N	2020-10-12 16:42:57.651017
81931	CGA	2020-09-28	2.62	2.62	2.55	2.55	5915	{}	\N	2020-10-12 16:42:58.577896
81932	CGA	2020-09-29	2.52	2.61	2.52	2.5599	13458	{}	\N	2020-10-12 16:42:58.588403
81933	CGA	2020-09-30	2.56	2.56	2.5	2.5	9696	{}	\N	2020-10-12 16:42:58.5955
81934	CGA	2020-10-01	2.46	2.83	2.46	2.71	143264	{}	\N	2020-10-12 16:42:58.601943
81935	CGA	2020-10-02	2.75	2.75	2.57	2.68	41006	{}	\N	2020-10-12 16:42:58.607884
81936	CGA	2020-10-05	2.69	2.69	2.45	2.6	20732	{}	\N	2020-10-12 16:42:58.613027
81937	CGA	2020-10-06	2.51	2.68	2.51	2.64	33254	{}	\N	2020-10-12 16:42:58.618097
81938	CGA	2020-10-07	2.63	2.63	1.77	2.22	494770	{}	\N	2020-10-12 16:42:58.623076
81939	CGA	2020-10-08	2.19	2.365	2.19	2.36	76942	{}	\N	2020-10-12 16:42:58.628027
81940	CGA	2020-10-09	2.38	2.38	2.206	2.3789	76956	{}	\N	2020-10-12 16:42:58.633005
81941	CGA	2020-10-12	2.34	2.34	2.26	2.33	16204	{}	\N	2020-10-12 16:42:58.638411
81942	CROC	2020-09-28	53.0566	53.0566	53.0566	53.0566	195	{}	\N	2020-10-12 16:43:02.408758
81943	CROC	2020-09-29	52.1862	52.1862	52.1862	52.1862	84	{}	\N	2020-10-12 16:43:02.415047
81944	CROC	2020-09-30	51.62	51.6702	51.62	51.6702	1159	{}	\N	2020-10-12 16:43:02.420231
81945	CROC	2020-10-01	51.36	51.46	51.3292	51.33	1740	{}	\N	2020-10-12 16:43:02.425135
81946	CROC	2020-10-02	51.69	51.86	51.68	51.73	3241	{}	\N	2020-10-12 16:43:02.430361
81947	CROC	2020-10-05	51.3	51.3632	51.3	51.3632	1401	{}	\N	2020-10-12 16:43:02.435089
81948	CROC	2020-10-06	51.835	52.2464	51.78	52.2464	1547	{}	\N	2020-10-12 16:43:02.439317
81949	CROC	2020-10-07	51.86	51.9798	51.86	51.9798	153	{}	\N	2020-10-12 16:43:02.443937
81950	CROC	2020-10-08	51.6003	51.64	51.6003	51.61	1432	{}	\N	2020-10-12 16:43:02.449034
81951	CROC	2020-10-09	51.25	51.25	50.57	50.57	5212	{}	\N	2020-10-12 16:43:02.453123
81952	CROC	2020-10-12	50.885	50.94	50.885	50.94	4330	{}	\N	2020-10-12 16:43:02.457919
81953	CSV	2020-09-28	21.56	22.23	21.53	21.99	87634	{}	\N	2020-10-12 16:43:04.836296
81954	CSV	2020-09-29	22.08	22.11	21.74	21.82	115705	{}	\N	2020-10-12 16:43:04.846821
81955	CSV	2020-09-30	21.72	22.4	21.69	22.31	136816	{}	\N	2020-10-12 16:43:04.853791
81956	CSV	2020-10-01	22.29	22.55	21.96	22.51	144231	{}	\N	2020-10-12 16:43:04.859263
81957	CSV	2020-10-02	22.3	23.55	22.255	23.5	203657	{}	\N	2020-10-12 16:43:04.864457
81958	CSV	2020-10-05	23.57	24.43	23.57	24.36	286270	{}	\N	2020-10-12 16:43:04.869662
81959	CSV	2020-10-06	24.37	24.48	23.48	23.58	229639	{}	\N	2020-10-12 16:43:04.87405
81960	CSV	2020-10-07	23.77	24.27	23.52	23.57	180049	{}	\N	2020-10-12 16:43:04.878195
81961	CSV	2020-10-08	23.75	23.98	23.7	23.79	167823	{}	\N	2020-10-12 16:43:04.882313
81962	CSV	2020-10-09	23.85	24.37	23.72	24	134809	{}	\N	2020-10-12 16:43:04.886813
81963	CSV	2020-10-12	23.99	24.12	23.82	23.915	23204	{}	\N	2020-10-12 16:43:04.891646
81964	CUT	2020-09-28	27.98	28.11	27.9065	28.09	18812	{}	\N	2020-10-12 16:43:07.018228
81965	CUT	2020-09-29	28.15	28.22	27.98	28.0236	14726	{}	\N	2020-10-12 16:43:07.030347
81966	CUT	2020-09-30	27.96	28.22	27.92	27.92	12916	{}	\N	2020-10-12 16:43:07.037286
81967	CUT	2020-10-01	28.29	28.45	28.19	28.26	12881	{}	\N	2020-10-12 16:43:07.042881
81968	CUT	2020-10-02	27.88	28.49	27.87	28.48	16841	{}	\N	2020-10-12 16:43:07.048283
81969	CUT	2020-10-05	28.64	28.9721	28.64	28.9721	11045	{}	\N	2020-10-12 16:43:07.053339
81970	CUT	2020-10-06	29.02	29.02	28.5484	28.5484	12215	{}	\N	2020-10-12 16:43:07.058217
81971	CUT	2020-10-07	28.85	29.15	28.81	29.125	13108	{}	\N	2020-10-12 16:43:07.062781
81972	CUT	2020-10-08	29.04	29.1776	28.89	29.1776	14800	{}	\N	2020-10-12 16:43:07.067097
81973	CUT	2020-10-09	29.45	29.56	29.2651	29.2651	12281	{}	\N	2020-10-12 16:43:07.072499
81974	CUT	2020-10-12	29.49	29.8451	29.49	29.8451	10323	{}	\N	2020-10-12 16:43:07.077186
81975	ATV	2020-09-28	14.6	14.6	14.32	14.32	1461	{}	\N	2020-10-12 16:43:08.288204
81976	ATV	2020-09-29	14.12	14.37	14.12	14.25	3948	{}	\N	2020-10-12 16:43:08.299087
81977	ATV	2020-09-30	14.61	14.8	14.61	14.8	3552	{}	\N	2020-10-12 16:43:08.307243
81978	ATV	2020-10-01	14.5	14.95	14.15	14.545	4010	{}	\N	2020-10-12 16:43:08.313012
81979	ATV	2020-10-02	14.2	14.95	14.2	14.75	8336	{}	\N	2020-10-12 16:43:08.318267
81980	ATV	2020-10-05	14.9	14.95	14.9	14.9401	6004	{}	\N	2020-10-12 16:43:08.323508
81981	ATV	2020-10-06	14.95	14.95	14.63	14.95	11520	{}	\N	2020-10-12 16:43:08.327971
81982	ATV	2020-10-07	14.6	14.775	14.38	14.71	9066	{}	\N	2020-10-12 16:43:08.331909
81983	ATV	2020-10-08	14.73	15.26	14.7	14.73	21824	{}	\N	2020-10-12 16:43:08.336127
81984	ATV	2020-10-09	14.99	15.075	14.815	15.075	9898	{}	\N	2020-10-12 16:43:08.340568
81985	ATV	2020-10-12	20.11	21	20.1	20.4983	197176	{}	\N	2020-10-12 16:43:08.344756
81986	BIL	2020-09-28	91.53	91.54	91.52	91.53	4105797	{}	\N	2020-10-12 16:43:09.798363
81987	BIL	2020-09-29	91.52	91.53	91.52	91.53	2012943	{}	\N	2020-10-12 16:43:09.803602
81988	BIL	2020-09-30	91.53	91.53	91.52	91.52	2522422	{}	\N	2020-10-12 16:43:09.809939
81989	BIL	2020-10-01	91.52	91.54	91.5113	91.52	1042566	{}	\N	2020-10-12 16:43:09.815504
81990	BIL	2020-10-02	91.53	91.53	91.52	91.52	2057818	{}	\N	2020-10-12 16:43:09.821002
81991	BIL	2020-10-05	91.53	91.53	91.52	91.52	1295096	{}	\N	2020-10-12 16:43:09.82647
81992	BIL	2020-10-06	91.52	91.53	91.52	91.53	2473710	{}	\N	2020-10-12 16:43:09.83208
81993	BIL	2020-10-07	91.52	91.53	91.52	91.52	961865	{}	\N	2020-10-12 16:43:09.83699
81994	BIL	2020-10-08	91.52	91.53	91.52	91.52	836075	{}	\N	2020-10-12 16:43:09.842404
81995	BIL	2020-10-09	91.52	91.53	91.52	91.52	975227	{}	\N	2020-10-12 16:43:09.847457
81996	BIL	2020-10-12	91.52	91.53	91.52	91.525	1469424	{}	\N	2020-10-12 16:43:09.852819
81997	BITA	2020-09-28	15.74	15.8	15.74	15.76	404101	{}	\N	2020-10-12 16:43:10.956244
81998	BITA	2020-09-29	15.77	15.81	15.75	15.77	729176	{}	\N	2020-10-12 16:43:10.968111
81999	BITA	2020-09-30	15.8	15.82	15.75	15.75	870047	{}	\N	2020-10-12 16:43:10.975814
82000	BITA	2020-10-01	15.78	15.81	15.77	15.77	935073	{}	\N	2020-10-12 16:43:10.98201
82001	BITA	2020-10-02	15.79	15.79	15.76	15.76	1485036	{}	\N	2020-10-12 16:43:10.987651
82002	BITA	2020-10-05	15.78	15.79	15.76	15.76	1423342	{}	\N	2020-10-12 16:43:10.993406
82003	BITA	2020-10-06	15.78	15.82	15.76	15.76	1785880	{}	\N	2020-10-12 16:43:10.998093
82004	BITA	2020-10-07	15.78	15.81	15.78	15.8	917789	{}	\N	2020-10-12 16:43:11.002992
82005	BITA	2020-10-08	15.8	15.85	15.79	15.81	1042808	{}	\N	2020-10-12 16:43:11.008456
82006	BITA	2020-10-09	15.84	15.86	15.81	15.82	692758	{}	\N	2020-10-12 16:43:11.013942
82007	BITA	2020-10-12	15.82	15.88	15.82	15.855	1019121	{}	\N	2020-10-12 16:43:11.019629
82008	BOIL	2020-09-28	40.12	42.9	39.695	41.82	776595	{}	\N	2020-10-12 16:43:13.851988
82009	BOIL	2020-09-29	36.87	37.51	33.7891	33.91	3147275	{}	\N	2020-10-12 16:43:13.865292
82010	BOIL	2020-09-30	34.67	35.75	33.09	34.63	1183798	{}	\N	2020-10-12 16:43:13.873769
82011	BOIL	2020-10-01	34.46	34.77	32.055	33.57	2059571	{}	\N	2020-10-12 16:43:13.880393
82012	BOIL	2020-10-02	32.76	35.0358	31.8	32.59	1824157	{}	\N	2020-10-12 16:43:13.885277
82013	BOIL	2020-10-05	37.4	39.4799	36.47	37.4	2551715	{}	\N	2020-10-12 16:43:13.890432
82014	BOIL	2020-10-06	35.58	37.298	33.55	34.04	2088648	{}	\N	2020-10-12 16:43:13.895465
82015	BOIL	2020-10-07	37.7	38.08	34.445	36.2	1724387	{}	\N	2020-10-12 16:43:13.900075
82016	BOIL	2020-10-08	34.97	37.16	34.44	36.7	1993944	{}	\N	2020-10-12 16:43:13.905118
82017	BOIL	2020-10-09	39.83	40.55	38.6	39.15	1532173	{}	\N	2020-10-12 16:43:13.910419
82018	BOIL	2020-10-12	41.9399	42.14	40.4408	41.09	876617	{}	\N	2020-10-12 16:43:13.915401
82019	BRX	2020-09-28	11.74	12.12	11.68	11.98	2385974	{}	\N	2020-10-12 16:43:14.922562
82020	BRX	2020-09-29	11.93	11.93	11.43	11.66	3176430	{}	\N	2020-10-12 16:43:14.931046
82021	BRX	2020-09-30	11.8	12.05	11.5	11.69	5697772	{}	\N	2020-10-12 16:43:14.93903
82022	BRX	2020-10-01	11.69	12.1	11.68	12.09	5125144	{}	\N	2020-10-12 16:43:14.947052
82023	BRX	2020-10-02	11.95	12.48	11.61	12.46	5349410	{}	\N	2020-10-12 16:43:14.954888
82024	BRX	2020-10-05	12.6	12.65	12.24	12.5	3937524	{}	\N	2020-10-12 16:43:14.962858
82025	BRX	2020-10-06	12.65	12.98	12.42	12.6	6084305	{}	\N	2020-10-12 16:43:14.970605
82026	BRX	2020-10-07	12.73	12.75	12.36	12.43	3985754	{}	\N	2020-10-12 16:43:14.978408
82027	BRX	2020-10-08	12.57	12.685	12.445	12.63	8631644	{}	\N	2020-10-12 16:43:14.985909
82028	BRX	2020-10-09	12.72	12.82	12.32	12.38	3530723	{}	\N	2020-10-12 16:43:14.991992
82029	BRX	2020-10-12	12.36	12.36	12.05	12.245	1314704	{}	\N	2020-10-12 16:43:14.997671
82030	BWZ	2020-09-28	31.23	31.4277	31.2001	31.285	6016	{}	\N	2020-10-12 16:43:16.92985
82031	BWZ	2020-09-29	31.3667	31.4	31.34	31.4	17172	{}	\N	2020-10-12 16:43:16.93827
82032	BWZ	2020-09-30	31.37	31.48	31.35	31.395	10116	{}	\N	2020-10-12 16:43:16.947129
82033	BWZ	2020-10-01	31.395	31.46	31.37	31.46	31397	{}	\N	2020-10-12 16:43:16.956101
82034	BWZ	2020-10-02	31.4134	31.48	31.37	31.425	54032	{}	\N	2020-10-12 16:43:16.964603
82035	BWZ	2020-10-05	31.45	31.55	31.45	31.53	15138	{}	\N	2020-10-12 16:43:16.973065
82036	BWZ	2020-10-06	31.53	31.56	31.45	31.5	33225	{}	\N	2020-10-12 16:43:16.981874
82037	BWZ	2020-10-07	31.5144	31.524	31.46	31.4766	4969	{}	\N	2020-10-12 16:43:16.990608
82038	BWZ	2020-10-08	31.48	31.534	31.4501	31.49	21757	{}	\N	2020-10-12 16:43:16.999741
82039	BWZ	2020-10-09	31.63	31.7246	31.63	31.68	6119	{}	\N	2020-10-12 16:43:17.008896
82040	BWZ	2020-10-12	31.72	31.72	31.69	31.69	1999	{}	\N	2020-10-12 16:43:17.017165
82041	CATH	2020-09-28	41.25	41.382	41.1939	41.3395	25962	{}	\N	2020-10-12 16:43:19.023573
82042	CATH	2020-09-29	41.3	41.38	41.05	41.13	24780	{}	\N	2020-10-12 16:43:19.037138
82043	CATH	2020-09-30	41.53	41.7897	41.26	41.5499	10870	{}	\N	2020-10-12 16:43:19.044945
82044	CATH	2020-10-01	41.82	41.82	41.43	41.56	9796	{}	\N	2020-10-12 16:43:19.051594
82045	CATH	2020-10-02	40.93	41.49	40.93	41.36	8196	{}	\N	2020-10-12 16:43:19.057114
82046	CATH	2020-10-05	41.55	42.01	41.55	42.005	23073	{}	\N	2020-10-12 16:43:19.06157
82047	CATH	2020-10-06	42.04	42.19	41.39	41.5204	8358	{}	\N	2020-10-12 16:43:19.066789
82048	CATH	2020-10-07	42.09	42.369	42	42.26	117408	{}	\N	2020-10-12 16:43:19.073183
82049	CATH	2020-10-08	42.46	42.55	42.4204	42.54	8680	{}	\N	2020-10-12 16:43:19.0772
82050	CATH	2020-10-09	42.95	42.98	42.745	42.9063	4011	{}	\N	2020-10-12 16:43:19.081817
82051	CATH	2020-10-12	43.41	43.46	43.09	43.46	1542	{}	\N	2020-10-12 16:43:19.086787
82052	CHMG	2020-09-28	28.41	28.8394	27.89	28.25	3655	{}	\N	2020-10-12 16:43:20.690126
82053	CHMG	2020-09-29	28.58	29.08	28.52	28.9	2285	{}	\N	2020-10-12 16:43:20.699222
82054	CHMG	2020-09-30	28.99	29.2299	27.88	28.87	4626	{}	\N	2020-10-12 16:43:20.705012
82055	CHMG	2020-10-01	29.245	29.245	28.37	29.15	8663	{}	\N	2020-10-12 16:43:20.711729
82056	CHMG	2020-10-02	28.4	30.18	28.4	30	5078	{}	\N	2020-10-12 16:43:20.717076
82057	CHMG	2020-10-05	29.95	30.64	29.507	30.64	6927	{}	\N	2020-10-12 16:43:20.722608
82058	CHMG	2020-10-06	30.18	31.51	30	30.29	10490	{}	\N	2020-10-12 16:43:20.728388
82059	CHMG	2020-10-07	30.31	31.5	30.31	31.5	8026	{}	\N	2020-10-12 16:43:20.734071
82060	CHMG	2020-10-08	31.5	32.25	31.25	31.9	5507	{}	\N	2020-10-12 16:43:20.740045
82061	CHMG	2020-10-09	32.24	32.7	32.24	32.7	2261	{}	\N	2020-10-12 16:43:20.7456
82062	CHMG	2020-10-12	32.5	33.345	32.5	33.345	1531	{}	\N	2020-10-12 16:43:20.752048
82063	CLS	2020-09-28	6.87	7.04	6.84	6.99	271599	{}	\N	2020-10-12 16:43:21.636544
82064	CLS	2020-09-29	6.99	7.05	6.91	6.95	1095959	{}	\N	2020-10-12 16:43:21.646355
82065	CLS	2020-09-30	6.92	7.015	6.875	6.9	622209	{}	\N	2020-10-12 16:43:21.653311
82066	CLS	2020-10-01	6.89	7.01	6.79	6.94	608714	{}	\N	2020-10-12 16:43:21.659753
82067	CLS	2020-10-02	6.79	7.14	6.78	7.06	696184	{}	\N	2020-10-12 16:43:21.666582
82068	CLS	2020-10-05	7.15	7.33	7.15	7.24	722266	{}	\N	2020-10-12 16:43:21.671267
82069	CLS	2020-10-06	7.27	7.43	7.18	7.2	615721	{}	\N	2020-10-12 16:43:21.676744
82070	CLS	2020-10-07	7.31	7.42	7.255	7.39	493974	{}	\N	2020-10-12 16:43:21.681799
82071	CLS	2020-10-08	7.47	7.64	7.42	7.61	431509	{}	\N	2020-10-12 16:43:21.686776
82072	CLS	2020-10-09	7.65	7.72	7.3	7.31	405017	{}	\N	2020-10-12 16:43:21.692239
82073	CLS	2020-10-12	7.32	7.4	7.315	7.32	73385	{}	\N	2020-10-12 16:43:21.698114
82074	CNX	2020-09-28	10.33	10.49	9.83	10.22	6054733	{}	\N	2020-10-12 16:43:22.591793
82075	CNX	2020-09-29	10.12	10.17	9.38	9.64	17411832	{}	\N	2020-10-12 16:43:22.607807
82076	CNX	2020-09-30	9.58	9.85	9.31	9.44	13777604	{}	\N	2020-10-12 16:43:22.620314
82077	CNX	2020-10-01	9.31	9.505	9.13	9.32	10736823	{}	\N	2020-10-12 16:43:22.628457
82078	CNX	2020-10-02	9.1	9.45	9.03	9.3	6471149	{}	\N	2020-10-12 16:43:22.636803
82079	CNX	2020-10-05	9.56	9.88	9.28	9.8	12499296	{}	\N	2020-10-12 16:43:22.64581
82080	CNX	2020-10-06	9.87	10.17	9.59	9.65	8034651	{}	\N	2020-10-12 16:43:22.654696
82081	CNX	2020-10-07	9.74	10.315	9.74	10.27	9494127	{}	\N	2020-10-12 16:43:22.661249
82082	CNX	2020-10-08	10.34	10.49	10.12	10.49	5894546	{}	\N	2020-10-12 16:43:22.667441
82083	CNX	2020-10-09	10.75	10.97	10.33	10.63	9364995	{}	\N	2020-10-12 16:43:22.672938
82084	CNX	2020-10-12	10.65	10.89	10.545	10.72	2592864	{}	\N	2020-10-12 16:43:22.678938
82085	CORR	2020-09-28	6.01	6.29	5.9882	6.21	242522	{}	\N	2020-10-12 16:43:24.430029
82086	CORR	2020-09-29	6.12	6.12	5.8045	6.05	433213	{}	\N	2020-10-12 16:43:24.43884
82087	CORR	2020-09-30	6.04	6.2675	5.785	5.84	400151	{}	\N	2020-10-12 16:43:24.444641
82088	CORR	2020-10-01	5.91	6.0831	5.82	6.03	383794	{}	\N	2020-10-12 16:43:24.452444
82089	CORR	2020-10-02	5.83	5.98	5.735	5.89	349027	{}	\N	2020-10-12 16:43:24.458439
82090	CORR	2020-10-05	6	6.0049	5.78	5.87	395606	{}	\N	2020-10-12 16:43:24.464231
82091	CORR	2020-10-06	5.88	6.13	5.75	5.85	471980	{}	\N	2020-10-12 16:43:24.47
82092	CORR	2020-10-07	5.87	6.09	5.82	5.87	492161	{}	\N	2020-10-12 16:43:24.475368
82093	CORR	2020-10-08	5.89	6.34	5.88	6.3	567565	{}	\N	2020-10-12 16:43:24.480826
82094	CORR	2020-10-09	6.35	6.49	6.01	6.04	749361	{}	\N	2020-10-12 16:43:24.486483
82095	CORR	2020-10-12	6.08	6.08	5.86	5.86	278610	{}	\N	2020-10-12 16:43:24.491876
82096	BKH	2020-09-28	53.78	54.505	53.56	53.92	504919	{}	\N	2020-10-12 16:43:27.221353
82097	BKH	2020-09-29	54.1	54.4	52.84	53.11	798917	{}	\N	2020-10-12 16:43:27.230421
82098	BKH	2020-09-30	53.44	53.8126	52.99	53.49	600543	{}	\N	2020-10-12 16:43:27.235919
82099	BKH	2020-10-01	53.41	54.05	52.84	53.95	815441	{}	\N	2020-10-12 16:43:27.24243
82100	BKH	2020-10-02	53.11	55.56	53.11	55.34	672557	{}	\N	2020-10-12 16:43:27.247387
82101	BKH	2020-10-05	55.79	55.905	54.81	55.85	582101	{}	\N	2020-10-12 16:43:27.251906
82102	BKH	2020-10-06	56.25	57.83	56.02	56.46	793577	{}	\N	2020-10-12 16:43:27.256817
82103	BKH	2020-10-07	56.81	57.6	56.55	57.44	757962	{}	\N	2020-10-12 16:43:27.262288
82104	BKH	2020-10-08	57.78	58.57	57.6	58.51	576767	{}	\N	2020-10-12 16:43:27.267723
82105	BKH	2020-10-09	58.8	58.84	57.52	57.79	665433	{}	\N	2020-10-12 16:43:27.272564
82106	BKH	2020-10-12	57.52	58.87	57.38	58.505	171229	{}	\N	2020-10-12 16:43:27.277866
82107	CARS	2020-09-28	7.73	8.12	7.73	8.04	898291	{}	\N	2020-10-12 16:43:30.734858
82108	CARS	2020-09-29	8.04	8.195	7.9066	8.12	1527720	{}	\N	2020-10-12 16:43:30.744576
82109	CARS	2020-09-30	8.11	8.4	8.035	8.08	1870119	{}	\N	2020-10-12 16:43:30.753496
82110	CARS	2020-10-01	8.18	8.79	8.12	8.78	2210249	{}	\N	2020-10-12 16:43:30.763493
82111	CARS	2020-10-02	8.42	8.72	8.24	8.61	1572214	{}	\N	2020-10-12 16:43:30.770179
82112	CARS	2020-10-05	8.7	8.9914	8.6701	8.93	1025480	{}	\N	2020-10-12 16:43:30.775343
82113	CARS	2020-10-06	9.1	9.16	8.69	8.7	1478649	{}	\N	2020-10-12 16:43:30.780262
82114	CARS	2020-10-07	8.81	8.95	8.65	8.75	1278534	{}	\N	2020-10-12 16:43:30.785699
82115	CARS	2020-10-08	8.92	9.03	8.76	8.91	801564	{}	\N	2020-10-12 16:43:30.790239
82116	CARS	2020-10-09	9	9.05	8.57	8.66	1256376	{}	\N	2020-10-12 16:43:30.795166
82117	CARS	2020-10-12	8.73	8.86	8.529	8.56	563180	{}	\N	2020-10-12 16:43:30.800708
82118	CCOI	2020-09-28	59.74	61.15	59.65	60.91	576746	{}	\N	2020-10-12 16:43:32.912805
82119	CCOI	2020-09-29	60.68	61.04	60.44	60.91	288447	{}	\N	2020-10-12 16:43:32.923603
82120	CCOI	2020-09-30	61.08	61.08	59.82	60.05	409408	{}	\N	2020-10-12 16:43:32.93219
82121	CCOI	2020-10-01	60.21	60.57	59.92	60.49	242354	{}	\N	2020-10-12 16:43:32.93912
82122	CCOI	2020-10-02	59.83	60.75	59	60.4	277018	{}	\N	2020-10-12 16:43:32.945472
82123	CCOI	2020-10-05	62.11	63.19	61.05	63.13	378212	{}	\N	2020-10-12 16:43:32.952084
82124	CCOI	2020-10-06	63.13	63.7	61.25	61.65	326134	{}	\N	2020-10-12 16:43:32.958355
82125	CCOI	2020-10-07	61.75	62.66	61.04	62.29	361148	{}	\N	2020-10-12 16:43:32.964337
82126	CCOI	2020-10-08	62.94	63.47	61.95	62.23	344131	{}	\N	2020-10-12 16:43:32.970013
82127	CCOI	2020-10-09	61.73	62.95	61.73	62.73	346539	{}	\N	2020-10-12 16:43:32.976676
82128	CCOI	2020-10-12	63	64.08	62.74	63.47	142713	{}	\N	2020-10-12 16:43:32.98308
82129	CERS	2020-09-28	6.15	6.215	6.05	6.13	1466476	{}	\N	2020-10-12 16:43:33.975688
82130	CERS	2020-09-29	6.13	6.4	6.08	6.355	1449947	{}	\N	2020-10-12 16:43:33.987195
82131	CERS	2020-09-30	6.35	6.4	6.195	6.26	841510	{}	\N	2020-10-12 16:43:33.99605
82132	CERS	2020-10-01	6.29	6.35	6.14	6.16	1357664	{}	\N	2020-10-12 16:43:34.002433
82133	CERS	2020-10-02	6	6.12	5.91	6.04	1103799	{}	\N	2020-10-12 16:43:34.008961
82134	CERS	2020-10-05	6.1	6.3	6.075	6.25	935998	{}	\N	2020-10-12 16:43:34.014326
82135	CERS	2020-10-06	6.28	6.62	6.2337	6.36	1141731	{}	\N	2020-10-12 16:43:34.019465
82136	CERS	2020-10-07	6.42	6.735	6.42	6.555	1822825	{}	\N	2020-10-12 16:43:34.025624
82137	CERS	2020-10-08	6.66	6.76	6.55	6.58	1385408	{}	\N	2020-10-12 16:43:34.032305
82138	CERS	2020-10-09	6.6	6.8701	6.58	6.82	1483660	{}	\N	2020-10-12 16:43:34.037258
82139	CERS	2020-10-12	6.93	7	6.785	6.8101	569176	{}	\N	2020-10-12 16:43:34.043321
82140	CN	2020-09-28	39.58	39.708	39.58	39.708	561	{}	\N	2020-10-12 16:43:35.781406
82141	CN	2020-09-29	39.72	39.87	39.72	39.87	300	{}	\N	2020-10-12 16:43:35.791769
82142	CN	2020-09-30	40.43	40.534	40.43	40.534	334	{}	\N	2020-10-12 16:43:35.800403
82143	CN	2020-10-01	40.75	40.9429	40.75	40.9429	607	{}	\N	2020-10-12 16:43:35.806633
82144	CN	2020-10-02	40.77	40.77	40.4174	40.4174	1163	{}	\N	2020-10-12 16:43:35.81324
82145	CN	2020-10-05	40.49	40.69	40.49	40.69	1156	{}	\N	2020-10-12 16:43:35.818962
82146	CN	2020-10-06	41.13	41.13	41.0091	41.0091	516	{}	\N	2020-10-12 16:43:35.82432
82147	CN	2020-10-07	41.42	41.44	41.31	41.31	1889	{}	\N	2020-10-12 16:43:35.830508
82148	CN	2020-10-08	41.45	41.58	41.4	41.5719	839	{}	\N	2020-10-12 16:43:35.838203
82149	CN	2020-10-09	41.85	41.94	41.8	41.91	1125	{}	\N	2020-10-12 16:43:35.84399
82150	CN	2020-10-12	42.5	42.58	42.5	42.58	1333	{}	\N	2020-10-12 16:43:35.850075
82151	CWI	2020-09-28	23.99	24.06	23.97	24.05	94000	{}	\N	2020-10-12 16:43:40.268376
82152	CWI	2020-09-29	24.04	24.0996	23.94	24	93773	{}	\N	2020-10-12 16:43:40.277892
82153	CWI	2020-09-30	24.03	24.18	23.97	24.05	48411	{}	\N	2020-10-12 16:43:40.284548
82154	CWI	2020-10-01	24.19	24.21	24.07	24.19	65496	{}	\N	2020-10-12 16:43:40.290469
82155	CWI	2020-10-02	23.91	24.16	23.91	24.08	93488	{}	\N	2020-10-12 16:43:40.296586
82156	CWI	2020-10-05	24.25	24.47	24.25	24.47	56065	{}	\N	2020-10-12 16:43:40.301695
82157	CWI	2020-10-06	24.47	24.535	24.2	24.26	91762	{}	\N	2020-10-12 16:43:40.307046
82158	CWI	2020-10-07	24.47	24.5699	24.45	24.5	146317	{}	\N	2020-10-12 16:43:40.313233
82159	CWI	2020-10-08	24.6	24.7	24.6	24.69	46534	{}	\N	2020-10-12 16:43:40.318367
82160	CWI	2020-10-09	24.79	24.89	24.76	24.86	34334	{}	\N	2020-10-12 16:43:40.32417
82161	CWI	2020-10-12	24.93	25.035	24.93	25.031	65241	{}	\N	2020-10-12 16:43:40.330378
82162	DCF	2020-09-28	8.02	8.0463	7.86	7.97	53941	{}	\N	2020-10-12 16:43:42.280489
82163	DCF	2020-09-29	7.92	8	7.91	7.91	27772	{}	\N	2020-10-12 16:43:42.291089
82164	DCF	2020-09-30	7.93	8.04	7.93	8.04	25298	{}	\N	2020-10-12 16:43:42.298121
82165	DCF	2020-10-01	7.98	8.05	7.9584	8.0099	23840	{}	\N	2020-10-12 16:43:42.303629
82166	DCF	2020-10-02	8	8.01	7.88	7.88	43602	{}	\N	2020-10-12 16:43:42.308389
82167	DCF	2020-10-05	7.98	7.98	7.874	7.9	58650	{}	\N	2020-10-12 16:43:42.312856
82168	DCF	2020-10-06	7.98	8	7.95	7.95	27368	{}	\N	2020-10-12 16:43:42.31813
82169	DCF	2020-10-07	8.01	8.04	7.94	7.96	78520	{}	\N	2020-10-12 16:43:42.322575
82170	DCF	2020-10-08	7.96	8.04	7.96	7.98	50332	{}	\N	2020-10-12 16:43:42.32682
82171	DCF	2020-10-09	8.14	8.14	8.0376	8.08	15482	{}	\N	2020-10-12 16:43:42.331762
82172	DCF	2020-10-12	8.04	8.0994	8.04	8.075	18670	{}	\N	2020-10-12 16:43:42.336276
82173	DGP	2020-09-28	43.86	44.67	43.7881	44.62	39623	{}	\N	2020-10-12 16:43:44.435855
82174	DGP	2020-09-29	44.73	45.7699	44.5501	45.57	38424	{}	\N	2020-10-12 16:43:44.446891
82175	DGP	2020-09-30	45.02	45.7699	44.5	44.86	22536	{}	\N	2020-10-12 16:43:44.453802
82176	DGP	2020-10-01	45.77	46.53	45.28	45.95	27032	{}	\N	2020-10-12 16:43:44.460125
82177	DGP	2020-10-02	45.95	46.12	45.42	45.95	34860	{}	\N	2020-10-12 16:43:44.465658
82178	DGP	2020-10-05	45.93	46.5499	45.9	46.37	29394	{}	\N	2020-10-12 16:43:44.471895
82179	DGP	2020-10-06	46.28	46.64	45	45	15839	{}	\N	2020-10-12 16:43:44.478081
82180	DGP	2020-10-07	45	45.3119	44.7	44.87	11503	{}	\N	2020-10-12 16:43:44.483296
82181	DGP	2020-10-08	45.1	45.45	44.95	45.45	8042	{}	\N	2020-10-12 16:43:44.489032
82182	DGP	2020-10-09	46.68	47.2	46.5	47.07	29109	{}	\N	2020-10-12 16:43:44.494032
82183	DGP	2020-10-12	46.8	46.8	46.39	46.695	12618	{}	\N	2020-10-12 16:43:44.499053
82184	DIAX	2020-09-28	13.33	13.45	13.26	13.32	214071	{}	\N	2020-10-12 16:43:45.565595
82185	DIAX	2020-09-29	13.4	13.42	13.25	13.34	257252	{}	\N	2020-10-12 16:43:45.575798
82186	DIAX	2020-09-30	13.32	13.56	13.32	13.53	420395	{}	\N	2020-10-12 16:43:45.585126
82187	DIAX	2020-10-01	13.72	13.76	13.6	13.7	562839	{}	\N	2020-10-12 16:43:45.593017
82188	DIAX	2020-10-02	13.6	13.61	13.5	13.57	225320	{}	\N	2020-10-12 16:43:45.600945
82189	DIAX	2020-10-05	13.58	13.73	13.58	13.68	180394	{}	\N	2020-10-12 16:43:45.606899
82190	DIAX	2020-10-06	13.75	13.83	13.56	13.58	222934	{}	\N	2020-10-12 16:43:45.612461
82191	DIAX	2020-10-07	13.65	13.79	13.65	13.77	282938	{}	\N	2020-10-12 16:43:45.617583
82192	DIAX	2020-10-08	13.85	13.92	13.78	13.88	151248	{}	\N	2020-10-12 16:43:45.623032
82193	DIAX	2020-10-09	13.92	13.92	13.82	13.84	200526	{}	\N	2020-10-12 16:43:45.628579
82194	DIAX	2020-10-12	13.92	13.93	13.821	13.92	138492	{}	\N	2020-10-12 16:43:45.634374
82195	DIT	2020-09-28	65.48	65.48	62.25	62.25	61	{}	\N	2020-10-12 16:43:46.981518
82196	DIT	2020-09-29	63.2	63.2	63.2	63.2	66	{}	\N	2020-10-12 16:43:46.991405
82197	DIT	2020-09-30	63.2027	64.59	63.2	64.59	252	{}	\N	2020-10-12 16:43:47.00171
82198	DIT	2020-10-01	64	64	62.7	62.7	189	{}	\N	2020-10-12 16:43:47.011441
82199	DIT	2020-10-05	62.31	62.31	62.31	62.31	19	{}	\N	2020-10-12 16:43:47.019515
82200	DIT	2020-10-06	63	63	62	62	116	{}	\N	2020-10-12 16:43:47.028466
82201	DIT	2020-10-08	62.5	62.5	62	62	226	{}	\N	2020-10-12 16:43:47.034826
82202	DIT	2020-10-12	61.9999	62	61.185	62	2071	{}	\N	2020-10-12 16:43:47.041067
82203	BH	2020-09-28	90.635	94.58	89.96	92.13	4584	{}	\N	2020-10-12 16:43:48.317568
82204	BH	2020-09-29	91.9457	93.36	89	90.82	7056	{}	\N	2020-10-12 16:43:48.324349
82205	BH	2020-09-30	90.3	94.59	88.84	89.01	12334	{}	\N	2020-10-12 16:43:48.329888
82206	BH	2020-10-01	89.99	93.38	87.01	91.42	12120	{}	\N	2020-10-12 16:43:48.335425
82207	BH	2020-10-02	89.19	92.65	88.94	90.83	2895	{}	\N	2020-10-12 16:43:48.340713
82208	BH	2020-10-05	91.82	92.9	88.9	90.05	6221	{}	\N	2020-10-12 16:43:48.346317
82209	BH	2020-10-06	90.91	94	87.23	87.64	8930	{}	\N	2020-10-12 16:43:48.351963
82210	BH	2020-10-07	88.7	92.88	88.61	91.21	8465	{}	\N	2020-10-12 16:43:48.357582
82211	BH	2020-10-08	92.44	93.2	90.12	91.2	6150	{}	\N	2020-10-12 16:43:48.364118
82212	BH	2020-10-09	91.88	92.84	90.3476	90.36	4226	{}	\N	2020-10-12 16:43:48.371458
82213	BH	2020-10-12	92.45	92.45	88.505	89.26	3977	{}	\N	2020-10-12 16:43:48.37828
82214	BIB	2020-09-28	69.5	69.5	67.8346	69.13	46690	{}	\N	2020-10-12 16:43:49.388434
82215	BIB	2020-09-29	69.18	70	68.46	69.3377	29185	{}	\N	2020-10-12 16:43:49.397246
82216	BIB	2020-09-30	69.64	71.31	69.11	69.96	64666	{}	\N	2020-10-12 16:43:49.405801
82217	BIB	2020-10-01	70.67	71.51	70	71.23	79903	{}	\N	2020-10-12 16:43:49.416018
82218	BIB	2020-10-02	69.09	71.15	67.73	68.37	80196	{}	\N	2020-10-12 16:43:49.425314
82219	BIB	2020-10-05	70.85	74.38	70.76	74.19	162679	{}	\N	2020-10-12 16:43:49.433038
82220	BIB	2020-10-06	74.37	75.03	72.2754	72.57	70922	{}	\N	2020-10-12 16:43:49.440019
82221	BIB	2020-10-07	73.52	76.18	73.33	75.61	90356	{}	\N	2020-10-12 16:43:49.446607
82222	BIB	2020-10-08	76.58	76.58	75.43	76.0551	35232	{}	\N	2020-10-12 16:43:49.453096
82223	BIB	2020-10-09	76.88	77.34	75.98	77.34	64754	{}	\N	2020-10-12 16:43:49.46032
82224	BIB	2020-10-12	78.59	78.66	76.78	78.2217	42735	{}	\N	2020-10-12 16:43:49.468384
82225	BKLN	2020-09-28	21.67	21.75	21.67	21.73	14127154	{}	\N	2020-10-12 16:43:50.437692
82226	BKLN	2020-09-29	21.74	21.75	21.68	21.72	4863388	{}	\N	2020-10-12 16:43:50.44927
82227	BKLN	2020-09-30	21.73	21.83	21.73	21.74	12384324	{}	\N	2020-10-12 16:43:50.45513
82228	BKLN	2020-10-01	21.77	21.82	21.73	21.73	10909966	{}	\N	2020-10-12 16:43:50.460318
82229	BKLN	2020-10-02	21.7	21.77	21.7	21.74	4543092	{}	\N	2020-10-12 16:43:50.46548
82230	BKLN	2020-10-05	21.81	21.88	21.8	21.83	13473558	{}	\N	2020-10-12 16:43:50.47152
82231	BKLN	2020-10-06	21.83	21.8581	21.78	21.79	8530427	{}	\N	2020-10-12 16:43:50.476365
82232	BKLN	2020-10-07	21.86	21.87	21.82	21.84	3517565	{}	\N	2020-10-12 16:43:50.481552
82233	BKLN	2020-10-08	21.84	21.905	21.83	21.84	3677203	{}	\N	2020-10-12 16:43:50.488524
82234	BKLN	2020-10-09	21.81	21.87	21.81	21.84	3884325	{}	\N	2020-10-12 16:43:50.494089
82235	BKLN	2020-10-12	21.86	21.93	21.8476	21.925	876502	{}	\N	2020-10-12 16:43:50.499545
82236	BLBD	2020-09-28	11.56	11.7	11.39	11.41	84795	{}	\N	2020-10-12 16:43:51.73143
82237	BLBD	2020-09-29	11.43	11.45	11.17	11.41	60100	{}	\N	2020-10-12 16:43:51.744283
82238	BLBD	2020-09-30	11.41	12.23	11.41	12.16	89987	{}	\N	2020-10-12 16:43:51.752641
82239	BLBD	2020-10-01	12.19	12.19	11.7	11.89	264075	{}	\N	2020-10-12 16:43:51.75918
82240	BLBD	2020-10-02	11.75	12.14	11.7	12.06	56773	{}	\N	2020-10-12 16:43:51.765469
82241	BLBD	2020-10-05	12.15	12.88	12.15	12.76	200528	{}	\N	2020-10-12 16:43:51.770811
82242	BLBD	2020-10-06	12.84	13.36	12.1866	12.48	198018	{}	\N	2020-10-12 16:43:51.775733
82243	BLBD	2020-10-07	12.54	12.79	12.4	12.65	75573	{}	\N	2020-10-12 16:43:51.780394
82244	BLBD	2020-10-08	12.75	13.25	12.5	12.77	40862	{}	\N	2020-10-12 16:43:51.785241
82245	BLBD	2020-10-09	12.88	13.17	12.43	12.48	31752	{}	\N	2020-10-12 16:43:51.790247
82246	BLBD	2020-10-12	12.195	12.52	12.195	12.52	14087	{}	\N	2020-10-12 16:43:51.79501
82247	BNO	2020-09-28	10.72	10.885	10.662	10.85	415012	{}	\N	2020-10-12 16:43:52.750009
82248	BNO	2020-09-29	10.7	10.71	10.34	10.45	743787	{}	\N	2020-10-12 16:43:52.759294
82249	BNO	2020-09-30	10.51	10.74	10.5011	10.65	430029	{}	\N	2020-10-12 16:43:52.766167
82250	BNO	2020-10-01	10.42	10.47	10.11	10.34	679424	{}	\N	2020-10-12 16:43:52.772049
82251	BNO	2020-10-02	9.86	10.09	9.82	9.91	1365737	{}	\N	2020-10-12 16:43:52.777502
82252	BNO	2020-10-05	10.37	10.565	10.33	10.49	1066656	{}	\N	2020-10-12 16:43:52.782283
82253	BNO	2020-10-06	10.76	10.83	10.6281	10.66	872032	{}	\N	2020-10-12 16:43:52.787034
82254	BNO	2020-10-07	10.59	10.66	10.47	10.65	477329	{}	\N	2020-10-12 16:43:52.791751
82255	BNO	2020-10-08	10.88	10.9871	10.81	10.98	656425	{}	\N	2020-10-12 16:43:52.796005
82256	BNO	2020-10-09	10.92	11.01	10.78	10.81	277284	{}	\N	2020-10-12 16:43:52.800597
82257	BNO	2020-10-12	10.7	10.7	10.46	10.53	327983	{}	\N	2020-10-12 16:43:52.806028
82258	BSX	2020-09-28	38.07	38.45	37.9825	38	6525067	{}	\N	2020-10-12 16:43:54.261392
82259	BSX	2020-09-29	38.18	38.26	37.54	37.76	11706103	{}	\N	2020-10-12 16:43:54.27453
82260	BSX	2020-09-30	37.83	38.71	37.73	38.21	14594446	{}	\N	2020-10-12 16:43:54.282391
82261	BSX	2020-10-01	38.55	38.94	38.185	38.42	10905080	{}	\N	2020-10-12 16:43:54.289008
82262	BSX	2020-10-02	37.78	38.66	37.762	38.48	11074892	{}	\N	2020-10-12 16:43:54.295428
82263	BSX	2020-10-05	38.75	39.34	38.66	39.24	8571344	{}	\N	2020-10-12 16:43:54.301186
82264	BSX	2020-10-06	39.8	40.4	39.25	39.33	15473746	{}	\N	2020-10-12 16:43:54.306831
82265	BSX	2020-10-07	39.58	40.435	39.58	40.18	10954679	{}	\N	2020-10-12 16:43:54.312352
82266	BSX	2020-10-08	40.25	40.57	40.19	40.49	8731823	{}	\N	2020-10-12 16:43:54.317635
82267	BSX	2020-10-09	41.04	41.25	40.695	41.14	12146284	{}	\N	2020-10-12 16:43:54.323545
82268	BSX	2020-10-12	41.44	41.86	41.22	41.85	6112947	{}	\N	2020-10-12 16:43:54.329058
82269	CAPE	2020-09-28	155.929	156.6752	155.929	156.3206	2932	{}	\N	2020-10-12 16:43:56.723878
82270	CAPE	2020-09-29	155.745	155.745	154.73	155.745	1275	{}	\N	2020-10-12 16:43:56.729624
82271	CAPE	2020-09-30	156.0045	156.0045	156.0045	156.0045	600	{}	\N	2020-10-12 16:43:56.736372
82272	CAPE	2020-10-01	157.87	157.87	155.0801	155.0801	2161	{}	\N	2020-10-12 16:43:56.741197
82273	CAPE	2020-10-02	152.16	156.3027	152.16	156.3027	1262	{}	\N	2020-10-12 16:43:56.746345
82274	CAPE	2020-10-05	157.94	158.29	157.42	158.29	4668	{}	\N	2020-10-12 16:43:56.750461
82275	CAPE	2020-10-06	158.64	158.6894	156.4315	156.4315	1830	{}	\N	2020-10-12 16:43:56.755422
82276	CAPE	2020-10-07	157.925	159	157.7	158.9	4995	{}	\N	2020-10-12 16:43:56.759578
82277	CAPE	2020-10-08	159.61	159.9512	159.61	159.9512	898	{}	\N	2020-10-12 16:43:56.763548
82278	CAPE	2020-10-09	160.76	161.15	160.6158	160.8864	4126	{}	\N	2020-10-12 16:43:56.767594
82279	CAPE	2020-10-12	162.805	162.805	162.805	162.805	653	{}	\N	2020-10-12 16:43:56.772191
82280	CCOR	2020-09-28	27.735	27.847	27.64	27.64	10660	{}	\N	2020-10-12 16:43:57.722875
82281	CCOR	2020-09-29	27.7	27.72	27.49	27.6	18316	{}	\N	2020-10-12 16:43:57.740209
82282	CCOR	2020-09-30	27.6	27.87	27.57	27.7393	264430	{}	\N	2020-10-12 16:43:57.746889
82283	CCOR	2020-10-01	27.74	27.77	27.54	27.56	141967	{}	\N	2020-10-12 16:43:57.752168
82284	CCOR	2020-10-02	27.27	27.6212	27.27	27.43	15810	{}	\N	2020-10-12 16:43:57.75786
82285	CCOR	2020-10-05	27.73	27.9	27.725	27.88	17274	{}	\N	2020-10-12 16:43:57.761997
82286	CCOR	2020-10-06	27.825	28.12	27.6	27.6	44891	{}	\N	2020-10-12 16:43:57.766217
82287	CCOR	2020-10-07	27.64	28.0081	27.64	28.0081	3449	{}	\N	2020-10-12 16:43:57.770534
82288	CCOR	2020-10-08	28.267	28.31	28.1308	28.165	274396	{}	\N	2020-10-12 16:43:57.77466
82289	CCOR	2020-10-09	28.2235	28.32	28.1501	28.22	61849	{}	\N	2020-10-12 16:43:57.778769
82290	CCOR	2020-10-12	28.1144	28.44	28.1144	28.44	3879	{}	\N	2020-10-12 16:43:57.783065
82291	CHEF	2020-09-28	14.32	14.75	14.17	14.36	546937	{}	\N	2020-10-12 16:43:58.729437
82292	CHEF	2020-09-29	14.25	14.4866	13.59	14.11	677132	{}	\N	2020-10-12 16:43:58.73956
82293	CHEF	2020-09-30	14.16	14.89	14.16	14.54	825007	{}	\N	2020-10-12 16:43:58.747683
82294	CHEF	2020-10-01	14.8	15.62	14.73	15.39	873697	{}	\N	2020-10-12 16:43:58.753798
82295	CHEF	2020-10-02	14.37	15.64	14.3	15.6	726899	{}	\N	2020-10-12 16:43:58.760062
82296	CHEF	2020-10-05	15.8	15.9508	14.78	14.8	1186021	{}	\N	2020-10-12 16:43:58.765322
82297	CHEF	2020-10-06	14.95	15.58	14.85	14.94	592359	{}	\N	2020-10-12 16:43:58.772529
82298	CHEF	2020-10-07	15.19	15.27	14.4	14.86	764105	{}	\N	2020-10-12 16:43:58.778619
82299	CHEF	2020-10-08	15.08	15.67	15.01	15.65	760144	{}	\N	2020-10-12 16:43:58.784034
82300	CHEF	2020-10-09	15.69	15.87	15.17	15.75	818817	{}	\N	2020-10-12 16:43:58.789351
82301	CHEF	2020-10-12	15.71	15.9095	15.28	15.56	195774	{}	\N	2020-10-12 16:43:58.794913
82302	CHT	2020-09-28	36.69	36.9	36.68	36.77	187004	{}	\N	2020-10-12 16:44:00.281629
82303	CHT	2020-09-29	36.92	36.98	36.82	36.94	497564	{}	\N	2020-10-12 16:44:00.291028
82304	CHT	2020-09-30	36.79	36.79	36.22	36.23	711670	{}	\N	2020-10-12 16:44:00.301328
82305	CHT	2020-10-01	36.3	36.46	36.3	36.41	235982	{}	\N	2020-10-12 16:44:00.307398
82306	CHT	2020-10-02	36.27	36.49	36.27	36.4	207694	{}	\N	2020-10-12 16:44:00.313663
82307	CHT	2020-10-05	36.8	36.99	36.77	36.85	320870	{}	\N	2020-10-12 16:44:00.319194
82308	CHT	2020-10-06	36.95	37.12	36.93	36.96	368296	{}	\N	2020-10-12 16:44:00.324145
82309	CHT	2020-10-07	36.68	36.89	36.68	36.82	345810	{}	\N	2020-10-12 16:44:00.329522
82310	CHT	2020-10-08	36.97	37.11	36.82	36.85	191575	{}	\N	2020-10-12 16:44:00.334375
82311	CHT	2020-10-09	36.98	37.05	36.85	37.01	218659	{}	\N	2020-10-12 16:44:00.33897
82312	CHT	2020-10-12	37.03	37.064	36.895	36.97	89752	{}	\N	2020-10-12 16:44:00.343512
82313	CMS	2020-09-28	61.5	62.24	61.17	61.4	1030434	{}	\N	2020-10-12 16:44:01.336422
82314	CMS	2020-09-29	61.63	61.96	61.015	61.11	2045461	{}	\N	2020-10-12 16:44:01.347416
82315	CMS	2020-09-30	61.5	61.94	61.15	61.41	3018389	{}	\N	2020-10-12 16:44:01.355285
82316	CMS	2020-10-01	61.49	61.95	61.19	61.68	3123260	{}	\N	2020-10-12 16:44:01.362259
82317	CMS	2020-10-02	61.32	62.01	60.97	61.66	2623633	{}	\N	2020-10-12 16:44:01.367954
82318	CMS	2020-10-05	61.71	61.9155	60.84	61.58	2935882	{}	\N	2020-10-12 16:44:01.37325
82319	CMS	2020-10-06	61.72	63.39	61.58	62.89	2462027	{}	\N	2020-10-12 16:44:01.378882
82320	CMS	2020-10-07	63.05	63.38	62.49	63.01	2005898	{}	\N	2020-10-12 16:44:01.383954
82321	CMS	2020-10-08	63.07	63.895	62.82	63.87	2243874	{}	\N	2020-10-12 16:44:01.389247
82322	CMS	2020-10-09	63.45	64.83	63.45	64.34	2583255	{}	\N	2020-10-12 16:44:01.394578
82323	CMS	2020-10-12	64.46	65.54	64.21	65.37	1094088	{}	\N	2020-10-12 16:44:01.399667
82324	CODX	2020-09-28	14.18	14.47	13.2	14.2	2043832	{}	\N	2020-10-12 16:44:03.273301
82325	CODX	2020-09-29	13.9857	14.43	13.81	13.92	939952	{}	\N	2020-10-12 16:44:03.287256
82326	CODX	2020-09-30	13.76	14.03	13.42	13.59	1065673	{}	\N	2020-10-12 16:44:03.296008
82327	CODX	2020-10-01	13.52	13.7279	12.86	12.97	1193414	{}	\N	2020-10-12 16:44:03.301937
82328	CODX	2020-10-02	13.96	14.98	13.11	13.96	6455551	{}	\N	2020-10-12 16:44:03.307464
82329	CODX	2020-10-05	13.75	14.89	13.45	14.83	2178908	{}	\N	2020-10-12 16:44:03.312642
82330	CODX	2020-10-06	14.83	15.39	14.2706	14.36	2162351	{}	\N	2020-10-12 16:44:03.317494
82331	CODX	2020-10-07	14.55	14.8599	14.41	14.8	866733	{}	\N	2020-10-12 16:44:03.323735
82332	CODX	2020-10-08	14.75	14.9	14.07	14.41	1368460	{}	\N	2020-10-12 16:44:03.329665
82333	CODX	2020-10-09	14.32	14.58	14.1604	14.42	712888	{}	\N	2020-10-12 16:44:03.336114
82334	CODX	2020-10-12	14.45	14.4571	13.78	14.13	508651	{}	\N	2020-10-12 16:44:03.344985
82335	DKL	2020-09-28	27.71	28.2832	27.4	27.97	40207	{}	\N	2020-10-12 16:44:05.360251
82336	DKL	2020-09-29	27.82	28.93	27.6	28.86	66478	{}	\N	2020-10-12 16:44:05.374087
82337	DKL	2020-09-30	29	29	28.36	28.36	163097	{}	\N	2020-10-12 16:44:05.383954
82338	DKL	2020-10-01	28.63	28.81	28.01	28.3	168832	{}	\N	2020-10-12 16:44:05.390784
82339	DKL	2020-10-02	27.15	28.11	26.67	28.11	129705	{}	\N	2020-10-12 16:44:05.396422
82340	DKL	2020-10-05	28.75	28.775	27.78	28.4	98290	{}	\N	2020-10-12 16:44:05.402451
82341	DKL	2020-10-06	28.5	29.34	28.05	28.63	113461	{}	\N	2020-10-12 16:44:05.408058
82342	DKL	2020-10-07	28.62	28.8	27.85	28.47	89405	{}	\N	2020-10-12 16:44:05.413349
82343	DKL	2020-10-08	28.94	29.87	28.45	29.75	84600	{}	\N	2020-10-12 16:44:05.419575
82344	DKL	2020-10-09	30.05	30.29	29.0278	29.07	117407	{}	\N	2020-10-12 16:44:05.425346
82345	DKL	2020-10-12	29.58	29.58	29.1414	29.3302	27368	{}	\N	2020-10-12 16:44:05.43096
82346	BLES	2020-09-28	27.835	27.87	27.8144	27.8266	5832	{}	\N	2020-10-12 16:44:08.605699
82347	BLES	2020-09-29	27.7	27.7019	27.6	27.655	8949	{}	\N	2020-10-12 16:44:08.61193
82348	BLES	2020-09-30	27.65	27.93	27.65	27.77	6811	{}	\N	2020-10-12 16:44:08.618375
82349	BLES	2020-10-01	28.0081	28.02	27.775	27.82	17493	{}	\N	2020-10-12 16:44:08.625044
82350	BLES	2020-10-02	27.825	28.066	27.76	28.066	52065	{}	\N	2020-10-12 16:44:08.632854
82351	BLES	2020-10-05	28.34	28.63	28.34	28.63	26028	{}	\N	2020-10-12 16:44:08.63996
82352	BLES	2020-10-06	28.73	28.81	28.28	28.3115	61013	{}	\N	2020-10-12 16:44:08.648038
82353	BLES	2020-10-07	28.66	28.79	28.64	28.785	19041	{}	\N	2020-10-12 16:44:08.656179
82354	BLES	2020-10-08	28.9436	29.099	28.9436	29.099	1953	{}	\N	2020-10-12 16:44:08.660892
82355	BLES	2020-10-09	29.29	29.41	29.225	29.28	17266	{}	\N	2020-10-12 16:44:08.665779
82356	BLES	2020-10-12	29.37	29.38	29.35	29.38	1320	{}	\N	2020-10-12 16:44:08.670998
82357	BLUE	2020-09-28	57.42	57.52	55.095	55.9	542924	{}	\N	2020-10-12 16:44:11.087902
82358	BLUE	2020-09-29	55.53	56.8984	54.91	55	457805	{}	\N	2020-10-12 16:44:11.100264
82359	BLUE	2020-09-30	55.21	56.05	53.84	53.95	640085	{}	\N	2020-10-12 16:44:11.108255
82360	BLUE	2020-10-01	54.32	55.09	53.03	53.85	1093321	{}	\N	2020-10-12 16:44:11.114682
82361	BLUE	2020-10-02	53.4	54.06	51.45	51.65	791325	{}	\N	2020-10-12 16:44:11.12058
82362	BLUE	2020-10-05	52.06	56.4	52.06	56.11	884512	{}	\N	2020-10-12 16:44:11.128365
82363	BLUE	2020-10-06	56.4	57	55.4	56.07	479304	{}	\N	2020-10-12 16:44:11.134823
82364	BLUE	2020-10-07	56.35	58.34	56.3	57.73	705250	{}	\N	2020-10-12 16:44:11.140189
82365	BLUE	2020-10-08	58.45	58.45	55.85	57.73	661191	{}	\N	2020-10-12 16:44:11.1449
82366	BLUE	2020-10-09	57.38	58.8349	57.21	57.6	416808	{}	\N	2020-10-12 16:44:11.149509
82367	BLUE	2020-10-12	58.03	58.41	57.61	58.36	228791	{}	\N	2020-10-12 16:44:11.154492
82368	BSTC	2020-09-28	52.6298	53.24	51.95	52.6	24234	{}	\N	2020-10-12 16:44:12.542549
82369	BSTC	2020-09-29	53	53	51.97	52.61	12104	{}	\N	2020-10-12 16:44:12.552395
82370	BSTC	2020-09-30	52.6	54.75	52.55	52.84	34181	{}	\N	2020-10-12 16:44:12.559738
82371	BSTC	2020-10-01	53.585	53.585	52.43	53	22414	{}	\N	2020-10-12 16:44:12.564981
82372	BSTC	2020-10-02	52.18	53.47	51.41	52.5	25852	{}	\N	2020-10-12 16:44:12.569983
82373	BSTC	2020-10-05	53.3	54.8	53.06	54.59	23286	{}	\N	2020-10-12 16:44:12.5745
82374	BSTC	2020-10-06	56.74	58.97	54.33	56.98	37389	{}	\N	2020-10-12 16:44:12.578879
82375	BSTC	2020-10-07	57.07	59.98	57.07	59.08	70197	{}	\N	2020-10-12 16:44:12.582877
82376	BSTC	2020-10-08	59.98	59.98	58.76	59.1	36076	{}	\N	2020-10-12 16:44:12.586979
82377	BSTC	2020-10-09	59.17	60.42	58.82	60.41	18510	{}	\N	2020-10-12 16:44:12.59123
82378	BSTC	2020-10-12	60.33	60.48	59.4396	60.45	4372	{}	\N	2020-10-12 16:44:12.595538
82379	CBPO	2020-09-28	110.7	111.95	110.645	111	56258	{}	\N	2020-10-12 16:44:13.92406
82380	CBPO	2020-09-29	111.09	111.77	110.61	111.26	141167	{}	\N	2020-10-12 16:44:13.935468
82381	CBPO	2020-09-30	111.7	112.25	110.77	111.14	93715	{}	\N	2020-10-12 16:44:13.943484
82382	CBPO	2020-10-01	111.1	112.29	111.1	111.68	43019	{}	\N	2020-10-12 16:44:13.949182
82383	CBPO	2020-10-02	111.45	112.09	110.78	111.1	33838	{}	\N	2020-10-12 16:44:13.954565
82384	CBPO	2020-10-05	111.53	112.2073	111.02	111.51	140075	{}	\N	2020-10-12 16:44:13.959404
82385	CBPO	2020-10-06	111.575	112.81	111.575	112.04	30318	{}	\N	2020-10-12 16:44:13.963943
82386	CBPO	2020-10-07	112.24	112.84	111.48	112.26	33036	{}	\N	2020-10-12 16:44:13.968015
82387	CBPO	2020-10-08	112.73	112.97	111.87	112.36	38958	{}	\N	2020-10-12 16:44:13.972537
82388	CBPO	2020-10-09	112.26	112.8	110.4401	112.56	43985	{}	\N	2020-10-12 16:44:13.977011
82389	CBPO	2020-10-12	113.14	113.195	111.81	112.29	17263	{}	\N	2020-10-12 16:44:13.981413
82390	CEMB	2020-09-28	51.517	51.625	51.38	51.59	47238	{}	\N	2020-10-12 16:44:16.242326
82391	CEMB	2020-09-29	51.67	51.74	51.351	51.59	48152	{}	\N	2020-10-12 16:44:16.257228
82392	CEMB	2020-09-30	51.65	51.71	51.65	51.71	10989	{}	\N	2020-10-12 16:44:16.267921
82393	CEMB	2020-10-01	51.56	51.59	51.41	51.59	16702	{}	\N	2020-10-12 16:44:16.274819
82394	CEMB	2020-10-02	51.49	51.55	51.44	51.53	23258	{}	\N	2020-10-12 16:44:16.280511
82395	CEMB	2020-10-05	51.28	51.63	51.28	51.54	43721	{}	\N	2020-10-12 16:44:16.28542
82396	CEMB	2020-10-06	51.64	51.64	51.4592	51.61	18201	{}	\N	2020-10-12 16:44:16.290504
82397	CEMB	2020-10-07	51.64	51.7	51.55	51.69	28090	{}	\N	2020-10-12 16:44:16.295579
82398	CEMB	2020-10-08	51.6453	51.89	51.63	51.87	34446	{}	\N	2020-10-12 16:44:16.300832
82399	CEMB	2020-10-09	51.88	51.88	51.55	51.86	30497	{}	\N	2020-10-12 16:44:16.305845
82400	CEMB	2020-10-12	51.91	51.91	51.6	51.85	11328	{}	\N	2020-10-12 16:44:16.310984
82401	CHEP	2020-09-28	12.84	13.72	12.84	13.68	8095	{}	\N	2020-10-12 16:44:17.616094
82402	CHEP	2020-09-29	13.05	13.6	12.94	13.38	11528	{}	\N	2020-10-12 16:44:17.623344
82403	CHEP	2020-09-30	13.15	13.58	13.15	13.37	2517	{}	\N	2020-10-12 16:44:17.630398
82404	CHEP	2020-10-01	12.84	13.3848	12.84	13.3848	4915	{}	\N	2020-10-12 16:44:17.637794
82405	CHEP	2020-10-02	13.81	13.81	13.81	13.81	1	{}	\N	2020-10-12 16:44:17.646275
82406	CHEP	2020-10-05	13.81	13.91	13.81	13.91	424	{}	\N	2020-10-12 16:44:17.653796
82407	CHEP	2020-10-06	13.755	13.755	13.755	13.755	1	{}	\N	2020-10-12 16:44:17.660461
82408	CHEP	2020-10-07	13.4	14.24	12.81	13.98	10626	{}	\N	2020-10-12 16:44:17.665868
82409	CHEP	2020-10-08	13.48	13.9401	12.992	13.9401	13960	{}	\N	2020-10-12 16:44:17.673121
82410	CHEP	2020-10-09	13.5	14.26	13.071	13.78	31810	{}	\N	2020-10-12 16:44:17.683226
82411	CHEP	2020-10-12	13.6	13.67	13.6	13.64	2114	{}	\N	2020-10-12 16:44:17.690359
82412	CHIX	2020-09-28	13.9	13.96	13.9	13.95	9109	{}	\N	2020-10-12 16:44:20.886076
82413	CHIX	2020-09-29	13.79	13.82	13.7473	13.79	26580	{}	\N	2020-10-12 16:44:20.896495
82414	CHIX	2020-09-30	13.79	13.915	13.752	13.835	25649	{}	\N	2020-10-12 16:44:20.903555
82415	CHIX	2020-10-01	13.97	14.04	13.91	14.04	9991	{}	\N	2020-10-12 16:44:20.909483
82416	CHIX	2020-10-02	13.78	13.952	13.76	13.88	17542	{}	\N	2020-10-12 16:44:20.915293
82417	CHIX	2020-10-05	13.88	13.92	13.8674	13.92	7729	{}	\N	2020-10-12 16:44:20.920572
82418	CHIX	2020-10-06	13.87	13.9079	13.8332	13.84	3243	{}	\N	2020-10-12 16:44:20.92514
82419	CHIX	2020-10-07	13.86	13.92	13.85	13.89	5345	{}	\N	2020-10-12 16:44:20.930785
82420	CHIX	2020-10-08	13.95	14	13.935	13.995	11159	{}	\N	2020-10-12 16:44:20.935561
82421	CHIX	2020-10-09	13.94	13.9899	13.94	13.96	8654	{}	\N	2020-10-12 16:44:20.940033
82422	CHIX	2020-10-12	14.38	14.495	14.3682	14.4001	20626	{}	\N	2020-10-12 16:44:20.944468
82423	CMI	2020-09-28	211	214.4	210.56	212.79	868962	{}	\N	2020-10-12 16:44:22.598818
82424	CMI	2020-09-29	213.48	214.185	210.61	212.9	1488137	{}	\N	2020-10-12 16:44:22.610122
82425	CMI	2020-09-30	213.86	217.26	209.49	211.16	2636210	{}	\N	2020-10-12 16:44:22.618778
82426	CMI	2020-10-01	212.89	213.09	205.145	207.04	2144935	{}	\N	2020-10-12 16:44:22.624897
82427	CMI	2020-10-02	203.67	212.61	203.51	210.34	1952629	{}	\N	2020-10-12 16:44:22.631158
82428	CMI	2020-10-05	213.22	219.8	212.25	217.85	2523803	{}	\N	2020-10-12 16:44:22.63611
82429	CMI	2020-10-06	218.33	220.815	215.65	216.33	2215476	{}	\N	2020-10-12 16:44:22.64151
82430	CMI	2020-10-07	219.07	223.6	218.76	221.52	1411969	{}	\N	2020-10-12 16:44:22.646885
82431	CMI	2020-10-08	222.09	222.5685	218.51	221.01	1458188	{}	\N	2020-10-12 16:44:22.651573
82432	CMI	2020-10-09	222.9	225.84	221.36	223.32	1834993	{}	\N	2020-10-12 16:44:22.656247
82433	CMI	2020-10-12	224.46	224.5028	220.89	222.68	490423	{}	\N	2020-10-12 16:44:22.661125
82434	CNS	2020-09-28	56.69	57.22	56.45	56.65	81095	{}	\N	2020-10-12 16:44:24.597552
82435	CNS	2020-09-29	56.56	56.75	55.54	56.04	136291	{}	\N	2020-10-12 16:44:24.609334
82436	CNS	2020-09-30	56.5	57.17	55.44	55.74	493391	{}	\N	2020-10-12 16:44:24.617125
82437	CNS	2020-10-01	55.79	56.45	55.245	55.93	217412	{}	\N	2020-10-12 16:44:24.622821
82438	CNS	2020-10-02	54.99	57.55	54.99	56.95	146218	{}	\N	2020-10-12 16:44:24.628525
82439	CNS	2020-10-05	57.53	58.24	56.79	57.84	134154	{}	\N	2020-10-12 16:44:24.634069
82440	CNS	2020-10-06	57.79	57.85	56.04	56.13	255945	{}	\N	2020-10-12 16:44:24.639011
82441	CNS	2020-10-07	56.73	57.41	55.915	56.81	312837	{}	\N	2020-10-12 16:44:24.643969
82442	CNS	2020-10-08	57.8	59.7	57.47	58.98	353250	{}	\N	2020-10-12 16:44:24.649204
82443	CNS	2020-10-09	59.6	59.895	57.82	59.2	306348	{}	\N	2020-10-12 16:44:24.654206
82444	CNS	2020-10-12	59.41	59.95	59.28	59.83	27222	{}	\N	2020-10-12 16:44:24.658925
82445	CNXT	2020-09-28	39.65	39.75	39.53	39.62	16394	{}	\N	2020-10-12 16:44:26.838345
82446	CNXT	2020-09-29	40.36	40.38	40.225	40.38	8405	{}	\N	2020-10-12 16:44:26.848283
82447	CNXT	2020-09-30	40.5	40.7	40.45	40.63	6856	{}	\N	2020-10-12 16:44:26.855676
82448	CNXT	2020-10-01	41.24	41.3299	41.16	41.28	26817	{}	\N	2020-10-12 16:44:26.861756
82449	CNXT	2020-10-02	40.6	41.03	40.6	40.77	2361	{}	\N	2020-10-12 16:44:26.86808
82450	CNXT	2020-10-05	41.15	41.33	41.14	41.28	6203	{}	\N	2020-10-12 16:44:26.873207
82451	CNXT	2020-10-06	41.36	41.5	41.2372	41.33	10556	{}	\N	2020-10-12 16:44:26.877893
82452	CNXT	2020-10-07	41.65	41.6999	41.57	41.61	10444	{}	\N	2020-10-12 16:44:26.882624
82453	CNXT	2020-10-08	41.56	41.8191	41.56	41.79	10406	{}	\N	2020-10-12 16:44:26.887558
82454	CNXT	2020-10-09	42.49	42.86	42.49	42.86	34816	{}	\N	2020-10-12 16:44:26.892572
82455	CNXT	2020-10-12	43.65	43.71	43.4883	43.62	18492	{}	\N	2020-10-12 16:44:26.897242
82456	COMT	2020-09-28	24.915	25.06	24.88	25.02	17635	{}	\N	2020-10-12 16:44:28.903096
82457	COMT	2020-09-29	24.85	24.9	24.63	24.7	14807	{}	\N	2020-10-12 16:44:28.913312
82458	COMT	2020-09-30	24.75	25	24.75	24.965	29988	{}	\N	2020-10-12 16:44:28.920395
82459	COMT	2020-10-01	24.88	24.88	24.5	24.76	20747	{}	\N	2020-10-12 16:44:28.926251
82460	COMT	2020-10-02	24.49	24.625	24.41	24.5	31381	{}	\N	2020-10-12 16:44:28.931953
82461	COMT	2020-10-05	24.88	24.9855	24.76	24.91	52766	{}	\N	2020-10-12 16:44:28.93659
82462	COMT	2020-10-06	25.13	25.36	25	25	32964	{}	\N	2020-10-12 16:44:28.94167
82463	COMT	2020-10-07	25.15	25.18	24.9901	25.08	25286	{}	\N	2020-10-12 16:44:28.946592
82464	COMT	2020-10-08	25.15	25.315	25.14	25.16	24749	{}	\N	2020-10-12 16:44:28.951602
82465	COMT	2020-10-09	25.318	25.36	25.15	25.16	23019	{}	\N	2020-10-12 16:44:28.956608
82466	COMT	2020-10-12	25.11	25.131	24.995	25.0267	11103	{}	\N	2020-10-12 16:44:28.961758
82467	CRUS	2020-09-28	63.65	65	63.43	64.95	642758	{}	\N	2020-10-12 16:44:35.840986
82468	CRUS	2020-09-29	64.75	66.29	64.5699	66	827710	{}	\N	2020-10-12 16:44:35.851673
82469	CRUS	2020-09-30	66	67.88	65.5	67.45	829255	{}	\N	2020-10-12 16:44:35.858949
82470	CRUS	2020-10-01	68.34	69.09	67.8486	68.38	455805	{}	\N	2020-10-12 16:44:35.866951
82471	CRUS	2020-10-02	66.47	67.43	66.24	66.86	632840	{}	\N	2020-10-12 16:44:35.873642
82472	CRUS	2020-10-05	67.51	69.81	67.175	69.76	812071	{}	\N	2020-10-12 16:44:35.878953
82473	CRUS	2020-10-06	69.45	71.405	69.45	69.88	637079	{}	\N	2020-10-12 16:44:35.883781
82474	CRUS	2020-10-07	70.93	71.259	69.98	70.37	509654	{}	\N	2020-10-12 16:44:35.889132
82475	CRUS	2020-10-08	70.53	71.22	69.78	70.36	383754	{}	\N	2020-10-12 16:44:35.894746
82476	CRUS	2020-10-09	70.86	72.06	70.82	71.94	344804	{}	\N	2020-10-12 16:44:35.900126
82477	CRUS	2020-10-12	72.84	73.355	71.865	73.325	244821	{}	\N	2020-10-12 16:44:35.905836
82478	PGRE	2020-09-28	7.19	7.41	7.175	7.18	2731626	{}	\N	2020-10-12 16:44:36.941516
82479	PGRE	2020-09-29	7.1	7.11	6.825	6.98	3818536	{}	\N	2020-10-12 16:44:36.949571
82480	PGRE	2020-09-30	6.99	7.19	6.94	7.08	4534108	{}	\N	2020-10-12 16:44:36.955142
82481	PGRE	2020-10-01	7.12	7.22	7.03	7.22	3731188	{}	\N	2020-10-12 16:44:36.962531
82482	PGRE	2020-10-02	7.03	7.44	7	7.42	2667605	{}	\N	2020-10-12 16:44:36.96932
82483	PGRE	2020-10-05	7.55	7.55	7.25	7.45	5588586	{}	\N	2020-10-12 16:44:36.975942
82484	PGRE	2020-10-06	7.51	7.52	7.26	7.32	5454250	{}	\N	2020-10-12 16:44:36.985355
82485	PGRE	2020-10-07	7.36	7.44	7.23	7.31	3231012	{}	\N	2020-10-12 16:44:36.995626
82486	PGRE	2020-10-08	7.4	7.53	7.3	7.53	4894132	{}	\N	2020-10-12 16:44:37.003357
82487	PGRE	2020-10-09	7.58	7.64	7.36	7.43	2751170	{}	\N	2020-10-12 16:44:37.009384
82488	PGRE	2020-10-12	7.41	7.48	7.34	7.425	924443	{}	\N	2020-10-12 16:44:37.015117
82489	BNSO	2020-09-28	5	5	4.01	4.65	44569	{}	\N	2020-10-12 16:44:38.291288
82490	BNSO	2020-09-29	4.57	4.6108	4.18	4.49	33221	{}	\N	2020-10-12 16:44:38.303121
82491	BNSO	2020-09-30	4.5	4.6299	4.21	4.4	17720	{}	\N	2020-10-12 16:44:38.309571
82492	BNSO	2020-10-01	4.31	4.45	4.3	4.45	7045	{}	\N	2020-10-12 16:44:38.315499
82493	BNSO	2020-10-02	4.2922	4.41	4.24	4.36	4815	{}	\N	2020-10-12 16:44:38.321725
82494	BNSO	2020-10-05	4.37	4.45	4.09	4.25	20014	{}	\N	2020-10-12 16:44:38.326634
82495	BNSO	2020-10-06	4.132	4.34	4.1	4.33	27986	{}	\N	2020-10-12 16:44:38.333088
82496	BNSO	2020-10-07	4.6445	4.72	4.44	4.7	36971	{}	\N	2020-10-12 16:44:38.338304
82497	BNSO	2020-10-08	4.4	4.7099	4.36	4.48	2463	{}	\N	2020-10-12 16:44:38.343164
82498	BNSO	2020-10-09	4.5692	4.5692	4.18	4.5	13079	{}	\N	2020-10-12 16:44:38.349166
82499	BNSO	2020-10-12	4.5	4.5	4.5	4.5	438	{}	\N	2020-10-12 16:44:38.353683
82500	BOSS	2020-09-28	24.447	24.939	24.447	24.939	1554	{}	\N	2020-10-12 16:44:40.171186
82501	BOSS	2020-09-29	24.8937	24.8937	24.8937	24.8937	236	{}	\N	2020-10-12 16:44:40.177144
82502	BOSS	2020-09-30	25.09	25.13	24.9649	24.9649	907	{}	\N	2020-10-12 16:44:40.181657
82503	BOSS	2020-10-01	25.2987	25.4494	25.2802	25.4494	2234	{}	\N	2020-10-12 16:44:40.187825
82504	BOSS	2020-10-02	25.4494	25.4494	25.19	25.355	738	{}	\N	2020-10-12 16:44:40.194549
82505	BOSS	2020-10-05	25.689	25.8339	25.635	25.8339	539	{}	\N	2020-10-12 16:44:40.200652
82506	BOSS	2020-10-06	26.075	26.125	25.764	25.764	2662	{}	\N	2020-10-12 16:44:40.206187
82507	BOSS	2020-10-07	26.23	26.2458	26.22	26.2458	365	{}	\N	2020-10-12 16:44:40.212054
82508	BOSS	2020-10-08	26.4339	26.5095	26.3991	26.5095	2124	{}	\N	2020-10-12 16:44:40.21818
82509	BOSS	2020-10-09	26.6899	26.6899	26.6899	26.6899	255	{}	\N	2020-10-12 16:44:40.225906
82510	BTEC	2020-09-28	47	47	46.545	46.9	5290	{}	\N	2020-10-12 16:44:42.134639
82511	BTEC	2020-09-29	47.25	47.3749	46.841	47.19	4340	{}	\N	2020-10-12 16:44:42.144952
82512	BTEC	2020-09-30	47.4424	47.9801	47.21	47.6229	23839	{}	\N	2020-10-12 16:44:42.15271
82513	BTEC	2020-10-01	47.565	47.76	47.36	47.56	5102	{}	\N	2020-10-12 16:44:42.159305
82514	BTEC	2020-10-02	46.69	47.775	46.69	46.98	6602	{}	\N	2020-10-12 16:44:42.165472
82515	BTEC	2020-10-05	47.98	49.2199	47.98	49.17	9149	{}	\N	2020-10-12 16:44:42.1709
82516	BTEC	2020-10-06	49.71	50.01	49.07	49.07	5786	{}	\N	2020-10-12 16:44:42.177204
82517	BTEC	2020-10-07	50.11	50.61	49.5	50.5	25125	{}	\N	2020-10-12 16:44:42.181484
82518	BTEC	2020-10-08	51.28	51.28	50.525	50.62	16546	{}	\N	2020-10-12 16:44:42.185996
82519	BTEC	2020-10-09	50.75	51.32	50.75	51.26	8035	{}	\N	2020-10-12 16:44:42.190751
82520	BTEC	2020-10-12	52.27	52.5	51.0998	51.4	6708	{}	\N	2020-10-12 16:44:42.195041
82521	CHIM	2020-09-28	17.82	17.82	17.82	17.82	18	{}	\N	2020-10-12 16:44:43.317474
82522	CHIM	2020-09-29	17.91	17.91	17.91	17.91	11	{}	\N	2020-10-12 16:44:43.325298
82523	CHIM	2020-09-30	17.72	17.903	17.72	17.903	805	{}	\N	2020-10-12 16:44:43.333944
82524	CHIM	2020-10-01	18.075	18.075	18.075	18.075	139	{}	\N	2020-10-12 16:44:43.340264
82525	CHIM	2020-10-02	17.885	17.885	17.885	17.885	53	{}	\N	2020-10-12 16:44:43.348044
82526	CHIM	2020-10-05	18.0299	18.1897	18.0299	18.1897	1011	{}	\N	2020-10-12 16:44:43.354717
82527	CHIM	2020-10-06	18.57	18.57	18.19	18.41	2555	{}	\N	2020-10-12 16:44:43.360641
82528	CHIM	2020-10-07	18.195	18.195	18.195	18.195	11	{}	\N	2020-10-12 16:44:43.366918
82529	CHIM	2020-10-08	18.34	18.34	18.34	18.34	36	{}	\N	2020-10-12 16:44:43.372358
82530	CHIM	2020-10-09	18.55	18.55	18.55	18.55	80	{}	\N	2020-10-12 16:44:43.378183
82531	CHIM	2020-10-12	18.58	18.8699	18.58	18.63	863	{}	\N	2020-10-12 16:44:43.383206
82532	CNDT	2020-09-28	3.11	3.345	3.11	3.21	1835012	{}	\N	2020-10-12 16:44:44.649975
82533	CNDT	2020-09-29	3.18	3.27	3.005	3.25	1532804	{}	\N	2020-10-12 16:44:44.659666
82534	CNDT	2020-09-30	3.22	3.27	3.16	3.18	1659623	{}	\N	2020-10-12 16:44:44.66734
82535	CNDT	2020-10-01	3.225	3.27	3.14	3.25	1219081	{}	\N	2020-10-12 16:44:44.67378
82536	CNDT	2020-10-02	3.1	3.2683	3.05	3.25	992861	{}	\N	2020-10-12 16:44:44.678862
82537	CNDT	2020-10-05	3.28	3.41	3.26	3.37	1024110	{}	\N	2020-10-12 16:44:44.683369
82538	CNDT	2020-10-06	3.41	3.47	3.28	3.28	1403444	{}	\N	2020-10-12 16:44:44.687909
82539	CNDT	2020-10-07	3.31	3.475	3.31	3.41	1290520	{}	\N	2020-10-12 16:44:44.692104
82540	CNDT	2020-10-08	3.43	3.555	3.41	3.51	1417299	{}	\N	2020-10-12 16:44:44.697036
82541	CNDT	2020-10-09	3.53	3.74	3.51	3.6	1484220	{}	\N	2020-10-12 16:44:44.701976
82542	CNDT	2020-10-12	3.7	3.72	3.46	3.58	905110	{}	\N	2020-10-12 16:44:44.706841
82543	COMB	2020-09-28	21.21	21.36	21.21	21.34	3600	{}	\N	2020-10-12 16:44:45.652328
82544	COMB	2020-09-29	21.09	21.09	20.95	20.95	6339	{}	\N	2020-10-12 16:44:45.663495
82545	COMB	2020-09-30	21.11	21.29	21.11	21.195	2190	{}	\N	2020-10-12 16:44:45.671891
82546	COMB	2020-10-01	21.0765	21.0765	20.94	21.04	5511	{}	\N	2020-10-12 16:44:45.677881
82547	COMB	2020-10-02	20.87	21.1	20.87	20.9	5615	{}	\N	2020-10-12 16:44:45.683664
82548	COMB	2020-10-05	21.32	21.395	21.22	21.281	3257	{}	\N	2020-10-12 16:44:45.688256
82549	COMB	2020-10-06	21.3	21.3	21.2427	21.2427	722	{}	\N	2020-10-12 16:44:45.692661
82550	COMB	2020-10-07	21.525	21.525	21.46	21.49	11101	{}	\N	2020-10-12 16:44:45.697474
82551	COMB	2020-10-08	21.58	21.6553	21.56	21.611	3579	{}	\N	2020-10-12 16:44:45.70227
82552	COMB	2020-10-09	21.9	22	21.9	21.966	9429	{}	\N	2020-10-12 16:44:45.706849
82553	COMB	2020-10-12	21.92	21.92	21.92	21.92	1085	{}	\N	2020-10-12 16:44:45.711609
82554	CTSO	2020-09-28	7.92	8	7.81	7.87	169521	{}	\N	2020-10-12 16:44:47.427358
82555	CTSO	2020-09-29	7.86	8.13	7.79	8.07	220929	{}	\N	2020-10-12 16:44:47.439175
82556	CTSO	2020-09-30	8.1	8.24	7.89	7.975	202713	{}	\N	2020-10-12 16:44:47.451006
82557	CTSO	2020-10-01	8.02	8.11	7.71	7.86	227891	{}	\N	2020-10-12 16:44:47.458696
82558	CTSO	2020-10-02	7.82	7.9703	7.66	7.86	198008	{}	\N	2020-10-12 16:44:47.465562
82559	CTSO	2020-10-05	7.86	8.15	7.86	8.08	154804	{}	\N	2020-10-12 16:44:47.470258
82560	CTSO	2020-10-06	8.1	8.34	8.065	8.11	192346	{}	\N	2020-10-12 16:44:47.476017
82561	CTSO	2020-10-07	8.19	8.45	8.1513	8.35	252537	{}	\N	2020-10-12 16:44:47.48071
82562	CTSO	2020-10-08	8.43	8.44	8.06	8.09	157533	{}	\N	2020-10-12 16:44:47.485319
82563	CTSO	2020-10-09	8.25	8.32	7.97	8.22	223540	{}	\N	2020-10-12 16:44:47.489673
82564	CTSO	2020-10-12	8.19	8.45	8.13	8.405	88394	{}	\N	2020-10-12 16:44:47.495501
82565	CVR	2020-09-28	22	22	21.1917	21.1917	1062	{}	\N	2020-10-12 16:44:48.779155
82566	CVR	2020-09-29	20.54	20.54	20.54	20.54	421	{}	\N	2020-10-12 16:44:48.788963
82567	CVR	2020-09-30	21.5521	21.6	20.4	21.6	996	{}	\N	2020-10-12 16:44:48.796594
82568	CVR	2020-10-05	20.55	21	20.52	21	1251	{}	\N	2020-10-12 16:44:48.801887
82569	CVR	2020-10-06	21	21.5931	21	21.0001	630	{}	\N	2020-10-12 16:44:48.807198
82570	CVR	2020-10-07	21.2002	22.45	21	21	2285	{}	\N	2020-10-12 16:44:48.812568
82571	CVR	2020-10-08	21.01	21.01	20.28	20.51	975	{}	\N	2020-10-12 16:44:48.81703
82572	CVR	2020-10-09	20.7	20.7	20.7	20.7	214	{}	\N	2020-10-12 16:44:48.821608
82573	CVR	2020-10-12	21.4783	21.4783	20.6	20.6	1457	{}	\N	2020-10-12 16:44:48.827752
82574	CWB	2020-09-28	68.43	68.86	68.325	68.77	1345498	{}	\N	2020-10-12 16:44:50.501939
82575	CWB	2020-09-29	68.83	68.985	68.5	68.73	788931	{}	\N	2020-10-12 16:44:50.513274
82576	CWB	2020-09-30	68.63	69.35	68.51	68.89	2145804	{}	\N	2020-10-12 16:44:50.521239
82577	CWB	2020-10-01	69.4	70.16	69.2611	70.16	1404033	{}	\N	2020-10-12 16:44:50.528253
82578	CWB	2020-10-02	69.16	69.85	68.81	69.32	1194233	{}	\N	2020-10-12 16:44:50.534209
82579	CWB	2020-10-05	69.62	70.25	69.62	70	1621415	{}	\N	2020-10-12 16:44:50.539188
82580	CWB	2020-10-06	70.15	70.8	69.4929	69.77	940270	{}	\N	2020-10-12 16:44:50.544255
82581	CWB	2020-10-07	70.57	70.9867	69.98	70.88	976343	{}	\N	2020-10-12 16:44:50.549542
82582	CWB	2020-10-08	71.57	71.57	71.04	71.19	1233109	{}	\N	2020-10-12 16:44:50.554716
82583	CWB	2020-10-09	71.71	71.905	71.19	71.75	966685	{}	\N	2020-10-12 16:44:50.5598
82584	CWB	2020-10-12	72.46	72.47	72.02	72.32	657169	{}	\N	2020-10-12 16:44:50.564852
82585	DAUD	2020-09-28	26.53	26.53	26.48	26.48	685	{}	\N	2020-10-12 16:44:52.431496
82586	DAUD	2020-09-29	25.6287	25.6287	25.6287	25.6287	163	{}	\N	2020-10-12 16:44:52.442936
82587	DAUD	2020-09-30	25.12	25.2	25.0999	25.1091	1365	{}	\N	2020-10-12 16:44:52.45173
82588	DAUD	2020-10-01	24.7265	24.7265	24.7265	24.7265	69	{}	\N	2020-10-12 16:44:52.457722
82589	DAUD	2020-10-02	25.1636	25.1636	25.1636	25.1636	51	{}	\N	2020-10-12 16:44:52.463705
82590	DAUD	2020-10-05	24.73	24.7951	24.73	24.7951	102	{}	\N	2020-10-12 16:44:52.468969
82591	DAUD	2020-10-06	25.02	25.752	25.02	25.703	537	{}	\N	2020-10-12 16:44:52.473833
82592	DAUD	2020-10-07	25.49	25.49	25.4496	25.4496	139	{}	\N	2020-10-12 16:44:52.479108
82593	DAUD	2020-10-08	25.0399	25.0399	25.0399	25.0399	86	{}	\N	2020-10-12 16:44:52.484143
82594	DAUD	2020-10-09	24.0457	24.0457	24.0457	24.0457	142	{}	\N	2020-10-12 16:44:52.488839
82595	DAUD	2020-10-12	24.39	24.39	24.38	24.38	1350	{}	\N	2020-10-12 16:44:52.494097
82596	DEM	2020-09-28	35.05	35.0534	34.81	34.93	377366	{}	\N	2020-10-12 16:44:53.799817
82597	DEM	2020-09-29	34.87	34.9699	34.7	34.86	421392	{}	\N	2020-10-12 16:44:53.809827
82598	DEM	2020-09-30	35.02	35.2999	35.02	35.24	297321	{}	\N	2020-10-12 16:44:53.817606
82599	DEM	2020-10-01	35.46	35.51	35.274	35.45	352008	{}	\N	2020-10-12 16:44:53.824191
82600	DEM	2020-10-02	34.96	35.36	34.9302	35.1	145885	{}	\N	2020-10-12 16:44:53.830397
82601	DEM	2020-10-05	35.2	35.5	35.2	35.5	202043	{}	\N	2020-10-12 16:44:53.835758
82602	DEM	2020-10-06	35.62	35.71	35.3	35.42	523034	{}	\N	2020-10-12 16:44:53.841615
82603	DEM	2020-10-07	35.6	35.7	35.44	35.61	159081	{}	\N	2020-10-12 16:44:53.846877
82604	DEM	2020-10-08	35.68	35.8899	35.6299	35.79	299214	{}	\N	2020-10-12 16:44:53.851296
82605	DEM	2020-10-09	35.91	36.11	35.8331	36	238208	{}	\N	2020-10-12 16:44:53.856167
82606	DEM	2020-10-12	36.02	36.1	35.92	36.0899	253569	{}	\N	2020-10-12 16:44:53.861949
82607	CCNE	2020-09-28	14.63	15	14.47	14.81	109915	{}	\N	2020-10-12 16:44:55.239115
82608	CCNE	2020-09-29	14.7	14.88	14.51	14.8	42852	{}	\N	2020-10-12 16:44:55.251257
82609	CCNE	2020-09-30	14.88	15.05	14.75	14.87	26985	{}	\N	2020-10-12 16:44:55.260138
82610	CCNE	2020-10-01	14.88	15.09	14.61	15.06	27966	{}	\N	2020-10-12 16:44:55.266817
82611	CCNE	2020-10-02	15.37	15.67	14.85	15.61	33386	{}	\N	2020-10-12 16:44:55.272981
82612	CCNE	2020-10-05	15.85	16.07	15.655	16.04	45215	{}	\N	2020-10-12 16:44:55.278219
82613	CCNE	2020-10-06	16.13	16.79	16.02	16.29	43146	{}	\N	2020-10-12 16:44:55.283469
82614	CCNE	2020-10-07	16.36	16.71	16.29	16.62	34383	{}	\N	2020-10-12 16:44:55.288842
82615	CCNE	2020-10-08	16.57	16.9	16.42	16.8	32572	{}	\N	2020-10-12 16:44:55.294309
82616	CCNE	2020-10-09	16.89	16.95	16.52	16.69	34156	{}	\N	2020-10-12 16:44:55.299434
82617	CCNE	2020-10-12	16.68	16.89	16.56	16.84	11290	{}	\N	2020-10-12 16:44:55.304586
82618	CDMOP	2020-09-28	27.235	27.235	27.235	27.235	528	{}	\N	2020-10-12 16:44:57.309189
82619	CDMOP	2020-09-29	27.4	27.4	26.82	26.875	2598	{}	\N	2020-10-12 16:44:57.319759
82620	CDMOP	2020-09-30	26.88	27.15	26.88	27	1285	{}	\N	2020-10-12 16:44:57.326695
82621	CDMOP	2020-10-01	27.3999	27.3999	26.9394	26.9394	935	{}	\N	2020-10-12 16:44:57.332358
82622	CDMOP	2020-10-02	27.125	27.1545	27.12	27.1545	1952	{}	\N	2020-10-12 16:44:57.337009
82623	CDMOP	2020-10-06	27	27.0001	27	27	2736	{}	\N	2020-10-12 16:44:57.341465
82624	CDMOP	2020-10-07	26.95	27.085	26.95	27.065	1672	{}	\N	2020-10-12 16:44:57.345882
82625	CDMOP	2020-10-08	27.099	27.099	27.065	27.065	1138	{}	\N	2020-10-12 16:44:57.350166
82626	CDMOP	2020-10-09	27.05	27.05	27.05	27.05	220	{}	\N	2020-10-12 16:44:57.354644
82627	CDMOP	2020-10-12	26.98	27.07	26.5	26.827	2337	{}	\N	2020-10-12 16:44:57.359258
82628	CSIQ	2020-09-28	33.15	34.21	32.5357	34.02	2779778	{}	\N	2020-10-12 16:44:59.660044
82629	CSIQ	2020-09-29	33.785	34.4	32.93	34.04	2016057	{}	\N	2020-10-12 16:44:59.666765
82630	CSIQ	2020-09-30	34.42	36.23	34.09	35.1	3318956	{}	\N	2020-10-12 16:44:59.67191
82631	CSIQ	2020-10-01	35.59	36.73	34.6557	36.35	1811449	{}	\N	2020-10-12 16:44:59.679184
82632	CSIQ	2020-10-02	35.1	36.99	35	35.9	1884590	{}	\N	2020-10-12 16:44:59.684192
82633	CSIQ	2020-10-05	36.5	38.41	36.49	38.25	2059741	{}	\N	2020-10-12 16:44:59.688995
82634	CSIQ	2020-10-06	38.56	39.99	37.63	38.89	3121183	{}	\N	2020-10-12 16:44:59.693455
82635	CSIQ	2020-10-07	40	42.74	39.7	40.78	3957747	{}	\N	2020-10-12 16:44:59.697956
82636	CSIQ	2020-10-08	41.6	42	38.53	39.41	4221396	{}	\N	2020-10-12 16:44:59.703065
82637	CSIQ	2020-10-09	39.52	42.05	39.29	41.62	2593877	{}	\N	2020-10-12 16:44:59.708093
82638	CSIQ	2020-10-12	40.995	41.1	37.06	38.23	3253700	{}	\N	2020-10-12 16:44:59.712476
82639	CWH	2020-09-28	29.12	29.4	27.46	28.46	2734526	{}	\N	2020-10-12 16:45:01.511911
82640	CWH	2020-09-29	28.72	29.75	28.5	28.85	3187957	{}	\N	2020-10-12 16:45:01.518724
82641	CWH	2020-09-30	28.7	30.85	28.7	29.75	5886553	{}	\N	2020-10-12 16:45:01.524241
82642	CWH	2020-10-01	30.12	31.26	29.6	31.21	3100318	{}	\N	2020-10-12 16:45:01.530192
82643	CWH	2020-10-02	30.06	31.64	30	31.2	2626510	{}	\N	2020-10-12 16:45:01.535127
82644	CWH	2020-10-05	31.52	32.63	31.5	32.45	2875842	{}	\N	2020-10-12 16:45:01.539851
82645	CWH	2020-10-06	32.35	32.96	31.41	31.64	3520363	{}	\N	2020-10-12 16:45:01.544674
82646	CWH	2020-10-07	32.48	33.24	31.84	32.4	2375630	{}	\N	2020-10-12 16:45:01.54925
82647	CWH	2020-10-08	32.64	32.85	30.6903	31.06	3210264	{}	\N	2020-10-12 16:45:01.553948
82648	CWH	2020-10-09	31.5	31.73	30.4601	30.97	2078752	{}	\N	2020-10-12 16:45:01.55964
82649	CWH	2020-10-12	31	31.2	30.12	30.8	858254	{}	\N	2020-10-12 16:45:01.564621
82650	DBVT	2020-09-28	1.74	1.775	1.7	1.74	974992	{}	\N	2020-10-12 16:45:04.841708
82651	DBVT	2020-09-29	1.75	1.75	1.7	1.71	224729	{}	\N	2020-10-12 16:45:04.850578
82652	DBVT	2020-09-30	1.77	1.799	1.73	1.73	345527	{}	\N	2020-10-12 16:45:04.85508
82653	DBVT	2020-10-01	1.73	1.76	1.7	1.71	337281	{}	\N	2020-10-12 16:45:04.860226
82654	DBVT	2020-10-02	1.68	1.71	1.66	1.7	137603	{}	\N	2020-10-12 16:45:04.864851
82655	DBVT	2020-10-05	1.68	1.71	1.65	1.65	209742	{}	\N	2020-10-12 16:45:04.86951
82656	DBVT	2020-10-06	1.8	1.83	1.76	1.78	444900	{}	\N	2020-10-12 16:45:04.875943
82657	DBVT	2020-10-07	1.77	1.81	1.75	1.77	209421	{}	\N	2020-10-12 16:45:04.880226
82658	DBVT	2020-10-08	1.75	1.79	1.72	1.72	165928	{}	\N	2020-10-12 16:45:04.88526
82659	DBVT	2020-10-09	1.74	1.83	1.74	1.79	125983	{}	\N	2020-10-12 16:45:04.890125
82660	DBVT	2020-10-12	1.75	1.79	1.75	1.7802	48835	{}	\N	2020-10-12 16:45:04.894835
82661	DFIN	2020-09-28	12.49	13.41	12.46	13.35	343250	{}	\N	2020-10-12 16:45:05.746815
82662	DFIN	2020-09-29	13.35	13.5893	12.93	13.19	597745	{}	\N	2020-10-12 16:45:05.754729
82663	DFIN	2020-09-30	13.1	13.55	13.1	13.36	632869	{}	\N	2020-10-12 16:45:05.760657
82664	DFIN	2020-10-01	13.44	13.63	12.96	13.2	439580	{}	\N	2020-10-12 16:45:05.765957
82665	DFIN	2020-10-02	12.82	13.54	12.73	13.43	297610	{}	\N	2020-10-12 16:45:05.770746
82666	DFIN	2020-10-05	13.58	13.75	13.26	13.32	337235	{}	\N	2020-10-12 16:45:05.775809
82667	DFIN	2020-10-06	13.41	14.3	13.36	13.95	800541	{}	\N	2020-10-12 16:45:05.780724
82668	DFIN	2020-10-07	14.78	14.87	14.34	14.53	756873	{}	\N	2020-10-12 16:45:05.785635
82669	DFIN	2020-10-08	14.62	14.79	14.48	14.76	353381	{}	\N	2020-10-12 16:45:05.790473
82670	DFIN	2020-10-09	14.84	14.86	14.14	14.35	462503	{}	\N	2020-10-12 16:45:05.795018
82671	DFIN	2020-10-12	14.5	14.5	14.025	14.04	186306	{}	\N	2020-10-12 16:45:05.79971
82672	DGS	2020-09-28	40.86	40.9545	40.64	40.82	249924	{}	\N	2020-10-12 16:45:07.309271
82673	DGS	2020-09-29	40.74	41.055	40.74	40.9	171432	{}	\N	2020-10-12 16:45:07.318182
82674	DGS	2020-09-30	41.02	41.42	41.02	41.35	186881	{}	\N	2020-10-12 16:45:07.325102
82675	DGS	2020-10-01	41.61	41.93	41.54	41.93	142327	{}	\N	2020-10-12 16:45:07.330226
82676	DGS	2020-10-02	41.26	41.74	41.26	41.43	139252	{}	\N	2020-10-12 16:45:07.334573
82677	DGS	2020-10-05	41.66	42.0299	41.66	41.92	180052	{}	\N	2020-10-12 16:45:07.338863
82678	DGS	2020-10-06	42.11	42.34	41.87	41.96	137618	{}	\N	2020-10-12 16:45:07.343312
82679	DGS	2020-10-07	42.3	42.6099	42.3	42.37	116089	{}	\N	2020-10-12 16:45:07.347537
82680	DGS	2020-10-08	42.53	42.83	42.4509	42.64	86592	{}	\N	2020-10-12 16:45:07.351532
82681	DGS	2020-10-09	42.75	43.1476	42.6901	42.92	96390	{}	\N	2020-10-12 16:45:07.355689
82682	DGS	2020-10-12	42.82	42.98	42.65	42.88	72669	{}	\N	2020-10-12 16:45:07.359952
82683	DIVB	2020-09-28	28.38	28.72	28.38	28.62	1692	{}	\N	2020-10-12 16:45:09.314008
82684	DIVB	2020-09-29	28.31	28.37	28.31	28.37	531	{}	\N	2020-10-12 16:45:09.319885
82685	DIVB	2020-09-30	28.6727	28.6727	28.6599	28.6599	696	{}	\N	2020-10-12 16:45:09.324996
82686	DIVB	2020-10-01	28.529	28.63	28.529	28.53	3592	{}	\N	2020-10-12 16:45:09.329685
82687	DIVB	2020-10-02	28.3825	28.63	28.3825	28.63	2391	{}	\N	2020-10-12 16:45:09.333875
82688	DIVB	2020-10-05	28.9	29.0794	28.9	29.0794	2326	{}	\N	2020-10-12 16:45:09.338636
82689	DIVB	2020-10-06	29.18	29.25	28.7714	28.7714	880	{}	\N	2020-10-12 16:45:09.343116
82690	DIVB	2020-10-07	29.19	29.28	29.1347	29.28	4807	{}	\N	2020-10-12 16:45:09.347324
82691	DIVB	2020-10-08	29.53	29.5852	29.53	29.5852	964	{}	\N	2020-10-12 16:45:09.351822
82692	DIVB	2020-10-09	29.84	29.84	29.71	29.71	864	{}	\N	2020-10-12 16:45:09.356191
82693	CIF	2020-09-28	2.3	2.34	2.29	2.29	115142	{}	\N	2020-10-12 16:45:10.695169
82694	CIF	2020-09-29	2.29	2.31	2.29	2.3	260688	{}	\N	2020-10-12 16:45:10.709114
82695	CIF	2020-09-30	2.33	2.33	2.3101	2.33	60678	{}	\N	2020-10-12 16:45:10.715992
82696	CIF	2020-10-01	2.34	2.35	2.31	2.34	97162	{}	\N	2020-10-12 16:45:10.722155
82697	CIF	2020-10-02	2.32	2.35	2.31	2.35	204844	{}	\N	2020-10-12 16:45:10.727454
82698	CIF	2020-10-05	2.34	2.35	2.33	2.345	184478	{}	\N	2020-10-12 16:45:10.733128
82699	CIF	2020-10-06	2.35	2.36	2.34	2.36	121264	{}	\N	2020-10-12 16:45:10.737353
82700	CIF	2020-10-07	2.36	2.38	2.35	2.38	62458	{}	\N	2020-10-12 16:45:10.741434
82701	CIF	2020-10-08	2.39	2.4	2.36	2.39	95974	{}	\N	2020-10-12 16:45:10.745799
82702	CIF	2020-10-09	2.39	2.4	2.37	2.4	87506	{}	\N	2020-10-12 16:45:10.750583
82703	CIF	2020-10-12	2.4	2.4	2.37	2.39	56368	{}	\N	2020-10-12 16:45:10.754847
82704	CSII	2020-09-28	36.48	37.49	36.48	36.92	159802	{}	\N	2020-10-12 16:45:12.333127
82705	CSII	2020-09-29	36.97	37.47	36.67	36.9	206898	{}	\N	2020-10-12 16:45:12.342189
82706	CSII	2020-09-30	35.138	39.71	35.138	39.345	508729	{}	\N	2020-10-12 16:45:12.348286
82707	CSII	2020-10-01	39.59	39.64	37.32	37.75	352022	{}	\N	2020-10-12 16:45:12.354915
82708	CSII	2020-10-02	37.09	38.11	36.23	37.5	277480	{}	\N	2020-10-12 16:45:12.360607
82709	CSII	2020-10-05	37.95	38.27	36.51	37.58	388200	{}	\N	2020-10-12 16:45:12.366229
82710	CSII	2020-10-06	37.85	38	36.22	36.89	387459	{}	\N	2020-10-12 16:45:12.372058
82711	CSII	2020-10-07	37.18	37.9	36.715	37.68	218655	{}	\N	2020-10-12 16:45:12.377095
82712	CSII	2020-10-08	38.35	38.35	37.24	37.5	332788	{}	\N	2020-10-12 16:45:12.382389
82713	CSII	2020-10-09	37.85	38.38	37.67	38.02	141205	{}	\N	2020-10-12 16:45:12.387882
82714	CSII	2020-10-12	38.14	38.5	37.58	37.605	48479	{}	\N	2020-10-12 16:45:12.393322
82715	CVLT	2020-09-28	40.54	41.01	40.18	40.97	221201	{}	\N	2020-10-12 16:45:14.000333
82716	CVLT	2020-09-29	41.23	41.36	40.87	41.15	213045	{}	\N	2020-10-12 16:45:14.011411
82717	CVLT	2020-09-30	41.15	41.38	40.55	40.8	227378	{}	\N	2020-10-12 16:45:14.019053
82718	CVLT	2020-10-01	40.97	41.59	40.74	41.41	270565	{}	\N	2020-10-12 16:45:14.025607
82719	CVLT	2020-10-02	38.9	41.03	38.9	40.44	363467	{}	\N	2020-10-12 16:45:14.031303
82720	CVLT	2020-10-05	40.79	41.71	40.6	41.43	295688	{}	\N	2020-10-12 16:45:14.036837
82721	CVLT	2020-10-06	41.49	42.9	41.26	41.95	379464	{}	\N	2020-10-12 16:45:14.041958
82722	CVLT	2020-10-07	42.12	43.12	42.12	42.48	261456	{}	\N	2020-10-12 16:45:14.046853
82723	CVLT	2020-10-08	42.64	43.22	42.51	43.2	210260	{}	\N	2020-10-12 16:45:14.052145
82724	CVLT	2020-10-09	43.54	43.87	43.32	43.81	188950	{}	\N	2020-10-12 16:45:14.056658
82725	CVLT	2020-10-12	44	44.73	43.85	44.55	54337	{}	\N	2020-10-12 16:45:14.061193
82726	CWS	2020-09-28	38.855	38.855	38.75	38.75	798	{}	\N	2020-10-12 16:45:15.498759
82727	CWS	2020-09-29	38.93	39.27	38.231	38.231	4722	{}	\N	2020-10-12 16:45:15.510031
82728	CWS	2020-09-30	39.3499	39.3499	38.9804	38.9804	663	{}	\N	2020-10-12 16:45:15.517301
82729	CWS	2020-10-01	39.22	39.22	39.0102	39.0102	678	{}	\N	2020-10-12 16:45:15.523811
82730	CWS	2020-10-02	38.9	38.9869	38.76	38.9869	760	{}	\N	2020-10-12 16:45:15.529492
82731	CWS	2020-10-05	39.16	39.2165	38.85	38.985	3832	{}	\N	2020-10-12 16:45:15.53419
82732	CWS	2020-10-06	39.6686	39.6686	39.1409	39.1409	279	{}	\N	2020-10-12 16:45:15.539264
82733	CWS	2020-10-07	39.54	39.7	39.54	39.7	531	{}	\N	2020-10-12 16:45:15.544726
82734	CWS	2020-10-08	40	40	39.84	39.96	1515	{}	\N	2020-10-12 16:45:15.54987
82735	CWS	2020-10-09	39.9	40.43	39.9	40.3699	4388	{}	\N	2020-10-12 16:45:15.554885
82736	CWS	2020-10-12	40.46	41	40.46	41	1555	{}	\N	2020-10-12 16:45:15.560107
82737	CZA	2020-09-28	65.45	66.49	65.45	66.49	22102	{}	\N	2020-10-12 16:45:16.530299
82738	CZA	2020-09-29	65.81	66.1262	65.37	65.55	9393	{}	\N	2020-10-12 16:45:16.53953
82739	CZA	2020-09-30	65.97	66.3583	65.4	65.9225	9395	{}	\N	2020-10-12 16:45:16.547394
82740	CZA	2020-10-01	65.95	66.5	65.88	66.19	14851	{}	\N	2020-10-12 16:45:16.554245
82741	CZA	2020-10-02	64.94	66.88	64.94	66.63	19503	{}	\N	2020-10-12 16:45:16.560248
82742	CZA	2020-10-05	67.18	68.38	67.1055	68.38	54997	{}	\N	2020-10-12 16:45:16.565742
82743	CZA	2020-10-06	67.87	68.81	67.5256	67.5256	13449	{}	\N	2020-10-12 16:45:16.571001
82744	CZA	2020-10-07	68.2	68.5201	68.0301	68.4707	12803	{}	\N	2020-10-12 16:45:16.576797
82745	CZA	2020-10-08	69.07	69.573	68.9343	69.523	5700	{}	\N	2020-10-12 16:45:16.581998
82746	CZA	2020-10-09	69.6	69.9688	69.45	69.7391	16283	{}	\N	2020-10-12 16:45:16.586852
82747	CZA	2020-10-12	69.78	70.0431	69.5964	69.95	10627	{}	\N	2020-10-12 16:45:16.591854
82748	DAN	2020-09-28	11.96	12.55	11.95	12.38	1408180	{}	\N	2020-10-12 16:45:17.963031
82749	DAN	2020-09-29	12.37	12.48	12.225	12.33	2201661	{}	\N	2020-10-12 16:45:17.975241
82750	DAN	2020-09-30	12.46	12.7	12.2	12.32	2339498	{}	\N	2020-10-12 16:45:17.983789
82751	DAN	2020-10-01	12.51	13.26	12.51	13.24	3435596	{}	\N	2020-10-12 16:45:17.990105
82752	DAN	2020-10-02	12.69	13.645	12.58	13.49	2673747	{}	\N	2020-10-12 16:45:17.995995
82753	DAN	2020-10-05	13.75	14.205	13.72	14.12	2483118	{}	\N	2020-10-12 16:45:18.001068
82754	DAN	2020-10-06	14.44	14.705	13.77	13.78	3321250	{}	\N	2020-10-12 16:45:18.005824
82755	DAN	2020-10-07	14.14	14.59	13.96	14.52	2439902	{}	\N	2020-10-12 16:45:18.010434
82756	DAN	2020-10-08	15.67	16.22	14.97	15.03	3462613	{}	\N	2020-10-12 16:45:18.015321
82757	DAN	2020-10-09	15.14	15.48	15.045	15.08	1927738	{}	\N	2020-10-12 16:45:18.020048
82758	DAN	2020-10-12	15.15	15.18	14.91	15.03	547606	{}	\N	2020-10-12 16:45:18.024825
82759	DMLP	2020-09-28	10.19	10.5	10.0847	10.42	52633	{}	\N	2020-10-12 16:45:20.825469
82760	DMLP	2020-09-29	10.315	10.57	10.235	10.5	40720	{}	\N	2020-10-12 16:45:20.837291
82761	DMLP	2020-09-30	10.38	10.505	10.15	10.33	46951	{}	\N	2020-10-12 16:45:20.845029
82762	DMLP	2020-10-01	10.32	10.49	10.08	10.31	47762	{}	\N	2020-10-12 16:45:20.851015
82763	DMLP	2020-10-02	10.25	10.55	10.15	10.32	50651	{}	\N	2020-10-12 16:45:20.85729
82764	DMLP	2020-10-05	10.47	10.8399	10.4485	10.54	34702	{}	\N	2020-10-12 16:45:20.862229
82765	DMLP	2020-10-06	10.71	10.71	10.21	10.28	52753	{}	\N	2020-10-12 16:45:20.86693
82766	DMLP	2020-10-07	10.21	10.36	10.125	10.3	53554	{}	\N	2020-10-12 16:45:20.871626
82767	DMLP	2020-10-08	10.31	10.58	10.31	10.46	85038	{}	\N	2020-10-12 16:45:20.876405
82768	DMLP	2020-10-09	10.51	10.53	10.3	10.39	53699	{}	\N	2020-10-12 16:45:20.881166
82769	DMLP	2020-10-12	10.4	10.55	10.4	10.5071	27288	{}	\N	2020-10-12 16:45:20.886036
82770	DOGZ	2020-09-28	1.06	1.1354	1.06	1.1354	1300	{}	\N	2020-10-12 16:45:22.735634
82771	DOGZ	2020-09-29	1.07	1.09	1.04	1.09	2336	{}	\N	2020-10-12 16:45:22.743857
82772	DOGZ	2020-09-30	1.0401	1.0899	1.0401	1.06	1158	{}	\N	2020-10-12 16:45:22.749697
82773	DOGZ	2020-10-01	1.05	1.05	1.05	1.05	270	{}	\N	2020-10-12 16:45:22.755561
82774	DOGZ	2020-10-02	1.05	1.05	1.04	1.04	2682	{}	\N	2020-10-12 16:45:22.761141
82775	DOGZ	2020-10-05	1.06	1.065	1.0436	1.05	2946	{}	\N	2020-10-12 16:45:22.765936
82776	DOGZ	2020-10-06	1.06	1.06	1.06	1.06	1783	{}	\N	2020-10-12 16:45:22.770925
82777	DOGZ	2020-10-07	1.14	1.14	1.13	1.13	1604	{}	\N	2020-10-12 16:45:22.776154
82778	DOGZ	2020-10-08	1.0999	1.0999	1.08	1.08	1547	{}	\N	2020-10-12 16:45:22.78091
82779	DOGZ	2020-10-09	1.1	1.1	1.0501	1.0501	2937	{}	\N	2020-10-12 16:45:22.785407
82780	DOGZ	2020-10-12	1.06	1.07	1.06	1.07	359	{}	\N	2020-10-12 16:45:22.790236
82781	DRW	2020-09-28	24.17	24.1999	24.15	24.15	10549	{}	\N	2020-10-12 16:45:23.750978
82782	DRW	2020-09-29	24	24.06	23.94	23.99	5998	{}	\N	2020-10-12 16:45:23.764331
82783	DRW	2020-09-30	24.2	24.345	24.2	24.285	3945	{}	\N	2020-10-12 16:45:23.77246
82784	DRW	2020-10-01	24.442	24.5658	24.44	24.5658	17216	{}	\N	2020-10-12 16:45:23.779531
82785	DRW	2020-10-02	24.451	24.6051	24.451	24.495	11703	{}	\N	2020-10-12 16:45:23.78583
82786	DRW	2020-10-05	24.65	24.75	24.65	24.75	18701	{}	\N	2020-10-12 16:45:23.791282
82787	DRW	2020-10-06	24.838	24.85	24.69	24.69	137186	{}	\N	2020-10-12 16:45:23.797157
82788	DRW	2020-10-07	24.9	24.9	24.78	24.86	5812	{}	\N	2020-10-12 16:45:23.802389
82789	DRW	2020-10-08	24.895	24.9399	24.84	24.84	13129	{}	\N	2020-10-12 16:45:23.808033
82790	DRW	2020-10-09	24.9199	24.9799	24.9	24.93	4077	{}	\N	2020-10-12 16:45:23.814311
82791	DRW	2020-10-12	24.98	25.097	24.95	25.071	12604	{}	\N	2020-10-12 16:45:23.820237
82792	DVYE	2020-09-28	29.91	29.95	29.65	29.75	84642	{}	\N	2020-10-12 16:45:25.385964
82793	DVYE	2020-09-29	29.7614	29.91	29.7	29.82	93583	{}	\N	2020-10-12 16:45:25.395918
82794	DVYE	2020-09-30	29.95	30.1899	29.9	30.11	140254	{}	\N	2020-10-12 16:45:25.40285
82795	DVYE	2020-10-01	30.17	30.17	29.86	30.02	4425107	{}	\N	2020-10-12 16:45:25.409349
82796	DVYE	2020-10-02	29.75	30.1102	29.75	29.9	180229	{}	\N	2020-10-12 16:45:25.415332
82797	DVYE	2020-10-05	29.87	30.105	29.8	30.05	308581	{}	\N	2020-10-12 16:45:25.420575
82798	DVYE	2020-10-06	30.27	30.35	29.95	30.04	602149	{}	\N	2020-10-12 16:45:25.42686
82799	DVYE	2020-10-07	30.14	30.29	30.0705	30.25	140984	{}	\N	2020-10-12 16:45:25.433497
82800	DVYE	2020-10-08	30.25	30.49	30.225	30.45	199062	{}	\N	2020-10-12 16:45:25.438973
82801	DVYE	2020-10-09	30.55	30.77	30.45	30.62	229221	{}	\N	2020-10-12 16:45:25.444145
82802	DVYE	2020-10-12	30.66	30.77	30.5801	30.74	25511	{}	\N	2020-10-12 16:45:25.449765
82803	COO	2020-09-28	337.37	340.68	333.705	334.12	195312	{}	\N	2020-10-12 16:45:26.936277
82804	COO	2020-09-29	335.49	339.925	333.26	333.59	366548	{}	\N	2020-10-12 16:45:26.94657
82805	COO	2020-09-30	334.87	340.535	334.5	337.12	521531	{}	\N	2020-10-12 16:45:26.954393
82806	COO	2020-10-01	339.97	341.61	336.67	338.77	413655	{}	\N	2020-10-12 16:45:26.960315
82807	COO	2020-10-02	335.27	337.99	329.3	335.43	357034	{}	\N	2020-10-12 16:45:26.966498
82808	COO	2020-10-05	338	343.72	338	342.82	418900	{}	\N	2020-10-12 16:45:26.971457
82809	COO	2020-10-06	341.75	349.83	340.81	342.02	432545	{}	\N	2020-10-12 16:45:26.976234
82810	COO	2020-10-07	343.1	351.74	343.1	350.87	368584	{}	\N	2020-10-12 16:45:26.981275
82811	COO	2020-10-08	350	356.61	349.1566	354.75	280461	{}	\N	2020-10-12 16:45:26.986569
82812	COO	2020-10-09	356.85	359.4	354.775	357.85	268611	{}	\N	2020-10-12 16:45:26.99142
82813	COO	2020-10-12	359.13	364.05	356.97	363.245	79418	{}	\N	2020-10-12 16:45:26.996272
82814	CYB	2020-09-28	26.26	26.27	25.9989	26.27	10556	{}	\N	2020-10-12 16:45:28.17007
82815	CYB	2020-09-29	26.18	26.2	25.95	26.18	5200	{}	\N	2020-10-12 16:45:28.181095
82816	CYB	2020-09-30	26.18	26.32	26.18	26.32	4283	{}	\N	2020-10-12 16:45:28.189753
82817	CYB	2020-10-01	26.26	26.4	26.02	26.39	13902	{}	\N	2020-10-12 16:45:28.196766
82818	CYB	2020-10-02	26.4	26.4	26.1	26.29	10139	{}	\N	2020-10-12 16:45:28.203363
82819	CYB	2020-10-05	26.4	26.4	26.35	26.3501	3084	{}	\N	2020-10-12 16:45:28.209145
82820	CYB	2020-10-06	26.35	26.4	26.35	26.38	3062	{}	\N	2020-10-12 16:45:28.214555
82821	CYB	2020-10-07	26.3787	26.4	26.37	26.38	7820	{}	\N	2020-10-12 16:45:28.219767
82822	CYB	2020-10-08	26.41	26.445	26.41	26.445	2036	{}	\N	2020-10-12 16:45:28.224892
82823	CYB	2020-10-09	26.44	26.99	26.44	26.69	9773	{}	\N	2020-10-12 16:45:28.229819
82824	CYB	2020-10-12	26.7	26.7	26.51	26.538	11470	{}	\N	2020-10-12 16:45:28.234882
82825	DEF	2020-09-28	55.205	55.6304	55.205	55.4432	11167	{}	\N	2020-10-12 16:45:29.192195
82826	DEF	2020-09-29	55.3201	55.6784	55.2777	55.28	12883	{}	\N	2020-10-12 16:45:29.203427
82827	DEF	2020-09-30	55.47	55.9589	55.455	55.525	10795	{}	\N	2020-10-12 16:45:29.210919
82828	DEF	2020-10-01	56	56	55.3633	55.5042	18369	{}	\N	2020-10-12 16:45:29.216979
82829	DEF	2020-10-02	54.89	55.5377	54.89	55.385	25563	{}	\N	2020-10-12 16:45:29.221837
82830	DEF	2020-10-05	55.56	56.065	55.56	56.04	39693	{}	\N	2020-10-12 16:45:29.226558
82831	DEF	2020-10-06	55.96	56.35	55.34	55.34	5983	{}	\N	2020-10-12 16:45:29.23058
82832	DEF	2020-10-07	56.01	56.34	55.9838	56.23	33434	{}	\N	2020-10-12 16:45:29.23454
82833	DEF	2020-10-08	56.5	56.6718	56.5	56.6599	5768	{}	\N	2020-10-12 16:45:29.238883
82834	DEF	2020-10-09	57.12	57.31	56.945	56.99	12595	{}	\N	2020-10-12 16:45:29.242884
82835	DEF	2020-10-12	57.5	57.7916	57.37	57.7916	7129	{}	\N	2020-10-12 16:45:29.247532
82836	DLN	2020-09-28	96.59	97.31	96.5	96.95	75732	{}	\N	2020-10-12 16:45:31.714947
82837	DLN	2020-09-29	96.96	96.96	95.93	96.24	74227	{}	\N	2020-10-12 16:45:31.724159
82838	DLN	2020-09-30	96.58	97.85	96.51	96.99	114558	{}	\N	2020-10-12 16:45:31.732016
82839	DLN	2020-10-01	97.54	97.72	96.65	97.14	83729	{}	\N	2020-10-12 16:45:31.738469
82840	DLN	2020-10-02	95.66	97.4398	95.66	96.95	225026	{}	\N	2020-10-12 16:45:31.744273
82841	DLN	2020-10-05	97.52	98.51	97.5188	98.49	114975	{}	\N	2020-10-12 16:45:31.748913
82842	DLN	2020-10-06	98.71	99.37	97.37	97.4	64927	{}	\N	2020-10-12 16:45:31.753592
82843	DLN	2020-10-07	98.2	99.18	98.2	98.97	35289	{}	\N	2020-10-12 16:45:31.758112
82844	DLN	2020-10-08	99.52	100	99.3	99.98	39246	{}	\N	2020-10-12 16:45:31.762824
82845	DLN	2020-10-09	100.37	100.76	100.052	100.36	67582	{}	\N	2020-10-12 16:45:31.768298
82846	DLN	2020-10-12	100.85	101.55	100.7199	101.53	26801	{}	\N	2020-10-12 16:45:31.773365
82847	DMRM	2020-09-28	47.53	47.6744	47.3273	47.6744	1303	{}	\N	2020-10-12 16:45:32.809839
82848	DMRM	2020-09-29	47.8	47.83	47.32	47.34	1405	{}	\N	2020-10-12 16:45:32.822797
82849	DMRM	2020-09-30	47.7	47.84	47.46	47.84	1528	{}	\N	2020-10-12 16:45:32.831739
82850	DMRM	2020-10-01	47.95	47.95	47.787	47.787	356	{}	\N	2020-10-12 16:45:32.838175
82851	DMRM	2020-10-02	47.78	48.08	47.1425	48.04	12698	{}	\N	2020-10-12 16:45:32.843848
82852	DMRM	2020-10-05	48.35	48.4364	48.2	48.4364	765	{}	\N	2020-10-12 16:45:32.848773
82853	DMRM	2020-10-06	48.69	48.77	48.3631	48.3631	708	{}	\N	2020-10-12 16:45:32.853402
82854	DMRM	2020-10-07	48.81	48.883	48.81	48.883	600	{}	\N	2020-10-12 16:45:32.858101
82855	DMRM	2020-10-08	49.05	49.05	48.8608	48.8608	600	{}	\N	2020-10-12 16:45:32.862899
82856	DMRM	2020-10-09	49.27	49.27	49.0814	49.0814	484	{}	\N	2020-10-12 16:45:32.867038
82857	DMRM	2020-10-12	49.21	49.21	49	49	802	{}	\N	2020-10-12 16:45:32.871193
82858	DNI	2020-09-28	9.33	9.39	9.3	9.34	101428	{}	\N	2020-10-12 16:45:35.850081
82859	DNI	2020-09-29	9.38	9.38	8.86	9.02	913258	{}	\N	2020-10-12 16:45:35.858989
82860	DNI	2020-09-30	9.04	9.145	9.04	9.1	376865	{}	\N	2020-10-12 16:45:35.865347
82861	DNI	2020-10-01	9.22	9.505	9.17	9.44	224614	{}	\N	2020-10-12 16:45:35.870832
82862	DNI	2020-10-02	9.3	9.56	9.25	9.5	203767	{}	\N	2020-10-12 16:45:35.876256
82863	DNI	2020-10-05	9.57	9.8352	9.57	9.79	185708	{}	\N	2020-10-12 16:45:35.880445
82864	DNI	2020-10-06	9.81	9.85	9.47	9.53	244271	{}	\N	2020-10-12 16:45:35.884623
82865	DNI	2020-10-07	9.56	9.76	9.51	9.56	306876	{}	\N	2020-10-12 16:45:35.889024
82866	DNI	2020-10-08	9.59	9.83	9.59	9.79	178501	{}	\N	2020-10-12 16:45:35.893126
82867	DOG	2020-09-28	44.19	44.26	43.8601	44.06	1327986	{}	\N	2020-10-12 16:45:38.043897
82868	DOG	2020-09-29	44.11	44.49	44.0698	44.3	379315	{}	\N	2020-10-12 16:45:38.052999
82869	DOG	2020-09-30	44.23	44.23	43.3899	43.79	1213441	{}	\N	2020-10-12 16:45:38.058994
82870	DOG	2020-10-01	43.51	43.94	43.36	43.71	719886	{}	\N	2020-10-12 16:45:38.065209
82871	DOG	2020-10-02	44.35	44.405	43.665	43.91	1052826	{}	\N	2020-10-12 16:45:38.07047
82872	DOG	2020-10-05	43.6	43.605	43.15	43.19	1503565	{}	\N	2020-10-12 16:45:38.076116
82873	DOG	2020-10-06	43	43.81	42.86	43.74	1916906	{}	\N	2020-10-12 16:45:38.081733
82874	DOG	2020-10-07	43.34	43.34	42.81	42.93	728914	{}	\N	2020-10-12 16:45:38.086564
82875	DOG	2020-10-08	42.69	42.95	42.67	42.7	563606	{}	\N	2020-10-12 16:45:38.091826
82876	DOG	2020-10-09	42.53	42.69	42.34	42.48	534583	{}	\N	2020-10-12 16:45:38.096647
82877	DOG	2020-10-12	42.3	42.3499	41.99	42.0001	282979	{}	\N	2020-10-12 16:45:38.101319
82878	DUSA	2020-09-28	24.98	25.1799	24.98	25.115	10356	{}	\N	2020-10-12 16:45:40.641327
82879	DUSA	2020-09-29	25.0941	25.0941	24.88	25.02	40736	{}	\N	2020-10-12 16:45:40.653874
82880	DUSA	2020-09-30	25.1001	25.385	25.1001	25.2	9994	{}	\N	2020-10-12 16:45:40.663363
82881	DUSA	2020-10-01	25.272	25.3899	25.2	25.36	4848	{}	\N	2020-10-12 16:45:40.671486
82882	DUSA	2020-10-02	24.95	25.4477	24.95	25.3898	11317	{}	\N	2020-10-12 16:45:40.677778
82883	DUSA	2020-10-05	25.7	25.78	25.66	25.73	21366	{}	\N	2020-10-12 16:45:40.683961
82884	DUSA	2020-10-06	25.93	25.9975	25.55	25.55	3311	{}	\N	2020-10-12 16:45:40.690016
82885	DUSA	2020-10-07	25.88	26	25.8	25.94	24010	{}	\N	2020-10-12 16:45:40.695949
82886	DUSA	2020-10-08	26.03	26.25	26.03	26.2139	10456	{}	\N	2020-10-12 16:45:40.701999
82887	DUSA	2020-10-09	26.36	26.4949	26.2656	26.425	9347	{}	\N	2020-10-12 16:45:40.707503
82888	DUSA	2020-10-12	26.59	26.8571	26.59	26.8447	1689	{}	\N	2020-10-12 16:45:40.713188
82889	DUSL	2020-09-28	19.97	20.75	19.97	20.26	88512	{}	\N	2020-10-12 16:45:41.67864
82890	DUSL	2020-09-29	20.4	20.48	19.58	19.77	81368	{}	\N	2020-10-12 16:45:41.690156
82891	DUSL	2020-09-30	20.14	20.6298	19.34	19.57	189552	{}	\N	2020-10-12 16:45:41.697182
82892	DUSL	2020-10-01	19.94	20.18	19.0592	19.371	120559	{}	\N	2020-10-12 16:45:41.703243
82893	DUSL	2020-10-02	18.35	20.4013	18.35	20.1	102200	{}	\N	2020-10-12 16:45:41.708637
82894	DUSL	2020-10-05	20.61	21.15	20.61	21.05	64499	{}	\N	2020-10-12 16:45:41.714138
82895	DUSL	2020-10-06	21.46	21.72	20.25	20.25	128484	{}	\N	2020-10-12 16:45:41.718322
82896	DUSL	2020-10-07	20.99	21.75	20.99	21.57	83494	{}	\N	2020-10-12 16:45:41.722524
82897	DUSL	2020-10-08	22.04	22.2599	21.57	22.245	68418	{}	\N	2020-10-12 16:45:41.728006
82898	DUSL	2020-10-09	22.75	22.949	22.3	22.46	84004	{}	\N	2020-10-12 16:45:41.732389
82899	DUSL	2020-10-12	22.53	23.049	22.53	23.049	46013	{}	\N	2020-10-12 16:45:41.736735
82900	EBR	2020-09-28	5.89	5.91	5.4	5.52	900062	{}	\N	2020-10-12 16:45:42.777862
82901	EBR	2020-09-29	5.41	5.505	5.33	5.43	1502466	{}	\N	2020-10-12 16:45:42.78785
82902	EBR	2020-09-30	5.44	5.53	5.41	5.52	2734257	{}	\N	2020-10-12 16:45:42.795413
82903	EBR	2020-10-01	5.39	5.4	5.28	5.39	1113114	{}	\N	2020-10-12 16:45:42.802267
82904	EBR	2020-10-02	5.35	5.505	5.29	5.34	861568	{}	\N	2020-10-12 16:45:42.808127
82905	EBR	2020-10-05	5.28	5.74	5.265	5.64	998970	{}	\N	2020-10-12 16:45:42.814521
82906	EBR	2020-10-06	5.73	5.88	5.59	5.66	945205	{}	\N	2020-10-12 16:45:42.820266
82907	EBR	2020-10-07	5.6	5.68	5.425	5.61	817496	{}	\N	2020-10-12 16:45:42.825636
82908	EBR	2020-10-08	5.5	5.61	5.49	5.58	789915	{}	\N	2020-10-12 16:45:42.831159
82909	EBR	2020-10-09	5.64	5.7486	5.58	5.64	661009	{}	\N	2020-10-12 16:45:42.836782
82910	EBR	2020-10-12	5.69	6.02	5.64	5.94	516347	{}	\N	2020-10-12 16:45:42.842444
82911	CRI	2020-09-28	88.67	90.7	87.93	88.69	899622	{}	\N	2020-10-12 16:45:43.763624
82912	CRI	2020-09-29	88.76	89.23	84.43	85.62	1342686	{}	\N	2020-10-12 16:45:43.775778
82913	CRI	2020-09-30	85.9	87.81	85.9	86.58	971926	{}	\N	2020-10-12 16:45:43.783233
82914	CRI	2020-10-01	86.31	88.3	85.68	87.55	1507189	{}	\N	2020-10-12 16:45:43.789744
82915	CRI	2020-10-02	86.05	89.15	85.6	88.58	1039423	{}	\N	2020-10-12 16:45:43.796009
82916	CRI	2020-10-05	89.22	90.4	85.96	86.39	1760332	{}	\N	2020-10-12 16:45:43.801859
82917	CRI	2020-10-06	86.77	87.34	82.87	83.88	1584080	{}	\N	2020-10-12 16:45:43.806691
82918	CRI	2020-10-07	84.73	86.67	84.73	86.53	1087497	{}	\N	2020-10-12 16:45:43.811323
82919	CRI	2020-10-08	87.28	87.73	86.07	87.63	502863	{}	\N	2020-10-12 16:45:43.817198
82920	CRI	2020-10-09	88.53	89.46	87.83	88.96	551093	{}	\N	2020-10-12 16:45:43.822788
82921	CRI	2020-10-12	89.17	89.7	87.03	87.945	385738	{}	\N	2020-10-12 16:45:43.827568
82922	CTR	2020-09-28	9.87	10.19	9.705	10.03	30918	{}	\N	2020-10-12 16:45:46.717636
82923	CTR	2020-09-29	10	10.27	10	10.184	129046	{}	\N	2020-10-12 16:45:46.729653
82924	CTR	2020-09-30	10.24	10.43	10.01	10.01	110694	{}	\N	2020-10-12 16:45:46.738002
82925	CTR	2020-10-01	10.09	10.248	9.72	9.75	100550	{}	\N	2020-10-12 16:45:46.743849
82926	CTR	2020-10-02	9.54	9.94	9.54	9.93	78929	{}	\N	2020-10-12 16:45:46.748832
82927	CTR	2020-10-05	10.06	10.61	10.06	10.61	94340	{}	\N	2020-10-12 16:45:46.755011
82928	CTR	2020-10-06	10.72	10.9924	10.5501	10.7	75832	{}	\N	2020-10-12 16:45:46.759338
82929	CTR	2020-10-07	10.67	10.99	10.5488	10.81	113614	{}	\N	2020-10-12 16:45:46.763288
82930	CTR	2020-10-08	10.89	11.56	10.89	11.46	98422	{}	\N	2020-10-12 16:45:46.767307
82931	CTR	2020-10-09	11.57	11.7999	11.3701	11.61	165010	{}	\N	2020-10-12 16:45:46.771338
82932	CTR	2020-10-12	11.6	11.6	11.43	11.4451	32248	{}	\N	2020-10-12 16:45:46.775617
82933	DBV	2020-09-28	23.2049	23.2049	23.2049	23.2049	9	{}	\N	2020-10-12 16:45:48.979882
82934	DBV	2020-09-29	23.165	23.165	23.165	23.165	4	{}	\N	2020-10-12 16:45:48.988173
82935	DBV	2020-09-30	23.35	23.35	23.35	23.35	119	{}	\N	2020-10-12 16:45:48.995172
82936	DBV	2020-10-01	23.35	23.48	23.35	23.48	773	{}	\N	2020-10-12 16:45:49.001394
82937	DBV	2020-10-02	23.4459	23.4459	23.355	23.355	255	{}	\N	2020-10-12 16:45:49.007281
82938	DBV	2020-10-05	23.55	23.55	23.4299	23.4299	124	{}	\N	2020-10-12 16:45:49.012692
82939	DBV	2020-10-06	23.3681	23.3681	23.3242	23.3242	120	{}	\N	2020-10-12 16:45:49.018349
82940	DBV	2020-10-07	23.28	23.3049	23.28	23.3049	254	{}	\N	2020-10-12 16:45:49.023488
82941	DBV	2020-10-08	23.3306	23.3306	23.3306	23.3306	405	{}	\N	2020-10-12 16:45:49.028509
82942	DBV	2020-10-09	23.465	23.4953	23.31	23.31	1833	{}	\N	2020-10-12 16:45:49.03393
82943	DCP	2020-09-28	10.64	11.085	10.56	11.03	1331947	{}	\N	2020-10-12 16:45:50.270796
82944	DCP	2020-09-29	11	11.49	10.68	11.45	1721620	{}	\N	2020-10-12 16:45:50.284337
82945	DCP	2020-09-30	11.46	12.21	11.13	11.17	3515553	{}	\N	2020-10-12 16:45:50.292287
82946	DCP	2020-10-01	11.34	11.4099	10.5902	10.88	1751303	{}	\N	2020-10-12 16:45:50.298682
82947	DCP	2020-10-02	10.48	11.33	10.43	11.21	993490	{}	\N	2020-10-12 16:45:50.305104
82948	DCP	2020-10-05	11.3	11.99	11.26	11.98	836474	{}	\N	2020-10-12 16:45:50.310818
82949	DCP	2020-10-06	12.15	12.55	11.62	11.81	1776514	{}	\N	2020-10-12 16:45:50.316009
82950	DCP	2020-10-07	12.03	12.14	11.788	12.01	612542	{}	\N	2020-10-12 16:45:50.321015
82951	DCP	2020-10-08	12.19	12.88	11.99	12.74	1435927	{}	\N	2020-10-12 16:45:50.325898
82952	DCP	2020-10-09	12.78	12.82	12.31	12.7	1553509	{}	\N	2020-10-12 16:45:50.330913
82953	DCP	2020-10-12	12.65	12.68	12.32	12.43	428650	{}	\N	2020-10-12 16:45:50.335875
82954	DDLS	2020-09-28	27.6597	27.715	27.64	27.7	5505	{}	\N	2020-10-12 16:45:52.932213
82955	DDLS	2020-09-29	27.6	27.7299	27.6	27.7299	1209	{}	\N	2020-10-12 16:45:52.945765
82956	DDLS	2020-09-30	27.68	27.75	27.65	27.7412	3142	{}	\N	2020-10-12 16:45:52.953973
82957	DDLS	2020-10-01	27.74	27.785	27.7	27.7	698	{}	\N	2020-10-12 16:45:52.960582
82958	DDLS	2020-10-02	27.35	27.77	27.35	27.67	2754	{}	\N	2020-10-12 16:45:52.966343
82959	DDLS	2020-10-05	28.0258	28.1663	28.0258	28.1663	657	{}	\N	2020-10-12 16:45:52.971769
82960	DDLS	2020-10-06	28.0721	28.1294	28.0584	28.0584	650	{}	\N	2020-10-12 16:45:52.9766
82961	DDLS	2020-10-07	28.23	28.2975	28.23	28.2975	1424	{}	\N	2020-10-12 16:45:52.98172
82962	DDLS	2020-10-08	28.45	28.45	28.4332	28.4332	1064	{}	\N	2020-10-12 16:45:52.986653
82963	DDLS	2020-10-09	28.59	28.59	28.5645	28.5645	999	{}	\N	2020-10-12 16:45:52.992055
82964	DDLS	2020-10-12	28.68	28.68	28.64	28.64	910	{}	\N	2020-10-12 16:45:52.997324
82965	DJD	2020-09-28	34.41	34.62	34.2477	34.42	12470	{}	\N	2020-10-12 16:45:54.632199
82966	DJD	2020-09-29	34.37	34.37	33.86	34.09	40282	{}	\N	2020-10-12 16:45:54.64247
82967	DJD	2020-09-30	34.21	34.605	34.21	34.36	12961	{}	\N	2020-10-12 16:45:54.6503
82968	DJD	2020-10-01	34.45	34.45	34.09	34.27	11101	{}	\N	2020-10-12 16:45:54.657811
82969	DJD	2020-10-02	33.8	34.38	33.7309	34.3	13267	{}	\N	2020-10-12 16:45:54.664144
82970	DJD	2020-10-05	34.5	34.7902	34.5	34.7902	10541	{}	\N	2020-10-12 16:45:54.669889
82971	DJD	2020-10-06	34.89	35.1799	34.5	34.5892	15625	{}	\N	2020-10-12 16:45:54.675755
82972	DJD	2020-10-07	34.59	35.0627	34.59	34.96	6896	{}	\N	2020-10-12 16:45:54.681708
82973	DJD	2020-10-08	35.16	35.5	35.16	35.4	15369	{}	\N	2020-10-12 16:45:54.687293
82974	DJD	2020-10-09	35.6	35.64	35.32	35.32	19020	{}	\N	2020-10-12 16:45:54.693362
82975	DJD	2020-10-12	35.5	35.72	35.5	35.72	3184	{}	\N	2020-10-12 16:45:54.69921
82976	DMO	2020-09-28	12.94	13.0855	12.92	13.08	73846	{}	\N	2020-10-12 16:45:56.278985
82977	DMO	2020-09-29	13.11	13.27	13.1	13.14	116262	{}	\N	2020-10-12 16:45:56.288671
82978	DMO	2020-09-30	13.15	13.175	13.0768	13.12	178046	{}	\N	2020-10-12 16:45:56.295646
82979	DMO	2020-10-01	13.21	13.25	13.09	13.23	106353	{}	\N	2020-10-12 16:45:56.301925
82980	DMO	2020-10-02	13.13	13.48	13.06	13.46	120090	{}	\N	2020-10-12 16:45:56.307842
82981	DMO	2020-10-05	13.4	13.55	13.4	13.52	77948	{}	\N	2020-10-12 16:45:56.313326
82982	DMO	2020-10-06	13.47	13.74	13.47	13.74	90476	{}	\N	2020-10-12 16:45:56.318539
82983	DMO	2020-10-07	13.75	13.75	13.675	13.75	46092	{}	\N	2020-10-12 16:45:56.323432
82984	DMO	2020-10-08	13.75	13.8	13.75	13.75	52502	{}	\N	2020-10-12 16:45:56.328541
82985	DMO	2020-10-09	13.75	13.75	13.56	13.62	62132	{}	\N	2020-10-12 16:45:56.333101
82986	DMO	2020-10-12	13.63	13.63	13.53	13.53	15240	{}	\N	2020-10-12 16:45:56.337832
82987	DMRS	2020-09-28	47.07	47.07	46.9648	46.9648	1051	{}	\N	2020-10-12 16:45:57.334474
82988	DMRS	2020-09-29	47.13	47.13	46.7373	46.74	1338	{}	\N	2020-10-12 16:45:57.344928
82989	DMRS	2020-09-30	47.07	47.13	46.8473	46.85	1428	{}	\N	2020-10-12 16:45:57.352498
82990	DMRS	2020-10-01	47.33	47.33	46.8511	46.8511	824	{}	\N	2020-10-12 16:45:57.35937
82991	DMRS	2020-10-02	47.21	47.2723	46.95	47.2723	3216	{}	\N	2020-10-12 16:45:57.36477
82992	DMRS	2020-10-05	47.6511	47.6511	47.29	47.54	1407	{}	\N	2020-10-12 16:45:57.369619
82993	DMRS	2020-10-06	47.6171	47.6171	47.6171	47.6171	0	{}	\N	2020-10-12 16:45:57.37453
82994	DMRS	2020-10-07	47.87	48	47.7731	47.7731	809	{}	\N	2020-10-12 16:45:57.379488
82995	DMRS	2020-10-09	48.0807	48.0807	48.0807	48.0807	302	{}	\N	2020-10-12 16:45:57.384459
82996	DMRS	2020-10-12	48.1211	48.1211	48.1211	48.1211	218	{}	\N	2020-10-12 16:45:57.38921
82997	DNL	2020-09-28	66.36	66.36	66.16	66.2099	5160	{}	\N	2020-10-12 16:45:59.563679
82998	DNL	2020-09-29	66.51	66.7	66.37	66.54	21920	{}	\N	2020-10-12 16:45:59.575708
82999	DNL	2020-09-30	66.49	66.8599	66.38	66.6562	9608	{}	\N	2020-10-12 16:45:59.584229
83000	DNL	2020-10-01	66.88	67.175	66.77	67.175	11560	{}	\N	2020-10-12 16:45:59.590169
83001	DNL	2020-10-02	66.29	66.96	66.29	66.8779	13955	{}	\N	2020-10-12 16:45:59.595763
83002	DNL	2020-10-05	67.27	67.79	67.27	67.76	8232	{}	\N	2020-10-12 16:45:59.601067
83003	DNL	2020-10-06	67.67	67.79	66.99	67.11	47630	{}	\N	2020-10-12 16:45:59.605759
83004	DNL	2020-10-07	67.6	67.77	67.4	67.71	14297	{}	\N	2020-10-12 16:45:59.610191
83005	DNL	2020-10-08	68.02	68.37	68.0001	68.35	13096	{}	\N	2020-10-12 16:45:59.614785
83006	DNL	2020-10-09	68.72	68.9982	68.68	68.93	10783	{}	\N	2020-10-12 16:45:59.61915
83007	DNL	2020-10-12	69.33	69.4586	69.2301	69.3301	4213	{}	\N	2020-10-12 16:45:59.623761
83008	ZKIN	2020-09-28	1.33	1.34	1.26	1.3	22277	{}	\N	2020-10-12 16:46:02.091584
83009	ZKIN	2020-09-29	1.31	1.32	1.26	1.3	32721	{}	\N	2020-10-12 16:46:02.102295
83010	ZKIN	2020-09-30	1.2759	1.29	1.2	1.2	7756	{}	\N	2020-10-12 16:46:02.110211
83011	ZKIN	2020-10-01	1.25	1.2699	1.22	1.25	6099	{}	\N	2020-10-12 16:46:02.116198
83012	ZKIN	2020-10-02	1.2	1.3	1.2	1.285	52290	{}	\N	2020-10-12 16:46:02.122135
83013	ZKIN	2020-10-05	1.2753	1.3	1.2753	1.29	9467	{}	\N	2020-10-12 16:46:02.127433
83014	ZKIN	2020-10-06	1.3	1.45	1.2801	1.41	117083	{}	\N	2020-10-12 16:46:02.132624
83015	ZKIN	2020-10-07	1.45	1.45	1.3757	1.41	12414	{}	\N	2020-10-12 16:46:02.13745
83016	ZKIN	2020-10-08	1.4086	1.4086	1.35	1.3869	15520	{}	\N	2020-10-12 16:46:02.142241
83017	ZKIN	2020-10-09	1.3831	1.4	1.33	1.4	8971	{}	\N	2020-10-12 16:46:02.146418
83018	ZKIN	2020-10-12	1.43	1.43	1.3512	1.4	5555	{}	\N	2020-10-12 16:46:02.150926
83019	ZOM	2020-09-28	0.1093	0.11	0.1	0.1026	33831802	{}	\N	2020-10-12 16:46:03.115416
83020	ZOM	2020-09-29	0.104	0.104	0.1	0.1015	16766659	{}	\N	2020-10-12 16:46:03.12528
83021	ZOM	2020-09-30	0.1	0.1101	0.0952	0.1101	18559984	{}	\N	2020-10-12 16:46:03.133145
83022	ZOM	2020-10-01	0.1	0.1015	0.0953	0.0979	19021650	{}	\N	2020-10-12 16:46:03.139606
83023	ZOM	2020-10-02	0.0938	0.0977	0.0925	0.0959	14671356	{}	\N	2020-10-12 16:46:03.146664
83024	ZOM	2020-10-05	0.0976	0.0989	0.091	0.096	13504577	{}	\N	2020-10-12 16:46:03.15188
83025	ZOM	2020-10-06	0.095	0.0959	0.092	0.092	14158692	{}	\N	2020-10-12 16:46:03.164719
83026	ZOM	2020-10-07	0.0939	0.094	0.091	0.0927	12387328	{}	\N	2020-10-12 16:46:03.168855
83027	ZOM	2020-10-08	0.0925	0.093	0.0895	0.092	16846395	{}	\N	2020-10-12 16:46:03.173031
83028	ZOM	2020-10-09	0.0911	0.0919	0.0895	0.0906	15940757	{}	\N	2020-10-12 16:46:03.17718
83029	ZOM	2020-10-12	0.09	0.09	0.085	0.0867	10341208	{}	\N	2020-10-12 16:46:03.181349
83030	DUST	2020-09-28	19.23	19.81	18.99	19.3	3697913	{}	\N	2020-10-12 16:46:04.264618
83031	DUST	2020-09-29	19.09	19.23	18.55	18.81	5386356	{}	\N	2020-10-12 16:46:04.274676
83032	DUST	2020-09-30	19.09	19.57	18.66	19.01	6605599	{}	\N	2020-10-12 16:46:04.282268
83033	DUST	2020-10-01	18.6	18.99	18.24	18.61	6231831	{}	\N	2020-10-12 16:46:04.289461
83034	DUST	2020-10-02	19.03	19.34	18.6	19.2	4898168	{}	\N	2020-10-12 16:46:04.294552
83035	DUST	2020-10-05	18.91	19.05	18.17	18.56	5463415	{}	\N	2020-10-12 16:46:04.298714
83036	DUST	2020-10-06	18.28	19.97	18.1	19.96	6629915	{}	\N	2020-10-12 16:46:04.302852
83037	DUST	2020-10-07	19.51	19.89	19.21	19.65	5741675	{}	\N	2020-10-12 16:46:04.307262
83038	DUST	2020-10-08	19.22	19.56	18.84	19.02	5522164	{}	\N	2020-10-12 16:46:04.311536
83039	DUST	2020-10-09	18.12	18.2	17.27	17.28	7584682	{}	\N	2020-10-12 16:46:04.315638
83040	DUST	2020-10-12	17.4	17.6389	17.1204	17.15	3286705	{}	\N	2020-10-12 16:46:04.319818
83041	ECHO	2020-09-28	26.14	27.14	26.14	26.81	124358	{}	\N	2020-10-12 16:46:06.10814
83042	ECHO	2020-09-29	26.71	27.05	26.24	26.26	82503	{}	\N	2020-10-12 16:46:06.120377
83043	ECHO	2020-09-30	26.35	26.82	25.64	25.77	138879	{}	\N	2020-10-12 16:46:06.128595
83044	ECHO	2020-10-01	26	26	25.27	25.545	156186	{}	\N	2020-10-12 16:46:06.134881
83045	ECHO	2020-10-02	25.24	26.46	25.2	26.3	216800	{}	\N	2020-10-12 16:46:06.140776
83046	ECHO	2020-10-05	26.58	27.61	26.58	27.43	124408	{}	\N	2020-10-12 16:46:06.145678
83047	ECHO	2020-10-06	27.68	27.84	25.98	26.06	254759	{}	\N	2020-10-12 16:46:06.150621
83048	ECHO	2020-10-07	26.23	27.32	25.575	27.12	170720	{}	\N	2020-10-12 16:46:06.155622
83049	ECHO	2020-10-08	27.52	28.42	27.04	27.57	151300	{}	\N	2020-10-12 16:46:06.160418
83050	ECHO	2020-10-09	28.1	28.49	27.26	27.52	78632	{}	\N	2020-10-12 16:46:06.165376
83051	ECHO	2020-10-12	27.46	27.79	27.08	27.755	24364	{}	\N	2020-10-12 16:46:06.170489
83052	EEH	2020-09-29	24.69	24.69	24.69	24.69	0	{}	\N	2020-10-12 16:46:07.763757
83053	EEH	2020-09-30	24.955	24.955	24.955	24.955	0	{}	\N	2020-10-12 16:46:07.773273
83054	EEH	2020-10-01	24.985	24.985	24.985	24.985	0	{}	\N	2020-10-12 16:46:07.788924
83055	EEH	2020-10-02	24.9	24.9	23.8593	23.8593	600	{}	\N	2020-10-12 16:46:07.796206
83056	EEH	2020-10-05	25.1133	25.1133	25.1133	25.1133	0	{}	\N	2020-10-12 16:46:07.802503
83057	EEH	2020-10-06	24.61	24.61	24.61	24.61	0	{}	\N	2020-10-12 16:46:07.809846
83058	EEH	2020-10-07	24.8802	24.8802	24.8802	24.8802	1	{}	\N	2020-10-12 16:46:07.815726
83059	EEH	2020-10-08	24.99	24.99	24.99	24.99	96	{}	\N	2020-10-12 16:46:07.821365
83060	EEH	2020-10-09	25.425	25.425	25.425	25.425	34	{}	\N	2020-10-12 16:46:07.826209
83061	EMBH	2020-09-28	22.28	22.3344	22.28	22.3344	426	{}	\N	2020-10-12 16:46:09.931997
83062	EMBH	2020-09-29	22.275	22.275	22.275	22.275	50	{}	\N	2020-10-12 16:46:09.941694
83063	EMBH	2020-09-30	22.39	22.4136	22.39	22.4136	168	{}	\N	2020-10-12 16:46:09.948383
83064	EMBH	2020-10-01	22.4862	22.4862	22.4862	22.4862	2	{}	\N	2020-10-12 16:46:09.954525
83065	EMBH	2020-10-02	22.3773	22.3773	22.3773	22.3773	0	{}	\N	2020-10-12 16:46:09.960171
83066	EMBH	2020-10-05	22.57	22.57	22.57	22.57	1	{}	\N	2020-10-12 16:46:09.965214
83067	EMBH	2020-10-06	22.46	22.515	22.46	22.515	398	{}	\N	2020-10-12 16:46:09.97046
83068	EMBH	2020-10-08	22.805	22.805	22.805	22.805	82	{}	\N	2020-10-12 16:46:09.97543
83069	EMBH	2020-10-09	22.9	22.9	22.9	22.9	7	{}	\N	2020-10-12 16:46:09.980306
83070	EMF	2020-09-28	15.19	15.27	15.19	15.22	14490	{}	\N	2020-10-12 16:46:12.656776
83071	EMF	2020-09-29	15.18	15.22	15.15	15.15	31710	{}	\N	2020-10-12 16:46:12.667999
83072	EMF	2020-09-30	15.2	15.38	15.2	15.37	45142	{}	\N	2020-10-12 16:46:12.675347
83073	EMF	2020-10-01	15.55	15.5524	15.295	15.47	109958	{}	\N	2020-10-12 16:46:12.681329
83074	EMF	2020-10-02	15.37	15.6	15.32	15.4	60202	{}	\N	2020-10-12 16:46:12.686254
83075	EMF	2020-10-05	15.41	15.58	15.2752	15.55	70758	{}	\N	2020-10-12 16:46:12.691406
83076	EMF	2020-10-06	15.6	15.69	15.1101	15.6	64542	{}	\N	2020-10-12 16:46:12.696144
83077	EMF	2020-10-07	15.66	15.8	15.66	15.8	67612	{}	\N	2020-10-12 16:46:12.700289
83078	EMF	2020-10-08	15.93	16	15.87	15.97	78758	{}	\N	2020-10-12 16:46:12.704551
83079	EMF	2020-10-09	16	16.17	15.97	16.06	72714	{}	\N	2020-10-12 16:46:12.708701
83080	EMF	2020-10-12	16.18	16.23	16.18	16.2	44184	{}	\N	2020-10-12 16:46:12.713146
83081	EQIX	2020-09-28	766.97	771.425	760.485	764.81	322631	{}	\N	2020-10-12 16:46:13.733219
83082	EQIX	2020-09-29	764.81	768.6	760.4204	763.46	281172	{}	\N	2020-10-12 16:46:13.744521
83083	EQIX	2020-09-30	767.57	770.16	752.675	759.14	300953	{}	\N	2020-10-12 16:46:13.753151
83084	EQIX	2020-10-01	764.26	771.61	756.02	768.36	320188	{}	\N	2020-10-12 16:46:13.760364
83085	EQIX	2020-10-02	764.38	779.81	760.34	778	345054	{}	\N	2020-10-12 16:46:13.767517
83086	EQIX	2020-10-05	777.79	788.27	762.81	788.26	295905	{}	\N	2020-10-12 16:46:13.773925
83087	EQIX	2020-10-06	786	796.94	775	790.83	379010	{}	\N	2020-10-12 16:46:13.779618
83088	EQIX	2020-10-07	790.82	805.765	789.61	803.26	340110	{}	\N	2020-10-12 16:46:13.785694
83089	EQIX	2020-10-08	803.04	835.59	797.5	824.4	610677	{}	\N	2020-10-12 16:46:13.791859
83090	EQIX	2020-10-09	834.28	834.28	825.68	829.35	343016	{}	\N	2020-10-12 16:46:13.798795
83091	EQIX	2020-10-12	832.88	834.35	823.94	830.37	106957	{}	\N	2020-10-12 16:46:13.805252
83092	ESGN	2020-09-28	22.16	22.16	22.16	22.16	7	{}	\N	2020-10-12 16:46:15.992643
83093	ESGN	2020-09-29	22.0283	22.0283	22.0283	22.0283	0	{}	\N	2020-10-12 16:46:16.001096
83094	ESGN	2020-09-30	22	22	21.81	21.835	1259	{}	\N	2020-10-12 16:46:16.007562
83095	ESGN	2020-10-01	21.78	21.8377	21.78	21.8377	153	{}	\N	2020-10-12 16:46:16.012913
83096	ESGN	2020-10-02	21.99	21.99	21.9611	21.9611	129	{}	\N	2020-10-12 16:46:16.018652
83097	ESGN	2020-10-05	22.32	22.32	22.32	22.32	42	{}	\N	2020-10-12 16:46:16.02342
83098	ESGN	2020-10-06	22.3	22.4037	22.225	22.225	3005	{}	\N	2020-10-12 16:46:16.028312
83099	ESGN	2020-10-07	22.3807	22.3807	22.3807	22.3807	150	{}	\N	2020-10-12 16:46:16.032449
83100	ESGN	2020-10-08	22.4377	22.519	22.4377	22.5178	1118	{}	\N	2020-10-12 16:46:16.036726
83101	ESGN	2020-10-09	22.575	22.575	22.575	22.575	5000	{}	\N	2020-10-12 16:46:16.041328
83102	EUM	2020-09-28	15.62	15.735	15.6	15.665	62484	{}	\N	2020-10-12 16:46:17.462315
83103	EUM	2020-09-29	15.7	15.73	15.63	15.67	54921	{}	\N	2020-10-12 16:46:17.473892
83104	EUM	2020-09-30	15.55	15.55	15.39	15.42	285406	{}	\N	2020-10-12 16:46:17.481667
83105	EUM	2020-10-01	15.34	15.38	15.27	15.27	82965	{}	\N	2020-10-12 16:46:17.48816
83106	EUM	2020-10-02	15.51	15.51	15.33	15.46	87440	{}	\N	2020-10-12 16:46:17.493677
83107	EUM	2020-10-05	15.35	15.35	15.245	15.25	95529	{}	\N	2020-10-12 16:46:17.498465
83108	EUM	2020-10-06	15.18	15.25	15.1	15.21	595587	{}	\N	2020-10-12 16:46:17.50322
83109	EUM	2020-10-07	15.05	15.11	15.01	15.04	106294	{}	\N	2020-10-12 16:46:17.507781
83110	EUM	2020-10-08	15	15.02	14.92	14.92	45604	{}	\N	2020-10-12 16:46:17.51244
83111	EUM	2020-10-09	14.9	14.92	14.79	14.83	57804	{}	\N	2020-10-12 16:46:17.516758
83112	EUM	2020-10-12	14.75	14.76	14.6701	14.6701	114717	{}	\N	2020-10-12 16:46:17.521415
83113	EWGS	2020-09-28	61.8193	61.8193	61.8193	61.8193	201	{}	\N	2020-10-12 16:46:18.960431
83114	EWGS	2020-09-29	62.3767	62.58	62.3767	62.58	3900	{}	\N	2020-10-12 16:46:18.972617
83115	EWGS	2020-09-30	62.7992	63.05	62.71	62.71	8715	{}	\N	2020-10-12 16:46:18.981107
83116	EWGS	2020-10-01	63.54	63.54	63.4367	63.4367	902	{}	\N	2020-10-12 16:46:18.987433
83117	EWGS	2020-10-02	63.06	63.2373	63.06	63.2373	336	{}	\N	2020-10-12 16:46:18.993394
83118	EWGS	2020-10-05	64.1544	64.45	64.1544	64.45	2028	{}	\N	2020-10-12 16:46:18.998335
83119	EWGS	2020-10-06	64.7769	64.83	63.7598	63.7598	9826	{}	\N	2020-10-12 16:46:19.003571
83120	EWGS	2020-10-07	64.48	64.48	63.81	64.05	7844	{}	\N	2020-10-12 16:46:19.009125
83121	EWGS	2020-10-08	64.09	64.245	64.0709	64.0709	727	{}	\N	2020-10-12 16:46:19.014806
83122	EWGS	2020-10-09	64.9	65.086	64.9	64.9503	13054	{}	\N	2020-10-12 16:46:19.020337
83123	EWGS	2020-10-12	65.08	65.31	65.08	65.31	609	{}	\N	2020-10-12 16:46:19.025758
83124	FFTY	2020-09-28	36.82	37.02	36.6	37	117382	{}	\N	2020-10-12 16:46:20.980479
83125	FFTY	2020-09-29	36.98	37.42	36.98	37.1	52922	{}	\N	2020-10-12 16:46:20.991848
83126	FFTY	2020-09-30	37.07	37.61	37	37.33	49951	{}	\N	2020-10-12 16:46:20.999808
83127	FFTY	2020-10-01	37.64	37.86	37.4605	37.79	136286	{}	\N	2020-10-12 16:46:21.00612
83128	FFTY	2020-10-02	37.05	37.58	36.98	37.13	68543	{}	\N	2020-10-12 16:46:21.011908
83129	FFTY	2020-10-05	37.43	38.11	37.43	38.11	125323	{}	\N	2020-10-12 16:46:21.017086
83130	FFTY	2020-10-06	38.19	38.73	37.81	37.89	77090	{}	\N	2020-10-12 16:46:21.023489
83131	FFTY	2020-10-07	38.38	38.66	38.38	38.61	67939	{}	\N	2020-10-12 16:46:21.029173
83132	FFTY	2020-10-08	39.02	39.02	38.5	38.78	26496	{}	\N	2020-10-12 16:46:21.03562
83133	FFTY	2020-10-09	39	39.39	38.98	39.3	67521	{}	\N	2020-10-12 16:46:21.041675
83134	FFTY	2020-10-12	39.81	39.9	39.51	39.9	47379	{}	\N	2020-10-12 16:46:21.04747
83135	FSS	2020-09-28	29.49	30.07	29.42	29.68	349946	{}	\N	2020-10-12 16:46:24.046314
83136	FSS	2020-09-29	29.67	29.86	29.11	29.59	509229	{}	\N	2020-10-12 16:46:24.059683
83137	FSS	2020-09-30	29.71	30.035	29.02	29.25	785283	{}	\N	2020-10-12 16:46:24.067812
83138	FSS	2020-10-01	29.25	29.69	28.83	29.23	558859	{}	\N	2020-10-12 16:46:24.075106
83139	FSS	2020-10-02	28.71	30.05	28.68	29.73	601014	{}	\N	2020-10-12 16:46:24.081277
83140	FSS	2020-10-05	29.99	30.46	29.99	30.26	738841	{}	\N	2020-10-12 16:46:24.086971
83141	FSS	2020-10-06	30.57	31.15	30.203	30.61	557598	{}	\N	2020-10-12 16:46:24.092513
83142	FSS	2020-10-07	30.8	31.16	30.685	31.02	736809	{}	\N	2020-10-12 16:46:24.097869
83143	FSS	2020-10-08	31.31	31.41	30.815	31.23	379660	{}	\N	2020-10-12 16:46:24.103344
83144	FSS	2020-10-09	31.56	31.815	31.07	31.2	316551	{}	\N	2020-10-12 16:46:24.108663
83145	FSS	2020-10-12	31.23	31.54	31.14	31.43	49484	{}	\N	2020-10-12 16:46:24.114095
83146	DWM	2020-09-28	45.48	45.68	45.4462	45.6	39860	{}	\N	2020-10-12 16:46:26.811726
83147	DWM	2020-09-29	45.58	45.64	45.382	45.49	16151	{}	\N	2020-10-12 16:46:26.818824
83148	DWM	2020-09-30	45.5	45.7207	45.2615	45.4734	28880	{}	\N	2020-10-12 16:46:26.823964
83149	DWM	2020-10-01	45.63	45.645	45.35	45.49	48812	{}	\N	2020-10-12 16:46:26.829792
83150	DWM	2020-10-02	45.01	45.6096	45.01	45.56	30586	{}	\N	2020-10-12 16:46:26.835529
83151	DWM	2020-10-05	45.89	46.21	45.89	46.15	36019	{}	\N	2020-10-12 16:46:26.842205
83152	DWM	2020-10-06	46.27	46.3	45.68	45.73	66533	{}	\N	2020-10-12 16:46:26.849115
83153	DWM	2020-10-07	46.08	46.219	46.0729	46.19	14566	{}	\N	2020-10-12 16:46:26.855169
83154	DWM	2020-10-08	46.29	46.48	46.28	46.4	27753	{}	\N	2020-10-12 16:46:26.861502
83155	DWM	2020-10-09	46.6	46.7	46.51	46.68	10453	{}	\N	2020-10-12 16:46:26.867392
83156	DWM	2020-10-12	46.78	46.91	46.682	46.91	6461	{}	\N	2020-10-12 16:46:26.872625
83157	DXR	2020-09-28	15.9957	15.9957	15.96	15.96	484	{}	\N	2020-10-12 16:46:28.03966
83158	DXR	2020-09-29	15.96	15.96	15.33	15.36	14939	{}	\N	2020-10-12 16:46:28.053316
83159	DXR	2020-09-30	15.82	15.85	15.12	15.75	27658	{}	\N	2020-10-12 16:46:28.06234
83160	DXR	2020-10-01	15.57	15.57	15.42	15.42	1138	{}	\N	2020-10-12 16:46:28.069086
83161	DXR	2020-10-02	15.77	15.77	15.77	15.77	309	{}	\N	2020-10-12 16:46:28.075184
83162	DXR	2020-10-05	15.9299	15.9299	15.5	15.51	1085	{}	\N	2020-10-12 16:46:28.080748
83163	DXR	2020-10-06	15.14	15.5533	15.14	15.33	2374	{}	\N	2020-10-12 16:46:28.085934
83164	DXR	2020-10-07	15.42	15.42	15.42	15.42	400	{}	\N	2020-10-12 16:46:28.091266
83165	DXR	2020-10-08	15.32	15.32	15.2861	15.315	1431	{}	\N	2020-10-12 16:46:28.09671
83166	DXR	2020-10-09	15.2543	15.3	15.01	15.18	29670	{}	\N	2020-10-12 16:46:28.101929
83167	DXR	2020-10-12	15.39	15.39	15.195	15.195	580	{}	\N	2020-10-12 16:46:28.107334
83168	EDOW	2020-09-28	24.18	24.3551	24.13	24.226	6508	{}	\N	2020-10-12 16:46:30.496605
83169	EDOW	2020-09-29	24.21	24.21	24.035	24.16	97462	{}	\N	2020-10-12 16:46:30.512371
83170	EDOW	2020-09-30	24.29	24.53	24.26	24.26	25862	{}	\N	2020-10-12 16:46:30.521483
83171	EDOW	2020-10-01	24.42	24.43	24.3	24.352	10794	{}	\N	2020-10-12 16:46:30.529082
83172	EDOW	2020-10-02	23.92	24.3671	23.92	24.3318	8097	{}	\N	2020-10-12 16:46:30.535487
83173	EDOW	2020-10-05	24.48	24.6344	24.43	24.6344	6566	{}	\N	2020-10-12 16:46:30.54179
83174	EDOW	2020-10-06	24.7207	24.775	24.26	24.3303	17394	{}	\N	2020-10-12 16:46:30.546805
83175	EDOW	2020-10-07	24.6367	24.8131	24.6029	24.8131	3960	{}	\N	2020-10-12 16:46:30.552305
83176	EDOW	2020-10-08	24.85	25	24.8301	24.9491	4951	{}	\N	2020-10-12 16:46:30.558086
83177	EDOW	2020-10-09	25.1062	25.1307	25.03	25.03	3222	{}	\N	2020-10-12 16:46:30.565248
83178	EDOW	2020-10-12	25.1498	25.2899	25.1498	25.2716	2534	{}	\N	2020-10-12 16:46:30.570848
83179	EET	2020-09-28	67.43	67.61	66.4494	66.94	22682	{}	\N	2020-10-12 16:46:32.752601
83180	EET	2020-09-29	66.7701	67.389	66.694	66.95	5395	{}	\N	2020-10-12 16:46:32.763257
83181	EET	2020-09-30	68.38	69.4508	68.26	69.01	19073	{}	\N	2020-10-12 16:46:32.770334
83182	EET	2020-10-01	69.99	70.41	69.5	70.31	9132	{}	\N	2020-10-12 16:46:32.776499
83183	EET	2020-10-02	69.4301	69.8099	68.6116	68.71	6963	{}	\N	2020-10-12 16:46:32.782816
83184	EET	2020-10-05	69.41	70.5997	69.41	70.5997	7072	{}	\N	2020-10-12 16:46:32.78809
83185	EET	2020-10-06	71.01	71.7099	70.63	70.89	10245	{}	\N	2020-10-12 16:46:32.793462
83186	EET	2020-10-07	72.135	72.635	72.105	72.635	4081	{}	\N	2020-10-12 16:46:32.799098
83187	EET	2020-10-08	73.11	73.65	73	73.65	3134	{}	\N	2020-10-12 16:46:32.804468
83188	EET	2020-10-09	73.83	75.01	73.83	74.5572	4837	{}	\N	2020-10-12 16:46:32.810234
83189	EET	2020-10-12	75.48	76.19	75.1951	76.111	22953	{}	\N	2020-10-12 16:46:32.815321
83190	EMLP	2020-09-28	19.15	19.3	19.03	19.15	311121	{}	\N	2020-10-12 16:46:34.302469
83191	EMLP	2020-09-29	19.18	19.24	19	19.16	650085	{}	\N	2020-10-12 16:46:34.3094
83192	EMLP	2020-09-30	19.18	19.285	18.95	19.03	575660	{}	\N	2020-10-12 16:46:34.315456
83193	EMLP	2020-10-01	19.06	19.1	18.8997	19.03	641205	{}	\N	2020-10-12 16:46:34.320795
83194	EMLP	2020-10-02	18.8	19.32	18.7809	19.24	439843	{}	\N	2020-10-12 16:46:34.325638
83195	EMLP	2020-10-05	19.42	19.61	19.28	19.61	661166	{}	\N	2020-10-12 16:46:34.330612
83196	EMLP	2020-10-06	19.68	19.85	19.53	19.63	691714	{}	\N	2020-10-12 16:46:34.335964
83197	EMLP	2020-10-07	19.77	19.9	19.7127	19.84	308441	{}	\N	2020-10-12 16:46:34.340752
83198	EMLP	2020-10-08	19.93	20.46	19.93	20.45	518681	{}	\N	2020-10-12 16:46:34.345592
83199	EMLP	2020-10-09	20.56	20.56	20.235	20.32	459558	{}	\N	2020-10-12 16:46:34.350413
83200	EMLP	2020-10-12	20.3596	20.41	20.295	20.37	112324	{}	\N	2020-10-12 16:46:34.35508
83201	EMTL	2020-09-28	49.77	49.81	49.7301	49.7301	4318	{}	\N	2020-10-12 16:46:37.503708
83202	EMTL	2020-09-29	49.725	49.8289	49.7	49.722	6518	{}	\N	2020-10-12 16:46:37.512445
83203	EMTL	2020-09-30	49.7877	50.26	49.7844	49.91	16134	{}	\N	2020-10-12 16:46:37.518696
83204	EMTL	2020-10-01	49.745	49.83	49.745	49.82	4196	{}	\N	2020-10-12 16:46:37.524682
83205	EMTL	2020-10-02	49.8	49.81	49.718	49.7311	5788	{}	\N	2020-10-12 16:46:37.529834
83206	EMTL	2020-10-05	49.84	49.84	49.61	49.61	5771	{}	\N	2020-10-12 16:46:37.53497
83207	EMTL	2020-10-06	49.87	49.87	49.7615	49.845	5812	{}	\N	2020-10-12 16:46:37.539787
83208	EMTL	2020-10-07	49.9799	49.9799	49.82	49.89	8278	{}	\N	2020-10-12 16:46:37.544323
83209	EMTL	2020-10-08	49.93	50.04	49.93	50.03	8488	{}	\N	2020-10-12 16:46:37.549215
83210	EMTL	2020-10-09	49.9	50.0637	49.9	50.0637	1294	{}	\N	2020-10-12 16:46:37.553907
83211	EMTL	2020-10-12	50.035	50.036	50.035	50.036	416	{}	\N	2020-10-12 16:46:37.558406
83212	EPZM	2020-09-28	12	12.22	11.75	12.17	562918	{}	\N	2020-10-12 16:46:39.717595
83213	EPZM	2020-09-29	12.19	12.36	11.9001	12.03	663407	{}	\N	2020-10-12 16:46:39.730705
83214	EPZM	2020-09-30	12.09	12.2	11.77	11.92	786880	{}	\N	2020-10-12 16:46:39.741279
83215	EPZM	2020-10-01	12	12.1	11.3	11.51	959960	{}	\N	2020-10-12 16:46:39.747538
83216	EPZM	2020-10-02	11.3	11.78	11.03	11.03	554596	{}	\N	2020-10-12 16:46:39.753546
83217	EPZM	2020-10-05	11.25	12.33	11.25	12.19	1514827	{}	\N	2020-10-12 16:46:39.758632
83218	EPZM	2020-10-06	12.57	12.61	11.94	12.06	960501	{}	\N	2020-10-12 16:46:39.763903
83219	EPZM	2020-10-07	12.4658	12.65	12.22	12.61	477849	{}	\N	2020-10-12 16:46:39.76938
83220	EPZM	2020-10-08	12.73	12.84	12.3	12.49	479141	{}	\N	2020-10-12 16:46:39.776086
83221	EPZM	2020-10-09	12.496	12.89	12.25	12.45	544415	{}	\N	2020-10-12 16:46:39.781805
83222	EPZM	2020-10-12	12.53	12.89	12.35	12.59	196388	{}	\N	2020-10-12 16:46:39.786966
83223	EQRR	2020-09-30	29.682	29.682	29.682	29.682	476	{}	\N	2020-10-12 16:46:41.090946
83224	EQRR	2020-10-06	30.7389	30.7389	29.9412	29.9412	228	{}	\N	2020-10-12 16:46:41.100625
83225	EQRR	2020-10-08	30.86	30.86	30.83	30.83	298	{}	\N	2020-10-12 16:46:41.107928
83226	EQRR	2020-10-09	31.42	31.42	31.41	31.41	206	{}	\N	2020-10-12 16:46:41.114014
83227	ESP	2020-09-28	19.41	19.41	18.63	18.63	6738	{}	\N	2020-10-12 16:46:42.249215
83228	ESP	2020-09-29	19.38	19.38	18.7	18.975	724	{}	\N	2020-10-12 16:46:42.260319
83229	ESP	2020-09-30	19.058	19.058	18.8	18.9973	729	{}	\N	2020-10-12 16:46:42.267716
83230	ESP	2020-10-01	18.99	19	18.53	18.755	2706	{}	\N	2020-10-12 16:46:42.273709
83231	ESP	2020-10-02	18.2	18.39	17.5	18.39	755	{}	\N	2020-10-12 16:46:42.279737
83232	ESP	2020-10-05	18.8	18.8	18.01	18.01	1560	{}	\N	2020-10-12 16:46:42.284913
83233	ESP	2020-10-06	17.9	18.1552	17.8	17.93	1184	{}	\N	2020-10-12 16:46:42.289873
83234	ESP	2020-10-07	18.4	18.4	18.4	18.4	283	{}	\N	2020-10-12 16:46:42.294859
83235	ESP	2020-10-08	18.23	18.4994	18.22	18.3	2954	{}	\N	2020-10-12 16:46:42.299811
83236	ESP	2020-10-09	18.3	18.3	18.3	18.3	770	{}	\N	2020-10-12 16:46:42.304437
83237	ESP	2020-10-12	18.02	18.16	18.02	18.16	455	{}	\N	2020-10-12 16:46:42.308895
83238	ETB	2020-09-28	13.54	13.65	13.4301	13.5	84397	{}	\N	2020-10-12 16:46:43.297672
83239	ETB	2020-09-29	13.43	13.55	13.41	13.53	166676	{}	\N	2020-10-12 16:46:43.308224
83240	ETB	2020-09-30	13.67	13.69	13.55	13.64	185184	{}	\N	2020-10-12 16:46:43.315229
83241	ETB	2020-10-01	13.66	13.76	13.66	13.715	103096	{}	\N	2020-10-12 16:46:43.321404
83242	ETB	2020-10-02	13.43	13.6832	13.4	13.64	191022	{}	\N	2020-10-12 16:46:43.32726
83243	ETB	2020-10-05	13.7	13.7445	13.63	13.74	91083	{}	\N	2020-10-12 16:46:43.33214
83244	ETB	2020-10-06	13.81	13.86	13.65	13.67	128960	{}	\N	2020-10-12 16:46:43.336907
83245	ETB	2020-10-07	13.71	13.91	13.71	13.89	145840	{}	\N	2020-10-12 16:46:43.341785
83246	ETB	2020-10-08	13.87	13.9421	13.8	13.81	154464	{}	\N	2020-10-12 16:46:43.346457
83247	ETB	2020-10-09	13.9	13.95	13.77	13.9	127024	{}	\N	2020-10-12 16:46:43.35032
83248	ETB	2020-10-12	13.98	14.11	13.91	14.091	47726	{}	\N	2020-10-12 16:46:43.354987
83249	DWCR	2020-09-29	27.34	27.34	27.3042	27.3042	5100	{}	\N	2020-10-12 16:46:44.47441
83250	EFG	2020-09-28	90.17	90.248	89.8852	90.14	276373	{}	\N	2020-10-12 16:46:47.453247
83251	EFG	2020-09-29	90.4	90.7386	90.14	90.46	331921	{}	\N	2020-10-12 16:46:47.463501
83252	EFG	2020-09-30	90.06	90.51	89.6503	89.88	251942	{}	\N	2020-10-12 16:46:47.471234
83253	EFG	2020-10-01	90.67	90.8	90.22	90.75	380843	{}	\N	2020-10-12 16:46:47.477536
83254	EFG	2020-10-02	89.86	90.4699	89.41	90.3	392262	{}	\N	2020-10-12 16:46:47.483701
83255	EFG	2020-10-05	90.84	91.4	90.7247	91.35	715827	{}	\N	2020-10-12 16:46:47.488878
83256	EFG	2020-10-06	90.93	91.13	89.93	90.07	290928	{}	\N	2020-10-12 16:46:47.493581
83257	EFG	2020-10-07	90.55	90.784	90.26	90.58	3971139	{}	\N	2020-10-12 16:46:47.498224
83258	EFG	2020-10-08	90.96	91.3	90.88	91.19	890426	{}	\N	2020-10-12 16:46:47.503613
83259	EFG	2020-10-09	91.72	92.224	91.62	92.19	550382	{}	\N	2020-10-12 16:46:47.50959
83260	EFG	2020-10-12	92.46	92.97	92.39	92.945	143884	{}	\N	2020-10-12 16:46:47.514889
83261	EGF	2020-09-28	13.4	13.4	13.1655	13.36	3722	{}	\N	2020-10-12 16:46:48.569162
83262	EGF	2020-09-29	13.4	13.4	13.36	13.38	4182	{}	\N	2020-10-12 16:46:48.579739
83263	EGF	2020-09-30	13.37	13.4799	13.3	13.38	38612	{}	\N	2020-10-12 16:46:48.586646
83264	EGF	2020-10-01	13.37	13.37	13.25	13.3	4282	{}	\N	2020-10-12 16:46:48.593243
83265	EGF	2020-10-02	13.26	13.39	13.2599	13.39	4814	{}	\N	2020-10-12 16:46:48.599541
83266	EGF	2020-10-05	13.44	13.44	13.337	13.37	18868	{}	\N	2020-10-12 16:46:48.604537
83267	EGF	2020-10-06	13.39	13.39	13.27	13.37	13494	{}	\N	2020-10-12 16:46:48.609533
83268	EGF	2020-10-07	13.43	13.43	13.3252	13.3252	14594	{}	\N	2020-10-12 16:46:48.614403
83269	EGF	2020-10-08	13.42	13.42	13.19	13.192	30258	{}	\N	2020-10-12 16:46:48.619385
83270	EGF	2020-10-09	13.26	13.26	12.9	13.12	123120	{}	\N	2020-10-12 16:46:48.624491
83271	EGF	2020-10-12	13.11	13.13	13.03	13.03	34584	{}	\N	2020-10-12 16:46:48.629413
83272	EWK	2020-09-28	17.05	17.05	16.99	16.99	1744	{}	\N	2020-10-12 16:46:51.35019
83273	EWK	2020-09-29	17.06	17.07	17.01	17.02	1076	{}	\N	2020-10-12 16:46:51.360457
83274	EWK	2020-09-30	17.055	17.09	16.9316	17.02	8328	{}	\N	2020-10-12 16:46:51.368405
83275	EWK	2020-10-01	17.14	17.14	17.01	17.09	191617	{}	\N	2020-10-12 16:46:51.374591
83276	EWK	2020-10-02	16.86	17.13	16.86	17.1	9716	{}	\N	2020-10-12 16:46:51.380524
83277	EWK	2020-10-05	17.38	17.41	17.34	17.41	7158	{}	\N	2020-10-12 16:46:51.385596
83278	EWK	2020-10-06	17.62	17.65	17.3912	17.3912	6631	{}	\N	2020-10-12 16:46:51.39043
83279	EWK	2020-10-07	17.52	17.57	17.47	17.57	12203	{}	\N	2020-10-12 16:46:51.395063
83280	EWK	2020-10-08	17.69	17.77	17.69	17.77	1888	{}	\N	2020-10-12 16:46:51.399995
83281	EWK	2020-10-09	17.9	17.93	17.9	17.9	5240	{}	\N	2020-10-12 16:46:51.404702
83282	EWK	2020-10-12	17.86	17.89	17.86	17.89	915	{}	\N	2020-10-12 16:46:51.409303
83283	FRAK	2020-09-28	57.2683	57.2683	57.2683	57.2683	69	{}	\N	2020-10-12 16:46:52.916326
83284	FRAK	2020-09-29	56.3	56.3	54.63	54.63	3697	{}	\N	2020-10-12 16:46:52.928159
83285	FRAK	2020-09-30	56.35	56.35	55.3066	55.3066	458	{}	\N	2020-10-12 16:46:52.936596
83286	FRAK	2020-10-01	53.82	53.82	53.5032	53.5032	442	{}	\N	2020-10-12 16:46:52.942675
83287	FRAK	2020-10-02	52.49	54.4344	52.49	54.4344	1057	{}	\N	2020-10-12 16:46:52.948621
83288	FRAK	2020-10-05	56.8227	56.8227	56.8227	56.8227	91	{}	\N	2020-10-12 16:46:52.953937
83289	FRAK	2020-10-06	57.42	57.42	55.4894	55.4894	683	{}	\N	2020-10-12 16:46:52.958775
83290	FRAK	2020-10-07	55.98	56.7476	55.69	56.7476	725	{}	\N	2020-10-12 16:46:52.963611
83291	FRAK	2020-10-08	57.46	59.0043	57.46	59.0043	1009	{}	\N	2020-10-12 16:46:52.968517
83292	FRAK	2020-10-09	58.43	58.43	58.43	58.43	361	{}	\N	2020-10-12 16:46:52.973557
83293	FRAK	2020-10-12	58.46	58.46	58.46	58.46	299	{}	\N	2020-10-12 16:46:52.978397
83294	FTXH	2020-09-28	23.86	23.86	23.7669	23.7669	533	{}	\N	2020-10-12 16:46:54.007025
83295	FTXH	2020-09-29	23.73	23.8277	23.615	23.6506	32250	{}	\N	2020-10-12 16:46:54.017935
83296	FTXH	2020-09-30	23.45	24.08	23.45	24.08	6763	{}	\N	2020-10-12 16:46:54.025307
83297	FTXH	2020-10-02	23.8225	24.76	23.805	24.04	8990	{}	\N	2020-10-12 16:46:54.031524
83298	FTXH	2020-10-05	24.04	24.188	23.99	24.04	2826	{}	\N	2020-10-12 16:46:54.038209
83299	FTXH	2020-10-06	24.0348	24.0721	24.0061	24.0061	1137	{}	\N	2020-10-12 16:46:54.043499
83300	FTXH	2020-10-07	23.9981	24.23	23.9981	24.23	1391	{}	\N	2020-10-12 16:46:54.048475
83301	FTXH	2020-10-08	24.58	24.68	24.5	24.65	12717	{}	\N	2020-10-12 16:46:54.054167
83302	FTXH	2020-10-09	24.7	24.875	24.622	24.825	1073	{}	\N	2020-10-12 16:46:54.059267
83303	FTXH	2020-10-12	25.36	25.36	24.885	24.885	2157	{}	\N	2020-10-12 16:46:54.064469
83304	FTXR	2020-09-28	23.71	23.8	23.5577	23.73	57235	{}	\N	2020-10-12 16:46:56.138074
83305	FTXR	2020-09-29	23.56	23.6997	23.34	23.34	132980	{}	\N	2020-10-12 16:46:56.149663
83306	FTXR	2020-09-30	23.6001	23.82	23.3	23.34	81309	{}	\N	2020-10-12 16:46:56.156677
83307	FTXR	2020-10-01	23.4999	23.7599	23.45	23.55	141667	{}	\N	2020-10-12 16:46:56.162431
83308	FTXR	2020-10-02	23.01	23.8599	23.01	23.74	63084	{}	\N	2020-10-12 16:46:56.167426
83309	FTXR	2020-10-05	24.01	24.24	23.99	24.12	130381	{}	\N	2020-10-12 16:46:56.173088
83310	FTXR	2020-10-06	24.25	24.4501	23.8	23.8	48511	{}	\N	2020-10-12 16:46:56.177727
83311	FTXR	2020-10-07	24.15	24.599	24.15	24.58	164128	{}	\N	2020-10-12 16:46:56.182187
83312	FTXR	2020-10-08	24.8	24.8099	24.51	24.76	116478	{}	\N	2020-10-12 16:46:56.186954
83313	FTXR	2020-10-09	24.78	24.85	24.62	24.77	28012	{}	\N	2020-10-12 16:46:56.191314
83314	FTXR	2020-10-12	24.77	24.91	24.77	24.875	33917	{}	\N	2020-10-12 16:46:56.195877
83315	GDL	2020-09-28	8.49	8.58	8.48	8.49	34260	{}	\N	2020-10-12 16:46:57.75441
83316	GDL	2020-09-29	8.54	8.56	8.4	8.4	86824	{}	\N	2020-10-12 16:46:57.76551
83317	GDL	2020-09-30	8.4	8.44	8.31	8.38	32742	{}	\N	2020-10-12 16:46:57.773012
83318	GDL	2020-10-01	8.475	8.475	8.39	8.4163	31594	{}	\N	2020-10-12 16:46:57.779141
83319	GDL	2020-10-02	8.4483	8.4483	8.4	8.41	11008	{}	\N	2020-10-12 16:46:57.784665
83320	GDL	2020-10-05	8.39	8.465	8.34	8.38	50128	{}	\N	2020-10-12 16:46:57.789627
83321	GDL	2020-10-06	8.37	8.44	8.37	8.384	130454	{}	\N	2020-10-12 16:46:57.794593
83322	GDL	2020-10-07	8.4716	8.505	8.4027	8.4027	24908	{}	\N	2020-10-12 16:46:57.799322
83323	GDL	2020-10-08	8.48	8.48	8.4208	8.4208	17824	{}	\N	2020-10-12 16:46:57.804896
83324	GDL	2020-10-09	8.41	8.46	8.4	8.4297	79896	{}	\N	2020-10-12 16:46:57.809795
83325	GDL	2020-10-12	8.46	8.52	8.46	8.51	8630	{}	\N	2020-10-12 16:46:57.814672
83326	DWLD	2020-09-28	25.47	25.63	25.43	25.56	26807	{}	\N	2020-10-12 16:46:59.956624
83327	DWLD	2020-09-29	25.54	25.58	25.4183	25.4684	8866	{}	\N	2020-10-12 16:46:59.96922
83328	DWLD	2020-09-30	25.54	25.93	25.54	25.795	7775	{}	\N	2020-10-12 16:46:59.9777
83329	DWLD	2020-10-01	25.955	26.13	25.95	26.07	14921	{}	\N	2020-10-12 16:46:59.984411
83330	DWLD	2020-10-02	25.64	26.1074	25.64	26.04	15139	{}	\N	2020-10-12 16:46:59.990449
83331	DWLD	2020-10-05	26.19	26.3791	26.18	26.3791	19482	{}	\N	2020-10-12 16:46:59.995745
83332	DWLD	2020-10-06	26.44	26.81	26.321	26.35	26404	{}	\N	2020-10-12 16:47:00.000888
83333	DWLD	2020-10-07	26.73	26.84	26.64	26.81	19161	{}	\N	2020-10-12 16:47:00.006034
83334	DWLD	2020-10-08	26.8135	27.08	26.8135	27.08	14468	{}	\N	2020-10-12 16:47:00.011151
83335	DWLD	2020-10-09	27.2	27.29	27.1085	27.21	9129	{}	\N	2020-10-12 16:47:00.016919
83336	DWLD	2020-10-12	27.48	27.74	27.48	27.74	11572	{}	\N	2020-10-12 16:47:00.022105
83337	DXC	2020-09-28	16.82	17.34	16.6	17.09	2956436	{}	\N	2020-10-12 16:47:01.155896
83338	DXC	2020-09-29	16.98	17.865	16.91	17.7	7304375	{}	\N	2020-10-12 16:47:01.166844
83339	DXC	2020-09-30	17.79	18.37	17.74	17.85	6593245	{}	\N	2020-10-12 16:47:01.17485
83340	DXC	2020-10-01	18.08	18.82	18.04	18.81	5542978	{}	\N	2020-10-12 16:47:01.18171
83341	DXC	2020-10-02	18.04	18.595	17.85	18.36	5197304	{}	\N	2020-10-12 16:47:01.187969
83342	DXC	2020-10-05	18.75	18.88	18.41	18.71	2687982	{}	\N	2020-10-12 16:47:01.193503
83343	DXC	2020-10-06	18.92	19.2	17.81	17.89	3454926	{}	\N	2020-10-12 16:47:01.198749
83344	DXC	2020-10-07	18.01	18.45	17.99	18.44	3949391	{}	\N	2020-10-12 16:47:01.204372
83345	DXC	2020-10-08	18.77	20.175	18.61	19.99	7574257	{}	\N	2020-10-12 16:47:01.209607
83346	DXC	2020-10-09	20.3	20.37	19.95	20.05	4416296	{}	\N	2020-10-12 16:47:01.214906
83347	DXC	2020-10-12	20.07	20.085	19.81	20.03	1185530	{}	\N	2020-10-12 16:47:01.220392
83348	EMD	2020-09-28	12.4	12.41	12.3101	12.41	219330	{}	\N	2020-10-12 16:47:03.058599
83349	EMD	2020-09-29	12.45	12.45	12.3	12.31	637846	{}	\N	2020-10-12 16:47:03.069848
83350	EMD	2020-09-30	12.31	12.3572	12.29	12.32	485078	{}	\N	2020-10-12 16:47:03.077842
83351	EMD	2020-10-01	12.37	12.51	12.3	12.47	386445	{}	\N	2020-10-12 16:47:03.082686
83352	EMD	2020-10-02	12.43	12.53	12.3	12.53	322570	{}	\N	2020-10-12 16:47:03.087359
83353	EMD	2020-10-05	12.55	12.71	12.55	12.67	350426	{}	\N	2020-10-12 16:47:03.091796
83354	EMD	2020-10-06	12.67	12.75	12.55	12.56	390487	{}	\N	2020-10-12 16:47:03.095665
83355	EMD	2020-10-07	12.7	12.7	12.65	12.66	239382	{}	\N	2020-10-12 16:47:03.099218
83356	EMD	2020-10-08	12.7	12.8323	12.7	12.75	284787	{}	\N	2020-10-12 16:47:03.102725
83357	EMD	2020-10-09	12.83	12.89	12.76	12.77	200088	{}	\N	2020-10-12 16:47:03.106616
83358	EMD	2020-10-12	12.82	12.8899	12.82	12.8624	112056	{}	\N	2020-10-12 16:47:03.110362
83359	EWM	2020-09-28	26.22	26.32	26	26.11	314358	{}	\N	2020-10-12 16:47:05.904975
83360	EWM	2020-09-29	26.09	26.22	26.045	26.17	175468	{}	\N	2020-10-12 16:47:05.917111
83361	EWM	2020-09-30	26.18	26.32	26.15	26.19	183384	{}	\N	2020-10-12 16:47:05.924777
83362	EWM	2020-10-01	26.03	26.1899	25.935	26.17	413301	{}	\N	2020-10-12 16:47:05.930622
83363	EWM	2020-10-02	26.09	26.275	26.09	26.21	450951	{}	\N	2020-10-12 16:47:05.936362
83364	EWM	2020-10-05	26.49	26.67	26.47	26.65	780415	{}	\N	2020-10-12 16:47:05.940653
83365	EWM	2020-10-06	26.36	26.52	26.32	26.38	985273	{}	\N	2020-10-12 16:47:05.945816
83366	EWM	2020-10-07	26.1	26.29	26.06	26.27	850884	{}	\N	2020-10-12 16:47:05.950757
83367	EWM	2020-10-08	26.48	26.56	26.405	26.55	453153	{}	\N	2020-10-12 16:47:05.955937
83368	EWM	2020-10-09	26.85	27.06	26.78	26.98	417988	{}	\N	2020-10-12 16:47:05.961394
83369	EWM	2020-10-12	26.51	26.75	26.51	26.75	158508	{}	\N	2020-10-12 16:47:05.96684
83370	EWSC	2020-09-28	47.5679	47.7427	47.5679	47.7427	516	{}	\N	2020-10-12 16:47:07.002584
83371	EWSC	2020-09-29	47.81	47.9	47.2	47.68	7724	{}	\N	2020-10-12 16:47:07.013442
83372	EWSC	2020-09-30	48.13	48.1798	47.4949	47.4949	1171	{}	\N	2020-10-12 16:47:07.020825
83373	EWSC	2020-10-01	47.33	48.0525	47.33	48.0525	722	{}	\N	2020-10-12 16:47:07.026365
83374	EWSC	2020-10-02	47.53	48.9926	47.53	48.9926	366	{}	\N	2020-10-12 16:47:07.031709
83375	EWSC	2020-10-05	48.85	50.0846	48.85	50.0846	1576	{}	\N	2020-10-12 16:47:07.036437
83376	EWSC	2020-10-06	50.34	51.222	49.95	49.95	5275	{}	\N	2020-10-12 16:47:07.041032
83377	EWSC	2020-10-07	50.498	50.79	50.498	50.79	397	{}	\N	2020-10-12 16:47:07.045895
83378	EWSC	2020-10-08	51.326	51.594	51.326	51.594	671	{}	\N	2020-10-12 16:47:07.050948
83379	EWSC	2020-10-09	52	52.1844	51.73	51.73	622	{}	\N	2020-10-12 16:47:07.056087
83380	EWSC	2020-10-12	51.81	51.81	51.81	51.81	162	{}	\N	2020-10-12 16:47:07.061219
83381	EYLD	2020-09-28	27.56	27.7	27.56	27.7	3257	{}	\N	2020-10-12 16:47:08.000828
83382	EYLD	2020-09-29	27.68	27.81	27.59	27.81	2358	{}	\N	2020-10-12 16:47:08.012506
83383	EYLD	2020-09-30	27.95	28.1196	27.95	28.11	4786	{}	\N	2020-10-12 16:47:08.021088
83384	EYLD	2020-10-01	28.15	28.15	27.95	27.99	3138	{}	\N	2020-10-12 16:47:08.027849
83385	EYLD	2020-10-02	27.9	28.075	27.83	28.075	1813	{}	\N	2020-10-12 16:47:08.034716
83386	EYLD	2020-10-05	28.2225	28.34	28.19	28.34	3181	{}	\N	2020-10-12 16:47:08.040461
83387	EYLD	2020-10-06	28.2788	28.42	28.14	28.2959	2972	{}	\N	2020-10-12 16:47:08.046491
83388	EYLD	2020-10-07	28.13	28.45	28.13	28.425	5011	{}	\N	2020-10-12 16:47:08.052723
83389	EYLD	2020-10-08	28.4	28.545	28.3801	28.5079	4593	{}	\N	2020-10-12 16:47:08.058819
83390	EYLD	2020-10-09	28.5889	28.83	28.5889	28.83	3441	{}	\N	2020-10-12 16:47:08.064967
83391	EYLD	2020-10-12	28.73	28.85	28.73	28.84	1857	{}	\N	2020-10-12 16:47:08.070915
83392	FIW	2020-09-28	59.45	60.2038	59.2551	59.94	14528	{}	\N	2020-10-12 16:47:09.155193
83393	FIW	2020-09-29	59.92	60.44	59.745	60.2223	13238	{}	\N	2020-10-12 16:47:09.171416
83394	FIW	2020-09-30	60.28	60.9	59.99	60.44	19639	{}	\N	2020-10-12 16:47:09.179225
83395	FIW	2020-10-01	60.8806	60.8806	60.275	60.65	16548	{}	\N	2020-10-12 16:47:09.186014
83396	FIW	2020-10-02	59.81	61.4499	59.81	61.1597	14372	{}	\N	2020-10-12 16:47:09.192008
83397	FIW	2020-10-05	61.59	62.24	61.59	62.24	47284	{}	\N	2020-10-12 16:47:09.196989
83398	FIW	2020-10-06	62.4557	63.19	62.16	62.16	9051	{}	\N	2020-10-12 16:47:09.202773
83399	FIW	2020-10-07	62.7	63.1628	62.7	63.11	18923	{}	\N	2020-10-12 16:47:09.207573
83400	FIW	2020-10-08	63.23	63.85	63.23	63.85	47422	{}	\N	2020-10-12 16:47:09.212124
83401	FIW	2020-10-09	64.18	64.64	64.05	64.33	25539	{}	\N	2020-10-12 16:47:09.216874
83402	FIW	2020-10-12	64.79	65.46	64.72	65.42	13783	{}	\N	2020-10-12 16:47:09.221802
83403	FNY	2020-09-28	53.05	53.3289	52.8602	53.25	129336	{}	\N	2020-10-12 16:47:11.446879
83404	FNY	2020-09-29	53.2	53.69	53.2	53.24	46536	{}	\N	2020-10-12 16:47:11.456843
83405	FNY	2020-09-30	53.4447	53.99	53.05	53.5	194096	{}	\N	2020-10-12 16:47:11.463602
83406	FNY	2020-10-01	53.96	54.4397	53.76	54.36	198411	{}	\N	2020-10-12 16:47:11.469794
83407	FNY	2020-10-02	53.96	54.32	53.85	54.1136	37660	{}	\N	2020-10-12 16:47:11.47634
83408	FNY	2020-10-05	54.81	55.4	54.79	55.4	28684	{}	\N	2020-10-12 16:47:11.481564
83409	FNY	2020-10-06	55.58	56.2963	55.1772	55.28	54829	{}	\N	2020-10-12 16:47:11.486448
83410	FNY	2020-10-07	55.75	56.52	55.75	56.43	457870	{}	\N	2020-10-12 16:47:11.491705
83411	FNY	2020-10-08	57.18	57.21	56.555	56.76	93371	{}	\N	2020-10-12 16:47:11.496678
83412	FNY	2020-10-09	57.34	57.65	57.23	57.6	31197	{}	\N	2020-10-12 16:47:11.501655
83413	FNY	2020-10-12	57.76	58.01	57.6742	57.8966	9339	{}	\N	2020-10-12 16:47:11.506768
83414	EBND	2020-09-28	26.14	26.16	25.9864	26.04	270526	{}	\N	2020-10-12 16:47:13.02194
83415	EBND	2020-09-29	26.13	26.19	26.0697	26.15	147267	{}	\N	2020-10-12 16:47:13.033538
83416	EBND	2020-09-30	26.19	26.309	26.1535	26.28	70208	{}	\N	2020-10-12 16:47:13.041195
83417	EBND	2020-10-01	26.28	26.35	26.28	26.32	94891	{}	\N	2020-10-12 16:47:13.047633
83418	EBND	2020-10-02	26.25	26.31	26.22	26.22	114340	{}	\N	2020-10-12 16:47:13.053105
83419	EBND	2020-10-05	26.29	26.41	26.29	26.35	151882	{}	\N	2020-10-12 16:47:13.057849
83420	EBND	2020-10-06	26.42	26.42	26.24	26.26	92997	{}	\N	2020-10-12 16:47:13.062561
83421	EBND	2020-10-07	26.32	26.3646	26.28	26.31	79611	{}	\N	2020-10-12 16:47:13.066886
83422	EBND	2020-10-08	26.35	26.4353	26.35	26.41	135455	{}	\N	2020-10-12 16:47:13.071092
83423	EBND	2020-10-09	26.57	26.64	26.52	26.58	153762	{}	\N	2020-10-12 16:47:13.075672
83424	EBND	2020-10-12	26.4935	26.57	26.49	26.53	123973	{}	\N	2020-10-12 16:47:13.079812
83425	EMDV	2020-09-28	48.69	48.83	48.5209	48.5209	6198	{}	\N	2020-10-12 16:47:15.838401
83426	EMDV	2020-09-29	48.48	48.66	48.45	48.64	7531	{}	\N	2020-10-12 16:47:15.845916
83427	EMDV	2020-09-30	49.07	49.2662	49.07	49.2662	892	{}	\N	2020-10-12 16:47:15.85186
83428	EMDV	2020-10-01	49.5	49.6205	49.4236	49.6205	2096	{}	\N	2020-10-12 16:47:15.857526
83429	EMDV	2020-10-02	49.09	49.51	49.09	49.3	9533	{}	\N	2020-10-12 16:47:15.862623
83430	EMDV	2020-10-05	49.8099	49.9	49.7549	49.7549	363	{}	\N	2020-10-12 16:47:15.867436
83431	EMDV	2020-10-06	49.9875	49.9875	49.7748	49.7748	550	{}	\N	2020-10-12 16:47:15.872593
83432	EMDV	2020-10-07	50.11	50.21	50.02	50.21	5855	{}	\N	2020-10-12 16:47:15.877016
83433	EMDV	2020-10-08	50.29	50.5	50.25	50.475	11719	{}	\N	2020-10-12 16:47:15.881126
83434	EMDV	2020-10-09	50.43	50.696	50.3	50.696	1828	{}	\N	2020-10-12 16:47:15.885535
83435	EMDV	2020-10-12	50.58	50.6699	50.58	50.6699	1532	{}	\N	2020-10-12 16:47:15.889934
83436	EUFN	2020-09-28	13.71	13.87	13.71	13.87	5698554	{}	\N	2020-10-12 16:47:17.515024
83437	EUFN	2020-09-29	13.74	13.775	13.62	13.68	1093936	{}	\N	2020-10-12 16:47:17.528778
83438	EUFN	2020-09-30	13.82	13.87	13.67	13.74	423959	{}	\N	2020-10-12 16:47:17.537035
83439	EUFN	2020-10-01	13.8	13.81	13.681	13.75	303720	{}	\N	2020-10-12 16:47:17.543278
83440	EUFN	2020-10-02	13.64	13.9	13.59	13.86	494240	{}	\N	2020-10-12 16:47:17.549877
83441	EUFN	2020-10-05	14.05	14.16	14.01	14.16	445560	{}	\N	2020-10-12 16:47:17.555069
83442	EUFN	2020-10-06	14.36	14.409	14.14	14.17	687926	{}	\N	2020-10-12 16:47:17.560023
83443	EUFN	2020-10-07	14.24	14.29	14.19	14.24	645985	{}	\N	2020-10-12 16:47:17.565023
83444	EUFN	2020-10-08	14.39	14.41	14.33	14.4	543750	{}	\N	2020-10-12 16:47:17.570065
83445	EUFN	2020-10-09	14.48	14.48	14.33	14.395	570417	{}	\N	2020-10-12 16:47:17.575446
83446	EUFN	2020-10-12	14.42	14.46	14.39	14.455	216594	{}	\N	2020-10-12 16:47:17.580294
83447	FENY	2020-09-28	8.11	8.25	8.1093	8.17	626275	{}	\N	2020-10-12 16:47:19.627417
83448	FENY	2020-09-29	8.14	8.15	7.85	7.95	1554889	{}	\N	2020-10-12 16:47:19.639603
83449	FENY	2020-09-30	7.97	8.07	7.87	7.9	316517	{}	\N	2020-10-12 16:47:19.649035
83450	FENY	2020-10-01	7.82	7.86	7.6317	7.68	810923	{}	\N	2020-10-12 16:47:19.655833
83451	FENY	2020-10-02	7.47	7.825	7.45	7.77	728197	{}	\N	2020-10-12 16:47:19.662698
83452	FENY	2020-10-05	7.86	8.01	7.82	8.01	737375	{}	\N	2020-10-12 16:47:19.667993
83453	FENY	2020-10-06	8.11	8.1664	7.86	7.88	724422	{}	\N	2020-10-12 16:47:19.673556
83454	FENY	2020-10-07	7.91	8.02	7.84	7.99	283813	{}	\N	2020-10-12 16:47:19.67889
83455	FENY	2020-10-08	8.07	8.3087	8.0337	8.3	619837	{}	\N	2020-10-12 16:47:19.683944
83456	FENY	2020-10-09	8.36	8.39	8.16	8.18	432609	{}	\N	2020-10-12 16:47:19.689686
83457	FENY	2020-10-12	8.17	8.18	8.0755	8.1484	798758	{}	\N	2020-10-12 16:47:19.695104
83458	FFTI	2020-09-28	23.86	23.93	23.84	23.86	48091	{}	\N	2020-10-12 16:47:21.254025
83459	FFTI	2020-09-29	23.85	23.96	23.85	23.91	46882	{}	\N	2020-10-12 16:47:21.263565
83460	FFTI	2020-09-30	23.84	23.95	23.84	23.88	47899	{}	\N	2020-10-12 16:47:21.269289
83461	FFTI	2020-10-01	23.84	23.96	23.84	23.93	43588	{}	\N	2020-10-12 16:47:21.275543
83462	FFTI	2020-10-02	23.93	23.9599	23.9001	23.91	20567	{}	\N	2020-10-12 16:47:21.280746
83463	FFTI	2020-10-05	23.955	23.99	23.93	23.93	38299	{}	\N	2020-10-12 16:47:21.285422
83464	FFTI	2020-10-06	23.94	23.9998	23.92	23.92	53100	{}	\N	2020-10-12 16:47:21.289749
83465	FFTI	2020-10-07	23.95	24	23.94	23.94	34499	{}	\N	2020-10-12 16:47:21.294038
83466	FFTI	2020-10-08	23.98	24.01	23.97	23.97	63386	{}	\N	2020-10-12 16:47:21.298125
83467	FFTI	2020-10-09	24.0213	24.0499	23.99	23.99	17759	{}	\N	2020-10-12 16:47:21.302142
83468	FFTI	2020-10-12	23.99	24.0883	23.99	24.0883	1852	{}	\N	2020-10-12 16:47:21.306154
83469	FJP	2020-09-28	49.28	49.3266	49.27	49.3266	566	{}	\N	2020-10-12 16:47:24.743581
83470	FJP	2020-09-29	49.48	49.48	49.25	49.25	2952	{}	\N	2020-10-12 16:47:24.756184
83471	FJP	2020-09-30	48.51	48.86	48.31	48.79	6644	{}	\N	2020-10-12 16:47:24.76385
83472	FJP	2020-10-02	48.29	48.3	48.29	48.3	630	{}	\N	2020-10-12 16:47:24.770654
83473	FJP	2020-10-05	48.81	48.9576	48.765	48.765	1483	{}	\N	2020-10-12 16:47:24.777396
83474	FJP	2020-10-06	48.6521	48.75	48.549	48.67	3148	{}	\N	2020-10-12 16:47:24.781912
83475	FJP	2020-10-07	48.95	49.005	48.9161	48.9286	3750	{}	\N	2020-10-12 16:47:24.786505
83476	FJP	2020-10-08	49.075	49.075	49.0609	49.0609	842	{}	\N	2020-10-12 16:47:24.79118
83477	FJP	2020-10-09	49.05	49.1	49.05	49.1	669	{}	\N	2020-10-12 16:47:24.795794
83478	FJP	2020-10-12	49.17	49.18	49.17	49.18	431	{}	\N	2020-10-12 16:47:24.800791
83479	FOX	2020-09-28	27.35	27.9	27.14	27.9	975152	{}	\N	2020-10-12 16:47:25.689385
83480	FOX	2020-09-29	27.94	28	27.26	27.82	853848	{}	\N	2020-10-12 16:47:25.700266
83481	FOX	2020-09-30	27.95	28.61	27.65	27.97	1305582	{}	\N	2020-10-12 16:47:25.708585
83482	FOX	2020-10-01	28.21	28.35	27.68	28.2	742420	{}	\N	2020-10-12 16:47:25.714486
83483	FOX	2020-10-02	27.91	28.415	27.64	28.17	1042309	{}	\N	2020-10-12 16:47:25.720311
83484	FOX	2020-10-05	28.42	28.95	28.19	28.89	573883	{}	\N	2020-10-12 16:47:25.726989
83485	FOX	2020-10-06	28.97	29.18	28.01	28.12	1298442	{}	\N	2020-10-12 16:47:25.732099
83486	FOX	2020-10-07	28.68	29.28	28.34	29.11	1007813	{}	\N	2020-10-12 16:47:25.737144
83487	FOX	2020-10-08	29.35	29.65	29.06	29.63	695328	{}	\N	2020-10-12 16:47:25.742024
83488	FOX	2020-10-09	29.91	30.04	29.16	29.2	818167	{}	\N	2020-10-12 16:47:25.746633
83489	FOX	2020-10-12	29.43	29.49	28.96	29.48	406590	{}	\N	2020-10-12 16:47:25.751196
83490	FTAI	2020-09-28	16.35	16.735	16.35	16.53	169946	{}	\N	2020-10-12 16:47:27.184331
83491	FTAI	2020-09-29	16.57	17.14	16.31	17.06	1159773	{}	\N	2020-10-12 16:47:27.195725
83492	FTAI	2020-09-30	17.2	17.3	16.8	17.13	1340787	{}	\N	2020-10-12 16:47:27.202681
83493	FTAI	2020-10-01	17.16	17.93	17.09	17.9	1722874	{}	\N	2020-10-12 16:47:27.209099
83494	FTAI	2020-10-02	17.35	17.93	17.1602	17.41	730812	{}	\N	2020-10-12 16:47:27.214572
83495	FTAI	2020-10-05	17.57	17.93	17.34	17.43	333113	{}	\N	2020-10-12 16:47:27.219176
83496	FTAI	2020-10-06	17.65	17.7306	16.99	17.16	1399016	{}	\N	2020-10-12 16:47:27.223966
83497	FTAI	2020-10-07	17.49	17.73	17.35	17.44	956555	{}	\N	2020-10-12 16:47:27.228352
83498	FTAI	2020-10-08	17.6	17.87	17.38	17.77	975758	{}	\N	2020-10-12 16:47:27.232785
83499	FTAI	2020-10-09	17.75	17.87	17.41	17.46	598352	{}	\N	2020-10-12 16:47:27.237181
83500	FTAI	2020-10-12	17.51	17.63	17.31	17.37	156074	{}	\N	2020-10-12 16:47:27.242006
83501	FUT	2020-09-28	39.18	39.19	39.1493	39.1493	614	{}	\N	2020-10-12 16:47:28.616264
83502	FUT	2020-09-29	39.19	39.1999	39.175	39.175	383	{}	\N	2020-10-12 16:47:28.626217
83503	FUT	2020-09-30	39.38	39.38	39.38	39.38	225	{}	\N	2020-10-12 16:47:28.63319
83504	FUT	2020-10-01	39.34	39.34	39.2	39.2	803	{}	\N	2020-10-12 16:47:28.639973
83505	FUT	2020-10-02	39.28	39.38	39.2443	39.2725	1138	{}	\N	2020-10-12 16:47:28.646092
83506	FUT	2020-10-05	39.48	39.48	39.39	39.39	2565	{}	\N	2020-10-12 16:47:28.650822
83507	FUT	2020-10-06	39.485	39.485	39.4	39.4	359	{}	\N	2020-10-12 16:47:28.655735
83508	FUT	2020-10-07	39.491	39.585	39.491	39.57	6198	{}	\N	2020-10-12 16:47:28.66093
83509	FUT	2020-10-08	39.5072	39.525	39.5072	39.525	639	{}	\N	2020-10-12 16:47:28.665736
83510	FUT	2020-10-09	39.7599	39.7599	39.66	39.66	401	{}	\N	2020-10-12 16:47:28.670842
83511	FUT	2020-10-12	39.69	39.75	39.69	39.75	1347	{}	\N	2020-10-12 16:47:28.675502
83512	FXG	2020-09-28	47.73	48.18	47.73	48.06	10270	{}	\N	2020-10-12 16:47:29.857442
83513	FXG	2020-09-29	48.0806	48.15	47.5504	47.64	48078	{}	\N	2020-10-12 16:47:29.868345
83514	FXG	2020-09-30	47.81	48.39	47.81	48.08	14431	{}	\N	2020-10-12 16:47:29.875354
83515	FXG	2020-10-01	48.15	48.38	47.89	48.26	264155	{}	\N	2020-10-12 16:47:29.881339
83516	FXG	2020-10-02	47.8	48.65	47.8	48.53	24651	{}	\N	2020-10-12 16:47:29.886518
83517	FXG	2020-10-05	48.64	49.06	48.64	49.05	20359	{}	\N	2020-10-12 16:47:29.892886
83518	FXG	2020-10-06	49.31	49.54	48.8073	48.94	7375	{}	\N	2020-10-12 16:47:29.897452
83519	FXG	2020-10-07	49.21	49.6765	49.21	49.49	45289	{}	\N	2020-10-12 16:47:29.901643
83520	FXG	2020-10-08	49.67	49.9301	49.665	49.89	10082	{}	\N	2020-10-12 16:47:29.906055
83521	FXG	2020-10-09	49.99	50.28	49.975	50.09	10959	{}	\N	2020-10-12 16:47:29.910672
83522	FXG	2020-10-12	50.37	50.665	50.2096	50.58	6297	{}	\N	2020-10-12 16:47:29.915206
83523	EDOG	2020-09-28	18.28	18.3146	18.28	18.3146	397	{}	\N	2020-10-12 16:47:31.451782
83524	EDOG	2020-09-29	18.35	18.384	18.275	18.31	3699	{}	\N	2020-10-12 16:47:31.465083
83525	EDOG	2020-09-30	18.4	18.4243	18.4	18.4	906	{}	\N	2020-10-12 16:47:31.473332
83526	EDOG	2020-10-01	18.42	18.49	18.38	18.49	6353	{}	\N	2020-10-12 16:47:31.480293
83527	EDOG	2020-10-02	18.45	18.45	18.3841	18.3841	312	{}	\N	2020-10-12 16:47:31.487198
83528	EDOG	2020-10-05	18.5	18.67	18.5	18.67	1806	{}	\N	2020-10-12 16:47:31.492747
83529	EDOG	2020-10-06	18.74	18.74	18.5359	18.5359	511	{}	\N	2020-10-12 16:47:31.49845
83530	EDOG	2020-10-07	18.5889	18.65	18.585	18.6	2586	{}	\N	2020-10-12 16:47:31.504023
83531	EDOG	2020-10-08	18.8699	18.8699	18.8594	18.8594	527	{}	\N	2020-10-12 16:47:31.509302
83532	EDOG	2020-10-09	18.98	19.04	18.98	18.9999	1026	{}	\N	2020-10-12 16:47:31.514968
83533	EDOG	2020-10-12	18.9555	18.9555	18.9555	18.9555	292	{}	\N	2020-10-12 16:47:31.520585
83534	ENIA	2020-09-28	6.57	6.62	6.415	6.44	1410028	{}	\N	2020-10-12 16:47:33.223017
83535	ENIA	2020-09-29	6.42	6.45	6.22	6.22	2350215	{}	\N	2020-10-12 16:47:33.23553
83536	ENIA	2020-09-30	6.28	6.475	6.225	6.46	4746930	{}	\N	2020-10-12 16:47:33.243178
83537	ENIA	2020-10-01	6.47	6.605	6.38	6.4	2889572	{}	\N	2020-10-12 16:47:33.249809
83538	ENIA	2020-10-02	6.35	6.48	6.33	6.46	1724108	{}	\N	2020-10-12 16:47:33.255727
83539	ENIA	2020-10-05	6.48	6.48	6.31	6.45	2643079	{}	\N	2020-10-12 16:47:33.261103
83540	ENIA	2020-10-06	6.46	6.48	6.255	6.31	2568110	{}	\N	2020-10-12 16:47:33.266547
83541	ENIA	2020-10-07	6.35	6.41	6.26	6.3	1631816	{}	\N	2020-10-12 16:47:33.271895
83542	ENIA	2020-10-08	6.31	6.39	6.27	6.38	1936904	{}	\N	2020-10-12 16:47:33.277243
83543	ENIA	2020-10-09	6.39	6.475	6.335	6.41	2649951	{}	\N	2020-10-12 16:47:33.282054
83544	ENIA	2020-10-12	6.4	6.44	6.32	6.35	679452	{}	\N	2020-10-12 16:47:33.287244
83545	EQAL	2020-09-28	32.02	32.34	32.02	32.2401	12408	{}	\N	2020-10-12 16:47:34.537655
83546	EQAL	2020-09-29	32.25	32.28	31.919	32.02	68746	{}	\N	2020-10-12 16:47:34.549358
83547	EQAL	2020-09-30	32.12	32.43	32.04	32.18	100640	{}	\N	2020-10-12 16:47:34.557032
83548	EQAL	2020-10-01	32.34	32.4099	32.105	32.32	33653	{}	\N	2020-10-12 16:47:34.564341
83549	EQAL	2020-10-02	31.71	32.64	31.71	32.55	31236	{}	\N	2020-10-12 16:47:34.570284
83550	EQAL	2020-10-05	32.67	33.19	32.67	33.19	47832	{}	\N	2020-10-12 16:47:34.575483
83551	EQAL	2020-10-06	33.27	33.6	32.93	32.99	24719	{}	\N	2020-10-12 16:47:34.58037
83552	EQAL	2020-10-07	33.4262	33.58	33.34	33.48	25204	{}	\N	2020-10-12 16:47:34.585337
83553	EQAL	2020-10-08	33.84	34.03	33.7762	34.03	42103	{}	\N	2020-10-12 16:47:34.590492
83554	EQAL	2020-10-09	34.14	34.24	34	34.06	53806	{}	\N	2020-10-12 16:47:34.595356
83555	EQAL	2020-10-12	34.27	34.27	34.0998	34.26	35932	{}	\N	2020-10-12 16:47:34.600754
83556	EQR	2020-09-28	52.4	52.91	51.87	52.54	2193757	{}	\N	2020-10-12 16:47:36.263765
83557	EQR	2020-09-29	52.3	52.46	50.63	51.09	5694228	{}	\N	2020-10-12 16:47:36.274327
83558	EQR	2020-09-30	51.53	51.95	50.69	51.33	5948309	{}	\N	2020-10-12 16:47:36.282311
83559	EQR	2020-10-01	51.51	53.5	51.28	53.44	5971036	{}	\N	2020-10-12 16:47:36.288722
83560	EQR	2020-10-02	52.31	55.44	52.03	55.09	12610711	{}	\N	2020-10-12 16:47:36.295234
83561	EQR	2020-10-05	56	56.41	54.58	55.6	5364723	{}	\N	2020-10-12 16:47:36.300883
83562	EQR	2020-10-06	55.81	56.16	54.8701	55.19	4616913	{}	\N	2020-10-12 16:47:36.30631
83563	EQR	2020-10-07	55.45	55.595	54.33	54.76	3475214	{}	\N	2020-10-12 16:47:36.311884
83564	EQR	2020-10-08	55.18	56.2	55	56.19	4163646	{}	\N	2020-10-12 16:47:36.317072
83565	EQR	2020-10-09	56.32	56.405	55.24	55.66	3231659	{}	\N	2020-10-12 16:47:36.322015
83566	EQR	2020-10-12	55.74	55.8	54.9	54.925	879604	{}	\N	2020-10-12 16:47:36.327006
83567	FANH	2020-09-28	17.36	17.54	16.75	16.9	102003	{}	\N	2020-10-12 16:47:38.03591
83568	FANH	2020-09-29	17.03	17.3	16.95	17.13	61515	{}	\N	2020-10-12 16:47:38.046283
83569	FANH	2020-09-30	17.34	17.45	16.71	16.75	38983	{}	\N	2020-10-12 16:47:38.053227
83570	FANH	2020-10-01	16.92	17.145	16.57	16.91	82612	{}	\N	2020-10-12 16:47:38.059328
83571	FANH	2020-10-02	16.72	17	16.68	16.83	92477	{}	\N	2020-10-12 16:47:38.065216
83572	FANH	2020-10-05	16.99	16.99	16.5	16.59	42546	{}	\N	2020-10-12 16:47:38.07036
83573	FANH	2020-10-06	16.51	16.92	16.41	16.64	82719	{}	\N	2020-10-12 16:47:38.075003
83574	FANH	2020-10-07	16.64	16.7699	16.49	16.54	42978	{}	\N	2020-10-12 16:47:38.079686
83575	FANH	2020-10-08	16.5	16.875	16.46	16.5	31622	{}	\N	2020-10-12 16:47:38.084346
83576	FANH	2020-10-09	16.52	16.86	16.26	16.42	56610	{}	\N	2020-10-12 16:47:38.089042
83577	FANH	2020-10-12	16.49	16.51	16.285	16.41	18111	{}	\N	2020-10-12 16:47:38.093236
83578	FBGX	2020-09-28	445.9104	445.9104	445.9104	445.9104	128	{}	\N	2020-10-12 16:47:39.071246
83579	FBGX	2020-09-29	444.6339	444.6339	444.6339	444.6339	330	{}	\N	2020-10-12 16:47:39.081808
83580	FBGX	2020-09-30	452.51	456.8	450	450	783	{}	\N	2020-10-12 16:47:39.089506
83581	FBGX	2020-10-01	459	461.012	459	461.012	181	{}	\N	2020-10-12 16:47:39.095452
83582	FBGX	2020-10-02	444.0736	444.0736	444.0736	444.0736	293	{}	\N	2020-10-12 16:47:39.101495
83583	FBGX	2020-10-05	449.35	460.9327	449.35	460.9327	352	{}	\N	2020-10-12 16:47:39.106366
83584	FBGX	2020-10-06	458	458	446.8116	446.8116	600	{}	\N	2020-10-12 16:47:39.11113
83585	FBGX	2020-10-07	455.75	463.4792	455.72	463.4792	3617	{}	\N	2020-10-12 16:47:39.116133
83586	FBGX	2020-10-08	464.58	464.58	462.6911	462.6911	537	{}	\N	2020-10-12 16:47:39.12083
83587	FBGX	2020-10-09	474.098	474.098	474.098	474.098	157	{}	\N	2020-10-12 16:47:39.125622
83588	FBGX	2020-10-12	494.98	500	494.98	500	1206	{}	\N	2020-10-12 16:47:39.130239
83589	FDM	2020-09-28	36.99	37.58	36.99	37.58	11744	{}	\N	2020-10-12 16:47:40.369342
83590	FDM	2020-09-29	37.43	37.5065	36.93	37.4275	6989	{}	\N	2020-10-12 16:47:40.383291
83591	FDM	2020-09-30	37.59	37.9172	37.2598	37.2598	10016	{}	\N	2020-10-12 16:47:40.392699
83592	FDM	2020-10-01	37.78	38.13	37.291	38.13	87839	{}	\N	2020-10-12 16:47:40.398844
83593	FDM	2020-10-02	37.3	38.37	37.24	38.24	88570	{}	\N	2020-10-12 16:47:40.404717
83594	FDM	2020-10-05	38.69	39.65	38.69	39.65	157212	{}	\N	2020-10-12 16:47:40.409366
83595	FDM	2020-10-06	39.8393	40.3467	39.4259	39.4259	3206	{}	\N	2020-10-12 16:47:40.4145
83596	FDM	2020-10-07	40.04	40.2369	39.8878	40.2369	7616	{}	\N	2020-10-12 16:47:40.419601
83597	FDM	2020-10-08	40.52	40.795	40.409	40.78	18267	{}	\N	2020-10-12 16:47:40.424356
83598	FDM	2020-10-09	41.05	41.23	40.64	40.64	19364	{}	\N	2020-10-12 16:47:40.429142
83599	FDM	2020-10-12	40.8	40.945	40.8	40.8767	1603	{}	\N	2020-10-12 16:47:40.434086
83600	FIBR	2020-09-28	101.47	101.64	101.46	101.5553	1649	{}	\N	2020-10-12 16:47:41.466748
83601	FIBR	2020-09-29	101.635	101.635	101.6	101.6	1032	{}	\N	2020-10-12 16:47:41.476802
83602	FIBR	2020-09-30	101.6523	101.6523	101.575	101.575	23727	{}	\N	2020-10-12 16:47:41.483209
83603	FIBR	2020-10-01	101.42	101.52	101.38	101.48	7600	{}	\N	2020-10-12 16:47:41.489258
83604	FIBR	2020-10-02	101.64	101.705	101.6001	101.68	11428	{}	\N	2020-10-12 16:47:41.494635
83605	FIBR	2020-10-05	101.609	101.609	101.4673	101.53	3225	{}	\N	2020-10-12 16:47:41.499024
83606	FIBR	2020-10-06	101.5739	101.5739	101.5095	101.5095	1484	{}	\N	2020-10-12 16:47:41.503991
83607	FIBR	2020-10-07	101.47	101.7	101.47	101.6017	3039	{}	\N	2020-10-12 16:47:41.508459
83608	FIBR	2020-10-08	101.6526	101.6526	101.58	101.59	2005	{}	\N	2020-10-12 16:47:41.512756
83609	FIBR	2020-10-09	101.59	101.74	101.59	101.61	2700	{}	\N	2020-10-12 16:47:41.517035
83610	FIBR	2020-10-12	101.7787	101.7787	101.76	101.76	918	{}	\N	2020-10-12 16:47:41.521709
83611	FINX	2020-09-28	37.78	38	37.65	37.78	195407	{}	\N	2020-10-12 16:47:43.739275
83612	FINX	2020-09-29	37.94	38.26	37.94	38.08	196435	{}	\N	2020-10-12 16:47:43.75168
83613	FINX	2020-09-30	38.1	38.37	37.74	38.03	201444	{}	\N	2020-10-12 16:47:43.758761
83614	FINX	2020-10-01	38.53	38.67	38.2986	38.61	169680	{}	\N	2020-10-12 16:47:43.764705
83615	FINX	2020-10-02	37.831	38.4	37.831	38.26	204051	{}	\N	2020-10-12 16:47:43.770072
83616	FINX	2020-10-05	38.52	38.92	38.52	38.89	188511	{}	\N	2020-10-12 16:47:43.775585
83617	FINX	2020-10-06	38.92	39.38	38.43	38.53	162194	{}	\N	2020-10-12 16:47:43.780342
83618	FINX	2020-10-07	39.07	39.4	38.92	39.34	149827	{}	\N	2020-10-12 16:47:43.78497
83619	FINX	2020-10-08	39.85	40	39.71	39.95	109340	{}	\N	2020-10-12 16:47:43.789475
83620	FINX	2020-10-09	40.29	40.623	40.09	40.57	137518	{}	\N	2020-10-12 16:47:43.794281
83621	FINX	2020-10-12	41.01	41.2	40.82	41.125	100360	{}	\N	2020-10-12 16:47:43.798711
83622	FIVA	2020-09-28	19.06	19.1703	19.06	19.1703	458	{}	\N	2020-10-12 16:47:44.923398
83623	FIVA	2020-09-29	18.98	19.0102	18.93	19.01	18114	{}	\N	2020-10-12 16:47:44.937681
83624	FIVA	2020-09-30	18.94	18.98	18.89	18.89	926	{}	\N	2020-10-12 16:47:44.94504
83625	FIVA	2020-10-01	18.8596	18.8596	18.8596	18.8596	375	{}	\N	2020-10-12 16:47:44.951181
83626	FIVA	2020-10-02	18.81	18.93	18.81	18.93	961	{}	\N	2020-10-12 16:47:44.957351
83627	FIVA	2020-10-05	19.15	19.2	19.1	19.2	689	{}	\N	2020-10-12 16:47:44.961991
83628	FIVA	2020-10-06	19.2783	19.2783	19.2	19.2	689	{}	\N	2020-10-12 16:47:44.966383
83629	FIVA	2020-10-07	19.25	19.2999	19.25	19.2999	570	{}	\N	2020-10-12 16:47:44.9711
83630	FIVA	2020-10-08	19.38	19.4119	19.38	19.4	3881	{}	\N	2020-10-12 16:47:44.975664
83631	FIVA	2020-10-09	19.4247	19.48	19.4247	19.48	675	{}	\N	2020-10-12 16:47:44.980819
83632	FIVA	2020-10-12	19.48	19.48	19.48	19.48	1238	{}	\N	2020-10-12 16:47:44.985075
83633	FLBR	2020-09-28	17.13	17.13	17.04	17.04	1008	{}	\N	2020-10-12 16:47:46.139367
83634	FLBR	2020-09-29	16.96	17	16.82	16.82	6102	{}	\N	2020-10-12 16:47:46.150486
83635	FLBR	2020-09-30	17.0601	17.1231	16.95	17.1231	40974	{}	\N	2020-10-12 16:47:46.157926
83636	FLBR	2020-10-01	16.845	17.1202	16.845	17.1202	601	{}	\N	2020-10-12 16:47:46.165194
83637	FLBR	2020-10-02	17	17	16.84	16.88	4061	{}	\N	2020-10-12 16:47:46.173286
83638	FLBR	2020-10-05	16.98	17.5566	16.93	17.5566	9099	{}	\N	2020-10-12 16:47:46.180353
83639	FLBR	2020-10-06	17.7401	17.95	17.2755	17.33	3991	{}	\N	2020-10-12 16:47:46.186868
83640	FLBR	2020-10-07	17.35	17.48	17.1	17.32	3895	{}	\N	2020-10-12 16:47:46.193702
83641	FLBR	2020-10-08	17.28	17.7584	17.28	17.7584	1449	{}	\N	2020-10-12 16:47:46.200608
83642	FLBR	2020-10-09	17.897	17.92	17.8617	17.92	689	{}	\N	2020-10-12 16:47:46.207625
83643	FLBR	2020-10-12	17.97	18.11	17.97	18.11	323	{}	\N	2020-10-12 16:47:46.214438
83644	EDIV	2020-09-28	24.09	24.18	23.9811	24.11	21601	{}	\N	2020-10-12 16:47:49.54298
83645	EDIV	2020-09-29	24	24.1592	23.9908	24.09	53504	{}	\N	2020-10-12 16:47:49.552523
83646	EDIV	2020-09-30	24.2	24.39	24.18	24.36	62495	{}	\N	2020-10-12 16:47:49.558915
83647	EDIV	2020-10-01	24.43	24.5	24.32	24.46	38748	{}	\N	2020-10-12 16:47:49.565022
83648	EDIV	2020-10-02	24.16	24.47	24.16	24.33	39028	{}	\N	2020-10-12 16:47:49.570559
83649	EDIV	2020-10-05	24.27	24.43	24.27	24.35	90729	{}	\N	2020-10-12 16:47:49.5756
83650	EDIV	2020-10-06	24.38	24.4465	24.2	24.26	78265	{}	\N	2020-10-12 16:47:49.58069
83651	EDIV	2020-10-07	24.42	24.4431	24.35	24.4	83601	{}	\N	2020-10-12 16:47:49.585322
83652	EDIV	2020-10-08	24.41	24.6	24.41	24.54	41282	{}	\N	2020-10-12 16:47:49.590121
83653	EDIV	2020-10-09	24.56	24.74	24.5275	24.66	44102	{}	\N	2020-10-12 16:47:49.594848
83654	EDIV	2020-10-12	24.7	24.77	24.66	24.77	13431	{}	\N	2020-10-12 16:47:49.599127
83655	EGO	2020-09-28	10.26	10.35	10.07	10.28	1837202	{}	\N	2020-10-12 16:47:50.833138
83656	EGO	2020-09-29	10.38	10.5706	10.315	10.54	3600353	{}	\N	2020-10-12 16:47:50.845759
83657	EGO	2020-09-30	10.43	10.66	10.345	10.55	3256305	{}	\N	2020-10-12 16:47:50.854854
83658	EGO	2020-10-01	10.68	10.73	10.4829	10.64	3438654	{}	\N	2020-10-12 16:47:50.861716
83659	EGO	2020-10-02	10.51	10.6325	10.4236	10.52	4998226	{}	\N	2020-10-12 16:47:50.867646
83660	EGO	2020-10-05	10.58	11.14	10.58	11.12	5182521	{}	\N	2020-10-12 16:47:50.87271
83661	EGO	2020-10-06	11.19	11.41	11.1	11.18	7525617	{}	\N	2020-10-12 16:47:50.877803
83662	EGO	2020-10-07	11.255	11.41	10.82	10.85	4502182	{}	\N	2020-10-12 16:47:50.882649
83663	EGO	2020-10-08	10.93	11.21	10.93	11.2	3099924	{}	\N	2020-10-12 16:47:50.887738
83664	EGO	2020-10-09	11.56	12.42	11.5	12.41	7189050	{}	\N	2020-10-12 16:47:50.893148
83665	EGO	2020-10-12	12.4	13.29	12.14	13.285	5370699	{}	\N	2020-10-12 16:47:50.898204
83666	FBND	2020-09-28	54.61	54.64	54.46	54.55	232773	{}	\N	2020-10-12 16:47:53.96169
83667	FBND	2020-09-29	54.55	54.69	54.55	54.66	247979	{}	\N	2020-10-12 16:47:53.972965
83668	FBND	2020-09-30	54.66	54.7	54.5285	54.65	216235	{}	\N	2020-10-12 16:47:53.981344
83669	FBND	2020-10-01	54.62	54.73	54.5884	54.73	708293	{}	\N	2020-10-12 16:47:53.98746
83670	FBND	2020-10-02	54.73	54.73	54.56	54.65	364199	{}	\N	2020-10-12 16:47:53.99347
83671	FBND	2020-10-05	54.48	54.58	54.45	54.54	242574	{}	\N	2020-10-12 16:47:53.99839
83672	FBND	2020-10-06	54.5	54.63	54.4363	54.61	158066	{}	\N	2020-10-12 16:47:54.003078
83673	FBND	2020-10-07	54.6	54.64	54.56	54.63	103282	{}	\N	2020-10-12 16:47:54.007747
83674	FBND	2020-10-08	54.51	54.71	54.51	54.68	113941	{}	\N	2020-10-12 16:47:54.012598
83675	FBND	2020-10-09	54.7792	54.7792	54.63	54.72	183075	{}	\N	2020-10-12 16:47:54.017301
83676	FBND	2020-10-12	54.78	54.79	54.72	54.77	104292	{}	\N	2020-10-12 16:47:54.02191
83677	FCAP	2020-09-28	56.84	57	56.84	57	2547	{}	\N	2020-10-12 16:47:55.909701
83678	FCAP	2020-09-29	57.09	59.95	56.7	56.7	1151	{}	\N	2020-10-12 16:47:55.920669
83679	FCAP	2020-09-30	56.87	58.99	55.57	55.57	1006	{}	\N	2020-10-12 16:47:55.928926
83680	FCAP	2020-10-01	56.05	58.75	56.05	56.8	1990	{}	\N	2020-10-12 16:47:55.936924
83681	FCAP	2020-10-02	58.43	58.43	58.43	58.43	785	{}	\N	2020-10-12 16:47:55.943002
83682	FCAP	2020-10-05	55.98	58.97	55.98	58.06	2945	{}	\N	2020-10-12 16:47:55.948017
83683	FCAP	2020-10-06	56.1	56.1	56.1	56.1	1315	{}	\N	2020-10-12 16:47:55.953301
83684	FCAP	2020-10-07	55.925	58.225	53.01	57.17	2387	{}	\N	2020-10-12 16:47:55.958246
83685	FCAP	2020-10-08	58	58	56.18	56.18	1162	{}	\N	2020-10-12 16:47:55.963128
83686	FCAP	2020-10-09	57.98	57.98	57.98	57.98	1102	{}	\N	2020-10-12 16:47:55.967946
83687	FCAP	2020-10-12	57.98	57.98	57.955	57.955	452	{}	\N	2020-10-12 16:47:55.972788
83688	FDTS	2020-09-29	36.5475	36.5475	36.535	36.535	186	{}	\N	2020-10-12 16:47:57.760891
83689	FDTS	2020-09-30	36.55	36.55	36.525	36.525	494	{}	\N	2020-10-12 16:47:57.767998
83690	FDTS	2020-10-02	36.4	36.4848	36.4	36.4848	323	{}	\N	2020-10-12 16:47:57.774363
83691	FDTS	2020-10-05	36.98	36.98	36.98	36.98	300	{}	\N	2020-10-12 16:47:57.779191
83692	FDTS	2020-10-06	37.32	37.3201	37.0714	37.0714	227	{}	\N	2020-10-12 16:47:57.784747
83693	FDTS	2020-10-07	36.98	37.01	36.93	37.01	3500	{}	\N	2020-10-12 16:47:57.790331
83694	FDTS	2020-10-08	37.56	37.58	37.54	37.58	2401	{}	\N	2020-10-12 16:47:57.79605
83695	FIXD	2020-09-28	55.21	55.21	55.08	55.12	1060385	{}	\N	2020-10-12 16:47:59.940813
83696	FIXD	2020-09-29	55.12	55.19	55.1	55.15	1589236	{}	\N	2020-10-12 16:47:59.951904
83697	FIXD	2020-09-30	55.13	55.15	54.94	55.03	1251858	{}	\N	2020-10-12 16:47:59.959848
83698	FIXD	2020-10-01	55.1	55.2	55.06	55.17	1058007	{}	\N	2020-10-12 16:47:59.96691
83699	FIXD	2020-10-02	55.17	55.2087	55.08	55.15	1398574	{}	\N	2020-10-12 16:47:59.972269
83700	FIXD	2020-10-05	55.07	55.09	54.97	55	897828	{}	\N	2020-10-12 16:47:59.977447
83701	FIXD	2020-10-06	55.0174	55.08	54.92	55.05	1041863	{}	\N	2020-10-12 16:47:59.982183
83702	FIXD	2020-10-07	55.02	55.055	54.96	54.99	742677	{}	\N	2020-10-12 16:47:59.9867
83703	FIXD	2020-10-08	54.94	55.06	54.94	55.06	708574	{}	\N	2020-10-12 16:47:59.991293
83704	FIXD	2020-10-09	55.09	55.11	54.97	55.03	1037935	{}	\N	2020-10-12 16:47:59.996158
83705	FIXD	2020-10-12	55.1	55.11	55.0693	55.095	383222	{}	\N	2020-10-12 16:48:00.000398
83706	FPF	2020-09-28	21.31	21.48	21.2	21.29	134534	{}	\N	2020-10-12 16:48:03.178649
83707	FPF	2020-09-29	21.3	21.44	21.2	21.37	221715	{}	\N	2020-10-12 16:48:03.18625
83708	FPF	2020-09-30	21.42	21.52	21.29	21.49	315244	{}	\N	2020-10-12 16:48:03.192663
83709	FPF	2020-10-01	21.49	21.5167	21.31	21.44	161951	{}	\N	2020-10-12 16:48:03.197989
83710	FPF	2020-10-02	21.23	21.5	21.23	21.48	213181	{}	\N	2020-10-12 16:48:03.203082
83711	FPF	2020-10-05	21.5	21.62	21.4147	21.52	201890	{}	\N	2020-10-12 16:48:03.208788
83712	FPF	2020-10-06	21.5	21.62	21.46	21.54	207705	{}	\N	2020-10-12 16:48:03.214008
83713	FPF	2020-10-07	21.66	21.778	21.63	21.73	166687	{}	\N	2020-10-12 16:48:03.219463
83714	FPF	2020-10-08	21.88	21.89	21.74	21.74	148046	{}	\N	2020-10-12 16:48:03.224638
83715	FPF	2020-10-09	21.8	21.8376	21.67	21.69	209972	{}	\N	2020-10-12 16:48:03.229952
83716	FPF	2020-10-12	21.74	21.84	21.7109	21.8	51634	{}	\N	2020-10-12 16:48:03.235006
83717	FTXO	2020-09-28	16.63	16.9879	16.63	16.85	39040	{}	\N	2020-10-12 16:48:04.976659
83718	FTXO	2020-09-29	16.83	16.83	16.48	16.68	32059	{}	\N	2020-10-12 16:48:04.988446
83719	FTXO	2020-09-30	16.79	17.0667	16.76	16.9	56842	{}	\N	2020-10-12 16:48:04.996349
83720	FTXO	2020-10-01	17.0627	17.0627	16.8	16.94	11228	{}	\N	2020-10-12 16:48:05.00245
83721	FTXO	2020-10-02	16.59	17.38	16.59	17.345	57081	{}	\N	2020-10-12 16:48:05.008565
83722	FTXO	2020-10-05	17.6831	18	17.6831	18	33801	{}	\N	2020-10-12 16:48:05.014016
83723	FTXO	2020-10-06	18.103	18.6299	17.91	17.94	18747	{}	\N	2020-10-12 16:48:05.019101
83724	FTXO	2020-10-07	18.18	18.5929	18.18	18.459	10068	{}	\N	2020-10-12 16:48:05.024755
83725	FTXO	2020-10-08	18.46	18.7066	18.46	18.7066	15698	{}	\N	2020-10-12 16:48:05.030193
83726	FTXO	2020-10-09	18.8099	18.8099	18.44	18.57	7397	{}	\N	2020-10-12 16:48:05.035427
83727	FTXO	2020-10-12	18.53	18.65	18.53	18.63	5049	{}	\N	2020-10-12 16:48:05.040712
83728	FXN	2020-09-28	5.61	5.76	5.595	5.69	316404	{}	\N	2020-10-12 16:48:07.388893
83729	FXN	2020-09-29	5.67	5.68	5.555	5.63	294094	{}	\N	2020-10-12 16:48:07.401445
83730	FXN	2020-09-30	5.69	5.725	5.571	5.6	276247	{}	\N	2020-10-12 16:48:07.408958
83731	FXN	2020-10-01	5.58	5.615	5.49	5.52	422808	{}	\N	2020-10-12 16:48:07.415316
83732	FXN	2020-10-02	5.36	5.67	5.35	5.62	170329	{}	\N	2020-10-12 16:48:07.420425
83733	FXN	2020-10-05	5.72	5.89	5.72	5.89	664251	{}	\N	2020-10-12 16:48:07.425247
83734	FXN	2020-10-06	5.97	6.03	5.745	5.77	924687	{}	\N	2020-10-12 16:48:07.429692
83735	FXN	2020-10-07	5.83	5.96	5.79	5.94	202383	{}	\N	2020-10-12 16:48:07.433837
83736	FXN	2020-10-08	5.99	6.14	5.95	6.14	878137	{}	\N	2020-10-12 16:48:07.439063
83737	FXN	2020-10-09	6.17	6.2016	6.04	6.08	174449	{}	\N	2020-10-12 16:48:07.444197
83738	FXN	2020-10-12	6.07	6.07	5.975	6.02	84921	{}	\N	2020-10-12 16:48:07.449493
83739	GPP	2020-09-28	6.805	7.31	6.76	7.1476	56230	{}	\N	2020-10-12 16:48:08.897301
83740	GPP	2020-09-29	7.09	7.19	6.84	7	58132	{}	\N	2020-10-12 16:48:08.909994
83741	GPP	2020-09-30	7.19	7.39	7	7	149664	{}	\N	2020-10-12 16:48:08.917654
83742	GPP	2020-10-01	7.01	7.26	6.91	7.21	56690	{}	\N	2020-10-12 16:48:08.923748
83743	GPP	2020-10-02	7.08	7.23	6.9601	7.22	28433	{}	\N	2020-10-12 16:48:08.930073
83744	GPP	2020-10-05	7.12	7.39	7.12	7.34	18319	{}	\N	2020-10-12 16:48:08.935129
83745	GPP	2020-10-06	7.405	7.4553	7.2101	7.2101	24527	{}	\N	2020-10-12 16:48:08.940333
83746	GPP	2020-10-07	7.25	7.7399	7.19	7.66	76641	{}	\N	2020-10-12 16:48:08.945655
83747	GPP	2020-10-08	7.7	7.79	7.4235	7.76	37974	{}	\N	2020-10-12 16:48:08.951048
83748	GPP	2020-10-09	7.7599	7.8	7.6191	7.74	15178	{}	\N	2020-10-12 16:48:08.957036
83749	GPP	2020-10-12	7.73	7.84	7.715	7.765	12947	{}	\N	2020-10-12 16:48:08.962611
83750	GRNB	2020-09-28	27.8	27.81	27.756	27.79	8423	{}	\N	2020-10-12 16:48:11.635105
83751	GRNB	2020-09-29	27.775	27.808	27.77	27.77	2706	{}	\N	2020-10-12 16:48:11.650327
83752	GRNB	2020-09-30	27.76	27.91	27.755	27.755	1245	{}	\N	2020-10-12 16:48:11.65818
83753	GRNB	2020-10-01	27.77	27.77	27.7175	27.75	9321	{}	\N	2020-10-12 16:48:11.664542
83754	GRNB	2020-10-02	27.72	27.74	27.7	27.73	5861	{}	\N	2020-10-12 16:48:11.671184
83755	GRNB	2020-10-05	27.71	27.71	27.6758	27.6758	8018	{}	\N	2020-10-12 16:48:11.676452
83756	GRNB	2020-10-06	27.63	27.72	27.63	27.6882	11586	{}	\N	2020-10-12 16:48:11.681302
83757	GRNB	2020-10-07	27.82	27.82	27.641	27.69	13849	{}	\N	2020-10-12 16:48:11.686321
83758	GRNB	2020-10-08	27.6829	27.76	27.6829	27.75	12952	{}	\N	2020-10-12 16:48:11.69155
83759	GRNB	2020-10-09	27.73	27.75	27.7	27.73	10272	{}	\N	2020-10-12 16:48:11.696722
83760	GRNB	2020-10-12	27.78	27.78	27.755	27.7608	8769	{}	\N	2020-10-12 16:48:11.702416
83761	EDV	2020-09-28	167.65	167.69	167.03	167.54	202120	{}	\N	2020-10-12 16:48:13.827033
83762	EDV	2020-09-29	167.5	168.075	167.275	167.6	263884	{}	\N	2020-10-12 16:48:13.839656
83763	EDV	2020-09-30	166.48	166.5816	164.6161	165.56	146581	{}	\N	2020-10-12 16:48:13.847672
83764	EDV	2020-10-01	163.83	165.705	163.5501	165.29	214408	{}	\N	2020-10-12 16:48:13.854161
83765	EDV	2020-10-02	165.345	165.57	163.55	164.05	617406	{}	\N	2020-10-12 16:48:13.860155
83766	EDV	2020-10-05	161.87	162.06	160.1325	160.24	493169	{}	\N	2020-10-12 16:48:13.865193
83767	EDV	2020-10-06	159.69	162.23	158.8288	161.08	435174	{}	\N	2020-10-12 16:48:13.870524
83768	EDV	2020-10-07	159.8799	161.025	159.06	159.61	204781	{}	\N	2020-10-12 16:48:13.875773
83769	EDV	2020-10-08	160.58	161	160.195	161	176400	{}	\N	2020-10-12 16:48:13.880842
83770	EDV	2020-10-09	159.67	161.4767	159.01	160.58	184515	{}	\N	2020-10-12 16:48:13.88602
83771	EDV	2020-10-12	161.25	161.25	160.79	160.9298	29845	{}	\N	2020-10-12 16:48:13.891379
83772	EFT	2020-09-28	11.92	11.965	11.92	11.94	69335	{}	\N	2020-10-12 16:48:15.122056
83773	EFT	2020-09-29	11.94	11.94	11.85	11.91	340140	{}	\N	2020-10-12 16:48:15.135378
83774	EFT	2020-09-30	11.94	11.94	11.88	11.88	349176	{}	\N	2020-10-12 16:48:15.143278
83775	EFT	2020-10-01	11.92	11.97	11.89	11.92	124642	{}	\N	2020-10-12 16:48:15.150123
83776	EFT	2020-10-02	11.9	11.98	11.87	11.97	126521	{}	\N	2020-10-12 16:48:15.156599
83777	EFT	2020-10-05	11.98	12.06	11.93	12.04	205158	{}	\N	2020-10-12 16:48:15.162197
83778	EFT	2020-10-06	12.03	12.07	11.97	12.05	189727	{}	\N	2020-10-12 16:48:15.167425
83779	EFT	2020-10-07	12.05	12.07	11.9991	12	313593	{}	\N	2020-10-12 16:48:15.172737
83780	EFT	2020-10-08	12.02	12.4	12.02	12.34	1064334	{}	\N	2020-10-12 16:48:15.178246
83781	EFT	2020-10-09	12.35	12.479	12.35	12.46	246852	{}	\N	2020-10-12 16:48:15.183833
83782	EFT	2020-10-12	12.48	12.5591	12.48	12.555	88562	{}	\N	2020-10-12 16:48:15.188688
83783	ETY	2020-09-28	10.93	10.99	10.88	10.91	332214	{}	\N	2020-10-12 16:48:19.99211
83784	ETY	2020-09-29	10.89	10.93	10.83	10.84	629953	{}	\N	2020-10-12 16:48:20.000031
83785	ETY	2020-09-30	10.92	10.97	10.86	10.89	1166286	{}	\N	2020-10-12 16:48:20.006559
83786	ETY	2020-10-01	10.95	11.02	10.95	10.97	537157	{}	\N	2020-10-12 16:48:20.012745
83787	ETY	2020-10-02	10.76	10.949	10.76	10.86	1014345	{}	\N	2020-10-12 16:48:20.017997
83788	ETY	2020-10-05	10.9	11.02	10.9	11.01	688524	{}	\N	2020-10-12 16:48:20.022925
83789	ETY	2020-10-06	11.05	11.1	10.92	10.96	687835	{}	\N	2020-10-12 16:48:20.028275
83790	ETY	2020-10-07	10.97	11.11	10.97	11.09	629590	{}	\N	2020-10-12 16:48:20.032775
83791	ETY	2020-10-08	11.19	11.195	11.12	11.15	444309	{}	\N	2020-10-12 16:48:20.037854
83792	ETY	2020-10-09	11.25	11.25	11.115	11.14	670662	{}	\N	2020-10-12 16:48:20.042623
83793	ETY	2020-10-12	11.19	11.28	11.19	11.27	296119	{}	\N	2020-10-12 16:48:20.047526
83794	EXPR	2020-09-28	0.6823	0.7625	0.6642	0.6659	8362814	{}	\N	2020-10-12 16:48:22.617275
83795	EXPR	2020-09-29	0.6701	0.6701	0.6103	0.6104	11635227	{}	\N	2020-10-12 16:48:22.630941
83796	EXPR	2020-09-30	0.6199	0.6599	0.605	0.61	13909528	{}	\N	2020-10-12 16:48:22.638641
83797	EXPR	2020-10-01	0.6221	0.844	0.615	0.7969	35066916	{}	\N	2020-10-12 16:48:22.645381
83798	EXPR	2020-10-02	0.745	0.8558	0.7211	0.8151	16588238	{}	\N	2020-10-12 16:48:22.651446
83799	EXPR	2020-10-05	0.8009	0.8492	0.76	0.76	6991455	{}	\N	2020-10-12 16:48:22.656773
83800	EXPR	2020-10-06	0.78	0.7873	0.6435	0.6586	13598687	{}	\N	2020-10-12 16:48:22.662261
83801	EXPR	2020-10-07	0.7035	0.7572	0.685	0.7038	11873775	{}	\N	2020-10-12 16:48:22.66734
83802	EXPR	2020-10-08	0.715	0.7582	0.7057	0.73	10023576	{}	\N	2020-10-12 16:48:22.672324
83803	EXPR	2020-10-09	0.7372	0.7779	0.7117	0.7522	11604446	{}	\N	2020-10-12 16:48:22.677678
83804	EXPR	2020-10-12	0.75	0.8487	0.75	0.7911	11352174	{}	\N	2020-10-12 16:48:22.682737
83805	FCVT	2020-09-28	39.89	40.3099	39.8	40.165	620096	{}	\N	2020-10-12 16:48:24.369021
83806	FCVT	2020-09-29	40.5499	40.5499	40.05	40.13	208403	{}	\N	2020-10-12 16:48:24.381992
83807	FCVT	2020-09-30	40.23	40.56	40.13	40.26	88846	{}	\N	2020-10-12 16:48:24.390157
83808	FCVT	2020-10-01	40.64	41.0499	40.39	40.9	200931	{}	\N	2020-10-12 16:48:24.397698
83809	FCVT	2020-10-02	40.1	40.85	40.09	40.44	36869	{}	\N	2020-10-12 16:48:24.404449
83810	FCVT	2020-10-05	40.765	41.11	40.765	41.11	27124	{}	\N	2020-10-12 16:48:24.409944
83811	FCVT	2020-10-06	41.238	41.46	40.84	40.88	33512	{}	\N	2020-10-12 16:48:24.415361
83812	FCVT	2020-10-07	41.03	41.59	41.03	41.59	28884	{}	\N	2020-10-12 16:48:24.420482
83813	FCVT	2020-10-08	41.5822	42.02	41.5822	41.7691	33996	{}	\N	2020-10-12 16:48:24.425478
83814	FCVT	2020-10-09	41.968	42.18	41.84	42.18	18308	{}	\N	2020-10-12 16:48:24.430696
83815	FCVT	2020-10-12	42.4236	42.54	42.36	42.5041	170124	{}	\N	2020-10-12 16:48:24.436067
83816	FIX	2020-09-28	51.02	52.08	50.98	51.84	153783	{}	\N	2020-10-12 16:48:25.880215
83817	FIX	2020-09-29	51.84	52.13	51.2	51.51	709065	{}	\N	2020-10-12 16:48:25.892087
83818	FIX	2020-09-30	51.77	52.955	51.2	51.51	391350	{}	\N	2020-10-12 16:48:25.899583
83819	FIX	2020-10-01	51.5	52.24	51.11	52	365075	{}	\N	2020-10-12 16:48:25.904984
83820	FIX	2020-10-02	51.24	52.97	51.24	52.59	223156	{}	\N	2020-10-12 16:48:25.909781
83821	FIX	2020-10-05	53	54.4	52.87	54.05	477431	{}	\N	2020-10-12 16:48:25.914458
83822	FIX	2020-10-06	54.83	56.37	54.57	55.19	711267	{}	\N	2020-10-12 16:48:25.918477
83823	FIX	2020-10-07	55.84	56.4618	55.38	56.2	680622	{}	\N	2020-10-12 16:48:25.922138
83824	FIX	2020-10-08	56.9	57.79	56.65	57.62	419846	{}	\N	2020-10-12 16:48:25.925612
83825	FIX	2020-10-09	57.74	58.11	56.78	57.64	271357	{}	\N	2020-10-12 16:48:25.929294
83826	FIX	2020-10-12	57.82	58.095	57.24	57.52	208787	{}	\N	2020-10-12 16:48:25.934526
83827	FLC	2020-09-28	21.37	21.4127	21.29	21.32	20947	{}	\N	2020-10-12 16:48:26.846216
83828	FLC	2020-09-29	21.43	21.462	21.18	21.28	93043	{}	\N	2020-10-12 16:48:26.858427
83829	FLC	2020-09-30	21.44	21.525	21.32	21.4	83385	{}	\N	2020-10-12 16:48:26.864971
83830	FLC	2020-10-01	21.36	21.56	21.3459	21.5	76458	{}	\N	2020-10-12 16:48:26.870687
83831	FLC	2020-10-02	21.3	21.5244	21.25	21.44	54413	{}	\N	2020-10-12 16:48:26.87629
83832	FLC	2020-10-05	21.44	21.54	21.33	21.45	89684	{}	\N	2020-10-12 16:48:26.881306
83833	FLC	2020-10-06	21.51	21.51	21.375	21.4	50437	{}	\N	2020-10-12 16:48:26.886247
83834	FLC	2020-10-07	21.37	21.77	21.37	21.77	37754	{}	\N	2020-10-12 16:48:26.890593
83835	FLC	2020-10-08	21.7	21.84	21.58	21.71	47583	{}	\N	2020-10-12 16:48:26.895226
83836	FLC	2020-10-09	21.58	21.71	21.55	21.55	76122	{}	\N	2020-10-12 16:48:26.899929
83837	FLC	2020-10-12	21.61	21.71	21.55	21.7016	19982	{}	\N	2020-10-12 16:48:26.90446
83838	FLJH	2020-09-28	25.329	25.394	25.329	25.394	242	{}	\N	2020-10-12 16:48:28.935519
83839	FLJH	2020-09-29	25.43	25.43	25.3877	25.3877	802	{}	\N	2020-10-12 16:48:28.950331
83840	FLJH	2020-09-30	25.1871	25.1871	25.1871	25.1871	4	{}	\N	2020-10-12 16:48:28.958407
83841	FLJH	2020-10-01	25.15	25.1682	25.15	25.1682	428	{}	\N	2020-10-12 16:48:28.964026
83842	FLJH	2020-10-02	24.99	24.99	24.971	24.99	665	{}	\N	2020-10-12 16:48:28.970127
83843	FLJH	2020-10-05	25.2989	25.2989	25.2989	25.2989	3	{}	\N	2020-10-12 16:48:28.975476
83844	FLJH	2020-10-06	25.2079	25.2079	25.2079	25.2079	17	{}	\N	2020-10-12 16:48:28.980396
83845	FLJH	2020-10-07	25.4308	25.4308	25.4308	25.4308	3	{}	\N	2020-10-12 16:48:28.98537
83846	FLJH	2020-10-08	25.46	25.48	25.4451	25.4451	1114	{}	\N	2020-10-12 16:48:28.99025
83847	FLJH	2020-10-09	25.3617	25.3617	25.3617	25.3617	14	{}	\N	2020-10-12 16:48:28.995205
83848	FLJH	2020-10-12	25.3	25.3	25.3	25.3	790	{}	\N	2020-10-12 16:48:29.000879
83849	FNX	2020-09-28	65.63	66.1348	65.4791	65.97	44921	{}	\N	2020-10-12 16:48:30.22251
83850	FNX	2020-09-29	65.68	66.08	65.34	65.56	16649	{}	\N	2020-10-12 16:48:30.234987
83851	FNX	2020-09-30	65.7122	66.531	65.6596	65.96	64696	{}	\N	2020-10-12 16:48:30.243226
83852	FNX	2020-10-01	66.535	66.83	66.19	66.83	12923	{}	\N	2020-10-12 16:48:30.249685
83853	FNX	2020-10-02	65.42	67.4939	65.42	67.31	15764	{}	\N	2020-10-12 16:48:30.25619
83854	FNX	2020-10-05	67.99	68.94	67.99	68.94	12447	{}	\N	2020-10-12 16:48:30.261892
83855	FNX	2020-10-06	68.89	70.2499	68.6701	68.68	68504	{}	\N	2020-10-12 16:48:30.267465
83856	FNX	2020-10-07	69.491	70.16	69.491	70.15	54241	{}	\N	2020-10-12 16:48:30.272777
83857	FNX	2020-10-08	70.76	70.85	70.3278	70.85	15983	{}	\N	2020-10-12 16:48:30.278209
83858	FNX	2020-10-09	71.55	71.55	70.8774	71.2	13824	{}	\N	2020-10-12 16:48:30.28337
83859	FNX	2020-10-12	71.435	71.7499	71.31	71.64	4937	{}	\N	2020-10-12 16:48:30.288705
83860	FPH	2020-09-28	4.45	4.6	4.45	4.5	78036	{}	\N	2020-10-12 16:48:31.925322
83861	FPH	2020-09-29	4.55	4.55	4.29	4.32	202328	{}	\N	2020-10-12 16:48:31.954282
83862	FPH	2020-09-30	4.32	4.41	4.14	4.17	1221638	{}	\N	2020-10-12 16:48:31.960039
83863	FPH	2020-10-01	4.21	4.36	4.14	4.34	1945707	{}	\N	2020-10-12 16:48:31.965186
83864	FPH	2020-10-02	4.29	4.84	4.29	4.81	509280	{}	\N	2020-10-12 16:48:31.970022
83865	FPH	2020-10-05	4.88	4.91	4.65	4.79	144789	{}	\N	2020-10-12 16:48:31.975338
83866	FPH	2020-10-06	4.79	4.85	4.55	4.67	129940	{}	\N	2020-10-12 16:48:31.980282
83867	FPH	2020-10-07	4.68	4.76	4.5	4.5	169430	{}	\N	2020-10-12 16:48:31.985245
83868	FPH	2020-10-08	4.52	4.695	4.4879	4.57	156477	{}	\N	2020-10-12 16:48:31.990252
83869	FPH	2020-10-09	4.61	4.65	4.5	4.58	142555	{}	\N	2020-10-12 16:48:31.995208
83870	FPH	2020-10-12	4.63	4.72	4.55	4.67	58954	{}	\N	2020-10-12 16:48:32.000147
83871	GAB	2020-09-28	5.16	5.24	5.16	5.18	460209	{}	\N	2020-10-12 16:48:32.935144
83872	GAB	2020-09-29	5.18	5.2	5.11	5.11	778923	{}	\N	2020-10-12 16:48:32.947086
83873	GAB	2020-09-30	5.11	5.19	5.06	5.09	1448120	{}	\N	2020-10-12 16:48:32.955264
83874	GAB	2020-10-01	5.13	5.16	5.07	5.07	768459	{}	\N	2020-10-12 16:48:32.961929
83875	GAB	2020-10-02	5	5.0887	5	5.06	1503243	{}	\N	2020-10-12 16:48:32.967694
83876	GAB	2020-10-05	5.1	5.15	5.07	5.13	774760	{}	\N	2020-10-12 16:48:32.973055
83877	GAB	2020-10-06	5.15	5.21	5.09	5.11	1436306	{}	\N	2020-10-12 16:48:32.978274
83878	GAB	2020-10-07	5.14	5.19	5.14	5.19	680572	{}	\N	2020-10-12 16:48:32.98361
83879	GAB	2020-10-08	5.2	5.24	5.195	5.23	974840	{}	\N	2020-10-12 16:48:32.98852
83880	GAB	2020-10-09	5.24	5.285	5.22	5.25	886317	{}	\N	2020-10-12 16:48:32.993447
83881	GAB	2020-10-12	5.32	5.37	5.28	5.3699	1039550	{}	\N	2020-10-12 16:48:32.998864
83882	GFN	2020-09-28	6.44	6.5732	6.37	6.49	20674	{}	\N	2020-10-12 16:48:34.386612
83883	GFN	2020-09-29	6.56	6.56	6.42	6.55	11454	{}	\N	2020-10-12 16:48:34.398416
83884	GFN	2020-09-30	6.58	6.58	6.26	6.33	13349	{}	\N	2020-10-12 16:48:34.406944
83885	GFN	2020-10-01	6.44	6.49	6.34	6.46	24546	{}	\N	2020-10-12 16:48:34.413059
83886	GFN	2020-10-02	6.37	6.6	6.12	6.39	10589	{}	\N	2020-10-12 16:48:34.419116
83887	GFN	2020-10-05	6.49	6.58	6.2	6.53	20630	{}	\N	2020-10-12 16:48:34.424127
83888	GFN	2020-10-06	6.58	6.65	6.3	6.45	14248	{}	\N	2020-10-12 16:48:34.429254
83889	GFN	2020-10-07	6.58	10.15	6.5	7.04	952583	{}	\N	2020-10-12 16:48:34.434506
83890	GFN	2020-10-08	7.35	7.35	6.7	7.08	59770	{}	\N	2020-10-12 16:48:34.439774
83891	GFN	2020-10-09	7.34	7.39	7.1859	7.36	53713	{}	\N	2020-10-12 16:48:34.444959
83892	GFN	2020-10-12	7.477	7.5	7.41	7.48	11787	{}	\N	2020-10-12 16:48:34.450961
83893	GRFS	2020-09-28	16.68	17.16	16.68	16.9	561935	{}	\N	2020-10-12 16:48:35.398795
83894	GRFS	2020-09-29	16.91	17.22	16.67	16.81	1067240	{}	\N	2020-10-12 16:48:35.409974
83895	GRFS	2020-09-30	16.87	17.42	16.87	17.35	1787884	{}	\N	2020-10-12 16:48:35.417718
83896	GRFS	2020-10-01	17.34	17.6	17.07	17.19	2326086	{}	\N	2020-10-12 16:48:35.42418
83897	GRFS	2020-10-02	17.3	17.42	17.07	17.07	1498484	{}	\N	2020-10-12 16:48:35.429866
83898	GRFS	2020-10-05	17.18	17.81	17.18	17.8	2053422	{}	\N	2020-10-12 16:48:35.434877
83899	GRFS	2020-10-06	17.9	18.22	17.83	17.93	1416672	{}	\N	2020-10-12 16:48:35.440154
83900	GRFS	2020-10-07	17.85	17.92	17.65	17.67	506161	{}	\N	2020-10-12 16:48:35.444998
83901	GRFS	2020-10-08	17.67	17.88	17.53	17.75	367299	{}	\N	2020-10-12 16:48:35.449969
83902	GRFS	2020-10-09	18.03	18.69	18.02	18.6	1174420	{}	\N	2020-10-12 16:48:35.454863
83903	GRFS	2020-10-12	18.53	18.75	18.39	18.51	531708	{}	\N	2020-10-12 16:48:35.459558
83904	EEMD	2020-09-28	17.46	17.5084	17.46	17.5084	580	{}	\N	2020-10-12 16:48:38.241303
83905	EEMD	2020-09-29	17.405	17.4642	17.405	17.4642	206	{}	\N	2020-10-12 16:48:38.248432
83906	EEMD	2020-09-30	17.59	17.6036	17.59	17.6036	306	{}	\N	2020-10-12 16:48:38.253572
83907	EEMD	2020-10-01	17.535	17.6266	17.535	17.6266	2529	{}	\N	2020-10-12 16:48:38.25862
83908	EEMD	2020-10-02	17.5025	17.5025	17.4748	17.4748	195	{}	\N	2020-10-12 16:48:38.263003
83909	EEMD	2020-10-05	17.5543	17.5543	17.5543	17.5543	87	{}	\N	2020-10-12 16:48:38.268357
83910	EEMD	2020-10-06	17.6599	17.6599	17.5303	17.5303	607	{}	\N	2020-10-12 16:48:38.273408
83911	EEMD	2020-10-07	17.63	17.63	17.577	17.577	568	{}	\N	2020-10-12 16:48:38.278608
83912	EEMD	2020-10-08	17.72	17.72	17.72	17.72	116	{}	\N	2020-10-12 16:48:38.283812
83913	EEMD	2020-10-09	17.86	17.86	17.86	17.86	9	{}	\N	2020-10-12 16:48:38.288818
83914	EGBN	2020-09-28	26.09	26.9602	26.09	26.56	121142	{}	\N	2020-10-12 16:48:40.389442
83915	EGBN	2020-09-29	26.36	26.36	25.72	26.3	86730	{}	\N	2020-10-12 16:48:40.397039
83916	EGBN	2020-09-30	26.68	28.06	26.68	26.79	204154	{}	\N	2020-10-12 16:48:40.403235
83917	EGBN	2020-10-01	26.52	26.95	26.19	26.95	136885	{}	\N	2020-10-12 16:48:40.409596
83918	EGBN	2020-10-02	26.455	28.01	26.455	27.83	102951	{}	\N	2020-10-12 16:48:40.415223
83919	EGBN	2020-10-05	28.26	28.96	27.7	28.89	166082	{}	\N	2020-10-12 16:48:40.420552
83920	EGBN	2020-10-06	29.01	30.67	29.01	29.52	176863	{}	\N	2020-10-12 16:48:40.4263
83921	EGBN	2020-10-07	29.93	30.29	29.31	29.68	136785	{}	\N	2020-10-12 16:48:40.432011
83922	EGBN	2020-10-08	30	30.6	29.66	30.38	93234	{}	\N	2020-10-12 16:48:40.437182
83923	EGBN	2020-10-09	30.72	30.72	29.84	30.03	72263	{}	\N	2020-10-12 16:48:40.443376
83924	EGBN	2020-10-12	29.8	30.25	29.8	30.02	24563	{}	\N	2020-10-12 16:48:40.448589
83925	FAN	2020-09-28	17.2	17.2	17.12	17.18	103637	{}	\N	2020-10-12 16:48:43.121583
83926	FAN	2020-09-29	17.33	17.4599	17.33	17.44	146041	{}	\N	2020-10-12 16:48:43.130599
83927	FAN	2020-09-30	17.5	17.5718	17.465	17.53	147572	{}	\N	2020-10-12 16:48:43.137571
83928	FAN	2020-10-01	17.77	17.97	17.77	17.96	353643	{}	\N	2020-10-12 16:48:43.144028
83929	FAN	2020-10-02	17.7	17.9615	17.631	17.95	162687	{}	\N	2020-10-12 16:48:43.149861
83930	FAN	2020-10-05	18.08	18.295	18.08	18.295	267716	{}	\N	2020-10-12 16:48:43.155035
83931	FAN	2020-10-06	18.4	18.45	18.22	18.27	282088	{}	\N	2020-10-12 16:48:43.159436
83932	FAN	2020-10-07	18.6	18.8	18.6	18.8	401582	{}	\N	2020-10-12 16:48:43.163799
83933	FAN	2020-10-08	19	19.03	18.91	18.975	415990	{}	\N	2020-10-12 16:48:43.168623
83934	FAN	2020-10-09	18.965	19.0199	18.84	18.9891	247577	{}	\N	2020-10-12 16:48:43.175323
83935	FAN	2020-10-12	19.08	19.1817	19.03	19.1725	82149	{}	\N	2020-10-12 16:48:43.17969
83936	FCA	2020-09-28	25.685	25.79	25.55	25.6	1423	{}	\N	2020-10-12 16:48:45.666315
83937	FCA	2020-09-29	25.62	25.685	25.58	25.685	772	{}	\N	2020-10-12 16:48:45.674909
83938	FCA	2020-09-30	25.91	26.0438	25.91	26.0438	815	{}	\N	2020-10-12 16:48:45.682145
83939	FCA	2020-10-01	26.4412	26.4412	26.43	26.43	400	{}	\N	2020-10-12 16:48:45.687335
83940	FCA	2020-10-05	25.9021	25.9021	25.7779	25.7779	685	{}	\N	2020-10-12 16:48:45.692946
83941	FCA	2020-10-06	26.45	26.81	26.45	26.55	1007	{}	\N	2020-10-12 16:48:45.697837
83942	FCA	2020-10-08	26.83	26.92	26.82	26.891	1598	{}	\N	2020-10-12 16:48:45.702464
83943	FCA	2020-10-09	27.03	27.1434	27.03	27.1434	4413	{}	\N	2020-10-12 16:48:45.707416
83944	FLL	2020-09-28	2	2.04	1.82	1.89	121131	{}	\N	2020-10-12 16:48:46.875027
83945	FLL	2020-09-29	1.9	1.9583	1.76	1.89	117503	{}	\N	2020-10-12 16:48:46.886552
83946	FLL	2020-09-30	1.86	1.96	1.83	1.94	113171	{}	\N	2020-10-12 16:48:46.894743
83947	FLL	2020-10-01	1.941	1.99	1.91	1.96	65769	{}	\N	2020-10-12 16:48:46.900691
83948	FLL	2020-10-02	1.95	2.01	1.9	2	63690	{}	\N	2020-10-12 16:48:46.906612
83949	FLL	2020-10-05	2	2.3299	1.99	2.25	395232	{}	\N	2020-10-12 16:48:46.912121
83950	FLL	2020-10-06	2.29	2.54	2.28	2.32	281784	{}	\N	2020-10-12 16:48:46.920898
83951	FLL	2020-10-07	2.4249	2.5	2.32	2.49	218885	{}	\N	2020-10-12 16:48:46.925541
83952	FLL	2020-10-08	2.49	2.49	2.35	2.43	145359	{}	\N	2020-10-12 16:48:46.930308
83953	FLL	2020-10-09	2.43	2.455	2.11	2.26	367061	{}	\N	2020-10-12 16:48:46.934806
83954	FLL	2020-10-12	2.32	2.32	2.214	2.26	38582	{}	\N	2020-10-12 16:48:46.93957
83955	FLOT	2020-09-28	50.66	50.7	50.66	50.68	338379	{}	\N	2020-10-12 16:48:47.980007
83956	FLOT	2020-09-29	50.7	50.71	50.69	50.71	363754	{}	\N	2020-10-12 16:48:47.990074
83957	FLOT	2020-09-30	50.68	50.72	50.67	50.72	336800	{}	\N	2020-10-12 16:48:47.998611
83958	FLOT	2020-10-01	50.68	50.71	50.65	50.66	483530	{}	\N	2020-10-12 16:48:48.005075
83959	FLOT	2020-10-02	50.68	50.69	50.665	50.69	525457	{}	\N	2020-10-12 16:48:48.010653
83960	FLOT	2020-10-05	50.68	50.73	50.66	50.73	646764	{}	\N	2020-10-12 16:48:48.01619
83961	FLOT	2020-10-06	50.72	50.72	50.68	50.71	399439	{}	\N	2020-10-12 16:48:48.02179
83962	FLOT	2020-10-07	50.69	50.71	50.68	50.71	671954	{}	\N	2020-10-12 16:48:48.027492
83963	FLOT	2020-10-08	50.69	50.71	50.66	50.71	949752	{}	\N	2020-10-12 16:48:48.032791
83964	FLOT	2020-10-09	50.71	50.72	50.67	50.69	657569	{}	\N	2020-10-12 16:48:48.038151
83965	FLOT	2020-10-12	50.68	50.69	50.67	50.685	154825	{}	\N	2020-10-12 16:48:48.043543
83966	FQAL	2020-09-28	38.84	38.955	38.56	38.87	22368	{}	\N	2020-10-12 16:48:49.169296
83967	FQAL	2020-09-29	38.97	38.97	38.71	38.75	15598	{}	\N	2020-10-12 16:48:49.178456
83968	FQAL	2020-09-30	38.82	39.32	38.79	39.06	179595	{}	\N	2020-10-12 16:48:49.185951
83969	FQAL	2020-10-01	39.34	39.4083	39.1	39.21	16910	{}	\N	2020-10-12 16:48:49.192302
83970	FQAL	2020-10-02	38.53	39.125	38.33	38.85	20341	{}	\N	2020-10-12 16:48:49.198221
83971	FQAL	2020-10-05	39.2	39.52	39.2	39.52	13528	{}	\N	2020-10-12 16:48:49.203439
83972	FQAL	2020-10-06	39.6389	39.8054	39.02	39.05	11917	{}	\N	2020-10-12 16:48:49.208802
83973	FQAL	2020-10-07	39.43	39.7	39.4	39.652	5294	{}	\N	2020-10-12 16:48:49.214384
83974	FQAL	2020-10-08	39.74	39.96	39.74	39.9	22119	{}	\N	2020-10-12 16:48:49.219766
83975	FQAL	2020-10-09	40.05	40.37	40.05	40.26	198433	{}	\N	2020-10-12 16:48:49.225064
83976	FQAL	2020-10-12	40.52	40.98	40.52	40.98	6504	{}	\N	2020-10-12 16:48:49.230174
83977	FTLS	2020-09-28	42.358	42.44	42.28	42.41	14487	{}	\N	2020-10-12 16:48:50.243615
83978	FTLS	2020-09-29	42.405	42.49	42.29	42.38	519713	{}	\N	2020-10-12 16:48:50.256845
83979	FTLS	2020-09-30	42.43	42.79	42.4246	42.565	36285	{}	\N	2020-10-12 16:48:50.264438
83980	FTLS	2020-10-01	42.72	42.94	42.6634	42.8546	16211	{}	\N	2020-10-12 16:48:50.27077
83981	FTLS	2020-10-02	42.53	42.8099	42.47	42.68	48194	{}	\N	2020-10-12 16:48:50.276297
83982	FTLS	2020-10-05	42.81	43.31	42.81	43.31	24589	{}	\N	2020-10-12 16:48:50.280809
83983	FTLS	2020-10-06	43.27	43.38	42.801	42.86	69746	{}	\N	2020-10-12 16:48:50.285185
83984	FTLS	2020-10-07	43.0283	43.309	43.0283	43.309	19266	{}	\N	2020-10-12 16:48:50.289367
83985	FTLS	2020-10-08	43.33	43.46	43.23	43.35	35201	{}	\N	2020-10-12 16:48:50.293352
83986	FTLS	2020-10-09	43.33	43.76	43.33	43.67	35104	{}	\N	2020-10-12 16:48:50.296983
83987	FTLS	2020-10-12	44	44.299	43.9204	44.299	79658	{}	\N	2020-10-12 16:48:50.301002
83988	FULT	2020-09-28	9.29	9.595	9.21	9.46	1212923	{}	\N	2020-10-12 16:48:51.452457
83989	FULT	2020-09-29	9.43	9.43	9.095	9.23	962675	{}	\N	2020-10-12 16:48:51.464505
83990	FULT	2020-09-30	9.28	9.48	9.215	9.33	1281011	{}	\N	2020-10-12 16:48:51.473188
83991	FULT	2020-10-01	9.4	9.43	9.15	9.3	864823	{}	\N	2020-10-12 16:48:51.479557
83992	FULT	2020-10-02	9.16	9.72	9.16	9.68	1009064	{}	\N	2020-10-12 16:48:51.485835
83993	FULT	2020-10-05	9.74	10.07	9.74	10.03	1323731	{}	\N	2020-10-12 16:48:51.491483
83994	FULT	2020-10-06	10.045	10.445	9.995	10.03	851086	{}	\N	2020-10-12 16:48:51.496789
83995	FULT	2020-10-07	10.16	10.38	10.15	10.26	866237	{}	\N	2020-10-12 16:48:51.501844
83996	FULT	2020-10-08	10.39	10.535	10.27	10.5	665193	{}	\N	2020-10-12 16:48:51.507297
83997	FULT	2020-10-09	10.53	10.55	10.28	10.37	681058	{}	\N	2020-10-12 16:48:51.513051
83998	FULT	2020-10-12	10.32	10.52	10.29	10.445	229096	{}	\N	2020-10-12 16:48:51.518852
83999	FV	2020-09-28	33.95	34.23	33.8701	34.145	174594	{}	\N	2020-10-12 16:48:53.542577
84000	FV	2020-09-29	34.08	34.3299	34.03	34.04	70162	{}	\N	2020-10-12 16:48:53.554216
84001	FV	2020-09-30	34.03	34.466	33.97	34.12	365655	{}	\N	2020-10-12 16:48:53.562281
84002	FV	2020-10-01	34.35	34.76	34.35	34.7	252836	{}	\N	2020-10-12 16:48:53.568617
84003	FV	2020-10-02	34.01	34.575	34.01	34.35	77278	{}	\N	2020-10-12 16:48:53.575487
84004	FV	2020-10-05	34.52	35.0274	34.52	35.01	69453	{}	\N	2020-10-12 16:48:53.581152
84005	FV	2020-10-06	35.119	35.42	34.57	34.659	90032	{}	\N	2020-10-12 16:48:53.586543
84006	FV	2020-10-07	34.88	35.45	34.88	35.34	69824	{}	\N	2020-10-12 16:48:53.591839
84007	FV	2020-10-08	35.57	35.72	35.4254	35.6397	113778	{}	\N	2020-10-12 16:48:53.597393
84008	FV	2020-10-09	35.71	36.1079	35.71	35.98	105221	{}	\N	2020-10-12 16:48:53.602669
84009	FV	2020-10-12	36.45	36.57	36.22	36.57	39762	{}	\N	2020-10-12 16:48:53.607955
84010	GDP	2020-09-28	7.64	8.05	7.55	7.92	17214	{}	\N	2020-10-12 16:48:54.798336
84011	GDP	2020-09-29	7.8	7.94	7.8	7.94	7805	{}	\N	2020-10-12 16:48:54.80698
84012	GDP	2020-09-30	7.8891	8	7.49	7.69	9336	{}	\N	2020-10-12 16:48:54.812888
84013	GDP	2020-10-01	7.69	7.69	7.38	7.45	8546	{}	\N	2020-10-12 16:48:54.819578
84014	GDP	2020-10-02	8	8.7	7.63	8.34	79974	{}	\N	2020-10-12 16:48:54.824769
84015	GDP	2020-10-05	8.5	9.2603	8.5	8.61	59514	{}	\N	2020-10-12 16:48:54.830227
84016	GDP	2020-10-06	8.99	9.54	7.71	8.06	59297	{}	\N	2020-10-12 16:48:54.835631
84017	GDP	2020-10-07	8.1	9.07	8.1	8.84	33526	{}	\N	2020-10-12 16:48:54.840768
84018	GDP	2020-10-08	9.02	9.18	8.42	8.98	12645	{}	\N	2020-10-12 16:48:54.84618
84019	GDP	2020-10-09	8.7824	9.11	8.07	9.04	43596	{}	\N	2020-10-12 16:48:54.851613
84020	GDP	2020-10-12	9.34	9.7301	9.13	9.43	12876	{}	\N	2020-10-12 16:48:54.856751
84021	EMCB	2020-09-28	73.66	73.66	73.19	73.19	724	{}	\N	2020-10-12 16:48:56.383035
84022	EMCB	2020-09-29	74.16	74.16	74.16	74.16	625	{}	\N	2020-10-12 16:48:56.394642
84023	EMCB	2020-09-30	73.32	74.16	72.7942	73.057	5504	{}	\N	2020-10-12 16:48:56.401216
84024	EMCB	2020-10-01	74.1799	74.1799	73.6772	73.6772	522	{}	\N	2020-10-12 16:48:56.407574
84025	EMCB	2020-10-02	72.92	74.142	72.92	72.99	1587	{}	\N	2020-10-12 16:48:56.413522
84026	EMCB	2020-10-05	73.0761	73.375	73.0761	73.375	422	{}	\N	2020-10-12 16:48:56.418945
84027	EMCB	2020-10-06	74.38	74.51	74.05	74.47	2911	{}	\N	2020-10-12 16:48:56.424203
84028	EMCB	2020-10-07	73.735	73.735	73.735	73.735	660	{}	\N	2020-10-12 16:48:56.429235
84029	EMCB	2020-10-08	74	74.86	74	74.75	1933	{}	\N	2020-10-12 16:48:56.434047
84030	EMCB	2020-10-12	73.95	74.52	73.5544	74.52	690	{}	\N	2020-10-12 16:48:56.439298
84031	EPHE	2020-09-28	26.51	26.57	26.37	26.5	75394	{}	\N	2020-10-12 16:48:58.421529
84032	EPHE	2020-09-29	26.49	26.57	26.36	26.4	36302	{}	\N	2020-10-12 16:48:58.434038
84033	EPHE	2020-09-30	26.56	26.9	26.56	26.58	59801	{}	\N	2020-10-12 16:48:58.442726
84034	EPHE	2020-10-01	26.79	26.95	26.64	26.86	132159	{}	\N	2020-10-12 16:48:58.449347
84035	EPHE	2020-10-02	26.99	27.2	26.86	27.03	51238	{}	\N	2020-10-12 16:48:58.455461
84036	EPHE	2020-10-05	26.96	27.18	26.8	26.85	130762	{}	\N	2020-10-12 16:48:58.460595
84037	EPHE	2020-10-06	26.66	26.79	26.56	26.72	62690	{}	\N	2020-10-12 16:48:58.465711
84038	EPHE	2020-10-07	26.56	26.73	26.46	26.5	49796	{}	\N	2020-10-12 16:48:58.470887
84039	EPHE	2020-10-08	26.66	26.8	26.56	26.74	100794	{}	\N	2020-10-12 16:48:58.476326
84040	EPHE	2020-10-09	26.5992	26.975	26.5992	26.75	46940	{}	\N	2020-10-12 16:48:58.48146
84041	EPHE	2020-10-12	26.83	26.95	26.67	26.8	54672	{}	\N	2020-10-12 16:48:58.4863
84042	ESBK	2020-09-28	10.585	10.75	10.548	10.75	4224	{}	\N	2020-10-12 16:49:00.285743
84043	ESBK	2020-09-29	10.75	11.1435	10.75	11.1179	3649	{}	\N	2020-10-12 16:49:00.294324
84044	ESBK	2020-09-30	11.02	11.02	11.02	11.02	764	{}	\N	2020-10-12 16:49:00.300334
84045	ESBK	2020-10-01	11.4397	11.4397	11.4397	11.4397	1220	{}	\N	2020-10-12 16:49:00.30566
84046	ESBK	2020-10-02	10.8693	10.8693	10.8693	10.8693	516	{}	\N	2020-10-12 16:49:00.311084
84047	ESBK	2020-10-05	11.05	11.17	10.96	10.96	710	{}	\N	2020-10-12 16:49:00.315641
84048	ESBK	2020-10-06	10.95	10.96	10.645	10.69	4009	{}	\N	2020-10-12 16:49:00.320191
84049	ESBK	2020-10-07	10.69	10.7	10.512	10.7	5720	{}	\N	2020-10-12 16:49:00.324655
84050	ESBK	2020-10-08	10.71	10.72	10.71	10.71	745	{}	\N	2020-10-12 16:49:00.329323
84051	ESBK	2020-10-09	10.5101	10.84	10.5101	10.83	2738	{}	\N	2020-10-12 16:49:00.333896
84052	EWX	2020-09-28	44.12	44.21	43.89	43.98	37678	{}	\N	2020-10-12 16:49:01.289284
84053	EWX	2020-09-29	44.02	44.2255	43.94	44.12	19777	{}	\N	2020-10-12 16:49:01.301386
84054	EWX	2020-09-30	44.18	44.57	44.18	44.41	46287	{}	\N	2020-10-12 16:49:01.308943
84055	EWX	2020-10-01	44.71	44.955	44.615	44.92	23501	{}	\N	2020-10-12 16:49:01.315875
84056	EWX	2020-10-02	44.29	44.76	44.29	44.49	21205	{}	\N	2020-10-12 16:49:01.322449
84057	EWX	2020-10-05	44.96	45.15	44.88	45.02	342988	{}	\N	2020-10-12 16:49:01.327615
84058	EWX	2020-10-06	45.34	45.42	45.0224	45.08	16854	{}	\N	2020-10-12 16:49:01.333278
84059	EWX	2020-10-07	45.29	45.4728	45.2	45.31	12193	{}	\N	2020-10-12 16:49:01.33857
84060	EWX	2020-10-08	45.41	45.5999	45.375	45.57	26418	{}	\N	2020-10-12 16:49:01.343875
84061	EWX	2020-10-09	45.65	46.17	45.56	45.98	189670	{}	\N	2020-10-12 16:49:01.349473
84062	EWX	2020-10-12	45.88	45.9	45.76	45.9	10254	{}	\N	2020-10-12 16:49:01.354639
84063	FBK	2020-09-28	24.63	25.5	24.58	25.13	118245	{}	\N	2020-10-12 16:49:03.103933
84064	FBK	2020-09-29	25.37	25.43	24.6	25.37	268055	{}	\N	2020-10-12 16:49:03.115757
84065	FBK	2020-09-30	25.65	25.92	25.03	25.12	246041	{}	\N	2020-10-12 16:49:03.123965
84066	FBK	2020-10-01	25.09	25.65	24.83	25.64	252367	{}	\N	2020-10-12 16:49:03.13013
84067	FBK	2020-10-02	25.07	26.63	24.99	26.46	286042	{}	\N	2020-10-12 16:49:03.137221
84068	FBK	2020-10-05	26.81	27.71	26.81	27.59	205906	{}	\N	2020-10-12 16:49:03.142568
84069	FBK	2020-10-06	28.04	28.26	26.83	26.96	365554	{}	\N	2020-10-12 16:49:03.14775
84070	FBK	2020-10-07	27.36	27.82	27.06	27.72	361973	{}	\N	2020-10-12 16:49:03.153202
84071	FBK	2020-10-08	28.03	28.49	27.69	28.32	168748	{}	\N	2020-10-12 16:49:03.158613
84072	FBK	2020-10-09	28.67	28.73	28.243	28.27	162915	{}	\N	2020-10-12 16:49:03.163757
84073	FBK	2020-10-12	28.12	28.63	28.11	28.46	49078	{}	\N	2020-10-12 16:49:03.16988
84074	FBT	2020-09-28	156.65	157.12	155.2	156.32	53978	{}	\N	2020-10-12 16:49:04.175751
84075	FBT	2020-09-29	156.47	157.55	155.76	156.61	68049	{}	\N	2020-10-12 16:49:04.188398
84076	FBT	2020-09-30	156.5	158.481	156.0901	157.26	83471	{}	\N	2020-10-12 16:49:04.196502
84077	FBT	2020-10-01	157.71	158.75	157.2026	158.34	69903	{}	\N	2020-10-12 16:49:04.202815
84078	FBT	2020-10-02	156.37	158.196	154.37	154.72	125716	{}	\N	2020-10-12 16:49:04.208621
84079	FBT	2020-10-05	156.54	160.87	156.54	160.56	152944	{}	\N	2020-10-12 16:49:04.213701
84080	FBT	2020-10-06	160.94	161.495	158.44	158.6	65200	{}	\N	2020-10-12 16:49:04.219015
84081	FBT	2020-10-07	159.54	163.1699	159.54	162.68	99557	{}	\N	2020-10-12 16:49:04.224183
84082	FBT	2020-10-08	163.52	163.57	162.05	163.03	47345	{}	\N	2020-10-12 16:49:04.229046
84083	FBT	2020-10-09	163.63	164	162.74	163.69	54773	{}	\N	2020-10-12 16:49:04.234187
84084	FBT	2020-10-12	165.09	165.4	163.47	164.855	23325	{}	\N	2020-10-12 16:49:04.239471
84085	FDRR	2020-09-28	32.31	32.49	32.25	32.385	54276	{}	\N	2020-10-12 16:49:07.987072
84086	FDRR	2020-09-29	32.39	32.39	32.1	32.14	14960	{}	\N	2020-10-12 16:49:07.999889
84087	FDRR	2020-09-30	32.25	32.7	32.25	32.45	69304	{}	\N	2020-10-12 16:49:08.008373
84088	FDRR	2020-10-01	32.62	32.65	32.29	32.46	21196	{}	\N	2020-10-12 16:49:08.015216
84089	FDRR	2020-10-02	32.03	32.485	31.844	32.32	81698	{}	\N	2020-10-12 16:49:08.020925
84090	FDRR	2020-10-05	32.51	32.9	32.51	32.89	46462	{}	\N	2020-10-12 16:49:08.026505
84091	FDRR	2020-10-06	32.95	33.18	32.5002	32.55	30013	{}	\N	2020-10-12 16:49:08.031408
84092	FDRR	2020-10-07	32.79	33.1484	32.67	33.09	79225	{}	\N	2020-10-12 16:49:08.036263
84093	FDRR	2020-10-08	33.25	33.3299	33.22	33.32	14052	{}	\N	2020-10-12 16:49:08.040942
84094	FDRR	2020-10-09	33.5	33.5625	33.391	33.425	22641	{}	\N	2020-10-12 16:49:08.045809
84095	FDRR	2020-10-12	33.61	33.928	33.61	33.928	11297	{}	\N	2020-10-12 16:49:08.050405
84096	FKO	2020-09-28	19.49	19.49	19.49	19.49	405	{}	\N	2020-10-12 16:49:09.284352
84097	FKO	2020-09-29	19.79	19.79	19.79	19.79	202	{}	\N	2020-10-12 16:49:09.292863
84098	FKO	2020-09-30	19.79	19.97	19.79	19.95	4867	{}	\N	2020-10-12 16:49:09.299031
84099	FKO	2020-10-02	19.45	19.74	19.45	19.74	1401	{}	\N	2020-10-12 16:49:09.305687
84100	FKO	2020-10-05	19.97	20.24	19.97	20.24	1777	{}	\N	2020-10-12 16:49:09.311875
84101	FKO	2020-10-06	20.55	20.57	20.4	20.4	12230	{}	\N	2020-10-12 16:49:09.316272
84102	FKO	2020-10-07	20.51	20.74	20.51	20.74	1355	{}	\N	2020-10-12 16:49:09.320959
84103	FKO	2020-10-08	20.95	20.97	20.95	20.97	214	{}	\N	2020-10-12 16:49:09.325791
84104	FKO	2020-10-12	20.91	20.94	20.91	20.94	1635	{}	\N	2020-10-12 16:49:09.330943
84105	FTA	2020-09-28	46.46	47.06	46.45	46.82	30171	{}	\N	2020-10-12 16:49:10.710568
84106	FTA	2020-09-29	46.72	46.7288	46.07	46.352	472764	{}	\N	2020-10-12 16:49:10.723249
84107	FTA	2020-09-30	46.55	47.17	46.4	46.67	106079	{}	\N	2020-10-12 16:49:10.730923
84108	FTA	2020-10-01	46.73	46.99	46.3371	46.66	211108	{}	\N	2020-10-12 16:49:10.736231
84109	FTA	2020-10-02	45.8	47.41	45.8	47.21	115400	{}	\N	2020-10-12 16:49:10.740678
84110	FTA	2020-10-05	47.54	48.2	47.54	48.12	85063	{}	\N	2020-10-12 16:49:10.745431
84111	FTA	2020-10-06	48.39	48.919	47.54	47.69	103355	{}	\N	2020-10-12 16:49:10.749432
84112	FTA	2020-10-07	48.14	48.68	48.14	48.59	104069	{}	\N	2020-10-12 16:49:10.753271
84113	FTA	2020-10-08	48.78	49.39	48.78	49.37	78007	{}	\N	2020-10-12 16:49:10.756857
84114	FTA	2020-10-09	49.65	49.65	49.2	49.27	124204	{}	\N	2020-10-12 16:49:10.765673
84115	FTA	2020-10-12	49.33	49.63	49.25	49.61	37618	{}	\N	2020-10-12 16:49:10.771522
84116	FUND	2020-09-28	5.9297	6	5.9297	5.9666	67143	{}	\N	2020-10-12 16:49:11.942973
84117	FUND	2020-09-29	6	6	5.93	5.94	22540	{}	\N	2020-10-12 16:49:11.950791
84118	FUND	2020-09-30	5.95	6.02	5.91	5.95	40597	{}	\N	2020-10-12 16:49:11.95703
84119	FUND	2020-10-01	6	6	5.88	5.94	37359	{}	\N	2020-10-12 16:49:11.963181
84120	FUND	2020-10-02	5.89	5.98	5.89	5.98	29501	{}	\N	2020-10-12 16:49:11.967846
84121	FUND	2020-10-05	5.98	6.09	5.94	5.99	373149	{}	\N	2020-10-12 16:49:11.972602
84122	FUND	2020-10-06	6	6.08	5.9765	5.99	237757	{}	\N	2020-10-12 16:49:11.977068
84123	FUND	2020-10-07	6.01	6.09	5.99	6.09	60428	{}	\N	2020-10-12 16:49:11.981788
84124	FUND	2020-10-08	6.04	6.18	6.04	6.16	68082	{}	\N	2020-10-12 16:49:11.98664
84125	FUND	2020-10-09	6.22	6.25	6.19	6.22	19239	{}	\N	2020-10-12 16:49:11.991388
84126	FUND	2020-10-12	6.27	6.3	6.248	6.3	9654	{}	\N	2020-10-12 16:49:11.996535
84127	FVE	2020-09-28	5.2	5.24	5.09	5.16	116916	{}	\N	2020-10-12 16:49:13.100941
84128	FVE	2020-09-29	5.1899	5.2	5.06	5.09	26783	{}	\N	2020-10-12 16:49:13.112251
84129	FVE	2020-09-30	5.09	5.17	5	5.075	44946	{}	\N	2020-10-12 16:49:13.119204
84130	FVE	2020-10-01	5.0764	5.0764	4.84	4.9	47711	{}	\N	2020-10-12 16:49:13.125023
84131	FVE	2020-10-02	4.89	5	4.75	4.96	37489	{}	\N	2020-10-12 16:49:13.131374
84132	FVE	2020-10-05	4.97	5.035	4.89	4.96	84480	{}	\N	2020-10-12 16:49:13.136325
84133	FVE	2020-10-06	5	5.05	4.89	4.89	70395	{}	\N	2020-10-12 16:49:13.141139
84134	FVE	2020-10-07	4.95	5.01	4.92	4.99	52077	{}	\N	2020-10-12 16:49:13.145921
84135	FVE	2020-10-08	5.03	5.1	4.9	5	49560	{}	\N	2020-10-12 16:49:13.150593
84136	FVE	2020-10-09	5.0526	5.095	4.99	5	60883	{}	\N	2020-10-12 16:49:13.155197
84137	FVE	2020-10-12	5.01	5.09	5.01	5.0701	6629	{}	\N	2020-10-12 16:49:13.160044
84138	GIII	2020-09-28	14.26	14.565	13.89	14.15	948696	{}	\N	2020-10-12 16:49:14.609434
84139	GIII	2020-09-29	14.15	14.2548	13.04	13.1	1115880	{}	\N	2020-10-12 16:49:14.620689
84140	GIII	2020-09-30	13.26	13.785	13.01	13.115	936923	{}	\N	2020-10-12 16:49:14.628621
84141	GIII	2020-10-01	13.19	13.93	13.14	13.93	741684	{}	\N	2020-10-12 16:49:14.634712
84142	GIII	2020-10-02	13.32	14.49	13.1401	14.425	689235	{}	\N	2020-10-12 16:49:14.64075
84143	GIII	2020-10-05	14.8	15.1	14.35	14.78	618966	{}	\N	2020-10-12 16:49:14.645651
84144	GIII	2020-10-06	15.03	15.1	14.17	14.28	554562	{}	\N	2020-10-12 16:49:14.653086
84145	GIII	2020-10-07	14.7	15.24	14.65	15.01	1039023	{}	\N	2020-10-12 16:49:14.65812
84146	GIII	2020-10-08	15.37	15.55	14.74	15.3	843776	{}	\N	2020-10-12 16:49:14.662963
84147	GIII	2020-10-09	14.92	15.7	14.92	15.4	707878	{}	\N	2020-10-12 16:49:14.66749
84148	GIII	2020-10-12	15.65	15.96	15.18	15.26	264829	{}	\N	2020-10-12 16:49:14.673722
84149	GPRK	2020-09-28	7.87	8.02	7.71	7.74	50037	{}	\N	2020-10-12 16:49:15.693265
84150	GPRK	2020-09-29	7.69	7.81	7.26	7.37	141468	{}	\N	2020-10-12 16:49:15.704551
84151	GPRK	2020-09-30	7.48	7.65	7.35	7.36	120849	{}	\N	2020-10-12 16:49:15.712903
84152	GPRK	2020-10-01	7.27	7.59	7.24	7.41	189496	{}	\N	2020-10-12 16:49:15.719339
84153	GPRK	2020-10-02	7.27	7.54	7.204	7.5	103546	{}	\N	2020-10-12 16:49:15.725438
84154	GPRK	2020-10-05	7.5	7.89	7.4	7.85	175648	{}	\N	2020-10-12 16:49:15.731355
84155	GPRK	2020-10-06	7.86	8.07	7.74	8.06	223179	{}	\N	2020-10-12 16:49:15.736642
84156	GPRK	2020-10-07	8.06	8.06	7.77	8.05	133278	{}	\N	2020-10-12 16:49:15.741559
84157	GPRK	2020-10-08	8.1	8.44	8.02	8.4	111515	{}	\N	2020-10-12 16:49:15.746888
84158	GPRK	2020-10-09	8.44	8.49	8.171	8.34	89776	{}	\N	2020-10-12 16:49:15.751534
84159	GPRK	2020-10-12	8.26	8.26	7.97	8.21	17362	{}	\N	2020-10-12 16:49:15.756308
84160	FCAL	2020-09-28	53.1	53.1	52.8723	52.96	7260	{}	\N	2020-10-12 16:49:18.414667
84161	FCAL	2020-09-29	52.922	52.9777	52.895	52.9777	5330	{}	\N	2020-10-12 16:49:18.429608
84162	FCAL	2020-09-30	52.93	52.93	52.77	52.86	1663	{}	\N	2020-10-12 16:49:18.438782
84163	FCAL	2020-10-01	52.85	52.8985	52.775	52.8189	4462	{}	\N	2020-10-12 16:49:18.445769
84164	FCAL	2020-10-02	52.8366	52.8374	52.75	52.8374	1374	{}	\N	2020-10-12 16:49:18.451797
84165	FCAL	2020-10-05	52.8304	52.8304	52.72	52.8	4280	{}	\N	2020-10-12 16:49:18.45696
84166	FCAL	2020-10-06	52.85	52.85	52.7	52.805	10269	{}	\N	2020-10-12 16:49:18.462418
84167	FCAL	2020-10-07	52.79	52.79	52.6442	52.73	3307	{}	\N	2020-10-12 16:49:18.468107
84168	FCAL	2020-10-08	52.685	52.74	52.675	52.68	1716	{}	\N	2020-10-12 16:49:18.473234
84169	FCAL	2020-10-09	52.76	52.76	52.6804	52.757	15799	{}	\N	2020-10-12 16:49:18.478708
84170	FCAL	2020-10-12	52.7885	52.7885	52.715	52.715	1435	{}	\N	2020-10-12 16:49:18.484365
84171	FPEI	2020-09-28	19.26	19.37	19.2501	19.37	83442	{}	\N	2020-10-12 16:49:19.454828
84172	FPEI	2020-09-29	19.35	19.35	19.3101	19.325	55117	{}	\N	2020-10-12 16:49:19.465263
84173	FPEI	2020-09-30	19.36	19.42	19.3	19.38	154362	{}	\N	2020-10-12 16:49:19.472853
84174	FPEI	2020-10-01	19.45	19.47	19.35	19.44	55999	{}	\N	2020-10-12 16:49:19.478279
84175	FPEI	2020-10-02	19.33	19.42	19.33	19.36	97509	{}	\N	2020-10-12 16:49:19.483493
84176	FPEI	2020-10-05	19.36	19.47	19.36	19.45	84825	{}	\N	2020-10-12 16:49:19.488613
84177	FPEI	2020-10-06	19.455	19.5	19.4	19.4	98905	{}	\N	2020-10-12 16:49:19.493247
84178	FPEI	2020-10-07	19.41	19.47	19.41	19.47	51421	{}	\N	2020-10-12 16:49:19.497845
84179	FPEI	2020-10-08	19.5068	19.56	19.4601	19.5	45198	{}	\N	2020-10-12 16:49:19.503038
84180	FPEI	2020-10-09	19.54	19.54	19.4701	19.52	28379	{}	\N	2020-10-12 16:49:19.507933
84181	FPEI	2020-10-12	19.48	19.54	19.48	19.54	6651	{}	\N	2020-10-12 16:49:19.512625
84182	GIGB	2020-09-28	55.0154	55.26	55.0154	55.26	60360	{}	\N	2020-10-12 16:49:22.070573
84183	GIGB	2020-09-29	55.245	55.3035	55.2294	55.25	164040	{}	\N	2020-10-12 16:49:22.083416
84184	GIGB	2020-09-30	55.18	55.215	55.06	55.16	48344	{}	\N	2020-10-12 16:49:22.09198
84185	GIGB	2020-10-01	54.92	55.229	54.92	55.22	51194	{}	\N	2020-10-12 16:49:22.098863
84186	GIGB	2020-10-02	55.21	55.21	54.98	55.2	126770	{}	\N	2020-10-12 16:49:22.104909
84187	GIGB	2020-10-05	55.11	55.189	55	55.09	36648	{}	\N	2020-10-12 16:49:22.109917
84188	GIGB	2020-10-06	55.06	55.2798	55.04	55.11	36902	{}	\N	2020-10-12 16:49:22.114798
84189	GIGB	2020-10-07	55.05	55.2	54.954	55.04	21270	{}	\N	2020-10-12 16:49:22.119976
84190	GIGB	2020-10-08	55.09	55.23	55.04	55.1428	67860	{}	\N	2020-10-12 16:49:22.125153
84191	GIGB	2020-10-09	55.1	55.28	55.1	55.28	19011	{}	\N	2020-10-12 16:49:22.130107
84192	GIGB	2020-10-12	55.27	55.47	55.27	55.42	13010	{}	\N	2020-10-12 16:49:22.135007
84193	GNTX	2020-09-28	25.68	26.01	25.58	25.71	937252	{}	\N	2020-10-12 16:49:23.296169
84194	GNTX	2020-09-29	25.775	25.845	25.6	25.62	1499626	{}	\N	2020-10-12 16:49:23.309203
84195	GNTX	2020-09-30	25.85	26.03	25.655	25.75	1870122	{}	\N	2020-10-12 16:49:23.317472
84196	GNTX	2020-10-01	25.83	26.15	25.78	26.04	1192810	{}	\N	2020-10-12 16:49:23.324043
84197	GNTX	2020-10-02	25.62	26.31	25.62	26.01	846771	{}	\N	2020-10-12 16:49:23.330833
84198	GNTX	2020-10-05	26.36	26.6482	26.28	26.56	1620915	{}	\N	2020-10-12 16:49:23.336658
84199	GNTX	2020-10-06	26.68	27.215	26.37	26.59	1562038	{}	\N	2020-10-12 16:49:23.342595
84200	GNTX	2020-10-07	26.89	27.465	26.71	27.18	1733914	{}	\N	2020-10-12 16:49:23.34837
84201	GNTX	2020-10-08	27.1051	27.4	27.055	27.23	1142834	{}	\N	2020-10-12 16:49:23.354102
84202	GNTX	2020-10-09	27.49	27.76	27.345	27.47	1037893	{}	\N	2020-10-12 16:49:23.36023
84203	GNTX	2020-10-12	27.65	27.74	27.53	27.67	305832	{}	\N	2020-10-12 16:49:23.367023
84204	GSBD	2020-09-28	15.8	16.4	15.8	16.04	373737	{}	\N	2020-10-12 16:49:24.864595
84205	GSBD	2020-09-29	15.5	15.6638	15.16	15.33	339311	{}	\N	2020-10-12 16:49:24.875554
84206	GSBD	2020-09-30	15.24	15.45	15.04	15.04	292767	{}	\N	2020-10-12 16:49:24.883179
84207	GSBD	2020-10-01	15.1	15.39	15.1	15.37	404800	{}	\N	2020-10-12 16:49:24.889173
84208	GSBD	2020-10-02	15.11	15.37	15.11	15.29	161947	{}	\N	2020-10-12 16:49:24.895371
84209	GSBD	2020-10-05	15.33	15.48	15.29	15.3	179458	{}	\N	2020-10-12 16:49:24.90062
84210	GSBD	2020-10-06	15.44	15.49	15.07	15.17	290923	{}	\N	2020-10-12 16:49:24.908484
84211	GSBD	2020-10-07	15.21	15.3169	15.14	15.25	367347	{}	\N	2020-10-12 16:49:24.914656
84212	GSBD	2020-10-08	15.25	15.42	15.24	15.4	227076	{}	\N	2020-10-12 16:49:24.921967
84213	GSBD	2020-10-09	15.46	15.46	15.22	15.26	179882	{}	\N	2020-10-12 16:49:24.928193
84214	GSBD	2020-10-12	15.36	15.36	15.09	15.15	167599	{}	\N	2020-10-12 16:49:24.934245
84215	HAYN	2020-09-28	17.62	18.15	17.62	17.82	93716	{}	\N	2020-10-12 16:49:26.250802
84216	HAYN	2020-09-29	17.75	18.03	17.19	17.34	77416	{}	\N	2020-10-12 16:49:26.264125
84217	HAYN	2020-09-30	17.44	17.76	17.07	17.09	87930	{}	\N	2020-10-12 16:49:26.272561
84218	HAYN	2020-10-01	17.05	17.51	17.05	17.33	65766	{}	\N	2020-10-12 16:49:26.279478
84219	HAYN	2020-10-02	17.01	17.86	17	17.74	76303	{}	\N	2020-10-12 16:49:26.286032
84220	HAYN	2020-10-05	18.11	18.46	17.85	17.98	55878	{}	\N	2020-10-12 16:49:26.291964
84221	HAYN	2020-10-06	18.18	18.44	17.59	17.63	66776	{}	\N	2020-10-12 16:49:26.297779
84222	HAYN	2020-10-07	17.66	17.95	17.6	17.86	71036	{}	\N	2020-10-12 16:49:26.303721
84223	HAYN	2020-10-08	18.015	18.375	17.88	18.21	52695	{}	\N	2020-10-12 16:49:26.310172
84224	HAYN	2020-10-09	18.41	18.42	17.78	18.07	72032	{}	\N	2020-10-12 16:49:26.315915
84225	HAYN	2020-10-12	17.835	18.2506	17.835	18.1	14338	{}	\N	2020-10-12 16:49:26.321052
84226	HBIO	2020-09-28	2.99	3.01	2.92	2.99	167218	{}	\N	2020-10-12 16:49:27.279182
84227	HBIO	2020-09-29	2.98	3.03	2.98	3.03	110437	{}	\N	2020-10-12 16:49:27.290738
84228	HBIO	2020-09-30	3.03	3.07	2.87	3.01	136830	{}	\N	2020-10-12 16:49:27.298006
84229	HBIO	2020-10-01	3.03	3.09	2.98	3.02	59474	{}	\N	2020-10-12 16:49:27.305066
84230	HBIO	2020-10-02	2.95	3.0887	2.93	3.05	63938	{}	\N	2020-10-12 16:49:27.311358
84231	HBIO	2020-10-05	3.06	3.2	3.0458	3.19	64982	{}	\N	2020-10-12 16:49:27.316358
84232	HBIO	2020-10-06	3.23	3.31	3.2	3.21	59308	{}	\N	2020-10-12 16:49:27.321415
84233	HBIO	2020-10-07	3.25	3.26	3.18	3.24	53897	{}	\N	2020-10-12 16:49:27.326212
84234	HBIO	2020-10-08	3.25	3.33	3.18	3.3	52811	{}	\N	2020-10-12 16:49:27.330995
84235	HBIO	2020-10-09	3.375	3.375	3.27	3.34	127971	{}	\N	2020-10-12 16:49:27.335989
84236	HBIO	2020-10-12	3.37	3.41	3.3501	3.39	122832	{}	\N	2020-10-12 16:49:27.340825
84237	IBDS	2020-09-28	27.09	27.15	27.05	27.15	22454	{}	\N	2020-10-12 16:49:30.638178
84238	IBDS	2020-09-29	27.1499	27.23	27.1279	27.185	27210	{}	\N	2020-10-12 16:49:30.650702
84239	IBDS	2020-09-30	27.09	27.19	27.09	27.13	31309	{}	\N	2020-10-12 16:49:30.658256
84240	IBDS	2020-10-01	27.17	27.17	27.055	27.15	57170	{}	\N	2020-10-12 16:49:30.664331
84241	IBDS	2020-10-02	27.09	27.16	27.0637	27.15	52943	{}	\N	2020-10-12 16:49:30.670245
84242	IBDS	2020-10-05	27.12	27.18	27.12	27.18	54976	{}	\N	2020-10-12 16:49:30.675867
84243	IBDS	2020-10-06	27.18	27.18	27.05	27.14	54865	{}	\N	2020-10-12 16:49:30.681476
84244	IBDS	2020-10-07	27.09	27.19	27.06	27.06	38600	{}	\N	2020-10-12 16:49:30.686899
84245	IBDS	2020-10-08	27.13	27.16	27.08	27.16	14451	{}	\N	2020-10-12 16:49:30.692571
84246	IBDS	2020-10-09	27.2	27.2084	27.1	27.17	31371	{}	\N	2020-10-12 16:49:30.697864
84247	IBDS	2020-10-12	27.23	27.26	27.17	27.21	21182	{}	\N	2020-10-12 16:49:30.703497
84248	IDLB	2020-09-30	26.92	26.92	26.92	26.92	203	{}	\N	2020-10-12 16:49:31.833162
84249	IDLB	2020-10-01	27.02	27.02	26.98	27.02	3479	{}	\N	2020-10-12 16:49:31.84316
84250	IDLB	2020-10-02	27.02	27.02	27.02	27.02	201	{}	\N	2020-10-12 16:49:31.849939
84251	IDLB	2020-10-05	27.201	27.3716	27.201	27.3716	200	{}	\N	2020-10-12 16:49:31.855989
84252	FLM	2020-09-28	42.4537	42.4537	42.4	42.4	254	{}	\N	2020-10-12 16:49:33.141809
84253	FLM	2020-09-29	42.22	42.33	42.22	42.33	215	{}	\N	2020-10-12 16:49:33.151364
84254	FLM	2020-09-30	42.08	42.24	41.93	41.97	1173	{}	\N	2020-10-12 16:49:33.157112
84255	FLM	2020-10-01	42.13	42.13	42.13	42.13	183	{}	\N	2020-10-12 16:49:33.162771
84256	FLM	2020-10-02	41.3	42.3	41.26	42.26	17010	{}	\N	2020-10-12 16:49:33.172389
84257	FLM	2020-10-05	43.27	43.294	43.27	43.28	800	{}	\N	2020-10-12 16:49:33.178384
84258	FLM	2020-10-06	43.55	43.55	43.1163	43.1163	2131	{}	\N	2020-10-12 16:49:33.184323
84259	FLM	2020-10-07	43.6	43.745	43.6	43.7	12032	{}	\N	2020-10-12 16:49:33.188929
84260	FLM	2020-10-08	44.09	44.21	44.08	44.19	4379	{}	\N	2020-10-12 16:49:33.19403
84261	FLM	2020-10-09	44.17	44.17	44.13	44.13	1330	{}	\N	2020-10-12 16:49:33.199041
84262	FNI	2020-09-28	47.41	47.41	46.84	47.16	7449	{}	\N	2020-10-12 16:49:34.746772
84263	FNI	2020-09-29	47.07	47.07	46.7301	46.85	6444	{}	\N	2020-10-12 16:49:34.756633
84264	FNI	2020-09-30	47.3	47.98	47.3	47.63	6552	{}	\N	2020-10-12 16:49:34.764256
84265	FNI	2020-10-01	48.0857	48.2888	48.0584	48.18	12099	{}	\N	2020-10-12 16:49:34.770223
84266	FNI	2020-10-02	47.5	47.9282	47.43	47.75	17950	{}	\N	2020-10-12 16:49:34.775327
84267	FNI	2020-10-05	48.21	48.2915	48	48.2915	23087	{}	\N	2020-10-12 16:49:34.779917
84268	FNI	2020-10-06	48.35	49.1099	48.35	48.62	11941	{}	\N	2020-10-12 16:49:34.784963
84269	FNI	2020-10-07	48.95	49.1935	48.95	49.12	6737	{}	\N	2020-10-12 16:49:34.790211
84270	FNI	2020-10-08	49.66	49.66	49.3607	49.43	5589	{}	\N	2020-10-12 16:49:34.795648
84271	FNI	2020-10-09	49.8	50.21	49.7726	50.115	5648	{}	\N	2020-10-12 16:49:34.800349
84272	FNI	2020-10-12	50.68	50.68	50.4902	50.5593	2401	{}	\N	2020-10-12 16:49:34.805056
84273	FPX	2020-09-28	95.42	96.19	94.945	95.62	63514	{}	\N	2020-10-12 16:49:35.803387
84274	FPX	2020-09-29	95.66	96.515	95.66	95.75	31580	{}	\N	2020-10-12 16:49:35.814816
84275	FPX	2020-09-30	95.71	97.2	95.71	96.64	52407	{}	\N	2020-10-12 16:49:35.822112
84276	FPX	2020-10-01	97.61	98.55	97.41	98.35	86889	{}	\N	2020-10-12 16:49:35.827903
84277	FPX	2020-10-02	96.22	98.624	96.22	97.71	43390	{}	\N	2020-10-12 16:49:35.833511
84278	FPX	2020-10-05	98.87	99.7464	98.87	99.68	67722	{}	\N	2020-10-12 16:49:35.838877
84279	FPX	2020-10-06	99.97	100.5314	98.3699	98.6	45231	{}	\N	2020-10-12 16:49:35.843734
84280	FPX	2020-10-07	99.63	100.61	99.63	100.08	50720	{}	\N	2020-10-12 16:49:35.848646
84281	FPX	2020-10-08	101.14	101.14	100.385	100.58	77939	{}	\N	2020-10-12 16:49:35.853999
84282	FPX	2020-10-09	101.29	102.26	101.29	102.26	89638	{}	\N	2020-10-12 16:49:35.85913
84283	FPX	2020-10-12	103.17	103.3	102.615	103.14	35883	{}	\N	2020-10-12 16:49:35.864514
84284	FUD	2020-09-28	15.405	15.415	15.12	15.415	4638	{}	\N	2020-10-12 16:49:36.760066
84285	FUD	2020-09-29	15.3493	15.7	15.13	15.59	3760	{}	\N	2020-10-12 16:49:36.771117
84286	FUD	2020-09-30	15.61	16.09	15.61	15.8	3458	{}	\N	2020-10-12 16:49:36.778737
84287	FUD	2020-10-01	15.99	16.17	15.858	16.04	6881	{}	\N	2020-10-12 16:49:36.784248
84288	FUD	2020-10-02	15.61	16.13	15.61	15.645	2229	{}	\N	2020-10-12 16:49:36.790824
84289	FUD	2020-10-05	16.16	16.16	15.785	15.785	3201	{}	\N	2020-10-12 16:49:36.795833
84290	FUD	2020-10-06	15.9975	16.31	15.9975	16.31	2957	{}	\N	2020-10-12 16:49:36.800568
84291	FUD	2020-10-07	16.4199	16.5	16.2223	16.3536	4615	{}	\N	2020-10-12 16:49:36.805745
84292	FUD	2020-10-08	16.24	16.6	16.24	16.6	2920	{}	\N	2020-10-12 16:49:36.810541
84293	FUD	2020-10-09	16.44	16.52	16.4	16.52	2930	{}	\N	2020-10-12 16:49:36.815674
84294	FUD	2020-10-12	16.38	16.5999	16.25	16.25	3334	{}	\N	2020-10-12 16:49:36.82116
84295	GECC	2020-09-28	3.46	3.57	3.415	3.56	287563	{}	\N	2020-10-12 16:49:38.136089
84296	GECC	2020-09-29	3.45	3.54	3.35	3.36	209012	{}	\N	2020-10-12 16:49:38.147649
84297	GECC	2020-09-30	3.41	3.45	3.36	3.38	55224	{}	\N	2020-10-12 16:49:38.155824
84298	GECC	2020-10-01	3.32	3.3593	3.11	3.23	175987	{}	\N	2020-10-12 16:49:38.164563
84299	GECC	2020-10-02	3.01	3.12	3	3.03	284399	{}	\N	2020-10-12 16:49:38.170507
84300	GECC	2020-10-05	3.03	3.112	3.03	3.05	118999	{}	\N	2020-10-12 16:49:38.176592
84301	GECC	2020-10-06	3.11	3.11	2.91	2.95	404988	{}	\N	2020-10-12 16:49:38.181915
84302	GECC	2020-10-07	2.95	3.1	2.95	2.99	295102	{}	\N	2020-10-12 16:49:38.187181
84303	GECC	2020-10-08	3.06	3.06	2.96	2.96	316315	{}	\N	2020-10-12 16:49:38.192915
84304	GECC	2020-10-09	3.06	3.06	2.95	3	309863	{}	\N	2020-10-12 16:49:38.198211
84305	GECC	2020-10-12	3.05	3.07	2.98	2.99	147673	{}	\N	2020-10-12 16:49:38.203113
84306	GSBC	2020-09-28	35.04	37.065	35.04	36.89	52028	{}	\N	2020-10-12 16:49:41.735253
84307	GSBC	2020-09-29	36.685	36.93	35.43	36.17	60706	{}	\N	2020-10-12 16:49:41.746224
84308	GSBC	2020-09-30	36.36	36.85	35.94	36.26	37914	{}	\N	2020-10-12 16:49:41.753667
84309	GSBC	2020-10-01	36.41	36.485	35.88	36.43	28337	{}	\N	2020-10-12 16:49:41.759611
84310	GSBC	2020-10-02	35.79	37.37	35.79	37.12	20820	{}	\N	2020-10-12 16:49:41.765703
84311	GSBC	2020-10-05	37.46	38.45	37.36	38.2	25951	{}	\N	2020-10-12 16:49:41.771507
84312	GSBC	2020-10-06	38.7	39.5	37.28	38.14	35575	{}	\N	2020-10-12 16:49:41.777382
84313	GSBC	2020-10-07	38.56	39.46	38.49	38.99	34135	{}	\N	2020-10-12 16:49:41.783303
84314	GSBC	2020-10-08	39.46	39.46	38.25	39.18	23588	{}	\N	2020-10-12 16:49:41.789211
84315	GSBC	2020-10-09	39.43	39.9999	38.66	39.07	21652	{}	\N	2020-10-12 16:49:41.795133
84316	GSBC	2020-10-12	39.03	40.13	38.78	39.515	3864	{}	\N	2020-10-12 16:49:41.80183
84317	HBANN	2020-09-28	26.72	27.08	26.72	27.06	4262	{}	\N	2020-10-12 16:49:42.994717
84318	HBANN	2020-09-29	26.941	27.09	26.7745	27.05	3329	{}	\N	2020-10-12 16:49:43.005185
84319	HBANN	2020-09-30	26.41	27.02	26.41	26.99	2000	{}	\N	2020-10-12 16:49:43.012621
84320	HBANN	2020-10-01	26.94	27.11	26.86	27.0664	3754	{}	\N	2020-10-12 16:49:43.018624
84321	HBANN	2020-10-02	26.99	27	26.99	27	970	{}	\N	2020-10-12 16:49:43.024192
84322	HBANN	2020-10-05	26.65	27	26.38	27	3881	{}	\N	2020-10-12 16:49:43.029851
84323	HBANN	2020-10-06	26.9	27	26.85	27	2479	{}	\N	2020-10-12 16:49:43.034786
84324	HBANN	2020-10-07	26.9	26.93	26.76	26.76	4198	{}	\N	2020-10-12 16:49:43.039764
84325	HBANN	2020-10-08	27.03	27.03	26.73	26.95	3152	{}	\N	2020-10-12 16:49:43.045113
84326	HBANN	2020-10-09	27.065	27.1	27	27.1	1343	{}	\N	2020-10-12 16:49:43.050367
84327	HBANN	2020-10-12	27.27	27.27	27.27	27.27	1004	{}	\N	2020-10-12 16:49:43.055499
84328	HBM	2020-09-28	4	4.1	4	4.1	932756	{}	\N	2020-10-12 16:49:44.540263
84329	HBM	2020-09-29	4.08	4.17	3.99	4.13	1312751	{}	\N	2020-10-12 16:49:44.552812
84330	HBM	2020-09-30	4.15	4.27	4.14	4.23	2063852	{}	\N	2020-10-12 16:49:44.561558
84331	HBM	2020-10-01	4.23	4.28	4.125	4.2	1666083	{}	\N	2020-10-12 16:49:44.56803
84332	HBM	2020-10-02	4.11	4.225	4.06	4.18	1022558	{}	\N	2020-10-12 16:49:44.574127
84333	HBM	2020-10-05	4.2	4.49	4.2	4.46	1831546	{}	\N	2020-10-12 16:49:44.579157
84334	HBM	2020-10-06	4.45	4.49	4.17	4.2	1439194	{}	\N	2020-10-12 16:49:44.58426
84335	HBM	2020-10-07	4.25	4.395	4.25	4.25	1348084	{}	\N	2020-10-12 16:49:44.590236
84336	HBM	2020-10-08	4.25	4.42	4.22	4.4	1782733	{}	\N	2020-10-12 16:49:44.595371
84337	HBM	2020-10-09	4.57	4.61	4.49	4.61	1463857	{}	\N	2020-10-12 16:49:44.600664
84338	HBM	2020-10-12	4.43	4.61	4.35	4.39	803852	{}	\N	2020-10-12 16:49:44.605873
84339	HEWU	2020-09-28	18.9901	19.0301	18.9901	19.0301	1151	{}	\N	2020-10-12 16:49:46.796681
84340	HEWU	2020-09-29	18.91	18.91	18.87	18.87	1553	{}	\N	2020-10-12 16:49:46.809669
84341	HEWU	2020-09-30	18.95	18.95	18.7887	18.7887	1075	{}	\N	2020-10-12 16:49:46.816904
84342	HEWU	2020-10-01	18.86	18.86	18.848	18.848	406	{}	\N	2020-10-12 16:49:46.822745
84343	HEWU	2020-10-02	18.75	18.75	18.75	18.75	165	{}	\N	2020-10-12 16:49:46.828821
84344	HEWU	2020-10-05	19.03	19.0956	19.03	19.0956	2026	{}	\N	2020-10-12 16:49:46.834009
84345	HEWU	2020-10-06	19.13	19.13	18.9406	18.9406	533	{}	\N	2020-10-12 16:49:46.838643
84346	HEWU	2020-10-07	19.0619	19.0619	19.0619	19.0619	2	{}	\N	2020-10-12 16:49:46.84319
84347	HEWU	2020-10-08	19.2244	19.2244	19.2244	19.2244	53	{}	\N	2020-10-12 16:49:46.847814
84348	HEWU	2020-10-09	19.25	19.25	19.2412	19.2412	750	{}	\N	2020-10-12 16:49:46.852491
84349	HEWU	2020-10-12	19.24	19.24	19.24	19.24	484	{}	\N	2020-10-12 16:49:46.856856
84350	HTZ	2020-09-28	1.23	1.26	1.21	1.22	5221774	{}	\N	2020-10-12 16:49:48.216061
84351	HTZ	2020-09-29	1.23	1.24	1.12	1.13	28876882	{}	\N	2020-10-12 16:49:48.229173
84352	HTZ	2020-09-30	1.14	1.15	1.04	1.11	28023706	{}	\N	2020-10-12 16:49:48.237051
84353	HTZ	2020-10-01	1.1	1.115	1.06	1.1	11542943	{}	\N	2020-10-12 16:49:48.242992
84354	HTZ	2020-10-02	1.04	1.13	1.01	1.08	14364342	{}	\N	2020-10-12 16:49:48.248955
84355	HTZ	2020-10-05	1.07	1.08	1.03	1.06	11511382	{}	\N	2020-10-12 16:49:48.253896
84356	HTZ	2020-10-06	1.06	1.24	1.04	1.09	35332731	{}	\N	2020-10-12 16:49:48.258487
84357	HTZ	2020-10-07	1.1	1.13	1.07	1.08	10165283	{}	\N	2020-10-12 16:49:48.263498
84358	HTZ	2020-10-08	1.09	1.18	1.07	1.12	18065704	{}	\N	2020-10-12 16:49:48.268474
84359	HTZ	2020-10-09	1.13	1.14	1.08	1.1	10182732	{}	\N	2020-10-12 16:49:48.273103
84360	HTZ	2020-10-12	1.1	1.12	1.08	1.0875	3943804	{}	\N	2020-10-12 16:49:48.27785
84361	HURC	2020-09-28	29.15	29.22	28.5	28.62	19896	{}	\N	2020-10-12 16:49:49.260887
84362	HURC	2020-09-29	28.65	28.66	28.18	28.55	2328	{}	\N	2020-10-12 16:49:49.273143
84363	HURC	2020-09-30	28.37	28.715	28.205	28.4	8326	{}	\N	2020-10-12 16:49:49.282371
84364	HURC	2020-10-01	28.6	28.87	27.72	27.95	9643	{}	\N	2020-10-12 16:49:49.288871
84365	HURC	2020-10-02	28.705	28.71	27.98	27.98	4103	{}	\N	2020-10-12 16:49:49.294644
84366	HURC	2020-10-05	28.5	30	28.0001	28.81	6512	{}	\N	2020-10-12 16:49:49.29978
84367	HURC	2020-10-06	29.36	29.85	28.0163	28.51	4822	{}	\N	2020-10-12 16:49:49.305383
84368	HURC	2020-10-07	28.92	29.22	28.43	28.97	9507	{}	\N	2020-10-12 16:49:49.311164
84369	HURC	2020-10-08	28.98	29.29	28.66	28.8	8931	{}	\N	2020-10-12 16:49:49.317661
84370	HURC	2020-10-09	29.11	29.11	28.3231	28.61	7376	{}	\N	2020-10-12 16:49:49.322666
84371	HURC	2020-10-12	28.53	28.9	28.3	28.89	1262	{}	\N	2020-10-12 16:49:49.327677
84372	OESX	2020-09-28	7.36	7.7501	7.3501	7.59	408134	{}	\N	2020-10-12 16:49:50.269871
84373	OESX	2020-09-29	7.51	7.83	7.45	7.73	422534	{}	\N	2020-10-12 16:49:50.282475
84374	OESX	2020-09-30	7.66	7.7199	7.38	7.57	456053	{}	\N	2020-10-12 16:49:50.289787
84375	OESX	2020-10-01	7.59	7.75	7.4457	7.6	288553	{}	\N	2020-10-12 16:49:50.295443
84376	OESX	2020-10-02	7.4	7.65	7.08	7.45	271773	{}	\N	2020-10-12 16:49:50.301104
84377	OESX	2020-10-05	7.62	7.82	7.47	7.77	288707	{}	\N	2020-10-12 16:49:50.306081
84378	OESX	2020-10-06	7.83	7.9	7.54	7.6	238527	{}	\N	2020-10-12 16:49:50.310678
84379	OESX	2020-10-07	7.75	8.1804	7.65	7.98	487960	{}	\N	2020-10-12 16:49:50.315174
84380	OESX	2020-10-08	8.09	9.3	8.03	8.45	2065407	{}	\N	2020-10-12 16:49:50.320408
84381	OESX	2020-10-09	8.749	9.1366	8.62	8.88	699660	{}	\N	2020-10-12 16:49:50.325392
84382	OESX	2020-10-12	9.14	9.19	8.9	9.09	289118	{}	\N	2020-10-12 16:49:50.330317
84383	FLMB	2020-09-28	26.959	26.959	26.93	26.93	851	{}	\N	2020-10-12 16:49:51.52066
84384	FLMB	2020-09-29	26.8801	26.96	26.88	26.928	11801	{}	\N	2020-10-12 16:49:51.529865
84385	FLMB	2020-09-30	26.86	26.9	26.83	26.83	4122	{}	\N	2020-10-12 16:49:51.536394
84386	FLMB	2020-10-01	26.8445	26.8445	26.825	26.825	197	{}	\N	2020-10-12 16:49:51.542333
84387	FLMB	2020-10-02	26.83	26.83	26.815	26.815	1795	{}	\N	2020-10-12 16:49:51.548065
84388	FLMB	2020-10-05	26.81	26.81	26.765	26.765	1470	{}	\N	2020-10-12 16:49:51.553796
84389	FLMB	2020-10-06	26.735	26.735	26.735	26.735	57	{}	\N	2020-10-12 16:49:51.559141
84390	FLMB	2020-10-07	26.67	26.685	26.67	26.685	488	{}	\N	2020-10-12 16:49:51.564452
84391	FLMB	2020-10-08	26.6734	26.705	26.665	26.665	3574	{}	\N	2020-10-12 16:49:51.569991
84392	FLMB	2020-10-09	26.63	26.7	26.63	26.6401	7839	{}	\N	2020-10-12 16:49:51.575344
84393	FSTA	2020-09-28	37.87	38.1799	37.87	37.94	89925	{}	\N	2020-10-12 16:49:55.146337
84394	FSTA	2020-09-29	38.05	38.0909	37.68	37.76	61530	{}	\N	2020-10-12 16:49:55.157009
84395	FSTA	2020-09-30	37.88	38.418	37.88	38.16	49684	{}	\N	2020-10-12 16:49:55.164724
84396	FSTA	2020-10-01	38.35	38.4837	38.0809	38.41	76193	{}	\N	2020-10-12 16:49:55.170128
84397	FSTA	2020-10-02	37.96	38.37	37.94	38.21	76895	{}	\N	2020-10-12 16:49:55.177041
84398	FSTA	2020-10-05	38.32	38.537	38.27	38.52	116993	{}	\N	2020-10-12 16:49:55.182279
84399	FSTA	2020-10-06	38.64	38.785	38.206	38.3	943254	{}	\N	2020-10-12 16:49:55.187431
84400	FSTA	2020-10-07	38.51	38.885	38.51	38.73	85886	{}	\N	2020-10-12 16:49:55.192283
84401	FSTA	2020-10-08	38.85	39.027	38.78	39	169689	{}	\N	2020-10-12 16:49:55.196758
84402	FSTA	2020-10-09	39.15	39.4431	39.15	39.32	156457	{}	\N	2020-10-12 16:49:55.201813
84403	FSTA	2020-10-12	39.5	39.93	39.4917	39.8858	73037	{}	\N	2020-10-12 16:49:55.206896
84404	GCBC	2020-09-28	22	22	21.51	21.55	3235	{}	\N	2020-10-12 16:49:57.217066
84405	GCBC	2020-09-29	21.6	21.87	21.6	21.87	808	{}	\N	2020-10-12 16:49:57.227011
84406	GCBC	2020-09-30	21.65	21.69	21.56	21.69	1729	{}	\N	2020-10-12 16:49:57.23387
84407	GCBC	2020-10-01	21.8	21.8	21.51	21.51	2801	{}	\N	2020-10-12 16:49:57.239955
84408	GCBC	2020-10-02	21.5	21.65	21.5	21.57	10491	{}	\N	2020-10-12 16:49:57.245722
84409	GCBC	2020-10-05	22.19	22.97	22.19	22.4	2648	{}	\N	2020-10-12 16:49:57.250878
84410	GCBC	2020-10-06	22.395	22.395	21.86	21.86	2611	{}	\N	2020-10-12 16:49:57.255896
84411	GCBC	2020-10-07	22.1514	22.45	22.1514	22.45	2305	{}	\N	2020-10-12 16:49:57.261719
84412	GCBC	2020-10-08	22.15	22.448	22.12	22.12	1961	{}	\N	2020-10-12 16:49:57.266975
84413	GCBC	2020-10-09	22	22.5	22	22.5	3192	{}	\N	2020-10-12 16:49:57.271902
84414	GHY	2020-09-28	13.23	13.31	13.23	13.28	162875	{}	\N	2020-10-12 16:49:58.350252
84415	GHY	2020-09-29	13.31	13.31	13.21	13.3	318123	{}	\N	2020-10-12 16:49:58.362012
84416	GHY	2020-09-30	13.36	13.36	13.25	13.25	331096	{}	\N	2020-10-12 16:49:58.370236
84417	GHY	2020-10-01	13.25	13.32	13.24	13.28	249782	{}	\N	2020-10-12 16:49:58.377488
84418	GHY	2020-10-02	13.23	13.56	13.22	13.36	100153	{}	\N	2020-10-12 16:49:58.384029
84419	GHY	2020-10-05	13.36	13.4	13.31	13.4	205164	{}	\N	2020-10-12 16:49:58.389175
84420	GHY	2020-10-06	13.4	13.48	13.39	13.4	166674	{}	\N	2020-10-12 16:49:58.394937
84421	GHY	2020-10-07	13.48	13.53	13.43	13.43	286580	{}	\N	2020-10-12 16:49:58.400387
84422	GHY	2020-10-08	13.46	13.51	13.42	13.46	150131	{}	\N	2020-10-12 16:49:58.40574
84423	GHY	2020-10-09	13.47	13.49	13.43	13.46	133636	{}	\N	2020-10-12 16:49:58.41112
84424	GHY	2020-10-12	13.44	13.49	13.44	13.48	86986	{}	\N	2020-10-12 16:49:58.416678
84425	GRC	2020-09-28	30.42	30.63	30.16	30.26	25159	{}	\N	2020-10-12 16:50:01.402564
84426	GRC	2020-09-29	30.26	30.41	29.98	30.16	71838	{}	\N	2020-10-12 16:50:01.412881
84427	GRC	2020-09-30	30.24	30.39	29.16	29.46	136928	{}	\N	2020-10-12 16:50:01.421252
84428	GRC	2020-10-01	29.45	29.76	29.05	29.49	150126	{}	\N	2020-10-12 16:50:01.427633
84429	GRC	2020-10-02	29	31.25	29	30.92	226469	{}	\N	2020-10-12 16:50:01.434057
84430	GRC	2020-10-05	31.13	31.93	31.13	31.82	97237	{}	\N	2020-10-12 16:50:01.439795
84431	GRC	2020-10-06	32.13	32.62	31.61	31.69	69950	{}	\N	2020-10-12 16:50:01.445475
84432	GRC	2020-10-07	32.03	32.48	31.68	32.32	97865	{}	\N	2020-10-12 16:50:01.451059
84433	GRC	2020-10-08	32.61	32.735	31.8	32.44	50396	{}	\N	2020-10-12 16:50:01.456447
84434	GRC	2020-10-09	32.68	33	32.5	32.77	50140	{}	\N	2020-10-12 16:50:01.461368
84435	GRC	2020-10-12	32.97	33.92	32.49	33.66	45584	{}	\N	2020-10-12 16:50:01.467095
84436	GRMN	2020-09-28	94.91	95.81	93.96	95.38	531396	{}	\N	2020-10-12 16:50:03.134595
84437	GRMN	2020-09-29	95.58	95.7818	94.86	95	458912	{}	\N	2020-10-12 16:50:03.145428
84438	GRMN	2020-09-30	95.34	95.835	94.37	94.86	681195	{}	\N	2020-10-12 16:50:03.152982
84439	GRMN	2020-10-01	95.78	95.93	94.35	94.68	583887	{}	\N	2020-10-12 16:50:03.158912
84440	GRMN	2020-10-02	93.75	95.75	93.74	94.93	462201	{}	\N	2020-10-12 16:50:03.164527
84441	GRMN	2020-10-05	95.74	96.66	95.54	96.54	552469	{}	\N	2020-10-12 16:50:03.169704
84442	GRMN	2020-10-06	96.54	97.18	94.92	95.15	532687	{}	\N	2020-10-12 16:50:03.174798
84443	GRMN	2020-10-07	95.87	97.2	95.8	96.92	570480	{}	\N	2020-10-12 16:50:03.179526
84444	GRMN	2020-10-08	97.51	98.21	96.92	98.13	482880	{}	\N	2020-10-12 16:50:03.184471
84445	GRMN	2020-10-09	98.47	99.45	98.32	98.82	521200	{}	\N	2020-10-12 16:50:03.189158
84446	GRMN	2020-10-12	99.27	99.49	98.74	99.3551	133356	{}	\N	2020-10-12 16:50:03.193733
84447	GSEW	2020-09-28	49.5	49.7799	49.3101	49.53	12246	{}	\N	2020-10-12 16:50:04.91718
84448	GSEW	2020-09-29	49.59	49.67	49.24	49.37	21954	{}	\N	2020-10-12 16:50:04.935008
84449	GSEW	2020-09-30	49.58	50.035	49.3322	49.7209	13771	{}	\N	2020-10-12 16:50:04.941568
84450	GSEW	2020-10-01	50.08	50.08	49.595	49.9	25986	{}	\N	2020-10-12 16:50:04.94779
84451	GSEW	2020-10-02	49.38	50.17	49.38	50.01	20560	{}	\N	2020-10-12 16:50:04.954036
84452	GSEW	2020-10-05	50.54	50.97	50.4797	50.93	34941	{}	\N	2020-10-12 16:50:04.959511
84453	GSEW	2020-10-06	50.89	51.5	50.4567	50.55	21975	{}	\N	2020-10-12 16:50:04.965478
84454	GSEW	2020-10-07	51.05	51.44	51.035	51.29	80073	{}	\N	2020-10-12 16:50:04.971
84455	GSEW	2020-10-08	51.7	51.91	51.61	51.86	259688	{}	\N	2020-10-12 16:50:04.976733
84456	GSEW	2020-10-09	52.37	52.37	51.93	52.04	15195	{}	\N	2020-10-12 16:50:04.981927
84457	GSEW	2020-10-12	52.58	52.58	52.21	52.4613	2738	{}	\N	2020-10-12 16:50:04.987451
84458	HBI	2020-09-28	15.86	15.97	15.63	15.79	4215015	{}	\N	2020-10-12 16:50:06.027913
84459	HBI	2020-09-29	15.79	15.79	15.315	15.44	6923175	{}	\N	2020-10-12 16:50:06.038538
84460	HBI	2020-09-30	15.47	15.88	15.47	15.75	9913981	{}	\N	2020-10-12 16:50:06.045148
84461	HBI	2020-10-01	15.85	16.13	15.525	15.95	8521117	{}	\N	2020-10-12 16:50:06.051695
84462	HBI	2020-10-02	15.5	16.175	15.29	16.15	6768581	{}	\N	2020-10-12 16:50:06.057878
84463	HBI	2020-10-05	16.26	16.76	16.11	16.74	12356211	{}	\N	2020-10-12 16:50:06.062984
84464	HBI	2020-10-06	16.82	17.175	16.61	16.61	13078433	{}	\N	2020-10-12 16:50:06.068016
84465	HBI	2020-10-07	16.95	17.33	16.77	16.8	10902592	{}	\N	2020-10-12 16:50:06.073064
84466	HBI	2020-10-08	16.82	17.07	16.72	16.97	7271695	{}	\N	2020-10-12 16:50:06.078599
84467	HBI	2020-10-09	17.04	17.25	16.845	17.21	8568803	{}	\N	2020-10-12 16:50:06.086324
84468	HBI	2020-10-12	17.25	17.32	17.09	17.23	3016878	{}	\N	2020-10-12 16:50:06.091623
84469	HSCZ	2020-09-28	28.9301	28.977	28.917	28.917	4086	{}	\N	2020-10-12 16:50:08.231848
84470	HSCZ	2020-09-29	28.9	28.9195	28.86	28.86	3296	{}	\N	2020-10-12 16:50:08.244978
84471	HSCZ	2020-09-30	28.9	29.0396	28.84	28.9101	11575	{}	\N	2020-10-12 16:50:08.252461
84472	HSCZ	2020-10-01	29.03	29.08	28.98	29.02	4467	{}	\N	2020-10-12 16:50:08.258935
84473	HSCZ	2020-10-02	28.76	29.0299	28.76	28.955	6688	{}	\N	2020-10-12 16:50:08.264921
84474	HSCZ	2020-10-05	29.28	29.45	29.28	29.45	4557	{}	\N	2020-10-12 16:50:08.269939
84475	HSCZ	2020-10-06	29.49	29.525	29.2986	29.2986	14859	{}	\N	2020-10-12 16:50:08.275334
84476	HSCZ	2020-10-07	29.43	29.48	29.37	29.425	2492	{}	\N	2020-10-12 16:50:08.280677
84477	HSCZ	2020-10-08	29.71	29.71	29.6	29.67	6231	{}	\N	2020-10-12 16:50:08.28571
84478	HSCZ	2020-10-09	29.73	29.76	29.7	29.74	21497	{}	\N	2020-10-12 16:50:08.290793
84479	HSCZ	2020-10-12	29.82	29.91	29.778	29.83	11894	{}	\N	2020-10-12 16:50:08.295807
84480	HTFA	2020-09-28	25.1	25.1	25.1	25.1	2986	{}	\N	2020-10-12 16:50:09.863846
84481	HTFA	2020-09-29	25.16	25.16	25.0552	25.1	2726	{}	\N	2020-10-12 16:50:09.873101
84482	HTFA	2020-09-30	25.15	25.15	25.025	25.06	13390	{}	\N	2020-10-12 16:50:09.879774
84483	HTFA	2020-10-01	25.15	25.15	25.045	25.1437	6018	{}	\N	2020-10-12 16:50:09.885318
84484	HTFA	2020-10-02	25.2	25.2	25.1999	25.2	1560	{}	\N	2020-10-12 16:50:09.89095
84485	HTFA	2020-10-05	25.2	25.25	25.2	25.25	2760	{}	\N	2020-10-12 16:50:09.89585
84486	HTFA	2020-10-06	25.24	25.25	25.2324	25.25	3390	{}	\N	2020-10-12 16:50:09.90035
84487	HTFA	2020-10-07	25.23	25.25	25.13	25.2	8082	{}	\N	2020-10-12 16:50:09.905226
84488	HTFA	2020-10-08	25.25	25.25	25.2316	25.2318	7008	{}	\N	2020-10-12 16:50:09.910111
84489	HTFA	2020-10-09	25.7499	25.7499	25.7499	25.7499	600	{}	\N	2020-10-12 16:50:09.915458
84490	HTUS	2020-09-28	26.37	26.63	25.79	26.46	9213	{}	\N	2020-10-12 16:50:12.308844
84491	HTUS	2020-09-29	26.3034	26.3034	26.3034	26.3034	39	{}	\N	2020-10-12 16:50:12.318076
84492	HTUS	2020-09-30	26.77	26.9175	26.55	26.55	3746	{}	\N	2020-10-12 16:50:12.324833
84493	HTUS	2020-10-01	26.77	26.9127	26.77	26.9127	1130	{}	\N	2020-10-12 16:50:12.33128
84494	HTUS	2020-10-02	26.6	26.8419	26.55	26.8419	1613	{}	\N	2020-10-12 16:50:12.336761
84495	HTUS	2020-10-05	26.85	26.98	26.85	26.98	405	{}	\N	2020-10-12 16:50:12.342015
84496	HTUS	2020-10-06	27.11	27.11	27.0304	27.0304	350	{}	\N	2020-10-12 16:50:12.34748
84497	HTUS	2020-10-07	27.207	27.207	27.207	27.207	0	{}	\N	2020-10-12 16:50:12.353052
84498	HTUS	2020-10-08	27.48	27.48	27.4	27.4205	1070	{}	\N	2020-10-12 16:50:12.358375
84499	HTUS	2020-10-09	27.65	27.732	27.65	27.732	236	{}	\N	2020-10-12 16:50:12.363815
84500	IDA	2020-09-28	81.26	81.91	80.51	81.23	209636	{}	\N	2020-10-12 16:50:14.241676
84501	IDA	2020-09-29	81.14	81.9	80	80.1	447091	{}	\N	2020-10-12 16:50:14.255043
84502	IDA	2020-09-30	80.47	81.08	79.29	79.9	703969	{}	\N	2020-10-12 16:50:14.263004
84503	IDA	2020-10-01	80.03	82.05	80.01	81.86	681418	{}	\N	2020-10-12 16:50:14.269291
84504	IDA	2020-10-02	81.13	83.42	81.04	82.97	635803	{}	\N	2020-10-12 16:50:14.275005
84505	IDA	2020-10-05	82.97	83.43	81.91	83.08	942618	{}	\N	2020-10-12 16:50:14.279178
84506	IDA	2020-10-06	83.43	86.49	82.98	86.13	977551	{}	\N	2020-10-12 16:50:14.28316
84507	IDA	2020-10-07	86.66	87.19	85.74	86.45	671156	{}	\N	2020-10-12 16:50:14.287273
84508	IDA	2020-10-08	86.88	87.24	86.08	87.13	376333	{}	\N	2020-10-12 16:50:14.292804
84509	IDA	2020-10-09	87.64	87.7	86.03	87.42	394212	{}	\N	2020-10-12 16:50:14.297214
84510	IDA	2020-10-12	87.4	88.36	86.89	88.26	165611	{}	\N	2020-10-12 16:50:14.301656
84511	FTSL	2020-09-28	46.23	46.41	46	46.23	311683	{}	\N	2020-10-12 16:50:16.442382
84512	FTSL	2020-09-29	46.22	46.29	46.12	46.27	407628	{}	\N	2020-10-12 16:50:16.453358
84513	FTSL	2020-09-30	46.24	46.3399	45.99	46.01	620244	{}	\N	2020-10-12 16:50:16.461628
84514	FTSL	2020-10-01	46.0706	46.29	46.065	46.24	348706	{}	\N	2020-10-12 16:50:16.468059
84515	FTSL	2020-10-02	46.07	46.24	46.01	46.15	142866	{}	\N	2020-10-12 16:50:16.473794
84516	FTSL	2020-10-05	46.19	46.34	46	46.3	152362	{}	\N	2020-10-12 16:50:16.478638
84517	FTSL	2020-10-06	46.5012	46.5012	46	46.12	147942	{}	\N	2020-10-12 16:50:16.483253
84518	FTSL	2020-10-07	46.214	46.36	46.14	46.3526	106472	{}	\N	2020-10-12 16:50:16.487868
84519	FTSL	2020-10-08	46.38	46.42	46.1786	46.33	83847	{}	\N	2020-10-12 16:50:16.491968
84520	FTSL	2020-10-09	46.43	46.5189	46.34	46.44	120668	{}	\N	2020-10-12 16:50:16.496614
84521	FTSL	2020-10-12	46.41	46.54	46.37	46.5261	27906	{}	\N	2020-10-12 16:50:16.501096
84522	FVL	2020-09-28	21.78	21.9895	21.78	21.95	15686	{}	\N	2020-10-12 16:50:20.008133
84523	FVL	2020-09-29	22	22	21.86	21.93	1732	{}	\N	2020-10-12 16:50:20.020155
84524	FVL	2020-09-30	21.99	22.1401	21.86	22.04	4148	{}	\N	2020-10-12 16:50:20.026922
84525	FVL	2020-10-01	22.08	22.14	22.08	22.14	2372	{}	\N	2020-10-12 16:50:20.033126
84526	FVL	2020-10-02	21.9616	22	21.9616	22	1129	{}	\N	2020-10-12 16:50:20.039156
84527	FVL	2020-10-05	22.12	22.3088	22.12	22.3088	25036	{}	\N	2020-10-12 16:50:20.043962
84528	FVL	2020-10-06	22.24	22.3501	22.0653	22.0653	2043	{}	\N	2020-10-12 16:50:20.048609
84529	FVL	2020-10-07	22.3051	22.4425	22.29	22.4425	4234	{}	\N	2020-10-12 16:50:20.054215
84530	FVL	2020-10-08	22.495	22.5299	22.46	22.5299	2644	{}	\N	2020-10-12 16:50:20.059678
84531	FVL	2020-10-09	22.55	22.7118	22.55	22.7	1520	{}	\N	2020-10-12 16:50:20.064237
84532	FVL	2020-10-12	22.84	22.98	22.84	22.98	1608	{}	\N	2020-10-12 16:50:20.068876
84533	GREK	2020-09-28	19.94	20.1	19.65	19.85	26535	{}	\N	2020-10-12 16:50:26.857784
84534	GREK	2020-09-29	19.85	20.075	19.655	19.83	37129	{}	\N	2020-10-12 16:50:26.865732
84535	GREK	2020-09-30	19.98	20.4	19.66	19.975	14574	{}	\N	2020-10-12 16:50:26.871449
84536	GREK	2020-10-01	20.09	20.245	19.65	20	22106	{}	\N	2020-10-12 16:50:26.876857
84537	GREK	2020-10-02	19.6	19.9	19.6	19.83	22052	{}	\N	2020-10-12 16:50:26.881765
84538	GREK	2020-10-05	20.08	20.48	19.975	20.4	27808	{}	\N	2020-10-12 16:50:26.885964
84539	GREK	2020-10-06	20.46	20.6805	20.22	20.22	47891	{}	\N	2020-10-12 16:50:26.89046
84540	GREK	2020-10-07	20.34	20.585	20.14	20.47	44934	{}	\N	2020-10-12 16:50:26.894879
84541	GREK	2020-10-08	20.61	20.705	20.4	20.61	22273	{}	\N	2020-10-12 16:50:26.899195
84542	GREK	2020-10-09	20.82	20.995	20.56	20.73	21701	{}	\N	2020-10-12 16:50:26.903151
84543	GREK	2020-10-12	20.35	20.54	20.1	20.39	22492	{}	\N	2020-10-12 16:50:26.907276
84544	HASI	2020-09-28	40.33	41.2	40.0625	40.93	400351	{}	\N	2020-10-12 16:50:28.377486
84545	HASI	2020-09-29	41.15	42.22	40.73	41.88	1324783	{}	\N	2020-10-12 16:50:28.388576
84546	HASI	2020-09-30	41.83	43.01	41.83	42.27	2059381	{}	\N	2020-10-12 16:50:28.395112
84547	HASI	2020-10-01	42.24	43.28	40.81	43.1	1474631	{}	\N	2020-10-12 16:50:28.400921
84548	HASI	2020-10-02	42.06	43.5	41.85	43.18	971261	{}	\N	2020-10-12 16:50:28.40625
84549	HASI	2020-10-05	43.52	44.95	43.27	44.03	1291599	{}	\N	2020-10-12 16:50:28.41095
84550	HASI	2020-10-06	44.01	45.69	43.7501	44.14	1197784	{}	\N	2020-10-12 16:50:28.415619
84551	HASI	2020-10-07	44.81	45.7	44.48	45.25	1442380	{}	\N	2020-10-12 16:50:28.42015
84552	HASI	2020-10-08	45.93	45.98	44.74	45.18	865662	{}	\N	2020-10-12 16:50:28.425163
84553	HASI	2020-10-09	45.24	45.8459	44.8127	45.82	819457	{}	\N	2020-10-12 16:50:28.429777
84554	HASI	2020-10-12	45.99	46.31	45.55	45.74	281073	{}	\N	2020-10-12 16:50:28.434152
84555	HCAP	2020-09-28	3.34	3.62	3.3	3.6	41262	{}	\N	2020-10-12 16:50:30.716219
84556	HCAP	2020-09-29	3.36	3.5712	3.36	3.52	5510	{}	\N	2020-10-12 16:50:30.728731
84557	HCAP	2020-09-30	3.5	3.5129	3.45	3.45	8102	{}	\N	2020-10-12 16:50:30.736705
84558	HCAP	2020-10-01	3.58	3.6	3.55	3.56	7270	{}	\N	2020-10-12 16:50:30.742994
84559	HCAP	2020-10-02	3.51	3.55	3.51	3.55	3723	{}	\N	2020-10-12 16:50:30.749403
84560	HCAP	2020-10-05	3.51	3.52	3.4	3.4148	10192	{}	\N	2020-10-12 16:50:30.754327
84561	HCAP	2020-10-06	3.4929	3.5302	3.4301	3.4871	2883	{}	\N	2020-10-12 16:50:30.759414
84562	HCAP	2020-10-07	3.39	3.4	3.32	3.355	9996	{}	\N	2020-10-12 16:50:30.764557
84563	HCAP	2020-10-08	3.35	3.37	3.34	3.35	12558	{}	\N	2020-10-12 16:50:30.770065
84564	HCAP	2020-10-09	3.38	3.38	3.3401	3.35	10047	{}	\N	2020-10-12 16:50:30.775332
84565	HCAP	2020-10-12	3.38	3.39	3.31	3.36	8120	{}	\N	2020-10-12 16:50:30.781306
84566	HCM	2020-09-28	33.26	33.27	32.33	32.4	118289	{}	\N	2020-10-12 16:50:33.386898
84567	HCM	2020-09-29	32.47	32.47	31.64	31.75	135004	{}	\N	2020-10-12 16:50:33.399394
84568	HCM	2020-09-30	31.97	32.53	31.39	32.3	170069	{}	\N	2020-10-12 16:50:33.406954
84569	HCM	2020-10-01	32.5	32.96	31.91	32.71	85086	{}	\N	2020-10-12 16:50:33.412907
84570	HCM	2020-10-02	32.74	32.74	31.86	31.91	62956	{}	\N	2020-10-12 16:50:33.418341
84571	HCM	2020-10-05	31.94	32.81	31.6993	32.64	105579	{}	\N	2020-10-12 16:50:33.423152
84572	HCM	2020-10-06	32.69	34.41	32.69	34	186925	{}	\N	2020-10-12 16:50:33.427759
84573	HCM	2020-10-07	34.06	34.06	32.42	33.48	220530	{}	\N	2020-10-12 16:50:33.432513
84574	HCM	2020-10-08	33.65	34.46	33.02	34.1	93497	{}	\N	2020-10-12 16:50:33.437341
84575	HCM	2020-10-09	34.29	34.54	33.72	34.36	94412	{}	\N	2020-10-12 16:50:33.442298
84576	HCM	2020-10-12	34.6	34.6	34.06	34.22	37213	{}	\N	2020-10-12 16:50:33.447775
84577	HQY	2020-09-28	48.83	50.81	48.77	49.52	974353	{}	\N	2020-10-12 16:50:34.95157
84578	HQY	2020-09-29	49.36	50.27	48.77	49.46	514900	{}	\N	2020-10-12 16:50:34.964287
84579	HQY	2020-09-30	49.5	51.78	49.5	51.37	883034	{}	\N	2020-10-12 16:50:34.972938
84580	HQY	2020-10-01	51.46	53.23	51.42	51.86	686271	{}	\N	2020-10-12 16:50:34.979498
84581	HQY	2020-10-02	50.62	53.34	50	52.23	569059	{}	\N	2020-10-12 16:50:34.985638
84582	HQY	2020-10-05	52.71	53.83	52.0107	52.54	414704	{}	\N	2020-10-12 16:50:34.991524
84583	HQY	2020-10-06	53.01	54.41	52.65	53.02	585242	{}	\N	2020-10-12 16:50:34.996894
84584	HQY	2020-10-07	53.19	54.99	53.15	54.54	617544	{}	\N	2020-10-12 16:50:35.002408
84585	HQY	2020-10-08	54.99	55.7	54.4	54.82	338098	{}	\N	2020-10-12 16:50:35.008087
84586	HQY	2020-10-09	55.58	55.93	54.65	55.2	357920	{}	\N	2020-10-12 16:50:35.013551
84587	HQY	2020-10-12	54.82	57.74	54.598	56.89	380543	{}	\N	2020-10-12 16:50:35.019342
84588	HSBC	2020-09-28	19.8	19.99	19.64	19.96	6062303	{}	\N	2020-10-12 16:50:37.043118
84589	HSBC	2020-09-29	19.26	19.495	19.17	19.4	12828919	{}	\N	2020-10-12 16:50:37.057877
84590	HSBC	2020-09-30	19.53	19.71	19.49	19.58	8555503	{}	\N	2020-10-12 16:50:37.065091
84591	HSBC	2020-10-01	19.64	19.6842	19.37	19.51	5682153	{}	\N	2020-10-12 16:50:37.072596
84592	HSBC	2020-10-02	19.54	20.06	19.53	19.87	9739265	{}	\N	2020-10-12 16:50:37.078023
84593	HSBC	2020-10-05	19.91	20.09	19.845	19.96	5356681	{}	\N	2020-10-12 16:50:37.082583
84594	HSBC	2020-10-06	20.23	20.355	19.94	19.99	7769152	{}	\N	2020-10-12 16:50:37.087195
84595	HSBC	2020-10-07	20.31	20.44	20.235	20.37	5031490	{}	\N	2020-10-12 16:50:37.09227
84596	HSBC	2020-10-08	20.32	20.4	20.21	20.32	4709674	{}	\N	2020-10-12 16:50:37.097235
84597	HSBC	2020-10-09	20.32	20.3497	20.05	20.13	4266470	{}	\N	2020-10-12 16:50:37.102401
84598	HSBC	2020-10-12	20.25	20.25	20.09	20.1662	1857761	{}	\N	2020-10-12 16:50:37.109039
84599	HTA	2020-09-28	25.9	26.1	25.74	26.01	1212748	{}	\N	2020-10-12 16:50:41.434229
84600	HTA	2020-09-29	25.85	26.08	25.43	25.91	2835509	{}	\N	2020-10-12 16:50:41.447407
84601	HTA	2020-09-30	26.05	26.42	25.745	26	3135009	{}	\N	2020-10-12 16:50:41.455075
84602	HTA	2020-10-01	25.67	26.36	25.62	26.35	3934774	{}	\N	2020-10-12 16:50:41.461019
84603	HTA	2020-10-02	25.99	27.08	25.77	26.8	4608715	{}	\N	2020-10-12 16:50:41.466344
84604	HTA	2020-10-05	27	27.09	26.32	26.48	2271835	{}	\N	2020-10-12 16:50:41.471195
84605	HTA	2020-10-06	26.5	26.81	26.15	26.41	3189562	{}	\N	2020-10-12 16:50:41.476207
84606	HTA	2020-10-07	26.45	26.57	26.005	26.08	4725193	{}	\N	2020-10-12 16:50:41.481891
84607	HTA	2020-10-08	26.22	26.62	26.06	26.46	2407025	{}	\N	2020-10-12 16:50:41.486694
84608	HTA	2020-10-09	26.65	26.7	25.99	26.09	2292711	{}	\N	2020-10-12 16:50:41.491886
84609	HTA	2020-10-12	26.05	26.25	25.86	26.06	519868	{}	\N	2020-10-12 16:50:41.495904
84610	HTRB	2020-09-28	41.485	41.59	41.455	41.56	50099	{}	\N	2020-10-12 16:50:43.741082
84611	HTRB	2020-09-29	41.36	41.5099	41.36	41.4291	37192	{}	\N	2020-10-12 16:50:43.75701
84612	HTRB	2020-09-30	41.45	41.45	41.3	41.36	119270	{}	\N	2020-10-12 16:50:43.764131
84613	HTRB	2020-10-01	41.37	41.47	41.3439	41.46	84320	{}	\N	2020-10-12 16:50:43.770617
84614	HTRB	2020-10-02	41.61	41.61	41.38	41.44	21055	{}	\N	2020-10-12 16:50:43.776557
84615	HTRB	2020-10-05	41.35	41.4	41.28	41.3	21543	{}	\N	2020-10-12 16:50:43.78257
84616	HTRB	2020-10-06	41.49	41.49	41.295	41.43	23297	{}	\N	2020-10-12 16:50:43.788701
84617	HTRB	2020-10-07	41.3279	41.4	41.325	41.35	50938	{}	\N	2020-10-12 16:50:43.794513
84618	HTRB	2020-10-08	41.35	41.43	41.3368	41.43	12159	{}	\N	2020-10-12 16:50:43.800084
84619	HTRB	2020-10-09	41.3627	41.42	41.3627	41.4	22355	{}	\N	2020-10-12 16:50:43.805287
84620	HTRB	2020-10-12	41.33	41.46	41.33	41.42	7386	{}	\N	2020-10-12 16:50:43.810412
84621	HYGH	2020-09-28	82.28	82.43	82.22	82.38	4558	{}	\N	2020-10-12 16:50:44.87983
84622	HYGH	2020-09-29	82.4	82.4	82.11	82.22	2240	{}	\N	2020-10-12 16:50:44.892123
84623	HYGH	2020-09-30	82.7671	82.8015	82.7671	82.8015	997	{}	\N	2020-10-12 16:50:44.900798
84624	HYGH	2020-10-01	83.01	83.05	82.98	83.04	2379	{}	\N	2020-10-12 16:50:44.908248
84625	HYGH	2020-10-02	82.7	82.83	82.615	82.83	1144	{}	\N	2020-10-12 16:50:44.915382
84626	HYGH	2020-10-05	83.07	83.4779	83.07	83.4779	2798	{}	\N	2020-10-12 16:50:44.921983
84627	HYGH	2020-10-06	83.5	83.77	83.3006	83.3006	2675	{}	\N	2020-10-12 16:50:44.928117
84628	HYGH	2020-10-07	83.54	83.9	83.54	83.9	10075	{}	\N	2020-10-12 16:50:44.933464
84629	HYGH	2020-10-08	83.87	83.87	83.7708	83.7708	1235	{}	\N	2020-10-12 16:50:44.938457
84630	HYGH	2020-10-09	83.78	84.0188	83.76	84.0188	2717	{}	\N	2020-10-12 16:50:44.943255
84631	HYGH	2020-10-12	84.2	84.38	84.2	84.38	1101	{}	\N	2020-10-12 16:50:44.948629
84632	ITM	2020-09-28	51.29	51.41	51.29	51.39	34751	{}	\N	2020-10-12 16:50:46.708277
84633	ITM	2020-09-29	51.4	51.44	51.29	51.44	127374	{}	\N	2020-10-12 16:50:46.721089
84634	ITM	2020-09-30	51.4	51.4095	51.2501	51.275	114094	{}	\N	2020-10-12 16:50:46.727627
84635	ITM	2020-10-01	51.14	51.26	51.12	51.17	193197	{}	\N	2020-10-12 16:50:46.733462
84636	ITM	2020-10-02	51.195	51.26	51.08	51.11	65133	{}	\N	2020-10-12 16:50:46.738495
84637	ITM	2020-10-05	51.11	51.23	51.03	51.04	61625	{}	\N	2020-10-12 16:50:46.7436
84638	ITM	2020-10-06	51.14	51.19	51.03	51.03	57859	{}	\N	2020-10-12 16:50:46.747806
84639	ITM	2020-10-07	51.06	51.09	51	51.01	47248	{}	\N	2020-10-12 16:50:46.755123
84640	ITM	2020-10-08	50.95	51.04	50.89	51.02	104647	{}	\N	2020-10-12 16:50:46.759394
84641	ITM	2020-10-09	51.02	51.05	50.9112	50.99	139702	{}	\N	2020-10-12 16:50:46.764246
84642	ITM	2020-10-12	50.89	51.038	50.89	51.0034	22226	{}	\N	2020-10-12 16:50:46.769948
84643	JCE	2020-09-28	12.61	12.6703	12.59	12.6	64253	{}	\N	2020-10-12 16:50:49.993594
84644	JCE	2020-09-29	12.66	12.67	12.53	12.59	95902	{}	\N	2020-10-12 16:50:50.005371
84645	JCE	2020-09-30	12.59	12.82	12.58	12.8	106836	{}	\N	2020-10-12 16:50:50.012446
84646	JCE	2020-10-01	12.89	12.96	12.75	12.84	147941	{}	\N	2020-10-12 16:50:50.018897
84647	JCE	2020-10-02	12.75	12.83	12.62	12.81	128758	{}	\N	2020-10-12 16:50:50.025543
84648	JCE	2020-10-05	12.88	13	12.82	13	129671	{}	\N	2020-10-12 16:50:50.031202
84649	JCE	2020-10-06	13.07	13.09	12.86	12.92	146114	{}	\N	2020-10-12 16:50:50.0368
84650	JCE	2020-10-07	13.11	13.18	13.035	13.18	96727	{}	\N	2020-10-12 16:50:50.042313
84651	JCE	2020-10-08	13.34	13.34	13.1119	13.23	95093	{}	\N	2020-10-12 16:50:50.047508
84652	JCE	2020-10-09	13.26	13.261	13.12	13.15	94918	{}	\N	2020-10-12 16:50:50.052987
84653	JCE	2020-10-12	13.17	13.2654	13.17	13.23	42806	{}	\N	2020-10-12 16:50:50.057982
84654	GHC	2020-09-28	417.4	417.4	403.46	404.94	29836	{}	\N	2020-10-12 16:50:52.137203
84655	GHC	2020-09-29	402	402.76	393.54	396.8	50718	{}	\N	2020-10-12 16:50:52.149883
84656	GHC	2020-09-30	396	410.97	396	404.11	79346	{}	\N	2020-10-12 16:50:52.156433
84657	GHC	2020-10-01	406.37	406.795	398.894	402.32	46964	{}	\N	2020-10-12 16:50:52.16221
84658	GHC	2020-10-02	394.39	415.3	393.93	408.14	60296	{}	\N	2020-10-12 16:50:52.167517
84659	GHC	2020-10-05	413.95	413.95	404.07	409.68	52293	{}	\N	2020-10-12 16:50:52.17252
84660	GHC	2020-10-06	411.82	420.92	409.95	414.61	42033	{}	\N	2020-10-12 16:50:52.176749
84661	GHC	2020-10-07	416.01	416.01	407.99	407.99	34248	{}	\N	2020-10-12 16:50:52.18128
84662	GHC	2020-10-08	411.09	415.14	408.6782	414.57	42661	{}	\N	2020-10-12 16:50:52.185957
84663	GHC	2020-10-09	414.86	416.3	413.37	413.62	21806	{}	\N	2020-10-12 16:50:52.190688
84664	GHC	2020-10-12	415	418.54	415	417.49	6454	{}	\N	2020-10-12 16:50:52.195091
84665	GLD	2020-09-28	175.83	176.805	175.09	176.7	7149944	{}	\N	2020-10-12 16:50:54.940091
84666	GLD	2020-09-29	177.18	178.33	176.95	178.19	8573230	{}	\N	2020-10-12 16:50:54.949212
84667	GLD	2020-09-30	177.71	178.61	176.66	177.12	8351316	{}	\N	2020-10-12 16:50:54.955822
84668	GLD	2020-10-01	178.69	179.5	178.0375	178.7	10382775	{}	\N	2020-10-12 16:50:54.960825
84669	GLD	2020-10-02	178.71	179.29	178.16	178.54	6998556	{}	\N	2020-10-12 16:50:54.965259
84670	GLD	2020-10-05	179.02	180.14	178.9	179.41	6965928	{}	\N	2020-10-12 16:50:54.970341
84671	GLD	2020-10-06	180.16	180.37	177.0513	177.3	8471105	{}	\N	2020-10-12 16:50:54.975272
84672	GLD	2020-10-07	177.09	177.42	176.43	177.22	6416952	{}	\N	2020-10-12 16:50:54.979992
84673	GLD	2020-10-08	178.18	178.45	176.67	177.85	6614371	{}	\N	2020-10-12 16:50:54.985737
84674	GLD	2020-10-09	180.13	181.17	179.93	181.08	15943307	{}	\N	2020-10-12 16:50:54.989893
84675	GLD	2020-10-12	180.57	180.83	180.19	180.7798	3038104	{}	\N	2020-10-12 16:50:54.994149
84676	GNMA	2020-09-28	50.94	51.13	50.94	51.07	48596	{}	\N	2020-10-12 16:50:58.433104
84677	GNMA	2020-09-29	51.069	51.11	50.99	51.06	53303	{}	\N	2020-10-12 16:50:58.444329
84678	GNMA	2020-09-30	51.12	51.12	51.0643	51.11	34183	{}	\N	2020-10-12 16:50:58.451275
84679	GNMA	2020-10-01	51.16	51.18	50.8408	51.03	124632	{}	\N	2020-10-12 16:50:58.45716
84680	GNMA	2020-10-02	50.95	51.06	50.95	50.95	42877	{}	\N	2020-10-12 16:50:58.462413
84681	GNMA	2020-10-05	50.99	51.0045	50.86	50.87	50110	{}	\N	2020-10-12 16:50:58.467492
84682	GNMA	2020-10-06	50.93	51.05	50.8401	51.04	48583	{}	\N	2020-10-12 16:50:58.472048
84683	GNMA	2020-10-07	51.02	51.07	51.006	51.02	51729	{}	\N	2020-10-12 16:50:58.476656
84684	GNMA	2020-10-08	50.6	51.08	50.6	51	31639	{}	\N	2020-10-12 16:50:58.481252
84685	GNMA	2020-10-09	50.98	51.03	50.91	50.97	58072	{}	\N	2020-10-12 16:50:58.485814
84686	GNMA	2020-10-12	51.03	51.0793	50.928	51.0184	40900	{}	\N	2020-10-12 16:50:58.490443
84687	GPRO	2020-09-28	4.37	4.49	4.34	4.47	2091404	{}	\N	2020-10-12 16:50:59.836698
84688	GPRO	2020-09-29	4.46	4.645	4.45	4.615	2484467	{}	\N	2020-10-12 16:50:59.843948
84689	GPRO	2020-09-30	4.62	4.66	4.49	4.53	2181916	{}	\N	2020-10-12 16:50:59.849974
84690	GPRO	2020-10-01	4.56	4.62	4.45	4.62	1682313	{}	\N	2020-10-12 16:50:59.855297
84691	GPRO	2020-10-02	4.46	4.6	4.45	4.56	1081196	{}	\N	2020-10-12 16:50:59.86051
84692	GPRO	2020-10-05	4.62	4.73	4.57	4.69	1271334	{}	\N	2020-10-12 16:50:59.865486
84693	GPRO	2020-10-06	4.75	5.09	4.72	4.965	4487182	{}	\N	2020-10-12 16:50:59.870615
84694	GPRO	2020-10-07	5.01	6.47	5.01	6.41	22500642	{}	\N	2020-10-12 16:50:59.876858
84695	GPRO	2020-10-08	6.33	6.53	5.98	6.21	11456527	{}	\N	2020-10-12 16:50:59.882738
84696	GPRO	2020-10-09	6.3	6.65	6.07	6.18	6910321	{}	\N	2020-10-12 16:50:59.888207
84697	GPRO	2020-10-12	6.2	6.72	6.14	6.6684	4604950	{}	\N	2020-10-12 16:50:59.894855
84698	HBP	2020-09-28	2.14	2.23	2.14	2.19	13902	{}	\N	2020-10-12 16:51:04.958538
84699	HBP	2020-09-29	2.17	2.25	2.1601	2.18	10076	{}	\N	2020-10-12 16:51:04.967837
84700	HBP	2020-09-30	2.23	2.253	2.1525	2.19	6964	{}	\N	2020-10-12 16:51:04.974289
84701	HBP	2020-10-01	2.26	2.293	2.2	2.245	21597	{}	\N	2020-10-12 16:51:04.980828
84702	HBP	2020-10-02	2.17	2.28	2.17	2.24	6025	{}	\N	2020-10-12 16:51:04.98663
84703	HBP	2020-10-05	2.265	2.3	2.26	2.26	17425	{}	\N	2020-10-12 16:51:04.991648
84704	HBP	2020-10-06	2.25	2.3	2.19	2.19	14336	{}	\N	2020-10-12 16:51:04.99724
84705	HBP	2020-10-07	2.2	2.32	2.2	2.23	8234	{}	\N	2020-10-12 16:51:05.0032
84706	HBP	2020-10-08	2.25	2.3	2.22	2.26	19055	{}	\N	2020-10-12 16:51:05.008526
84707	HBP	2020-10-09	2.3	2.45	2.27	2.4	33836	{}	\N	2020-10-12 16:51:05.013353
84708	HBP	2020-10-12	2.45	2.48	2.3601	2.3768	7518	{}	\N	2020-10-12 16:51:05.018447
84709	HCKT	2020-09-28	11.87	11.905	11.42	11.65	128352	{}	\N	2020-10-12 16:51:07.832144
84710	HCKT	2020-09-29	11.59	11.655	11.45	11.58	109008	{}	\N	2020-10-12 16:51:07.849528
84711	HCKT	2020-09-30	11.61	11.72	11.165	11.18	145915	{}	\N	2020-10-12 16:51:07.858081
84712	HCKT	2020-10-01	11.21	11.44	11.05	11.39	141518	{}	\N	2020-10-12 16:51:07.865186
84713	HCKT	2020-10-02	11.11	11.47	11.11	11.38	132371	{}	\N	2020-10-12 16:51:07.871307
84714	HCKT	2020-10-05	11.49	11.68	11.42	11.61	100156	{}	\N	2020-10-12 16:51:07.87655
84715	HCKT	2020-10-06	11.65	12.02	11.44	11.63	110941	{}	\N	2020-10-12 16:51:07.882247
84716	HCKT	2020-10-07	11.81	12.18	11.71	11.93	142154	{}	\N	2020-10-12 16:51:07.887797
84717	HCKT	2020-10-08	11.98	12.32	11.77	12.21	187267	{}	\N	2020-10-12 16:51:07.892688
84718	HCKT	2020-10-09	12.26	12.41	11.91	12.13	95836	{}	\N	2020-10-12 16:51:07.899353
84719	HCKT	2020-10-12	12.12	12.45	12.02	12.4036	46236	{}	\N	2020-10-12 16:51:07.904569
84720	IBDD	2020-09-28	27.29	27.325	27.2804	27.325	1028	{}	\N	2020-10-12 16:51:10.297501
84721	IBDD	2020-09-29	27.37	27.37	27.3	27.37	9631	{}	\N	2020-10-12 16:51:10.31024
84722	IBDD	2020-09-30	27.31	27.35	27.3	27.31	12838	{}	\N	2020-10-12 16:51:10.319498
84723	IBDD	2020-10-01	27.51	27.51	27.28	27.3046	2129	{}	\N	2020-10-12 16:51:10.328586
84724	IBDD	2020-10-02	27.2913	27.3	27.25	27.25	2410	{}	\N	2020-10-12 16:51:10.337888
84725	IBDD	2020-10-05	27.29	27.3101	27.29	27.3101	697	{}	\N	2020-10-12 16:51:10.347007
84726	IBDD	2020-10-06	27.36	27.36	27.3	27.35	4786	{}	\N	2020-10-12 16:51:10.355535
84727	IBDD	2020-10-07	27.34	27.39	27.3044	27.325	14365	{}	\N	2020-10-12 16:51:10.361745
84728	IBDD	2020-10-08	27.389	27.3899	27.31	27.31	5553	{}	\N	2020-10-12 16:51:10.367116
84729	IBDD	2020-10-09	27.3393	27.379	27.3006	27.3006	2934	{}	\N	2020-10-12 16:51:10.372273
84730	IBDD	2020-10-12	27.375	27.375	27.3	27.3	2071	{}	\N	2020-10-12 16:51:10.378435
84731	IBUY	2020-09-28	88.33	89.14	87.741	88.86	160334	{}	\N	2020-10-12 16:51:12.330653
84732	IBUY	2020-09-29	88.83	89.27	88.2293	88.72	98773	{}	\N	2020-10-12 16:51:12.341303
84733	IBUY	2020-09-30	88.93	89.86	87.7489	88.23	145919	{}	\N	2020-10-12 16:51:12.348704
84734	IBUY	2020-10-01	89.15	91.86	89.15	91.86	183652	{}	\N	2020-10-12 16:51:12.355446
84735	IBUY	2020-10-02	89.47	91.075	89.2703	90.46	304904	{}	\N	2020-10-12 16:51:12.361344
84736	IBUY	2020-10-05	91.08	92.29	90.8795	92.16	208961	{}	\N	2020-10-12 16:51:12.366371
84737	IBUY	2020-10-06	92.22	93.085	90.401	90.67	180893	{}	\N	2020-10-12 16:51:12.3714
84738	IBUY	2020-10-07	91.97	93.62	91.97	93.47	228118	{}	\N	2020-10-12 16:51:12.37597
84739	IBUY	2020-10-08	94.35	94.45	93.45	93.96	97593	{}	\N	2020-10-12 16:51:12.380531
84740	IBUY	2020-10-09	94.52	95.44	94.38	95.27	114155	{}	\N	2020-10-12 16:51:12.385194
84741	IBUY	2020-10-12	96.44	97.29	95.83	97.29	102143	{}	\N	2020-10-12 16:51:12.390175
84742	IJJ	2020-09-28	133.56	135.515	133.56	135.03	66377	{}	\N	2020-10-12 16:51:19.297528
84743	IJJ	2020-09-29	135.1	135.1	132.65	133.78	71555	{}	\N	2020-10-12 16:51:19.30871
84744	IJJ	2020-09-30	134.23	136.24	133.92	134.86	71455	{}	\N	2020-10-12 16:51:19.316883
84745	IJJ	2020-10-01	135.43	136.54	134.7443	136.54	65511	{}	\N	2020-10-12 16:51:19.322091
84746	IJJ	2020-10-02	133.93	139.2472	133.93	138.61	88959	{}	\N	2020-10-12 16:51:19.327624
84747	IJJ	2020-10-05	139.95	142.19	139.95	142.15	72405	{}	\N	2020-10-12 16:51:19.332354
84748	IJJ	2020-10-06	142.97	145.36	141.28	141.65	158773	{}	\N	2020-10-12 16:51:19.337854
84749	IJJ	2020-10-07	143.14	144.58	143.14	144.05	59714	{}	\N	2020-10-12 16:51:19.342843
84750	IJJ	2020-10-08	145.1	145.72	144.3382	145.68	51831	{}	\N	2020-10-12 16:51:19.347797
84751	IJJ	2020-10-09	146.75	146.75	145.14	145.44	43887	{}	\N	2020-10-12 16:51:19.353011
84752	IJJ	2020-10-12	145.93	146.7203	145.54	146.43	36197	{}	\N	2020-10-12 16:51:19.357401
84753	GHYB	2020-09-28	48.15	48.25	48.08	48.16	28596	{}	\N	2020-10-12 16:51:23.17294
84754	GHYB	2020-09-29	48.16	48.32	48.1206	48.32	295580	{}	\N	2020-10-12 16:51:23.187831
84755	GHYB	2020-09-30	48.44	48.46	48.36	48.3799	110600	{}	\N	2020-10-12 16:51:23.197453
84756	GHYB	2020-10-01	48.21	48.4099	48.21	48.37	2668	{}	\N	2020-10-12 16:51:23.203025
84757	GHYB	2020-10-02	48.18	48.36	48.08	48.12	116551	{}	\N	2020-10-12 16:51:23.208114
84758	GHYB	2020-10-05	48.48	48.6099	48.48	48.5999	6921	{}	\N	2020-10-12 16:51:23.213486
84759	GHYB	2020-10-06	48.61	48.7732	48.5345	48.5345	103811	{}	\N	2020-10-12 16:51:23.218454
84760	GHYB	2020-10-07	48.76	48.85	48.73	48.73	18793	{}	\N	2020-10-12 16:51:23.22333
84761	GHYB	2020-10-08	48.8174	48.94	48.8174	48.93	3336	{}	\N	2020-10-12 16:51:23.227778
84762	GHYB	2020-10-09	48.92	49	48.92	48.98	3381	{}	\N	2020-10-12 16:51:23.233409
84763	GHYB	2020-10-12	49.095	49.3	49.095	49.3	6478	{}	\N	2020-10-12 16:51:23.237944
84764	GLMD	2020-09-28	3.59	3.6916	3.52	3.54	51439	{}	\N	2020-10-12 16:51:24.476463
84765	GLMD	2020-09-29	3.73	3.73	3.46	3.46	68327	{}	\N	2020-10-12 16:51:24.483229
84766	GLMD	2020-09-30	3.5	3.55	3.36	3.53	266653	{}	\N	2020-10-12 16:51:24.487851
84767	GLMD	2020-10-01	3.55	3.6234	3.51	3.57	50062	{}	\N	2020-10-12 16:51:24.49333
84768	GLMD	2020-10-02	3.55	3.62	3.49	3.61	38966	{}	\N	2020-10-12 16:51:24.498684
84769	GLMD	2020-10-05	3.63	3.69	3.55	3.58	70121	{}	\N	2020-10-12 16:51:24.503323
84770	GLMD	2020-10-06	3.64	3.64	3.28	3.36	90108	{}	\N	2020-10-12 16:51:24.509413
84771	GLMD	2020-10-07	3.44	3.74	3.4	3.69	129279	{}	\N	2020-10-12 16:51:24.517527
84772	GLMD	2020-10-08	3.75	3.853	3.5	3.53	107293	{}	\N	2020-10-12 16:51:24.528088
84773	GLMD	2020-10-09	3.7003	3.7003	3.55	3.55	43354	{}	\N	2020-10-12 16:51:24.539363
84774	GLMD	2020-10-12	3.6	3.7172	3.43	3.45	43795	{}	\N	2020-10-12 16:51:24.548001
84775	GPX	2020-09-28	10	10.685	9.86	10.63	50603	{}	\N	2020-10-12 16:51:26.490479
84776	GPX	2020-09-29	10.63	10.7433	9.44	9.5	70045	{}	\N	2020-10-12 16:51:26.503648
84777	GPX	2020-09-30	9.55	9.88	9.52	9.64	71165	{}	\N	2020-10-12 16:51:26.512636
84778	GPX	2020-10-01	9.7	9.72	9.33	9.54	89556	{}	\N	2020-10-12 16:51:26.518673
84779	GPX	2020-10-02	9.36	9.79	8.91	9.36	198571	{}	\N	2020-10-12 16:51:26.523769
84780	GPX	2020-10-05	9.4	10.3	9.4	10.24	75694	{}	\N	2020-10-12 16:51:26.528689
84781	GPX	2020-10-06	10.36	10.555	10.01	10.35	41079	{}	\N	2020-10-12 16:51:26.534891
84782	GPX	2020-10-07	10.28	10.7	10.06	10.6	36270	{}	\N	2020-10-12 16:51:26.540272
84783	GPX	2020-10-08	10.76	11.17	10.4275	11.05	42714	{}	\N	2020-10-12 16:51:26.544985
84784	GPX	2020-10-09	11.1	11.1	10.65	10.75	28070	{}	\N	2020-10-12 16:51:26.549829
84785	GPX	2020-10-12	10.65	10.89	10.4	10.83	9220	{}	\N	2020-10-12 16:51:26.55539
84786	GRES	2020-09-28	25.7999	25.8	25.72	25.8	2936	{}	\N	2020-10-12 16:51:29.99093
84787	GRES	2020-09-29	25.6901	25.6901	25.52	25.6599	651	{}	\N	2020-10-12 16:51:30.005875
84788	GRES	2020-09-30	25.67	25.72	25.57	25.581	2957	{}	\N	2020-10-12 16:51:30.014359
84789	GRES	2020-10-01	25.61	25.66	25.52	25.66	1673	{}	\N	2020-10-12 16:51:30.019966
84790	GRES	2020-10-02	25.28	25.666	25.28	25.666	3821	{}	\N	2020-10-12 16:51:30.026082
84791	GRES	2020-10-05	24.43	25.84	24.43	25.8263	118098	{}	\N	2020-10-12 16:51:30.031288
84792	GRES	2020-10-06	25.86	25.86	25.5148	25.5148	1482	{}	\N	2020-10-12 16:51:30.036219
84793	GRES	2020-10-07	25.78	25.7899	25.59	25.76	4087	{}	\N	2020-10-12 16:51:30.041753
84794	GRES	2020-10-08	25.84	25.9171	25.75	25.9171	1907	{}	\N	2020-10-12 16:51:30.046994
84795	GRES	2020-10-09	26.08	26.08	26	26.01	800	{}	\N	2020-10-12 16:51:30.051816
84796	GRES	2020-10-12	26.09	26.13	26.09	26.13	347	{}	\N	2020-10-12 16:51:30.058583
84797	HBMD	2020-09-28	9.1399	9.21	8.98	9.02	16537	{}	\N	2020-10-12 16:51:32.538341
84798	HBMD	2020-09-29	8.81	8.9558	8.81	8.95	7969	{}	\N	2020-10-12 16:51:32.5526
84799	HBMD	2020-09-30	9.04	9.04	8.88	8.95	10200	{}	\N	2020-10-12 16:51:32.560963
84800	HBMD	2020-10-01	8.97	9.16	8.97	9.1	12158	{}	\N	2020-10-12 16:51:32.567381
84801	HBMD	2020-10-02	9.1	9.33	9.1	9.3	20792	{}	\N	2020-10-12 16:51:32.573411
84802	HBMD	2020-10-05	9.49	9.575	9.31	9.51	20881	{}	\N	2020-10-12 16:51:32.578577
84803	HBMD	2020-10-06	9.47	9.78	9.2201	9.51	19144	{}	\N	2020-10-12 16:51:32.584064
84804	HBMD	2020-10-07	9.76	9.95	9.67	9.91	19617	{}	\N	2020-10-12 16:51:32.588933
84805	HBMD	2020-10-08	9.41	10.05	9.41	9.96	34322	{}	\N	2020-10-12 16:51:32.594057
84806	HBMD	2020-10-09	9.985	9.99	9.89	9.92	4306	{}	\N	2020-10-12 16:51:32.599763
84807	HBMD	2020-10-12	9.725	10	9.725	9.88	1973	{}	\N	2020-10-12 16:51:32.604907
84808	HYS	2020-09-28	93.27	93.57	93.1001	93.31	660574	{}	\N	2020-10-12 16:51:35.10057
84809	HYS	2020-09-29	93.26	93.88	93.26	93.87	125485	{}	\N	2020-10-12 16:51:35.11214
84810	HYS	2020-09-30	93.6	94.12	93.6	93.82	96103	{}	\N	2020-10-12 16:51:35.120456
84811	HYS	2020-10-01	93.8	93.98	93.34	93.95	265836	{}	\N	2020-10-12 16:51:35.126983
84812	HYS	2020-10-02	93.36	94.02	93.3157	93.84	376996	{}	\N	2020-10-12 16:51:35.133181
84813	HYS	2020-10-05	93.9334	94.47	93.88	94.15	240495	{}	\N	2020-10-12 16:51:35.138601
84814	HYS	2020-10-06	94.46	94.77	94	94.23	446790	{}	\N	2020-10-12 16:51:35.144126
84815	HYS	2020-10-07	94.49	94.57	94.2	94.45	227187	{}	\N	2020-10-12 16:51:35.149517
84816	HYS	2020-10-08	94.64	94.68	94.37	94.51	173430	{}	\N	2020-10-12 16:51:35.155412
84817	HYS	2020-10-09	94.78	94.9199	94.6208	94.85	139224	{}	\N	2020-10-12 16:51:35.161291
84818	HYS	2020-10-12	95.09	95.33	95.02	95.29	25925	{}	\N	2020-10-12 16:51:35.166725
84819	IFV	2020-09-28	18.69	18.7146	18.57	18.58	20601	{}	\N	2020-10-12 16:51:37.293215
84820	IFV	2020-09-29	18.6347	18.72	18.5401	18.72	19026	{}	\N	2020-10-12 16:51:37.305286
84821	IFV	2020-09-30	18.74	18.89	18.7	18.79	49713	{}	\N	2020-10-12 16:51:37.312908
84822	IFV	2020-10-01	18.95	19.0682	18.931	19	63265	{}	\N	2020-10-12 16:51:37.319697
84823	IFV	2020-10-02	18.75	19.02	18.75	18.905	8170	{}	\N	2020-10-12 16:51:37.326161
84824	IFV	2020-10-05	19	19.16	19	19.13	7644	{}	\N	2020-10-12 16:51:37.331574
84825	IFV	2020-10-06	19.23	19.3371	19.0764	19.09	9745	{}	\N	2020-10-12 16:51:37.336806
84826	IFV	2020-10-07	19.34	19.3474	19.2101	19.31	17438	{}	\N	2020-10-12 16:51:37.342134
84827	IFV	2020-10-08	19.4	19.55	19.39	19.46	35715	{}	\N	2020-10-12 16:51:37.347791
84828	IFV	2020-10-09	19.61	19.6738	19.5665	19.67	11570	{}	\N	2020-10-12 16:51:37.353264
84829	IFV	2020-10-12	19.78	19.8238	19.715	19.8	5835	{}	\N	2020-10-12 16:51:37.358687
84830	IMTB	2020-09-28	52.36	52.36	52.2723	52.3064	10034	{}	\N	2020-10-12 16:51:39.207815
84831	IMTB	2020-09-29	52.45	52.45	52.3001	52.3459	7327	{}	\N	2020-10-12 16:51:39.216174
84832	IMTB	2020-09-30	52.395	52.3995	52.35	52.35	5923	{}	\N	2020-10-12 16:51:39.223132
84833	IMTB	2020-10-01	52.17	52.35	52.17	52.35	3947	{}	\N	2020-10-12 16:51:39.229884
84834	IMTB	2020-10-02	52.31	52.343	52.2214	52.343	7936	{}	\N	2020-10-12 16:51:39.235867
84835	IMTB	2020-10-05	52.41	52.41	52.25	52.25	15769	{}	\N	2020-10-12 16:51:39.241324
84836	IMTB	2020-10-06	52.3	52.3	52.22	52.26	9787	{}	\N	2020-10-12 16:51:39.246732
84837	IMTB	2020-10-07	52.26	52.305	52.2	52.2492	62948	{}	\N	2020-10-12 16:51:39.252097
84838	IMTB	2020-10-08	52.24	52.3271	52.24	52.24	3082	{}	\N	2020-10-12 16:51:39.257509
84839	IMTB	2020-10-09	52.29	52.37	52.25	52.2656	13609	{}	\N	2020-10-12 16:51:39.262819
84840	IMTB	2020-10-12	52.38	52.41	52.3017	52.33	5406	{}	\N	2020-10-12 16:51:39.268364
84841	IQDY	2020-09-28	23.18	23.18	23.1366	23.1366	1215	{}	\N	2020-10-12 16:51:40.668884
84842	IQDY	2020-09-29	23.0895	23.19	23.0895	23.17	7717	{}	\N	2020-10-12 16:51:40.679755
84843	IQDY	2020-09-30	23.2355	23.2355	23.0509	23.0509	24743	{}	\N	2020-10-12 16:51:40.688961
84844	IQDY	2020-10-01	23.13	23.244	23.13	23.244	38251	{}	\N	2020-10-12 16:51:40.694964
84845	IQDY	2020-10-02	23.24	23.24	23.2373	23.2373	164	{}	\N	2020-10-12 16:51:40.700528
84846	IQDY	2020-10-05	23.38	23.5172	23.38	23.5172	1938	{}	\N	2020-10-12 16:51:40.705633
84847	IQDY	2020-10-06	23.5455	23.5455	23.3424	23.3424	2316	{}	\N	2020-10-12 16:51:40.710586
84848	IQDY	2020-10-07	23.5037	23.5037	23.5037	23.5037	13	{}	\N	2020-10-12 16:51:40.715654
84849	IQDY	2020-10-08	23.65	23.72	23.65	23.72	518	{}	\N	2020-10-12 16:51:40.721646
84850	IQDY	2020-10-09	23.9	23.93	23.852	23.852	6304	{}	\N	2020-10-12 16:51:40.727957
84851	IQDY	2020-10-12	23.9795	23.9795	23.9795	23.9795	151	{}	\N	2020-10-12 16:51:40.733785
84852	GHYG	2020-09-28	47.58	47.7851	47.58	47.71	4349	{}	\N	2020-10-12 16:51:42.403213
84853	GHYG	2020-09-29	47.695	47.8425	47.68	47.8425	4329	{}	\N	2020-10-12 16:51:42.417293
84854	GHYG	2020-09-30	47.85	48.0522	47.85	47.98	3101	{}	\N	2020-10-12 16:51:42.42784
84855	GHYG	2020-10-01	47.93	48.19	47.86	48.18	28783	{}	\N	2020-10-12 16:51:42.434117
84856	GHYG	2020-10-02	47.96	48.05	47.905	48.05	7434	{}	\N	2020-10-12 16:51:42.438769
84857	GHYG	2020-10-05	48.01	48.36	48.01	48.32	8177	{}	\N	2020-10-12 16:51:42.443909
84858	GHYG	2020-10-06	48.51	48.55	48.271	48.36	17302	{}	\N	2020-10-12 16:51:42.44889
84859	GHYG	2020-10-07	48.21	48.52	48.21	48.47	13109	{}	\N	2020-10-12 16:51:42.453862
84860	GHYG	2020-10-08	48.51	48.56	48.4	48.5	5698	{}	\N	2020-10-12 16:51:42.458839
84861	GHYG	2020-10-09	48.55	48.71	48.531	48.71	4774	{}	\N	2020-10-12 16:51:42.464122
84862	GHYG	2020-10-12	48.768	48.9	48.768	48.8756	2078	{}	\N	2020-10-12 16:51:42.469453
84863	GQRE	2020-09-28	52.25	52.65	52.25	52.575	11483	{}	\N	2020-10-12 16:51:46.246016
84864	GQRE	2020-09-29	52.44	52.44	51.96	52.15	15355	{}	\N	2020-10-12 16:51:46.261416
84865	GQRE	2020-09-30	52.6	52.65	52.2	52.2	2278	{}	\N	2020-10-12 16:51:46.26936
84866	GQRE	2020-10-01	52.84	53.2883	52.59	53.2883	14085	{}	\N	2020-10-12 16:51:46.276162
84867	GQRE	2020-10-02	52.7	54.02	52.7	53.94	35017	{}	\N	2020-10-12 16:51:46.281879
84868	GQRE	2020-10-05	54.21	54.3656	54.06	54.3656	2257	{}	\N	2020-10-12 16:51:46.287985
84869	GQRE	2020-10-06	54.285	54.59	53.99	53.99	9000	{}	\N	2020-10-12 16:51:46.293178
84870	GQRE	2020-10-07	54.12	54.2461	53.88	54.0772	7554	{}	\N	2020-10-12 16:51:46.298281
84871	GQRE	2020-10-08	54.49	54.62	54.36	54.62	11052	{}	\N	2020-10-12 16:51:46.303268
84872	GQRE	2020-10-09	54.47	54.67	54.43	54.5	10718	{}	\N	2020-10-12 16:51:46.308506
84873	GQRE	2020-10-12	54.6	54.7528	54.6	54.7528	3493	{}	\N	2020-10-12 16:51:46.313128
84874	HCHC	2020-09-28	2.28	2.44	2.26	2.4	219401	{}	\N	2020-10-12 16:51:48.647987
84875	HCHC	2020-09-29	2.38	2.58	2.38	2.55	972195	{}	\N	2020-10-12 16:51:48.658642
84876	HCHC	2020-09-30	2.54	2.68	2.39	2.42	1317614	{}	\N	2020-10-12 16:51:48.665979
84877	HCHC	2020-10-01	2.51	2.535	2.39	2.5	573494	{}	\N	2020-10-12 16:51:48.672242
84878	HCHC	2020-10-02	2.41	2.55	2.41	2.5	875279	{}	\N	2020-10-12 16:51:48.677217
84879	HCHC	2020-10-05	2.52	2.72	2.52	2.61	820756	{}	\N	2020-10-12 16:51:48.682096
84880	HCHC	2020-10-06	2.65	2.7	2.46	2.47	365759	{}	\N	2020-10-12 16:51:48.686902
84881	HCHC	2020-10-07	2.42	2.47	2.27	2.28	731014	{}	\N	2020-10-12 16:51:48.690946
84882	HCHC	2020-10-08	2.29	2.42	2.28	2.35	410661	{}	\N	2020-10-12 16:51:48.694814
84883	HCHC	2020-10-09	2.38	2.53	2.365	2.53	385606	{}	\N	2020-10-12 16:51:48.698716
84884	HCHC	2020-10-12	2.54	2.59	2.38	2.38	239736	{}	\N	2020-10-12 16:51:48.702763
84885	ICD	2020-09-28	2.52	2.69	2.4758	2.63	138795	{}	\N	2020-10-12 16:51:51.342171
84886	ICD	2020-09-29	2.85	2.85	2.33	2.56	349018	{}	\N	2020-10-12 16:51:51.353679
84887	ICD	2020-09-30	2.5	2.68	2.42	2.44	217530	{}	\N	2020-10-12 16:51:51.361976
84888	ICD	2020-10-01	2.5	2.59	2.37	2.39	113496	{}	\N	2020-10-12 16:51:51.368447
84889	ICD	2020-10-02	2.31	2.44	2.29	2.39	162019	{}	\N	2020-10-12 16:51:51.375983
84890	ICD	2020-10-05	2.51	2.55	2.33	2.39	142680	{}	\N	2020-10-12 16:51:51.381554
84891	ICD	2020-10-06	2.5	2.55	2.38	2.44	133373	{}	\N	2020-10-12 16:51:51.386674
84892	ICD	2020-10-07	2.43	2.5	2.37	2.46	86408	{}	\N	2020-10-12 16:51:51.391303
84893	ICD	2020-10-08	2.45	2.73	2.4311	2.64	618601	{}	\N	2020-10-12 16:51:51.397025
84894	ICD	2020-10-09	2.75	3.95	2.54	2.91	24840996	{}	\N	2020-10-12 16:51:51.402783
84895	ICD	2020-10-12	2.65	2.8	2.4795	2.71	1175768	{}	\N	2020-10-12 16:51:51.407661
84896	ICOW	2020-09-28	23.1	23.2925	23.1	23.2925	947	{}	\N	2020-10-12 16:51:53.381408
84897	ICOW	2020-09-29	23.2803	23.2803	23.2803	23.2803	215	{}	\N	2020-10-12 16:51:53.39208
84898	ICOW	2020-09-30	23.3901	23.3901	23.225	23.23	5199	{}	\N	2020-10-12 16:51:53.40185
84899	ICOW	2020-10-01	23.2601	23.3483	23.26	23.3483	448	{}	\N	2020-10-12 16:51:53.407144
84900	ICOW	2020-10-02	23.2103	23.3169	23.2103	23.3169	323	{}	\N	2020-10-12 16:51:53.412887
84901	ICOW	2020-10-05	23.56	23.85	23.56	23.85	2578	{}	\N	2020-10-12 16:51:53.417597
84902	ICOW	2020-10-06	23.9	23.94	23.63	23.6301	2276	{}	\N	2020-10-12 16:51:53.421912
84903	ICOW	2020-10-07	23.8567	24.01	23.8567	23.96	1467	{}	\N	2020-10-12 16:51:53.426327
84904	ICOW	2020-10-08	24.05	24.1699	24.05	24.14	2789	{}	\N	2020-10-12 16:51:53.431037
84905	ICOW	2020-10-09	24.2847	24.2847	24.25	24.25	619	{}	\N	2020-10-12 16:51:53.435533
84906	ICOW	2020-10-12	24.25	24.25	24.2	24.21	1648	{}	\N	2020-10-12 16:51:53.439998
84907	IDV	2020-09-28	25.01	25.12	24.98	25.11	745162	{}	\N	2020-10-12 16:51:54.336858
84908	IDV	2020-09-29	25.03	25.05	24.802	24.89	721897	{}	\N	2020-10-12 16:51:54.345017
84909	IDV	2020-09-30	24.96	25.12	24.845	24.93	808185	{}	\N	2020-10-12 16:51:54.351153
84910	IDV	2020-10-01	25.05	25.0698	24.89	25.02	919019	{}	\N	2020-10-12 16:51:54.357109
84911	IDV	2020-10-02	24.84	25.26	24.83	25.185	765413	{}	\N	2020-10-12 16:51:54.362023
84912	IDV	2020-10-05	25.43	25.68	25.43	25.65	655264	{}	\N	2020-10-12 16:51:54.366984
84913	IDV	2020-10-06	25.74	25.7767	25.4	25.45	900851	{}	\N	2020-10-12 16:51:54.371211
84914	IDV	2020-10-07	25.69	25.72	25.595	25.64	701609	{}	\N	2020-10-12 16:51:54.37578
84915	IDV	2020-10-08	25.78	25.89	25.7672	25.87	808642	{}	\N	2020-10-12 16:51:54.380701
84916	IDV	2020-10-09	25.97	26.0199	25.905	25.97	499286	{}	\N	2020-10-12 16:51:54.384857
84917	IDV	2020-10-12	26	26.09	25.9601	26.08	188670	{}	\N	2020-10-12 16:51:54.389585
84918	IHC	2020-09-28	36.62	37.02	36.28	36.28	9368	{}	\N	2020-10-12 16:51:55.42472
84919	IHC	2020-09-29	37.29	38	36.72	38	29620	{}	\N	2020-10-12 16:51:55.43417
84920	IHC	2020-09-30	38.12	38.67	37.71	37.71	12642	{}	\N	2020-10-12 16:51:55.441259
84921	IHC	2020-10-01	38.33	39.32	38.04	39	16991	{}	\N	2020-10-12 16:51:55.448072
84922	IHC	2020-10-02	38.03	40.04	38.03	39.75	11834	{}	\N	2020-10-12 16:51:55.453246
84923	IHC	2020-10-05	40.01	41.26	39.39	39.39	23554	{}	\N	2020-10-12 16:51:55.458225
84924	IHC	2020-10-06	40.55	40.8	38.35	38.35	21041	{}	\N	2020-10-12 16:51:55.464479
84925	IHC	2020-10-07	39.85	40.49	39.71	39.82	10946	{}	\N	2020-10-12 16:51:55.469941
84926	IHC	2020-10-08	40.245	40.245	39.44	39.44	8098	{}	\N	2020-10-12 16:51:55.475012
84927	IHC	2020-10-09	40.07	40.67	39.81	40.67	8176	{}	\N	2020-10-12 16:51:55.481024
84928	IHC	2020-10-12	40.58	40.58	39.45	39.45	2448	{}	\N	2020-10-12 16:51:55.486396
84929	IMLP	2020-09-28	7.48	7.6993	7.48	7.6993	773	{}	\N	2020-10-12 16:51:58.665692
84930	IMLP	2020-09-29	7.64	7.7199	7.64	7.7199	206	{}	\N	2020-10-12 16:51:58.674529
84931	IMLP	2020-09-30	7.5974	7.5974	7.5974	7.5974	83	{}	\N	2020-10-12 16:51:58.681022
84932	IMLP	2020-10-01	7.5169	7.5169	7.5169	7.5169	106	{}	\N	2020-10-12 16:51:58.687758
84933	IMLP	2020-10-02	7.6425	7.6425	7.6425	7.6425	9	{}	\N	2020-10-12 16:51:58.693623
84934	IMLP	2020-10-05	7.8974	7.8974	7.8974	7.8974	101	{}	\N	2020-10-12 16:51:58.698823
84935	IMLP	2020-10-06	7.7546	7.7546	7.7546	7.7546	115	{}	\N	2020-10-12 16:51:58.70419
84936	IMLP	2020-10-07	7.75	7.75	7.69	7.71	1600	{}	\N	2020-10-12 16:51:58.709775
84937	IMLP	2020-10-08	8.2338	8.2338	8.2338	8.2338	107	{}	\N	2020-10-12 16:51:58.714501
84938	IMLP	2020-10-09	8.3	8.3	8.21	8.25	1515	{}	\N	2020-10-12 16:51:58.719238
84939	IRBT	2020-09-28	77.88	78.95	77.07	77.2	339786	{}	\N	2020-10-12 16:52:00.1401
84940	IRBT	2020-09-29	77.26	78.2	76.68	77.64	233381	{}	\N	2020-10-12 16:52:00.14748
84941	IRBT	2020-09-30	77.75	78.84	75.66	75.9	485768	{}	\N	2020-10-12 16:52:00.155858
84942	IRBT	2020-10-01	76.75	79.49	75.4	79.31	751312	{}	\N	2020-10-12 16:52:00.165326
84943	IRBT	2020-10-02	77.27	79.96	76.7504	78.89	493395	{}	\N	2020-10-12 16:52:00.172835
84944	IRBT	2020-10-05	79.5149	81.04	79	80.99	414769	{}	\N	2020-10-12 16:52:00.179827
84945	IRBT	2020-10-06	81.19	83.46	80.55	80.83	461158	{}	\N	2020-10-12 16:52:00.18663
84946	IRBT	2020-10-07	81.2	83.79	81.2	83.08	439222	{}	\N	2020-10-12 16:52:00.192669
84947	IRBT	2020-10-08	84.01	86.97	83.4901	86.5	495757	{}	\N	2020-10-12 16:52:00.199355
84948	IRBT	2020-10-09	86.915	91.23	86.78	89.83	818425	{}	\N	2020-10-12 16:52:00.204871
84949	IRBT	2020-10-12	91	91.4	85.55	87.245	447905	{}	\N	2020-10-12 16:52:00.211353
84950	JACK	2020-09-28	81.53	82.065	79.58	80.78	486823	{}	\N	2020-10-12 16:52:02.411194
84951	JACK	2020-09-29	80.26	80.27	78.29	78.5	615928	{}	\N	2020-10-12 16:52:02.421651
84952	JACK	2020-09-30	78.88	80.14	78.58	79.31	496211	{}	\N	2020-10-12 16:52:02.428652
84953	JACK	2020-10-01	79.87	81.21	79.44	80.76	482150	{}	\N	2020-10-12 16:52:02.434424
84954	JACK	2020-10-02	78.62	84.54	78.62	83.52	907239	{}	\N	2020-10-12 16:52:02.440067
84955	JACK	2020-10-05	83.78	85.14	83.21	84.46	486168	{}	\N	2020-10-12 16:52:02.444982
84956	JACK	2020-10-06	84.56	85.465	81	81.43	468465	{}	\N	2020-10-12 16:52:02.449684
84957	JACK	2020-10-07	82.62	83.84	81.72	82.37	394617	{}	\N	2020-10-12 16:52:02.45406
84958	JACK	2020-10-08	83.22	83.22	81.57	83.15	286270	{}	\N	2020-10-12 16:52:02.458647
84959	JACK	2020-10-09	83.87	84.53	83.01	84.29	290325	{}	\N	2020-10-12 16:52:02.463783
84960	JACK	2020-10-12	84.51	84.89	83.22	84.475	117834	{}	\N	2020-10-12 16:52:02.468454
84961	GRU	2020-09-28	2.845	2.93	2.845	2.93	259	{}	\N	2020-10-12 16:52:04.336541
84962	GRU	2020-09-29	2.88	2.88	2.88	2.88	131	{}	\N	2020-10-12 16:52:04.35048
84963	GRU	2020-09-30	2.88	2.99	2.88	2.99	6602	{}	\N	2020-10-12 16:52:04.359396
84964	GRU	2020-10-01	2.94	2.94	2.925	2.925	135	{}	\N	2020-10-12 16:52:04.365801
84965	GRU	2020-10-02	2.9	2.96	2.84	2.84	5366	{}	\N	2020-10-12 16:52:04.371736
84966	GRU	2020-10-05	2.99	3.02	2.88	3.02	9563	{}	\N	2020-10-12 16:52:04.376735
84967	GRU	2020-10-06	2.96	3.07	2.94	3.015	15271	{}	\N	2020-10-12 16:52:04.381508
84968	GRU	2020-10-07	3.015	3.04	3.015	3.04	2649	{}	\N	2020-10-12 16:52:04.386524
84969	GRU	2020-10-08	3	3.01	3	3.01	740	{}	\N	2020-10-12 16:52:04.393692
84970	GRU	2020-10-09	2.945	2.945	2.945	2.945	735	{}	\N	2020-10-12 16:52:04.401771
84971	GRU	2020-10-12	3	3.08	2.9951	3.0699	4156	{}	\N	2020-10-12 16:52:04.410222
84972	GVA	2020-09-28	17.94	18.4	17.94	18.15	301057	{}	\N	2020-10-12 16:52:05.418866
84973	GVA	2020-09-29	18	18.075	17.38	17.74	640783	{}	\N	2020-10-12 16:52:05.431662
84974	GVA	2020-09-30	17.89	18.34	17.6	17.61	627642	{}	\N	2020-10-12 16:52:05.438419
84975	GVA	2020-10-01	17.55	17.96	17.44	17.72	615034	{}	\N	2020-10-12 16:52:05.44486
84976	GVA	2020-10-02	17.25	18.52	17.12	18.37	1220840	{}	\N	2020-10-12 16:52:05.449881
84977	GVA	2020-10-05	18.48	19.32	18.34	18.59	1175750	{}	\N	2020-10-12 16:52:05.45415
84978	GVA	2020-10-06	18.87	19.03	18.09	18.11	1089317	{}	\N	2020-10-12 16:52:05.459144
84979	GVA	2020-10-07	18.26	18.895	18.26	18.77	1014428	{}	\N	2020-10-12 16:52:05.463663
84980	GVA	2020-10-08	18.99	19.63	18.9364	19.6	857612	{}	\N	2020-10-12 16:52:05.469945
84981	GVA	2020-10-09	19.75	20	19.15	19.76	795785	{}	\N	2020-10-12 16:52:05.475654
84982	GVA	2020-10-12	19.76	20.2782	19.4013	20.12	320812	{}	\N	2020-10-12 16:52:05.480092
84983	HBNC	2020-09-28	9.75	10.28	9.66	10.19	192894	{}	\N	2020-10-12 16:52:08.283864
84984	HBNC	2020-09-29	10.22	10.7164	9.76	10.04	98673	{}	\N	2020-10-12 16:52:08.292718
84985	HBNC	2020-09-30	10.1	10.28	10.03	10.09	113417	{}	\N	2020-10-12 16:52:08.298458
84986	HBNC	2020-10-01	10.05	10.2	9.87	10.16	154726	{}	\N	2020-10-12 16:52:08.303665
84987	HBNC	2020-10-02	9.91	10.46	9.85	10.44	74173	{}	\N	2020-10-12 16:52:08.308121
84988	HBNC	2020-10-05	10.48	10.71	10.36	10.67	96539	{}	\N	2020-10-12 16:52:08.313016
84989	HBNC	2020-10-06	10.7195	11.35	10.65	11.08	210521	{}	\N	2020-10-12 16:52:08.31749
84990	HBNC	2020-10-07	11.2	11.48	11.18	11.37	168023	{}	\N	2020-10-12 16:52:08.321734
84991	HBNC	2020-10-08	11.38	11.68	11.18	11.605	113156	{}	\N	2020-10-12 16:52:08.326234
84992	HBNC	2020-10-09	11.74	11.77	11.59	11.67	106756	{}	\N	2020-10-12 16:52:08.331116
84993	HBNC	2020-10-12	11.68	11.91	11.64	11.84	22658	{}	\N	2020-10-12 16:52:08.335907
84994	HCCI	2020-09-28	13.35	13.92	13.35	13.56	26875	{}	\N	2020-10-12 16:52:10.29718
84995	HCCI	2020-09-29	13.57	13.92	13	13.32	27354	{}	\N	2020-10-12 16:52:10.303614
84996	HCCI	2020-09-30	13.11	13.49	13.11	13.35	22783	{}	\N	2020-10-12 16:52:10.308934
84997	HCCI	2020-10-01	13.51	13.96	13.22	13.88	29329	{}	\N	2020-10-12 16:52:10.314634
84998	HCCI	2020-10-02	13.51	14.2	13.05	13.96	22519	{}	\N	2020-10-12 16:52:10.322576
84999	HCCI	2020-10-05	14.03	14.17	13.5	13.96	48808	{}	\N	2020-10-12 16:52:10.330822
85000	HCCI	2020-10-06	13.94	15.1	13.94	14.3	41486	{}	\N	2020-10-12 16:52:10.339319
85001	HCCI	2020-10-07	14.5	14.82	14.3	14.36	46339	{}	\N	2020-10-12 16:52:10.348933
85002	HCCI	2020-10-08	14.36	14.7	14.12	14.6	30002	{}	\N	2020-10-12 16:52:10.355766
85003	HCCI	2020-10-09	14.7	14.7	14.23	14.49	11518	{}	\N	2020-10-12 16:52:10.362788
85004	HCCI	2020-10-12	15.04	15.04	13.97	14.12	10011	{}	\N	2020-10-12 16:52:10.369163
85005	HNDL	2020-09-28	24.69	24.72	24.58	24.67	36430	{}	\N	2020-10-12 16:52:12.864857
85006	HNDL	2020-09-29	24.53	24.69	24.53	24.69	24624	{}	\N	2020-10-12 16:52:12.872801
85007	HNDL	2020-09-30	24.72	24.72	24.6	24.66	16831	{}	\N	2020-10-12 16:52:12.878952
85008	HNDL	2020-10-01	24.6611	24.85	24.6611	24.74	22286	{}	\N	2020-10-12 16:52:12.883852
85009	HNDL	2020-10-02	24.75	24.76	24.5883	24.68	33503	{}	\N	2020-10-12 16:52:12.888234
85010	HNDL	2020-10-05	24.76	24.83	24.7001	24.75	70118	{}	\N	2020-10-12 16:52:12.893608
85011	HNDL	2020-10-06	24.8286	24.8286	24.71	24.73	23562	{}	\N	2020-10-12 16:52:12.897881
85012	HNDL	2020-10-07	24.83	24.83	24.7117	24.7573	31641	{}	\N	2020-10-12 16:52:12.902342
85013	HNDL	2020-10-08	24.92	25.02	24.77	24.85	33573	{}	\N	2020-10-12 16:52:12.906795
85014	HNDL	2020-10-09	24.99	24.99	24.8483	24.8818	30241	{}	\N	2020-10-12 16:52:12.911087
85015	HNDL	2020-10-12	25.11	25.11	24.9004	25.0661	16483	{}	\N	2020-10-12 16:52:12.915749
85016	HOVNP	2020-09-28	6.44	6.49	6.44	6.458	1171	{}	\N	2020-10-12 16:52:13.759929
85017	HOVNP	2020-09-29	6.45	6.89	6.4	6.85	7390	{}	\N	2020-10-12 16:52:13.770227
85018	HOVNP	2020-09-30	7.4299	7.4299	6.78	7.15	9374	{}	\N	2020-10-12 16:52:13.776901
85019	HOVNP	2020-10-01	7.135	7.135	7.135	7.135	502	{}	\N	2020-10-12 16:52:13.783004
85020	HOVNP	2020-10-05	7.11	7.11	7.11	7.11	1301	{}	\N	2020-10-12 16:52:13.788088
85021	HOVNP	2020-10-06	7.17	7.17	7.17	7.17	1427	{}	\N	2020-10-12 16:52:13.794319
85022	HOVNP	2020-10-07	7.29	7.4	7.145	7.4	901	{}	\N	2020-10-12 16:52:13.800159
85023	HOVNP	2020-10-08	7.02	7.49	7.02	7.4877	4257	{}	\N	2020-10-12 16:52:13.805092
85024	HOVNP	2020-10-09	6.9736	6.9736	6.9736	6.9736	500	{}	\N	2020-10-12 16:52:13.811083
85025	HOVNP	2020-10-12	6.57	6.58	5.52	6.58	6740	{}	\N	2020-10-12 16:52:13.816884
85026	HRZN	2020-09-28	12	12.52	11.9634	12.28	201984	{}	\N	2020-10-12 16:52:14.844701
85027	HRZN	2020-09-29	12.36	12.4	12.15	12.2	106890	{}	\N	2020-10-12 16:52:14.851231
85028	HRZN	2020-09-30	12.21	12.45	12.2	12.28	111855	{}	\N	2020-10-12 16:52:14.857599
85029	HRZN	2020-10-01	12.31	12.5	12.265	12.46	146323	{}	\N	2020-10-12 16:52:14.864415
85030	HRZN	2020-10-02	12.3	12.4	12.25	12.35	104865	{}	\N	2020-10-12 16:52:14.870573
85031	HRZN	2020-10-05	12.357	12.58	12.357	12.55	111883	{}	\N	2020-10-12 16:52:14.878186
85032	HRZN	2020-10-06	12.56	12.79	12.4837	12.57	104119	{}	\N	2020-10-12 16:52:14.885636
85033	HRZN	2020-10-07	12.67	12.86	12.62	12.77	120634	{}	\N	2020-10-12 16:52:14.891083
85034	HRZN	2020-10-08	12.84	12.97	12.8	12.88	129120	{}	\N	2020-10-12 16:52:14.89676
85035	HRZN	2020-10-09	12.87	12.98	12.81	12.88	71079	{}	\N	2020-10-12 16:52:14.901995
85036	HRZN	2020-10-12	12.9	12.9499	12.8648	12.87	55898	{}	\N	2020-10-12 16:52:14.906656
85037	HT	2020-09-28	5.5	5.85	5.44	5.68	411070	{}	\N	2020-10-12 16:52:15.882054
85038	HT	2020-09-29	5.65	5.659	5.42	5.56	691263	{}	\N	2020-10-12 16:52:15.893477
85039	HT	2020-09-30	5.61	5.76	5.37	5.54	789607	{}	\N	2020-10-12 16:52:15.906387
85040	HT	2020-10-01	5.54	5.68	5.41	5.66	1124618	{}	\N	2020-10-12 16:52:15.914558
85041	HT	2020-10-02	5.47	5.73	5.39	5.72	672286	{}	\N	2020-10-12 16:52:15.924438
85042	HT	2020-10-05	5.68	5.83	5.35	5.56	864117	{}	\N	2020-10-12 16:52:15.932179
85043	HT	2020-10-06	5.6	5.72	5.45	5.47	587404	{}	\N	2020-10-12 16:52:15.941351
85044	HT	2020-10-07	5.53	5.555	5.355	5.48	819950	{}	\N	2020-10-12 16:52:15.950154
85045	HT	2020-10-08	5.43	5.55	5.36	5.54	751679	{}	\N	2020-10-12 16:52:15.956883
85046	HT	2020-10-09	5.58	5.58	5.45	5.56	574078	{}	\N	2020-10-12 16:52:15.96257
85047	HT	2020-10-12	5.56	5.6	5.42	5.45	281857	{}	\N	2020-10-12 16:52:15.968938
85048	IAU	2020-09-28	17.87	17.97	17.79	17.94	15996634	{}	\N	2020-10-12 16:52:17.311637
85049	IAU	2020-09-29	18	18.1167	17.985	18.11	17148952	{}	\N	2020-10-12 16:52:17.319097
85050	IAU	2020-09-30	18.06	18.15	17.95	17.99	22306411	{}	\N	2020-10-12 16:52:17.332041
85051	IAU	2020-10-01	18.16	18.24	18.09	18.14	24190688	{}	\N	2020-10-12 16:52:17.341119
85052	IAU	2020-10-02	18.17	18.216	18.1	18.13	25054552	{}	\N	2020-10-12 16:52:17.350242
85053	IAU	2020-10-05	18.18	18.305	18.18	18.24	32992378	{}	\N	2020-10-12 16:52:17.359737
85054	IAU	2020-10-06	18.31	18.33	17.99	18	35534036	{}	\N	2020-10-12 16:52:17.369122
85055	IAU	2020-10-07	17.9921	18.03	17.925	17.99	18925121	{}	\N	2020-10-12 16:52:17.377747
85056	IAU	2020-10-08	18.1099	18.1379	17.95	18.08	13200896	{}	\N	2020-10-12 16:52:17.385036
85057	IAU	2020-10-09	18.3	18.41	18.28	18.41	23680190	{}	\N	2020-10-12 16:52:17.393257
85058	IAU	2020-10-12	18.355	18.38	18.31	18.369	9325632	{}	\N	2020-10-12 16:52:17.399639
85059	IDOG	2020-09-28	21.75	21.75	21.65	21.69	8348	{}	\N	2020-10-12 16:52:18.408279
85060	IDOG	2020-09-29	21.7	21.71	21.491	21.57	28927	{}	\N	2020-10-12 16:52:18.420142
85061	IDOG	2020-09-30	21.64	21.73	21.48	21.56	52829	{}	\N	2020-10-12 16:52:18.427916
85062	IDOG	2020-10-01	21.68	21.69	21.46	21.6	511870	{}	\N	2020-10-12 16:52:18.433966
85063	IDOG	2020-10-02	21.289	21.655	21.289	21.62	68771	{}	\N	2020-10-12 16:52:18.441386
85064	IDOG	2020-10-05	21.82	22.0299	21.795	22.01	27797	{}	\N	2020-10-12 16:52:18.446997
85065	IDOG	2020-10-06	22.21	22.2499	21.92	21.99	22598	{}	\N	2020-10-12 16:52:18.451998
85066	IDOG	2020-10-07	22.1503	22.21	22.1	22.125	37513	{}	\N	2020-10-12 16:52:18.457131
85067	IDOG	2020-10-08	22.33	22.485	22.29	22.485	33688	{}	\N	2020-10-12 16:52:18.462526
85068	IDOG	2020-10-09	22.55	22.56	22.4101	22.47	26340	{}	\N	2020-10-12 16:52:18.466964
85069	IDOG	2020-10-12	22.44	22.565	22.4	22.5602	9339	{}	\N	2020-10-12 16:52:18.471741
85070	IJS	2020-09-28	121.11	123.8	121.11	123.25	107677	{}	\N	2020-10-12 16:52:19.397928
85071	IJS	2020-09-29	122.97	123.25	120.62	122.05	102123	{}	\N	2020-10-12 16:52:19.404719
85072	IJS	2020-09-30	122.36	124.53	121.54	122.42	93951	{}	\N	2020-10-12 16:52:19.410636
85073	IJS	2020-10-01	123.45	124.24	122.042	124.21	123149	{}	\N	2020-10-12 16:52:19.416356
85074	IJS	2020-10-02	121.75	126.695	121.75	126.34	131517	{}	\N	2020-10-12 16:52:19.420983
85075	IJS	2020-10-05	127.48	129.45	127.47	129.34	85317	{}	\N	2020-10-12 16:52:19.425697
85076	IJS	2020-10-06	130.59	133.21	129.06	129.18	274959	{}	\N	2020-10-12 16:52:19.429826
85077	IJS	2020-10-07	130.69	131.97	130.3939	131.58	245190	{}	\N	2020-10-12 16:52:19.434195
85078	IJS	2020-10-08	132.51	133.93	131.84	133.73	93533	{}	\N	2020-10-12 16:52:19.43912
85079	IJS	2020-10-09	135	135.31	133.321	133.7	99600	{}	\N	2020-10-12 16:52:19.444179
85080	IJS	2020-10-12	134	134.56	133.48	134.18	55294	{}	\N	2020-10-12 16:52:19.448813
85081	HAWX	2020-09-28	26.68	26.68	25.843	26.62	72642	{}	\N	2020-10-12 16:52:20.965992
85082	HAWX	2020-09-29	26.64	26.64	26.4551	26.54	7740	{}	\N	2020-10-12 16:52:20.975874
85083	HAWX	2020-09-30	26.65	26.65	26.465	26.49	6022	{}	\N	2020-10-12 16:52:20.9817
85084	HAWX	2020-10-01	26.649	26.65	26.5872	26.61	18868	{}	\N	2020-10-12 16:52:20.987726
85085	HAWX	2020-10-02	26.45	26.6	26.45	26.59	5500	{}	\N	2020-10-12 16:52:20.993677
85086	HAWX	2020-10-05	26.7273	26.88	26.7273	26.88	7815	{}	\N	2020-10-12 16:52:20.998887
85087	HAWX	2020-10-06	26.8361	26.8999	26.7008	26.7008	4301	{}	\N	2020-10-12 16:52:21.003686
85088	HAWX	2020-10-07	26.91	26.9899	26.91	26.9472	7288	{}	\N	2020-10-12 16:52:21.009066
85089	HAWX	2020-10-08	27.21	27.21	27.0467	27.18	47011	{}	\N	2020-10-12 16:52:21.013891
85090	HAWX	2020-10-09	27.08	27.24	27.08	27.205	8432	{}	\N	2020-10-12 16:52:21.018578
85091	HAWX	2020-10-12	27.37	27.37	27.23	27.2706	1324	{}	\N	2020-10-12 16:52:21.022861
85092	HCA	2020-09-28	122.48	123.86	120.82	121.36	1097011	{}	\N	2020-10-12 16:52:23.104615
85093	HCA	2020-09-29	122	122.15	117.78	120.18	2122245	{}	\N	2020-10-12 16:52:23.117409
85094	HCA	2020-09-30	121.1	126.35	120.55	124.68	3722022	{}	\N	2020-10-12 16:52:23.124987
85095	HCA	2020-10-01	124.74	125.6988	121.48	124.56	3601966	{}	\N	2020-10-12 16:52:23.131502
85096	HCA	2020-10-02	121.5	128.18	121.455	127.05	2895632	{}	\N	2020-10-12 16:52:23.138006
85097	HCA	2020-10-05	128.18	130.78	126.89	130.17	2959216	{}	\N	2020-10-12 16:52:23.142871
85098	HCA	2020-10-06	130.79	132.4	128.21	128.37	2416575	{}	\N	2020-10-12 16:52:23.148074
85099	HCA	2020-10-07	130	132.34	128.56	129.07	1953440	{}	\N	2020-10-12 16:52:23.1531
85100	HCA	2020-10-08	129.89	132.48	128.95	131.83	2242186	{}	\N	2020-10-12 16:52:23.158261
85101	HCA	2020-10-09	139.5	140	133.2	133.96	5517920	{}	\N	2020-10-12 16:52:23.163602
85102	HCA	2020-10-12	135	135.285	129.89	130.63	1232260	{}	\N	2020-10-12 16:52:23.169145
85103	HCI	2020-09-28	50.09	51.73	50.09	50.81	40051	{}	\N	2020-10-12 16:52:24.352691
85104	HCI	2020-09-29	50.78	50.78	49.15	49.7	55683	{}	\N	2020-10-12 16:52:24.363517
85105	HCI	2020-09-30	49.93	50.49	48.66	49.29	86033	{}	\N	2020-10-12 16:52:24.371328
85106	HCI	2020-10-01	48.98	49.3884	48.43	49.18	71084	{}	\N	2020-10-12 16:52:24.377831
85107	HCI	2020-10-02	48.51	49.6	48.49	48.63	63873	{}	\N	2020-10-12 16:52:24.384123
85108	HCI	2020-10-05	48.68	49.405	48.545	48.67	55804	{}	\N	2020-10-12 16:52:24.390021
85109	HCI	2020-10-06	49.23	49.94	48.56	49.3	55542	{}	\N	2020-10-12 16:52:24.395285
85110	HCI	2020-10-07	49.05	49.05	47.86	47.91	70107	{}	\N	2020-10-12 16:52:24.400387
85111	HCI	2020-10-08	48.38	48.8	47.4	47.66	75095	{}	\N	2020-10-12 16:52:24.405665
85112	HCI	2020-10-09	47.67	48	47.23	47.75	46315	{}	\N	2020-10-12 16:52:24.410486
85113	HCI	2020-10-12	47.39	48.25	47.39	48.05	28382	{}	\N	2020-10-12 16:52:24.415571
85114	HTBI	2020-09-28	13.63	13.94	13.55	13.77	47435	{}	\N	2020-10-12 16:52:26.175619
85115	HTBI	2020-09-29	13.59	13.72	13.48	13.72	26812	{}	\N	2020-10-12 16:52:26.185591
85116	HTBI	2020-09-30	13.71	13.78	13.57	13.61	31952	{}	\N	2020-10-12 16:52:26.192236
85117	HTBI	2020-10-01	13.47	13.78	13.36	13.67	26849	{}	\N	2020-10-12 16:52:26.198682
85118	HTBI	2020-10-02	13.79	13.96	13.67	13.86	35556	{}	\N	2020-10-12 16:52:26.204239
85119	HTBI	2020-10-05	14.04	14.44	12.81	13.99	85744	{}	\N	2020-10-12 16:52:26.209357
85120	HTBI	2020-10-06	14.19	14.42	13.76	14.07	68832	{}	\N	2020-10-12 16:52:26.214484
85121	HTBI	2020-10-07	14.355	14.43	14.1	14.39	99974	{}	\N	2020-10-12 16:52:26.219263
85122	HTBI	2020-10-08	14.51	15.43	14.45	15.28	335652	{}	\N	2020-10-12 16:52:26.224278
85123	HTBI	2020-10-09	15.47	15.47	14.63	14.86	22485	{}	\N	2020-10-12 16:52:26.230345
85124	HTBI	2020-10-12	14.545	14.89	14.545	14.86	8531	{}	\N	2020-10-12 16:52:26.235588
85125	HUSV	2020-09-28	27.12	27.4	27.11	27.295	17498	{}	\N	2020-10-12 16:52:27.117747
85126	HUSV	2020-09-29	27.3455	27.39	27.1501	27.2485	22633	{}	\N	2020-10-12 16:52:27.124851
85127	HUSV	2020-09-30	27.29	27.5	27.1708	27.34	145694	{}	\N	2020-10-12 16:52:27.131095
85128	HUSV	2020-10-01	27.36	27.4812	27.2685	27.37	14178	{}	\N	2020-10-12 16:52:27.13762
85129	HUSV	2020-10-02	27.04	27.4899	27.04	27.41	11712	{}	\N	2020-10-12 16:52:27.142938
85130	HUSV	2020-10-05	27.5	27.7106	27.46	27.68	81826	{}	\N	2020-10-12 16:52:27.149459
85131	HUSV	2020-10-06	27.7326	27.92	27.46	27.51	10661	{}	\N	2020-10-12 16:52:27.154905
85132	HUSV	2020-10-07	27.67	27.87	27.67	27.81	654829	{}	\N	2020-10-12 16:52:27.160304
85133	HUSV	2020-10-08	27.96	28.01	27.9005	27.97	260962	{}	\N	2020-10-12 16:52:27.166037
85134	HUSV	2020-10-09	28.08	28.2564	28.08	28.15	76365	{}	\N	2020-10-12 16:52:27.171658
85135	HUSV	2020-10-12	28.29	28.45	28.26	28.44	5608	{}	\N	2020-10-12 16:52:27.176874
85136	ICE	2020-09-28	101.28	102.31	101.11	101.23	1801082	{}	\N	2020-10-12 16:52:29.53653
85137	ICE	2020-09-29	101.41	101.78	96.9	99.17	8801534	{}	\N	2020-10-12 16:52:29.543484
85138	ICE	2020-09-30	99.54	101.15	99.17	100.05	4648222	{}	\N	2020-10-12 16:52:29.549525
85139	ICE	2020-10-01	101.09	102.105	100.53	101.43	4364478	{}	\N	2020-10-12 16:52:29.555329
85140	ICE	2020-10-02	100.23	101.8539	100.19	101.34	3646317	{}	\N	2020-10-12 16:52:29.560698
85141	ICE	2020-10-05	101.78	102.07	100.77	101.81	3253717	{}	\N	2020-10-12 16:52:29.566323
85142	ICE	2020-10-06	102.49	102.49	100.6	101.01	4649972	{}	\N	2020-10-12 16:52:29.571996
85143	ICE	2020-10-07	101.48	102.16	100.29	100.85	4672549	{}	\N	2020-10-12 16:52:29.577279
85144	ICE	2020-10-08	101.29	102.27	100.94	101.44	4480131	{}	\N	2020-10-12 16:52:29.583047
85145	ICE	2020-10-09	101.97	102.29	101.16	101.39	3713528	{}	\N	2020-10-12 16:52:29.592811
85146	ICE	2020-10-12	101.68	102.78	101.55	102.64	3260540	{}	\N	2020-10-12 16:52:29.599752
85147	INT	2020-09-28	21.71	22.23	21.71	21.94	403886	{}	\N	2020-10-12 16:52:32.918668
85148	INT	2020-09-29	21.82	21.82	21.05	21.34	556343	{}	\N	2020-10-12 16:52:32.925706
85149	INT	2020-09-30	21.58	21.93	20.94	21.19	633466	{}	\N	2020-10-12 16:52:32.931262
85150	INT	2020-10-01	21.14	21.43	20.86	21.27	915052	{}	\N	2020-10-12 16:52:32.935989
85151	INT	2020-10-02	20.79	22.16	20.75	22.03	804162	{}	\N	2020-10-12 16:52:32.940548
85152	INT	2020-10-05	22.48	23.08	22.48	22.92	644246	{}	\N	2020-10-12 16:52:32.945663
85153	INT	2020-10-06	23.29	23.47	22.47	22.52	937135	{}	\N	2020-10-12 16:52:32.952493
85154	INT	2020-10-07	22.72	23.03	22.6685	22.8	637880	{}	\N	2020-10-12 16:52:32.957182
85155	INT	2020-10-08	23.18	23.4	22.88	23.36	560522	{}	\N	2020-10-12 16:52:32.962514
85156	INT	2020-10-09	23.71	23.71	23.115	23.4	542213	{}	\N	2020-10-12 16:52:32.969046
85157	INT	2020-10-12	23.31	23.53	22.98	23.2	104676	{}	\N	2020-10-12 16:52:32.974718
85158	ONEV	2020-09-28	77.9045	77.97	77.7683	77.7683	2119	{}	\N	2020-10-12 16:52:34.166004
85159	ONEV	2020-09-29	77.85	77.85	77.08	77.45	1328	{}	\N	2020-10-12 16:52:34.178543
85160	ONEV	2020-09-30	78.08	78.08	77.325	77.6314	2761	{}	\N	2020-10-12 16:52:34.186833
85161	ONEV	2020-10-01	77.92	78.1065	77.7239	77.95	3102	{}	\N	2020-10-12 16:52:34.19319
85162	ONEV	2020-10-02	77.3909	78.6722	77.3909	78.5344	19218	{}	\N	2020-10-12 16:52:34.198915
85163	ONEV	2020-10-05	78.82	79.88	78.82	79.78	2386	{}	\N	2020-10-12 16:52:34.203721
85164	ONEV	2020-10-06	80.39	80.39	79.3699	79.3699	2234	{}	\N	2020-10-12 16:52:34.208569
85165	ONEV	2020-10-07	80.26	80.52	80.26	80.52	1284	{}	\N	2020-10-12 16:52:34.214095
85166	ONEV	2020-10-08	81.465	81.56	81.465	81.56	2072	{}	\N	2020-10-12 16:52:34.219207
85167	ONEV	2020-10-09	82.13	82.13	81.784	81.784	1448	{}	\N	2020-10-12 16:52:34.224605
85168	ONEV	2020-10-12	82.0423	82.59	82.0423	82.58	5106	{}	\N	2020-10-12 16:52:34.231513
85169	IDT	2020-09-28	6.33	6.58	6.33	6.5	54429	{}	\N	2020-10-12 16:52:36.75034
85170	IDT	2020-09-29	6.5	6.73	6.43	6.55	124988	{}	\N	2020-10-12 16:52:36.758987
85171	IDT	2020-09-30	6.68	6.9	6.46	6.58	193022	{}	\N	2020-10-12 16:52:36.765602
85172	IDT	2020-10-01	6.73	6.92	6.58	6.88	874938	{}	\N	2020-10-12 16:52:36.771966
85173	IDT	2020-10-02	8.45	10.49	8.15	9.57	23748722	{}	\N	2020-10-12 16:52:36.777907
85174	IDT	2020-10-05	9.5	10.15	9.21	10	1816074	{}	\N	2020-10-12 16:52:36.78297
85175	IDT	2020-10-06	10.32	11.86	10.32	11.16	1711632	{}	\N	2020-10-12 16:52:36.78777
85176	IDT	2020-10-07	11.18	11.3457	10.8	10.83	564468	{}	\N	2020-10-12 16:52:36.792319
85177	IDT	2020-10-08	10.92	11.66	10.69	10.85	525511	{}	\N	2020-10-12 16:52:36.796681
85178	IDT	2020-10-09	11	11.2	10.525	11.02	357726	{}	\N	2020-10-12 16:52:36.801069
85179	IDT	2020-10-12	11.06	11.19	10.83	10.97	83707	{}	\N	2020-10-12 16:52:36.805423
85180	IJR	2020-09-28	69.29	70.69	69.29	70.43	2519785	{}	\N	2020-10-12 16:52:38.125499
85181	IJR	2020-09-29	70.39	70.49	69.26	70.03	2694399	{}	\N	2020-10-12 16:52:38.137524
85182	IJR	2020-09-30	70.16	71.38	69.65	70.23	5058691	{}	\N	2020-10-12 16:52:38.145536
85183	IJR	2020-10-01	70.69	71.26	69.96	71.23	2720707	{}	\N	2020-10-12 16:52:38.15132
85184	IJR	2020-10-02	69.87	72.27	69.7	71.92	5275503	{}	\N	2020-10-12 16:52:38.157381
85185	IJR	2020-10-05	72.62	73.65	72.61	73.53	2546108	{}	\N	2020-10-12 16:52:38.164046
85186	IJR	2020-10-06	74.15	75.5	73.41	73.51	4341228	{}	\N	2020-10-12 16:52:38.169893
85187	IJR	2020-10-07	74.3	74.91	73.95	74.56	13031555	{}	\N	2020-10-12 16:52:38.175091
85188	IJR	2020-10-08	75.15	75.77	74.83	75.68	3668151	{}	\N	2020-10-12 16:52:38.180085
85189	IJR	2020-10-09	76.33	76.62	75.66	76.08	6521816	{}	\N	2020-10-12 16:52:38.18498
85190	IJR	2020-10-12	76.41	76.62	75.98	76.4	1183252	{}	\N	2020-10-12 16:52:38.190616
85191	IRWD	2020-09-28	9.83	9.87	9.41	9.53	809747	{}	\N	2020-10-12 16:52:39.542207
85192	IRWD	2020-09-29	8.86	9.5	8.63	9.42	8010173	{}	\N	2020-10-12 16:52:39.555394
85193	IRWD	2020-09-30	8.96	9.495	8.7	8.995	4797527	{}	\N	2020-10-12 16:52:39.563848
85194	IRWD	2020-10-01	9.33	9.33	8.655	9.28	6619124	{}	\N	2020-10-12 16:52:39.569617
85195	IRWD	2020-10-02	9.16	9.56	9.0646	9.305	5521348	{}	\N	2020-10-12 16:52:39.574531
85196	IRWD	2020-10-05	9.32	9.48	9.13	9.36	2302396	{}	\N	2020-10-12 16:52:39.579815
85197	IRWD	2020-10-06	9.44	9.8	9.235	9.5	2031254	{}	\N	2020-10-12 16:52:39.583876
85198	IRWD	2020-10-07	9.61	9.885	9.47	9.86	2125554	{}	\N	2020-10-12 16:52:39.588808
85199	IRWD	2020-10-08	9.9	10.505	9.77	10.49	2468080	{}	\N	2020-10-12 16:52:39.592996
85200	IRWD	2020-10-09	10.56	10.62	10.26	10.49	1961613	{}	\N	2020-10-12 16:52:39.597072
85201	IRWD	2020-10-12	10.5	10.6	10.3	10.475	572280	{}	\N	2020-10-12 16:52:39.60099
85202	IXP	2020-09-28	65.63	65.635	65.1786	65.57	24327	{}	\N	2020-10-12 16:52:40.537765
85203	IXP	2020-09-29	65.785	65.9	65.4826	65.74	21210	{}	\N	2020-10-12 16:52:40.551406
85204	IXP	2020-09-30	65.77	66.51	65.71	65.96	7641	{}	\N	2020-10-12 16:52:40.559181
85205	IXP	2020-10-01	66.56	66.84	66.42	66.84	15366	{}	\N	2020-10-12 16:52:40.565849
85206	IXP	2020-10-02	65.76	66.39	65.63	65.72	3848	{}	\N	2020-10-12 16:52:40.574017
85207	IXP	2020-10-05	66.28	66.91	66.23	66.82	18155	{}	\N	2020-10-12 16:52:40.580038
85208	IXP	2020-10-06	66.66	66.98	65.94	65.94	4440	{}	\N	2020-10-12 16:52:40.585356
85209	IXP	2020-10-07	66.35	66.59	66	66.42	3942	{}	\N	2020-10-12 16:52:40.590301
85210	IXP	2020-10-08	66.69	67.23	66.69	67.23	7110	{}	\N	2020-10-12 16:52:40.595019
85211	IXP	2020-10-09	67.59	67.77	67.5201	67.77	8004	{}	\N	2020-10-12 16:52:40.600146
85212	IXP	2020-10-12	68.51	69.36	68.31	69.36	4147	{}	\N	2020-10-12 16:52:40.605305
85213	IYW	2020-09-28	299.49	299.56	296.43	299.46	287497	{}	\N	2020-10-12 16:52:41.755821
85214	IYW	2020-09-29	299.36	300.44	297.95	298.81	105226	{}	\N	2020-10-12 16:52:41.765795
85215	IYW	2020-09-30	298.57	304.08	298.385	301.07	145819	{}	\N	2020-10-12 16:52:41.772865
85216	IYW	2020-10-01	305.3	305.65	303.29	305.14	137482	{}	\N	2020-10-12 16:52:41.779667
85217	IYW	2020-10-02	298.06	303.15	296.5301	297.65	103985	{}	\N	2020-10-12 16:52:41.785662
85218	IYW	2020-10-05	299.78	304.32	299.78	304.31	83325	{}	\N	2020-10-12 16:52:41.790249
85219	IYW	2020-10-06	303.88	306.5	299.14	300.19	102464	{}	\N	2020-10-12 16:52:41.794955
85220	IYW	2020-10-07	302.8	306.13	302.8	305.39	307196	{}	\N	2020-10-12 16:52:41.799855
85221	IYW	2020-10-08	307.95	308.3016	306.17	307.35	74905	{}	\N	2020-10-12 16:52:41.804967
85222	IYW	2020-10-09	309.06	312.03	309.06	311.9	96764	{}	\N	2020-10-12 16:52:41.809754
85223	IYW	2020-10-12	316.89	321.84	315.8599	321.84	51921	{}	\N	2020-10-12 16:52:41.814929
85224	JAZZ	2020-09-28	140.95	141.82	139.99	141.49	462292	{}	\N	2020-10-12 16:52:42.859706
85225	JAZZ	2020-09-29	141	141.12	138.705	139.74	430453	{}	\N	2020-10-12 16:52:42.872613
85226	JAZZ	2020-09-30	139.95	143.86	139.63	142.59	588446	{}	\N	2020-10-12 16:52:42.880548
85227	JAZZ	2020-10-01	141.94	145.87	141.27	144.13	486256	{}	\N	2020-10-12 16:52:42.887012
85228	JAZZ	2020-10-02	142.47	144.77	141.16	143.5	438941	{}	\N	2020-10-12 16:52:42.893004
85229	JAZZ	2020-10-05	145	145	143.31	144.5	532433	{}	\N	2020-10-12 16:52:42.897979
85230	JAZZ	2020-10-06	144.74	144.74	140.28	140.79	620601	{}	\N	2020-10-12 16:52:42.902884
85231	JAZZ	2020-10-07	142.25	144.19	141.1118	143.32	444673	{}	\N	2020-10-12 16:52:42.908329
85232	JAZZ	2020-10-08	148	154.875	147.86	154.82	1595615	{}	\N	2020-10-12 16:52:42.914284
85233	JAZZ	2020-10-09	154.23	155.73	153.06	155.41	847627	{}	\N	2020-10-12 16:52:42.919938
85234	JAZZ	2020-10-12	155.5	155.5	151.95	155.305	219907	{}	\N	2020-10-12 16:52:42.926753
85235	KBWB	2020-09-28	37.08	37.88	37.07	37.56	310050	{}	\N	2020-10-12 16:52:43.834213
85236	KBWB	2020-09-29	37.44	37.44	36.6	37	981669	{}	\N	2020-10-12 16:52:43.842786
85237	KBWB	2020-09-30	37.18	37.95	37.07	37.49	575014	{}	\N	2020-10-12 16:52:43.848905
85238	KBWB	2020-10-01	37.83	37.98	37.24	37.67	643447	{}	\N	2020-10-12 16:52:43.855164
85239	KBWB	2020-10-02	36.91	38.58	36.91	38.44	436050	{}	\N	2020-10-12 16:52:43.861541
85240	KBWB	2020-10-05	38.85	39.67	38.85	39.6	837486	{}	\N	2020-10-12 16:52:43.867845
85241	KBWB	2020-10-06	40.16	40.73	39.13	39.27	2550151	{}	\N	2020-10-12 16:52:43.874779
85242	KBWB	2020-10-07	39.81	40.7548	39.81	40.41	807570	{}	\N	2020-10-12 16:52:43.881168
85243	KBWB	2020-10-08	40.55	41.07	40.38	41.06	490556	{}	\N	2020-10-12 16:52:43.887063
85244	KBWB	2020-10-09	41.2	41.35	40.525	40.89	1043618	{}	\N	2020-10-12 16:52:43.892711
85245	KBWB	2020-10-12	40.92	41.27	40.79	41.06	792130	{}	\N	2020-10-12 16:52:43.898007
85246	KEN	2020-09-28	20.24	20.3402	20.19	20.19	1070	{}	\N	2020-10-12 16:52:45.195147
85247	KEN	2020-09-29	21.26	21.26	21.1757	21.1757	1474	{}	\N	2020-10-12 16:52:45.203829
85248	KEN	2020-09-30	21.66	22.86	21.66	22.86	7498	{}	\N	2020-10-12 16:52:45.210278
85249	KEN	2020-10-02	22.97	22.97	22.97	22.97	372	{}	\N	2020-10-12 16:52:45.216049
85250	KEN	2020-10-05	23.87	24.7	23.87	24.7	13122	{}	\N	2020-10-12 16:52:45.220958
85251	KEN	2020-10-06	24.16	24.35	24.16	24.35	1199	{}	\N	2020-10-12 16:52:45.226489
85252	KEN	2020-10-08	24.33	24.33	23.75	23.75	1354	{}	\N	2020-10-12 16:52:45.23137
85253	KIE	2020-09-28	27.57	28.25	27.57	28.01	317558	{}	\N	2020-10-12 16:52:46.149152
85254	KIE	2020-09-29	27.95	28	27.47	27.83	170481	{}	\N	2020-10-12 16:52:46.162261
85255	KIE	2020-09-30	27.89	28.26	27.8	28.03	160214	{}	\N	2020-10-12 16:52:46.169732
85256	KIE	2020-10-01	28.27	28.57	28.04	28.35	383104	{}	\N	2020-10-12 16:52:46.175226
85257	KIE	2020-10-02	27.94	28.97	27.94	28.8	458421	{}	\N	2020-10-12 16:52:46.180127
85258	KIE	2020-10-05	29.06	29.6	29.06	29.52	591800	{}	\N	2020-10-12 16:52:46.185157
85259	KIE	2020-10-06	29.68	29.91	29.11	29.19	542989	{}	\N	2020-10-12 16:52:46.189892
85260	KIE	2020-10-07	29.46	29.73	29.29	29.52	164885	{}	\N	2020-10-12 16:52:46.194522
85261	KIE	2020-10-08	29.67	30.025	29.62	29.85	122301	{}	\N	2020-10-12 16:52:46.199161
85262	KIE	2020-10-09	30	30.056	29.74	29.8	192006	{}	\N	2020-10-12 16:52:46.203683
85263	KIE	2020-10-12	29.86	30.29	29.79	30.17	58748	{}	\N	2020-10-12 16:52:46.208485
85264	LACQ	2020-09-28	10.25	10.55	10.2	10.55	1247	{}	\N	2020-10-12 16:52:47.358823
85265	LACQ	2020-09-29	10.7	10.7	10.69	10.7	1334	{}	\N	2020-10-12 16:52:47.368529
85266	LACQ	2020-10-01	10.12	10.525	10.11	10.525	691	{}	\N	2020-10-12 16:52:47.374662
85267	LACQ	2020-10-02	10.795	10.795	10.795	10.795	454	{}	\N	2020-10-12 16:52:47.380024
85268	LACQ	2020-10-05	10.46	10.46	10.46	10.46	217	{}	\N	2020-10-12 16:52:47.385492
85269	LACQ	2020-10-07	10.5635	10.5635	10.5635	10.5635	406	{}	\N	2020-10-12 16:52:47.390203
85270	LACQ	2020-10-08	10.45	10.4908	10.45	10.4908	698	{}	\N	2020-10-12 16:52:47.39493
85271	LBJ	2020-09-28	44.43	45	39.69	40.469	32624	{}	\N	2020-10-12 16:52:48.970067
85272	LBJ	2020-09-29	40.22	41.135	38.9948	39	26962	{}	\N	2020-10-12 16:52:48.980395
85273	LBJ	2020-09-30	39.75	41.41	39.75	41.25	26517	{}	\N	2020-10-12 16:52:48.988413
85274	LBJ	2020-10-01	40.73	40.895	39.1434	40.41	18785	{}	\N	2020-10-12 16:52:48.9946
85275	LBJ	2020-10-02	39.81	41.3719	39.55	40.05	7717	{}	\N	2020-10-12 16:52:49.00001
85276	LBJ	2020-10-05	41.55	43.9	41.1647	43.85	16619	{}	\N	2020-10-12 16:52:49.005053
85277	LBJ	2020-10-06	45.6	45.67	41.8738	42.3939	30903	{}	\N	2020-10-12 16:52:49.010825
85278	LBJ	2020-10-07	43.82	43.82	42.2973	42.9	8426	{}	\N	2020-10-12 16:52:49.015966
85279	LBJ	2020-10-08	43	47.31	43	47.125	61966	{}	\N	2020-10-12 16:52:49.021144
85280	LBJ	2020-10-09	47.47	49.76	47.15	48.805	16983	{}	\N	2020-10-12 16:52:49.026293
85281	LBJ	2020-10-12	48.82	49.06	48.5	49.0093	37484	{}	\N	2020-10-12 16:52:49.031877
85282	IFGL	2020-09-28	23.99	24.05	23.9216	24.01	29585	{}	\N	2020-10-12 16:52:50.259767
85283	IFGL	2020-09-29	23.91	23.91	23.74	23.85	45808	{}	\N	2020-10-12 16:52:50.270662
85284	IFGL	2020-09-30	23.91	24.0872	23.91	23.94	24345	{}	\N	2020-10-12 16:52:50.277953
85285	IFGL	2020-10-01	24.13	24.27	24.12	24.27	56589	{}	\N	2020-10-12 16:52:50.283605
85286	IFGL	2020-10-02	24.01	24.32	24.01	24.32	41299	{}	\N	2020-10-12 16:52:50.28883
85287	IFGL	2020-10-05	24.4	24.53	24.4	24.47	31317	{}	\N	2020-10-12 16:52:50.293195
85288	IFGL	2020-10-06	24.43	24.55	24.335	24.38	20154	{}	\N	2020-10-12 16:52:50.297948
85289	IFGL	2020-10-07	24.45	24.55	24.33	24.33	59990	{}	\N	2020-10-12 16:52:50.301933
85290	IFGL	2020-10-08	24.415	24.43	24.37	24.42	46932	{}	\N	2020-10-12 16:52:50.306152
85291	IFGL	2020-10-09	24.483	24.52	24.4	24.45	52024	{}	\N	2020-10-12 16:52:50.310018
85292	IFGL	2020-10-12	24.5	24.61	24.45	24.585	5496	{}	\N	2020-10-12 16:52:50.313801
85293	IQDE	2020-09-28	19.52	19.52	19.5	19.5001	7507	{}	\N	2020-10-12 16:52:51.618598
85294	IQDE	2020-09-29	19.54	19.57	19.53	19.5639	3526	{}	\N	2020-10-12 16:52:51.629988
85295	IQDE	2020-09-30	19.64	19.64	19.52	19.52	5911	{}	\N	2020-10-12 16:52:51.637463
85296	IQDE	2020-10-01	19.62	19.6685	19.6	19.6685	19739	{}	\N	2020-10-12 16:52:51.6436
85297	IQDE	2020-10-02	19.67	19.67	19.66	19.66	115226	{}	\N	2020-10-12 16:52:51.650395
85298	IQDE	2020-10-05	19.75	19.9175	19.75	19.9175	1340	{}	\N	2020-10-12 16:52:51.658003
85299	IQDE	2020-10-06	19.84	19.87	19.67	19.67	8527	{}	\N	2020-10-12 16:52:51.663696
85300	IQDE	2020-10-07	19.8142	19.8406	19.8	19.8406	579	{}	\N	2020-10-12 16:52:51.669174
85301	IQDE	2020-10-08	19.89	19.9944	19.89	19.9944	763	{}	\N	2020-10-12 16:52:51.673969
85302	IQDE	2020-10-09	20.19	20.19	20.17	20.185	1049	{}	\N	2020-10-12 16:52:51.680664
85303	IQDE	2020-10-12	20.235	20.235	20.23	20.23	206	{}	\N	2020-10-12 16:52:51.686124
85304	IR	2020-09-28	36.03	36.77	35.8801	36.05	2459145	{}	\N	2020-10-12 16:52:52.609473
85305	IR	2020-09-29	36.18	36.55	35.67	35.87	2855467	{}	\N	2020-10-12 16:52:52.621486
85306	IR	2020-09-30	35.91	36.4254	35.21	35.6	3989293	{}	\N	2020-10-12 16:52:52.62898
85307	IR	2020-10-01	35.89	36.09	35.13	35.57	2562833	{}	\N	2020-10-12 16:52:52.635872
85308	IR	2020-10-02	34.99	36.58	34.86	36.08	3558320	{}	\N	2020-10-12 16:52:52.643137
85309	IR	2020-10-05	36.59	37.11	36.45	36.91	4354156	{}	\N	2020-10-12 16:52:52.649271
85310	IR	2020-10-06	37.01	37.43	36.38	36.45	3963630	{}	\N	2020-10-12 16:52:52.655679
85311	IR	2020-10-07	36.92	37.15	36.44	36.66	7978809	{}	\N	2020-10-12 16:52:52.661514
85312	IR	2020-10-08	36.9	37.31	36.49	37.23	4192450	{}	\N	2020-10-12 16:52:52.666712
85313	IR	2020-10-09	37.58	37.78	37.1	37.13	3501632	{}	\N	2020-10-12 16:52:52.672182
85314	IR	2020-10-12	37.3	37.41	36.905	37.2	1001234	{}	\N	2020-10-12 16:52:52.67743
85315	IYE	2020-09-28	16.37	16.62	16.34	16.44	1782418	{}	\N	2020-10-12 16:52:54.962112
85316	IYE	2020-09-29	16.4	16.4261	15.85	16.01	1414386	{}	\N	2020-10-12 16:52:54.969183
85317	IYE	2020-09-30	16.12	16.27	15.87	15.96	1501821	{}	\N	2020-10-12 16:52:54.974324
85318	IYE	2020-10-01	15.79	15.85	15.39	15.46	1592953	{}	\N	2020-10-12 16:52:54.979592
85319	IYE	2020-10-02	15.09	15.76	15.04	15.63	2268062	{}	\N	2020-10-12 16:52:54.985043
85320	IYE	2020-10-05	15.8	16.12	15.74	16.11	1799914	{}	\N	2020-10-12 16:52:54.989499
85321	IYE	2020-10-06	16.33	16.43	15.83	15.87	2158670	{}	\N	2020-10-12 16:52:54.994836
85322	IYE	2020-10-07	15.96	16.15	15.8	16.1	1065956	{}	\N	2020-10-12 16:52:54.999712
85323	IYE	2020-10-08	16.25	16.72	16.17	16.7	1588168	{}	\N	2020-10-12 16:52:55.004212
85324	IYE	2020-10-09	16.89	16.89	16.4339	16.46	1597397	{}	\N	2020-10-12 16:52:55.009337
85325	IYE	2020-10-12	16.44	16.47	16.25	16.4	528725	{}	\N	2020-10-12 16:52:55.014415
85326	JKI	2020-09-28	130.9578	131.99	130.9578	131.4117	4025	{}	\N	2020-10-12 16:52:56.931581
85327	JKI	2020-09-29	131.69	131.69	129.69	130.6	6037	{}	\N	2020-10-12 16:52:56.941556
85328	JKI	2020-09-30	130.23	132.2	130.23	131.01	7959	{}	\N	2020-10-12 16:52:56.947778
85329	JKI	2020-10-01	131.86	131.86	130.42	130.42	6512	{}	\N	2020-10-12 16:52:56.953253
85330	JKI	2020-10-02	130.4601	133.135	130.4601	133.135	4219	{}	\N	2020-10-12 16:52:56.958459
85331	JKI	2020-10-05	134.3	135.4924	134.3	135.485	46988	{}	\N	2020-10-12 16:52:56.964204
85332	JKI	2020-10-06	136.59	137.3178	134.47	134.885	21481	{}	\N	2020-10-12 16:52:56.970072
85333	JKI	2020-10-07	136.2867	136.7104	135.805	136.7104	2596	{}	\N	2020-10-12 16:52:56.975499
85334	JKI	2020-10-08	138.28	139.22	137.59	139.18	24470	{}	\N	2020-10-12 16:52:56.980376
85335	JKI	2020-10-09	140.33	140.33	138.615	138.91	4375	{}	\N	2020-10-12 16:52:56.984873
85336	JKI	2020-10-12	138.76	139.47	138.7208	139.47	2930	{}	\N	2020-10-12 16:52:56.989198
85337	JSD	2020-09-28	12.32	12.37	12.32	12.34	55805	{}	\N	2020-10-12 16:52:58.130951
85338	JSD	2020-09-29	12.33	12.4	12.32	12.35	35756	{}	\N	2020-10-12 16:52:58.143545
85339	JSD	2020-09-30	12.29	12.4	12.25	12.38	104724	{}	\N	2020-10-12 16:52:58.151522
85340	JSD	2020-10-01	12.4	12.4599	12.32	12.36	61880	{}	\N	2020-10-12 16:52:58.15787
85341	JSD	2020-10-02	12.32	12.43	12.32	12.39	99515	{}	\N	2020-10-12 16:52:58.162535
85342	JSD	2020-10-05	12.42	12.55	12.42	12.55	47218	{}	\N	2020-10-12 16:52:58.166626
85343	JSD	2020-10-06	12.55	12.55	12.47	12.5	33501	{}	\N	2020-10-12 16:52:58.170859
85344	JSD	2020-10-07	12.58	12.58	12.5	12.5	45682	{}	\N	2020-10-12 16:52:58.175156
85345	JSD	2020-10-08	12.49	12.5412	12.46	12.49	62476	{}	\N	2020-10-12 16:52:58.17942
85346	JSD	2020-10-09	12.47	12.5	12.46	12.46	47168	{}	\N	2020-10-12 16:52:58.183725
85347	JSD	2020-10-12	12.51	12.56	12.476	12.54	11433	{}	\N	2020-10-12 16:52:58.187982
85348	KEX	2020-09-28	36.7	37.47	36.42	37.08	914779	{}	\N	2020-10-12 16:52:59.48997
85349	KEX	2020-09-29	36.99	37.04	35.9	35.93	860363	{}	\N	2020-10-12 16:52:59.501472
85350	KEX	2020-09-30	36.17	37.15	35.9	36.17	803240	{}	\N	2020-10-12 16:52:59.510506
85351	KEX	2020-10-01	36.42	36.835	35.72	36.01	1158811	{}	\N	2020-10-12 16:52:59.517602
85352	KEX	2020-10-02	35.17	36.7	35.1	36.52	1021625	{}	\N	2020-10-12 16:52:59.523544
85353	KEX	2020-10-05	36.82	37.25	36.64	37.12	1622088	{}	\N	2020-10-12 16:52:59.528491
85354	KEX	2020-10-06	37.69	38.31	36.37	36.73	1730227	{}	\N	2020-10-12 16:52:59.533489
85355	KEX	2020-10-07	37.12	38.35	36.885	38.26	3196491	{}	\N	2020-10-12 16:52:59.538339
85356	KEX	2020-10-08	38.68	39.55	38.21	39.05	1267085	{}	\N	2020-10-12 16:52:59.543057
85357	KEX	2020-10-09	39.31	39.61	38.38	38.51	1493519	{}	\N	2020-10-12 16:52:59.548057
85358	KEX	2020-10-12	38.65	38.65	37.65	37.95	170290	{}	\N	2020-10-12 16:52:59.552819
85359	LAD	2020-09-28	223.99	229.9	221.09	228.32	220071	{}	\N	2020-10-12 16:53:02.124517
85360	LAD	2020-09-29	228.04	228.32	222.3	222.95	358679	{}	\N	2020-10-12 16:53:02.135436
85361	LAD	2020-09-30	220.04	229.9	215.21	227.94	1620068	{}	\N	2020-10-12 16:53:02.143139
85362	LAD	2020-10-01	234.22	248.3844	232.24	246.77	5216933	{}	\N	2020-10-12 16:53:02.149586
85363	LAD	2020-10-02	242.12	274.86	241.51	271.51	3478240	{}	\N	2020-10-12 16:53:02.155282
85364	LAD	2020-10-05	272.27	284.63	271.89	282.18	2152544	{}	\N	2020-10-12 16:53:02.161414
85365	LAD	2020-10-06	283.7	285.99	263.44	269.09	1204780	{}	\N	2020-10-12 16:53:02.166825
85366	LAD	2020-10-07	275.87	288.56	274.2207	283.79	925481	{}	\N	2020-10-12 16:53:02.172552
85367	LAD	2020-10-08	285.1	285.83	274.26	281.01	1082363	{}	\N	2020-10-12 16:53:02.178163
85368	LAD	2020-10-09	284.16	285.81	278.53	283.62	614615	{}	\N	2020-10-12 16:53:02.183722
85369	LAD	2020-10-12	285	285.66	275.78	275.985	248128	{}	\N	2020-10-12 16:53:02.189308
85370	IIN	2020-09-28	11.92	12.07	11.6511	12.06	27295	{}	\N	2020-10-12 16:53:05.063047
85371	IIN	2020-09-29	12.0507	12.23	11.74	12.2	16432	{}	\N	2020-10-12 16:53:05.074759
85372	IIN	2020-09-30	12.26	12.455	12.12	12.18	28981	{}	\N	2020-10-12 16:53:05.082004
85373	IIN	2020-10-01	12.215	12.42	12.19	12.27	25398	{}	\N	2020-10-12 16:53:05.089192
85374	IIN	2020-10-02	11.92	12.285	11.92	11.96	26361	{}	\N	2020-10-12 16:53:05.096709
85375	IIN	2020-10-05	12.1267	12.48	12.075	12.39	51364	{}	\N	2020-10-12 16:53:05.101963
85376	IIN	2020-10-06	12.4	13.2	12.33	12.89	69643	{}	\N	2020-10-12 16:53:05.107518
85377	IIN	2020-10-07	12.82	13.32	12.82	13.3	14063	{}	\N	2020-10-12 16:53:05.113449
85378	IIN	2020-10-08	13.42	13.75	13.39	13.66	17839	{}	\N	2020-10-12 16:53:05.118375
85379	IIN	2020-10-09	13.6	14	13.6	13.78	15560	{}	\N	2020-10-12 16:53:05.123863
85380	IIN	2020-10-12	13.87	13.87	13.57	13.71	3679	{}	\N	2020-10-12 16:53:05.128969
85381	INVH	2020-09-28	28.05	28.58	27.88	28.54	2819044	{}	\N	2020-10-12 16:53:06.080863
85382	INVH	2020-09-29	28.53	28.53	27.91	28	7811988	{}	\N	2020-10-12 16:53:06.091656
85383	INVH	2020-09-30	28.15	28.24	27.57	27.99	10379725	{}	\N	2020-10-12 16:53:06.099583
85384	INVH	2020-10-01	28.16	28.78	28.16	28.77	10841035	{}	\N	2020-10-12 16:53:06.106645
85385	INVH	2020-10-02	28.27	29.39	28.12	29.31	7424490	{}	\N	2020-10-12 16:53:06.113233
85386	INVH	2020-10-05	29.49	29.61	29.15	29.52	4956569	{}	\N	2020-10-12 16:53:06.119131
85387	INVH	2020-10-06	29.62	29.75	29.23	29.41	5191546	{}	\N	2020-10-12 16:53:06.124811
85388	INVH	2020-10-07	29.65	29.73	28.89	29.23	5575517	{}	\N	2020-10-12 16:53:06.13022
85389	INVH	2020-10-08	29.25	29.9	29.23	29.87	5004148	{}	\N	2020-10-12 16:53:06.135454
85390	INVH	2020-10-09	30.08	30.13	29.55	29.63	5509922	{}	\N	2020-10-12 16:53:06.1423
85391	INVH	2020-10-12	29.59	29.725	29.26	29.55	1112798	{}	\N	2020-10-12 16:53:06.148269
85392	IQDG	2020-09-28	32.52	32.57	32.465	32.57	3781	{}	\N	2020-10-12 16:53:07.084084
85393	IQDG	2020-09-29	32.7847	32.802	32.6299	32.7	41186	{}	\N	2020-10-12 16:53:07.09391
85394	IQDG	2020-09-30	32.55	32.69	32.48	32.65	8966	{}	\N	2020-10-12 16:53:07.100793
85395	IQDG	2020-10-01	32.81	32.9094	32.7551	32.9094	15184	{}	\N	2020-10-12 16:53:07.106769
85396	IQDG	2020-10-02	32.69	32.845	32.69	32.825	6179	{}	\N	2020-10-12 16:53:07.113319
85397	IQDG	2020-10-05	33	33.14	33	33.14	4905	{}	\N	2020-10-12 16:53:07.118624
85398	IQDG	2020-10-06	33.08	33.08	32.6	32.677	46051	{}	\N	2020-10-12 16:53:07.124244
85399	IQDG	2020-10-07	32.96	33.0099	32.9	33.0043	6133	{}	\N	2020-10-12 16:53:07.129284
85400	IQDG	2020-10-08	33.12	33.18	33.06	33.15	20381	{}	\N	2020-10-12 16:53:07.134161
85401	IQDG	2020-10-09	33.34	33.53	33.34	33.515	9407	{}	\N	2020-10-12 16:53:07.139051
85402	IQDG	2020-10-12	33.71	33.772	33.6994	33.772	2561	{}	\N	2020-10-12 16:53:07.143861
85403	KOLD	2020-09-28	33.62	33.94	31.65	32.45	744484	{}	\N	2020-10-12 16:53:09.454822
85404	KOLD	2020-09-29	36.11	38.94	35.68	38.88	1272387	{}	\N	2020-10-12 16:53:09.461967
85405	KOLD	2020-09-30	37.89	39.7	36.8506	38.01	733206	{}	\N	2020-10-12 16:53:09.468278
85406	KOLD	2020-10-01	38.31	40.89	37.89	39.23	668603	{}	\N	2020-10-12 16:53:09.474341
85407	KOLD	2020-10-02	40.07	41.15	37.6	40.07	968911	{}	\N	2020-10-12 16:53:09.479929
85408	KOLD	2020-10-05	34.01	35.19	31.33	34.27	2877578	{}	\N	2020-10-12 16:53:09.485368
85409	KOLD	2020-10-06	35.96	38	34.4	37.45	1183626	{}	\N	2020-10-12 16:53:09.490588
85410	KOLD	2020-10-07	33.3401	36.97	33	35.07	1044194	{}	\N	2020-10-12 16:53:09.495783
85411	KOLD	2020-10-08	36.25	36.705	34.09	34.56	1173756	{}	\N	2020-10-12 16:53:09.501114
85412	KOLD	2020-10-09	31.54	32.72	30.9	32.29	2121939	{}	\N	2020-10-12 16:53:09.506106
85413	KOLD	2020-10-12	29.97	31.14	29.76	30.3	798707	{}	\N	2020-10-12 16:53:09.510965
85414	KTOS	2020-09-28	19.3	19.7	19.1496	19.35	1043109	{}	\N	2020-10-12 16:53:13.380859
85415	KTOS	2020-09-29	19.35	19.57	19.095	19.46	655584	{}	\N	2020-10-12 16:53:13.39086
85416	KTOS	2020-09-30	19.57	19.77	19	19.28	896190	{}	\N	2020-10-12 16:53:13.397921
85417	KTOS	2020-10-01	19.39	19.58	19.14	19.4	616292	{}	\N	2020-10-12 16:53:13.404164
85418	KTOS	2020-10-02	19.18	19.69	18.95	19.62	472229	{}	\N	2020-10-12 16:53:13.409961
85419	KTOS	2020-10-05	19.86	20.16	19.68	20.02	712566	{}	\N	2020-10-12 16:53:13.415237
85420	KTOS	2020-10-06	20.24	20.47	19.69	19.7	588372	{}	\N	2020-10-12 16:53:13.420123
85421	KTOS	2020-10-07	20.09	20.87	20.08	20.43	990960	{}	\N	2020-10-12 16:53:13.424504
85422	KTOS	2020-10-08	20.73	21.105	20.48	21.01	631104	{}	\N	2020-10-12 16:53:13.429406
85423	KTOS	2020-10-09	21.25	21.59	20.8	20.89	763033	{}	\N	2020-10-12 16:53:13.435415
85424	KTOS	2020-10-12	20.82	21.4	20.57	21.24	329130	{}	\N	2020-10-12 16:53:13.43997
85425	LABU	2020-09-28	54.14	54.14	51.32	52.77	1578023	{}	\N	2020-10-12 16:53:15.149773
85426	LABU	2020-09-29	52.69	54.87	51.62	54.05	1187551	{}	\N	2020-10-12 16:53:15.160211
85427	LABU	2020-09-30	53.98	56.1373	52.1633	53.55	1381990	{}	\N	2020-10-12 16:53:15.167448
85428	LABU	2020-10-01	54.3	55.69	53.1	55.69	1497574	{}	\N	2020-10-12 16:53:15.173855
85429	LABU	2020-10-02	52.4	55.9	51.1	51.7	1867854	{}	\N	2020-10-12 16:53:15.179574
85430	LABU	2020-10-05	54.18	59.79	54.04	59.63	2196189	{}	\N	2020-10-12 16:53:15.184911
85431	LABU	2020-10-06	59.84	61.87	57.34	58.1	1476485	{}	\N	2020-10-12 16:53:15.190051
85432	LABU	2020-10-07	59.56	64.46	59.46	63.68	1354054	{}	\N	2020-10-12 16:53:15.194761
85433	LABU	2020-10-08	65.62	65.7	62.95	64.42	1194113	{}	\N	2020-10-12 16:53:15.199924
85434	LABU	2020-10-09	65.6365	66.73	64.072	66.34	1114824	{}	\N	2020-10-12 16:53:15.205353
85435	LABU	2020-10-12	68	68.38	65.55	67.2	588150	{}	\N	2020-10-12 16:53:15.210786
85436	LITB	2020-09-28	1.58	1.735	1.52	1.63	441633	{}	\N	2020-10-12 16:53:16.748296
85437	LITB	2020-09-29	1.62	1.7871	1.5801	1.75	659909	{}	\N	2020-10-12 16:53:16.75845
85438	LITB	2020-09-30	1.76	2.0299	1.75	1.93	1954258	{}	\N	2020-10-12 16:53:16.765563
85439	LITB	2020-10-01	1.89	1.94	1.85	1.88	421373	{}	\N	2020-10-12 16:53:16.771106
85440	LITB	2020-10-02	1.79	1.8819	1.7899	1.87	530942	{}	\N	2020-10-12 16:53:16.777889
85441	LITB	2020-10-05	1.86	1.9	1.83	1.89	224328	{}	\N	2020-10-12 16:53:16.783201
85442	LITB	2020-10-06	1.84	1.97	1.799	1.81	341512	{}	\N	2020-10-12 16:53:16.78887
85443	LITB	2020-10-07	1.86	1.97	1.86	1.97	357250	{}	\N	2020-10-12 16:53:16.793614
85444	LITB	2020-10-08	2.01	2.45	2.01	2.3	3027847	{}	\N	2020-10-12 16:53:16.798231
85445	LITB	2020-10-09	2.34	2.76	2.23	2.74	3056746	{}	\N	2020-10-12 16:53:16.803189
85446	LITB	2020-10-12	2.81	2.84	2.51	2.52	1690357	{}	\N	2020-10-12 16:53:16.807942
85447	VST	2020-09-28	18.16	18.21	17.935	17.97	3760606	{}	\N	2020-10-12 16:53:19.98446
85448	VST	2020-09-29	18.15	18.86	18.15	18.53	17084492	{}	\N	2020-10-12 16:53:19.999257
85449	VST	2020-09-30	18.67	18.98	18.33	18.86	15465808	{}	\N	2020-10-12 16:53:20.006844
85450	VST	2020-10-01	18.94	18.96	17.855	17.89	10659365	{}	\N	2020-10-12 16:53:20.013555
85451	VST	2020-10-02	17.44	18.04	17.21	17.99	8201790	{}	\N	2020-10-12 16:53:20.018861
85452	VST	2020-10-05	18.04	18.225	17.79	17.81	11652480	{}	\N	2020-10-12 16:53:20.025715
85453	VST	2020-10-06	17.85	18.22	17.74	17.95	11631376	{}	\N	2020-10-12 16:53:20.032447
85454	VST	2020-10-07	18	18.04	17.76	17.83	11507856	{}	\N	2020-10-12 16:53:20.037818
85455	VST	2020-10-08	17.94	18.31	17.9	18.28	10462826	{}	\N	2020-10-12 16:53:20.043061
85456	VST	2020-10-09	18.36	18.43	17.99	17.99	6776082	{}	\N	2020-10-12 16:53:20.048729
85457	VST	2020-10-12	17.94	18.1001	17.78	18.09	1988862	{}	\N	2020-10-12 16:53:20.054095
85458	INDB	2020-09-28	50.69	51.68	50.26	50.34	130241	{}	\N	2020-10-12 16:53:22.037118
85459	INDB	2020-09-29	50.19	51.24	49.25	50.63	118123	{}	\N	2020-10-12 16:53:22.047623
85460	INDB	2020-09-30	50.78	52.76	50.78	52.38	247037	{}	\N	2020-10-12 16:53:22.055206
85461	INDB	2020-10-01	52.12	52.95	51.49	52.15	133558	{}	\N	2020-10-12 16:53:22.061749
85462	INDB	2020-10-02	51.11	53.54	51.0047	52.89	149679	{}	\N	2020-10-12 16:53:22.068313
85463	INDB	2020-10-05	53.49	54.58	52.4125	54.31	108534	{}	\N	2020-10-12 16:53:22.073995
85464	INDB	2020-10-06	55.22	56.81	54.3	55.36	175928	{}	\N	2020-10-12 16:53:22.080918
85465	INDB	2020-10-07	55.69	56.67	54.87	55.34	130103	{}	\N	2020-10-12 16:53:22.087267
85466	INDB	2020-10-08	55.93	56.43	55.31	55.72	65393	{}	\N	2020-10-12 16:53:22.093077
85467	INDB	2020-10-09	56.44	56.58	54.97	55.04	108402	{}	\N	2020-10-12 16:53:22.099759
85468	INDB	2020-10-12	55	55.39	54.565	55.14	13724	{}	\N	2020-10-12 16:53:22.105617
85469	INKM	2020-09-28	31.03	31.0676	30.995	31.0676	1870	{}	\N	2020-10-12 16:53:23.09101
85470	INKM	2020-09-29	30.955	31	30.944	31	1652	{}	\N	2020-10-12 16:53:23.104753
85471	INKM	2020-09-30	31.145	31.145	31.03	31.03	5517	{}	\N	2020-10-12 16:53:23.113056
85472	INKM	2020-10-01	31.13	31.1557	31.11	31.1557	6129	{}	\N	2020-10-12 16:53:23.120239
85473	INKM	2020-10-02	31.2302	31.28	31.2302	31.26	2716	{}	\N	2020-10-12 16:53:23.12632
85474	INKM	2020-10-05	31.4111	31.464	31.4111	31.464	312	{}	\N	2020-10-12 16:53:23.133249
85475	INKM	2020-10-06	31.57	31.64	31.3672	31.3672	3153	{}	\N	2020-10-12 16:53:23.140543
85476	INKM	2020-10-07	31.575	31.575	31.575	31.575	186	{}	\N	2020-10-12 16:53:23.146329
85477	INKM	2020-10-08	31.7	31.79	31.69	31.78	7392	{}	\N	2020-10-12 16:53:23.152878
85478	INKM	2020-10-09	31.8249	31.8249	31.7815	31.7815	842	{}	\N	2020-10-12 16:53:23.158668
85479	INKM	2020-10-12	31.8001	31.859	31.8	31.859	2835	{}	\N	2020-10-12 16:53:23.164582
85480	IP	2020-09-28	41.51	41.76	40.675	41.17	3261527	{}	\N	2020-10-12 16:53:24.48253
85481	IP	2020-09-29	41.26	41.26	39.8	40.28	5295911	{}	\N	2020-10-12 16:53:24.50268
85482	IP	2020-09-30	40.35	41.01	40.2	40.54	4862696	{}	\N	2020-10-12 16:53:24.512821
85483	IP	2020-10-01	40.79	41.1612	39.835	39.95	5225463	{}	\N	2020-10-12 16:53:24.522595
85484	IP	2020-10-02	39.31	40.77	39.1501	40.48	5414890	{}	\N	2020-10-12 16:53:24.532213
85485	IP	2020-10-05	41.05	41.82	40.87	41.43	4829514	{}	\N	2020-10-12 16:53:24.54119
85486	IP	2020-10-06	41.5	41.74	40.63	40.67	5122548	{}	\N	2020-10-12 16:53:24.550171
85487	IP	2020-10-07	41.09	41.98	40.965	41.81	5533509	{}	\N	2020-10-12 16:53:24.564592
85488	IP	2020-10-08	41.81	42.51	41.79	42.49	4077947	{}	\N	2020-10-12 16:53:24.573236
85489	IP	2020-10-09	43.55	44.2	42.67	42.67	9286599	{}	\N	2020-10-12 16:53:24.582917
85490	IP	2020-10-12	43.94	45.02	43.36	44.8	5285865	{}	\N	2020-10-12 16:53:24.591612
85491	IPO	2020-09-28	52.26	52.62	51.38	51.65	377089	{}	\N	2020-10-12 16:53:25.921048
85492	IPO	2020-09-29	51.9	52.1999	51.6537	51.85	266987	{}	\N	2020-10-12 16:53:25.932546
85493	IPO	2020-09-30	51.9	52.74	51.7038	52.28	259607	{}	\N	2020-10-12 16:53:25.940404
85494	IPO	2020-10-01	52.8	53.44	52.5446	53.39	265019	{}	\N	2020-10-12 16:53:25.946771
85495	IPO	2020-10-02	51.57	53.6799	51.5	52.76	349651	{}	\N	2020-10-12 16:53:25.952754
85496	IPO	2020-10-05	53.498	53.69	53.0201	53.59	144268	{}	\N	2020-10-12 16:53:25.95777
85497	IPO	2020-10-06	53.82	54.44	53.02	53.37	184041	{}	\N	2020-10-12 16:53:25.962894
85498	IPO	2020-10-07	53.86	54.955	53.86	54.71	214521	{}	\N	2020-10-12 16:53:25.969051
85499	IPO	2020-10-08	55.24	55.3399	54.695	54.81	184385	{}	\N	2020-10-12 16:53:25.973914
85500	IPO	2020-10-09	55.44	56	55.21	55.9	212185	{}	\N	2020-10-12 16:53:25.978993
85501	IPO	2020-10-12	56.61	56.77	56.11	56.49	170139	{}	\N	2020-10-12 16:53:25.983734
85502	IQLT	2020-09-28	31.39	31.44	31.29	31.44	557497	{}	\N	2020-10-12 16:53:27.579235
85503	IQLT	2020-09-29	31.46	31.55	31.33	31.455	720994	{}	\N	2020-10-12 16:53:27.589219
85504	IQLT	2020-09-30	31.3799	31.54	31.22	31.29	539126	{}	\N	2020-10-12 16:53:27.595702
85505	IQLT	2020-10-01	31.55	31.61	31.38	31.6	1794839	{}	\N	2020-10-12 16:53:27.602853
85506	IQLT	2020-10-02	31.26	31.62	31.24	31.56	617217	{}	\N	2020-10-12 16:53:27.608974
85507	IQLT	2020-10-05	31.79	32	31.78	31.99	417174	{}	\N	2020-10-12 16:53:27.615061
85508	IQLT	2020-10-06	31.89	31.92	31.495	31.57	934710	{}	\N	2020-10-12 16:53:27.621078
85509	IQLT	2020-10-07	31.73	31.895	31.72	31.84	313555	{}	\N	2020-10-12 16:53:27.626516
85510	IQLT	2020-10-08	31.9653	32.1	31.93	32.1	325607	{}	\N	2020-10-12 16:53:27.632973
85511	IQLT	2020-10-09	32.27	32.3925	32.22	32.39	264250	{}	\N	2020-10-12 16:53:27.63974
85512	IQLT	2020-10-12	32.49	32.6278	32.43	32.6	334093	{}	\N	2020-10-12 16:53:27.646756
85513	IROQ	2020-09-28	15.25	15.25	15.25	15.25	371	{}	\N	2020-10-12 16:53:29.843349
85514	IROQ	2020-09-29	15.32	15.32	15.32	15.32	301	{}	\N	2020-10-12 16:53:29.850735
85515	IROQ	2020-09-30	15.45	15.48	15.45	15.47	1552	{}	\N	2020-10-12 16:53:29.856311
85516	IROQ	2020-10-01	15.46	15.46	15.25	15.25	2640	{}	\N	2020-10-12 16:53:29.86157
85517	IROQ	2020-10-05	15.33	15.33	15.33	15.33	356	{}	\N	2020-10-12 16:53:29.86637
85518	IROQ	2020-10-06	15.64	16.13	15.64	15.7	4351	{}	\N	2020-10-12 16:53:29.870903
85519	IROQ	2020-10-07	15.69	15.7	15.69	15.7	926	{}	\N	2020-10-12 16:53:29.875066
85520	IROQ	2020-10-08	15.8005	15.8005	15.75	15.75	693	{}	\N	2020-10-12 16:53:29.879161
85521	IROQ	2020-10-09	15.915	16.43	15.89	16.43	3880	{}	\N	2020-10-12 16:53:29.883229
85522	IWS	2020-09-28	80.54	81.55	80.54	81.18	673396	{}	\N	2020-10-12 16:53:31.446125
85523	IWS	2020-09-29	81.35	81.35	80.08	80.47	530451	{}	\N	2020-10-12 16:53:31.45968
85524	IWS	2020-09-30	80.76	81.69	80.31	80.84	549692	{}	\N	2020-10-12 16:53:31.466849
85525	IWS	2020-10-01	81.24	81.75	80.7405	81.44	546585	{}	\N	2020-10-12 16:53:31.472287
85526	IWS	2020-10-02	80.08	82.6248	80.01	82.28	305293	{}	\N	2020-10-12 16:53:31.477691
85527	IWS	2020-10-05	83.04	83.9779	83.04	83.86	291476	{}	\N	2020-10-12 16:53:31.482507
85528	IWS	2020-10-06	84.33	85.18	83.07	83.22	940388	{}	\N	2020-10-12 16:53:31.486901
85529	IWS	2020-10-07	84.07	84.87	84.07	84.67	430954	{}	\N	2020-10-12 16:53:31.491068
85530	IWS	2020-10-08	85.18	85.97	85.06	85.9	457232	{}	\N	2020-10-12 16:53:31.495497
85531	IWS	2020-10-09	86.47	86.6375	85.67	85.87	306551	{}	\N	2020-10-12 16:53:31.499974
85532	IWS	2020-10-12	86.28	86.46	85.94	86.3557	225452	{}	\N	2020-10-12 16:53:31.504189
85533	IXUS	2020-09-28	58.26	58.38	58.13	58.34	1164400	{}	\N	2020-10-12 16:53:32.685686
85534	IXUS	2020-09-29	58.26	58.45	58.06	58.25	974470	{}	\N	2020-10-12 16:53:32.698873
85535	IXUS	2020-09-30	58.29	58.66	58.17	58.34	1707047	{}	\N	2020-10-12 16:53:32.706949
85536	IXUS	2020-10-01	58.71	58.75	58.41	58.75	1466771	{}	\N	2020-10-12 16:53:32.713681
85537	IXUS	2020-10-02	57.98	58.6375	57.94	58.49	1020902	{}	\N	2020-10-12 16:53:32.72015
85538	IXUS	2020-10-05	58.93	59.36	58.91	59.34	955778	{}	\N	2020-10-12 16:53:32.725366
85539	IXUS	2020-10-06	59.4011	59.5854	58.78	58.89	1050711	{}	\N	2020-10-12 16:53:32.73069
85540	IXUS	2020-10-07	59.38	59.55	59.27	59.42	1419937	{}	\N	2020-10-12 16:53:32.736025
85541	IXUS	2020-10-08	59.76	59.92	59.67	59.89	456193	{}	\N	2020-10-12 16:53:32.741066
85542	IXUS	2020-10-09	60.09	60.39	60.07	60.34	620828	{}	\N	2020-10-12 16:53:32.745819
85543	IXUS	2020-10-12	60.54	60.779	60.46	60.74	308909	{}	\N	2020-10-12 16:53:32.75372
85544	IYM	2020-09-28	98.11	99.4317	98.11	98.57	36607	{}	\N	2020-10-12 16:53:34.81606
85545	IYM	2020-09-29	98.53	98.9923	97.66	97.76	64195	{}	\N	2020-10-12 16:53:34.824566
85546	IYM	2020-09-30	98	99.57	98	98.75	178464	{}	\N	2020-10-12 16:53:34.830777
85547	IYM	2020-10-01	99.3	99.54	97.08	97.38	2138135	{}	\N	2020-10-12 16:53:34.835813
85548	IYM	2020-10-02	96.3	99.01	96.3	98.35	310281	{}	\N	2020-10-12 16:53:34.840154
85549	IYM	2020-10-05	99.22	100.86	99.22	100.16	233285	{}	\N	2020-10-12 16:53:34.844
85550	IYM	2020-10-06	101.02	101.6	98.95	98.95	193036	{}	\N	2020-10-12 16:53:34.848618
85551	IYM	2020-10-07	100.31	101.7996	100.31	101.56	57453	{}	\N	2020-10-12 16:53:34.853042
85552	IYM	2020-10-08	102.33	102.78	102.07	102.33	51810	{}	\N	2020-10-12 16:53:34.857166
85553	IYM	2020-10-09	103.25	103.64	102.5901	103.33	79494	{}	\N	2020-10-12 16:53:34.862512
85554	IYM	2020-10-12	103.88	103.88	102.99	103.5487	33987	{}	\N	2020-10-12 16:53:34.867886
85555	JAGX	2020-09-28	0.319	0.319	0.30231	0.306	2385187	{}	\N	2020-10-12 16:53:35.762127
85556	JAGX	2020-09-29	0.31	0.314799	0.29	0.297	3113070	{}	\N	2020-10-12 16:53:35.773591
85557	JAGX	2020-09-30	0.2941	0.2999	0.2839	0.2895	3451393	{}	\N	2020-10-12 16:53:35.782017
85558	JAGX	2020-10-01	0.319	0.32	0.29	0.3031	9098892	{}	\N	2020-10-12 16:53:35.789601
85559	JAGX	2020-10-02	0.3	0.3043	0.2911	0.3	2454229	{}	\N	2020-10-12 16:53:35.796021
85560	JAGX	2020-10-05	0.3	0.3149	0.292	0.31	5286457	{}	\N	2020-10-12 16:53:35.802082
85561	JAGX	2020-10-06	0.3402	0.407	0.31	0.3318	50986923	{}	\N	2020-10-12 16:53:35.808892
85562	JAGX	2020-10-07	0.3368	0.3466	0.3075	0.3169	12020010	{}	\N	2020-10-12 16:53:35.815675
85563	JAGX	2020-10-08	0.3362	0.34	0.32031	0.3326	4754738	{}	\N	2020-10-12 16:53:35.821641
85564	JAGX	2020-10-09	0.334	0.34	0.3158	0.3221	2939159	{}	\N	2020-10-12 16:53:35.827244
85565	JAGX	2020-10-12	0.3245	0.3245	0.3142	0.3199	1567299	{}	\N	2020-10-12 16:53:35.831893
85566	JPNL	2020-09-28	52.15	52.7507	52.15	52.74	14888	{}	\N	2020-10-12 16:53:37.728294
85567	JPNL	2020-09-29	52.8	53.2619	52.549	52.73	6263	{}	\N	2020-10-12 16:53:37.737825
85568	JPNL	2020-09-30	51.675	51.9301	51.4201	51.52	3124	{}	\N	2020-10-12 16:53:37.744373
85569	JPNL	2020-10-01	51.56	51.59	51.03	51.59	3756	{}	\N	2020-10-12 16:53:37.74999
85570	JPNL	2020-10-02	49.43	50.7134	49.43	50.6719	3840	{}	\N	2020-10-12 16:53:37.755283
85571	JPNL	2020-10-05	51.52	51.92	51.47	51.92	2718	{}	\N	2020-10-12 16:53:37.760346
85572	JPNL	2020-10-06	52.28	52.4323	51.91	51.91	1744	{}	\N	2020-10-12 16:53:37.764779
85573	JPNL	2020-10-07	52.25	52.4	52.25	52.28	1706	{}	\N	2020-10-12 16:53:37.769699
85574	JPNL	2020-10-08	52.8	52.94	52.3599	52.3599	956	{}	\N	2020-10-12 16:53:37.774113
85575	JPNL	2020-10-09	52.33	52.5	52.33	52.4294	7886	{}	\N	2020-10-12 16:53:37.778693
85576	JPNL	2020-10-12	52.83	53.09	52.42	53.09	1860	{}	\N	2020-10-12 16:53:37.783091
85577	K	2020-09-28	63.56	63.8	62.83	63.38	1289449	{}	\N	2020-10-12 16:53:39.951162
85578	K	2020-09-29	63.77	63.87	63.08	63.44	3225423	{}	\N	2020-10-12 16:53:39.96121
85579	K	2020-09-30	63.73	64.74	63.535	64.59	3957718	{}	\N	2020-10-12 16:53:39.968492
85580	K	2020-10-01	64.72	64.8	63.6606	64.38	3179262	{}	\N	2020-10-12 16:53:39.97547
85581	K	2020-10-02	63.77	65.22	63.4	64.71	2851352	{}	\N	2020-10-12 16:53:39.98167
85582	K	2020-10-05	64.84	65.52	64.51	65.45	3598151	{}	\N	2020-10-12 16:53:39.987297
85583	K	2020-10-06	65.49	65.855	64.8627	65.16	2603125	{}	\N	2020-10-12 16:53:39.992561
85584	K	2020-10-07	65.14	65.56	64.36	64.86	2588359	{}	\N	2020-10-12 16:53:39.997122
85585	K	2020-10-08	65.06	65.43	64.74	65.22	1517093	{}	\N	2020-10-12 16:53:40.002088
85586	K	2020-10-09	65.56	66.22	65.15	65.85	2447973	{}	\N	2020-10-12 16:53:40.007162
85587	K	2020-10-12	66	66.7	65.89	66.455	1347586	{}	\N	2020-10-12 16:53:40.012358
85588	OUSM	2020-09-28	26.04	26.349	26.04	26.3	20700	{}	\N	2020-10-12 16:53:41.97295
85589	OUSM	2020-09-29	26.279	26.279	25.98	26.18	7613	{}	\N	2020-10-12 16:53:41.982383
85590	OUSM	2020-09-30	26.2801	26.455	26.155	26.155	6656	{}	\N	2020-10-12 16:53:41.988823
85591	OUSM	2020-10-01	26.43	26.43	26.21	26.285	7509	{}	\N	2020-10-12 16:53:41.995484
85592	OUSM	2020-10-02	25.9	26.669	25.9	26.655	12617	{}	\N	2020-10-12 16:53:42.001223
85593	OUSM	2020-10-05	26.8799	27.055	26.8501	27.041	15224	{}	\N	2020-10-12 16:53:42.006206
85594	OUSM	2020-10-06	27.1	27.4899	26.975	26.98	36248	{}	\N	2020-10-12 16:53:42.011544
85595	OUSM	2020-10-07	27.2891	27.36	27.1684	27.32	7701	{}	\N	2020-10-12 16:53:42.016683
85596	OUSM	2020-10-08	27.59	27.7429	27.59	27.7429	7938	{}	\N	2020-10-12 16:53:42.022162
85597	OUSM	2020-10-09	27.85	27.9882	27.77	27.83	8694	{}	\N	2020-10-12 16:53:42.027546
85598	OUSM	2020-10-12	27.95	28.18	27.95	28.15	5034	{}	\N	2020-10-12 16:53:42.032864
85599	INFN	2020-09-28	5.94	6	5.89	5.96	1102174	{}	\N	2020-10-12 16:53:43.141659
85600	INFN	2020-09-29	5.96	6.255	5.94	6.19	1412539	{}	\N	2020-10-12 16:53:43.154976
85601	INFN	2020-09-30	6.19	6.27	6.09	6.16	1554500	{}	\N	2020-10-12 16:53:43.163357
85602	INFN	2020-10-01	6.24	6.245	6.045	6.13	1178585	{}	\N	2020-10-12 16:53:43.169467
85603	INFN	2020-10-02	5.95	6.145	5.9326	6.13	920037	{}	\N	2020-10-12 16:53:43.174924
85604	INFN	2020-10-05	6.2	6.29	6.1822	6.28	1226383	{}	\N	2020-10-12 16:53:43.180017
85605	INFN	2020-10-06	6.22	6.42	6.22	6.31	1544316	{}	\N	2020-10-12 16:53:43.184701
85606	INFN	2020-10-07	6.48	6.5	6.345	6.45	1222476	{}	\N	2020-10-12 16:53:43.189146
85607	INFN	2020-10-08	6.59	6.9	6.5	6.77	1975932	{}	\N	2020-10-12 16:53:43.193799
85608	INFN	2020-10-09	6.77	6.9881	6.69	6.94	2053600	{}	\N	2020-10-12 16:53:43.198596
85609	INFN	2020-10-12	6.98	7	6.75	6.81	829993	{}	\N	2020-10-12 16:53:43.203962
85610	IPAC	2020-09-28	56.63	56.84	56.5184	56.8	102711	{}	\N	2020-10-12 16:53:44.365577
85611	IPAC	2020-09-29	56.74	56.755	56.5401	56.59	26162	{}	\N	2020-10-12 16:53:44.379356
85612	IPAC	2020-09-30	56.11	56.5011	56.09	56.25	35941	{}	\N	2020-10-12 16:53:44.386911
85613	IPAC	2020-10-01	56.41	56.479	56.2	56.44	91455	{}	\N	2020-10-12 16:53:44.393794
85614	IPAC	2020-10-02	55.76	56.3288	55.76	56.2395	14065	{}	\N	2020-10-12 16:53:44.399847
85615	IPAC	2020-10-05	56.65	56.9	56.61	56.9	44509	{}	\N	2020-10-12 16:53:44.405248
85616	IPAC	2020-10-06	56.9201	57.0561	56.5928	56.69	23715	{}	\N	2020-10-12 16:53:44.410483
85617	IPAC	2020-10-07	56.97	57.12	56.96	57.12	21681	{}	\N	2020-10-12 16:53:44.416076
85618	IPAC	2020-10-08	57.28	57.3699	57.15	57.25	28877	{}	\N	2020-10-12 16:53:44.421133
85619	IPAC	2020-10-09	57.27	57.4291	57.27	57.3525	13637	{}	\N	2020-10-12 16:53:44.426097
85620	IPAC	2020-10-12	57.47	57.6665	57.36	57.6665	10278	{}	\N	2020-10-12 16:53:44.430875
85621	IPKW	2020-09-28	31.55	31.56	31.47	31.53	43561	{}	\N	2020-10-12 16:53:45.370097
85622	IPKW	2020-09-29	31.66	31.66	31.39	31.49	6742	{}	\N	2020-10-12 16:53:45.384153
85623	IPKW	2020-09-30	31.42	31.56	31.31	31.33	18918	{}	\N	2020-10-12 16:53:45.393372
85624	IPKW	2020-10-01	31.6	31.7	31.46	31.7	9000	{}	\N	2020-10-12 16:53:45.399703
85625	IPKW	2020-10-02	31.95	31.97	31.8899	31.9135	44262	{}	\N	2020-10-12 16:53:45.404845
85626	IPKW	2020-10-05	32.21	32.495	32.21	32.49	8526	{}	\N	2020-10-12 16:53:45.409719
85627	IPKW	2020-10-06	32.56	32.6275	32.2595	32.28	4436	{}	\N	2020-10-12 16:53:45.414603
85628	IPKW	2020-10-07	32.7	32.79	32.7	32.79	5884	{}	\N	2020-10-12 16:53:45.419998
85629	IPKW	2020-10-08	33.0953	33.24	33.0953	33.23	7932	{}	\N	2020-10-12 16:53:45.424934
85630	IPKW	2020-10-09	33.65	33.77	33.645	33.685	6806	{}	\N	2020-10-12 16:53:45.430116
85631	IPKW	2020-10-12	33.67	33.7895	33.635	33.7895	36778	{}	\N	2020-10-12 16:53:45.434875
85632	IQDF	2020-09-28	20.6	20.7	20.585	20.66	53679	{}	\N	2020-10-12 16:53:46.360914
85633	IQDF	2020-09-29	20.72	20.73	20.5801	20.655	62725	{}	\N	2020-10-12 16:53:46.373628
85634	IQDF	2020-09-30	20.64	20.7588	20.6	20.68	126945	{}	\N	2020-10-12 16:53:46.38175
85635	IQDF	2020-10-01	20.8	20.8	20.69	20.78	40965	{}	\N	2020-10-12 16:53:46.387668
85636	IQDF	2020-10-02	20.55	20.8229	20.55	20.76	38944	{}	\N	2020-10-12 16:53:46.393072
85637	IQDF	2020-10-05	20.86	21.02	20.86	21	46849	{}	\N	2020-10-12 16:53:46.39798
85638	IQDF	2020-10-06	21	21.02	20.76	20.785	47615	{}	\N	2020-10-12 16:53:46.40263
85639	IQDF	2020-10-07	20.9	20.97	20.87	20.945	46497	{}	\N	2020-10-12 16:53:46.407091
85640	IQDF	2020-10-08	21.04	21.12	21.02	21.105	42974	{}	\N	2020-10-12 16:53:46.411466
85641	IQDF	2020-10-09	21.27	21.38	21.25	21.31	173033	{}	\N	2020-10-12 16:53:46.415779
85642	IQDF	2020-10-12	21.37	21.44	21.35	21.43	17025	{}	\N	2020-10-12 16:53:46.420186
85643	IRET	2020-09-28	64.86	67.14	64.55	67.09	73705	{}	\N	2020-10-12 16:53:47.389086
85644	IRET	2020-09-29	66.29	66.64	64.76	65.95	130567	{}	\N	2020-10-12 16:53:47.403489
85645	IRET	2020-09-30	65.98	67.32	64.13	65.17	303153	{}	\N	2020-10-12 16:53:47.411262
85646	IRET	2020-10-01	65.01	67.255	65.01	67.14	138606	{}	\N	2020-10-12 16:53:47.419136
85647	IRET	2020-10-02	66.02	68.68	66.02	68.27	89329	{}	\N	2020-10-12 16:53:47.424739
85648	IRET	2020-10-05	68.66	68.66	67	67.96	105162	{}	\N	2020-10-12 16:53:47.430193
85649	IRET	2020-10-06	68.02	69.99	67.69	68.54	120823	{}	\N	2020-10-12 16:53:47.43576
85650	IRET	2020-10-07	69.08	69.226	67.405	67.42	96950	{}	\N	2020-10-12 16:53:47.440684
85651	IRET	2020-10-08	68.01	69.03	68.01	68.91	64341	{}	\N	2020-10-12 16:53:47.44535
85652	IRET	2020-10-09	69.54	69.825	68.5	69.45	76446	{}	\N	2020-10-12 16:53:47.450092
85653	IRET	2020-10-12	69.18	69.78	68.55	69.405	13883	{}	\N	2020-10-12 16:53:47.454926
85654	IXJ	2020-09-28	71.31	71.6	71.0901	71.23	108908	{}	\N	2020-10-12 16:53:49.194521
85655	IXJ	2020-09-29	71.26	71.65	70.9251	71.17	115727	{}	\N	2020-10-12 16:53:49.205889
85656	IXJ	2020-09-30	71.26	72.29	71.26	71.82	246070	{}	\N	2020-10-12 16:53:49.21378
85657	IXJ	2020-10-01	72.21	72.24	71.3701	71.61	224151	{}	\N	2020-10-12 16:53:49.220404
85658	IXJ	2020-10-02	70.89	71.59	70.765	71.08	101776	{}	\N	2020-10-12 16:53:49.226168
85659	IXJ	2020-10-05	71.5	72.31	71.42	72.25	67476	{}	\N	2020-10-12 16:53:49.230819
85660	IXJ	2020-10-06	72.34	72.34	71.27	71.31	41458	{}	\N	2020-10-12 16:53:49.235374
85661	IXJ	2020-10-07	71.64	72.32	71.56	72.11	117872	{}	\N	2020-10-12 16:53:49.239532
85662	IXJ	2020-10-08	72.54	72.79	72.44	72.66	64051	{}	\N	2020-10-12 16:53:49.243793
85663	IXJ	2020-10-09	73.08	73.53	72.99	73.37	82340	{}	\N	2020-10-12 16:53:49.248733
85664	IXJ	2020-10-12	73.76	74.07	73.55	74.015	22342	{}	\N	2020-10-12 16:53:49.253634
85665	JHMF	2020-09-28	34.0461	34.0461	34.0461	34.0461	289	{}	\N	2020-10-12 16:53:51.021661
85666	JHMF	2020-09-29	33.72	33.85	33.72	33.8407	24618	{}	\N	2020-10-12 16:53:51.033956
85667	JHMF	2020-09-30	33.81	34.32	33.81	34.32	1441	{}	\N	2020-10-12 16:53:51.041611
85668	JHMF	2020-10-01	34.25	34.31	34.2008	34.2008	782	{}	\N	2020-10-12 16:53:51.047649
85669	JHMF	2020-10-02	34.4904	34.4904	34.4904	34.4904	135	{}	\N	2020-10-12 16:53:51.053235
85670	JHMF	2020-10-05	35.05	35.1515	35.05	35.1515	828	{}	\N	2020-10-12 16:53:51.058351
85671	JHMF	2020-10-06	35.39	35.54	34.8073	34.8073	732	{}	\N	2020-10-12 16:53:51.063701
85672	JHMF	2020-10-07	35.13	35.38	35.13	35.38	1352	{}	\N	2020-10-12 16:53:51.069231
85673	JHMF	2020-10-08	35.5	35.88	35.5	35.8	1604	{}	\N	2020-10-12 16:53:51.074828
85674	JHMF	2020-10-09	35.9466	35.9466	35.9466	35.9466	210	{}	\N	2020-10-12 16:53:51.080103
85675	JHMF	2020-10-12	35.99	36.0222	35.99	36.0222	688	{}	\N	2020-10-12 16:53:51.085668
85676	JPUS	2020-09-28	73.36	74.0399	73.36	73.67	13852	{}	\N	2020-10-12 16:53:52.457953
85677	JPUS	2020-09-29	73.73	74	73.23	73.35	58503	{}	\N	2020-10-12 16:53:52.469351
85678	JPUS	2020-09-30	74.09	74.54	73.53	73.95	84910	{}	\N	2020-10-12 16:53:52.47659
85679	JPUS	2020-10-01	74.26	74.5402	73.91	74.13	23920	{}	\N	2020-10-12 16:53:52.481699
85680	JPUS	2020-10-02	73.6	74.62	73.6	74.4	20120	{}	\N	2020-10-12 16:53:52.48678
85681	JPUS	2020-10-05	75.24	75.7	75.24	75.7	8936	{}	\N	2020-10-12 16:53:52.491775
85682	JPUS	2020-10-06	76.07	76.57	74.9257	75.03	30956	{}	\N	2020-10-12 16:53:52.496544
85683	JPUS	2020-10-07	76	76.3599	75.9006	76.315	30986	{}	\N	2020-10-12 16:53:52.500892
85684	JPUS	2020-10-08	76.905	77.1195	76.821	77.1195	10167	{}	\N	2020-10-12 16:53:52.505745
85685	JPUS	2020-10-09	77.6	77.8	77.29	77.56	10459	{}	\N	2020-10-12 16:53:52.50984
85686	JPUS	2020-10-12	77.8	78.2236	77.7	78.2236	4625	{}	\N	2020-10-12 16:53:52.514002
85687	KFYP	2020-09-28	29.75	29.83	29.56	29.56	2313	{}	\N	2020-10-12 16:53:53.752042
85688	KFYP	2020-09-29	29.5	29.71	29.3401	29.425	1846	{}	\N	2020-10-12 16:53:53.764745
85689	KFYP	2020-09-30	29.42	29.5	29.05	29.5	2240	{}	\N	2020-10-12 16:53:53.772425
85690	KFYP	2020-10-01	30	30.54	29.88	30.13	26584	{}	\N	2020-10-12 16:53:53.77837
85691	KFYP	2020-10-02	29.39	29.87	29.21	29.81	1789	{}	\N	2020-10-12 16:53:53.784083
85692	KFYP	2020-10-05	29.59	29.8899	29.3971	29.83	3084	{}	\N	2020-10-12 16:53:53.789571
85693	KFYP	2020-10-06	29.93	30.13	29.93	30.05	1552	{}	\N	2020-10-12 16:53:53.794332
85694	KFYP	2020-10-07	29.98	30.75	29.65	30.35	20509	{}	\N	2020-10-12 16:53:53.799213
85695	KFYP	2020-10-08	30.04	30.62	30.04	30.6	985	{}	\N	2020-10-12 16:53:53.803933
85696	KFYP	2020-10-09	30.37	30.37	30.125	30.37	2720	{}	\N	2020-10-12 16:53:53.80853
85697	KFYP	2020-10-12	30.49	30.7	30.22	30.4737	3162	{}	\N	2020-10-12 16:53:53.813087
85698	KIO	2020-09-28	13.61	13.67	13.5201	13.59	99869	{}	\N	2020-10-12 16:53:55.135306
85699	KIO	2020-09-29	13.57	13.629	13.43	13.46	209252	{}	\N	2020-10-12 16:53:55.148312
85700	KIO	2020-09-30	13.55	13.6	13.45	13.53	179121	{}	\N	2020-10-12 16:53:55.157042
85701	KIO	2020-10-01	13.53	13.78	13.53	13.74	284929	{}	\N	2020-10-12 16:53:55.163741
85702	KIO	2020-10-02	13.69	13.8	13.6304	13.8	137680	{}	\N	2020-10-12 16:53:55.169663
85703	KIO	2020-10-05	13.91	14.01	13.77	14	244341	{}	\N	2020-10-12 16:53:55.174853
85704	KIO	2020-10-06	14.03	14.13	14	14.1	240416	{}	\N	2020-10-12 16:53:55.179832
85705	KIO	2020-10-07	14.13	14.15	14.1	14.14	188468	{}	\N	2020-10-12 16:53:55.184819
85706	KIO	2020-10-08	14.12	14.125	13.87	14.02	191992	{}	\N	2020-10-12 16:53:55.189936
85707	KIO	2020-10-09	14.12	14.12	13.91	13.92	182703	{}	\N	2020-10-12 16:53:55.19478
85708	KIO	2020-10-12	13.92	14.0366	13.92	13.9725	64683	{}	\N	2020-10-12 16:53:55.199815
85709	KMF	2020-09-28	4.1	4.2199	4.05	4.15	148589	{}	\N	2020-10-12 16:53:56.052721
85710	KMF	2020-09-29	4.15	4.18	4.07	4.12	609761	{}	\N	2020-10-12 16:53:56.062566
85711	KMF	2020-09-30	4.16	4.19	4	4.03	1059733	{}	\N	2020-10-12 16:53:56.069339
85712	KMF	2020-10-01	4.02	4.06	3.99	4	260601	{}	\N	2020-10-12 16:53:56.075882
85713	KMF	2020-10-02	3.9	4.07	3.9	4.05	263511	{}	\N	2020-10-12 16:53:56.082221
85714	KMF	2020-10-05	4.09	4.3	4.09	4.3	574858	{}	\N	2020-10-12 16:53:56.08774
85715	KMF	2020-10-06	4.35	4.42	4.22	4.3	675360	{}	\N	2020-10-12 16:53:56.093204
85716	KMF	2020-10-07	4.3	4.5	4.26	4.38	992662	{}	\N	2020-10-12 16:53:56.098624
85717	KMF	2020-10-08	4.42	4.64	4.4	4.63	517954	{}	\N	2020-10-12 16:53:56.104603
85718	KMF	2020-10-09	4.65	4.69	4.57	4.6	667358	{}	\N	2020-10-12 16:53:56.109714
85719	KMF	2020-10-12	4.61	4.645	4.54	4.57	138602	{}	\N	2020-10-12 16:53:56.114916
85720	LACQU	2020-09-28	10.7893	10.8005	10.7	10.7	3404	{}	\N	2020-10-12 16:53:56.94006
85721	LACQU	2020-10-02	11	11	11	11	172	{}	\N	2020-10-12 16:53:56.94868
85722	IRR	2020-09-28	2.44	2.46	2.43	2.43	194107	{}	\N	2020-10-12 16:53:58.705772
85723	IRR	2020-09-29	2.44	2.44	2.39	2.41	165270	{}	\N	2020-10-12 16:53:58.717834
85724	IRR	2020-09-30	2.42	2.46	2.3977	2.44	446534	{}	\N	2020-10-12 16:53:58.728984
85725	IRR	2020-10-01	2.32	2.37	2.3	2.34	327624	{}	\N	2020-10-12 16:53:58.735081
85726	IRR	2020-10-02	2.26	2.34	2.2526	2.31	240990	{}	\N	2020-10-12 16:53:58.740463
85727	IRR	2020-10-05	2.32	2.36	2.2901	2.34	171168	{}	\N	2020-10-12 16:53:58.745235
85728	IRR	2020-10-06	2.36	2.4	2.33	2.33	127574	{}	\N	2020-10-12 16:53:58.749823
85729	IRR	2020-10-07	2.34	2.3657	2.33	2.36	160724	{}	\N	2020-10-12 16:53:58.754564
85730	IRR	2020-10-08	2.38	2.41	2.34	2.39	369756	{}	\N	2020-10-12 16:53:58.759288
85731	IRR	2020-10-09	2.4	2.4	2.36	2.36	85055	{}	\N	2020-10-12 16:53:58.764
85732	IRR	2020-10-12	2.35	2.37	2.35	2.3602	16708	{}	\N	2020-10-12 16:53:58.76872
85733	JBT	2020-09-28	96.27	98.55	95.8	95.94	222448	{}	\N	2020-10-12 16:54:02.156677
85734	JBT	2020-09-29	95.91	96.4	93.89	94.13	394348	{}	\N	2020-10-12 16:54:02.170218
85735	JBT	2020-09-30	94	94.5	91.46	91.89	1074779	{}	\N	2020-10-12 16:54:02.177129
85736	JBT	2020-10-01	92.02	93.3	89.09	89.39	643009	{}	\N	2020-10-12 16:54:02.183386
85737	JBT	2020-10-02	88.0508	90.27	87.56	89.53	540531	{}	\N	2020-10-12 16:54:02.188221
85738	JBT	2020-10-05	90.96	92.78	90.38	90.82	488127	{}	\N	2020-10-12 16:54:02.192899
85739	JBT	2020-10-06	91.98	92.05	89.24	89.31	643045	{}	\N	2020-10-12 16:54:02.197076
85740	JBT	2020-10-07	90	90.58	88.57	89.27	619094	{}	\N	2020-10-12 16:54:02.200722
85741	JBT	2020-10-08	90.17	90.5	89.38	89.91	413042	{}	\N	2020-10-12 16:54:02.204688
85742	JBT	2020-10-09	91.08	91.48	89.52	90.23	228406	{}	\N	2020-10-12 16:54:02.208701
85743	JBT	2020-10-12	90.21	90.74	89.45	90.34	61399	{}	\N	2020-10-12 16:54:02.212872
85744	JHMM	2020-09-28	37.32	37.68	37.32	37.53	157641	{}	\N	2020-10-12 16:54:03.381946
85745	JHMM	2020-09-29	37.53	37.6787	37.2593	37.41	79250	{}	\N	2020-10-12 16:54:03.392544
85746	JHMM	2020-09-30	37.53	37.955	37.35	37.62	49803	{}	\N	2020-10-12 16:54:03.403091
85747	JHMM	2020-10-01	37.82	38.04	37.675	37.91	126740	{}	\N	2020-10-12 16:54:03.40923
85748	JHMM	2020-10-02	37.29	38.2	37.18	38.06	109612	{}	\N	2020-10-12 16:54:03.415343
85749	JHMM	2020-10-05	38.39	38.9	38.39	38.88	221384	{}	\N	2020-10-12 16:54:03.420628
85750	JHMM	2020-10-06	39.07	39.46	38.54	38.64	148629	{}	\N	2020-10-12 16:54:03.425598
85751	JHMM	2020-10-07	39.01	39.3997	39.01	39.29	168537	{}	\N	2020-10-12 16:54:03.430509
85752	JHMM	2020-10-08	39.68	39.82	39.51	39.82	159672	{}	\N	2020-10-12 16:54:03.434916
85753	JHMM	2020-10-09	40.09	40.14	39.84	39.95	230069	{}	\N	2020-10-12 16:54:03.439127
85754	JHMM	2020-10-12	40.23	40.28	40.121	40.2599	27867	{}	\N	2020-10-12 16:54:03.443371
85755	JQUA	2020-09-28	32.25	32.45	32.25	32.41	866339	{}	\N	2020-10-12 16:54:04.825242
85756	JQUA	2020-09-29	32.35	32.46	32.2275	32.28	26990	{}	\N	2020-10-12 16:54:04.836646
85757	JQUA	2020-09-30	32.41	32.75	32.3001	32.45	30448	{}	\N	2020-10-12 16:54:04.843127
85758	JQUA	2020-10-01	32.67	32.705	32.52	32.66	25655	{}	\N	2020-10-12 16:54:04.849602
85759	JQUA	2020-10-02	32.11	32.56	32.11	32.41	51040	{}	\N	2020-10-12 16:54:04.855214
85760	JQUA	2020-10-05	32.62	32.9461	32.62	32.93	27106	{}	\N	2020-10-12 16:54:04.860063
85761	JQUA	2020-10-06	33	33.25	32.54	32.61	31581	{}	\N	2020-10-12 16:54:04.864655
85762	JQUA	2020-10-07	32.9501	33.15	32.93	33.13	40747	{}	\N	2020-10-12 16:54:04.869618
85763	JQUA	2020-10-08	33.38	33.46	33.29	33.46	24726	{}	\N	2020-10-12 16:54:04.874213
85764	JQUA	2020-10-09	33.57	33.79	33.57	33.69	38689	{}	\N	2020-10-12 16:54:04.880034
85765	JQUA	2020-10-12	33.935	34.155	33.9317	34.155	4791	{}	\N	2020-10-12 16:54:04.884707
85766	KIDS	2020-09-28	44.82	45.11	43.305	45.07	122261	{}	\N	2020-10-12 16:54:07.656508
85767	KIDS	2020-09-29	45.13	46.825	45.13	46.39	182576	{}	\N	2020-10-12 16:54:07.668136
85768	KIDS	2020-09-30	46.65	47.22	45.52	45.92	214176	{}	\N	2020-10-12 16:54:07.676304
85769	KIDS	2020-10-01	45.83	46.985	45.0327	46.42	134351	{}	\N	2020-10-12 16:54:07.682735
85770	KIDS	2020-10-02	45.51	45.9	45.07	45.56	114270	{}	\N	2020-10-12 16:54:07.688336
85771	KIDS	2020-10-05	46.05	48.24	45.76	47.93	133654	{}	\N	2020-10-12 16:54:07.693361
85772	KIDS	2020-10-06	48.44	50.305	48.21	48.8	173989	{}	\N	2020-10-12 16:54:07.698848
85773	KIDS	2020-10-07	50.81	54.16	50.07	51.59	243774	{}	\N	2020-10-12 16:54:07.703778
85774	KIDS	2020-10-08	50.61	51.81	47.56	47.79	461948	{}	\N	2020-10-12 16:54:07.708626
85775	KIDS	2020-10-09	48.19	48.33	46.27	46.78	177312	{}	\N	2020-10-12 16:54:07.713461
85776	KIDS	2020-10-12	46.49	46.49	44.9	45.89	83924	{}	\N	2020-10-12 16:54:07.717962
85777	LADR	2020-09-28	7	7.5	7	7.31	1147135	{}	\N	2020-10-12 16:54:08.985509
85778	LADR	2020-09-29	7.27	7.355	7.04	7.22	1461732	{}	\N	2020-10-12 16:54:08.996575
85779	LADR	2020-09-30	7.21	7.4711	7.07	7.12	1855434	{}	\N	2020-10-12 16:54:09.003838
85780	LADR	2020-10-01	7.2	7.4	7.07	7.39	2008969	{}	\N	2020-10-12 16:54:09.009817
85781	LADR	2020-10-02	7.21	7.62	7.1	7.61	1863207	{}	\N	2020-10-12 16:54:09.014983
85782	LADR	2020-10-05	7.64	7.67	7.49	7.53	1293240	{}	\N	2020-10-12 16:54:09.020216
85783	LADR	2020-10-06	7.55	7.65	7.26	7.29	2172196	{}	\N	2020-10-12 16:54:09.025442
85784	LADR	2020-10-07	7.36	7.45	7.165	7.31	1999360	{}	\N	2020-10-12 16:54:09.030429
85785	LADR	2020-10-08	7.4	7.71	7.31	7.66	1620372	{}	\N	2020-10-12 16:54:09.03519
85786	LADR	2020-10-09	7.72	7.74	7.28	7.28	1736459	{}	\N	2020-10-12 16:54:09.040052
85787	LADR	2020-10-12	7.28	7.36	7.2301	7.285	565502	{}	\N	2020-10-12 16:54:09.045
85788	LBTYB	2020-09-28	22.7	22.7	21.3726	21.3726	1132	{}	\N	2020-10-12 16:54:10.617216
85789	LBTYB	2020-10-02	21.4	21.4	21.4	21.4	262	{}	\N	2020-10-12 16:54:10.628642
85790	LBTYB	2020-10-06	20.77	20.77	20.77	20.77	228	{}	\N	2020-10-12 16:54:10.637038
85791	LBTYB	2020-10-07	20.931	20.931	20.931	20.931	402	{}	\N	2020-10-12 16:54:10.643948
85792	LBTYB	2020-10-08	21.655	21.655	21.655	21.655	348	{}	\N	2020-10-12 16:54:10.650086
85793	LBTYB	2020-10-12	22.44	22.44	22.44	22.44	225	{}	\N	2020-10-12 16:54:10.655407
85794	LEDS	2020-09-28	2.95	3.24	2.71	2.87	379946	{}	\N	2020-10-12 16:54:11.628099
85795	LEDS	2020-09-29	2.88	2.88	2.71	2.73	27706	{}	\N	2020-10-12 16:54:11.639231
85796	LEDS	2020-09-30	2.68	2.7364	2.6207	2.68	31250	{}	\N	2020-10-12 16:54:11.646923
85797	LEDS	2020-10-01	2.67	2.725	2.63	2.725	7973	{}	\N	2020-10-12 16:54:11.653038
85798	LEDS	2020-10-02	2.66	2.93	2.66	2.91	25273	{}	\N	2020-10-12 16:54:11.660085
85799	LEDS	2020-10-05	2.94	2.94	2.79	2.9	34110	{}	\N	2020-10-12 16:54:11.664822
85800	LEDS	2020-10-06	2.87	3.02	2.85	2.89	48039	{}	\N	2020-10-12 16:54:11.669685
85801	LEDS	2020-10-07	2.88	3.16	2.8598	3	56915	{}	\N	2020-10-12 16:54:11.674408
85802	LEDS	2020-10-08	3	3.015	2.9001	2.945	6593	{}	\N	2020-10-12 16:54:11.679049
85803	LEDS	2020-10-09	3.02	3.02	2.9	3	18945	{}	\N	2020-10-12 16:54:11.68355
85804	LEDS	2020-10-12	2.94	2.97	2.85	2.9445	8665	{}	\N	2020-10-12 16:54:11.689081
85805	LSBK	2020-09-28	13.4	13.4	13.4	13.4	934	{}	\N	2020-10-12 16:54:14.610645
85806	LSBK	2020-09-29	13.4	13.95	13.4	13.4	1633	{}	\N	2020-10-12 16:54:14.622279
85807	LSBK	2020-10-01	13.42	13.45	13.42	13.45	1030	{}	\N	2020-10-12 16:54:14.63002
85808	LSBK	2020-10-02	13.64	13.64	13.64	13.64	302	{}	\N	2020-10-12 16:54:14.636662
85809	LSBK	2020-10-05	13.45	13.6	13.45	13.54	627	{}	\N	2020-10-12 16:54:14.643318
85810	LSBK	2020-10-06	13.11	13.11	13.11	13.11	292	{}	\N	2020-10-12 16:54:14.648504
85811	LSBK	2020-10-07	13.6	13.6663	13.6	13.6663	2371	{}	\N	2020-10-12 16:54:14.653457
85812	LSBK	2020-10-08	13.7632	13.7632	13.66	13.66	5757	{}	\N	2020-10-12 16:54:14.658182
85813	LSBK	2020-10-12	14	14.0001	14	14.0001	1512	{}	\N	2020-10-12 16:54:14.663633
85814	MAIN	2020-09-28	30.08	30.641	29.93	30.36	358887	{}	\N	2020-10-12 16:54:16.668395
85815	MAIN	2020-09-29	30.27	30.3	29.49	29.68	819486	{}	\N	2020-10-12 16:54:16.677788
85816	MAIN	2020-09-30	29.56	30.2	29.44	29.57	815688	{}	\N	2020-10-12 16:54:16.685292
85817	MAIN	2020-10-01	29.69	30.69	29.68	30.62	650082	{}	\N	2020-10-12 16:54:16.690819
85818	MAIN	2020-10-02	29.8	30.83	29.69	30.75	617131	{}	\N	2020-10-12 16:54:16.695868
85819	MAIN	2020-10-05	30.78	30.945	30.4	30.42	589317	{}	\N	2020-10-12 16:54:16.700932
85820	MAIN	2020-10-06	30.48	30.7034	29.95	30	601003	{}	\N	2020-10-12 16:54:16.705246
85821	MAIN	2020-10-07	30.15	30.52	29.95	30.27	590156	{}	\N	2020-10-12 16:54:16.709417
85822	MAIN	2020-10-08	30.26	30.69	30.26	30.65	368971	{}	\N	2020-10-12 16:54:16.713981
85823	MAIN	2020-10-09	30.75	31.16	30.685	30.71	463744	{}	\N	2020-10-12 16:54:16.718557
85824	MAIN	2020-10-12	30.72	30.8867	30.13	30.2238	366066	{}	\N	2020-10-12 16:54:16.722892
85825	MAV	2020-09-28	11.05	11.06	10.94	10.97	88837	{}	\N	2020-10-12 16:54:18.322571
85826	MAV	2020-09-29	10.96	11.0099	10.92	10.99	42058	{}	\N	2020-10-12 16:54:18.331654
85827	MAV	2020-09-30	11.08	11.08	10.92	10.92	84355	{}	\N	2020-10-12 16:54:18.337432
85828	MAV	2020-10-01	11.05	11.09	11.02	11.06	152341	{}	\N	2020-10-12 16:54:18.343132
85829	MAV	2020-10-02	11.06	11.08	10.9456	11.01	61424	{}	\N	2020-10-12 16:54:18.348628
85830	MAV	2020-10-05	11.07	11.1	10.9	10.97	55788	{}	\N	2020-10-12 16:54:18.35358
85831	MAV	2020-10-06	11.01	11.03	10.92	11.02	95211	{}	\N	2020-10-12 16:54:18.357805
85832	MAV	2020-10-07	11.1	11.18	11.04	11.1	136760	{}	\N	2020-10-12 16:54:18.361751
85833	MAV	2020-10-08	11.15	11.15	11.02	11.06	99160	{}	\N	2020-10-12 16:54:18.365858
85834	MAV	2020-10-09	11.1	11.1	10.93	10.93	112812	{}	\N	2020-10-12 16:54:18.369741
85835	MAV	2020-10-12	11.06	11.16	10.9601	10.9958	89806	{}	\N	2020-10-12 16:54:18.373786
85836	IYK	2020-09-28	148.79	149.09	148.24	148.79	28670	{}	\N	2020-10-12 16:54:20.311039
85837	IYK	2020-09-29	149.18	149.18	147.74	147.82	25532	{}	\N	2020-10-12 16:54:20.322579
85838	IYK	2020-09-30	148.32	150.31	148.32	149.35	20864	{}	\N	2020-10-12 16:54:20.330739
85839	IYK	2020-10-01	151.4	151.4	149.73	151.37	95738	{}	\N	2020-10-12 16:54:20.337549
85840	IYK	2020-10-02	148.74	150.21	148.52	149.54	21914	{}	\N	2020-10-12 16:54:20.343391
85841	IYK	2020-10-05	150.75	151.0967	150.2197	151.03	68517	{}	\N	2020-10-12 16:54:20.348695
85842	IYK	2020-10-06	151.7	151.8	149.31	149.31	10344	{}	\N	2020-10-12 16:54:20.35414
85843	IYK	2020-10-07	150.89	152.345	150.8425	152.08	7581	{}	\N	2020-10-12 16:54:20.359824
85844	IYK	2020-10-08	153.35	153.46	152.66	152.995	19901	{}	\N	2020-10-12 16:54:20.36487
85845	IYK	2020-10-09	153.46	154.8	153.46	154.4	12389	{}	\N	2020-10-12 16:54:20.369817
85846	IYK	2020-10-12	155.53	156.57	155.53	156.555	5010	{}	\N	2020-10-12 16:54:20.374473
85847	IZEA	2020-09-28	0.9864	1.0265	0.97	0.98	1171564	{}	\N	2020-10-12 16:54:22.289722
85848	IZEA	2020-09-29	0.98	1.02	0.97	1.015	1286954	{}	\N	2020-10-12 16:54:22.301543
85849	IZEA	2020-09-30	1.03	1.0419	0.95	0.9954	2489763	{}	\N	2020-10-12 16:54:22.309643
85850	IZEA	2020-10-01	0.9797	0.988	0.9501	0.9607	979350	{}	\N	2020-10-12 16:54:22.3158
85851	IZEA	2020-10-02	0.91	0.9555	0.9011	0.93	866250	{}	\N	2020-10-12 16:54:22.321166
85852	IZEA	2020-10-05	0.91	0.959	0.91	0.9427	633334	{}	\N	2020-10-12 16:54:22.326025
85853	IZEA	2020-10-06	0.9433	0.9544	0.88	0.9103	1202975	{}	\N	2020-10-12 16:54:22.330225
85854	IZEA	2020-10-07	0.91	0.94	0.9	0.9273	963123	{}	\N	2020-10-12 16:54:22.334572
85855	IZEA	2020-10-08	0.92	0.95	0.9155	0.9305	924881	{}	\N	2020-10-12 16:54:22.33918
85856	IZEA	2020-10-09	0.92	0.95	0.92	0.9379	1131450	{}	\N	2020-10-12 16:54:22.343816
85857	IZEA	2020-10-12	0.93	0.9598	0.923	0.9349	762616	{}	\N	2020-10-12 16:54:22.348187
85858	JAX	2020-09-28	5.3	5.37	5.095	5.1	82520	{}	\N	2020-10-12 16:54:23.313247
85859	JAX	2020-09-29	5.09	5.32	5.05	5.23	167983	{}	\N	2020-10-12 16:54:23.322344
85860	JAX	2020-09-30	5.23	5.36	5.15	5.2	161158	{}	\N	2020-10-12 16:54:23.328785
85861	JAX	2020-10-01	5.18	5.345	5.146	5.3	128283	{}	\N	2020-10-12 16:54:23.334835
85862	JAX	2020-10-02	5.16	5.29	5.12	5.27	122889	{}	\N	2020-10-12 16:54:23.339923
85863	JAX	2020-10-05	5.22	5.57	5.22	5.38	266411	{}	\N	2020-10-12 16:54:23.344619
85864	JAX	2020-10-06	5.43	5.8	5.34	5.46	293649	{}	\N	2020-10-12 16:54:23.348714
85865	JAX	2020-10-07	5.51	5.65	5.48	5.59	135282	{}	\N	2020-10-12 16:54:23.353632
85866	JAX	2020-10-08	5.635	5.635	5.46	5.56	143428	{}	\N	2020-10-12 16:54:23.357777
85867	JAX	2020-10-09	5.41	5.45	5.21	5.4	283901	{}	\N	2020-10-12 16:54:23.362267
85868	JAX	2020-10-12	5.42	5.45	5.35	5.37	30639	{}	\N	2020-10-12 16:54:23.366497
85869	JHME	2020-09-28	11.7235	11.7235	11.7235	11.7235	138	{}	\N	2020-10-12 16:54:24.473807
85870	JHME	2020-09-29	11.31	11.46	11.31	11.46	181947	{}	\N	2020-10-12 16:54:24.483442
85871	JHME	2020-09-30	11.3504	11.3504	11.3504	11.3504	450	{}	\N	2020-10-12 16:54:24.489834
85872	JHME	2020-10-01	11.21	11.21	10.925	10.925	7704	{}	\N	2020-10-12 16:54:24.495797
85873	JHME	2020-10-02	11.1772	11.1772	11.1425	11.1425	178	{}	\N	2020-10-12 16:54:24.501831
85874	JHME	2020-10-05	11.36	11.503	11.36	11.503	391	{}	\N	2020-10-12 16:54:24.506265
85875	JHME	2020-10-06	11.63	11.64	11.3	11.3	3584	{}	\N	2020-10-12 16:54:24.510459
85876	JHME	2020-10-07	11.41	11.4726	11.2999	11.4726	5106	{}	\N	2020-10-12 16:54:24.514246
85877	JHME	2020-10-08	11.887	11.8873	11.887	11.8873	1593	{}	\N	2020-10-12 16:54:24.518203
85878	JHME	2020-10-09	11.83	11.84	11.7177	11.7177	24023	{}	\N	2020-10-12 16:54:24.522141
85879	JHME	2020-10-12	11.6	11.705	11.59	11.705	26973	{}	\N	2020-10-12 16:54:24.526016
85880	KFFB	2020-09-30	6.2	6.2664	6.2	6.2664	3469	{}	\N	2020-10-12 16:54:25.635482
85881	KFFB	2020-10-01	6.06	6.2462	6.06	6.2462	432	{}	\N	2020-10-12 16:54:25.650232
85882	KFFB	2020-10-02	6.0905	6.29	6.06	6.29	1485	{}	\N	2020-10-12 16:54:25.659154
85883	KFFB	2020-10-05	6.2498	6.3	6.18	6.18	993	{}	\N	2020-10-12 16:54:25.66619
85884	KFFB	2020-10-06	6.19	6.19	6.19	6.19	309	{}	\N	2020-10-12 16:54:25.672837
85885	KFFB	2020-10-07	6.225	6.225	6.225	6.225	400	{}	\N	2020-10-12 16:54:25.678233
85886	KFFB	2020-10-08	6.16	6.16	6.15	6.15	893	{}	\N	2020-10-12 16:54:25.683652
85887	KFFB	2020-10-12	6.1997	6.28	6.1997	6.28	3501	{}	\N	2020-10-12 16:54:25.689115
85888	KOF	2020-09-28	41	41.54	40.32	40.42	130278	{}	\N	2020-10-12 16:54:27.789722
85889	KOF	2020-09-29	40.38	40.6	39.37	39.67	190566	{}	\N	2020-10-12 16:54:27.799239
85890	KOF	2020-09-30	40.07	40.89	39.9	40.72	235717	{}	\N	2020-10-12 16:54:27.806947
85891	KOF	2020-10-01	40.83	41.32	40.234	40.57	152346	{}	\N	2020-10-12 16:54:27.81388
85892	KOF	2020-10-02	40.04	41.6396	40.02	41.33	196980	{}	\N	2020-10-12 16:54:27.819201
85893	KOF	2020-10-05	41.67	42.07	41.53	41.96	143359	{}	\N	2020-10-12 16:54:27.824986
85894	KOF	2020-10-06	41.54	42.26	41	41.35	177747	{}	\N	2020-10-12 16:54:27.830568
85895	KOF	2020-10-07	41.74	42.09	41.74	41.98	120613	{}	\N	2020-10-12 16:54:27.836301
85896	KOF	2020-10-08	42.08	42.44	41.91	42.35	289011	{}	\N	2020-10-12 16:54:27.843102
85897	KOF	2020-10-09	42.89	43.56	42.55	42.72	326003	{}	\N	2020-10-12 16:54:27.848845
85898	KOF	2020-10-12	42.72	43.21	42.72	42.865	52740	{}	\N	2020-10-12 16:54:27.854933
85899	LDUR	2020-09-28	102.05	102.19	102.05	102.06	64914	{}	\N	2020-10-12 16:54:28.838232
85900	LDUR	2020-09-29	102.06	102.19	102.03	102.06	43828	{}	\N	2020-10-12 16:54:28.848731
85901	LDUR	2020-09-30	102.17	102.18	102.035	102.1797	107471	{}	\N	2020-10-12 16:54:28.85701
85902	LDUR	2020-10-01	101.88	102.03	101.84	102	73918	{}	\N	2020-10-12 16:54:28.86331
85903	LDUR	2020-10-02	102.16	102.19	101.9	102.03	126967	{}	\N	2020-10-12 16:54:28.868938
85904	LDUR	2020-10-05	102.05	102.05	101.98	102.04	37424	{}	\N	2020-10-12 16:54:28.874097
85905	LDUR	2020-10-06	102.0017	102.22	101.99	102.04	98142	{}	\N	2020-10-12 16:54:28.878983
85906	LDUR	2020-10-07	102.01	102.1716	102	102.08	91531	{}	\N	2020-10-12 16:54:28.883999
85907	LDUR	2020-10-08	102.03	102.08	102.03	102.03	53809	{}	\N	2020-10-12 16:54:28.88907
85908	LDUR	2020-10-09	102.05	102.07	101.94	102.05	54911	{}	\N	2020-10-12 16:54:28.893855
85909	LDUR	2020-10-12	101.99	102.03	101.9501	102.01	54610	{}	\N	2020-10-12 16:54:28.898807
85910	LTC	2020-09-28	34.21	35.04	34.18	34.73	257959	{}	\N	2020-10-12 16:54:29.888918
85911	LTC	2020-09-29	34.69	34.75	33.76	34.63	405430	{}	\N	2020-10-12 16:54:29.898147
85912	LTC	2020-09-30	34.95	35.42	34.41	34.86	635453	{}	\N	2020-10-12 16:54:29.905038
85913	LTC	2020-10-01	34.84	35.78	34.7268	35.77	520510	{}	\N	2020-10-12 16:54:29.911214
85914	LTC	2020-10-02	35.22	36.6	35	36.42	582425	{}	\N	2020-10-12 16:54:29.917034
85915	LTC	2020-10-05	36.5	36.84	35.63	35.83	396381	{}	\N	2020-10-12 16:54:29.922627
85916	LTC	2020-10-06	36.07	36.46	35.51	35.53	592548	{}	\N	2020-10-12 16:54:29.928483
85917	LTC	2020-10-07	35.69	35.86	35.13	35.59	560819	{}	\N	2020-10-12 16:54:29.933819
85918	LTC	2020-10-08	36	36.46	35.6896	36.13	257812	{}	\N	2020-10-12 16:54:29.939594
85919	LTC	2020-10-09	36.35	36.49	35.51	35.68	274102	{}	\N	2020-10-12 16:54:29.944829
85920	LTC	2020-10-12	35.77	36.22	35.53	35.93	119202	{}	\N	2020-10-12 16:54:29.950152
85921	LULU	2020-09-28	317.88	319.2908	310.595	318.73	1382411	{}	\N	2020-10-12 16:54:31.361422
85922	LULU	2020-09-29	318.74	325.2399	315.97	323.67	1299922	{}	\N	2020-10-12 16:54:31.375615
85923	LULU	2020-09-30	328.8006	334.341	326.81	329.37	1885131	{}	\N	2020-10-12 16:54:31.384509
85924	LULU	2020-10-01	334.32	343.2699	332.62	341.05	1622921	{}	\N	2020-10-12 16:54:31.391038
85925	LULU	2020-10-02	333	339.2644	327.73	332.67	1717892	{}	\N	2020-10-12 16:54:31.397486
85926	LULU	2020-10-05	334	337.97	327.026	337.87	1391192	{}	\N	2020-10-12 16:54:31.403306
85927	LULU	2020-10-06	335.66	339.49	327.39	329.29	1493700	{}	\N	2020-10-12 16:54:31.408877
85928	LULU	2020-10-07	333.38	339.2	333.095	338.91	1199955	{}	\N	2020-10-12 16:54:31.41471
85929	LULU	2020-10-08	343.9	343.935	337.17	337.94	1054696	{}	\N	2020-10-12 16:54:31.420116
85930	LULU	2020-10-09	340.81	349.35	338.4	346.46	1680721	{}	\N	2020-10-12 16:54:31.425251
85931	LULU	2020-10-12	351.2	351.55	342.825	348.18	658187	{}	\N	2020-10-12 16:54:31.430887
85932	LW	2020-09-28	65.74	67.81	65.74	66.77	663467	{}	\N	2020-10-12 16:54:32.948969
85933	LW	2020-09-29	66.45	67.07	65.08	65.29	1486602	{}	\N	2020-10-12 16:54:32.961194
85934	LW	2020-09-30	66.03	67.15	65.67	66.27	1685800	{}	\N	2020-10-12 16:54:32.968264
85935	LW	2020-10-01	66.84	67.33	66.02	66.67	1575085	{}	\N	2020-10-12 16:54:32.974748
85936	LW	2020-10-02	65.6	68.22	65.35	67.93	1371761	{}	\N	2020-10-12 16:54:32.980872
85937	LW	2020-10-05	68.35	69.93	68.21	69.55	1944520	{}	\N	2020-10-12 16:54:32.986478
85938	LW	2020-10-06	70	71.705	69.38	70.16	3536424	{}	\N	2020-10-12 16:54:32.992031
85939	LW	2020-10-07	72	74.25	71.22	72.21	5061482	{}	\N	2020-10-12 16:54:32.997683
85940	LW	2020-10-08	72.63	74.68	72.36	74.33	2144297	{}	\N	2020-10-12 16:54:33.002765
85941	LW	2020-10-09	74.56	75	72.66	73.21	2604082	{}	\N	2020-10-12 16:54:33.008415
85942	LW	2020-10-12	73.45	73.57	72.64	72.815	325498	{}	\N	2020-10-12 16:54:33.012844
85943	MCA	2020-09-28	14.41	14.4885	14.39	14.45	35615	{}	\N	2020-10-12 16:54:34.420195
85944	MCA	2020-09-29	14.45	14.49	14.37	14.39	169048	{}	\N	2020-10-12 16:54:34.430687
85945	MCA	2020-09-30	14.36	14.41	14.31	14.36	117722	{}	\N	2020-10-12 16:54:34.438438
85946	MCA	2020-10-01	14.36	14.4084	14.31	14.4	67384	{}	\N	2020-10-12 16:54:34.444172
85947	MCA	2020-10-02	14.31	14.48	14.26	14.37	131714	{}	\N	2020-10-12 16:54:34.449701
85948	MCA	2020-10-05	14.33	14.44	14.22	14.35	157278	{}	\N	2020-10-12 16:54:34.455417
85949	MCA	2020-10-06	14.29	14.36	14.24	14.35	52326	{}	\N	2020-10-12 16:54:34.459931
85950	MCA	2020-10-07	14.4	14.45	14.3907	14.45	57398	{}	\N	2020-10-12 16:54:34.464198
85951	MCA	2020-10-08	14.49	14.51	14.415	14.51	65872	{}	\N	2020-10-12 16:54:34.468989
85952	MCA	2020-10-09	14.46	14.51	14.26	14.3803	229846	{}	\N	2020-10-12 16:54:34.473244
85953	MCA	2020-10-12	14.42	14.42	14.35	14.38	25448	{}	\N	2020-10-12 16:54:34.477716
85954	MFL	2020-09-28	13.16	13.23	13.13	13.13	34679	{}	\N	2020-10-12 16:54:35.708531
85955	MFL	2020-09-29	13.21	13.29	13.195	13.27	100392	{}	\N	2020-10-12 16:54:35.72231
85956	MFL	2020-09-30	13.21	13.41	13.21	13.41	166082	{}	\N	2020-10-12 16:54:35.732197
85957	MFL	2020-10-01	13.48	13.53	13.32	13.46	83124	{}	\N	2020-10-12 16:54:35.738015
85958	MFL	2020-10-02	13.37	13.5176	13.33	13.48	72940	{}	\N	2020-10-12 16:54:35.743175
85959	MFL	2020-10-05	13.5	13.52	13.41	13.45	68522	{}	\N	2020-10-12 16:54:35.747986
85960	MFL	2020-10-06	13.51	13.52	13.32	13.42	129320	{}	\N	2020-10-12 16:54:35.752596
85961	MFL	2020-10-07	13.52	13.53	13.45	13.5	128250	{}	\N	2020-10-12 16:54:35.756983
85962	MFL	2020-10-08	13.52	13.55	13.47	13.52	189658	{}	\N	2020-10-12 16:54:35.761126
85963	MFL	2020-10-09	13.55	13.59	13.39	13.51	39884	{}	\N	2020-10-12 16:54:35.765461
85964	MFL	2020-10-12	13.58	13.58	13.525	13.555	28172	{}	\N	2020-10-12 16:54:35.769926
85965	MITK	2020-09-28	12.66	12.99	12.5101	12.93	635764	{}	\N	2020-10-12 16:54:37.153228
85966	MITK	2020-09-29	12.81	13.23	12.62	13.01	995321	{}	\N	2020-10-12 16:54:37.164249
85967	MITK	2020-09-30	13.03	13.06	12.64	12.74	397444	{}	\N	2020-10-12 16:54:37.171113
85968	MITK	2020-10-01	12.84	13.33	12.6209	13.27	504321	{}	\N	2020-10-12 16:54:37.176801
85969	MITK	2020-10-02	12.89	13.3877	12.81	13.05	432687	{}	\N	2020-10-12 16:54:37.181813
85970	MITK	2020-10-05	13.38	13.38	12.8901	13.25	510820	{}	\N	2020-10-12 16:54:37.18678
85971	MITK	2020-10-06	13.3	13.34	12.96	13.07	304502	{}	\N	2020-10-12 16:54:37.191018
85972	MITK	2020-10-07	13.08	13.73	13.08	13.4	687017	{}	\N	2020-10-12 16:54:37.195177
85973	MITK	2020-10-08	13.59	13.65	13.26	13.52	347792	{}	\N	2020-10-12 16:54:37.19967
85974	MITK	2020-10-09	13.6	13.96	13.56	13.89	412050	{}	\N	2020-10-12 16:54:37.204651
85975	MITK	2020-10-12	14.4424	14.499	13.17	13.31	613392	{}	\N	2020-10-12 16:54:37.209474
85976	MMTM	2020-09-28	146.7	147.3328	146.37	147.301	2016	{}	\N	2020-10-12 16:54:38.191499
85977	MMTM	2020-09-30	147.29	149.17	147.29	149.17	1320	{}	\N	2020-10-12 16:54:38.202128
85978	MMTM	2020-10-01	149.66	149.66	148.7984	149.28	3036	{}	\N	2020-10-12 16:54:38.211059
85979	MMTM	2020-10-02	148.1171	148.1171	146.8444	146.8444	987	{}	\N	2020-10-12 16:54:38.218935
85980	MMTM	2020-10-05	149.385	149.9647	148.8496	149.9647	1140	{}	\N	2020-10-12 16:54:38.22591
85981	MMTM	2020-10-06	149.975	150.2262	147.7743	147.7743	1081	{}	\N	2020-10-12 16:54:38.232412
85982	MMTM	2020-10-07	149.5701	150.3961	149.5701	150.3961	936	{}	\N	2020-10-12 16:54:38.238124
85983	MMTM	2020-10-08	151	151.1088	150.8699	150.95	1916	{}	\N	2020-10-12 16:54:38.2441
85984	MMTM	2020-10-09	152.795	152.8387	152.65	152.8387	1032	{}	\N	2020-10-12 16:54:38.249613
85985	MMTM	2020-10-12	154.81	154.81	154.62	154.62	532	{}	\N	2020-10-12 16:54:38.25546
85986	JBHT	2020-09-28	130.26	131.535	129.95	130.55	552807	{}	\N	2020-10-12 16:54:39.218426
85987	JBHT	2020-09-29	130.52	131.205	128.57	128.63	416965	{}	\N	2020-10-12 16:54:39.23173
85988	JBHT	2020-09-30	129.12	129.47	125.31	126.38	762850	{}	\N	2020-10-12 16:54:39.240141
85989	JBHT	2020-10-01	126.6723	127.35	124.4706	125.21	639148	{}	\N	2020-10-12 16:54:39.246561
85990	JBHT	2020-10-02	124.82	128.35	124.3047	127.49	1040862	{}	\N	2020-10-12 16:54:39.252538
85991	JBHT	2020-10-05	127.99	131.35	127.67	130.55	1466458	{}	\N	2020-10-12 16:54:39.258352
85992	JBHT	2020-10-06	131.05	132.825	129.48	130.55	723152	{}	\N	2020-10-12 16:54:39.263157
85993	JBHT	2020-10-07	130.99	134.27	130.99	132.57	678406	{}	\N	2020-10-12 16:54:39.268162
85994	JBHT	2020-10-08	133.66	135.53	133.496	135.3	506760	{}	\N	2020-10-12 16:54:39.2729
85995	JBHT	2020-10-09	135.71	136.08	134.055	135.22	367125	{}	\N	2020-10-12 16:54:39.277821
85996	JBHT	2020-10-12	136.5	138.27	136.03	137.31	251422	{}	\N	2020-10-12 16:54:39.282598
85997	KMPR	2020-09-28	68.17	68.7796	67.85	68.14	153932	{}	\N	2020-10-12 16:54:41.088377
85998	KMPR	2020-09-29	67.87	68.16	66	67	241258	{}	\N	2020-10-12 16:54:41.097371
85999	KMPR	2020-09-30	67.19	67.74	66.1	66.83	537104	{}	\N	2020-10-12 16:54:41.104515
86000	KMPR	2020-10-01	66.73	67.5332	65.835	66.36	263638	{}	\N	2020-10-12 16:54:41.110865
86001	KMPR	2020-10-02	65.41	67.63	65.41	66.99	231481	{}	\N	2020-10-12 16:54:41.116041
86002	KMPR	2020-10-05	67.9	68.74	67.52	68.1	401584	{}	\N	2020-10-12 16:54:41.12272
86003	KMPR	2020-10-06	68.46	69.08	67.02	67.5	372193	{}	\N	2020-10-12 16:54:41.128159
86004	KMPR	2020-10-07	67.97	68.67	66.97	67.02	296280	{}	\N	2020-10-12 16:54:41.13412
86005	KMPR	2020-10-08	67.3	67.9	66.72	66.92	256644	{}	\N	2020-10-12 16:54:41.141644
86006	KMPR	2020-10-09	66.9	67.72	66.44	66.56	603909	{}	\N	2020-10-12 16:54:41.147139
86007	KMPR	2020-10-12	66.49	68.46	66.49	68.2	474452	{}	\N	2020-10-12 16:54:41.152856
86008	KWEB	2020-09-28	67.34	67.44	66.41	67.25	823084	{}	\N	2020-10-12 16:54:42.176475
86009	KWEB	2020-09-29	67.03	67.14	66.58	66.77	2749135	{}	\N	2020-10-12 16:54:42.18949
86010	KWEB	2020-09-30	67.42	68.59	67.26	68.18	2502659	{}	\N	2020-10-12 16:54:42.197852
86011	KWEB	2020-10-01	68.68	69.22	68.2	68.97	1754090	{}	\N	2020-10-12 16:54:42.204372
86012	KWEB	2020-10-02	67.46	68.66	67.3896	67.63	2458410	{}	\N	2020-10-12 16:54:42.210179
86013	KWEB	2020-10-05	68.04	68.29	67.66	68.27	1395537	{}	\N	2020-10-12 16:54:42.215324
86014	KWEB	2020-10-06	68.87	70.32	68.79	69.56	1889190	{}	\N	2020-10-12 16:54:42.220342
86015	KWEB	2020-10-07	70.44	70.71	70.03	70.49	1384168	{}	\N	2020-10-12 16:54:42.225527
86016	KWEB	2020-10-08	70.73	70.89	70.22	70.61	1527658	{}	\N	2020-10-12 16:54:42.23055
86017	KWEB	2020-10-09	70.83	72.08	70.81	72.07	1107560	{}	\N	2020-10-12 16:54:42.235682
86018	KWEB	2020-10-12	73.03	73.59	72.81	73.37	1528868	{}	\N	2020-10-12 16:54:42.240604
86019	LBTYA	2020-09-28	21.1	21.43	21.06	21.33	776299	{}	\N	2020-10-12 16:54:44.212054
86020	LBTYA	2020-09-29	21.48	21.63	21.06	21.39	1262233	{}	\N	2020-10-12 16:54:44.226458
86021	LBTYA	2020-09-30	21.53	21.71	20.97	21.01	1124672	{}	\N	2020-10-12 16:54:44.234733
86022	LBTYA	2020-10-01	20.96	21.135	20.41	20.54	1558961	{}	\N	2020-10-12 16:54:44.242122
86023	LBTYA	2020-10-02	20.32	20.43	19.78	20.12	1868714	{}	\N	2020-10-12 16:54:44.249603
86024	LBTYA	2020-10-05	20.24	20.72	20.1501	20.69	842193	{}	\N	2020-10-12 16:54:44.25486
86025	LBTYA	2020-10-06	20.69	21.23	20.69	20.8	815491	{}	\N	2020-10-12 16:54:44.259813
86026	LBTYA	2020-10-07	20.84	21.17	20.77	20.99	843277	{}	\N	2020-10-12 16:54:44.265098
86027	LBTYA	2020-10-08	21.01	21.23	20.74	20.8	1291784	{}	\N	2020-10-12 16:54:44.26996
86028	LBTYA	2020-10-09	20.95	22.09	20.95	22.07	1360041	{}	\N	2020-10-12 16:54:44.274982
86029	LBTYA	2020-10-12	22.21	22.65	22.05	22.49	507179	{}	\N	2020-10-12 16:54:44.280024
86030	LFUS	2020-09-28	172.08	177.1	171.24	176.81	58278	{}	\N	2020-10-12 16:54:46.359275
86031	LFUS	2020-09-29	176.05	179.45	176.04	177.73	73070	{}	\N	2020-10-12 16:54:46.371171
86032	LFUS	2020-09-30	178.29	180.2	176.37	177.34	90994	{}	\N	2020-10-12 16:54:46.378919
86033	LFUS	2020-10-01	178.23	185.24	177.36	183.83	151668	{}	\N	2020-10-12 16:54:46.384857
86034	LFUS	2020-10-02	180.06	182.54	179.03	181.14	45261	{}	\N	2020-10-12 16:54:46.390382
86035	LFUS	2020-10-05	183.92	186.88	183.92	186.64	42850	{}	\N	2020-10-12 16:54:46.395795
86036	LFUS	2020-10-06	188.39	188.39	183.51	185.36	104859	{}	\N	2020-10-12 16:54:46.400821
86037	LFUS	2020-10-07	188.23	190	186.04	189.38	84461	{}	\N	2020-10-12 16:54:46.405948
86038	LFUS	2020-10-08	190	190	187.6	188.05	56857	{}	\N	2020-10-12 16:54:46.410578
86039	LFUS	2020-10-09	189.97	189.97	186.55	187.5	94421	{}	\N	2020-10-12 16:54:46.415051
86040	LFUS	2020-10-12	188.8	189.045	187.39	188.125	23742	{}	\N	2020-10-12 16:54:46.4197
86041	LKOR	2020-09-28	62.3945	62.4943	62.3945	62.4943	204	{}	\N	2020-10-12 16:54:47.715256
86042	LKOR	2020-09-29	62.5	62.5	62.32	62.32	1884	{}	\N	2020-10-12 16:54:47.726856
86043	LKOR	2020-09-30	62.06	62.1734	62.06	62.1734	2175	{}	\N	2020-10-12 16:54:47.73642
86044	LKOR	2020-10-01	62.01	62.3399	62.01	62.3399	1142	{}	\N	2020-10-12 16:54:47.742145
86045	LKOR	2020-10-02	62.2761	62.2761	62.2761	62.2761	100	{}	\N	2020-10-12 16:54:47.748504
86046	LKOR	2020-10-05	62	62.2	62	62.01	2134	{}	\N	2020-10-12 16:54:47.752964
86047	LKOR	2020-10-06	62.05	62.05	61.9466	61.9466	1120	{}	\N	2020-10-12 16:54:47.756978
86048	LKOR	2020-10-07	61.945	62.1535	61.7745	61.7745	610	{}	\N	2020-10-12 16:54:47.761043
86049	LKOR	2020-10-08	62.1429	62.26	62.0371	62.0371	4863	{}	\N	2020-10-12 16:54:47.765181
86050	LKOR	2020-10-09	62.25	62.2637	62.07	62.2637	4742	{}	\N	2020-10-12 16:54:47.769239
86051	LKOR	2020-10-12	62.41	62.41	62.41	62.41	114	{}	\N	2020-10-12 16:54:47.773813
86052	LNC	2020-09-28	31.52	32.68	31.41	32.05	2558013	{}	\N	2020-10-12 16:54:49.145259
86053	LNC	2020-09-29	31.83	32.21	30.52	30.94	4270298	{}	\N	2020-10-12 16:54:49.156931
86054	LNC	2020-09-30	31.11	31.94	30.86	31.33	5834263	{}	\N	2020-10-12 16:54:49.165476
86055	LNC	2020-10-01	31.9	32.71	31.15	31.89	4243562	{}	\N	2020-10-12 16:54:49.171601
86056	LNC	2020-10-02	31.16	32.855	31.02	32.64	2696024	{}	\N	2020-10-12 16:54:49.17789
86057	LNC	2020-10-05	33.43	34.75	33.3	34.29	6041688	{}	\N	2020-10-12 16:54:49.183587
86058	LNC	2020-10-06	35.01	35.72	33.7	33.73	5441925	{}	\N	2020-10-12 16:54:49.189058
86059	LNC	2020-10-07	34.42	35.62	34.28	35.12	4678356	{}	\N	2020-10-12 16:54:49.194328
86060	LNC	2020-10-08	35.3	35.65	34.75	35.63	2670883	{}	\N	2020-10-12 16:54:49.199517
86061	LNC	2020-10-09	36.18	36.23	34.81	34.85	2680097	{}	\N	2020-10-12 16:54:49.204596
86062	LNC	2020-10-12	35	35.46	34.75	35.16	898859	{}	\N	2020-10-12 16:54:49.209912
86063	MBCN	2020-09-28	19.28	19.4	18.562	19.27	5147	{}	\N	2020-10-12 16:54:50.96387
86064	MBCN	2020-09-29	18.85	19.87	18.7	19.87	5838	{}	\N	2020-10-12 16:54:50.976584
86065	MBCN	2020-09-30	20	20.45	19.02	19.02	4271	{}	\N	2020-10-12 16:54:50.98427
86066	MBCN	2020-10-01	19.5	19.646	18.84	19.25	7481	{}	\N	2020-10-12 16:54:50.989888
86067	MBCN	2020-10-02	18.95	19.49	18.95	19.12	3211	{}	\N	2020-10-12 16:54:50.994467
86068	MBCN	2020-10-05	18.95	19.95	18.95	19.59	4276	{}	\N	2020-10-12 16:54:50.999808
86069	MBCN	2020-10-06	19.73	19.78	19.23	19.23	5487	{}	\N	2020-10-12 16:54:51.004131
86070	MBCN	2020-10-07	19.62	20.49	19.615	20.49	7798	{}	\N	2020-10-12 16:54:51.008084
86071	MBCN	2020-10-08	19.7588	20.75	19.7588	20.6	4544	{}	\N	2020-10-12 16:54:51.012186
86072	MBCN	2020-10-09	20.66	21	20.66	20.75	3596	{}	\N	2020-10-12 16:54:51.017085
86073	MBCN	2020-10-12	20.6	20.6	20.6	20.6	924	{}	\N	2020-10-12 16:54:51.021653
86074	MHD	2020-09-28	15.24	15.31	15.169	15.24	17170	{}	\N	2020-10-12 16:54:52.349915
86075	MHD	2020-09-29	15.27	15.27	15.21	15.25	52994	{}	\N	2020-10-12 16:54:52.362102
86076	MHD	2020-09-30	15.25	15.27	15.21	15.21	35414	{}	\N	2020-10-12 16:54:52.370227
86077	MHD	2020-10-01	15.27	15.32	15	15.31	77448	{}	\N	2020-10-12 16:54:52.376842
86078	MHD	2020-10-02	15.34	15.34	15.23	15.27	51246	{}	\N	2020-10-12 16:54:52.383458
86079	MHD	2020-10-05	15.32	15.32	15.18	15.24	48608	{}	\N	2020-10-12 16:54:52.389029
86080	MHD	2020-10-06	15.29	15.29	15.16	15.26	49874	{}	\N	2020-10-12 16:54:52.393802
86081	MHD	2020-10-07	15.32	15.32	15.22	15.31	41836	{}	\N	2020-10-12 16:54:52.398816
86082	MHD	2020-10-08	15.34	15.34	15.2	15.32	41282	{}	\N	2020-10-12 16:54:52.404066
86083	MHD	2020-10-09	15.27	15.34	15.21	15.33	74414	{}	\N	2020-10-12 16:54:52.408793
86084	MHD	2020-10-12	15.4	15.4	15.35	15.37	5394	{}	\N	2020-10-12 16:54:52.41321
86085	MWA	2020-09-28	10.44	10.6301	10.4	10.47	1315503	{}	\N	2020-10-12 16:54:53.358444
86086	MWA	2020-09-29	10.52	10.52	10.3	10.4	3363899	{}	\N	2020-10-12 16:54:53.37243
86087	MWA	2020-09-30	10.42	10.55	10.28	10.39	2363399	{}	\N	2020-10-12 16:54:53.381175
86088	MWA	2020-10-01	10.39	10.47	10.24	10.34	2151963	{}	\N	2020-10-12 16:54:53.389285
86089	MWA	2020-10-02	10.14	10.68	10.13	10.55	2578661	{}	\N	2020-10-12 16:54:53.395784
86090	MWA	2020-10-05	10.61	10.725	10.49	10.55	1205054	{}	\N	2020-10-12 16:54:53.401355
86091	MWA	2020-10-06	10.55	10.7401	10.45	10.46	2012852	{}	\N	2020-10-12 16:54:53.406616
86092	MWA	2020-10-07	10.57	10.72	10.54	10.67	1819736	{}	\N	2020-10-12 16:54:53.411826
86093	MWA	2020-10-08	10.76	10.95	10.73	10.9	3039085	{}	\N	2020-10-12 16:54:53.417016
86094	MWA	2020-10-09	11	11.03	10.8	10.81	2165986	{}	\N	2020-10-12 16:54:53.42208
86095	MWA	2020-10-12	10.81	11.12	10.81	11.03	612385	{}	\N	2020-10-12 16:54:53.42764
86096	MYD	2020-09-28	13.43	13.53	13.43	13.49	79305	{}	\N	2020-10-12 16:54:54.386737
86097	MYD	2020-09-29	13.52	13.55	13.44	13.44	164592	{}	\N	2020-10-12 16:54:54.399803
86098	MYD	2020-09-30	13.47	13.49	13.42	13.42	96095	{}	\N	2020-10-12 16:54:54.408061
86099	MYD	2020-10-01	13.45	13.51	13.42	13.5	156848	{}	\N	2020-10-12 16:54:54.413088
86100	MYD	2020-10-02	13.42	13.5	13.42	13.5	123516	{}	\N	2020-10-12 16:54:54.417875
86101	MYD	2020-10-05	13.5	13.5	13.34	13.38	278622	{}	\N	2020-10-12 16:54:54.422707
86102	MYD	2020-10-06	13.43	13.44	13.37	13.42	141909	{}	\N	2020-10-12 16:54:54.427
86103	MYD	2020-10-07	13.51	13.53	13.42	13.47	174528	{}	\N	2020-10-12 16:54:54.431309
86104	MYD	2020-10-08	13.52	13.54	13.47	13.49	70046	{}	\N	2020-10-12 16:54:54.435814
86105	MYD	2020-10-09	13.54	13.55	13.495	13.55	107404	{}	\N	2020-10-12 16:54:54.439864
86106	MYD	2020-10-12	13.57	13.57	13.5445	13.565	39370	{}	\N	2020-10-12 16:54:54.443863
86107	NCB	2020-09-28	15.25	15.62	15.25	15.6	2886	{}	\N	2020-10-12 16:54:56.718341
86108	NCB	2020-09-29	15.62	15.62	15.6	15.6	10438	{}	\N	2020-10-12 16:54:56.73297
86109	NCB	2020-09-30	15.7186	15.7186	15.61	15.64	4142	{}	\N	2020-10-12 16:54:56.740905
86110	NCB	2020-10-01	15.75	15.75	15.56	15.6499	11606	{}	\N	2020-10-12 16:54:56.74721
86111	NCB	2020-10-02	15.58	15.69	15.58	15.67	10546	{}	\N	2020-10-12 16:54:56.752887
86112	NCB	2020-10-05	15.64	15.6896	15.64	15.6799	19626	{}	\N	2020-10-12 16:54:56.758725
86113	NCB	2020-10-06	15.42	15.853	15.42	15.64	19568	{}	\N	2020-10-12 16:54:56.764263
86114	NCB	2020-10-07	15.6219	15.65	15.62	15.64	41446	{}	\N	2020-10-12 16:54:56.769541
86115	NCB	2020-10-08	15.61	15.65	15.57	15.6064	19926	{}	\N	2020-10-12 16:54:56.775418
86116	NCB	2020-10-09	15.6	15.61	15.5999	15.61	2764	{}	\N	2020-10-12 16:54:56.781096
86117	NCB	2020-10-12	15.6175	15.63	15.61	15.6222	26210	{}	\N	2020-10-12 16:54:56.786644
86118	NGL	2020-09-28	3.59	3.73	3.5	3.73	881343	{}	\N	2020-10-12 16:54:58.109649
86119	NGL	2020-09-29	3.75	3.95	3.68	3.9	854270	{}	\N	2020-10-12 16:54:58.121165
86120	NGL	2020-09-30	3.95	4.06	3.89	3.96	1826040	{}	\N	2020-10-12 16:54:58.128626
86121	NGL	2020-10-01	3.99	4.12	3.9	4.06	707383	{}	\N	2020-10-12 16:54:58.134212
86122	NGL	2020-10-02	3.85	4.08	3.85	4.05	554409	{}	\N	2020-10-12 16:54:58.139693
86123	NGL	2020-10-05	4.03	4.16	4.01	4.16	596461	{}	\N	2020-10-12 16:54:58.14523
86124	NGL	2020-10-06	4.16	4.25	3.81	3.92	2456923	{}	\N	2020-10-12 16:54:58.150854
86125	NGL	2020-10-07	3.97	4.0499	3.73	3.79	1911556	{}	\N	2020-10-12 16:54:58.1559
86126	NGL	2020-10-08	3.92	4.23	3.875	4.19	1944398	{}	\N	2020-10-12 16:54:58.161392
86127	NGL	2020-10-09	4.24	4.24	4.025	4.04	617723	{}	\N	2020-10-12 16:54:58.166099
86128	NGL	2020-10-12	4.01	4.04	3.9	3.965	296454	{}	\N	2020-10-12 16:54:58.170867
86129	NMS	2020-09-28	13.99	14.1	13.99	14.06	12112	{}	\N	2020-10-12 16:54:59.47243
86130	NMS	2020-09-29	14.2	14.2	14.0646	14.0646	16764	{}	\N	2020-10-12 16:54:59.485249
86131	NMS	2020-09-30	14.15	14.1698	14.099	14.099	3334	{}	\N	2020-10-12 16:54:59.492924
86132	NMS	2020-10-01	14.25	14.34	14.1962	14.34	3386	{}	\N	2020-10-12 16:54:59.50242
86133	NMS	2020-10-02	14.63	14.63	14.02	14.13	27698	{}	\N	2020-10-12 16:54:59.507845
86134	NMS	2020-10-05	14.14	14.27	14.14	14.27	1948	{}	\N	2020-10-12 16:54:59.512267
86135	NMS	2020-10-06	14.39	14.4	14.35	14.4	3210	{}	\N	2020-10-12 16:54:59.51692
86136	NMS	2020-10-07	14.3343	14.41	14.23	14.25	10004	{}	\N	2020-10-12 16:54:59.52158
86137	NMS	2020-10-08	14.367	14.367	14.3609	14.3609	2086	{}	\N	2020-10-12 16:54:59.526039
86138	NMS	2020-10-09	14.39	14.64	14.36	14.51	6430	{}	\N	2020-10-12 16:54:59.530724
86139	NMS	2020-10-12	14.455	14.455	14.455	14.455	204	{}	\N	2020-10-12 16:54:59.535462
86140	JETS	2020-09-28	17.16	17.58	17.04	17.3	3872817	{}	\N	2020-10-12 16:55:00.596914
86141	JETS	2020-09-29	17.22	17.24	16.7711	16.86	2912464	{}	\N	2020-10-12 16:55:00.610236
86142	JETS	2020-09-30	16.99	17.43	16.85	16.9	3496780	{}	\N	2020-10-12 16:55:00.619403
86143	JETS	2020-10-01	17.07	17.2	16.905	17.12	2713488	{}	\N	2020-10-12 16:55:00.626243
86144	JETS	2020-10-02	16.56	17.55	16.521	17.33	7509126	{}	\N	2020-10-12 16:55:00.632214
86145	JETS	2020-10-05	17.51	17.58	17.27	17.46	2946840	{}	\N	2020-10-12 16:55:00.637014
86146	JETS	2020-10-06	17.68	17.865	17.135	17.17	6020928	{}	\N	2020-10-12 16:55:00.641648
86147	JETS	2020-10-07	17.58	17.73	17.475	17.65	3653792	{}	\N	2020-10-12 16:55:00.646614
86148	JETS	2020-10-08	17.94	18.0701	17.56	17.96	6092497	{}	\N	2020-10-12 16:55:00.65116
86149	JETS	2020-10-09	18	18.18	17.7513	17.99	4829507	{}	\N	2020-10-12 16:55:00.655794
86150	JETS	2020-10-12	18.01	18.04	17.81	17.9621	3108166	{}	\N	2020-10-12 16:55:00.660294
86151	KBE	2020-09-28	29.11	29.86	29.11	29.6	2397294	{}	\N	2020-10-12 16:55:02.804952
86152	KBE	2020-09-29	29.53	29.53	28.745	29.19	1422713	{}	\N	2020-10-12 16:55:02.814935
86153	KBE	2020-09-30	29.24	29.83	29.2	29.45	1453817	{}	\N	2020-10-12 16:55:02.82383
86154	KBE	2020-10-01	29.61	29.84	29.24	29.74	1087848	{}	\N	2020-10-12 16:55:02.831144
86155	KBE	2020-10-02	29.19	30.6899	29.1	30.55	1706258	{}	\N	2020-10-12 16:55:02.836957
86156	KBE	2020-10-05	30.87	31.72	30.87	31.65	2447716	{}	\N	2020-10-12 16:55:02.841777
86157	KBE	2020-10-06	31.99	32.87	31.515	31.61	7802177	{}	\N	2020-10-12 16:55:02.846931
86158	KBE	2020-10-07	32.07	32.84	32.07	32.46	5037843	{}	\N	2020-10-12 16:55:02.851999
86159	KBE	2020-10-08	32.68	33.09	32.54	32.98	3472451	{}	\N	2020-10-12 16:55:02.856562
86160	KBE	2020-10-09	33.27	33.394	32.565	32.75	1278413	{}	\N	2020-10-12 16:55:02.861585
86161	KBE	2020-10-12	32.74	33.08	32.63	32.92	682918	{}	\N	2020-10-12 16:55:02.866809
86162	KFS	2020-09-28	2.78	2.85	2.78	2.85	8664	{}	\N	2020-10-12 16:55:04.533046
86163	KFS	2020-09-29	2.86	2.98	2.79	2.98	40078	{}	\N	2020-10-12 16:55:04.544414
86164	KFS	2020-09-30	2.93	3	2.92	2.95	56338	{}	\N	2020-10-12 16:55:04.552927
86165	KFS	2020-10-01	2.92	2.95	2.87	2.9	54659	{}	\N	2020-10-12 16:55:04.5598
86166	KFS	2020-10-02	2.83	3	2.83	2.92	75410	{}	\N	2020-10-12 16:55:04.56531
86167	KFS	2020-10-05	2.91	3.05	2.91	3	23852	{}	\N	2020-10-12 16:55:04.569942
86168	KFS	2020-10-06	2.955	3.1	2.955	3.07	43888	{}	\N	2020-10-12 16:55:04.574438
86169	KFS	2020-10-07	2.95	3.1	2.95	3.07	24418	{}	\N	2020-10-12 16:55:04.579079
86170	KFS	2020-10-08	3.06	3.06	2.92	3.03	134573	{}	\N	2020-10-12 16:55:04.583846
86171	KFS	2020-10-09	3.02	3.06	3.02	3.02	5585	{}	\N	2020-10-12 16:55:04.590159
86172	KFS	2020-10-12	3	3.08	3	3.04	3622	{}	\N	2020-10-12 16:55:04.594967
86173	LDP	2020-09-28	23.33	23.36	23.03	23.13	116272	{}	\N	2020-10-12 16:55:06.294271
86174	LDP	2020-09-29	23.13	23.16	23.03	23.14	136804	{}	\N	2020-10-12 16:55:06.311413
86175	LDP	2020-09-30	23.25	23.39	23.17	23.37	344050	{}	\N	2020-10-12 16:55:06.317657
86176	LDP	2020-10-01	23.43	23.49	23.2208	23.45	254444	{}	\N	2020-10-12 16:55:06.322964
86177	LDP	2020-10-02	23.27	23.4281	23.22	23.3	147102	{}	\N	2020-10-12 16:55:06.327849
86178	LDP	2020-10-05	23.4	23.4515	23.33	23.41	117508	{}	\N	2020-10-12 16:55:06.332873
86179	LDP	2020-10-06	23.49	23.6	23.37	23.5	198075	{}	\N	2020-10-12 16:55:06.337595
86180	LDP	2020-10-07	23.64	23.72	23.559	23.72	81910	{}	\N	2020-10-12 16:55:06.342128
86181	LDP	2020-10-08	23.84	23.9	23.7	23.9	68180	{}	\N	2020-10-12 16:55:06.346982
86182	LDP	2020-10-09	23.94	23.94	23.7381	23.85	88273	{}	\N	2020-10-12 16:55:06.351371
86183	LDP	2020-10-12	23.86	23.9474	23.85	23.87	27533	{}	\N	2020-10-12 16:55:06.355306
86184	LEGR	2020-09-28	31.165	31.289	31.1537	31.2799	15956	{}	\N	2020-10-12 16:55:07.307178
86185	LEGR	2020-09-29	31.24	31.24	31.12	31.19	4322	{}	\N	2020-10-12 16:55:07.317937
86186	LEGR	2020-09-30	31.415	31.42	31.19	31.3498	6886	{}	\N	2020-10-12 16:55:07.325343
86187	LEGR	2020-10-01	31.55	31.6199	31.39	31.4748	6419	{}	\N	2020-10-12 16:55:07.331571
86188	LEGR	2020-10-02	31.04	31.41	31.04	31.25	4887	{}	\N	2020-10-12 16:55:07.337182
86189	LEGR	2020-10-05	31.73	31.9102	31.6867	31.9102	3686	{}	\N	2020-10-12 16:55:07.342142
86190	LEGR	2020-10-06	32.05	32.1233	31.6432	31.6432	5181	{}	\N	2020-10-12 16:55:07.346902
86191	LEGR	2020-10-07	32.03	32.06	32.015	32.06	3268	{}	\N	2020-10-12 16:55:07.351713
86192	LEGR	2020-10-08	32.3	32.4353	32.3	32.4353	3371	{}	\N	2020-10-12 16:55:07.356341
86193	LEGR	2020-10-09	32.645	32.729	32.56	32.729	3430	{}	\N	2020-10-12 16:55:07.361408
86194	LEGR	2020-10-12	32.89	33.02	32.8823	33.0167	2116	{}	\N	2020-10-12 16:55:07.366865
86195	LMNX	2020-09-28	25.49	26.43	25.37	25.91	621091	{}	\N	2020-10-12 16:55:09.142341
86196	LMNX	2020-09-29	25.81	26.29	25.68	26.21	419704	{}	\N	2020-10-12 16:55:09.153871
86197	LMNX	2020-09-30	26.25	26.51	25.91	26.25	394792	{}	\N	2020-10-12 16:55:09.161762
86198	LMNX	2020-10-01	26.4	26.48	25.73	25.96	409570	{}	\N	2020-10-12 16:55:09.167987
86199	LMNX	2020-10-02	25.53	26.257	25.37	25.78	267039	{}	\N	2020-10-12 16:55:09.173214
86200	LMNX	2020-10-05	26.01	26.81	25.81	26.6	330144	{}	\N	2020-10-12 16:55:09.178255
86201	LMNX	2020-10-06	26.81	27.32	26.67	26.84	339924	{}	\N	2020-10-12 16:55:09.183166
86202	LMNX	2020-10-07	26.99	27.2	24.4301	25.26	1587582	{}	\N	2020-10-12 16:55:09.187946
86203	LMNX	2020-10-08	25.3	25.44	24.305	24.56	573438	{}	\N	2020-10-12 16:55:09.1924
86204	LMNX	2020-10-09	24.74	24.93	24.23	24.3	532518	{}	\N	2020-10-12 16:55:09.196909
86205	LMNX	2020-10-12	24.62	24.66	23.89	24.1	565907	{}	\N	2020-10-12 16:55:09.201218
86206	MEAR	2020-09-28	50.16	50.229	50.16	50.22	31359	{}	\N	2020-10-12 16:55:11.269293
86207	MEAR	2020-09-29	50.19	50.23	50.15	50.23	47636	{}	\N	2020-10-12 16:55:11.279753
86208	MEAR	2020-09-30	50.16	50.22	50.16	50.19	36983	{}	\N	2020-10-12 16:55:11.286143
86209	MEAR	2020-10-01	50.176	50.21	50.1502	50.2	9090	{}	\N	2020-10-12 16:55:11.29214
86210	MEAR	2020-10-02	50.14	50.19	50.14	50.14	430625	{}	\N	2020-10-12 16:55:11.297461
86211	MEAR	2020-10-05	50.08	50.19	50.08	50.17	35161	{}	\N	2020-10-12 16:55:11.302548
86212	MEAR	2020-10-06	50.16	50.1811	50.13	50.1506	21889	{}	\N	2020-10-12 16:55:11.307412
86213	MEAR	2020-10-07	50.14	50.26	50.14	50.26	540203	{}	\N	2020-10-12 16:55:11.312558
86214	MEAR	2020-10-08	50.19	50.24	50.189	50.23	44034	{}	\N	2020-10-12 16:55:11.317636
86215	MEAR	2020-10-09	50.21	50.22	50.18	50.2	34120	{}	\N	2020-10-12 16:55:11.323089
86216	MEAR	2020-10-12	50.22	50.22	50.17	50.172	6563	{}	\N	2020-10-12 16:55:11.329196
86217	MFC	2020-09-28	14.02	14.25	13.97	14.16	1759105	{}	\N	2020-10-12 16:55:13.028201
86218	MFC	2020-09-29	14.05	14.12	13.75	13.9	3188891	{}	\N	2020-10-12 16:55:13.039365
86219	MFC	2020-09-30	13.97	14.16	13.84	13.91	4309738	{}	\N	2020-10-12 16:55:13.047481
86220	MFC	2020-10-01	13.91	14.09	13.89	13.92	2805145	{}	\N	2020-10-12 16:55:13.054096
86221	MFC	2020-10-02	13.74	14.2354	13.7	14.19	3085830	{}	\N	2020-10-12 16:55:13.060338
86222	MFC	2020-10-05	14.31	14.4999	14.27	14.43	2457890	{}	\N	2020-10-12 16:55:13.066716
86223	MFC	2020-10-06	14.6	14.64	14.315	14.4	3711135	{}	\N	2020-10-12 16:55:13.071935
86224	MFC	2020-10-07	14.51	14.715	14.495	14.66	2312246	{}	\N	2020-10-12 16:55:13.076824
86225	MFC	2020-10-08	14.74	14.82	14.59	14.68	2903462	{}	\N	2020-10-12 16:55:13.081901
86226	MFC	2020-10-09	14.75	14.835	14.63	14.69	3143103	{}	\N	2020-10-12 16:55:13.087267
86227	MFC	2020-10-12	14.74	14.861	14.7	14.795	476286	{}	\N	2020-10-12 16:55:13.092592
86228	MGP	2020-09-28	28.45	28.88	28.355	28.74	545330	{}	\N	2020-10-12 16:55:14.854741
86229	MGP	2020-09-29	28.11	28.53	27.88	28.25	2162978	{}	\N	2020-10-12 16:55:14.86388
86230	MGP	2020-09-30	28.48	28.98	27.77	27.98	2508963	{}	\N	2020-10-12 16:55:14.871168
86231	MGP	2020-10-01	28.03	28.28	27.795	28.2	2451532	{}	\N	2020-10-12 16:55:14.877906
86232	MGP	2020-10-02	27.9	28.46	27.78	28.26	2192459	{}	\N	2020-10-12 16:55:14.883779
86233	MGP	2020-10-05	28.46	28.58	27.94	28.17	1428866	{}	\N	2020-10-12 16:55:14.889227
86234	MGP	2020-10-06	28.35	28.49	28.05	28.25	1607417	{}	\N	2020-10-12 16:55:14.894379
86235	MGP	2020-10-07	28.4	28.53	27.87	28.01	1754779	{}	\N	2020-10-12 16:55:14.899436
86236	MGP	2020-10-08	28.06	28.48	27.965	28.21	1101794	{}	\N	2020-10-12 16:55:14.904548
86237	MGP	2020-10-09	28.5	28.5	28.06	28.14	1781900	{}	\N	2020-10-12 16:55:14.909685
86238	MGP	2020-10-12	28.22	28.22	27.94	28.065	498980	{}	\N	2020-10-12 16:55:14.914948
86239	MLAB	2020-09-28	247.53	252.56	247.53	251.06	23857	{}	\N	2020-10-12 16:55:16.562178
86240	MLAB	2020-09-29	242.1	256.32	242.1	255.87	26536	{}	\N	2020-10-12 16:55:16.569454
86241	MLAB	2020-09-30	256.92	258.31	250.23	254.76	36211	{}	\N	2020-10-12 16:55:16.574984
86242	MLAB	2020-10-01	255.465	258.63	253.88	258.4	29379	{}	\N	2020-10-12 16:55:16.580463
86243	MLAB	2020-10-02	254.68	260.3	254.68	259	24368	{}	\N	2020-10-12 16:55:16.585326
86244	MLAB	2020-10-05	259	263.245	259	260.55	19762	{}	\N	2020-10-12 16:55:16.590595
86245	MLAB	2020-10-06	264.95	269.44	262.45	266.21	30080	{}	\N	2020-10-12 16:55:16.596013
86246	MLAB	2020-10-07	267.95	275	267.09	273.41	32733	{}	\N	2020-10-12 16:55:16.601193
86247	MLAB	2020-10-08	275	277.88	275	277.88	28677	{}	\N	2020-10-12 16:55:16.606471
86248	MLAB	2020-10-09	279.95	284.08	279.1501	282.97	24759	{}	\N	2020-10-12 16:55:16.612434
86249	MLAB	2020-10-12	285	286.35	279.78	285.73	6467	{}	\N	2020-10-12 16:55:16.617282
86250	JHML	2020-09-28	41.4099	41.74	41.4099	41.63	28596	{}	\N	2020-10-12 16:55:17.993864
86251	JHML	2020-09-29	41.66	41.6922	41.35	41.42	60959	{}	\N	2020-10-12 16:55:18.004148
86252	JHML	2020-09-30	41.56	42.1	41.46	41.76	50377	{}	\N	2020-10-12 16:55:18.0107
86253	JHML	2020-10-01	41.99	42.05	41.755	41.97	108181	{}	\N	2020-10-12 16:55:18.016647
86254	JHML	2020-10-02	41.25	41.98	41.25	41.76	53080	{}	\N	2020-10-12 16:55:18.022142
86255	JHML	2020-10-05	42.11	42.55	42.11	42.55	21006	{}	\N	2020-10-12 16:55:18.02737
86256	JHML	2020-10-06	42.65	42.92	41.99	42.06	58676	{}	\N	2020-10-12 16:55:18.032292
86257	JHML	2020-10-07	42.48	42.86	42.46	42.78	69565	{}	\N	2020-10-12 16:55:18.037166
86258	JHML	2020-10-08	43.005	43.1912	42.93	43.19	1211886	{}	\N	2020-10-12 16:55:18.042061
86259	JHML	2020-10-09	43.46	43.64	43.331	43.5	219562	{}	\N	2020-10-12 16:55:18.047139
86260	JHML	2020-10-12	43.82	44.14	43.75	44.1015	10817	{}	\N	2020-10-12 16:55:18.052068
86261	JPXN	2020-09-28	64.18	64.4701	64.18	64.4701	1078	{}	\N	2020-10-12 16:55:20.078011
86262	JPXN	2020-09-29	64.61	64.92	64.5079	64.5079	14922	{}	\N	2020-10-12 16:55:20.087105
86263	JPXN	2020-09-30	63.8	64.13	63.8	63.98	2167	{}	\N	2020-10-12 16:55:20.092973
86264	JPXN	2020-10-01	64.01	64.01	63.81	63.97	973	{}	\N	2020-10-12 16:55:20.097988
86265	JPXN	2020-10-02	63.474	63.66	63.42	63.62	1414	{}	\N	2020-10-12 16:55:20.102854
86266	JPXN	2020-10-05	64.01	64.208	64.01	64.208	2221	{}	\N	2020-10-12 16:55:20.106796
86267	JPXN	2020-10-06	64.33	64.405	64.07	64.07	1050	{}	\N	2020-10-12 16:55:20.111074
86268	JPXN	2020-10-07	64.34	64.3842	64.296	64.3842	1192	{}	\N	2020-10-12 16:55:20.115771
86269	JPXN	2020-10-08	64.58	64.58	64.4141	64.4141	458	{}	\N	2020-10-12 16:55:20.120174
86270	JPXN	2020-10-09	64.31	64.4122	64.3	64.38	1898	{}	\N	2020-10-12 16:55:20.124351
86271	JWN	2020-09-28	12.29	12.6	12.11	12.27	7412289	{}	\N	2020-10-12 16:55:21.293821
86272	JWN	2020-09-29	12.24	12.24	11.72	11.79	18994116	{}	\N	2020-10-12 16:55:21.303374
86273	JWN	2020-09-30	11.98	12.32	11.875	11.92	22234799	{}	\N	2020-10-12 16:55:21.310207
86274	JWN	2020-10-01	11.95	12.65	11.82	12.6	19766318	{}	\N	2020-10-12 16:55:21.316447
86275	JWN	2020-10-02	12.03	12.83	12	12.8	17741170	{}	\N	2020-10-12 16:55:21.321549
86276	JWN	2020-10-05	13	13.395	12.98	13.32	14776908	{}	\N	2020-10-12 16:55:21.326383
86277	JWN	2020-10-06	13.5	13.7527	12.8	12.9	14161292	{}	\N	2020-10-12 16:55:21.330925
86278	JWN	2020-10-07	13.08	13.31	12.82	13.11	17795273	{}	\N	2020-10-12 16:55:21.335093
86279	JWN	2020-10-08	13.28	14.2	13.11	14.17	21877406	{}	\N	2020-10-12 16:55:21.339217
86280	JWN	2020-10-09	14.38	14.505	13.73	13.86	20287380	{}	\N	2020-10-12 16:55:21.343696
86281	JWN	2020-10-12	13.86	14.58	13.78	13.8899	12252061	{}	\N	2020-10-12 16:55:21.348048
86282	KBWP	2020-09-28	55.64	57.04	55.64	56.7135	2602	{}	\N	2020-10-12 16:55:23.300325
86283	KBWP	2020-09-29	56.51	56.51	55.49	56.2	2214	{}	\N	2020-10-12 16:55:23.310927
86284	KBWP	2020-09-30	56.35	56.7	56.0743	56.39	4482	{}	\N	2020-10-12 16:55:23.31905
86285	KBWP	2020-10-01	56.961	56.961	56.0107	56.0107	1769	{}	\N	2020-10-12 16:55:23.325696
86286	KBWP	2020-10-02	56.9854	57.1692	56.87	56.8701	2480	{}	\N	2020-10-12 16:55:23.332191
86287	KBWP	2020-10-05	57.27	58.28	57.27	58.07	13441	{}	\N	2020-10-12 16:55:23.337523
86288	KBWP	2020-10-06	58.3	58.62	57.46	57.5	2127	{}	\N	2020-10-12 16:55:23.343232
86289	KBWP	2020-10-07	57.72	58.2299	57.3399	57.78	6031	{}	\N	2020-10-12 16:55:23.348896
86290	KBWP	2020-10-08	58.9008	58.9008	58.3709	58.3709	852	{}	\N	2020-10-12 16:55:23.354293
86291	KBWP	2020-10-09	58.98	59.009	58.51	58.75	3203	{}	\N	2020-10-12 16:55:23.359921
86292	KBWP	2020-10-12	58.6	59.58	58.6	59.435	1729	{}	\N	2020-10-12 16:55:23.365878
86293	KEYS	2020-09-28	98.79	100.42	98.7	100.25	1189837	{}	\N	2020-10-12 16:55:26.947789
86294	KEYS	2020-09-29	100.49	101.3	99.595	99.6	1582560	{}	\N	2020-10-12 16:55:26.956181
86295	KEYS	2020-09-30	99.69	100.11	98.32	98.78	2332013	{}	\N	2020-10-12 16:55:26.962046
86296	KEYS	2020-10-01	99.68	99.94	97	97.48	2579212	{}	\N	2020-10-12 16:55:26.967153
86297	KEYS	2020-10-02	96	98.99	95.48	98.27	2256188	{}	\N	2020-10-12 16:55:26.973048
86298	KEYS	2020-10-05	99.29	99.9	98.31	99.69	2020098	{}	\N	2020-10-12 16:55:26.977852
86299	KEYS	2020-10-06	99.69	102.15	99.3	99.49	1888300	{}	\N	2020-10-12 16:55:26.982481
86300	KEYS	2020-10-07	100.53	102	100.06	101.16	1615065	{}	\N	2020-10-12 16:55:26.98737
86301	KEYS	2020-10-08	101.4	103.395	101.4	103.18	2073858	{}	\N	2020-10-12 16:55:26.991874
86302	KEYS	2020-10-09	103.5	106.29	103.375	105.26	2177642	{}	\N	2020-10-12 16:55:26.996435
86303	KEYS	2020-10-12	105.41	105.95	104.76	105.7	619664	{}	\N	2020-10-12 16:55:27.001109
86304	KGC	2020-09-28	8.89	8.89	8.61	8.87	15937174	{}	\N	2020-10-12 16:55:28.927229
86305	KGC	2020-09-29	8.92	9.08	8.8811	8.94	16145030	{}	\N	2020-10-12 16:55:28.941359
86306	KGC	2020-09-30	8.9	8.925	8.67	8.82	16690017	{}	\N	2020-10-12 16:55:28.949453
86307	KGC	2020-10-01	8.97	9.19	8.85	9.05	21488626	{}	\N	2020-10-12 16:55:28.955916
86308	KGC	2020-10-02	9.02	9.09	8.85	8.89	14557289	{}	\N	2020-10-12 16:55:28.962129
86309	KGC	2020-10-05	8.94	9.31	8.92	9.14	20309505	{}	\N	2020-10-12 16:55:28.967596
86310	KGC	2020-10-06	9.2	9.24	8.64	8.65	27179462	{}	\N	2020-10-12 16:55:28.973002
86311	KGC	2020-10-07	8.76	8.83	8.41	8.43	26593493	{}	\N	2020-10-12 16:55:28.978047
86312	KGC	2020-10-08	8.52	8.67	8.455	8.62	23849634	{}	\N	2020-10-12 16:55:28.982795
86313	KGC	2020-10-09	8.86	9.12	8.83	9.1	23602237	{}	\N	2020-10-12 16:55:28.987366
86314	KGC	2020-10-12	9.09	9.23	8.95	9.205	6685432	{}	\N	2020-10-12 16:55:28.991497
86315	KRNT	2020-09-28	60.41	62.685	60.19	62.67	216271	{}	\N	2020-10-12 16:55:30.049151
86316	KRNT	2020-09-29	62.37	64.8	62.205	63.87	502685	{}	\N	2020-10-12 16:55:30.058822
86317	KRNT	2020-09-30	63.79	65.29	63.26	64.87	303929	{}	\N	2020-10-12 16:55:30.066156
86318	KRNT	2020-10-01	65.48	66.2	64.59	65.07	286634	{}	\N	2020-10-12 16:55:30.072052
86319	KRNT	2020-10-02	63.5	66.14	63	64.8	298713	{}	\N	2020-10-12 16:55:30.077063
86320	KRNT	2020-10-05	65.8	65.8	63.085	63.26	150752	{}	\N	2020-10-12 16:55:30.081915
86321	KRNT	2020-10-06	63.34	64.135	62.6	63.27	155460	{}	\N	2020-10-12 16:55:30.086575
86322	KRNT	2020-10-07	64.42	65.31	64.025	64.63	134069	{}	\N	2020-10-12 16:55:30.091483
86323	KRNT	2020-10-08	65.38	67.27	64.38	67.27	197020	{}	\N	2020-10-12 16:55:30.097201
86324	KRNT	2020-10-09	67.49	69.05	67.49	68.95	198200	{}	\N	2020-10-12 16:55:30.102042
86325	KRNT	2020-10-12	69.67	69.67	68.22	69.23	76176	{}	\N	2020-10-12 16:55:30.106733
86326	LEG	2020-09-28	41.74	42.3	41.49	42.15	862049	{}	\N	2020-10-12 16:55:31.302097
86327	LEG	2020-09-29	42.14	42.15	40.555	40.72	1288510	{}	\N	2020-10-12 16:55:31.315811
86328	LEG	2020-09-30	40.99	41.9	40.875	41.17	1623369	{}	\N	2020-10-12 16:55:31.325687
86329	LEG	2020-10-01	41.58	42.15	41.16	41.65	1528133	{}	\N	2020-10-12 16:55:31.331401
86330	LEG	2020-10-02	41	42.81	40.651	42.66	1548081	{}	\N	2020-10-12 16:55:31.336734
86331	LEG	2020-10-05	42.96	43.48	42.75	43.39	911517	{}	\N	2020-10-12 16:55:31.342455
86332	LEG	2020-10-06	43.7	43.9	42.11	42.3	1256423	{}	\N	2020-10-12 16:55:31.347914
86333	LEG	2020-10-07	42.9	44.14	42.83	43.73	1807224	{}	\N	2020-10-12 16:55:31.353085
86334	LEG	2020-10-08	44.09	44.74	43.88	44.23	1278233	{}	\N	2020-10-12 16:55:31.358502
86335	LEG	2020-10-09	44.94	45.38	44.44	44.94	1671197	{}	\N	2020-10-12 16:55:31.364014
86336	LEG	2020-10-12	44.94	45.53	44.41	44.539	316260	{}	\N	2020-10-12 16:55:31.36966
86337	LGLV	2020-09-28	107.66	108.9239	107.66	108.46	44811	{}	\N	2020-10-12 16:55:32.600771
86338	LGLV	2020-09-29	108.37	108.547	107.525	107.9	40446	{}	\N	2020-10-12 16:55:32.612319
86339	LGLV	2020-09-30	108.01	109.0063	107.66	108.37	11980	{}	\N	2020-10-12 16:55:32.620367
86340	LGLV	2020-10-01	108.8	109.5	108.37	109.07	36349	{}	\N	2020-10-12 16:55:32.626873
86341	LGLV	2020-10-02	107.4	109.74	107.4	109.28	21644	{}	\N	2020-10-12 16:55:32.632588
86342	LGLV	2020-10-05	109.98	110.63	109.8222	110.57	26572	{}	\N	2020-10-12 16:55:32.63784
86343	LGLV	2020-10-06	110.75	111.4565	109.5701	109.59	97102	{}	\N	2020-10-12 16:55:32.642824
86344	LGLV	2020-10-07	110.3	111	110.2901	110.71	16477	{}	\N	2020-10-12 16:55:32.647413
86345	LGLV	2020-10-08	111.25	112.0806	111.186	111.93	12505	{}	\N	2020-10-12 16:55:32.651959
86346	LGLV	2020-10-09	112.36	112.8	112.01	112.33	36607	{}	\N	2020-10-12 16:55:32.656452
86347	LGLV	2020-10-12	112.81	113.46	112.645	113.46	13099	{}	\N	2020-10-12 16:55:32.661121
86348	LTRPA	2020-09-28	1.74	1.79	1.7101	1.78	694963	{}	\N	2020-10-12 16:55:35.839455
86349	LTRPA	2020-09-29	1.78	1.79	1.69	1.76	797322	{}	\N	2020-10-12 16:55:35.85062
86350	LTRPA	2020-09-30	1.7849	1.825	1.68	1.73	1259278	{}	\N	2020-10-12 16:55:35.85824
86351	LTRPA	2020-10-01	1.73	1.79	1.7	1.78	564846	{}	\N	2020-10-12 16:55:35.864494
86352	LTRPA	2020-10-02	1.66	1.8	1.655	1.74	734808	{}	\N	2020-10-12 16:55:35.870285
86353	LTRPA	2020-10-05	1.75	1.79	1.68	1.72	585022	{}	\N	2020-10-12 16:55:35.875684
86354	LTRPA	2020-10-06	1.75	1.93	1.7464	1.78	1076908	{}	\N	2020-10-12 16:55:35.880889
86355	LTRPA	2020-10-07	1.8	1.88	1.78	1.87	915325	{}	\N	2020-10-12 16:55:35.885674
86356	LTRPA	2020-10-08	1.913	1.92	1.84	1.87	495061	{}	\N	2020-10-12 16:55:35.89044
86357	LTRPA	2020-10-09	1.9	1.9	1.81	1.84	1322521	{}	\N	2020-10-12 16:55:35.895154
86358	LTRPA	2020-10-12	1.87	1.88	1.82	1.8719	182751	{}	\N	2020-10-12 16:55:35.89997
86359	MFNC	2020-09-28	9.12	9.5006	9.04	9.31	20490	{}	\N	2020-10-12 16:55:37.42416
86360	MFNC	2020-09-29	9.48	9.96	9.33	9.82	12457	{}	\N	2020-10-12 16:55:37.434531
86361	MFNC	2020-09-30	9.63	9.73	9.525	9.65	8600	{}	\N	2020-10-12 16:55:37.44161
86362	MFNC	2020-10-01	9.32	9.79	9.28	9.46	18996	{}	\N	2020-10-12 16:55:37.448034
86363	MFNC	2020-10-02	9.28	9.58	9.25	9.54	5437	{}	\N	2020-10-12 16:55:37.454065
86364	MFNC	2020-10-05	9.54	9.89	9.4	9.88	12550	{}	\N	2020-10-12 16:55:37.459967
86365	MFNC	2020-10-06	9.94	10.145	9.8	9.8	9191	{}	\N	2020-10-12 16:55:37.465818
86366	MFNC	2020-10-07	9.9101	10.15	9.9101	10	16460	{}	\N	2020-10-12 16:55:37.471197
86367	MFNC	2020-10-08	10	10.15	9.99	10.01	14276	{}	\N	2020-10-12 16:55:37.476525
86368	MFNC	2020-10-09	10.13	10.25	10.085	10.2	18154	{}	\N	2020-10-12 16:55:37.482137
86369	MFNC	2020-10-12	10.29	10.585	10.29	10.495	6171	{}	\N	2020-10-12 16:55:37.487565
86370	MSEX	2020-09-28	62.25	63.355	61.42	62.37	35364	{}	\N	2020-10-12 16:55:38.954598
86371	MSEX	2020-09-29	62.37	63.46	61.58	62.48	23184	{}	\N	2020-10-12 16:55:38.965248
86372	MSEX	2020-09-30	62.59	62.84	61.52	62.15	32513	{}	\N	2020-10-12 16:55:38.973065
86373	MSEX	2020-10-01	62.18	63.5	62.01	63.42	29595	{}	\N	2020-10-12 16:55:38.979737
86374	MSEX	2020-10-02	62.5	63.78	62.395	63.02	26928	{}	\N	2020-10-12 16:55:38.985193
86375	MSEX	2020-10-05	63.37	64.05	62.2505	63.18	27123	{}	\N	2020-10-12 16:55:38.990554
86376	MSEX	2020-10-06	63.61	64.6	61.81	62.74	24323	{}	\N	2020-10-12 16:55:38.995877
86377	MSEX	2020-10-07	63.32	63.825	62.78	63.35	46231	{}	\N	2020-10-12 16:55:39.001106
86378	MSEX	2020-10-08	63.451	64.9	63.451	64.32	27908	{}	\N	2020-10-12 16:55:39.006696
86379	MSEX	2020-10-09	64.35	64.8	63.7	64.28	22173	{}	\N	2020-10-12 16:55:39.011916
86380	MSEX	2020-10-12	64.17	66.74	64.17	66.52	18140	{}	\N	2020-10-12 16:55:39.016431
86381	MUSA	2020-09-28	131.99	133.63	130.44	132.15	225421	{}	\N	2020-10-12 16:55:41.021382
86382	MUSA	2020-09-29	132.02	132.23	128.46	128.64	491197	{}	\N	2020-10-12 16:55:41.036922
86383	MUSA	2020-09-30	128.64	130.36	127.93	128.27	508828	{}	\N	2020-10-12 16:55:41.045987
86384	MUSA	2020-10-01	128.17	130.055	127.12	127.92	390552	{}	\N	2020-10-12 16:55:41.052997
86385	MUSA	2020-10-02	125.76	128.68	124.99	127.65	506750	{}	\N	2020-10-12 16:55:41.05986
86386	MUSA	2020-10-05	128.76	129.77	125.44	127.61	425372	{}	\N	2020-10-12 16:55:41.065999
86387	MUSA	2020-10-06	128.67	130.61	127.42	127.6	443602	{}	\N	2020-10-12 16:55:41.072429
86388	MUSA	2020-10-07	127.8	128.9	127.07	127.87	376351	{}	\N	2020-10-12 16:55:41.078924
86389	MUSA	2020-10-08	128.76	130.59	127.88	130.49	324302	{}	\N	2020-10-12 16:55:41.084525
86390	MUSA	2020-10-09	131.68	131.68	129.48	129.75	168308	{}	\N	2020-10-12 16:55:41.090813
86391	MUSA	2020-10-12	130.23	131.8	130.05	131.37	64602	{}	\N	2020-10-12 16:55:41.096144
86392	MXC	2020-09-28	4.18	4.225	4.04	4.15	5127	{}	\N	2020-10-12 16:55:43.328322
86393	MXC	2020-09-29	6.02	6.49	4.43	5.1	1729208	{}	\N	2020-10-12 16:55:43.33922
86394	MXC	2020-09-30	4.44	4.77	4.3709	4.77	63058	{}	\N	2020-10-12 16:55:43.347062
86395	MXC	2020-10-01	4.7044	4.72	4.2	4.36	33111	{}	\N	2020-10-12 16:55:43.353308
86396	MXC	2020-10-02	4.33	4.85	4.33	4.56	84146	{}	\N	2020-10-12 16:55:43.35869
86397	MXC	2020-10-05	4.66	4.66	4.33	4.4107	13672	{}	\N	2020-10-12 16:55:43.363508
86398	MXC	2020-10-06	4.41	4.6799	4.31	4.4001	39921	{}	\N	2020-10-12 16:55:43.369865
86399	MXC	2020-10-07	4.36	4.75	4.3	4.615	54684	{}	\N	2020-10-12 16:55:43.37473
86400	MXC	2020-10-08	4.75	8.79	4.6	6	6913593	{}	\N	2020-10-12 16:55:43.37967
86401	MXC	2020-10-09	5.25	5.85	5.2	5.5	364626	{}	\N	2020-10-12 16:55:43.38437
86402	MXC	2020-10-12	5.1	5.78	5.1	5.4408	204407	{}	\N	2020-10-12 16:55:43.388981
86403	NI	2020-09-28	22.18	22.28	21.84	21.85	3650839	{}	\N	2020-10-12 16:55:44.819088
86404	NI	2020-09-29	21.97	22.125	21.605	21.96	5757668	{}	\N	2020-10-12 16:55:44.830842
86405	NI	2020-09-30	22.13	22.29	21.96	22	7574909	{}	\N	2020-10-12 16:55:44.839347
86406	NI	2020-10-01	22.03	22.09	21.635	22	13532221	{}	\N	2020-10-12 16:55:44.84679
86407	NI	2020-10-02	21.79	22.32	21.67	22.14	9163142	{}	\N	2020-10-12 16:55:44.853618
86408	NI	2020-10-05	22.17	22.25	21.85	21.99	9004298	{}	\N	2020-10-12 16:55:44.859041
86409	NI	2020-10-06	22.04	22.795	21.95	22.41	4989521	{}	\N	2020-10-12 16:55:44.86407
86410	NI	2020-10-07	22.43	22.52	21.98	22.05	8647628	{}	\N	2020-10-12 16:55:44.86927
86411	NI	2020-10-08	22.15	22.485	22.07	22.36	8885166	{}	\N	2020-10-12 16:55:44.874309
86412	NI	2020-10-09	22.41	22.6	22.22	22.49	5330021	{}	\N	2020-10-12 16:55:44.87929
86413	NI	2020-10-12	22.49	23.095	22.44	23.045	2707745	{}	\N	2020-10-12 16:55:44.884523
86414	KGRN	2020-09-28	29.26	29.89	29.26	29.7	9445	{}	\N	2020-10-12 16:55:46.991093
86415	KGRN	2020-09-29	29.99	30.32	29.85	30.195	5005	{}	\N	2020-10-12 16:55:47.001822
86416	KGRN	2020-09-30	31.2	31.389	31.0455	31.1078	10522	{}	\N	2020-10-12 16:55:47.009846
86417	KGRN	2020-10-01	31.81	31.81	31.42	31.6999	6868	{}	\N	2020-10-12 16:55:47.015416
86418	KGRN	2020-10-02	31.42	32.04	31.42	31.49	9450	{}	\N	2020-10-12 16:55:47.020608
86419	KGRN	2020-10-05	31.82	31.99	31.4802	31.81	8879	{}	\N	2020-10-12 16:55:47.025942
86420	KGRN	2020-10-06	32.47	32.47	32	32.03	21986	{}	\N	2020-10-12 16:55:47.030308
86421	KGRN	2020-10-07	33.63	33.63	32.19	32.59	30356	{}	\N	2020-10-12 16:55:47.035013
86422	KGRN	2020-10-08	33.16	33.53	33.14	33.34	28527	{}	\N	2020-10-12 16:55:47.040135
86423	KGRN	2020-10-09	33.01	34.09	32.8	33.92	15592	{}	\N	2020-10-12 16:55:47.044663
86424	KGRN	2020-10-12	34.35	34.35	33.25	33.49	14304	{}	\N	2020-10-12 16:55:47.049218
86425	KINS	2020-09-28	6.13	6.15	5.74	5.77	21400	{}	\N	2020-10-12 16:55:48.298755
86426	KINS	2020-09-29	5.7719	5.85	5.74	5.84	7346	{}	\N	2020-10-12 16:55:48.30888
86427	KINS	2020-09-30	5.99	5.99	5.79	5.89	10291	{}	\N	2020-10-12 16:55:48.315732
86428	KINS	2020-10-01	5.98	5.98	5.76	5.88	12804	{}	\N	2020-10-12 16:55:48.321472
86429	KINS	2020-10-02	5.76	6	5.76	6	14213	{}	\N	2020-10-12 16:55:48.326867
86430	KINS	2020-10-05	5.7705	6.1	5.7705	6.02	10144	{}	\N	2020-10-12 16:55:48.331176
86431	KINS	2020-10-06	5.89	6.28	5.83	6.2	8731	{}	\N	2020-10-12 16:55:48.33519
86432	KINS	2020-10-07	6.23	6.24	6.11	6.24	37070	{}	\N	2020-10-12 16:55:48.339133
86433	KINS	2020-10-08	6.0762	6.09	5.85	6.02	30385	{}	\N	2020-10-12 16:55:48.343222
86434	KINS	2020-10-09	5.83	6.09	5.83	6.02	8878	{}	\N	2020-10-12 16:55:48.347092
86435	KINS	2020-10-12	6.01	6.21	5.9501	6.1409	20469	{}	\N	2020-10-12 16:55:48.351329
86436	KVHI	2020-09-28	9.24	9.46	9.16	9.4	131548	{}	\N	2020-10-12 16:55:49.289155
86437	KVHI	2020-09-29	9.39	9.5	9.17	9.23	57320	{}	\N	2020-10-12 16:55:49.300975
86438	KVHI	2020-09-30	9.23	9.5741	8.93	9.01	70503	{}	\N	2020-10-12 16:55:49.309069
86439	KVHI	2020-10-01	8.98	9.2	8.97	9.12	36050	{}	\N	2020-10-12 16:55:49.314892
86440	KVHI	2020-10-02	8.94	9.63	8.94	9.31	75065	{}	\N	2020-10-12 16:55:49.320879
86441	KVHI	2020-10-05	9.33	9.59	9.31	9.58	101412	{}	\N	2020-10-12 16:55:49.326178
86442	KVHI	2020-10-06	9.63	9.75	9.45	9.46	44134	{}	\N	2020-10-12 16:55:49.331147
86443	KVHI	2020-10-07	9.62	9.73	9.4	9.67	40692	{}	\N	2020-10-12 16:55:49.335357
86444	KVHI	2020-10-08	9.75	9.77	9.35	9.6	38225	{}	\N	2020-10-12 16:55:49.339615
86445	KVHI	2020-10-09	9.635	9.72	9.35	9.48	36568	{}	\N	2020-10-12 16:55:49.344214
86446	KVHI	2020-10-12	9.57	9.74	9.36	9.52	17397	{}	\N	2020-10-12 16:55:49.34859
86447	MLSS	2020-09-28	1.41	1.44	1.37	1.43	212064	{}	\N	2020-10-12 16:55:51.887372
86448	MLSS	2020-09-29	1.4318	1.48	1.43	1.46	58511	{}	\N	2020-10-12 16:55:51.896895
86449	MLSS	2020-09-30	1.43	1.47	1.37	1.41	105810	{}	\N	2020-10-12 16:55:51.904106
86450	MLSS	2020-10-01	1.45	1.55	1.36	1.52	245370	{}	\N	2020-10-12 16:55:51.91034
86451	MLSS	2020-10-02	1.46	1.63	1.46	1.58	139986	{}	\N	2020-10-12 16:55:51.915817
86452	MLSS	2020-10-05	1.58	1.68	1.51	1.64	162525	{}	\N	2020-10-12 16:55:51.921352
86453	MLSS	2020-10-06	1.66	1.7	1.6	1.6	130110	{}	\N	2020-10-12 16:55:51.926107
86454	MLSS	2020-10-07	1.62	1.87	1.6154	1.86	280048	{}	\N	2020-10-12 16:55:51.931112
86455	MLSS	2020-10-08	1.9	1.95	1.69	1.75	374362	{}	\N	2020-10-12 16:55:51.936735
86456	MLSS	2020-10-09	1.75	1.89	1.71	1.76	316239	{}	\N	2020-10-12 16:55:51.941524
86457	MLSS	2020-10-12	1.75	1.82	1.735	1.75	83776	{}	\N	2020-10-12 16:55:51.946556
86458	MOM	2020-09-28	31.41	31.41	30.239	31.03	11726	{}	\N	2020-10-12 16:55:52.941501
86459	MOM	2020-09-29	31.14	31.51	30.7375	31.3	9096	{}	\N	2020-10-12 16:55:52.952358
86460	MOM	2020-09-30	30.7973	31.1	30.74	30.9585	7886	{}	\N	2020-10-12 16:55:52.958728
86461	MOM	2020-10-01	31.67	31.67	30.8091	31.41	7741	{}	\N	2020-10-12 16:55:52.965018
86462	MOM	2020-10-02	31.4554	31.54	31	31.13	2814	{}	\N	2020-10-12 16:55:52.970618
86463	MOM	2020-10-05	31.18	31.18	30.5573	30.995	2952	{}	\N	2020-10-12 16:55:52.97527
86464	MOM	2020-10-06	30.7	30.7	29.925	30.68	7530	{}	\N	2020-10-12 16:55:52.980059
86465	MOM	2020-10-07	30.84	30.9	30.5854	30.5854	1845	{}	\N	2020-10-12 16:55:52.984386
86466	MOM	2020-10-08	30.2918	30.2918	30.0966	30.0966	3048	{}	\N	2020-10-12 16:55:52.989195
86467	MOM	2020-10-09	30.0625	31.18	29.9	31.18	15714	{}	\N	2020-10-12 16:55:52.993708
86468	MOM	2020-10-12	31.13	31.13	29.839	30.6702	8935	{}	\N	2020-10-12 16:55:52.998323
86469	MPAA	2020-09-28	15.23	15.92	15.23	15.4	91954	{}	\N	2020-10-12 16:55:54.02714
86470	MPAA	2020-09-29	15.7	15.7	15.18	15.27	49943	{}	\N	2020-10-12 16:55:54.038061
86471	MPAA	2020-09-30	15.26	15.92	15.26	15.56	82580	{}	\N	2020-10-12 16:55:54.04457
86472	MPAA	2020-10-01	15.63	16.1	15.41	15.87	66424	{}	\N	2020-10-12 16:55:54.051156
86473	MPAA	2020-10-02	15.45	15.77	15.45	15.59	111893	{}	\N	2020-10-12 16:55:54.057472
86474	MPAA	2020-10-05	15.82	16.49	15.7	16.12	80648	{}	\N	2020-10-12 16:55:54.062298
86475	MPAA	2020-10-06	16.595	16.8	15.82	15.98	135242	{}	\N	2020-10-12 16:55:54.066299
86476	MPAA	2020-10-07	16.04	16.18	15.63	15.67	104321	{}	\N	2020-10-12 16:55:54.070499
86477	MPAA	2020-10-08	15.66	15.96	15.47	15.69	46324	{}	\N	2020-10-12 16:55:54.075814
86478	MPAA	2020-10-09	15.88	16.115	15.72	15.75	83063	{}	\N	2020-10-12 16:55:54.080952
86479	MPAA	2020-10-12	15.75	16.43	15.54	15.98	32103	{}	\N	2020-10-12 16:55:54.085539
86480	MVIN	2020-09-28	39.82	39.8706	39.82	39.8706	399	{}	\N	2020-10-12 16:55:55.067201
86481	MVIN	2020-09-29	39.815	39.925	39.815	39.925	795	{}	\N	2020-10-12 16:55:55.076741
86482	MVIN	2020-09-30	39.83	39.88	39.82	39.88	3289	{}	\N	2020-10-12 16:55:55.083884
86483	MVIN	2020-10-01	39.9839	39.9955	39.9364	39.9955	1241	{}	\N	2020-10-12 16:55:55.089715
86484	MVIN	2020-10-02	39.8419	39.926	39.8419	39.9203	1577	{}	\N	2020-10-12 16:55:55.095081
86485	MVIN	2020-10-05	40.27	40.35	40.25	40.35	830	{}	\N	2020-10-12 16:55:55.09948
86486	MVIN	2020-10-06	40.1452	40.2	39.83	39.83	1713	{}	\N	2020-10-12 16:55:55.103709
86487	MVIN	2020-10-07	40.0544	40.0544	40.0544	40.0544	169	{}	\N	2020-10-12 16:55:55.1081
86488	MVIN	2020-10-08	40.09	40.1496	40.07	40.1496	1376	{}	\N	2020-10-12 16:55:55.112726
86489	MVIN	2020-10-09	40.44	40.46	40.295	40.44	82331	{}	\N	2020-10-12 16:55:55.117471
86490	MVIN	2020-10-12	40.5119	40.7	40.5119	40.6998	41958	{}	\N	2020-10-12 16:55:55.122622
86491	MXF	2020-09-28	10.05	10.15	10.01	10.05	33540	{}	\N	2020-10-12 16:55:56.166451
86492	MXF	2020-09-29	10.09	10.1369	9.97	10.085	70960	{}	\N	2020-10-12 16:55:56.175779
86493	MXF	2020-09-30	10.19	10.3	10.19	10.26	48722	{}	\N	2020-10-12 16:55:56.182263
86494	MXF	2020-10-01	10.37	10.38	10.16	10.25	34806	{}	\N	2020-10-12 16:55:56.187677
86495	MXF	2020-10-02	10.1	10.3319	9.75	10.32	26172	{}	\N	2020-10-12 16:55:56.192119
86496	MXF	2020-10-05	10.42	10.49	10.3827	10.45	43437	{}	\N	2020-10-12 16:55:56.197915
86497	MXF	2020-10-06	10.46	10.485	10.31	10.31	53020	{}	\N	2020-10-12 16:55:56.202468
86498	MXF	2020-10-07	10.46	10.5755	10.33	10.48	111930	{}	\N	2020-10-12 16:55:56.206543
86499	MXF	2020-10-08	10.56	10.82	10.56	10.79	37950	{}	\N	2020-10-12 16:55:56.210579
86500	MXF	2020-10-09	10.9	10.99	10.89	10.9356	31868	{}	\N	2020-10-12 16:55:56.215137
86501	MXF	2020-10-12	10.93	10.97	10.8701	10.945	18322	{}	\N	2020-10-12 16:55:56.219075
86502	NGE	2020-09-28	9.34	9.35	9.14	9.35	15061	{}	\N	2020-10-12 16:55:57.568554
86503	NGE	2020-09-29	9.35	9.45	9.325	9.45	4227	{}	\N	2020-10-12 16:55:57.580623
86504	NGE	2020-09-30	9.45	9.47	9.315	9.3925	7856	{}	\N	2020-10-12 16:55:57.588336
86505	NGE	2020-10-01	9.5	9.5	9.3742	9.47	13741	{}	\N	2020-10-12 16:55:57.594025
86506	NGE	2020-10-02	9.5	9.75	9.375	9.72	14167	{}	\N	2020-10-12 16:55:57.600198
86507	NGE	2020-10-05	9.75	10	9.7	9.9	17504	{}	\N	2020-10-12 16:55:57.605769
86508	NGE	2020-10-06	10	10.75	9.99	10.22	22818	{}	\N	2020-10-12 16:55:57.610306
86509	NGE	2020-10-07	10.25	10.4199	10.22	10.26	8690	{}	\N	2020-10-12 16:55:57.61476
86510	NGE	2020-10-08	10.28	10.5	10.27	10.3	11901	{}	\N	2020-10-12 16:55:57.620074
86511	NGE	2020-10-09	10.2	10.34	10.18	10.18	6071	{}	\N	2020-10-12 16:55:57.62453
86512	NGE	2020-10-12	10.18	10.21	10.0499	10.08	10233	{}	\N	2020-10-12 16:55:57.62892
86513	NXP	2020-09-28	16.9	17.02	16.79	16.79	25202	{}	\N	2020-10-12 16:55:59.001684
86514	NXP	2020-09-29	16.85	16.98	16.53	16.57	68322	{}	\N	2020-10-12 16:55:59.014109
86515	NXP	2020-09-30	17	17	16.5701	16.69	55612	{}	\N	2020-10-12 16:55:59.021987
86516	NXP	2020-10-01	16.83	16.98	16.76	16.945	54126	{}	\N	2020-10-12 16:55:59.02774
86517	NXP	2020-10-02	17	17	16.9	16.95	39210	{}	\N	2020-10-12 16:55:59.032551
86518	NXP	2020-10-05	16.78	16.99	16.78	16.99	88638	{}	\N	2020-10-12 16:55:59.037822
86519	NXP	2020-10-06	16.99	17.08	16.9827	17	30184	{}	\N	2020-10-12 16:55:59.042256
86520	NXP	2020-10-07	17	17.12	16.85	16.95	48214	{}	\N	2020-10-12 16:55:59.049499
86521	NXP	2020-10-08	16.88	17.08	16.865	16.98	28990	{}	\N	2020-10-12 16:55:59.054499
86522	NXP	2020-10-09	17.49	17.49	16.91	16.91	35390	{}	\N	2020-10-12 16:55:59.059396
86523	NXP	2020-10-12	17.47	17.5	17.06	17.28	55294	{}	\N	2020-10-12 16:55:59.064113
86524	OFC	2020-09-28	23.13	23.99	23.02	23.81	957356	{}	\N	2020-10-12 16:56:00.518122
86525	OFC	2020-09-29	23.45	23.505	22.765	23.44	1322381	{}	\N	2020-10-12 16:56:00.530612
86526	OFC	2020-09-30	23.7	23.98	23.43	23.72	3038146	{}	\N	2020-10-12 16:56:00.537356
86527	OFC	2020-10-01	23.72	24.4	23.42	24.39	1715319	{}	\N	2020-10-12 16:56:00.543969
86528	OFC	2020-10-02	23.91	25.07	23.73	25.03	2392501	{}	\N	2020-10-12 16:56:00.549193
86529	OFC	2020-10-05	25.22	25.33	24.7	25.17	1846167	{}	\N	2020-10-12 16:56:00.554109
86530	OFC	2020-10-06	25.36	25.54	24.86	25.2	3075740	{}	\N	2020-10-12 16:56:00.558607
86531	OFC	2020-10-07	25.39	25.39	24.69	24.75	2384296	{}	\N	2020-10-12 16:56:00.56268
86532	OFC	2020-10-08	24.89	25.54	24.87	25.53	1295898	{}	\N	2020-10-12 16:56:00.566789
86533	OFC	2020-10-09	25.67	25.86	25.2	25.79	2393807	{}	\N	2020-10-12 16:56:00.571157
86534	OFC	2020-10-12	25.72	25.87	25.45	25.66	928089	{}	\N	2020-10-12 16:56:00.575609
86535	PBSM	2020-09-28	26.17	26.1738	26.143	26.143	780	{}	\N	2020-10-12 16:56:03.346855
86536	PBSM	2020-09-29	26	26.0588	26	26.0588	1317	{}	\N	2020-10-12 16:56:03.355859
86537	PBSM	2020-09-30	26.33	26.33	26.1477	26.1477	220	{}	\N	2020-10-12 16:56:03.362002
86538	PBSM	2020-10-01	26.44	26.4649	26.44	26.4649	669	{}	\N	2020-10-12 16:56:03.367382
86539	PBSM	2020-10-02	26.6661	26.6901	26.6661	26.6901	236	{}	\N	2020-10-12 16:56:03.372804
86540	PBSM	2020-10-05	27.3098	27.3098	27.3098	27.3098	112	{}	\N	2020-10-12 16:56:03.377705
86541	PBSM	2020-10-06	27.81	27.845	27.3302	27.3302	2616	{}	\N	2020-10-12 16:56:03.38282
86542	PBSM	2020-10-07	27.8247	27.8247	27.8247	27.8247	118	{}	\N	2020-10-12 16:56:03.387931
86543	PBSM	2020-10-08	27.99	28.0929	27.99	28.0929	2714	{}	\N	2020-10-12 16:56:03.392807
86544	PBSM	2020-10-09	29.47	29.47	28.4	28.4	626	{}	\N	2020-10-12 16:56:03.39775
86545	MAG	2020-09-28	15.89	15.89	15.05	15.18	451636	{}	\N	2020-10-12 16:56:06.754329
86546	MAG	2020-09-29	16	16.82	15.5	16.57	1114044	{}	\N	2020-10-12 16:56:06.768207
86547	MAG	2020-09-30	16.5	16.6437	15.96	16.25	520642	{}	\N	2020-10-12 16:56:06.777179
86548	MAG	2020-10-01	16.5	16.97	16.38	16.78	508573	{}	\N	2020-10-12 16:56:06.784676
86549	MAG	2020-10-02	16.59	17.14	16.45	16.79	347466	{}	\N	2020-10-12 16:56:06.790519
86550	MAG	2020-10-05	17.03	17.47	16.9637	17.3	352057	{}	\N	2020-10-12 16:56:06.795887
86551	MAG	2020-10-06	17.42	17.425	16.31	16.41	430306	{}	\N	2020-10-12 16:56:06.800661
86552	MAG	2020-10-07	16.57	16.93	16.37	16.53	267173	{}	\N	2020-10-12 16:56:06.805569
86553	MAG	2020-10-08	16.77	17.0999	16.68	16.98	277279	{}	\N	2020-10-12 16:56:06.8106
86554	MAG	2020-10-09	17.43	18.5	17.41	18.43	804010	{}	\N	2020-10-12 16:56:06.815575
86555	MAG	2020-10-12	18.67	18.99	18.19	18.93	354980	{}	\N	2020-10-12 16:56:06.820636
86556	MIC	2020-09-28	26.21	26.355	25.8	25.99	499000	{}	\N	2020-10-12 16:56:07.908571
86557	MIC	2020-09-29	25.9	26.375	25.73	26.35	1205083	{}	\N	2020-10-12 16:56:07.917053
86558	MIC	2020-09-30	26.36	26.94	26.32	26.89	3483808	{}	\N	2020-10-12 16:56:07.922391
86559	MIC	2020-10-01	27	27.22	26.48	26.85	1713950	{}	\N	2020-10-12 16:56:07.927923
86560	MIC	2020-10-02	26.26	26.81	26.26	26.49	2036897	{}	\N	2020-10-12 16:56:07.933052
86561	MIC	2020-10-05	26.71	26.93	26.19	26.68	3244895	{}	\N	2020-10-12 16:56:07.937089
86562	MIC	2020-10-06	27	27.26	26.1	26.15	1281208	{}	\N	2020-10-12 16:56:07.941372
86563	MIC	2020-10-07	26.35	26.54	25.78	25.95	1348826	{}	\N	2020-10-12 16:56:07.94636
86564	MIC	2020-10-08	26.14	26.71	25.95	26.59	1099615	{}	\N	2020-10-12 16:56:07.950592
86565	MIC	2020-10-09	26.78	26.94	26.23	26.75	848800	{}	\N	2020-10-12 16:56:07.954693
86566	MIC	2020-10-12	26.79	27.13	26.5563	26.89	157567	{}	\N	2020-10-12 16:56:07.959174
86567	MTD	2020-09-28	958.84	981.93	958.84	968.47	87297	{}	\N	2020-10-12 16:56:09.693635
86568	MTD	2020-09-29	974.04	981.4	959.89	968.59	187186	{}	\N	2020-10-12 16:56:09.705968
86569	MTD	2020-09-30	974.22	986.785	958.3	965.75	335840	{}	\N	2020-10-12 16:56:09.713025
86570	MTD	2020-10-01	974.64	985.7734	964	970.92	189153	{}	\N	2020-10-12 16:56:09.719957
86571	MTD	2020-10-02	959.6	977.63	959.6	966.92	162658	{}	\N	2020-10-12 16:56:09.72576
86572	MTD	2020-10-05	979.95	993.06	979.95	990.43	199836	{}	\N	2020-10-12 16:56:09.731261
86573	MTD	2020-10-06	992.28	1009.95	992.28	993.85	232698	{}	\N	2020-10-12 16:56:09.736056
86574	MTD	2020-10-07	977.02	1013.23	977.02	1011.27	218042	{}	\N	2020-10-12 16:56:09.740991
86575	MTD	2020-10-08	1015.45	1024.9199	1011.86	1012.08	185632	{}	\N	2020-10-12 16:56:09.745855
86576	MTD	2020-10-09	1021.11	1029.615	1015.26	1025.46	179072	{}	\N	2020-10-12 16:56:09.751242
86577	MTD	2020-10-12	1027.6	1033.9	1023.47	1026.825	66955	{}	\N	2020-10-12 16:56:09.756365
86578	MTUM	2020-09-28	146.43	146.43	144.79	146.04	740413	{}	\N	2020-10-12 16:56:10.703452
86579	MTUM	2020-09-29	145.94	146.63	145.49	145.94	544062	{}	\N	2020-10-12 16:56:10.712543
86580	MTUM	2020-09-30	145.98	148.6297	145.95	147.4	880806	{}	\N	2020-10-12 16:56:10.71898
86581	MTUM	2020-10-01	149.04	149.42	148.13	149.25	1538205	{}	\N	2020-10-12 16:56:10.724684
86582	MTUM	2020-10-02	146.35	148.38	145.5	145.8	861747	{}	\N	2020-10-12 16:56:10.73035
86583	MTUM	2020-10-05	146.93	149.06	146.93	149.06	547711	{}	\N	2020-10-12 16:56:10.735252
86584	MTUM	2020-10-06	149.26	149.45	146.31	146.75	624162	{}	\N	2020-10-12 16:56:10.740106
86585	MTUM	2020-10-07	148.12	149.89	148.12	149.43	473011	{}	\N	2020-10-12 16:56:10.744533
86586	MTUM	2020-10-08	150.79	150.79	149.69	150.02	605835	{}	\N	2020-10-12 16:56:10.749144
86587	MTUM	2020-10-09	150.69	151.88	150.61	151.88	576014	{}	\N	2020-10-12 16:56:10.75456
86588	MTUM	2020-10-12	153.44	155.25	153.19	155.0124	169088	{}	\N	2020-10-12 16:56:10.759148
86589	MUNI	2020-09-28	56.28	56.29	56.2501	56.2601	15446	{}	\N	2020-10-12 16:56:11.746534
86590	MUNI	2020-09-29	56.27	56.29	56.2	56.275	47118	{}	\N	2020-10-12 16:56:11.759914
86591	MUNI	2020-09-30	56.26	56.26	56.2	56.2	74578	{}	\N	2020-10-12 16:56:11.768959
86592	MUNI	2020-10-01	56.13	56.1934	56.062	56.11	48976	{}	\N	2020-10-12 16:56:11.774769
86593	MUNI	2020-10-02	56.14	56.15	56.1001	56.14	22786	{}	\N	2020-10-12 16:56:11.779499
86594	MUNI	2020-10-05	56.14	56.15	56.1	56.1	28534	{}	\N	2020-10-12 16:56:11.784352
86595	MUNI	2020-10-06	56.11	56.11	56.01	56.0101	87097	{}	\N	2020-10-12 16:56:11.788266
86596	MUNI	2020-10-07	56.0499	56.0499	55.94	55.96	77468	{}	\N	2020-10-12 16:56:11.79211
86597	MUNI	2020-10-08	55.94	56.03	55.94	56.02	35123	{}	\N	2020-10-12 16:56:11.797248
86598	MUNI	2020-10-09	56	56.03	55.9987	56.01	98995	{}	\N	2020-10-12 16:56:11.801895
86599	MUNI	2020-10-12	56.04	56.04	55.982	56	17380	{}	\N	2020-10-12 16:56:11.807229
86600	MVV	2020-09-28	30.53	31.34	30.53	31.26	45696	{}	\N	2020-10-12 16:56:13.336236
86601	MVV	2020-09-29	31.2	31.31	30.63	30.87	12854	{}	\N	2020-10-12 16:56:13.344676
86602	MVV	2020-09-30	31.04	31.85	30.8394	31.27	42282	{}	\N	2020-10-12 16:56:13.352005
86603	MVV	2020-10-01	31.59	32.04	31.32	32.04	206800	{}	\N	2020-10-12 16:56:13.358166
86604	MVV	2020-10-02	30.97	32.89	30.97	32.65	80892	{}	\N	2020-10-12 16:56:13.362993
86605	MVV	2020-10-05	33.21	34.18	33.21	34.18	51033	{}	\N	2020-10-12 16:56:13.368383
86606	MVV	2020-10-06	34.6	35.34	33.67	33.82	56384	{}	\N	2020-10-12 16:56:13.37352
86607	MVV	2020-10-07	34.59	35.19	34.59	35.05	45719	{}	\N	2020-10-12 16:56:13.378832
86608	MVV	2020-10-08	35.65	35.77	35.28	35.77	50848	{}	\N	2020-10-12 16:56:13.383981
86609	MVV	2020-10-09	36.4029	36.48	35.721	35.94	33188	{}	\N	2020-10-12 16:56:13.389024
86610	MVV	2020-10-12	36.31	36.4	35.99	36.33	57102	{}	\N	2020-10-12 16:56:13.393983
86611	MYF	2020-09-28	13.28	13.39	13.28	13.355	45918	{}	\N	2020-10-12 16:56:14.96404
86612	MYF	2020-09-29	13.4	13.4	13.31	13.4	44148	{}	\N	2020-10-12 16:56:14.974574
86613	MYF	2020-09-30	13.4	13.4	13.321	13.37	38700	{}	\N	2020-10-12 16:56:14.981438
86614	MYF	2020-10-01	13.39	13.4	13.28	13.375	32958	{}	\N	2020-10-12 16:56:14.987413
86615	MYF	2020-10-02	13.38	13.38	13.31	13.36	41914	{}	\N	2020-10-12 16:56:14.992889
86616	MYF	2020-10-05	13.33	13.345	13.2226	13.33	30221	{}	\N	2020-10-12 16:56:14.997885
86617	MYF	2020-10-06	13.26	13.32	13.26	13.32	16120	{}	\N	2020-10-12 16:56:15.002497
86618	MYF	2020-10-07	13.33	13.381	13.27	13.35	26372	{}	\N	2020-10-12 16:56:15.007097
86619	MYF	2020-10-08	13.37	13.51	13.19	13.43	81706	{}	\N	2020-10-12 16:56:15.011641
86620	MYF	2020-10-09	13.38	14.0889	13.36	13.62	37184	{}	\N	2020-10-12 16:56:15.016116
86621	MYF	2020-10-12	13.61	13.61	13.45	13.55	4124	{}	\N	2020-10-12 16:56:15.020275
86622	NEE	2020-09-28	283.46	286.03	282.64	284.14	1733541	{}	\N	2020-10-12 16:56:16.313572
86623	NEE	2020-09-29	284.1	285.93	282.2	283.12	3577950	{}	\N	2020-10-12 16:56:16.325576
86624	NEE	2020-09-30	280.785	283.13	267.14	277.56	11613551	{}	\N	2020-10-12 16:56:16.332491
86625	NEE	2020-10-01	279.08	284.04	278.64	282.76	4954045	{}	\N	2020-10-12 16:56:16.338459
86626	NEE	2020-10-02	278.77	287.15	278.5	281.17	4097857	{}	\N	2020-10-12 16:56:16.343836
86627	NEE	2020-10-05	283.84	288.98	282.93	287.91	3267247	{}	\N	2020-10-12 16:56:16.348731
86628	NEE	2020-10-06	288.64	292.5	285.26	290.28	3631473	{}	\N	2020-10-12 16:56:16.353208
86629	NEE	2020-10-07	292.95	297.77	289.5	297.12	5113391	{}	\N	2020-10-12 16:56:16.358027
86630	NEE	2020-10-08	298.67	301.98	296.93	301.04	2896215	{}	\N	2020-10-12 16:56:16.362845
86631	NEE	2020-10-09	303.72	303.72	300.295	302.91	3221502	{}	\N	2020-10-12 16:56:16.367338
86632	NEE	2020-10-12	303	306.99	302	306.75	1215151	{}	\N	2020-10-12 16:56:16.372145
86633	NXPI	2020-09-28	122.46	124.17	121.905	124	1990181	{}	\N	2020-10-12 16:56:18.318018
86634	NXPI	2020-09-29	126.1	126.14	123.16	123.29	1530593	{}	\N	2020-10-12 16:56:18.330424
86635	NXPI	2020-09-30	123.87	126.77	123.22	124.81	1820678	{}	\N	2020-10-12 16:56:18.338368
86636	NXPI	2020-10-01	129.63	132.17	128	130.99	3235086	{}	\N	2020-10-12 16:56:18.344517
86637	NXPI	2020-10-02	127.17	130.48	126.64	127.39	1981832	{}	\N	2020-10-12 16:56:18.350695
86638	NXPI	2020-10-05	129.48	132.07	129	132	2197075	{}	\N	2020-10-12 16:56:18.356588
86639	NXPI	2020-10-06	131.7	135.01	130.39	130.9	1998657	{}	\N	2020-10-12 16:56:18.3616
86640	NXPI	2020-10-07	133.425	134.91	133.005	133.6	1738189	{}	\N	2020-10-12 16:56:18.36668
86641	NXPI	2020-10-08	134.99	135.92	134.13	134.83	2169376	{}	\N	2020-10-12 16:56:18.371973
86642	NXPI	2020-10-09	142.01	145.15	138.8	141.53	4917260	{}	\N	2020-10-12 16:56:18.376572
86643	NXPI	2020-10-12	143	143.25	140.41	141.6	1285849	{}	\N	2020-10-12 16:56:18.381364
86644	OAS	2020-09-28	0.4173	0.4184	0.4	0.4117	10470973	{}	\N	2020-10-12 16:56:19.569611
86645	OAS	2020-09-29	0.42	0.421	0.39	0.4094	11886948	{}	\N	2020-10-12 16:56:19.578863
86646	OAS	2020-09-30	0.2706	0.3	0.19	0.28	154325765	{}	\N	2020-10-12 16:56:19.584863
86647	OAS	2020-10-01	0.236	0.2517	0.2102	0.2115	75777898	{}	\N	2020-10-12 16:56:19.590476
86648	OAS	2020-10-02	0.2236	0.227899	0.1711	0.1711	157255538	{}	\N	2020-10-12 16:56:19.595399
86649	OAS	2020-10-05	0.1935	0.1939	0.171	0.177	67224746	{}	\N	2020-10-12 16:56:19.600129
86650	OAS	2020-10-06	0.18	0.189	0.1721	0.1755	65179333	{}	\N	2020-10-12 16:56:19.604958
86651	OAS	2020-10-07	0.1773	0.1799	0.17	0.1734	42935406	{}	\N	2020-10-12 16:56:19.610014
86652	OAS	2020-10-08	0.1731	0.176	0.16	0.1695	67144309	{}	\N	2020-10-12 16:56:19.614279
86653	OAS	2020-10-09	0.172	0.175	0.1503	0.155	74764741	{}	\N	2020-10-12 16:56:19.61843
86654	OCIO	2020-09-28	27.959	27.9623	27.959	27.9623	1000	{}	\N	2020-10-12 16:56:22.459792
86655	OCIO	2020-09-29	27.926	27.926	27.926	27.926	1	{}	\N	2020-10-12 16:56:22.473773
86656	OCIO	2020-09-30	27.9591	28.1098	27.9591	28.1098	409	{}	\N	2020-10-12 16:56:22.482039
86657	OCIO	2020-10-01	28.1092	28.1092	28.1092	28.1092	7	{}	\N	2020-10-12 16:56:22.488533
86658	OCIO	2020-10-05	28.14	28.255	28.14	28.255	516	{}	\N	2020-10-12 16:56:22.494726
86659	OCIO	2020-10-06	28.3	28.339	28.1092	28.1092	1861	{}	\N	2020-10-12 16:56:22.499941
86660	OCIO	2020-10-07	28.3253	28.3366	28.3253	28.3366	553	{}	\N	2020-10-12 16:56:22.504791
86661	OCIO	2020-10-08	28.4	28.4733	28.4	28.4733	400	{}	\N	2020-10-12 16:56:22.509534
86662	OCIO	2020-10-09	28.575	28.5927	28.575	28.5927	250	{}	\N	2020-10-12 16:56:22.514337
86663	OCIO	2020-10-12	28.85	28.85	28.83	28.83	351	{}	\N	2020-10-12 16:56:22.518928
86664	LEU	2020-09-28	8.55	9.08	8.41	8.83	111831	{}	\N	2020-10-12 16:56:23.489925
86665	LEU	2020-09-29	8.95	9.1	8.5	8.69	69708	{}	\N	2020-10-12 16:56:23.5047
86666	LEU	2020-09-30	8.77	8.77	8.32	8.37	109590	{}	\N	2020-10-12 16:56:23.513178
86667	LEU	2020-10-01	8.49	9.21	8.35	9.2	91235	{}	\N	2020-10-12 16:56:23.519824
86668	LEU	2020-10-02	8.79	9.3	8.415	8.63	122586	{}	\N	2020-10-12 16:56:23.525837
86669	LEU	2020-10-05	8.81	9.5	8.73	9.28	89373	{}	\N	2020-10-12 16:56:23.531419
86670	LEU	2020-10-06	9.28	9.97	9.17	9.37	100467	{}	\N	2020-10-12 16:56:23.536636
86671	LEU	2020-10-07	9.42	10.24	9.38	10.09	138472	{}	\N	2020-10-12 16:56:23.542029
86672	LEU	2020-10-08	10.34	10.67	9.9101	10.11	131987	{}	\N	2020-10-12 16:56:23.547281
86673	LEU	2020-10-09	10.26	10.3163	9.525	9.84	113468	{}	\N	2020-10-12 16:56:23.552463
86674	LEU	2020-10-12	9.94	10.2	9.77	9.9	42072	{}	\N	2020-10-12 16:56:23.557497
86675	LIT	2020-09-28	38.35	38.48	38.04	38.21	354406	{}	\N	2020-10-12 16:56:25.786426
86676	LIT	2020-09-29	38.93	39.4499	38.73	39.26	493437	{}	\N	2020-10-12 16:56:25.797537
86677	LIT	2020-09-30	39.6	40.1876	39.46	40.05	474982	{}	\N	2020-10-12 16:56:25.805946
86678	LIT	2020-10-01	40.49	40.56	40.3	40.54	393287	{}	\N	2020-10-12 16:56:25.812823
86679	LIT	2020-10-02	39.43	40.4899	39.23	39.98	302993	{}	\N	2020-10-12 16:56:25.81899
86680	LIT	2020-10-05	40.41	41.04	40.23	40.98	421640	{}	\N	2020-10-12 16:56:25.824458
86681	LIT	2020-10-06	41.25	41.35	40.65	40.75	487506	{}	\N	2020-10-12 16:56:25.829774
86682	LIT	2020-10-07	41.22	41.635	41.2	41.59	504653	{}	\N	2020-10-12 16:56:25.83501
86683	LIT	2020-10-08	41.87	41.9499	41.7	41.76	399408	{}	\N	2020-10-12 16:56:25.840333
86684	LIT	2020-10-09	42.31	42.75	42.25	42.66	548174	{}	\N	2020-10-12 16:56:25.845362
86685	LIT	2020-10-12	43.11	43.3	42.74	42.86	333124	{}	\N	2020-10-12 16:56:25.850371
86686	LVHI	2020-09-28	22.07	22.1407	22.07	22.1407	531	{}	\N	2020-10-12 16:56:26.885711
86687	LVHI	2020-09-29	22.07	22.07	22.0141	22.0141	541	{}	\N	2020-10-12 16:56:26.898455
86688	LVHI	2020-09-30	21.985	22.01	21.91	21.91	2507	{}	\N	2020-10-12 16:56:26.905989
86689	LVHI	2020-10-01	21.9999	22.059	21.941	22.029	6720	{}	\N	2020-10-12 16:56:26.912161
86690	LVHI	2020-10-02	22.2099	22.2099	22.101	22.101	2304	{}	\N	2020-10-12 16:56:26.918253
86691	LVHI	2020-10-05	22.2	22.3011	22.2	22.3011	4336	{}	\N	2020-10-12 16:56:26.9234
86692	LVHI	2020-10-06	22.44	22.44	22.2048	22.2048	6558	{}	\N	2020-10-12 16:56:26.928329
86693	LVHI	2020-10-07	22.265	22.3277	22.23	22.2311	3098	{}	\N	2020-10-12 16:56:26.933247
86694	LVHI	2020-10-08	22.39	22.3955	22.29	22.35	3906	{}	\N	2020-10-12 16:56:26.93791
86695	LVHI	2020-10-09	22.35	22.4	22.32	22.365	8173	{}	\N	2020-10-12 16:56:26.943235
86696	LVHI	2020-10-12	22.405	22.4599	22.36	22.4599	2985	{}	\N	2020-10-12 16:56:26.948449
86697	MAR	2020-09-28	95.05	97.39	93.94	96	2878402	{}	\N	2020-10-12 16:56:29.797611
86698	MAR	2020-09-29	96.19	96.19	92.81	93.215	2129044	{}	\N	2020-10-12 16:56:29.812341
86699	MAR	2020-09-30	94.41	96.02	92.15	92.58	2721944	{}	\N	2020-10-12 16:56:29.820075
86700	MAR	2020-10-01	93.22	94.95	92.51	94.35	2069871	{}	\N	2020-10-12 16:56:29.826633
86701	MAR	2020-10-02	91.7	95.62	91.44	95.25	2079276	{}	\N	2020-10-12 16:56:29.832615
86702	MAR	2020-10-05	95.78	97.24	95.13	96.78	2125693	{}	\N	2020-10-12 16:56:29.837057
86703	MAR	2020-10-06	98.15	100.33	96.52	96.92	2823747	{}	\N	2020-10-12 16:56:29.84122
86704	MAR	2020-10-07	98.24	99.93	98.01	99.01	1822603	{}	\N	2020-10-12 16:56:29.845197
86705	MAR	2020-10-08	99.81	100.44	98.205	100.31	1772869	{}	\N	2020-10-12 16:56:29.849695
86706	MAR	2020-10-09	101.29	102.09	99.26	99.35	1446415	{}	\N	2020-10-12 16:56:29.854277
86707	MAR	2020-10-12	99.46	100.49	98.84	100.25	1056080	{}	\N	2020-10-12 16:56:29.859121
86708	MC	2020-09-28	33.43	34.45	33.43	34.24	266970	{}	\N	2020-10-12 16:56:32.644584
86709	MC	2020-09-29	34.17	34.35	33.6	34.19	599076	{}	\N	2020-10-12 16:56:32.655442
86710	MC	2020-09-30	34.28	35.415	34.28	35.14	843060	{}	\N	2020-10-12 16:56:32.662321
86711	MC	2020-10-01	35.37	35.5	34.65	35.13	572116	{}	\N	2020-10-12 16:56:32.668172
86712	MC	2020-10-02	35.5	36.635	35.13	36.03	1274298	{}	\N	2020-10-12 16:56:32.67363
86713	MC	2020-10-05	36.38	36.99	35.76	36.67	870821	{}	\N	2020-10-12 16:56:32.678305
86714	MC	2020-10-06	37	37.54	36.5	36.69	1101099	{}	\N	2020-10-12 16:56:32.682638
86715	MC	2020-10-07	37.25	37.67	37	37.32	1185982	{}	\N	2020-10-12 16:56:32.687285
86716	MC	2020-10-08	37.71	38.16	37.51	38.1	1142976	{}	\N	2020-10-12 16:56:32.692423
86717	MC	2020-10-09	37.48	38.91	37.44	38.36	1072451	{}	\N	2020-10-12 16:56:32.697175
86718	MC	2020-10-12	38.82	39.19	38.34	38.53	370162	{}	\N	2020-10-12 16:56:32.702218
86719	MGC	2020-09-28	119.22	119.765	118.95	119.49	74611	{}	\N	2020-10-12 16:56:33.728431
86720	MGC	2020-09-29	119.51	119.6867	118.74	118.99	50813	{}	\N	2020-10-12 16:56:33.737542
86721	MGC	2020-09-30	119.2	121.078	119.2	119.9	47260	{}	\N	2020-10-12 16:56:33.744375
86722	MGC	2020-10-01	121.02	121.1996	120.11	120.82	92794	{}	\N	2020-10-12 16:56:33.749704
86723	MGC	2020-10-02	118.66	120.06	118.66	119.23	43476	{}	\N	2020-10-12 16:56:33.755104
86724	MGC	2020-10-05	120.17	121.41	120.13	121.41	114310	{}	\N	2020-10-12 16:56:33.75957
86725	MGC	2020-10-06	121.4	122.05	119.42	119.61	56943	{}	\N	2020-10-12 16:56:33.764428
86726	MGC	2020-10-07	121.0088	121.909	120.81	121.725	43106	{}	\N	2020-10-12 16:56:33.768517
86727	MGC	2020-10-08	122.55	122.6	122.01	122.6	56881	{}	\N	2020-10-12 16:56:33.772692
86728	MGC	2020-10-09	123.17	123.8262	123.045	123.82	77594	{}	\N	2020-10-12 16:56:33.776664
86729	MGC	2020-10-12	124.78	126.246	124.735	126.21	31865	{}	\N	2020-10-12 16:56:33.780752
86730	MXIM	2020-09-28	66.52	68.1	66.28	68.02	2302796	{}	\N	2020-10-12 16:56:35.522092
86731	MXIM	2020-09-29	67.73	68.67	67.2702	67.84	1761344	{}	\N	2020-10-12 16:56:35.533329
86732	MXIM	2020-09-30	68.02	68.84	67.38	67.61	2704950	{}	\N	2020-10-12 16:56:35.54101
86733	MXIM	2020-10-01	68.42	69.51	68.35	69.28	2768146	{}	\N	2020-10-12 16:56:35.547702
86734	MXIM	2020-10-02	68.32	68.7	66.3	66.38	2338524	{}	\N	2020-10-12 16:56:35.553762
86735	MXIM	2020-10-05	66.6	68.33	66.54	68.25	4614671	{}	\N	2020-10-12 16:56:35.559418
86736	MXIM	2020-10-06	67.84	69.72	67.48	67.61	2169526	{}	\N	2020-10-12 16:56:35.565206
86737	MXIM	2020-10-07	68.19	70.21	68.18	70.07	4442459	{}	\N	2020-10-12 16:56:35.570734
86738	MXIM	2020-10-08	70.3	71.7	70.3	71.44	5155900	{}	\N	2020-10-12 16:56:35.576664
86739	MXIM	2020-10-09	71.94	73.87	71.94	73.01	5608776	{}	\N	2020-10-12 16:56:35.582094
86740	MXIM	2020-10-12	73.77	74.68	73.67	74.395	797019	{}	\N	2020-10-12 16:56:35.587472
86741	NKE	2020-09-28	125.16	126.19	123.885	124.32	7811724	{}	\N	2020-10-12 16:56:37.105476
86742	NKE	2020-09-29	124.34	127.1	123.96	126.35	15727466	{}	\N	2020-10-12 16:56:37.115301
86743	NKE	2020-09-30	126.1	127.28	125.1647	125.54	17345616	{}	\N	2020-10-12 16:56:37.122616
86744	NKE	2020-10-01	127.73	127.73	125.31	126.64	10981106	{}	\N	2020-10-12 16:56:37.129066
86745	NKE	2020-10-02	124.35	126.8	123.59	126.64	14963465	{}	\N	2020-10-12 16:56:37.134742
86746	NKE	2020-10-05	127.07	128.5	126.45	127.91	16333895	{}	\N	2020-10-12 16:56:37.139778
86747	NKE	2020-10-06	128.84	130.44	127.2654	127.65	11958101	{}	\N	2020-10-12 16:56:37.144702
86748	NKE	2020-10-07	128.4	130.41	128.25	130.06	8841352	{}	\N	2020-10-12 16:56:37.149325
86749	NKE	2020-10-08	130.4	130.9289	129.51	129.71	7280485	{}	\N	2020-10-12 16:56:37.154036
86750	NKE	2020-10-09	129.99	131.19	129.3254	130.98	8118985	{}	\N	2020-10-12 16:56:37.158473
86751	NKE	2020-10-12	131.19	131.34	129.511	129.83	5028732	{}	\N	2020-10-12 16:56:37.163158
86752	NMY	2020-09-28	13.15	13.15	13.11	13.12	36062	{}	\N	2020-10-12 16:56:39.731305
86753	NMY	2020-09-29	13.12	13.1912	13.1099	13.18	169112	{}	\N	2020-10-12 16:56:39.74163
86754	NMY	2020-09-30	13.2033	13.2033	13.11	13.16	49764	{}	\N	2020-10-12 16:56:39.748385
86755	NMY	2020-10-01	13.25	13.25	13.15	13.23	60617	{}	\N	2020-10-12 16:56:39.754299
86756	NMY	2020-10-02	13.23	13.24	13.21	13.21	28427	{}	\N	2020-10-12 16:56:39.760161
86757	NMY	2020-10-05	13.22	13.23	13.15	13.22	93042	{}	\N	2020-10-12 16:56:39.765773
86758	NMY	2020-10-06	13.25	13.25	13.15	13.18	61886	{}	\N	2020-10-12 16:56:39.771061
86759	NMY	2020-10-07	13.18	13.21	13.16	13.21	25224	{}	\N	2020-10-12 16:56:39.775778
86760	NMY	2020-10-08	13.19	13.19	13.15	13.15	61740	{}	\N	2020-10-12 16:56:39.780609
86761	NMY	2020-10-09	13.2028	13.25	13.2028	13.2301	14562	{}	\N	2020-10-12 16:56:39.784778
86762	NMY	2020-10-12	13.25	13.25	13.2	13.2325	90096	{}	\N	2020-10-12 16:56:39.788886
86763	OCC	2020-09-28	2.6	2.9	2.6	2.8572	108414	{}	\N	2020-10-12 16:56:40.847814
86764	OCC	2020-09-29	2.95	3.07	2.8328	2.9425	31486	{}	\N	2020-10-12 16:56:40.856498
86765	OCC	2020-09-30	3.04	3.2	2.96	3.09	26797	{}	\N	2020-10-12 16:56:40.863782
86766	OCC	2020-10-01	3.19	3.48	3.19	3.4597	50045	{}	\N	2020-10-12 16:56:40.869749
86767	OCC	2020-10-02	3.48	3.99	3.25	3.77	137005	{}	\N	2020-10-12 16:56:40.875273
86768	OCC	2020-10-05	3.81	4.62	3.75	3.88	136416	{}	\N	2020-10-12 16:56:40.881035
86769	OCC	2020-10-06	3.88	3.88	3.3408	3.6	62787	{}	\N	2020-10-12 16:56:40.885871
86770	OCC	2020-10-07	3.6	3.77	3.46	3.56	20730	{}	\N	2020-10-12 16:56:40.890962
86771	OCC	2020-10-08	3.5101	3.616	3.215	3.215	20025	{}	\N	2020-10-12 16:56:40.896217
86772	OCC	2020-10-09	3.13	3.21	2.85	2.9173	63300	{}	\N	2020-10-12 16:56:40.901164
86773	OCC	2020-10-12	2.88	3.1099	2.88	2.9467	20677	{}	\N	2020-10-12 16:56:40.907021
86774	ODC	2020-09-28	37.2	37.498	36.85	36.92	9041	{}	\N	2020-10-12 16:56:42.105466
86775	ODC	2020-09-29	36.79	37.075	36.06	36.89	17058	{}	\N	2020-10-12 16:56:42.116384
86776	ODC	2020-09-30	36.61	36.9	35.77	35.77	14169	{}	\N	2020-10-12 16:56:42.124613
86777	ODC	2020-10-01	36.52	36.86	36.52	36.63	14397	{}	\N	2020-10-12 16:56:42.130818
86778	ODC	2020-10-02	36.22	36.41	36.2	36.36	9055	{}	\N	2020-10-12 16:56:42.136946
86779	ODC	2020-10-05	36.96	37.28	35.8203	37.19	18640	{}	\N	2020-10-12 16:56:42.142479
86780	ODC	2020-10-06	37.25	37.84	36.28	36.3	16418	{}	\N	2020-10-12 16:56:42.147792
86781	ODC	2020-10-07	36.59	37.54	36.59	37.25	16268	{}	\N	2020-10-12 16:56:42.153159
86782	ODC	2020-10-08	37.25	37.4	36.73	36.73	17726	{}	\N	2020-10-12 16:56:42.158232
86783	ODC	2020-10-09	36.9	37.17	36.37	37	12251	{}	\N	2020-10-12 16:56:42.16349
86784	ODC	2020-10-12	37	37.35	36.84	37.17	6566	{}	\N	2020-10-12 16:56:42.169187
86785	OSB	2020-09-28	29.15	29.525	28.9	29.19	347846	{}	\N	2020-10-12 16:56:44.262734
86786	OSB	2020-09-29	29.27	29.99	29.11	29.76	357414	{}	\N	2020-10-12 16:56:44.272831
86787	OSB	2020-09-30	29.72	29.79	29.22	29.5	524222	{}	\N	2020-10-12 16:56:44.280457
86788	OSB	2020-10-01	29.74	30.58	29.74	30.32	359216	{}	\N	2020-10-12 16:56:44.286409
86789	OSB	2020-10-02	29.74	30.46	29.46	30.22	374380	{}	\N	2020-10-12 16:56:44.291923
86790	OSB	2020-10-05	30.26	31.1	30.26	30.92	434811	{}	\N	2020-10-12 16:56:44.29699
86791	OSB	2020-10-06	31.11	31.12	30.27	30.31	371105	{}	\N	2020-10-12 16:56:44.301605
86792	OSB	2020-10-07	30.69	31.72	30.69	31.6	562965	{}	\N	2020-10-12 16:56:44.306319
86793	OSB	2020-10-08	31.81	32.09	30.82	31.64	532982	{}	\N	2020-10-12 16:56:44.311463
86794	OSB	2020-10-09	32.06	32.22	31.25	31.33	515520	{}	\N	2020-10-12 16:56:44.316326
86795	OSB	2020-10-12	31.48	31.98	31.4501	31.6	65659	{}	\N	2020-10-12 16:56:44.320917
86796	MAGA	2020-09-28	25.53	25.6524	25.53	25.6524	584	{}	\N	2020-10-12 16:56:48.200495
86797	MAGA	2020-09-29	25.5537	25.5537	25.25	25.25	1419	{}	\N	2020-10-12 16:56:48.211047
86798	MAGA	2020-09-30	25.62	25.62	25.4907	25.4907	465	{}	\N	2020-10-12 16:56:48.218926
86799	MAGA	2020-10-01	25.4	25.4	25.3131	25.3131	303	{}	\N	2020-10-12 16:56:48.225564
86800	MAGA	2020-10-02	25.6984	25.6984	25.6984	25.6984	117	{}	\N	2020-10-12 16:56:48.231397
86801	MAGA	2020-10-05	26.06	26.1533	26.06	26.1533	9640	{}	\N	2020-10-12 16:56:48.237676
86802	MAGA	2020-10-06	26.3659	26.49	25.9469	25.9469	656	{}	\N	2020-10-12 16:56:48.243252
86803	MAGA	2020-10-07	26.19	26.41	26.19	26.29	4202	{}	\N	2020-10-12 16:56:48.248047
86804	MAGA	2020-10-08	26.65	26.8	26.65	26.8	671	{}	\N	2020-10-12 16:56:48.253015
86805	MAGA	2020-10-09	26.7625	26.7625	26.7625	26.7625	196	{}	\N	2020-10-12 16:56:48.257703
86806	MAGA	2020-10-12	26.92	26.92	26.92	26.92	202	{}	\N	2020-10-12 16:56:48.26234
86807	MDIV	2020-09-28	13.37	13.5687	13.37	13.49	77017	{}	\N	2020-10-12 16:56:49.131528
86808	MDIV	2020-09-29	13.53	13.55	13.371	13.54	94400	{}	\N	2020-10-12 16:56:49.142161
86809	MDIV	2020-09-30	13.58	13.65	13.446	13.45	74115	{}	\N	2020-10-12 16:56:49.149032
86810	MDIV	2020-10-01	13.46	13.6199	13.41	13.6	78545	{}	\N	2020-10-12 16:56:49.15536
86811	MDIV	2020-10-02	13.41	13.7343	13.4	13.64	117252	{}	\N	2020-10-12 16:56:49.160827
86812	MDIV	2020-10-05	13.75	13.85	13.68	13.85	72881	{}	\N	2020-10-12 16:56:49.165635
86813	MDIV	2020-10-06	13.89	13.99	13.73	13.81	63251	{}	\N	2020-10-12 16:56:49.169735
86814	MDIV	2020-10-07	13.85	13.8999	13.7601	13.8	86416	{}	\N	2020-10-12 16:56:49.174472
86815	MDIV	2020-10-08	13.8774	14.0899	13.8774	14.02	159951	{}	\N	2020-10-12 16:56:49.178655
86816	MDIV	2020-10-09	14.0679	14.0999	13.97	14.03	88698	{}	\N	2020-10-12 16:56:49.183004
86817	MDIV	2020-10-12	14.09	14.09	14.01	14.0501	33922	{}	\N	2020-10-12 16:56:49.188181
86818	MEIP	2020-09-28	3.205	3.205	3.06	3.09	400299	{}	\N	2020-10-12 16:56:50.142918
86819	MEIP	2020-09-29	3.08	3.1501	3	3.15	486780	{}	\N	2020-10-12 16:56:50.123681
86820	MEIP	2020-09-30	3.16	3.19	3.08	3.12	395521	{}	\N	2020-10-12 16:56:50.131851
86821	MEIP	2020-10-01	3.14	3.14	2.95	2.96	875680	{}	\N	2020-10-12 16:56:50.138802
86822	MEIP	2020-10-02	2.92	3.11	2.88	3.05	1011965	{}	\N	2020-10-12 16:56:50.144769
86823	MEIP	2020-10-05	3.05	3.285	3.0317	3.22	994223	{}	\N	2020-10-12 16:56:50.150089
86824	MEIP	2020-10-06	3.18	3.34	3.13	3.25	905002	{}	\N	2020-10-12 16:56:50.155655
86825	MEIP	2020-10-07	3.26	3.345	3.24	3.32	642288	{}	\N	2020-10-12 16:56:50.160545
86826	MEIP	2020-10-08	3.36	3.37	3.13	3.14	647767	{}	\N	2020-10-12 16:56:50.165357
86827	MEIP	2020-10-09	3.16	3.18	3.06	3.08	318433	{}	\N	2020-10-12 16:56:50.170348
86828	MEIP	2020-10-12	3.2	3.28	3.09	3.15	304076	{}	\N	2020-10-12 16:56:50.17519
86829	MFDX	2020-09-28	24.05	24.1003	24.03	24.087	1341	{}	\N	2020-10-12 16:56:52.227395
86830	MFDX	2020-09-29	24.17	24.17	24.0984	24.0984	122263	{}	\N	2020-10-12 16:56:52.237878
86831	MFDX	2020-09-30	24.05	24.13	23.8215	24.05	80346	{}	\N	2020-10-12 16:56:52.245638
86832	MFDX	2020-10-01	24.17	24.19	24.05	24.19	13939	{}	\N	2020-10-12 16:56:52.251932
86833	MFDX	2020-10-02	23.96	24.14	23.96	24.13	12015	{}	\N	2020-10-12 16:56:52.258887
86834	MFDX	2020-10-05	24.345	24.4775	24.345	24.4775	3677	{}	\N	2020-10-12 16:56:52.264259
86835	MFDX	2020-10-06	24.47	24.48	24.24	24.25	1635	{}	\N	2020-10-12 16:56:52.269622
86836	MFDX	2020-10-07	24.46	24.51	24.42	24.45	29618	{}	\N	2020-10-12 16:56:52.274727
86837	MFDX	2020-10-08	24.56	24.591	24.54	24.591	5052	{}	\N	2020-10-12 16:56:52.280629
86838	MFDX	2020-10-09	24.73	24.7626	24.69	24.7626	4484	{}	\N	2020-10-12 16:56:52.285898
86839	MFDX	2020-10-12	24.83	24.86	24.81	24.85	2999	{}	\N	2020-10-12 16:56:52.291888
86840	MOH	2020-09-28	163.39	165.49	161.15	163.07	387387	{}	\N	2020-10-12 16:56:53.578481
86841	MOH	2020-09-29	159	172.6	159	171.8	1062143	{}	\N	2020-10-12 16:56:53.59159
86842	MOH	2020-09-30	173.13	184.2	171.4	183.04	1361554	{}	\N	2020-10-12 16:56:53.599747
86843	MOH	2020-10-01	182.53	188.38	181.22	188.18	1786753	{}	\N	2020-10-12 16:56:53.606738
86844	MOH	2020-10-02	185.96	194.88	185.6804	193.36	1169738	{}	\N	2020-10-12 16:56:53.612493
86845	MOH	2020-10-05	195	201.87	195	201.32	1008025	{}	\N	2020-10-12 16:56:53.617846
86846	MOH	2020-10-06	202.04	202.4	196.38	198.18	786145	{}	\N	2020-10-12 16:56:53.622487
86847	MOH	2020-10-07	199.69	203.9	198.83	203.53	948469	{}	\N	2020-10-12 16:56:53.627641
86848	MOH	2020-10-08	205.5	206.1	199.1	199.75	456092	{}	\N	2020-10-12 16:56:53.633508
86849	MOH	2020-10-09	201.17	201.92	197.905	199.2	531454	{}	\N	2020-10-12 16:56:53.638307
86850	MOH	2020-10-12	200.9	207.65	200.005	203.85	245834	{}	\N	2020-10-12 16:56:53.643417
86851	MXI	2020-09-28	69.86	69.98	69.54	69.71	116181	{}	\N	2020-10-12 16:56:55.421087
86852	MXI	2020-09-29	69.82	70.0774	69.51	69.71	69404	{}	\N	2020-10-12 16:56:55.43241
86853	MXI	2020-09-30	69.69	70.1999	69.515	69.86	27819	{}	\N	2020-10-12 16:56:55.439801
86854	MXI	2020-10-01	70.13	70.13	69.41	69.56	43270	{}	\N	2020-10-12 16:56:55.446043
86855	MXI	2020-10-02	68.6	70.109	68.6	69.88	28512	{}	\N	2020-10-12 16:56:55.451481
86856	MXI	2020-10-05	70.33	71.06	70.33	71	23614	{}	\N	2020-10-12 16:56:55.457001
86857	MXI	2020-10-06	71.0512	71.0512	69.67	69.67	34075	{}	\N	2020-10-12 16:56:55.461291
86858	MXI	2020-10-07	70.52	71.42	70.52	71.31	32226	{}	\N	2020-10-12 16:56:55.465614
86859	MXI	2020-10-08	71.67	71.88	71.605	71.81	17413	{}	\N	2020-10-12 16:56:55.470049
86860	MXI	2020-10-09	72.47	72.75	72.29	72.53	41666	{}	\N	2020-10-12 16:56:55.474402
86861	MXI	2020-10-12	72.74	72.81	72.42	72.81	26593	{}	\N	2020-10-12 16:56:55.47867
86862	MYGN	2020-09-28	12.605	12.8599	12.51	12.66	397959	{}	\N	2020-10-12 16:56:56.778923
86863	MYGN	2020-09-29	12.67	12.78	12.38	12.74	419802	{}	\N	2020-10-12 16:56:56.789428
86864	MYGN	2020-09-30	12.805	13.23	12.805	13.04	476761	{}	\N	2020-10-12 16:56:56.796623
86865	MYGN	2020-10-01	13.14	13.3	12.95	13.17	417613	{}	\N	2020-10-12 16:56:56.802196
86866	MYGN	2020-10-02	12.9	13.24	12.845	12.99	400676	{}	\N	2020-10-12 16:56:56.807547
86867	MYGN	2020-10-05	13.06	13.5029	12.92	13.32	552383	{}	\N	2020-10-12 16:56:56.813011
86868	MYGN	2020-10-06	13.5	13.84	13.28	13.29	611248	{}	\N	2020-10-12 16:56:56.817698
86869	MYGN	2020-10-07	13.37	13.5	12.94	13.07	629345	{}	\N	2020-10-12 16:56:56.822835
86870	MYGN	2020-10-08	13.14	13.45	13.14	13.43	412962	{}	\N	2020-10-12 16:56:56.828377
86871	MYGN	2020-10-09	13.6	13.72	13.3	13.615	482068	{}	\N	2020-10-12 16:56:56.833496
86872	MYGN	2020-10-12	13.72	13.89	13.35	13.76	106362	{}	\N	2020-10-12 16:56:56.838313
86873	MYOS	2020-09-28	1.37	1.42	1.37	1.42	24080	{}	\N	2020-10-12 16:56:57.856631
86874	MYOS	2020-09-29	1.42	1.48	1.375	1.45	81311	{}	\N	2020-10-12 16:56:57.866052
86875	MYOS	2020-09-30	1.44	1.48	1.36	1.37	127185	{}	\N	2020-10-12 16:56:57.874933
86876	MYOS	2020-10-01	1.37	1.4399	1.3662	1.4	70383	{}	\N	2020-10-12 16:56:57.882988
86877	MYOS	2020-10-02	1.37	1.44	1.36	1.43	30813	{}	\N	2020-10-12 16:56:57.889882
86878	MYOS	2020-10-05	1.4	1.47	1.4	1.4	73749	{}	\N	2020-10-12 16:56:57.895809
86879	MYOS	2020-10-06	1.41	1.4108	1.38	1.39	33187	{}	\N	2020-10-12 16:56:57.901343
86880	MYOS	2020-10-07	1.38	1.385	1.3	1.35	124075	{}	\N	2020-10-12 16:56:57.907834
86881	MYOS	2020-10-08	1.35	1.399	1.35	1.38	50904	{}	\N	2020-10-12 16:56:57.913171
86882	MYOS	2020-10-09	1.3823	1.3823	1.34	1.35	74746	{}	\N	2020-10-12 16:56:57.918388
86883	MYOS	2020-10-12	1.32	1.38	1.32	1.369	22478	{}	\N	2020-10-12 16:56:57.923586
86884	NKG	2020-09-28	12.25	12.31	12.215	12.22	37410	{}	\N	2020-10-12 16:56:59.65258
86885	NKG	2020-09-29	12.2	12.23	12.1403	12.1794	82878	{}	\N	2020-10-12 16:56:59.665818
86886	NKG	2020-09-30	12.19	12.2	12.115	12.2	88180	{}	\N	2020-10-12 16:56:59.673501
86887	NKG	2020-10-01	12.2	12.2	12.165	12.181	27471	{}	\N	2020-10-12 16:56:59.679929
86888	NKG	2020-10-02	12.19	12.25	12.1	12.23	136140	{}	\N	2020-10-12 16:56:59.686068
86889	NKG	2020-10-05	12.21	12.24	12.175	12.22	74545	{}	\N	2020-10-12 16:56:59.691338
86890	NKG	2020-10-06	12.23	12.28	12.229	12.23	48748	{}	\N	2020-10-12 16:56:59.695703
86891	NKG	2020-10-07	12.24	12.447	12.15	12.2633	43450	{}	\N	2020-10-12 16:56:59.700064
86892	NKG	2020-10-08	12.3	12.3	12.2641	12.2641	8168	{}	\N	2020-10-12 16:56:59.704397
86893	NKG	2020-10-09	12.29	12.325	12.27	12.28	14828	{}	\N	2020-10-12 16:56:59.708539
86894	NKG	2020-10-12	12.27	12.305	12.27	12.3	17540	{}	\N	2020-10-12 16:56:59.713226
86895	ODFL	2020-09-28	184.6	187.79	184.46	187.35	760150	{}	\N	2020-10-12 16:57:01.374035
86896	ODFL	2020-09-29	186.21	188.29	184.9	185.11	1287042	{}	\N	2020-10-12 16:57:01.384611
86897	ODFL	2020-09-30	186	187	179.82	180.75	759528	{}	\N	2020-10-12 16:57:01.391727
86898	ODFL	2020-10-01	182.45	184.91	180.5	182.77	901810	{}	\N	2020-10-12 16:57:01.397753
86899	ODFL	2020-10-02	180.69	187.16	180.69	184.56	565383	{}	\N	2020-10-12 16:57:01.404068
86900	ODFL	2020-10-05	185.14	189.57	185.14	188.46	701295	{}	\N	2020-10-12 16:57:01.40888
86901	ODFL	2020-10-06	188.88	192.18	186.06	189.11	719411	{}	\N	2020-10-12 16:57:01.41378
86902	ODFL	2020-10-07	190.921	194.14	190.025	192.93	742079	{}	\N	2020-10-12 16:57:01.418999
86903	ODFL	2020-10-08	194.03	197.28	193	197.11	614678	{}	\N	2020-10-12 16:57:01.424397
86904	ODFL	2020-10-09	198.03	200.51	196.89	199.76	846320	{}	\N	2020-10-12 16:57:01.429441
86905	ODFL	2020-10-12	201.06	203.5	200.575	201.68	277728	{}	\N	2020-10-12 16:57:01.434596
86906	MFT	2020-09-28	13.33	13.43	13.33	13.4	7227	{}	\N	2020-10-12 16:57:04.148602
86907	MFT	2020-09-29	13.43	13.4495	13.41	13.41	13450	{}	\N	2020-10-12 16:57:04.158719
86908	MFT	2020-09-30	13.38	13.72	13.3775	13.7	54932	{}	\N	2020-10-12 16:57:04.165055
86909	MFT	2020-10-01	13.77	13.77	13.6704	13.77	18382	{}	\N	2020-10-12 16:57:04.171226
86910	MFT	2020-10-02	13.75	13.7899	13.6044	13.76	5942	{}	\N	2020-10-12 16:57:04.177145
86911	MFT	2020-10-05	13.71	13.8	13.7	13.8	20230	{}	\N	2020-10-12 16:57:04.181917
86912	MFT	2020-10-06	13.7	13.79	13.68	13.79	11393	{}	\N	2020-10-12 16:57:04.187369
86913	MFT	2020-10-07	13.73	13.77	13.73	13.77	8800	{}	\N	2020-10-12 16:57:04.19243
86914	MFT	2020-10-08	13.79	14.27	13.7306	13.81	25638	{}	\N	2020-10-12 16:57:04.197561
86915	MFT	2020-10-09	13.74	13.87	13.66	13.78	19638	{}	\N	2020-10-12 16:57:04.202452
86916	MFT	2020-10-12	13.79	14.1852	13.79	14.1852	464	{}	\N	2020-10-12 16:57:04.207206
86917	MIDD	2020-09-28	92.07	93.26	90.75	90.9	771943	{}	\N	2020-10-12 16:57:06.533887
86918	MIDD	2020-09-29	90.41	90.75	87.75	88.72	459308	{}	\N	2020-10-12 16:57:06.547227
86919	MIDD	2020-09-30	88.48	91.45	88.48	89.71	514973	{}	\N	2020-10-12 16:57:06.554897
86920	MIDD	2020-10-01	90.19	91.18	89.285	90.27	517809	{}	\N	2020-10-12 16:57:06.560947
86921	MIDD	2020-10-02	88.4	94.745	88.4	93.72	729778	{}	\N	2020-10-12 16:57:06.566478
86922	MIDD	2020-10-05	94.93	96.65	94.72	96.17	344372	{}	\N	2020-10-12 16:57:06.57214
86923	MIDD	2020-10-06	96.44	98.725	95.5	96.33	1297359	{}	\N	2020-10-12 16:57:06.578026
86924	MIDD	2020-10-07	98	99.97	97.81	99.235	574024	{}	\N	2020-10-12 16:57:06.583003
86925	MIDD	2020-10-08	100	100.76	98.26	100.7	492233	{}	\N	2020-10-12 16:57:06.588236
86926	MIDD	2020-10-09	102.1	103.71	101.52	102.31	511717	{}	\N	2020-10-12 16:57:06.593235
86927	MIDD	2020-10-12	102.06	103.27	101.52	102.81	138420	{}	\N	2020-10-12 16:57:06.598361
86928	NEO	2020-09-28	36.25	36.63	35.84	36.16	581975	{}	\N	2020-10-12 16:57:08.948428
86929	NEO	2020-09-29	36.2	37.17	35.93	36.87	707403	{}	\N	2020-10-12 16:57:08.95933
86930	NEO	2020-09-30	37.08	38.25	36.75	36.89	1374825	{}	\N	2020-10-12 16:57:08.966288
86931	NEO	2020-10-01	36.85	37.25	35.74	36.835	568611	{}	\N	2020-10-12 16:57:08.97224
86932	NEO	2020-10-02	35.9	36.8	35.8	36.08	443947	{}	\N	2020-10-12 16:57:08.977607
86933	NEO	2020-10-05	36.35	37.31	36.35	37.23	352310	{}	\N	2020-10-12 16:57:08.982488
86934	NEO	2020-10-06	37.42	39.23	37.42	38.56	705938	{}	\N	2020-10-12 16:57:08.986974
86935	NEO	2020-10-07	39.17	39.45	38.35	39.01	638554	{}	\N	2020-10-12 16:57:08.9914
86936	NEO	2020-10-08	39.54	39.73	38.89	39.19	385080	{}	\N	2020-10-12 16:57:08.995796
86937	NEO	2020-10-09	39.84	40.47	39.2401	40.42	291411	{}	\N	2020-10-12 16:57:09.000351
86938	NEO	2020-10-12	40.85	40.85	39.91	39.91	279540	{}	\N	2020-10-12 16:57:09.004758
86939	NGHCN	2020-09-28	26.11	26.1861	26.105	26.1861	16086	{}	\N	2020-10-12 16:57:10.203168
86940	NGHCN	2020-09-29	26.19	26.2	26.15	26.18	7668	{}	\N	2020-10-12 16:57:10.214437
86941	NGHCN	2020-09-30	25.77	25.82	25.53	25.53	34825	{}	\N	2020-10-12 16:57:10.221536
86942	NGHCN	2020-10-01	25.79	25.95	25.69	25.95	33304	{}	\N	2020-10-12 16:57:10.2271
86943	NGHCN	2020-10-02	25.93	25.97	25.93	25.95	21468	{}	\N	2020-10-12 16:57:10.233658
86944	NGHCN	2020-10-05	25.855	26.03	25.855	26.005	19281	{}	\N	2020-10-12 16:57:10.239758
86945	NGHCN	2020-10-06	25.85	26.07	25.85	26.07	11141	{}	\N	2020-10-12 16:57:10.244954
86946	NGHCN	2020-10-07	26.02	26.07	25.89	26.05	5596	{}	\N	2020-10-12 16:57:10.250539
86947	NGHCN	2020-10-08	26	26.08	26	26	7208	{}	\N	2020-10-12 16:57:10.255774
86948	NGHCN	2020-10-09	25.9954	25.9954	25.79	25.9	6430	{}	\N	2020-10-12 16:57:10.260902
86949	NGHCN	2020-10-12	25.9	25.92	25.9	25.91	1195	{}	\N	2020-10-12 16:57:10.266054
86950	NIM	2020-09-28	10.46	10.49	10.45	10.49	6707	{}	\N	2020-10-12 16:57:12.507712
86951	NIM	2020-09-29	10.49	10.49	10.4307	10.48	19260	{}	\N	2020-10-12 16:57:12.519655
86952	NIM	2020-09-30	10.49	10.49	10.422	10.49	31932	{}	\N	2020-10-12 16:57:12.527789
86953	NIM	2020-10-01	10.6	10.6	10.4942	10.54	18200	{}	\N	2020-10-12 16:57:12.533843
86954	NIM	2020-10-02	10.56	10.62	10.56	10.6	38414	{}	\N	2020-10-12 16:57:12.539685
86955	NIM	2020-10-05	10.65	10.65	10.51	10.59	28335	{}	\N	2020-10-12 16:57:12.544695
86956	NIM	2020-10-06	10.62	10.62	10.42	10.42	14147	{}	\N	2020-10-12 16:57:12.549515
86957	NIM	2020-10-07	10.64	10.64	10.34	10.35	27488	{}	\N	2020-10-12 16:57:12.554566
86958	NIM	2020-10-08	10.5	10.5	10.36	10.37	20576	{}	\N	2020-10-12 16:57:12.55926
86959	NIM	2020-10-09	10.49	10.49	10.42	10.42	11232	{}	\N	2020-10-12 16:57:12.564636
86960	NIM	2020-10-12	10.5	10.5	10.42	10.42	7598	{}	\N	2020-10-12 16:57:12.570256
86961	NOA	2020-09-28	6.59	6.87	6.59	6.8	49294	{}	\N	2020-10-12 16:57:14.361154
86962	NOA	2020-09-29	6.82	6.82	6.45	6.6	126708	{}	\N	2020-10-12 16:57:14.374584
86963	NOA	2020-09-30	6.61	6.715	6.47	6.51	129522	{}	\N	2020-10-12 16:57:14.382179
86964	NOA	2020-10-01	6.45	6.57	6.1	6.23	354512	{}	\N	2020-10-12 16:57:14.388472
86965	NOA	2020-10-02	6.03	6.34	6	6.29	152368	{}	\N	2020-10-12 16:57:14.39398
86966	NOA	2020-10-05	6.37	6.48	6.37	6.42	79122	{}	\N	2020-10-12 16:57:14.399415
86967	NOA	2020-10-06	6.44	6.44	6.2951	6.3	117615	{}	\N	2020-10-12 16:57:14.404213
86968	NOA	2020-10-07	6.32	6.46	6.3	6.39	53306	{}	\N	2020-10-12 16:57:14.408887
86969	NOA	2020-10-08	6.49	6.78	6.455	6.74	136312	{}	\N	2020-10-12 16:57:14.413672
86970	NOA	2020-10-09	6.82	6.8499	6.58	6.61	113445	{}	\N	2020-10-12 16:57:14.418377
86971	NOA	2020-10-12	6.61	6.61	6.39	6.4459	47096	{}	\N	2020-10-12 16:57:14.422885
86972	NWFL	2020-09-28	23.84	24.7299	23.84	24.45	5176	{}	\N	2020-10-12 16:57:15.453378
86973	NWFL	2020-09-29	24.45	24.74	23.31	24.11	3349	{}	\N	2020-10-12 16:57:15.465077
86974	NWFL	2020-09-30	24.545	24.545	24.3003	24.32	2389	{}	\N	2020-10-12 16:57:15.472852
86975	NWFL	2020-10-01	24.1	24.87	24.1	24.57	4191	{}	\N	2020-10-12 16:57:15.479192
86976	NWFL	2020-10-02	24.29	25.35	24.29	25.35	2775	{}	\N	2020-10-12 16:57:15.485408
86977	NWFL	2020-10-05	24.61	26.25	24.61	25.2	6863	{}	\N	2020-10-12 16:57:15.490536
86978	NWFL	2020-10-06	25.7	25.77	24.66	24.66	5921	{}	\N	2020-10-12 16:57:15.495364
86979	NWFL	2020-10-07	24.7567	25.85	24.7567	25.85	3557	{}	\N	2020-10-12 16:57:15.499853
86980	NWFL	2020-10-08	25.82	26	25.01	25.03	12231	{}	\N	2020-10-12 16:57:15.504332
86981	NWFL	2020-10-09	25.14	25.72	25.14	25.58	2790	{}	\N	2020-10-12 16:57:15.508917
86982	NWFL	2020-10-12	25.32	25.32	25.32	25.32	452	{}	\N	2020-10-12 16:57:15.513492
86983	NYMTP	2020-09-28	20.25	20.52	20.1131	20.5	21965	{}	\N	2020-10-12 16:57:16.536975
86984	NYMTP	2020-09-29	20.33	20.76	20.25	20.59	8491	{}	\N	2020-10-12 16:57:16.548245
86985	NYMTP	2020-09-30	20.65	20.75	20.11	20.23	28724	{}	\N	2020-10-12 16:57:16.555603
86986	NYMTP	2020-10-01	20.29	20.35	20.07	20.35	5861	{}	\N	2020-10-12 16:57:16.56222
86987	NYMTP	2020-10-02	20.32	20.32	20.02	20.3	4817	{}	\N	2020-10-12 16:57:16.568077
86988	NYMTP	2020-10-05	20.24	20.53	20.12	20.46	8339	{}	\N	2020-10-12 16:57:16.572759
86989	NYMTP	2020-10-06	20.46	20.53	20.38	20.5	4807	{}	\N	2020-10-12 16:57:16.57754
86990	NYMTP	2020-10-07	21.24	21.24	20.21	20.49	9102	{}	\N	2020-10-12 16:57:16.582489
86991	NYMTP	2020-10-08	21.04	21.25	20.51	20.71	4902	{}	\N	2020-10-12 16:57:16.587135
86992	NYMTP	2020-10-09	20.57	20.66	20.5	20.5	307697	{}	\N	2020-10-12 16:57:16.591854
86993	NYMTP	2020-10-12	20.5	20.5	20.36	20.4	2252	{}	\N	2020-10-12 16:57:16.596759
86994	OCSL	2020-09-28	4.8	4.91	4.77	4.87	256791	{}	\N	2020-10-12 16:57:17.567372
86995	OCSL	2020-09-29	4.84	4.86	4.78	4.84	201997	{}	\N	2020-10-12 16:57:17.579777
86996	OCSL	2020-09-30	4.88	4.955	4.83	4.84	416236	{}	\N	2020-10-12 16:57:17.58825
86997	OCSL	2020-10-01	4.86	4.935	4.8	4.93	459943	{}	\N	2020-10-12 16:57:17.594434
86998	OCSL	2020-10-02	4.88	4.91	4.85	4.89	184275	{}	\N	2020-10-12 16:57:17.601337
86999	OCSL	2020-10-05	4.91	4.93	4.84	4.87	178314	{}	\N	2020-10-12 16:57:17.606641
87000	OCSL	2020-10-06	4.8723	4.89	4.815	4.82	164284	{}	\N	2020-10-12 16:57:17.611962
87001	OCSL	2020-10-07	4.86	4.885	4.815	4.83	235514	{}	\N	2020-10-12 16:57:17.617272
87002	OCSL	2020-10-08	4.85	4.93	4.82	4.93	280084	{}	\N	2020-10-12 16:57:17.622579
87003	OCSL	2020-10-09	4.92	4.9284	4.88	4.91	159674	{}	\N	2020-10-12 16:57:17.627595
87004	OCSL	2020-10-12	4.93	4.93	4.78	4.88	353159	{}	\N	2020-10-12 16:57:17.632673
87005	OMFS	2020-09-28	23.75	24.1388	23.75	24.09	807	{}	\N	2020-10-12 16:57:20.465553
87006	OMFS	2020-09-29	23.6495	23.815	23.5394	23.815	793	{}	\N	2020-10-12 16:57:20.477931
87007	OMFS	2020-09-30	23.88	24.08	23.68	23.68	1314	{}	\N	2020-10-12 16:57:20.485986
87008	OMFS	2020-10-01	23.8229	24.0546	23.8229	24.0546	574	{}	\N	2020-10-12 16:57:20.491802
87009	OMFS	2020-10-02	24.1175	24.6579	24.1175	24.6579	467	{}	\N	2020-10-12 16:57:20.497425
87010	OMFS	2020-10-05	25.14	25.2598	25.14	25.2598	370	{}	\N	2020-10-12 16:57:20.502806
87011	OMFS	2020-10-06	25.1558	25.1558	25.1558	25.1558	1	{}	\N	2020-10-12 16:57:20.507769
87012	OMFS	2020-10-07	25.6981	25.7266	25.6981	25.7266	399	{}	\N	2020-10-12 16:57:20.513005
87013	OMFS	2020-10-08	25.951	26.28	25.9201	26.28	1333	{}	\N	2020-10-12 16:57:20.517919
87014	OMFS	2020-10-09	26.24	26.24	26.1702	26.1702	429	{}	\N	2020-10-12 16:57:20.522776
87015	OMFS	2020-10-12	26.21	26.21	26.21	26.21	345	{}	\N	2020-10-12 16:57:20.52798
87016	ONEQ	2020-09-28	431.51	432.889	428.5289	432.88	45780	{}	\N	2020-10-12 16:57:21.781969
87017	ONEQ	2020-09-29	432.65	433.5	430.5548	430.81	14003	{}	\N	2020-10-12 16:57:21.792516
87018	ONEQ	2020-09-30	432.17	438.86	432.17	434	24678	{}	\N	2020-10-12 16:57:21.799846
87019	ONEQ	2020-10-01	439	440.745	437.01	439.5	37731	{}	\N	2020-10-12 16:57:21.805828
87020	ONEQ	2020-10-02	430	437.2045	428.3801	430.28	57838	{}	\N	2020-10-12 16:57:21.811291
87021	ONEQ	2020-10-05	434.005	440.95	433.15	439.81	27683	{}	\N	2020-10-12 16:57:21.816403
87022	ONEQ	2020-10-06	439.97	443.1198	433.1078	434.2459	25064	{}	\N	2020-10-12 16:57:21.821087
87023	ONEQ	2020-10-07	436.64	442.45	436.64	442.45	23701	{}	\N	2020-10-12 16:57:21.825547
87024	ONEQ	2020-10-08	445.39	445.54	442.45	443.885	21044	{}	\N	2020-10-12 16:57:21.830163
87025	ONEQ	2020-10-09	446.99	449.99	446	449.84	21395	{}	\N	2020-10-12 16:57:21.835316
87026	ONEQ	2020-10-12	456.69	463.075	455.08	462.199	20029	{}	\N	2020-10-12 16:57:21.841201
87027	OTTR	2020-09-28	36.44	36.8	36.27	36.54	126202	{}	\N	2020-10-12 16:57:24.707353
87028	OTTR	2020-09-29	36.57	36.7288	35.82	36.04	65449	{}	\N	2020-10-12 16:57:24.719257
87029	OTTR	2020-09-30	36.18	36.55	35.95	36.19	60777	{}	\N	2020-10-12 16:57:24.732164
87030	OTTR	2020-10-01	36.22	36.64	35.88	36.6	96788	{}	\N	2020-10-12 16:57:24.740156
87031	OTTR	2020-10-02	36.12	37.23	36.12	37.05	90608	{}	\N	2020-10-12 16:57:24.745835
87032	OTTR	2020-10-05	37.28	37.78	37.22	37.57	88286	{}	\N	2020-10-12 16:57:24.751051
87033	OTTR	2020-10-06	37.8	39.09	37.708	38.25	114986	{}	\N	2020-10-12 16:57:24.756734
87034	OTTR	2020-10-07	38.45	38.87	38.17	38.67	109127	{}	\N	2020-10-12 16:57:24.761914
87035	OTTR	2020-10-08	38.97	39.63	38.85	39.51	106465	{}	\N	2020-10-12 16:57:24.767407
87036	OTTR	2020-10-09	39.55	39.65	38.86	39.01	73280	{}	\N	2020-10-12 16:57:24.773207
87037	OTTR	2020-10-12	39.06	39.496	38.885	39.25	18295	{}	\N	2020-10-12 16:57:24.778963
87038	OXFD	2020-09-28	11.82	11.86	11.65	11.77	110265	{}	\N	2020-10-12 16:57:26.180597
87039	OXFD	2020-09-29	11.79	11.93	11.77	11.82	162044	{}	\N	2020-10-12 16:57:26.193625
87040	OXFD	2020-09-30	11.84	12	11.535	11.7	72846	{}	\N	2020-10-12 16:57:26.200987
87041	OXFD	2020-10-01	11.7	11.76	11.465	11.59	90885	{}	\N	2020-10-12 16:57:26.207986
87042	OXFD	2020-10-02	11.45	11.75	11.035	11.67	37141	{}	\N	2020-10-12 16:57:26.213967
87043	OXFD	2020-10-05	11.9	12.07	11.48	11.82	68551	{}	\N	2020-10-12 16:57:26.21914
87044	OXFD	2020-10-06	12.03	12.03	11.62	11.81	169493	{}	\N	2020-10-12 16:57:26.224117
87045	OXFD	2020-10-07	11.89	13.17	11.88	13.07	1057785	{}	\N	2020-10-12 16:57:26.228876
87046	OXFD	2020-10-08	13.17	13.46	12.7	13.22	215307	{}	\N	2020-10-12 16:57:26.233744
87047	OXFD	2020-10-09	13.34	13.58	12.94	13.58	83950	{}	\N	2020-10-12 16:57:26.238827
87048	OXFD	2020-10-12	13.64	13.89	13.14	13.18	37743	{}	\N	2020-10-12 16:57:26.243621
87049	MLPC	2020-09-29	6.36	6.4364	6.36	6.4364	2026	{}	\N	2020-10-12 16:57:27.727454
87050	MLPC	2020-09-30	6.5753	6.5753	6.4547	6.4547	166	{}	\N	2020-10-12 16:57:27.736907
87051	MLPC	2020-10-01	6.1179	6.1179	6.1179	6.1179	150	{}	\N	2020-10-12 16:57:27.743984
87052	MLPC	2020-10-02	6.18	6.2932	6.18	6.2932	896	{}	\N	2020-10-12 16:57:27.750837
87053	MLPC	2020-10-05	6.4759	6.4759	6.4759	6.4759	74	{}	\N	2020-10-12 16:57:27.756452
87054	MLPC	2020-10-06	6.4963	6.4963	6.4963	6.4963	5	{}	\N	2020-10-12 16:57:27.762031
87055	MLPC	2020-10-07	6.4385	6.4602	6.4	6.4602	305	{}	\N	2020-10-12 16:57:27.767331
87056	MLPC	2020-10-08	6.57	6.8022	6.57	6.8022	3782	{}	\N	2020-10-12 16:57:27.772388
87057	MLPC	2020-10-09	6.62	6.8151	6.62	6.8151	287	{}	\N	2020-10-12 16:57:27.777639
87058	MLPC	2020-10-12	6.8	6.8	6.8	6.8	1004	{}	\N	2020-10-12 16:57:27.782872
87059	MPV	2020-09-28	11.2682	11.35	10.79	10.88	21373	{}	\N	2020-10-12 16:57:29.056989
87060	MPV	2020-09-29	10.9	10.9599	10.66	10.85	33332	{}	\N	2020-10-12 16:57:29.071056
87061	MPV	2020-09-30	10.83	11.0278	10.73	10.86	53650	{}	\N	2020-10-12 16:57:29.077867
87062	MPV	2020-10-01	10.89	10.92	10.7	10.9	63284	{}	\N	2020-10-12 16:57:29.083273
87063	MPV	2020-10-02	10.93	10.93	10.65	10.77	27184	{}	\N	2020-10-12 16:57:29.088003
87064	MPV	2020-10-05	10.72	10.8265	10.7	10.72	20510	{}	\N	2020-10-12 16:57:29.092615
87065	MPV	2020-10-06	10.7	10.86	10.6501	10.69	48510	{}	\N	2020-10-12 16:57:29.096717
87066	MPV	2020-10-07	10.77	10.89	10.474	10.59	36176	{}	\N	2020-10-12 16:57:29.10078
87067	MPV	2020-10-08	10.59	10.86	10.59	10.6843	45156	{}	\N	2020-10-12 16:57:29.104631
87068	MPV	2020-10-09	10.78	10.82	10.6019	10.8	56414	{}	\N	2020-10-12 16:57:29.108902
87069	MPV	2020-10-12	10.69	10.775	10.69	10.775	26054	{}	\N	2020-10-12 16:57:29.112774
87070	MX	2020-09-28	13.6	13.81	13.46	13.79	316211	{}	\N	2020-10-12 16:57:30.039639
87071	MX	2020-09-29	13.85	14.05	13.75	13.78	468527	{}	\N	2020-10-12 16:57:30.052989
87072	MX	2020-09-30	13.77	13.81	13.5	13.7	800552	{}	\N	2020-10-12 16:57:30.061335
87073	MX	2020-10-01	13.76	13.93	13.75	13.8	364830	{}	\N	2020-10-12 16:57:30.068909
87074	MX	2020-10-02	13.53	13.8	13.38	13.55	429844	{}	\N	2020-10-12 16:57:30.07386
87075	MX	2020-10-05	13.56	13.95	13.56	13.8	308239	{}	\N	2020-10-12 16:57:30.078679
87076	MX	2020-10-06	13.9	14	13.65	13.76	730360	{}	\N	2020-10-12 16:57:30.083113
87077	MX	2020-10-07	13.89	14.75	13.795	14.54	1601999	{}	\N	2020-10-12 16:57:30.087715
87078	MX	2020-10-08	14.56	14.74	14.22	14.23	830480	{}	\N	2020-10-12 16:57:30.092132
87079	MX	2020-10-09	14.25	14.78	14.21	14.27	1072916	{}	\N	2020-10-12 16:57:30.096403
87080	MX	2020-10-12	14.5	14.5	13.99	14.025	224722	{}	\N	2020-10-12 16:57:30.100679
87081	NEAR	2020-09-28	50.09	50.14	50.09	50.1	1105214	{}	\N	2020-10-12 16:57:32.226133
87082	NEAR	2020-09-29	50.1099	50.12	50.09	50.1	2315883	{}	\N	2020-10-12 16:57:32.237549
87083	NEAR	2020-09-30	50.09	50.13	50.09	50.11	969392	{}	\N	2020-10-12 16:57:32.245337
87084	NEAR	2020-10-01	50.09	50.1	50.08	50.085	395186	{}	\N	2020-10-12 16:57:32.252221
87085	NEAR	2020-10-02	50.07	50.09	50.07	50.085	415193	{}	\N	2020-10-12 16:57:32.258468
87086	NEAR	2020-10-05	50.08	50.11	50.08	50.1	1620131	{}	\N	2020-10-12 16:57:32.263823
87087	NEAR	2020-10-06	50.11	50.11	50.09	50.11	757608	{}	\N	2020-10-12 16:57:32.269099
87088	NEAR	2020-10-07	50.08	50.1	50.08	50.09	17741051	{}	\N	2020-10-12 16:57:32.273766
87089	NEAR	2020-10-08	50.11	50.12	50.09	50.1	1305322	{}	\N	2020-10-12 16:57:32.279098
87090	NEAR	2020-10-09	50.12	50.12	50.1	50.11	717307	{}	\N	2020-10-12 16:57:32.284089
87091	NEAR	2020-10-12	50.1	50.11	50.1	50.11	480253	{}	\N	2020-10-12 16:57:32.288569
87092	NGD	2020-09-28	1.71	1.73	1.62	1.68	4603680	{}	\N	2020-10-12 16:57:33.277576
87093	NGD	2020-09-29	1.7	1.75	1.68	1.72	4510018	{}	\N	2020-10-12 16:57:33.288647
87094	NGD	2020-09-30	1.72	1.72	1.65	1.7	4200436	{}	\N	2020-10-12 16:57:33.295911
87095	NGD	2020-10-01	1.73	1.81	1.695	1.79	6308847	{}	\N	2020-10-12 16:57:33.30285
87096	NGD	2020-10-02	1.79	1.81	1.725	1.74	4108057	{}	\N	2020-10-12 16:57:33.3082
87097	NGD	2020-10-05	1.77	1.96	1.76	1.96	7124218	{}	\N	2020-10-12 16:57:33.312702
87098	NGD	2020-10-06	1.98	2.22	1.97	1.98	16348462	{}	\N	2020-10-12 16:57:33.317476
87099	NGD	2020-10-07	2.02	2.0565	1.91	1.94	6080019	{}	\N	2020-10-12 16:57:33.322463
87100	NGD	2020-10-08	1.99	2	1.93	1.95	4372480	{}	\N	2020-10-12 16:57:33.327019
87101	NGD	2020-10-09	2.06	2.11	1.98	2.09	6556403	{}	\N	2020-10-12 16:57:33.332616
87102	NGD	2020-10-12	2.14	2.1401	2.04	2.085	2280629	{}	\N	2020-10-12 16:57:33.338292
87103	NINE	2020-09-28	1.16	1.38	1.16	1.29	249852	{}	\N	2020-10-12 16:57:35.917417
87104	NINE	2020-09-29	1.29	1.4678	1.17	1.2	1945462	{}	\N	2020-10-12 16:57:35.926916
87105	NINE	2020-09-30	1.23	1.26	1.1	1.13	487958	{}	\N	2020-10-12 16:57:35.933971
87106	NINE	2020-10-01	1.13	1.19	1.1	1.12	349068	{}	\N	2020-10-12 16:57:35.940518
87107	NINE	2020-10-02	1.07	1.2	1.06	1.16	452078	{}	\N	2020-10-12 16:57:35.946326
87108	NINE	2020-10-05	1.18	1.2299	1.1311	1.17	237418	{}	\N	2020-10-12 16:57:35.951636
87109	NINE	2020-10-06	1.2	1.25	1.13	1.16	279010	{}	\N	2020-10-12 16:57:35.956911
87110	NINE	2020-10-07	1.16	1.225	1.15	1.18	253220	{}	\N	2020-10-12 16:57:35.961771
87111	NINE	2020-10-08	1.19	1.3992	1.17	1.29	688558	{}	\N	2020-10-12 16:57:35.967506
87112	NINE	2020-10-09	1.36	1.44	1.3	1.41	834056	{}	\N	2020-10-12 16:57:35.97227
87113	NINE	2020-10-12	1.5	1.5	1.253	1.264	347693	{}	\N	2020-10-12 16:57:35.978019
87114	NKSH	2020-09-28	25.83	25.83	25.1	25.64	7323	{}	\N	2020-10-12 16:57:38.601741
87115	NKSH	2020-09-29	25.2067	25.22	25.14	25.22	1635	{}	\N	2020-10-12 16:57:38.612201
87116	NKSH	2020-09-30	25.49	25.5	25.3	25.33	4305	{}	\N	2020-10-12 16:57:38.61999
87117	NKSH	2020-10-01	25.45	25.72	25.26	25.26	3644	{}	\N	2020-10-12 16:57:38.626286
87118	NKSH	2020-10-02	24.89	27.2	24.75	26	16704	{}	\N	2020-10-12 16:57:38.631928
87119	NKSH	2020-10-05	26.5	26.85	26.25	26.6	5266	{}	\N	2020-10-12 16:57:38.637174
87120	NKSH	2020-10-06	26.5	28	26.5	26.5	4544	{}	\N	2020-10-12 16:57:38.642205
87121	NKSH	2020-10-07	26.9515	27.44	26.53	26.98	6750	{}	\N	2020-10-12 16:57:38.64708
87122	NKSH	2020-10-08	25.945	26.965	25.945	26.5	7696	{}	\N	2020-10-12 16:57:38.651757
87123	NKSH	2020-10-09	24.5147	26.27	24.5147	26.22	8204	{}	\N	2020-10-12 16:57:38.656767
87124	NKSH	2020-10-12	26.45	26.5	25.96	26	8357	{}	\N	2020-10-12 16:57:38.661627
87125	NM	2020-09-28	1.8	1.8	1.74	1.74	28357	{}	\N	2020-10-12 16:57:39.585222
87126	NM	2020-09-29	1.75	1.8175	1.73	1.8	15782	{}	\N	2020-10-12 16:57:39.597624
87127	NM	2020-09-30	1.78	1.805	1.73	1.76	34766	{}	\N	2020-10-12 16:57:39.606264
87128	NM	2020-10-01	1.79	1.82	1.75	1.81	59272	{}	\N	2020-10-12 16:57:39.612928
87129	NM	2020-10-02	1.82	1.91	1.82	1.91	79696	{}	\N	2020-10-12 16:57:39.619051
87130	NM	2020-10-05	1.92	1.99	1.89	1.92	129024	{}	\N	2020-10-12 16:57:39.624598
87131	NM	2020-10-06	1.93	1.96	1.85	1.86	82869	{}	\N	2020-10-12 16:57:39.629583
87132	NM	2020-10-07	1.9	1.9	1.82	1.85	44352	{}	\N	2020-10-12 16:57:39.634818
87133	NM	2020-10-08	1.84	1.9209	1.84	1.915	40790	{}	\N	2020-10-12 16:57:39.640005
87134	NM	2020-10-09	1.94	1.94	1.87	1.87	45922	{}	\N	2020-10-12 16:57:39.644987
87135	NM	2020-10-12	1.877	1.8836	1.82	1.82	9948	{}	\N	2020-10-12 16:57:39.649988
87136	NYMTO	2020-09-28	20.63	20.91	20.42	20.42	17492	{}	\N	2020-10-12 16:57:41.555814
87137	NYMTO	2020-09-29	20.59	20.8107	20.45	20.74	12275	{}	\N	2020-10-12 16:57:41.566662
87138	NYMTO	2020-09-30	20.38	20.55	20.2	20.41	14074	{}	\N	2020-10-12 16:57:41.57404
87139	NYMTO	2020-10-01	20.48	20.52	20.21	20.425	2012	{}	\N	2020-10-12 16:57:41.579824
87140	NYMTO	2020-10-02	20.34	20.34	20.15	20.21	4063	{}	\N	2020-10-12 16:57:41.585782
87141	NYMTO	2020-10-05	20.23	20.53	20.18	20.34	7212	{}	\N	2020-10-12 16:57:41.591076
87142	NYMTO	2020-10-06	20.21	20.45	20.18	20.2801	7582	{}	\N	2020-10-12 16:57:41.59638
87143	NYMTO	2020-10-07	20.315	20.4	20.23	20.4	8025	{}	\N	2020-10-12 16:57:41.602117
87144	NYMTO	2020-10-08	20.47	20.6	20.255	20.59	4504	{}	\N	2020-10-12 16:57:41.607062
87145	NYMTO	2020-10-09	20.505	20.6299	20.352	20.6299	127990	{}	\N	2020-10-12 16:57:41.612103
87146	NYMTO	2020-10-12	20.3001	20.48	20.25	20.48	4116	{}	\N	2020-10-12 16:57:41.617499
87147	OCSI	2020-09-28	6.43	6.6536	6.39	6.53	48267	{}	\N	2020-10-12 16:57:42.984921
87148	OCSI	2020-09-29	6.575	6.61	6.45	6.45	14632	{}	\N	2020-10-12 16:57:42.997512
87149	OCSI	2020-09-30	6.67	6.67	6.51	6.51	47933	{}	\N	2020-10-12 16:57:43.004808
87150	OCSI	2020-10-01	6.57	6.67	6.57	6.62	35930	{}	\N	2020-10-12 16:57:43.010963
87151	OCSI	2020-10-02	6.36	6.75	6.36	6.7	45357	{}	\N	2020-10-12 16:57:43.016591
87152	OCSI	2020-10-05	6.7431	6.78	6.67	6.76	41742	{}	\N	2020-10-12 16:57:43.021271
87153	OCSI	2020-10-06	6.765	6.8189	6.72	6.76	38629	{}	\N	2020-10-12 16:57:43.026049
87154	OCSI	2020-10-07	6.9	6.9	6.72	6.7981	51730	{}	\N	2020-10-12 16:57:43.030433
87155	OCSI	2020-10-08	6.78	6.79	6.71	6.7516	36149	{}	\N	2020-10-12 16:57:43.034708
87156	OCSI	2020-10-09	6.7	6.74	6.69	6.73	37599	{}	\N	2020-10-12 16:57:43.038746
87157	OCSI	2020-10-12	6.76	6.76	6.6139	6.65	13202	{}	\N	2020-10-12 16:57:43.042739
87158	OEUR	2020-09-28	23.76	23.8199	23.71	23.71	6547	{}	\N	2020-10-12 16:57:44.037245
87159	OEUR	2020-09-29	23.85	23.85	23.69	23.69	3242	{}	\N	2020-10-12 16:57:44.048344
87160	OEUR	2020-09-30	23.77	23.84	23.6	23.625	94281	{}	\N	2020-10-12 16:57:44.055067
87161	OEUR	2020-10-01	23.68	23.8	23.68	23.8	3318	{}	\N	2020-10-12 16:57:44.060892
87162	OEUR	2020-10-02	23.71	23.85	23.7	23.82	1214	{}	\N	2020-10-12 16:57:44.065866
87163	OEUR	2020-10-05	24.02	24.13	24.004	24.13	3160	{}	\N	2020-10-12 16:57:44.070153
87164	OEUR	2020-10-06	23.97	24.055	23.67	23.74	7635	{}	\N	2020-10-12 16:57:44.074025
87165	OEUR	2020-10-07	23.73	23.82	23.73	23.82	1028	{}	\N	2020-10-12 16:57:44.078059
87166	OEUR	2020-10-08	23.9	24.09	23.8599	23.93	2609	{}	\N	2020-10-12 16:57:44.081906
87167	OEUR	2020-10-09	24.0187	24.13	24.0187	24.13	16089	{}	\N	2020-10-12 16:57:44.086068
87168	OEUR	2020-10-12	24.26	24.34	24.26	24.34	3018	{}	\N	2020-10-12 16:57:44.090047
87169	OXBR	2020-09-28	1.55	9.62	1.55	2.95	141740700	{}	\N	2020-10-12 16:57:45.454809
87170	OXBR	2020-09-29	2.4	2.64	2.04	2.17	8014385	{}	\N	2020-10-12 16:57:45.46497
87171	OXBR	2020-09-30	1.99	2	1.67	1.76	2900698	{}	\N	2020-10-12 16:57:45.47163
87172	OXBR	2020-10-01	1.74	2.49	1.72	1.92	13142669	{}	\N	2020-10-12 16:57:45.47756
87173	OXBR	2020-10-02	1.8	1.96	1.72	1.77	1535874	{}	\N	2020-10-12 16:57:45.482785
87174	OXBR	2020-10-05	1.8	1.9072	1.68	1.78	763905	{}	\N	2020-10-12 16:57:45.486982
87175	OXBR	2020-10-06	1.89	1.95	1.77	1.82	1321314	{}	\N	2020-10-12 16:57:45.491129
87176	OXBR	2020-10-07	1.88	1.88	1.76	1.78	657878	{}	\N	2020-10-12 16:57:45.495553
87177	OXBR	2020-10-08	1.86	3.18	1.8	2.33	59403328	{}	\N	2020-10-12 16:57:45.499815
87178	OXBR	2020-10-09	2.12	2.4	1.97	2.37	2676087	{}	\N	2020-10-12 16:57:45.50393
87179	OXBR	2020-10-12	2.22	2.29	2.07	2.13	586952	{}	\N	2020-10-12 16:57:45.507905
87180	OXY	2020-09-28	10.42	10.65	10.31	10.34	22377909	{}	\N	2020-10-12 16:57:46.626212
87181	OXY	2020-09-29	10.34	10.35	9.87	9.99	56988952	{}	\N	2020-10-12 16:57:46.641615
87182	OXY	2020-09-30	10.02	10.27	9.94	10.01	45346088	{}	\N	2020-10-12 16:57:46.649886
87183	OXY	2020-10-01	9.97	9.98	9.57	9.68	65102343	{}	\N	2020-10-12 16:57:46.656172
87184	OXY	2020-10-02	9.25	9.9	9.23	9.87	56032027	{}	\N	2020-10-12 16:57:46.661947
87185	OXY	2020-10-05	10.1	10.52	10.01	10.51	48903681	{}	\N	2020-10-12 16:57:46.667013
87186	OXY	2020-10-06	10.67	10.78	10.04	10.13	57500350	{}	\N	2020-10-12 16:57:46.671662
87187	OXY	2020-10-07	10.23	10.32	10.12	10.28	34052325	{}	\N	2020-10-12 16:57:46.676551
87188	OXY	2020-10-08	10.41	11.2	10.3301	11.18	54345798	{}	\N	2020-10-12 16:57:46.680998
87189	OXY	2020-10-09	11.2	11.335	10.74	10.82	52247118	{}	\N	2020-10-12 16:57:46.685798
87190	OXY	2020-10-12	10.64	10.69	10.4	10.645	19825743	{}	\N	2020-10-12 16:57:46.690471
87191	MTDR	2020-09-28	8.32	8.74	8.16	8.58	2710321	{}	\N	2020-10-12 16:57:49.998556
87192	MTDR	2020-09-29	8.54	8.57	7.8701	8.34	4679577	{}	\N	2020-10-12 16:57:50.011005
87193	MTDR	2020-09-30	8.33	8.665	8.205	8.26	5192099	{}	\N	2020-10-12 16:57:50.01985
87194	MTDR	2020-10-01	8.07	8.19	7.82	8.1	5905644	{}	\N	2020-10-12 16:57:50.026637
87195	MTDR	2020-10-02	7.74	8.42	7.74	8.29	6313118	{}	\N	2020-10-12 16:57:50.033085
87196	MTDR	2020-10-05	8.52	8.86	8.46	8.8	5070107	{}	\N	2020-10-12 16:57:50.038522
87197	MTDR	2020-10-06	9	9.1984	8.31	8.32	4815921	{}	\N	2020-10-12 16:57:50.043836
87198	MTDR	2020-10-07	8.44	8.78	8.3	8.73	4688249	{}	\N	2020-10-12 16:57:50.049097
87199	MTDR	2020-10-08	8.85	9.4	8.68	9.4	4958318	{}	\N	2020-10-12 16:57:50.054659
87200	MTDR	2020-10-09	9.45	9.56	9.06	9.14	4363047	{}	\N	2020-10-12 16:57:50.059884
87201	MTDR	2020-10-12	9.03	9.03	8.605	8.96	1789251	{}	\N	2020-10-12 16:57:50.06537
87202	NBL	2020-09-28	8.68	8.85	8.655	8.75	13752675	{}	\N	2020-10-12 16:57:52.135959
87203	NBL	2020-09-29	8.71	8.77	8.415	8.52	12955886	{}	\N	2020-10-12 16:57:52.145072
87204	NBL	2020-09-30	8.58	8.63	8.48	8.55	9355210	{}	\N	2020-10-12 16:57:52.151116
87205	NBL	2020-10-01	8.45	8.51	8.28	8.34	18308840	{}	\N	2020-10-12 16:57:52.157035
87206	NBL	2020-10-02	8.16	8.51	8.12	8.46	11722480	{}	\N	2020-10-12 16:57:52.162147
87207	NGHC	2020-09-28	33.91	34.05	33.9	33.9	358971	{}	\N	2020-10-12 16:57:53.09137
87208	NGHC	2020-09-29	33.92	33.94	33.9	33.91	230901	{}	\N	2020-10-12 16:57:53.103732
87209	NGHC	2020-09-30	33.94	33.95	33.7	33.76	540056	{}	\N	2020-10-12 16:57:53.111749
87210	NGHC	2020-10-01	33.85	33.91	33.82	33.83	495296	{}	\N	2020-10-12 16:57:53.118636
87211	NGHC	2020-10-02	33.8	33.89	33.75	33.75	693485	{}	\N	2020-10-12 16:57:53.125161
87212	NGHC	2020-10-05	33.8	33.87	33.76	33.79	465562	{}	\N	2020-10-12 16:57:53.131301
87213	NGHC	2020-10-06	33.8	33.82	33.75	33.77	341463	{}	\N	2020-10-12 16:57:53.13677
87214	NGHC	2020-10-07	33.82	33.88	33.79	33.81	1041831	{}	\N	2020-10-12 16:57:53.14222
87215	NGHC	2020-10-08	33.83	33.85	33.805	33.84	888986	{}	\N	2020-10-12 16:57:53.147479
87216	NGHC	2020-10-09	33.94	33.94	33.84	33.865	430330	{}	\N	2020-10-12 16:57:53.153708
87217	NGHC	2020-10-12	33.87	33.9	33.85	33.885	326244	{}	\N	2020-10-12 16:57:53.159704
87218	NID	2020-09-28	13.37	13.37	13.29	13.315	24853	{}	\N	2020-10-12 16:57:54.642477
87219	NID	2020-09-29	13.29	13.33	13.29	13.33	62472	{}	\N	2020-10-12 16:57:54.653542
87220	NID	2020-09-30	13.28	13.41	13.28	13.36	129212	{}	\N	2020-10-12 16:57:54.66361
87221	NID	2020-10-01	13.46	13.46	13.4026	13.429	78512	{}	\N	2020-10-12 16:57:54.669794
87222	NID	2020-10-02	13.36	13.46	13.23	13.3	235334	{}	\N	2020-10-12 16:57:54.676097
87223	NID	2020-10-05	13.28	13.334	13.25	13.26	71598	{}	\N	2020-10-12 16:57:54.68268
87224	NID	2020-10-06	13.32	13.38	13.26	13.29	141744	{}	\N	2020-10-12 16:57:54.688551
87225	NID	2020-10-07	13.32	13.32	13.22	13.22	140250	{}	\N	2020-10-12 16:57:54.694042
87226	NID	2020-10-08	13.25	13.34	13.25	13.3	186860	{}	\N	2020-10-12 16:57:54.699865
87227	NID	2020-10-09	13.3	13.37	13.2957	13.35	71092	{}	\N	2020-10-12 16:57:54.705562
87228	NID	2020-10-12	13.33	13.4	13.33	13.3898	46152	{}	\N	2020-10-12 16:57:54.711913
87229	NLR	2020-09-28	44.15	44.4443	44.15	44.355	1506	{}	\N	2020-10-12 16:57:55.721612
87230	NLR	2020-09-29	44.35	44.35	44.0911	44.0911	288	{}	\N	2020-10-12 16:57:55.732914
87231	NLR	2020-09-30	44.22	44.57	44.01	44.57	2797	{}	\N	2020-10-12 16:57:55.741451
87232	NLR	2020-10-01	44.65	45.06	44.576	44.87	15893	{}	\N	2020-10-12 16:57:55.748019
87233	NLR	2020-10-02	45.24	45.24	45.24	45.24	19	{}	\N	2020-10-12 16:57:55.75406
87234	NLR	2020-10-05	45.18	45.78	45.18	45.78	1800	{}	\N	2020-10-12 16:57:55.759758
87235	NLR	2020-10-06	46.06	46.06	45.561	45.77	2689	{}	\N	2020-10-12 16:57:55.766223
87236	NLR	2020-10-07	45.9032	46.1056	45.9032	46.1056	415	{}	\N	2020-10-12 16:57:55.772313
87237	NLR	2020-10-08	46.5188	46.6885	46.5188	46.6885	584	{}	\N	2020-10-12 16:57:55.77844
87238	NLR	2020-10-09	46.57	46.7119	46.57	46.7119	342	{}	\N	2020-10-12 16:57:55.784081
87239	NLR	2020-10-12	46.31	46.31	46.31	46.31	708	{}	\N	2020-10-12 16:57:55.790323
87240	NOBL	2020-09-28	71.98	72.6	71.91	72.14	333247	{}	\N	2020-10-12 16:57:56.976254
87241	NOBL	2020-09-29	72.25	72.25	71.36	71.57	470757	{}	\N	2020-10-12 16:57:56.991388
87242	NOBL	2020-09-30	71.95	72.77	71.6	72.15	356480	{}	\N	2020-10-12 16:57:56.999732
87243	NOBL	2020-10-01	72.54	72.7735	71.67	72.07	364303	{}	\N	2020-10-12 16:57:57.007467
87244	NOBL	2020-10-02	71.01	72.94	71.01	72.64	453606	{}	\N	2020-10-12 16:57:57.013702
87245	NOBL	2020-10-05	73.09	73.787	73.09	73.78	330729	{}	\N	2020-10-12 16:57:57.019192
87246	NOBL	2020-10-06	73.96	74.58	72.97	73.1	424898	{}	\N	2020-10-12 16:57:57.025127
87247	NOBL	2020-10-07	73.8881	74.4498	73.8	74.25	297130	{}	\N	2020-10-12 16:57:57.030584
87248	NOBL	2020-10-08	74.61	75.13	74.576	75.08	406695	{}	\N	2020-10-12 16:57:57.035952
87249	NOBL	2020-10-09	75.36	75.8025	75.2	75.42	351718	{}	\N	2020-10-12 16:57:57.041235
87250	NOBL	2020-10-12	75.5	76.1496	75.5	76.03	235911	{}	\N	2020-10-12 16:57:57.046456
87251	NVMI	2020-09-28	51.13	51.92	50.416	51.64	84244	{}	\N	2020-10-12 16:57:57.963003
87252	NVMI	2020-09-29	51.47	52.9	51.43	52.45	99192	{}	\N	2020-10-12 16:57:57.974586
87253	NVMI	2020-09-30	52.29	52.9	52	52.14	66802	{}	\N	2020-10-12 16:57:57.982889
87254	NVMI	2020-10-01	53.19	54.11	53.19	54.11	63204	{}	\N	2020-10-12 16:57:57.988977
87255	NVMI	2020-10-02	52.75	53.04	52.09	52.56	52526	{}	\N	2020-10-12 16:57:57.995103
87256	NVMI	2020-10-05	53.06	54	53.06	53.78	32037	{}	\N	2020-10-12 16:57:58.000041
87257	NVMI	2020-10-06	53.8	54.33	53.205	53.23	42497	{}	\N	2020-10-12 16:57:58.004632
87258	NVMI	2020-10-07	53.91	54.489	53.36	53.89	57826	{}	\N	2020-10-12 16:57:58.010401
87259	NVMI	2020-10-08	54.09	55.17	54.0201	55.08	45159	{}	\N	2020-10-12 16:57:58.015688
87260	NVMI	2020-10-09	55.61	56.58	55.53	56.05	55166	{}	\N	2020-10-12 16:57:58.020373
87261	NVMI	2020-10-12	56.38	57.14	56.38	56.72	27174	{}	\N	2020-10-12 16:57:58.026063
87262	NXST	2020-09-28	96.11	96.335	94.4	94.54	319911	{}	\N	2020-10-12 16:57:59.478966
87263	NXST	2020-09-29	94.47	94.76	90.42	91.4	298355	{}	\N	2020-10-12 16:57:59.491907
87264	NXST	2020-09-30	91.44	93.3	88.94	89.93	321988	{}	\N	2020-10-12 16:57:59.499914
87265	NXST	2020-10-01	90.69	91.26	89.28	89.53	315614	{}	\N	2020-10-12 16:57:59.506877
87266	NXST	2020-10-02	87.6	91.2	87.21	89.96	392314	{}	\N	2020-10-12 16:57:59.513375
87267	NXST	2020-10-05	91.33	92.58	89.76	90.53	407485	{}	\N	2020-10-12 16:57:59.518856
87268	NXST	2020-10-06	91.14	92.0462	88.87	89.36	289398	{}	\N	2020-10-12 16:57:59.524292
87269	NXST	2020-10-07	90.24	92.365	90.24	91.66	413584	{}	\N	2020-10-12 16:57:59.52995
87270	NXST	2020-10-08	92.59	94.32	92.315	94.21	247637	{}	\N	2020-10-12 16:57:59.535205
87271	NXST	2020-10-09	94.88	95.6835	91.68	92.16	235972	{}	\N	2020-10-12 16:57:59.540725
87272	NXST	2020-10-12	92.64	93.74	91.58	92.98	67053	{}	\N	2020-10-12 16:57:59.546293
87273	ODP	2020-09-28	18.62	18.885	18.38	18.45	364411	{}	\N	2020-10-12 16:58:00.792459
87274	ODP	2020-09-29	18.41	18.5	17.86	18.3	365355	{}	\N	2020-10-12 16:58:00.80069
87275	ODP	2020-09-30	18.37	19.49	18.37	19.45	586404	{}	\N	2020-10-12 16:58:00.806649
87276	ODP	2020-10-01	19.41	20.315	19.4	20.21	356015	{}	\N	2020-10-12 16:58:00.812532
87277	ODP	2020-10-02	19.77	20.71	19.4101	20.58	425713	{}	\N	2020-10-12 16:58:00.818116
87278	ODP	2020-10-05	20.96	22.22	20.8919	22.14	327609	{}	\N	2020-10-12 16:58:00.823373
87279	ODP	2020-10-06	22.28	23.0604	21.92	22.025	599107	{}	\N	2020-10-12 16:58:00.829081
87280	ODP	2020-10-07	22.33	23.24	22.1109	22.75	609696	{}	\N	2020-10-12 16:58:00.834321
87281	ODP	2020-10-08	22.9	23.25	22.59	23.21	496719	{}	\N	2020-10-12 16:58:00.839523
87282	ODP	2020-10-09	23.5	23.73	22.475	22.63	393392	{}	\N	2020-10-12 16:58:00.844932
87283	ODP	2020-10-12	22.55	22.965	21.78	21.92	94266	{}	\N	2020-10-12 16:58:00.850144
87284	PACB	2020-09-28	9.35	9.64	9.025	9.6	3713191	{}	\N	2020-10-12 16:58:03.360084
87285	PACB	2020-09-29	9.53	10.01	9.51	9.8	4334142	{}	\N	2020-10-12 16:58:03.372409
87286	PACB	2020-09-30	9.87	10.11	9.83	9.87	4132839	{}	\N	2020-10-12 16:58:03.381055
87287	PACB	2020-10-01	10.01	10.65	9.97	10.5	4956221	{}	\N	2020-10-12 16:58:03.387373
87288	PACB	2020-10-02	11.4	11.795	10.79	11.71	8103814	{}	\N	2020-10-12 16:58:03.393298
87289	PACB	2020-10-05	12.07	12.83	11.99	12.74	8636663	{}	\N	2020-10-12 16:58:03.39913
87290	PACB	2020-10-06	12.87	12.9	11.33	12.17	8799328	{}	\N	2020-10-12 16:58:03.404162
87291	PACB	2020-10-07	12.38	12.71	12.05	12.23	5947382	{}	\N	2020-10-12 16:58:03.4094
87292	PACB	2020-10-08	12.45	13.21	12.28	12.73	8101796	{}	\N	2020-10-12 16:58:03.414435
87293	PACB	2020-10-09	12.755	13.54	12.7482	13.5	5372740	{}	\N	2020-10-12 16:58:03.419486
87294	PACB	2020-10-12	13.76	13.9	12.34	12.49	4421168	{}	\N	2020-10-12 16:58:03.424648
87295	PANL	2020-09-28	2.5	2.5	2.36	2.42	33579	{}	\N	2020-10-12 16:58:04.424041
87296	PANL	2020-09-29	2.28	2.5999	2.28	2.59	9227	{}	\N	2020-10-12 16:58:04.435777
87297	PANL	2020-09-30	2.6	2.6	2.43	2.59	21961	{}	\N	2020-10-12 16:58:04.444005
87298	PANL	2020-10-01	2.565	2.65	2.56	2.59	7923	{}	\N	2020-10-12 16:58:04.452736
87299	PANL	2020-10-02	2.52	2.69	2.5	2.62	19856	{}	\N	2020-10-12 16:58:04.45873
87300	PANL	2020-10-05	2.66	2.85	2.65	2.85	20727	{}	\N	2020-10-12 16:58:04.464443
87301	PANL	2020-10-06	2.84	2.85	2.7	2.7	16850	{}	\N	2020-10-12 16:58:04.469515
87302	PANL	2020-10-07	2.675	2.82	2.675	2.82	9715	{}	\N	2020-10-12 16:58:04.474782
87303	PANL	2020-10-08	2.87	3.18	2.86	3.08	139619	{}	\N	2020-10-12 16:58:04.480042
87304	PANL	2020-10-09	3.2	3.22	3.05	3.18	85823	{}	\N	2020-10-12 16:58:04.485392
87305	PANL	2020-10-12	3.25	3.28	3.035	3.0701	67467	{}	\N	2020-10-12 16:58:04.490775
87306	PFI	2020-09-28	36.815	37.0074	36.8	36.9577	3223	{}	\N	2020-10-12 16:58:05.423676
87307	PFI	2020-09-29	37.05	37.05	36.67	36.67	1578	{}	\N	2020-10-12 16:58:05.435784
87308	PFI	2020-09-30	36.89	37.25	36.795	37.12	6529	{}	\N	2020-10-12 16:58:05.444197
87309	PFI	2020-10-01	37.61	37.61	37.1089	37.3408	4010	{}	\N	2020-10-12 16:58:05.451445
87310	PFI	2020-10-02	37.096	37.9005	37.096	37.9005	1502	{}	\N	2020-10-12 16:58:05.457725
87311	PFI	2020-10-05	37.92	38.4109	37.92	38.395	4839	{}	\N	2020-10-12 16:58:05.463174
87312	PFI	2020-10-06	38.67	38.67	37.9776	37.9776	1717	{}	\N	2020-10-12 16:58:05.468154
87313	PFI	2020-10-07	38.48	38.48	38.3152	38.3152	1778	{}	\N	2020-10-12 16:58:05.473409
87314	PFI	2020-10-08	38.64	38.84	38.5399	38.84	4931	{}	\N	2020-10-12 16:58:05.478406
87315	PFI	2020-10-09	38.79	39.17	38.79	39.1338	5067	{}	\N	2020-10-12 16:58:05.483447
87316	PFI	2020-10-12	39.12	39.4999	39.12	39.375	2208	{}	\N	2020-10-12 16:58:05.488825
87317	MOV	2020-09-28	10.38	10.59	10.27	10.39	150227	{}	\N	2020-10-12 16:58:10.18974
87318	MOV	2020-09-29	10.3	10.315	9.66	9.91	538806	{}	\N	2020-10-12 16:58:10.199862
87319	MOV	2020-09-30	9.93	10.1261	9.69	9.94	494115	{}	\N	2020-10-12 16:58:10.207714
87320	MOV	2020-10-01	9.99	10.36	9.85	10.3	403001	{}	\N	2020-10-12 16:58:10.213599
87321	MOV	2020-10-02	10.12	10.61	10.04	10.52	309893	{}	\N	2020-10-12 16:58:10.219
87322	MOV	2020-10-05	10.57	11	10.57	10.9	333668	{}	\N	2020-10-12 16:58:10.224067
87323	MOV	2020-10-06	11.09	11.1	10.39	10.48	345301	{}	\N	2020-10-12 16:58:10.22867
87324	MOV	2020-10-07	10.56	10.73	10.51	10.65	279737	{}	\N	2020-10-12 16:58:10.233074
87325	MOV	2020-10-08	10.77	10.8999	10.59	10.88	201976	{}	\N	2020-10-12 16:58:10.238263
87326	MOV	2020-10-09	10.91	11.01	10.61	10.94	262951	{}	\N	2020-10-12 16:58:10.243271
87327	MOV	2020-10-12	10.89	11.01	10.745	10.89	87123	{}	\N	2020-10-12 16:58:10.248022
87328	MTNB	2020-09-28	0.798	0.8	0.768	0.7726	993691	{}	\N	2020-10-12 16:58:11.351766
87329	MTNB	2020-09-29	0.7826	0.794	0.7645	0.7656	1359023	{}	\N	2020-10-12 16:58:11.361555
87330	MTNB	2020-09-30	0.77	0.7899	0.76	0.766	743973	{}	\N	2020-10-12 16:58:11.367924
87331	MTNB	2020-10-01	0.78	0.78	0.75	0.7655	786367	{}	\N	2020-10-12 16:58:11.374429
87332	MTNB	2020-10-02	0.75	0.82	0.75	0.7792	1035666	{}	\N	2020-10-12 16:58:11.379251
87333	MTNB	2020-10-05	0.79	0.8038	0.78001	0.7931	990056	{}	\N	2020-10-12 16:58:11.383412
87334	MTNB	2020-10-06	0.8	0.8479	0.79	0.79	2102652	{}	\N	2020-10-12 16:58:11.388005
87335	MTNB	2020-10-07	0.81	0.8395	0.81	0.8282	1577352	{}	\N	2020-10-12 16:58:11.3921
87336	MTNB	2020-10-08	0.82	0.87	0.81	0.8527	1942717	{}	\N	2020-10-12 16:58:11.396093
87337	MTNB	2020-10-09	0.85	0.8839	0.84	0.8561	1235865	{}	\N	2020-10-12 16:58:11.400123
87338	MTNB	2020-10-12	0.885	0.9	0.86	0.89001	1159181	{}	\N	2020-10-12 16:58:11.404244
87339	MTSI	2020-09-28	32.98	33.09	31.81	33.09	376507	{}	\N	2020-10-12 16:58:12.489203
87340	MTSI	2020-09-29	33.07	34.57	33.0227	34.15	364581	{}	\N	2020-10-12 16:58:12.504969
87341	MTSI	2020-09-30	34.15	34.92	33.63	34.01	433022	{}	\N	2020-10-12 16:58:12.51279
87342	MTSI	2020-10-01	34.71	35.11	34.25	35.11	247831	{}	\N	2020-10-12 16:58:12.519372
87343	MTSI	2020-10-02	33.86	34.6	33.5	33.8	292324	{}	\N	2020-10-12 16:58:12.526169
87344	MTSI	2020-10-05	34.15	35.06	34.15	35.05	255946	{}	\N	2020-10-12 16:58:12.531664
87345	MTSI	2020-10-06	35.13	36.2	34.69	34.795	266308	{}	\N	2020-10-12 16:58:12.537477
87346	MTSI	2020-10-07	35.45	36.24	34.8	35.16	261128	{}	\N	2020-10-12 16:58:12.542819
87347	MTSI	2020-10-08	35.48	35.88	35.18	35.34	225473	{}	\N	2020-10-12 16:58:12.548055
87348	MTSI	2020-10-09	35.92	36.81	35.92	36.35	192819	{}	\N	2020-10-12 16:58:12.553211
87349	MTSI	2020-10-12	37.25	37.25	36.19	36.42	81351	{}	\N	2020-10-12 16:58:12.558149
87350	MUC	2020-09-28	13.94	14.03	13.91	14.01	89469	{}	\N	2020-10-12 16:58:17.964949
87351	MUC	2020-09-29	14.01	14.04	13.99	13.99	115164	{}	\N	2020-10-12 16:58:17.975083
87352	MUC	2020-09-30	14.01	14.01	13.93	13.98	103520	{}	\N	2020-10-12 16:58:17.982043
87353	MUC	2020-10-01	13.94	14.09	13.91	14.07	206102	{}	\N	2020-10-12 16:58:17.988336
87354	MUC	2020-10-02	13.96	14.09	13.96	14.04	175002	{}	\N	2020-10-12 16:58:17.994122
87355	MUC	2020-10-05	14.1	14.16	13.98	14.1	143465	{}	\N	2020-10-12 16:58:17.999333
87356	MUC	2020-10-06	14.05	14.12	14.03	14.08	139282	{}	\N	2020-10-12 16:58:18.004351
87357	MUC	2020-10-07	14.15	14.24	14.08	14.17	103114	{}	\N	2020-10-12 16:58:18.009483
87358	MUC	2020-10-08	14.2	14.218	14.13	14.2	107770	{}	\N	2020-10-12 16:58:18.014831
87359	MUC	2020-10-09	14.28	14.28	14.16	14.25	123412	{}	\N	2020-10-12 16:58:18.020193
87360	MUC	2020-10-12	14.31	14.31	14.25	14.28	26786	{}	\N	2020-10-12 16:58:18.025462
87361	MVIS	2020-09-28	1.945	1.99	1.875	1.93	2496682	{}	\N	2020-10-12 16:58:18.924858
87362	MVIS	2020-09-29	1.93	2	1.82	1.91	2748029	{}	\N	2020-10-12 16:58:18.938021
87363	MVIS	2020-09-30	1.9	1.97	1.88	1.95	1950147	{}	\N	2020-10-12 16:58:18.94635
87364	MVIS	2020-10-01	1.97	2.13	1.95	2.1	4934060	{}	\N	2020-10-12 16:58:18.952438
87365	MVIS	2020-10-02	2.01	2.08	1.965	2.02	2287559	{}	\N	2020-10-12 16:58:18.958138
87366	MVIS	2020-10-05	2.12	2.18	2.01	2.08	2778401	{}	\N	2020-10-12 16:58:18.963204
87367	MVIS	2020-10-06	2.1	2.225	2.08	2.13	3881545	{}	\N	2020-10-12 16:58:18.967886
87368	MVIS	2020-10-07	2.19	2.29	2.11	2.12	6114780	{}	\N	2020-10-12 16:58:18.972334
87369	MVIS	2020-10-08	2.14	2.27	1.9	2.24	11441002	{}	\N	2020-10-12 16:58:18.976517
87370	MVIS	2020-10-09	2.67	2.89	2.33	2.61	37331328	{}	\N	2020-10-12 16:58:18.980745
87371	MVIS	2020-10-12	2.55	2.61	2.42	2.5928	3962822	{}	\N	2020-10-12 16:58:18.985335
87372	MXDU	2020-09-28	33.0098	33.0098	33.0098	33.0098	80	{}	\N	2020-10-12 16:58:22.102013
87373	MXDU	2020-09-29	32.951	32.951	32.951	32.951	0	{}	\N	2020-10-12 16:58:22.111342
87374	MXDU	2020-09-30	33.3601	33.3764	33.2146	33.2146	903	{}	\N	2020-10-12 16:58:22.11719
87375	MXDU	2020-10-01	33.4585	33.4585	33.4585	33.4585	10	{}	\N	2020-10-12 16:58:22.12207
87376	MXDU	2020-10-02	33.07	33.5699	33.07	33.45	1175	{}	\N	2020-10-12 16:58:22.127271
87377	MXDU	2020-10-05	33.8438	33.8438	33.8438	33.8438	0	{}	\N	2020-10-12 16:58:22.131626
87378	MXDU	2020-10-06	33.9541	33.9541	33.5636	33.5636	341	{}	\N	2020-10-12 16:58:22.136035
87379	MXDU	2020-10-07	34.1759	34.1759	34.1625	34.1625	769	{}	\N	2020-10-12 16:58:22.140263
87380	MXDU	2020-10-08	34.323	34.323	34.323	34.323	0	{}	\N	2020-10-12 16:58:22.14441
87381	MXDU	2020-10-09	34.5462	34.5462	34.5462	34.5462	0	{}	\N	2020-10-12 16:58:22.148828
87382	NGS	2020-09-28	8.31	8.41	8.25	8.3	31754	{}	\N	2020-10-12 16:58:25.000724
87383	NGS	2020-09-29	8.31	8.61	8.31	8.58	55726	{}	\N	2020-10-12 16:58:25.014071
87384	NGS	2020-09-30	8.64	8.75	8.37	8.45	83880	{}	\N	2020-10-12 16:58:25.021955
87385	NGS	2020-10-01	8.44	8.93	8.36	8.88	140964	{}	\N	2020-10-12 16:58:25.028135
87386	NGS	2020-10-02	8.74	9.1	8.6	9.07	239697	{}	\N	2020-10-12 16:58:25.033859
87387	NGS	2020-10-05	9.16	9.24	8.87	9	60250	{}	\N	2020-10-12 16:58:25.039569
87388	NGS	2020-10-06	8.89	9.15	8.84	8.85	51187	{}	\N	2020-10-12 16:58:25.044325
87389	NGS	2020-10-07	8.93	9.27	8.93	9.14	52526	{}	\N	2020-10-12 16:58:25.049397
87390	NGS	2020-10-08	9.11	9.74	9.11	9.66	102350	{}	\N	2020-10-12 16:58:25.054104
87391	NGS	2020-10-09	9.745	9.92	9.4701	9.8	120425	{}	\N	2020-10-12 16:58:25.058413
87392	NGS	2020-10-12	9.87	9.87	9.33	9.35	40531	{}	\N	2020-10-12 16:58:25.062584
87393	NIB	2020-09-28	31.64	31.9	31.12	31.19	6044	{}	\N	2020-10-12 16:58:26.438543
87394	NIB	2020-09-29	31.04	31.5199	31.01	31.4155	4671	{}	\N	2020-10-12 16:58:26.450061
87395	NIB	2020-09-30	31.04	31.04	30.57	30.57	5833	{}	\N	2020-10-12 16:58:26.457066
87396	NIB	2020-10-01	30.03	30.337	29.7211	30.02	11639	{}	\N	2020-10-12 16:58:26.462633
87397	NIB	2020-10-02	29.73	30.04	29.7267	29.88	16331	{}	\N	2020-10-12 16:58:26.468201
87398	NIB	2020-10-05	30.07	30.07	29.403	29.49	15372	{}	\N	2020-10-12 16:58:26.473299
87399	NIB	2020-10-06	29.38	29.51	29.06	29.18	10804	{}	\N	2020-10-12 16:58:26.477819
87400	NIB	2020-10-07	29.22	29.65	29.15	29.61	5993	{}	\N	2020-10-12 16:58:26.482127
87401	NIB	2020-10-08	29.79	29.835	29.61	29.835	3024	{}	\N	2020-10-12 16:58:26.486518
87402	NIB	2020-10-09	29.41	29.58	29	29.49	15701	{}	\N	2020-10-12 16:58:26.4905
87403	NIB	2020-10-12	29.25	29.335	29.1	29.2855	1430	{}	\N	2020-10-12 16:58:26.494444
87404	NMT	2020-09-28	13.89	13.89	13.89	13.89	466	{}	\N	2020-10-12 16:58:28.975417
87405	NMT	2020-09-29	13.91	13.95	13.75	13.9	13964	{}	\N	2020-10-12 16:58:28.986308
87406	NMT	2020-09-30	13.91	14.08	13.85	14.02	32818	{}	\N	2020-10-12 16:58:28.994177
87407	NMT	2020-10-01	14.05	14.07	13.875	14.07	32172	{}	\N	2020-10-12 16:58:29.00043
87408	NMT	2020-10-02	13.94	14.17	13.94	14.15	11814	{}	\N	2020-10-12 16:58:29.00626
87409	NMT	2020-10-05	14.1	14.13	14.08	14.09	5794	{}	\N	2020-10-12 16:58:29.01142
87410	NMT	2020-10-06	14.06	14.24	14.06	14.24	9038	{}	\N	2020-10-12 16:58:29.016717
87411	NMT	2020-10-07	14.14	14.19	14.03	14.19	16088	{}	\N	2020-10-12 16:58:29.022146
87412	NMT	2020-10-08	14.15	14.32	14.13	14.26	11696	{}	\N	2020-10-12 16:58:29.028426
87413	NMT	2020-10-12	14.091	14.26	14.091	14.26	2744	{}	\N	2020-10-12 16:58:29.032983
87414	OCN	2020-09-28	22	23.31	22	22.55	59719	{}	\N	2020-10-12 16:58:30.092522
87415	OCN	2020-09-29	22.53	22.7938	21.09	21.64	123975	{}	\N	2020-10-12 16:58:30.100504
87416	OCN	2020-09-30	21.93	22.1	21.045	21.12	224974	{}	\N	2020-10-12 16:58:30.10845
87417	OCN	2020-10-01	21.42	22.57	20.61	22.05	157600	{}	\N	2020-10-12 16:58:30.116811
87418	OCN	2020-10-02	21.68	23.54	21.16	23.45	111592	{}	\N	2020-10-12 16:58:30.125844
87419	OCN	2020-10-05	23.79	25.22	23.22	24.61	194618	{}	\N	2020-10-12 16:58:30.133658
87420	OCN	2020-10-06	24.72	25.43	24.07	24.07	219790	{}	\N	2020-10-12 16:58:30.140847
87421	OCN	2020-10-07	24.51	25.88	24.51	25.37	188875	{}	\N	2020-10-12 16:58:30.149409
87422	OCN	2020-10-08	25.47	25.98	24.65	25.68	96156	{}	\N	2020-10-12 16:58:30.156425
87423	OCN	2020-10-09	25.93	27.705	25.675	27.1	126860	{}	\N	2020-10-12 16:58:30.16413
87424	OCN	2020-10-12	27.11	27.235	26.5	26.6	56594	{}	\N	2020-10-12 16:58:30.170321
87425	OLD	2020-09-28	25.19	25.27	25.19	25.225	3615	{}	\N	2020-10-12 16:58:31.43622
87426	OLD	2020-09-29	25.27	25.38	25.15	25.15	7488	{}	\N	2020-10-12 16:58:31.447723
87427	OLD	2020-09-30	25.22	25.6861	25.1747	25.19	3217	{}	\N	2020-10-12 16:58:31.459929
87428	OLD	2020-10-01	25.4697	25.823	25.4697	25.823	514	{}	\N	2020-10-12 16:58:31.466831
87429	OLD	2020-10-02	25.33	26.1	25.33	26.1	4793	{}	\N	2020-10-12 16:58:31.472523
87430	OLD	2020-10-05	26.07	26.4	25.37	26.4	2578	{}	\N	2020-10-12 16:58:31.478089
87431	OLD	2020-10-06	26.24	26.31	26.01	26.01	1829	{}	\N	2020-10-12 16:58:31.485222
87432	OLD	2020-10-07	26.2141	26.4	25.8893	26.0205	4292	{}	\N	2020-10-12 16:58:31.490872
87433	OLD	2020-10-08	26.04	26.46	26.04	26.3149	3617	{}	\N	2020-10-12 16:58:31.495936
87434	OLD	2020-10-09	26.2	26.305	26.09	26.19	2327	{}	\N	2020-10-12 16:58:31.501437
87435	OLD	2020-10-12	26.3	26.3	26.22	26.22	511	{}	\N	2020-10-12 16:58:31.506931
87436	PAYC	2020-09-28	288.68	291.65	286	290.83	303074	{}	\N	2020-10-12 16:58:33.03091
87437	PAYC	2020-09-29	292.57	304.93	292.23	303.83	1293364	{}	\N	2020-10-12 16:58:33.041174
87438	PAYC	2020-09-30	302.35	312.72	301.285	311.3	1761943	{}	\N	2020-10-12 16:58:33.049365
87439	PAYC	2020-10-01	315.56	321.29	313.53	320.44	1539570	{}	\N	2020-10-12 16:58:33.056007
87440	PAYC	2020-10-02	326	335.2	320.95	331.53	1596284	{}	\N	2020-10-12 16:58:33.062172
87441	PAYC	2020-10-05	331.89	344.65	327.02	342.87	1327078	{}	\N	2020-10-12 16:58:33.067592
87442	PAYC	2020-10-06	342.38	346.15	335.3	339.28	1373608	{}	\N	2020-10-12 16:58:33.072887
87443	PAYC	2020-10-07	341.99	353.15	335.61	351.83	1114526	{}	\N	2020-10-12 16:58:33.078583
87444	PAYC	2020-10-08	359.7	361.92	354.635	359.53	981573	{}	\N	2020-10-12 16:58:33.083524
87445	PAYC	2020-10-09	361	368.43	360.8	366	1041983	{}	\N	2020-10-12 16:58:33.088608
87446	PAYC	2020-10-12	368.48	370.32	358.07	367.05	385800	{}	\N	2020-10-12 16:58:33.095227
87447	NMM	2020-09-28	5.82	6.11	5.82	5.93	56521	{}	\N	2020-10-12 16:58:35.484121
87448	NMM	2020-09-29	5.91	6.0261	5.8	6.01	103126	{}	\N	2020-10-12 16:58:35.495587
87449	NMM	2020-09-30	5.93	6.1999	5.93	6.02	170662	{}	\N	2020-10-12 16:58:35.502553
87450	NMM	2020-10-01	6.05	6.16	5.88	5.95	200039	{}	\N	2020-10-12 16:58:35.508329
87451	NMM	2020-10-02	5.95	6.29	5.9108	6.26	266383	{}	\N	2020-10-12 16:58:35.514593
87452	NMM	2020-10-05	6.34	6.69	6.34	6.67	276628	{}	\N	2020-10-12 16:58:35.520002
87453	NMM	2020-10-06	6.67	6.79	6.42	6.46	80985	{}	\N	2020-10-12 16:58:35.525078
87454	NMM	2020-10-07	6.52	6.63	6.36	6.46	103420	{}	\N	2020-10-12 16:58:35.531318
87455	NMM	2020-10-08	6.47	6.8	6.47	6.77	132526	{}	\N	2020-10-12 16:58:35.536723
87456	NMM	2020-10-09	6.8	6.81	6.6127	6.71	45405	{}	\N	2020-10-12 16:58:35.543106
87457	NMM	2020-10-12	6.65	6.7169	6.45	6.605	108428	{}	\N	2020-10-12 16:58:35.549583
87458	NURE	2020-09-28	23.28	23.4026	23.28	23.3496	15139	{}	\N	2020-10-12 16:58:38.185561
87459	NURE	2020-09-29	22.8	22.9499	22.8	22.9342	3506	{}	\N	2020-10-12 16:58:38.193638
87460	NURE	2020-09-30	23.03	23.06	22.7358	22.99	8196	{}	\N	2020-10-12 16:58:38.200495
87461	NURE	2020-10-01	23.207	23.7006	23.207	23.7006	4400	{}	\N	2020-10-12 16:58:38.20761
87462	NURE	2020-10-02	23.64	24.23	23.6077	24.23	238401	{}	\N	2020-10-12 16:58:38.213379
87463	NURE	2020-10-05	23.9607	24.333	23.9607	24.333	1965496	{}	\N	2020-10-12 16:58:38.220009
87464	NURE	2020-10-06	24.22	24.4687	24.2	24.21	2346	{}	\N	2020-10-12 16:58:38.225178
87465	NURE	2020-10-07	24.31	24.33	24.23	24.33	37858	{}	\N	2020-10-12 16:58:38.230193
87466	NURE	2020-10-08	24.6701	24.69	24.61	24.69	80011	{}	\N	2020-10-12 16:58:38.235849
87467	NURE	2020-10-09	24.5	24.545	24.44	24.52	10726	{}	\N	2020-10-12 16:58:38.241185
87468	NURE	2020-10-12	24.4	24.4001	24.4	24.4001	366	{}	\N	2020-10-12 16:58:38.245977
87469	NZF	2020-09-28	14.64	14.68	14.5956	14.64	207269	{}	\N	2020-10-12 16:58:39.622886
87470	NZF	2020-09-29	14.67	14.71	14.6395	14.7	501602	{}	\N	2020-10-12 16:58:39.633369
87471	NZF	2020-09-30	14.71	14.72	14.65	14.69	502129	{}	\N	2020-10-12 16:58:39.64117
87472	NZF	2020-10-01	14.77	14.77	14.73	14.75	494729	{}	\N	2020-10-12 16:58:39.647361
87473	NZF	2020-10-02	14.74	14.79	14.74	14.78	410730	{}	\N	2020-10-12 16:58:39.653224
87474	NZF	2020-10-05	14.78	14.79	14.68	14.74	758434	{}	\N	2020-10-12 16:58:39.658369
87475	NZF	2020-10-06	14.73	14.785	14.7219	14.76	384439	{}	\N	2020-10-12 16:58:39.664019
87476	NZF	2020-10-07	14.77	14.85	14.77	14.78	281493	{}	\N	2020-10-12 16:58:39.669639
87477	NZF	2020-10-08	14.8	14.84	14.78	14.84	352794	{}	\N	2020-10-12 16:58:39.675126
87478	NZF	2020-10-09	14.87	14.96	14.83	14.91	386558	{}	\N	2020-10-12 16:58:39.68055
87479	NZF	2020-10-12	14.94	14.96	14.9	14.93	214683	{}	\N	2020-10-12 16:58:39.686006
87480	OEC	2020-09-28	12.2	12.86	12.2	12.54	354793	{}	\N	2020-10-12 16:58:42.846911
87481	OEC	2020-09-29	12.5	12.78	12.19	12.64	700638	{}	\N	2020-10-12 16:58:42.858808
87482	OEC	2020-09-30	12.7	12.83	12.38	12.51	920018	{}	\N	2020-10-12 16:58:42.865505
87483	OEC	2020-10-01	12.52	12.8	12.37	12.78	677238	{}	\N	2020-10-12 16:58:42.871978
87484	OEC	2020-10-02	12.44	13.6	12.44	13.54	720639	{}	\N	2020-10-12 16:58:42.877579
87485	OEC	2020-10-05	13.67	14.42	13.67	14.36	781036	{}	\N	2020-10-12 16:58:42.882481
87486	OEC	2020-10-06	14.75	14.92	14.26	14.27	1018833	{}	\N	2020-10-12 16:58:42.887611
87487	OEC	2020-10-07	14.57	14.92	14.43	14.9	1379345	{}	\N	2020-10-12 16:58:42.892935
87488	OEC	2020-10-08	14.95	15.26	14.735	15.03	1226534	{}	\N	2020-10-12 16:58:42.897561
87489	OEC	2020-10-09	15.16	15.47	14.9	15.15	753480	{}	\N	2020-10-12 16:58:42.902613
87490	OEC	2020-10-12	15.16	15.23	14.965	15.15	283322	{}	\N	2020-10-12 16:58:42.908215
87491	PBD	2020-09-28	20.97	21.27	20.97	21.24	63694	{}	\N	2020-10-12 16:58:45.523998
87492	PBD	2020-09-29	21.48	21.6792	21.39	21.65	24368	{}	\N	2020-10-12 16:58:45.53583
87493	PBD	2020-09-30	21.87	22.107	21.81	21.94	292753	{}	\N	2020-10-12 16:58:45.544563
87494	PBD	2020-10-01	22.28	22.51	22.21	22.44	58858	{}	\N	2020-10-12 16:58:45.5511
87495	PBD	2020-10-02	22	22.65	22	22.49	49786	{}	\N	2020-10-12 16:58:45.557684
87496	PBD	2020-10-05	22.83	23.26	22.83	23.25	64040	{}	\N	2020-10-12 16:58:45.563242
87497	PBD	2020-10-06	23.44	23.64	23.11	23.21	47570	{}	\N	2020-10-12 16:58:45.568358
87498	PBD	2020-10-07	23.83	24.19	23.66	24.13	104976	{}	\N	2020-10-12 16:58:45.574034
87499	PBD	2020-10-08	24.58	24.609	24.2801	24.37	143060	{}	\N	2020-10-12 16:58:45.579646
87500	PBD	2020-10-09	24.36	24.74	24.3338	24.7	87044	{}	\N	2020-10-12 16:58:45.584879
87501	PBD	2020-10-12	24.96	24.96	24.56	24.72	67014	{}	\N	2020-10-12 16:58:45.590365
87502	PBDM	2020-09-28	23.5589	23.5589	23.53	23.53	1058	{}	\N	2020-10-12 16:58:47.480847
87503	PBDM	2020-09-29	23.62	23.62	23.52	23.52	696	{}	\N	2020-10-12 16:58:47.488278
87504	PBDM	2020-09-30	23.56	23.56	23.37	23.37	959	{}	\N	2020-10-12 16:58:47.49412
87505	PBDM	2020-10-01	23.54	23.585	23.54	23.585	569	{}	\N	2020-10-12 16:58:47.498953
87506	PBDM	2020-10-02	23.5941	23.5941	23.43	23.43	6003	{}	\N	2020-10-12 16:58:47.503613
87507	PBDM	2020-10-05	23.8153	23.8969	23.8153	23.8969	338	{}	\N	2020-10-12 16:58:47.508469
87508	PBDM	2020-10-06	23.89	23.89	23.55	23.57	1103	{}	\N	2020-10-12 16:58:47.513401
87509	PBDM	2020-10-07	23.83	23.9177	23.83	23.9177	870	{}	\N	2020-10-12 16:58:47.517613
87510	PBDM	2020-10-08	24.19	24.19	24.05	24.05	246	{}	\N	2020-10-12 16:58:47.521828
87511	PBDM	2020-10-09	24.1701	24.2059	24.16	24.2059	642	{}	\N	2020-10-12 16:58:47.526405
87512	PDN	2020-09-28	29.71	29.75	29.67	29.75	14703	{}	\N	2020-10-12 16:58:48.619504
87513	PDN	2020-09-29	29.75	29.769	29.63	29.71	13736	{}	\N	2020-10-12 16:58:48.631209
87514	PDN	2020-09-30	29.64	29.854	29.635	29.74	63225	{}	\N	2020-10-12 16:58:48.63949
87515	PDN	2020-10-01	29.87	29.954	29.78	29.95	17657	{}	\N	2020-10-12 16:58:48.646448
87516	PDN	2020-10-02	29.49	29.83	29.49	29.8	39328	{}	\N	2020-10-12 16:58:48.652655
87517	PDN	2020-10-05	30.11	30.33	30.11	30.33	23178	{}	\N	2020-10-12 16:58:48.658363
87518	PDN	2020-10-06	30.3	30.39	30.06	30.1	16347	{}	\N	2020-10-12 16:58:48.664348
87519	PDN	2020-10-07	30.2569	30.35	30.22	30.31	42407	{}	\N	2020-10-12 16:58:48.669889
87520	PDN	2020-10-08	30.47	30.56	30.45	30.54	10460	{}	\N	2020-10-12 16:58:48.675985
87521	PDN	2020-10-09	30.59	30.72	30.581	30.69	19029	{}	\N	2020-10-12 16:58:48.681821
87522	PDN	2020-10-12	30.69	30.845	30.69	30.83	9860	{}	\N	2020-10-12 16:58:48.687675
87523	PFF	2020-09-28	36.38	36.55	36.38	36.55	6576449	{}	\N	2020-10-12 16:58:49.732818
87524	PFF	2020-09-29	36.52	36.55	36.395	36.51	4305753	{}	\N	2020-10-12 16:58:49.742725
87525	PFF	2020-09-30	36.55	36.57	36.35	36.45	6909200	{}	\N	2020-10-12 16:58:49.751012
87526	PFF	2020-10-01	36.23	36.52	36.23	36.52	10278150	{}	\N	2020-10-12 16:58:49.757793
87527	PFF	2020-10-02	36.25	36.57	36.25	36.53	8712657	{}	\N	2020-10-12 16:58:49.763712
87528	PFF	2020-10-05	36.64	36.76	36.6	36.75	6247887	{}	\N	2020-10-12 16:58:49.769296
87529	PFF	2020-10-06	36.7606	36.9099	36.66	36.78	13408321	{}	\N	2020-10-12 16:58:49.774392
87530	PFF	2020-10-07	36.8549	36.94	36.82	36.94	5632584	{}	\N	2020-10-12 16:58:49.77978
87531	PFF	2020-10-08	37	37.05	36.95	37.01	7047942	{}	\N	2020-10-12 16:58:49.78485
87532	PFF	2020-10-09	37.03	37.075	36.9	37.04	4779230	{}	\N	2020-10-12 16:58:49.790062
87533	PFF	2020-10-12	37.1	37.11	36.95	37.07	3019421	{}	\N	2020-10-12 16:58:49.795499
87534	PFIG	2020-09-28	27.2604	27.34	27.2604	27.295	5280	{}	\N	2020-10-12 16:58:51.172131
87535	PFIG	2020-09-29	27.35	27.35	27.3143	27.33	7861	{}	\N	2020-10-12 16:58:51.183214
87536	PFIG	2020-09-30	27.2999	27.33	27.2912	27.3064	2743	{}	\N	2020-10-12 16:58:51.191889
87537	PFIG	2020-10-01	27.33	27.35	27.3001	27.3001	9924	{}	\N	2020-10-12 16:58:51.198221
87538	PFIG	2020-10-02	27.305	27.345	27.305	27.345	956	{}	\N	2020-10-12 16:58:51.20353
87539	PFIG	2020-10-05	27.3321	27.335	27.27	27.27	4466	{}	\N	2020-10-12 16:58:51.208852
87540	PFIG	2020-10-06	27.28	27.3397	27.27	27.27	3677	{}	\N	2020-10-12 16:58:51.214012
87541	PFIG	2020-10-07	27.3516	27.3838	27.25	27.36	15224	{}	\N	2020-10-12 16:58:51.219092
87542	PFIG	2020-10-08	27.3498	27.4	27.28	27.38	145921	{}	\N	2020-10-12 16:58:51.224003
87543	PFIG	2020-10-09	27.31	27.4	27.31	27.36	7048	{}	\N	2020-10-12 16:58:51.229481
87544	PFIG	2020-10-12	27.35	27.35	27.34	27.34	10242	{}	\N	2020-10-12 16:58:51.234524
87545	PFPT	2020-09-28	106.58	107.39	105.23	105.72	500621	{}	\N	2020-10-12 16:58:52.788529
87546	PFPT	2020-09-29	105.66	106.905	104.84	104.87	486096	{}	\N	2020-10-12 16:58:52.797786
87547	PFPT	2020-09-30	104.66	106.76	103.55	105.55	556256	{}	\N	2020-10-12 16:58:52.805541
87548	PFPT	2020-10-01	105.98	106.845	104.06	104.36	704661	{}	\N	2020-10-12 16:58:52.811848
87549	PFPT	2020-10-02	102.42	104.91	101.225	101.85	718538	{}	\N	2020-10-12 16:58:52.817871
87550	PFPT	2020-10-05	102.61	104.31	101.22	103.62	683642	{}	\N	2020-10-12 16:58:52.823111
87551	PFPT	2020-10-06	103.69	106.45	103.69	104.52	392431	{}	\N	2020-10-12 16:58:52.827976
87552	PFPT	2020-10-07	104.72	109.34	104.5685	108.5	551669	{}	\N	2020-10-12 16:58:52.833351
87553	PFPT	2020-10-08	109	110.14	106	107.84	337846	{}	\N	2020-10-12 16:58:52.839209
87554	PFPT	2020-10-09	108.1	109.77	108.1	109.36	428026	{}	\N	2020-10-12 16:58:52.844373
87555	PFPT	2020-10-12	110.81	112.05	110.2452	111.54	114524	{}	\N	2020-10-12 16:58:52.849899
87556	PGF	2020-09-28	18.72	18.81	18.72	18.77	508613	{}	\N	2020-10-12 16:58:54.120172
87557	PGF	2020-09-29	18.79	18.79	18.71	18.78	340235	{}	\N	2020-10-12 16:58:54.131684
87558	PGF	2020-09-30	18.79	18.81	18.69	18.74	313034	{}	\N	2020-10-12 16:58:54.140357
87559	PGF	2020-10-01	18.76	18.86	18.7	18.84	362534	{}	\N	2020-10-12 16:58:54.146247
87560	PGF	2020-10-02	18.77	18.9	18.7447	18.89	480998	{}	\N	2020-10-12 16:58:54.152342
87561	PGF	2020-10-05	18.91	18.96	18.89	18.95	331018	{}	\N	2020-10-12 16:58:54.157383
87562	PGF	2020-10-06	18.95	18.99	18.904	18.95	398912	{}	\N	2020-10-12 16:58:54.161704
87563	PGF	2020-10-07	18.99	19.02	18.96	18.99	335607	{}	\N	2020-10-12 16:58:54.166674
87564	PGF	2020-10-08	19.01	19.02	18.97	19	519316	{}	\N	2020-10-12 16:58:54.172047
87565	PGF	2020-10-09	18.96	18.9933	18.91	18.97	404176	{}	\N	2020-10-12 16:58:54.176927
87566	PGF	2020-10-12	18.95	18.9897	18.92	18.96	197079	{}	\N	2020-10-12 16:58:54.182092
87567	PGR	2020-09-28	93.56	94.965	93.25	93.98	1780917	{}	\N	2020-10-12 16:58:55.471503
87568	PGR	2020-09-29	94.13	94.77	93.59	94.15	3237528	{}	\N	2020-10-12 16:58:55.479958
87569	PGR	2020-09-30	94.3	94.99	93.49	94.67	5912254	{}	\N	2020-10-12 16:58:55.488436
87570	PGR	2020-10-01	95.08	95.44	92.725	93.46	3889465	{}	\N	2020-10-12 16:58:55.496496
87571	PGR	2020-10-02	92.87	95.3	92.4	94.21	3033421	{}	\N	2020-10-12 16:58:55.504098
87572	PGR	2020-10-05	95.18	96.075	94.28	95.98	2613895	{}	\N	2020-10-12 16:58:55.510872
87573	PGR	2020-10-06	96.1	96.38	94.45	95.12	3782789	{}	\N	2020-10-12 16:58:55.517081
87574	PGR	2020-10-07	95.25	97	95.25	96.41	4394918	{}	\N	2020-10-12 16:58:55.523281
87575	PGR	2020-10-08	96.79	98.02	96.31	97.97	2710825	{}	\N	2020-10-12 16:58:55.528817
87576	PGR	2020-10-09	98.04	99.9	97.68	99.88	3674814	{}	\N	2020-10-12 16:58:55.535383
87577	PGR	2020-10-12	99.99	101.44	99.48	101.29	2136735	{}	\N	2020-10-12 16:58:55.540527
87578	OC	2020-09-28	67.73	69.6	67.73	68.81	1043098	{}	\N	2020-10-12 16:58:57.231261
87579	OC	2020-09-29	69.1	69.1	67.58	67.63	1761497	{}	\N	2020-10-12 16:58:57.243322
87580	OC	2020-09-30	67.82	69.52	67.82	68.81	2136348	{}	\N	2020-10-12 16:58:57.250836
87581	OC	2020-10-01	69.62	70.58	68.93	69.66	1859505	{}	\N	2020-10-12 16:58:57.25717
87582	OC	2020-10-02	67.79	70.99	67.77	70.68	1752576	{}	\N	2020-10-12 16:58:57.261907
87583	OC	2020-10-05	71.77	73.03	71.41	72.76	1528553	{}	\N	2020-10-12 16:58:57.266588
87584	OC	2020-10-06	73.21	75.65	73.07	73.6	2337366	{}	\N	2020-10-12 16:58:57.271801
87585	OC	2020-10-07	74.71	75.5	74.29	75	1673334	{}	\N	2020-10-12 16:58:57.276172
87586	OC	2020-10-08	75.75	76.08	74.28	74.65	1556540	{}	\N	2020-10-12 16:58:57.280508
87587	OC	2020-10-09	75.54	76.08	74.59	74.94	1410206	{}	\N	2020-10-12 16:58:57.284787
87588	OC	2020-10-12	75.74	75.88	75.085	75.37	504386	{}	\N	2020-10-12 16:58:57.289282
87589	OGE	2020-09-28	29.59	29.7	29.285	29.66	1119320	{}	\N	2020-10-12 16:58:58.211399
87590	OGE	2020-09-29	29.77	30.27	29.67	29.67	1757243	{}	\N	2020-10-12 16:58:58.221963
87591	OGE	2020-09-30	29.92	30.12	29.67	29.99	2617812	{}	\N	2020-10-12 16:58:58.23171
87592	OGE	2020-10-01	29.96	30.535	29.91	30.2	2260419	{}	\N	2020-10-12 16:58:58.240476
87593	OGE	2020-10-02	29.8	31.14	29.76	31.06	1794717	{}	\N	2020-10-12 16:58:58.247272
87594	OGE	2020-10-05	31.17	32.18	31.04	32.03	3296134	{}	\N	2020-10-12 16:58:58.254028
87595	OGE	2020-10-06	32.23	32.99	31.9	32.38	3100220	{}	\N	2020-10-12 16:58:58.260317
87596	OGE	2020-10-07	32.38	32.5	31.8	32.04	1965711	{}	\N	2020-10-12 16:58:58.26648
87597	OGE	2020-10-08	32.22	32.775	32.1	32.74	1693796	{}	\N	2020-10-12 16:58:58.27311
87598	OGE	2020-10-09	32.52	32.595	32	32.02	1562043	{}	\N	2020-10-12 16:58:58.279772
87599	OGE	2020-10-12	31.94	32.3	31.84	32.14	417765	{}	\N	2020-10-12 16:58:58.285288
87600	OXLC	2020-09-28	4.3	4.34	4.23	4.33	647033	{}	\N	2020-10-12 16:58:59.553781
87601	OXLC	2020-09-29	4.33	4.4	4.3099	4.39	596208	{}	\N	2020-10-12 16:58:59.569118
87602	OXLC	2020-09-30	4.37	4.49	4.36	4.4	563836	{}	\N	2020-10-12 16:58:59.57738
87603	OXLC	2020-10-01	4.45	4.49	4.4	4.49	501223	{}	\N	2020-10-12 16:58:59.584946
87604	OXLC	2020-10-02	4.31	4.52	4.25	4.47	794637	{}	\N	2020-10-12 16:58:59.591814
87605	OXLC	2020-10-05	4.47	4.57	4.47	4.55	529113	{}	\N	2020-10-12 16:58:59.597603
87606	OXLC	2020-10-06	4.57	4.59	4.47	4.5009	573239	{}	\N	2020-10-12 16:58:59.603082
87607	OXLC	2020-10-07	4.56	4.59	4.51	4.55	364587	{}	\N	2020-10-12 16:58:59.608512
87608	OXLC	2020-10-08	4.57	4.6	4.5325	4.59	663425	{}	\N	2020-10-12 16:58:59.613837
87609	OXLC	2020-10-09	4.62	4.69	4.59	4.68	876291	{}	\N	2020-10-12 16:58:59.61941
87610	OXLC	2020-10-12	4.68	4.7	4.66	4.6889	411799	{}	\N	2020-10-12 16:58:59.624972
87611	PBF	2020-09-28	5.76	6.2	5.72	6.02	8027357	{}	\N	2020-10-12 16:59:01.464273
87612	PBF	2020-09-29	6.02	6.09	5.82	5.89	8710370	{}	\N	2020-10-12 16:59:01.472425
87613	PBF	2020-09-30	5.7	6	5.65	5.69	16189094	{}	\N	2020-10-12 16:59:01.479561
87614	PBF	2020-10-01	5.71	6	5.43	5.58	22429736	{}	\N	2020-10-12 16:59:01.485724
87615	PBF	2020-10-02	5.37	6.06	5.37	5.98	19310715	{}	\N	2020-10-12 16:59:01.491395
87616	PBF	2020-10-05	6.14	6.36	6.085	6.23	9487451	{}	\N	2020-10-12 16:59:01.496965
87617	PBF	2020-10-06	6.36	6.555	6.05	6.07	8155509	{}	\N	2020-10-12 16:59:01.50202
87618	PBF	2020-10-07	6.1	6.24	5.985	6.21	9114658	{}	\N	2020-10-12 16:59:01.507583
87619	PBF	2020-10-08	6.24	6.52	6.13	6.51	6907258	{}	\N	2020-10-12 16:59:01.513202
87620	PBF	2020-10-09	6.6	6.62	6.11	6.17	8589053	{}	\N	2020-10-12 16:59:01.518334
87621	PBF	2020-10-12	6.12	6.16	5.975	6.05	3091149	{}	\N	2020-10-12 16:59:01.523425
87622	PBEE	2020-09-28	24.0598	24.0598	24.0598	24.0598	188	{}	\N	2020-10-12 16:59:05.159695
87623	PBEE	2020-09-29	24.0085	24.0085	24.0085	24.0085	313	{}	\N	2020-10-12 16:59:05.170009
87624	PBEE	2020-09-30	24.4372	24.4372	24.4372	24.4372	132	{}	\N	2020-10-12 16:59:05.176702
87625	PBEE	2020-10-01	24.58	24.79	24.58	24.75	820	{}	\N	2020-10-12 16:59:05.183505
87626	PBEE	2020-10-02	24.445	24.4602	24.445	24.4602	1614	{}	\N	2020-10-12 16:59:05.188666
87627	PBEE	2020-10-05	24.6964	24.6964	24.6964	24.6964	158	{}	\N	2020-10-12 16:59:05.193936
87628	PBEE	2020-10-06	24.783	24.783	24.783	24.783	198	{}	\N	2020-10-12 16:59:05.200504
87629	PBEE	2020-10-07	24.97	25.0781	24.97	25.0781	276	{}	\N	2020-10-12 16:59:05.206372
87630	PBEE	2020-10-08	25.02	25.29	25.02	25.29	504	{}	\N	2020-10-12 16:59:05.211812
87631	PBEE	2020-10-09	25.3598	25.3598	25.3598	25.3598	143	{}	\N	2020-10-12 16:59:05.217517
87632	PETZ	2020-09-28	1.14	1.17	1.09	1.169	5068	{}	\N	2020-10-12 16:59:06.455778
87633	PETZ	2020-09-29	1.2	1.48	1.15	1.48	82847	{}	\N	2020-10-12 16:59:06.466196
87634	PETZ	2020-09-30	1.36	1.3927	1.2601	1.3076	45630	{}	\N	2020-10-12 16:59:06.472263
87635	PETZ	2020-10-01	1.25	1.3396	1.2295	1.295	15569	{}	\N	2020-10-12 16:59:06.478083
87636	PETZ	2020-10-02	1.23	1.29	1.2025	1.29	1665	{}	\N	2020-10-12 16:59:06.48369
87637	PETZ	2020-10-05	1.231	1.231	1.231	1.231	1154	{}	\N	2020-10-12 16:59:06.488538
87638	PETZ	2020-10-06	1.23	1.27	1.23	1.25	5387	{}	\N	2020-10-12 16:59:06.492766
87639	PETZ	2020-10-07	1.22	1.55	1.19	1.48	135462	{}	\N	2020-10-12 16:59:06.497379
87640	PETZ	2020-10-08	1.3435	1.47	1.34	1.34	14855	{}	\N	2020-10-12 16:59:06.501946
87641	PETZ	2020-10-09	1.34	1.43	1.34	1.38	1846	{}	\N	2020-10-12 16:59:06.506568
87642	PETZ	2020-10-12	1.29	1.38	1.29	1.375	4497	{}	\N	2020-10-12 16:59:06.510739
87643	PEY	2020-09-28	14.28	14.48	14.28	14.4	169182	{}	\N	2020-10-12 16:59:08.940521
87644	PEY	2020-09-29	14.4	14.406	14.13	14.235	217277	{}	\N	2020-10-12 16:59:08.950671
87645	PEY	2020-09-30	14.32	14.46	14.24	14.36	214432	{}	\N	2020-10-12 16:59:08.958046
87646	PEY	2020-10-01	14.37	14.4194	14.22	14.33	353040	{}	\N	2020-10-12 16:59:08.964077
87647	PEY	2020-10-02	14.14	14.66	14.14	14.61	239707	{}	\N	2020-10-12 16:59:08.96987
87648	PEY	2020-10-05	14.7	14.895	14.7	14.87	206845	{}	\N	2020-10-12 16:59:08.974724
87649	PEY	2020-10-06	14.96	15.2	14.83	14.86	806843	{}	\N	2020-10-12 16:59:08.979511
87650	PEY	2020-10-07	14.97	15.16	14.97	15.11	398378	{}	\N	2020-10-12 16:59:08.984373
87651	PEY	2020-10-08	15.18	15.3291	15.18	15.32	209663	{}	\N	2020-10-12 16:59:08.989139
87652	PEY	2020-10-09	15.4	15.48	15.235	15.24	178923	{}	\N	2020-10-12 16:59:08.99374
87653	PEY	2020-10-12	15.3	15.41	15.2401	15.3831	160286	{}	\N	2020-10-12 16:59:08.998451
87654	PGHY	2020-09-28	21.51	21.5873	21.36	21.53	169289	{}	\N	2020-10-12 16:59:09.92129
87655	PGHY	2020-09-29	21.52	21.6265	21.4	21.58	29495	{}	\N	2020-10-12 16:59:09.932369
87656	PGHY	2020-09-30	21.5655	21.69	21.5601	21.615	20436	{}	\N	2020-10-12 16:59:09.939787
87657	PGHY	2020-10-01	21.69	21.69	21.49	21.56	46291	{}	\N	2020-10-12 16:59:09.945898
87658	PGHY	2020-10-02	21.64	21.77	21.58	21.6875	39755	{}	\N	2020-10-12 16:59:09.95189
87659	PGHY	2020-10-05	21.75	21.75	21.6	21.74	26290	{}	\N	2020-10-12 16:59:09.957364
87660	PGHY	2020-10-06	21.73	21.75	21.55	21.61	112967	{}	\N	2020-10-12 16:59:09.962507
87661	PGHY	2020-10-07	21.56	21.74	21.56	21.74	28638	{}	\N	2020-10-12 16:59:09.96769
87662	PGHY	2020-10-08	21.74	21.8	21.66	21.79	35255	{}	\N	2020-10-12 16:59:09.973269
87663	PGHY	2020-10-09	21.93	21.93	21.7	21.83	38470	{}	\N	2020-10-12 16:59:09.978485
87664	PGHY	2020-10-12	21.77	21.8499	21.77	21.7998	7282	{}	\N	2020-10-12 16:59:09.983789
87665	HIX	2020-09-28	6.43	6.44	6.4	6.44	370946	{}	\N	2020-10-12 16:59:11.469257
87666	HIX	2020-09-29	6.42	6.44	6.38	6.4	736200	{}	\N	2020-10-12 16:59:11.480655
87667	HIX	2020-09-30	6.41	6.45	6.38	6.4	1872735	{}	\N	2020-10-12 16:59:11.487762
87668	HIX	2020-10-01	6.43	6.45	6.41	6.42	1194342	{}	\N	2020-10-12 16:59:11.493998
87669	HIX	2020-10-02	6.38	6.45	6.38	6.43	324400	{}	\N	2020-10-12 16:59:11.500099
87670	HIX	2020-10-05	6.43	6.49	6.4218	6.48	222356	{}	\N	2020-10-12 16:59:11.505442
87671	HIX	2020-10-06	6.48	6.52	6.47	6.49	216325	{}	\N	2020-10-12 16:59:11.510601
87672	HIX	2020-10-07	6.52	6.5399	6.5	6.5	153262	{}	\N	2020-10-12 16:59:11.515421
87673	HIX	2020-10-08	6.53	6.5799	6.525	6.56	228996	{}	\N	2020-10-12 16:59:11.521232
87674	HIX	2020-10-09	6.55	6.59	6.55	6.56	366857	{}	\N	2020-10-12 16:59:11.52624
87675	HIX	2020-10-12	6.59	6.6	6.5773	6.58	138280	{}	\N	2020-10-12 16:59:11.531196
87676	HP	2020-09-28	14.58	15.04	14.5	14.86	1627465	{}	\N	2020-10-12 16:59:12.656945
87677	HP	2020-09-29	14.69	14.8	14.1	14.57	3667073	{}	\N	2020-10-12 16:59:12.667159
87678	HP	2020-09-30	14.73	15.28	14.335	14.65	5602529	{}	\N	2020-10-12 16:59:12.673913
87679	HP	2020-10-01	14.33	14.39	13.31	13.48	7713627	{}	\N	2020-10-12 16:59:12.679688
87680	HP	2020-10-02	12.93	14.105	12.87	14.02	3824136	{}	\N	2020-10-12 16:59:12.685604
87681	HP	2020-10-05	14.59	14.8417	14.24	14.56	2684022	{}	\N	2020-10-12 16:59:12.692492
87682	HP	2020-10-06	14.93	15.16	14.13	14.39	3686931	{}	\N	2020-10-12 16:59:12.700392
87683	HP	2020-10-07	14.35	14.695	14.08	14.61	3034193	{}	\N	2020-10-12 16:59:12.705977
87684	HP	2020-10-08	14.88	15.47	14.73	15.46	3235752	{}	\N	2020-10-12 16:59:12.712329
87685	HP	2020-10-09	15.6	15.62	14.85	15.02	2769078	{}	\N	2020-10-12 16:59:12.718362
87686	HP	2020-10-12	14.85	15.01	14.74	14.89	780650	{}	\N	2020-10-12 16:59:12.724282
87687	IBDP	2020-09-28	26.39	26.46	26.39	26.46	98907	{}	\N	2020-10-12 16:59:13.76444
87688	IBDP	2020-09-29	26.47	26.47	26.4193	26.45	258706	{}	\N	2020-10-12 16:59:13.775438
87689	IBDP	2020-09-30	26.45	26.46	26.4201	26.445	93390	{}	\N	2020-10-12 16:59:13.782312
87690	IBDP	2020-10-01	26.43	26.43	26.27	26.37	130294	{}	\N	2020-10-12 16:59:13.788896
87691	IBDP	2020-10-02	26.41	26.42	26.368	26.405	125034	{}	\N	2020-10-12 16:59:13.794734
87692	IBDP	2020-10-05	26.43	26.51	26.34	26.34	150382	{}	\N	2020-10-12 16:59:13.800412
87693	IBDP	2020-10-06	26.37	26.43	26.3698	26.4	469717	{}	\N	2020-10-12 16:59:13.806128
87694	IBDP	2020-10-07	26.43	26.439	26.361	26.41	161846	{}	\N	2020-10-12 16:59:13.812473
87695	IBDP	2020-10-08	26.42	26.44	26.41	26.41	124454	{}	\N	2020-10-12 16:59:13.817917
87696	IBDP	2020-10-09	26.4	26.45	26.4	26.4	75559	{}	\N	2020-10-12 16:59:13.823744
87697	IBDP	2020-10-12	26.4	26.51	26.4	26.48	44203	{}	\N	2020-10-12 16:59:13.829811
87698	IBP	2020-09-28	97.75	101.07	96.85	100.67	176145	{}	\N	2020-10-12 16:59:15.144577
87699	IBP	2020-09-29	100.7	101.92	99.555	100.32	275819	{}	\N	2020-10-12 16:59:15.156783
87700	IBP	2020-09-30	100.82	103.51	100.648	101.75	433177	{}	\N	2020-10-12 16:59:15.163871
87701	IBP	2020-10-01	102.81	107.09	101.28	106.93	377983	{}	\N	2020-10-12 16:59:15.169513
87702	IBP	2020-10-02	104.21	110.07	103.59	109.05	368164	{}	\N	2020-10-12 16:59:15.174957
87703	IBP	2020-10-05	107.67	110	105.33	108.71	549234	{}	\N	2020-10-12 16:59:15.179743
87704	IBP	2020-10-06	109.93	110.69	108.1	109.2	549782	{}	\N	2020-10-12 16:59:15.184627
87705	IBP	2020-10-07	111.8	112.6	108.945	110.99	501276	{}	\N	2020-10-12 16:59:15.190423
87706	IBP	2020-10-08	112.66	114.7	111.64	113.91	331789	{}	\N	2020-10-12 16:59:15.195282
87707	IBP	2020-10-09	115.39	118.75	114.64	118.71	421911	{}	\N	2020-10-12 16:59:15.19934
87708	IBP	2020-10-12	119.26	121.67	118.53	120.86	137100	{}	\N	2020-10-12 16:59:15.203537
87709	INFR	2020-09-28	27.67	27.8199	27.6298	27.7221	3992	{}	\N	2020-10-12 16:59:17.040403
87710	INFR	2020-10-01	27.8	27.8	27.545	27.5645	1723	{}	\N	2020-10-12 16:59:17.049753
87711	INFR	2020-10-02	27.67	28.05	27.67	28.05	1640	{}	\N	2020-10-12 16:59:17.056124
87712	INFR	2020-10-05	28.15	28.1799	28.04	28.0716	19691	{}	\N	2020-10-12 16:59:17.06125
87713	INFR	2020-10-06	28.02	28.16	28.02	28.02	2419	{}	\N	2020-10-12 16:59:17.065635
87714	INFR	2020-10-07	28.1402	28.355	28.14	28.355	1121	{}	\N	2020-10-12 16:59:17.07013
87715	INFR	2020-10-08	28.24	28.47	28.24	28.339	2898	{}	\N	2020-10-12 16:59:17.074246
87716	INFR	2020-10-09	28.16	28.205	28.16	28.205	997	{}	\N	2020-10-12 16:59:17.078377
87717	IPAY	2020-09-28	54.21	54.68	54.16	54.31	118301	{}	\N	2020-10-12 16:59:18.789439
87718	IPAY	2020-09-29	54.33	54.8209	54.31	54.43	61442	{}	\N	2020-10-12 16:59:18.800869
87719	IPAY	2020-09-30	54.33	54.889	53.96	54.4	90958	{}	\N	2020-10-12 16:59:18.807295
87720	IPAY	2020-10-01	54.78	55.31	54.66	55.15	139977	{}	\N	2020-10-12 16:59:18.813604
87721	IPAY	2020-10-02	54.26	54.925	54.033	54.73	104581	{}	\N	2020-10-12 16:59:18.818889
87722	IPAY	2020-10-05	55.29	55.91	55.29	55.9	89993	{}	\N	2020-10-12 16:59:18.824226
87723	IPAY	2020-10-06	56	56.6	55.0604	55.22	100129	{}	\N	2020-10-12 16:59:18.82984
87724	IPAY	2020-10-07	55.75	56.02	55.7	55.85	114379	{}	\N	2020-10-12 16:59:18.834886
87725	IPAY	2020-10-08	56.36	56.95	56.36	56.95	127713	{}	\N	2020-10-12 16:59:18.840431
87726	IPAY	2020-10-09	57.18	57.73	57.135	57.62	103357	{}	\N	2020-10-12 16:59:18.845264
87727	IPAY	2020-10-12	58.08	58.27	57.75	58.2	64720	{}	\N	2020-10-12 16:59:18.850196
87728	IQV	2020-09-28	153.5	156.99	153.5	155.31	904523	{}	\N	2020-10-12 16:59:20.862179
87729	IQV	2020-09-29	155.25	158.05	154.61	154.78	1660833	{}	\N	2020-10-12 16:59:20.8747
87730	IQV	2020-09-30	155.44	158.97	155	157.63	1534021	{}	\N	2020-10-12 16:59:20.881899
87731	IQV	2020-10-01	159.16	160.8	157.475	158.33	1530858	{}	\N	2020-10-12 16:59:20.887957
87732	IQV	2020-10-02	156.41	158.38	155.81	156.83	1025462	{}	\N	2020-10-12 16:59:20.893862
87733	IQV	2020-10-05	158.6	161.65	157.81	160.62	1120723	{}	\N	2020-10-12 16:59:20.899056
87734	IQV	2020-10-06	161.15	162.92	159.4	159.4	893662	{}	\N	2020-10-12 16:59:20.904023
87735	IQV	2020-10-07	160.32	164.72	160.32	164.19	1139729	{}	\N	2020-10-12 16:59:20.90907
87736	IQV	2020-10-08	164.57	166.105	163.83	164.49	1116938	{}	\N	2020-10-12 16:59:20.914216
87737	IQV	2020-10-09	165.38	167.54	164.97	166.76	1800842	{}	\N	2020-10-12 16:59:20.919098
87738	IQV	2020-10-12	167.94	171.5	166.49	171.45	894887	{}	\N	2020-10-12 16:59:20.923992
87739	IVW	2020-09-28	229.98	230.72	228.95	230.49	406609	{}	\N	2020-10-12 16:59:22.839211
87740	IVW	2020-09-29	230.48	231.2	229.24	229.65	285854	{}	\N	2020-10-12 16:59:22.852054
87741	IVW	2020-09-30	229.99	233.67	229.85	231.08	420100	{}	\N	2020-10-12 16:59:22.859516
87742	IVW	2020-10-01	233.72	234.28	232.11	233.39	644281	{}	\N	2020-10-12 16:59:22.866876
87743	IVW	2020-10-02	229.08	231.85	228.49	229.49	498586	{}	\N	2020-10-12 16:59:22.873521
87744	IVW	2020-10-05	231	234.1	231	233.94	354034	{}	\N	2020-10-12 16:59:22.879503
87745	IVW	2020-10-06	233.6	235.0378	229.56	230.07	359902	{}	\N	2020-10-12 16:59:22.885084
87746	IVW	2020-10-07	232.71	234.62	232.22	234.21	379752	{}	\N	2020-10-12 16:59:22.890717
87747	IVW	2020-10-08	235.66	235.77	234.7	235.59	297013	{}	\N	2020-10-12 16:59:22.896065
87748	IVW	2020-10-09	236.78	238.82	236.6	238.72	237612	{}	\N	2020-10-12 16:59:22.902064
87749	IVW	2020-10-12	241.45	244.85	241.0354	244.56	211479	{}	\N	2020-10-12 16:59:22.906896
87750	JDST	2020-09-28	11.33	11.73	11.175	11.34	6371188	{}	\N	2020-10-12 16:59:24.125583
87751	JDST	2020-09-29	11.28	11.34	10.9003	11.03	5527487	{}	\N	2020-10-12 16:59:24.136898
87752	JDST	2020-09-30	11.28	11.52	11.0311	11.29	5961993	{}	\N	2020-10-12 16:59:24.143379
87753	JDST	2020-10-01	10.99	11.2	10.65	10.92	6323345	{}	\N	2020-10-12 16:59:24.148948
87754	JDST	2020-10-02	11.2	11.435	10.9101	11.25	5239504	{}	\N	2020-10-12 16:59:24.154177
87755	JDST	2020-10-05	11.04	11.1199	10.61	10.77	5451998	{}	\N	2020-10-12 16:59:24.158906
87756	JDST	2020-10-06	10.56	11.57	10.435	11.51	7855002	{}	\N	2020-10-12 16:59:24.163189
87757	JDST	2020-10-07	11.16	11.42	10.9599	11.25	5489758	{}	\N	2020-10-12 16:59:24.167525
87758	JDST	2020-10-08	11.01	11.14	10.69	10.74	6317208	{}	\N	2020-10-12 16:59:24.172074
87759	JDST	2020-10-09	10.23	10.26	9.58	9.62	10297732	{}	\N	2020-10-12 16:59:24.176475
87760	JDST	2020-10-12	9.76	9.91	9.5	9.53	2566301	{}	\N	2020-10-12 16:59:24.180922
87761	PGX	2020-09-28	14.78	14.83	14.76	14.81	3264858	{}	\N	2020-10-12 16:59:25.582586
87762	PGX	2020-09-29	14.81	14.82	14.76	14.8	2633245	{}	\N	2020-10-12 16:59:25.594401
87763	PGX	2020-09-30	14.82	14.83	14.72	14.73	2447954	{}	\N	2020-10-12 16:59:25.601006
87764	PGX	2020-10-01	14.8	14.87	14.74	14.86	5619216	{}	\N	2020-10-12 16:59:25.607451
87765	PGX	2020-10-02	14.75	14.88	14.75	14.87	3193958	{}	\N	2020-10-12 16:59:25.614061
87766	PGX	2020-10-05	14.89	14.93	14.8736	14.9	2736005	{}	\N	2020-10-12 16:59:25.62049
87767	PGX	2020-10-06	14.92	14.97	14.89	14.92	3358307	{}	\N	2020-10-12 16:59:25.625883
87768	PGX	2020-10-07	14.97	14.98	14.93	14.955	1724637	{}	\N	2020-10-12 16:59:25.630994
87769	PGX	2020-10-08	14.99	15.02	14.9599	14.97	2088033	{}	\N	2020-10-12 16:59:25.636554
87770	PGX	2020-10-09	15	15	14.91	14.95	3024003	{}	\N	2020-10-12 16:59:25.642347
87771	PGX	2020-10-12	14.97	14.9822	14.93	14.975	1026111	{}	\N	2020-10-12 16:59:25.649058
87772	PHO	2020-09-28	39.21	39.589	39.21	39.41	27476	{}	\N	2020-10-12 16:59:26.622136
87773	PHO	2020-09-29	39.62	39.6451	39.275	39.49	26386	{}	\N	2020-10-12 16:59:26.631957
87774	PHO	2020-09-30	39.36	40.05	39.36	39.68	31812	{}	\N	2020-10-12 16:59:26.638746
87775	PHO	2020-10-01	39.76	40.08	39.5501	39.82	23942	{}	\N	2020-10-12 16:59:26.644458
87776	PHO	2020-10-02	39.27	40.2899	39.17	40.05	31790	{}	\N	2020-10-12 16:59:26.64994
87777	PHO	2020-10-05	40.2458	40.73	40.2458	40.73	104194	{}	\N	2020-10-12 16:59:26.654864
87778	PHO	2020-10-06	40.79	41.34	40.64	40.64	31198	{}	\N	2020-10-12 16:59:26.659876
87779	PHO	2020-10-07	40.95	41.37	40.86	41.3	40258	{}	\N	2020-10-12 16:59:26.66482
87780	PHO	2020-10-08	41.53	41.6492	41.36	41.6123	71440	{}	\N	2020-10-12 16:59:26.669757
87781	PHO	2020-10-09	41.7	42.19	41.7	42.07	54764	{}	\N	2020-10-12 16:59:26.674744
87782	PHO	2020-10-12	42.34	42.76	42.34	42.72	72590	{}	\N	2020-10-12 16:59:26.679598
87783	PIH	2020-09-28	4.01	4.1536	3.85	3.85	3532	{}	\N	2020-10-12 16:59:28.439742
87784	PIH	2020-09-29	4.045	4.045	3.88	3.9	514	{}	\N	2020-10-12 16:59:28.450849
87785	PIH	2020-09-30	3.95	4.05	3.85	3.85	605	{}	\N	2020-10-12 16:59:28.457692
87786	PIH	2020-10-01	3.85	3.85	3.76	3.76	2020	{}	\N	2020-10-12 16:59:28.463443
87787	PIH	2020-10-02	3.84	3.84	3.84	3.84	158	{}	\N	2020-10-12 16:59:28.469375
87788	PIH	2020-10-05	3.83	4.05	3.7834	3.81	1839	{}	\N	2020-10-12 16:59:28.474805
87789	PIH	2020-10-06	3.71	4.04	3.69	3.75	1279	{}	\N	2020-10-12 16:59:28.479864
87790	PIH	2020-10-07	3.77	3.77	3.77	3.77	328	{}	\N	2020-10-12 16:59:28.484875
87791	PIH	2020-10-09	3.8309	3.89	3.75	3.75	2166	{}	\N	2020-10-12 16:59:28.489774
87792	PIH	2020-10-12	3.76	3.76	3.7503	3.7503	1013	{}	\N	2020-10-12 16:59:28.494588
87793	PSCI	2020-09-28	62.25	63.37	62.25	63.17	1321	{}	\N	2020-10-12 16:59:29.515366
87794	PSCI	2020-09-29	63.32	63.32	62.7963	62.7963	91669	{}	\N	2020-10-12 16:59:29.534078
87795	PSCI	2020-09-30	63.43	63.5701	62.5646	62.5646	707	{}	\N	2020-10-12 16:59:29.54213
87796	PSCI	2020-10-01	62.895	62.895	62.895	62.895	3348	{}	\N	2020-10-12 16:59:29.548557
87797	PSCI	2020-10-02	62.33	63.7524	62.33	63.7524	1740	{}	\N	2020-10-12 16:59:29.554318
87798	PSCI	2020-10-05	64.81	65.0527	64.66	65.0527	1347	{}	\N	2020-10-12 16:59:29.560376
87799	PSCI	2020-10-06	65.78	66.4243	65.26	65.26	4111	{}	\N	2020-10-12 16:59:29.565837
87800	PSCI	2020-10-07	66.46	66.46	65.79	66.2473	4592	{}	\N	2020-10-12 16:59:29.571517
87801	PSCI	2020-10-08	67.31	67.39	66.5	67.22	19258	{}	\N	2020-10-12 16:59:29.576935
87802	PSCI	2020-10-09	67.53	67.95	67.5136	67.5136	2252	{}	\N	2020-10-12 16:59:29.585158
87803	PSCI	2020-10-12	67.93	67.93	67.93	67.93	862	{}	\N	2020-10-12 16:59:29.592606
87804	QEMM	2020-09-28	56.04	56.04	55.701	55.84	4642	{}	\N	2020-10-12 16:59:30.794968
87805	QEMM	2020-09-29	55.9	55.99	55.72	55.9722	3357	{}	\N	2020-10-12 16:59:30.8053
87806	QEMM	2020-09-30	56.1	56.5599	56.1	56.49	2078	{}	\N	2020-10-12 16:59:30.811961
87807	QEMM	2020-10-01	56.7107	57.01	56.7107	57.01	4936	{}	\N	2020-10-12 16:59:30.818235
87808	QEMM	2020-10-02	56.72	56.72	56.39	56.39	1428	{}	\N	2020-10-12 16:59:30.823463
87809	QEMM	2020-10-05	57.145	57.26	57.0347	57.26	2535	{}	\N	2020-10-12 16:59:30.831248
87810	QEMM	2020-10-06	57.28	57.61	57.11	57.11	2935	{}	\N	2020-10-12 16:59:30.837334
87811	QEMM	2020-10-07	57.665	57.83	57.61	57.71	219106	{}	\N	2020-10-12 16:59:30.845051
87812	QEMM	2020-10-08	57.91	58.205	57.9	58.11	14817	{}	\N	2020-10-12 16:59:30.850828
87813	QEMM	2020-10-09	58.11	58.73	58.11	58.605	6878	{}	\N	2020-10-12 16:59:30.856637
87814	QTS	2020-09-28	63.3	64.22	63.3	64.14	382223	{}	\N	2020-10-12 16:59:32.745135
87815	QTS	2020-09-29	64.58	65.21	63.49	63.93	809746	{}	\N	2020-10-12 16:59:32.753002
87816	QTS	2020-09-30	64.21	64.51	62.65	63.02	1447468	{}	\N	2020-10-12 16:59:32.769954
87817	QTS	2020-10-01	63	63.43	62.29	63.37	1098764	{}	\N	2020-10-12 16:59:32.777809
87818	QTS	2020-10-02	62.84	64.5	62.84	64.13	562998	{}	\N	2020-10-12 16:59:32.783929
87819	QTS	2020-10-05	64.45	65.92	63.81	65.7	1713359	{}	\N	2020-10-12 16:59:32.789882
87820	QTS	2020-10-06	66.02	66.215	64.16	64.48	1153394	{}	\N	2020-10-12 16:59:32.794953
87821	QTS	2020-10-07	64.64	65.89	64.64	65.43	1046103	{}	\N	2020-10-12 16:59:32.800005
87822	QTS	2020-10-08	65.86	66.95	65.52	66.67	852618	{}	\N	2020-10-12 16:59:32.80484
87823	QTS	2020-10-09	67.12	67.48	66.4	66.92	948127	{}	\N	2020-10-12 16:59:32.810129
87824	QTS	2020-10-12	66.95	67.77	66.57	67.41	132316	{}	\N	2020-10-12 16:59:32.814785
87825	RDUS	2020-09-28	11.11	11.11	10.1511	10.8	1254552	{}	\N	2020-10-12 16:59:34.131894
87826	RDUS	2020-09-29	10.8	11.31	10.72	11.15	1112215	{}	\N	2020-10-12 16:59:34.1448
87827	RDUS	2020-09-30	11.17	11.66	11.17	11.34	560896	{}	\N	2020-10-12 16:59:34.151685
87828	RDUS	2020-10-01	11.35	11.73	11.34	11.55	335276	{}	\N	2020-10-12 16:59:34.157614
87829	RDUS	2020-10-02	11.38	11.5	10.995	11.33	378271	{}	\N	2020-10-12 16:59:34.162886
87830	RDUS	2020-10-05	11.49	12.23	11.3	12.18	485362	{}	\N	2020-10-12 16:59:34.167924
87831	RDUS	2020-10-06	12.3	12.7	12.09	12.29	740369	{}	\N	2020-10-12 16:59:34.172704
87832	RDUS	2020-10-07	12.35	13.31	12.17	13.26	718468	{}	\N	2020-10-12 16:59:34.177236
87833	RDUS	2020-10-08	13.43	13.53	12.87	13.5	448615	{}	\N	2020-10-12 16:59:34.182115
87834	RDUS	2020-10-09	13.61	13.89	13.15	13.86	475995	{}	\N	2020-10-12 16:59:34.186641
87835	RDUS	2020-10-12	13.9	13.97	13.37	13.62	82094	{}	\N	2020-10-12 16:59:34.191431
87836	RENN	2020-09-28	3.64	4.16	3.5501	4.02	416515	{}	\N	2020-10-12 16:59:35.255479
87837	RENN	2020-09-29	4	4.1399	3.71	3.97	445383	{}	\N	2020-10-12 16:59:35.266609
87838	RENN	2020-09-30	3.92	4.125	3.63	4.01	419100	{}	\N	2020-10-12 16:59:35.274016
87839	RENN	2020-10-01	4.02	4.13	3.98	4.05	170347	{}	\N	2020-10-12 16:59:35.280322
87840	RENN	2020-10-02	3.83	4.09	3.8101	4.02	266202	{}	\N	2020-10-12 16:59:35.286131
87841	RENN	2020-10-05	3.99	4.0899	3.66	3.77	449492	{}	\N	2020-10-12 16:59:35.291388
87842	RENN	2020-10-06	3.69	4.155	3.69	4.01	694210	{}	\N	2020-10-12 16:59:35.296031
87843	RENN	2020-10-07	4.08	4.84	3.95	4.78	956434	{}	\N	2020-10-12 16:59:35.300918
87844	RENN	2020-10-08	4.65	4.8299	4.37	4.6	321827	{}	\N	2020-10-12 16:59:35.305653
87845	RENN	2020-10-09	4.48	4.78	4.16	4.16	399319	{}	\N	2020-10-12 16:59:35.310408
87846	RENN	2020-10-12	4.11	4.52	4.07	4.2401	405108	{}	\N	2020-10-12 16:59:35.314933
87847	RGEN	2020-09-28	145.64	147.545	144.36	147.28	213991	{}	\N	2020-10-12 16:59:37.739012
87848	RGEN	2020-09-29	147.94	152.41	147.91	150.59	247547	{}	\N	2020-10-12 16:59:37.751545
87849	RGEN	2020-09-30	149.98	150.5	145.99	147.54	601734	{}	\N	2020-10-12 16:59:37.757546
87850	RGEN	2020-10-01	148	150.89	146.52	149.63	246355	{}	\N	2020-10-12 16:59:37.763411
87851	RGEN	2020-10-02	147.69	150.88	146.97	148.08	539163	{}	\N	2020-10-12 16:59:37.768393
87852	RGEN	2020-10-05	149.58	159.98	149.55	159.8	855598	{}	\N	2020-10-12 16:59:37.772657
87853	RGEN	2020-10-06	160.72	165.44	160.72	163.72	585223	{}	\N	2020-10-12 16:59:37.777505
87854	RGEN	2020-10-07	166.34	167.73	164.72	165.03	470795	{}	\N	2020-10-12 16:59:37.78246
87855	RGEN	2020-10-08	166.32	167.64	161.87	164.18	408489	{}	\N	2020-10-12 16:59:37.788067
87856	RGEN	2020-10-09	165.32	171.9897	163.7676	170.5	363649	{}	\N	2020-10-12 16:59:37.793495
87857	RGEN	2020-10-12	173.03	175.45	170.7291	174.41	183079	{}	\N	2020-10-12 16:59:37.79869
87858	RIGS	2020-09-28	24.43	24.59	24.385	24.5237	10420	{}	\N	2020-10-12 16:59:39.431679
87859	RIGS	2020-09-29	24.44	24.59	24.428	24.59	4849	{}	\N	2020-10-12 16:59:39.442671
87860	RIGS	2020-09-30	24.5303	24.599	24.18	24.52	90961	{}	\N	2020-10-12 16:59:39.449515
87861	RIGS	2020-10-01	24.58	24.6	24.4901	24.54	9166	{}	\N	2020-10-12 16:59:39.455646
87862	RIGS	2020-10-02	24.5599	24.5599	24.4901	24.55	55338	{}	\N	2020-10-12 16:59:39.461564
87863	RIGS	2020-10-05	24.5	24.6	24.4	24.6	25643	{}	\N	2020-10-12 16:59:39.466509
87864	RIGS	2020-10-06	24.58	24.6116	24.53	24.55	18085	{}	\N	2020-10-12 16:59:39.471933
87865	RIGS	2020-10-07	24.565	24.74	24.5201	24.62	39123	{}	\N	2020-10-12 16:59:39.477382
87866	RIGS	2020-10-08	24.6603	24.75	24.57	24.57	33324	{}	\N	2020-10-12 16:59:39.482978
87867	RIGS	2020-10-09	24.6	24.75	24.5243	24.75	21186	{}	\N	2020-10-12 16:59:39.487999
87868	RIGS	2020-10-12	24.7	24.7394	24.6913	24.715	5564	{}	\N	2020-10-12 16:59:39.492729
87869	SCHX	2020-09-28	80.15	80.575	79.94	80.39	784880	{}	\N	2020-10-12 16:59:41.159487
87870	SCHX	2020-09-29	80.36	80.54	79.85	80.03	818472	{}	\N	2020-10-12 16:59:41.17091
87871	SCHX	2020-09-30	80.19	81.3901	80.12	80.63	1366753	{}	\N	2020-10-12 16:59:41.177709
87872	SCHX	2020-10-01	81.25	81.53	80.765	81.26	765871	{}	\N	2020-10-12 16:59:41.183949
87873	SCHX	2020-10-02	79.83	80.96	79.67	80.49	734483	{}	\N	2020-10-12 16:59:41.189865
87874	SCHX	2020-10-05	81.04	81.9558	81.0373	81.94	652151	{}	\N	2020-10-12 16:59:41.194694
87875	SCHX	2020-10-06	82	82.525	80.69	80.85	861418	{}	\N	2020-10-12 16:59:41.199992
87876	SCHX	2020-10-07	81.6	82.43	81.6	82.23	555454	{}	\N	2020-10-12 16:59:41.20508
87877	SCHX	2020-10-08	82.75	82.94	82.5	82.92	678075	{}	\N	2020-10-12 16:59:41.210413
87878	SCHX	2020-10-09	83.36	83.8	83.2438	83.65	805859	{}	\N	2020-10-12 16:59:41.216144
87879	SCHX	2020-10-12	84.29	85.1401	84.22	85.075	460387	{}	\N	2020-10-12 16:59:41.221246
87880	SECT	2020-09-28	30.46	30.91	30.46	30.81	59794	{}	\N	2020-10-12 16:59:42.584146
87881	SECT	2020-09-29	30.91	30.95	30.6861	30.7	38551	{}	\N	2020-10-12 16:59:42.595118
87882	SECT	2020-09-30	31.0016	31.27	30.83	31.05	35406	{}	\N	2020-10-12 16:59:42.602156
87883	SECT	2020-10-01	31.23	31.3099	31.09	31.26	38055	{}	\N	2020-10-12 16:59:42.608247
87884	SECT	2020-10-02	30.87	31.14	30.87	31.14	62325	{}	\N	2020-10-12 16:59:42.614524
87885	SECT	2020-10-05	31.4499	31.56	31.3401	31.5	27345	{}	\N	2020-10-12 16:59:42.619774
87886	SECT	2020-10-06	31.96	31.96	31.05	31.16	73330	{}	\N	2020-10-12 16:59:42.625258
87887	SECT	2020-10-07	31.2836	31.73	31.2836	31.66	78313	{}	\N	2020-10-12 16:59:42.630357
87888	SECT	2020-10-08	31.8912	31.96	31.82	31.94	50875	{}	\N	2020-10-12 16:59:42.636375
87889	SECT	2020-10-09	31.98	32.2599	31.98	32.22	42311	{}	\N	2020-10-12 16:59:42.642028
87890	SECT	2020-10-12	32.42	32.73	32.392	32.73	2512	{}	\N	2020-10-12 16:59:42.647269
87891	PICO	2020-09-28	8.98	9.1035	8.98	9.01	33558	{}	\N	2020-10-12 16:59:44.616979
87892	PICO	2020-09-29	8.97	8.99	8.9	8.97	44619	{}	\N	2020-10-12 16:59:44.626584
87893	PICO	2020-09-30	8.9368	9.01	8.92	8.96	44995	{}	\N	2020-10-12 16:59:44.634302
87894	PICO	2020-10-01	8.97	8.97	8.81	8.89	31171	{}	\N	2020-10-12 16:59:44.640812
87895	PICO	2020-10-02	8.69	9.06	8.69	8.99	29346	{}	\N	2020-10-12 16:59:44.6468
87896	PICO	2020-10-05	9.02	9.1	9.02	9.04	18770	{}	\N	2020-10-12 16:59:44.652852
87897	PICO	2020-10-06	9.16	9.29	8.96	9.02	27052	{}	\N	2020-10-12 16:59:44.658694
87898	PICO	2020-10-07	9.17	9.445	9.04	9.09	48263	{}	\N	2020-10-12 16:59:44.664937
87899	PICO	2020-10-08	9.19	9.58	9.09	9.38	55905	{}	\N	2020-10-12 16:59:44.670551
87900	PICO	2020-10-09	9.46	9.47	9.15	9.19	37883	{}	\N	2020-10-12 16:59:44.676212
87901	PICO	2020-10-12	9.2	9.29	9.2	9.25	5562	{}	\N	2020-10-12 16:59:44.681858
87902	PILL	2020-09-28	15.21	15.21	14.7	14.91	27652	{}	\N	2020-10-12 16:59:46.953998
87903	PILL	2020-09-29	14.65	14.955	14.65	14.87	6783	{}	\N	2020-10-12 16:59:46.964689
87904	PILL	2020-09-30	14.92	15.7962	14.82	15.1412	21189	{}	\N	2020-10-12 16:59:46.972361
87905	PILL	2020-10-01	15.66	15.8	15.37	15.63	35789	{}	\N	2020-10-12 16:59:46.978616
87906	PILL	2020-10-02	14.77	15.57	14.75	15.2683	31197	{}	\N	2020-10-12 16:59:46.98428
87907	PILL	2020-10-05	16.6	17.54	16.6	17.52	100376	{}	\N	2020-10-12 16:59:46.989498
87908	PILL	2020-10-06	17.53	18.1006	17.15	17.18	34213	{}	\N	2020-10-12 16:59:46.994302
87909	PILL	2020-10-07	17.55	18.45	17.51	18.31	38862	{}	\N	2020-10-12 16:59:46.999264
87910	PILL	2020-10-08	18.68	19.59	18.68	19.52	60290	{}	\N	2020-10-12 16:59:47.00464
87911	PILL	2020-10-09	19.9107	20.12	19.55	19.78	92153	{}	\N	2020-10-12 16:59:47.009515
87912	PILL	2020-10-12	19.95	19.95	19.2802	19.7	36200	{}	\N	2020-10-12 16:59:47.014734
87913	PNFP	2020-09-28	35.65	36.25	35.37	35.56	659446	{}	\N	2020-10-12 16:59:49.974624
87914	PNFP	2020-09-29	35.73	35.73	34.11	34.84	576735	{}	\N	2020-10-12 16:59:49.986945
87915	PNFP	2020-09-30	34.94	35.98	34.94	35.59	489798	{}	\N	2020-10-12 16:59:49.994532
87916	PNFP	2020-10-01	35.64	36.29	35.24	35.97	397060	{}	\N	2020-10-12 16:59:50.001605
87917	PNFP	2020-10-02	35.2	37.85	35.2	37.55	556982	{}	\N	2020-10-12 16:59:50.007951
87918	PNFP	2020-10-05	37.97	39.29	37.97	38.64	579971	{}	\N	2020-10-12 16:59:50.013519
87919	PNFP	2020-10-06	39.15	40.37	38.51	38.66	618312	{}	\N	2020-10-12 16:59:50.019151
87920	PNFP	2020-10-07	39.03	40.25	39.03	39.95	327852	{}	\N	2020-10-12 16:59:50.024917
87921	PNFP	2020-10-08	40.13	40.58	39.9	40.23	310607	{}	\N	2020-10-12 16:59:50.030655
87922	PNFP	2020-10-09	40.51	40.89	39.6	39.88	266012	{}	\N	2020-10-12 16:59:50.03628
87923	PNFP	2020-10-12	39.69	40.55	39.65	40.26	102365	{}	\N	2020-10-12 16:59:50.042074
87924	PQG	2020-09-28	10.29	10.49	10.16	10.17	229712	{}	\N	2020-10-12 16:59:51.74662
87925	PQG	2020-09-29	10.09	10.26	10.05	10.24	240671	{}	\N	2020-10-12 16:59:51.757608
87926	PQG	2020-09-30	10.23	10.54	10.22	10.26	426149	{}	\N	2020-10-12 16:59:51.764531
87927	PQG	2020-10-01	10.24	10.8	9.96	10.12	748244	{}	\N	2020-10-12 16:59:51.770839
87928	PQG	2020-10-02	9.83	10.47	9.78	10.39	214786	{}	\N	2020-10-12 16:59:51.776664
87929	PQG	2020-10-05	10.43	10.79	10.43	10.72	200783	{}	\N	2020-10-12 16:59:51.782049
87930	PQG	2020-10-06	10.86	11.12	10.705	10.77	298691	{}	\N	2020-10-12 16:59:51.787344
87931	PQG	2020-10-07	10.9	11	10.69	10.91	501361	{}	\N	2020-10-12 16:59:51.792512
87932	PQG	2020-10-08	10.41	10.87	10.41	10.84	485220	{}	\N	2020-10-12 16:59:51.797512
87933	PQG	2020-10-09	10.95	11.21	10.86	11.17	224082	{}	\N	2020-10-12 16:59:51.802464
87934	PQG	2020-10-12	11.18	11.24	11.09	11.19	38324	{}	\N	2020-10-12 16:59:51.808095
87935	PRFZ	2020-09-28	115.88	117.6121	115.88	117.34	18497	{}	\N	2020-10-12 16:59:53.074315
87936	PRFZ	2020-09-29	117.01	117.45	115.82	116.8	14100	{}	\N	2020-10-12 16:59:53.086895
87937	PRFZ	2020-09-30	116.93	118.49	116.76	117.0912	20169	{}	\N	2020-10-12 16:59:53.094928
87938	PRFZ	2020-10-01	117.65	118.92	117.1653	118.92	18804	{}	\N	2020-10-12 16:59:53.100992
87939	PRFZ	2020-10-02	116.45	120.37	116.45	120.05	36093	{}	\N	2020-10-12 16:59:53.106232
87940	PRFZ	2020-10-05	121.03	123.01	121.03	122.87	15600	{}	\N	2020-10-12 16:59:53.111315
87941	PRFZ	2020-10-06	123.49	125.66	122.45	122.45	37026	{}	\N	2020-10-12 16:59:53.116326
87942	PRFZ	2020-10-07	123.91	125.05	123.91	124.69	24805	{}	\N	2020-10-12 16:59:53.121234
87943	PRFZ	2020-10-08	125.79	126.36	125.43	126.36	37520	{}	\N	2020-10-12 16:59:53.126219
87944	PRFZ	2020-10-09	127.38	127.53	126.5001	127.03	19402	{}	\N	2020-10-12 16:59:53.131071
87945	PRFZ	2020-10-12	127.39	127.7499	127.01	127.48	13770	{}	\N	2020-10-12 16:59:53.136807
87946	PRF	2020-09-28	113.45	114.4799	113.45	114.09	85746	{}	\N	2020-10-12 16:59:55.159525
87947	PRF	2020-09-29	113.78	113.92	112.73	113.01	46203	{}	\N	2020-10-12 16:59:55.172494
87948	PRF	2020-09-30	113.4	115	113.23	113.81	92619	{}	\N	2020-10-12 16:59:55.179843
87949	PRF	2020-10-01	114.45	114.82	113.58	114.3	65059	{}	\N	2020-10-12 16:59:55.185678
87950	PRF	2020-10-02	112.3	114.8697	112.3	114.32	144331	{}	\N	2020-10-12 16:59:55.191822
87951	PRF	2020-10-05	115.13	116.48	115.13	116.36	42034	{}	\N	2020-10-12 16:59:55.196886
87952	PRF	2020-10-06	116.9378	117.7	114.97	115.1881	47461	{}	\N	2020-10-12 16:59:55.201814
87953	PRF	2020-10-07	116.2	117.489	116.2	117.2249	58503	{}	\N	2020-10-12 16:59:55.207194
87954	PRF	2020-10-08	117.89	118.77	117.741	118.77	55632	{}	\N	2020-10-12 16:59:55.212698
87955	PRF	2020-10-09	119.33	119.5499	118.65	119.03	51739	{}	\N	2020-10-12 16:59:55.217928
87956	PRF	2020-10-12	119.48	120.3799	119.425	120.27	20261	{}	\N	2020-10-12 16:59:55.22297
87957	PSCU	2020-09-28	44.5	44.6963	44.5	44.6963	648	{}	\N	2020-10-12 16:59:56.457825
87958	PSCU	2020-09-29	44.5431	44.5431	44.34	44.34	794	{}	\N	2020-10-12 16:59:56.471277
87959	PSCU	2020-09-30	44.5304	44.83	44.5304	44.83	536	{}	\N	2020-10-12 16:59:56.478613
87960	PSCU	2020-10-01	44.95	44.95	44.95	44.95	10697	{}	\N	2020-10-12 16:59:56.485473
87961	PSCU	2020-10-02	45.3327	45.43	45.3327	45.43	351	{}	\N	2020-10-12 16:59:56.491601
87962	PSCU	2020-10-05	45.7082	45.932	45.582	45.932	2992	{}	\N	2020-10-12 16:59:56.497034
87963	PSCU	2020-10-06	46.2747	46.835	46.0694	46.0694	1362	{}	\N	2020-10-12 16:59:56.502755
87964	PSCU	2020-10-07	46.02	46.1883	45.72	46.1883	1309	{}	\N	2020-10-12 16:59:56.50887
87965	PSCU	2020-10-08	46.19	46.37	46.19	46.37	1538	{}	\N	2020-10-12 16:59:56.514339
87966	PSCU	2020-10-09	46.83	47.0599	46.83	47.0599	3804	{}	\N	2020-10-12 16:59:56.519764
87967	PSCU	2020-10-12	47.34	47.34	46.78	47.26	2075	{}	\N	2020-10-12 16:59:56.525065
87968	PYN	2020-09-28	9.32	9.34	9.12	9.3299	9960	{}	\N	2020-10-12 16:59:57.95325
87969	PYN	2020-09-29	9.34	9.34	9.09	9.2	39138	{}	\N	2020-10-12 16:59:57.963616
87970	PYN	2020-09-30	9.23	9.23	9.06	9.17	18990	{}	\N	2020-10-12 16:59:57.970384
87971	PYN	2020-10-01	9.25	9.2879	9.02	9.19	35900	{}	\N	2020-10-12 16:59:57.975923
87972	PYN	2020-10-02	9.17	9.17	9.1	9.12	28352	{}	\N	2020-10-12 16:59:57.981558
87973	PYN	2020-10-05	9.22	9.245	9	9.02	35508	{}	\N	2020-10-12 16:59:57.986991
87974	PYN	2020-10-06	9.06	9.06	8.82	8.89	31968	{}	\N	2020-10-12 16:59:57.991811
87975	PYN	2020-10-07	8.86	8.99	8.86	8.885	19818	{}	\N	2020-10-12 16:59:57.996511
87976	PYN	2020-10-08	8.9	8.9	8.84	8.86	23136	{}	\N	2020-10-12 16:59:58.001489
87977	PYN	2020-10-09	8.97	8.97	8.83	8.83	8802	{}	\N	2020-10-12 16:59:58.006634
87978	PYN	2020-10-12	8.92	8.95	8.8	8.8	4658	{}	\N	2020-10-12 16:59:58.011797
87979	PZA	2020-09-28	26.69	26.7	26.65	26.68	198963	{}	\N	2020-10-12 17:00:00.817391
87980	PZA	2020-09-29	26.68	26.72	26.6677	26.715	229206	{}	\N	2020-10-12 17:00:00.828666
87981	PZA	2020-09-30	26.71	26.71	26.62	26.64	291669	{}	\N	2020-10-12 17:00:00.835953
87982	PZA	2020-10-01	26.6	26.65	26.51	26.6	480609	{}	\N	2020-10-12 17:00:00.841525
87983	PZA	2020-10-02	26.5671	26.61	26.54	26.57	333393	{}	\N	2020-10-12 17:00:00.847323
87984	PZA	2020-10-05	26.58	26.59	26.5	26.5	395008	{}	\N	2020-10-12 17:00:00.852167
87985	PZA	2020-10-06	26.5208	26.5299	26.47	26.49	294285	{}	\N	2020-10-12 17:00:00.856631
87986	PZA	2020-10-07	26.47	26.4991	26.41	26.42	300742	{}	\N	2020-10-12 17:00:00.861392
87987	PZA	2020-10-08	26.41	26.4611	26.36	26.38	857736	{}	\N	2020-10-12 17:00:00.86622
87988	PZA	2020-10-09	26.38	26.44	26.3619	26.38	363059	{}	\N	2020-10-12 17:00:00.871412
87989	PZA	2020-10-12	26.39	26.46	26.39	26.45	91282	{}	\N	2020-10-12 17:00:00.876202
87990	QEFA	2020-09-28	62.08	62.17	61.9201	62.17	19214	{}	\N	2020-10-12 17:00:02.351202
87991	QEFA	2020-09-29	62.17	62.28	61.875	62.12	47051	{}	\N	2020-10-12 17:00:02.36204
87992	QEFA	2020-09-30	61.92	62.26	61.751	61.84	21034	{}	\N	2020-10-12 17:00:02.370216
87993	QEFA	2020-10-01	62.19	62.3	61.97	62.14	137068	{}	\N	2020-10-12 17:00:02.377953
87994	QEFA	2020-10-02	61.45	62.2482	61.45	62.04	24118	{}	\N	2020-10-12 17:00:02.384576
87995	QEFA	2020-10-05	62.55	63.01	62.55	62.88	25357	{}	\N	2020-10-12 17:00:02.389736
87996	QEFA	2020-10-06	62.95	63.09	62.23	62.26	18849	{}	\N	2020-10-12 17:00:02.394879
87997	QEFA	2020-10-07	62.84	62.8958	62.65	62.77	38454	{}	\N	2020-10-12 17:00:02.400215
87998	QEFA	2020-10-08	62.96	63.2	62.96	63.11	39742	{}	\N	2020-10-12 17:00:02.405305
87999	QEFA	2020-10-09	63.35	63.6474	63.3	63.53	101378	{}	\N	2020-10-12 17:00:02.410149
88000	QEFA	2020-10-12	63.8914	63.98	63.68	63.96	7771	{}	\N	2020-10-12 17:00:02.415895
88001	QNST	2020-09-28	15.34	15.79	15.34	15.75	180629	{}	\N	2020-10-12 17:00:03.42688
88002	QNST	2020-09-29	15.78	16	15.53	15.79	272173	{}	\N	2020-10-12 17:00:03.436944
88003	QNST	2020-09-30	15.88	16.23	15.765	15.84	280655	{}	\N	2020-10-12 17:00:03.443576
88004	QNST	2020-10-01	15.92	16.53	15.68	16.49	285837	{}	\N	2020-10-12 17:00:03.44945
88005	QNST	2020-10-02	16.16	16.85	16.05	16.81	484596	{}	\N	2020-10-12 17:00:03.455572
88006	QNST	2020-10-05	17	17.03	16.445	16.95	328908	{}	\N	2020-10-12 17:00:03.460562
88007	QNST	2020-10-06	17.11	17.24	16.46	16.53	532804	{}	\N	2020-10-12 17:00:03.465752
88008	QNST	2020-10-07	16.86	16.86	16.41	16.6	474457	{}	\N	2020-10-12 17:00:03.470812
88009	QNST	2020-10-08	16.76	17.01	16.6	16.98	250951	{}	\N	2020-10-12 17:00:03.475298
88010	QNST	2020-10-09	17.24	17.275	16.92	17.19	382882	{}	\N	2020-10-12 17:00:03.479739
88011	QNST	2020-10-12	17.03	17.44	16.94	17.11	161746	{}	\N	2020-10-12 17:00:03.484568
88012	PHK	2020-09-28	5.36	5.39	5.34	5.38	334213	{}	\N	2020-10-12 17:00:05.888143
88013	PHK	2020-09-29	5.38	5.48	5.3701	5.46	747287	{}	\N	2020-10-12 17:00:05.897796
88014	PHK	2020-09-30	5.47	5.51	5.4611	5.47	688062	{}	\N	2020-10-12 17:00:05.904008
88015	PHK	2020-10-01	5.53	5.54	5.48	5.5	943368	{}	\N	2020-10-12 17:00:05.909452
88016	PHK	2020-10-02	5.41	5.54	5.405	5.52	1059928	{}	\N	2020-10-12 17:00:05.914509
88017	PHK	2020-10-05	5.54	5.55	5.49	5.52	1321050	{}	\N	2020-10-12 17:00:05.920527
88018	PHK	2020-10-06	5.52	5.53	5.42	5.46	1279278	{}	\N	2020-10-12 17:00:05.925959
88019	PHK	2020-10-07	5.5	5.53	5.5	5.51	619666	{}	\N	2020-10-12 17:00:05.931173
88020	PHK	2020-10-08	5.52	5.53	5.5	5.52	723073	{}	\N	2020-10-12 17:00:05.937925
88021	PHK	2020-10-09	5.48	5.5074	5.45	5.46	742772	{}	\N	2020-10-12 17:00:05.94418
88022	PHK	2020-10-12	5.5	5.52	5.47	5.5099	693148	{}	\N	2020-10-12 17:00:05.950412
88023	PICK	2020-09-28	26.89	27.1	26.79	26.93	112883	{}	\N	2020-10-12 17:00:08.045622
88024	PICK	2020-09-29	26.93	26.93	26.5801	26.68	215010	{}	\N	2020-10-12 17:00:08.065946
88025	PICK	2020-09-30	26.57	26.95	26.57	26.78	62731	{}	\N	2020-10-12 17:00:08.072831
88026	PICK	2020-10-01	26.79	26.955	26.63	26.8	62990	{}	\N	2020-10-12 17:00:08.078573
88027	PICK	2020-10-02	26.45	26.89	26.36	26.87	124592	{}	\N	2020-10-12 17:00:08.083736
88028	PICK	2020-10-05	27.1	27.4	27.1	27.29	105846	{}	\N	2020-10-12 17:00:08.088515
88029	PICK	2020-10-06	27.45	27.49	26.81	26.81	123872	{}	\N	2020-10-12 17:00:08.093823
88030	PICK	2020-10-07	27.04	27.5075	27.04	27.4	551721	{}	\N	2020-10-12 17:00:08.098835
88031	PICK	2020-10-08	27.46	27.7	27.41	27.7	490644	{}	\N	2020-10-12 17:00:08.103904
88032	PICK	2020-10-09	27.98	28.2	27.95	28.0949	48605	{}	\N	2020-10-12 17:00:08.108706
88033	PICK	2020-10-12	28.05	28.0665	27.895	28.0469	19441	{}	\N	2020-10-12 17:00:08.113406
88034	PREF	2020-09-28	20.04	20.13	20.02	20.04	48065	{}	\N	2020-10-12 17:00:09.422997
88035	PREF	2020-09-29	20.04	20.13	20.04	20.06	81167	{}	\N	2020-10-12 17:00:09.432733
88036	PREF	2020-09-30	20.0612	20.18	20.04	20.09	75534	{}	\N	2020-10-12 17:00:09.439324
88037	PREF	2020-10-01	20.19	20.19	20.04	20.07	37200	{}	\N	2020-10-12 17:00:09.445166
88038	PREF	2020-10-02	20.02	20.12	20.02	20.05	344196	{}	\N	2020-10-12 17:00:09.450288
88039	PREF	2020-10-05	20.25	20.25	20.045	20.045	88838	{}	\N	2020-10-12 17:00:09.454874
88040	PREF	2020-10-06	20.083	20.0899	19.99	19.99	86876	{}	\N	2020-10-12 17:00:09.459877
88041	PREF	2020-10-07	20.08	20.09	20.0692	20.09	78883	{}	\N	2020-10-12 17:00:09.464161
88042	PREF	2020-10-08	20.1	20.1	20.06	20.08	65729	{}	\N	2020-10-12 17:00:09.468588
88043	PREF	2020-10-09	20.08	20.09	20.06	20.08	42643	{}	\N	2020-10-12 17:00:09.473341
88044	PREF	2020-10-12	20.051	20.1	20.051	20.09	31502	{}	\N	2020-10-12 17:00:09.478236
88045	QED	2020-09-28	22.6554	22.6554	22.6554	22.6554	158	{}	\N	2020-10-12 17:00:13.368028
88046	QED	2020-09-29	22.6585	22.6585	22.6585	22.6585	30	{}	\N	2020-10-12 17:00:13.376188
88047	QED	2020-09-30	22.728	22.728	22.7006	22.7006	141	{}	\N	2020-10-12 17:00:13.382745
88048	QED	2020-10-01	22.7989	22.7989	22.7989	22.7989	15	{}	\N	2020-10-12 17:00:13.388975
88049	QED	2020-10-02	22.767	22.767	22.75	22.75	2799	{}	\N	2020-10-12 17:00:13.394068
88050	QED	2020-10-05	22.87	22.87	22.8664	22.8664	444	{}	\N	2020-10-12 17:00:13.398956
88051	QED	2020-10-06	22.91	22.91	22.8357	22.8357	46383	{}	\N	2020-10-12 17:00:13.403781
88052	QED	2020-10-07	22.96	22.98	22.9475	22.95	14636	{}	\N	2020-10-12 17:00:13.408746
88053	QED	2020-10-08	23.04	23.04	22.9941	22.9941	8941	{}	\N	2020-10-12 17:00:13.413802
88054	QED	2020-10-09	23.06	23.077	23.0475	23.07	3142	{}	\N	2020-10-12 17:00:13.418639
88055	QED	2020-10-12	23.081	23.1215	23.081	23.12	840	{}	\N	2020-10-12 17:00:13.423697
88056	QQQ	2020-09-28	276.615	277.2	271.674	277.2	57890502	{}	\N	2020-10-12 17:00:15.737418
88057	QQQ	2020-09-29	276.72	277.71	275.3719	275.95	26869747	{}	\N	2020-10-12 17:00:15.750615
88058	QQQ	2020-09-30	276.09	281.09	275.8717	277.84	49180660	{}	\N	2020-10-12 17:00:15.756896
88059	QQQ	2020-10-01	281.8	282.8794	279.84	282.25	48890687	{}	\N	2020-10-12 17:00:15.762747
88060	QQQ	2020-10-02	276.01	282.2412	273.44	274.31	73856691	{}	\N	2020-10-12 17:00:15.76827
88061	QQQ	2020-10-05	276.34	280.49	276.23	280.16	27760373	{}	\N	2020-10-12 17:00:15.772983
88062	QQQ	2020-10-06	279.29	281.19	274.25	275.16	48557336	{}	\N	2020-10-12 17:00:15.777975
88063	QQQ	2020-10-07	277.83	280.74	277.28	279.92	27210661	{}	\N	2020-10-12 17:00:15.782816
88064	QQQ	2020-10-08	282.32	282.47	280.31	281.41	25660092	{}	\N	2020-10-12 17:00:15.787284
88065	QQQ	2020-10-09	283.09	285.79	282.71	285.71	30055195	{}	\N	2020-10-12 17:00:15.791609
88066	QQQ	2020-10-12	290.6	295.875	289.47	295.18	38554281	{}	\N	2020-10-12 17:00:15.796363
88067	QMOM	2020-09-28	41.05	41.2688	40.915	41.2688	3118	{}	\N	2020-10-12 17:00:16.983208
88068	QMOM	2020-09-29	41.385	41.81	41.385	41.61	6176	{}	\N	2020-10-12 17:00:16.996338
88069	QMOM	2020-09-30	41.58	42.254	41.58	41.85	7754	{}	\N	2020-10-12 17:00:17.004289
88070	QMOM	2020-10-01	42.53	43.093	42.4982	42.95	15490	{}	\N	2020-10-12 17:00:17.010487
88071	QMOM	2020-10-02	41.9588	43.265	41.9588	42.729	20370	{}	\N	2020-10-12 17:00:17.017014
88072	QMOM	2020-10-05	43.01	43.5356	43.01	43.5356	17458	{}	\N	2020-10-12 17:00:17.022557
88073	QMOM	2020-10-06	43.7627	44.1433	43.3822	43.3822	8458	{}	\N	2020-10-12 17:00:17.0282
88074	QMOM	2020-10-07	44.1	44.42	44.0101	44.41	11269	{}	\N	2020-10-12 17:00:17.034402
88075	QMOM	2020-10-08	44.2	44.3602	43.9	43.9	17756	{}	\N	2020-10-12 17:00:17.040133
88076	QMOM	2020-10-09	44.6419	44.88	44.5801	44.88	6633	{}	\N	2020-10-12 17:00:17.045771
88077	QMOM	2020-10-12	45.35	45.35	45.02	45.0201	8016	{}	\N	2020-10-12 17:00:17.051698
88078	QUIK	2020-09-28	3.12	3.19	3.05	3.15	59255	{}	\N	2020-10-12 17:00:18.775005
88079	QUIK	2020-09-29	3.18	3.33	3.1394	3.16	21494	{}	\N	2020-10-12 17:00:18.785012
88080	QUIK	2020-09-30	3.14	3.19	3.0945	3.11	14362	{}	\N	2020-10-12 17:00:18.791467
88081	QUIK	2020-10-01	3.16	3.2	3.03	3.11	38346	{}	\N	2020-10-12 17:00:18.798038
88082	QUIK	2020-10-02	3.09	3.15	2.98	2.98	59396	{}	\N	2020-10-12 17:00:18.804547
88083	QUIK	2020-10-05	3	3.1102	2.98	3.1	30738	{}	\N	2020-10-12 17:00:18.809527
88084	QUIK	2020-10-06	3.08	3.14	3.01	3.02	31625	{}	\N	2020-10-12 17:00:18.814682
88085	QUIK	2020-10-07	3.07	3.19	3.04	3.1	46401	{}	\N	2020-10-12 17:00:18.819612
88086	QUIK	2020-10-08	3.1679	3.186	3.1201	3.17	18377	{}	\N	2020-10-12 17:00:18.825553
88087	QUIK	2020-10-09	3.23	3.3	3.18	3.22	26352	{}	\N	2020-10-12 17:00:18.830934
88088	QUIK	2020-10-12	3.24	3.2699	3.12	3.16	9956	{}	\N	2020-10-12 17:00:18.836731
88089	QUAD	2020-09-28	2.98	3.21	2.98	3.17	139323	{}	\N	2020-10-12 17:00:20.885292
88090	QUAD	2020-09-29	3.17	3.17	3.0375	3.09	84033	{}	\N	2020-10-12 17:00:20.895467
88091	QUAD	2020-09-30	3.08	3.1	2.99	3.03	339144	{}	\N	2020-10-12 17:00:20.903672
88092	QUAD	2020-10-01	3.04	3.04	2.88	2.97	392778	{}	\N	2020-10-12 17:00:20.90932
88093	QUAD	2020-10-02	2.88	2.985	2.81	2.93	300201	{}	\N	2020-10-12 17:00:20.915099
88094	QUAD	2020-10-05	2.97	3.02	2.87	2.98	262876	{}	\N	2020-10-12 17:00:20.920316
88095	QUAD	2020-10-06	3	3.16	3	3.02	230189	{}	\N	2020-10-12 17:00:20.925261
88096	QUAD	2020-10-07	3.05	3.06	2.915	2.93	298313	{}	\N	2020-10-12 17:00:20.931007
88097	QUAD	2020-10-08	3.01	3.09	2.9713	3.07	265487	{}	\N	2020-10-12 17:00:20.936111
88098	QUAD	2020-10-09	3.13	3.14	2.98	2.99	141414	{}	\N	2020-10-12 17:00:20.941051
88099	QUAD	2020-10-12	2.98	3.14	2.98	3.03	129714	{}	\N	2020-10-12 17:00:20.945789
88100	QWLD	2020-09-28	83.361	83.5425	83.361	83.5425	534	{}	\N	2020-10-12 17:00:21.923067
88101	QWLD	2020-09-29	83.54	83.54	83.2886	83.2886	702	{}	\N	2020-10-12 17:00:21.934933
88102	QWLD	2020-09-30	83.09	83.6716	83.09	83.6716	1283	{}	\N	2020-10-12 17:00:21.942087
88103	QWLD	2020-10-01	83.785	83.8456	83.785	83.8456	127	{}	\N	2020-10-12 17:00:21.947396
88104	QWLD	2020-10-02	83.5484	83.5484	83.5484	83.5484	96	{}	\N	2020-10-12 17:00:21.953022
88105	QWLD	2020-10-05	84.4905	84.776	84.4905	84.776	398	{}	\N	2020-10-12 17:00:21.957829
88106	QWLD	2020-10-06	83.9609	83.9609	83.9609	83.9609	9	{}	\N	2020-10-12 17:00:21.964377
88107	QWLD	2020-10-07	84.755	84.759	84.755	84.759	589	{}	\N	2020-10-12 17:00:21.970026
88108	QWLD	2020-10-08	85.5591	85.5591	85.5591	85.5591	106	{}	\N	2020-10-12 17:00:21.975217
88109	QWLD	2020-10-09	85.82	86.161	85.82	86.161	894	{}	\N	2020-10-12 17:00:21.979356
88110	QWLD	2020-10-12	86.4	86.81	86.4	86.81	1337	{}	\N	2020-10-12 17:00:21.984687
88111	RDNT	2020-09-28	14.73	15.2	14.73	14.99	308825	{}	\N	2020-10-12 17:00:23.825783
88112	RDNT	2020-09-29	15.04	15.13	14.68	14.81	235783	{}	\N	2020-10-12 17:00:23.835235
88113	RDNT	2020-09-30	14.84	15.69	14.76	15.35	308705	{}	\N	2020-10-12 17:00:23.842901
88114	RDNT	2020-10-01	15.39	15.605	15.25	15.33	185611	{}	\N	2020-10-12 17:00:23.848447
88115	RDNT	2020-10-02	15	15.58	14.55	15.45	156258	{}	\N	2020-10-12 17:00:23.854193
88116	RDNT	2020-10-05	15.7	16.03	15.632	15.8	162255	{}	\N	2020-10-12 17:00:23.858907
88117	RDNT	2020-10-06	16.03	16.335	15.4	15.69	270028	{}	\N	2020-10-12 17:00:23.863239
88118	RDNT	2020-10-07	15.89	16.035	15.65	15.835	305401	{}	\N	2020-10-12 17:00:23.867895
88119	RDNT	2020-10-08	16.01	17	15.8	16.8	197300	{}	\N	2020-10-12 17:00:23.872831
88120	RDNT	2020-10-09	17.08	17.15	16.4	16.9	163709	{}	\N	2020-10-12 17:00:23.877811
88121	RDNT	2020-10-12	16.91	17.37	16.795	17.07	30230	{}	\N	2020-10-12 17:00:23.882919
88122	RETA	2020-09-28	94.56	94.65	88.17	93.53	623855	{}	\N	2020-10-12 17:00:25.45726
88123	RETA	2020-09-29	92.4001	95.28	92.21	94.78	251112	{}	\N	2020-10-12 17:00:25.469149
88124	RETA	2020-09-30	94.69	99.78	94.6	97.42	326223	{}	\N	2020-10-12 17:00:25.477844
88125	RETA	2020-10-01	98.75	103.7399	97.88	103.45	226797	{}	\N	2020-10-12 17:00:25.486392
88126	RETA	2020-10-02	100.94	103.69	100.02	102.03	141122	{}	\N	2020-10-12 17:00:25.493021
88127	RETA	2020-10-05	103.33	106.45	102.3	106.24	194752	{}	\N	2020-10-12 17:00:25.501824
88128	RETA	2020-10-06	106.83	113.36	106.295	111.05	325257	{}	\N	2020-10-12 17:00:25.509976
88129	RETA	2020-10-07	111.08	112.9216	109.48	109.9	146875	{}	\N	2020-10-12 17:00:25.518473
88130	RETA	2020-10-08	110.87	112.2	109.53	111.77	136486	{}	\N	2020-10-12 17:00:25.525978
88131	RETA	2020-10-09	112.18	112.23	109.69	109.73	160107	{}	\N	2020-10-12 17:00:25.533787
88132	RETA	2020-10-12	110.12	112	108	110.13	50865	{}	\N	2020-10-12 17:00:25.539835
88133	RFEM	2020-09-28	59.81	59.81	59.47	59.61	4111	{}	\N	2020-10-12 17:00:27.7581
88134	RFEM	2020-09-29	59.38	59.67	59.37	59.52	4687	{}	\N	2020-10-12 17:00:27.771167
88135	RFEM	2020-09-30	60.1	60.72	59.94	60.64	3804	{}	\N	2020-10-12 17:00:27.78038
88136	RFEM	2020-10-01	61	61.135	60.69	61.135	5162	{}	\N	2020-10-12 17:00:27.787303
88137	RFEM	2020-10-02	60.4	60.88	60.4	60.5199	4604	{}	\N	2020-10-12 17:00:27.794166
88138	RFEM	2020-10-05	60.7893	61.06	60.7893	61.04	4202	{}	\N	2020-10-12 17:00:27.800132
88139	RFEM	2020-10-06	61.53	61.76	61.29	61.29	3106	{}	\N	2020-10-12 17:00:27.807315
88140	RFEM	2020-10-07	61.87	62.12	61.87	62.07	4011	{}	\N	2020-10-12 17:00:27.815101
88141	RFEM	2020-10-08	62.23	62.51	62.23	62.51	1623	{}	\N	2020-10-12 17:00:27.821565
88142	RFEM	2020-10-09	62.73	63.02	62.73	62.89	1846	{}	\N	2020-10-12 17:00:27.828428
88143	RFEM	2020-10-12	63.09	63.36	63.0399	63.15	1640	{}	\N	2020-10-12 17:00:27.834013
88144	PHM	2020-09-28	44.94	45.82	44.57	45.76	1543136	{}	\N	2020-10-12 17:00:29.596052
88145	PHM	2020-09-29	46.18	46.36	45.06	45.27	3541642	{}	\N	2020-10-12 17:00:29.60341
88146	PHM	2020-09-30	45.72	47.06	45.51	46.29	8899601	{}	\N	2020-10-12 17:00:29.610286
88147	PHM	2020-10-01	46.85	47.49	45.97	47.46	5531869	{}	\N	2020-10-12 17:00:29.618588
88148	PHM	2020-10-02	46.52	47.8	46.47	47.53	3861289	{}	\N	2020-10-12 17:00:29.625
88149	PHM	2020-10-05	47.62	48.31	46.535	47.72	5077123	{}	\N	2020-10-12 17:00:29.632216
88150	PHM	2020-10-06	47.72	48.02	46.39	46.62	3823892	{}	\N	2020-10-12 17:00:29.637467
88151	PHM	2020-10-07	47.39	47.97	46.85	47.13	3151415	{}	\N	2020-10-12 17:00:29.643657
88152	PHM	2020-10-08	48.17	49.29	47.99	48.97	5343041	{}	\N	2020-10-12 17:00:29.650574
88153	PHM	2020-10-09	49.31	49.7	48.66	48.98	3734021	{}	\N	2020-10-12 17:00:29.655995
88154	PHM	2020-10-12	49.33	49.498	48.58	48.87	1055088	{}	\N	2020-10-12 17:00:29.662904
88155	POWL	2020-09-28	24.02	25.17	24.02	24.47	42117	{}	\N	2020-10-12 17:00:30.671484
88156	POWL	2020-09-29	24.4	24.6	23.9	24.45	49164	{}	\N	2020-10-12 17:00:30.682394
88157	POWL	2020-09-30	24.57	24.84	23.92	24.13	51456	{}	\N	2020-10-12 17:00:30.690059
88158	POWL	2020-10-01	24.3	24.3	23.81	23.99	49659	{}	\N	2020-10-12 17:00:30.69587
88159	POWL	2020-10-02	23.51	24.885	23.51	24.6	45075	{}	\N	2020-10-12 17:00:30.701401
88160	POWL	2020-10-05	24.69	25.16	24.325	25.16	42709	{}	\N	2020-10-12 17:00:30.706276
88161	POWL	2020-10-06	25.33	25.97	24.22	24.59	36693	{}	\N	2020-10-12 17:00:30.710568
88162	POWL	2020-10-07	25.11	25.26	24.49	25	46021	{}	\N	2020-10-12 17:00:30.715071
88163	POWL	2020-10-08	25.37	26.14	25.02	25.96	41659	{}	\N	2020-10-12 17:00:30.719248
88164	POWL	2020-10-09	26.2	26.86	26.005	26.16	49226	{}	\N	2020-10-12 17:00:30.723262
88165	POWL	2020-10-12	26.16	26.715	25.7	26.215	12455	{}	\N	2020-10-12 17:00:30.727715
88166	PROV	2020-09-28	11.69	11.815	11.4	11.73	30909	{}	\N	2020-10-12 17:00:32.472576
88167	PROV	2020-09-29	11.86	11.9	11.68	11.74	1717	{}	\N	2020-10-12 17:00:32.481082
88168	PROV	2020-09-30	11.7	11.91	11.7	11.9	1473	{}	\N	2020-10-12 17:00:32.487866
88169	PROV	2020-10-01	11.9	11.9	11.78	11.83	4084	{}	\N	2020-10-12 17:00:32.494983
88170	PROV	2020-10-02	11.66	12.19	11.66	12.08	11614	{}	\N	2020-10-12 17:00:32.502729
88171	PROV	2020-10-05	12.18	12.75	12.17	12.56	9641	{}	\N	2020-10-12 17:00:32.507711
88172	PROV	2020-10-06	12.68	12.86	12.615	12.85	15235	{}	\N	2020-10-12 17:00:32.51212
88173	PROV	2020-10-07	12.92	13.49	12.4301	13.49	12567	{}	\N	2020-10-12 17:00:32.518123
88174	PROV	2020-10-08	13.47	13.92	13.28	13.7	11560	{}	\N	2020-10-12 17:00:32.528026
88175	PROV	2020-10-09	14.14	14.14	13.55	13.71	2465	{}	\N	2020-10-12 17:00:32.535073
88176	PROV	2020-10-12	13.66	13.88	13.61	13.88	1052	{}	\N	2020-10-12 17:00:32.545002
88177	QEP	2020-09-28	0.97	0.9949	0.94	0.9663	3352357	{}	\N	2020-10-12 17:00:34.992335
88178	QEP	2020-09-29	0.9663	0.9789	0.9125	0.9284	5018943	{}	\N	2020-10-12 17:00:35.004137
88179	QEP	2020-09-30	0.95	0.95	0.8805	0.9028	10183252	{}	\N	2020-10-12 17:00:35.011678
88180	QEP	2020-10-01	0.9087	0.92199	0.8706	0.8808	7125615	{}	\N	2020-10-12 17:00:35.017794
88181	QEP	2020-10-02	0.85	0.929	0.84	0.886	6718995	{}	\N	2020-10-12 17:00:35.023258
88182	QEP	2020-10-05	0.9092	0.9472	0.9001	0.92	5499636	{}	\N	2020-10-12 17:00:35.029013
88183	QEP	2020-10-06	0.94	0.9847	0.9232	0.9467	9462123	{}	\N	2020-10-12 17:00:35.033156
88184	QEP	2020-10-07	0.966	0.9718	0.9301	0.954	3778278	{}	\N	2020-10-12 17:00:35.037242
88185	QEP	2020-10-08	0.98	1.08	0.957201	1.05	5266467	{}	\N	2020-10-12 17:00:35.04177
88186	QEP	2020-10-09	1.09	1.1	0.9726	0.9884	6587857	{}	\N	2020-10-12 17:00:35.045958
88187	QEP	2020-10-12	1.02	1.02	0.951	0.961	1947684	{}	\N	2020-10-12 17:00:35.049958
88188	QTEC	2020-09-28	116.54	117.23	115.59	117.18	92744	{}	\N	2020-10-12 17:00:37.674395
88189	QTEC	2020-09-29	117.06	118.06	117	117.25	207543	{}	\N	2020-10-12 17:00:37.686357
88190	QTEC	2020-09-30	117	118.59	116.67	117.22	166959	{}	\N	2020-10-12 17:00:37.695314
88191	QTEC	2020-10-01	118.9	119.82	118.5168	119.39	271639	{}	\N	2020-10-12 17:00:37.70262
88192	QTEC	2020-10-02	116.67	118.46	115.93	116.33	223967	{}	\N	2020-10-12 17:00:37.708563
88193	QTEC	2020-10-05	117.15	118.96	117.15	118.95	151698	{}	\N	2020-10-12 17:00:37.714565
88194	QTEC	2020-10-06	118.5	120.32	117.37	117.56	151548	{}	\N	2020-10-12 17:00:37.720808
88195	QTEC	2020-10-07	119.12	120.13	118.8845	119.93	633041	{}	\N	2020-10-12 17:00:37.726886
88196	QTEC	2020-10-08	120.88	121.2	120.41	121.01	125211	{}	\N	2020-10-12 17:00:37.732055
88197	QTEC	2020-10-09	122.48	123.102	122.31	122.89	96564	{}	\N	2020-10-12 17:00:37.737135
88198	QTEC	2020-10-12	124.35	126	123.935	125.56	84646	{}	\N	2020-10-12 17:00:37.742546
88199	QUMU	2020-09-28	4.62	4.85	4.62	4.76	36480	{}	\N	2020-10-12 17:00:39.010386
88200	QUMU	2020-09-29	4.68	4.9	4.68	4.79	28439	{}	\N	2020-10-12 17:00:39.019877
88201	QUMU	2020-09-30	4.8823	4.9	4.53	4.61	57244	{}	\N	2020-10-12 17:00:39.028047
88202	QUMU	2020-10-01	4.61	4.9	4.61	4.82	35136	{}	\N	2020-10-12 17:00:39.034104
88203	QUMU	2020-10-02	4.77	4.9	4.61	4.72	41924	{}	\N	2020-10-12 17:00:39.040467
88204	QUMU	2020-10-05	4.83	4.855	4.7	4.7	24131	{}	\N	2020-10-12 17:00:39.047144
88205	QUMU	2020-10-06	4.7	4.8	4.28	4.3	59921	{}	\N	2020-10-12 17:00:39.052466
88206	QUMU	2020-10-07	4.425	4.56	4.3	4.5	49569	{}	\N	2020-10-12 17:00:39.058697
88207	QUMU	2020-10-08	4.53	4.56	4.43	4.52	19893	{}	\N	2020-10-12 17:00:39.064375
88208	QUMU	2020-10-09	4.52	4.61	4.52	4.58	16723	{}	\N	2020-10-12 17:00:39.069851
88209	QUMU	2020-10-12	4.57	4.65	4.57	4.58	1767	{}	\N	2020-10-12 17:00:39.075567
88210	QURE	2020-09-28	37.37	38	36.28	36.41	370679	{}	\N	2020-10-12 17:00:40.118178
88211	QURE	2020-09-29	36.8	37	35.76	35.98	365225	{}	\N	2020-10-12 17:00:40.128522
88212	QURE	2020-09-30	36	37.6	36	36.83	252795	{}	\N	2020-10-12 17:00:40.137608
88213	QURE	2020-10-01	36.68	37.26	36.475	37.24	216893	{}	\N	2020-10-12 17:00:40.152675
88214	QURE	2020-10-02	36.69	37.13	35.32	35.35	283076	{}	\N	2020-10-12 17:00:40.164196
88215	QURE	2020-10-05	35.73	37.92	35.69	37.88	812145	{}	\N	2020-10-12 17:00:40.171889
88216	QURE	2020-10-06	38.1	38.58	37.15	37.65	320904	{}	\N	2020-10-12 17:00:40.180886
88217	QURE	2020-10-07	37.67	38.37	37.22	37.51	210391	{}	\N	2020-10-12 17:00:40.188614
88218	QURE	2020-10-08	37.8	37.9831	36.63	37.14	178057	{}	\N	2020-10-12 17:00:40.196911
88219	QURE	2020-10-09	37.79	37.79	36.61	36.77	180452	{}	\N	2020-10-12 17:00:40.203897
88220	QURE	2020-10-12	36.89	37.67	36.26	37.04	65929	{}	\N	2020-10-12 17:00:40.213468
88221	RELV	2020-09-29	3.3	3.48	3.16	3.32	18722	{}	\N	2020-10-12 17:00:43.039197
88222	RELV	2020-09-30	3.39	3.42	3.39	3.42	2881	{}	\N	2020-10-12 17:00:43.046634
88223	RELV	2020-10-01	3.33	3.33	3.33	3.33	167	{}	\N	2020-10-12 17:00:43.051998
88224	RELV	2020-10-02	3.365	3.365	3.365	3.365	254	{}	\N	2020-10-12 17:00:43.05912
88225	RELV	2020-10-06	3.25	3.25	3.25	3.25	386	{}	\N	2020-10-12 17:00:43.065443
88226	RELV	2020-10-08	3.41	3.441	3.35	3.3974	4893	{}	\N	2020-10-12 17:00:43.071307
88227	RELV	2020-10-12	3.5	3.97	3.45	3.45	42502	{}	\N	2020-10-12 17:00:43.078138
88228	RETO	2020-09-28	0.5353	0.5353	0.4704	0.5003	183276	{}	\N	2020-10-12 17:00:44.086036
88229	RETO	2020-09-29	0.54	0.54	0.5	0.527	68327	{}	\N	2020-10-12 17:00:44.097075
88230	RETO	2020-09-30	0.5	0.5202	0.4955	0.5	107850	{}	\N	2020-10-12 17:00:44.104764
88231	RETO	2020-10-01	0.5	0.526	0.49	0.4936	66791	{}	\N	2020-10-12 17:00:44.110192
88232	RETO	2020-10-02	0.49	0.5137	0.4682	0.500101	126644	{}	\N	2020-10-12 17:00:44.115464
88233	RETO	2020-10-05	0.4838	0.538921	0.4838	0.538899	314869	{}	\N	2020-10-12 17:00:44.120749
88234	RETO	2020-10-06	0.5269	0.539	0.49	0.52	267557	{}	\N	2020-10-12 17:00:44.12518
88235	RETO	2020-10-07	0.514	0.544	0.485	0.528	405009	{}	\N	2020-10-12 17:00:44.129414
88236	RETO	2020-10-08	0.5144	0.5389	0.5011	0.5291	183478	{}	\N	2020-10-12 17:00:44.134131
88237	RETO	2020-10-09	0.53	0.82	0.52	0.57	7833011	{}	\N	2020-10-12 17:00:44.138846
88238	RETO	2020-10-12	0.5702	0.64	0.55	0.5783	743503	{}	\N	2020-10-12 17:00:44.142935
88239	RFDA	2020-09-28	34.7117	34.7658	34.7117	34.7658	2048	{}	\N	2020-10-12 17:00:45.155131
88240	RFDA	2020-09-29	34.85	34.85	34.59	34.723	1287	{}	\N	2020-10-12 17:00:45.167774
88241	RFDA	2020-09-30	35.0142	35.0676	34.9208	34.93	5675	{}	\N	2020-10-12 17:00:45.176729
88242	RFDA	2020-10-01	35.14	35.18	34.91	35.045	11244	{}	\N	2020-10-12 17:00:45.18328
88243	RFDA	2020-10-02	34.49	35.032	34.49	34.92	1904	{}	\N	2020-10-12 17:00:45.189199
88244	RFDA	2020-10-05	35.1176	35.445	35.1176	35.445	2286	{}	\N	2020-10-12 17:00:45.194301
88245	RFDA	2020-10-06	35.465	35.59	35.0307	35.0307	564	{}	\N	2020-10-12 17:00:45.199582
88246	RFDA	2020-10-07	35.4519	35.6313	35.41	35.52	6309	{}	\N	2020-10-12 17:00:45.204546
88247	RFDA	2020-10-08	35.82	35.88	35.58	35.83	11077	{}	\N	2020-10-12 17:00:45.209179
88248	RFDA	2020-10-09	36.045	36.1899	36.0392	36.1899	8853	{}	\N	2020-10-12 17:00:45.213896
88249	RFDA	2020-10-12	36.5	36.77	36.47	36.77	4909	{}	\N	2020-10-12 17:00:45.218604
88250	RFDI	2020-09-28	56.37	56.5271	56.3458	56.485	3732	{}	\N	2020-10-12 17:00:46.941842
88251	RFDI	2020-09-29	56.7718	56.86	56.5501	56.69	2655	{}	\N	2020-10-12 17:00:46.953579
88252	RFDI	2020-09-30	56.63	56.9	56.46	56.51	9084	{}	\N	2020-10-12 17:00:46.961968
88253	RFDI	2020-10-01	56.94	56.94	56.67	56.9	7044	{}	\N	2020-10-12 17:00:46.968763
88254	RFDI	2020-10-02	56.4319	56.85	56.4319	56.75	3727	{}	\N	2020-10-12 17:00:46.974217
88255	RFDI	2020-10-05	57.2739	57.4847	57.23	57.4847	4544	{}	\N	2020-10-12 17:00:46.979311
88256	RFDI	2020-10-06	57.33	57.345	56.61	56.655	5794	{}	\N	2020-10-12 17:00:46.984552
88257	RFDI	2020-10-07	57.03	57.26	57.03	57.18	5816	{}	\N	2020-10-12 17:00:46.989561
88258	RFDI	2020-10-08	57.49	57.68	57.47	57.67	27809	{}	\N	2020-10-12 17:00:46.994476
88259	RFDI	2020-10-09	57.99	58.3287	57.99	58.28	9681	{}	\N	2020-10-12 17:00:46.999345
88260	RFDI	2020-10-12	58.55	58.74	58.52	58.74	2832	{}	\N	2020-10-12 17:00:47.004231
88261	RNLC	2020-09-28	23.935	24.05	23.9281	23.9281	25725	{}	\N	2020-10-12 17:00:49.389481
88262	RNLC	2020-09-29	23.73	23.8108	23.73	23.8108	670	{}	\N	2020-10-12 17:00:49.39889
88263	RNLC	2020-09-30	23.69	24.13	23.69	24.13	3563	{}	\N	2020-10-12 17:00:49.405826
88264	RNLC	2020-10-01	24.035	24.0478	23.96	23.96	3615	{}	\N	2020-10-12 17:00:49.411479
88265	RNLC	2020-10-02	23.57	24.0284	23.57	24.0284	710	{}	\N	2020-10-12 17:00:49.416777
88266	RNLC	2020-10-05	24.2967	24.4261	24.2967	24.4261	779	{}	\N	2020-10-12 17:00:49.421958
88267	RNLC	2020-10-06	24.5445	24.65	24.2355	24.2355	1100	{}	\N	2020-10-12 17:00:49.426938
88268	RNLC	2020-10-07	24.57	24.6615	24.57	24.6615	339	{}	\N	2020-10-12 17:00:49.431779
88269	RNLC	2020-10-08	24.8527	24.95	24.84	24.84	10977	{}	\N	2020-10-12 17:00:49.436555
88270	RNLC	2020-10-09	25.2	25.2	25.0173	25.0173	1431	{}	\N	2020-10-12 17:00:49.440982
88271	PHX	2020-09-28	1.65	1.7799	1.62	1.73	100335	{}	\N	2020-10-12 17:00:51.281849
88272	PHX	2020-09-29	1.75	1.7665	1.66	1.66	149703	{}	\N	2020-10-12 17:00:51.2899
88273	PHX	2020-09-30	1.62	1.66	1.43	1.43	636987	{}	\N	2020-10-12 17:00:51.297545
88274	PHX	2020-10-01	1.5	1.53	1.41	1.43	236042	{}	\N	2020-10-12 17:00:51.303239
88275	PHX	2020-10-02	1.4	1.535	1.4	1.49	117583	{}	\N	2020-10-12 17:00:51.308315
88276	PHX	2020-10-05	1.5	1.6	1.5	1.54	171000	{}	\N	2020-10-12 17:00:51.31298
88277	PHX	2020-10-06	1.58	1.6001	1.46	1.47	227129	{}	\N	2020-10-12 17:00:51.317484
88278	PHX	2020-10-07	1.48	1.54	1.45	1.48	143297	{}	\N	2020-10-12 17:00:51.322048
88279	PHX	2020-10-08	1.47	1.7	1.47	1.65	294575	{}	\N	2020-10-12 17:00:51.326359
88280	PHX	2020-10-09	1.81	1.84	1.63	1.64	477846	{}	\N	2020-10-12 17:00:51.330686
88281	PHX	2020-10-12	1.68	1.69	1.6	1.6714	173770	{}	\N	2020-10-12 17:00:51.33482
88282	PII	2020-09-28	92.59	93.04	91.51	92.08	433819	{}	\N	2020-10-12 17:00:54.152916
88283	PII	2020-09-29	96.04	96.13	90.22	91.31	2326888	{}	\N	2020-10-12 17:00:54.166063
88284	PII	2020-09-30	93.18	96.65	92.92	94.34	3279749	{}	\N	2020-10-12 17:00:54.174264
88285	PII	2020-10-01	95.65	96.8119	93.81	94.54	1139473	{}	\N	2020-10-12 17:00:54.181058
88286	PII	2020-10-02	91.58	99.85	91.58	98.83	1312446	{}	\N	2020-10-12 17:00:54.187014
88287	PII	2020-10-05	102.39	103.91	100.5594	102.03	1363141	{}	\N	2020-10-12 17:00:54.192877
88288	PII	2020-10-06	102.26	103.921	101.05	101.42	1322531	{}	\N	2020-10-12 17:00:54.1979
88289	PII	2020-10-07	102.5	104.33	102.46	104	1096948	{}	\N	2020-10-12 17:00:54.204
88290	PII	2020-10-08	105	105	99.77	101.06	1448364	{}	\N	2020-10-12 17:00:54.209277
88291	PII	2020-10-09	101.5	101.8062	99.58	99.88	703955	{}	\N	2020-10-12 17:00:54.214392
88292	PII	2020-10-12	100.78	101.1524	98.51	98.7	320822	{}	\N	2020-10-12 17:00:54.219613
88293	PTNR	2020-09-28	3.67	3.9373	3.67	3.83	11160	{}	\N	2020-10-12 17:00:57.173951
88294	PTNR	2020-09-29	3.75	3.75	3.7167	3.7167	1323	{}	\N	2020-10-12 17:00:57.184158
88295	PTNR	2020-09-30	3.76	3.92	3.76	3.91	2331	{}	\N	2020-10-12 17:00:57.191441
88296	PTNR	2020-10-01	3.8	3.9173	3.78	3.85	5061	{}	\N	2020-10-12 17:00:57.197369
88297	PTNR	2020-10-02	3.76	3.95	3.73	3.85	18201	{}	\N	2020-10-12 17:00:57.202859
88298	PTNR	2020-10-05	3.85	4.21	3.85	4.02	24638	{}	\N	2020-10-12 17:00:57.207865
88299	PTNR	2020-10-06	4.02	4.13	3.83	3.97	13498	{}	\N	2020-10-12 17:00:57.213269
88300	PTNR	2020-10-07	4	4	3.84	3.88	19078	{}	\N	2020-10-12 17:00:57.218027
88301	PTNR	2020-10-08	3.93	4.05	3.8001	3.93	6708	{}	\N	2020-10-12 17:00:57.222788
88302	PTNR	2020-10-09	3.81	3.9793	3.8	3.886	5592	{}	\N	2020-10-12 17:00:57.227827
88303	PTNR	2020-10-12	3.87	4.06	3.83	4.06	6165	{}	\N	2020-10-12 17:00:57.232847
88304	QQEW	2020-09-28	85.85	86.45	85.58	86.45	116642	{}	\N	2020-10-12 17:00:58.83755
88305	QQEW	2020-09-29	86.3	86.6	85.9281	86.03	138885	{}	\N	2020-10-12 17:00:58.84718
88306	QQEW	2020-09-30	86.15	87.29	86.08	86.38	100805	{}	\N	2020-10-12 17:00:58.858556
88307	QQEW	2020-10-01	87.27	87.77	86.96	87.5775	95807	{}	\N	2020-10-12 17:00:58.866829
88308	QQEW	2020-10-02	85.82	86.88	85.41	85.73	69082	{}	\N	2020-10-12 17:00:58.874835
88309	QQEW	2020-10-05	86.41	87.42	86.41	87.42	74915	{}	\N	2020-10-12 17:00:58.88402
88310	QQEW	2020-10-06	87.21	88.16	86.12	86.33	111666	{}	\N	2020-10-12 17:00:58.890541
88311	QQEW	2020-10-07	87.21	88.13	87.19	87.97	75420	{}	\N	2020-10-12 17:00:58.900859
88312	QQEW	2020-10-08	88.48	88.7	88.185	88.7	120850	{}	\N	2020-10-12 17:00:58.911153
88313	QQEW	2020-10-09	89.31	89.9563	89.2138	89.82	45106	{}	\N	2020-10-12 17:00:58.920328
88314	QQEW	2020-10-12	90.87	91.8937	90.5299	91.72	20674	{}	\N	2020-10-12 17:00:58.929366
88315	QUAL	2020-09-28	103.11	103.62	102.76	103.36	860431	{}	\N	2020-10-12 17:01:00.694169
88316	QUAL	2020-09-29	103.5	103.6391	102.73	103.04	507660	{}	\N	2020-10-12 17:01:00.703244
88317	QUAL	2020-09-30	103.37	104.72	103.16	103.74	849227	{}	\N	2020-10-12 17:01:00.71504
88318	QUAL	2020-10-01	104.46	104.9	103.74	104.31	859204	{}	\N	2020-10-12 17:01:00.72557
88319	QUAL	2020-10-02	102.5408	104.13	102.38	103.48	574071	{}	\N	2020-10-12 17:01:00.73342
88320	QUAL	2020-10-05	104.43	105.23	104.2268	105.19	616020	{}	\N	2020-10-12 17:01:00.740879
88321	QUAL	2020-10-06	105.207	106.095	103.7323	103.92	537946	{}	\N	2020-10-12 17:01:00.748589
88322	QUAL	2020-10-07	104.91	105.84	104.7745	105.61	928609	{}	\N	2020-10-12 17:01:00.754957
88323	QUAL	2020-10-08	106.45	106.58	106.07	106.56	700759	{}	\N	2020-10-12 17:01:00.761866
88324	QUAL	2020-10-09	107.18	107.74	106.94	107.59	1051671	{}	\N	2020-10-12 17:01:00.768917
88325	QUAL	2020-10-12	108.4	109.218	108.13	109.1	208789	{}	\N	2020-10-12 17:01:00.775896
88326	QVM	2020-09-28	21.512	21.512	21.512	21.512	15	{}	\N	2020-10-12 17:01:04.148294
88327	QVM	2020-09-29	21.5045	21.5045	21.5045	21.5045	0	{}	\N	2020-10-12 17:01:04.156862
88328	QVM	2020-09-30	21.56	21.6	21.56	21.6	25100	{}	\N	2020-10-12 17:01:04.164046
88329	QVM	2020-10-01	21.5854	21.5854	21.5854	21.5854	57	{}	\N	2020-10-12 17:01:04.169823
88330	QVM	2020-10-02	21.5848	21.5848	21.5848	21.5848	1	{}	\N	2020-10-12 17:01:04.175453
88331	QVM	2020-10-05	21.648	21.648	21.648	21.648	120	{}	\N	2020-10-12 17:01:04.180767
88332	QVM	2020-10-06	21.628	21.628	21.628	21.628	1	{}	\N	2020-10-12 17:01:04.186285
88333	QVM	2020-10-07	21.6873	21.6873	21.6873	21.6873	19	{}	\N	2020-10-12 17:01:04.191233
88334	QVM	2020-10-08	21.7972	21.7972	21.7972	21.7972	11	{}	\N	2020-10-12 17:01:04.197525
88335	QVM	2020-10-09	21.855	21.855	21.855	21.855	60	{}	\N	2020-10-12 17:01:04.202724
88336	RFEU	2020-09-30	58.79	58.79	58.79	58.79	719	{}	\N	2020-10-12 17:01:05.208473
88337	RFEU	2020-10-01	58.76	58.98	58.76	58.98	229	{}	\N	2020-10-12 17:01:05.220119
88338	RFEU	2020-10-02	58.86	59.055	58.86	59.045	841	{}	\N	2020-10-12 17:01:05.227844
88339	RFEU	2020-10-05	59.64	59.64	59.635	59.635	648	{}	\N	2020-10-12 17:01:05.234385
88340	RFEU	2020-10-06	59.455	59.54	59.34	59.44	718	{}	\N	2020-10-12 17:01:05.240085
88341	RFEU	2020-10-07	59.265	59.28	59.265	59.28	925	{}	\N	2020-10-12 17:01:05.245085
88342	RFEU	2020-10-08	59.7	59.7	59.7	59.7	146	{}	\N	2020-10-12 17:01:05.251472
88343	RFEU	2020-10-09	60.4	60.6	60.4	60.51	1432	{}	\N	2020-10-12 17:01:05.257074
88344	RFEU	2020-10-12	60.78	60.78	60.78	60.78	284	{}	\N	2020-10-12 17:01:05.262058
88345	RING	2020-09-28	32.56	32.58	31.8401	32.26	102550	{}	\N	2020-10-12 17:01:06.317185
88346	RING	2020-09-29	32.42	32.91	32.3859	32.76	91749	{}	\N	2020-10-12 17:01:06.330533
88347	RING	2020-09-30	32.66	32.8	32.12	32.66	79930	{}	\N	2020-10-12 17:01:06.338971
88348	RING	2020-10-01	32.92	33.2087	32.65	32.88	81141	{}	\N	2020-10-12 17:01:06.34525
88349	RING	2020-10-02	32.56	32.8099	32.26	32.29	196911	{}	\N	2020-10-12 17:01:06.350511
88350	RING	2020-10-05	32.65	33.28	32.57	32.96	184089	{}	\N	2020-10-12 17:01:06.356099
88351	RING	2020-10-06	33.12	33.14	31.44	31.44	124160	{}	\N	2020-10-12 17:01:06.360904
88352	RING	2020-10-07	32.05	32.37	31.56	31.68	77474	{}	\N	2020-10-12 17:01:06.365225
88353	RING	2020-10-08	32.02	32.289	31.797	32.18	64986	{}	\N	2020-10-12 17:01:06.369151
88354	RING	2020-10-09	32.97	33.74	32.92	33.7027	402350	{}	\N	2020-10-12 17:01:06.373026
88355	RING	2020-10-12	33.72	33.87	33.29	33.8096	49734	{}	\N	2020-10-12 17:01:06.376981
88356	ROBO	2020-09-28	47.43	47.75	47.43	47.74	75177	{}	\N	2020-10-12 17:01:08.031953
88357	ROBO	2020-09-29	47.98	48.29	47.815	48.02	182279	{}	\N	2020-10-12 17:01:08.041776
88358	ROBO	2020-09-30	48.1	48.5	47.9874	48.15	105651	{}	\N	2020-10-12 17:01:08.048995
88359	ROBO	2020-10-01	48.41	48.7499	48.41	48.67	92749	{}	\N	2020-10-12 17:01:08.054766
88360	ROBO	2020-10-02	47.85	48.45	47.7	48.27	76557	{}	\N	2020-10-12 17:01:08.060151
88361	ROBO	2020-10-05	48.36	49.05	48.34	49.03	103305	{}	\N	2020-10-12 17:01:08.064581
88362	ROBO	2020-10-06	49.01	49.59	48.73	48.76	167607	{}	\N	2020-10-12 17:01:08.069397
88363	ROBO	2020-10-07	49.16	49.72	49.16	49.67	95646	{}	\N	2020-10-12 17:01:08.074468
88364	ROBO	2020-10-08	49.81	50.07	49.76	50.01	210359	{}	\N	2020-10-12 17:01:08.079357
88365	ROBO	2020-10-09	50.27	50.7	50.27	50.66	116179	{}	\N	2020-10-12 17:01:08.084116
88366	ROBO	2020-10-12	50.76	51.128	50.75	50.9899	49349	{}	\N	2020-10-12 17:01:08.089204
88367	RRD	2020-09-28	1.41	1.55	1.41	1.47	659141	{}	\N	2020-10-12 17:01:09.02852
88368	RRD	2020-09-29	1.46	1.57	1.43	1.52	1590522	{}	\N	2020-10-12 17:01:09.040109
88369	RRD	2020-09-30	1.51	1.58	1.44	1.46	935338	{}	\N	2020-10-12 17:01:09.047914
88370	RRD	2020-10-01	1.47	1.51	1.41	1.48	1559131	{}	\N	2020-10-12 17:01:09.055063
88371	RRD	2020-10-02	1.43	1.47	1.4	1.46	418831	{}	\N	2020-10-12 17:01:09.061337
88372	RRD	2020-10-05	1.46	1.48	1.41	1.43	587158	{}	\N	2020-10-12 17:01:09.066536
88373	RRD	2020-10-06	1.44	1.49	1.4345	1.45	981422	{}	\N	2020-10-12 17:01:09.072112
88374	RRD	2020-10-07	1.47	1.49	1.43	1.46	545398	{}	\N	2020-10-12 17:01:09.077325
88375	RRD	2020-10-08	1.47	1.49	1.44	1.47	537078	{}	\N	2020-10-12 17:01:09.082566
88376	RRD	2020-10-09	1.46	1.47	1.39	1.4	1152200	{}	\N	2020-10-12 17:01:09.088224
88377	RRD	2020-10-12	1.4	1.4244	1.255	1.32	1127893	{}	\N	2020-10-12 17:01:09.09362
88378	RSX	2020-09-28	20.87	20.95	20.7	20.83	3501323	{}	\N	2020-10-12 17:01:10.671792
88379	RSX	2020-09-29	20.74	20.84	20.53	20.72	3285880	{}	\N	2020-10-12 17:01:10.683596
88380	RSX	2020-09-30	20.87	21.06	20.855	20.95	3462084	{}	\N	2020-10-12 17:01:10.692837
88381	RSX	2020-10-01	20.97	21.01	20.8	20.88	4198255	{}	\N	2020-10-12 17:01:10.699088
88382	RSX	2020-10-02	20.59	20.79	20.59	20.7	3150814	{}	\N	2020-10-12 17:01:10.705829
88383	RSX	2020-10-05	20.75	21.04	20.75	21.04	2796452	{}	\N	2020-10-12 17:01:10.711507
88384	RSX	2020-10-06	21.23	21.25	20.91	21.01	5153907	{}	\N	2020-10-12 17:01:10.716433
88385	RSX	2020-10-07	20.8	20.855	20.69	20.78	3781541	{}	\N	2020-10-12 17:01:10.721561
88386	RSX	2020-10-08	21.01	21.095	20.93	21.04	3166285	{}	\N	2020-10-12 17:01:10.726802
88387	RSX	2020-10-09	21.13	21.25	21.095	21.18	2675029	{}	\N	2020-10-12 17:01:10.731462
88388	RSX	2020-10-12	21.3	21.305	21.2	21.25	1744515	{}	\N	2020-10-12 17:01:10.737182
88389	PICB	2020-09-28	28.01	28.01	27.9	28	3038	{}	\N	2020-10-12 17:01:11.755327
88390	PICB	2020-09-29	27.99	28.09	27.99	28.07	3844	{}	\N	2020-10-12 17:01:11.767898
88391	PICB	2020-09-30	28.09	28.13	28.07	28.07	4083	{}	\N	2020-10-12 17:01:11.775935
88392	PICB	2020-10-01	28.2	28.22	27.92	28.1799	143239	{}	\N	2020-10-12 17:01:11.781759
88393	PICB	2020-10-02	28.04	28.3287	28.04	28.09	16249	{}	\N	2020-10-12 17:01:11.787567
88394	PICB	2020-10-05	28.26	28.39	28.15	28.32	18342	{}	\N	2020-10-12 17:01:11.792815
88395	PICB	2020-10-06	28.31	28.33	28.16	28.23	19542	{}	\N	2020-10-12 17:01:11.797833
88396	PICB	2020-10-07	28.2074	28.24	28.16	28.16	26626	{}	\N	2020-10-12 17:01:11.803166
88397	PICB	2020-10-08	28.14	28.44	28.14	28.21	24214	{}	\N	2020-10-12 17:01:11.808375
88398	PICB	2020-10-09	28.31	28.52	28.29	28.52	8414	{}	\N	2020-10-12 17:01:11.813631
88399	PICB	2020-10-12	28.52	28.56	28.4197	28.56	7010	{}	\N	2020-10-12 17:01:11.818905
88400	PRMW	2020-09-28	13.98	14.26	13.98	14.19	612460	{}	\N	2020-10-12 17:01:12.817977
88401	PRMW	2020-09-29	14.26	14.26	13.94	13.95	1527957	{}	\N	2020-10-12 17:01:12.829269
88402	PRMW	2020-09-30	14.02	14.35	14.02	14.2	1813726	{}	\N	2020-10-12 17:01:12.83646
88403	PRMW	2020-10-01	14.24	14.41	14.08	14.26	1640364	{}	\N	2020-10-12 17:01:12.843239
88404	PRMW	2020-10-02	13.97	14.38	13.92	14.2	1427871	{}	\N	2020-10-12 17:01:12.848447
88405	PRMW	2020-10-05	14.29	14.36	14.01	14.28	1791369	{}	\N	2020-10-12 17:01:12.853416
88406	PRMW	2020-10-06	14.39	14.52	14.18	14.25	1338998	{}	\N	2020-10-12 17:01:12.858162
88407	PRMW	2020-10-07	14.32	14.54	14.3	14.45	1503834	{}	\N	2020-10-12 17:01:12.862595
88408	PRMW	2020-10-08	14.58	14.64	14.39	14.44	2571839	{}	\N	2020-10-12 17:01:12.867476
88409	PRMW	2020-10-09	14.47	14.56	14.245	14.4	1460133	{}	\N	2020-10-12 17:01:12.872386
88410	PRMW	2020-10-12	14.32	14.56	14.32	14.48	616793	{}	\N	2020-10-12 17:01:12.877114
88411	PSCD	2020-09-28	63.495	63.495	63.3369	63.3369	539	{}	\N	2020-10-12 17:01:16.692411
88412	PSCD	2020-09-30	63.57	63.57	62.9129	62.9129	2948	{}	\N	2020-10-12 17:01:16.706321
88413	PSCD	2020-10-01	63.38	65.07	63.185	65.01	12409	{}	\N	2020-10-12 17:01:16.716179
88414	PSCD	2020-10-02	63.4	65.93	63.4	65.93	1199	{}	\N	2020-10-12 17:01:16.723393
88415	PSCD	2020-10-05	67.11	67.77	66.995	67.77	1429	{}	\N	2020-10-12 17:01:16.728799
88416	PSCD	2020-10-06	68.68	69.43	68.17	69.43	4447	{}	\N	2020-10-12 17:01:16.734103
88417	PSCD	2020-10-07	68.84	68.84	68.14	68.63	891	{}	\N	2020-10-12 17:01:16.73948
88418	PSCD	2020-10-09	75.55	75.55	69.97	70.36	3637	{}	\N	2020-10-12 17:01:16.744586
88419	PSCD	2020-10-12	75.58	75.58	70.3652	70.43	4169	{}	\N	2020-10-12 17:01:16.749396
88420	QQQE	2020-09-28	63.8	64.35	63.8	64.29	19285	{}	\N	2020-10-12 17:01:17.934083
88421	QQQE	2020-09-29	64.32	64.56	64.01	64.09	48979	{}	\N	2020-10-12 17:01:17.944322
88422	QQQE	2020-09-30	64.245	65.01	64.11	64.48	19059	{}	\N	2020-10-12 17:01:17.965814
88423	QQQE	2020-10-01	65.18	65.41	64.82	65.41	38516	{}	\N	2020-10-12 17:01:17.981028
88424	QQQE	2020-10-02	63.96	64.75	63.74	63.86	29854	{}	\N	2020-10-12 17:01:17.988117
88425	QQQE	2020-10-05	64.24	65.16	64.24	65.13	28162	{}	\N	2020-10-12 17:01:17.99373
88426	QQQE	2020-10-06	65.21	65.62	64.17	64.29	20015	{}	\N	2020-10-12 17:01:17.999624
88427	QQQE	2020-10-07	64.96	65.6613	64.96	65.58	21338	{}	\N	2020-10-12 17:01:18.006565
88428	QQQE	2020-10-08	65.96	66.1	65.8401	66.1	129226	{}	\N	2020-10-12 17:01:18.014964
88429	QQQE	2020-10-09	66.57	67.029	66.43	66.92	48114	{}	\N	2020-10-12 17:01:18.02299
88430	QQQE	2020-10-12	67.6	68.4357	67.4475	68.41	34703	{}	\N	2020-10-12 17:01:18.029084
88431	QTWO	2020-09-28	93.53	94.24	91.73	92.89	365425	{}	\N	2020-10-12 17:01:18.931231
88432	QTWO	2020-09-29	92.96	93.61	91.36	91.65	431645	{}	\N	2020-10-12 17:01:18.942008
88433	QTWO	2020-09-30	91.34	93.34	90.47	91.26	694910	{}	\N	2020-10-12 17:01:18.950703
88434	QTWO	2020-10-01	92.3	94	91.68	93.77	604560	{}	\N	2020-10-12 17:01:18.957966
88435	QTWO	2020-10-02	91.84	93.78	90.83	92.08	391411	{}	\N	2020-10-12 17:01:18.964065
88436	QTWO	2020-10-05	92.53	95.2	92.53	94.04	745015	{}	\N	2020-10-12 17:01:18.971036
88437	QTWO	2020-10-06	94.63	98.205	94.26	96.57	869690	{}	\N	2020-10-12 17:01:18.976206
88438	QTWO	2020-10-07	97.17	98.43	97.17	98.12	558066	{}	\N	2020-10-12 17:01:18.981771
88439	QTWO	2020-10-08	99.47	99.47	97.59	97.86	1977029	{}	\N	2020-10-12 17:01:18.9878
88440	QTWO	2020-10-09	98.58	98.86	96.91	97.84	987925	{}	\N	2020-10-12 17:01:18.993103
88441	QTWO	2020-10-12	98.81	99.27	97.0509	98.68	173576	{}	\N	2020-10-12 17:01:18.998149
88442	RDVY	2020-09-28	32.49	32.79	32.44	32.68	146869	{}	\N	2020-10-12 17:01:19.898372
88443	RDVY	2020-09-29	32.65	32.65	32.25	32.42	158073	{}	\N	2020-10-12 17:01:19.909287
88444	RDVY	2020-09-30	32.51	32.96	32.4714	32.65	266829	{}	\N	2020-10-12 17:01:19.917323
88445	RDVY	2020-10-01	32.85	33.0151	32.6179	32.83	458165	{}	\N	2020-10-12 17:01:19.924625
88446	RDVY	2020-10-02	32.13	33.03	32.13	32.84	237744	{}	\N	2020-10-12 17:01:19.930581
88447	RDVY	2020-10-05	33.18	33.59	33.18	33.57	273789	{}	\N	2020-10-12 17:01:19.936237
88448	RDVY	2020-10-06	33.75	34.11	33.14	33.22	217659	{}	\N	2020-10-12 17:01:19.941963
88449	RDVY	2020-10-07	33.59	33.925	33.59	33.87	1414157	{}	\N	2020-10-12 17:01:19.947205
88450	RDVY	2020-10-08	34.04	34.26	33.98	34.26	249230	{}	\N	2020-10-12 17:01:19.952994
88451	RDVY	2020-10-09	34.4462	34.56	34.25	34.41	180252	{}	\N	2020-10-12 17:01:19.958582
88452	RDVY	2020-10-12	34.58	34.865	34.55	34.85	151672	{}	\N	2020-10-12 17:01:19.964264
88453	RETL	2020-09-28	215.2	222.01	215.2	221.7455	10828	{}	\N	2020-10-12 17:01:20.916055
88454	RETL	2020-09-29	226.16	226.16	209.3004	213.5753	11558	{}	\N	2020-10-12 17:01:20.92731
88455	RETL	2020-09-30	213.49	223	212.1039	216.44	6184	{}	\N	2020-10-12 17:01:20.935097
88456	RETL	2020-10-01	223.72	235.1019	221	234.12	10999	{}	\N	2020-10-12 17:01:20.942463
88457	RETL	2020-10-02	218.93	239.327	217.075	238.6301	17623	{}	\N	2020-10-12 17:01:20.949121
88458	RETL	2020-10-05	244.82	258.65	244.82	258.45	16936	{}	\N	2020-10-12 17:01:20.954985
88459	RETL	2020-10-06	263.2	265.45	242.1569	243.83	23023	{}	\N	2020-10-12 17:01:20.961864
88460	RETL	2020-10-07	250.9	265.44	250.9	263.97	17865	{}	\N	2020-10-12 17:01:20.968082
88461	RETL	2020-10-08	267.71	275.3682	264.12	275.3682	16086	{}	\N	2020-10-12 17:01:20.973912
88462	RETL	2020-10-09	281.81	284.7462	275.29	275.3611	18598	{}	\N	2020-10-12 17:01:20.980096
88463	RETL	2020-10-12	282.86	283.7851	279.479	280.29	7514	{}	\N	2020-10-12 17:01:20.989211
88464	RIBT	2020-09-28	0.432	0.432	0.3843	0.4	294905	{}	\N	2020-10-12 17:01:22.094769
88465	RIBT	2020-09-29	0.4	0.416	0.4	0.4115	202098	{}	\N	2020-10-12 17:01:22.105697
88466	RIBT	2020-09-30	0.4099	0.4299	0.4002	0.42	247114	{}	\N	2020-10-12 17:01:22.113447
88467	RIBT	2020-10-01	0.4214	0.4256	0.41	0.4101	39044	{}	\N	2020-10-12 17:01:22.119689
88468	RIBT	2020-10-02	0.4	0.408799	0.3922	0.3943	192584	{}	\N	2020-10-12 17:01:22.125058
88469	RIBT	2020-10-05	0.395	0.402	0.392	0.392	142499	{}	\N	2020-10-12 17:01:22.130404
88470	RIBT	2020-10-06	0.399	0.3997	0.372	0.3902	494817	{}	\N	2020-10-12 17:01:22.134922
88471	RIBT	2020-10-07	0.395	0.3988	0.377	0.3862	310833	{}	\N	2020-10-12 17:01:22.139791
88472	RIBT	2020-10-08	0.39	0.395	0.3786	0.392	372224	{}	\N	2020-10-12 17:01:22.144569
88473	RIBT	2020-10-09	0.4003	0.429	0.395	0.426	286007	{}	\N	2020-10-12 17:01:22.149283
88474	RIBT	2020-10-12	0.44	0.4674	0.42	0.455021	374877	{}	\N	2020-10-12 17:01:22.153542
88475	RWJ	2020-09-28	59.11	60.705	59.11	60.6963	10439	{}	\N	2020-10-12 17:01:23.429209
88476	RWJ	2020-09-29	59.86	59.92	58.989	59.5715	8309	{}	\N	2020-10-12 17:01:23.437013
88477	RWJ	2020-09-30	59.94	60.43	59.555	59.58	8808	{}	\N	2020-10-12 17:01:23.446208
88478	RWJ	2020-10-01	60.27	60.455	59.795	60.2767	8155	{}	\N	2020-10-12 17:01:23.452681
88479	RWJ	2020-10-02	59.5939	61.696	59.5939	61.696	2999	{}	\N	2020-10-12 17:01:23.458128
88480	RWJ	2020-10-05	61.89	63.215	61.89	63.16	9926	{}	\N	2020-10-12 17:01:23.465043
88481	RWJ	2020-10-06	63.62	64.88	62.96	62.96	7817	{}	\N	2020-10-12 17:01:23.473946
88482	RWJ	2020-10-07	63.98	64.4994	63.8014	64.4994	7255	{}	\N	2020-10-12 17:01:23.480777
88483	RWJ	2020-10-08	65.25	65.96	64.99	65.96	7933	{}	\N	2020-10-12 17:01:23.486819
88484	RWJ	2020-10-09	66.36	66.5621	65.8796	65.8796	9053	{}	\N	2020-10-12 17:01:23.491834
88485	RWJ	2020-10-12	65.97	66.1838	65.97	66.0999	2354	{}	\N	2020-10-12 17:01:23.498437
88486	RWO	2020-09-28	39.36	39.821	39.36	39.71	137705	{}	\N	2020-10-12 17:01:26.344871
88487	RWO	2020-09-29	39.58	39.59	39.04	39.25	93919	{}	\N	2020-10-12 17:01:26.352129
88488	RWO	2020-09-30	39.38	39.81	39.1215	39.5	338485	{}	\N	2020-10-12 17:01:26.357884
88489	RWO	2020-10-01	39.62	40.32	39.5747	40.24	191648	{}	\N	2020-10-12 17:01:26.362694
88490	RWO	2020-10-02	39.7597	40.88	39.7597	40.8	126016	{}	\N	2020-10-12 17:01:26.36769
88491	RWO	2020-10-05	41.168	41.18	40.61	41.07	105371	{}	\N	2020-10-12 17:01:26.375002
88492	RWO	2020-10-06	41.15	41.41	40.76	40.85	153301	{}	\N	2020-10-12 17:01:26.379781
88493	RWO	2020-10-07	41.09	41.09	40.72	40.84	119452	{}	\N	2020-10-12 17:01:26.385446
88494	RWO	2020-10-08	41.04	41.36	41.04	41.32	105104	{}	\N	2020-10-12 17:01:26.391154
88495	RWO	2020-10-09	41.49	41.49	41.0506	41.23	100221	{}	\N	2020-10-12 17:01:26.395915
88496	RWO	2020-10-12	41.27	41.355	41.0598	41.275	20148	{}	\N	2020-10-12 17:01:26.400665
88497	SA	2020-09-28	18.6	18.69	18.08	18.49	179271	{}	\N	2020-10-12 17:01:27.588644
88498	SA	2020-09-29	18.52	18.95	18.46	18.89	370583	{}	\N	2020-10-12 17:01:27.602829
88499	SA	2020-09-30	18.72	19.04	18.49	18.78	402676	{}	\N	2020-10-12 17:01:27.610113
88500	SA	2020-10-01	18.84	19.1983	18.685	18.97	348211	{}	\N	2020-10-12 17:01:27.616056
88501	SA	2020-10-02	18.79	19	18.62	18.78	301666	{}	\N	2020-10-12 17:01:27.621747
88502	SA	2020-10-05	18.79	19.36	18.71	19.14	358334	{}	\N	2020-10-12 17:01:27.62732
88503	SA	2020-10-06	19.2	19.4	18.48	18.5	314136	{}	\N	2020-10-12 17:01:27.631602
88504	SA	2020-10-07	18.77	18.8319	18.28	18.36	330414	{}	\N	2020-10-12 17:01:27.636217
88505	SA	2020-10-08	18.54	18.96	18.52	18.95	326865	{}	\N	2020-10-12 17:01:27.640847
88506	SA	2020-10-09	19.42	19.93	19.3243	19.87	488789	{}	\N	2020-10-12 17:01:27.64603
88507	SA	2020-10-12	19.63	19.87	19.54	19.8299	159144	{}	\N	2020-10-12 17:01:27.650631
88508	PHYS	2020-09-28	15	15	14.915	15	2342941	{}	\N	2020-10-12 17:01:28.932146
88509	PHYS	2020-09-29	15.13	15.13	15.02	15.12	1948239	{}	\N	2020-10-12 17:01:28.944266
88510	PHYS	2020-09-30	15.11	15.18	15	15.05	1594928	{}	\N	2020-10-12 17:01:28.951868
88511	PHYS	2020-10-01	15.1999	15.1999	15.1	15.13	2374235	{}	\N	2020-10-12 17:01:28.958194
88512	PHYS	2020-10-02	15.08	15.187	15.08	15.15	1265526	{}	\N	2020-10-12 17:01:28.963869
88513	PHYS	2020-10-05	15.2	15.29	15.195	15.27	1575023	{}	\N	2020-10-12 17:01:28.96873
88514	PHYS	2020-10-06	15.35	15.35	15.04	15.08	1288790	{}	\N	2020-10-12 17:01:28.973603
88515	PHYS	2020-10-07	15.09	15.09	14.99	15.06	1345254	{}	\N	2020-10-12 17:01:28.978379
88516	PHYS	2020-10-08	15.13	15.16	15.03	15.13	673397	{}	\N	2020-10-12 17:01:28.983274
88517	PHYS	2020-10-09	15.23	15.34	15.2	15.28	3715499	{}	\N	2020-10-12 17:01:28.987716
88518	PHYS	2020-10-12	15.29	15.3225	15.22	15.3225	728576	{}	\N	2020-10-12 17:01:28.992363
88519	PRO	2020-09-28	32.58	33.04	31.98	32.03	293939	{}	\N	2020-10-12 17:01:30.190534
88520	PRO	2020-09-29	32.25	32.41	31.62	32.07	300050	{}	\N	2020-10-12 17:01:30.201313
88521	PRO	2020-09-30	32.04	32.8	31.76	31.94	491332	{}	\N	2020-10-12 17:01:30.208732
88522	PRO	2020-10-01	32.16	32.47	31.39	32.18	663496	{}	\N	2020-10-12 17:01:30.214552
88523	PRO	2020-10-02	31.35	32.7	31.05	32.25	539960	{}	\N	2020-10-12 17:01:30.219431
88524	PRO	2020-10-05	32.8	32.88	32.24	32.71	508058	{}	\N	2020-10-12 17:01:30.224181
88525	PRO	2020-10-06	33.05	33.81	32.47	32.63	450547	{}	\N	2020-10-12 17:01:30.229363
88526	PRO	2020-10-07	32.85	33.89	32.71	33.64	526582	{}	\N	2020-10-12 17:01:30.234138
88527	PRO	2020-10-08	34.21	34.72	33.63	34.65	309468	{}	\N	2020-10-12 17:01:30.238855
88528	PRO	2020-10-09	34.63	34.86	33.86	34.51	360612	{}	\N	2020-10-12 17:01:30.244367
88529	PRO	2020-10-12	34.61	35.44	34.26	35.395	126211	{}	\N	2020-10-12 17:01:30.249762
88530	PSCE	2020-09-28	3.01	3.0905	2.985	3.06	28369	{}	\N	2020-10-12 17:01:31.223576
88531	PSCE	2020-09-29	3.04	3.1	2.94	3.01	11936	{}	\N	2020-10-12 17:01:31.235125
88532	PSCE	2020-09-30	3.06	3.09	3.01	3.03	5594	{}	\N	2020-10-12 17:01:31.243877
88533	PSCE	2020-10-01	3.03	3.03	2.9	2.92	46485	{}	\N	2020-10-12 17:01:31.250372
88534	PSCE	2020-10-02	2.82	3.0499	2.82	3.04	13230	{}	\N	2020-10-12 17:01:31.256317
88535	PSCE	2020-10-05	3	3.129	3	3.12	124006	{}	\N	2020-10-12 17:01:31.261823
88536	PSCE	2020-10-06	3.2	3.2299	3.09	3.12	57764	{}	\N	2020-10-12 17:01:31.267505
88537	PSCE	2020-10-07	3.1426	3.25	3.1426	3.25	2974	{}	\N	2020-10-12 17:01:31.272631
88538	PSCE	2020-10-08	3.27	3.35	3.245	3.33	19973	{}	\N	2020-10-12 17:01:31.277994
88539	PSCE	2020-10-09	3.37	3.38	3.2947	3.3156	36025	{}	\N	2020-10-12 17:01:31.283197
88540	PSCE	2020-10-12	3.295	3.3109	3.28	3.303	4919	{}	\N	2020-10-12 17:01:31.288429
88541	QUOT	2020-09-28	7.49	7.63	7.43	7.5	278862	{}	\N	2020-10-12 17:01:33.534642
88542	QUOT	2020-09-29	7.55	7.57	7.32	7.55	409246	{}	\N	2020-10-12 17:01:33.549086
88543	QUOT	2020-09-30	7.64	7.645	7.36	7.38	713365	{}	\N	2020-10-12 17:01:33.557494
88544	QUOT	2020-10-01	7.39	7.71	7.33	7.71	1133740	{}	\N	2020-10-12 17:01:33.564308
88545	QUOT	2020-10-02	7.5	7.8	7.48	7.78	602754	{}	\N	2020-10-12 17:01:33.570322
88546	QUOT	2020-10-05	7.85	8.16	7.8	8.12	702805	{}	\N	2020-10-12 17:01:33.575732
88547	QUOT	2020-10-06	8.26	8.26	7.73	7.75	717316	{}	\N	2020-10-12 17:01:33.581151
88548	QUOT	2020-10-07	7.91	8.08	7.87	8.07	938135	{}	\N	2020-10-12 17:01:33.586204
88549	QUOT	2020-10-08	8.22	8.41	8.17	8.34	654932	{}	\N	2020-10-12 17:01:33.59069
88550	QUOT	2020-10-09	8.51	8.95	8.49	8.63	971629	{}	\N	2020-10-12 17:01:33.595436
88551	QUOT	2020-10-12	8.61	8.7	8.438	8.52	203576	{}	\N	2020-10-12 17:01:33.600707
88552	RESI	2020-09-28	8.75	9.09	8.75	8.96	258082	{}	\N	2020-10-12 17:01:34.887165
88553	RESI	2020-09-29	8.96	9	8.68	8.84	434747	{}	\N	2020-10-12 17:01:34.899713
88554	RESI	2020-09-30	8.91	9.07	8.69	8.74	750595	{}	\N	2020-10-12 17:01:34.907133
88555	RESI	2020-10-01	8.72	9.07	8.72	9.06	538298	{}	\N	2020-10-12 17:01:34.912699
88556	RESI	2020-10-02	8.91	9.28	8.895	9.27	560991	{}	\N	2020-10-12 17:01:34.917546
88557	RESI	2020-10-05	9.3	9.45	9.14	9.44	565528	{}	\N	2020-10-12 17:01:34.922367
88558	RESI	2020-10-06	9.48	9.63	9.21	9.25	855331	{}	\N	2020-10-12 17:01:34.927232
88559	RESI	2020-10-07	9.26	9.36	9.13	9.24	474118	{}	\N	2020-10-12 17:01:34.932136
88560	RESI	2020-10-08	9.56	9.93	9.46	9.63	1478945	{}	\N	2020-10-12 17:01:34.937103
88561	RESI	2020-10-09	9.61	9.75	9.5	9.58	525427	{}	\N	2020-10-12 17:01:34.94203
88562	RESI	2020-10-12	9.54	9.72	9.44	9.62	144793	{}	\N	2020-10-12 17:01:34.94682
88563	REZ	2020-09-28	60.98	61.83	60.93	61.51	35217	{}	\N	2020-10-12 17:01:36.928077
88564	REZ	2020-09-29	61.47	61.47	60.145	60.74	32880	{}	\N	2020-10-12 17:01:36.939236
88565	REZ	2020-09-30	60.91	61.52	60.3003	60.97	23665	{}	\N	2020-10-12 17:01:36.94727
88566	REZ	2020-10-01	61.28	62.91	61.28	62.91	62704	{}	\N	2020-10-12 17:01:36.953406
88567	REZ	2020-10-02	61.84	64.5399	61.7	64.36	30637	{}	\N	2020-10-12 17:01:36.95913
88568	REZ	2020-10-05	64.3	64.77	63.37	64.51	33927	{}	\N	2020-10-12 17:01:36.964669
88569	REZ	2020-10-06	64.55	64.9374	63.77	64.01	46417	{}	\N	2020-10-12 17:01:36.969841
88570	REZ	2020-10-07	64	64.28	63.43	63.88	27165	{}	\N	2020-10-12 17:01:36.974675
88571	REZ	2020-10-08	64.06	65.05	64.06	64.93	28162	{}	\N	2020-10-12 17:01:36.979549
88572	REZ	2020-10-09	65.23	65.23	64.2117	64.47	28192	{}	\N	2020-10-12 17:01:36.984547
88573	REZ	2020-10-12	64.35	64.6291	64	64.31	13948	{}	\N	2020-10-12 17:01:36.990214
88574	RVT	2020-09-28	12.6025	12.6801	12.55	12.59	292640	{}	\N	2020-10-12 17:01:38.367834
88575	RVT	2020-09-29	12.63	12.68	12.52	12.58	376114	{}	\N	2020-10-12 17:01:38.379451
88576	RVT	2020-09-30	12.58	12.82	12.5755	12.64	576553	{}	\N	2020-10-12 17:01:38.387764
88577	RVT	2020-10-01	12.67	12.84	12.64	12.76	374636	{}	\N	2020-10-12 17:01:38.39325
88578	RVT	2020-10-02	12.61	12.92	12.595	12.9	530986	{}	\N	2020-10-12 17:01:38.398325
88579	RVT	2020-10-05	12.98	13.24	12.98	13.24	506231	{}	\N	2020-10-12 17:01:38.404419
88580	RVT	2020-10-06	13.33	13.4873	13.24	13.27	506048	{}	\N	2020-10-12 17:01:38.408641
88581	RVT	2020-10-07	13.39	13.465	13.345	13.41	515035	{}	\N	2020-10-12 17:01:38.412912
88582	RVT	2020-10-08	13.5	13.59	13.46	13.53	330530	{}	\N	2020-10-12 17:01:38.417684
88583	RVT	2020-10-09	13.63	13.73	13.56	13.62	545009	{}	\N	2020-10-12 17:01:38.423024
88584	RVT	2020-10-12	13.71	13.73	13.645	13.725	201824	{}	\N	2020-10-12 17:01:38.42813
88585	RWR	2020-09-28	77.69	79.055	77.69	78.72	122565	{}	\N	2020-10-12 17:01:39.546219
88586	RWR	2020-09-29	78.5	78.51	76.98	77.82	110740	{}	\N	2020-10-12 17:01:39.558608
88587	RWR	2020-09-30	78.13	78.99	77.14	77.97	84039	{}	\N	2020-10-12 17:01:39.566797
88588	RWR	2020-10-01	78.26	80.08	78.09	80.03	99757	{}	\N	2020-10-12 17:01:39.573638
88589	RWR	2020-10-02	78.64	81.95	78.64	81.68	56225	{}	\N	2020-10-12 17:01:39.579971
88590	RWR	2020-10-05	82.12	82.22	80.64	82.06	167055	{}	\N	2020-10-12 17:01:39.585475
88591	RWR	2020-10-06	82.23	82.99	81.39	81.67	398256	{}	\N	2020-10-12 17:01:39.590628
88592	RWR	2020-10-07	82.18	82.18	81.25	81.62	56606	{}	\N	2020-10-12 17:01:39.595806
88593	RWR	2020-10-08	82.01	83.19	82.01	83.14	305202	{}	\N	2020-10-12 17:01:39.60098
88594	RWR	2020-10-09	83.52	83.52	82.35	82.59	76503	{}	\N	2020-10-12 17:01:39.606088
88595	RWR	2020-10-12	82.51	82.81	81.9039	82.51	27761	{}	\N	2020-10-12 17:01:39.611426
88596	RXD	2020-09-28	11.93	11.93	11.805	11.8778	6425	{}	\N	2020-10-12 17:01:40.623809
88597	RXD	2020-09-29	11.86	11.9	11.7359	11.89	3336	{}	\N	2020-10-12 17:01:40.637332
88598	RXD	2020-09-30	11.83	11.83	11.461	11.461	3694	{}	\N	2020-10-12 17:01:40.64476
88599	RXD	2020-10-01	11.4	11.669	11.4	11.669	3997	{}	\N	2020-10-12 17:01:40.651006
88600	RXD	2020-10-02	11.73	11.7857	11.73	11.7857	480	{}	\N	2020-10-12 17:01:40.657479
88601	RXD	2020-10-05	11.73	11.73	11.3008	11.3008	2095	{}	\N	2020-10-12 17:01:40.662669
88602	RXD	2020-10-06	11.3	11.5381	11.28	11.5381	2193	{}	\N	2020-10-12 17:01:40.667775
88603	RXD	2020-10-07	11.0791	11.0791	11.0791	11.0791	417	{}	\N	2020-10-12 17:01:40.672994
88604	RXD	2020-10-08	10.9865	10.9865	10.9865	10.9865	717	{}	\N	2020-10-12 17:01:40.678171
88605	RXD	2020-10-09	10.88	10.89	10.7699	10.85	8154	{}	\N	2020-10-12 17:01:40.683022
88606	RXD	2020-10-12	10.6764	10.68	10.6764	10.68	270	{}	\N	2020-10-12 17:01:40.687994
88607	RYAM	2020-09-28	3.285	3.46	3.285	3.41	397432	{}	\N	2020-10-12 17:01:42.651389
88608	RYAM	2020-09-29	3.43	3.43	3.045	3.21	1005598	{}	\N	2020-10-12 17:01:42.664289
88609	RYAM	2020-09-30	3.22	3.32	3.18	3.2	896611	{}	\N	2020-10-12 17:01:42.673155
88610	RYAM	2020-10-01	3.21	3.24	3.01	3.05	944847	{}	\N	2020-10-12 17:01:42.679639
88611	RYAM	2020-10-02	3	3.46	3	3.42	1379073	{}	\N	2020-10-12 17:01:42.684361
88612	RYAM	2020-10-05	3.43	3.86	3.43	3.82	1561707	{}	\N	2020-10-12 17:01:42.689377
88613	RYAM	2020-10-06	3.87	4.14	3.8238	4.01	2019243	{}	\N	2020-10-12 17:01:42.69519
88614	RYAM	2020-10-07	4.11	4.23	3.99	4.1	1222194	{}	\N	2020-10-12 17:01:42.700303
88615	RYAM	2020-10-08	4.12	4.26	4.06	4.14	724598	{}	\N	2020-10-12 17:01:42.704697
88616	RYAM	2020-10-09	4.23	4.24	4.02	4.06	623000	{}	\N	2020-10-12 17:01:42.709167
88617	RYAM	2020-10-12	4.06	4.06	3.77	3.78	422234	{}	\N	2020-10-12 17:01:42.713912
88618	RYU	2020-09-28	93.91	94.18	93.23	93.248	20734	{}	\N	2020-10-12 17:01:44.251305
88619	RYU	2020-09-29	93.69	93.8	92.9754	93.6725	5890	{}	\N	2020-10-12 17:01:44.264178
88620	RYU	2020-09-30	93.74	94.79	93.74	94.36	13676	{}	\N	2020-10-12 17:01:44.271789
88621	RYU	2020-10-01	94.74	95.198	94.24	94.7645	10575	{}	\N	2020-10-12 17:01:44.277839
88622	RYU	2020-10-02	94.11	96.7409	94.11	96.43	9145	{}	\N	2020-10-12 17:01:44.283494
88623	RYU	2020-10-05	96.31	97.48	96.14	97.33	47602	{}	\N	2020-10-12 17:01:44.289134
88624	RYU	2020-10-06	97.1	99.095	97.1	98.41	10731	{}	\N	2020-10-12 17:01:44.294152
88625	RYU	2020-10-07	98.75	99.12	98.0001	98.7611	19924	{}	\N	2020-10-12 17:01:44.299404
88626	RYU	2020-10-08	99.4499	100.67	99.4499	100.48	8033	{}	\N	2020-10-12 17:01:44.303881
88627	RYU	2020-10-09	101.14	101.14	99.86	100.3245	15474	{}	\N	2020-10-12 17:01:44.308407
88628	RYU	2020-10-12	100.58	101.36	100.58	101.22	2537	{}	\N	2020-10-12 17:01:44.313366
88629	SCHD	2020-09-28	55	55.665	55	55.46	2074450	{}	\N	2020-10-12 17:01:45.516449
88630	SCHD	2020-09-29	55.4438	55.5	54.67	54.92	1602079	{}	\N	2020-10-12 17:01:45.526566
88631	SCHD	2020-09-30	55.09	55.83	54.89	55.3	1633625	{}	\N	2020-10-12 17:01:45.534453
88632	SCHD	2020-10-01	55.58	55.76	54.8839	55.27	1078165	{}	\N	2020-10-12 17:01:45.540527
88633	SCHD	2020-10-02	54.44	55.83	54.37	55.52	1619763	{}	\N	2020-10-12 17:01:45.546717
88634	SCHD	2020-10-05	56.024	56.805	56.02	56.73	977103	{}	\N	2020-10-12 17:01:45.551621
88635	SCHD	2020-10-06	56.95	57.71	56.24	56.36	1657545	{}	\N	2020-10-12 17:01:45.556341
88636	SCHD	2020-10-07	56.88	57.63	56.88	57.51	1083940	{}	\N	2020-10-12 17:01:45.561054
88637	SCHD	2020-10-08	58	58.4343	57.878	58.37	1227759	{}	\N	2020-10-12 17:01:45.565676
88638	SCHD	2020-10-09	58.77	58.88	58.3201	58.48	1352212	{}	\N	2020-10-12 17:01:45.570268
88639	SCHD	2020-10-12	58.7	58.9799	58.59	58.85	729964	{}	\N	2020-10-12 17:01:45.5747
88640	PIE	2020-09-28	19.12	19.155	19.04	19.14	130291	{}	\N	2020-10-12 17:01:46.497694
88641	PIE	2020-09-29	19.075	19.17	19.0509	19.13	20684	{}	\N	2020-10-12 17:01:46.508087
88642	PIE	2020-09-30	19.17	19.3	19.17	19.28	86613	{}	\N	2020-10-12 17:01:46.515819
88643	PIE	2020-10-01	19.395	19.6328	19.37	19.55	646939	{}	\N	2020-10-12 17:01:46.52239
88644	PIE	2020-10-02	19.24	19.4738	19.24	19.35	124489	{}	\N	2020-10-12 17:01:46.528491
88645	PIE	2020-10-05	19.48	19.6231	19.467	19.59	98826	{}	\N	2020-10-12 17:01:46.534164
88646	PIE	2020-10-06	19.75	19.8488	19.6516	19.75	58879	{}	\N	2020-10-12 17:01:46.539424
88647	PIE	2020-10-07	19.79	19.97	19.79	19.93	55855	{}	\N	2020-10-12 17:01:46.544737
88648	PIE	2020-10-08	20.02	20.11	19.95	20.08	125267	{}	\N	2020-10-12 17:01:46.55056
88649	PIE	2020-10-09	20.03	20.27	20.03	20.235	29634	{}	\N	2020-10-12 17:01:46.555658
88650	PIE	2020-10-12	20.3	20.3	20.22	20.275	21552	{}	\N	2020-10-12 17:01:46.560884
88651	PXH	2020-09-28	17.45	17.52	17.36	17.41	716580	{}	\N	2020-10-12 17:01:48.565417
88652	PXH	2020-09-29	17.345	17.39	17.27	17.33	362144	{}	\N	2020-10-12 17:01:48.581387
88653	PXH	2020-09-30	17.42	17.6	17.42	17.55	484871	{}	\N	2020-10-12 17:01:48.596486
88654	PXH	2020-10-01	17.68	17.71	17.565	17.71	387514	{}	\N	2020-10-12 17:01:48.604234
88655	PXH	2020-10-02	17.46	17.64	17.46	17.5	737104	{}	\N	2020-10-12 17:01:48.612646
88656	PXH	2020-10-05	17.53	17.72	17.53	17.69	393380	{}	\N	2020-10-12 17:01:48.619737
88657	PXH	2020-10-06	17.79	17.85	17.62	17.67	420318	{}	\N	2020-10-12 17:01:48.62983
88658	PXH	2020-10-07	17.76	17.8501	17.73	17.77	502712	{}	\N	2020-10-12 17:01:48.641525
88659	PXH	2020-10-08	17.84	17.97	17.82	17.95	919916	{}	\N	2020-10-12 17:01:48.649075
88660	PXH	2020-10-09	17.95	18.08	17.91	18.01	230321	{}	\N	2020-10-12 17:01:48.65733
88661	PXH	2020-10-12	18.12	18.2132	18.12	18.2	93178	{}	\N	2020-10-12 17:01:48.6641
88662	RNDM	2020-09-28	47.54	47.76	47.54	47.63	23189	{}	\N	2020-10-12 17:01:50.507466
88663	RNDM	2020-09-30	47.37	47.69	47.37	47.4	17453	{}	\N	2020-10-12 17:01:50.516173
88664	RNDM	2020-10-01	47.8479	47.8479	47.731	47.82	1369	{}	\N	2020-10-12 17:01:50.52348
88665	RNDM	2020-10-05	48.3159	48.3159	48.3117	48.3117	696	{}	\N	2020-10-12 17:01:50.529709
88666	RNDM	2020-10-06	48.4509	48.4509	48.4509	48.4509	477	{}	\N	2020-10-12 17:01:50.535203
88667	RNDM	2020-10-07	48.262	48.28	48.262	48.28	569	{}	\N	2020-10-12 17:01:50.540805
88668	RNDM	2020-10-08	48.6599	48.679	48.47	48.55	2841	{}	\N	2020-10-12 17:01:50.546435
88669	RNDM	2020-10-09	48.8999	48.9329	48.8999	48.9329	301	{}	\N	2020-10-12 17:01:50.551645
88670	RWLK	2020-09-28	1.11	1.11	1.03	1.06	105089	{}	\N	2020-10-12 17:01:51.543882
88671	RWLK	2020-09-29	1.07	1.09	1.0581	1.07	107981	{}	\N	2020-10-12 17:01:51.555971
88672	RWLK	2020-09-30	1.08	1.11	1.05	1.08	105974	{}	\N	2020-10-12 17:01:51.563958
88673	RWLK	2020-10-01	1.11	1.14	1.08	1.11	111585	{}	\N	2020-10-12 17:01:51.570081
88674	RWLK	2020-10-02	1.07	1.13	1.03	1.06	108474	{}	\N	2020-10-12 17:01:51.575706
88675	RWLK	2020-10-05	1.08	1.0801	1.04	1.07	61485	{}	\N	2020-10-12 17:01:51.580434
88676	RWLK	2020-10-06	1.06	1.13	1.06	1.092	142254	{}	\N	2020-10-12 17:01:51.584587
88677	RWLK	2020-10-07	1.09	1.12	1.06	1.07	131569	{}	\N	2020-10-12 17:01:51.588754
88678	RWLK	2020-10-08	1.1	1.2	1.1	1.2	249745	{}	\N	2020-10-12 17:01:51.592914
88679	RWLK	2020-10-09	1.21	1.22	1.15	1.15	115075	{}	\N	2020-10-12 17:01:51.597423
88680	RWLK	2020-10-12	1.2	1.2	1.15	1.17	37432	{}	\N	2020-10-12 17:01:51.601833
88681	SAL	2020-09-28	32.77	33.01	32.75	33.01	3344	{}	\N	2020-10-12 17:01:55.600706
88682	SAL	2020-09-29	32.35	32.35	32.12	32.25	1949	{}	\N	2020-10-12 17:01:55.608334
88683	SAL	2020-09-30	32.55	33.12	31.5	31.61	3979	{}	\N	2020-10-12 17:01:55.613912
88684	SAL	2020-10-01	31.61	33	31.61	31.67	2976	{}	\N	2020-10-12 17:01:55.61837
88685	SAL	2020-10-02	30.93	32.63	30.32	31.68	2036	{}	\N	2020-10-12 17:01:55.622554
88686	SAL	2020-10-05	31.76	31.76	31.76	31.76	1202	{}	\N	2020-10-12 17:01:55.626491
88687	SAL	2020-10-06	31.76	32.16	31.01	32.16	2170	{}	\N	2020-10-12 17:01:55.630499
88688	SAL	2020-10-07	32.86	33.48	32.38	33.48	4454	{}	\N	2020-10-12 17:01:55.634163
88689	SAL	2020-10-08	33.48	33.5	33.39	33.5	2013	{}	\N	2020-10-12 17:01:55.637886
88690	SAL	2020-10-09	33.9	35.7991	33.2	34.75	9378	{}	\N	2020-10-12 17:01:55.64159
88691	SBM	2020-09-28	14.5102	14.5551	14.4816	14.5551	642	{}	\N	2020-10-12 17:01:59.125039
88692	SBM	2020-09-29	14.6619	14.6619	14.6619	14.6619	9	{}	\N	2020-10-12 17:01:59.134537
88693	SBM	2020-09-30	14.542	14.542	14.5064	14.5064	863	{}	\N	2020-10-12 17:01:59.139906
88694	SBM	2020-10-01	14.7133	14.7133	14.7133	14.7133	1	{}	\N	2020-10-12 17:01:59.144952
88695	SBM	2020-10-02	14.5609	14.5609	14.5609	14.5609	122	{}	\N	2020-10-12 17:01:59.149544
88696	SBM	2020-10-05	14.3	14.3	14.28	14.28	430	{}	\N	2020-10-12 17:01:59.15414
88697	SBM	2020-10-06	14.4731	14.4731	14.4731	14.4731	0	{}	\N	2020-10-12 17:01:59.158506
88698	SBM	2020-10-07	14.0941	14.0941	14.0941	14.0941	111	{}	\N	2020-10-12 17:01:59.163
88699	SBM	2020-10-08	13.9944	13.9944	13.9944	13.9944	0	{}	\N	2020-10-12 17:01:59.167682
88700	SBM	2020-10-09	13.875	13.875	13.875	13.875	82	{}	\N	2020-10-12 17:01:59.172161
88701	SCHE	2020-09-28	26.45	26.51	26.2663	26.36	1043566	{}	\N	2020-10-12 17:02:00.05769
88702	SCHE	2020-09-29	26.27	26.425	26.2563	26.35	531707	{}	\N	2020-10-12 17:02:00.069509
88703	SCHE	2020-09-30	26.52	26.82	26.52	26.76	1189177	{}	\N	2020-10-12 17:02:00.077742
88704	SCHE	2020-10-01	26.97	27.0473	26.85	27.03	5180040	{}	\N	2020-10-12 17:02:00.084692
88705	SCHE	2020-10-02	26.67	26.94	26.67	26.72	606078	{}	\N	2020-10-12 17:02:00.091184
88706	SCHE	2020-10-05	26.84	27.05	26.84	27.045	485968	{}	\N	2020-10-12 17:02:00.097238
88707	SCHE	2020-10-06	27.16	27.3259	27.04	27.12	1203843	{}	\N	2020-10-12 17:02:00.103007
88708	SCHE	2020-10-07	27.35	27.4399	27.25	27.34	1854376	{}	\N	2020-10-12 17:02:00.109165
88709	SCHE	2020-10-08	27.44	27.607	27.41	27.56	753896	{}	\N	2020-10-12 17:02:00.115256
88710	SCHE	2020-10-09	27.63	27.83	27.5641	27.78	877940	{}	\N	2020-10-12 17:02:00.121234
88711	SCHE	2020-10-12	27.95	28.09	27.89	28.0799	436398	{}	\N	2020-10-12 17:02:00.127397
88712	SCHN	2020-09-28	19.61	20.29	19.47	19.93	220588	{}	\N	2020-10-12 17:02:01.912312
88713	SCHN	2020-09-29	19.84	19.84	19.18	19.45	192385	{}	\N	2020-10-12 17:02:01.92264
88714	SCHN	2020-09-30	19.35	19.715	19.12	19.23	292255	{}	\N	2020-10-12 17:02:01.930148
88715	SCHN	2020-10-01	19.19	19.57	18.85	19.14	206679	{}	\N	2020-10-12 17:02:01.936011
88716	SCHN	2020-10-02	18.78	19.93	18.78	19.67	260665	{}	\N	2020-10-12 17:02:01.941686
88717	SCHN	2020-10-05	19.8	20.4	19.76	20.15	154682	{}	\N	2020-10-12 17:02:01.946715
88718	SCHN	2020-10-06	20.31	20.81	19.99	20.005	167332	{}	\N	2020-10-12 17:02:01.951501
88719	SCHN	2020-10-07	20.26	21	20	20.77	184549	{}	\N	2020-10-12 17:02:01.956286
88720	SCHN	2020-10-08	20.95	21.35	20.8	21.11	242603	{}	\N	2020-10-12 17:02:01.960814
88721	SCHN	2020-10-09	21.43	21.43	20.82	21.09	162120	{}	\N	2020-10-12 17:02:01.965293
88722	SCHN	2020-10-12	21.02	21.18	20.67	21.06	48816	{}	\N	2020-10-12 17:02:01.970487
88723	SELF	2020-09-28	4.1	4.13	4.08	4.08	28558	{}	\N	2020-10-12 17:02:02.922326
88724	SELF	2020-09-29	4.08	4.1	4.03	4.06	7943	{}	\N	2020-10-12 17:02:02.930629
88725	SELF	2020-09-30	4.06	4.09	4.01	4.01	18222	{}	\N	2020-10-12 17:02:02.936813
88726	SELF	2020-10-01	4.06	4.1	4.05	4.09	29125	{}	\N	2020-10-12 17:02:02.942341
88727	SELF	2020-10-02	4.0799	4.085	4.0636	4.08	13403	{}	\N	2020-10-12 17:02:02.947542
88728	SELF	2020-10-05	4.08	4.115	4.08	4.08	13978	{}	\N	2020-10-12 17:02:02.952434
88729	SELF	2020-10-06	4.08	4.0984	4.03	4.03	6964	{}	\N	2020-10-12 17:02:02.956662
88730	SELF	2020-10-07	4.09	4.09	4.05	4.06	5713	{}	\N	2020-10-12 17:02:02.961301
88731	SELF	2020-10-08	4.0334	4.1	4.01	4.06	34926	{}	\N	2020-10-12 17:02:02.965473
88732	SELF	2020-10-09	3.94	4.08	3.94	3.995	24770	{}	\N	2020-10-12 17:02:02.969429
88733	SELF	2020-10-12	3.975	3.99	3.975	3.9801	1891	{}	\N	2020-10-12 17:02:02.973466
88734	SLDB	2020-09-28	2.09	2.11	2.02	2.04	116358	{}	\N	2020-10-12 17:02:05.420746
88735	SLDB	2020-09-29	2.06	2.12	2.0113	2.06	67036	{}	\N	2020-10-12 17:02:05.431387
88736	SLDB	2020-09-30	2.185	2.185	2.03	2.03	62884	{}	\N	2020-10-12 17:02:05.438369
88737	SLDB	2020-10-01	5.21	6.1	3.43	3.46	197288240	{}	\N	2020-10-12 17:02:05.444215
88738	SLDB	2020-10-02	3.35	4.25	3.25	4.18	19052683	{}	\N	2020-10-12 17:02:05.45033
88739	SLDB	2020-10-05	4.46	4.89	4.3	4.75	7863967	{}	\N	2020-10-12 17:02:05.454968
88740	SLDB	2020-10-06	4.83	4.97	4.22	4.37	4290367	{}	\N	2020-10-12 17:02:05.459394
88741	SLDB	2020-10-07	4.1982	4.35	4.01	4.1	1939274	{}	\N	2020-10-12 17:02:05.463948
88742	SLDB	2020-10-08	4.1	4.24	3.76	3.97	1533243	{}	\N	2020-10-12 17:02:05.467697
88743	SLDB	2020-10-09	3.91	4.19	3.91	3.96	749833	{}	\N	2020-10-12 17:02:05.471584
88744	SLDB	2020-10-12	3.91	4	3.82	3.9189	264678	{}	\N	2020-10-12 17:02:05.475338
88745	RXL	2020-09-28	126.74	127.25	125.83	126.06	3914	{}	\N	2020-10-12 17:02:06.6133
88746	RXL	2020-09-29	127.73	127.73	125.565	126.29	842	{}	\N	2020-10-12 17:02:06.624027
88747	RXL	2020-09-30	126.51	131.67	126.51	130.78	3060	{}	\N	2020-10-12 17:02:06.631138
88748	RXL	2020-10-01	132	132	129.09	129.09	732	{}	\N	2020-10-12 17:02:06.636701
88749	RXL	2020-10-02	125.79	127.69	125.79	127.56	499	{}	\N	2020-10-12 17:02:06.641859
88750	RXL	2020-10-05	129.5937	132.3371	129.5937	132.3371	797	{}	\N	2020-10-12 17:02:06.647592
88751	RXL	2020-10-06	132.12	132.2901	129.22	129.22	2524	{}	\N	2020-10-12 17:02:06.652217
88752	RXL	2020-10-07	133.68	134.23	133.68	134.23	1450	{}	\N	2020-10-12 17:02:06.657356
88753	RXL	2020-10-08	135.26	135.74	135.21	135.45	2129	{}	\N	2020-10-12 17:02:06.662625
88754	RXL	2020-10-09	137.28	138.35	137.28	138.12	3751	{}	\N	2020-10-12 17:02:06.668007
88755	RXL	2020-10-12	139.89	140.6182	138.79	140.53	8793	{}	\N	2020-10-12 17:02:06.673919
88756	RZG	2020-09-28	105.09	105.8227	104.68	105.8068	2753	{}	\N	2020-10-12 17:02:08.311307
88757	RZG	2020-09-29	105.9501	105.9501	105.32	105.32	1283	{}	\N	2020-10-12 17:02:08.320737
88758	RZG	2020-09-30	107.225	107.5111	106.3964	106.3964	1672	{}	\N	2020-10-12 17:02:08.328625
88759	RZG	2020-10-01	107.2931	107.7318	106.73	107.7318	1708	{}	\N	2020-10-12 17:02:08.335615
88760	RZG	2020-10-02	104.93	108.6855	104.93	108.6855	1902	{}	\N	2020-10-12 17:02:08.340948
88761	RZG	2020-10-05	110.5321	111.3173	110.35	111.3173	19840	{}	\N	2020-10-12 17:02:08.346197
88762	RZG	2020-10-06	111.69	112.7998	110.8635	110.8635	5135	{}	\N	2020-10-12 17:02:08.351319
88763	RZG	2020-10-07	111.7	112.225	111.7	112.17	762	{}	\N	2020-10-12 17:02:08.356392
88764	RZG	2020-10-08	113.155	113.7879	112.89	113.7879	6300	{}	\N	2020-10-12 17:02:08.361688
88765	RZG	2020-10-09	114.11	115.3439	114.11	115.2476	3897	{}	\N	2020-10-12 17:02:08.367528
88766	RZG	2020-10-12	115.6999	115.6999	115.193	115.193	1699	{}	\N	2020-10-12 17:02:08.372402
88767	SIFY	2020-09-28	1.02	1.05	0.97	1.03	74789	{}	\N	2020-10-12 17:02:09.68483
88768	SIFY	2020-09-29	1.005	1.02	0.99	1.01	35799	{}	\N	2020-10-12 17:02:09.692372
88769	SIFY	2020-09-30	1.02	1.02	0.99	1.02	48730	{}	\N	2020-10-12 17:02:09.699451
88770	SIFY	2020-10-01	1.03	1.05	1.03	1.0485	35034	{}	\N	2020-10-12 17:02:09.706263
88771	SIFY	2020-10-02	1.03	1.05	1.03	1.04	26544	{}	\N	2020-10-12 17:02:09.71264
88772	SIFY	2020-10-05	1.04	1.1	1.04	1.0893	52303	{}	\N	2020-10-12 17:02:09.719697
88773	SIFY	2020-10-06	1.1	1.15	1.0927	1.11	92860	{}	\N	2020-10-12 17:02:09.727551
88774	SIFY	2020-10-07	1.1244	1.14	1.1	1.12	28589	{}	\N	2020-10-12 17:02:09.734894
88775	SIFY	2020-10-08	1.14	1.15	1.08	1.1001	62035	{}	\N	2020-10-12 17:02:09.742185
88776	SIFY	2020-10-09	1.09	1.14	1.09	1.09	45495	{}	\N	2020-10-12 17:02:09.749589
88777	SIFY	2020-10-12	1.15	1.15	1.1	1.1	25733	{}	\N	2020-10-12 17:02:09.757521
88778	SPYD	2020-09-28	27.32	27.76	27.32	27.59	1993956	{}	\N	2020-10-12 17:02:11.927639
88779	SPYD	2020-09-29	27.59	27.59	26.9818	27.23	704714	{}	\N	2020-10-12 17:02:11.937946
88780	SPYD	2020-09-30	27.35	27.7	27.17	27.4	726264	{}	\N	2020-10-12 17:02:11.946248
88781	SPYD	2020-10-01	27.5	27.58	27.13	27.43	1037021	{}	\N	2020-10-12 17:02:11.952363
88782	SPYD	2020-10-02	27.05	28.0425	26.98	27.93	842131	{}	\N	2020-10-12 17:02:11.958407
88783	SPYD	2020-10-05	28.15	28.47	28.07	28.44	729199	{}	\N	2020-10-12 17:02:11.96381
88784	SPYD	2020-10-06	28.59	29.06	28.22	28.3	948046	{}	\N	2020-10-12 17:02:11.968714
88785	SPYD	2020-10-07	28.52	28.89	28.52	28.78	555387	{}	\N	2020-10-12 17:02:11.973989
88786	SPYD	2020-10-08	28.94	29.34	28.94	29.34	888958	{}	\N	2020-10-12 17:02:11.978993
88787	SPYD	2020-10-09	29.5	29.5849	29.03	29.1	786562	{}	\N	2020-10-12 17:02:11.983955
88788	SPYD	2020-10-12	29.18	29.195	28.9801	29.1155	472752	{}	\N	2020-10-12 17:02:11.989018
88789	TBNK	2020-09-28	20.84	20.84	20.3	20.52	9031	{}	\N	2020-10-12 17:02:12.923721
88790	TBNK	2020-09-29	20.29	20.54	20.125	20.4	5510	{}	\N	2020-10-12 17:02:12.933995
88791	TBNK	2020-09-30	20.3	20.48	20.04	20.23	11917	{}	\N	2020-10-12 17:02:12.94128
88792	TBNK	2020-10-01	20.2	20.3	19.8	19.92	14997	{}	\N	2020-10-12 17:02:12.947382
88793	TBNK	2020-10-02	19.82	20.415	19.82	20.16	14602	{}	\N	2020-10-12 17:02:12.952715
88794	TBNK	2020-10-05	20.43	21.43	20.125	21.11	16156	{}	\N	2020-10-12 17:02:12.957855
88795	TBNK	2020-10-06	21.38	22.25	21.05	21.05	16772	{}	\N	2020-10-12 17:02:12.962287
88796	TBNK	2020-10-07	21.44	21.92	21.14	21.92	6969	{}	\N	2020-10-12 17:02:12.966777
88797	TBNK	2020-10-08	22	22.25	20.66	20.86	23262	{}	\N	2020-10-12 17:02:12.971404
88798	TBNK	2020-10-09	21.29	21.49	20.94	21	6292	{}	\N	2020-10-12 17:02:12.976331
88799	TBNK	2020-10-12	20.85	21.33	20.85	21.33	2610	{}	\N	2020-10-12 17:02:12.980522
88800	TESS	2020-09-28	5.91	6.09	5.72	5.98	24594	{}	\N	2020-10-12 17:02:13.967407
88801	TESS	2020-09-29	5.98	5.98	5.56	5.69	13755	{}	\N	2020-10-12 17:02:13.97948
88802	TESS	2020-09-30	5.63	5.735	5.37	5.37	11381	{}	\N	2020-10-12 17:02:13.986523
88803	TESS	2020-10-01	5.41	5.68	5.41	5.67	10850	{}	\N	2020-10-12 17:02:13.992126
88804	TESS	2020-10-02	5.58	5.6955	5.53	5.69	5685	{}	\N	2020-10-12 17:02:13.998444
88805	TESS	2020-10-05	5.68	6.15	5.64	5.96	15600	{}	\N	2020-10-12 17:02:14.003576
88806	TESS	2020-10-06	6.09	6.15	5.94	5.94	6566	{}	\N	2020-10-12 17:02:14.007944
88807	TESS	2020-10-07	6.02	6.1371	5.69	5.94	13479	{}	\N	2020-10-12 17:02:14.012779
88808	TESS	2020-10-08	6.07	6.1999	5.88	6.14	22065	{}	\N	2020-10-12 17:02:14.017212
88809	TESS	2020-10-09	6.14	6.14	5.76	5.92	13144	{}	\N	2020-10-12 17:02:14.02196
88810	TESS	2020-10-12	6	6	5.958	5.958	2025	{}	\N	2020-10-12 17:02:14.026454
88811	TIP	2020-09-28	126.55	126.6899	126.48	126.67	2246992	{}	\N	2020-10-12 17:02:16.110628
88812	TIP	2020-09-29	126.77	126.95	126.71	126.88	1154612	{}	\N	2020-10-12 17:02:16.121675
88813	TIP	2020-09-30	126.9	126.9	126.49	126.5	6243052	{}	\N	2020-10-12 17:02:16.12925
88814	TIP	2020-10-01	126.07	126.295	125.93	126.06	10657791	{}	\N	2020-10-12 17:02:16.135884
88815	TIP	2020-10-02	126.1	126.15	125.97	126.08	3131643	{}	\N	2020-10-12 17:02:16.141823
88816	TIP	2020-10-05	126.06	126.06	125.685	125.7	2694040	{}	\N	2020-10-12 17:02:16.146335
88817	TIP	2020-10-06	125.8	125.94	125.55	125.72	2200544	{}	\N	2020-10-12 17:02:16.150657
88818	TIP	2020-10-07	125.75	125.81	125.61	125.71	1515065	{}	\N	2020-10-12 17:02:16.155304
88819	TIP	2020-10-08	125.95	126.21	125.8735	126.21	1945676	{}	\N	2020-10-12 17:02:16.160026
88820	TIP	2020-10-09	126.19	126.27	126	126.25	1930603	{}	\N	2020-10-12 17:02:16.165505
88821	TIP	2020-10-12	126.19	126.26	126.1589	126.19	460652	{}	\N	2020-10-12 17:02:16.170542
88822	TRXC	2020-09-28	0.35	0.355	0.341	0.3503	1015609	{}	\N	2020-10-12 17:02:17.598172
88823	TRXC	2020-09-29	0.355	0.36	0.342	0.354	1166876	{}	\N	2020-10-12 17:02:17.608024
88824	TRXC	2020-09-30	0.3555	0.3599	0.346	0.3474	824302	{}	\N	2020-10-12 17:02:17.61448
88825	TRXC	2020-10-01	0.3421	0.3518	0.341	0.343	1049735	{}	\N	2020-10-12 17:02:17.620079
88826	TRXC	2020-10-02	0.35	0.36	0.343	0.3468	902471	{}	\N	2020-10-12 17:02:17.625212
88827	TRXC	2020-10-05	0.36	0.3649	0.346	0.346	991225	{}	\N	2020-10-12 17:02:17.630119
88828	TRXC	2020-10-06	0.35	0.3627	0.347	0.35	865477	{}	\N	2020-10-12 17:02:17.634983
88829	TRXC	2020-10-07	0.41	0.445	0.363	0.3968	14652706	{}	\N	2020-10-12 17:02:17.639653
88830	TRXC	2020-10-08	0.39	0.4	0.375	0.3934	1817763	{}	\N	2020-10-12 17:02:17.644158
88831	TRXC	2020-10-09	0.39	0.3921	0.3801	0.3907	1175862	{}	\N	2020-10-12 17:02:17.64898
88832	TRXC	2020-10-12	0.3729	0.4014	0.37	0.3913	1853776	{}	\N	2020-10-12 17:02:17.65337
88833	TYO	2020-09-28	7.7424	7.7499	7.73	7.7499	1429	{}	\N	2020-10-12 17:02:19.680682
88834	TYO	2020-09-29	7.7184	7.7184	7.7	7.71	1550	{}	\N	2020-10-12 17:02:19.691661
88835	TYO	2020-09-30	7.78	7.81	7.7737	7.7737	683	{}	\N	2020-10-12 17:02:19.700273
88836	TYO	2020-10-01	7.755	7.755	7.755	7.755	133	{}	\N	2020-10-12 17:02:19.707133
88837	TYO	2020-10-02	7.79	7.8	7.77	7.77	2309	{}	\N	2020-10-12 17:02:19.713497
88838	TYO	2020-10-05	7.87	7.93	7.87	7.93	4874	{}	\N	2020-10-12 17:02:19.718448
88839	TYO	2020-10-06	7.9296	7.93	7.86	7.86	8323	{}	\N	2020-10-12 17:02:19.72341
88840	TYO	2020-10-07	7.91	7.9462	7.91	7.9462	1179	{}	\N	2020-10-12 17:02:19.728516
88841	TYO	2020-10-08	7.9	7.9048	7.89	7.9048	4147	{}	\N	2020-10-12 17:02:19.736838
88842	TYO	2020-10-09	7.94	7.94	7.9	7.91	3065	{}	\N	2020-10-12 17:02:19.743677
88843	TYO	2020-10-12	7.92	7.92	7.9	7.9	2382	{}	\N	2020-10-12 17:02:19.748535
88844	VWO	2020-09-28	42.67	42.79	42.39	42.57	8067364	{}	\N	2020-10-12 17:02:21.347486
88845	VWO	2020-09-29	42.45	42.7076	42.4083	42.55	13419447	{}	\N	2020-10-12 17:02:21.359513
88846	VWO	2020-09-30	42.89	43.31	42.85	43.24	13516943	{}	\N	2020-10-12 17:02:21.367705
88847	VWO	2020-10-01	43.56	43.66	43.3	43.62	19434452	{}	\N	2020-10-12 17:02:21.374138
88848	VWO	2020-10-02	42.97	43.505	42.93	43.14	11442660	{}	\N	2020-10-12 17:02:21.380922
88849	VWO	2020-10-05	43.34	43.7	43.31	43.7	5668802	{}	\N	2020-10-12 17:02:21.386575
88850	VWO	2020-10-06	43.9	44.13	43.68	43.83	10409150	{}	\N	2020-10-12 17:02:21.392027
88851	VWO	2020-10-07	44.17	44.34	44.03	44.24	8160934	{}	\N	2020-10-12 17:02:21.397911
88852	VWO	2020-10-08	44.3962	44.61	44.2742	44.57	7637568	{}	\N	2020-10-12 17:02:21.403511
88853	VWO	2020-10-09	44.65	44.98	44.5561	44.88	8363038	{}	\N	2020-10-12 17:02:21.409696
88854	VWO	2020-10-12	45.16	45.41	45.05	45.38	5633359	{}	\N	2020-10-12 17:02:21.416039
88855	SBLK	2020-09-28	6.96	6.9799	6.76	6.845	275757	{}	\N	2020-10-12 17:02:22.520498
88856	SBLK	2020-09-29	6.76	7	6.73	6.745	341533	{}	\N	2020-10-12 17:02:22.529755
88857	SBLK	2020-09-30	6.8	7.0101	6.8	6.89	320733	{}	\N	2020-10-12 17:02:22.537427
88858	SBLK	2020-10-01	7	7.15	6.86	7.11	537074	{}	\N	2020-10-12 17:02:22.546056
88859	SBLK	2020-10-02	7.07	7.38	7	7.37	629491	{}	\N	2020-10-12 17:02:22.553534
88860	SBLK	2020-10-05	7.42	7.84	7.4	7.69	705975	{}	\N	2020-10-12 17:02:22.561105
88861	SBLK	2020-10-06	7.73	7.84	7.33	7.42	622258	{}	\N	2020-10-12 17:02:22.568124
88862	SBLK	2020-10-07	7.41	7.5	7.11	7.18	387503	{}	\N	2020-10-12 17:02:22.575625
88863	SBLK	2020-10-08	7.25	7.47	7.2	7.44	456955	{}	\N	2020-10-12 17:02:22.583085
88864	SBLK	2020-10-09	7.39	7.47	7.12	7.13	429462	{}	\N	2020-10-12 17:02:22.588509
88865	SBLK	2020-10-12	7.1	7.13	6.93	7.01	145260	{}	\N	2020-10-12 17:02:22.593101
88866	SEAS	2020-09-28	20.95	21.08	20.18	20.64	1510186	{}	\N	2020-10-12 17:02:23.811694
88867	SEAS	2020-09-29	20.55	20.64	19.49	20.01	1932696	{}	\N	2020-10-12 17:02:23.818743
88868	SEAS	2020-09-30	20.2	20.77	19.6	19.72	1989818	{}	\N	2020-10-12 17:02:23.825468
88869	SEAS	2020-10-01	19.9	19.95	19.34	19.66	1600813	{}	\N	2020-10-12 17:02:23.832559
88870	SEAS	2020-10-02	18.74	20.34	18.63	20.21	1814178	{}	\N	2020-10-12 17:02:23.838079
88871	SEAS	2020-10-05	20.33	20.615	19.93	20.54	1749803	{}	\N	2020-10-12 17:02:23.843825
88872	SEAS	2020-10-06	21.47	22.2224	20.93	20.95	4392987	{}	\N	2020-10-12 17:02:23.850552
88873	SEAS	2020-10-07	21.35	22.05	21.05	21.76	3130826	{}	\N	2020-10-12 17:02:23.857059
88874	SEAS	2020-10-08	22	22.36	21.56	22.01	1801543	{}	\N	2020-10-12 17:02:23.861697
88875	SEAS	2020-10-09	22.12	22.85	21.88	22.68	2739751	{}	\N	2020-10-12 17:02:23.868054
88876	SEAS	2020-10-12	23	23	22.16	22.47	670706	{}	\N	2020-10-12 17:02:23.874849
88877	SLMBP	2020-09-28	32.32	33.7	32.3	33.05	2884	{}	\N	2020-10-12 17:02:25.14312
88878	SLMBP	2020-09-29	32.37	33.7	32	33.355	9988	{}	\N	2020-10-12 17:02:25.150796
88879	SLMBP	2020-09-30	32.87	33.7	32.35	33.15	3891	{}	\N	2020-10-12 17:02:25.160026
88880	SLMBP	2020-10-01	33.05	33.7	33.01	33.7	3134	{}	\N	2020-10-12 17:02:25.167151
88881	SLMBP	2020-10-02	33.57	33.58	33.57	33.58	966	{}	\N	2020-10-12 17:02:25.17615
88882	SLMBP	2020-10-05	33.7	33.7	33.67	33.69	2262	{}	\N	2020-10-12 17:02:25.183615
88883	SLMBP	2020-10-06	33.65	35	33.65	35	5896	{}	\N	2020-10-12 17:02:25.191915
88884	SLMBP	2020-10-07	34.96	34.98	34.96	34.98	1090	{}	\N	2020-10-12 17:02:25.199003
88885	SLMBP	2020-10-08	34.98	34.98	34.98	34.98	1180	{}	\N	2020-10-12 17:02:25.205557
88886	SLMBP	2020-10-09	34.83	34.83	34.66	34.66	471	{}	\N	2020-10-12 17:02:25.211567
88887	SLMBP	2020-10-12	34.97	34.97	34.97	34.97	133	{}	\N	2020-10-12 17:02:25.21754
88888	SMLF	2020-09-28	37.0093	37.33	36.9313	37.25	58752	{}	\N	2020-10-12 17:02:26.551278
88889	SMLF	2020-09-29	37.29	37.29	36.825	37.12	48651	{}	\N	2020-10-12 17:02:26.558134
88890	SMLF	2020-09-30	37.489	37.6399	37.03	37.27	42177	{}	\N	2020-10-12 17:02:26.56418
88891	SMLF	2020-10-01	37.46	37.76	37.28	37.75	62682	{}	\N	2020-10-12 17:02:26.570094
88892	SMLF	2020-10-02	37.07	38.17	37.07	38.06	35139	{}	\N	2020-10-12 17:02:26.57664
88893	SMLF	2020-10-05	38.45	38.94	38.31	38.91	47191	{}	\N	2020-10-12 17:02:26.582535
88894	SMLF	2020-10-06	39.08	39.637	38.77	38.8	412139	{}	\N	2020-10-12 17:02:26.588214
88895	SMLF	2020-10-07	39.29	39.52	39.2421	39.41	48981	{}	\N	2020-10-12 17:02:26.59453
88896	SMLF	2020-10-08	39.72	39.9	39.5401	39.9	109699	{}	\N	2020-10-12 17:02:26.600514
88897	SMLF	2020-10-09	40.21	40.3313	40.0005	40.14	844541	{}	\N	2020-10-12 17:02:26.606262
88898	SMLF	2020-10-12	40.29	40.48	40.1981	40.4097	24059	{}	\N	2020-10-12 17:02:26.612467
88899	SMMV	2020-09-28	29.11	29.3975	29.0974	29.33	299827	{}	\N	2020-10-12 17:02:27.624631
88900	SMMV	2020-09-29	29.35	29.38	29.0101	29.18	127996	{}	\N	2020-10-12 17:02:27.634679
88901	SMMV	2020-09-30	29.2	29.4499	29.0211	29.2	439360	{}	\N	2020-10-12 17:02:27.64146
88902	SMMV	2020-10-01	29.25	29.46	29.1062	29.43	335177	{}	\N	2020-10-12 17:02:27.648189
88903	SMMV	2020-10-02	29.07	29.6377	29.0699	29.54	156459	{}	\N	2020-10-12 17:02:27.6533
88904	SMMV	2020-10-05	29.79	30.01	29.7213	29.97	170235	{}	\N	2020-10-12 17:02:27.659148
88905	SMMV	2020-10-06	30.12	30.455	29.8887	29.92	125012	{}	\N	2020-10-12 17:02:27.664412
88906	SMMV	2020-10-07	30.1	30.26	30.05	30.22	142761	{}	\N	2020-10-12 17:02:27.669745
88907	SMMV	2020-10-08	30.4	30.52	30.3154	30.51	88577	{}	\N	2020-10-12 17:02:27.676198
88908	SMMV	2020-10-09	30.64	30.6996	30.4901	30.62	75530	{}	\N	2020-10-12 17:02:27.683683
88909	SMMV	2020-10-12	30.74	30.9	30.64	30.8799	39538	{}	\N	2020-10-12 17:02:27.691463
88910	SNP	2020-09-28	40.84	40.95	40.39	40.55	270844	{}	\N	2020-10-12 17:02:28.702082
88911	SNP	2020-09-29	40.41	40.41	39.94	40.24	317458	{}	\N	2020-10-12 17:02:28.711951
88912	SNP	2020-09-30	40.53	40.72	40.32	40.46	213328	{}	\N	2020-10-12 17:02:28.719571
88913	SNP	2020-10-01	40.59	40.7	40.15	40.31	374099	{}	\N	2020-10-12 17:02:28.725721
88914	SNP	2020-10-02	39.72	40.25	39.72	40.03	291721	{}	\N	2020-10-12 17:02:28.731468
88915	SNP	2020-10-05	40.3	40.81	40.19	40.81	365888	{}	\N	2020-10-12 17:02:28.737143
88916	SNP	2020-10-06	41.21	41.45	40.82	40.85	227195	{}	\N	2020-10-12 17:02:28.742266
88917	SNP	2020-10-07	41.02	41.085	40.78	41.05	154122	{}	\N	2020-10-12 17:02:28.747271
88918	SNP	2020-10-08	41.15	41.43	41.0568	41.39	194260	{}	\N	2020-10-12 17:02:28.752057
88919	SNP	2020-10-09	41.3	41.3	40.82	40.88	229459	{}	\N	2020-10-12 17:02:28.757272
88920	SNP	2020-10-12	40.91	41.42	40.88	41.4	167280	{}	\N	2020-10-12 17:02:28.762064
88921	SOXS	2020-09-28	41.51	42.27	39.55	39.55	983069	{}	\N	2020-10-12 17:02:30.135854
88922	SOXS	2020-09-29	39.82	39.82	38.28	39.28	1526918	{}	\N	2020-10-12 17:02:30.148391
88923	SOXS	2020-09-30	39.51	39.84	38	39.21	1641375	{}	\N	2020-10-12 17:02:30.156318
88924	SOXS	2020-10-01	37.7	38.1275	36.41	36.73	1541870	{}	\N	2020-10-12 17:02:30.163574
88925	SOXS	2020-10-02	39.39	40.12	37.73	40.11	1626432	{}	\N	2020-10-12 17:02:30.169645
88926	SOXS	2020-10-05	38.86	38.87	36.09	36.09	1083889	{}	\N	2020-10-12 17:02:30.174803
88927	SOXS	2020-10-06	36.24	37.1199	34.11	36.66	2288541	{}	\N	2020-10-12 17:02:30.179459
88928	SOXS	2020-10-07	35.18	35.18	34.24	34.44	1210856	{}	\N	2020-10-12 17:02:30.184663
88929	SOXS	2020-10-08	33.6	33.9567	33.04	33.2	1258003	{}	\N	2020-10-12 17:02:30.189862
88930	SOXS	2020-10-09	32.01	32.2	31.22	31.47	2253484	{}	\N	2020-10-12 17:02:30.19462
88931	SOXS	2020-10-12	30.33	30.7419	29.82	30.18	982726	{}	\N	2020-10-12 17:02:30.199846
88932	SUI	2020-09-28	144.85	148.54	144.24	147.55	558467	{}	\N	2020-10-12 17:02:31.159225
88933	SUI	2020-09-29	147.44	147.44	138.69	140.29	2745343	{}	\N	2020-10-12 17:02:31.170756
88934	SUI	2020-09-30	139.59	141.21	136.02	140.61	3266781	{}	\N	2020-10-12 17:02:31.179692
88935	SUI	2020-10-01	143	149.38	141.62	148.83	6252182	{}	\N	2020-10-12 17:02:31.185982
88936	SUI	2020-10-02	146.88	150.77	145.775	150.34	2480938	{}	\N	2020-10-12 17:02:31.191959
88937	SUI	2020-10-05	150.17	151.37	148.23	150.55	1574586	{}	\N	2020-10-12 17:02:31.196892
88938	SUI	2020-10-06	150.55	150.815	146	147.17	1761174	{}	\N	2020-10-12 17:02:31.201611
88939	SUI	2020-10-07	147.72	149.1	145.64	148.5	813259	{}	\N	2020-10-12 17:02:31.206545
88940	SUI	2020-10-08	149.49	152.39	148.33	150.58	919225	{}	\N	2020-10-12 17:02:31.211478
88941	SUI	2020-10-09	151.51	151.61	147.62	148.14	802613	{}	\N	2020-10-12 17:02:31.216205
88942	SUI	2020-10-12	148.43	149.05	147.26	148.03	305031	{}	\N	2020-10-12 17:02:31.221193
88943	SUSB	2020-09-28	26.01	26.08	26.01	26.049	54449	{}	\N	2020-10-12 17:02:32.459396
88944	SUSB	2020-09-29	26.13	26.13	26.03	26.06	65284	{}	\N	2020-10-12 17:02:32.466114
88945	SUSB	2020-09-30	26.12	26.12	26.0711	26.105	63587	{}	\N	2020-10-12 17:02:32.471141
88946	SUSB	2020-10-01	26.09	26.09	26.04	26.065	55160	{}	\N	2020-10-12 17:02:32.476509
88947	SUSB	2020-10-02	26.11	26.11	26.051	26.095	47032	{}	\N	2020-10-12 17:02:32.485294
88948	SUSB	2020-10-05	26.08	26.12	26.0303	26.09	41931	{}	\N	2020-10-12 17:02:32.496413
88949	SUSB	2020-10-06	26.01	26.12	26.01	26.09	196157	{}	\N	2020-10-12 17:02:32.502775
88950	SUSB	2020-10-07	26.02	26.09	26	26.02	504785	{}	\N	2020-10-12 17:02:32.508134
88951	SUSB	2020-10-08	26.1	26.1	26.05	26.07	85945	{}	\N	2020-10-12 17:02:32.513064
88952	SUSB	2020-10-09	26	26.12	26	26.07	138753	{}	\N	2020-10-12 17:02:32.518201
88953	SUSB	2020-10-12	26.11	26.14	26.0762	26.1031	41480	{}	\N	2020-10-12 17:02:32.523235
88954	SVXY	2020-09-28	35.22	35.2438	34.98	35.12	2223360	{}	\N	2020-10-12 17:02:33.950968
88955	SVXY	2020-09-29	35.09	35.73	35.09	35.5	2314744	{}	\N	2020-10-12 17:02:33.960193
88956	SVXY	2020-09-30	35.73	35.93	35.29	35.5	2374899	{}	\N	2020-10-12 17:02:33.967741
88957	SVXY	2020-10-01	35.67	35.67	35.12	35.36	2476678	{}	\N	2020-10-12 17:02:33.974433
88958	SVXY	2020-10-02	34.5	35.18	34.43	34.87	3472297	{}	\N	2020-10-12 17:02:33.979471
88959	SVXY	2020-10-05	34.98	35.52	34.83	35.38	2013520	{}	\N	2020-10-12 17:02:33.984822
88960	SVXY	2020-10-06	35.54	35.86	34.975	35.15	2917949	{}	\N	2020-10-12 17:02:33.98969
88961	SVXY	2020-10-07	35.51	35.84	35.41	35.72	1840732	{}	\N	2020-10-12 17:02:33.994681
88962	SVXY	2020-10-08	35.91	36.47	35.82	36.43	2702280	{}	\N	2020-10-12 17:02:33.999743
88963	SVXY	2020-10-09	37	37.39	36.96	37.38	2560842	{}	\N	2020-10-12 17:02:34.004936
88964	SVXY	2020-10-12	37.72	37.767	37.38	37.64	1007676	{}	\N	2020-10-12 17:02:34.010084
88965	TPOR	2020-09-28	20.1	20.49	19.965	20.17	121166	{}	\N	2020-10-12 17:02:36.804875
88966	TPOR	2020-09-29	20.47	20.47	19.42	19.43	155825	{}	\N	2020-10-12 17:02:36.815724
88967	TPOR	2020-09-30	19.58	20.28	18.75	19.01	125294	{}	\N	2020-10-12 17:02:36.82325
88968	TPOR	2020-10-01	19.5	19.67	18.71	18.92	108739	{}	\N	2020-10-12 17:02:36.829045
88969	TPOR	2020-10-02	18.16	19.88	17.77	19.4	142907	{}	\N	2020-10-12 17:02:36.834754
88970	TPOR	2020-10-05	19.88	20.47	19.77	20.36	157023	{}	\N	2020-10-12 17:02:36.839781
88971	TPOR	2020-10-06	20.61	21.2	19.78	19.83	198711	{}	\N	2020-10-12 17:02:36.844712
88972	TPOR	2020-10-07	20.75	21.6574	20.5401	21.38	155847	{}	\N	2020-10-12 17:02:36.849529
88973	TPOR	2020-10-08	21.92	22	21.11	21.84	105030	{}	\N	2020-10-12 17:02:36.854333
88974	TPOR	2020-10-09	22.45	22.71	21.835	22.38	140260	{}	\N	2020-10-12 17:02:36.858963
88975	TPOR	2020-10-12	22.92	22.92	22.3532	22.5298	103206	{}	\N	2020-10-12 17:02:36.863712
88976	SCHM	2020-09-28	55.23	55.75	55.23	55.55	364719	{}	\N	2020-10-12 17:02:38.647588
88977	SCHM	2020-09-29	55.54	55.72	55.0321	55.23	334135	{}	\N	2020-10-12 17:02:38.657307
88978	SCHM	2020-09-30	55.44	56.0599	55.11	55.48	392524	{}	\N	2020-10-12 17:02:38.665563
88979	SCHM	2020-10-01	55.889	56.172	55.56	56.05	2699440	{}	\N	2020-10-12 17:02:38.67329
88980	SCHM	2020-10-02	55.04	56.57	55.0001	56.39	249560	{}	\N	2020-10-12 17:02:38.680814
88981	SCHM	2020-10-05	56.91	57.62	56.91	57.59	1611645	{}	\N	2020-10-12 17:02:38.68773
88982	SCHM	2020-10-06	57.91	58.6	57.21	57.35	246234	{}	\N	2020-10-12 17:02:38.692392
88983	SCHM	2020-10-07	57.9	58.53	57.865	58.41	783084	{}	\N	2020-10-12 17:02:38.697237
88984	SCHM	2020-10-08	58.85	59.12	58.6656	59.12	452662	{}	\N	2020-10-12 17:02:38.702071
88985	SCHM	2020-10-09	59.5	59.58	59.12	59.34	235283	{}	\N	2020-10-12 17:02:38.707117
88986	SCHM	2020-10-12	59.56	59.69	59.33	59.5844	99883	{}	\N	2020-10-12 17:02:38.71164
88987	SCHZ	2020-09-28	56.08	56.16	56.08	56.16	745406	{}	\N	2020-10-12 17:02:39.726066
88988	SCHZ	2020-09-29	56.1543	56.25	56.1543	56.21	602769	{}	\N	2020-10-12 17:02:39.73924
88989	SCHZ	2020-09-30	56.21	56.21	56.04	56.08	771618	{}	\N	2020-10-12 17:02:39.747848
88990	SCHZ	2020-10-01	55.97	56.08	55.92	56.06	1099019	{}	\N	2020-10-12 17:02:39.754119
88991	SCHZ	2020-10-02	56.08	56.08	55.9857	56.01	1448404	{}	\N	2020-10-12 17:02:39.760036
88992	SCHZ	2020-10-05	55.95	55.95	55.82	55.83	604110	{}	\N	2020-10-12 17:02:39.76552
88993	SCHZ	2020-10-06	55.92	55.97	55.803	55.85	642840	{}	\N	2020-10-12 17:02:39.772076
88994	SCHZ	2020-10-07	55.83	55.87	55.76	55.8	470775	{}	\N	2020-10-12 17:02:39.781582
88995	SCHZ	2020-10-08	55.82	55.91	55.82	55.91	595287	{}	\N	2020-10-12 17:02:39.790414
88996	SCHZ	2020-10-09	55.85	55.899	55.79	55.87	515383	{}	\N	2020-10-12 17:02:39.802598
88997	SCHZ	2020-10-12	55.9	55.9699	55.9	55.965	178687	{}	\N	2020-10-12 17:02:39.812399
88998	SEDG	2020-09-28	205.13	219	204.55	218.29	1399097	{}	\N	2020-10-12 17:02:40.79595
88999	SEDG	2020-09-29	219	231.8289	219	231.5	1557629	{}	\N	2020-10-12 17:02:40.806707
89000	SEDG	2020-09-30	233.45	245.78	228.1	238.35	1571499	{}	\N	2020-10-12 17:02:40.814264
89001	SEDG	2020-10-01	243	249.73	235.22	248.87	1336573	{}	\N	2020-10-12 17:02:40.820635
89002	SEDG	2020-10-02	240.6	257.25	239.5	251.58	1317388	{}	\N	2020-10-12 17:02:40.826579
89003	SEDG	2020-10-05	258.45	272.63	257.01	272.59	1406207	{}	\N	2020-10-12 17:02:40.83142
89004	SEDG	2020-10-06	273.67	275.85	262.4	271.87	1926783	{}	\N	2020-10-12 17:02:40.836634
89005	SEDG	2020-10-07	278.98	308.17	276.5701	305.15	2187600	{}	\N	2020-10-12 17:02:40.841069
89006	SEDG	2020-10-08	311	314	278.56	289	3211018	{}	\N	2020-10-12 17:02:40.845271
89007	SEDG	2020-10-09	292	311.2416	286.71	305.35	1860867	{}	\N	2020-10-12 17:02:40.849931
89008	SEDG	2020-10-12	310	311.5964	288	295.79	1263833	{}	\N	2020-10-12 17:02:40.854479
89009	SHYD	2020-09-28	24.28	24.34	24.26	24.29	25329	{}	\N	2020-10-12 17:02:41.824594
89010	SHYD	2020-09-29	24.29	24.31	24.26	24.26	22820	{}	\N	2020-10-12 17:02:41.831668
89011	SHYD	2020-09-30	24.3	24.38	24.27	24.38	56464	{}	\N	2020-10-12 17:02:41.836598
89012	SHYD	2020-10-01	24.2626	24.32	24.221	24.26	43137	{}	\N	2020-10-12 17:02:41.841735
89013	SHYD	2020-10-02	24.31	24.31	24.16	24.16	43213	{}	\N	2020-10-12 17:02:41.847171
89014	SHYD	2020-10-05	24.2	24.2999	24.1901	24.22	26441	{}	\N	2020-10-12 17:02:41.852798
89015	SHYD	2020-10-06	24.22	24.29	24.22	24.27	28388	{}	\N	2020-10-12 17:02:41.858893
89016	SHYD	2020-10-07	24.23	24.29	24.23	24.29	42460	{}	\N	2020-10-12 17:02:41.866955
89017	SHYD	2020-10-08	24.23	24.31	24.225	24.31	60597	{}	\N	2020-10-12 17:02:41.888304
89018	SHYD	2020-10-09	24.31	24.32	24.21	24.285	26655	{}	\N	2020-10-12 17:02:41.897625
89019	SHYD	2020-10-12	24.2975	24.32	24.17	24.2114	49287	{}	\N	2020-10-12 17:02:41.908479
89020	SMSI	2020-09-28	3.68	3.74	3.625	3.69	249842	{}	\N	2020-10-12 17:02:42.914064
89021	SMSI	2020-09-29	3.68	3.75	3.6283	3.73	166181	{}	\N	2020-10-12 17:02:42.92504
89022	SMSI	2020-09-30	3.71	3.84	3.675	3.73	318703	{}	\N	2020-10-12 17:02:42.932644
89023	SMSI	2020-10-01	3.75	3.79	3.6799	3.69	227681	{}	\N	2020-10-12 17:02:42.939382
89024	SMSI	2020-10-02	3.65	3.696	3.6	3.62	372820	{}	\N	2020-10-12 17:02:42.944333
89025	SMSI	2020-10-05	3.63	3.69	3.6193	3.68	482270	{}	\N	2020-10-12 17:02:42.949093
89026	SMSI	2020-10-06	3.7	3.73	3.64	3.69	451807	{}	\N	2020-10-12 17:02:42.954725
89027	SMSI	2020-10-07	3.74	3.75	3.68	3.745	364586	{}	\N	2020-10-12 17:02:42.961001
89028	SMSI	2020-10-08	3.76	3.91	3.75	3.81	373332	{}	\N	2020-10-12 17:02:42.969019
89029	SMSI	2020-10-09	3.82	3.91	3.82	3.86	236281	{}	\N	2020-10-12 17:02:42.975861
89030	SMSI	2020-10-12	3.86	3.91	3.7563	3.8	146319	{}	\N	2020-10-12 17:02:42.980893
89031	SOGO	2020-09-28	8.64	8.68	8.63	8.67	509361	{}	\N	2020-10-12 17:02:44.000488
89032	SOGO	2020-09-29	8.88	8.9	8.85	8.87	24407324	{}	\N	2020-10-12 17:02:44.010589
89033	SOGO	2020-09-30	8.88	8.9	8.86	8.89	4677025	{}	\N	2020-10-12 17:02:44.023692
89034	SOGO	2020-10-01	8.89	8.9	8.87	8.88	3525711	{}	\N	2020-10-12 17:02:44.032814
89035	SOGO	2020-10-02	8.88	8.9	8.875	8.89	2529802	{}	\N	2020-10-12 17:02:44.039539
89036	SOGO	2020-10-05	8.89	8.935	8.865	8.88	4749833	{}	\N	2020-10-12 17:02:44.045351
89037	SOGO	2020-10-06	8.88	8.895	8.87	8.87	1989788	{}	\N	2020-10-12 17:02:44.051121
89038	SOGO	2020-10-07	8.88	8.89	8.87	8.87	2174483	{}	\N	2020-10-12 17:02:44.056897
89039	SOGO	2020-10-08	8.88	8.89	8.87	8.87	1216069	{}	\N	2020-10-12 17:02:44.062406
89040	SOGO	2020-10-09	8.87	8.88	8.87	8.88	2056391	{}	\N	2020-10-12 17:02:44.068028
89041	SOGO	2020-10-12	8.88	8.88	8.87	8.88	811094	{}	\N	2020-10-12 17:02:44.074738
89042	SPYV	2020-09-28	30.06	30.3184	30.0452	30.16	751011	{}	\N	2020-10-12 17:02:45.960699
89043	SPYV	2020-09-29	30.17	30.18	29.82	29.93	1187153	{}	\N	2020-10-12 17:02:45.970248
89044	SPYV	2020-09-30	30.03	30.4832	30.0164	30.2	1435066	{}	\N	2020-10-12 17:02:45.977906
89045	SPYV	2020-10-01	30.34	30.44	30.05	30.22	1160327	{}	\N	2020-10-12 17:02:45.982998
89046	SPYV	2020-10-02	29.84	30.465	29.77	30.3	1140509	{}	\N	2020-10-12 17:02:45.988906
89047	SPYV	2020-10-05	30.52	30.799	30.48	30.76	771703	{}	\N	2020-10-12 17:02:45.994539
89048	SPYV	2020-10-06	30.86	31.0923	30.425	30.48	1343999	{}	\N	2020-10-12 17:02:46.00076
89049	SPYV	2020-10-07	30.73	31.07	30.72	30.97	2937244	{}	\N	2020-10-12 17:02:46.008855
89050	SPYV	2020-10-08	31.14	31.35	31.08	31.35	1361815	{}	\N	2020-10-12 17:02:46.017019
89051	SPYV	2020-10-09	31.56	31.57	31.3	31.41	856888	{}	\N	2020-10-12 17:02:46.023412
89052	SPYV	2020-10-12	31.51	31.68	31.43	31.63	317378	{}	\N	2020-10-12 17:02:46.029068
89053	TAC	2020-09-28	5.88	6.08	5.88	6.02	109750	{}	\N	2020-10-12 17:02:47.499537
89054	TAC	2020-09-29	6.04	6.16	6.02	6.15	199097	{}	\N	2020-10-12 17:02:47.511149
89055	TAC	2020-09-30	6.14	6.215	6.115	6.15	248639	{}	\N	2020-10-12 17:02:47.519452
89056	TAC	2020-10-01	6.2	6.31	6.15	6.29	205087	{}	\N	2020-10-12 17:02:47.525783
89057	TAC	2020-10-02	6.2	6.23	6.12	6.16	256727	{}	\N	2020-10-12 17:02:47.532314
89058	TAC	2020-10-05	6.19	6.31	6.17	6.28	406120	{}	\N	2020-10-12 17:02:47.539859
89059	TAC	2020-10-06	6.29	6.35	6.21	6.26	217276	{}	\N	2020-10-12 17:02:47.545322
89060	TAC	2020-10-07	6.3	6.33	6.23	6.25	142481	{}	\N	2020-10-12 17:02:47.552255
89061	TAC	2020-10-08	6.23	6.35	6.23	6.35	99356	{}	\N	2020-10-12 17:02:47.562248
89062	TAC	2020-10-09	6.4	6.4	6.29	6.39	216950	{}	\N	2020-10-12 17:02:47.571416
89063	TAC	2020-10-12	6.35	6.36	6.325	6.36	24774	{}	\N	2020-10-12 17:02:47.578852
89064	TAL	2020-09-28	72.96	75.29	72.1	74.28	3779537	{}	\N	2020-10-12 17:02:48.648465
89065	TAL	2020-09-29	74.2	74.94	72.23	74.73	2911658	{}	\N	2020-10-12 17:02:48.660517
89066	TAL	2020-09-30	74.81	76.53	74.59	76.04	3430204	{}	\N	2020-10-12 17:02:48.670828
89067	TAL	2020-10-01	76.8	77.2571	76.19	76.74	2143726	{}	\N	2020-10-12 17:02:48.676901
89068	TAL	2020-10-02	75.41	77	74.8	75.67	1839809	{}	\N	2020-10-12 17:02:48.68331
89069	TAL	2020-10-05	76.27	77.14	75.06	76.43	3391847	{}	\N	2020-10-12 17:02:48.689504
89070	TAL	2020-10-06	77.36	77.8	76.35	76.93	3131017	{}	\N	2020-10-12 17:02:48.695121
89071	TAL	2020-10-07	78.47	80.505	78.03	79.76	3552588	{}	\N	2020-10-12 17:02:48.701531
89072	TAL	2020-10-08	79.59	79.75	77.93	78.58	3313655	{}	\N	2020-10-12 17:02:48.707631
89073	TAL	2020-10-09	79.26	80.66	77.93	79.38	3007227	{}	\N	2020-10-12 17:02:48.714192
89074	TAL	2020-10-12	79.59	80.26	78.81	79.12	1924396	{}	\N	2020-10-12 17:02:48.721886
89075	THC	2020-09-28	23.01	24.1	23.01	23.64	1763270	{}	\N	2020-10-12 17:02:49.78863
89076	THC	2020-09-29	23.59	23.81	22.52	23.49	2992225	{}	\N	2020-10-12 17:02:49.798584
89077	THC	2020-09-30	23.76	25.21	23.76	24.51	3427101	{}	\N	2020-10-12 17:02:49.775653
89078	THC	2020-10-01	24.49	24.94	23.59	24.87	2511885	{}	\N	2020-10-12 17:02:49.781729
89079	THC	2020-10-02	24.09	26.07	24.06	25.83	2539016	{}	\N	2020-10-12 17:02:49.788418
89080	THC	2020-10-05	26.3	27.91	26.08	27.81	2537069	{}	\N	2020-10-12 17:02:49.794239
89081	THC	2020-10-06	28.31	28.41	26.79	26.8	2917659	{}	\N	2020-10-12 17:02:49.799948
89082	THC	2020-10-07	27.24	28.48	27.24	27.37	2684490	{}	\N	2020-10-12 17:02:49.805472
89083	THC	2020-10-08	27.56	28.39	27.44	28.26	1327816	{}	\N	2020-10-12 17:02:49.810769
89084	THC	2020-10-09	30.07	30.08	27.8	28.47	3532674	{}	\N	2020-10-12 17:02:49.816661
89085	THC	2020-10-12	28.3	28.66	27.21	27.5	654494	{}	\N	2020-10-12 17:02:49.822653
89086	TLH	2020-09-28	167.62	167.73	167.3	167.47	73049	{}	\N	2020-10-12 17:02:50.811841
89087	TLH	2020-09-29	167.63	167.9	167.41	167.59	63525	{}	\N	2020-10-12 17:02:50.82469
89088	TLH	2020-09-30	167.06	167.4599	165.79	166.49	221691	{}	\N	2020-10-12 17:02:50.832683
89089	TLH	2020-10-01	165.71	166.77	165.48	166.55	171243	{}	\N	2020-10-12 17:02:50.839297
89090	TLH	2020-10-02	166.73	166.82	165.69	166.05	139460	{}	\N	2020-10-12 17:02:50.847713
89091	TLH	2020-10-05	164.64	164.9899	163.63	163.64	244419	{}	\N	2020-10-12 17:02:50.853928
89092	TLH	2020-10-06	163.38	164.995	163.055	164.34	292956	{}	\N	2020-10-12 17:02:50.859222
89093	TLH	2020-10-07	163.65	164.19	163.05	163.31	269617	{}	\N	2020-10-12 17:02:50.864326
89094	TLH	2020-10-08	163.88	164.115	163.69	164.03	164869	{}	\N	2020-10-12 17:02:50.870169
89095	TLH	2020-10-09	163.57	164.273	163.09	163.9	107743	{}	\N	2020-10-12 17:02:50.877124
89096	TLH	2020-10-12	164.2297	164.269	163.68	163.96	32567	{}	\N	2020-10-12 17:02:50.885489
89097	TPH	2020-09-28	17.32	17.79	17.13	17.74	972809	{}	\N	2020-10-12 17:02:51.913792
89098	TPH	2020-09-29	17.79	17.89	17.27	17.47	2240392	{}	\N	2020-10-12 17:02:51.926912
89099	TPH	2020-09-30	17.53	18.345	17.43	18.14	3117378	{}	\N	2020-10-12 17:02:51.934366
89100	TPH	2020-10-01	18.33	18.58	18.17	18.56	2512757	{}	\N	2020-10-12 17:02:51.940476
89101	TPH	2020-10-02	18.08	19	17.98	18.87	2073942	{}	\N	2020-10-12 17:02:51.945498
89102	TPH	2020-10-05	19.01	19.25	18.21	18.67	2544084	{}	\N	2020-10-12 17:02:51.950105
89103	TPH	2020-10-06	18.84	18.92	17.94	17.95	2490625	{}	\N	2020-10-12 17:02:51.955208
89104	TPH	2020-10-07	18.3	18.64	18	18.19	2217328	{}	\N	2020-10-12 17:02:51.960578
89105	TPH	2020-10-08	18.49	18.75	18.29	18.74	1856032	{}	\N	2020-10-12 17:02:51.965194
89106	TPH	2020-10-09	19.05	19.19	18.81	19	2068875	{}	\N	2020-10-12 17:02:51.969903
89107	TPH	2020-10-12	19.06	19.2095	18.72	18.99	648194	{}	\N	2020-10-12 17:02:51.974795
89108	SDP	2020-09-28	11.27	11.27	11.04	11.04	9951	{}	\N	2020-10-12 17:02:53.083691
89109	SDP	2020-09-29	11.2	11.2	11.1292	11.1292	5176	{}	\N	2020-10-12 17:02:53.097733
89110	SDP	2020-09-30	10.99	11.115	10.95	10.97	4976	{}	\N	2020-10-12 17:02:53.106074
89111	SDP	2020-10-01	10.96	11.1	10.73	10.73	8108	{}	\N	2020-10-12 17:02:53.112799
89112	SDP	2020-10-02	10.9401	10.9401	10.5054	10.5054	2442	{}	\N	2020-10-12 17:02:53.119046
89113	SDP	2020-10-05	10.29	10.3094	10.24	10.2404	1396	{}	\N	2020-10-12 17:02:53.125366
89114	SDP	2020-10-06	10.279	10.279	9.88	10.0529	1340	{}	\N	2020-10-12 17:02:53.131078
89115	SDP	2020-10-07	10.05	10.05	9.9453	9.9453	920	{}	\N	2020-10-12 17:02:53.136762
89116	SDP	2020-10-08	9.81	9.84	9.606	9.639	13020	{}	\N	2020-10-12 17:02:53.14236
89117	SDP	2020-10-09	9.61	9.75	9.5759	9.659	143691	{}	\N	2020-10-12 17:02:53.147371
89118	SDP	2020-10-12	9.66	9.66	9.44	9.46	160132	{}	\N	2020-10-12 17:02:53.152696
89119	SFIG	2020-09-28	51.59	51.615	51.59	51.615	664	{}	\N	2020-10-12 17:02:54.785764
89120	SFIG	2020-09-29	51.615	51.615	51.615	51.615	3	{}	\N	2020-10-12 17:02:54.79465
89121	SFIG	2020-09-30	51.6133	51.615	51.6133	51.615	721	{}	\N	2020-10-12 17:02:54.801985
89122	SFIG	2020-10-01	51.64	51.8899	51.64	51.8899	2303	{}	\N	2020-10-12 17:02:54.808444
89123	SFIG	2020-10-02	51.76	51.76	51.705	51.705	373	{}	\N	2020-10-12 17:02:54.814859
89124	SFIG	2020-10-05	51.899	51.899	51.705	51.705	943	{}	\N	2020-10-12 17:02:54.819867
89125	SFIG	2020-10-06	51.6201	51.72	51.6201	51.72	2542	{}	\N	2020-10-12 17:02:54.825147
89126	SFIG	2020-10-07	51.8	51.8	51.8	51.8	1126	{}	\N	2020-10-12 17:02:54.830794
89127	SFIG	2020-10-08	51.6806	51.725	51.64	51.725	6127	{}	\N	2020-10-12 17:02:54.835837
89128	SFIG	2020-10-09	51.8021	51.85	51.755	51.755	1638	{}	\N	2020-10-12 17:02:54.840816
89129	SFIG	2020-10-12	51.84	51.92	51.84	51.92	1855	{}	\N	2020-10-12 17:02:54.84548
89130	SFUN	2020-09-28	11.1	11.1	10.77	10.87	2171	{}	\N	2020-10-12 17:02:55.792637
89131	SFUN	2020-09-29	11.02	11.03	10.71	11	4338	{}	\N	2020-10-12 17:02:55.806773
89132	SFUN	2020-09-30	11.03	12.25	10.97	11.51	19174	{}	\N	2020-10-12 17:02:55.815057
89133	SFUN	2020-10-01	11.62	12.28	11.62	11.75	18840	{}	\N	2020-10-12 17:02:55.821453
89134	SFUN	2020-10-02	11.57	11.995	11.45	11.5	14489	{}	\N	2020-10-12 17:02:55.827575
89135	SFUN	2020-10-05	11.5	12	11.5	11.8	12932	{}	\N	2020-10-12 17:02:55.833874
89136	SFUN	2020-10-06	11.9	12.4829	10.38	10.7853	94281	{}	\N	2020-10-12 17:02:55.841561
89137	SFUN	2020-10-07	10.88	11.0899	10.6401	10.81	15680	{}	\N	2020-10-12 17:02:55.847331
89138	SFUN	2020-10-08	10.9	11.01	10.551	10.6	19028	{}	\N	2020-10-12 17:02:55.852558
89139	SFUN	2020-10-09	10.71	10.98	10.5	10.7	26872	{}	\N	2020-10-12 17:02:55.857539
89140	SFUN	2020-10-12	10.59	10.59	10.5	10.5	10040	{}	\N	2020-10-12 17:02:55.862418
89141	SKYW	2020-09-28	30.44	31.9192	30.32	31.21	419981	{}	\N	2020-10-12 17:02:57.995716
89142	SKYW	2020-09-29	30.67	30.67	29.26	29.82	623680	{}	\N	2020-10-12 17:02:58.01024
89143	SKYW	2020-09-30	30.26	31.54	29.75	29.86	571327	{}	\N	2020-10-12 17:02:58.019691
89144	SKYW	2020-10-01	30.12	30.78	29.92	30.76	319871	{}	\N	2020-10-12 17:02:58.026334
89145	SKYW	2020-10-02	29.54	31.9799	29.5	31.71	364263	{}	\N	2020-10-12 17:02:58.032627
89146	SKYW	2020-10-05	31.83	32.43	31.38	31.56	244183	{}	\N	2020-10-12 17:02:58.037783
89147	SKYW	2020-10-06	32.29	32.4	30.61	30.63	558176	{}	\N	2020-10-12 17:02:58.042992
89148	SKYW	2020-10-07	31.85	32	31.07	31.45	520385	{}	\N	2020-10-12 17:02:58.048397
89149	SKYW	2020-10-08	32.32	32.4599	30.87	31.5	438206	{}	\N	2020-10-12 17:02:58.053554
89150	SKYW	2020-10-09	31.74	32.17	30.88	31.79	296895	{}	\N	2020-10-12 17:02:58.058884
89151	SKYW	2020-10-12	31.68	31.77	31.04	31.36	102608	{}	\N	2020-10-12 17:02:58.064326
89152	SOHO	2020-09-28	1.81	1.9	1.76	1.9	74637	{}	\N	2020-10-12 17:03:02.104034
89153	SOHO	2020-09-29	1.9	1.91	1.8	1.82	96909	{}	\N	2020-10-12 17:03:02.112897
89154	SOHO	2020-09-30	1.85	1.89	1.8	1.8	25044	{}	\N	2020-10-12 17:03:02.120137
89155	SOHO	2020-10-01	1.81	1.85	1.78	1.82	42224	{}	\N	2020-10-12 17:03:02.126095
89156	SOHO	2020-10-02	1.7	1.86	1.68	1.85	87734	{}	\N	2020-10-12 17:03:02.131404
89157	SOHO	2020-10-05	1.85	1.88	1.81	1.82	50528	{}	\N	2020-10-12 17:03:02.137318
89158	SOHO	2020-10-06	1.83	1.88	1.7801	1.79	44926	{}	\N	2020-10-12 17:03:02.143164
89159	SOHO	2020-10-07	1.8	1.83	1.78	1.81	29632	{}	\N	2020-10-12 17:03:02.148418
89160	SOHO	2020-10-08	1.81	1.83	1.79	1.82	36951	{}	\N	2020-10-12 17:03:02.153568
89161	SOHO	2020-10-09	1.82	1.82	1.7501	1.77	49879	{}	\N	2020-10-12 17:03:02.158626
89162	SOHO	2020-10-12	1.79	1.82	1.71	1.7514	56494	{}	\N	2020-10-12 17:03:02.163682
89163	SQQQ	2020-09-28	24.22	24.99	24.04	24.06	70394794	{}	\N	2020-10-12 17:03:03.119431
89164	SQQQ	2020-09-29	24.18	24.52	23.905	24.37	50003932	{}	\N	2020-10-12 17:03:03.131667
89165	SQQQ	2020-09-30	24.32	24.39	23.01	23.85	81534220	{}	\N	2020-10-12 17:03:03.140204
89166	SQQQ	2020-10-01	22.85	23.34	22.57	22.73	66041685	{}	\N	2020-10-12 17:03:03.147489
89167	SQQQ	2020-10-02	24.22	24.86	23.33	24.61	105158516	{}	\N	2020-10-12 17:03:03.154338
89168	SQQQ	2020-10-05	24.1	24.1199	22.97	23.06	45295165	{}	\N	2020-10-12 17:03:03.160383
89169	SQQQ	2020-10-06	23.26	24.52	22.8	24.3	83669050	{}	\N	2020-10-12 17:03:03.166255
89170	SQQQ	2020-10-07	23.58	23.73	22.81	23.02	49768695	{}	\N	2020-10-12 17:03:03.171802
89171	SQQQ	2020-10-08	22.43	22.92	22.4	22.64	38045594	{}	\N	2020-10-12 17:03:03.177236
89172	SQQQ	2020-10-09	22.24	22.3398	21.59	21.62	41727951	{}	\N	2020-10-12 17:03:03.182854
89173	SQQQ	2020-10-12	20.51	20.76	19.31	19.475	43257249	{}	\N	2020-10-12 17:03:03.190052
89174	SRET	2020-09-28	7.81	7.98	7.81	7.93	269358	{}	\N	2020-10-12 17:03:04.413943
89175	SRET	2020-09-29	7.93	7.93	7.7568	7.83	181835	{}	\N	2020-10-12 17:03:04.429132
89176	SRET	2020-09-30	7.86	7.985	7.82	7.85	220066	{}	\N	2020-10-12 17:03:04.437104
89177	SRET	2020-10-01	7.89	7.98	7.8289	7.96	241891	{}	\N	2020-10-12 17:03:04.445009
89178	SRET	2020-10-02	7.86	8.12	7.83	8.11	263910	{}	\N	2020-10-12 17:03:04.451377
89179	SRET	2020-10-05	8.1	8.1	7.97	8.04	258358	{}	\N	2020-10-12 17:03:04.456821
89180	SRET	2020-10-06	8.04	8.12	7.95	7.95	225220	{}	\N	2020-10-12 17:03:04.46205
89181	SRET	2020-10-07	8	8.02	7.9	7.97	191703	{}	\N	2020-10-12 17:03:04.467122
89182	SRET	2020-10-08	7.98	8.13	7.98	8.12	259276	{}	\N	2020-10-12 17:03:04.47201
89183	SRET	2020-10-09	8.16	8.18	8.03	8.05	138147	{}	\N	2020-10-12 17:03:04.477974
89184	SRET	2020-10-12	8.06	8.08	8.01	8.02	234202	{}	\N	2020-10-12 17:03:04.482991
89185	SYRS	2020-09-28	9.29	9.54	8.9798	9.38	382556	{}	\N	2020-10-12 17:03:05.610667
89186	SYRS	2020-09-29	9.35	9.72	9.21	9.56	453538	{}	\N	2020-10-12 17:03:05.623073
89187	SYRS	2020-09-30	9.5	9.7	8.57	8.84	380845	{}	\N	2020-10-12 17:03:05.632007
89188	SYRS	2020-10-01	8.93	9.21	8.64	8.92	328882	{}	\N	2020-10-12 17:03:05.63989
89189	SYRS	2020-10-02	8.51	8.83	8.17	8.18	327773	{}	\N	2020-10-12 17:03:05.646122
89190	SYRS	2020-10-05	8.28	9.13	8.28	9.04	431059	{}	\N	2020-10-12 17:03:05.651891
89191	SYRS	2020-10-06	9.06	9.3492	8.78	8.79	498030	{}	\N	2020-10-12 17:03:05.657558
89192	SYRS	2020-10-07	8.82	9.87	8.82	9.73	603593	{}	\N	2020-10-12 17:03:05.662895
89193	SYRS	2020-10-08	9.74	10.16	9.47	9.59	448472	{}	\N	2020-10-12 17:03:05.667989
89194	SYRS	2020-10-09	9.7	9.9	9.51	9.65	346619	{}	\N	2020-10-12 17:03:05.673052
89195	SYRS	2020-10-12	9.7	10.09	9.56	9.76	132648	{}	\N	2020-10-12 17:03:05.678237
89196	TCBIP	2020-09-28	25.34	25.71	25.34	25.71	9352	{}	\N	2020-10-12 17:03:07.044647
89197	TCBIP	2020-09-29	25.49	25.7	25.4253	25.7	2426	{}	\N	2020-10-12 17:03:07.055872
89198	TCBIP	2020-09-30	25.56	25.6604	25.18	25.18	22577	{}	\N	2020-10-12 17:03:07.063488
89199	TCBIP	2020-10-01	25.19	25.7	25.19	25.7	17612	{}	\N	2020-10-12 17:03:07.068911
89200	TCBIP	2020-10-02	25.47	25.74	25.38	25.74	8734	{}	\N	2020-10-12 17:03:07.07391
89201	TCBIP	2020-10-05	25.7	25.94	25.7	25.94	10775	{}	\N	2020-10-12 17:03:07.078672
89202	TCBIP	2020-10-06	25.93	25.96	25.79	25.95	7673	{}	\N	2020-10-12 17:03:07.082817
89203	TCBIP	2020-10-07	25.95	25.96	25.7265	25.96	3646	{}	\N	2020-10-12 17:03:07.087365
89204	TCBIP	2020-10-08	25.86	25.96	25.65	25.96	5279	{}	\N	2020-10-12 17:03:07.091742
89205	TCBIP	2020-10-09	25.89	25.96	25.4283	25.95	3874	{}	\N	2020-10-12 17:03:07.096165
89206	TCBIP	2020-10-12	25.92	25.95	25.55	25.95	1764	{}	\N	2020-10-12 17:03:07.100458
89207	TEL	2020-09-28	96.92	98.26	96.795	97.09	1484573	{}	\N	2020-10-12 17:03:08.262227
89208	TEL	2020-09-29	97.43	97.97	96.725	97.45	3190482	{}	\N	2020-10-12 17:03:08.27194
89209	TEL	2020-09-30	97.69	99.64	97.17	97.74	2869646	{}	\N	2020-10-12 17:03:08.27902
89210	TEL	2020-10-01	99.3	99.41	97.56	98.55	2570003	{}	\N	2020-10-12 17:03:08.284563
89211	TEL	2020-10-02	96.16	99.31	95	98.36	1830210	{}	\N	2020-10-12 17:03:08.289315
89212	TEL	2020-10-05	99.67	101.63	99.565	101.37	2933466	{}	\N	2020-10-12 17:03:08.295083
89213	TEL	2020-10-06	101.4	103.1	101.075	101.83	3798588	{}	\N	2020-10-12 17:03:08.300319
89214	TEL	2020-10-07	103.15	105.17	103.12	104.55	2871504	{}	\N	2020-10-12 17:03:08.305351
89215	TEL	2020-10-08	105.32	106.26	104.68	106.01	2351816	{}	\N	2020-10-12 17:03:08.310538
89216	TEL	2020-10-09	107.64	109.27	106.97	107.35	2735381	{}	\N	2020-10-12 17:03:08.315446
89217	TEL	2020-10-12	107.84	108.385	107.12	107.4	1391035	{}	\N	2020-10-12 17:03:08.320448
89218	TPX	2020-09-28	90.19	90.76	88.81	90.3	442143	{}	\N	2020-10-12 17:03:09.759361
89219	TPX	2020-09-29	90.04	90.38	87.845	88.24	816036	{}	\N	2020-10-12 17:03:09.770025
89220	TPX	2020-09-30	88.46	90.345	88.46	89.19	660592	{}	\N	2020-10-12 17:03:09.777386
89221	TPX	2020-10-01	90.73	91.99	89.075	89.72	935030	{}	\N	2020-10-12 17:03:09.783875
89222	TPX	2020-10-02	87.54	92.7992	86.35	91.56	1247537	{}	\N	2020-10-12 17:03:09.789706
89223	TPX	2020-10-05	92.27	93.45	90.542	91.75	1032190	{}	\N	2020-10-12 17:03:09.79508
89224	TPX	2020-10-06	92.49	92.77	88.5	88.85	1268607	{}	\N	2020-10-12 17:03:09.800192
89225	TPX	2020-10-07	90.74	92.11	86.93	88.08	1491436	{}	\N	2020-10-12 17:03:09.805333
89226	TPX	2020-10-08	89	90.61	87.6	89.1	1274199	{}	\N	2020-10-12 17:03:09.810303
89227	TPX	2020-10-09	90.28	91.66	89.41	90.14	662171	{}	\N	2020-10-12 17:03:09.815295
89228	TPX	2020-10-12	90.89	92.75	90.51	91.685	314589	{}	\N	2020-10-12 17:03:09.820234
89229	TTC	2020-09-28	84.19	84.79	83.455	84.21	348055	{}	\N	2020-10-12 17:03:10.865522
89230	TTC	2020-09-29	84.62	84.68	83.31	83.74	691534	{}	\N	2020-10-12 17:03:10.881309
89231	TTC	2020-09-30	83.87	85.21	83.41	83.95	1595743	{}	\N	2020-10-12 17:03:10.888472
89232	TTC	2020-10-01	84.62	85.04	83.61	83.89	660041	{}	\N	2020-10-12 17:03:10.895362
89233	TTC	2020-10-02	82.67	85.34	82.5	84.52	724973	{}	\N	2020-10-12 17:03:10.902196
89234	TTC	2020-10-05	85.24	85.9	84.41	85.3	949034	{}	\N	2020-10-12 17:03:10.907616
89235	TTC	2020-10-06	85.49	86.65	84.88	85.26	825593	{}	\N	2020-10-12 17:03:10.913421
89236	TTC	2020-10-07	86.19	87.005	85.75	86.37	1285037	{}	\N	2020-10-12 17:03:10.918739
89237	TTC	2020-10-08	86.52	87.4176	85.805	86.99	693266	{}	\N	2020-10-12 17:03:10.924286
89238	TTC	2020-10-09	87.59	88.18	86.6	87.66	916350	{}	\N	2020-10-12 17:03:10.929848
89239	TTC	2020-10-12	88.36	88.93	87.79	88	183157	{}	\N	2020-10-12 17:03:10.935267
89240	SFHY	2020-09-28	47.71	47.77	47.71	47.77	2541	{}	\N	2020-10-12 17:03:13.484934
89241	SFHY	2020-09-29	47.68	47.68	47.68	47.68	5	{}	\N	2020-10-12 17:03:13.497939
89242	SFHY	2020-09-30	47.8817	47.9041	47.8817	47.9041	286	{}	\N	2020-10-12 17:03:13.507582
89243	SFHY	2020-10-01	47.96	47.9937	47.96	47.9937	1418	{}	\N	2020-10-12 17:03:13.514208
89244	SFHY	2020-10-02	47.77	47.859	47.77	47.8122	4525	{}	\N	2020-10-12 17:03:13.521248
89245	SFHY	2020-10-05	48.03	48.03	47.91	47.95	7192	{}	\N	2020-10-12 17:03:13.526297
89246	SFHY	2020-10-06	48.11	48.17	47.89	47.8901	12862	{}	\N	2020-10-12 17:03:13.531414
89247	SFHY	2020-10-07	48.15	48.175	48.1498	48.15	696	{}	\N	2020-10-12 17:03:13.536796
89248	SFHY	2020-10-08	48.23	48.23	48.15	48.22	945	{}	\N	2020-10-12 17:03:13.542738
89249	SFHY	2020-10-09	48.319	48.39	48.26	48.325	2522	{}	\N	2020-10-12 17:03:13.547716
89250	SFHY	2020-10-12	48.55	48.55	48.55	48.55	100	{}	\N	2020-10-12 17:03:13.553689
89251	SHY	2020-09-28	86.5065	86.51	86.49	86.51	2759733	{}	\N	2020-10-12 17:03:14.831339
89252	SHY	2020-09-29	86.51	86.515	86.5	86.5	2646315	{}	\N	2020-10-12 17:03:14.843247
89253	SHY	2020-09-30	86.51	86.51	86.49	86.51	3107213	{}	\N	2020-10-12 17:03:14.849915
89254	SHY	2020-10-01	86.45	86.47	86.45	86.45	3851846	{}	\N	2020-10-12 17:03:14.855337
89255	SHY	2020-10-02	86.46	86.47	86.44	86.46	3069826	{}	\N	2020-10-12 17:03:14.860147
89256	SHY	2020-10-05	86.45	86.45	86.42	86.43	3124218	{}	\N	2020-10-12 17:03:14.865599
89257	SHY	2020-10-06	86.43	86.44	86.42	86.43	2610573	{}	\N	2020-10-12 17:03:14.870166
89258	SHY	2020-10-07	86.4	86.42	86.4	86.42	3631864	{}	\N	2020-10-12 17:03:14.874029
89259	SHY	2020-10-08	86.42	86.43	86.41	86.42	2026238	{}	\N	2020-10-12 17:03:14.878844
89260	SHY	2020-10-09	86.43	86.43	86.41	86.42	1988967	{}	\N	2020-10-12 17:03:14.883335
89261	SHY	2020-10-12	86.42	86.43	86.41	86.42	1003509	{}	\N	2020-10-12 17:03:14.887587
89262	SOHOO	2020-09-28	5.19	5.74	5.19	5.69	4013	{}	\N	2020-10-12 17:03:16.233211
89263	SOHOO	2020-09-29	5.69	5.69	4.8457	5.38	6086	{}	\N	2020-10-12 17:03:16.247195
89264	SOHOO	2020-09-30	5.22	5.5	5.21	5.46	3571	{}	\N	2020-10-12 17:03:16.254425
89265	SOHOO	2020-10-01	5.31	5.62	5.18	5.62	5367	{}	\N	2020-10-12 17:03:16.260108
89266	SOHOO	2020-10-02	5.5	5.76	5.5	5.68	1757	{}	\N	2020-10-12 17:03:16.264939
89267	SOHOO	2020-10-05	5.675	5.69	5.25	5.69	4187	{}	\N	2020-10-12 17:03:16.270163
89268	SOHOO	2020-10-06	5.67	5.69	5.48	5.69	701	{}	\N	2020-10-12 17:03:16.274303
89269	SOHOO	2020-10-07	5.33	5.59	5.33	5.59	2554	{}	\N	2020-10-12 17:03:16.278129
89270	SOHOO	2020-10-08	5.31	5.54	5.25	5.54	2558	{}	\N	2020-10-12 17:03:16.282422
89271	SOHOO	2020-10-09	5.5	5.55	5.44	5.44	4413	{}	\N	2020-10-12 17:03:16.286468
89272	SOHOO	2020-10-12	5.2701	5.35	5.26	5.3	2504	{}	\N	2020-10-12 17:03:16.290255
89273	SPVM	2020-09-28	35.042	35.042	34.9335	34.9335	846	{}	\N	2020-10-12 17:03:18.28601
89274	SPVM	2020-09-29	34.848	34.848	34.56	34.73	665	{}	\N	2020-10-12 17:03:18.294632
89275	SPVM	2020-09-30	35.1349	35.214	34.73	34.96	2398	{}	\N	2020-10-12 17:03:18.300963
89276	SPVM	2020-10-01	34.73	34.99	34.73	34.8216	3187	{}	\N	2020-10-12 17:03:18.306768
89277	SPVM	2020-10-02	35.241	35.3431	35.241	35.3431	1561	{}	\N	2020-10-12 17:03:18.312012
89278	SPVM	2020-10-05	35.94	36.0037	35.9262	36.0037	2723	{}	\N	2020-10-12 17:03:18.316987
89279	SPVM	2020-10-06	36.231	36.33	35.6	35.6	2017	{}	\N	2020-10-12 17:03:18.321675
89280	SPVM	2020-10-07	36.35	36.35	36.12	36.24	1250	{}	\N	2020-10-12 17:03:18.326264
89281	SPVM	2020-10-08	36.33	36.83	36.33	36.83	3253	{}	\N	2020-10-12 17:03:18.331137
89282	SPVM	2020-10-09	36.8375	36.94	36.75	36.75	1029	{}	\N	2020-10-12 17:03:18.335584
89283	SQ	2020-09-28	160.49	164.5	158.23	160.76	10524429	{}	\N	2020-10-12 17:03:19.647761
89284	SQ	2020-09-29	164	167.56	163.13	164.81	24100779	{}	\N	2020-10-12 17:03:19.659052
89285	SQ	2020-09-30	164.47	166.19	161.23	162.55	13066068	{}	\N	2020-10-12 17:03:19.666246
89286	SQ	2020-10-01	166.14	168.61	164.5801	168.49	13295567	{}	\N	2020-10-12 17:03:19.672706
89287	SQ	2020-10-02	164.05	171.24	163.8006	169.61	22153671	{}	\N	2020-10-12 17:03:19.678022
89288	SQ	2020-10-05	172.43	181.89	171.5	180.92	33392328	{}	\N	2020-10-12 17:03:19.682813
89289	SQ	2020-10-06	180.13	182.24	172.61	175.34	24859653	{}	\N	2020-10-12 17:03:19.687689
89290	SQ	2020-10-07	180.61	182.75	179.02	180.18	14292965	{}	\N	2020-10-12 17:03:19.692267
89291	SQ	2020-10-08	183.18	185.3699	181.72	183.5	14440583	{}	\N	2020-10-12 17:03:19.696836
89292	SQ	2020-10-09	185.12	188.74	183.73	187.28	13423944	{}	\N	2020-10-12 17:03:19.701656
89293	SQ	2020-10-12	190.68	191.94	184.26	187.34	10650824	{}	\N	2020-10-12 17:03:19.706345
89294	SRAX	2020-09-28	2.75	2.87	2.68	2.7506	90365	{}	\N	2020-10-12 17:03:21.036206
89295	SRAX	2020-09-29	2.81	2.87	2.62	2.82	112994	{}	\N	2020-10-12 17:03:21.049154
89296	SRAX	2020-09-30	2.75	3.1	2.63	2.96	338608	{}	\N	2020-10-12 17:03:21.057962
89297	SRAX	2020-10-01	3.1	3.1	2.65	2.75	96704	{}	\N	2020-10-12 17:03:21.066527
89298	SRAX	2020-10-02	2.65	2.97	2.65	2.75	203742	{}	\N	2020-10-12 17:03:21.073294
89299	SRAX	2020-10-05	2.75	2.8	2.52	2.59	64594	{}	\N	2020-10-12 17:03:21.079282
89300	SRAX	2020-10-06	2.57	2.7	2.57	2.6	45722	{}	\N	2020-10-12 17:03:21.085567
89301	SRAX	2020-10-07	2.55	2.6799	2.55	2.61	22407	{}	\N	2020-10-12 17:03:21.091554
89302	SRAX	2020-10-08	2.59	2.65	2.32	2.58	106919	{}	\N	2020-10-12 17:03:21.097412
89303	SRAX	2020-10-09	2.53	2.59	2.5	2.5	21466	{}	\N	2020-10-12 17:03:21.103339
89304	SRAX	2020-10-12	2.45	2.59	2.37	2.52	37729	{}	\N	2020-10-12 17:03:21.108871
89305	SZK	2020-09-28	12.67	12.6874	12.6	12.6874	2339	{}	\N	2020-10-12 17:03:22.202221
89306	SZK	2020-09-29	12.7	12.8236	12.6795	12.8236	1932	{}	\N	2020-10-12 17:03:22.210881
89307	SZK	2020-09-30	12.38	12.4906	12.38	12.4906	278	{}	\N	2020-10-12 17:03:22.218325
89308	SZK	2020-10-01	12.4	12.4	12.2632	12.3	4791	{}	\N	2020-10-12 17:03:22.22454
89309	SZK	2020-10-02	12.7102	12.7446	12.45	12.481	6875	{}	\N	2020-10-12 17:03:22.22981
89310	SZK	2020-10-05	12.2349	12.2349	12.2349	12.2349	40	{}	\N	2020-10-12 17:03:22.23468
89311	SZK	2020-10-06	12.4578	12.4578	12.4578	12.4578	23	{}	\N	2020-10-12 17:03:22.239621
89312	SZK	2020-10-07	12.195	12.195	12.015	12.015	787	{}	\N	2020-10-12 17:03:22.244718
89313	SZK	2020-10-08	11.9278	11.9278	11.9278	11.9278	239	{}	\N	2020-10-12 17:03:22.249587
89314	SZK	2020-10-09	11.781	11.781	11.722	11.722	673	{}	\N	2020-10-12 17:03:22.254106
89315	SZK	2020-10-12	11.4163	11.4163	11.371	11.371	690	{}	\N	2020-10-12 17:03:22.258895
89316	TAIT	2020-09-28	2.3	2.34	2.3	2.32	5435	{}	\N	2020-10-12 17:03:23.239377
89317	TAIT	2020-09-29	2.32	2.39	2.32	2.35	16549	{}	\N	2020-10-12 17:03:23.250459
89318	TAIT	2020-09-30	2.358	2.4	2.3406	2.4	7547	{}	\N	2020-10-12 17:03:23.258541
89319	TAIT	2020-10-01	2.4001	2.4169	2.36	2.37	7529	{}	\N	2020-10-12 17:03:23.264671
89320	TAIT	2020-10-02	2.37	2.38	2.2901	2.33	14141	{}	\N	2020-10-12 17:03:23.270662
89321	TAIT	2020-10-05	2.3	2.3278	2.2	2.21	46191	{}	\N	2020-10-12 17:03:23.276371
89322	TAIT	2020-10-06	2.23	2.31	2.21	2.26	10462	{}	\N	2020-10-12 17:03:23.281492
89323	TAIT	2020-10-07	2.27	2.37	2.26	2.3304	4248	{}	\N	2020-10-12 17:03:23.287221
89324	TAIT	2020-10-08	2.3561	2.37	2.34	2.37	7922	{}	\N	2020-10-12 17:03:23.292087
89325	TAIT	2020-10-09	2.35	2.365	2.35	2.365	2168	{}	\N	2020-10-12 17:03:23.296988
89326	TAIT	2020-10-12	2.32	2.35	2.31	2.3473	4452	{}	\N	2020-10-12 17:03:23.301828
89327	TLEH	2020-09-28	26.9369	26.9369	26.9369	26.9369	13	{}	\N	2020-10-12 17:03:24.92331
89328	TLEH	2020-09-29	26.9279	26.9279	26.9279	26.9279	0	{}	\N	2020-10-12 17:03:24.933052
89329	TLEH	2020-09-30	27.2	27.2	27.2	27.2	53	{}	\N	2020-10-12 17:03:24.940637
89330	TLEH	2020-10-01	27.32	27.3969	27.32	27.3969	1543	{}	\N	2020-10-12 17:03:24.947116
89331	TLEH	2020-10-02	27.1647	27.1647	27.1647	27.1647	0	{}	\N	2020-10-12 17:03:24.953439
89332	TLEH	2020-10-05	27.3998	27.3998	27.3998	27.3998	2	{}	\N	2020-10-12 17:03:24.960101
89333	TLEH	2020-10-06	27.4658	27.4658	27.4658	27.4658	0	{}	\N	2020-10-12 17:03:24.965555
89334	TLEH	2020-10-07	27.6843	27.6843	27.6843	27.6843	0	{}	\N	2020-10-12 17:03:24.971058
89335	TLEH	2020-10-08	27.835	27.835	27.835	27.835	0	{}	\N	2020-10-12 17:03:24.976473
89336	TLTE	2020-09-28	47.1	47.15	46.88	46.94	35072	{}	\N	2020-10-12 17:03:26.625923
89337	TLTE	2020-09-29	47.03	47.12	46.92	47.0695	9322	{}	\N	2020-10-12 17:03:26.646139
89338	TLTE	2020-09-30	47.49	47.655	47.49	47.5346	5232	{}	\N	2020-10-12 17:03:26.664866
89339	TLTE	2020-10-01	47.89	48.06	47.73	48.01	16713	{}	\N	2020-10-12 17:03:26.672254
89340	TLTE	2020-10-02	47.55	47.73	47.55	47.6106	4191	{}	\N	2020-10-12 17:03:26.679986
89341	TLTE	2020-10-05	47.895	48.1493	47.895	48.1493	1128	{}	\N	2020-10-12 17:03:26.685555
89342	TLTE	2020-10-06	48.38	48.4694	48.114	48.114	3320	{}	\N	2020-10-12 17:03:26.690592
89343	TLTE	2020-10-07	48.54	48.675	48.5	48.64	3483	{}	\N	2020-10-12 17:03:26.695552
89344	TLTE	2020-10-08	48.9	48.9841	48.8739	48.91	5969	{}	\N	2020-10-12 17:03:26.700112
89345	TLTE	2020-10-09	49.21	49.35	49.21	49.31	4643	{}	\N	2020-10-12 17:03:26.705085
89346	TLTE	2020-10-12	49.44	49.5301	49.38	49.51	3121	{}	\N	2020-10-12 17:03:26.710011
89347	TYG	2020-09-28	14.84	14.87	14.43	14.77	147224	{}	\N	2020-10-12 17:03:28.033445
89348	TYG	2020-09-29	14.75	15.0615	14.68	14.97	211455	{}	\N	2020-10-12 17:03:28.040358
89349	TYG	2020-09-30	15.04	15.2028	14.62	14.79	280578	{}	\N	2020-10-12 17:03:28.045381
89350	TYG	2020-10-01	14.69	14.8678	14.52	14.62	100043	{}	\N	2020-10-12 17:03:28.05253
89351	TYG	2020-10-02	14.26	14.8333	14.26	14.55	265983	{}	\N	2020-10-12 17:03:28.058455
89352	TYG	2020-10-05	14.9	15.59	14.9	15.55	847892	{}	\N	2020-10-12 17:03:28.065832
89353	TYG	2020-10-06	15.65	15.97	15.34	15.34	299014	{}	\N	2020-10-12 17:03:28.075072
89354	TYG	2020-10-07	15.33	15.9	15.33	15.64	190562	{}	\N	2020-10-12 17:03:28.084332
89355	TYG	2020-10-08	15.77	16.48	15.77	16.42	255996	{}	\N	2020-10-12 17:03:28.093365
89356	TYG	2020-10-09	16.52	16.74	16.23	16.32	335896	{}	\N	2020-10-12 17:03:28.105339
89357	TYG	2020-10-12	16.26	16.3389	16.21	16.29	69465	{}	\N	2020-10-12 17:03:28.114617
89358	UONEK	2020-09-28	1.25	1.54	1.14	1.29	16777294	{}	\N	2020-10-12 17:03:29.308138
89359	UONEK	2020-09-29	1.16	1.54	1.13	1.42	6980297	{}	\N	2020-10-12 17:03:29.322609
89360	UONEK	2020-09-30	1.21	1.296	1.15	1.17	1790533	{}	\N	2020-10-12 17:03:29.331948
89361	UONEK	2020-10-01	1.16	1.2	1.12	1.14	954942	{}	\N	2020-10-12 17:03:29.338484
89362	UONEK	2020-10-02	1.06	1.2	1.06	1.16	927506	{}	\N	2020-10-12 17:03:29.344237
89363	UONEK	2020-10-05	1.16	1.18	1.1	1.13	638371	{}	\N	2020-10-12 17:03:29.350614
89364	UONEK	2020-10-06	1.13	1.24	1.1	1.16	1295090	{}	\N	2020-10-12 17:03:29.356189
89365	UONEK	2020-10-07	1.15	1.45	1.13	1.31	4335370	{}	\N	2020-10-12 17:03:29.361217
89366	UONEK	2020-10-08	1.4	1.52	1.25	1.32	4960616	{}	\N	2020-10-12 17:03:29.366914
89367	UONEK	2020-10-09	1.28	1.3086	1.2	1.21	1099057	{}	\N	2020-10-12 17:03:29.371992
89368	UONEK	2020-10-12	1.24	1.2499	1.14	1.1511	525433	{}	\N	2020-10-12 17:03:29.376831
89369	SGC	2020-09-28	22.66	23	22.5	22.82	59132	{}	\N	2020-10-12 17:03:30.733282
89370	SGC	2020-09-29	22.83	23.0523	21.81	22.28	91248	{}	\N	2020-10-12 17:03:30.743075
89371	SGC	2020-09-30	22.37	23.535	22.37	23.23	114054	{}	\N	2020-10-12 17:03:30.749596
89372	SGC	2020-10-01	23.44	25.68	23.39	25.23	141217	{}	\N	2020-10-12 17:03:30.755606
89373	SGC	2020-10-02	24.63	25.295	23.53	24.43	66283	{}	\N	2020-10-12 17:03:30.761133
89374	SGC	2020-10-05	24.53	24.7977	23.9601	24.19	42361	{}	\N	2020-10-12 17:03:30.76652
89375	SGC	2020-10-06	24.33	24.8986	23.9	23.97	35648	{}	\N	2020-10-12 17:03:30.771383
89376	SGC	2020-10-07	24.19	25.62	24.06	25.45	50018	{}	\N	2020-10-12 17:03:30.775815
89377	SGC	2020-10-08	25.66	25.915	25.07	25.45	37289	{}	\N	2020-10-12 17:03:30.78066
89378	SGC	2020-10-09	25.49	25.49	25	25.16	33982	{}	\N	2020-10-12 17:03:30.78549
89379	SGC	2020-10-12	25.16	25.32	24.9	25.05	9386	{}	\N	2020-10-12 17:03:30.791858
89380	SLM	2020-09-28	7.99	8.12	7.95	7.97	3307717	{}	\N	2020-10-12 17:03:31.740254
89381	SLM	2020-09-29	7.96	8.065	7.9074	7.96	2245275	{}	\N	2020-10-12 17:03:31.750476
89382	SLM	2020-09-30	7.98	8.16	7.98	8.09	4612741	{}	\N	2020-10-12 17:03:31.759716
89383	SLM	2020-10-01	8.08	8.17	8	8.14	2745385	{}	\N	2020-10-12 17:03:31.767108
89384	SLM	2020-10-02	8.01	8.45	7.95	8.3	5326994	{}	\N	2020-10-12 17:03:31.77331
89385	SLM	2020-10-05	8.38	8.495	8.3	8.47	3438812	{}	\N	2020-10-12 17:03:31.777706
89386	SLM	2020-10-06	8.54	8.71	8.445	8.46	3751157	{}	\N	2020-10-12 17:03:31.782775
89387	SLM	2020-10-07	8.52	8.655	8.35	8.56	3392303	{}	\N	2020-10-12 17:03:31.787514
89388	SLM	2020-10-08	8.65	8.82	8.59	8.75	3361755	{}	\N	2020-10-12 17:03:31.792098
89389	SLM	2020-10-09	8.82	8.91	8.7101	8.76	4310415	{}	\N	2020-10-12 17:03:31.796634
89390	SLM	2020-10-12	8.79	8.96	8.76	8.89	887302	{}	\N	2020-10-12 17:03:31.80138
89391	SLRC	2020-09-28	15.94	16.56	15.94	16.33	117857	{}	\N	2020-10-12 17:03:33.020863
89392	SLRC	2020-09-29	16.26	16.4	15.89	15.94	151296	{}	\N	2020-10-12 17:03:33.033487
89393	SLRC	2020-09-30	16.03	16.25	15.79	15.85	140151	{}	\N	2020-10-12 17:03:33.041591
89394	SLRC	2020-10-01	15.84	15.95	15.67	15.86	110375	{}	\N	2020-10-12 17:03:33.048775
89395	SLRC	2020-10-02	15.74	16.36	15.66	16.29	138469	{}	\N	2020-10-12 17:03:33.055224
89396	SLRC	2020-10-05	16.32	16.56	16.2	16.36	125759	{}	\N	2020-10-12 17:03:33.060374
89397	SLRC	2020-10-06	16.39	16.52	16.26	16.26	139970	{}	\N	2020-10-12 17:03:33.065546
89398	SLRC	2020-10-07	16.33	16.6	16.27	16.48	130964	{}	\N	2020-10-12 17:03:33.07128
89399	SLRC	2020-10-08	16.54	16.7249	16.5	16.58	121862	{}	\N	2020-10-12 17:03:33.076272
89400	SLRC	2020-10-09	16.62	16.7	16.51	16.57	77467	{}	\N	2020-10-12 17:03:33.081648
89401	SLRC	2020-10-12	16.55	16.6999	16.35	16.39	23853	{}	\N	2020-10-12 17:03:33.086838
89402	SRC	2020-09-28	34.89	35.45	34.44	35.18	558304	{}	\N	2020-10-12 17:03:34.151003
89403	SRC	2020-09-29	34.39	34.48	33.27	33.71	735049	{}	\N	2020-10-12 17:03:34.164481
89404	SRC	2020-09-30	34.07	34.525	33.23	33.75	1490182	{}	\N	2020-10-12 17:03:34.172887
89405	SRC	2020-10-01	33.92	34.7	33.6	34.69	1184824	{}	\N	2020-10-12 17:03:34.177366
89406	SRC	2020-10-02	33.97	35.81	33.66	35.7	945165	{}	\N	2020-10-12 17:03:34.181713
89407	SRC	2020-10-05	36	36.08	34.905	35.34	858626	{}	\N	2020-10-12 17:03:34.187176
89408	SRC	2020-10-06	35.65	35.83	34.91	35.11	910991	{}	\N	2020-10-12 17:03:34.191259
89409	SRC	2020-10-07	35.4	35.46	34.83	34.9	641810	{}	\N	2020-10-12 17:03:34.194989
89410	SRC	2020-10-08	35.03	35.31	34.72	35	938053	{}	\N	2020-10-12 17:03:34.198994
89411	SRC	2020-10-09	35.37	35.44	34.66	34.71	925483	{}	\N	2020-10-12 17:03:34.203068
89412	SRC	2020-10-12	34.71	34.76	34.21	34.67	220020	{}	\N	2020-10-12 17:03:34.207028
89413	TAN	2020-09-28	58.99	60.57	58.7	60.54	1506383	{}	\N	2020-10-12 17:03:35.435568
89414	TAN	2020-09-29	61.31	62.8	61.19	62.52	1354306	{}	\N	2020-10-12 17:03:35.44828
89415	TAN	2020-09-30	63.37	65.7611	63.1145	64.79	1722295	{}	\N	2020-10-12 17:03:35.45622
89416	TAN	2020-10-01	66.02	67.18	65.17	67.18	1552892	{}	\N	2020-10-12 17:03:35.461716
89417	TAN	2020-10-02	65.44	67.98	65.1	66.59	1635996	{}	\N	2020-10-12 17:03:35.466295
89418	TAN	2020-10-05	68.35	70.55	68.0264	70.55	2434192	{}	\N	2020-10-12 17:03:35.471107
89419	TAN	2020-10-06	71.44	72.6	70.06	71.32	3219168	{}	\N	2020-10-12 17:03:35.4752
89420	TAN	2020-10-07	72.81	75.5199	72.8	75.18	2908264	{}	\N	2020-10-12 17:03:35.479047
89421	TAN	2020-10-08	77.5	77.5	74.23	75	4110141	{}	\N	2020-10-12 17:03:35.482872
89422	TAN	2020-10-09	73.93	76.36	73.28	75.66	2210206	{}	\N	2020-10-12 17:03:35.487031
89423	TAN	2020-10-12	76.25	76.27	72.25	73.56	2167032	{}	\N	2020-10-12 17:03:35.491062
89424	TMF	2020-09-28	41.19	41.19	40.8	40.97	461613	{}	\N	2020-10-12 17:03:37.196918
89425	TMF	2020-09-29	41.14	41.41	40.91	41.1	719045	{}	\N	2020-10-12 17:03:37.209113
89426	TMF	2020-09-30	40.49	40.49	39.39	39.99	1023350	{}	\N	2020-10-12 17:03:37.217443
89427	TMF	2020-10-01	39.47	40.41	39.22	40.22	1023896	{}	\N	2020-10-12 17:03:37.224069
89428	TMF	2020-10-02	40.33	40.3362	39.27	39.74	835488	{}	\N	2020-10-12 17:03:37.229986
89429	TMF	2020-10-05	38.38	38.38	37.25	37.25	1018447	{}	\N	2020-10-12 17:03:37.234991
89430	TMF	2020-10-06	37.23	38.62	36.74	37.93	1076689	{}	\N	2020-10-12 17:03:37.239655
89431	TMF	2020-10-07	37.31	37.95	36.84	37.19	604367	{}	\N	2020-10-12 17:03:37.244622
89432	TMF	2020-10-08	37.61	37.8	37.3984	37.65	632908	{}	\N	2020-10-12 17:03:37.249518
89433	TMF	2020-10-09	37.3	37.8	36.78	37.65	548163	{}	\N	2020-10-12 17:03:37.254129
89434	TMF	2020-10-12	37.93	37.99	37.695	37.79	238765	{}	\N	2020-10-12 17:03:37.258674
89435	TTGT	2020-09-28	42.5	44.47	42.5	44.07	436836	{}	\N	2020-10-12 17:03:38.990136
89436	TTGT	2020-09-29	44.29	44.46	43.43	43.85	383101	{}	\N	2020-10-12 17:03:39.004089
89437	TTGT	2020-09-30	43.96	44.9	43.75	43.96	349926	{}	\N	2020-10-12 17:03:39.011515
89438	TTGT	2020-10-01	44.3	44.83	43.605	44.57	168487	{}	\N	2020-10-12 17:03:39.018184
89439	TTGT	2020-10-02	43.87	44.9839	43.41	44.6	163694	{}	\N	2020-10-12 17:03:39.023711
89440	TTGT	2020-10-05	45	46.42	45	46.23	294269	{}	\N	2020-10-12 17:03:39.029085
89441	TTGT	2020-10-06	46.28	47.7	45.71	46.61	298634	{}	\N	2020-10-12 17:03:39.033658
89442	TTGT	2020-10-07	47.26	48.11	46.8	48.01	268964	{}	\N	2020-10-12 17:03:39.038372
89443	TTGT	2020-10-08	48.67	48.94	47.54	48.15	150791	{}	\N	2020-10-12 17:03:39.042987
89444	TTGT	2020-10-09	48.27	49.67	48.11	49.44	170598	{}	\N	2020-10-12 17:03:39.048069
89445	TTGT	2020-10-12	49.37	49.95	49.17	49.9	50541	{}	\N	2020-10-12 17:03:39.053158
89446	UCIB	2020-09-28	13.58	13.58	13.19	13.2323	1829	{}	\N	2020-10-12 17:03:41.597037
89447	UCIB	2020-09-29	13.12	13.12	13.12	13.12	123	{}	\N	2020-10-12 17:03:41.608277
89448	UCIB	2020-09-30	13.24	13.295	13.24	13.295	2038	{}	\N	2020-10-12 17:03:41.616238
89449	UCIB	2020-10-01	13.075	13.075	13.075	13.075	0	{}	\N	2020-10-12 17:03:41.622191
89450	UCIB	2020-10-02	13.06	13.07	12.985	12.985	2672	{}	\N	2020-10-12 17:03:41.627846
89451	UCIB	2020-10-05	13.2051	13.21	13.18	13.21	7817	{}	\N	2020-10-12 17:03:41.633324
89452	UCIB	2020-10-06	13.3	13.36	13.2256	13.2256	3396	{}	\N	2020-10-12 17:03:41.638883
89453	UCIB	2020-10-07	13.58	13.58	13.33	13.33	610	{}	\N	2020-10-12 17:03:41.64387
89454	UCIB	2020-10-08	13.485	13.485	13.485	13.485	0	{}	\N	2020-10-12 17:03:41.648761
89455	UCIB	2020-10-09	13.56	13.6	13.56	13.6	600	{}	\N	2020-10-12 17:03:41.653956
89456	UEUR	2020-09-28	17.35	17.49	17.35	17.49	5459	{}	\N	2020-10-12 17:03:43.582937
89457	UEUR	2020-09-29	17.7	17.94	17.7	17.9181	9432	{}	\N	2020-10-12 17:03:43.593801
89458	UEUR	2020-09-30	17.71	17.809	17.64	17.79	16305	{}	\N	2020-10-12 17:03:43.600861
89459	UEUR	2020-10-01	17.93	17.9574	17.91	17.9574	2013	{}	\N	2020-10-12 17:03:43.607181
89460	UEUR	2020-10-02	17.7899	17.7899	17.7221	17.75	1539	{}	\N	2020-10-12 17:03:43.614147
89461	UEUR	2020-10-05	18.175	18.2199	18.14	18.18	3751	{}	\N	2020-10-12 17:03:43.619792
89462	UEUR	2020-10-06	18.28	18.28	17.9	17.98	3263	{}	\N	2020-10-12 17:03:43.625361
89463	UEUR	2020-10-07	18.075	18.08	18.0419	18.0419	1464	{}	\N	2020-10-12 17:03:43.632488
89464	UEUR	2020-10-08	17.9963	17.9963	17.9963	17.9963	160	{}	\N	2020-10-12 17:03:43.640098
89465	UEUR	2020-10-09	18.36	18.44	18.33	18.43	10397	{}	\N	2020-10-12 17:03:43.647106
89466	UEUR	2020-10-12	18.355	18.355	18.29	18.29	1244	{}	\N	2020-10-12 17:03:43.653489
89467	UNVR	2020-09-28	16.54	17.24	16.495	17.22	2116048	{}	\N	2020-10-12 17:03:46.103135
89468	UNVR	2020-09-29	17.33	17.33	16.67	16.84	1695814	{}	\N	2020-10-12 17:03:46.115256
89469	UNVR	2020-09-30	16.93	17.1899	16.745	16.88	3076835	{}	\N	2020-10-12 17:03:46.121976
89470	UNVR	2020-10-01	16.97	17.18	16.8	17.1	2802832	{}	\N	2020-10-12 17:03:46.128871
89471	UNVR	2020-10-02	16.62	17.525	16.62	17.48	3150526	{}	\N	2020-10-12 17:03:46.134692
89472	UNVR	2020-10-05	17.6	17.86	17.5	17.64	3477322	{}	\N	2020-10-12 17:03:46.139327
89473	UNVR	2020-10-06	17.95	17.98	17.31	17.35	3568095	{}	\N	2020-10-12 17:03:46.144868
89474	UNVR	2020-10-07	17.63	18.009	17.6	17.73	3016093	{}	\N	2020-10-12 17:03:46.149449
89475	UNVR	2020-10-08	17.99	18.4	17.78	18.4	4639557	{}	\N	2020-10-12 17:03:46.15417
89476	UNVR	2020-10-09	18.45	18.485	17.96	18.09	2203614	{}	\N	2020-10-12 17:03:46.15858
89477	UNVR	2020-10-12	18.26	18.45	18.07	18.32	515329	{}	\N	2020-10-12 17:03:46.163122
89478	USDP	2020-09-28	3.0501	3.24	3.0501	3.225	62634	{}	\N	2020-10-12 17:03:47.111884
89479	USDP	2020-09-29	3.2	3.2	3.09	3.19	194156	{}	\N	2020-10-12 17:03:47.124838
89480	USDP	2020-09-30	3.17	3.415	3.14	3.16	152928	{}	\N	2020-10-12 17:03:47.133075
89481	USDP	2020-10-01	3.16	3.2	3.07	3.14	97871	{}	\N	2020-10-12 17:03:47.139649
89482	USDP	2020-10-02	3	3.285	3	3.27	73906	{}	\N	2020-10-12 17:03:47.145828
89483	USDP	2020-10-05	3.37	3.37	3.2078	3.32	82230	{}	\N	2020-10-12 17:03:47.151329
89484	USDP	2020-10-06	3.3	3.3429	3.25	3.3	93418	{}	\N	2020-10-12 17:03:47.156204
89485	USDP	2020-10-07	3.35	3.35	3.21	3.29	35306	{}	\N	2020-10-12 17:03:47.161031
89486	USDP	2020-10-08	3.27	3.3235	3.22	3.29	94376	{}	\N	2020-10-12 17:03:47.166408
89487	USDP	2020-10-09	3.28	3.35	3.25	3.25	74308	{}	\N	2020-10-12 17:03:47.171428
89488	USDP	2020-10-12	3.25	3.2501	3.21	3.23	15972	{}	\N	2020-10-12 17:03:47.176101
89489	USMV	2020-09-28	63.55	63.97	63.41	63.64	2742581	{}	\N	2020-10-12 17:03:48.129072
89490	USMV	2020-09-29	63.71	63.855	63.26	63.36	3116752	{}	\N	2020-10-12 17:03:48.142362
89491	USMV	2020-09-30	63.6	64.2499	63.35	63.73	3406377	{}	\N	2020-10-12 17:03:48.150922
89492	USMV	2020-10-01	64.19	64.32	63.65	64	3381462	{}	\N	2020-10-12 17:03:48.157253
89493	USMV	2020-10-02	63.11	64.09	63.11	63.75	4339969	{}	\N	2020-10-12 17:03:48.162314
89494	USMV	2020-10-05	64.06	64.62	64.06	64.59	2722011	{}	\N	2020-10-12 17:03:48.167323
89495	USMV	2020-10-06	64.57	65.025	63.84	63.92	3979893	{}	\N	2020-10-12 17:03:48.171786
89496	USMV	2020-10-07	64.42	64.89	64.38	64.69	3536788	{}	\N	2020-10-12 17:03:48.176185
89497	USMV	2020-10-08	64.86	65.33	64.86	65.3	2211409	{}	\N	2020-10-12 17:03:48.181555
89498	USMV	2020-10-09	65.55	65.9086	65.41	65.72	2052025	{}	\N	2020-10-12 17:03:48.187022
89499	USMV	2020-10-12	65.85	66.435	65.83	66.35	1566879	{}	\N	2020-10-12 17:03:48.192013
89500	SHSP	2020-09-28	10.61	10.95	10.5	10.74	33503	{}	\N	2020-10-12 17:03:49.839627
89501	SHSP	2020-09-29	10.5	10.855	9.9304	10.58	66930	{}	\N	2020-10-12 17:03:49.851918
89502	SHSP	2020-09-30	10.64	11.28	10.64	11.15	51225	{}	\N	2020-10-12 17:03:49.858746
89503	SHSP	2020-10-01	11.18	11.75	11	11.6	40663	{}	\N	2020-10-12 17:03:49.864538
89504	SHSP	2020-10-02	11.34	11.79	11.26	11.41	17182	{}	\N	2020-10-12 17:03:49.869867
89505	SHSP	2020-10-05	11.48	11.48	11.23	11.23	16555	{}	\N	2020-10-12 17:03:49.875437
89506	SHSP	2020-10-06	11.4	11.53	10.97	11	31926	{}	\N	2020-10-12 17:03:49.880724
89507	SHSP	2020-10-07	11	11.39	11	11.22	21779	{}	\N	2020-10-12 17:03:49.88581
89508	SHSP	2020-10-08	11.24	11.42	11.24	11.33	14067	{}	\N	2020-10-12 17:03:49.890966
89509	SHSP	2020-10-09	11.41	11.8	10.6	10.6	37766	{}	\N	2020-10-12 17:03:49.896129
89510	SHSP	2020-10-12	10.87	11.35	10.74	10.74	82015	{}	\N	2020-10-12 17:03:49.90228
89511	SIL	2020-09-28	43.25	43.5	42.21	42.84	345212	{}	\N	2020-10-12 17:03:51.27068
89512	SIL	2020-09-29	43.32	43.91	43.005	43.46	353405	{}	\N	2020-10-12 17:03:51.281564
89513	SIL	2020-09-30	43.37	43.841	42.71	43.34	382466	{}	\N	2020-10-12 17:03:51.289738
89514	SIL	2020-10-01	44.11	44.63	43.6083	44.24	212218	{}	\N	2020-10-12 17:03:51.295675
89515	SIL	2020-10-02	44.01	44.26	43.39	43.51	243107	{}	\N	2020-10-12 17:03:51.300836
89516	SIL	2020-10-05	43.99	44.95	43.9302	44.57	315431	{}	\N	2020-10-12 17:03:51.306059
89517	SIL	2020-10-06	44.59	44.7999	42.45	42.56	446271	{}	\N	2020-10-12 17:03:51.310371
89518	SIL	2020-10-07	43.08	43.62	42.7403	43.06	347486	{}	\N	2020-10-12 17:03:51.31544
89519	SIL	2020-10-08	43.54	44.1416	43.33	43.86	313615	{}	\N	2020-10-12 17:03:51.319621
89520	SIL	2020-10-09	45.05	46.33	44.61	46.26	915161	{}	\N	2020-10-12 17:03:51.32368
89521	SIL	2020-10-12	46.37	46.73	45.71	46.63	239490	{}	\N	2020-10-12 17:03:51.328193
89522	SKOR	2020-09-28	54.73	54.86	54.72	54.86	1307	{}	\N	2020-10-12 17:03:52.947773
89523	SKOR	2020-09-29	54.82	54.8837	54.77	54.77	7336	{}	\N	2020-10-12 17:03:52.955969
89524	SKOR	2020-09-30	54.815	54.86	54.77	54.77	11788	{}	\N	2020-10-12 17:03:52.962683
89525	SKOR	2020-10-01	54.65	54.81	54.65	54.75	5112	{}	\N	2020-10-12 17:03:52.967994
89526	SKOR	2020-10-02	54.83	54.8399	54.759	54.83	6651	{}	\N	2020-10-12 17:03:52.973489
89527	SKOR	2020-10-05	54.82	55.04	54.77	54.8057	30532	{}	\N	2020-10-12 17:03:52.979008
89528	SKOR	2020-10-06	54.81	54.82	54.72	54.72	2467	{}	\N	2020-10-12 17:03:52.983731
89529	SKOR	2020-10-07	54.75	54.98	54.68	54.68	9162	{}	\N	2020-10-12 17:03:52.988496
89530	SKOR	2020-10-08	54.86	54.9	54.77	54.89	11446	{}	\N	2020-10-12 17:03:52.994053
89531	SKOR	2020-10-09	54.8	54.9	54.8	54.895	37914	{}	\N	2020-10-12 17:03:52.998625
89532	SMIN	2020-09-28	35.34	35.38	35.225	35.38	6090	{}	\N	2020-10-12 17:03:54.002476
89533	SMIN	2020-09-29	35.28	35.59	35.1094	35.4	45477	{}	\N	2020-10-12 17:03:54.014414
89534	SMIN	2020-09-30	35.53	35.92	35.53	35.83	55512	{}	\N	2020-10-12 17:03:54.02227
89535	SMIN	2020-10-01	35.91	36.08	35.45	36.08	66216	{}	\N	2020-10-12 17:03:54.028767
89536	SMIN	2020-10-02	35.27	35.78	35.27	35.57	18927	{}	\N	2020-10-12 17:03:54.034868
89537	SMIN	2020-10-05	35.89	36.33	35.89	36.21	28867	{}	\N	2020-10-12 17:03:54.040102
89538	SMIN	2020-10-06	36.095	36.33	35.72	35.92	66858	{}	\N	2020-10-12 17:03:54.044915
89539	SMIN	2020-10-07	36.14	36.14	35.915	36.005	11911	{}	\N	2020-10-12 17:03:54.050029
89540	SMIN	2020-10-08	36.04	36.04	35.76	35.99	10557	{}	\N	2020-10-12 17:03:54.054867
89541	SMIN	2020-10-09	35.92	36.22	35.8801	36.11	31361	{}	\N	2020-10-12 17:03:54.059729
89542	SMIN	2020-10-12	35.79	35.8	35.665	35.8	8383	{}	\N	2020-10-12 17:03:54.064262
89543	SOR	2020-09-28	36.41	37.04	36.41	36.5	15863	{}	\N	2020-10-12 17:03:56.214212
89544	SOR	2020-09-29	36.47	36.6614	36.285	36.49	29594	{}	\N	2020-10-12 17:03:56.225422
89545	SOR	2020-09-30	36.42	37.5499	36.42	36.85	46431	{}	\N	2020-10-12 17:03:56.23245
89546	SOR	2020-10-01	37.04	37.3311	36.9	37.11	21758	{}	\N	2020-10-12 17:03:56.239092
89547	SOR	2020-10-02	37.11	37.17	36.6468	37.04	16754	{}	\N	2020-10-12 17:03:56.24468
89548	SOR	2020-10-05	37.12	37.4142	37.11	37.19	4876	{}	\N	2020-10-12 17:03:56.250674
89549	SOR	2020-10-06	37.26	37.41	36.81	36.81	10348	{}	\N	2020-10-12 17:03:56.25486
89550	SOR	2020-10-07	36.95	37.47	36.83	37.2	30006	{}	\N	2020-10-12 17:03:56.2598
89551	SOR	2020-10-08	37.46	37.725	37.46	37.59	8554	{}	\N	2020-10-12 17:03:56.264605
89552	SOR	2020-10-09	37.59	37.99	37.17	37.17	15754	{}	\N	2020-10-12 17:03:56.270014
89553	SOR	2020-10-12	37.44	37.879	37.44	37.879	4524	{}	\N	2020-10-12 17:03:56.274832
89554	SPDV	2020-09-28	21.29	21.59	21.29	21.48	19843	{}	\N	2020-10-12 17:03:57.21075
89555	SPDV	2020-09-29	21.397	21.42	21.189	21.42	4198	{}	\N	2020-10-12 17:03:57.222034
89556	SPDV	2020-09-30	21.6	21.62	21.4515	21.4515	5222	{}	\N	2020-10-12 17:03:57.229613
89557	SPDV	2020-10-01	21.46	21.46	21.1801	21.26	6790	{}	\N	2020-10-12 17:03:57.235769
89558	SPDV	2020-10-02	21.14	21.56	21.14	21.505	2810	{}	\N	2020-10-12 17:03:57.24147
89559	SPDV	2020-10-05	21.58	21.8601	21.58	21.83	6316	{}	\N	2020-10-12 17:03:57.246798
89560	SPDV	2020-10-06	21.89	22.18	21.7233	21.7233	4066	{}	\N	2020-10-12 17:03:57.251534
89561	SPDV	2020-10-07	22.015	22.124	21.94	22.124	2799	{}	\N	2020-10-12 17:03:57.257149
89562	SPDV	2020-10-08	22.325	22.4545	22.29	22.4545	2318	{}	\N	2020-10-12 17:03:57.261909
89563	SPDV	2020-10-09	22.52	22.5299	22.28	22.28	2647	{}	\N	2020-10-12 17:03:57.266851
89564	SPDV	2020-10-12	22.44	22.44	22.26	22.3801	5937	{}	\N	2020-10-12 17:03:57.27183
89565	SPMO	2020-09-28	49.21	49.46	49.12	49.3901	11141	{}	\N	2020-10-12 17:03:58.53098
89566	SPMO	2020-09-29	49.5564	49.78	49.4526	49.78	16826	{}	\N	2020-10-12 17:03:58.543133
89567	SPMO	2020-09-30	49.56	50.2277	49.5224	49.98	7703	{}	\N	2020-10-12 17:03:58.551381
89568	SPMO	2020-10-01	50.41	50.52	50.24	50.5	105491	{}	\N	2020-10-12 17:03:58.558267
89569	SPMO	2020-10-02	49.9052	50.0731	49.31	49.31	5848	{}	\N	2020-10-12 17:03:58.565143
89570	SPMO	2020-10-05	50.0469	50.445	50.0469	50.445	9216	{}	\N	2020-10-12 17:03:58.570348
89571	SPMO	2020-10-06	51	51	49.6109	49.6109	7070	{}	\N	2020-10-12 17:03:58.575543
89572	SPMO	2020-10-07	50.46	50.55	50.135	50.52	10592	{}	\N	2020-10-12 17:03:58.580572
89573	SPMO	2020-10-08	50.61	50.65	50.43	50.6096	11939	{}	\N	2020-10-12 17:03:58.58551
89574	SPMO	2020-10-09	50.83	51.19	50.8201	51.1327	15641	{}	\N	2020-10-12 17:03:58.590386
89575	SPMO	2020-10-12	51.86	52.52	51.69	52.44	10235	{}	\N	2020-10-12 17:03:58.595529
89576	SREV	2020-09-28	1.42	1.52	1.4	1.45	230744	{}	\N	2020-10-12 17:04:00.063096
89577	SREV	2020-09-29	1.44	1.44	1.36	1.38	49657	{}	\N	2020-10-12 17:04:00.074656
89578	SREV	2020-09-30	1.4	1.48	1.37	1.47	133594	{}	\N	2020-10-12 17:04:00.083255
89579	SREV	2020-10-01	1.47	1.5	1.4	1.485	147187	{}	\N	2020-10-12 17:04:00.089605
89580	SREV	2020-10-02	1.45	1.52	1.42	1.44	106505	{}	\N	2020-10-12 17:04:00.095636
89581	SREV	2020-10-05	1.46	1.47	1.385	1.42	161715	{}	\N	2020-10-12 17:04:00.101219
89582	SREV	2020-10-06	1.42	1.48	1.39	1.4	81788	{}	\N	2020-10-12 17:04:00.107092
89583	SREV	2020-10-07	1.44	1.44	1.36	1.41	100695	{}	\N	2020-10-12 17:04:00.113095
89584	SREV	2020-10-08	1.44	1.47	1.4	1.45	90895	{}	\N	2020-10-12 17:04:00.118438
89585	SREV	2020-10-09	1.49	1.51	1.45	1.47	100173	{}	\N	2020-10-12 17:04:00.124283
89586	SREV	2020-10-12	1.44	1.46	1.41	1.41	30828	{}	\N	2020-10-12 17:04:00.12988
89587	SSD	2020-09-28	94.54	96.88	94.54	96.43	177591	{}	\N	2020-10-12 17:04:01.596055
89588	SSD	2020-09-29	96.41	97.96	96.31	97.12	321364	{}	\N	2020-10-12 17:04:01.606795
89589	SSD	2020-09-30	97.23	98.73	96.76	97.16	455847	{}	\N	2020-10-12 17:04:01.616104
89590	SSD	2020-10-01	97.94	98.61	96.46	97.37	402362	{}	\N	2020-10-12 17:04:01.622717
89591	SSD	2020-10-02	92	92	89.8	90.62	2532384	{}	\N	2020-10-12 17:04:01.628885
89592	SSD	2020-10-05	91.58	92.89	91.241	92.2	1168546	{}	\N	2020-10-12 17:04:01.635087
89593	SSD	2020-10-06	93.46	94.14	89.155	90.66	17558849	{}	\N	2020-10-12 17:04:01.641504
89594	SSD	2020-10-07	91.63	94.66	91.63	93.28	1113926	{}	\N	2020-10-12 17:04:01.648618
89595	SSD	2020-10-08	94.52	95.39	93.34	94.57	641615	{}	\N	2020-10-12 17:04:01.656695
89596	SSD	2020-10-09	95.51	95.9	93.7	95.23	381144	{}	\N	2020-10-12 17:04:01.664289
89597	SSD	2020-10-12	95.94	96.515	94.59	94.74	126473	{}	\N	2020-10-12 17:04:01.671151
89598	TXN	2020-09-28	140.22	142	139.72	141.94	3324047	{}	\N	2020-10-12 17:04:02.725882
89599	TXN	2020-09-29	142.29	143.0532	141.22	141.5	2656989	{}	\N	2020-10-12 17:04:02.732472
89600	TXN	2020-09-30	141.16	144.2501	140.515	142.79	5417611	{}	\N	2020-10-12 17:04:02.737479
89601	TXN	2020-10-01	145.44	147.22	143.99	144.91	4246697	{}	\N	2020-10-12 17:04:02.742354
89602	TXN	2020-10-02	142.08	143.93	140.84	141.09	3702536	{}	\N	2020-10-12 17:04:02.747318
89603	TXN	2020-10-05	142.29	144.23	141.57	144.08	2950954	{}	\N	2020-10-12 17:04:02.752037
89604	TXN	2020-10-06	143.85	147.14	142.97	143.85	3278565	{}	\N	2020-10-12 17:04:02.756531
89605	TXN	2020-10-07	145.34	147.29	144.81	146.75	2769579	{}	\N	2020-10-12 17:04:02.760806
89606	TXN	2020-10-08	146.83	148.31	146.56	148.06	2299091	{}	\N	2020-10-12 17:04:02.766195
89607	TXN	2020-10-09	150.08	151.52	148.85	150.91	4132029	{}	\N	2020-10-12 17:04:02.771221
89608	TXN	2020-10-12	152.58	155.24	151.83	154.5725	2647007	{}	\N	2020-10-12 17:04:02.775788
89609	UBG	2020-09-28	49.4	49.4	46.7579	46.7579	626	{}	\N	2020-10-12 17:04:04.459155
89610	UBG	2020-09-29	47.1188	47.1188	47.1188	47.1188	0	{}	\N	2020-10-12 17:04:04.467708
89611	UBG	2020-09-30	47.0741	47.0741	47.0741	47.0741	2	{}	\N	2020-10-12 17:04:04.476808
89612	UBG	2020-10-01	47.4405	47.4405	47.4405	47.4405	1	{}	\N	2020-10-12 17:04:04.485078
89613	UBG	2020-10-02	47.3782	47.3782	47.3782	47.3782	1	{}	\N	2020-10-12 17:04:04.490881
89614	UBG	2020-10-05	47.6143	47.6143	47.6143	47.6143	8	{}	\N	2020-10-12 17:04:04.495745
89615	UBG	2020-10-06	47.3939	47.3939	47.3939	47.3939	192	{}	\N	2020-10-12 17:04:04.500472
89616	UBG	2020-10-07	46.925	46.925	46.925	46.925	13	{}	\N	2020-10-12 17:04:04.504863
89617	UBG	2020-10-08	47.0367	47.0367	47.0367	47.0367	40	{}	\N	2020-10-12 17:04:04.509342
89618	UBG	2020-10-09	47.9182	47.9182	47.9182	47.9182	2	{}	\N	2020-10-12 17:04:04.514046
89619	W	2020-09-28	300.96	307.47	298.3	300.74	1378422	{}	\N	2020-10-12 17:04:06.046453
89620	W	2020-09-29	302.39	304.65	293.01	294.66	2484043	{}	\N	2020-10-12 17:04:06.056133
89621	W	2020-09-30	297.44	307.65	286.61	291.01	4284177	{}	\N	2020-10-12 17:04:06.063237
89622	W	2020-10-01	297	324.21	295.01	317.38	7992984	{}	\N	2020-10-12 17:04:06.070597
89623	W	2020-10-02	308.31	319.532	300.5401	305.36	4598157	{}	\N	2020-10-12 17:04:06.077954
89624	W	2020-10-05	308.86	314.49	306.5301	309.72	2765932	{}	\N	2020-10-12 17:04:06.082678
89625	W	2020-10-06	310.9	311.7	295.02	295.76	3639122	{}	\N	2020-10-12 17:04:06.088562
89626	W	2020-10-07	306.79	310.99	303	308.48	3329256	{}	\N	2020-10-12 17:04:06.093166
89627	W	2020-10-08	308.9	311.59	303.6401	304.07	2082287	{}	\N	2020-10-12 17:04:06.09766
89628	W	2020-10-09	312.96	313.03	291.81	296.19	6031305	{}	\N	2020-10-12 17:04:06.103495
89629	W	2020-10-12	297.97	301.5	293.7	299.185	3388630	{}	\N	2020-10-12 17:04:06.108311
89630	SJB	2020-09-28	20.05	20.095	20	20.05	193186	{}	\N	2020-10-12 17:04:07.309205
89631	SJB	2020-09-29	20.02	20.0893	20	20.01	130435	{}	\N	2020-10-12 17:04:07.31801
89632	SJB	2020-09-30	20.01	20.01	19.91	19.93	58392	{}	\N	2020-10-12 17:04:07.32442
89633	SJB	2020-10-01	19.8799	19.9	19.85	19.86	62764	{}	\N	2020-10-12 17:04:07.330302
89634	SJB	2020-10-02	19.96	19.96	19.85	19.87	44376	{}	\N	2020-10-12 17:04:07.335627
89635	SJB	2020-10-05	19.8	19.8099	19.7	19.72	67214	{}	\N	2020-10-12 17:04:07.341851
89636	SJB	2020-10-06	19.71	19.765	19.645	19.7601	79764	{}	\N	2020-10-12 17:04:07.347776
89637	SJB	2020-10-07	19.69	19.7	19.67	19.68	91318	{}	\N	2020-10-12 17:04:07.353262
89638	SJB	2020-10-08	19.61	19.66	19.61	19.635	45738	{}	\N	2020-10-12 17:04:07.359146
89639	SJB	2020-10-09	19.59	19.63	19.58	19.59	25703	{}	\N	2020-10-12 17:04:07.364486
89640	SJB	2020-10-12	19.54	19.55	19.47	19.48	11236	{}	\N	2020-10-12 17:04:07.369838
89641	SNSR	2020-09-28	25.8	26.05	25.76	26.04	59684	{}	\N	2020-10-12 17:04:09.849103
89642	SNSR	2020-09-29	26.12	26.34	26.03	26.2	57494	{}	\N	2020-10-12 17:04:09.858373
89643	SNSR	2020-09-30	26.23	26.47	26.15	26.2	27305	{}	\N	2020-10-12 17:04:09.864485
89644	SNSR	2020-10-01	26.55	26.79	26.55	26.79	45466	{}	\N	2020-10-12 17:04:09.870331
89645	SNSR	2020-10-02	26.35	26.615	26.2712	26.45	39377	{}	\N	2020-10-12 17:04:09.875782
89646	SNSR	2020-10-05	26.58	27.05	26.58	27.05	63555	{}	\N	2020-10-12 17:04:09.881212
89647	SNSR	2020-10-06	27.13	27.32	26.7839	26.79	47864	{}	\N	2020-10-12 17:04:09.88617
89648	SNSR	2020-10-07	27.15	27.3943	27.15	27.35	37772	{}	\N	2020-10-12 17:04:09.891154
89649	SNSR	2020-10-08	27.54	27.65	27.44	27.64	48307	{}	\N	2020-10-12 17:04:09.895699
89650	SNSR	2020-10-09	27.85	28.085	27.85	27.98	35940	{}	\N	2020-10-12 17:04:09.900559
89651	SNSR	2020-10-12	28.25	28.34	28.1901	28.3	33082	{}	\N	2020-10-12 17:04:09.905367
89652	SOI	2020-09-28	6.28	6.61	6.28	6.46	341100	{}	\N	2020-10-12 17:04:10.896353
89653	SOI	2020-09-29	6.41	6.59	6.3	6.58	849312	{}	\N	2020-10-12 17:04:10.906475
89654	SOI	2020-09-30	6.6	6.67	6.25	6.34	493444	{}	\N	2020-10-12 17:04:10.912456
89655	SOI	2020-10-01	6.26	6.5	6.245	6.49	467281	{}	\N	2020-10-12 17:04:10.917837
89656	SOI	2020-10-02	6.26	6.52	6.22	6.45	404981	{}	\N	2020-10-12 17:04:10.923567
89657	SOI	2020-10-05	6.52	6.72	6.48	6.67	283100	{}	\N	2020-10-12 17:04:10.928891
89658	SOI	2020-10-06	6.75	6.765	6.42	6.44	874645	{}	\N	2020-10-12 17:04:10.934438
89659	SOI	2020-10-07	6.47	6.54	6.37	6.42	302022	{}	\N	2020-10-12 17:04:10.939763
89660	SOI	2020-10-08	6.51	6.68	6.51	6.63	604412	{}	\N	2020-10-12 17:04:10.94528
89661	SOI	2020-10-09	6.63	6.65	6.48	6.49	343351	{}	\N	2020-10-12 17:04:10.95051
89662	SOI	2020-10-12	6.47	6.5	6.35	6.425	107410	{}	\N	2020-10-12 17:04:10.955965
89663	SONA	2020-09-28	8.52	8.78	8.52	8.63	63178	{}	\N	2020-10-12 17:04:13.015684
89664	SONA	2020-09-29	8.6	8.63	8.45	8.57	107976	{}	\N	2020-10-12 17:04:13.02207
89665	SONA	2020-09-30	8.59	8.73	8.57	8.68	60528	{}	\N	2020-10-12 17:04:13.027521
89666	SONA	2020-10-01	8.73	8.83	8.58	8.8	144742	{}	\N	2020-10-12 17:04:13.033243
89667	SONA	2020-10-02	8.71	9.28	8.6633	9.23	93454	{}	\N	2020-10-12 17:04:13.038395
89668	SONA	2020-10-05	8.905	9.47	8.905	9.47	45922	{}	\N	2020-10-12 17:04:13.043285
89669	SONA	2020-10-06	9.6	9.83	9.38	9.5	53218	{}	\N	2020-10-12 17:04:13.0485
89670	SONA	2020-10-07	9.63	9.81	9.63	9.77	42747	{}	\N	2020-10-12 17:04:13.053269
89671	SONA	2020-10-08	9.88	9.99	9.715	9.955	45033	{}	\N	2020-10-12 17:04:13.058521
89672	SONA	2020-10-09	10.05	10.05	9.81	9.91	30327	{}	\N	2020-10-12 17:04:13.063649
89673	SONA	2020-10-12	9.75	9.915	9.75	9.8	10270	{}	\N	2020-10-12 17:04:13.069214
89674	SPAB	2020-09-28	30.77	30.84	30.77	30.83	5155043	{}	\N	2020-10-12 17:04:14.045212
89675	SPAB	2020-09-29	30.85	30.86	30.83	30.85	902365	{}	\N	2020-10-12 17:04:14.052373
89676	SPAB	2020-09-30	30.85	30.85	30.76	30.81	2904592	{}	\N	2020-10-12 17:04:14.058605
89677	SPAB	2020-10-01	30.74	30.78	30.7	30.78	3839768	{}	\N	2020-10-12 17:04:14.064533
89678	SPAB	2020-10-02	30.76	30.7757	30.7261	30.75	1949221	{}	\N	2020-10-12 17:04:14.069824
89679	SPAB	2020-10-05	30.69	30.6959	30.65	30.6573	1508895	{}	\N	2020-10-12 17:04:14.074641
89680	SPAB	2020-10-06	30.65	30.72	30.63	30.67	1609756	{}	\N	2020-10-12 17:04:14.079989
89681	SPAB	2020-10-07	30.71	30.71	30.61	30.65	1402977	{}	\N	2020-10-12 17:04:14.084904
89682	SPAB	2020-10-08	30.68	30.7	30.65	30.7	760066	{}	\N	2020-10-12 17:04:14.089901
89683	SPAB	2020-10-09	30.65	30.7	30.64	30.7	2195725	{}	\N	2020-10-12 17:04:14.094884
89684	SPAB	2020-10-12	30.73	30.735	30.69	30.735	325037	{}	\N	2020-10-12 17:04:14.099829
89685	SQM	2020-09-28	30.72	31.285	30.43	30.9	1644077	{}	\N	2020-10-12 17:04:15.630106
89686	SQM	2020-09-29	30.83	31.575	30.82	31.35	1874587	{}	\N	2020-10-12 17:04:15.642454
89687	SQM	2020-09-30	31.35	32.48	31.35	32.42	2061219	{}	\N	2020-10-12 17:04:15.651126
89688	SQM	2020-10-01	33.11	33.11	31.46	31.57	3043218	{}	\N	2020-10-12 17:04:15.657874
89689	SQM	2020-10-02	30.93	33.1	30.77	32.51	1386425	{}	\N	2020-10-12 17:04:15.664633
89690	SQM	2020-10-05	33.5	35.5421	33.38	35.46	3992053	{}	\N	2020-10-12 17:04:15.669869
89691	SQM	2020-10-06	35.58	35.6689	34.44	34.56	2718227	{}	\N	2020-10-12 17:04:15.675063
89692	SQM	2020-10-07	35.07	36.14	34.89	35.42	3676307	{}	\N	2020-10-12 17:04:15.680594
89693	SQM	2020-10-08	35.22	36.41	35.13	35.43	3442984	{}	\N	2020-10-12 17:04:15.685595
89694	SQM	2020-10-09	35.81	36.3599	35.3501	35.54	1664259	{}	\N	2020-10-12 17:04:15.691413
89695	SQM	2020-10-12	35.91	36.1095	35.18	35.52	552549	{}	\N	2020-10-12 17:04:15.697159
89696	SRLN	2020-09-28	44.43	44.63	44.3501	44.6	848913	{}	\N	2020-10-12 17:04:17.651714
89697	SRLN	2020-09-29	44.64	44.68	44.48	44.65	390733	{}	\N	2020-10-12 17:04:17.666725
89698	SRLN	2020-09-30	44.68	44.88	44.67	44.72	470961	{}	\N	2020-10-12 17:04:17.676466
89699	SRLN	2020-10-01	44.44	44.7	44.44	44.62	332376	{}	\N	2020-10-12 17:04:17.682582
89700	SRLN	2020-10-02	44.54	44.54	44.4401	44.54	505179	{}	\N	2020-10-12 17:04:17.688383
89701	SRLN	2020-10-05	44.58	44.7	44.52	44.7	1046664	{}	\N	2020-10-12 17:04:17.693139
89702	SRLN	2020-10-06	44.69	44.7	44.29	44.54	886131	{}	\N	2020-10-12 17:04:17.697475
89703	SRLN	2020-10-07	44.6	44.74	44.39	44.67	382199	{}	\N	2020-10-12 17:04:17.702321
89704	SRLN	2020-10-08	44.75	44.75	44.65	44.7	275966	{}	\N	2020-10-12 17:04:17.706746
89705	SRLN	2020-10-09	44.66	44.78	44.65	44.69	2003401	{}	\N	2020-10-12 17:04:17.711256
89706	SRLN	2020-10-12	44.79	45.0299	44.69	44.81	204007	{}	\N	2020-10-12 17:04:17.715725
89707	STIP	2020-09-28	103.94	104.07	103.94	104.04	131747	{}	\N	2020-10-12 17:04:19.080683
89708	STIP	2020-09-29	104.1	104.1399	104.0611	104.125	185637	{}	\N	2020-10-12 17:04:19.092817
89709	STIP	2020-09-30	104.14	104.16	104.06	104.06	95593	{}	\N	2020-10-12 17:04:19.10106
89710	STIP	2020-10-01	103.54	103.62	103.5347	103.57	114257	{}	\N	2020-10-12 17:04:19.107424
89711	STIP	2020-10-02	103.53	103.57	103.51	103.53	123977	{}	\N	2020-10-12 17:04:19.113202
89712	STIP	2020-10-05	103.61	103.61	103.55	103.56	129044	{}	\N	2020-10-12 17:04:19.118642
89713	STIP	2020-10-06	103.62	103.64	103.56	103.58	243696	{}	\N	2020-10-12 17:04:19.124174
89714	STIP	2020-10-07	103.63	103.65	103.58	103.65	125929	{}	\N	2020-10-12 17:04:19.12948
89715	STIP	2020-10-08	103.74	103.78	103.6906	103.76	176559	{}	\N	2020-10-12 17:04:19.134575
89716	STIP	2020-10-09	103.8	103.81	103.76	103.79	85297	{}	\N	2020-10-12 17:04:19.139669
89717	STIP	2020-10-12	103.79	103.79	103.75	103.78	31708	{}	\N	2020-10-12 17:04:19.145064
89718	TATT	2020-09-28	5.22	5.74	4.86	5.6	629646	{}	\N	2020-10-12 17:04:21.912333
89719	TATT	2020-09-29	5.34	5.4	5.12	5.29	217526	{}	\N	2020-10-12 17:04:21.923758
89720	TATT	2020-09-30	5.15	5.27	4.92	4.93	134121	{}	\N	2020-10-12 17:04:21.931743
89721	TATT	2020-10-01	4.9	5.0172	4.59	4.5902	97531	{}	\N	2020-10-12 17:04:21.938408
89722	TATT	2020-10-02	4.61	4.8257	4.54	4.6	33884	{}	\N	2020-10-12 17:04:21.944051
89723	TATT	2020-10-05	4.99	4.99	4.5	4.5301	36252	{}	\N	2020-10-12 17:04:21.948999
89724	TATT	2020-10-06	4.54	4.72	4.458	4.475	35105	{}	\N	2020-10-12 17:04:21.954195
89725	TATT	2020-10-07	4.545	4.59	4.2813	4.42	28657	{}	\N	2020-10-12 17:04:21.958955
89726	TATT	2020-10-08	4.438	4.75	4.29	4.64	62214	{}	\N	2020-10-12 17:04:21.963749
89727	TATT	2020-10-09	4.56	4.73	4.4701	4.49	17937	{}	\N	2020-10-12 17:04:21.968461
89728	TATT	2020-10-12	4.5	4.5	4.41	4.45	6283	{}	\N	2020-10-12 17:04:21.973097
89729	TLYS	2020-09-28	6.3	6.56	6.3	6.42	93706	{}	\N	2020-10-12 17:04:23.526673
89730	TLYS	2020-09-29	6.33	6.34	6.04	6.09	490110	{}	\N	2020-10-12 17:04:23.532808
89731	TLYS	2020-09-30	6.12	6.38	5.98	6.03	515568	{}	\N	2020-10-12 17:04:23.538569
89732	TLYS	2020-10-01	6.04	6.2299	5.96	5.99	617100	{}	\N	2020-10-12 17:04:23.543779
89733	TLYS	2020-10-02	5.9	6.33	5.83	6.21	1049853	{}	\N	2020-10-12 17:04:23.549214
89734	TLYS	2020-10-05	6.27	6.54	6.26	6.46	443230	{}	\N	2020-10-12 17:04:23.55438
89735	TLYS	2020-10-06	6.55	6.59	6.21	6.24	377892	{}	\N	2020-10-12 17:04:23.559337
89736	TLYS	2020-10-07	6.37	6.49	6.3	6.37	282033	{}	\N	2020-10-12 17:04:23.564195
89737	TLYS	2020-10-08	6.65	6.74	6.4201	6.66	665061	{}	\N	2020-10-12 17:04:23.568971
89738	TLYS	2020-10-09	6.83	6.83	6.49	6.7	598398	{}	\N	2020-10-12 17:04:23.573573
89739	TLYS	2020-10-12	6.85	6.99	6.763	6.84	264351	{}	\N	2020-10-12 17:04:23.578791
89740	TMV	2020-09-28	49.28	49.7	49.28	49.46	118933	{}	\N	2020-10-12 17:04:24.605828
89741	TMV	2020-09-29	49.31	49.54	48.97	49.35	147992	{}	\N	2020-10-12 17:04:24.615953
89742	TMV	2020-09-30	50.06	51.36	50.06	50.66	164372	{}	\N	2020-10-12 17:04:24.623625
89743	TMV	2020-10-01	51.23	51.64	50.08	50.08	125541	{}	\N	2020-10-12 17:04:24.629531
89744	TMV	2020-10-02	50.16	51.53	50.15	51.02	295589	{}	\N	2020-10-12 17:04:24.635073
89745	TMV	2020-10-05	52.7	54.1	52.7	54.1	158624	{}	\N	2020-10-12 17:04:24.640281
89746	TMV	2020-10-06	54.31	54.89	52.18	53.1	454220	{}	\N	2020-10-12 17:04:24.645467
89747	TMV	2020-10-07	53.99	54.67	53.2	54.31	89809	{}	\N	2020-10-12 17:04:24.650383
89748	TMV	2020-10-08	53.6	53.89	53.3	53.38	137317	{}	\N	2020-10-12 17:04:24.655607
89749	TMV	2020-10-09	54.05	54.72	53.32	53.46	167817	{}	\N	2020-10-12 17:04:24.660664
89750	TMV	2020-10-12	53.13	53.43	53	53.32	56892	{}	\N	2020-10-12 17:04:24.665711
89751	USL	2020-09-28	15.21	15.28	15.001	15.16	36726	{}	\N	2020-10-12 17:04:25.613406
89752	USL	2020-09-29	15.03	15.03	14.535	14.69	124311	{}	\N	2020-10-12 17:04:25.626624
89753	USL	2020-09-30	14.69	15.13	14.69	15.05	57482	{}	\N	2020-10-12 17:04:25.633996
89754	USL	2020-10-01	14.7	14.8056	14.29	14.62	182166	{}	\N	2020-10-12 17:04:25.640574
89755	USL	2020-10-02	14	14.327	13.95	14.06	302470	{}	\N	2020-10-12 17:04:25.646277
89756	USL	2020-10-05	14.65	14.96	14.6	14.85	236496	{}	\N	2020-10-12 17:04:25.651501
89757	USL	2020-10-06	15.14	15.3101	15.03	15.05	180549	{}	\N	2020-10-12 17:04:25.656404
89758	USL	2020-10-07	14.93	15.05	14.8342	15.03	61967	{}	\N	2020-10-12 17:04:25.661684
89759	USL	2020-10-08	15.28	15.42	15.25	15.37	137192	{}	\N	2020-10-12 17:04:25.666742
89760	USL	2020-10-09	15.26	15.43	15.15	15.18	86232	{}	\N	2020-10-12 17:04:25.67192
89761	USL	2020-10-12	15.04	15.04	14.7244	14.8669	45830	{}	\N	2020-10-12 17:04:25.677182
89762	SPHQ	2020-09-28	38.38	38.535	38.26	38.48	195728	{}	\N	2020-10-12 17:04:27.114678
89763	SPHQ	2020-09-29	38.45	38.58	38.25	38.32	664261	{}	\N	2020-10-12 17:04:27.127138
89764	SPHQ	2020-09-30	38.42	39.04	38.42	38.64	1060711	{}	\N	2020-10-12 17:04:27.134547
89765	SPHQ	2020-10-01	38.97	39.08	38.63	38.78	586229	{}	\N	2020-10-12 17:04:27.141106
89766	SPHQ	2020-10-02	38.12	38.56	38.1	38.26	513332	{}	\N	2020-10-12 17:04:27.147274
89767	SPHQ	2020-10-05	38.52	38.99	38.52	38.99	264528	{}	\N	2020-10-12 17:04:27.152142
89768	SPHQ	2020-10-06	39.04	39.3874	38.54	38.61	321203	{}	\N	2020-10-12 17:04:27.156996
89769	SPHQ	2020-10-07	38.92	39.39	38.92	39.28	552936	{}	\N	2020-10-12 17:04:27.16207
89770	SPHQ	2020-10-08	39.53	39.537	39.36	39.49	388517	{}	\N	2020-10-12 17:04:27.166759
89771	SPHQ	2020-10-09	39.66	39.91	39.65	39.84	192692	{}	\N	2020-10-12 17:04:27.171392
89772	SPHQ	2020-10-12	40.14	40.5133	40.1	40.4732	114942	{}	\N	2020-10-12 17:04:27.176203
89773	SQBG	2020-09-28	4.6	4.75	4.44	4.45	46371	{}	\N	2020-10-12 17:04:28.523079
89774	SQBG	2020-09-29	4.53	4.927	4.52	4.8499	45806	{}	\N	2020-10-12 17:04:28.532913
89775	SQBG	2020-09-30	4.71	4.8999	4.57	4.58	45179	{}	\N	2020-10-12 17:04:28.540112
89776	SQBG	2020-10-01	4.6341	4.97	4.57	4.72	75137	{}	\N	2020-10-12 17:04:28.546947
89777	SQBG	2020-10-02	4.73	5.2	4.5447	4.94	218463	{}	\N	2020-10-12 17:04:28.552902
89778	SQBG	2020-10-05	4.92	5.01	4.6701	4.77	28877	{}	\N	2020-10-12 17:04:28.557978
89779	SQBG	2020-10-06	4.8297	4.945	4.69	4.74	19671	{}	\N	2020-10-12 17:04:28.563044
89780	SQBG	2020-10-07	4.89	4.895	4.61	4.64	32191	{}	\N	2020-10-12 17:04:28.568218
89781	SQBG	2020-10-08	4.79	5.15	4.7568	4.96	77125	{}	\N	2020-10-12 17:04:28.573849
89782	SQBG	2020-10-09	5.03	5.05	4.85	5.03	40915	{}	\N	2020-10-12 17:04:28.579223
89783	SQBG	2020-10-12	4.8671	5.92	4.8671	5.23	555092	{}	\N	2020-10-12 17:04:28.584098
89784	SRDX	2020-09-28	38.81	39.95	38.81	38.97	53331	{}	\N	2020-10-12 17:04:29.627248
89785	SRDX	2020-09-29	39.03	39.38	38.72	39.11	39210	{}	\N	2020-10-12 17:04:29.640902
89786	SRDX	2020-09-30	39.25	39.84	38.67	38.91	61090	{}	\N	2020-10-12 17:04:29.649031
89787	SRDX	2020-10-01	38.97	39.32	38.33	38.9	48447	{}	\N	2020-10-12 17:04:29.6562
89788	SRDX	2020-10-02	38.3	40.6	38.275	39.98	91554	{}	\N	2020-10-12 17:04:29.662407
89789	SRDX	2020-10-05	40.16	40.61	39.12	39.39	44451	{}	\N	2020-10-12 17:04:29.667757
89790	SRDX	2020-10-06	39.72	40.5	38.97	39.46	55885	{}	\N	2020-10-12 17:04:29.67369
89791	SRDX	2020-10-07	39.68	40.5	39.515	40.02	74684	{}	\N	2020-10-12 17:04:29.678687
89792	SRDX	2020-10-08	40.02	41.314	40.02	41.12	45470	{}	\N	2020-10-12 17:04:29.683745
89793	SRDX	2020-10-09	41.14	42.57	41.14	42.39	40091	{}	\N	2020-10-12 17:04:29.688762
89794	SRDX	2020-10-12	42.2	42.92	42.2	42.92	4843	{}	\N	2020-10-12 17:04:29.693833
89795	SWX	2020-09-28	63.02	63.145	62.24	62.44	276163	{}	\N	2020-10-12 17:04:31.720084
89796	SWX	2020-09-29	62.33	62.525	61.17	62.08	320562	{}	\N	2020-10-12 17:04:31.731597
89797	SWX	2020-09-30	62.3	63.28	62.3	63.1	542617	{}	\N	2020-10-12 17:04:31.738777
89798	SWX	2020-10-01	62.88	63.325	62.05	62.85	444115	{}	\N	2020-10-12 17:04:31.74478
89799	SWX	2020-10-02	62.45	64.35	62.2	64	390048	{}	\N	2020-10-12 17:04:31.750572
89800	SWX	2020-10-05	64.14	64.98	63.32	64.8	497354	{}	\N	2020-10-12 17:04:31.755242
89801	SWX	2020-10-06	65.28	66.75	64.86	65.86	470293	{}	\N	2020-10-12 17:04:31.759685
89802	SWX	2020-10-07	65.78	66.41	65.32	66.07	421150	{}	\N	2020-10-12 17:04:31.764645
89803	SWX	2020-10-08	66.77	67.83	66.44	67.66	353258	{}	\N	2020-10-12 17:04:31.769167
89804	SWX	2020-10-09	67.4	67.7	66.48	67.06	384068	{}	\N	2020-10-12 17:04:31.774149
89805	SWX	2020-10-12	66.68	68.145	66.63	67.65	121191	{}	\N	2020-10-12 17:04:31.778709
89806	TQQQ	2020-09-28	129.27	130	125.57	129.72	33165174	{}	\N	2020-10-12 17:04:34.362781
89807	TQQQ	2020-09-29	129.48	130.8389	127.53	128.36	27268216	{}	\N	2020-10-12 17:04:34.374424
89808	TQQQ	2020-09-30	128.59	135.52	128.25	130.9	41551904	{}	\N	2020-10-12 17:04:34.382876
89809	TQQQ	2020-10-01	136.57	138.07	133.78	137.03	33548455	{}	\N	2020-10-12 17:04:34.389048
89810	TQQQ	2020-10-02	128.12	133.63	124.33	125.73	48039770	{}	\N	2020-10-12 17:04:34.394888
89811	TQQQ	2020-10-05	128.47	134.1	128.25	133.66	25910594	{}	\N	2020-10-12 17:04:34.400405
89812	TQQQ	2020-10-06	132.39	135.12	125.15	126.48	41930724	{}	\N	2020-10-12 17:04:34.405855
89813	TQQQ	2020-10-07	130.17	134.11	129.38	133.13	27620222	{}	\N	2020-10-12 17:04:34.411717
89814	TQQQ	2020-10-08	136.4	136.6067	133.54	135.09	24686087	{}	\N	2020-10-12 17:04:34.416863
89815	TQQQ	2020-10-09	137.54	141.385	136.96	141.1	23892526	{}	\N	2020-10-12 17:04:34.422267
89816	TQQQ	2020-10-12	148.4	156.37	146.86	155.23	19208012	{}	\N	2020-10-12 17:04:34.427674
89817	USHY	2020-09-28	39.27	39.37	39.2	39.3	1222283	{}	\N	2020-10-12 17:04:36.988284
89818	USHY	2020-09-29	39.35	39.36	39.185	39.32	1140966	{}	\N	2020-10-12 17:04:37.000141
89819	USHY	2020-09-30	39.36	39.53	39.34	39.47	696198	{}	\N	2020-10-12 17:04:37.009254
89820	USHY	2020-10-01	39.4	39.438	39.32	39.41	2840212	{}	\N	2020-10-12 17:04:37.015585
89821	USHY	2020-10-02	39.24	39.42	39.2	39.38	765838	{}	\N	2020-10-12 17:04:37.022093
89822	USHY	2020-10-05	39.5	39.625	39.5	39.6	2423284	{}	\N	2020-10-12 17:04:37.02718
89823	USHY	2020-10-06	39.68	39.76	39.49	39.5	11476208	{}	\N	2020-10-12 17:04:37.032326
89824	USHY	2020-10-07	39.69	39.69	39.6	39.66	1249284	{}	\N	2020-10-12 17:04:37.037593
89825	USHY	2020-10-08	39.81	39.81	39.72	39.79	851168	{}	\N	2020-10-12 17:04:37.042613
89826	USHY	2020-10-09	39.82	39.875	39.76	39.87	960638	{}	\N	2020-10-12 17:04:37.047863
89827	USHY	2020-10-12	40.01	40.11	39.96	40.08	791609	{}	\N	2020-10-12 17:04:37.053114
89828	USO	2020-09-28	28.5	28.86	28.3	28.69	2453158	{}	\N	2020-10-12 17:04:38.432899
89829	USO	2020-09-29	28.31	28.32	27.28	27.7	5378937	{}	\N	2020-10-12 17:04:38.443406
89830	USO	2020-09-30	27.94	28.5617	27.89	28.29	3458349	{}	\N	2020-10-12 17:04:38.450326
89831	USO	2020-10-01	27.76	27.86	26.765	27.51	6561607	{}	\N	2020-10-12 17:04:38.456214
89832	USO	2020-10-02	26.25	26.89	26.09	26.35	8595444	{}	\N	2020-10-12 17:04:38.461393
89833	USO	2020-10-05	27.58	28.16	27.44	27.93	7955193	{}	\N	2020-10-12 17:04:38.465967
89834	USO	2020-10-06	28.66	28.91	28.31	28.39	6353657	{}	\N	2020-10-12 17:04:38.470644
89835	USO	2020-10-07	28.16	28.41	27.89	28.39	5542770	{}	\N	2020-10-12 17:04:38.4748
89836	USO	2020-10-08	28.99	29.2	28.75	29.14	6617034	{}	\N	2020-10-12 17:04:38.47902
89837	USO	2020-10-09	28.99	29.25	28.59	28.69	4637521	{}	\N	2020-10-12 17:04:38.483061
89838	USO	2020-10-12	28.4	28.4	27.73	27.8764	2896228	{}	\N	2020-10-12 17:04:38.487187
89839	VAW	2020-09-28	133.79	135.51	133.649	134.31	50726	{}	\N	2020-10-12 17:04:41.283288
89840	VAW	2020-09-29	134.46	134.8725	133.188	133.46	33948	{}	\N	2020-10-12 17:04:41.295552
89841	VAW	2020-09-30	133.89	135.76	133.565	134.61	140260	{}	\N	2020-10-12 17:04:41.303343
89842	VAW	2020-10-01	135.04	135.87	132.76	133.26	324304	{}	\N	2020-10-12 17:04:41.309795
89843	VAW	2020-10-02	131.38	135.42	131.315	134.52	167958	{}	\N	2020-10-12 17:04:41.315559
89844	VAW	2020-10-05	135.85	138.1645	135.7601	137.35	90451	{}	\N	2020-10-12 17:04:41.320972
89845	VAW	2020-10-06	137.97	139.37	135.57	135.68	181922	{}	\N	2020-10-12 17:04:41.325862
89846	VAW	2020-10-07	137.54	139.63	137.54	139.34	543587	{}	\N	2020-10-12 17:04:41.330787
89847	VAW	2020-10-08	140.39	141	140.19	140.91	53856	{}	\N	2020-10-12 17:04:41.33575
89848	VAW	2020-10-09	141.93	142.5	140.8712	141.75	69980	{}	\N	2020-10-12 17:04:41.341337
89849	VAW	2020-10-12	142.5	142.69	141.58	142.45	50652	{}	\N	2020-10-12 17:04:41.346226
89850	VIRT	2020-09-28	23.14	23.7	22.84	23.36	1664607	{}	\N	2020-10-12 17:04:42.436285
89851	VIRT	2020-09-29	23.36	23.4617	22.825	23.07	1178759	{}	\N	2020-10-12 17:04:42.448764
89852	VIRT	2020-09-30	23.16	23.43	22.9	23.01	2542791	{}	\N	2020-10-12 17:04:42.456244
89853	VIRT	2020-10-01	23.01	23.39	22.77	23.02	947195	{}	\N	2020-10-12 17:04:42.462408
89854	VIRT	2020-10-02	22.9	23.55	22.75	23.46	898319	{}	\N	2020-10-12 17:04:42.468114
89855	VIRT	2020-10-05	23.6	23.76	23.32	23.56	1037831	{}	\N	2020-10-12 17:04:42.47303
89856	VIRT	2020-10-06	23.57	23.89	23	23.44	875537	{}	\N	2020-10-12 17:04:42.477875
89857	VIRT	2020-10-07	23.41	23.8	22.98	23.035	1213161	{}	\N	2020-10-12 17:04:42.482893
89858	VIRT	2020-10-08	22.99	23.15	22.43	22.49	962312	{}	\N	2020-10-12 17:04:42.487627
89859	VIRT	2020-10-09	22.49	22.65	22.15	22.25	853275	{}	\N	2020-10-12 17:04:42.492278
89860	VIRT	2020-10-12	22.22	22.505	22.18	22.35	385955	{}	\N	2020-10-12 17:04:42.497199
89861	SOL	2020-09-28	2.16	2.25	2.08	2.21	1576952	{}	\N	2020-10-12 17:04:44.857113
89862	SOL	2020-09-29	2.17	2.185	2.05	2.1	1698058	{}	\N	2020-10-12 17:04:44.869692
89863	SOL	2020-09-30	2.11	2.16	2.05	2.07	2034967	{}	\N	2020-10-12 17:04:44.878086
89864	SOL	2020-10-01	2.09	2.23	2.06	2.16	3379075	{}	\N	2020-10-12 17:04:44.88434
89865	SOL	2020-10-02	2.1	2.14	2.05	2.07	2010688	{}	\N	2020-10-12 17:04:44.889722
89866	SOL	2020-10-05	2.07	2.1799	2	2.13	2037732	{}	\N	2020-10-12 17:04:44.895076
89867	SOL	2020-10-06	2.15	2.96	2.15	2.64	12748258	{}	\N	2020-10-12 17:04:44.899478
89868	SOL	2020-10-07	2.96	3.64	2.83	3.5	27026524	{}	\N	2020-10-12 17:04:44.903893
89869	SOL	2020-10-08	3.44	3.45	3	3.02	6382277	{}	\N	2020-10-12 17:04:44.90839
89870	SOL	2020-10-09	3.1	4.33	3.1	4.15	17570700	{}	\N	2020-10-12 17:04:44.912965
89871	SOL	2020-10-12	3.56	3.8	3.28	3.39	11702871	{}	\N	2020-10-12 17:04:44.917457
89872	SOXL	2020-09-28	240.62	250.99	236.43	250.76	944387	{}	\N	2020-10-12 17:04:45.876396
89873	SOXL	2020-09-29	249.12	259.08	249.12	252.85	1263811	{}	\N	2020-10-12 17:04:45.887096
89874	SOXL	2020-09-30	250.51	260.98	249	253.19	1249958	{}	\N	2020-10-12 17:04:45.893686
89875	SOXL	2020-10-01	262.905	271.18	260.11	268.91	1139636	{}	\N	2020-10-12 17:04:45.900667
89876	SOXL	2020-10-02	250.07	261.78	244.26	244.94	1504955	{}	\N	2020-10-12 17:04:45.90937
89877	SOXL	2020-10-05	251.86	268.88	251.53	268.85	956474	{}	\N	2020-10-12 17:04:45.913834
89878	SOXL	2020-10-06	267.94	283.3416	261.01	264.26	1016925	{}	\N	2020-10-12 17:04:45.917701
89879	SOXL	2020-10-07	275.56	281.6799	275.26	280.2	853028	{}	\N	2020-10-12 17:04:45.92199
89880	SOXL	2020-10-08	287.55	291.488	284.29	289.93	717928	{}	\N	2020-10-12 17:04:45.926909
89881	SOXL	2020-10-09	300.29	307.44	298.85	305.31	1170314	{}	\N	2020-10-12 17:04:45.931817
89882	SOXL	2020-10-12	316.34	321.34	312.02	318.215	711152	{}	\N	2020-10-12 17:04:45.936269
89883	SPHB	2020-09-28	41.52	42.25	41.5	41.96	366935	{}	\N	2020-10-12 17:04:46.894917
89884	SPHB	2020-09-29	41.89	41.89	40.9054	41.27	93119	{}	\N	2020-10-12 17:04:46.906319
89885	SPHB	2020-09-30	41.44	42.24	41.21	41.54	202258	{}	\N	2020-10-12 17:04:46.913584
89886	SPHB	2020-10-01	41.76	41.95	41.22	41.62	145400	{}	\N	2020-10-12 17:04:46.919856
89887	SPHB	2020-10-02	40.61	42.45	40.47	42.23	248296	{}	\N	2020-10-12 17:04:46.925256
89888	SPHB	2020-10-05	42.74	43.36	42.74	43.31	1896732	{}	\N	2020-10-12 17:04:46.929965
89889	SPHB	2020-10-06	43.73	44.2114	42.57	42.66	322813	{}	\N	2020-10-12 17:04:46.934206
89890	SPHB	2020-10-07	43.35	43.94	43.335	43.79	162814	{}	\N	2020-10-12 17:04:46.938664
89891	SPHB	2020-10-08	44.25	44.76	43.97	44.76	340846	{}	\N	2020-10-12 17:04:46.943108
89892	SPHB	2020-10-09	45.08	45.23	44.3801	44.53	108377	{}	\N	2020-10-12 17:04:46.9475
89893	SPHB	2020-10-12	44.65	44.66	44.34	44.52	55704	{}	\N	2020-10-12 17:04:46.952169
89894	SRE	2020-09-28	117.9	118.72	116.575	116.71	1313107	{}	\N	2020-10-12 17:04:48.294043
89895	SRE	2020-09-29	116.98	117.99	115.65	116.18	1861550	{}	\N	2020-10-12 17:04:48.306851
89896	SRE	2020-09-30	116.96	118.85	116.67	118.36	3147978	{}	\N	2020-10-12 17:04:48.314823
89897	SRE	2020-10-01	119.38	120.25	118.28	119.13	2586620	{}	\N	2020-10-12 17:04:48.321206
89898	SRE	2020-10-02	118.06	121.335	117.7	120.5	2418488	{}	\N	2020-10-12 17:04:48.326865
89899	SRE	2020-10-05	121.42	121.86	119.685	121.77	1883469	{}	\N	2020-10-12 17:04:48.332685
89900	SRE	2020-10-06	122.1	124.86	121.25	122.23	2200876	{}	\N	2020-10-12 17:04:48.337346
89901	SRE	2020-10-07	122.39	123.5	121.035	122.72	2140920	{}	\N	2020-10-12 17:04:48.34227
89902	SRE	2020-10-08	123.49	127.21	123.175	126.71	2046970	{}	\N	2020-10-12 17:04:48.347051
89903	SRE	2020-10-09	127.51	127.51	124.69	126.01	1781054	{}	\N	2020-10-12 17:04:48.352097
89904	SRE	2020-10-12	125.67	127.26	124.98	127	701168	{}	\N	2020-10-12 17:04:48.357043
89905	TAGS	2020-09-28	18.13	18.13	18.13	18.13	19	{}	\N	2020-10-12 17:04:51.004533
89906	TAGS	2020-09-29	17.58	17.98	17.58	17.98	577	{}	\N	2020-10-12 17:04:51.013519
89907	TAGS	2020-09-30	18	18.4077	18	18.4077	882	{}	\N	2020-10-12 17:04:51.019917
89908	TAGS	2020-10-01	18.63	18.63	18.63	18.63	35	{}	\N	2020-10-12 17:04:51.025239
89909	TAGS	2020-10-02	18.16	18.435	18.16	18.435	892	{}	\N	2020-10-12 17:04:51.030878
89910	TAGS	2020-10-05	18.5	18.7298	18.5	18.7298	107	{}	\N	2020-10-12 17:04:51.036225
89911	TAGS	2020-10-06	18.715	18.715	18.715	18.715	37	{}	\N	2020-10-12 17:04:51.041621
89912	TAGS	2020-10-07	18.8379	18.9	18.8379	18.9	141	{}	\N	2020-10-12 17:04:51.046865
89913	TAGS	2020-10-08	18.7	18.7	18.7	18.7	64	{}	\N	2020-10-12 17:04:51.052419
89914	TAGS	2020-10-09	18.715	18.9	18.715	18.9	489	{}	\N	2020-10-12 17:04:51.057895
89915	TECS	2020-09-28	13.42	13.845	13.37	13.39	3197215	{}	\N	2020-10-12 17:04:51.991823
89916	TECS	2020-09-29	13.46	13.59	13.26	13.51	1786234	{}	\N	2020-10-12 17:04:52.003615
89917	TECS	2020-09-30	13.51	13.54	12.755	13.2	2667104	{}	\N	2020-10-12 17:04:52.011224
89918	TECS	2020-10-01	12.66	13	12.575	12.76	2518693	{}	\N	2020-10-12 17:04:52.017398
89919	TECS	2020-10-02	13.71	13.875	13.0701	13.72	4622634	{}	\N	2020-10-12 17:04:52.022657
89920	TECS	2020-10-05	13.41	13.45	12.81	12.82	2574786	{}	\N	2020-10-12 17:04:52.027882
89921	TECS	2020-10-06	12.92	13.54	12.55	13.4	3557702	{}	\N	2020-10-12 17:04:52.032917
89922	TECS	2020-10-07	13	13.02	12.59	12.68	2220700	{}	\N	2020-10-12 17:04:52.037948
89923	TECS	2020-10-08	12.35	12.6	12.34	12.49	1683304	{}	\N	2020-10-12 17:04:52.043777
89924	TECS	2020-10-09	12.27	12.3	11.8907	11.91	2143368	{}	\N	2020-10-12 17:04:52.048714
89925	TECS	2020-10-12	11.36	11.56	10.82	10.89	1589484	{}	\N	2020-10-12 17:04:52.053512
89926	TILT	2020-09-28	121.32	122.3807	121.32	121.99	11713	{}	\N	2020-10-12 17:04:53.943756
89927	TILT	2020-09-29	122.11	122.11	120.73	121.23	9045	{}	\N	2020-10-12 17:04:53.95749
89928	TILT	2020-09-30	122.12	123.1599	121.93	122	36929	{}	\N	2020-10-12 17:04:53.965333
89929	TILT	2020-10-01	122.39	123.12	122.32	122.74	9031	{}	\N	2020-10-12 17:04:53.972939
89930	TILT	2020-10-02	122.48	123.33	122.1226	123.18	4341	{}	\N	2020-10-12 17:04:53.979293
89931	TILT	2020-10-05	124.337	125.37	124.337	125.37	3028	{}	\N	2020-10-12 17:04:53.985118
89932	TILT	2020-10-06	125.86	126.83	123.75	123.75	6252	{}	\N	2020-10-12 17:04:53.990646
89933	TILT	2020-10-07	125.77	126.3816	125.74	126.3816	4486	{}	\N	2020-10-12 17:04:53.996035
89934	TILT	2020-10-08	127.16	127.6044	127	127.6044	6312	{}	\N	2020-10-12 17:04:54.0014
89935	TILT	2020-10-09	128.495	128.59	127.89	128.2035	7490	{}	\N	2020-10-12 17:04:54.007558
89936	TILT	2020-10-12	129.16	129.75	129.16	129.69	9660	{}	\N	2020-10-12 17:04:54.013904
89937	TNA	2020-09-28	29.72	30.94	29.63	30.75	12485904	{}	\N	2020-10-12 17:04:55.314591
89938	TNA	2020-09-29	30.63	30.8592	29.47	30.3	14551933	{}	\N	2020-10-12 17:04:55.326897
89939	TNA	2020-09-30	30.6135	31.8812	29.78	30.6	14990211	{}	\N	2020-10-12 17:04:55.334921
89940	TNA	2020-10-01	31.15	32.035	30.39	31.99	11965959	{}	\N	2020-10-12 17:04:55.340765
89941	TNA	2020-10-02	30.27	32.883	30.02	32.45	16180987	{}	\N	2020-10-12 17:04:55.346079
89942	TNA	2020-10-05	33.46	35.27	33.46	35.12	9164111	{}	\N	2020-10-12 17:04:55.350762
89943	TNA	2020-10-06	36.08	37.7683	34.64	34.78	16009869	{}	\N	2020-10-12 17:04:55.355345
89944	TNA	2020-10-07	36.13	37.4484	36.06	37.11	10530144	{}	\N	2020-10-12 17:04:55.360023
89945	TNA	2020-10-08	38.29	38.65	37.485	38.37	10874587	{}	\N	2020-10-12 17:04:55.364649
89946	TNA	2020-10-09	39.42	39.67	38.3208	38.95	13684373	{}	\N	2020-10-12 17:04:55.368952
89947	TNA	2020-10-12	39.49	39.975	38.88	39.4356	4866785	{}	\N	2020-10-12 17:04:55.373571
89948	TX	2020-09-28	19.39	19.685	18.95	19.1	201891	{}	\N	2020-10-12 17:04:57.346925
89949	TX	2020-09-29	19.07	19.07	18.53	18.53	393904	{}	\N	2020-10-12 17:04:57.358989
89950	TX	2020-09-30	18.5	19.04	18.5	18.83	1304624	{}	\N	2020-10-12 17:04:57.366142
89951	TX	2020-10-01	19.05	19.59	18.84	19.12	387753	{}	\N	2020-10-12 17:04:57.371957
89952	TX	2020-10-02	18.87	19.585	18.8101	19.38	245510	{}	\N	2020-10-12 17:04:57.377928
89953	TX	2020-10-05	19.48	20	19.36	19.93	294960	{}	\N	2020-10-12 17:04:57.38316
89954	TX	2020-10-06	19.97	20.27	19.449	19.56	409230	{}	\N	2020-10-12 17:04:57.388271
89955	TX	2020-10-07	19.93	20.2	19.75	20.02	537125	{}	\N	2020-10-12 17:04:57.39308
89956	TX	2020-10-08	20.25	20.26	19.58	19.87	487025	{}	\N	2020-10-12 17:04:57.397744
89957	TX	2020-10-09	20	20.05	19.08	19.7	535488	{}	\N	2020-10-12 17:04:57.402597
89958	TX	2020-10-12	19.81	19.96	19.72	19.825	95286	{}	\N	2020-10-12 17:04:57.407889
89959	UEPS	2020-09-28	3.21	3.24	3.15	3.21	201795	{}	\N	2020-10-12 17:04:58.569519
89960	UEPS	2020-09-29	3.46	3.46	3.34	3.36	639398	{}	\N	2020-10-12 17:04:58.5806
89961	UEPS	2020-09-30	3.35	3.4	3.31	3.37	250787	{}	\N	2020-10-12 17:04:58.588322
89962	UEPS	2020-10-01	3.42	3.47	3.42	3.46	479875	{}	\N	2020-10-12 17:04:58.594759
89963	UEPS	2020-10-02	3.38	3.48	3.3	3.4	123151	{}	\N	2020-10-12 17:04:58.600655
89964	UEPS	2020-10-05	3.47	3.485	3.4	3.46	176856	{}	\N	2020-10-12 17:04:58.606089
89965	UEPS	2020-10-06	3.48	3.4999	3.33	3.44	259932	{}	\N	2020-10-12 17:04:58.611362
89966	UEPS	2020-10-07	3.47	3.47	3.37	3.42	290035	{}	\N	2020-10-12 17:04:58.61633
89967	UEPS	2020-10-08	3.42	3.425	3.37	3.42	141664	{}	\N	2020-10-12 17:04:58.62151
89968	UEPS	2020-10-09	3.42	3.435	3.39	3.41	112183	{}	\N	2020-10-12 17:04:58.626783
89969	UEPS	2020-10-12	3.4	3.44	3.26	3.28	86813	{}	\N	2020-10-12 17:04:58.631863
89970	URE	2020-09-28	54.12	55.44	54.12	55.04	20176	{}	\N	2020-10-12 17:04:59.612139
89971	URE	2020-09-29	54.53	54.94	53.48	54.24	36742	{}	\N	2020-10-12 17:04:59.622446
89972	URE	2020-09-30	54.71	55.75	53.54	54.68	30429	{}	\N	2020-10-12 17:04:59.629785
89973	URE	2020-10-01	55.24	56.73	54.62	56.73	40590	{}	\N	2020-10-12 17:04:59.6362
89974	URE	2020-10-02	55.1	58.71	54.88	58.37	45540	{}	\N	2020-10-12 17:04:59.642247
89975	URE	2020-10-05	58.96	59.24	57.08	59.1	18658	{}	\N	2020-10-12 17:04:59.647512
89976	URE	2020-10-06	59.3	59.98	57.96	58.33	27779	{}	\N	2020-10-12 17:04:59.652439
89977	URE	2020-10-07	59.32	59.32	58.01	58.55	14132	{}	\N	2020-10-12 17:04:59.657434
89978	URE	2020-10-08	59.38	60.66	59.38	60.38	23194	{}	\N	2020-10-12 17:04:59.662158
89979	URE	2020-10-09	60.92	61.01	59.58	59.91	18332	{}	\N	2020-10-12 17:04:59.666822
89980	URE	2020-10-12	59.5	60.3588	59.37	60.25	13484	{}	\N	2020-10-12 17:04:59.671738
89981	WAT	2020-09-28	193.41	195.15	192.305	193.29	267643	{}	\N	2020-10-12 17:05:01.867762
89982	WAT	2020-09-29	193.07	195.115	191.62	193.39	529020	{}	\N	2020-10-12 17:05:01.87869
89983	WAT	2020-09-30	194.19	198.05	194.19	195.68	712366	{}	\N	2020-10-12 17:05:01.885482
89984	WAT	2020-10-01	197.25	198.68	194.07	195.68	868806	{}	\N	2020-10-12 17:05:01.891743
89985	WAT	2020-10-02	193.49	197.58	192.54	195.89	544145	{}	\N	2020-10-12 17:05:01.897276
89986	WAT	2020-10-05	197.96	200.955	197.0615	198.65	518714	{}	\N	2020-10-12 17:05:01.902135
89987	WAT	2020-10-06	199.17	200.18	195.81	196.68	443760	{}	\N	2020-10-12 17:05:01.906559
89988	WAT	2020-10-07	196.46	201.23	196.46	200.09	738857	{}	\N	2020-10-12 17:05:01.911084
89989	WAT	2020-10-08	201.65	202.29	199.2	201.19	487546	{}	\N	2020-10-12 17:05:01.915855
89990	WAT	2020-10-09	201.89	203.2	200.69	202.8	520664	{}	\N	2020-10-12 17:05:01.920089
89991	WAT	2020-10-12	204.01	204.963	202.26	203.515	175265	{}	\N	2020-10-12 17:05:01.924401
89992	TSBK	2020-09-28	16.92	17.35	16.92	17.11	8159	{}	\N	2020-10-12 17:05:03.421863
89993	TSBK	2020-09-29	17.11	17.64	17.11	17.64	5617	{}	\N	2020-10-12 17:05:03.431759
89994	TSBK	2020-09-30	17.82	18.18	17.6901	18	9481	{}	\N	2020-10-12 17:05:03.438729
89995	TSBK	2020-10-01	18.15	18.44	18.1	18.1	19516	{}	\N	2020-10-12 17:05:03.444213
89996	TSBK	2020-10-02	17.94	18.85	17.94	18.58	9878	{}	\N	2020-10-12 17:05:03.449304
89997	TSBK	2020-10-05	18.58	19.01	18.1243	18.57	13430	{}	\N	2020-10-12 17:05:03.454203
89998	TSBK	2020-10-06	18.69	18.9	18.0701	18.19	9619	{}	\N	2020-10-12 17:05:03.458817
89999	TSBK	2020-10-07	18.37	18.71	18.37	18.41	38054	{}	\N	2020-10-12 17:05:03.462772
90000	TSBK	2020-10-08	18.59	19.46	18.59	19.22	11190	{}	\N	2020-10-12 17:05:03.466844
90001	TSBK	2020-10-09	19.49	19.645	19.2512	19.3	7413	{}	\N	2020-10-12 17:05:03.471097
90002	TSBK	2020-10-12	19.49	19.49	19.31	19.31	2117	{}	\N	2020-10-12 17:05:03.475176
90003	TTT	2020-09-28	27.61	27.81	27.59	27.72	14824	{}	\N	2020-10-12 17:05:04.940173
90004	TTT	2020-09-29	27.62	27.67	27.44	27.62	12083	{}	\N	2020-10-12 17:05:04.949946
90005	TTT	2020-09-30	28.02	28.72	28.02	28.35	58858	{}	\N	2020-10-12 17:05:04.957596
90006	TTT	2020-10-01	28.7745	28.87	28.05	28.21	17068	{}	\N	2020-10-12 17:05:04.963583
90007	TTT	2020-10-02	28.1	28.77	28.1	28.57	29527	{}	\N	2020-10-12 17:05:04.969062
90008	TTT	2020-10-05	29.52	30.26	29.52	30.25	67253	{}	\N	2020-10-12 17:05:04.973622
90009	TTT	2020-10-06	30.4	30.7117	29.27	29.74	60379	{}	\N	2020-10-12 17:05:04.97827
90010	TTT	2020-10-07	30.27	30.61	29.85	30.41	44597	{}	\N	2020-10-12 17:05:04.982895
90011	TTT	2020-10-08	30.01	30.14	29.87	29.93	55639	{}	\N	2020-10-12 17:05:04.987315
90012	TTT	2020-10-09	30.29	30.665	29.85	29.98	38515	{}	\N	2020-10-12 17:05:04.99204
90013	TTT	2020-10-12	29.76	29.96	29.73	29.9	11936	{}	\N	2020-10-12 17:05:04.996698
90014	UEVM	2020-09-28	39.01	39.02	38.791	38.92	2315	{}	\N	2020-10-12 17:05:06.34509
90015	UEVM	2020-09-29	38.74	38.945	38.74	38.945	3053	{}	\N	2020-10-12 17:05:06.356899
90016	UEVM	2020-09-30	38.86	39.3	38.86	39.22	11782	{}	\N	2020-10-12 17:05:06.365481
90017	UEVM	2020-10-01	39.33	39.6051	39.33	39.6051	1480	{}	\N	2020-10-12 17:05:06.371839
90018	UEVM	2020-10-02	39.2001	39.43	39.1999	39.1999	3781	{}	\N	2020-10-12 17:05:06.377298
90019	UEVM	2020-10-05	39.5211	39.73	39.5211	39.73	108529	{}	\N	2020-10-12 17:05:06.38267
90020	UEVM	2020-10-06	39.715	39.82	39.53	39.59	25706	{}	\N	2020-10-12 17:05:06.387324
90021	UEVM	2020-10-07	39.7916	39.865	39.75	39.865	1408	{}	\N	2020-10-12 17:05:06.392446
90022	UEVM	2020-10-08	40.03	40.21	40.03	40.2	68679	{}	\N	2020-10-12 17:05:06.397217
90023	UEVM	2020-10-09	40.3118	40.49	40.3118	40.4005	1345	{}	\N	2020-10-12 17:05:06.401949
90024	UEVM	2020-10-12	40.48	40.62	40.48	40.6068	2712	{}	\N	2020-10-12 17:05:06.406543
90025	UFS	2020-09-28	27.22	27.88	27.22	27.55	818239	{}	\N	2020-10-12 17:05:07.338838
90026	UFS	2020-09-29	27.67	27.67	26.49	26.66	1165755	{}	\N	2020-10-12 17:05:07.352015
90027	UFS	2020-09-30	26.73	26.99	26.01	26.27	1524978	{}	\N	2020-10-12 17:05:07.360039
90028	UFS	2020-10-01	26.45	26.67	25.94	26.32	908192	{}	\N	2020-10-12 17:05:07.366928
90029	UFS	2020-10-02	25.56	26.75	25.56	26.37	1454085	{}	\N	2020-10-12 17:05:07.373184
90030	UFS	2020-10-05	26.87	27.215	26.64	27.1	804904	{}	\N	2020-10-12 17:05:07.378399
90031	UFS	2020-10-06	27.44	27.47	26.18	26.23	744317	{}	\N	2020-10-12 17:05:07.383727
90032	UFS	2020-10-07	26.55	27.07	26.41	27.04	1161517	{}	\N	2020-10-12 17:05:07.389081
90033	UFS	2020-10-08	27.36	28.22	27.21	28.21	1199292	{}	\N	2020-10-12 17:05:07.394237
90034	UFS	2020-10-09	28.38	28.77	27.61	27.63	882569	{}	\N	2020-10-12 17:05:07.399387
90035	UFS	2020-10-12	27.74	28.235	27.53	27.69	565805	{}	\N	2020-10-12 17:05:07.404473
90036	USLB	2020-09-28	31.86	31.9202	31.86	31.9202	380	{}	\N	2020-10-12 17:05:09.29514
90037	USLB	2020-09-29	31.7136	31.7136	31.6487	31.6487	114	{}	\N	2020-10-12 17:05:09.312517
90038	USLB	2020-10-05	32.24	32.435	32.24	32.435	101	{}	\N	2020-10-12 17:05:09.319592
90039	USLB	2020-10-08	32.8829	32.925	32.8829	32.925	300	{}	\N	2020-10-12 17:05:09.325391
90040	USLB	2020-10-09	33.04	33.139	33.04	33.139	1229	{}	\N	2020-10-12 17:05:09.330804
90041	USLB	2020-10-12	33.31	33.31	33.31	33.31	100	{}	\N	2020-10-12 17:05:09.336406
90042	UST	2020-09-28	76.2	76.23	76.1224	76.14	12464	{}	\N	2020-10-12 17:05:11.604619
90043	UST	2020-09-29	76.29	76.39	76.285	76.36	5978	{}	\N	2020-10-12 17:05:11.615655
90044	UST	2020-09-30	76.21	76.21	75.83	75.9	47654	{}	\N	2020-10-12 17:05:11.62357
90045	UST	2020-10-01	75.69	76.04	75.59	75.96	75806	{}	\N	2020-10-12 17:05:11.629778
90046	UST	2020-10-02	76.18	76.18	75.76	75.81	84651	{}	\N	2020-10-12 17:05:11.634963
90047	UST	2020-10-05	75.41	75.42	74.939	74.95	10344	{}	\N	2020-10-12 17:05:11.639939
90048	UST	2020-10-06	74.97	75.28	74.94	75.28	4086	{}	\N	2020-10-12 17:05:11.644093
90049	UST	2020-10-07	74.98	74.98	74.77	74.77	5699	{}	\N	2020-10-12 17:05:11.648432
90050	UST	2020-10-08	74.92	75.02	74.91	75.01	6824	{}	\N	2020-10-12 17:05:11.653113
90051	UST	2020-10-09	75.02	75.02	74.732	74.9292	2671	{}	\N	2020-10-12 17:05:11.657734
90052	UST	2020-10-12	74.98	74.98	74.89	74.895	3767	{}	\N	2020-10-12 17:05:11.662148
90053	UTI	2020-09-28	5.25	5.59	5.25	5.41	241419	{}	\N	2020-10-12 17:05:14.112873
90054	UTI	2020-09-29	5.39	5.45	5.02	5.03	1836999	{}	\N	2020-10-12 17:05:14.124937
90055	UTI	2020-09-30	5.05	5.4	5	5.08	855243	{}	\N	2020-10-12 17:05:14.133208
90056	UTI	2020-10-01	5.15	5.19	4.91	5.1	294788	{}	\N	2020-10-12 17:05:14.140079
90057	UTI	2020-10-02	4.97	5.12	4.87	5.04	311865	{}	\N	2020-10-12 17:05:14.146258
90058	UTI	2020-10-05	5.09	5.23	5.08	5.21	389866	{}	\N	2020-10-12 17:05:14.1512
90059	UTI	2020-10-06	5.24	5.6	5.22	5.42	488802	{}	\N	2020-10-12 17:05:14.156176
90060	UTI	2020-10-07	5.41	5.62	5.41	5.53	202149	{}	\N	2020-10-12 17:05:14.160807
90061	UTI	2020-10-08	5.62	5.73	5.49	5.59	212838	{}	\N	2020-10-12 17:05:14.165425
90062	UTI	2020-10-09	5.6	5.72	5.35	5.37	155072	{}	\N	2020-10-12 17:05:14.169715
90063	UTI	2020-10-12	5.35	5.4	5.17	5.18	97642	{}	\N	2020-10-12 17:05:14.174094
90064	VAMO	2020-09-28	17.6942	17.6942	17.6942	17.6942	184	{}	\N	2020-10-12 17:05:15.096296
90065	VAMO	2020-09-29	17.8	17.93	17.8	17.93	1791	{}	\N	2020-10-12 17:05:15.107957
90066	VAMO	2020-09-30	18.07	18.13	17.9694	17.9694	8543	{}	\N	2020-10-12 17:05:15.115577
90067	VAMO	2020-10-01	18.08	18.08	18.04	18.04	527	{}	\N	2020-10-12 17:05:15.121337
90068	VAMO	2020-10-02	18.2577	18.2577	18.2577	18.2577	529	{}	\N	2020-10-12 17:05:15.127002
90069	VAMO	2020-10-05	18.5	18.5	18.4438	18.4438	1210	{}	\N	2020-10-12 17:05:15.132015
90070	VAMO	2020-10-06	18.4646	18.4646	18.4646	18.4646	130	{}	\N	2020-10-12 17:05:15.136677
90071	VAMO	2020-10-07	18.93	18.935	18.77	18.8839	1194	{}	\N	2020-10-12 17:05:15.141493
90072	VAMO	2020-10-08	18.9811	18.9811	18.9811	18.9811	19	{}	\N	2020-10-12 17:05:15.146038
90073	VAMO	2020-10-09	19.1367	19.1367	19.1367	19.1367	105	{}	\N	2020-10-12 17:05:15.151296
90074	VAMO	2020-10-12	19.02	19.02	19.02	19.02	566	{}	\N	2020-10-12 17:05:15.155774
90075	VIS	2020-09-28	145.05	147.2968	145.05	146.2	137794	{}	\N	2020-10-12 17:05:16.356605
90076	VIS	2020-09-29	146.16	146.53	144.72	145.17	68723	{}	\N	2020-10-12 17:05:16.367521
90077	VIS	2020-09-30	145.51	147.4	144.08	144.96	103179	{}	\N	2020-10-12 17:05:16.376778
90078	VIS	2020-10-01	145.78	146.344	143.86	144.88	92308	{}	\N	2020-10-12 17:05:16.381827
90079	VIS	2020-10-02	142.3	147.05	142.03	146.45	64379	{}	\N	2020-10-12 17:05:16.386841
90080	VIS	2020-10-05	147.61	149.11	147.61	148.93	46526	{}	\N	2020-10-12 17:05:16.392032
90081	VIS	2020-10-06	149.51	150.8	147.37	147.39	90582	{}	\N	2020-10-12 17:05:16.396683
90082	VIS	2020-10-07	149.26	151	149.26	150.49	108824	{}	\N	2020-10-12 17:05:16.401216
90083	VIS	2020-10-08	151.42	151.96	150.34	151.96	109768	{}	\N	2020-10-12 17:05:16.405881
90084	VIS	2020-10-09	152.75	153.5194	152.16	152.48	115631	{}	\N	2020-10-12 17:05:16.410254
90085	VIS	2020-10-12	153.24	153.6996	152.865	153.5688	45008	{}	\N	2020-10-12 17:05:16.414867
90086	VMOT	2020-09-28	23.725	23.756	23.68	23.756	501	{}	\N	2020-10-12 17:05:17.669659
90087	VMOT	2020-09-29	23.8594	23.9	23.8582	23.8582	1419	{}	\N	2020-10-12 17:05:17.680356
90088	VMOT	2020-09-30	23.9	23.9545	23.89	23.9	1466	{}	\N	2020-10-12 17:05:17.687554
90089	VMOT	2020-10-01	24.06	24.13	24.06	24.11	1341	{}	\N	2020-10-12 17:05:17.693217
90090	VMOT	2020-10-02	24.0272	24.0818	24.0272	24.0818	633	{}	\N	2020-10-12 17:05:17.699028
90091	VMOT	2020-10-05	24.36	24.3994	24.34	24.3994	1507	{}	\N	2020-10-12 17:05:17.704011
90092	VMOT	2020-10-06	24.5	24.5	24.2675	24.2675	328	{}	\N	2020-10-12 17:05:17.708607
90093	VMOT	2020-10-07	24.51	24.58	24.51	24.55	2422	{}	\N	2020-10-12 17:05:17.713501
90094	VMOT	2020-10-08	24.66	24.66	24.62	24.6393	1065	{}	\N	2020-10-12 17:05:17.718123
90095	VMOT	2020-10-09	24.85	24.85	24.7976	24.8069	1498	{}	\N	2020-10-12 17:05:17.722864
90096	VMOT	2020-10-12	24.841	24.841	24.841	24.841	837	{}	\N	2020-10-12 17:05:17.72771
90097	TURN	2020-09-28	1.9	1.93	1.89	1.92	9919	{}	\N	2020-10-12 17:05:18.838068
90098	TURN	2020-09-29	1.92	1.94	1.87	1.89	25540	{}	\N	2020-10-12 17:05:18.850848
90099	TURN	2020-09-30	1.91	1.91	1.88	1.89	9272	{}	\N	2020-10-12 17:05:18.858021
90100	TURN	2020-10-01	1.9	1.9095	1.86	1.89	21668	{}	\N	2020-10-12 17:05:18.865843
90101	TURN	2020-10-02	1.85	1.8786	1.84	1.85	53555	{}	\N	2020-10-12 17:05:18.871885
90102	TURN	2020-10-05	1.87	1.89	1.82	1.85	41738	{}	\N	2020-10-12 17:05:18.877184
90103	TURN	2020-10-06	1.85	1.9	1.82	1.86	41642	{}	\N	2020-10-12 17:05:18.882752
90104	TURN	2020-10-07	1.845	1.89	1.84	1.8758	12860	{}	\N	2020-10-12 17:05:18.888157
90105	TURN	2020-10-08	1.88	2.02	1.88	2	166061	{}	\N	2020-10-12 17:05:18.893525
90106	TURN	2020-10-09	2.01	2.02	1.99	2	45187	{}	\N	2020-10-12 17:05:18.898828
90107	TURN	2020-10-12	2.01	2.01	1.9667	1.98	16241	{}	\N	2020-10-12 17:05:18.90416
90108	ULE	2020-09-28	14.5	14.5	14.44	14.48	9673	{}	\N	2020-10-12 17:05:20.088375
90109	ULE	2020-09-29	14.6	14.705	14.6	14.705	6451	{}	\N	2020-10-12 17:05:20.097414
90110	ULE	2020-09-30	14.67	14.69	14.63	14.65	1359	{}	\N	2020-10-12 17:05:20.103648
90111	ULE	2020-10-01	14.7	14.715	14.6603	14.71	3986	{}	\N	2020-10-12 17:05:20.109399
90112	ULE	2020-10-02	14.62	14.63	14.6	14.6	661	{}	\N	2020-10-12 17:05:20.114432
90113	ULE	2020-10-05	14.7998	14.83	14.7686	14.83	3985	{}	\N	2020-10-12 17:05:20.119641
90114	ULE	2020-10-06	14.818	14.818	14.69	14.75	3739	{}	\N	2020-10-12 17:05:20.124374
90115	ULE	2020-10-07	14.7418	14.77	14.55	14.77	1578	{}	\N	2020-10-12 17:05:20.129426
90116	ULE	2020-10-08	14.67	14.7294	14.67	14.7294	743	{}	\N	2020-10-12 17:05:20.134436
90117	ULE	2020-10-09	14.88	14.91	14.87	14.8929	2401	{}	\N	2020-10-12 17:05:20.139215
90118	ULE	2020-10-12	14.8404	14.845	14.84	14.8447	3117	{}	\N	2020-10-12 17:05:20.144141
90119	URTY	2020-09-28	35.26	36.685	35.1697	36.47	571955	{}	\N	2020-10-12 17:05:21.153295
90120	URTY	2020-09-29	36.32	36.5766	34.98	35.93	446782	{}	\N	2020-10-12 17:05:21.167865
90121	URTY	2020-09-30	36.27	37.81	35.36	36.31	719903	{}	\N	2020-10-12 17:05:21.175895
90122	URTY	2020-10-01	36.94	38	36.05	37.98	625388	{}	\N	2020-10-12 17:05:21.182436
90123	URTY	2020-10-02	35.84	39.01	35.64	38.52	2165752	{}	\N	2020-10-12 17:05:21.188348
90124	URTY	2020-10-05	39.74	41.83	39.7	41.67	1044764	{}	\N	2020-10-12 17:05:21.193698
90125	URTY	2020-10-06	42.77	44.79	41.09	41.34	2208426	{}	\N	2020-10-12 17:05:21.198505
90126	URTY	2020-10-07	42.88	44.41	42.8	44	993985	{}	\N	2020-10-12 17:05:21.203655
90127	URTY	2020-10-08	45.41	45.84	44.46	45.53	1060996	{}	\N	2020-10-12 17:05:21.208642
90128	URTY	2020-10-09	46.77	47.02	45.49	46.23	653967	{}	\N	2020-10-12 17:05:21.213655
90129	URTY	2020-10-12	46.77	47.4	46.1453	46.77	509235	{}	\N	2020-10-12 17:05:21.218629
90130	USTB	2020-09-28	51.7	51.7078	51.682	51.705	88737	{}	\N	2020-10-12 17:05:23.744496
90131	USTB	2020-09-29	51.73	51.735	51.7101	51.73	120863	{}	\N	2020-10-12 17:05:23.753325
90132	USTB	2020-09-30	51.76	51.77	51.75	51.75	90279	{}	\N	2020-10-12 17:05:23.760692
90133	USTB	2020-10-01	51.77	51.805	51.7621	51.7621	22195	{}	\N	2020-10-12 17:05:23.767207
90134	USTB	2020-10-02	51.7683	51.96	51.7584	51.809	13018	{}	\N	2020-10-12 17:05:23.77344
90135	USTB	2020-10-05	51.75	51.77	51.74	51.75	26104	{}	\N	2020-10-12 17:05:23.781529
90136	USTB	2020-10-06	51.7922	51.81	51.7731	51.7798	2714	{}	\N	2020-10-12 17:05:23.78705
90137	USTB	2020-10-07	51.74	52.13	51.735	51.75	15440	{}	\N	2020-10-12 17:05:23.792075
90138	USTB	2020-10-08	51.7429	51.76	51.7429	51.7586	1330	{}	\N	2020-10-12 17:05:23.79738
90139	USTB	2020-10-09	51.77	51.8	51.75	51.765	6320	{}	\N	2020-10-12 17:05:23.802514
90140	USTB	2020-10-12	51.74	51.78	51.74	51.77	1402	{}	\N	2020-10-12 17:05:23.807611
90141	UTSL	2020-09-28	22.96	23.31	22.57	22.58	33764	{}	\N	2020-10-12 17:05:24.785268
90142	UTSL	2020-09-29	22.81	23.29	22.41	22.7	23721	{}	\N	2020-10-12 17:05:24.796496
90143	UTSL	2020-09-30	23.24	23.57	22.7263	23.36	33671	{}	\N	2020-10-12 17:05:24.804751
90144	UTSL	2020-10-01	23.56	24.11	23.41	23.9676	58926	{}	\N	2020-10-12 17:05:24.810261
90145	UTSL	2020-10-02	23.19	25.15	23.19	24.8167	80636	{}	\N	2020-10-12 17:05:24.815877
90146	UTSL	2020-10-05	25.02	25.95	24.56	25.75	50025	{}	\N	2020-10-12 17:05:24.820963
90147	UTSL	2020-10-06	25.86	27.2598	25.63	26.52	138435	{}	\N	2020-10-12 17:05:24.825635
90148	UTSL	2020-10-07	26.92	27.22	26.385	27.009	82573	{}	\N	2020-10-12 17:05:24.830933
90149	UTSL	2020-10-08	27.245	28.51	27.12	28.38	89497	{}	\N	2020-10-12 17:05:24.836065
90150	UTSL	2020-10-09	28.95	28.9999	27.7989	28.37	109407	{}	\N	2020-10-12 17:05:24.840667
90151	UTSL	2020-10-12	28.51	29.19	28.2764	29.07	64762	{}	\N	2020-10-12 17:05:24.845296
90152	VECO	2020-09-28	11.65	12.15	11.65	11.98	370971	{}	\N	2020-10-12 17:05:25.85134
90153	VECO	2020-09-29	11.96	12.3	11.84	12.01	966644	{}	\N	2020-10-12 17:05:25.860873
90154	VECO	2020-09-30	11.95	12.17	11.59	11.67	411787	{}	\N	2020-10-12 17:05:25.867532
90155	VECO	2020-10-01	11.79	12.265	11.665	12.23	478530	{}	\N	2020-10-12 17:05:25.873096
90156	VECO	2020-10-02	11.82	12.03	11.56	11.62	303822	{}	\N	2020-10-12 17:05:25.879342
90157	VECO	2020-10-05	11.8	11.94	11.65	11.8	330740	{}	\N	2020-10-12 17:05:25.884198
90158	VECO	2020-10-06	11.87	12.16	11.83	11.96	537453	{}	\N	2020-10-12 17:05:25.888849
90159	VECO	2020-10-07	12.11	12.35	12.0101	12.19	314293	{}	\N	2020-10-12 17:05:25.89335
90160	VECO	2020-10-08	12.23	12.35	12.15	12.245	380288	{}	\N	2020-10-12 17:05:25.8975
90161	VECO	2020-10-09	12.48	12.92	12.45	12.67	480257	{}	\N	2020-10-12 17:05:25.902001
90162	VECO	2020-10-12	12.88	12.88	12.6	12.7	83802	{}	\N	2020-10-12 17:05:25.906201
90163	VGIT	2020-09-28	70.54	70.55	70.52	70.54	644886	{}	\N	2020-10-12 17:05:27.239194
90164	VGIT	2020-09-29	70.57	70.6	70.56	70.58	738181	{}	\N	2020-10-12 17:05:27.250147
90165	VGIT	2020-09-30	70.56	70.56	70.44	70.48	756138	{}	\N	2020-10-12 17:05:27.258347
90166	VGIT	2020-10-01	70.34	70.46	70.31	70.41	874407	{}	\N	2020-10-12 17:05:27.264854
90167	VGIT	2020-10-02	70.45	70.455	70.35	70.39	837203	{}	\N	2020-10-12 17:05:27.271211
90168	VGIT	2020-10-05	70.3	70.3	70.15	70.16	577015	{}	\N	2020-10-12 17:05:27.276507
90169	VGIT	2020-10-06	70.18	70.3	70.15	70.23	527706	{}	\N	2020-10-12 17:05:27.281425
90170	VGIT	2020-10-07	70.15	70.19	70.09	70.13	817826	{}	\N	2020-10-12 17:05:27.286821
90171	VGIT	2020-10-08	70.17	70.215	70.15	70.21	401097	{}	\N	2020-10-12 17:05:27.291689
90172	VGIT	2020-10-09	70.17	70.22	70.11	70.18	564278	{}	\N	2020-10-12 17:05:27.296687
90173	VGIT	2020-10-12	70.18	70.22	70.16	70.1696	235359	{}	\N	2020-10-12 17:05:27.301519
90174	VLU	2020-09-28	101.62	102.31	101.62	102.31	440	{}	\N	2020-10-12 17:05:28.292334
90175	VLU	2020-09-29	101.43	101.43	100.7829	100.7829	629	{}	\N	2020-10-12 17:05:28.300224
90176	VLU	2020-09-30	101.72	101.8024	101.72	101.8024	522	{}	\N	2020-10-12 17:05:28.306815
90177	VLU	2020-10-01	102.1655	102.1655	101.67	101.67	364	{}	\N	2020-10-12 17:05:28.31339
90178	VLU	2020-10-02	102.2397	102.2397	102.2397	102.2397	123	{}	\N	2020-10-12 17:05:28.32012
90179	VLU	2020-10-05	104.0511	104.0511	104.0511	104.0511	140	{}	\N	2020-10-12 17:05:28.32663
90180	VLU	2020-10-06	104.53	104.53	102.9569	102.9569	393	{}	\N	2020-10-12 17:05:28.333782
90181	VLU	2020-10-07	104.8265	104.8265	104.8265	104.8265	14	{}	\N	2020-10-12 17:05:28.340334
90182	VLU	2020-10-08	106.1892	106.1892	106.1892	106.1892	77	{}	\N	2020-10-12 17:05:28.345209
90183	VLU	2020-10-09	106.855	106.855	106.4034	106.4034	437	{}	\N	2020-10-12 17:05:28.349831
90184	VLU	2020-10-12	106.56	106.79	106.56	106.79	340	{}	\N	2020-10-12 17:05:28.354637
90185	VNO	2020-09-28	34.48	35.1	33.84	34.76	1255777	{}	\N	2020-10-12 17:05:29.359466
90186	VNO	2020-09-29	34.36	34.73	32.97	33.8	2515462	{}	\N	2020-10-12 17:05:29.367192
90187	VNO	2020-09-30	34.18	34.85	33.24	33.71	3228454	{}	\N	2020-10-12 17:05:29.373303
90188	VNO	2020-10-01	33.87	34.66	33.46	34.65	2401609	{}	\N	2020-10-12 17:05:29.379374
90189	VNO	2020-10-02	33.6	35.91	33.34	35.71	2679049	{}	\N	2020-10-12 17:05:29.3848
90190	VNO	2020-10-05	36	36.42	34.9	35.57	2592783	{}	\N	2020-10-12 17:05:29.389648
90191	VNO	2020-10-06	35.92	36.36	35.03	35.26	3385192	{}	\N	2020-10-12 17:05:29.394373
90192	VNO	2020-10-07	35.39	36.06	35.22	35.67	2419057	{}	\N	2020-10-12 17:05:29.399171
90193	VNO	2020-10-08	35.93	36.68	35.67	36.67	2191432	{}	\N	2020-10-12 17:05:29.404023
90194	VNO	2020-10-09	36.95	37.183	35.28	35.52	2168651	{}	\N	2020-10-12 17:05:29.408643
90195	VNO	2020-10-12	35.5	35.51	34.99	35.3	850855	{}	\N	2020-10-12 17:05:29.413282
90196	VNOM	2020-09-28	7.68	7.84	7.605	7.74	223217	{}	\N	2020-10-12 17:05:30.493494
90197	VNOM	2020-09-29	7.68	7.74	7.36	7.68	312754	{}	\N	2020-10-12 17:05:30.508765
90198	VNOM	2020-09-30	7.7	7.8	7.43	7.52	293710	{}	\N	2020-10-12 17:05:30.516506
90199	VNOM	2020-10-01	7.51	7.52	7.184	7.34	533846	{}	\N	2020-10-12 17:05:30.522401
90200	VNOM	2020-10-02	7.12	7.6	7.0522	7.46	417764	{}	\N	2020-10-12 17:05:30.527421
90201	VNOM	2020-10-05	7.58	7.792	7.45	7.74	210963	{}	\N	2020-10-12 17:05:30.5317
90202	VNOM	2020-10-06	7.89	7.95	7.53	7.57	324204	{}	\N	2020-10-12 17:05:30.535827
90203	VNOM	2020-10-07	7.63	7.6813	7.4701	7.57	154195	{}	\N	2020-10-12 17:05:30.540143
90204	VNOM	2020-10-08	7.67	8.09	7.52	8.02	276701	{}	\N	2020-10-12 17:05:30.544406
90205	VNOM	2020-10-09	8.12	8.13	7.75	7.84	235784	{}	\N	2020-10-12 17:05:30.549146
90206	VNOM	2020-10-12	7.9	7.9234	7.56	7.665	170401	{}	\N	2020-10-12 17:05:30.55392
90207	VOE	2020-09-28	100.68	101.93	100.5	101.43	410356	{}	\N	2020-10-12 17:05:32.742547
90208	VOE	2020-09-29	101.41	101.61	100.19	100.75	409145	{}	\N	2020-10-12 17:05:32.749825
90209	VOE	2020-09-30	101.21	102.25	100.59	101.37	433364	{}	\N	2020-10-12 17:05:32.755505
90210	VOE	2020-10-01	101.75	102.18	100.86	101.66	251548	{}	\N	2020-10-12 17:05:32.760294
90211	VOE	2020-10-02	100	103.225	100	102.83	328488	{}	\N	2020-10-12 17:05:32.765399
90212	VOE	2020-10-05	103.56	104.78	103.56	104.61	333089	{}	\N	2020-10-12 17:05:32.770159
90213	VOE	2020-10-06	105.17	106.16	103.52	103.63	316505	{}	\N	2020-10-12 17:05:32.775393
90214	VOE	2020-10-07	104.67	105.7302	104.48	105.45	280064	{}	\N	2020-10-12 17:05:32.779959
90215	VOE	2020-10-08	106.04	107.2	105.93	107.19	346143	{}	\N	2020-10-12 17:05:32.787278
90216	VOE	2020-10-09	107.8	108	106.7472	106.93	176153	{}	\N	2020-10-12 17:05:32.792626
90217	VOE	2020-10-12	107.33	107.57	106.93	107.44	148632	{}	\N	2020-10-12 17:05:32.798309
90218	VONG	2020-09-28	221.86	221.98	220.215	221.94	509088	{}	\N	2020-10-12 17:05:34.313741
90219	VONG	2020-09-29	221.95	222.6179	221.23	221.36	49789	{}	\N	2020-10-12 17:05:34.327786
90220	VONG	2020-09-30	221.87	225.34	221.81	222.9	150228	{}	\N	2020-10-12 17:05:34.334157
90221	VONG	2020-10-01	225.67	226.2185	224.57	225.92	64266	{}	\N	2020-10-12 17:05:34.339838
90222	VONG	2020-10-02	220.95	224.295	220.5	221.49	58587	{}	\N	2020-10-12 17:05:34.345387
90223	VONG	2020-10-05	222.68	225.9	222.68	225.77	69038	{}	\N	2020-10-12 17:05:34.350899
90224	VONG	2020-10-06	225.48	226.7	221.71	221.94	76901	{}	\N	2020-10-12 17:05:34.355182
90225	VONG	2020-10-07	224.73	226.64	224.3533	226.34	92544	{}	\N	2020-10-12 17:05:34.360072
90226	VONG	2020-10-08	227.99	227.99	226.5586	227.3	60922	{}	\N	2020-10-12 17:05:34.364635
90227	VONG	2020-10-09	228.69	230.83	228.5499	230.83	64847	{}	\N	2020-10-12 17:05:34.369596
90228	VONG	2020-10-12	233.9	236.6586	232.9434	236.35	38739	{}	\N	2020-10-12 17:05:34.374534
90229	USFD	2020-09-28	23.27	23.46	22.95	23.04	1452511	{}	\N	2020-10-12 17:05:37.395274
90230	USFD	2020-09-29	22.99	23	22.015	22.1	5146427	{}	\N	2020-10-12 17:05:37.405991
90231	USFD	2020-09-30	22.21	22.86	22.08	22.22	4661006	{}	\N	2020-10-12 17:05:37.413173
90232	USFD	2020-10-01	22.44	22.67	22.2	22.58	3335794	{}	\N	2020-10-12 17:05:37.419618
90233	USFD	2020-10-02	21.74	22.895	21.6	22.8	4834057	{}	\N	2020-10-12 17:05:37.42615
90234	USFD	2020-10-05	23.2	23.355	22.235	22.41	3331742	{}	\N	2020-10-12 17:05:37.431612
90235	USFD	2020-10-06	22.61	24.08	22.6	23.19	7343573	{}	\N	2020-10-12 17:05:37.436788
90236	USFD	2020-10-07	23.6	23.73	23.29	23.39	3574878	{}	\N	2020-10-12 17:05:37.442403
90237	USFD	2020-10-08	23.75	24.0803	23.58	23.77	4807751	{}	\N	2020-10-12 17:05:37.448304
90238	USFD	2020-10-09	23.91	24.46	23.44	24.31	4174646	{}	\N	2020-10-12 17:05:37.453291
90239	USFD	2020-10-12	24.35	25.22	23.86	25.035	2992570	{}	\N	2020-10-12 17:05:37.458621
90240	V	2020-09-28	200.63	201.72	199.83	200.32	5670796	{}	\N	2020-10-12 17:05:40.360594
90241	V	2020-09-29	200.58	202.38	199.37	199.44	11159890	{}	\N	2020-10-12 17:05:40.373499
90242	V	2020-09-30	200.85	202.54	198.89	199.97	17562753	{}	\N	2020-10-12 17:05:40.380716
90243	V	2020-10-01	202.21	205.195	201.6357	203.35	14246379	{}	\N	2020-10-12 17:05:40.387685
90244	V	2020-10-02	200	202.83	199.4	201.46	11931765	{}	\N	2020-10-12 17:05:40.394369
90245	V	2020-10-05	202.71	203.65	201.05	203.54	12013569	{}	\N	2020-10-12 17:05:40.400407
90246	V	2020-10-06	204.11	205.1	199.71	200.45	11305332	{}	\N	2020-10-12 17:05:40.406182
90247	V	2020-10-07	202	202.79	199.99	202.47	13487117	{}	\N	2020-10-12 17:05:40.411513
90248	V	2020-10-08	204	204.3	202.23	202.98	8073253	{}	\N	2020-10-12 17:05:40.416979
90249	V	2020-10-09	204	207.08	203.675	206.64	10998584	{}	\N	2020-10-12 17:05:40.422203
90250	V	2020-10-12	206.94	207.97	205.73	207.62	6931842	{}	\N	2020-10-12 17:05:40.427382
90251	VGK	2020-09-28	52.54	52.67	52.45	52.65	3038343	{}	\N	2020-10-12 17:05:41.554918
90252	VGK	2020-09-29	52.65	52.86	52.38	52.6	1742111	{}	\N	2020-10-12 17:05:41.566677
90253	VGK	2020-09-30	52.64	52.94	52.26	52.43	6901375	{}	\N	2020-10-12 17:05:41.573452
90254	VGK	2020-10-01	52.84	52.905	52.5101	52.84	2958608	{}	\N	2020-10-12 17:05:41.579317
90255	VGK	2020-10-02	52.21	52.98	52.21	52.82	3313324	{}	\N	2020-10-12 17:05:41.584753
90256	VGK	2020-10-05	53.32	53.77	53.32	53.77	2257470	{}	\N	2020-10-12 17:05:41.589475
90257	VGK	2020-10-06	53.83	53.84	52.935	53.03	3218437	{}	\N	2020-10-12 17:05:41.593827
90258	VGK	2020-10-07	53.45	53.575	53.29	53.48	3625594	{}	\N	2020-10-12 17:05:41.597946
90259	VGK	2020-10-08	53.77	53.98	53.73	53.98	3212615	{}	\N	2020-10-12 17:05:41.602642
90260	VGK	2020-10-09	54.35	54.55	54.25	54.5	3037107	{}	\N	2020-10-12 17:05:41.607131
90261	VGK	2020-10-12	54.67	54.89	54.615	54.83	967848	{}	\N	2020-10-12 17:05:41.611533
90262	VJET	2020-09-28	7.12	7.4	7.12	7.34	5647	{}	\N	2020-10-12 17:05:43.330896
90263	VJET	2020-09-29	7.39	8.67	7.295	8.315	20888	{}	\N	2020-10-12 17:05:43.342093
90264	VJET	2020-09-30	8.38	9.1	8.235	8.46	69124	{}	\N	2020-10-12 17:05:43.349415
90265	VJET	2020-10-01	8.45	8.583	8.28	8.415	8287	{}	\N	2020-10-12 17:05:43.356242
90266	VJET	2020-10-02	8.82	10.49	8.82	9.66	118715	{}	\N	2020-10-12 17:05:43.36254
90267	VJET	2020-10-05	9.82	9.83	8.75	9.12	18859	{}	\N	2020-10-12 17:05:43.36745
90268	VJET	2020-10-06	9.02	9.24	8.5801	8.8	11862	{}	\N	2020-10-12 17:05:43.372339
90269	VJET	2020-10-07	8.9098	11	8.89	10.25	101677	{}	\N	2020-10-12 17:05:43.377609
90270	VJET	2020-10-08	10.25	10.5	9.22	9.32	32787	{}	\N	2020-10-12 17:05:43.382518
90271	VJET	2020-10-09	9.64	10.5	9.1501	9.69	39199	{}	\N	2020-10-12 17:05:43.387339
90272	VJET	2020-10-12	10.07	10.5	9.69	9.978	11959	{}	\N	2020-10-12 17:05:43.39226
90273	VLRS	2020-09-28	7.1	7.2	6.99	6.99	193992	{}	\N	2020-10-12 17:05:44.294304
90274	VLRS	2020-09-29	6.96	7.02	6.82	6.9	588043	{}	\N	2020-10-12 17:05:44.308067
90275	VLRS	2020-09-30	6.83	7.24	6.83	7.21	558273	{}	\N	2020-10-12 17:05:44.316292
90276	VLRS	2020-10-01	7.5	7.67	7.4	7.52	989471	{}	\N	2020-10-12 17:05:44.322811
90277	VLRS	2020-10-02	7.3	7.615	7.29	7.55	593992	{}	\N	2020-10-12 17:05:44.328343
90278	VLRS	2020-10-05	7.63	7.85	7.57	7.79	436686	{}	\N	2020-10-12 17:05:44.334244
90279	VLRS	2020-10-06	7.92	8.05	7.66	7.74	962313	{}	\N	2020-10-12 17:05:44.338953
90280	VLRS	2020-10-07	7.73	7.965	7.66	7.76	496772	{}	\N	2020-10-12 17:05:44.343294
90281	VLRS	2020-10-08	7.72	7.9	7.68	7.86	395078	{}	\N	2020-10-12 17:05:44.348164
90282	VLRS	2020-10-09	7.94	8.06	7.9	8.04	1153661	{}	\N	2020-10-12 17:05:44.352781
90283	VLRS	2020-10-12	8.1	8.39	8.025	8.29	375449	{}	\N	2020-10-12 17:05:44.357044
90284	VNLA	2020-09-28	50.37	50.4	50.36	50.36	392329	{}	\N	2020-10-12 17:05:46.754315
90285	VNLA	2020-09-29	50.41	50.41	50.38	50.4	381762	{}	\N	2020-10-12 17:05:46.767747
90286	VNLA	2020-09-30	50.41	50.42	50.39	50.39	582337	{}	\N	2020-10-12 17:05:46.776736
90287	VNLA	2020-10-01	50.38	50.39	50.37	50.38	507929	{}	\N	2020-10-12 17:05:46.784058
90288	VNLA	2020-10-02	50.375	50.395	50.3701	50.38	387039	{}	\N	2020-10-12 17:05:46.793232
90289	VNLA	2020-10-05	50.39	50.415	50.38	50.39	489382	{}	\N	2020-10-12 17:05:46.799388
90290	VNLA	2020-10-06	50.4	50.4	50.34	50.37	733873	{}	\N	2020-10-12 17:05:46.80539
90291	VNLA	2020-10-07	50.39	50.4	50.38	50.4	491175	{}	\N	2020-10-12 17:05:46.811468
90292	VNLA	2020-10-08	50.39	50.43	50.38	50.38	396784	{}	\N	2020-10-12 17:05:46.817069
90293	VNLA	2020-10-09	50.41	50.43	50.3801	50.42	330045	{}	\N	2020-10-12 17:05:46.823196
90294	VNLA	2020-10-12	50.44	50.45	50.4301	50.4401	84225	{}	\N	2020-10-12 17:05:46.829264
90295	VNQI	2020-09-28	48.54	48.65	48.441	48.57	813144	{}	\N	2020-10-12 17:05:48.142994
90296	VNQI	2020-09-29	48.27	48.4473	48.0313	48.24	356216	{}	\N	2020-10-12 17:05:48.15417
90297	VNQI	2020-09-30	48.42	48.825	48.37	48.61	376891	{}	\N	2020-10-12 17:05:48.162914
90298	VNQI	2020-10-01	48.93	49.1999	48.8601	49.12	634670	{}	\N	2020-10-12 17:05:48.170054
90299	VNQI	2020-10-02	48.72	49.22	48.631	49.1	334850	{}	\N	2020-10-12 17:05:48.174397
90300	VNQI	2020-10-05	49.33	49.6299	49.3029	49.61	260728	{}	\N	2020-10-12 17:05:48.180649
90301	VNQI	2020-10-06	49.63	49.8499	49.29	49.3	205536	{}	\N	2020-10-12 17:05:48.186065
90302	VNQI	2020-10-07	49.56	49.6499	49.42	49.51	549145	{}	\N	2020-10-12 17:05:48.190336
90303	VNQI	2020-10-08	49.64	49.79	49.54	49.73	267861	{}	\N	2020-10-12 17:05:48.196841
90304	VNQI	2020-10-09	49.68	49.86	49.6095	49.82	237113	{}	\N	2020-10-12 17:05:48.201214
90305	VNQI	2020-10-12	49.79	50.1	49.78	50.0352	262691	{}	\N	2020-10-12 17:05:48.2054
90306	VOT	2020-09-28	178.88	179.88	178.2323	179.62	189018	{}	\N	2020-10-12 17:05:49.645494
90307	VOT	2020-09-29	179.53	180.545	178.94	179.14	105363	{}	\N	2020-10-12 17:05:49.655226
90308	VOT	2020-09-30	179.77	181.61	178.993	180.14	159211	{}	\N	2020-10-12 17:05:49.664117
90309	VOT	2020-10-01	181.13	182.81	181.13	182.53	148206	{}	\N	2020-10-12 17:05:49.671303
90310	VOT	2020-10-02	179.51	182.47	179.0301	181.4	213950	{}	\N	2020-10-12 17:05:49.677987
90311	VOT	2020-10-05	182.73	184.47	182.5814	184.29	90706	{}	\N	2020-10-12 17:05:49.684846
90312	VOT	2020-10-06	184.87	186.4938	182.465	183.01	182618	{}	\N	2020-10-12 17:05:49.696178
90313	VOT	2020-10-07	184.66	186.45	184.54	185.72	128742	{}	\N	2020-10-12 17:05:49.702029
90314	VOT	2020-10-08	187.1	187.38	186.58	187.22	133732	{}	\N	2020-10-12 17:05:49.707213
90315	VOT	2020-10-09	188.76	189.71	188.58	189.16	106152	{}	\N	2020-10-12 17:05:49.71295
90316	VOT	2020-10-12	190.54	191.86	190.01	191.6384	65828	{}	\N	2020-10-12 17:05:49.718103
90317	VSS	2020-09-28	104.52	104.85	104.31	104.72	205971	{}	\N	2020-10-12 17:05:50.67106
90318	VSS	2020-09-29	104.86	105.01	104.4	104.86	109000	{}	\N	2020-10-12 17:05:50.681629
90319	VSS	2020-09-30	104.8	105.7871	104.8	105.29	211059	{}	\N	2020-10-12 17:05:50.688714
90320	VSS	2020-10-01	105.98	106.56	105.75	106.49	424833	{}	\N	2020-10-12 17:05:50.694153
90321	VSS	2020-10-02	105.07	105.8325	105.07	105.52	2213446	{}	\N	2020-10-12 17:05:50.699972
90322	VSS	2020-10-05	106.47	107.34	106.47	107.32	157236	{}	\N	2020-10-12 17:05:50.70522
90323	VSS	2020-10-06	107.61	107.96	106.4	106.69	114580	{}	\N	2020-10-12 17:05:50.710812
90324	VSS	2020-10-07	107.43	107.74	107.19	107.49	1199720	{}	\N	2020-10-12 17:05:50.715989
90325	VSS	2020-10-08	107.92	108.18	107.6809	108.09	750945	{}	\N	2020-10-12 17:05:50.720797
90326	VSS	2020-10-09	108.64	109.28	108.61	109.16	347793	{}	\N	2020-10-12 17:05:50.725733
90327	VSS	2020-10-12	109.26	109.835	109.2	109.7999	63410	{}	\N	2020-10-12 17:05:50.730488
90328	PH	2020-09-28	203.14	206.71	202.4501	202.94	516602	{}	\N	2020-10-12 17:05:52.020136
90329	PH	2020-09-29	203.13	204.64	200.43	202.18	730659	{}	\N	2020-10-12 17:05:52.031288
90330	PH	2020-09-30	203.41	206.77	199.89	202.34	1832505	{}	\N	2020-10-12 17:05:52.038893
90331	PH	2020-10-01	203.43	205.095	198.81	200.73	1457911	{}	\N	2020-10-12 17:05:52.045066
90332	PH	2020-10-02	198.13	210.98	196.98	207.3	1790750	{}	\N	2020-10-12 17:05:52.050516
90333	PH	2020-10-05	210.83	214.04	210.13	213.14	1639802	{}	\N	2020-10-12 17:05:52.055693
90334	PH	2020-10-06	213.38	217	209.43	209.84	1030387	{}	\N	2020-10-12 17:05:52.060623
90335	PH	2020-10-07	212.98	217.84	212.8	217.55	1186956	{}	\N	2020-10-12 17:05:52.065177
90336	PH	2020-10-08	218.69	218.765	215.31	217.91	811811	{}	\N	2020-10-12 17:05:52.070259
90337	PH	2020-10-09	220.19	221.16	217.73	218.11	1105350	{}	\N	2020-10-12 17:05:52.075109
90338	PH	2020-10-12	219.19	219.495	216.63	218.71	699683	{}	\N	2020-10-12 17:05:52.079597
90339	PRGX	2020-09-28	4.73	4.935	4.73	4.87	7668	{}	\N	2020-10-12 17:05:54.992442
90340	PRGX	2020-09-29	4.89	4.89	4.71	4.8	5575	{}	\N	2020-10-12 17:05:55.004368
90341	PRGX	2020-09-30	4.78	4.85	4.75	4.76	5829	{}	\N	2020-10-12 17:05:55.013285
90342	PRGX	2020-10-01	4.82	5.025	4.82	5	8388	{}	\N	2020-10-12 17:05:55.01946
90343	PRGX	2020-10-02	5.03	5.2	4.31	4.8	186658	{}	\N	2020-10-12 17:05:55.02522
90344	PRGX	2020-10-05	4.81	5.0886	4.7	4.86	98030	{}	\N	2020-10-12 17:05:55.030246
90345	PRGX	2020-10-06	4.72	5	4.65	4.91	62419	{}	\N	2020-10-12 17:05:55.03541
90346	PRGX	2020-10-07	5.02	5.03	4.8499	4.8499	9330	{}	\N	2020-10-12 17:05:55.042191
90347	PRGX	2020-10-08	4.8901	4.98	4.75	4.95	56525	{}	\N	2020-10-12 17:05:55.047409
90348	PRGX	2020-10-09	4.94	5.1	4.75	4.81	63076	{}	\N	2020-10-12 17:05:55.053735
90349	PRGX	2020-10-12	4.87	5	4.62	4.74	21681	{}	\N	2020-10-12 17:05:55.060164
90350	PSMB	2020-09-28	13.6488	13.71	13.62	13.71	1287	{}	\N	2020-10-12 17:05:57.013998
90351	PSMB	2020-09-29	13.661	13.665	13.62	13.665	3576	{}	\N	2020-10-12 17:05:57.023586
90352	PSMB	2020-09-30	13.72	13.7375	13.621	13.641	2995	{}	\N	2020-10-12 17:05:57.03029
90353	PSMB	2020-10-01	13.695	13.7202	13.695	13.7202	1250	{}	\N	2020-10-12 17:05:57.036027
90354	PSMB	2020-10-02	13.56	13.755	13.56	13.755	3286	{}	\N	2020-10-12 17:05:57.04174
90355	PSMB	2020-10-05	13.86	13.89	13.86	13.89	8418	{}	\N	2020-10-12 17:05:57.047303
90356	PSMB	2020-10-06	13.92	13.9993	13.86	13.87	6720	{}	\N	2020-10-12 17:05:57.05256
90357	PSMB	2020-10-07	13.945	13.945	13.945	13.945	241	{}	\N	2020-10-12 17:05:57.057213
90358	PSMB	2020-10-08	14.0338	14.0452	14.0338	14.0452	1027	{}	\N	2020-10-12 17:05:57.06197
90359	PSMB	2020-10-09	14.085	14.085	14.085	14.085	196	{}	\N	2020-10-12 17:05:57.066383
90360	PSMB	2020-10-12	14.1699	14.17	14.1263	14.17	2271	{}	\N	2020-10-12 17:05:57.070519
90361	PTLC	2020-09-28	29.32	29.51	29.26	29.43	238221	{}	\N	2020-10-12 17:05:58.532528
90362	PTLC	2020-09-29	29.4	29.5	29.24	29.29	150491	{}	\N	2020-10-12 17:05:58.545185
90363	PTLC	2020-09-30	29.37	29.8	29.331	29.5	215322	{}	\N	2020-10-12 17:05:58.553292
90364	PTLC	2020-10-01	29.79	29.84	29.54	29.65	438645	{}	\N	2020-10-12 17:05:58.560362
90365	PTLC	2020-10-02	29.16	29.5964	29.16	29.38	340111	{}	\N	2020-10-12 17:05:58.565344
90366	PTLC	2020-10-05	29.48	29.9565	29.48	29.95	242861	{}	\N	2020-10-12 17:05:58.57052
90367	PTLC	2020-10-06	29.79	30.1435	29.4601	29.57	181890	{}	\N	2020-10-12 17:05:58.575277
90368	PTLC	2020-10-07	29.64	30.107	29.64	30.04	470597	{}	\N	2020-10-12 17:05:58.579865
90369	PTLC	2020-10-08	30.26	30.29	30.13	30.29	218926	{}	\N	2020-10-12 17:05:58.5844
90370	PTLC	2020-10-09	30.43	30.5908	30.4	30.56	297061	{}	\N	2020-10-12 17:05:58.589194
90371	PTLC	2020-10-12	30.79	31.1	30.75	31.091	89109	{}	\N	2020-10-12 17:05:58.594095
90372	QUS	2020-09-28	94.1531	94.59	93.9258	94.14	29331	{}	\N	2020-10-12 17:06:00.178171
90373	QUS	2020-09-29	94.17	94.4439	93.66	93.83	14694	{}	\N	2020-10-12 17:06:00.191854
90374	QUS	2020-09-30	94.4953	95.3006	94.04	94.63	37070	{}	\N	2020-10-12 17:06:00.199358
90375	QUS	2020-10-01	95.19	95.19	94.5698	94.9	20690	{}	\N	2020-10-12 17:06:00.206157
90376	QUS	2020-10-02	94.1653	94.9	93.9795	94.45	305778	{}	\N	2020-10-12 17:06:00.211749
90377	QUS	2020-10-05	95.17	95.96	95.08	95.9	25157	{}	\N	2020-10-12 17:06:00.217003
90378	QUS	2020-10-06	96.04	96.799	94.75	94.97	14137	{}	\N	2020-10-12 17:06:00.222253
90379	QUS	2020-10-07	96.02	96.56	95.825	96.3	305591	{}	\N	2020-10-12 17:06:00.227553
90380	QUS	2020-10-08	96.95	97.2199	96.7	97.19	283077	{}	\N	2020-10-12 17:06:00.232529
90381	QUS	2020-10-09	97.7	98.06	97.459	97.87	123639	{}	\N	2020-10-12 17:06:00.237605
90382	QUS	2020-10-12	98.46	99.21	98.46	99.21	3329	{}	\N	2020-10-12 17:06:00.243029
90383	REG	2020-09-28	38.59	39.07	38.19	38.56	924033	{}	\N	2020-10-12 17:06:01.791099
90384	REG	2020-09-29	38.25	38.6399	37	37.77	768578	{}	\N	2020-10-12 17:06:01.803552
90385	REG	2020-09-30	38.14	38.78	37.575	38.02	1226472	{}	\N	2020-10-12 17:06:01.811433
90386	REG	2020-10-01	38.05	39.41	38.05	39.4	1108705	{}	\N	2020-10-12 17:06:01.818328
90387	REG	2020-10-02	38.57	40.38	38.22	40.2	1186873	{}	\N	2020-10-12 17:06:01.82504
90388	REG	2020-10-05	40.62	40.8	39.455	40.11	872715	{}	\N	2020-10-12 17:06:01.830524
90389	REG	2020-10-06	40.48	40.85	39.755	39.95	877417	{}	\N	2020-10-12 17:06:01.835583
90390	REG	2020-10-07	40.24	40.34	39.165	39.43	669851	{}	\N	2020-10-12 17:06:01.841747
90391	REG	2020-10-08	39.73	40.47	39.58	40.44	1965699	{}	\N	2020-10-12 17:06:01.846964
90392	REG	2020-10-09	40.93	40.93	39.45	39.58	660575	{}	\N	2020-10-12 17:06:01.852855
90393	REG	2020-10-12	39.52	40.01	39.0349	39.51	202701	{}	\N	2020-10-12 17:06:01.857938
90394	RF	2020-09-28	11.27	11.53	11.2336	11.51	8951777	{}	\N	2020-10-12 17:06:03.421869
90395	RF	2020-09-29	11.41	11.57	11.19	11.37	14004437	{}	\N	2020-10-12 17:06:03.435417
90396	RF	2020-09-30	11.47	11.705	11.35	11.53	17840995	{}	\N	2020-10-12 17:06:03.443473
90397	RF	2020-10-01	11.54	11.655	11.38	11.53	14561440	{}	\N	2020-10-12 17:06:03.450755
90398	RF	2020-10-02	11.34	12.03	11.3	11.95	14986386	{}	\N	2020-10-12 17:06:03.457237
90399	RF	2020-10-05	12.12	12.58	12.1	12.57	15694282	{}	\N	2020-10-12 17:06:03.462568
90400	RF	2020-10-06	12.77	12.995	12.32	12.39	18951167	{}	\N	2020-10-12 17:06:03.468072
90401	RF	2020-10-07	12.61	12.93	12.54	12.71	15261176	{}	\N	2020-10-12 17:06:03.473203
90402	RF	2020-10-08	12.83	13.05	12.665	13.04	19337449	{}	\N	2020-10-12 17:06:03.478291
90403	RF	2020-10-09	13.06	13.13	12.7	12.79	12737346	{}	\N	2020-10-12 17:06:03.483504
90404	RF	2020-10-12	12.78	12.86	12.715	12.85	5537355	{}	\N	2020-10-12 17:06:03.488706
90405	RWL	2020-09-28	53.51	54.1206	53.51	53.9	25549	{}	\N	2020-10-12 17:06:05.129572
90406	RWL	2020-09-29	53.7473	53.84	53.2357	53.45	22572	{}	\N	2020-10-12 17:06:05.142256
90407	RWL	2020-09-30	53.68	54.47	53.68	53.93	34118	{}	\N	2020-10-12 17:06:05.150121
90408	RWL	2020-10-01	54.16	54.26	53.73	54.02	59747	{}	\N	2020-10-12 17:06:05.155888
90409	RWL	2020-10-02	53.46	54.2427	53.46	54.2427	18591	{}	\N	2020-10-12 17:06:05.161725
90410	RWL	2020-10-05	54.53	54.9627	54.53	54.88	11298	{}	\N	2020-10-12 17:06:05.166333
90411	RWL	2020-10-06	55.05	55.48	54.18	54.28	38465	{}	\N	2020-10-12 17:06:05.170832
90412	RWL	2020-10-07	54.9839	55.35	54.91	55.23	130113	{}	\N	2020-10-12 17:06:05.175364
90413	RWL	2020-10-08	55.44	55.98	55.44	55.98	280810	{}	\N	2020-10-12 17:06:05.179846
90414	RWL	2020-10-09	56.29	56.3245	55.9359	56.08	16439	{}	\N	2020-10-12 17:06:05.184298
90415	RWL	2020-10-12	56.2	56.7284	56.2	56.71	11851	{}	\N	2020-10-12 17:06:05.188752
90416	SE	2020-09-28	159	162	157.01	159.97	3774069	{}	\N	2020-10-12 17:06:08.438225
90417	SE	2020-09-29	159.41	161.51	154.39	155.15	4937500	{}	\N	2020-10-12 17:06:08.451741
90418	SE	2020-09-30	155.1	158.54	153.86	154.04	4573989	{}	\N	2020-10-12 17:06:08.462982
90419	SE	2020-10-01	157	160.44	154.52	160	6145926	{}	\N	2020-10-12 17:06:08.468315
90420	SE	2020-10-02	154.5	160.09	154.33	155.36	5943826	{}	\N	2020-10-12 17:06:08.473239
90421	SE	2020-10-05	158.88	162.05	155.21	157.58	6701341	{}	\N	2020-10-12 17:06:08.477566
90422	SE	2020-10-06	159	167.9397	158.13	163.44	11282556	{}	\N	2020-10-12 17:06:08.481535
90423	SE	2020-10-07	167.43	171.3	165.45	167.42	8530143	{}	\N	2020-10-12 17:06:08.485467
90424	SE	2020-10-08	170.49	172	168.15	169.29	6673348	{}	\N	2020-10-12 17:06:08.48991
90425	SE	2020-10-09	170	172.5	166.2801	172.12	5218990	{}	\N	2020-10-12 17:06:08.494704
90426	SE	2020-10-12	175	175.6	166.9006	169.0412	4268085	{}	\N	2020-10-12 17:06:08.499372
90427	SEED	2020-09-28	9.82	10.37	9.82	10.05	33732	{}	\N	2020-10-12 17:06:09.576511
90428	SEED	2020-09-29	10.05	10.2899	9.34	9.675	35569	{}	\N	2020-10-12 17:06:09.587501
90429	SEED	2020-09-30	9.68	10.19	9.0101	9.4	22365	{}	\N	2020-10-12 17:06:09.595288
90430	SEED	2020-10-01	9.7199	9.775	9.45	9.75	26325	{}	\N	2020-10-12 17:06:09.601131
90431	SEED	2020-10-02	9.1401	9.85	9.1401	9.7	36396	{}	\N	2020-10-12 17:06:09.606395
90432	SEED	2020-10-05	9.53	10.25	9.53	10	30207	{}	\N	2020-10-12 17:06:09.611144
90433	SEED	2020-10-06	10.01	10.2193	9.81	9.98	17535	{}	\N	2020-10-12 17:06:09.615824
90434	SEED	2020-10-07	10.12	10.24	9.895	10.14	22705	{}	\N	2020-10-12 17:06:09.620504
90435	SEED	2020-10-08	9.93	10.6	9.93	10.6	90943	{}	\N	2020-10-12 17:06:09.625464
90436	SEED	2020-10-09	10.6	10.6	10.101	10.43	60273	{}	\N	2020-10-12 17:06:09.630369
90437	SEED	2020-10-12	10.8	10.8	10.16	10.498	35021	{}	\N	2020-10-12 17:06:09.635162
90438	VRNS	2020-09-28	115.67	116.3979	114.35	115.13	202894	{}	\N	2020-10-12 17:06:10.869511
90439	VRNS	2020-09-29	115.32	117.5195	114.33	115.59	179554	{}	\N	2020-10-12 17:06:10.882692
90440	VRNS	2020-09-30	112.95	117.84	112.95	115.42	148128	{}	\N	2020-10-12 17:06:10.891158
90441	VRNS	2020-10-01	116.15	120.8	116.15	120.16	264243	{}	\N	2020-10-12 17:06:10.898472
90442	VRNS	2020-10-02	117.13	120.47	116.2501	116.87	228522	{}	\N	2020-10-12 17:06:10.904275
90443	VRNS	2020-10-05	117.79	120.975	117.02	119.68	269933	{}	\N	2020-10-12 17:06:10.908863
90444	VRNS	2020-10-06	120.17	123.16	119.03	119.45	212233	{}	\N	2020-10-12 17:06:10.913564
90445	VRNS	2020-10-07	120.31	124.13	120.31	123.19	311423	{}	\N	2020-10-12 17:06:10.91881
90446	VRNS	2020-10-08	124.84	125.45	122.08	122.86	191075	{}	\N	2020-10-12 17:06:10.923205
90447	VRNS	2020-10-09	126.98	127.225	124.37	125.16	309601	{}	\N	2020-10-12 17:06:10.927254
90448	VRNS	2020-10-12	126.04	128.4	125.7	126.89	86787	{}	\N	2020-10-12 17:06:10.932454
90449	VTI	2020-09-28	169.32	170.3	168.8902	169.99	3371150	{}	\N	2020-10-12 17:06:12.018734
90450	VTI	2020-09-29	169.84	170.229	168.69	169.04	2172401	{}	\N	2020-10-12 17:06:12.03117
90451	VTI	2020-09-30	169.51	171.9679	169.25	170.31	4833336	{}	\N	2020-10-12 17:06:12.039216
90452	VTI	2020-10-01	171.72	172.25	170.61	171.73	3208199	{}	\N	2020-10-12 17:06:12.044983
90453	VTI	2020-10-02	168.65	171.3073	168.23	170.39	2379567	{}	\N	2020-10-12 17:06:12.04984
90454	VTI	2020-10-05	171.61	173.53	171.33	173.46	2537007	{}	\N	2020-10-12 17:06:12.054703
90455	VTI	2020-10-06	173.73	174.98	171.01	171.32	2808982	{}	\N	2020-10-12 17:06:12.059205
90456	VTI	2020-10-07	172.99	174.729	172.94	174.31	1982630	{}	\N	2020-10-12 17:06:12.063592
90457	VTI	2020-10-08	175.53	175.89	174.88	175.84	2388395	{}	\N	2020-10-12 17:06:12.06806
90458	VTI	2020-10-09	176.9	177.6344	176.44	177.32	2495652	{}	\N	2020-10-12 17:06:12.072435
90459	VTI	2020-10-12	178.65	180.34	178.4125	180.0249	2049141	{}	\N	2020-10-12 17:06:12.077058
90460	VXF	2020-09-28	129.1	130.1999	128.96	129.91	258005	{}	\N	2020-10-12 17:06:13.314013
90461	VXF	2020-09-29	129.62	129.8666	128.525	128.94	258418	{}	\N	2020-10-12 17:06:13.324174
90462	VXF	2020-09-30	129.3	130.92	128.755	129.72	574433	{}	\N	2020-10-12 17:06:13.330549
90463	VXF	2020-10-01	130.84	132.25	130.32	132.25	391415	{}	\N	2020-10-12 17:06:13.336818
90464	VXF	2020-10-02	129.8	132.68	129.2731	131.99	347316	{}	\N	2020-10-12 17:06:13.342818
90465	VXF	2020-10-05	133.2	135.08	132.96	134.95	928219	{}	\N	2020-10-12 17:06:13.347548
90466	VXF	2020-10-06	135.73	137.33	133.74	134.27	1047299	{}	\N	2020-10-12 17:06:13.352183
90467	VXF	2020-10-07	135.69	137.19	135.69	136.89	489165	{}	\N	2020-10-12 17:06:13.356665
90468	VXF	2020-10-08	138.18	138.5	137.344	138.15	364775	{}	\N	2020-10-12 17:06:13.361265
90469	VXF	2020-10-09	139.09	139.62	138.64	139.22	288196	{}	\N	2020-10-12 17:06:13.36599
90470	VXF	2020-10-12	140.04	140.41	139.37	139.991	160050	{}	\N	2020-10-12 17:06:13.370731
90471	WCN	2020-09-28	104.08	105.87	103.83	105	793695	{}	\N	2020-10-12 17:06:16.140464
90472	WCN	2020-09-29	105	105.565	104.23	104.24	1283671	{}	\N	2020-10-12 17:06:16.151853
90473	WCN	2020-09-30	104.5	104.85	103.37	103.8	1282959	{}	\N	2020-10-12 17:06:16.159848
90474	WCN	2020-10-01	103.93	104.43	102.29	103	1413771	{}	\N	2020-10-12 17:06:16.166207
90475	WCN	2020-10-02	101.92	103.11	101.765	102.62	818606	{}	\N	2020-10-12 17:06:16.171553
90476	WCN	2020-10-05	103.12	103.27	102.03	102.32	1119841	{}	\N	2020-10-12 17:06:16.177975
90477	WCN	2020-10-06	102.53	103.54	101.91	102.37	1015121	{}	\N	2020-10-12 17:06:16.182835
90478	WCN	2020-10-07	102.8	104.06	102.4	103.72	803290	{}	\N	2020-10-12 17:06:16.187808
90479	WCN	2020-10-08	104.2	104.38	103.22	103.36	490952	{}	\N	2020-10-12 17:06:16.192436
90480	WCN	2020-10-09	103.93	104.24	103.275	103.76	932742	{}	\N	2020-10-12 17:06:16.197355
90481	WCN	2020-10-12	103.9	105.269	103.86	104.91	395941	{}	\N	2020-10-12 17:06:16.201898
90482	WERN	2020-09-28	43.14	43.63	42.495	43.43	788070	{}	\N	2020-10-12 17:06:17.152913
90483	WERN	2020-09-29	43.23	43.735	42.87	42.94	554540	{}	\N	2020-10-12 17:06:17.167335
90484	WERN	2020-09-30	43.19	43.19	41.62	41.99	792969	{}	\N	2020-10-12 17:06:17.175379
90485	WERN	2020-10-01	41.74	42.33	41.17	42.07	767480	{}	\N	2020-10-12 17:06:17.181213
90486	WERN	2020-10-02	41.72	43.2	41	42.95	702571	{}	\N	2020-10-12 17:06:17.186473
90487	WERN	2020-10-05	43.27	43.74	42.62	43.45	606816	{}	\N	2020-10-12 17:06:17.191478
90488	WERN	2020-10-06	43.55	43.74	42.91	43.03	582749	{}	\N	2020-10-12 17:06:17.196105
90489	WERN	2020-10-07	43.35	44.14	43.21	43.39	735691	{}	\N	2020-10-12 17:06:17.201028
90490	WERN	2020-10-08	43.79	44.12	43.445	44.06	661785	{}	\N	2020-10-12 17:06:17.206101
90491	WERN	2020-10-09	43.69	43.96	43.105	43.59	593030	{}	\N	2020-10-12 17:06:17.211333
90492	WERN	2020-10-12	43.8	44.44	43.46	43.46	158643	{}	\N	2020-10-12 17:06:17.216216
90493	WFC	2020-09-28	23.99	24.27	23.76	23.82	41837282	{}	\N	2020-10-12 17:06:18.282843
90494	WFC	2020-09-29	23.72	23.72	23.07	23.26	75764555	{}	\N	2020-10-12 17:06:18.293456
90495	WFC	2020-09-30	23.36	23.87	23.25	23.51	81271465	{}	\N	2020-10-12 17:06:18.300117
90496	WFC	2020-10-01	23.57	23.81	23.16	23.56	76675649	{}	\N	2020-10-12 17:06:18.307127
90497	WFC	2020-10-02	23.15	24.09	23.08	23.99	51172590	{}	\N	2020-10-12 17:06:18.312515
90498	WFC	2020-10-05	24.22	24.6	24.22	24.45	44422453	{}	\N	2020-10-12 17:06:18.317518
90499	WFC	2020-10-06	24.68	25	24.0402	24.18	58710128	{}	\N	2020-10-12 17:06:18.322686
90500	WFC	2020-10-07	24.42	24.93	24.39	24.81	69121270	{}	\N	2020-10-12 17:06:18.327601
90501	WFC	2020-10-08	24.88	25.325	24.67	25.3	53376850	{}	\N	2020-10-12 17:06:18.332604
90502	WFC	2020-10-09	25.39	25.51	25.01	25.3	54848169	{}	\N	2020-10-12 17:06:18.338344
90503	WFC	2020-10-12	25.2	25.55	25.15	25.425	25806131	{}	\N	2020-10-12 17:06:18.343613
90504	XLU	2020-09-28	58.98	59.46	58.77	58.87	9669479	{}	\N	2020-10-12 17:06:19.991986
90505	XLU	2020-09-29	59.05	59.4508	58.58	58.85	8844366	{}	\N	2020-10-12 17:06:20.004489
90506	XLU	2020-09-30	59.22	59.67	58.88	59.38	11846046	{}	\N	2020-10-12 17:06:20.012567
90507	XLU	2020-10-01	59.53	60.13	59.41	59.98	12191568	{}	\N	2020-10-12 17:06:20.018467
90508	XLU	2020-10-02	59.5	61	59.37	60.69	15711533	{}	\N	2020-10-12 17:06:20.023961
90509	XLU	2020-10-05	60.85	61.62	60.44	61.45	10820808	{}	\N	2020-10-12 17:06:20.028548
90510	XLU	2020-10-06	61.49	62.665	61.22	61.97	15212235	{}	\N	2020-10-12 17:06:20.032823
90511	XLU	2020-10-07	62.29	62.61	61.885	62.35	11552279	{}	\N	2020-10-12 17:06:20.037139
90512	XLU	2020-10-08	62.62	63.59	62.58	63.49	13583896	{}	\N	2020-10-12 17:06:20.041363
90513	XLU	2020-10-09	63.84	63.87	63.05	63.48	8082615	{}	\N	2020-10-12 17:06:20.045478
90514	XLU	2020-10-12	63.52	64.115	63.36	64.045	3815768	{}	\N	2020-10-12 17:06:20.049664
90515	YLDE	2020-09-28	32.63	32.63	32.4817	32.4817	330	{}	\N	2020-10-12 17:06:22.959783
90516	YLDE	2020-10-01	32.6882	32.6882	32.6481	32.675	1288	{}	\N	2020-10-12 17:06:22.967879
90517	YLDE	2020-10-02	32.6	32.6414	32.4893	32.4893	468	{}	\N	2020-10-12 17:06:22.973856
90518	YLDE	2020-10-05	32.87	32.919	32.87	32.919	311	{}	\N	2020-10-12 17:06:22.97907
90519	YLDE	2020-10-06	32.935	32.935	32.5974	32.5974	237	{}	\N	2020-10-12 17:06:22.98427
90520	YLDE	2020-10-08	33.19	33.325	33.16	33.325	1597	{}	\N	2020-10-12 17:06:22.988624
90521	ZROZ	2020-09-28	173.99	174.3062	173.11	173.24	24105	{}	\N	2020-10-12 17:06:24.609439
90522	ZROZ	2020-09-29	173.88	174.4	173.5	173.82	19038	{}	\N	2020-10-12 17:06:24.624073
90523	ZROZ	2020-09-30	172.65	172.65	170.36	171	74245	{}	\N	2020-10-12 17:06:24.632842
90524	ZROZ	2020-10-01	170	171.9686	169.5025	171.52	68124	{}	\N	2020-10-12 17:06:24.638576
90525	ZROZ	2020-10-02	171.55	171.5537	169.4	170.17	71183	{}	\N	2020-10-12 17:06:24.644288
90526	ZROZ	2020-10-05	167.58	168.1599	165.1241	165.53	141224	{}	\N	2020-10-12 17:06:24.650327
90527	ZROZ	2020-10-06	165	167.9	164.01	166.59	85639	{}	\N	2020-10-12 17:06:24.65601
90528	ZROZ	2020-10-07	165.34	167	164.3225	164.79	48485	{}	\N	2020-10-12 17:06:24.661685
90529	ZROZ	2020-10-08	165.7	166.39	165.3126	166.1	24986	{}	\N	2020-10-12 17:06:24.66763
90530	ZROZ	2020-10-09	165.13	166.3925	164.2501	165.88	44341	{}	\N	2020-10-12 17:06:24.67311
90531	ZROZ	2020-10-12	166.09	166.799	165.6869	166.275	21797	{}	\N	2020-10-12 17:06:24.677613
90532	VUSE	2020-09-28	28.6	28.96	28.6	28.9499	11224	{}	\N	2020-10-12 17:06:25.640227
90533	VUSE	2020-09-29	28.64	28.7539	28.5	28.65	6116	{}	\N	2020-10-12 17:06:25.652873
90534	VUSE	2020-09-30	28.9	28.9893	28.8	28.84	6796	{}	\N	2020-10-12 17:06:25.659798
90535	VUSE	2020-10-01	28.79	29.04	28.77	29.04	49459	{}	\N	2020-10-12 17:06:25.66628
90536	VUSE	2020-10-02	28.56	29.36	28.56	29.3096	7188	{}	\N	2020-10-12 17:06:25.671933
90537	VUSE	2020-10-05	29.7059	29.915	29.69	29.87	12148	{}	\N	2020-10-12 17:06:25.677075
90538	VUSE	2020-10-06	29.96	30.3699	29.75	29.75	15866	{}	\N	2020-10-12 17:06:25.68187
90539	VUSE	2020-10-07	29.88	30.34	29.88	30.34	10140	{}	\N	2020-10-12 17:06:25.686386
90540	VUSE	2020-10-08	30.4	30.6526	30.4	30.6526	14994	{}	\N	2020-10-12 17:06:25.690842
90541	VUSE	2020-10-09	30.6861	30.8341	30.65	30.6934	21325	{}	\N	2020-10-12 17:06:25.69524
90542	VUSE	2020-10-12	30.74	30.9578	30.74	30.9578	4145	{}	\N	2020-10-12 17:06:25.699465
90543	VWOB	2020-09-28	78.786	78.81	78.645	78.72	181414	{}	\N	2020-10-12 17:06:27.607078
90544	VWOB	2020-09-29	78.74	78.8	78.58	78.64	142295	{}	\N	2020-10-12 17:06:27.617646
90545	VWOB	2020-09-30	78.66	79.16	78.66	78.9	533695	{}	\N	2020-10-12 17:06:27.625618
90546	VWOB	2020-10-01	78.93	78.95	78.62	78.78	290478	{}	\N	2020-10-12 17:06:27.632048
90547	VWOB	2020-10-02	78.69	78.76	78.35	78.6	185919	{}	\N	2020-10-12 17:06:27.637359
90548	VWOB	2020-10-05	78.74	78.905	78.7	78.76	156488	{}	\N	2020-10-12 17:06:27.642312
90549	VWOB	2020-10-06	78.91	79.01	78.6966	78.71	248876	{}	\N	2020-10-12 17:06:27.647441
90550	VWOB	2020-10-07	79.02	79.15	78.8987	79.05	138385	{}	\N	2020-10-12 17:06:27.65282
90551	VWOB	2020-10-08	79.29	79.65	79.27	79.65	351297	{}	\N	2020-10-12 17:06:27.658623
90552	VWOB	2020-10-09	79.76	79.87	79.4	79.83	467730	{}	\N	2020-10-12 17:06:27.664685
90553	VWOB	2020-10-12	79.92	79.94	79.7998	79.94	64207	{}	\N	2020-10-12 17:06:27.6702
90554	WCC	2020-09-28	42.71	44.68	42.34	43.73	424118	{}	\N	2020-10-12 17:06:29.404454
90555	WCC	2020-09-29	43.89	43.89	42.69	43.53	659916	{}	\N	2020-10-12 17:06:29.414258
90556	WCC	2020-09-30	43.73	45.2	43.51	44.02	662700	{}	\N	2020-10-12 17:06:29.421429
90557	WCC	2020-10-01	44.34	44.71	43.62	44.67	978894	{}	\N	2020-10-12 17:06:29.427532
90558	WCC	2020-10-02	43.41	46.45	43.11	46.19	891481	{}	\N	2020-10-12 17:06:29.432748
90559	WCC	2020-10-05	46.89	48.3	46.85	47.82	1075370	{}	\N	2020-10-12 17:06:29.437581
90560	WCC	2020-10-06	48.54	48.675	46.53	46.81	945095	{}	\N	2020-10-12 17:06:29.442858
90561	WCC	2020-10-07	47.75	48.77	47.7182	48.35	998122	{}	\N	2020-10-12 17:06:29.447979
90562	WCC	2020-10-08	48.8	49.16	47.63	48.95	689249	{}	\N	2020-10-12 17:06:29.453435
90563	WCC	2020-10-09	49.78	49.78	48.04	48.09	635919	{}	\N	2020-10-12 17:06:29.458139
90564	WCC	2020-10-12	48.03	48.9	47.65	48.4067	431205	{}	\N	2020-10-12 17:06:29.463156
90565	WEA	2020-09-28	12.98	13.01	12.92	12.98	33149	{}	\N	2020-10-12 17:06:31.033298
90566	WEA	2020-09-29	13.01	13.01	12.85	12.95	105369	{}	\N	2020-10-12 17:06:31.044271
90567	WEA	2020-09-30	12.9	13.09	12.87	12.92	411318	{}	\N	2020-10-12 17:06:31.053143
90568	WEA	2020-10-01	12.96	13.0199	12.84	12.99	70120	{}	\N	2020-10-12 17:06:31.05964
90569	WEA	2020-10-02	12.89	12.97	12.86	12.97	37636	{}	\N	2020-10-12 17:06:31.065977
90570	WEA	2020-10-05	12.98	13.015	12.94	12.98	51198	{}	\N	2020-10-12 17:06:31.07115
90571	WEA	2020-10-06	12.96	13.05	12.93	13	64179	{}	\N	2020-10-12 17:06:31.076694
90572	WEA	2020-10-07	13.05	13.1	13.05	13.1	27202	{}	\N	2020-10-12 17:06:31.083656
90573	WEA	2020-10-08	13.12	13.19	13.07	13.19	53880	{}	\N	2020-10-12 17:06:31.089861
90574	WEA	2020-10-09	13.19	13.28	13.15	13.18	62267	{}	\N	2020-10-12 17:06:31.09605
90575	WEA	2020-10-12	13.16	13.29	13.16	13.25	44706	{}	\N	2020-10-12 17:06:31.102659
90576	WLDN	2020-09-28	28.06	29.38	27.375	28.51	52016	{}	\N	2020-10-12 17:06:32.337245
90577	WLDN	2020-09-29	28.54	29.31	28.07	28.44	156886	{}	\N	2020-10-12 17:06:32.34803
90578	WLDN	2020-09-30	28.08	28.56	25.42	25.51	114088	{}	\N	2020-10-12 17:06:32.355592
90579	WLDN	2020-10-01	25.76	27.385	25.64	27.02	84999	{}	\N	2020-10-12 17:06:32.36194
90580	WLDN	2020-10-02	26.4	27.72	26.4	26.9	38197	{}	\N	2020-10-12 17:06:32.366565
90581	WLDN	2020-10-05	27.3	27.52	26.4	26.7	33467	{}	\N	2020-10-12 17:06:32.371989
90582	WLDN	2020-10-06	26.42	27.415	26.1	26.6	39076	{}	\N	2020-10-12 17:06:32.3762
90583	WLDN	2020-10-07	26.77	28.32	26.77	28.15	108106	{}	\N	2020-10-12 17:06:32.380514
90584	WLDN	2020-10-08	28.37	29.25	27.945	28.74	26215	{}	\N	2020-10-12 17:06:32.384877
90585	WLDN	2020-10-09	28.6	29.1	28.08	28.41	25697	{}	\N	2020-10-12 17:06:32.389484
90586	WLDN	2020-10-12	28.4	28.4288	28.01	28.015	4596	{}	\N	2020-10-12 17:06:32.395007
90587	XNTK	2020-09-28	117.64	117.64	116.23	117.4232	15837	{}	\N	2020-10-12 17:06:33.850461
90588	XNTK	2020-09-29	117.34	118.46	117.186	117.83	30844	{}	\N	2020-10-12 17:06:33.859118
90589	XNTK	2020-09-30	118	119.7	118	118.71	17073	{}	\N	2020-10-12 17:06:33.867499
90590	XNTK	2020-10-01	120.71	121.17	120.0175	121.14	22883	{}	\N	2020-10-12 17:06:33.874925
90591	XNTK	2020-10-02	118.02	120.187	117.29	117.29	28746	{}	\N	2020-10-12 17:06:33.879824
90592	XNTK	2020-10-05	118.48	120.08	118.48	120.08	57616	{}	\N	2020-10-12 17:06:33.885004
90593	XNTK	2020-10-06	120.08	121.08	118.2601	118.6	23507	{}	\N	2020-10-12 17:06:33.890183
90594	XNTK	2020-10-07	120.03	121.43	120.03	121.279	18278	{}	\N	2020-10-12 17:06:33.895233
90595	XNTK	2020-10-08	122.58	122.58	121.54	121.88	10778	{}	\N	2020-10-12 17:06:33.900231
90596	XNTK	2020-10-09	122.76	123.36	122.5699	123.36	16748	{}	\N	2020-10-12 17:06:33.905119
90597	XNTK	2020-10-12	125.21	126.2285	124.7971	126.2285	16801	{}	\N	2020-10-12 17:06:33.909851
90598	VV	2020-09-28	155.57	156.31	155.15	155.93	198148	{}	\N	2020-10-12 17:06:35.971383
90599	VV	2020-09-29	155.96	156.23	154.93	155.24	165146	{}	\N	2020-10-12 17:06:35.984391
90600	VV	2020-09-30	155.62	157.92	155.51	156.44	359835	{}	\N	2020-10-12 17:06:35.993124
90601	VV	2020-10-01	157.79	158.14	156.64	157.61	225038	{}	\N	2020-10-12 17:06:35.999215
90602	VV	2020-10-02	154.86	156.899	154.8	156.06	265188	{}	\N	2020-10-12 17:06:36.004666
90603	VV	2020-10-05	157.11	158.81	157.11	158.67	170861	{}	\N	2020-10-12 17:06:36.010283
90604	VV	2020-10-06	158.82	159.8186	156.37	156.57	208786	{}	\N	2020-10-12 17:06:36.015062
90605	VV	2020-10-07	158.03	159.59	158.03	159.13	2200691	{}	\N	2020-10-12 17:06:36.019818
90606	VV	2020-10-08	160.26	160.59	159.81	160.59	128830	{}	\N	2020-10-12 17:06:36.024642
90607	VV	2020-10-09	161.41	162.25	161.1532	162.06	311363	{}	\N	2020-10-12 17:06:36.029973
90608	VV	2020-10-12	163.28	165	163.12	164.7069	85667	{}	\N	2020-10-12 17:06:36.035188
90609	WDFC	2020-09-28	191.4	192.1	189.2	189.44	61980	{}	\N	2020-10-12 17:06:38.415445
90610	WDFC	2020-09-29	188.94	189.93	186.5	188.18	69238	{}	\N	2020-10-12 17:06:38.425818
90611	WDFC	2020-09-30	188.63	190.99	188.63	189.59	86386	{}	\N	2020-10-12 17:06:38.43427
90612	WDFC	2020-10-01	189.41	189.41	183.55	187.4	107143	{}	\N	2020-10-12 17:06:38.441885
90613	WDFC	2020-10-02	185.89	188.5	185.13	187.8	60331	{}	\N	2020-10-12 17:06:38.448916
90614	WDFC	2020-10-05	188.66	190	187.65	189.9	49917	{}	\N	2020-10-12 17:06:38.454045
90615	WDFC	2020-10-06	191.29	194.51	189.59	193.97	80781	{}	\N	2020-10-12 17:06:38.4598
90616	WDFC	2020-10-07	194.67	195.9	193.095	195.12	87217	{}	\N	2020-10-12 17:06:38.465208
90617	WDFC	2020-10-08	196.68	198.31	195.845	196.77	72228	{}	\N	2020-10-12 17:06:38.470191
90618	WDFC	2020-10-09	196.41	198.865	196.41	198.22	61521	{}	\N	2020-10-12 17:06:38.476011
90619	WDFC	2020-10-12	197.93	199.71	197.555	199.71	10754	{}	\N	2020-10-12 17:06:38.481495
90620	WEC	2020-09-28	96.36	97.46	95.72	96.48	817333	{}	\N	2020-10-12 17:06:39.627573
90621	WEC	2020-09-29	96.99	97.54	96.14	96.66	1741979	{}	\N	2020-10-12 17:06:39.639278
90622	WEC	2020-09-30	97	97.61	96.375	96.9	2604097	{}	\N	2020-10-12 17:06:39.646589
90623	WEC	2020-10-01	96.99	98.31	96.87	98.25	1964167	{}	\N	2020-10-12 17:06:39.654485
90624	WEC	2020-10-02	97.66	98.315	96.755	97.71	2414012	{}	\N	2020-10-12 17:06:39.66054
90625	WEC	2020-10-05	97.93	98.455	96.82	98.14	2273252	{}	\N	2020-10-12 17:06:39.665555
90626	WEC	2020-10-06	98.15	99.58	97.33	98.51	2868915	{}	\N	2020-10-12 17:06:39.670274
90627	WEC	2020-10-07	98.78	99.48	97.46	98.05	1786499	{}	\N	2020-10-12 17:06:39.674565
90628	WEC	2020-10-08	98.43	99.09	98.16	99.03	2055414	{}	\N	2020-10-12 17:06:39.679222
90629	WEC	2020-10-09	99.42	99.88	98.615	99.44	1672854	{}	\N	2020-10-12 17:06:39.684335
90630	WEC	2020-10-12	99.6	100.94	99.33	100.5854	413072	{}	\N	2020-10-12 17:06:39.689828
90631	WPS	2020-09-28	31.47	31.5335	31.401	31.53	8001	{}	\N	2020-10-12 17:06:41.150303
90632	WPS	2020-09-29	31.32	31.34	31.1801	31.31	7000	{}	\N	2020-10-12 17:06:41.157859
90633	WPS	2020-09-30	31.31	31.51	31.31	31.41	27016	{}	\N	2020-10-12 17:06:41.163695
90634	WPS	2020-10-01	31.49	31.66	31.43	31.65	17684	{}	\N	2020-10-12 17:06:41.169942
90635	WPS	2020-10-02	31.63	31.82	31.63	31.67	56184	{}	\N	2020-10-12 17:06:41.175267
90636	WPS	2020-10-05	32.04	32.129	31.99	32.129	10050	{}	\N	2020-10-12 17:06:41.180619
90637	WPS	2020-10-06	32.11	32.1999	31.905	31.94	7298	{}	\N	2020-10-12 17:06:41.186176
90638	WPS	2020-10-07	32.03	32.0958	31.97	31.97	6740	{}	\N	2020-10-12 17:06:41.193428
90639	WPS	2020-10-08	32.0414	32.13	32.03	32.13	2428	{}	\N	2020-10-12 17:06:41.198627
90640	WPS	2020-10-09	32.07	32.1674	32.01	32.11	3953	{}	\N	2020-10-12 17:06:41.204875
90641	WPS	2020-10-12	32.16	32.2673	32.16	32.2	2337	{}	\N	2020-10-12 17:06:41.210831
90642	XES	2020-09-28	28.05	29.26	28.05	28.59	95192	{}	\N	2020-10-12 17:06:42.424699
90643	XES	2020-09-29	28.43	28.43	27.41	28.26	56128	{}	\N	2020-10-12 17:06:42.43794
90644	XES	2020-09-30	28.32	29.1	27.65	27.96	53123	{}	\N	2020-10-12 17:06:42.446063
90645	XES	2020-10-01	27.78	27.78	26.84	27.28	47676	{}	\N	2020-10-12 17:06:42.452618
90646	XES	2020-10-02	26.32	28.2	26.225	28	63414	{}	\N	2020-10-12 17:06:42.458562
90647	XES	2020-10-05	28.56	29.0227	28.37	28.9	57870	{}	\N	2020-10-12 17:06:42.464117
90648	XES	2020-10-06	29.43	29.93	28.4902	28.52	49328	{}	\N	2020-10-12 17:06:42.468858
90649	XES	2020-10-07	28.68	28.84	28.03	28.69	25655	{}	\N	2020-10-12 17:06:42.473918
90650	XES	2020-10-08	29.16	30.32	28.95	30.22	48987	{}	\N	2020-10-12 17:06:42.479185
90651	XES	2020-10-09	30.43	30.6357	29.4709	29.71	51831	{}	\N	2020-10-12 17:06:42.484229
90652	XES	2020-10-12	29.6	29.62	28.982	29.22	14752	{}	\N	2020-10-12 17:06:42.490081
90653	YANG	2020-09-28	25.95	26.23	25.85	25.85	241070	{}	\N	2020-10-12 17:06:43.486623
90654	YANG	2020-09-29	26.49	26.49	26	26.2	178472	{}	\N	2020-10-12 17:06:43.499103
90655	YANG	2020-09-30	25.28	25.34	24.49	24.63	545328	{}	\N	2020-10-12 17:06:43.506974
90656	YANG	2020-10-01	23.91	24.28	23.6	23.72	315364	{}	\N	2020-10-12 17:06:43.513146
90657	YANG	2020-10-02	24.86	24.86	23.75	24.52	473089	{}	\N	2020-10-12 17:06:43.519651
90658	YANG	2020-10-05	24.69	24.75	24.19	24.2	353773	{}	\N	2020-10-12 17:06:43.525771
90659	YANG	2020-10-06	23.73	23.93	23.33	23.73	554016	{}	\N	2020-10-12 17:06:43.53094
90660	YANG	2020-10-07	23.12	23.34	22.85	22.98	419172	{}	\N	2020-10-12 17:06:43.536583
90661	YANG	2020-10-08	22.9	23.03	22.58	22.61	244193	{}	\N	2020-10-12 17:06:43.541881
90662	YANG	2020-10-09	22.88	22.88	22.32	22.46	211622	{}	\N	2020-10-12 17:06:43.546417
90663	YANG	2020-10-12	20.94	21.06	20.63	20.685	263453	{}	\N	2020-10-12 17:06:43.550951
90664	WBIF	2020-09-28	25.93	26.018	25.93	26.018	1442	{}	\N	2020-10-12 17:06:45.385016
90665	WBIF	2020-09-29	26.01	26.04	25.9861	25.9861	5079	{}	\N	2020-10-12 17:06:45.396487
90666	WBIF	2020-09-30	26.12	26.2455	26.0503	26.0503	5592	{}	\N	2020-10-12 17:06:45.406978
90667	WBIF	2020-10-01	26.181	26.27	26.18	26.2019	2812	{}	\N	2020-10-12 17:06:45.414329
90668	WBIF	2020-10-02	25.99	26.1829	25.99	26.15	2677	{}	\N	2020-10-12 17:06:45.420536
90669	WBIF	2020-10-05	26.2744	26.37	26.2744	26.37	3940	{}	\N	2020-10-12 17:06:45.425603
90670	WBIF	2020-10-06	26.44	26.44	26.1267	26.1267	5953	{}	\N	2020-10-12 17:06:45.430199
90671	WBIF	2020-10-07	26.35	26.38	26.27	26.38	11358	{}	\N	2020-10-12 17:06:45.436558
90672	WBIF	2020-10-08	26.53	26.6414	26.53	26.6414	1153	{}	\N	2020-10-12 17:06:45.441286
90673	WBIF	2020-10-09	26.67	26.81	26.67	26.81	2013	{}	\N	2020-10-12 17:06:45.447012
90674	WBIF	2020-10-12	26.935	26.935	26.935	26.935	163	{}	\N	2020-10-12 17:06:45.452935
90675	WEAT	2020-09-28	5.42	5.5	5.38	5.49	133566	{}	\N	2020-10-12 17:06:46.419678
90676	WEAT	2020-09-29	5.49	5.5099	5.45	5.485	145239	{}	\N	2020-10-12 17:06:46.431491
90677	WEAT	2020-09-30	5.52	5.79	5.52	5.71	328592	{}	\N	2020-10-12 17:06:46.44054
90678	WEAT	2020-10-01	5.72	5.72	5.64	5.66	194609	{}	\N	2020-10-12 17:06:46.447295
90679	WEAT	2020-10-02	5.59	5.6938	5.59	5.68	104858	{}	\N	2020-10-12 17:06:46.454232
90680	WEAT	2020-10-05	5.68	5.78	5.68	5.77	168664	{}	\N	2020-10-12 17:06:46.459981
90681	WEAT	2020-10-06	5.76	5.9084	5.76	5.81	310833	{}	\N	2020-10-12 17:06:46.465023
90682	WEAT	2020-10-07	5.87	5.97	5.87	5.94	200466	{}	\N	2020-10-12 17:06:46.470699
90683	WEAT	2020-10-08	5.99	6	5.82	5.83	528503	{}	\N	2020-10-12 17:06:46.475949
90684	WEAT	2020-10-09	5.85	5.91	5.78	5.83	320308	{}	\N	2020-10-12 17:06:46.480454
90685	WEAT	2020-10-12	5.83	5.85	5.76	5.81	288971	{}	\N	2020-10-12 17:06:46.485328
90686	WRB	2020-09-28	61.63	62.2925	61.34	61.62	666686	{}	\N	2020-10-12 17:06:48.486973
90687	WRB	2020-09-29	61.44	61.76	60.115	60.97	1300697	{}	\N	2020-10-12 17:06:48.500267
90688	WRB	2020-09-30	61.36	61.92	60.62	61.15	1625303	{}	\N	2020-10-12 17:06:48.508624
90689	WRB	2020-10-01	61.28	61.57	60.17	60.79	1002221	{}	\N	2020-10-12 17:06:48.515748
90690	WRB	2020-10-02	59.91	61.7	59.91	61.27	730513	{}	\N	2020-10-12 17:06:48.521772
90691	WRB	2020-10-05	61.93	62.31	61.66	62.04	831764	{}	\N	2020-10-12 17:06:48.528133
90692	WRB	2020-10-06	62.34	62.69	61.24	61.54	1040416	{}	\N	2020-10-12 17:06:48.533818
90693	WRB	2020-10-07	61.9	62.47	61.21	61.81	898871	{}	\N	2020-10-12 17:06:48.539662
90694	WRB	2020-10-08	62.16	63.22	61.91	63.18	696791	{}	\N	2020-10-12 17:06:48.546103
90695	WRB	2020-10-09	63.61	63.95	63	63.02	1287077	{}	\N	2020-10-12 17:06:48.551418
90696	WRB	2020-10-12	63.1	64.13	62.74	64.095	254466	{}	\N	2020-10-12 17:06:48.557537
90697	WWD	2020-09-28	77.8	80.19	77.37	78.82	269541	{}	\N	2020-10-12 17:06:50.383248
90698	WWD	2020-09-29	78.86	80.07	78.13	78.55	324353	{}	\N	2020-10-12 17:06:50.391977
90699	WWD	2020-09-30	78.83	81.17	78.83	80.16	532972	{}	\N	2020-10-12 17:06:50.399085
90700	WWD	2020-10-01	80.66	81.8	79.74	81.03	284297	{}	\N	2020-10-12 17:06:50.405041
90701	WWD	2020-10-02	78.36	82.315	78.195	81.85	179753	{}	\N	2020-10-12 17:06:50.410022
90702	WWD	2020-10-05	82.89	84.53	82.7	83.95	150298	{}	\N	2020-10-12 17:06:50.414946
90703	WWD	2020-10-06	84.81	85.92	82.53	82.84	232792	{}	\N	2020-10-12 17:06:50.419682
90704	WWD	2020-10-07	83.77	85.62	83.42	84.82	185961	{}	\N	2020-10-12 17:06:50.424628
90705	WWD	2020-10-08	85.18	86.49	84.07	86.2	159610	{}	\N	2020-10-12 17:06:50.429536
90706	WWD	2020-10-09	87.39	87.39	85.41	86.74	198902	{}	\N	2020-10-12 17:06:50.434761
90707	WWD	2020-10-12	87.35	87.35	85.97	86.79	53411	{}	\N	2020-10-12 17:06:50.439873
90708	XAR	2020-09-28	88.49	90.4984	88.49	89.55	80600	{}	\N	2020-10-12 17:06:52.041637
90709	XAR	2020-09-29	89.38	89.39	88.03	88.58	76797	{}	\N	2020-10-12 17:06:52.051742
90710	XAR	2020-09-30	88.37	89.605	86.62	87.34	115229	{}	\N	2020-10-12 17:06:52.059907
90711	XAR	2020-10-01	87.67	89.2212	87.67	88.59	110913	{}	\N	2020-10-12 17:06:52.065905
90712	XAR	2020-10-02	86.49	89.61	86.12	89.36	75251	{}	\N	2020-10-12 17:06:52.071663
90713	XAR	2020-10-05	90	90.88	89.61	90.02	116960	{}	\N	2020-10-12 17:06:52.077006
90714	XAR	2020-10-06	90.71	91.87	88.97	89.05	76955	{}	\N	2020-10-12 17:06:52.082686
90715	XAR	2020-10-07	90.21	91.3999	90.21	90.99	132902	{}	\N	2020-10-12 17:06:52.087915
90716	XAR	2020-10-08	91.52	92.72	91.38	92.72	51571	{}	\N	2020-10-12 17:06:52.096502
90717	XAR	2020-10-09	93.18	93.3824	92.1	92.1	71661	{}	\N	2020-10-12 17:06:52.102124
90718	XAR	2020-10-12	92.33	92.69	91.7	92.45	42734	{}	\N	2020-10-12 17:06:52.107201
90719	XLG	2020-09-28	260.88	261.02	259.52	260.91	42540	{}	\N	2020-10-12 17:06:53.975351
90720	XLG	2020-09-29	260.98	261.0185	259.26	259.85	58359	{}	\N	2020-10-12 17:06:53.986938
90721	XLG	2020-09-30	260.11	264.515	260.11	261.94	59886	{}	\N	2020-10-12 17:06:53.994535
90722	XLG	2020-10-01	264.76	264.76	262.54	264.0364	37118	{}	\N	2020-10-12 17:06:54.000133
90723	XLG	2020-10-02	259.3	261.96	258.791	259.3	61321	{}	\N	2020-10-12 17:06:54.00569
90724	XLG	2020-10-05	261.19	264.01	261.08	264.01	25416	{}	\N	2020-10-12 17:06:54.01091
90725	XLG	2020-10-06	263.89	264.6988	259.0119	259.5	30108	{}	\N	2020-10-12 17:06:54.015225
90726	XLG	2020-10-07	262	264.46	261.73	264.114	27095	{}	\N	2020-10-12 17:06:54.019567
90727	XLG	2020-10-08	265.66	265.67	264.9178	265.54	26032	{}	\N	2020-10-12 17:06:54.023634
90728	XLG	2020-10-09	266.61	268.79	266.61	268.79	24119	{}	\N	2020-10-12 17:06:54.02758
90729	XLG	2020-10-12	271.4	275.36	271.4	275.36	14544	{}	\N	2020-10-12 17:06:54.032385
90730	XSPA	2020-09-28	1.91	2.09	1.86	1.98	5235378	{}	\N	2020-10-12 17:06:55.317079
90731	XSPA	2020-09-29	1.9	1.9589	1.89	1.91	2133153	{}	\N	2020-10-12 17:06:55.326086
90732	XSPA	2020-09-30	1.87	1.98	1.87	1.97	2566209	{}	\N	2020-10-12 17:06:55.33372
90733	XSPA	2020-10-01	1.95	1.97	1.91	1.95	1803689	{}	\N	2020-10-12 17:06:55.339584
90734	XSPA	2020-10-02	1.87	1.96	1.86	1.92	1966718	{}	\N	2020-10-12 17:06:55.34579
90735	XSPA	2020-10-05	1.9442	2.39	1.92	2.26	22686406	{}	\N	2020-10-12 17:06:55.350797
90736	XSPA	2020-10-06	2.43	2.89	2.27	2.53	37102540	{}	\N	2020-10-12 17:06:55.355198
90737	XSPA	2020-10-07	2.5471	2.555	2.34	2.39	7434877	{}	\N	2020-10-12 17:06:55.360616
90738	XSPA	2020-10-08	2.43	2.44	2.34	2.37	4553197	{}	\N	2020-10-12 17:06:55.365782
90739	XSPA	2020-10-09	2.45	2.48	2.32	2.35	4051393	{}	\N	2020-10-12 17:06:55.370759
90740	XSPA	2020-10-12	2.33	2.35	2.22	2.22	2272901	{}	\N	2020-10-12 17:06:55.375183
90741	YINN	2020-09-28	13.95	13.99	13.8	13.97	1283493	{}	\N	2020-10-12 17:06:56.389197
90742	YINN	2020-09-29	13.61	13.91	13.6	13.79	651625	{}	\N	2020-10-12 17:06:56.397102
90743	YINN	2020-09-30	14.27	14.69	14.25	14.57	1563263	{}	\N	2020-10-12 17:06:56.403271
90744	YINN	2020-10-01	15.02	15.23	14.8272	15.15	1426442	{}	\N	2020-10-12 17:06:56.409379
90745	YINN	2020-10-02	14.44	15.12	14.44	14.65	1641358	{}	\N	2020-10-12 17:06:56.414963
90746	YINN	2020-10-05	14.55	14.85	14.5	14.85	1361537	{}	\N	2020-10-12 17:06:56.420017
90747	YINN	2020-10-06	15.15	15.36	14.99	15.12	1820368	{}	\N	2020-10-12 17:06:56.424636
90748	YINN	2020-10-07	15.55	15.7	15.36	15.62	1706099	{}	\N	2020-10-12 17:06:56.43139
90749	YINN	2020-10-08	15.65	15.8719	15.56	15.84	955341	{}	\N	2020-10-12 17:06:56.436464
90750	YINN	2020-10-09	15.68	16.05	15.65	15.96	830808	{}	\N	2020-10-12 17:06:56.441734
90751	YINN	2020-10-12	17.01	17.245	16.9387	17.21	1390415	{}	\N	2020-10-12 17:06:56.447002
90752	YPF	2020-09-28	3.69	3.84	3.675	3.72	1089345	{}	\N	2020-10-12 17:06:58.247257
90753	YPF	2020-09-29	3.75	3.75	3.56	3.61	3703233	{}	\N	2020-10-12 17:06:58.259183
90754	YPF	2020-09-30	3.63	3.805	3.57	3.57	2481605	{}	\N	2020-10-12 17:06:58.268935
90755	YPF	2020-10-01	3.56	3.7	3.53	3.65	1888698	{}	\N	2020-10-12 17:06:58.275635
90756	YPF	2020-10-02	3.58	3.7	3.535	3.63	1868463	{}	\N	2020-10-12 17:06:58.282023
90757	YPF	2020-10-05	3.67	3.77	3.61	3.73	2552298	{}	\N	2020-10-12 17:06:58.287751
90758	YPF	2020-10-06	3.79	3.86	3.56	3.57	2737678	{}	\N	2020-10-12 17:06:58.292673
90759	YPF	2020-10-07	3.59	3.72	3.51	3.66	1999837	{}	\N	2020-10-12 17:06:58.297926
90760	YPF	2020-10-08	3.66	3.82	3.63	3.81	1667034	{}	\N	2020-10-12 17:06:58.303144
90761	YPF	2020-10-09	3.89	3.895	3.77	3.87	1768431	{}	\N	2020-10-12 17:06:58.30884
90762	YPF	2020-10-12	3.9	3.9	3.72	3.785	621474	{}	\N	2020-10-12 17:06:58.314329
90763	WBIY	2020-09-28	18.6105	18.7696	18.6105	18.7696	2201	{}	\N	2020-10-12 17:06:59.708942
90764	WBIY	2020-09-29	18.586	18.586	18.36	18.5143	5020	{}	\N	2020-10-12 17:06:59.720549
90765	WBIY	2020-09-30	18.719	18.719	18.52	18.52	15451	{}	\N	2020-10-12 17:06:59.729102
90766	WBIY	2020-10-01	18.5946	18.5946	18.434	18.4566	4049	{}	\N	2020-10-12 17:06:59.735214
90767	WBIY	2020-10-02	18.03	18.7906	18.03	18.72	13557	{}	\N	2020-10-12 17:06:59.741047
90768	WBIY	2020-10-05	18.97	19.178	18.97	19.178	4055	{}	\N	2020-10-12 17:06:59.749034
90769	WBIY	2020-10-06	19.4	19.415	19.1118	19.1118	8201	{}	\N	2020-10-12 17:06:59.7549
90770	WBIY	2020-10-07	19.22	19.47	19.22	19.47	91761	{}	\N	2020-10-12 17:06:59.760347
90771	WBIY	2020-10-08	19.71	19.7894	19.71	19.7894	1251	{}	\N	2020-10-12 17:06:59.765228
90772	WBIY	2020-10-09	20.02	20.02	19.68	19.72	5242	{}	\N	2020-10-12 17:06:59.770066
90773	WBIY	2020-10-12	19.75	19.7666	19.75	19.7666	312	{}	\N	2020-10-12 17:06:59.775117
90774	WD	2020-09-28	50.3	51.63	50.3	51	236459	{}	\N	2020-10-12 17:07:01.105498
90775	WD	2020-09-29	51.03	51.359	50.15	51.13	349540	{}	\N	2020-10-12 17:07:01.116769
90776	WD	2020-09-30	51.54	53.33	51.54	53	611928	{}	\N	2020-10-12 17:07:01.124878
90777	WD	2020-10-01	53	54.89	52.67	54.78	547029	{}	\N	2020-10-12 17:07:01.132082
90778	WD	2020-10-02	53.53	57.75	53.14	57.7	646320	{}	\N	2020-10-12 17:07:01.138056
90779	WD	2020-10-05	58.44	59.1414	57.54	58.55	609922	{}	\N	2020-10-12 17:07:01.143206
90780	WD	2020-10-06	59.6	61.07	58.225	58.88	750570	{}	\N	2020-10-12 17:07:01.148726
90781	WD	2020-10-07	60.19	61	58.28	58.64	355714	{}	\N	2020-10-12 17:07:01.153919
90782	WD	2020-10-08	59.38	60.045	58.37	59.5	390248	{}	\N	2020-10-12 17:07:01.158692
90783	WD	2020-10-09	60.02	60.565	58.7242	59.15	279130	{}	\N	2020-10-12 17:07:01.163357
90784	WD	2020-10-12	59.5	59.86	58.9784	58.9784	94126	{}	\N	2020-10-12 17:07:01.168084
90785	WWE	2020-09-28	39.8	40.36	39.21	39.87	523790	{}	\N	2020-10-12 17:07:02.459177
90786	WWE	2020-09-29	39.95	40.15	39.44	40.03	812303	{}	\N	2020-10-12 17:07:02.47122
90787	WWE	2020-09-30	40.64	41.09	40.11	40.47	1861315	{}	\N	2020-10-12 17:07:02.479035
90788	WWE	2020-10-01	40.84	41.22	39.97	40.37	1409713	{}	\N	2020-10-12 17:07:02.485839
90789	WWE	2020-10-02	39.89	40.5	39.34	40.16	1710467	{}	\N	2020-10-12 17:07:02.491725
90790	WWE	2020-10-05	40.43	40.48	38.945	39.22	1025010	{}	\N	2020-10-12 17:07:02.496873
90791	WWE	2020-10-06	39.76	40.9	39.47	39.76	1209080	{}	\N	2020-10-12 17:07:02.502125
90792	WWE	2020-10-07	39.94	40.44	39.38	40.08	1087289	{}	\N	2020-10-12 17:07:02.507216
90793	WWE	2020-10-08	40.33	40.54	39.55	39.85	579928	{}	\N	2020-10-12 17:07:02.512479
90794	WWE	2020-10-09	39.96	40.335	39.51	40.16	676752	{}	\N	2020-10-12 17:07:02.517405
90795	WWE	2020-10-12	40.42	40.81	40.07	40.16	326980	{}	\N	2020-10-12 17:07:02.522596
90796	XBI	2020-09-28	111.75	111.85	109.87	110.94	3098384	{}	\N	2020-10-12 17:07:03.763918
90797	XBI	2020-09-29	110.77	112.34	110.1	111.69	2624144	{}	\N	2020-10-12 17:07:03.773184
90798	XBI	2020-09-30	111.72	113.23	110.46	111.43	3156665	{}	\N	2020-10-12 17:07:03.781001
90799	XBI	2020-10-01	111.96	112.94	111.1	112.83	3785847	{}	\N	2020-10-12 17:07:03.787271
90800	XBI	2020-10-02	111.08	113.06	109.82	110.36	3796677	{}	\N	2020-10-12 17:07:03.792744
90801	XBI	2020-10-05	111.91	116.02	111.91	115.81	4436615	{}	\N	2020-10-12 17:07:03.797739
90802	XBI	2020-10-06	116.02	117.32	114.44	114.89	3286619	{}	\N	2020-10-12 17:07:03.802543
90803	XBI	2020-10-07	116.05	119.1098	115.77	118.57	3761410	{}	\N	2020-10-12 17:07:03.807691
90804	XBI	2020-10-08	119.95	119.95	118.17	119.15	2677237	{}	\N	2020-10-12 17:07:03.813522
90805	XBI	2020-10-09	119.6	120.51	118.87	120.25	2541768	{}	\N	2020-10-12 17:07:03.8195
90806	XBI	2020-10-12	121.31	121.41	119.8	120.34	1204293	{}	\N	2020-10-12 17:07:03.824707
90807	XENT	2020-09-28	16.49	16.88	16.105	16.55	301289	{}	\N	2020-10-12 17:07:04.861191
90808	XENT	2020-09-29	16.55	16.63	16.115	16.44	202313	{}	\N	2020-10-12 17:07:04.871128
90809	XENT	2020-09-30	16.51	16.75	16.175	16.31	264737	{}	\N	2020-10-12 17:07:04.878501
90810	XENT	2020-10-01	16.34	16.37	15.9	16.14	234700	{}	\N	2020-10-12 17:07:04.884002
90811	XENT	2020-10-02	15.91	16.41	15.9	16.19	224292	{}	\N	2020-10-12 17:07:04.889342
90812	XENT	2020-10-05	16.4	17.06	16.35	17.06	277095	{}	\N	2020-10-12 17:07:04.893773
90813	XENT	2020-10-06	17.25	17.73	17.05	17.08	140433	{}	\N	2020-10-12 17:07:04.898241
90814	XENT	2020-10-07	17.19	18.28	17.19	18.01	225732	{}	\N	2020-10-12 17:07:04.902516
90815	XENT	2020-10-08	17.29	18.47	17.12	18.31	388354	{}	\N	2020-10-12 17:07:04.907424
90816	XENT	2020-10-09	18.53	18.53	17.685	17.95	189444	{}	\N	2020-10-12 17:07:04.911634
90817	XENT	2020-10-12	18.13	18.25	17.765	18.12	34105	{}	\N	2020-10-12 17:07:04.915644
90818	XWEB	2020-09-28	119.16	119.16	117.3384	118.67	2463	{}	\N	2020-10-12 17:07:06.113753
90819	XWEB	2020-09-29	119.2801	119.7018	118.3329	119.15	1396	{}	\N	2020-10-12 17:07:06.124668
90820	XWEB	2020-09-30	119.5484	119.5484	117.43	118.31	8454	{}	\N	2020-10-12 17:07:06.133004
90821	XWEB	2020-10-01	120	122.4	120	122.11	7073	{}	\N	2020-10-12 17:07:06.139825
90822	XWEB	2020-10-02	118.92	121.899	118.92	121.5087	7888	{}	\N	2020-10-12 17:07:06.14609
90823	XWEB	2020-10-05	122.38	123.7591	121.92	123.7591	3690	{}	\N	2020-10-12 17:07:06.152607
90824	XWEB	2020-10-06	124.84	124.94	122.19	122.19	5745	{}	\N	2020-10-12 17:07:06.158011
90825	XWEB	2020-10-07	124.51	125.3883	124.4343	125.3883	6980	{}	\N	2020-10-12 17:07:06.162551
90826	XWEB	2020-10-08	125.89	126.12	125.01	125.06	4721	{}	\N	2020-10-12 17:07:06.167465
90827	XWEB	2020-10-09	126.58	126.99	126.565	126.7566	4861	{}	\N	2020-10-12 17:07:06.172427
90828	XWEB	2020-10-12	128.96	128.97	128.6	128.6	8656	{}	\N	2020-10-12 17:07:06.177245
90829	YRCW	2020-09-28	3.89	4.095	3.79	3.91	1425105	{}	\N	2020-10-12 17:07:07.796697
90830	YRCW	2020-09-29	3.92	4.065	3.81	3.82	714766	{}	\N	2020-10-12 17:07:07.809121
90831	YRCW	2020-09-30	3.83	4.21	3.83	3.92	1388451	{}	\N	2020-10-12 17:07:07.816986
90832	YRCW	2020-10-01	3.96	4.17	3.8519	4.125	1106799	{}	\N	2020-10-12 17:07:07.822825
90833	YRCW	2020-10-02	3.95	4.29	3.95	4.24	1309023	{}	\N	2020-10-12 17:07:07.82772
90834	YRCW	2020-10-05	4.25	4.54	4.24	4.45	1661282	{}	\N	2020-10-12 17:07:07.832515
90835	YRCW	2020-10-06	4.49	4.82	4.49	4.53	1956789	{}	\N	2020-10-12 17:07:07.83642
90836	YRCW	2020-10-07	4.68	5.13	4.65	4.8	2352241	{}	\N	2020-10-12 17:07:07.84092
90837	YRCW	2020-10-08	5.01	5.07	4.82	4.9	1048351	{}	\N	2020-10-12 17:07:07.844923
90838	YRCW	2020-10-09	4.9	5.03	4.68	4.99	1374172	{}	\N	2020-10-12 17:07:07.848842
90839	YRCW	2020-10-12	5.01	5.405	5	5.1499	1889114	{}	\N	2020-10-12 17:07:07.852734
90840	YYY	2020-09-28	14.75	14.75	14.6101	14.63	91296	{}	\N	2020-10-12 17:07:10.117672
90841	YYY	2020-09-29	14.6335	14.64	14.55	14.6	57373	{}	\N	2020-10-12 17:07:10.128423
90842	YYY	2020-09-30	14.62	14.7	14.6	14.66	148385	{}	\N	2020-10-12 17:07:10.135393
90843	YYY	2020-10-01	14.74	14.74	14.6232	14.71	99305	{}	\N	2020-10-12 17:07:10.140712
90844	YYY	2020-10-02	14.55	14.71	14.55	14.69	101054	{}	\N	2020-10-12 17:07:10.14602
90845	YYY	2020-10-05	14.69	14.8499	14.69	14.82	83124	{}	\N	2020-10-12 17:07:10.150722
90846	YYY	2020-10-06	14.83	14.905	14.7933	14.81	75447	{}	\N	2020-10-12 17:07:10.154798
90847	YYY	2020-10-07	14.85	14.92	14.85	14.87	140112	{}	\N	2020-10-12 17:07:10.158971
90848	YYY	2020-10-08	14.9	14.97	14.9	14.96	88235	{}	\N	2020-10-12 17:07:10.162979
90849	YYY	2020-10-09	14.985	15.01	14.9701	14.99	59219	{}	\N	2020-10-12 17:07:10.166949
90850	YYY	2020-10-12	14.97	15.05	14.97	15.04	62181	{}	\N	2020-10-12 17:07:10.171121
90851	WDIV	2020-09-28	53.11	53.5118	53.11	53.43	29297	{}	\N	2020-10-12 17:07:14.936636
90852	WDIV	2020-09-29	53.29	53.3	52.815	53.05	14155	{}	\N	2020-10-12 17:07:14.951721
90853	WDIV	2020-09-30	53.38	53.525	52.96	53.15	175346	{}	\N	2020-10-12 17:07:14.958604
90854	WDIV	2020-10-01	53.48	53.5098	53.16	53.49	190851	{}	\N	2020-10-12 17:07:14.963676
90855	WDIV	2020-10-02	52.99	53.81	52.99	53.74	173180	{}	\N	2020-10-12 17:07:14.969486
90856	WDIV	2020-10-05	54.17	54.47	54.1	54.47	203779	{}	\N	2020-10-12 17:07:14.973545
90857	WDIV	2020-10-06	54.56	54.76	54.03	54.05	227854	{}	\N	2020-10-12 17:07:14.977817
90858	WDIV	2020-10-07	54.56	54.63	54.371	54.63	172671	{}	\N	2020-10-12 17:07:14.982097
90859	WDIV	2020-10-08	54.845	55.15	54.845	55.08	31987	{}	\N	2020-10-12 17:07:14.986125
90860	WDIV	2020-10-09	55.24	55.29	54.95	55.0588	22273	{}	\N	2020-10-12 17:07:14.99018
90861	WDIV	2020-10-12	55.2	55.4	55.12	55.4	6745	{}	\N	2020-10-12 17:07:14.994014
90862	WYNN	2020-09-28	73.11	74.57	72.43	73.62	3152883	{}	\N	2020-10-12 17:07:16.226768
90863	WYNN	2020-09-29	72.98	73.09	70.55	70.98	4183213	{}	\N	2020-10-12 17:07:16.239324
90864	WYNN	2020-09-30	71.94	73.94	71.18	71.81	4362574	{}	\N	2020-10-12 17:07:16.247659
90865	WYNN	2020-10-01	72.14	72.715	71.11	72.07	4140426	{}	\N	2020-10-12 17:07:16.25465
90866	WYNN	2020-10-02	70	73.44	69.6	73.2	3036366	{}	\N	2020-10-12 17:07:16.260402
90867	WYNN	2020-10-05	73.19	74.07	72.03	73.85	3394674	{}	\N	2020-10-12 17:07:16.265578
90868	WYNN	2020-10-06	74.57	77.735	74.015	74.32	5016879	{}	\N	2020-10-12 17:07:16.27066
90869	WYNN	2020-10-07	75.92	75.95	74.23	75.06	2835117	{}	\N	2020-10-12 17:07:16.275183
90870	WYNN	2020-10-08	75.28	76.0898	73.91	74.32	2724355	{}	\N	2020-10-12 17:07:16.279791
90871	WYNN	2020-10-09	74.63	75.53	72.45	72.67	3085687	{}	\N	2020-10-12 17:07:16.284223
90872	WYNN	2020-10-12	72.85	73.99	72.35	73.48	1167932	{}	\N	2020-10-12 17:07:16.288678
90873	XPER	2020-09-28	11.54	11.69	11.42	11.51	434145	{}	\N	2020-10-12 17:07:17.28519
90874	XPER	2020-09-29	11.49	11.68	11.4	11.57	529516	{}	\N	2020-10-12 17:07:17.296075
90875	XPER	2020-09-30	11.58	11.74	11.26	11.49	804470	{}	\N	2020-10-12 17:07:17.303978
90876	XPER	2020-10-01	11.52	11.66	11.25	11.35	731850	{}	\N	2020-10-12 17:07:17.309627
90877	XPER	2020-10-02	11.16	11.33	11.03	11.16	568697	{}	\N	2020-10-12 17:07:17.314917
90878	XPER	2020-10-05	11.27	11.59	11.27	11.57	525151	{}	\N	2020-10-12 17:07:17.319945
90879	XPER	2020-10-06	11.66	12.37	11.66	11.73	1327734	{}	\N	2020-10-12 17:07:17.323961
90880	XPER	2020-10-07	11.9	12.31	11.78	12.22	1214118	{}	\N	2020-10-12 17:07:17.328009
90881	XPER	2020-10-08	12.37	12.51	12.14	12.45	748066	{}	\N	2020-10-12 17:07:17.332283
90882	XPER	2020-10-09	12.4	12.72	12.29	12.37	810810	{}	\N	2020-10-12 17:07:17.336527
90883	XPER	2020-10-12	12.32	12.96	12.29	12.94	555552	{}	\N	2020-10-12 17:07:17.34097
90884	XSD	2020-09-28	122.28	124.42	122.02	124.39	272053	{}	\N	2020-10-12 17:07:18.332432
90885	XSD	2020-09-29	124.44	126.12	124.39	125.34	62454	{}	\N	2020-10-12 17:07:18.342803
90886	XSD	2020-09-30	125.03	127.17	125	125.73	77833	{}	\N	2020-10-12 17:07:18.349546
90887	XSD	2020-10-01	127.65	129.14	127.2	128.81	93151	{}	\N	2020-10-12 17:07:18.356101
90888	XSD	2020-10-02	125.59	127.71	125.1	125.1	40145	{}	\N	2020-10-12 17:07:18.362292
90889	XSD	2020-10-05	126.36	129.73	126.36	129.7	61091	{}	\N	2020-10-12 17:07:18.367117
90890	XSD	2020-10-06	129.88	132.75	129.19	129.57	69052	{}	\N	2020-10-12 17:07:18.371917
90891	XSD	2020-10-07	131.58	132.94	131.58	132.71	45847	{}	\N	2020-10-12 17:07:18.37645
90892	XSD	2020-10-08	134.15	135.16	133.86	134.62	40111	{}	\N	2020-10-12 17:07:18.3812
90893	XSD	2020-10-09	136.67	138.5793	136.67	138.21	74385	{}	\N	2020-10-12 17:07:18.386069
90894	XSD	2020-10-12	140.08	140.08	138.04	138.594	25888	{}	\N	2020-10-12 17:07:18.391338
90895	YLD	2020-09-28	37.6146	37.68	37.6146	37.6518	2809	{}	\N	2020-10-12 17:07:19.782025
90896	YLD	2020-09-29	37.72	37.72	37.635	37.635	3738	{}	\N	2020-10-12 17:07:19.792888
90897	YLD	2020-09-30	37.71	37.83	37.54	37.7186	6818	{}	\N	2020-10-12 17:07:19.80123
90898	YLD	2020-10-01	37.68	37.68	37.5801	37.64	7175	{}	\N	2020-10-12 17:07:19.808784
90899	YLD	2020-10-02	37.59	37.6324	37.59	37.6	1657	{}	\N	2020-10-12 17:07:19.814318
90900	YLD	2020-10-05	37.779	37.8903	37.75	37.8903	3051	{}	\N	2020-10-12 17:07:19.819083
90901	YLD	2020-10-06	37.915	37.93	37.71	37.75	2986	{}	\N	2020-10-12 17:07:19.823968
90902	YLD	2020-10-07	37.9	38.015	37.871	38.015	3018	{}	\N	2020-10-12 17:07:19.829007
90903	YLD	2020-10-08	38.1	38.11	38.05	38.0963	2814	{}	\N	2020-10-12 17:07:19.833798
90904	YLD	2020-10-09	38.22	38.22	38.12	38.18	872	{}	\N	2020-10-12 17:07:19.838383
90905	YLD	2020-10-12	38.2555	38.35	38.2555	38.28	3882	{}	\N	2020-10-12 17:07:19.842928
90906	WDAY	2020-09-28	218.54	221.5	215.25	220.34	2321816	{}	\N	2020-10-12 17:07:21.506778
90907	WDAY	2020-09-29	221.28	222.88	219.7	220.78	1693967	{}	\N	2020-10-12 17:07:21.515913
90908	WDAY	2020-09-30	220.52	222.5	214	215.13	2051991	{}	\N	2020-10-12 17:07:21.526146
90909	WDAY	2020-10-01	222.75	222.75	216.7	220.52	2299018	{}	\N	2020-10-12 17:07:21.531707
90910	WDAY	2020-10-02	219.29	225.55	218.0008	218.98	1622273	{}	\N	2020-10-12 17:07:21.538238
90911	WDAY	2020-10-05	219.67	224.58	217.58	222.04	1280383	{}	\N	2020-10-12 17:07:21.543466
90912	WDAY	2020-10-06	223.18	225.69	215.72	217.01	1411022	{}	\N	2020-10-12 17:07:21.549934
90913	WDAY	2020-10-07	227.29	230.26	223.6	227.21	2582707	{}	\N	2020-10-12 17:07:21.555823
90914	WDAY	2020-10-08	229.23	229.595	222.6	224.04	828761	{}	\N	2020-10-12 17:07:21.560954
90915	WDAY	2020-10-09	226	226.68	224.325	225.73	817086	{}	\N	2020-10-12 17:07:21.566506
90916	WDAY	2020-10-12	229.36	230.14	224.755	226.745	597399	{}	\N	2020-10-12 17:07:21.572476
90917	WEN	2020-09-28	21.56	22.195	21.51	22.15	2623208	{}	\N	2020-10-12 17:07:22.935867
90918	WEN	2020-09-29	22.08	22.44	21.74	22.35	3169825	{}	\N	2020-10-12 17:07:22.95107
90919	WEN	2020-09-30	22.4	22.58	22.23	22.295	2413785	{}	\N	2020-10-12 17:07:22.960893
90920	WEN	2020-10-01	22.44	22.95	22.4	22.86	2338420	{}	\N	2020-10-12 17:07:22.967886
90921	WEN	2020-10-02	22.39	23.34	22.33	23.18	1937092	{}	\N	2020-10-12 17:07:22.974125
90922	WEN	2020-10-05	23.11	23.66	23.04	23.55	1898732	{}	\N	2020-10-12 17:07:22.97971
90923	WEN	2020-10-06	23.65	23.82	22.91	23	1951663	{}	\N	2020-10-12 17:07:22.9851
90924	WEN	2020-10-07	23.25	23.4	23.11	23.29	1909367	{}	\N	2020-10-12 17:07:22.990702
90925	WEN	2020-10-08	23.38	23.87	23.38	23.785	1858457	{}	\N	2020-10-12 17:07:22.996609
90926	WEN	2020-10-09	23.96	24.7	23.86	24.31	4309675	{}	\N	2020-10-12 17:07:23.001764
90927	WEN	2020-10-12	24.41	24.4464	23.81	23.8131	1135341	{}	\N	2020-10-12 17:07:23.007209
90928	WSM	2020-09-28	87.85	88.84	87.65	88.36	685914	{}	\N	2020-10-12 17:07:24.342108
90929	WSM	2020-09-29	88.94	90.39	88.01	89.25	1415599	{}	\N	2020-10-12 17:07:24.352921
90930	WSM	2020-09-30	89.76	91.4	89.25	90.44	2839555	{}	\N	2020-10-12 17:07:24.360827
90931	WSM	2020-10-01	91.8	94.285	91	93.76	2840958	{}	\N	2020-10-12 17:07:24.367899
90932	WSM	2020-10-02	91.34	96.32	91	94.51	2464168	{}	\N	2020-10-12 17:07:24.374394
90933	WSM	2020-10-05	94.77	99.305	94.77	99.03	2412171	{}	\N	2020-10-12 17:07:24.380814
90934	WSM	2020-10-06	99.99	100.56	96.95	97.2	2478021	{}	\N	2020-10-12 17:07:24.386582
90935	WSM	2020-10-07	98.54	99.96	96.73	96.73	2364629	{}	\N	2020-10-12 17:07:24.392633
90936	WSM	2020-10-08	97.6	98.85	96.58	98.84	1736963	{}	\N	2020-10-12 17:07:24.398371
90937	WSM	2020-10-09	99.25	100.62	98.39	100.17	3587794	{}	\N	2020-10-12 17:07:24.403378
90938	WSM	2020-10-12	100.83	101.95	98.71	100.1	581063	{}	\N	2020-10-12 17:07:24.4085
90939	XMLV	2020-09-28	41.53	41.85	41.51	41.77	534597	{}	\N	2020-10-12 17:07:26.382594
90940	XMLV	2020-09-29	41.7	41.8503	41.36	41.56	460036	{}	\N	2020-10-12 17:07:26.395149
90941	XMLV	2020-09-30	41.66	42.0125	41.46	41.66	422747	{}	\N	2020-10-12 17:07:26.404262
90942	XMLV	2020-10-01	41.75	42.03	41.5925	41.96	508213	{}	\N	2020-10-12 17:07:26.411732
90943	XMLV	2020-10-02	41.49	42.37	41.486	42.17	444528	{}	\N	2020-10-12 17:07:26.420064
90944	XMLV	2020-10-05	42.43	42.9498	42.43	42.92	465330	{}	\N	2020-10-12 17:07:26.425891
90945	XMLV	2020-10-06	43.01	43.55	42.7301	42.81	353157	{}	\N	2020-10-12 17:07:26.431494
90946	XMLV	2020-10-07	43.13	43.31	43.02	43.17	409857	{}	\N	2020-10-12 17:07:26.437033
90947	XMLV	2020-10-08	43.4	43.54	43.26	43.52	339896	{}	\N	2020-10-12 17:07:26.442492
90948	XMLV	2020-10-09	43.69	43.8	43.5	43.59	505410	{}	\N	2020-10-12 17:07:26.447729
90949	XMLV	2020-10-12	43.72	44.09	43.71	43.975	126007	{}	\N	2020-10-12 17:07:26.452852
90950	ZNGA	2020-09-28	9.03	9.08	8.91	9.07	14301300	{}	\N	2020-10-12 17:07:27.796358
90951	ZNGA	2020-09-29	9.06	9.25	9.03	9.12	15347296	{}	\N	2020-10-12 17:07:27.804326
90952	ZNGA	2020-09-30	9.2	9.23	9.05	9.12	21317481	{}	\N	2020-10-12 17:07:27.810859
90953	ZNGA	2020-10-01	9.22	9.61	9.21	9.49	27088812	{}	\N	2020-10-12 17:07:27.817885
90954	ZNGA	2020-10-02	9.295	9.56	9.26	9.32	22244451	{}	\N	2020-10-12 17:07:27.823208
90955	ZNGA	2020-10-05	9.4	9.59	9.33	9.57	15572977	{}	\N	2020-10-12 17:07:27.828999
90956	ZNGA	2020-10-06	9.58	9.72	9.51	9.57	20897870	{}	\N	2020-10-12 17:07:27.835033
90957	ZNGA	2020-10-07	9.6258	9.68	9.55	9.58	23493360	{}	\N	2020-10-12 17:07:27.840387
90958	ZNGA	2020-10-08	9.64	9.64	9.29	9.3	17763899	{}	\N	2020-10-12 17:07:27.845669
90959	ZNGA	2020-10-09	9.36	9.42	9.3	9.31	14420665	{}	\N	2020-10-12 17:07:27.850993
90960	ZNGA	2020-10-12	9.4	9.45	9.34	9.34	6789758	{}	\N	2020-10-12 17:07:27.85623
90961	WES	2020-09-28	7.42	7.77	7.38	7.71	2065968	{}	\N	2020-10-12 17:07:30.335666
90962	WES	2020-09-29	7.71	8.13	7.48	8.05	4590364	{}	\N	2020-10-12 17:07:30.346417
90963	WES	2020-09-30	8.1	8.37	7.84	8	6209370	{}	\N	2020-10-12 17:07:30.354639
90964	WES	2020-10-01	8	8.09	7.72	7.79	2092303	{}	\N	2020-10-12 17:07:30.360478
90965	WES	2020-10-02	7.5	8.1	7.47	7.99	1315038	{}	\N	2020-10-12 17:07:30.366851
90966	WES	2020-10-05	7.98	8.38	7.98	8.38	1878173	{}	\N	2020-10-12 17:07:30.371917
90967	WES	2020-10-06	8.5	8.7	8.11	8.32	2132758	{}	\N	2020-10-12 17:07:30.377042
90968	WES	2020-10-07	8.33	8.46	8.236	8.35	2900978	{}	\N	2020-10-12 17:07:30.382154
90969	WES	2020-10-08	8.46	8.9	8.3	8.85	4122070	{}	\N	2020-10-12 17:07:30.387794
90970	WES	2020-10-09	8.95	9	8.64	8.89	1574773	{}	\N	2020-10-12 17:07:30.393292
90971	WES	2020-10-12	8.86	8.93	8.67	8.83	1345906	{}	\N	2020-10-12 17:07:30.398277
90972	WRK	2020-09-28	36.07	36.35	34.32	34.98	4214607	{}	\N	2020-10-12 17:07:32.104708
90973	WRK	2020-09-29	35.01	35.02	33.82	34.25	5729869	{}	\N	2020-10-12 17:07:32.115265
90974	WRK	2020-09-30	34.6	35.41	34.25	34.74	4034817	{}	\N	2020-10-12 17:07:32.121521
90975	WRK	2020-10-01	35	35.39	33.8	34.06	4215533	{}	\N	2020-10-12 17:07:32.127514
90976	WRK	2020-10-02	33.48	35.34	33.18	35.02	5072372	{}	\N	2020-10-12 17:07:32.133458
90977	WRK	2020-10-05	35.63	36.99	35.36	36.77	7444342	{}	\N	2020-10-12 17:07:32.138476
90978	WRK	2020-10-06	36.67	37.06	35.95	36.11	5748883	{}	\N	2020-10-12 17:07:32.143262
90979	WRK	2020-10-07	36.7	37.88	36.6585	37.19	5867063	{}	\N	2020-10-12 17:07:32.148181
90980	WRK	2020-10-08	37.29	37.88	36.8843	37.77	3326999	{}	\N	2020-10-12 17:07:32.152638
90981	WRK	2020-10-09	38.2	38.57	37.44	37.8	5472320	{}	\N	2020-10-12 17:07:32.157402
90982	WRK	2020-10-12	38.93	40.17	38.68	40	4404754	{}	\N	2020-10-12 17:07:32.162306
90983	YLCO	2020-09-28	14.38	14.45	14.3505	14.3983	10410	{}	\N	2020-10-12 17:07:35.194705
90984	YLCO	2020-09-29	14.48	14.5924	14.425	14.55	10944	{}	\N	2020-10-12 17:07:35.205817
90985	YLCO	2020-09-30	14.66	14.7	14.5753	14.7	40543	{}	\N	2020-10-12 17:07:35.212311
90986	YLCO	2020-10-01	14.68	14.86	14.665	14.86	11888	{}	\N	2020-10-12 17:07:35.219037
90987	YLCO	2020-10-02	14.84	14.95	14.77	14.8701	10613	{}	\N	2020-10-12 17:07:35.224605
90988	YLCO	2020-10-05	14.86	15.1	14.83	14.985	480962	{}	\N	2020-10-12 17:07:35.229221
90989	YLCO	2020-10-06	15.13	15.175	14.97	15.02	38521	{}	\N	2020-10-12 17:07:35.233674
90990	YLCO	2020-10-07	15.27	15.37	15.167	15.32	81119	{}	\N	2020-10-12 17:07:35.238372
90991	YLCO	2020-10-08	15.39	15.472	15.36	15.41	23813	{}	\N	2020-10-12 17:07:35.243167
90992	YLCO	2020-10-09	15.47	15.54	15.44	15.53	40219	{}	\N	2020-10-12 17:07:35.247747
90993	YLCO	2020-10-12	15.62	15.62	15.5101	15.5534	27012	{}	\N	2020-10-12 17:07:35.251986
90994	WBIG	2020-09-28	22.52	22.7198	22.52	22.6858	2257	{}	\N	2020-10-12 17:07:36.182175
90995	WBIG	2020-09-29	22.68	22.728	22.665	22.7269	6896	{}	\N	2020-10-12 17:07:36.191771
90996	WBIG	2020-09-30	22.83	22.94	22.745	22.745	11833	{}	\N	2020-10-12 17:07:36.197898
90997	WBIG	2020-10-01	22.91	22.9634	22.8	22.86	5841	{}	\N	2020-10-12 17:07:36.204266
90998	WBIG	2020-10-02	22.67	22.88	22.67	22.845	4973	{}	\N	2020-10-12 17:07:36.210328
90999	WBIG	2020-10-05	22.9699	23.04	22.952	23.04	6282	{}	\N	2020-10-12 17:07:36.215275
91000	WBIG	2020-10-06	23.08	23.16	22.8287	22.8287	14612	{}	\N	2020-10-12 17:07:36.221002
91001	WBIG	2020-10-07	23.012	23.0439	22.94	23.02	17617	{}	\N	2020-10-12 17:07:36.227049
91002	WBIG	2020-10-08	23.19	23.28	23.19	23.28	2307	{}	\N	2020-10-12 17:07:36.232407
91003	WBIG	2020-10-09	23.37	23.44	23.37	23.44	1447	{}	\N	2020-10-12 17:07:36.238013
91004	WBIG	2020-10-12	23.52	23.54	23.52	23.54	478	{}	\N	2020-10-12 17:07:36.244798
91005	YNDX	2020-09-28	64.3	64.53	63.64	64.21	1935155	{}	\N	2020-10-12 17:07:37.201328
91006	YNDX	2020-09-29	63.95	64.77	63.77	64.62	1878646	{}	\N	2020-10-12 17:07:37.21067
91007	YNDX	2020-09-30	64.77	65.46	64.58	65.25	1524231	{}	\N	2020-10-12 17:07:37.217471
91008	YNDX	2020-10-01	65.65	65.79	63.3	63.56	2531696	{}	\N	2020-10-12 17:07:37.224158
91009	YNDX	2020-10-02	62.32	63.62	62.25	62.5	1476574	{}	\N	2020-10-12 17:07:37.23032
91010	YNDX	2020-10-05	62.56	64	62.36	63.66	1709727	{}	\N	2020-10-12 17:07:37.235914
91011	YNDX	2020-10-06	63.77	64.01	61.75	61.88	2818773	{}	\N	2020-10-12 17:07:37.241658
91012	YNDX	2020-10-07	62.07	62.26	60.63	60.85	2932050	{}	\N	2020-10-12 17:07:37.247928
91013	YNDX	2020-10-08	61.55	61.5896	60.57	60.79	2586125	{}	\N	2020-10-12 17:07:37.25416
91014	YNDX	2020-10-09	60.81	61.95	60.57	60.93	2465215	{}	\N	2020-10-12 17:07:37.260394
91015	YNDX	2020-10-12	61.6	61.83	60.95	61.48	1066988	{}	\N	2020-10-12 17:07:37.266816
\.


--
-- TOC entry 3026 (class 0 OID 16455)
-- Dependencies: 207
-- Data for Name: ticker_data; Type: TABLE DATA; Schema: public; Owner: liu
--

COPY public.ticker_data (symbol, name, description, tags, similar_tickers, industry, sector, exchange, short_ratio, create_tstamp, modify_tstamp) FROM stdin;
AAPL	Apple Inc.	Apple Inc is designs, manufactures and markets mobile communication and media devices and personal computers, and sells a variety of related software, services, accessories, networking solutions and third-party digital content and applications.	{Technology,"Consumer Electronics","Computer Hardware"}	{MSFT,NOK,IBM,HPQ,GOOGL,BB,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:11:59.289447	2020-10-11 23:38:31.023428
ABC	AmerisourceBergen Corporation	AmerisourceBergen Corp is a pharmaceutical services company. It provides drug distribution and related healthcare services and solutions to pharmacy, physician, and manufacturer customers based in the United States and Canada.	{Healthcare,"Medical Distribution"}	{HSIC,OMCL,PDCO,CAH,OMI,CVS,RAD,MCK,HLF,WBA,XLV}	Medical Distribution	Healthcare	New York Stock Exchange	\N	2020-10-11 23:11:59.315763	2020-10-11 23:38:31.02582
ABEO	Abeona Therapeutics Inc.	Abeona Therapeutics Inc is a clinical-stage biopharmaceutical company. The firm is focused on developing gene therapies and plasma-based products for life-threatening rare genetic diseases.	{Healthcare,Biotechnology}	{BDSI,AMAG,IRWD,JNJ,SGYP,MDGL,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:11:59.329032	2020-10-11 23:38:31.025871
ABCD	Cambium Learning Group Inc.	Cambium Learning Group Inc is educational solutions and services company. Its brands Learning A-Z, Voyager Sopris Learning, ExploreLearning and Kurzweil Education, provides technology solutions for students and teachers.	{"Consumer Defensive","Education & Training Services",Education}	{BFAM,HMHC}	Education	Consumer Defensive	NASDAQ Capital Market	\N	2020-10-11 23:11:59.322192	2020-10-11 23:38:31.028166
ABIL	Ability Inc.	Ability Inc is a holding company which provides advanced interception, geolocation and cyber intelligence products and solutions that serve the needs and increasing challenges of security and intelligence agencies.	{Technology,"Communication Equipment"}	{XLK}	Communication Equipment	Technology	NASDAQ Capital Market	\N	2020-10-11 23:11:59.342929	2020-10-11 23:38:31.031328
ABM	ABM Industries Incorporated	ABM Industries Inc is engaged in providing end-to-end integrated facility solutions. The services offered by the company includes Electrical, Energy, Facilities Engineering, Landscape and Turf and Parking and Transportation Services.	{Industrials,"Business Services"}	{CTAS,HCSG,EME,MOC,SP}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.349497	2020-10-11 23:38:31.023428
ABR	Arbor Realty Trust	Arbor Realty Trust Inc is a real estate specialty finance firm. It invests in a diversified portfolio of structured finance assets in the multi-family and commercial real estate markets. It also invests in real estate-related bridge and mezzanine loans.	{"Real Estate","REIT - Diversified",REITs}	{STWD,BXMT,CMCT}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:11:59.356353	2020-10-11 23:38:31.024353
AC	Associated Capital Group Inc.	Associated Capital Group Inc is a diversified global financial services company. It operates in the business of alternative investment management, institutional research services and cash and other assets through subsidiaries.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{NOAH,OAK}	Brokers & Exchanges	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.370436	2020-10-11 23:38:31.021426
ABTX	Allegiance Bancshares Inc.	Allegiance Bancshares Inc is a bank holding company. The company through its subsidiary provides diversified commercial banking services to Houston metropolitan area-based small to medium-sized businesses and individual customers.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:11:59.363055	2020-10-11 23:38:31.02423
ACBI	Atlantic Capital Bancshares Inc.	Atlantic Capital Bancshares Inc provides a range of credit, treasury management, and deposit products and services to emerging growth businesses, middle market corporations, commercial real estate developers and investors, and private clients.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:11:59.377336	2020-10-11 23:38:31.028166
AAP	Advance Auto Parts Inc W/I	Advance Auto Parts Inc is an automotive aftermarket parts provider serving professional installers, DIY (do-it-yourself) customer and independently-owned operators.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{MNRO,PRTS,AZO,HZO,GPC,ORLY}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:11:59.282913	2020-10-11 23:38:31.021426
ABBV	AbbVie Inc.	AbbVie Inc is a biopharmaceutical company that discovers, develops, manufactures and sells a range of pharmaceutical products for treating chronic autoimmune diseases, virology and neurological disorders.	{Healthcare,"Drug Manufacturers - Major","Drug Manufacturers"}	{AMGN,NBIX,GILD,BIIB,AZN,GSK,BMY,LLY,PFE,XLV}	Drug Manufacturers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:11:59.315326	2020-10-11 23:38:31.021426
ABCB	Ameris Bancorp	Ameris Bancorp provides business through its wholly owned banking subsidiary, Ameris Bank. Services include commercial real estate loans, which accounts for the major source of its income.	{"Financial Services","Banks - Regional - US",Banks}	{CBAN,LION,BBT,STI,SGB,STBZ,UCBI}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:11:59.322207	2020-10-11 23:38:31.023428
ABEOW	Abeona Therapeutics (Warrant)	Abeona Therapeutics, Inc. is a clinical stage biopharmaceutical company, which engages in the development of gene therapy for severe and life threatening rare diseases. It programs include EB-101 (gene-corrected skin grafts) for recessive dystrophic epidermolysis bullosa (RDEB); ABO-102 (AAV-SGSH), an adeno-associated virus (AAV) based gene therapy for Sanfilippo syndrome type A (MPS IIIA) and ABO-101 (AAV NAGLU), an AAV based gene therapy for Sanfilippo syndrome type B (MPS IIIB). The company was founded in 1974 and is headquartered in Dallas, TX.	{}	{}	Pharmaceuticals Generic	Health Technology	NasdaqCM	\N	2020-10-11 23:11:59.334354	2020-10-11 23:38:31.024353
ABDC	Alcentra Capital Corp.	Alcentra Capital Corp is a specialty finance company operating as a non-diversified, closed-end management investment company. It provides customized debt and equity financing solutions to lower middle-market companies.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:11:59.328425	2020-10-11 23:38:31.023007
ABG	Asbury Automotive Group Inc	Asbury Automotive Group Inc is an US-based automotive retailer. The company owns and operates approximately 90 vehicle franchises. Its franchise brand name includes BMW, Honda, Mercedes-Benz, Ford, Jaguar, Land Rover, Porsche, Toyota, Volvo and others.	{"Consumer Cyclical","Auto & Truck Dealerships",Autos}	{CPRT,CRMT,RUSHA,LAD,AN,GPI,SAH,KMX,PAG}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:11:59.340951	2020-10-11 23:38:31.021426
ABIO	ARCA biopharma Inc.	ARCA biopharma Inc is a biopharmaceutical company. The Company is engaged in developing genetically-targeted therapies for cardiovascular diseases. Its product candidate is Gencaro.	{Healthcare,Biotechnology}	{UTHR,BTX,PTN,CYTK,MDT,EIGR,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:11:59.347543	2020-10-11 23:38:31.028166
ABMD	ABIOMED Inc.	Abiomed Inc is a medical device company. It provides mechanical circulatory support devices to aid heart recovery to heart failure patients.	{Healthcare,"Medical Devices"}	{ATRC,BSX,TFX,XLV}	Medical Devices	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.354517	2020-10-11 23:38:31.025871
ABT	Abbott Laboratories	Abbott Laboratories is a health care company that manufactures medical devices, blood glucose monitoring kits, nutritional healthcare products, diagnostic products and equipment, and branded generic drugs.	{Healthcare,"Medical Devices"}	{HOLX,BSX,BMY,JNJ,PFE,TMO,MDT,MYL,XLV}	Medical Devices	Healthcare	New York Stock Exchange	\N	2020-10-11 23:11:59.361275	2020-10-11 23:38:31.023007
ABUS	Arbutus Biopharma Corporation	Arbutus Biopharma Corp is a biopharmaceutical business engaged in advancing novel RNA interference therapeutics and discovering, developing and commercializing a cure for patients suffering from chronic hepatitis B infection.	{Healthcare,Biotechnology}	{PDLI,INFI,CNAT,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.368286	2020-10-11 23:38:31.02582
ACAD	ACADIA Pharmaceuticals Inc.	ACADIA Pharmaceuticals Inc is a biotechnology company which develops and commercializes biopharmaceutical products to address central nervous system disorders. It aims to discover small molecule drugs to address Parkinson's, Alzheimer's and schizophrenia.	{Healthcare,Biotechnology}	{VNDA,BMY,JNJ,LLY,PFE,ADMS,CBIO,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.375501	2020-10-11 23:38:31.028166
ACER	Acer Therapeutics Inc.	Acer Therapeutics Inc is a pharmaceutical company that acquires, develops and intends to commercialize therapies for patients with serious rare diseases with critical unmet medical need.	{Healthcare,Biotechnology}	{BIIB,MNTA,NVS,TEVA,FBIO,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:11:59.39041	2020-10-11 23:38:31.025871
ACM	AECOM	AECOM provides consulting and engineering services to clients in transportation, facilities, environmental, and other markets. It mainly designs, builds, and operates infrastructure projects for governments, businesses, and organizations.	{Industrials,"Engineering & Construction"}	{TTEK,GVA,JEC,KBR,HIL,EGL}	Engineering & Construction	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.420266	2020-10-11 23:38:31.023007
ACN	Accenture plc Class A (Ireland)	Accenture PLC is a professional service company. It provides consulting, technology and outsourcing services to Communications, Media and Technology, Financial services, Health and Public service, Products and Resources segments.	{Technology,"Information Technology Services","Application Software"}	{CTSH,ERIC,SAP,NOK,IBM,WIT,G,INFY,DXC,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:11:59.421526	2020-10-11 23:38:31.021426
ACTG	Acacia Research Corporation	Acacia Research Corporation is an intermediary in the patent marketplace, facilitating efficiency and delivering monetary rewards to patent owners.	{Industrials,"Business Services"}	{SPEX}	Business Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:11:59.432688	2020-10-11 23:38:31.021426
ACST	Acasti Pharma Inc.	Acasti Pharma Inc is engaged in the research, development, and commercialization of prescription drugs using omega-3 fatty acids derived from krill oil.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:11:59.432392	2020-10-11 23:38:31.02582
ADAP	Adaptimmune Therapeutics plc	Adaptimmune Therapeutics PLC along with its subsidiaries is a clinical-stage biopharmaceutical company. It is engaged in novel cancer immunotherapy products based on its T-cell receptor platform.	{Healthcare,Biotechnology}	{NVAX,SGMO,ENZ,QURE,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.441806	2020-10-11 23:38:31.02582
ADES	Advanced Emissions Solutions Inc.	Advanced Emissions Solutions Inc is engaged in providing environmental technologies and specialty chemicals to the coal-burning utility industry.	{Industrials,"Pollution & Treatment Controls","Industrial Products"}	{LECO,THR,KRNT,MCRN}	Industrial Products	Industrials	NASDAQ Global Market	\N	2020-10-11 23:11:59.44447	2020-10-11 23:38:31.024353
ADRA	BLDRS Asia 50 ADR Index Fund	The investment seeks to provide investment results that correspond generally before fees and expenses, to the price and yield performance of the Bank of New York Mellon Asia 50 ADR Index.\n The fund typically invests 95% of assets in the securities that comprise the index. The index is intended to give investors a benchmark for tracking the price and yield performance of Depositary Receipts from the Asian market. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:11:59.45406	2020-10-11 23:38:31.021426
ADRE	BLDRS Emerging Markets 50 ADR Index Fund	The investment seeks to provide investment results that correspond generally, before fees and expenses, to the price and yield performance of the Bank of New York Mellon Emerging Markets 50 ADR Index. \n The fund typically invests substantially all of its assets in the securities that make up the index. The index is intended to give investors a benchmark for tracking the price and yield performance of Emerging Markets Depositary Receipts. the fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:11:59.454664	2020-10-11 23:38:31.02423
ADTN	ADTRAN Inc.	Adtran Inc is engaged in providing networking and communications equipment. Its products enable voice, data, video and Internet communications across a variety of network infrastructures.	{Technology,"Communication Equipment"}	{CSCO,EXTR,AUDC,ERIC,JNPR,CALX,CIEN,DZSI,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:11:59.464778	2020-10-11 23:38:31.028166
ADVM	Adverum Biotechnologies Inc.	Adverum Biotechnologies Inc is a gene therapy company committed to discovering and developing novel medicines that can offer potentially life-changing therapeutic benefit to patients living with rare diseases or diseases of the eye.	{Healthcare,Biotechnology}	{A,XLRN,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:11:59.466851	2020-10-11 23:38:31.02423
AEE	Ameren Corporation	Ameren Corp is an American power company. It operates a rate-regulated electric generation, transmission, and distribution business and a rate-regulated natural gas transmission and distribution business in Missouri and Illinois.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{OTTR,LNT,CMS,D,DUK,WEC,XEL,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:11:59.47452	2020-10-11 23:38:31.024353
MRUS	Merus N.V.	Merus NV is a clinical-stage immuno-oncology company developing innovative full-length human bispecific antibody therapeutics, referred to as Biclonics.	{Healthcare,Biotechnology}	{PDLI,PIRS,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:07.217809	2020-10-11 23:38:31.024353
ACGLO	Arch Capital Group Ltd. Depositary Shares Each Representing 1/1000th Interest in a Share of5.45% Non-Cumulative Preferred Shares Series F	Arch Capital Group Ltd provides insurance, reinsurance and mortgage. It provides property, casualty and mortgage insurance and reinsurance lines, and focus on writing specialty lines of insurance and reinsurance.	{"Financial Services","Insurance - Diversified",Insurance}	{}	Insurance	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:11:59.402221	2020-10-11 23:38:31.023007
ACIW	ACI Worldwide Inc.	ACI Worldwide Inc together with its subsidiaries develops, markets, installs, and supports software products and services to facilitate electronic payments. It leverages its distribution network in the Americas, Europe, the Middle East, and Africa.	{Technology,"Software - Application","Application Software"}	{INTU,EPAY,FISV,JKHY,NCR,FICO,IBM,FIS,MA,ORCL,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:11:59.416876	2020-10-11 23:38:31.023428
ACLS	Axcelis Technologies Inc.	Axcelis Technologies Inc produces ion implantation and other processing equipment used in the fabrication of semiconductor chips in the United States, Europe, and Asia. It also provides extensive aftermarket service and support.	{Technology,"Semiconductor Equipment & Materials",Semiconductors}	{AMAT,CVV,VECO,ASYS,LRCX,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:11:59.417922	2020-10-11 23:38:31.024353
ACRS	Aclaris Therapeutics Inc.	Aclaris Therapeutics Inc is a clinical-stage specialty pharmaceutical company which focuses on identifying, developing and commercializing differentiated drugs to address significant unmet needs in dermatology.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.428715	2020-10-11 23:38:31.023007
ACSI	American Customer Satisfaction	The investment seeks to track the performance of the American Customer Satisfaction Investable Index.\n Under normal circumstances, at least 80% of the fund's total assets will be invested in the component securities of the index. Construction of the index begins with over 350 ACSI Companies across 43 industries and 10 economic sectors. The initial universe is then screened to eliminate companies whose stock is not principally listed on a U.S. exchange, whose stock does not meet minimum liquidity requirements, or whose ACSI Score falls below its respective industry average. The remaining companies are included in the index.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:11:59.428768	2020-10-11 23:38:31.024353
ACWV	iShares Edge MSCI Min Vol Global	The investment seeks to track the investment results of the MSCI ACWI Minimum Volatility (USD) Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index measures the combined performance of equity securities in both emerging and developed markets that, in the aggregate, have lower volatility relative to the broader developed and emerging markets.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:11:59.44154	2020-10-11 23:38:31.031328
ACWX	iShares MSCI ACWI Ex US Index Fund	The investment seeks to track the investment results of the MSCI ACWI ex USA Index composed of large- and mid-capitalization non-U.S. equities.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is a free float-adjusted market capitalization-weighted index designed to measure the combined equity market performance of developed and emerging markets countries, excluding the United States.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:11:59.441154	2020-10-11 23:38:31.023428
ADOM	ADOMANI Inc.	ADOMANI Inc provides complete zero-emission electric and hybrid vehicle solutions to school bus and medium to heavy-duty fleet operators.	{"Consumer Cyclical","Auto Parts",Autos}	{}	Autos	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:11:59.45433	2020-10-11 23:38:31.02582
ADMS	Adamas Pharmaceuticals Inc.	Adamas Pharmaceuticals Inc is a pharmaceutical company focused on the development and commercialization of therapeutics targeting chronic disorders of the central nervous systems. The company has developed and got approved the drug GOCOVRI.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{GSK,MRK,SUPN,ABBV,ZGNX,CBIO,XLV}	Drug Manufacturers	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:11:59.454143	2020-10-11 23:38:31.028166
ADSW	Advanced Disposal Services Inc.	Advanced Disposal Services Inc provides ideas and solutions to the business of a clean environment. It offers non-hazardous solid waste collection, transfer, recycling and disposal services to the commercial, industrial and residential customers.	{Industrials,"Waste Management"}	{PESI}	Waste Management	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.464181	2020-10-11 23:38:31.028166
SNHNL	Senior Housing Properties Trust 6.25% Senior Notes Due 2046		{}	{}			Nasdaq Global Select	\N	2020-10-11 23:12:11.483357	2020-10-11 23:38:31.024353
ACHC	Acadia Healthcare Company Inc.	Acadia Healthcare Co Inc operates mental healthcare facilities in the United States. It addresses behavioral abnormalities in psychiatric facilities, substance abuse facilities, and residential treatment centers.	{Healthcare,"Medical Care","Health Care Providers"}	{ENSG,NHC,UHS,CSU,DVCR,GEN,RHE,XLV}	Health Care Providers	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.405736	2020-10-11 23:38:31.028166
ACOR	Acorda Therapeutics Inc.	Acorda Therapeutics Inc is a biopharmaceutical company. It is engaged in the identification, development & commercialization of novel therapies for the treatment of neurological disorders.	{Healthcare,Biotechnology}	{BMRN,RGEN,MNTA,MYL,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.422672	2020-10-11 23:38:31.02582
ACV	AllianzGI Diversified Income & Convertible Fund of Beneficial Interest	AllianzGI Diversified Income & Convertible Fund is a diversified, closed-end management investment company. The Fund's investment objective is to provide total return through a combination of current income and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.433774	2020-10-11 23:38:31.028166
ADP	Automatic Data Processing Inc.	Automatic Data Processing Inc is a provider of miscellaneous business solutions. Its business includes provision of technology-based outsourcing solutions to employers, vehicle retailers and manufacturers.	{Industrials,"Business Services"}	{INTU,BBSI,PAYX,EFX,ADS,ACN,NSP,WAGE,TNET,FNF}	Business Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:11:59.453851	2020-10-11 23:38:31.024353
AED	AEGON N.V. Perp. Cap. Securities (Netherlands)	Aegon NV is engaged in providing life insurance, pensions and asset management products and services in the USA, Europe, and Asia. It offers life and protection, and accidental death products as well as individual savings and retirement products.	{"Financial Services","Insurance - Diversified",Insurance}	{}	Insurance	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.474583	2020-10-11 23:38:31.02423
AEP	American Electric Power Company Inc.	American Electric Power Co Inc is a regulated utilities operator. The company generates, transmits and distributes electricity to more than 5 million retail customers across over 10 states in the United States of America.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{ETR,AES,SO,EXC,D,PPL,NRG,DUK,XEL,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:11:59.484613	2020-10-11 23:38:31.02423
ADS	Alliance Data Systems Corporation	Alliance Data Systems Corp provides marketing, loyalty, and credit solutions in the United States, Canada, and other countries. The business activity of the group is functioned through LoyaltyOne, Epsilon, and Card service segment.	{"Financial Services","Credit Services"}	{FISV,TSS,GPN,FIS,PCOM}	Credit Services	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.463353	2020-10-11 23:38:31.023428
AEZS	Aeterna Zentaris Inc.	AEterna Zentaris Inc is a specialty biopharmaceutical company engaged in developing novel treatments in oncology, endocrinology and women's health.	{Healthcare,Biotechnology}	{KERX,SPPI,HALO,NVS,BMY,SNY,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:11:59.49732	2020-10-11 23:38:31.02423
AFSI	AmTrust Financial Services Inc.	AmTrust Financial Services Inc underwrites and provides property and casualty insurance products, including workers' compensation, commercial automobile, general liability and warranty coverage, in the United States and internationally.	{"Financial Services","Insurance - Property & Casualty"}	{WRB,CNA,MKL,ORI,RLI,TRV,WTM}	Insurance - Property & Casualty	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:11:59.511706	2020-10-11 23:38:31.023007
AGFS	AgroFresh Solutions Inc.	AgroFresh Solutions Inc creates agricultural technologies that preserve the quality and value of fresh produce and flowers. One such product is SmartFresh, which blocks the effects of ethylene and delays ripening while leaving no visible residue.	{"Basic Materials","Agricultural Inputs",Agriculture}	{INSE}	Agriculture	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:11:59.521401	2020-10-11 23:38:31.025871
AGN	Allergan plc	Allergan PLC is a specialty pharmaceutical company engaged in the development, manufacturing, marketing, and distribution of brand name pharmaceutical products, medical aesthetics, biosimilar and OTC pharmaceutical products.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{ABT,MRK,ZTS,MYL,XLV}	Drug Manufacturers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:11:59.530798	2020-10-11 23:38:31.028166
AGS	PlayAGS Inc.	PlayAGS Inc acts as a designer and supplier of gaming products and services for the gaming industry. It supplies electronic gaming machines (EGM), server-based systems and back-office systems which are used by casinos, and various gaming locations.	{"Consumer Cyclical",Gambling,"Travel & Leisure"}	{}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:11:59.542216	2020-10-11 23:38:31.02582
AHL	Aspen Insurance Holdings Limited	Aspen Insurance Holdings Ltd is a Bermudian insurance and reinsurance company. It offers property and casualty insurance, marine, aviation and energy insurance, financial and professional lines insurance, property catastrophe reinsurance, among others.	{"Financial Services","Insurance - Reinsurance","Insurance - Specialty"}	{ACGL,RNR,RE,AXS}	Insurance - Specialty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.5525	2020-10-11 23:38:31.021426
AIMT	Aimmune Therapeutics Inc.	Aimmune Therapeutics Inc is engaged in the field of biotechnology. It researches and develops drug candidates for the treatment of peanut and other food allergies.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.564192	2020-10-11 23:38:31.024353
ACHV	Achieve Life Sciences Inc.	Achieve Life Sciences Inc is a United States based clinical-stage biopharmaceutical company. It is focused on the development of novel next generation cancer therapeutics.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:11:59.410097	2020-10-11 23:38:31.023007
ACRE	Ares Commercial Real Estate Corporation	Ares Commercial Real Estate Corp is a specialty finance company and a Real Estate Investment Trust providing commercial real estate loans and related investments. The company addresses the financing needs of the middle-market commercial real estate.	{"Real Estate","REIT - Diversified",REITs}	{}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:11:59.425845	2020-10-11 23:38:31.023428
ADMA	ADMA Biologics Inc	ADMA Biologics Inc is a biopharmaceutical company. The company develops and manufactures specialty plasma-based biologics for the proposed treatment of immune deficiencies and prevention of certain infectious diseases.	{Healthcare,Biotechnology}	{NVAX,RGEN,VICL,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:11:59.446629	2020-10-11 23:38:31.028166
ADZ	DB Agriculture Short ETN due April 1 2038	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the Deutsche Bank Liquid Commodity index - Optimum Yield Agriculture.\n The fund is a senior unsecured obligation that allows investors to take a short view on the performance of the agriculture sector. The index is composed of roughly equal percentages of corn, wheat, soybean, and sugar futures contracts.	{}	{}			OTC Pink	\N	2020-10-11 23:11:59.4703	2020-10-11 23:38:31.028166
AEIS	Advanced Energy Industries Inc.	Advanced Energy Industries Inc is engaged in providing precision power-conversion measurement and control solutions which transforms power into various forms for use in manufacturing and industrial equipment and applications.	{Technology,"Electronic Components","Computer Hardware"}	{AMAT,CVV,MKSI,COHU,ENTG,BRKS,UCTT,AETI,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:11:59.482818	2020-10-11 23:38:31.028166
AER	Aercap Holdings N.V.	AerCap Holdings NV provides aircraft leasing and aviation finance services. Its activities also include aircraft asset management, corporate services, remarketing of aircraft, collecting rental, maintenance payments and monitoring aircraft maintenance.	{Industrials,"Airports & Air Services","Transportation & Logistics"}	{WLFC,TGH,GE,AYR,CAI,FLY,ACY,AL}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.493874	2020-10-11 23:38:31.031328
ADRU	BLDRS Europe 100 ADR Index Fund	The investment seeks to provide investment results that correspond generally, before fees and expenses, to the price and yield performance of the Bank of New York Mellon Europe Select ADR Index.\n The fund typically invests substantially all of its assets in the securities that make up the index. The index is intended to give investors a benchmark for tracking the price and yield performance of European Depositary Receipts. the fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:11:59.457353	2020-10-11 23:38:31.025871
AFTY	CSOP FTSE China A50	The investment seeks to provide investment results that, before fees and expenses, track the performance of the FTSE China A50 Net Total Return Index.\n The fund will typically invest at least 80% of its total assets in the securities included in its underlying index. The index is comprised of A-Shares issued by the 50 largest companies in the China A-Shares market. The fund may invest the remainder of its assets in investments that are not included in the index, but which the Adviser believes will help the fund track the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.511806	2020-10-11 23:38:31.021426
AGI	Alamos Gold Inc. Class A	Alamos Gold Inc is engaged in the exploration, mine development and extraction of precious metals. It owns and operates the Mulatos mine and holds the mineral rights to the Salamandra group of concessions in the State of Sonora, Mexico.	{"Basic Materials",Gold,"Metals & Mining"}	{ABX,EGO,GG,AUY,K,NGD}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:11:59.527785	2020-10-11 23:38:31.024353
AGQ	ProShares Ultra Silver	The investment seeks to provide daily investment results (before fees and expenses) that correspond to twice (200%) the daily performance of silver bullion as measured by the U.S. Dollar fixing price for delivery in London.\nThe “Ultra” funds seek results for a single day that match (before fees and expenses) two times (2x) the daily performance of a benchmark. It does not seek to achieve their stated objective over a period greater than a single day.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.53963	2020-10-11 23:38:31.031328
AHH	Armada Hoffler Properties Inc.	Armada Hoffler Properties Inc is a full service real estate company that develops, constructs & owns institutional grade office, retail & multifamily properties in the Mid-Atlantic United States. It also offers general contracting & development services.	{"Real Estate","REIT - Diversified",REITs}	{NEN,IOR,LMRK}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:11:59.552083	2020-10-11 23:38:31.024353
AIMC	Altra Industrial Motion Corp.	Altra Industrial Motion Corp is a designer, producer and marketer of electro-mechanical power transmission products. Its products include clutches and brakes, overrunning clutches, engineered bearing assemblies, gearing and gear motors, and belted drives.	{Industrials,"Diversified Industrials","Industrial Products"}	{AMOT,ROLL,GE,RBC,TKR,KAMN,RXN}	Industrial Products	Industrials	Nasdaq Global Select	\N	2020-10-11 23:11:59.563357	2020-10-11 23:38:31.02582
GLBS	Globus Maritime Limited	Globus Maritime Ltd owns and operates fleet of dry bulk motor vessels. The Company provides maritime services for the transportation of dry cargo products.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{}	Transportation & Logistics	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:03.5769	2020-10-11 23:38:31.028166
ACIM	SPDR MSCI ACWI IMI	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the MSCI ACWI IMI Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index and in depositary receipts based on securities comprising the index. The index is a free float-adjusted market capitalization-weighted index that is designed to measure the combined equity market performance of developed and emerging markets. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.410434	2020-10-11 23:38:31.02423
ACU	Acme United Corporation.	Acme United Corp is a supplier of cutting, measuring, first aid and sharpening products to the school, home, office, hardware, sporting goods and industrial markets.	{"Consumer Defensive","Household & Personal Products","Consumer Packaged Goods"}	{JAKK,LCUT,NWL,JNJ,ACCO,SVT,EPC}	Consumer Packaged Goods	Consumer Defensive	NYSE American	\N	2020-10-11 23:11:59.433652	2020-10-11 23:38:31.028166
ADI	Analog Devices Inc.	Analog Devices Inc is engaged in the semiconductor industry. Its business involves designing, manufacturing and marketing of high-performance analog, mixed-signal and digital signal processing integrated circuits used in all types of electronic equipment.	{Technology,Semiconductors}	{MCHP,CRUS,MXIM,SMTC,SLAB,ON,STM,TXN,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:11:59.444388	2020-10-11 23:38:31.023007
ADRD	BLDRS Developed Markets 100 ADR Index Fund	The investment seeks to provide investment results that correspond generally, before fees and expenses, to the price and yield performance of the Bank of New York Mellon Developed Markets 100 ADR Index.\n The fund typically invests substantially all of its assets in the securities that make up the index. The index is intended to give investors a benchmark for tracking the price and yield performance of Developed Markets Depositary Receipts. the fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:11:59.454616	2020-10-11 23:38:31.023007
ADT	ADT Inc.	ADT Inc provides monitored security, interactive home and business automation and related monitoring services.	{Industrials,"Security & Protection Services","Consulting & Outsourcing"}	{}	Consulting & Outsourcing	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.464771	2020-10-11 23:38:31.031328
AEGN	Aegion Corp	Aegion Corp is an engineering and construction company combining technologies with market expertise. It provides solutions for the strengthening of buildings, bridges and other structures.	{Industrials,"Engineering & Construction"}	{STRL,PRIM,MTZ,DY,GVA,VVC,TISI}	Engineering & Construction	Industrials	Nasdaq Global Select	\N	2020-10-11 23:11:59.481756	2020-10-11 23:38:31.023007
AES	The AES Corporation	The AES Corp is a power generation and utility company. It is engaged in providing affordable, sustainable energy through its diverse portfolio of thermal and renewable generation facilities as well as distribution businesses.	{Utilities,"Utilities - Diversified","Utilities - Regulated"}	{ENIA,EOCC,SRE,AEP,D,NRG,ORA,SPWR,DUK,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:11:59.494436	2020-10-11 23:38:31.028166
AFH	Atlas Financial Holdings Inc.	Atlas Financial Holdings Inc underwrites commercial auto mobile insurance policies in the United States. It offers insurance for taxi cabs, limousine, non-emergency para-transit and livery and business autos.	{"Financial Services","Insurance - Property & Casualty"}	{STFC,ACGL,HALL}	Insurance - Property & Casualty	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:11:59.509558	2020-10-11 23:38:31.028166
AGEN	Agenus Inc.	Agenus Inc along with its subsidiaries is a pharmaceutical company. It is engaged in discovery and development of revolutionary new treatments that engage the body's immune system to benefit patients suffering from cancer.	{Healthcare,Biotechnology}	{CLDX,GSK,BMY,JNJ,MRK,STML,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:11:59.519301	2020-10-11 23:38:31.031328
AGGY	WisdomTree Yield Enhanced U.S. Aggregate Bond Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the Bloomberg Barclays U.S. Aggregate Enhanced Yield Index (the "index").\n Under normal circumstances, the fund will invest at least 80% of its total asset in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is designed to broadly capture the U.S. investment grade, fixed income securities market while seeking to enhance yield within desired risk parameters and constraints. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.527639	2020-10-11 23:38:31.021426
AGNCB	AGNC Investment Corp. Depositary Shares representing 1/1000th Series B Preferred Stock	AGNC Investment Corp is an internally-managed real estate investment trust. It invests in agency mortgage-backed securities on a leveraged basis, financed through collateralized borrowings structured as repurchase agreements.	{"Real Estate","REIT - Residential",REITs}	{}	REITs	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:11:59.537164	2020-10-11 23:38:31.02582
AGTC	Applied Genetic Technologies Corporation	Applied Genetic Technologies Corp is a clinical-stage biotechnology company focused on gene therapy through a nontoxic adeno-associated virus. The company also offers products such as promoters, expression cassettes, and capsids for gene therapy.	{Healthcare,Biotechnology}	{BLUE,OHRP,DRNA,EIGR,ONVO,ACER,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:11:59.547336	2020-10-11 23:38:31.028166
AI	Arlington Asset Investment Corp Class A (new)	Arlington Asset Investment Corp is an investment firm. It focuses on acquiring and holding a levered portfolio of residential mortgage-backed securities (MBS) consisting of agency MBS and private-label MBS.	{"Financial Services","Specialty Finance",Banks}	{IMH,EFC,WPC}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.558698	2020-10-11 23:38:31.028166
ACIA	Acacia Communications Inc.	Acacia Communications Inc provides optical interconnect products used in communication services industry. It offers sophisticated modules for digital signal processing (DSP) and optical functions required to process network traffic across the world.	{Technology,"Communication Equipment"}	{SQNS,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:11:59.409781	2020-10-11 23:38:31.021426
ACP	Aberdeen Income Credit Strategies Fund	Avenue Income Credit Strategies Fund is a non diversified, closed-end management investment company. Its investment objective is to seek a high level of current income with a secondary objective of capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.425237	2020-10-11 23:38:31.031328
ACWI	iShares MSCI ACWI Index Fund	The investment seeks to track the investment results of the MSCI ACWI composed of large- and mid-capitalization developed and emerging market equities.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is a free float-adjusted market capitalization index designed to measure the combined equity market performance of developed and emerging markets countries.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:11:59.436876	2020-10-11 23:38:31.025871
ADM	Archer-Daniels-Midland Company	Archer-Daniels Midland Co is a major processor of oilseeds, corn, wheat, and other agricultural commodities. The company's end products include vegetable oil and meal, corn sweeteners, flour, feed ingredients, and ethanol.	{"Consumer Defensive","Farm Products","Consumer Packaged Goods"}	{TSN,BG,THS,CZZ,SEB,MGPI,INGR}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:11:59.446193	2020-10-11 23:38:31.023428
ADRO	Aduro Biotech Inc.	Aduro Biotech Inc is a clinical-stage biotechnology company focused on the discovery, development and commercialization of therapies that transform the treatment of different diseases.	{Healthcare,Biotechnology}	{SRPT,AFMD,PRTO,CHRS,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.457307	2020-10-11 23:38:31.02582
ADX	Adams Diversified Equity Fund Inc.	Adams Diversified Equity Fund Inc is a diversified investment company. Its investment objectives is preservation of capital, the attainment of reasonable income from investments, and an opportunity for capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.47022	2020-10-11 23:38:31.023007
AEG	AEGON N.V.	Aegon NV is engaged in providing life insurance, pensions and asset management products and services in the USA, Europe, and Asia. It offers life and protection, and accidental death products as well as individual savings and retirement products.	{"Financial Services","Insurance - Diversified",Insurance}	{PUK,LNC,MET}	Insurance	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.480876	2020-10-11 23:38:31.02582
AEY	ADDvantage Technologies Group Inc.	Addvantage Technologies Group Inc sells new, surplus and refurbished cable television equipment to cable television operators or other resellers. The Company's business segments are Cable Television and Telecommunications.	{Technology,"Electronics Distribution","Computer Hardware"}	{CSCO,TESS,KEQU,CMCSA,BDR,GLW,KOOL,XLK}	Computer Hardware	Technology	NASDAQ Global Market	\N	2020-10-11 23:11:59.497064	2020-10-11 23:38:31.023007
AFMD	Affimed N.V.	Affimed NV is a clinical-stage biopharmaceutical company. It is engaged in discovering and developing targeted cancer immunotherapies. Its product candidates are being developed in the field of immuno-oncology to fight tumor cells.	{Healthcare,Biotechnology}	{ACOR,ADRO,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:11:59.511821	2020-10-11 23:38:31.031328
AGG	iShares Core U.S. Aggregate Bond	The investment seeks to track the investment results of the Bloomberg Barclays U.S. Aggregate Bond Index.\n The index measures the performance of the total U.S. investment-grade bond market. The fund generally invests at least 90% of its net assets in component securities of its underlying index and in investments that have economic characteristics that are substantially identical to the economic characteristics of the component securities of its underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.527785	2020-10-11 23:38:31.028166
AGRO	Adecoagro S.A.	Adecoagro SA is an agricultural company in South America, with operations in Argentina, Brazil and Uruguay. It is engaged in farming crops and other agricultural products, dairy operations, sugar, ethanol and energy production and land transformation.	{"Consumer Defensive","Farm Products","Consumer Packaged Goods"}	{CRESY,BG,SEB}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:11:59.53977	2020-10-11 23:38:31.023007
AHC	A.H. Belo Corporation (TX)	A H Belo Corp is a local news and information publishing company with commercial printing, distribution and direct mail capabilities, as well as expertise in emerging media and digital marketing.	{"Consumer Cyclical",Publishing}	{DJCO,LEE,NYT,NWSA,NEWM,TGNA,MNI,SSP}	Publishing	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:11:59.5511	2020-10-11 23:38:31.025871
AIG	American International Group Inc.	American International Group Inc is an insurance company. The company through its subsidiaries provides property casualty insurance, life insurance, retirement products, mortgage insurance and other financial services in the United States and abroad.	{"Financial Services","Insurance - Diversified",Insurance}	{ACGL,SAFT,PUK,AEG,AFG,HMN,WTM}	Insurance	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.563234	2020-10-11 23:38:31.025871
AIT	Applied Industrial Technologies Inc.	Applied Industrial Technologies Inc is an industrial distributor in North America, Australia, and New Zealand, serving MRO (maintenance, repair, and operations) and OEM (original equipment manufacturing) customers in diversified industries.	{Industrials,"Industrial Distribution"}	{FAST,DXPE,LAWS,MSM,AXE,WCC,CW,GPC,GWW,KAMN}	Industrial Distribution	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.574603	2020-10-11 23:38:31.02423
ACIU	AC Immune SA	AC Immune SA is a clinical stage biopharmaceutical company. It discovers, designs and develops novel, proprietary medicines for treatment of neurodegenerative diseases with the help of proprietary platforms.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:11:59.417663	2020-10-11 23:38:31.02582
ACRX	AcelRx Pharmaceuticals Inc.	AcelRx Pharmaceuticals Inc is a specialty pharmaceutical company. It is engaged in the development and commercialization of therapies for the treatment of acute pain.	{Healthcare,"Medical Devices"}	{MDCO,DRRX,BDSI,INSY,REPH,CARA,XLV}	Medical Devices	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:11:59.428715	2020-10-11 23:38:31.025871
ACY	AeroCentury Corp.	AeroCentury Corp is an aircraft operating lessor and finance company which specializes in leasing regional turboprop and jet aircraft and engines to regional air carriers.	{Industrials,"Rental & Leasing Services","Consulting & Outsourcing"}	{TGH,AER,AL,TRTN}	Consulting & Outsourcing	Industrials	NYSE American	\N	2020-10-11 23:11:59.441352	2020-10-11 23:38:31.028166
AE	Adams Resources & Energy Inc.	Adams Resources & Energy Inc along with its subsidiaries is engaged in marketing of crude oil, natural gas and petroleum products; tank truck transportation of liquid chemicals; and oil and gas exploration and production.	{Energy,"Oil & Gas Integrated","Oil & Gas - Integrated"}	{INT,GEL,GLP,SPH,DKL,PBFX,CEQP}	Oil & Gas - Integrated	Energy	NYSE American	\N	2020-10-11 23:11:59.473133	2020-10-11 23:38:31.023007
AEM	Agnico Eagle Mines Limited	Agnico Eagle Mines Ltd is a Canadian gold mining company. Its mining properties are located in Canada, Finland and Mexico as well as in United States and Sweden.	{"Basic Materials",Gold,"Metals & Mining"}	{EGO,AUY,NGD,CDE,HMY,AGI}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:11:59.483596	2020-10-11 23:38:31.024353
ADSK	Autodesk Inc.	Autodesk Inc is engaged in the software industry. It develops software for a number of markets such as construction and media.	{Technology,"Software - Application","Application Software"}	{ANSS,FFIV,ADBE,AAPL,SNPS,TRMB,NUAN,RHT,DDD,PTC,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:11:59.463684	2020-10-11 23:38:31.021426
AFB	AllianceBernstein National Municipal Income Fund Inc	Alliancebernstein National Muni Inc Fd is a diversified, closed-end management investment company. It seeks to provide high current income exempt from regular federal income tax by investing substantially all of its net assets in municipal securities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.498473	2020-10-11 23:38:31.031328
AG	First Majestic Silver Corp. (Canada)	First Majestic Silver Corp. engages in the acquisition, exploration, development, and production of mineral properties with a focus on silver projects in Mexico.	{"Basic Materials",Silver,"Metals & Mining"}	{PAAS,MAG,EXK,SVM,AXU,HL,GPL,WPM}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:11:59.511915	2020-10-11 23:38:31.02423
AGGP	IQ Enhanced Core Plus Bond U.S.	The investment seeks to track the price and yield performance of its underlying index, the IQ Enhanced Core Plus Bond U.S. Index.\n The fund is a "fund of funds" which means it invests, under normal circumstances, at least 80% of its net assets, plus the amount of any borrowings for investment purposes, in the investments included in its underlying index, which includes underlying funds. The underlying index seeks to outperform the U.S. dollar-denominated taxable fixed income universe by using a combination of short- and long-term momentum to overweight and underweight various sectors of the investment grade and high yield fixed income securities market.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.528202	2020-10-11 23:38:31.02582
AGNCN	AGNC Investment Corp. Depositary Shares Each Representing a 1/1000th Interest in a Share of 7.00% Series C Fixed-To-Floating Rate Cumulative Redeemable Preferre	AGNC Investment Corp is an internally-managed real estate investment trust. It invests in agency mortgage-backed securities on a leveraged basis, financed through collateralized borrowings structured as repurchase agreements.	{"Real Estate","REIT - Residential",REITs}	{}	REITs	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:11:59.537777	2020-10-11 23:38:31.025871
AGX	Argan Inc.	Argan Inc through its subsidiary operates in the power industry services segment. It provides a full range of engineering, procurement and construction services for power-generating projects. It also engages in industrial fabrication and field services.	{Industrials,"Engineering & Construction"}	{SCHN,NDSN,TAYD,PRIM,GV,KBR}	Engineering & Construction	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.548399	2020-10-11 23:38:31.028166
AHT	Ashford Hospitality Trust Inc	Ashford Hospitality Trust Inc is engaged in the real estate sector. It is focused on investing in the hospitality industry, especially on full-service upscale and upper-upscale hotels.	{"Real Estate","REIT - Hotel & Motel",REITs}	{LHO,HST,DRH,CHSP,RLJ,INN,SOHO,HT,HPT}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:11:59.558638	2020-10-11 23:38:31.028166
AIR	AAR Corp.	AAR Corp along with its subsidiaries is engaged in providing products & services including overhaul, repair & engineering services to aviation & government & defense markets. Its business segments are Aviation Services and Expeditionary Services.	{Industrials,"Aerospace & Defense"}	{LMT,BA,SPR,ERJ,SAVE}	Aerospace & Defense	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.569181	2020-10-11 23:38:31.021426
AJG	Arthur J. Gallagher & Co.	Arthur J. Gallagher & Co is engaged in providing insurance brokerage and consulting services and third-party claims settlement and administration services to both domestic and international entities.	{"Financial Services","Insurance Brokers","Brokers & Exchanges"}	{BRO,MMC,AON,WLTW}	Brokers & Exchanges	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.583588	2020-10-11 23:38:31.021426
AFG	American Financial Group Inc.	American Financial Group Inc is a property and casualty insurance company that operates through its subsidiaries. It sells fixed and fixed-indexed annuities in the retail, financial institutions and education markets.	{"Financial Services","Insurance - Property & Casualty"}	{CINF,HIG,WRB,L,MKL,RLI,TRV}	Insurance - Property & Casualty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.5002	2020-10-11 23:38:31.023428
AFL	AFLAC Incorporated	Aflac Inc is a general business holding company and acts as a management company. It offers supplemental health and life insurance in the U.S. and Japan.	{"Financial Services","Insurance - Life"}	{AAME,UNM,CI,AIZ,CNO,PFG}	Insurance - Life	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.511362	2020-10-11 23:38:31.024353
AGIO	Agios Pharmaceuticals Inc.	Agios Pharmaceuticals Inc is a biopharmaceutical company which focuses on developing treatments geared towards cancer and rare genetic disorders of metabolism. It also emphasizes on developing potentially transformative small-molecule medicines.	{Healthcare,Biotechnology}	{ALXN,BMRN,CELG,EXEL,GSK,LLY,PFE,XNCR,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.52803	2020-10-11 23:38:31.031328
AGO	Assured Guaranty Ltd.	Assured Guaranty Ltd is in the financial services industry. The company provides insurance that protects holders of debt instruments and other monetary obligations from defaults in scheduled payments.	{"Financial Services","Insurance - Specialty"}	{ACGL,RDN,MBI,ESNT}	Insurance - Specialty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.539034	2020-10-11 23:38:31.028166
AGZ	iShares Agency Bond	The investment seeks to track the investment results of the Bloomberg Barclays U.S. Agency Bond Index.\n The underlying index measures the performance of the agency sector of the U.S. government bond market and is comprised of investment-grade U.S. dollar-denominated publicly-issued government agency bonds or debentures. The fund generally will invest at least 90% of its assets in the component securities of the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.549281	2020-10-11 23:38:31.023007
AIA	iShares Asia 50 ETF	The investment seeks to track the S&P Asia 50TM, which is a total float-adjusted, market capitalization-weighted index that is designed to measure the performance of the 50 leading companies listed in four Asian countries or regions: Hong Kong, Singapore, South Korea and Taiwan.\n The fund generally will invest at least 90% of its assets in the component securities of the index and in investments that have economic characteristics that are substantially identical to the component securities and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:11:59.560238	2020-10-11 23:38:31.023007
AIRG	Airgain Inc.	Airgain Inc provides embedded antenna technologies used to enable high performance wireless networking across home, enterprise, & industrial devices. It designs, develops, and engineers its antenna products for original equipment & design manufacturers.	{Technology,"Communication Equipment"}	{NOK,MXL,RESN,XLK}	Communication Equipment	Technology	NASDAQ Capital Market	\N	2020-10-11 23:11:59.572093	2020-10-11 23:38:31.023007
AJRD	Aerojet Rocketdyne Holdings Inc.	Aerojet Rocketdyne Holdings Inc manufactures aerospace and defense products and systems which develops and manufactures propulsion systems for defense and space applications, and armaments for precision tactical and long-range weapon systems applications.	{Industrials,"Aerospace & Defense"}	{LMT,BA,MOG.A,NOC,ASTC}	Aerospace & Defense	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.584066	2020-10-11 23:38:31.02582
AKS	AK Steel Holding Corporation	AK Steel Holding Corp is engaged in manufacturing steel products such as flat-rolled, carbon, electrical, and stainless steel as well as tubular products for the automotive, construction, appliance, and machinery end markets.	{"Basic Materials",Steel}	{SCHN,USAP,STLD,AP,CRS,CMC,NUE,ATI,X,MT}	Steel	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:11:59.596889	2020-10-11 23:38:31.021426
ALDR	Alder BioPharmaceuticals Inc.	Alder BioPharmaceuticals Inc is a biotechnology company. It is engaged in the discovery, development, and commercialization of therapeutic antibodies addresses diseases with limited medical options. The company operates across the U.S and Australia.	{Healthcare,Biotechnology}	{AMGN,BSTC,BMY,JNJ,LLY,PFE,ABBV,RDUS,HZNP,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:11:59.60985	2020-10-11 23:38:31.028166
ALKS	Alkermes plc	Alkermes PLC is a fully integrated, biopharmaceutical company. It applies its scientific expertise and proprietary technologies to research, develop and commercialize pharmaceutical products to address unmet medical needs of patients.	{Healthcare,Biotechnology}	{ACOR,AZN,NVS,BMY,JNJ,PFE,SNY,TEVA,ZGNX,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.629245	2020-10-11 23:38:31.023007
ALSN	Allison Transmission Holdings Inc.	Allison Transmission Holdings Inc is engaged in the design amd manufacturing of commercial and defense fully-automatic transmissions. It manufactures transmissions for medium- and heavy-duty commercial vehicles and defense vehicles.	{"Consumer Cyclical","Auto Parts",Autos}	{SYPR,SPAR,NAV,HON,THO,WGO,MTOR,CNHI}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:11:59.641657	2020-10-11 23:38:31.024353
GLO	Clough Global Opportunities Fund	Clough Global Opportunities Fund is a closed-end investment fund, which operates with the objective of generating capital appreciation and income for its investors.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:03.592349	2020-10-11 23:38:31.028166
AEMD	Aethlon Medical Inc.	Aethlon Medical Inc is a medical technology company. It operates through two segments namely Aethlon for therapeutic applications and ESI for diagnostic applications.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{BASI,COHR,BRKR,PKI,PACB,SPA,AXDX,INO,XLK}	Computer Hardware	Technology	NASDAQ Capital Market	\N	2020-10-11 23:11:59.484565	2020-10-11 23:38:31.023007
AERI	Aerie Pharmaceuticals Inc.	Aerie Pharmaceuticals Inc is a clinical-stage pharmaceutical company. It is engaged in the discovery, development and commercialization of first-in-class therapies for the treatment of patients with glaucoma and other diseases of the eye.	{Healthcare,"Drug Manufacturers - Major","Drug Manufacturers"}	{ALIM,MRK,INSM,OPHT,LXRX,XLV}	Drug Manufacturers	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:11:59.494462	2020-10-11 23:38:31.023007
AFI	Armstrong Flooring Inc.	Armstrong Flooring Inc designs, manufactures, sources, and sells resilient and wood flooring products, which are primarily used for construction and renovation of residential, commercial, and institutional buildings.	{"Basic Materials","Building Materials"}	{AFI}	Building Materials	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:11:59.509558	2020-10-11 23:38:31.025871
AGCO	AGCO Corporation	AGCO Corp is a Georgia-based company which manufactures and distributes agricultural equipment and related replacement parts in various parts of the country. It sells a full range of agricultural equipment, including tractors, combines, among others.	{Industrials,"Farm & Construction Equipment","Farm & Construction Machinery"}	{ARTW,ALG,DE,LNN,TTC}	Farm & Construction Machinery	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.518791	2020-10-11 23:38:31.02582
AGLE	Aeglea BioTherapeutics Inc.	Aeglea BioTherapeutics Inc is a biopharmaceutical company committed to developing enzyme-based therapeutics in the field of amino acid metabolism, which will transform the lives of patients with inborn errors of metabolism and cancer.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:11:59.528148	2020-10-11 23:38:31.023007
AGND	WisdomTree Barclays U.S. Aggregate Bond Negative D	The investment seeks to track the price and yield performance, before fees and expenses, of the Bloomberg Barclays Rate Hedged U.S. Aggregate Bond Index, Negative Five Duration (the "index").\n The index is designed to provide long exposure to the Bloomberg Barclays U.S. Aggregate Bond Index while seeking to manage interest rate risk through the use of short positions in U.S. Treasury securities. The fund employs a "passive management" - or indexing - investment approach designed to track the performance of the index. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:11:59.539069	2020-10-11 23:38:31.023428
AGYS	Agilysys Inc.	Agilysys Inc develops industry-specific technology solutions for business management. The company offers solutions to manage business operations such as property management, dining reservations, labor management, and document management.	{Technology,"Software - Application","Application Software"}	{TECD,BYD,AXE,NCR,MTN,SNX,PAR,CZR,NCLH,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:11:59.549088	2020-10-11 23:38:31.02423
AIEQ	AI Powered Equity	The investment seeks capital appreciation.\n The fund is actively managed and invests primarily in equity securities listed on a U.S. exchange based on the results of a proprietary, quantitative model (the "EquBot Model") developed by EquBot LLC ("EquBot") that runs on the Watson™ platform. EquBot, the fund's sub-adviser, is a technology based company focused on applying artificial intelligence ("AI") based solutions to investment analyses. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.562153	2020-10-11 23:38:31.021426
AIRI	Air Industries Group	Air Industries Group is an aerospace and defense company. It manufactures and designs structural parts and assemblies that focus on flight safety, including landing gear, flight controls, throttle quadrants, jet engines and other components.	{Industrials,"Aerospace & Defense"}	{ESLT,ATRO,TGI,CVU,DCO,TDG}	Aerospace & Defense	Industrials	NYSE American	\N	2020-10-11 23:11:59.572723	2020-10-11 23:38:31.02582
AJX	Great Ajax Corp.	Great Ajax Corp is engaged in acquiring, investing in and managing a portfolio of mortgage loans. The Company operates in a single segment focused on non-performing mortgages and re-performing mortgages.	{"Real Estate","REIT - Diversified",REITs}	{}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:11:59.583866	2020-10-11 23:38:31.023007
AKRX	Akorn Inc.	Akorn Inc is a United States based generic pharmaceutical company. It develops, manufactures and markets generic and branded prescription pharmaceuticals as well as private-label over-the-counter consumer health products and animal health pharmaceuticals.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{NVS,ABT,JNJ,PFE,MRK,TEVA,MYL,XLV}	Drug Manufacturers	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.597057	2020-10-11 23:38:31.02582
ALBO	Albireo Pharma Inc.	Albireo Pharma Inc is a development-stage pharmaceutical company. It engages in the research and development of the drug reformulation technology.	{Healthcare,Biotechnology}	{NKTR,HALO,NVO,LLY,MRK,ESPR,LXRX,MNKD,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:11:59.609441	2020-10-11 23:38:31.02423
ALL	Allstate Corporation (The)	Allstate Corp, through its subsidiaries, is primarily engaged in the property-liability insurance business and the life insurance, retirement and investment products business. It offers its products in the United States and Canada.	{"Financial Services","Insurance - Property & Casualty"}	{SAFT,UNM,HMN,LNC,PGR,PRU,TRV}	Insurance - Property & Casualty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.629516	2020-10-11 23:38:31.021426
ALSK	Alaska Communications Systems Group Inc.	Alaska Communications Systems Group Inc provides integrated communications services in Alaska. It provides local, long-distance, and wireless telephone services, Internet access, and data services to residential and business customers.	{"Communication Services","Telecom Services"}	{CNSL,CTL,VZ,T,HCOM,CBB}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:11:59.641638	2020-10-11 23:38:31.028166
AFK	VanEck Vectors-Africa Index	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the MVIS® GDP Africa Index.\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The index includes local listings of companies that are incorporated in Africa and listings of companies incorporated outside of Africa but that have at least 50% of their revenues/related assets in Africa.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.511067	2020-10-11 23:38:31.028166
AGFSW	AgroFresh Solutions Inc. Warrants		{}	{}			Nasdaq Global Select	\N	2020-10-11 23:11:59.525909	2020-10-11 23:38:31.023007
AGNC	AGNC Investment Corp.	AGNC Investment Corp is an internally-managed real estate investment trust. It invests in agency mortgage-backed securities on a leveraged basis, financed through collateralized borrowings structured as repurchase agreements.	{"Real Estate","REIT - Residential",REITs}	{}	REITs	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:11:59.534065	2020-10-11 23:38:31.021426
AGT	iShares MSCI Argentina and Global Exposure	The investment seeks to track the investment results of the MSCI All Argentina 25/50 Index (the "underlying index").\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is designed to measure the broad Argentina equity universe, while including a minimum number of constituents, as defined by MSCI, Inc. (the "index provider" or "MSCI"). The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:11:59.542086	2020-10-11 23:38:31.023428
AHPI	Allied Healthcare Products Inc.	Allied Healthcare Products Inc is engaged in manufacturing a variety of respiratory products used in the healthcare industry in a wide range of hospital and alternate site settings, including sub-acute care facilities and emergency medical care.	{Healthcare,"Medical Devices"}	{MASI,CAH,RMD,GE,TFX,TMO,ELMD,DYNT,INGN,XLV}	Medical Devices	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:11:59.555245	2020-10-11 23:38:31.023428
AIN	Albany International Corporation	Albany International Corp engages in the business of advanced textiles and materials processing. The company operates across the world offering machine clothing products, engineered fabric, and engineered composites for automotive and aerospace industry.	{"Consumer Cyclical","Textile Manufacturing","Homebuilding & Construction"}	{CRWS,GE,KAI,TILE}	Homebuilding & Construction	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:11:59.566419	2020-10-11 23:38:31.023428
AIW	Arlington Asset Investment Corp 6.625% Notes due 2023	Arlington Asset Investment Corp is an investment firm. It focuses on acquiring and holding a levered portfolio of residential mortgage-backed securities (MBS) consisting of agency MBS and private-label MBS.	{"Financial Services","Specialty Finance",Banks}	{}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.578731	2020-10-11 23:38:31.025871
AKER	Akers Biosciences Inc	Akers Biosciences Inc mainly develops and manufactures rapid, point-of-care screening and testing products designed to bring health-related information directly to the patient or clinician in a time- and cost-efficient manner.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{XLV}	Medical Instruments & Equipment	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:11:59.593709	2020-10-11 23:38:31.025871
ALE	Allete Inc.	ALLETE Inc is engaged in the genration, transmission and distribution of electric power in the United States.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{AT,LNT,AEE,EXC,NWE,NEE,DUK,WEC,XEL,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:11:59.609518	2020-10-11 23:38:31.025871
ALIM	Alimera Sciences Inc.	Alimera Sciences Inc is a biopharmaceutical company which is engaged in the research, development, and commercialization of ophthalmic pharmaceuticals.	{Healthcare,"Drug Manufacturers - Major","Drug Manufacturers"}	{REGN,INSM,OPHT,AERI,XLV}	Drug Manufacturers	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:11:59.629088	2020-10-11 23:38:31.031328
ALNA	Allena Pharmaceuticals Inc.	Allena Pharmaceuticals Inc is a clinical biopharmaceutical company. It focuses on developing and commercializing oral enzyme therapeutics to treat patients with rare and severe metabolic and kidney disorders.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.638864	2020-10-11 23:38:31.023007
ALTR	Altair Engineering Inc.	Altair Engineering Inc is a provider of enterprise-class engineering software. It also provides services related to its software including consulting, training, implementation services, and support, along with technical services for product development.	{Technology,"Software - Infrastructure","Application Software"}	{OTEX,PANW,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:11:59.651801	2020-10-11 23:38:31.028166
AMBC	Ambac Financial Group Inc.	Ambac Financial Group Inc through its subsidiaries provides insurance and other financial services. The company's Financial Guarantee segment offers insurance policies and credit derivative contracts.	{"Financial Services","Insurance - Specialty"}	{RDN,STC,UIHC}	Insurance - Specialty	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:11:59.665068	2020-10-11 23:38:31.028166
AMGN	Amgen Inc.	Amgen Inc is a U.S. based biotechnology company, with historical expertise in renal disease and cancer supportive care products. The firm discovers, develops, manufactures and delivers human therapeutics.	{Healthcare,Biotechnology}	{ALXN,CELG,GILD,BIIB,GSK,BMY,JNJ,PFE,ABBV,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.679343	2020-10-11 23:38:31.02582
MVO	MV Oil Trust	MV Oil Trust is a trust that receives royalty payments from the oil and natural gas assets owned by MV Partners, which produces oil, natural gas and natural gas liquids.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:07.390011	2020-10-11 23:38:31.028166
AFC	Allied Capital Corporation 6.875% Notes due April 15 2047	Ares Capital Corp is a specialty finance company that is a closed-end, non-diversified management investment company. Its investment objective is to generate both current income and capital appreciation through debt and equity investments.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.498552	2020-10-11 23:38:31.021426
AFHBL	Atlas Financial Holdings Inc. 6.625% Senior Unsecured Notes Due 2022		{}	{}			NASDAQ Global Market	\N	2020-10-11 23:11:59.509795	2020-10-11 23:38:31.02582
AGM	Federal Agricultural Mortgage Corporation	Federal Agricultural Mortgage Corp provides agricultural real estate and rural housing mortgage loans in the secondary market in the U.S. Its operations consist of four segments: Farm & Ranch, USDA Guarantees, Rural Utilities, and Institutional Credit.	{"Financial Services","Credit Services"}	{CIM}	Credit Services	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.531141	2020-10-11 23:38:31.02423
AGRX	Agile Therapeutics Inc.	Agile Therapeutics Inc is a women's health specialty pharmaceutical company. The Company is engaged in the development and commercialization of transdermal patch technology for use in contraception products.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{JNJ,PFE,MRK,ATRS,TXMD,CORI,MYL,XLV}	Drug Manufacturers	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:11:59.541821	2020-10-11 23:38:31.021426
AHPAU	Avista Healthcare Public Acquisition Corp. Unit	Avista Healthcare Public Acquisition Corp is a newly organized blank check company. The Company was formed for the purpose of effecting a merger, share exchange, asset acquisition, stock purchase, reorganization or similar business combination.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:11:59.55405	2020-10-11 23:38:31.02582
AINC	Ashford Inc. (Holding Company)	Ashford Inc provides asset management and advisory services to other entities, within the hospitality industry.	{"Financial Services","Asset Management"}	{GDOT}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:11:59.566518	2020-10-11 23:38:31.02423
AIY	Apollo Investment Corporation 6.875% Senior Notes due 2043	Apollo Investment Corp is a closed-end, non-diversified management investment company. It invests in various forms of debt investments including secured and unsecured loans, mezzanine investments, and/or equity in private middle-market companies.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.580921	2020-10-11 23:38:31.031328
AKP	Alliance California Municipal Income Fund Inc	Alliance California Municipal Income Fund Inc is a closed-end management investment company. The fund seeks to provide high current income exempt from regular federal and california state income tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.594559	2020-10-11 23:38:31.023007
ALCO	Alico Inc.	Alico Inc is an agribusiness and land management company. It is engaged in production of citrus groves, improved farmland, cattle ranching and conservation, and related support operations.	{"Consumer Defensive","Farm Products","Consumer Packaged Goods"}	{LMNR,FDP,BG}	Consumer Packaged Goods	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:11:59.611141	2020-10-11 23:38:31.02582
ALJJ	ALJ Regional Holdings Inc.	ALJ Regional Holdings Inc is a holding company. The company operates through its subsidiaries namely, Faneuil Ind, Floors-N-More LLC and Phoenix Color Corp.	{Industrials,"Business Services"}	{SCHN,AKS,NUE,X}	Business Services	Industrials	NASDAQ Global Market	\N	2020-10-11 23:11:59.629185	2020-10-11 23:38:31.02582
ALO	Alio Gold Inc. (Canada)	Alio Gold Inc is a gold mining company. It is engaged in exploration, development and production in Mexico. Its projects include San Francisco mine in Sonora, Mexico and the development stage Ana Paula project in Guerrero, Mexico.	{"Basic Materials",Gold,"Metals & Mining"}	{MUX}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:11:59.640767	2020-10-11 23:38:31.02423
AMAG	AMAG Pharmaceuticals Inc.	AMAG Pharmaceuticals Inc is a biopharmaceutical company, which utilizes the technology for the development and commercialization of a therapeutic iron compound to treat iron deficiency anemia and to aid in the diagnosis of cancer.	{Healthcare,Biotechnology}	{RMTI,AKRX,HOLX,KERX,ABC,SNY,NAVB,TXMD,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.658953	2020-10-11 23:38:31.031328
AMD	Advanced Micro Devices Inc.	Advanced Micro Devices Inc designs and produces microprocessors and low-power processor solutions for the computer, communications, and consumer electronics industries.	{Technology,Semiconductors}	{INTC,MCHP,NVDA,TXN,HPQ,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:11:59.670358	2020-10-11 23:38:31.024353
AMLP	Alerian MLP	The investment seeks total return, with an emphasis on income as the source of that total return.\n The fund seeks to achieve its investment objective by investing in a global portfolio of fixed income securities of various maturities, ratings and currency denominations. The adviser intends to utilize various investment strategies in a broad array of fixed income sectors. The fund may purchase fixed income securities issued by U.S. or foreign corporations or financial institutions, including debt securities of all types and maturities, convertible securities and preferred stocks. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.69184	2020-10-11 23:38:31.024353
AIZ	Assurant Inc.	Assurant Inc is a provider of specialized insurance products and related services in North America and selected other international markets.	{"Financial Services","Insurance - Diversified",Insurance}	{HIG,UNM,AFL,MET,UNH,AET,CNO,PFG}	Insurance	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.583689	2020-10-11 23:38:31.023428
AKR	Acadia Realty Trust	Acadia Realty Trust is a real estate investment trust which focuses on the ownership, acquisition, development, and management of high-quality retail properties located primarily in the United States.	{"Real Estate","REIT - Retail",REITs}	{CUZ,COR}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:11:59.596812	2020-10-11 23:38:31.028166
ALD	WisdomTree Asia Local Debt Fund	The investment seeks a high level of total return consisting of both income and capital appreciation.\n The fund normally invests at least 80% of its net assets, plus the amount of any borrowings for investment purposes, in Local Debt. The advisor intends to focus its investments on Local Debt issued by governments (national, state, and local), government agencies and instrumentalities, and government sponsored enterprises. The fund may invest up to 20% of its assets in debt instruments denominated in U.S. dollars issued by Asian governments and government sponsored enterprises. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.609827	2020-10-11 23:38:31.031328
ALK	Alaska Air Group Inc.	Alaska Air Group Inc operates Alaska, Virgin America and Horizon Air airlines. The company through its subsidiaries provides passenger air, freight and mail services within the state of Alaska and on the West Coast.	{Industrials,Airlines}	{SKYW,JBLU,HA,ALGT,LUV,DAL,AAL,UAL}	Airlines	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.629095	2020-10-11 23:38:31.025871
ALPN	Alpine Immune Sciences Inc.	Alpine Immune Sciences Inc is engaged in discovering and developing modern, protein-based immunotherapies targeting the immune synapse to treat cancer, inflammation, and other diseases.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:11:59.640391	2020-10-11 23:38:31.021426
AMCA	iShares Russell 1000 Pure U.S. Revenue ETF	The investment seeks to track the investment results of the Russell 1000® Pure Domestic Exposure Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index. The index composed of U.S. companies exhibiting higher domestic sales as a proportion of the company's total sales relative to other large- and mid-capitalization U.S. equities. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:11:59.668728	2020-10-11 23:38:31.031328
AMID	American Midstream Partners LP representing Limited Partner Interests	American Midstream Partners LP is the United States-based natural gas company. It owns, operates, develops and acquires a portfolio of natural gas midstream energy assets.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{ENB,TRP,EPD,DCP,SEP,KMI,ENLK}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:11:59.683214	2020-10-11 23:38:31.023428
AMPH	Amphastar Pharmaceuticals Inc.	Amphastar Pharmaceuticals Inc is a specialty pharmaceutical company engaged in developing, manufacturing, marketing and selling generic and proprietary injectable & inhalation products. Its products are used in hospitals or urgent care clinical settings.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{OSIR,SCYX,AGRX,XLV}	Drug Manufacturers	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.701432	2020-10-11 23:38:31.021426
AMSF	AMERISAFE Inc.	AMERISAFE Inc is a specialty provider of workers' compensation insurance to employers engaged in hazardous industries, mainly construction, trucking, manufacturing, oil and gas and agriculture.	{"Financial Services","Insurance - Property & Casualty"}	{ANAT,ACGL,MKL,EIG,TRV}	Insurance - Property & Casualty	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:11:59.715035	2020-10-11 23:38:31.023428
AN	AutoNation Inc.	AutoNation Inc is an automotive retailer in the United States. The company offers all types of automotive products and services, including new vehicles, used vehicles, parts and accessories, automotive services, automotive finance and insurance products.	{"Consumer Cyclical","Auto & Truck Dealerships",Autos}	{CPRT,RUSHA,LAD,GPI,SAH,ABG,KMX,PAG}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:11:59.72639	2020-10-11 23:38:31.028166
ANH	Anworth Mortgage Asset Corporation	Anworth Mortgage Asset Corp is engaged in the business of investment, financing and management of leveraged portfolio of residential mortgage-backed securities and residential mortgage loans which presently includes different types of investments.	{"Real Estate","REIT - Diversified",REITs}	{NYMT,NLY,DX,MFA,TWO}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:11:59.740236	2020-10-11 23:38:31.021426
AOM	iShares Core Moderate Allocation	The investment seeks to track the investment results of the S&P Target Risk Moderate Index composed of a portfolio of underlying equity and fixed income funds intended to represent a moderate target risk allocation strategy.\n The fund is a fund of funds and seeks its investment objective by investing primarily in other iShares Underlying Funds that themselves seek investment results corresponding to their own respective underlying indexes. It generally will invest at least 90% of its assets in the component securities of the underlying index. The index measures the performance of the S&P Dow Jones Indices LLC proprietary allocation model.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.753718	2020-10-11 23:38:31.028166
AJXA	Great Ajax Corp. 7.25% Convertible Senior Notes due 2024		{}	{}			New York Stock Exchange	\N	2020-10-11 23:11:59.588778	2020-10-11 23:38:31.028166
AKTS	Akoustis Technologies Inc.	Akoustis Technologies Inc operates in the telecommunications and fiber optics sector. The company designs and manufactures innovative radio frequency (RF) filters enabling the RF front-end (RFFE) of Mobile Wireless devices.	{Technology,"Communication Equipment"}	{XLK}	Communication Equipment	Technology	NASDAQ Capital Market	\N	2020-10-11 23:11:59.600597	2020-10-11 23:38:31.028166
ALEX	Alexander & Baldwin Inc. REIT Holding Company	Alexander & Baldwin Inc is primarily a real estate company. It operates through three segments namely Commercial real estate, land operations and materials and construction. Most of its revenue is derived from managing retail and office properties.	{"Real Estate","REIT - Diversified",REITs}	{SSW,DAC,GNK}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:11:59.617348	2020-10-11 23:38:31.028166
ALRN	Aileron Therapeutics Inc.	Aileron Therapeutics Inc is a clinical-stage biopharmaceutical company that is focused on developing and commercializing a novel class of therapeutics called stapled peptides. The company's lead candidate is ALRN-6924.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:11:59.641605	2020-10-11 23:38:31.031328
ALV	Autoliv Inc.	Autoliv Inc is involved in the auto parts industry. Its products mainly include passive and active safety systems.	{"Consumer Cyclical","Auto Parts",Autos}	{MGA,GNTX,TM,LEA,APTV,VC}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:11:59.655624	2020-10-11 23:38:31.023428
AMBR	Amber Road Inc.	Amber Road Inc is a provider of cloud-based global trade management solution. The company provides various solutions such as Global Sourcing, Production Management, Export Management, Supply Chain Visibility, Duty Management, Global Knowledge and others.	{Technology,"Software - Application","Application Software"}	{DSGX,FFIV,MANH,AMSWA,SAP,ORCL,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:11:59.668822	2020-10-11 23:38:31.023428
AMH	American Homes 4 Rent of Beneficial Interest	American Homes 4 Rent is an internally managed real estate investment trust (REIT) focused on acquiring, renovating, leasing and operating single-family homes as rental properties.	{"Real Estate","REIT - Residential",REITs}	{EQR,CPT,ESS,AIV,UDR,IRT}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:11:59.681338	2020-10-11 23:38:31.031328
AMPE	Ampio Pharmaceuticals Inc.	Ampio Pharmaceuticals Inc is a biopharmaceutical company. The company is engaged in the development of therapies to treat prevalent inflammatory conditions for which there are limited treatment options.	{Healthcare,Biotechnology}	{REGN,RIGL,ALIM,ABT,JNJ,PFE,OPHT,AERI,FLXN,XLV}	Biotechnology	Healthcare	NYSE American	\N	2020-10-11 23:11:59.695215	2020-10-11 23:38:31.023007
AMSC	American Superconductor Corporation	American Superconductor Corp provides megawatt-scale solutions that lower the cost of wind power and enhance the performance of the power grid. The Company's business segments are Wind and Grid.	{Technology,"Electronic Components","Computer Hardware"}	{FELE,POWL,WWD,BRKR,AETI,AOS,AZZ,HUBB,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:11:59.708612	2020-10-11 23:38:31.023007
AMZA	InfraCap MLP	The investment seeks total return primarily through investments in equity securities of publicly traded master limited partnerships and limited liability companies taxed as partnerships ("MLPs").\n Under normal market conditions, the fund will invest not less than 80% of its net assets (plus the amount of any borrowings for investment purposes) in equity securities of MLPs in the energy infrastructure sector. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.721453	2020-10-11 23:38:31.024353
ANF	Abercrombie & Fitch Company	Abercrombie & Fitch Co is a retailer that sells apparel, personal care products and accessories for men, women and kids under Abercrombie & Fitch, Abercrombie kids, and Hollister brands. It operates online stores as well as direct-to-consumer operations.	{"Consumer Cyclical","Apparel Stores","Retail - Apparel & Specialty"}	{URBN,ROST,ZUMZ,AEO,BKE,GPS,TJX,EXPR,TLYS}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:11:59.734507	2020-10-11 23:38:31.028166
ANY	Sphere 3D Corp.	Sphere 3D Corp is a technology development company. It delivers containerization and virtualization technologies along with data management products that enable workload-optimized solutions.	{Technology,"Software - Application","Application Software"}	{XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:11:59.746677	2020-10-11 23:38:31.02582
AOSL	Alpha and Omega Semiconductor Limited	Alpha & Omega Semiconductor Ltd designs, develops and supplies a broad range of power semiconductors for computing, consumer electronics, communication and industrial applications.	{Technology,Semiconductors}	{DIOD,SMTC,ON,MPWR,VSH,TXN,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:11:59.758379	2020-10-11 23:38:31.02423
APH	Amphenol Corporation	Amphenol Corp is engaged in the field of electronic components. It designs and manufactures electrical, electronic and fiber optic connectors, interconnect systems and coaxial and flat-ribbon cables.	{Technology,"Electronic Components","Computer Hardware"}	{RFIL,OCC,AVX,MEI,BDC,TEL,COMM,ITT,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:11:59.771267	2020-10-11 23:38:31.023007
APT	Alpha Pro Tech Ltd.	Alpha Pro Tech Ltd develops, manufactures & markets disposable protective apparel & infection control products for cleanroom, industrial, pharmaceutical, medical & dental markets. It also manufactures building supply construction weatherization products.	{"Basic Materials","Building Materials"}	{ALGN,CAH,MMM,MLSS,XRAY}	Building Materials	Basic Materials	NYSE American	\N	2020-10-11 23:11:59.783492	2020-10-11 23:38:31.031328
AKBA	Akebia Therapeutics Inc.	Akebia Therapeutics Inc is a biopharmaceutical company. It focuses on the development and commercialization of novel therapeutics to patients with kidney disease through hypoxia-inducible factor (HIF) biology.	{Healthcare,Biotechnology}	{AMGN,KERX,NKTR,NVS,JNJ,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:11:59.592756	2020-10-11 23:38:31.02423
ALDX	Aldeyra Therapeutics Inc.	Aldeyra Therapeutics Inc is biotechnology company focused on the development of products for inflammation, inborn errors of metabolism, and other diseases, related to endogenously generated toxic and pro-inflammatory chemical species known as aldehydes.	{Healthcare,Biotechnology}	{IMMU,REGN,FPRX,ANTH,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:11:59.610097	2020-10-11 23:38:31.021426
ALGN	Align Technology Inc.	Align Technology Inc designs, manufactures and markets a system of clear aligner therapy, intra-oral scanners and CAD/CAM digital services used in dentistry, orthodontics, and dental records storage. Its segments are Clear Aligner and Scanners & Services.	{Healthcare,"Medical Devices"}	{DHR,MMM,XRAY,XLV}	Medical Devices	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.62897	2020-10-11 23:38:31.023428
ALRM	Alarm.com Holdings Inc.	Alarm.com Holdings Inc is a cloud-based software platform solution for the connected property. The company through its cloud-based services makes connected home technology accessible to home and business owners.	{Technology,"Software - Application","Application Software"}	{GIB,VRNS,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:11:59.640694	2020-10-11 23:38:31.025871
ALX	Alexander's Inc.	Alexander's Inc is a real estate investment trust which leases, manages, and develops properties. The company's portfolio comprises retail and office properties located in metropolitan and suburban areas of New York City.	{"Real Estate","REIT - Retail",REITs}	{VNO,FR,IRT,SNH}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:11:59.655608	2020-10-11 23:38:31.028166
AMC	AMC Entertainment Holdings Inc. Class A	AMC Entertainment Holdings Inc is engaged in theatrical exhibition. It is principally involved in the theatrical exhibition business and owns, operates or has interests in theatres located in the United States.	{"Consumer Cyclical","Media - Diversified",Entertainment}	{RDI,MCS,CNK}	Entertainment	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:11:59.668937	2020-10-11 23:38:31.028166
AMKR	Amkor Technology Inc.	Amkor Technology Inc is a provider of outsourced semiconductor packaging and test services to integrated device manufacturers, fabless semiconductor companies, and contract foundries.	{Technology,Semiconductors}	{INTC,MU,QCOM,ON,TSM,SMI,UMC,TXN,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:11:59.683671	2020-10-11 23:38:31.028166
AMS	American Shared Hospital Services	American Shared Hospital Services mainly provides gamma knife stereotactic radiosurgery equipment and radiation therapy and related equipment various medical centers in the United States.	{Healthcare,"Medical Care","Health Care Providers"}	{DRAD,FONR,VAR,CYH,CCM,HCA,INFU,PINC,XLV}	Health Care Providers	Healthcare	NYSE American	\N	2020-10-11 23:11:59.707062	2020-10-11 23:38:31.031328
AMX	America Movil S.A.B. de C.V.n Depository Receipt Series L	America Movil SAB de CV is a telecom company. It provides telecommunications services in various countries which include wireless service, fixed-line, broadband and Pay TV services.	{"Communication Services","Telecom Services"}	{DISH,TEO,TEF,T,TSU,VIV}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:11:59.720036	2020-10-11 23:38:31.031328
ANET	Arista Networks Inc.	Arista Networks Inc is a supplier of cloud networking solutions that use software innovations to address the needs of large-scale Internet companies, cloud service providers, and next-generation data centers for enterprise support.	{Technology,"Computer Systems","Computer Hardware"}	{CSCO,MSFT,SILC,JCS,LTRX,JNPR,HPQ,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:11:59.730659	2020-10-11 23:38:31.024353
ANTM	Anthem Inc.	Anthem Inc is a health benefits company offering a network-based managed care plans to the large and small employer, individual, Medicaid and Medicare markets. Its business segments are Commercial and Specialty Business, Government Business and Other.	{Healthcare,"Health Care Plans"}	{CI,HUM,UNH,AET,CNC,XLV}	Health Care Plans	Healthcare	New York Stock Exchange	\N	2020-10-11 23:11:59.743139	2020-10-11 23:38:31.031328
AOS	A.O. Smith Corporation	A.O. Smith Corp manufactures and markets water heaters and boilers, for residential and commercial end markets. The Company has two reporting segments; North America and Rest of World.	{Industrials,"Diversified Industrials","Industrial Products"}	{AAON,FELE,LII,LXU,RBC,CUO,AMSC,BWXT}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.757702	2020-10-11 23:38:31.02582
APDNW	Applied DNA Sciences (Warrant)	Applied DNA Sciences, Inc. engages in the security solutions. It provides biotechnology-driven solutions to help protect products, brands, and intellectual property of companies, governments and consumers from theft, counterfeiting, fraud and diversion. Its products include the following brands: Applied DNA Sciences, SigNature, molecular tags, SigNature, T molecular tags, fiberTyping, DNAnet, digitalDNA, SigNify, BackTrac, Beacon, and CertainT. The company was founded in 1983 and is headquartered in Stony Brook, NY.	{}	{}	Information Technology Services	Technology Services	https://api.robinhood.com/markets/XNAS/	\N	2020-10-11 23:11:59.766775	2020-10-11 23:38:31.023428
APOPW	Cellect Biotechnology Ltd. Warrants to Purchase ADR (1 WT and $7.50 to purchase 1 ADS)		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:11:59.777754	2020-10-11 23:38:31.028166
AKG	Asanko Gold Inc.	Asanko Gold Inc is a gold producer. Its principal asset is the Asanko Gold Mine, located in Ghana, West Africa. It owns two deposits Nkran and Esaase and a 3 Mtpa carbon-in-leach (CIL) processing plant facility.	{"Basic Materials",Gold,"Metals & Mining"}	{IAG,NSU,TRX}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:11:59.593891	2020-10-11 23:38:31.031328
ALG	Alamo Group Inc.	Alamo Group Inc is engaged in the design and manufacturing of agricultural equipment and infrastructure maintenance equipment. Its products include tractor-mounted mowing and other vegetation maintenance equipment, among others.	{Industrials,"Farm & Construction Equipment","Farm & Construction Machinery"}	{AGCO,DE,LNN,SNA,TTC,SCX}	Farm & Construction Machinery	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.626097	2020-10-11 23:38:31.023007
ALLY	Ally Financial Inc.	Ally Financial Inc is a US based financial service provider. It mainly serves automotive dealers and their retail customers. The company through its subsidiary operates insurance and corporate finance business.	{"Financial Services","Specialty Finance",Banks}	{NICK,CPSS,CACC,DFS,CIT,RM,SC}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.634488	2020-10-11 23:38:31.024353
ALT	Altimmune Inc.	Altimmune Inc is a clinical stage immunotherapeutic biotechnology company. It is involved in the business of developing products that engage, stimulate and improve immune responses for the prevention and treatment of diseases.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:11:59.64817	2020-10-11 23:38:31.023007
AMAT	Applied Materials Inc.	Applied Materials Inc is a supplier of semiconductor manufacturing equipment. It provides manufacturing equipment, services and software to the semiconductor, flat panel display, solar photovoltaic and related industries.	{Technology,"Semiconductor Equipment & Materials",Semiconductors}	{ASYS,KLAC,LRCX,SNPS,TER,ASML,ACLS,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:11:59.661961	2020-10-11 23:38:31.021426
AMEH	Apollo Medical Holdings Inc.	Apollo Medical Holdings Inc is a physician-centric, integrated healthcare delivery company serving Medicare, Commercial and Medi-Cal beneficiaries in California. Its physician network consists of hospitalists, primary care and specialist physicians.	{Healthcare,"Medical Care","Health Care Providers"}	{NHC,CSU,ACHC,DVCR,GEN,XLV}	Health Care Providers	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:11:59.675606	2020-10-11 23:38:31.02423
AMMA	Alliance MMA Inc.	Alliance MMA Inc is a mixed martial arts promotion company which promotes mixed martial arts, or MMA, events including live MMA event promotions, MMA content distribution, and sponsorships and promotion.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{}	Travel & Leisure	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:11:59.692446	2020-10-11 23:38:31.02423
AMRS	Amyris Inc.	Amyris Inc is an industrial biotechnology company engaged in the engineering, manufacturing, and sales of products in markets such as cosmetics, flavors and fragrances, solvents and cleaners, polymers, lubricants, healthcare products, and fuels.	{"Basic Materials","Specialty Chemicals",Chemicals}	{}	Chemicals	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:11:59.70478	2020-10-11 23:38:31.02582
AMUB	ETRACS Alerian MLP Index ETN Series B due July 18 2042	The investment seeks to provide exposure to potential price appreciation in the Alerian MLP Index (the “index”).\n The ETRACS Alerian MLP Index ETN Series B due July 18, 2042 (the “Securities”) is senior unsecured debt securities issued by UBS. The index measures the composite performance of energy master limited partnerships (“MLPs”), and is calculated by S&P Dow Jones Indices using a float-adjusted, capitalization-weighted methodology.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.718404	2020-10-11 23:38:31.023428
ANCX	Access National Corporation	Access National Corp provides commercial credit, deposit, mortgage and wealth management services to middle market businesses and associated professionals mainly in Greater Washington.	{"Financial Services","Banks - Regional - US",Banks}	{AMNB,TOWN,OPOF,SONA,BOTJ,ESXB,VBFC}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:11:59.73096	2020-10-11 23:38:31.025871
ANIP	ANI Pharmaceuticals Inc.	ANI Pharmaceuticals Inc is a specialty pharmaceutical company. The Company, together with its subsidiary is engaged in developing, manufacturing, and marketing branded and generic prescription pharmaceuticals.	{Healthcare,Biotechnology}	{INFI,LCI,NVS,PFE,JAZZ,TEVA,EVOK,MYL,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:11:59.74231	2020-10-11 23:38:31.023007
AON	Aon plc Class A (UK)	Aon PLC acts as a provider of risk management services, insurance and reinsurance brokerage, human resource consulting, and outsourcing solutions. The commercial risk solutions generate maximum revenue.	{"Financial Services","Insurance Brokers","Brokers & Exchanges"}	{ACGL,MMC,AXS,WLTW}	Brokers & Exchanges	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.754839	2020-10-11 23:38:31.021426
APDN	Applied DNA Sciences Inc	Applied DNA Sciences Inc creates security solutions addressing the challenges of supply chain security, brand protection or law enforcement applications.	{Industrials,"Security & Protection Services","Consulting & Outsourcing"}	{EFII,CBT,SXT,HUN}	Consulting & Outsourcing	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:11:59.76647	2020-10-11 23:38:31.021426
APOP	Cellect Biotechnology Ltd.	Cellect Biotechnology Ltd is an emerging biotechnology company engaged in developing a novel technology platform known as ApoGraft that selects stem cells in order to improve the safety and efficacy of regenerative medicine and stem cell therapies.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:11:59.776236	2020-10-11 23:38:31.023428
ALB	Albemarle Corporation	Albemarle Corp is a developer, manufacturer and marketer of engineered specialty chemicals. It provides services to end markets including petroleum refining, consumer electronics, construction, automotive, lubricants and pharmaceuticals.	{"Basic Materials","Specialty Chemicals",Chemicals}	{XOM,GRA,PBR,POL,WLK,HUN,DWDP}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:11:59.607424	2020-10-11 23:38:31.023007
ALGT	Allegiant Travel Company	Allegiant Travel Co provides travel services in the United States. It sells air transportation on a stand-alone basis, or bundled with air-related and third-party services and products.	{Industrials,Airlines}	{SKYW,JBLU,HA,ALK,LUV,DAL,AAL,SAVE,UAL}	Airlines	Industrials	Nasdaq Global Select	\N	2020-10-11 23:11:59.62895	2020-10-11 23:38:31.02423
ALNY	Alnylam Pharmaceuticals Inc.	Alnylam Pharmaceuticals Inc is a biopharmaceutical company. It is engaged in developing novel therapeutics based on RNA interference, or RNAi. RNAi is a naturally occurring biological pathway within cells for regulating the expression of specific genes.	{Healthcare,Biotechnology}	{REGN,MDCO,NVS,SNY,TEVA,SRPT,IONS,ARWR,OPK,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.639039	2020-10-11 23:38:31.023428
AMBCW	Ambac Financial Group Inc. Warrants	Ambac Financial Group Inc through its subsidiaries provides insurance and other financial services. The company's Financial Guarantee segment offers insurance policies and credit derivative contracts.	{"Financial Services","Insurance - Specialty"}	{}	Insurance - Specialty	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:11:59.66542	2020-10-11 23:38:31.02582
AMG	Affiliated Managers Group Inc.	Affiliated Managers Group Inc is an asset management company. It has equity investments in boutique investment management firms. It provides centralized assistance in strategic matters, marketing, distribution, product development and operations.	{"Financial Services","Asset Management"}	{WDR,EV,BEN,LM,BLK,WHG,AB,PZN,IVZ,GBL}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.678991	2020-10-11 23:38:31.028166
AMN	AMN Healthcare Services Inc	AMN Healthcare Services Inc is a healthcare staffing firm. It provides healthcare workforce solutions and staffing services. AMN provides nurse and allied healthcare staffing, locum tenens staffing and physician permanent placement services.	{Healthcare,"Medical Care","Health Care Providers"}	{RCMT,KELYA,KFRC,CCRN,XLV}	Health Care Providers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:11:59.692601	2020-10-11 23:38:31.025871
AMRHW	Ameri Holdings Inc. Warrant		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:11:59.704057	2020-10-11 23:38:31.025871
AMTD	TD Ameritrade Holding Corporation	TD Ameritrade Holding Corp operates in the securities brokerage industry. The company offers securities brokerage services and related technology-based financial services to retail investors, traders and independent registered investment advisors.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{ETFC,MS,BAC,RJF,SCHW,SF,LPLA,IBKR}	Brokers & Exchanges	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:11:59.717595	2020-10-11 23:38:31.02423
ANDX	Andeavor Logistics LP representing Limited Partner Interests	Andeavor Logistics LP is a part of the logistics industry in the United States. It caters to oil and gas producers in natural gas and crude oil gathering.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{GEL,MMP,HEP,TLP,NS,SEMG,BKEP,MPLX}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:11:59.73061	2020-10-11 23:38:31.023007
ANIK	Anika Therapeutics Inc.	Anika Therapeutics Inc develops, manufactures and commercializes therapeutic products for tissue protection, healing and repair.	{Healthcare,Biotechnology}	{STAA,HALO,JNJ,SYK,ZBH,HRC,DYNT,MDT,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.742273	2020-10-11 23:38:31.025871
AOR	iShares Core Growth Allocation	The investment seeks to track the investment results of the S&P Target Risk Growth Index composed of a portfolio of underlying equity and fixed income funds intended to represent a growth allocation target risk strategy.\n The fund is a fund of funds and seeks its investment objective by investing primarily in other iShares Underlying Funds that themselves seek investment results corresponding to their own respective underlying indexes. It generally will invest at least 90% of its assets in the component securities of the underlying index. The index measures the performance of the S&P Dow Jones Indices LLC proprietary allocation model.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.757301	2020-10-11 23:38:31.031328
APEI	American Public Education Inc.	American Public Education Inc is a provider of online and campus-based postsecondary education.	{"Consumer Defensive","Education & Training Services",Education}	{CECO,LINC,LOPE,NAUH,BPI,ATGE}	Education	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:11:59.767623	2020-10-11 23:38:31.02582
APPF	AppFolio Inc.	AppFolio Inc is a provider of industry-specific, cloud-based software solutions for small and medium-sized businesses (SMBs) in the property management and legal industries.	{Technology,"Software - Application","Application Software"}	{DWCH,QLYS,VEEV,QTWO,XLK}	Application Software	Technology	NASDAQ Global Market	\N	2020-10-11 23:11:59.778435	2020-10-11 23:38:31.02423
APU	AmeriGas Partners L.P.	AmeriGas Partners LP is a retail propane distributor in the United States and conducts its business through its subsidiary, AmeriGas Propane, L.P. The company also sells, installs, and services propane appliances, including heating systems.	{Utilities,"Utilities - Regulated Gas","Utilities - Regulated"}	{FGP,EPD,UGI,UGP,SGU,SPH,VLP,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:11:59.788421	2020-10-11 23:38:31.028166
ALFA	AlphaClone Alternative Alpha	The investment seeks to track the price and yield, before fees and expenses, of the AlphaClone Hedge Fund Masters Index.\n The fund uses a "passive management" approach to track the performance, before fees and expenses, of the index. The index utilizes a proprietary, quantitative Clone Score methodology developed by AlphaClone, Inc., the fund's index provider, to replicate the U.S. equity securities favored as investments by hedge funds and institutional investors. At least 80% of the fund's total assets (exclusive of any collateral held from securities lending) will be invested in the component securities of the index.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:11:59.617808	2020-10-11 23:38:31.024353
ALLE	Allegion plc	Allegion PLC provides security products and solutions that keep people safe, secure and productive. Its products include door closers, exit devices, frames, biometrics, special surfaces, video surveillance, wireless connectivity, and cloud technology.	{Industrials,"Security & Protection Services","Consulting & Outsourcing"}	{BRC,SWK,CIX,NL,FBHS}	Consulting & Outsourcing	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.630217	2020-10-11 23:38:31.028166
ALOT	AstroNova Inc.	AstroNova Inc designs, develops, manufactures and distributes specialty printers and data acquisition and analysis systems that acquire, store, analyze and present data in multiple formats.	{Technology,"Computer Systems","Computer Hardware"}	{NATI,TACT,MRCY,IART,ZBRA,USAT,DHR,DOV,DDD,XLK}	Computer Hardware	Technology	NASDAQ Global Market	\N	2020-10-11 23:11:59.640592	2020-10-11 23:38:31.02582
ALXN	Alexion Pharmaceuticals Inc.	Alexion Pharmaceuticals Inc is a biopharmaceutical company. It is engaged in serving patients with severe and ultra-rare disorders through the development and commercialization of life-transforming therapeutic products.	{Healthcare,Biotechnology}	{CELG,GILD,REGN,VRTX,BIIB,AGIO,RARE,IONS,QGEN,XON,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.657366	2020-10-11 23:38:31.025871
AME	AMETEK Inc.	AMETEK Inc is a manufacturer of electronic instruments and electromechanical devices with operations in North America, Europe, Asia and South America.	{Industrials,"Diversified Industrials","Industrial Products"}	{BRKR,A,DHR,EMR,ESL,GE,ROP,TDY,ESE}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.670265	2020-10-11 23:38:31.028166
AMJL	Credit Suisse X-Links Monthly Pay 2xLeveraged Alerian MLP Index Exchange Traded Notes due May 16 2036	The investment seeks to provide a monthly compounded 2x leveraged long exposure to the price return version of the Alerian MLP Index (the ¿index¿).\n The index measures the performance of leading energy Master Limited Partnerships, or MLPs (the ¿index constituents¿), as selected and ranked by the index Sponsor in accordance with the index methodology. The index constituents represent approximately 85% of total energy MLP float-adjusted market capitalization.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.682926	2020-10-11 23:38:31.021426
AMOV	America Movil S.A.B. de C.V. Class An Depositary Shares	America Movil SAB de CV is a telecom company. It provides telecommunications services in various countries which include wireless service, fixed-line, broadband and Pay TV services.	{"Communication Services","Telecom Services"}	{DISH,TEO,TEF,T,TSU,VIV}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:11:59.694346	2020-10-11 23:38:31.023428
AMRK	A-Mark Precious Metals Inc.	A-Mark Precious Metals Inc along with its subsidiaries is a full-service precious metals trading company. The Company offers products including gold, silver, platinum and palladium for storage and delivery in the form of coins, bars, wafers and grain.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{EZPW,UAMY,ENVA}	Brokers & Exchanges	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:11:59.704919	2020-10-11 23:38:31.02423
AMTX	Aemetis Inc	Aemetis Inc is a renewable fuels and biochemical company. It is engaged in the production of fuels and chemicals through acquisition, development and commercialization of technologies that replace traditional petroleum based products.	{"Basic Materials","Specialty Chemicals",Chemicals}	{GPRE,CBM,LYB,WLKP}	Chemicals	Basic Materials	NASDAQ Global Market	\N	2020-10-11 23:11:59.717928	2020-10-11 23:38:31.028166
ANCB	Anchor Bancorp	Anchor Bancorp is engaged in providing banking and other financial services. Its operations includes commercial banking services, such as lending activities, deposit products, and other cash management services.	{"Financial Services","Savings & Cooperative Banks",Banks}	{PROV,RVSB,TSBK,BYFC,FFNW,TBNK,FSBW,BANC}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:11:59.730714	2020-10-11 23:38:31.023428
ANGO	AngioDynamics Inc.	AngioDynamics Inc designs, manufactures and sells medical, surgical and diagnostic devices used by professional healthcare providers for the treatment of peripheral vascular disease and for use in oncology and surgical settings.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{ICUI,MMSI,LMAT,BSX,JNJ,TFX,XLV}	Medical Instruments & Equipment	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.739926	2020-10-11 23:38:31.028166
FHB	First Hawaiian Inc.	First Hawaiian Inc is a United States-based bank holding company. It provides a diversified range of banking services to consumer and commercial customers, including deposit products, lending services and wealth management and trust services.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:02.890474	2020-10-11 23:38:31.024353
AMCX	AMC Networks Inc.	AMC Networks Inc is a player in the media industry. The company owns and operates several of cable television's brands delivering high quality content to audiences and a valuable platform to distributors and advertisers.	{"Consumer Cyclical","Media - Diversified",Entertainment}	{CMCSA,DISCA,DIS,VIAB,FOXA,MSGN}	Entertainment	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:11:59.670128	2020-10-11 23:38:31.021426
AMJ	JPMorgan Chase Capital XVI JP Morgan Alerian MLP ETN	The investment seeks to replicate, net of expenses, the Alerian MLP Index. \n The index tracks the performance of midstream energy Master Limited Partnerships.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.683245	2020-10-11 23:38:31.025871
AMRB	American River Bankshares	American River Bankshares is a bank holding company, which serves the commercial banking needs of small to mid-sized businesses. It accepts checking and savings deposits, offers money market deposit accounts and certificates of deposit, among others.	{"Financial Services","Banks - Regional - US",Banks}	{TCBK,CVBF,HTBK,CVCY,BOCH,BMRC,OVLY}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:11:59.701043	2020-10-11 23:38:31.024353
AMSWA	American Software Inc. Class A Common Stock	American Software Inc develops enterprise management and supply chain related software and services. Its solutions consist of global sourcing, workflow management, customer service applications, and ERP solutions.	{Technology,"Software - Application","Application Software"}	{MSFT,MANH,SAP,IBM,ORCL,AMBR,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:11:59.714689	2020-10-11 23:38:31.028166
AMZN	Amazon.com Inc.	Amazon.com Inc is an online retailer. The Company sells its products through the website which provides services, such as advertising services and co-branded credit card agreements. It also offers electronic devices like Kindle e-readers and Fire tablets.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{MSFT,COST,EBAY,OSTK,MELI,BKS,WMT,CBS,GOOGL}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:11:59.724514	2020-10-11 23:38:31.02582
ANGI	ANGI Homeservices Inc.	ANGI Homeservices Inc creates digital marketplace for home services, connecting millions of homeowners across the globe with home service professionals. It brands include HomeAdvisor, Angie's List, mHelpDesk, HomeStars (Canada), and MyHammer (Germany).	{Technology,"Internet Content & Information","Online Media"}	{XLK}	Online Media	Technology	Nasdaq Global Select	\N	2020-10-11 23:11:59.736481	2020-10-11 23:38:31.02582
APAM	Artisan Partners Asset Management Inc. Class A	Artisan Partners Asset Management Inc is a global investment management firm offering investment strategies in asset classes such as equity and bonds to its clients and investors. The firm has approximately $95 billion in total assets under management.	{"Financial Services","Asset Management"}	{GROW,VRTS,WDR,EV,BEN,LM,WETF,MN,LPLA}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.760894	2020-10-11 23:38:31.031328
APLE	Apple Hospitality REIT Inc.	Apple Hospitality REIT Inc is a real estate investment trust based in the United States. It invests mainly in hotel properties with the objective of acquiring income to be transferred to its investors.	{"Real Estate","REIT - Hotel & Motel",REITs}	{NYMT,O,NTP}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:11:59.773046	2020-10-11 23:38:31.031328
APRN	Blue Apron Holdings Inc	Blue Apron Holdings Inc creates original recipes, which are sent along with fresh, seasonal ingredients, directly to customers for them to prepare, cook, and enjoy.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{KONA,APRN}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:11:59.783194	2020-10-11 23:38:31.024353
AQ	Aquantia Corp.	Aquantia Corp is a fabless semiconductor company. It is engaged in design, development and sale of network communication integrated circuits for Ethernet connectivity in the data center, enterprise infrastructure and access markets.	{Technology,Semiconductors}	{JBL,PKE,SANM,XLK}	Semiconductors	Technology	New York Stock Exchange	\N	2020-10-11 23:11:59.794356	2020-10-11 23:38:31.024353
ARCH	Arch Coal Inc. Class A	Arch Coal Inc is a coal producer in the United States, which sells coal to power plants, steel mills and industrial facilities.	{"Basic Materials",Coal}	{PCY}	Coal	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:11:59.808415	2020-10-11 23:38:31.031328
ARGX	argenx SE	argenx SE, along with its subsidiaries is engaged in the bio-technical research on drugs and pharma processes for the treatment of oncological, autoimmune and inflammatory diseases.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.818929	2020-10-11 23:38:31.025871
ARMK	Aramark	Aramark is engaged in the restaurant business. Apart from its core focus on providing food services, Aramark engages in providing facilities and uniform services. It caters to education, healthcare and leisure markets.	{"Consumer Cyclical",Restaurants}	{CTAS,GE,UNF}	Restaurants	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:11:59.829714	2020-10-11 23:38:31.028166
ARTW	Art's-Way Manufacturing Co. Inc.	Art's-Way Manufacturing Co Inc is a manufacturer of agricultural equipment. It has three reportable segments: Agricultural Products, Modular Buildings and Tools.	{Industrials,"Farm & Construction Equipment","Farm & Construction Machinery"}	{ALG,AGCO,DE,LNN,TTC}	Farm & Construction Machinery	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:11:59.841392	2020-10-11 23:38:31.025871
AMRN	Amarin Corporation plc	Amarin Corp PLC is a biopharmaceutical company with expertise in lipid science. The company is engaged in commercialization and development of therapeutics to improve cardiovascular health.	{Healthcare,Biotechnology}	{AZN,GSK,ALKS,TEVA,ABBV,IONS,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:11:59.704665	2020-10-11 23:38:31.024353
AMU	ETRACS Alerian MLP Index ETN	The investment seeks to replicate, net of expenses, the Alerian MLP Index.\n The index measures the performance of 50 prominent energy master limited partnerships. It constituents earn the majority of their cash flow from the transportation, storage, processing or production of energy commodities. The Alerian MLP Index is calculated, maintained and published by S&P in consultation with the index sponsor.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.717707	2020-10-11 23:38:31.025871
ANDE	The Andersons Inc.	Andersons Inc is predominantly an agricultural products distributor in the United States. It is also engaged in the manufacture of ethanol, fertilizers and rail operations.	{"Consumer Defensive","Food Distribution","Retail - Defensive"}	{JCTCF,ALCO,UVV,WMT,BG,MOS}	Retail - Defensive	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:11:59.730924	2020-10-11 23:38:31.031328
ANSS	ANSYS Inc.	Ansys Inc functions in the software industry. It develops and markets engineering software and services catering to engineers, designers, researchers and students across a broad spectrum of industries and academia.	{Technology,"Software - Application","Application Software"}	{ADSK,CDNS,SNPS,A,GD,PTC,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:11:59.743332	2020-10-11 23:38:31.023428
AP	Ampco-Pittsburgh Corporation	Ampco-Pittsburgh Corp and its subsidiaries produces forged hardened steel rolls used in cold rolling by producers of steel, aluminum & other metals throughout the world. Its segments include Forged and Cast Engineered Products & Air and Liquid Processing.	{Industrials,"Diversified Industrials","Industrial Products"}	{USAP,STLD,SIM,GGB,CRS,CMC,NUE}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.757878	2020-10-11 23:38:31.023007
APEN	Apollo Endosurgery Inc.	Apollo Endosurgery Inc is a developer of medical devices in the United States. The company's major focus area is obesity and gastrointestinal diseases.	{Healthcare,"Medical Devices"}	{XLV}	Medical Devices	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:11:59.770515	2020-10-11 23:38:31.02423
APPS	Digital Turbine Inc.	Digital Turbine Inc through its subsidiaries, delivers end-to-end products and solutions for mobile operators, app advertisers, device OEMs and other third parties.	{Technology,"Software - Application","Application Software"}	{CSGS,MNDO,SNCR,EVOL,DOX,SMSI,GEC,XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:11:59.782467	2020-10-11 23:38:31.028166
AQB	AquaBounty Technologies Inc.	AquaBounty Technologies Inc is a biotechnology company. The company is engaged in the business of conducting research and development of the commercial viability of a group of proteins known as antifreeze proteins.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:11:59.794537	2020-10-11 23:38:31.023428
ARCC	Ares Capital Corporation	Ares Capital Corp is a specialty finance company that is a closed-end, non-diversified management investment company. Its investment objective is to generate both current income and capital appreciation through debt and equity investments.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:11:59.806811	2020-10-11 23:38:31.024353
ARDX	Ardelyx Inc.	Ardelyx Inc is a biotechnology company focused on the discovery, development, and commercialization of molecule and polymeric therapeutics to treat the gastrointestinal tract and cardiorenal diseases.	{Healthcare,Biotechnology}	{AMGN,KERX,AZN,ABT,SNY,OPK,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:11:59.816351	2020-10-11 23:38:31.023007
ARKQ	ARK Industrial Innovation	The investment seeks long-term growth of capital.\n The fund is an actively-managed exchange-traded fund that will invest under normal circumstances primarily (at least 80% of its assets) in domestic and foreign equity securities of companies that are relevant to the fund's investment theme of industrial innovation. Substantially all of the fund's assets will be invested in equity securities, including common stocks, partnership interests, business trust shares and other equity investments or ownership interests in business enterprises. The advisor currently intends to use only ADRs when purchasing foreign securities. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.826824	2020-10-11 23:38:31.02582
ARR	ARMOUR Residential REIT Inc.	ARMOUR Residential REIT Inc invests in and manages a leveraged portfolio of residential mortgage backed securities. It invests in residential mortgage backed securities issued or guaranteed by a United States Government-sponsored entity.	{"Real Estate","REIT - Residential",REITs}	{NYMT,DX,CMO,IVR,MITT}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:11:59.838413	2020-10-11 23:38:31.021426
ASB	Associated Banc-Corp	Associated Banc-Corp is a commercial bank holding company. It provides banking and related financial services to consumer and commercial customers.	{"Financial Services","Banks - Regional - US",Banks}	{PEBO,THFF,HTLF,WTFC,HBAN,FBIZ,WFC,TCF,SGB,ONB}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.850025	2020-10-11 23:38:31.023007
FEMB	First Trust Emerging Markets Local Currency Bond E	The investment seeks maximum total return and current income.\n Under normal market conditions, the fund seeks to achieve its investment objective by investing at least 80% of its net assets (including investment borrowings) in bonds, notes and bills issued or guaranteed by entities incorporated or domiciled in emerging market countries (collectively, "Bonds") that are denominated in the local currency of the issuer. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.807805	2020-10-11 23:38:31.028166
AMT	American Tower Corporation (REIT)	American Tower Corp is a real estate investment trust and an owner, operator and developer of multitenant communications real estate. The company leases the space of communications sites to wireless service providers, and wireless data providers.	{"Communication Services","Telecom Services"}	{DRE,CTL,PCH,PW,TDS,LAND,CTT,CCI,SIR}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:11:59.717521	2020-10-11 23:38:31.021426
ANAB	AnaptysBio Inc.	AnaptysBio Inc is a biotechnology company. The company is engaged in developing first-in-class antibody product candidates focused on unmet medical needs in inflammation and immuno-oncology. Its product include ANB020 and ANB019.	{Healthcare,Biotechnology}	{MNTA,MLNT,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.726905	2020-10-11 23:38:31.021426
ANGL	VanEck Vectors Fallen Angel High Yield Bond	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the ICE BofAML US Fallen Angel High Yield Index (the "Fallen Angel Index").\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The index is comprised of below investment grade corporate bonds denominated in U.S. dollars that were rated investment grade at the time of issuance.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.737113	2020-10-11 23:38:31.02423
APB	Asia Pacific Fund Inc. (The)	Asia Pacific Fund Inc is a United States based closed-end management investment company. Its investment objective is to achieve long-term capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.761484	2020-10-11 23:38:31.028166
APLS	Apellis Pharmaceuticals Inc.	Apellis Pharmaceuticals Inc is a clinical-stage biopharmaceutical company. The company is engaged in the discovery and development of novel therapeutic compounds for autoimmune and inflammatory diseases.	{Healthcare,Biotechnology}	{MCK,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.773053	2020-10-11 23:38:31.025871
APTS	Preferred Apartment Communities Inc.	Preferred Apartment Communities Inc acquires and operates multifamily properties in select targeted markets throughout the United States. Its business segments are multifamily communities, real estate related financing, and retail.	{"Real Estate","REIT - Residential",REITs}	{EQR,CPT,MAA,ESS,BRT,UDR,IRT}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:11:59.784885	2020-10-11 23:38:31.02582
AQUA	Evoqua Water Technologies Corp.	Evoqua Water Technologies Corp is a provider of mission critical water treatment solutions. It operates in three segments namely: Industrial, Municipal and Products.	{Industrials,"Pollution & Treatment Controls","Industrial Products"}	{}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.798836	2020-10-11 23:38:31.025871
ARCO	Arcos Dorados Holdings Inc. Class A Shares	Arcos Dorados Holdings Inc operates and franchises McDonald's restaurants. It operates in the QSR sub-segment of the fast food segment of the Latin American and Caribbean food service industry.	{"Consumer Cyclical",Restaurants}	{JACK,SONC,TAST,MCD,YUM,GTIM,WEN}	Restaurants	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:11:59.808843	2020-10-11 23:38:31.025871
ARGT	Global X MSCI Argentina	The investment seeks to provide investment results that correspond generally to the price and yield performance, before fees and expenses, of the MSCI All Argentina 25/50 Index.\n The fund invests at least 80% of its total assets in the securities of the underlying index and in American Depositary Receipts ("ADRs") and Global Depositary Receipts ("GDRs") based on the securities in the underlying index. The underlying index is designed to represent the performance of the broad Argentina equity universe, while including a minimum number of constituents. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.818913	2020-10-11 23:38:31.023428
ARL	American Realty Investors Inc.	American Realty Investors Inc acquires, develops and owns income-producing residential and commercial real estate properties.	{"Real Estate","Real Estate - General","Real Estate Services"}	{INTG,MAYS,TCI,NEN,CTO,HHC,AHH,FCE.A}	Real Estate Services	Real Estate	New York Stock Exchange	\N	2020-10-11 23:11:59.828827	2020-10-11 23:38:31.023428
ARTX	Arotech Corporation	Arotech Corp is a defense & security products & services company. It is engaged in two business areas, interactive simulation for military, law enforcement & commercial markets; & batteries & charging systems for military, commercial and medical markets.	{Industrials,"Diversified Industrials","Industrial Products"}	{ULBI,CUB,ENS,ESNC}	Industrial Products	Industrials	NASDAQ Global Market	\N	2020-10-11 23:11:59.841392	2020-10-11 23:38:31.023428
FET	Forum Energy Technologies Inc.	Forum Energy Technologies Inc is an oil and gas company that manufactures and distributes products and engages in aftermarket services and related operations. The company serves the subsea, drilling, completion, production and infrastructure sectors.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{NOV,SLB,DRQ,HLX,AROC,BHGE}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:02.821822	2020-10-11 23:38:31.028166
AMWD	American Woodmark Corporation	American Woodmark Corp manufactures and distributes kitchen cabinets and vanities for the remodeling and new home construction markets.	{"Consumer Cyclical","Home Furnishings & Fixtures","Manufacturing - Apparel & Furniture"}	{PATK,MAS,FBHS,CSTE}	Manufacturing - Apparel & Furniture	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:11:59.71848	2020-10-11 23:38:31.023007
ANTH	Anthera Pharmaceuticals Inc.	Anthera Pharmaceuticals Inc is a biopharmaceutical company. It is engaged in developing and commercializing therapeutics to treat serious diseases.	{Healthcare,Biotechnology}	{VIVO,DRNA,AFMD,VKTX,XLV}	Biotechnology	Healthcare	OTC Pink	\N	2020-10-11 23:11:59.743556	2020-10-11 23:38:31.024353
APA	Apache Corporation	Apache Corp is engaged in the oil and gas sector. As an independent energy company, its activities include exploration, development and production of natural gas, crude oil and natural gas liquids.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{CHK,PXD,DVN,APC,OXY,EOG,CVX,MRO,COP,NBL}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:11:59.75835	2020-10-11 23:38:31.024353
APF	Morgan Stanley Asia-Pacific Fund Inc.	Morgan Stanley Asia Pacific Fund Inc is a closed-end management investment company. The fund's investment objective is to seek long-term capital appreciation through investments in equity securities, debt securities and other structured investments.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.770515	2020-10-11 23:38:31.028166
APPN	Appian Corporation	Appian Corp provides low-code software development platform as a service that enables organizations to rapidly develop powerful and unique applications.	{Technology,"Software - Infrastructure","Application Software"}	{OTEX,PANW,XLK}	Application Software	Technology	NASDAQ Global Market	\N	2020-10-11 23:11:59.780951	2020-10-11 23:38:31.021426
APWC	Asia Pacific Wire & Cable Corporation Limited	Asia Pacific Wire & Cable Corp Ltd manufactures and distributes telecommunications and power cable and enameled wire products in Singapore, Thailand, Australia, the People's Republic of China.	{Technology,"Electronics Distribution","Computer Hardware"}	{WIRE,OCC,BDC,TEL,XLK}	Computer Hardware	Technology	NASDAQ Global Market	\N	2020-10-11 23:11:59.79311	2020-10-11 23:38:31.031328
ARC	ARC Document Solutions Inc.	ARC Document Solutions Inc engaged in providing document management solutions to businesses, including non-residential segment of architecture, engineering & construction industry. Its offering include; onsite, digital, color & traditional reprographics.	{Industrials,"Business Services"}	{EXLS,CAJ,DLX,PFMT,XRX}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.802368	2020-10-11 23:38:31.02582
ARDM	Aradigm Corporation	Aradigm Corp is a United States based specialty pharmaceutical company. It develops and commercializes drugs delivered by inhalation for the prevention and treatment of severe respiratory diseases.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{GILD,GSK,JNJ,PFE,INSM,INVA,MNKD,XLV}	Drug Manufacturers	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:11:59.81569	2020-10-11 23:38:31.031328
ARKR	Ark Restaurants Corp.	Ark Restaurants Corp through its subsidiaries owns and operates restaurants and bars. The Company is also engaged in fast food concepts and catering operations.	{"Consumer Cyclical",Restaurants}	{RRGB,TXRH,BJRI,DRI,EAT,BDL,BLMN,SAUC,RICK}	Restaurants	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:11:59.827246	2020-10-11 23:38:31.023007
ARTNA	Artesian Resources Corporation Class A Non-Voting Common Stock	Artesian Resources Corp through its subsidiaries offers water, wastewater and other services on the Delmarva Peninsula. It distributes water for public and private fire protection, to residential, commercial, industrial, municipal and utility customers.	{Utilities,"Utilities - Regulated Water","Utilities - Regulated"}	{YORW,CTWS,MSEX,CDZI,CWT,AWR,WTR,AWK,SJW,XLU}	Utilities - Regulated	Utilities	Nasdaq Global Select	\N	2020-10-11 23:11:59.841501	2020-10-11 23:38:31.024353
ASFI	Asta Funding Inc.	Asta Funding Inc through its subsidiaries is engaged in the business of acquiring, managing, servicing and recovering on portfolios of consumer receivables.	{"Financial Services","Credit Services"}	{ECPG,CIT,SLM,ATLC,PRAA}	Credit Services	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:11:59.854742	2020-10-11 23:38:31.025871
ASML	ASML Holding N.V.	ASML Holding NV is a part of the semiconductor industry. Its products include memory chip and logic chip, TWINSCAN systems, equipped with i-line, KrF and ArF light sources.	{Technology,"Semiconductor Equipment & Materials",Semiconductors}	{INTC,KLAC,PLAB,TSM,CAJ,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:11:59.86438	2020-10-11 23:38:31.02423
AST	Asterias Biotherapeutics Inc. Common Series A	Asterias Biotherapeutics Inc is a biotechnology company engaged in developing and commercializing therapies in the fields of cell therapy and regenerative medicine.	{Healthcare,Biotechnology}	{INCY,ADRO,NBY,XLV}	Biotechnology	Healthcare	NYSE American	\N	2020-10-11 23:11:59.875734	2020-10-11 23:38:31.02423
AT	Atlantic Power Corporation (Canada)	Atlantic Power Corp is an independent electric power producer company that owns and operates a diverse fleet of power generation assets in the United States and Canada.	{Utilities,"Utilities - Independent Power Producers"}	{EIX,AEP,IDA,SO,ALE,DUK,XEL,XLU}	Utilities - Independent Power Producers	Utilities	New York Stock Exchange	\N	2020-10-11 23:11:59.887607	2020-10-11 23:38:31.021426
ATI	Allegheny Technologies Incorporated	Allegheny Technologies Inc is a diversified specialty metals producer. Its products include titanium and titanium alloys, nickel-based alloys and specialty steels, precision forgings, castings and machined components, and zirconium.	{Industrials,"Metal Fabrication","Industrial Products"}	{USAP,HAYN,AKS,CRS,KMT,VMI,FRD}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.899834	2020-10-11 23:38:31.031328
APO	Apollo Global Management LLC Class A Representing Class A Limitied Liability Company Interests	Apollo Global Management LLC is an alternative-asset management company. It invests and manages funds on behalf of prominent pension, endowment and sovereign wealth funds as well as other institutional and individual investors.	{"Financial Services","Asset Management"}	{BX,OZM,OAK,CG,ARES}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.773364	2020-10-11 23:38:31.024353
APTI	Apptio Inc.	Apptio Inc develops and sells Technology Business Management or TGM solutions. Its cloud-based platform and SaaS applications enable IT leaders to analyze, optimize and plan technology investment and benchmark financial and operational performance.	{Technology,"Software - Application","Application Software"}	{XLK}	Application Software	Technology	NASDAQ Global Market	\N	2020-10-11 23:11:59.783561	2020-10-11 23:38:31.023007
AQN	Algonquin Power & Utilities Corp.	Algonquin Power & Utilities Corp is a renewable energy and regulated utility company. It is engages in hydroelectric, wind, thermal and solar power facilities, and sustainable utility distribution businesses.	{Utilities,"Utilities - Independent Power Producers"}	{PCG,WTR,AWK,XLU}	Utilities - Independent Power Producers	Utilities	New York Stock Exchange	\N	2020-10-11 23:11:59.794512	2020-10-11 23:38:31.023007
ARES	Ares Management L.P. representing Limited Partner Interests	Ares Management LP is a global alternative asset manager. It offers its investor a range of investment strategies and seeks to deliver attractive performance to a growing investor base.	{"Financial Services","Asset Management"}	{BX,OZM,APO,CG}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.818339	2020-10-11 23:38:31.024353
ARLP	Alliance Resource Partners L.P.	Alliance Resource Partners LP is a producer and marketer of coal in the United States. Most of its operations are prevalent in the Illinois Basin, which entails underground mining complexes in Illinois, Indiana, Kentucky, Maryland and West Virginia.	{"Basic Materials",Coal}	{HNRG,NRP,CNX}	Coal	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:11:59.828845	2020-10-11 23:38:31.025871
ARRS	ARRIS International plc	ARRIS International PLC is a media entertainment and data communications solutions provider. It provides Customer Premises Equipment including set-top, gateway, DSL and cable modem; and Network and Cloud including video infrastructure.	{Technology,"Communication Equipment"}	{HLIT,SEAC,NTGR,MSI,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:11:59.838925	2020-10-11 23:38:31.02423
ASC	Ardmore Shipping Corporation	Ardmore Shipping Corp owns and operates fleet of medium size product and chemical tankers to provide seaborne transportation of petroleum products and chemicals worldwide.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{GLNG,NAT,TK,SFL,TGP,TOO,TNK,GMLP,DHT,FRO}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.849638	2020-10-11 23:38:31.021426
ASHX	Xtrackers MSCI China A Inclusion Equity	The investment seeks investment results that correspond generally to the performance, before fees and expenses, of the MSCI China A Inclusion Index.\n The fund will normally invest at least 80% of its total assets in securities (including depositary receipts in respect of such securities) of issuers that comprise the underlying index. The underlying index is designed to track the equity market performance of China A-Shares that are accessible through the Shanghai-Hong Kong Stock Connect program or the Shenzhen-Hong Kong Stock Connect program. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.8613	2020-10-11 23:38:31.023007
ASPU	Aspen Group Inc.	Aspen Group Inc, together with its subsidiaries, is engaged in offering distance-learning education services. Its Program offering includes; Associates, Bachelor's, Master's, Doctoral Degree & other Certification Programs.	{"Consumer Defensive","Education & Training Services",Education}	{}	Education	Consumer Defensive	NASDAQ Capital Market	\N	2020-10-11 23:11:59.874955	2020-10-11 23:38:31.023428
ATAX	America First Multifamily Investors L.P.	America First Multifamily Investors LP is a US-based company engaged in acquiring a portfolio of mortgage revenue bonds that are issued by state and local housing authorities to provide construction and permanent financing.	{"Financial Services","Specialty Finance",Banks}	{OCN,EFC,WD,HASI,LADR}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:11:59.889362	2020-10-11 23:38:31.025871
ATHN	athenahealth Inc.	athenahealth Inc provides healthcare IT services through cloud-based and mobile application products. Their services include revenue cycle and practice management, electronic health records, patient engagement, and clinical decision support.	{Technology,"Health Information Services","Application Software"}	{CERN,MDRX,HMSY,MCK,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:11:59.899257	2020-10-11 23:38:31.028166
ATNX	Athenex Inc.	Athenex Inc is a global biopharmaceutical company dedicated to the discovery, development and commercialization of novel therapies for the treatment of cancer.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.909191	2020-10-11 23:38:31.023428
ATRC	AtriCure Inc.	AtriCure Inc is a medical device company. It develops, manufactures and sells cardiac surgical ablation systems designed to create precise lesions, scars in cardiac, heart, and tissue. It generates revenue by selling disposable surgical devices.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{ABMD,COO,BSX,JNJ,TFX,EW,MDT,XLV}	Medical Instruments & Equipment	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:11:59.919963	2020-10-11 23:38:31.02582
APOG	Apogee Enterprises Inc.	Apogee Enterprises Inc is engaged in the design and development of value-added glass solutions for enclosing commercial buildings and framing art.	{"Basic Materials","Building Materials"}	{LBY,CRH,VMI,IBP,AA}	Building Materials	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:11:59.774771	2020-10-11 23:38:31.02582
APTO	Aptose Biosciences Inc.	Aptose Biosciences Inc is a clinical-stage biotechnology company committed to discovering and developing personalized therapies addressing unmet medical needs in oncology.	{Healthcare,Biotechnology}	{SGEN,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:11:59.784734	2020-10-11 23:38:31.025871
ARA	American Renal Associates Holdings Inc	American Renal Associates Holdings Inc through its subsidiary is engaged in providing kidney dialysis services for patients suffering from chronic kidney failure, also known as end stage renal disease, or ESRD.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{FMS,DVA,ARA,XLV}	Medical Diagnostics & Research	Healthcare	New York Stock Exchange	\N	2020-10-11 23:11:59.801708	2020-10-11 23:38:31.021426
ARCW	ARC Group Worldwide Inc.	ARC Group Worldwide Inc is a manufacturer and 3D printing service provider. The Company's business segments are Precision Components Group, 3DMT Group, Flanges and Fittings Group and Wireless Group.	{Industrials,"Metal Fabrication","Industrial Products"}	{}	Industrial Products	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:11:59.81327	2020-10-11 23:38:31.028166
ARNC	Arconic Inc.	Arconic Inc is engaged in manufacturing value-added aluminum and specialty metals products for a wide variety of industrial end markets, including aerospace and defense, building and construction, and automotive.	{Industrials,"Diversified Industrials","Industrial Products"}	{KALU,APWC,ARNC}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.832061	2020-10-11 23:38:31.031328
ARW	Arrow Electronics Inc.	Arrow Electronics Inc is a value-added distributor of electronic components, software, and data center infrastructure hardware. The company primarily sells semiconductors, software, storage, and servers to electronic equipment manufacturers and resellers.	{Technology,"Electronics Distribution","Computer Hardware"}	{TAIT,RELL,TECD,BSQR,IBM,SNX,AVT,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:11:59.841629	2020-10-11 23:38:31.031328
ASGN	ASGN Incorporated	ASGN Inc, formerly On Assignment Inc provides skilled and professionals in the growing life sciences, healthcare, and technology sectors. The company's operating business segments are Apex and Oxford.	{Industrials,"Staffing & Outsourcing Services","Employment Services"}	{}	Employment Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.856167	2020-10-11 23:38:31.031328
ASRVP	AmeriServ Financial Inc. AmeriServ Financial Trust I	Ameriserv Financial Capital Trust I offers a complete range of trust and financial services.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:11:59.875686	2020-10-11 23:38:31.025871
ASUR	Asure Software Inc	Asure Software Inc provides cloud-based software-as-a-service time and labor management and workspace management solutions. It offers solutions including AsureSpace workplace management solutions and AsureForce time and labor management solutions.	{Technology,"Software - Application","Application Software"}	{RCMT,VRNT,SCS,PFE,CRM,MRK,PCTY,IRM,WDAY,XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:11:59.886051	2020-10-11 23:38:31.028166
ATHX	Athersys Inc.	Athersys Inc is a biopharmaceutical company engaged in the discovery and development of therapeutic product candidates designed to extend and improve the quality of human life.	{Healthcare,Biotechnology}	{AMGN,USLM,OSIR,BMY,JNJ,PFE,SNY,VCEL,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:11:59.899678	2020-10-11 23:38:31.023428
ATRS	Antares Pharma Inc.	Antares Pharma Inc is a specialty pharmaceutical company. It is engaged in developing and commercializing self-administered parenteral pharmaceutical products and technologies including novel, pressure-assisted injectors.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{WST,BDX,JNJ,LLY,PFE,RVP,TEVA,ANIP,MYL,XLV}	Medical Instruments & Equipment	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:11:59.920039	2020-10-11 23:38:31.021426
AUG	Auryn Resources Inc.	Auryn Resources Inc is a junior exploration company focused on the acquisition and exploration of mineral resource properties. It has two material mineral properties, the Committee Bay Project, a gold exploration property, and Homestake Ridge Project.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:11:59.933585	2020-10-11 23:38:31.023428
AVA	Avista Corporation	Avista Corp is engaged in the utilities sector. Its business includes generation, transmission and distribution of natural gas to customers in the United States.	{Utilities,"Utilities - Diversified","Utilities - Regulated"}	{PCG,BKH,CMS,IDA,MDU,SCG,NWE,POR,WEC,XEL,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:11:59.943749	2020-10-11 23:38:31.023007
AVGR	Avinger Inc.	Avinger Inc is a United States-based medical device company. It is engaged in designing, manufacturing and selling image-guided, catheter-based systems that are used by physicians to treat patients with peripheral artery disease.	{Healthcare,"Medical Devices"}	{XLV}	Medical Devices	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:11:59.957121	2020-10-11 23:38:31.02582
AWF	Alliancebernstein Global High Income Fund	Alliancebernstein Glb High Inc Fund Inc is a United States based diversified, closed-end management investment company. Its primary investment objective is to seek high current income, and secondarily, capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.969405	2020-10-11 23:38:31.023428
AR	Antero Resources Corporation	Antero Resources Corp is an independent oil and natural gas company, which acquires, explores, produces, and develops natural gas, natural gas liquids, and oil properties in the United States.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{COG,RRC,EQT,ECR}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:11:59.801101	2020-10-11 23:38:31.02423
ARD	Ardagh Group S.A.	Ardagh Group SA supplies rigid packaging solutions including metal and glass containers for food and beverage markets. Its end users include beer, wine, spirits, soft drink, energy drinks, juices, flavored waters, food, seafood and nutrition industries.	{"Consumer Cyclical","Packaging & Containers"}	{OI}	Packaging & Containers	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:11:59.813283	2020-10-11 23:38:31.028166
AROC	Archrock Inc.	Archrock Inc opeartes in the U.S. natural gas contract operations services business and provides natural gas compression services to customers in the oil and natural gas industry throughout the U.S., and supplies aftermarket services.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{SPN,SLB,HON,OII,TTI,NGS,FET,BHGE}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:11:59.834124	2020-10-11 23:38:31.02582
ARWR	Arrowhead Pharmaceuticals Inc.	Arrowhead Pharmaceuticals Inc is a biotechnology company. It is engaged in the development of medicine to treat diseases such as hepatitis B, and thrombosis with a genetic origin, primarily divided by the overproduction of one or more proteins.	{Healthcare,Biotechnology}	{SHPG,ALNY,JNJ,LGND,DRNA,INO,ABEO,IONS,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.844797	2020-10-11 23:38:31.028166
ASH	Ashland Global Holdings Inc.	Ashland Global Holdings Inc is engaged in the chemical industry. Its chemical based products find application in a diverse range of industries.	{"Basic Materials",Chemicals}	{MNRO,AZO,CMI,GPC,WMT,HUN,GT,DWDP}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:11:59.856151	2020-10-11 23:38:31.02582
ASRV	AmeriServ Financial Inc.	AmeriServ Financial Inc is a bank holding company. The company through its subsidiary provides an array of retail banking services, lending, depository, real estate mortgage loans, short and medium-term loans, and others.	{"Financial Services","Banks - Regional - US",Banks}	{DNBF,EMCF,BMTC,CCNE,FULT,STBA,FCF,FNB,PFIS}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:11:59.875146	2020-10-11 23:38:31.024353
ASX	ASE Technology Holding Co. Ltd. American Depositary Shares (each representing Two)	ASE Technology Holding Co Ltd offers semiconductor packaging and testing services including semiconductor packaging, production of interconnect materials, engineering testing and wafer probing.	{Technology,Semiconductors}	{XLK}	Semiconductors	Technology	New York Stock Exchange	\N	2020-10-11 23:11:59.886505	2020-10-11 23:38:31.023428
ATH	Athene Holding Ltd. Class A	Athene Holding Ltd is a retirement services company that issues, reinsures and acquires retirement savings products designed for the increasing number of individuals and institutions seeking to fund retirement needs.	{"Financial Services","Insurance - Diversified",Insurance}	{NMIH,OXBR}	Insurance	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.897744	2020-10-11 23:38:31.021426
ATNM	Actinium Pharmaceuticals Inc. (Delaware)	Actinium Pharmaceuticals Inc is a biopharmaceutical company engaged in the development of innovative-targeted therapies for patients with cancers lacking effective treatment options. The company's product portfolio includes Iomab-B and Actimab-A.	{Healthcare,Biotechnology}	{PGNX,BMRN,GNCA,IOVA,XLV}	Biotechnology	Healthcare	NYSE American	\N	2020-10-11 23:11:59.907522	2020-10-11 23:38:31.028166
ATSG	Air Transport Services Group Inc	Air Transport Services Group Inc, through its subsidiaries, provides aircraft leasing, airline operations, aircraft maintenance and other support services to the air cargo transportation and package delivery industries.	{Industrials,"Airports & Air Services","Transportation & Logistics"}	{AIRT,AAWW,SINO,UPS,FDX}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:11:59.920743	2020-10-11 23:38:31.024353
ATXI	Avenue Therapeutics Inc.	Avenue Therapeutics Inc is a specialty pharmaceutical company focused on the development and commercialization of an intravenous, or IV, formulation of tramadol HCl, or IV Tramadol, for the management of moderate to moderately severe postoperative pain.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{OMER,XLV}	Drug Manufacturers	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:11:59.931969	2020-10-11 23:38:31.024353
AVAV	AeroVironment Inc.	AeroVironment Inc designs, develops, produces, supports and operates technological portfolio of products and services for government agencies. It supplies unmanned aircraft systems, UAS, tactical missile systems and related services.	{Industrials,"Aerospace & Defense"}	{LMT,AIR,BA,GD,NOC,TXT,SPR,HII}	Aerospace & Defense	Industrials	Nasdaq Global Select	\N	2020-10-11 23:11:59.944768	2020-10-11 23:38:31.023428
AVID	Avid Technology Inc.	Avid Technology Inc is a provider of technology solutions. Its products enable the creation and monetization of audio and video content for digital media content production.	{Technology,"Electronic Gaming & Multimedia","Application Software"}	{ADBE,AAPL,BTN,SNE,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:11:59.958184	2020-10-11 23:38:31.023428
AVY	Avery Dennison Corporation	Avery Dennison Corp is a manufacturer of miscellaneous business equipment. It is engaged in the production of pressure-sensitive materials, office products and a variety of tickets, tags, labels and other converted products.	{Industrials,"Business Equipment","Industrial Products"}	{BMS,MMM,HUN,ACCO}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.967634	2020-10-11 23:38:31.024353
ARAY	Accuray Incorporated	Accuray Inc is a radiation oncology company that develops, manufactures, sells and supports precise treatment solutions. Its products includes the CyberKnife Systems and the TomoTherapy Systems.	{Healthcare,"Medical Devices"}	{HOLX,SYK,VAR,ISR,ICAD,SSKN,XLV}	Medical Devices	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.801799	2020-10-11 23:38:31.028166
ARKK	ARK Innovation	The investment seeks long-term growth of capital.\n The fund is an actively-managed exchange-traded fund ("ETF") that will invest under normal circumstances primarily (at least 65% of its assets) in domestic and foreign equity securities of companies that are relevant to the fund's investment theme of disruptive innovation. Its investments in foreign equity securities will be in both developed and emerging markets. The advisor currently intends to use only American Depositary Receipts ("ADRs") when purchasing foreign securities. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.824394	2020-10-11 23:38:31.021426
ARQL	ArQule Inc.	Arqule Inc is a biopharmaceutical company, engaged in the research and development of therapeutics to treat cancers and rare diseases.	{Healthcare,Biotechnology}	{AMGN,EXEL,ARRY,INFI,AVEO,LLY,CYTK,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:11:59.836586	2020-10-11 23:38:31.023007
ASCMA	Ascent Capital Group Inc. Series A Common Stock	Ascent Capital Group Inc offers security products to consumers in the United States. Its primary product is the MONI brand of home security solutions, which include a security alarms monitoring system which detect burglaries, fires, and medical alerts.	{Industrials,"Security & Protection Services","Consulting & Outsourcing"}	{BCO,MOC,JCI}	Consulting & Outsourcing	Industrials	Nasdaq Global Select	\N	2020-10-11 23:11:59.851463	2020-10-11 23:38:31.02423
ASIX	AdvanSix Inc.	AdvanSix Inc manufactures Nylon 6, a polymer resin used to produce engineered plastics, fibers, filaments & films that are used in end-products as automotive & electronic components, carpets, sports apparel, fishing nets & food and industrial packaging.	{"Basic Materials",Chemicals}	{POL}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:11:59.862039	2020-10-11 23:38:31.023428
ASPS	Altisource Portfolio Solutions S.A.	Altisource Portfolio Solutions SA is a marketplace and transaction solutions provider for the real estate, mortgage and consumer debt industries. It operates in segments including mortgage services, financial services and technology services.	{Industrials,"Business Services"}	{ATAX,WD,LADR}	Business Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:11:59.875153	2020-10-11 23:38:31.021426
ASTE	Astec Industries Inc.	Astec Industries Inc designs, engineers, manufactures and markets equipment and components used in road building, utility and related construction activities as well as other products. It also serves the energy market by supplying various products.	{Industrials,"Farm & Construction Equipment","Farm & Construction Machinery"}	{CMCO,GENC,TAYD,NOV,TEX,CAT,FET,MTW}	Farm & Construction Machinery	Industrials	Nasdaq Global Select	\N	2020-10-11 23:11:59.884685	2020-10-11 23:38:31.031328
ATMP	Barclays ETN Plus Select MLP	The investment seeks to replicate, net of expenses, the Atlantic Trust Select MLP Index.\n The index seeks to capture returns that may be available from investing in a basket of direct or indirect interests in master limited partnerships (“MLPs”), limited liability companies (“LLCs”) and corporations (collectively, “MLP Interests”) that are selected pursuant to the Atlantic Trust Select MLP Strategy.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.907172	2020-10-11 23:38:31.023007
ATRI	Atrion Corporation	Atrion Corp supplies medical devices and related components. The company makes various products such as fluid delivery devices and sells it worldwide. It aims to serve the patients with heart disease and eye problems.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{STAA,COO,TFX,VUZI,TEAR,XLV}	Medical Instruments & Equipment	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.920278	2020-10-11 23:38:31.025871
AU	AngloGold Ashanti Limited	Anglogold Ashanti Ltd is a gold mining company. The firm conducts its mining activities at various sites that are across the world.	{"Basic Materials",Gold,"Metals & Mining"}	{ABX,IAG,GG,K,GOLD,GFI,NEM,SBGL}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:11:59.933077	2020-10-11 23:38:31.025871
AVD	American Vanguard Corporation ($0.10 Par Value)	American Vanguard Corp is a specialty chemical manufacturer that develops and markets safe and effective products for agricultural, commercial and commercial uses. It manufactures and formulates chemicals for crops, human and animal protection.	{"Basic Materials","Agricultural Inputs",Agriculture}	{HWKN,FMC,SMG,MBII}	Agriculture	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:11:59.947217	2020-10-11 23:38:31.031328
AVNW	Aviat Networks Inc.	Aviat Networks Inc is a networking solutions provider. It offers digital microwave radios and other short-haul communication products and systems to cellular telephone companies, large corporations, government and military agencies in the US and overseas.	{Technology,"Communication Equipment"}	{CRNT,CAMP,MSI,UBNT,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:11:59.958176	2020-10-11 23:38:31.023007
AWK	American Water Works Company Inc.	American Water Works Co Inc provides water and wastewater services to residential, commercial and industrial customers in the United States and Canada.	{Utilities,"Utilities - Regulated Water","Utilities - Regulated"}	{YORW,ARTNA,CTWS,MSEX,CDZI,CWT,AWR,WTR,SJW,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:11:59.968704	2020-10-11 23:38:31.023007
FEYE	FireEye Inc.	FireEye Inc provides cybersecurity solution for detecting, preventing and resolving cyber-attacks that evade legacy signature-based security products. Its solutions include traditional and next-generation firewalls, IPS, anti-virus, and gateways.	{Technology,"Software - Application","Application Software"}	{INTC,CHKP,SYMC,IBM,PFPT,PANW,QLYS,HPQ,IMPV,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:02.828325	2020-10-11 23:38:31.02582
ARE	Alexandria Real Estate Equities Inc.	Alexandria Real Estate Equities Inc is an urban office REIT engaged in life science and technology campuses through ownership, operation, management and selective redevelopment and development of properties in office/laboratory space.	{"Real Estate","REIT - Office",REITs}	{ROIC,VNO,CHSP,DOC}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:11:59.8177	2020-10-11 23:38:31.02582
ARKW	ARK Web x.0	The investment seeks long-term growth of capital.\n The fund is an actively-managed exchange-traded fund ("ETF") that will invest under normal circumstances primarily (at least 80% of its assets) in domestic and foreign equity securities of companies that are relevant to the fund's investment theme of Web x.0. Under normal circumstances, substantially all of the fund's assets will be invested in equity securities, including common stocks, partnership interests, business trust shares and other equity investments or ownership interests in business enterprises. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.826824	2020-10-11 23:38:31.024353
AROW	Arrow Financial Corporation	Arrow Financial Corp is a holding company. It provides various advisory and administrative services and coordinates the general policies and operation of the banks as well as online and mobile banking, mortgages, commercial loans, investments and other.	{"Financial Services","Banks - Regional - US",Banks}	{BDGE,FISI,FLIC,NBTB,SBNY,TMP,CBU,FNB,EVBN}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:11:59.835922	2020-10-11 23:38:31.028166
ASA	ASA Gold and Precious Metals Limited	ASA Gold And Precious Metals Ltd is a closed-end, non-diversified investment company. It provides investors a vehicle to invest in a portfolio of companies engaged in the exploration, mining or processing of gold, silver or other precious minerals.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.847904	2020-10-11 23:38:31.02582
ASHR	Xtrackers Harvest CSI 300 China A-Shares	The investment seeks investment results that correspond generally to the performance, before fees and expenses, of the CSI 300 Index.\n The fund will normally invest at least 80% of its total assets in securities of issuers that comprise the underlying index. The underlying index is designed to reflect the price fluctuation and performance of the China A-Share market and is composed of the 300 largest and most liquid stocks in the China A-Share market. The underlying index includes small-cap, mid-cap, and large-cap stocks.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.858675	2020-10-11 23:38:31.028166
ASNS	Arsanis Inc.	Arsanis Inc is a biopharmaceutical company. It is engaged in the discovery, development and commercialization of monoclonal antibodies for the targeted immunotherapy of serious infections.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:11:59.874442	2020-10-11 23:38:31.028166
ASYS	Amtech Systems Inc.	Amtech Systems Inc is a manufacturer of capital equipment, including thermal processing, silicon wafer handling automation, and related consumables used in fabricating solar cells, LED and semiconductor devices.	{Technology,"Semiconductor Equipment & Materials",Semiconductors}	{AMAT,CVV,KLAC,LRCX,BRKS,ACLS,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:11:59.886642	2020-10-11 23:38:31.024353
ATGE	Adtalem Global Education Inc.	Adtalem Global Education Inc is a provider of educational services. The company offers programs in business, healthcare, professional education, and technology and business.	{"Consumer Defensive","Education & Training Services",Education}	{CECO,LINC,APEI,LOPE,NAUH}	Education	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:11:59.896892	2020-10-11 23:38:31.028166
ATNI	ATN International Inc.	ATN International Inc is a telecommunications and utilities company. It provides telecommunications services to rural, niche and other under-served markets and geographies in the United States, Bermuda and the Caribbean.	{"Communication Services","Telecom Services"}	{ALSK,CMCSA,CTL,VZ,TI,T,IDT,CBB}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:11:59.90776	2020-10-11 23:38:31.021426
ATRO	Astronics Corporation	Astronics Corp supplies products to the aerospace, defense and other industries.The company has two reportable segments namely Aerospace and Test Systems. It generates a majority of its revenue from the Aerospace segment.	{Industrials,"Aerospace & Defense"}	{CVU,DCO,ESL,TDG,SPR,MOG.B,AIRI}	Aerospace & Defense	Industrials	Nasdaq Global Select	\N	2020-10-11 23:11:59.919841	2020-10-11 23:38:31.023428
ATVI	Activision Blizzard Inc	Activision Blizzard is a developer and publisher of interactive entertainment content and services. It develops and distributes content and services on video game consoles, personal computers, and mobile devices.	{Technology,"Electronic Gaming & Multimedia","Application Software"}	{MSFT,TTWO,EA,GLUU,SNE,PTE,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:11:59.929097	2020-10-11 23:38:31.023007
AVAL	Grupo Aval Acciones y Valores S.A. ADR (Each representing 20 preferred shares)	Grupo Aval Acciones y Valores SA is a banking company. The Company provides financial services and products ranging from traditional banking services, such as making loans and taking deposits, to pension and severance fund management.	{"Financial Services","Banks - Regional - Latin America",Banks}	{BK,C}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.944673	2020-10-11 23:38:31.028166
AVH	Avianca Holdings S.A. American Depositary Shares (Each representing 8 preferred Shares)	Avianca Holdings SA is a Panamanian company that represents the integrated operation of several airlines that operate nationally and internationally.	{Industrials,Airlines}	{CPA,DAL,LTM,VLRS,AAL,GOL,UAL}	Airlines	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.958092	2020-10-11 23:38:31.025871
KALU	Kaiser Aluminum Corporation	Kaiser Aluminum Corp is engaged in the production of semi-fabricated specialty aluminum products. The Company has one operating segment; Fabricated Products.	{"Basic Materials",Aluminum,"Metals & Mining"}	{CENX,ACH,CSTM,AA}	Metals & Mining	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:12:05.65545	2020-10-11 23:38:31.028166
ARCT	Arcturus Therapeutics Ltd.	Arcturus Therapeutics Ltd is a biotechnology company developing novel RNA medicines. The company develops novel RNA medicines in therapeutic areas including infectious disease, cystic fibrosis, nonalcoholic steatohepatitis (NASH) and rare liver diseases.	{Healthcare,Biotechnology}	{SHPG,PSTI,NVS,JNJ,LLY,SUPN,TEVA,ITCI,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:11:59.811691	2020-10-11 23:38:31.021426
ARI	Apollo Commercial Real Estate Finance Inc	Apollo Commercial Real Estate Finance Inc invests, acquires, and manages various real-estate related debt investments across the United States. The subordinate loans and first-mortgage loans account for the vast majority of the portfolio on a cost basis.	{"Real Estate","REIT - Diversified",REITs}	{NYMT,O,NTP}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:11:59.821114	2020-10-11 23:38:31.028166
ARNA	Arena Pharmaceuticals Inc.	Arena Pharmaceuticals Inc is a biopharmaceutical company. The Company is engaged in discovering, developing and commercializing novel drugs that target G protein-coupled receptors, or GPCRs, to address unmet medical needs.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.830269	2020-10-11 23:38:31.02423
ARRY	Array BioPharma Inc.	Array BioPharma Inc is a biopharmaceutical company. It is engaged in the discovery, development and commercialization of targeted small molecule drugs to treat patients afflicted with cancer.	{Healthcare,Biotechnology}	{AMGN,CELG,AZN,NVS,LLY,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:11:59.841119	2020-10-11 23:38:31.028166
ASMB	Assembly Biosciences Inc.	Assembly Biosciences Inc is a clinical stage biotechnology company advancing two innovative platform programs, a new class of oral therapeutics for the treatment of hepatitis B virus infection and novel class of oral synthetic live biotherapeutics.	{Healthcare,Biotechnology}	{PGNX,IRWD,SGYP,CNAT,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:11:59.863368	2020-10-11 23:38:31.024353
ASPN	Aspen Aerogels Inc.	Aspen Aerogels Inc is an energy technology company that designs, develops and manufactures high-performance aerogel insulation used in energy infrastructure and construction facilities.	{"Basic Materials","Building Materials"}	{UFPT,CXDC,CBT,OC}	Building Materials	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:11:59.875196	2020-10-11 23:38:31.02582
ATAI	ATA Inc.	ATA Inc is a computer-based testing service provider. The company offers testing systems such as test development, test delivery, and data management. It also provides test-based training, education programs, and test preparation courses to individuals.	{"Consumer Defensive","Education & Training Services",Education}	{CTXS,TAL,HMHC}	Education	Consumer Defensive	NASDAQ Global Market	\N	2020-10-11 23:11:59.887264	2020-10-11 23:38:31.02582
ATHM	Autohome Inc. American Depositary Shares each representing one class A.	Autohome Inc through its subsidiaries is engaged in providing online advertising and dealer subscription services in China. The company offers online destination for automobile consumers through its two websites and through mobile application.	{Technology,"Internet Content & Information","Online Media"}	{SINA,JOBS,BITA,WUBA,TRUE,SFUN,XLK}	Online Media	Technology	New York Stock Exchange	\N	2020-10-11 23:11:59.898416	2020-10-11 23:38:31.024353
ATO	Atmos Energy Corporation	Atmos Energy Corp is engaged in the utilities sector. It distributes natural gas to residential, commercial, public authority & industrial customers.	{Utilities,"Utilities - Regulated Gas","Utilities - Regulated"}	{RGCO,NI,OKE,CPK,NFG,VVC,CNP,OGS,SR,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:11:59.910723	2020-10-11 23:38:31.024353
ATTO	Atento S.A.	Atento S.A is a provider of CRM BPO services and solutions in Latin America.	{Industrials,"Business Services"}	{FORR,CRAI,HCKT,VEC}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.921791	2020-10-11 23:38:31.028166
AUDC	AudioCodes Ltd.	AudioCodes Ltd is a Voice-over-IP technology company providing advanced Voice-over-IP, converged VoIP and Data networking products and communications applications for Service Providers and Enterprises. It has offices in more than 20 countries.	{Technology,"Communication Equipment"}	{CSCO,ADTN,NTGR,JNPR,TXN,RBBN,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:11:59.933916	2020-10-11 23:38:31.031328
AVB	AvalonBay Communities Inc.	AvalonBay Communities Inc operates in the real estate sector. As an investment trust, it develops multifamily communities in the United States.	{"Real Estate","REIT - Residential",REITs}	{EQR,MAA,ESS,AIV,UDR,IRT}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:11:59.946975	2020-10-11 23:38:31.028166
AVT	Avnet Inc.	Avnet Inc is a value-added distributor of electronic components, enterprise computer and storage products, IT solutions and services and embedded systems. Its business segments are Electronics Marketing and Technology Solutions.	{Technology,"Electronics Distribution","Computer Hardware"}	{INTC,TAIT,RELL,TECD,ARW,IBM,SNX,HPQ,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:11:59.961338	2020-10-11 23:38:31.031328
AWX	Avalon Holdings Corporation	Avalon Holdings Corp is the US-based waste management company. It provides waste management services to industrial, commercial, municipal and governmental customers in selected northeastern and midwestern U.S. markets.	{Industrials,"Waste Management"}	{HCCI,ECOL,RSG,WM,PESI}	Waste Management	Industrials	NYSE American	\N	2020-10-11 23:11:59.973752	2020-10-11 23:38:31.02423
ASM	Avino Silver & Gold Mines Ltd. (Canada)	Avino Silver & Gold Mines Ltd is engaged in the production and sale of silver, gold, and copper and the acquisition, exploration and advancement of mineral properties.	{"Basic Materials",Silver,"Metals & Mining"}	{PAAS,MAG,AXU,AG,GPL}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:11:59.863352	2020-10-11 23:38:31.028166
ASNA	Ascena Retail Group Inc.	Ascena Retail Group Inc, through its subsidiaries, operates as a specialty retailer of apparel for women and tween girls and boys. The Company offers apparel, accessories, footwear, and lifestyle products, such as bedroom furnishings and electronics.	{"Consumer Cyclical","Apparel Stores","Retail - Apparel & Specialty"}	{CHS,CATO,CBK,NWY,LB}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:11:59.872525	2020-10-11 23:38:31.025871
ATEC	Alphatec Holdings Inc.	Alphatec Holdings Inc is a medical technology company. It is engaged in the design, development and promotion of products for the surgical treatment of spine disorders. Its product portfolio includes Biolocs, cervical, interbody, and others.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{IART,NUVA,SYK,ZBH,GMED,MDT,XLV}	Medical Instruments & Equipment	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.890505	2020-10-11 23:38:31.02423
ATKR	Atkore International Group Inc.	Atkore International Group Inc manufactures Electrical Raceway products for the non-residential construction and renovation markets and Mechanical Products & Solutions for the construction and industrial market.	{Industrials,"Diversified Industrials","Industrial Products"}	{NOVT}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.902504	2020-10-11 23:38:31.025871
ATOM	Atomera Incorporated	Atomera Inc is engaged in the business of developing, commercializing and licensing proprietary processes and technologies for the semiconductor industry. Its Mears Silicon Technology that can be applied as a transistor channel enhancement.	{Technology,"Semiconductor Equipment & Materials",Semiconductors}	{XLK}	Semiconductors	Technology	NASDAQ Capital Market	\N	2020-10-11 23:11:59.914722	2020-10-11 23:38:31.028166
ATTU	Attunity Ltd.	Attunity Ltd is a provider of data integration and Big Data management software solutions that enable availability, delivery and management of data across heterogeneous enterprise platforms, organizations, and the cloud.	{Technology,"Software - Application","Application Software"}	{MSFT,TISA,PRGS,IBM,ORCL,HPQ,XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:11:59.92414	2020-10-11 23:38:31.031328
AUSE	WisdomTree Australia Dividend Index	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree Australia Dividend Index.\n Under normal circumstances, at least 95% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in the component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a fundamentally weighted index that is comprised of high-dividend yielding companies in Australia. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.938563	2020-10-11 23:38:31.024353
AVDL	Avadel Pharmaceuticals plc	Avadel Pharmaceuticals PLC is a specialty pharmaceutical company developing and commercializing pharmaceutical products. The company's products include Bloxiverz, Vazculep, Akovaz, Karbinal ER, AcipHex Sprinkle, Cefaclor and Flexichamber.	{Healthcare,Biotechnology}	{DRRX,NKTR,GSK,RDY,ALKS,TEVA,EGRX,HRTX,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:11:59.949394	2020-10-11 23:38:31.021426
AVX	AVX Corporation	AVX Corp manufactures and supplies passive electronic components interconnect devices and related products including ceramic and tantalum capacitors, film capacitors, and other components. It operates majorly in Asia, and rest in the Americas and Europe.	{Technology,"Electronic Components","Computer Hardware"}	{MXWL,BELFA,LFUS,APH,VSH,TEL,KEM,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:11:59.961439	2020-10-11 23:38:31.02582
AWRE	Aware Inc.	Aware Inc is a software and services company. Its products are used in government and commercial biometrics systems. It also offers engineering services related to software customization, integration, and installation, and complete systems development.	{Technology,"Software - Application","Application Software"}	{ADBE,AAPL,MSTR,FIX,PBI,MMM,INVE,IDN,TEUM,XLK}	Application Software	Technology	NASDAQ Global Market	\N	2020-10-11 23:11:59.973723	2020-10-11 23:38:31.02582
AXR	AMREP Corporation	Amrep Corp, through its subsidiaries, is engaged in the Real Estate business and the Fulfillment Services business.	{Industrials,"Business Services"}	{TWMC,CHKE,SYKE,INOD,EXLS,BKS,WNS,BR}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.985788	2020-10-11 23:38:31.028166
AYX	Alteryx Inc. Class A	Alteryx Inc is an American provider of self-service data analytics software. Its subscription-based platform allows organizations to easily prepare, blend, and analyze data from a multitude of sources and more quickly benefit from data-driven decisions.	{Technology,"Software - Application","Application Software"}	{OTEX,PANW,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:11:59.994959	2020-10-11 23:38:31.028166
BAB	Invesco Taxable Municipal Bond	The investment seeks to track the investment results (before fees and expenses) of the ICE BofAML US Taxable Municipal Securities Plus Index (the "underlying index").\n The fund generally will invest at least 80% of its total assets in taxable municipal securities that comprise the underlying index. The underlying index is designed to track the performance of U.S. dollar-denominated taxable municipal debt publicly issued by U.S. states and territories, and their political subdivisions, in the U.S. market.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.011406	2020-10-11 23:38:31.023007
ASND	Ascendis Pharma A/S	Ascendis Pharma A/S is a biotechnology company that applies its TransCon technology to develop a pipeline of prodrug therapies with profiles that address markets with unmet medical needs.	{Healthcare,Biotechnology}	{AMGN,AGEN,ADXS,AFMD,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.87367	2020-10-11 23:38:31.031328
ATEN	A10 Networks Inc.	A10 Networks Inc provides application networking technologies. Its solutions enable enterprises, service providers, web giants and government organizations to accelerate, secure and optimize the performance of data center applications and networks.	{Technology,"Software - Infrastructure","Application Software"}	{CSCO,MSFT,CTXS,FFIV,RDWR,JNPR,ORCL,IMPV,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:11:59.893272	2020-10-11 23:38:31.023007
ATLO	Ames National Corporation	Ames National Corporation is a bank holding company providing savings accounts, demand deposits, money market accounts and loans comprising real estate, residential, equipment and personal and other services.	{"Financial Services","Banks - Regional - US",Banks}	{UMBF,WTBA,CBSH,GSBC,LARK,QCRH,EFSC,HWBK,MOFG,WFC}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:11:59.905652	2020-10-11 23:38:31.02423
ATRA	Atara Biotherapeutics Inc.	Atara Biotherapeutics Inc is engaged in the development of pharmaceutical products. The company's focus lies on developing therapies addressing serious-life threatening diseases in the United States.	{Healthcare,Biotechnology}	{IMGN,AGTC,AFMD,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.915627	2020-10-11 23:38:31.023007
ATUS	Altice USA Inc. Class A	Altice USA Inc operates in the telecommunications, media and entertainment business. The company has two segments: Cablevision and Cequel through which it provides digital cable television, high-speed Internet, voice, and data products and services.	{"Communication Services","Telecom Services"}	{S}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:11:59.924942	2020-10-11 23:38:31.02423
AUMN	Golden Minerals Company	Golden Minerals Co is an exploration stage company. The company is engaged mining, construction and exploration of precious metals mineral properties in Mexico and Argentina.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{PAAS,MAG,CDE,HL,GPL,TAHO}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:11:59.937928	2020-10-11 23:38:31.021426
AVP	Avon Products Inc.	Avon Products Inc is involved in the household products market. It lays emphasis on manufacturing beauty products including skincare, fragrance, personal care and hair care. The company owns brands such as ANEW, Avon Color and Avon Care.	{"Consumer Defensive","Household & Personal Products","Consumer Packaged Goods"}	{EL,NUS,CL,KMB,PG,REV,COTY,FOSL}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:11:59.958092	2020-10-11 23:38:31.028166
AWP	Aberdeen Global Premier Properties Fund of Beneficial Interest	Alpine Global Premier Properties Fund is a United States based diversified, closed-end management investment company. The Fund's primary investment objective is capital appreciation. The Fund's secondary investment objective is high current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.970512	2020-10-11 23:38:31.028166
AXON	Axovant Sciences Ltd.	Axovant Sciences Ltd is a clinical-stage biopharmaceutical company focused on the acquisition, development and commercialization of novel therapeutics for the treatment of dementia.	{Healthcare,Biotechnology}	{NVAX,FLXN,RDUS,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.980827	2020-10-11 23:38:31.02582
AXU	Alexco Resource Corp (Canada)	Alexco Resource Corp through its subsidiary provides consulting, remediation solutions and project management services in Canada and the USA. It is also engaged in the process of mining, exploring and developing mineral properties in Canada.	{Industrials,"Waste Management"}	{PAAS,MAG,EXK,SVM,HL,AG,GPL,WPM}	Waste Management	Industrials	NYSE American	\N	2020-10-11 23:11:59.991965	2020-10-11 23:38:31.025871
AZRX	AzurRx BioPharma Inc.	AzurRx BioPharma Inc through its subsidiary is engaged in the research and development of non-systemic biologics for the treatment of patients with gastrointestinal disorders. Its products under development includes MS1819 and AZ1101.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:00.00552	2020-10-11 23:38:31.02582
BAND	Bandwidth Inc.	Bandwidth Inc provides cloud-based communications platform to the companies. Its solutions consist of software application programming interface for voice and text functionality and its owned and managed purpose-built internet protocol voice network.	{Technology,"Software - Infrastructure","Application Software"}	{NEWT,ITT,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:00.026652	2020-10-11 23:38:31.028166
BASI	Bioanalytical Systems Inc.	Bioanalytical Systems Inc is a contract research organization providing drug discovery and development services and analytical instruments. The company focuses on the bioanalytical, preclinical, and clinical research needs of drug developers.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{HBIO,DHR,GE,TMO,PKI,CRL,SCYX,IQV,XLV}	Medical Diagnostics & Research	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:00.038882	2020-10-11 23:38:31.02423
BBGI	Beasley Broadcast Group Inc.	Beasley Broadcast Group Inc is a radio broadcasting company. The company operates radio stations including FM and AM radio stations located in large and mid-sized markets in the United States.	{"Consumer Cyclical","Broadcasting - Radio",Entertainment}	{}	Entertainment	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:00.054091	2020-10-11 23:38:31.021426
ASR	Grupo Aeroportuario del Sureste S.A. de C.V.	Grupo Aeroportuario del Sureste SAB de CV holds concessions to operate, maintain and develop close to nine airports in the southeast region of Mexico located in Cancun, Cozumel, Merida, Huatulco, Oaxaca, Veracruz, Villahermosa, Tapachula and Minatitlan.	{Industrials,"Airports & Air Services","Transportation & Logistics"}	{OMAB,PAC,NEE}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.875153	2020-10-11 23:38:31.028166
ASV	ASV Holdings Inc.	ASV Holdings Inc is a designer and manufacturer of compact construction equipment with emphasis on compact track loaders. It also serves as a private label original equipment manufacturer for several manufacturers.	{Industrials,"Farm & Construction Equipment","Farm & Construction Machinery"}	{TEX,PLOW,MTW}	Farm & Construction Machinery	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:11:59.886447	2020-10-11 23:38:31.028166
ATLC	Atlanticus Holdings Corporation	Atlanticus Holdings Corp is focused on providing financial services. It offers financial products and services to a market represented by credits risks that regulators classify as sub-prime.	{"Financial Services","Credit Services"}	{ASFI,ECPG,AXP,DFS}	Credit Services	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:11:59.902791	2020-10-11 23:38:31.02582
ATOS	Atossa Genetics Inc.	Atossa Genetics Inc mainly focuses on the development of novel therapeutics and delivery methods for the treatment of breast cancer and other breast conditions.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{XLV}	Medical Diagnostics & Research	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:11:59.914852	2020-10-11 23:38:31.031328
ATU	Actuant Corporation	Actuant Corp is a diversified industrial company. The company designs, manufactures, and distributes range of industrial products and systems to various markets. It has Industrial, Energy and Engineered solutions segments.	{Industrials,"Diversified Industrials","Industrial Products"}	{SNHY,CR,PH}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.924294	2020-10-11 23:38:31.028166
AUPH	Aurinia Pharmaceuticals Inc	Aurinia Pharmaceuticals Inc is a clinical stage pharmaceutical company. The firm is engaged in the development of a therapeutic drug to treat autoimmune diseases, in particular lupus nephritis.	{Healthcare,Biotechnology}	{REGN,KERX,PIRS,AEZS,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:11:59.938388	2020-10-11 23:38:31.02423
AVEO	AVEO Pharmaceuticals Inc.	AVEO Pharmaceuticals Inc is a biopharmaceutical company focused on discovering, developing and commercializing novel cancer therapeutics. The firm offers products such as Tivozanib, Ficlatuzmab, AV-203, AV-380, and AV-353.	{Healthcare,Biotechnology}	{AMGN,ARQL,REGN,BIIB,BMY,LLY,GTXI,MACK,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:11:59.951402	2020-10-11 23:38:31.024353
AVXL	Anavex Life Sciences Corp.	Anavex Life Sciences Corp is a clinical stage bio-pharmaceutical company. The Company is engaged in the developments drug candidates to treat Alzheimer's disease, other central nervous system (CNS) diseases, and various types of cancer.	{Healthcare,Biotechnology}	{BCRX,CORT,ECYT,PTN,LGND,ABIO,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:11:59.963314	2020-10-11 23:38:31.02423
AXAS	Abraxas Petroleum Corporation	Abraxas Petroleum Corp is an independent energy company engaged in the acquisition, exploitation, development and production of oil and gas in the United States.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{GPOR,AREX,DVN,EOG,XEC,SM,EPE,CRK}	Oil & Gas - E&P	Energy	NASDAQ Capital Market	\N	2020-10-11 23:11:59.975083	2020-10-11 23:38:31.024353
AXTA	Axalta Coating Systems Ltd.	Axalta Coating Systems Ltd is a manufacturer, marketer, and distributor of high-performance coatings systems. It operates its business in two segments, performance coatings, and transportation coatings.	{"Basic Materials","Specialty Chemicals",Chemicals}	{FOE,PPG,RPM,SHW}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:11:59.990077	2020-10-11 23:38:31.031328
AZN	Astrazeneca PLC	AstraZeneca PLC belongs to the healthcare sector. The company develops and markets pharmaceutical products for various therapeutic classes such as gastrointestinal, cardiovascular, respiratory, cancer, immunology, and neuroscience.	{Healthcare,"Drug Manufacturers - Major","Drug Manufacturers"}	{CELG,BIIB,GSK,ABT,BMY,JNJ,LLY,PFE,SNY,MRK,XLV}	Drug Manufacturers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:00.001333	2020-10-11 23:38:31.023007
BABA	Alibaba Group Holding Limited American Depositary Shares each representing one	Alibaba Group Holding Ltd is an online and mobile commerce company. It operates China's most popular online marketplaces such as Taobao (C2C), Tmall (B2C), and Juhuasuan. It also engages in advertising, digital media, cloud computing, and other.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{AMZN,EBAY,OSTK}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:00.015652	2020-10-11 23:38:31.028166
BANFP	BancFirst Corporation 7.2% Cumulative Trust Preferred Securities	BFC Capital Trust II is a business trust.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.030462	2020-10-11 23:38:31.021426
BBBY	Bed Bath & Beyond Inc.	Bed Bath & Beyond Inc operates a chain of retail stores. It operates under the names Bed Bath & Beyond, Christmas Tree Shops and Harmon and Harmon Face Values.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:00.043924	2020-10-11 23:38:31.024353
AUTO	AutoWeb Inc.	AutoWeb Inc, formerly Autobytel Inc is an automotive marketing services company which assists automotive retail dealers and automotive manufacturers to market and sell new and used vehicles to consumers through various online programs.	{Technology,"Internet Content & Information","Online Media"}	{ISIG,SGRP,TRUE,XLK}	Online Media	Technology	NASDAQ Capital Market	\N	2020-10-11 23:11:59.940353	2020-10-11 23:38:31.025871
AVGO	Broadcom Inc.	Broadcom Inc is engaged in designing, developing and supplying analog and digital semiconductor connectivity solutions. Its product portfolio serves wired infrastructure, wireless communications, enterprise storage and industrial and other end markets.	{Technology,Semiconductors}	{XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:11:59.955458	2020-10-11 23:38:31.02423
AVYA	Avaya Holdings Corp.	Avaya Holdings Corp, together with its subsidiaries, provides real-time business collaboration and communications solutions. Its business is segmented into three - Global Communications Solutions, Avaya Networking, and Avaya Global Services.	{Industrials,"Business Services"}	{}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.96793	2020-10-11 23:38:31.021426
AXE	Anixter International Inc.	Anixter International Inc is a value-added distributor of communications and security, electrical wire, and cable products and fasteners.	{Industrials,"Industrial Distribution"}	{PKOH,FAST,DXPE,HWCC,MSM,AIT,WCC,GWW,HDS}	Industrial Distribution	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.979368	2020-10-11 23:38:31.031328
AXSM	Axsome Therapeutics Inc.	Axsome Therapeutics Inc is a pharmaceutical company in the United States. It focuses on developing novel therapeutics for the redressal of diseases related to the central nervous system.	{Healthcare,Biotechnology}	{PDLI,PIRS,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:11:59.990228	2020-10-11 23:38:31.021426
AZO	AutoZone Inc.	AutoZone Inc is a retailer and distributor of automotive replacement parts in the United States. The company's store sell a diverse range of essentials and auto parts for cars, sport utility vehicles, vans and light trucks.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{MNRO,PRTS,HZO,AAP,ORLY}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:00.002612	2020-10-11 23:38:31.021426
BAF	BlackRock Municipal Income Investment Quality Trust	Blackrock Municipal Income Investment Quality Trust is a diversified investment fund with an investment objective is to provide current income exempt from U.S. federal income tax, including the alternative minimum tax and Florida intangible property tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.023169	2020-10-11 23:38:31.021426
BANX	StoneCastle Financial Corp	StoneCastle Financial Corp. is a non-diversified, closed-end management investment company. Its objective is to provide stockholders with current income, and to a lesser extent capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.035599	2020-10-11 23:38:31.031328
BBDO	Banco Bradesco Sa American Depositary Shares (each representing one)	Bank Bradesco SA is one of the largest banks in Brazil. The bank provides services through a network of more than 4,700 locations offering insurance and banking facilities like deposits and loans to individuals and business corporations.	{"Financial Services","Banks - Regional - Latin America",Banks}	{}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.049887	2020-10-11 23:38:31.023428
BBT	BB&T Corporation	BB&T Corporation is a financial holding company. It is a regional banking company with a presence primarily in the Southeastern United States. The company provides a range of banking services, and loans to businesses and consumers.	{"Financial Services","Banks - Regional - US",Banks}	{FCBC,CHCO,HBAN,UBSI,PNC,STI,FHN,BKU,SNV}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.063198	2020-10-11 23:38:31.02582
BCACU	Bison Capital Acquisition Corp. Units Consisting of 1 OS 1/2 WT and 1 RT	Bison Capital Acquisition Corp is a newly organized blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:00.078054	2020-10-11 23:38:31.02423
BCM	iPath Pure Beta Broad Commodity ETN	The investment seeks to provide investors with exposure to the Barclays Commodity Index Pure Beta Total Return. \nThe Barclays Commodity Index Pure Beta Total Return (the "index") is comprised of a basket of exchange traded futures contracts and reflects the returns that are potentially available through an unleveraged investment in the futures contracts on certain physical commodities. For each commodity, the index may roll into one of a number of futures contracts with varying expiration dates, as selected using the Barclays Pure Beta Series 2 Methodology.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.089833	2020-10-11 23:38:31.023428
BCTF	Bancorp 34 Inc.	Bancorp 34 Inc provides a variety of banking services to individuals and businesses through its branches. Its products and services are demand deposits, certificate of deposits, savings and money market accounts, and others.	{"Financial Services","Banks - Regional - US",Banks}	{KFFB}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:00.102091	2020-10-11 23:38:31.028166
BDL	Flanigan's Enterprises Inc.	Flanigan'S Enterprises Inc operates as a chain of small cocktail lounges and packages liquor stores throughout South Florida.	{"Consumer Cyclical",Restaurants}	{DAVE,ARKR,CAKE,TXRH,BJRI,KONA,EAT,BLMN,RICK}	Restaurants	Consumer Cyclical	NYSE American	\N	2020-10-11 23:12:00.116658	2020-10-11 23:38:31.028166
FFBCW	First Financial Bancorp. Warrant 12/23/2018	First Financial Bancorp operates in the financial services sector. As a banking company, it conducts retail and commercial banking and commercial finance operations. It mainly caters to real estate sector with real estate loans.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:02.83231	2020-10-11 23:38:31.024353
AUY	Yamana Gold Inc. (Canada)	Yamana Gold Inc is a Canada-based gold miner operating a portfolio of mines in Brazil, Chile, Argentina, and Mexico. The company also holds stakes in the Alumbrera mine and Canadian Malartic mine.	{"Basic Materials",Gold,"Metals & Mining"}	{ABX,EGO,NG,GG,BBL,K,RIO,MUX,AGI}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:11:59.943731	2020-10-11 23:38:31.02582
AVK	Advent Claymore Convertible Securities and Income Fund	Advent Claymore Convt Security & Income Fund, formerly known as Advent Claymore Convt Security & Income is a diversified, closed-end management investment company.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.95838	2020-10-11 23:38:31.028166
AWR	American States Water Company	American States Water Co is engaged in the purchase, production, and distribution of water. It also provides water and/or wastewater services at various military installations. It operates in three segments water, electric and contracted service.	{Utilities,"Utilities - Regulated Water","Utilities - Regulated"}	{YORW,ARTNA,CTWS,MSEX,CDZI,PCYO,CWT,WTR,AWK,SJW,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:11:59.970338	2020-10-11 23:38:31.028166
AXGN	AxoGen Inc.	AxoGen Inc is a medical technology company dedicated to peripheral nerve repair. It provide products and education to improve surgical treatment algorithms for peripheral nerve injuries.	{Healthcare,"Medical Devices"}	{BSX,TMO,MDT,XLV}	Medical Devices	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:11:59.979677	2020-10-11 23:38:31.023428
AY	Atlantica Yield plc	Atlantica Yield PLC owns, manages and acquires renewable energy, conventional power, electric transmission lines and other contracted revenue-generating assets, focused on North America, South America and Europe.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{ENIA,CIG,ELP,CPL,EDN,XLU}	Utilities - Regulated	Utilities	Nasdaq Global Select	\N	2020-10-11 23:11:59.991782	2020-10-11 23:38:31.02582
AZUL	Azul S.A. American Depositary Shares (each representing three preferred shares)	Azul SA through its subsidiaries provides airline passenger and cargo services. The company's airlines flies to select international destinations, including Fort Lauderdale, Orlando, and Lisbon.	{Industrials,Airlines}	{JBLU}	Airlines	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:00.00689	2020-10-11 23:38:31.02423
BAH	Booz Allen Hamilton Holding Corporation	Booz Allen Hamilton Holding Corp provides management and technology consulting, engineering, analytics, digital solutions, and cyber expertise to U.S. and international governments, major corporations, and not-for-profit organizations.	{Industrials,"Business Services"}	{}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:00.023489	2020-10-11 23:38:31.031328
BAR	GraniteShares Gold Trust Shares of Beneficial Interest	The investment seeks to reflect generally the performance of the price of gold. The Shares are intended to constitute a simple and cost-effective means of making an investment similar to an investment in gold.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.035225	2020-10-11 23:38:31.023428
BBP	Virtus LifeSci Biotech Products	The investment seeks investment results that correspond, before fees and expenses, to the price and yield performance of the LifeSci Biotechnology Products Index.\n Under normal market conditions, the fund will invest not less than 80% of its assets in component securities of the index. The index seeks to track the performance of the common stock of U.S. exchange-listed biotechnology companies with at least one drug therapy approved by the U.S. Food and Drug Administration ("FDA") for marketing. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.057291	2020-10-11 23:38:31.023428
BCAC	Bison Capital Acquisition Corp.	Bison Capital Acquisition Corp is a newly organized blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:00.07517	2020-10-11 23:38:31.021426
BCI	Aberdeen Standard Bloomberg All Commodity Strategy K-1 Free	The investment seeks to provide a total return designed to exceed the performance of the Bloomberg Commodity IndexSM which is calculated on an excess return basis.\n The fund is an actively managed exchange traded fund that seeks to provide a total return designed to exceed the performance of the index. It will generally seek to hold similar interests to those included in the index and will seek exposure to many of the commodities included in the index under the same futures rolling schedule as the index. The fund will also hold short-term fixed-income securities. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.088788	2020-10-11 23:38:31.028166
BCPC	Balchem Corporation	Balchem Corp is a chemical manufacturer. It develops and sells chemical based ingredients for the food and pharmaceutical industries.	{"Basic Materials",Chemicals}	{IPHS,CDXS,EMN,IFF,MMM,KMG,KHC,STE}	Chemicals	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:12:00.097912	2020-10-11 23:38:31.031328
BDCS	UBS AG Exchange Traded Access Securities (E TRACS) Linked to the Wells Fargo Business Development Company Index due April 26 2041	The investment seeks to replicate, net of expenses, the performance of the Wells Fargo Business Development Company Index.\n The index is a float adjusted, capitalization-weighted index that is intended to measure the performance of all Business Development Companies (“BDC”) that are listed on the New York Stock Exchange or NASDAQ and satisfy specified market capitalization and other eligibility requirements. The BDC business model is to lend to small and midsized companies at high yield equivalent rates while also at times taking equity stakes in such companies.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.109743	2020-10-11 23:38:31.02582
AXL	American Axle & Manufacturing Holdings Inc.	American Axle & Mfg Holdings Inc manufactures, engineers, designs and validates driveline systems and drivetrain components and chassis modules. Its products are mostly used in light trucks, Sports Utility Vehicles, commercial vehicles and passenger cars.	{"Consumer Cyclical","Auto Parts",Autos}	{MGA,STRT,BWA,SUP,TEN,DAN,LEA,SRI,MLR,TOWR}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:11:59.980685	2020-10-11 23:38:31.024353
AYI	Acuity Brands Inc (Holding Company)	Acuity Brands Inc through its subsidiaries provides lighting and building management solutions and services for commercial, institutional, industrial, and residential applications. Its products include luminaires, lighting controls and components etc.	{Technology,"Electronic Components","Computer Hardware"}	{PHG,OESX,HUBB,RVLT,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:11:59.991468	2020-10-11 23:38:31.024353
AZRE	Azure Power Global Limited Equity Shares	Azure Power Global Ltd produces and sells solar power in India. It is engaged in the development, construction, ownership, operation, maintenance and management of solar power plants and generation of solar energy.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{VSLR,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:00.003313	2020-10-11 23:38:31.031328
BABY	Natus Medical Incorporated	Natus Medical Inc provides newborn care, neurology healthcare products and services used for screening, diagnosis, detection, treatment, monitoring and tracking of medical ailments in newborn care, epilepsy, and balance and mobility disorders.	{Healthcare,"Medical Devices"}	{MASI,CNMD,COO,GE,TMO,PHG,LIVN,AHPI,NURO,XLV}	Medical Devices	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:00.02049	2020-10-11 23:38:31.025871
BANR	Banner Corporation	Banner Corp operates through the Banner Bank and Islanders Bank. It provides commercial banking services and financial products to individuals, businesses, and public sector entities in its primary market areas.	{"Financial Services","Banks - Regional - US",Banks}	{UMPQ,COLB,HFWA,EWBC,FFNW}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.033615	2020-10-11 23:38:31.02582
BB	BlackBerry Limited	BlackBerry Ltd is a Canada-based designer, manufacturer and marketer of wireless solutions for the mobile communications market. It also owns QNX, a leader in software used in automotive infotainment systems.	{Technology,"Communication Equipment"}	{XLK}	Communication Equipment	Technology	New York Stock Exchange	\N	2020-10-11 23:12:00.044243	2020-10-11 23:38:31.023007
BBOX	Black Box Corporation	Black Box Corp is a technology solution provider. It provides solutions to design, build, manage and secure IT infrastructure of its customers.	{Technology,"Communication Equipment"}	{XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:00.057593	2020-10-11 23:38:31.02423
BBW	Build-A-Bear Workshop Inc.	Build-A-Bear Workshop Inc is a specialty retailer which offers "make your own stuffed animal" interactive entertainment experience in which guests visit a variety of stations to make and customize a stuffed animal.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{MAT,WINA,DIS,OMEX}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:00.074182	2020-10-11 23:38:31.025871
BCOV	Brightcove Inc.	Brightcove Inc offers cloud solutions to deliver and monetize video across connected devices. The company supports its clients to effectively operate their businesses be it broadcasting, publishing, marketing, or enterprise communications.	{Technology,"Software - Application","Application Software"}	{XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:00.097935	2020-10-11 23:38:31.021426
BDCZ	ETRACS Wells Fargo Business Development Company Index ETN Series B due April 26 2041	The investment seeks to provide exposure to the performance of the Wells Fargo® Business Development Company Index.\n The ETRACS Wells Fargo® Business Development Company Index ETN Series B due April 26, 2041 are senior unsecured debt securities issued by UBS AG. The index is intended to measure the performance of all business development companies listed on the NYSE or NASDAQ that satisfy specified market capitalization and other eligibility requirements. It refers to the business development companies included in the Wells Fargo® Business Development Company Index as the “index constituents.” The index is a proprietary index of Wells Fargo Securities, LLC.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.109543	2020-10-11 23:38:31.031328
BDSI	BioDelivery Sciences International Inc.	BioDelivery Sciences International Inc is a specialty pharmaceutical company. It is developing and commercializing new applications of approved therapeutics to address important unmet medical needs.	{Healthcare,Biotechnology}	{MNOV,JNJ,PFE,ACRX,INSY,ENDP,TTNP,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:00.12183	2020-10-11 23:38:31.028166
BELFB	Bel Fuse Inc. Class B Common Stock	Bel Fuse Inc designs and manufactures electronic components that protect and connect electronic circuits. Its product portfolio is divided into three categories namely, magnetic solutions, power solutions and protection, and connectivity solutions.	{Technology,"Electronic Components","Computer Hardware"}	{LFUS,TRNS,AVX,APH,ESP,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:00.132276	2020-10-11 23:38:31.021426
BFO	Blackrock Florida Municipal 2020 Term Trust	Blackrock Florida Muni 2020 Term Tr is a non-diversified investment fund with an investment objective to provide current income exempt from regular U.S. federal income tax and Florida intangible personal property tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.147294	2020-10-11 23:38:31.024353
AXS	Axis Capital Holdings Limited	Axis Capital Holdings Ltd is a global property and casualty insurance company. The firm along with its subsidiaries provides its clients and distribution partners with various products and services around the world.	{"Financial Services","Insurance - Property & Casualty"}	{ACGL,MHLD,RNR,AHL}	Insurance - Property & Casualty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.990052	2020-10-11 23:38:31.02423
AZPN	Aspen Technology Inc.	Aspen Technology Inc is engaged in providing process optimization and asset performance management software solutions. It has two operating segments including subscription and software and services.	{Technology,"Software - Application","Application Software"}	{SAP,ROK,GE,HON,QADB,PTC,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:00.003397	2020-10-11 23:38:31.025871
BAC	Bank of America Corporation	Bank of America Corporation is a bank holding and a financial holding company. The company provides financial products and services to people, companies and institutional investors.	{"Financial Services","Banks - Global",Banks}	{TRMK,BCS,BBT,WFC,TCF,JPM,USB,C}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.023213	2020-10-11 23:38:31.023428
BAP	Credicorp Ltd.	Credicorp Ltd is a financial services company in Peru. It provides financial services and products. It operates in four segments namely Banking, Insurance, Pension fund and Investment banking.	{"Financial Services","Banks - Regional - Latin America",Banks}	{BNS,RY,TD,GGAL,HSBC,CIB,BBVA,BCH,BBD,ITUB}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.035553	2020-10-11 23:38:31.023007
BBF	BlackRock Municipal Income Investment Trust	Blackrock Municipal Inc Invstmt Tr is a closed-end management investment company. Its investment objective is to provide current income exempt from regular federal income tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.049621	2020-10-11 23:38:31.02582
BBSI	Barrett Business Services Inc.	Barrett Business Services Inc is a provider of staffing solutions in the United States. It conducts its business through two services; Professional Employer Service and Staffing.	{Industrials,"Staffing & Outsourcing Services","Employment Services"}	{ADP,KELYA,KFRC,MAN,RHI,TBI,MHH,NSP,TNET}	Employment Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:00.06227	2020-10-11 23:38:31.028166
BCC	Boise Cascade L.L.C.	Boise Cascade Co is a lumber and wood products manufacturer and distributor. It mainly caters to the construction industry through its array of products comprise of plywood, lumber, roofing and composite decking.	{"Basic Materials","Lumber & Wood Production","Forest Products"}	{}	Forest Products	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:00.086563	2020-10-11 23:38:31.031328
BCO	Brinks Company (The)	The Brink's Co along with its subsidiaries provides secure logistics & security solutions including cash-in-transit, ATM replenishment and maintenance, cash management services, including vault outsourcing, money processing, and intelligent safe services.	{Industrials,"Security & Protection Services","Consulting & Outsourcing"}	{}	Consulting & Outsourcing	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:00.096985	2020-10-11 23:38:31.025871
BCX	BlackRock Resources of Beneficial Interest	BlackRock Resources & Commodities Strategy Trust is a investment trust. Its investment objective is to seek high current income and current gains, with a secondary objective of capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.108292	2020-10-11 23:38:31.021426
BDR	Blonder Tongue Laboratories Inc.	Blonder Tongue Laboratories Inc is a technology-development and manufacturing company which provides television signal encoding, transcoding, digital transport, and broadband product solutions in the United States.	{Technology,"Communication Equipment"}	{CSCO,INTC,HLIT,COMM,VIAV,XLK}	Communication Equipment	Technology	NYSE American	\N	2020-10-11 23:12:00.122199	2020-10-11 23:38:31.021426
BEP	Brookfield Renewable Partners L.P.	Brookfield Renewable Partners LP is a renewable power generating company. It portfolio of renewable power generating facilities is spread across North America, Colombia, Brazil, Europe, and other countries.	{Utilities,"Utilities - Independent Power Producers"}	{NEP,XLU}	Utilities - Independent Power Producers	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:00.134871	2020-10-11 23:38:31.02582
BFS	Saul Centers Inc.	Saul Centers Inc is a self-managed real estate investment trust which invests in, operates, and develops retail and commercial properties. Its portfolio includes community and neighborhood shopping centers, office properties, and mixed-use properties.	{"Real Estate","REIT - Retail",REITs}	{CPT,ADC,SRC,ARR}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:00.14937	2020-10-11 23:38:31.025871
BGG	Briggs & Stratton Corporation	Briggs & Stratton Corp is a producer of gasoline engines and outdoor power equipment. It operates in two segments including Engines segment and Products segment.	{Industrials,"Diversified Industrials","Industrial Products"}	{BC,CMI,DE,GE,HD,TTC,GNRC}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:00.167177	2020-10-11 23:38:31.023428
BHBK	Blue Hills Bancorp Inc.	Blue Hills Bancorp Inc is a banking solutions provider in the United States. Its products range from checking, savings, and term certificate accounts to home equity loans and commercial real estate loans.	{"Financial Services","Banks - Regional - US",Banks}	{PFBI,PLBC,PKBK,FNCB}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.201695	2020-10-11 23:38:31.023007
AXTI	AXT Inc	AXT Inc is a developer and producer of high-performance compound and single element semiconductor substrates known as wafers. It provides such alternative or specialty materials in the form of substrates or wafers, compound, and substrates.	{Technology,"Semiconductor Equipment & Materials",Semiconductors}	{VECO,ASYS,KOPN,LRCX,PLAB,SWKS,QRVO,ACLS,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:11:59.991901	2020-10-11 23:38:31.02423
AZZ	AZZ Inc.	AZZ Inc is a provider of specialized products and services designed to support industrial, nuclear and electrical applications. The Company's business segments are Energy and Metal Coating.	{Industrials,"Diversified Industrials","Industrial Products"}	{POWL,AETI,NOV,RBC,VMI,AMSC,HUBB}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:00.00689	2020-10-11 23:38:31.028166
BAK	Braskem SA ADR	Braskem SA is a petrochemical company. It produces a diversified portfolio of petrochemical and thermoplastic products, thermoplastic resins, including polyethylene, polypropylene and PVC.	{"Basic Materials",Chemicals}	{PBR,CE}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:00.024498	2020-10-11 23:38:31.02582
BAS	Basic Energy Services Inc.	Basic Energy Services Inc provides well site services to oil and natural gas drilling and producing companies. Its operations are grouped into Completion and Remedial Services, Fluid Services, Well Servicing, and Contract Drilling.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{PBFX,PARR}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:00.036481	2020-10-11 23:38:31.025871
BBD	Banco Bradesco Sa American Depositary Shares	Bank Bradesco SA is one of the largest banks in Brazil. The bank provides services through a network of more than 4,700 locations offering insurance and banking facilities like deposits and loans to individuals and business corporations.	{"Financial Services","Banks - Regional - Latin America",Banks}	{CM,RY,HSBC,BBVA,BCH,MUFG,ITUB}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.047138	2020-10-11 23:38:31.031328
BBRC	Columbia Beyond BRICs	The investment seeks investment results that correspond (before fees and expenses) to the price and yield performance of the FTSE Beyond BRICs Index.\n The fund will invest at least 80% of its net assets in companies included in the index and the advisor expects to be substantially invested at such times, with at least 95% of its net assets invested in these securities. The index is a market capitalization-weighted index designed to represent the performance of a diversified basket of 90 companies in emerging and frontier markets excluding BRIC countries and Taiwan, as defined by FTSE's Country Classification System.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.058971	2020-10-11 23:38:31.021426
BCACR	Bison Capital Acquisition Corp. Rights		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:00.075458	2020-10-11 23:38:31.023428
BCLI	Brainstorm Cell Therapeutics Inc.	Brainstorm Cell Therapeutics Inc is a biotechnology company. The Company develops novel adult stem cell therapies for debilitating neurodegenerative disorders such as ALS, MS, and PD among others.	{Healthcare,Biotechnology}	{GILD,SGEN,BIIB,MNTA,AGEN,CYTK,XLRN,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:00.089588	2020-10-11 23:38:31.023007
BCRX	BioCryst Pharmaceuticals Inc.	Biocryst Pharmaceuticals Inc is a pharmaceutical company that designs, optimizes and develops novel small-molecule drugs that block key enzymes involved in the infectious and inflammatory diseases. It has its focus on oral treatments for rare diseases.	{Healthcare,Biotechnology}	{NVAX,GILD,RGEN,CLDX,GSK,EBS,SRPT,NNVC,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:00.100741	2020-10-11 23:38:31.023007
BDJ	Blackrock Enhanced Equity Dividend Trust	BlackRock Enhanced Equity Dividend Trust is a United States closed-end management investment company. Its investment objective is to seek current income and current gains, with a secondary objective of long-term capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.113032	2020-10-11 23:38:31.023428
BEF	Aberdeen Standard Bloomberg Energy Commodity Longer Dated Strategy K-1 Free	The investment seeks to provide a total return designed to exceed the performance of the Bloomberg Energy Index 3 Month ForwardSM which is calculated on an excess return basis.\n The fund is an actively managed exchange traded fund that seeks to provide a total return designed to exceed the performance of the index. It will generally seek to hold similar interests to those included in the index and will seek exposure to many of the commodities included in the index under the same futures rolling schedule as the index. The fund will also hold short-term fixed-income securities. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.127976	2020-10-11 23:38:31.028166
BFIN	BankFinancial Corporation	BankFinancial Corp is a bank holding company of BankFinancial, F.S.B., which is a full-service, community-oriented federal savings bank principally engaged in the business of commercial, family and personal banking.	{"Financial Services","Banks - Regional - US",Banks}	{FDEF,FCAP,TFSL,IROQ}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.140085	2020-10-11 23:38:31.023007
BGCP	BGC Partners Inc.	BGC Partners Inc is engaged in capital market operations. It caters mainly to the real estate market with products like fixed income securities, interest rate swaps, foreign exchange, equities and equity derivatives.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{NDAQ,ITG,HF,INTL,ICE}	Brokers & Exchanges	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.153098	2020-10-11 23:38:31.031328
BGT	BlackRock Floating Rate Income Trust	Blackrock Floating Rate Income Trust has an investment objective to provide a high level of current income. Its secondary investment objective is to seek the preservation of capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.181965	2020-10-11 23:38:31.02582
AYR	Aircastle Limited	Aircastle Ltd is a holding company. It mainly invests in aviation assets, including acquiring, leasing, managing and selling high-utility commercial jet aircraft.	{Industrials,"Rental & Leasing Services","Consulting & Outsourcing"}	{WLFC,TGH,AER,CAI,FLY,ACY,AL}	Consulting & Outsourcing	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.992298	2020-10-11 23:38:31.028166
B	Barnes Group Inc.	Barnes Group Inc is engaged in the aerospace industry and provides industrial services. The company provides engineered products for aircraft, technologies, and solutions.	{Industrials,"Diversified Industrials","Industrial Products"}	{TGI,BA,CW,GE,KMT,MLI,TKR,NPO,HEI.A,SPLP}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:00.010104	2020-10-11 23:38:31.024353
BAM	Brookfield Asset Management Inc.	Brookfield Asset Management Inc is an alternative asset management company focused on property, renewable energy, infrastructure and private equity.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.024581	2020-10-11 23:38:31.02423
BATRA	Liberty Media Corporation Series A Liberty Braves Common Stock	Liberty Braves Group through its subsidiary indirectly owns the Atlanta Braves Major League Baseball club, the Atlanta Braves' stadium and associated real estate projects.	{"Consumer Cyclical",Leisure,"Travel & Leisure","Media - Diversified",Entertainment}	{}	Entertainment	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:00.043836	2020-10-11 23:38:31.021426
BBN	BlackRock Taxable Municipal Bond Trust of Beneficial Interest	BlackRock Taxable Municipal Bond Trust is a diversified, closed-end management investment company. Its investment objective is to seek high current income, with a secondary objective of capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.05652	2020-10-11 23:38:31.023007
BBX	BBX Capital Corporation Class A	BBX Capital Corp is a Florida-based holding company. Its subsidiaries and primary investments, include BBX Capital, Bluegreen Corporation, and Middle Market. The BBX Capital segment consists of BBX Capital's monetization and management of legacy assets.	{"Real Estate","Real Estate Services"}	{SIEB,VAC,FPAY}	Real Estate Services	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:00.074155	2020-10-11 23:38:31.02423
BCE	BCE Inc.	BCE Inc is a telecommunication company. It caters to residential, business and wholesale customers with solutions for all their communications needs.	{"Communication Services","Telecom Services"}	{RCI,SJR,AAPL,BT,TU,S}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:00.088169	2020-10-11 23:38:31.02423
BCV	Bancroft Fund Ltd.	Bancroft Fund Ltd is a diversified closed-end management investment company. The funds' main objective is to provide income and potential for capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:00.103102	2020-10-11 23:38:31.028166
BDN	Brandywine Realty Trust	Brandywine Realty Trust is a self-administered and self-managed REIT. The company provides leasing, property management, development, redevelopment, acquisition and other tenant-related services.	{"Real Estate","REIT - Office",REITs}	{}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:00.116608	2020-10-11 23:38:31.02423
BELFA	Bel Fuse Inc. Class A Common Stock	Bel Fuse Inc designs and manufactures electronic components that protect and connect electronic circuits. Its product portfolio is divided into three categories namely, magnetic solutions, power solutions and protection, and connectivity solutions.	{Technology,"Electronic Components","Computer Hardware"}	{LFUS,TRNS,AVX,APH,ESP,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:00.129923	2020-10-11 23:38:31.023428
BFY	BlackRock New York Municipal Income Trust II	Blackrock New York Muni Income Tr II is a perpetual closed-end municipal bond fund. Its objective is to provide current income exempt from regular Federal income taxes and and New York State and New York City personal income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.149315	2020-10-11 23:38:31.02582
BGIO	BlackRock 2022 Global Income Opportunity Trust of Beneficial Interest	BlackRock 2022 Global Income Opportunity is a closed end management investment company. Its objective is high current income and its secondary objective is capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.175903	2020-10-11 23:38:31.024353
BHK	Blackrock Core Bond Trust	Blackrock Core Bond Trust is a United States based closed-end management investment company. Its objective is to provide current income and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.202021	2020-10-11 23:38:31.028166
BIIB	Biogen Inc.	Biogen Inc is a biopharmaceutical company. It is engaged in discovering, developing, manufacturing and delivering therapies to patients for the treatment of neurological, rare and autoimmune diseases.	{Healthcare,Biotechnology}	{AMGN,CELG,GILD,SGMO,ACOR,BIO,JNJ,PFE,SNY,ABBV,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:00.221381	2020-10-11 23:38:31.02423
BIV	Vanguard Intermediate-Term Bond	The investment seeks the performance of the Bloomberg Barclays U.S. 5-10 Year Government/Credit Float Adjusted Index.\n Bloomberg Barclays U.S. 5-10 Year Government/Credit Float Adjusted Index includes all medium and larger issues of U.S. government, investment-grade corporate and investment-grade international dollar-denominated bonds that have maturities between 5 and 10 years and are publicly issued. All of its investments will be selected through the sampling process, and at least 80% of its assets will be invested in bonds held in the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.238085	2020-10-11 23:38:31.02423
AYTU	Aytu BioScience Inc.	Aytu BioScience Inc is engaged in acquiring, developing and commercializing products in the field of urology. It focuses on hypogonadism, prostate cancer, urinary tract infections and male infertility.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:11:59.995046	2020-10-11 23:38:31.023428
BA	The Boeing Company	Boeing Co designs, develops, manufactures, sells, services and supports commercial jetliners, military aircraft, satellites, missile defense, human space flight and launch systems and services.	{Industrials,"Aerospace & Defense"}	{LMT,GD,GE,HON,NOC,TXT,UTX,RTN,SPR,ERJ}	Aerospace & Defense	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:00.012279	2020-10-11 23:38:31.021426
BANF	BancFirst Corporation	BancFirst Corp through its subsidiaries provides retail & commercial banking services; It also offers trust services & acts as executor, administrator, trustee, transfer agent & provides item processing, research & other fiduciary services.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.02744	2020-10-11 23:38:31.028166
BAX	Baxter International Inc.	Baxter International Inc operates in the healthcare industry. It provides renal and hospital products such as infusion systems and systems, parenteral nutrition therapies, and biosurgery products.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{ICUI,HALO,CAH,OMI,BDX,BSX,SYK,TMO,ZBH,MDT,XLV}	Medical Instruments & Equipment	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:00.043829	2020-10-11 23:38:31.028166
BBK	Blackrock Municipal Bond Trust	Blackrock Municipal Bond Trust is a diversified, closed-end management investment company. The trust's investment objective is to provide current income exempt from regular federal income tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.056135	2020-10-11 23:38:31.028166
BBY	Best Buy Co. Inc.	Best Buy Co Inc is a U.S transnational consumer electronics company. Its products include tablets and computers, televisions, mobile phones, large and small appliances, entertainment products, digital imaging and related accessories.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{AMZN,AAPL,CONN,SNE,GME,AAN,VOXX}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:00.074517	2020-10-11 23:38:31.028166
BCBP	BCB Bancorp Inc. (NJ)	BCB Bancorp Inc is a bank holding company. The company's primary business is the ownership and operation of BCB Community Bank. It offers FDIC-insured deposit products and invests funds held in deposit accounts at the Bank.	{"Financial Services","Banks - Regional - US",Banks}	{UNTY,EMCF,SSFN,FCCY,PKBK,BKJ,TRCB,VLY}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:00.08622	2020-10-11 23:38:31.02582
BCOR	Blucora Inc.	Blucora Inc provides technology-enabled financial solutions to consumers, small business owners, and tax professionals. It offers products in wealth management and tax preparation services and helps individuals manage their financial lives.	{"Financial Services","Asset Management"}	{MSFT,INTU,TST,BIDU,IAC,HRB,YNDX,GOOGL}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.098071	2020-10-11 23:38:31.02423
BDC	Belden Inc	Belden Inc is a part of the electronics components industry. Its range of solutions includes broadcast solutions, enterprise connectivity solutions, and industrial connectivity solutions.	{Technology,"Electronic Components","Computer Hardware"}	{XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:00.109187	2020-10-11 23:38:31.02423
BECN	Beacon Roofing Supply Inc.	Beacon Roofing Supply Inc is engaged in the distribution of residential and non-residential roofing materials in the United States and Canada.	{"Basic Materials","Building Materials"}	{BLDR,POOL,BCC,HBP,BXC}	Building Materials	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:12:00.125152	2020-10-11 23:38:31.02582
BERY	Berry Global Group Inc.	Berry Global Group Inc is a plastics packaging products manufacturing company. It operates three segments and sells containers, plastic drink cups, printed bags for food products, and baby diapers, feminine hygiene products, tapes, and plastic films.	{"Consumer Cyclical","Packaging & Containers"}	{SLGN,CTIB,ATR,SEE,BMS,TG,MMM,DWDP}	Packaging & Containers	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:00.137901	2020-10-11 23:38:31.02423
BG	Bunge Limited	Bunge Ltd is an agribusiness and food company with integrated operations. It acts as an oilseed processor and seller of packaged vegetable oils. The company also grows and processes Brazilian sugarcane to produce sugar and ethanol.	{"Consumer Defensive","Farm Products","Consumer Packaged Goods"}	{ADM,CZZ,SEB,AGRO,MOS,INGR}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:00.151509	2020-10-11 23:38:31.02423
BGSF	BG Staffing Inc	BG Staffing Inc provides temporary staffing services in the U.S. in Multifamily, Professional, and Commercial. Its temporary staffing services consist of on-demand or short-term staffing assignments, contract staffing, & on-site management administration.	{Industrials,"Staffing & Outsourcing Services","Employment Services"}	{RHI,AMN,TBI}	Employment Services	Industrials	NYSE American	\N	2020-10-11 23:12:00.181678	2020-10-11 23:38:31.023428
BHF	Brighthouse Financial Inc.	Brighthouse Financial Inc is a provider of annuity products and life insurance in the United States.	{"Financial Services","Insurance - Life"}	{}	Insurance - Life	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.201713	2020-10-11 23:38:31.024353
BATRK	Liberty Media Corporation Series C Liberty Braves Common Stock	Liberty Braves Group through its subsidiary indirectly owns the Atlanta Braves Major League Baseball club, the Atlanta Braves' stadium and associated real estate projects.	{"Consumer Cyclical",Leisure,"Travel & Leisure","Media - Diversified",Entertainment}	{}	Entertainment	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:00.043854	2020-10-11 23:38:31.028166
BBVA	Banco Bilbao Vizcaya Argentaria S.A.	Banco Bilbao Vizcaya Argentaria SA is an diversified financial group providing retail banking, wholesale banking, asset management and private banking services. Its activities are located in Spain, Mexico, South America, the United States and Turkey.	{"Financial Services","Banks - Global",Banks}	{}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.074332	2020-10-11 23:38:31.024353
BCH	Banco De Chile ADS	Banco De Chile is a financial services company. Through a network of approximately 399 branch locations it provides banking services to large corporations, small and medium-size businesses, and individuals.	{"Financial Services","Banks - Regional - Latin America",Banks}	{}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.089136	2020-10-11 23:38:31.028166
BCRH	Blue Capital Reinsurance Holdings Ltd.	Blue Capital Reinsurance Holdings Ltd is a reinsurance holding company. The company through its subsidiaries provides collateralized reinsurance in the property catastrophe market and invests in insurance-linked securities.	{"Financial Services","Insurance - Reinsurance","Insurance - Specialty"}	{MHLD,RNR,RE,TPRE}	Insurance - Specialty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.099641	2020-10-11 23:38:31.024353
BDD	DB Base Metals Double Long Exchange Traded Notes due June 1 2038	The investment seeks to track the price and yield performance, before fees and expenses, 200% of the daily return of the Deutsche Bank Liquid Commodity index - Optimum Yield Industrial Metals Excess Return.\n The fund allows investors to take a leveraged view on the performance of the industrial metals sector. The index is a rules-based index composed of futures contracts on some of the most liquid and widely used base metals, aluminum, zinc and copper.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.110091	2020-10-11 23:38:31.024353
BDXA	Becton Dickinson and Company Depositary Shares each Representing a 1/20th Interest in a Share of 6.125% Mandatory Convertible Preferred Stock Series A $1.00 par	Becton, Dickinson and Co is a healthcare company. It manufacture and sales medical supplies, devices, laboratory equipment and diagnostic products. It operates in two segments BD Medical and BD Life Sciences.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{XLV}	Medical Instruments & Equipment	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:00.124188	2020-10-11 23:38:31.031328
BEN	Franklin Resources Inc.	Franklin Resources Inc is an investment management organization which provides investment management and related services to retail, institutional and high net-worth clients under Franklin, Templeton, Franklin Mutual Series, Franklin Bissett brand names.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.135111	2020-10-11 23:38:31.031328
BFK	BlackRock Municipal Income Trust	Blackrock Municipal Income Trust is a closed-end municipal bond fund whose investment objective is to provide current income exempt from regular U.S. federal income tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.145443	2020-10-11 23:38:31.023428
BGFV	Big 5 Sporting Goods Corporation	Big 5 Sporting Goods Corp is a sporting goods retailer. Products include athletic shoes, apparel & accessories, outdoor & athletic equipment for team sports, fitness, camping, hunting, fishing, tennis, golf, winter & summer recreation & roller sports.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{HIBB,FL,DKS}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:00.156317	2020-10-11 23:38:31.023007
BGY	Blackrock Enhanced International Dividend Trust	Blackrock Enhanced International Dividend Trust has an investment objective to provide current income and current gains, with a secondary objective of long-term capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.195444	2020-10-11 23:38:31.025871
BHV	BlackRock Virginia Municipal Bond Trust	Blackrock Virginia Municipal Bond Trust is a perpetual closed-end municipal bond fund. Its objective is to provide current income exempt from regular Federal income taxes and Virginia personal income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.209131	2020-10-11 23:38:31.028166
BIO	Bio-Rad Laboratories Inc. Class A	Bio-Rad Laboratories Inc manufactures and supplies the life science research, healthcare, analytical chemistry and other markets with a products and systems used to separate complex chemical and biological materials.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{HBIO,A,BDX,DHR,GE,JNJ,TMO,PKI,XLV}	Medical Diagnostics & Research	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:00.223767	2020-10-11 23:38:31.028166
BIZD	VanEck Vectors BDC Income	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the MVIS® US Business Development Companies Index.\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The index is comprised of BDCs. BDCs are vehicles whose principal business is to invest in, lend capital to or provide services to privately-held companies or thinly traded U.S. public companies.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.243734	2020-10-11 23:38:31.021426
GPMT	Granite Point Mortgage Trust Inc.	Granite Point Mortgage Trust Inc focuses primarily on directly originating, investing in and managing senior floating-rate commercial mortgage loans and other debt and debt-like commercial real estate investments.	{"Real Estate","REIT - Residential",REITs}	{NYMT,O,NTP}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:03.738826	2020-10-11 23:38:31.023428
BCACW	Bison Capital Acquisition Corp. Warrant		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:00.08622	2020-10-11 23:38:31.023007
BCOM	B Communications Ltd.	B Communications Ltd is a telecommunications provider, providing domestic fixed-line, cellular and international communication services, Internet services, multi-channel television, television and radio broadcasts, satellite broadcasts and NEP services.	{"Communication Services","Telecom Services"}	{IGLD,PTNR,CEL,TU}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.098261	2020-10-11 23:38:31.02582
BDX	Becton Dickinson and Company	Becton, Dickinson and Co is a healthcare company. It manufacture and sales medical supplies, devices, laboratory equipment and diagnostic products. It operates in two segments BD Medical and BD Life Sciences.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{XLV}	Medical Instruments & Equipment	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:00.1231	2020-10-11 23:38:31.025871
BEMO	Aptus Behavioral Momentum	The investment seeks to track the performance, before fees and expenses, of the Aptus Behavioral Momentum Index.\n The index uses an objective, rules-based methodology to implement a systematic trend-following strategy that directs 100% of its exposure to either equity exposure or treasure exposure. The fund invests at least 80% of its total assets in the component securities of the index. The fund generally may invest up to 20% of its total assets (exclusive of any collateral held from securities lending) in securities or other investments not included in the index, but which the Adviser believes will help the fund track the index. It is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:00.134089	2020-10-11 23:38:31.028166
BFOR	Barron's 400	The investment seeks investment results that correspond generally, before fees and expenses, to the performance of the Barron's 400SM Index (the "underlying index").\n The underlying index is a rules-based index intended to give investors a means of tracking the overall performance of high performing equity securities of U.S. companies. The fund will invest at least 80% of its total assets in the equity securities which comprise the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.148291	2020-10-11 23:38:31.028166
BGR	BlackRock Energy and Resources Trust	BlackRock Energy & Resources is a closed-end equity fund providing total return through a combination of current income, gains and capital appreciation by investing in equity securities and derivatives of energy and natural resources companies.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.178172	2020-10-11 23:38:31.02423
BHE	Benchmark Electronics Inc.	Benchmark Electronics Inc is engaged in manufacturing, designing and engineering services and product life cycle solutions. The company offer its services to original equipment manufacturers of industrial equipment, and telecommunication equipment.	{Technology,"Contract Manufacturers","Computer Hardware"}	{CLS,FLEX,SGMA,KBAL,PLXS,SMTX,IEC,JBL,CTS,SANM,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:00.202111	2020-10-11 23:38:31.02423
BIF	Boulder Growth & Income Fund Inc.	Boulder Growth & Income Fund is a closed end management investment company. Its invest ment objective is to seek total return.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.216397	2020-10-11 23:38:31.028166
BIP	Brookfield Infrastructure Partners LP Limited Partnership Units	Brookfield Infrastructure Partners LP owns and operates utilities, transport and energy businesses, and freehold timberlands in North and South America, Australia, Europe, and the Asia-Pacific region.	{Utilities,"Utilities - Diversified","Utilities - Regulated"}	{TAC,KEP,AES,NFG,CPL,POR,VALE,CIG.C,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:00.230189	2020-10-11 23:38:31.023007
BK	Bank of New York Mellon Corporation (The)	Bank of New York Mellon Corporation is an investment company. The company provides investment management, investment services and wealth management that help institutions and individuals succeed in markets all over the world.	{"Financial Services","Asset Management"}	{CM,NTRS,STT,WFC,JPM,C}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.253116	2020-10-11 23:38:31.025871
BKJ	Bancorp of New Jersey Inc	Bancorp of New Jersey Inc is a bank holding company for Bank of New Jersey. It provides traditional commercial banking, accepting deposits from the general public, individuals, businesses, non-profit organizations and governmental units.	{"Financial Services","Savings & Cooperative Banks",Banks}	{UNTY,SSFN,LBAI,PKBK,TRCB,VLY}	Banks	Financial Services	NYSE American	\N	2020-10-11 23:12:00.27294	2020-10-11 23:38:31.023007
BL	BlackLine Inc.	BlackLine Inc provides cloud-based software platform designed to transform and modernize accounting and finance operations for organizations of all types and sizes. Its products include journal entry, account reconciliations, and task management.	{Technology,"Software - Infrastructure","Application Software"}	{PRGS,PXLW,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:00.295703	2020-10-11 23:38:31.031328
BLHY	Virtus Newfleet Dynamic Credit	The investment seeks to provide a high level of current income and, secondarily, capital appreciation.\n Under normal market conditions, the fund will invest not less than 80% of its net assets (plus the amount of any borrowings for investment purposes) in credit investments, and in derivatives and other instruments that have economic characteristics similar to such investments. Credit investments include corporate and sovereign bonds, including U.S. Treasury securities; loans (including, without limitation, through loan participations and assignments); and securitized instruments.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.312737	2020-10-11 23:38:31.021426
BCEI	Bonanza Creek Energy Inc.	Bonanza Creek Energy Inc is an exploration and production company. It is engaged in the extraction of oil and associated liquids-rich natural gas in the United States. Its operations are focused in Rocky Mountain and in the Mid-Continent region.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{APC,NOG,CLR,OAS,PDCE,REN,WLL}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:00.089217	2020-10-11 23:38:31.021426
BCS	Barclays PLC	Barclays PLC is a financial services provider engaged in personal banking, credit cards, corporate and investment banking and wealth and investment management. It operates in Europe, the Americas, Africa and Asia.	{"Financial Services","Banks - Global",Banks}	{}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.100756	2020-10-11 23:38:31.023428
BDGE	Bridge Bancorp Inc.	Bridge Bancorp Inc is a bank holding company for The Bridgehampton National Bank. It provides commercial and consumer banking business, including saving and deposits from the consumers, commercial and real estate loans and home equity loans.	{"Financial Services","Banks - Regional - US",Banks}	{FISI,AROW,FLIC,NBTB,CBU,FNB,EVBN}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.111587	2020-10-11 23:38:31.023007
BEAT	BioTelemetry Inc.	BioTelemetry Inc provides monitoring services and digital population health management for healthcare providers, medical device manufacturing and centralized core laboratory services for clinical research.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{ANGO,NUVA,INFU,XLV}	Medical Diagnostics & Research	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:00.125205	2020-10-11 23:38:31.024353
BFAM	Bright Horizons Family Solutions Inc.	Bright Horizons Family Solutions Inc is a provider of high-quality child care and early education services as well as other services designed to help employers and families address the challenges of work and life.	{"Consumer Cyclical","Personal Services"}	{ADUS,ABCD,CRCM}	Personal Services	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:00.138299	2020-10-11 23:38:31.028166
BGB	Blackstone / GSO Strategic Credit Fund	Blackstone / GSO Strategic Credit Fund is a diversified, closed-end management investment company. Its primary investment objective is to seek high current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.152042	2020-10-11 23:38:31.028166
BGS	B&G Foods Inc.	B&G Foods Inc processes and sells packaged food products in the United States. Through its brands such as Ac'cent, B&G, B&M and Baker's Joy it sells vegetables, canned meats, spices and other packaged food products.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{HAIN,CPB,CAG,GIS,MKC,SJM,THS,POST,KHC}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:00.182844	2020-10-11 23:38:31.028166
BHP	BHP Billiton Limited	BHP Billiton Ltd is a natural resources company. It is engaged in producing commodities, including iron ore, metallurgical and energy coal, conventional and unconventional oil and gas, copper, aluminium, manganese, uranium, nickel and silver.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{TECK,RIO,XOM,BP,CVX,RDS.A,VALE}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:00.201815	2020-10-11 23:38:31.02582
BIG	Big Lots Inc.	Big Lots Inc is engaged in operating discount retail stores. It provides a broad range of merchandise, including food, consumables, soft home products, hard home products, furniture, electronics and accessories, and seasonal products.	{"Consumer Defensive","Discount Stores","Retail - Defensive"}	{COST,FRED,DLTR,TGT,DG,FIVE}	Retail - Defensive	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:00.216374	2020-10-11 23:38:31.023428
BKD	Brookdale Senior Living Inc.	Brookdale Senior Living Inc is a leading operator of senior living communities throughout the United States. The company along with its ancillary services program offers a range of outpatient therapy, home health, personalized living and hospice services.	{Healthcare,"Long-Term Care Facilities","Health Care Providers"}	{BIOS,CSU,FVE,RHE,XLV}	Health Care Providers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:00.255706	2020-10-11 23:38:31.028166
BKN	BlackRock Investment Quality Municipal Trust Inc. (The)	Blackrock Investment Quality Muni Tr is a diversified investment fund with the investment objective to provide high current income exempt from regular U.S. federal income tax consistent with the preservation of capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.277768	2020-10-11 23:38:31.025871
BLDP	Ballard Power Systems Inc.	Ballard Power Systems Inc is engaged in the design, development, manufacture, sale and service of proton exchange membrane fuel cell products for a variety of applications, focusing on commercial stage and development stage markets.	{Industrials,"Diversified Industrials","Industrial Products"}	{HYGS,CSIQ,PLUG,ESNC,CPST,FCEL}	Industrial Products	Industrials	NASDAQ Global Market	\N	2020-10-11 23:12:00.300066	2020-10-11 23:38:31.025871
BLMN	Bloomin' Brands Inc.	Bloomin Brands Inc operates restaurants under the brand name Outback Steakhouse, Carrabba's Italian Grill, Bonefish Grill, and Fleming's Prime Steakhouse and Wine Bar. The company has the presence in more than 18 countries.	{"Consumer Cyclical",Restaurants}	{ARKR,BJRI,RUTH,CBRL,EAT,BDL,DFRG}	Restaurants	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:00.318356	2020-10-11 23:38:31.021426
BMRN	BioMarin Pharmaceutical Inc.	Biomarin Pharmaceutical Inc is on rare-disease therapies. It develops and commercializes pharmaceuticals for serious diseases and medical conditions.	{Healthcare,Biotechnology}	{ALXN,SHPG,SGMO,FOLD,CPRX,SNY,ESPR,RARE,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:00.357861	2020-10-11 23:38:31.021426
BERN	Bernstein U.S. Research Fund	The investment seeks to provide investment results that, before fees and expenses, track the performance of the Bernstein U.S. Research Index (the "index").\n The fund will normally invest at least 80% of its total assets in component securities of the index. The index is designed to measure the performance of large-cap U.S. stocks rated "Outperform" by sell-side analysts of Sanford C. Bernstein & Co., LLC ("Bernstein" or the "index provider") and ranked within one of the top three quintiles of Bernstein's published quantitative alpha model.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:00.137128	2020-10-11 23:38:31.025871
BFZ	BlackRock California Municipal Income Trust	Blackrock CA Muni Income Tr is a United States based closed-end management investment company. Its investment objective is to provide current income exempt from regular U.S. federal income tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.151601	2020-10-11 23:38:31.021426
BGNE	BeiGene Ltd.	BeiGene Ltd is a biopharmaceutical company engaged in the discovery and development of molecularly targeted and immuno-oncology drugs for the treatment of cancer. Its product includes BGB-3111, BGB-A317 and BGB-290 and one immuno-oncology agent.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:00.175487	2020-10-11 23:38:31.025871
BHB	Bar Harbor Bankshares Inc.	Bar Harbor Bankshares Inc is a bank holding company. The company and its subsidiaries offer full range of financial services products, brokerage services through third party brokerage arrangement and also trust and investment management services.	{"Financial Services","Banks - Regional - US",Banks}	{CAC,EBTC,UNB,BPFH,CNBKA,INDB,WASH,FNLC,NBN}	Banks	Financial Services	NYSE American	\N	2020-10-11 23:12:00.20137	2020-10-11 23:38:31.031328
BIDU	Baidu Inc.	Baidu Inc is one of the world's leading Chinese-language Internet search engine. The company through its website primarily serves over 0.8 million advertisers which include small and medium-size businesses.	{Technology,"Internet Content & Information","Online Media"}	{SINA,FENG,YNDX,BCOR,GOOGL,RENN,XLK}	Online Media	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:00.215492	2020-10-11 23:38:31.024353
BIT	BlackRock Multi-Sector Income Trust of Beneficial Interest	BlackRock Multi-Sector Income Trust is a closed-end investment management company. The fund's primary investment objective is to provide high level of current income, with the secondary objective is to seek the preservation of capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.231779	2020-10-11 23:38:31.023428
BKEPP	Blueknight Energy Partners L.P. L.L.C. Series A Preferred Units	Blueknight Energy Partners LP provides terminalling, gathering, transportation, and storage services for companies that produce, distribute, and market crude oil and liquid asphalt cement.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{}	Oil & Gas - Midstream	Energy	NASDAQ Global Market	\N	2020-10-11 23:12:00.268315	2020-10-11 23:38:31.024353
BKU	BankUnited Inc.	BankUnited Inc functions in the financial sector of the United States. As a commercial banking service provider, it caters to corporate and retail clients through lending and deposit products.	{"Financial Services","Banks - Regional - US",Banks}	{CCBG,CSFL,COF,BBT,STI,SBCF,OPHC}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.28952	2020-10-11 23:38:31.02423
BLOK	Amplify Transformational Data Sharing	The investment seeks to provide investors with total return.\n The fund is an actively managed ETF that seeks to provide total return by investing at least 80% of its net assets (including investment borrowings) in the equity securities of companies actively involved in the development and utilization of transformational data sharing technologies. It may invest in non-U.S. equity securities, including depositary receipts. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.328175	2020-10-11 23:38:31.028166
BME	Blackrock Health Sciences Trust	Blackrock Health Sciences Trust has an investment objective to provide total return through a combination of current income, current gains and long-term capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.351522	2020-10-11 23:38:31.023428
BMY	Bristol-Myers Squibb Company	Bristol-Myers Squibb Co is engaged in discovery, development, licensing, manufacturing, marketing, distribution and sale of biopharmaceuticals products. The company operates in one segment that is Biopharmaceuticals.	{Healthcare,"Drug Manufacturers - Major","Drug Manufacturers"}	{AMGN,CELG,AZN,NVS,JNJ,LLY,PFE,SNY,MRK,ABBV,XLV}	Drug Manufacturers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:00.364834	2020-10-11 23:38:31.028166
BNY	BlackRock New York Municipal Income Trust	Blackrock NY Municipal Income Tr has an investment objective to provide current income exempt from regular U.S. federal income tax and New York State and New York City personal income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.381015	2020-10-11 23:38:31.023428
BOM	DB Base Metals Double Short Exchange Traded Notes due June 1 2038	The investment seeks to track the price and yield performance, before fees and expenses, 200% of the inverse daily performance of the Deutsche Bank Liquid Commodity index - Optimum Yield Industrial Metals Excess Return.\n The fund allows investors to take a short view on the performance of the industrial metals sector. The index is a rules-based index composed of futures contracts on some of the most liquid and widely used base metals, aluminum, zinc and copper.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.394523	2020-10-11 23:38:31.028166
BGH	Barings Global Short Duration High Yield Fund of Beneficial Interests	Barings Global Short Duration High Yield Fund is a diversified, closed-end management investment company. Its investment objective is to seek as high a level of current income as the Adviser determines is consistent with capital preservation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.167738	2020-10-11 23:38:31.021426
BGX	Blackstone GSO Long Short Credit Income Fund	Blackstone/GSO Dynamic Credit Income is a diversified, closed-end management investment company.The Fund's primary investment objective is to seek high current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.196356	2020-10-11 23:38:31.028166
BICK	First Trust Global BICK Index Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the ISE BICKTM.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks or in depositary receipts representing securities in the index. The index is designed to provide a benchmark for investors interested in tracking some of the largest and most liquid public companies that are domiciled in Brazil, India, China (including Hong Kong) and South Korea that are accessible for investment by U.S. investors.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:00.21422	2020-10-11 23:38:31.025871
BIS	ProShares UltraShort Nasdaq Biotechnology	The investment seeks daily investment results that correspond to two times the inverse (-2x) of the daily performance of the Nasdaq Biotechnology Index®.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index is a modified capitalization weighted index that includes securities of Nasdaq listed companies that are classified as either biotechnology or pharmaceutical. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:00.23022	2020-10-11 23:38:31.028166
BKE	Buckle Inc. (The)	Buckle, Inc. is a retailer of casual apparel, footwear, and accessories for fashion conscious young men and women. It operates 468 stores located in 44 states throughout the continental United States.	{"Consumer Cyclical","Apparel Stores","Retail - Apparel & Specialty"}	{URBN,ZUMZ,ANF,AEO,GPS,JWN,TLYS}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:00.256659	2020-10-11 23:38:31.021426
BKS	Barnes & Noble Inc.	Barnes & Noble Inc is a bookseller, content, and a commerce company providing customers access to trade books, textbooks, magazines, newspapers and other content across its multi-channel distribution platform. Its segments are B&N Retail and NOOK.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{AMZN,NWSA,CHGG}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:00.280501	2020-10-11 23:38:31.023428
BLDR	Builders FirstSource Inc.	Builders FirstSource Inc is a supplier and manufacturer of structural and related building products for residential new construction in the United States.	{"Basic Materials","Building Materials"}	{HD,LOW,NX,LL,BCC,DOOR,BMCH,BXC}	Building Materials	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:12:00.300704	2020-10-11 23:38:31.028166
BLKB	Blackbaud Inc.	Blackbaud Inc provides cloud software, services, expertise, and data intelligence. It provides its services to nonprofits, foundations, corporations, education institutions, healthcare institutions, and the individual change agents who support them.	{Technology,"Software - Application","Application Software"}	{MSFT,CTXS,CRM,QADA,NTWK,PTC,ORCL,VEEV,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:00.318403	2020-10-11 23:38:31.031328
BMA	Banco Macro S.A. ADR (representing Ten Class B)	Banco Macro SA is a commercial bank standard banking products and services. It provides its service to retail customer and small to medium-size businesses.	{"Financial Services","Banks - Regional - Latin America",Banks}	{GGAL,SAN,BFR,BSAC,C}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.346644	2020-10-11 23:38:31.028166
BMTC	Bryn Mawr Bank Corporation	Bryn Mawr Bank Corp, through Bryn Mawr Trust Company, offers personal and business banking services, consumer and commercial loans, equipment leasing, mortgages, insurance and wealth management services.	{"Financial Services","Banks - Regional - US",Banks}	{CZNC,NWFL,CCNE,FULT,PWOD,STBA,UVSP,ASRV,FNB,TSC}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.359983	2020-10-11 23:38:31.023428
BNED	Barnes & Noble Education Inc	Barnes & Noble Education, Inc. operates bookstores for college and university campuses in the United States. It also provides digital education services for enhancing the academic and social purpose of educational institutions.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{BNED}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:00.374748	2020-10-11 23:38:31.02582
BOH	Bank of Hawaii Corporation	Bank of Hawaii Corp is a bank holding company. It provides a range of financial services and products primarily to customers in Hawaii, Guam and other Pacific Islands.	{"Financial Services","Banks - Regional - US",Banks}	{CATY,CVBF,HWC,HBAN,EWBC,BMRC,MTB,CFR,CPF}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.386946	2020-10-11 23:38:31.023007
FFC	Flaherty & Crumrine Preferred Securities Income Fund Incorporated	Flaherty & Crumrine Preferred Securities Income Fund Inc is a diversified closed-end management investment company. The Fund's investment objective is to provide its common shareholders with high current income consistent with the preservation of capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.839896	2020-10-11 23:38:31.028166
BHVN	Biohaven Pharmaceutical Holding Company Ltd.	Biohaven Pharmaceutical Holding Co Ltd is a clinical-stage biopharmaceutical company with a portfolio of innovative, late-stage product candidates targeting neurological diseases, including rare disorders.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:00.211838	2020-10-11 23:38:31.02423
BIOC	Biocept Inc.	Biocept Inc is an early commercial-stage molecular oncology diagnostics company. It develops and commercializes proprietary circulating tumor cell and circulating tumor DNA assays utilizing a standard blood sample.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{XLV}	Medical Diagnostics & Research	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:00.223887	2020-10-11 23:38:31.021426
BJRI	BJ's Restaurants Inc.	BJ's Restaurants Inc is involved in the business of owning and operating restaurants.	{"Consumer Cyclical",Restaurants}	{DAVE,CAKE,RRGB,KONA,EAT,CMG,CHUY,BLMN}	Restaurants	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:00.243434	2020-10-11 23:38:31.028166
BKI	Black Knight Inc.	Black Knight Inc is a provider of software, data and analytics solutions to the mortgage and consumer loan, real estate and capital market verticals.The company's solutions include Software Solutions and Data and Analytics Solutions.	{Technology,"Software - Infrastructure","Application Software"}	{SSNC,NVEE,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:00.271775	2020-10-11 23:38:31.031328
BLCM	Bellicum Pharmaceuticals Inc.	Bellicum Pharmaceuticals Inc is a clinical-stage biopharmaceutical company. The firm is primarily engaged in the research of cellular immunotherapies for hematological cancers, solid tumors, and orphan inherited blood diseases.	{Healthcare,Biotechnology}	{ALDX,CORI,TTOO,ALQA,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:00.296934	2020-10-11 23:38:31.028166
BLIN	Bridgeline Digital Inc.	Bridgeline Digital Inc is a developer of web application management software and web applications to maximize the performance of critical websites, intranets, and online stores.	{Technology,"Software - Application","Application Software"}	{XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:00.314529	2020-10-11 23:38:31.02582
BLRX	BioLineRx Ltd.	BioLine Rx Ltd is a clinical stage biopharmaceutical development company engaged in identifying, in-licensing & developing therapeutic compounds. Its pipeline product include BL-8040, BL-9020, BL-5010, BL-1230, and AGI-134.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:00.330632	2020-10-11 23:38:31.02423
BMLP	Dorsey Wright MLP Index ETNs due December 10 2036	The fund seeks to link to the performance of the DWA MLP Select™ Index (the “index”). \n The index includes 15 master limited partnerships (“MLPs”) based on the proprietary Dorsey Wright Relative Strength Ranking Methodology. By comparing the price relationship between each MLP, the index seeks to determine which MLPs are currently showing outperformance relative to their peers within the Index universe.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:00.352187	2020-10-11 23:38:31.02423
BND	Vanguard Total Bond Market ETF	The investment seeks the performance of Bloomberg Barclays U.S. Aggregate Float Adjusted Index.\n Bloomberg Barclays U.S. Aggregate Float Adjusted Index represents a wide spectrum of public, investment-grade, taxable, fixed income securities in the United States-including government, corporate, and international dollar-denominated bonds, as well as mortgage-backed and asset-backed securities-all with maturities of more than 1 year. All of its investments will be selected through the sampling process, and at least 80% of its assets will be invested in bonds held in the index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:00.366454	2020-10-11 23:38:31.023007
BNTC	Benitec Biopharma Limited	Benitec Biopharma Ltd is active in the healthcare domain. It has a pipeline of in-house and partnered therapeutic programs based on its patented gene-silencing technology.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{XLV}	Drug Manufacturers	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:00.379246	2020-10-11 23:38:31.028166
BOKFL	BOK Financial Corporation 5.375% Subordinated Notes due 2056		{}	{}			Nasdaq Global Select	\N	2020-10-11 23:12:00.392366	2020-10-11 23:38:31.024353
BORN	China New Borun Corporation American Depositary Shares	China New Borun Corp is a producer and distributor of corn-based edible alcohol in the People's Republic of China. Its edible alcohol products are sold as an ingredient to producers of baijiu, a grain-based alcoholic beverage.	{"Consumer Defensive","Beverages - Wineries & Distilleries","Beverages - Alcoholic"}	{DEO,STKL,ADM,BG,MGPI,INGR}	Beverages - Alcoholic	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:00.406555	2020-10-11 23:38:31.02423
BPFH	Boston Private Financial Holdings Inc.	Boston Private Financial Holdings Inc is a wealth management company. It offers various wealth management services to high net worth individuals, families, businesses and select institution.	{"Financial Services","Banks - Regional - US",Banks}	{EBTC,UNB,CNBKA,INDB,WASH,BRKL,WBS,BHB,NBN,SAL}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.420321	2020-10-11 23:38:31.028166
BPRN	Bank Of Princeton (The)	The Bank of Princeton is a full service bank. The company provides personal and business lending and deposit services. Its personal banking and business banking services include checking accounts and savings accounts.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.431806	2020-10-11 23:38:31.024353
BRACU	Black Ridge Acquisition Corp. Unit	Black Ridge Acquisition Corp is a blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:00.444477	2020-10-11 23:38:31.024353
GRA	W.R. Grace & Co.	W R Grace & Co is engaged in the production and sale of specialty chemicals and specialty materials. It operates through two segments, namely Catalysts Technologies and Materials Technologies.	{"Basic Materials","Specialty Chemicals",Chemicals}	{IOSP,ALB,FOE,RPM,BAK,HUN,CE,DWDP}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:03.769666	2020-10-11 23:38:31.02582
BID	Sotheby's	Sothebys is a global art business company. It conducts auctions and private sales of fine art, antiques and decorative art, jewelry, and collectibles.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{RBA,CLCT,LQDT,MOV,JWN,TIF,KAR}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:00.214592	2020-10-11 23:38:31.031328
BIOL	Biolase Inc.	Biolase Inc is a medical device company that develops, manufactures, and markets lasers in dentistry and medicine and also markets and distributes dental imaging equipment.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{XLV}	Medical Instruments & Equipment	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:00.227848	2020-10-11 23:38:31.025871
BJZ	Blackrock California Municipal 2018 Term Trust	Blackrock CA Muni 2018 Term Trust is a United States based closed-end management investment company. Its investment objective is to provide current income exempt from regular U.S. federal and California income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.25381	2020-10-11 23:38:31.023007
BKK	Blackrock Municipal 2020 Term Trust	Blackrock Municipal 2020 Term Trust is a diversified, closed-end management investment company. The Fund's investment objective is to provide total return through a combination of current income and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.276974	2020-10-11 23:38:31.028166
BLFS	BioLife Solutions Inc.	BioLife Solutions Inc manufactures proprietary and generic cGMP biopreservation media products, generic formulations of blood stem cell freezing media products, custom product formulation packaging services and contract aseptic manufacturing formulation.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{HEB,EVGN,XLV}	Medical Instruments & Equipment	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:00.312896	2020-10-11 23:38:31.031328
BLPH	Bellerophon Therapeutics Inc.	Bellerophon Therapeutics Inc is a clinical-stage biotherapeutics company. It develops products that are focused on the treatment of various types of pulmonary hypertension disorder.	{Healthcare,Biotechnology}	{BAX,LNTH,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:00.32957	2020-10-11 23:38:31.023428
BMRC	Bank of Marin Bancorp	Bank of Marin Bancorp is a bank holding company. It conducts business primarily through its wholly-owned subsidiary. The Bank, provides a wide range of financial services to customers predominantly professionals, small and middle-market businesses.	{"Financial Services","Banks - Regional - US",Banks}	{UMPQ,HTBK,WABC,SIVB,WAL,GBNK}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:00.35636	2020-10-11 23:38:31.02582
BNS	Bank Nova Scotia Halifax Pfd 3	Bank of Nova Scotia is an international bank and a financial service provider. The company provides personal and commercial banking, wealth management and private banking, corporate and investment banking, and capital markets services.	{"Financial Services","Banks - Global",Banks}	{BMO,CM,RY,TD,HSBC,CIB}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.374815	2020-10-11 23:38:31.024353
BOSC	B.O.S. Better Online Solutions	BOS Better Online Solutions Ltd provides radio frequency identification (RFID), mobile, and supply chain solutions in Israel and internationally. It operates through two divisions, Supply Chain Solutions; and (RFID) and Mobile Solutions.	{Industrials,"Aerospace & Defense"}	{SCSC,ESLT,ZBRA,GE,HON,TEVA}	Aerospace & Defense	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:00.406835	2020-10-11 23:38:31.025871
BPL	Buckeye Partners L.P.	Buckeye Partners LP is a US-based company which owns and operates a diversified network of integrated assets providing midstream logistic solutions, consisting of the transportation, storage, and marketing of liquid petroleum products.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{HEP,DCP,MPC,PBA,MPLX,GLOP,ANDX}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:00.42235	2020-10-11 23:38:31.028166
BRACR	Black Ridge Acquisition Corp. Right		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:00.446143	2020-10-11 23:38:31.02423
BRPA	Big Rock Partners Acquisition Corp.	Big Rock Partners Acquisition Corp is a United States based blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:00.46295	2020-10-11 23:38:31.025871
BSCL	Invesco BulletShares 2021 Corporate Bond	The investment seeks investment results that correspond generally to the performance, before the fund's fees and expenses, of an investment grade corporate bond index called the Nasdaq BulletShares® USD Corporate Bond 2021 Index.\n The fund will invest at least 80% of its total assets in component securities that comprise the index. The index is designed to represent the performance of a held-to-maturity portfolio of U.S. dollar-denominated investment-grade corporate bonds with effective maturities in the year 2021. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.480951	2020-10-11 23:38:31.021426
OBLN	Obalon Therapeutics Inc.	Obalon Therapeutics Inc is a commercial-stage medical device company focused on developing and commercializing medical devices to treat obese and overweight people by facilitating weight loss.	{Healthcare,"Medical Devices"}	{NUVA,XLV}	Medical Devices	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:08.248353	2020-10-11 23:38:31.021426
BJK	VanEck Vectors Gaming	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the MVIS® Global Gaming Index.\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The index includes common stocks and depositary receipts of companies that generate at least 50% of their revenues from (or, in certain circumstances, have at least 50% of their assets related to) gaming. This includes casinos and casino hotels, sports betting and lottery services as well as gaming services, gaming technology and gaming equipment. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.243878	2020-10-11 23:38:31.024353
BKF	iShares MSCI BRIC Index Fund	The investment seeks to track the investment results of the MSCI BRIC Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is a free float-adjusted market capitalization index that is designed to measure the combined equity market performance in Brazil, Russia, India and China ("BRIC").	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.270707	2020-10-11 23:38:31.02582
BKT	BlackRock Income Trust Inc. (The)	Blackrock Income Trust Inc is a diversified investment fund with an investment objective to manage a portfolio of high-quality securities to achieve both preservation of capital and high monthly income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.289225	2020-10-11 23:38:31.024353
BLH	Blackrock New York Municipal 2018 Term Trust	Blackrock NY Municipal 2018 Term Tr operates as a closed end management investment company. Its investment objective is to provide current income exempt from regular U.S. federal income tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.313371	2020-10-11 23:38:31.028166
BLV	Vanguard Long-Term Bond	The investment seeks to track the performance of the Bloomberg Barclays U.S. Long Government/Credit Float Adjusted Index.\n Bloomberg Barclays U.S. Long Government/Credit Float Adjusted Index includes all medium and larger issues of U.S. government, investment-grade corporate, and investment-grade international dollar-denominated bonds that have maturities of greater than 10 years and are publicly issued. All of its investments will be selected through the sampling process, and at least 80% of the fund's assets will be invested in bonds held in the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.334881	2020-10-11 23:38:31.02582
BMI	Badger Meter Inc.	Badger Meter Inc is a manufacturer and marketer of products incorporating liquid flow measurement and control technologies. It serves water utilities, municipalities and commercial and industrial customers.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{ITRI,GTLS,EMR,IEX,ROP,MWA,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:00.353347	2020-10-11 23:38:31.028166
BNDC	FlexShares Core Select Bond Fund	The investment seeks total return and preservation of capital.\n The fund invests at least 80% of its net assets in U.S. dollar-denominated investment-grade fixed-income securities either directly or indirectly through exchange-traded funds and other registered investment companies. The fund may invest, without limitation, in mortgage- or asset-backed securities, including to-be-announced transactions, and purchase and sell securities on a when-issued, delayed delivery or forward commitment basis.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.368896	2020-10-11 23:38:31.025871
BOE	Blackrock Enhanced Global Dividend Trust of Beneficial Interest	Blackrock Enhanced Global Dividend Trust has an investment objective to provide current income and current gains, with a secondary investment objective of long-term capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.384352	2020-10-11 23:38:31.028166
BOOM	DMC Global Inc.	DMC Global Inc operates a diversified family of technical product and process businesses serving the energy, industrial and infrastructure markets.	{Industrials,"Metal Fabrication","Industrial Products"}	{STLD,HOLX,AKS,SLB,B,CRS,X,WFT,BHGE}	Industrial Products	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:00.403555	2020-10-11 23:38:31.028166
BPK	Blackrock Municipal 2018 Term Trust	Blackrock Municipal 2018 Term Trust is a diversified, closed-end management investment company. The Fund's investment objective is to provide total return through a combination of current income and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.422101	2020-10-11 23:38:31.031328
BPT	BP Prudhoe Bay Royalty Trust	BP Prudhoe Bay Royalty Trust was formed for the purpose of owning and administering the royalty interest.	{Energy,"Oil & Gas Refining & Marketing","Oil & Gas - Refining & Marketing"}	{MARPS,DMLP,PHX,PBT,SJT}	Oil & Gas - Refining & Marketing	Energy	New York Stock Exchange	\N	2020-10-11 23:12:00.433769	2020-10-11 23:38:31.021426
BRFS	BRF S.A.	BRF SA produces fresh and frozen protein foods that include poultry, pork, bovine, processed foods, and other. The company also offers margarine, mayonnaise, mustard, ketchup, and frozen prepared entrees, such as lasagna and pizzas.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{BRID,SAFM,UL,HRL,TSN,BG,SEB}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:00.447392	2020-10-11 23:38:31.021426
BRO	Brown & Brown Inc.	Brown & Brown Inc is an insurance agent and broker. It offers insurance products and services, primarily in the areas of property, casualty, and employee benefits, by connecting customers with insurance companies.	{"Financial Services","Insurance Brokers","Brokers & Exchanges"}	{AJG,MMC,AON,WLTW}	Brokers & Exchanges	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.461226	2020-10-11 23:38:31.02582
BKCC	BlackRock Capital Investment Corporation	BlackRock Capital Investment Corporation is an investment management enterprise. Its investment objective includes generating income and capital appreciation by investing in debt and equity instruments.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.253508	2020-10-11 23:38:31.031328
BLCN	Reality Shares Nasdaq NextGen Economy ETF	The investment seeks long-term growth by tracking the investment returns, before fees and expenses, of the Reality Shares Nasdaq Blockchain Economy Index (the "index").\n Under normal circumstances, at least 80% of the fund's assets, other than collateral held from securities lending, if any, will be invested in component securities of the index. The index is designed to measure the returns of companies that are committing material resources to developing, researching, supporting, innovating or utilizing blockchain technology for their proprietary use or for use by others ("Blockchain Companies"). The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:00.296478	2020-10-11 23:38:31.02582
BLK	BlackRock Inc.	BlackRock Inc provides investment management services to institutional clients and to individual investors. Its products include single- and multi-asset portfolios investing in equities, fixed income, alternatives and money market instruments.	{"Financial Services","Asset Management"}	{BAC,AMG,LM,AB,IVZ,WETF}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.314953	2020-10-11 23:38:31.023007
BLW	Blackrock Limited Duration Income Trust	Blackrock Ltd Duration Income Trust is a diversified investment fund with an investment objective to provide current income and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.335651	2020-10-11 23:38:31.031328
BMRA	Biomerica Inc.	Biomerica Inc develops, manufactures, and markets medical diagnostic products designed for the early detection and monitoring of chronic diseases and medical conditions.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{ALGN,PDEX,MLSS,XRAY,XLV}	Medical Instruments & Equipment	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:00.354256	2020-10-11 23:38:31.031328
BNDX	Vanguard Total International Bond ETF	The investment seeks to track the performance of a benchmark index that measures the investment return of non-U.S. dollar-denominated investment-grade bonds.\n The fund employs an indexing investment approach designed to track the performance of the Bloomberg Barclays Global Aggregate ex-USD Float Adjusted RIC Capped Index (USD Hedged). This index provides a broad-based measure of the global, investment-grade, fixed-rate debt markets. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:00.373253	2020-10-11 23:38:31.031328
BOMN	Boston Omaha Corporation	Boston Omaha Corp is a public company with businesses engaged in several sectors, including advertising, insurance and real estate.	{Industrials,Conglomerates}	{INUV,QUOT}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:00.397542	2020-10-11 23:38:31.023428
BOX	Box Inc. Class A	Box Inc provides cloud content management platform that enables organizations to securely manage cloud content while allowing easy, secure access and sharing of content from anywhere, on any device.	{Technology,"Software - Infrastructure","Application Software"}	{JKHY,TDC,INXN,CDW,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:00.41143	2020-10-11 23:38:31.024353
BPOPM	Popular Inc. Popular Capital Trust II	Popular Capital Trust II is a statutory trust. It is engaged in issuing capital securities.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.427187	2020-10-11 23:38:31.02582
BRF	VanEck Vectors Brazil Small-Cap	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the MVIS® Brazil Small-Cap Index.\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The index includes securities of Brazilian small-capitalization companies. A company is generally considered to be a Brazilian company if it is incorporated in Brazil or is incorporated outside of Brazil but has at least 50% of its revenues/related assets in Brazil.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.446937	2020-10-11 23:38:31.025871
BRPAR	Big Rock Partners Acquisition Corp. Right		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:00.462303	2020-10-11 23:38:31.023428
BSCM	Invesco BulletShares 2022 Corporate Bond	The investment seeks investment results that correspond generally to the performance, before the fund's fees and expenses, of an investment grade corporate bond index called the Nasdaq BulletShares® USD Corporate Bond 2022 Index.\n The fund will invest at least 80% of its total assets in component securities that comprise the index. The index is designed to represent the performance of a held-to-maturity portfolio of U.S. dollar-denominated investment-grade corporate bonds with effective maturities in the year 2022. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.480312	2020-10-11 23:38:31.024353
BSJK	Invesco BulletShares 2020 High Yield Corporate Bond	The investment seeks investment results that correspond generally to the performance, before the fund's fees and expenses, of a high yield corporate bond index called the Nasdaq BulletShares® USD High Yield Corporate Bond 2020 Index.\n The fund will invest at least 80% of its total assets in component securities that comprise the index. The index is designed to represent the performance of a held-to-maturity portfolio of U.S. dollar-denominated high yield corporate bonds with effective maturities in the year 2020. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.492104	2020-10-11 23:38:31.02582
OBSV	ObsEva SA	ObsEva SA is a clinical stage biopharmaceutical company. It is engaged in the development and commercialization of novel therapeutics for serious conditions that comprise a woman's reproductive health and pregnancy.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:08.259305	2020-10-11 23:38:31.02582
BKSC	Bank of South Carolina Corp.	Bank of South Carolina Corp, through its subsidiary, operates as an independent, community oriented, commercial bank providing a range of financial services and products.	{"Financial Services","Banks - Regional - US",Banks}	{FCCO,FBNC,ABCB,SFST}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:00.280397	2020-10-11 23:38:31.028166
BLD	TopBuild Corp.	TopBuild Corp is an installer and distributor of insulation products to the United States construction industry.	{"Basic Materials","Building Materials"}	{EME,PWR,IESC}	Building Materials	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:00.299907	2020-10-11 23:38:31.023428
BLL	Ball Corporation	Ball Corp is the world's largest metal can manufacturer. The company primarily serves the North American food can market with its packaging products and the US defense industry by offering various components used in the space mission.	{"Consumer Cyclical","Packaging & Containers"}	{SLGN,SEE,BA,NOC,PKG,CCK,TAP,WRK}	Packaging & Containers	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:00.318286	2020-10-11 23:38:31.025871
BMCH	BMC Stock Holdings Inc.	BMC Stock Holdings Inc is a diversified lumber and building materials distributor and solutions provider. Its primary products are lumber and lumber sheet goods, millwork, doors, flooring, windows, structural components, among others.	{"Basic Materials","Building Materials"}	{JCTCF,BLDR,LPX,LL,BCC,BXC}	Building Materials	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:12:00.351651	2020-10-11 23:38:31.025871
BNCL	Beneficial Bancorp Inc.	Beneficial Bancorp Inc offers consumer and commercial banking services to individuals, businesses and nonprofit organizations throughout the Philadelphia and Southern New Jersey area.	{"Financial Services","Banks - Regional - US",Banks}	{WVFC,ESSA,PBIP}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.36493	2020-10-11 23:38:31.028166
BOCH	Bank of Commerce Holdings (CA)	Bank of Commerce Holdings Inc is a bank holding company operating for Redding Bank of Commerce. It provides a variety of financial services and products for business and consumer customers which include checking, savings and loans and advances.	{"Financial Services","Banks - Regional - US",Banks}	{TCBK,AMRB,CATY,CVBF,HTBK,EWBC,CVCY,BSRR,PPBI,SIVB}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:00.385432	2020-10-11 23:38:31.02423
BOND	PIMCO Active Bond Exchange-Traded Fund Exchange-Traded Fund	The investment seeks current income and long-term capital appreciation, consistent with prudent investment management.\n The fund normally invests at least 80% of its assets in a diversified portfolio of Fixed Income Instruments of varying maturities, which may be represented by forwards or derivatives such as options, futures contracts, or swap agreement. It invests primarily in investment grade debt securities, but may invest up to 30% of its total assets in high yield securities, as rated by Moody's, S&P or Fitch, or, if unrated, as determined by PIMCO.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.400416	2020-10-11 23:38:31.02582
BOXL	Boxlight Corporation	Boxlight Corp designs, produces and distributes interactive projectors and 70" HD and 84" 4k interactive LED flat panels.	{Technology,"Communication Equipment"}	{LRN,HMHC,GHC,XLK}	Communication Equipment	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:00.413613	2020-10-11 23:38:31.02582
BPOP	Popular Inc.	Popular Inc is a publicly owned bank holding company, which offers retail and commercial banking services through its banking subsidiary, Banco Popular de Puerto Rico. The Bank also offers insurance and reinsurance services.	{"Financial Services","Banks - Regional - US",Banks}	{FCBC,FITB,UVSP,MTB,STI,FBP,OFG}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.425502	2020-10-11 23:38:31.025871
BRACW	Black Ridge Acquisition Corp. Warrant		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:00.446006	2020-10-11 23:38:31.023007
BRN	Barnwell Industries Inc.	Barnwell Industries Inc is engaged in the acquisition and development of oil and natural gas properties. It business segments include oil and natural gas, land investment and contract drilling.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{CRZO,CHK,NFX,DNR,TELL}	Oil & Gas - E&P	Energy	NYSE American	\N	2020-10-11 23:12:00.46031	2020-10-11 23:38:31.02423
BSCQ	Invesco BulletShares 2026 Corporate Bond	The investment seeks investment results that correspond generally to the performance, before the fund's fees and expenses, of an investment grade corporate bond index called the Nasdaq BulletShares® USD Corporate Bond 2026 Index.\n The fund will invest at least 80% of its total assets in component securities that comprise the 2026 Index. The 2026 Index is designed to represent the performance of a held-to-maturity portfolio of U.S. dollar-denominated investment-grade corporate bonds with effective maturities in the year 2026. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.485986	2020-10-11 23:38:31.025871
BSJO	Invesco BulletShares 2024 High Yield Corporate Bond	The investment seeks investment results that correspond generally to the performance, before the fund's fees and expenses, of a high yield corporate bond index called the Nasdaq BulletShares® USD High Yield Corporate Bond 2024 Index.\n The fund will invest at least 80% of its total assets in component securities that comprise the underlying index. The underlying index is designed to represent the performance of a held-to-maturity portfolio of U.S. dollar-denominated high yield corporate bonds with effective maturities in the year 2024. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.496789	2020-10-11 23:38:31.025871
BKYI	BIO-key International Inc.	BIO-key International Inc a fingerprint biometric technology company. It develops and markets advanced fingerprint biometric identification and identity verification technologies. Its products are SideSwipe, SideTouch, EcoID, Web-key, and ID director.	{Industrials,"Security & Protection Services","Consulting & Outsourcing"}	{NSIT,BSQR,WSTG,ATVI,SNX}	Consulting & Outsourcing	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:00.289312	2020-10-11 23:38:31.023007
BLE	BlackRock Municipal Income Trust II	Blackrock Municipal Income Trust II is a perpetual closed-end municipal bond fund. Its objective is to provide current income exempt from regular Federal income taxes and Maryland personal income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.306192	2020-10-11 23:38:31.02423
BLMT	BSB Bancorp Inc.	BSB Bancorp Inc is a bank holding company. Its subsidiary is Massachusetts chartered savings bank. It provides financial services to individuals, families, and businesses. Its business is divided into personal banking, business banking, and lending.	{"Financial Services","Banks - Regional - US",Banks}	{HIFS,PBCT,WEBK,UBNK,WNEB}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:00.321881	2020-10-11 23:38:31.028166
BLX	Banco Latinoamericano de Comercio Exterior S.A.	Banco Latinoamericano de Comercio Exterior SA is a supernational bank. It is principally engaged in providing trade financing to selected commercial banks, middle-market companies and corporations in the region.	{"Financial Services","Banks - Regional - Latin America",Banks}	{GGAL,BAP,BMA,CIB,BFR,BBD,ITUB,ITCB}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.345967	2020-10-11 23:38:31.023007
BMS	Bemis Company Inc.	Bemis Co Inc operates in the packaging industry. The company engages in the manufacture of packaging and pressure sensitive materials. Majority of the products are sold to the food industry. The United States packaging segment generates maximum revenue.	{"Consumer Cyclical","Packaging & Containers"}	{ATR,SEE,AVY,SON,MMM,CCF,BERY}	Packaging & Containers	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:00.359909	2020-10-11 23:38:31.024353
BOJA	Bojangles' Inc.	Bojangles Inc is a restaurant operator and franchisor. It has company restaurants, party franchised restaurants, and independent franchised restaurants operating under the Bojangles' name.	{"Consumer Cyclical",Restaurants}	{LUB,FRGI,CHUY,BLMN}	Restaurants	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:00.392272	2020-10-11 23:38:31.021426
BOOT	Boot Barn Holdings Inc.	Boot Barn Holdings Inc operates specialty retail stores that sell western and work boots and related apparel and accessories. The Company operates retail locations throughout the U.S. and sells its merchandise via the Internet.	{"Consumer Cyclical","Apparel Stores","Retail - Apparel & Specialty"}	{SCVL,TLYS}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:00.404552	2020-10-11 23:38:31.021426
BP	BP p.l.c.	BP PLC is an integrated oil and gas company. Its business activities involved oil and natural gas exploration, midstream transportation, supply and trading of crude oil, petrochemicals products and related services to wholesale and retail customers.	{Energy,"Oil & Gas Integrated","Oil & Gas - Integrated"}	{E,TOT,XOM,CVX,COP,RDS.A}	Oil & Gas - Integrated	Energy	New York Stock Exchange	\N	2020-10-11 23:12:00.417071	2020-10-11 23:38:31.025871
BPOPN	Popular Inc. Popular Capital Trust I -6.70% Cumulative Monthly Income Trust Preferred Securities	Popular Capital Trust I is a statutory trust. It is engaged in issuing capital securities.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.430178	2020-10-11 23:38:31.028166
BR	Broadridge Financial Solutions Inc.Common Stock	Broadridge Financial Solutions Inc provides investor communications & technology-driven solutions to banks, broker-dealers and corporate issuers. Its services include investor & customer communications, securities processing & data & analytics solutions.	{Industrials,"Business Services"}	{TRI,SYKE,INOD,PAYX,EXLS,IBM,WNS,MSCI,CLGX}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:00.442093	2020-10-11 23:38:31.028166
BRGL	Bernstein Global Research Fund	The investment seeks to provide investment results that, before fees and expenses, track the performance of the Bernstein Global Research Index.\n The fund will normally invest at least 80% of its total assets in component securities of the index. The index is designed to measure the performance of large-cap global stocks rated "Outperform" by sell-side analysts of Sanford C. Bernstein & Co., LLC ("Bernstein" or the "index provider") and ranked within one of the top three quintiles of Bernstein's published quantitative alpha model.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:00.453138	2020-10-11 23:38:31.028166
BRS	Bristow Group Inc.	Bristow Group Inc is an industrial aviation services provider. The company is a helicopter service provider to the offshore energy industry with international operations.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{PHII,CKH,ERA,TDW}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:00.465929	2020-10-11 23:38:31.028166
BSCP	Invesco BulletShares 2025 Corporate Bond	The investment seeks investment results that correspond generally to the performance of the Nasdaq BulletShares® USD Corporate Bond 2025 Index.\n The fund will invest at least 80% of its total assets in component securities that comprise the index. The index is designed to represent the performance of a held-to-maturity portfolio of U.S. dollar-denominated investment-grade corporate bonds with effective maturities in the year 2025. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.483601	2020-10-11 23:38:31.023428
BMO	Bank Of Montreal	Bank of Montreal provides diversified financial services primarily in North America. The company offers broad range of personal and commercial banking, wealth management and investment banking products and services.	{"Financial Services","Banks - Global",Banks}	{BNS,CM,RY,TD,WFC,RBS,C}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.354741	2020-10-11 23:38:31.023007
BNFT	Benefitfocus Inc.	Benefitfocus Inc is a cloud-based benefits management platform for consumers, employers, insurance carriers, and brokers. The firm's solutions support core benefit plans including healthcare, dental, life, and disability insurance.	{Technology,"Software - Application","Application Software"}	{ADP,PAYX,EBIX,SAP,GWRE,ORCL,MJCO,XLK}	Application Software	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:00.37566	2020-10-11 23:38:31.02423
BOLD	Audentes Therapeutics Inc.	Audentes Therapeutics Inc is a biotechnology company. It is engaged in developing and commercializing gene therapy products for patients suffering from serious, life-threatening rare diseases caused by single gene defects.	{Healthcare,Biotechnology}	{PDLI,PIRS,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:00.39355	2020-10-11 23:38:31.031328
BOTJ	Bank of the James Financial Group Inc.	Bank of the James Financial Group Inc is a bank holding company which provides retail and commercial banking, deposit services, lending services and other banking services.	{"Financial Services","Banks - Regional - US",Banks}	{FBSS,TOWN,SONA,UBSH,VBFC}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:00.408728	2020-10-11 23:38:31.023428
BPMC	Blueprint Medicines Corporation	Blueprint Medicines Corp is a biopharmaceutical company. It is focused on improving the lives of patients with diseases driven by abnormal kinase activation. Its drug candidates include avapritinib, BLU 554, BLU-782 and BLU-667.	{Healthcare,Biotechnology}	{ISR,NLNK,GTXI,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:00.422216	2020-10-11 23:38:31.023007
BQH	Blackrock New York Municipal Bond Trust of Beneficial Interest	Blackrock New York Municipal Bond Trust is a closed-end municipal bond fund with the investment objective to provide current income exempt from regular Federal income taxes and New York State and New York City personal income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.434519	2020-10-11 23:38:31.023007
BRG	Bluerock Residential Growth REIT Inc. Class A	Bluerock Residential Growth REIT Inc invests in institutional-quality apartment properties in US growth markets. The Company's objective is to maximize returns through investments in Class A institutional-quality apartment properties in United States.	{"Real Estate","REIT - Residential",REITs}	{EQR,APTS}	REITs	Real Estate	NYSE American	\N	2020-10-11 23:12:00.450715	2020-10-11 23:38:31.02423
BRPAU	Big Rock Partners Acquisition Corp. Unit	Big Rock Partners Acquisition Corp is a United States based blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:00.464529	2020-10-11 23:38:31.024353
BSBR	Banco Santander Brasil SA American Depositary Shares each representing one unit	Banco Santander (Brasil) SA operates as a multiple service bank. It is engaged in investment, lending and financing, mortgage lending, leasing, credit card operations and foreign exchange. The company operates in Brazil and internationally.	{"Financial Services","Banks - Regional - Latin America",Banks}	{}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.479456	2020-10-11 23:38:31.031328
BSQR	BSQUARE Corporation	BSQUARE Corp provides software solutions and related engineering services to companies that develop smart, connected systems. It offers asset utilization, warranty cost reduction and other solutions.	{Technology,"Software - Application","Application Software"}	{INTC,MSFT,ADBE,QCOM,WSTG,ARW,SNX,GOOGL,AVT,XLK}	Application Software	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:00.505664	2020-10-11 23:38:31.028166
BTI	British American Tobacco Industries p.l.c. ADR	British American Tobacco PLC manufactures and sells cigarettes and other tobacco products under the brands Dunhill, Kent, Lucky Strike, Pall Mall and Rothmans. It also offers Vapour Products, like e-cigarettes, and Tobacco Heating Products.	{"Consumer Defensive",Tobacco,"Tobacco Products"}	{VGR,MO,PM}	Tobacco Products	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:00.520097	2020-10-11 23:38:31.025871
BVAL	Brand Value	The investment seeks to track the performance, before fees and expenses, of the EQM Brand Value Index (the "index").\n Under normal circumstances, at least 80% of the fund's total assets (exclusive of any collateral held from securities lending) will be invested in the component securities of the index. The index measures the performance of up to 50 publicly-traded U.S. companies based on a proprietary analysis of the value of their brand relative to their market capitalization.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.539331	2020-10-11 23:38:31.028166
BWX	SPDR Bloomberg Barclays Intl Treasury Bd	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the price and yield performance of the Bloomberg Barclays Global Treasury ex-US Capped Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index or in securities that the Adviser determines have economic characteristics that are substantially identical to the economic characteristics of the securities that comprise the index. The index is designed to track the fixed-rate local currency sovereign debt of investment grade countries outside the United States. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.553412	2020-10-11 23:38:31.028166
BOKF	BOK Financial Corporation	BOK Financial Corp is a financial company providing full service banking in the United States. The company through its subsidiary BOKF, NA, operates in three segments, Commercial Banking, Consumer Banking and Wealth Management services.	{"Financial Services","Banks - Regional - US",Banks}	{BANF,IBOC,HWC,SFNC,IBKC,CFR,WFC,TCF,RF}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.392974	2020-10-11 23:38:31.025871
BPFHW	Boston Private Financial Holdings Inc. Warrants to purchase 1 share of common stock @ $8.00/share	Boston Private Financial Holdings Inc is a wealth management company. It offers various wealth management services to high net worth individuals, families, businesses and select institution.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.420291	2020-10-11 23:38:31.021426
BPY	Brookfield Property Partners L.P.	Brookfield Property Partners LP owns, operates, and invests in commercial properties in North America, Europe, Australia, and Brazil. Its operating segment includes Core Office, Core Retail, Opportunistic and Corporate Segments.	{"Real Estate","Real Estate Services"}	{JOE,JLL,KW}	Real Estate Services	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:00.434468	2020-10-11 23:38:31.031328
BREW	Craft Brew Alliance Inc.	Craft Brew Alliance Inc is a craft brewing company. It brews craft beers in four company-owned breweries located in Portland, Oregon; the Seattle suburb of Woodinville, Washington; Portsmouth, New Hampshire; & Kailua-Kona, Hawaii; & in Memphis, Tennessee.	{"Consumer Defensive","Beverages - Brewers","Beverages - Alcoholic"}	{SAM,TAP,BUD}	Beverages - Alcoholic	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:00.445952	2020-10-11 23:38:31.02582
BRKS	Brooks Automation Inc.	Brooks Automation Inc is a provider of automation and cryogenic solutions for multiple applications and markets. The company serves the semiconductor capital equipment market and sample management market for life sciences.	{Technology,"Semiconductor Equipment & Materials",Semiconductors}	{AMAT,CLS,AEIS,IVAC,CVV,ASYS,MKSI,COHU,LRCX,ENTG,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:00.45891	2020-10-11 23:38:31.028166
BRT	BRT Apartments Corp. (MD)	BRT Apartments Corp, formerly BRT Realty Trust is engaged in ownership and operation and development of multi-family properties, real estate lending, and the ownership, operation and development of commercial, mixed use and other real estate assets.	{"Real Estate","REIT - Residential",REITs}	{CPT,MAA,ESS,AIV,CLI,AVB,APTS,IRT}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:00.470622	2020-10-11 23:38:31.02423
BSCO	Invesco BulletShares 2024 Corporate Bond	The investment seeks investment results that correspond generally to the performance, before the fund's fees and expenses, of an investment grade corporate bond index called the Nasdaq BulletShares® USD Corporate Bond 2024 Index.\n The fund will invest at least 80% of its total assets in component securities that comprise the index. The index is designed to represent the performance of a held-to-maturity portfolio of U.S. dollar-denominated investment grade corporate bonds with effective maturities in the year 2024. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.484185	2020-10-11 23:38:31.028166
BSJP	Invesco BulletShares 2025 High Yield Corporate Bond	The investment seeks investment results that correspond generally to the performance, before the fund's fees and expenses, of a high yield corporate bond index called the Nasdaq BulletShares® USD High Yield Corporate Bond 2025 Index.\n The fund will invest at least 80% of its total assets in component securities that comprise the index. The index is designed to represent the performance of a held-to-maturity portfolio of U.S. dollar-denominated high yield corporate bonds with effective maturities in the year 2025. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.497364	2020-10-11 23:38:31.028166
BSRR	Sierra Bancorp	Sierra Bancorp is a bank holding company to Bank of the Sierra which offers a range of retail and commercial banking services primarily in Tulare, Kern, Fresno, and Kings Counties in Central California, and in select markets in Southern California.	{"Financial Services","Banks - Regional - US",Banks}	{TCBK,AMRB,CATY,CVBF,EWBC,CVCY,PPBI,BOCH,BMRC,OVLY}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.507154	2020-10-11 23:38:31.02582
BTG	B2Gold Corp (Canada)	B2Gold Corp is a Vancouver-based gold producer with five operating mines, one in Mali, one inNamibia, one in the Philippines and two in Nicaragua and a portfolio of other evaluation and exploration assets.	{"Basic Materials",Gold,"Metals & Mining"}	{CDE,HL,MUX}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:00.519083	2020-10-11 23:38:31.02582
BURG	Chanticleer Holdings Inc.	Chanticleer Holdings Inc, along with its subsidiaries owns, operates and franchises fast-casual dining concepts domestically and internationally. The company owns and operates Hooters branded restaurants in international markets.	{"Consumer Cyclical",Restaurants}	{CAKE,RRGB,TXRH,BJRI,DRI,BDL,BLMN}	Restaurants	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:00.531239	2020-10-11 23:38:31.024353
BW	Babcock & Wilcox Enterprises Inc.	Babcock & Wilcox Enterprises Inc is engaged in the manufacturing of fossil and renewable power generation equipment. It offers a broad suite of boiler products and environmental systems and services for power and industrial uses.	{Technology,"Electronic Components","Computer Hardware"}	{GNRC,SPWR,HUBB,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:00.543317	2020-10-11 23:38:31.028166
FLGT	Fulgent Genetics Inc.	Fulgent Genetics Inc is a technology company offering genetic testing to provide physicians with clinically actionable diagnostic information used to improve the overall quality of patient care.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{NEO,NVTA,ONCS,NTRA,XLV}	Medical Diagnostics & Research	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:03.049806	2020-10-11 23:38:31.023007
BPI	Bridgepoint Education Inc.	Bridgepoint Education Inc is an education services company. It offers campus and online postsecondary programs through two universities, Ashford University and University of the Rockies. The firm offers degree programs in multiple fields.	{"Consumer Defensive","Education & Training Services",Education}	{CECO,LINC,APEI,LOPE,NAUH,ATGE}	Education	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:00.422152	2020-10-11 23:38:31.023428
BPTH	Bio-Path Holdings Inc.	Bio-Path Holdings Inc is a clinical and preclinical stage oncology focused antisense drug development company utilizing novel technology achieving systemic delivery for specific protein inhibition for any gene product that is over-expressed in disease.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{XLV}	Drug Manufacturers	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:00.434403	2020-10-11 23:38:31.023428
BRC	Brady Corporation	Brady Corp is a manufacturer and supplier of identification solutions and workplace safety products that identify and protect premises, products and people.	{Industrials,"Security & Protection Services","Consulting & Outsourcing"}	{TACT,LABL,ZBRA,DHR,HON,MMM,ODC}	Consulting & Outsourcing	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:00.445803	2020-10-11 23:38:31.028166
BRKL	Brookline Bancorp Inc.	Brookline Bancorp Inc operates as a multi-bank holding company. Its offers commercial, business and retail banking services, including a full complement of cash management products, on-line banking services, consumer and residential loans.	{"Financial Services","Banks - Regional - US",Banks}	{EBTC,BPFH,CNBKA,INDB,WASH,PBCT,WBS,NBN,BHLB,WNEB}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.456126	2020-10-11 23:38:31.023428
BRPAW	Big Rock Partners Acquisition Corp. Warrant		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:00.465141	2020-10-11 23:38:31.031328
BSAC	Banco Santander - Chile ADS	Banco Santander Chile is a commercial and retail bank. It provides a broad range of commercial and retail banking services including Chilean peso and foreign currency denominated loans to finance a variety of commercial and retail banking services.	{"Financial Services","Banks - Regional - Latin America",Banks}	{BAP,SAN,CIB,BFR,BCH,BMA,ITCB}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.477581	2020-10-11 23:38:31.025871
BSD	BlackRock Strategic Municipal Trust Inc. (The)	Blackrock Strategic Municipal Trust is a diversified closed-end management investment company.The Trust's objectives are to provide current income that is exempt from regular federal income tax and to invest in municipal bonds.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.488529	2020-10-11 23:38:31.021426
BSL	Blackstone GSO Senior Floating Rate Term Fund of Beneficial Interest	Blackstone/GSO Senior Floating Rate Trm is a closed-end term fund. Its objective is to seek high current income, with a secondary objective to seek preservation of capital, consistent with its primary goal of high current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.497333	2020-10-11 23:38:31.02423
BTE	Baytex Energy Corp	Baytex Energy Corp is engaged in the oil and gas industry. Its key business includes exploration and production of crude oil and natural gas.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{ERF,CPG}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:00.516582	2020-10-11 23:38:31.023428
BTX	BioTime Inc.	BioTime Inc focuses on regenerative medicine technologies in ophthalmology, soft tissue reconstruction, and the local delivery of cells and therapeutics.	{Healthcare,Biotechnology}	{GERN,MNTA,PDLI,OMER,BAX,SNY,TEVA,TECH,GENE,XLV}	Biotechnology	Healthcare	NYSE American	\N	2020-10-11 23:12:00.525771	2020-10-11 23:38:31.023007
BUY	USCF SummerHaven SHPEI Index Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the SummerHaven Private Equity Strategy IndexSM ("SHPEI").\n Under normal market conditions, the fund generally invests substantially all, but at least 80%, of its total assets in the common stocks comprising SHPEI. It generally invests in substantially all of the common stocks comprising SHPEI and in approximately the same proportions as SHPEI. SHPEI attempts to replicate the long-term (i.e., 10 years or more) return characteristics of diversified private equity allocations.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.536195	2020-10-11 23:38:31.023428
BWFG	Bankwell Financial Group Inc.	Bankwell Financial Group Inc is a bank holding company which offers variety of financial services through its banking subsidiary, Bankwell Bank. Services offered by the company includes deposits, loans and advances, credit facilities and other.	{"Financial Services","Banks - Regional - US",Banks}	{UNB,BPFH,CNBKA,WASH,FNLC,WBS,BHB,NBN,SAL,PNBK}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:00.547154	2020-10-11 23:38:31.021426
BXMX	Nuveen S&P 500 Buy-Write Income Fund of Beneficial Interest	Nuveen S&P 500 Buy-Write Income Fund is closed-ended investment company. Its objective is to provide a high level of current income and gains through investments in diversified equity portfolio.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.559914	2020-10-11 23:38:31.025871
BYM	Blackrock Municipal Income Quality Trust of Beneficial Interest	Blackrock Municipal Income Quality Trust is a closed-end municipal bond fund. Its investment objective is to provide current income exempt from U.S. federal income taxes, including the alternative minimum tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.569499	2020-10-11 23:38:31.021426
BPMP	BP Midstream Partners LP representing Limited Partner Interests	BP Midstream Partners LP owns, operates, develops and acquires pipelines and other midstream assets.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{PAA,PPL,PSXP,PAGP}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:00.425355	2020-10-11 23:38:31.02423
BRAC	Black Ridge Acquisition Corp.	Black Ridge Acquisition Corp is a blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:00.443455	2020-10-11 23:38:31.025871
BRID	Bridgford Foods Corporation	Bridgford Foods Corporation manufactures, markets, and distributes frozen and snack food products in the United States. The business segments are processing and distribution of frozen products, and processing and distribution of snack food products.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{JJSF,SAFM,CPB}	Consumer Packaged Goods	Consumer Defensive	NASDAQ Global Market	\N	2020-10-11 23:12:00.453432	2020-10-11 23:38:31.031328
BRQS	Borqs Technologies Inc.	Borqs Technologies Inc provides software, development services and products providing customizable, differentiated and scalable Android-based smart connected devices and cloud service solutions.	{Technology,"Software - Application","Application Software"}	{PDFS,OMCL,XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:00.465698	2020-10-11 23:38:31.021426
BSCK	Invesco BulletShares 2020 Corporate Bond	The investment seeks investment results that correspond generally to the performance, before the fund's fees and expenses, of an investment grade corporate bond index called the Nasdaq BulletShares® USD Corporate Bond 2020 Index.\n The fund normally invests at least 80% of its total assets in component securities that comprise the index. The index is designed to represent the performance of a held-to-maturity portfolio of U.S. dollar-denominated investment-grade corporate bonds with effective maturities in the year 2020. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.479466	2020-10-11 23:38:31.023428
BSET	Bassett Furniture Industries Incorporated	Bassett Furniture Industries Inc is a manufacturer, importer and retailer of mid-priced home furnishings. The company's business segments are Wholesale, Retail and Logistical services.	{"Consumer Cyclical","Home Furnishings & Fixtures","Manufacturing - Apparel & Furniture"}	{FLXS,HOFT,ETH,LZB}	Manufacturing - Apparel & Furniture	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:00.490494	2020-10-11 23:38:31.024353
BSPM	Biostar Pharmaceuticals Inc.	Biostar Pharmaceuticals Inc, through its subsidiaries is engaged in the development, manufacturing and marketing of pharmaceutical products for a variety of diseases and conditions in the People's Republic of China.	{Healthcare,"Drug Manufacturers - Major","Drug Manufacturers"}	{NATR,PRPH,USNA,XLV}	Drug Manufacturers	Healthcare	OTC Pink	\N	2020-10-11 23:12:00.50104	2020-10-11 23:38:31.023428
BTAL	AGFiQ U.S. Market Neutral Anti-Beta Fund	The investment seeks performance results that correspond to the price and yield performance, before fees and expenses, of the Dow Jones U.S. Thematic Market Neutral Anti-Beta Index.\n The fund invests at least 80% of its net assets in common stock of the long positions in the underlying index and sells short at least 80% of the short positions in the underlying index. The underlying index is a long/short market neutral index that is dollar-neutral. As such, it identifies long and short securities positions of approximately equal dollar amounts.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.513802	2020-10-11 23:38:31.028166
BUI	BlackRock Utility Infrastructure & Power Opportunities Trust	BlackRock Utility Infrastructure & Power Opportunities Trust is a diversified investment company. Its investment objective is to provide total return through a combination of current income, current gains and long-term capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.531212	2020-10-11 23:38:31.025871
BVSN	BroadVision Inc.	BroadVision Inc develops, markets, and supports enterprise portal applications that enable companies to unify e-business infrastructure and conduct both interactions and transactions with employees, partners and customers.	{Technology,"Software - Application","Application Software"}	{MSFT,PRGS,DWCH,SAP,RHT,IBM,CRM,ORCL,XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:00.542823	2020-10-11 23:38:31.031328
BWXT	BWX Technologies Inc.	BWX Technologies Inc supplies nuclear components and fuel to the U.S. government, provides technical, management and site services; and supplies precision manufactured components and services for the commercial nuclear power industry.	{Industrials,"Aerospace & Defense"}	{JEC,NOC,AOS,FLR}	Aerospace & Defense	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:00.554057	2020-10-11 23:38:31.031328
BXS	BancorpSouth Bank	BancorpSouth Bank conducts banking and financial services operations. It provides financial services to individuals and small-to-medium size businesses.	{"Financial Services","Banks - Regional - US",Banks}	{HWC,TRMK,IBKC,PNFP,FBMS,RNST,WFC,STI,FHN,RF}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.568114	2020-10-11 23:38:31.028166
CACC	Credit Acceptance Corporation	Credit Acceptance Corp is a consumer finance company that offers automobile dealers financing programs that enables them to sell vehicles to consumers regardless of their credit history.	{"Financial Services","Credit Services"}	{NICK,CPSS,ATLC,SC}	Credit Services	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.586742	2020-10-11 23:38:31.024353
CAL	Caleres Inc.	Caleres Inc is a global footwear retailer and wholesaler. Its business activities include the operation of retail shoe stores and e-commerce websites as well as the design, sourcing and marketing of footwear for women and men.	{"Consumer Cyclical","Footwear & Accessories","Manufacturing - Apparel & Furniture"}	{SCVL,SHOO,RCKY,SKX,GCO,WWW,FL,DSW}	Manufacturing - Apparel & Furniture	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:00.604393	2020-10-11 23:38:31.02582
BRSS	Global Brass and Copper Holdings Inc.	Global Brass and Copper Holdings Inc is a fabricator, distributor and processor of specialized copper and brass products. Engages in metal melting and casting, rolling, extruding and stamping to fabricate finished and semi-finished alloy products.	{Industrials,"Metal Fabrication","Industrial Products"}	{MLI,ATI,MTRN}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:00.468236	2020-10-11 23:38:31.028166
BSCN	Invesco BulletShares 2023 Corporate Bond	The investment seeks investment results that correspond generally to the performance, before the fund's fees and expenses, of an investment grade corporate bond index called the Nasdaq BulletShares® USD Corporate Bond 2023 Index.\n The fund will invest at least 80% of its total assets in component securities that comprise the index. The index is designed to represent the performance of a held-to-maturity portfolio of U.S. dollar-denominated investment-grade corporate bonds with effective maturities in the year 2023. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.482057	2020-10-11 23:38:31.028166
BSE	Blackrock New York Municipal Income Quality Trust of Beneficial Interest	Blackrock NY Insured Municipal Income Tr is a closed end management investment company. Its objective is to provide current income exempt from federal income tax, including alternative minimum tax, and New York State & New York City personal income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.490455	2020-10-11 23:38:31.02423
BSM	Black Stone Minerals L.P. representing limited partner interests	Black Stone Minerals LP is the owner of oil and natural gas mineral interests in the United States. The company also owns non-operated working interests in certain oil and natural gas properties.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{CRZO,ICON,COG}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:00.499071	2020-10-11 23:38:31.021426
BSV	Vanguard Short-Term Bond	The investment seeks to track the performance of Bloomberg Barclays U.S. 1-5 Year Government/Credit Float Adjusted Index.\n Bloomberg Barclays U.S. 1-5 Year Government/Credit Float Adjusted Index includes all medium and larger issues of U.S. government, investment-grade corporate, and investment-grade international dollar-denominated bonds that have maturities between 1 and 5 years and are publicly issued. All of its investments will be selected through the sampling process, and at least 80% of its assets will be invested in bonds held in the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.507962	2020-10-11 23:38:31.02423
BUD	Anheuser-Busch Inbev SA Sponsored ADR (Belgium)	Anheuser-Busch InBev SA/NV is engaged in the production, marketing, and distribution of beer and other malt beverage brands. Its brands includes Budweiser, Corona, Stella Artois, Beck's, Leffe, Hoegaarden, Castle Lager and others.	{"Consumer Defensive","Beverages - Brewers","Beverages - Alcoholic"}	{DEO,SAM,CCU,TAP}	Beverages - Alcoholic	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:00.528527	2020-10-11 23:38:31.031328
BVXV	BiondVax Pharmaceuticals Ltd.	BiondVax Pharmaceuticals Ltd is a clinical stage biopharmaceutical company. The company is engaged in developing, manufacturing and commercializing immunomodulation therapies for infectious diseases.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:00.542971	2020-10-11 23:38:31.023007
BXE	Bellatrix Exploration Ltd (Canada)	Bellatrix Exploration Ltd is an oil and gas exploration company based in Calgary. It is engaged in exploration and development of light oil and liquids-rich natural gas in the Western Canada Sedimentary Basin.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{EGY,PHX,AR}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:00.55593	2020-10-11 23:38:31.021426
BY	Byline Bancorp Inc.	Byline Bancorp Inc is a bank holding company. It conducts all its business activities through its subsidiary, Byline Bank, a full service commercial bank.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.565629	2020-10-11 23:38:31.031328
BZM	BlackRock Maryland Municipal Bond Trust of beneficial interest	Blackrock Maryland Municipal Bond Trust is a closed-end management investment company. Its investment objective is to provide current income exempt from regular federal income taxes and Maryland personal income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.575607	2020-10-11 23:38:31.028166
CAG	ConAgra Brands Inc.	Conagra Brands Inc is in the business of trading and merchandising packaged food in a variety of categories and in various retail channels across frozen, refrigerated, and shelf-stable temperature classes in North America.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{LANC,HAIN,GIS,SJM,THS,MDLZ,DF,PEP}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:00.597914	2020-10-11 23:38:31.023007
CAMP	CalAmp Corp.	CalAmp Corp provides wireless communications solutions applications to customers. Its business activities are organized into its Wireless DataCom and Satellite business segments.	{Technology,"Communication Equipment"}	{SWIR,CMTL,PCTI,CRNT,MSI,UBNT,AVNW,INSG,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:00.610904	2020-10-11 23:38:31.02423
CARO	Carolina Financial Corporation	Carolina Financial Corp is a US-based bank holding company. The bank's operating segment includes Community banking, mortgage banking and other. Generating, a majority of its revenue from the Community banking segment.	{"Financial Services","Banks - Regional - US",Banks}	{CZFC,NFBK}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:00.625086	2020-10-11 23:38:31.023007
BSCR	Invesco BulletShares 2027 Corporate Bond	The investment seeks investment results that correspond generally to the performance, before the fund's fees and expenses, of an investment grade corporate bond index called the Nasdaq BulletShares® USD Corporate Bond 2027 Index.\n The fund will invest at least 80% of its total assets in component securities that comprise the 2027 Index. The index is designed to represent the performance of a held-to-maturity portfolio of U.S. dollar-denominated investment-grade corporate bonds with effective maturities in the year 2027. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.486271	2020-10-11 23:38:31.031328
BSMX	Banco Santander (Mexico) S.A. Institucion de Banca Multiple Grupo Financiero Santander Mexico	Banco Santander Mexico SA is a financial services holding company. It provides financial and related services including retail and commercial banking, securities underwriting and brokerage and custody services, and asset management.	{"Financial Services","Banks - Regional - Latin America",Banks}	{}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.499606	2020-10-11 23:38:31.023007
BT	BT Group plc American Depositary Shares	BT Group PLC is a provider of telecommunications networks and services. It sells fixed-voice, broadband, mobile and TV products and services to consumers in the UK.	{"Communication Services","Telecom Services"}	{TEF,TI,ORAN,VOD}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:00.509285	2020-10-11 23:38:31.023007
BTO	John Hancock Financial Opportunities Fund	John Hancock Bank & Thrift Opprtnty Fund is a United States based closed-end, diversified management investment company. Its investment objective is to provide a high level of total return consisting of long-term capital appreciation and current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.51934	2020-10-11 23:38:31.023428
BUSE	First Busey Corporation	First Busey Corporation is a financial holding company, whose subsidiaries provides retail and commercial banking services, remittance processing, and offers financial products and services.	{"Financial Services","Banks - Regional - US",Banks}	{OSBC,WTFC,MCBC,FMBI,MBFI}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.533885	2020-10-11 23:38:31.02423
BWA	BorgWarner Inc.	BorgWarner Inc is a supplier of auto parts. Its products comprise technologies for combustion, hybrid and electric vehicles. The company operates in two business segments that are Engine and Drivetrain. The Engine segment generates maximum revenue.	{"Consumer Cyclical","Auto Parts",Autos}	{MGA,NAV,CMI,HON,MOD,OSK,TEN,DAN,LEA,VC}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:00.547503	2020-10-11 23:38:31.023428
BXP	Boston Properties Inc.	Boston Properties Inc is a fully integrated, self-administered and self-managed real estate investment trust. The company owns, develops and manages office properties in the United States.	{"Real Estate","REIT - Office",REITs}	{TCO,VNO,CBL,DLR}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:00.565248	2020-10-11 23:38:31.024353
BYSI	BeyondSpring Inc.	BeyondSpring Inc is engaged in clinical stage biopharmaceutical activities focused on the development of cancer therapies.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:00.573118	2020-10-11 23:38:31.02423
C	Citigroup Inc.	Citigroup Inc is a financial-services company with business spanning more than 100 countries. Its core business is providing banking services, investment banking, cash management, and various other products and services.	{"Financial Services","Banks - Global",Banks}	{BCS,MS,BAC,DB,HSBC,WFC,JPM,USB,MUFG}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.580956	2020-10-11 23:38:31.02582
CACI	CACI International Inc. Class A	CACI International Inc is a provider of information solutions and services to the U.S. government. Its segments include domestic operations and international operations.	{Technology,"Information Technology Services","Application Software"}	{CPSI,EGOV,ICFI,KEYW,LMT,HRS,LDOS,SAIC,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:00.58883	2020-10-11 23:38:31.031328
CAH	Cardinal Health Inc.	Cardinal Health Inc is a healthcare services company. It provides services and products for hospital systems, pharmacies, ambulatory surgery centers, clinical laboratories and physician offices.	{Healthcare,"Medical Distribution"}	{HSIC,PDCO,OMI,CVS,RAD,TFX,MCK,ABC,HLF,PINC,XLV}	Medical Distribution	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:00.598349	2020-10-11 23:38:31.02423
CALM	Cal-Maine Foods Inc.	Cal-Maine Foods Inc is a producer and marketer of shell eggs in the United States. The Company's main business is the production, grading, packaging, marketing and distribution of shell eggs.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{COST,SAFM,FDP,SYY,WMT,IBA,PPC}	Consumer Packaged Goods	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:00.606186	2020-10-11 23:38:31.023007
CAPL	CrossAmerica Partners LP representing limited partner interests	CrossAmerica Partners LP is engaged in the wholesale distribution of motor fuels, consisting of gasoline and diesel fuel. It also owns and leases real estate used in the retail distribution of motor fuels.	{Energy,"Oil & Gas Refining & Marketing","Oil & Gas - Refining & Marketing"}	{CLNE,FGP,GLP,SRLP,SUN}	Oil & Gas - Refining & Marketing	Energy	New York Stock Exchange	\N	2020-10-11 23:12:00.617678	2020-10-11 23:38:31.028166
CATB	Catabasis Pharmaceuticals Inc.	Catabasis Pharmaceuticals Inc is a clinical-stage biopharmaceutical company. The company's primary focus is on treatments for rare diseases.	{Healthcare,Biotechnology}	{GERN,NBY,DFFN,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:00.638589	2020-10-11 23:38:31.024353
BST	BlackRock Science and Technology Trust of Beneficial Interest	BlackRock Science and Technology Trust is a closed end management investment company. Its objective is to provide income and total return through a combination of current income, current gains and long-term capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.509375	2020-10-11 23:38:31.031328
BTN	Ballantyne Strong Inc.	Ballantyne Strong Inc designs, integrates and installs technology solutions for applications; develops and delivers out-of-home messaging, advertising and communications; manufactures projection screens & lighting products; and provides managed services.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{AVID,IKNX,SNE,IMAX}	Travel & Leisure	Consumer Cyclical	NYSE American	\N	2020-10-11 23:12:00.519051	2020-10-11 23:38:31.02423
BURL	Burlington Stores Inc.	Burlington Stores Inc is a retailer of everyday apparel at low prices. It offers apparel including ladies sportswear, menswear, coats and family footwear as well as baby furniture, accessories, home decor and gifts.	{"Consumer Defensive","Discount Stores","Retail - Defensive"}	{FRED,CTRN,DDS,JCP,TGT,M}	Retail - Defensive	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:00.531416	2020-10-11 23:38:31.028166
BVN	Buenaventura Mining Company Inc.	Buenaventura Mining Co Inc is a mining firm in Peru. Its business involves exploring for, mining, and processing gold and silver.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{PAAS,EXK,CDE,FCX,HL,NEM,SCCO,AG,WPM}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:00.540578	2020-10-11 23:38:31.024353
BXC	Bluelinx Holdings Inc.	BlueLinx Holdings Inc is a distributer of building products in the United States. The company operates through a network of warehouses, supplying products such as plywood, lumber, roofing and insulation.	{"Basic Materials","Building Materials"}	{JCTCF,PATK,BECN,BLDR,LL,BCC,HBP}	Building Materials	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:00.556493	2020-10-11 23:38:31.028166
CAE	CAE Inc.	CAE Inc and its subsidiaries design, manufacture and supply simulation equipment. It provides training and develop integrated training solutions for defence and security markets, commercial airlines, business aircraft operators and helicopter operators.	{Industrials,"Aerospace & Defense"}	{LMT,BA,GD,UTX,RTN,COL,LLL}	Aerospace & Defense	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:00.593723	2020-10-11 23:38:31.023428
CALI	China Auto Logistics Inc.	China Auto Logistics Inc is engaged in providing imported automobile sales and trading services and web-based automobile trading sales and information platform to its customers.	{"Consumer Cyclical","Auto & Truck Dealerships",Autos}	{LAD,AUTO}	Autos	Consumer Cyclical	OTC Grey Market	\N	2020-10-11 23:12:00.605781	2020-10-11 23:38:31.025871
CANF	Can-Fite Biopharma Ltd Sponsored ADR (Israel)	Can Fite Biofarma Ltd is a clinical stage biopharmaceutical company. It develops orally bioavailable small molecule therapeutic products for the treatment of autoimmune-inflammatory, oncological and ophthalmic diseases.	{Healthcare,Biotechnology}	{MNTA,MLNT,XLV}	Biotechnology	Healthcare	NYSE American	\N	2020-10-11 23:12:00.618276	2020-10-11 23:38:31.023428
CASH	Meta Financial Group Inc.	Meta Financial Group Inc is a unitary savings and loan holding company. The Company is engaged in two core businesses namely, retail banking and Meta Payment Systems.	{"Financial Services","Savings & Cooperative Banks",Banks}	{SMBC,HMNF,GFED,LARK,COF}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.632799	2020-10-11 23:38:31.031328
CATS	Catasys Inc.	Catasys Inc is a healthcare services company, provides health services designed to assist health plans & employees to manage & treat their high cost substance dependence members through a network of healthcare providers & its employees.	{Healthcare,"Medical Care","Health Care Providers"}	{HMSY,MGLN,XLV}	Health Care Providers	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:00.649661	2020-10-11 23:38:31.031328
CBLI	Cleveland BioLabs Inc.	Cleveland BioLabs Inc is a biopharmaceutical company developing novel approaches to activate immune system & address medical needs. Its proprietary platform of toll-like immune receptor activators are used in radiation, oncology immunotherapy & vaccine.	{Healthcare,Biotechnology}	{AMGN,GSK,NVS,BMY,LLY,SNY,OPK,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:00.66424	2020-10-11 23:38:31.024353
CBRL	Cracker Barrel Old Country Store Inc.	Cracker Barrel Old Country Store Inc operates full-service restaurants in the United States. It offers home-style country cooking featuring many of its own recipes for breakfast, lunch, and dinner.	{"Consumer Cyclical",Restaurants}	{JACK,ARKR,DENN,DRI,EAT,DIN,BLMN}	Restaurants	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:00.674902	2020-10-11 23:38:31.02582
CC	Chemours Company (The)	The Chemours Co is a chemical manufacturing company that produces and develops titanium technologies, fluoroproducts and chemical solutions. It also produces refrigerants and industrial resins.	{"Basic Materials","Specialty Chemicals",Chemicals}	{SCL}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:00.686902	2020-10-11 23:38:31.021426
CCIH	ChinaCache International Holdings Ltd.	ChinaCache International Holdings Ltd is a provider of Internet content and application delivery services in China. It provides a portfolio of services and solutions through its network to businesses, government agencies and other enterprises.	{Technology,"Internet Content & Information","Online Media"}	{AKAM,LLNW,CEA,CHA,INXN,VNET,BCOV,INAP,XLK}	Online Media	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:00.698425	2020-10-11 23:38:31.021426
BSTI	BEST Inc. American Depositary Shares each representing one Class A	BEST Inc provides New Retail platform which offers technology-enabled integrated solutions such as supply chain management, express delivery, freight, merchandise sourcing, cross-border supply chain, last-mile, financial and value-added services.	{Technology,"Software - Infrastructure","Application Software",Industrials,Trucking,"Transportation & Logistics"}	{RRTS,RLGT}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:00.507861	2020-10-11 23:38:31.025871
BTT	BlackRock Municipal 2030 Target Term Trust	Blackrock Municipal 2030 Target Term Trust is a closed-end management investment company. Its investment objective is to provide current income exempt from regular federal income tax and to return $25.00 per common share to holders of common shares.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.521579	2020-10-11 23:38:31.021426
BWEN	Broadwind Energy Inc.	Broadwind Energy Inc provides technologically value products and services to energy, mining and infrastructure sector customers in the U.S. It provides products & services to turbine manufacturers and wind farm developers & operators.	{Industrials,"Diversified Industrials","Industrial Products"}	{WWD,GE,AMSC,OPTT,CPST}	Industrial Products	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:00.547027	2020-10-11 23:38:31.02423
BXG	Bluegreen Vacations Corporation	Bluegreen Corporation is a sales, marketing and management company, focused on the vacation ownership industry. Its business has been conducted through two operating segments Sales of VOIs and financing and Resort operations and club management.	{"Consumer Cyclical","Resorts & Casinos","Travel & Leisure"}	{}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:00.558108	2020-10-11 23:38:31.02423
BYFC	Broadway Financial Corporation	Broadway Financial Corp through its subsidiary, Broadway Federal Bank, f.s.b is engaged in the business of attracting deposits from the general public and investing those deposits in mortgage loans.	{"Financial Services","Savings & Cooperative Banks",Banks}	{PROV,RVSB,TSBK,FFNW,TBNK,ANCB,FSBW,BANC}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:00.570263	2020-10-11 23:38:31.02582
CAAS	China Automotive Systems Inc.	China Automotive Systems Inc through its subsidiary is in the business of supplying steering systems and components to China automotive industry. Its products include Steering systems, steering columns, steering oil pumps and steering hoses.	{"Consumer Cyclical","Auto Parts",Autos}	{SORL,AXL,TEN,DAN,FOXF}	Autos	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:00.58263	2020-10-11 23:38:31.028166
CADE	Cadence Bancorporation Class A	Cadence Bancorp is a bank holding company and is the parent company of Cadence Bank, N.A. It offers broad range of banking and wealth management services to businesses, high net worth individuals, business owners and retail customers.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.593833	2020-10-11 23:38:31.02582
CALF	Pacer US Small Cap Cash Cows 100	The investment seeks to track the total return performance, before fees and expenses, of the Pacer US Small Cap Cash Cows Index (the "index").\n Under normal circumstances, at least 80% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in the component securities of the index. The index uses an objective, rules-based methodology to provide exposure to small-capitalization U.S. companies with high free cash flow yields. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:00.604627	2020-10-11 23:38:31.021426
CAR	Avis Budget Group Inc.	Avis Budget Group Inc provides automotive vehicle rental and car sharing services. It offers various rental services under the Avis, Budget, and Zipcar brands. The firm has its own network of rental locations and licenses its brands to franchisees.	{Industrials,"Rental & Leasing Services","Consulting & Outsourcing"}	{UHAL,F,PAG,HTZ}	Consulting & Outsourcing	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:00.617886	2020-10-11 23:38:31.024353
CART	Carolina Trust BancShares Inc.	Carolina Trust BancShares Inc is a bank holding company. The Company provides banking needs of individuals and businesses in Lincoln, Gaston, Rutherford, and Catawba Counties and surrounding areas.	{"Financial Services","Banks - Regional - US",Banks}	{PEBK,FBNC,FCNCA}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:00.627466	2020-10-11 23:38:31.023428
CAT	Caterpillar Inc.	Caterpillar Inc caters to the farming industry in the United States. It manufactures construction & mining equipment, diesel & natural gas engines, industrial gas turbines and diesel-electric locomotives.	{Industrials,"Farm & Construction Equipment","Farm & Construction Machinery"}	{ASTE,CMI,DE,GE,CNHI}	Farm & Construction Machinery	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:00.63806	2020-10-11 23:38:31.028166
CATYW	Cathay General Bancorp Warrant	Cathay General Bancorp is a holding company for Cathay Bank, which offers various financial services for individuals, professionals, and small to medium-sized businesses in California. It provides loans, savings, time deposits, and others.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.648613	2020-10-11 23:38:31.021426
CBH	AllianzGI Convertible & Income 2024 Target Term Fund of Beneficial Interest	AllianzGI Convert & Inc 2024 Target Term is a newly organized, diversified, limited-term closed-end management investment company. The Fund's investment objectives are to provide a high level of income and to returns.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.662232	2020-10-11 23:38:31.028166
BTU	Peabody Energy Corporation	Peabody Energy Corp is a coal company. It also markets and brokers coal, both as principal and agent, and trades coal and freight-related contracts. The company own interest in nearly 25 coal mines.	{"Basic Materials",Coal}	{}	Coal	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:00.524227	2020-10-11 23:38:31.028166
BWG	BrandywineGLOBAL Global Income Opportunities Fund Inc.	Legg Mason BW Global Income Opps Fd is a non-diversified, closed-end management investment company. The primary investment objective is to provide current income and to seek capital appreciation as secondary objective.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.548221	2020-10-11 23:38:31.024353
BXMT	Blackstone Mortgage Trust Inc.	Blackstone Mortgage Trust Inc is a real estate company. It originates and purchases senior mortgage loans collateralized by properties in North America and Europe.	{"Real Estate","REIT - Diversified",REITs}	{ABR,STWD,CMCT}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:00.559289	2020-10-11 23:38:31.023428
BZUN	Baozun Inc.	Baozun Inc provides its customers with end-to-end e-commerce solutions including the sales of apparel, home & electronic products, online store design, visual merchandising & marketing, online operations, customer service, warehousing & order fulfillment.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{MDSO,CDK,CIDM}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:00.576797	2020-10-11 23:38:31.031328
CAF	Morgan Stanley China A Share Fund Inc.	Morgan Stanley China A Share Fund Inc is a non-diversified, closed-end management investment company. Its investment objective is to seek capital growth by investing assets in A-shares of companies listed on the Shanghai and Shenzhen Stock Exchanges.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.595371	2020-10-11 23:38:31.021426
CANE	Teucrium Sugar Fund ETV	The investment seeks to have the daily changes in percentage terms of the shares’ NAV reflect the daily changes in percentage terms of a weighted average of the closing settlement prices for three futures contracts for sugar that are traded on ICE Futures US.\n The fund seeks to achieve its investment objective by investing under normal market conditions in Benchmark Component Futures Contracts or, in certain circumstances, in other Sugar Futures Contracts traded on ICE Futures or the New York Mercantile Exchange (“NYMEX”), or on foreign exchanges.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.612039	2020-10-11 23:38:31.02582
CARV	Carver Bancorp Inc.	Carver Bancorp Inc through its banking subsidiary operates as a unitary savings and loan holding company. The Company is engaged in providing consumer and commercial banking services offering deposit products, including demand, savings and time deposits.	{"Financial Services","Savings & Cooperative Banks",Banks}	{FFIC,ESBK,TRST,GCBC,LSBK}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:00.630051	2020-10-11 23:38:31.025871
CATC	Cambridge Bancorp	Cambridge Bancorp is a state chartered, federally registered bank. The bank offers a full range of commercial and consumer banking services through its network of full-service banking offices in Massachusetts.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:00.646624	2020-10-11 23:38:31.021426
CBAY	CymaBay Therapeutics Inc.	CymaBay Therapeutics Inc is an American biotechnology enterprise. Its product candidates include Arhalofenate, MBX-8025, and MBX-2982, which are meant to combat serious metabolic diseases.	{Healthcare,Biotechnology}	{BMY,PFE,PTCT,XNCR,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:00.656727	2020-10-11 23:38:31.024353
CBPX	Continental Building Products Inc.	Continental Building Products Inc is a manufacturer of gypsum wallboard and complementary finishing products. Geographically, its manufacturing facilities and sales efforts are concentrated in the eastern United States and eastern Canada.	{"Basic Materials","Building Materials"}	{USG,EXP,WRK}	Building Materials	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:00.675195	2020-10-11 23:38:31.025871
CBU	Community Bank System Inc.	Community Bank System Inc is a provider of financial services. The company, in association with its subsidiaries offers community banking services to retail, commercial, and municipal customers.	{"Financial Services","Banks - Regional - US",Banks}	{BDGE,FISI,AROW,FLIC,NBTB,TMP,EVBN,CHMG}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.686815	2020-10-11 23:38:31.028166
CCI	Crown Castle International Corp. (REIT)	Crown Castle International Corp operates wireless tower communication sites in the United States. It owns, operates and leases shared wireless infrastructure, including towers and other structures. Site rental generates maximum revenue for the company.	{"Real Estate","REIT - Diversified",REITs}	{RCI,CHL,AMT,TDS,VOD}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:00.697628	2020-10-11 23:38:31.023007
CCO	Clear Channel Outdoor Holdings Inc. Class A	Clear Channel Outdoor Holdings Inc is a outdoor advertising company. It is engaged in providing advertising opportunities through billboards, street furniture displays, transit displays and other out-of-home advertising displays to its clients.	{"Consumer Cyclical","Marketing Services","Advertising & Marketing Services"}	{OUT,LAMR}	Advertising & Marketing Services	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:00.708954	2020-10-11 23:38:31.028166
CCZ	Comcast Holdings ZONES	Comcast Corp is a media and technology company. It mainly involves in the cable industry and owns Comcast Cable, E!, NBCUniversal, and the NBC broadcast network.	{"Communication Services","Pay TV"}	{}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:00.722823	2020-10-11 23:38:31.028166
BTZ	BlackRock Credit Allocation Income Trust	Blackrock Preferred & Equity Advtg Tr is a taxable closed-end Trust with the investment objective of seeking current income, current gains and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.526042	2020-10-11 23:38:31.02582
BVXVW	BiondVax Pharmaceuticals (Warrant)	BiondVax Pharmaceuticals Ltd. is a clinical stage biopharmaceutical company focused on developing and commercializing immunomodulation therapies for infectious diseases. It works on M-001, a synthetic peptide-based protein targeting both seasonal and pandemic strains of the influenza virus. The company was founded by Ron Babecoff and Rami Epstein on July 22, 2003 and is headquartered in Ness Ziona, Israel.	{}	{}	Pharmaceuticals Major	Health Technology	NasdaqCM	\N	2020-10-11 23:12:00.542954	2020-10-11 23:38:31.025871
BX	The Blackstone Group L.P. Representing Limited Partnership Interests	Blackstone Group LP is an alternative asset manager. Its alternative asset management businesses include investment vehicles focused on private equity, real estate, hedge fund solutions, non-investment grade credit and secondary funds.	{"Financial Services","Asset Management"}	{GHL,LAZ,EVR,OZM,APO,CG}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.554322	2020-10-11 23:38:31.023007
BZH	Beazer Homes USA Inc.	Beazer Homes USA Inc is engaged in the construction industry. It specializes in residential construction, especially single-family housing and multi-unit building. The company also focuses on land purchasing and development.	{"Consumer Cyclical","Residential Construction","Homebuilding & Construction"}	{PHM,MHO,NVR,DHI,HOV,MTH,MDC,KBH,NWHM,LGIH}	Homebuilding & Construction	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:00.575417	2020-10-11 23:38:31.024353
CACG	ClearBridge All Cap Growth ETF	The investment seeks to achieve long-term capital appreciation.\n The fund seeks to invest in a diversified portfolio of large, medium and small capitalization stocks that have the potential for above-average long-term earnings and/or cash flow growth. The fund's subadviser uses a bottom-up investment process that seeks to find inefficiently priced companies with strong fundamentals, incentive-driven management teams, dominant positions in niche markets and/or goods and services that are in high customer demand.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:00.587644	2020-10-11 23:38:31.025871
CAJ	Canon Inc. American Depositary Shares	Canon Inc is a provider of business equipment. It manufactures office multi function devices, plain paper copying machines, laser printers, inkjet printers, cameras, diagnostic equipment and lithography equipment.	{Industrials,"Business Equipment","Industrial Products"}	{HPQ,XRX}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:00.599754	2020-10-11 23:38:31.025871
CAMT	Camtek Ltd.	Camtek Ltd provides automated & technologically solutions enhancing production processes & increasing products yields, enabling & supporting customers technologies in semiconductor fabrication and in printed circuit industries.	{Technology,"Semiconductor Equipment & Materials",Semiconductors}	{ORBK,CYBE,KLAC,NANO,NVMI,TER,RTEC,XLK}	Semiconductors	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:00.612321	2020-10-11 23:38:31.031328
CARG	CarGurus Inc.	CarGurus Inc is an online automotive marketplace connecting buyers and sellers of new and used cars. It operates online marketplaces in Canada, the United Kingdom, and Germany.	{Technology,"Internet Content & Information","Online Media"}	{NEWT,ITT,XLK}	Online Media	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:00.624451	2020-10-11 23:38:31.031328
CASM	CAS Medical Systems Inc.	CAS Medical Systems Inc is a medical technology company. The company develops, manufactures and markets non-invasive patient monitoring products that are vital to patient care. Its products include FORE-SIGHT and FORE-SIGHT ELITE brand.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{MASI,UTMD,GE,PHG,NURO,XLV}	Medical Instruments & Equipment	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:00.635994	2020-10-11 23:38:31.025871
CB	Chubb Limited	Chubb Ltd provides commercial and personal property and casualty insurance, personal accident and supplemental health insurance (A&H), reinsurance and life insurance to a diverse group of clients.	{"Financial Services","Insurance - Property & Casualty"}	{ACGL,RNR}	Insurance - Property & Casualty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.648803	2020-10-11 23:38:31.028166
CBIO	Catalyst Biosciences Inc.	Catalyst Biosciences Inc focuses on developing novel medicines to address serious medical conditions for individuals in need of treatment options.	{Healthcare,Biotechnology}	{CORT,AZN,BMY,JNJ,PFE,MRK,ABBV,ADMS,ZGNX,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:00.661538	2020-10-11 23:38:31.028166
CBS	CBS Corporation Class B	CBS Corp is a media company. It operates businesses within the media and entertainment industries, including cable networks, content production and distribution, television and radio stations, Internet-based businesses, and consumer publishing.	{"Consumer Cyclical","Media - Diversified",Entertainment}	{MSFT,CMCSA,TV,DIS,VIAB,NWSA,FOXA,GOOGL}	Entertainment	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:00.674586	2020-10-11 23:38:31.021426
CBZ	CBIZ Inc.	CBIZ Inc through its subsidiaries, provides professional business services. Its business units have been aggregated into three practice groups: Financial Services, Benefits and Insurance Services and National Practices.	{Industrials,"Business Services"}	{FORR,PRGX,HRB}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:00.686824	2020-10-11 23:38:31.031328
CCL	Carnival Corporation	Carnival Corp is leisure travel company providing vacations to major cruise destinations across the world. It operates over 100 cruise ships brands including Carnival Cruise Lines, Holland America, Princess Cruise, Seabourn and fathom, and Cunard Line.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{RCL,CUK,WYND,MAR,IHG,NCLH}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:00.701801	2020-10-11 23:38:31.025871
BYD	Boyd Gaming Corporation	Boyd Gaming Corp owns and operates more than twenty gaming properties in about seven states. Boyd operates its business into Gaming, Food and beverage and Room segment, out of which it derives majority of its revenue from Gaming segment.	{"Consumer Cyclical","Resorts & Casinos","Travel & Leisure"}	{MCRI,PENN,WYNN,MGM,LVS}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:00.566104	2020-10-11 23:38:31.023007
BZQ	ProShares UltraShort MSCI Brazil Capped	The investment seeks daily investment results, before fees and expenses, that correspond to two times the inverse (-2x) of the daily performance of the MSCI Brazil 25/50 Index®.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index is a free-float-adjusted, market capitalization-weighted index that is designed to measure the equity market performance of the Brazilian market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.576349	2020-10-11 23:38:31.025871
CAC	Camden National Corporation	Camden National Corporation provides banking services and products to individuals and commercial customers. Its services also comprise treasury and investment banking solutions.	{"Financial Services","Banks - Regional - US",Banks}	{EBTC,BPFH,INDB,WASH,FNLC,WBS,BHB,NBN,SAL,PNBK}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.585376	2020-10-11 23:38:31.02423
CAI	CAI International Inc.	CAI International Inc is a transportation finance and logistics company. It purchases equipment and then leases it to container shipping lines, freight forwarders and other transportation companies.	{Industrials,"Rental & Leasing Services","Consulting & Outsourcing"}	{MINI,WLFC,GFN,TGH,AYR,AER,ACY,AL}	Consulting & Outsourcing	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:00.598237	2020-10-11 23:38:31.028166
CALX	Calix Inc	Calix Inc provides broadband communications access systems and software. It delivers the wide variety of services to residential and business customers domestically and internationally.	{Technology,"Communication Equipment"}	{ADTN,FFIV,ERIC,JNPR,CIEN,DZSI,XLK}	Communication Equipment	Technology	New York Stock Exchange	\N	2020-10-11 23:12:00.609489	2020-10-11 23:38:31.028166
CARA	Cara Therapeutics Inc.	Cara Therapeutics Inc is an emerging biotechnology company. It is focused on developing and commercializing new chemical entities designed to alleviate pain and pruritus by selectively targeting kappa opioid receptors.	{Healthcare,Biotechnology}	{DRRX,ACOR,CPIX,JNJ,PFE,ACRX,TRVN,REPH,FLXN,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:00.619434	2020-10-11 23:38:31.02582
CARZ	First Trust NASDAQ Global Auto Index Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the NASDAQ OMX Global Auto Index(SM).\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks or in depositary receipts representing securities in the index. The index is designed to track the performance of the largest and most liquid companies engaged in manufacturing of automobiles. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:00.630044	2020-10-11 23:38:31.021426
CATO	Cato Corporation (The) Class A	Cato Corp is a fashion specialty retailer. The Company operates women's fashion specialty retail stores in the southeastern United States under two segment: the operation of a fashion specialty stores segment and a credit card segment.	{"Consumer Cyclical","Apparel Stores","Retail - Apparel & Specialty"}	{DEST,CBK,NWY,ASNA,LB,BEBE}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:00.648033	2020-10-11 23:38:31.025871
CBB	Cincinnati Bell Inc.	Cincinnati Bell Inc provides wireline and wireless telecommunication services. It also provides technology solutions such as offering information technology, data center management, and other services to enterprises.	{"Communication Services","Telecom Services"}	{ALSK,CMCSA,CTL,HCOM,TDS}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:00.657717	2020-10-11 23:38:31.02423
CCC	Churchill Capital Corp - Ordinary Shares - Class A		{Industrials}	{}			New York Stock Exchange	\N	2020-10-11 23:12:00.690176	2020-10-11 23:38:31.023428
CCJ	Cameco Corporation	Cameco Corp is engaged in the exploration and the development, mining, refining, conversion, fabrication and trading of uranium for sale as fuel for generating electricity in nuclear power reactors in Canada and other countries.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{BBL,DNN,BHP}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:00.699943	2020-10-11 23:38:31.028166
CCRN	Cross Country Healthcare Inc.	Cross Country Healthcare Inc provides healthcare staffing, recruiting, and workforce solutions. The company operates business across three segments and mainly offers nurse and allied staffing, travel nurse, and branch-based local nurse in the US.	{Industrials,"Staffing & Outsourcing Services","Employment Services"}	{KELYB,MAN,AMN}	Employment Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:00.714564	2020-10-11 23:38:31.024353
GRAM	Grana y Montero S.A.A. American Depositary Shares each representing five	Grana y Montero SAA is a Peruvian engineering firm. It provides procurement, and construction services for sectors, including mining, power, infrastructure, hotels, and others.	{Industrials,"Engineering & Construction"}	{}	Engineering & Construction	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:03.768785	2020-10-11 23:38:31.023428
CABO	Cable One Inc.	Cable One Inc mainly provides broadband, voice, and video services. It derives revenue from the provision of broadband, voice, and video services to residential and business customers.	{"Communication Services","Telecom Services"}	{RNET,HCOM,GTT,LBRDK}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:00.585197	2020-10-11 23:38:31.021426
CAKE	The Cheesecake Factory Incorporated	Cheesecake Factory Inc operates casual dining restaurants. It also operates bakery production facilities which produce desserts for its restaurants, international licenses and third-party bakery customers.	{"Consumer Cyclical",Restaurants}	{RRGB,EAT,CMG,ARCO}	Restaurants	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:00.60126	2020-10-11 23:38:31.031328
CAPR	Capricor Therapeutics Inc.	Capricor Therapeutics Inc is a clinical stage biotechnology company developing products for the treatment of cardiovascular diseases with a focus on heart failure.	{Healthcare,Biotechnology}	{CLDX,BTX,JNJ,PTN,PTLA,CYTK,ABIO,EIGR,CLBS,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:00.618901	2020-10-11 23:38:31.025871
CASI	CASI Pharmaceuticals Inc.	CASI Pharmaceuticals Inc is engaged in research and development of biopharmaceutical products. It mainly focuses on acquisition, development and commercialization of innovative therapeutics addressing cancer and other unmet medical needs.	{Healthcare,Biotechnology}	{AMGN,NBIX,CELG,RGEN,SGEN,BTX,CYCC,FENC,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:00.633876	2020-10-11 23:38:31.023428
CBAN	Colony Bankcorp Inc.	Colony Bankcorp Inc is a bank holding company. The company, through its subsidiary, provides traditional banking products and services to commercial and consumer customers in its markets.	{"Financial Services","Banks - Regional - US",Banks}	{UBSI,LION,ABCB,SGB,STBZ,UCBI,UBSH,SNV}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:00.65451	2020-10-11 23:38:31.023007
CBMG	Cellular Biomedicine Group Inc.	Cellular Biomedicine Group Inc is a part of the healthcare sector. It is engaged in the development of new treatments for cancerous and degenerative diseases.	{Healthcare,Biotechnology}	{AGIO,MGNX,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:00.668458	2020-10-11 23:38:31.023428
CBSHP	Commerce Bancshares Inc. Depositary Shares each representing a 1/1000th interest of 6.00% Series B Non-Cumulative Perpetual Preferred Stock	Commerce Bancshares Inc is engaged in banking business, providing a range of retail, corporate, investment, trust, and asset management products and services to individuals and businesses.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.678564	2020-10-11 23:38:31.023428
CCD	Calamos Dynamic Convertible & Income Fund	Calamos Dynamic Convertible and Income Fund is a diversified, closed-end management investment company. The Fund's investment objective is to provide total return through a combination of current income and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.693702	2020-10-11 23:38:31.024353
CCMP	Cabot Microelectronics Corporation	Cabot Microelectronics Corp is the supplier of polishing slurries & a growing polishing pad, used in the manufacture of integrated circuit devices within the semiconductor industry, in a process called chemical mechanical planarization.	{Technology,Semiconductors}	{INTC,COHR,CBT,PKE,ROG,MTRN,XPER,RBCN,DWDP,AVT,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:00.707041	2020-10-11 23:38:31.023428
CCS	Century Communities Inc.	Century Communities Inc is engaged in the development, design, construction, marketing & sale of single-family attached & detached homes in metropolitan areas in Colorado, Austin & San Antonio, Texas, Houston, Texas, Las Vegas, Nevada & Atlanta, Georgia.	{"Consumer Cyclical","Residential Construction","Homebuilding & Construction"}	{PHM,MHO,LEN,KBH,NWHM}	Homebuilding & Construction	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:00.715664	2020-10-11 23:38:31.02582
CDL	Compass EMP US Large Cap High Dividend 100 Volatil	The investment seeks to provide investment results that track the performance of the Nasdaq Victory US Large Cap High Dividend 100 Volatility Weighted Index.\n The fund seeks to achieve its investment objective by investing at least 80% of its assets directly or indirectly in the securities included in the Nasdaq Victory US Large Cap High Dividend 100 Volatility Weighted Index. The index universe begins with the stocks included in the Nasdaq Victory US Large Cap 500 Volatility Weighted Index, a volatility weighted index comprised of the 500 largest U.S. companies by market capitalization with positive earnings in each of the four most recent quarters.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:00.729918	2020-10-11 23:38:31.024353
CDW	CDW Corporation	CDW Corp provides IT solutions to small, medium and large business, government, education and healthcare customers in the U.S. and Canada, and the UK. Its offerings include mobility, security, and data center optimization.	{Technology,"Information Technology Services","Application Software"}	{NSIT,CNXN,AAPL,TECD,PLUS,PCMI,SYX,SNX,HPQ,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:00.745404	2020-10-11 23:38:31.02582
CEL	Cellcom Israel Ltd.	Cellcom Israel Ltd provides wireless service in Israel. It offers basic voice services and data services such as web browsing and music downloads. It is also developing a next-generation wireless service that supports higher data capacity.	{"Communication Services","Telecom Services"}	{IGLD,PTNR,ERIC,BCOM,TKC,TI}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:00.766195	2020-10-11 23:38:31.021426
CEN	Center Coast Brookfield MLP & Energy Infrastructure Fund	Center Coast MLP & Infrastructure Fund is a closed end management investment company. Its objective is to provide a high level of total return with an emphasis on distributions to shareholders.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.779536	2020-10-11 23:38:31.023007
CADC	China Advanced Construction Materials Group Inc.	China Advanced Construction Materials Group Inc through its subsidiaries manufactures construction materials for large scale infrastructure, commercial and residential developments.	{"Basic Materials","Building Materials"}	{CX,VMC,USCR}	Building Materials	Basic Materials	NASDAQ Capital Market	\N	2020-10-11 23:12:00.5937	2020-10-11 23:38:31.028166
CALA	Calithera Biosciences Inc.	Calithera Biosciences Inc is a clinical-stage biopharmaceutical company focused on discovering and developing novel small molecule drugs directed against tumor metabolism and tumor immunology targets for the treatment of cancer.	{Healthcare,Biotechnology}	{ORPN,PIRS,CSBR,ELOX,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:00.605027	2020-10-11 23:38:31.024353
CASA	Casa Systems Inc.	Casa Systems Inc provides software-centric infrastructure solutions that allow cable service providers to deliver voice, video and data services over a single platform at multi-gigabit speeds.	{Technology,"Communication Equipment"}	{NOK,MXL,RESN,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:00.63034	2020-10-11 23:38:31.02582
CATM	Cardtronics plc	Cardtronics PLC provides convenient automated consumer financial services through its network of automated teller machines and multi-function financial services kiosks. The Company operates in the U.S., Europe and rest of the world.	{Industrials,"Business Services"}	{EEFT,FISV,DBD,GPN,FIS,OCN,VNET,WEX,EVRI}	Business Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:00.647929	2020-10-11 23:38:31.02423
CBD	Companhia Brasileira de Distribuicao ADS	Companhia Brasileira De Distribuicao is a Brazilian retail and distribution group. It operates in the food retailing, clothing, home appliances, electronics and other products segment through its chain of hypermarkets, supermarkets, and department stores.	{"Consumer Cyclical","Department Stores","Retail - Apparel & Specialty"}	{KR,WMT,WMK}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:00.657511	2020-10-11 23:38:31.02582
CBM	Cambrex Corporation	Cambrex Corp is a biotechnology company focused on developing and commercializing new and generic therapeutics. It provides products and services that accelerate and improve the development and commercialization of new and generic therapeutics.	{Healthcare,Biotechnology}	{NAII,BASI,ACET,VIVO,CYAN,CDXS,CRL,TLGT,XLV}	Biotechnology	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:00.667208	2020-10-11 23:38:31.023007
CBT	Cabot Corporation	Cabot Corp is a specialty chemicals and performance materials Company. The Company offers rubber and specialty grade carbon blacks, fumed metal oxides, inkjet colorants, aerogel, cesium formate drilling fluids and activated carbon.	{"Basic Materials","Specialty Chemicals",Chemicals}	{XOM,GRA,GLW,KOP,ASPN,DWDP}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:00.67963	2020-10-11 23:38:31.02423
CCF	Chase Corporation	Chase Corp is a provider of industrial protective materials. Its products include tapes, laminates, sealants and coatings to be used as protective materials in the industrial segment and construction segment.	{Industrials,"Diversified Industrials","Industrial Products"}	{BMS,FUL,PPG,KWR,RPM,SHW,AZZ,MMM}	Industrial Products	Industrials	NYSE American	\N	2020-10-11 23:12:00.69591	2020-10-11 23:38:31.028166
CCU	Compania Cervecerias Unidas S.A.	United Breweries Co Inc is the largest producer of beer in Chile and the second- largest producer of beer in Argentina. The company also produces nonalcoholic beverages, wines, and spirits.	{"Consumer Defensive","Beverages - Brewers","Beverages - Alcoholic"}	{DEO,SAM,AKO.B,STZ.B,BUD,TAP.A}	Beverages - Alcoholic	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:00.721127	2020-10-11 23:38:31.023428
CDNA	CareDx Inc.	CareDx Inc is a commercial stage company. It develops, markets and delivers a diagnostic surveillance solution for heart transplant recipients to help clinicians make personalized treatment decisions throughout a transplant patient's lifetime.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{AGIO,TTOO,ALQA,XLV}	Medical Diagnostics & Research	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:00.732876	2020-10-11 23:38:31.031328
CEE	The Central and Eastern Europe Fund Inc. (The)	Central Europe & Russia Fund Inc is a closed end management investment company. Its objective is high current income and its secondary objective is capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.749869	2020-10-11 23:38:31.028166
CEIX	CONSOL Energy Inc.	CONSOL Energy Inc is a Canonsburg-based producer and exporter of high-BTU bituminous thermal coal. It owns and operates longwall mining operations in the nation and export terminals on the Eastern seaboard.	{"Basic Materials",Coal}	{}	Coal	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:00.763792	2020-10-11 23:38:31.028166
CENX	Century Aluminum Company	Century Aluminum Co is engaged in producing aluminum. It produces high purity aluminum, standard grade aluminum sow and tee bars, and value-added billet and foundry products which are traded internationally.	{"Basic Materials",Aluminum,"Metals & Mining"}	{KALU,ACH,AA}	Metals & Mining	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:12:00.780443	2020-10-11 23:38:31.024353
CETXW	Cemtrex Inc. Series 1 Warrant		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:00.796167	2020-10-11 23:38:31.031328
CFFI	C&F Financial Corporation	C&F Financial Corp through its subsidiaries offers banking and related financial services to both individuals and businesses. It operates in three business activities: Retail Banking, Mortgage Banking and Consumer Finance.	{"Financial Services","Banks - Regional - US",Banks}	{CCNE,IBKC,ABCB,COF,BBT,MTB,PNC,WFC,FUSB}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.81282	2020-10-11 23:38:31.021426
CASS	Cass Information Systems Inc	Cass Information Systems Inc provides payment and information processing services to large manufacturing, distribution and retail enterprises across the United States.	{Industrials,"Business Services"}	{EEFT,TSS,WEX,NEWT}	Business Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:00.636242	2020-10-11 23:38:31.028166
CBA	ClearBridge American Energy MLP Fund Inc.	ClearBridge American Energy MLP Fund Inc a non-diversified, closed-end management investment company. Its investment objective is to provide a high level of total return, with an equal emphasis on current distributions and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.652498	2020-10-11 23:38:31.028166
CBL	CBL & Associates Properties Inc.	CBL & Associates Properties Inc is a real estate investment trust. It owns, develops, acquires, leases, manages, and operates regional shopping malls, open-air and mixed-use centers, community centers and office properties and outlet centers.	{"Real Estate","REIT - Retail",REITs}	{TCO,BXP,CONE,CTT}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:00.664326	2020-10-11 23:38:31.025871
CBSH	Commerce Bancshares Inc.	Commerce Bancshares Inc is engaged in banking business, providing a range of retail, corporate, investment, trust, and asset management products and services to individuals and businesses.	{"Financial Services","Banks - Regional - US",Banks}	{UMBF,WTFC,GFED,GSBC,QCRH,EFSC,HWBK,MOFG,TCF,NBHC}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.675924	2020-10-11 23:38:31.023007
CCCL	China Ceramics Co. Ltd.	China Ceramics Co Ltd manufactures and sells ceramic tiles used for exterior siding and for interior flooring & design in buildings. Its product categories are: porcelain tiles, glazed tiles, glazed porcelain tiles, rustic tiles & polished glazed tiles.	{"Basic Materials","Building Materials"}	{AWI}	Building Materials	Basic Materials	NASDAQ Capital Market	\N	2020-10-11 23:12:00.689825	2020-10-11 23:38:31.02423
CDE	Coeur Mining Inc.	Coeur Mining Inc is a silver, gold, and zinc producer. It has mines located in the United States, Mexico and Bolivia, a silver streaming interest in Australia and exploration projects in Mexico and Argentina.	{"Basic Materials",Gold,"Metals & Mining"}	{ABX,K,HL,NEM,AG}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:00.723504	2020-10-11 23:38:31.02423
CDNS	Cadence Design Systems Inc.	Cadence Design Systems Inc develops system design enablement solutions that are used to design whole electronics systems, complex integrated circuits, and electronic devices. The firm provides maintenance services for its software, hardware, and others.	{Technology,"Software - Application","Application Software"}	{ANSS,ADSK,SNPS,PDFS,CEVA,TER,IMI,PTC,XPER,QTRH,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:00.733425	2020-10-11 23:38:31.02423
CE	Celanese Corporation	Celanese Corp is a chemical producer. It focus is to produce industrial specialties and acetyl which are used in various end markets including coatings and adhesives. It also produces cellulose derivatives.	{"Basic Materials",Chemicals}	{EMN,XOM,GRA,OMN,WLK,DWDP}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:00.749345	2020-10-11 23:38:31.021426
CELG	Celgene Corporation	Celgene Corp is a biotech company that discovers, develops, and commercializes therapies designed to treat cancer and immune-inflammatory related diseases. Its registered brands are Abraxane, Istodax, Otezla, Pomalyst, Revlimid, Thalomid, Vidaza.	{Healthcare,Biotechnology}	{AMGN,ALXN,GILD,ILMN,BIIB,JNJ,LLY,EPZM,OMED,XLRN,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:00.767015	2020-10-11 23:38:31.023428
CENTA	Central Garden & Pet Company Class A Common Stock Nonvoting	Central Garden & Pet Co is a marketer and producer of branded products and distributor of third-party products in the pet and lawn and garden supplies industries in the United States.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{PETS,HD,SMG,WMT}	Consumer Packaged Goods	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:00.780664	2020-10-11 23:38:31.028166
CETXP	Cemtrex Inc. Series 1 Preferred Stock	Cemtrex Inc is a diversified technology company. The Company provides electronic manufacturing services of electronic system assemblies, instruments and emission monitors for industrial processes, industrial air filtration & environmental control systems.	{Industrials,"Pollution & Treatment Controls","Industrial Products"}	{}	Industrial Products	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:00.797017	2020-10-11 23:38:31.024353
CFBI	Community First Bancshares Inc.	Community First Bancshares Inc operates as a mutual holding company which offers personal and commercial banking services. It offers savings accounts, credit card, mortgage, commercial and consumer loans, money transfers and other banking services.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:00.811766	2020-10-11 23:38:31.031328
CGBD	TCG BDC Inc.	TCG BDC Inc is an externally managed, non-diversified closed-end investment company. Its objective is to generate current income and capital appreciation primarily through debt investments in U.S. middle market companies.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.824855	2020-10-11 23:38:31.023428
FFHG	Formula Folios Hedged Growth	The investment seeks to provide capital growth.\n The fund seeks to achieve its investment objective by investing primarily in domestic equity securities of any market capitalization and U.S. Treasuries through other unaffiliated exchange traded funds ("ETFs") (including leveraged ETFs and inverse ETFs). The adviser allocates the fund's assets equally between two proprietary investment models. The adviser's first investment model identifies trends in the equity markets. The adviser's second investment model uses two sub-strategies.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:02.840352	2020-10-11 23:38:31.021426
CATY	Cathay General Bancorp	Cathay General Bancorp is a holding company for Cathay Bank, which offers various financial services for individuals, professionals, and small to medium-sized businesses in California. It provides loans, savings, time deposits, and others.	{"Financial Services","Banks - Regional - US",Banks}	{CVBF,HTBK,EWBC,CVCY,BSRR,PPBI,BMRC,FRC}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.648622	2020-10-11 23:38:31.023428
CBK	Christopher & Banks Corporation	Christopher & Banks Corp is a retailer of women's apparel and accessories. The company through its wholly owned subsidiaries operates retail stores selling women's apparel and accessories in the United States.	{"Consumer Cyclical","Apparel Stores","Retail - Apparel & Specialty"}	{CHS,CATO,NWY,ASNA,LB}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:00.662422	2020-10-11 23:38:31.021426
CBOE	Cboe Global Markets Inc.	Cboe Global Markets Inc is an exchange holding company. It offers trading options on various market indexes, futures, U.S. and European equities, exchange-traded products, and multi-asset volatility products.	{"Financial Services","Financial Exchanges","Brokers & Exchanges"}	{NDAQ,MKTX,CME,ICE}	Brokers & Exchanges	Financial Services	Cboe Global Markets EDGX	\N	2020-10-11 23:12:00.673364	2020-10-11 23:38:31.031328
CBTX	CBTX Inc.	CBTX Inc is a bank holding company that operates through its wholly-owned subsidiary, CommunityBank of Texas. It focuses on providing commercial banking solutions to small and mid-sized businesses and professionals.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.686284	2020-10-11 23:38:31.028166
CCK	Crown Holdings Inc.	Crown Holdings Inc is one of the world's largest producer of metal packaging products. The company manufactures beverage cans, metal food cans, and closures as well as aerosol cans. It operates mainly in US and Europe.	{"Consumer Cyclical","Packaging & Containers"}	{SLGN,ATR,GEF.B,BLL,OI,BUD}	Packaging & Containers	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:00.70046	2020-10-11 23:38:31.031328
CCRC	China Customer Relations Centers Inc.	China Customer Relations Centers Inc is a BPO service provider. It provides voice based BPO services to different industries such as E- commerce, telecommunications, financial services and manufacturing.	{Industrials,"Business Services"}	{NEWT,ITT}	Business Services	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:00.713107	2020-10-11 23:38:31.031328
CDEV	Centennial Resource Development Inc.	Centennial Resource Development Inc is an independent oil and natural gas company focused on the development and acquisition of unconventional oil and associated liquids-rich natural gas reserves in the Permian Basin.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{PUMP}	Oil & Gas - E&P	Energy	NASDAQ Capital Market	\N	2020-10-11 23:12:00.724401	2020-10-11 23:38:31.02582
CDTX	Cidara Therapeutics Inc.	Cidara Therapeutics Inc mainly develops anti-infective drugs in the United States. It attends to serious fungal infections through its drug candidates and proprietary immunotherapy technology platform, Cloudbreak.	{Healthcare,Biotechnology}	{SPPI,ECYT,CBMG,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:00.737234	2020-10-11 23:38:31.024353
CEF	Sprott Physical Gold and Silver Trust Units	Central Fund of Canada Ltd is an investment company. Its aims to invest the shareholder's funds in long-term holdings of gold and silver bullion for capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE Arca	\N	2020-10-11 23:12:00.753712	2020-10-11 23:38:31.028166
CELC	Celcuity Inc.	Celcuity Inc is a cellular analysis company. The company is discovering new cancer sub-types and commercializing diagnostic tests.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{XLV}	Medical Diagnostics & Research	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:00.76483	2020-10-11 23:38:31.025871
CEMI	Chembio Diagnostics Inc.	Chembio Diagnostics Inc develops, manufactures, and commercializes point of care diagnostic tests that detect infectious diseases. Its products are rapid tests for the detection of HIV 1/2 antibodies, and for the detection of HIV and Syphilis antibodies.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{QDEL,OSUR,TRIB,VRML,BIO,RIOT,XLV}	Medical Diagnostics & Research	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:00.778099	2020-10-11 23:38:31.025871
CERN	Cerner Corporation	Cerner Corp is a supplier of healthcare information technology and offers software, professional services, medical device integration, remote hosting and employer health, and wellness services. The Domestic segment generates maximum revenue.	{Technology,"Health Information Services","Application Software"}	{MDRX,OMCL,CPSI,HMSY,STRM,GE,MCK,MMM,DXC,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:00.791378	2020-10-11 23:38:31.028166
CF	CF Industries Holdings Inc.	CF Industries Holdings Inc is a leading producer and distributor of nitrogen fertilizers. The company operates various nitrogen facilities in America UK, and Canada. It primarily serves the international agriculture sector.	{"Basic Materials","Agricultural Inputs",Agriculture}	{SQM,SMG,CGA,UAN}	Agriculture	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:00.806416	2020-10-11 23:38:31.02582
CFR	Cullen/Frost Bankers Inc.	Cullen/Frost Bankers Inc is a financial holding company and a bank holding company. It provides a range of products and services throughout Texas markets, with a concentration on the major metropolitan areas.	{"Financial Services","Banks - Regional - US",Banks}	{FFIN,IBOC,SBSI,BOKF,HWC,TCBI,PB,IBTX,LTXB}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.819052	2020-10-11 23:38:31.02423
CHCO	City Holding Company	City Holding Co, through its subsidiary, provides banking, trust and investment management and other financial solutions. It provides commercial banking, consumer banking, mortgage banking and wealth management and trust services.	{"Financial Services","Banks - Regional - US",Banks}	{PFBI,HBAN,UBSI,SMMF,BBT,PNC,WFC,STI,JPM}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.840754	2020-10-11 23:38:31.024353
CBFV	CB Financial Services Inc.	CB Financial Services Inc, through its subsidiary Community Bank operates as a commercial bank offers residential & commercial real estate loans, industrial loans, consumer loans, as well as a variety of deposit products for individuals & businesses.	{"Financial Services","Banks - Regional - US",Banks}	{MPB,MOFG}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:00.657505	2020-10-11 23:38:31.031328
CBON	VanEck Vectors ChinaAMC China Bond	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the ChinaBond China High Quality Bond Index.\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The index is comprised of fixed-rate, Renminbi ("RMB")-denominated bonds issued in the People's Republic of China ("China" or the "PRC") by Chinese credit, governmental and quasi-governmental (e.g., policy banks) issuers ("RMB Bonds"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.67466	2020-10-11 23:38:31.028166
CCA	MFS California Municipal Fund of Beneficial Interest	MFS California Municipal Fund is a closed-end fund which seeks to earn high current income exempt from federal income tax and California personal income tax as well as capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:00.68811	2020-10-11 23:38:31.025871
CCM	Concord Medical Services Holdings Limited ADS (Each represents three)	Concord Medical Services Holdings Ltd owns and operates radiotherapy and diagnostic imaging centers in China. The company leases radiotherapy and diagnostic imaging equipment and provides management services to hospitals.	{Healthcare,"Medical Care","Health Care Providers"}	{CYH,THC,KANG,XLV}	Health Care Providers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:00.701576	2020-10-11 23:38:31.02582
CCXI	ChemoCentryx Inc.	ChemoCentryx Inc is a biopharmaceutical company. It is engaged in discovering, developing and commercializing orally-administered therapeutics to treat autoimmune diseases, inflammatory disorders and cancer.	{Healthcare,Biotechnology}	{AMGN,BIIB,GSK,BMY,JNJ,PFE,MRK,TEVA,ABBV,FBIO,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:00.72283	2020-10-11 23:38:31.028166
CDOR	Condor Hospitality Trust Inc.	Condor Hospitality Trust Inc is a self-administered real estate investment trust. The company is engaged in the investment and ownership of high quality select service, limited service, extended stay, and compact full service hotels.	{"Real Estate","REIT - Hotel & Motel",REITs}	{HST,AHT,PEB,CHSP,RLJ,INN,HT,HPT}	REITs	Real Estate	NYSE American	\N	2020-10-11 23:12:00.733761	2020-10-11 23:38:31.028166
CECE	CECO Environmental Corp.	CECO Environmental Corp along with its subsidiaries, is an environmental technology company providing air pollution control products and services.	{Industrials,"Pollution & Treatment Controls","Industrial Products"}	{ENTG,FTEK,DCI}	Industrial Products	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:00.74926	2020-10-11 23:38:31.02423
CEI	Camber Energy Inc.	Camber Energy Inc is an independent oil and gas company. It is engaged in the development of crude oil and natural gas in Texas and Oklahoma.	{Energy,"Oil & Gas Drilling","Oil & Gas - Drilling"}	{}	Oil & Gas - Drilling	Energy	NYSE American	\N	2020-10-11 23:12:00.762832	2020-10-11 23:38:31.023007
CENT	Central Garden & Pet Company	Central Garden & Pet Co is a marketer and producer of branded products and distributor of third-party products in the pet and lawn and garden supplies industries in the United States.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{PETS,HD,SMG,WMT}	Consumer Packaged Goods	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:00.77904	2020-10-11 23:38:31.021426
CETV	Central European Media Enterprises Ltd.	Central European Media Enterprises Ltd is a media and entertainment company. It develops and operates television channels and stations in Central and Eastern Europe.	{"Consumer Cyclical","Broadcasting - TV",Entertainment}	{TV,DIS,CBS,VIAB}	Entertainment	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:00.792701	2020-10-11 23:38:31.02582
CEY	VictoryShares Emerging Market High Div Volatility	The investment seeks to provide investment results that track the performance of the Nasdaq Victory Emerging Market High Dividend 100 Volatility Weighted Index before fees and expenses.\n The fund seeks to achieve its investment objective by investing, under normal market conditions, at least 80% of its assets directly or indirectly in the securities included in the Nasdaq Victory Emerging Market High Dividend 100 Volatility Weighted Index (the "index"). The index identifies the 100 highest dividend yielding stocks in the Nasdaq Victory Emerging Market 500 Volatility Weighted Index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:00.80661	2020-10-11 23:38:31.028166
CFRX	ContraFect Corporation	ContraFect Corp is a biotechnology company engaged in the development of protein and antibody therapeutic products for life-threatening infectious diseases. The Company's product candidates are CF-301, CF-404.	{Healthcare,Biotechnology}	{VIVO,VTL,ALDX,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:00.819847	2020-10-11 23:38:31.02582
CHA	China Telecom Corp Ltd ADS	China Telecom Corp Ltd offers a range of wireline and mobile telecommunications services including wireline voice, mobile voice, the Internet, telecommunications network resource services, value-added services, and other related services.	{"Communication Services","Telecom Services"}	{CHU,CHL}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:00.836459	2020-10-11 23:38:31.02582
CCBG	Capital City Bank Group	Capital City Bank Group Inc operates through its subsidiary Capital City Bank which provides traditional deposit and credit services, asset management, mortgage banking, merchant services, bank cards, data processing, and securities brokerage services.	{"Financial Services","Banks - Regional - US",Banks}	{CSFL,BBT,USB,BKU,SBCF,OPHC}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.689922	2020-10-11 23:38:31.02582
CCLP	CSI Compressco LP	CSI Compressco LP provides compression services and equipment for natural gas and oil production, gathering, transportation, processing, and storage. It also sells standard and custom designed compressor packages and oilfield fluid pump systems.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{SPN,SLB,HAL,OII,TTI,BHGE}	Oil & Gas - Services	Energy	Nasdaq Global Select	\N	2020-10-11 23:12:00.701683	2020-10-11 23:38:31.024353
CCT	Corporate Capital Trust Inc.	Corporate Capital Trust Inc is a non-diversified closed-end management investment company. Its investment objective is to provide its shareholders with income and long-term capital appreciation, by investing in the debt of privately owned U.S. companies.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.715757	2020-10-11 23:38:31.025871
CDMO	Avid Bioservices Inc.	Avid Bioservices Inc is engaged in development and cGMP manufacturing of biopharmaceutical products derived from mammalian cell culture.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:00.731957	2020-10-11 23:38:31.025871
CDXC	ChromaDex Corporation	ChromaDex Corp is a natural products company. The company discovers, acquires, develops and commercializes patented and proprietary ingredient technologies that address the dietary supplement, food, beverage, skincare and pharmaceutical markets.	{Healthcare,Biotechnology}	{NAII,SGMO,EXEL,SPEX,ACER,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:00.746217	2020-10-11 23:38:31.023428
CEFS	Exchange Listed Funds Trust	The investment seeks to provide capital appreciation and dividend income.\n The fund is an actively managed exchange-traded fund ("ETF") that seeks to achieve its investment objective by normally investing at least 80% of its net assets, plus the amount of any borrowings for investment purposes, in securities issued by closed-end funds (the "underlying funds"). The advisor expects to invest in underlying funds operated by a diversified group of closed-end fund managers ("underlying fund managers"). The fund normally invests in underlying funds that primarily pursue high income opportunities.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:00.76164	2020-10-11 23:38:31.031328
CEM	ClearBridge Energy MLP Fund Inc.	Legg Mason Partners Fund Advisor, LLC acts as an investment adviser for each funds or master funds of Legg Mason.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.772182	2020-10-11 23:38:31.028166
CET	Central Securities Corporation	Central Securities Corp is a closed-end management investment company. Its investment objective is the long-term growth of capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:00.792373	2020-10-11 23:38:31.025871
CEZ	Victory CEMP Emerging Market Volatility Wtd Index	The investment seeks to provide investment results that track the performance of the Nasdaq Victory Emerging Market 500 Volatility Weighted Index before fees and expenses.\n The fund seeks to achieve its investment objective by investing, under normal market conditions, at least 80% of its assets directly or indirectly in the securities included in the Nasdaq Victory Emerging Market 500 Volatility Weighted Index (the "index"), an unmanaged, volatility weighted index maintained exclusively by the index provider. The index combines fundamental criteria with individual security risk control achieved through volatility weighting of individual securities.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:00.806409	2020-10-11 23:38:31.02423
CFMS	Conformis Inc.	Conformis Inc is a medical technology company which uses iFit Image-to-Implant technology platform to develop, manufacture and sells joint replacement implants that are individually sized and shaped to fit each patient's unique anatomy.	{Healthcare,"Medical Devices"}	{MASI,NEOG,DXCM,XLV}	Medical Devices	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:00.818427	2020-10-11 23:38:31.024353
CGO	Calamos Global Total Return Fund	Calamos Global Total Return Fund is a closed-end management investment company. Its investment objective is to provide total return through a combination of capital appreciation and current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.833597	2020-10-11 23:38:31.023007
CHDN	Churchill Downs Incorporated	Churchill Downs Inc is a gambling company operating the Churchill Downs racetrack. The reportable business segments are racing, casinos, twin spires, corporate, and other investments.	{"Consumer Cyclical",Gambling,"Travel & Leisure"}	{PENN,ISCA,SGMS,TRK,DVD}	Travel & Leisure	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:00.844728	2020-10-11 23:38:31.023007
CHFC	Chemical Financial Corporation	Chemical Financial Corp is a financial services provider. The firm conducts its business of commercial banking through its primary subsidiary, Chemical Bank, which offers banking and fiduciary products.	{"Financial Services","Banks - Regional - US",Banks}	{MBWM,MCBC,FITB,HBAN,MBTF,MFNC,IBCP}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.856413	2020-10-11 23:38:31.031328
CHIQ	Global X China Consumer	The investment seeks to provide investment results that correspond generally to the price and yield performance, before fees and expenses, of the Solactive China Consumer Total Return Index.\n The fund invests at least 80% of its total assets in the securities of the underlying index and in American Depositary Receipts ("ADRs") and Global Depositary Receipts ("GDRs") based on the securities in the underlying index. The underlying index is designed to measure the equity performance of the investable universe of companies in the consumer sector of the Chinese economy, as defined by Solactive AG. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.86665	2020-10-11 23:38:31.02582
CDTI	CDTI Advanced Materials Inc.	CDTi Advanced Materials Inc is a cleantech emission control company. It designs, develops and manufactures sustainable solutions to reduce emissions of on-and off-road engine applications.	{Industrials,"Pollution & Treatment Controls","Industrial Products"}	{}	Industrial Products	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:00.734866	2020-10-11 23:38:31.028166
CDXS	Codexis Inc.	Codexis Inc is a biotech company. It develops proprietary biocatalysts-enzymes or microbes that initiate or accelerate chemical reactions. It offers its products to the pharmaceutical industry. The company also develops advanced biofuels.	{Healthcare,Biotechnology}	{BCPC,PFE,RDS.A,MRK,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:00.745367	2020-10-11 23:38:31.031328
CELGZ	Celgene Corporation Contingent Value Right	Celgene Corp is a biotech company that discovers, develops, and commercializes therapies designed to treat cancer and immune-inflammatory related diseases. Its registered brands are Abraxane, Istodax, Otezla, Pomalyst, Revlimid, Thalomid, Vidaza.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:00.769961	2020-10-11 23:38:31.02582
CETX	Cemtrex Inc.	Cemtrex Inc is a diversified technology company. The Company provides electronic manufacturing services of electronic system assemblies, instruments and emission monitors for industrial processes, industrial air filtration & environmental control systems.	{Industrials,"Pollution & Treatment Controls","Industrial Products"}	{ISNS,CECE,FTEK,KEYS}	Industrial Products	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:00.793078	2020-10-11 23:38:31.023007
CFO	VictoryShares US 500 Enhanced Volatility Wtd ETF	The investment seeks to provide investment results that track the performance of the Nasdaq Victory US Large Cap 500 Long/Cash Volatility Weighted Index before fees and expenses.\n The fund seeks to achieve its investment objective by investing, under normal market conditions, at least 80% of its assets directly or indirectly in the securities included in the Nasdaq Victory US Large Cap 500 Long/Cash Volatility Weighted Index, an unmanaged, volatility weighted index maintained exclusively by the index provider. The index identifies the 500 largest U.S. stocks by market capitalization measured at the time the index's constituent securities are determined.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:00.818528	2020-10-11 23:38:31.028166
CGW	Invesco S&P Global Water Index	The investment seeks investment results that correspond generally to the performance, before the fund's fees and expenses, of an equity index called the S&P Global Water Index.\n The fund will invest at least 90% of its total assets in common stock and American depositary receipts ("ADRs") that comprise the index and depositary receipts representing common stocks included in the index. At each rebalancing the index is comprised of 50 equity securities selected, based on investment and other criteria, from a universe of companies listed on global developed market exchanges. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.835859	2020-10-11 23:38:31.02423
CHE	Chemed Corp	Chemed Corp through its subsidiaries, provides hospice and palliative care services to its patients through a network of physicians, registered nurses, home health aides, social workers, clergy and volunteers and also plumbing and drain cleaning services.	{Healthcare,"Medical Care","Health Care Providers"}	{AMED,LHCG,XLV}	Health Care Providers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:00.845895	2020-10-11 23:38:31.02423
CHGX	Change Finance U.S. Large Cap Fossil Fuel Free	The investment seeks to track the performance, before fees and expenses, of the Change Finance Diversified Impact U.S. Large Cap Fossil Fuel Free Index.\n Under normal circumstances, at least 80% of the fund's total assets will be invested in the component securities of the index. The index was developed by Change Finance, PBC, the fund's investment adviser and index provider, and uses an objective, rules-based methodology to measure the performance of an equal-weighted portfolio of approximately 100 large cap U.S.-listed companies that meet a diverse set of environmental, social, and governance standards.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.856947	2020-10-11 23:38:31.02423
CHL	China Mobile Limited	China Mobile Ltd is a telecom operator in China. It provides cellular telecommunications and related services. The company offers voice services, voice value-added services and telecommunications network planning.	{"Communication Services","Telecom Services"}	{ERIC,CHU,CHA,VEON}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:00.874246	2020-10-11 23:38:31.023428
CHN	China Fund Inc. (The)	The China Fund Inc is a non-diversified, closed-end management investment company. Its investment objective is long-term capital appreciation which it seeks to achieve by investing in equity securities with trading in China and outside China.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.885334	2020-10-11 23:38:31.025871
CHSCO	CHS Inc Class B Cumulative Redeemable Preferred Stock	CHS Inc is engaged in providing grain, foods, and energy resources to businesses and consumers. Its segments include Energy, Agriculture, Corporate and Others.	{"Consumer Defensive","Farm Products","Consumer Packaged Goods"}	{}	Consumer Packaged Goods	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:00.893929	2020-10-11 23:38:31.024353
CHW	Calamos Global Dynamic Income Fund	Calamos Global Dynamic Income Fund operates as a closed end management investment company. Its investment strategy is to generate high level of current income with the objective of capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.905331	2020-10-11 23:38:31.023428
CDC	Compass EMP US EQIncome 100 Enhanced Volatity Wtd	The investment seeks to provide investment results that track the performance of the Nasdaq Victory US Large Cap High Dividend 100 Long/Cash Volatility Weighted Index before fees and expenses.\n The fund seeks to achieve its investment objective by investing, under normal market conditions, at least 80% of its assets directly or indirectly in the securities included in the Nasdaq Victory US Large Cap 100 High Dividend Long/Cash Volatility Weighted Index. The index identifies the 100 highest dividend yielding stocks in the Nasdaq Victory US Large Cap 500 Volatility Weighted Index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:00.723078	2020-10-11 23:38:31.021426
CDR	Cedar Realty Trust Inc.	Cedar Realty Trust Inc is a real estate investment trust. It owns, acquires, develops, redevelops, and manages grocery and drugstore anchored community shopping centers. The company generates revenue in the form of rent.	{"Real Estate","REIT - Retail",REITs}	{ANH,NNN,NSA}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:00.734778	2020-10-11 23:38:31.021426
CEFL	ETRACS Monthly Pay 2X Leveraged Closed-End Fund ETN	The investment seeks a return linked to the performance of the price return version of the ISE High Income™ Index. \n The ETRACS Monthly Pay 2xLeveraged Closed-End Fund ETN due December 10, 2043 (the “Securities”) are a series of Monthly Pay 2xLeveraged Exchange Traded Access Securities (ETRACS) linked to the price return. The index measures the performance of 30 U.S. closed-end funds, as selected and ranked by the index sponsor in accordance with the index methodology.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.761058	2020-10-11 23:38:31.024353
CELP	Cypress Energy Partners L.P. representing limited partner interests	Cypress Energy Partners LP provides saltwater disposal and other water and environmental services to U.S. onshore oil and natural gas producers and trucking companies.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{OII,ENSV,AROC}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:00.771113	2020-10-11 23:38:31.02423
CEO	CNOOC Limited	CNOOC Ltd is an oil and gas company. It is engaged in the exploration, development, production and sale of crude oil, natural gas and other petroleum products.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{E,TOT,XOM,BP,PTR,SNP,RDS.A}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:00.780422	2020-10-11 23:38:31.031328
CEW	WisdomTree Emerging Currency Strategy Fund	The investment seeks to achieve total returns reflective of both money market rates in selected emerging market countries available to foreign investors and changes to the value of these currencies relative to the U.S. dollar.\n The fund will invest, under normal circumstances, at least 80% of its net assets, plus the amount of any borrowings for investment purposes, in investments whose combined performance is tied economically to selected emerging market countries. It generally will maintain a weighted average portfolio maturity of 90 days or less with respect to the money market securities in its portfolio. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.806451	2020-10-11 23:38:31.021426
CFG	Citizens Financial Group Inc.	Citizens Financial Group Inc is a retail bank holding company. It provides retail and commercial banking products and services. Its business segments include Consumer Banking and Commercial Banking.	{"Financial Services","Banks - Regional - US",Banks}	{CATY,HWC,STI,PB}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.817724	2020-10-11 23:38:31.023007
CGNX	Cognex Corporation	Cognex Corp is a provider of machine vision products that capture and analyze visual information in order to automate tasks in manufacturing processes. Its products are used to automate the manufacture and tracking of discrete items.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{MKSI,CYBE,HURC,ZBRA,SMIT,AME,DHR,HON,ROP,NOVT,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:00.832534	2020-10-11 23:38:31.031328
CHCT	Community Healthcare Trust Incorporated	Community Healthcare Trust Inc is a real estate investment trust focusing on healthcare facilities in the United States. It leases its properties to hospitals or other healthcare service providers.	{"Real Estate","REIT - Healthcare Facilities",REITs}	{RYN,AMT,ALEX}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:00.844376	2020-10-11 23:38:31.028166
CHII	Global X China Industrials	The investment seeks to provide investment results that correspond generally to the price and yield performance, before fees and expenses, of the Solactive China Industrials Total Return Index.\n The fund invests at least 80% of its total assets in the securities of the underlying index and in American Depositary Receipts ("ADRs") and Global Depositary Receipts ("GDRs") based on the securities in the underlying index. The underlying index is designed to measure the equity performance of the investable universe of companies in the industrials sector of the Chinese economy. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.862795	2020-10-11 23:38:31.023428
CHMA	Chiasma Inc.	Chiasma Inc is a biopharmaceutical company which focuses on improving the lives of patients who face challenges associated with existing treatments for their rare and debilitating diseases.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:00.875256	2020-10-11 23:38:31.028166
CEA	China Eastern Airlines Corporation Ltd.	China Eastern Airlines Corp Ltd is an air carrier operating in China. The Company and its subsidiaries are engaged in the operation of civil aviation, including the provision of domestic, regional and international passenger airline services.	{Industrials,Airlines}	{ZNH,DAL}	Airlines	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:00.750176	2020-10-11 23:38:31.025871
CELH	Celsius Holdings Inc.	Celsius Holdings Inc is engaged in the development, marketing, sale and distribution of functional calorie-burning fitness beverages under the Celsius brand name.	{"Consumer Defensive","Beverages - Soft Drinks","Beverages - Non-Alcoholic"}	{}	Beverages - Non-Alcoholic	Consumer Defensive	NASDAQ Capital Market	\N	2020-10-11 23:12:00.770271	2020-10-11 23:38:31.031328
CEQP	Crestwood Equity Partners LP	Crestwood Equity Partners LP develops, acquires, owns or controls, and operates fee-based assets and operations within the energy midstream sector.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{CHK,FGP,APU,EPD,OXY,CVX,GLP,SPH,PBF}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:00.782764	2020-10-11 23:38:31.023428
CEVA	CEVA Inc.	CEVA Inc is a licensor of cellular, multimedia and connectivity technologies to semiconductor companies and OEMs serving the mobile, consumer, automotive and Internet of things markets.	{Technology,Semiconductors}	{INTC,MCHP,DSPG,NXPI,STM,ADI,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:00.796691	2020-10-11 23:38:31.023428
CFFN	Capitol Federal Financial Inc.	Capitol Federal Financial Inc is a financial services company operating through its subsidiary. It offers construction loans and other consumer loans and retail deposit accounts, including checking, savings, money market, IRA and certificates of deposit.	{"Financial Services","Savings & Cooperative Banks",Banks}	{SMBC,HMNF,CZWI,NBHC}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.815557	2020-10-11 23:38:31.023428
CGEN	Compugen Ltd.	Compugen Ltd is a therapeutic discovery company. The Company's pipeline consists of early-stage immuno-oncology programs aimed at harnessing the immune system to fight cancer.	{Healthcare,Biotechnology}	{SGEN,PDLI,PSTI,BTX,TEVA,GENE,EVGN,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:00.830275	2020-10-11 23:38:31.024353
CHEK	Check-Cap Ltd.	Check-Cap Ltd is a clinical stage medical diagnostics company engaged in the development of an ingestible imaging capsule that utilizes low-dose X-rays for CRC screening.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{RMD,XLV}	Medical Diagnostics & Research	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:00.851888	2020-10-11 23:38:31.023428
CHRS	Coherus BioSciences Inc.	Coherus BioSciences Inc is a biotechnology company that develops biosimilar therapeutics. The company seeks to ensure successful drug development and manufacturing under very aggressive timelines.	{Healthcare,Biotechnology}	{RGEN,ACOR,AFMD,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:00.885476	2020-10-11 23:38:31.031328
CHSCP	CHS Inc 8%  Cumulative Redeemable Preferred Stock	CHS Inc is engaged in providing grain, foods, and energy resources to businesses and consumers. Its segments include Energy, Agriculture, Corporate and Others.	{"Consumer Defensive","Farm Products","Consumer Packaged Goods"}	{}	Consumer Packaged Goods	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:00.895903	2020-10-11 23:38:31.025871
CIA	Citizens Inc. Class A ($1.00 Par)	Citizens Inc is an insurance holding company. The company through its subsidiaries provides life insurance to the individuals in the United States. It has life insurance, home service, and other non-insurance enterprises segments.	{"Financial Services","Insurance - Life"}	{SNFCA,TMK}	Insurance - Life	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.907468	2020-10-11 23:38:31.02582
CIO	City Office REIT Inc.	City Office REIT Inc is engaged in managing, operating, owning and acquiring office properties located within a specified target markets, located in metropolitan areas in the Southern and Western United States.	{"Real Estate","REIT - Office",REITs}	{INN,HASI,CHMI,SNR}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:00.934535	2020-10-11 23:38:31.023428
CIZ	Compass EMP International 500 Enhanced Volatility	The investment seeks to track the performance of the Nasdaq Victory International 500 Long/Cash Volatility Weighted Index before fees and expenses.\n The fund seeks to achieve its investment objective by investing, under normal market conditions, at least 80% of its assets directly or indirectly in the securities included in the Nasdaq Victory International 500 Long/Cash Volatility Weighted Index, an unmanaged, volatility weighted index maintained exclusively by the index provider. The index identifies the 500 largest foreign companies by market capitalization measured at the time the index's constituent securities are determined.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:00.950859	2020-10-11 23:38:31.031328
CLAR	Clarus Corporation	Clarus Corp is engaged in designing, manufacturing, and marketing outdoor performance equipment and apparel for climbing, mountaineering, backpacking, skiing, cycling, and other year-round outdoor recreation activities.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{}	Travel & Leisure	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:00.964888	2020-10-11 23:38:31.031328
CLGX	CoreLogic Inc.	CoreLogic, Inc. is a provider of consumer, financial and property information, analytics and services to business and government. It provides detailed coverage of property, mortgages, consumer credit, tenancy, risk and related performance information.	{Industrials,"Business Services"}	{INTX,RP,EFX,MCO,DNB,BR}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:00.980575	2020-10-11 23:38:31.02423
CDZI	Cadiz Inc.	Cadiz Inc US based company, acquires and develops land with water resources for various uses, including groundwater supply, groundwater storage and agriculture.	{Utilities,"Utilities - Regulated Water","Utilities - Regulated"}	{YORW,ARTNA,CTWS,MSEX,PCYO,CWT,AWR,WTR,AWK,SJW,XLU}	Utilities - Regulated	Utilities	NASDAQ Global Market	\N	2020-10-11 23:12:00.74708	2020-10-11 23:38:31.023007
CERC	Cerecor Inc.	Cerecor Inc is a clinical stage biopharmaceutical company which is engaged in developing drug candidates for patients with neurological and psychiatric disorders. The company owns a right to COMTi platform.	{Healthcare,Biotechnology}	{PETS,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:00.794807	2020-10-11 23:38:31.021426
CFA	Compass EMP U.S. 500 Volatility Weighted Index ETF	The investment seeks to provide investment results that track the performance of the Nasdaq Victory US Large Cap 500 Volatility Weighted Index before fees and expenses.\n The fund seeks to achieve its investment objective by investing, under normal market conditions, at least 80% of its net assets directly or indirectly in the securities included in the Nasdaq Victory US Large Cap 500 Volatility Weighted Index, an unmanaged, volatility weighted index maintained exclusively by the index provider. The index identifies the 500 largest U.S. stocks by market capitalization measured at the time the index's constituent securities are determined.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:00.809903	2020-10-11 23:38:31.025871
CFX	Colfax Corporation	Colfax Corp is an industrial manufacturing and engineering company. The company provides gas and fluid handling and fabrication services to customers around the world. It also produces equipment and fillers for welding and cutting applications.	{Industrials,"Diversified Industrials","Industrial Products"}	{LECO,GTLS,FLS,AP,GRC,GGG,IEX,SPXC,XYL,ITT}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:00.822601	2020-10-11 23:38:31.025871
CGIX	Cancer Genetics Inc.	Cancer Genetics Inc is an early-stage diagnostics company which develops, commercializes and provides molecular- and biomarker-based tests and services that enable physicians to personalize the clinical management of each individual patient.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{GHDX,ENZ,XLV}	Medical Diagnostics & Research	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:00.831926	2020-10-11 23:38:31.028166
CHD	Church & Dwight Company Inc.	Church & Dwight Co Inc is engaged in the household products industry. It manufactures and markets personal care products under brands like Oxiclean, Spinbrush, First Response and Trojan.	{"Consumer Defensive","Household & Personal Products","Consumer Packaged Goods"}	{OBCI,CAW,CLX,CL,ECL,FMC,JNJ,PG}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:00.846062	2020-10-11 23:38:31.028166
CHI	Calamos Convertible Opportunities and Income Fund	Calamos Convertible Opprtnty & Income operates as a closed end management investment company. Its investment objective is to provide total return through a combination of capital appreciation and current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.860891	2020-10-11 23:38:31.028166
CHMI	Cherry Hill Mortgage Investment Corporation	Cherry Hill Mortgage Investment Corp is a real estate investment company. It invests in residential mortgage assets with the objective of generating current yields for its stockholders.	{"Real Estate","REIT - Residential",REITs}	{RWT,MFA,HASI,CIM}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:00.878656	2020-10-11 23:38:31.024353
CHSP	Chesapeake Lodging Trust of Beneficial Interest	Chesapeake Lodging Trust is a self-advised real estate investment trust. The company invests in upper-upscale hotels in business and convention markets and select-service hotels in urban settings in USA.	{"Real Estate","REIT - Hotel & Motel",REITs}	{LHO,HST,AHT,DRH,RLJ,INN,HT,HPT}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:00.897136	2020-10-11 23:38:31.02582
CID	Compass EMP International High Dividend 100 Volati	The investment seeks to provide investment results that track the performance of the Nasdaq Victory Dividend Accelerator Index before fees and expenses.\n The fund seeks to achieve its investment objective by investing, under normal market conditions, at least 80% of its assets in securities included in the Nasdaq Victory Dividend Accelerator Index (the "index"). The index utilizes a rules-based approach designed to identify dividend paying stocks with a higher likelihood of future dividend growth.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:00.915276	2020-10-11 23:38:31.024353
CIR	CIRCOR International Inc.	Circor International Inc designs, manufactures and markets engineered products and sub-systems used in the Oil & Gas, power generation, aerospace, defense, and other industrial markets. Its segments are CIRCOR Energy & CIRCOR Aerospace & Defense.	{Industrials,"Diversified Industrials","Industrial Products"}	{SNHY,FLS,CW,DOV,GE,PH,MWA,ITT}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:00.936153	2020-10-11 23:38:31.021426
CIZN	Citizens Holding Company	Citizens Holding Co is a one-bank holding company. It is engaged in commercial and personal banking activities such as deposits, loans, letters of credit, mortgage loans, personal & corporate trust services, credit life insurance & title insurance.	{"Financial Services","Banks - Regional - US",Banks}	{TRMK,FBMS,RNST,FHN,SYBT,FUSB,BXS,PFBX}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:00.951611	2020-10-11 23:38:31.028166
CLBS	Caladrius Biosciences Inc.	Caladrius Biosciences Inc is a biopharmaceutical company. The company through its subsidiary is developing cellular therapeutics to treat diseases.	{Healthcare,Biotechnology}	{RGEN,ATHX,PSTI,CBMG,VCEL,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:00.968285	2020-10-11 23:38:31.025871
CFBK	Central Federal Corporation	Central Federal Corp is a financial holding company for CFBank. The company through its subsidiary offers a variety of financial services including residential lending and full service retail banking services and products.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:00.811639	2020-10-11 23:38:31.028166
CG	The Carlyle Group L.P.	The Carlyle Group LP is a global alternative asset management firm. It operates business across, Corporate Private Equity, Global Credit, Real Assets, and Investment Solutions segments.	{"Financial Services","Asset Management"}	{BX,OZM,APO,ARES}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.824527	2020-10-11 23:38:31.021426
CHCI	Comstock Holding Companies Inc.	Comstock Holding Co Inc is a multi-faceted real estate development and construction service company. It operates in three segments namely Homebuilding, Multi-family and Real Estate services segment.	{"Real Estate","Real Estate Services"}	{HGSH}	Real Estate Services	Real Estate	NASDAQ Capital Market	\N	2020-10-11 23:12:00.841429	2020-10-11 23:38:31.021426
CHH	Choice Hotels International Inc.	Choice Hotels International Inc is a hotel franchisor. Its lodging properties include Comfort Inn, Comfort Suites, Quality, Clarion, Sleep Inn, Econo Lodge, Rodeway Inn, MainStay Suites, Suburban Extended Stay Hotel, Cambria hotels and suites.	{"Consumer Cyclical",Lodging,"Travel & Leisure"}	{WYND,MAR,HLT,IHG}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:00.857848	2020-10-11 23:38:31.021426
CHKR	Chesapeake Granite Wash Trust representing beneficial interests in the Trust	Chesapeake Granite Wash Trust owns royalty interests in oil and natural gas producing wells. The Trust has royalty rights to horizontal wells in the Colony Granite Wash area in Washita County, Oklahoma.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{FCX,SDT}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:00.874045	2020-10-11 23:38:31.028166
CHSCL	CHS Inc Class B Cumulative Redeemable Preferred Stock Series 4	CHS Inc is engaged in providing grain, foods, and energy resources to businesses and consumers. Its segments include Energy, Agriculture, Corporate and Others.	{"Consumer Defensive","Farm Products","Consumer Packaged Goods"}	{}	Consumer Packaged Goods	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:00.891081	2020-10-11 23:38:31.023007
CIC	Capitol Investment Corp. IV Class A	Capitol Investment Corp IV is a blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:00.914248	2020-10-11 23:38:31.031328
CII	Blackrock Capital and Income Fund Inc.	Blackrock Enhanced Cap&Inc Fund, Inc is a United States based closed-end management investment company. Its objective is to provide total return through a combination of current income, current gains and long-term capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.927796	2020-10-11 23:38:31.02423
CIVBP	Civista Bancshares Inc. Depositary Shares Each Representing a 1/40th Interest in a 6.50% Noncumulative Redeemable Convertible Perpetual Preferred Share Series B	Civista Bancshares Inc is a financial holding company which is involved in collecting customer deposits, making loans, purchasing securities, and offering trust services to its clients.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:00.944195	2020-10-11 23:38:31.023007
CKH	SEACOR Holdings Inc.	SEACOR Holdings Inc owns, operates & markets equipment's in the offshore oil and gas, shipping and logistics industries.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{PHII,KEX,HOS,BRS,ERA,TDW}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:00.956539	2020-10-11 23:38:31.023007
CLDR	Cloudera Inc.	Cloudera Inc sells subscriptions and services for its data management, machine learning and advanced analytics platform. This platform delivers capabilities for data management, machine learning and advanced analytics.	{Technology,"Software - Application","Application Software"}	{PRGS,PXLW,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:00.973368	2020-10-11 23:38:31.024353
CLIRW	ClearSign Combustion Corporation Warrant		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:00.990191	2020-10-11 23:38:31.024353
CLR	Continental Resources Inc.	Continental Resources Inc is an independent crude oil and natural gas exploration, and production company. The company operates primarily in the Williston Basin and in southern Oklahoma.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{NFX,DNR,EOG,MRO,NBL,NOG,QEP,BCEI,WLL}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:01.009876	2020-10-11 23:38:31.025871
CMCL	Caledonia Mining Corporation Plc	Caledonia Mining Corp PLC and its subsidiaries are engaged in the operation of a gold mine and the exploration and development of mineral properties for precious metals. Its segments include Corporate, Zimbabwe, South Africa and Zambia.	{"Basic Materials",Gold,"Metals & Mining"}	{NSU,SOI,GEM}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:01.039807	2020-10-11 23:38:31.023007
CMFN	CM Finance Inc	CM Finance Inc is an externally managed, non-diversified closed-end management investment company. It is a specialty finance company that invests in the debt of U.S. middle-market companies.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:01.05167	2020-10-11 23:38:31.031328
CMSS	CM Seven Star Acquisition Corporation	CM Seven Star Acquisition Corp is a blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:01.069956	2020-10-11 23:38:31.031328
CNACR	Constellation Alpha Capital Corp. Right		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:01.08965	2020-10-11 23:38:31.031328
CNI	Canadian National Railway Company	Canadian National Railway Co is engaged in the transport sector. Its primary occupation is the rail and related transportation business.	{Industrials,Railroads,"Transportation & Logistics"}	{CP,GWR,NSC,UNP,KSU,CSX}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:01.104742	2020-10-11 23:38:31.021426
CHAU	Direxion Daily CSI 300 China A Share Bull 2X Shares	The investment seeks daily investment results, before fees and expenses, of 200% of the daily performance of the CSI 300 Index.\n The fund invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, ETFs that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index is a modified free-float market capitalization weighted index comprised of the largest and most liquid stocks in the Chinese A-share market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.838941	2020-10-11 23:38:31.023428
CHFS	CHF Solutions Inc.	CHF Solutions Inc focuses on commercializing the Aquadex FlexFlow System which is designed to remove excess fluid primarily excess salt and water) from patients suffering from fluid overload who have failed diuretic therapy.	{Healthcare,"Medical Devices"}	{ABMD,BABY,XLV}	Medical Devices	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:00.856494	2020-10-11 23:38:31.025871
CHKE	Cherokee Inc.	Cherokee Inc is a marketer and manager of a portfolio of fashion and lifestyle brands it owns, licensing the Cherokee, Hi-Tec, Magnum, 50 Peaks, Interceptor, Hawk Signature, Tony Hawk, Liz Lange, Completely Me by Liz Lange, Flip Flop Shops among others.	{"Consumer Cyclical","Apparel Stores","Retail - Apparel & Specialty"}	{SGC,ICON,KSS,TJX,VFC,DLA,PVH,RL}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:00.873745	2020-10-11 23:38:31.021426
CHS	Chico's FAS Inc.	Chico's FAS Inc is an apparel retailer of women's private branded clothing and accessories through products through retail stores, catalog and via Internet. The brands include Chico's, White House/Black Market, Soma Intimates, and Boston Proper.	{"Consumer Cyclical","Apparel Stores","Retail - Apparel & Specialty"}	{CATO,CBK,NWY,ASNA,FRAN,LB}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:00.889556	2020-10-11 23:38:31.023428
CHU	China Unicom (Hong Kong) Ltd	China Unicom (Hong Kong) Ltd provides cellular and fixed-line voice and related value-added services, broadband and other Internet-related services, and business and data communications services in the People's Republic of China.	{"Communication Services","Telecom Services"}	{SKM,CHL,BT,CHA}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:00.899836	2020-10-11 23:38:31.025871
CIM	Chimera Investment Corporation	Chimera Investment Corp is a real estate investment trust that invests in a portfolio of mortgage assets. Its investments include various government-sponsored agency such as residential mortgage-backed securities and commercial mortgage-backed securities.	{"Real Estate","REIT - Diversified",REITs}	{AGM,RWT,NLY,CHMI}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:00.929447	2020-10-11 23:38:31.025871
CIT	CIT Group Inc (DEL)	CIT Group Inc is a banking corporation provides financing, leasing and advisory services to middle-market companies in a diverse group of industries. The group offers retail banking through its subsidiary. It also syndicates and sells finance receivables.	{"Financial Services","Banks - Regional - US",Banks}	{SLM,RM,ALLY}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.94108	2020-10-11 23:38:31.028166
CLD	Cloud Peak Energy Inc	Cloud Peak Energy Inc is a coal producer in the USA. The company owns and operates three surface coal mines: the Antelope Mine, the Cordero Rojo Mine, and the Spring Creek Mine and two development projects in the Northern Powder River Basin.	{"Basic Materials",Coal}	{ARLP,CNX}	Coal	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:00.970573	2020-10-11 23:38:31.021426
CLI	Mack-Cali Realty Corporation	Mack-Cali Realty Corp is a part of the real estate sector. As a self-managed real estate investment trust, it is engaged in the leasing, management, acquisition and development works.	{"Real Estate","REIT - Office",REITs}	{VNO,HIW,BXP,SLG,LXP,BRT,CUZ,DEI}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:00.987317	2020-10-11 23:38:31.025871
CLRB	Cellectar Biosciences Inc.	Cellectar Biosciences Inc is a biopharmaceutical company. It is engaged in developing compounds for treatment and imaging of cancer. Its portfolio consists of PET imaging agents, optical imaging agents, and therapeutic agents.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:01.009567	2020-10-11 23:38:31.023428
CLUB	Town Sports International Holdings Inc.	Town Sports International Holdings Inc is an owner and operator of fitness clubs in the Northeast and Mid-Atlantic regions of the United States.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{CLCT,BPI}	Travel & Leisure	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:01.027384	2020-10-11 23:38:31.025871
CM	Canadian Imperial Bank of Commerce	Canadian Imperial Bank of Commerce is a Canadian-based financial institution. The company serves its clients through retail and business banking, wealth management and wholesale banking.	{"Financial Services","Banks - Global",Banks}	{BMO,BNS,RY,TD,WFC,BK,C}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.037917	2020-10-11 23:38:31.028166
CMCSA	Comcast Corporation Class A Common Stock	Comcast Corp is a media and technology company. It mainly involves in the cable industry and owns Comcast Cable, E!, NBCUniversal, and the NBC broadcast network.	{"Communication Services","Pay TV"}	{RCI,SJR,DISH,CTL,DIS,CBS,VIAB,FOXA}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:12:01.048821	2020-10-11 23:38:31.02582
CHIE	Global X China Energy	The investment seeks to provide investment results that correspond generally to the price and yield performance, before fees and expenses, of the Solactive China Energy Total Return Index.\n The fund invests at least 80% of its total assets in the securities of the underlying index and in American Depositary Receipts ("ADRs") and Global Depositary Receipts ("GDRs") based on the securities in the underlying index. The underlying index is designed to measure the equity performance of the investable universe of companies in the energy sector of the Chinese economy. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.860374	2020-10-11 23:38:31.023007
CHKP	Check Point Software Technologies Ltd.	Check Point Software Technologies Ltd develops, markets and supports a wide range of products and services for IT security. The company offers its customers a portfolio of network security, endpoint security, data security and management solutions.	{Technology,"Software - Application","Application Software"}	{CSCO,MSFT,CYRN,SYMC,FTNT,JNPR,IBM,PANW,FEYE,HPQ,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:00.873838	2020-10-11 23:38:31.02423
CHSCM	CHS Inc Class B Reset Rate Cumulative Redeemable Preferred Stock Series 3	CHS Inc is engaged in providing grain, foods, and energy resources to businesses and consumers. Its segments include Energy, Agriculture, Corporate and Others.	{"Consumer Defensive","Farm Products","Consumer Packaged Goods"}	{}	Consumer Packaged Goods	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:00.890907	2020-10-11 23:38:31.028166
CHUY	Chuy's Holdings Inc.	Chuy's Holdings Inc is a full-service restaurant concept. It offers menu of authentic, freshly-prepared Mexican and Tex Mex inspired food.	{"Consumer Cyclical",Restaurants}	{DAVE,RRGB,TXRH,BJRI,KONA,DRI,BDL,BLMN}	Restaurants	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:00.902164	2020-10-11 23:38:31.024353
CIBR	First Trust NASDAQ CEA Cybersecurity ETF	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the Nasdaq CTA Cybersecurity IndexSM (the "index").\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks or in depositary receipts that comprise the index. The index will include securities of companies classified as "cyber security" companies by the CTA. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:00.912464	2020-10-11 23:38:31.028166
CIL	Compass EMP International 500 Volatility Weighted	The investment seeks to provide investment results that track the performance of the Nasdaq Victory International 500 Volatility Weighted Index before fees and expenses.\n The fund seeks to achieve its investment objective by investing, under normal market conditions, at least 80% of its assets directly or indirectly in the securities included in the Nasdaq Victory International 500 Volatility Weighted Index, an unmanaged, volatility weighted index maintained exclusively by the index provider. The index identifies the 500 largest foreign companies by market capitalization measured at the time the index's constituent securities are determined.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:00.92978	2020-10-11 23:38:31.023007
CIVI	Civitas Solutions Inc.	Civitas Solutions Inc is a provider of home- and community-based health and human services to must-serve individuals with intellectual, developmental, physical or behavioral disabilities and other special needs.	{Healthcare,"Long-Term Care Facilities","Health Care Providers"}	{HCA,THC,XLV}	Health Care Providers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:00.949147	2020-10-11 23:38:31.024353
CL	Colgate-Palmolive Company	Colgate-Palmolive Co is a consumer products company. It provides services such as oral care, personal care, home care and pet nutrition.	{"Consumer Defensive","Household & Personal Products","Consumer Packaged Goods"}	{UL,AVP,CAW,CHD,CLX,KMB,PG}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:00.964636	2020-10-11 23:38:31.028166
CLDX	Celldex Therapeutics Inc.	Celldex Therapeutics Inc is a biopharmaceutical company. The company is engaged in the development and commercialization of immunotherapy technologies for the treatment of cancer and other difficult-to-treat diseases.	{Healthcare,Biotechnology}	{AMGN,SGEN,GSK,BMY,PFE,EBS,AGEN,VICL,CYTR,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:00.977387	2020-10-11 23:38:31.028166
CLIX	ProShares Long Online/Short Stores	The investment seeks investment results, before fees and expenses, that track the performance of the ProShares Long Online/Short Stores Index (the "index").\n The fund invests in financial instruments that ProShare Advisors believes, in combination, will enable the fund, before fees and expenses, to track the performance of the index. The index consists of long positions in the online retailers included in the ProShares Online Retail Index and short positions in the "bricks and mortar" retailers included in the Solactive-ProShares Bricks and Mortar Retail Store Index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.990741	2020-10-11 23:38:31.02423
CLTL	Invesco Treasury Collateral	The investment seeks to track the investment results (before fees and expenses) of the ICE U.S. Treasury Short Bond Index.\n The fund generally will invest at least 80% of its total assets in the components of the index. The index is designed to measure the performance of U.S. Treasury Obligations with a maximum remaining maturity of 12 months. "U.S. Treasury Obligations" refer to securities issued or guaranteed by the U.S. Treasury where the payment of principal and interest is backed by the full faith and credit of the U.S. government. They include U.S. Treasury notes, bills and bonds. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.026031	2020-10-11 23:38:31.024353
CHRW	C.H. Robinson Worldwide Inc.	C.H. Robinson Worldwide Inc is a non-asset-based third-party logistics provider. The company provides domestic freight brokerage services. It also operates a growing air and ocean forwarding unit and a legacy produce-sourcing operation.	{Industrials,"Integrated Shipping & Logistics","Transportation & Logistics"}	{HUBG,EXPD,JBHT,UPS,FDX,RRTS,RLGT}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:00.886222	2020-10-11 23:38:31.02423
CHTR	Charter Communications Inc.	Charter Communications Inc is a providers of cable services in the United States. The company offers entertainment, information and communications solutions to residential and commercial customers.	{"Communication Services","Pay TV"}	{CMCSA,DISH,VZ,T}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.900242	2020-10-11 23:38:31.031328
CIB	BanColombia S.A.	BanColombia SA operates as a financial institution in Panama, El Salvador, Puerto Rico, Guatemala, the Cayman Islands, and Peru. The company offers financial products and services to a diversified individual, corporate, and government customer.	{"Financial Services","Banks - Regional - Latin America",Banks}	{BNS,BAP,FRAN,SAN,BBVA,BCH,BBD,ITUB,BLX}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.911566	2020-10-11 23:38:31.023007
CIGI	Colliers International Group Inc.	Colliers International Group Inc is a commercial real estate firm. The company provides commercial real estate services to corporate and institutional clients. It also offers outsourcing and advisory services, lease brokerage, and sales brokerage.	{"Real Estate","Real Estate Services"}	{JLL,CBRE}	Real Estate Services	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:00.927574	2020-10-11 23:38:31.028166
CIVB	Civista Bancshares Inc.	Civista Bancshares Inc is a financial holding company which is involved in collecting customer deposits, making loans, purchasing securities, and offering trust services to its clients.	{"Financial Services","Banks - Regional - US",Banks}	{PEBO,UBCP,OVBC,UBOH,HBAN,WSBC,LCNB,FMNB,SBFG}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:00.941912	2020-10-11 23:38:31.02582
CKPT	Checkpoint Therapeutics Inc.	Checkpoint Therapeutics Inc is an immuno-oncology biopharmaceutical company which focuses on the acquisition, development, and commercialization of novel, non-chemotherapy, immune-enhanced combination treatments for patients with solid tumor cancers.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:00.957391	2020-10-11 23:38:31.028166
CLDC	China Lending Corporation	China Lending Corporation provides financial services through its subsidiaries in China. It caters to companies in various market fields such as supply chain finance, manufacturing, real estate, mineral, and energy.	{"Financial Services","Credit Services"}	{SAND,ONDK}	Credit Services	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:00.971477	2020-10-11 23:38:31.023007
CLH	Clean Harbors Inc.	Clean Harbors Inc along with its subsidiaries, provides environmental, energy and industrial services.	{Industrials,"Waste Management"}	{SRCL,ECOL,WM,CVA,VTNR,PESI}	Waste Management	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:00.98351	2020-10-11 23:38:31.021426
CLMT	Calumet Specialty Products Partners L.P.	Calumet Specialty Products Partners LP is engaged in processing crude oil into specialty products including lubricating oils, solvents, and waxes, and fuel products including gasoline, diesel, and jet fuel.	{Energy,"Oil & Gas Refining & Marketing","Oil & Gas - Refining & Marketing"}	{VLO,XOM,MUR,CVX,CVI,MPC,HFC,PSX,DK}	Oil & Gas - Refining & Marketing	Energy	Nasdaq Global Select	\N	2020-10-11 23:12:00.998842	2020-10-11 23:38:31.031328
CLSD	Clearside Biomedical Inc.	Clearside Biomedical Inc is a clinical biopharmaceutical company. It is engaged in developing drug therapies to treat blinding diseases of the eye. Its product candidates focuses on diseases affecting the retina.	{Healthcare,Biotechnology}	{NEO,NVTA,ONCS,NTRA,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:01.021633	2020-10-11 23:38:31.02423
CMC	Commercial Metals Company	Commercial Metals Co manufactures, recycles and markets steel and metal products and related materials and services. Its products and services include metal recycling, mill products, fabrication,construction products etc.	{"Basic Materials",Steel}	{SCHN,STLD,AKS,SIM,GGB,AP,MLI,NUE,WOR,X}	Steel	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:01.039735	2020-10-11 23:38:31.023428
CMPR	Cimpress N.V	Cimpress NV is a Dutch company that focuses on the mass customization of physical products, such as embroidered apparel, signage, marketing materials, and photo products.	{"Consumer Cyclical","Marketing Services","Advertising & Marketing Services"}	{LABL,INWK,SFLY,DLX,RRD}	Advertising & Marketing Services	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:01.062242	2020-10-11 23:38:31.028166
CMTA	Clementia Pharmaceuticals Inc.	Clementia Pharmaceuticals Inc is a clinical stage biopharmaceutical company. It is engaged in developing new treatments for people with ultra-rare bone disorders and other diseases.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:01.078603	2020-10-11 23:38:31.023428
CNACW	Constellation Alpha Capital Corp. Warrant		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:01.094319	2020-10-11 23:38:31.028166
CNK	Cinemark Holdings Inc Inc.	Cinemark Holdings Inc is a motion picture exhibitor operating in the United States. The company operates over 4,000 screens across the country mainly in midsize cities or suburbs of large cities.	{"Consumer Cyclical","Media - Diversified",Entertainment}	{RDI,MCS,AMC}	Entertainment	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:01.106944	2020-10-11 23:38:31.023428
CHNR	China Natural Resources Inc.	China Natural Resources Inc through its subsidiaries conducts acquisition and exploitation of mining rights, including the exploration, mineral extraction, processing and sale of iron, zinc & other nonferrous metals, and copper concentrate.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{HBM,WRN}	Metals & Mining	Basic Materials	NASDAQ Capital Market	\N	2020-10-11 23:12:00.885372	2020-10-11 23:38:31.021426
CI	Cigna Corporation	Cigna Corp operates in the healthcare industry. It provides medical, dental, disability, life and accident insurance products.	{Healthcare,"Health Care Plans"}	{HUM,UNH,AET,CNC,ANTM,XLV}	Health Care Plans	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:00.908101	2020-10-11 23:38:31.028166
CIG	Comp En De Mn Cemig ADS American Depositary Shares	Energy Company of Minas Gerais is a Brazilian power company that is engaged in generation, transmission and distribution and sale of electricity.	{Utilities,"Utilities - Diversified","Utilities - Regulated"}	{SID,EBR.B,ELP,FE,ED,CPL,VALE,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:00.927568	2020-10-11 23:38:31.031328
CIX	CompX International Inc.	Compx International Inc is engaged in manufacturing of security products used in the recreational transportation, postal, office and institutional furniture's; and also manufactures stainless steel exhaust systems, and throttle controls.	{Industrials,"Security & Protection Services","Consulting & Outsourcing"}	{NNBR,TKR,VHI,SWK,NL,EML,ALLE}	Consulting & Outsourcing	Industrials	NYSE American	\N	2020-10-11 23:12:00.948513	2020-10-11 23:38:31.025871
CLDT	Chatham Lodging Trust (REIT) of Beneficial Interest	Chatham Lodging Trust is an internally-managed real estate investment trust. The company is organized to invest in upscale extended-stay and premium-branded select-service hotels.	{"Real Estate","REIT - Hotel & Motel",REITs}	{HST,AHT,SHO,DRH,CHSP,RLJ,INN,HPT}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:00.977599	2020-10-11 23:38:31.023428
CLLS	Cellectis S.A.	Cellectis SA is a gene-editing company, employing its core proprietary technologies to develop products in the emerging field of immuno-oncology. Its two segment includes Therapeutics and Plants.	{Healthcare,Biotechnology}	{REGN,QURE,CALA,NVLN,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:00.991608	2020-10-11 23:38:31.02582
CLRBZ	Cellectar Biosciences Inc. Series A Warrants		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:01.015145	2020-10-11 23:38:31.031328
CLVS	Clovis Oncology Inc.	Clovis Oncology Inc is a biopharmaceutical company focused on acquiring, developing and commercializing cancer treatments in the United States, Europe and other international markets.	{Healthcare,Biotechnology}	{BMRN,CELG,AZN,JNJ,LLY,PFE,TSRO,ABBV,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:01.026031	2020-10-11 23:38:31.023007
CMBS	iShares CMBS Bond	The investment seeks to track the investment results of the Bloomberg Barclays U.S. CMBS (ERISA Only) Index.\n The index measures the performance of investment-grade commercial mortgage-backed securities ("CMBS"), which are classes of securities (known as "certificates") that represent interests in "pools" of commercial mortgages. The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.038378	2020-10-11 23:38:31.031328
CMCTP	CIM Commercial Trust Corporation Series L Preferred Stock	CIM Commercial Trust Corp is a real estate investment trust. The company along with its subsidiaries is engaged in acquiring, owning, and operating Class A and creative office properties in urban communities throughout the United States.	{"Real Estate","REIT - Office",REITs}	{}	REITs	Real Estate	NASDAQ Global Market	\N	2020-10-11 23:12:01.049592	2020-10-11 23:38:31.02423
CMP	Compass Minerals Intl Inc	Compass Minerals International Inc is associated with the mining industry. It produces minerals, including salt, sulfate of potash specialty fertilizer and magnesium chloride.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{IPI,CINR}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:01.059499	2020-10-11 23:38:31.025871
CMSSU	CM Seven Star Acquisition Corporation Unit	CM Seven Star Acquisition Corp is a blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:01.070511	2020-10-11 23:38:31.023007
CNA	CNA Financial Corporation	CNA Financial Corp is a property and casualty insurance company managing its core business into three segments Specialty, Commercial and International. Its services include risk management, information services, warranty and claims administration.	{"Financial Services","Insurance - Property & Casualty"}	{ACGL,HALL,HIG,WRB,L,RLI,WTM}	Insurance - Property & Casualty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.086954	2020-10-11 23:38:31.021426
CNHI	CNH Industrial N.V.	CNH Industrial NV is engaged in capital goods sector. It designs, manufactures and sells agricultural and construction equipment such as trucks, commercial vehicles, buses, engines and transmissions for vehicles and engines for marine applications.	{Industrials,"Farm & Construction Equipment","Farm & Construction Machinery"}	{NAV,AGCO,CAT,DE}	Farm & Construction Machinery	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:01.103434	2020-10-11 23:38:31.023007
CNP	CenterPoint Energy Inc (Holding Co)	CenterPoint Energy Inc owns and operates a portfolio of energy-related businesses. The company primarily provides transmission and distribution services and has natural gas distribution systems in the United States.	{Utilities,"Utilities - Regulated Gas","Utilities - Regulated"}	{OGE,OKE,SRE,ATO,MDU,D,OGS,SR,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:01.118714	2020-10-11 23:38:31.02423
CHSCN	CHS Inc Preferred Class B Series 2 Reset Rate	CHS Inc is engaged in providing grain, foods, and energy resources to businesses and consumers. Its segments include Energy, Agriculture, Corporate and Others.	{"Consumer Defensive","Farm Products","Consumer Packaged Goods"}	{}	Consumer Packaged Goods	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:00.892831	2020-10-11 23:38:31.028166
CHY	Calamos Convertible and High Income Fund	Calamos Convertible & High Income Fund is a diversified, closed-end management investment company. Its objective is to provide total return through a combination of capital appreciation and current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.906226	2020-10-11 23:38:31.02423
CIEN	Ciena Corporation	Ciena Corp is a network strategy and technology company. It provides network hardware, software and services that support the transport, switching, aggregation, service delivery and management of video, data, and voice traffic on communications networks.	{Technology,"Communication Equipment"}	{CSCO,ADTN,INFN,JNPR,CALX,XLK}	Communication Equipment	Technology	New York Stock Exchange	\N	2020-10-11 23:12:00.918198	2020-10-11 23:38:31.02582
CJ	C&J Energy Services Inc.	C&J Energy Services Inc provides support services to the oil & gas industry in the United States. Its array of services mainly includes hydraulic fracturing, wireline logging, pressure pumping and pressure testing.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{PUMP}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:00.952613	2020-10-11 23:38:31.021426
CLB	Core Laboratories N.V.	Core Laboratories NV is a provider of proprietary and patented reservoir description, production enhancement & reservoir management services to the oil and gas industry. The company operates in more than 50 countries.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{SLB,HAL,OII,HLX,FI,AROC}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:00.966711	2020-10-11 23:38:31.024353
CLFD	Clearfield Inc.	Clearfield Inc manufactures and sells fiber management and enclosure platform that consolidates, distributes and protects fiber as it moves from the inside plant to the outside plant and all the way to the home, business and cell site.	{Technology,"Communication Equipment"}	{PLPC,JCS,RFIL,OCC,FNSR,APH,GLW,EMR,BDC,TEL,XLK}	Communication Equipment	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:00.979845	2020-10-11 23:38:31.028166
CLM	Cornerstone Strategic Value Fund Inc.	Cornerstone Strategic Value Fund Inc is a closed-end management investment company. The Fund's investment objective is to seek long-term capital appreciation through investment in equity securities of U.S. and non-U.S. companies.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:00.993486	2020-10-11 23:38:31.023007
CLRG	IQ Chaikin U.S. Large Cap ETF	The investment seeks investment results that track (before fees and expenses) the price and yield performance of the Nasdaq Chaikin Power US Large Cap Index (the "underlying index").\n The underlying index is an equally weighted index of large-capitalization securities. The fund generally will invest in all of the securities that comprise its underlying index in proportion to their weightings in the underlying index. It invests, under normal circumstances, at least 80% of the value of its assets (net assets plus the amount of any borrowings for investment purposes) in securities of large-capitalization U.S. issuers.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.016234	2020-10-11 23:38:31.028166
CLW	Clearwater Paper Corporation	Clearwater Paper Corp is a producer of tissue and paperboard products. The company manufactures consumer tissue, away-from-home tissue, parent roll tissue, bleached paperboard and pulp at manufacturing facilities across the nation.	{"Basic Materials","Paper & Paper Products","Forest Products"}	{GLT,IP,KMB,PKG,NP,TIS}	Forest Products	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:01.028662	2020-10-11 23:38:31.028166
CMA	Comerica Incorporated	Comerica Inc is a financial services company. It operates in three business segments, The Business Bank, The Retail Bank, and Wealth Management. It operates in geographic markets including Michigan, California and Texas, Arizona and Florida.	{"Financial Services","Banks - Regional - US",Banks}	{HBAN,WSBC,KEY,BAC,CFR,PNC,WFC,JPM,RF}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.038594	2020-10-11 23:38:31.025871
CMF	iShares California Muni Bond	The investment seeks to track the investment results of the S&P California AMT-Free Municipal Bond IndexTM.\n The index measures the performance of the investment-grade segment of the California municipal bond market. The fund generally will invest at least 90% of its assets in the component securities of the index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents, including shares of money market funds advised by BFA or its affiliates, as well as in securities not included in the index, but which BFA believes will help the fund track the index. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.051357	2020-10-11 23:38:31.023428
CMO	Capstead Mortgage Corporation	Capstead Mortgage Corp operates as a self-managed real estate investment trust. The company together with its subsidiaries invests in leveraged portfolio of residential mortgage pass-through securities consisting of adjustable-rate mortgage securities.	{"Real Estate","REIT - Residential",REITs}	{NYMT,DX,MFA,IVR,PMT,ARR}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:01.059189	2020-10-11 23:38:31.02423
CINR	Ciner Resources LP representing Limited Partner Interests	Ciner Resources LP is engaged in the production of natural soda ash from its facility located in Green River, Wyoming.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{CMP,SLCA,MPVD}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:00.93493	2020-10-11 23:38:31.031328
CJJD	China Jo-Jo Drugstores Inc.	China Jo-Jo Drugstores Inc is a retail and wholesale distributor of pharmaceutical and other healthcare products in the People's Republic of China. Its segments include retail drugstores, online pharmacy, drug wholesale and herb farming.	{"Consumer Defensive","Pharmaceutical Retailers","Retail - Defensive"}	{PETS,CVS,RAD,WBA}	Retail - Defensive	Consumer Defensive	NASDAQ Capital Market	\N	2020-10-11 23:12:00.95487	2020-10-11 23:38:31.02423
CLCT	Collectors Universe Inc.	Collectors Universe Inc provides authentication and grading services to dealers and collectors of high-value coins, trading cards, event tickets, autographs and memorabilia.	{Industrials,"Business Services"}	{LQDT,BID,GME}	Business Services	Industrials	NASDAQ Global Market	\N	2020-10-11 23:12:00.969527	2020-10-11 23:38:31.02423
CLGN	CollPlant Holdings Ltd.	Collplant Holdings Ltd is a biotechnology company focused on regenerative medicine. It focused on developing & commercializing tissue repair products. Its products are based on recombinant human collagen produced from genetically-modified tobacco plants.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:00.980703	2020-10-11 23:38:31.031328
CLNC	Colony Credit Real Estate Inc. Class A	Colony Credit Real Estate Inc, formerly Colony NorthStar Credit Real Estate Inc is a commercial real estate investment trust. The company operates in industrial, healthcare, hospitality and opportunistic equity and debt industry internationally.	{"Real Estate","REIT - Office",REITs}	{}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:01.004425	2020-10-11 23:38:31.028166
CLRO	ClearOne Inc.	ClearOne Inc designs, develops and sells conferencing, collaboration, streaming and digital signage solutions for audio/voice and visual communication. Its products inlcude Digital Signage, Media Carts, Microphone , and UC Voice.	{Technology,"Communication Equipment"}	{CSCO,LOGI,PLT,VCRA,XLK}	Communication Equipment	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:01.018655	2020-10-11 23:38:31.021426
CLWT	Euro Tech Holdings Company Limited	Euro Tech Holdings Co Ltd markets and trades water and wastewater related process control, analytical and testing instruments, disinfection equipment, supplies and related automation systems.	{Industrials,"Pollution & Treatment Controls","Industrial Products"}	{RBA,NATI,DXPE,AIT,A,DHR,TMO}	Industrial Products	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:01.031291	2020-10-11 23:38:31.02582
CME	CME Group Inc.	CME Group Inc is an operator of financial exchanges. It offers products like futures & options and foreign exchange.	{"Financial Services","Financial Exchanges","Brokers & Exchanges"}	{NDAQ,MKTX,MSCI,ICE,CBOE,IBKR}	Brokers & Exchanges	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:01.054052	2020-10-11 23:38:31.021426
CMU	MFS Municipal Income Trust	Mfs High Yield Municipal Trust is a United States based diversified, closed-end management investment company. Its investment objective is to seek high current income exempt from federal income tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.080518	2020-10-11 23:38:31.028166
CNCR	Loncar Cancer Immunotherapy ETF	The investment seeks to track the total return performance, before fees and expenses, of the Loncar Cancer Immunotherapy Index.\n The index is composed of the common stock of approximately 30 pharmaceutical or biotechnology companies identified by the fund's index provider, as having a high strategic focus on the development of drugs that harness the body's own immune system to fight cancer. The adviser attempts to invest all, or substantially all, of its assets in the component securities and ADRs that make up the index. Normally it will invest at least 80% of its total assets in the component securities of the index. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.097188	2020-10-11 23:38:31.028166
CNNE	Cannae Holdings Inc.	Cannae Holdings Inc is a diversified holding company. The company holds investments in a variety of industries including Restaurants, Technology Enabled Healthcare Services, Financial Services and more.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.108779	2020-10-11 23:38:31.028166
CNXN	PC Connection Inc.	PC Connection Inc is a provider of information technology or IT solutions. It provides IT products including computer systems, software and peripheral equipment, networking communications that it purchases from manufacturers and distributors.	{Technology,"Computer Distribution","Computer Hardware"}	{NSIT,TECD,PCMI,OSTK,STMP,SYX,SNX,CDW,EVLV,HPQ,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:01.125594	2020-10-11 23:38:31.023007
COF	Capital One Financial Corporation	Capital One Financial Corp operates in the financial domain. Through its banking and non-banking subsidiaries, it markets financial products and services.	{"Financial Services","Credit Services"}	{CASH,DFS,BKU}	Credit Services	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.13838	2020-10-11 23:38:31.023007
COM	Direxion Auspice Broad Commodity Strategy	The investment seeks to provide total return that exceeds that of the Auspice Broad Commodity Index over a complete market cycle.\n The fund is an actively managed ETF that seeks to provide total return that exceeds that of the index. It will generally seek to maintain a portfolio of instruments similar to those included in the index by utilizing exchange-traded commodity futures contracts, swap contracts and investments in other investment companies or exchange-traded notes, thereby obtaining exposure to the commodities markets. The index is a rules-based index that attempts to capture trends in the commodity markets. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.155459	2020-10-11 23:38:31.023007
CIK	Credit Suisse Asset Management Income Fund Inc.	Credit Suisse Asset Mgmt Income Fund Inc is a closed-end management investment company. The investment objective of the fund is to seek current income through investment in debt securities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:00.928583	2020-10-11 23:38:31.024353
CISN	Cision Ltd.	Cision Ltd is a media communication technology and analytics company. It has created Cision Communications Cloud, a earned media cloud-based platform. Its solutions also include PR Newswire, Gorkana, PRWeb, Help a Reporter Out (HARO) and iContact.	{Technology,"Software - Application","Application Software"}	{XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:00.939218	2020-10-11 23:38:31.02423
CKX	CKX Lands Inc.	CKX Lands Inc owns land, timber, and mineral interests and collects income through its ownership in the form of oil and gas royalties, timber sales, and surface leases for farming, right of way and other uses.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{PHX,PBT,CRT,SBR}	Oil & Gas - E&P	Energy	NYSE American	\N	2020-10-11 23:12:00.963172	2020-10-11 23:38:31.02582
CLF	Cleveland-Cliffs Inc.	Cleveland-Cliffs Inc, formerly Cliffs Natural Resources Inc is an iron ore producer. The company operates mining properties in North America, United States, Canada and Australia. It also owns a large chromite project in Canada.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:00.979394	2020-10-11 23:38:31.02582
CLPR	Clipper Realty Inc	Clipper Realty Inc is a self-administered and self-managed real estate company that acquires, owns, manages, operates and repositions multi-family residential and commercial properties in the New York metropolitan area.	{"Real Estate","REIT - Diversified",REITs}	{NYMT,O,NTP}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:01.007161	2020-10-11 23:38:31.021426
CMCM	Cheetah Mobile Inc. American Depositary Shares each representing 10 Class	Cheetah Mobile Inc is a software applications provider. The company's products provide assistance in online marketing and mobile internet security, among others.	{Technology,"Software - Application","Application Software"}	{AAPL,BIDU,GOOGL,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:01.042633	2020-10-11 23:38:31.024353
CMG	Chipotle Mexican Grill Inc.	Chipotle Mexican Grill Inc together with its subsidiaries operates Chipotle Mexican Grill restaurants, which serves a menu of burritos, tacos, burrito bowls and salads, made using fresh ingredients.	{"Consumer Cyclical",Restaurants}	{PZZA,JACK,DENN,BJRI,MCD,YUM,DPZ,FRGI}	Restaurants	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:01.053383	2020-10-11 23:38:31.023007
CMRE	Costamare Inc. $0.0001 par value	Costamare Inc is an international owner of containerships. It provides marine transportation services by chartering its container vessels to liner operators under long, medium and short-term time charters.	{Industrials,"Integrated Shipping & Logistics","Transportation & Logistics"}	{DSX,SSW,DAC,GSL,ESEA,DCIX}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:01.064606	2020-10-11 23:38:31.02582
CMTL	Comtech Telecommunications Corp.	Comtech Telecommunications Corp is engaged in designing, developing, producing and marketing products, systems and services for communications solutions. It is engaged in two business segments, Commercial Solutions and Government Solutions Segment.	{Technology,"Communication Equipment"}	{GILT,KVHI,VSAT,FEIM,CAMP,GD,HRS,COL,LLL,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:01.079787	2020-10-11 23:38:31.025871
CNCE	Concert Pharmaceuticals Inc.	Concert Pharmaceuticals Inc is a clinical-stage biopharmaceutical company. It aims to discover and develop small-molecule drugs for new treatments to cure cystic fibrosis, inflammation, narcolepsy, residual schizophrenia, and Alzheimer's agitation.	{Healthcare,Biotechnology}	{CELG,JAZZ,ABBV,GWPH,TRVN,ARDX,IONS,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:01.094903	2020-10-11 23:38:31.021426
CNMD	CONMED Corporation	Conmed Corp is a medical technology company with an emphasis on surgical devices and equipment for minimally invasive procedures and monitoring. The Company's products serve the clinical areas, surgeons and physicians.	{Healthcare,"Medical Devices"}	{IART,BSX,JNJ,SYK,TFX,SNN,ZBH,MDT,XLV}	Medical Devices	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:01.107592	2020-10-11 23:38:31.02582
CNTF	China TechFaith Wireless Communication Technology Limited	China TechFaith Wireless Comm Tech Ltd is a China-based mobile solutions provider for the global mobile handsets market, with a focus on the original design and development of specialized mobile handsets. It is also engaged in real estate business.	{Technology,"Communication Equipment"}	{AAPL,BB,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:01.119102	2020-10-11 23:38:31.023428
CO	Global Cord Blood Corporation	Global Cord Blood Corp, formerly China Cord Blood Corp, provides cord blood banking services. The company provides cord blood processing and storage services.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:01.132061	2020-10-11 23:38:31.031328
COL	Rockwell Collins Inc.	Rockwell Collins Inc is an aviation and high-integrity solutions company serving commercial and military customers. It also provides information management services through voice and data communication networks and solutions worldwide.	{Industrials,"Aerospace & Defense"}	{FLIR,VSAT,ESLT,TGI,BA,HRS,HON,NOC,RTN,LLL}	Aerospace & Defense	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:01.14681	2020-10-11 23:38:31.024353
CLRBW	Cellectar Biosciences Inc. Warrants		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:01.013738	2020-10-11 23:38:31.023007
CLX	Clorox Company (The)	Clorox Co is engaged in the household product market. The Company sells its products primarily through grocery stores, e-commerce channels and medical supply distributors.	{"Consumer Defensive","Household & Personal Products","Consumer Packaged Goods"}	{OBCI,CHD}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:01.037208	2020-10-11 23:38:31.021426
CMD	Cantel Medical Corp.	Cantel Medical Corp is dedicated in delivering infection prevention products and services for patients, caregivers, and other healthcare providers which improve outcomes, enhance safety and help save lives.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{RMTI,CNMD,FMS,DHR,ECL,JNJ,TFX,MMM,STE,XRAY,XLV}	Medical Instruments & Equipment	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:01.053657	2020-10-11 23:38:31.028166
CMT	Core Molding Technologies Inc	Core Molding Technologies Inc along with its subsidiaries operates in the plastics market in a family of products known as reinforced plastics. The company is a manufacturer of sheet molding compound and molder of fiberglass reinforced plastics.	{"Consumer Cyclical","Rubber & Plastics",Autos}	{UFPT,MYE,TG}	Autos	Consumer Cyclical	NYSE American	\N	2020-10-11 23:12:01.080301	2020-10-11 23:38:31.02582
CNAT	Conatus Pharmaceuticals Inc.	Conatus Pharmaceuticals Inc is a biotechnology company. It focuses on the development and commercialization of novel medicines to treat liver disease.	{Healthcare,Biotechnology}	{VRTX,IRWD,PFE,ICPT,ENTA,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:01.092236	2020-10-11 23:38:31.025871
CNSL	Consolidated Communications Holdings Inc.	Consolidated Communications Holdings Inc is a provider of telecom services in the United States. The company's offering spans local and long-distance services, broadband Internet access and cloud data services.	{"Communication Services","Telecom Services"}	{}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:12:01.116828	2020-10-11 23:38:31.031328
CODA	Coda Octopus Group Inc.	Coda Octopus Group Inc designs and manufactures patented real time 3D sonar solutions and other products for sale to the subsea, defense, mining and marine sciences markets. Its segments are Marine Technology Business, and Marine Engineering Business.	{Industrials,"Aerospace & Defense"}	{}	Aerospace & Defense	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:01.1344	2020-10-11 23:38:31.025871
COLB	Columbia Banking System Inc.	Columbia Banking System Inc is active in the financial services domain. It is the holding company for Columbia State Bank, which provides products for the small and medium sized business. It also provides services to professionals and individuals.	{"Financial Services","Banks - Regional - US",Banks}	{HFWA,EWBC,SIVB,BMRC,BOH,BANR}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:01.152737	2020-10-11 23:38:31.02582
COR	CoreSite Realty Corporation	CoreSite Realty Corp is a fully integrated, self-administered and self-managed real estate investment trust. The Company is engaged in the business of owning, acquiring, constructing and operating data centers.	{"Real Estate","REIT - Office",REITs}	{FR,EGP,MNR,DLR,STAG,PLD,REXR}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:01.17085	2020-10-11 23:38:31.031328
COT	Cott Corporation	Cott Corp along with its subsidiaries is engaged in manufacturing and selling beverages. Its is route based service company that offers bottled water and coffee roasting, blending of iced tea and extract solutions for the U.S. foodservice industry.	{"Consumer Defensive","Beverages - Soft Drinks","Beverages - Non-Alcoholic"}	{COKE,FIZZ,KO,REED,MNST,CCEP,PEP}	Beverages - Non-Alcoholic	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:01.182885	2020-10-11 23:38:31.02423
CPE	Callon Petroleum Company	Callon Petroleum Co is engaged in the exploration, development, acquisition, and production of oil and natural gas properties. The company generates its revenue from the sale of Oil.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{GPOR,EPD,MUR,OII,PAA,MCF,WTI,RDS.A}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:01.199088	2020-10-11 23:38:31.024353
CPRT	Copart Inc.	Copart Inc provides online auctions and vehicle remarketing services in United States, Canada, United Kingdom and Brazil. The company offers vehicle sellers with services to process and sell vehicles over the internet.	{"Consumer Cyclical","Auto & Truck Dealerships",Autos}	{CRMT,RUSHA,LAD,AN,GPI,SAH,KMX,PAG,KAR}	Autos	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:01.215526	2020-10-11 23:38:31.021426
CPT	Camden Property Trust	Camden Property Trust a Texas real estate investment trust engaged in the ownership, management, development, redevelopment, acquisition, and construction of multifamily apartment communities.	{"Real Estate","REIT - Residential",REITs}	{MAA,ESS,AIV,UDR,IRT,AMH}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:01.224193	2020-10-11 23:38:31.025871
CRBP	Corbus Pharmaceuticals Holdings Inc.	Corbus Pharmaceuticals Holdings Inc is a Phase 3 clinical-stage pharmaceutical company focused on the development and commercialization of novel therapeutics to treat rare, chronic, and serious inflammatory and fibrotic diseases.	{Healthcare,Biotechnology}	{VNDA,FATE,VKTX,FBIO,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:01.23966	2020-10-11 23:38:31.025871
FFIN	First Financial Bankshares Inc.	First Financial Bankshares Inc is engaged in providing financial services. It operates through its subsidiaries in conducting full-service commercial banking business. The majority of its services are intended for the real estate sector.	{"Financial Services","Banks - Regional - US",Banks}	{IBOC,SBSI,TCBI,CFR,PB,IBTX,LTXB}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:02.847111	2020-10-11 23:38:31.023007
CLSN	Celsion Corporation	Celsion Corp is a fully-integrated oncology drug development company. It is engaged in developing a portfolio of cancer treatments, including directed chemotherapies, immunotherapies and RNA- or DNA-based therapies.	{Healthcare,Biotechnology}	{ARQL,IMGN,EXEL,KERX,SPPI,ZIOP,ECYT,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:01.023838	2020-10-11 23:38:31.023428
CLXT	Calyxt Inc.	Calyxt Inc is an agricultural biotechnology firm. The company engages in the development of food and agricultural business. It develops specialty food ingredients, such as oils and high fiber wheat, and food crops.	{"Consumer Defensive","Farm Products","Consumer Packaged Goods"}	{PDLI,PIRS}	Consumer Packaged Goods	Consumer Defensive	NASDAQ Global Market	\N	2020-10-11 23:12:01.037415	2020-10-11 23:38:31.02423
CMCT	CIM Commercial Trust Corporation	CIM Commercial Trust Corp is a real estate investment trust. The company along with its subsidiaries is engaged in acquiring, owning, and operating Class A and creative office properties in urban communities throughout the United States.	{"Real Estate","REIT - Office",REITs}	{ABR,STWD,BXMT}	REITs	Real Estate	NASDAQ Global Market	\N	2020-10-11 23:12:01.048921	2020-10-11 23:38:31.025871
CMSSR	CM Seven Star Acquisition Corporation Right		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:01.074327	2020-10-11 23:38:31.028166
CNC	Centene Corporation	Centene Corp offers healthcare plans to United States government sponsored healthcare programs. The company primarily focuses on uninsured individuals and helps them access care facilities, and social services through government subsidized programs.	{Healthcare,"Health Care Plans"}	{HUM,AET,MOH,WCG,GTS,ANTM,XLV}	Health Care Plans	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:01.094541	2020-10-11 23:38:31.023428
CNTY	Century Casinos Inc.	Century Casinos Inc is an casino entertainment company that develops and operates gaming establishments, as well as related lodging, restaurant and entertainment facilities around the world.	{"Consumer Cyclical","Resorts & Casinos","Travel & Leisure"}	{MCRI,PENN,WYNN,BYD,MGM,LVS,UWN,ERI}	Travel & Leisure	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:01.1264	2020-10-11 23:38:31.021426
COHR	Coherent Inc.	Coherent Inc is a supplier of photonics-based solutions in a range of commercial and scientific research applications. It designs, manufactures, services and markets lasers and related accessories for a diverse group of customers.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{IIVI,IPGP,LPTH,OCLR,IEX,VIAV,NOVT,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:01.142321	2020-10-11 23:38:31.028166
CONE	CyrusOne Inc	CyrusOne Inc is an owner, operator and developer of enterprise-class, carrier-neutral data center properties. The Company provides mission-critical data center facilities that protect and ensure the continued operation of IT infrastructure.	{"Real Estate","REIT - Diversified",REITs}	{DLR,TRNO,COR,STAG,REXR,QTS}	REITs	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:01.1649	2020-10-11 23:38:31.025871
COUP	Coupa Software Incorporated	Coupa Software Inc is a US-based firm that offers spend management solutions through the Software-as-a-Service business model. The company primarily offers cloud-based spend management platforms through various applications to its clients.	{Technology,"Software - Infrastructure","Application Software"}	{PRGS,PXLW,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:01.186113	2020-10-11 23:38:31.028166
CPAH	CounterPath Corporation	CounterPath Corp designs, develops and sells software and services that enable enterprises and telecommunication service providers to deliver Unified Communications (UC) services, including voice, video, messaging and collaboration functionality.	{Technology,"Software - Application","Application Software"}	{CMG,RCI,CSCO,ERIC,MITL,GEC,XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:01.197726	2020-10-11 23:38:31.025871
CPLP	Capital Product Partners L.P.	Capital Product Partners LP is a shipping company. Its owned fleet of ships consists of crude oil tankers and container carrier vessels.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{GASS,TNP,NAT,SFL,TOO,STNG,DLNG,FRO,GOGL,DRYS}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:01.216757	2020-10-11 23:38:31.024353
CRAK	VanEck Vectors Oil Refiners	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the MVIS® Global Oil Refiners Index.\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The index includes equity securities and depositary receipts of companies that generate at least 50% of their revenues from crude oil refining. Products of these companies may include gasoline, diesel, jet fuel, fuel oil, naphtha, and other petrochemicals. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.235107	2020-10-11 23:38:31.02423
CRH	CRH PLC American Depositary Shares	CRH PLC manufactures and distributes building materials products. It sells frame elements, and value-added exterior products primarily to nonresidential projects.	{"Basic Materials","Building Materials"}	{CX,VMC}	Building Materials	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:01.251132	2020-10-11 23:38:31.024353
CRMT	America's Car-Mart Inc.	America's Car-Mart Inc is an automotive retailing company. It sells older model used vehicles & provides financing for customers.	{"Consumer Cyclical","Auto & Truck Dealerships",Autos}	{CPRT,AN,GPI,SAH,KMX,PAG,KAR}	Autos	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:01.265371	2020-10-11 23:38:31.021426
CRTO	Criteo S.A.	Criteo SA is a technology company providing digital performance marketing. It sells internet display advertisements featuring product-level recommendations either directly to clients or to advertising agencies.	{Technology,"Internet Content & Information","Online Media"}	{WIX,GOOGL,INAP,XLK}	Online Media	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:01.277439	2020-10-11 23:38:31.02423
CMRX	Chimerix Inc.	Chimerix Inc is a biopharmaceutical company which focuses on the discovery, development, and commercialization of novel, oral antivirals in areas of unmet medical needs.	{Healthcare,Biotechnology}	{NVAX,SIGA,GILD,JNJ,EBS,MRK,NLNK,NNVC,VICL,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:01.064321	2020-10-11 23:38:31.023428
CNAC	Constellation Alpha Capital Corp.	Constellation Alpha Capital Corp is a blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:01.088097	2020-10-11 23:38:31.024353
CNET	ChinaNet Online Holdings Inc.	ChinaNet Online Holdings Inc is a B2B, integrated internet service provider for expanding small & medium enterprises sales networks in China. It is engaged in advertising industry conducting its business through its subsidiaries and operating entities.	{"Consumer Cyclical","Advertising Agencies","Advertising & Marketing Services"}	{CMCSA,BIDU,AMCN,TLRA}	Advertising & Marketing Services	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:01.103456	2020-10-11 23:38:31.024353
CNOB	ConnectOne Bancorp Inc.	ConnectOne Bancorp Inc is a holding company. The company through its holding provides banking services including lending, depository and related financial services to commercial, industrial and governmental customers.	{"Financial Services","Banks - Regional - US",Banks}	{UNTY,SSFN,PGC,LBAI,PKBK,BKJ,TRCB,VLY}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:01.115358	2020-10-11 23:38:31.025871
CNXM	CNX Midstream Partners LP representing limited partner interests	CNX Midstream Partners LP owns, operates, develops and acquires natural gas gathering and other midstream energy assets in the Marcellus Shale in Pennsylvania and West Virginia.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:01.124661	2020-10-11 23:38:31.028166
CODI	Compass Diversified Holdings Shares of Beneficial Interest	Compass Diversified Holdings is a diversified businesses conglomerate. Through its holdings in multiple companies, it provides waste management services, baby products, hemp-based foods, printed circuit boards and others.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:01.133542	2020-10-11 23:38:31.02582
COHU	Cohu Inc.	Cohu Inc is a supplier of semiconductor test and inspection handlers, micro-electro mechanical system (MEMS) test modules, test contactors and thermal sub-systems used by semiconductor manufacturers and test subcontractors.	{Technology,"Semiconductor Equipment & Materials",Semiconductors}	{AEHR,NANO,FORM,BRKS,MOG.A,TER,INTT,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:01.144711	2020-10-11 23:38:31.031328
CORP	Pimco Investment Grade Corporate Bond Index Exchange-Traded Fund	The investment seeks to provide total return that closely corresponds, before fees and expenses, to the total return of the ICE BofAML US Corporate Index.\n The fund invests at least 80% of its total assets (exclusive of collateral held from securities lending) in the component securities of the ICE BofAML US Corporate Index (the "underlying index"). The underlying index is an unmanaged index comprised of U.S. dollar denominated investment grade corporate debt securities publicly issued in the U.S. domestic market with at least one year remaining term to final maturity.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.17484	2020-10-11 23:38:31.021426
COWN	Cowen Inc.	Cowen Inc is a United States-based company. It provides alternative investment management, investment banking, research, sales and trading and prime brokerage services through its two business segments, investment management and broker-dealer.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{SF,LTS,PJC,EVR,JMP,OPY}	Brokers & Exchanges	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:01.188507	2020-10-11 23:38:31.028166
CPHI	China Pharma Holdings Inc.	China Pharma Holding Inc is mainly engaged in the development, manufacture and marketing of pharmaceutical products for human use in connection with a variety of high-incidence and high-mortality diseases and medical conditions prevalent in China.	{Healthcare,"Drug Manufacturers - Major","Drug Manufacturers"}	{GSK,NVS,JNJ,RDY,SNY,TEVA,MYL,CBPO,XLV}	Drug Manufacturers	Healthcare	NYSE American	\N	2020-10-11 23:12:01.204293	2020-10-11 23:38:31.028166
CPK	Chesapeake Utilities Corporation	Chesapeake Utilities Corp is a diversified energy company engaged, through its operating divisions and subsidiaries, in regulated and unregulated energy businesses.	{Utilities,"Utilities - Regulated Gas","Utilities - Regulated"}	{RGCO,NI,ATO,NJR,SJI,SR,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:01.213611	2020-10-11 23:38:31.02582
CPTA	Capitala Finance Corp.	Capitala Finance Corp is a non-diversified closed-end investment management company. The firm and its subsidiaries provide capital to lower and traditional middle market companies through credit-focused funds.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:01.224771	2020-10-11 23:38:31.024353
CRAY	Cray Inc	Cray Inc designs, develops, and supports high-performance computer systems, commonly known as supercomputers and/or clusters, and provide storage solutions, software and engineering services related to HPC systems.	{Technology,"Computer Systems","Computer Hardware"}	{RSYS,NTAP,IBM,TDC,HPQ,SMCI,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:01.235744	2020-10-11 23:38:31.023428
CREG	China Recycling Energy Corporation	China Recycling Energy Corp through its subsidiaries, sells and leases energy saving systems and equipment to its customers, in the People's Republic of China.	{Industrials,"Waste Management"}	{HCCI,ECOL,AWX}	Waste Management	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:01.249789	2020-10-11 23:38:31.031328
CMSSW	CM Seven Star Acquisition Corporation Warrant		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:01.074091	2020-10-11 23:38:31.024353
CNACU	Constellation Alpha Capital Corp. Unit	Constellation Alpha Capital Corp is a blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:01.089855	2020-10-11 23:38:31.023007
CNFR	Conifer Holdings Inc.	Conifer Holdings Inc is an insurance company. It provides insurance coverage for commercial and personal product lines.	{"Financial Services","Insurance - Property & Casualty"}	{PIH}	Insurance - Property & Casualty	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:01.102509	2020-10-11 23:38:31.02423
CNO	CNO Financial Group Inc.	CNO Financial Group Inc is a holding company for a group of insurance companies. It develops, markets and administers health insurance, annuity, individual life insurance and other insurance products.	{"Financial Services","Insurance - Life"}	{MFC,UNM,AFL,LNC,MET,AIZ,RGA,PFG}	Insurance - Life	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.114764	2020-10-11 23:38:31.028166
CNYA	iShares MSCI China A	The investment seeks to track the investment results of the MSCI China A Inclusion Index composed of domestic Chinese equities that trade on the Shanghai or Shenzhen Stock Exchange.\n The fund generally will invest at least 90% of its assets in the component securities of the index and in investments that have economic characteristics that are substantially identical to the component securities of the index. The index is designed to measure the equity market performance in the People's Republic of China, as represented by "A-shares" that are accessible through the Shanghai Connect or the Shenzhen-Hong Kong Stock Connect program. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:01.128306	2020-10-11 23:38:31.028166
COHN	Cohen & Company Inc.	Cohen & Co Inc is engaged in providing investment industry services, private company services, and management advisory services to various mutual funds, private companies, and business owners.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{RJF,SF,PJC,OPY,COWN}	Brokers & Exchanges	Financial Services	NYSE American	\N	2020-10-11 23:12:01.141469	2020-10-11 23:38:31.028166
CONN	Conn's Inc.	Conn's Inc is a holding company engaged in specialty retail. It is a specialty retailer that offers branded consumer goods and related services and also credit solutions for consumers. The firm operates its business through retail stores and website.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{TSCO,RCII,BBY,HD,AAN,SHOS}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:01.16542	2020-10-11 23:38:31.02582
CORT	Corcept Therapeutics Incorporated	Corcept Therapeutics Inc is engaged in the discovery, development and commercialization of drugs for the treatment of severe metabolic, psychiatric and oncologic disorders by modulating the effects of cortisol.	{Healthcare,Biotechnology}	{NVS,BMY,JNJ,LLY,PFE,ESPR,MYL,CBIO,MNKD,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:01.179967	2020-10-11 23:38:31.024353
CPB	Campbell Soup Company	Campbell Soup Co operates in the packaged foods industry. The products within its scope include meals and beverages, biscuits and snacks.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{HAIN,UL,CAG,TSN,FLO,THS,MKC.V,MDLZ}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:01.198936	2020-10-11 23:38:31.028166
CPSS	Consumer Portfolio Services Inc.	Consumer Portfolio Services Inc is a specialty finance company. It purchases & services retail automobile contracts originated by franchised automobile and independent dealers in USA in the sale of new & used automobiles, light trucks & passenger vans.	{"Financial Services","Credit Services"}	{NICK,CACC,SC}	Credit Services	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:01.218473	2020-10-11 23:38:31.023428
CQQQ	Invesco China Technology	The investment seeks investment results that correspond generally to the performance, before the fund's fees and expenses, of an equity index called the AlphaShares China Technology Index.\n The fund will invest at least 90% of its total assets in common stocks, ADRs, ADSs, GDRs and IDRs that comprise the index and depositary receipts or shares representing common stocks included in the index. The index is designed to measure and monitor the performance of the universe of publicly-traded companies which are based in mainland China, Hong Kong or Macau, are in the Information Technology Sector. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.231894	2020-10-11 23:38:31.021426
CRED	iShares U.S. Credit Bond ETF	The investment seeks to track the investment results of the ICE BofAML US Corporate Index (the "underlying index").\n The fund generally invests at least 90% of its assets in securities of the underlying index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index. The underlying index measures the performance of investment-grade corporate bonds that are U.S. dollar-denominated.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.246053	2020-10-11 23:38:31.023007
CRK	Comstock Resources Inc.	Comstock Resources Inc along with its subsidiaries is engaged in the acquisition, development, production and exploration of oil and natural gas.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{GPOR,CRZO,AXAS,NFX,PXD,MRO,XEC,SM}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:01.258566	2020-10-11 23:38:31.02423
CRT	Cross Timbers Royalty Trust	Cross Timbers Royalty Trust operates as an express trust in the United States.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{MARPS,PHX,PBT,MTR,NRT,SBR,SJT}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:01.276832	2020-10-11 23:38:31.02582
CSBR	Champions Oncology Inc.	Champions Oncology Inc is engaged in the development and sale of advanced technology solutions and products to personalize the development and use of oncology drugs.	{Healthcare,Biotechnology}	{EXEL,CLDX,CRL,HSKA,NSTG,RIOT,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:01.291078	2020-10-11 23:38:31.021426
CNBKA	Century Bancorp Inc. Class A Common Stock	Century Bancorp Inc is a Massachusetts based commercial bank that provides a range of services to commercial enterprises, state and local governments and agencies, non-profit organizations and individuals.	{"Financial Services","Banks - Regional - US",Banks}	{EBTC,UNB,BPFH,INDB,WASH,BRKL,WBS,BHB,NBN,SAL}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:01.093488	2020-10-11 23:38:31.02582
CNQ	Canadian Natural Resources Limited	Canadian Natural Resources Ltd is an oil and natural gas producers in Western Canada. Its portfolio includes light and medium oil, heavy oil, bitumen, synthetic oil, natural gas liquids, and natural gas.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{IMO,ECA,SU,CVE,TOT,OXY,VET}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:01.116326	2020-10-11 23:38:31.023007
CNY	Market Vectors Chinese Renminbi/USD ETN	The investment seeks to replicate, net of expenses, the S&P Chinese Renminbi Total Return Index.\n The index tracks the value of the Chinese RMB relative to the U.S. dollar. It follows the value of non-deliverable, three-month currency forward contracts that are rolled at three-month intervals and includes daily accrued interest.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.127271	2020-10-11 23:38:31.02423
COG	Cabot Oil & Gas Corporation	Cabot Oil & Gas Corp is an independent oil and gas company engaged in the development and exploration of oil and gas properties located in North America.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{CHK,APC,RRC,SWN,EOG,NBL,EQT,SM,AR,ECR}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:01.140464	2020-10-11 23:38:31.02423
COLM	Columbia Sportswear Company	Columbia Sportswear Co is engaged in designing, sourcing, marketing and distributing outdoor and active lifestyle apparel, footwear, accessories and equipment. It sells under four brands, Columbia, Sorel, Mountain Hardwear, and prAna.	{"Consumer Cyclical","Apparel Manufacturing","Manufacturing - Apparel & Furniture"}	{SKX,OXM,VFC,WWW,UAA,PVH,RL,DECK}	Manufacturing - Apparel & Furniture	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:01.155194	2020-10-11 23:38:31.02423
COP	ConocoPhillips	ConocoPhillips operates in the oil & gas sector. Its primary business involves exploration and production of crude oil and natural gas.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{TOT,XOM,DVN,APC,BP,PTR,CVX,MRO,RDS.A,HES}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:01.166668	2020-10-11 23:38:31.028166
COWNZ	Cowen Inc. 7.35% Senior Notes Due 2027		{}	{}			Nasdaq Global Select	\N	2020-10-11 23:12:01.188592	2020-10-11 23:38:31.031328
CPG	Crescent Point Energy Corporation (Canada)	Crescent Point Energy Corp is in the business of acquiring, developing and holding interests in petroleum and natural gas properties and other related assets.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{BTE}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:01.204098	2020-10-11 23:38:31.028166
CPL	CPFL Energia S.A. American Depositary Shares	CPFL Energia SA is engaged in the distribution, generation and commercialization of electric energy in Brazil. It distributes electricity mainly in the states of Sao Paulo and Rio Grande do Sul.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{ENIA,KEP,EBR.B,CIG,ELP,AES,EXC,EDN,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:01.21515	2020-10-11 23:38:31.02423
CQP	Cheniere Energy Partners LP	Cheniere Energy Partners LP engages in the liquified natural gas export business. The firm operates through its Sabine Pass liquefaction/terminal/pipeline facilities in Louisiana.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{OKE,UGI,LNG,CVX,COP,DCP}	Oil & Gas - Midstream	Energy	NYSE American	\N	2020-10-11 23:12:01.229889	2020-10-11 23:38:31.02582
CREE	Cree Inc.	Cree Inc manufactures lighting-class light emitting diode (LED) products, lighting products and bandgap semiconductor products for power and radio-frequency (RF) applications.	{Technology,Semiconductors}	{VECO,DIOD,IIVI,ON,AOSL,QRVO,LEDS,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:01.246666	2020-10-11 23:38:31.02582
CRL	Charles River Laboratories International Inc.	Charles River Laboratories International Inc is a pharmaceutical company providing drug discovery and development services. The company provides animal models for laboratory testing, and offers manufacturing testing services to various industries.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{BASI,RGEN,TMO,SCYX,IQV,XLV}	Medical Diagnostics & Research	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:01.260686	2020-10-11 23:38:31.031328
CRS	Carpenter Technology Corporation	Carpenter Technology Corp manufactures, fabricates and distributes specialty metals that include nickel, cobalt, titanium, and chromium. It operates in two business segments: Specialty Alloys Operations and Performance Engineered Products.	{Industrials,"Metal Fabrication","Industrial Products"}	{USAP,STLD,HAYN,AKS,CMC,KMT,NUE,ATI,X,BOOM}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:01.270387	2020-10-11 23:38:31.031328
CRZO	Carrizo Oil & Gas Inc.	Carrizo Oil & Gas Inc along with its subsidiaries is engaged in the exploration, development and production of oil and gas from resource plays located in the United States.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{CHK,NFX,DNR,EOG,EPM,EPE}	Oil & Gas - E&P	Energy	Nasdaq Global Select	\N	2020-10-11 23:12:01.284099	2020-10-11 23:38:31.024353
COE	China Online Education Group American depositary shares each representing 15 Class A	China Online Education Group is engaged in providing online English language education services to students in the People's Republic of China.	{"Consumer Defensive","Education & Training Services",Education}	{RENN}	Education	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:01.14062	2020-10-11 23:38:31.023428
COLD	Americold Realty Trust	Americold Realty Trust mainly owns and operates temperature-controlled warehouses. It provides its customers with handling and other warehouse services related to the products stored in buildings.	{"Real Estate","REIT - Industrial",REITs}	{}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:01.152954	2020-10-11 23:38:31.021426
COPX	Global X Copper Miners	The investment seeks to provide investment results that correspond generally to the price and yield performance, before fees and expenses, of the Solactive Global Copper Miners Total Return Index.\n The fund invests at least 80% of its total assets in the securities of the underlying index and in American Depositary Receipts ("ADRs") and Global Depositary Receipts ("GDRs") based on the securities in the underlying index. The underlying index is designed to measure broad-based equity market performance of global companies involved in the copper mining industry. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.169752	2020-10-11 23:38:31.02423
COST	Costco Wholesale Corporation	Costco Wholesale Corp and its subsidiaries operate membership warehouses. The company offers its members low prices on a limited selection of nationally branded and select private-label products in merchandise categories.	{"Consumer Defensive","Discount Stores","Retail - Defensive"}	{PSMT,DLTR,WMT,TGT,BIG,DG,FIVE}	Retail - Defensive	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:01.18329	2020-10-11 23:38:31.025871
CPAC	Cementos Pacasmayo S.A.A. American Depositary Shares (Each representing five)	Cementos Pacasmayo SAA is a cement manufacturer in the northern region of Peru. It produces, distributes & sells cement and cement-related materials, such as concrete blocks and ready-mix concrete.	{"Basic Materials","Building Materials"}	{CRH,CX,EXP}	Building Materials	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:01.195567	2020-10-11 23:38:31.02582
CPIX	Cumberland Pharmaceuticals Inc.	Cumberland Pharmaceuticals Inc is a specialty pharmaceutical company. It is engaged in the acquisition, development and commercialization of branded prescription products. Its target markets are hospital acute care and gastroenterology.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{AKRX,DRRX,IRWD,PFE,MRK,MYL,PTX,XLV}	Drug Manufacturers	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:01.20674	2020-10-11 23:38:31.025871
CPSH	CPS Technologies Corp.	CPS Technologies Corp provides material solutions to the transportation, automotive, energy, computing/Internet, telecommunications, aerospace, defense, and oil and gas end markets.	{Technology,"Electronic Components","Computer Hardware"}	{CENT,KEM,CCF,MTRN,XLK}	Computer Hardware	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:01.216347	2020-10-11 23:38:31.028166
CPTAL	Capitala Finance Corp. 6% Notes Due 2022		{}	{}			Nasdaq Global Select	\N	2020-10-11 23:12:01.229248	2020-10-11 23:38:31.028166
CRC	California Resources Corporation	California Resources Corp is an oil and natural gas exploration and production company, which operates within the State of California. It explores, produces, gathers, processes and markets crude oil, natural gas and natural gas liquids.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{AREX,UNT,SN,MTDR}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:01.240313	2020-10-11 23:38:31.028166
CRIS	Curis Inc.	Curis Inc is a biotechnology company engaged in developing and commercializing drug candidates for the treatment of human cancers. Its pipeline products include CUDC-907, CA-170, CA-4948, CA-327, and Erivedge.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:01.254397	2020-10-11 23:38:31.023007
CROX	Crocs Inc.	Crocs Inc is engaged in the design, development, manufacturing, marketing and distribution of consumer products including footwear and accessories for men, women and children. The business activity is functioned through retail chains.	{"Consumer Cyclical","Footwear & Accessories","Manufacturing - Apparel & Furniture"}	{SHOO,SKX,WWW,DECK,CAL}	Manufacturing - Apparel & Furniture	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:01.268475	2020-10-11 23:38:31.025871
CRUSC	Calvert Ultra-Short Duration Income NextShares	The investment seeks to maximize income, to the extent consistent with preservation of capital, through investment in short-term bonds and income-producing securities.\n The fund seeks to achieve its investment objective by investing, under normal circumstances, at least 80% of its net assets (including borrowings for investment purposes) in a portfolio of floating-rate securities (e.g., corporate floating-rate securities) and securities with durations of less than or equal to one year. Under normal circumstances, the fund’s average portfolio duration will be less than one year.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.279402	2020-10-11 23:38:31.023428
CSB	Compass EMP US Small Cap High Dividend 100 Volatil	The investment seeks to provide investment results that track the performance of the Nasdaq Victory US Small Cap High Dividend 100 Volatility Weighted Index.\n The fund seeks to achieve its investment objective by investing, under normal market conditions, at least 80% of its assets directly or indirectly in the securities included in the Nasdaq Victory US Small Cap High Dividend 100 Volatility Weighted Index, an unmanaged, volatility weighted index maintained exclusively by the index provider. The index identifies the 100 highest dividend yielding stocks in the Nasdaq Victory US Small Cap 500 Volatility Weighted Index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.292928	2020-10-11 23:38:31.028166
COKE	Coca-Cola Bottling Co. Consolidated	Coca-Cola Bottling Co Consolidated together with its subsidiaries, produces, markets, and distributes nonalcoholic beverages.	{"Consumer Defensive","Beverages - Soft Drinks","Beverages - Non-Alcoholic"}	{COT,FIZZ,KOF,AKO.B,KO,MNST,CCEP,PEP}	Beverages - Non-Alcoholic	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:01.145412	2020-10-11 23:38:31.025871
CORN	Teucrium Corn Fund ETV	The investment seeks to have the daily changes in percentage terms of the shares’ NAV reflect the daily changes in percentage terms of a weighted average of the closing settlement prices for three futures contracts for corn that are traded on the Chicago Board of Trade.\n The fund invests under normal market conditions in Benchmark Component Futures Contracts or, in certain circumstances, in other Corn Futures Contracts traded on the CBOT or on foreign exchanges.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.177143	2020-10-11 23:38:31.028166
CPA	Copa Holdings S.A. Class A	Copa Holdings SA is engaged in investing in airlines and aviation-related companies. The Company through its subsidiaries provides international air transportation for passengers, cargo and mail services.	{Industrials,Airlines}	{JBLU,CEA,DAL,LTM,VLRS,AVH,AAL,GOL,UAL}	Airlines	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:01.191779	2020-10-11 23:38:31.02423
CPHC	Canterbury Park Holding Corporation	Canterbury Park Holding Corp is involved in pari-mutual wagering on horse races and unbanked card games. It is also engaged in related services and activities such as food and beverage, publication sales, catering, and events.	{"Consumer Cyclical",Gambling,"Travel & Leisure"}	{PENN,ISCA,CHDN,SGMS,DDE,FLL,ERI,GDEN,NYNY}	Travel & Leisure	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:01.204318	2020-10-11 23:38:31.021426
CPST	Capstone Turbine Corporation	Capstone Turbine Corp is a gas turbine manufacturer. The company develops, manufactures, markets and services microturbine technology solutions for use in stationary distributed power generation applications.	{Industrials,"Diversified Industrials","Industrial Products"}	{WWD,CAT,GE,UTX,PLUG,BWEN,OPTT}	Industrial Products	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:01.221345	2020-10-11 23:38:31.02582
CR	Crane Co.	Crane Co is a diversified manufacturer of engineered industrial products. The company maintains a wide product portfolio of valves, payment acceptance and dispensing products and aerospace components.	{Industrials,"Diversified Industrials","Industrial Products"}	{SNHY,PKOH,OFLX,NCR,PH,ATU,COL,LLL}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:01.234466	2020-10-11 23:38:31.031328
CRESY	Cresud S.A.C.I.F. y A. American Depositary Shares each representing ten shares of Common Stock	Cresud SACIF y A is an Argentinean agricultural land development company and its primary activities include the acquisition development, and sale of land. The company also produces agricultural commodities, such as crops, beef, and milk.	{"Consumer Defensive","Farm Products","Consumer Packaged Goods"}	{ALCO,BG,AGRO}	Consumer Packaged Goods	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:01.249903	2020-10-11 23:38:31.021426
CRM	Salesforce.com Inc	Salesforce.com Inc is a software-as-a-service company that provides enterprise cloud computing solutions, offering social and mobile cloud apps and platform services, as well as professional services to facilitate the adoption of its solutions.	{Technology,"Software - Application","Application Software"}	{MSFT,INTU,ADBE,SYMC,BVSN,SAP,ORCL,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:01.261585	2020-10-11 23:38:31.024353
CRSP	CRISPR Therapeutics AG	CRISPR Therapeutics AG is a gene editing company. It is engaged in the development of CRISPR/Cas9-based therapeutics.	{Healthcare,Biotechnology}	{PDLI,PIRS,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:01.273788	2020-10-11 23:38:31.021426
CS	Credit Suisse Group American Depositary Shares	Credit Suisse Group AG is a Swiss-based company providing banking and financial services. It offers various services such as investment banking, wealth management, and asset management.	{"Financial Services","Banks - Global",Banks}	{BMO,CM,BCS,BAC,DB,ITUB}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.286351	2020-10-11 23:38:31.02582
CSGS	CSG Systems International Inc.	CSG Systems International Inc provides business support solutions mainly to the communications industry. It offers cable and satellite care, content management, convergent rating, mediation and data management, as well as routing services in the US.	{Technology,"Software - Infrastructure","Application Software"}	{MNDO,SNCR,EVOL,CLGX,ORCL,DOX,APPS,GEC,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:01.300757	2020-10-11 23:38:31.02582
CSTE	Caesarstone Ltd.	Caesarstone Ltd manufactures engineered quartz surfaces. Its products are used in both residential and commercial buildings as counter-tops, vanities, wall cladding, floors, and other interior surfaces.	{"Basic Materials","Building Materials"}	{AMWD,MPVD}	Building Materials	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:12:01.318493	2020-10-11 23:38:31.025871
CTBB	Qwest Corporation 6.5% Notes due 2056		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:01.333216	2020-10-11 23:38:31.02423
CTLT	Catalent Inc.	Catalent Inc is a drug development, delivery and supply partner company. It engages in advanced delivery technologies and development solutions for drugs, biologics, and consumer health products across US, Europe and other international markets.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{JAZZ,SUPN,AERI,INVA,XLV}	Drug Manufacturers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:01.349459	2020-10-11 23:38:31.02582
COLL	Collegium Pharmaceutical Inc.	Collegium Pharmaceutical Inc is a specialty pharmaceutical company which is engaged in developing and planning to commercialize abuse-deterrent products through its patented DETERx platform technology.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{UTHR,HZNP,XLV}	Drug Manufacturers	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:01.1523	2020-10-11 23:38:31.023428
CORE	Core-Mark Holding Company Inc.	Core-Mark Holding Co Inc offers a range of branded and private label products including cigarettes, food, candy, health and beauty products, and beverages. It offers products, marketing programs and technology solutions in the U.S. and Canada.	{"Consumer Defensive","Food Distribution","Retail - Defensive"}	{UNFI,WILC,SYY,DIT,CHEF,SPTN}	Retail - Defensive	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:01.173537	2020-10-11 23:38:31.024353
COTY	Coty Inc. Class A	Coty Inc is an American cosmetics company. Its products include fragrances, color cosmetics, hair, body and skin care products. The company markets its products under Calvin Klein, Chloe, Clairol, Marc Jacobs, Wella, Rimmel, and Bourjois brands worldwide.	{"Consumer Defensive","Household & Personal Products","Consumer Packaged Goods"}	{IPAR,EL,AVP,PG,REV}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:01.186732	2020-10-11 23:38:31.023007
CPER	United States Copper Index Fund ETV	The investment seeks the daily changes in percentage terms of its shares’ per share net asset value (“NAV”) to reflect the daily changes in percentage terms of the SummerHaven Copper Index Total ReturnSM (the “SCI”), less CPER’s expenses.\n The SCI is designed to reflect the performance of the investment returns from a portfolio of copper futures contracts on the COMEX exchange.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.201142	2020-10-11 23:38:31.023007
CPS	Cooper-Standard Holdings Inc.	Cooper-Standard Holdings Inc is engaged in manufacturing sealing, fuel and brake delivery, fluid transfer and anti-vibration systems components subsystems, and modules. Its products are used in passenger vehicles and light trucks.	{"Consumer Cyclical","Auto Parts",Autos}	{MGA,AXL,TTM,DAN}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:01.216332	2020-10-11 23:38:31.031328
CPTAG	Capitala Finance Corp. 5.75% Convertible Notes Due 2022		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:01.229641	2020-10-11 23:38:31.028166
CRCM	Care.com Inc.	Care.com Inc is an online marketplace for finding and managing family care and also e-commerce service selling curated products designed for families. It helps families address their particular lifecycle of care needs.	{Technology,"Internet Content & Information","Online Media"}	{EBAY,BFAM,TRUE,DHX,GOOGL,IPDN,XLK}	Online Media	Technology	New York Stock Exchange	\N	2020-10-11 23:12:01.244876	2020-10-11 23:38:31.02423
CRMD	CorMedix Inc	Cormedix Inc is a commercial-stage biopharmaceutical company. It focuses on development and commercializing therapeutic products for the prevention and treatment of infectious and inflammatory diseases.	{Healthcare,Biotechnology}	{UTMD,BDX,EBS,TTPH,AKAO,XLV}	Biotechnology	Healthcare	NYSE American	\N	2020-10-11 23:12:01.262816	2020-10-11 23:38:31.023428
CRWS	Crown Crafts Inc.	Crown Crafts Inc through its subsidiaries operates in the infant and toddler products segment within the consumer products industry. It designs and distributes, infant and toddler bedding and blankets, bibs, disposable products and accessories.	{"Consumer Cyclical","Apparel Manufacturing","Manufacturing - Apparel & Furniture"}	{SUMR,AIN,CULP,LEG,VFC,TPX}	Manufacturing - Apparel & Furniture	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:01.2808	2020-10-11 23:38:31.025871
CSF	Compass EMP U.S. Discovery 500 Enhanced Volatility	The investment seeks investment results that match the performance of the Nasdaq Victory US Small Cap 500 Long/Cash Volatility Weighted Index before fees and expenses.\n The fund seeks to achieve its investment objective by investing, under normal market conditions, at least 80% of its assets directly or indirectly in the securities included in the Nasdaq Victory US Small Cap 500 Long/Cash Volatility Weighted Index. The index identifies the 500 largest U.S. companies with market capitalizations of less than $3 billion measured at the time the index's constituent securities are determined.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.2919	2020-10-11 23:38:31.023428
CSML	IQ Chaikin U.S. Small Cap ETF	The investment seeks investment results that track (before fees and expenses) the price and yield performance of its underlying index, the Nasdaq Chaikin Power US Small Cap Index.\n The fund invests at least 80% of the value of its assets (net assets plus the amount of any borrowings for investment purposes) in securities of small-capitalization U.S. issuers. The underlying index applies the Chaikin Power Gauge®, a quantitative multi-factor model that seeks to identify securities that are expected to outperform peers, to select securities from the Nasdaq US 1500 Index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.306078	2020-10-11 23:38:31.02423
CSTM	Constellium N.V.	Constellium NV designs and manufactures innovative specialty rolled and extruded aluminium products, serving primarily the packaging, aerospace, and automotive end-markets. Constellium operates production sites in Europe, North America and China.	{"Basic Materials",Aluminum,"Metals & Mining"}	{PKOH,KALU,AA}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:01.318847	2020-10-11 23:38:31.028166
CTHR	Charles & Colvard Ltd.	Charles & Colvard Ltd is engaged in manufacturing, marketing and distribution of moissanite jewels, finished jewelry and fashion finished jewelry for sale in the international jewelry market.	{"Consumer Cyclical","Luxury Goods","Retail - Apparel & Specialty"}	{CREE,KGJI,MOV,TIF}	Retail - Apparel & Specialty	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:01.340848	2020-10-11 23:38:31.031328
CORI	Corium International Inc.	Corium International Inc is a commercial stage biopharmaceutical company. The company develops, manufactures and commercializes specialty pharmaceutical products that leverage its experience in transdermal and transmucosal delivery systems.	{Healthcare,Biotechnology}	{JNJ,PG,MRK,TEVA,AGRX,MYL,ZGNX,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:01.17449	2020-10-11 23:38:31.023007
COWZ	Pacer US Cash Cows 100	The investment seeks to track the total return performance, before fees and expenses, of the Pacer US Small Cap Cash Cows Index (the "index").\n Under normal circumstances, at least 80% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in the component securities of the index. The index uses an objective, rules-based methodology to provide exposure to small-capitalization U.S. companies with high free cash flow yields. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:01.188727	2020-10-11 23:38:31.023428
CPI	IQ Real Return	The investment seeks investment results that correspond generally to the price and yield performance of its underlying index, the IQ Real Return Index.\n The fund is a "fund of funds" which means it invests, under normal circumstances, at least 80% of its net assets, plus the amount of any borrowings for investment purposes, in the investments included in its underlying index, which includes underlying funds. The underlying index consists of a number of components ("underlying index components") selected in accordance with the rules-based methodology of such underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.204818	2020-10-11 23:38:31.023428
CPRX	Catalyst Pharmaceuticals Inc.	Catalyst Pharmaceuticals Inc is a biopharmaceutical company. The company is engaged in the development and commercialization of therapies for people with rare debilitating diseases, chronic neuromuscular and neurological diseases.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{BMRN,ACOR,ABT,JNJ,PFE,SRPT,ARDM,XLV}	Drug Manufacturers	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:01.216852	2020-10-11 23:38:31.028166
CRBN	iShares MSCI ACWI Low Carbon Target	The investment seeks to track the investment results of the MSCI ACWI Low Carbon Target Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The underlying index is designed to address two dimensions of carbon exposure - carbon emissions and potential carbon emissions from fossil fuel reserves.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.237437	2020-10-11 23:38:31.028166
CRHM	CRH Medical Corporation of Beneficial Interest	CRH Medical Corp provides anesthesiology services to gastroenterologists in the United States through its subsidiaries and also specializes in the treatment of hemorrhoids utilizing its treatment protocol and patented proprietary technology.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{OMER,XLV}	Medical Instruments & Equipment	Healthcare	NYSE American	\N	2020-10-11 23:12:01.252441	2020-10-11 23:38:31.025871
CROP	IQ Global Agribusiness Small Cap	The investment seeks investment results that correspond generally to the price and yield performance of its underlying index, the IQ Global Agribusiness Small Cap Index.\n The fund invests at least 80% of its net assets, plus the amount of any borrowings for investment purposes, in the investments included in its underlying index. The underlying index is a rules based, modified capitalization weighted, float adjusted index intended to give investors a means of tracking the overall performance of the global, small capitalization sector of publicly traded companies that are engaged primarily in the agribusiness sector. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.267503	2020-10-11 23:38:31.023007
CSA	Compass EMP US Small Cap 500 Volatility Weighted I	The investment seeks to provide investment results that track the performance of the Nasdaq Victory US Small Cap 500 Volatility Weighted Index before fees and expenses.\n The fund seeks to achieve its investment objective by investing, under normal market conditions, at least 80% of its assets directly or indirectly in the securities included in the Nasdaq Victory US Small Cap 500 Volatility Weighted Index, an unmanaged, volatility weighted index maintained exclusively by the index provider. The index combines fundamental criteria with individual security risk control achieved through volatility weighting of individual securities.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.290891	2020-10-11 23:38:31.023007
CSLT	Castlight Health Inc. Class B	Castlight Health Inc offers a health benefits platform that engages employees to make health care decisions and enables employers to communicate and measure their benefit programs.	{Technology,"Software - Application","Application Software"}	{MSFT,CERN,HSTM,CPSI,STRM,MODN,VEEV,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:01.302578	2020-10-11 23:38:31.021426
CSTR	CapStar Financial Holdings Inc.	CapStar Financial Holdings Inc is a bank holding company. The Company operates as a commercial bank providing products and services including, commercial and industrial loans, private banking and wealth management services.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:01.317809	2020-10-11 23:38:31.021426
CSX	CSX Corporation	CSX Corp is a railroad operating company with operations spread across the Eastern United States. The company hauls shipments of coal products, chemicals, intermodal containers, and a diverse mix of other merchandise.	{Industrials,Railroads,"Transportation & Logistics"}	{CNI,CP,GWR,NSC,UNP,KSU}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:01.330339	2020-10-11 23:38:31.023007
CP	Canadian Pacific Railway Limited	Canadian Pacific Railway Ltd is engaged in rail transportation. It provides freight transportation services, logistics solutions, and supply chain expertise in Canada and the United States.	{Industrials,Railroads,"Transportation & Logistics"}	{CNI,GWR,NSC,UNP,KSU,CSX}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:01.190287	2020-10-11 23:38:31.021426
CPF	Central Pacific Financial Corp New	Central Pacific Financial Corp provides financial services, especially banking products. As a holding company of Central Pacific Bank, it renders loans and treasury products to individuals and corporates in the United States.	{"Financial Services","Banks - Regional - US",Banks}	{CWBC,CVBF,EWBC,CVCY,BSRR,PPBI,SIVB,BOH,BANR}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.204036	2020-10-11 23:38:31.031328
CPSI	Computer Programs and Systems Inc.	Computer Programs and Systems Inc is engaged in providing healthcare solutions and services for community hospitals and post-acute care facilities.	{Technology,"Health Information Services","Application Software"}	{CERN,HSTM,MDRX,STRM,CACI,MCK,CSLT,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:01.216964	2020-10-11 23:38:31.023007
CRAI	CRA InternationalInc.	CRA International, Inc. is a global consulting firm offering economic, financial, and strategic consulting services. The company offers services to major law firms, corporations, accounting firms, and governments in the US and international markets.	{Industrials,"Business Services"}	{RECN,HURN,ICFI,III,NCI,GPX,HRB,FCN}	Business Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:01.235691	2020-10-11 23:38:31.023007
CRF	Cornerstone Total Return Fund Inc. (The)	Cornerstone Total Return Fund Inc is a diversified closed-end management company. Its objective is to seek long-term capital appreciation through investment in equity securities of U.S and non-U.S. companies.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:01.252453	2020-10-11 23:38:31.023428
CRNT	Ceragon Networks Ltd.	Ceragon Networks Ltd is a wireless backhaul specialist company providing wireless backhaul solutions that enable cellular operators and other wireless service providers to deliver voice and data services, enabling smart-phone applications.	{Technology,"Communication Equipment"}	{CAMP,NOK,MSI,UBNT,AVNW,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:01.26394	2020-10-11 23:38:31.02582
CRVS	Corvus Pharmaceuticals Inc.	Corvus Pharmaceuticals Inc is a clinical stage biopharmaceutical company. It is engaged in development and commercialization of novel immuno-oncology therapies that are designed to harness the immune system to attack cancer cells.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:01.280026	2020-10-11 23:38:31.028166
CSCO	Cisco Systems Inc.	Cisco Systems Inc is a supplier of data networking equipment and software. Its products include routers, switches, access equipment, and security and network management software which allow data communication among dispersed computer networks.	{Technology,"Communication Equipment"}	{MSFT,CTXS,FFIV,NTGR,ERIC,JNPR,IBM,VMW,MSI,HPQ,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:01.292231	2020-10-11 23:38:31.02423
CSL	Carlisle Companies Incorporated	Carlisle Companies Inc manufactures construction materials. Its products include insulation materials, rubber, thermoplastic polyolefin, and polyvinyl chloride roofing membranes used predominantly on non-residential low-sloped roofs.	{"Consumer Cyclical","Rubber & Plastics",Autos,Industrials,Conglomerates}	{RAVN,OCC,NWL,TWI,APH,AME,BA,TXT,MMM,ITT}	Conglomerates	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:01.302738	2020-10-11 23:38:31.028166
CSU	Capital Senior Living Corporation	Capital Senior Living Corp provides senior living services to the elderly, including independent living, assisted living, skilled nursing and home care services.	{Healthcare,"Long-Term Care Facilities","Health Care Providers"}	{ENSG,NHC,BKD,DVCR,GEN,FVE,RHE,XLV}	Health Care Providers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:01.318791	2020-10-11 23:38:31.024353
CTDD	Qwest Corporation 6.75% Notes due 2057		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:01.338691	2020-10-11 23:38:31.02582
CTO	Consolidated-Tomoka Land Co.	Consolidated-Tomoka Land Co. is a diversified real estate operating company. Principally, it owns and manages commercial real estate properties in the United States.	{"Real Estate","Real Estate - General","Real Estate Services"}	{INTG,MAYS,ARL}	Real Estate Services	Real Estate	NYSE American	\N	2020-10-11 23:12:01.352235	2020-10-11 23:38:31.028166
CTXR	Citius Pharmaceuticals Inc.	Citius Pharmaceuticals Inc is a specialty pharmaceutical company engaged in development and commercialization of therapeutic products for markets using patented or proprietary formulations and modified drug delivery technology.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:01.367834	2020-10-11 23:38:31.021426
CUBI	Customers Bancorp Inc	Customers Bancorp Inc through its wholly-owned subsidiary Customers Bank provides financial products and services to small and middle-market businesses, not-for-profits, and consumers. It also provides lending to mortgage banking businesses.	{"Financial Services","Banks - Regional - US",Banks}	{CZNC,NWFL,FRBK,BMTC,MPB,PWOD,STBA,UVSP}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.379957	2020-10-11 23:38:31.023428
CVA	Covanta Holding Corporation	Covanta Holding Corp is engaged in the businesses of waste and energy services. The company also holds interests in energy-from-waste facilities in Ireland and Italy.	{Industrials,"Waste Management"}	{CWST,HCCI,AES,CLH,WM,D,NRG,ORA,DUK}	Waste Management	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:01.399144	2020-10-11 23:38:31.021426
CRR	Carbo Ceramics Inc.	Carbo Ceramics Inc is engaged in the supply of ceramic proppant. It provides fracture simulation software, fracture design, engineering and consulting services and a broad range of technologies for spill prevention, containment and countermeasures.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{SLB,HAL,MMM,SLCA}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:01.268342	2020-10-11 23:38:31.028166
CRVL	CorVel Corp.	CorVel Corp provides case management, claims administration, and medical bill review services to payors to administer, manage and control the cost of workers compensation and other healthcare benefits.	{"Financial Services","Insurance Brokers","Brokers & Exchanges"}	{INFU,PINC,TVTY}	Brokers & Exchanges	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:01.279547	2020-10-11 23:38:31.028166
CSGP	CoStar Group Inc.	CoStar Group Inc provides commercial real estate data covering over 4 million properties from various subsectors from commercial areas. It operates five flagship brands: CoStar, LoopNet, Apartments.com, BizBuySell, and LandsofAmerica.	{"Real Estate","Real Estate Services"}	{TTGT}	Real Estate Services	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:01.295932	2020-10-11 23:38:31.028166
CSPI	CSP Inc.	CSP Inc and its subsidiaries are engaged in developing and marketing IT integration solutions, security and managed services and purpose built network adapters, as well as cluster computer systems.	{Technology,"Information Technology Services","Application Software"}	{CSCO,INTC,AGYS,GE,IBM,HPQ,SMCI,XLK}	Application Software	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:01.312606	2020-10-11 23:38:31.023428
CTAS	Cintas Corporation	Cintas Corp is a provider of corporate identity uniforms. It products include rental and sale of entrance mats, mops, shop towels, and restroom supplies. The company also offers safety supplies, fire-suppression systems maintenance and others.	{Industrials,"Business Services"}	{GNTX,HCSG,ABM,UNF,ARMK}	Business Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:01.330899	2020-10-11 23:38:31.028166
CTIC	CTI BioPharma Corp.	CTI BioPharma Corp is a biopharmaceutical company. It is focused on the acquisition, development and commercialization of novel targeted therapies.	{Healthcare,Biotechnology}	{AMGN,INCY,CELG,NVS,BAX,BMY,LLY,TEVA,MDGL,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:01.343822	2020-10-11 23:38:31.024353
CTS	CTS Corporation	CTS Corp is a designer and manufacturer of electronic components and actuators to OEMs in the automotive, communications, medical, defense and aerospace, industrial and computer markets. The company manufactures products in North America, Europe and Asia.	{Technology,"Electronic Components","Computer Hardware"}	{CLS,PLXS,IEC,JBL,AVX,BHE,VSH,ATU,VPG,SANM,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:01.363399	2020-10-11 23:38:31.025871
CUR	Neuralstem Inc.	Neuralstem Inc is a biotechnology company. It is engaged in the development and commercialization of treatments for central nervous system disease based on transplanting human neural stem cells and the use of small molecule drugs.	{Healthcare,Biotechnology}	{GERN,NBIX,ACET,ACOR,BTX,CRL,SUPN,CYTK,ITCI,TTNP,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:01.393061	2020-10-11 23:38:31.028166
CVGI	Commercial Vehicle Group Inc.	Commercial Vehicle Group Inc is a supplier of cab related products and systems for the global commercial vehicle market. The operating segments are global truck and bus (GTB) segment and global construction and agriculture (GCA) segment.	{"Consumer Cyclical","Auto Parts",Autos}	{MPAA,FSTR,NAV,TWI,WBC,LEA,SRI,MTOR,ALSN,VC}	Autos	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:01.41412	2020-10-11 23:38:31.028166
CVTI	Covenant Transportation Group Inc.	Covenant Transportation Group Inc, together with its wholly-owned subsidiaries offers truckload transportation and freight brokerage services to customers throughout the continental United States.	{Industrials,Trucking,"Transportation & Logistics"}	{PTSI,HTLD,MRTN,USAK,WERN,ULH,RRTS,KNX}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:01.428288	2020-10-11 23:38:31.023007
CWBR	CohBar Inc.	CohBar Inc is a research stage biotechnology company. The company conducts investigational research into Mitochondrial-Derived Peptides to evaluate their therapeutic potential through in vitro and in vivo models.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:01.439645	2020-10-11 23:38:31.023428
CXH	MFS Investment Grade Municipal Trust	Mfs Investment Grade Municipal Trust is a United States based diversified closed-end management investment company. Its investment objective is to seek high current income exempt from federal income tax, but may also consider capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.452855	2020-10-11 23:38:31.025871
CYOU	Changyou.com Limited	Changyou.com Ltd is an online game developer and operator in China. It develops, operates and licenses multiplayer online role-playing games.	{Technology,"Electronic Gaming & Multimedia","Application Software"}	{YY,GIGM,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:01.478742	2020-10-11 23:38:31.023007
CZWI	Citizens Community Bancorp Inc.	Citizens Community Bancorp Inc is a bank holding company. It provides consumer banking products and services in Wisconsin, Minnesota, and Michigan.	{"Financial Services","Banks - Regional - US",Banks}	{FDEF,HMNF,UCFC,FBC}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:01.490676	2020-10-11 23:38:31.02423
CRY	CryoLife Inc.	Cryolife Inc is engaged in medical device manufacturing and distribution and in the processing and distribution of implantable human tissues for use in cardiac and vascular surgeries. It has two segments: Medical Devices and Preservation Services.	{Healthcare,"Medical Devices"}	{IART,BAX,BSX,JNJ,SYK,TFX,EW,MDT,XLV}	Medical Devices	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:01.282601	2020-10-11 23:38:31.031328
CSFL	CenterState Bank Corporation	CenterState Bank Corp is a bank holding company, which through its subsidiaries, provides a range of consumer and commercial banking services to individuals, businesses and industries.	{"Financial Services","Banks - Regional - US",Banks}	{CCBG,CWST,STI,BKU,PESI,SBCF,SNV,OPHC}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:01.295698	2020-10-11 23:38:31.031328
CSOD	Cornerstone OnDemand Inc.	Cornerstone OnDemand Inc provides talent management solutions delivered as Software-as-a-Service. The company provides an integrated cloud-based platform to addresses talent needs through from recruitment, onboarding, and training up to analytics.	{Technology,"Software - Application","Application Software"}	{MSFT,ADP,ULTI,IBM,PCTY,PAYC,WDAY,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:01.309336	2020-10-11 23:38:31.02582
CSWC	Capital Southwest Corporation	Capital Southwest Corporation is an investment company in USA. Its investment objective to produce attractive risk-adjusted returns by generating current income from debt investments and capital appreciation from equity and equity related investments.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:01.325297	2020-10-11 23:38:31.021426
CTBI	Community Trust Bancorp Inc.	Community Trust Bancorp Inc is a bank holding company. The company provides commercial and personal banking and trust and wealth management activities.	{"Financial Services","Banks - Regional - US",Banks}	{CZFC,CIZN,RNST,SYBT,BXS}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:01.338181	2020-10-11 23:38:31.028166
CTSH	Cognizant Technology Solutions Corporation	Cognizant Technology Solutions Corp is an IT services provider. The company provides services such as technology consulting, application outsourcing, systems integration, business process services, and cloud services.	{Technology,"Information Technology Services","Application Software"}	{PRFT,IBM,WIT,ACN,INFY,HPQ,DXC,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:01.36389	2020-10-11 23:38:31.023007
CUBA	The Herzfeld Caribbean Basin Fund Inc.	Herzfeld Caribbean Basin Fund Inc operates as a closed-end management investment company. The Company's investment objective is to obtain long-term capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:01.377079	2020-10-11 23:38:31.025871
CUZ	Cousins Properties Incorporated	Cousins Properties Inc is a real estate investment trust engaged in developing, acquiring, leasing, managing and owning office properties and opportunistic mixed-use developments. Its properties are located mainly in North Carolina, Texas and Georgia.	{"Real Estate","REIT - Diversified",REITs}	{VNO,BXP,SLG,CLI,DEI,FOR}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:01.397522	2020-10-11 23:38:31.025871
CVGW	Calavo Growers Inc.	Calavo Growers Inc is in the avocado industry and an expanding provider of value-added fresh food. It sells avocados to a group of supermarket chains, wholesalers, food service and other distributors, under brand labels, as well as private labels.	{"Consumer Defensive","Farm Products","Consumer Packaged Goods"}	{UNFI,CORE,SYY,DIT}	Consumer Packaged Goods	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:01.41592	2020-10-11 23:38:31.023007
CWAI	CWA Income	The investment seeks to provide total return.\n Under normal circumstances, the fund seeks to achieve its investment objective by investing its assets in a diversified portfolio of corporate bonds with varying maturities and equity securities. It normally will not invest more than 20% of its total assets in equity securities. The fund may invest without limit in securities denominated in foreign currencies and may invest without limit in U.S. dollar-denominated securities of foreign issuers.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:01.433132	2020-10-11 23:38:31.031328
CYAD	Celyad SA	Celyad SA is a biopharmaceutical company. The company is engaged in stem cell-based therapies for the treatment of cardiovascular diseases. It is developing therapeutic therapies based on two distinct technology platforms in cardiology and oncology.	{Healthcare,Biotechnology}	{BSTC,ICLR,CTLT,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:01.464179	2020-10-11 23:38:31.023007
CYHHZ	Community Health Systems Inc. Series A Contingent Value Rights	Community Health Systems Inc is a operators of general acute care hospitals. The company operates in hospital operations which includes its general acute care hospitals and related healthcare entities.	{Healthcare,"Medical Care","Health Care Providers"}	{XLV}	Health Care Providers	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:01.478343	2020-10-11 23:38:31.024353
CZNC	Citizens & Northern Corp	Citizens & Northern Corp is a Pennsylvania based community bank operating through its subsidiaries. It provides banking services, including deposit and loan products for personal and commercial customers.	{"Financial Services","Banks - Regional - US",Banks}	{ORRF,NWFL,FRBK,BMTC,PWOD,STBA,UVSP,ACNB,CUBI}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:01.490043	2020-10-11 23:38:31.023428
CSM	ProShares Large Cap Core Plus	The investment seeks investment results, before fees and expenses, that track the performance of the Credit Suisse 130/30 Large Cap Index (the "index").\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should track the performance of the index. The index is designed to replicate an investment strategy that establishes either long or short positions in the stocks of 500 leading large-cap U.S. companies (the "Universe") by applying a rules-based ranking and weighting methodology. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:01.303581	2020-10-11 23:38:31.024353
CSSE	Chicken Soup for the Soul Entertainment Inc.	Chicken Soup for the Soul Entertainment Inc is a producer and distributor of television and online video programs.	{"Consumer Cyclical","Broadcasting - TV",Entertainment}	{MSGN}	Entertainment	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:01.317313	2020-10-11 23:38:31.023007
CSWI	CSW Industrials Inc.	CSW Industrials Inc is a diversified industrial growth company with well-established, scalable platforms and domain expertise across three segments, Industrial Products, and Specialty Chemicals.	{Industrials,"Diversified Industrials","Industrial Products"}	{QCOM,SCKT,PRKR}	Industrial Products	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:01.328027	2020-10-11 23:38:31.023428
CTIB	CTI Industries Corporation	CTI Industries Corp develops, produces and distributes consumer products. The company is engaged in producing film products for commercial and industrial use. The company sells its products throughout United States.	{"Consumer Cyclical","Rubber & Plastics",Autos}	{TRS,ATR,TUP,SEE,ACU,CSS,MYE,MMM,BERY}	Autos	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:01.341663	2020-10-11 23:38:31.023428
CTT	CatchMark Timber Trust Inc. Class A	CatchMark Timber Trust Inc is a real estate investment trust. It primarily invests in timberlands and its earnings originate from the sale of timber. Most of its operations are restricted to the United States.	{"Real Estate","REIT - Diversified",REITs}	{DRE,JOE,PCH,INN,PW,LAND,SIR}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:01.366169	2020-10-11 23:38:31.02582
CUE	Cue Biopharma Inc.	Cue Biopharma Inc is a biopharmaceutical company developing a novel and proprietary class of biologic drugs for the selective modulation of the human immune system to treat a broad range of cancers and autoimmune disorders.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:01.38058	2020-10-11 23:38:31.02582
CURE	Direxion Daily Healthcare Bull 3X Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the daily performance of the Health Care Select Sector Index.\n The fund invests at least 80% of its net assets in securities of the index, ETFs that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index includes domestic companies from the healthcare sector, which includes the following industries: pharmaceuticals; health care equipment and supplies; health care providers and services; biotechnology; life sciences tools and services; and health care technology. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.391544	2020-10-11 23:38:31.023007
CVE	Cenovus Energy Inc	Cenovus Energy Inc is engaged in developing, producing and marketing crude oil, natural gas liquids and natural gas in Canada with marketing activities and refining operations in the United States.	{Energy,"Oil & Gas Integrated","Oil & Gas - Integrated"}	{CNQ,IMO,SU,APA,EOG,MRO,PSX}	Oil & Gas - Integrated	Energy	New York Stock Exchange	\N	2020-10-11 23:12:01.40584	2020-10-11 23:38:31.028166
CVLY	Codorus Valley Bancorp Inc	Codorus Valley Bancorp Inc is a bank holding company. The Company through its subsidiaries offers business and consumer banking services. It also provides personal banking, mortgage banking, wealth management and real estate settlement services.	{"Financial Services","Banks - Regional - US",Banks}	{CZNC,ORRF,EMCF,NWFL,BMTC,CCNE,FULT,STBA,FNB}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:01.415417	2020-10-11 23:38:31.02423
CVV	CVD Equipment Corporation	CVD Equipment Corp designs and manufactures chemical vapor deposition, gas control and other equipment and process solutions used to develop and manufacture materials and coatings for research and industrial applications.	{Industrials,"Diversified Industrials","Industrial Products"}	{AMAT,VECO,ASYS,LRCX,ACLS}	Industrial Products	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:01.431719	2020-10-11 23:38:31.024353
CWST	Casella Waste Systems Inc.	Casella Waste Systems Inc is a vertically-integrated solid waste, recycling, and resource management services company. It provides resource management expertise and services to residential, commercial, municipal and industrial customers.	{Industrials,"Waste Management"}	{IDSA,RSG,WM,PESI,WCN}	Waste Management	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:01.448109	2020-10-11 23:38:31.028166
CY	Cypress Semiconductor Corporation	Cypress Semiconductor Corp operates as a semiconductor company in the United States and internationally. It delivers various high-performance, mixed-signal, programmable solutions.	{Technology,Semiconductors}	{MCHP,IDTI,MXIM,SMTC,SLAB,ON,TXN,ADI,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:01.461112	2020-10-11 23:38:31.028166
CYCCP	Cyclacel Pharmaceuticals Inc. 6% Convertible Preferred Stock	Cyclacel Pharmaceuticals Inc mainly develops targeted medicines for cancer and other proliferative diseases. The company focuses in hematology and oncology-based development pipeline of novel drug candidates.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:01.471438	2020-10-11 23:38:31.025871
CSQ	Calamos Strategic Total Return Fund	Calamos Strategic Total Return Fund is a a diversified, closed-end management investment company. The fund's investment strategy is to provide total return through a combination of capital appreciation and current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:01.313377	2020-10-11 23:38:31.031328
CSWCL	Capital Southwest Corporation 5.95% Notes due 2022		{}	{}			Nasdaq Global Select	\N	2020-10-11 23:12:01.328079	2020-10-11 23:38:31.031328
CTG	Computer Task Group Incorporated	Computer Task Group Inc is an IT solutions and staffing services company with operations in North America and Europe. The company's services include IT solutions and IT and other staffing.	{Technology,"Information Technology Services","Application Software"}	{KFRC,IBM,MAN,RHI,MHH,TSRI,NTWK,JOB,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:01.340378	2020-10-11 23:38:31.021426
CTRE	CareTrust REIT Inc.	CareTrust REIT Inc is engaged in the ownership, acquisition and leasing of healthcare-related properties. The company also owns and operates independent living facilities.	{"Real Estate","REIT - Healthcare Facilities",REITs}	{GTY,EXR,WELL}	REITs	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:01.353251	2020-10-11 23:38:31.023428
CTXRW	Citius Pharmaceuticals Inc. Warrant		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:01.370711	2020-10-11 23:38:31.024353
CUK	Carnival Plc ADS ADS	Carnival PLC is an UK-based cruise company providing leisure travel to major cruise destinations throughout the world. Its portfolio of cruise brands includes AIDA, Carnival Cruise Line, Costa, Cunard, Holland America Line, and other.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{RCL,CCL,MAR,NCLH}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:01.388859	2020-10-11 23:38:31.031328
CVBF	CVB Financial Corporation	CVB Financial Corp is a bank holding company for Citizens Business Bank, which provides banking, lending, and investment services. It also provides trust and investment-related services to customers through its CitizensTrust Division.	{"Financial Services","Banks - Regional - US",Banks}	{AMRB,CATY,SRCE,PMBC,CVCY,BSRR,PPBI,BOCH,PACW,BXS}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:01.40411	2020-10-11 23:38:31.024353
CVNA	Carvana Co. Class A	Carvana Co is an eCommerce platform for buying used cars. It uses proprietary algorithms to optimize its nationally pooled inventory of over 7,300 vehicles, inspect and recondition our vehicles and operate its own logistics network.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{KMX,KAR,CVNA}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:01.419252	2020-10-11 23:38:31.028166
CVX	Chevron Corporation	Chevron Corp engages in the exploration, production, and refining operations worldwide. Its oil refineries are spread across the United States, South Africa, and Asia with an approximate capacity of 2 million barrels of oil a day.	{Energy,"Oil & Gas Integrated","Oil & Gas - Integrated"}	{SU,TOT,PBR.A,XOM,BP,COP,RDS.A,HES,PSX}	Oil & Gas - Integrated	Energy	New York Stock Exchange	\N	2020-10-11 23:12:01.43127	2020-10-11 23:38:31.025871
CXDC	China XD Plastics Company Limited	China XD Plastics company Ltd manufactures plastics for automobiles. Its product includes automobile exteriors, for car interiors and other components.	{"Consumer Cyclical","Rubber & Plastics",Autos}	{UFPT,CMT,ASPN}	Autos	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:01.451232	2020-10-11 23:38:31.021426
CYBE	CyberOptics Corporation	CyberOptics Corp is engaged in developing & manufacturing high precision sensing technology solutions. It manufactures 3D and 2D optical sensors for use in its own proprietary SMT inspection systems and for sale to original equipment manufacturers.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{AMAT,CVV,ASYS,CGNX,DAIO,KLAC,KLIC,BRKS,DOV,RTEC,XLK}	Computer Hardware	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:01.465546	2020-10-11 23:38:31.024353
CYTK	Cytokinetics Incorporated	Cytokinetics Inc is a biopharmaceutical company. It is engaged in the discovery and development of novel small molecule therapeutics that modulate muscle function for the potential treatment of serious diseases and medical conditions.	{Healthcare,Biotechnology}	{AMGN,ARQL,ARRY,ACOR,LGND,TRVN,ABIO,EIGR,GTXI,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:01.479301	2020-10-11 23:38:31.02423
CZZ	Cosan Limited Class A	Cosan Ltd is engaged in piped natural gas distribution, logistics services including transportation, port loading and storage of sugar, purchase, sale and leasing of agricultural land, production and distribution of lubricants and other investments.	{Utilities,"Utilities - Diversified","Utilities - Regulated"}	{UGP,PBR,BG,RDS.A,SEB,AGRO,MUSA,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:01.491872	2020-10-11 23:38:31.028166
DBE	Invesco DB Energy Fund	The investment seeks to track the DBIQ Optimum Yield Energy Index Excess Return™ (DBIQ-OY Energy ER™), which is intended to reflect the energy sector.\n The index Commodities consist of Light, Sweet Crude Oil (WTI), Heating Oil, Brent Crude Oil, RBOB Gasoline and Natural Gas.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.528502	2020-10-11 23:38:31.023007
DBP	Invesco DB Precious Metals Fund	The investment seeks to track the DBIQ Optimum Yield Energy Index Excess Return™ (DBIQ-OY Energy ER™), which is intended to reflect the energy sector.\n The index Commodities consist of Light, Sweet Crude Oil (WTI), Heating Oil, Brent Crude Oil, RBOB Gasoline and Natural Gas.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.542211	2020-10-11 23:38:31.021426
CSS	CSS Industries Inc.	CSS Industries Inc is a consumer products company. It is engaged in the design, manufacture, procurement, distribution and sale of seasonal and all occasion social expression products to mass market retailers.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{CTIB,SFLY,DLX,WMT,MMM,ACCO}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:01.318067	2020-10-11 23:38:31.028166
CTB	Cooper Tire & Rubber Company	Cooper Tire & Rubber Co is an American tire producer. It is engaged in manufacturing and marketing replacement tires. The company specializes in the design, manufacture, marketing and sales of passenger car and light truck tires.	{"Consumer Cyclical","Rubber & Plastics",Autos}	{GT}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:01.330975	2020-10-11 23:38:31.024353
CTMX	CytomX Therapeutics Inc.	CytomX Therapeutics Inc is an oncology-focused biopharmaceutical company which is engaged in developing antibody therapeutics based on probody technology platform.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:01.350455	2020-10-11 23:38:31.031328
CTWS	Connecticut Water Service Inc.	Connecticut Water Service Inc is a non-operating holding company. The company is engaged in supplying water to its customers. It operates in three segments; Water Activities, Real Estate Transactions, and Services and Rentals.	{Utilities,"Utilities - Regulated Water","Utilities - Regulated"}	{YORW,ARTNA,MSEX,CDZI,CWT,AWR,WTR,AWK,SJW,XLU}	Utilities - Regulated	Utilities	Nasdaq Global Select	\N	2020-10-11 23:12:01.368448	2020-10-11 23:38:31.023428
CUI	CUI Global Inc.	CUI Global Inc is An electronics platform company. The company operates in two reportable segments: Power and Electromechanical segment and Energy segment. It generates a majority of its revenue from the Power and Electromechanical segment.	{Technology,"Electronic Components","Computer Hardware"}	{VICR,IIN,A,EMR,QRVO,QBAK,XLK}	Computer Hardware	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:01.38188	2020-10-11 23:38:31.021426
CVI	CVR Energy Inc.	CVR Energy Inc through its subsidiaries is engaged in the petroleum refining and nitrogen fertilizer manufacturing industries. Its business segments include petroleum and nitrogen fertilizer.	{Energy,"Oil & Gas Refining & Marketing","Oil & Gas - Refining & Marketing"}	{CLMT,NFX,VLO,MPC,HFC,PSX,DK}	Oil & Gas - Refining & Marketing	Energy	New York Stock Exchange	\N	2020-10-11 23:12:01.4146	2020-10-11 23:38:31.02582
CVS	CVS Health Corporation	CVS Health Corp operates in the healthcare sector. Its primary business is that of operating retail pharmacies. The company has three reportable segments namely Pharmacy Services, Retail/LTC, and Corporate.	{Healthcare,"Health Care Plans"}	{PETS,BIOS,RAD,WBA,XLV}	Health Care Plans	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:01.426322	2020-10-11 23:38:31.02423
CWCO	Consolidated Water Co. Ltd.	Consolidated Water Co Ltd develops and operates seawater desalination plants. The company's operating segment includes Retail, bulk, services and manufacturing. It generates a majority of its revenue from the Bulk segment.	{Utilities,"Utilities - Regulated Water","Utilities - Regulated"}	{YORW,CTWS,MSEX,CWT,AWR,SBS,AWK,XLU}	Utilities - Regulated	Utilities	Nasdaq Global Select	\N	2020-10-11 23:12:01.440487	2020-10-11 23:38:31.023007
CXO	Concho Resources Inc.	Concho Resources Inc is an oil and natural gas company. It is engaged in the acquisition, development, exploitation and exploration of producing oil and natural gas properties. It operates in the Delaware Basin.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{PXD,APA,EGN,COP,LPI}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:01.45314	2020-10-11 23:38:31.02423
CYRN	CYREN Ltd.	CYREN Ltd is a provider of cloud-based security technology solutions against cyber threats, malware attacks, information leaks, legal liability and productivity loss. Its solutions protect the web, email and mobile transactions.	{Technology,"Software - Infrastructure","Application Software"}	{INTC,MSFT,CHKP,TISA,SYMC,ZIXI,IBM,PFPT,QLYS,GOOGL,XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:01.481086	2020-10-11 23:38:31.023428
DAC	Danaos Corporation	Danaos Corp owns and operates containerships, chartering its vessels to container shipping companies across the world. It deploys its containership fleet mainly under multi-year charters with liner companies.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{DSX,SSW,NM,NMM,CMRE,ESEA,DCIX}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:01.498167	2020-10-11 23:38:31.023007
DARE	Dare Bioscience Inc.	Dare Bioscience Inc is a healthcare company. The company develops and commercializes products in women's reproductive health. Its segment includes spectrum of categories like: contraception, vaginal health, pain and fertility.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:01.512203	2020-10-11 23:38:31.028166
DBAW	Xtrackers MSCI All World ex US Hedged Equity	The investment seeks investment results that correspond generally to the performance, of the MSCI ACWI ex USA US Dollar Hedged Index.\n The fund, using a "passive" or indexing investment approach, seeks investment results that correspond generally to the performance, of the underlying index, which is designed to track the performance of equity securities in developed and emerging stock markets while mitigating exposure to fluctuations between the value of the USD and the currencies of the countries included in the underlying index. It will invest at least 80% of its total assets in component securities of the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.524414	2020-10-11 23:38:31.025871
CTEK	CynergisTek Inc.	CynergisTek Inc is the United States-based outsourced document management solutions provider. It is engaged in the business of providing fully-outsourced managed document services, workflow solutions and IT security consulting services.	{Industrials,"Business Services"}	{EXLS,III,SFE,PFMT,ARC}	Business Services	Industrials	NYSE American	\N	2020-10-11 23:12:01.340411	2020-10-11 23:38:31.025871
CTRL	Control4 Corporation	Control4 Corp provides solutions and products for home controlling and smart home. The company also sells third party connected devices through its own online store.	{Technology,"Electronic Components","Computer Hardware"}	{FARO,AAPL,ELSE,AME,VPG,SANM,GOOGL,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:01.359447	2020-10-11 23:38:31.024353
CTXS	Citrix Systems Inc.	Citrix Systems Inc provides virtualization, networking and cloud infrastructure solutions. It provides a complete and integrated portfolio of application delivery, virtualization, mobility, network delivery and file sharing solutions.	{Technology,"Software - Infrastructure","Application Software"}	{CSCO,MSFT,ADBE,LOGM,RHT,IBM,VMW,ORCL,HPQ,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:01.371269	2020-10-11 23:38:31.02423
CULP	Culp Inc.	Culp Inc manufacturers, sources, and markets mattress fabrics and sewn covers used for covering mattresses & box springs and upholstery fabrics including cut and sewn kits used in production of upholstered furniture.	{"Consumer Cyclical","Textile Manufacturing","Homebuilding & Construction"}	{CRWS,FLXS,AIN,LZB,LEG,TPX,UFI}	Homebuilding & Construction	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:01.388678	2020-10-11 23:38:31.024353
CVCO	Cavco Industries Inc.	Cavco Industries Inc designs and produces factory-built homes under the Cavco Homes, Fleetwood Homes, and Palm Harbor Homes brands. It also produces modular homes, park model homes and vacation cabins, as well as commercial structures, among others.	{"Consumer Cyclical","Residential Construction","Homebuilding & Construction"}	{PATK,GFN,SKY}	Homebuilding & Construction	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:01.403753	2020-10-11 23:38:31.02423
CVM	Cel-Sci Corporation ($0.001 Par Value)	CEL-SCI Corp is a biotechnology company. It is engaged in the research and development directed at improving the treatment of cancer and other diseases by utilizing the immune system, the body's natural defense system.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NYSE American	\N	2020-10-11 23:12:01.416701	2020-10-11 23:38:31.024353
CWT	California Water Service Group	California Water Service Group is engaged in the production, purchase, storage, treatment, testing, distribution and sale of water for domestic, industrial, public and irrigation uses, and for fire protection.	{Utilities,"Utilities - Regulated Water","Utilities - Regulated"}	{YORW,ARTNA,CTWS,MSEX,CDZI,PCYO,AWR,WTR,AWK,SJW,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:01.447661	2020-10-11 23:38:31.031328
CYBR	CyberArk Software Ltd.	CyberArk Software Ltd operates in the software infrastructure industry. It sells software platforms which protect its clients from internet security breaches.	{Technology,"Software - Infrastructure","Application Software"}	{JKHY,NOW,LDOS,BOX,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:01.468222	2020-10-11 23:38:31.02423
CYTXW	Cytori Therapeutics Inc. Warrant	Cytori Therapeutics Inc is a specialty therapeutics company. It is committed to providing meaningful and quality therapeutic options with a broad utility that benefit patients and healthcare providers around the world.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:01.487472	2020-10-11 23:38:31.021426
DAKT	Daktronics Inc.	Daktronics Inc along with its subsidiaries is engaged in the design, manufacture and sale of electronic display systems and related products for Commercial, Live Events, Schools and Theatres, and Transportation.	{Technology,"Computer Systems","Computer Hardware"}	{LYTS,BTN,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:01.503114	2020-10-11 23:38:31.024353
DBA	Invesco DB Agriculture Fund	The investment seeks to track changes, whether positive or negative, in the level of the DBIQ Diversified Agriculture Index Excess Return™ (the “index”) over time, plus the excess, if any, of the sum of the fund’s Treasury Income, Money Market Income and T-Bill ETF Income, over the expenses of the fund. The index, which is comprised of one or more underlying commodities (“index commodities”), is intended to reflect the agricultural sector.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.518393	2020-10-11 23:38:31.02582
DBGR	Xtrackers MSCI Germany Hedged Equity	The investment seeks investment results that correspond generally to the performance, before fees and expenses, of the MSCI Germany US Dollar Hedged Index.\n The fund, using a "passive" or indexing investment approach, seeks investment results that correspond generally to the performance, before fees and expenses, of the underlying index, which is designed to track the performance of the German equity market while mitigating exposure to fluctuations between the value of the U.S. dollar and the euro. It will invest at least 80% of its total assets (but typically far more) in component securities of the underlying index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.53431	2020-10-11 23:38:31.025871
DCAR	DropCar Inc.	DropCar Inc is a provider of app-based automotive logistics and concierge services for both consumers and the automotive industry.	{Technology,"Software - Application","Application Software"}	{XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:01.547665	2020-10-11 23:38:31.025871
CTRN	Citi Trends Inc.	Citi Trends Inc is a retailer of urban fashion apparel and accessories for the entire family. The Company's merchandise offerings are designed to appeal to the preferences of fashion conscious consumers, particularly African-Americans.	{"Consumer Cyclical","Apparel Stores","Retail - Apparel & Specialty"}	{URBN,ROST,SMRT,ANF,AEO,BKE,GPS,JWN,TJX,SSI}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:01.359495	2020-10-11 23:38:31.021426
CTY	Qwest Corporation 6.125% Notes due 2053		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:01.373536	2020-10-11 23:38:31.028166
CURO	CURO Group Holdings Corp.	CURO Group Holdings Corp is a consumer finance company. It offers a broad range of consumer finance products including unsecured installment loans, secured installment loans, open-end loans and single-pay loans.	{"Financial Services","Credit Services"}	{}	Credit Services	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.392199	2020-10-11 23:38:31.023428
CVEO	Civeo Corporation (Canada)	Civeo Corp is an integrated provider of long-term and temporary remote site accommodations, logistics and facility management services to the natural resource industry. It generates most of its revenues from Australia and the United States.	{Industrials,"Business Services"}	{ULTI,BSQR,GEC}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:01.405938	2020-10-11 23:38:31.023428
CVY	Invesco Zacks Multi-Asset Income	The investment seeks to track the investment results (before fees and expenses) of the Zacks Multi-Asset Income Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in securities that comprise the underlying index and depositary receipts representing securities that comprise the underlying index. The underlying index was comprised of approximately 149 securities selected, based on investment and other criteria, from a universe of domestic and international companies listed on major U.S. exchanges. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.43128	2020-10-11 23:38:31.028166
CX	Cemex S.A.B. de C.V. Sponsored ADR	Cemex SAB de CV is engaged in the production, marketing, distribution and sale of cement, ready-mix concrete, aggregates and other construction materials.	{"Basic Materials","Building Materials"}	{MLM,CRH,VMC,EXP}	Building Materials	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:01.449225	2020-10-11 23:38:31.023007
CXSE	WisdomTree China Dividend Ex-Financials Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree China ex-State-Owned Enterprises Index.\n Under normal circumstances, at least 80% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a float-adjusted market cap weighted index that consists of common stocks in China, excluding common stocks of "state-owned enterprises." The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.459705	2020-10-11 23:38:31.02582
CYCC	Cyclacel Pharmaceuticals Inc.	Cyclacel Pharmaceuticals Inc mainly develops targeted medicines for cancer and other proliferative diseases. The company focuses in hematology and oncology-based development pipeline of novel drug candidates.	{Healthcare,Biotechnology}	{CELG,ARRY,SPPI,JNJ,LLY,STML,CASI,ABEO,SNSS,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:01.471471	2020-10-11 23:38:31.021426
CYTR	CytRx Corporation	CytRx Corp is a biopharmaceutical research and development company in the United States. Its key drug candidate is aldoxorubicin, which is developed to address serious diseases such as soft tissue sarcoma (STS).	{Healthcare,Biotechnology}	{AMGN,CELG,SGEN,CLDX,GSK,NVS,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:01.484574	2020-10-11 23:38:31.031328
D	Dominion Energy Inc.	Dominion Energy Inc is a producer and transporter of energy. It manages its daily operations through three operating segments namely Dominion Virginia Power of DVP, Dominion Energy and Dominion Generation.	{Utilities,"Utilities - Diversified","Utilities - Regulated"}	{ETR,NI,FE,AEE,SRE,CMS,EXC,SCG,CNP,DUK,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:01.498057	2020-10-11 23:38:31.021426
DAL	Delta Air Lines Inc.	Delta Air Lines Inc provides scheduled air transportation for passengers and cargo throughout the United States and across the world.	{Industrials,Airlines}	{SKYW,JBLU,HA,ALGT,CEA,ALK,LUV,AAL,SAVE,UAL}	Airlines	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:01.508533	2020-10-11 23:38:31.02582
DB	Deutsche Bank AG	Deutsche Bank AG is a leading private bank. It offers investment, financial and related products and services to private individuals, corporate entities and institutional clients around the world.	{"Financial Services","Banks - Regional - Europe",Banks}	{BCS,CS,SAN,GS,HSBC,JPM}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.518321	2020-10-11 23:38:31.028166
DBEZ	Xtrackers MSCI Eurozone Hedged Equity	The investment seeks investment results that correspond generally to the performance, before fees and expenses, of the MSCI EMU IMI US Dollar Hedged Index.\n The fund, using a "passive" or indexing investment approach, seeks investment results that correspond generally to the performance, of the underlying index, which is designed to track the performance of equity securities based in the countries in the European Monetary Union, while seeking to mitigate exposure to fluctuations between the value of the U.S. dollar and the euro. It will invest at least 80% of its total assets in component securities of the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.533343	2020-10-11 23:38:31.028166
CUB	Cubic Corporation	Cubic Corp caters to the defense industry. It designs and develops of technology systems and products, which are made to suit defense requirements.	{Industrials,"Aerospace & Defense"}	{CAE,FLIR,KTOS,LMT,CW,GD,NOC,TDY,COL,LLL}	Aerospace & Defense	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:01.375187	2020-10-11 23:38:31.028166
CUO	Continental Materials Corporation	Continental Materials Corp manufactures heating and cooling systems. The company operates within two industry groups, Heating, Ventilation and Air Conditioning and Construction Products.	{Industrials,"Diversified Industrials","Industrial Products"}	{AAON,LII,AOS,IR}	Industrial Products	Industrials	NYSE American	\N	2020-10-11 23:12:01.388825	2020-10-11 23:38:31.02423
CVCY	Central Valley Community Bancorp	Central Valley Community Bancorp is a bank holding company, which through its subsidiary provides financial services in its main market area in California. Its services include personal banking and business banking services.	{"Financial Services","Banks - Regional - US",Banks}	{TCBK,AMRB,CWBC,CVBF,EWBC,BSRR,PPBI,BMRC,OVLY}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:01.40424	2020-10-11 23:38:31.031328
CVRS	Corindus Vascular Robotics Inc. (DE)	Corindus Vascular Robotics Inc is engaged in the design, manufacture, and sale of precision vascular robotic-assisted systems for use in interventional vascular procedures the CorPath System. Its product includes the CorPath 200 and CorPath GRX Systems.	{Healthcare,"Medical Devices"}	{BSX,XLV}	Medical Devices	Healthcare	NYSE American	\N	2020-10-11 23:12:01.424622	2020-10-11 23:38:31.023428
CWBC	Community West Bancshares	Community West Bancshares is a bank holding company in the United States. It earns revenue through the interest income derived from the provision of lending products.	{"Financial Services","Banks - Regional - US",Banks}	{TCBK,CATY,HTBK,EWBC,CVCY,BSRR,PPBI,SIVB,OPB}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:01.436722	2020-10-11 23:38:31.02423
CXW	CoreCivic Inc.	CoreCivic Inc is a real estate investment trust that offers site, design, build, finance, own, maintain and lease core real estate assets to government partners. The company is involved in the ownership and operation of private prisons.	{"Real Estate","REIT - Diversified",REITs}	{GEO}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:01.460852	2020-10-11 23:38:31.028166
CYD	China Yuchai International Limited	China Yuchai International Ltd manufactures, assembles and sells a variety of light, medium and heavy-duty engines for trucks, buses, passenger vehicles, construction equipment and marine and agriculture applications in China.	{Industrials,"Diversified Industrials","Industrial Products"}	{BGG,BC,CMI}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:01.471713	2020-10-11 23:38:31.02582
CYTX	Cytori Therapeutics Inc.	Cytori Therapeutics Inc is a specialty therapeutics company. It is committed to providing meaningful and quality therapeutic options with a broad utility that benefit patients and healthcare providers around the world.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:01.485205	2020-10-11 23:38:31.02582
DAIO	Data I/O Corporation	Data I/O Corp is a provider of advanced programming and intellectual property management solutions which are used in the manufacturing of flash memory, microcontrollers and flash-memory based intelligent devices.	{Technology,"Electronic Components","Computer Hardware"}	{MSFT,AEHR,CYBE,JBL,MMM,WTT,MSI,SANM,AVT,XLK}	Computer Hardware	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:01.502136	2020-10-11 23:38:31.028166
DAVE	Famous Dave's of America Inc.	Famous Dave's of America Inc is a casual dining barbeque bar-and-grill chain. It develops, owns, and operates restaurants in US which offers smoked, barbeque, grilled meat, and entree items using prepared proprietary seasonings, sauces, and mixes.	{"Consumer Cyclical",Restaurants}	{ARKR,TXRH,BJRI,KONA,EAT,BDL,CHUY,BLMN,SAUC}	Restaurants	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:01.51834	2020-10-11 23:38:31.031328
DBEM	Xtrackers MSCI Emerging Markets Hedged Equity	The investment seeks investment results that correspond generally to the performance, before fees and expenses, of the MSCI EM US Dollar Hedged Index.\n The fund, using a "passive" or indexing investment approach, seeks investment results that correspond generally to the performance, before fees and expenses, of the underlying index, which is designed to track emerging market performance while mitigating exposure to fluctuations between the value of the U.S. dollar and the currencies of the countries included in the underlying index. It will invest at least 80% of its total assets in component securities of the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.530336	2020-10-11 23:38:31.02582
DBO	Invesco DB Oil Fund	The investment seeks to track the DBIQ Optimum Yield Crude Oil Index Excess Return™ (DBIQ-OY CL ER™), which is intended to reflect the changes in market value of crude oil. \n The single index Commodity consists of Light, Sweet Crude Oil (WTI).	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.542135	2020-10-11 23:38:31.02423
DCIX	Diana Containerships Inc.	Diana Containerships Inc is a provider of shipping transportation services through its ownership of containerships.	{Industrials,"Integrated Shipping & Logistics","Transportation & Logistics"}	{DSX,DAC,CMRE,ESEA,SHIP}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:01.554071	2020-10-11 23:38:31.023428
DDF	Delaware Investments Dividend & Income Fund Inc.	Delaware Investments Div & Income Fund is a closed-end management investment company. Its investment objective is to seek high current income; capital appreciation is a secondary objective.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.568702	2020-10-11 23:38:31.025871
CUBE	CubeSmart	CubeSmart is a real estate investment trust that acquires, owns, and manages self-storage facilities throughout the United States. Its portfolio consists of buildings with numerous enclosed storage areas for both residential and commercial customers.	{"Real Estate","REIT - Industrial",REITs}	{PSA,EXR,LSI}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:01.380123	2020-10-11 23:38:31.023007
CUTR	Cutera Inc.	Cutera Inc is a medical device company. The company is engaged in designing, development, manufacturing, marketing and servicing of laser and other energy based aesthetics systems for practitioners.	{Healthcare,"Medical Devices"}	{MSON,IRIX,DYNT,FCRE,XLV}	Medical Devices	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:01.397433	2020-10-11 23:38:31.02582
CW	Curtiss-Wright Corporation	Curtiss-Wright Corp is a diversified manufacturing and service company. It designs, manufactures, and overhauls precision components and provides engineered products and services to the aerospace, defense, automotive, and metalworking industries.	{Industrials,"Diversified Industrials","Industrial Products"}	{WWD,LMT,GD,MOG.A,NOC,TXT,UTX,TDY,COL,MWA}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:01.431321	2020-10-11 23:38:31.02582
CWEB	Direxion Daily CSI China Internet Index Bull 2X Shares	The investment seeks daily investment results, of 200% of the daily performance of the CSI Overseas China Internet Index.\n The fund invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, ETFs that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index is designed to measure the performance of the investable universe of publicly traded China-based companies whose primary business or businesses are in the Internet and Internet-related sectors, as defined by CSI. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.441504	2020-10-11 23:38:31.028166
CXE	MFS High Income Municipal Trust	Mfs High Income Municipal Trust is a diversified closed-end management investment company. Its investment objective is to seek high current income exempt from federal income tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.451889	2020-10-11 23:38:31.024353
CYAN	Cyanotech Corporation	Cyanotech Corp cultivates and produces natural products derived from microalgae for the nutritional supplements market. The company's products include Spirulina Pacifica and natural astaxanthin.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{NATR,PRPH,CBM,USNA,MTEX}	Consumer Packaged Goods	Consumer Defensive	NASDAQ Capital Market	\N	2020-10-11 23:12:01.464464	2020-10-11 23:38:31.031328
CYH	Community Health Systems Inc.	Community Health Systems Inc is a operators of general acute care hospitals. The company operates in hospital operations which includes its general acute care hospitals and related healthcare entities.	{Healthcare,"Medical Care","Health Care Providers"}	{LPNT,UHS,SEM,SSY,HCA,THC,XLV}	Health Care Providers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:01.47677	2020-10-11 23:38:31.028166
CZR	Caesars Entertainment Corporation	Caesars Entertainment Corp along is engaged in providing casino-entertainment and hospitality services. Its facilities include gaming offerings, food and beverage outlets, hotel and convention space, and non-gaming entertainment options.	{"Consumer Cyclical","Resorts & Casinos","Travel & Leisure"}	{SGMS,UWN,FLL,ERI,GDEN,NYNY}	Travel & Leisure	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:01.490701	2020-10-11 23:38:31.02582
DALT	Anfield Capital Diversified Alternatives ETF	The investment seeks to provide capital growth and income.\n The fund is an actively managed exchange traded fund ("ETF") that is a fund of funds. It seeks to achieve its investment objective by investing primarily in alternative asset classes and securities that represent sectors, market segments or asset classes that do not represent the general investment universe. The fund will implement this strategy primarily through investments in unaffiliated ETFs, closed-end funds ("CEFs"), business development companies ("BDCs") and real estate investment trusts ("REITs").	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:01.508641	2020-10-11 23:38:31.023428
DBAP	Xtrackers MSCI Asia Pacific ex Japan Hedged Equity	The investment seeks investment results that correspond generally to the performance, of the MSCI Asia Pacific ex Japan US Dollar Hedged Index.\n The fund, using a "passive" or indexing investment approach, seeks investment results that correspond generally to the performance, of the underlying index, which is designed to track developed and emerging market performance in the Asia Pacific region while mitigating exposure to fluctuations between the value of the USD and the currencies of the countries included in the underlying index. It will invest at least 80% of its total assets in component securities of the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.522318	2020-10-11 23:38:31.023428
DBEU	Xtrackers MSCI Europe Hedged Equity	The investment seeks investment results that correspond generally to the performance, before fees and expenses, of the MSCI Europe US Dollar Hedged Index.\n The fund, using a "passive" or indexing investment approach, seeks investment results that correspond generally to the performance, of the underlying index, which is designed to track the performance of the developed markets in Europe, while mitigating exposure to fluctuations between the value of the U.S. dollar and the currencies of the countries included in the underlying index. It will invest at least 80% of its total assets in component securities of the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.53264	2020-10-11 23:38:31.031328
FINZ	ProShares UltraPro Short Financial Select Sector	The investment seeks daily investment results, before fees and expenses, that correspond to three times the inverse (-3x) of the daily performance of the S&P Financial Select Sector Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index represents the financial industry group of the S&P 500 Index ("S&P 500"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.979477	2020-10-11 23:38:31.02423
CVU	CPI Aerostructures Inc.	CPI Aerostructures Inc is engaged in the contract production of structural aircraft parts for fixed wing aircraft and helicopters in both the commercial and defense markets.	{Industrials,"Aerospace & Defense"}	{ATRO,TGI,BA,DCO,TDG,SPR,AIRI}	Aerospace & Defense	Industrials	NYSE American	\N	2020-10-11 23:12:01.429802	2020-10-11 23:38:31.028166
CXP	Columbia Property Trust Inc.	Columbia Property Trust, Inc. operates as a real estate investment trust. The Company through its subsidiaries acquires, develops, owns, leases, and operates real properties including office properties and hotel.	{"Real Estate","REIT - Office",REITs}	{}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:01.458383	2020-10-11 23:38:31.023428
CYRX	CryoPort Inc.	CryoPort Inc provides cryogenic logistics solutions to the life sciences industry through its proprietary packaging, information technology, and cold chain logistics expertise. It offers logistics solutions for stem cells and CAR-T cells.	{"Consumer Cyclical","Packaging & Containers"}	{GFN,GEF,WAIR}	Packaging & Containers	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:01.478308	2020-10-11 23:38:31.028166
CZFC	Citizens First Corporation	Citizens First Corp is the bank holding company. It is engaged in the business of accepting demand, savings, time deposits and providing commercial, consumer and mortgage loans to the general public.	{"Financial Services","Banks - Regional - US",Banks}	{CTBI,TRMK,BBT,FHN,SYBT}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:01.48942	2020-10-11 23:38:31.024353
DAX	Horizons DAX Germany ETF	The investment seeks to provide investment results that closely correspond, before fees and expenses, generally to the price and yield performance of the DAX® Index.\n The fund will invest at least 80% of its total assets in the common stock of companies included in index. The index tracks the segment of the largest and most actively traded companies - known as blue chips - on the German equities market. The index contains the shares of the 30 largest German companies in terms of liquidity and market capitalization admitted to the Frankfurt Stock Exchange in the Prime Standard segment. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.517408	2020-10-11 23:38:31.024353
DBB	Invesco DB Base Metals Fund	The investment seeks to track the DBIQ Optimum Yield Industrial Metals Index Excess Return™ (DBIQ-OY Industrial Metals ER™), which is intended to reflect the base metals sector. \n The index Commodities consist of Aluminum, Zinc and Copper – Grade A.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.526179	2020-10-11 23:38:31.021426
DBJP	Xtrackers MSCI Japan Hedged Equity	The investment seeks investment results that correspond generally to the performance, before fees and expenses, of the MSCI EM US Dollar Hedged Index.\n The fund, using a "passive" or indexing investment approach, seeks investment results that correspond generally to the performance, before fees and expenses, of the underlying index, which is designed to track emerging market performance while mitigating exposure to fluctuations between the value of the U.S. dollar and the currencies of the countries included in the underlying index. It will invest at least 80% of its total assets in component securities of the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.537901	2020-10-11 23:38:31.023428
DCHF	ETNs linked to the VelocityShares Daily 4X Long USD vs. CHF Index	The investment seeks to track the daily performance of the VelocityShares Daily 4X Long USD vs. CHF Index.\nThe index is designed to provide 4 times leveraged exposure, reset daily, to changes in the spot exchange rate between an underlying pair of currencies consisting of the U.S. dollar and a foreign currency. It provides long exposure to one currency (the “long currency”) in the underlying currency pair relative to the other currency (the “reference currency”). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.551232	2020-10-11 23:38:31.031328
DDD	3D Systems Corporation	3D Systems Corp is a developer of computer systems and components. Its suite of products includes 3-D printers, print materials, digital design software and on-demand manufacturing services.	{Technology,"Computer Systems","Computer Hardware"}	{MRCY,ADSK,CAJ,GE,HON,MMM,PRLB,XONE,VJET,HPQ,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:01.560759	2020-10-11 23:38:31.02582
DDP	DB Commodity Short ETN due April 1 2038	The investment seeks to replicate, net of expenses, the inverse of the Deutsche Bank Liquid Commodity Total Return Index. \n The index is intended to reflect changes in the market value of certain commodity futures contracts based on crude oil, heating oil, corn, wheat, gold and aluminum. The T-Bill Index is intended to approximate the returns from investing in 3-month United States Treasury bills on a rolling basis.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.572873	2020-10-11 23:38:31.023007
DEFA	iShares Adaptive Currency Hedged MSCI EAFE	The investment seeks to track the investment results of the MSCI EAFE Adaptive Hedge to USD Index.\n The fund generally will invest at least 90% of its assets in the component securities and other instruments of the index and in investments that have economic characteristics that are substantially identical to the component securities of the index. The index is an equity benchmark for international stock performance with the foreign currency exposure of the securities included in the underlying index dynamically hedged against the USD determined systematically by four currency risk indicators: carry, momentum, value, and volatility. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:01.589272	2020-10-11 23:38:31.023007
DESP	Despegar.com Corp.	Despegar.com Corp is an online travel company. The company is engaged in two business segments: Air, which consists of the sale of airline tickets and packages, hotels and other travel products which consists of travel packages.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:01.60826	2020-10-11 23:38:31.02582
DAG	DB Agriculture Double Long ETN due April 1 2038	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the Deutsche Bank Liquid Commodity index - Optimum Yield Agriculture.\n The fund is a senior unsecured obligation that allows investors to take a leveraged view on the performance of the agriculture sector. The index is composed of roughly equal percentages of corn, wheat, soybean, and sugar futures contracts.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.500175	2020-10-11 23:38:31.031328
DAR	Darling Ingredients Inc.	Darling Ingredients Inc is a developer and producer of natural ingredients from edible and inedible bio-nutrients, creating customized specialty solutions for customers in pharmaceutical, food, pet food, fuel, and fertilizer industries.	{Industrials,"Waste Management"}	{VLO,ADM,HON,BG,SEB,POST,REGI,INGR}	Waste Management	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:01.512274	2020-10-11 23:38:31.02423
DBEF	Xtrackers MSCI EAFE Hedged Equity	The investment seeks investment results that correspond generally to the performance, before fees and expenses, of the MSCI EAFE US Dollar Hedged Index.\n The fund, using a "passive" or indexing investment approach, seeks investment results that correspond generally to the performance, before fees and expenses, of the underlying index, which is designed to track developed market performance while mitigating exposure to fluctuations between the value of the U.S. dollar and the currencies of the countries included in the underlying index. It will invest at least 80% of its total assets in component securities of the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.529201	2020-10-11 23:38:31.028166
DCI	Donaldson Company Inc.	Donaldson Co Inc is a provider of industrial goods. It manufactures filtration systems such as dust, fume, and mist collectors.	{Industrials,"Diversified Industrials","Industrial Products"}	{CECE,ENTG,FTEK,CAT,PH,UTX,PPIH}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:01.554071	2020-10-11 23:38:31.02423
DDS	Dillard's Inc.	Dillard's Inc is an American fashion apparel, cosmetics and home furnishings retailer. The company also operates a general contracting construction company engaged in constructing and remodeling stores of the company.	{"Consumer Cyclical","Department Stores","Retail - Apparel & Specialty"}	{KSS,JWN,JCP,TJX,SSI,M,SHOS}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:01.573625	2020-10-11 23:38:31.021426
DELT	Delta Technology Holdings Limited	Delta Technology Holdings Ltd is a fine & specialty chemical company. It is engaged in producing & distributing organic compound including para-chlorotoluene, ortho-chlorotoluene, PCT/OCT downstream product, unsaturated polyester resin & other by-product.	{"Basic Materials","Specialty Chemicals",Chemicals}	{LOOP}	Chemicals	Basic Materials	NASDAQ Capital Market	\N	2020-10-11 23:12:01.590155	2020-10-11 23:38:31.02423
DFEN	Direxion Daily Aerospace & Defense Bull 3X Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the daily performance of the Dow Jones U.S. Select Aerospace & Defense Index.\n The fund invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, ETFs that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index attempts to measure the performance of the aerospace and defense industry of the U.S. equity market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.61888	2020-10-11 23:38:31.028166
DGRS	WisdomTree U.S. SmallCap Dividend Growth Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree U.S. SmallCap Quality Dividend Growth Index.\n The index is a fundamentally weighted index that consists of the small-capitalization segment of dividend-paying U.S. common stocks with growth characteristics. Under normal circumstances, at least 80% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.651557	2020-10-11 23:38:31.02582
DHF	Dreyfus High Yield Strategies Fund	Dreyfus High Yield Strategies Fund is a non-diversified, closed-end management investment company. The fund's primary investment objective is to seek high current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.662298	2020-10-11 23:38:31.031328
DHX	DHI Group Inc.	DHI Group Inc is a provider of data, insights and employment connections through its specialized services including technology and security clearance, financial services, energy, healthcare, and hospitality.	{Industrials,"Staffing & Outsourcing Services","Employment Services"}	{MSFT,KELYB,JOBS,MAN,FB,NWSA,CRCM,TGNA,JOB,IPDN}	Employment Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:01.671682	2020-10-11 23:38:31.028166
DIM	WisdomTree International MidCap Dividend Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree International MidCap Dividend Index.\n The fund normally invests at least 95% of its total assets (exclusive of collateral held from securities lending) in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a fundamentally weighted index that is comprised of the mid-capitalization segment of the dividend-paying market in the industrialized world outside the U.S. and Canada. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.683773	2020-10-11 23:38:31.025871
DBD	Diebold Nixdorf Incorporated	Diebold Nixdorf Inc provides integrated software, financial self-service delivery and security systems to the financial, commercial retail and other markets. The company generates a majority of its revenue from the United States.	{Technology,"Software - Application","Application Software"}	{USAT,NCR,PBI,BCO,PAR,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:01.527352	2020-10-11 23:38:31.02423
DBL	DoubleLine Opportunistic Credit Fund of Beneficial Interest	Doubleline Opportunistic Credit Fund operates as a closed-end management investment company. Its investment objective is to seek high total investment return by providing high level of current income and the potential for capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.53965	2020-10-11 23:38:31.023007
DDE	Dover Downs Gaming & Entertainment Inc	Dover Downs Gaming & Entertainment Inc is a gaming and entertainment company whose operations consist of: Dover Downs Casino; Dover Downs Hotel and Conference Center; and Dover Downs Raceway.	{"Consumer Cyclical",Gambling,"Travel & Leisure"}	{PENN,CPHC,CHDN,SGMS,FLL,ERI,NYNY}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:01.565037	2020-10-11 23:38:31.02423
DDWM	WisdomTree Dynamic Currency Hedged International Equity Fund	The investment seeks to track the WisdomTree Dynamic Currency Hedged International Equity Index.\n The fund will invest at least 80% of its total assets in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a dividend weighted index designed to provide exposure to equity securities in the industrialized world, excluding Canada and the United States, while at the same time dynamically hedging currency exposure to fluctuations between the value of the applicable foreign currencies and the USD. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:01.579342	2020-10-11 23:38:31.031328
DERM	Dermira Inc.	Dermira Inc is a biopharmaceutical company. It is engaged in identifying, developing and commercializing therapies to improve the lives of patients with dermatologic diseases.	{Healthcare,Biotechnology}	{BCRX,ATHX,ATNM,ATRA,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:01.599264	2020-10-11 23:38:31.023428
DEZU	iShares Adaptive Currency Hedged MSCI Eurozone	The investment seeks to track the investment results of the MSCI EMU Adaptive Hedge to USD Index.\n The fund generally will invest at least 90% of its assets in the component securities and other instruments of the index and in investments that have economic characteristics that are substantially identical to the component securities of the index. The index has been developed as an equity benchmark for the EMU countries with the foreign currency exposure of the securities included in the underlying index dynamically hedged against the USD determined systematically by four currency risk indicators: carry, momentum, value, and volatility. It is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:01.614033	2020-10-11 23:38:31.023428
DFND	Realty Shares DIVCON Dividend Defender	The investment seeks long-term capital appreciation by tracking the performance, before fees and expenses, of the Reality Shares DIVCON Dividend Defender Index.\n The index is designed to select the companies for a long position that have the highest probability of increasing their dividend in a 12-month period and select the companies for a short position that have the highest probability of decreasing their dividend in a 12-month period. Under normal circumstances, at least 80% of the fund's assets (other than collateral held from securities lending, if any) will be invested in component securities of the benchmark index. It is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:01.622407	2020-10-11 23:38:31.023428
DGICB	Donegal Group Inc. Class B Common Stock	Donegal Group Inc is an US-based insurance holding company offering personal and commercial lines of property and casualty insurance to businesses and individuals. It products include protection against automobile accidents, theft and other.	{"Financial Services","Insurance - Property & Casualty"}	{ERIE,CINF,STFC,WRB,MKL,RLI,TRV,UFCS}	Insurance - Property & Casualty	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:01.642246	2020-10-11 23:38:31.023428
DGX	Quest Diagnostics Incorporated	Quest Diagnostics Inc provides diagnostic testing, information, and services, providing insights that enable patients and physicians to make healthcare decisions. It provides diagnostics for insurers and healthcare information technology businesses.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{PMD,GHDX,ENZ,LH,NEO,PRPO,QGEN,XLV}	Medical Diagnostics & Research	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:01.655394	2020-10-11 23:38:31.023007
DIG	ProShares Ultra Oil & Gas	The investment seeks daily investment results that correspond to two times (2x) the daily performance of the Dow Jones U.S. Oil & GasSM Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index measures the performance of certain companies in the oil and gas sector of the U.S. equity market. Component companies include, among others, exploration and production, integrated oil and gas, oil equipment and services, pipelines, renewable energy equipment companies and alternative fuel producers. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.68345	2020-10-11 23:38:31.02582
FISK	Empire State Realty OP L.P. Series 250 Operating Partnership Units Representing Limited Partnership Interests	Empire State Realty OP LP owns, operates, acquires and repositions properties such as Empire State Building, One Grand Central Place in Manhattan and the greater New York metropolitan area. It operates in two segments, Real Estate and Observatory.	{"Real Estate","REIT - Diversified",REITs}	{}	REITs	Real Estate	NYSE Arca	\N	2020-10-11 23:12:02.987581	2020-10-11 23:38:31.025871
DBC	Invesco DB Commodity Index Tracking Fund	The investment seeks to track changes, whether positive or negative, in the level of the Deutsche Bank G10 Currency Future Harvest Index®—Excess Return.\n The index is comprised of long futures positions in the three eligible index currencies associated with the highest interest rates and short futures positions in the three eligible index currencies associated with the lowest interest rates. The index’s six component currencies from time-to-time, comprised of the three long and three short futures positions, are referred to as the index currencies and are used to calculate the value of the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.530562	2020-10-11 23:38:31.024353
DBS	Invesco DB Silver Fund	The investment seeks to track the DBIQ Optimum Yield Silver Index Excess Return™ (DBIQ-OY SI ER™), which is intended to reflect the changes in market value of silver. \n The single index Commodity consists of Silver. The fund holds Treasury Securities, money market mutual funds and T-Bill ETFs for margin and/or cash management purposes only.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.542367	2020-10-11 23:38:31.028166
DCOM	Dime Community Bancshares Inc.	Dime Community Bancshares Inc operates as the holding company for The Dime Savings Bank of Williamsburgh. It provides financial services and loans for multifamily housing in the United States.	{"Financial Services","Savings & Cooperative Banks",Banks}	{FFIC,ESBK,TRST,GCBC,LSBK,CARV,NFBK}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:01.554506	2020-10-11 23:38:31.024353
DDEZ	WisdomTree Dynamic Currency Hedged Europe Equity Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree Dynamic Currency Hedged Europe Equity Index.\n Under normal circumstances, the fund will invest at least 80% of its total assets in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a dividend weighted index designed to provide exposure to European equity securities while at the same time dynamically hedging currency exposure to fluctuations between the value of the euro and the USD. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:01.56698	2020-10-11 23:38:31.031328
DE	Deere & Company	Deere & Co is engaged in manufacturing agricultural, construction, turf, and forestry machinery. It also provides secured and lease-based financing to support dealer inventory and customer purchases.	{Industrials,"Farm & Construction Equipment","Farm & Construction Machinery"}	{ARTW,HMC,AGCO,CAT,LNN,TTC}	Farm & Construction Machinery	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:01.581621	2020-10-11 23:38:31.02423
DEMG	Xtrackers FTSE Emerging Comprehensive Factor	The investment seeks investment results that correspond generally to the performance, before fees and expenses, of the FTSE Emerging Comprehensive Factor Index.\n The fund will invest at least 80% of its total assets (but typically far more) in component securities (including depositary receipts in respect of such securities) of the underlying index. The index is designed to track the equity market performance of companies in emerging market countries selected on the investment style criteria of value, momentum, quality, low volatility and size. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.593783	2020-10-11 23:38:31.031328
DEUR	ETNs linked to the VelocityShares Daily 4X Long USD vs. EUR Index	The investment seeks to track the daily performance of the VelocityShares Daily 4X Long USD vs. EUR Index.\nThe index is designed to provide 4 times leveraged exposure, reset daily, to changes in the spot exchange rate between an underlying pair of currencies consisting of the U.S. dollar and a foreign currency. It provides long exposure to one currency (the “long currency”) in the underlying currency pair relative to the other currency (the “reference currency”). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.606882	2020-10-11 23:38:31.031328
DFE	WisdomTree Europe SmallCap Dividend Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree Europe SmallCap Dividend Index.\n Under normal circumstances, at least 95% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in the component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a fundamentally weighted index that is comprised of the small-capitalization segment of the European dividend-paying market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.618063	2020-10-11 23:38:31.023007
DFVL	iPath US Treasury 5 Year Bull ETN	The investment seeks to provide investors with exposure to the Barclays 5Y US Treasury Futures Targeted Exposure Index™. \nThe Barclays 5Y US Treasury Futures Targeted Exposure Index™ (the "index") is designed to decrease in response to an increase in the 5-year Treasury note yields and to increase in response to a decrease in 5-year Treasury note yields. The index targets a fixed level of sensitivity to changes in the yield of the current "cheapest-to-deliver" note underlying the relevant 5-year Treasury futures contract at a given point in time.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.629663	2020-10-11 23:38:31.025871
DGLD	VS 3X INV GOLD	The investment seeks to replicate, net of expenses, three times the opposite (inverse) of the S&P GSCI Gold index ER.\n The index comprises futures contracts on a single commodity. The fluctuations in the values of it are intended generally to correlate with changes in the price of gold in global markets.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.641529	2020-10-11 23:38:31.028166
DGZ	DB Gold Short ETN due February 15 2038	The investment seeks to replicate, net of expenses, the inverse of the daily performance of the Deutsche Bank Liquid Commodity index - Optimum Yield Gold Excess Return.\n The index is intended to reflect changes in the market value of certain gold futures contracts and is comprised of a single unfunded gold futures contract.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.66111	2020-10-11 23:38:31.021426
DCO	Ducommun Incorporated	Ducommun Inc through its subsidiaries provides engineering and manufacturing services for high-performance products and high-cost-of failure applications used in the aerospace, defense, industrial, natural resources, medical and other industries.	{Industrials,"Aerospace & Defense"}	{ATRO,CVU,MOG.A,UTX,TDG,SPR,AIRI}	Aerospace & Defense	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:01.554906	2020-10-11 23:38:31.025871
DDG	ProShares Short Oil & Gas	The investment seeks daily investment results that correspond to the inverse (-1x) of the daily performance of the Dow Jones U.S. Oil & GasSM Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index measures the performance of certain companies in the oil and gas sector of the U.S. equity market. Component companies include, among others, exploration and production, integrated oil and gas, oil equipment and services, pipelines, renewable energy equipment companies and alternative fuel producers. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.569351	2020-10-11 23:38:31.023428
DECK	Deckers Outdoor Corporation	Deckers Outdoor Corp is engaged in designing, marketing and distribution of footwear, apparel and accessories developed for both everyday casual lifestyles use. Its primary brands include UGG, Teva, and Sanuk.	{"Consumer Cyclical","Footwear & Accessories","Manufacturing - Apparel & Furniture"}	{SHOO,COLM,CROX,SKX,NKE,WWW}	Manufacturing - Apparel & Furniture	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:01.582554	2020-10-11 23:38:31.025871
DEO	Diageo plc	Diageo PLC is a part of the beverage industry. Its business involves the supply of alcoholic beverage under a collection of brands such as Johnnie Walker, Crown Royal, J&B, Buchanan's and Windsor whiskies, Smirnoff, Ciroc and Ketel One vodkas.	{"Consumer Defensive","Beverages - Wineries & Distilleries","Beverages - Alcoholic"}	{DEO,BF.B,STZ.B,TAP,ROX,BUD,CCU}	Beverages - Alcoholic	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:01.597769	2020-10-11 23:38:31.025871
DEX	Delaware Enhanced Global Dividend of Beneficial Interest	Delaware Enhanced Global Dividend & Inc is a diversified closed-end management investment company. The primary investment objective of the Fund is to seek current income, with a secondary objective of capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.613093	2020-10-11 23:38:31.02582
DFP	Flaherty & Crumrine Dynamic Preferred and Income Fund Inc.	Flaherty & Crumrine Pref&Inc Allocation is a non-diversified, closed-end management investment company. Its investment objective is to seek total return, with an emphasis on high current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.626411	2020-10-11 23:38:31.028166
DGBP	ETNs linked to the VelocityShares Daily 4X Long USD vs. GBP Index	The investment seeks to track the daily performance of the VelocityShares Daily 4X Long USD vs. GBP Index.\nThe index is designed to provide 4 times leveraged exposure, reset daily, to changes in the spot exchange rate between an underlying pair of currencies consisting of the U.S. dollar and a foreign currency. It provides long exposure to one currency (the “long currency”) in the underlying currency pair relative to the other currency (the “reference currency”). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.639781	2020-10-11 23:38:31.031328
DGSE	DGSE Companies Inc.	DGSE Companies Inc is in the business of buying and selling jewelry, diamonds, fine watches, rare coins and currency, precious metal bullion products, scrap gold, silver, platinum and palladium as well as collectibles and other valuables.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{TIF,SIG,ICAD,BGI}	Retail - Apparel & Specialty	Consumer Cyclical	NYSE American	\N	2020-10-11 23:12:01.653353	2020-10-11 23:38:31.025871
DHT	DHT Holdings Inc.	DHT Holdings Inc is a crude oil tanker company and owns more than twenty crude oil tankers that are in operation. The company operates through its integrated management companies in Oslo, Norway and Singapore.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{GLNG,CPLP,NAT,TK,SFL,TGP,TNK,ASC,FRO,GOGL}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:01.667914	2020-10-11 23:38:31.028166
DISCK	Discovery Inc.	Discovery Inc formerly known as Discovery Communications Inc is a media and entertainment company. It provides programming across multiple distribution platforms.	{"Consumer Cyclical","Media - Diversified",Entertainment}	{CMCSA,DIS,CBS,AMCX,VIAB,FOXA}	Entertainment	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:01.692095	2020-10-11 23:38:31.025871
DK	Delek US Holdings Inc.	Delek US Holdings Inc is an oil and gas company that is engaged in petroleum refining, transportation, storage and wholesale of crude oil and intermediate and refined products and convenience store retailing.	{Energy,"Oil & Gas Refining & Marketing","Oil & Gas - Refining & Marketing"}	{CLMT,VLO,MUR,OIS,RDS.A,CVI,MPC,HFC}	Oil & Gas - Refining & Marketing	Energy	New York Stock Exchange	\N	2020-10-11 23:12:01.712752	2020-10-11 23:38:31.02423
DMB	Dreyfus Municipal Bond Infrastructure Fund Inc.	Dreyfus Muni Bond Infrastructure Fund is a diversified, closed-end management investment company. The Fund's investment objective is to provide total return through a combination of current income and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.741244	2020-10-11 23:38:31.028166
DNKN	Dunkin' Brands Group Inc.	Dunkin Brands Group Inc is franchisors of quick service restaurants serving hot and cold coffee and baked goods, as well as hard-serve ice cream. Its franchisee brands are Dunkin' Donuts and Baskin-Robbins.	{"Consumer Cyclical",Restaurants}	{SBUX,MCD,SJM,WEN,DF}	Restaurants	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:01.760571	2020-10-11 23:38:31.025871
DCPH	Deciphera Pharmaceuticals Inc.	Deciphera Pharmaceuticals Inc is a clinical-stage biopharmaceutical company developing new drugs to improve the lives of cancer patients.	{Healthcare,Biotechnology}	{PDLI,PIRS,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:01.556846	2020-10-11 23:38:31.023007
DEI	Douglas Emmett Inc.	Douglas Emmett Inc acquires, develops, and manages office and multifamily properties in United States. The company mainly focuses on owning office buildings in communities with large supply constraints, such as Los Angeles.	{"Real Estate","REIT - Office",REITs}	{VNO,MAC,BXP,SLG,CLI,OFC,CUZ,FSP}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:01.589586	2020-10-11 23:38:31.02582
DEUS	Xtrackers Russell 1000 Comprehensive Factor	The investment seeks investment results that correspond generally to the performance, before fees and expenses, of the Russell 1000 Comprehensive Factor Index.\n The fund, using a "passive" or indexing investment approach, seeks investment results that correspond generally to the performance of the underlying index, which is designed to track the equity market performance of companies in the United States selected on the investment style criteria ("factors") of value, momentum, quality, low volatility and size. It will invest at least 80% of its total assets (but typically far more) in component securities of the underlying index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.608539	2020-10-11 23:38:31.02423
DFNL	Davis Select Financial ETF	The investment seeks long-term growth of capital.\n The fund's investment adviser, uses the Davis Investment Discipline to invest, under normal market conditions, at least 80% of its net assets plus any borrowings for investment purposes in securities issued by companies principally engaged in the financial services sector. The fund's portfolio generally contains between 15 and 35 companies. It invests, principally, in common stocks. The fund may invest in large, medium or small companies without regard to market capitalization and may invest in issuers in foreign countries, including countries with developed or emerging markets. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.624947	2020-10-11 23:38:31.02582
DGRW	WisdomTree U.S. Dividend Growth Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree U.S. Quality Dividend Growth Index.\n Under normal circumstances, at least 80% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a fundamentally weighted index that consists of dividend-paying U.S. common stocks with growth characteristics. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.653626	2020-10-11 23:38:31.023428
DHVW	Diamond Hill Valuation-Weighted 500	The investment seeks to track the price and total return performance, before fees and expenses, of the Diamond Hill Valuation-Weighted 500 Index.\n The fund invests at least 80% of the fund's total assets (exclusive of any collateral held from securities lending) in the component securities of the index. The index is composed of the common stock of approximately 500 U.S.-listed companies weighted based on a proprietary, patent-pending valuation methodology developed by Diamond Hill Capital Management, Inc., the fund's investment adviser.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.669187	2020-10-11 23:38:31.025871
DIOD	Diodes Incorporated	Diodes Inc is a manufacturer & supplier of application-specific standard products within the broad discrete, logic, analog & mixed-signal semiconductor markets, serving the consumer electronics, computing, communications, industrial & automotive markets.	{Technology,Semiconductors}	{ON,AOSL,NXPI,VSH,TXN,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:01.687697	2020-10-11 23:38:31.024353
DJP	iPath Bloomberg Commodity Index Total Return ETN	The investment seeks to provide investors with exposure to the Dow Jones-UBS Commodity Index Total ReturnService Mark. \nThe Dow Jones-UBS Commodity Index Total ReturnService Mark (the "index") reflects the returns that are potentially available through an unleveraged investment in the futures contracts on physical commodities comprising the index plus the rate of interest that could be earned on cash collateral invested in specified Treasury Bills. The index is a rolling index rebalancing annually.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.70938	2020-10-11 23:38:31.031328
DLNG	Dynagas LNG Partners LP	Dynagas LNG Partners LP is active in owning, operating, and acquiring LNG carriers. Its vessels are employed on multi-time charter with energy companies. The company has contracted under multi-year charters with BG Group, Gazprom, Statoil, and Yamal.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{GLNG,GASS,CPLP,TNP,TGP,GMLP,GLOG,GLOP}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:01.725188	2020-10-11 23:38:31.025871
DM	Dominion Energy Midstream Partners LP representing Limited Partner Interests	Dominion Energy Midstream Partners LP is engaged in providing natural gas transportation, storage and regasification services. Its operations consist of LNG import and storage services and the transportation of domestic natural gas and regasified LNG.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{EPD,AMID,SXE}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:01.737458	2020-10-11 23:38:31.031328
FISV	Fiserv Inc.	Fiserv Inc is a business solutions provider. Its service offering primarily includes electronic processing services.	{Industrials,"Business Services"}	{INTU,JKHY,EBAY,ACIW,TSS,FIS,MA,WU,V}	Business Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:02.989574	2020-10-11 23:38:31.028166
DDM	ProShares Ultra Dow30	The investment seeks daily investment results, before fees and expenses, that correspond to two times (2x) the daily performance of the Dow Jones Industrial AverageSM Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index is a price-weighted index and includes 30 large-cap, "blue-chip" U.S. stocks, excluding utility and transportation companies. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.571729	2020-10-11 23:38:31.024353
DEEF	Xtrackers FTSE Developed ex US Comprehensive Factor	The investment seeks investment results that correspond generally to the performance, before fees and expenses, of the FTSE Developed ex US Comprehensive Factor Index.\n The fund will normally invest at least 80% of its net assets, plus the amount of any borrowings for investment purposes, in equity securities of issuers from developed markets countries other than the United States. The index is designed to track the equity market performance of companies in developed countries selected on the investment style criteria of value, momentum, quality, low volatility and size. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.585535	2020-10-11 23:38:31.028166
DES	WisdomTree U.S. SmallCap Dividend Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree U.S. SmallCap Dividend Index.\n Under normal circumstances, at least 95% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a fundamentally weighted index measuring the performance of the small-capitalization segment of the U.S. dividend-paying market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.599833	2020-10-11 23:38:31.028166
DEWJ	iShares Adaptive Currency Hedged MSCI Japan	The investment seeks to track the investment results of the MSCI Japan Adaptive Hedge to USD Index.\n The fund generally will invest at least 90% of its assets in the component securities and other instruments of the index and in investments that have economic characteristics that are substantially identical to the component securities. The index has been developed as an equity benchmark for stocks traded primarily on the Tokyo Stock Exchange with the foreign currency exposure of the securities dynamically hedged against the USD determined systematically by four currency risk indicators: carry, momentum, value, and volatility. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:01.611841	2020-10-11 23:38:31.021426
DFRG	Del Frisco's Restaurant Group Inc.	Del Frisco's Restaurant Group Inc develops, owns and operates three contemporary restaurants. It currently operates 53 restaurants in 23 states and the District of Columbia.	{"Consumer Cyclical",Restaurants}	{ARKR,CAKE,BJRI,RUTH,DRI,BLMN}	Restaurants	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:01.626278	2020-10-11 23:38:31.021426
DGICA	Donegal Group Inc. Class A Common Stock	Donegal Group Inc is an US-based insurance holding company offering personal and commercial lines of property and casualty insurance to businesses and individuals. It products include protection against automobile accidents, theft and other.	{"Financial Services","Insurance - Property & Casualty"}	{ERIE,CINF,STFC,WRB,MKL,RLI,TRV,UFCS}	Insurance - Property & Casualty	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:01.63994	2020-10-11 23:38:31.02582
DGT	SPDR Global Dow ETF (based on The Global Dow)	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the Global Dow that tracks the performance of multinational blue-chip issuers.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index is made up of 150 companies from around the world. It may purchase a subset of the securities in the index in an effort to hold a portfolio of securities with generally the same risk and return characteristics of the index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.655304	2020-10-11 23:38:31.028166
DHIL	Diamond Hill Investment Group Inc.	Diamond Hill Investment Group Inc sponsors, markets, and provides investment advisory and related services to various U.S. and foreign clients including mutual funds, separate accounts and private investment funds.	{"Financial Services","Asset Management"}	{SEIC,TROW,AMG,FII,WHG,AMP,PZN,SAMG}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:01.665639	2020-10-11 23:38:31.024353
DIN	Dine Brands Global Inc.	Dine Brands Global Inc, formerly DineEquity Inc, owns, franchises and operates restaurant concepts, Applebee's Neighborhood Grill and Bar within the casual dining category and International House of Pancakes in the family dining category of restaurants.	{"Consumer Cyclical",Restaurants}	{RRGB,DENN,CBRL,DRI,EAT}	Restaurants	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:01.683206	2020-10-11 23:38:31.028166
DIV	Global X Super Dividend	The investment seeks to provide investment results that correspond generally to the price and yield performance, before fees and expenses, of the Indxx SuperDividend® U.S. Low Volatility Index ("underlying index").\n The fund invests at least 80% of its total assets in the securities of the underlying index. The underlying index tracks the performance of 50 equally-weighted common stocks, including Master Limited Partnerships ("MLPs") and Real Estate Investment Trusts ("REITs") that rank among the highest dividend yielding equity securities in the United States.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.697102	2020-10-11 23:38:31.028166
FIS	Fidelity National Information Services Inc.	Fidelity National Information Services Inc offers a range of solutions in retail and institutional banking, payments, asset and wealth management, risk and compliance, treasury and insurance and provides consulting and outsourcing services.	{Industrials,"Business Services"}	{EEFT,FISV,ACIW,TSS,ADS,GPN,MA,V}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:02.988139	2020-10-11 23:38:31.028166
DDT	Dillard's Capital Trust I	Dillard's Capital Trust I is a finance company. The Company invest in department store.	{"Consumer Cyclical","Department Stores","Retail - Apparel & Specialty"}	{}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:01.578678	2020-10-11 23:38:31.02582
DEST	Destination Maternity Corporation	Destination Maternity Corporation is engaged in designing, manufacturing and sale of maternity apparel and related accessories in the United States.	{"Consumer Cyclical","Apparel Stores","Retail - Apparel & Specialty"}	{PLCE,CATO,CBK,CRI,NWY,ASNA,LB,BEBE}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:01.606176	2020-10-11 23:38:31.023007
DF	Dean Foods Company	Dean Foods Co processes, sells, and distributes branded and private label dairy and dairy case products. It offers milk, ice cream, cultured dairy products, juices and teas to retailers, food service outlets, and governmental entities across the US.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{LWAY,HAIN,CAG,SJM,KHC}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:01.617138	2020-10-11 23:38:31.028166
DFS	Discover Financial Services	Discover Financial Services is engaged in direct consumer banking business. It mainly provides banking services such as accepting deposits, offering credit card, residential mortgage, loans and other banking facilities to its customers.	{"Financial Services","Credit Services"}	{ZION,TFSL,COF,STI}	Credit Services	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.629389	2020-10-11 23:38:31.031328
DGAZ	VS 3X INV NAT GAS	The investment seeks to replicate, net of expenses, three times the opposite (inverse) of this GSCI Natural Gas Index ER.\n The index comprises futures contracts on a single commodity and is calculated according to the methodology of the S&P GSCI Index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.637896	2020-10-11 23:38:31.024353
DGRE	WisdomTree Emerging Markets Dividend Growth Fund	The investment seeks income and capital appreciation.\n The fund seeks to achieve its investment objective by investing primarily in emerging market dividend-paying common stocks with growth characteristics. The adviser, using a disciplined model-based process focused on a long-term approach to investing, seeks to identify dividend-paying companies with strong corporate profitability and sustainable growth characteristics. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.650151	2020-10-11 23:38:31.023007
DHDG	WisdomTree Dynamic Currency Hedged International Quality Dividend Growth Fund	The investment seeks to track the price and yield performance, of the WisdomTree Dynamic Currency Hedged International Quality Dividend Growth Index.\n At least 80% of the fund's total assets will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index consists of dividend-paying paying common stocks with growth characteristics of companies in the industrialized world, excluding Canada and the United States. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:01.662228	2020-10-11 23:38:31.028166
DIA	SPDR Dow Jones Industrial Average	The investment seeks to provide investment results that, before expenses, correspond generally to the price and yield performance of the Dow Jones Industrial Average (the "DJIA").The Trust seeks to achieve its investment objective by holding a portfolio of the common stocks that are included in the DJIA (the "Portfolio"), with the weight of each stock in the Portfolio substantially corresponding to the weight of such stock in the DJIA.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.677404	2020-10-11 23:38:31.028166
DISH	DISH Network Corporation	DISH Network Corp provides a direct broadcast satellite subscription television service in the United States. It offers pay-TV services under the DISH brand and the Sling brand.	{"Communication Services","Pay TV"}	{VSAT,NFLX,CMCSA,SATS,DIS,AMX,GSAT,CHTR}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:12:01.694453	2020-10-11 23:38:31.02582
DJCI	E-TRACS Linked to the Bloomberg Commodity Index Total Return due October 31 2039	The investment seeks to replicate, net of expenses, the DJ-UBS Commodity Index Total Return Index.\n The index measures the collateralized returns from a basket of 19 commodity futures contracts representing the energy, precious metals, industrial metals, grains, softs and livestock sectors. In addition, the index is rebalanced once a year to ensure that no commodity sector may constitute more than 33% of the index as of the date of such rebalancing.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.707263	2020-10-11 23:38:31.025871
DLBR	Citigroup Global Markets Holdings Inc VelocityShares Short LIBOR ETN	The Short LIBOR ETNs are linked to the Janus Velocity Short LIBOR Index (the “Short LIBOR Index”). The Short LIBOR Index aims to approximate the daily performance of a hypothetical short position in the composite forward LIBOR rate, as if the composite forward LIBOR rate itself were an asset that could be shorted. It does so by tracking the return on a hypothetical long position in Eurodollar futures contracts, where that position is recalibrated daily to result in a return over the next Index Business Day that approximates the inverse of the percentage change in the composite forward LIBOR rate over that next day, subject to the short LIBOR floor.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.720858	2020-10-11 23:38:31.028166
DLPNW	Dolphin Entertainment Inc. Warrant		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:01.73116	2020-10-11 23:38:31.028166
DMRC	Digimarc Corporation	Digimarc Corp is an Information Technology company. It develops solutions, licenses intellectual property, and provides development services to global business partners across industries.	{Technology,"Information Technology Services","Application Software"}	{OTEX,ADBE,NLSN,TIVO,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:01.744299	2020-10-11 23:38:31.023007
DEA	Easterly Government Properties Inc.	Easterly Government Properties Inc is a real estate investment trust. The company is engaged in the acquisition, development and management of Class A commercial properties that are leased to U.S. Government agencies.	{"Real Estate","REIT - Office",REITs}	{DLR,UE}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:01.582343	2020-10-11 23:38:31.028166
DENN	Denny's Corporation	Denny's Corp operates as a full-service restaurant chain. The Company through its wholly-owned subsidiary owns and operates the Denny's restaurant brand.	{"Consumer Cyclical",Restaurants}	{DAVE,CAKE,RRGB,TXRH,BJRI,CBRL,EAT,DIN}	Restaurants	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:01.597544	2020-10-11 23:38:31.024353
DEW	WisdomTree Global High Dividend Fund	The investment seeks to track the price and yield performance of the WisdomTree Global High Dividend Index.\n The fund will invest at least 95% of its total assets in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a fundamentally weighted index that is comprised of high dividend-yielding companies selected from the WisdomTree Global Dividend Index, which defines the dividend-paying universe of companies in the U.S., developed countries and emerging markets throughout the world. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.60967	2020-10-11 23:38:31.024353
DFFN	Diffusion Pharmaceuticals Inc.	Diffusion Pharmaceuticals Inc is a clinical stage biotechnology company focused on extending the life expectancy of cancer patients by improving the effectiveness of current standard-of-care treatments, including radiation therapy and chemotherapy.	{Healthcare,Biotechnology}	{LYV,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:01.618192	2020-10-11 23:38:31.025871
DFVS	iPath US Treasury 5-year Bear ETN	The investment seeks to provide investors with inverse exposure to the Barclays 5Y US Treasury Futures Targeted Exposure Index™. \nThe Barclays 5Y US Treasury Futures Targeted Exposure Index™ (the "index") is designed to decrease in response to an increase in the 5-year Treasury note yields and to increase in response to a decrease in 5-year Treasury note yields. The index targets a fixed level of sensitivity to changes in the yield of the current "cheapest-to-deliver" note underlying the relevant 5-year Treasury futures contract at a given point in time.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.631218	2020-10-11 23:38:31.028166
DGII	Digi International Inc.	Digi International Inc is a provider of Machine to Machine and Internet of Things (IoT) networking hardware products and solutions that enable the connection, monitoring and control of local or remote physical assets by electronic means.	{Technology,"Communication Equipment"}	{INTC,RSYS,QCOM,SLAB,ERIC,VZ,HPQ,AVT,SMCI,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:01.641189	2020-10-11 23:38:31.025871
DGRO	iShares Core Dividend Growth	The investment seeks to track the investment results of the Morningstar® US Dividend Growth IndexSM.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index. The underlying index is a subset of the Morningstar® US Market IndexSM, which is a diversified broad market index that represents approximately 97% of the market capitalization of publicly-traded U.S. stocks.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.651644	2020-10-11 23:38:31.031328
DHI	D.R. Horton Inc.	D.R. Horton Inc is a residential homebuilder. It constructs for the purpose of sale, homes in the states and metropolitan markets of the United States, under the names of D.R. Horton, America's Builder, Emerald Homes and Breland Homes.	{"Consumer Cyclical","Residential Construction","Homebuilding & Construction"}	{PHM,MHO,NVR,HOV,LEN,MTH,MDC,TOL,KBH}	Homebuilding & Construction	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:01.662843	2020-10-11 23:38:31.02582
DHY	Credit Suisse High Yield Bond Fund	Credit Suisse High Yield Bond Fund is a closed-end management investment company. The investment objective of the company is to seek current income through investment in debt securities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:01.675181	2020-10-11 23:38:31.021426
DIS	The Walt Disney Company	Walt Disney Co together with its subsidiaries is a diversified worldwide entertainment company with operations in four business segments: Media Networks, Parks and Resorts, Studio Entertainment, and Consumer Products & Interactive Media.	{"Consumer Cyclical","Media - Diversified",Entertainment}	{CMCSA,DISCA,SNE,CBS,PSO,AMCX,VIAB,FOXA}	Entertainment	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:01.68814	2020-10-11 23:38:31.031328
DIVC	Citigroup Inc. C-Tracks ETN Miller/Howard Strategic Dividend Reinvestors Due 9/16/2014	The investment seeks to provide exposure to the performance of the Miller/Howard Strategic Dividend Index Total Return.\n The C-Tracks Exchange-Traded Notes Miller/Howard Strategic Dividend Reinvestor, are unsecured senior debt securities. The index is designed to track the performance of 30 equally weighted stocks traded on U.S. exchanges selected quarterly pursuant to rules based upon certain quantitative fundamental factors, including dividend yield, expected growth of dividend yield, market valuation relative to book value, return on invested capital relative to price-to-earnings ratio and trailing 26-week stock price momentum.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.702168	2020-10-11 23:38:31.021426
FIT	Fitbit Inc. Class A	Fitbit Inc provides fitness tracking devices, wearable devices, and the health and fitness goals. It provides platform to combine connected health and fitness devices with software and services.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{AAPL,NKE,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:02.991127	2020-10-11 23:38:31.021426
DESC	Xtrackers Russell 2000 Comprehensive Factor	The investment seeks investment results that correspond generally to the performance, before fees and expenses, of the Russell 2000 Comprehensive Factor Index (the "underlying index").\n The fund, using a "passive" or indexing investment approach, seeks investment results that correspond generally to the performance, before fees and expenses, of the underlying index, which is designed to track the equity market performance of companies in the United States selected on the investment style criteria ("factors") of value, momentum, quality, low volatility and size. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.603758	2020-10-11 23:38:31.028166
DFJ	WisdomTree Japan SmallCap Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree Japan SmallCap Dividend Index.\n Under normal circumstances, at least 95% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in the component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is comprised of dividend-paying small capitalization companies in Japan. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.622088	2020-10-11 23:38:31.02423
DGL	Invesco DB Gold Fund	The investment seeks to track the DBIQ Optimum Yield Gold Index Excess Return™ (DBIQ-OY GC ER™), which is intended to reflect the changes in market value of gold. \nThe single index Commodity consists of Gold.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.641574	2020-10-11 23:38:31.021426
DHS	WisdomTree U.S. High Dividend Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree U.S. High Dividend Index.\n Under normal circumstances, at least 95% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a fundamentally weighted index that is comprised of companies with high dividend yields selected from the WisdomTree U.S. Dividend Index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.666737	2020-10-11 23:38:31.02423
DIAL	Columbia Diversified Fixed Income Allocation	The investment seeks investment results that closely correspond to the performance of the Beta Advantage® Multi-Sector Bond Index.\n The fund invests at least 80% of its assets in securities within the index or in securities, that have economic characteristics that are substantially the same as the economic characteristics of the securities within the index. The index reflects a rules-based multi-sector strategic beta approach to measuring the performance of the debt market through representation of six sectors of the debt market in the index, each focused on yield, quality, and liquidity of the particular eligible universe.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.67652	2020-10-11 23:38:31.023428
DISCA	Discovery Inc. Series A Common Stock	Discovery Inc formerly known as Discovery Communications Inc is a media and entertainment company. It provides programming across multiple distribution platforms.	{"Consumer Cyclical","Media - Diversified",Entertainment}	{CMCSA,DIS,CBS,AMCX,VIAB,FOXA}	Entertainment	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:01.689076	2020-10-11 23:38:31.023007
DIVA	AGFiQ Hedged Dividend Income Fund	The investment seeks performance results that correspond to the price and yield performance, before fees and expenses, of the Indxx Hedged Dividend Income Index.\n The fund seeks to track the performance of the Indxx Hedged Dividend Income Index by investing at least 80% of its net assets (plus any borrowing for investment purposes) in common stock of the long positions in the underlying index and sells short at least 80% of the short positions in the underlying index. The index is a long/short index in which the long positions, in the aggregate, have approximately twice the weight as the short positions, in the aggregate.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.698457	2020-10-11 23:38:31.023428
DJPY	ETNs linked to the VelocityShares Daily 4X Long USD vs. JPY Index	The investment seeks to track the daily performance of the VelocityShares Daily 4X Long USD vs. JPY Index.\nThe index is designed to provide 4 times leveraged exposure, reset daily, to changes in the spot exchange rate between an underlying pair of currencies consisting of the U.S. dollar and a foreign currency. It provides long exposure to one currency (the “long currency”) in the underlying currency pair relative to the other currency (the “reference currency”). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.711798	2020-10-11 23:38:31.021426
DLHC	DLH Holdings Corp.	DLH Holdings Corp is a a payroll staffing company. It provides services and solutions in two market areas: Healthcare Delivery Solutions and Logistics & Technical Services.	{Industrials,"Staffing & Outsourcing Services","Employment Services"}	{BBSI,KFRC,AMN,BGSF}	Employment Services	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:01.723453	2020-10-11 23:38:31.021426
DLTR	Dollar Tree Inc.	Dollar Tree Inc operates discount retail stores in the United States and Canada. It operates the stores under the names of Dollar Tree, Family Dollar and Dollar Tree Canada.	{"Consumer Defensive","Discount Stores","Retail - Defensive"}	{FRED,TGT,BIG,DG,FIVE}	Retail - Defensive	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:01.735108	2020-10-11 23:38:31.021426
FIV	First Trust Senior Floating Rate 2022 Target Term Fund of Beneficial Interest	FIRST TRUST SENIOR FLOATING RATE 2022 TARGET TERM FUND is a diversified, closed-end management investment company. The Fund's investment objectives are to seek a high level of current income and to provide returns on investment.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.990875	2020-10-11 23:38:31.023007
DG	Dollar General Corporation	Dollar General Corp is a discount retailer in the United States. It offers a selection of merchandise, including consumables, seasonal, home products and apparel.	{"Consumer Defensive","Discount Stores","Retail - Defensive"}	{COST,FRED,DLTR,WMT,TGT,BIG,FIVE}	Retail - Defensive	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:01.631884	2020-10-11 23:38:31.023007
DGLY	Digital Ally Inc.	Digital Ally Inc produces digital video imaging and storage products for use in law enforcement, security and commercial applications.	{Industrials,"Security & Protection Services","Consulting & Outsourcing"}	{MSN,NPK,RGR,AOBC,LLL}	Consulting & Outsourcing	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:01.649937	2020-10-11 23:38:31.028166
DHR	Danaher Corporation	Danaher Corp designs, manufactures and markets professional, medical, industrial and commercial products and services. It markets its products under the brand of Beckman Coulter, Aperio, Dexis, Chemtreat and others.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research",Industrials,"Diversified Industrials","Industrial Products"}	{AME,A,BIO,EMR,GE,HON,TMO,UTX,ABB,MMM,XLV}	Medical Diagnostics & Research	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:01.66562	2020-10-11 23:38:31.023428
DISCB	Discovery Inc. Series B Common Stock	Discovery Inc formerly known as Discovery Communications Inc is a media and entertainment company. It provides programming across multiple distribution platforms.	{"Consumer Cyclical","Media - Diversified",Entertainment}	{CMCSA,DIS,CBS,AMCX,VIAB,FOXA}	Entertainment	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:01.689479	2020-10-11 23:38:31.028166
DKS	Dick's Sporting Goods Inc	Dick's Sporting Goods Inc is sporting goods retailer offering an extensive assortment of authentic, high-quality sports equipment, apparel, footwear and accessories through associates, in-store services and unique specialty shop-in-shops.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{BGFV,HIBB,SPWH}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:01.713886	2020-10-11 23:38:31.023007
DLPH	Delphi Technologies PLC	Delphi Technologies PLC develops, designs and manufactures integrated powertrain technologies. Its portfolio includes advanced fuel injection systems, actuators, valvetrain products, sensors, electronic control modules and power electronics.	{"Consumer Cyclical","Auto Parts",Autos}	{TEN,WBC}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:01.727306	2020-10-11 23:38:31.024353
DMRI	DeltaShares S&P International Managed Risk	The investment seeks to track the investment results, before fees and expenses, of the S&P EPAC Ex. Korea LargeMidCap Managed Risk 2.0 Index.\n Under normal market conditions, the fund invests a substantial portion, but at least 80%, of its assets, exclusive of collateral held from securities lending, in securities comprising the underlying index. The underlying index seeks to achieve these objectives by allocating weightings among the S&P EPAC Ex. Korea LargeMidCap Index, the S&P U.S. Treasury Bond Current 5-Year Index and the S&P U.S. Treasury Bill 0-3 Month Index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.74643	2020-10-11 23:38:31.028166
DNLI	Denali Therapeutics Inc.	Denali Therapeutics Inc is a clinical-stage biopharmaceutical company. The company is engaged in developing therapeutics for neurodegenerative diseases, including Alzheimer's disease, Parkinson's disease and amyotrophic lateral sclerosis.	{Healthcare,Biotechnology}	{PDLI,PIRS,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:01.763363	2020-10-11 23:38:31.02423
DOOR	Masonite International Corporation (Canada)	Masonite International Corp manufactures and distributes interior and exterior doors used for residential and commercial properties. The company operates its business worldwide and generates most of its revenues from North America.	{"Basic Materials","Building Materials"}	{BLDR,MHK,NX,FBHS}	Building Materials	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:01.784625	2020-10-11 23:38:31.023007
DOX	Amdocs Limited	Amdocs Ltd is engaged in the software industry. Its software products and services cater to the communications, media and entertainment industry.	{Technology,"Software - Infrastructure","Application Software"}	{CSGS,MNDO,ERIC,SNCR,EVOL,ORCL,APPS,HPQ,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:01.796474	2020-10-11 23:38:31.028166
DRD	DRDGOLD Limited American Depositary Shares	DRDGold Ltd is a South African gold mining company engaged in surface gold tailings retreatment including exploration, extraction, processing and smelting.	{"Basic Materials",Gold,"Metals & Mining"}	{NSU,TRX,HMY,GFI,AU,SBGL}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:01.809189	2020-10-11 23:38:31.025871
DRR	Market Vectors Double Short Euro ETN	The investment seeks to replicate, net of expenses, the Double Short Euro Index.\n As the index is two-times leveraged, for every 1% weakening of the euro relative to the U.S. dollar, the level of the index will generally increase by 2%, while for every 1% strengthening of the euro relative to the U.S. dollar, the index will generally decrease by 2%.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.817951	2020-10-11 23:38:31.024353
DIVO	Amplify YieldShares CWP Dividend & Option Income	The investment seeks to provide current income as its primary investment objective and to provide capital appreciation as its secondary investment objective.\n Under normal circumstances, the fund invests at least 80% of its total assets in dividend-paying U.S. exchange-traded equity securities ("Equity Securities") and will opportunistically utilize an "option strategy" consisting of writing (selling) U.S. exchange-traded covered call options on such Equity Securities. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.702504	2020-10-11 23:38:31.031328
DL	China Distance Education Holdings Limited American Depositary Shares	China Distance Education Holdings Ltd along with its subsidiaries provides online and offline professional education services and sells related products in China. It provides training and test preparation courses in for various industries.	{"Consumer Defensive","Education & Training Services",Education}	{FC,TAL,TEDU,ATGE}	Education	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:01.717813	2020-10-11 23:38:31.025871
DLTH	Duluth Holdings Inc.	Duluth Holdings Inc is an apparel company. Its products comprise of shirts, pants, and casual wear under brands like Alaskan Hardgear, Armachillo, Ballroom, Buck Naked, Cab Commander, Crouch Gusset, Dry on the Fly and Duluth Trading company.	{"Consumer Cyclical","Apparel Stores","Retail - Apparel & Specialty"}	{RCKY,KBSF}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:01.733699	2020-10-11 23:38:31.023428
DNB	Dun & Bradstreet Corporation (The)	Dun & Bradstreet Corp is a source of commercial data, analytics and insight on businesses. Its business segments are Americas including USA, and Canada and Non-Americas including UK, China, India and other European and Asia Pacific regions.	{Industrials,"Business Services"}	{MORN,INTX,FDS,EFX,MCO,CLGX,FOX,SPGI}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:01.754389	2020-10-11 23:38:31.021426
DNR	Denbury Resources Inc.	Denbury Resources Inc is a part of the energy sector. Its business is to explore and produce oil and gas resources in the United States.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{CRZO,CHK,NFX,PXD,APC,EOG,XEC,SM,QEP}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:01.767701	2020-10-11 23:38:31.028166
DOTAR	Draper Oakwood Technology Acquisition Inc. Right		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:01.788071	2020-10-11 23:38:31.02423
DPW	DPW Holdings Inc.	DPW Holdings Inc through its subsidiary provides mission-critical applications and lifesaving services to targets markets that are of high barrier to enter with products and services which are not likely to be commoditized.	{Technology,"Electronic Components","Computer Hardware"}	{XLK}	Computer Hardware	Technology	NYSE American	\N	2020-10-11 23:12:01.799182	2020-10-11 23:38:31.031328
DRNA	Dicerna Pharmaceuticals Inc.	Dicerna Pharmaceuticals Inc is a biopharmaceutical company. It develops ribonucleic acid interference-based pharmaceuticals for treating diseases involving liver, including rare, chronic liver, cardiovascular and viral infections.	{Healthcare,Biotechnology}	{AMGN,ALNY,PFE,MRK,IONS,ARWR,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:01.817491	2020-10-11 23:38:31.023007
DSL	DoubleLine Income Solutions Fund of Beneficial Interests	DoubleLine Income Solutions Fund is a closed-end fund. The Fund's primary investment objective is to seek high income and its secondary objective is to seek capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.833146	2020-10-11 23:38:31.023007
DTEC	ALPS ETF Trust Disruptive Technologies	The investment seeks investment results that correspond (before fees and expenses) generally to the performance of the Indxx Disruptive Technologies Index (the "underlying index").\n The fund will invest at least 80% of its net assets in securities that comprise the underlying index. The underlying index is designed to identify the companies using disruptive technologies in each of ten thematic areas: Healthcare Innovation, Internet of Things, Clean Energy and Smart Grid, Cloud Computing, Data and Analytics, FinTech, Robotics and Artificial Intelligence, Cybersecurity, 3D Printing, and Mobile Payments. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:01.846698	2020-10-11 23:38:31.028166
DTYS	iPath US Treasury 10-year Bear ETN	The investment seeks to provide investors with inverse exposure to the Barclays 10Y US Treasury Futures Targeted Exposure Index™. \nThe Barclays 10Y US Treasury Futures Targeted Exposure Index™ (the "index") is designed to decrease in response to an increase in the 10-year Treasury note yields and to increase in response to a decrease in 10-year Treasury note yields. The index targets a fixed level of sensitivity to changes in the yield of the current "cheapest-to-deliver" note underlying the relevant 10-year Treasury futures contract at a given point in time.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.869102	2020-10-11 23:38:31.031328
DVMT	Dell Technologies Inc. Class V	Dell-VMWare Tracking Stock is an information technology company. The company offers virtualization and cloud infrastructure solutions, which enable organizations to manage IT resources across complex multi-cloud, multi-device environments.	{Technology,"Software - Infrastructure","Application Software"}	{XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:01.888393	2020-10-11 23:38:31.023428
DWFI	SPDR Dorsey Wright Fixed Income Allocation ETF	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the price and yield performance of the Dorsey Wright Fixed Income Allocation Index.\n The index is comprised of exchange-traded funds advised by SSGA Funds Management, Inc. The selection universe for the index includes U.S.-listed fixed income ETFs advised by SSGA FM or its affiliates that are designed to target exposure to fixed income securities. The fund invests at least 80%, of its total assets in securities comprising the index. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.924437	2020-10-11 23:38:31.021426
DJCO	Daily Journal Corp. (S.C.)	Daily Journal Corp publishes newspapers and web sites covering California and Arizona and produces several specialized information services. It also serves as a newspaper representative specializing in public notice advertising.	{"Consumer Cyclical",Publishing}	{LEE,NYT,AHC,NWSA,NEWM,TGNA,MNI,SSP}	Publishing	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:01.707153	2020-10-11 23:38:31.028166
DLB	Dolby Laboratories	Dolby Laboratories Inc is an audio and imaging technologies company that transforms entertainment and communications at the cinema, at home, at work and on mobile devices. It derives its revenue from licensing audio technologies.	{"Consumer Cyclical","Media - Diversified",Entertainment}	{SNE,IMAX,SSTK,EROS,CIDM}	Entertainment	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:01.721335	2020-10-11 23:38:31.028166
DLPN	Dolphin Entertainment Inc.	Dolphin Entertainment Inc is in the business of production and distribution of digital entertainment content. It produces digital series and also launched feature film division dedicated to the production of motion pictures for family audiences.	{Technology,"Internet Content & Information","Online Media"}	{CARB,MEET,XLK}	Online Media	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:01.730407	2020-10-11 23:38:31.02582
DMF	Dreyfus Municipal Income Inc.	Dreyfus Municipal Income Inc is a non-diversified closed-end management investment company. The Company's investment objective is to maximize current income exempt from federal income tax to the extent consistent with the preservation of capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:01.741884	2020-10-11 23:38:31.024353
DOGS	Arrow Dogs of the World	The investment seeks long-term capital appreciation by tracking the investment results of the AI Dogs of the World ex US Total Return Index (the "index").\n Substantially all of the fund's total assets will be invested in the component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index selects stock baskets representing the top 75% of the market capitalization for each country selected.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.776877	2020-10-11 23:38:31.028166
DOTAU	Draper Oakwood Technology Acquisition Inc. Unit	Draper Oakwood Technology Acquisition Inc is a blank check company formed for the purpose of entering into a merger, share exchange, asset acquisition, stock purchase, recapitalization, reorganization or other similar business.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:01.791095	2020-10-11 23:38:31.031328
DQ	DAQO New Energy Corp. American Depositary Shares each representing five	Daqo New Energy Corp is polysilicon manufacturer based in China. The company manufactures and sells polysilicon to photovoltaic product manufacturers, who further process its polysilicon into ingots, wafers, cells and modules for solar power solutions.	{Technology,"Semiconductor Equipment & Materials",Semiconductors}	{CCMP,JKS,HQCL,SOL,DWDP,XLK}	Semiconductors	Technology	New York Stock Exchange	\N	2020-10-11 23:12:01.801935	2020-10-11 23:38:31.028166
DRQ	Dril-Quip Inc.	Dril-Quip Inc is in the oil and gas industry. It manufactures offshore drilling and production equipment for use in deepwater.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{NOV,SLB,NR,OIS,FTK,FET,BHGE}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:01.817633	2020-10-11 23:38:31.02582
DSE	Duff & Phelps Select Energy MLP Fund Inc.	Duff & Phelps Select Energy MLP Fund Inc is a closed-end management investment company. Its investment objective is to seek a high level of total return resulting from a combination of current tax-deferred distributions and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.827542	2020-10-11 23:38:31.028166
DSU	Blackrock Debt Strategies Fund Inc.	Blackrock Debt Strategies Fund Inc is a closed-end management investment company. Its investment objective is to provide current income by investing primarily in a diversified portfolio of US companies' debt instruments including corporate loans.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.837427	2020-10-11 23:38:31.025871
DTH	WisdomTree International High Dividend Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree International High Dividend Index.\n Under normal circumstances, at least 95% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a fundamentally weighted index that is comprised of companies with high dividend yields selected from the WisdomTree International Equity Index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.848052	2020-10-11 23:38:31.025871
DLA	Delta Apparel Inc.	Delta Apparel Inc is an international apparel design, marketing, manufacturing and sourcing company that offers a diverse portfolio of lifestyle basics and branded activewear apparel, headwear and related accessory products.	{"Consumer Cyclical","Apparel Manufacturing","Manufacturing - Apparel & Furniture"}	{GIL,COLM,SGC,NKE,OXM,PVH,EVK}	Manufacturing - Apparel & Furniture	Consumer Cyclical	NYSE American	\N	2020-10-11 23:12:01.717292	2020-10-11 23:38:31.023428
DLR	Digital Realty Trust Inc.	Digital Realty Trust Inc is a real estate investment trust. It owns and operates IT-related properties includes datacenter infrastructure that supports the applications and day-to-day operations of social networking and mobile communications.	{"Real Estate","REIT - Office",REITs}	{COR,CONE,QTS}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:01.731762	2020-10-11 23:38:31.02423
DNBF	DNB Financial Corp	DNB Financial Corp is a bank holding company. The bank is a full service commercial bank providing banking services to individuals and small to medium sized businesses.	{"Financial Services","Banks - Regional - US",Banks}	{CZNC,EMCF,FRBK,FULT,PWOD,ASRV,FCF,FNB,ACNB,PFIS}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:01.754552	2020-10-11 23:38:31.024353
DO	Diamond Offshore Drilling Inc.	Diamond Offshore Drilling Inc is engaged in the oil and gas sector. Its business involves offshore oil and gas drilling. The company offers a range of services in various markets, including the deep water and harsh environment.	{Energy,"Oil & Gas Drilling","Oil & Gas - Drilling"}	{PTEN,HP,RIG,RDC,ESV,NE}	Oil & Gas - Drilling	Energy	New York Stock Exchange	\N	2020-10-11 23:12:01.769534	2020-10-11 23:38:31.021426
DON	WisdomTree U.S. MidCap Dividend Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree U.S. MidCap Dividend Index.\n Under normal circumstances, at least 95% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a fundamentally weighted index that is comprised of the mid-capitalization segment of the U.S. dividend-paying market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.781617	2020-10-11 23:38:31.02582
DOTAW	Draper Oakwood Technology Acquisition Inc. Warrant		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:01.792546	2020-10-11 23:38:31.028166
DPZ	Domino's Pizza Inc	Domino's Pizza Inc is a quick service pizza restaurant chain. It is engaged in retail sales of food, equipment and supplies to company-owned and franchised Domino's Pizza stores, and receipt of royalties and fees from Domino's Pizza franchisees.	{"Consumer Cyclical",Restaurants}	{PZZA,JACK,DENN,MCD,YUM,CMG,WEN,FRSH,RAVE}	Restaurants	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:01.801103	2020-10-11 23:38:31.023007
DRE	Duke Realty Corporation	Duke Realty Corp is a self-administered and self-managed REIT. The company is engaged in the ownership, management and development of industrial and medical office real estate. It also provides real estate services to third-party owners.	{"Real Estate","REIT - Industrial",REITs}	{VTR,UHT,HCP,SBRA,PCH,TIPT,CTT,WELL}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:01.809237	2020-10-11 23:38:31.024353
DRUA	Dominion Energy Inc. 2016 Series A 5.25% Enhanced Junior Subordinated Notes due 2076		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:01.822589	2020-10-11 23:38:31.025871
DSKE	Daseke Inc.	Daseke Inc is an owner of open deck equipment and provider of open deck transportation and logistics solutions in North America. Its operating segment includes Flatbed Solutions and Specialized Solutions.	{Industrials,Trucking,"Transportation & Logistics"}	{ULH,YRCW}	Transportation & Logistics	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:01.832301	2020-10-11 23:38:31.025871
DTE	DTE Energy Company	DTE Energy Co operates in the utilities sector. The company mainly involves in the business of power generation. It also provides natural gas distribution services.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:01.842132	2020-10-11 23:38:31.028166
DTO	DB Crude Oil Double Short ETN due June 1 2038	The investment seeks to track the price and yield performance, before fees and expenses, 200% of the inverse daily performance of the Deutsche Bank Liquid Commodity Index - Optimum Yield Oil Excess Return.\n The fund allows investors to take a short view on the performance of the index. The index is a rules-based index composed of futures contracts on light sweet crude oil (WTI) and is intended to reflect the performance of crude oil.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.85195	2020-10-11 23:38:31.028166
DTW	DTE Energy Company 2017 Series E 5.25% Junior Subordinated Debentures due 2077		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:01.862879	2020-10-11 23:38:31.028166
DUKH	Duke Energy Corporation 5.125% Junior Subordinated Debentures due 2073	Duke Energy Corp operates regulates utilities across the US. The company distributes electricity and gas to its customers and engages in three segments: electric utilities and infrastructure; gas utilities and infrastructure; and commercial renewables.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:01.87372	2020-10-11 23:38:31.02582
FISI	Financial Institutions Inc.	Financial Institutions Inc offers full range of banking and related financial services to consumer,commercial and municipal customers through its bank and nonbank subsidiaries.	{"Financial Services","Banks - Regional - US",Banks}	{BDGE,AROW,FLIC,NBTB,CBU,EVBN}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:02.987516	2020-10-11 23:38:31.024353
DLBS	iPath US Treasury Long Bond Bear ETN	The investment seeks to provide investors with inverse exposure to the Barclays Long Bond US Treasury Futures Targeted Exposure Index™. \nThe Barclays Long Bond US Treasury Futures Targeted Exposure Index™ (the "index") is designed to decrease in response to an increase in the long-dated Treasury bond yields and to increase in response to a decrease in long-dated Treasury bond yields. The index targets a fixed level of sensitivity to changes in the yield of the current "cheapest-to-deliver" bond underlying the relevant long-dated Treasury futures contract at a given point in time.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.721086	2020-10-11 23:38:31.031328
DLS	WisdomTree International SmallCap Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree International SmallCap Dividend Index.\n Under normal circumstances, at least 95% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is comprised of the small-capitalization segment of the dividend-paying market in the industrialized world outside the U.S. and Canada. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.732965	2020-10-11 23:38:31.028166
DMPI	DelMar Pharmaceuticals Inc.	DelMar Pharmaceuticals Inc is a clinical stage drug development company. It is engaged in research and development of cancer indications.	{Healthcare,Biotechnology}	{NVLN,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:01.743929	2020-10-11 23:38:31.031328
DNP	DNP Select Income Fund Inc.	The primary investment objective is current income and long-term growth of income. Capital appreciation is a secondary objective. The Fund primarily invests in a diversified portfolio of equity and fixed-income securities of companies in the public utilities industry. The Fund invests in various sectors, including electric and gas, communication, real estate investment trusts, non-utility and utility.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.765204	2020-10-11 23:38:31.023428
DOO	WisdomTree International Dividend Top 100 Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree International Dividend ex-Financials Index.\n Under normal circumstances, at least 95% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in the component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a fundamentally weighted index that is comprised of high dividend-yielding international common stocks outside the financial sector. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.784141	2020-10-11 23:38:31.023428
DPST	Direxion Daily Regional Banks Bull 3X Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the daily performance of the S&P Regional Banks Select Industry Index.\n The fund invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, ETFs that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index is a modified equal-weighted index that is designed to measure performance of the stocks comprising the S&P Total Market Index that are classified in the GICS regional banks sub-industry. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.799138	2020-10-11 23:38:31.023428
DRIO	DarioHealth Corp.	DarioHealth Corp is a digital health company. It is engaged in development of the Dario Smart Diabetes Management Solution platform for diabetes management.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{EXAS,IONS,XLV}	Medical Diagnostics & Research	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:01.814115	2020-10-11 23:38:31.02423
DRV	Drexion Daily Real Estate Bear 3x Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the inverse of the daily performance of the MSCI US REIT IndexSM.\n The fund invests in swap agreements, futures contracts, short positions or other financial instruments that, in combination, provide inverse or short leveraged exposure to the index equal to at least 80% of its net assets (plus borrowing for investment purposes). The index is a free float-adjusted market capitalization weighted index that is comprised of equity REITs that are included in the MSCI US Investable Market 2500 Index, with the exception of specialty equity REITs. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.823212	2020-10-11 23:38:31.028166
DSLV	VS 3X INV SILVER	The investment seeks to replicate, net of expenses, three times the opposite (inverse) of the S&P GSCI Silver index ER.\n The index comprises futures contracts on a single commodity. The fluctuations in the values of it are intended generally to correlate with changes in the price of silver in global markets.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.833929	2020-10-11 23:38:31.02423
DTEA	DAVIDsTEA Inc.	DAVIDsTEA Inc is a retailer of packaged tea products in Canada. It sells loose-leaf teas, pre-packaged teas, tea sachets, tea accessories and beverages through its stores and website.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{FARM,MKC.V}	Consumer Packaged Goods	Consumer Defensive	NASDAQ Global Market	\N	2020-10-11 23:12:01.844415	2020-10-11 23:38:31.023428
DUC	Duff & Phelps Utility & Corporate Bond Trust Inc.	To seek high current income consistent with investing in securities of investment grade quality. It will invest substantially in a diversified portfolio of Utility Income Securities, Corporate Income Securities, Mortgage-Backed Securities and Asset-Backed Securities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.867245	2020-10-11 23:38:31.024353
DVAX	Dynavax Technologies Corporation	Dynavax Technologies Corp is a clinical-stage biopharmaceutical company. The company focuses on leveraging the power of the body's innate and adaptive immune responses through toll-like receptor ("TLR") stimulation.	{Healthcare,Biotechnology}	{GILD,BIIB,MNTA,AZN,GSK,PFE,MRK,AGEN,VICL,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:01.882754	2020-10-11 23:38:31.025871
DLX	Deluxe Corporation	Deluxe Corp provides various business solutions. It provides printed forms to businesses, including checks, deposit tickets, billing forms, invoices and personnel forms. It also provides marketing services including web design and hosting.	{Industrials,"Business Services"}	{INWK,EBF,RRD}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:01.73734	2020-10-11 23:38:31.023007
DMRL	DeltaShares S&P 500 Managed Risk	The investment seeks to track the investment results, before fees and expenses, of the S&P EPAC Ex. Korea LargeMidCap Managed Risk 2.0 Index.\n Under normal market conditions, the fund invests a substantial portion, but at least 80%, of its assets, exclusive of collateral held from securities lending, in securities comprising the underlying index. The underlying index seeks to achieve these objectives by allocating weightings among the S&P EPAC Ex. Korea LargeMidCap Index, the S&P U.S. Treasury Bond Current 5-Year Index and the S&P U.S. Treasury Bill 0-3 Month Index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.749126	2020-10-11 23:38:31.02423
DNN	Denison Mines Corp (Canada)	Denison Mines Corp is a Canadian-based uranium exploration and development company focused on the Athabasca Basin region of northern Saskatchewan. It also provides mine decommissioning services through its Denison Environmental Services division.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{CCJ,URG,KEP,UEC,WWR}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:01.761681	2020-10-11 23:38:31.031328
DOD	Deutsche Bank AG ELEMENTS Dogs of the Dow Total Return Index Note ELEMENTS Dogs of the Dow Linked to the Dow Jones High Yield Select 10 Total Return Index due N	The investment seeks to replicate, net of expenses, the Dow Jones High Yield Select 10 Total Return Index.\n The index tracks the stocks with the highest dividend yield in Dow Jones Industrial Average.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.775149	2020-10-11 23:38:31.031328
DORM	Dorman Products Inc.	Dorman Products Inc is a supplier of replacement parts and fasteners for passenger cars, light trucks, and heavy duty trucks in the automotive aftermarket.	{"Consumer Cyclical","Auto Parts",Autos}	{LKQ,CVGI,AZO,GPC,SMP,AAP,ORLY,VC}	Autos	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:01.787777	2020-10-11 23:38:31.028166
DPLO	Diplomat Pharmacy Inc.	Diplomat Pharmacy Inc is a provider of pharmaceutical drugs in the United States. It generates its revenue through the sale of prescriptions drugs for serious diseases such as hepatitis and multiple sclerosis.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{ACET,CAH,MCK,ABC,XLV}	Drug Manufacturers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:01.797432	2020-10-11 23:38:31.02423
DRIP	Direxion Daily S&P Oil & Gas Exp. & Prod. Bear 3X Shares	The investment seeks daily investment results, of 300% of the inverse (or opposite) of the daily performance of the S&P Oil & Gas Exploration & Production Select Industry Index.\n The fund invests in swap agreements, futures contracts, short positions or other financial instruments that, in combination, provide inverse (opposite) or short leveraged exposure to the index equal to at least 80% of the fund's net assets (plus borrowing for investment purposes). The index is designed to measure the performance of a sub-industry or group of sub-industries determined based on the Global Industry Classification Standards ("GICS"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.814426	2020-10-11 23:38:31.023428
DSM	Dreyfus Strategic Municipal Bond Fund Inc.	Dreyfus Strategic Municipal Bond Fund is a diversified closed-end management investment company. Its investment objective is to seek to maximize current income exempt from federal income tax to the extent consistent with the preservation of capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.833707	2020-10-11 23:38:31.028166
DTD	WisdomTree U.S. Total Dividend Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree U.S. Dividend Index.\n The fund invests at least 95% of its total assets (exclusive of collateral held from securities lending) in the component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a fundamentally-weighted index that is comprised of U.S. companies listed on a U.S. stock market that pay regular cash dividends. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.842239	2020-10-11 23:38:31.031328
DTN	WisdomTree U.S. Dividend ex-Financials Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree U.S. Dividend ex-Financials Index.\n The fund normally invests at least 95% of its total assets (exclusive of collateral held from securities lending) in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is comprised of the 10 highest dividend-yielding companies in each sector, selected from the three hundred largest companies by market value in the WisdomTree U.S. Dividend Index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.850939	2020-10-11 23:38:31.02423
DNOW	NOW Inc.	NOW Inc is a distributor to energy products and products for industrial applications. Its products includes consumable maintenance, repair and operating supplies, pipe, valves, fittings, flanges, electrical, artificial lift solutions, pumps, mill tools.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{EEP,EEQ,PSXP,ENLC}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:01.764531	2020-10-11 23:38:31.02582
DOL	WisdomTree International LargeCap Dividend Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree International LargeCap Dividend Index.\n The fund normally invests at least 95% of its total assets (exclusive of collateral held from securities lending) in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a fundamentally weighted index that is comprised of the large-capitalization segment of the dividend-paying market in the industrialized world outside the U.S. and Canada. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.779834	2020-10-11 23:38:31.024353
DOVA	Dova Pharmaceuticals Inc.	Dova Pharmaceuticals Inc is a pharmaceutical company focused on acquiring, developing and commercializing drug candidates for diseases that are treated by specialist physicians, with an initial focus on addressing thrombocytopenia.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:01.796385	2020-10-11 23:38:31.024353
DRAD	Digirad Corporation	Digirad Corp develops nuclear technology for mobile cardiovascular imaging for various application. The company commercialized the first nuclear gamma camera and offers its products to doctors' offices and hospitals.	{Healthcare,"Medical Devices"}	{RDNT,FONR,AMS,CYH,CCM,ICAD,THC,XLV}	Medical Devices	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:01.809934	2020-10-11 23:38:31.02582
DRRX	DURECT Corporation	Durect Corp is a biopharmaceutical company. It develops pharmaceutical products based on two categories which include new chemical entities and drug delivery programs. The lead candidate for the company is DUR-928.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{JNJ,LLY,PFE,PCRX,HRTX,CORI,ZGNX,XLV}	Drug Manufacturers	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:01.820104	2020-10-11 23:38:31.02423
DSI	iShares MSCI KLD 400 Social	The investment seeks to track the investment results of the MSCI KLD 400 Social Index composed of U.S. companies that have positive environmental, social and governance characteristics as identified by the index provider.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index is a free float-adjusted market capitalization index designed to target U.S. companies that have positive environmental, social and governance ("ESG") characteristics.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.830226	2020-10-11 23:38:31.023428
DSUM	Invesco Chinese Yuan Dim Sum Bond	The investment seeks to track the investment results (before fees and expenses) of the FTSE Custom Dim Sum (Offshore CNY) Bond Index (the "underlying index").\n The fund generally invests at least 80% of its total assets in Chinese Renminbi ("RMB")-denominated bonds that comprise the underlying index. The underlying index measures the performance of RMB-denominated "Dim Sum" bonds that are issued and settled outside of mainland China. Dim Sum bonds are RMB-denominated and generally are issued in Hong Kong by a variety of entities ranging from governments to corporations.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.839417	2020-10-11 23:38:31.024353
DTQ	DTE Energy Company 2012 Series C 5.25% Junior Subordinate Debentures due December 1 2062	DTE Energy Co operates in the utilities sector. The company mainly involves in the business of power generation. It also provides natural gas distribution services.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:01.853328	2020-10-11 23:38:31.023007
DUG	ProShares UltraShort Oil & Gas	The investment seeks daily investment results, before fees and expenses, that correspond to two times the inverse (-2x) of the daily performance of the Dow Jones U.S. Oil & GasSM Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index seeks to measure the performance of certain companies in the oil and gas sector of the U.S. equity market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.868327	2020-10-11 23:38:31.028166
DVD	Dover Motorsports Inc.	Dover Motorsports Inc is a marketers and promoters of motorsports entertainment. Through its subsidiaries, it owns and operates Dover International Speedway and Nashville Superspeedway. It is also engaged in harness horse racing operations.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{ISCA,CHDN,TRK}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:01.883873	2020-10-11 23:38:31.024353
DWAS	Invesco DWA SmallCap Momentum ETF	The investment seeks to track the investment results (before fees and expenses) of the Dorsey Wright® SmallCap Technical Leaders Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in equity securities of small capitalization companies that comprise the underlying index. The index provider selects such securities pursuant to its proprietary selection methodology, which is designed to identify securities that demonstrate powerful relative strength characteristics.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.902098	2020-10-11 23:38:31.023007
DOC	Physicians Realty Trust of Beneficial Interest	Physicians Realty Trust is a real estate investment trust. It focusses its investments on healthcare facilities such as hospitals, medical office buildings and diagnostic facilities located in the United States.	{"Real Estate","REIT - Healthcare Facilities",REITs}	{HR,ARE,VTR,LTC,NHI,HCP,WELL}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:01.769089	2020-10-11 23:38:31.02423
DOTA	Draper Oakwood Technology Acquisition Inc.	Draper Oakwood Technology Acquisition Inc is a blank check company formed for the purpose of entering into a merger, share exchange, asset acquisition, stock purchase, recapitalization, reorganization or other similar business.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:01.789353	2020-10-11 23:38:31.021426
DRH	Diamondrock Hospitality Company	Diamondrock Hospitality Co functions in the real estate sector. As a real estate investment trust, the company primarily focuses on a portfolio of lodging facilities such as hotels and resorts.	{"Real Estate","REIT - Hotel & Motel",REITs}	{LHO,HST,AHT,PEB,CHSP,RLJ,HPT,CDOR}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:01.810371	2020-10-11 23:38:31.021426
DS	Drive Shack Inc.	Drive Shack Inc. is a owner and operator of golf-related leisure and entertainment businesses. It's operating business segments are Traditional Golf properties, Entertainment Golf venues and Debt Investments.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:01.825016	2020-10-11 23:38:31.02582
DSX	Diana Shipping inc.	Diana Shipping Inc is a global shipping company. The company engages in transportation of dry-bulk cargoes and commodities such as coal, iron ore, and grain. Its operates dry-bulk carriers, most of which are Panamax and Capesize vessels.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{NM,NMM,SB,ESEA,SBLK,GNK,EGLE,DRYS,DCIX}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:01.840836	2020-10-11 23:38:31.02423
DTRM	Determine Inc.	Determine Inc provides SaaS Source to Pay and Enterprise Contract Lifecyclem Management solutions. Its software suite includes strategic sourcing, supplier management, contract management and procure-to-pay applications.	{Technology,"Software - Infrastructure","Application Software"}	{SAP,IBM,CRM,ECOM,ORCL,XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:01.855454	2020-10-11 23:38:31.031328
DUK	Duke Energy Corporation (Holding Company)	Duke Energy Corp operates regulates utilities across the US. The company distributes electricity and gas to its customers and engages in three segments: electric utilities and infrastructure; gas utilities and infrastructure; and commercial renewables.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{PCG,FE,AEE,AEP,SO,D,SCG,PPL,NEE,XEL,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:01.87256	2020-10-11 23:38:31.025871
DVYA	iShares Asia / Pacific Dividend 30 Index Fund Exchange Traded Fund	The investment seeks to track the investment results of the Dow Jones Asia/Pacific Select Dividend 30 Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The underlying index measures the stock performance of high dividend paying companies listed in Australia, Hong Kong, Japan, New Zealand and Singapore. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.892621	2020-10-11 23:38:31.028166
DXJ	WisdomTree Japan Hedged Equity Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree Japan Hedged Equity Index.\n Under normal circumstances, at least 95% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in the component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is designed to provide exposure to Japanese equity markets while at the same time neutralizing exposure to fluctuations of the Japanese yen relative to the U.S. dollar. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.973238	2020-10-11 23:38:31.023007
DZSI	DASAN Zhone Solutions Inc.	DASAN Zhone Solutions Inc designs, develops and manufactures communications network equipment for telecommunications operators and enterprises.	{Technology,"Communication Equipment"}	{ADTN,AWRE,NTGR,JNPR,CALX,CIEN,RBBN,XLK}	Communication Equipment	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:02.008364	2020-10-11 23:38:31.031328
EAI	Entergy Arkansas Inc. First Mortgage Bonds 4.875% Series Due September 1 2066		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:02.028142	2020-10-11 23:38:31.023007
EBIX	Ebix Inc.	Ebix Inc provides software and e-commerce solutions to the insurance, finance and healthcare industries.	{Technology,"Software - Application","Application Software"}	{TRI,ULTI,JW.A,CRM,GWRE,BNFT,MJCO,DXC,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:02.039883	2020-10-11 23:38:31.021426
EBSB	Meridian Bancorp Inc.	Meridian Bancorp Inc owns and operates the East Boston Savings Bank. Through East Boston Savings Bank, it offers deposit accounts, loan products to individuals and businesses. The company operates its business across multiple locations in Massachusetts.	{"Financial Services","Savings & Cooperative Banks",Banks}	{FCAP,IBKC,LSBK,NWBI}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:02.052738	2020-10-11 23:38:31.031328
DOV	Dover Corporation	Dover Corp is a multi-business operator in North America. It has footprint in engineered systems, energy, fluids and refrigeration and food equipment businesses.	{Industrials,"Diversified Industrials","Industrial Products"}	{NOV,FLS,DHR,GE,IEX,ITW,PH,SPXC,ATU,IR}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:01.792709	2020-10-11 23:38:31.025871
DRI	Darden Restaurants Inc.	Darden Restaurants Inc is engaged in the food & beverage industry. Its primary occupation involves the operation of dining restaurants under trade names such as Olive Garden, LongHorn Steakhouse and Bahama Breeze.	{"Consumer Cyclical",Restaurants}	{BJRI,RUTH,CBRL,DIN,DFRG,BLMN}	Restaurants	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:01.81145	2020-10-11 23:38:31.028166
DSPG	DSP Group Inc.	DSP Group provides wireless chipset solutions for converged communications. Its product portfolio includes Cordless Phones, Gateways, Home Automation, Voice over Internet Protocol, and HDClear.	{Technology,Semiconductors}	{INTC,CRUS,QCOM,NXPI,SQNS,TXN,ADI,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:01.83428	2020-10-11 23:38:31.031328
DTJ	DTE Energy Company 2016 Series B 5.375% Junior Subordinated Debentures due 2076		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:01.849231	2020-10-11 23:38:31.024353
DTYL	iPath US Treasury 10 Year Bull ETN	The investment seeks to provide investors with exposure to the Capital 10Y US Treasury Futures Targeted Exposure Index™. \nThe Barclays 10Y US Treasury Futures Targeted Exposure Index™ (the "index") is designed to decrease in response to an increase in the 10-year Treasury note yields and to increase in response to a decrease in 10-year Treasury note yields. The index targets a fixed level of sensitivity to changes in the yield of the current "cheapest-to-deliver" note underlying the relevant 10-year Treasury futures contract at a given point in time.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.86367	2020-10-11 23:38:31.02423
DVCR	Diversicare Healthcare Services Inc.	Diversicare Healthcare Services Inc provides long term care services to its nursing center patients. Its services include nursing, ancillary health care services and assisted living.	{Healthcare,"Long-Term Care Facilities","Health Care Providers"}	{ENSG,NHC,CSU,GEN,FVE,RHE,XLV}	Health Care Providers	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:01.884097	2020-10-11 23:38:31.021426
DWAQ	Invesco DWA NASDAQ Momentum ETF	The investment seeks to track the investment results (before fees and expenses) of the Dorsey Wright® NASDAQ Technical Leaders Index.\n The fund generally will invest at least 90% of its total assets in the securities that comprise the underlying index. The underlying index is composed of approximately 100 securities from an eligible universe of approximately 1,000 of the largest capitalization companies whose securities are included within the NASDAQ US Benchmark Index, except U.S.-listed American depositary receipts ("ADRs") or foreign securities that trade on The Nasdaq Stock Market.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.900457	2020-10-11 23:38:31.024353
DWDP	DowDuPont Inc.	DowDuPont Inc is jointly owned by Dow and DuPont for the purpose of forming publicly traded companies in the agriculture, materials science and specialty products sectors.	{"Basic Materials",Chemicals}	{}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:01.920946	2020-10-11 23:38:31.02582
DWX	SPDR S&P International Dividend	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the S&P International Dividend Opportunities® Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index and in depositary receipts based on securities comprising the index. The index is designed to measure the performance of 100 high-yielding international common stocks. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.946464	2020-10-11 23:38:31.024353
DXGE	WisdomTree Germany Hedged Equity Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree Germany Hedged Equity Index (the "index").\n The fund normally invests at least 80% of its total assets in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a dividend weighted index designed to provide exposure to Germany equity markets while at the same time neutralizing exposure to fluctuations of the value of the euro relative to the U.S. dollar. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.966225	2020-10-11 23:38:31.02423
DY	Dycom Industries Inc.	Dycom Industries Inc provides specialty contracting services throughout the United States and in Canada. Its services include engineering, construction, maintenance and installation services to telecommunications providers.	{Industrials,"Engineering & Construction"}	{MYRG,MTZ,EME,PWR,IESC}	Engineering & Construction	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:01.998684	2020-10-11 23:38:31.028166
E	ENI S.p.A.	Eni SpA together with its subsidiaries offers integrated energy services. It is engaged in exploring, producing, and refining oil and gas.	{Energy,"Oil & Gas Integrated","Oil & Gas - Integrated"}	{TOT,PBR.A,XOM,BP,PTR,CVX,MRO,RDS.A}	Oil & Gas - Integrated	Energy	New York Stock Exchange	\N	2020-10-11 23:12:02.020818	2020-10-11 23:38:31.028166
EAT	Brinker International Inc.	Brinker International Inc operates casual-dining restaurants. It franchises brands includes Maggiano's and Chili's Grill & Bar.	{"Consumer Cyclical",Restaurants}	{ARKR,RRGB,BJRI,CBRL,DIN,BDL,BLMN}	Restaurants	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:02.037347	2020-10-11 23:38:31.028166
DPG	Duff & Phelps Global Utility Income Fund Inc.	Duff & Phelps Global Utility Income is an investment fund. The Fund's investment objective is to seek total return, resulting primarily from a high level of current incom, with an emphasis on providing tax-advantaged dividend income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.800535	2020-10-11 23:38:31.021426
DRN	Direxion Daily Real Estate Bull 3x Shares	The investment seeks daily investment results of 300% of the daily performance of the MSCI US REIT IndexSM.\n The fund invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, ETFs that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index is a free float-adjusted market capitalization weighted index that is comprised of REITs that are included in the MSCI US Investable Market 2500 Index. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.817125	2020-10-11 23:38:31.031328
DSGX	The Descartes Systems Group Inc.	The Descartes Systems Group Inc provides logistics technology solutions for the transportation and logistics, distribution, manufacturing, and retail, business services and public sector industries.	{Technology,"Software - Application","Application Software"}	{MANH,AMSWA,SAP,ORCL,AMBR,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:01.830298	2020-10-11 23:38:31.021426
DSWL	Deswell Industries Inc.	Deswell Industries Inc is a manufacturer of injection-molded plastic parts and components and electronic products.	{"Consumer Cyclical","Rubber & Plastics",Autos,Industrials,Conglomerates}	{CTIB,TRS,CMT,MYE,TG,BERY}	Conglomerates	Industrials	NASDAQ Global Market	\N	2020-10-11 23:12:01.839769	2020-10-11 23:38:31.023007
DTUL	iPath US Treasury 2 Yr Bull ETN	The investment seeks to provide investors with exposure to the Barclays 2Y US Treasury Futures Targeted Exposure Index™. \nThe Barclays 2Y US Treasury Futures Targeted Exposure Index™ (the "index") is designed to decrease in response to an increase in the 2-year Treasury note yields and to increase in response to a decrease in 2-year Treasury note yields. The index targets a fixed level of sensitivity to changes in the yield of the current "cheapest-to-deliver" note underlying the relevant 2-year Treasury futures contract at a given point in time.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.858097	2020-10-11 23:38:31.021426
DVP	Deep Value	The investment seeks to track the price and total return performance, before fees and expenses, of the Deep Value Index.\n The index is composed of the common stock of typically 20 companies included in the S&P 500® Index that have been selected through a proprietary ranking system developed by Tiedemann Wealth Management, LLC, that evaluates the earnings and cash flows of each company to create a final universe of companies that are deeply undervalued as compared to the S&P 500® Index overall. Under normal circumstances, at least 80% of the fund's total assets will be invested in the component securities of the index. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.889274	2020-10-11 23:38:31.023007
DWCH	Datawatch Corporation	Datawatch Corp provides visual data discovery software that optimizes multiple sources and a variety of data that delivers next generation analytics to reveal insights for improving business.	{Technology,"Software - Application","Application Software"}	{MSTR,BVSN,IBM,TDC,SPLK,DATA,VRNS,XRX,XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:01.904277	2020-10-11 23:38:31.023428
DWSN	Dawson Geophysical Company	Dawson Geophysical Co is a provider of seismic data acquisition services throughout the continental United States and Canada. The company acquires geophysical data using the latest in 3-D survey techniques.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{LPSN,RNG,IO}	Oil & Gas - Services	Energy	Nasdaq Global Select	\N	2020-10-11 23:12:01.941071	2020-10-11 23:38:31.023428
DXD	ProShares UltraShort Dow30	The investment seeks daily investment results, before fees and expenses, that correspond to two times the inverse (-2x) of the daily performance of the Dow Jones Industrial Average® Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index is a price-weighted index and includes 30 large-cap, "blue-chip" U.S. stocks, excluding utility and transportation companies. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.966383	2020-10-11 23:38:31.02582
DZK	Direxion Developed Markets Bull 3x Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the daily performance of the MSCI EAFE® Index.\n The fund invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, ETFs that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index is a free float-adjusted market capitalization weighted index that is designed to measure the performance of large- and mid-capitalization companies across the 21 developed market countries around the world, excluding the U.S. and Canada. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.007479	2020-10-11 23:38:31.02582
EBAY	eBay Inc.	eBay Inc is a commerce platform and provides online marketplace for sale of goods. Its marketplace platforms include its online marketplace located at www.ebay.com, its localized counterparts and the eBay mobile apps.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{MSFT,AMZN,COST,FISV,OSTK,MELI,GRPN,EXPE,GOOGL}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:02.035789	2020-10-11 23:38:31.02582
DTV	DTE Energy Company 2016 Corporate Units	DTE Energy Co operates in the utilities sector. The company mainly involves in the business of power generation. It also provides natural gas distribution services.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:01.86277	2020-10-11 23:38:31.023428
DVA	DaVita Inc.	DaVita Inc is engaged in operating kidney dialysis centers and providing related lab services in dialysis centers and in contracted hospitals across USA. It also operates other ancillary services and strategic initiatives.	{Healthcare,"Medical Care","Health Care Providers"}	{LPNT,NXTM,FMS,HNGR,UHS,LH,MD,THC,XLV}	Health Care Providers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:01.878007	2020-10-11 23:38:31.023007
DVY	iShares Select Dividend ETF	The investment seeks to track the investment results of the Dow Jones U.S. Select Dividend Index composed of relatively high dividend paying U.S. equities.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index measures the performance of a selected group of equity securities issued by companies that have provided relatively high dividend yields on a consistent basis over time.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.891806	2020-10-11 23:38:31.021426
DWPP	First Trust Dorsey Wright People's Portfolio ETF	The investment seeks results that correspond generally to an index called the Nasdaq Dorsey Wright People's Portfolio Index.\n The fund will normally invest at least 80% of its total assets (including investment borrowings) in common stocks or U.S. Treasury Bills ("T-Bills") that comprise the index. The index is designed to provide exposure to either the U.S. equity market or T-Bills, based upon daily relative strength readings resulting from a wholly-quantitative process.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.930728	2020-10-11 23:38:31.023007
DXJS	WisdomTree Japan Hedged SmallCap Equity Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree Japan Hedged SmallCap Equity Index.\n The fund normally invests at least 80% of its total assets in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a dividend weighted index designed to provide exposure to Japanese equity markets while at the same time neutralizing exposure to fluctuations of the value of the Japanese yen relative to the U.S. dollar. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.972608	2020-10-11 23:38:31.021426
DYB	WisdomTree Dynamic Bearish U.S. Equity Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree Dynamic Bearish U.S. Equity Index.\n Under normal circumstances, at least 80% of the fund's total assets will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index includes: (i) long U.S. equity positions (the "Long Equity Basket") or long U.S. Treasury positions (the "Long Treasury Basket"); and (ii) short U.S. equity positions (the "Short Equity Basket"). The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:02.004573	2020-10-11 23:38:31.023007
EAD	Wells Fargo Income Opportunities Fund	Wells Fargo Income Opportunities Fund is a diversified closed-end management investment company. Its products and services are mutual funds, closed-end funds, managed accounts services and variable trust funds.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:02.02539	2020-10-11 23:38:31.02423
EBF	Ennis Inc.	Ennis Inc provides business equipment to various industries. It is engaged in manufacturing, designing and selling business forms and other printed business products.	{Industrials,"Business Equipment","Industrial Products"}	{INWK,DLX,OXM,RRD}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:02.038158	2020-10-11 23:38:31.02423
EC	Ecopetrol S.A. American Depositary Shares	Ecopetrol SA is an oil company. The firm explores, develops, and conducts oil production activities internationally. It operates in Latin America, United States and also engages in refining and marketing of crude oil and its byproducts.	{Energy,"Oil & Gas Integrated","Oil & Gas - Integrated"}	{YPF,E,TOT,XOM,OXY,BP,PBR,CVX,MRO,HES}	Oil & Gas - Integrated	Energy	New York Stock Exchange	\N	2020-10-11 23:12:02.056886	2020-10-11 23:38:31.02423
ECL	Ecolab Inc.	Ecolab Inc produces and markets cleaning and sanitation products for the hospitality, healthcare, and industrial markets. It offers a variety of products and services, which include dish and laundry washing systems, pest control, and repair services.	{"Basic Materials","Specialty Chemicals",Chemicals}	{GRA,CMD,CHD,CLX,IFF,PPG,PG,ROL,NEU,MOS}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:02.072144	2020-10-11 23:38:31.02423
EDC	Direxion Emerging Markets Bull 3X Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the daily performance of the MSCI Emerging Markets IndexSM.\n The fund invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, exchange-traded funds ("ETFs") that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index is a free float-adjusted market capitalization weighted index that is designed to represent the performance of large- and mid-capitalizations securities across the 24 emerging market countries. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.090084	2020-10-11 23:38:31.02423
FITB	Fifth Third Bancorp	Fifth Third Bancorp provides banking products and services. The company offers checking, savings and money market accounts and credit products. Its segments are Commercial Banking, Branch Banking, Consumer Lending and Wealth and Asset Management.	{"Financial Services","Banks - Regional - US",Banks}	{WTFC,CBSH,HBAN,BAC,PNC,WFC,STI,TCF,LCNB}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:02.995716	2020-10-11 23:38:31.031328
DTY	DTE Energy Company 2016 Series F 6.00% Junior Subordinated Debentures due 2076		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:01.862792	2020-10-11 23:38:31.023007
DVN	Devon Energy Corporation	Devon Energy Corp is engaged in the oil and gas sector. It is an independent energy company involved in the exploration, development and production of oil, natural gas and NGLs.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{CHK,NFX,PXD,APC,OXY,EOG,BHP,MRO,COP,QEP}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:01.888494	2020-10-11 23:38:31.02423
DWAT	Arrow DWA Tactical ETF	The investment seeks to achieve long-term capital appreciation with capital preservation as a secondary objective.\n The fund primarily invests in other exchange traded funds ("ETFs") that each invests primarily in domestic and foreign (including emerging markets) (i) equity securities of any market capitalization, (ii) fixed-income securities of any credit quality, or (iii) alternative assets. It may invest up to 25% of its total assets in the Tactical Subsidiary.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.903466	2020-10-11 23:38:31.028166
DXJF	WisdomTree Japan Hedged Financials Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree Japan Hedged Financials Index (the "index").\n Under normal circumstances, at least 80% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index consists of companies incorporated in Japan that trade primarily on the Tokyo Stock Exchange. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.973356	2020-10-11 23:38:31.028166
DYNC	Vistra Energy Corp. 7.00% Tangible Equity Units	Vistra Energy Corp through its subsidiaries is in electricity market activities including power generation, wholesale energy sales and purchases, commodity risk management and retail sales of electricity to end users.	{Utilities,"Utilities - Independent Power Producers"}	{XLU}	Utilities - Independent Power Producers	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:02.005086	2020-10-11 23:38:31.025871
ECC	Eagle Point Credit Company Inc.	In the primary CLO market, the fund seeks to invest in CLO securities that\nthe Adviser believes have the potential to generate attractive risk-adjusted returns and to outperform other similar CLO securities issued within the respective vintage period. \n\nThe Fund intends to pursue a differentiated strategy within the CLO market focused on:\n• proactive sourcing and identification of investment opportunities;\n• utilization of our methodical and rigorous investment analysis and due diligence process;\n• active involvement at the CLO structuring and formation stage; and\n• taking, in many instances, significant stakes in CLO equity and junior debt tranches.	{"Financial Services","Asset Management"}	{}			New York Stock Exchange	\N	2020-10-11 23:12:02.058359	2020-10-11 23:38:31.023007
EDEN	iShares Inc MSCI Denmark	The investment seeks to track the investment results of the MSCI Denmark IMI 25/50 Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is designed to measure broad-based equity market performance in Denmark. A capping methodology is applied that limits the weight of any single component to a maximum of 25% of the underlying index. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:02.095696	2020-10-11 23:38:31.02582
EDU	New Oriental Education & Technology Group Inc. Sponsored ADR representing 1 (Cayman Islands)	New Oriental Education & Technology Group Inc provides private educational services in China based on the number of program offerings, total student enrollments, and geographic presence.	{"Consumer Defensive","Education & Training Services",Education}	{LOPE,LRN,PSO,HMHC}	Education	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:02.110427	2020-10-11 23:38:31.031328
EELV	Invesco S&P Emerging Markets Low Volatility	The investment seeks to track the investment results (before fees and expenses) of the S&P BMI Emerging Markets Low Volatility IndexTM (the "underlying index").\n The fund will invest at least 90% of its total assets in the securities of companies that comprise the underlying index. S&P Dow Jones Indices LLC ("S&P DJI" or the "index provider") compiles, maintains and calculates the underlying index, which is designed to measure the performance of 200 of the least volatile stocks of the S&P Emerging Plus LargeMidCap Index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.127593	2020-10-11 23:38:31.02582
EEP	Enbridge Energy L.P. Class A	Enbridge Energy Partners LP provides crude oil and liquid petroleum transportation and storage services. In addition, it also provides natural gas gathering, processing, transportation, and marketing services in the United States of America.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{GEL,EPD,PAA,MMP,HEP,DCP,EEQ,BKEP,DKL,SXE}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:02.13851	2020-10-11 23:38:31.028166
EEX	Emerald Expositions Events Inc.	Emerald Expositions Events Inc is an operator of B2B trade shows in the United States by NSF.	{Industrials,"Business Services"}	{NEWT,ITT}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:02.147456	2020-10-11 23:38:31.02423
DVHL	ETRACS 2xLeveraged Diversified High Income ETN	The investment seeks a return linked to the NYSE® Diversified High Income Index. \n The ETRACS Monthly Pay 2xLeveraged Diversified High Income ETN due November 12, 2043 (the “Securities”) are a series of Monthly Pay 2xLeveraged Exchange Traded Access Securities (ETRACS). The Securities are senior unsecured debt securities issued by UBS AG (UBS). The index measures the performance of a diversified basket of publicly-traded securities that historically have paid high dividends or distributions. The index constituent Securities must satisfy certain dividend or distribution yield and frequency criteria, liquidity criteria and other eligibility requirements.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.886207	2020-10-11 23:38:31.028166
DWIN	PowerShares DWA Tactical Multi-Asset Income Portfo	The investment seeks investment results that generally correspond to the price and yield of the Dorsey Wright® Multi-Asset Income Index.\n The fund generally will invest at least 90% of its total assets in securities that comprise the underlying index. It is a "fund of funds," meaning that it invests substantially all of its assets in the shares of other ETFs eligible for inclusion in the underlying index, rather than in securities of individual companies. The underlying index seeks to gain exposure to the income-producing segments of the U.S. securities market that display the strongest relative strength, as evaluated on a monthly basis. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.925005	2020-10-11 23:38:31.028166
DX	Dynex Capital Inc.	Dynex Capital Inc is a real estate investment trust. It internally manages mortgage real estate investment trust, or mortgage REIT, which invests in residential and commercial mortgage-backed securities on a leveraged basis.	{"Real Estate","REIT - Residential",REITs}	{NYMT,CMO,MFA,IVR,PMT}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:01.95382	2020-10-11 23:38:31.021426
DXPE	DXP Enterprises Inc.	DXP Enterprises Inc is engaged in the business of distributing, maintenance, repair and operating products, and service to industrial customers.	{Industrials,"Industrial Distribution"}	{FAST,LAWS,TITN,MSM,AXE,AIT,GE,MMM,KAMN,HDS}	Industrial Distribution	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:01.978116	2020-10-11 23:38:31.031328
DYNT	Dynatronics Corporation	Dynatronics CorP is engaged in designing, manufacturing and distributing medical devices, therapeutic and medical treatment tables, rehabilitation equipment, custom athletic training treatment tables and equipment, institutional cabinetry and others.	{Healthcare,"Medical Devices"}	{ANIK,CNMD,CAH,HNGR,JNJ,SYK,AHPI,FCRE,XLV}	Medical Devices	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:02.004274	2020-10-11 23:38:31.021426
EAB	Entergy Arkansas Inc. First Mortgage Bonds 4.90% Series Due December 1 2052	Entergy Corporation is a Fortune 500 integrated energy Company engaged primarily in electric power production and retail distribution operations.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:02.021175	2020-10-11 23:38:31.023428
EARS	Auris Medical Holding AG	Auris Medical Holding AG is a clinical-stage biopharmaceutical company. It is engaged in the development of novel products for the treatment of inner ear disorders. Its product candidate is AM-101.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:02.034745	2020-10-11 23:38:31.024353
EBMT	Eagle Bancorp Montana Inc.	Eagle Bancorp Montana Inc is a bank holding company. The company through its subsidiary is engaged in providing banking services including accepting deposits and investing in loans and securities.	{"Financial Services","Banks - Regional - US",Banks}	{WAFD,PROV,RVSB,TSBK,BYFC,FFNW,TBNK,ANCB,BANC}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:02.048628	2020-10-11 23:38:31.025871
ECCA	Eagle Point Credit Company Inc. Series A Term Preferred Stock due 2022	Eagle Point Credit Co Inc is a non-diversified, closed-end management investment company. The Company's investment objective is to generate high current income, with a secondary objective to generate capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.060928	2020-10-11 23:38:31.021426
ECNS	iShares MSCI China Small-Cap	The investment seeks to track the investment results of the MSCI China Small Cap Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is a free float-adjusted market capitalization-weighted index designed to measure the performance of equity securities in the bottom 14% by market capitalization of the Chinese equity securities markets, as represented by the H-Shares and B-Shares markets.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.073343	2020-10-11 23:38:31.02582
EDAP	EDAP TMS S.A.	Edap TMS SA is a holding company. The company through its divisions HIFU and UDS,develops, produces and markets minimally invasive medical devices for urological diseases. Its divisions operate in Europe, the Americas, Asia and the rest of the world.	{Healthcare,"Medical Devices"}	{RMTI,NXTM,BSX,XLV}	Medical Devices	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:02.085889	2020-10-11 23:38:31.023007
EDF	Stone Harbor Emerging Markets Income Fund of Beneficial Interest	Stone Harbour Emerg. Mkts Fixed Inc Fund is a closed-end management investment company. Its investment objective is to maximize total return, which consists of income on its investments and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.095685	2020-10-11 23:38:31.031328
DWTR	PowerShares DWA Tactical Sector Rotation Portfolio	The investment seeks investment results that generally correspond (before fees and expenses) of the Dorsey Wright® Sector 4 Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in securities that comprise the underlying index. It is a "fund of funds," meaning that it invests its assets in the shares of other exchange-traded funds ("ETFs") eligible for inclusion in the underlying index. The underlying index seeks to gain exposure to the sectors of the U.S. equity markets that display the strongest relative strength, as evaluated on a monthly basis. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.944494	2020-10-11 23:38:31.028166
DXLG	Destination XL Group Inc.	Destination XL Group Inc is a retailer of men's clothing. It operates through brands like Polo, Reebok, Oak Hill, Robert Graham, Lacoste, Levi's and True Nation.	{"Consumer Cyclical","Apparel Stores","Retail - Apparel & Specialty"}	{ZUMZ,AEO,BKE,GPS,EXPR,FRAN,RL,TLRD}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:01.976725	2020-10-11 23:38:31.024353
DYSL	Dynasil Corporation of America	Dynasil Corp of America is a provider of scintillation materials for radiation detection and imaging, in customized optical coatings, and in optical components for the photonics industry.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{IIVI,REFR,LPTH,GLW,TMO,XLK}	Computer Hardware	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:02.005121	2020-10-11 23:38:31.028166
EAST	Eastside Distilling Inc.	Eastside Distilling Inc is a producer and marketer of hand-crafted spirits in beverage alcohol categories like bourbon, American whiskey, rum, and vodka.	{"Consumer Defensive","Beverages - Wineries & Distilleries","Beverages - Alcoholic"}	{MDCA}	Beverages - Alcoholic	Consumer Defensive	NASDAQ Capital Market	\N	2020-10-11 23:12:02.036311	2020-10-11 23:38:31.031328
EBS	Emergent Biosolutions Inc.	Emergent BioSolutions Inc is a biotechnology company operating in two divisions: biodefense and biosciences. The company focuses on countermeasures that address public health threats around hematology and oncology therapeutics.	{Healthcare,Biotechnology}	{SIGA,CELG,CLDX,GSK,PFE,MRK,CMRX,SNGX,XLV}	Biotechnology	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:02.051282	2020-10-11 23:38:31.024353
ECF	Ellsworth Growth and Income Fund Ltd.	Ellsworth Growth and Income Fund Ltd operate as a diversified, closed-end management investment company. It invests in convertible and equity securities, with the objectives of providing income and the potential for capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:02.06366	2020-10-11 23:38:31.025871
EDN	Empresa Distribuidora Y Comercializadora Norte S.A. (Edenor) Empresa Distribuidora Y Comercializadora Norte S.A. (Edenor) American Depositary Shares	Empresa Distribuidora y Comercializadora Norte SA distributes electricity in Argentina. The firm serves the northern portion of the city of Buenos Aires primarily to residential, commercial, industrial, and public service.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{ENIA,CIG,ELP,CPL,PAM,AY,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:02.104276	2020-10-11 23:38:31.023007
EEA	The European Equity Fund Inc.	The European Equity Fund Inc is a United States based diversified, closed-end management investment company. The fund seeks long-term capital appreciation through investment primarily in equity and equity-linked securities of issuers domiciled in Europe.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.121379	2020-10-11 23:38:31.023007
EEMO	Invesco S&P Emerging Markets Momentum	The investment seeks to track the investment results (before fees and expenses) of the S&P Momentum Emerging Plus LargeMidCap Index (the "underlying index").\n The fund will invest at least 90% of its total assets in the securities of companies that comprise the underlying index, as well as ADRs and GDRs that are based on the securities in the underlying index. Strictly in accordance with its guidelines and mandated procedures, the index provider compiles, maintains and calculates the underlying index, which is composed of constituents of the S&P Emerging Plus LargeMidCap IndexTM that have the highest "momentum score."	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.133852	2020-10-11 23:38:31.02423
EEV	ProShares UltraShort MSCI Emerging Markets	The investment seeks daily investment results, before fees and expenses, that correspond to two times the inverse (-2x) of the daily performance of the MSCI Emerging Markets Index®.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index includes 85% of the free float-adjusted market capitalization in each industry group in emerging market countries. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.147367	2020-10-11 23:38:31.031328
EFII	Electronics for Imaging Inc.	Electronics for Imaging Inc is a digital printing innovation company. It provides digital printing services to printer manufacturers, commercial photo labs, sign shops, and billboard graphics providers.	{Technology,"Computer Systems","Computer Hardware"}	{CAJ,MMM,HPQ,XRX,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:02.161345	2020-10-11 23:38:31.025871
EFZ	ProShares Short MSCI EAFE	The investment seeks daily investment results, before fees and expenses, that correspond to the inverse (-1x) of the daily performance of the MSCI EAFE Index®.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index includes 85% of the free float-adjusted, market capitalization in each industry group in developed market countries, excluding the U.S. and Canada. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.179267	2020-10-11 23:38:31.023428
DWT	VelocityShares 3x Inverse Crude Oil ETNs linked to the S&P GSCI Crude Oil Index ER	The investment seeks to replicate, net of expenses, three times the opposite (inverse) of the S&P GSCI® Crude Oil Index ER. \n The index tracks a hypothetical position in the nearest-to-expiration NYMEX light sweet crude oil futures contract, which is rolled each month into the futures contract expiring in the next month. The value of the Index fluctuates with changes in the price of the relevant NYMEX light sweet crude oil futures contracts.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.941471	2020-10-11 23:38:31.031328
DXCM	DexCom Inc.	DexCom Inc designs, develops and commercializes continuous glucose monitoring systems for ambulatory use by people with diabetes and for use by healthcare providers in the hospital for the treatment of both diabetic and non-diabetic patients.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{QDEL,WST,ABT,BDX,JNJ,EW,MDT,TNDM,XLV}	Medical Diagnostics & Research	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:01.964155	2020-10-11 23:38:31.023428
DYLS	WisdomTree Dynamic Long/Short U.S. Equity Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree Dynamic Long/Short U.S. Equity Index (the "index").\n Under normal circumstances, at least 80% of the fund's total assets will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index includes long U.S. equity positions (the "Long Equity Basket") and, at times, short U.S. equity positions (the "Short Equity Basket"). The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:02.005443	2020-10-11 23:38:31.02423
EARN	Ellington Residential Mortgage REIT of Beneficial Interest	Ellington Residential Mortgage REIT acquires, invests in and manages residential mortgage- and real estate-related assets, with a primary focus on residential mortgage-backed securities through its wholly owned subsidiaries.	{"Real Estate","REIT - Residential",REITs}	{NYMT,WMC,EFC,MITT,ORC,TWO}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:02.028903	2020-10-11 23:38:31.028166
ECH	iShares Inc. MSCI Chile	The investment seeks to track the investment results of the MSCI Chile IMI 25/50 Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The underlying index is a free float-adjusted market capitalization index that is designed to measure broad-based equity market performance in Chile. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:02.067198	2020-10-11 23:38:31.024353
ECT	ECA Marcellus Trust I of Beneficial Interest	ECA Marcellus Trust I owns royalty interests in fourteen producing horizontal natural gas wells producing from the Marcellus Shale formation.	{Energy,"Oil & Gas Integrated","Oil & Gas - Integrated"}	{SBR,VOC,CHKR}	Oil & Gas - Integrated	Energy	New York Stock Exchange	\N	2020-10-11 23:12:02.083558	2020-10-11 23:38:31.031328
EDOM	WisdomTree Europe Domestic Economy Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree Europe Domestic Economy Index.\n Under normal circumstances, the fund will invest at least 80% of its total assets in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is designed with a goal of maximizing exposure to European companies with significant European revenue that may benefit from an economic recovery in Europe and increased buying power by Europeans. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:02.109534	2020-10-11 23:38:31.025871
EES	WisdomTree U.S. SmallCap Earnings Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree U.S. SmallCap Earnings Index.\n Under normal circumstances, at least 95% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a fundamentally weighted index that is comprised of earnings-generating companies within the small-capitalization segment of the U.S. stock market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.139796	2020-10-11 23:38:31.023428
EFC	Ellington Financial LLC representing Limitied Liability Company Interests no par valu	Ellington Financial LLC is a specialty finance company in United States. Its objective is to generate attractive, risk-adjusted total returns by making investments in residential mortgage-backed securities, corporate debt and other commercial debt.	{"Financial Services","Specialty Finance",Banks}	{ATAX,AI,WD,HASI,LADR}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.153796	2020-10-11 23:38:31.028166
EFO	ProShares Ultra MSCI EAFE	The investment seeks daily investment results, before fees and expenses, that correspond to two times (2x) the daily performance of the MSCI EAFE Index®.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index includes 85% of the free float-adjusted, market capitalization in each industry group in developed market countries, excluding the U.S. and Canada. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.16596	2020-10-11 23:38:31.021426
DXYN	The Dixie Group Inc.	The Dixie Group Inc manufactures and sells carpet and rugs to high-end residential and commercial customers. The company's brands include Fabrica, Masland Residential, Dixie Home, Masland Contract and Avant Contract.	{"Consumer Cyclical","Textile Manufacturing","Homebuilding & Construction"}	{MLHR,LOW,MHK,KNL,UFI,TILE}	Homebuilding & Construction	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:01.995003	2020-10-11 23:38:31.023428
DZZ	DB Gold Double Short ETN due February 15 2038	The investment seeks to replicate, net of expenses, twice of the inverse of the daily performance of the Deutsche Bank Liquid Commodity index - Optimum Yield Gold Excess Return.\n The index is intended to reflect changes in the market value of certain gold futures contracts and is comprised of a single unfunded gold futures contract.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.01153	2020-10-11 23:38:31.024353
EAE	Entergy Arkansas Inc. First Mortgage Bonds 4.75% Series due June 1 2063	Entergy Corporation is a Fortune 500 integrated energy Company engaged primarily in electric power production and retail distribution operations.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:02.026321	2020-10-11 23:38:31.025871
EBAYL	eBay Inc. 6.0% Notes Due 2056	eBay Inc is a commerce platform and provides online marketplace for sale of goods. Its marketplace platforms include its online marketplace located at www.ebay.com, its localized counterparts and the eBay mobile apps.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:02.036873	2020-10-11 23:38:31.023428
ECCB	Eagle Point Credit Company Inc. 7.75% Series B Term Preferred Stock due 2026	Eagle Point Credit Co Inc is a non-diversified, closed-end management investment company. The Company's investment objective is to generate high current income, with a secondary objective to generate capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.061237	2020-10-11 23:38:31.028166
ECOL	US Ecology Inc.	US Ecology Inc is active within the sphere of waste management in the United States. The company operates in two segments namely Environmental services and Field and industrial services. It generates a majority of revenue from the Environmental Services.	{Industrials,"Waste Management"}	{HCCI,CLH,WM,VHI,AWX,VTNR,PESI}	Waste Management	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:02.073281	2020-10-11 23:38:31.023007
ECYT	Endocyte Inc.	Endocyte Inc is a biopharmaceutical company. It is engaged in developing targeted therapies for the treatment of cancer and inflammatory diseases.	{Healthcare,Biotechnology}	{AMGN,NKTR,GE,JNJ,MRK,VSTM,ABBV,MDGL,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:02.084278	2020-10-11 23:38:31.028166
EDIT	Editas Medicine Inc.	Editas Medicine Inc is a genome editing company which is engaged in treating patients with genetically defined diseases by correcting disease-causing genes.	{Healthcare,Biotechnology}	{MNTA,MLNT,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:02.102272	2020-10-11 23:38:31.028166
EEI	Ecology and Environment Inc.	Ecology and Environment Inc is an environmental consulting company. It offers professional services to clients in industry including energy, natural resource management & restoration, green programs, hazardous material services & health sciences.	{Industrials,"Waste Management"}	{WLDN,HIL,QRHC}	Waste Management	Industrials	NASDAQ Global Market	\N	2020-10-11 23:12:02.1279	2020-10-11 23:38:31.023428
EEMX	SPDR MSCI Emerging Markets Fuel Reserves Free	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the MSCI Emerging Markets ex Fossil Fuels Index.\n Under normal market conditions, the fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index and in depositary receipts based on securities comprising the index. The index is designed to measure the performance of companies in the MSCI Emerging Markets Index that are "fossil fuel reserves free," which are defined as companies that do not own fossil fuel reserves. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.138907	2020-10-11 23:38:31.021426
EFAX	SPDR MSCI EAFE Fossil Fuel Reserves Free	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the MSCI EAFE ex Fossil Fuels Index.\n Under normal market conditions, the fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index and in depositary receipts based on securities comprising the index. The index is designed to measure the performance of companies in the MSCI EAFE Index that are "fossil fuel reserves free," which are defined as companies that do not own fossil fuel reserves. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.152595	2020-10-11 23:38:31.028166
EFR	Eaton Vance Senior Floating-Rate Fund of Beneficial Interest	Eaton Vance Senior Floating Rate Trust is a United States based diversified, closed-end management investment company. Its primary investment objective is to provide a high level of current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.167881	2020-10-11 23:38:31.02582
EA	Electronic Arts Inc.	Electronic Arts Inc operates in the technology sector. It develops, markets, publishes and distributes video game software and content of various platforms.	{Technology,"Electronic Gaming & Multimedia","Application Software"}	{MSFT,TTWO,GLUU,ATVI,SNE,DIS,ZNGA,PTE,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:02.019028	2020-10-11 23:38:31.021426
EBTC	Enterprise Bancorp Inc	Enterprise Bancorp Inc is an American banking service provider. It specializes in savings and cooperative banking products such as commercial and consumer loan products, deposit products and cash management services.	{"Financial Services","Savings & Cooperative Banks",Banks}	{CAC,UNB,BPFH,CNBKA,INDB,WASH,BRKL,WBS,NBN,SAL}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:02.056694	2020-10-11 23:38:31.028166
ECOM	ChannelAdvisor Corporation	ChannelAdvisor Corp is engaged in providing software-as-a-service solutions that enables retailers and manufacturers to integrate, manage and optimize their merchandise sales across hundreds of online channels.	{Technology,"Software - Application","Application Software"}	{AMSWA,IBM,CRM,DTRM,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:02.077127	2020-10-11 23:38:31.021426
EDBI	Legg Mason Emerging Markets Diversified Core ETF	The investment seeks to track the investment results of the QS DBI Emerging Markets Diversified Index.\n The fund will invest at least 80% of its net assets, plus borrowings for investment purposes, if any, in securities that compose the underlying index. The underlying index is composed of publicly traded equity securities in emerging markets.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.089149	2020-10-11 23:38:31.021426
EDI	Stone Harbor Emerging Markets Total Income Fund of Beneficial Interests	Stone Harbor Emg Mkts Total Income Fund is a non-diversified, closed-end management investment company. Its investment objective is to maximize total return, which consists of income & capital appreciation from investments in emerging markets securities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.10221	2020-10-11 23:38:31.024353
EE	El Pasoectric Company	El Paso Electric Co is a public utility company which is engaged in the generation, transmission and distribution of electricity in in the Southwestern U.S.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{ETR,EIX,OGE,AEP,EXC,D,PNM,NRG,NEE,XEL,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:02.11839	2020-10-11 23:38:31.025871
EEMV	iShares Edge MSCI Min Vol Emerging Markets	The investment seeks to track the investment results of the MSCI Emerging Markets Minimum Volatility (USD) Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index measures the performance of equity securities in global emerging markets that, in the aggregate, have lower volatility relative to the broader global emerging markets.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:02.135655	2020-10-11 23:38:31.023007
EFBI	Eagle Financial Bancorp Inc.	Eagle Financial Bancorp Inc takes deposits from the general public and investing those deposits in one- to four-family residential real estate loans, commercial real estate and land loans, construction loans and home equity loans and lines of credit.	{"Financial Services","Savings & Cooperative Banks",Banks}	{}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:02.153881	2020-10-11 23:38:31.021426
EFSC	Enterprise Financial Services Corporation	Enterprise Financial Services Corporation is a financial holding company. It offers banking and wealth management services to individuals and business customers located in the St. Louis, Kansas City and Phoenix metropolitan markets.	{"Financial Services","Banks - Regional - US",Banks}	{UMBF,ATLO,WTBA,GFED,CBSH,GSBC,LARK,QCRH,HWBK,MOFG}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:02.169199	2020-10-11 23:38:31.028166
EGL	Engility Holdings Inc.	Engility Holdings Inc is engaged in deploying skilled workforce to the U.S. government. It provides a range of engineering, technical, analytical, advisory, training, logistics and support services.	{Industrials,"Staffing & Outsourcing Services","Employment Services"}	{VSEC,KEYW,LMT,GD,NOC,RTN,ACM,SAIC,LLL}	Employment Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:02.190921	2020-10-11 23:38:31.02582
EIA	Eaton Vance California Municipal Bond Fund II of Beneficial Interest $.01 par value	Eaton Vance California Municipal Bond Fund II is a closed-end investment company. Its objective is to provide current income exempt from regular federal income tax, including alternative minimum tax and in state-specific funds, tax in its specified state.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:02.213107	2020-10-11 23:38:31.028166
EIX	Edison International	Edison International is engaged in the utilities sector. The company's business mainly involves generation and distribution of electric power. It also invests in infrastructure and energy assets, including renewable energy.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{PCG,FE,SRE,AEP,HE,PNW,AVA,PNM,POR,XEL,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:02.226359	2020-10-11 23:38:31.02423
ELLO	Ellomay Capital Ltd (Israel)	Ellomay Capital Ltd is engaged in the business of energy and infrastructure and its operations include production of renewable and clean energy.	{Utilities,"Utilities - Independent Power Producers"}	{BWXT,CREG,XLU}	Utilities - Independent Power Producers	Utilities	NYSE American	\N	2020-10-11 23:12:02.239146	2020-10-11 23:38:31.021426
FITBI	Fifth Third Bancorp Depositary Share repstg 1/1000th Ownership Interest Perp Pfd Series I	Fifth Third Bancorp provides banking products and services. The company offers checking, savings and money market accounts and credit products. Its segments are Commercial Banking, Branch Banking, Consumer Lending and Wealth and Asset Management.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:02.992914	2020-10-11 23:38:31.02582
ECCY	Eagle Point Credit Company Inc. 6.75% Notes due 2027		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:02.062332	2020-10-11 23:38:31.031328
ECON	Columbia Emerging Markets Consumer	The investment seeks investment results that correspond (before fees and expenses) to the price and yield performance of the Dow Jones Emerging Markets Consumer Titans 30TM Index.\n The fund will invest at least 80% of its net assets in securities of Emerging Markets Consumer companies included in the index and the advisor generally expects to be substantially invested at such times, with at least 95% of its net assets invested in these securities. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.07992	2020-10-11 23:38:31.023428
EDD	Morgan Stanley Emerging Markets Domestic Debt Fund Inc.	Morgan Stanley Emerging Markets Domestic is a closed-end management investment company that seeks to achieve its investment objectives by investing, under normal circumstances, at least 80% of its managed assets in emerging markets domestic debt.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.092249	2020-10-11 23:38:31.023428
EDUC	Educational Development Corporation	Educational Development Corp is the United States trade publisher of the line of educational children's books produced in the United Kingdom by Usborne Publishing Limited. It has two reportable segments: EDC Publishing and UBAM .	{"Consumer Cyclical",Publishing}	{}	Publishing	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:02.113234	2020-10-11 23:38:31.02423
EFAD	ProShares Trust MSCI EAFE Dividend Growers	The investment seeks investment results, before fees and expenses, that track the performance of the MSCI EAFE Dividend Masters Index (the "index").\n The index, constructed and maintained by MSCI, targets companies that are currently members of the MSCI EAFE Index ("MSCI EAFE") and have increased dividend payments each year for at least 10 years. The index contains a minimum of 40 stocks, which are equally weighted. Under normal circumstances, the fund will invest at least 80% of its total assets in component securities.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:02.148656	2020-10-11 23:38:31.025871
EFFE	Global X JPMorgan Efficiente Index	The investment seeks investment results that correspond generally to the price and yield performance, before fees and expenses, of the JPMorgan ETF Efficiente 10 TR Series X Index.\n The fund invests at least 80% of its total assets in the securities of the underlying index. The underlying index tracks the total return performance of a portfolio of eleven exchange-traded funds (each, an "ETF") and two exchange-traded products (each an "ETP") (each ETP and ETF a "Constituent" and together the "Constituents"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.15696	2020-10-11 23:38:31.031328
EFV	iShares MSCI EAFE Value	The investment seeks to track the investment results of the MSCI EAFE Value Index composed of developed market equities, excluding the U.S. and Canada, that exhibit value characteristics.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index is a subset of the MSCI EAFE Index. Constituents of the underlying index include securities from Europe, Australasia and the Far East.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:02.176899	2020-10-11 23:38:31.031328
EGPT	VanEck Vectors Egypt Index	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the MVIS® Egypt Index.\n The fund will normally invest at least 80% of its total assets in securities that comprise the fund's benchmark index. The index includes securities of Egyptian companies. A company is generally considered to be an Egyptian company if it is incorporated in Egypt or is incorporated outside Egypt but has at least 50% of its revenues/related assets in Egypt. Such companies may include micro-, small- and medium-capitalization companies. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.203783	2020-10-11 23:38:31.02423
EHTH	eHealth Inc.	eHealth Inc is a health insurance exchange where individuals, families and small businesses compare health insurance products from insurers side-by-side and purchase and enroll in coverage through its website or customer care centers.	{"Financial Services","Insurance Brokers","Brokers & Exchanges"}	{BRO,AJG,AET,AON,HIIQ,WLTW,FANH}	Brokers & Exchanges	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:02.211737	2020-10-11 23:38:31.023007
EIP	Eaton Vance Pennsylvania Municipal Bond Fund of Befeficial Interest $.01 par value	Eaton Vance Pennsylvania Municipal Bond Fund is a closed-end investment company. Its objective is to provide current income exempt from regular federal income tax, including alternative minimum tax, & in state specific funds, tax in its specified state.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:02.221331	2020-10-11 23:38:31.021426
EL	Estee Lauder Companies Inc. (The)	The Estee Lauder Companies Inc is a beauty products manufacturer providing Skin care, Makeup, Fragrance and Hair care services.	{"Consumer Defensive","Household & Personal Products","Consumer Packaged Goods"}	{IPAR,NUS,UL,AVP,PG,REV,COTY}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:02.230442	2020-10-11 23:38:31.024353
ELSE	Electro-Sensors Inc.	Electro-Sensors Inc is engaged in manufacturing and selling variety of monitoring systems that measure actual machine production and operation rates, as well as systems that regulate the speed of related machines in production processes.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{FLIR,CGNX,HURC,SMIT,AME,DHR,EMR,VPG,XLK}	Computer Hardware	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:02.24189	2020-10-11 23:38:31.023428
ECR	Eclipse Resources Corporation	Eclipse Resources Corp is an independent exploration and production company. The company is engaged in the acquisition and development of oil and natural gas properties in the Appalachian Basin.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{COG,RRC,D,EQT,AR}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:02.084052	2020-10-11 23:38:31.028166
EEMA	iShares MSCI Emerging Markets Asia Index Fund	The investment seeks to track the investment results of the MSCI Emerging Markets (EM) Asia Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The underlying index is designed to measure equity market performance in the emerging market countries of Asia. It may include large-, mid- or small-capitalization companies.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.130598	2020-10-11 23:38:31.031328
EFAS	Global X MSCI SuperDividend EAFE ETF	The investment seeks to provide investment results that correspond generally to the price and yield performance, before fees and expenses, of the MSCI EAFE Top 50 Dividend Index ("underlying index").\n The fund invests at least 80% of its total assets in the securities of the underlying index. The underlying index tracks the performance of 50 equally-weighted companies that rank among the highest dividend yielding equity securities in Europe, Australasia and the Far East, as defined by MSCI, the provider of the underlying index ("index provider"). The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.149109	2020-10-11 23:38:31.023428
EFOI	Energy Focus Inc.	Energy Focus Inc along with its subsidiaries is engaged in the design, development, manufacturing, marketing, and installation of lighting systems and solutions. Its product offerings include military maritime, globe lights, berth lights and others.	{"Consumer Cyclical","Home Furnishings & Fixtures","Manufacturing - Apparel & Furniture"}	{LYTS,OESX,RVLT}	Manufacturing - Apparel & Furniture	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:02.165671	2020-10-11 23:38:31.023007
EGAN	eGain Corporation	eGain Corp provides cloud-based and on-site customer engagement software solutions. The Company is engaged in development, license, implementation and support of its customer service infrastructure software solutions.	{Technology,"Software - Application","Application Software"}	{CSCO,ULTI,PRGS,LPSN,VRNT,CRM,SREV,ORCL,FIVN,XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:02.183653	2020-10-11 23:38:31.021426
EHI	Western Asset Global High Income Fund Inc	Western Asset Global High Inc Fd Inc is a non-diversified, closed-end management investment company. Its investment objective is high current income and total return by investing in global portfolio of securities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.204833	2020-10-11 23:38:31.024353
EIM	Eaton Vance Municipal Bond Fund of Beneficial Interest $.01 par value	Eaton Vance Municipal Bond Fund is a non-diversified closed-end management investment company. It consists of Municipal bond fund, California municipal bond fund, and New York municipal bond fund.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:02.219054	2020-10-11 23:38:31.028166
ELGX	Endologix Inc.	Endologix Inc develops, manufactures, markets and sells medical devices for the treatment of aortic disorders. Its products are intended for the treatment of abdominal aortic aneurysms.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{HOLX,MMSI,ANGO,LMAT,CSII,BAX,BSX,MDT,XLV}	Medical Instruments & Equipment	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:02.236467	2020-10-11 23:38:31.028166
ELU	Entergy Louisiana Inc. First Mortgage Bonds 4.70% Series due June 1 2063		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:02.249053	2020-10-11 23:38:31.024353
EMCG	WisdomTree Emerging Markets Consumer Growth Fund	The investment seeks income and capital appreciation.\n The fund seeks to achieve its investment objective by investing primarily in consumer growth stocks in emerging markets that provide the best combined rank of growth, quality, and valuation characteristics. The adviser, using a disciplined model-based process focused on a long-term approach to investing, seeks to identify consumer stocks that benefit from long-term growth in emerging market economies. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.259122	2020-10-11 23:38:31.021426
EMHY	iShares Emerging Markets High Yield Bond	The investment seeks to track the investment results of the Morningstar® Emerging Markets High Yield Bond IndexSM.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents, including shares of money market funds advised by BFA or its affiliates. The index tracks the performance of the below-investment-grade U.S. dollar-denominated emerging market sovereign and corporate high yield bond market. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:02.270703	2020-10-11 23:38:31.021426
EMKR	EMCORE Corporation	EMCORE Corp is a provider of compound semiconductor-based components and subsystems for the broadband, fibre optic, satellite and terrestrial solar power markets.	{Technology,Semiconductors}	{FNSR,OCLR,BA,BDC,NPTN,AAOI,VIAV,ARRS,XLK}	Semiconductors	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:02.280259	2020-10-11 23:38:31.025871
FIVN	Five9 Inc.	Five9 Inc provides cloud software for contact centers. The Company is engaged in providing research, development and client support services for contact centers.	{Technology,"Software - Infrastructure","Application Software"}	{CTXS,NICE,LPSN,CRM,SREV,EGAN,ZEN,XLK}	Application Software	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:03.000426	2020-10-11 23:38:31.024353
ED	Consolidated Edison Inc.	Consolidated Edison Inc functions in the utilities market. It owns energy infrastructure projects and its main business operations are its regulated electric, gas and steam delivery businesses. The CECONY segment generates maximum revenue.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{NI,FE,CMS,PEG,EXC,D,SCG,UTL,ES,WEC,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:02.086553	2020-10-11 23:38:31.025871
EDGE	Edge Therapeutics Inc.	Edge Therapeutics Inc is a United States based company engaged in the research and development of medical therapies. Its products address serious unmet neurological disorders.	{Healthcare,Biotechnology}	{PDLI,PIRS,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:02.099344	2020-10-11 23:38:31.028166
EFA	iShares MSCI EAFE	The investment seeks to track the investment results of the MSCI EAFE Index composed of large- and mid-capitalization developed market equities, excluding the U.S. and Canada.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The index measures the equity market performance of developed markets outside of the U.S. and Canada. The underlying index may include large- or mid-capitalization companies.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.14751	2020-10-11 23:38:31.024353
EFL	Eaton Vance Floating-Rate 2022 Target Term Trust of Beneficial Interest	EATON VANCE FLOAT-RATE 2022 TARGET TERM is a newly organized, diversified, closed-end management investment company.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.163226	2020-10-11 23:38:31.024353
EGI	Entree Resources Ltd.	Entree Resources Ltd is a Canadian mining company with a unique carried 20% or 30% joint venture interest in an integral part of the Oyu Tolgoi copper-gold mining project in Mongolia.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{RIO,TRQ}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:02.188202	2020-10-11 23:38:31.028166
EGP	EastGroup Properties Inc.	EastGroup Properties Inc is an equity real estate investment trust. It is engaged in the acquisition, development, and operation of industrial properties in major urban areas within the Sunbelt region of the United States.	{"Real Estate","REIT - Industrial",REITs}	{FR,MNR,DLR,COR,STAG,PLD,REXR,QTS}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:02.200341	2020-10-11 23:38:31.031328
EIGI	Endurance International Group Holdings Inc.	Endurance International Group Holdings Inc is a provider of cloud-based platform solutions. It offers domains, website builders, web hosting, email, security, storage, site backup, SEO, and SEM, social media services, website analytics, among others.	{Technology,"Software - Application","Application Software"}	{VRSN,BCOV,GOOGL,INAP,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:02.21525	2020-10-11 23:38:31.02582
ELD	WisdomTree Emerging Markets Local Debt Fund	The investment seeks a high level of total return consisting of both income and capital appreciation.\n The fund seeks to achieve its investment objective through investment in bonds and other debt instruments denominated in the local currencies of emerging market countries. Under normal circumstances, it will invest at least 80% of its net assets, plus the amount of any borrowings for investment purposes, in Local Debt. The Advisor attempts to maintain an aggregate portfolio duration of between two and ten years under normal market conditions. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.231631	2020-10-11 23:38:31.031328
ELS	Equity Lifestyle Properties Inc.	Equity Lifestyle Properties Inc is a real estate company. It owns and operates lifestyle-oriented properties consisting of manufactured home communities and recreational vehicle resorts.	{"Real Estate","REIT - Residential",REITs}	{CPT,SUI,ESS,ACC,UMH,IRT}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:02.241729	2020-10-11 23:38:31.02423
EMCF	Emclaire Financial Corp	Emclaire Financial Corp is a financial holding Company. It offers retail and commercial financial products and services to the customers in western Pennsylvania through its wholly owned subsidiary bank, the Farmers National Bank of Emlenton.	{"Financial Services","Banks - Regional - US",Banks}	{WSFS,DNBF,FULT,STBA,SVBI,ASRV,BCBP,FNB,ACNB,PFIS}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:02.254615	2020-10-11 23:38:31.02423
EME	EMCOR Group Inc.	EMCOR Group Inc an electrical and mechanical construction and facilities services firms. It services include installation of electrical distribution systems, voice and data communication systems, and environmental controls within buildings.	{Industrials,"Engineering & Construction"}	{FIX,PWR,DY,UTX,FLR,TPC,IESC}	Engineering & Construction	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:02.264273	2020-10-11 23:38:31.028166
EMJ	Eaton Vance New Jersey Municipal Bond Fund Common Shres of Beneficial Interest $.01 par value	Eaton Vance New Jersey Municipal Bond Fund is a closed-end investment company. Its objective is to provide current income exempt from regular federal income tax, including alternative minimum tax, and, in state specific funds, tax in its specified state.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:02.279329	2020-10-11 23:38:31.028166
EMR	Emerson Electric Company	Emerson Electric Co is engaged in designing and supplying products and technology, and delivering engineering services and solutions in industrial, commercial and consumer markets.	{Industrials,"Diversified Industrials","Industrial Products"}	{MSFT,AME,DHR,GE,IBM,HON,ROP,ABB,COMM}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:02.29173	2020-10-11 23:38:31.028166
EDZ	Direxion Emerging Markets Bear 3X Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the inverse of the daily performance of the MSCI Emerging Markets IndexSM.\n The fund invests in swap agreements, futures contracts, short positions or other financial instruments that, in combination, provide inverse or short leveraged exposure to the index equal to at least 80% of the fund's net assets (plus borrowing for investment purposes). The index is a free float-adjusted market capitalization weighted index that is designed to represent the performance of large- and mid-capitalizations securities across the 24 emerging market countries. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.116121	2020-10-11 23:38:31.024353
EEM	iShares MSCI Emerging Index Fund	The investment seeks to track the investment results of the MSCI Emerging Markets Index.\n The fund generally invests at least 90% of its assets in the securities of its underlying index and in depositary receipts representing securities in its underlying index. The index is designed to measure equity market performance in the global emerging markets. It may invest the remainder of its assets in other securities, including securities not in the underlying index, but which BFA believes will help the fund track the index, and in other investments, including futures contracts, options on futures contracts, other types of options and swaps related to its index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.128469	2020-10-11 23:38:31.028166
EEQ	Enbridge Energy Management LLC Shares repstg limited liability company interests	Enbridge Energy Management LLC is a publicly traded limited liability company that manages and controls the business and affairs of Enbridge Energy Partners, L.P., a master limited partnership that engages in transportation and distribution of energy.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{GEL,PAA,EEP,MMP,HEP,TLP,BKEP,SXE,PSXP,PAGP}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:02.138881	2020-10-11 23:38:31.028166
EFAV	iShares Edge MSCI Min Vol EAFE	The investment seeks the investment results of the MSCI EAFE Minimum Volatility (USD) Index composed of developed market equities that, in the aggregate, have lower volatility characteristics relative to the broader developed equity markets, excluding the U.S. and Canada.\n The fund generally will invest at least 90% of its assets in the component securities of the index and in investments that have economic characteristics that are substantially identical to the component securities of the index. The index measures the performance of international equity securities that in the aggregate have lower volatility relative to the MSCI EAFE Index.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:02.15052	2020-10-11 23:38:31.023007
EFNL	iShares Inc MSCI Finland	The investment seeks to track the investment results of the MSCI Finland IMI 25/50 Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is designed to measure broad-based equity market performance in Finland. A capping methodology is applied that limits the weight of any single component to a maximum of 25% of the underlying index. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:02.164532	2020-10-11 23:38:31.023428
EFX	Equifax Inc.	Equifax Inc provides information solutions and human resources business process outsourcing services for businesses, governments and consumers. It also provides database management, fraud detection, marketing, business credit, and analytical services.	{Industrials,"Business Services"}	{ADP,FISV,PAYX,INTX,FICO,MCO,DNB,FIS,CLGX}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:02.176767	2020-10-11 23:38:31.023007
EGLE	Eagle Bulk Shipping Inc.	Eagle Bulk Shipping Inc owns fleets of Supramax/Ultramax dry bulk vessels. The company transports major and minor bulk cargoes, including coal, grain, ore, petcoke, cement and fertilizer with its 41 owned and operated fleet of Handymax dry bulk vessels.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{DSX,NM,SHIP}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:02.193784	2020-10-11 23:38:31.023007
EHC	Encompass Health Corporation	Encompass Health Corp, formerly Healthsouth Corp provides post-acute healthcare services in the US through a network of inpatient rehabilitation hospitals, home health and hospice agencies. It operates Inpatient Rehabilitation and Home Health and Hospice.	{Healthcare,"Medical Care","Health Care Providers"}	{XLV}	Health Care Providers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:02.204262	2020-10-11 23:38:31.02582
EIG	Employers Holdings Inc	Employers Holdings Inc is engaged in the commercial property and casualty insurance industry. The company provides workers' compensation insurance and related services to select, small American businesses.	{"Financial Services","Insurance - Specialty"}	{AMSF,HIG,AFL,MKL,TRV,HIIQ,ANTM}	Insurance - Specialty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.214087	2020-10-11 23:38:31.02423
EIV	Eaton Vance Municipal Bond Fund II of Beneficial Interest $.01 Par Value	Eaton Vance Municipal Bond Fund II is a closed-end investment company. Its objective is to provide current income exempt from regular federal income tax, including alternative minimum tax, and, in state specific funds, tax in its specified state.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:02.226326	2020-10-11 23:38:31.023428
ELJ	Entergy Louisiana Inc. First Mortgage Bonds 5.25% Series due July 1 2052		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:02.238684	2020-10-11 23:38:31.023007
EMAG	VanEck Vectors Emerging Markets Aggregate Bond	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of MVIS® EM Aggregate Bond Index (the "EM Aggregate Bond Index").\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The index is comprised of emerging market sovereign bonds and corporate bonds denominated in U.S. dollars, Euros or local emerging market currencies. The index includes both investment grade and below investment grade rated securities. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.251967	2020-10-11 23:38:31.025871
EEFT	Euronet Worldwide Inc.	Euronet Worldwide Inc is an electronic payments provider. It offers payment and transaction processing and distribution solutions to financial institutional, retailers, service providers and individual consumers.	{Industrials,"Business Services"}	{EBAY,UEPS,GPN,FIS,WU,V,MGI,CATM}	Business Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:02.121949	2020-10-11 23:38:31.021426
EEMS	Ishares MSCI Emerging Markets Small Cap Index Fund	The investment seeks to track the investment results of the MSCI Emerging Markets Small Cap Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is designed to measure the performance of equity securities of small-capitalization companies whose market capitalization represents the bottom 14% of companies in emerging market countries, as measured by market capitalization.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.13517	2020-10-11 23:38:31.025871
EFF	Eaton vance Floating-Rate Income Plus Fund of Beneficial Interest	EV Floating-Rate Income Plus Fund is a diversified, closed-end management investment company. The Fund's investment objective is total return, with an emphasis on income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.156066	2020-10-11 23:38:31.02582
EFU	ProShares UltraShort MSCI EAFE	The investment seeks daily investment results, before fees and expenses, that correspond to two times the inverse (-2x) of the daily performance of the MSCI EAFE Index®.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index includes 85% of the free float-adjusted, market capitalization in each industry group in developed market countries, excluding the U.S. and Canada. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.170185	2020-10-11 23:38:31.025871
EGIF	Eagle Growth and Income Opportunities Fund of Beneficial Interest	Eagle Growth and Income Opportunities Fund is a non-diversified, closed-end management investment company. The company's investment objective is to provide total return through a combination of current income and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.188651	2020-10-11 23:38:31.031328
EGRX	Eagle Pharmaceuticals Inc.	Eagle Pharmaceuticals Inc is a specialty pharmaceutical company. The company develops and commercializes injectable products in the areas of critical care, orphan diseases, and oncology.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{AMGN,MDCO,GSK,NVS,BMY,JNJ,LLY,TEVA,MDGL,XLV}	Drug Manufacturers	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:02.204672	2020-10-11 23:38:31.025871
EIRL	iShares Trust MSCI Ireland	The investment seeks to track the investment results of the MSCI All Ireland Capped Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is a free float-adjusted market capitalization-weighted index that aims to reflect the performance of Irish equities securities of companies. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.221905	2020-10-11 23:38:31.028166
ELLI	Ellie Mae Inc.	Ellie Mae Inc is a provider of on-demand software solutions and services for the residential mortgage industry. Its mortgage management solutions help streamline and automate the process of originating and funding new mortgage loans.	{Technology,"Software - Application","Application Software"}	{VRSK,SSNC,FICO,CLGX,MITK,ZG,XRX,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:02.238964	2020-10-11 23:38:31.031328
EMAN	eMagin Corporation	eMagin Corp is a manufacturer of microdisplays using OLED technology. The Company designs, develops, manufactures, and markets OLED on silicon microdisplays, virtual imaging products and related products.	{Technology,"Semiconductor Equipment & Materials",Semiconductors}	{RSYS,IVAC,KOPN,QCOM,HON,COL,LPL,MVIS,LLL,SMCI,XLK}	Semiconductors	Technology	NYSE American	\N	2020-10-11 23:12:02.252024	2020-10-11 23:38:31.024353
EMGF	iShares Edge MSCI Multifactor Emerging Markets	The investment seeks to track the investment results of the MSCI Emerging Markets Diversified Multiple-Factor Index.\n The underlying index is designed to select equity securities from the MSCI Emerging Markets Index that have high exposure to four investment style factors: value, quality, momentum and low size, while maintaining a level of risk similar to that of the parent index. The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. It is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:02.269084	2020-10-11 23:38:31.025871
EMN	Eastman Chemical Company	Eastman Chemical Co is a advanced materials and specialty additives company that is engaged in producing and selling a portfolio of chemicals, plastics and fibers.	{"Basic Materials",Chemicals}	{SSL,XOM,GRA,FMC,POL,MMM,WLK,CE,ASH,DWDP}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:02.285694	2020-10-11 23:38:31.028166
ENB	Enbridge Inc	Enbridge Inc serves the oil & gas industry. Its key activity involves gathering and transportation of crude oil and natural gas.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{IMO,TRP,APC,EPD,BP,EEP,HES,EEQ,PBA}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:02.300511	2020-10-11 23:38:31.02423
EGHT	8x8 Inc	8x8 Inc is a technology company. It provides cloud-based, enterprise-class software solutions that transform the way businesses communicate and collaborate globally. The company operates through two segments: America and Europe.	{"Communication Services","Telecom Services"}	{CNSL,CTL,VZ,T,VG,IDT}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:02.185616	2020-10-11 23:38:31.025871
EGN	Energen Corporation	Energen Corp is a domestic oil and gas exploration and production company. Its reserves are in the Permian basin of west Texas and San Juan Basin in New Mexico.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{PXD,OKE,SRE,MXC,NFG,WMB,LPI,REI}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:02.196494	2020-10-11 23:38:31.028166
EHT	Eaton Vance High Income 2021 Target Term Trust of Beneficial Interest	Eaton Vance High Income 2021 Target Term is a closed-end management investment company. Its investment objectives is high current income and to return $9.85 per share, the original net asset value per common share before deducting offering costs.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.211279	2020-10-11 23:38:31.023428
EIO	Eaton Vance Ohio Municipal Bond Fundd of Befeficial Interest $.01 par value	Eaton Vance Ohio Municipal Bond Fund is a closed-end investment company. Its objective is to provide current income exempt from regular federal income tax, including alternative minimum tax, and, in state specific funds, tax in its specified state.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:02.220395	2020-10-11 23:38:31.031328
ELC	Entergy Louisiana Inc. Collateral Trust Mortgage Bonds 4.875 % Series due September 1 2066		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:02.231395	2020-10-11 23:38:31.025871
ELTK	Eltek Ltd.	Eltek Ltd manufactures and supplies technologically custom made circuitry solutions for use in sophisticated and compact electronic products. It provides specialized services and solutions in Israel, Europe, North America and Asia.	{Technology,"Contract Manufacturers","Computer Hardware"}	{FLEX,SGMA,ORBK,TTMI,SMTX,TDY,SANM,XLK}	Computer Hardware	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:02.245572	2020-10-11 23:38:31.028166
EMCI	EMC Insurance Group Inc.	EMC Insurance Group Inc is a non-life insurance service provider. Its key product is property and casualty insurance. It is also involved in the reinsurance business.	{"Financial Services","Insurance - Property & Casualty"}	{Y,WRB,MKL,RLI,TRV,WTM,UFCS}	Insurance - Property & Casualty	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:02.258687	2020-10-11 23:38:31.023428
EMIH	Xtrackers Emerging Markets Bond - Interest Rate Hedged	The investment seeks investment results that correspond generally to the performance, before fees and expenses, of the Solactive Emerging Markets Bond - Interest Rate Hedged Index.\n The fund will invest at least 80% of its total assets, but typically far more, in instruments that comprise the underlying index. The index is comprised of (a) long positions in U.S. dollar-denominated government debt issued by emerging market countries, and (b) short positions in U.S. Treasury notes or bonds of, in aggregate, approximate equivalent duration to the emerging markets sovereign debt. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:02.27663	2020-10-11 23:38:31.02582
EMP	Entergy Mississippi Inc. First Mortgage Bonds 4.90% Series Due October 1 2066		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:02.288884	2020-10-11 23:38:31.023428
ENBL	Enable Midstream Partners LP representing limited partner interests	Enable Midstream Partners LP owns, operates, and develops natural gas and crude oil infrastructure assets in the United States. The company gathers, process, transports, and stores natural gas.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{OGE,XOM,DVN,EEP,CNP,EEQ,KMI}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:02.302073	2020-10-11 23:38:31.031328
ENSG	The Ensign Group Inc.	Ensign Group Inc offers skilled nursing, assisted and independent living, home health and hospice and other ancillary services. The Transitional and Skilled Services generates maximum revenue for the company.	{Healthcare,"Long-Term Care Facilities","Health Care Providers"}	{NHC,CSU,DVCR,GEN,FVE,RHE,XLV}	Health Care Providers	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:02.320114	2020-10-11 23:38:31.024353
ENZ	Enzo Biochem Inc. ($0.01 Par Value)	Enzo Biochem Inc is engaged in research, development, manufacturing and marketing of health care products, platforms and services. The Company's business segments are Enzo Clinical Labs, Enzo Life Sciences, and Enzo Therapeutics.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{PMD,RDNT,DGX,LH,NEO,QGEN,XLV}	Medical Diagnostics & Research	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:02.33681	2020-10-11 23:38:31.02423
EPC	Edgewell Personal Care Company	Edgewell Personal Care Co is a manufacturer and marketer of personal care products in the wet shave, sun and skin care, feminine care and infant care categories. It has a portfolio of over 25 brands and a broad global footprint in more than 50 countries.	{"Consumer Defensive","Household & Personal Products","Consumer Packaged Goods"}	{CLX,KMB,ENS,REV}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:02.355634	2020-10-11 23:38:31.024353
EPRF	Innovator S&P Investment Grade Preferred	The investment seeks investment results that generally correspond (before fees and expenses) to the price and yield of the S&P U.S. High Quality Preferred Stock Index.\n The fund generally will invest at least 90% of its total assets in U.S.-listed preferred stocks that comprise the index. The index is designed to provide exposure to U.S.-listed preferred stocks that meet the minimum size, liquidity, type of issuance, and quality criteria. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:02.369092	2020-10-11 23:38:31.028166
EGOV	NIC Inc.	NIC Inc provides eGovernment services, which helps government use the Internet to reduce internal costs, increase efficiencies and provides services to businesses and citizens.	{Technology,"Software - Application","Application Software"}	{MSFT,CTSH,ACIW,CACI,IBM,WYY,ACN,UIS,SYNC,ORCL,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:02.196561	2020-10-11 23:38:31.023428
EHIC	eHi Car Services Limited American Depositary Shares	eHi Car Services Ltd provides one-stop car rentals and car services to individuals and corporate and institutional clients. It offers self-drive car rentals to both individual customers and corporate clients.	{Industrials,"Rental & Leasing Services","Consulting & Outsourcing"}	{MA,CAR,HTZ}	Consulting & Outsourcing	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:02.209511	2020-10-11 23:38:31.021426
EKSO	Ekso Bionics Holdings Inc.	Ekso Bionics Holdings Inc designs, develops and sells wearable bionic human exoskeletons that have applications in healthcare, industrial, military, and consumer markets.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{IART,RVP,LENS,XLV}	Medical Instruments & Equipment	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:02.228669	2020-10-11 23:38:31.02582
ELP	Companhia Paranaense de Energia (COPEL)	Energy Company of Parana generates, transmits, distributes, and sells electricity and piped gas in the Brazilian State of Parana. It also provides telecommunication and other services to industrial customers, along with others.	{Utilities,"Utilities - Diversified","Utilities - Regulated"}	{ENIA,KEP,EBR.B,CIG,GGB,AES,PBR,CPL,EDN,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:02.239704	2020-10-11 23:38:31.025871
EMB	iShares J.P. Morgan USD Emerging Markets Bond ETF	The investment seeks to track the investment results of the J.P. Morgan EMBI® Global Core Index composed of U.S. dollar-denominated, emerging market bonds.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index. The index is a broad, diverse U.S. dollar-denominated emerging markets debt benchmark that tracks the total return of actively traded external debt instruments in emerging market countries. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.252231	2020-10-11 23:38:31.02582
EMES	Emerge Energy Services LP representing Limited Partner Interests	Emerge Energy Services LP is a diversified energy services company engaged in the ownership, operation, and acquisition and development of an energy service assets.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services","Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{SLB,MUR,UNP,GLP,CZZ,BPL,HCLP,BHGE}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:02.265095	2020-10-11 23:38:31.024353
EMLC	VanEck Vectors J.P. Morgan EM Local Currency Bond	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the J.P. Morgan GBI-EM Global Core Index.\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The index is comprised of bonds issued by emerging market governments and denominated in the local currency of the issuer. It may concentrate its investments in a particular industry or group of industries to the extent that the index concentrates in an industry or group of industries. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.281726	2020-10-11 23:38:31.024353
EMX	EMX Royalty Corporation (Canada)	EMX Royalty Corp and its subsidiaries operates as a royalty and prospect generator engaged in the exploring for, and generating royalties from, metals and minerals properties.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:02.295083	2020-10-11 23:38:31.025871
ENG	ENGlobal Corporation	ENGlobal Corp is a provider of engineering and other professional project services related to design, fabrication, procurement, maintenance, environmental and other governmental compliance and construction management.	{Industrials,"Engineering & Construction"}	{MYRG,JEC,HUN,ACM,TISI,IESC}	Engineering & Construction	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:02.30693	2020-10-11 23:38:31.024353
ENS	EnerSys	EnerSys manufactures, markets and distributes industrial batteries and related products such as chargers, power equipment and battery accessories.	{Technology,"Electronic Components","Computer Hardware"}	{ULBI,ARTX,EPC,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:02.319185	2020-10-11 23:38:31.025871
ENX	Eaton Vance New York Municipal Bond Fund of Beneficial Interest $.01 par value	Eaton Vance New York Municipal Bond Fund is principally engaged in managing investment funds and providing investment management and advisory services to high-net-worth individuals and institutions.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:02.333879	2020-10-11 23:38:31.024353
EPAM	EPAM Systems Inc.	EPAM Systems Inc is a provider of software engineering solutions and information technology services to clients in different locations. Its services include software development, application maintenance & support, application testing, & licensing.	{Technology,"Information Technology Services","Application Software"}	{MSFT,BBOX,CTSH,IBM,WIT,ACN,INFY,DXC,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:02.346891	2020-10-11 23:38:31.02423
EPP	iShares MSCI Pacific Ex-Japan Index Fund	The investment seeks to track the investment results of the MSCI Pacific ex Japan Index.\n The fund normally invests at least 95% of its total assets in the securities of its underlying index and in depositary receipts representing securities in its underlying index. It will at all times invest at least 90% of its total assets in such securities. The underlying index consists of stocks from the following four countries or regions: Australia, Hong Kong, New Zealand and Singapore. It may include large-, mid- or small-capitalization companies.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.364679	2020-10-11 23:38:31.023007
EGY	VAALCO Energy Inc.	VAALCO Energy Inc is an independent energy company. It is engaged in the acquisition, exploration, development and production of crude oil and natural gas.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{ZN,CHK,MRO,NBL,QEP,TAT}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:02.204183	2020-10-11 23:38:31.028166
EIDO	iShares MSCI Indonesia	The investment seeks to track the investment results of the MSCI Indonesia Investable Market Index (IMI).\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is a free float-adjusted market capitalization-weighted index designed to measure the performance of equity securities listed on stock exchanges in Indonesia. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.214405	2020-10-11 23:38:31.024353
EIS	iShares Inc MSCI Israel	The investment seeks to track the investment results of the MSCI Israel Capped Investable Market Index (IMI).\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is a free float-adjusted market capitalization index designed to measure broad-based equity market performance in Israel. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.22593	2020-10-11 23:38:31.023007
ELMD	Electromed Inc.	Electromed Inc develops, manufactures, markets and sells products that provide airway clearance therapy, including the SmartVest Airway Clearance System and related products.	{Healthcare,"Medical Devices"}	{MASI,MMSI,RMD,HRC,INGN,AHPI,XTLB,XLV}	Medical Devices	Healthcare	NYSE American	\N	2020-10-11 23:12:02.239017	2020-10-11 23:38:31.02582
ELY	Callaway Golf Company	Callaway Golf Co is engaged in designing, manufacturing and sale of golf clubs and related accessories. It sells golf clubs, golf balls and golf accessories.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{ESCA,JOUT,NKE,NLS}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:02.251345	2020-10-11 23:38:31.023007
EMI	Eaton Vance Michigan Municipal Income Trust Shares of Beneficial Interest	Eaton Vance Michigan Muni Income Trust is a United States based non-diversified closed-end investment company. Its investment objective is to provide current income exempt from regular federal income tax and taxes in its specified state.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:02.271593	2020-10-11 23:38:31.02423
EMMS	Emmis Communications Corporation	Emmis Communications Corp is a diversified media company engaged in radio broadcasting. It owns around 10 FM and AM radio stations in New York, Los Angeles, St. Louis, Austin, Indianapolis and Terre Haute, IN.	{"Consumer Cyclical","Broadcasting - Radio",Entertainment}	{SALM,BBGI,DIS,ETM,UONEK,SGA}	Entertainment	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:02.281726	2020-10-11 23:38:31.021426
EMTY	ProShares Decline of the Retail Store	The investment seeks capital appreciation from the decline of bricks and mortar retailers through short exposure (-1x) to the Solactive-ProShares Bricks and Mortar Retail Store Index (the "index").\n The fund invests in financial instruments that the Advisor believes, in combination, should produce daily returns consistent with the fund's investment objective. The index seeks to measure the performance of publicly traded "bricks and mortar" retail companies whose retail revenue is derived principally from in-store sales. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.294127	2020-10-11 23:38:31.028166
ENJ	Entergy New Orleans LLC First Mortgage Bonds 5.0% Series due December 1 2052		{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:02.308037	2020-10-11 23:38:31.028166
ENSV	Enservco Corporation	Enservco Corp and its wholly-owned subsidiaries provides well enhancement and fluid management services to the domestic onshore oil and natural gas industry. These services include frac water heating, hot oiling and acidizing among others.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{TTI,CELP}	Oil & Gas - Services	Energy	NYSE American	\N	2020-10-11 23:12:02.321673	2020-10-11 23:38:31.028166
ENVA	Enova International Inc.	Enova International Inc is a technology and analytics company focused on providing online financial services. It originates, guarantees or purchases consumer loans including short-term loans, the line of credit accounts and installment loans.	{"Financial Services","Credit Services"}	{EZPW,AL,AMRK}	Credit Services	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.333399	2020-10-11 23:38:31.028166
EOS	Eaton Vance Enhance Equity Income Fund II	Eaton Vance Enhanced Equity Income II is a diversified, closed-end management investment company. The Fund's primary investment objective is to provide current income, with a secondary objective of capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.344877	2020-10-11 23:38:31.025871
EPOL	iShares Trust MSCI Poland	The investment seeks to track the investment results of the MSCI Poland IMI 25/50 Index.\n The fund generally will invest at least 90% of its assets in the component securities of the index and in investments that have economic characteristics that are substantially identical to the component securities of the index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The index is a free float-adjusted market capitalization-weighted index designed to measure the performance of equity securities listed on stock exchanges in Poland. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.363429	2020-10-11 23:38:31.023428
EIGR	Eiger BioPharmaceuticals Inc.	Eiger BioPharmaceuticals Inc is a clinical stage biopharmaceutical company. It is focused on bringing to market novel product candidates for the treatment of orphan diseases.	{Healthcare,Biotechnology}	{UTHR,PTLA,CYTK,ABIO,AVDL,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:02.215207	2020-10-11 23:38:31.025871
ELF	e.l.f. Beauty Inc.	e.l.f. Beauty Inc is a cosmetic company based in the United States. It mainly cosmetic accessories for women such as eyeliner, mascara, lipstick, the foundation for face, moisturizer, and cleanser under the e.l.f brand.	{"Consumer Defensive","Household & Personal Products","Consumer Packaged Goods"}	{}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:02.235606	2020-10-11 23:38:31.028166
ELVT	Elevate Credit Inc.	Elevate Credit Inc provides technology-driven, progressive online credit solutions to non-prime consumers. It uses advanced technology & proprietary risk analytics to provide convenient & responsible financial options to its customers.	{"Financial Services","Credit Services"}	{HX}	Credit Services	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.249275	2020-10-11 23:38:31.028166
EMIF	iShares S&P Emerging Markets Infrastructure Ix Fd	The investment seeks to track the investment results of the S&P Emerging Markets Infrastructure IndexTM, which is designed to track the performance of 30 of the largest publicly listed companies in the infrastructure industry in emerging markets.\n The fund generally will invest at least 90% of its assets in the component securities of the index and in investments that have economic characteristics that are substantially identical to the component securities of the index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the index. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.273102	2020-10-11 23:38:31.031328
EMO	ClearBridge Energy MLP Opportunity Fund Inc.	ClearBridge Energy MLP Opportunity Fund is a non-diversified, closed-end management investment company. Its investment objective is to provide long-term investors a high level of total return with an emphasis on cash distributions.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.287827	2020-10-11 23:38:31.02423
ENDP	Endo International plc	Endo International PLC is a generics and specialty branded pharmaceutical company. The company sells generic products in the US which includes pain management, urology, central nervous system disorders, immunosuppression, and oncology and among others.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{DRRX,BDSI,MNK,CARA,ZGNX,XLV}	Drug Manufacturers	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:02.303264	2020-10-11 23:38:31.028166
ENPH	Enphase Energy Inc.	Enphase Energy Inc delivers energy management technology for the solar industry. The company designs, develops, manufactures and sells home energy solutions that connect solar generation, energy storage, and management on one intelligent platform.	{Technology,"Semiconductor Equipment & Materials",Semiconductors}	{FSLR,ABB,SPWR,XLK}	Semiconductors	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:02.319256	2020-10-11 23:38:31.02582
ENZL	iShares MSCI New Zealand Capped ETF	The investment seeks to track the investment results of the MSCI New Zealand IMI 25/50 Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is a free float-adjusted market capitalization-weighted index designed to measure the performance of equity securities in the top 99% by market capitalization of equity securities listed on stock exchanges in New Zealand. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.337897	2020-10-11 23:38:31.028166
EPE	EP Energy Corporation Class A	EP Energy Corp is an oil exploration and production company. It is in acquisition and development of unconventional oil and natural gas properties. It operates in Eagle Ford Shale, the Altamont field, and the Permian program.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{CRZO,DVN,EOG,CVX,KMI,MTDR}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:02.355926	2020-10-11 23:38:31.02582
EPU	iShares Trust MSCI Peru	The investment seeks to track the investment results of the MSCI All Peru Capped Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is an index composed of Peruvian equities. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.373907	2020-10-11 23:38:31.024353
EQWS	Invesco Russell 2000 Equal Weight	The investment seeks to track the investment results (before fees and expenses) of the Russell 2000® Equal Weight Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the component securities that comprise the underlying index. The underlying index is designed to measure the performance of approximately 2,000 equally-weighted securities of mid- and small-cap U.S. companies.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.401192	2020-10-11 23:38:31.025871
ERM	EquityCompass Risk Manager	The investment seeks to provide long term capital appreciation with capital preservation as a secondary objective.\n The fund will seek to achieve its investment objectives by investing in equity securities of companies domiciled in the U.S. or listed on a U.S. exchange. The fund's strategy seeks to provide exposure to U.S. listed equity securities and to avoid large, prolonged market losses and reduce volatility. It may invest all or a portion of its assets in cash, cash equivalents, money market funds and/or short-term fixed income ETFs. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.414436	2020-10-11 23:38:31.023007
EMITF	Elbit Imaging Ltd.	Elbit Imaging Ltd is an Israel based company provides real estate services. The business activity of the group is operated through Commercial centers, Medical Industries, and Plots in India.	{"Real Estate","Real Estate Services"}	{MAYS,CTO,PLD}	Real Estate Services	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:02.276333	2020-10-11 23:38:31.023007
EMSH	ProShares Short Term USD Emerging Markets Bond	The investment seeks investment results, before fees and expenses, that track the performance of the DBIQ Short Duration Emerging Market Bond Index (the "index").\n The index is comprised of a diversified portfolio of USD-denominated Emerging Market bonds that have less than or equal to five years remaining to maturity that are issued by Emerging Market sovereign governments, non-sovereign government agencies and entities, and corporations with significant government ownership. The fund will invest in U.S. dollar-denominated debt securities issued by Sovereign, Sub-Sovereign or Quasi-Sovereign issuers.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:02.293318	2020-10-11 23:38:31.031328
ENFC	Entegra Financial Corp.	Entegra Financial Corp is a bank holding company. The company offers deposit accounts, including savings accounts, certificates of deposit, money market accounts, commercial and regular checking accounts, and individual retirement accounts.	{"Financial Services","Banks - Regional - US",Banks}	{FFWM}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:02.306107	2020-10-11 23:38:31.023007
ENR	Energizer Holdings Inc.	Energizer Holdings Inc is a manufacturer, marketer, and distributor of household batteries, specialty batteries and lighting products. It offers batteries using many technologies including lithium, alkaline, carbon zinc, and silver oxide.	{Technology,"Electronic Components","Computer Hardware"}	{XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:02.318909	2020-10-11 23:38:31.023428
ENY	Invesco Canadian Energy Income	The investment seeks to track the investment results (before fees and expenses) of the S&P/TSX High Income Energy Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities that comprise the underlying index, as well as American depositary receipts ("ADRs") and global depositary receipts ("GDRs") that represent securities in the underlying index. The underlying index is designed to provide exposure to high-yielding Canadian equity securities in the energy sector. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.336233	2020-10-11 23:38:31.02582
EPD	Enterprise Products Partners L.P.	Enterprise Products Partners LP is a limited partnership which is engaged in transports and processes natural gas, natural gas liquids, crude oil, refined products and petrochemicals.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{APC,PAA,WMB,EEP,ET,DCP,KMI}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:02.356634	2020-10-11 23:38:31.028166
EQGP	EQGP Holdings, LP	EQT GP Holdings LP is an integrated energy company with an emphasis on natural gas production, gathering, and transmission. The Company is an investment grade natural gas producer with approximately 630,000 gross acres in the Marcellus Shale.	{Energy,"Oil & Gas Integrated","Oil & Gas - Integrated"}	{EPD,ET,SEP,ENLC}	Oil & Gas - Integrated	Energy	New York Stock Exchange	\N	2020-10-11 23:12:02.382323	2020-10-11 23:38:31.031328
EQWL	Invesco Russell Top 200 Equal Weight	The investment seeks to track the investment results (before fees and expenses) of the Russell Top 200 Equal Weight Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the component securities that comprise the underlying index. The underlying index is designed to measure the performance of approximately 200 equally weighted securities of large cap U.S. companies.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.400181	2020-10-11 23:38:31.031328
ERIE	Erie Indemnity Company	Erie Indemnity Co provides sales, underwriting and policy issuance services to the policyholders of Erie Insurance Exchange. It also operates as a property & casualty insurer through its wholly-owned subsidiaries.	{"Financial Services","Insurance Brokers","Brokers & Exchanges"}	{CINF,SIGI,STFC,DGICA,KINS,ORI,TRV,UFCS}	Brokers & Exchanges	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:02.411161	2020-10-11 23:38:31.031328
ESCA	Escalade Incorporated	Escalade Inc manufactures and distributes sporting goods. The types of sports for which products are offered include archery, table tennis, basketball, play systems, fitness, game tables, billiards, darting, and other outdoor games.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{JOUT,ELY,NLS}	Travel & Leisure	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:02.426901	2020-10-11 23:38:31.028166
ESGG	FlexShares STOXX Global ESG Impact Index Fund	The investment seeks investment results that correspond generally to the price and yield performance, before fees and expenses, of the STOXX® Global ESG Impact Index.\n The fund will invest at least 80% of its total assets (exclusive of collateral held from securities lending) in the securities of the index and in ADRs and GDRs based on the securities in the index. The index is an optimized index designed to provide broad market exposure that is tilted toward global companies that score better with respect to a small set of ESG characteristics and to provide the potential for attractive risk-adjusted performance relative to the STOXX® Global 1800 Index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.436727	2020-10-11 23:38:31.023007
KFRC	Kforce Inc.	Kforce Inc is a staffing solutions provider. It specializes in proving professional and technical staffing services to varied markets.	{Industrials,"Staffing & Outsourcing Services","Employment Services"}	{BBSI,CTG,KELYA,MAN,RHI,TBI,MHH,HSON}	Employment Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:05.740687	2020-10-11 23:38:31.024353
EMFM	Global X Next Emerging & Frontier	The investment seeks to provide investment results that correspond generally to the price and yield performance, before fees and expenses, of the Solactive Next Emerging & Frontier Index ("underlying index").\n The fund invests at least 80% of its total assets in the securities of the underlying index and in American Depositary Receipts ("ADRs") and Global Depositary Receipts ("GDRs") based on the securities in the underlying index. The underlying index is designed to reflect equity performance of Next Emerging markets and Frontier markets companies.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.266553	2020-10-11 23:38:31.023428
EML	Eastern Company (The)	The Eastern Co is a manufacturer and seller of industrial hardware, security products and metal castings. It operates in three business segments: Industrial Hardware, Security Products and Metal Products.	{Industrials,"Tools & Accessories","Industrial Products"}	{PFIN,NNBR,MLI,SWK,CIX,NL,SPLP}	Industrial Products	Industrials	NASDAQ Global Market	\N	2020-10-11 23:12:02.280089	2020-10-11 23:38:31.031328
EMQQ	Emerging Markets Internet and Ecommerce ETF (The)	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the price and yield performance of EMQQ The Emerging Markets Internet & Ecommerce IndexTM (the "index").\n The fund will normally invest at least 80% of its total assets in securities of the index or in depositary receipts representing securities of the index. The index is designed to measure the performance of an investable universe of publicly-traded, emerging market internet and ecommerce companies. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.291506	2020-10-11 23:38:31.023007
ENIC	Enel Chile S.A. American Depositary Shares (Each representing 50 shares of)	Enel Chile SA is an electricity utility company, which through its subsidiaries and affiliates is engaged in the generation, transmission and distribution of electricity businesses in Chile.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{HNP,HE,FTS,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:02.307585	2020-10-11 23:38:31.025871
ENT	Global Eagle Entertainment Inc.	Global Eagle Entertainment Inc is a full service provider of content, connectivity and digital media solutions for airlines. It offers Wi-Fi, movies, television, music, interactive software, as well as portable in-flight entertainment services.	{"Consumer Cyclical","Media - Diversified",Entertainment}	{CMCSA,IRDM,DIS,CBS,VIAB,FOXA}	Entertainment	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:02.321724	2020-10-11 23:38:31.028166
ENV	Envestnet Inc	Envestnet Inc provides wealth-management technology and services to financial advisors, investors and financial service providers. It engages in unifying financial advisors' applications and services, and provides personalized financial apps and services.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{TRI,VALU,MORN,FDS,MSCI}	Brokers & Exchanges	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.332543	2020-10-11 23:38:31.025871
EOG	EOG Resources Inc.	EOG Resources Inc explores, develops, produces and markets crude oil and natural gas in the USA, Trinidad and Tobago, United Kingdom, China, Argentina and, from time to time, select other international areas.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{CHK,NFX,PXD,DVN,APC,APA,OXY,MRO,COP,NBL}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:02.344248	2020-10-11 23:38:31.031328
EPM	Evolution Petroleum Corporation Inc.	Evolution Petroleum Corp is a petroleum company engaged in the acquisition, exploitation and development of properties for the production of crude oil and natural gas in United States.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{CRZO,DNR,HUSA,SM}	Oil & Gas - E&P	Energy	NYSE American	\N	2020-10-11 23:12:02.362962	2020-10-11 23:38:31.028166
EQC	Equity Commonwealth of Beneficial Interest	Equity Commonwealth is an internally managed and self-advised real estate investment trust. The company is engaged in the ownership and operation of office and industrial buildings throughout the United States.	{"Real Estate","REIT - Office",REITs}	{RMAX,SNH,SIR}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:02.378543	2020-10-11 23:38:31.025871
ERA	Era Group Inc.	Era Group Inc provides helicopters and related services. The company's helicopters used to transport personnel to, from and between offshore oil and gas production platforms, drilling rigs and other installations.	{Industrials,"Airports & Air Services","Transportation & Logistics"}	{PHII,CKH,BRS}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:02.401525	2020-10-11 23:38:31.023007
ERUS	iShares MSCI Russia	The investment seeks to track the investment results of the MSCI Russia 25/50 Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is designed to measure the performance of equity securities listed on stock exchanges in Russia. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.416895	2020-10-11 23:38:31.028166
ESG	FlexShares STOXX US ESG Impact Index Fund	The investment seeks investment results that correspond generally to the price and yield performance, before fees and expenses, of the STOXX® USA ESG Impact Index.\n The fund will invest at least 80% of its total assets (exclusive of collateral held from securities lending) in the securities of the underlying index. The underlying index is an optimized index designed to provide broad market exposure that is tilted toward U.S. companies that score better with respect to a small set of ESG characteristics and to provide the potential for attractive risk-adjusted performance relative to the STOXX® USA 900 Index, as determined by the index provider.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.43016	2020-10-11 23:38:31.028166
EMXC	iShares MSCI Emerging Markets ex China ETF	The investment seeks to track the investment results of the MSCI Emerging Markets ex China Index (the "underlying index"), which is designed to measure equity market performance in global emerging markets.\n The underlying index is a free float-adjusted market capitalization weighted index that captures large- and mid-capitalization stocks across 23 of the 24 Emerging Markets countries, excluding China. The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.297412	2020-10-11 23:38:31.021426
ENLC	EnLink Midstream LLC representing Limited Partner Interests	EnLink Midstream LLC is an integrated midstream company. The company, through its subsidiaries, is engaged in the gathering, transmission, processing, fractionation, brine services and marketing, to producers of natural gas, crude oil and condens.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{PAA,MMP,NS,SEMG,SXE,PSXP,PBFX,PAGP}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:02.310547	2020-10-11 23:38:31.02423
ENTA	Enanta Pharmaceuticals Inc.	Enanta Pharmaceuticals Inc is a biotechnology company. It is engaged in the research and development of small molecule drugs for infectious diseases. The Group uses its chemistry-driven approach & drug discovery capabilities to develop its products.	{Healthcare,Biotechnology}	{VRTX,ACHN,NVS,BMY,JNJ,MRK,ABBV,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:02.328349	2020-10-11 23:38:31.02423
EOI	Eaton Vance Enhance Equity Income Fundd Equity Income Fund Shares of Beneficial Interest	Eaton Vance Enhanced Equity Income Fund offers mutual funds that invest across a wide array of equity, income and alternative strategies. Each of them seeks to address the varied investment goals of today.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.34457	2020-10-11 23:38:31.021426
EPIX	ESSA Pharma Inc.	ESSA Pharma Inc is a pharmaceutical company. The company is in preclinical stage, focused on the development of small molecule drugs for the treatment of castration-resistant prostate cancer.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{XLV}	Drug Manufacturers	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:02.362334	2020-10-11 23:38:31.025871
EQBK	Equity Bancshares Inc.	Equity Bancshares Inc is a full-service community bank. It offers services such as commercial loans, consumer banking, mortgage loans & treasury management services. The bank generates a majority of its revenue from the interest income, fees, & interest.	{"Financial Services","Banks - Regional - US",Banks}	{MPB,MOFG}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:02.376888	2020-10-11 23:38:31.028166
EQS	Equus Total Return Inc.	Equus Total Return Inc is a business development company providing financing solutions for privately-owned middle market and small capitalization companies.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.395923	2020-10-11 23:38:31.028166
ERJ	Embraer S.A.	Embraer SA is engaged in designing, development, manufacture, sale and after-sale support stages of aircraft for the commercial and executive aviation markets, as well as offering integrated solutions and systems for the defense and security industry.	{Industrials,"Aerospace & Defense"}	{LMT,AIR,BA,GD,NOC,TXT,UTX,SPR}	Aerospace & Defense	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:02.411199	2020-10-11 23:38:31.02423
ESBA	Empire State Realty OP L.P. Series ES Operating Partnership Units Representing Limited Partnership Interests	Empire State Realty OP LP owns, operates, acquires and repositions properties such as Empire State Building, One Grand Central Place in Manhattan and the greater New York metropolitan area. It operates in two segments, Real Estate and Observatory.	{"Real Estate","REIT - Diversified",REITs}	{}	REITs	Real Estate	NYSE Arca	\N	2020-10-11 23:12:02.423988	2020-10-11 23:38:31.023007
ESGE	iShares MSCI EM ESG Select ETF	The investment seeks to track the investment results of the MSCI Emerging Markets Extended ESG Focus Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is an optimized equity index designed to reflect the equity performance of companies that have favorable ESG characteristics, while exhibiting risk and return characteristics similar to those of the MSCI Market Cap Weighted Index (the "parent index"). The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.435106	2020-10-11 23:38:31.021426
ESIO	Electro Scientific Industries Inc.	Electro Scientific Industries Inc is a supplier of laser-based manufacturing solutions for the microtechnology industry.	{Technology,Semiconductors}	{AMAT,ASYS,ORBK,CYBE,KLIC,BRKS,TER,NOVT,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:02.446847	2020-10-11 23:38:31.021426
ESS	Essex Property Trust Inc.	Essex Property Trust Inc owns and develops multifamily apartments. Its portfolio comprises of garden-style, midrise, and high-rise multifamily assets in the United States.	{"Real Estate","REIT - Residential",REITs}	{EQR,MAA,AIV,AVB,UDR,IRT,AMH}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:02.460179	2020-10-11 23:38:31.02423
MUX	McEwen Mining Inc.	McEwen Mining Inc is a mining and minerals exploration company. It is focused on precious and base metals in Argentina, Mexico and the United States.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{AUY,HL,AG,AGI}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:07.374452	2020-10-11 23:38:31.024353
ENFR	Alerian Energy Infrastructure	The investment seeks investment results that correspond (before fees and expenses) generally to the price and yield performance of its underlying index, the Alerian Midstream Energy Select Index.\n The underlying index is a composite of North American energy infrastructure companies engaged in the pipeline transportation, storage, and processing of energy commodities. The fund will normally invest at least 90% of its total assets in securities that comprise the underlying index. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.306465	2020-10-11 23:38:31.02582
ENOR	iShares Inc MSCI Norway	The investment seeks to track the investment results of the MSCI Norway IMI 25/50 Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is designed to measure broad-based equity market performance in Norway. A capping methodology is applied that limits the weight of any single component to a maximum of 25% of the underlying index. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:02.317175	2020-10-11 23:38:31.023007
ENTX	Entera Bio Ltd.	Entera Bio Ltd is a biotechnology company specializing in the oral delivery of large molecules and biologics. It conducts clinical trials on its proprietary technology which allows oral delivery of large molecules and biologics.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:02.332524	2020-10-11 23:38:31.023007
EPAY	Bottomline Technologies Inc.	Bottomline Technologies Inc provides trusted and easy-to-use set of cloud-based digital banking, fraud prevention, payment, financial document, insurance and healthcare solutions.	{Technology,"Software - Application","Application Software"}	{MSFT,ADBE,AMSWA,ACIW,SAP,IBM,ORCL,QTWO,DXC,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:02.350507	2020-10-11 23:38:31.023007
EPR	EPR Properties	EPR Properties is a self-administered real estate investment trust that invests in, develops, and manages properties in the United States and Canada. The company primarily invests in three segments of properties: entertainment, recreation, and education.	{"Real Estate","REIT - Retail",REITs}	{HR,FR,ALX,HST}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:02.367534	2020-10-11 23:38:31.02423
EQFN	Equitable Financial Corp.	Equitable Financial Corp is a bank holding company. The company is engaged in attracting deposits from the general public and using those funds to originate commercial loans, agricultural loans and one- to four-family residential real estate loans.	{"Financial Services","Savings & Cooperative Banks",Banks}	{MPB,MOFG}	Banks	Financial Services	OTCQX Marketplace	\N	2020-10-11 23:12:02.382471	2020-10-11 23:38:31.02423
ERH	Wells Fargo Utilities and High Income Fund	Wells Fargo Utilities and High Income Fund is a closed-end management investment company. Its objective is to seek a high level of current income and moderate capital growth, with an emphasis on providing tax-advantaged dividend income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:02.406683	2020-10-11 23:38:31.023428
ERY	Direxion Daily Energy Bear 3X Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the inverse of the daily performance of the Energy Select Sector Index.\n The fund invests in swap agreements, futures contracts, short positions or other financial instruments that, in combination, provide inverse or short leveraged exposure to the index equal to at least 80% of the fund's net assets (plus borrowing for investment purposes). The index is provided by S&P Dow Jones Indices and includes domestic companies from the energy sector which includes the following industries: oil, gas and consumable fuels; and energy equipment and services. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.418846	2020-10-11 23:38:31.024353
ESES	Eco-Stim Energy Solutions Inc.	Eco-Stim Energy Solutions Inc is an technology-driven independent oilfield services company. The Company provides well stimulation, coiled tubing and field management services to the domestic and international upstream oil and gas industry.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{SPN,HAL,HLX,USAC}	Oil & Gas - Services	Energy	NASDAQ Capital Market	\N	2020-10-11 23:12:02.429963	2020-10-11 23:38:31.02582
ESGS	Columbia Sustainable U.S. Equity Income	The investment seeks investment results that, before fees and expenses, closely correspond to the performance of the Beta Advantage® Sustainable Global Equity Income 200 Index.\n The fund invests at least 80% of its assets in the component securities of the index and depositary receipts representing such securities. The index, which typically holds common stocks and depositary receipts, was constructed to provide exposure to U.S. and foreign (developed market) large- and mid-cap companies (located in developed markets) that are believed to offer sustainable levels of income, as well as total return opportunity.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.441592	2020-10-11 23:38:31.024353
ESNT	Essent Group Ltd.	Essent Group Ltd is a housing finance provider offering credit protection to lenders and mortgage investors by covering a portion of the unpaid principal balance of a mortgage in the event of a default.	{"Financial Services","Specialty Finance",Banks}	{RNR,RDN,MTG,AGO,NMIH}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.455827	2020-10-11 23:38:31.028166
ESTR	Estre Ambiental Inc.	Estre Ambiental Inc, formerly known as Boulevard Acquisition Corp II is a blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Global Market	\N	2020-10-11 23:12:02.467114	2020-10-11 23:38:31.023007
ETM	Entercom Communications Corp.	Entercom Communications Corp is a radio broadcasting company in the United States. It operates a nationwide portfolio more than 100 radio stations in over 20 markets in the United States.	{"Consumer Cyclical","Broadcasting - Radio",Entertainment}	{SALM,BBGI,EMMS,UONEK,SGA}	Entertainment	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:02.479785	2020-10-11 23:38:31.024353
ENO	Entergy New Orleans LLC First Mortgage Bonds 5.50% Series due April 1 2066		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:02.317168	2020-10-11 23:38:31.031328
ENTR	ERShares Entrepreneur 30	The investment seeks investment results that correspond (before fees and expenses) generally to the performance of its underlying index, the Entrepreneur 30 Index.\n Under normal circumstances, the fund will invest at least 80% of its net assets, plus any borrowings for investment purposes, in securities of companies included in the Entrepreneur 30 Index. The index comprises 30 U.S. Companies with the highest market capitalizations and composite scores based on six criteria. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.329112	2020-10-11 23:38:31.031328
EOD	Wells Fargo Global Dividend Opportunity Fund	Wells Fargo Advantage Glbl Div Opport is a diversified closed-end management investment company. Its investment objective is to seek a high level of current income and long-term growth of capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.342903	2020-10-11 23:38:31.023428
EPV	ProShares UltraShort FTSE Europe	The investment seeks daily investment results, before fees and expenses, that correspond to two times the inverse (-2x) of the daily performance of the FTSE Developed Europe All Cap Index®.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index is a free float-adjusted market cap weighted index representing the performance of large, mid- and small cap companies in Developed European markets, including the UK. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.374885	2020-10-11 23:38:31.021426
EQLT	Workplace Equality Portfolio	The investment seeks investment results that correspond generally to the price and yield performance of its underlying index, the Workplace Equality Index™ (the "index").\n The fund will normally invest at least 90% of its total assets in securities that comprise the underlying index. The underlying index consists of approximately 200 publicly traded stocks of U.S. and foreign companies which support equality for LGBT employees through their workplace practices, including non-discrimination policies regarding sexual orientation and gender identity and providing full benefits for same-sex spouses, domestic partners and transgender individuals.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.38681	2020-10-11 23:38:31.023007
EQT	EQT Corporation	EQT Corp is a natural gas exploration and production company. It produces natural gas from the proved natural gas, natural gas liquid and crude oil reserves; and also provides gathering, transmission and storage services for the company's produced gas.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{OKE,COG,EPD,RRC,SWN,WMB,AR}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:02.395655	2020-10-11 23:38:31.02423
ERI	Eldorado Resorts Inc.	Eldorado Resorts Inc is a gaming and hospitality company. It owns and operates gaming facilities located in Ohio, Louisiana, Nevada, Pennsylvania and West Virginia.	{"Consumer Cyclical","Resorts & Casinos","Travel & Leisure"}	{PENN,CPHC,CHDN,DDE,UWN,CZR,FLL,NYNY}	Travel & Leisure	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:02.40744	2020-10-11 23:38:31.024353
ERYP	Erytech Pharma S.A.	Erytech Pharma SA is a France based clinical-stage biopharmaceutical company. It is engaged conducting activities in research and development in the fields of treatment for acute leukemia and other orphan diseases.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:02.422565	2020-10-11 23:38:31.023428
ESGL	Oppenheimer Revenue Weighted ETF Trust ESG Revenue	The investment seeks to provide investment results that correspond generally, before fees and expenses, to the performance of the OFI Revenue Weighted ESG Index™ (the "underlying index").\n The fund will normally invest at least 80% of its net assets, plus any borrowings for investment purposes, in the securities of companies included in the underlying index. The underlying index is constructed using a rules-based methodology to select companies from within the S&P 500® Index that have strong environmental, social and governance practices, and then re-weight those companies according to the revenue earned, subject to a maximum 5% per company weighting.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.441405	2020-10-11 23:38:31.031328
ESND	Essendant Inc.	Essendant Inc is a wholesale distributor of workplace items engaged in manufacturing of office equipment's. It operates through four segments, business and facility essentials, ORS industrial, CPO, and automotive.	{Industrials,"Business Equipment","Industrial Products"}	{SCSC,SYX,WCC,ACU,GPC,ODP}	Industrial Products	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:02.458062	2020-10-11 23:38:31.023428
ESTRW	Estre Ambiental Inc. Warrants		{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.470448	2020-10-11 23:38:31.028166
ETO	Eaton Vance Tax-Advantage Global Dividend Opp	Eaton Vance Tax-advtgd Global Divid Oppo is a diversified, closed-end management investment company. Its investment objective is to provide a high level of after-tax total return consisting of tax-advantaged dividend income and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.48142	2020-10-11 23:38:31.023428
EUDV	ProShares MSCI Europe Dividend Growers	The investment seeks investment results, before fees and expenses, that track the performance of the MSCI Europe Dividend Masters Index (the "index").\n The fund will invest at least 80% of its total assets in component securities (i.e., securities of the index and comparable securities that have economic characteristics that are substantially identical to the economic characteristics of the securities of the index). The index, constructed and maintained by MSCI, targets companies that are currently members of the MSCI Europe Index ("MSCI Europe") and have increased dividend payments each year for at least 10 years.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:02.494979	2020-10-11 23:38:31.024353
ENTG	Entegris Inc.	Entegris Inc is a developer, manufacturer and supplier of products and materials used in processing and manufacturing in the semiconductor and other technology industries. The company sells its products through a direct salesforce and distributors.	{Technology,"Semiconductor Equipment & Materials",Semiconductors}	{AEIS,MKSI,UCTT,DCI,ITW,PH,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:02.329448	2020-10-11 23:38:31.021426
EOT	Eaton Vance Municipal Income Trust EATON VANCE NATIONAL MUNICIPAL OPPORTUNITIES TRUST	Eaton Vance National Municipal Opportunities Trust is a diversified, closed end management investment company. The company's investment objective is to provide current income exempt from regular federal income tax and to achieve capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.346541	2020-10-11 23:38:31.028166
EPI	WisdomTree India Earnings Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree India Earnings Index.\n Under normal circumstances, at least 95% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is comprised of companies incorporated and traded in India that are profitable and that are eligible to be purchased by foreign investors as of the annual index screening date. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.361645	2020-10-11 23:38:31.021426
EPS	WisdomTree U.S. Earnings 500 Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree U.S. Earnings 500 Index.\n Under normal circumstances, at least 95% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a fundamentally weighted index that is comprised of earnings-generating companies within the large-capitalization segment of the U.S. Stock Market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.371384	2020-10-11 23:38:31.031328
EQL	ALPS Equal Sector Weight	The investment seeks investment results that replicate as closely as possible, before fees and expenses, the performance of the NYSE Select Sector Equal Weight Index (the "underlying index").\n In order to track the underlying index, the fund will use a "fund of funds" approach, and seek to achieve its investment objective by investing at least 90% of its total assets in the shares of the Underlying Sector ETFs. The underlying index is an index of ETFs comprised of all active Select Sector SPDR® ETFs in an equal weighted portfolio. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.386794	2020-10-11 23:38:31.021426
ERF	Enerplus Corporation	Enerplus Corp is a crude oil and natural gas exploration and development company. The company's oil and natural gas properties are located in western Canada in the provinces of Alberta, British Columbia and Saskatchewan and in the United States.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{BTE,VET}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:02.40568	2020-10-11 23:38:31.02582
ERX	Direxion Energy Bull 3X Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the daily performance of the Energy Select Sector Index.\n The fund invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, ETFs that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index is provided by S&P Dow Jones Indices (the "index provider") and includes domestic companies from the energy sector which includes the following industries: oil, gas and consumable fuels; and energy equipment and services. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.41677	2020-10-11 23:38:31.02582
ESGD	iShares MSCI EAFE ESG Select ETF	The investment seeks to track the investment results of the MSCI EAFE Extended ESG Focus Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The underlying index is composed of large- and mid-capitalization developed market equities, excluding the U.S. and Canada that have positive environmental, social and governance characteristics as identified by the index provider. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.430499	2020-10-11 23:38:31.02423
ESGR	Enstar Group Limited	Enstar Group Ltd acquires and manages insurance and reinsurance companies in run-off and portfolios of insurance and reinsurance business in run-off, and providing management, consulting and other services to the insurance and reinsurance industry.	{"Financial Services","Insurance - Diversified",Insurance}	{MHLD,RNR,RE,AXS,AHL,TPRE}	Insurance	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:02.442136	2020-10-11 23:38:31.028166
ESTE	Earthstone Energy Inc. Class A	Earthstone Energy Inc is an independent oil and natural gas development and production company. The company primarily focuses on the acquisition, development, exploration, and production of onshore, crude oil and natural gas reserves.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{NFX,MRO,NOG,CLR,OAS,SM,REN,USEG}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:02.462879	2020-10-11 23:38:31.031328
ETHO	Etho Climate Leadership U.S.	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the Etho Climate Leadership Index - US (the "index").\n The index tracks the performance of the equity securities of a diversified set of U.S. companies that are leaders in their industry with respect to their carbon impact. "Carbon impact" calculated based on the total greenhouse gas ("GHG") emissions from a company's operations, fuel use, supply chain, and business activities, divided by the company's market capitalization. The fund will invest at least 80% of its total assets in the component securities of the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.476915	2020-10-11 23:38:31.023007
ERC	Wells Fargo Multi-Sector Income Fund no par value	Wells Fargo Multi-Sector Income Fund is an asset management company. The company aims to provide its investors a high level of income by investing across companies in various sectors such as oil and gas, energy and banks.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:02.401414	2020-10-11 23:38:31.021426
EROS	Eros International PLC A	Eros International PLC is an Indian film production company. It distributes movies to multiplexes and theatres, broadcasts television content and offers digital platforms through its website www.erosnow.com and Youtube.	{"Consumer Cyclical","Media - Diversified",Entertainment}	{DIS,ENT,VIAB}	Entertainment	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:02.416159	2020-10-11 23:38:31.025871
ESE	ESCO Technologies Inc.	ESCO Technologies Inc designs and manufactures specialty filtration products including hydraulic filter elements and fluid control devices used in commercial aerospace applications, filter mechanisms used in micro-propulsion devices for satellites.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{CGNX,HURC,ITRI,AME,DHR,EMR,HON,MOG.A,ROP,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:02.428064	2020-10-11 23:38:31.025871
ESRX	Express Scripts Holding Company	Express Scripts Holding Co is a pharmacy benefit manager in the United States. It offers healthcare management and administration services such as managed care organizations, health insurers, workers' compensation plans and government health programs.	{Healthcare,"Health Care Plans"}	{CVS,RAD,PINC,WBA,XLV}	Health Care Plans	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:02.460022	2020-10-11 23:38:31.024353
ETN	Eaton Corporation PLC	Eaton Corp PLC is a power management company. It provides energy-efficient solutions that help its customers effectively manage electrical, hydraulic and mechanical power. Eaton's products include UPS systems, cylinders, circuit breakers, among others.	{Industrials,"Diversified Industrials","Industrial Products"}	{EMR,RBC,AOS}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:02.481503	2020-10-11 23:38:31.031328
EUDG	WisdomTree Europe Quality Dividend Growth Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree Europe Quality Dividend Growth Index.\n At least 80% of the fund's total assets will be invested in the component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index consists of dividend-paying common stocks of companies with growth characteristics that are incorporated and listed on a stock exchange in one of the following countries: Germany, Switzerland, United Kingdom, etc. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.493266	2020-10-11 23:38:31.031328
EURZ	Xtrackers Eurozone Equity	The investment seeks investment results that correspond generally to the performance, before fees and expenses, of the NASDAQ Eurozone Large Mid Cap Index (the "underlying index").\n The fund will invest at least 80% of its total assets (but typically far more) in component securities (including depositary receipts in respect of such securities) of the underlying index. The underlying index is designed to track the performance of equity securities of large- and mid-capitalization companies based in the countries in the Economic and Monetary Union (the "EMU" or "Eurozone") of the European Union ("EU"). The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:02.503589	2020-10-11 23:38:31.024353
EVBN	Evans Bancorp Inc.	Evans Bancorp Inc, through its wholly owned subsidiaries, provides banking services to consumer and commercial customers in Western New York as well as sells various premium-based insurance policies on a commission basis.	{"Financial Services","Banks - Regional - US",Banks}	{BDGE,AROW,NBTB,TMP,MTB,CBU}	Banks	Financial Services	NYSE American	\N	2020-10-11 23:12:02.51375	2020-10-11 23:38:31.02582
EVI	EnviroStar Inc.	EnviroStar Inc through its subsidiary sells commercial and industrial laundry and drycleaning equipment and steam and hot water boilers manufactured by others, as well as related replacement parts and accessories. It also provides maintenance services.	{"Consumer Cyclical","Personal Services"}	{MIDD,DOV,ECL,SXI,TNC,INS,MTW}	Personal Services	Consumer Cyclical	NYSE American	\N	2020-10-11 23:12:02.525825	2020-10-11 23:38:31.024353
EVO	Eaton Vance Ohio Municipal Income Trust Shares of Beneficial Interest	Eaton Vance Ohio Municipal Income Trust is a non-diversified, closed-end management investment company which seeks to provide current income exempt from regular federal income tax and taxes in its specified state.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:02.535754	2020-10-11 23:38:31.023007
EVT	Eaton Vance Tax Advantaged Dividend Income Fund of Beneficial Interest	Eaton Vance Tax Adv Dividend Income is a diversified, closed-end management investment company. Its objective is to provide a high level of after-tax total return consisting mainly of tax-advantaged dividend income and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.546762	2020-10-11 23:38:31.023007
FIZZ	National Beverage Corp.	National Beverage Corp is engaged in the development, manufacturing, marketing and sale of flavored beverage products. It offers Sparkling Waters, Juices, Energy Drinks and Carbonated Soft Drinks.	{"Consumer Defensive","Beverages - Soft Drinks","Beverages - Non-Alcoholic"}	{COT,COKE,KO,REED,MNST,CCEP,PEP}	Beverages - Non-Alcoholic	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:03.011657	2020-10-11 23:38:31.02423
EQWM	Invesco Russell Midcap Equal Weight	The investment seeks to track the investment results (before fees and expenses) of the S&P/LSTA U.S. Leveraged Loan 100 Index (the "underlying index").\n The fund generally will invest at least 80% of its net assets (plus any borrowings for investment purposes) in senior loans that comprise the underlying index. Banks and other lending institutions generally issue senior loans to corporations, partnerships or other entities ("borrowers"). These borrowers operate in a variety of industries and geographic regions, including foreign countries.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.400351	2020-10-11 23:38:31.028166
ERII	Energy Recovery Inc.	Energy Recovery Inc manufactures products that are used in the desalination process. The company serves the oil and gas, chemical processing and water industries across the world.	{Industrials,"Pollution & Treatment Controls","Industrial Products"}	{FLS,GE,GRC,GGG,IEX,SPXC,CFX,GHM}	Industrial Products	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:02.411228	2020-10-11 23:38:31.021426
ES	Eversource Energy (D/B/A)	Eversource Energy is a public utility holding company, which is engaged in the energy delivery business. The company operates through electric distribution, electric transmission and natural gas distribution segments.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{LNT,ED,D,SCG,PPL,UTL,AGR,NGG,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:02.423194	2020-10-11 23:38:31.021426
ESGF	Oppenheimer Global ESG Revenue	The investment seeks to provide investment results that correspond generally, before fees and expenses, to the performance of the OFI Revenue Weighted Global ESG Index (the "underlying index").\n The fund will normally invest at least 80% of its net assets, plus any borrowings for investment purposes, in the securities of companies included in the underlying index. The underlying index is constructed using a rules-based methodology to select companies from within the MSCI All Country World Index that have strong ESG practices and then re-weight those companies according to the revenue earned, subject to a maximum 5% per company weighting.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.436365	2020-10-11 23:38:31.023428
ESLT	Elbit Systems Ltd.	Elbit Systems Ltd develops and supplies a portfolio of airborne, land and naval systems and products for defense, homeland security and commercial aviation applications. The company also provides support services.	{Industrials,"Aerospace & Defense"}	{FLIR,RADA,LMT,BA,GD,HRS,NOC,RTN,COL,LLL}	Aerospace & Defense	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:02.449075	2020-10-11 23:38:31.031328
ESQ	Esquire Financial Holdings Inc.	Esquire Financial Holdings Inc is a registered bank holding company. It is an independent, full-service national bank that serves the banking needs of law professionals, professional service firms, small to mid-sized businesses and individuals.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:02.458941	2020-10-11 23:38:31.021426
ETFC	E*TRADE Financial Corporation	E*TRADE Financial Corp is a financial services company. It provides brokerage and related products and services to individual retail investors. It also provides investor-focused banking products and sweep deposits to retail investors.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{SIEB,AMTD,RJF,SCHW,SF,LTS,GCAP,LPLA,IBKR}	Brokers & Exchanges	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:02.472225	2020-10-11 23:38:31.021426
ETSY	Etsy Inc.	Etsy Inc operates a marketplace where people around the world connect, both online and offline, to make, sell and buy goods.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{AMZN,EBAY}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:02.485129	2020-10-11 23:38:31.02582
EUFX	ProShares Short Euro	The investment seeks results for a single day that match (before fees and expenses) the inverse (-1x) of the daily performance of the U.S. dollar price of the euro.\nThe fund obtains short exposures to its benchmark through futures contracts on its underlying currency. It may also invest in forward contracts if the market for a specific futures contract experiences emergencies (e.g., natural disaster, terrorist attack or an act of God) or disruptions (e.g., a trading halt or a flash crash) that would prevent or make it impractical for the fund to obtain the appropriate amount of investment exposure to the affected futures contracts directly.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.497722	2020-10-11 23:38:31.023428
EV	Eaton Vance Corporation	Eaton Vance Corp is engaged in providing asset management and investment advisory services to institutional and individual investors, high-net-worth clients, institutional separate accounts, and retail managed accounts.	{"Financial Services","Asset Management"}	{GROW,TROW,VRTS,WDR,FII,BEN,LM,MN,APAM}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.509608	2020-10-11 23:38:31.023428
EVGBC	Eaton Vance Global Income Builder NextShares	The investment seeks total return.\n The fund invests in securities and other instruments to establish investment exposures to credit markets around the world. It invests at least 80% of its net assets (plus any borrowings for investment purposes) in credit-related investments (the “80% Policy”). For purposes of this 80% Policy, “credit-related investments” are fixed income, variable rate, and floating-rate debt investments as well as derivatives that provide exposure to such investments.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.518873	2020-10-11 23:38:31.02582
EVN	Eaton Vance Municipal Income Trust	Eaton Vance Municipal Income Trust is a United States based diversified, closed-end management investment company. Its investment objective is to provide current income exempt from regular federal income tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.537355	2020-10-11 23:38:31.023428
EVV	Eaton Vance Limited Duration Income Fund of Beneficial Interest	Eaton Vance Ltd Duration Income Fund is a diversified closed-end management investment company. The company's investment objective is to provide a high level of current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:02.55134	2020-10-11 23:38:31.028166
ERIC	Ericsson	Telefonaktiebolaget L M Ericsson is a telecom services provider based in Sweden. It provides wireless telecommunication solutions globally.	{Technology,"Communication Equipment"}	{CSCO,MSFT,HLIT,QCOM,NOK,JNPR,HRS,MSI,CIEN,BB,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:02.408063	2020-10-11 23:38:31.028166
ESEA	Euroseas Ltd.	Euroseas Ltd is a shipping company. Along with its subsidiaries it primarily owns and operates a fleet consisting of dry-bulk, container, and multipurpose dry cargo vessels that are employed on spot and time charters.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{DSX,NM,DAC,NMM,SBLK,GNK,DRYS,DCIX}	Transportation & Logistics	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:02.429101	2020-10-11 23:38:31.024353
ESGU	iShares MSCI USA ESG Optimized ETF	The investment seeks to track the investment results of the MSCI USA Extended ESG Focus Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The underlying index is an optimized equity index designed to reflect the equity performance of U.S. companies that have favorable environmental, social and governance ("ESG") characteristics, while exhibiting risk and return characteristics similar to those of the MSCI USA Index. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.444539	2020-10-11 23:38:31.02582
ESPR	Esperion Therapeutics Inc.	Esperion Therapeutics Inc is a pharmaceutical company focused on developing and commercializing oral, low-density lipoprotein cholesterol, or LDL-C, lowering therapies for the treatment of patients with elevated LDL-C.	{Healthcare,Biotechnology}	{BMRN,REGN,FOLD,PFE,RARE,LXRX,ALBO,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:02.458883	2020-10-11 23:38:31.028166
ESXB	Community Bankers Trust Corporation.	Community Bankers Trust Corp is single-bank holding company for Essex Bank. It operates in three business lines namely retail and small business banking, commercial and industrial banking and real estate lending.	{"Financial Services","Banks - Regional - US",Banks}	{FBSS,SONA,BOTJ,UBSH,VBFC}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:02.470606	2020-10-11 23:38:31.028166
ETV	Eaton Vance Corporation Tax-Managed Buy-Write Opportunities Fund of Beneficial Interest	Eaton Vance Tax-Managed Buy-Write Opp is a United States based diversified, closed-end management investment company. Its primary investment objective is to provide current income and gains. The secondary objective of the fund is capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.484647	2020-10-11 23:38:31.021426
EUMV	iShares Edge MSCI Min Vol Europe	The investment seeks to track the investment results of the MSCI Europe Minimum Volatility (USD) Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index measures the performance of European developed market equity securities that, in the aggregate, have lower volatility characteristics relative to the broader European developed equity markets.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.498627	2020-10-11 23:38:31.021426
EVBG	Everbridge Inc.	Everbridge Inc is a software company that provides critical communications and enterprise safety applications that enable customers to automate and accelerate the process of keeping people safe and businesses running during critical events.	{Technology,"Software - Application","Application Software"}	{XLK}	Application Software	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:02.515358	2020-10-11 23:38:31.021426
EVK	Ever-Glory International Group Inc.	Ever-Glory International Group Inc retails branded fashion apparel and operates as an apparel supply chain solution provider. The company operates a retail business with over 1000 retail locations, also offering global apparel supply chain solutions.	{"Consumer Cyclical","Apparel Manufacturing","Manufacturing - Apparel & Furniture"}	{GIII,DLA}	Manufacturing - Apparel & Furniture	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:02.530551	2020-10-11 23:38:31.02423
EVRI	Everi Holdings Inc.	Everi Holdings Inc is engaged in providing video and mechanical reel gaming content and technology solutions, integrated gaming payments solutions and compliance and efficiency software.	{"Consumer Cyclical",Gambling,"Travel & Leisure"}	{PENN,GPIC,BYD}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:02.541014	2020-10-11 23:38:31.025871
EWA	iShares MSCI Australia Index Fund	The investment seeks to track the investment results of the MSCI Australia Index composed of Australian equities.\n The fund generally invests at least 90% of its assets in the securities of its underlying index and in depositary receipts representing securities in its underlying index. The underlying index may include large-, mid- or small-capitalization companies. The components of the underlying index, and the degree to which these components represent certain industries, are likely to change over time. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.553852	2020-10-11 23:38:31.025871
EWN	iShares MSCI Netherlands Index Fund	The investment seeks to track the investment results of the MSCI Netherlands IMI 25/50 Index.\n The fund will at all times invest at least 80% of its assets in the securities of its underlying index and in depositary receipts representing securities in its underlying index. The underlying index uses a capping methodology to limit the weight of any single component to a maximum of 25% of the underlying index. The underlying index may include large-, mid- or small- capitalization companies. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.569163	2020-10-11 23:38:31.02582
ESNC	EnSync Inc.	EnSync Inc is engaged in developing, licensing and manufacturing energy management systems solutions serving the commercial and industrialand multi-tenant building, utility, and off-grid markets.	{Industrials,"Diversified Industrials","Industrial Products"}	{EFL,ARTX,ENS,HPJ,PLUG,SPWR,FCEL}	Industrial Products	Industrials	NYSE American	\N	2020-10-11 23:12:02.450774	2020-10-11 23:38:31.023007
ESRT	Empire State Realty Trust Inc. Class A	Empire State Realty Trust Inc owns, operates, acquires and repositions properties such as Empire State Building, One Grand Central Place in Manhattan and the greater New York metropolitan area. It operates in two segments, Real Estate and Observatory.	{"Real Estate","REIT - Diversified",REITs}	{HIW,BXP,SLG,CLI,LXP,CUZ,LM}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:02.4599	2020-10-11 23:38:31.025871
ETH	Ethan Allen Interiors Inc.	Ethan Allen Interiors Inc operates in the home furnishing industry. Its suite of products includes furniture, bedding, lighting and rugs.	{"Consumer Cyclical","Home Furnishings & Fixtures","Manufacturing - Apparel & Furniture"}	{BSET,BBBY,FLXS,HVT,LZB,PIR,WSM,TPX}	Manufacturing - Apparel & Furniture	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:02.472842	2020-10-11 23:38:31.025871
ETW	Eaton Vance Corporation Tax-Managed Global Buy-Write Opportunites Fund of Beneficial Interest	Eaton Vance Tax-mangd Glo Buy-write Oppo is a closed-end management investment company. Its investment objective is to provide current income and gains, with a secondary objective of capital appreciation with investments in common stocks and options.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.488145	2020-10-11 23:38:31.02423
EUO	ProShares UltraShort Euro	The investment seeks daily results that match (before fees and expenses) twice the inverse (-2x) of the daily performance of the U.S. Dollar price of the Euro.\nThe “UltraShort” funds seek results for a single day that match (before fees and expenses) two times the inverse (-2x) of the daily performance of a benchmark. It does not seek to achieve their stated objectives over a period greater than a single day.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.499556	2020-10-11 23:38:31.02582
EVFM	Evofem Biosciences Inc.	Evofem Biosciences Inc provides woman-controlled protection from sexually transmitted infections. It addresses women's unmet medical needs through the discovery, development and commercialization of healthcare products.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:02.515436	2020-10-11 23:38:31.023007
EVOK	Evoke Pharma Inc.	Evoke Pharma Inc is a specialty pharmaceutical company which focuses on the development of drugs to treat gastrointestinal (GI) disorders and diseases. It is engaged in developing an investigational metoclopramide nasal spray called Gimoti.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{PGNX,IRWD,SGYP,ANIP,INVA,XLV}	Drug Manufacturers	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:02.536689	2020-10-11 23:38:31.024353
EVTC	Evertec Inc.	Evertec Inc is a transaction processing business in Latin America. Its business segments are Merchant Acquiring, Payment Services - Puerto Rico & Caribbean, Payment Services - Latin America, and Business Solutions.	{Technology,"Software - Infrastructure","Application Software"}	{XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:02.548218	2020-10-11 23:38:31.024353
EWG	iShares MSCI Germany Index Fund	The investment seeks to track the investment results of the MSCI Germany Index.\n The fund will at all times invest at least 80% of its assets in the securities of its underlying index and in depositary receipts representing securities in its underlying index. The underlying index consists of stocks traded primarily on the Frankfurt Stock Exchange. It may include large-, mid- or small-capitalization companies. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.561392	2020-10-11 23:38:31.028166
EWP	iShares Inc MSCI Spain	The investment seeks to track the investment results of the MSCI Spain 25/50 Index.\n The fund will at all times invest at least 80% of its assets in the securities of its underlying index and in depositary receipts representing securities in its underlying index. The underlying index is a free float-adjusted market capitalization-weighted index with a capping methodology applied to issuer weights so that no single issuer of a component exceeds 25% of the underlying index weight, and all issuers with a weight above 5% do not cumulatively exceed 50% of the underlying index weight. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.574526	2020-10-11 23:38:31.02423
EWV	ProShares UltraShort MSCI Japan	The investment seeks daily investment results, before fees and expenses, that correspond to two times the inverse (-2x) of the daily performance of the MSCI Japan Index®.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index adjusts the market capitalization of Index constituents for free float and targets for Index inclusion 85% of the free float-adjusted market capitalization in each industry group in Japan. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.594437	2020-10-11 23:38:31.021426
EXI	iShares Global Industrials	The investment seeks to track the investment results of the S&P Global 1200 Industrials IndexTM.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index. The index measures the performance of companies that the index provider deems to be part of the industrials sector of the economy and that the index provider believes are important to global markets.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.612667	2020-10-11 23:38:31.028166
ESSA	ESSA Bancorp Inc.	ESSA Bancorp Inc is a bank holding company. The Company through its subsidiary ESSA Bank & Trust provides financial services to individuals, families and businesses. ESSA Bank & Trust is a community-oriented savings association.	{"Financial Services","Savings & Cooperative Banks",Banks}	{WVFC,NWBI,PFS,PBIP,BNCL}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:02.461747	2020-10-11 23:38:31.023428
ETJ	Eaton Vance Risk-Managed Diversified Equity Income Fund of Beneficial Interest	Eaton Vance Risk-Mgd Diversified Eq Inc is a a diversified, closed-end management investment company. The Fund's primary investment objective is to provide current income and gains, with a secondary objective of capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.478125	2020-10-11 23:38:31.028166
EUSA	iShares MSCI USA Equal Weighted	The investment seeks to track the investment results of the MSCI USA Equal Weighted Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The index is an equally-weighted securities index that measures the performance of equity securities in the top 85% by market capitalization of equity securities listed on stock exchanges in the United States and which represents an alternative weighting scheme to its market capitalization-weighted parent index, the MSCI USA Index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.506877	2020-10-11 23:38:31.023007
EVFTC	Eaton Vance Floating-Rate NextShares	The investment seek to provide a high level of current income.\n Under normal circumstances, the fund invests at least 80% of its net assets (plus any borrowings for investment purposes) in income producing floating rate loans and other floating rate debt securities. It invests primarily in senior floating rate loans of domestic and foreign borrowers (“Senior Loans”). The fund may invest up to 25% of its total assets in foreign Senior Loans. Foreign Senior Loans must be denominated in U.S. dollars, euros, British pounds, Swiss francs, Canadian dollars, or Australian dollars.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.516321	2020-10-11 23:38:31.024353
EVH	Evolent Health Inc Class A	Evolent Health Inc is engaged in healthcare delivery and payment. It markets and sells its services to providers throughout the United States.	{Technology,"Health Information Services","Application Software"}	{CRY,NHC,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:02.524945	2020-10-11 23:38:31.031328
EW	Edwards Lifesciences Corporation	Edwards Lifesciences Corp is a part of the healthcare sector. The company specializes in the treatment of structural heart disease. It manufactures heart valves and repair products used to replace or repair a patient's diseased or defective heart valve.	{Healthcare,"Medical Devices"}	{ICUI,MMSI,LMAT,CRY,BSX,MDT,XLV}	Medical Devices	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:02.551778	2020-10-11 23:38:31.021426
EWH	iShares MSCI Hong Kong Index Fund	The investment seeks to track the investment results of the MSCI Hong Kong Index.\n The fund will at all times invest at least 80% of its assets in the securities of its underlying index and in depositary receipts representing securities in its underlying index. The underlying index consists of stocks traded primarily on the Stock Exchange of Hong Kong Limited (SEHK). It may include large-, mid- or small-capitalization companies. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.563952	2020-10-11 23:38:31.031328
EWRE	Invesco S&P 500 Equal Weight Real Estate	The investment seeks to track the investment results (before fees and expenses) of the S&P 500® Equal Weight Real Estate Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities that comprise the underlying index. The underlying index is an equal-weighted version of the S&P 500® Real Estate Index. Strictly in accordance with its guidelines and mandated procedures, the index provider compiles, maintains and calculates the underlying index, which is comprised of common stocks of companies in the real estate sector of the S&P 500® Index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.582164	2020-10-11 23:38:31.023007
EWZ	iShares Inc MSCI Brazil	The investment seeks to track the investment results of the MSCI Brazil 25/50 Index.\n The fund invests at least 95% of its assets in the securities of its underlying index and in depositary receipts representing securities in its underlying index. The index, consists of stocks traded primarily on B3 (the largest Brazilian exchange), is a free float-adjusted market capitalization-weighted index with a capping methodology applied to issuer weights so that no single issuer of a component exceeds 25% of the underlying index weight, and all issuers with weight above 5% do not cumulatively exceed 50% of the underlying index weight. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.598426	2020-10-11 23:38:31.025871
EXLS	ExlService Holdings Inc.	ExlService Holdings Inc is an operations management and analytics company. It helps businesses enhance growth and profitability. The company's operating business segments are Operations Management, and Analytics.	{Industrials,"Business Services"}	{INOD,WLDN,WNS,BR,G,AXR,INFY}	Business Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:02.617485	2020-10-11 23:38:31.021426
EXTR	Extreme Networks Inc.	Extreme Networks Inc is a manufacturer of communication equipment. It designs, develops and manufactures wired and wireless network infrastructure equipment and develops the software for network management, policy, analytics, security and access controls.	{Technology,"Communication Equipment"}	{CSCO,ADTN,ERIC,INFN,JNPR,MSI,HPQ,RBBN,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:02.630507	2020-10-11 23:38:31.021426
ETG	Eaton Vance Tax-Advantaged Global Dividend Income Fund of Beneficial Interest	Eaton Vance Tax Advgd Gbl Div Inc Fd is a diversified, closed-end management investment company. Its investment objective is to provide a high level of after-tax total return consisting of tax-advantaged dividend income and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.474633	2020-10-11 23:38:31.02423
ETX	Eaton Vance Municipal Income 2028 Term Trust of Beneficial Interest	Eaton Vance Municipal Income 2028 Term Trust is a closed-end management investment company. Its investment objective is to provide current income exempt from regular federal income tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.487981	2020-10-11 23:38:31.028166
EURL	Direxion Daily FTSE Europe Bull 3x Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the daily performance of the FTSE Developed Europe All Cap Index.\n The fund invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, exchange-traded funds ("ETFs") that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index is a market capitalization weighted index that is designed to measure the equity market performance of large-, mid- and small-cap companies in developed markets in Europe. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.502489	2020-10-11 23:38:31.02423
EVF	Eaton Vance Senior Income Trust	Eaton Vance Senior Income Trust is a closed-end management investment company. Its investment objective is to provide a high level of current income consistent with the preservation of capital by investing primarily in senior, secured floating-rate loans.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.514186	2020-10-11 23:38:31.02423
EVJ	Eaton Vance New Jersey Municipal Income Trust Shares of Beneficial Interest	Eaton Vance NJ Muni Income Trust is a diversified closed-end investment company. Its investment objective is to provide current income exempt from regular federal income tax and taxes in its specified state.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:02.527507	2020-10-11 23:38:31.028166
EVOL	Evolving Systems Inc.	Evolving Systems Inc is a provider of software solutions and services to the wireless, wireline and cable markets. The company operates one operating segment which includes two revenue types: license fees revenue and services revenue.	{Technology,"Software - Application","Application Software"}	{CSGS,MNDO,SNCR,IBM,DOX,APPS,SMSI,GEC,XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:02.537281	2020-10-11 23:38:31.031328
EWC	iShares MSCI Canada Index Fund	The investment seeks to track the investment results of the MSCI Canada Custom Capped Index.\n The fund will at all times invest at least 90% of its assets in the securities of its underlying index and in depositary receipts representing securities in its underlying index. The underlying index is designed to measure broad-based equity performance in Canada. The underlying index uses a capping methodology to limit the weight of any single component to a maximum of 25% of the underlying index. The underlying index may include large-, mid- or small-capitalization companies.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.555529	2020-10-11 23:38:31.024353
EWQ	iShares MSCI France Index Fund	The investment seeks to track the investment results of the MSCI France Index.\n The fund will at all times invest at least 80% of its assets in the securities of its underlying index and in depositary receipts representing securities in its underlying index. The underlying index consists of stocks traded primarily on the Paris Stock Exchange. It may include large-, mid- or small-capitalization companies. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.575084	2020-10-11 23:38:31.031328
EWUS	Ishares MSCI United Kingdom Small Cap	The investment seeks to track the investment results of the MSCI United Kingdom Small Cap Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is designed to measure the performance of equity securities of small-capitalization companies whose market capitalization, as calculated by the index provider, represents the bottom 14% of the United Kingdom securities market.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:02.590849	2020-10-11 23:38:31.028166
EXD	Eaton Vance Tax-Advantaged Bond of Beneficial Interest	Eaton Vance Tax-Advantaged Bond and Option Strategies Fund is a diversified, closed-end management investment company. The fund's investment objective is to provide tax-advantaged current income and gains.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.604934	2020-10-11 23:38:31.028166
EXP	Eagle Materials Inc	Eagle Materials Inc supplies building products which are used in residential, industrial, commercial, and infrastructure construction. Its segments include Cement, Concrete and Aggregates, Gypsum Wallboard, Recycled Paperboard, and Oil and Gas Proppants.	{"Basic Materials","Building Materials"}	{USLM,USG,MLM,CRH,VMC,CBPX,CPAC}	Building Materials	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:02.618629	2020-10-11 23:38:31.023007
EYE	National Vision Holdings Inc.	National Vision Holdings Inc is an optical retailers in the United States. Its brands include Eyeglass World, Vision Centre, Vista Optical, among others.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{BCI,EYE}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:02.632413	2020-10-11 23:38:31.023428
ETR	Entergy Corporation	Entergy Corp is an energy company. It is engaged in electric power production and retail electric distribution operations in portions of Arkansas, Mississippi, Texas, and Louisiana, including the City of New Orleans.	{Utilities,"Utilities - Diversified","Utilities - Regulated"}	{EE,OGE,ED,AEP,CMS,SO,EXC,D,PPL,DUK,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:02.484958	2020-10-11 23:38:31.023007
EVA	Enviva Partners LP representing limited partner interests	Enviva Partners LP is a producer and supplier of Biomass products. The company procure wood fiber and process it into utility-grade wood pellets. It owns and operates six industrial-scale production plants in the Southeastern United States.	{"Basic Materials","Lumber & Wood Production","Forest Products"}	{ED,VVC,OESX}	Forest Products	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:02.511229	2020-10-11 23:38:31.028166
EVGN	Evogene Ltd.	Evogene Ltd is a biotechnology company for the improvement of crop productivity. It utilizes a proprietary integrated technology infrastructure to enhance seed traits underlying crop productivity.	{Healthcare,Biotechnology}	{CGEN,XXII,DWDP,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:02.520365	2020-10-11 23:38:31.021426
EVLV	EVINE Live Inc.	EVINE Live Inc is a multiplatform video commerce company that offers a mix of proprietary, exclusive and name brands directly to consumers in an engaging and informative shopping experience through TV, online and mobile devices. It operates Evine.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{CNXN,PCMI,OSTK,PRTS,AMCX,MSGN,GAIA}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:02.531264	2020-10-11 23:38:31.028166
EVR	Evercore Inc. Class A	Evercore Inc is an investment banking and investment management firm. It provides financial advisory services to clients on mergers, acquisitions, divestitures and other strategic corporate transactions.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{GHL,LAZ,HF,BX}	Brokers & Exchanges	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.541331	2020-10-11 23:38:31.02582
EVX	VanEck Vectors Environmental Services	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the NYSE Arca Environmental Services Index.\n The fund normally invests at least 80% of its total assets in common stocks and ADRs of companies involved in the environmental services industry. The index is comprised of companies that engage in business activities that may benefit from the global increase in demand for consumer waste disposal, removal and storage of industrial by-products, and the management of associated resources and includes securities of companies related environmental services. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.551567	2020-10-11 23:38:31.031328
EWJ	iShares MSCI Japan Index Fund	The investment seeks to track the investment results of the MSCI Japan Index.\n The fund will at all times invest at least 90% of its assets in the securities of its underlying index and in depositary receipts representing securities in its underlying index. The underlying index consists of stocks traded primarily on the Tokyo Stock Exchange. It may include large-, mid- or small-capitalization companies.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.564732	2020-10-11 23:38:31.023007
EWU	iShares MSCI United Kingdom	The investment seeks to track the investment results of the MSCI United Kingdom Index.\n The fund will at all times invest at least 90% of its assets in the securities of its underlying index and in depositary receipts representing securities in its underlying index. The underlying index consists of stocks traded primarily on the London Stock Exchange. The underlying index may include large-, mid- or small-capitalization companies. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.593436	2020-10-11 23:38:31.028166
EXG	Eaton Vance Tax-Managed Global Diversified Equity Income Fund of Beneficial Interest	Eaton Vance Tax-Mgd Glbl Div Eq Inc is a United States based diversified, closed-end management investment company. Its primary investment objective is to provide current income and gains. The secondary objective of the fund is capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.613893	2020-10-11 23:38:31.02582
EYEN	Eyenovia Inc.	Eyenovia Inc is a clinical stage biopharmaceutical company. It is engaged in developing a pipeline of ophthalmology products utilizing patented piezo-print technology to deliver micro-therapeutics topically to the eye.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:02.634703	2020-10-11 23:38:31.024353
FAD	First Trust Multi Cap Growth AlphaDEX Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of the Nasdaq AlphaDEX® Multi Cap Growth Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is designed to select growth stocks from the NASDAQ US 500 Large Cap Index, NASDAQ US 600 Mid Cap Index and NASDAQ US 700 Small Cap Index that may generate positive alpha, or risk-adjusted returns, relative to traditional indices through the use of the AlphaDEX® selection methodology.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.66005	2020-10-11 23:38:31.028166
FAS	Direxion Financial Bull 3X Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the daily performance of the Russell 1000® Financial Services Index.\n The fund invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, exchange-traded funds ("ETFs") that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index is a subset of the Russell 1000® Index that measures the performance of the securities classified in the financial services sector of the large-capitalization U.S. equity market. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.67785	2020-10-11 23:38:31.025871
EURN	Euronav NV	Euronav NV provides international maritime shipping and offshore services engaged in the transportation and storage of crude oil. The company owns and operates modern fleet of around 55 vessels.	{Industrials,"Integrated Shipping & Logistics","Transportation & Logistics"}	{NM,NMM}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:02.50223	2020-10-11 23:38:31.028166
EVC	Entravision Communications Corporation	Entravision Communications Corp is a media company. Its business activities include integrated media and marketing solutions comprised of acclaimed television, radio, digital properties, events, and data analytics services.	{"Consumer Cyclical","Broadcasting - TV",Entertainment}	{SBGI,SALM,NXST,TV,DIS,CBS,FOXA,SSP,SGA}	Entertainment	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:02.513594	2020-10-11 23:38:31.028166
EVP	Eaton Vance Pennsylvania Municipal Income Trust Shares of Beneficial Interest	Eaton Vance PA Muni Income Trust is a non-diversified, closed-end management investment company which seeks to provide current income exempt from regular federal income tax and taxes in its specified state.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:02.540298	2020-10-11 23:38:31.021426
EWBC	East West Bancorp Inc.	East West Bancorp Inc is the holding company for East West Bank, a full-service commercial bank with cross-border operations concentrated in the United States and China. Its reportable segments are Retail Banking, Commercial Banking and Other.	{"Financial Services","Banks - Global",Banks}	{TCBK,CWBC,CATY,PPBI,SIVB,BMRC,FRC}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:02.554804	2020-10-11 23:38:31.02582
EWL	iShares Inc MSCI Switzerland	The investment seeks to track the investment results of the MSCI Switzerland 25/50 Index.\n The fund will at all times invest at least 80% of its assets in the securities of its underlying index and in depositary receipts representing securities in its underlying index. The underlying index is a free float-adjusted market capitalization-weighted index with a capping methodology applied to issuer weights so that no single issuer of a component exceeds 25% of the underlying index weight, and all issuers with a weight above 5% do not cumulatively exceed 50% of the underlying index weight. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.567305	2020-10-11 23:38:31.024353
EWW	iShares Inc MSCI Mexico	The investment seeks to track the investment results of the MSCI Mexico IMI 25/50 Index.\n The fund will at all times invest at least 80% of its assets in the securities of its underlying index and in depositary receipts representing securities in its underlying index. The underlying index is a free float-adjusted market capitalization-weighted index with a capping methodology applied to issuer weights so that no single issuer of a component exceeds 25% of the underlying index weight, and all issuers with a weight above 5% do not cumulatively exceed 50% of the underlying index weight. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.591446	2020-10-11 23:38:31.02582
EXEL	Exelixis Inc.	Exelixis Inc is a biotechnology company. It is involved in discovery, development, and commercialization of new medicines for people with cancer. The Cabometyx product generates maximum revenue for the company.	{Healthcare,Biotechnology}	{AMGN,ARQL,AVEO,GSK,BMY,JNJ,PFE,SNY,MRK,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:02.608916	2020-10-11 23:38:31.02423
EXR	Extra Space Storage Inc	Extra Space Storage Inc is a fully integrated, self-administered and self-managed real estate investment trust. It owns, operates, manages, acquires, develops and redevelops professionally managed self-storage facilities.	{"Real Estate","REIT - Industrial",REITs}	{PSA,AKR,SP,LSI}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:02.629143	2020-10-11 23:38:31.031328
EZJ	ProShares Ultra MSCI Japan	The investment seeks daily investment results, before fees and expenses, that correspond to two times (2x) the daily performance of the MSCI Japan Index®.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index adjusts the market capitalization of Index constituents for free float and targets for Index inclusion 85% of the free float-adjusted market capitalization in each industry group in Japan. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.648075	2020-10-11 23:38:31.028166
FALN	iShares Fallen Angels USD Bond ETF	The investment seeks to track the investment results of the Bloomberg Barclays US High Yield Fallen Angel 3% Capped Index composed of U.S. dollar-denominated, high yield corporate bonds that were previously rated investment grade.\n The fund generally will invest at least 90% of its assets in the component securities of the index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The index is designed to reflect the performance of U.S. dollar denominated, high yield corporate bonds that were previously rated investment grade. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.664133	2020-10-11 23:38:31.021426
FATE	Fate Therapeutics Inc.	Fate Therapeutics Inc is a clinical-stage biopharmaceutical company engaged in the development of programmed cellular immunotherapies for cancer and immune disorders.	{Healthcare,Biotechnology}	{NBIX,BDX,SRPT,PTCT,IONS,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:02.682342	2020-10-11 23:38:31.028166
FBMS	The First Bancshares Inc.	First Bancshares Inc is a financial holding company providing products and services offered by regional banks. It also offers deposit services and commercial and personal loans.	{"Financial Services","Banks - Regional - US",Banks}	{TRMK,CIZN,RNST,FHN,RF,BXS,PFBX}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:02.705364	2020-10-11 23:38:31.024353
EUSC	WisdomTree Europe Hedged SmallCap Equity Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree Europe Hedged SmallCap Equity Index.\n Under normal circumstances, at least 80% of the fund's total assets will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a dividend weighted index designed to provide exposure to small cap equity securities within Europe, while at the same time neutralizing exposure to fluctuations between the value of the euro and the U.S. dollar. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.507134	2020-10-11 23:38:31.031328
EVG	Eaton Vance Short Diversified Income Fund Duration Diversified Income Fund of Beneficial Interest	Eaton Vance Short Duration Divsfd Inc is a diversified, closed-end management investment company. Its investment objective is to provide a high level of current income and to seek capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.517293	2020-10-11 23:38:31.025871
EVM	Eaton Vance California Municipal Bond Fund of Beneficial Interest $.01 par value	Eaton Vance California Municipal Bond Fund is an investment company. Its investment objective is to provide current income exempt from regular federal income tax, including alternative minimum tax, in state specific funds, taxes in its specified state.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:02.537661	2020-10-11 23:38:31.02582
EVY	Eaton Vance New York Municipal Income Trust Shares of Beneficial Interest	Eaton Vance NY Muni Income Trust is a United States based non-diversified closed-end investment company. Its investment objective is to provide current income exempt from regular federal income tax and taxes in its specified state.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:02.552434	2020-10-11 23:38:31.02423
EWI	iShares Inc MSCI Italy	The investment seeks to track the investment results of the MSCI Italy 25/50 Index.\n The fund will at all times invest at least 80% of its assets in the securities of its underlying index and in depositary receipts representing securities in its underlying index. The underlying index is a free float-adjusted market capitalization-weighted index with a capping methodology applied to issuer weights so that no single issuer of a component exceeds 25% of the underlying index weight, and all issuers with a weight above 5% do not cumulatively exceed 50% of the underlying index weight. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.565479	2020-10-11 23:38:31.021426
EWS	iShares Inc MSCI Singapore	The investment seeks to track the investment results of the MSCI Singapore 25/50 Index.\n The fund will at all times invest at least 80% of its assets in the securities of its underlying index and in depositary receipts representing securities in its underlying index. The index is designed to measure the performance of the large- and mid-cap segments of the Singapore market. A capping methodology is applied that limits the weight of any single component to a maximum of 25% of the underlying index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.587725	2020-10-11 23:38:31.024353
EXAS	Exact Sciences Corporation	Exact Sciences Corp operates in the healthcare sector. It is a diagnostics company focused on the early detection and prevention of colorectal cancer.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{MYGN,GHDX,VRML,IVC,ATRS,GEN,VCYT,XLV}	Medical Diagnostics & Research	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:02.606059	2020-10-11 23:38:31.023428
EXPE	Expedia Group Inc.	Expedia Group Inc is an online travel company. It offers holiday packages, rental cars, cruises, as well as destination services and activities. The Core OTA segment generates maximum revenue for the company.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{CTRP,MMYT,TRIP,TZOO}	Travel & Leisure	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:02.622318	2020-10-11 23:38:31.02423
EYEG	Eyegate Pharmaceuticals Inc.	Eyegate Pharmaceuticals Inc focuses on developing and commercializing products for treating diseases and disorders of the eye.	{Healthcare,Biotechnology}	{FLKS,MYOS,KALV,SNOA,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:02.634078	2020-10-11 23:38:31.023007
EZT	Entergy Texas Inc First Mortgage Bonds 5.625% Series due June 1 2064		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:02.649889	2020-10-11 23:38:31.021426
FAM	First Trust/Aberdeen Global Opportunity Income Fund of Beneficial Interest	First Trust/aberdeen Global Oppo Income is a diversified, closed-end management investment company. The fund's primary investment objective is to seek a high level of current income, and to seek capital appreciation as a secondary objective.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.664126	2020-10-11 23:38:31.025871
FAST	Fastenal Company	Fastenal Co is an industrial supplies company. It distributes maintains, repairs, and operates supplies to industrial and construction customers. Its product portfolio includes OEM fasteners, safety products, tools and equipment.	{Industrials,"Industrial Distribution"}	{LAWS,MSM,AXE,WCC,GWW,HDS}	Industrial Distribution	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:02.67738	2020-10-11 23:38:31.031328
FB	Facebook Inc.	Facebook Inc is the world's largest online social network. Its products are Facebook, Instagram, Messenger, WhatsApp, and Oculus. Its products enable people to connect and share through mobile devices and personal computers.	{Technology,"Internet Content & Information","Online Media"}	{MSFT,IAC,TWTR,WB,GOOGL,RENN,MEET,XLK}	Online Media	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:02.690979	2020-10-11 23:38:31.02423
EVLMC	Eaton Vance TABS 5-to-15 Year Laddered Municipal B	The investment seeks current income exempt from regular federal income tax. \n Under normal market conditions, the fund invests at least 80% of its net assets (plus any borrowings for investment purposes) in municipal obligations with final maturities of between five and fifteen years, the interest on which is exempt from regular federal income tax (the “80% Policy”).	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.530919	2020-10-11 23:38:31.025871
EVSTC	Eaton Vance Stock NextShares	The investment seeks long-term capital appreciation by investing in a diversified portfolio of equity securities.\n Under normal market conditions, the fund invests at least 80% of its net assets (plus any borrowings for investment purposes) in a diversified portfolio of common stocks (the “80% Policy”). It may invest in companies with a broad range of market capitalizations, including smaller companies. The advisor generally intends to maintain investments in all or substantially all of the market sectors represented in the S&P 500 Index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.542855	2020-10-11 23:38:31.023428
EWD	iShares Inc MSCI Sweden	The investment seeks to track the investment results of the MSCI Sweden 25/50 Index.\n The fund will at all times invest at least 80% of its assets in the securities of its underlying index and in depositary receipts representing securities in its underlying index. The underlying index is designed to measure the performance of the large- and mid-cap segments of the Swedish market. A capping methodology is applied that limits the weight of any single component to a maximum of 25% of the underlying index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.555615	2020-10-11 23:38:31.023428
EWMC	Invesco S&P MidCap 400 Equal Weight	The investment seeks to track the investment results (before fees and expenses) of the S&P MidCap 400® Equal Weight Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities that comprise the underlying index. The underlying index is an equal-weighted version of the S&P MidCap 400® Index. The index provider compiles, maintains and calculates the underlying index, which consists of U.S. common equities listed on the NYSE and NASDAQ, and also may include equity interests in real estate investment trusts ("REITs") and business development companies ("BDCs"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.568126	2020-10-11 23:38:31.023428
EWT	iShares Inc MSCI Taiwan	The investment seeks to track the investment results of the MSCI Taiwan 25/50 Index.\n The fund will at all times invest at least 80% of its assets in the securities of its underlying index and in depositary receipts representing securities in its underlying index. The index is designed to measure the performance of the large- and mid-cap segments of the Taiwanese market. A capping methodology is applied that limits the weight of any single component to a maximum of 25% of the underlying index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.587669	2020-10-11 23:38:31.023428
EXC	Exelon Corporation	Exelon Corp is a power retailer engaged in the utilities sector of United States. The company involves in the power generation and transmission activities. The Generation segment generates maximum revenue for the company.	{Utilities,"Utilities - Diversified","Utilities - Regulated"}	{ETR,EIX,FE,AEE,PEG,D,PPL,NRG,NEE,XEL,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:02.605718	2020-10-11 23:38:31.028166
EXPD	Expeditors International of Washington Inc.	Expeditors International of Washington Inc is a third-party logistics provider focused on international freight forwarding. The firm operates more than 150 full-service office locations worldwide.	{Industrials,"Integrated Shipping & Logistics","Transportation & Logistics"}	{HUBG,CHRW,UPS,FDX,RLGT}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:02.622534	2020-10-11 23:38:31.024353
EYESW	Second Sight Medical Products Inc. Warrants		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:02.644263	2020-10-11 23:38:31.028166
FAAR	First Trust Alternative Absolute Return Strategy ETF	The investment seeks to provide investors with long-term total return.\n The fund is an actively managed exchange-traded fund that seeks to achieve long-term total return through long and short investments in exchange-traded commodity futures contracts ("Commodity Futures") through a wholly-owned subsidiary of the fund organized under the laws of the Cayman Islands (the "Subsidiary"). It will not invest directly in Commodity Futures. The advisor expects to gain exposure to these investments exclusively by investing in the Subsidiary. The Subsidiary is advised by First Trust Advisors L.P., the fund's investment advisor. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.655708	2020-10-11 23:38:31.028166
FAUS	First Trust Australia AlphaDex fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the NASDAQ AlphaDEX® Australia Index.\n The fund normally invests at least 90% of its net assets (including investment borrowings) in common stocks and/or depositary receipts that comprise the index. The index is designed to select stocks from the NASDAQ Australia Index (the "base index") that may generate positive alpha, or risk-adjusted returns, relative to traditional indices through the use of the AlphaDEX® selection methodology.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.682064	2020-10-11 23:38:31.021426
FBHS	Fortune Brands Home & Security Inc.	Fortune Brands Home & Security Inc is a leading home and security products company. It operates four business segments; through which it sells branded products such as cabinets and vanities, sells faucets, showers, locks and security devices, and doors.	{"Consumer Cyclical","Home Furnishings & Fixtures","Manufacturing - Apparel & Furniture"}	{AMWD,PATK,MAS,AWI}	Manufacturing - Apparel & Furniture	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:02.694336	2020-10-11 23:38:31.023007
EWO	iShares Inc MSCI Austria	The investment seeks to track the investment results of the MSCI Austria IMI 25/50 Index.\n The fund generally invests at least 90% of its assets in the securities of its underlying index and in depositary receipts representing securities in the index. The index is a free float-adjusted market capitalization-weighted index with a capping methodology applied to issuer weights so that no single issuer of a component exceeds 25% of the index weight, and all issuers with a weight above 5% do not cumulatively exceed 50% of the index weight. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.573236	2020-10-11 23:38:31.021426
EWZS	iShares MSCI Brazil Small-Cap ETF	The investment seeks to track the investment results of the MSCI Brazil Small Cap Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is a free float-adjusted market capitalization-weighted index designed to measure the performance of equity securities in the bottom 14% by market capitalization of equity securities listed on stock exchanges in Brazil.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.603457	2020-10-11 23:38:31.023007
EXK	Endeavour Silver Corporation (Canada)	Endeavour Silver Corp is a precious metals mining company. The company is primarily engaged in silver mining and owns three high grade, underground, silver-gold mines in Mexico.	{"Basic Materials",Silver,"Metals & Mining"}	{PAAS,MAG,SVM,AXU,CDE,HL,AG,GPL,WPM}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:02.614891	2020-10-11 23:38:31.025871
EXPO	Exponent Inc.	Exponent Inc is a science and engineering consulting firm. It provides the solution to the problems of automotive, aviation, chemical, construction, consumer products, government, and health industry.	{Industrials,"Business Services"}	{FORR,HURN,LUNA,HCKT,MMS,NCI,GPX,FCN,HIL,LTBR}	Business Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:02.625501	2020-10-11 23:38:31.028166
EYES	Second Sight Medical Products Inc.	Second Sight Medical Products Inc develops, manufactures and markets implantable prosthetic devices that can restore some functional vision to patients blinded by outer retinal degenerations, such as Retinitis Pigmentosa.	{Healthcare,"Medical Devices"}	{CUTR,IVC,XLV}	Medical Devices	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:02.63942	2020-10-11 23:38:31.02423
EZU	iShares MSCI Eurozone	The investment seeks to track the investment results of the MSCI EMU Index composed of large- and mid-capitalization equities from developed market countries that use the euro as their official currency.\n The fund generally invests at least 95% of its assets in the securities of its underlying index and in depositary receipts representing securities in its underlying index. The index consists of stocks from the following 10 developed market countries: Austria; Belgium; Finland; France; Germany; Ireland; Italy; the Netherlands; Portugal; and Spain.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:02.655195	2020-10-11 23:38:31.023428
FANZ	ProSports Sponsors	The investment seeks to provide investment results that, before fees and expenses, track the performance of the ProSports Sponsors Index.\n The fund will normally invest at least 80% of its total assets in securities of the index. The index is designed to measure the performance of companies which are official sponsors of the major professional football, baseball, hockey and basketball leagues in the United States ("ProSports Sponsors") or national sports broadcasters/internet streamers with rights agreements with such leagues ("ProSports Broadcasters").	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:02.671183	2020-10-11 23:38:31.02423
FBM	Foundation Building Materials Inc.	Foundation Building Materials Inc is a specialty distributor of wallboard and suspended ceiling systems in the United States and Canada. The company operates through Specialty Building Products and Mechanical Insulation segments.	{Industrials,"Industrial Distribution"}	{HDS,HBP,IBP,GMS}	Industrial Distribution	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:02.703662	2020-10-11 23:38:31.023007
FCAN	First Trust Canada AlphaDEX Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the NASDAQ AlphaDEX® Canada Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks and/or depositary receipts that comprise the index. The index is designed to select stocks from the NASDAQ Canada Index (the "base index") that may generate positive alpha, or risk-adjusted returns, relative to traditional indices through the use of the AlphaDEX® selection methodology.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.723144	2020-10-11 23:38:31.028166
FCF	First Commonwealth Financial Corporation	First Commonwealth Financial Corp provides banking services through the First Commonwealth Bank. It entails commercial and consumer banking products such as personal checking accounts, loans, wealth management and insurance.	{"Financial Services","Banks - Regional - US",Banks}	{DNBF,EMCF,BMTC,CCNE,FULT,PWOD,STBA,ASRV,FNB,PFIS}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.734876	2020-10-11 23:38:31.021426
FLDM	Fluidigm Corporation	Fluidigm Corp creates, manufactures, and markets technologies and life-science tools focused on the exploration and analysis of single cells, as well as the industrial application of genomics, based upon microfluidics and mass cytometry technologies.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{LMNX,ILMN,A,BIO,TMO,PKI,PACB,NSTG,XLV}	Medical Diagnostics & Research	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:03.035456	2020-10-11 23:38:31.023007
EWY	iShares Inc MSCI South Korea	The investment seeks to track the investment results of the MSCI Korea 25/50 Index.\n The fund will at all times invest at least 80% of its assets in the securities of its underlying index and in depositary receipts representing securities in its underlying index. The underlying index is a free float-adjusted market capitalization-weighted index with a capping methodology applied to issuer weights so that no single issuer of a component exceeds 25% of the underlying index weight, and all issuers with a weight above 5% do not cumulatively exceed 50% of the underlying index weight. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.595117	2020-10-11 23:38:31.024353
EXFO	EXFO Inc	EXFO Inc offers technology solutions for wireless and wireline network operators across the telecom industry. The company also provides test and service assurance solutions for 2G, 3G and 4G/long term evolution wireless networks.	{Technology,"Communication Equipment"}	{AWRE,RDCM,NTCT,PCTI,A,GLW,DHR,VIAV,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:02.613705	2020-10-11 23:38:31.031328
EXT	WisdomTree U.S. Total Earnings Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree U.S. Earnings Index.\n Under normal circumstances, at least 95% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a fundamentally weighted index that is comprised of earnings-generating companies within the broad U.S. stock market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.629835	2020-10-11 23:38:31.025871
EZM	WisdomTree U.S. MidCap Earnings Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree U.S. MidCap Earnings Index.\n Under normal circumstances, at least 95% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a fundamentally weighted index that is comprised of earnings-generating companies within the mid-capitalization segment of the U.S. stock market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.650368	2020-10-11 23:38:31.023007
FANG	Diamondback Energy Inc.	Diamondback Energy Inc is an independent oil and gas producer in the United States. The company engages in acquisition, development, exploration and exploitation of unconventional, onshore oil and natural gas reserves in the Permian Basin in West Texas.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{AREX,PXD,OXY,CXO,PE}	Oil & Gas - E&P	Energy	Nasdaq Global Select	\N	2020-10-11 23:12:02.668491	2020-10-11 23:38:31.031328
FAZ	Direxion Financial Bear 3X Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the inverse (or opposite) of the daily performance of the Russell 1000® Financial Services Index.\n The fund invests in swap agreements, futures contracts, short positions or other financial instruments that, in combination, provide inverse or short leveraged exposure to the index equal to at least 80% of the fund's net assets (plus borrowing for investment purposes). The index is a subset of the Russell 1000® Index that measures the performance of the securities classified in the financial services sector of the large-capitalization U.S. equity market. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.687653	2020-10-11 23:38:31.02582
FBR	Fibria Celulose S.A.	Fibria Celulose SA is a producer and distributor of eucalyptus pulp. The firm classifies its end products into three segments: Sanitary Paper, Printing and Writing, and Special Papers.	{"Basic Materials","Paper & Paper Products","Forest Products"}	{MERC,IP}	Forest Products	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:02.710215	2020-10-11 23:38:31.02423
FCN	FTI Consulting Inc.	FTI Consulting Inc along with its subsidiaries is a business advisory firm. It provides services including corporate finance/restructuring, forensic and litigation consulting, economic consulting, technology and strategic communications.	{Industrials,"Business Services"}	{RECN,HURN,CRAI,ICFI,III,HCKT,NCI}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:02.736187	2020-10-11 23:38:31.028166
FCSC	Fibrocell Science Inc.	Fibrocell Science Inc is a commercial-stage, autologous cellular therapeutic company that develops products for aesthetic, medical and scientific applications.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{XLV}	Drug Manufacturers	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:02.746901	2020-10-11 23:38:31.02423
FDL	First Trust Morningstar	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the Morningstar® Dividend Leaders IndexSM.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The objective of the index is to offer investors a benchmark for dividend portfolios as well as a means to invest in a portfolio of stocks that have a consistent record of growing dividends as well as the ability to sustain them. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.76148	2020-10-11 23:38:31.023007
FDVV	Fidelity High Dividend	The investment seeks to provide investment returns that correspond, before fees and expenses, generally to the performance of the Fidelity High Dividend Index℠.\n The fund normally invests at least 80% of assets in securities included in the underlying index and in depository receipts representing securities included in the underlying index. The underlying index is designed to reflect the performance of stocks of large and mid-capitalization high-dividend-paying companies that are expected to continue to pay and grow their dividends.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.795906	2020-10-11 23:38:31.023428
EXIV	VelocityShares 1x Daily Inverse VSTOXX Futures ETN	The investment seeks to link to the VSTOXX Short-Term Futures Inverse Investable Index (the “Inverse Investable Index”).\n The Inverse Investable Index replicates the performance of a short position in a portfolio of VSTOXX futures designed to provide a short exposure to constant-maturity one-month forward, one-month implied volatilities on the underlying EURO STOXX 50® Index, taking into account the bid-ask spread in the roll procedure and the daily reset costs, which will reduce the value of the Inverse Securities.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:02.615753	2020-10-11 23:38:31.023428
EXTN	Exterran Corporation	Exterran Corp provides compression, production, and processing products and services supporting production and transportation of oil and natural gas for integrated oil and natural gas companies, an independent oil and natural gas producers and processors.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{NOV,OIS}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:02.630521	2020-10-11 23:38:31.02582
EZA	iShares MSCI South Africa Index Fund	The investment seeks to track the investment results of the MSCI South Africa 25/50 Index.\n The fund normally invests at least 95% of its total assets in the securities of its underlying index and in depositary receipts representing securities in its underlying index. The underlying index uses a capping methodology to limit the weight of any single component to a maximum of 25% of the underlying index. The underlying index may include large-, mid- or small- capitalization companies. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.646717	2020-10-11 23:38:31.031328
FAF	First American Corporation (New)	First American Financial Corp is a financial service provider offering title insurance, closing and/or escrow services, and related services in connection with residential and commercial real estate transactions.	{"Financial Services","Insurance - Specialty"}	{ITIC,ORI,STC,ARGO}	Insurance - Specialty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.661089	2020-10-11 23:38:31.023007
FAT	FAT Brands Inc.	FAT Brands Inc is a multi-brand restaurant franchising company. It develops, markets, and acquires predominantly fast casual restaurant concepts. The company provides turkey burgers, chicken Sandwiches, chicken tenders, burgers, and others.	{"Consumer Cyclical",Restaurants}	{LUB}	Restaurants	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:02.67738	2020-10-11 23:38:31.023007
FBC	Flagstar Bancorp Inc.	Flagstar Bancorp, Inc. is a savings and loan holding company. The company through its subsidiaries offers commercial, small business, and consumer banking services mortgage and banking products to consumer, commercial, and government customers.	{"Financial Services","Savings & Cooperative Banks",Banks}	{FCAP,TFSL,HMST,WSBF}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.69358	2020-10-11 23:38:31.028166
FBP	First BanCorp.	First BanCorp is a financial holding company. Its operating segments include Commercial and Corporate Banking; Consumer (Retail) Banking; Mortgage Banking; Treasury and Investments; United States Operations; and Virgin Islands Operations.	{"Financial Services","Banks - Regional - US",Banks}	{BNS,FCNCA,NWBI,SGB,BOTJ,BPOP,OFG,UBSH}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.710504	2020-10-11 23:38:31.028166
FCB	FCB Financial Holdings Inc. Class A	FCB Financial Holdings Inc provides a range of financial services, especially banking services in the United States. Its list of services includes lending and deposits, wealth management, insurance and capital market operations.	{"Financial Services","Banks - Regional - US",Banks}	{PFBI,RBCAA,IBCP,STBZ}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.727572	2020-10-11 23:38:31.023428
FCO	Aberdeen Global Income Fund Inc.	Aberdeen Global Income Fund Inc is a closed-end, non-diversified management investment company. The fund's principal investment objective is to provide high current income by investing primarily in fixed income securities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:02.741185	2020-10-11 23:38:31.023007
FDC	First Data Corporation Class A	First Data Corp is a provider of electronic commerce and payment solutions for merchants, financial institutions and card issuers globally. It segments are Global Business Solutions, Global Financial Solutions, Network & Security Solutions.	{Industrials,"Business Services"}	{PFPT,PFM,PCYG}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:02.756699	2020-10-11 23:38:31.02582
FDS	FactSet Research Systems Inc.	FactSet Research Systems Inc is a financial and economic data and analytics service provider. It aggregates content from hundreds of databases, reformats it, and then furnishes the data to research analysts, and other financial professionals.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{TRI,MORN,MCO,MSCI,ENV,NWSA,FOX,SPGI}	Brokers & Exchanges	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.783517	2020-10-11 23:38:31.023007
FEIM	Frequency Electronics Inc.	Frequency Electronics provides precision time, frequency generation and synchronisation products and subsystems. Its products are used in on-board satellites and in ground-based communication stations.	{Technology,"Communication Equipment"}	{KVHI,CMTL,GD,NOC,TDY,COL,LLL,XLK}	Communication Equipment	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:02.805077	2020-10-11 23:38:31.023007
FLEU	Barclays ETN FI Enhanced Europe 50 ETN Series B	The investment seeks a return linked to a leveraged participation in the quarterly performance of the STOXX Europe 50® USD (Gross Return) Index (the "index").\nThe index is composed of 50 European blue-chip companies (the "index constituents") selected from within the STOXX Europe 600 Index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.0415	2020-10-11 23:38:31.025871
FAB	First Trust Multi Cap Value AlphaDEX Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of the Nasdaq AlphaDEX® Multi Cap Value Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is designed to select value stocks from the NASDAQ US 500 Large Cap Index, NASDAQ US 600 Mid Cap Index and NASDAQ US 700 Small Cap Index that may generate positive alpha, or risk-adjusted returns, relative to traditional indices through the use of the AlphaDEX® selection methodology.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.657512	2020-10-11 23:38:31.02423
FARO	FARO Technologies Inc.	Faro Technologies Inc designs, develops, manufactures, markets and supports software driven, three-dimensional (3-D) measurement, imaging and realization systems. Its business segments are Factory Metrology, Construction BIM-CIM and Other.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{FLIR,OSIS,MTSC,PRCP,TRMB,AME,GE,NOC,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:02.67554	2020-10-11 23:38:31.028166
FBIZ	First Business Financial Services Inc.	First Business Financial Services Inc is engaged in commercial banking business. It operates as business bank engaged in delivering commercial banking products, including commercial loans and commercial real estate loans.	{"Financial Services","Banks - Regional - US",Banks}	{WTFC,CVBF,FITB,SRCE,MBFI,PACW,WFC,TCF,ASB}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:02.699552	2020-10-11 23:38:31.025871
FBZ	First Trust Brazil AlphaDEX Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the NASDAQ AlphaDEX® Brazil Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks and/or depositary receipts that comprise the index. The index is designed to select stocks from the NASDAQ Brazil Index (the "base index") that may generate positive alpha, or risk-adjusted returns, relative to traditional indices through the use of the AlphaDEX® selection methodology.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.714591	2020-10-11 23:38:31.023007
FCCO	First Community Corporation	First Community Corp offers traditional banking products and services for professionals and small-to-medium-sized businesses, including consumer and commercial, mortgage, brokerage and investment and insurance services.	{"Financial Services","Banks - Regional - US",Banks}	{BKSC,SFST,STI,SNV}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:02.728491	2020-10-11 23:38:31.024353
FCOR	Fidelity Corporate Bond	The investment seeks a high level of current income.\n Normally, the fund invests at least 80% of its assets in investment-grade corporate bonds and other corporate debt securities and repurchase agreements for those securities. It is managed to have similar overall interest rate risk to the Bloomberg Barclays U.S. Credit Bond Index. The fund invests in lower-quality debt securities.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.742818	2020-10-11 23:38:31.024353
FDBC	Fidelity D & D Bancorp Inc.	Fidelity D & D Bancorp Inc is a bank holding company for the Fidelity Deposit and Discount Bank. It provides various financial services to individuals and corporate customers in Lackawanna and Luzerne Counties, Pennsylvania.	{"Financial Services","Banks - Regional - US",Banks}	{BDGE,EMCF,LARK,FNLC,BAC,CBU,PFIS,VLY}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:02.753521	2020-10-11 23:38:31.023007
FDMO	Fidelity Momentum Factor	The investment seeks to provide investment returns that correspond, before fees and expenses, generally to the performance of the Fidelity U.S. Momentum Factor Index℠.\n The fund normally invests at least 80% of assets in securities included in the Fidelity U.S. Momentum Factor Index℠, which is designed to reflect the performance of stocks of large and mid-capitalization U.S. companies that exhibit positive momentum signals. It may lend securities to earn income for the fund.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.765286	2020-10-11 23:38:31.02582
FDUS	Fidus Investment Corporation	Fidus Investment Corp is a closed-end non-diversified management investment company. It provides customised debt and equity financing solutions to lower middle-market companies.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:02.796178	2020-10-11 23:38:31.028166
FEM	First Trust Emerging Markets AlphaDEX Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the NASDAQ AlphaDEX® Emerging Markets Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks and/or depositary receipts that comprise the index. The index is designed to select stocks from the NASDAQ Emerging Markets Index (the "base index") that may generate positive alpha, or risk-adjusted returns, relative to traditional indices through the use of the AlphaDEX® selection methodology.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.807661	2020-10-11 23:38:31.028166
FEP	First Trust Europe AlphaDEX Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the NASDAQ AlphaDEX® Europe Index (the "index").\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks and/or depositary receipts that comprise the index. The index is designed to select stocks from the NASDAQ Europe Index (the "base index") that may generate positive alpha, or risk-adjusted returns, relative to traditional indices through the use of the AlphaDEX® selection methodology.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.819312	2020-10-11 23:38:31.025871
EZPW	EZCORP Inc.	EZCORP Inc is the provider of pawn loans in the United States and Mexico. The Company's segments are U.S. Pawn, Mexico Pawn, and Other International.	{"Financial Services","Credit Services"}	{AMRK,ENVA,FCFS}	Credit Services	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:02.649921	2020-10-11 23:38:31.024353
FAX	Aberdeen Asia-Pacific Income Fund Inc	Aberdeen Asia-pacific Income Fund Inc is a United States based closed-end, non-diversified management investment company. Its principal investment objective is to seek current income along with capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:02.682271	2020-10-11 23:38:31.024353
FBIOP	Fortress Biotech Inc. 9.375% Series A Cumulative Redeemable Perpetual Preferred Stock	Fortress Biotech Inc is a biopharmaceutical company. The firm is involved in the business of acquiring, developing and commercializing pharmaceutical and biotechnology products.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:02.695913	2020-10-11 23:38:31.028166
FBSS	Fauquier Bankshares Inc.	Fauquier Bankshares Inc is a registered bank holding company. It owns The Fauquier Bank which provides a range of consumer and commercial banking services to individuals, businesses, and industries.	{"Financial Services","Banks - Regional - US",Banks}	{SONA,BOTJ,ESXB,UBSH,VBFC}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:02.711603	2020-10-11 23:38:31.023428
FCBC	First Community Bankshares Inc.	First Community Bancshares Inc is a financial holding company. It provides commercial banking products and services through First Community Bank including demand deposit accounts, savings and money market accounts, loans and other lines of credit.	{"Financial Services","Banks - Regional - US",Banks}	{TOWN,SRCE,TRMK,UBSI,SMMF,RNST,BBT,STI,SNV,BXS}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:02.727952	2020-10-11 23:38:31.023007
FCNCA	First Citizens BancShares Inc. Class A Common Stock	First Citizens BancShares Inc (DE) is a part of the financial sector in the United States. FCB provides a wide range of retail and commercial banking services, including traditional lending and deposit-taking.	{"Financial Services","Banks - Regional - US",Banks}	{BOKF,CBSH,PEBK,FBNC,BBT,WFC,STI,TCF}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:02.740057	2020-10-11 23:38:31.023428
FDIV	First Trust Strategic Income ETF	The investment seeks risk-adjusted income; capital appreciation is the secondary objective.\n The fund is a multi-manager, multi-strategy actively managed ETF. Its investment categories will be: (i) high yield corporate bonds, and first lien senior secured floating rate bank loans; (ii) mortgage-related investments; (iii) preferred securities; (iv) international sovereign bonds, including emerging markets debt; (v) equity securities of Energy Infrastructure Companies, certain of which are referred to as master limited partnerships; and (vi) dividend paying U.S. exchange-traded equity securities of companies and depositary receipts. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.757863	2020-10-11 23:38:31.021426
FEDU	Four Seasons Education (Cayman) Inc. American Depositary Shares each two ADSs representing one	Four Seasons Education (Cayman) Inc is engaged in provision of after-school education services for kindergarten, elementary and middle school students in the People's Republic of China.	{"Consumer Defensive","Education & Training Services",Education}	{LRN,HMHC,GHC}	Education	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:02.798919	2020-10-11 23:38:31.02423
FENC	Fennec Pharmaceuticals Inc.	Fennec Pharmaceuticals Inc is a clinical-stage biotechnology company. It is primarily engaged in research and development activities.	{Healthcare,Biotechnology}	{AMGN,GSK,BMY,JNJ,LLY,PFE,SNY,INSY,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:02.814287	2020-10-11 23:38:31.023428
FFBC	First Financial Bancorp.	First Financial Bancorp operates in the financial services sector. As a banking company, it conducts retail and commercial banking and commercial finance operations. It mainly caters to real estate sector with real estate loans.	{"Financial Services","Banks - Regional - US",Banks}	{PEBO,UBCP,OVBC,UBOH,HBAN,WSBC,PRK,LCNB,FMNB,SBFG}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:02.831878	2020-10-11 23:38:31.025871
FFHL	Fuwei Films (Holdings) Co. Ltd.	Fuwei Films (Holdings) Co Ltd along with its subsidiaries is engaged in the production and distribution of BOPET film, high quality plastic film widely used in packaging, imaging, electronics, electrical and magnetic products in China.	{"Consumer Cyclical","Packaging & Containers"}	{RAVN,CXDC,TG}	Packaging & Containers	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:02.842355	2020-10-11 23:38:31.031328
FFR	First Trust FTSE EPRA/NAREIT Global Real Estate Index Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the FTSE EPRA/NAREIT Developed Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks or in depositary receipts representing securities in the index. The index is modified market cap weighted based on free float market capitalization and tracks the performance of listed real estate companies or real estate investment trusts ("REITs") in North America, Europe and Asia.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.853728	2020-10-11 23:38:31.023428
FARM	Farmer Brothers Company	Farmer Bros Co is a national coffee roaster, wholesaler and distributor of coffee, tea and culinary products serving restaurant and convenience store chains, hotels, casinos, hospitals, and gourmet coffee houses, and grocery chains.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{JVA,SJM,THS,KHC}	Consumer Packaged Goods	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:02.676634	2020-10-11 23:38:31.02582
FBIO	Fortress Biotech Inc.	Fortress Biotech Inc is a biopharmaceutical company. The firm is involved in the business of acquiring, developing and commercializing pharmaceutical and biotechnology products.	{Healthcare,Biotechnology}	{CELG,BIIB,NVS,ABT,JNJ,SNY,CCXI,TEVA,SNSS,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:02.694635	2020-10-11 23:38:31.021426
FCAU	Fiat Chrysler Automobiles N.V.	Fiat Chrysler Automobiles NV engages in the designing, engineering, manufacturing and selling of passenger cars, light commercial vehicles, components and production systems worldwide. It manufactures automotive parts, engines, and industrial robots.	{"Consumer Cyclical","Auto Manufacturers",Autos}	{TSLA,PII,F,GM}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:02.723127	2020-10-11 23:38:31.028166
FCFS	FirstCash Inc.	FirstCash Inc is an operator of retail-based pawn stores in the U.S. and Latin America. It is engaged in operation of full-service pawn stores which make small pawn loans secured by personal property.	{"Financial Services","Credit Services"}	{EZPW,WINA,V}	Credit Services	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:02.734885	2020-10-11 23:38:31.031328
FCT	First Trust Senior Floating Rate Income Fund II of Beneficial Interest	First Trust Senior Floating Rate Income Fund II is a diversified, closed-end management investment company. Its investment objective is to seek a high level of current income. As a secondary objective, the fund attempts to preserve capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.746949	2020-10-11 23:38:31.028166
FDEU	First Trust Dynamic Europe Equity Income Fund of Beneficial Interest	First Trust Dynamic Europe Equity Income Fund is a non-diversified, closed-end management investment company. The fund's investment objective is to provide a high level of current income with a secondary focus on capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.757926	2020-10-11 23:38:31.024353
FDP	Fresh Del Monte Produce Inc.	Fresh Del Monte Produce Inc is a vertically integrated producers, marketers and distributors of fresh and fresh-cut fruit and vegetables. It is also produces and distributes prepared fruit and vegetables, juices, beverages and snacks.	{"Consumer Defensive","Farm Products","Consumer Packaged Goods"}	{LMNR,SAFM,CVGW,SJM}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:02.775659	2020-10-11 23:38:31.025871
FEI	First Trust MLP and Energy Income Fund of Beneficial Interest	First Trust MLP and Energy Income Fund is an asset management company. The company invests mainly in energy-related infrastructure assets such as pipelines, power transmission, and distribution.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.805811	2020-10-11 23:38:31.025871
FEU	SPDR DJ STOXX 50 Etf	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the STOXX® Europe 50 Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. It employs a sampling strategy, which means that the fund is not required to purchase all of the securities represented in the index. The index is a market capitalization weighted index designed to represent the performance of some of the largest companies across all components of the 19 STOXX Europe 600 Supersector Indexes. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.821059	2020-10-11 23:38:31.024353
FFBW	FFBW Inc.	FFBW Inc is a United States based community bank. It offers banking and financial products and services such as checking accounts, savings accounts, credit cards, home equity loans, mortgage center and others.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:02.839158	2020-10-11 23:38:31.02423
FFTG	FormulaFolios Tactical Growth	The investment seeks long-term total return.\n The fund is an actively managed exchange traded fund ("ETF") that is a fund of funds. It seeks to achieve its investment objective by investing primarily in foreign and domestic growth-oriented equity securities of any market capitalization, domestic investment grade fixed income securities (bonds) of any maturity or duration, domestic real estate investment trusts ("REITs"), and commodities (gold) securities through unaffiliated ETFs.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:02.854569	2020-10-11 23:38:31.031328
FGBI	First Guaranty Bancshares Inc.	First Guaranty Bancshares Inc is a bank holding company. The company, along with its subsidiary, provides personalized commercial banking services to businesses, professionals and individuals.	{"Financial Services","Banks - Regional - US",Banks}	{CZNC,UBSH}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:02.867395	2020-10-11 23:38:31.02423
FC	Franklin Covey Company	Franklin Covey Co is a performance improvement company. It provides training & consulting services to both organizations & individuals. The company also provide clients with training in management skills, relationship skills & individual effectiveness.	{"Consumer Defensive","Education & Training Services",Education}	{}	Education	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:02.715606	2020-10-11 23:38:31.025871
FCG	First Trust Natural Gas	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the ISE-Revere Natural GasTM Index.\n The fund will normally invest at least 90% of its net assets in equity securities that comprise the index. The index uses a linear-based capitalization-weighted methodology for each of the MLP and non-MLP group of constituents that initially ranks the equity securities based on market capitalization and average daily trading volume, and then adjusts the combined rankings of each equity security by a factor relating to its market capitalization.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.736147	2020-10-11 23:38:31.028166
FCX	Freeport-McMoRan Inc.	Freeport-McMoRan Inc is a mining company. It operates large, long-lived geographically diverse assets with reserves of copper, gold, and molybdenum. It also has mining operations in the Americas.	{"Basic Materials",Copper,"Metals & Mining"}	{TECK,BBL,RIO,APC,NEM,BHP,SCCO,VALE}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:02.747719	2020-10-11 23:38:31.023428
FDLO	Fidelity Low Volatility Factor	The investment seeks to provide investment returns that correspond, before fees and expenses, generally to the performance of the Fidelity U.S. Low Volatility Factor Index℠.\n The fund normally invests at least 80% of assets in securities included in the Fidelity U.S. Low Volatility Factor Index℠, which is designed to reflect the performance of stocks of large and mid-capitalization U.S. companies with lower volatility than the broader market. It may lend securities to earn income for the fund.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.760668	2020-10-11 23:38:31.02423
FELP	Foresight Energy LP representing Limited Partner Interests	Foresight Energy LP is a producer and marketer of thermal coal. It sells coal mainly to electric utility companies in the United States.	{"Basic Materials",Coal}	{ARLP,NRP,CNX}	Coal	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:02.80685	2020-10-11 23:38:31.031328
FEO	First Trust/Aberdeen Emerging Opportunity Fund of Beneficial Interest	First Trust/aberdeen Emerging Opp Fund is a United States based diversified, closed-end management investment company. Its investment objective is to provide a high level of total return.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.816432	2020-10-11 23:38:31.031328
FEX	First Trust Large Cap Core AlphaDEX Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the Nasdaq AlphaDEX® Large Cap Core Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is designed to select stocks from the NASDAQ US 500 Large Cap Index (the "base index") that may generate positive alpha, or risk-adjusted returns, relative to traditional indices through the use of the AlphaDEX® selection methodology.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.828053	2020-10-11 23:38:31.028166
FFG	FBL Financial Group Inc.	FBL Financial Group Inc sells individual life insurance and annuity products principally under the consumer brand name Farm Bureau Financial Services.	{"Financial Services","Insurance - Life"}	{UNM,CIA,CNO}	Insurance - Life	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.840544	2020-10-11 23:38:31.028166
FFNW	First Financial Northwest Inc.	First Financial Northwest Inc is a bank holding company for First Savings Bank Northwest. The bank's business consists of attracting deposits from the public and utilizing these funds commercial real estate, construction, business and consumer loans.	{"Financial Services","Banks - Regional - US",Banks}	{WAFD,PROV,RVSB,TSBK,BYFC,TBNK,ANCB,FSBW,BANC}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:02.851468	2020-10-11 23:38:31.02423
FGD	First Trust DJ Global Select Dividend	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the Dow Jones Global Select Dividend IndexSM.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks or in depositary receipts representing securities in the index. The index is an indicated annual dividend yield weighted index of 100 stocks selected from the developed-market portion of the Dow Jones World IndexSM.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.869157	2020-10-11 23:38:31.023428
FG	FGL Holdings	FGL Holdings is a Cayman-based holding company. Its principal operations are conducted through life insurance and annuity products. Through its holdings it offers annuity and life insurance service, and reinsurer services in Bermuda and USA.	{"Financial Services","Insurance - Life"}	{}	Insurance - Life	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.862595	2020-10-11 23:38:31.025871
FI	Frank's International N.V.	Frank's International NV provides tubular services, tubular fabrication and specialty well construction and well intervention solutions to both offshore and onshore exploration and production companies.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{SPN,NOV,DRQ,GE,TTI,RDS.A,WFT,BHGE}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:02.885593	2020-10-11 23:38:31.02582
FCEF	First Trust CEF Income Opportunity ETF	The investment seeks to provide current income with a secondary emphasis on total return.\n The fund will seek to achieve its investment objectives by investing at least 80% of its net assets (including investment borrowings) in a portfolio of closed-end investment companies that are listed and traded in the United States on registered exchanges ("closed-end funds"). In selecting the closed-end funds in which the fund will invest ("underlying funds"), the fund's investment advisor will utilize a range of investment approaches. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.729238	2020-10-11 23:38:31.02582
FCOM	Fidelity MSCI Telecommunication Services Index	The investment seeks to provide investment returns that correspond, before fees and expenses, generally to the performance of the MSCI USA IMI Telecommunication Services 25/50 Index.\n The fund invests at least 80% of assets in securities included in the fund's underlying index. The fund's underlying index is the MSCI USA IMI Telecommunication Services 25/50 Index, which represents the performance of the telecommunication services sector in the U.S. equity market. It may or may not hold all of the securities in the MSCI USA IMI Telecommunication Services 25/50 Index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.743181	2020-10-11 23:38:31.025871
FDD	First Trust Dow Jones STOXX Select Dividend 30 Index Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the STOXX® Europe Select Dividend 30 Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks or in depositary receipts representing securities in the index. The index consists of 30 high dividend-yielding securities selected from the STOXX® Europe 600 Index, including secondary lines of those companies (where there are multiple lines of equity capital in a company).	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.75828	2020-10-11 23:38:31.025871
FDX	FedEx Corporation	FedEx Corp is the world's largest express courier delivery firm. The company provides courier services through various divisions which include express, ground, office, freight, cross border, trade networks and supply chain.	{Industrials,"Integrated Shipping & Logistics","Transportation & Logistics"}	{AIRT,EXPD,CHRW,ATSG,UPS}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:02.797098	2020-10-11 23:38:31.02582
FEMS	First Trust Emerging Markets Small Cap AlphaDEX Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the NASDAQ AlphaDEX® Emerging Markets Small Cap Index.\n The fund will normally invest at least 90% of its net assets in common stocks and/or depositary receipts that comprise the index. The index is designed to select small cap stocks from the NASDAQ Emerging Markets Index (the "base index") that may generate positive alpha, or risk-adjusted returns, relative to traditional indices through the use of the AlphaDEX® selection methodology.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.810107	2020-10-11 23:38:31.021426
FEZ	SPDR DJ Euro STOXX 50 Etf	The investment seeks investment results that, before fees and expenses, correspond generally to the total return performance of the EURO STOXX 50® Index.\n The fund employs a sampling strategy, which means that the fund is not required to purchase all of the securities represented in the index. It generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index is designed to represent the performance of some of the largest companies across components of the 19 EURO STOXX Supersector Indexes. The EURO STOXX Supersector Indexes are subsets of the EURO STOXX Index. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.830152	2020-10-11 23:38:31.023428
FFIU	Fieldstone UVA Unconstrained Medium-Term Fixed Income	The investment seeks current income with limited risk to principal.\n The fund seeks to achieve its investment objective of current income by investing principally in fixed income securities of any kind with a dollar-weighted average effective duration of between three and seven years. Under normal market conditions, Sub-Adviser intends to invest at least 80% of its net assets (plus the amount of borrowings for investment purposes) in such securities. Fixed income securities include bonds, debt securities, and income-producing instruments of any kind issued by governmental or private-sector entities.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.847477	2020-10-11 23:38:31.025871
FGEN	FibroGen Inc	FibroGen Inc is a biopharmaceutical company. It is engaged in the discovery, development and commercialization of novel therapeutics to treat serious unmet medical needs.	{Healthcare,Biotechnology}	{ILMN,HBIO,AXDX,XLRN,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:02.867757	2020-10-11 23:38:31.028166
FGP	Ferrellgas Partners L.P.	Ferrellgas Partners LP is engaged in the retail distribution of propane and related equipment and midstream operations, which include crude oil logistics.	{Energy,"Oil & Gas Refining & Marketing","Oil & Gas - Refining & Marketing"}	{MMLP,CLNE,APU,LNG,GLP,SPH,NGL,SRLP,CAPL,CEQP}	Oil & Gas - Refining & Marketing	Energy	New York Stock Exchange	\N	2020-10-11 23:12:02.875749	2020-10-11 23:38:31.023007
FHN	First Horizon National Corporation	First Horizon National Corp is engaged in providing financial products and services, including traditional lending and deposit taking, to consumer and commercial customers.	{"Financial Services","Banks - Regional - US",Banks}	{PNFP,BBT,PNC,STI,SNV}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.885584	2020-10-11 23:38:31.028166
FIF	First Trust Energy Infrastructure Fund of Beneficial Interest	First Trust Energy Infrastructure Fund is a non-diversified, closed-end management investment company. Its investment objective is to seek a high level of total return with an emphasis on current distributions paid to shareholders.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.910582	2020-10-11 23:38:31.025871
FCCY	1st Constitution Bancorp (NJ)	1st Constitution Bancorp is a commercial bank. It provides a range of commercial and consumer banking services to individuals, families, institutions and small businesses in central and coastal New Jersey and the Fort Lee area of Bergen County.	{"Financial Services","Banks - Regional - US",Banks}	{UNTY,PROV,PGC,LBAI,BCBP,PKBK,BKJ,TRCB}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:02.728168	2020-10-11 23:38:31.025871
FCRE	FC Global Realty Incorporated	FC Global Realty Inc is a real estate investment company. It is focused on holding investments in a diversified portfolio of commercial and residential real estate properties and other real estate investments.	{"Real Estate","Real Estate - General","Real Estate Services"}	{}	Real Estate Services	Real Estate	OTC Pink	\N	2020-10-11 23:12:02.746193	2020-10-11 23:38:31.021426
FDT	First Trust Developed Markets Ex-US AlphaDEX Fund	The investment seeks results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the NASDAQ AlphaDEX® Developed Markets Ex-US Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks and/or depositary receipts that comprise the index. The index is designed to select stocks from the NASDAQ Developed Markets Ex-US Index that may generate positive alpha, or risk-adjusted returns, relative to traditional indices through the use of the AlphaDEX® selection methodology.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.796324	2020-10-11 23:38:31.024353
FENG	Phoenix New Media Limited American Depositary Shares each representing 8 Class A.	Phoenix New Media Ltd is a media company providing premium content on an integrated platform across internet, mobile and TV channels in China. The company organizes its operations into two main segments: Net Advertising Services and Paid Services.	{Technology,"Internet Content & Information","Online Media"}	{SINA,BIDU,CHL,WB,XLK}	Online Media	Technology	New York Stock Exchange	\N	2020-10-11 23:12:02.818263	2020-10-11 23:38:31.02423
FFA	First Trust Enhanced Equity Income Fund	First Trust Enhanced Equity Income Fund is a diversified, closed-end management investment company. Its investment objective is to provide a high level of current income and gains and, to a lesser extent, capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.831685	2020-10-11 23:38:31.023007
FFIC	Flushing Financial Corporation	Flushing Financial Corp attracts retail deposits from the general public and invests those deposits together with funds generated from ongoing operations and borrowings.	{"Financial Services","Savings & Cooperative Banks",Banks}	{DCOM,ESBK,GCBC,LSBK,CARV,NFBK}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:02.845797	2020-10-11 23:38:31.02582
FFWM	First Foundation Inc.	First Foundation Inc is a financial services company. It focuses on providing investment management, wealth planning, consulting, banking products and services, life insurance services, property and casualty insurance services to their clients.	{"Financial Services","Banks - Regional - US",Banks}	{BHB,BWFG}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:02.864233	2020-10-11 23:38:31.021426
FGM	First Trust Germany AlphaDEX Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the NASDAQ AlphaDEX® Germany Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks and/or depositary receipts that comprise the index. The index is designed to select stocks from the NASDAQ Germany Index (the "base index") that may generate positive alpha, or risk-adjusted returns, relative to traditional indices through the use of the AlphaDEX® selection methodology.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.871485	2020-10-11 23:38:31.031328
FIDI	Fidelity International High Dividend	The investment seeks to provide investment returns that correspond, before fees and expenses, generally to the performance of the Fidelity ® International High Dividend Index.\n The fund normally invests at least 80% of assets in securities included in the Fidelity ® International High Dividend Index and in depository receipts representing securities included in the index. The Fidelity ® International High Dividend Index is designed to reflect the performance of stocks of large- and mid-capitalization developed international high dividend-paying companies that are expected to continue to pay and grow their dividends.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.898481	2020-10-11 23:38:31.023007
FICO	Fair Isaac Corproation	Fair Isaac Corp is a provider of analytic, software and data management products and services that enables businesses to automate, improve and connect decisions. It operates in three segments: applications, scores, and tools.	{Technology,"Software - Infrastructure","Application Software"}	{TRI,PEGA,FISV,INTX,ACIW,VRSK,EFX,IBM,CLGX,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:02.895659	2020-10-11 23:38:31.02423
FIHD	UBS AG FI Enhanced Global High Yield ETN due March 3 2026	The investment seeks a return linked to the MSCI World High Dividend Yield USD Gross Total Return Index. \n The ETN is a series of FI Enhanced ETNs. The Securities are senior unsecured debt securities issued by UBS AG. The Securities are designed to provide a two times leveraged long exposure to the performance of the index compounded on a quarterly basis, reduced by the Accrued Fees. The index is designed to track the performance of large- and mid-cap stocks (excluding REITS) across 23 developed markets countries tracked by the MSCI World Index with higher than average dividend yields that are potentially both sustainable and persistent.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.913363	2020-10-11 23:38:31.028166
FCEL	FuelCell Energy Inc.	FuelCell Energy Inc is an integrated fuel cell company. It designs, manufactures, sells, installs, operates and services ultra-clean, efficient stationary fuel cell power plants for distributed power generation.	{Utilities,"Utilities - Independent Power Producers"}	{BLDP,EIX,APD,PH,PKX,NRG,PLUG,SPWR,ESNC,XLU}	Utilities - Independent Power Producers	Utilities	NASDAQ Global Market	\N	2020-10-11 23:12:02.730245	2020-10-11 23:38:31.02423
FCPT	Four Corners Property Trust Inc.	Four Corners Property Trust Inc is a real estate investment trust. It focuses its efforts on investing primarily in restaurants based in the United States.	{"Real Estate","REIT - Hotel & Motel",REITs}	{NYMT,O,NTP}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:02.745078	2020-10-11 23:38:31.02582
FDEF	First Defiance Financial Corp.	First Defiance Financial Corp operates a unitary thrift holding company that provides communities based financial services in northwest Ohio, northeast Indiana, and southeastern Michigan.	{"Financial Services","Savings & Cooperative Banks",Banks}	{UCFC,FCAP,CZWI,TFSL,FBC,WAYN}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:02.756708	2020-10-11 23:38:31.031328
FELE	Franklin Electric Co. Inc.	Franklin Electric Co Inc along with its subsidiaries designs manufactures and distributes water and fuel pumping systems, composed of submersible motors, pumps, electronic controls and related parts and equipment.	{Industrials,"Diversified Industrials","Industrial Products"}	{AMOT,FLS,AME,DOV,GRC,RBC,AOS,CFX,SVT,XYL}	Industrial Products	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:02.811115	2020-10-11 23:38:31.024353
FF	FutureFuel Corp.	FutureFuel Corp through its subsidiary manufactures diversified chemical products, bio-based products comprised of biofuels, and bio-based specialty chemical products. The company's two reporting segments include chemicals and biofuels.	{"Basic Materials",Chemicals}	{GPRE,ADM,HON,REGI}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:02.830341	2020-10-11 23:38:31.031328
FFIV	F5 Networks Inc.	F5 Networks Inc provides software-defined application services designed to ensure that applications delivered over Internet Protocol networks are available to any user, anywhere, anytime, on any device and on any network.	{Technology,"Software - Infrastructure","Application Software"}	{CSCO,RDWR,ALLT,JNPR,RHT,RNWK,ATEN,IMPV,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:02.848309	2020-10-11 23:38:31.024353
FHK	First Trust Hong Kong AlphaDEX Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the NASDAQ AlphaDEX® Hong Kong Index.\n The fund normally invests at least 90% of its net assets (including investment borrowings) in common stocks and/or depositary receipts that comprise the index. The index is designed to select stocks from the NASDAQ Hong Kong Index (the "base index") that may generate positive alpha, or risk-adjusted returns, relative to traditional indices through the use of the AlphaDEX® selection methodology.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.878371	2020-10-11 23:38:31.028166
FHLC	Fidelity MSCI Health Care Index	The investment seeks to provide investment returns that correspond, before fees and expenses, generally to the performance of the MSCI USA IMI Health Care Index.\n The fund invests at least 80% of assets in securities included in the fund's underlying index. The fund's underlying index is the MSCI USA IMI Health Care Index, which represents the performance of the health care sector in the U.S. equity market. It may or may not hold all of the securities in the MSCI USA IMI Health Care Index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.885773	2020-10-11 23:38:31.021426
FIDU	Fidelity MSCI Industrials Index	The investment seeks to provide investment returns that correspond, before fees and expenses, generally to the performance of the MSCI USA IMI Industrials Index.\n The fund invests at least 80% of assets in securities included in the fund's underlying index. The fund's underlying index is the MSCI USA IMI Industrials Index, which represents the performance of the industrial sector in the U.S. equity market. It may or may not hold all of the securities in the MSCI USA IMI Industrials Index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.910779	2020-10-11 23:38:31.028166
FIEG	FI Enhanced Gloabl High Yield ETN	The investment seeks a return Linked to the MSCI World High Dividend Yield USD Gross Total Return Index due October 12, 2023 (the “securities”).	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.910412	2020-10-11 23:38:31.021426
FILL	iShares MSCI Global Energy Producers Fund	The investment seeks to track the investment results of the MSCI ACWI Select Energy Producers Investable Market Index (IMI).\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index measures the combined performance of equity securities of companies in both developed and emerging markets that are primarily engaged in the business of energy exploration and production. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.915609	2020-10-11 23:38:31.02582
FINU	ProShares UltraPro Financial Select Sector	The investment seeks daily investment results, before fees and expenses, that correspond to three times (3x) the daily performance of the S&P Financial Select Sector Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index represents the financial industry group of the S&P 500 Index ("S&P 500"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.918267	2020-10-11 23:38:31.023007
FDIS	Fidelity MSCI Consumer Discretionary Index	The investment seeks to provide investment returns that correspond, before fees and expenses, generally to the performance of the MSCI USA IMI Consumer Discretionary Index.\n The fund invests at least 80% of assets in securities included in the fund's underlying index. The fund's underlying index is the MSCI USA IMI Consumer Discretionary Index, which represents the performance of the consumer discretionary sector in the U.S. equity market. It may or may not hold all of the securities in the MSCI USA IMI Consumer Discretionary Index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.757345	2020-10-11 23:38:31.028166
FDN	First Trust DJ Internet Index Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the Dow Jones Internet Composite Index (SM) (the "index").\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is designed to include only companies whose primary focus is Internet-related. To be eligible for inclusion in the index, a company must generate at least 50% of its revenues from Internet commerce or services. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.768971	2020-10-11 23:38:31.028166
FE	FirstEnergy Corp.	FirstEnergy Corp is a diversified energy company. It is involved in the generation, transmission and distribution of electricity. It operates in segments such as Regulated Distribution, Regulated Transmission and CES.	{Utilities,"Utilities - Diversified","Utilities - Regulated"}	{ED,AEP,PEG,SO,EXC,D,PPL,NRG,NEE,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:02.797098	2020-10-11 23:38:31.028166
FEN	First Trust Energy Income and Growth Fund	First Trust Energy Income & Growth Fund is a non-diversified, closed-end management investment company in US. Its investment objectives are to seek a high level of after-tax total return with an emphasis on current distributions paid to shareholders.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:02.813641	2020-10-11 23:38:31.02582
FEUZ	First Trust Eurozone AlphaDEX ETF	The investment seeks investment results that correspond generally to the price and yield of an index called the NASDAQ® AlphaDEX® Eurozone Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks and/or depositary receipts that comprise the index. The index is designed to select stocks from the NASDAQ Eurozone Index that may generate positive alpha, or risk-adjusted returns, relative to traditional passive-style indices through the use of the AlphaDEX® selection methodology.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.82937	2020-10-11 23:38:31.021426
FFSG	FormulaFolios Smart Growth	The investment seeks to provide capital growth.\n The fund seeks to achieve its investment objective by investing through other unaffiliated ETFs primarily in domestic and foreign growth-oriented equity securities of any market capitalization and US Treasuries or other cash equivalents. It will generally be 100% invested in growth-oriented equity ETFs when the adviser's investment models indicate a bullish trend for the equity market, and will generally be 50% invested in growth-oriented equity ETFs and 50% invested in US treasuries and/or other cash equivalents to hedge risk when the adviser's models indicate a bearish trend for the equity markets.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:02.85437	2020-10-11 23:38:31.028166
FGB	First Trust Specialty Finance and Financial Opportunities Fund	First Trust Specialty Fnc & Fncl Oppor is a non-diversified, closed-end management investment company. The fund's primary investment objective is to seek a high level of current income. It seeks positive total return as a secondary objective.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.866806	2020-10-11 23:38:31.02582
FIBK	First Interstate BancSystem Inc.	First Interstate BancSystem Inc is a financial and bank holding company. It delivers a range of banking products and services to individuals, businesses, municipalities and other entities.	{"Financial Services","Banks - Regional - US",Banks}	{HTLF,GBCI,ZION,WFC,TCF,USB,WAL,NBHC,GBNK}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:02.886445	2020-10-11 23:38:31.031328
FIEE	UBS AG FI Enhanced Europe 50 ETN due February 12 2026	The investment seeks to provide a two times leveraged long exposure to the performance of the index compounded on a quarterly basis, reduced by the Accrued Fees.\n The UBS AG FI Enhanced Europe 50 ETN are a series of FI Enhanced ETNs linked to the STOXX Europe 50® USD (Gross Return) Index. The Securities are senior unsecured debt securities issued by UBS AG (UBS). The index is derived from the STOXX Europe 600 Index (the “parent index”), which is further subdivided into 19 STOXX Regional Total Market Index (TMI) Supersector Indices (the “Supersector Indices”). The index is composed of 50 European blue-chip companies from within the parent index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.907924	2020-10-11 23:38:31.024353
FII	Federated Investors Inc.	Federated Investors Inc provides investment management products and related financial services. The company sponsors, markets and provides investment-related services to various investment products, including mutual funds and Separate Accounts.	{"Financial Services","Asset Management"}	{GROW,SEIC,TROW,VRTS,WDR,EV,BEN,LM,MN}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.913542	2020-10-11 23:38:31.031328
ACGL	Arch Capital Group Ltd.	Arch Capital Group Ltd provides insurance, reinsurance and mortgage. It provides property, casualty and mortgage insurance and reinsurance lines, and focus on writing specialty lines of insurance and reinsurance.	{"Financial Services","Insurance - Diversified",Insurance}	{WRB,MKL,RE,AXS,CB}	Insurance	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:11:59.400433	2020-10-11 23:38:31.02582
ACMR	ACM Research Inc.	ACM Research Inc develops, manufactures and sells single-wafer wet cleaning equipment which is used for semiconductor industry. Its product is designed to remove random defects from a wafer surface effectively, without damaging a wafer or its features.	{Technology,"Semiconductor Equipment & Materials",Semiconductors}	{LRCX,XLK}	Semiconductors	Technology	NASDAQ Global Market	\N	2020-10-11 23:11:59.419677	2020-10-11 23:38:31.028166
ACT	AdvisorShares Vice ETF	The investment seeks long-term capital appreciation.\n The fund is an actively-managed exchange-traded fund that invests at least 80% of its net assets (plus any borrowings for investment purposes) in securities of companies that derive at least 50% of their net revenue from tobacco and alcoholic beverages and companies that derive at least 50% of their net revenue from the marijuana and hemp industry or have at least 50% of their company assets dedicated to lawful research and development of cannabis or cannabinoid-related products. It will invest primarily in U.S. exchange listed equity securities, including common and preferred stock and ADRs.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:11:59.431302	2020-10-11 23:38:31.02423
ADBE	Adobe Inc.	Adobe Systems Inc offers a line of software and services for content creation and the measurement of digital advertising and marketing. Its software applications includes Photoshop and Lightroom, Adobe Analytics, Media Optimizer and Campaign Manager.	{Technology,"Software - Application","Application Software"}	{MSFT,INTU,ADSK,SYMC,NUAN,TYPE,SAP,RHT,CRM,ORCL,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:11:59.441681	2020-10-11 23:38:31.02423
ADNT	Adient plc	Adient PLC is engaged in supplying automotive seating. It designs, manufactures and markets seating systems and components for passenger cars, commercial vehicles and light trucks, including vans, pick-up trucks and sport/crossover utility vehicles.	{"Consumer Cyclical","Auto Parts",Autos}	{}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:11:59.454406	2020-10-11 23:38:31.025871
ADUS	Addus HomeCare Corporation	Addus HomeCare Corp provides home and community based services, which are social in nature and are provided in the home, focused on the dual eligible population.	{Healthcare,"Medical Care","Health Care Providers"}	{AMED,LHCG,NHC,GEN,FVE,XLV}	Health Care Providers	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.465667	2020-10-11 23:38:31.024353
AEH	AEGON N.V. Perp. Cap Secs.	Aegon NV is engaged in providing life insurance, pensions and asset management products and services in the USA, Europe, and Asia. It offers life and protection, and accidental death products as well as individual savings and retirement products.	{"Financial Services","Insurance - Diversified",Insurance}	{}	Insurance	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.482787	2020-10-11 23:38:31.02582
AET	Aetna Inc.	Aetna Inc is a diversified health care benefits company. The company offers traditional, voluntary and consumer-directed health insurance products and related services, including medical, pharmacy, dental, and behavioral health.	{Healthcare,"Health Care Plans"}	{CI,HUM,UNH,CNC,WCG,ANTM,XLV}	Health Care Plans	Healthcare	New York Stock Exchange	\N	2020-10-11 23:11:59.495872	2020-10-11 23:38:31.024353
AFT	Apollo Senior Floating Rate Fund Inc.	Apollo Senior Floating Rate Fund is a non-diversified, closed-end management investment company. Its investment objective is to seek current income and preservation of capital by investing in senior, secured loans.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.512181	2020-10-11 23:38:31.023428
AGGE	IQ Enhanced Core Bond U.S.	The investment seeks to track (before fees and expenses) the price and yield performance of its underlying index, the IQ Enhanced Core Bond U.S. Index.\n The fund is a "fund of funds" which means it invests, under normal circumstances, at least 80% of its net assets, plus the amount of any borrowings for investment purposes, in the investments included in its underlying index. The underlying index seeks to outperform the U.S. dollar-denominated taxable fixed income universe by using a combination of short- and long-term momentum to overweight and underweight various sectors of the investment grade U.S. fixed income securities market.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.527683	2020-10-11 23:38:31.023428
AGR	Avangrid Inc.	Avangrid Inc is a diversified energy and utility company with operations in more than 20 states. The company operates regulated utilities and electricity generation through two lines of business. Avangrid Networks and Avangrid Renewables.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{NI,FE,ED,CPK,PPL,NRG,NEE,UTL,ES,NGG,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:11:59.539914	2020-10-11 23:38:31.024353
AGZD	WisdomTree Barclays U.S. Aggregate Bond Zero Durat	The investment seeks to track the price and yield performance, before fees and expenses, of the Bloomberg Barclays Rate Hedged U.S. Aggregate Bond Index, Zero Duration (the "index").\n The index is designed to provide long exposure to the Bloomberg Barclays U.S. Aggregate Bond Index while seeking to manage interest rate risk through the use of short positions in U.S. Treasury securities. The fund normally invests at least 80% of its total assets in the component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:11:59.550407	2020-10-11 23:38:31.031328
AIF	Apollo Tactical Income Fund Inc.	Apollo Tactical Income Fund Inc. is a closed-end management investment company. Its investment objective is to sell current income and preservation of capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.56314	2020-10-11 23:38:31.031328
FIVE	Five Below Inc.	Five Below Inc is a retailer offering merchandise for teen and pre-teen customers. Its merchandise includes Style, Room, Sports, Tech, Crafts, Party, Candy and Now. It offers a broad range of discount merchandise at prices up to $5.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{FRED,DLTR,BIG,DG}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:03.000356	2020-10-11 23:38:31.028166
FLAU	Franklin FTSE Australia	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the FTSE Australia RIC Capped Index (the FTSE Australia Capped Index).\n Under normal market conditions, the fund invests at least 80% of its assets in the component securities of the FTSE Australia Capped Index and in depositary receipts representing such securities. The FTSE Australia Capped Index is based on the FTSE Australia Index and is designed to measure the performance of Australian large- and mid-capitalization stocks. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.02992	2020-10-11 23:38:31.025871
FLEX	Flex Ltd.	Flex Ltd is a provider of Sketch-to-Scale services. It provides designing, manufacturing and supply chain services of packaged consumer electronics and industrial products to various industries and end-markets.	{Technology,"Contract Manufacturers","Computer Hardware"}	{CLS,PLXS,TTMI,SMTX,IEC,JBL,BHE,SANM,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:03.042336	2020-10-11 23:38:31.028166
FLIC	The First of Long Island Corporation	First of Long Island Corp is a holding company which provides financial services through its subsidiary, The First National Bank of Long Island. It offers services of saving account and lending services to professionals, consumers and public bodies.	{"Financial Services","Banks - Regional - US",Banks}	{BDGE,FISI,AROW,NBTB,CBU,EVBN}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:03.052198	2020-10-11 23:38:31.024353
FLKS	Flex Pharma Inc.	Flex Pharma Inc is a biotechnology company. The company is engaged in developing proprietary treatments for nocturnal leg cramps and spasms associated with severe neuromuscular conditions.	{Healthcare,Biotechnology}	{ADMP,FOMX,ALBO,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:03.062137	2020-10-11 23:38:31.021426
FLO	Flowers Foods Inc.	Flowers Foods Inc functions in the food and beverage industry. It primarily manufactures and sells packaged bakery products under the brand names such as Nature's Own, Whitewheat and Wonder Bread.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{JJSF,CPB,THS,POST}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:03.072987	2020-10-11 23:38:31.02582
FLQL	Franklin LibertyQ U.S. Equity	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the LibertyQ U.S. Large Cap Equity Index (the U.S. Large Cap underlying index).\n Under normal market conditions, the fund invests at least 80% of its assets in the component securities of the U.S. Large Cap underlying index. The U.S. Large Cap underlying index is based on the Russell 1000® Index using a methodology developed with Franklin Templeton to reflect Franklin Templeton's desired investment strategy.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:03.084124	2020-10-11 23:38:31.02582
FLTW	Franklin FTSE Taiwan	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the FTSE Taiwan RIC Capped Index (the FTSE Taiwan Capped Index).\n Under normal market conditions, the fund invests at least 80% of its assets in the component securities of the FTSE Taiwan Capped Index and in depositary receipts representing such securities. The FTSE Taiwan Capped Index is based on the FTSE Taiwan Index and is designed to measure the performance of Taiwanese large- and mid-capitalization stocks. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.098116	2020-10-11 23:38:31.023428
FMBH	First Mid-Illinois Bancshares Inc.	First Mid Illinois Bancshares is a financial services company. It also provides data processing services to affiliates and insurance products and services to customers through its subsidiaries.	{"Financial Services","Banks - Regional - US",Banks}	{FCBC,OVBC,FCNCA,STBA,TCBI,RNST,HOMB,MOFG,TMP,BXS}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:03.111749	2020-10-11 23:38:31.031328
FMO	Fiduciary/Claymore MLP Opportunity Fund of Beneficial Interest	Fiduciary/Claymore MLP Opportunity Fund is a non-diversified, closed-end management investment company. Its investment objective is to provide a high level of after-tax total return with an emphasis on current distributions paid to shareholders.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.126339	2020-10-11 23:38:31.02582
FNDE	Schwab Fundamental Emerging Markets Large Company Index	The investment seeks to track as closely as possible, before fees and expenses, the total return of the Russell RAFI™ Emerging Markets Large Company Index.\n The fund will invest at least 80% of its net assets in stocks included in the index, including depositary receipts representing securities of the index; which may be in the form of American Depositary Receipts, Global Depositary Receipts and European Depositary Receipts. The index measures the performance of the large company size segment by fundamental overall company scores, which are created using as the universe the companies included in the Russell Emerging Markets Index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.143409	2020-10-11 23:38:31.023428
FNGU	MicroSectors FANG Index 3X Leveraged ETNs due January 8 2038	The investment seeks return linked to a three times leveraged participation in the daily performance of the NYSE FANG+™ Index, total return (the “index”).\nThe notes are intended to be daily trading tools for sophisticated investors to manage daily trading risks as part of an overall diversified portfolio. The index is an equal-dollar weighted index designed to represent a segment of the technology and consumer discretionary sectors consisting of highly-traded growth stocks of technology and tech-enabled companies.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.15358	2020-10-11 23:38:31.024353
FL	Foot Locker Inc.	Foot Locker Inc operates in the footwear industry. Its core business involves retailing of athletic shoes and apparel through mall based stores.	{"Consumer Cyclical","Footwear & Accessories","Manufacturing - Apparel & Furniture"}	{SCVL,GCO,NKE,DSW,CAL}	Manufacturing - Apparel & Furniture	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:03.021509	2020-10-11 23:38:31.024353
FLCA	Franklin FTSE Canada	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the FTSE Canada RIC Capped Index (the FTSE Canada Capped Index).\n Under normal market conditions, the fund invests at least 80% of its assets in the component securities of the FTSE Canada Capped Index and in depositary receipts representing such securities. The FTSE Canada Capped Index is based on the FTSE Canada Index and is designed to measure the performance of Canadian large- and mid-capitalization stocks. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.033245	2020-10-11 23:38:31.031328
FLHK	Franklin FTSE Hong Kong	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of FTSE Hong Kong RIC Capped Index (the FTSE Hong Kong Capped Index).\n Under normal market conditions, the fund invests at least 80% of its assets in the component securities of the FTSE Hong Kong Capped Index and in depositary receipts representing such securities. The FTSE Hong Kong Capped Index is based on the FTSE Hong Kong Index and is designed to measure the performance of Hong Kong large- and mid-capitalization stocks. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.049769	2020-10-11 23:38:31.023428
FLKR	Franklin FTSE South Korea	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the FTSE South Korea RIC Capped Index (the FTSE South Korea Capped Index).\n Under normal market conditions, the fund invests at least 80% of its assets in the component securities of the FTSE South Korea Capped Index and in depositary receipts representing such securities. The FTSE South Korea Capped Index is based on the FTSE South Korea Index and is designed to measure the performance of South Korean large- and mid-capitalization stocks. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.059193	2020-10-11 23:38:31.02582
FLMI	Franklin Templeton ETF Trust Liberty Intermediate Municipal Opportunities	The investment seeks a high level of current income that is exempt from federal income taxes.\n Under normal market conditions, the fund invests at least 80% of its net assets in municipal securities whose interest is free from federal income taxes, including the federal alternative minimum tax. Although the fund tries to invest all of its assets in tax-free securities, it is possible that up to 20% of the fund's net assets may be in securities that pay interest that may be subject to the federal alternative minimum tax and, although not anticipated, in securities that pay interest subject to other federal or state income taxes.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.071121	2020-10-11 23:38:31.028166
FLQH	Franklin LibertyQ International Equity Hedged	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the LibertyQ International Equity Hedged Index.\n The fund invests at least 80% of its assets in the component securities of the index and in depositary receipts representing such securities. The index is designed to have higher returns than an equivalent unhedged index when the component currencies are weakening relative to the U.S. dollar.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.084319	2020-10-11 23:38:31.025871
FLTR	VanEck Vectors Investment Grade Floating Rate	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the MVIS® US Investment Grade Floating Rate Index (the "Floating Rate Index").\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The index is comprised of U.S. dollar-denominated floating rate notes issued by corporate entities or similar commercial entities that are public reporting companies in the United States and rated investment grade. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.098162	2020-10-11 23:38:31.02423
FMB	First Trust Managed Municipal ETF	The investment seeks to generate current income that is exempt from regular federal income taxes and its secondary objective is long term capital appreciation.\n Under normal market conditions, the fund seeks to achieve its investment objectives by investing at least 80% of its net assets (including investment borrowings) in municipal debt securities that pay interest that is exempt from regular federal income taxes (collectively, "Municipal Securities"). It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.110223	2020-10-11 23:38:31.02423
FMK	First Trust Mega Cap AlphaDEX Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the Nasdaq AlphaDEX® Mega Cap Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is designed to select mega cap stocks from the NASDAQ US 500 Large Cap Index (the "base index") that may generate positive alpha, or risk adjusted-returns, relative to traditional indices through the use of the AlphaDEX® selection methodology.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.125938	2020-10-11 23:38:31.02423
FNDB	Schwab Fundamental U.S. Broad Market Index	The investment seeks to track as closely as possible, before fees and expenses, the total return of the Russell RAFI™ US Index.\n The fund will invest at least 90% of its net assets (including, for this purpose, any borrowings for investment purposes) in stocks included in the index. The index measures the performance of the constituent companies by fundamental overall company scores, which are created using as the universe the companies included in the Russell 3000® Index. It may invest up to 10% of its net assets in securities not included in the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.143716	2020-10-11 23:38:31.02582
FKU	First Trust United Kingdom AlphaDEX Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the NASDAQ AlphaDEX® United Kingdom Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks and/or depositary receipts that comprise the index. The index is designed to select stocks from the NASDAQ United Kingdom Index (the "base index") that may generate positive alpha, or risk-adjusted returns, relative to traditional indices through the use of the AlphaDEX® selection methodology.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.020231	2020-10-11 23:38:31.02582
FLCH	Franklin FTSE China	The investment seeks to provide investment results that closely correspond the FTSE China RIC Capped Index (the FTSE China Capped Index).\n Under normal market conditions, the fund invests at least 80% of its assets in the component securities of the index and in depositary receipts representing such securities. The index is based on the FTSE China Index and is designed to measure the performance of Chinese large- and mid-capitalization stocks, as represented by H-Shares and B-Shares. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.033311	2020-10-11 23:38:31.021426
FLGR	Franklin FTSE Germany	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the FTSE Germany RIC Capped Index (the FTSE Germany Capped Index).\n Under normal market conditions, the fund invests at least 80% of its assets in the component securities of the FTSE Germany Capped Index and in depositary receipts representing such securities. The FTSE Germany Capped Index is based on the FTSE Germany Index and is designed to measure the performance of German large- and mid-capitalization stocks. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.04683	2020-10-11 23:38:31.02582
FLOW	SPX FLOW Inc.	SPX FLOW Inc is engaged in engineering, designing and manufacturing of flow control systems integral to process, blend, transport, meter, filters and dehydrate various types of fluids, gases and powders across a range of applications.	{Industrials,"Diversified Industrials","Industrial Products"}	{MIDD,SXI,WBT}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:03.076733	2020-10-11 23:38:31.023007
FLRN	SPDR Bloomberg Barclays Investment Grade Floating Rate	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the price and yield performance of the Bloomberg Barclays U.S. Dollar Floating Rate Note < 5 Years Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index or in securities that the Adviser determines have economic characteristics that are substantially identical to the economic characteristics of the securities that comprise the index. The index is designed to measure the performance of U.S. dollar-denominated, investment grade floating rate notes. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.087384	2020-10-11 23:38:31.024353
FLY	Fly Leasing Limited	Fly Leasing Ltd is principally engaged in purchasing commercial aircraft which are leased under multi-year contracts to a diverse group of airlines across the world.	{Industrials,"Rental & Leasing Services","Consulting & Outsourcing"}	{WLFC,TGH,GE,AYR,AER,CAI,ACY,AL}	Consulting & Outsourcing	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:03.102072	2020-10-11 23:38:31.02582
FMCIW	Forum Merger II Corporation Warrant		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:03.113566	2020-10-11 23:38:31.02582
FMN	Federated Premier Municipal Income Fund	Federated Premier Municipal Income Fund is an US-based diversified, closed-end management investment company. The investment objective of the fund is to provide current income exempt from federal income tax, including the federal alternative minimum tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.126194	2020-10-11 23:38:31.028166
FNDA	Schwab Fundamental U.S. Small Company Index	The investment seeks to track as closely as possible, before fees and expenses, the total return of the Russell RAFI™ US Small Company Index.\n The fund normally will invest at least 90% of its net assets (including, for this purpose, any borrowings for investment purposes) in stocks included in the index. The index measures the performance of the small company size segment by fundamental overall company scores, which are created using as the universe the companies included in the Russell 3000® Index. It may invest up to 10% of its net assets in securities not included in the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.142507	2020-10-11 23:38:31.028166
FNKO	Funko Inc.	Funko Inc is a pop culture consumer products company. It manufactures, designs and markets licensed pop culture products.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{}	Travel & Leisure	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:03.157814	2020-10-11 23:38:31.028166
FOANC	Gabelli Food of All Nations NextShares	The investment seeks to provide capital appreciation. \nUnder normal market conditions, the fund invests at least 80% of its net assets, plus borrowings for investment purposes, in common stocks and preferred stocks of domestic and foreign companies of all capitalization ranges in the food and beverage industries. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.167338	2020-10-11 23:38:31.024353
FORK	Fuling Global Inc.	Fuling Global Inc is a specialized production and distribution company for environmentally-friendly plastic serviceware with primary customers from the United States and European countries.	{"Consumer Cyclical","Home Furnishings & Fixtures","Manufacturing - Apparel & Furniture"}	{RAVN,TUP,OMN,MYE}	Manufacturing - Apparel & Furniture	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:03.179511	2020-10-11 23:38:31.024353
FOXA	Twenty-First Century Fox Inc.	Twenty-First Century Fox Inc is a diversified media and entertainment company. It operates in various business segments namely Cable Network Programming, Television, Filmed Entertainment, and Direct Broadcast Satellite Television.	{"Consumer Cyclical","Media - Diversified",Entertainment}	{DIS,CBS,AMCX,VIAB,NWSA,GHC,TGNA,LGF.A}	Entertainment	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:03.190125	2020-10-11 23:38:31.021426
FLAG	Exchange Traded Concepts Trust FLAG-Forensic Accounting Long-Short	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the price and yield performance of the FLAGLSX-Forensic Accounting Long-Short Index.\n The fund will normally invest at least 80% of its total assets in securities of the index. The index seeks to provide enhanced exposure to U.S. equities by offering a long/short portfolio selected from the 1000 largest U.S. companies by market capitalization, while providing additional return potential through the careful and systematic selection and shorting of stocks based on forensic accounting analysis.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.022818	2020-10-11 23:38:31.02423
FLGB	Franklin FTSE United Kingdom	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the FTSE UK RIC Capped Index (the FTSE UK Capped Index).\n Under normal market conditions, the fund invests at least 80% of its assets in the component securities of the FTSE UK Capped Index and in depositary receipts representing such securities. The FTSE UK Capped Index is based on the FTSE UK Index and is designed to measure the performance of United Kingdom large- and mid-capitalization stocks. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.043875	2020-10-11 23:38:31.021426
FLIR	FLIR Systems Inc.	FLIR Systems Inc is engaged in the scientific instrument industry. It develops detection apparatus such as thermal imaging systems and visible-light imaging systems.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{ESLT,CUB,GD,HON,NOC,TMO,UTX,RTN,LLL,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:03.054872	2020-10-11 23:38:31.02423
FLLV	Franklin Liberty U.S. Low Volatility	The investment seeks capital appreciation with an emphasis on lower volatility.\n The fund invests at least 80% of its net assets in U.S. investments. It invests primarily in equity securities (principally common stocks) of U.S. companies. The fund seeks capital appreciation while providing a lower level of volatility than the broader equity market as measured by the Russell 1000 Index. It may invest a portion of its assets in mid-capitalization companies.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.065104	2020-10-11 23:38:31.025871
FLQD	Franklin LibertyQ Global Dividend	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the LibertyQ Global Dividend Index.\n The fund invests at least 80% of its assets in the component securities of the index and in depositary receipts representing such securities. The index includes stocks from developed and emerging market countries with high and persistent dividend income that have favorable exposure to a quality investment style factor.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.078064	2020-10-11 23:38:31.021426
FLT	FleetCor Technologies Inc.	Fleetcor Technologies, Inc. provides fuel cards, commercial payment and data solutions, lodging and transportation management services, stored value solutions, and workforce payment products and services to businesses, retailers, and commercial fleets.	{Industrials,"Business Services"}	{EEFT,GPN,FIS,MA,V,WEX}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:03.093658	2020-10-11 23:38:31.023007
FM	iShares MSCI Frontier 100 Fund	The investment seeks to track the investment results of the MSCI Frontier Markets 100 Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is designed to measure equity market performance of frontier markets while putting stronger emphasis on tradability compared to the MSCI Frontier Markets IMI (the "parent index").	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.10478	2020-10-11 23:38:31.024353
FMF	First Trust Morningstar Managed Futures Strategy Fund	The investment seeks to achieve positive returns that uses as a benchmark, the Morningstar Diversified Futures Index.\n The fund is an actively managed exchange-traded fund that seeks to achieve positive returns that are not directly correlated to broad market equity or fixed income returns. It seeks to exceed the performance of the Benchmark by actively selecting investments for the fund with varying maturities from the underlying components of the Benchmark.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.118454	2020-10-11 23:38:31.025871
FMY	First Trust Motgage Income Fund of Beneficial Interest	First Trust Mortgage Income Fund is a United States based diversified closed-end management investment company. The fund's primary investment objective is to seek a high level of current income. Its secondary investment objective is to preserve capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.131228	2020-10-11 23:38:31.028166
FNDC	Schwab Fundamental International Small Company Index	The investment seeks to track as closely as possible, before fees and expenses, the total return of the Russell RAFI™ Developed ex US Small Company Index.\n The fund will invest at least 90% of its net assets in stocks included in the index, including depositary receipts representing securities of the index, which may be in the form of American Depositary receipts, Global Depositary receipts and European Depositary receipts. The index measures the performance of the small company size segment by fundamental overall company scores, which are created using as the universe the companies included in the Russell Developed ex US Index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.144285	2020-10-11 23:38:31.021426
FNHC	FedNat Holding Company	FedNat Holding Co, formerly Federated National Holding Co. is an insurance holding company controlling all steps in the insurance underwriting, distribution and claims processes through its subsidiaries.	{"Financial Services","Insurance - Property & Casualty"}	{}	Insurance - Property & Casualty	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:03.153553	2020-10-11 23:38:31.021426
FLCO	Franklin Liberty Investment Grade Corporate	The investment seeks a high level of current income as is consistent with prudent investing, while seeking preservation of capital.\n Under normal market conditions, the fund invests at least 80% of its net assets in investment grade corporate debt securities and investments. It invests primarily in U.S. dollar denominated corporate debt securities issued by U.S. and foreign companies. The fund may invest in debt securities of any maturity or duration. The fund's focus on the credit quality of its portfolio is intended to reduce credit risk and help to preserve the fund's capital.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.034118	2020-10-11 23:38:31.02582
FLIO	Franklin Liberty International Opportunities	The investment seeks long-term capital appreciation.\n Under normal market conditions, the fund invests predominantly in equity securities of international companies in developed, developing and frontier markets outside of the U.S. across the entire market capitalization spectrum. It invests in equity securities of international companies in regions/countries that may include, but are not limited to, Africa; Australia; Canada; Mexico; Latin America; South America; Europe (including the United Kingdom); Asia (including Japan, South Korea, China, India and Russia); and the Middle East.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.055152	2020-10-11 23:38:31.031328
FLQS	Franklin LibertyQ U.S. Small Cap Equity	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the LibertyQ U.S. Small Cap Equity Index.\n Under normal market conditions, the fund invests at least 80% of its assets in the component securities of the underlying index. The underlying index is based on the Russell 2000® Index using a methodology developed with Franklin Templeton to reflect Franklin Templeton's desired investment strategy.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:03.084804	2020-10-11 23:38:31.031328
FLWS	1-800 FLOWERS.COM Inc.	1-800-Flowers.com Inc is a U.S. based provider of gourmet food and gift baskets, consumer floral, and BloomNet wire service.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{SBUX,BBBY,BKS}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:03.098253	2020-10-11 23:38:31.028166
FMC	FMC Corporation	FMC Corp is a diversified chemical company that serves agricultural, consumer and industrial markets internationally with solutions, applications and products. It also produces health and nutrition products, as well as lithium derivatives.	{"Basic Materials","Agricultural Inputs",Agriculture}	{EMN,AVD,CHD,PPG,MBII,ASH,DWDP}	Agriculture	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:03.114072	2020-10-11 23:38:31.023428
FNB	F.N.B. Corporation	F N B Corp provides a range of commercial banking, consumer banking, and wealth management through a subsidiary network.	{"Financial Services","Banks - Regional - US",Banks}	{BMTC,FULT,HBAN,STBA,UVSP,PNFP,MTB,FCF,RF,ACNB}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.134511	2020-10-11 23:38:31.02423
FNG	AdvisorShares New Tech and Media	The investment seeks long-term capital appreciation.\n The fund is an actively managed ETF that seeks to achieve its investment objective by investing, under normal circumstances, at least 80% of its net assets in securities of technology and media companies. It will invest primarily in U.S. exchange-listed equity securities, including common and preferred stock and ADRs, of technology and technology-related companies, including innovative and fast-growing technologies. The fund will concentrate its investments in the software and services industry within the information technology sector. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.147093	2020-10-11 23:38:31.031328
FOF	Cohen & Steers Closed-End Opportunity Fund Inc.	Cohen & Steers Closed-end Opp Fd Inc is a diversified, closed-end management investment company. The company's investment objective is to achieve total return.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.169691	2020-10-11 23:38:31.023428
FPA	First Trust Asia Pacific Ex-Japan AlphaDEX Fund	The investment seeks results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the NASDAQ AlphaDEX® Asia Pacific Ex-Japan Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks and/or depositary receipts that comprise the index. The index is designed to select stocks from the NASDAQ Asia Pacific Ex-Japan Index that may generate positive alpha, or risk-adjusted returns, relative to traditional indices through the use of the AlphaDEX® selection methodology.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.195062	2020-10-11 23:38:31.031328
FPXI	First Trust International IPO ETF	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an index called the IPOX International Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks and/or depositary receipts that comprise the index. The index is a market-cap weighted portfolio measuring the performance of the top 50 non-U.S. companies, including companies domiciled in emerging markets, ranked quarterly by market capitalization in the IPOX® Global Composite Index (the "base index"). The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.211071	2020-10-11 23:38:31.031328
FREL	Fidelity MSCI Real Estate Index	The investment seeks to provide investment returns that correspond, before fees and expenses, generally to the performance of the MSCI USA IMI Real Estate Index.\n The fund invests at least 80% of assets in securities included in the fund's underlying index. The fund's underlying index is the MSCI USA IMI Real Estate Index, which represents the performance of the real estate sector in the U.S. equity market. It may or may not hold all of the securities in the MSCI USA IMI Real Estate Index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.228157	2020-10-11 23:38:31.031328
FRSH	Papa Murphy's Holdings Inc.	Papa Murphy's Holdings Inc is the franchisor and operator of Take 'N' Bake pizza chain in the United States. The company operates in three business segments namely Domestic Franchise, Domestic Company Stores and International.	{"Consumer Cyclical",Restaurants}	{NATH,PZZA,YUM,DPZ,PBPB,RAVE}	Restaurants	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:03.243868	2020-10-11 23:38:31.025871
FLEE	Franklin FTSE Europe	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the FTSE Developed Europe RIC Capped Index (the FTSE Developed Europe Capped Index).\n Under normal market conditions, the fund invests at least 80% of its assets in the component securities of the FTSE Developed Europe Capped Index and in depositary receipts representing such securities. The FTSE Developed Europe Capped Index is based on the FTSE Developed Europe Index and is designed to measure the performance of large- and mid-capitalization stocks from developed European countries.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.035692	2020-10-11 23:38:31.024353
FLGE	Credit Suisse FI Large Cap Growth Enhanced ETN	The investment seeks a leveraged return linked to the performance of the Russell 1000® Growth Index Total Return (the “index”), an index that seeks to track the performance of the large-cap growth segment of the U.S. equity market.\nThe ETNs are subject to a leverage factor of 2.0, but the effective leverage will vary with changes in the Closing Indicative Value of the ETNs since the previous Rebalance Event.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.046801	2020-10-11 23:38:31.028166
FLJP	Franklin FTSE Japan	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the FTSE Japan RIC Capped Index (the FTSE Japan Capped Index).\n Under normal market conditions, the fund invests at least 80% of its assets in the component securities of the FTSE Japan Capped Index and in depositary receipts representing such securities. The FTSE Japan Capped Index is based on the FTSE Japan Index and is designed to measure the performance of Japanese large- and mid-capitalization stocks.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.058736	2020-10-11 23:38:31.023007
FLQG	Franklin LibertyQ Global Equity	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the LibertyQ Global Equity Index.\n The fund invests at least 80% of its assets in the component securities of the index and in depositary receipts representing such securities. The index includes stocks from developed and emerging market countries that have favorable exposure to multiple investment style factors.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.081253	2020-10-11 23:38:31.02423
FLTB	Fidelity Limited Term Bond	The investment seeks to provide a high rate of income.\n The fund normally invests at least 80% of assets in investment-grade debt securities (those of medium and high quality) of all types and repurchase agreements for those securities. It is managed to have similar overall interest rate risk to the Fidelity Limited Term Composite Index. Normally, the fund maintains a dollar-weighted average maturity between two and five years.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.095562	2020-10-11 23:38:31.025871
FMCIU	Forum Merger II Corporation Unit		{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:03.112899	2020-10-11 23:38:31.028166
FMS	Fresenius Medical Care AG	Fresenius Medical Care AG & Co. KGaA is a kidney dialysis company. It develops and manufactures a full range of dialysis machines, systems, and disposable products. The company provides dialysis care and related services.	{Healthcare,"Medical Care","Health Care Providers"}	{AMGN,RMTI,IDXX,NXTM,DGX,BAX,CMD,TFX,LH,DVA,XLV}	Health Care Providers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:03.127234	2020-10-11 23:38:31.023007
FND	Floor & Decor Holdings Inc.	Floor & Decor Holdings Inc is a specialty retailer of hard surface flooring and related accessories. It offers tile, wood, laminate and natural stone flooring along with decorative and installation accessories.	{"Consumer Cyclical","Home Improvement Stores","Retail - Apparel & Specialty"}	{HDS,HBP,IBP,GMS}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:03.140628	2020-10-11 23:38:31.031328
FNK	First Trust Mid Cap Value AlphaDEX Fund	The investment seeks results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the Nasdaq AlphaDEX® Mid Cap Value Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is designed to select value stocks from the NASDAQ US 600 Mid Cap Index (the "base index") that may generate positive alpha, or risk-adjusted returns, relative to traditional indices through the use of the AlphaDEX® selection methodology.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.157404	2020-10-11 23:38:31.02582
FNWB	First Northwest Bancorp	First Northwest Bancorp is a bank holding company for First Federal Savings and Loan Association. The Company is engaged in the business of attracting deposits and providing lending services to the communities it serves.	{"Financial Services","Savings & Cooperative Banks",Banks}	{MPB,MOFG}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:03.166097	2020-10-11 23:38:31.02423
FONR	Fonar Corporation	Fonar Corp is engaged in the research, development, production and marketing of medical scanning equipment. The Company operates in two segments manufacturing and servicing of medical equipment and management of diagnostic imaging centers.	{Healthcare,"Medical Devices"}	{HOLX,DRAD,AMS,GE,PHG,DVA,NAVB,ICAD,MDT,XLV}	Medical Devices	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:03.177182	2020-10-11 23:38:31.031328
FORTY	Formula Systems (1985) Ltd. ADS represents 1 ordinary shares	Formula Systems (1985) Ltd is an IT solutions and services holdings company. It is engaged in providing software solutions and services, software product marketing and support.	{Technology,"Information Technology Services","Application Software"}	{MSFT,PEGA,SAP,FICO,IBM,ORCL,HPQ,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:03.186565	2020-10-11 23:38:31.02582
FLEH	Franklin FTSE Europe Hedged	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the FTSE Developed Europe RIC Capped Hedged to USD Index (the FTSE Developed Europe Capped Hedged Index).\n Under normal market conditions, the fund invests at least 80% of its assets in the component securities of the index and in depositary receipts representing such securities, including indirectly through the Franklin FTSE Europe ETF. The index is based on the FTSE Developed Europe Hedged to USD Index and is designed to measure the performance of large- and mid-capitalization stocks from developed European countries.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.038063	2020-10-11 23:38:31.02423
FLN	First Trust Latin America AlphaDEX Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the NASDAQ AlphaDEX® Latin America Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks and/or depositary receipts that comprise the index. The index is designed to select stocks from the NASDAQ Latin America Index (the "base index") that may generate positive alpha, or risk-adjusted returns, relative to traditional indices through the use of the AlphaDEX® selection methodology.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.073442	2020-10-11 23:38:31.031328
FLR	Fluor Corporation	Fluor Corp is an engineering company that provides procurement, construction and maintenance as well as project management services to a range of customers including oil and gas, manufacturing, and mining companies, along with the U.S. federal government.	{Industrials,"Engineering & Construction"}	{MTZ,PWR,JEC,KBR,ACM,TPC,BWXT}	Engineering & Construction	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:03.086595	2020-10-11 23:38:31.028166
FLXS	Flexsteel Industries Inc.	Flexsteel Industries Inc manufactures, imports and markets residential and commercial upholstered and wood furniture products in the United States. Its products include sofas, loveseats and rocker-reclining chairs.	{"Consumer Cyclical","Home Furnishings & Fixtures","Manufacturing - Apparel & Furniture"}	{BSET,HOFT,ETH,LZB,NTZ,SNBR}	Manufacturing - Apparel & Furniture	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:03.100942	2020-10-11 23:38:31.031328
FMCI	Forum Merger II Corporation		{Industrials}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:03.112899	2020-10-11 23:38:31.025871
FMHI	First Trust Municipal High Income ETF	The investment seeks to provide federally tax-exempt income, and its secondary objective will be long-term capital appreciation.\n Under normal market conditions, the fund seeks to achieve its investment objectives by investing at least 80% of its net assets (including investment borrowings) in municipal debt securities that pay interest that is exempt from regular federal income taxes. Municipal securities are generally issued by or on behalf of states, territories or possessions of the U.S. and the District of Columbia and their political subdivisions, agencies, authorities and other instrumentalities. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.125544	2020-10-11 23:38:31.021426
FNCL	Fidelity MSCI Financials Index	The investment seeks to provide investment returns that correspond, before fees and expenses, generally to the performance of the MSCI USA IMI Financials Index.\n The fund invests at least 80% of assets in securities included in the fund's underlying index. The fund's underlying index is the MSCI USA IMI Financials Index, which represents the performance of the financial sector in the U.S. equity market. It may or may not hold all of the securities in the MSCI USA IMI Financials Index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.139645	2020-10-11 23:38:31.024353
FNGD	MicroSectors FANG Index -3X Inverse Leveraged ETNs due January 8 2038	The investment seeks return linked to a three times inverse leveraged participation in the daily performance of the NYSE Fang+™ Index, total return (the “index”).\nThe notes are intended to be daily trading tools for sophisticated investors to manage daily trading risks as part of an overall diversified portfolio. The index is an equal-dollar weighted index designed to represent a segment of the technology and consumer discretionary sectors consisting of highly-traded growth stocks of technology and tech-enabled companies.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.150659	2020-10-11 23:38:31.02423
FNV	Franco-Nevada Corporation	Franco-Nevada Corp is a gold royalty and stream company. It is also engaged in silver, platinum group metals, oil & gas and others. Its portfolio includes approximately 340 mineral and oil & gas assets covering properties from production to exploration.	{"Basic Materials",Gold,"Metals & Mining"}	{IAG,EGO,CDE}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:03.164271	2020-10-11 23:38:31.023007
FOMX	Foamix Pharmaceuticals Ltd.	Foamix Pharmaceuticals Ltd is a clinical-stage specialty pharmaceutical company. It is engaged in developing and commercializing minocycline foam for the treatment of acne, rosacea, impetigo and other skin conditions. Its products include FMX101 & FMX103.	{Healthcare,Biotechnology}	{AKAO,ZGNX,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:03.174635	2020-10-11 23:38:31.02582
FORR	Forrester Research Inc.	Forrester Research Inc is a provider of business services. Its portfolio of services include independent research, data, and advisory services.	{Industrials,"Business Services"}	{EXPO,SCOR,IT,NLSN}	Business Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:03.184899	2020-10-11 23:38:31.028166
FPAY	FlexShopper Inc.	FlexShopper Inc is engaged in the business of providing certain types of durable goods to consumers on a lease-to-own basis and providing lease-to-own terms to consumers of third-party retailers & e-tailers.	{"Financial Services","Credit Services"}	{ECPG,LOAN,OMF}	Credit Services	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:03.196427	2020-10-11 23:38:31.023428
GLTR	Aberdeen Standard Physical Precious Metals Basket Shares	The investment objective of the Trust is for the Shares to reflect the performance of the price of physical gold, silver, platinum and palladium in the proportions held by the Trust, less the expenses of the Trust’s operations. \n The Shares are designed for investors who want a cost-effective and convenient way to invest in a basket of Bullion with minimal credit risk.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.617397	2020-10-11 23:38:31.028166
FLFR	Franklin FTSE France	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the FTSE France RIC Capped Index (the FTSE France Capped Index).\n Under normal market conditions, the fund invests at least 80% of its assets in the component securities of the FTSE France Capped Index and in depositary receipts representing such securities. The FTSE France Capped Index is based on the FTSE France Index and is designed to measure the performance of French large- and mid-capitalization stocks. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.044296	2020-10-11 23:38:31.031328
FLIY	Franklin FTSE Italy	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the FTSE Italy RIC Capped Index (the FTSE Italy Capped Index).\n Under normal market conditions, the fund invests at least 80% of its assets in the component securities of the FTSE Italy Capped Index and in depositary receipts representing such securities. The FTSE Italy Capped Index is based on the FTSE Italy Index and is designed to measure the performance of Italian large- and mid-capitalization stocks. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.055278	2020-10-11 23:38:31.025871
FLQE	Franklin LibertyQ Emerging Markets	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the LibertyQ Emerging Markets Index.\n The fund invests at least 80% of its assets in the component securities of the index and in depositary receipts representing such securities. The index seeks to achieve a lower level of risk and higher risk-adjusted performance than the MSCI Emerging Markets Index over the long term by applying a multi-factor selection process.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.078283	2020-10-11 23:38:31.023428
FLS	Flowserve Corporation	Flowserve Corp is a manufacturer and aftermarket service provider of comprehensive flow control systems. It develops and manufactures precision-engineered flow control equipment integral to the movement, control and protection of the flow of materials.	{Industrials,"Diversified Industrials","Industrial Products"}	{DOV,GRC,GGG,IEX,ITW,PH,CFX,IR,XYL,ITT}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:03.090761	2020-10-11 23:38:31.021426
FMAT	Fidelity MSCI Materials Index	The investment seeks to provide investment returns that correspond, before fees and expenses, generally to the performance of the MSCI USA IMI Materials Index.\n The fund invests at least 80% of assets in securities included in the fund's underlying index. The fund's underlying index is the MSCI USA IMI Materials Index, which represents the performance of the materials sector in the U.S. equity market. It may or may not hold all of the securities in the MSCI USA IMI Materials Index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.107043	2020-10-11 23:38:31.021426
FMDG	Fieldstone Merlin Dynamic Large Cap Growth	The investment seeks long-term capital appreciation.\n The fund seeks to achieve its investment objective of long term capital appreciation by investing principally in large cap growth securities of any kind, and, under normal circumstances, the advisor intends to invest at least 80% of its net assets (plus the amount of borrowings for investment purposes) in such securities. The fund will invest in the stock of 25 large cap growth firms selected by Merlin as being, in the opinion of Merlin, among the most favorable growth opportunities, regardless of industry or sector.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.118815	2020-10-11 23:38:31.024353
FN	Fabrinet	Fabrinet is engaged in providing outsourced manufacturing services to original equipment manufacturers. It offers a wide range of optical and electro-mechanical manufacturing capabilities across the whole producing process.	{Technology,"Consumer Electronics","Computer Hardware"}	{CLS,COHR,IIVI,KBAL,PLXS,IPGP,OCLR,BHE,SANM,VIAV,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:03.132703	2020-10-11 23:38:31.025871
FNF	FNF Group of Fidelity National Financial Inc.	Fidelity National Financial Inc is a title insurance company. It provides title insurance, escrow, and other title-related services. The firm also provides technology and transaction services to the real estate and mortgage industries.	{"Financial Services","Insurance - Specialty"}	{ITIC,ORI,STC,FAF,ARGO}	Insurance - Specialty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.147352	2020-10-11 23:38:31.023007
FNSR	Finisar Corporation	Finisar Corp provides optical subsystems and components that are used in data communication and telecommunication applications. It sells to manufacturers of storage systems, network equipment, telecommunication equipment and their contract manufacturers.	{Technology,"Communication Equipment"}	{CSCO,OCLR,NPTN,EMKR,AAOI,VIAV,ARRS,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:03.160329	2020-10-11 23:38:31.031328
FONE	First Trust NASDAQ OMX CEA Smartphone Index Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the NASDAQ CTA Smartphone Index(SM).\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks or in depositary receipts representing securities in the index. The index is designed to track the performance of companies engaged in the Smartphone segment of the telecommunications and technology sectors.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.177535	2020-10-11 23:38:31.028166
GRBK	Green Brick Partners Inc.	Green Brick Partners Inc is a real estate development company. The company acquires and develops land, as well as providing land and construction financing to its controlled builders.	{"Real Estate","Real Estate - General","Real Estate Services"}	{GPRE,REX,REGI,PEIX}	Real Estate Services	Real Estate	NASDAQ Capital Market	\N	2020-10-11 23:12:03.774871	2020-10-11 23:38:31.021426
FLMX	Franklin FTSE Mexico	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the FTSE Mexico RIC Capped Index (the FTSE Mexico Capped Index).\n Under normal market conditions, the fund invests at least 80% of its assets in the component securities of the FTSE Mexico Capped Index and in depositary receipts representing such securities. The FTSE Mexico Capped Index is based on the FTSE Mexico Index and is designed to measure the performance of Mexican large- and mid-capitalization stocks. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.07115	2020-10-11 23:38:31.02423
FLQM	Franklin LibertyQ U.S. Mid Cap Equity	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the LibertyQ U.S. Mid Cap Equity Index (the U.S. Mid Cap underlying index).\n Under normal market conditions, the fund invests at least 80% of its assets in the component securities of the U.S. Mid Cap underlying index. The U.S. Mid Cap underlying index is based on the Russell Midcap® Index using a methodology developed with Franklin Templeton to reflect Franklin Templeton's desired investment strategy.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:03.084777	2020-10-11 23:38:31.028166
FLXN	Flexion Therapeutics Inc.	Flexion Therapeutics Inc is a development stage specialty pharmaceutical company. The Company is engaged in the development and commercialization of novel, long-acting, injectable pain therapies including anti-inflammatory and analgesic therapies.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{JNJ,PFE,PCRX,ACRX,INSY,AMPE,REPH,CARA,XLV}	Drug Manufacturers	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:03.099344	2020-10-11 23:38:31.028166
FMBI	First Midwest Bancorp Inc.	First Midwest Bancorp Inc provides a broad range of commercial, retail, treasury, and wealth management products and services, to commercial and industrial, commercial real estate, municipal, and consumer customers.	{"Financial Services","Banks - Regional - US",Banks}	{OSBC,WTFC,MBFI,BUSE}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:03.113728	2020-10-11 23:38:31.028166
FMX	Fomento Economico Mexicano S.A.B. de C.V.	Fomento Economico Mexicano SAB de CV is a part of the beverage industry. Its business involves production and distribution of soft drinks.	{"Consumer Defensive","Beverages - Brewers","Beverages - Alcoholic"}	{COKE,KOF,AKO.B,KO,CCEP,PEP}	Beverages - Alcoholic	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:03.130712	2020-10-11 23:38:31.031328
FNDX	Schwab Fundamental U.S. Large Company Index	The investment seeks to track as closely as possible, before fees and expenses, the total return of the Russell RAFI™ US Large Company Index.\n The fund normally will invest at least 90% of its net assets (including, for this purpose, any borrowings for investment purposes) in stocks included in the index. The index measures the performance of the large company size segment by fundamental overall company scores, which are created using as the universe the companies included in the Russell 3000® Index. It may invest up to 10% of its net assets in securities not included in the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.144073	2020-10-11 23:38:31.028166
FNLC	First Bancorp Inc (ME)	First Bancorp Inc is a Damariscotta based bank holding company of The First National Bank of Damariscotta providing a range of retail and commercial banking services including deposit accounts and consumer, commercial and mortgage loans.	{"Financial Services","Banks - Regional - US",Banks}	{CAC,EBTC,BPFH,CNBKA,INDB,WBS,BHB,NBN,SAL,PNBK}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:03.164782	2020-10-11 23:38:31.028166
FOXF	Fox Factory Holding Corp.	Fox Factory Holding Corp designs and manufactures recreational vehicles in the United States. Its products include bicycles, snowmobiles, mountain bikes, and motorcycles. Most of its revenue is acquired from the U.S market.	{"Consumer Cyclical","Recreational Vehicles",Autos}	{CAAS,AXL,TEN,DAN,TOWR,VC}	Autos	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:03.191175	2020-10-11 23:38:31.024353
FPL	First Trust New Opportunities MLP & Energy Fund of Beneficial Interest	First Trust New Opportunities MLP & Energy Fund is a non-diversified, closed-end management investment company. Its investment objective is to seek a high level of total return with an emphasis on current distributions paid to common shareholders.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.204544	2020-10-11 23:38:31.023007
FRBA	First Bank	First Bank provides its customers with a variety of financial services targeting all segments of the retail and corporate market, in addition to Fund Management, Brokerage, Leasing, Property Management businesses and Private Banking.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:03.221571	2020-10-11 23:38:31.021426
FRN	Invesco Frontier Markets	The investment seeks to track the investment results (before fees and expenses) of the BNY Mellon New Frontier Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities that comprise the underlying index. The underlying index is comprised of liquid American depositary receipts ("ADRs") listed on a U.S. exchange, global depositary receipts ("GDRs") traded on the London Stock Exchange, and ordinary share classes of equity securities listed on exchanges in Frontier Market countries that meet certain trading volume and free-float market capitalization criteria. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.237248	2020-10-11 23:38:31.023428
FSB	Franklin Financial Network Inc.	Franklin Financial Network Inc, through its subsidiary, provides banking and related financial services including commercial banking, consumer banking, mortgage loans, deposits, wealth management/trust services and other products and services.	{"Financial Services","Banks - Regional - US",Banks}	{PWOD,PRK}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.254321	2020-10-11 23:38:31.028166
GRF	Eagle Capital Growth Fund Inc.	Eagle Capital Growth Fund Inc is a diversified closed-end investment company.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:03.781746	2020-10-11 23:38:31.02423
FLRT	AdvisorShares Pacific Asset Enhanced Floating Rate	The investment seeks to provide a high level of current income.\n The fund seeks to achieve its investment objective by selecting a focused portfolio comprised primarily of income producing floating rate loans and floating rate debt securities. Under normal circumstances, it will invest at least 80% of its net assets (plus any borrowings for investment purposes) in floating rate loans and other floating rate debt securities and in derivatives or other instruments that have economic characteristics similar to such securities (such as swap agreements, including, but not limited to, total return swaps, credit default swaps, and interest rate swaps).	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.089948	2020-10-11 23:38:31.023428
FMAO	Farmers & Merchants Bancorp Inc.	Farmers & Merchants Bancorp Inc is a bank holding company. It is involved in providing financial services through its bank, especially retail and corporate banking services. It derives its revenue from interests received on lending to retail markets.	{"Financial Services","Banks - Regional - US",Banks}	{FITB,PBCT,NWBI,BBT,TCF,HBMD,OPB,WNEB}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:03.108917	2020-10-11 23:38:31.023007
FMNB	Farmers National Banc Corp.	Farmers National Banc Corp and its subsidiaries operate in the domestic banking, trust, retirement consulting, insurance and financial management industries. Its business consists of owning and supervising its subsidiaries.	{"Financial Services","Banks - Regional - US",Banks}	{PEBO,UBCP,OVBC,UBOH,HBAN,NBTB,PRK,SGB,SBFG,CIVB}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:03.128695	2020-10-11 23:38:31.023428
FNDF	Schwab Fundamental International Large Company Index	The investment seeks to track as closely as possible, before fees and expenses, the total return of the Russell RAFI™ Developed ex US Large Company Index.\n The fund will invest at least 90% of its net assets in stocks included in the index, including depositary receipts representing securities of the index; which may be in the form of American Depositary receipts, Global Depositary receipts and European Depositary receipts. The index measures the performance of the large company size segment by fundamental overall company scores, which are created using as the universe the companies included in the Russell Developed ex US Index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.143767	2020-10-11 23:38:31.025871
FNJN	Finjan Holdings Inc.	Finjan Holdings Inc is a cybersecurity business engaged in licensing and enforcement, providing advisory services, developing mobile security applications, and investing in cybersecurity technologies and intellectual property.	{Technology,"Software - Application","Application Software"}	{PEGA,XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:03.157427	2020-10-11 23:38:31.025871
FOE	Ferro Corporation	Ferro Corp is a producer of specialty materials and chemicals that are sold to the manufacturers who, in turn, make products for many end-use markets. It offers electronic materials, color and glass performance materials, pigments, powders & oxides.	{"Basic Materials","Specialty Chemicals",Chemicals}	{GRA,OMN,CBT,PPG,RPM,SXT,HUN,KRO,DWDP}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:03.168017	2020-10-11 23:38:31.021426
FORD	Forward Industries Inc.	Forward Industries Inc is engaged in the designing, marketing and distribution of protective solutions for hand held electronic devices. The Company's protective solutions include soft-sided carrying cases, bags, clips, hand straps and other accessories.	{"Consumer Cyclical","Footwear & Accessories","Manufacturing - Apparel & Furniture"}	{TLF,FOSL,TPR}	Manufacturing - Apparel & Furniture	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:03.178821	2020-10-11 23:38:31.023428
FOSL	Fossil Group Inc.	Fossil Group Inc is engaged in designing, marketing and distribution of consumer fashion accessories. It offers men's and women's fashion watches and jewelry, handbags, small leather goods, belts, sunglasses, soft accessories and select apparel.	{"Consumer Cyclical","Footwear & Accessories","Manufacturing - Apparel & Furniture"}	{MOV,TPR}	Manufacturing - Apparel & Furniture	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:03.188436	2020-10-11 23:38:31.028166
FPI	Farmland Partners Inc.	Farmland Partners Inc owns and aims to acquire high-quality farmland throughout North America. The company is an internally managed real estate company and owns and contracts for over 120,000 acres of farm land, along with storage facilities.	{"Real Estate","REIT - Diversified",REITs}	{O,KRC,PMT}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:03.202254	2020-10-11 23:38:31.028166
FRA	Blackrock Floating Rate Income Strategies Fund Inc	Blackrock Floating Rate Inc Stra Fd Inc investment objective is to provide shareholders with high current income & preservation of capital with investment in a diversified, leveraged portfolio consisting of floating rate debt securities and instruments.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.216062	2020-10-11 23:38:31.025871
FRI	First Trust S&P REIT Index Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the S&P United States REIT Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is a subset of the S&P Developed REIT Index, which measures the performance of more than 200 REITs or REIT-like structures in 15 developed markets.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.23467	2020-10-11 23:38:31.028166
FRTA	Forterra Inc.	Forterra Inc is engaged in manufacturing pipe and precast products in the United States and Eastern Canada for a variety of water-related infrastructure applications, including water transmission, distribution and drainage.	{"Basic Materials","Building Materials"}	{FRTA}	Building Materials	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:12:03.249264	2020-10-11 23:38:31.02582
FOLD	Amicus Therapeutics Inc.	Amicus Therapeutics Inc is a biopharmaceutical company. It is engaged in the discovery, development and commercialization of next-generation medicines.	{Healthcare,Biotechnology}	{BMRN,SHPG,BIIB,GSK,NVO,PLX,ESPR,LXRX,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:03.170711	2020-10-11 23:38:31.023007
FORM	FormFactor Inc.	FormFactor Inc is a technology solutions provider. Its core business involves manufacture of advanced semiconductor probe card products.	{Technology,Semiconductors}	{AMAT,AEHR,COHU,MU,NANO,TRT,INTT,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:03.182923	2020-10-11 23:38:31.025871
FR	First Industrial Realty Trust Inc.	First Industrial Realty Trust Inc is a self-administered and fully integrated real estate company. It owns and operates industrial real estate and provides a supply chain solutions to multinational corporations and regional customers.	{"Real Estate","REIT - Industrial",REITs}	{EGP,MNR,DLR,COR,STAG,PLD,REXR,QTS}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:03.21524	2020-10-11 23:38:31.028166
FRO	Frontline Ltd.	Frontline Ltd is an oil tanker shipping firm. The company operates a fleet of crude oil tankers that transports crude oil internationally under short- and medium-term contracts. Some of the vessels also transport raw materials such as iron ore and coal.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{GLNG,NAT,TK,SFL,TGP,TOO,TNK,DHT,ASC,GOGL}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:03.237248	2020-10-11 23:38:31.02582
FSACW	Federal Street Acquisition Corp. Warrant		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:03.252198	2020-10-11 23:38:31.031328
FSLR	First Solar Inc.	First Solar Inc is a provider of solar energy solutions. It designs, manufactures and sells PV solar modules with a thin-film semiconductor technology. It also manufactures crystalline silicon solar modules.	{Technology,Solar,Semiconductors}	{CSIQ,GE,SPWR,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:03.264088	2020-10-11 23:38:31.023428
FT	Franklin Universal Trust	Franklin Universal Trust is a closed-end management investment company. The fund's primary investment objective is to provide high, current income consistent with preservation of capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.274714	2020-10-11 23:38:31.021426
FTF	Franklin Limited Duration Income Trust of Beneficial Interest	Franklin Templeton Ltd Duration Inc Tr is a closed-end investment company. It aims to provide substantial current income to its investors by investing in fixed income securities such as corporate bonds.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:03.286899	2020-10-11 23:38:31.024353
FTR	Frontier Communications Corporation	Frontier Communications Corp is a communication company providing phone, Internet, and other data transport services to residential and business customers. It offers a variety of services and products on a standalone basis and as packaged solutions.	{"Communication Services","Telecom Services"}	{SHEN,CTL,TU,GTT}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:12:03.302526	2020-10-11 23:38:31.028166
FTXG	First Trust Nasdaq Food & Beverage ETF	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the Nasdaq US Smart Food & Beverage Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the underlying index. The underlying index is designed to select food and beverage stocks from the NASDAQ US Benchmark Index based on a ranking methodology of three price factors which aims to select companies. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.313228	2020-10-11 23:38:31.024353
FUL	H. B. Fuller Company	H.B. Fuller Company is a formulator, manufacturer and marketer of adhesives, sealants and other specialty chemical products for applications in various markets, including automotive, electronics, construction products, durable assembly, and packaging.	{"Basic Materials","Specialty Chemicals",Chemicals}	{PPG,RPM,SHW}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:03.327362	2020-10-11 23:38:31.025871
FVAL	Fidelity Value Factor	The investment seeks to provide investment returns that correspond, before fees and expenses, generally to the performance of the Fidelity U.S. Value Factor Index℠.\n The fund normally invests at least 80% of assets in securities included in the Fidelity U.S. Value Factor Index℠, which is designed to reflect the performance of stocks of large and mid-capitalization U.S. companies that have attractive valuations. It may lend securities to earn income for the fund.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.339195	2020-10-11 23:38:31.021426
FXC	Invesco CurrencyShares Canadian Dollar Trust	The investment seeks to track the price of the Canadian Dollar, net of trust expenses.\n The fund seeks to reflect the price of the Canadian Dollar. The sponsor believes that, for many investors, the shares represent a cost-effective investment relative to traditional means of investing in the foreign exchange market.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.358084	2020-10-11 23:38:31.02582
MVT	Blackrock MuniVest Fund II Inc.	Blackrock Munivest Fund II Inc is a diversified, closed-end management investment company. The Fund's investment objective is to provide total return through a combination of current income and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.395011	2020-10-11 23:38:31.02582
FOR	Forestar Group Inc	Forestar Group Inc is a residential and mixed-use real estate development company and also engaged in mineral resources and other segments.	{"Real Estate","Real Estate - General","Real Estate Services"}	{STRS,JOE,CKX,TRC,HHC,FCE.A}	Real Estate Services	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:03.178776	2020-10-11 23:38:31.02423
FPE	First Trust Preferred Securities and Income ETF	The investment seeks total return and to provide current income.\n Under normal market conditions, the fund invests at least 80% of its net assets (including investment borrowings) in preferred securities ("Preferred Securities") and income-producing debt securities ("Income Securities"). The fund invests in securities that are traded over-the-counter or listed on an exchange.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.196794	2020-10-11 23:38:31.02423
FRME	First Merchants Corporation	First Merchants Corp through its subsidiaries provides its customers with financial services delivered locally by bankers. It offers personal banking, business banking, mortgage lending, cash management services, wealth management and insurance.	{"Financial Services","Banks - Regional - US",Banks}	{THFF,LKFN,GABC,SRCE,HBNC,MFSF,ONB}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:03.237118	2020-10-11 23:38:31.028166
FSACU	Federal Street Acquisition Corp. Unit consisting of One Common Stock and Half of a Warrant	Federal Street Acquisition Corp is a blank check company formed for the purpose of effecting a merger, capital stock exchange, asset acquisition, stock purchase, reorganization or similar business combination with one or more businesses.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:03.251529	2020-10-11 23:38:31.028166
FSM	Fortuna Silver Mines Inc (Canada)	Fortuna Silver Mines Inc is a Canada-based precious metals producer. Its business operations comprised of mining and related activities in Latin America, including exploration, extraction, and processing of silver- lead, zinc, and silver-gold.	{"Basic Materials",Silver,"Metals & Mining"}	{SA}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:03.263929	2020-10-11 23:38:31.028166
FSZ	First Trust Switzerland AlphaDEX Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the NASDAQ AlphaDEX® Switzerland Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks and/or depositary receipts that comprise the index. The index is designed to select stocks from the NASDAQ Switzerland Index (the "base index") that may generate positive alpha, or risk-adjusted returns, relative to traditional indices through the use of the AlphaDEX® selection methodology.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.273829	2020-10-11 23:38:31.02423
FTEC	Fidelity MSCI Information Technology Index	The investment seeks to provide investment returns that correspond, before fees and expenses, generally to the performance of the MSCI USA IMI Information Technology Index.\n The fund invests at least 80% of assets in securities included in the fund's underlying index. The fund's underlying index is the MSCI USA IMI Information Technology Index, which represents the performance of the information technology sector in the U.S. equity market. It may or may not hold all of the securities in the MSCI USA IMI Information Technology Index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.285478	2020-10-11 23:38:31.02423
FTNW	FTE Networks Inc.	FTE Networks Inc acts as a provider of international and regional telecommunications and technology systems and infrastructure services. It also offers managed information technology, telecommunications services and staffing solutions.	{Technology,"Communication Equipment"}	{S,XLK}	Communication Equipment	Technology	NYSE American	\N	2020-10-11 23:12:03.301392	2020-10-11 23:38:31.02423
FTV	Fortive Corporation	Fortive Corp is a United States based company diversified industrial growth company. It has two business segments namely, Professional Instrumentation and Industrial Technologies.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{MKSI,MLAB,PRCP,LGL,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:03.312198	2020-10-11 23:38:31.023007
FTXN	First Trust Nasdaq Oil & Gas ETF	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of the Nasdaq US Smart Oil & Gas Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is designed to select oil and gas stocks from the NASDAQ US Benchmark Index based on a ranking methodology of three price factors which aims to select companies that exhibit: strong growth through high average 3-, 6-, 9- and 12-month price return; value; and low expected volatility based on historical stock price fluctuation. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.322145	2020-10-11 23:38:31.028166
FUV	Arcimoto Inc.	Arcimoto Inc is an electric vehicle company. It is engaged in designing and developing two-seat, three-wheeled electric vehicle. Its product include SRK Generation 8 electric vehicle.	{"Consumer Cyclical","Recreational Vehicles",Autos}	{TSRI}	Autos	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:03.336639	2020-10-11 23:38:31.023428
FWONK	Liberty Media Corporation Series C	Liberty Formula One Group is a holding company which owns interests in media, communications and entertainment businesses. The company makes investments in various businesses across the international markets.	{"Consumer Cyclical","Broadcasting - TV",Entertainment}	{NXST}	Entertainment	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:03.354343	2020-10-11 23:38:31.023428
FPRX	Five Prime Therapeutics Inc.	Five Prime Therapeutics Inc is a clinical-stage biotechnology company. It is engaged in discovering and developing novel protein therapeutics to improve lives of patients with serious diseases.	{Healthcare,Biotechnology}	{ARQL,REGN,GSK,BMY,JNJ,LLY,PFE,XNCR,LXRX,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:03.206926	2020-10-11 23:38:31.025871
FRD	Friedman Industries Inc.	Friedman Industries Inc is engaged in steel processing, pipe manufacturing and processing and steel and pipe distribution. The Company has two product groups coil and tubular products.	{"Basic Materials",Steel}	{ZEUS,USAP,AKS,RS,HSC,NUE,WOR,ATI,X}	Steel	Basic Materials	NYSE American	\N	2020-10-11 23:12:03.22746	2020-10-11 23:38:31.023428
FRBK	Republic First Bancorp Inc.	Republic First Bancorp Inc through its subsidiary offers commercial and consumer banking services to individuals, small and medium-sized businesses, executives, professionals and professional organizations.	{"Financial Services","Banks - Regional - US",Banks}	{CZNC,ORRF,CCNE,MPB,PWOD,STBA,ACNB,TSC,CUBI}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:03.225086	2020-10-11 23:38:31.023007
FRPT	Freshpet Inc.	Freshpet Inc manufactures and markets refrigerated meals and treats for dogs and cats. The company's products are distributed throughout the United States and Canada into retail classes including Grocery and Mass.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{SMG,POST,PAHC}	Consumer Packaged Goods	Consumer Defensive	NASDAQ Global Market	\N	2020-10-11 23:12:03.241399	2020-10-11 23:38:31.02423
FRSX	Foresight Autonomous Holdings Ltd.	Foresight Autonomous Holdings Ltd is a technology company which is engaged in the design, development and commercialization of 3D multi-camera-based advanced Driver Assistance Systems. Its product include Eyes-On.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{STRS,XLK}	Computer Hardware	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:03.246957	2020-10-11 23:38:31.023007
FSBC	FSB Bancorp Inc.	FSB Bancorp Inc is a bank holding company. The Company through its banking subsidiaries is engaged in providing banking products and services including checking and savings, lending and other mortgage services.	{"Financial Services","Banks - Regional - US",Banks}	{FDEF,WSFS,OCFC,BFIN,CSV,BKU,BHLB,EBSB,WAYN}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:03.255216	2020-10-11 23:38:31.024353
FSD	First Trust High Income Long Short Fund of Beneficial Interest	First Trust High Income Long/Short Fund is an asset management company. Its business objective is to provide a high income to its investors by investing primarily in high-yield corporate fixed-income securities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.261315	2020-10-11 23:38:31.021426
FSTR	L.B. Foster Company	L.B. Foster Co manufactures, fabricates and distributes products and services for the rail, construction, energy and utility markets.	{Industrials,Railroads,"Transportation & Logistics"}	{ZEUS,STLD,CVGI,RAIL,ARII,GBX,RS,WAB}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:03.27318	2020-10-11 23:38:31.028166
FTAG	First Trust Indxx Global Agriculture ETF	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the Indxx Global Agriculture Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks and/or depositary receipts that comprise the index. The index is a market capitalization weighted index designed to measure the performance of companies that are directly or indirectly engaged in improving agricultural yields. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.279135	2020-10-11 23:38:31.023428
FTEO	FRONTEO Inc.	Fronteo Inc is a technology and services company. It offers eDiscovery and Forensic services to government, domestic and international law firms and private companies such as automobile, machinery, precision and electrical.	{Technology,"Information Technology Services","Application Software"}	{XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:03.287073	2020-10-11 23:38:31.028166
FTLB	First Trust Low Beta Income ETF	The investment seeks to provide current income.\n The fund will pursue its objective by investing in equity securities listed on U.S. exchanges and by utilizing an "option strategy" consisting of buying U.S. exchange-traded put options on the Standard & Poor's 500® Index (the "index") and writing (selling) U.S. exchange-traded covered call options on the index. The call options written by the fund will be a laddered portfolio of call options with expirations of less than one year, written at-the-money to slightly out-of-the-money. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.29777	2020-10-11 23:38:31.028166
FTSI	FTS International Inc.	FTS International Inc is a provider of hydraulic fracturing services in North America based on both active and total horsepower of its equipment. It operates in the United States.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:03.309414	2020-10-11 23:38:31.021426
FRAN	Francesca's Holdings Corporation	Francescas Holdings Corp is a specialty retailer, which operates a nationwide-chain of boutiques selling apparel, jewelry, accessories and gifts.	{"Consumer Cyclical","Apparel Stores","Retail - Apparel & Specialty"}	{URBN,ZUMZ,CHS,NWY,EXPR,TLRD}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:03.217605	2020-10-11 23:38:31.02582
FRGI	Fiesta Restaurant Group Inc.	Fiesta Restaurant Group Inc through its subsidiaries owns, operates, and franchises fast-casual restaurants. It operates its fast-casual restaurants under the Pollo Tropical and Taco Cabana brand names.	{"Consumer Cyclical",Restaurants}	{JACK,LUB,SYY,YUM,DPZ,CMG}	Restaurants	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:03.232186	2020-10-11 23:38:31.024353
FSAC	Federal Street Acquisition Corp.	Federal Street Acquisition Corp is a blank check company formed for the purpose of effecting a merger, capital stock exchange, asset acquisition, stock purchase, reorganization or similar business combination with one or more businesses.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:03.25134	2020-10-11 23:38:31.023428
FSCT	ForeScout Technologies Inc.	ForeScout Technologies Inc develops technology that discovers and classifies IP-based devices in real time as they connect to the network and continuously monitors and assesses their security posture.	{Technology,"Software - Application","Application Software"}	{NTGR,XLK}	Application Software	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:03.25656	2020-10-11 23:38:31.02582
FSNN	Fusion Connect Inc.	Fusion Connect Inc, formerly Fusion Telecommunications International Inc offers cloud communications, cloud connectivity, cloud infrastructure, cloud computing, and managed cloud-based applications solutions, and domestic and international voice services.	{"Communication Services","Telecom Services"}	{}	Communication Services	Communication Services	NASDAQ Capital Market	\N	2020-10-11 23:12:03.266174	2020-10-11 23:38:31.031328
FTD	FTD Companies Inc.	FTD Companies Inc is a floral and gifting company. The company provides floral, specialty foods, gift and related products and services to consumers, retail florists, and other retail locations and companies in need of floral and gifting solutions.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{FLWS,PRTS,LE}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:03.286541	2020-10-11 23:38:31.028166
FTFT	Future FinTech Group Inc.	Future FinTech Group Inc is a holding company. The Company is engaged in the production and sale of fruit juice concentrates, fruit beverages and other fruit-related products.	{"Consumer Defensive","Beverages - Soft Drinks","Beverages - Non-Alcoholic"}	{}	Beverages - Non-Alcoholic	Consumer Defensive	NASDAQ Global Market	\N	2020-10-11 23:12:03.286571	2020-10-11 23:38:31.031328
FTI	TechnipFMC plc	TechnipFMC PLC is an oil and gas engineering and construction firm. The company owns a large fleet of vessels used to install offshore pipelines. It also provides onshore and offshore services for various projects.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{NOV,OIS}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:03.296326	2020-10-11 23:38:31.023007
FTK	Flotek Industries Inc.	Flotek Industries Inc develops and supplies chemistry and services to the oil and gas industries, and high value compounds to companies that make cleaning products, cosmetics, food and beverages and other products.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{NOV,SLB,DRQ,ECL,NR,OIS,BHGE}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:03.297863	2020-10-11 23:38:31.025871
FTSM	First Trust Enhanced Short Maturity ETF	The investment seeks current income, consistent with preservation of capital and daily liquidity.\n Under normal market conditions, the Advisor intends to achieve its investment objective by investing at least 80% of its net assets in a portfolio of U.S. dollar-denominated fixed- and variable-rate instruments (collectively, "Fixed Income Securities") issued by U.S. and non-U.S. public and private sector entities. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.311549	2020-10-11 23:38:31.031328
FTVA	Aptus Fortified Value	The investment seeks to track the performance, before fees and expenses, of the Aptus Fortified Value Index (the "index").\n Under normal circumstances, at least 80% of the fund's total assets (exclusive of any collateral held from securities lending) will be invested in the component securities of the index. The index is a rules-based, equal-weighted index that is designed to gain exposure to 50 of the most undervalued U.S.-listed common stocks and real estate investment trusts ("REITs"), while hedging against significant U.S. equity market declines when the market is overvalued. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:03.31191	2020-10-11 23:38:31.023428
FUE	AB Svensk Ekportkredit (Swedish Export Credit Corporation) ELEMENTS Linked to the MLCX Biofuels Index - Total Return Structured Product	The investment seeks to replicate, net of expenses, the MLCX Biofuels Total Return Index.\n The index is designed to reflect the performance of a fully collateralized investment in the seven exchange-traded futures contracts on seven physical commodities: barley, canola, corn, rapeseed, soybeans, soybean oil and sugar.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.323914	2020-10-11 23:38:31.023007
FRPH	FRP Holdings Inc.	FRP Holdings Inc is a real estate company. The business activities of the group are operated through Asset Management Segment, Mining Royalty Lands Segment, Land Development and Construction Segment and RiverFront on the Anacostia Segment.	{"Real Estate","Real Estate Services"}	{GRIF,TRNO,IOR}	Real Estate Services	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:03.238345	2020-10-11 23:38:31.031328
FSBW	FS Bancorp Inc.	FS Bancorp Inc is a bank holding company for 1st Security Bank. The bank delivers banking and financial services to local families, local and regional businesses and industry niches within distinct Puget Sound area communities.	{"Financial Services","Banks - Regional - US",Banks}	{PROV,RVSB,TSBK,BYFC,FFNW,TBNK,ANCB,BANC}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:03.256363	2020-10-11 23:38:31.025871
FSFG	First Savings Financial Group Inc.	First Savings Financial Group Inc is a saving and loan holding company. The Company through its subsidiary offers traditional financial services to consumers and businesses in its main market area.	{"Financial Services","Banks - Regional - US",Banks}	{FDEF,FCAP,IROQ,FBC}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:03.260761	2020-10-11 23:38:31.02423
FSP	Franklin Street Properties Corp.	Franklin Street Properties Corp operates as a real estate investment trust focused on commercial real estate investments in office markets.	{"Real Estate","REIT - Office",REITs}	{VNO,HIW,BDN,CLI,OFC,LXP}	REITs	Real Estate	NYSE American	\N	2020-10-11 23:12:03.266768	2020-10-11 23:38:31.02582
FTC	First Trust Large Cap Growth AlphaDEX Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the Nasdaq AlphaDEX® Large Cap Growth Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is designed to select growth stocks from the NASDAQ US 500 Large Cap Index (the "base index") that may generate positive alpha, or risk-adjusted returns, relative to traditional indices through the use of the AlphaDEX® selection methodology.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.281634	2020-10-11 23:38:31.02582
FTGC	First Trust Global Tactical Commodity Strategy Fund	The investment seeks to provide total return by providing investors with commodity exposure while seeking a relatively stable risk profile.\n The fund is an actively managed exchange-traded fund ("ETF") that seeks to achieve attractive risk adjusted return by investing in commodity futures contracts and exchange-traded commodity linked instruments (collectively, "Commodities Instruments") through a wholly-owned subsidiary of the fund organized under the laws of the Cayman Islands (the "Subsidiary"). The advisor expects to gain exposure to these investments exclusively by investing in the Subsidiary.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.289533	2020-10-11 23:38:31.02582
FTHI	First Trust High Income ETF	The investment seeks current income; capital appreciation is a secondary objective.\n The fund will pursue its objectives by investing in equity securities listed on U.S. exchanges and by utilizing an "option strategy" consisting of writing (selling) U.S. exchange-traded covered call options on the Standard & Poor's 500® Index (the "index"). The call options written by the fund will be a laddered portfolio of call options with expirations of less than one year, written at-the-money to slightly out-of-the-money. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.291143	2020-10-11 23:38:31.021426
FTNT	Fortinet Inc.	Fortinet Inc software solutions provider. The company develops and sells a portfolio of security software products which includes firewall, threat detection, WLAN and switching, application security, identity and access management.	{Technology,"Software - Application","Application Software"}	{CSCO,ADTN,CHKP,FFIV,SYMC,NTGR,JNPR,PANW,FEYE,HPQ,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:03.299692	2020-10-11 23:38:31.023428
FTSD	Franklin Liberty Short Duration U.S. Government	The investment seeks a high level of current income as is consistent with prudent investing, while seeking preservation of capital.\n Under normal market conditions, the fund invests at least 80% of its net assets in securities issued or guaranteed by the U.S. government, its agencies, or instrumentalities. It is an actively managed exchange-traded fund ("ETF") that does not seek to replicate the performance of a specified index. To pursue its investment goal, it may invest in certain interest rate-related derivative transactions, principally U.S. Treasury and fixed income index futures contracts.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.309721	2020-10-11 23:38:31.028166
FTXD	First Trust Nasdaq Retail ETF	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of the Nasdaq US Smart Retail Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is designed to select retail stocks from the NASDAQ US Benchmark Index based on a ranking methodology of three price factors which aims to select companies that exhibit: strong growth through high average 3-, 6-, 9- and 12-month price return; value; and low expected volatility based on historical stock price fluctuation. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.313192	2020-10-11 23:38:31.028166
FUN	Cedar Fair L.P.	Cedar Fair LP is a regional amusement park operator. It owns amusement parks, outdoor water parks, indoor water park and hotels across the United States.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{SIX,SEAS}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:03.325191	2020-10-11 23:38:31.02582
FTCS	First Trust Capital Strength ETF	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the Capital Strength Index(SM).\n The fund will normally invest at least 90% of its net assets in common stocks that comprise the index. In constructing the index, the index provider begins with the largest 500 U.S. companies included in the NASDAQ US Benchmark Index and excludes the following: companies with less than $1 billion in cash and short term investments; companies with long-term debt divided by market capitalization greater than 30%; and companies with return on equity less than 15%.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.283745	2020-10-11 23:38:31.025871
FTRI	First Trust Indxx Global Natural Resources Income	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the Indxx Global Natural Resources Income Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks and/or depositary receipts that comprise the index. The index is a 50-stock free float adjusted market capitalization weighted index designed to measure the market performance of the 50 highest dividend yielding companies involved in the upstream segment of the natural resources sector. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.304495	2020-10-11 23:38:31.02582
FTXL	First Trust Nasdaq Semiconductor ETF	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of the Nasdaq US Smart Semiconductor Index.\n The fund invests at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is designed to select semiconductor stocks from the NASDAQ US Benchmark Index based on a ranking methodology of three price factors which aims to select companies that exhibit: strong growth through high average 3-, 6-, 9- and 12-month price return; value; and low expected volatility based on historical stock price fluctuation. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.315439	2020-10-11 23:38:31.02582
FUNC	First United Corporation	First United Corp is a financial services provider in the United States. It owns the stakes to banks which provide checking, savings, different types of loans, insurance and trust services to individuals and businesses.	{"Financial Services","Banks - Regional - US",Banks}	{GLBZ,SASR,SHBI,EGBN,OLBK,FNB,HBMD}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:03.329655	2020-10-11 23:38:31.028166
FWDB	Madrona Global Bond	The investment seeks investment results that exceed the price and yield performance of its benchmark, the Bloomberg Barclays U.S. Aggregate Bond Index.\n The Sub-Advisor selects a portfolio of fixed income (bond) exchange-traded funds ("Underlying ETFs") and other exchange-traded products, including, but not limited to, exchange-traded notes ("ETNs"), exchange-traded currency trusts, and exchange-traded commodity pools. It invests in ETPs that provide exposure to at least 12 distinct global bond classes, that may include treasury bonds, municipal bonds, and high-yield U.S. corporate bonds (sometimes referred to as "junk bonds").	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.347365	2020-10-11 23:38:31.02423
FXD	First Trust Cons. Discret. AlphaDEX	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the StrataQuant® Consumer Discretionary Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is a modified equal-dollar weighted index designed by IDI to objectively identify and select stocks from the Russell 1000® Index in the consumer discretionary sector that may generate positive alpha relative to traditional passive-style indices through the use of the AlphaDEX® selection methodology.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.362235	2020-10-11 23:38:31.028166
FXY	Invesco CurrencyShares Japanese Yen Trust	The investment seeks to track the price of the Japanese Yen, net of trust expenses.\n The fund seeks to reflect the price of the Japanese Yen. The sponsor believes that, for many investors, the shares represent a cost-effective investment relative to traditional means of investing in the foreign exchange market.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.378451	2020-10-11 23:38:31.024353
GABC	German American Bancorp Inc.	German American Bancorp is a bank holding company. The firm is in the business of retail and commercial banking, comprehensive financial planning, full-service brokerage and trust administration and a range of personal and corporate insurance products.	{"Financial Services","Banks - Regional - US",Banks}	{THFF,LKFN,FRME,SRCE,HBNC,MFSF,ONB,ASB}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:03.393207	2020-10-11 23:38:31.023428
GAMR	ETFMG Video Game Tech	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the price and yield performance of the EEFund Video Game Tech Index.\n The index tracks the performance of the common stock of exchange-listed companies across the globe that are actively engaged in a business activity supporting or utilizing the video gaming industry. The fund normally invests at least 80% of its total assets, exclusive of collateral held from securities lending, in securities, ADRs, or GDRs of Video Gaming Companies. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.404812	2020-10-11 23:38:31.02423
GBIL	Goldman Sachs Access Treasury 0-1 Year	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the FTSE US Treasury 0-1 Year Composite Select Index.\n The fund seeks to achieve its investment objective by investing at least 80% of its assets (exclusive of collateral held from securities lending) in securities included in its underlying index. The index is designed to measure the performance of U.S. Treasury Obligations with a maximum remaining maturity of 12 months. The investment adviser uses a representative sampling strategy to manage the fund.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.419725	2020-10-11 23:38:31.023428
GBR	New Concept Energy Inc	New Concept Energy Inc is an oil and gas company which through its subsidiaries, owns and operates oil and gas wells and mineral leases in Athens and Meigs Counties in Ohio and in Calhoun, Jackson and Roane Counties in West Virginia.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{CKX,ECT}	Oil & Gas - E&P	Energy	NYSE American	\N	2020-10-11 23:12:03.43016	2020-10-11 23:38:31.024353
FVC	First Trust Dorsey Wright Dynamic Focus 5 ETF	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an index called the Indxx Blockchain Index (the "index").\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks and depositary receipts that comprise the index. The index is designed to track the performance of companies that are either actively using, investing in, developing, or have products that are poised to benefit from blockchain technology and/or the potential for increased efficiency that it provides to various business processes. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.3403	2020-10-11 23:38:31.028166
FWRD	Forward Air Corporation	Forward Air Corp serves in the logistics industry based in the United States. It is a provider of time-definite truck transportation to the North American deferred air freight shipping market.	{Industrials,"Integrated Shipping & Logistics","Transportation & Logistics"}	{EXPD,ODFL,ULH,SAIA,FDX,RRTS,ARCB}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:03.354603	2020-10-11 23:38:31.031328
FXI	iShares China Large-Cap	The investment seeks to track the investment results of the FTSE China 50 Index composed of large-capitalization Chinese equities that trade on the Hong Kong Stock Exchange.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The index is designed to track the performance of the largest companies in the Chinese equity market that trade on the Hong Kong Stock Exchange and are available to international investors, as determined by FTSE International Limited (the "index provider" or "FTSE"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.368249	2020-10-11 23:38:31.031328
FYC	First Trust Small Cap Growth AlphaDEX Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the Nasdaq AlphaDEX® Small Cap Growth Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is designed to select growth stocks from the NASDAQ US 700 Small Cap Index (the "base index") that may generate positive alpha, or risk-adjusted returns, relative to traditional indices through the use of the AlphaDEX® selection methodology.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.384288	2020-10-11 23:38:31.025871
GALT	Galectin Therapeutics Inc.	Galectin Therapeutics Inc is a clinical stage biopharmaceutical company. The company is engaged in drug research and development to create new therapies for fibrotic disease and cancer.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{ARQL,MNTA,MEIP,SVRA,XLV}	Drug Manufacturers	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:03.403945	2020-10-11 23:38:31.028166
GBLI	Global Indemnity Limited	Global Indemnity Ltd a specialty property and casualty insurer. It offers binding authority, program, brokerage, direct and reinsurance services through its distribution channel in United States.	{"Financial Services","Insurance - Property & Casualty"}	{NAVG,WRB,MKL,RLI,TRV,ARGO}	Insurance - Property & Casualty	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:03.4209	2020-10-11 23:38:31.02423
GCOW	Pacer Global Cash Cows Dividend	The investment seeks to track the total return performance, before fees and expenses, of the Pacer Global Cash Cows Dividend Index.\n Under normal circumstances, the fund will invest at least 80% of its total assets (exclusive of collateral held from securities lending) in the components of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities (e.g., depositary receipts). The index uses an objective, rules-based methodology to provide exposure to global companies with high dividend yields backed by a high free cash flow yield. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:03.442619	2020-10-11 23:38:31.025871
GDS	GDS Holdings Limited	GDS Holdings Ltd is a developer and operator of data centers in PRC. The Company offers colocation and managed services, including managed cloud services to internet companies, financial institutions, telecommunications and IT service providers.	{Technology,"Data Storage","Computer Hardware"}	{PDFS,OMCL,XLK}	Computer Hardware	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:03.46266	2020-10-11 23:38:31.031328
GEM	Goldman Sachs ActiveBeta Emerging Markets Equity	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the Goldman Sachs ActiveBeta® Emerging Markets Equity Index.\n The fund invests at least 80% of its assets (exclusive of collateral held from securities lending) in securities included in its underlying index, in depositary receipts representing securities included in its underlying index and in underlying stocks in respect of depositary receipts included in its underlying index. The index is designed to deliver exposure to equity securities of emerging market issuers.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.482602	2020-10-11 23:38:31.024353
GFED	Guaranty Federal Bancshares Inc.	Guaranty Federal Bancshares Inc is a savings and loan holding company. It offers commercial real estate loans, multi-family residential mortgage loans, permanent one- to four-family residential mortgage loans, business, consumer and other loans..	{"Financial Services","Banks - Regional - US",Banks}	{SMBC,HMNF,CBSH,GSBC,LARK,QCRH,CASH,HWBK}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:03.502939	2020-10-11 23:38:31.02423
GGM	Guggenheim Credit Allocation Fund of Beneficial Interest	Guggenheim Credit Allocation Fund is a diversified, closed-end management investment company. The Fund's investment objective is to seek total return through a combination of current income and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.518548	2020-10-11 23:38:31.023007
FVD	First Trust VL Dividend	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the Value Line® Dividend Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is a modified equal-dollar weighted index comprised of U.S. exchange-listed securities of companies that pay above-average dividends and have potential for capital appreciation.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.341535	2020-10-11 23:38:31.02582
FWP	Forward Pharma A/S	Forward Pharma A/S is a Danish biopharmaceutical company preparing to initiate a clinical trial using FP187, a proprietary formulation of dimethyl fumarate, or DMF, for the treatment of multiple sclerosis, or MS, patients.	{Healthcare,Biotechnology}	{NLNK,RTRX,GNCA,ADRO,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:03.354357	2020-10-11 23:38:31.023007
FXH	First Trust Health Care AlphaDEX	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the StrataQuant® Health Care Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is a modified equal-dollar weighted index designed by IDI to objectively identify and select stocks from the Russell 1000® Index in the health care sector that may generate positive alpha relative to traditional passive-style indices through the use of the AlphaDEX® selection methodology.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.364674	2020-10-11 23:38:31.028166
FXR	First Trust Industrials AlphaDEX	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the StrataQuant® Industrials Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is a modified equal-dollar weighted index designed by IDI to objectively identify and select stocks from the Russell 1000® Index in the industrials and producer durables sectors that may generate positive alpha relative to traditional passive-style indices through the use of the AlphaDEX® selection methodology.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.374754	2020-10-11 23:38:31.023007
FYT	First Trust Small Cap Value AlphaDEX Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the Nasdaq AlphaDEX® Small Cap Value Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is designed to select value stocks from the NASDAQ US 700 Small Cap Index (the "base index") that may generate positive alpha, or risk-adjusted returns, relative to traditional indices through the use of the AlphaDEX® selection methodology.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.385917	2020-10-11 23:38:31.021426
GAIA	Gaia Inc.	Gaia Inc along with its subsidiaries provides yoga, fitness & wellness products with media, subscription & travel services to customers who value personal development, wellness, ecological lifestyles, responsible media & conscious community.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{AMZN,CNXN,PCMI,OSTK,SYX,BBY,DIS,DKS,EVLV,LGF.A}	Retail - Apparel & Specialty	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:03.398605	2020-10-11 23:38:31.024353
GBAB	Guggenheim Taxable Municipal Managed Duration Trust of Beneficial Interest	Guggenheim Build America Mngd Duration is a diversified closed-end management investment company. Its investment objective is to provide current income with a secondary objective of long-term capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.415792	2020-10-11 23:38:31.02582
GBT	Global Blood Therapeutics Inc.	Global Blood Therapeutics Inc is a clinical-stage biopharmaceutical company in the United States. Its major focus lies on developing therapeutics for the treatment of blood-based disorders such as red blood cell sickling.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:03.431721	2020-10-11 23:38:31.021426
GCV	Gabelli Convertible and Income Securities Fund Inc. (The)	Primary objective is to seek a high level of total return through a combination of current income and capital appreciation by investing in convertible and income-producing securities. At least 80% of the Funds net assets will be invested in a combination of the latter.	{"Financial Services","Asset Management"}	{}			New York Stock Exchange	\N	2020-10-11 23:12:03.447842	2020-10-11 23:38:31.023428
GDVD	Principal Active Global Dividend Income	The investment seeks current income and long-term growth of income and capital.\n The fund is an actively managed ETF that seeks to achieve its investment objective by investing, under normal circumstances, at least 80% of its net assets, plus any borrowings for investment purposes, in dividend-paying equity securities at the time of purchase. It invests in equity securities of small, medium and large market capitalization companies and in growth and value stocks.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:03.463404	2020-10-11 23:38:31.028166
GENE	Genetic Technologies Ltd	Genetic Technologies Ltd is a molecular diagnostics company which offers predictive testing and assessment tools to help physicians manage women's health. It has developed BREVAGenplus, which is a is a clinically validated risk assessment test.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{MYGN,NEOG,CGEN,DGX,BTX,DHR,TMO,LH,XLV}	Medical Diagnostics & Research	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:03.488682	2020-10-11 23:38:31.02423
FUTY	Fidelity MSCI Utilities Index	The investment seeks to provide investment returns that correspond, before fees and expenses, generally to the performance of the MSCI USA IMI Utilities Index.\n The fund invests at least 80% of assets in securities included in the fund's underlying index. The fund's underlying index is the MSCI USA IMI Utilities Index, which represents the performance of the utilities sector in the U.S. equity market. It may or may not hold all of the securities in the MSCI USA IMI Utilities Index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.334636	2020-10-11 23:38:31.028166
FXA	Invesco CurrencyShares Australian Dollar Trust	The investment objective of the fund is for the Shares to reflect the price in USD of the Australian Dollar.\nThe Shares are intended to provide institutional and retail investors with a simple, cost-effective means of gaining investment benefits similar to those of holding Australian Dollars. The costs of purchasing Shares should not exceed the costs associated with purchasing any other publicly-traded equity securities.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.357199	2020-10-11 23:38:31.024353
FXL	First Trust Technology AlphaDEX	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the StrataQuant® Technology Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is a modified equal-dollar weighted index designed by IDI to objectively identify and select stocks from the Russell 1000® Index in the technology sector that may generate positive alpha relative to traditional passive-style indices through the use of the AlphaDEX® selection methodology.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.370134	2020-10-11 23:38:31.024353
GAA	Cambria Global Asset Allocation	The investment seeks to track the performance, before fees and expenses, of the Cambria Global Asset Allocation Index.\n Under normal market conditions, the fund invests at least 80% of its total assets in the components of the underlying index and in depositary receipts representing components of the underlying index. The underlying index is designed to provide diversified exposure, including inverse exposure, to all of the major world asset classes in the various regions, countries and sectors around the globe.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:03.388451	2020-10-11 23:38:31.02423
GARS	Garrison Capital Inc.	Garrison Capital Inc is a closed-end management investment company. Its investment objective is to generate current income and capital appreciation by making investments generally in debt securities and loans of U.S. based middle-market companies.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:03.405688	2020-10-11 23:38:31.023428
GBL	Gamco Investors Inc.	GAMCO Investors Inc provides asset-management services for individual and institutional investors. Through its subsidiaries, it manages separate accounts; Gabelli Funds, and Gabelli Securities. The company's product mix is more inclined towards equities.	{"Financial Services","Asset Management"}	{AMG,WDR,EV,BEN,BLK,AB,IVZ,OAK}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.421094	2020-10-11 23:38:31.023007
GCO	Genesco Inc.	Genesco Inc is a specialty retailer, which is engaged in designing, sourcing, marketing and distribution of footwear, apparel and accessories through retail stores.	{"Consumer Cyclical","Apparel Stores","Retail - Apparel & Specialty"}	{SCVL,FL,DSW,CAL}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:03.436203	2020-10-11 23:38:31.021426
GDO	Western Asset Global Corporate Defined Opportunity Fund Inc.	Western Asset Global Corp Defined Opp is a closed-end management investment company with investment objective to provide current income and then to liquidate and distribute substantially all of its net assets to stockholders on or about December 2, 2024.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.457506	2020-10-11 23:38:31.028166
GE	General Electric Company	General Electric Co is a digital industrial company. It operates in various segments, including power and water, oil and gas, energy management, aviation, healthcare, transportation, appliances and lighting, and more.	{Industrials,"Diversified Industrials","Industrial Products"}	{CAT,DHR,HON,TMO,UTX,PHG,MMM,MDT}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:03.473456	2020-10-11 23:38:31.024353
GEO	Geo Group Inc (The) REIT	The GEO Group Inc is a healthcare facility real estate investment trust. It is engaged in ownership, leasing and management of correctional, detention and re-entry facilities, and the provision of community based services.	{"Real Estate","REIT - Healthcare Facilities",REITs}	{CXW}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:03.489738	2020-10-11 23:38:31.023007
GF	New Germany Fund Inc. (The)	New Germany Fund is a United States based non-diversified, closed-end management investment company. As the investment objective, the fund seeks long-term capital appreciation through investment in middle market German equities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.500561	2020-10-11 23:38:31.023428
GFY	Western Asset Variable Rate Strategic Fund Inc.	Western Asset Variable Rate Strategic Fd is a non-diversified, closed-end management investment company. Its investment objective is to maintain a high level of current income by investing in variable rate instruments of U.S. and non-U.S. issuers.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.514202	2020-10-11 23:38:31.02423
FXB	Invesco CurrencyShares British Pound Sterling Trust	The investment seeks to reflect the price in USD of the British Pound Sterling.\n The shares are intended to provide institutional and retail investors with a simple, cost-effective means of gaining investment benefits similar to those of holding British Pounds Sterling.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.356824	2020-10-11 23:38:31.02423
FXO	First Trust Financials AlphaDEX	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the StrataQuant® Financials Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is a modified equal-dollar weighted index designed by IDI to objectively identify and select stocks from the Russell 1000® Index in the financial services sector that may generate positive alpha relative to traditional passive-style indices through the use of the AlphaDEX® selection methodology.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.372273	2020-10-11 23:38:31.028166
FYX	First Trust Small Cap Core AlphaDEX Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the Nasdaq AlphaDEX® Small Cap Core Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is designed to select stocks from the NASDAQ US 700 Small Cap Index (the "base index") that may generate positive alpha, or risk-adjusted returns, relative to traditional indices through the use of the AlphaDEX® selection methodology.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.389449	2020-10-11 23:38:31.028166
GASL	Direxion Daily Natural Gas Related Bull 3X Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the daily performance of the ISE-Revere Natural Gas IndexTM.\n The fund, under normal circumstances, invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, exchange-traded funds ("ETFs") that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index is designed to take advantage of both event-driven news and long-term trends in the natural gas industry. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.407106	2020-10-11 23:38:31.023007
GBF	iShares Government/Credit Bond	The investment seeks to track the investment results of the Bloomberg Barclays U.S. government/Credit Bond Index (the "underlying index").\n The underlying index measures the performance of U.S. dollar-denominated U.S. Treasury bonds, government-related bonds and investment-grade U.S. corporate bonds that have a remaining maturity of greater than or equal to one year. The fund generally invests at least 90% of its assets in securities of the underlying index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.419155	2020-10-11 23:38:31.028166
GCC	WisdomTree Continuous Commodity Index Fund	The investment seeks, through its investment in the master fund, to reflect the performance of the index, over time, less the expenses of the fund and the master fund’s overall operations.\n The master fund invests in a portfolio of index commodities, as well as holding cash and United States Treasury securities and other high credit quality short-term fixed income securities for deposit with the master fund’s Commodity Broker as margin. The Continuous Commodity Total Return Index is a broad based commodity index that reflects the price movement of 17 exchange-traded futures contracts.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.435316	2020-10-11 23:38:31.028166
GD	General Dynamics Corporation	General Dynamics Corp is an aerospace and defense company. It offers products and services in business aviation including combat vehicles, weapons systems and munitions, shipbuilding, and communication and information technology services and solutions.	{Industrials,"Aerospace & Defense"}	{KTOS,LMT,BA,HON,NOC,TXT,RTN,COL}	Aerospace & Defense	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:03.450236	2020-10-11 23:38:31.031328
GDV	Gabelli Dividend & Income Trust of Beneficial Interest	The funds investment objective is to provide a high level of total return on its assets with an emphasis on dividends and income. Under normal market conditions, the Fund will invest at least 80% of the value of its total assets in dividend paying or income producing equity or debt securities. Under normal market conditions, at least 50% of the Funds assets will consist of dividend paying equity securities. The Fund may invest up to 35% of its total assets in securities on non-US issuers and up to 25% of its total assets in securities of issuers in a single industry. There is no minimum credit rating for debt securities in which the Fund may invest, although the Fund will not invest more than 10% of total assets in fixed income non-convertible securities.	{"Financial Services","Asset Management"}	{}			New York Stock Exchange	\N	2020-10-11 23:12:03.463153	2020-10-11 23:38:31.023428
GECCM	Great Elm Capital Corp. 6.75% Notes Due 2025		{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.476699	2020-10-11 23:38:31.023007
GENY	Principal Millennials Index ETF	The investment seeks to provide investment results that closely correspond, before expenses, to the performance of the Nasdaq Global Millennial Opportunity Index (the "index").\n Under normal circumstances, the fund invests at least 80% of its net assets, plus any borrowings for investment purposes, in equity securities of companies that compose the index at the time of purchase. The index uses a quantitative model designed to identify equity securities of companies in the Nasdaq Global Index (the "parent index") that are impacted by the spending and lifestyle activities of the Millennial generation, which refers to people born from 1980 to the mid-2000s.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.488715	2020-10-11 23:38:31.028166
OCFC	OceanFirst Financial Corp.	OceanFirst Financial Corp operates in the banking services segment. Its key business is the provision of loans and deposit products to customers in the United States.	{"Financial Services","Savings & Cooperative Banks",Banks}	{MGYR,ORIT,PFS,NFBK,KRNY,MSBF}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:08.267641	2020-10-11 23:38:31.025871
FWONA	Liberty Media Corporation Series A	Liberty Formula One Group is a holding company which owns interests in media, communications and entertainment businesses. The company makes investments in various businesses across the international markets.	{"Consumer Cyclical","Broadcasting - TV",Entertainment}	{NXST}	Entertainment	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:03.349738	2020-10-11 23:38:31.028166
FXCH	Invesco CurrencyShares Chinese Renminbi Trust	The investment seeks to reflect the price in USD of the Chinese Renminbi.\nThe Shares are intended to provide institutional and retail investors with a simple, cost-effective means of gaining investment benefits similar to those of holding Chinese Renminbi. The costs of purchasing Shares should not exceed the costs associated with purchasing any other publicly-traded equity securities.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.362235	2020-10-11 23:38:31.025871
FXP	ProShares Ultrashort FTSE China 50	The investment seeks daily investment results, before fees and expenses, that correspond to two times the inverse (-2x) of the daily performance of the FTSE China 50 Index®.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index consists of 50 of the largest and most liquid Chinese stocks listed and traded on the Stock Exchange of Hong Kong. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.374251	2020-10-11 23:38:31.025871
FYLD	Cambria Foreign Shareholder Yield	The investment seeks investment results that correspond (before fees and expenses) generally to the price and yield performance of its underlying index, the Cambria Foreign Shareholder Yield Index (the "underlying index").\n Under normal market conditions, the fund will invest at least 80% of its total assets in the components of the underlying index and in depositary receipts representing components of the underlying index. The fund may invest up to 20% of its net assets in instruments not included in the underlying index. The underlying index is comprised of equity securities of issuers in developed foreign markets.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:03.38512	2020-10-11 23:38:31.031328
GAIN	Gladstone Investment Corporation	Gladstone Investment Corp is a business development company which is managed externally. Its investment manager is Gladstone Management Corporation. It invests in debt and equity securities of established private businesses operating in the United States.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:03.399225	2020-10-11 23:38:31.025871
GASS	StealthGas Inc.	StealthGas Inc through its subsidiaries owns a fleet of LPG carriers that provides seaborne transportation services to LPG producers and users. It also provides crude oil and product carriers chartered to oil producers, refiners and commodities traders.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{CPLP,TNP,TK,SFL,STNG,ASC,DLNG,FRO}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:03.41286	2020-10-11 23:38:31.024353
GBLIL	Global Indemnity Limited 7.875% Subordinated Notes due 2047		{}	{}			Nasdaq Global Select	\N	2020-10-11 23:12:03.424651	2020-10-11 23:38:31.02582
GCE	Clarymore CEF GS Connect ETN	The investment seeks to replicate, net of expenses, the Claymore CEF Index.\n The index measures the performance of a basket of closed-end funds selected and weighted based on a predefined set of criteria such as liquidity, distributions and market valuation, among other factors.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.435376	2020-10-11 23:38:31.023007
GDEN	Golden Entertainment Inc.	Golden Entertainment Inc is an US based company which focuses on distributed gaming, casino and resort operations. It organizes its business in two segments; Distributed Gaming and Casinos.	{"Consumer Cyclical",Gambling,"Travel & Leisure"}	{PENN,CNTY,CPHC,SGMS,DDE,UWN,CZR,FLL,ERI,NYNY}	Travel & Leisure	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:03.451263	2020-10-11 23:38:31.023007
GDXJ	VanEck Vectors Junior Gold Miners	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the MVIS® Global Junior Gold Miners Index.\n The fund normally invests at least 80% of its total assets in securities that comprise the index. The index includes companies that generate at least 50% of their revenues from gold and/or silver mining/royalties/streaming or have mining projects with the potential to generate at least 50% of their revenues from gold and/or silver when developed. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.467365	2020-10-11 23:38:31.02582
GEL	Genesis Energy L.P.	Genesis Energy LP provides onshore pipeline transportation of crude oil and carbon dioxide, offshore pipeline transportation of crude oil in the Gulf of Mexico, refinery services, marine transportation, and supply and logistics service.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{EPD,EEP,DCP,NS,EEQ,HFC,PSX,MPLX,DK,ANDX}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:03.480279	2020-10-11 23:38:31.02582
GERN	Geron Corporation	Geron Corp is a biopharmaceutical company. It supports the clinical stage development of a telomerase inhibitor, imetelstat.	{Healthcare,Biotechnology}	{INCY,GILD,IMMU,OSIR,BTX,STML,TECH,CTIC,CUR,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:03.493049	2020-10-11 23:38:31.025871
GFNCP	General Finance Corporation Cumulative Redeemable Perpetual Preferred Series C	General Finance Corp manufactures, leases & sells storage & freight containers, portable container buildings, portable liquid storage tank containers & modular buildings. The Company operates in Asia-Pacific area and North America.	{Industrials,"Rental & Leasing Services","Consulting & Outsourcing"}	{}	Consulting & Outsourcing	Industrials	NASDAQ Global Market	\N	2020-10-11 23:12:03.508071	2020-10-11 23:38:31.031328
GGO	The Gabelli Go Anywhere Trust of Beneficial Interest	The Gabelli Go Anywhere Trust is a non-diversified, closed-end investment management company. Its investment objective is total return, consisting of capital appreciation and current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:03.523962	2020-10-11 23:38:31.021426
FWDD	Madrona Domestic	The investment seeks long-term capital appreciation above the capital appreciation of its benchmark, the S&P 500 Index.\n The fund seeks to achieve the fund's investment objective by selecting a portfolio of up to 500 of the largest U.S.-traded equity securities. The Sub-Advisor selects the securities for the fund's portfolio using a weighted allocation system based on consensus analyst estimates of the present value of future expected earnings relative to the share price of each security.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.347184	2020-10-11 23:38:31.028166
FXF	Invesco CurrencyShares Swiss Franc Trust	The investment seeks to track the price of the Swiss Franc, net of trust expenses.\n The fund seeks to reflect the price of the Swiss Franc. The sponsor believes that, for many investors, the shares represent a cost-effective investment relative to traditional means of investing in the foreign exchange market.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.363735	2020-10-11 23:38:31.023428
FXS	Invesco CurrencyShares Swedish Krona Trust	The investment seeks to reflect the price in USD of the Swedish Krona.\n The shares are intended to provide institutional and retail investors with a simple, cost-effective means of gaining investment benefits similar to those of holding Swedish Kronor. The costs of purchasing shares should not exceed the costs associated with purchasing any other publicly-traded equity securities.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.374859	2020-10-11 23:38:31.02423
G	Genpact Limited	Genpact Ltd is a business process and technology management services company that provides finance and accounting, collections and customer service, insurance services, supply chain and procurement, analytics and enterprise application services.	{Industrials,"Business Services"}	{ADP,CTSH,EXLS,ASPS,IBM,ACN,INFY,XRX}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:03.387156	2020-10-11 23:38:31.023007
GAL	SPDR SSgA Global Allocation	The investment seeks to provide capital appreciation.\n The Adviser primarily invests the assets of the fund among exchange traded products ("ETPs") that provide balanced exposure to domestic and international debt and equity securities. The fund typically allocates approximately 60% of its assets to equity securities, though this percentage can vary based on the Adviser's tactical decisions.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.399832	2020-10-11 23:38:31.031328
GASX	Direxion Daily Natural Gas Related Bear 3X Shares	The investment seeks daily investment results, net of expenses, of 300% of the inverse of the daily performance of the ISE-Revere Natural Gas IndexTM.\n The fund, under normal circumstances, invests in swap agreements, futures contracts, short positions or other financial instruments that, in combination, provide inverse (opposite) or short leveraged exposure to the index equal to at least 80% of the fund's net assets (plus borrowing for investment purposes). The index is designed to take advantage of both event-driven news and long-term trends in the natural gas industry. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.413268	2020-10-11 23:38:31.025871
GBNK	Guaranty Bancorp	Guaranty Bancorp is a bank holding company providing banking and other financial services throughout Colorado market to consumers and to small and medium-sized businesses, including the owners and employees of those businesses.	{"Financial Services","Banks - Regional - US",Banks}	{EWBC,GBCI,ZION,FIBK,WAL,NBHC}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:03.425242	2020-10-11 23:38:31.025871
GCI	Gannett Co. Inc.	Gannett Co., Inc. operates as a multi-platform news and information company. It spreads the news all over the USA and UK through digital, mobile, and print products.	{"Consumer Cyclical",Publishing}	{}	Publishing	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:03.436195	2020-10-11 23:38:31.024353
GDDY	GoDaddy Inc. Class A	GoDaddy Inc, through its subsidiaries provides a variety of domain name registration and website hosting services. It also provides cloud-based solutions.	{Technology,"Internet Content & Information","Online Media"}	{LPSN,MCHX,LLNW,XLK}	Online Media	Technology	New York Stock Exchange	\N	2020-10-11 23:12:03.451598	2020-10-11 23:38:31.028166
GDX	VanEck Vectors Gold Miners	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the NYSE Arca Gold Miners Index.\n The fund normally invests at least 80% of its total assets in common stocks and depositary receipts of companies involved in the gold mining industry. The index is a modified market-capitalization weighted index primarily comprised of publicly traded companies involved in the mining for gold and silver. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.46705	2020-10-11 23:38:31.023007
GEF	Greif Inc. Class A	Greif Inc produces industrial packaging products with manufacturing facilities located in over many countries. Its products are steel, fibre and plastic drums, intermediate bulk containers, closure systems for industrial packaging and among others.	{"Consumer Cyclical","Packaging & Containers"}	{SLGN,GFN,TRS,SON,CCK,CYRX}	Packaging & Containers	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:03.479953	2020-10-11 23:38:31.031328
GER	Goldman Sachs MLP Energy Renaissance Fund	Goldman Sachs MLP and Energy Renaissance Fund is a non-diversified, closed-end management investment company. It seeks a high level of total return with an emphasis on current distributions to shareholders.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.491956	2020-10-11 23:38:31.031328
GFF	Griffon Corporation	Griffon Corp manufactures and markets residential, commercial and industrial garage doors to professional installing dealers and home center retail chains. It also provides non-powered landscaping products for homeowners and professionals.	{"Basic Materials","Building Materials"}	{LMT,BMS,BA,ESL,GD,NOC,SON,UTX,PGTI}	Building Materials	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:03.50442	2020-10-11 23:38:31.028166
GGB	Gerdau S.A.	Gerdau SA is engaged in the production and commercialization of steel products. The company's mills located in Argentina, Brazil, Canada, Chile, Colombia, the United States, India, Mexico, Peru, among others.	{"Basic Materials",Steel}	{STLD,SID,SIM,CMC,NUE,X,PKX,VALE,MT}	Steel	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:03.517347	2020-10-11 23:38:31.028166
FXSG	Invesco CurrencyShares Singapore Dollar Trust	The investment objective of the trust is for the shares to reflect the price in USD of the Singapore Dollar. \nThe Shares are intended to provide institutional and retail investors with a simple, cost-effective means of gaining investment benefits similar to those of holding Singapore Dollars. The costs of purchasing shares should not exceed the costs associated with purchasing any other publicly-traded equity securities.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.375927	2020-10-11 23:38:31.028166
GAM	General American Investors Inc.	To achieve long-term capital appreciation, with lesser emphasis placed on current income. The Company invests principally in common stocks believed by its management to have better than average growth potential. Normally, a substantially fully invested position in equities is maintained.	{"Financial Services","Asset Management"}	{}			New York Stock Exchange	\N	2020-10-11 23:12:03.404275	2020-10-11 23:38:31.02582
GBCI	Glacier Bancorp Inc.	Glacier Bancorp Inc is a part of the financial services domain. It is the holding company for the Glacier Bank, which offers a wide range of products and services including deposit, loans and mortgage origination services.	{"Financial Services","Banks - Regional - US",Banks}	{ZION,FIBK,WFC,USB,WAL,NBHC,GBNK}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:03.416182	2020-10-11 23:38:31.028166
GBX	Greenbrier Companies Inc. (The)	Greenbrier Companies Inc designs, manufactures & markets railroad freight car equipment in North America & Europe, marine barges in North America and provides wheel services, railcar refurbishment and parts, leasing & other services to the railroad.	{Industrials,Railroads,"Transportation & Logistics"}	{RAIL,ARII,TRN,WAB}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:03.431466	2020-10-11 23:38:31.023428
GCP	GCP Applied Technologies Inc.	GCP Applied Technologies Inc produces and sells specialty construction chemicals, specialty building materials, and packaging sealants and coatings. The company earns the majority of its revenue from sale of concrete admixtures and cement additives.	{"Basic Materials","Specialty Chemicals",Chemicals}	{INDU,NL,NGVT}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:03.44619	2020-10-11 23:38:31.02423
GDOT	Green Dot Corporation Class A $0.001 par value	Green Dot Corp is a financial technology company. The firm provides financial services for consumers in the United States such as Account Services GoBank mobile checking and Visa- or Mastercard-branded prepaid debit cards.	{"Financial Services","Credit Services"}	{EBAY,TSS,WMT,MA,V}	Credit Services	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.460763	2020-10-11 23:38:31.025871
GEC	Great Elm Capital Group Inc.	Great Elm Capital Group Inc is a diversified investment company. The company operates in three business lines including investment management, real estate and merchant banking.	{"Financial Services","Asset Management"}	{CSGS,SNCR,EVOL}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:03.473981	2020-10-11 23:38:31.028166
GENC	Gencor Industries Inc.	Gencor Industries Inc manufactures heavy machinery used in the production of highway construction materials, synthetic fuels and environmental control equipment. Its products include asphalt plants, combustion systems and fluid heat transfer systems.	{Industrials,"Farm & Construction Equipment","Farm & Construction Machinery"}	{CMCO,ARTW,ASTE,CAT,MTW}	Farm & Construction Machinery	Industrials	NASDAQ Global Market	\N	2020-10-11 23:12:03.487348	2020-10-11 23:38:31.023428
GEVO	Gevo Inc.	Gevo Inc is a renewable chemicals and biofuels company. It is engaged in the development and commercialization of alternatives to petroleum-based products based on isobutanol produced from renewable feedstocks.	{"Basic Materials","Specialty Chemicals",Chemicals}	{}	Chemicals	Basic Materials	NASDAQ Capital Market	\N	2020-10-11 23:12:03.498892	2020-10-11 23:38:31.028166
GGAL	Grupo Financiero Galicia S.A.	Grupo Financiero Galicia SA is a financial services holding company. The company through its subsidiaries, provides general banking services, proprietary brand credit card services, personal loans, insurance and other services.	{"Financial Services","Banks - Regional - Latin America",Banks}	{HSBC,BFR,SHG,BBD,BMA,BLX,C}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:03.514762	2020-10-11 23:38:31.023428
GHL	Greenhill & Co. Inc.	Greenhill & Co Inc is an investment bank providing financial advice on mergers, acquisitions, restructurings, financing and capital-raising to corporations, partnerships, institutions and governments.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{LAZ,EVR}	Brokers & Exchanges	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.533147	2020-10-11 23:38:31.02582
GII	SPDR S&P Global Infrastructure	The investment seeks investment results that, before fees and expenses, correspond generally to the total return performance of the S&P Global Infrastructure Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index and in depositary receipts based on securities comprising the index. The index is comprised of 75 of the largest publicly listed infrastructure companies that meet specific investability requirements. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.546175	2020-10-11 23:38:31.025871
GJO	Synthetic Fixed-Income Securities Inc. on behalf of STRATS(SM) Trust for Wal-Mart Stores Inc. Securities Series 2004-5		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:03.55926	2020-10-11 23:38:31.023428
FXZ	First Trust Materials AlphaDEX Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the StrataQuant® Materials Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is a modified equal-dollar weighted index designed by IDI to objectively identify and select stocks from the Russell 1000® Index in the materials and processing sector that may generate positive alpha relative to traditional passive-style indices through the use of the AlphaDEX® selection methodology.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.384717	2020-10-11 23:38:31.02582
GAINM	Gladstone Investment Corporation 6.25% Series D Cumulative Term Preferred Stock	Gladstone Investment Corp is a business development company which is managed externally. Its investment manager is Gladstone Management Corporation. It invests in debt and equity securities of established private businesses operating in the United States.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:03.401032	2020-10-11 23:38:31.021426
GATX	GATX Corporation	GATX Corp is a provider of railcar leasing and maintenance services. The company operates business through four segments offering railcar leasing and maintenance, asset-related financial, and management services.	{Industrials,"Rental & Leasing Services","Consulting & Outsourcing"}	{WLFC,ARII,CAT,GE,R,TRN}	Consulting & Outsourcing	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:03.413574	2020-10-11 23:38:31.021426
GCAP	GAIN Capital Holdings Inc.	GAIN Capital Holdings Inc is engaged in providing trading services and solutions in over-the-counter, or OTC, and exchange-traded markets. It offers financial products, including foreign exchange, or forex, and precious metals.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{ETFC,LTS,INTL,IBKR}	Brokers & Exchanges	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.434827	2020-10-11 23:38:31.031328
GDI	Gardner Denver Holdings Inc.	Gardner Denver Holdings Inc provides flow control and compression equipment and associated aftermarket parts, consumables, and services. It manufactures compressor, pump, vacuum and blower products.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{TTI,NGS,USAC,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:03.45303	2020-10-11 23:38:31.021426
GDXS	ProShares UltraShort Gold Miners	The investment seeks daily investment results, before fees and expenses, that correspond to two times the inverse (-2x) of the daily performance of the NYSE Arca Gold Miners Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index is a modified market-capitalization weighted index primarily composed of publicly traded companies involved in the mining for gold and silver. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.469065	2020-10-11 23:38:31.02423
GEMP	Gemphire Therapeutics Inc.	Gemphire Therapeutics Inc is a clinical-stage biopharmaceutical company developing and commercializing therapies for the treatment of dyslipidemia, a serious medical condition that increases the risk of life threatening cardiovascular disease.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:03.484894	2020-10-11 23:38:31.025871
GFI	Gold Fields Limited American Depositary Shares	Gold Fields Ltd is engaged in underground and surface gold and copper mining and related activities, including exploration, development, extraction, processing and smelting. It operates in South Africa, Australia, Ghana, and Peru.	{"Basic Materials",Gold,"Metals & Mining"}	{ABX,IAG,AEM,RGLD,GG,K,BVN,AU,NEM,SBGL}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:03.505494	2020-10-11 23:38:31.021426
GGZ	Gabelli Global Small and Mid Cap Value Trust (The) of Beneficial Interest	The Gabelli Global Small and Mid Cap Value Trust is a closed-end diversified management investment company whose investment objective is long-term capital growth. It invests in equity securities (such as common stock and preferred stock) of companies.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.524487	2020-10-11 23:38:31.031328
GIB	CGI Group Inc.	CGI Group Inc manages information technology services. The Company's services include the management of IT and business functions, systems integration and consulting as well as the sale of software solutions.	{Technology,"Information Technology Services","Application Software"}	{ECA,BCE,CTSH,MDSO,WIT,INFY,TU,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:03.540215	2020-10-11 23:38:31.021426
GILD	Gilead Sciences Inc.	Gilead Sciences Inc is a biopharmaceutical company that discovers, develops and commercializes new medicines in areas of unmet medical need. Its products target a number of areas, such as HIV, liver diseases, cardiovascular and other diseases.	{Healthcare,Biotechnology}	{AMGN,ALXN,CELG,VRTX,ILMN,BIIB,JNJ,LLY,MRK,ABBV,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:03.551721	2020-10-11 23:38:31.02423
GJP	Synthetic Fixed-Income Securities Inc. on behalf of STRATS (SM) Trust for Dominion Resources Inc. Securities Series 2005-6 Floating Rate Structured Repackaged A		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:03.567779	2020-10-11 23:38:31.031328
GLF	GulfMark Offshore Inc.	GulfMark Offshore Inc provides offshore marine support and transportation services to companies involved in the offshore exploration and production of oil and natural gas.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{}	Oil & Gas - Services	Energy	NYSE American	\N	2020-10-11 23:12:03.585191	2020-10-11 23:38:31.02582
GLP	Global Partners LP representing Limited Partner Interests	Global Partners LP is a midstream logistics and marketing company. It is engaged in the purchasing, selling and logistics of transporting petroleum and related products, including domestic and Canadian crude oil, gasoline and gasoline blendstocks.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{CLNE,APU,INT,SGU,SPH,AE,CAPL,SUN,CEQP}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:03.60551	2020-10-11 23:38:31.02582
GDXX	ProShares Ultra Gold Miners	The investment seeks daily investment results, before fees and expenses, that correspond to two times (2x) the daily performance of the NYSE Arca Gold Miners Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index is a modified market-capitalization weighted index primarily composed of publicly traded companies involved in the mining for gold and silver. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.474467	2020-10-11 23:38:31.025871
GEOS	Geospace Technologies Corporation	Geospace Technologies Corp is engaged in designing and manufacturing of instruments and equipment used in the oil and gas industry for acquiring seismic data to locate, characterize and monitor hydrocarbon producing reservoirs.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{MIND,OSIS,PRCP,SLB,APH,HAL,IO}	Oil & Gas - Services	Energy	Nasdaq Global Select	\N	2020-10-11 23:12:03.490432	2020-10-11 23:38:31.021426
GFNSL	General Finance Corporation Senior Notes due 2021		{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.508565	2020-10-11 23:38:31.02582
GGT	Gabelli Multi-Media Trust Inc. (The)	The primary objective is to achieve long-term capital growth through investment in global multimedia stocks, with income growth as a secondary objective. The Fund seeks opportunities for long-term growth by investing in the global telecommunications, media, publishing and entertainment industries. Also the Fund will invest in companies participating in emerging technological advances in interactive services and products.	{"Financial Services","Asset Management"}	{}			New York Stock Exchange	\N	2020-10-11 23:12:03.525094	2020-10-11 23:38:31.028166
GIG	GigCapital Inc.	GigCapital Inc is a blank check company. The company was formed for the purpose of effecting a merger, capital stock exchange, asset acquisition, stock purchase, reorganization or similar business combination with one or more businesses.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:03.543558	2020-10-11 23:38:31.023428
GJR	Synthetic Fixed-Income Securities Inc. STRATS Trust for Procter&Gamble Securities Series 2006-1		{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.568851	2020-10-11 23:38:31.02423
GLOB	Globant S.A.	Globant SA is a software infrastructure provider. Its main products are digital journey, which used mobile apps and sensors in tracking consumer behavior pattern for its clients.	{Technology,"Information Technology Services","Application Software"}	{NTCT,CSLT,WK,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:03.59514	2020-10-11 23:38:31.021426
GLUU	Glu Mobile Inc.	Glu Mobile Inc develops, publishes and markets games designed to appeal to users of smartphones and tablet devices. Its gaming brands include Blood and Glory, Contract Killer, Deer Hunter, Eternity Warriors, Frontline Commando and Heroes of Destiny.	{Technology,"Electronic Gaming & Multimedia","Application Software"}	{MSFT,TTWO,AMZN,AAPL,EA,ATVI,ZNGA,FB,GOOGL,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:03.618732	2020-10-11 23:38:31.02582
GLW	Corning Incorporated	Corning Inc is the leading manufacturer and supplier of specialty glass and ceramics. Its segments are Display Technologies, Optical Communications, Environmental Technologies, Specialty Materials and Life Sciences.	{Technology,"Electronic Components","Computer Hardware"}	{OCC,CLFD,PLT,PPG,TMO,MMM,OC,TEL,COMM,VIAV,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:03.625309	2020-10-11 23:38:31.028166
GMRE	Global Medical REIT Inc.	Global Medical REIT Inc owns, operates, develops and manages specialty medical properties including medical office buildings, outpatient treatment & diagnostic facilities, physical group practice clinics, ambulatory surgery centers and treatment centers.	{"Real Estate","REIT - Healthcare Facilities",REITs}	{SKT,WSR,PDM,STAG,BRX,SIR}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:03.644981	2020-10-11 23:38:31.028166
GMTA	GATX Corporation 5.625% Senior Notes due 2066		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:03.645761	2020-10-11 23:38:31.02582
GNK	Genco Shipping & Trading Limited New (Marshall Islands)	Genco Shipping & Trading Ltd is an international shipping company. It transports iron ore, coal, grain, steel products and other dry bulk cargoes along shipping routes through the ownership and operation of dry bulk carrier vessels.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{DSX,NM,ALEX,ESEA}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:03.664715	2020-10-11 23:38:31.031328
GNT	GAMCO Natural Resources Gold & Income Trust	GAMCO Natural Resources, Gold & Income Trust is a non-diversified closed-end management investment company. The company's primary investment objective is to provide a high level of current income from interest, dividends, and option premiums.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.678712	2020-10-11 23:38:31.021426
GECCL	Great Elm Capital Corp. 6.50% Notes due 2022		{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.475223	2020-10-11 23:38:31.021426
GEN	Genesis Healthcare Inc.	Genesis Healthcare Inc is a post-acute care provider in the United States. The company operates skilled nursing centers and assisted/senior living communities. It also supplies rehabilitation and respiratory therapy.	{Healthcare,"Long-Term Care Facilities","Health Care Providers"}	{ENSG,NHC,CSU,ACHC,DVCR,FVE,RHE,XLV}	Health Care Providers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:03.485461	2020-10-11 23:38:31.028166
GES	Guess? Inc.	Guess? Inc designs, markets, distributes and licenses a lifestyle collection of contemporary apparel and accessories for men, women and children that reflects the American lifestyle and European fashion sensibilities.	{"Consumer Cyclical","Apparel Stores","Retail - Apparel & Specialty"}	{URBN,SHOO,ROST,ANF,AEO,GPS,JWN,CRI,PVH,BEBE}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:03.498918	2020-10-11 23:38:31.024353
GGG	Graco Inc.	Graco Inc is engaged in the equipment manufacturing business. Its products include systems and equipment used to move, measure, control, dispense and spray fluid and powder materials.	{Industrials,"Diversified Industrials","Industrial Products"}	{GTLS,FLS,DOV,GRC,IEX,ITW,CFX,XYL,ITT}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:03.517788	2020-10-11 23:38:31.024353
GHM	Graham Corporation	Graham Corp designs, manufactures and sells critical equipment for the energy, defense and chemical/petrochemical industries. It also designs and manufactures custom-engineered ejectors, vacuum pumping systems, surface condensers and vacuum systems.	{Industrials,"Metal Fabrication","Industrial Products"}	{CECE,SPXC,NGS}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:03.532808	2020-10-11 23:38:31.028166
GIGM	GigaMedia Limited	GigaMedia Ltd is a publisher of online games.It has online game operations in Taiwan, Hong Kong and Macau and a cloud computing software and services business based in Taipei.	{Technology,"Electronic Gaming & Multimedia","Application Software"}	{CYOU,ZNGA,GRVY,XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:03.543648	2020-10-11 23:38:31.028166
GIS	General Mills Inc.	General Mills Inc is a global manufacturer and marketer of branded consumer foods, such as ready-to-eat breakfast cereals, refrigerated dough and other baking items, snack foods, ice cream, and yogurt.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{HAIN,CAG,K,SJM,THS,POST,MDLZ}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:03.557057	2020-10-11 23:38:31.02582
GKOS	Glaukos Corporation	Glaukos Corp is an ophthalmic medical technology company focused on the development and commercialization of breakthrough products and procedures designed to transform the treatment of glaucoma.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{IART,ICUI,ATRC,BSX,XLV}	Medical Instruments & Equipment	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:03.570659	2020-10-11 23:38:31.025871
GLDD	Great Lakes Dredge & Dock Corporation	Great Lakes Dredge & Dock Corp is a provider of dredging services in the United States. The Company also owns specialty contracting service providers which offers environmental, remediation and geotechnical services throughout the USA.	{Industrials,"Engineering & Construction"}	{MTRX,STRL,MYRG,PRIM,MTZ,DY,GVA,ORN,TPC}	Engineering & Construction	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:03.580618	2020-10-11 23:38:31.031328
GLQ	Clough Global Equity Fund of Beneficial Interest	Clough Global Equity Fund is a closed-end management investment company. Its investment objective is to provide a high level of total return.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:03.606928	2020-10-11 23:38:31.025871
GME	GameStop Corporation	GameStop Corp is an multichannel video game retailer. It sells new and pre-owned video game hardware, physical and digital video game software and accessories through GameStop, EB Games and Micromania stores.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{AAPL,TECD,BBY,T}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:03.629143	2020-10-11 23:38:31.024353
GMOM	Cambria Global Momentum	The investment seeks to preserve and grow capital from investments in the U.S. and foreign equity, fixed income, commodity and currency markets, independent of market direction.\n The fund is considered a "fund of funds" that seeks to achieve its investment objective by primarily investing in other exchange-traded funds and other exchange traded products including, but not limited to, exchange-traded notes, exchange traded currency trusts, closed-end funds, and real estate investment trusts that offer diversified exposure, including inverse exposure, to global regions, countries, styles and sectors.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.645252	2020-10-11 23:38:31.031328
GMO	General Moly Inc.	General Moly Inc is engaged in the business of the exploration, development and mining of properties containing molybdenum in the United States.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{FCX}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:03.642848	2020-10-11 23:38:31.021426
KEY	KeyCorp	KeyCorp is present in the financial service domain. Its core revenue is derived from the provision of retail and commercial banking, investment management and consumer finance services.	{"Financial Services","Banks - Regional - US",Banks}	{WSBC,ZION,NRIM,CMA,CFR,PNC,WFC,STI,JPM,BHLB}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.736256	2020-10-11 23:38:31.031328
GG	Goldcorp Inc.	Goldcorp Inc is gold mining company. It is engaged in the operation, exploration, development, and acquisition of precious metal properties in Canada, the United States, Mexico, and Central and South America.	{"Basic Materials",Gold,"Metals & Mining"}	{ABX,AEM,AUY,NGD,AU,NEM,TAHO,TRQ,WPM}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:03.514402	2020-10-11 23:38:31.025871
GHDX	Genomic Health Inc.	Genomic Health Inc is a healthcare company. It provides genomic-based diagnostic tests that address both the overtreatment and optimal treatment of early stage cancer.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{MYGN,DGX,LH,GEN,QGEN,XLV}	Medical Diagnostics & Research	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:03.531471	2020-10-11 23:38:31.023428
GIL	Gildan Activewear Inc. Class A Sub. Vot.	Gildan Activewear Inc is an apparel manufacturer in North America. Its suite of products includes T-shirts, fleece, socks, and underwear. It has markets in the United States, Canada and other regions of the world.	{"Consumer Cyclical","Apparel Manufacturing","Manufacturing - Apparel & Furniture"}	{EBF,VFC,HBI,DLA,EVK}	Manufacturing - Apparel & Furniture	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:03.551824	2020-10-11 23:38:31.021426
GJT	Synthetic Fixed-Income Securities Inc. Floating Rate Structured Repackaged Asset-Backed Trust Securities Certificates Series 2006-3		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:03.569904	2020-10-11 23:38:31.02582
GLDW	SPDR Long Dollar Gold Trust	The investment seeks to track the performance of the Solactive GLD® Long USD Gold Index (the “index”), less the expenses of the fund’s operations. \n The index is a transparent, rules-based index published by Solactive AG (the “index provider”). The index seeks to track the daily performance of a long position in physical gold (as represented by the Gold Price) and a short position in a basket (the “FX Basket”) of non-U.S. currencies (i.e., a long USD exposure versus the FX Basket). The fund’s holdings generally will consist entirely of Gold Bullion.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.584149	2020-10-11 23:38:31.028166
GLRE	Greenlight Reinsurance Ltd.	Greenlight Capital Re Ltd conducts its reinsurance operations through underwriting traditional property and casualty reinsurance, and risk innovation and strategic partnerships. A majority of its business is sourced through reinsurance brokers.	{"Financial Services","Insurance - Reinsurance","Insurance - Specialty"}	{MHLD,RNR,Y,RE,TPRE}	Insurance - Specialty	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:03.607473	2020-10-11 23:38:31.023007
GLYC	GlycoMimetics Inc.	GlycoMimetics Inc is a clinical stage biotechnology company. It is engaged in discovery and development of novel glycomimetic drugs to address unmet medical needs resulting from diseases in which carbohydrate biology plays a key role.	{Healthcare,Biotechnology}	{JNJ,PFE,SNY,PTN,BLUE,EIGR,SVRA,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:03.624894	2020-10-11 23:38:31.023007
GMLPP	Golar LNG Partners LP 8.75% Series A Cumulative Redeemable Preferred Units	Golar LNG Partners LP owns and operates floating storage regasification units (FSRUs) and liquefied natural gas (LNG) carriers under long-term time charters. It carries its operation in only one segment namely LNG market segment.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:03.640863	2020-10-11 23:38:31.023007
GNCA	Genocea Biosciences Inc.	Genocea Biosciences Inc is a biopharmaceutical company that discovers and develops novel vaccines and immunotherapies to address diseases with unmet needs. The Company's products include GEN-003 and GEN-004.	{Healthcare,Biotechnology}	{NVAX,PFE,MRK,AGEN,NLNK,VICL,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:03.658559	2020-10-11 23:38:31.028166
GNUS	Genius Brands International Inc.	Genius Brands International Inc is a media company that creates and licenses animated multimedia entertainment content. The company distributes its content in all formats as well as a broad range of consumer products based on its characters.	{"Consumer Cyclical","Media - Diversified",Entertainment}	{BTN,VIAB,FOXA}	Entertainment	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:03.689387	2020-10-11 23:38:31.031328
GOL	Gol Linhas Aereas Inteligentes S.A. Sponsored ADR representing 2 Pfd Shares	Gol Intelligent Airlines Inc is a low-cost carrier that provides regular and non-regular air transportation services of passengers, cargo and mailbags, domestically or internationally.	{Industrials,Airlines}	{CEA,CPA,DAL,LTM,VLRS,AVH}	Airlines	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:03.70131	2020-10-11 23:38:31.031328
GOOD	Gladstone Commercial Corporation	Gladstone Commercial Corp is a real estate investment trust. The company is engaged in investing in and owning net leased industrial, commercial and retail real property and making long-term industrial and commercial mortgage loans.	{"Real Estate","REIT - Diversified",REITs}	{OFC,NHI,OLP}	REITs	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:03.71161	2020-10-11 23:38:31.02582
GORO	Gold Resource Corporation	Gold Resource Corp is a US-based company that produces metal concentrates and dore. The metal concentrates contain gold, silver, copper, lead, and zinc, and the dore contains only gold and silver. The company operates in Oaxaca (Mexico) and Nevada (USA).	{"Basic Materials",Gold,"Metals & Mining"}	{RGLD,XPL,AUMN,HL,SCCO,TAHO}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:03.717743	2020-10-11 23:38:31.02423
GGN	GAMCO Global Gold Natural Resources & Income Trust	GAMCO Global Gold, Natural Resources & Income Trust is a non-diversified closed-end investment company. Its investment objective is to provide a current income and capital appreciation through investing in gold and natural resources industries.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:03.522838	2020-10-11 23:38:31.02582
GILT	Gilat Satellite Networks Ltd.	Gilat Satellite Networks Ltd provides broadband satellite communication and networking solutions and services.	{Technology,"Communication Equipment"}	{VSAT,CMTL,CRNT,LORL,SATS,GD,HRS,GSAT,LLL,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:03.553468	2020-10-11 23:38:31.028166
GLADN	Gladstone Capital Corporation 6.00% Series 2024 Term Preferred Stock	Gladstone Capital Corp is an externally managed, closed-end, non-diversified management investment company.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:03.576729	2020-10-11 23:38:31.028166
GLOG	GasLog Ltd.	GasLog Ltd is primarily engaged in the ownership, operation and management of vessels in the LNG market, providing maritime services for the transportation of LNG on a worldwide basis and LNG vessel management services.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{GLNG,GASS,TNP,TK,TGP,STNG,GMLP,GLOP,DLNG}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:03.595132	2020-10-11 23:38:31.028166
GLT	Glatfelter	P.H. Glatfelter Co is a manufacturer of specialty papers and fiber-based engineered materials. It operates with three segments namely Composite Fibers, Advanced Airlaid Materials and Specialty Papers.	{"Basic Materials","Paper & Paper Products","Forest Products"}	{UFS,SWM,IP,PKG,NP,CLW}	Forest Products	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:03.613639	2020-10-11 23:38:31.024353
GMFL	Invesco Multi-Factor Large Cap	The investment seeks to replicate as closely as possible, before fees and expenses, the daily performance of the Guggenheim Multi-Factor Large Cap Index Total Return.\n The fund invests primarily in equity securities to meet its investment objective of replicating the underlying index as closely as possible, before fees and expenses. The underlying index component securities are selected using a Multi-Factor Composite Score, a rules-based methodology, to seek to identify those securities that will enhance returns, reduce risk and improve diversification. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.634675	2020-10-11 23:38:31.02423
GNC	GNC Holdings Inc. Class A	GNC Holdings Inc is a health and wellness products retailer. Its product offering includes health and wellness products such as vitamins, herbal supplements, sports nutrition products, and diet products.	{"Consumer Defensive","Pharmaceutical Retailers","Retail - Defensive"}	{RAD,HLF,VSI,WBA,NHTC}	Retail - Defensive	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:03.658122	2020-10-11 23:38:31.023007
GNMX	Aevi Genomic Medicine Inc.	Aevi Genomic Medicine Inc is a clinical-stage biopharmaceutical company. It is engaged in translating genetic discoveries into novel therapies to improve the lives of children and adults with pediatric onset life altering diseases.	{Healthcare,Biotechnology}	{FORR,NKTR,JNJ,NRC,OMEX,XON,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:03.672484	2020-10-11 23:38:31.023428
GNRX	Market Vectors Generic Drugs ETF	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the Indxx Global Generics & New Pharma Index.\n The index includes exchange-listed companies, on a global basis, that derive a significant proportion of their revenues (or that have the potential to derive a significant proportion of their revenues) from the generic drug industry, or that have a primary business focus on the generic drug industry. The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.676653	2020-10-11 23:38:31.024353
GNTY	Guaranty Bancshares Inc.	Guaranty Bancshares Inc is a bank holding company providing commercial and consumer banking products and services, as well as mortgage, trust, and wealth management products and services.	{"Financial Services","Banks - Regional - US",Banks}	{MPB,MOFG}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:03.685924	2020-10-11 23:38:31.028166
GOLD	Randgold Resources Limited	Randgold Resources Ltd is engaged in the exploration and development of gold deposits in Sub-Saharan Africa. It operates 5 gold mines in Africa - Loulo, Gounkoto and Morila in Mali, Tongon in Côte d'Ivoire and Kibali in the DRC.	{"Basic Materials",Gold,"Metals & Mining"}	{AEM,GFI,AU}	Metals & Mining	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:12:03.70469	2020-10-11 23:38:31.02423
GOLF	Acushnet Holdings Corp.	Acushnet Holdings Corp is engaged in the design, development, manufacture and distribution of performance-driven golf products.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{NLS}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:03.705504	2020-10-11 23:38:31.025871
GOVT	iShares U.S. Treasury Bond	The investment seeks to track the investment results of the ICE U.S. Treasury Core Bond Index (the "underlying index").\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index, but which the advisor believes will help the fund track the index. The underlying index measures the performance of public obligations of the U.S. Treasury that have a remaining maturity greater than one year and less than or equal to thirty years.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:03.726374	2020-10-11 23:38:31.025871
GIFI	Gulf Island Fabrication Inc.	Gulf Island Fabrication Inc is a fabricator of offshore drilling. The company operates in segments namely Fabrication division, Shipyards division, Services division, and corporate division.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{ROCK,VMI,CVX}	Oil & Gas - Services	Energy	Nasdaq Global Select	\N	2020-10-11 23:12:03.540116	2020-10-11 23:38:31.02423
GIM	Templeton Global Income Fund Inc.	Templeton Global Income Fund is a closed-end fund. It seeks high, current income, with a secondary goal of capital appreciation. Under normal market conditions.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.556172	2020-10-11 23:38:31.023007
GLOP	GasLog Partners LP representing limited partnership interests	GasLog Partners LP owns, operates & acquires liquefied natural gas (LNG) carriers engaged in LNG transportation under long-term charters. Its fleet consists of approximately five LNG carriers, including three vessels with modern tri-fuel diesel-electric.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{GLNG,TGP,BPL,GLOG,PBA,DLNG}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:03.599597	2020-10-11 23:38:31.031328
GLV	Clough Global Dividend and Income Fund of beneficial interest	Clough Global Dividend and Income Fund is a closed-end management investment company .Its investment objective is to provide a high level of total return.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:03.620318	2020-10-11 23:38:31.021426
GMLP	Golar LNG Partners LP	Golar LNG Partners LP owns and operates floating storage regasification units (FSRUs) and liquefied natural gas (LNG) carriers under long-term time charters. It carries its operation in only one segment namely LNG market segment.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{GLNG,TNP,TK,SFL,TGP,TOO,TNK,GLOG,DHT,FRO}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:03.640373	2020-10-11 23:38:31.025871
GNE	Genie Energy Ltd. Class B Stock	Genie Energy Ltd through its subsidiaries resells electricity and natural gas to residential and small business customers operating in the Mid-Atlantic and Midwestern United States. It also provides brokerage and advisory services to commercial customers.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{NI,FE,ED,NFG,UGI,EXC}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:03.661619	2020-10-11 23:38:31.028166
GNR	SPDR S&P Global Natural Resources	The investment seeks investment results that, before fees and expenses, correspond generally to the total return performance of the S&P Global Natural Resources Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index and in depositary receipts based on securities comprising the index. The index is comprised of 90 of the largest U.S. and foreign publicly traded companies, based on market capitalization, in natural resources and commodities businesses that meet certain investability requirements. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.673881	2020-10-11 23:38:31.023007
GOF	Guggenheim Strategic Opportunities Fund of Beneficial Interest	Claymore/guggenheim Stra Oppor FD is a diversified, closed-end management investment company. The company refers to the global asset management and investment advisory division of Guggenheim Partners.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.697229	2020-10-11 23:38:31.028166
GOODO	Gladstone Commercial Corporation 7.50% Series B Cumulative Redeemable Preferred Stock	Gladstone Commercial Corp is a real estate investment trust. The company is engaged in investing in and owning net leased industrial, commercial and retail real property and making long-term industrial and commercial mortgage loans.	{"Real Estate","REIT - Diversified",REITs}	{}	REITs	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:03.713238	2020-10-11 23:38:31.028166
GOGO	Gogo Inc.	Gogo Inc is a US-based in-flight connectivity systems and service provider. The company through aero communications, in-flight broadband, and wireless in-cabin digital entertainment solutions for the aviation industry across the world.	{"Communication Services","Telecom Services"}	{ORBC,IRDM,COL,ENT,I,GSAT}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:12:03.700004	2020-10-11 23:38:31.021426
GOOS	Canada Goose Holdings Inc. Subordinate	Canada Goose Holdings Inc is a designer, manufacturer, distributor and retailer of premium outerwear for men, women and children. Its products are sold through select outdoor, luxury and online retailers and distributors.	{"Consumer Cyclical","Apparel Manufacturing","Manufacturing - Apparel & Furniture"}	{}	Manufacturing - Apparel & Furniture	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:03.719119	2020-10-11 23:38:31.023428
GPAQU	Gordon Pointe Acquisition Corp. Unit	Gordon Pointe Acquisition Corp is a blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:03.729406	2020-10-11 23:38:31.028166
GPM	Guggenheim Enhanced Equity Income Fund	Guggenheim Enhanced Equity Income Fund is a closed end management investment company. Its objective is high current income and its secondary objective is capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.736387	2020-10-11 23:38:31.028166
GJH	Synthetic Fixed-Income Securities Inc 6.375% (STRATS) Cl A-1		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:03.55681	2020-10-11 23:38:31.028166
GLAD	Gladstone Capital Corporation	Gladstone Capital Corp is an externally managed, closed-end, non-diversified management investment company.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:03.570792	2020-10-11 23:38:31.021426
GLL	ProShares UltraShort Gold	The investment seeks to match two times the inverse (-2x) of the daily performance of the gold bullion as measured by the U.S. dollar p.m. LBMA Gold Price.\nThe fund seeks to meet its investment objective by investing under normal market conditions in any one of, or combinations of, Financial Instruments (including swap agreements, futures contracts and forward contracts) based on the fund’s benchmark. It may invest up to 100% of its assets in any of these types of cash or cash equivalent securities.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.589272	2020-10-11 23:38:31.024353
GLOW	Glowpoint Inc.	Glowpoint Inc is a provider of video collaboration services and network aalications. Its services enable customers to use video conferencing as an efficient and effective method of communication for their business meetings.	{"Communication Services","Telecom Services"}	{CTL,VZ,T,HCOM,GTT}	Communication Services	Communication Services	NYSE American	\N	2020-10-11 23:12:03.599621	2020-10-11 23:38:31.02423
GLU	Gabelli Global Utility of Beneficial Ownership	Gabelli Global Utility & Income Trust is a non-diversified, closed-end management investment company. The Fund's objective is to seek a consistent level of after-tax total return.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:03.615947	2020-10-11 23:38:31.02423
GMF	SPDR S&P Emerging Asia Pacific	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the S&P Asia Pacific Emerging BMI Index based upon the emerging markets of the Asia Pacific region.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index and in depositary receipts (including ADRs or GDRs) based on securities comprising the index. The index is a market capitalization weighted index designed to define and measure the investable universe of publicly traded companies domiciled in emerging Asian Pacific markets. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.63322	2020-10-11 23:38:31.023428
GMZ	Goldman Sachs MLP Income Opportunities Fund	Goldman Sachs MLP Income Opportunities Fund is a non-diversified, closed-end management investment company. Its investment objective is to seek high level of total return with an emphasis on current distributions to shareholders.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.65583	2020-10-11 23:38:31.023428
GNMK	GenMark Diagnostics Inc.	GenMark Diagnostics Inc is a molecular diagnostics company. The Company develops and commercializes its proprietary eSensor detection technology for the support of instruments and molecular tests.	{Healthcare,"Medical Devices"}	{HOLX,LMNX,PKI,QGEN,XLV}	Medical Devices	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:03.673461	2020-10-11 23:38:31.025871
GNW	Genworth Financial Inc	Genworth Financial Inc is a insurance holding company which operates in mortgage insurance and long-term care insurance. Its products include long-term care insurance, life insurance and fixed annuities businesses and others.	{"Financial Services","Insurance - Diversified",Insurance}	{AAME,IHC,UNM,LNC,MET,PRU,CNO}	Insurance	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.691496	2020-10-11 23:38:31.025871
GOGL	Golden Ocean Group Limited	Golden Ocean Group Ltd is a dry bulk shipping company based in Bermuda, mainly operating in the Capesize and Panamax market segments.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{GLNG,NAT,SFL,DSX,NMM,SALT,SHIP,FRO,SBLK}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:03.697636	2020-10-11 23:38:31.023007
GOOG	Alphabet Inc.	Alphabet Inc is a provider of internet content products and portals. Its suite of brands includes Search, Android, YouTube, Apps, Maps & Ads.	{Technology,"Internet Content & Information","Online Media"}	{MSFT,AKAM,ADBE,AAPL,EBAY,BIDU,IAC,YNDX,FB,BCOR,XLK}	Online Media	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:03.714896	2020-10-11 23:38:31.028166
GOODP	Gladstone Commercial Corporation 7.75% Series A Cumulative Redeemable Preferred Stock	Gladstone Commercial Corp is a real estate investment trust. The company is engaged in investing in and owning net leased industrial, commercial and retail real property and making long-term industrial and commercial mortgage loans.	{"Real Estate","REIT - Diversified",REITs}	{}	REITs	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:03.71359	2020-10-11 23:38:31.023007
GPI	Group 1 Automotive Inc.	Group 1 Automotive Inc sells new and used cars and light trucks. Its activities include sale service and insurance contracts; provides automotive maintenance and repair services; and sell vehicle parts.	{"Consumer Cyclical","Auto & Truck Dealerships",Autos}	{CPRT,CRMT,RUSHA,LAD,AN,SAH,ABG,KMX,PAG}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:03.731453	2020-10-11 23:38:31.021426
GPK	Graphic Packaging Holding Company	Graphic Packaging Holding Co is a holding company engaged in providing paper-based packaging solutions to food, beverage and other consumer products companies. It is a producer of folding cartons in Europe, Americas and Asia Pacific.	{"Consumer Cyclical","Packaging & Containers"}	{PKG,WRK}	Packaging & Containers	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:03.732695	2020-10-11 23:38:31.02423
GLDI	Credit Suisse X-Links Silver Call ETN IOPV	The investment seeks to replicate, net of expenses, the Credit Suisse NASDAQ Gold FLOWSTM 103 Index.\n The index measures the return of a “covered call” strategy on the shares of the SPDR® Gold Trust (the “GLD Shares”) by reflecting changes in the price of the GLD Shares and the notional option premiums received from the sale of monthly call options on the GLD Shares less notional trading costs incurred in connection with the covered call strategy.	{"Communication Services","Telecom Services"}	{}	Communication Services	Communication Services	NASDAQ Global Market	\N	2020-10-11 23:12:03.581824	2020-10-11 23:38:31.023428
GLPG	Galapagos NV	Galapagos NV is a clinical-stage biotechnology company. It is engaged in the discovery and development of small molecule medicines which address unmet medical conditions.	{Healthcare,Biotechnology}	{AZN,GSK,ABT,JNJ,SNY,MRK,TEVA,ABBV,FBIO,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:03.602807	2020-10-11 23:38:31.023428
GMED	Globus Medical Inc. Class A	Globus Medical Inc is a medical device company. It operates under two divisions that include innovative fusion and disruptive technology developing products for patients with musculoskeletal disorders for various treatment.	{Healthcare,"Medical Devices"}	{NUVA,RTIX,SYK,ZBH,ATEC,XLV}	Medical Devices	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:03.632099	2020-10-11 23:38:31.031328
GNBC	Green Bancorp Inc.	Green Bancorp Inc is a bank holding company. The company through its wholly owned subsidiaries provides commercial and private banking services mainly to Texas based customers.	{"Financial Services","Banks - Regional - US",Banks}	{WTBA,CWBC,FFWM}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:03.657289	2020-10-11 23:38:31.024353
GNRC	Generac Holdlings Inc.	Generac Holdings Inc is a designer and manufacturer of power generation equipment and other engine powered products. The company primarily serves the residential, light commercial, industrial and construction markets.	{Industrials,"Diversified Industrials","Industrial Products"}	{WWD,AMOT,BGG,CAT,CMI,RBC,SVT,BWXT,CPST}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:03.674447	2020-10-11 23:38:31.028166
GOEX	Global X Gold Explorers	The investment seeks to provide investment results that correspond generally to the price and yield performance, before fees and expenses, of the Solactive Global Gold Explorers & Developers Total Return Index.\n The fund invests at least 80% of its total assets in the securities of the index and in American Depositary Receipts and Global Depositary Receipts based on the securities in the underlying index. The underlying index is a free float-adjusted, liquidity-tested and market capitalization-weighted index that is designed to measure broad-based equity market performance of global companies involved in gold exploration. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.697211	2020-10-11 23:38:31.023428
GOOGL	Alphabet Inc.	Alphabet Inc is a provider of internet content products and portals. Its suite of brands includes Search, Android, YouTube, Apps, Maps & Ads.	{Technology,"Internet Content & Information","Online Media"}	{MSFT,AKAM,ADBE,AAPL,EBAY,BIDU,IAC,YNDX,FB,BCOR,XLK}	Online Media	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:03.716586	2020-10-11 23:38:31.031328
GPC	Genuine Parts Company	Genuine Parts Co sells automotive parts, industrial and electrical components, and office products. The company sells bearings, power transmission, hydraulic and other products to its commercial and retail customers.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{LKQ,DORM,PRTS,SNA,SMP,AAP,ORLY,ESND}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:03.732008	2020-10-11 23:38:31.02582
GPN	Global Payments Inc.	Global Payments Inc has interests in offering business solutions. Its primary activity includes payment solutions for merchants, value-added resellers, enterprise software providers, financial institutions and government agencies.	{Industrials,"Business Services"}	{INTU,EEFT,FISV,EBAY,TSS,FIS,MA,WU}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:03.745768	2020-10-11 23:38:31.025871
GPRE	Green Plains Inc.	Green Plains Inc is an integrated ethanol producer. The company's business segments are Ethanol Production, Corn Oil Production, Agribusiness and Marketing and Distribution. Geographically, it generates its revenue from the United States.	{"Basic Materials","Specialty Chemicals",Chemicals}	{REX,VLO,ADM,FF,PEIX,GRBK}	Chemicals	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:12:03.757109	2020-10-11 23:38:31.028166
GPS	Gap Inc. (The)	Gap Inc is a global apparel and accessories retailer. The company offers products for men, women, and children under various brands such as Gap, Banana Republic, Old Navy, Athleta, and Intermix.	{"Consumer Cyclical","Apparel Stores","Retail - Apparel & Specialty"}	{URBN,GES,ANF,AEO,BKE,EXPR,TLYS}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:03.757648	2020-10-11 23:38:31.028166
GJS	Goldman Sachs Group Securities STRATS Trust for Series 2006-2		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:03.569177	2020-10-11 23:38:31.024353
GLBZ	Glen Burnie Bancorp	Glen Burnie Bancorp is a bank holding company. The Company through its subsidiary provides commercial & retail banking including acceptance of demand & time deposits and the origination of loans to individuals, associations, partnerships & corporations.	{"Financial Services","Banks - Regional - US",Banks}	{SASR,ORRF,SHBI,EGBN,FUNC,OLBK,MTB,HBMD}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:03.578845	2020-10-11 23:38:31.023007
GLNG	Golar LNG Limited	Golar LNG Ltd is a midstream liquid natural gas company. It is engaged in the transportation, regasification and liquefaction and trading of liquid natural gas. The company's reportable segment are Vessal operations, FLNG, and Power.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{NAT,TK,SFL,TGP,TOO,TNK,STNG,GMLP,GLOG,GOGL}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:03.591892	2020-10-11 23:38:31.025871
GLPI	Gaming and Leisure Properties Inc.	Gaming and Leisure Properties Inc is a self-managed Pennsylvania REIT. It is engaged in acquiring, financing, and owning real estate property to be leased to gaming operators in triple-net lease arrangements.	{"Real Estate","REIT - Diversified",REITs}	{LHO,HST,DRH,PEB,CHSP,INN,HPT}	REITs	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:03.603858	2020-10-11 23:38:31.021426
GM	General Motors Company	General Motors Company is an American multinational corporation. It designs, builds and sells cars, trucks and automobile parts. The company also provides automotive financing services through General Motors Financial Company, Inc.	{"Consumer Cyclical","Auto Manufacturers",Autos}	{GNTX,TSLA,HMC,TM,F,TTM}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:03.625604	2020-10-11 23:38:31.028166
GMS	GMS Inc.	GMS Inc is a North American-based supplier of building materials. The company principally operates through three product lines: Wallboards, Ceilings, and Steel Framings. The majority of the company's revenue is generated by the Wallboard segment.	{"Basic Materials","Building Materials"}	{HDS,HBP,IBP,GMS}	Building Materials	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:03.645186	2020-10-11 23:38:31.028166
GNL	Global Net Lease Inc.	Global Net Lease Inc is a real estate investment trust that manages a diversified portfolio of commercial real estate properties. It is engaged in the ownership, management, operating, lease, acquisition, investment in, and sale of the portfolio assets.	{"Real Estate","REIT - Diversified",REITs}	{ANH,DEI,IRET}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:03.666708	2020-10-11 23:38:31.02582
GOAU	US Global GO Gold and Precious Metal Miners	The investment seeks to track the performance, before fees and expenses, of the U.S. Global Go Gold and Precious Metal Miners Index.\n Under normal circumstances, at least 80 percent of the fund's total assets (exclusive of any collateral held from securities lending) will be invested in the component securities of the index and depositary receipts representing component securities. The index is composed of the exchange-listed common stock (or American Depositary Receipts ("ADRs")) of U.S. and international (including emerging markets) companies that earn at least 50 percent of their aggregate revenue from precious metals. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.694856	2020-10-11 23:38:31.02423
GOODM	Gladstone Commercial Corporation Series D Cumulative Redeemable Preferred Stock	Gladstone Commercial Corp is a real estate investment trust. The company is engaged in investing in and owning net leased industrial, commercial and retail real property and making long-term industrial and commercial mortgage loans.	{"Real Estate","REIT - Diversified",REITs}	{}	REITs	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:03.713898	2020-10-11 23:38:31.024353
GPJA	Georgia Power Company Series 2017A 5.00% Junior Subordinated Notes due October 1 2077		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:03.732087	2020-10-11 23:38:31.023007
GPL	Great Panther Silver Limited (Canada)	Great Panther Silver Ltd is engaged in the acquisition, exploration, and development of mineral properties within the Americas and Mexico. Its mineral properties include the El Horcon, Santa Rosa, Plomo projects, and Argosy project.	{"Basic Materials",Silver,"Metals & Mining"}	{PAAS,MAG,EXK,SVM,AXU,HL,AG}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:03.738564	2020-10-11 23:38:31.031328
GPOR	Gulfport Energy Corporation	Gulfport Energy Corp is an independent oil and natural gas exploration and production company. It is engaged in the exploration, exploitation, acquisition and production of natural gas, natural gas liquids and crude oil in the United States.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{AXAS,AREX,NFX,XOM,DVN,EOG,XEC,HES,FANG,WLL}	Oil & Gas - E&P	Energy	Nasdaq Global Select	\N	2020-10-11 23:12:03.752949	2020-10-11 23:38:31.02582
FRC	FIRST REPUBLIC BANK	First Republic Bank is engaged in providing personalized, relationship-based services, including private banking, private business banking, real estate lending and wealth management services, including trust and custody services.	{"Financial Services","Banks - Regional - US",Banks}	{UMPQ,CATY,CVBF,EWBC,SIVB,BMRC,BAC}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.225945	2020-10-11 23:38:31.028166
FRT	Federal Realty Investment Trust	Federal Realty Investment Trust is an equity real estate investment trust specializing in the ownership, management, and redevelopment of retail and mixed-use properties located primarily in densely populated and affluent communities.	{"Real Estate","REIT - Retail",REITs}	{MAC,BDN,PMT}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:03.245975	2020-10-11 23:38:31.021426
FSI	Flexible Solutions International Inc.	Flexible Solutions International Inc together with its subsidiaries develops, manufactures and markets specialty chemicals which slow the evaporation of water.	{"Basic Materials","Specialty Chemicals",Chemicals}	{NTIC,IOSP,ALB,GRA}	Chemicals	Basic Materials	NYSE American	\N	2020-10-11 23:12:03.262792	2020-10-11 23:38:31.023007
FSV	FirstService Corporation	FirstService Corp operates in the property services sector. It serves through service platforms including FirstService Residential which manages residential communities and FirstService Brands which provides property services.	{"Real Estate","Real Estate Services"}	{CBRE,ALEX,TPHS,FVE}	Real Estate Services	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:03.273854	2020-10-11 23:38:31.024353
FTEK	Fuel Tech Inc.	Fuel Tech Inc provides boiler optimization, efficiency improvement, air pollution reduction and similar services to utility and industrial customers worldwide. It also provides nitrogen oxide reduction techniques, and helps improve the combustion units.	{Industrials,"Pollution & Treatment Controls","Industrial Products"}	{CECE,ENTG,DCI}	Industrial Products	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:03.286663	2020-10-11 23:38:31.023428
FTS	Fortis Inc.	Fortis Inc is an electric and gas utility holding company. The company's regulated utilities serves customers across Canada and in the United States and the Caribbean.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{HNP,HE,FTS,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:03.304537	2020-10-11 23:38:31.031328
FUSB	First US Bancshares Inc.	First US Bancshares Inc, formerly United Security Bancshares Inc operates as the holding company for First United Security Bank, which provides commercial banking business and banking services in Alabama and Mississippi.	{"Financial Services","Banks - Regional - US",Banks}	{AUBN,CTBI,FBMS,RNST,SFBS,SYBT,BXS,PFBX}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:03.332416	2020-10-11 23:38:31.02423
FWDI	Madrona International	The investment seeks to provide long-term capital appreciation above the capital appreciation of its international benchmarks, such as the MSCI EAFE Index, the fund's primary benchmark, and the BNY Mellon Classic ADR Index, the fund's secondary benchmark.\n The Sub-Advisor seeks to achieve the fund's investment objective by selecting a portfolio primarily composed of American Depository Receipts ("ADRs") from among the largest issuers of Europe, Australasia and the Far East ("EAFE") and Canada. The fund's portfolio may also include ADRs that provide exposure to the largest issuers in markets deemed to be emerging markets.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.349801	2020-10-11 23:38:31.021426
FXE	Invesco CurrencyShares Euro Currency Trust	The investment objective of the Trust is for the Shares to reflect the price in USD of the Euro. The Shares are intended to provide institutional and retail investors with a simple, cost-effective means of gaining investment benefits similar to those of holding euro.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.362774	2020-10-11 23:38:31.021426
FXU	First Trust Utilities AlphaDEX Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the StrataQuant® Utilities Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is a modified equal-dollar weighted index designed by IDI to objectively identify and select stocks from the Russell 1000® Index in the utilities sector that may generate positive alpha relative to traditional passive-style indices through the use of the AlphaDEX® selection methodology.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.376457	2020-10-11 23:38:31.023428
GARD	Realty Shares DIVCON Dividend Guard	The investment seeks long-term capital appreciation by tracking the performance, before fees and expenses, of the Reality Shares DIVCON Leaders Dividend Index.\n Under normal circumstances, at least 80% of the fund's assets will be invested in component securities of the benchmark index. The benchmark index is designed to capitalize on the theory that, over time, companies that consistently grow their dividends tend to have investment returns above overall market returns, and companies that do not grow (or cut) their dividends tend to have investment returns below overall market returns. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:03.40518	2020-10-11 23:38:31.023007
GBDC	Golub Capital BDC Inc.	Golub Capital BDC Inc is a closed-end, non-diversified management investment company. Its investment objective is to generate current income and capital appreciation by investing in senior secured and one stop loans in U.S. middle-market companies.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:03.4192	2020-10-11 23:38:31.031328
GRID	First Trust NASDAQ Clean Edge Smart Grid Infrastru	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the NASDAQ OMX® Clean Edge® Smart Grid Infrastructure IndexSM.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks or in depositary receipts representing securities in the index. The index includes companies that are primarily engaged and involved in electric grid, electric meters and devices, networks, energy storage and management, and enabling software used by the smart grid infrastructure sector. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.781871	2020-10-11 23:38:31.024353
GROW	U.S. Global Investors Inc.	US Global Investors Inc with and its subsidiaries provide investment advisory services and administrative services. The Company offers its services to U.S. Global Investors Funds.	{"Financial Services","Asset Management"}	{TROW,VRTS,WDR,FII,EV,BEN,LM,MN,APAM}	Asset Management	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:03.796304	2020-10-11 23:38:31.023428
GSAT	Globalstar Inc.	Globalstar Inc is a telecommunications company based in the United States. It provides communications services such as two-way voice and data transmission.	{"Communication Services","Telecom Services"}	{CMTL,LORL,ORBC,SATS,DISH,IRDM,ENT,I}	Communication Services	Communication Services	NYSE American	\N	2020-10-11 23:12:03.810705	2020-10-11 23:38:31.021426
GSG	iShares GSCI Commodity-Indexed Trust Fund	The investment seeks to track the results of a fully collateralized investment in futures contracts on the S&P GSCI™ Total Return Index composed of a diversified group of commodities futures.\n The Trust holds long positions in index futures whose settlement value at expiration is determined based on the value of the S&P GSCI-ER at that time. It also earns interest on its non-cash Collateral Assets. The index reflects the return of the S&P GSCI-ER, together with the return on specified U.S. Treasury securities that are deemed to have been held to collateralize a hypothetical long position in the futures contracts comprising the S&P GSCI™.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.829055	2020-10-11 23:38:31.024353
GSLC	Goldman Sachs ActiveBeta U.S. Large Cap Equity	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the Goldman Sachs ActiveBeta® U.S. Large Cap Equity Index.\n The fund seeks to achieve its investment objective by investing at least 80% of its assets (exclusive of collateral held from securities lending) in securities included in its underlying index, in depositary receipts representing securities included in its underlying index and in underlying stocks in respect of depositary receipts included in its underlying index. The index is designed to deliver exposure to equity securities of large capitalization U.S. issuers.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.840211	2020-10-11 23:38:31.031328
GTLS	Chart Industries Inc.	Chart Industries Inc is a manufacturer of engineered equipment for the industrial gas, energy and biomedical industries. Its segments include Energy and Chemicals, Distribution and Storage, and Biomedical.	{Industrials,"Metal Fabrication","Industrial Products"}	{AME,BMI,GE,GGG,IEX,CFX,INGN,KOOL}	Industrial Products	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:03.874941	2020-10-11 23:38:31.023428
GUNR	FlexShares Global Upstream Natural Resources Index Fund	The investment seeks investment results that correspond generally to the price and yield performance, before fees and expenses, of the Morningstar® Global Upstream Natural Resources IndexSM.\n The fund will invest at least 80% of its total assets (exclusive of collateral held from securities lending) in the securities of the index and in ADRs and GDRs based on the securities in the index. The index reflects the performance of a selection of equity securities that are traded in or are issued by companies domiciled in global developed or emerging markets, as determined by the index provider pursuant to its index methodology. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.898465	2020-10-11 23:38:31.028166
GVI	iShares Intermediate Government/Credit Bond	The investment seeks to track the investment results of the Bloomberg Barclays U.S. Intermediate Government/Credit Bond Index.\n The fund generally invests at least 90% of its assets in securities of the underlying index. The index measures the performance of U.S. dollar-denominated U.S. Treasury bonds, government-related bonds (i.e., U.S. and non-U.S. agencies, sovereign, supranational and local authority debt) and investment-grade U.S. corporate bonds that have a remaining maturity of greater than one year and less than or equal to ten years.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:03.914217	2020-10-11 23:38:31.02582
GWW	W.W. Grainger Inc.	W.W. Grainger Inc is a broad line, a business-to-business distributor of maintenance, repair and operating (MRO) supplies and other related products and services. Its services consist of contact centers, catalogs, inventory management solution and others.	{Industrials,"Industrial Distribution"}	{FAST,LAWS,HWCC,MSM,AXE,WCC,HSC,SNA,HDS}	Industrial Distribution	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:03.928906	2020-10-11 23:38:31.024353
HACK	ETFMG Prime Cyber Security	The investment seeks investment results that, before fees and expenses, correspond generally to the price and yield performance of the Prime Cyber Defense Index.\n The fund invests at least 80% of its total assets in the component securities of the index and in ADRs and GDRs based on the component securities in the index. The index tracks the performance of the exchange-listed equity securities of companies across the globe that (i) engage in providing cyber security applications or services as a vital component of its overall business or (ii) provide hardware or software for cyber security activities as a vital component of its overall business.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.942819	2020-10-11 23:38:31.028166
GRMY	Xtrackers Germany Equity	The investment seeks investment results that correspond generally to the performance, before fees and expenses, of the NASDAQ Germany Large Mid Cap Index (the "underlying index").\n The fund will invest at least 80% of its total assets (but typically far more) in component securities (including depositary receipts in respect of such securities) of the underlying index. The underlying index is designed to track the performance of the German large- and mid- capitalization equity market. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:03.791857	2020-10-11 23:38:31.02582
GRVY	GRAVITY Co. Ltd.	GRAVITY Co Ltd is a developer and publisher of online games in Taiwan, Korea, the Philippines and Thailand based on the number of peak concurrent users as compiled from various statistical data available from public sources.	{Technology,"Electronic Gaming & Multimedia","Application Software"}	{MSFT,EA,ATVI,ZNGA,XLK}	Application Software	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:03.804909	2020-10-11 23:38:31.028166
GSH	Guangshen Railway Company Limited	Guangshen Railway Co Ltd provides railroad passenger, transportation, freight transportation, railway network usage and other transportation-related services.	{Industrials,Railroads,"Transportation & Logistics"}	{CNI,GWR,UNP,KSU}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:03.831659	2020-10-11 23:38:31.025871
GSM	Ferroglobe PLC	Ferroglobe PLC provides silicon and specialty metals. It produces silicon metal and silicon and manganese based alloy, serving customers in the specialty chemical, aluminum, solar, steel and ductile iron foundry industries.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{HAYN,GLW,SO,DWDP}	Metals & Mining	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:12:03.84176	2020-10-11 23:38:31.023007
GT	The Goodyear Tire & Rubber Company	Goodyear Tire & Rubber Co is engaged in manufacturing of tires for various applications. The company's products are used in automobiles, trucks, buses, and others. It also manufactures and markets rubber-related chemicals for various applications.	{"Consumer Cyclical","Rubber & Plastics",Autos}	{CTB}	Autos	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:03.858736	2020-10-11 23:38:31.024353
GTT	GTT Communications Inc.	GTT Communications Inc provides network services, internet services, managed network and security services, and voice and unified communication services to enterprise, carrier, and government clients.	{"Communication Services","Telecom Services"}	{SHEN,CMCSA,CTL,VZ,T,IDT,GLOW}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:03.883438	2020-10-11 23:38:31.025871
GULF	WisdomTree Middle East Dividend Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree Middle East Dividend Index.\n Under normal circumstances, at least 95% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in the component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a fundamentally weighted index that is comprised of companies in the Middle East region that pay regular cash dividends on shares of common stock. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.89844	2020-10-11 23:38:31.02582
GVIP	Goldman Sachs Hedge Industry VIP	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the Goldman Sachs Hedge Fund VIP Index®.\n The fund seeks to achieve its investment objective by investing at least 80% of its assets in securities included in its underlying index, in depositary receipts representing securities included in its underlying index and in underlying stocks in respect of depositary receipts included in its underlying index. The index is designed to deliver exposure to equity securities whose performance is expected to influence the long portfolios of hedge funds.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.914297	2020-10-11 23:38:31.028166
GWX	SPDR S&P International SmallCap	The investment seeks investment results that, before fees and expenses, correspond generally to the total return performance of the S&P Developed Ex-U.S. under USD2 Billion Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index and in depositary receipts based on securities comprising the index. The index is a market capitalization weighted index designed to define and measure the investable universe of publicly traded small-cap companies, as defined by the index, domiciled in developed countries outside the United States. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.928276	2020-10-11 23:38:31.02582
H	Hyatt Hotels Corporation Class A	Hyatt Hotels Corp develop, own, operate, manage, franchise, license, or provide services to a portfolio of properties, consisting of full-service hotels, select service hotels, resorts, and other properties.	{"Consumer Cyclical",Lodging,"Travel & Leisure"}	{MTN,WYND,MAR,HLT,IHG}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:03.940553	2020-10-11 23:38:31.024353
HALL	Hallmark Financial Services Inc.	Hallmark Financial Services Inc is an insurance holding company, which is engaged in the sale of property and casualty insurance products to businesses and individuals. The Specialty Commercial segment generates maximum revenue for the company.	{"Financial Services","Insurance - Property & Casualty"}	{CINF,AFG,WRB,CNA,MKL,ORI,TRV,AFH}	Insurance - Property & Casualty	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:03.955881	2020-10-11 23:38:31.023428
KF	Korea Fund Inc. (The)	Korea Fund Inc is a United States based closed-end, non-diversified management investment company. Its investment objective is to seek long-term capital appreciation through investment in securities, primarily equity securities, of Korean companies.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.740594	2020-10-11 23:38:31.028166
GRIF	Griffin Industrial Realty Inc.	Griffin Industrial Realty Inc through its subsidiary operates a real estate business. The company is engaged in the development, management, and leasing of commercial and industrial properties. Its property includes Class A industrial, and others.	{"Real Estate","Real Estate Services"}	{BLDR,LL,IOR}	Real Estate Services	Real Estate	NASDAQ Global Market	\N	2020-10-11 23:12:03.784237	2020-10-11 23:38:31.023428
GRN	Barclays Bank PLC	The investment seeks to provide investors with exposure to the Barclays Global Carbon Index Total Return™. \nThe Barclays Global Carbon Index Total Return™ (the "index") is designed to measure the performance of the most liquid carbon-related credit plans. Each carbon-related credit plan included in the index is represented by the most liquid instrument available in the marketplace. The index expects to incorporate new carbon-related credit plans as they develop around the world.	{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:03.796046	2020-10-11 23:38:31.02423
GRX	The Gabelli Healthcare & Wellness Trust of Beneficial Interest	The Healthcare Trust will invest at least 80% of its assets, plus borrowings made for investment purposes, in equity securities such as common stock and preferred stock and income producing securities, such as fixed income debt securities and securities convertible into common stock, of domestic and foreign companies in the healthcare and wellness industries. The remaining 20% of its assets may be invested in other securities, including stocks, debt obligations and money market instruments, as well as certain derivative instruments in the healthcare and wellness industries or other industries.	{"Financial Services","Asset Management"}	{}			New York Stock Exchange	\N	2020-10-11 23:12:03.80983	2020-10-11 23:38:31.025871
GSC	Goldman Sachs Connect S&P Enhanced Commodity Total Return Strategy Index	The investment seeks to replicates, net of expenses, the S&P GSCI Enhanced Commodity Total Return Strategy Index.\n The index reflects the total returns that are potentially available through an unleveraged investment in the same futures contracts as are included in the S&P GSCI.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.822147	2020-10-11 23:38:31.02423
GSIE	Goldman Sachs ActiveBeta International Equity	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the Goldman Sachs ActiveBeta® International Equity Index.\n The fund invests at least 80% of its assets (exclusive of collateral held from securities lending) in securities included in its underlying index, in depositary receipts representing securities included in its underlying index and in underlying stocks in respect of depositary receipts included in its underlying index. The index is designed to deliver exposure to equity securities of developed market issuers outside of the United States.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.834335	2020-10-11 23:38:31.02582
GSP	Barclays Bank PLC iPath Exchange Traded Notes due June 12 2036 Linked to GSCI Total Return Index	The investment seeks to provide investors with exposure to the S&P GSCI® Total Return Index. \nThe S&P GSCI® Total Return Index provides exposure to the returns potentially available through an unleveraged investment in the contracts comprising the S&P GSCI® plus the Treasury Bill rate of interest that could be earned on funds committed to the trading of the underlying contracts.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.844484	2020-10-11 23:38:31.024353
GSY	Invesco Ultra Short Duration	The investment seeks maximum current income, consistent with preservation of capital and daily liquidity.\n The fund will invest at least 80% of its net assets in fixed income securities and in ETFs and closed-end funds that invest substantially all of their assets in fixed income securities. It uses a low duration strategy to seek to outperform the ICE BofAML US Treasury Bill Index in addition to providing returns in excess of those available in U.S. Treasury bills, government repurchase agreements, and money market funds, while seeking to provide preservation of capital and daily liquidity. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.855534	2020-10-11 23:38:31.02582
GTIM	Good Times Restaurants Inc.	Good Times Restaurants Inc through its wholly owned subsidiary is engaged in developing, owning, operating and franchising hamburger-oriented drive-through restaurants under the name Good Times Burgers and Frozen Custard.	{"Consumer Cyclical",Restaurants}	{JACK,SONC,TAST,MCD,ARCO,WEN,ZOES}	Restaurants	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:03.87316	2020-10-11 23:38:31.028166
GTY	Getty Realty Corporation	Getty Realty Corp is a real estate investment trust that owns, leases and finances convenience store and gasoline station properties which are operates under the brand names 76, Aloha, BP, Citgo, Conoco, Exxon, Getty, Mobil, Shell and Valero.	{"Real Estate","REIT - Retail",REITs}	{MAC,ADC,KIM}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:03.89289	2020-10-11 23:38:31.024353
GUDB	Sage ESG Intermediate Credit	The investment seeks to replicate investment results that generally correspond, before fees and expenses, to the performance of the Sage ESG Intermediate Credit Index.\n The fund generally will invest at least 80% of its total assets in the component securities of the SAGE ESG Credit Index (the "index"). The index consists of corporate bonds selected from the Barclays Capital U.S. Intermediate Credit Bond Index that meet Environmental, Social and Governance (ESG) criteria jointly developed by the fund's investment adviser, Sage Advisory Services LTD Co. (the "Adviser"), and Sustainalytics.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:03.895118	2020-10-11 23:38:31.02423
GV	Goldfield Corporation (The)	The Goldfield Corp through subsidiaries is engaged in the construction of electrical infrastructure for the utility industry and industrial customers, in the Southeast and mid-Atlantic regions of the United States and Texas.	{Industrials,"Engineering & Construction"}	{MYRG,PRIM,KBR,NEE,AMRC,AGX,DUK}	Engineering & Construction	Industrials	NYSE American	\N	2020-10-11 23:12:03.910825	2020-10-11 23:38:31.023428
GRPN	Groupon Inc.	Groupon Inc offers marketing services by selling vouchers through online local marketplaces that can be redeemed for goods or services with third-party merchants. The company's segments include North America and International.	{Technology,"Internet Content & Information","Online Media"}	{AMZN,EBAY,INWK,LYV,XOXO,VIPS,GOOGL,WMT,XLK}	Online Media	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:03.800578	2020-10-11 23:38:31.024353
GSB	GlobalSCAPE Inc.	GlobalSCAPE Inc develops and sells computer software that provides secure information exchange, data transfer and sharing capabilities for enterprises and consumers.	{Technology,"Software - Application","Application Software"}	{CSCO,MSFT,CTXS,IBM,MOBL,SMSI,XLK}	Application Software	Technology	NYSE American	\N	2020-10-11 23:12:03.813347	2020-10-11 23:38:31.028166
GSEU	Goldman Sachs ActiveBeta Europe Equity	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the Goldman Sachs ActiveBeta® Europe Equity Index (the "index").\n The fund seeks to achieve its investment objective by investing at least 80% of its assets (exclusive of collateral held from securities lending) in securities included in its underlying index, in depositary receipts representing securities included in its underlying index and in underlying stocks in respect of depositary receipts included in its underlying index. The index is designed to deliver exposure to equity securities of developed markets issuers in Europe.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.826236	2020-10-11 23:38:31.031328
GSK	GlaxoSmithKline PLC	GlaxoSmithKline PLC creates, discovers, develops, manufactures and markets pharmaceutical products including respiratory and antiviral, vaccines, over-the-counter (OTC) medicines and health-related consumer products.	{Healthcare,"Drug Manufacturers - Major","Drug Manufacturers"}	{SHPG,AZN,NVS,PFE,SNY,MRK,TEVA,DVAX,INVA,XLV}	Drug Manufacturers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:03.838445	2020-10-11 23:38:31.021426
GSS	Golden Star Resources Ltd	Golden Star Resources Ltd is a gold-mining company. It is engaged in production of gold in Ghana, West Africa. The company also conducts gold exploration in West Africa and South America.	{"Basic Materials",Gold,"Metals & Mining"}	{}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:03.848922	2020-10-11 23:38:31.028166
GTE	Gran Tierra Energy Inc.	Gran Tierra Energy Inc is an independent energy company. It is engaged in the acquisition, exploration, development, and production of oil and gas properties in Colombia producing light crude oil, with medium crude and natural gas.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{EC,YPF,PBR,RDS.A,GPRK}	Oil & Gas - E&P	Energy	NYSE American	\N	2020-10-11 23:12:03.860736	2020-10-11 23:38:31.023007
GTN	Gray Communications Systems Inc.	Gray Television Inc is a television broadcast company which owns and operates television stations and digital assets in United States.	{"Consumer Cyclical","Broadcasting - TV",Entertainment}	{SBGI,NXST,DIS,EVC,CBS,FOXA,SSP}	Entertainment	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:03.874941	2020-10-11 23:38:31.031328
GTYH	GTY Technology Holdings Inc.	GTY Technology Holdings Inc is a blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:03.892206	2020-10-11 23:38:31.023428
GTYHU	GTY Technology Holdings Inc. Unit	GTY Technology Holdings Inc is a blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:03.893339	2020-10-11 23:38:31.028166
GUSH	Direxion Daily S&P Oil & Gas Exp. & Prod. Bull 3X Shares	The investment seeks daily investment results, of 300% of the daily performance of the S&P Oil & Gas Exploration & Production Select Industry Index.\n The fund, under normal circumstances, invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, exchange-traded funds ("ETFs") that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index is designed to measure the performance of a sub-industry or group of sub-industries determined based on the Global Industry Classification Standards ("GICS"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.904786	2020-10-11 23:38:31.023007
GVP	GSE Systems Inc.	GSE Systems Inc is a software solutions provider. Most of its revenue is generated through the sale of applications catering to training and consulting. It sells its products mainly in the United States.	{Technology,"Software - Application","Application Software"}	{ADSK,SOFO,EMR,GE,HON,ESE,HII,LLL,XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:03.914165	2020-10-11 23:38:31.028166
GWGH	GWG Holdings Inc	GWG Holdings Inc is a specialty finance company & a purchaser of life insurance policies in the secondary market. The Company & its subsidiaries offer investors the opportunity for higher yield & increased diversification through alternative investments.	{"Financial Services","Insurance - Life"}	{LNC,MET}	Insurance - Life	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:03.919537	2020-10-11 23:38:31.025871
GXG	Global X MSCI Colombia	The investment seeks to provide investment results that correspond generally to the price and yield performance, before fees and expenses, of the MSCI All Colombia Select 25/50 Index.\n The fund invests at least 80% of its total assets in the securities of the underlying index and in American Depositary Receipts ("ADRs") and Global Depositary Receipts ("GDRs") based on the securities in the underlying index. It also invests at least 80% of its total assets in securities of companies that are economically tied to Colombia. The underlying index is designed to represent the performance of the broad Colombia equity universe. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.930655	2020-10-11 23:38:31.028166
GRUB	GrubHub Inc.	GrubHub Inc is the online and mobile platform for restaurant pick-up and delivery orders. The company provides restaurants with more orders, helps to serve diners better and facilitates delivery logistics in many markets.	{Technology,"Internet Content & Information","Online Media"}	{BDL,BLMN,BURG,XLK}	Online Media	Technology	New York Stock Exchange	\N	2020-10-11 23:12:03.802651	2020-10-11 23:38:31.023007
GSIT	GSI Technology Inc.	GSI Technology Inc develops and markets high performance memory products, including "Very Fast" static random access memory, or SRAM, and low latency dynamic random access memory, or LDRAM.	{Technology,Semiconductors}	{RMBS,CY,IDTI,MU,MOSY,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:03.838428	2020-10-11 23:38:31.028166
GSUM	Gridsum Holding Inc.	Gridsum Holding Inc provides sophisticated data analysis software for multinational and domestic enterprises and government agencies in China.	{Technology,"Software - Application","Application Software"}	{OTEX,PANW,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:03.853282	2020-10-11 23:38:31.023428
GTHX	G1 Therapeutics Inc.	G1 Therapeutics Inc is a clinical-stage biopharmaceutical company. It engages in developing novel, small-molecule therapies to target significant unmet needs to treat cancer.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:03.870513	2020-10-11 23:38:31.02423
GTXI	GTx Inc.	GTx Inc is a biopharmaceutical company. It is engaged in the discovery, development and commercialization of small molecules for treatment of cancer, including treatments for breast and prostate cancer and other medical conditions.	{Healthcare,Biotechnology}	{AMGN,AVEO,GSK,JNJ,LLY,PFE,TR,LGND,CYTK,RMCF,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:03.88878	2020-10-11 23:38:31.023007
GURU	Global X Guru Index	The investment seeks investment results that correspond generally to the price and yield performance, before fees and expenses, of the Solactive Guru Index ("underlying index").\n The fund invests at least 80% of its total assets in the securities of the underlying index and in American Depositary Receipts ("ADRs") and Global Depositary Receipts ("GDRs") based on the securities in the underlying index. The underlying index is comprised of the top U.S. listed equity positions reported on Form 13F by a select group of entities characterized as hedge funds.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.903406	2020-10-11 23:38:31.025871
GWRE	Guidewire Software Inc.	Guidewire Software Inc is a provider of software products for property and casualty insurers. It provides Internet-based software platforms for core insurance operations, including underwriting and policy administration, claims management and billing.	{Technology,"Software - Infrastructure","Application Software"}	{PEGA,EBIX,IBM,BNFT,MJCO,DXC,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:03.924471	2020-10-11 23:38:31.023007
GWR	Genesee & Wyoming Inc. Class A	Genesee & Wyoming Inc owns and leases short line and regional freight railroads across the world. It provides rail freight services, contract coal loading and railcar switching for industrial customers.	{Industrials,Railroads,"Transportation & Logistics"}	{CNI,CP,NSC,UNP,KSU,CSX}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:03.920125	2020-10-11 23:38:31.031328
GYLD	Arrow Dow Jones Global Yield ETF	The investment seeks investment results that generally correspond, before fees and expenses, to the price and yield performance of the Dow Jones Global Composite Yield Index (the "underlying index").\n The fund uses a "passive" or "indexing" investment approach to seek to track the price and yield performance of the underlying index. It invests at least 80% of its total assets in the component securities of the underlying index (or depositary receipts representing those securities). The underlying index seeks to identify the 150 highest yielding investable securities in the world within three "asset classes."	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.934266	2020-10-11 23:38:31.023007
HAFC	Hanmi Financial Corporation	Hanmi Financial Corp is a community bank conducting general banking business. It provides its services to small- to medium-sized businesses. It also offers insurance products such as; life, commercial, automobile, health, and property and casualty.	{"Financial Services","Banks - Regional - US",Banks}	{CVBF,HTBK,WABC,PMBC,UBFO,PPBI,RNST,PACW,CFR}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:03.952692	2020-10-11 23:38:31.021426
HAL	Halliburton Company	Halliburton Co is a provider of services and products to the upstream oil and natural gas industry throughout the lifecycle of the reservoir, from locating hydrocarbons and managing geological data, to drilling and formation evaluation.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{CKH,SPN,NOV,SLB,NR,OII,RES,TTI,RDS.A,BHGE}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:03.953693	2020-10-11 23:38:31.02582
HBAN	Huntington Bancshares Incorporated	Huntington Bancshares Inc is a multi-state financial holding company. It provides full-service commercial, small business, consumer, and mortgage banking services, as well as automobile financing, investment management services, and among others.	{"Financial Services","Banks - Regional - US",Banks}	{OVBC,UBOH,FITB,FFBC,WSBC,BBT,BAC,FNB,PRK,LCNB}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:03.968456	2020-10-11 23:38:31.024353
GS	Goldman Sachs Group Inc. (The)	Goldman Sachs Group Inc is engaged in capital market activities. Its services primarily include investment banking operations.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{BCS,MS,BAC,DB,JPM,SMFG,C}	Brokers & Exchanges	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.810501	2020-10-11 23:38:31.02423
GSL	Global Ship Lease Inc New Class A	Global Ship Lease Inc owns a fleet of modern containerships of diverse sizes and charters the vessels out under long-term, fixed-rate charters to reputable container shipping companies.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:03.839182	2020-10-11 23:38:31.02423
GSSC	GS ActiveBeta U.S. Small Cap Equity	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the Goldman Sachs ActiveBeta® U.S. Small Cap Equity Index.\n The fund invests at least 80% of its assets in securities included in its index. The index is designed to deliver exposure to equity securities of small-capitalization U.S. issuers. The index is constructed using the patented ActiveBeta® Portfolio Construction Methodology, which was developed to provide exposure to the "factors" that are commonly tied to a stock's outperformance relative to market returns. These factors include value, momentum, quality and low volatility.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.850557	2020-10-11 23:38:31.028166
GTES	Gates Industrial Corporation plc	Gates Industrial Corp PLC manufactures power transmission and fluid power products and components for industrial and automotive applications. It operates in two segments including Power Transmission and Fluid Power.	{Industrials,"Diversified Industrials","Industrial Products"}	{}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:03.861894	2020-10-11 23:38:31.028166
GTS	Triple-S Management Corporation Class B	Triple-S Management Corp is a Puerto Rico-based company serving the managed care industry. The company offers a broad portfolio of managed care products, life insurance, accident and disability insurance and property and casualty insurance.	{"Financial Services","Insurance - Diversified",Insurance}	{HUM,CNC,MOH,WCG}	Insurance	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.87714	2020-10-11 23:38:31.02582
GTYHW	GTY Technology Holdings Inc. Warrants		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:03.893805	2020-10-11 23:38:31.031328
GUT	Gabelli Utility Trust (The)	The Funds primary investment objective is to achieve long-term growth of capital and income. The Fund will invest in ordinary stock and other securities of foreign and domestic companies involved to a substantial extent in providing products, services or equipment for the generation or distribution of electricity, gas and water and the provisions of telecommunications services or infrastructure operations, such as airports, toll roads and municipal services.	{"Financial Services","Asset Management"}	{}			New York Stock Exchange	\N	2020-10-11 23:12:03.906657	2020-10-11 23:38:31.024353
GWPH	GW Pharmaceuticals Plc	GW Pharmaceuticals PLC is engaged in the research, development and commercialisation of cannabinoid prescription medicines using botanical extracts derived from the Cannabis Sativa plant.	{Healthcare,"Drug Manufacturers - Major","Drug Manufacturers"}	{NVS,INSY,XLV}	Drug Manufacturers	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:03.920547	2020-10-11 23:38:31.023428
GWRS	Global Water Resources Inc.	Global Water Resources Inc is a water resource management company. It operates and manages water, wastewater and recycled water utilities in strategically located communities, principally in metropolitan Phoenix, Arizona.	{Utilities,"Utilities - Regulated Water","Utilities - Regulated"}	{GWRS,XLU}	Utilities - Regulated	Utilities	NASDAQ Global Market	\N	2020-10-11 23:12:03.926206	2020-10-11 23:38:31.02423
GYC	Corporate Asset Backed Corp CABCOoration CABCO Series 2004-102 Trust (SBC Communications Inc.) Collared Floating Rate Cllable Certificates		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:03.933632	2020-10-11 23:38:31.025871
HA	Hawaiian Holdings Inc.	Hawaiian Holdings Inc provides air transportation for passengers and cargo. Routes are principally focused on transportation between the Hawaiian Islands, United States, and the South Pacific regions of Asia and Australia.	{Industrials,Airlines}	{SKYW,JBLU,ALGT,ALK,LUV,DAL,AAL,SAVE,UAL}	Airlines	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:03.94083	2020-10-11 23:38:31.023428
HAIN	The Hain Celestial Group Inc.	The Hain Celestial Group Inc manufactures, markets, and sells natural, GMO (genetically modified organism)-free, and organic products across several foods, beverage, and personal-care categories.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{CPB,GIS,SJM,THS,MDLZ,KHC}	Consumer Packaged Goods	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:03.952541	2020-10-11 23:38:31.025871
HAO	Invesco China Small Cap	The investment seeks to track investment results (before fees and expenses) of the AlphaShares China Small Cap Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities (including American depositary receipts ("ADRs") and global depositary receipts ("GDRs")) that comprise the underlying index. The underlying index is comprised of equity securities of publicly-traded mainland China companies of small capitalization. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.955881	2020-10-11 23:38:31.024353
HBCP	Home Bancorp Inc.	Home Bancorp Inc is a bank holding company. It attracts deposits from the general public and uses those funds to invest in loans and securities.	{"Financial Services","Savings & Cooperative Banks",Banks}	{SMBC,HFBL,PBSK,BHLB,LTXB}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:03.9734	2020-10-11 23:38:31.02423
GSJY	Goldman Sachs ActiveBeta Japan Equity	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the Goldman Sachs ActiveBeta® U.S. Small Cap Equity Index.\n The fund invests at least 80% of its assets in securities included in its index. The index is designed to deliver exposure to equity securities of small-capitalization U.S. issuers. The index is constructed using the patented ActiveBeta® Portfolio Construction Methodology, which was developed to provide exposure to the "factors" that are commonly tied to a stock's outperformance relative to market returns. These factors include value, momentum, quality and low volatility.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.838868	2020-10-11 23:38:31.023428
GSV	Gold Standard Ventures Corporation (Canada)	Gold Standard Ventures Corp is an advanced stage gold exploration company. It is also engaged in acquiring, exploring mineral projects, with project situated across Nevada for gold and other minerals.	{"Basic Materials",Gold,"Metals & Mining"}	{}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:03.852926	2020-10-11 23:38:31.025871
GTO	Invesco Total Return Bond	The investment seeks maximum total return, comprised of income and capital appreciation.\n The fund will normally invest in a portfolio of fixed income instruments of varying maturities and of any credit quality. It will normally invest at least 80% of its net assets (plus any borrowings for investment purposes) in fixed income instruments, which may be represented by certain derivative instruments, and also include exchange-traded funds ("ETFs") and closed-end funds ("CEFs") that invest substantially all of their assets in fixed income instruments (which may include ETFs and CEFs affiliated with the fund). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.876729	2020-10-11 23:38:31.021426
GURE	Gulf Resources Inc.	Gulf Resources Inc manufactures and trades bromine and crude salt, and manufactures and sells chemical products used in oil and gas field exploration, oil and gas distribution, oil field drilling, wastewater processing, papermaking chemical agents.	{"Basic Materials","Specialty Chemicals",Chemicals}	{BCPC,OLN,ASH}	Chemicals	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:12:03.900903	2020-10-11 23:38:31.021426
GVAL	Cambria Global Value	The investment seeks investment results that correspond (before fees and expenses) generally to the price and yield performance of its underlying index, the Cambria Global Value Index (the "underlying index").\n Under normal market conditions, the fund will invest at least 80% of its total assets in the components of the underlying index and in depositary receipts representing components of the underlying index. The underlying index is comprised of equity securities of issuers located in developed and emerging countries, as well as exchange-traded funds composed of issuers located in such countries.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.912752	2020-10-11 23:38:31.02423
GXC	SPDR S&P China	The investment seeks investment results that, before fees and expenses, correspond generally to the total return performance of the S&P China BMI Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index and in depositary receipts based on securities comprising the index. The index is a market capitalization weighted index designed to define and measure the investable universe of publicly traded companies domiciled in China available to foreign investors. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.930464	2020-10-11 23:38:31.028166
GXF	Global X FTSE Nordic Region	The investment seeks investment results that correspond generally to the price and yield performance, before fees and expenses, of the FTSE Nordic 30 Index.\n The fund invests at least 80% of its total assets in the securities of the underlying index and in American Depositary Receipts ("ADRs") and Global Depositary Receipts ("GDRs") based on the securities in the underlying index. The underlying index is designed to reflect the equity market performance of companies in Sweden, Denmark, Norway and Finland. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.930209	2020-10-11 23:38:31.021426
HAE	Haemonetics Corporation	Haemonetics Corp is a health care company. It provides products enabling in the processing, handling and analysis of blood and portfolio of integrated devices and information management tools to improve clinical outcomes.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{CERS,BAX,BDX,JNJ,SYK,TMO,PKI,MDT,XLV}	Medical Instruments & Equipment	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:03.948571	2020-10-11 23:38:31.028166
HALO	Halozyme Therapeutics Inc.	Halozyme Therapeutics Inc is a biotechnology company. It is engaged in research on human enzymes that alter the extracellular matrix and tumor environment.	{Healthcare,Biotechnology}	{BAX,JNJ,LLY,PFE,SNY,ALBO,MNKD,XON,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:03.956963	2020-10-11 23:38:31.031328
HAS	Hasbro Inc.	Hasbro Inc provides children and family leisure time and entertainment products and services which includes manufacturing and marketing of games and toys. It reaches customers by leveraging its well-known brands such as Transformers, Monopoly, and Nerf.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{JAKK,EA,MAT,SGMS,ATVI,DIS,VIAB,PTE}	Travel & Leisure	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:03.959194	2020-10-11 23:38:31.028166
HBANO	Huntington Bancshares Incorporated Depositary Shares	Huntington Bancshares Inc is a multi-state financial holding company. It provides full-service commercial, small business, consumer, and mortgage banking services, as well as automobile financing, investment management services, and among others.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:03.9734	2020-10-11 23:38:31.023007
HAIR	Restoration Robotics Inc.	Restoration Robotics Inc is a medical device company. It develops and commercializes a robotic device, the ARTAS System, that assists physicians in surgery.	{Healthcare,"Medical Devices"}	{XLV}	Medical Devices	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:03.952421	2020-10-11 23:38:31.023007
HCAPZ	Harvest Capital Credit Corporation 6.125% Notes due 2022		{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.992157	2020-10-11 23:38:31.02582
HDG	ProShares Hedge Replication	The investment seeks investment results, before fees and expenses, that track the performance of the Merrill Lynch Factor Model-Exchange Series (the "benchmark").\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should track the performance of the benchmark. The benchmark seeks to provide the risk and return characteristics of the hedge fund asset class by targeting a high correlation to the HFRI Fund Weighted Composite Index (the "HFRI"). The HFRI is designed to reflect hedge fund industry performance through an equally weighted composite of over 2000 constituent funds. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.023054	2020-10-11 23:38:31.023428
HDS	HD Supply Holdings Inc.	HD Supply Holdings Inc is an industrial distributor. It provides a broad range of products and value-added services to customers in various sectors such as maintenance, repair and operations, infrastructure and power and specialty construction.	{Industrials,"Industrial Distribution"}	{FAST,HWCC,MSM,AXE,WCC,GWW,WSO,DNOW}	Industrial Distribution	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:04.034614	2020-10-11 23:38:31.028166
HEFA	iShares Currency Hedged MSCI EAFE	The investment seeks to track the investment results of the MSCI EAFE® 100% Hedged to USD Index.\n The fund generally will invest at least 90% of its assets in the component securities (including indirect investments through the underlying fund) and other instruments of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is an equity benchmark for its international stock performance with the currency risk inherent in the securities included in the underlying index hedged to the U.S. dollar on a monthly basis.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:04.049893	2020-10-11 23:38:31.023428
HEWI	iShares Currency Hedged MSCI Italy	The investment seeks to track the investment results of the MSCI Italy 25/50 100% Hedged to USD Index.\n The fund generally will invest at least 90% of its assets in the component securities (including indirect investments through the underlying fund) and other instruments of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index consists of stocks traded primarily on the Milan Stock Exchange with the currency risk inherent in the securities included in the underlying index hedged to the U.S. dollar on a monthly basis.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.073905	2020-10-11 23:38:31.031328
HF	HFF Inc. Class A	HFF Inc is a commercial real estate and capital markets services provider. The company acts as an investment bank and broker to ensure debt placement, investment sales, structured finance, loan sales and commercial loan servicing.	{"Real Estate","Real Estate Services"}	{NVR,JLL,EVR,CBRE}	Real Estate Services	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:04.088872	2020-10-11 23:38:31.023007
HFWA	Heritage Financial Corporation	Heritage Financial Corp is a bank holding company. The company through its subsidiary provides commercial lending and deposit relationships with small businesses and their owners in its market areas and attracting deposits from the general public.	{"Financial Services","Banks - Regional - US",Banks}	{COLB,FRC,BANR}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:04.101574	2020-10-11 23:38:31.024353
HHS	Harte-Hanks Inc.	Harte-Hanks Inc is a multi-channel marketing company. The marketing services offered include agency and digital services, database marketing solutions and business-to-business lead generation, direct mail and contact centers.	{"Consumer Cyclical","Marketing Services","Advertising & Marketing Services"}	{}	Advertising & Marketing Services	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:04.123359	2020-10-11 23:38:31.024353
HIIQ	Health Insurance Innovations Inc.	Health Insurance Innovations Inc is a developer, distributor and cloud-based administrator of affordable individual health and family insurance plans including short-term medical insurance plans, guaranteed-issue and underwritten hospital indemnity plans.	{Healthcare,"Health Care Plans"}	{EHTH,CI,XLV}	Health Care Plans	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:04.140382	2020-10-11 23:38:31.021426
OCLR	Oclaro Inc.	Oclaro Inc is engaged in providing laser and optical components, modules and subsystems for the telecommunication and data communication equipment companies.	{Technology,Semiconductors}	{CSCO,FNSR,NPTN,EMKR,AAOI,VIAV,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:08.272182	2020-10-11 23:38:31.02423
HEEM	iShares Currency Hedged MSCI Emerging Markets	The investment seeks to track the investment results of the MSCI Emerging Markets 100% Hedged to USD Index.\n The fund generally will invest at least 90% of its assets in the component securities (including indirect investments through the underlying fund) and other instruments of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is an equity benchmark for global emerging markets stock performance with the currency risk inherent in the securities included in the index hedged to the U.S. dollar on a monthly basis.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:04.041187	2020-10-11 23:38:31.023007
HEWJ	iShares Currency Hedged MSCI Japan	The investment seeks to track the investment results of the MSCI Japan 100% Hedged to USD Index.\n The fund generally will invest at least 90% of its assets in the component securities (including indirect investments through the underlying fund) and other instruments of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index consists of stocks traded primarily on the Tokyo Stock Exchange with the currency risk inherent in the securities included in the underlying index hedged to the U.S. dollar on a monthly basis.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.074779	2020-10-11 23:38:31.021426
HFBC	HopFed Bancorp Inc.	HopFed Bancorp Inc serves as a savings and loan holding company. The Bank attracts deposits from the general public and investing such deposits in loans secured by single family residential real estate and investment securities.	{"Financial Services","Savings & Cooperative Banks",Banks}	{KFFB,HBCP,PBSK,BHLB,LTXB}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:04.091757	2020-10-11 23:38:31.02582
HHC	The Howard Hughes Corporation	The Howard Hughes Corp is a real estate company. It is in the development of master planned communities and other strategic real estate development opportunities across the United States.	{"Real Estate","Real Estate - General","Real Estate Services"}	{TCI,NEN,CTO}	Real Estate Services	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:04.11588	2020-10-11 23:38:31.021426
HII	Huntington Ingalls Industries Inc.	Huntington Ingalls Industries Inc is engaged in designing, building, overhauling & repairing ships for United States Navy and Coast Guard. The company also provides refueling and complex overhaul services for nuclear-powered aircraft carriers.	{Industrials,"Aerospace & Defense"}	{AVAV,LMT,GD,HON,NOC,AJRD}	Aerospace & Defense	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:04.13981	2020-10-11 23:38:31.028166
HLI	Houlihan Lokey Inc. Class A	Houlihan Lokey Inc is an investment bank with expertise in mergers and acquisitions, capital markets and strategic consulting. It operates in three segments namely, Corporate Finance, Financial Restructuring and Financial Advisory Services.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{MKTX,MS,OPY}	Brokers & Exchanges	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.178709	2020-10-11 23:38:31.023007
HMNF	HMN Financial Inc.	HMN Financial Inc is a stock savings bank holding company that owns 100% of Home Federal Savings Bank. It operates retail banking and loan production facilities in Minnesota, Iowa, and Wisconsin.	{"Financial Services","Savings & Cooperative Banks",Banks}	{SMBC,LARK,CASH,CZWI,CFFN}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:04.205284	2020-10-11 23:38:31.025871
HNI	HNI Corporation	HNI Corp provides office furniture and hearth products. It manufactures and markets products including metal and wood commercial and home office furniture and gas, electric, wood and biomass burning fireplaces and stoves for home.	{Industrials,"Business Equipment","Industrial Products"}	{MLHR,KEQU,KBAL,VIRC,SCS,SNA,KNL}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:04.220042	2020-10-11 23:38:31.021426
HOLX	Hologic Inc.	Hologic Inc develops, manufactures and supplies diagnostics products, medical imaging systems and surgical products for women's health. Its products are used in diagnosis of human diseases and screen donated human blood and plasma.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{DGX,ABT,BDX,BSX,GE,JNJ,TMO,PHG,XLV}	Medical Instruments & Equipment	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:04.239789	2020-10-11 23:38:31.028166
HOV	Hovnanian Enterprises Inc. Class A	Hovnanian Enterprises Inc is an American construction company. It designs, constructs, markets and sells single-family detached homes, attached townhomes and condominiums, urban infill and active lifestyle homes in planned residential developments.	{"Consumer Cyclical","Residential Construction","Homebuilding & Construction"}	{CVCO,PHM,NVR,DHI,LEN,MDC,TOL,KBH,CHCI}	Homebuilding & Construction	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:04.259563	2020-10-11 23:38:31.028166
HQL	TeklaLife Sciences Investors	Tekla Life Sciences Investors is a United States based closed-end management investment company. Its investment objective is to seek long-term capital appreciation by investing primarily in the securities of life sciences companies.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.284449	2020-10-11 23:38:31.023007
HDAW	Xtrackers MSCI All World ex US High Dividend Yield Equity	The investment seeks investment results that correspond generally to the performance of the MSCI ACWI ex US High Dividend Yield Index.\n The fund will invest at least 80% of its total assets (but typically far more) in component securities of the underlying index. The underlying index is designed to reflect the performance of equities (excluding REITs) in its parent index, the MSCI ACWI ex US Index, with higher dividend income and quality characteristics than average dividend yields of equities in the parent index, where such higher dividend income and quality characteristics are both sustainable and persistent.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.0183	2020-10-11 23:38:31.028166
HDP	Hortonworks Inc.	Hortonworks Inc is engaged in developing, distributing and supporting a new class of data management software solutions built on open source technology.	{Technology,"Software - Infrastructure","Application Software"}	{RNG,MOBL,WK,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:04.03313	2020-10-11 23:38:31.023007
HEES	H&E Equipment Services Inc.	H&E Equipment Services Inc is an integrated equipment services company. It rents, sells and provides parts and service support for hi-lift or aerial work platform equipment, cranes, earthmoving equipment, and industrial lift trucks categories.	{Industrials,"Rental & Leasing Services","Consulting & Outsourcing"}	{MIND,URI,CCI}	Consulting & Outsourcing	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:04.048923	2020-10-11 23:38:31.021426
HEWG	iShares Currency Hedged MSCI Germany ETF	The investment seeks to track the investment results of the MSCI Germany 100% Hedged to USD Index.\n The fund generally will invest at least 90% of its assets in the component securities (including indirect investments through the underlying fund) and other instruments of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index consists of stocks traded primarily on the Frankfurt Stock Exchange with the currency risk inherent in the securities included in the underlying index hedged to the U.S. dollar on a monthly basis.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:04.072123	2020-10-11 23:38:31.023007
HFBL	Home Federal Bancorp Inc. of Louisiana	Home Federal Bancorp Inc of louisiana operates as the holding company for Home Federal Bank. It offers financial services to individuals, corporate entities and other organizations in northwest Louisiana.	{"Financial Services","Savings & Cooperative Banks",Banks}	{KFFB,HBCP,OFED}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:04.093632	2020-10-11 23:38:31.031328
HIE	Miller/Howard High Income Equity Fund of Beneficial Interest	Miller/Howard High Income Equity Fund is a closed-end, non-diversified management investment company. The fund's primary investment objective is to seek a high level of current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.12558	2020-10-11 23:38:31.031328
HIPS	GraniteShares HIPS US High Income	The investment seeks to track the performance, before fees and expenses, of the TFMS HIPS 300 Index.\n The fund employs a "passive management"-or indexing-investment approach designed to track the performance of the index. The rules-based index measures the performance of up to 300 high income U.S.-listed securities that typically have "pass-through" structures that require them to distribute substantially all of their earnings to shareholders as cash distributions. This "high income, pass-through" strategy is known as HIPS.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.152496	2020-10-11 23:38:31.025871
HLF	Herbalife Nutrition Ltd.	Herbalife Nutrition Ltd is a nutrition company. The company has five revenue segments: Weight Management; Targeted Nutrition; Energy, Sports, and Fitness; Outer Nutrition; and Literature, Promotional, and Other.	{"Consumer Defensive","Household & Personal Products","Consumer Packaged Goods"}	{NATR,NUS,AVP,MED,USNA,GNC,MTEX}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:04.177828	2020-10-11 23:38:31.024353
HMHC	Houghton Mifflin Harcourt Company	Houghton Mifflin Harcourt Co provides content, services and technology to both educational institutions and consumers, reaching over 50 million students in more than 150 countries. It also published trade and reference materials.	{"Consumer Defensive","Education & Training Services",Education}	{}	Education	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:04.195762	2020-10-11 23:38:31.028166
HMTA	HomeTown Bankshares Corporation	HomeTown Bankshares Corp is a bank holding company. The company through its holding provides banking services to individuals, small and medium-sized businesses, and larger companies.	{"Financial Services","Banks - Regional - US",Banks}	{CFFI,CWBC,UBSH}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:04.214742	2020-10-11 23:38:31.023428
HOFT	Hooker Furniture Corporation	Hooker Furniture Corp is a designer, marketer, importer of casegoods, leather and fabric-upholstered furniture for the residential, hospitality and contract markets. It also manufactures residential custom leather- and custom fabric-upholstered furniture.	{"Consumer Cyclical","Home Furnishings & Fixtures","Manufacturing - Apparel & Furniture"}	{BSET,FLXS,LZB,NTZ,NVFY,SNBR}	Manufacturing - Apparel & Furniture	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:04.233608	2020-10-11 23:38:31.023428
HDGE	Ranger Equity Bear Bear	The investment seeks capital appreciation through short sales of domestically traded equity securities.\n The Sub-Advisor seeks to achieve the fund's investment objective by short selling a portfolio of liquid mid- and large-cap U.S. exchange-traded equity securities, ETFs, ETNs and other exchange-traded products. The fund invests at least 80% of its net assets, plus any borrowings for investment purposes, in short positions in equity securities. The Sub-Advisor implements a bottom-up, fundamental, research driven security selection process.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.024836	2020-10-11 23:38:31.02423
HDSN	Hudson Technologies Inc.	Hudson Technologies Inc is an industrial products manufacturer. Its core products are refrigerants and industrial gases which are used for commercial air conditioning and processing.	{Industrials,"Diversified Industrials","Industrial Products"}	{URI,GWW,WSO,ITGR}	Industrial Products	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:04.035532	2020-10-11 23:38:31.023428
HEI	Heico Corporation	Heico Corp through its subsidiaries is engaged in manufacturing electronic equipment for the aviation, defense, space, medical, telecommunications and electronics industries throughout the United States and internationally.	{Industrials,"Aerospace & Defense"}	{TGI,ESL,GE,SIF,UTX,WAIR}	Aerospace & Defense	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:04.067201	2020-10-11 23:38:31.024353
HEWL	iShares Currency Hedged MSCI Switzerland	The investment seeks to track the investment results of the MSCI Switzerland 25/50 100% Hedged to USD Index, which consists of stocks traded primarily on the Zurich Stock Exchange with the currency risk inherent in the securities included in the underlying index hedged to the U.S. dollar on a monthly basis.\n The fund generally will invest at least 90% of its assets in the component securities (including indirect investments through the underlying fund) and other instruments of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.077719	2020-10-11 23:38:31.02423
HFC	HollyFrontier Corporation	HollyFrontier Corp is a petroleum refiner producing light products such as gasoline, diesel fuel, jet fuel, specialty lubricant products, and specialty and modified asphalt. Its products are supplied to refiners, convenience store chains and retailers.	{Energy,"Oil & Gas Refining & Marketing","Oil & Gas - Refining & Marketing"}	{}	Oil & Gas - Refining & Marketing	Energy	New York Stock Exchange	\N	2020-10-11 23:12:04.093632	2020-10-11 23:38:31.021426
HGSD	WisdomTree Global Hedged SmallCap Dividend Fund	The investment seeks to track the price and yield performance of the WisdomTree Global Hedged SmallCap Dividend Index.\n Under normal circumstances, the fund will invest at least 80% of its total assets in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a fundamentally weighted index designed to provide exposure to small-capitalization companies selected from the WisdomTree Global Dividend Index. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:04.108135	2020-10-11 23:38:31.028166
HIFS	Hingham Institution for Savings	Hingham Institution for Savings is a savings bank situated in Hingham, Massachusetts. It is involved in the business of residential and commercial real estate mortgage lending and also financial and banking services.	{"Financial Services","Savings & Cooperative Banks",Banks}	{BRKL,NWBI,BLMT,WEBK,BHLB,UBNK,EBSB,WNEB}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:04.132878	2020-10-11 23:38:31.02582
HIVE	Aerohive Networks Inc.	Aerohive Networks Inc designs and develops cloud networking and enterprise wi-fi solutions in Americas, Europe, the Middle East and Africa, and the Asia Pacific region.	{Technology,"Communication Equipment"}	{CSCO,ADTN,AAPL,JNPR,HPQ,XLK}	Communication Equipment	Technology	New York Stock Exchange	\N	2020-10-11 23:12:04.159293	2020-10-11 23:38:31.02423
HL	Hecla Mining Company	Hecla Mining Co produces lead, zinc and bulk concentrates, which it sells to custom smelters, and unrefined bullion bars containing gold and silver, which are further refined before sale to precious metals traders.	{"Basic Materials",Gold,"Metals & Mining"}	{PAAS,MAG,EXK,SVM,AUMN,CDE,AG,TAHO,WPM}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:04.177387	2020-10-11 23:38:31.02423
HMG	HMG/Courtland Properties Inc.	HMG/Courtland Properties Inc is the real estate investment trust. The company's business is the ownership and management of income-producing commercial properties.	{"Real Estate","REIT - Retail",REITs}	{VNO,JLL,CHSP,AAT}	REITs	Real Estate	NYSE American	\N	2020-10-11 23:12:04.194774	2020-10-11 23:38:31.024353
HMSY	HMS Holdings Corp	HMS Holdings Corp is a United States-based holding company. It operates in the healthcare insurance benefit cost containment marketplace. The company's primary offerings comprise coordination of benefits, payment integrity and eligibility verification.	{Industrials,"Business Services"}	{MGLN,ATHN,WCG,ESRX}	Business Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:04.213543	2020-10-11 23:38:31.02423
HOG	Harley-Davidson Inc.	Harley-Davidson Inc produces and sells heavyweight motorcycles, as well as offers motorcycle parts, accessories, and related services. It operates in two segments, Motorcycles and Related Products, and Financial Services.	{"Consumer Cyclical","Recreational Vehicles",Autos}	{HMC,PII}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:04.236217	2020-10-11 23:38:31.02582
HDLV	ETRACS Montly Pay 2xLeveraged US High Dividend Low Volatility ETN due September 30 2044	The investment seeks a return linked to the performance of the price return version of the Solactive US High Dividend Low Volatility Index (the “index”).\n The Securities provide a monthly compounded two times leveraged long exposure to the performance of the index, reduced by the Accrued Fees. The index is designed to measure the performance of 40 dividend yielding, relatively lower volatility index constituent Securities from the universe of the largest 1,000 U.S. listed stocks by market capitalization.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.025451	2020-10-11 23:38:31.025871
HEBT	Hebron Technology Co. Ltd.	Hebron Technology Co Ltd develops, manufactures and sells customized installation of valves and pipe fittings for use in pharmaceutical, biological and other clean industries.	{Industrials,"Diversified Industrials","Industrial Products"}	{HEBT}	Industrial Products	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:04.038125	2020-10-11 23:38:31.025871
HEP	Holly Energy Partners L.P.	Holly Energy Partners LP is engaged in operating a system of petroleum product and crude oil pipelines, storage tanks, distribution terminals and loading rack facilities.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{PAA,MMP,TLP,NS,SEMG,BKEP,HFC,DKL,PSXP,PAGP}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:04.064444	2020-10-11 23:38:31.02423
HEWY	iShares Currency Hedged MSCI South Korea	The investment seeks to track the investment results of the MSCI Korea 25/50 100% Hedged to USD Index.\n The fund generally will invest at least 90% of its assets in the component securities (including indirect investments through the underlying fund) and other instruments of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the index. The index consists of stocks traded primarily on the Stock Market Division of the Korea Exchange with the currency risk inherent in the securities included in the index hedged to the U.S. dollar on a monthly basis.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.088398	2020-10-11 23:38:31.028166
HFXE	IQ 50 Percent Hedged FTSE Europe	The investment seeks investment results that track (before fees and expenses) the price and yield performance of the Nasdaq Chaikin Power US Large Cap Index (the "underlying index").\n The underlying index is an equally weighted index of large-capitalization securities. The fund generally will invest in all of the securities that comprise its underlying index in proportion to their weightings in the underlying index. It invests, under normal circumstances, at least 80% of the value of its assets (net assets plus the amount of any borrowings for investment purposes) in securities of large-capitalization U.S. issuers.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.101666	2020-10-11 23:38:31.023428
HI	Hillenbrand Inc	Hillenbrand Inc is a diversified industrial company that makes and sells premium business-to-business products and services. Its segments include, the Process Equipment Group and Batesville.	{Industrials,"Diversified Industrials","Industrial Products"}	{MATW,KMB,SCI,STON,WTW}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:04.1264	2020-10-11 23:38:31.028166
HILO	Columbia EM Quality Dividend	The investment seeks investment results that correspond (before fees and expenses) to the price and yield performance of the Beta Advantage® Emerging Markets Quality Dividend Index.\n The fund will invest at least 80% of its net assets in developing market companies included in the underlying index and the advisor generally expects to be substantially invested at such times, with at least 95% of its net assets invested in these securities. The underlying index is an equal-weighted index designed to represent a portfolio of approximately 50 companies in developing markets.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.143778	2020-10-11 23:38:31.031328
HJPX	iShares Currency Hedged JPX-Nikkei 400	The investment seeks to track the investment results of the JPX-Nikkei 400 Net Total Return USD Hedged Index.\n The fund generally will invest at least 90% of its assets in the component securities (including indirect investments through the underlying fund) and other instruments of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is composed of broad-based Japanese equities while mitigating exposure to fluctuations between the value of the Japanese yen and the U.S. dollar.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.166939	2020-10-11 23:38:31.02582
HLTH	Nobilis Health Corp.	Nobilis Health Corp is a full-service healthcare development and management company, with 24 locations across Texas and Arizona, including 4 hospitals, 10 ASCs and 10 clinics. It also partners with another 35 facilities.	{Healthcare,"Medical Care","Health Care Providers"}	{XLV}	Health Care Providers	Healthcare	NYSE American	\N	2020-10-11 23:12:04.185835	2020-10-11 23:38:31.02582
HMTV	Hemisphere Media Group Inc.	Hemisphere Media Group Inc is a Spanish language media company. It is engaged in pure-play U.S. Spanish-language TV/cable network business.	{"Communication Services","Pay TV"}	{CMCSA,DISCA,TV,DIS,AMCX,VIAB,FOXA}	Communication Services	Communication Services	NASDAQ Global Market	\N	2020-10-11 23:12:04.218229	2020-10-11 23:38:31.024353
HOLI	Hollysys Automation Technologies Ltd.	Hollysys Automation Technologies Ltd is a provider of automation and control technologies and applications in China. Its services include online equipment management services, upgrade services, inspection services, and among others.	{Industrials,"Diversified Industrials","Industrial Products"}	{PRCP,ROK,EMR,ABB}	Industrial Products	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:04.23495	2020-10-11 23:38:31.028166
HCC	Warrior Met Coal Inc.	Warrior Met Coal Inc produces and exports metallurgical coal for the steel industry. The company has underground mines located in Brookwood, Alabama, southwest of Birmingham and near Tuscaloosa.	{"Basic Materials",Coal}	{}	Coal	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:03.993458	2020-10-11 23:38:31.028166
HDMV	First Trust Exchange-Traded Fund III Horizon Managed Volatility Developed International	The investment seeks to provide capital appreciation.\n Under normal market conditions, the fund seeks to achieve its investment objective by investing at least 80% of its net assets (including investment borrowings) in common stocks and depositary receipts of developed market companies listed and traded on non-U.S. exchanges that the Sub-Advisor believes exhibit low future expected volatility. Under normal market conditions, it will invest in at least three countries and at least 40% of its net assets in countries other than the United States. The fund has significant investments in companies operating in Europe. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.026222	2020-10-11 23:38:31.021426
HEAR	Turtle Beach Corporation	Turtle Beach Corp is mainly a provider of audio peripherals in the United States. Its products have application across various platforms such as gaming consoles, personal computers, and mobile phones.	{Technology,"Communication Equipment"}	{XLK}	Communication Equipment	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:04.038192	2020-10-11 23:38:31.02423
HEQ	John Hancock Hedged Equity & Income Fund of Beneficial Interest	John Hancock Hedged Equity & Income Fund is a closed-end management investment company. Its investment objective is to provide total return with a focus on current income and gains and also consisting of long-term capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.067423	2020-10-11 23:38:31.028166
HEWP	iShares Currency Hedged MSCI Spain	The investment seeks to track the investment results of the MSCI Spain 25/50 100% Hedged to USD Index (the "underlying index"), which consists of stocks traded primarily on the Madrid Stock Exchange with the currency risk inherent in the securities included in the underlying index hedged to the U.S. dollar on a monthly basis.\n The fund generally will invest at least 90% of its assets in the component securities (including indirect investments through the underlying fund) and other instruments of the index and in investments that have economic characteristics that are substantially identical to the component securities of the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.081582	2020-10-11 23:38:31.024353
HFXJ	IQ 50 Percent Hedged FTSE Japan	The investment seeks investment results that correspond (before fees and expenses) generally to the price and yield performance of its underlying index, the FTSE Japan 50% Hedged to USD Index (the "underlying index").\n The underlying index is an equity benchmark for Japan, with approximately half of the underlying index's exposure to the Japanese yen "hedged" against the U.S. dollar on a monthly basis. The fund invests, under normal circumstances, at least 80% of its net assets, plus the amount of any borrowings for investment purposes, in the securities and other instruments included in its underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.10305	2020-10-11 23:38:31.02582
HIHO	Highway Holdings Limited	Highway Holdings Ltd is a holding company. It manufactures and supplies various metal, plastic and electric parts, components and products to its original equipment manufacturing (OEM) clients.	{Industrials,"Metal Fabrication","Industrial Products"}	{}	Industrial Products	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:04.139176	2020-10-11 23:38:31.02423
HJV	MS Structured Asset Corp. SATURNS J.C. Penney Company Inc. Debenture Backed Series 2007-1 7% Callable Class A Units		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:04.16589	2020-10-11 23:38:31.023428
HLX	Helix Energy Solutions Group Inc.	Helix Energy Solutions Group Inc is an offshore energy services company that provides specialty services to the offshore energy industry, with emphasis on well intervention and robotics operations.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{SPN,HAL,EPD,OII,TTI}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:04.18855	2020-10-11 23:38:31.023428
HMNY	Helios and Matheson Analytics Inc	Helios and Matheson Analytics Inc provides IT solutions and services. Its services include application value management, application development, integration, independent validation and infrastructure services.	{Technology,"Information Technology Services","Application Software"}	{XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:04.209236	2020-10-11 23:38:31.021426
HNP	Huaneng Power Intl	Huaneng Power International Inc is an independent power producer. The company is engaged in developing, constructing, operating and managing power plants throughout China.	{Utilities,"Utilities - Diversified","Utilities - Regulated","Utilities - Independent Power Producers"}	{AES,AEP,XLU}	Utilities - Independent Power Producers	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:04.22783	2020-10-11 23:38:31.025871
HOME	At Home Group Inc.	At Home Group Inc is a home decor superstore. The company is engaged in providing assortment of products for every room, in every style at everyday low prices.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:04.24967	2020-10-11 23:38:31.021426
HPI	John Hancock Preferred Income Fund of Beneficial Interest	John Hancock Preferred Income Fund is a closed-end, diversified management investment company. Its investment objective is to provide a high level of current income consistent with preservation of capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.269329	2020-10-11 23:38:31.025871
HCSG	Healthcare Services Group Inc.	Healthcare Services Group Inc provides management, administrative and operating expertise and services to the housekeeping, laundry, linen, facility maintenance and dietary service departments of the health care industry in United States.	{Industrials,"Business Services"}	{CTAS,ABM,UNF,ARMK,TLRD}	Business Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:04.011562	2020-10-11 23:38:31.02582
HEWC	iShares Currency Hedged MSCI Canada	The investment seeks to track the investment results of the MSCI Canada 100% Hedged to USD Index, which consists of stocks traded primarily on the Toronto Stock Exchange with the currency risk inherent in the securities included in the underlying index hedged to the U.S. dollar on a monthly basis.\n The fund generally will invest at least 90% of its assets in the component securities (including indirect investments through the underlying fund) and other instruments of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.071536	2020-10-11 23:38:31.02582
HEZU	iShares Currency Hedged MSCI Eurozone	The investment seeks to track the investment results of the MSCI EMU 100% Hedged to USD Index.\n The fund generally will invest at least 90% of its assets in the component securities (including indirect investments through the underlying fund) and other instruments of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is an equity benchmark for the Economic and Monetary Union countries with the currency risk inherent in the securities included in the underlying index hedged to the U.S. dollar on a monthly basis.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.09011	2020-10-11 23:38:31.025871
HGV	Hilton Grand Vacations Inc.	Hilton Grand Vacations Inc. is a timeshare company that markets and sells vacation ownership intervals (VOI), manages resorts in leisure and urban destinations, and operates a points-based vacation club.	{"Consumer Cyclical","Resorts & Casinos","Travel & Leisure"}	{NYMT,O,NTP}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:04.116118	2020-10-11 23:38:31.02423
HIG	Hartford Financial Services Group Inc. (The)	Hartford Financial Services Group Inc provides insurance and financial services to individual and business customers in the United States.	{"Financial Services","Insurance - Diversified",Insurance}	{SIGI,ACGL,HALL,AEG,AFG,RLI,AIZ,GNW}	Insurance	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.138032	2020-10-11 23:38:31.025871
HIW	Highwoods Properties Inc.	Highwoods Properties Inc is a real estate investment trust. The firm provides leasing, management, development, construction and other customer-related services for its properties and for third parties.	{"Real Estate","REIT - Office",REITs}	{VNO,BDN,CLI,OFC,LXP,FSP}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:04.158897	2020-10-11 23:38:31.023007
HLT	Hilton Worldwide Holdings Inc.	Hilton Worldwide Holdings Inc operates hotels and resorts. The company manages a total or approximately 750,000 rooms across over 10 brands. Some of its brands include Hampton, Hilton, Home2, Curio, Canopy, and Tru.	{"Consumer Cyclical",Lodging,"Travel & Leisure"}	{CHH,WYND,H,MAR,IHG}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:04.185785	2020-10-11 23:38:31.025871
HMOP	Hartford Municipal Opportunities	The investment seeks to provide current income that is generally exempt from federal income taxes and long-term total return.\n The fund invests in investment grade and non-investment grade municipal securities (known as "junk bonds") that the sub-adviser considers to be attractive from a yield perspective while considering total return. At least 80% of the fund's net assets must be invested in municipal securities, and up to 35% of its net assets may be invested in non-investment grade municipal securities. The fund may invest in securities of any maturity or duration.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.211707	2020-10-11 23:38:31.02582
HOLD	AdvisorShares Sage Core Reserves	The investment seeks to preserve capital while maximizing income.\n Sage Advisory Services, Ltd. Co. (the "Sub-Advisor") seeks to achieve the fund's investment objective by investing in a variety of fixed income securities, including bonds, forwards and instruments issued by U.S. and foreign issuers. It will invest in U.S. dollar-denominated investment grade debt securities, including mortgage- or asset-backed securities, rated Baa- or higher by Moody's Investors Service, Inc. ("Moody's"), or equivalently rated by Standard & Poor's Ratings Services ("S&P") or Fitch, Inc. ("Fitch"), or, if unrated, determined by the Sub-Advisor to be of comparable quality.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.235404	2020-10-11 23:38:31.031328
HONE	HarborOne Bancorp Inc.	HarborOne Bancorp Inc is a bank holding company. The company through its subsidiary offers traditional deposit accounts, including checking accounts, savings accounts, money market accounts and certificate of deposit accounts.	{"Financial Services","Savings & Cooperative Banks",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:04.252217	2020-10-11 23:38:31.023428
HPJ	Highpower International Inc	Highpower International Inc is mainly involved in the production of lithium batteries, which are used for a variety of applications such as mobile devices and wireless communication products. It sells mainly in the Chinese market.	{Technology,"Electronic Components","Computer Hardware"}	{ESP,EPC,CBAK,XLK}	Computer Hardware	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:04.269264	2020-10-11 23:38:31.02582
HRI	Herc Holdings Inc.	Herc Holdings Inc is an engaged in the industrial equipment rental business. Its fleet include aerial, earthmoving, material handling, trucks and trailers and lighting. It also provides pumping solutions, power generation, climate control equipment.	{Industrials,"Rental & Leasing Services","Consulting & Outsourcing"}	{MIND,HRI}	Consulting & Outsourcing	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:04.292563	2020-10-11 23:38:31.02423
HDB	HDFC Bank Limited	HDFC Bank Ltd is a banking and financial services company. It offers retail banking, wholesale banking and other banking services such as accounts and deposits, loans, insurance, forex and among others.	{"Financial Services","Banks - Regional - Asia",Banks}	{IBN}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.021628	2020-10-11 23:38:31.031328
HE	Hawaiian Electric Industries Inc.	Hawaiian Electric Industries Inc is engaged in generating, purchasing, transmission, distribution and selling of electric energy in the State of Hawaii.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{EIX,PCG,AES,IDA,PNW,PNM,POR,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:04.036751	2020-10-11 23:38:31.031328
HES	Hess Corporation	Hess Corp is an oil and gas company involved in the exploration, development, production, transportation, purchase and sale of crude oil, natural gas liquids, and natural gas with production operations.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{TOT,XOM,APC,OXY,BP,SNP,CVX,MRO,COP,RDS.A}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:04.07089	2020-10-11 23:38:31.028166
HFGIC	Hartford Global Impact NextShares Fund	The investment seeks long-term capital appreciation.\n The fund seeks to achieve its objective by investing all of its assets in shares of the Global Impact Master Portfolio (the "Master Portfolio;" references to the "fund" include, where applicable, the Master Portfolio), which has the same investment objective and strategy as the fund. It invests in equity securities of issuers located throughout the world, including non-dollar securities and securities of emerging market issuers.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:04.097455	2020-10-11 23:38:31.02423
HGH	Hartford Financial Services Group Inc. (The) 7.875% Fixed to Floating Rate Junior Subordinated Debentures due 2042		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:04.108965	2020-10-11 23:38:31.031328
HIBB	Hibbett Sports Inc.	Hibbett Sports Inc is a United States-based operator of sporting goods retail stores in small to mid-sized markets mainly in the Southeast, Southwest, Mid-Atlantic and the Midwest.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{BGFV,RGS,FL,DKS}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:04.125055	2020-10-11 23:38:31.023428
HIMX	Himax Technologies Inc.	Himax Technologies Inc is a fabless semiconductor solution provider dedicated to display imaging processing technologies. It offers display driver ICs and timing controllers used in TVs, laptops, monitors, mobile phones, tablets, and digital cameras.	{Technology,Semiconductors}	{CY,MXIM,TXN,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:04.154289	2020-10-11 23:38:31.028166
HLG	Hailiang Education Group Inc.	Hailiang Education Group Inc is provider of private K-12 educational services in China. It operates centrally managed schools including Zhuji Hailiang Foreign Language School, Zhuji Private High School and Tianma Experimental School.	{"Consumer Defensive","Education & Training Services",Education}	{HLG}	Education	Consumer Defensive	NASDAQ Global Market	\N	2020-10-11 23:12:04.182568	2020-10-11 23:38:31.028166
HMN	Horace Mann Educators Corporation	Horace Mann Educators Corp is an insurance holding company. The company markets and underwrites personal lines of property and casualty insurance, retirement products and life insurance in the United States of America.	{"Financial Services","Insurance - Diversified",Insurance}	{ERIE,ALL,Y,PGR,MET,AIG,NGHC}	Insurance	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.201488	2020-10-11 23:38:31.028166
HOMB	Home BancShares Inc.	Home BancShares Inc is active in the financial services sector in the United States. The bank provides a broad range of commercial and retail banking, as well as related financial services. The bank's main source of net revenue is net interest income.	{"Financial Services","Banks - Regional - US",Banks}	{SBSI,BOKF,HWC,SFNC,IBKC,TCBI,CFR,PB,BXS}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:04.239558	2020-10-11 23:38:31.02423
HOPE	Hope Bancorp Inc.	Hope Bancorp Inc operates in the financial service sector. It provides banking products to small and medium-sized businesses.	{"Financial Services","Banks - Regional - US",Banks}	{CVBF,HTBK,EWBC,WABC,CVCY,PFBC}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:04.25917	2020-10-11 23:38:31.028166
HPS	John Hancock Preferred Income Fund III Preferred Income Fund III	John Hancock Preferred Income Fund III is a closed-end management investment company. Its investment objective is to provide a high level of current income consistent with preservation of capital, and to provide growth of capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.276161	2020-10-11 23:38:31.02423
HR	Healthcare Realty Trust Incorporated	Healthcare Realty Trust Inc is a healthcare facility real estate investment trust. It focuses on owning, leasing, and managing outpatient facilities and other healthcare properties. It generates all of its revenue in the United States.	{"Real Estate","REIT - Healthcare Facilities",REITs}	{ARE,LTC,NHI,MPW,HCP,DOC,TIPT,WELL}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:04.290555	2020-10-11 23:38:31.024353
HSIC	Henry Schein Inc.	Henry Schein Inc is a provider of healthcare products and services to office-based dental, animal health and medical practitioners. The company also serves dental laboratories, government and institutional health care clinics.	{Healthcare,"Medical Distribution"}	{XLV}	Medical Distribution	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:04.307706	2020-10-11 23:38:31.02582
HD	Home Depot Inc. (The)	The Home Depot Inc is a home improvement specialty retailer. Its stores sell an assortment of building materials, home improvement and lawn and garden products and provide a number of services.	{"Consumer Cyclical","Home Improvement Stores","Retail - Apparel & Specialty"}	{BLDR,LOW,LL,SHOS}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:04.014498	2020-10-11 23:38:31.024353
HDV	iShares Core High Dividend	The investment seeks to track the investment results of the Morningstar® Dividend Yield Focus IndexSM composed of relatively high dividend paying U.S. equities.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The underlying index is comprised of qualified income paying securities that are screened for superior company quality and financial health as determined by Morningstar, Inc.'s ("Morningstar" or the "index provider") proprietary index methodology. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.035582	2020-10-11 23:38:31.024353
HELE	Helen of Troy Limited	Helen Of Troy Ltd is a consumer products company. It designs, develops, imports and distributes Housewares, Healthcare / Home Environment, and Personal Care products.	{"Consumer Defensive","Household & Personal Products","Consumer Packaged Goods"}	{LCUT,IRBT,NWL,PG,PHG,REV}	Consumer Packaged Goods	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:04.06193	2020-10-11 23:38:31.031328
HFRO	Highland Floating Rate Opportunities Fund	Highland Floating Rate Opportunities Fund is a United States based non-diversified, closed-end management investment company. Its investment objective is to provide a high level of current income, consistent with preservation of capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.101651	2020-10-11 23:38:31.028166
HGSH	China HGS Real Estate Inc.	China HGS Real Estate Inc along with its subsidiaries is engaged in real estate development activities. It is primarily in the construction and sale of residential apartments, car parks, and commercial properties.	{"Real Estate","Real Estate - General","Real Estate Services"}	{STRS,JLL,XIN}	Real Estate Services	Real Estate	NASDAQ Capital Market	\N	2020-10-11 23:12:04.115305	2020-10-11 23:38:31.025871
HIL	Hill International Inc.	Hill International Inc provides professional services. It has project management services and construction claims services segments.	{Industrials,"Business Services"}	{EXPO,EEI,TTEK,WLDN,NCI,JEC,FLR,KBR,ACM,TPC}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:04.142713	2020-10-11 23:38:31.024353
HJLI	Hancock Jaffe Laboratories Inc.	Hancock Jaffe Laboratories Inc development stage medical device company. It develops and sells biological tissue solutions to treat patients with coronary, vascular, end stage renal and peripheral arterial diseases in the United States and Europe.	{Healthcare,"Medical Devices"}	{XLV}	Medical Devices	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:04.162392	2020-10-11 23:38:31.028166
HLNE	Hamilton Lane Incorporated	Hamilton Lane Inc is a holding company. The company through its holdings is a registered investment adviser providing asset management and advisory services to institutional investors, to design, build and manage private markets portfolios.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{}	Brokers & Exchanges	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:04.184941	2020-10-11 23:38:31.028166
HMST	HomeStreet Inc.	HomeStreet Inc is a diversified financial services company. It is engaged in real estate lending. Its operating segment includes Commercial and Consumer Banking Segment and Mortgage Banking Segment.	{"Financial Services","Savings & Cooperative Banks",Banks}	{UBSI,FBC,WSBF,UBSH}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:04.213089	2020-10-11 23:38:31.023007
HNRG	Hallador Energy Company	Hallador Energy Co is primarily a coal mining company with operations in Indiana, USA, and it is part of the electric power generation industry. The company also has equity interests in oil and gas exploration companies in both Indiana and Michigan.	{"Basic Materials",Coal}	{ARLP,FELP,NRP}	Coal	Basic Materials	NASDAQ Capital Market	\N	2020-10-11 23:12:04.233806	2020-10-11 23:38:31.024353
HPE	Hewlett Packard Enterprise Company	Hewlett Packard Enterprise Co is a communication equipment company that provides servers, storage, networking and technology services. Its business segments are Enterprise Group, Software, Enterprise Services, Financial Services and Corporate Investments.	{Technology,"Communication Equipment"}	{XLK}	Communication Equipment	Technology	New York Stock Exchange	\N	2020-10-11 23:12:04.264382	2020-10-11 23:38:31.023007
HQH	Tekla Healthcare Investors	Tekla Healthcare Investors is a United States based non-diversified closed-end management investment company. Its objective is to seek long-term capital appreciation by investing primarily in securities of healthcare companies.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.280661	2020-10-11 23:38:31.031328
HRTX	Heron Therapeutics Inc.	Heron Therapeutics Inc is a biotechnology company, developing products to address unmet medical needs. It is developing novel, patient-focused solutions that apply its science and technologies to already-approved pharmacological agents.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{SRDX,DRRX,NKTR,ACHN,LGND,PCRX,ALKS,INSY,XLV}	Drug Manufacturers	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:04.300481	2020-10-11 23:38:31.023007
HLIT	Harmonic Inc.	Harmonic Inc designs, manufactures and sells video infrastructure products and system solutions. It enables service providers to efficiently deliver the next generation of broadcast and on-demand services.	{Technology,"Communication Equipment"}	{XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:04.183892	2020-10-11 23:38:31.021426
HMLP	Hoegh LNG Partners LP representing Limited Partner Interests	Hoegh LNG Partners LP owns and operates floating storage and regasification units (FSRUs), under long-term charters, which it defines as charters of five or more years.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:04.197894	2020-10-11 23:38:31.031328
HMY	Harmony Gold Mining Company Limited	Harmony Gold Mining Co Ltd is a gold mining and exploration company. Its projects include Bambanani, Doornkop, Joel, Kusasalethu, Masimong, Phakisa, among others.	{"Basic Materials",Gold,"Metals & Mining"}	{IAG,GOLD,GFI,AU,DRD,SBGL}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:04.217311	2020-10-11 23:38:31.028166
HNW	Pioneer Diversified High Income Trust of Beneficial Interest	Pioneer Diversified High Income Trust is a closed-end investment fund in the United States. Its business involves investing primarily in high yield bonds to earn a high level of current income and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:04.234353	2020-10-11 23:38:31.023007
HOML	ETRACS Monthly Reset 2xLeveraged ISE Exclusively Homebuilders ETN due March 13 2045	The investment seeks a return linked to the total return version of the ISE Exclusively Homebuilders Index (the “index”).\n The ETRACS Monthly Reset 2xLeveraged ISE Exclusively Homebuilders ETN due March 13, 2045 (the “Securities”) are a series of Monthly Reset 2xLeveraged Exchange Traded Access Securities (ETRACS). The index is focused exclusively on homebuilding stocks. The Securities are designed to provide a two times leveraged long exposure to the performance of the index compounded on a monthly basis, reduced by the Accrued Fees.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.250677	2020-10-11 23:38:31.025871
HPF	John Hancock Pfd Income Fund II Pfd Income Fund II	John Hancock Preferred Income Fund II is a United States based closed-end, diversified management investment company. Its primary objective is to provide a high level of current income consistent with preservation of capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.264373	2020-10-11 23:38:31.024353
HQCL	Hanwha Q CELLS Co. Ltd.	Hanwha Q CELLS Co Ltd is a solar product's manufacturer. Its core business involves the development of solar modules and management of downstream solar farms.	{Technology,Solar,Semiconductors}	{XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:04.277449	2020-10-11 23:38:31.028166
HRTG	Heritage Insurance Holdings Inc.	Heritage Insurance Holdings Inc is a property and casualty insurance holding company. The company provides personal residential insurance for single-family homeowners and condominium owners, rental property insurance and commercial residential insurance.	{"Financial Services","Insurance - Property & Casualty"}	{CINF,ACGL,RLI}	Insurance - Property & Casualty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.296238	2020-10-11 23:38:31.021426
HSON	Hudson Global Inc.	Hudson Global Inc is a staffing firm that provides permanent recruitment, contract consulting, legal eDiscovery, recruitment process outsourcing and talent management solutions. It also provides other recruitment and talent-related solutions.	{Industrials,"Staffing & Outsourcing Services","Employment Services"}	{BBSI,HSII,KELYA,KFRC,KFY,MAN,RHI,MHH,JOB}	Employment Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:04.316288	2020-10-11 23:38:31.028166
HTBK	Heritage Commerce Corp	Heritage Commerce Corporation operates as a bank holding company. The company through its subsidiary offers commercial banking services to small and medium-sized businesses and their owners, managers and employees.	{"Financial Services","Banks - Regional - US",Banks}	{TCBK,AMRB,CWBC,CATY,CVBF,BSRR,PPBI,BOCH,BMRC}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:04.32663	2020-10-11 23:38:31.021426
HTLF	Heartland Financial USA Inc.	Heartland Financial USA Inc is a multi-bank holding company. The company through its subsidiaries provides commercial and retail banking to individuals and businesses in the communities across the United States.	{"Financial Services","Banks - Regional - US",Banks}	{UMBF,BOKF,CBSH,FCNCA,GSBC,QCRH,WFC,TCF,USB,ASB}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:04.346931	2020-10-11 23:38:31.023007
HUN	Huntsman Corporation	Huntsman Corp acts as a manufacturer of differentiated organic chemical products and of inorganic chemical products. Its products mainly comprise a broad range of chemicals and formulations which are used in various applications.	{"Basic Materials",Chemicals}	{EMN,SSL,POL,KRO,ASH,DWDP}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:04.363139	2020-10-11 23:38:31.024353
HY	Hyster-Yale Materials Handling Inc. Class A	Hyster-Yale Materials Handling Inc designs, engineers, manufactures, sells, and services a line of lift trucks, attachments, and aftermarket parts. It also manufactures components, such as frames, masts, and transmissions.	{Industrials,"Truck Manufacturing"}	{MNTX,TEX,CMI,WNC}	Truck Manufacturing	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:04.399573	2020-10-11 23:38:31.021426
HMC	Honda Motor Company Ltd.	Honda Motor Co Ltd develops, manufactures and markets motorcycles, automobiles and power products. The company produces motorcycles with engine displacement ranging from the 50cc to the 1800cc class. It produces all-terrain and side-by-side vehicles.	{"Consumer Cyclical","Auto Manufacturers",Autos}	{TM,F,TTM,GM}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:04.19475	2020-10-11 23:38:31.02423
HNNA	Hennessy Advisors Inc.	Hennessy Advisors Inc is a publicly traded investment management firm. The Company is engaged in managing, servicing and marketing open-end mutual funds, the Hennessy Funds.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:04.222445	2020-10-11 23:38:31.028166
HON	Honeywell International Inc.	Honeywell International Inc is a diversified technology and manufacturing company serving customers with aerospace products and services, energy efficient products and solutions, specialty chemicals, electronic, refining and petrochemicals.	{Industrials,"Diversified Industrials","Industrial Products"}	{}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:04.250871	2020-10-11 23:38:31.02582
HPQ	HP Inc.	HP Inc provides products, technologies, software, solutions and services to individual consumers, small- and medium-sized businesses including customers in the government, health and education sectors.	{Technology,"Computer Systems","Computer Hardware"}	{XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:04.275994	2020-10-11 23:38:31.028166
HRL	Hormel Foods Corporation	Hormel Foods Corp is engaged in manufacturing and sale of variety of food products primarily, pork and turkey and the marketing of those products throughout the United States and internationally.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:04.294727	2020-10-11 23:38:31.02582
HSII	Heidrick & Struggles International Inc.	Heidrick & Struggles International Inc is an advisory firm providing executive search, leadership consulting and culture shaping services.	{Industrials,"Staffing & Outsourcing Services","Employment Services"}	{}	Employment Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:04.310727	2020-10-11 23:38:31.02423
HTHT	Huazhu Group Limited	Huazhu Group Ltd, formerly China Lodging Group Ltd is a multi-brand hotel group in China. It provides training, reservation and support services to the franchised hotels. The hotel brands of the company include Joya Hotel, Manxin Hotel, and Starway Hotel.	{"Consumer Cyclical",Lodging,"Travel & Leisure"}	{}	Travel & Leisure	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:04.342406	2020-10-11 23:38:31.02423
HUBS	HubSpot Inc.	HubSpot Inc provides a cloud-based inbound marketing and sales platform which features integrated applications to help businesses attract visitors to websites, convert visitors into leads, close leads into customers and delight customers.	{Technology,"Software - Application","Application Software"}	{DMRC,PRO,PAYC,ZEN,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:04.356524	2020-10-11 23:38:31.031328
HVT	Haverty Furniture Companies Inc.	Haverty Furniture Companies Inc is a retailer of residential furniture and accessories. It sells home furnishings in its retail stores and via its website and offers mattresses products such as Sealy, Serta, Stearns & Foster, and Tempur-Pedic.	{"Consumer Cyclical","Home Improvement Stores","Retail - Apparel & Specialty"}	{ETH,PIR,WSM,RH}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:04.389214	2020-10-11 23:38:31.02582
HYD	VanEck Vectors High Yield Municipal Index	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the Bloomberg Barclays Municipal Custom High Yield Composite Index.\n The fund normally invests at least 80% of its total assets in securities that comprise the benchmark index. The index is comprised of publicly traded municipal bonds that cover the U.S. dollar denominated high yield long-term tax-exempt bond market.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.406029	2020-10-11 23:38:31.028166
HYI	Western Asset High Yield Defined Opportunity Fund Inc.	Western Asset High Yield Defined Opps., is a a non-diversified, limited-term, closed-end management investment company. Its investment objective is to provide high income and seek capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.423766	2020-10-11 23:38:31.02582
HYUP	Xtrackers High Beta High Yield Bond	The investment seeks investment results that correspond generally to the performance, before fees and expenses, of the Solactive USD High Yield Corporates Total Market High Beta Index (the "underlying index").\n The fund will invest at least 80% of its total assets, (but typically far more) in component securities of the underlying index. The underlying index is designed to track the performance of the segment of the U.S. dollar denominated high yield corporate bond market that exhibits higher overall beta to the broader high yield corporate fixed income market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.442106	2020-10-11 23:38:31.02582
IAF	Aberdeen Australia Equity Fund Inc	Aberdeen Australia Equity Fund Inc is a closed-end investment solutions provider. Its business lies in investing mainly in equity instruments belonging to the Australian Stock Exchange.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:04.458625	2020-10-11 23:38:31.023428
IAMXR	I-AM Capital Acquisition Company Right		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:04.469204	2020-10-11 23:38:31.028166
IPI	Intrepid Potash Inc	Intrepid Potash Inc produces and sells potash and potash byproducts. The company operates through two main product segments: Potash and Trio primarily producing selling potash and specialty fertilizer that consists of potassium, sulfate, and magnesium.	{"Basic Materials","Agricultural Inputs",Agriculture}	{SMG,CMP,CF,MOS,CINR}	Agriculture	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:04.950555	2020-10-11 23:38:31.025871
HPP	Hudson Pacific Properties Inc.	Hudson Pacific Properties Inc is a real estate investment trust engaged in owning, operating and acquiring office properties and media and entertainment properties in selected growth markets in Northern and Southern California and the Pacific Northwest.	{"Real Estate","REIT - Office",REITs}	{VNO,BDN,KRC,SLG,CLI,OFC,FSP}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:04.273749	2020-10-11 23:38:31.023428
HRC	Hill-Rom Holdings Inc	Hill-Rom Holdings Inc is engaged in selling and renting patient support and near-patient technologies and services, and health information technology solutions in the U.S and Canada.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{CAH,GE,IVC,SYK,ZBH,ELMD,STE,XLV}	Medical Instruments & Equipment	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:04.290524	2020-10-11 23:38:31.023428
HSPX	Horizons ETF Trust I S&P 500 Covered Call	The investment seeks investment results that, before fees and expenses, generally correspond to the performance of the CBOE S&P 500 2% OTM BuyWrite Index.\n The fund invests at least 80% of its total assets in securities that comprise its underlying index. The underlying index is comprised of all the equity securities in the S&P 500 Index (the "reference index") in substantially similar weight as the reference index and short (written) call options on up to 100% of the S&P 500 Index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.31857	2020-10-11 23:38:31.023428
HTBX	Heat Biologics Inc.	Heat Biologics Inc is a US-based immuno-oncology company. It is engaged in developing therapies which activate a patient's immune system against cancer utilizing an engineered form of gp96.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:04.328386	2020-10-11 23:38:31.02423
HTY	John Hancock Tax-Advantaged Global Shareholder Yield Fund of Beneficial Interest	John Hancock Tax-Advantaged Global Shareholder Yield Fund is a closed-end management investment company. Its investment objective is to provide total return consisting of a high level of current income and gains and long-term capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.350384	2020-10-11 23:38:31.028166
HUNTU	Hunter Maritime Acquisition Corp. Unit	Hunter Maritime Acquisition Corp is a blank check company. The Company was formed for the purpose of acquiring, through a merger, capital stock exchange, asset acquisition, debt acquisition, stock purchase, reorganization or other similar business.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:04.37267	2020-10-11 23:38:31.028166
HX	Hexindai Inc.	Hexindai Inc is a consumer lending marketplace based in Beijing, China. The company provides borrowers with access to credit through its online marketplace.	{"Financial Services","Credit Services"}	{HX}	Credit Services	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:04.391828	2020-10-11 23:38:31.023007
HYG	iShares iBoxx $ High Yield Corporate Bond	The investment seeks to track the investment results of the Markit iBoxx® USD Liquid High Yield Index (the "underlying index").\n The underlying index is a rules-based index consisting of liquid, U.S. dollar-denominated, high yield corporate bonds for sale in the United States. The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.414074	2020-10-11 23:38:31.02423
HYLV	IndexIQ ETF Trust IQ S&P High Yield Low Volatility Bond	The investment seeks investment results that track (before fees and expenses) the price and yield performance of its underlying index, the S&P U.S. High Yield Low Volatility Corporate Bond Index.\n The fund uses a "Representative Sampling" strategy in seeking to track the performance of the underlying index. The underlying index is comprised of U.S. dollar denominated high yield corporate bonds that have been selected in accordance with a rules-based methodology that seeks to identify securities that, in the aggregate, are expected to have lower volatility relative to the broad U.S. dollar denominated high yield corporate bond market.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.430729	2020-10-11 23:38:31.024353
I	Intelsat S.A.	Intelsat SA provides satellite communication services and ground facilities related to the satellite operations and control, and teleport services.	{"Communication Services","Telecom Services"}	{LORL,IRDM,HRS,ENT,GSAT}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:04.455652	2020-10-11 23:38:31.028166
IBD	Inspire Corporate Bond Impact	The investment seeks to replicate investment results that generally correspond to the Inspire Corporate Bond Impact Index.\n The fund generally will invest at least 80% of its total assets in the component securities of the index. The index provider selects domestic corporate bonds issued by companies that have market capitalizations of $5 billion or more and are included in the S&P 500 Investment Grade Corporate Bond Index using the Inspire Impact Score®, a proprietary selection methodology that is designed to assign a score to a particular security based on the security's alignment with biblical values and the positive impact that company has on the world.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.485136	2020-10-11 23:38:31.025871
IBKC	IBERIABANK Corporation	IBERIABANK Corp provides commercial banking as well as financial services, including retail banking, private banking, and mortgages.	{"Financial Services","Banks - Regional - US",Banks}	{BOKF,HWC,SFNC,TRMK,HOMB,BBT,STI,RF,BXS}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:04.511947	2020-10-11 23:38:31.023428
HPT	Hospitality Properties Trust	Hospitality Properties Trust is a real estate investment trust providing capital to owners and operators in hospitality industries. Its segments consist of hotel real estate investments and travel center real estate investment.	{"Real Estate","REIT - Hotel & Motel",REITs}	{}	REITs	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:04.277249	2020-10-11 23:38:31.021426
HRB	H&R Block Inc.	H&R Block Inc is a consumer tax services provider. It provides assisted income tax return preparation, digital do-it-yourself tax solutions and other services and products related to income tax return preparation to the public in the United States.	{"Consumer Cyclical","Personal Services"}	{INTU,ADP,PRGX,CBZ,WU,BCOR,MGI}	Personal Services	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:04.29042	2020-10-11 23:38:31.028166
HSC	Harsco Corporation	Harsco Corp is a provider of industrial services and engineered products. The company has three reportable segment namely Harsco Metals and Minerals, Harsco Industrial and Harsco Rail.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{DOV,ESL,GE,TRN}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:04.306502	2020-10-11 23:38:31.023428
HSY	The Hershey Company	The Hershey Co operates as a confectionery manufacturer in the United States engaged in the chocolate industry. The firm has more than 80 brands, including Hershey's, Reese's, Kit Kat, Twizzlers, and Ice Breakers with a business presence across the world.	{"Consumer Defensive",Confectioners,"Consumer Packaged Goods"}	{JBSS,HAIN,GIS,MDLZ,RMCF}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:04.321373	2020-10-11 23:38:31.024353
HTH	Hilltop Holdings Inc.	Hilltop Holdings Inc is a diversified financial holding company. It provides business and consumer banking services from offices located throughout Texas. It also provides broker-dealer, mortgage origination, and insurance segments.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.342273	2020-10-11 23:38:31.02582
HUBG	Hub Group Inc.	Hub Group Inc is an asset-light freight transportation management company, which offers intermodal, truck brokerage and logistics services in the United States, Canada, and Mexico.	{Industrials,"Integrated Shipping & Logistics","Transportation & Logistics"}	{EXPD,JBHT,ODFL,CHRW,R,RRTS,RLGT}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:04.355112	2020-10-11 23:38:31.021426
HUSA	Houston American Energy Corporation	Houston American Energy Corp is an independent oil and gas company. The Company is engaged in the development, exploration, exploitation, acquisition and production of natural gas and crude oil properties in the U.S. Gulf Coast region and South America.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{NFX,DNR,APC,APA,OXY,EOG}	Oil & Gas - E&P	Energy	NYSE American	\N	2020-10-11 23:12:04.381415	2020-10-11 23:38:31.031328
HYDB	iShares Edge High Yield Defensive Bond	The investment seeks to track the investment results of the BlackRock High Yield Defensive Bond Index.\n The fund will invest at least 80% of its assets in the component securities of the underlying index and may invest up to 20% of its assets in certain index futures, options, options on index futures, swap contracts or other derivatives. The index consists of U.S. dollar-denominated, high yield corporate bonds. Component securities include publicly-issued debt of U.S. corporations and U.S. dollar-denominated, publicly issued debt of non-U.S. corporations or similar entities.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:04.405375	2020-10-11 23:38:31.028166
HYHG	ProShares High Yield Interest Rate Hedged	The investment seeks investment results, before fees and expenses, that track the performance of the FTSE High Yield (Treasury Rate-Hedged) Index (the "index").\n Under normal circumstances, the fund will invest at least 80% of its total assets in high-yield bonds included in the index. The index is comprised of (a) long positions in U.S. dollar-denominated high yield corporate bonds ("high yield bonds") and (b) short positions in U.S. Treasury notes or bonds ("Treasury Securities") of, in aggregate, approximate equivalent duration to the high yield bonds.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:04.423586	2020-10-11 23:38:31.031328
HYT	Blackrock Corporate High Yield Fund Inc.	Blackrock Corp High Yield Fund VI Inc is a United States based closed-end management investment company. Its investment objective is to provide shareholders with current income and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.439778	2020-10-11 23:38:31.031328
IAE	Voya Asia Pacific High Dividend Equity Income Fund ING Asia Pacific High Dividend Equity Income Fund of Beneficial Interest	Voya Asia Pacific High Dividend Equity Income Fund is a United States based diversified, closed-end fund. The fund's investment objective is total return through a combination of current income, capital gains and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.457527	2020-10-11 23:38:31.028166
IAT	iShares U.S. Regional Banks	The investment seeks to track the investment results of the Dow Jones U.S. Select Regional Banks Index.\n The fund generally invests at least 90% of its assets in securities of the index and in depositary receipts representing securities of the index. The underlying index measures the performance of the regional bank sector of the U.S. equity market and is a subset of the Dow Jones U.S. Bank Index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.470737	2020-10-11 23:38:31.024353
HRS	Harris Corporation	Harris Corp develops, manufactures and sells technology-based solutions that serve government and commercial customers' mission-critical challenges. Its products include communications systems, serving defense and public safety networks amongst other.	{Technology,"Communication Equipment"}	{VSAT,CMTL,GD,NOC,RTN,COL,MSI,ITT,LLL,XLK}	Communication Equipment	Technology	New York Stock Exchange	\N	2020-10-11 23:12:04.293737	2020-10-11 23:38:31.028166
HSKA	Heska Corporation	Heska Corp is engaged in developing, manufacturing, marketing, selling and supporting veterinary products. The Company's core focus is on the canine and feline companion animal health markets.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{IDXX,ICCC,NVS,LLY,PFE,MRK,ZTS,XLV}	Medical Diagnostics & Research	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:04.31471	2020-10-11 23:38:31.021426
HTD	John Hancock Tax Advantaged Dividend Income Fund of Beneficial Interest	John Hancock Tax-advantaged Div Inc Fd is a United States based closed-end, diversified management investment company. Its investment objective is to provide a high level of after-tax total return from dividend income and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.329686	2020-10-11 23:38:31.02582
HTLD	Heartland Express Inc.	Heartland Express Inc is a short-to-medium-haul truckload carrier of general commodities. The company provides transportation service to shippers, using late-model equipment and a combined fleet of company-owned and owner-operator tractors.	{Industrials,Trucking,"Transportation & Logistics"}	{}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:04.346647	2020-10-11 23:38:31.024353
HUM	Humana Inc.	Humana Inc is a health care company that offers insurance products and health and wellness services. Its segments include Retail, Employer Group and Healthcare Services.	{Healthcare,"Health Care Plans"}	{CI,UNH,AET,CNC,MOH,ANTM,XLV}	Health Care Plans	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:04.363361	2020-10-11 23:38:31.028166
HUSE	Strategy Shares US Market Rotation	The investment seeks capital appreciation.\n Under normal circumstances, at least 80% of the fund's net assets, plus any borrowings for investment purposes, will be invested in securities of U.S. companies and/or the U.S. government, or in other investment companies that principally invest in such securities. It will invest in companies within each of the large-cap, mid-cap and small-cap U.S. equity segments (each a "Market Segment").	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.380931	2020-10-11 23:38:31.028166
HYAC	Haymaker Acquisition Corp.	Haymaker Acquisition Corp is a blank check company. It is formed for the purpose of effecting a merger, capital stock exchange, asset acquisition, stock purchase, reorganization, or similar business combination with one or more businesses.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:04.400588	2020-10-11 23:38:31.031328
HYDW	Xtrackers Low Beta High Yield Bond	The investment seeks investment results that correspond generally to the performance, before fees and expenses, of the Solactive USD High Yield Corporates Total Market Low Beta Index (the "underlying index").\n The fund will invest at least 80% of its total assets, (but typically far more) in component securities of the underlying index. The underlying index is designed to track the performance of the segment of the U.S. dollar denominated high yield corporate bond market that exhibits lower overall beta to the broader high yield corporate fixed income market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.4101	2020-10-11 23:38:31.021426
HYLS	First Trust High Yield Long/Short ETF	The investment seeks current income; capital appreciation is the secondary objective.\n Under normal market conditions, the fund invests at least 80% of its net assets (including investment borrowings) in high yield debt securities that are rated below investment grade at the time of purchase or unrated securities deemed by the fund's advisor to be of comparable quality.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:04.429146	2020-10-11 23:38:31.02423
HZNP	Horizon Pharma plc	Horizon Pharma PLC is a pharmaceutical company. The company focuses on researching, developing and commercializing medicines that address unmet treatment needs for rare and rheumatic diseases.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{AMGN,BSTC,AZN,PFE,RDY,MRK,ALDR,RDUS,XLV}	Drug Manufacturers	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:04.450625	2020-10-11 23:38:31.024353
IBA	Industrias Bachoco S.A.B. de C.V.	Industrias Bachoco SAB de CV operates as a poultry producer in Mexico. It is engaged in breeding, processing and marketing of chicken, eggs, swine, and other products.	{"Consumer Defensive","Farm Products","Consumer Packaged Goods"}	{CALM,SAFM,FDP,TSN,SEB,PPC}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:04.477062	2020-10-11 23:38:31.023007
IBDK	iShares iBonds Dec 2019 Term Corporate	The investment seeks to track the investment results of the Bloomberg Barclays December 2019 Maturity Corporate Index which composed of U.S. dollar-denominated, investment-grade corporate bonds maturing in 2019.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index. The underlying index includes U.S. dollar-denominated, investment-grade securities publicly issued by U.S. and non-U.S. corporate issuers that have $300 million or more of outstanding face value at the time of inclusion.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.497142	2020-10-11 23:38:31.028166
IBKCO	IBERIABANK Corporation Depositary Shares Representing Series C Fixed to Floating	IBERIABANK Corp provides commercial banking as well as financial services, including retail banking, private banking, and mortgages.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:04.516214	2020-10-11 23:38:31.028166
IBOC	International Bancshares Corporation	International Bancshares Corp is a financial holding company. It mainly provides retail and commercial banking to small and medium-size businesses. The company generates net revenue primarily through net interest income.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:04.53709	2020-10-11 23:38:31.028166
HSGX	Histogenics Corporation	Histogenics Corp is a regenerative medicine company engaged in developing and commercializing products in the musculoskeletal segment of the marketplace.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:04.30725	2020-10-11 23:38:31.024353
HSTM	HealthStream Inc.	HealthStream Inc provides software-as-a-service-based workforce development solutions for healthcare organizations. Its segment includes Healthstream workforce solutions, Healthstream patient experience solutions, and Healthstream provider solutions.	{Technology,"Health Information Services","Application Software"}	{LPNT,CYH,THC,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:04.322499	2020-10-11 23:38:31.028166
HTGM	HTG Molecular Diagnostics Inc.	HTG Molecular Diagnostics Inc is a commercial stage company. It develops & markets a novel technology platform to facilitate the routine use of complex molecular profiling with its focus on clinical applications.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{EXEL,RDUS,AFMD,CALA,XLV}	Medical Diagnostics & Research	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:04.336493	2020-10-11 23:38:31.021426
HUNT	Hunter Maritime Acquisition Corp.	Hunter Maritime Acquisition Corp is a blank check company. The Company was formed for the purpose of acquiring, through a merger, capital stock exchange, asset acquisition, debt acquisition, stock purchase, reorganization or other similar business.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:04.363637	2020-10-11 23:38:31.023007
HWCC	Houston Wire & Cable Company	Houston Wire & Cable Co is a provider of electrical and mechanical wire and cable, hardware, industrial fasteners and related services to the U.S. market.	{Industrials,"Industrial Distribution"}	{}	Industrial Distribution	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:04.38937	2020-10-11 23:38:31.02423
HYEM	VanEck Vectors Emerging Markets High Yield Bond	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the ICE BofAML Diversified High Yield US Emerging Markets Corporate Plus Index.\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The index is comprised of U.S. dollar denominated bonds issued by non-sovereign emerging market issuers that have a below investment grade rating and that are issued in the major domestic and Eurobond markets.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.413463	2020-10-11 23:38:31.024353
HYIH	Xtrackers High Yield Corporate Bond - Interest Rate Hedged	The investment seeks investment results that correspond generally to the performance, before fees and expenses, of the Solactive High Yield Corporate Bond - Interest Rate Hedged Index.\n The fund will invest at least 80% of its total assets, but typically far more, in instruments that comprise the underlying index. The index is comprised of (a) long positions in U.S. dollar-denominated high yield corporate bonds and (b) short positions in U.S. Treasury notes or bonds ("Treasury Securities") of, in aggregate, approximate equivalent duration to the high yield bonds.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:04.427218	2020-10-11 23:38:31.023007
HYZD	WisdomTree BofA Merrill Lynch High Yield Bond Zero	The investment seeks to track the price and yield performance of the BofA Merrill Lynch 0-5 Year U.S. High Yield Constrained, Zero Duration Index.\n The index is designed to provide long exposure to the BofA Merrill Lynch 0-5 Year U.S. High Yield Constrained Index while seeking to manage interest rate risk through the use of short positions in U.S. Treasury securities. Under normal circumstances, at least 80% of the fund's total assets will be invested in the component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:04.446407	2020-10-11 23:38:31.025871
IAI	iShares U.S. Broker-Dealers & Securities Exchanges	The investment seeks to track the investment results of the Dow Jones U.S. Select Investment Services Index composed of U.S. equities in the investment services sector.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index measures the performance of the investment services sector of the U.S. equity market. The fund may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.463089	2020-10-11 23:38:31.025871
IBB	iShares Nasdaq Biotechnology Index Fund	The investment seeks to track the investment results of the NASDAQ Biotechnology Index, which contains securities of companies listed on NASDAQ that are classified according to the Industry Classification Benchmark as either biotechnology or pharmaceuticals and that also meet other eligibility criteria determined by Nasdaq, Inc.\n The fund generally invests at least 90% of its assets in securities of the index and in depositary receipts representing securities of the index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:04.480406	2020-10-11 23:38:31.02423
IBDM	iShares iBonds Dec 2021 Term Corporate	The investment seeks to track the investment results of the Bloomberg Barclays December 2021 Maturity Corporate Index which composed of U.S. dollar-denominated, investment-grade corporate bonds maturing in 2021.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index. The underlying index is composed of U.S. dollar-denominated, taxable, investment-grade corporate bonds scheduled to mature after December 31, 2020 and before December 16, 2021. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.499212	2020-10-11 23:38:31.023007
HST	Host Hotels & Resorts Inc.	Host Hotels & Resorts Inc is a lodging real estate investment trust. It owns properties and conducts operations. Its properties include luxury and upper-upscale hotels.	{"Real Estate","REIT - Hotel & Motel",REITs}	{AHT,DRH,PEB,CHSP,RLJ,INN,SOHO,HT,HPT}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:04.319431	2020-10-11 23:38:31.023007
HUD	Hudson Ltd. Class A	Hudson Ltd is a travel retailer in North America. The company operates Hudson News, Hudson Booksellers, cafes, specialty retail and duty-free shops in airports and transportation terminals in the United States and Canada.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:04.355737	2020-10-11 23:38:31.02423
HVBC	HV Bancorp Inc.	HV Bancorp Inc is a holding company of Huntingdon Valley Bank. The Company through its holding operates as a community-oriented bank offering financial products and services to meet the needs of its customers.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:04.386559	2020-10-11 23:38:31.025871
HYB	New America High Income Fund Inc. (The)	New America High Income Fund Inc is a diversified closed end management investment company. Its investment objective is to provide high current income while seeking to preserve stockholders' capital by investing in high-yield fixed-income securities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.405125	2020-10-11 23:38:31.02582
IAC	IAC/InterActiveCorp	IAC/InterActiveCorp is an Internet media company engaged in a variety of businesses. It's business segments include Match Group, HomeAdvisor, video, applications, publishing, and other.	{Technology,"Internet Content & Information","Online Media"}	{MSFT,BIDU,DIS,CBS,BCOR,FOXA,GOOGL,LEAF,XLK}	Online Media	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:04.457332	2020-10-11 23:38:31.023007
IBCE	iShares iBonds Mar 2023 Term Corporate ex-Financials	The investment seeks to track the investment results of the Bloomberg Barclays 2023 Maturity High Quality Corporate Index which is composed of U.S. dollar-denominated, investment-grade corporate bonds, excluding financials, maturing after March 31, 2022 and before April 1, 2023.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index, and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index, but which BFA believes will help the fund track the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.483197	2020-10-11 23:38:31.021426
IBDO	iShares iBonds Dec 2023 Term Corporate	The investment seeks to track the investment results of the Bloomberg Barclays December 2023 Maturity Corporate Index which composed of U.S. dollar-denominated, investment-grade corporate bonds maturing in 2023.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index. The underlying index is composed of U.S. dollar-denominated, taxable, investment-grade corporate bonds scheduled to mature after December 31, 2022 and before December 16, 2023.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.503754	2020-10-11 23:38:31.025871
IBMH	iShares iBonds Sep 2019 Term Muni Bond	The investment seeks to track the investment results of the S&P AMT-Free Municipal Series 2019 IndexTM.\n The fund generally invests at least 90% of its assets in the component securities of the underlying index, and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index. The underlying index measures the performance of investment-grade, non-callable U.S. municipal bonds maturing in 2019.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.524925	2020-10-11 23:38:31.025871
ICAD	icad inc.	icad Inc is a provider of image analysis, work flow solutions and radiation therapy for early identification and treatment of cancer. The company has cancer detection and cancer therapy segments.	{Technology,"Software - Application","Application Software"}	{XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:04.544937	2020-10-11 23:38:31.031328
ICF	iShares Cohen & Steers REIT	The investment seeks to track the investment results of the Cohen & Steers Realty Majors Index composed of U.S. real estate investment trusts ("REITs").\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The objective of the underlying index is to represent relatively large and liquid REITs that may benefit from future consolidation and securitization of the U.S. real estate industry. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:04.565657	2020-10-11 23:38:31.031328
KFY	Korn/Ferry International	Korn/Ferry International is a management recruitment firm. It provides talent management solutions that help clients to attract, develop, retain and sustain their talent.	{Industrials,"Staffing & Outsourcing Services","Employment Services"}	{BBSI,HSII,KELYA,MAN,NSP,HSON,JOB}	Employment Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:05.741499	2020-10-11 23:38:31.023428
HTGC	Hercules Capital Inc.	Hercules Capital Inc is a specialty finance company engaged in providing senior secured loans to high-growth, venture capital-backed companies in a variety of technology, life sciences and sustainable and renewable technology industries.	{"Financial Services","Specialty Finance",Banks}	{}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.333324	2020-10-11 23:38:31.023428
HURN	Huron Consulting Group Inc.	Huron Consulting Group Inc is a provider of operational and financial consulting services. The company helps clients in various industries to improve performance, transform the enterprise, reduce costs, leverage technology, recover from distress.	{Industrials,"Business Services"}	{RECN,CRAI,ICFI,III,HCKT,MMS,NCI,FCN}	Business Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:04.373645	2020-10-11 23:38:31.021426
HXL	Hexcel Corporation	Hexcel Corp is a composites company. It develops, manufactures and markets composites, including carbon fibers, reinforcements, honeycomb, adhesives and composite structures, for use in Commercial Aerospace, Space and Defense, and Industrial markets.	{Industrials,"Aerospace & Defense"}	{TGI,CVU,BA,SPR}	Aerospace & Defense	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:04.39237	2020-10-11 23:38:31.028166
HYDD	Direxion Daily High Yield Bear 2X Shares	The investment seeks daily investment results, before fees and expenses, of 200% of the inverse of the daily performance of the Bloomberg Barclays US High Yield Very Liquid Index.\n The fund invests in swap agreements, futures contracts, short positions or other financial instruments that, in combination, provide inverse or short leveraged exposure to the index equal to at least 80% of the fund's net assets (plus borrowing for investment purposes). The index measures the performance of publicly issued U.S. Dollar denominated, non-investment grade or high-yield, fixed-rate, taxable corporate bonds, also known as "junk bonds." The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.409294	2020-10-11 23:38:31.023007
HYLB	Xtrackers USD High Yield Corporate Bond	The investment seeks investment results that correspond generally to the performance, before fees and expenses, of the Solactive USD High Yield Corporates Total Market Index (the "underlying index").\n The fund will normally invest at least 80% of its net assets, plus the amount of any borrowings for investment purposes, in high yield corporate bonds. The index comprised of U.S. dollar-denominated high yield corporate bonds. It will concentrate its investments (i.e., hold 25% or more of its total assets) in a particular industry or group of industries to the extent that its underlying index is concentrated. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.427664	2020-10-11 23:38:31.028166
HZO	MarineMax Inc. (FL)	MarineMax Inc is a recreational boat and yacht dealer in the United States. The company offers new and used recreational boats and related marine products, including engines, trailers, parts, and accessories.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{CPRT,PRTS,AZO,AAP,MPX,ORLY}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:04.455661	2020-10-11 23:38:31.02423
IART	Integra LifeSciences Holdings Corporation	Integra Lifesciences Holdings Corp is engaged in development, manufacturing, and marketing of product lines from its technology for applications ranging from burn and deep tissue wounds to repair of dura mater in the brain to repair of nerve and tendon.	{Healthcare,"Medical Devices"}	{CNMD,RTIX,CAH,JNJ,SYK,TFX,ZBH,MDT,XRAY,XLV}	Medical Devices	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:04.470943	2020-10-11 23:38:31.023428
IBDH	iShares iBonds Dec 2018 Term Corporate	The investment seeks to track the investment results of the Bloomberg Barclays December 2018 Maturity Corporate Index, which is composed of U.S. dollar-denominated, investment-grade corporate bonds maturing in 2018.\n The fund generally will invest at least 90% of its assets in the component securities (including indirect investments through the underlying fund) of the underlying index. The underlying index includes U.S. dollar-denominated, investment-grade securities publicly issued by U.S. and non-U.S. corporate issuers that have $300 million or more of outstanding face value at the time of inclusion.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.491125	2020-10-11 23:38:31.031328
IBDR	iShares iBonds Dec 2026 Term Corporate	The investment seeks to track the investment results of the Bloomberg Barclays December 2026 Maturity Corporate Index composed of U.S. dollar-denominated, investment-grade corporate bonds maturing in 2026.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index, and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The index includes U.S. dollar-denominated, investment-grade securities publicly issued by U.S. and non-U.S. corporate issuers that have $300 million or more of outstanding face value at the time of inclusion. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.509098	2020-10-11 23:38:31.028166
IBMI	iShares iBonds Sep 2020 Term Muni Bond	The investment seeks to track the investment results of the S&P AMT-Free Municipal Series 2020 IndexTM.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index, and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index, but which the advisor believes will help the fund track the underlying index. The underlying index measures the performance of investment-grade, non-callable U.S. municipal bonds maturing in 2020.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.524557	2020-10-11 23:38:31.031328
HTGX	Hercules Capital Inc. 6.25% Notes due 2024		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:04.341276	2020-10-11 23:38:31.031328
HUBB	Hubbell Inc	Hubbell Inc is engaged in designing, manufacturing and sale of electrical and electronic products for non-residential and residential construction, industrial and utility applications. The company's business segments are Electrical and Power.	{Technology,"Electronic Components","Computer Hardware"}	{HON,AZZ,AYI,TEL,AMSC,RVLT,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:04.354719	2020-10-11 23:38:31.02582
HWBK	Hawthorn Bancshares Inc.	Hawthorn Bancshares Inc is a bank holding company. The company through its subsidiary, Union State Bancshares, Inc. is engaged in providing financial products and services.	{"Financial Services","Banks - Regional - US",Banks}	{UMBF,ATLO,WTBA,GFED,CBSH,GSBC,LARK,QCRH,EFSC,MOFG}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:04.38915	2020-10-11 23:38:31.023428
HYACW	Haymaker Acquisition Corp. Warrant		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:04.409198	2020-10-11 23:38:31.023428
HYMB	SPDR Nuveen S&P High Yield Municipal Bond	The investment seeks to provide investment results that correspond generally to the price and yield performance of the S&P Municipal Yield Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index or in securities that the Sub-Adviser determines have economic characteristics that are substantially identical to the economic characteristics of the securities that comprise the index. The index tracks the U.S. high yield municipal bond market and to provide income that is exempt from federal income taxes. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.430737	2020-10-11 23:38:31.021426
HYXU	iShares International High Yield Bond	The investment seeks to track the investment results of the Markit iBoxx Global Developed Markets ex-US High Yield Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents, including shares of money market funds advised by BFA or its affiliates. The index is a rules-based index consisting of high yield corporate bonds denominated in euros, British pounds sterling and Canadian dollars.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:04.446368	2020-10-11 23:38:31.023428
IAGG	iShares International Aggregate Bond Fund	The investment seeks to track the investment results of the Bloomberg Barclays Global Aggregate ex USD 10% Issuer Capped (Hedged) Index composed of global non-U.S. dollar-denominated investment-grade bonds that mitigates exposure to fluctuations between the value of the component currencies and the U.S. dollar.\n The fund generally will invest at least 90% of its assets in the component securities and other instruments of the underlying index. The index measures the performance of the global investment-grade bond market. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:04.462399	2020-10-11 23:38:31.02582
IBCD	iShares iBonds Mar 2020 Term Corporate ex-Financials Term	The investment seeks to track the investment results of the Bloomberg Barclays 2020 Maturity High Quality Corporate Index which is composed of U.S. dollar-denominated, investment-grade corporate bonds, excluding financials, maturing after March 31, 2019 and before April 1, 2020.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index, and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.480936	2020-10-11 23:38:31.028166
IBDN	iShares iBonds Dec 2022 Term Corporate	The investment seeks to track the investment results of the Bloomberg Barclays December 2022 Maturity Corporate Index which composed of U.S. dollar-denominated, investment-grade corporate bonds maturing in 2022.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index. The underlying index is composed of U.S. dollar-denominated, taxable, investment-grade corporate bonds scheduled to mature after December 31, 2021 and before December 16, 2022.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.50285	2020-10-11 23:38:31.024353
IBM	International Business Machines Corporation	International Business Machines Corp offers a variety of IT services along with software, and hardware. It has operations in over 170 countries and provides planning, build, manage, and maintain IT infrastructure, platforms, applications, and services.	{Technology,"Information Technology Services","Application Software"}	{CSCO,INTC,MSFT,AAPL,SAP,VMW,ACN,ORCL,HPQ,GOOGL,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:04.52407	2020-10-11 23:38:31.021426
IBTX	Independent Bank Group Inc	Independent Bank Group Inc is a bank holding company. It provides relationship-driven commercial banking products and services to businesses, professionals, and individuals.	{"Financial Services","Banks - Regional - US",Banks}	{FFIN,IBOC,SBSI,IBKC,TCBI,CFR,PB,LTXB}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:04.544213	2020-10-11 23:38:31.025871
ICLK	iClick Interactive Asia Group Limited	iClick Interactive Asia Group Ltd offers independent online marketing technology platform in China. It serves as an integrated cross-channel gateway that provides marketers with innovative & cost-effective ways to optimize their online marketing efforts.	{"Consumer Cyclical","Marketing Services","Advertising & Marketing Services"}	{}	Advertising & Marketing Services	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:04.570772	2020-10-11 23:38:31.025871
ICUI	ICU Medical Inc.	ICU Medical Inc is engaged in the development, manufacture and sale of medical devices used in infusion therapy, oncology and critical care applications. Its products include needlefree connectors, nanoclave, chemoclock, chemoclave, and among others.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{MMSI,ANGO,PODD,BAX,BDX,TFX,EW,XLV}	Medical Instruments & Equipment	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:04.583567	2020-10-11 23:38:31.028166
HUNTW	Hunter Maritime Acquisition Corp. Warrant		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:04.37246	2020-10-11 23:38:31.025871
HWKN	Hawkins Inc.	Hawkins Inc distributes bulk chemicals and blends, manufactures and distributes specialty chemicals for its customers in various industries. The company conduct its business in three segments, Industrial, Water Treatment and Health and Nutrition.	{"Basic Materials","Specialty Chemicals",Chemicals}	{}	Chemicals	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:12:04.388942	2020-10-11 23:38:31.024353
HYACU	Haymaker Acquisition Corp. Unit	Haymaker Acquisition Corp is a blank check company. It is formed for the purpose of effecting a merger, capital stock exchange, asset acquisition, stock purchase, reorganization, or similar business combination with one or more businesses.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:04.404048	2020-10-11 23:38:31.025871
HYLD	High Yield	The investment seeks high current income with a secondary goal of capital appreciation.\n The Sub-Advisor seeks to achieve the fund's investment objective by selecting a focused portfolio of high-yield debt securities, which include senior and subordinated corporate debt obligations (such as loans, bonds, debentures, notes and commercial paper). The fund does not have any portfolio maturity limitation and may invest its assets in instruments with short-term, medium-term or long-term maturities. It invests at least 80% of its net assets (plus any borrowings for investment purposes) in high-yield debt securities.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.428527	2020-10-11 23:38:31.023428
HZN	Horizon Global Corporation	Horizon Global Corp designs, manufactures and distributes towing, trailering, cargo management and related products for original equipment to aftermarket, automotive, retail and OE markets.	{"Consumer Cyclical","Auto Parts",Autos}	{MPAA,LKQ,CTB}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:04.447707	2020-10-11 23:38:31.021426
IAK	iShares U.S. Insurance	The investment seeks to track the investment results of the Dow Jones U.S. Select Insurance Index composed of U.S. equities in the insurance sector.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index measures the performance of the insurance sector of the U.S. equity market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.464518	2020-10-11 23:38:31.021426
IBDL	iShares iBonds Dec 2020 Term Corporate	The investment seeks to track the investment results of the Bloomberg Barclays December 2020 Maturity Corporate Index (the "underlying index").\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index. The underlying index is composed of U.S. dollar-denominated, taxable, investment-grade corporate bonds scheduled to mature after December 31, 2019 and before December 16, 2020.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.498399	2020-10-11 23:38:31.028166
IBIO	iBio Inc.	iBio Inc is a biotechnology company. It is engaged in commercializing its proprietary platform technologies, iBioLaunch for vaccines and therapeutic proteins & iBioModulator for vaccine enhancement and developing select product candidates.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NYSE American	\N	2020-10-11 23:12:04.511671	2020-10-11 23:38:31.031328
IBML	iShares iBonds Dec 2023 Term Muni Bond	The investment seeks to track the investment results of the S&P AMT-Free Municipal Series Dec 2023 IndexTM composed of investment-grade U.S. municipal bonds maturing after December 31, 2022 and before December 2, 2023.\n The fund generally will invest at least 90% of its assets in the component securities of the index, and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The index includes municipal bonds primarily from issuers that are state or local governments or agencies such that the interest on the bonds is exempt from U.S. federal income taxes. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:04.535589	2020-10-11 23:38:31.028166
ICHR	Ichor Holdings	Ichor Holdings Ltd is engaged in the design, engineering and manufacturing of critical fluid delivery subsystems for semiconductor capital equipment.	{Technology,"Semiconductor Equipment & Materials",Semiconductors}	{QRVO,QTNA,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:04.56702	2020-10-11 23:38:31.023428
ICVT	iShares Convertible Bond	The investment seeks to track the investment results of the Bloomberg Barclays U.S. Convertible Cash Pay Bond > $250MM Index (the "underlying index").\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index. The underlying index is a subset of the Bloomberg Barclays U.S. Convertibles: Cash Pay Bonds Index, which measures the performance of the U.S. dollar-denominated convertibles market.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:04.583783	2020-10-11 23:38:31.023428
IDN	Intellicheck Inc.	Intellicheck Inc is a technology company. It is engaged in developing, integrating and marketing identity authentication systems for applications including mobile & handheld access control and security systems for the government & commercial markets.	{Technology,"Software - Application","Application Software"}	{XLK}	Application Software	Technology	NYSE American	\N	2020-10-11 23:12:04.600222	2020-10-11 23:38:31.025871
IAG	Iamgold Corporation	Iamgold Corp is a midtier gold miner based in Toronto. It explores, develops and operates gold mining properties.	{"Basic Materials",Gold,"Metals & Mining"}	{NSU,TGB,TRX,GOLD,AGI}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:04.46273	2020-10-11 23:38:31.031328
IBCP	Independent Bank Corporation	Independent Bank Corp (Ionia MI) is a bank holding company. It offers products & services, including checking and savings accounts, commercial lending, direct and indirect consumer financing, mortgage lending and safe deposit box services.	{"Financial Services","Banks - Regional - US",Banks}	{MBWM,MCBC,CHFC,SRCE,HBAN,MBTF,MFNC}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:04.481848	2020-10-11 23:38:31.02582
IBDQ	iShares iBonds Dec 2025 Term Corporate	The investment seeks to track the investment results of the Bloomberg Barclays December 2025 Maturity Corporate Index which composed of U.S. dollar-denominated, investment-grade corporate bonds maturing in 2025.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index. The underlying index is composed of U.S. dollar-denominated, taxable, investment-grade corporate bonds scheduled to mature after December 31, 2024 and before December 16, 2025.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.507662	2020-10-11 23:38:31.023007
IBMK	iShares iBonds Dec 2022 Term Muni Bond	The investment seeks to track the investment results of the S&P AMT-Free Municipal Series Dec 2022 IndexTM, which measures the performance of investment-grade, non-callable U.S. municipal bonds maturing in 2022.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index, and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents, including shares of money market funds advised by BFA or its affiliates, as well as in securities not included in the underlying index, but which BFA believes will help the fund track the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.527585	2020-10-11 23:38:31.023428
ICAN	SerenityShares Impact	The investment seeks to track the total return performance, before fees and expenses, of the SSI Impact Index.\n Under normal circumstances, at least 80% of the fund's total assets (exclusive of any collateral held from securities lending) will be invested in the component securities of the index. The index is constructed using a rules-based methodology to create a diversified, investable, modified market capitalization weighted portfolio of U.S.-listed stocks or American Depositary Receipts ("ADRs") of companies that generate revenues from products or services that address a variety of societal, social, and environmental challenges.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.546875	2020-10-11 23:38:31.02582
ICL	Israel Chemicals Limited	Israel Chemicals Ltd is a specialty minerals company. The company develops, manufactures, markets and sells fertilizers. Its business segments are the Essential Minerals Segment and the Specialty Solutions Segment.	{"Basic Materials","Agricultural Inputs",Agriculture}	{SQM,ALB,GRA,CBT,CF,MOS}	Agriculture	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:04.565582	2020-10-11 23:38:31.021426
IDCC	InterDigital Inc.	InterDigital Inc functions within the communication segment. It develops technologies that enable and enhance wireless communications.	{"Communication Services","Telecom Services"}	{INTC,AAPL,XLNX,QUIK,ERIC,OCLR,SNE,BB}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:12:04.590842	2020-10-11 23:38:31.023007
IDRA	Idera Pharmaceuticals Inc.	Idera Pharmaceuticals Inc is a clinical stage biotechnology company. It is engaged in the discovery, development and commercialization of novel therapeutics for oncology and rare diseases.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:04.605309	2020-10-11 23:38:31.02423
IEC	IEC Electronics Corp.	IEC Electronics Corp provides electronic contract manufacturing services to companies that produce life-saving and mission critical products for the medical, industrial, aerospace and defense sectors.	{Technology,"Contract Manufacturers","Computer Hardware"}	{NSYS,FLEX,SGMA,KBAL,PLXS,TTMI,JBL,BHE,CTS,SANM,XLK}	Computer Hardware	Technology	NYSE American	\N	2020-10-11 23:12:04.625395	2020-10-11 23:38:31.025871
IESC	IES Holdings Inc.	IES Holdings Inc is a holding company. The company owns and manages operating subsidiaries comprised of provider of industrial products and infrastructure services to a variety of end markets.	{Industrials,"Engineering & Construction"}	{EME,FIX,PWR,DY}	Engineering & Construction	Industrials	NASDAQ Global Market	\N	2020-10-11 23:12:04.637811	2020-10-11 23:38:31.025871
IFIX	Xtrackers Barclays International Corporate Bond Hedged	The investment seeks to correspond generally to the Barclays Global Aggregate Corporate Ex USD Bond Index (USD Hedged).\n The fund will normally invest at least 80% of its net assets in corporate bonds and in instruments designed to hedge against its exposure to non-U.S. currencies. The index is designed to track the performance of investment grade corporate debt publicly issued in developed and emerging markets (excluding all securities denominated in USD) in the industrial, utility and financial sectors while mitigating exposure to fluctuations between the value of the USD and the currencies of the countries included in the index. It is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:04.652439	2020-10-11 23:38:31.031328
IGI	Western Asset Investment Grade Defined Opportunity Trust Inc.	Western Asset Invstm Grd Dfnd Opp Tr Inc operates as a non-diversified closed end management investment company.Its primary investment objective is to provide current income and then to liquidate and distribute all of the funds net assets to stockholders.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.67462	2020-10-11 23:38:31.024353
IBKR	Interactive Brokers Group Inc.	Interactive Brokers Group Inc is an automated electronic broker and market maker engaged in routing orders and executing and processing trades in securities, futures, foreign exchange instruments, and mutual funds.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{NDAQ,MKTX,AMTD,CME,GS,SCHW,ICE,CBOE}	Brokers & Exchanges	Financial Services	Investors Exchange	\N	2020-10-11 23:12:04.522447	2020-10-11 23:38:31.02423
IBND	SPDR Bloomberg Barclays International Corporate Bond	The investment seeks to provide investment results that correspond generally to the price and yield performance of the Bloomberg Barclays Global Aggregate ex-USD >$1B: Corporate Bond Index.\n The fund invests substantially all, but at least 80%, of its total assets in the securities comprising the index or in securities that the Adviser determines have economic characteristics that are substantially identical to the economic characteristics of the securities that comprise the index. The index is designed to be a broad based measure of the global investment grade, fixed rate, fixed income corporate markets outside the United States. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.535451	2020-10-11 23:38:31.02582
ICCH	ICC Holdings Inc.	ICC Holdings Inc is a specialty insurance carrier primarily underwriting commercial multi-peril, liquor liability, workers' compensation, and umbrella liability coverages for the food and beverage industry.	{"Financial Services","Insurance - Specialty"}	{MIC,MKL,MCY,NGHC}	Insurance - Specialty	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:04.553308	2020-10-11 23:38:31.028166
ICLN	iShares S&P Global Clean Energy Index Fund	The investment seeks to track the S&P Global Clean Energy IndexTM.\n The fund generally invests at least 90% of its assets in the component securities of the index and in investments that have economic characteristics that are substantially identical to the component securities and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the index. The index is designed to track the performance of approximately 30 of what are will be the most liquid and tradable securities of global companies involved in clean energy related businesses. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:04.572696	2020-10-11 23:38:31.02423
IDHD	Invesco S&P International Developed High Dividend Low Volatility	The investment seeks to track the investment results (before fees and expenses) of the S&P EPAC Ex-Korea Low Volatility High Dividend Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in securities that comprise the underlying index. The index provider compiles, maintains and calculates the underlying index, which is composed of 100 securities in the S&P EPAC Ex-Korea LargeMidCap Index that historically have provided high dividend yields with lower volatility. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:04.595311	2020-10-11 23:38:31.031328
IDX	VanEck Vectors Indonesia Index	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the MVIS® Indonesia Index.\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The index includes securities of Indonesian companies. A company is generally considered to be an Indonesian company if it is incorporated in Indonesia or is incorporated outside of Indonesia but has at least 50% of its revenues/related assets in Indonesia. Such companies may include small- and medium-capitalization companies. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.623188	2020-10-11 23:38:31.028166
IEUR	iShares Core MSCI Europe	The investment seeks to track the investment results of the MSCI Europe IMI.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is a free float-adjusted market capitalization-weighted index which consists of securities from the following 15 developed market countries or regions: Austria, Belgium, Denmark, Finland, France, Germany, Ireland, Italy, the Netherlands, Norway, Portugal, Spain, Sweden, Switzerland and the United Kingdom.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.638182	2020-10-11 23:38:31.031328
IGA	Voya Global Advantage and Premium Opportunity Fund of Beneficial Interest	Voya Global Advantage and Premium Opportunity Fund is a diversified, closed-end management investment company. Its primary investment objective is to provide a high level of income. The secondary investment objective of the fund is capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.661629	2020-10-11 23:38:31.021426
IGOV	iShares S&P/Citigroup International Treasury Bond	The investment seeks to track the investment results of the S&P International Sovereign Ex-U.S. Bond Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The underlying index is a broad, diverse, market value-weighted index designed to measure the performance of sovereign bonds issued in local currencies by developed market countries, as classified by SPDJI, outside the U.S. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:04.685057	2020-10-11 23:38:31.028166
IHT	InnSuites Hospitality Trust Shares of Beneficial Interest	InnSuites Hospitality Trust is a real estate investment trust. It owns and operates four hotels, provides management services for six hotels and provides trademark license services for seven hotels.	{"Real Estate","REIT - Hotel & Motel",REITs}	{}	REITs	Real Estate	NYSE American	\N	2020-10-11 23:12:04.708468	2020-10-11 23:38:31.025871
IBMJ	iShares iBonds Dec 2021 Term Muni Bond	The investment seeks to track the investment results of the S&P AMT-Free Municipal Series Dec 2021 IndexTM, which measures the performance of investment-grade, non-callable U.S. municipal bonds maturing in 2021.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index, and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents, including shares of money market funds advised by BFA or its affiliates, as well as in securities not included in the underlying index, but which BFA believes will help the fund track the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.527182	2020-10-11 23:38:31.024353
ICBK	County Bancorp Inc.	County Bancorp Inc operates as a community-oriented business bank, servicing both business customers and individuals. It provides banking and related financial service including real estate lending, business services, and agricultural finance.	{"Financial Services","Savings & Cooperative Banks",Banks}	{CATY,SLCT}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:04.550477	2020-10-11 23:38:31.023428
ICOL	iShares Inc MSCI Colombia	The investment seeks to track the investment results of the MSCI All Colombia Capped Index composed of Colombian equities.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The index is designed to represent the performance of the broad Colombia equity universe. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.576311	2020-10-11 23:38:31.028166
IDEV	iShares Core MSCI International Developed Markets	The investment seeks to track the investment results of the MSCI World ex USA Investable Market Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The underlying index is free float adjusted, market cap weighted, and is designed to measure large-, mid- and small-capitalization equity market performance and includes stocks from North America, Europe, Australasia and the Far East. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.591067	2020-10-11 23:38:31.02423
IDSY	I.D. Systems Inc.	I.D. Systems Inc, together with its subsidiaries, develops, markets and sells wireless machine-to-machine solutions for managing and securing high-value enterprise assets.	{Technology,"Communication Equipment"}	{KVHI,CMTL,SCKT,HON,CAR,CLRO,XLK}	Communication Equipment	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:04.609616	2020-10-11 23:38:31.028166
IEF	iShares 7-10 Year Treasury Bond ETF	The investment seeks to track the investment results of the ICE U.S. Treasury 7-10 Year Bond Index.\n The fund generally invests at least 90% of its assets in the bonds of the underlying index and at least 95% of its assets in U.S. government bonds. The underlying index measures the performance of public obligations of the U.S. Treasury that have a remaining maturity of greater than seven years and less than or equal to ten years.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:04.626296	2020-10-11 23:38:31.028166
IEV	iShares Europe	The investment seeks to track the investment results of the S&P Europe 350TM, which measures the performance of the stocks of leading companies in the following countries: Austria, Belgium, Denmark, Finland, France, Germany, Ireland, Italy, Luxembourg, the Netherlands, Norway, Portugal, Spain, Sweden, Switzerland and the United Kingdom.\n The fund generally invests at least 90% of its assets in securities of the index and in depositary receipts representing securities of the index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.638826	2020-10-11 23:38:31.02582
IGLD	Internet Gold Golden Lines Ltd.	Internet Gold-Golden Lines Ltd through its holding, provides communication services. It offers domestic fixed-line, cellular & international communication services, multi-channel television, satellite broadcasts & Internet among other services.	{"Communication Services","Telecom Services"}	{PTNR,ERIC,BCOM,TI,CEL}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:12:04.678857	2020-10-11 23:38:31.02423
IHD	Voya Emerging Markets High Income Dividend Equity Fund	Voya Emerging Markets High Dividend Equity Fund is a diversified closed-end management investment company. Its investment objective is providing total return through a combination of current income, capital gains and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.697332	2020-10-11 23:38:31.021426
III	Information Services Group Inc.	Information Services Group Inc provides technology insights, market intelligence and advisory services.The Company provides four lines of service: Research, Consulting and Managed Services and Events.	{Industrials,"Business Services"}	{}	Business Services	Industrials	NASDAQ Global Market	\N	2020-10-11 23:12:04.71528	2020-10-11 23:38:31.02582
ISDR	Issuer Direct Corporation	Issuer Direct Corp provides disclosure management solution, shareholder communication tools and cloud-based compliance technologies. It works with clients in the financial service industry including brokerage firms, corporate issuers and mutual funds.	{Technology,"Information Technology Services","Application Software"}	{III,IRM,CTEK,XLK}	Application Software	Technology	NYSE American	\N	2020-10-11 23:12:05.041885	2020-10-11 23:38:31.02423
IBN	ICICI Bank Limited	ICICI Bank Ltd is a banking company. It is engaged in providing banking and financial services including retail banking, corporate banking, and treasury operations through its branches located across India.	{"Financial Services","Banks - Regional - Asia",Banks}	{}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.534564	2020-10-11 23:38:31.023007
ICCC	ImmuCell Corporation	ImmuCell Corp is engaged in the development, acquisition, manufacture and sale of products that improve the health and productivity of cows for the dairy and beef industries. Its products include First Defense, Wipe Out Dairy Wipes and CMT.	{Healthcare,Biotechnology}	{BASI,IDXX,MRK,HSKA,ZTS,PETX,KIN,RIOT,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:04.552602	2020-10-11 23:38:31.023007
ICLR	ICON plc	Icon PLC is a contract research organization that provides outsourced development services to the pharmaceutical, biotechnology, and medical device industries.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{BASI,MDSO,CRL,SCYX,IQV,XLV}	Medical Diagnostics & Research	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:04.572309	2020-10-11 23:38:31.02582
IDSA	Industrial Services of America Inc.	Industrial Services of America Inc is a waste and recycling company in the United States. It is engaged in the buying, processing and marketing of ferrous and non-ferrous metals and other recyclable commodities.	{Industrials,"Waste Management"}	{}	Waste Management	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:04.608474	2020-10-11 23:38:31.028166
IEFA	iShares Core MSCI EAFE	The investment seeks to track the investment results of the MSCI EAFE IMI Index composed of large-, mid- and small-capitalization developed market equities, excluding the U.S. and Canada.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is designed to measure large-, mid- and small-capitalization equity market performance and includes stocks from Europe, Australasia and the Far East.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:04.626323	2020-10-11 23:38:31.021426
IFEU	iShares FTSE EPRA/NAREIT Europe Index Fund	The investment seeks to track the investment results of the FTSE EPRA Nareit Developed Europe Index composed of real estate equities in developed European markets.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The underlying index measures the stock performance of companies engaged in the ownership and development of developed European real estate markets. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:04.644732	2020-10-11 23:38:31.021426
IFN	India Fund Inc. (The)	India Fund Inc is an asset management company. Its investment objective is long-term capital appreciation, which the Fund seeks to achieve by investing primarily in the equity securities of Indian companies.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.658916	2020-10-11 23:38:31.023428
IGM	iShares North American Tech	The investment seeks to track the investment results of the S&P North American Technology Sector Index.\n The fund invests at least 90% of its assets in securities of the index and in depositary receipts representing securities of the index. The index includes companies in the following categories: producers of sophisticated computer-related devices; providers of communications equipment and internet services; producers of computer and internet software; consultants for information technology; providers of computer services; and semiconductors and semiconductor equipment manufacturers. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.681973	2020-10-11 23:38:31.021426
IHDG	WisdomTree International Hedged Quality Dividend Growth Fund	The investment seeks to track the price and yield performance of the WisdomTree International Hedged Quality Dividend Growth Index.\n The fund will invest at least 80% of its total assets in the component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index consists of dividend-paying common stocks with growth characteristics of companies in the industrialized world, excluding Canada and the U.S., while at the same time neutralizing exposure to fluctuations of the value of foreign currencies relative to the USD. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.696339	2020-10-11 23:38:31.028166
IIIN	Insteel Industries Inc.	Insteel Industries Inc manufactures steel wire reinforcing products for concrete construction applications. It manufactures & markets PC strand & welded wire reinforcement, including ESM, concrete pipe reinforcement & standard welded wire reinforcement.	{Industrials,"Metal Fabrication","Industrial Products"}	{NWPX,USAP,OFLX,CMC,NUE,VMI,ASPN}	Industrial Products	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:04.718266	2020-10-11 23:38:31.031328
IJT	iShares S&P Small-Cap 600 Growth ETF	The investment seeks to track the investment results of the S&P SmallCap 600 Growth IndexTM, which measures the performance of the small-capitalization growth sector of the U.S. equity market.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index, but which the advisor believes will help the fund track the underlying index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:04.736764	2020-10-11 23:38:31.02582
IMGN	ImmunoGen Inc.	Immunogen Inc is a clinical-stage biotechnology company that develops targeted anticancer therapeutics. It has developed its ADC technology to enable the creation of anticancer products.	{Healthcare,Biotechnology}	{AMGN,SGEN,ECYT,NVS,BMY,LLY,PFE,SNY,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:04.755674	2020-10-11 23:38:31.028166
ICSH	iShares Ultra Short-Term Bond	The investment seeks to provide current income consistent with preservation of capital.\n The fund seeks to achieve its investment objective by investing, under normal circumstances, at least 80% of its net assets in a portfolio of U.S. dollar-denominated investment-grade fixed- and floating-rate debt securities that are rated BBB- or higher by S&P Global Ratings and/or Fitch Ratings, Inc. ("Fitch"), or Baa3 or higher by Moody's Investors Service, Inc. ("Moody's"), or, if unrated, determined by BFA to be of equivalent quality.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:04.583706	2020-10-11 23:38:31.021426
IDHQ	Invesco S&P International Developed Quality	The investment seeks to track the investment results (before fees and expenses) of the S&P Quality Developed ex-U.S. LargeMidCap Index (the "underlying index").\n The fund will invest at least 90% of its total assets in securities that comprise the underlying index. The index provider compiles, maintains and calculates the underlying index, which is constructed from constituents of the S&P Developed ex-U.S. LargeMidCap Index that the index provider identifies as being of the highest quality-that is, stocks of companies that seek to generate higher revenue and cash flow than their counterparts through prudent use of assets and finances.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.59625	2020-10-11 23:38:31.021426
IDTI	Integrated Device Technology Inc.	Integrated Device Technology Inc develops system-level solutions that optimize its customers' applications. The company operates through two segments namely the Communications, and the Computing, Consumer and Industrial.	{Technology,Semiconductors}	{XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:04.612922	2020-10-11 23:38:31.02582
IEP	Icahn Enterprises L.P.	Icahn Enterprises LP owns a diverse range of businesses in the United States. Its most revenue generating segment is the Automotive segment.	{Industrials,Conglomerates}	{MNRO,AZO,AAP,ORLY}	Conglomerates	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:04.635898	2020-10-11 23:38:31.024353
IFLY	ETFMG Drone Economy Strategy	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the price and yield performance of the Reality Shares Drone™ Index.\n The index tracks the performance of the equity securities of companies across the globe and across a wide variety of industries that are actively involved in the development, research, or utilization of drones. The fund normally invests at least 80% of its total assets, exclusive of collateral held from securities lending, in securities, ADRs, or GDRs of Drone Economy Companies. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.652391	2020-10-11 23:38:31.025871
IGEB	iShares Edge Investment Grade Enhanced Bond	The investment seeks to track the investment results of the BlackRock Investment Grade Enhanced Bond Index.\n The fund will invest at least 80% of its assets in the component securities of the underlying index and may invest up to 20% of its assets in certain index futures, options, options on index futures, swap contracts or other derivatives, cash and cash equivalents, other investment companies, as well as in securities and other instruments not included in the underlying index, but which BFA believes will help the fund track the underlying index. The index consists of U.S. dollar-denominated, investment-grade corporate bonds.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:04.668524	2020-10-11 23:38:31.02582
IGN	iShares North American Tech-Multimedia Networking	The investment seeks to track the investment results of the S&P North American Technology Multimedia Networking Index composed of North American equities in the multimedia and networking technology sectors.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index measures the performance of U.S.-traded stocks of communication equipment companies in the U.S. and Canada. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.685141	2020-10-11 23:38:31.028166
IHI	iShares U.S. Medical Devices	The investment seeks to track the investment results of the Dow Jones U.S. Select Medical Equipment Index composed of U.S. equities in the medical devices sector.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index includes medical equipment companies, including manufacturers and distributors of medical devices such as magnetic resonance imaging (MRI) scanners, prosthetics, pacemakers, X-ray machines, and other non-disposable medical devices. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.70716	2020-10-11 23:38:31.031328
IIVI	II-VI Incorporated	II-VI Inc develops and manufactures engineered materials and opto-electronic components and products for precision use in industrial, optical communications, military, semiconductor and life science applications.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{CREE,COHR,LPTH,GLW,NOC,DYSL,VIAV,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:04.728137	2020-10-11 23:38:31.024353
ILPT	Industrial Logistics Properties Trust	Industrial Logistics Properties Trust is a real estate investment trust formed in Maryland. It owns and leases industrial and logistics properties throughout the United States, and owns more-than 250 properties in 24 other states.	{"Real Estate","REIT - Industrial",REITs}	{}	REITs	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:04.747673	2020-10-11 23:38:31.025871
IMMU	Immunomedics Inc.	Immunomedics Inc is a clinical-stage biopharmaceutical company. The Company is engaged in developing monoclonal antibody-based products for the targeted treatment of cancer, autoimmune and other serious diseases.	{Healthcare,Biotechnology}	{AMGN,SGEN,BIIB,BMY,LLY,PFE,NAVB,ANTH,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:04.766403	2020-10-11 23:38:31.025871
KANG	iKang Healthcare Group Inc.	iKang Healthcare Group Inc provides preventive healthcare solutions including medical examinations services and value-added services including disease screening, dental services and other services in China.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{CYH,CCM,THC,XLV}	Medical Diagnostics & Research	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:05.658781	2020-10-11 23:38:31.021426
ICFI	ICF International Inc.	ICF International Inc provides professional services and technology-based solutions to government and commercial clients, including management, technology, and policy consulting and implementation services.	{Industrials,"Business Services"}	{}	Business Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:04.565295	2020-10-11 23:38:31.023007
ICPT	Intercept Pharmaceuticals Inc.	Intercept Pharmaceuticals Inc is a biopharmaceutical company. It is focused on the development and commercialization of novel therapeutics to treat non-viral, progressive liver diseases with high unmet medical need.	{Healthcare,Biotechnology}	{GILD,MNOV,BMY,LLY,CNAT,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:04.584185	2020-10-11 23:38:31.025871
IDLV	Invesco S&P International Developed Low Volatility	The investment seeks to track the investment results (before fees and expenses) of the S&P BMI International Developed Low VolatilityTM Index (the "underlying index").\n The fund will invest at least 90% of its total assets in the securities of companies that comprise the underlying index. S&P Dow Jones Indices ("S&P DJI" or the "index provider") compiles, maintains and calculates the underlying index, which is designed to measure the performance of 200 of the least volatile stocks of the S&P Developed ex-U.S. & South Korea LargeMidCap Index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.598648	2020-10-11 23:38:31.024353
IDXG	Interpace Diagnostics Group Inc.	Interpace Diagnostics Group Inc is a United States based integrated commercial and bioinformatics company. It is engaged in developing and commercializing molecular diagnostic tests.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{PFE,VEEV,TNET,PRPO,IQV,XLV}	Medical Diagnostics & Research	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:04.62076	2020-10-11 23:38:31.02423
IEUS	iShares Developed Small Cap ex-North America	The investment seeks to track the investment results of the MSCI Europe Small Cap Index composed of small-capitalization developed market equities in Europe.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is a free float-adjusted, market capitalization-weighted index that captures small-capitalization representation across the 15 developed market countries in Europe.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:04.638565	2020-10-11 23:38:31.028166
IGF	iShares Global Infrastructure ETF	The investment seeks to track the S&P Global Infrastructure IndexTM.\n The fund generally invests at least 90% of its assets in the component securities of the index and in investments that have economic characteristics that are substantially identical to the component securities and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The index is designed to track performance of the stocks of large infrastructure companies in developed or emerging markets that must be domiciled in developed markets, or whose stocks are listed on developed market exchanges around the world.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:04.667973	2020-10-11 23:38:31.025871
IGR	CBRE Clarion Global Real Estate Income Fund	CBRE Clarion Global Real Estate Income Fund is a United States based diversified, closed-end management investment company. Its objective is high current income and its secondary objective is capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.685008	2020-10-11 23:38:31.02582
IHE	iShares U.S. Pharmaceutical	The investment seeks to track the investment results of the Dow Jones U.S. Select Pharmaceuticals Index.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index measures the performance of the pharmaceuticals sector of the U.S. equity market. The underlying index includes pharmaceutical companies such as manufacturers of prescription or over-the-counter drugs or vaccines, but excludes producers of vitamins. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.69887	2020-10-11 23:38:31.02582
IHY	VanEck Vectors International High Yield Bond	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the ICE BofAML Global ex-US Issuers High Yield Constrained Index.\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The index is comprised of below investment grade bonds issued by corporations located throughout the world (which may include emerging market countries) excluding the United States denominated in Euros, U.S. dollars, Canadian dollars or pound sterling and issued in the major domestic or Eurobond markets.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.71403	2020-10-11 23:38:31.02423
IJH	iShares Core S&P Mid-Cap	The investment seeks to track the investment results of the S&P MidCap 400® (the "underlying index"), which measures the performance of the mid-capitalization sector of the U.S. equity market.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index, but which the advisor believes will help the fund track the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.729555	2020-10-11 23:38:31.025871
ICON	Iconix Brand Group Inc.	Iconix Brand Group Inc is a brand management company. It is engaged in licensing, marketing and providing trend direction for a portfolio of consumer brands. It operates through five segments: men's, women's, home, entertainment and international.	{"Consumer Cyclical","Apparel Manufacturing","Manufacturing - Apparel & Furniture"}	{}	Manufacturing - Apparel & Furniture	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:04.576123	2020-10-11 23:38:31.028166
IDE	Voya Infrastructure Industrials and Materials Fund of Beneficial Interest	Voya Infrastructure, Industrials and Materials Fund is a diversified closed-end management investment company. The company's investment objective is total return through a combination of current income, capital gains and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.592591	2020-10-11 23:38:31.02582
IDXX	IDEXX Laboratories Inc.	IDEXX Laboratories Inc develops, manufactures and distributes products and provide services for companion animal veterinary, livestock and poultry, dairy and water testing markets.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{HSIC,NEOG,VIVO,ABT,TMO,HSKA,XLV}	Medical Diagnostics & Research	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:04.622707	2020-10-11 23:38:31.023428
IEZ	iShares U.S. Oil Equipment & Services	The investment seeks to track the investment results of the Dow Jones U.S. Select Oil Equipment & Services Index.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying fund measures the performance of the oil equipment and services sector of the U.S. equity market and includes companies that are suppliers of equipment or services to oil fields and offshore platforms, such as drilling, exploration, seismic information services and platform construction. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.645251	2020-10-11 23:38:31.028166
IGHG	ProShares Investment Grade-Interest Rate Hedged	The investment seeks investment results, before fees and expenses, that track the performance of the FTSE Corporate Investment Grade (Treasury Rate-Hedged) Index (the "index").\n The index is comprised of (a) long positions in USD-denominated investment grade corporate bonds issued by both U.S. and foreign domiciled companies; and (b) short positions in U.S. Treasury notes or bonds ("Treasury Securities") of, in aggregate, approximate equivalent duration to the investment grade bonds. The fund will invest at least 80% of its total assets in component securities (i.e., securities of the index) and invest at least 80% of its total assets in investment grade bonds.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:04.669056	2020-10-11 23:38:31.031328
IGRO	iShares International Dividend Growth	The investment seeks to track the investment results of the Morningstar® Global ex-US Dividend Growth IndexSM.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The index is a dividend dollars weighted index that seeks to measure the performance of international equities selected based on a consistent history of growing dividends. It is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:04.68653	2020-10-11 23:38:31.023007
IHF	iShares U.S. Health Care Providers	The investment seeks to track the investment results of the Dow Jones U.S. Select Health Care Providers Index composed of U.S. equities in the healthcare providers sector.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index measures the performance of the healthcare providers sector of the U.S. equity market. The fund may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.700639	2020-10-11 23:38:31.023007
IID	Voya International High Dividend Equity Income Fund of Beneficial Interest	Voya International High Dividend Equity Income Fund is a diversified, closed-end management investment company. The primary objective is to seek current income and current gains. Its secondary objective is to achieve long-term capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.71528	2020-10-11 23:38:31.021426
IJK	iShares S&P Mid-Cap 400 Growth	The investment seeks to track the investment results of the S&P MidCap 400 Growth IndexTM, which measures the performance of the mid-capitalization growth sector of the U.S. equity market.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index, but which the advisor believes will help the fund track the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.735425	2020-10-11 23:38:31.028166
IMH	Impac Mortgage Holdings Inc.	Impac Mortgage Holdings Inc, through its subsidiaries, offers residential mortgage services in the U.S. Its business activities are organized in three operating segments: Mortgage Lending, Real Estate Services and the Long?Term Mortgage Portfolio.	{"Financial Services","Specialty Finance",Banks}	{AI,EFC}	Banks	Financial Services	NYSE American	\N	2020-10-11 23:12:04.755987	2020-10-11 23:38:31.02582
IMOS	ChipMOS TECHNOLOGIES INC.	ChipMOS TECHNOLOGIES Inc is a semiconductor testing and packaging solutions company. The Company is engaged in research, development, manufacturing and sale of integrated circuits and related assembly, and providing testing services.	{Technology,Semiconductors}	{XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:04.772268	2020-10-11 23:38:31.028166
IDMO	Invesco S&P International Developed Momentum	The investment seeks to track the investment results (before fees and expenses) of the S&P Momentum Developed ex-U.S. & South Korea LargeMidCap IndexTM (the "underlying index").\n The fund will invest at least 90% of its total assets in the securities of companies that comprise the underlying index. Strictly in accordance with its guidelines and mandated procedures, the index provider compiles, maintains and calculates the underlying index, which is composed of constituents of the S&P Developed ex-U.S. & South Korea LargeMidCap Index that have the highest "momentum score."	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.599958	2020-10-11 23:38:31.023007
IDU	iShares U.S. Utilities	The investment seeks to track the investment results of the Dow Jones U.S. Utilities Index.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index measures the performance of the utilities sector of the U.S. equity market and may include large-, mid- or small-capitalization companies. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.614229	2020-10-11 23:38:31.023007
IEO	iShares U.S. Oil & Gas Exploration & Production	The investment seeks to track the investment results of the Dow Jones U.S. Select Oil Exploration & Production Index composed of U.S. equities in the oil and gas exploration and production sector.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index measures the performance of the oil exploration and production sector of the U.S. equity market. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:04.632421	2020-10-11 23:38:31.023007
IFMK	iFresh Inc.	iFresh Inc is a Asian/Chinese grocery supermarket chain in the North Eastern U.S. providing food and other merchandise hard to find in mainstream grocery stores. It currently has eight retail supermarkets across New York, Massachusetts and Florida.	{"Consumer Defensive","Grocery Stores","Retail - Defensive"}	{}	Retail - Defensive	Consumer Defensive	NASDAQ Capital Market	\N	2020-10-11 23:12:04.652439	2020-10-11 23:38:31.024353
IGE	iShares North American Natural Resources	The investment seeks to track the investment results of the S&P North American Natural Resources Sector Index composed of North American equities in the natural resources sector.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index measures the performance of U.S.-traded stocks of natural resource-related companies in the U.S. and Canada. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:04.66773	2020-10-11 23:38:31.02423
IGT	International Game Technology	International Game Technology PLC is a gaming technology company. Gaming services offered are lottery management services, online and instants lotteries, electronic gaming machines, sports betting, interactive gaming, and commercial services.	{"Consumer Cyclical",Gambling,"Travel & Leisure"}	{ADSK,FTNT}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:04.689295	2020-10-11 23:38:31.031328
IHTA	Invesco High Income 2024 Target Term Fund of Beneficial Interest No par value per share	The Fund’s investment objectives are to provide a high level of current income and to return $9.835 per share (the original net asset value (“NAV”) per common share before deducting offering costs of $0.02 per share) to holders of common shares on or about December 1, 2024 (the “Termination Date”). The objective to return the Fund’s original NAV is not an express or implied guarantee obligation of the Fund or any other entity.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.708787	2020-10-11 23:38:31.024353
IIPR	Innovative Industrial Properties Inc	Innovative Industrial Properties Inc is engaged in the acquisition, ownership and management of specialized industrial properties leased to state-licensed operators for their regulated medical-use cannabis facilities.	{"Real Estate","Real Estate - General","Real Estate Services"}	{NYMT,O,NTP}	Real Estate Services	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:04.724651	2020-10-11 23:38:31.021426
ILF	iShares Latin America 40	The investment seeks to track the investment results of the S&P Latin America 40TM composed of 40 of the largest Latin American equities.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. It seeks to track the investment results of the S&P Latin America 40TM (the "underlying index"), which is comprised of selected equities trading on the exchanges of five Latin American countries. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.737423	2020-10-11 23:38:31.023007
IMRNW	Immuron Limited Warrants		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:04.777528	2020-10-11 23:38:31.024353
INCY	Incyte Corporation	Incyte Corp is a biopharmaceutical company. It discovers, develops and commercializes proprietary therapeutics. Its product JAKAFI, a JAK1 and JAK2 inhibitor is used for treatment of patients with intermediate or high risk myelofibrosis.	{Healthcare,Biotechnology}	{AMGN,GERN,CELG,AVEO,NVS,LLY,PFE,LGND,ARDX,IONS,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:04.796559	2020-10-11 23:38:31.023007
KALV	KalVista Pharmaceuticals Inc.	KalVista Pharmaceuticals Inc is a pharmaceuticals company, which focuses on the discovery, development, and commercialization of small molecule protease inhibitors for a range of diseases.	{Healthcare,Biotechnology}	{XENT,EYEG,KOOL,SNOA,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:05.657931	2020-10-11 23:38:31.023428
IEI	iShares 3-7 Year Treasury Bond ETF	The investment seeks to track the investment results of the ICE U.S. Treasury 3-7 Year Bond Index (the "underlying index").\n The fund generally invests at least 90% of its assets in the bonds of the underlying index and at least 95% of its assets in U.S. government bonds. The underlying index measures the performance of public obligations of the U.S. Treasury that have a remaining maturity of greater than three years and less than or equal to seven years.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:04.627829	2020-10-11 23:38:31.031328
IEX	IDEX Corporation	IDEX Corp produces and distributes flow meters, injectors, and other fluid-handling pump modules. Its business segments are Fluid and Metering Technologies, Health and Science Technologies and Fire and Safety/Diversified Products.	{Industrials,"Diversified Industrials","Industrial Products"}	{}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:04.643989	2020-10-11 23:38:31.023428
IFRX	InflaRx N.V.	InflaRx NV is a biopharmaceutical company. It discovers and develops potent and specific inhibitors of the complement activation factor known as C5a.	{Healthcare,Biotechnology}	{PDLI,PIRS,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:04.658933	2020-10-11 23:38:31.028166
IGIH	Xtrackers Investment Grade Bond - Interest Rate Hedged	The investment seeks investment results that correspond generally to the performance, before fees and expenses, of the Solactive Investment Grade Bond - Interest Rate Hedged Index.\n The fund will invest at least 80% of its total assets, but typically far more, in instruments that comprise the underlying index. The index is comprised of (a) long positions in U.S. dollar-denominated investment-grade corporate bonds and (b) short positions in U.S. Treasury notes or bonds ("Treasury Securities") of, in aggregate, approximate equivalent duration to the investment-grade corporate bonds.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:04.675216	2020-10-11 23:38:31.023428
IHG	Intercontinental Hotels Group American Depositary Shares (Each representing one)	InterContinental Hotels Group PLC owns, manages, franchises, and leases hotels and resorts. It operates hotels under its brands, which includes InterContinental, Kimpton, HUALUXE, Crowne Plaza, Hotel Indigo, EVEN Hotels, and o amongst others.	{"Consumer Cyclical",Lodging,"Travel & Leisure"}	{CHH,WYND,H,MAR,HLT}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:04.704022	2020-10-11 23:38:31.028166
IIM	Invesco Value Municipal Income Trust	Invesco Value Municipal Income Trust is a diversified, closed-end management investment company. Its investment objective is to provide common shareholders with current income which is exempt from federal income tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.720459	2020-10-11 23:38:31.023007
IKNX	Ikonics Corporation	Ikonics Corp is engaged in the development, manufacture, and selling of photosensitive liquids (emulsions) and films for the screen printing and awards and recognition industries, and dye sublimation markets.	{"Basic Materials","Specialty Chemicals",Chemicals}	{AMAT,PLAB,BTN,ITW}	Chemicals	Basic Materials	NASDAQ Capital Market	\N	2020-10-11 23:12:04.740704	2020-10-11 23:38:31.028166
IMI	Intermolecular Inc.	Intermolecular Inc provides support materials for the semiconductor industry. It generates revenue through the sale of its products to a semiconductor device, semiconductor materials, and equipment companies mainly in the Asia-Pacific markets.	{Technology,"Semiconductor Equipment & Materials",Semiconductors}	{CDNS,SNPS,ENTG,PDFS,XPER,QTRH,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:04.755979	2020-10-11 23:38:31.028166
IMOM	Alpha Architect International Quantitative Momentum	The investment seeks to track the total return performance, before fees and expenses, of the Alpha Architect International Quantitative Momentum Index.\n The fund will normally invest at least 80% of its total assets in the component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities (e.g., depositary receipts). The index uses a 5-step, quantitative, rules-based methodology to identify a portfolio of approximately 40-50 non-U.S. equity securities with positive momentum. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:04.772399	2020-10-11 23:38:31.02582
INBK	First Internet Bancorp	First Internet Bancorp is a bank holding company that conducts its business activities through its subsidiary, First Internet Bank of Indiana. It offers Residential Mortgage, Consumer, Commercial Real Estate & Commercial and Industrial Lending.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:04.791576	2020-10-11 23:38:31.02582
ISRG	Intuitive Surgical Inc.	Intuitive Surgical Inc designs, manufactures, markets da Vinci Surgical Systems, and related instruments. The da Vinci surgery, combines the benefits of minimally invasive surgery for patients with the ease of use, precision and dexterity of open surgery.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{HOLX,FMS,BAX,BDX,BSX,SYK,TFX,VAR,SNN,MDT,XLV}	Medical Instruments & Equipment	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:05.058752	2020-10-11 23:38:31.02582
IEMG	iShares Core MSCI Emerging Markets	The investment seeks to track the investment results of the MSCI Emerging Markets Investable Market Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is designed to measure large-, mid- and small-cap equity market performance in the global emerging markets.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.631009	2020-10-11 23:38:31.02582
IFF	Internationa Flavors & Fragrances Inc.	International Flavors & Fragrances Inc is engaged in manufacturing and supplying of flavors and fragrances for the food, beverage, personal care and household products industries either in the form of compounds or individual ingredients.	{"Basic Materials","Specialty Chemicals",Chemicals}	{BCPC,UG,EL,GRA,PG,SCL,SXT}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:04.649581	2020-10-11 23:38:31.02423
IGD	Voya Global Equity Dividend and Premium Opportunity Fund	Voya Global Equity Dividend and Premium Opportunity Fund is a diversified, closed-end management investment company. The primary objective of the fund is to provide a high level of income. Its secondary objective is capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.663846	2020-10-11 23:38:31.028166
IGV	iShares North American Tech-Software	The investment seeks to track the investment results of the S&P North American Technology Software Index composed of North American equities in the software sector.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The index measures the performance of U.S.-traded stocks of software-related companies in the U.S. and Canada. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:04.688124	2020-10-11 23:38:31.025871
IHIT	Invesco High Income 2023 Target Term Fund of Beneficial Interest	Invesco High Income 2023 Target Term Fund is closed-end management investment company. Its investment objectives is to provide a high level of current income and to return original net asset value before deducting offering costs to shareholders.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.70836	2020-10-11 23:38:31.023428
ILTB	iShares Core 10 Year USD Bond	The investment seeks to track the investment results of the Bloomberg Barclays U.S. Universal 10+ Year Index (the "underlying index").\n The underlying index measures the performance of U.S. dollar-denominated taxable bonds that are rated either investment-grade or high yield with remaining maturities greater than ten years. The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.748466	2020-10-11 23:38:31.031328
IMMR	Immersion Corporation	Immersion Corp is a software licensing company. It is engaged in creation, design, development, and licensing of patented haptic technologies that allow people to use their sense of touch more fully when operating digital devices.	{Technology,"Software - Application","Application Software"}	{MSFT,ALOT,KTCC,SYNA,LOGI,MEI,TXN,GOOGL,VC,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:04.765804	2020-10-11 23:38:31.031328
IMTM	iShares Edge MSCI Intl Momentum Factor	The investment seeks to track the investment results of the MSCI World ex USA Momentum Index that measures the performance of international developed large- and mid-capitalization stocks exhibiting relatively higher momentum characteristics.\n The fund will invest at least 90% of its assets in the component securities of the index and in investments that have economic characteristics that are substantially identical to the component securities of the index. The index consists of stocks exhibiting relatively higher momentum characteristics than the traditional market capitalization-weighted parent index, the MSCI World ex USA Index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.781312	2020-10-11 23:38:31.025871
INCO	Columbia India Consumer	The investment seeks investment results that correspond to the price and yield performance of the Indxx India Consumer Index.\n The fund will invest at least 80% of its net assets in Indian consumer companies included in the index and the advisor generally expects to be substantially invested at such times, with at least 95% of its net assets invested in these securities. The index is a maximum 30-stock free-float adjusted market capitalization-weighted index designed to measure the market performance of companies in the consumer industry in India, as defined by Indxx's proprietary methodology. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.796632	2020-10-11 23:38:31.024353
INDUU	Industrea Acquisition Corp. Unit	Industrea Acquisition Corp is a newly organized blank check company formed for the purpose of effecting a merger, capital stock exchange, asset acquisition, stock purchase, reorganization or similar business combination with one or more businesses.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:04.80264	2020-10-11 23:38:31.023428
INFO	IHS Markit Ltd.	IHS Markit Ltd is a provider of data and analytics to corporate and sovereign clients across various industries. It products include economic forecasting, cost modeling, risk assessments and scenario analysis, competitive benchmarking and consulting.	{Industrials,"Business Services"}	{TRI,MORN,FDS,IBM,MSCI,ENV,RELX}	Business Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:04.825478	2020-10-11 23:38:31.02423
INNTW	Innovate Biopharmaceuticals Inc. Warrant		{}	{}			OTC Pink	\N	2020-10-11 23:12:04.848864	2020-10-11 23:38:31.021426
IIF	Morgan Stanley India Investment Fund Inc.	Morgan Stanley India Investment Fund is a non-diversified, closed-end management investment company. Its investment objective is to achieve long-term capital appreciation through investments primarily in equity securities of Indian Issuers.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.716155	2020-10-11 23:38:31.028166
IMAX	Imax Corporation	Imax Corp is an entertainment technology company, specializing in motion picture technologies and large-format motion picture presentations. It offers end-to-end cinematic solution combining proprietary software, theater architecture and equipment.	{"Consumer Cyclical","Media - Diversified",Entertainment}	{}	Entertainment	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:04.749546	2020-10-11 23:38:31.02423
IMO	Imperial Oil Limited	Imperial Oil Ltd is engaged in the exploration for, and production and sale of, crude oil and natural gas. Its operations are conducted in three main segments namely Upstream, Downstream and Chemical.	{Energy,"Oil & Gas Integrated","Oil & Gas - Integrated"}	{CNQ,SU,CVE,XOM,RDS.A,PBA,ATH}	Oil & Gas - Integrated	Energy	NYSE American	\N	2020-10-11 23:12:04.770392	2020-10-11 23:38:31.02423
INAP	Internap Corporation	Internap Corp is engaged in providing high-performance data center services including colocation, managed hosting, cloud, and network services.	{Technology,"Software - Application","Application Software"}	{AMZN,VRSN,AKAM,LLNW,IBM,T,CARB,EQIX,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:04.78508	2020-10-11 23:38:31.031328
INDY	iShares S&P India Nifty 50 Index Fund	The investment seeks to track the investment results of the Nifty 50 IndexTM composed of 50 of the largest Indian equities.\n The Subsidiary and the fund will collectively invest at least 90% of the fund's assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The underlying index measures the equity performance of the top 50 companies by free float market capitalization whose equity securities trade in the Indian securities markets. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:04.807073	2020-10-11 23:38:31.02582
INGN	Inogen Inc	Inogen Inc is a medical technology company that develops, manufactures and markets portable oxygen concentrators used to deliver supplemental long-term oxygen therapy to patients suffering from chronic respiratory conditions.	{Healthcare,"Medical Devices"}	{MASI,GTLS,RMD,IVC,PHG,ELMD,AHPI,XTLB,XLV}	Medical Devices	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:04.83372	2020-10-11 23:38:31.024353
INGR	Ingredion Incorporated	Ingredion Inc is a major supplier of high-quality food and industrial ingredients to customers around the world. The ingredients are used by customers in the food, beverage, animal feed, paper and corrugating, and brewing industries, among others.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{ADM,BG,MGPI}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:04.833828	2020-10-11 23:38:31.023007
INO	Inovio Pharmaceuticals Inc.	Inovio Pharmaceuticals Inc is a clinical stage biopharmaceutical company. It develops active DNA immunotherapies and vaccines in combination with proprietary electroporation delivery devices to prevent and treat cancers and infectious diseases.	{Healthcare,Biotechnology}	{DRRX,GSK,NVS,JNJ,PFE,MRK,CBLI,ABEO,ARWR,VICL,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:04.851627	2020-10-11 23:38:31.028166
INOV	Inovalon Holdings Inc.	Inovalon Holdings Inc is a United State based technology company. It delivers seamless, end-to-end platforms that bring the benefits of big data and large-scale analytics to the point of care.	{Technology,"Health Information Services","Application Software"}	{XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:04.856162	2020-10-11 23:38:31.028166
INSW	International Seaways Inc.	International Seaways Inc operates a fleet of oceangoing vessels engaged in the transportation of crude oil and petroleum products in the International Flag trades.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{SMHI}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:04.873396	2020-10-11 23:38:31.02582
INST	Instructure Inc.	Instructure, Inc. provides cloud-based learning management platform for academic institutions and companies worldwide. The company offers its platform through a Software-as-a-Service business model.	{Technology,"Software - Application","Application Software"}	{OTEX,PANW,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:04.870147	2020-10-11 23:38:31.023428
INTT	inTest Corporation	inTest Corp is an independent designer, manufacturer and marketer of thermal, mechanical and electrical products that are used by semiconductor manufacturers in conjunction with ATE, in the testing of ICs.	{Technology,Semiconductors}	{AEHR,COHU,CAMT,FORM,A,TER,RTEC,XLK}	Semiconductors	Technology	NYSE American	\N	2020-10-11 23:12:04.886416	2020-10-11 23:38:31.031328
KAMN	Kaman Corporation	Kaman Corp is engaged in the distribution of power transmission, motion control, electrical and automation, and fluid power industrial distributor with operations throughout the United States.	{Industrials,"Aerospace & Defense"}	{NNBR,ROLL,LMT,GPC,PH,TKR,RXN,DNOW}	Aerospace & Defense	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:05.659054	2020-10-11 23:38:31.024353
ILMN	Illumina Inc.	Illumina Inc engages in life science tools and machines for analyzing genetic material. The company primarily engages in microarray and genome sequencing machines and disposables. It also provides sequencing services through its products.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{XLV}	Medical Diagnostics & Research	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:04.742084	2020-10-11 23:38:31.024353
IMMP	Immutep Limited	Immutep Ltd, formerly Prima Biomed Ltd is a biotechnology company. It is engaged in the development of immunotherapeutic products for cancer and autoimmune disease. Their operations are based in Australia and Europe.	{Healthcare,Biotechnology}	{ILMN,RDUS,FGEN,HTGM,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:04.761812	2020-10-11 23:38:31.023007
IMTE	Integrated Media Technology Limited	Integrated Media Technology Ltd is engaged in development, sale and distribution of autostereoscopic 3D display, 3D conversion equipment and software, development and sale of 3D autostereoscopic technology and provision of 3D consultancy services.	{Technology,"Electronic Gaming & Multimedia","Application Software"}	{XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:04.781756	2020-10-11 23:38:31.023428
INDA	Ishares MSCI India	The investment seeks to track the investment results of the MSCI India Index composed of Indian equities.\n The fund generally will collectively invest at least 90% of its assets in the component securities of the index and in investments that have economic characteristics that are substantially identical to the component securities of the index. The index measures the performance of equity securities of companies whose market capitalization, as calculated by the index provider, represents the top 85% of companies in the Indian securities market. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:04.79992	2020-10-11 23:38:31.025871
INFU	InfuSystems Holdings Inc.	InfuSystems Holdings Inc is a provider of ambulatory infusion pumps and related services in the United States. It provides services to hospitals, oncology practices and facilities and other alternate site health care providers.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{CRVL,PINC,TVTY,XLV}	Medical Instruments & Equipment	Healthcare	NYSE American	\N	2020-10-11 23:12:04.83108	2020-10-11 23:38:31.023007
INNT	Innovate Biopharmaceuticals Inc.	Innovate Biopharmaceuticals Inc is a clinical stage biotechnology company. The company is engaged in developing novel medicines for autoimmune/ inflammatory disease.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:04.849076	2020-10-11 23:38:31.02582
INPX	Inpixon	Inpixon through its subsidiaries provides data analytics and location-based products and related services for the cyber-security and internet of things markets. The company's segments include Indoor positioning analytics and Infrastructure.	{Technology,"Software - Application","Application Software"}	{XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:04.858089	2020-10-11 23:38:31.023428
INSG	Inseego Corp.	Inseego Corp enables mobile applications for large enterprise verticals, service providers and small medium businesses. Its portfolio consists of Enterprise SaaS and IoT and Mobile solutions.	{Technology,"Communication Equipment"}	{XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:04.865279	2020-10-11 23:38:31.021426
INTC	Intel Corporation	Intel Corp is the world's largest chipmaker. It engaged in making a semiconductor chip. It designs and manufactures integrated digital technology products like integrated circuits, for industries such as computing and communications.	{Technology,Semiconductors}	{AMAT,QCOM,NVDA,TXN,AMD,HPQ,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:04.876061	2020-10-11 23:38:31.028166
INTG	The Intergroup Corporation	The Intergroup Corp is a real estate development company. It is engaged in the operation of multi-family residential properties, operation of hotel, and in the investment of its cash in marketable securities and other investments.	{"Real Estate","Real Estate Services"}	{MAYS,TCI,NEN,ARL}	Real Estate Services	Real Estate	NASDAQ Capital Market	\N	2020-10-11 23:12:04.879455	2020-10-11 23:38:31.024353
INUV	Inuvo Inc.	Inuvo Inc is an internet advertising technology and digital publishing company. The Company along with its subsidiaries develops technology to deliver content and targeted advertisements over the internet.	{"Consumer Cyclical","Marketing Services","Advertising & Marketing Services"}	{}	Advertising & Marketing Services	Consumer Cyclical	NYSE American	\N	2020-10-11 23:12:04.893136	2020-10-11 23:38:31.02582
INTX	Intersections Inc.	Intersections Inc provides subscription based consumer protection services that help consumers understand and monitor their credit profiles and other personal information and protect themselves against identity theft or fraud.	{Industrials,"Business Services"}	{}	Business Services	Industrials	NASDAQ Global Market	\N	2020-10-11 23:12:04.891187	2020-10-11 23:38:31.02423
INXX	Columbia India Infrastructure	The investment seeks investment results that correspond to the price and yield performance of the Indxx India Infrastructure Index.\n Under normal circumstances, the fund will invest at least 80% of its net assets in Indian infrastructure companies included in the index and the advisor generally expects to be substantially invested at such times, with at least 95% of its net assets invested in these securities. The index is a maximum 30-stock free-float adjusted market capitalization-weighted index designed to measure the market performance of companies in the infrastructure industry in India. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.904568	2020-10-11 23:38:31.028166
KBLM	KBL Merger Corp. IV	KBL Merger Corp IV is a blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:05.678454	2020-10-11 23:38:31.021426
IMKTA	Ingles Markets Incorporated Class A Common Stock	Ingles Markets Inc is a supermarket chain in the Southeast United States. It operates grocery retail stores offering grocery, meat and dairy products, produce, frozen foods and other perishables, and non-food products..	{"Consumer Defensive","Grocery Stores","Retail - Defensive"}	{KR,WMK}	Retail - Defensive	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:04.758795	2020-10-11 23:38:31.023428
IMRN	Immuron Limited	Immuron Ltd is a biopharmaceutical company which develops polyclonal antibodies for the treatment and prevention of diseases. It products include Travelan, IMM-124E for NASH and IMM-529 for Clostridium difficile infection.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:04.777093	2020-10-11 23:38:31.028166
INBKL	First Internet Bancorp 6.0% Fixed-to-Floating Rate Subordinated Notes due 2026		{}	{}			Nasdaq Global Select	\N	2020-10-11 23:12:04.796418	2020-10-11 23:38:31.028166
ING	ING Group N.V.	ING Groep NV operates in the financial services domain. Its key areas of operation include retail and commercial banking.	{"Financial Services","Banks - Global",Banks}	{BAP,JPM,KB,C}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.833591	2020-10-11 23:38:31.023428
INS	Intelligent Systems Corporation	Intelligent Systems Corp is engaged in the business of providing technology solutions and processing services to the financial technology & services market referred to as the FinTech industry.	{Industrials,"Diversified Industrials","Industrial Products",Technology,"Software - Infrastructure","Application Software"}	{MIDD,SXI,TNC,CVR,EVI,SPLP,XLK}	Application Software	Technology	NYSE American	\N	2020-10-11 23:12:04.857543	2020-10-11 23:38:31.025871
INSM	Insmed Inc.	Insmed Inc is a global biotechnology company. It is focused on developing novel therapies that focus primarily on lung diseases.	{Healthcare,Biotechnology}	{SHPG,VRTX,ARRY,ALIM,NVS,SRPT,OPHT,AERI,ARDM,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:04.868703	2020-10-11 23:38:31.02423
INTF	iShares Edge MSCI Multifactor Intl	The investment seeks to track the investment results of the MSCI World ex USA Diversified Multiple-Factor Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is designed to select equity securities from the MSCI World ex USA Index (the "parent index") that have high exposure to four investment style factors: value, quality, momentum and low size, while maintaining a level of risk similar to that of the parent index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.87883	2020-10-11 23:38:31.025871
INVE	Identiv Inc.	Identiv Inc provides secure identification solutions in the connected world, including premises, as well as information centers.	{Technology,"Computer Systems","Computer Hardware"}	{XLK}	Computer Hardware	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:04.895921	2020-10-11 23:38:31.023428
IOO	iShares Global 100	The investment seeks to track the investment results of the S&P Global 100TM (the "underlying index"), which is designed to measure the performance of the stocks of 100 large-capitalization global companies.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index, but which the advisor believes will help the fund track the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.914344	2020-10-11 23:38:31.021426
IOSP	Innospec Inc.	Innospec Inc develops, manufactures, blends, markets, and supplies fuel additives, oilfield chemicals, personal care and other specialty chemicals. Its segments are Fuel Specialties; Performance Chemicals, Oilfield Services, and Octane Additives.	{"Basic Materials","Specialty Chemicals",Chemicals}	{NTIC,ALB,GRA,SXT,NEU,HUN}	Chemicals	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:12:04.922174	2020-10-11 23:38:31.024353
IPB	Merrill Lynch & Co. Inc. 6.0518% Index Plus Trust Certificates Series 2003-1		{}	{}			NYSE American	\N	2020-10-11 23:12:04.933391	2020-10-11 23:38:31.023007
IPE	SPDR Bloomberg Barclays TIPS	The investment seeks to provide investment results that correspond generally to the price and yield performance of the Bloomberg Barclays U.S. Government Inflation-Linked Bond Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index or in securities that the adviser determines have economic characteristics that are substantially identical to the economic characteristics of the securities that comprise the index. The index is designed to measure the performance of the inflation protected public obligations of the U.S. Treasury, commonly known as "TIPS." It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.941156	2020-10-11 23:38:31.024353
IMPV	Imperva Inc.	Imperva Inc is a provider of cyber-security solutions that protect business-critical data and applications whether in the cloud or on premises. Its products include database activity monitor, database firewall, and others.	{Technology,"Software - Application","Application Software"}	{INTC,CTXS,FFIV,SYMC,IBM,QLYS,ORCL,ATEN,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:04.772423	2020-10-11 23:38:31.021426
INB	Cohen & Steers Global Income Builder Inc. of Beneficial Interest	Cohen & Steers Global Inc Builder, Inc is a diversified closed-end management investment company. Its investment objective is total return with an emphasis on high current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.786645	2020-10-11 23:38:31.021426
INFI	Infinity Pharmaceuticals Inc.	Infinity Pharmaceuticals Inc is a biopharmaceutical company focused on developing novel medicines for the treatment of specific cancers. It is focused on advancing IPI-549, an orally administered, clinical-stage, immuno-oncology product candidate.	{Healthcare,Biotechnology}	{AMGN,GILD,SGEN,NVS,BMY,PFE,ABBV,MDGL,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:04.812159	2020-10-11 23:38:31.025871
INN	Summit Hotel Properties Inc.	Summit Hotel Properties Inc is a self-managed hotel investment company. The Company is engaged in acquiring and owning premium-branded select-service hotels in the upscale and upper midscale segments of the U.S. lodging industry.	{"Real Estate","REIT - Hotel & Motel",REITs}	{LHO,DRH,PEB,CHSP,RLJ,SOHO,HT,HPT,CDOR}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:04.837242	2020-10-11 23:38:31.025871
INR	Market Vectors Indian Rupee/USD ETN	The investment seeks to replicate, net of expenses, the S&P Indian Rupee Total Return Index.\n The index tracks the value of the Indian rupee relative to the U.S. dollar. It follows the value of non-deliverable, three-month currency forward contracts that are rolled at three-month intervals and includes daily accrued interest.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.857562	2020-10-11 23:38:31.024353
INSI	Insight Select Income Fund	Rivus Bond Fund is a diversified closed-end, management investment company. The objectives of the fund is to maintain a high level of income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.867348	2020-10-11 23:38:31.031328
INTU	Intuit Inc.	Intuit Inc creates business and financial management solutions for small businesses, consumers, and accounting professionals. The company's products and services include QuickBooks and TurboTax.	{Technology,"Software - Application","Application Software"}	{ADBE,ADP,FISV,PAYX,SYMC,NUAN,SAP,HRB,FIS,ORCL,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:04.889216	2020-10-11 23:38:31.021426
IO	Ion Geophysical Corporation	ION Geophysical Corp provides a variety of seismic products and services, including seismic acquisition equipment, navigation and data-management software products, seismic data processing services to oil and natural gas industry.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{DWSN}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:04.908407	2020-10-11 23:38:31.023428
IOVA	Iovance Biotherapeutics Inc.	Iovance Biotherapeutics Inc is a biopharmaceutical company in the United States. Its lead pipeline candidate, LN-144, uses the tumor-infiltrating lymphocytes to treat serious diseases.	{Healthcare,Biotechnology}	{HUBG,EXPD,FWRD,RRTS,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:04.922799	2020-10-11 23:38:31.028166
IPAR	Inter Parfums Inc.	Inter Parfums Inc operates in the fragrance business and manufactures, markets and distributes fragrances and fragrance related products.	{"Consumer Defensive","Household & Personal Products","Consumer Packaged Goods"}	{EL,AVP,CAW,GPS,SBH,REV,ASNA,COTY,BEBE}	Consumer Packaged Goods	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:04.92666	2020-10-11 23:38:31.025871
IPFF	iShares International Preferred Stock	The investment seeks to track the investment results of the S&P International Preferred Stock IndexTM.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The index measures the performance of a select group of preferred stocks from non-U.S. developed market issuers and traded on non-U.S. developed market exchanges as defined by the index provider. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:04.940376	2020-10-11 23:38:31.028166
IPG	Interpublic Group of Companies Inc. (The)	The Interpublic Group of Companies Inc is an advertising and marketing service company. Its services include consumer advertising, digital marketing, communications planning & media buying, public relations & specialized communications disciplines.	{"Consumer Cyclical","Advertising Agencies","Advertising & Marketing Services"}	{MDCA,EBAY,NCMI,OMC,ADS,LAMR}	Advertising & Marketing Services	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:04.942657	2020-10-11 23:38:31.028166
INDL	Direxion Daily MSCI India Bull 3x Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the daily performance of the MSCI India Index.\n The fund, under normal circumstances, invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, exchange-traded funds ("ETFs") that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index is designed to measure the performance of the large- and mid-capitalization segments of the Indian equity market, covering approximately 85% of the Indian equity universe. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.801188	2020-10-11 23:38:31.028166
INSE	Inspired Entertainment Inc.	Inspired Entertainment Inc is a gaming technology company. It offers Virtual Sports, Mobile Gaming and Server-Based Gaming systems to regulated betting, gaming and lottery operators around the world.	{"Consumer Cyclical",Gambling,"Travel & Leisure"}	{}	Travel & Leisure	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:04.862256	2020-10-11 23:38:31.023007
INVA	Innoviva Inc.	Innoviva Inc is a biopharmaceutical company that is engaged in the discovery, development and commercialization of small molecule medicines. The firm collaborates with and receives funding from GlaxoSmithKline.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:04.895277	2020-10-11 23:38:31.028166
INWK	InnerWorkings Inc.	InnerWorkings Inc is a marketing supply chain company that provides marketing execution solutions including advertising materials, branded merchandise, product packaging, and retail displays.	{"Consumer Cyclical","Marketing Services","Advertising & Marketing Services"}	{LABL,CMPR,DLX,EBF,QUAD,TGNA,RRD}	Advertising & Marketing Services	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:04.899839	2020-10-11 23:38:31.024353
IOR	Income Opportunity Realty Investors Inc.	Income Opportunity Realty Investors Inc invests in equity interests in real estate through direct equity investments and partnerships and finances real estate and real estate-related activities through investments in mortgage loans.	{"Real Estate","Real Estate - General","Real Estate Services"}	{GRIF,STRS,TCI,CTO}	Real Estate Services	Real Estate	NYSE American	\N	2020-10-11 23:12:04.923346	2020-10-11 23:38:31.025871
IPAS	iPass Inc.	iPass Inc provides global mobile connectivity, offering Wi-Fi access on any mobile device on the Wi-Fi network. The iPass SmartConnect platform takes the guesswork out of Wi-Fi, automatically connecting customers to the best hotspot for their needs.	{"Communication Services","Telecom Services"}	{}	Communication Services	Communication Services	OTC Pink	\N	2020-10-11 23:12:04.926855	2020-10-11 23:38:31.023428
IPDN	Professional Diversity Network Inc.	Professional Diversity Network Inc is an operator of professional networks. It serves many communities, including Women, Hispanic-Americans, African-Americans, Asian-Americans, Disabled, Military Professionals, and Lesbian, Gay, Bisexual, and Transgender.	{Technology,"Internet Content & Information","Online Media"}	{MSFT,FB,TWTR,CRCM,TRUE,DHX,GOOGL,XLK}	Online Media	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:04.941037	2020-10-11 23:38:31.02423
IPGP	IPG Photonics Corporation	IPG Photonics Corp is a developer and manufacturer of fiber lasers and amplifiers for diverse applications in numerous markets. The company sells its products globally to OEMs, system integrators and end users in a wide range of diverse markets.	{Technology,"Semiconductor Equipment & Materials",Semiconductors}	{COHR,IIVI,LPTH,NOC,UTX,CIEN,VIAV,NOVT,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:04.944833	2020-10-11 23:38:31.02582
IPOA	Social Capital Hedosophia Holdings Corp. Class A par value $0.0001	Social Capital Hedosophia Holdings Corp is a blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:04.960708	2020-10-11 23:38:31.023428
IPOS	Renaissance Capital Greenwich Fund	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the Renaissance International IPO Index.\n The fund normally invests at least 80% of its total assets in securities that comprise the index. The index is comprised of common stocks, depositary receipts, real estate investment trusts ("REITs") and partnership units. The fund may also invest up to 20% of its assets in certain futures, options, and swap contracts, cash and cash equivalents, as well as in common stocks not included in the index but which will help the fund track the index. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.962024	2020-10-11 23:38:31.028166
INFY	Infosys Limited American Depositary Shares	Infosys Ltd is engaged in consulting, technology, outsourcing and next-generation services. It offers software re-engineering, maintenance, systems integration, package evaluation and implementation and infrastructure management services.	{Technology,"Information Technology Services","Application Software"}	{MSFT,CTSH,SAP,IBM,WIT,ACN,INFY,ORCL,HPQ,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:04.832383	2020-10-11 23:38:31.028166
INOD	Innodata Inc.	Innodata Inc provides knowledge process outsourcing services, and publishing and related information technology services that help media and information services companies create, manage and maintain their products.	{Technology,"Information Technology Services","Application Software"}	{MSFT,EXLS,WNS,BR,AXR,XRX,XLK}	Application Software	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:04.853215	2020-10-11 23:38:31.031328
INTL	INTL FCStone Inc.	INTL FCStone Inc is a financial services organization. It is engaged in providing financial products and advisory and execution services that help its clients access market liquidity, maximize profits and manage risk.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{BGCP,ITG,GCAP}	Brokers & Exchanges	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:04.883098	2020-10-11 23:38:31.023007
INXN	InterXion Holding N.V. (0.01 nominal value)	InterXion Holding NV provides cloud & carrier-neutral colocation data centre services in Europe. It offers services including space, power, cooling & a secure environment in which to house its customers' computing, network, storage and IT infrastructure.	{Technology,"Internet Content & Information","Online Media"}	{AKAM,LLNW,IBM,EQIX,HPQ,INAP,XLK}	Online Media	Technology	New York Stock Exchange	\N	2020-10-11 23:12:04.900877	2020-10-11 23:38:31.031328
IONS	Ionis Pharmaceuticals Inc.	Ionis Pharmaceuticals Inc is the RNA-targeted drug discovery and development company. Its broad pipeline consists of close to 32 drugs to treat a wide variety of diseases with an emphasis on cardiovascular, metabolic, severe, and rare diseases.	{Healthcare,Biotechnology}	{SGMO,ALNY,AZN,DGX,BMY,PFE,SRPT,LXRX,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:04.912944	2020-10-11 23:38:31.023007
IOTS	Adesto Technologies Corporation	Adesto Technologies Corp is a provider of application-specific and ultra-low power non-volatile memory products. It offers four product types namely DataFlash, Fusion Flash, Mavriq and Moneta.	{Technology,"Semiconductor Memory",Semiconductors}	{QRVO,QTNA,XLK}	Semiconductors	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:04.922599	2020-10-11 23:38:31.02582
IPCI	Intellipharmaceutics International Inc.	IntelliPharmaCeutics International Inc is a pharmaceutical company. The Company is engaged in the research, development and manufacture of novel and generic controlled-release and targeted-release oral solid dosage drugs.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:04.940239	2020-10-11 23:38:31.031328
IPHI	Inphi Corporation $0.001 par value	Inphi Corp is engaged in designing, development and sale of high-speed analog connectivity components that operate to maintain, amplify and improve signal integrity in a wide variety of applications.	{Technology,Semiconductors}	{CSCO,IDTI,MU,SMTC,TXN,MTSI,ADI,XLK}	Semiconductors	Technology	New York Stock Exchange	\N	2020-10-11 23:12:04.946863	2020-10-11 23:38:31.023428
IPIC	iPic Entertainment Inc.	IPic Entertainment Inc through its subsidiaries owns and operates the restaurant, full-service bar, theater auditoriums with in-theater dining venues for private events, family and business functions and other corporate-sponsored events.	{"Consumer Cyclical","Media - Diversified",Entertainment}	{}	Entertainment	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:04.959382	2020-10-11 23:38:31.023007
IPWR	Ideal Power Inc.	Ideal Power Inc develops, markets and distributes electrical power conversion products using its proprietary technology called Power Packet Switching Architecture. The company sells its product primarily to systems integrators.	{Industrials,"Diversified Industrials","Industrial Products"}	{VICR,IIN,AETI}	Industrial Products	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:04.961736	2020-10-11 23:38:31.024353
IQI	Invesco Quality Municipal Income Trust	Invesco Quality Municipal Income Trust is a diversified, closed-end management investment company. Its investment objective is to provide current income which is exempt from federal income tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.982448	2020-10-11 23:38:31.025871
IRIX	IRIDEX Corporation	IRIDEX Corp designs, manufactures, and markets laser equipment used in the ophthalmologic and aesthetic fields.	{Healthcare,"Medical Devices"}	{XLV}	Medical Devices	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:05.007478	2020-10-11 23:38:31.023007
GWB	Great Western Bancorp Inc.	Great Western Bancorp Inc is a bank holding firm. The company through its subsidiary provides relationship-based banking and agri-business banking in the United States. It mainly engages in providing loans for commercial real estate, and agricultural.	{"Financial Services","Banks - Regional - US",Banks}	{ANCX,ISBC,BANC,SSB}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.916028	2020-10-11 23:38:31.021426
GZT	Gazit-Globe Ltd.	Gazit-Globe Ltd is a developer and operator of supermarkets and shopping centers. It owns, develops and operates supermarket-anchored shopping centers in North America, Brazil, Northern Europe, Central-Eastern Europe, Israel, and other.	{"Real Estate","Real Estate Services"}	{CTO,HHC,OBAS}	Real Estate Services	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:03.93791	2020-10-11 23:38:31.031328
HAP	VanEck Vectors Natural Resources	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the VanEck® Natural Resources Index.\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The index is comprised of publicly traded companies engaged (derive greater than 50% of revenues from applicable sources) in the production and distribution of commodities and commodity-related products and services in the following sectors: 1) Agriculture; 2) Alternatives (Water & Alternative Energy); 3) Base and Industrial Metals; 4) Energy; 5) Forest Products; and 6) Precious Metals.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.960095	2020-10-11 23:38:31.02423
HBB	Hamilton Beach Brands Holding Company Class A	Hamilton Beach Brands Holding Co, through its subsidiaries, is engaged in consumer, commercial and specialty small appliances and specialty retail. The company earns maximum revenue from the sale of appliances.	{Technology,"Consumer Electronics","Computer Hardware"}	{NPK,HBB,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:03.977508	2020-10-11 23:38:31.031328
HDEF	Xtrackers MSCI EAFE High Dividend Yield Equity	The investment seeks investment results that correspond generally to the performance, before fees and expenses, of the MSCI EAFE High Dividend Yield Index.\n The fund will invest at least 80% of its total assets (but typically far more) in component securities of the underlying index. The underlying index is designed to reflect the performance of equities in its parent index, the MSCI EAFE Index, with higher dividend income and quality characteristics than average dividend yields of equities in the parent index, where such higher dividend income and quality characteristics are both sustainable and persistent.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.022371	2020-10-11 23:38:31.028166
HEDJ	WisdomTree Europe Hedged Equity Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree Europe Hedged Equity Index.\n The fund invests at least 95% of its total assets (exclusive of collateral held from securities lending) in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index provides exposure to European equity securities, particularly shares of European exporters, while at the same time neutralizing exposure to fluctuations between the value of the U.S. dollar and the euro. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.039374	2020-10-11 23:38:31.02582
HESM	Hess Midstream Partners LP Representing Limited Partner Interests	Hess Midstream Partners LP owns, operates, develops and acquires a diverse set of midstream assets to provide services to Hess and third-party customers. Its segments are Gathering, Processing and Storage and Terminaling and Export.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:04.068928	2020-10-11 23:38:31.025871
HEWW	iShares Currency Hedged MSCI Mexico	The investment seeks to track the investment results of the MSCI Mexico IMI 25/50 100% Hedged to USD Index.\n The fund generally will invest at least 90% of its assets in the component securities (including indirect investments through the underlying fund) and other instruments of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the index. The index consists of stocks traded primarily on the Mexican Stock Exchange with the currency risk inherent in the securities included in the index hedged to the U.S. dollar on a monthly basis.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.086183	2020-10-11 23:38:31.028166
HFXI	IQ 50 Percent Hedged FTSE International	The investment seeks investment results that correspond generally to the price and yield performance of its underlying index, the FTSE Developed ex North America 50% Hedged to USD Index (the "underlying index").\n The underlying index is an equity benchmark of international stocks from developed markets, with approximately half of the currency exposure of the securities included in the underlying index "hedged" against the U.S. dollar on a monthly basis. The fund invests, under normal circumstances, at least 80% of its net assets, plus the amount of any borrowings for investment purposes, in the securities and other instruments included in its underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.103122	2020-10-11 23:38:31.023007
HIFR	InfraREIT Inc.	InfraREIT Inc owns regulated electricity delivery infrastructure assets in Texas and the southwestern US. Its assets are spread across Texas which include power lines, substations, transmission towers managed externally managed by Hunt Utility Services.	{"Real Estate","REIT - Diversified",REITs}	{}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:04.130494	2020-10-11 23:38:31.023007
HIO	Western Asset High Income Opportunity Fund Inc.	Western Asset High Income Oppor Fd Inc is a diversified closed-end management investment company. The company's primary objective is to seek high current income and capital appreciation is a secondary objective.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.152312	2020-10-11 23:38:31.023428
IRCP	IRSA Propiedades Comerciales S.A.	IRSA Propiedades Comerciales SA is a real estate company. The company is engaged in holding, leasing, managing, developing, operating and acquiring shopping malls and office buildings.	{"Real Estate","Real Estate Services"}	{IRS,JLL,FOR}	Real Estate Services	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:04.99248	2020-10-11 23:38:31.028166
IRM	Iron Mountain Incorporated (Delaware)Common Stock REIT	Iron Mountain Inc provides records, information management services, and data management services, along with the complex storage and information management such as rising storage rental costs, and increased litigation, and disaster recovery requirements.	{Industrials,"Business Services"}	{ADP,FISV,EFX}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:05.014829	2020-10-11 23:38:31.02582
ISBC	Investors Bancorp Inc.	Investors Bancorp Inc operates full-service community bank in the greater New Jersey and New York metropolitan areas that offers banking products and services to consumer, business, commercial real estate, nonprofit and local government customers.	{"Financial Services","Savings & Cooperative Banks",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:05.035574	2020-10-11 23:38:31.025871
ISNS	Image Sensing Systems Inc.	Image Sensing Systems Inc develops, and markets software-based computer enabled detection products for use in traffic, safety, security, police and parking applications. The company operates through two segments namely Intersection and Highway.	{Technology,"Software - Application","Application Software"}	{XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:05.054463	2020-10-11 23:38:31.023007
ITCI	Intra-Cellular Therapies Inc.	Intra-Cellular Therapies Inc is a biopharmaceutical company. It is engaged in discovery and clinical development of small molecule drugs that address underserved medical needs in neuropsychiatric and neurological disorders.	{Healthcare,Biotechnology}	{PDLI,MNOV,AGTC,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:05.077793	2020-10-11 23:38:31.02423
ITRN	Ituran Location and Control Ltd.	Ituran Location and Control Ltd offers location-based services, such as stolen vehicle recovery & tracking services. It also provides wireless communication products in connection to location-based services & fleet management, & among others.	{Technology,"Communication Equipment"}	{MAGS,PNTR,NSSC,TRMB,RNET,JCI,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:05.099123	2020-10-11 23:38:31.02423
IVE	iShares S&P 500 Value	The investment seeks to track the investment results of the S&P 500 Value IndexTM, which measures the performance of the large-capitalization value sector of the U.S. equity market.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index, but which the advisor believes will help the fund track the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.11723	2020-10-11 23:38:31.023007
IVR	INVESCO MORTGAGE CAPITAL INC	Invesco Mortgage Capital Inc is a REIT which invests, finances, and manages residential and commercial mortgage-backed securities and mortgage loans. The company's portfolio consists of residential and commercial mortgage-backed securities.	{"Real Estate","REIT - Diversified",REITs}	{}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:05.140244	2020-10-11 23:38:31.02582
IWN	iShares Russell 2000 Value	The investment seeks to track the investment results of the Russell 2000 Value Index, which measures the performance of the small-capitalization value sector of the U.S. equity market.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index, but which the advisor believes will help the fund track the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.160381	2020-10-11 23:38:31.023428
IX	Orix Corp Ads	ORIX Corp is a diversified financial services company with operations in Corporate Financial Services, Maintenance Leasing, Real Estate, Investment and Operation, Retail and Overseas Business.	{"Financial Services","Credit Services"}	{CACC,NMR,SC}	Credit Services	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.175598	2020-10-11 23:38:31.023007
IYH	iShares U.S. Healthcare	The investment seeks to track the investment results of the Dow Jones U.S. Health Care Index composed of U.S. equities in the healthcare sector.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index measures the performance of the healthcare sector of the U.S. equity market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.229669	2020-10-11 23:38:31.028166
IZRL	ARK Israel Innovative Technology	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the ARK Israeli Innovation Index ("index").\n The fund normally invests at least 80% of its total assets in securities that are included in the fund's benchmark index, depositary receipts representing securities included in the index or underlying stocks in respect of depositary receipts included in the index. The index includes equity securities and depositary receipts of exchange listed companies that are incorporated and/or domiciled in Israel ("Israeli Companies"). The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:05.256106	2020-10-11 23:38:31.02582
JBGS	JBG SMITH Properties	JBG SMITH Properties is a real estate company that owns, manages, and develops high-quality, mixed use real estate located in urban, infill locations.	{"Real Estate","REIT - Diversified",REITs}	{NEN,MMI}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:05.271604	2020-10-11 23:38:31.031328
IRDM	Iridium Communications Inc	Iridium Communications Inc is a United States based company which is engaged in providing mobile voice and data communications services through a constellation of low earth orbiting satellites.	{"Communication Services","Telecom Services"}	{}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:12:04.996921	2020-10-11 23:38:31.023428
IRT	Independence Realty Trust Inc.	Independence Realty Trust Inc is a real estate investment trust. It owns apartment properties in geographic non-gateway markets. The company generates its revenue in the form of Rental income, Reimbursement and other income.	{"Real Estate","REIT - Residential",REITs}	{EQR,MAA,AIV,AVB,UDR}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:05.023092	2020-10-11 23:38:31.021426
ISF	ING Group N.V. Perp Hybrid Cap Secs (Netherlands)	ING Groep NV operates in the financial services domain. Its key areas of operation include retail and commercial banking.	{"Financial Services","Banks - Global",Banks}	{}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.043117	2020-10-11 23:38:31.024353
ISTB	iShares Core 1-5 Year USD Bond ETF	The investment seeks to track the investment results of the Bloomberg Barclays U.S. Universal 1-5 Year Index, which measures the performance of U.S. dollar-denominated taxable bonds that are rated either investment-grade or high yield with remaining effective maturities between one and five years.\n The fund invests at least 90% of its assets in the component securities of the index and in investments that have economic characteristics that are substantially identical to the component securities and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:05.071102	2020-10-11 23:38:31.028166
ITCB	Itau CorpBanca American Depositary Shares (each representing 1500 shares of no par value)	Itau Corpbanca provides commercial and retail banking services to its customers. The Company, through its subsidiaries, provides financial advisory services, mutual fund management, insurance brokerage and securities brokerage services.	{"Financial Services","Banks - Regional - Latin America",Banks}	{}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.076454	2020-10-11 23:38:31.028166
ITI	Iteris Inc.	Iteris Inc provides intelligent information solutions to the traffic management & global agribusiness market. The Company is engaged in the development and application of technologies and software-based information systems that reduce traffic congestion.	{Technology,"Communication Equipment"}	{CALX,VCRA,UTSI,XLK}	Communication Equipment	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:05.094166	2020-10-11 23:38:31.021426
IUSG	iShares Core S&P U.S. Growth ETF	The investment seeks to track the investment results of the S&P 900 Growth Index (the "underlying index").\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index measures the performance of the large- and mid- capitalization growth sector of the U.S. equity market.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:05.113399	2020-10-11 23:38:31.023007
IVAL	Alpha Architect International Quantitative Value	The investment seeks to track the total return performance, before fees and expenses, of the Alpha Architect International Quantitative Value Index.\n The fund normally will invest at least 80% of its total assets in the component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index uses a 5-step, quantitative, rules-based methodology to identify a portfolio of approximately 40-50 undervalued non-U.S. equity securities or their depositary receipts with the potential for capital appreciation.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:05.116722	2020-10-11 23:38:31.024353
IVOO	Vanguard S&P Mid-Cap 400	The investment seeks to track the performance of a benchmark index that measures the investment return of mid-capitalization stocks in the United States.\n The fund employs an indexing investment approach designed to track the performance of the S&P MidCap 400® Index. The index measures the performance of mid-capitalization stocks in the United States. The Advisor attempts to replicate the target index by investing all, or substantially all, of its assets in the stocks that make up the index, holding each stock in approximately the same proportion as its weighting in the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.134526	2020-10-11 23:38:31.028166
IVV	iShares Core S&P 500	The investment seeks to track the investment results of the S&P 500 (the "underlying index"), which measures the performance of the large-capitalization sector of the U.S. equity market.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index, but which the advisor believes will help the fund track the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.143148	2020-10-11 23:38:31.023428
IRDMB	Iridium Communications Inc 6.75% Series B Cumulative Perpetual Convertible Preferred Stock	Iridium Communications Inc is a United States based company which is engaged in providing mobile voice and data communications services through a constellation of low earth orbiting satellites.	{"Communication Services","Telecom Services"}	{}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:12:05.002165	2020-10-11 23:38:31.028166
IRTC	iRhythm Technologies Inc.	iRhythm Technologies Inc is a commercial-stage digital healthcare company redefining the way cardiac arrhythmias are clinically diagnosed by combining wearable biosensing technology with cloud-based data analytics and machine-learning capabilities.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{XLV}	Medical Instruments & Equipment	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:05.026194	2020-10-11 23:38:31.023007
ISHG	iShares S&P/Citigroup 1-3 Yr Intl Treasury Bond	The investment seeks to track the investment results of the S&P International Sovereign Ex-U.S. 1-3 Year Bond Index.\n The fund will invest at least 90% of its assets in the component securities of the index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The index is a broad, market value weighted index designed to measure the performance of sovereign bonds issued in local currencies by developed market countries, as classified by SPDJI, outside the U.S. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:05.048653	2020-10-11 23:38:31.021426
ISTR	Investar Holding Corporation	Investar Holding Corp is a financial holding company. It is engaged in offering commercial and retail lending products throughout market areas, including business loans to small to medium-sized businesses as well as loans to individuals.	{"Financial Services","Banks - Regional - US",Banks}	{CCBG,SFBC,SBFG}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:05.071362	2020-10-11 23:38:31.021426
ITGR	Integer Holdings Corporation	Integer Holdings Corp is the medical device outsource manufacturer serving cardiac, neuromodulation, orthopedics,vascular and surgical markets. The company's brands include Greatbatch Medical, Lake Region Medical and Electrochem.	{Healthcare,"Medical Devices"}	{BSX,JNJ,SYK,EW,ZBH,SPA,MDT,XLV}	Medical Devices	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:05.091856	2020-10-11 23:38:31.023007
IVAC	Intevac Inc.	Intevac Inc provides thin film solutions. It provides vacuum deposition equipment for a range of thin-film applications and digital night-vision technologies and products to the defense industry.	{Technology,"Electronic Components","Computer Hardware"}	{AMAT,CVV,VECO,ASYS,CAJ,RTN,COL,WDC,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:05.112754	2020-10-11 23:38:31.028166
IWD	iShares Russell 1000 Value	The investment seeks to track the investment results of the Russell 1000® Value Index (the "underlying index"), which measures the performance of large- and mid- capitalization value sectors of the U.S. equity market.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index, but which the advisor believes will help the fund track the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.152659	2020-10-11 23:38:31.028166
IWP	iShares Russell Midcap Growth	The investment seeks to track the investment results of the Russell Midcap Growth Index, which measures the performance of the mid-capitalization growth sector of the U.S. equity market.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index, but which the advisor believes will help the fund track the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.165445	2020-10-11 23:38:31.021426
IXN	iShares Global Tech	The investment seeks to track the investment results of the S&P Global 1200 Information Technology IndexTM.\n The fund generally invests at least 90% of its assets in securities of the index and in depositary receipts representing securities of the index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the index. The index measures the performance of companies that the index provider deems to be part of the information technology sector of the economy and that the index provider believes are important to global markets. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.183582	2020-10-11 23:38:31.021426
KBH	KB Home	KB Home is engaged in the residential construction market. The Company's offers a variety of new homes, move-up and active adult homebuyers, including attached and detached single-family residential homes.	{"Consumer Cyclical","Residential Construction","Homebuilding & Construction"}	{PHM,MHO,NVR,DHI,HOV,LEN,MDC,WLH}	Homebuilding & Construction	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:05.678239	2020-10-11 23:38:31.02423
IRL	New Ireland Fund Inc (The)	New Ireland Fund Inc is a non-diversified, closed-end management investment company. The fund's investment objective is long-term capital appreciation through investment primarily in equity securities of Irish companies.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.005164	2020-10-11 23:38:31.025871
ISG	ING Group N.V. Perpetual Dent Secs 6.125%	ING Groep NV operates in the financial services domain. Its key areas of operation include retail and commercial banking.	{"Financial Services","Banks - Global",Banks}	{}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.043674	2020-10-11 23:38:31.028166
ISSC	Innovative Solutions and Support Inc.	Innovative Solutions and Support Inc is a systems integrator that manufactures, sells, & services air data equipment, engine display systems, standby equipment, primary flight guidance & cockpit display systems for retrofit applications & OEM.	{Industrials,"Aerospace & Defense"}	{RADA,ATRO,WWD,BA,ESL,COL,TDG,AIRI,LLL}	Aerospace & Defense	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:05.068436	2020-10-11 23:38:31.02423
ITG	Investment Technology Group Inc.	Investment Technology Group Inc is an execution and research broker that partners with portfolio managers and traders to provide financial technology and data-driven insights throughout the investment process.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{BGCP,RJF,SCHW,INTL}	Brokers & Exchanges	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.08939	2020-10-11 23:38:31.023428
IVC	Invacare Corporation	Invacare Corp is a manufacturer and distributor of medical equipment used in non-acute care settings. The company sells its products to home medical equipment providers with retail and e-commerce channels, residential care operators, and distributors.	{Healthcare,"Medical Devices"}	{OMI,SYK,TFX,HRC,INGN,STE,SNOA,AHPI,XLV}	Medical Devices	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:05.116296	2020-10-11 23:38:31.025871
IVOV	Vanguard S&P Mid-Cap 400 Value	The investment seeks to track the performance of a benchmark index that measures the investment return of mid-capitalization value stocks in the United States.\n The fund employs an indexing investment approach designed to track the performance of the S&P MidCap 400® Value Index, which represents the value companies, as determined by the index sponsor, of the S&P MidCap 400 Index. The Advisor attempts to replicate the target index by investing all, or substantially all, of its assets in the stocks that make up the index, holding each stock in approximately the same proportion as its weighting in the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.135682	2020-10-11 23:38:31.023007
IWF	iShares Russell 1000 Growth	The investment seeks to track the investment results of the Russell 1000® Growth Index, which measures the performance of large- and mid-capitalization growth sectors of the U.S. equity market.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index, but which the advisor believes will help the fund track the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.15608	2020-10-11 23:38:31.028166
IWO	iShares Russell 2000 Growth	The investment seeks to track the investment results of the Russell 2000 Growth Index, which measures the performance of the small-capitalization growth sector of the U.S. equity market.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index, but which the advisor believes will help the fund track the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.164214	2020-10-11 23:38:31.025871
IWY	iShares Russell Top 200 Growth	The investment seeks to track the investment results of the Russell Top 200 Growth Index, which measures the performance of the largest capitalization growth sector of the U.S. equity market.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index, but which the advisor believes will help the fund track the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.174517	2020-10-11 23:38:31.024353
KBLMU	KBL Merger Corp. IV Unit	KBL Merger Corp IV is a blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:05.680106	2020-10-11 23:38:31.024353
IRS	IRSA Inversiones Y Representaciones S.A.	IRSA Inversiones y Representaciones SA is a diversified Argentine real estate firm. It property portfolio includes shopping centers, hotels, office buildings and residential development.	{"Real Estate","Real Estate - General","Real Estate Services"}	{EMITF,IRCP,FCE.A,IHG}	Real Estate Services	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:05.027448	2020-10-11 23:38:31.028166
ISIG	Insignia Systems Inc.	Insignia Systems Inc markets in-store advertising products, programs and services to retailers and consumer packaged goods manufacturers.	{"Consumer Cyclical","Marketing Services","Advertising & Marketing Services"}	{SGRP,NCMI,AUTO}	Advertising & Marketing Services	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:05.050393	2020-10-11 23:38:31.023428
ISZE	iShares Edge MSCI Intl Size Factor	The investment seeks to track the investment results of the MSCI World ex USA Risk Weighted Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index. The index is based on a traditional market capitalization-weighted parent index, the MSCI World ex USA Index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.07149	2020-10-11 23:38:31.023007
ITEQ	BlueStar Israel Technology	The investment seeks to provide investment results that correspond generally to the total return performance of the TASE-BlueStar Israel Global Technology Index™ ("TA-BIGITech™").\n The fund will invest at least 80% of its total assets in the component securities of the index and in depositary receipts representing such securities. As a result, normally the fund will invest at least 80% of its total assets in Israeli technology companies. TA-BIGITech™ was created in 2013 by BlueStar Global Investors LLC d/b/a BlueStar Indexes® and the ISE and tracks the performance of exchange-listed Israeli technology operating companies. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.089637	2020-10-11 23:38:31.02582
ITW	Illinois Tool Works Inc.	Illinois Tool Works Inc manufactures diversified range of industrial products and equipment. Its segments are Automotive OEM; Food Equipment; Test & Measurement and Electronics; Welding; Polymers & Fluids; Construction Products; and Specialty Products.	{Industrials,"Diversified Industrials","Industrial Products"}	{LECO,MIDD,NDSN,FLS,DOV,GGG,PH,MMM,IR,SWK}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:05.108669	2020-10-11 23:38:31.023428
IVOG	Vanguard S&P Mid-Cap 400 Growth	The investment seeks to track the performance of a benchmark index that measures the investment return of mid-capitalization growth stocks in the United States.\n The fund employs an indexing investment approach designed to track the performance of the S&P MidCap 400® Growth Index, which represents the growth companies, as determined by the index sponsor, of the S&P MidCap 400 Index. The Advisor attempts to replicate the target index by investing all, or substantially all, of its assets in the stocks that make up the index, holding each stock in approximately the same proportion as its weighting in the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.130703	2020-10-11 23:38:31.021426
IVZ	Invesco Ltd	Invesco Ltd is an independent investment management firm. The company provides retail and institutional clients with an array of investment management capabilities. It has presence in the institutional and retail markets.	{"Financial Services","Asset Management"}	{AMG,LM,BLK,CNS,LAZ,AB,BX,GBL,OAK,CG}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.150918	2020-10-11 23:38:31.025871
IWR	iShares Russell Mid-Cap	The investment seeks to track the investment results of the Russell Midcap Index, which measures the performance of the mid-capitalization sector of the U.S. equity market.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index, but which the advisor believes will help the fund track the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.16687	2020-10-11 23:38:31.031328
IYLD	iShares Morningstar Multi-Asset Income	The investment seeks to track the investment results of the Morningstar® Multi-Asset High Income IndexSM.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The index is broadly diversified and seeks to deliver high current income while maintaining long-term capital appreciation. The fund is a fund-of-funds and invests primarily in the securities of the underlying funds that themselves seek investment results corresponding to their own underlying indexes.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:05.238357	2020-10-11 23:38:31.021426
IRMD	iRadimed Corporation	iRadimed Corp develops, manufactures, markets and distributes Magnetic Resonance Imaging compatible products to acute care facilities and outpatient imaging centers, both in the United States and internationally.	{Healthcare,"Medical Devices"}	{DXCM,ATRC,COO,EW,XLV}	Medical Devices	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:05.015697	2020-10-11 23:38:31.024353
ISD	PGIM Short Duration High Yield Fund Inc.	Prudential Short Term High Yield Fund is active in the financial services domain. It invests primarily in a diversified portfolio of below investment grade bonds.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.041987	2020-10-11 23:38:31.02582
ISRA	VanEck Vectors Israel	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the BlueStar Israel Global Index®.\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The index is comprised of equity securities, which may include depositary receipts, of publicly traded companies that are generally considered by BlueStar Global Investors, LLC to be Israeli companies. It may also utilize depositary receipts to seek performance that corresponds to the fund's benchmark index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.058654	2020-10-11 23:38:31.024353
ITB	iShares U.S. Home Construction	The investment seeks to track the investment results of the Dow Jones U.S. Select Home Construction Index composed of U.S. equities in the home construction sector.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index measures the performance of the home construction sector of the U.S. equity market. The fund may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents. It is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:05.076346	2020-10-11 23:38:31.031328
ITRI	Itron Inc.	Itron Inc is a technology and services company. It provides solutions that measure, manage, and analyze energy and water use. Its solutions include standard meters and smart metering products, metering systems, and services.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{BMI,GE,ROP,AVA,ESE,MWA,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:05.09503	2020-10-11 23:38:31.028166
ITUB	Itau Unibanco Banco Holding SA American Depositary Shares (Each repstg 500 Preferred shares)	Itau Unibanco Holding SA operates through banks. Its services comprise of investment banking, credit card, financing, leasing, and foreign trade financing.	{"Financial Services","Banks - Regional - Latin America",Banks}	{BNS,RY,BAP,SAN,HSBC,CIB,JPM,BCH,BBD,C}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.108422	2020-10-11 23:38:31.031328
IVH	Ivy High Income Opportunities Fund of Beneficial Interest	Ivy High Income Opportunities Fund is a non-diversified, closed-end management investment company. Its investment objective is to seek to provide total return through a combination of a high level of current income and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.130447	2020-10-11 23:38:31.031328
IWB	iShares Russell 1000	The investment seeks to track the investment results of the Russell 1000® Index, which measures the performance of large- and mid- capitalization sectors of the U.S. equity market.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index, but which the advisor believes will help the fund track the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.150575	2020-10-11 23:38:31.02423
IWV	iShares Russell 3000	The investment seeks to track the investment results of the Russell 3000® Index, which measures the performance of the broad U.S. equity market.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, including shares of money market funds advised by the advisor or its affiliates, as well as in securities not included in the underlying index, but which the advisor believes will help the fund track the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.167865	2020-10-11 23:38:31.02582
IYC	iShares U.S. Consumer Services	The investment seeks to track the investment results of the Dow Jones U.S. Consumer Services Index composed of U.S. equities in the consumer services sector.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index measures the performance of the consumer services sector of the U.S. equity market.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.191321	2020-10-11 23:38:31.023007
IYR	iShares U.S. Real Estate	The investment seeks to track the investment results of the Dow Jones U.S. Real Estate Index.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying fund measures the performance of the real estate sector of the U.S. equity market and may include large-, mid- or small-capitalization companies.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.242373	2020-10-11 23:38:31.031328
ISMD	Inspire Small/Mid Cap Impact	The investment seeks to replicate investment results that generally correspond to the Inspire Small/Mid Cap Impact Index.\n The fund generally will invest at least 80% of its total assets in the component securities. The index provider selects domestic small and mid capitalization equity securities included in the Russell 2,000 Index and S&P 400 Index using the Inspire Impact Score®, a proprietary selection methodology that is designed to assign a score to a particular security based on the security's alignment with biblical values and the positive impact that company has on the world through various environmental, social and governance criterion.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.054529	2020-10-11 23:38:31.031328
IT	Gartner Inc.	Gartner Inc is an information technology research and advisory company. It offers services to business leaders in high-tech and telecom enterprises and professional services firms, to supply chain professionals and technology investors.	{Technology,"Information Technology Services","Application Software"}	{FORR,NCI,NLSN,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:05.072654	2020-10-11 23:38:31.024353
ITIC	Investors Title Company	Investors Title Co is engaged in the issuance of residential and commercial title insurance through ITIC and NITIC. It also provides tax-deferred real property exchange services, investment management and trust services.	{"Financial Services","Insurance - Specialty"}	{}	Insurance - Specialty	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:05.094971	2020-10-11 23:38:31.031328
IUSV	iShares Core S&P U.S. Value ETF	The investment seeks to track the investment results of the S&P 900 Value Index (the "underlying index"), which measures the performance of the large- and mid-capitalization value sector of the U.S. equity market.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index, but which the advisor believes will help the fund track the underlying index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:05.111396	2020-10-11 23:38:31.021426
IVFGC	Ivy Focused Growth NextShares	The investment seeks to provide growth of capital.\n The fund seeks to achieve its objective by investing primarily in a portfolio of common stocks issued by large-capitalization, growth-oriented companies with above-average levels of profitability and that the fund’s investment manager, believes have the ability to sustain growth over the long term. Although it primarily invests in securities issued by large capitalization companies, it may invest in securities issued by companies of any size. Growth-oriented companies are those whose earnings IICO believes are likely to grow faster than the economy. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:05.129967	2020-10-11 23:38:31.02582
IWL	iShares Russell Top 200	The investment seeks to track the investment results of the Russell Top 200® Index, which measures the performance of the largest capitalization sector of the U.S. equity market.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents, including shares of money market funds advised by the advisor or its affiliates, as well as in securities not included in the underlying index, but which the advisor believes will help the fund track the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.158121	2020-10-11 23:38:31.031328
IXC	iShares Global Energy	The investment seeks to track the S&P Global 1200 Energy IndexTM.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index. The index measures the performance of companies that the index provider deems to be part of the energy sector of the economy and that the index provider believes are important to global markets. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.178386	2020-10-11 23:38:31.028166
IYG	iShares U.S. Financial Services	The investment seeks to track the investment results of the Dow Jones U.S. Financial Services Index composed of U.S. equities in the financial services sector.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index measures the performance of the financial services sector of the U.S. equity market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.229525	2020-10-11 23:38:31.024353
IYT	iShares Transportation Average	The investment seeks to track the investment results of the Dow Jones Transportation Average Index composed of U.S. equities in the transportation sector.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index measures the performance of large, well-known companies within the transportation sector of the U.S. equity market. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:05.242796	2020-10-11 23:38:31.028166
IYY	iShares Dow Jones U.S.	The investment seeks to track the investment results of the Dow Jones U.S. Index composed of U.S. equities.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index aims to consistently represent the top 95% of U.S. companies based on a float-adjusted market capitalization, excluding the very smallest and least liquid stocks.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.251678	2020-10-11 23:38:31.028166
ISCF	iShares Edge MSCI Intl Size Factor	The investment seeks to track the investment results of the MSCI World ex USA Small Cap Diversified Multiple-Factor Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The underlying index is designed to select equity securities from the MSCI World ex USA Small Cap Index that have high exposure to four investment style factors: value, quality, momentum and low size, while maintaining a level of risk similar to that of the parent index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.042011	2020-10-11 23:38:31.028166
ISR	IsoRay Inc.	IsoRay Inc designs and develops personalized brachytherapy products. The company treats prostate cancer, brain, liver, breast cancer, colorectal cancer, gynecological cancer, lung cancer, ocular melanoma and pancreatic cancer.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{HOLX,ANGO,ARAY,GE,VAR,SSKN,XLV}	Medical Instruments & Equipment	Healthcare	NYSE American	\N	2020-10-11 23:12:05.05873	2020-10-11 23:38:31.025871
ITA	iShares U.S. Aerospace & Defense	The investment seeks to track the investment results of the Dow Jones U.S. Select Aerospace & Defense Index composed of U.S. equities in the aerospace and defense sector.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index measures the performance of the aerospace and defense sector of the U.S. equity market. Aerospace companies in the index include manufacturers, assemblers and distributors of aircraft and aircraft parts. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:05.076417	2020-10-11 23:38:31.025871
ITOT	iShares Core S&P Total U.S. Stock Market	The investment seeks to track the investment results of the S&P Total Market Index™ (TMI), which is comprised of the common equities included in the S&P 500® and the S&P Completion Index™.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, including shares of money market funds advised by BFA or its affiliates, as well as in securities not included in the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.095214	2020-10-11 23:38:31.024353
IUSB	iShares Core Total USD Bond Market ETF	The investment seeks to track the investment results of the Bloomberg Barclays U.S. Universal Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents, including shares of money market funds advised by BFA or its affiliates, as well as in securities not included in the underlying index, but which BFA believes will help the fund track the underlying index. The index measures the performance of U.S. dollar-denominated taxable bonds that are rated either investment-grade or high yield.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:05.111396	2020-10-11 23:38:31.02582
IVLU	iShares Edge MSCI Intl Value Factor	The investment seeks to track the investment results of the MSCI World ex USA Enhanced Value Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents as well as in securities not included in the underlying index. The index is based on a traditional market capitalization-weighted parent index, the MSCI World ex USA Index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.13037	2020-10-11 23:38:31.02423
IWC	iShares Microcap	The investment seeks to track the investment results of the Russell Microcap Index, which measures the performance of the microcap sector of the U.S. equity market.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index, but which the advisor believes will help the fund track the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.151226	2020-10-11 23:38:31.024353
IWX	iShares Russell Top 200 Value	The investment seeks to track the investment results of the Russell Top 200 Value Index, which measures the performance of the largest capitalization value sector of the U.S. equity market.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents, including shares of money market funds advised by BFA or its affiliates, as well as in securities not included in the underlying index, but which BFA believes will help the fund track the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.171323	2020-10-11 23:38:31.028166
IYF	iShares U.S. Financial	The investment seeks to track the investment results of the Dow Jones U.S. Financials Index composed of U.S. equities in the financial sector.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index measures the performance of the financial sector of the U.S. equity market.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.227606	2020-10-11 23:38:31.028166
JAKK	JAKKS Pacific Inc.	Jakks Pacific Inc is a toy and leisure products manufacturing company. Its products offering include traditional toys and electronics such as action figures, toy vehicles, dolls and accessories, ride-on toys, toys for pets.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{MAT,ROCK,NWL,DIS,HAS}	Travel & Leisure	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:05.262898	2020-10-11 23:38:31.025871
JBN	Select Asset Inc. on behalf of Corporate Backed Callable Trust Certificates J.C. Penney Debenture Backed Series 2007-1 Trust		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:05.281134	2020-10-11 23:38:31.021426
JD	JD.com Inc.	JD.com Inc is an online direct sales company in China. It acquires products from suppliers and sells them directly to its customers through its website and mobile applications.	{Technology,"Internet Content & Information","Online Media","Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{VIPS,LITB,LEJU,JMEI,BABA}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:05.312738	2020-10-11 23:38:31.02582
JHMI	John Hancock Multifactor Industrials	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the John Hancock Dimensional Industrials Index (the index).\n The fund normally invests at least 80% of its net assets (plus any borrowings for investment purposes) in securities that compose the fund's benchmark index. The index is designed to comprise securities in the industrials sector within the U.S. Universe whose market capitalizations are larger than that of the 1001st largest U.S. company at the time of reconstitution. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.361788	2020-10-11 23:38:31.025871
JILL	J. Jill Inc.	J.Jill Inc owns and operates women's clothing stores. Its brand portfolio comprises of J.Jill, The J.Jill Wearever Collection and Pure Jill under which it sells jackets, sweaters, knit tops and tees.	{"Consumer Cyclical","Apparel Stores","Retail - Apparel & Specialty"}	{LB,SFIX}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:05.380588	2020-10-11 23:38:31.021426
JJU	Barclays Bank PLC	The investment seeks to provide investors with exposure to the Dow Jones-UBS Aluminum Subindex Total ReturnService Mark.\nThe Dow Jones-UBS Aluminum Subindex Total ReturnService Mark (the "index") reflects the returns that are potentially available through an unleveraged investment in the futures contracts on aluminum. The index currently consists of one futures contract on the commodity of aluminum which is included in the Dow Jones-UBS Commodity Index Total ReturnService Mark.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.408497	2020-10-11 23:38:31.024353
JLS	Nuveen Mortgage Opportunity Term Fund	Nuveen Mortgage Opportunity Term Fund operates as a closed end management investment company. Its investment objective is to generate attractive total returns through opportunistic investments in mortgage backed securities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.479064	2020-10-11 23:38:31.025871
JMU	JMU Limited	JMU Ltd offers online platform for providing B2B services to food-industry suppliers and customers in China. It offers products through online direct sales and online marketplace. Its products include food ingredients, seasonings, furniture and other.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{}	Retail - Apparel & Specialty	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:05.513351	2020-10-11 23:38:31.021426
JP	Jupai Holdings Limited American Depositary Shares each representing six	Jupai Holdings Ltd provides wealth management products to the high net worth individuals in the People's Republic of China.	{"Financial Services","Asset Management"}	{EVH,JP}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.537681	2020-10-11 23:38:31.031328
JPI	Nuveen Preferred and Income Term Fund of Beneficial Interest	The Fund invests at least 80% of its managed assets in preferred and other income producing securities. At least 60% of its managed assets are rated investment grade (BBB/Baa or better by one of the nationally recognized statistical rating organizations “NRSROs”) at the time of investment. The Fund may invest up to 40% in securities issued by non-U.S. domiciled companies. However, all securities will be denominated in U.S. dollars. The Fund will focus on securities issued by financial and insurance firms. The Fund will employ leverage.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.56319	2020-10-11 23:38:31.031328
JPS	Nuveen Preferred & Income Securities Fund	Nuveen Preferred and Income Securities Fund is a diversified closed end management investment company. Its investment objective is to provide high current income consistent with capital preservation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.589526	2020-10-11 23:38:31.028166
JSM	Navient Corporation	SLM Corp is a saving, planning and paying for college company providing private education loans to students or their families. It operates a consumer savings network that provides financial rewards on everyday purchases to help families save for college.	{"Financial Services","Credit Services"}	{}	Credit Services	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:05.61556	2020-10-11 23:38:31.021426
JSMD	Janus Henderson Small/Mid Cap Growth Alpha ETF	The investment seeks investment results that correspond generally, to the performance of its underlying index, the Janus Small/Mid Cap Growth Alpha Index.\n The fund pursues its investment objective by normally investing at least 80% of its net assets in the securities that comprise the underlying index. The underlying index is composed of common stocks of small- and medium-sized companies that are included in the Solactive Small/Mid Cap Index, a universe of 2,500 small- and medium-sized capitalization stocks.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:05.617457	2020-10-11 23:38:31.031328
JASN	Jason Industries Inc.	Jason Industries Inc is an industrial manufacturing company operating the following four businesses: finishing, seating, acoustics and components.	{Industrials,"Diversified Industrials","Industrial Products"}	{WWD,DORM,MOD,THRM}	Industrial Products	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:05.264093	2020-10-11 23:38:31.021426
JCAP	Jernigan Capital Inc.	Jernigan Capital Inc is a real estate finance company. It offers capital to private developers, owners, and operators of self-storage facilities.	{"Real Estate","REIT - Industrial",REITs}	{}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:05.294197	2020-10-11 23:38:31.031328
JE	Just Energy Group Inc. (Canada)	Just Energy Group Inc is a retail energy provider specializing in electricity and natural gas commodities, energy efficiency solutions and renewable energy options.	{Utilities,"Utilities - Diversified","Utilities - Regulated"}	{ETR,AVA,POR,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:05.316827	2020-10-11 23:38:31.023007
JHD	Nuveen High Income December 2019 Target Term Fund of Beneficial Interest	Nuveen High Income Dec 2019 Target Term is a diversified, closed-end management investment company. Its investment objective is to provide a high level of current income & return the original net asset value per common share on or about December 1, 2019.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.337677	2020-10-11 23:38:31.02582
JHMT	John Hancock Multifactor Technology	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the John Hancock Dimensional Technology Index.\n The fund normally invests at least 80% of its net assets (plus any borrowings for investment purposes) in securities that compose the fund's benchmark index. The index is designed to comprise securities in the technology sector within the U.S. Universe whose market capitalizations are larger than that of the 1001st largest U.S. company at the time of reconstitution. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.373349	2020-10-11 23:38:31.02582
JJS	Barclays Bank PLC	The investment seeks to provide investors with exposure to the Dow Jones-UBS Softs Subindex Total ReturnService Mark. \nThe Dow Jones-UBS Softs Subindex Total ReturnService Mark (the "index") reflects the returns that are potentially available through an unleveraged investment in the futures contracts on soft commodities. The index is currently composed of three soft commodities contracts (coffee, cotton and sugar) which are included in the Dow Jones-UBS Commodity Index Total ReturnService Mark.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.397495	2020-10-11 23:38:31.031328
JKJ	iShares Morningstar Small-Cap	The investment seeks to track the investment results of the Morningstar® Small Core IndexSM composed of small-capitalization U.S. equities.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index measures the performance of stocks issued by small-capitalization companies that have exhibited average "growth" and "value" characteristics as determined by Morningstar, Inc.'s ("Morningstar" or the "index provider") proprietary index methodology.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.454403	2020-10-11 23:38:31.023007
JMLP	Nuveen All Cap Energy MLP Opportunities Fund of Beneficial Interest	Nuveen All Cap Energy MLP Opportunities Fund is a non-diversified closed-end management investment company. Its investment objective is to provide a high level of total return including current distributions and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.49392	2020-10-11 23:38:31.031328
JNJ	Johnson & Johnson	Johnson & Johnson is a holding company, which is engaged in the research and development, manufacture and sale of products in the health care field within its Consumer, Pharmaceutical and Medical Devices segments.	{Healthcare,"Drug Manufacturers - Major","Drug Manufacturers"}	{GSK,NVS,ABT,LLY,PFE,SYK,SNY,MRK,MDT,XLV}	Drug Manufacturers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:05.517093	2020-10-11 23:38:31.02582
JOUT	Johnson Outdoors Inc.	Johnson Outdoors Inc is a global manufacturer and marketer of branded seasonal, outdoor recreation products. It offers products under Watercraft, Diving, Marine Electronics and Outdoor Gear.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{ESCA,ELY,NLS}	Travel & Leisure	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:05.537889	2020-10-11 23:38:31.023428
JPHF	J P Morgan Chase & Co	The investment seeks to provide long-term total return.\n The fund will seek to achieve its investment objective by allocating assets across several different investment strategies, including traditional and alternative investment strategies, such as those utilized by certain hedge funds. The strategies identified by the adviser for the fund fall into the following broad categories: Equity Long/Short, Event Driven and Macro/Managed Futures strategies. The fund will invest its assets based on a systematic investment process for securities selection and asset allocation.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.55669	2020-10-11 23:38:31.024353
JPN	Xtrackers Japan JPX-Nikkei 400 Equity	The investment seeks investment results that correspond generally to the performance, before fees and expenses, of the JPX-Nikkei 400 Net Total Return Index.\n The index is designed to track the performance of equity securities of issuers who are primarily listed on the JASDAQ Stock Exchange or the following sections of the Tokyo Stock Exchange: the first section (for large companies), the second section (for middle-sized companies) and the "Mothers" (for high-growth and emerging stocks). The fund will invest at least 80% of its total assets in component securities of the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.576571	2020-10-11 23:38:31.024353
JBSS	John B. Sanfilippo & Son Inc.	John B Sanfilippo & Son Inc is a processor and distributor of peanuts, pecans, cashews, walnuts, almonds and other nuts in the United States. It also markets and distributes a diverse product line of food and snack products.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{HSY,KHC}	Consumer Packaged Goods	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:05.288275	2020-10-11 23:38:31.02582
JCS	Communications Systems Inc.	Communications Systems Inc, together with its subsidiaries, manufactures and sells modular connecting and wiring devices, digital subscriber line filters, structured wiring systems, and media and rate conversion products in North America and EMEA region.	{Technology,"Communication Equipment"}	{CSCO,SILC,ADTN,NTGR,LTRX,GE,COMM,ANET,HPQ,ARRS,XLK}	Communication Equipment	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:05.306371	2020-10-11 23:38:31.023428
JEMD	Nuveen Emerging Markets Debt 2022 Target Term Fund of Beneficial Interest $0.01 par value per share	Nuveen Emrg Mkts Debt 2022 Target Term is a non-diversified, closed-end management investment company. Its investment objectives is to provide a high level of current income and to return $9.85 per share per Common Share.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.323806	2020-10-11 23:38:31.02423
JHMC	John Hancock Multifactor Consumer Discretionary	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the John Hancock Dimensional Consumer Discretionary Index.\n The fund normally invests at least 80% of its net assets (plus any borrowings for investment purposes) in securities that compose the fund's benchmark index. The index is designed to comprise securities in the consumer discretionary sector within the U.S. Universe whose market capitalizations are larger than that of the 1001st largest U.S. company at the time of reconstitution. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.348352	2020-10-11 23:38:31.023428
JHS	John Hancock Income Securities Trust	John Hancock Income Securities Trust is a United States based closed-end, diversified management investment company. Its primary objective is to generate a high level of current income consistent with prudent investment risk.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.375123	2020-10-11 23:38:31.023007
JJP	Barclays Bank PLC	The investment seeks to provide investors with exposure to the Dow Jones-UBS Precious Metals Subindex Total ReturnService Mark. \nThe Dow Jones-UBS Precious Metals Subindex Total ReturnService Mark (the "index") reflects the returns that are potentially available through an unleveraged investment in the futures contracts on precious metals commodities. The index is currently composed of two precious metals commodities contracts (gold and silver) which are included in the Dow Jones-UBS Commodity Index Total ReturnService Mark.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.397258	2020-10-11 23:38:31.02582
JKH	iShares Morningstar Mid-Cap Growth	The investment seeks to track the investment results of the Morningstar® Mid Growth IndexSM composed of mid-capitalization U.S. equities that exhibit growth characteristics.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index measures the performance of stocks issued by mid-capitalization companies that have exhibited above-average "growth" characteristics as determined by Morningstar, Inc.'s ("Morningstar" or the "index provider") proprietary index methodology.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.416324	2020-10-11 23:38:31.031328
JLL	Jones Lang LaSalle Incorporated	Jones Lang LaSalle Inc provides a range of real estate-related services including leasing, property and project management, and capital markets advisory to owners, occupiers, and investors worldwide.	{"Real Estate","Real Estate Services"}	{BGCP,HF,KW,CBRE,MMI,CIGI}	Real Estate Services	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:05.472333	2020-10-11 23:38:31.021426
JMPB	JMP Group Inc 8.00% Senior Notes due 2023	JMP Group LLC is a financial service provider. Its activities involve investment banking, sales and trading, and equity research services.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{}	Brokers & Exchanges	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.506326	2020-10-11 23:38:31.023428
JOB	GEE Group Inc.	GEE Group Inc provides professional placement services throughout the United States. It specializes in the placement of information technology, engineering, and accounting professionals for direct hire and contract staffing.	{Industrials,"Staffing & Outsourcing Services","Employment Services"}	{HSII,KFY,MAN,RHI,MHH,HSON}	Employment Services	Industrials	NYSE American	\N	2020-10-11 23:12:05.528369	2020-10-11 23:38:31.025871
JPEH	JPMorgan Diversified Return Europe Currency Hedged	The investment seeks investment results that closely correspond, to the performance of the JP Morgan Diversified Factor Europe Currency Hedged (USD) Index.\n The fund principally invests in the JPMorgan Diversified Return Europe Equity ETF and forward foreign currency contracts. The underlying index has been developed to represent European stock performance, with the foreign currency risk of the securities included in the underlying index hedged against the U.S. dollar on a monthly basis. The underlying index is comprised of equity securities across developed Europe selected to represent a diversified set of factor characteristics: value, momentum, and quality.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.547731	2020-10-11 23:38:31.02582
JPIN	JPMorgan Diversified Return International Equity	The investment seeks investment results that closely correspond, before fees and expenses, to the performance of the JP Morgan Diversified Factor International Equity Index.\n The fund will invest at least 80% of its net assets in securities included in the underlying index. The underlying index is comprised of equity securities across developed global markets (excluding North America) selected to represent a diversified set of factor characteristics: value, price momentum and quality.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.568194	2020-10-11 23:38:31.021426
JBLU	JetBlue Airways Corporation	JetBlue Airways Corp is engaged in the airlines industry. It primarily provides passenger airline service. It also extends its services to cargo transport, travel agency operations and travel insurance.	{Industrials,Airlines}	{SKYW,HA,ALGT,ALK,LUV,DAL,AAL,SAVE,UAL}	Airlines	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:05.279498	2020-10-11 23:38:31.02423
JCO	Nuveen Credit Opportunities 2022 Target Term Fund of Beneficial Interest	Nuveen Credit Opportunities 2022 Target Term is a closed-end investment fund. The Fund invests in a portfolio of below investment grade corporate bonds. Its objective is to provide high current income from a portfolio of high-yield corporate debt.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.297796	2020-10-11 23:38:31.023007
JFR	Nuveen Floating Rate Income Fund	Nuveen Floating Rate Income Fund is a diversified closed end management investment company. Its investment objective is to achieve a high level of current income. It invests in adjustable rate loans, primarily secured senior loans.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.330287	2020-10-11 23:38:31.028166
JHMA	John Hancock Multifactor Materials	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the John Hancock Dimensional Materials Index (the index).\n The fund normally invests at least 80% of its net assets (plus any borrowings for investment purposes) in securities that compose the fund's benchmark index. The index is designed to comprise securities in the materials sector within the U.S. Universe whose market capitalizations are larger than that of the 1001st largest U.S. company at the time of reconstitution. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.34722	2020-10-11 23:38:31.028166
JHMU	John Hancock Multifactor Utilities	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the John Hancock Dimensional Utilities Index (the index).\n The fund normally invests at least 80% of its net assets (plus any borrowings for investment purposes) in securities that compose the fund's benchmark index. The index is designed to comprise securities in the utilities sector within the U.S. Universe whose market capitalizations are larger than that of the 1001st largest U.S. company at the time of reconstitution. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.375197	2020-10-11 23:38:31.02423
JJT	Barclays Bank PLC	The investment seeks to provide investors with exposure to the Dow Jones-UBS Tin Subindex Total ReturnService Mark. \nThe Dow Jones-UBS Tin Subindex Total ReturnService Mark (the "index") reflects the returns that are potentially available through an unleveraged investment in the futures contracts on tin. The index currently consists of one futures contract on the commodity of tin which is included in the Dow Jones-UBS Commodity Index Total ReturnService Mark.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.40117	2020-10-11 23:38:31.023428
JKL	iShares Morningstar Small-Cap Value ETFnd	The investment seeks to track the investment results of the Morningstar® Small Value IndexSM composed of small-capitalization U.S. equities that exhibit value characteristics.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index measures the performance of stocks issued by small-capitalization companies that have exhibited "value" characteristics as determined by Morningstar, Inc.'s ("Morningstar" or the "index provider") proprietary index methodology.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.454496	2020-10-11 23:38:31.028166
JMM	Nuveen Multi-Market Income Fund (MA)	Nuveen Multi-Market Income Fund is a United States based closed-end management investment company. The investment objective of the company is to provide high monthly income consistent with prudent risk to capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.500837	2020-10-11 23:38:31.02582
JNUG	Direxion Daily Junior Gold Miners Index Bull 3X Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the daily performance of the MVIS Global Junior Gold Miners Index.\n The fund invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, ETFs that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index includes companies from markets that are freely investable to foreign investors, including "emerging markets," as that term is defined by the index provider. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.528518	2020-10-11 23:38:31.023428
JPEU	JPMorgan Diversified Return Europe Equity	The investment seeks investment results that closely correspond, before fees and expenses, to the performance of the JP Morgan Diversified Factor Europe Equity Index.\n The fund will invest at least 80% of its net assets in securities included in the underlying index. The underlying index is comprised of equity securities across developed Europe selected to represent a diversified set of factor characteristics: value, momentum, and quality. The fund may invest in depositary receipts representing securities included in the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.556661	2020-10-11 23:38:31.025871
JPMB	JPMorgan USD Emerging Markets Sovereign Bond	The investment seeks investment results that closely correspond, before fees and expenses, to the performance of the JPMorgan Emerging Markets Risk-Aware Bond Index.\n The fund will invest at least 80% of its assets in securities included in the underlying index. The underlying index is comprised of liquid, U.S. dollar-denominated sovereign and quasi-sovereign fixed and floating rate debt securities from emerging markets selected using a rules-based methodology that was developed and is owned by the adviser. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.575373	2020-10-11 23:38:31.025871
JBK	Lehman ABS 3.50 3.50% Adjustable Corp Backed Tr Certs GS Cap I	It is a trust company	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:05.288186	2020-10-11 23:38:31.025871
JCP	J.C. Penney Company Inc.	JC Penney Co Inc is an American omnichannel retailer. The company sells merchandise and services to consumers through department stores and its own website.	{"Consumer Cyclical","Department Stores","Retail - Apparel & Specialty"}	{DDS,KSS,JWN,TGT,SSI,M,SHOS,LE}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:05.304524	2020-10-11 23:38:31.024353
JHA	Nuveen High Income December 2018 Target Term Fund	Nuveen High Income Dec 2018 Target Term is a diversified, closed-end management investment company. Its investment objective is to provide a high level of current income & return the original net asset value per common share on or about December 1, 2018.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.33483	2020-10-11 23:38:31.031328
JJA	iPathA Series B Bloomberg Agriculture Subindex Total Return ETN	The investment seeks return linked to the performance of the Bloomberg Agriculture Subindex Total ReturnSM.\nThe ETN offers exposure to futures contracts and not direct exposure to the physical commodities. The index is composed of one or more futures contracts on the relevant commodity (the “index components”) and is intended to reflect the returns that are potentially available through (1) an unleveraged investment in those contracts plus (2) the rate of interest that could be earned on cash collateral invested in specified Treasury Bills.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.383761	2020-10-11 23:38:31.025871
JKD	iShares Morningstar Large-Cap	The investment seeks to track the investment results of the Morningstar® Large Core IndexSM composed of large-capitalization U.S. equities.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index measures the performance of stocks issued by large-capitalization companies that have exhibited average "growth" and "value" characteristics as determined by Morningstar, Inc.'s ("Morningstar" or the "index provider") proprietary index methodology.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.40946	2020-10-11 23:38:31.021426
JKS	JinkoSolar Holding Company Limited American Depositary Shares (each representing 4)	JinkoSolar Holding Co Ltd is a producer of solar energy monocrystalline silicon. It has built a vertically-integrated solar power product value chain from recovered silicon materials to solar modules with the solar module as its product.	{Technology,Solar,Semiconductors}	{CSIQ,DQ,HQCL,SOL,XLK}	Semiconductors	Technology	New York Stock Exchange	\N	2020-10-11 23:12:05.472362	2020-10-11 23:38:31.028166
JMPD	JMP Group LLC 7.25% Senior Notes due 2027		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:05.506419	2020-10-11 23:38:31.023007
JOE	St. Joe Company (The)	The St. Joe Co is a real estate company. It operates residential real estate and commercial real estate properties and resorts.	{"Real Estate","Real Estate - General","Real Estate Services"}	{POPE,TRC,CBRE,FCE.A,FOR}	Real Estate Services	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:05.528207	2020-10-11 23:38:31.024353
JPED	JPMorgan Event Driven	The investment seeks to provide long-term total return.\n The fund seeks to achieve its investment objective by employing an event-driven investment strategy, primarily investing in companies that the fund's adviser believes will be impacted by pending or anticipated corporate or special situation events. It will do so based on its systematic investment process. The fund will generally invest its assets globally to gain exposure, either directly or through the use of derivatives, to equity securities (across market capitalizations) in developed markets.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.548144	2020-10-11 23:38:31.028166
JPLS	JPMorgan Long/Short	The investment seeks to provide long-term total return.\n The fund seeks to profit from the relative returns of equity securities by maintaining long and short positions. Under normal market conditions, the adviser employs the Equity Long/Short strategy to access certain return factors. The strategy involves simultaneously investing in equities (i.e., investing long) that the adviser believes are attractive based on relevant return factors and selling equities (selling short) that the adviser believes are unattractive based on relevant return factors.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.568879	2020-10-11 23:38:31.028166
JPSE	JPMorgan Diversified Return U.S. Small Cap Equity	The investment seeks investment results that closely correspond, before fees and expenses, to the performance of the JP Morgan Diversified Factor US Small Cap Equity Index.\n The fund will invest at least 80% of its assets in securities included in the underlying index. "Assets" means net assets, plus the amount of borrowing for investment purposes. The underlying index is comprised of U.S. equity securities selected to represent a diversified set of factor characteristics. The rules based proprietary multi-factor selection process utilizes the following characteristics: relative valuation, momentum and quality.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.589133	2020-10-11 23:38:31.025871
JRS	Nuveen Real Estate Income Fund of Beneficial Interest	Nuveen Real Estate Income Fund is a closed-end investment company. The fund's investment objective is to provide high current income and capital appreciation by investing at least 90% of its total assets in income-producing common stocks, and others.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.607669	2020-10-11 23:38:31.023007
JT	Jianpu Technology Inc. American depositary shares each two representing five Class A	Jianpu Technology Inc through its subsidiaries is engaged in the operation of its platform for providing online discovery and recommendation services of financial products. It also offers advertising, marketing and other services.	{Technology,"Internet Content & Information","Online Media"}	{NOAH,OAK,XLK}	Online Media	Technology	New York Stock Exchange	\N	2020-10-11 23:12:05.632551	2020-10-11 23:38:31.023007
JDD	Nuveen Diversified Dividend and Income Fund Shares of Beneficial Interest	Nuveen Diversified Dividend & Income is a diversified closed end management investment company. Its investment objectives are high current income and total return.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.312227	2020-10-11 23:38:31.028166
JHB	Nuveen High Income November 2021 Target Term Fund	Nuveen High Income Nov 2021 Target Term is a closed-Ended investment management fund. The fund has an objective to provide a high level of current income and to return the original net asset value (NAV) per common share.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.336668	2020-10-11 23:38:31.023007
JHY	Nuveen High Income 2020 Target Term Fund of Beneficial Interest	Nuveen High Income 2020 Target Term Fund is a closed-end fund company. The company's objective is to provide a high level of current income and to return the net asset value (NAV) per common share on or about November 1, 2020.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.37928	2020-10-11 23:38:31.024353
JKE	iShares Morningstar Large-Cap Growth	The investment seeks to track the investment results of the Morningstar® Large Growth IndexSM composed of large-capitalization U.S. equities that exhibit growth characteristics.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index measures the performance of stocks issued by large-capitalization companies that have exhibited above-average "growth" characteristics as determined by Morningstar, Inc.'s ("Morningstar" or the "index provider") proprietary index methodology.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.409648	2020-10-11 23:38:31.025871
JKK	iShares Morningstar Small-Cap Growth	The investment seeks to track the investment results of the Morningstar® Small Growth IndexSM composed of small-capitalization U.S. equities that exhibit growth characteristics.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index measures the performance of stocks issued by small-capitalization companies that have exhibited above-average "growth" characteristics as determined by Morningstar, Inc.'s ("Morningstar" or the "index provider") proprietary index methodology.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.454232	2020-10-11 23:38:31.024353
JMOM	JPMorgan U.S. Momentum Factor	The investment seeks investment results that closely correspond, before fees and expenses, to the performance of the JP Morgan US Momentum Factor Index.\n The fund will invest at least 80% of its assets in securities included in the underlying index. "Assets" means net assets, plus the amount of borrowing for investment purposes. The underlying index is comprised of U.S. equity securities selected to represent positive momentum factor characteristics.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.500582	2020-10-11 23:38:31.028166
JNPR	Juniper Networks Inc.	Juniper Networks Inc operates in the communication equipment industry. It designs, develops and sells products and services that provide customers with network infrastructure. It caters to communication service providers and companies.	{Technology,"Communication Equipment"}	{XLK}	Communication Equipment	Technology	New York Stock Exchange	\N	2020-10-11 23:12:05.528369	2020-10-11 23:38:31.023007
JPGB	JPMorgan Global Bond Opportunities	The investment seeks to provide total return.\n Under normal circumstances, the fund will invest at least 80% of its assets in bonds. It will invest at least 40% of its assets in countries other than the United States. The fund may invest in developed or emerging markets. It generally invests at least 25% of its assets in securities that, at the time of purchase are rated investment grade or the unrated equivalent. "Assets" means net assets plus the amount of borrowings for investment purposes. The fund may also use currency related transactions involving currency derivatives as part of its primary investment strategy.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:05.555135	2020-10-11 23:38:31.023007
JPME	JPMorgan Diversified Return U.S. Mid Cap Equity	The investment seeks investment results that closely correspond, before fees and expenses, to the performance of the JP Morgan Diversified Factor US Mid Cap Equity Index.\n The fund will invest at least 80% of its net assets in securities included in the underlying index. The underlying index is comprised of U.S. equity securities selected to represent a diversified set of factor characteristics: value, momentum, and quality.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.574701	2020-10-11 23:38:31.023007
JPT	Nuveen Preferred and Income 2022 Term Fund of Beneficial Interest	Nuveen Preferred and Income 2022 Term is a closed-end management investment company. Its investment objective is to provide a high level of current income and total return by investing in preferred and other income-producing securities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.593127	2020-10-11 23:38:31.023007
JRVR	James River Group Holdings Ltd.	James River Group Holdings Ltd owns and operates a group of specialty insurance and reinsurance companies founded by members of its management team. It is organized for the purpose of acquiring & managing insurance and reinsurance entities.	{"Financial Services","Insurance - Property & Casualty"}	{L,CB,AFH,NMIH}	Insurance - Property & Casualty	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:05.611781	2020-10-11 23:38:31.024353
JTPY	JetPay Corporation	JetPay Corp is a provider of payment services. It offers debit and credit card processing, payroll and human capital management services, and card services to businesses and their employees throughout the United States.	{Technology,"Software - Infrastructure","Application Software"}	{PMTS,XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:05.636165	2020-10-11 23:38:31.02582
JCI	Johnson Controls International plc	Johnson Controls International PLC manufacturers, installs, and services HVAC systems, building management systems and controls, and industrial refrigeration systems. The firm along with Tyco International offers fire and security products and services.	{Industrials,"Engineering & Construction"}	{ASCMA,HON,TXT,UTX,MMM}	Engineering & Construction	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:05.293753	2020-10-11 23:38:31.02423
JEC	Jacobs Engineering Group Inc.	Jacobs Engineering Group Inc is engaged in the engineering industry. The company provides technical, professional & construction services including engineering, design & architectural, construction management, operations & maintenance.	{Industrials,"Engineering & Construction"}	{TTEK,PWR,GVA,FLR,KBR,ACM,BWXT}	Engineering & Construction	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:05.322523	2020-10-11 23:38:31.024353
JHSC	John Hancock Multifactor Small Cap	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the John Hancock Dimensional Small Cap Index (the index).\n The fund normally invests at least 80% of its net assets (plus any borrowings for investment purposes) in securities that compose the fund's index. The index is designed to comprise a subset of securities in the U.S. Universe issued by companies whose market capitalizations are smaller than the 750th largest U.S. company but excluding the smallest 4% of U.S. companies at the time of reconstitution.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.375169	2020-10-11 23:38:31.031328
JKG	iShares Morningstar Mid-Cap	The investment seeks to track the investment results of the Morningstar® Mid Core IndexSM composed of mid-capitalization U.S. equities.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index measures the performance of stocks issued by mid-capitalization companies that have exhibited average "growth" and "value" characteristics as determined by Morningstar, Inc.'s ("Morningstar" or the "index provider") proprietary index methodology.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.418301	2020-10-11 23:38:31.02423
JMF	Nuveen Energy MLP Total Return Fund of Beneficial Interest	Nuveen Energy MLP Total Return Fund is a closed-end management investment company. Its objective is to provide a high level of current income and long-term capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.483651	2020-10-11 23:38:31.023428
JMT	Nuveen Mortgage Opportunity Term Fund 2 of Beneficial Interest	Nuveen Mortgage Opportunity Term Fund 2 is a non-diversified, closed-end management investment company. Its investment objective is to generate attractive total returns through opportunistic investments in mortgage-backed securities (MBS).	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.512602	2020-10-11 23:38:31.024353
JOF	Japan Smaller Capitalization Fund Inc	Japan Smaller Capitalization Fund Inc is a non-diversified, closed-end management investment company. Its investment objective is to seek long-term capital appreciation through investments primarily in smaller capitalization Japanese equity securities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.531502	2020-10-11 23:38:31.028166
JPEM	JPMorgan Diversified Return Emerging Markets Equity	The investment seeks investment results that closely correspond, before fees and expenses, to the performance of the JP Morgan Diversified Factor Emerging Markets Equity Index.\n The fund will invest at least 80% of its assets in securities included in the underlying index. "Assets" means net assets, plus the amount of borrowing for investment purposes. The underlying index is comprised of equity securities from emerging markets selected to represent a diversified set of factor characteristics.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.553781	2020-10-11 23:38:31.02423
JPM	JP Morgan Chase & Co.	JPMorgan Chase & Co is a financial services firm and a banking institution. It is engaged in investment banking, commercial banking, treasury and securities services, asset management, retail financial services, and credit card businesses.	{"Financial Services","Banks - Global",Banks}	{}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.569582	2020-10-11 23:38:31.02582
JPST	JPMorgan Ultra-Short Income	The investment seeks to provide current income while seeking to maintain a low volatility of principal.\n Under normal circumstances, the fund seeks to achieve its investment objective by investing at least 80% of its Assets in investment grade, U.S. dollar denominated short-term fixed, variable and floating rate debt. "Assets" means net assets, plus the amount of borrowings for investment purposes. As part of its principal investment strategy, it may invest in corporate securities, asset-backed securities, mortgage-backed and mortgage-related securities, and high quality money market instruments such as commercial paper and certificates of deposit.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:05.588728	2020-10-11 23:38:31.02582
JRO	Nuveen Floating Rate Income Opportuntiy Fund Shares of Beneficial Interest	Nuveen Floating Rate Income Opp Fd is a diversified closed end management investment company. Its investment objective is to achieve a high level of current income. It invests its managed assets in adjustable rate loans, primarily secured senior loans.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.607817	2020-10-11 23:38:31.02423
JTA	Nuveen Tax-Advantaged Total Return Strategy Fund of Beneficial Interest	Nuveen Tax Advantaged Total Ret Strat Fd is a closed end fund. Its investment objective is to achieve a high level of after-tax total return consisting primarily of tax-advantaged dividend income and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.632413	2020-10-11 23:38:31.02423
JBR	Select Asset Inc. Corporate Backed Callable Trust Certificates J.C. Penney Debenture-Backed Series 2006-1 Class A-1		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:05.286734	2020-10-11 23:38:31.023007
JCOM	j2 Global Inc.	j2 Global Inc together with its subsidiaries provides cloud services to businesses of all sizes, from individuals to enterprises, and license its intellectual property ("IP") to third parties.	{Technology,"Software - Infrastructure","Application Software"}	{MSFT,AKAM,LLNW,VZ,CRM,TDS,RNG,EQIX,GOOGL,INAP,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:05.302891	2020-10-11 23:38:31.021426
JELD	JELD-WEN Holding Inc.	JELD-WEN Holding Inc manufactures, distributes and deigns residential interior and exterior doors,windows and related products for residential and light commercial markets. The company markets it products under JELD-WEN brand.	{"Basic Materials","Building Materials"}	{}	Building Materials	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:05.324197	2020-10-11 23:38:31.023428
JHI	John Hancock Investors Trust	John Hancock Investors Trust is a United States based diversified, closed-end management investment company. It seeks to generate income for distribution to its shareholders. The secondary objective of the company is capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.342595	2020-10-11 23:38:31.028166
JHMH	John Hancock Multifactor Healthcare	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the John Hancock Dimensional Healthcare Index.\n The fund normally invests at least 80% of its net assets (plus any borrowings for investment purposes) in securities that compose the fund's benchmark index. The index is designed to comprise securities in the healthcare sector within the U.S. Universe whose market capitalizations are larger than that of the 1001st largest U.S. company at the time of reconstitution. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.360314	2020-10-11 23:38:31.028166
JHX	James Hardie Industries plc American Depositary Shares (Ireland)	James Hardie Industries PLC is engaged in the construction material industry. Its products are used in new residential construction, manufactured housing, repair and remodeling, and commercial and industrial applications.	{"Basic Materials","Building Materials"}	{CRH,CX,EXP,USCR}	Building Materials	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:05.37592	2020-10-11 23:38:31.023428
JJSF	J & J Snack Foods Corp.	J&J Snack Foods Corp manufactures nutritional snack foods and distributes frozen beverages that are marketed nationally to the food service and retail supermarket industries. Its brand names include SUPERPRETZEL, BAVARIAN BAKERY, LUIGI'S and WHOLE FRUIT.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{BRID,CAG,THS,MDLZ,PEP}	Consumer Packaged Goods	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:05.400543	2020-10-11 23:38:31.023007
JMP	JMP Group LLC	JMP Group LLC is a financial service provider. Its activities involve investment banking, sales and trading, and equity research services.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{}	Brokers & Exchanges	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.501197	2020-10-11 23:38:31.02423
JOBS	51job Inc.	51job Inc is a provider of integrated human resource services in China. It provides these services in two categories: Recruitment Advertising Services and Other Human Resource Related Services.	{Industrials,"Staffing & Outsourcing Services","Employment Services"}	{SINA,BIDU,WUBA,DHX,JOB,RENN}	Employment Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:05.528831	2020-10-11 23:38:31.02423
JPGE	JPMorgan Diversified Return Global Equity	The investment seeks investment results that closely correspond, before fees and expenses, to the performance of the JP Morgan Diversified Factor Global Developed Equity Index.\n The index is comprised of equity securities across developed global markets selected to represent a diversified set of factor characteristics: value, price momentum and quality. The fund will invest at least 80% of its net assets in securities included in the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.556818	2020-10-11 23:38:31.023428
JPMV	iShares Edge MSCI Min Vol Japan	The investment seeks to track the investment results of the MSCI Japan Minimum Volatility (USD) Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index measures the performance of Japanese equity securities that, in the aggregate, have lower volatility characteristics relative to the broader Japanese equity markets.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.576548	2020-10-11 23:38:31.028166
JRJC	China Finance Online Co. Limited	China Finance Online Co Ltd provides financial information and services including news, data, analytics, securities investment advisory and brokerage-related services.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{TST,SINA,BIDU,CHA}	Brokers & Exchanges	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:05.603915	2020-10-11 23:38:31.028166
JSYNR	Jensyn Acquistion Corp. Rights		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:05.625208	2020-10-11 23:38:31.025871
JVAL	JPMorgan U.S. Value Factor	The investment seeks investment results that closely correspond, before fees and expenses, to the performance of the JP Morgan US Value Factor Index.\n The fund will invest at least 80% of its assets in securities included in the underlying index. "Assets" means net assets, plus the amount of borrowing for investment purposes. The underlying index is comprised of U.S. equity securities selected to represent value factor characteristics.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.63765	2020-10-11 23:38:31.031328
JCTCF	Jewett-Cameron Trading Company	Jewett-Cameron Trading Co Ltd, through its subsidiaries, is engaged in the manufacturing and distribution of specialty metal products and wholesale distribution of wood products to home centers and other retailers located in the United States.	{"Basic Materials","Lumber & Wood Production","Forest Products"}	{HD,LL,BCC,BMCH,BXC}	Forest Products	Basic Materials	NASDAQ Capital Market	\N	2020-10-11 23:12:05.307813	2020-10-11 23:38:31.028166
JEQ	Aberdeen Japan Equity Fund Inc.	Aberdeen Japan Equity Fund Inc is a United States based closed-end diversified management investment company. Its investment objective is to outperform over the long term, on a total return basis.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.324851	2020-10-11 23:38:31.02582
JHG	Janus Henderson Group plc	Janus Henderson Group PLC is an investment management company. It invests in equities, fixed income, property, and private equity.	{"Financial Services","Asset Management"}	{NOAH,OAK}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.341626	2020-10-11 23:38:31.024353
JJM	Barclays Bank PLC	The investment seeks to provide investors with exposure to the Dow Jones-UBS Industrial Metals Subindex Total ReturnService Mark. \nThe index reflects the returns that are potentially available through an unleveraged investment in the futures contracts on industrial metal commodities. It is currently composed of four futures contracts on industrial metals, three of which are traded on the London Metal Exchange and the other of which is traded on the COMEX division of the New York Mercantile Exchange and is a sub-index of the Dow Jones-UBS Commodity Index Total ReturnService Mark.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.395168	2020-10-11 23:38:31.028166
JKHY	Jack Henry & Associates Inc.	Jack Henry & Associates Inc is a provider of information processing solutions for community banks. Its products and services include processing transactions, automating business processes and managing information.	{Industrials,"Business Services"}	{FISV,ACIW,IBM,BOH,FIS,CLGX,AVT}	Business Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:05.426615	2020-10-11 23:38:31.023428
JMIN	JPMorgan U.S. Minimum Volatility	The investment seeks investment results that closely correspond, before fees and expenses, to the performance of the JP Morgan US Minimum Volatility Index.\n The fund will invest at least 80% of its assets in securities included in the underlying index. "Assets" means net assets, plus the amount of borrowing for investment purposes. The underlying index measures the performance of U.S. equity securities selected using a rules-based process that is designed so the underlying index targets lower volatility than the Russell 1000 Index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.494075	2020-10-11 23:38:31.023007
JNK	SPDR Bloomberg Barclays High Yield Bond	The investment seeks to provide investment results that correspond generally to the price and yield performance of the Bloomberg Barclays High Yield Very Liquid Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index or in securities that the Adviser determines have economic characteristics that are substantially identical to the economic characteristics of the securities that comprise the index. The index is designed to measure the performance of publicly issued U.S. dollar denominated high yield corporate bonds with above-average liquidity. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.51693	2020-10-11 23:38:31.031328
JONE	Jones Energy Inc. Class A	Jones Energy Inc is an exploration and production company in the U.S. Mid-Continent. The company operates in the Anadarko basin.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:05.53788	2020-10-11 23:38:31.023007
JPHY	JPMorgan Disciplined HY	The investment seeks a high level of income; capital appreciation is a secondary objective.\n The fund invests primarily in high yield, high risk debt securities. Under normal circumstances, it invests at least 80% of its assets in high yield securities. For purposes of this policy, "assets" means net assets plus the amount of borrowings for investment purposes. The fund may invest up to 100% of its total assets in below investment grade or unrated securities. The adviser uses a multi-factor security selection process that uses factors that the adviser believes reflect liquidity and issuer quality.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:05.557744	2020-10-11 23:38:31.028166
JPMF	JPMorgan Managed Futures Strategy	The investment seeks to provide long-term total return.\n The fund will generally invest its assets globally to gain exposure, either directly or through the use of derivatives, to equity securities (across market capitalizations) in developed markets, debt securities (including below investment grade or high yield securities), commodities and currencies (including in emerging markets). It may invest in fixed income securities of any average weighted maturity or duration. The fund may use both long and short positions (achieved primarily through the use of derivative instruments). It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.577128	2020-10-11 23:38:31.023428
JQC	Nuveen Credit Strategies Income Fund Shares of Beneficial Interest	Nuveen Credit Strategies Income Fund is a diversified closed end management investment company. Its investment objective is to provide high current income and total return by investing its managed assets in senior secured and second lien loans.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.596778	2020-10-11 23:38:31.023428
JVA	Coffee Holding Co. Inc.	Coffee Holding Co Inc is a wholesale coffee manufacturer distributing roasted and blended coffees to customers in the U.S. and Canada. The firm works with supermarkets, other wholesalers, and specialty roasters.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{FARM,SJM}	Consumer Packaged Goods	Consumer Defensive	NASDAQ Capital Market	\N	2020-10-11 23:12:05.637954	2020-10-11 23:38:31.023428
JDIV	JPMorgan U.S. Dividend	The investment seeks investment results that closely correspond, before fees and expenses, to the performance of the JP Morgan US Dividend Index.\n The fund will invest at least 80% of its assets in securities included in the underlying index. "Assets" means net assets, plus the amount of borrowing for investment purposes. The underlying index is comprised of higher yielding U.S. equity securities.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.311797	2020-10-11 23:38:31.031328
JGH	Nuveen Global High Income Fund of Beneficial Interest	Nuveen Global High Income Fund is a closed-ended investment management fund. Its investment objective is to provide a high level of current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.331911	2020-10-11 23:38:31.028166
JHMD	John Hancock Exchange-Traded Fund Trust Multifactor Developed International	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the John Hancock Dimensional Large Cap Index.\n The fund normally invests at least 80% of its net assets (plus any borrowings for investment purposes) in securities that compose the fund's benchmark index. The index is designed to comprise a subset of securities in the U.S. Universe issued by companies whose market capitalizations are larger than that of the 801st largest U.S. company at the time of reconstitution.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.355525	2020-10-11 23:38:31.02423
JHMS	John Hancock Multifactor Consumer Staples	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the John Hancock Dimensional Consumer Staples Index (the index).\n The fund normally invests at least 80% of its net assets (plus any borrowings for investment purposes) in securities that compose the fund's benchmark index. The index is designed to comprise securities in the consumer staples sector within the U.S. Universe whose market capitalizations are larger than that of the 1001st largest U.S. company at the time of reconstitution. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.372676	2020-10-11 23:38:31.024353
JJE	Barclays Bank PLC	The investment seeks to provide investors with exposure to the Dow Jones-UBS Energy Subindex Total ReturnService Mark. \nThe Dow Jones-UBS Energy Subindex Total ReturnService Mark (the "index") reflects the returns that are potentially available through an unleveraged investment in the futures contracts on energy commodities. The index is currently composed of four energy-related commodities contracts (crude oil, heating oil, natural gas and unleaded gasoline) which are included in the Dow Jones-UBS Commodity Index Total ReturnService Mark.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.390018	2020-10-11 23:38:31.028166
JKF	iShares Morningstar Large-Cap Value	The investment seeks to track the investment results of the Morningstar® Large Value IndexSM composed of large-capitalization U.S. equities that exhibit value characteristics.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index measures the performance of stocks issued by large-capitalization companies that have exhibited above-average "value" characteristics as determined by Morningstar, Inc.'s ("Morningstar" or the "index provider") proprietary index methodology.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.414413	2020-10-11 23:38:31.028166
JMEI	Jumei International Holding Limited American Depositary Shares each representing one Class A	Jumei International Holding Ltd is an online retailer of beauty products. The product category includes cosmetics, skin care, cosmetic applicators, fragrance and body care.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:05.483877	2020-10-11 23:38:31.02423
JNCE	Jounce Therapeutics Inc.	Jounce Therapeutics Inc is a clinical stage immunotherapy company. It is engaged in transforming the treatment pf cancer by developing therapies that enable the immune system to attack tumors and provide long-lasting benefits to patients.	{Healthcare,Biotechnology}	{MNTA,MLNT,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:05.517896	2020-10-11 23:38:31.028166
JPC	Nuveen Preferred & Income Opportunities Fund	Nuveen Preferred & Income Opportunities Fund is a diversified, closed end management investment company. Its investment objective is to provide high current income and total return by investing its managed assets in preferred securities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.542504	2020-10-11 23:38:31.021426
JPIH	JPMorgan Diversified Return International Currency Hedged	The investment seeks investment results that closely correspond to the performance of the JP Morgan Diversified Factor International Currency Hedged (USD) Index.\n The underlying index is comprised of equity securities across developed global markets (excluding North America) selected to represent a diversified set of factor characteristics: value, momentum, and quality. The fund principally invests in the JPMorgan Diversified Return International Equity ETF (the underlying fund) and forward foreign currency contracts.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.567885	2020-10-11 23:38:31.02423
JSML	Janus Henderson Small Cap Growth Alpha ETF	The investment seeks investment results that correspond generally, before fees and expenses, to the performance of its underlying index, the Janus Small Cap Growth Alpha Index.\n The fund pursues its investment objective by normally investing at least 80% of its net assets in the securities that comprise the underlying index. The underlying index is composed of common stocks of small-sized companies that are included in the Solactive Small Cap Index, a universe of 2,000 small-sized capitalization stocks.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:05.61773	2020-10-11 23:38:31.023428
JRI	Nuveen Real Asset Income and Growth Fund of Beneficial Interest	Nuveen Real Asset Income and Growth Fund is a closed end fund. Its objective is to provide a high level of current income and long-term capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.600867	2020-10-11 23:38:31.025871
JXI	iShares Global Utilities	The investment seeks to track the investment results of the S&P Global 1200 Utilities IndexTM.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index. A significant portion of the underlying index is represented by securities of companies in the electric utilities and utilities industries or sectors.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.64384	2020-10-11 23:38:31.023007
KAR	KAR Auction Services Inc	KAR Auction Services Inc is a used car dealer with operations in US and Canada. The firm offers a total package for wholesale-used car transactions. Its services include financing, repossessions, repairs, transportation, warranty, and inventory services.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{CPRT,CRMT,LAD,KMX}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:05.662907	2020-10-11 23:38:31.028166
KARS	KraneShares Electric Vehicles and Future Mobility Index	The investment seeks to provide investment results that, correspond generally to the price and yield performance of the Solactive Electric Vehicles and Future Mobility Index.\n The fund will invest at least 80% of its total assets in components of the index, depositary receipts, including ADRs, representing such components, and securities underlying depositary receipts in the index. The index is designed to track the equity market performance of companies engaged in the production of electric vehicles or their components or in other initiatives that may change the future of mobility, as determined by index provider. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.662924	2020-10-11 23:38:31.023007
KBLMR	KBL Merger Corp. IV Right		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:05.679118	2020-10-11 23:38:31.028166
KBWR	Invesco KBW Regional Banking ETF	The investment seeks to track the investment results (before fees and expenses) of the KBW Nasdaq Regional Banking Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in securities of publicly traded mid-capitalization companies that do business as regional banks and thrifts listed on U.S. stock markets and that comprise the underlying index. The index provider compile, maintain and calculate the underlying index, which is a modified-market capitalization-weighted index comprised of securities of 50 mid-cap banking companies that are publicly listed in the United States.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:05.694731	2020-10-11 23:38:31.023007
KBWD	Invesco KBW High Dividend Yield Financial ETF	The investment seeks to track the investment results (before fees and expenses) of the KBW Nasdaq Financial Sector Dividend Yield Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities of financial companies that are listed on a U.S. national securities exchange and are principally engaged in the business of providing financial services and products, including banking, insurance and diversified financial services, in the United States and that comprise the underlying index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:05.694287	2020-10-11 23:38:31.024353
KCE	SPDR S&P Capital Markets	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the S&P Capital Markets Select Industry Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index represents the capital markets segment of the S&P Total Market Index ("S&P TMI") and tracks the performance of publicly traded companies that do business as broker-dealers, asset managers, trust and custody banks or exchanges. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.705138	2020-10-11 23:38:31.028166
KDMN	Kadmon Holdings Inc.	Kadmon Holdings Inc is a fully integrated biopharmaceutical company engaged in the discovery, development and commercialization of small molecules and biologics within autoimmune and fibrotic diseases, oncology and genetic diseases.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:05.714632	2020-10-11 23:38:31.02582
KEP	Korea Electric Power Corporation	Korea Electric Power Corp is engaged in the utilities sector. Its business involves the generation, transmission and distribution of electricity and development of electric power resources in Korea.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{ENIA,SCG,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:05.724902	2020-10-11 23:38:31.028166
JTD	Nuveen Tax-Advantaged Dividend Growth Fund of Beneficial Interest	Nuveen Tax-advantaged Dividend Gr Fund is a closed-ended investment company. Its investment objective is to provide an attractive level of tax-advantaged distributions and capital appreciation by investing in dividend-paying equity securities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.63558	2020-10-11 23:38:31.024353
KAI	Kadant Inc	Kadant Inc develops, manufactures, and markets equipment and products for the papermaking, paper recycling and other process industries. It also designs and manufactures stranders and related equipment used in the production of oriented strand board.	{Industrials,"Diversified Industrials","Industrial Products"}	{CMCO,JAKK,AIN,ITW}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:05.650263	2020-10-11 23:38:31.025871
KALA	Kala Pharmaceuticals Inc.	Kala Pharmaceuticals Inc is a biopharmaceutical company focused on the development and commercialization of therapeutics using its proprietary nanoparticle-based Mucus Penetrating Particles technology, with focus on the treatment of eye diseases.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:05.653752	2020-10-11 23:38:31.02582
KB	KB Financial Group Inc	KB Financial Group Inc is in the financial service domain. It provides commercial banking, credit cards, asset management, life insurance, capital markets activities and international banking and financial services.	{"Financial Services","Banks - Regional - Asia",Banks}	{IBN,SHG,MUFG,BK,MFG,WF}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.667166	2020-10-11 23:38:31.031328
KBAL	Kimball International Inc.	Kimball International Inc manufactures furniture products and services under three brands: National, Kimball Office, and Kimball Hospitality. It provides solutions for the workplace, learning, healing, and hospitality environments.	{Industrials,"Business Equipment","Industrial Products"}	{SGMA,MLHR,PLXS,IEC,JBL,SCS,BHE,HNI,KNL,SANM}	Industrial Products	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:05.672415	2020-10-11 23:38:31.02582
KBLMW	KBL Merger Corp. IV Warrant		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:05.682717	2020-10-11 23:38:31.031328
KBR	KBR Inc.	KBR Inc is an engineering, construction and services company supporting the hydrocarbons and international government services market sectors.	{Industrials,"Engineering & Construction"}	{PRIM,E,MTZ,GVA,JEC,FLR,TPC}	Engineering & Construction	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:05.686873	2020-10-11 23:38:31.023007
KBWY	Invesco KBW Premium Yield Equity REIT ETF	The investment seeks to track the investment results (before fees and expenses) of the KBW Nasdaq Premium Yield Equity REIT Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities of small- and mid-cap equity REITs in the United States that comprise the underlying index. It generally invests in all of the securities comprising the underlying index in proportion to their weightings in the underlying index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:05.698652	2020-10-11 23:38:31.031328
KCAPL	KCAP Financial Inc. 6.125% Notes due 2022		{}	{}			Nasdaq Global Select	\N	2020-10-11 23:12:05.702085	2020-10-11 23:38:31.028166
KELYA	Kelly Services Inc. Class A Common Stock	Kelly Services Inc is a provider of workforce solutions and consulting and staffing services. The company assigns professional and technical employees in the fields of education, legal, health care, and creative services.	{Industrials,"Staffing & Outsourcing Services","Employment Services"}	{BBSI,KFRC,MAN,RHI,TBI,HSON}	Employment Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:05.714384	2020-10-11 23:38:31.023007
KE	Kimball Electronics Inc.	Kimball Electronics Inc is a contract electronic manufacturing services company. The Company is engaged in producing electronics for the automotive, medical, industrial, and public safety markets.	{Technology,"Electronic Components","Computer Hardware"}	{XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:05.714374	2020-10-11 23:38:31.021426
KEM	KEMET Corporation	Kemet Corp together with its subsidiaries, manufactures and sells passive electronic components under the KEMET brand worldwide and the TOKIN brand in Japan and Korea.	{Technology,"Electronic Components","Computer Hardware"}	{MXWL,VICR,AVX,FMC,VSH,KN,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:05.714548	2020-10-11 23:38:31.024353
KEQU	Kewaunee Scientific Corporation	Kewaunee Scientific Corp is engaged in designing, manufacturing, and installation of laboratory, healthcare, and technical furniture products. Its products include fume hoods, casework, safety cabinets, exhaust systems, and energy saving controls.	{Industrials,"Business Equipment","Industrial Products"}	{MLHR,VIRC,SCS,HNI,KNL,KOOL}	Industrial Products	Industrials	NASDAQ Global Market	\N	2020-10-11 23:12:05.725065	2020-10-11 23:38:31.025871
JSYNU	Jensyn Acquistion Corp. Unit	Jensyn Acquisition Corp is a blank check company. It was formed for the purpose of entering into a merger, share exchange, asset acquisition, stock purchase, reorganization or other similar business combination with one or more target business.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:05.626579	2020-10-11 23:38:31.028166
JYNT	The Joint Corp.	The Joint Corp develops, owns, operates, supports and manages chiropractic clinics through direct ownership, management arrangements, franchising and the sale of regional developer rights throughout the United States.	{Healthcare,"Medical Care","Health Care Providers"}	{THC,TDOC,XLV}	Health Care Providers	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:05.650178	2020-10-11 23:38:31.028166
KBA	KraneShares Bosera MSCI China A	The investment seeks to provide investment results that, before fees and expenses, correspond to the price and yield performance of the MSCI China A Inclusion Index (the "underlying index").\n The fund will invest at least 80% of its total assets in securities of the underlying index, depositary receipts representing such securities and securities underlying depositary receipts in the underlying index. The index reflects the Chinese renminbi -denominated equity securities listed on the Shenzhen or Shanghai Stock Exchanges ("A Shares") included in the MSCI Emerging Markets Index assuming that index's methodology permitted the full inclusion of A Shares.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.67124	2020-10-11 23:38:31.028166
KBSF	KBS Fashion Group Limited	KBS Fashion Group Ltd is a casual menswear company in China. The company is engaged in designing, manufacturing, marketing, and selling of its fashion menswear, through network of stores.	{"Consumer Cyclical","Apparel Manufacturing","Manufacturing - Apparel & Furniture"}	{RCKY,KBSF}	Manufacturing - Apparel & Furniture	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:05.692314	2020-10-11 23:38:31.028166
KCAP	KCAP Financial Inc.	KCAP Financial Inc is a non-diversified closed-end investment company. It invests in senior secured term loans and mezzanine debt primarily in privately-held middle market companies, asset management companies and debt and subordinated securities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:05.699316	2020-10-11 23:38:31.02423
KCNY	KraneShares E Fund China Commercial Paper	The investment seeks to provide investment results that, before fees and expenses, correspond to the price and yield performance of the CSI Diversified High Grade Commercial Paper Index.\n The fund will invest at least 80% of its total assets in securities of the underlying index. The underlying index seeks to track the performance of investment grade on-shore renminbi -denominated commercial paper issued by sovereign, quasi-sovereign and corporate issuers in the People's Republic of China's and traded in the Chinese Inter-Bank Bond Market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.711788	2020-10-11 23:38:31.025871
KELYB	Kelly Services Inc. Class B Common Stock	Kelly Services Inc is a provider of workforce solutions and consulting and staffing services. The company assigns professional and technical employees in the fields of education, legal, health care, and creative services.	{Industrials,"Staffing & Outsourcing Services","Employment Services"}	{BBSI,KFRC,MAN,RHI,TBI,HSON}	Employment Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:05.71456	2020-10-11 23:38:31.031328
KEMQ	KraneShares Emerging Markets Consumer Technology Index	The investment seeks to provide investment results that track the price and yield performance of Solactive Emerging Market Consumer Technology Index.\n Under normal circumstances, the fund will invest at least 80% of its total assets in components of the index, depositary receipts, including ADRs, representing such components and securities underlying depositary receipts in the index. The index is composed of the equity securities of the 50 companies with the largest market capitalization that are headquartered in certain emerging market and frontier countries and engaged in the consumer or technology sectors. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.719724	2020-10-11 23:38:31.028166
KERX	Keryx Biopharmaceuticals Inc.	Keryx Biopharmaceuticals Inc focuses on developing medicines to people with kidney diseases. The marketed product of the company is auryxia tablet, which is a iron based medicine.	{Healthcare,Biotechnology}	{MNTA,AMAG,SPHS,ARDX,AEZS,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:05.729983	2020-10-11 23:38:31.02423
ISCA	International Speedway Corporation	International Speedway Corp is an owner of motorsports entertainment facilities and promoter of motorsports themed entertainment activities in the United States.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{}	Travel & Leisure	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:05.040538	2020-10-11 23:38:31.031328
ISRL	Isramco Inc.	Isramco Inc is an independent oil and natural gas company, engaged in the exploration, development, and production of predominately oil and natural gas properties located onshore in the United States and offshore Israel.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{PNRG,CHK,PXD,APC,EGN,OXY,NBL,QEP}	Oil & Gas - E&P	Energy	NASDAQ Capital Market	\N	2020-10-11 23:12:05.06527	2020-10-11 23:38:31.028166
ITE	SPDR Bloomberg Barclays Intermediate Term Treasury	The investment seeks to provide investment results that correspond generally to the price and yield performance of the Bloomberg Barclays 3-10 Year U.S. Treasury Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index or in securities that the Adviser determines have economic characteristics that are substantially identical to the economic characteristics of the securities that comprise the index. The index is designed to measure the performance of intermediate term (3-10 years) public obligations of the U.S. Treasury. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.085561	2020-10-11 23:38:31.028166
ITT	ITT Inc.	ITT Inc is a manufacturer of engineered, customized solutions for the energy, transportation and industrial markets. Its operations are divided into Industrial Process, Motion Technologies, Interconnect Solutions, and Control Technologies.	{Industrials,"Diversified Industrials","Industrial Products"}	{APH,FLS,BA,IEX,HON,NOC,PH,RTN,CFX,LLL}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:05.106255	2020-10-11 23:38:31.028166
IVENC	Ivy Focused Energy NextShares Fund	The investment seeks to provide capital appreciation with a secondary objective of providing current income.\nThe fund seeks to achieve its objectives by investing primarily in the common stocks of companies that IICO believes offer a high yield and that possess quality, valuation and growth potential characteristics. It seeks to invest in companies that meet all three of these criteria. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:05.131125	2020-10-11 23:38:31.023428
IWM	iShares Russell 2000	The investment seeks to track the investment results of the Russell 2000® Index, which measures the performance of the small-capitalization sector of the U.S. equity market.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index, but which the advisor believes will help the fund track the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.159399	2020-10-11 23:38:31.023007
IXG	iShares Global Financial	The investment seeks to track the investment results of the S&P Global 1200 Financials IndexTM.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index. The index measures the performance of companies that the index provider deems to be part of the financial sector of the economy and that the index provider believes are important to global markets.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.179095	2020-10-11 23:38:31.025871
IYJ	iShares U.S. Industrials	The investment seeks to track the investment results of the Dow Jones U.S. Industrials Index composed of U.S. equities in the industrials sector.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index measures the performance of the industrials sector of the U.S. equity market.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:05.233037	2020-10-11 23:38:31.025871
IYZ	iShares U.S. Telecommunications	The investment seeks to track the investment results of the Dow Jones U.S. Select Telecommunications Index.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index measures the performance of the telecommunications sector of the U.S. equity market and may include large-, mid- or small-capitalization companies. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:05.249694	2020-10-11 23:38:31.021426
JBL	Jabil Inc.	Jabil Inc is a provider of electronic manufacturing services and solutions. It provides electronic design, production and product management services to companies in the aerospace and defense, automotive.	{Technology,"Contract Manufacturers","Computer Hardware"}	{CLS,FLEX,SGMA,KBAL,PLXS,SMTX,IEC,BHE,SANM,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:05.278943	2020-10-11 23:38:31.028166
KHC	The Kraft Heinz Company	The Kraft Heinz Co is a food and beverage company. It manufactures and market food and beverage products, including condiments and sauces, cheese and dairy, meals, meat, refreshment beverages, and other grocery products.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{HAIN,GIS,TSN,SJM,THS,MDLZ,PEP}	Consumer Packaged Goods	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:05.762653	2020-10-11 23:38:31.025871
KLDW	Knowledge Leaders Developed World	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the Knowledge Leaders Developed World Index.\n The fund will normally invest at least 80% of its total assets in securities of the index. The index is designed to measure the performance of issuers in developed markets countries that are considered to be "Knowledge Leaders," as defined and determined by the index provider, based on a proprietary selection model developed by the index provider and incorporated into the index methodology. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.802942	2020-10-11 23:38:31.02423
KMPH	KemPharm Inc.	KemPharm Inc is a clinical-stage specialty pharmaceutical company engaged in the discovery and development of proprietary prodrugs that believe will be improved versions of widely prescribed, approved drugs.	{Healthcare,Biotechnology}	{JAZZ,AGRX,TBPH,COLL,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:05.820596	2020-10-11 23:38:31.028166
KNL	Knoll Inc.	Knoll Inc designs and manufactures appliances such as furniture, textiles, leathers, dining chairs for home and office use. The company operates through three segments: office, studio, and covering.	{Industrials,"Business Equipment","Industrial Products"}	{MLHR,KEQU,KBAL,VIRC,SCS,LZB,HNI,TILE}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:05.832442	2020-10-11 23:38:31.02423
KOOL	Cesca Therapeutics Inc.	Cesca Therapeutics Inc is engaged in the development and manufacturing of automated blood and bone marrow processing systems that enable the separation, processing and preservation of cell and tissue therapy products.	{Healthcare,"Medical Devices"}	{CERS,KEQU,HAE,SYK,XLV}	Medical Devices	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:05.868555	2020-10-11 23:38:31.025871
KRYS	Krystal Biotech Inc.	Krystal Biotech Inc is a biopharmaceutical company. It uses gene therapy treatments for patients suffering from rare debilitating disorders.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:05.909891	2020-10-11 23:38:31.028166
KTP	Corts 7.625 Pfd		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:05.936069	2020-10-11 23:38:31.031328
L	Loews Corporation	Loews Corp deals in commercial property and casualty insurance business. Its other business operations are offshore oil and gas drilling rigs, transportation, storage of natural gas, and gathering and operation of a chain of hotels.	{"Financial Services","Insurance - Property & Casualty"}	{ALL,AFG,WRB,CNA,MKL,PGR,RLI,TRV,CB,WTM}	Insurance - Property & Casualty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.959878	2020-10-11 23:38:31.02582
LALT	PowerShares Multi-Strategy Alternative Portfolio	The investment seeks a positive total return that has a low correlation to the broader securities markets.\n The fund is an actively managed exchange-traded fund ("ETF") that seeks to achieve its investment objective by investing in a combination of equity securities, financial futures contracts, forward currency contracts and other securities. The sub-adviser selects investments for inclusion in the fund's portfolio with reference to the components of the Morgan Stanley Multi-Strategy Alternative Index (the "Benchmark") through a quantitative process that seeks to exceed the Benchmark's performance.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:05.982414	2020-10-11 23:38:31.02423
LBAI	Lakeland Bancorp Inc.	Lakeland Bancorp Inc is a bank holding company. It offers retail banking, business banking, and investment programs.	{"Financial Services","Banks - Regional - US",Banks}	{UNTY,SSFN,PGC,FCCY,PKBK,BKJ,TRCB}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:06.005284	2020-10-11 23:38:31.028166
LBRT	Liberty Oilfield Services Inc. Class A	Liberty Oilfield Services Inc is a provider of hydraulic fracturing services to onshore oil and natural gas E&P companies in North America.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:06.024045	2020-10-11 23:38:31.021426
LCII	LCI Industries	LCI Industries Inc supplies domestically and internationally components for the original equipment manufacturers of recreational vehicles and adjacent industries including buses; trailers used to haul boats, livestock, equipment and other cargo.	{"Consumer Cyclical","Recreational Vehicles",Autos}	{FLXS,PATK,SPAR,SKY,SUP,THO,WGO,SRI}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:06.047467	2020-10-11 23:38:31.021426
LE	Lands' End Inc.	Lands' End Inc is a multi-channel retailer of casual clothing, accessories and footwear, as well as home products. The Company offers products through catalogs, landsend.com, and operates in two segments; Direct and Retail.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{DDS,KSS,JWN,JCP,SSI,M,SHOS}	Retail - Apparel & Specialty	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:06.066856	2020-10-11 23:38:31.024353
LEN	Lennar Corporation Class A	Lennar Corp is a homebuilding service provider. Its operations include the construction and sale of single-family attached and detached homes, and to a lesser extent multi-level residential buildings.	{"Consumer Cyclical","Residential Construction","Homebuilding & Construction"}	{PHM,MHO,NVR,DHI,HOV,MDC,TOL,KBH}	Homebuilding & Construction	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:06.091376	2020-10-11 23:38:31.028166
KIM	Kimco Realty Corporation	Kimco Realty Corp is real estate investment trust that owns and operates neighborhood and community open-air shopping centers in North America.	{"Real Estate","REIT - Retail",REITs}	{CLI,LXP,WRE,BX}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:05.77456	2020-10-11 23:38:31.024353
KL	Kirkland Lake Gold Ltd.	Kirkland Lake Gold Ltd is a Canada-based gold producer. The company operates in one segment consisting of a mining and milling operation. The principal product from the mining operations of the company is the sale of gold dore bars.	{"Basic Materials",Gold,"Metals & Mining"}	{XPL,PVG}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:05.791224	2020-10-11 23:38:31.024353
KMI	Kinder Morgan Inc.	Kinder Morgan Inc is an energy infrastructure company. The Company transports natural gas, refined petroleum products, crude oil, condensate, CO2 and other products.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{XOM,EPD,SEP,AMID,SXE}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:05.80751	2020-10-11 23:38:31.028166
KMX	CarMax Inc	CarMax Inc is engaged in the automobile dealership business. Its services involve selling vehicles which includes both used and new vehicles.	{"Consumer Cyclical","Auto & Truck Dealerships",Autos}	{CPRT,CRMT,RUSHA,AN,GPI,PAG,KAR}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:05.825366	2020-10-11 23:38:31.02582
KNOP	KNOT Offshore Partners LP representing Limited Partner Interests	KNOT Offshore Partners LP owns, operates and shuttles tankers under long-term charters in North Sea and Brazil. The company provides crude oil loading, transportation, and storage services under time charters and bareboat charters.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{TNP,NAT,TK,PBR,SFL,TOO,TNK,DHT,FRO}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:05.838763	2020-10-11 23:38:31.024353
KODK	Eastman Kodak Company Common New	Eastman Kodak Co is a technology company providing imaging for business. It provides commercial products and services in technologies including materials science; digital imaging science and software; and deposition processes.	{Technology,"Consumer Electronics","Computer Hardware"}	{PRCP,DBD,GPRO,NOVT,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:05.860741	2020-10-11 23:38:31.023428
KOIN	Innovation Shares NextGen Protocol	The investment seeks to provide investment results that, before fees and expenses, track the performance of the Innovation Labs Blockchain Innovators Index (the "index").\n The fund will normally invest at least 80% of its total assets in securities of the index. The index was designed by the fund's index creator, to measure the performance of a diversified group of publicly-listed companies that use, or are involved in, blockchain ("Blockchain Innovators").	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.86347	2020-10-11 23:38:31.02582
KORP	American Century Diversified Corporate Bond	The investment seeks to provide current income.\n The portfolio managers will invest at least 80% of the fund's net assets, plus any borrowings for investment purposes, in corporate debt securities and corporate debt investments. Under normal market conditions, the weighted average duration of the fund's portfolio is expected to be between three and seven years.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.87714	2020-10-11 23:38:31.028166
KPTI	Karyopharm Therapeutics Inc.	Karyopharm Therapeutics Inc. is a clinical-stage pharmaceutical company. It focuses on the discovery, development, and commercialization of first-in-class drugs directed against nuclear transport and related targets to treat cancer and other diseases.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{IMMU,EXEL,ECYT,ABT,BMY,MRK,XLV}	Drug Manufacturers	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:05.881844	2020-10-11 23:38:31.02423
KREF	KKR Real Estate Finance Trust Inc.	KKR Real Estate Finance Trust Inc is a real estate finance company that focuses primarily on originating and acquiring senior mortgage loans secured by CRE assets.	{"Financial Services","Specialty Finance",Banks}	{NYMT,O,NTP}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.895747	2020-10-11 23:38:31.025871
KSU	Kansas City Southern	Kansas City Southern is engaged in the development of roadways and railways. The company is focused on growing the north/south freight corridor connecting key commercial and industrial markets in the United States.	{Industrials,Railroads,"Transportation & Logistics"}	{CNI,CP,GWR,NSC,UNP,CSX}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:05.918919	2020-10-11 23:38:31.028166
KT	KT Corporation	KT Corp operates in the telecom industry. Its services include local, long distance, and international calling, satellite communication, data transmission, and wireless telephone services.	{"Communication Services","Telecom Services"}	{SKM,IIJI,SIFY,CHT}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:05.928371	2020-10-11 23:38:31.025871
KW	Kennedy-Wilson Holdings Inc.	Kennedy-Wilson Holdings Inc is a global real estate investment company. It owns, operates and invests in real estate through investment management platform. It generates a majority of its revenue in the form of Rental income.	{"Real Estate","Real Estate Services"}	{JLL,HF,ARL,APTS,MMI}	Real Estate Services	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:05.948603	2020-10-11 23:38:31.025871
KIN	Kindred Biosciences Inc.	Kindred Biosciences Inc is a development-stage biopharmaceutical company. It is engaged in saving and improving the lives of pets. The Company's product candidates are SentiKind, KIND-010 and KIND-012.	{Healthcare,Biotechnology}	{ICCC,LLY,MRK,HSKA,ZTS,PETX,PAHC,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:05.776381	2020-10-11 23:38:31.023007
KLAC	KLA-Tencor Corporation	KLA-Tencor Corp designs and manufactures yield-management and process-monitoring systems for the semiconductor industry. The company also provides systems for optical metrology and e-beam metrology.	{Technology,"Semiconductor Equipment & Materials",Semiconductors}	{AMAT,AEIS,ASYS,LRCX,NANO,NVMI,CAMT,ASML,RTEC,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:05.795826	2020-10-11 23:38:31.023428
KMPA	Kemper Corporation 7.375% Subordinated Debentures due 2054		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:05.811568	2020-10-11 23:38:31.021426
KN	Knowles Corporation	Knowles Corporation manufactures and sells micro-acoustic, specialty components, and human interface solutions and offers products such as hearing aid components, micro-electro-mechanical systems, microphones and transducers, among others.	{Technology,"Communication Equipment"}	{FEIM,AVX,CTS,GE,LGL,XLK}	Communication Equipment	Technology	New York Stock Exchange	\N	2020-10-11 23:12:05.828587	2020-10-11 23:38:31.023007
KNSL	Kinsale Capital Group Inc.	Kinsale Capital Group Inc is an insurance holding company. The company is engaged in offering property, casualty and specialty insurance products. It offers property insurance to Primary Property, Excess Property, and Inland Marine divisions.	{"Financial Services","Insurance - Property & Casualty"}	{MFC,NAVG,MHLD,MTG}	Insurance - Property & Casualty	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:05.842791	2020-10-11 23:38:31.021426
KOL	VanEck Vectors Coal	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the MVIS® Global Coal Index.\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The index includes companies in the global coal industry that generate at least 50% of their revenues from coal operation (production, mining and cokeries), transportation of coal, production of coal mining equipment as well as from storage and trade. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.863931	2020-10-11 23:38:31.023428
KOPN	Kopin Corporation	Kopin Corp is a developer of wearable technologies and display products which include components and concept systems.	{Technology,Semiconductors}	{CREE,AAPL,SWKS,HIMX,COL,LPL,EMAN,MSI,MVIS,HPQ,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:05.876931	2020-10-11 23:38:31.024353
KORU	Direxion Daily South Korea Bull 3X Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the daily performance of the MSCI Korea 25/50 Index.\n The fund invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, exchange-traded funds ("ETFs") that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index is designed to measure the performance of the large- and mid-cap segments of the South Korean equity market, covering approximately 85% of the free float-adjusted market capitalization of South Korean issuers. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.877796	2020-10-11 23:38:31.023428
KRA	Kraton Corporation	Kraton Corp is a specialty chemicals company which is engaged in manufacturing styrenic block copolymers and other engineered polymers. It also manufactures value-added specialty products. The majority of revenue comes from the United States and Europe.	{"Basic Materials","Specialty Chemicals",Chemicals}	{LNDC,EMN,XOM,POL,WLK,HUN,TSE}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:05.892057	2020-10-11 23:38:31.02582
KRE	SPDR S&P Regional Banking	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of an index derived from the regional banking segment of the U.S. banking industry.\n In seeking to track the performance of the S&P Regional Banks Select Industry Index (the "index"), the fund employs a sampling strategy. It generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index represents the regional banks segment of the S&P Total Market Index ("S&P TMI"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.892715	2020-10-11 23:38:31.024353
KRP	Kimbell Royalty Partners Representing Limited Partner Interests	Kimbell Royalty Partners LP is a limited partnership formed to own and acquire mineral and royalty interests in oil and natural gas properties throughput United States.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{NRT,QTRH}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:05.909708	2020-10-11 23:38:31.024353
KSM	DWS Strategic Municipal Income Trust	DWS Strategic Municipal Income Trust is a closed-end, diversified management investment company. The fund's investment objective is to provide a high level of current income exempt from federal income tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.909944	2020-10-11 23:38:31.023428
KTCC	Key Tronic Corporation	Key Tronic Corp is engaged in electronic manufacturing services for original equipment manufacturers and also manufactures keyboards and other input devices.	{Technology,"Computer Systems","Computer Hardware"}	{NSYS,SGMA,MRCY,PLXS,IMMR,IEC,JBL,BHE,SANM,XLK}	Computer Hardware	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:05.93005	2020-10-11 23:38:31.021426
KMDA	Kamada Ltd.	Kamada Ltd is an orphan drug, plasma-derived protein therapeutics company and develops & produces specialty plasma-derived protein therapeutics. Its flagship product include Glassia.	{Healthcare,Biotechnology}	{VRTX,CGEN,ANTH,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:05.807548	2020-10-11 23:38:31.025871
KMT	Kennametal Inc.	Kennametal Inc is engaged in developing and manufacturing metalworking tools and wear-resistant engineered components and coatings. The company's operating segments include Industrial, Widia and Infrastructure.	{Industrials,"Tools & Accessories","Industrial Products"}	{LECO,FAST,HURC,MSM,CRS,DOV,SPXC}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:05.827521	2020-10-11 23:38:31.023428
KO	Coca-Cola Company (The)	Coca-Cola Co is nonalcoholic beverage company which manufactures a variety of carbonated and noncarbonated brands, including Coca-Cola, Diet Coke, Fanta, Sprite, Minute Maid, Powerade, and Dasani.	{"Consumer Defensive","Beverages - Soft Drinks","Beverages - Non-Alcoholic"}	{COT,FIZZ,KOF,REED,MNST,CCEP,PEP}	Beverages - Non-Alcoholic	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:05.860764	2020-10-11 23:38:31.031328
KR	Kroger Company (The)	The Kroger Co operates grocery stores. Its coverage includes retail food and drug stores, multi-department stores, jewelry stores and convenience stores.	{"Consumer Defensive","Grocery Stores","Retail - Defensive"}	{IMKTA,WMT,SFM}	Retail - Defensive	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:05.882748	2020-10-11 23:38:31.028166
KRG	Kite Realty Group Trust	Kite Realty Group Trust is a full-service, vertically integrated real estate company. It is engaged in the ownership, operation, management, leasing, acquisition, and development of neighborhood and community shopping centers in the United States.	{"Real Estate","REIT - Retail",REITs}	{}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:05.898963	2020-10-11 23:38:31.028166
KSS	Kohl's Corporation	Kohl's Corp is an American broadlines retailer. It operates department stores that sell moderately priced exclusive and national brand apparel, footwear, accessories, beauty and home products. The company also has an e-commerce platform.	{"Consumer Cyclical","Department Stores","Retail - Apparel & Specialty"}	{DDS,JWN,JCP,TGT,M,SHOS,LE}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:05.915832	2020-10-11 23:38:31.02423
KURA	Kura Oncology Inc.	Kura Oncology Inc is a clinical stage biopharmaceutical company, engages in discovering and developing therapeutics for the treatment of solid tumors and blood cancers.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:05.939249	2020-10-11 23:38:31.028166
KWR	Quaker Chemical Corporation	Quaker Chemical Corp manufactures and sells chemicals and chemical-based products. The company's product portfolio includes rolling lubricants, machining and grinding compounds, and corrosion preventives used by steel and aluminum manufacturers.	{"Basic Materials","Specialty Chemicals",Chemicals}	{NTIC,ACET,WDFC,HWKN,IPHS,CCF}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:05.94879	2020-10-11 23:38:31.023007
LABD	Direxion Daily S&P Biotech Bear 3X Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the inverse of the daily performance of the S&P Biotechnology Select Industry Index.\n The fund, under normal circumstances, invests in swap agreements, futures contracts, short positions or other financial instruments that, in combination, provide inverse (opposite) or short leveraged exposure to the index equal to at least 80% of the fund's net assets (plus borrowing for investment purposes). The index is designed to measure the performance of the biotechnology sub-industry based on the Global Industry Classification Standards ("GICS"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.960341	2020-10-11 23:38:31.02423
LAMR	Lamar Advertising Company	Lamar Advertising Co operates in the advertising industry. It leases space for advertising on billboards, buses, shelters, benches, logo plates and airport terminals.	{"Real Estate","REIT - Diversified",REITs}	{NCMI,AMCN,IPG,OMC,CCO,OUT}	REITs	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:05.984738	2020-10-11 23:38:31.028166
LAUR	Laureate Education Inc.	Laureate Education Inc is an international community of universities. The company provides higher education programs and services to students through an international network of licensed universities and higher education institutions.	{"Consumer Defensive","Education & Training Services",Education}	{NAUH,LAUR}	Education	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:06.005381	2020-10-11 23:38:31.023007
MSBI	Midland States Bancorp Inc.	Midland States Bancorp Inc through its subsidiary provides traditional community banking and other complementary financial services, including commercial lending, residential mortgage origination, merchant services and prime consumer lending.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:07.235617	2020-10-11 23:38:31.031328
KIQ	Kelso Technologies Inc	Kelso Technologies Inc is a railroad equipment supplier. It is engaged in producing & selling proprietary tank car service equipment used in the safe loading, unloading and containment of hazardous materials during transport.	{Industrials,Railroads,"Transportation & Logistics"}	{RAIL,TRN,WAB}	Transportation & Logistics	Industrials	NYSE American	\N	2020-10-11 23:12:05.785483	2020-10-11 23:38:31.028166
KLIC	Kulicke and Soffa Industries Inc.	Kulicke & Soffa Industries Inc is engaged in the semiconductor industry. It manufactures and sells capital equipment and expendable tools used to assemble semiconductor devices.	{Technology,"Semiconductor Equipment & Materials",Semiconductors}	{AMAT,CVV,ASYS,KLAC,TXN,ASML,RTEC,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:05.806422	2020-10-11 23:38:31.023007
KNX	Knight-Swift Transportation Holdings Inc.	Knight-Swift Transportation Holdings Inc through its subsidiaries, operates the largest truckload fleet in North America as well as extensive rail intermodal and non-asset based freight brokerage and logistics management services.	{Industrials,Trucking,"Transportation & Logistics"}	{PTSI,CVTI,HTLD,MRTN,USAK,WERN,ULH}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:05.848175	2020-10-11 23:38:31.028166
KOSS	Koss Corporation	Koss Corporation is a U.S based company engaged in the design, manufacture and sale of stereo headphones and related accessory products.	{Technology,"Consumer Electronics","Computer Hardware"}	{UEIC,IIN,LRAD,MSN,VOXX,XLK}	Computer Hardware	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:05.880636	2020-10-11 23:38:31.031328
KRNY	Kearny Financial	Kearny Financial Corp operates federally-chartered stock savings bank. The company is engaged in the business of attracting deposits from the public. These deposits are used to originate or purchase loans for its portfolios and invest in securities.	{"Financial Services","Savings & Cooperative Banks",Banks}	{OCFC,MGYR,ORIT,PFS,NFBK,MSBF}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:05.905865	2020-10-11 23:38:31.02582
KTF	DWS Municipal Income Trust	DWS Municipal Income Trust is a closed end fund. Its objective is to provide high current income exempt from federal income tax by investing in a diversified portfolio of investment-grade tax-exempt securities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.931128	2020-10-11 23:38:31.02582
KURE	KraneShares MSCI All China Health Care Index	The investment seeks to provide investment results that, before fees and expenses, track the price and yield performance of the MSCI China All Shares Health Care 10/40 Index (the "underlying index").\n The fund will invest at least 80% of its total assets in components of the underlying index, depositary receipts, including American depositary receipts, representing such components and securities underlying depositary receipts in the underlying index. The underlying index is a free float adjusted market capitalization weighted index designed to track the equity market performance of Chinese companies engaged in the healthcare sector. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.942702	2020-10-11 23:38:31.02582
LABL	Multi-Color Corporation	Multi-Color Corp provides label solutions for home & personal care, liquor, food & beverage, healthcare & consumer products. It also provides print methods including flexographic, lithographic, rotogravure, letterpress & in-house pre-press services.	{Industrials,"Business Services"}	{INWK,CMPR,BRC,RRD}	Business Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:05.959866	2020-10-11 23:38:31.024353
LAC	Lithium Americas Corp.	Lithium Americas Corp is focused on is advancing two lithium projects, the Cauchari-Olaroz Lithium Project located in Jujuy, Argentina and Lithium Nevada project located in north-western Nevada and on the manufacturing and sales of organoclay product.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:05.969118	2020-10-11 23:38:31.025871
LAKE	Lakeland Industries Inc.	Lakeland Industries Inc manufactures and sells safety garments & accessories for industrial protective clothing market. It offers limited use / disposable protective clothing, chemical protective suits, firefighting & heat protective apparel etc.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{SGC,KMB,HON,OXM,VFC,DLA,DECK,XLV}	Medical Instruments & Equipment	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:05.978516	2020-10-11 23:38:31.024353
LARK	Landmark Bancorp Inc.	Landmark Bancorp Inc is a bank holding company which through its subsidiary, Landmark National Bank, attracts deposits from the general public and uses such deposits to originate commercial real estate and municipal and consumer loans.	{"Financial Services","Banks - Regional - US",Banks}	{SMBC,UMBF,ATLO,WTBA,GFED,CBSH,GSBC,HWBK,MOFG}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:05.99739	2020-10-11 23:38:31.02582
LAWS	Lawson Products Inc.	Lawson Products Inc is a distributor of products and services to the industrial, commercial, institutional and governmental maintenance, repair and operations marketplace. It also offers automotive, electrical, and material handling equipment.	{Industrials,"Industrial Distribution"}	{FAST,DXPE,HEES,MSM,AXE,AIT,B,GWW,KAMN}	Industrial Distribution	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:06.001831	2020-10-11 23:38:31.021426
KIRK	Kirkland's Inc.	Kirkland's Inc is a retailer of home decor and gifts. Its stores present a selection of distinctive merchandise, including framed art, mirrors, wall decor, candles & related items, lamps, decorative accessories, accent furniture, textiles & among others.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{URBN,BBBY,ETH,PIR,TJX,WSM,GSY,RH}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:05.787318	2020-10-11 23:38:31.021426
KMB	Kimberly-Clark Corporation	Kimberly-Clark Corp is engaged in the manufacturing and marketing of a range of products made from natural or synthetic fibers using technologies in fibers, nonwovens, and absorbency. It offers personal care, consumer tissue, and K-C professional.	{"Consumer Defensive","Household & Personal Products","Consumer Packaged Goods"}	{}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:05.808611	2020-10-11 23:38:31.02582
KNOW	Direxion All Cap Insider Sentiment Shares	The investment seeks investment results before fees and expenses that track the Sabrient Multi-Cap Insider/Analyst Quant-Weighted Index.\n The fund, under normal circumstances, invests at least 80% of its assets in the securities that comprise the index. The index is composed of 100 stocks selected from the S&P 1500® by the index provider using a quantitative methodology. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.839325	2020-10-11 23:38:31.028166
KOP	Koppers Holdings Inc.	Koppers Holdings Inc functions within the specialty chemicals industry. Its products and services are used in railroad, specialty chemical, utility, rubber & construction industries.	{"Basic Materials","Specialty Chemicals",Chemicals}	{OMN,CBT,NSC,UNP,HUN,CSX}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:05.870684	2020-10-11 23:38:31.023007
KRO	Kronos Worldwide Inc	Kronos Worldwide Inc manufactures and sells titanium dioxide pigments which are used in a variety of products including coatings on automobiles, traffic paint, aircraft, machines, appliances, and in both commercial and residential interiors and exteriors.	{"Basic Materials","Specialty Chemicals",Chemicals}	{VHI,SXT,HUN,NL,TROX,TORM}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:05.906455	2020-10-11 23:38:31.023007
KTH	Structures Products Cp 8% CorTS Issued by Peco Energy Cap Tr II Preferred Stock		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:05.931505	2020-10-11 23:38:31.024353
KYN	Kayne Anderson MLP/Midstream Investment Company	The investment objective is to obtain a high after-tax total return by investing at least 85% of our total assets including borrowings in energy-related master limited partnerships (MLPs) and their affiliates and in other companies that, as their principal business, operate assets used in the gathering, transporting, processing, storing, refining, distributing, mining or marketing natural gas, natural gas liquids, crude oil, refined petroleum products or coal.	{"Financial Services","Asset Management"}	{}			New York Stock Exchange	\N	2020-10-11 23:12:05.953647	2020-10-11 23:38:31.021426
LANC	Lancaster Colony Corporation	Lancaster Colony Corp is a manufacturer and marketer of specialty food products for the retail and food service markets. The Company offers dressings, sauces, croutons, fruit glazes, vegetable dips, fruit dips, frozen breads and yeast rolls.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{SENEA,JJSF,HAIN,CPB,CAG,SJM,THS}	Consumer Packaged Goods	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:05.990604	2020-10-11 23:38:31.025871
LB	L Brands Inc.	L Brands Inc is a women's intimate, personal care, and beauty retailer. The company through its retail outlets sells products under various brands such as Victoria's Secret, Pink, Bath and Body Works, La Senza, and Henri Bendel brands.	{"Consumer Cyclical","Apparel Stores","Retail - Apparel & Specialty"}	{ROST,CHS,GPS,TJX,TGT,CBK,NWY,EXPR,ASNA}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:06.006317	2020-10-11 23:38:31.024353
LBCC	Long Blockchain Corp.	Long Blockchain Corp is engaged in developing and investing in globally scalable blockchain technology solutions that creates long term value for its shareholders and the wider community by investing in and developing businesses that are "on-chain".	{}	{XLK}	Application Software	Technology	OTC Pink	\N	2020-10-11 23:12:06.011692	2020-10-11 23:38:31.025871
LCA	Landcadia Holdings Inc.	Landcadia Holdings Inc is a blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:06.036423	2020-10-11 23:38:31.025871
KKR	KKR & Co. Inc. Class A	KKR & Co LP is a global investment firm which invests across multiple asset classes including private equity, energy, infrastructure, real estate, growth equity, credit and hedge funds.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.791258	2020-10-11 23:38:31.028166
KNDI	Kandi Technologies Group Inc.	Kandi Technologies Group Inc is an automobile manufacturer with focus on electric vehicles primarily in the Chinese market. Other product offerings include all-terrain vehicles, go-karts, utility vehicles, battery packs, motor vehicles, among others.	{"Consumer Cyclical","Auto Manufacturers",Autos}	{FSS,PII,TTM}	Autos	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:05.832194	2020-10-11 23:38:31.025871
KONA	Kona Grill Inc.	Kona Grill Inc owns and operates upscale casual dining restaurants in the United States. Its restaurants offer prepared food items and alcoholic beverages.	{"Consumer Cyclical",Restaurants}	{DAVE,CAKE,RRGB,TXRH,BJRI,CHUY,BLMN,SAUC}	Restaurants	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:05.872403	2020-10-11 23:38:31.02582
KRMA	Global X Conscious Companies ETF	The investment seeks to provide investment results that correspond generally to the price and yield performance, before fees and expenses, of the Concinnity Conscious Companies Index.\n The fund invests at least 80% of its total assets in the securities of the underlying index. The underlying index is designed to provide exposure to companies listed in the U.S. that operate their businesses in a sustainable and responsible manner, as measured by their ability to achieve positive outcomes that are consistent with a multi-stakeholder operating system ("MsOS"), as defined by the provider of the underlying index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:05.905708	2020-10-11 23:38:31.023428
KTOV	Kitov Pharma Ltd.	Kitov Pharma Ltd is engaged in the development of a combination drug that aims to treat osteoarthritis (OA) pain and hypertension called KIT-302 and NT219.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:05.933848	2020-10-11 23:38:31.023428
KZIA	Kazia Therapeutics Limited	Kazia Therapeutics Ltd is an oncology-focused biotechnology company, engaged in developing anti-cancer drugs. It has two clinical staged drug development candidates and an early stage discovery program.	{Healthcare,Biotechnology}	{ADM,PRAN,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:05.955771	2020-10-11 23:38:31.028166
LAND	Gladstone Land Corporation	Gladstone Land Corp is an externally-managed real estate company. The Company invests in farmlands located in agricultural markets throughout the USA. It owns farms and cooling facilities for the storage and assembly of boxes for shipping produce.	{"Real Estate","REIT - Industrial",REITs}	{PCH,PW,CTT,SIR}	REITs	Real Estate	NASDAQ Global Market	\N	2020-10-11 23:12:05.993099	2020-10-11 23:38:31.023428
LAZ	Lazard LTD. LTD. Class A	Lazard Ltd is a financial advisory and asset management firm. It offers corporate, institutional, government and individuals with financial advisory services for mergers and acquisitions, restructurings, capital structure, capital raising and others.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{CS,DB,GHL,EVR,BX,IX}	Brokers & Exchanges	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.001686	2020-10-11 23:38:31.028166
LBC	Luther Burbank Corporation	Luther Burbank Corp is a bank holding company. The company through its subsidiary provides real estate secured lending in attractive metropolitan areas along the West Coast.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:06.009564	2020-10-11 23:38:31.02423
LBRDK	Liberty Broadband Corporation	Liberty Broadband Corp serves in the telecommunications industry in the United States. It provides cable services to both residential and small to medium businesses through its fiber, hybrid fiber, and coaxial cable infrastructure.	{"Communication Services","Pay TV"}	{BCOM,VG,RNET,GTT}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:12:06.02468	2020-10-11 23:38:31.023007
LBTYK	Liberty Global plc	Liberty Global PLC is a holding company engaged in making investments into international television and broadband companies. Through its subsidiaries, it offers telecommunication, broadband services and cable network.	{"Communication Services","Pay TV"}	{RCI,SJR,DISH,CHTR}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:12:06.028407	2020-10-11 23:38:31.028166
LCNB	LCNB Corporation	LCNB Corp is the holding company for the National Bank. Its business involves the offering of personal and business banking solutions in the United States.	{"Financial Services","Banks - Regional - US",Banks}	{PEBO,UBCP,UBOH,FFBC,FUNC,HBAN,PRK,CIVB}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:06.048922	2020-10-11 23:38:31.023007
LCI	Lannett Co Inc	Lannett Co Inc develops, manufactures, packages, markets and distributes generic versions of branded pharmaceutical products. Its products include solid oral dosage forms, including oral solutions, nasal, topicals or parentarels.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{AKRX,GSK,NVS,ABT,JNJ,PFE,MYL,XLV}	Drug Manufacturers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:06.046229	2020-10-11 23:38:31.023428
LC	LendingClub Corporation	LendingClub Corp is an online marketplace connecting borrowers and investors to engage in transactions relating to standard or custom program loans.	{"Financial Services","Credit Services"}	{WRLD}	Credit Services	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.031887	2020-10-11 23:38:31.02582
LDOS	Leidos Holdings Inc.	Leidos Holdings Inc is engaged in providing scientific, engineering, system integration, technical services, and solutions to various government entities, including the U.S. Department of Defense and the Department of Homeland Security.	{Technology,"Information Technology Services","Application Software"}	{OSIS,LMT,CACI,GD,NOC,UIS,LLL,DXC,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:06.059567	2020-10-11 23:38:31.028166
LEMB	iShares J.P. Morgan EM Local Currency Bond	The investment seeks to track the investment results of the J.P. Morgan GBI-EM Global Diversified 15% Cap 4.5% Floor Index composed of local currency denominated, emerging market sovereign bonds.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents, including shares of money market funds advised by BFA or its affiliates. The index tracks the performance of local currency-denominated sovereign bond markets of emerging market countries. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.089604	2020-10-11 23:38:31.02582
LFIN	Longfin Corp - Ordinary Shares - Class A	Longfin Corp provides Financing and FX services to importers and exporters. It specializes in structured trade finance solutions and physical commodities finance solutions for finance houses and trading platforms.	{}	{NTXFF,NOAH,OAK,XLK}	Application Software	Technology	OTC Caveat Emptor	\N	2020-10-11 23:12:06.105865	2020-10-11 23:38:31.028166
LH	Laboratory Corporation of America Holdings	Laboratory Corp of America Holdings is a healthcare diagnostics company. It is engaged in providing comprehensive clinical laboratory and end-to-end drug development services. It operates in two segments: LabCorp Diagnostics and Covance Drug Development.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{IDXX,DGX,ENZ,NEO,PRPO,QGEN,XLV}	Medical Diagnostics & Research	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:06.127534	2020-10-11 23:38:31.024353
LION	Fidelity Southern Corporation	Fidelity Southern Corporation operates as the bank holding company based in the metropolitan Atlanta area. The bank provides a full range of financial products and services for retail customers and small to medium-sized businesses.	{"Financial Services","Banks - Regional - US",Banks}	{CCBG,CBAN,FCNCA,ABCB,STI,SGB,STBZ,UCBI,IBTX}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:06.148581	2020-10-11 23:38:31.024353
LKSD	LSC Communications Inc.	LSC Communications Inc provides printing services primarily in the United States. It caters to publishers of magazines, catalogs, retail inserts, books, and directories.	{Industrials,"Business Services"}	{LABL,KRNT,LKSD}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:06.180752	2020-10-11 23:38:31.028166
LMB	Limbach Holdings Inc.	Limbach Holdings Inc is a commercial specialty contractor in the fields of heating, ventilation, air conditioning, plumbing, electrical and building controls for new and renovated buildings, maintenance services, energy retrofits and equipment upgrades.	{Industrials,"Engineering & Construction"}	{RUN,LMB}	Engineering & Construction	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:06.201574	2020-10-11 23:38:31.02423
LODE	Comstock Mining Inc.	Comstock Mining Inc is a mining company with a focus on gold and silver deposits in the Nevada region. Its operations are divided into two segments, namely mining and real estate. The majority revenues are generated from the mining segment.	{"Basic Materials",Gold,"Metals & Mining"}	{RGLD,VGZ}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:06.25296	2020-10-11 23:38:31.02423
LOXO	Loxo Oncology Inc.	Loxo Oncology Inc is a biopharmaceutical company. It is engaged in the development of selective medicines for patients with genetically defined cancers.	{Healthcare,Biotechnology}	{PBH,GNC,ATRS,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:06.289257	2020-10-11 23:38:31.024353
LPT	Liberty Property Trust	Liberty Property Trust is a self-administered and self-managed real estate investment trust. The company primarily manages and leases industrial and office properties throughout the United States.	{"Real Estate","REIT - Office",REITs}	{}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:06.307537	2020-10-11 23:38:31.028166
LRGF	iShares Edge MSCI Multifactor USA	The investment seeks to track the investment results of the MSCI USA Diversified Multiple-Factor Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The underlying index is designed to select equity securities from the MSCI USA Index (the "parent index") that have high exposure to four investment style factors: value, quality, momentum and low size, while maintaining a level of risk similar to that of the parent index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.326371	2020-10-11 23:38:31.031328
LSXMK	Liberty Media Corporation Series C Liberty SiriusXM Common Stock	Liberty SiriusXM Group through its subsidiary holding is engaged in providing a subscription based satellite radio service. It transmits music, sports, entertainment, comedy, talk, news, traffic and weather channels, as well as infotainment services.	{"Consumer Cyclical","Broadcasting - Radio",Entertainment}	{}	Entertainment	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:06.350352	2020-10-11 23:38:31.028166
LCAHW	Landcadia Holdings Inc. Warrant		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:06.045755	2020-10-11 23:38:31.031328
LDRS	Innovator IBD ETF Leaders	The investment seeks to track, before fees and expenses, the performance of the IBD® ETF Leaders Index.\n The fund will normally invest at least 80% of its net assets (including investment borrowings) in the exchange-traded investment companies that comprise the index. It generally invests in all of the securities comprising the index in proportion to the weightings of the securities in the index. The index is a rules-based index owned and developed by Investor's Business Daily® designed to attempt to outperform the broader equity and fixed income markets. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.065523	2020-10-11 23:38:31.021426
LEJU	Leju Holdings Limited American Depositary Shares each representing one	Leju Holdings Ltd provides O2O real estate services in China. It offers real estate e-commerce, online advertising and online listing services through its online platform, which comprises local websites and various mobile applications.	{"Real Estate","Real Estate Services"}	{BIDU,ZG,SFUN}	Real Estate Services	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:06.083949	2020-10-11 23:38:31.023428
LFEQ	VanEck Vectors NDR CMG Long/Flat Allocation	The investment seeks to replicate as closely as possible, the price and yield performance of the Ned Davis Research CMG US Large Cap Long/Flat Index.\n The fund normally invests at least 80% of its total assets in securities that track and/or comprise the fund's benchmark index. The index is a rules-based index that follows a proprietary model developed by Ned Davis Research, Inc. in conjunction with CMG Capital Management Group, Inc. To help limit potential loss associated with adverse market conditions, the model produces trade signals that dictate the index's equity allocation ranging from 100% fully invested to 100% in cash. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.105532	2020-10-11 23:38:31.02582
LGL	LGL Group Inc. (The)	LGL Group Inc is a U.S based holding company. It operates through electronic components segment, which includes all products manufactured and sold by its subsidiaries.	{Technology,"Electronic Components","Computer Hardware"}	{LFUS,SWKS,AVX,CTS,DOV,KN,XLK}	Computer Hardware	Technology	NYSE American	\N	2020-10-11 23:12:06.120967	2020-10-11 23:38:31.023428
LILA	Liberty Latin America Ltd.	Liberty Latin America Ltd is a telecommunications company. It offers communications and entertainment services including video, broadband internet, telephony and mobile services to its residential and business customers.	{"Communication Services","Pay TV"}	{}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:12:06.140804	2020-10-11 23:38:31.028166
LITE	Lumentum Holdings Inc.	Lumentum Holdings Inc offers optical and photonic products addressing end markets including data communications and telecommunications networking and industrial and commercial lasers for manufacturing, inspection and life-science applications.	{Technology,"Communication Equipment"}	{OCC,OCLR,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:06.158611	2020-10-11 23:38:31.021426
LM	Legg Mason Inc.	Legg Mason Inc is an asset management company. It provides related financial services to individuals, institutions, corporations and municipalities through its subsidiaries.	{"Financial Services","Asset Management"}	{GROW,VRTS,AMG,WDR,EV,BEN,BLK,AB,IVZ,MN}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.202742	2020-10-11 23:38:31.031328
LND	Brasilagro Brazilian Agric Real Estate Co Sponsored ADR (Brazil)	Brasilagro - Cia Bras de Prop Agricolas is engaged in acquisition, development, operation and sale of rural properties suitable for agricultural activities.	{"Consumer Defensive","Farm Products","Consumer Packaged Goods"}	{ANDE,ALCO,SHI,BG,SEB,AGRO,TREC}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:06.231592	2020-10-11 23:38:31.02582
LOOP	Loop Industries Inc.	Loop Industries Inc is associated with waste recycling activities primarily in the United States. It employs the de-polymerization process to convert waste plastic products into raw materials for the manufacture of virgin plastic.	{Industrials,"Waste Management"}	{NSIT,MANT,BSQR,VHC,CSLT}	Waste Management	Industrials	NASDAQ Global Market	\N	2020-10-11 23:12:06.270507	2020-10-11 23:38:31.024353
LOGI	Logitech International S.A.	Logitech International SA designs, manufactures and markets products that allow people to connect through music, gaming, video, computing, and other digital platforms.	{Technology,"Computer Systems","Computer Hardware"}	{CSCO,MSFT,IMMR,SYNA,PLT,SNE,VOXX,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:06.253706	2020-10-11 23:38:31.028166
LPSN	LivePerson Inc.	LivePerson Inc along with its subsidiaries provides digital engagement solutions offering a cloud-based platform which enables businesses to proactively connect with consumers through chat, voice, and content delivery.	{Technology,"Software - Infrastructure","Application Software"}	{CSCO,MSFT,NICE,CRM,SREV,EGAN,ORCL,FIVN,ZEN,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:06.298968	2020-10-11 23:38:31.023007
LRCX	Lam Research Corporation	Lam Research Corp functions in the semiconductor industry. It develops and sells semiconductor processing systems that are used in the fabrication of integrated circuits.	{Technology,"Semiconductor Equipment & Materials",Semiconductors}	{AMAT,CVV,VECO,ASYS,KLAC,ACLS,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:06.319667	2020-10-11 23:38:31.02423
LSTR	Landstar System Inc.	Landstar System Inc is an asset-light provider of integrated transportation management solutions. The company offers services to its customers across multiple transportation modes.	{Industrials,"Integrated Shipping & Logistics","Transportation & Logistics"}	{PTSI,CVTI,HTLD,JBHT,USAK,WERN,ULH,RRTS,KNX}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:06.340942	2020-10-11 23:38:31.023428
LDF	Latin American Discovery Fund Inc. (The)	Latin American Discovery Fund Inc is a United States based non-diversified, closed-end management investment company. Its investment objective is long-term capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.057146	2020-10-11 23:38:31.02423
LEE	Lee Enterprises Incorporated	Lee Enterprises Inc is a premier publisher of local news, information and advertising in midsize markets, with 46 daily newspapers and a joint interest in four others, rapidly growing online sites.	{"Consumer Cyclical",Publishing}	{DJCO,NYT,AHC,NWSA,NEWM,TGNA,MNI,SSP}	Publishing	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:06.079981	2020-10-11 23:38:31.028166
LEXEA	Liberty Expedia Holdings Inc. Series A Common Stock	Liberty Expedia Holdings Inc is an online travel company. It operates a strong brand portfolio with global reach, targeting a broad range of travelers, travel suppliers, and advertisers.	{Technology,"Internet Content & Information","Online Media"}	{XLK}	Online Media	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:06.100969	2020-10-11 23:38:31.028166
LINC	Lincoln Educational Services Corporation	Lincoln Educational Services Corp provides diversified career-oriented post-secondary education to recent high school graduates and working adults. It offers programs in automotive technology, healthcare and hospitality service and information technology.	{"Consumer Defensive","Education & Training Services",Education}	{CECO,APEI,LOPE,NAUH,BPI,ATGE}	Education	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:06.141207	2020-10-11 23:38:31.021426
LIVN	LivaNova PLC	LivaNova PLC is a medical technology company formed by the merger of Sorin S.p.A, engaged in the treatment of cardiovascular diseases, and Cyberonics Inc., a medical device company with core expertise in neuromodulation.	{Healthcare,"Medical Devices"}	{CNMD,BABY,BSX,PKI,XLV}	Medical Devices	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:06.167332	2020-10-11 23:38:31.02423
LMHB	Legg Mason Inc. 5.45% Junior Subordinated Notes due 2056		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:06.207431	2020-10-11 23:38:31.028166
LMRKO	Landmark Infrastructure Partners LP Preferred Units	Landmark Infrastructure Partners LP acquires, owns and manages a portfolio of real property interests that it leases to companies in the wireless communication, outdoor advertising and renewable power generation industries.	{"Real Estate","Real Estate Services"}	{}	Real Estate Services	Real Estate	NASDAQ Global Market	\N	2020-10-11 23:12:06.224516	2020-10-11 23:38:31.021426
LOB	Live Oak Bancshares Inc.	Live Oak Bancshares Inc is a nationwide lender to small businesses. It offers loans comprising small business administration loans; conventional commercial business loans to small- and medium-sized businesses; and construction loans.	{"Financial Services","Savings & Cooperative Banks",Banks}	{MPB,MOFG}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:06.253497	2020-10-11 23:38:31.02582
LORL	Loral Space and Communications Inc.	Loral Space & Communications Inc is a provider of satellite-based communications services. It offers video distribution and contribution, and occasional use services as well as telecommunication carrier and integrator services through its subsidiaries.	{Technology,"Communication Equipment"}	{GILT,VSAT,CMTL,SATS,DISH,HRS,I,GSAT,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:06.273094	2020-10-11 23:38:31.02423
LPG	Dorian LPG Ltd.	Dorian LPG Ltd is an LPG shipping company engaged in owning and operating liquefied petroleum gas tankers.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{TGP,SALT,FRO,DRYS}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:06.291477	2020-10-11 23:38:31.031328
LQDH	iShares Interest Rate Hedged Corporate Bond	The investment seeks to mitigate the interest rate risk of a portfolio composed of U.S. dollar-denominated, investment-grade corporate bonds.\n The fund seeks to invest, at least 80% of its net assets in U.S. dollar-denominated investment-grade bonds, in one or more underlying funds that principally invest in investment-grade bonds, and in U.S. Treasury securities (or cash equivalents). It is an actively managed exchange-traded fund that does not seek to replicate the performance of a specified index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.316726	2020-10-11 23:38:31.028166
LSI	Life Storage Inc.	Life Storage Inc is a fully integrated, self-administered and self-managed real estate investment trust engaged in acquiring and managing self-storage properties.	{"Real Estate","REIT - Industrial",REITs}	{PSA,AKR,EXR}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:06.337578	2020-10-11 23:38:31.02582
LTPZ	Pimco 15 Year U.S. TIPS Index Exchange-Traded Fund	The investment seeks to provide total return that closely corresponds, before fees and expenses, to the total return of the ICE BofAML 15+ Year US Inflation-Linked Treasury Index.\n The fund invests at least 80% of its total assets (exclusive of collateral held from securities lending) in the component securities of the ICE BofAML 15+ Year US Inflation-Linked Treasury Index (the "underlying index"). The underlying index is an unmanaged index comprised of Treasury Inflation-Protected Securities ("TIPS") with a maturity of at least 15 years.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.361903	2020-10-11 23:38:31.025871
LCUT	Lifetime Brands Inc.	Lifetime Brands Inc is a Delaware based company headquartered in New York. It designs, sources and sells branded kitchenware, tableware and other products. The company has three operating segments including U.S Wholesale, International, and Retail Direct.	{"Consumer Cyclical","Home Furnishings & Fixtures","Manufacturing - Apparel & Furniture"}	{HELE,LBY,TUP,SVT,RL}	Manufacturing - Apparel & Furniture	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:06.051385	2020-10-11 23:38:31.024353
LEA	Lear Corporation	Lear Corp designs, develops, and manufactures automotive seating and electrical systems and components. Its products include frames and mechanisms, covers, foam, and headrests as well as wiring harnesses, terminals and connectors and others.	{"Consumer Cyclical","Auto Parts",Autos}	{MGA,GNTX,DAN,APTV,VC}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:06.067404	2020-10-11 23:38:31.031328
LEXEB	Liberty Expedia Holdings Inc. Series B Common Stock	Liberty Expedia Holdings Inc is an online travel company. It operates a strong brand portfolio with global reach, targeting a broad range of travelers, travel suppliers, and advertisers.	{Technology,"Internet Content & Information","Online Media"}	{XLK}	Online Media	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:06.100753	2020-10-11 23:38:31.023428
LGI	Lazard Global Total Return and Income Fund	Lazard Global Total Return & Inc Fd Inc is a United States based diversified, closed-end management investment company. Its investment objective is total return consisting of capital appreciation and income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.118773	2020-10-11 23:38:31.02423
LILAK	Liberty Latin America Ltd.	Liberty Latin America Ltd is a telecommunications company. It offers communications and entertainment services including video, broadband internet, telephony and mobile services to its residential and business customers.	{"Communication Services","Pay TV"}	{}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:12:06.139805	2020-10-11 23:38:31.023007
LJPC	La Jolla Pharmaceutical Company	La Jolla Pharmaceutical Co is an American biopharmaceutical company. It is focused on the discovery, development and commercialization of therapies intended to improve outcomes in patients suffering from life-threatening diseases.	{Healthcare,Biotechnology}	{AMGN,BCRX,RGEN,ICPT,TTNP,VICL,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:06.163279	2020-10-11 23:38:31.02582
LLEX	Lilis Energy Inc.	Lilis Energy Inc is an upstream independent oil and gas company engaged in the acquisition, drilling and production of oil and natural gas properties and prospects. Its current operating activities are focused on the Denver-Julesburg Basin.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{COG,SM,REN}	Oil & Gas - E&P	Energy	NYSE American	\N	2020-10-11 23:12:06.184683	2020-10-11 23:38:31.023428
LMBS	First Trust Low Duration Mortgage Opportunities ET	The investment seeks to generate current income with a secondary objective of capital appreciation.\n Under normal market conditions, the fund will seek to achieve its investment objectives by investing at least 80% of its net assets (including investment borrowings) in mortgage-related debt securities and other mortgage-related instruments (collectively, "Mortgage-Related Investments"). The advisor normally expects to invest in Mortgage-Related Investments tied to residential and commercial mortgages. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:06.204611	2020-10-11 23:38:31.021426
LNG	Cheniere Energy Inc.	Cheniere Energy Inc is a Houston-based energy company primarily engaged in LNG-related businesses. It owns and operates the Sabine Pass LNG terminal in Louisiana. The business segments are LNG terminal business and LNG and natural gas marketing business.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{MMLP,CLNE,TOT,OKE,SRE,CVX,DCP,CQP,NGL}	Oil & Gas - Midstream	Energy	NYSE American	\N	2020-10-11 23:12:06.236387	2020-10-11 23:38:31.031328
LONE	Lonestar Resources US Inc.	Lonestar Resources US Inc is an independent oil and natural gas company, focused on the development, production and acquisition of unconventional oil, natural gas liquids (NGLs) and natural gas properties in the Eagle Ford Shale in Texas.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{}	Oil & Gas - E&P	Energy	NASDAQ Global Market	\N	2020-10-11 23:12:06.266521	2020-10-11 23:38:31.023428
LOWC	SPDR MSCI ACWI Low Carbon Target	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the MSCI ACWI Low Carbon Target Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index and in depositary receipts based on securities comprising the index. The index is designed to address two dimensions of carbon exposure - carbon emissions and fossil fuel reserves, expressed as potential emissions. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.289604	2020-10-11 23:38:31.023428
LPX	Louisiana-Pacific Corporation	Louisiana-Pacific Corp is a building products manufacturer used in home construction. The Company's business segments are North America Oriented Strand Board, Siding, Engineered Wood Products and South America.	{"Basic Materials","Building Materials"}	{UFPI,BCC,BMCH}	Building Materials	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:06.309399	2020-10-11 23:38:31.021426
LRN	K12 Inc	K12 Inc is a technology-based education company. It offers alternative programs to traditional on-campus schooling. It also operates state-funded virtual charter schools around the United States.	{"Consumer Defensive","Education & Training Services",Education}	{EDU,GHC,TWOU}	Education	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:06.326178	2020-10-11 23:38:31.023007
LDL	Lydall Inc.	Lydall Inc, through its subsidiaries designs and manufactures specialty engineered filtration media, industrial thermal insulating solutions, automotive thermal and acoustical barriers, medical filtration media and devices.	{"Consumer Cyclical","Auto Parts",Autos}	{SHLO,BWA,AWR,F,DAN,UFI,KOOL}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:06.058891	2020-10-11 23:38:31.02582
LEAD	Realty Shares DIVCON Leaders Dividend	The investment seeks long-term capital appreciation by tracking the performance, before fees and expenses, of the Reality Shares DIVCON Leaders Dividend Index.\n Under normal circumstances, at least 80% of the fund's assets will be invested in component securities of the benchmark index. The benchmark index is designed to capitalize on the theory that, over time, companies that consistently grow their dividends tend to have investment returns above overall market returns, and companies that do not grow (or cut) their dividends tend to have investment returns below overall market returns. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:06.073896	2020-10-11 23:38:31.02582
LENS	Presbia PLC	Presbia PLC manufactures ophthalmic devices. The Company is engaged in developing and marketing a proprietary optical lens implant for treating presbyopia, the age-related loss of the ability to focus on near objects.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{NUVA,EKSO,TNDM,XLV}	Medical Instruments & Equipment	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:06.092462	2020-10-11 23:38:31.024353
LFVN	Lifevantage Corporation	Lifevantage Corp is engaged in the identification, research, development and distribution of nutraceutical dietary supplements and skin care products including Protandim, LifeVantage TrueScience and Canine Health.	{"Consumer Defensive","Household & Personal Products","Consumer Packaged Goods"}	{NATR,NXTM,CYAN,USNA,MTEX,MDGL,RELV}	Consumer Packaged Goods	Consumer Defensive	NASDAQ Capital Market	\N	2020-10-11 23:12:06.11206	2020-10-11 23:38:31.025871
LHO	LaSalle Hotel Properties	LaSalle Hotel Properties is a real estate investment trust. It is engaged in buying, owning, redeveloping and leasing mainly upscale and luxury full-service hotels located in convention, resort and urban business markets.	{"Real Estate","REIT - Hotel & Motel",REITs}	{HST,DRH,PEB,RLJ,INN,SOHO,HT,HPT,CDOR}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:06.132306	2020-10-11 23:38:31.025871
LINK	Interlink Electronics Inc.	Interlink Electronics Inc designs, develops, manufactures and sells a range of force-sensing technologies that incorporate its materials technology, firmware and software into a portfolio of standard products and custom solutions.	{Technology,"Electronic Components","Computer Hardware"}	{MRCY,ALOT,CSPI,USAT,XLK}	Computer Hardware	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:06.147499	2020-10-11 23:38:31.02582
LLL	L3 Technologies Inc.	L3 Technologies Inc is a provider of various communication and electronic systems and products used on military, homeland security and commercial platforms. It is also a contractor in aerospace systems, security and detection systems, and pilot training.	{Industrials,"Aerospace & Defense"}	{FLIR,KTOS,CUB,HRS,HON,NOC,RTN,COL,ESP,ITT}	Aerospace & Defense	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:06.185518	2020-10-11 23:38:31.023007
LMHA	Legg Mason Inc. 6.375% Junior Subordinated Notes due 2056		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:06.207261	2020-10-11 23:38:31.023428
LMT	Lockheed Martin Corporation	Lockheed Martin Corp is a security and aerospace company engaged in the research, design, development, manufacture, integration, and sustainment of technology systems, products, and services.	{Industrials,"Aerospace & Defense"}	{AVAV,KTOS,BA,GD,NOC,RTN,COL}	Aerospace & Defense	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:06.226843	2020-10-11 23:38:31.023428
LNT	Alliant Energy Corporation	Alliant Energy Corp operates as a regulated investor-owned public utility holding company. It provides regulated electricity and natural gas services in the Midwest through its subsidiaries.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{MGEE,DTE,AEE,CMS,ALE,DUK,ES,WEC,XEL,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:06.242629	2020-10-11 23:38:31.024353
LOMA	Loma Negra Compania Industrial Argentina Sociedad Anonima ADS	Loma Negra Cia Industria Argentina SA is a cement producer in Argentina. The company offers cement, masonry cement, lime, concrete and aggregates.	{"Basic Materials","Building Materials"}	{}	Building Materials	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:06.261892	2020-10-11 23:38:31.021426
LPCN	Lipocine Inc.	Lipocine Inc is a specialty pharmaceutical company focused on applying its oral drug delivery technology for the development of pharmaceutical products in the area of men's and women's health.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{SIGA,SGEN,ECYT,COLL,XLV}	Drug Manufacturers	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:06.289257	2020-10-11 23:38:31.028166
LPTH	LightPath Technologies Inc.	LightPath Technologies Inc manufactures optical components and assemblies including precision molded glass aspheric optics, infrared aspheric lenses, GRADIUM glass lenses and other optical materials used to produce products that manipulate light.	{Technology,"Electronic Components","Computer Hardware"}	{FLIR,COHR,IIVI,REFR,IPGP,DYSL,VIAV,XLK}	Computer Hardware	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:06.309291	2020-10-11 23:38:31.024353
LEO	Dreyfus Strategic Municipals Inc.	Dreyfus Strategic Municipals Inc is a diversified closed-end management investment company. The fund's investment objective is to maximize current income exempt from federal income tax to the extent consistent with the preservation of capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.096234	2020-10-11 23:38:31.023007
LGC	Legacy Acquisition Corp. Class A par value $0.0001 per share	Legacy Acquisition Corp is a blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:06.111998	2020-10-11 23:38:31.031328
LHCG	LHC Group	LHC Group Inc provides post-acute health care services to patients through its home nursing agencies, community-based services agencies, hospice agencies and long-term acute care hospitals.	{Healthcare,"Medical Care","Health Care Providers"}	{AMED,BIOS,NHC,CHE,XLV}	Health Care Providers	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:06.131114	2020-10-11 23:38:31.02582
LIQT	LiqTech International Inc.	LiqTech International Inc provides technologies for gas & liquid purification by manufacturing ceramic silicon carbide filters. It provides ceramic membranes for liquid filtration & diesel particulate filters for soot exhaust particles from diesel engine.	{Industrials,"Pollution & Treatment Controls","Industrial Products"}	{ASTE,GENC,IR}	Industrial Products	Industrials	NYSE American	\N	2020-10-11 23:12:06.150399	2020-10-11 23:38:31.025871
LKQ	LKQ Corporation	LKQ Corp is a provider of alternative and mechanical replacement products. The company distributes vehicle replacement parts, components, and systems used in repair and maintenance as well as specialty products and accessories.	{"Consumer Cyclical","Auto Parts",Autos}	{CPRT,DORM,PRTS,LAD,GPC,AAP,KAR,ORLY}	Autos	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:06.176665	2020-10-11 23:38:31.028166
LMAT	LeMaitre Vascular Inc.	LeMaitre Vascular Inc provides medical devices and human tissue cryopreservation services for the treatment of peripheral vascular disease. The company develops, manufactures, and markets vascular devices to address the needs of vascular surgeons.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{ELGX,ANGO,CSII,BAX,BSX,TFX,EW,XLV}	Medical Instruments & Equipment	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:06.196969	2020-10-11 23:38:31.02582
LMRK	Landmark Infrastructure Partners LP	Landmark Infrastructure Partners LP acquires, owns and manages a portfolio of real property interests that it leases to companies in the wireless communication, outdoor advertising and renewable power generation industries.	{"Real Estate","Real Estate Services"}	{IOR,AHH}	Real Estate Services	Real Estate	NASDAQ Global Market	\N	2020-10-11 23:12:06.221654	2020-10-11 23:38:31.031328
LNN	Lindsay Corporation	Lindsay Corp designs, manufactures, and sells irrigation systems used in the agricultural industry. It also provides proprietary water management and road infrastructure products and services. The firm has Irrigation and Infrastructure reportable systems.	{Industrials,"Farm & Construction Equipment","Farm & Construction Machinery"}	{ARTW,ALG,AGCO,DE,TTC}	Farm & Construction Machinery	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:06.242856	2020-10-11 23:38:31.023007
LOR	Lazard World Dividend & Income Fund Inc.	Lazard World Dividend & Income Fd is a United States based diversified, closed-end management investment company. Its investment objective is total return through a combination of dividends, income and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.273114	2020-10-11 23:38:31.023007
LPNT	LifePoint Health Inc.	LifePoint Health Inc is a healthcare company. The company through its subsidiaries owns and operates community hospitals, regional health systems, physician practices, outpatient centers, and post-acute facilities.	{Healthcare,"Medical Care","Health Care Providers"}	{UHS,CYH,SEM,SSY,HCA,THC,XLV}	Health Care Providers	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:06.297355	2020-10-11 23:38:31.02423
LRET	ETRACS Monthly Pay 2xLeveraged MSCI US REIT Index ETN due May 5 2045	The investment seeks a return linked to the monthly compounded 2x leveraged performance of MSCI US REIT Index, less investor fees.\n The MSCI US REIT Index is a free float©\\adjusted market capitalization weighted index that is comprised of equity REIT securities. The index includes securities with exposure to core real estate (e.g. residential and retail properties) as well as securities with exposure to other types of real estate (e.g. casinos and theaters).	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.323677	2020-10-11 23:38:31.028166
LSXMA	Liberty Media Corporation Series A Liberty SiriusXM Common Stock	Liberty SiriusXM Group through its subsidiary holding is engaged in providing a subscription based satellite radio service. It transmits music, sports, entertainment, comedy, talk, news, traffic and weather channels, as well as infotainment services.	{"Consumer Cyclical","Broadcasting - Radio",Entertainment}	{}	Entertainment	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:06.34412	2020-10-11 23:38:31.021426
LTRX	Lantronix Inc.	Lantronix Inc is a networking company. It provides machine to machine (M2M) and Internet of Things (IoT) solutions. It also provides a broad portfolio of products intended to enhance the value of electronic devices and machines.	{Technology,"Communication Equipment"}	{CSCO,SILC,JCS,DGII,NTGR,EMR,ANET,XLK}	Communication Equipment	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:06.368485	2020-10-11 23:38:31.023428
MSI	Motorola Solutions Inc.	Motorola Solutions Inc is a communications equipment manufacturer. It provides communications infrastructure, devices such as two-way portable radios, and an array of communications network management and support services.	{Technology,"Communication Equipment"}	{QCOM,ERIC,CAMP,HRS,HON,WYY,UBNT,HPQ,BB,XLK}	Communication Equipment	Technology	New York Stock Exchange	\N	2020-10-11 23:12:07.255141	2020-10-11 23:38:31.02582
LFC	China Life Insurance Company Limited American Depositary Shares	China Life Insurance Co Ltd is a life insurance provider in China. The company offers life insurance, annuity, health and accident insurance products, covering survival, pension, disease, medical, physical death, disability and other protection coverage.	{"Financial Services","Insurance - Life"}	{PUK,AFL,CNO}	Insurance - Life	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.105732	2020-10-11 23:38:31.021426
LGND	Ligand Pharmaceuticals Incorporated	Ligand Pharmaceuticals Inc is a biopharmaceutical company. Its business model is based upon the concept of acquiring royalty revenue generating assets and coupling them with a lean corporate cost structure.	{Healthcare,Biotechnology}	{AMGN,SPPI,GSK,BAX,BMY,JNJ,LLY,PFE,MRK,GTXI,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:06.125696	2020-10-11 23:38:31.028166
LINDW	Lindblad Expeditions Holdings Inc. Warrant	Lindblad Expeditions Holdings Inc provides expedition cruising and adventure travel experiences. The company operates in two segments namely, Lindblad and Natural habitat.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{}	Travel & Leisure	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:06.144288	2020-10-11 23:38:31.028166
LIVE	Live Ventures Incorporated	Live Ventures Inc provides online marketing solutions to small-to-medium sized local businesses that boost customer awareness & merchant visibility. The company also acquires profitable and well managed companies.	{Industrials,Conglomerates}	{MSFT,AMZN,MCHX,VZ,T,GRPN,TZOO,GOOGL}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:06.163196	2020-10-11 23:38:31.023428
LLNW	Limelight Networks Inc.	Limelight Networks Inc provides digital content delivery network and value added services. Its services include content delivery, video content management, website and web application acceleration, website and content security, and cloud storage services.	{Technology,"Internet Content & Information","Online Media"}	{MSFT,AMZN,AKAM,RNWK,JCOM,BCOV,EQIX,INAP,XLK}	Online Media	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:06.188455	2020-10-11 23:38:31.024353
LMNR	Limoneira Co	Limoneira Co is mainly engaged in the farm production business in the United States. It cultivates and sells citrus fruits domestically and abroad. The company is also involved in rental operations and real estate development.	{"Consumer Defensive","Farm Products","Consumer Packaged Goods"}	{ALCO,FDP}	Consumer Packaged Goods	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:06.213164	2020-10-11 23:38:31.028166
LNDC	Landec Corporation	Landec Corp and its subsidiaries design, develop, manufacture and market differentiated products in food and biomaterials markets and license technology applications to partners.	{"Consumer Defensive","Farm Products","Consumer Packaged Goods"}	{FDP,APD,CPB,GIS,POL,SJM,WLK,HUN,KHC}	Consumer Packaged Goods	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:06.231336	2020-10-11 23:38:31.025871
LOCO	El Pollo Loco Holdings Inc.	El Pollo Loco Holdings Inc through its subsidiary develops, franchises, licenses, and operates quick-service restaurants under the El Pollo Loco name.	{"Consumer Cyclical",Restaurants}	{JACK,SONC,WEN,QSR}	Restaurants	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:06.25339	2020-10-11 23:38:31.028166
LOV	Spark Networks Inc. American Depositary Shares (each representing one-tenth of an)	Spark Networks SE operates a global online dating site. The company is engaged in creating communities that help individuals form life-long relationships with others that share their interests and values.	{Technology,"Internet Content & Information","Online Media"}	{XLK}	Online Media	Technology	NYSE American	\N	2020-10-11 23:12:06.273983	2020-10-11 23:38:31.02582
LPI	Laredo Petroleum Inc.	Laredo Petroleum Inc is an independent energy company. It is focused on the exploration, development and acquisition of oil and natural gas properties mainly in the Permian Basin in West Texas.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{PXD,EGN,MXC,CXO,REI,CRK}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:06.294309	2020-10-11 23:38:31.028166
LQD	iShares iBoxx $ Investment Grade Corporate Bond	The investment seeks to track the investment results of the Markit iBoxx® USD Liquid Investment Grade Index composed of U.S. dollar-denominated, investment-grade corporate bonds.\n The fund generally invests at least 90% of its assets in securities of the underlying index and at least 95% of its assets in investment-grade corporate bonds. The underlying index is designed to provide a broad representation of the U.S. dollar-denominated liquid investment-grade corporate bond market.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.315462	2020-10-11 23:38:31.023428
LSCC	Lattice Semiconductor Corporation	Lattice Semiconductor Corp is a developer of semiconductor technology. Its product lines consist of programmable logic devices, video connectivity application-specific standard products, and wave devices.	{Technology,Semiconductors}	{CY,SMTC,SNPS,XLNX,QUIK,TXN,AMD,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:06.335178	2020-10-11 23:38:31.025871
LTM	LATAM Airlines Group S.A.	LATAM Airlines Group SA is engaged in passenger and cargo transportation. It functions under two brands, which are LAN and TAM.	{Industrials,Airlines}	{CPA,VLRS,AVH,AAL,GOL}	Airlines	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:06.359812	2020-10-11 23:38:31.028166
LUB	Luby's Inc.	Luby's Inc owns and operates restaurants in the United States. Its business segments are company-owned restaurants, franchise operations and culinary contract services.	{"Consumer Cyclical",Restaurants}	{NATH,CAKE}	Restaurants	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:06.386356	2020-10-11 23:38:31.02582
LGIH	LGI Homes Inc.	LGI Homes Inc is a residential building company. It is engaged in the design, construction, marketing and sale of new homes in markets in Texas, Arizona, Florida, Georgia, New Mexico, South Carolina, North Carolina, Colorado, Washington and Tennessee.	{"Consumer Cyclical","Residential Construction","Homebuilding & Construction"}	{PHM,NVR,DHI,HOV,MDC,KBH,BZH,WLH,NWHM}	Homebuilding & Construction	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:06.121429	2020-10-11 23:38:31.028166
LIND	Lindblad Expeditions Holdings Inc.	Lindblad Expeditions Holdings Inc provides expedition cruising and adventure travel experiences. The company operates in two segments namely, Lindblad and Natural habitat.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{}	Travel & Leisure	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:06.143926	2020-10-11 23:38:31.023428
LKFN	Lakeland Financial Corporation	Lakeland Financial Corp is a bank holding company, which provides commercial, retail, wealth advisory and investment management services.	{"Financial Services","Banks - Regional - US",Banks}	{THFF,GABC,FRME,SRCE,HBNC,MFSF,ONB}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:06.166577	2020-10-11 23:38:31.023007
LLIT	Lianluo Smart Limited	Lianluo Smart Ltd design, develop and market its own branded medical products and medical components. It also distributes medical products designed and manufactured by other companies.	{Healthcare,"Medical Devices"}	{XLV}	Medical Devices	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:06.184588	2020-10-11 23:38:31.021426
LMLP	ETRACS Monthly Pay 2xLeveraged Wells Fargo MLP Ex-Energy ETN	The investment seeks a return linked to the monthly compounded 2x leveraged performance of Wells Fargo® Master Limited Partnership Ex-Energy Index, less investor fees.\n The index is intended to measure the performance of all non-energy master limited partnerships (“MLPs”) listed on the New York Stock Exchange (“NYSE”), NYSE MKT or NASDAQ that satisfy market capitalization and other eligibility requirements.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.207304	2020-10-11 23:38:31.024353
LMRKP	Landmark Infrastructure Partners LP 8.00% Series A Cumulative Redeemable Perpetual Preferred Units	Landmark Infrastructure Partners LP acquires, owns and manages a portfolio of real property interests that it leases to companies in the wireless communication, outdoor advertising and renewable power generation industries.	{"Real Estate","Real Estate Services"}	{}	Real Estate Services	Real Estate	NASDAQ Global Market	\N	2020-10-11 23:12:06.226078	2020-10-11 23:38:31.028166
LOAN	Manhattan Bridge Capital Inc	Manhattan Bridge Capital Inc is engaged in originating, servicing and managing a portfolio of first mortgage loans. It offers short-term, secured, non-banking loans to real estate investors to fund their acquisition and construction of properties.	{"Real Estate","REIT - Diversified",REITs}	{IMH,AI,EFC}	REITs	Real Estate	NASDAQ Capital Market	\N	2020-10-11 23:12:06.25296	2020-10-11 23:38:31.021426
LOPE	Grand Canyon Education Inc.	Grand Canyon Education Inc is an American for-profit educational company that operates a private university in Phoenix, Arizona, as well as online educational programs.	{"Consumer Defensive","Education & Training Services",Education}	{CECO,LINC,APEI,NAUH,BPI,ATGE}	Education	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:06.273279	2020-10-11 23:38:31.025871
LPL	LG Display Co Ltd AMERICAN DEPOSITORY SHARES	LG Display Co Ltd operates in the consumer electronics industry. Its suite of products comprise of televisions, notebook computers, desktop monitors, tablet computers and mobile devices.	{Technology,"Consumer Electronics","Computer Hardware"}	{OLED,KOPN,HON,COL,AUO,EMAN,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:06.294301	2020-10-11 23:38:31.025871
LRAD	LRAD Corporation	LRAD Corp develops and delivers directed acoustic products that beam, focus and control sound over relatively short and long distances.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{UEIC,KOSS,NSSC,MSN,UUU,VOXX,XLK}	Computer Hardware	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:06.317175	2020-10-11 23:38:31.025871
LTL	ProShares Ultra Telecommunications	The investment seeks daily investment results, before fees and expenses, that correspond to two times (2x) the daily performance of the Dow Jones U.S. Select TelecommunicationsSM Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index measures the performance of certain companies in the technology sector of the U.S. equity market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.356767	2020-10-11 23:38:31.02582
LUNA	Luna Innovations Incorporated	Luna Innovations Inc engages in providing unique capabilities in high speed optoelectronics and high performance fiber optic test products for the telecommunications industry and distributed fiber optic sensing for the aerospace and automotive industries.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{COHR,ISRG,A,DHR,HON,PHG,ADI,AXDX,VIAV,XLK}	Computer Hardware	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:06.387117	2020-10-11 23:38:31.021426
LXFT	Luxoft Holding Inc. Class A	Luxoft Holding Inc is a software development company. It develops and sells packaged software and other technology to companies in the financial services, automotive, telecommunications and energy sectors.	{Technology,"Software - Application","Application Software"}	{CTSH,ACN,EPAM,INFY,DXC,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:06.416458	2020-10-11 23:38:31.028166
LZB	La-Z-Boy Incorporated	La-Z-Boy Inc through its subsidiaries manufactures, markets, imports, distributes and retails upholstery furniture products. In addition, it imports, distributes and retails accessories and wood furniture products.	{"Consumer Cyclical","Home Furnishings & Fixtures","Manufacturing - Apparel & Furniture"}	{BSET,FLXS,HOFT,ETH,LEG,NTZ,SNBR}	Manufacturing - Apparel & Furniture	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:06.435664	2020-10-11 23:38:31.028166
LIFE	aTyr Pharma Inc.	aTyr Pharma Inc is a biotherapeutics company, engaged in the discovery and clinical development of medicines for patients suffering from severe, rare diseases using its Physiocrine biology.	{Healthcare,Biotechnology}	{IMGN,VCYT,AGTC,PFNX,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:06.130782	2020-10-11 23:38:31.031328
LL	Lumber Liquidators Holdings Inc	Lumber Liquidators Holdings Inc is a retailer of hardwood flooring in North America. Its products include foreign and domestic hardwood species, engineered hardwood, laminate, resilient vinyl, bamboo, and cork flooring.	{"Consumer Cyclical","Home Improvement Stores","Retail - Apparel & Specialty"}	{JCTCF,TSCO,BLDR,HD,LOW,BCC,BMCH,BXC}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:06.178318	2020-10-11 23:38:31.02582
LMFA	LM Funding America Inc.	LM Funding America Inc is a finance company that provides funding to nonprofit community associations located in the state of Florida and to a lesser extent in the states of Washington and Colorado.	{"Financial Services","Credit Services"}	{}	Credit Services	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:06.204375	2020-10-11 23:38:31.023007
LN	LINE Corporation American Depositary Shares (each representing one share of)	LINE Corp develops calling and messaging applications for mobile phones in Japan and internationally. Its LINE business portal segment includes communication and content service and advertising service.	{Technology,"Software - Application","Application Software"}	{OTEX,PANW,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:06.230242	2020-10-11 23:38:31.023007
LNTH	Lantheus Holdings Inc.	Lantheus Holdings Inc develops diagnostic products for the healthcare sector. Its chief product is imaging agents used by physicians in diagnostic procedures such as cardiac ultrasounds and X-ray imaging.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{NEOG,QTNT,CTSO,XLV}	Medical Diagnostics & Research	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:06.250437	2020-10-11 23:38:31.023428
LOW	Lowe's Companies Inc.	Lowe's Companies Inc is a home-improvement products dealer. The firm's stores offer products and services for home decorating, maintenance, repair, and remodeling.	{"Consumer Cyclical","Home Improvement Stores","Retail - Apparel & Specialty"}	{BLDR,HD,LL,SHOS}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:06.276893	2020-10-11 23:38:31.021426
LPTX	Leap Therapeutics Inc.	Leap Therapeutics Inc is a biopharmaceutical company . The company is engaged in acquiring and developing novel therapeutics at the leading edge of cancer biology.	{Healthcare,Biotechnology}	{PDLI,PIRS,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:06.308331	2020-10-11 23:38:31.031328
LRGE	ClearBridge Large Cap Growth ESG ETF	The investment seeks long-term capital appreciation through investing in large-capitalization companies with the potential for high future earnings growth.\n Under normal circumstances, the fund seeks to meet its investment objective by investing at least 80% of its net assets, plus borrowings for investment purposes, if any, in equity securities or other instruments with similar economic characteristics of U.S. companies with large market capitalizations that meet its financial and environmental, social and governance ("ESG") criteria.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:06.325514	2020-10-11 23:38:31.024353
LSXMB	Liberty Media Corporation Series B Liberty SiriusXM Common Stock	Liberty SiriusXM Group through its subsidiary holding is engaged in providing a subscription based satellite radio service. It transmits music, sports, entertainment, comedy, talk, news, traffic and weather channels, as well as infotainment services.	{"Consumer Cyclical","Broadcasting - Radio",Entertainment}	{}	Entertainment	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:06.349705	2020-10-11 23:38:31.031328
LTSL	Ladenburg Thalmann Financial Services Inc 6.50% Senior Notes due 2027		{}	{}			NYSE American	\N	2020-10-11 23:12:06.374227	2020-10-11 23:38:31.031328
LVHE	Legg Mason Emerging Markets Low Volatility High Dividend	The investment seeks to track the investment results of the QS Emerging Markets Low Volatility High Dividend Hedged Index.\n The fund will invest at least 80% of its net assets, plus borrowings for investment purposes, if any, in securities that compose its underlying index. The underlying index seeks to provide stable income through investments in stocks of profitable companies in emerging markets outside of the U.S. with relatively high dividend yields or anticipated dividend yields and lower price and earnings volatility, while mitigating exposure to exchange-rate fluctuations between the USD and currencies in which the component securities are denominated.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:06.393093	2020-10-11 23:38:31.028166
LYV	Live Nation Entertainment Inc.	Live Nation Entertainment Inc is a live entertainment company, and a producer of live music concerts. It is involved in live music events, venue operations, and ticketing services.	{"Consumer Cyclical","Media - Diversified",Entertainment}	{EBAY,WWE,MSGN}	Entertainment	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:06.430584	2020-10-11 23:38:31.02582
MAC	Macerich Company (The)	Macerich Co is a real estate investment trust. Its business includes ownership, development, redevelopment, management and leasing of regional and community shopping centers.	{"Real Estate","REIT - Retail",REITs}	{TCO,CBL,BXP}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:06.449172	2020-10-11 23:38:31.021426
MARK	Remark Holdings Inc.	Remark Holdings Inc is an international digital media company. The company provides digital media experiences across multiple verticals, such as travel and entertainment and young adult lifestyle.	{Technology,"Internet Content & Information","Online Media"}	{XLK}	Online Media	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:06.486162	2020-10-11 23:38:31.028166
LLY	Eli Lilly and Company	Eli Lilly and Co is a pharmaceutical company with a focus on neuroscience, endocrinology, oncology, and immunology. It discovers, develops, manufactures, and sells human pharmaceutical products and animal health products.	{Healthcare,"Drug Manufacturers - Major","Drug Manufacturers"}	{AMGN,GSK,NVS,BMY,JNJ,PFE,SNY,MRK,ABBV,XLV}	Drug Manufacturers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:06.188916	2020-10-11 23:38:31.025871
LNGR	Global X Longevity Thematic ETF	The investment seeks results that correspond generally to the price and yield performance, before fees and expenses, of the Indxx Global Longevity Thematic Index.\n The fund invests more than 80% of its total assets in the securities of the underlying index. The underlying index is designed to provide exposure to exchange-listed companies in developed markets that facilitate the demographic trend of longer average life spans and the aging of the global population, including but not limited to companies involved in biotechnology, medical devices, pharmaceuticals, senior living facilities and specialized health care services. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:06.235868	2020-10-11 23:38:31.028166
LOGM	LogMeIn Inc.	LogMeIn Inc provides cloud-based collaboration and connectivity solutions. The company offers services in three business lines that mainly provide services such as communication and collaboration, customer engagement and support, and identity and access.	{Technology,"Software - Application","Application Software"}	{CSCO,MSFT,CTXS,ADBE,SYMC,MSTR,IBM,VMW,GOOGL,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:06.260276	2020-10-11 23:38:31.031328
LPLA	LPL Financial Holdings Inc.	LPL Financial Holdings Inc operates as a broker through its subsidiaries. The company provides a platform of proprietary technology, brokerage, and investment advisor services to financial advisors and institutions.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{TROW,AMTD,ETFC,MS,RJF,SCHW,SF,LTS,NMR}	Brokers & Exchanges	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:06.297801	2020-10-11 23:38:31.02582
LQDT	Liquidity Services Inc.	Liquidity Service Inc is an online auction marketplace for surplus and salvage assets. It enables buyers and sellers to transact in an efficient, automated online auction environment offering over approximately 500 product categories.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{AMZN,CNXN,EBAY,OSTK,CLCT,STMP,SYX,BID,WMT}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:06.315566	2020-10-11 23:38:31.02582
LSST	Natixis ETF Trust Loomis Sayles Short Duration Income	The investment seeks current income consistent with preservation of capital.\n Under normal circumstances, the fund will invest at least 80% of its net assets (plus any borrowings made for investment purposes) in fixed-income securities such as bonds, notes and debentures, as well as other investments that the subadviser believes have similar economic characteristics (such as loans). It is anticipated that the fund's weighted average duration will generally be between one and three years. The fund invests primarily in investment-grade fixed-income securities.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.340949	2020-10-11 23:38:31.02423
LTRPB	Liberty TripAdvisor Holdings Inc. Series B Common Stock	Liberty TripAdvisor Holdings Inc provides online travel site services and online retail through its subsidiaries TripAdvisor and BuySeasons.	{Technology,"Internet Content & Information","Online Media"}	{PCOM,XLK}	Online Media	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:06.365584	2020-10-11 23:38:31.02423
LUV	Southwest Airlines Company	Southwest Airlines Co is one of largest domestic carrier in the United States. The company operates approximately 700 aircraft constituting an all-Boeing fleet and specializes in short-haul flights and several international locations.	{Industrials,Airlines}	{SKYW,JBLU,HA,ALGT,ALK,DAL,AAL,SAVE,UAL}	Airlines	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:06.387948	2020-10-11 23:38:31.024353
LVUS	Hartford Multifactor Low Volatility US Equity	The investment seeks to provide investment results that, before fees and expenses, correspond to the total return performance of the Hartford Multifactor Low Volatility US Equity Index.\n The fund generally invests at least 80 percent of its assets in the equity securities that are components of the index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, including money market funds, as well as in securities not included in the index. The index is designed to outperform a U.S. capitalization-weighted universe with up to one quarter less volatility over a complete market cycle.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:06.401531	2020-10-11 23:38:31.023428
LYG	Lloyds Banking Group Plc American Depositary Shares	Lloyds Banking Group PLC provides a wide range of banking and financial services. Its core business activities are retail, commercial and corporate banking, general insurance, and life, pensions and investment provision.	{"Financial Services","Banks - Regional - Europe",Banks}	{RY,BCS,SAN,CIB,RBS}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.424372	2020-10-11 23:38:31.028166
MAGS	Magal Security Systems Ltd.	Magal Security Systems Ltd develops, manufactures, markets and sells computerized security systems. Its systems are used to protect aircraft, national borders and sensitive facilities, including military bases, power plant installations, among others.	{Industrials,"Security & Protection Services","Consulting & Outsourcing"}	{ITRN,NSSC,HON,SPCB,JCI}	Consulting & Outsourcing	Industrials	NASDAQ Global Market	\N	2020-10-11 23:12:06.459048	2020-10-11 23:38:31.025871
MAS	Masco Corporation	Masco Corp provides homebuilding solutions the United States. Its array of products entails faucets, showerheads, paints and coatings.	{"Basic Materials","Building Materials"}	{AMWD,PATK,AWI,FBHS}	Building Materials	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:06.489922	2020-10-11 23:38:31.02423
LTBR	Lightbridge Corporation	Lightbridge Corp along with its subsidiaries is a nuclear fuel technology company and it participates in the nuclear power industry in the U.S. and internationally. Its operating segments include: nuclear fuel technology and nuclear consulting business.	{Industrials,"Business Services"}	{EXPO,LUNA,GPX,SO,EXC,D,MG,DUK,SPEX}	Business Services	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:06.349316	2020-10-11 23:38:31.024353
LTS	Ladenburg Thalmann Financial Services Inc	Ladenburg Thalmann Financial Services Inc is a financial company. It is engaged in independent brokerage and advisory services, investment banking, investment research, institutional sales and trading, asset management services and trust services.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{SIEB,ETFC,RJF,SF,GCAP,LPLA}	Brokers & Exchanges	Financial Services	NYSE American	\N	2020-10-11 23:12:06.369386	2020-10-11 23:38:31.028166
LWAY	Lifeway Foods Inc.	Lifeway Foods Inc manufactures probiotic, cultured, functional dairy and non-dairy health food products.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{THS,DF}	Consumer Packaged Goods	Consumer Defensive	NASDAQ Global Market	\N	2020-10-11 23:12:06.406662	2020-10-11 23:38:31.024353
LYL	Dragon Victory International Limited	Dragon Victory International Ltd offers crowdfunding opportunities to entrepreneurs and funding sources. It provides its services through its online platform.	{"Financial Services","Credit Services"}	{MKTX,MS,OPY}	Credit Services	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:06.425286	2020-10-11 23:38:31.031328
MACK	Merrimack Pharmaceuticals Inc.	Merrimack Pharmaceuticals Inc is a biotechnology company. It discovers, develops, and commercializes innovative medicines consisting of novel therapeutics paired with diagnostics to treat various types of cancer.	{Healthcare,Biotechnology}	{GERN,CELG,IMMU,AVEO,BTX,SNY,CLVS,MDGL,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:06.448307	2020-10-11 23:38:31.023007
MANU	Manchester United Ltd. Class A	Manchester United PLC and its subsidiaries operate a professional football club together with related and ancillary activities. It also operates Manchester United branded soccer schools, physical retail channels and e-commerce retail channels.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{TRK,WWE,LYV}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:06.479957	2020-10-11 23:38:31.02582
MAXR	Maxar Technologies Ltd.	Maxar Technologies Ltd is an integrated space and geospatial intelligence company with a full range of space technology solutions for commercial and government customers including satellites, Earth imagery, geospatial data and analytics.	{Technology,"Communication Equipment"}	{PFPT,PFM,PCYG,XLK}	Communication Equipment	Technology	New York Stock Exchange	\N	2020-10-11 23:12:06.501753	2020-10-11 23:38:31.02582
MBIN	Merchants Bancorp	Merchants Bancorp is a bank holding company. It operates multi-family housing and healthcare facility financing and servicing, mortgage warehouse financing, retail and correspondent residential mortgage banking, agricultural lending and community banking.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:06.529649	2020-10-11 23:38:31.028166
MCC	Medley Capital Corporation	Medley Capital Corp is a non-diversified closed end management investment company in US. Its objective is to generate current income and capital appreciation by lending directly to privately held middle market companies.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.564693	2020-10-11 23:38:31.025871
MCK	McKesson Corporation	McKesson Corp provides medicines, pharmaceutical supplies, information and care management products and services across the healthcare industry. It operates with two segments namely McKesson Distribution Solutions and McKesson Technology Solutions.	{Healthcare,"Medical Distribution"}	{HSIC,CERN,MDRX,CPSI,PDCO,CAH,OMI,ABC,PINC,XLV}	Medical Distribution	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:06.5933	2020-10-11 23:38:31.025871
MCS	Marcus Corporation (The)	Marcus Corp is engaged in two business segments, which are movie theatres and hotels and resorts.	{"Consumer Cyclical","Media - Diversified",Entertainment}	{RDI,CNK,AMC}	Entertainment	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:06.61324	2020-10-11 23:38:31.021426
MDGL	Madrigal Pharmaceuticals Inc.	Madrigal Pharmaceuticals Inc is a company focused on the development of novel compounds for the treatment of cardiovascular-metabolic diseases and nonalcoholic steatohepatitis (NASH).	{Healthcare,Biotechnology}	{AMGN,CELG,SGEN,INFI,ECYT,JNJ,PFE,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:06.633639	2020-10-11 23:38:31.023007
MDSO	Medidata Solutions Inc.	Medidata Solutions Inc provides cloud-based solutions for life sciences. Its customers are pharmaceutical, biotechnology, and medical device companies, academic institutions, and contract research organizations.	{Technology,"Health Information Services","Application Software"}	{SLP,ABT,ORCL,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:06.662434	2020-10-11 23:38:31.024353
MELI	MercadoLibre Inc.	MercadoLibre Inc is an e-commerce company. It hosts the online commerce platform in Latin America, which is focused on enabling e-commerce and its related services. The company generates most of its revenue from Brazil, Argentina, Venezuela, and Mexico.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{BMO,AMZN,CSGP,EBAY,LQDT,CBD,BID,CRM,FB,GOOGL}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:06.696793	2020-10-11 23:38:31.024353
LVHD	Legg Mason Low Volatility High Dividend ETF	The investment seeks to track the investment results of the QS Low Volatility High Dividend Index (the "underlying index").\n The fund will invest at least 80% of its net assets, plus borrowings for investment purposes, if any, in securities that compose the underlying index. The underlying index is composed of equity securities of U.S. companies with relatively high yield and low price and earnings volatility.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:06.392919	2020-10-11 23:38:31.02423
LXU	LSB Industries Inc.	LSB Industries Inc is a chemical manufacturing, marketing, and engineering company. Principally, it is engaged in production and marketing of chemical products.	{"Basic Materials",Chemicals}	{AAON,LII,APD,AOS,UTX,CF,UAN}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:06.423801	2020-10-11 23:38:31.023428
MARPS	Marine Petroleum Trust Units of Beneficial Interest	Marine Petroleum Trust is a U.S based royalty trust. It provides administration and liquidation of rights to payments from oil and natural gas leases in the Gulf of Mexico.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{PBT,CRT,MTR,SBR,SJT}	Oil & Gas - Services	Energy	NASDAQ Capital Market	\N	2020-10-11 23:12:06.488539	2020-10-11 23:38:31.024353
MB	MINDBODY Inc.	MINDBODY Inc is a provider of cloud-based business management software for the wellness services industry. The company offers its software platform to its subscribers as a subscription-based service.	{Technology,"Software - Application","Application Software"}	{RNWK,ZNGA,SHSP,XLK}	Application Software	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:06.510525	2020-10-11 23:38:31.02423
MBSD	FlexShares Disciplined Duration MBS Index Fund	The investment seeks investment results that correspond generally to the price and yield performance, before fees and expenses, of the BofA Merrill Lynch Constrained Duration US Mortgage Backed Securities IndexSM.\n The fund generally will invest under normal circumstances at least 80% of its total assets (exclusive of collateral held from securities lending) in the securities of the underlying index and in "TBA Transactions" that represent securities in the underlying index. The underlying index reflects the performance of a selection of investment-grade U.S. agency residential mortgage-backed pass-through securities.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:06.534506	2020-10-11 23:38:31.021426
MCB	Metropolitan Bank Holding Corp.	Metropolitan Bank Holding Corp provide a broad range of business, commercial and retail banking products and services to small businesses, middle-market enterprises, public entities and affluent individuals in the New York metropolitan area.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.558203	2020-10-11 23:38:31.023428
MCHP	Microchip Technology Incorporated	Microchip Technology Inc develops, manufactures and sells semiconductor products used for a range of embedded control applications. It also provides license Flash-IP solutions.	{Technology,Semiconductors}	{CY,MXIM,SLAB,ON,STM,TXN,ADI,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:06.580889	2020-10-11 23:38:31.023007
MCRO	IQ Hedge Macro Tracker	The investment seeks investment results that correspond (before fees and expenses) generally to the price and yield performance of its underlying index, the IQ Hedge Macro Index.\n The fund is a "fund of funds" which means it invests, under normal circumstances, at least 80% of its net assets, plus the amount of any borrowings for investment purposes, in the investments included in its underlying index, which includes underlying funds. The underlying index consists of a number of components ("underlying index Components") selected in accordance with IndexIQ's rules-based methodology of such underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.60789	2020-10-11 23:38:31.023007
MDGS	Medigus Ltd.	Medigus Ltd is engaged in the development, manufacturing, and marketing of surgical endostaplers and direct vision systems for minimally invasive medical procedures.	{Healthcare,"Medical Devices"}	{XLV}	Medical Devices	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:06.634585	2020-10-11 23:38:31.031328
MDLZ	Mondelez International Inc.	Mondelez International Inc is associated with the confectionery industry. It manufactures and markets snack food and beverage products like biscuits, chocolate, gum and candy, and beverages.The sale of biscuits generate maximum revenue for the company.	{"Consumer Defensive",Confectioners,"Consumer Packaged Goods"}	{JJSF,HAIN,CPB,CAG,GIS,HSY,K,PEP}	Consumer Packaged Goods	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:06.651798	2020-10-11 23:38:31.021426
MDYV	SPDR S&P 400 Mid Cap Value ETF (based on S&P MidCap 400 Value Index--symbol: MGD	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the S&P MidCap 400 Value Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index measures the performance of the mid-capitalization value segment of the U.S. equity market. It may purchase a subset of the securities in the index in an effort to hold a portfolio of securities with generally the same risk and return characteristics of the index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.680331	2020-10-11 23:38:31.024353
MEN	Blackrock MuniEnhanced Fund Inc	Blackrock Muni Enhanced Fund Inc is a diversified investment fund with an investment objective to provide shareholders with as high a level of current income exempt from U.S. federal income taxes as is consistent with its investment policies.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.702529	2020-10-11 23:38:31.025871
LXFR	Luxfer Holdings PLC	Luxfer Holdings PLC is a materials technology company. It manufactures and supplies high-performance materials, components and gas cylinders to customers in environmental technologies, healthcare technologies, protection and specialty technologies.	{Industrials,"Diversified Industrials","Industrial Products"}	{LECO,TRS,WOR}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:06.41676	2020-10-11 23:38:31.02582
M	Macy's Inc	Macy's Inc is an omnichannel retailer. It sells merchandise, including men's, women's and children's apparel and accessories, cosmetics, home furnishings and other consumer goods.	{"Consumer Cyclical","Department Stores","Retail - Apparel & Specialty"}	{DDS,KSS,JWN,JCP,TJX,TGT,SHOS}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:06.442178	2020-10-11 23:38:31.025871
MAN	ManpowerGroup	ManpowerGroup Inc is engaged in the employment services industry. It offers permanent, temporary and contract recruitment services; assessment and selection services; training and development services; outsourcing services, and among others.	{Industrials,"Staffing & Outsourcing Services","Employment Services"}	{KELYA,KFRC,KFY,RHI}	Employment Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:06.478335	2020-10-11 23:38:31.023007
MATW	Matthews International Corporation	Matthews International Corp is a designer, manufacturer and marketer of memorialization products and brand solutions. Its products and services include cast bronze memorials, caskets, cast and etched architectural products, cremation equipment and others.	{"Consumer Cyclical","Personal Services"}	{SCI,HI,CSV,STON}	Personal Services	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:06.501254	2020-10-11 23:38:31.028166
MBTF	M B T Financial Corp	MBT Financial Corp is a bank holding company. The firm holds independently owned community bank in Southeast Michigan.	{"Financial Services","Banks - Regional - US",Banks}	{MBWM,MCBC,CHFC,HBAN,MFNC,IBCP}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:06.540825	2020-10-11 23:38:31.023428
MCHI	iShares MSCI China ETF	The investment seeks to track the investment results of the MSCI China Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is a free float-adjusted market capitalization-weighted index designed to measure the performance of equity securities in the top 85% in market capitalization of the Chinese equity securities markets, as represented by the H-Shares and B-Shares markets. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:06.580537	2020-10-11 23:38:31.031328
MCV	Medley Capital Corporation 6.125% Senior Notes due 2023	Medley Capital Corp is a non-diversified closed end management investment company in US. Its objective is to generate current income and capital appreciation by lending directly to privately held middle market companies.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.617645	2020-10-11 23:38:31.02423
MDYG	SPDR S&P 400 Mid Cap Growth ETF (based on S&P MidCap 400 Growth Index--symbol: MUV)	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the S&P MidCap 400 Growth Index that tracks the performance of medium capitalization exchange traded U.S. equity securities exhibiting "growth" characteristics.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index measures the performance of the mid-capitalization growth segment of the U.S. equity market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.680057	2020-10-11 23:38:31.023007
MEOH	Methanex Corporation	Methanex Corp is engaged in the production of chemicals. Its primary product is methanol, a commodity chemical that is used to make a range of industrial, consumer and energy products.	{"Basic Materials",Chemicals}	{TOT,BP}	Chemicals	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:12:06.703286	2020-10-11 23:38:31.023428
MFD	Macquarie First Trust Global	Macquarie/First Trust Global Infrastructure/Utilities Dividend&Income Fund is a United States based non-diversified closed-end management company. Its objective is to seek high current return.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.727923	2020-10-11 23:38:31.023428
MFO	MFA Financial Inc. 8.00% Senior Notes due 2042	MFA Financial Inc through its subsidiaries is engaged in real estate finance business. The company invests in residential mortgage assets, including Agency MBS, Non-Agency MBS, residential whole loans and CRT securities.	{"Real Estate","REIT - Diversified",REITs}	{}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:06.749274	2020-10-11 23:38:31.02582
MSG	The Madison Square Garden Company Class A (New)	The Madison Square Garden Co is a live sports and entertainment company in USA. Its segments include MSG Sports and MSG Entertainment. Revenues are generated from sale of tickets for events and distributions of league-wide television contracts.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{MSG}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:07.254905	2020-10-11 23:38:31.024353
LVS	Las Vegas Sands Corp.	Las Vegas Sands Corp operates fully integrated resorts with casino, hotel, entertainment. The company owns the Venetian Macao, Sands Macao, in Macau, the Marina Bay Sands resort in Singapore, the Venetian and Sands Bethlehem casinos in the United States.	{"Consumer Cyclical","Resorts & Casinos","Travel & Leisure"}	{MCRI,PENN,CNTY,WYNN,MLCO,BYD,MGM}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:06.398769	2020-10-11 23:38:31.021426
LYB	LyondellBasell Industries NV Class A (Netherlands)	LyondellBasell Industries NV is a petrochemical producer operating in US and Europe. It produces polyethylene and propylene oxide used in various consumer and industrial end products.	{"Basic Materials","Specialty Chemicals",Chemicals}	{SSL,BAK,WLK,CE,RDS.A,DWDP}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:06.423919	2020-10-11 23:38:31.021426
MAA	Mid-America Apartment Communities Inc.	Mid-America Apartment Communities Inc is a multifamily focused, self-administered and self-managed REIT. The Company owns, operates, acquires and develops apartment communities in the Southeast and Southwest region of the United States.	{"Real Estate","REIT - Residential",REITs}	{EQR,CPT,ESS,AVB,UDR,IRT}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:06.448062	2020-10-11 23:38:31.031328
MARA	Marathon Patent Group Inc.	Marathon Patent Group Inc is a patent and patent rights acquisition and licensing company. The company acquires patents and patent rights from their owners including individual inventors to Fortune 500 companies.	{Industrials,"Business Services"}	{ACTG}	Business Services	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:06.480102	2020-10-11 23:38:31.023428
MAT	Mattel Inc.	Mattel Inc designs, manufactures, and markets a variety of toy products through sales to its customers and directly to consumers. The products include fashion dolls, infant and preschool products, toy cars, and electrical vehicles, among others.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{JAKK,EA,DIS,BBW,HAS}	Travel & Leisure	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:06.497672	2020-10-11 23:38:31.021426
MBFIO	MB Financial Inc. Depositary Shares Each Representing a 1/40th Interest in a Share of 6.000% Noncumulative Perpetual Preferred Stock Series C	MB Financial Inc is a financial holding company. It offers an array of financial services primarily to small and middle market businesses and individuals in the markets it serves through MB Financial Bank.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:06.519237	2020-10-11 23:38:31.021426
MBUU	Malibu Boats Inc.	Malibu Boats Inc is a designer, manufacturer and marketer of performance sport boats. Its boats are used for water sports, including water skiing, wakeboarding and wake surfing, as well as general recreational boating.	{"Consumer Cyclical","Recreational Vehicles",Autos}	{TWIN,BC,GM,MPX}	Autos	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:06.54132	2020-10-11 23:38:31.02423
MCD	McDonald's Corporation	McDonald's Corp operates a chain of restaurants with approximately 37,200 restaurants in 120 countries. It also receives franchise royalties, and licensing pacts. The company offers a uniform value-priced menu with special additions based on regions.	{"Consumer Cyclical",Restaurants}	{SBUX,JACK,SONC,TAST,YUM,CMG,GTIM,ARCO,WEN}	Restaurants	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:06.569331	2020-10-11 23:38:31.024353
MCN	Madison Covered Call & Equity Strategy Fund	Madison/claymore Covr Call & EqtStrtgFd is a diversified, closed-end management investment company. Its primary objective is to provide a high level of current income and current gains. Its secondary objective is to achieve long-term capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.608382	2020-10-11 23:38:31.024353
MDXG	MiMedx Group Inc	MiMedx Group Inc is a developer, manufacturer and marketer of patent-protected regenerative biomaterial products and bioimplants processed from human amniotic membrane.	{Healthcare,"Medical Devices"}	{NUVA,CRY,TFX,HRC,MDT,STE,ATEC,SNOA,XLV}	Medical Devices	Healthcare	OTC Pink	\N	2020-10-11 23:12:06.675252	2020-10-11 23:38:31.031328
MET	MetLife Inc.	MetLife Inc provides life insurance, annuities, employee benefits and asset management in the United States, Japan, Latin America, Asia, Europe and the Middle East.	{"Financial Services","Insurance - Life"}	{PUK,AEG,AFG,HMN,AIZ,CNO,VOYA}	Insurance - Life	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.714182	2020-10-11 23:38:31.02423
MFEM	PIMCO Equitiy Series RAFI Dynamic Multi-Factor Emerging Markets Equity	The investment seeks to track the investment results of the RAFI Dynamic Multi-Factor Developed Ex-U.S. Index.\n The fund seeks to achieve its investment objective by investing at least 80% of its total assets (exclusive of collateral held from securities lending) in the component securities of the RAFI Dynamic Multi-Factor Developed Ex-U.S. Index. The underlying index is constructed by the index provider using a rules-based approach to construct factor portfolios within the underlying index. The underlying index consists of "factor portfolios," each of which emphasizes one of the following factors: value, low volatility, quality, momentum and size.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.731119	2020-10-11 23:38:31.031328
LX	LexinFintech Holdings Ltd.	LexinFintech Holdings Ltd is an online consumer finance which focuses on serving the credit needs of educated young adults in China.	{"Financial Services","Credit Services"}	{PDFS,OMCL}	Credit Services	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:06.411801	2020-10-11 23:38:31.028166
LYTS	LSI Industries Inc.	LSI Industries Inc is engaged in providing corporate visual image solutions through the combination of digital and screen graphics capabilities and indoor and outdoor lighting products, lighting control systems and related professional services.	{Technology,"Electronic Components","Computer Hardware"}	{AYI,OESX,RVLT,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:06.429048	2020-10-11 23:38:31.023007
MAB	Eaton Vance Massachusetts Municipal Bond Fund of Beneficial Interest $.01 par value	Eaton Vance Massachusetts Municipal Bond Fund is a closed-end investment company. Its objective is to provide current income exempt from regular federal income tax, including alternative minimum tax, &, in state specific funds, tax in its specified state.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:06.448307	2020-10-11 23:38:31.024353
MAMS	MAM Software Group Inc.	MAM Software Group Inc through its subsidiaries is engaged in providing integrated information management solutions and services, and also provides cloud-based software solutions for the automotive aftermarket sector.	{Technology,"Software - Infrastructure","Application Software"}	{MSFT,AZO,FIVN,XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:06.471687	2020-10-11 23:38:31.028166
MASI	Masimo Corporation	Masimo Corp is a medical technology company. The company develops, manufactures, and markets noninvasive monitoring technologies. Its product includes noninvasive blood constituent and brain and breath monitoring.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{UFPI,CASM,CNMD,BABY,CAH,OMI,RMD,GE,TMO,PHG,XLV}	Medical Instruments & Equipment	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:06.490619	2020-10-11 23:38:31.023007
MBFI	MB Financial Inc.	MB Financial Inc is a financial holding company. It offers an array of financial services primarily to small and middle market businesses and individuals in the markets it serves through MB Financial Bank.	{"Financial Services","Banks - Regional - US",Banks}	{OSBC,WTFC,FMBI,BUSE}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:06.511851	2020-10-11 23:38:31.028166
MBOT	Microbot Medical Inc.	Microbot Medical Inc is a pre-clinical medical device company. It specializes in research, design and development of next generation micro-robotics assisted medical technologies.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{XLV}	Medical Instruments & Equipment	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:06.53411	2020-10-11 23:38:31.024353
MCI	Barings Corporate Investors	Barings Corporate Investors is a United States based closed-end management investment company. Its investment objective is to maintain a portfolio of securities providing a fixed yield and at the same time offering an opportunity for capital gains.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.586262	2020-10-11 23:38:31.023428
MDC	M.D.C. Holdings Inc.	M.D.C. Holdings Inc is engaged in the residential construction industry. Its home building operations consists of construction and sale of single-family detached homes and financial services including mortgage loans and title agency services.	{"Consumer Cyclical","Residential Construction","Homebuilding & Construction"}	{PHM,MHO,NVR,DHI,HOV,LEN,KBH,BZH,TMHC}	Homebuilding & Construction	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:06.631257	2020-10-11 23:38:31.023428
MDLY	Medley Management Inc. Class A	Medley Management Inc is an asset management company providing its services mainly to retail and institutional investors. It provides credit-focused investment strategies through various funds and products.	{"Financial Services","Asset Management"}	{TROW,WDR,PZN}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.646134	2020-10-11 23:38:31.025871
MDRX	Allscripts Healthcare Solutions Inc.	Allscripts Healthcare Solutions Inc is a provider of clinical, financial, connectivity and information solutions and related professional services that empower hospitals, physicians and post-acute organizations to deliver world-class outcomes.	{Technology,"Software - Application","Application Software"}	{CERN,CPSI,HMSY,NUAN,STRM,ATHN,GE,MCK,XRX,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:06.658789	2020-10-11 23:38:31.028166
MEDP	Medpace Holdings Inc.	Medpace Holdings Inc is a clinical contract research organisation. It is engaged in providing scientifically-driven clinical research-based drug and medical device development services to the biotechnology, pharmaceutical and medical device industries.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{XLV}	Medical Diagnostics & Research	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:06.684874	2020-10-11 23:38:31.021426
MERC	Mercer International Inc.	Mercer International Inc operates in the pulp business and its operations are located in Germany and Western Canada. The Company is a producer of market northern bleached softwood kraft, or NBSK, pulp in the world.	{"Basic Materials","Paper & Paper Products","Forest Products"}	{UFS,FBR,CFX,RFP}	Forest Products	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:12:06.70943	2020-10-11 23:38:31.031328
MFCB	MFC Bancorp Ltd.	MFC Bancorp Ltd is a trade finance and supply chain company. The Company provides supply chain services, logistics and other trade and structured finance services to producers and consumers.	{"Financial Services","Asset Management"}	{BAM,DXPE,INTL}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.728848	2020-10-11 23:38:31.028166
LXRX	Lexicon Pharmaceuticals Inc.	Lexicon Pharmaceuticals Inc operates as a biopharmaceutical company engaged in discovery and development of breakthrough treatments for human disease. The company mainly designs and develops drugs for therapeutic areas.	{Healthcare,Biotechnology}	{BMRN,REGN,ILMN,BTX,TMO,LGND,ESPR,AERI,RARE,ALBO,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:06.421146	2020-10-11 23:38:31.02423
MA	Mastercard Incorporated	Mastercard Inc is a technology company that generates revenue by charging fees to its customers based on both the dollar volume of card activity and the number of transactions processed through the network.	{"Financial Services","Credit Services"}	{AMZN,TSS,VZ,FIS,T,WU,V,WEX,TMUS}	Credit Services	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.444437	2020-10-11 23:38:31.02423
MANT	ManTech International Corporation	Mantech International Corp provides technologies and solutions for mission-critical national security programs for the Intelligence Community, the departments of Defence, State, Homeland Security, Health and Human Services, Veterans Affairs and Justice.	{Technology,"Software - Application","Application Software"}	{MRCY,ICFI,CACI,CUB,HRS,BAH,LDOS,SAIC,EGL,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:06.477304	2020-10-11 23:38:31.021426
MATX	Matson Inc.	Matson Inc is a holding company, consisting of two segments, ocean transportation and logistics. The company also provides container stevedoring, container equipment maintenance and other terminal services for MatNav and other ocean carriers.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{GLNG,GNK}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:06.500672	2020-10-11 23:38:31.025871
MBII	Marrone Bio Innovations Inc.	Marrone Bio Innovations Inc is a producer of bio-based pest management and plant health products. The company offers its products to the markets that use conventional chemical pesticides, including certain agricultural and water markets.	{"Basic Materials","Agricultural Inputs",Agriculture}	{HWKN,AVD,FMC,SMG,DWDP}	Agriculture	Basic Materials	NASDAQ Capital Market	\N	2020-10-11 23:12:06.525456	2020-10-11 23:38:31.023007
MBVX	MabVax Therapeutics Holdings Inc.	MabVax Therapeutics Holdings Inc is a development-stage biopharmaceutical company which discovers, develops, and commercializes small molecule drugs to treat cancer-related illnesses.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	OTC Pink	\N	2020-10-11 23:12:06.543306	2020-10-11 23:38:31.028166
MCEP	Mid-Con Energy Partners LP	Mid-Con Energy Partners LP acquires, explores, develops, and produces oil and natural gas properties in North America.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{TGC}	Oil & Gas - E&P	Energy	Nasdaq Global Select	\N	2020-10-11 23:12:06.570403	2020-10-11 23:38:31.028166
MCO	Moody's Corporation	Moody's Corporation publishes credit ratings, and research reports on fixed-income securities, issuers of securities, and other credit obligations. The company primarily provides opinions and reports to investors and institutions.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{MORN,INTX,EFX,DNB,MSCI,CLGX,SPGI}	Brokers & Exchanges	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.601745	2020-10-11 23:38:31.031328
MCX	Medley Capital Corporation 6.50% Notes due 2021		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:06.617171	2020-10-11 23:38:31.031328
MDLX	Medley LLC 6.875% Senior Notes due 2026		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:06.647354	2020-10-11 23:38:31.023428
MDWD	MediWound Ltd.	MediWound Ltd is a biopharmaceutical company engaged in developing, manufacturing and commercializing novel products to address unmet needs in the fields of severe burns, chronic and other hard-to-heal wounds, connective tissue disorders and others.	{Healthcare,Biotechnology}	{BSTC,JNJ,SNN,TARO,TEVA,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:06.667217	2020-10-11 23:38:31.021426
MEI	Methode Electronics Inc.	Methode Electronics Inc is a part of the electronic components industry. Its main products include components and subsystem devices.	{Technology,"Electronic Components","Computer Hardware"}	{RFIL,OCC,AVX,APH,MMM,TEL,SRI,AAOI,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:06.694415	2020-10-11 23:38:31.023007
MFA	MFA Financial Inc.	MFA Financial Inc through its subsidiaries is engaged in real estate finance business. The company invests in residential mortgage assets, including Agency MBS, Non-Agency MBS, residential whole loans and CRT securities.	{"Real Estate","REIT - Diversified",REITs}	{NYMT,DX,ANH,CMO,IVR,PMT}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:06.717814	2020-10-11 23:38:31.028166
MFIN	Medallion Financial Corp.	Medallion Financial Corp is a specialty finance company that originates, acquires, and services loans that finance taxicab medallions and various types of commercial businesses. The company operates through Lending and Investing operation segments.	{"Financial Services","Credit Services"}	{}	Credit Services	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:06.736617	2020-10-11 23:38:31.024353
MGA	Magna International Inc.	Magna International Inc is an auto parts supplier. Its products include chassis, interior, exterior, seating and powertrain.	{"Consumer Cyclical","Auto Parts",Autos}	{STRT,BWA,ALV,AXL,F,DAN,LEA,TOWR,VC}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:06.77795	2020-10-11 23:38:31.023007
MGNX	MacroGenics Inc.	Macrogenics Inc is a clinical-stage biopharmaceutical company. It focusses on discovering and developing monoclonal antibody-based therapeutics for the treatment of cancer, as well as various autoimmune disorders and infectious diseases.	{Healthcare,Biotechnology}	{AMGN,GILD,BMY,LLY,PFE,MRK,STML,XNCR,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:06.803237	2020-10-11 23:38:31.031328
LXP	Lexington Realty Trust	Lexington Realty Trust is a REIT that owns a diversified portfolio of equity and debt investments in single-tenant commercial properties. It has equity ownership interests in over 195 consolidated real estate properties, located in around 40 states.	{"Real Estate","REIT - Diversified",REITs}	{AIV,AKR,UHT,NNN}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:06.41883	2020-10-11 23:38:31.025871
MANH	Manhattan Associates Inc.	Manhattan Associates Inc develops, sells, deploys, services and maintains software solutions designed to manage supply chains, inventory and omni-channel operations for retailers, wholesalers, manufacturers, logistics providers and other organizations.	{Technology,"Software - Application","Application Software"}	{DSGX,MSFT,PEGA,AMSWA,IBM,DATA,AMBR,PAYC,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:06.482587	2020-10-11 23:38:31.02423
MBB	iShares MBS ETF	The investment seeks to track the investment results of the Bloomberg Barclays U.S. MBS Index.\n The fund seeks to track the performance of the underlying index by investing at least 90% of its assets in the securities of the underlying index and in investments that provide substantially similar exposure to securities in the underlying index. The index measures the performance of investment-grade mortgage-backed pass-through securities ("MBS") issued or guaranteed by U.S. government agencies.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:06.509745	2020-10-11 23:38:31.023428
MBIO	Mustang Bio Inc.	Mustang Bio Inc is a clinical-stage biopharmaceutical company. It is engaged in the development and commercialization of novel cancer immunotherapy products designed to utilize the power of the patient's own immune system to eliminate cancer cells.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:06.533647	2020-10-11 23:38:31.031328
MCEF	First Trust Municipal CEF Income Opportunity ETF	The investment seeks to provide current income.\n The fund will seek to achieve its investment objectives by investing at least 80% of its net assets (including investment borrowings) in a portfolio of closed-end investment companies that are listed and traded in the United States on registered exchanges ("closed-end funds") which invest primarily in municipal debt securities some or all of which pay interest that is exempt from regular federal income taxes (collectively, "Municipal Securities"). It may also invest in exchange-traded funds ("ETFs"). The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:06.569901	2020-10-11 23:38:31.02423
MCR	MFS Charter Income Trust	MFS Charter Income Trust is a United States based closed-end fund. Its investment objective is to seek high current income. The fund invests the assets primarily in debt instruments.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.605574	2020-10-11 23:38:31.02423
MCY	Mercury General Corporation	Mercury General Corp is an insurance holding company. It is engaged in writing personal automobile insurance as well as homeowners, commercial automobile, commercial property, mechanical breakdown, and umbrella insurance.	{"Financial Services","Insurance - Property & Casualty"}	{ERIE,SAFT,ALL,PGR}	Insurance - Property & Casualty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.622693	2020-10-11 23:38:31.024353
MDR	McDermott International Inc.	McDermott International Inc is an engineering, procurement, construction and installation company engaged in designing and executing complex offshore oil and gas projects.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:06.656386	2020-10-11 23:38:31.031328
MED	MEDIFAST INC	Medifast Inc is a health products manufacturer in the United States. It sells products such as meal replacements, snacks, hydration products and vitamins mainly through an online platform.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{NTRI,WTW}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:06.686702	2020-10-11 23:38:31.023428
MEXX	Direxion Daily MSCI Mexico Bull 3X Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the daily performance of the MSCI Mexico IMI 25/50 Index.\n The fund invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, ETFs that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index is designed to measure the performance of the large-, mid- and small-capitalization segments of the Mexican equity market, covering approximately 99% of the free float-adjusted market capitalization in Mexico. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.71746	2020-10-11 23:38:31.024353
MFINL	Medallion Financial Corp. 9.000% Notes due 2021		{}	{}			Nasdaq Global Select	\N	2020-10-11 23:12:06.74279	2020-10-11 23:38:31.028166
MGEE	MGE Energy Inc.	MGE Energy Inc is a public utility holding company. The company, through its subsidiaries, generates and distributes electricity and purchases and distributes natural gas.	{Utilities,"Utilities - Diversified","Utilities - Regulated"}	{OTTR,DTE,LNT,AEE,AEP,CMS,ALE,WEC,XEL,XLU}	Utilities - Regulated	Utilities	Nasdaq Global Select	\N	2020-10-11 23:12:06.775852	2020-10-11 23:38:31.025871
MGPI	MGP Ingredients Inc.	MGP Ingredients Inc through its subsidiaries is engaged in producing distillery products derived from corn & other feedstock including rye, barley, barley malt and milo; and ingredient products derived from wheat flour to serve packaged goods industry.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{VLO,ADM,BG,INGR}	Consumer Packaged Goods	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:06.805414	2020-10-11 23:38:31.023007
QRHC	Quest Resource Holding Corporation.	Quest Resource Holding Corporation provides one-stop management programs to reuse, recycle, and dispose of a wide variety of waste streams and recyclables generated by the client's business.	{Industrials,"Waste Management"}	{SCHN,ARCI,EEI,CVA}	Waste Management	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:09.945502	2020-10-11 23:38:31.023007
MAYS	J. W. Mays Inc.	JW Mays Inc is a US based real estate company. It is engaged in leasing of real estate and operates a number of commercial real estate properties in New York.	{"Real Estate","Real Estate - General","Real Estate Services"}	{INTG,TCI,NEN,CTO,IHT,ARL,AHH,FPI,FCE.A}	Real Estate Services	Real Estate	NASDAQ Capital Market	\N	2020-10-11 23:12:06.502656	2020-10-11 23:38:31.031328
MBI	MBIA Inc.	MBIA Inc is engaged in providing financial guarantee insurance to the United States public finance markets. Its segments are U.S. public finance insurance; corporate; international and structured finance insurance.	{"Financial Services","Insurance - Property & Casualty"}	{RDN,MTG,ORI,AGO}	Insurance - Property & Casualty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.521615	2020-10-11 23:38:31.02423
MBWM	Mercantile Bank Corporation	Mercantile Bank Corp is a bank holding company. It provides banking services offering deposit products including checking, savings, and term certificate accounts, and lending products including commercial, residential mortgage and installment loans.	{"Financial Services","Banks - Regional - US",Banks}	{MCBC,CHFC,FITB,SRCE,MBTF,MFNC,IBCP,GNK,LTBR}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:06.547824	2020-10-11 23:38:31.028166
MCF	Contango Oil & Gas Company	Contango Oil & Gas Company is an independent energy company. It is engaged in the exploration, development, production and acquisition of crude oil, natural gas and natural gas liquids reserves in the Gulf of Mexico.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{CPE,PXD,DVN,EOG,WTI}	Oil & Gas - E&P	Energy	NYSE American	\N	2020-10-11 23:12:06.57475	2020-10-11 23:38:31.02582
MCRB	Seres Therapeutics Inc.	Seres Therapeutics Inc is a microbiome therapeutics platform company which is engaged in developing biological drugs which are referred as ecobiotic microbiome therapeutics.	{Healthcare,Biotechnology}	{XLRN,XNCR,BLCM,XON,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:06.606734	2020-10-11 23:38:31.028166
MD	Mednax Inc.	Mednax Inc is a provider of physician services that are newborn, anesthesia, maternal-fetal, teleradiology, pediatric cardiology and other pediatric subspecialty care. Anesthesiology and Neonatology and other pediatric subspecialties generate more income.	{Healthcare,"Medical Care","Health Care Providers"}	{LPNT,ATHN,UHS,CYH,THC,XLV}	Health Care Providers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:06.624871	2020-10-11 23:38:31.028166
MDU	MDU Resources Group Inc.	MDU Resources Group Inc operates as a diversified natural resource company. The company mines aggregates and markets crushed stone, sand, gravel and related construction materials, including ready-mixed concrete, cement, asphalt, and liquid asphalt.	{"Basic Materials","Building Materials"}	{SRE,BKH,VMC,AVA,VVC,CNP,NWE,XEL}	Building Materials	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:06.663684	2020-10-11 23:38:31.023428
MEET	The Meet Group Inc.	MeetMe Inc is a social network where new people meet on mobile platforms to connect and chat with each other. The company offers online marketing where marketers display their advertisements in different formats and in different locations.	{Technology,"Internet Content & Information","Online Media"}	{TST,IAC,CBS,XOXO,FB,BCOR,TWTR,WB,GOOGL,RENN,XLK}	Online Media	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:06.691738	2020-10-11 23:38:31.031328
METC	Ramaco Resources Inc.	Ramaco Resources Inc is a operator and developer of high-quality, low-cost metallurgical coal in central and southern West Virginia and southwestern Pennsylvania.	{"Basic Materials",Coal}	{METC,CCR,SES}	Coal	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:12:06.714193	2020-10-11 23:38:31.023007
MFM	MFS Municipal Income Trust	MFS Municipal Income Trust is a United States based diversified closed-end management investment company. Its investment objective is to seek high current income exempt from federal income tax, but may also consider capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.747256	2020-10-11 23:38:31.031328
MGIC	Magic Software Enterprises Ltd.	Magic Software Enterprises Ltd is a software development company. It provides application development, business process integration platforms, vertical software solutions and professional services to develop mobile and cloud based business applications.	{Technology,"Software - Application","Application Software"}	{MSFT,FORTY,SAP,IBM,ORCL,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:06.790303	2020-10-11 23:38:31.023428
MHH	Mastech Digital Inc	Mastech Digital Inc is a provider of Information Technology (IT) staffing services. The Company recruit through Recruiting Centers located in the U.S. and India that deliver a full range of recruiting and sourcing services.	{Industrials,"Staffing & Outsourcing Services","Employment Services"}	{BBSI,KELYA,KFRC,RHI,TBI,HSON,JOB}	Employment Services	Industrials	NYSE American	\N	2020-10-11 23:12:06.826203	2020-10-11 23:38:31.02423
MILN	Global X Millennials Thematic ETF	The investment seeks to provide investment results that correspond generally to the price and yield performance, before fees and expenses, of the Indxx Millennials Thematic Index.\n The fund invests more than 80% of its total assets in the securities of the underlying index. The underlying index is designed to measure the performance of U.S. listed companies that provide exposure to the millennial generation, (collectively, "Millennial Companies"), as defined by the index provider. The millennial generation refers to the demographic in the U.S. with birth years ranging from 1980 to 2000. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:06.866707	2020-10-11 23:38:31.023007
MSN	Emerson Radio Corporation	Emerson Radio Corp designs, sources, imports and markets housewares and consumer electronic products and licenses its trademarks to others for products domestically and internationally.	{Technology,"Consumer Electronics","Computer Hardware"}	{KOSS,LRAD,NC,VOXX,XLK}	Computer Hardware	Technology	NYSE American	\N	2020-10-11 23:12:07.269129	2020-10-11 23:38:31.023007
MBT	Mobile TeleSystems PJSC	Mobile TeleSystems PJSC is wireless telephone operator in Russia. It provides fixed-line broadband, data transfer, pay-TV and various value-added services, including Big Data, cloud services, financial services, e-commerce, and others.	{"Communication Services","Telecom Services"}	{TKC,ORAN,VOD,VEON}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:06.539271	2020-10-11 23:38:31.025871
MCBC	Macatawa Bank Corporation	Macatawa Bank Corp is a holding company of Macatawa Bank. It offers commercial and personal banking services, brokerage services, online banking and lending funds to individual customers and corporates through its subsidiary, Macatawa Bank.	{"Financial Services","Banks - Regional - US",Banks}	{MBWM,CHFC,MBTF,MFNC,IBCP,BUSE}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:06.562914	2020-10-11 23:38:31.021426
MCHX	Marchex Inc.	Marchex Inc is a mobile advertising technology company. It provides products and services for businesses of all sizes. The company's analytics technology can facilitate call quality, analyze calls and measure the outcomes of calls.	{"Consumer Cyclical","Marketing Services","Advertising & Marketing Services"}	{MSFT,IAC,INUV,GOOGL}	Advertising & Marketing Services	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:06.582426	2020-10-11 23:38:31.021426
MCRN	Milacron Holdings Corp.	Milacron Holdings Corp manufactures, distributes and services engineered and customized systems within plastic technology and processing industry. The Melt Delivery and Control Systems generate maximum revenue for the company.	{Industrials,"Diversified Industrials","Industrial Products"}	{LECO,THR,IEP,ADES}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:06.606804	2020-10-11 23:38:31.028166
MDCA	MDC Partners Inc.	MDC Partners Inc is a marketing company whose core activities involve direct marketing, mobile marketing, market research, data analytics, design & branding, social media communications and e-commerce.	{"Consumer Cyclical","Marketing Services","Advertising & Marketing Services"}	{NCMI,IPG,OMC}	Advertising & Marketing Services	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:06.627654	2020-10-11 23:38:31.021426
MDLQ	Medley LLC 7.25% Notes due 2024		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:06.642928	2020-10-11 23:38:31.024353
MDT	Medtronic plc.	Medtronic PLC is a medical technology company. It is primarily manufactures and sells device-based medical therapies. It operates in Cardiac and Vascular, Minimally Invasive Therapies, Restorative Therapies and Diabetes segments.	{Healthcare,"Medical Devices"}	{IART,CNMD,OMI,ABT,BAX,BSX,JNJ,SYK,EW,ZBH,XLV}	Medical Devices	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:06.661257	2020-10-11 23:38:31.025871
MESO	Mesoblast Limited	Mesoblast Ltd is engaged in the development of biologic products for the broad field of regenerative medicine. Its proprietary regenerative medicine technology platform is based on specialized cells known as mesenchymal lineage adult stem cells.	{Healthcare,Biotechnology}	{ZIOP,OSIR,ATHX,PSTI,JNJ,SNY,TEVA,VCEL,CLBS,SNGX,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:06.710654	2020-10-11 23:38:31.021426
MFGP	Micro Focus Intl PLC ADS each representing One Ord Sh	Micro Focus International PLC is a United Kingdom-based provider of software solutions. It provides software that allows companies to develop, test, deploy, assess and modernize business-critical enterprise applications.	{Technology,"Software - Application","Application Software"}	{XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:06.735619	2020-10-11 23:38:31.02423
MFV	MFS Special Value Trust	MFS Special Value Trust is a United States based diversified closed-end management investment company. Its investment objective is to seek high current income, but may also consider capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.75646	2020-10-11 23:38:31.024353
MGLN	Magellan Health Inc.	Magellan Health Inc provides managed healthcare business. It provides services to health plans, insurance companies, employers, labor unions and various governmental agencies.	{Healthcare,"Health Care Plans"}	{CSGP,QNST,ACN,XLV}	Health Care Plans	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:06.799554	2020-10-11 23:38:31.028166
MHF	Western Asset Municipal High Income Fund Inc.	Western Asset Municipal High Income Fund Inc is a diversified closed-end management investment company. The company's objective is to seek high current income exempt from federal income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.823741	2020-10-11 23:38:31.023428
MHN	Blackrock MuniHoldings New York Quality Fund Inc.	Blackrock Muniholdings NY Insured FD is a closed-end management investment company. Its investment objective is to provide shareholders with current income exempt from federal income tax and New York State and New York City personal income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.83724	2020-10-11 23:38:31.02423
MBRX	Moleculin Biotech Inc.	Moleculin Biotech Inc is a clinical-stage pharmaceutical company. The company is engaged in the development of anti-cancer drug candidates. Its drug development technologies include liposomal Annamycin, WP1122 and WP1066.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:06.534082	2020-10-11 23:38:31.02582
MCFT	MCBC Holdings Inc.	MCBC Holdings Inc is a designer and manufacturer of inboard tournament ski boats and luxury performance V-drive runabouts under the MasterCraft brand and high-end saltwater fishing boats under the Hydra-Sports brand.	{"Consumer Cyclical","Recreational Vehicles",Autos}	{}	Autos	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:06.57711	2020-10-11 23:38:31.028166
MCRI	Monarch Casino & Resort Inc.	Monarch Casino & Resort Inc owns and operates the Atlantis Casino Resort Spa, a hotel or casino facility in Reno, Nevada; the Monarch Casino Black Hawk in Black Hawk, Colorado and real estate proximate to the Atlantis and Monarch Casino Black Hawk.	{"Consumer Cyclical","Resorts & Casinos","Travel & Leisure"}	{PENN,CNTY,WYNN,BYD,MGM,LVS,UWN}	Travel & Leisure	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:06.606211	2020-10-11 23:38:31.02582
MDB	MongoDB Inc.	MongoDB Inc develops and sells subscriptions to a modern, general purpose database platform. It also provides post-contract support, training, and consulting services for its offerings.	{Technology,"Software - Application","Application Software"}	{PRGS,PXLW,XLK}	Application Software	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:06.625968	2020-10-11 23:38:31.025871
MDP	Meredith Corporation	Meredith Corp is a media company operating in America. The company focuses on publishing and marketing entertainment content which include home, family, food and lifestyle.	{"Consumer Cyclical",Publishing}	{SCHL,CMCSA,WMT,JW.A,PSO,XOXO,TGNA,SPGI,SSP}	Publishing	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:06.655803	2020-10-11 23:38:31.023007
MDY	SPDR MidCap Trust Series I	The investment seeks to provide investment results that, before expenses, correspond generally to the price and yield performance of the S&P MidCap 400® Index™.\n The Trust seeks to achieve its investment objective by holding a portfolio of the common stocks that are included in the index (the “Portfolio”), with the weight of each stock in the Portfolio substantially corresponding to the weight of such stock in the index. The index is composed of four hundred (400) selected stocks, all of which are listed on national stock exchanges, and span a broad range of major industry groups.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.678463	2020-10-11 23:38:31.02423
MFG	Mizuho Financial Group Inc. Sponosred ADR (Japan)	Mizuho Financial Group Inc engages in banking, trust banking, securities and other businesses related to financial services. The company provides financial products and services in relation to deposits, lending and exchange settlement.	{"Financial Services","Banks - Regional - Asia",Banks}	{IBN,SHG,MUFG,KB,SMFG}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.7312	2020-10-11 23:38:31.021426
MFUS	PIMCO Equitiy Series RAFI Dynamic Multi-Factor U.S. Equity	The investment seeks to track the investment results of the RAFI Dynamic Multi-Factor U.S. Index.\n The fund seeks to achieve its investment objective by investing at least 80% of its total assets (exclusive of collateral held from securities lending) in the component securities of the RAFI Dynamic Multi-Factor U.S. Index. The underlying index is constructed by RAFI Indices, LLC (the "index provider") using a rules-based approach to construct factor portfolios within the underlying index. The underlying index consists of "factor portfolios," each of which emphasizes one of the following factors: value, low volatility, quality, momentum and size.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.757338	2020-10-11 23:38:31.02423
MGK	Vanguard Mega Cap Growth	The investment seeks to track the performance of a benchmark index.\n The fund employs an indexing investment approach designed to track the performance of the CRSP US Mega Cap Growth Index. The index is a float-adjusted, market-capitalization-weighted index designed to measure equity market performance of mega-capitalization growth stocks in the United States. The Advisor attempts to replicate the target index by investing all, or substantially all, of its assets in the stocks that make up the index, holding each stock in approximately the same proportion as its weighting in the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.794495	2020-10-11 23:38:31.02423
MGYR	Magyar Bancorp Inc.	Magyar Bancorp Inc through its subsidiaries is engaged in attracting retail deposits from the general public and investing those deposits together with funds generated from operations in loans and investment securities.	{"Financial Services","Banks - Regional - US",Banks}	{OCFC,ORIT,PFS,KRNY,MSBF}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:06.818157	2020-10-11 23:38:31.025871
MHLA	Maiden Holdings Ltd. 6.625% Notes due 2046		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:06.834424	2020-10-11 23:38:31.023007
MICR	Micron Solutions Inc.	Micron Solutions Inc produces engineered medical device components requiring precision machining and injection molding. It also manufactures devices and equipment for military, law enforcement, automotive and consumer product applications.	{Healthcare,"Medical Devices"}	{XLV}	Medical Devices	Healthcare	NYSE American	\N	2020-10-11 23:12:06.848918	2020-10-11 23:38:31.024353
MIME	Mimecast Limited	Mimecast Ltd is a software solutions provider. Its key business involves provision of a software platform to evade internet security issues such as malware and spam.	{Technology,"Software - Infrastructure","Application Software"}	{OTEX,PANW,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:06.865962	2020-10-11 23:38:31.02582
MG	Mistras Group Inc	MISTRAS Group Inc provides technology-enabled asset protection solutions and systems. Services include routine inspections, asset integrity management, and asset integrity assessments.	{Industrials,"Security & Protection Services","Consulting & Outsourcing"}	{EXPO,LUNA,XOM,GLW,GD,GE,HON,CVX,TISI,LTBR}	Consulting & Outsourcing	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:06.766991	2020-10-11 23:38:31.028166
MGRC	McGrath RentCorp	McGrath RentCorp is a rental company offering modular building & portable storage, electronic test equipment, containment solutions for the storage of hazardous & non-hazardous liquids & solids; & also manufactures and sells modular classroom buildings.	{Industrials,"Rental & Leasing Services","Consulting & Outsourcing"}	{MINI,RCII,HEES,GFN,URI,AAN}	Consulting & Outsourcing	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:06.807182	2020-10-11 23:38:31.028166
MHLD	Maiden Holdings Ltd.	Maiden Holdings Ltd provides reinsurance products and services to regional and specialty property and casualty insurers.	{"Financial Services","Insurance - Reinsurance","Insurance - Specialty"}	{RNR,RE,AXS,AHL,RGA,TPRE}	Insurance - Specialty	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:06.835391	2020-10-11 23:38:31.031328
MICT	MICT Inc.	Micronet Enertec Technologies Inc designs develops, manufactures and sells rugged mobile computing devices that provide fleet operators and field workforces with computing solutions; and simulators and automatic test equipment.	{Industrials,"Aerospace & Defense"}	{}	Aerospace & Defense	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:06.855943	2020-10-11 23:38:31.021426
MITL	Mitel Networks Corporation	Mitel Networks Corp provides business communications and collaboration software and services. Its innovations make it easier to connect and collaborate, while reducing costs. The company operates in two segments: Cloud and Enterprise.	{Technology,"Communication Equipment"}	{CSCO,MSFT,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:06.884358	2020-10-11 23:38:31.028166
MKTX	MarketAxess Holdings Inc.	MarketAxess Holdings Inc provides electronic fixed-income trading platform. It's trading categories include US and European high-grade corporate, European Government, and high-yield bonds. The firm also provides market information and analytic tools.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{NDAQ,CME,ICE,CBOE}	Brokers & Exchanges	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:06.935543	2020-10-11 23:38:31.028166
MLPA	Global X MLP	The investment seeks to provide investment results that correspond generally to the price and yield performance, before fees and expenses, of the Solactive MLP Infrastructure Index ("underlying index").\n The fund invests at least 80% of its net assets in the securities of the underlying index. Moreover, at least 80% of the fund's net assets will be invested in securities that have economic characteristics of the Master Limited Partnership ("MLP") asset class. The underlying index is intended to give investors a means of tracking the performance of the energy infrastructure MLP asset class in the United States. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.956304	2020-10-11 23:38:31.025871
MLPX	Global X MLP & Energy Infrastructure	The investment seeks to provide investment results that correspond generally to the price and yield performance, before fees and expenses, of the Solactive MLP & Energy Infrastructure Index.\n The fund invests at least 80% of its total assets in the securities of the index. It also invests at least 80% of its total assets in securities of master limited partnerships and energy infrastructure corporations. The fund's 80% investment policies are non-fundamental and require 60 days' prior written notice to shareholders before they can be changed. The index tracks the performance of midstream energy infrastructure MLPs and corporations. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.977431	2020-10-11 23:38:31.02582
MLVF	Malvern Bancorp Inc.	Malvern Bancorp Inc is engaged in attracting deposits from businesses and the general public in Chester County, Pennsylvania and investing those deposits, together with borrowings and funds.	{"Financial Services","Savings & Cooperative Banks",Banks}	{WVFC,ESSA,NWBI,PBIP}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:06.996502	2020-10-11 23:38:31.028166
MMI	Marcus & Millichap Inc.	Marcus & Millichap, Inc. is a national brokerage firm engaged in commercial real estate investment sales, financing, research and advisory services.	{"Real Estate","Real Estate Services"}	{JLL,HF,KW,RMAX}	Real Estate Services	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:07.014215	2020-10-11 23:38:31.02582
MMU	Western Asset Managed Municipals Fund Inc.	Western Asset Managed Municipals Fd Inc is a non-diversified, closed-end management investment company. The fund seeks to maximize current income exempt from federal income tax as is consistent with preservation of principal.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.035245	2020-10-11 23:38:31.02423
MNKD	MannKind Corporation	MannKind Corp is engaged in the discovery, development, and commercialization of therapeutic products for diseases such as diabetes and cancer. Its products and technologies include Afrezza, Dry powder formulation, Inhalation profiling, and others.	{Healthcare,Biotechnology}	{NKTR,HALO,NVO,LLY,PFE,MRK,ARDM,ALBO,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:07.062294	2020-10-11 23:38:31.028166
MO	Altria Group Inc.	Altria Group Inc along with its subsidiaries is engaged in the manufacture and sale of cigarettes, machine-made large cigars, pipe tobacco, smokeless tobacco products and wine.	{"Consumer Defensive",Tobacco,"Tobacco Products"}	{BTI,VGR,PM,XXII}	Tobacco Products	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:07.08591	2020-10-11 23:38:31.023428
MFSF	MutualFirst Financial Inc.	MutualFirst Financial Inc business consists of attracting retail deposits from the general public, including some brokered deposits, and investing those funds in loans secured by first mortgages on owner-occupied, among others.	{"Financial Services","Banks - Regional - US",Banks}	{THFF,LKFN,GABC,SRCE,HBNC,FSFG}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:06.749218	2020-10-11 23:38:31.023428
MGI	Moneygram International Inc.	MoneyGram International Inc is a money transfer and payment services company. Its products include money transfers, bill payment services, money order services and official check processing.	{"Financial Services","Credit Services"}	{EEFT,TSS,GPN,FIS,WU,WEX}	Credit Services	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:06.778198	2020-10-11 23:38:31.021426
MGV	Vanguard Mega Cap Value	The investment seeks the performance of a benchmark index.\n The fund employs an indexing investment approach designed to track the performance of the CRSP US Mega Cap Value Index. The index is a float-adjusted, market-capitalization-weighted index designed to measure equity market performance of mega-capitalization value stocks in the United States. The Advisor attempts to replicate the target index by investing all, or substantially all, of its assets in the stocks that make up the index, holding each stock in approximately the same proportion as its weighting in the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.822316	2020-10-11 23:38:31.031328
MIK	The Michaels Companies Inc.	The Michaels Companies Inc owns and operates a chain of specialty retail stores in 49 states and Canada featuring arts, crafts, framing, floral, home decor, seasonal merchandise for the hobbyist and do-it-yourself home decorator.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{PIR,BBW,GME,PAG}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:06.864248	2020-10-11 23:38:31.025871
MITT	AG Mortgage Investment Trust Inc.	AG Mortgage Investment Trust Inc invest in, acquire and manage a diversified portfolio of residential mortgage assets, other real estate-related securities and financial assets.	{"Real Estate","REIT - Diversified",REITs}	{NYMT,DX,CMO,IVR,PMT,ARR}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:06.891053	2020-10-11 23:38:31.025871
MKSI	MKS Instruments Inc.	MKS Instruments Inc is a provider of instruments, subsystems and process control solutions that measure, control, power, deliver, monitor and analyze critical parameters of manufacturing processes to improve process performance and productivity.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:06.91585	2020-10-11 23:38:31.024353
MLNX	Mellanox Technologies Ltd.	Mellanox Technologies Ltd is engaged in the development, manufacturing, marketing and sales of end-to-end Ethernet and InfiniBand intelligent interconnect solutions and services for servers, storage, and hyper-converged infrastructure.	{Technology,Semiconductors}	{XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:06.94891	2020-10-11 23:38:31.023428
MLPZ	UBS AG ETRACS ETRACS 2xMonthly Leveraged S&P MLP Index ETN Series B due February 12 2046	The investment seeks to provide two times leveraged long exposure to the compounded monthly performance of the S&P MLP Index (the “index”).\n The index is designed to measure leading Master Limited Partnerships (“MLPs”) and limited liability companies (“LLCs”) that trade on major U.S. exchanges. As the vast majority of traded partnerships have operations in the oil and gas industries, the index focuses on companies in the Global Industry Classification Standard® (“GICS”) Energy Sector and the GICS Gas Utilities Industry.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.979572	2020-10-11 23:38:31.023007
MMC	Marsh & McLennan Companies Inc.	Marsh & McLennan Companies Inc is a professional services firm which provides advice and solutions in the areas of risk, strategy, and human capital.	{"Financial Services","Insurance Brokers","Brokers & Exchanges"}	{ACGL,BRO,AJG,AXS,AON,WLTW}	Brokers & Exchanges	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.996817	2020-10-11 23:38:31.028166
MMLP	Martin Midstream Partners L.P.	Martin Midstream Partners LP provides terminalling and storage services for petroleum products and by-products; natural gas services; sulfur and sulfur-based products and marine transportation services.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{FGP,GEL,EPD,KEX,PAA,LNG,CVX,NGL,SXE}	Oil & Gas - Midstream	Energy	Nasdaq Global Select	\N	2020-10-11 23:12:07.018323	2020-10-11 23:38:31.02423
MN	Manning & Napier Inc. Class A	Manning & Napier Inc is an independent investment management firm. It provides investment management services through separately managed accounts, mutual funds, and collective investment trust funds, as well as a variety of consultative services.	{"Financial Services","Asset Management"}	{GROW,VRTS,WDR,FII,EV,BEN,LM,WETF,APAM}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.04455	2020-10-11 23:38:31.031328
MNRO	Monro Inc.	Monro Inc provides automotive undercar repair and tire services in the United States. The Company provides a broad range of services on passenger cars, light trucks and vans for brakes; mufflers and exhaust systems and steering among others.	{"Consumer Cyclical","Auto Parts",Autos}	{PNTR,AN,GPI,AZO,AAP,ABG,ORLY,IEP,ASH}	Autos	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:07.07047	2020-10-11 23:38:31.023007
MOC	Command Security Corporation	Command Security Corp is a provider of uniformed security officers and aviation security services to commercial, financial, industrial, aviation and governmental customers in the the United States.	{Industrials,"Security & Protection Services","Consulting & Outsourcing"}	{ABM,BCO}	Consulting & Outsourcing	Industrials	NYSE American	\N	2020-10-11 23:12:07.089215	2020-10-11 23:38:31.02423
MOMO	Momo Inc.	Momo Inc is a mobile-based social networking platform company. The company enables its users to establish and expand social relationships based on location and interests.	{Technology,"Internet Content & Information","Online Media"}	{VRSN,AKAM,GLUU,WIX,XLK}	Online Media	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:07.10864	2020-10-11 23:38:31.028166
MGEN	Miragen Therapeutics Inc.	Miragen Therapeutics Inc is a clinical-stage biopharmaceutical company which engaged in discovering and developing proprietary RNA (Ribonucleic acid)-targeted therapeutics.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{RDNT,DGX,ENZ,LH,XLV}	Medical Diagnostics & Research	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:06.775395	2020-10-11 23:38:31.028166
MGM	MGM Resorts International	MGM Resorts International owns and operates casino, hotel and entertainment resorts. The company offers its services through its properties in Las Vegas include Bellagio, MGM Grand, Mandalay Bay, Mirage, Luxor, and New York-New York.	{"Consumer Cyclical","Resorts & Casinos","Travel & Leisure"}	{MCRI,PENN,CNTY,WYNN,MLCO,BYD,LVS}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:06.799179	2020-10-11 23:38:31.024353
MHE	BlackRock Massachusetts Tax-Exempt Trust	BlackRock Massachusetts Tax-Exempt Trust is a closed-end management investment company. Its objective is to provide shareholders with as high a level of income exempt from both regular federal income taxes and Massachusetts personal income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.822853	2020-10-11 23:38:31.021426
MHNC	Maiden Holdings North America Ltd. 7.75% Notes due 2043		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:06.838042	2020-10-11 23:38:31.025871
MIE	Cohen & Steers MLP Income and Energy Opportunity Fund Inc.	Cohen & Steers MLP Income and Energy Opp is a a non-diversified, closed-end management investment company. The Fund's investment objective is to provide attractive total return, comprised of high current income and price appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.861402	2020-10-11 23:38:31.02423
MINDP	Mitcham Industries Inc. Series A 9.00% Series A Cumulative Preferred Stock	Mitcham Industries Inc provides equipment leasing, sales, and service to the seismic industry. The company operates in two business segments. The Marine Technology Products segment and Equipment Leasing.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:06.880205	2020-10-11 23:38:31.024353
MIXT	MiX Telematics Limited American Depositary Shares each representing 25	MiX Telematics Ltd provides fleet and mobile asset management solutions delivered as Software as a Service. The company also offers smartphone applications to support all of the tracking products in its portfolio.	{Technology,"Software - Infrastructure","Application Software"}	{QCOM,TRMB,DHR,GOOGL,BHGE,PEP,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:06.89913	2020-10-11 23:38:31.023428
MLM	Martin Marietta Materials Inc.	Martin Marietta Materials Inc is a natural-resource-based building materials company. It supplies aggregates products used for the construction of infrastructure, nonresidential, and residential projects.	{"Basic Materials","Building Materials"}	{USLM,CRH,OXY,VMC}	Building Materials	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:06.944458	2020-10-11 23:38:31.021426
MLPG	UBS AG Exchange Traded Access Securities (E-TRACS) Linked to the Alerian Natural Gas MLP Index due July 9 2040	The investment seeks to replicate, net of expenses, the Alerian Natural Gas MLP Index.\n The index is an equal-weighted composite of the 15 largest natural gas infrastructure Master Limited Partnerships by market capitalization. The index, whose constituents earn the majority of their cash flow from the transportation, storage, and processing of natural gas and natural gas liquids, provides investors with a benchmark for the infrastructure component of the natural gas industry.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.962516	2020-10-11 23:38:31.02423
MMDMW	Modern Media Acquisition Corp. Warrant		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:07.007737	2020-10-11 23:38:31.025871
MMSI	Merit Medical Systems Inc.	Merit Medical Systems Inc designs, develops, manufactures and sells medical devices used in interventional and diagnostic procedures. Its products are used in thoracic surgery, interventional nephrology, vascular surgery, and oncology and pain management.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{ICUI,ANGO,BSX,JNJ,TFX,EW,MDT,XLV}	Medical Instruments & Equipment	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:07.029196	2020-10-11 23:38:31.028166
MNA	IQ Merger Arbitrage	The investment seeks investment results that correspond generally to the price and yield performance of its underlying index, the IQ Merger Arbitrage Index.\n The fund invests at least 80% of its net assets, plus the amount of any borrowings for investment purposes, in the investments included in its underlying index. The underlying index seeks to employ a systematic investment process designed to identify opportunities in companies whose equity securities trade in developed markets, including the U.S., and which are involved in announced mergers, acquisitions and other buyout-related transactions. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:07.045459	2020-10-11 23:38:31.028166
MNP	Western Asset Municipal Partners Fund Inc.	Western Asset Municipal Partners Fd Inc is a diversified, closed-end management investment company. The fund's primary investment objective is to seek a high level of current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.067836	2020-10-11 23:38:31.02423
MNTX	Manitex International Inc.	Manitex International Inc, through its subsidiaries, is engaged in the design, manufacture and distribution of engineered lifting solutions. Its products include boom trucks, truck cranes and sign cranes.	{Industrials,"Diversified Industrials","Industrial Products"}	{TEX,NC,HY,MTW}	Industrial Products	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:07.085223	2020-10-11 23:38:31.02582
MGF	MFS Government Markets Income Trust	MFS Government Markets Income Trust is a United States based diversified, closed-end management investment company. Its investment objective is to seek high current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.783419	2020-10-11 23:38:31.02582
MHI	Pioneer Municipal High Income Trust of Beneficial Interest	Pioneer Municipal High Income Trust is a diversified, closed-end management investment company. Its investment objective is to seek a high level of current income exempt from regular federal income tax and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.831322	2020-10-11 23:38:31.028166
MIN	MFS Intermediate Income Trust	MFS Intermediate Income Trust is a diversified closed-end management investment company. The fund's investment objective is to seek high current income, but may also consider capital appreciation. It invests the assets primarily in debt instruments.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.871466	2020-10-11 23:38:31.021426
MIW	Eaton Vance Michigan Municipal Bond Fund of Beneficial Interest $.01 par value	Eaton Vance Michigan Municipal Bond Fund is a closed-end investment company. Its objective is to provide current income exempt from regular federal income tax, including alternative minimum tax, and, in state specific funds, tax in its specified state.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:06.888911	2020-10-11 23:38:31.02582
MKL	Markel Corporation	Markel Corp is engaged in the business of property and casualty insurance. It focuses primarily on specialty lines, such as executive liability to commercial equine insurance. It also invests in bakery equipment manufacturing and residential homebuilding.	{"Financial Services","Insurance - Property & Casualty"}	{SIGI,ACGL,AFG,RLI,AHL,WTM}	Insurance - Property & Casualty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.908422	2020-10-11 23:38:31.028166
MLP	Maui Land & Pineapple Company Inc.	Maui Land & Pineapple Co Inc is engaged in community development and resort operations. The Company owns approximately 23,000 acres of land on Maui and develops, sells, and manages residential, resort, commercial, and industrial real estate.	{"Real Estate","Real Estate - General","Real Estate Services"}	{SENEA,LANC,CDZI,CWT,AWR,SJM,AWK,SJW,BEL}	Real Estate Services	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:06.948661	2020-10-11 23:38:31.028166
MLPO	Credit Suisse Group Exchange Traded Notes due December 4 2034 Linked to the S&P MLP Index	The investment seeks to provide exposure to the price return version of the S&P MLP Index (the ¿index¿).\n The index includes both master limited partnerships and publicly traded limited liability companies. In addition, the ETNs allow investors the potential to receive quarterly coupons that generally reflect the net cash distributions made by the index constituents, reduced by the application of the fees.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.966394	2020-10-11 23:38:31.031328
MMAC	MMA Capital Management LLC	MMA Capital Management LLC provides finance to housing and infrastructure in U.S. It operates through business lines including Leveraged Bonds, Low-Income Housing Tax Credits, and Energy capital.	{"Financial Services","Specialty Finance",Banks}	{}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:06.996492	2020-10-11 23:38:31.02582
MMIT	IQ MacKay Municipal Intermediate	The investment seeks current income exempt from federal income tax.\n The fund is an actively managed ETF and thus does not seek to replicate the performance of a specific index. Instead, it uses an active management strategy to meet its investment objective. The fund, under normal circumstances, invests at least 80% of its assets (net assets plus borrowings for investment purposes) in debt securities whose interest is, in the opinion of bond counsel for the issuer at the time of issuance, exempt from federal income tax ("Municipal Bonds").	{}	{}			NYSE Arca	\N	2020-10-11 23:12:07.016677	2020-10-11 23:38:31.028166
MMT	MFS Multimarket Income Trust	MFS Multimarket Income Trust is a United States based diversified closed-end management investment company. Its investment objective is to seek high current income, but may also consider capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.033346	2020-10-11 23:38:31.023428
MNI	McClatchy Company (The)	The McClatchy Co is a news and information publisher of publications such as the Miami Herald, the Kansas City Star, the Sacramento Bee, the Charlotte Observer, the (Raleigh) News and Observer, and the (Fort Worth) Star-Telegram.	{"Consumer Cyclical",Publishing}	{DJCO,LEE,NYT,AHC,NWSA,NEWM,TGNA,SSP}	Publishing	Consumer Cyclical	NYSE American	\N	2020-10-11 23:12:07.058785	2020-10-11 23:38:31.023007
MOD	Modine Manufacturing Company	Modine Manufacturing Co manufactures thermal management systems & components, bringing heating & cooling technology and solutions to diversified markets.	{"Consumer Cyclical","Auto Parts",Autos}	{CVGI,NAV,BWA,HON,SMP,UTX,DAN,MTOR,VC}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:07.091408	2020-10-11 23:38:31.021426
MORN	Morningstar Inc.	Morningstar Inc is a provider of independent investment research on two core sectors data and research. It offers data on investments such as mutual funds, stocks, exchange-traded funds, closed-end funds, separate accounts, and variable annuities.	{Industrials,"Business Services"}	{TRI,TST,VALU,FDS,MCO,DNB,MSCI,ENV,NWSA,SPGI}	Business Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:07.115631	2020-10-11 23:38:31.021426
MGU	Macquarie Global Infrastructure Total Return Fund Inc.	Macquarie Global Infrastructure Total Re is a diversified, closed-end investment management company. Its investment objective is to provide to its common stockholders a high level of total return consisting of dividends and other income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.81192	2020-10-11 23:38:31.02582
MHK	Mohawk Industries Inc.	Mohawk Industries Inc is a flooring manufacturer that creates products to enhance residential and commercial spaces around the world. The company's business segments are Global Ceramic, Flooring North America and Flooring Rest of the World.	{"Consumer Cyclical","Home Furnishings & Fixtures","Manufacturing - Apparel & Furniture"}	{DXYN,TREX,AWI,FBHS,TILE,DOOR}	Manufacturing - Apparel & Furniture	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:06.830125	2020-10-11 23:38:31.028166
MIDU	Direxion Mid Cap Bull 3X Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the daily performance of the S&P MidCap® 400 Index.\n The fund, under normal circumstances, invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, exchange-traded funds ("ETFs") that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index measures the performance of 400 mid-sized companies in the United States. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.850803	2020-10-11 23:38:31.031328
MIND	Mitcham Industries Inc.	Mitcham Industries Inc provides equipment leasing, sales, and service to the seismic industry. The company operates in two business segments. The Marine Technology Products segment and Equipment Leasing.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{HEES,GATX,NGS,ACY,GEOS,CCI,AROC,IO,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:06.880319	2020-10-11 23:38:31.023428
MKC	McCormick & Company Incorporated	McCormick & Co Inc acts as a manufacturer, marketer, and distributor of spices, herbs, extracts, seasonings, and other flavorings. It serves its products to quick-service restaurants, retail grocery chains, and packaged food processors.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{HAIN,CAG,TSN,BGS,THS,PEP}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:06.90439	2020-10-11 23:38:31.02423
MLN	VanEck Vectors AMT-Free Long Municipal Index	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the Bloomberg Barclays AMT-Free Long Continuous Municipal Index.\n The fund normally invests at least 80% of its total assets in fixed income securities that comprise the index. The index is comprised of publicly traded municipal bonds that cover the U.S. dollar denominated long-term tax-exempt bond market.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.944282	2020-10-11 23:38:31.02423
MLR	Miller Industries Inc.	Miller Industries Inc. is a manufacturer of towing and recovery equipment. It products are marketed under the brand names of Century, Challenger, Holmes, Champion, Eagle, Titan, Jige, Boniface, Vulcan, and Chevron.	{"Consumer Cyclical","Auto Parts",Autos}	{SPAR,OSK,SUP,AXL,WBC,SRI,TOWR}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:06.985673	2020-10-11 23:38:31.023428
MMP	Magellan Midstream Partners L.P. Limited Partnership	Magellan Midstream Partners LP is engaged in the transportation, storage and distribution of refined petroleum products. The company operates through three segments: refined products, crude oil and marine storage.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{PAA,EEP,HEP,NS,EEQ,BKEP,MPLX,DKL,PSXP,PAGP}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:07.021654	2020-10-11 23:38:31.02582
MNDO	MIND C.T.I. Ltd.	MIND C.T.I. Ltd develops, manufactures, markets & implements real-time & off-line convergent billing & customer care software solutions for communication providers, including traditional wireline and wireless, VoIP, and broadband IP network operators.	{Technology,"Software - Infrastructure","Application Software"}	{CSGS,MTSL,SNCR,EVOL,DOX,APPS,XLK}	Application Software	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:07.050403	2020-10-11 23:38:31.02582
MNST	Monster Beverage Corporation	Monster Beverage Corp develops, markets, sells and distributes energy drink beverages and/or concentrates for energy drink beverages. Its brands include Monster Energy, Java Monster, Monster Rehab, NOS, Full Throttle, Mother, among others.	{"Consumer Defensive","Beverages - Soft Drinks","Beverages - Non-Alcoholic"}	{COT,FIZZ,KO,REED,CCEP,PEP}	Beverages - Non-Alcoholic	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:07.078109	2020-10-11 23:38:31.028166
MOGLC	Gabelli Media Mogul NextShares	The fund seeks to provide capital appreciation.\n Under normal market conditions, the fund invests at least 80% of its net assets, plus borrowings for investment purposes, in the media industry. Media industry companies are companies that are primarily engaged in the production, sale and distribution of goods or services used in the media industry. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:07.098849	2020-10-11 23:38:31.031328
MPA	Blackrock MuniYield Pennsylvania Quality Fund	Blackrock Muniyield PA Insured Fd is a closed end fund with investment objective is to provide shareholders with current income exempt from federal and California income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.1415	2020-10-11 23:38:31.024353
MPX	Marine Products Corporation	Marine Products Corp designs, manufactures and sells fiberglass powerboats Chaparral and Robalo brands.	{"Consumer Cyclical","Recreational Vehicles",Autos}	{BC,HZO,MBUU}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:07.163569	2020-10-11 23:38:31.024353
MRK	Merck & Company Inc. (new)	Merck & Co Inc is a health care company that delivers health solutions through its prescription medicines, vaccines, biologic therapies and animal health products.	{Healthcare,"Drug Manufacturers - Major","Drug Manufacturers"}	{AMGN,GSK,NVS,BMY,JNJ,LLY,PFE,SNY,TEVA,XLV}	Drug Manufacturers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:07.185561	2020-10-11 23:38:31.024353
MHO	M/I Homes Inc.	M/I Homes Inc is a part of the residential construction industry. It primarily builds single-family residential properties for the purpose of sale.	{"Consumer Cyclical","Residential Construction","Homebuilding & Construction"}	{PHM,NVR,DHI,HOV,MTH,MDC,KBH,BZH,TMHC}	Homebuilding & Construction	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:06.845685	2020-10-11 23:38:31.02582
MINT	PIMCO Enhanced Short Maturity Active Exchange-Traded Fund	The investment seeks maximum current income, consistent with preservation of capital and daily liquidity.\n The fund invests at least 80% of its net assets in a diversified portfolio of Fixed Income Instruments of varying maturities, which may be represented by forwards. "Fixed Income Instruments" include bonds, debt securities and other similar instruments issued by various U.S. and non-U.S. public- or private-sector entities. The average portfolio duration of this fund will vary based on PIMCO's market forecasts and will normally not exceed one year.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.883907	2020-10-11 23:38:31.02423
MLHR	Herman Miller Inc.	Herman Miller Inc is engaged in research, designing, manufacturing, and distribution of interior furnishings. It operates through several focused businesses, brands, and distribution channels.	{Industrials,"Business Equipment","Industrial Products"}	{KEQU,KBAL,VIRC,SCS,LZB,LEG,HNI,KNL}	Industrial Products	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:06.94252	2020-10-11 23:38:31.02582
MLPI	UBS AG ETN	The investment seeks to replicate, net of expenses, the Alerian MLP Infrastructure Index.\n The index provides exposure to the infrastructure component of the Master Limited Partnership asset class. Its constituents each earn at least 50% of their EBITDA from assets that are not directly exposed to changes in commodity prices. The index is a composite of 25 energy infrastructure MLPs.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.966093	2020-10-11 23:38:31.023428
MMIN	IQ MacKay Municipal Insured	The investment seeks current income exempt from federal income tax.\n The fund is an actively managed ETF and thus does not seek to replicate the performance of a specific index. Instead, it uses an active management strategy to meet its investment objective. The fund, under normal circumstances, invests at least 80% of its assets (net assets plus borrowings for investment purposes) in: (i) debt securities whose interest is, in the opinion of bond counsel for the issuer at the time of issuance, exempt from federal Income tax ("Municipal Bonds"); and (ii) debt securities covered by an insurance policy guaranteeing the payment of principal and interest.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:07.015158	2020-10-11 23:38:31.031328
MMYT	MakeMyTrip Limited	MakeMyTrip Ltd is an online travel company in India. Through its primary website, www.makemytrip.com, and mobile platforms, travellers can research, plan and book travel services and products in India as well as overseas.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{CTRP,EXPE}	Travel & Leisure	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:07.044493	2020-10-11 23:38:31.024353
MNOV	MediciNova Inc.	MediciNova Inc is a biopharmaceutical company. It is engaged in acquiring and developing small molecule therapeutics for the treatment of serious diseases with unmet medical needs.	{Healthcare,Biotechnology}	{BDSI,NVS,LLY,SNY,ALKS,TEVA,TTNP,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:07.066858	2020-10-11 23:38:31.024353
MOAT	VanEck Vectors Morningstar Wide Moat	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the Morningstar® Wide Moat Focus IndexSM.\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The index is comprised of securities issued by companies that Morningstar, Inc. ("Morningstar") determines to have sustainable competitive advantages based on a proprietary methodology that considers quantitative and qualitative factors ("wide moat companies"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:07.085718	2020-10-11 23:38:31.025871
MOO	VanEck Vectors Agribusiness	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the MVIS® Global Agribusiness Index.\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The index includes equity securities of companies that generate at least 50% of their revenues from agri-chemicals, animal health and fertilizers, seeds and traits, from farm/irrigation equipment and farm machinery, aquaculture and fishing, livestock, cultivation and plantations and trading of agricultural products. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:07.109495	2020-10-11 23:38:31.024353
MOSY	MoSys Inc.	MoSys Inc together with its subsidiaries is a fabless semiconductor company focused on the development and sale of integrated circuits (ICs) for the high-speed networking, communications, storage and computing markets.	{Technology,Semiconductors}	{RMBS,CY,MU,GSIT,PXLW,IPHI,XLK}	Semiconductors	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:07.125927	2020-10-11 23:38:31.031328
MPW	Medical Properties Trust Inc.	Medical Properties Trust Inc is a self-advised real estate investment trust. It is engaged in the business of investing in, owning, and leasing healthcare real estate.	{"Real Estate","REIT - Healthcare Facilities",REITs}	{HR,ARE,VTR,LTC,NHI,SBRA,DOC,TIPT}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:07.153331	2020-10-11 23:38:31.031328
MRCC	Monroe Capital Corporation	Monroe Capital Corp is a specialty finance company providing financing to lower middle-market companies in U.S & Canada. It provides customized financing solutions focused mainly on senior, unitranche and junior secured debt.	{"Financial Services","Specialty Finance",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:07.175833	2020-10-11 23:38:31.031328
MINI	Mobile Mini Inc.	Mobile Mini Inc provides fixed and portable storage solutions. It offers solid and liquid containment units on a rental basis.	{Industrials,"Rental & Leasing Services","Consulting & Outsourcing"}	{MGRC,GFN,CAI}	Consulting & Outsourcing	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:06.881954	2020-10-11 23:38:31.031328
MLI	Mueller Industries Inc.	Mueller Industries Inc is a manufacturer and distributor of copper, brass, aluminum, and plastic products. The product range consists of copper tube and fittings, line sets, brass and copper alloy rod, and aluminum impact extrusions.	{Industrials,"Metal Fabrication","Industrial Products"}	{KALU,TG,SPLP,BRSS,AA}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:06.944607	2020-10-11 23:38:31.024353
MMD	MainStay MacKay DefinedTerm Municipal Opportunities Fund	MainStay DefinedTerm Municipal Opps Fund is a diversified closed-end management investment company. Its investment objective is to seek current income exempt from regular U.S. Federal income taxes as well as total return.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.000218	2020-10-11 23:38:31.023428
MMM	3M Company	3M Co is a diversified technology company. It manufactures a diverse array of industrial and consumer products. Its business segments are Industrial, Safety and Graphics, Health Care, Electronics and Energy, and Consumer.	{Industrials,"Diversified Industrials","Industrial Products"}	{AMZN,ALGN,DHR,GE,ITW,JNJ,HON,HPQ,XRAY,JCI}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:07.021567	2020-10-11 23:38:31.021426
MNE	Blackrock Muni New York Intermediate Duration Fund Inc	Blackrock Muni NY Interm Dur FD has investment objective to provide shareholders with high current income exempt from federal income tax and New York State and New York City personal income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.051337	2020-10-11 23:38:31.025871
MNTA	Momenta Pharmaceuticals Inc.	Momenta Pharmaceuticals Inc is a biotechnology company. It is engaged in developing generic versions of complex drugs, biosimilars and novel therapeutics for oncology and autoimmune disease.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{AMGN,MDCO,BIIB,NVS,BAX,BMY,JNJ,PFE,MRK,TEVA,XLV}	Drug Manufacturers	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:07.083033	2020-10-11 23:38:31.028166
MORL	ETRACS Monthly Pay 2XLeveraged Mortgage REIT ETN	The investment seeks to link to the Market Vectors® Global Mortgage REITs Index.\n The index tracks the overall performance of publicly-traded mortgage REITs that derive at least 50% of their revenues from mortgage-related activities. The Securities are senior unsecured debt securities issued by UBS AG (UBS). The Securities provide a monthly compounded two times leveraged long exposure to the performance of the Index, reduced by the Accrued Fees.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:07.114154	2020-10-11 23:38:31.025871
MOXC	Moxian Inc.	Moxian Inc engages in the business of operating a social network platform that integrates social media and business into one single platform. The company has two primary core products: Moxian User App and Moxian Business App.	{Technology,"Internet Content & Information","Online Media"}	{YNDX,JCOM,FB,GOOGL,XLK}	Online Media	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:07.133664	2020-10-11 23:38:31.028166
MPWR	Monolithic Power Systems Inc.	Monolithic Power Systems Inc is a provider of high-performance power solutions. These solutions are found in industrial applications, telecommunication infrastructures, cloud computing, automotive, and consumer applications.	{Technology,Semiconductors}	{POWI,MXIM,SMTC,OIIM,ADI,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:07.160288	2020-10-11 23:38:31.028166
MRIN	Marin Software Incorporated	Marin Software Inc provides a cloud-based digital advertising management solution for search, display, social and mobile advertising channels to improve financial performance, realize efficiencies and time savings and improve business decisions.	{Technology,"Software - Application","Application Software"}	{ADBE,SYMC,SREV,GOOGL,XLK}	Application Software	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:07.184481	2020-10-11 23:38:31.025871
MRVL	Marvell Technology Group Ltd.	Marvell Technology Group Ltd and its subsidiaries is a fabless semiconductor provider of application-specific standard products. The company's product portfolio includes devices for storage, networking and connectivity.	{Technology,Semiconductors}	{INTC,DSPG,CY,QCOM,NVDA,STM,TXN,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:07.22335	2020-10-11 23:38:31.025871
MSFT	Microsoft Corporation	Microsoft Corp is a technology company. It develop, license, and support a wide range of software products and services. Its business is organized into three segments: Productivity and Business Processes, Intelligent Cloud, and More Personal Computing.	{Technology,"Software - Infrastructure","Application Software"}	{CSCO,AAPL,SAP,RHT,IBM,ORCL,HPQ,GOOGL,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:07.24899	2020-10-11 23:38:31.025871
MINC	AdvisorShares Newfleet Multi-sector Income	The investment seeks to provide current income consistent with preservation of capital, while limiting fluctuations in net asset value ("NAV") due to changes in interest rates.\n The Sub-Advisor applies a time-tested approach and extensive credit research to capitalize on opportunities across undervalued areas of the bond markets. The fund principally invests in investment-grade securities, which are securities with credit ratings within the four highest rating categories of a nationally recognized statistical rating organization or, if unrated, those securities that the Sub-Advisor determines to be of comparable quality.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.875391	2020-10-11 23:38:31.028166
MIY	Blackrock MuniYield Michigan Quality Fund Inc.	Blackrock Muniyield Michigan Quality Fund Inc is a closed-end fund. Its investment objective is to provide shareholders with as high a level of current income exempt from federal and Michigan income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.900294	2020-10-11 23:38:31.031328
MLNT	Melinta Therapeutics Inc.	Melinta Therapeutics Inc is a commercial-stage biopharmaceutical company. It is developing antibiotic, Baxdela (delafloxacin), for the treatment of adults with serious skin infections known as acute bacterial skin and skin structure infections.	{Healthcare,Biotechnology}	{JNJ,EBS,NBY,INVA,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:06.944523	2020-10-11 23:38:31.031328
MLPE	C-Tracks ETN on Miller/Howard Fundamental MLP Index Series B Due July 13 2026	The C-Tracks are designed for investors who seek exposure to the performance of the Miller/Howard MLP Fundamental Index and any distributions that may be paid on the securities included in the Index, reduced by an accrued investor fee. The Index is designed to measure the performance of 25 energy master limited partnerships (¿MLPs¿) selected quarterly by a methodology that is based upon certain quantitative fundamental factors of publicly traded MLPs, including distribution growth, estimated capital expenditures and distribution coverage.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.961511	2020-10-11 23:38:31.024353
MMDMR	Modern Media Acquisition Corp. Right		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:07.000615	2020-10-11 23:38:31.021426
MMV	Eaton Vance Massachusetts Municipal Income Trust Shares of Beneficial Interest	Eaton Vance MA Muni Income Trust is a non-diversified closed-end management investment company which seeks to provide current income exempt from regular federal income tax and taxes in its specified state.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:07.03967	2020-10-11 23:38:31.021426
MNLO	Menlo Therapeutics Inc.	Menlo Therapeutics Inc is a biopharmaceutical company focused on the development and commercialization of serlopitant for the treatment of pruritus, associated with dermatologic conditions such as atopic dermatitis, psoriasis and prurigo nodularis.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:07.065298	2020-10-11 23:38:31.021426
MODN	Model N Inc.	Model N Inc provides revenue management cloud solutions for life science and technology companies. Its portfolio includes Revenue Cloud, Revenue Cloud for Pharma, Revenue Cloud for MedTech and Revenue Cloud for High Tech.	{Technology,"Software - Application","Application Software"}	{CERN,HSTM,MDRX,SSNC,ORCL,VEEV,CSLT,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:07.093299	2020-10-11 23:38:31.028166
MOS	Mosaic Company (The)	Mosaic Co has its activities based in the agriculture industry. Its business involves production and marketing of crop enhancement products such as concentrated phosphate and potash crop nutrients.	{"Basic Materials","Agricultural Inputs",Agriculture}	{BG,CF,IPI,VALE,ICL}	Agriculture	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:07.118161	2020-10-11 23:38:31.02423
MPLX	MPLX LP Representing Limited Partner Interests	MPLX LP is a limited partnership company that owns, operates develops and acquires midstream energy infrastructure assets. It gathers, processes and transports natural gas, crude oil, and refined petroleum products.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{PAA,MMP,HEP,NS,MPC,DKL,SXE,PSXP,ANDX}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:07.145676	2020-10-11 23:38:31.021426
MRC	MRC Global Inc.	MRC Global Inc along with its subsidiaries is an industrial distributor of pipe, valves and fittings and related products and services to energy industries. Its other products are pipe, steel fitting, and tubular goods.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{RS,CVX,MPC,KAMN,PSX,DNOW}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:07.172265	2020-10-11 23:38:31.021426
MRNS	Marinus Pharmaceuticals Inc.	Marinus Pharmaceuticals Inc is a clinical stage biopharmaceutical company. It is focused on developing and commercializing innovative therapeutics to treat epilepsy and neuropsychiatric disorders.	{Healthcare,Biotechnology}	{ACRX,IMMY,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:07.197313	2020-10-11 23:38:31.02423
MSA	MSA Safety Incorporated	MSA Safety Inc develops, manufactures & supplies safety products that protect people & facility infrastructure. Its products include fixed gas & flame detection systems, portable gas detection instruments, head protection product & fall protection device.	{Industrials,"Security & Protection Services","Consulting & Outsourcing"}	{MATW,NPO,HI,FIT}	Consulting & Outsourcing	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:07.225683	2020-10-11 23:38:31.023428
MJCO	Majesco	Majesco is a United States based company which provides core insurance software, consulting and services for business transformation for the insurance industry.	{Technology,"Software - Application","Application Software"}	{EBIX,FICO,MITK,GWRE,BNFT,XLK}	Application Software	Technology	NYSE American	\N	2020-10-11 23:12:06.903411	2020-10-11 23:38:31.021426
MLCO	Melco Resorts & Entertainment Limited	Melco Resorts and Entertainment Ltd is developer, owner and operator of casino gaming and entertainment resort facilities in Asia. Its segments are City of Dreams, Altira Macau, Studio City, Mocha Clubs, Corporate and Others, and City of Dreams Manila.	{"Consumer Cyclical","Resorts & Casinos","Travel & Leisure"}	{WYNN,MGM,LVS}	Travel & Leisure	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:06.941921	2020-10-11 23:38:31.023007
MLPY	Morgan Stanley Cushing MLP High Income Index ETN	The investment seeks to track the performance, net of expenses, of the Cushing MLP High Income index.\n The index is a criteria-weighted index tracking the performance of 30 master limited partnerships (MLPs) which hold energy infrastructure and related shipping assets in North America.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.978395	2020-10-11 23:38:31.021426
MMDM	Modern Media Acquisition Corp.	Modern Media Acquisition Corp is a blank check company. The Company was formed for the purpose of effecting a merger, share exchange, asset acquisition, stock purchase, reorganization, recapitalization or other similar business combination.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:07.003052	2020-10-11 23:38:31.023007
MNR	Monmouth Real Estate Investment Corporation Class A	Monmouth Real Estate Investment Corp is engaged in the ownership and management of industrial buildings subject to long-term net leases, mainly to investment grade tenants.	{"Real Estate","REIT - Industrial",REITs}	{DEST,DRE,FR,CLI,UBA,OLP,CBK,NWY,SIR}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:07.068447	2020-10-11 23:38:31.031328
MOFG	MidWestOne Financial Group Inc.	MidWestOne Financial Group Inc is a bank holding company. The company through its subsidiary provides commercial and retail lending services to individuals, businesses, governmental units and institutional customers in central and east-central Iowa.	{"Financial Services","Banks - Regional - US",Banks}	{UMBF,ATLO,WTBA,GFED,CBSH,GSBC,QCRH,EFSC,HWBK,ASB}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:07.092274	2020-10-11 23:38:31.023007
MORT	VanEck Vectors Mortgage REIT Income	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the MVIS® US Mortgage REITs Index (the "Mortgage REITs Index").\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The Mortgage REITs Index may include small-, medium- and large-capitalization companies. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:07.115798	2020-10-11 23:38:31.023428
MPC	Marathon Petroleum Corporation	Marathon Petroleum Corp is engaged in the oil & gas sector. The company is engaged in refining, marketing, retail and transportation of petroleum products in the United States.	{Energy,"Oil & Gas Refining & Marketing","Oil & Gas - Refining & Marketing"}	{VLO,XOM,BP,CVX,HFC,PSX,DK}	Oil & Gas - Refining & Marketing	Energy	New York Stock Exchange	\N	2020-10-11 23:12:07.14522	2020-10-11 23:38:31.023428
MRBK	Meridian Corporation	Meridian Bank is a full-service, state-chartered commercial bank providing personal, business lending and deposit services.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:07.171634	2020-10-11 23:38:31.02582
MRRL	ETRACS Monthly Pay 2xLeveraged Mortgage REIT ETN Series B due October 16 2042	The investment seeks a return linked to the Market Vectors® Global Mortgage REITs Index.\n The ETRACS Monthly Pay 2xLeveraged Mortgage REIT ETN Series B due October 16, 2042 is a series of Monthly Pay 2xLeveraged ETRACS. The index tracks the overall performance of publicly-traded mortgage REITs that derive at least 50% of their revenues from mortgage-related activities. The Securities are senior unsecured debt securities issued by UBS AG. The Securities are two times leveraged with respect to the index, and, as a result, will benefit from two times any positive, but will be exposed to two times any negative, compounded monthly performance of the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:07.199247	2020-10-11 23:38:31.02582
MSD	Morgan Stanley Emerging Markets Debt Fund Inc.	Morgan Stanley Emerg Mkts Debt Fd Inc seek as a primary investment objective to produce high current income and as a secondary objective to seek capital appreciation by investing in a sovereign, quasi-sovereign and corporate debt securities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.24411	2020-10-11 23:38:31.028166
MSM	MSC Industrial Direct Company Inc.	MSC Industrial Direct Co Inc distributes maintenance, repair and operating supplies to an industrial-oriented customer base.	{Industrials,"Industrial Distribution"}	{FAST,DXPE,LAWS,AXE,AIT,WCC,GWW,WAIR,KAMN,HDS}	Industrial Distribution	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:07.268547	2020-10-11 23:38:31.028166
MTBC	Medical Transcription Billing Corp.	Medical Transcription Billing Corp is a healthcare information technology company that provides suite of proprietary web-based solutions, together with related business services to healthcare providers.	{Technology,"Health Information Services","Application Software"}	{PAYX,BCOR,PCTY,XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:07.279622	2020-10-11 23:38:31.028166
MTECU	MTech Acquisition Corp. Unit	MTech Acquisition Corp is a blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:07.294373	2020-10-11 23:38:31.023007
MLPB	ETRACS Alerian MLP Infrastructure Index ETN Series B due April 2 2040	The investment seeks to provide exposure to potential price appreciation in the Alerian MLP Infrastructure Index.\n The ETRACS Alerian MLP Infrastructure Index ETN Series B is senior unsecured debt securities issued by UBS. The index constituents earn the majority of their cash flow from gathering and processing, liquefaction, midstream services, pipeline transportation, rail terminaling and storage of energy commodities.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.95972	2020-10-11 23:38:31.023007
MMDMU	Modern Media Acquisition Corp. Unit	Modern Media Acquisition Corp is a blank check company. The Company was formed for the purpose of effecting a merger, share exchange, asset acquisition, stock purchase, reorganization, recapitalization or other similar business combination.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:07.002761	2020-10-11 23:38:31.024353
MMS	Maximus Inc.	Maximus Inc provides business process services to government health and human services agencies in the United States and to foreign governments.	{Industrials,"Business Services"}	{MGLN,HURN,ICFI,HCKT,SRT,NCI,FCN,UNH,HPQ,XRX}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:07.027185	2020-10-11 23:38:31.025871
MNK	Mallinckrodt plc	Mallinckrodt PLC is a drug manufacturing company that develops, manufactures, markets and distributes specialty pharmaceutical and biopharmaceutical products. It operates in two segments including branded pharmaceuticals and generic drugs.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{GE,JNJ,TEVA,MYL,ZGNX,XLV}	Drug Manufacturers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:07.063347	2020-10-11 23:38:31.023428
MOBL	MobileIron Inc.	MobileIron Inc provides mobile IT platform for enterprises to secure and manage mobile applications, or apps, content and devices.	{Technology,"Software - Application","Application Software"}	{CTXS,SYMC,IBM,GSB,VMW,GOOGL,BB,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:07.086427	2020-10-11 23:38:31.024353
MOTI	VanEck Vectors Morningstar International Moat	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the Morningstar® Global ex-US Moat Focus IndexSM.\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The index is comprised of securities issued by companies that Morningstar, Inc. ("Morningstar") determines have sustainable competitive advantages based on a proprietary methodology that considers quantitative and qualitative factors ("wide and narrow moat companies"). It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:07.126689	2020-10-11 23:38:31.02582
MPO	Midstates Petroleum Company Inc.	Midstates Petroleum Co Inc is in the business of oil and natural gas exploration and production. The company focuses on the application of drilling and completion techniques in oil and liquid rich basins in the United States.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:07.145385	2020-10-11 23:38:31.02582
MRAM	Everspin Technologies Inc.	Everspin Technologies Inc is a provider of magnetoresistive random access memory (MRAM) products. The Company operates its business as one operating segment for purposes of assessing performance and making operating decisions.	{Technology,"Semiconductor Memory",Semiconductors}	{QRVO,QTNA,XLK}	Semiconductors	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:07.169121	2020-10-11 23:38:31.023428
MRO	Marathon Oil Corporation	Marathon Oil Corp is an exploration and production company. It focuses on producing crude oil and condensate, natural gas liquids and natural gas as well as bitumen from oil sands deposits.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{XOM,DVN,APC,MUR,OXY,CVX,COP,RDS.A,HES}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:07.197827	2020-10-11 23:38:31.023428
MS	Morgan Stanley	Morgan Stanley is a global investment bank. The company has institutional securities, wealth-management, and asset-management segments.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{BCS,GS,BAC,DB,JPM,MUFG,C}	Brokers & Exchanges	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.224918	2020-10-11 23:38:31.02423
MSF	Morgan Stanley Emerging Markets Fund Inc.	Morgan Stanley Emerging Markets Fund Inc is United States based diversified, closed-end management investment company. Its investment objective is long-term capital appreciation through investments primarily in emerging country equity securities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.248509	2020-10-11 23:38:31.02423
MSTR	MicroStrategy Incorporated	MicroStrategy Inc is a provider of enterprise-ready analytics, mobile, and security software platforms. The Company provides system-of-record reporting and interactive visualization capabilities offered on any device or in the cloud.	{Technology,"Software - Application","Application Software"}	{MSFT,PEGA,EBAY,DWCH,IBM,TDC,SPLK,DATA,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:07.269342	2020-10-11 23:38:31.021426
MTCH	Match Group Inc.	Match Group Inc primarily offers online dating products and services through over 45 online dating brands, Tinder being the most well known. The company also has a non-dating business which provides test preparation and tutoring services.	{Technology,"Internet Content & Information","Online Media"}	{RENN,XLK}	Online Media	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:07.284194	2020-10-11 23:38:31.028166
MOSC	Mosaic Acquisition Corp. Class A	Mosaic Acquisition Corp is a blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:07.118614	2020-10-11 23:38:31.028166
MPB	Mid Penn Bancorp	Mid Penn Bancorp Inc is a bank holding company, which through its holding provides commercial banking & trust business, making available to the community a range of financial services including mortgage & home equity loans, & commercial & consumer loans.	{"Financial Services","Banks - Regional - US",Banks}	{CZNC,ORRF,NWFL,FRBK,BMTC,CCNE,PWOD,STBA,UVSP,ACNB}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:07.143498	2020-10-11 23:38:31.025871
MQY	Blackrock MuniYield Quality Fund Inc.	Blackrock Muniyield Quality Fund Inc is a closed-end fund. Its objective is to provide shareholders with as high a level of current income exempt from US federal income taxes as is consistent with its investment policies & prudent investment management.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.164484	2020-10-11 23:38:31.02423
MRLN	Marlin Business Services Corp.	Marlin Business Services Corp is a provider of credit products and services to small businesses. The company provide equipment financing programs to manufacturers, distributors and dealers. It derives its revenue from interest and fees from loans.	{"Financial Services","Credit Services"}	{AYR,CIT,TRTN}	Credit Services	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:07.185437	2020-10-11 23:38:31.023007
MRTX	Mirati Therapeutics Inc.	Mirati Therapeutics Inc is a biotechnology company. The company's focuses on precision medicine for oncology therapeutics. The company provides bioavailable inhibitors with distinct target profiles aiming to treat cancer patients.	{Healthcare,Biotechnology}	{HBP,SPHS,NYMX,TA,NVLN,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:07.211832	2020-10-11 23:38:31.031328
MSBF	MSB Financial Corp.	MSB Financial Corp is a holding company for Millington Savings Bank. Its primary business is attracting retail deposits from the general public and using those deposits for lending and investing activities.	{"Financial Services","Savings & Cooperative Banks",Banks}	{OCFC,MGYR,ORIT,PFS,KRNY}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:07.233081	2020-10-11 23:38:31.02582
MSGN	MSG Networks Inc.	MSG Networks Inc is engaged in content development and distribution. It owns and operates two award-winning regional sports and entertainment networks, MSG Network and MSG+.	{"Consumer Cyclical","Media - Diversified",Entertainment}	{CMCSA,DISCA,DIS,CBS,AMCX,VIAB,FOXA}	Entertainment	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:07.253467	2020-10-11 23:38:31.023428
MTB	M&T Bank Corporation	M&T Bank Corp offers banking services. Its reportable segments are Business Banking, Commercial Banking, Commercial Real Estate, Discretionary Portfolio, Residential Mortgage Banking and Retail Banking.	{"Financial Services","Banks - Regional - US",Banks}	{WTFC,SHBI,CBSH,EGBN,NBTB,BAC,PNC,WFC,EVBN,BKU}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.276747	2020-10-11 23:38:31.02582
MTH	Meritage Homes Corporation	Meritage Homes Corp is a designer and builder of single-family homes. The company offer homes that are designed to appeal to a wide range of homebuyers primarily focused on first-time and first move-up buyers. Its operations are spread across USA.	{"Consumer Cyclical","Residential Construction","Homebuilding & Construction"}	{MHO,NVR,DHI,HOV,MDC,KBH,BZH,WLH,LGIH}	Homebuilding & Construction	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:07.302118	2020-10-11 23:38:31.02582
MTL	Mechel PAO American Depositary Shares (Each rep. 1)	Mechel PAO is engaged in production and sale of coal, coke and chemical products, iron ore and limestone, semi-finished steel products and supplies electricity and heat energy to its group companies and external customers.	{"Basic Materials",Steel}	{BBL,RIO,NUE,BHP,X,PKX,MT}	Steel	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:07.31236	2020-10-11 23:38:31.025871
MTSC	MTS Systems Corporation	MTS Systems Corp is a supplier of test systems and position sensors. Its operations are organized and managed in two business segments; the Test segment and the Sensors segment.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{FARO,PRCP,GE,ITW,MOG.A,GEOS,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:07.324503	2020-10-11 23:38:31.02582
MTT	Western Asset Municipal Defined Opportunity Trust Inc	Western Asset Mu Defined Opp Tr Inc is a non-diversified, closed-end management investment company. Its investment objective is to provide high current income exempt from federal income tax and then to liquidate and distribute all of the Fund's.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.339516	2020-10-11 23:38:31.025871
MTZ	MasTec Inc.	MasTec Inc is an infrastructure construction company engaged in engineering, building, installation, maintenance and upgrade of communications, energy and utility infrastructure. The company offers its services under the MasTec service mark.	{Industrials,"Engineering & Construction"}	{MYRG,PRIM,PWR,DY,FLR}	Engineering & Construction	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:07.347758	2020-10-11 23:38:31.028166
MPVD	Mountain Province Diamonds Inc.	Mountain Province Diamonds Inc is engaged in the discovery and development of diamond properties in Canadas Northwest Territory. Its project includes the Gahcho Kue Diamond Mine.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{CMP}	Metals & Mining	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:12:07.148297	2020-10-11 23:38:31.023007
MRCY	Mercury Systems Inc	Mercury Systems Inc is a commercial provider of secure processing subsystems designed and made in the USA for critical defense and intelligence programs. The Company delivers affordable solutions, service and support to defense prime contractors.	{Technology,"Computer Systems","Computer Hardware"}	{INTC,RSYS,ALOT,SYNA,GD,TXT,RTN,DDD,LLL,SMCI,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:07.177753	2020-10-11 23:38:31.028166
MRT	MedEquities Realty Trust Inc.	MedEquities Realty Trust Inc is a self-managed and self-administered company that invests in a diversified mix of healthcare properties and healthcare-related real estate debt investments.	{"Real Estate","REIT - Healthcare Facilities",REITs}	{NYMT,O,NTP}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:07.200312	2020-10-11 23:38:31.021426
MSB	Mesabi Trust	Mesabi Trust operates as a royalty trust in the United States. The company involves in the conservation and protection of the assets held.	{"Financial Services","Asset Management"}	{MARPS,MSM,AIT}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.230943	2020-10-11 23:38:31.023007
MSL	MidSouth Bancorp	Midsouth Bancorp Inc is a financial holding company. The company is a financial holding company. It operates in the community banking business by providing banking services to commercial and retail customers through the Bank.	{"Financial Services","Banks - Regional - US",Banks}	{BANF,FFIN,BOKF,HWC,SFNC,IBKC,CFR,PB}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.259246	2020-10-11 23:38:31.028166
MTBCP	Medical Transcription Billing Corp. 11% Series A Cumulative Redeemable Perpetual Preferred Stock	Medical Transcription Billing Corp is a healthcare information technology company that provides suite of proprietary web-based solutions, together with related business services to healthcare providers.	{Technology,"Health Information Services","Application Software"}	{XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:07.284471	2020-10-11 23:38:31.024353
MTG	MGIC Investment Corporation	MGIC Investment Corp is engaged in the financial services domain. Its core business is the provision of private mortgage insurance and ancillary services. Insurance premiums generate maximum revenue for the company.	{"Financial Services","Insurance - Specialty"}	{RDN,MBI,ORI,HTH,AIG,ESNT,NMIH}	Insurance - Specialty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.301428	2020-10-11 23:38:31.028166
MTN	Vail Resorts Inc.	Vail Resorts Inc is a mountain resort operator. It owns four ski resorts in Colorado, one ski resort in Lake Tahoe, and one summer resort in Grand Teton, Wyoming. Its operating segments are Mountain, Lodging and Real Estate.	{"Consumer Cyclical","Resorts & Casinos","Travel & Leisure"}	{H,FUN,MAR,HLT}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:07.313303	2020-10-11 23:38:31.02423
MTRX	Matrix Service Company	Matrix Service Co provides engineering, fabrication, infrastructure, construction, and maintenance services to the oil, gas, power, petrochemical, industrial, mining and minerals markets.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{GLDD,MYRG,PRIM,EME,OII,TTI,TISI,AROC}	Oil & Gas - Services	Energy	Nasdaq Global Select	\N	2020-10-11 23:12:07.323517	2020-10-11 23:38:31.028166
MTRN	Materion Corporation	Materion Corp is a producer of metal products. It is an integrated producer of high-performance advanced engineered materials used in a variety of electrical, electronic, thermal, and structural applications.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{BRSS,VIAV}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:07.31891	2020-10-11 23:38:31.024353
MTSL	MER Telemanagement Solutions Ltd.	MER Telemanagement Solutions Ltd is engaged in providing services and solutions in video advertising over the internet and mobile devices and in the call accounting and telecom expense management (TEM) markets.	{"Communication Services","Telecom Services"}	{EPAY,NICE,BOSC,MNDO,IBM}	Communication Services	Communication Services	NASDAQ Capital Market	\N	2020-10-11 23:12:07.334975	2020-10-11 23:38:31.02423
MTW	Manitowoc Company Inc. (The)	Manitowoc Co Inc provides services to the farming industry. It manufactures markets and supports product lines of mobile telescopic cranes, tower cranes, crawler cranes and boom trucks.	{Industrials,"Farm & Construction Equipment","Farm & Construction Machinery"}	{CMCO,GENC,MIDD,MNTX,TEX,CAT,DE,DOV}	Farm & Construction Machinery	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:07.345667	2020-10-11 23:38:31.021426
MU	Micron Technology Inc.	Micron Technology Inc along with its subsidiaries provide memory and storage solutions. Its product portfolio consists of memory and storage technologies such as DRAM, NAND, NOR and 3D XPoint memory.	{Technology,"Semiconductor Memory",Semiconductors}	{INTC,RMBS,PLAB,STM,WDC,MOSY,NLST,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:07.347664	2020-10-11 23:38:31.028166
MUB	iShares National Muni Bond	The investment seeks to track the investment results of the S&P National AMT-Free Municipal Bond IndexTM.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The index measures the performance of the investment-grade segment of the U.S. municipal bond market.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:07.356137	2020-10-11 23:38:31.031328
MQT	Blackrock MuniYield Quality Fund II Inc.	Blackrock Muniyield Quality Fund Ii is a closed-end fund. Its objective is to provide shareholders with as high a level of current income exempt from federal income taxes as is consistent with its investment policies and prudent investment management.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.164473	2020-10-11 23:38:31.025871
MRTN	Marten Transport Ltd.	Marten Transport Ltd is a temperature-sensitive truckload carrier in US. It is engaged in transporting and distributing food and other consumer packaged goods which requires a temperature-controlled or insulated environment.	{Industrials,Trucking,"Transportation & Logistics"}	{PTSI,CVTI,HTLD,USAK,WERN,ULH,RRTS,KNX}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:07.207415	2020-10-11 23:38:31.028166
MSON	MISONIX Inc.	Misonix Inc designs, manufactures, markets and develops minimally invasive ultrasonic surgical device products. Its products include the BoneScalpel surgical system, the SonaStar Surgical Aspirator and the SonicOne Wound Cleansing and Debridement System.	{Healthcare,"Medical Devices"}	{IART,ISRG,JNJ,BVX,SYK,MDT,KOOL,AHPI,XLV}	Medical Devices	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:07.271271	2020-10-11 23:38:31.031328
MTEX	Mannatech Incorporated	Mannatech Inc develops and sells nutritional supplements, topical and skin care products, and weight-management products. Its products portfolio includes Ambrotose complex, Ambrotose AO, NutriVerus, and OsoLean, among others.	{"Consumer Defensive","Household & Personal Products","Consumer Packaged Goods"}	{NATR,LCUT,PRPH,NUS,HLF,USNA,NHTC,RELV}	Consumer Packaged Goods	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:07.29615	2020-10-11 23:38:31.021426
MTFB	Motif Bio plc	Motif Bio PLC is a United Kingdom-based pharmaceutical products manufacturer. It is mainly focused on the development of antibiotic drugs to fight bacterial infections such as acute bacterial skin and skin structure infections.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:07.299358	2020-10-11 23:38:31.028166
MTOR	Meritor Inc.	Meritor Inc is a supplier of integrated systems, modules and components to original equipment manufacturers and the aftermarket for the commercial vehicle, transportation and industrial sectors.	{"Consumer Cyclical","Auto Parts",Autos}	{SYPR,CVGI,OSK,AXL,WBC,DAN,SRI,ALSN}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:07.317963	2020-10-11 23:38:31.021426
MTR	Mesa Royalty Trust	Mesa Royalty Trust is a grantor trust. It holds interests in various oil and gas producing properties in the United States.	{"Financial Services","Asset Management"}	{MARPS,PHX,PBT,CRT,NRT,SBR,SJT,TPL}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.319135	2020-10-11 23:38:31.028166
MTX	Minerals Technologies Inc.	Minerals Technologies Inc mines, produces, and sells mineral-based products. The company operates in four segments namely Specialty Minerals, Refractories, Performance Materials, and Energy Services segment.	{"Basic Materials",Chemicals}	{EMN,MLM,FMC,OLN,SCL}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:07.345404	2020-10-11 23:38:31.02582
MUA	Blackrock MuniAssets Fund Inc	Blackrock Muniassets Fund Inc is a diversified investment fund with an objective to provide high current income exempt from U.S. federal income taxes by investing primarily in a portfolio of medium- to lower-grade or unrated municipal obligations.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.351069	2020-10-11 23:38:31.024353
MUJ	Blackrock MuniHoldings New Jersey Quality Fund Inc.	BlackRock MuniHoldings New Jersey Quality Fund Inc is a closed-end management investment company. Its investment objective is to provide shareholders with current income exempt from U.S. federal income tax and New Jersey personal income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.364338	2020-10-11 23:38:31.028166
MUR	Murphy Oil Corporation	Murphy Oil Corp is an oil and gas exploration and production company. The company is engaged in the development of unconventional resources that are situated in the United States, Canada, Malaysia and all other countries.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{TOT,VLO,XOM,DVN,APC,CVX,MRO,COP,HES}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:07.369589	2020-10-11 23:38:31.028166
MUS	Blackrock MuniHoldings Quality Fund Inc	Blackrock Muniholdings Quality Fund Inc is a closed-end fund. Its investment objective is to provide shareholders with current income exempt from federal income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.371943	2020-10-11 23:38:31.023007
MRGR	ProShares Merger	The investment seeks investment results, before fees and expenses, that track the performance of the S&P® Merger Arbitrage Index (the "index").\n The fund is designed to track the performance of the index and provide exposure to a global merger arbitrage strategy. The index, and by extension the fund, seeks to produce consistent, positive returns in virtually all market environments, although there are no assurances it will achieve this result. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:07.182081	2020-10-11 23:38:31.028166
MSCI	MSCI Inc	MSCI Inc is an independent provider of research-driven insights and tools for institutional investors. It offers content, applications and services to support the needs of institutional investors throughout their investment processes.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{TRI,MORN,FDS,MCO,BR,ENV,FOXA,EQIX,SPGI}	Brokers & Exchanges	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.235775	2020-10-11 23:38:31.028166
MT	Arcelor Mittal NY Registry Shares NEW	ArcelorMittal SA is an integrated steel and mining company. The company produces diversified portfolio of iron ore and coal assets as well as steel flat products, including sheet and plate, and long products, including bars, rods and structural shapes.	{"Basic Materials",Steel}	{SID,GGB,CMC,NUE,X,PKX,TX,VALE}	Steel	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:07.273771	2020-10-11 23:38:31.02423
MTEM	Molecular Templates Inc.	Molecular Templates Inc is a clinical-stage biopharmaceutical company. It develops next generation immunotoxins known as Engineered Toxin Bodies for the treatment of individuals living with various cancer.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:07.29534	2020-10-11 23:38:31.024353
MTFBW	Motif Bio plc Warrant		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:07.298475	2020-10-11 23:38:31.031328
MTLS	Materialise NV	Materialise NV is a provider of 3D printing services. The products and services of the group are organized in the three segments Medical, Software and Manufacturing.	{Technology,"Software - Application","Application Software"}	{EVOL,SPSC,RP,VEEV,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:07.311013	2020-10-11 23:38:31.023428
MTP	Midatech Pharma PLC	Midatech Pharma PLC is a nanomedicine company engaged in the development and commercialization of multiple therapeutic products to enhance the delivery of medicines in diseases with high unmet medical needs.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:07.319084	2020-10-11 23:38:31.031328
MUE	Blackrock MuniHoldings Quality Fund II Inc.	Blackrock Muniholdings Ins Fd Ii Inc is a diversified, closed-end management investment company. The Fund's investment objective is to provide total return through a combination of current income and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.35735	2020-10-11 23:38:31.02423
MUH	Blackrock MuniHoldings Fund II Inc.	Blackrock Muniholdings Fund II , Inc is a diversified, closed-end management investment company. The Fund's investment objective is to provide total return through a combination of current income and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.35784	2020-10-11 23:38:31.025871
MUI	Blackrock Muni Intermediate Duration Fund Inc	Blackrock Muni Interm Duration Fund Inc is a diversified investment fund with an investment objective is to provide common shareholders with high current income exempt from U.S. federal income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.359259	2020-10-11 23:38:31.02582
MVBF	MVB Financial Corp.	MVB Financial Corp is a bank holding company. It has three reportable segments: commercial and retail banking; mortgage banking; and financial holding company.	{"Financial Services","Banks - Regional - US",Banks}	{TRST,MFSF,COF,STT,NYCB}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:07.376199	2020-10-11 23:38:31.02582
MVC	MVC Capital Inc.	MVC Capital, Inc., is an externally managed, non-diversified, closed-end management investment company. It provides equity and debt investment capital to fund growth, acquisitions and recapitalizations of small and middle-market companies.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.38004	2020-10-11 23:38:31.031328
MVCD	MVC Capital Inc. 6.25% Senior Notes due 2022		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:07.379238	2020-10-11 23:38:31.025871
MVF	Blackrock MuniVest Fund Inc.	Blackrock Munivest Fund Inc is an investment management firm. It provides investment and risk management services to institutional and retail clients.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.385463	2020-10-11 23:38:31.028166
KOS	Kosmos Energy Ltd.	Kosmos Energy Ltd is an independent oil and gas exploration and production company which focuses on frontier and emerging areas along the Atlantic Margin. It emphases on field developments designed to accelerate production.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{APC,CVX,NBL,HES}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:05.877683	2020-10-11 23:38:31.021426
KRC	Kilroy Realty Corporation	Kilroy Realty Corp operates in the real estate sector. Its main business involves the development and management of real estate assets in the United States.	{"Real Estate","REIT - Office",REITs}	{EQR,DEI,PMT,EQC}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:05.892825	2020-10-11 23:38:31.023007
KSA	iShares Trust MSCI Saudi Arabia	The investment seeks to track the investment results of the MSCI Saudi Arabia IMI 25/50 Index.\n The fund generally will invest at least 90% of its assets in the component securities of the index and in investments that have economic characteristics that are substantially identical to the component securities of the index. The index is a free float-adjusted market capitalization-weighted index with a capping methodology applied to issuer weights so that no single issuer of a component exceeds 25% of the index weight, and all issuers with a weight above 5% do not cumulatively exceed 50% of the index weight. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.909741	2020-10-11 23:38:31.031328
KTN	Structured Products Corp 8.205% CorTS 8.205% Corporate Backed Trust Securities (CorTS)		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:05.931496	2020-10-11 23:38:31.023007
KXI	iShares Global Consumer Staples	The investment seeks to track the S&P Global 1200 Consumer Staples IndexTM.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index. The index measures the performance of companies that the index provider deems to be part of the consumer staples sector of the economy and that the index provider believes are important to global markets.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.948896	2020-10-11 23:38:31.028166
LACQW	Leisure Acquisition Corp. Warrant		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:05.971657	2020-10-11 23:38:31.023007
LANDP	Gladstone Land Corporation 6.375% Series A Cumulative Term Preferred Stock	Gladstone Land Corp is an externally-managed real estate company. The Company invests in farmlands located in agricultural markets throughout the USA. It owns farms and cooling facilities for the storage and assembly of boxes for shipping produce.	{"Real Estate","REIT - Industrial",REITs}	{}	REITs	Real Estate	NASDAQ Global Market	\N	2020-10-11 23:12:05.997511	2020-10-11 23:38:31.031328
LBRDA	Liberty Broadband Corporation Class A Common Stock	Liberty Broadband Corp serves in the telecommunications industry in the United States. It provides cable services to both residential and small to medium businesses through its fiber, hybrid fiber, and coaxial cable infrastructure.	{"Communication Services","Pay TV"}	{BCOM,RNET,HCOM,GTT}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:12:06.020855	2020-10-11 23:38:31.031328
LD	iPath Bloomberg Lead Subindex Total Return SM Index ETN	The investment seeks to provide investors with exposure to the Dow Jones-UBS Lead Subindex Total ReturnService Mark. \nThe Dow Jones-UBS Lead Subindex Total ReturnService Mark (the "index") reflects the returns that are potentially available through an unleveraged investment in the futures contracts on lead. The index currently consists of one futures contract on the commodity of lead which is included in the Dow Jones-UBS Commodity Index Total ReturnService Mark.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.053305	2020-10-11 23:38:31.028166
LECO	Lincoln Electric Holdings Inc.	Lincoln Electric Holdings, Inc., manufactures welding, cutting and brazing products. Its welding product includes arc welding power sources, wire feeding systems, robotic welding packages, fume extraction equipment, consumable electrodes and fluxes.	{Industrials,"Tools & Accessories","Industrial Products"}	{HURC,ITW,KMT,CFX,SWK}	Industrial Products	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:06.074766	2020-10-11 23:38:31.02423
LEVB	Level Brands Inc.	Level Brands Inc is an US based branding and marketing company. The business activity of the group is operated through four division's namely, professional products division, licensing division, entertainment division and health and wellness.	{"Consumer Cyclical","Marketing Services","Advertising & Marketing Services"}	{}	Advertising & Marketing Services	Consumer Cyclical	NYSE American	\N	2020-10-11 23:12:06.097543	2020-10-11 23:38:31.02423
LGCY	Legacy Reserves Inc.	Legacy Reserves LP is an upstream limited partnership which acquires and exploits oil and natural gas properties located in the Permian Basin, Mid-continent, East Texas and Rocky Mountain regions of the United States.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{}	Oil & Gas - E&P	Energy	Nasdaq Global Select	\N	2020-10-11 23:12:06.117841	2020-10-11 23:38:31.023007
LII	Lennox International Inc.	Lennox International Inc through its subsidiaries is a provider of climate control solutions. It designs, manufactures and markets products for heating, ventilation, air conditioning and refrigeration markets.	{Industrials,"Diversified Industrials","Industrial Products"}	{AAON,DOV,LXU,AOS,SXI,UTX,IR,CUO}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:06.132256	2020-10-11 23:38:31.02423
MRSN	Mersana Therapeutics Inc.	Mersana Therapeutics Inc is a clinical stage biopharmaceutical company focused on developing antibody drug conjugates, or ADCs, that offer a clinically meaningful benefit for cancer patients with significant unmet need.	{Healthcare,Biotechnology}	{PDLI,PIRS,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:07.200498	2020-10-11 23:38:31.028166
MXE	Mexico Equity and Income Fund Inc. (The)	Mexico Equity and Income Fund Inc a closed-end, non-diversified management investment company. Its investment objective is to seek long-term capital appreciation through investments in securities, mainly equity, listed on the Mexican Stock Exchange.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.409682	2020-10-11 23:38:31.025871
MXWL	Maxwell Technologies Inc.	Maxwell Technologies Inc develops, manufactures and markets energy storage and power delivery products for transportation, industrial, information technology and other applications and microelectronic products for space and satellite applications.	{Technology,"Electronic Components","Computer Hardware"}	{MU,POWL,ON,AVX,TDY,KEM,TXN,ADI,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:07.423726	2020-10-11 23:38:31.028166
MYND	MYnd Analytics Inc.	MYnd Analytics Inc focuses on developing a decision support tool to help physicians reduce trial and error treatment in mental health and provide more personalized care to patients.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{XLV}	Medical Diagnostics & Research	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:07.45756	2020-10-11 23:38:31.028166
MZA	Blackrock MuniYield Arizona Fund Inc.	Blackrock Muniyield Arizona Fund, Inc is a closed-end management investment company. Its objective is to provide shareholders with as high a level of current income exempt from federal and Arizona income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.472448	2020-10-11 23:38:31.023428
NAII	Natural Alternatives International Inc.	Natural Alternatives International provides contract manufacturing services to companies that market & distribute vitamins, minerals, herbs and other nutritional supplements, as well as other health care products to customers within and outside the U.S.	{"Consumer Defensive","Household & Personal Products","Consumer Packaged Goods"}	{ACET,CYAN,ABT,CBM,MTEX}	Consumer Packaged Goods	Consumer Defensive	NASDAQ Global Market	\N	2020-10-11 23:12:07.486911	2020-10-11 23:38:31.028166
NAP	Navios Maritime Midstream Partners LP representing limited partner interests	Navios Maritime Midstream Partners LP owns and operates very large crude oil tankers under long term employment contracts with international oil companies, refiners and large vessel operators. The company's fleet consists of Vlccs.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{GLNG,TGP,FRO}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:07.505751	2020-10-11 23:38:31.028166
NBB	Nuveen Build America Bond Fund of Beneficial Interest	Nuveen Build America Bond Fund invests in a diversified portfolio of Build America Bonds. Its objective is current income through investments in taxable municipal securities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.542566	2020-10-11 23:38:31.024353
NBO	Neuberger Berman New York Municipal Fund Inc.	Neuberger Berman NY Interm Muni Fund is non-diversified, closed-end fund. Its investment objective is to provide a high level of current income exempt from regular federal income tax and New york city personal income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:07.569545	2020-10-11 23:38:31.024353
NCI	Navigant Consulting Inc.	Navigant Consulting Inc is an independent professional services firm. It provides professional services to clients in the disputes, investigations and economics, Financial Services Advisory and Compliance, healthcare and energy.	{Industrials,"Business Services"}	{RECN,HURN,CRAI,IT,FCN}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:07.591757	2020-10-11 23:38:31.02423
NDRA	ENDRA Life Sciences Inc.	ENDRA Life Sciences Inc develops diagnostics and research products to improve the specificity of clinical ultrasound. Its products allow researchers and clinicians to generate images to identify and cure diseases.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{XLV}	Medical Diagnostics & Research	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:07.622458	2020-10-11 23:38:31.021426
NEON	Neonode Inc.	Neonode Inc develops and licenses user interfaces and optical infrared touch technology. It licenses its multi-touch technology to Original Equipment Manufacturers and Original Design Manufacturers.	{Technology,"Electronic Components","Computer Hardware"}	{XLK}	Computer Hardware	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:07.65862	2020-10-11 23:38:31.025871
NEWR	New Relic Inc.	New Relic Inc is a software-as-a-service provider of software analytics products which allow users to monitor software performance with .NET, Java, JavaScript, Node.js, PHP, Python, and Ruby applications deployed in a cloud or in a data center.	{Technology,"Software - Infrastructure","Application Software"}	{SSNC,SPSC,DATA,CSLT,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:07.709379	2020-10-11 23:38:31.023428
NFO	Invesco Insider Sentiment	The investment seeks to track the investment results (before fees and expenses) of the Nasdaq US Insider Sentiment Index.\n The fund will invest at least 90% of its total assets in securities that comprise the underlying index and depositary receipts representing common stocks included in the underlying index (or underlying securities representing depositary receipts included in the underlying index). The companies eligible for the underlying index are derived from its starting universe, the Nasdaq US Large Mid Cap Index, which is designed to track the performance of mid- to large-capitalization U.S. companies. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:07.733032	2020-10-11 23:38:31.031328
MXL	MaxLinear Inc.	MaxLinear Inc is a semiconductor company providing integrated, radio-frequency and mixed-signal integrated circuits for broadband communications and data center, metro and long-haul transport network applications.	{Technology,Semiconductors}	{INTC,SLAB,NXPI,AAOI,XLK}	Semiconductors	Technology	New York Stock Exchange	\N	2020-10-11 23:12:07.423123	2020-10-11 23:38:31.023007
MYI	Blackrock MuniYield Quality Fund III Inc	Blackrock Muniyield Insured Fund, Inc is a closed ended fund that seeks to provide shareholders with as high a level of current income exempt from U.S. federal income taxes as is consistent with its investment policies and prudent investment management.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.446693	2020-10-11 23:38:31.023007
MYJ	Blackrock MuniYield New Jersey Fund Inc	Blackrock Muniyield New Jersey Fund Inc is a closed end management investment company. The fund's objective is to provide a high a level of current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.446704	2020-10-11 23:38:31.024353
MYOK	MyoKardia Inc.	MyoKardia Inc is a clinical stage biopharmaceutical company which is engaged in discovering, developing and commercializing targeted therapies for the treatment of serious and neglected rare cardiovascular diseases.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:07.46385	2020-10-11 23:38:31.025871
MYSZ	My Size Inc.	My Size Inc focuses on the development of an application which assists the consumer to accurately take the measurements of his or her own body to fit clothing in the best way possible without the need to try them on using a smartphone.	{Technology,"Software - Application","Application Software"}	{RENN,XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:07.469665	2020-10-11 23:38:31.031328
NAC	Nuveen California Quality Municipal Income Fund	Nuveen California Quality Municipal Income Fund is a diversified closed end management investment company. It seeks current income exempt from both regular federal income taxes and California personal income tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.483784	2020-10-11 23:38:31.024353
NANO	Nanometrics Incorporated	Nanometrics Inc along with its subsidiaries provides process control metrology and inspection systems used in the fabrication of integrated circuits, high-brightness light-emitting diode, discrete components and data storage devices.	{Technology,"Semiconductor Equipment & Materials",Semiconductors}	{AMAT,ASYS,COHU,KLAC,NVMI,FORM,AME,TER,RTEC,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:07.50268	2020-10-11 23:38:31.021426
NAOV	NanoVibronix Inc.	NanoVibronix Inc designs noninvasive biological response-activating devices which target wound healing and pain therapy and can be administered at home, without the assistance of medical professionals.	{Healthcare,"Medical Devices"}	{XLV}	Medical Devices	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:07.505556	2020-10-11 23:38:31.024353
NAVB	Navidea Biopharmaceuticals Inc.	Navidea Biopharmaceuticals Inc is a biotechnology company engaged in precision medicine. Its products are designed to assist in the identification of undetected diseases and develop treatment for cancer, autoimmune conditions, and inflammatory diseases.	{Healthcare,Biotechnology}	{IMMU,FONR,AMAG,CAH,GE,LLY,ICAD,XLV}	Biotechnology	Healthcare	NYSE American	\N	2020-10-11 23:12:07.540434	2020-10-11 23:38:31.024353
NAVG	The Navigators Group Inc.	Navigators Group Inc is an international insurance company engaged in marine insurance. The Company also provides property and casualty insurance.	{"Financial Services","Insurance - Property & Casualty"}	{AMSF,Y,WRB,CNA,L,MKL,MTG,RLI,TRV,WTM}	Insurance - Property & Casualty	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:07.541783	2020-10-11 23:38:31.023428
NBLX	Noble Midstream Partners LP Representing Limited Partner Interests	Noble Midstream Partners LP provides crude oil, natural gas, and water-related midstream services through long-term, fixed-fee contracts. The company's operating segments include the Gathering system, Fresh water Delivery and Investment and others.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{PAA,PPL,PSXP,PAGP}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:07.566844	2020-10-11 23:38:31.031328
NBIX	Neurocrine Biosciences Inc.	Neurocrine Biosciences Inc engages in the discovery and development of drugs for the treatment of neurological and endocrine-related diseases and disorders in the United States.	{Healthcare,Biotechnology}	{RGEN,ATHX,LLY,PFE,PTN,ABBV,ANIP,CASI,PRTK,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:07.562885	2020-10-11 23:38:31.021426
NC	NACCO Industries Inc.	NACCO Industries Inc along with its subsidiaries is engaged mining coal primarily for use in power generation and provides value-added services for natural resource companies.	{"Basic Materials",Coal}	{BNSO,MSN,TEX}	Coal	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:07.587142	2020-10-11 23:38:31.023007
NCBS	Nicolet Bankshares Inc.	Nicolet Bankshares Inc is a bank holding company. The bank offers various commercial banking services in Northeastern Wisconsin and the Upper Peninsula of Michigan.	{"Financial Services","Banks - Regional - US",Banks}	{FFIN,NKSH,FBSS}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:07.589653	2020-10-11 23:38:31.02582
MYE	Myers Industries Inc.	Myers Industries Inc manufactures a range of polymer products for industrial, agricultural, automotive, commercial and consumer markets. It also manufactures plastic reusable material handling containers and pallets.	{"Consumer Cyclical","Rubber & Plastics",Autos}	{DSWL,PKOH,CTIB,TRS,CMT,DOV,NPO,BERY}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:07.436199	2020-10-11 23:38:31.021426
MZZ	ProShares UltraShort MidCap400	The investment seeks daily investment results, before fees and expenses, that correspond to two times the inverse (-2x) of the daily performance of the S&P MidCap 400® Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index is a measure of mid-size company U.S. stock market performance. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:07.481749	2020-10-11 23:38:31.028166
NATH	Nathan's Famous Inc.	Nathan's Famous Inc operates fast food franchises. Its brand portfolio includes names like of Nathan's Famous, Arthur Treacher's Fish and Chips, Kenny Rogers Roasters and Miami Subs.	{"Consumer Cyclical",Restaurants}	{JACK,CAKE,CAG,DPZ,CNK,SIX,PBPB}	Restaurants	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:07.50854	2020-10-11 23:38:31.025871
NAVI	Navient Corporation	Navient Corp is a financial service provider that offers business processing services to education, healthcare, and government clients at the federal, state, and local levels. The company also provides asset management services.	{"Financial Services","Credit Services"}	{NNI,SLM}	Credit Services	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:07.542079	2020-10-11 23:38:31.028166
NBRV	Nabriva Therapeutics plc	Nabriva Therapeutics AG is engaged in the research and development of novel anti-infective agents to treat serious infections, with a focus on the pleuromutilin class of antibiotics.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:07.569437	2020-10-11 23:38:31.028166
NCS	NCI Building Systems Inc.	NCI Building Systems Inc is an integrated manufacturer and marketer of metal products for the nonresidential construction industry. The Company's products are used in repair, retrofit and new construction activities in North America.	{Industrials,"Metal Fabrication","Industrial Products"}	{APOG,ROCK,AIN,MAS,NUE,AWI,PGTI}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:07.607966	2020-10-11 23:38:31.028166
NCSM	NCS Multistage Holdings Inc.	NCS Multistage Holdings Inc provides engineered products and support services that facilitates the optimization of oil and natural gas well completions and field development strategies.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{PUMP}	Oil & Gas - Services	Energy	Nasdaq Global Select	\N	2020-10-11 23:12:07.612908	2020-10-11 23:38:31.028166
NDRAW	ENDRA Life Sciences Inc. Warrants		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:07.633482	2020-10-11 23:38:31.025871
NDSN	Nordson Corporation	Nordson Corp engineers, manufactures and markets differentiated products and systems used for adhesive, coating, sealant and biomaterial dispensing, fluid management, testing and inspection, curing, and surface treatment.	{Industrials,"Diversified Industrials","Industrial Products"}	{ASTE,TAYD,DOV,GGG,ITW}	Industrial Products	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:07.63335	2020-10-11 23:38:31.023007
NEOG	Neogen Corporation	Neogen Corp, through its subsidiaries, develops, manufactures and markets a line of products dedicated to food and animal safety.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{IDXX,ILMN,VIVO,BDX,DHR,TMO,PKI,PACB,BIO.B,XLV}	Medical Diagnostics & Research	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:07.65974	2020-10-11 23:38:31.021426
NEPT	Neptune Wellness Solutions Inc.	Neptune Technologies & Bioressources Inc is a nutrition products company engaged in customizing nutrition solutions, specialty ingredients and consumer brands. It offers premium krill oil under OCEANO3 and other ingredients such as marine and seed oils.	{Healthcare,Biotechnology,"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{}	Consumer Packaged Goods	Consumer Defensive	NASDAQ Capital Market	\N	2020-10-11 23:12:07.671317	2020-10-11 23:38:31.023428
NETS	Netshoes (Cayman) Limited	Netshoes (Cayman) Ltd is a holding company. The Company through its subsidiaries offers sports and lifestyle products athletic shoes, jerseys, apparel, accessories & sporting equipment from international, local and private brands as well as fashion.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{HIBB,SPWH,NETS}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:07.699859	2020-10-11 23:38:31.021426
NEWM	New Media Investment Group Inc.	New Media Investment Group Inc owns, operates and invests in local media assets. The company focuses on two categories namely consumers and small to medium-sized businesses.	{"Consumer Cyclical",Publishing}	{DJCO,LEE,NYT,AHC,NWSA,TGNA,MNI,SSP}	Publishing	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:07.70929	2020-10-11 23:38:31.028166
NFG	National Fuel Gas Company	National Fuel Gas Co is a diversified energy company. It is engaged in the production, gathering, transportation, distribution and marketing of natural gas.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{OKE,ATO,NJR,SJI,SWX,UGI,CVX,SR}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:07.720032	2020-10-11 23:38:31.028166
NFLX	Netflix Inc.	Netflix Inc provides multimedia content through the internet. It provides a subscription based platform for television shows, movies and original series.	{"Consumer Cyclical","Media - Diversified",Entertainment}	{BCE,AMZN,CMCSA,DISH,DIS,P,VIAB,FOXA}	Entertainment	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:07.726301	2020-10-11 23:38:31.028166
MYC	Blackrock MuniYield California Fund Inc.	Blackrock Muniyield California Fund, Inc is a non-diversified, closed-end management investment company. The fund's investment objective is to provide shareholders with as high a level of current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.426306	2020-10-11 23:38:31.028166
MYN	Blackrock MuniYield New York Quality Fund Inc.Common Stock	Blackrock Muniyield NY Insured FD is a closed end management investment company. Its objective is to provide shareholders with a high level of current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.450829	2020-10-11 23:38:31.023428
MYRG	MYR Group Inc.	MYR Group Inc provides specialty electrical construction services. It offers design, engineering, procurement, construction, upgrade, maintenance and repair services on transmission and distribution network.	{Industrials,"Engineering & Construction"}	{MTRX,TTEK,ENG,PRIM,MTZ,PWR,DY}	Engineering & Construction	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:07.470562	2020-10-11 23:38:31.02423
NAD	Nuveen Quality Municipal Income Fund	Nuveen Quality Municipal Income Fund is a diversified, closed end management investment company. Its investment objective is to provide current income by investing in a portfolio of municipal obligations issued by state and local government authorities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.486335	2020-10-11 23:38:31.023007
NAT	Nordic American Tankers Limited	Nordic American Tankers Ltd is an American shipping company. It operate as an independent Suezmax tanker owner and operator. It offers services through various routes worldwide.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{GLNG,CPLP,TK,SFL,TGP,TOO,DHT,ASC,FRO,GOGL}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:07.507955	2020-10-11 23:38:31.023007
NAZ	Nuveen Arizona Quality Municipal Income Fund	Nuveen Arizona Quality Municipal Income Fund is a closed-end investment company. The fund's investment objective is to provide current income exempt from both regular federal and designated state income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.541935	2020-10-11 23:38:31.023007
NCLH	Norwegian Cruise Line Holdings Ltd.	Norwegian Cruise Line Holdings Ltd is a cruise company which operates the Norwegian, Oceania Cruises and Regent brands. It offers accommodations, multiple dining venues, bars and lounges, spa, casino and retail shopping areas.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{CKH,RCL,CUK,CCL}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:07.596966	2020-10-11 23:38:31.028166
NCTY	The9 Limited	The9 Ltd is an online game operator and developer in China. The company develops and operates online games, including web games, social games and mobile games.	{Technology,"Electronic Gaming & Multimedia","Application Software"}	{XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:07.616542	2020-10-11 23:38:31.02582
NDLS	Noodles & Company	Noodles & Co is a fast casual restaurant concept offering lunch and dinner. Its menu includes a variety of cooked-to-order dishes, including noodles and pasta, soups, salads, sandwiches, and appetizers.	{"Consumer Cyclical",Restaurants}	{ARKR,BJRI,KONA,EAT,CHUY,BLMN}	Restaurants	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:07.621336	2020-10-11 23:38:31.028166
NEA	Nuveen AMT-Free Quality Municipal Income Fund of Beneficial Interest Par Value $.01	Nuveen AMT-Free Municipal Income Fund is a diversified closed-end management investment company. Its investment objective is to provide current income exempt from regular federal income tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.637618	2020-10-11 23:38:31.031328
NEOS	Neos Therapeutics Inc.	Neos Therapeutics Inc is a pharmaceutical company, engaged in the development, manufacture, and commercialization of products for the treatment of attention deficit hyperactivity disorder using its drug delivery technologies.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{OMER,XLV}	Drug Manufacturers	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:07.665562	2020-10-11 23:38:31.031328
NEP	NextEra Energy Partners LP representing limited partner interests	NextEra Energy Partners LP is a limited partnership formed to acquire, manage and own contracted clean energy projects with stable long-term cash flows. The renewable energy sales generate maximum revenue for the company.	{Utilities,"Utilities - Independent Power Producers"}	{ORA,PEGI,XLU}	Utilities - Independent Power Producers	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:07.665768	2020-10-11 23:38:31.024353
NETE	Net Element Inc.	Net Element Inc is a transaction processing technology and value-added solutions company. Its segments include Transaction Solutions, Mobile Payment Solutions, Online Payment Solutions, and Corporate Overhead.	{Technology,"Internet Content & Information","Online Media"}	{PCYG,XLK}	Online Media	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:07.695714	2020-10-11 23:38:31.025871
NEXTW	NextDecade Corporation - Warrants (13/02/2022)		{}	{}			OTC Pink	\N	2020-10-11 23:12:07.716667	2020-10-11 23:38:31.025871
MYL	Mylan N.V.	Mylan NV is a generic pharmaceutical manufacturer which develops, licenses, manufactures, markets and distributes generic, over-the-counter (OTC) products in a variety of dosage forms and therapeutic categories.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{LCI,PFE,RDY,SNY,TEVA,XLV}	Drug Manufacturers	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:07.450446	2020-10-11 23:38:31.028166
MYY	ProShares Short MidCap400	The investment seeks daily investment results that correspond to the inverse (-1x) of the daily performance of the S&P MidCap 400®.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index is a measure of mid-size company U.S. stock market performance. It is a float-adjusted, market capitalization-weighted index of 400 U.S. operating companies and real estate investment trusts selected through a process that assesses criteria such as liquidity, price, market capitalization and financial viability. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:07.470739	2020-10-11 23:38:31.024353
NAIL	Direxion Daily Homebuilders & Supplies Bull 3X Shares	The investment seeks daily investment results of 300% of the daily performance of the Dow Jones U.S. Select Home Construction Index.\n The fund invests at least 80% of its net assets in securities of the index, ETFs that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index measures U.S. companies in the home construction sector that provide a wide range of products and services related to homebuilding, including home construction and producers, sellers and suppliers of building materials, furnishings and fixtures and also home improvement retailers. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:07.48755	2020-10-11 23:38:31.025871
NATR	Nature's Sunshine Products Inc.	Natures Sunshine Products Inc is a natural health and wellness company. The company together with its subsidiaries is engaged in the manufacturing and direct selling of nutritional and personal care products.	{"Consumer Defensive","Household & Personal Products","Consumer Packaged Goods"}	{PRPH,NUS,HLF,USNA,MTEX,RELV}	Consumer Packaged Goods	Consumer Defensive	NASDAQ Capital Market	\N	2020-10-11 23:12:07.520237	2020-10-11 23:38:31.031328
NBHC	National Bank Holdings Corporation	National Bank Holdings Corp is a bank holding company. It offers a full range of traditional banking products and financial services to both commercial and consumer clients through its subsidiary NBH Bank, N.A.	{"Financial Services","Banks - Regional - US",Banks}	{UMBF,CBSH,GBCI,BBVA,JPM,FRC,C,HBMD,GBNK}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.550949	2020-10-11 23:38:31.023007
NBTB	NBT Bancorp Inc.	NBT Bancorp Inc provides financial services. The company, through community banks, mainly provides consumer loans, home equity loans, mortgages, business banking loans, and commercial loans.	{"Financial Services","Banks - Regional - US",Banks}	{BDGE,FISI,AROW,FLIC,SBNY,KEY,TMP,CBU,EVBN}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:07.569957	2020-10-11 23:38:31.025871
NCMI	National CineMedia Inc.	National CineMedia Inc is a holding company. The company through its subsidiary, operates digital in-theatre media network in North America, through which it sells in-theatre and online advertising and promotions.	{"Consumer Cyclical","Marketing Services","Advertising & Marketing Services"}	{MSFT,CMCSA,IPG,OMC,CBS,FOXA,LAMR}	Advertising & Marketing Services	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:07.597001	2020-10-11 23:38:31.023007
NDAQ	Nasdaq Inc.	Nasdaq Inc is a provider of trading, clearing, exchange technology, regulatory, securities listing, information and public company services.	{"Financial Services","Financial Exchanges","Brokers & Exchanges"}	{MKTX,CME,CBOE}	Brokers & Exchanges	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:07.619903	2020-10-11 23:38:31.031328
NEBUU	Nebula Acquisition Corporation Unit	Nebula Acquisition Corp is a blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:07.639859	2020-10-11 23:38:31.024353
NERV	Minerva Neurosciences Inc	Minerva Neurosciences Inc is a clinical-stage biopharmaceutical company. It is engaged in the development and commercialization of a portfolio of product candidates to treat patients suffering from central nervous system, or CNS, diseases.	{Healthcare,Biotechnology}	{CLVS,DVAX,NBY,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:07.670209	2020-10-11 23:38:31.02582
NEU	NewMarket Corp	NewMarket Corp manufactures and sells petroleum additives used in lubricating oils and fuels to enhance their performance in machinery, vehicles, and other equipment. The petroleum additives market has two products: lubricant additives and fuel additives.	{"Basic Materials","Specialty Chemicals",Chemicals}	{IOSP,XOM,CVX,RDS.A,TREC}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:07.700387	2020-10-11 23:38:31.023007
NEWA	Newater Technology Inc.	Newater Technology Inc engages in wastewater management. The company manufactures and sells filtration products that are used to treat, recycle, and discharge wastewater. It also offers traditional treatment solutions, such as activated carbon and resins.	{Industrials,"Waste Management"}	{}	Waste Management	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:07.704287	2020-10-11 23:38:31.02582
NFBK	Northfield Bancorp Inc.	Northfield Bancorp Inc through its subsidiary, provides banking services to individuals and corporate customers in Richmond and Kings Counties in New York, and Hunterdon, Mercer, Union and Middlesex counties in New Jersey.	{"Financial Services","Banks - Regional - US",Banks}	{FFIC,DCOM,GCBC}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:07.719641	2020-10-11 23:38:31.023007
MYOV	Myovant Sciences Ltd.	Myovant Sciences Ltd is a clinical-stage biopharmaceutical company. The Company is focused on developing and commercializing therapies for women's health diseases and other endocrine-related disorders. Its product candidate is relugolix.	{Healthcare,Biotechnology}	{PDLI,PIRS,XLV}	Biotechnology	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:07.46669	2020-10-11 23:38:31.028166
NAK	Northern Dynasty Minerals Ltd.	Northern Dynasty Minerals Ltd is a mineral exploration company. It is engaged in developing the Pebble Project, a copper-gold-molybdenum mineral project located in Alaska, United States of America.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{NG,SA,AUY,NGD,AAU,EGI}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:07.49029	2020-10-11 23:38:31.02582
NATI	National Instruments Corporation	National Instruments Corp designs, manufactures & sells computer-based virtual instrumentation software & hardware products to scientists and engineers. Its products include embedded control and monitoring hardware, NI LabVIEW, and NI LabWindows/CVI.	{Technology,"Software - Application","Application Software"}	{ANSS,SNPS,AME,A,DHR,TER,WTT,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:07.513976	2020-10-11 23:38:31.02423
NBD	Nuveen Build America Bond Opportunity Fund of Beneficial Interest	Nuveen Build America Oppurtunity Fund is a closed-end management investment company. Its investment objective is to provide current income through investments in taxable municipal securities by investing in a portfolio of taxable municipal securities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.543183	2020-10-11 23:38:31.02423
NBY	NovaBay Pharmaceuticals Inc.	NovaBay Pharmaceuticals Inc is a biopharmaceutical company engaged in commercializing and developing non-antibiotic anti-infective products to address the unmet therapeutic needs of the eye care market. Its product categories include NEUTROX and CELLERX.	{Healthcare,Biotechnology}	{SIGA,JNJ,TTPH,NNVC,SNOA,MLNT,XLV}	Biotechnology	Healthcare	NYSE American	\N	2020-10-11 23:12:07.579213	2020-10-11 23:38:31.021426
NCR	NCR Corporation	NCR Corp is a technology company, which provides products and services, which enable businesses to connect, interact and transact with their customers.	{Technology,"Information Technology Services","Application Software"}	{ZBRA,AGYS,USAT,DBD,PAR,HPQ,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:07.602007	2020-10-11 23:38:31.021426
NESR	National Energy Services Reunited Corp.	National Energy Services Reunited Corp is a newly-organized blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:07.683513	2020-10-11 23:38:31.028166
NEWT	Newtek Business Services Corp.	Newtek Business Services Corp provides business services. Its services include financial, payroll, payment, technology and insurance solutions. It is also involved in investments in early stage companies.	{"Financial Services","Asset Management"}	{CBZ,RLGY,RMAX,ARES}	Asset Management	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:07.708924	2020-10-11 23:38:31.024353
NFEC	NF Energy Saving Corporation	NF Energy Saving Corp is engaged in the provision of energy saving technology consulting, optimization design services, energy saving reconstruction of pipeline networks and contractual energy management services.	{Industrials,"Diversified Industrials","Industrial Products"}	{PH,CIR,MWA}	Industrial Products	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:07.719761	2020-10-11 23:38:31.021426
NFRA	FlexShares STOXX Global Broad Infrastructure Index Fund	The investment seeks investment results that correspond generally to the price and yield performance, before fees and expenses, of the STOXX® Global Broad Infrastructure Index.\n The fund will invest at least 80% of its total assets (exclusive of collateral held from securities lending) in the securities of the underlying index and in ADRs and GDRs based on the securities in the underlying index. The underlying index reflects the performance of a selection of equity securities of infrastructure-related companies that are domiciled or traded in developed and emerging markets around the world (including the U.S.). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:07.734414	2020-10-11 23:38:31.024353
NGVC	Natural Grocers by Vitamin Cottage Inc.	Natural Grocers by Vitamin Cottage Inc is a specialty retailer of natural, organic groceries and dietary supplements.	{"Consumer Defensive","Grocery Stores","Retail - Defensive"}	{UNFI,KR,WMT,DIT,VSI,SFM}	Retail - Defensive	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:07.765377	2020-10-11 23:38:31.021426
NGVT	Ingevity Corporation	Ingevity Corp manufactures and sells chemical products mainly in North America. Its business primarily depends on the sale of specialty chemicals for use in applications like asphalt paving, agrochemicals, and adhesives.	{"Basic Materials","Specialty Chemicals",Chemicals}	{INDU,NL,NGVT}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:07.769415	2020-10-11 23:38:31.025871
NHS	Neuberger Berman High Yield Strategies Fund	Neuberger Berman High Yield Stra Fd is a closed-end management investment company. Its investment objective is to seek high total return through investment in the portfolio that consists primarily of high yield debt securities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:07.786868	2020-10-11 23:38:31.02423
MYNDW	MYnd Analytics Inc. Warrant		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:07.462107	2020-10-11 23:38:31.021426
MZOR	Mazor Robotics Ltd.	Mazor Robotics Ltd is engaged in the development, production and marketing of medical devices for supporting surgical procedures in the field of orthopedics and neurosurgery.	{Healthcare,"Medical Devices"}	{CNMD,ISRG,SYK,ZBH,GMED,MDT,XLV}	Medical Devices	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:07.481371	2020-10-11 23:38:31.021426
NANR	SPDR S&P North American Natural Resources	The investment seeks to track the performance of the S&P BMI North American Natural Resources Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index comprises publicly traded large- and mid-capitalization U.S. and Canadian companies in the natural resources and commodities businesses that meet certain investability requirements and are classified within the sub-industries of one of three natural resources categories: energy, metals & mining or agriculture. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:07.505556	2020-10-11 23:38:31.031328
NBEV	New Age Beverages Corporation	New Age Beverages Corp is engaged in positioning itself as a healthy beverage company integrating its búcha Live Kombucha business, strengthening the businesses foundation and processes.	{"Consumer Defensive","Beverages - Soft Drinks","Beverages - Non-Alcoholic"}	{BREW,TAP.A}	Beverages - Non-Alcoholic	Consumer Defensive	NASDAQ Capital Market	\N	2020-10-11 23:12:07.541763	2020-10-11 23:38:31.023428
NBW	Neuberger Berman California Municipal Fund Inc	Neuberger Berman CA Interm Muni Fund is a closed-end management investment company that invests in municipal securities. Its objective is to provide common stockholders with a high level of current income exempt from federal income tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:07.571699	2020-10-11 23:38:31.031328
NCOM	National Commerce Corporation	National Commerce Corp is a bank holding company. The company and its subsidiary provide banking services and also factoring services.	{"Financial Services","Banks - Regional - US",Banks}	{HTLF,STBA,BPOP}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:07.597258	2020-10-11 23:38:31.025871
NDP	Tortoise Energy Independence Fund Inc.	Tortoise Energy Independence Fund, Inc. is a non-diversified closed-end management investment company. The company's primary investment objective to seek a high level of total return with an emphasis on current distributions.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.621945	2020-10-11 23:38:31.02423
NEM	Newmont Mining Corporation	Newmont Mining Corp is engaged in the production of gold in the United States, Australia, Peru, Ghana and Suriname. It is also engaged in the production of copper, principally through Boddington in Australia and Phoenix in the United States.	{"Basic Materials",Gold,"Metals & Mining"}	{ABX,AEM,RGLD,GG,FCX,BVN,GFI,AU,SBGL,AA}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:07.64448	2020-10-11 23:38:31.02423
NES	Nuverra Environmental Solutions Inc.	Nuverra Environmental Solutions Inc provides full-cycle environmental solutions to consumers focused on the development and ongoing production of oil and natural gas from shale formations in the United States.	{Industrials,"Waste Management"}	{PESI}	Waste Management	Industrials	NYSE American	\N	2020-10-11 23:12:07.681917	2020-10-11 23:38:31.028166
NEXT	NextDecade Corporation	NextDecade Corp develops land-based and floating LNG projects, creating opportunities in the global integrated natural gas industry.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{NEXT}	Oil & Gas - E&P	Energy	NASDAQ Capital Market	\N	2020-10-11 23:12:07.712428	2020-10-11 23:38:31.028166
NFX	Newfield Exploration Company	Newfield Exploration Co is an exploration and production company. It explores, develops and produces crude oil, natural gas and natural gas liquids. Its areas of operation include basins of Oklahoma and North Dakota and uinta basin of Utah.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{CHK,DNR,DVN,APC,SWN,NBL,XEC,QEP,HK}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:07.737626	2020-10-11 23:38:31.021426
NGHCP	National General Holdings Corp 7.50% Non-Cumulative Preferred Stock Series A	National General Holdings Corp is a specialty personal lines insurance holding company. The company manages its business through two segments, Property and Casualty segment and Accident and Health segment.	{"Financial Services","Insurance - Property & Casualty"}	{}	Insurance - Property & Casualty	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:07.755618	2020-10-11 23:38:31.031328
NHI	National Health Investors Inc.	National Health Investors Inc is a healthcare facility REIT. It invests in various healthcare facilities, including independent living facilities, senior-living campuses, and medical office buildings.	{"Real Estate","REIT - Healthcare Facilities",REITs}	{BFS,CBL,NHC}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:07.786615	2020-10-11 23:38:31.024353
NHF	NexPoint Strategic Opportunities Fund	NexPoint Credit Strategies Fund is a non-diversified, closed-end management investment company. Its investment objective is to provide both current income and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.781516	2020-10-11 23:38:31.031328
NAKD	Naked Brand Group Limited		{"Consumer Cyclical","Apparel Manufacturing","Manufacturing - Apparel & Furniture"}	{}	Manufacturing - Apparel & Furniture	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:07.495579	2020-10-11 23:38:31.023428
NAUH	National American University Holdings Inc.	National American University Holdings Inc is a provider of postsecondary education for the needs of working adults and other non-traditional students. It also operates a real estate business known as Fairway Hills Developments, or Fairway Hills.	{"Consumer Defensive","Education & Training Services",Education}	{CECO,LINC,APEI,LOPE,BPI,ATGE}	Education	Consumer Defensive	NASDAQ Global Market	\N	2020-10-11 23:12:07.528602	2020-10-11 23:38:31.021426
NBH	Neuberger Berman Municipal Fund Inc.	Neuberger Berman Intermediate Muni Fund is a closed-end management investment company that invests mainly in municipal securities. Its investment objective is to provide common stockholders a high level of current income exempt from Federal income tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:07.550844	2020-10-11 23:38:31.02582
NBN	Northeast Bancorp	Northeast Bancorp is a bank holding company of Northeast Bank. It provides online banking, mobile banking, business services and financing options.	{"Financial Services","Banks - Regional - US",Banks}	{CAC,EBTC,UNB,BPFH,CNBKA,INDB,BRKL,FNLC,BHB,WNEB}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:07.568469	2020-10-11 23:38:31.028166
NCA	Nuveen California Municipal Value Fund Inc.	Nuveen California Municipal Value Fund Inc is a diversified closed end management investment company. Its objective is to provide current income exempt from both regular federal and California state income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.588034	2020-10-11 23:38:31.023428
NCV	AllianzGI Convertible & Income Fund	AllianzGI Convertible & Income Fund is a diversified, closed end management investment company. Its investment objective is to provide total return through a combination of capital appreciation and high current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.620567	2020-10-11 23:38:31.023428
NE	Noble Corporation (UK)	Noble Corp PLC is an offshore drilling contractor for the oil and gas industry. It performs contract drilling services with its fleet of mobile offshore drilling units.	{Energy,"Oil & Gas Drilling","Oil & Gas - Drilling"}	{DO,HP,NBR,RIG}	Oil & Gas - Drilling	Energy	New York Stock Exchange	\N	2020-10-11 23:12:07.635657	2020-10-11 23:38:31.02582
NEV	Nuveen Enhanced Municipal Value Fund of Beneficial Interest	Nuveen Enhanced Municipal Value Fund is a diversified closed-end management investment company. Its investment objective is to provide current income by investing in a portfolio of municipal obligations issued by state and local government authorities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.704204	2020-10-11 23:38:31.031328
NFLT	Virtus Newfleet Multi-Sector Bond	The investment seeks to provide a high level of current income and, secondarily, capital appreciation.\n Under normal market conditions, the fund will invest not less than 80% of its net assets (plus the amount of any borrowings for investment purposes) in bonds. The Sub-Adviser seeks to select securities using a sector rotation approach and seeks to adjust the proportion of fund investments in various sectors and sub-sectors in an effort to obtain higher relative returns.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:07.723679	2020-10-11 23:38:31.023428
NG	Novagold Resources Inc.	Novagold Resources Inc is a mineral exploration company operating in the gold mining industry, focused on advancing permitting of the Donlin Gold project in Alaska. The Company also has interest in the Galore Creek project in British Columbia, Canada.	{"Basic Materials",Gold,"Metals & Mining"}	{ABX,TECK,SA,GG}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:07.738343	2020-10-11 23:38:31.023007
NGHCO	National General Holdings Corp Depositary Shares	National General Holdings Corp is a specialty personal lines insurance holding company. The company manages its business through two segments, Property and Casualty segment and Accident and Health segment.	{"Financial Services","Insurance - Property & Casualty"}	{}	Insurance - Property & Casualty	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:07.747444	2020-10-11 23:38:31.023428
NH	NantHealth Inc.	NantHealth Inc is a personalize healthcare company enabling improved patient outcomes and treatment decisions for critical illnesses. Its system infrastructure includes software, middleware and hardware modules.	{Technology,"Health Information Services","Application Software"}	{PFPT,PFM,PCYG,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:07.7719	2020-10-11 23:38:31.028166
NHLD	National Holdings Corporation	National Holdings Corp provides financial services. It offers, through its broker-dealer and investment advisory subsidiaries, investment banking, merger and acquisition and advisory services, trading in securities, and full service retail brokerage.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{RJF,SCHW,LAZ,EVR}	Brokers & Exchanges	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:07.783777	2020-10-11 23:38:31.023428
NWBI	Northwest Bancshares Inc.	Northwest Bancshares Inc is a stock savings bank and community-oriented financial institution. It operates banks in Pennsylvania, New York, Ohio and Maryland offering personal and business deposit and loan products.	{"Financial Services","Savings & Cooperative Banks",Banks}	{WVFC,NYCB,BNCL}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:08.139677	2020-10-11 23:38:31.02582
NHLDW	National Holdings Corporation Warrants		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:07.78551	2020-10-11 23:38:31.025871
NIE	AllianzGI Equity & Convertible Income Fund	AllianzGI Equity & Convertible Income Fund is a diversified, closed-end management investment company. Its investment objective is to seek total return comprised of capital appreciation, current income and gains.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.803392	2020-10-11 23:38:31.023428
NK	NantKwest Inc.	NantKwest Inc is a biotechnology firm operating mainly in the United States. It undertakes research and discovery of drugs to cure virally-induced cancers, bacterial and fungal diseases. Its pipeline product includes taNK, haNK, and aNK.	{Healthcare,Biotechnology}	{MNTA,MLNT,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:07.819483	2020-10-11 23:38:31.02423
NL	NL Industries Inc.	NL Industries Inc is a holding company and operates in the component products industry through its majority owned subsidiary, CompX International Inc. The Company operates in the chemicals industry through its interest in Kronos Worldwide, Inc.	{Industrials,"Industrial Distribution"}	{SYNL,NNBR,TKR,VHI,HUN,CIX,KRO,EML,ALLE}	Industrial Distribution	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:07.834425	2020-10-11 23:38:31.028166
NLY	Annaly Capital Management Inc	Annaly Capital Management Inc is a part of the real estate sector. As a mortgage real estate investment trust, it owns a portfolio of real estate related investments, including mortgage pass-through certificates and collateralized mortgage obligations.	{"Real Estate","REIT - Diversified",REITs}	{NYMT,RWT,DX,IVR,PMT,CIM}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:07.849631	2020-10-11 23:38:31.024353
NNN	National Retail Properties	National Retail Properties Inc is a real estate investment trust that invests in and develops properties throughout the United States. Most of the company's properties are located in the Midwest, South, and Southeast of the United States.	{"Real Estate","REIT - Retail",REITs}	{EGP,WY,HCP,EPR}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:07.885919	2020-10-11 23:38:31.023428
NODK	NI Holdings Inc.	NI Holdings Inc is a stock holding company of Nodak Mutual Insurance company and its subsidiaries. The firm and its subsidiaries provide insurance related to auto, home, life and annuities, business, farm and ranch and crop and casualty.	{"Financial Services","Insurance - Property & Casualty"}	{}	Insurance - Property & Casualty	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:07.901906	2020-10-11 23:38:31.021426
NOW	ServiceNow Inc.	ServiceNow Inc provides cloud-based solutions that define, structure, manage, and automate services for enterprise operations in North America, Europe, the Middle East, Africa, the Asia Pacific, and other countries.	{Technology,"Software - Application","Application Software"}	{MSFT,QCOM,RHT,FICO,IBM,ZEN,HPQ,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:07.924218	2020-10-11 23:38:31.02423
NRIM	Northrim BanCorp Inc	Northrim BanCorp Inc is a bank holding company for Northrim Bank. It offers commercial and consumer loan and deposit products, investment products, and electronic banking services over the Internet.	{"Financial Services","Banks - Regional - US",Banks}	{UMPQ,CATY,CVBF,HTBK,EWBC,PLBC,PPBI,KEY,WFC}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:07.949821	2020-10-11 23:38:31.02423
NSC	Norfolk Southern Corporation	Norfolk Southern Corp conducts rail transportation business. It transports raw materials, intermediate products, and finished goods in the United States.	{Industrials,Railroads,"Transportation & Logistics"}	{CNI,CP,GWR,UNP,KSU,CSX}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:07.96777	2020-10-11 23:38:31.024353
NTAP	NetApp Inc.	NetApp Inc provides technology solutions. It is engaged in the design, manufacturing, marketing, and technical support of high-performance storage and data management solutions. Its core product offering include data storage devices such as flash drives.	{Technology,"Data Storage","Computer Hardware"}	{STX,TDC,WDC,HPQ,QTM,SMCI,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:07.992055	2020-10-11 23:38:31.031328
NTLA	Intellia Therapeutics Inc.	Intellia Therapeutics Inc is a gene editing company focused on the development of proprietary, potentially curative therapeutics utilizing a biological tool known as the CRISPR/Cas9 system.	{Healthcare,Biotechnology}	{PDLI,PIRS,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:08.011876	2020-10-11 23:38:31.028166
NTRSP	Northern Trust Corporation Depository Shares	Northern Trust Corp is a provider of asset servicing, fund administration, asset management, fiduciary and banking solutions for corporations, institutions, families and individuals.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:08.030904	2020-10-11 23:38:31.02582
NUEM	NuShares ESG Emerging Markets Equity	The investment seeks to track the investment results, before fees and expenses, of the TIAA ESG Emerging Markets Equity Index (the "index").\n In seeking to track the investment results of the index, the advisor attempts to replicate the index by investing all, or substantially all, of its assets in the securities represented in the index in approximately the same proportions as the index. The index identifies equity securities from the base index that satisfy certain ESG criteria, based on ESG performance data collected by MSCI ESG Research, Inc.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:08.052652	2020-10-11 23:38:31.028166
NHA	Nuveen Municipal 2021 Target Term Fund Fund	Nuveen Municipal 2021 Target Term is a closed end fund. Its objective is to provide a high level of current income and long-term capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.778165	2020-10-11 23:38:31.02423
NICE	NICE Ltd	NICE Ltd is an Israel-based provider of software solutions. It provides software solutions that enable operationalize Big Data and realize the business value. Nice has operations in Israel, Americas, Europe, and Asia Pacific.	{Technology,"Software - Application","Application Software"}	{PEGA,LPSN,VRNT,ACIW,FICO,IBM,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:07.79611	2020-10-11 23:38:31.024353
NJR	NewJersey Resources Corporation	New Jersey Resources Corp is a utilities service provider. The company mainly provides other retail and wholesale energy services.	{Utilities,"Utilities - Regulated Gas","Utilities - Regulated"}	{NI,OKE,CPK,NFG,SJI,VVC,CNP,SR,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:07.816793	2020-10-11 23:38:31.023428
NMIH	NMI Holdings Inc	NMI Holdings Inc through its subsidiaries provides private mortgage guaranty insurance. The company offers two types of mortgage insurance; Primary mortgage insurance and Pool insurance.	{"Financial Services","Insurance - Property & Casualty"}	{ERIE,ALL,RDN,HMN,MTG,AIG,HCI,NGHC,ESNT}	Insurance - Property & Casualty	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:07.85477	2020-10-11 23:38:31.02423
NOAH	Noah Holdings Limited	Noah Holdings Ltd is a wealth management service provider. It focuses on global wealth investment and asset allocation services for high net worth individuals and enterprises in China.	{"Financial Services","Asset Management"}	{ENV,MC}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.890136	2020-10-11 23:38:31.02423
NOVT	Novanta Inc.	Novanta Inc,designs, develops, manufactures and sells precision photonic and motion control components and subsystems to original equipment manufacturers in the medical, industrial, electronics and scientific markets.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{CVV,VECO,CGNX,COHR,ESIO,IIVI,ZBRA,IPGP,DHR,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:07.917548	2020-10-11 23:38:31.021426
NQP	Nuveen Pennsylvania Quality Municipal Income Fund	Nuveen Pennsylvania Quality Municipal Income Fund is a diversified closed end management investment company. Its investment objective is to provide current income exempt from both regular federal and designated state income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.942001	2020-10-11 23:38:31.023007
NS	Nustar Energy L.P.	NuStar Energy LP is engaged in refining, marketing and operating petroleum product terminals and petroleum liquids pipelines.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{GEL,PAA,MMP,HEP,TLP,BKEP,MPLX,DKL,PSXP}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:07.964596	2020-10-11 23:38:31.023007
NSU	Nevsun Resources Ltd	Nevsun Resources Ltd is a mid-tier base metals company. The company is engaged in the acquisition, exploration, development and operation of mineral property interests. Its projects include Timok in Serbi, and Bisha Mine in Eritrea.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{IAG,TRX,AKG,CMCL}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:07.984539	2020-10-11 23:38:31.021426
NTG	Tortoise MLP Fund Inc.	Tortoise MLP Fund Inc is an investment manager specializing in listed energy investments. It invests in master limited partnerships and their affiliates.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.00062	2020-10-11 23:38:31.02582
NTRP	Neurotrope Inc.	Neurotrope Inc is biopharmaceutical company with product candidates in pre-clinical and clinical development. It is engaged in developing a product platform based upon a drug candidate called bryostatin for the treatment of Alzheimer's disease.	{Healthcare,Biotechnology}	{CTXS,CTG,NUAN,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:08.025551	2020-10-11 23:38:31.02423
NUAN	Nuance Communications Inc.	Nuance Communications Inc provides speech, imaging and keypad solutions for businesses, organizations and consumers worldwide.	{Technology,"Software - Application","Application Software"}	{CSCO,MSFT,INTU,ADBE,CERN,MDRX,IBM,HPQ,GOOGL,XRX,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:08.047701	2020-10-11 23:38:31.025871
NUMV	NuShares ESG Mid-Cap Value	The investment seeks to track the investment results, before fees and expenses, of the TIAA ESG USA Mid-Cap Value Index (the "index").\n Under normal market conditions, the fund invests at least 80% of the sum of its net assets and the amount of any borrowings for investment purposes in component securities of the index. The index is comprised of equity securities issued by mid-capitalization companies listed on U.S. exchanges that meet certain environmental, social, and governance ("ESG") criteria.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:08.077064	2020-10-11 23:38:31.024353
NVCN	Neovasc Inc.	Neovasc Inc is a specialty medical device company. It develops, manufactures and markets products for the rapidly growing cardiovascular marketplace. Its products include the Tiara and the Neovasc Reducer.	{Healthcare,"Medical Devices"}	{XLV}	Medical Devices	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:08.106691	2020-10-11 23:38:31.031328
NHTC	Natural Health Trends Corp.	Natural Health Trends Corp is engaged in the direct selling industry, selling lifestyle enhancement products, cosmetics, personal care and dietary supplements, and also what is called network marketing or multi-level marketing.	{"Consumer Defensive","Household & Personal Products","Consumer Packaged Goods"}	{PCMI,LCUT,NUS,USNA,MTEX,GAIA,RELV}	Consumer Packaged Goods	Consumer Defensive	NASDAQ Capital Market	\N	2020-10-11 23:12:07.786335	2020-10-11 23:38:31.02582
NIQ	Nuveenn Intermediate Duration Quality Municipal Term Fund of Beneficial Interest	Nuveen Inter Dur Quality Muni Term Fund is a closed-end management investment company. The Fund's investment objective is to provide current income exempt from regular federal income tax with a secondary objective of seeking additional total return.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.813871	2020-10-11 23:38:31.024353
NLNK	NewLink Genetics Corporation	NewLink Genetics Corp is a clinical stage immuno-oncology company based in US. It focuses on developing novel immunotherapeutic products for the treatment of cancer. Its portfolio contains biologic and small-molecule immuno-oncology product candidates.	{Healthcare,Biotechnology}	{SIGA,SVA,BMY,JNJ,LLY,PFE,SRPT,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:07.835886	2020-10-11 23:38:31.023428
NMR	Nomura Holdings Inc ADR American Depositary Shares	Nomura Holdings Inc is a financial services company in Japan. The company serves various individuals, corporations, financial institutions, governments and governmental agencies.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{DB,MUFG,IX,MFG}	Brokers & Exchanges	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.858652	2020-10-11 23:38:31.02582
NNDM	Nano Dimension Ltd.	Nano Dimension Ltd is engaged in research and development of a three dimensional printer that prints prototypes for electronic circuits and development of nanotechnology based conductive ink, which are products that are necessary to the 3-D printer.	{Technology,"Computer Systems","Computer Hardware"}	{XLK}	Computer Hardware	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:07.877607	2020-10-11 23:38:31.028166
NOVN	Novan Inc.	Novan Inc together with its subsidiaries is a North Carolina based clinical-stage drug development company. It is engaged in the development and commercialization of nitric oxide-based therapies in dermatology.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:07.916251	2020-10-11 23:38:31.028166
NRE	NorthStar Realty Europe Corp.	NorthStar Realty Europe Corp is a real estate investment trust in the United States. It has interests in the real estate office properties located across Europe, especially in Germany.	{"Real Estate","REIT - Office",REITs}	{NYMT,O,NTP}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:07.944607	2020-10-11 23:38:31.021426
NSA	National Storage Affiliates Trust of Beneficial Interest	National Storage Affiliates Trust is a real estate investment trust. The company makes investments in self-storage properties located in the United States.	{"Real Estate","REIT - Industrial",REITs}	{SLG,AKR,MITT,SNH}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:07.965361	2020-10-11 23:38:31.021426
NSYS	Nortech Systems Incorporated	Nortech Systems Inc offers a full range of value-added engineering, technical and manufacturing services and support including project management, design, testing, prototyping, manufacturing, supply chain management and post-market services.	{Technology,"Electronic Components","Computer Hardware"}	{SGMA,KTCC,KBAL,IEC,XLK}	Computer Hardware	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:07.98589	2020-10-11 23:38:31.023428
NTIP	Network-1 Technologies Inc.	Network-1 Technologies Inc is engaged in the development, licensing and protection of the intellectual property assets. It identifies and acquires intellectual property from companies & inventors and implement industry licensing programs.	{Technology,"Communication Equipment"}	{SLP,VRNT,ASUR,XLK}	Communication Equipment	Technology	NYSE American	\N	2020-10-11 23:12:08.007479	2020-10-11 23:38:31.024353
NTWK	NETSOL Technologies Inc.	Netsol Technologies Inc and subsidiaries designs, develops, markets, and exports proprietary software products to customers in the automobile finance and leasing, banking, healthcare, and financial services industries globally.	{Technology,"Software - Application","Application Software"}	{MSFT,AMSWA,IBM,WIT,PAR,CAR,INFY,ORCL,HTZ,XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:08.031225	2020-10-11 23:38:31.023007
NUDM	NuShares ESG International Developed Markets Equity	The investment seeks to track the investment results, before fees and expenses, of the TIAA ESG International Developed Markets Equity Index (the "index").\n In seeking to track the investment results of the index, the advisor attempts to replicate the index by investing all, or substantially all, of its assets in the securities represented in the index in approximately the same proportions as the index. The index identifies equity securities from the base index that satisfy certain ESG criteria, based on ESG performance data collected by MSCI ESG Research, Inc.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:08.047803	2020-10-11 23:38:31.023428
NWPX	Northwest Pipe Company	Northwest Pipe Co is a manufacturer of engineered steel pipe water systems in North America. It operates in only one segment, Water Transmission.	{Industrials,"Metal Fabrication","Industrial Products"}	{IIIN,SYNL,ROCK,OFLX,NOV,VMI,X,TS,MWA,PPIH}	Industrial Products	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:08.156372	2020-10-11 23:38:31.024353
NLSN	Nielsen N.V.	Nielsen Holdings PLC is a professional services company providing performance management services. It helps its clients to enhance their interactions with consumers and make critical business decisions. Its segments are Buy and Watch.	{Industrials,"Business Services"}	{FORR,SCOR,IT,IBM,JW.B,TGNA,GOOGL,SPGI}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:07.841559	2020-10-11 23:38:31.02423
NMRD	Nemaura Medical Inc.	Nemaura Medical Inc owns a diagnostic medical device company specializing in discovering, developing and commercializing specialty medical devices.	{Healthcare,"Medical Devices"}	{XLV}	Medical Devices	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:07.860341	2020-10-11 23:38:31.023428
NNBR	NN Inc.	NN Inc manufactures precision bearing components, industrial plastic products and precision metal components serving variety of markets on a global basis.	{Industrials,"Tools & Accessories","Industrial Products"}	{ROLL,TKR,CIX,KAMN,RXN}	Industrial Products	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:07.878168	2020-10-11 23:38:31.021426
NOG	Northern Oil and Gas Inc.	Northern Oil & Gas Inc is an independent energy company engaged in the acquisition, exploration, development and production of oil and natural gas properties in the Bakken and Three Forks formations within the Williston Basin in North Dakota and Montana.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{NFX,CLR,OAS,REN,BCEI,ESTE,WLL}	Oil & Gas - E&P	Energy	NYSE American	\N	2020-10-11 23:12:07.903834	2020-10-11 23:38:31.024353
NPN	Nuveen Pennsylvania Municipal Value Fund of Beneficial Interest	Nuveen Pennsylvania Municipal Value Fund is a closed-end investment company. It primarily invests its assets in a portfolio of municipal securities with an objective to provide current income exempt from regular federal and Pennsylvania income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.92686	2020-10-11 23:38:31.025871
NRT	North European Oil Royality Trust	North European Oil Royalty Trust is a grantor trust. The Company holds overriding royalty rights covering gas and oil production in certain concessions or leases in the Federal Republic of Germany.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{MARPS,PBT,CRT,MTR,SBR,SJT}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:07.953649	2020-10-11 23:38:31.025871
NSS	NuStar Logistics L.P. 7.625% Fixed-to-Floating Rate Subordinated Notes due 2043		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:07.981254	2020-10-11 23:38:31.028166
NTES	NetEase Inc.	NetEase Inc functions in the internet content market. Through online communities, it provides Chinese language content and value added services.	{Technology,"Internet Content & Information","Online Media"}	{XLK}	Online Media	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:08.000381	2020-10-11 23:38:31.023007
NTRA	Natera Inc.	Natera Inc is a provider of diagnostic services predominantly in the United States. It offers various tests which detect chromosomal abnormalities, reasons for miscarriages and paternity.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{VRTX,PRTA,PRTO,PRTK,XLV}	Medical Diagnostics & Research	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:08.020344	2020-10-11 23:38:31.028166
NULV	NuShares ESG Large-Cap Value	The investment seeks to track the investment results, before fees and expenses, of the TIAA ESG USA Large-Cap Value Index (the "index").\n Under normal market conditions, the fund invests at least 80% of the sum of its net assets and the amount of any borrowings for investment purposes in component securities of the index. The index is comprised of equity securities issued by large capitalization companies listed on U.S. exchanges that meet certain environmental, social, and governance ("ESG") criteria.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:08.057118	2020-10-11 23:38:31.023007
NUSA	NuShares Enhanced Yield 1-5 Year U.S. Aggregate Bond	The investment seeks to track the investment results, before fees and expenses, of the ICE BofAML Enhanced Yield 1-5 Year US Broad Bond Index (the "index").\n Under normal market conditions, the fund invests at least 80% of the sum of its net assets and the amount of any borrowings for investment purposes in component securities of the index. The index is designed to broadly capture the 1-5 year U.S. investment grade fixed income market, as represented by a modified version of the ICE BofAML 1-5 Year US Broad Market Index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.087692	2020-10-11 23:38:31.02582
NVFY	Nova Lifestyle Inc	Nova Lifestyle Inc is a designer and lifestyle furniture company. It distributes and retails contemporary styled residential furniture. The firm's marketing and sales platform offer retail, online selection, and purchase fulfillment internationally.	{"Consumer Cyclical","Home Furnishings & Fixtures","Manufacturing - Apparel & Furniture"}	{FLXS,HOFT,LZB,SNBR}	Manufacturing - Apparel & Furniture	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:08.114236	2020-10-11 23:38:31.02423
NVTR	Nuvectra Corporation	Nuvectra Corp is a medical device company in the United States. It manufactures devices to address chronic neurological disorders. The company has two reportable segments namely Nuvectra and NeuroNexus.	{Healthcare,"Medical Devices"}	{NUVA,XLV}	Medical Devices	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:08.132734	2020-10-11 23:38:31.024353
NWN	Northwest Natural Holding Company	Northwest Natural Gas along with its subsidiaries distributes natural gas in Oregon, Washington and California. The company has two business segments: Utility segment and Gas Storage segment.	{Utilities,"Utilities - Regulated Gas","Utilities - Regulated"}	{XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:08.153707	2020-10-11 23:38:31.023007
NKX	Nuveen California AMT-Free Quality Municipal Income Fund	Nuveen California AMT-Free Quality Municipal Income Fund is a diversified closed-end management investment company which seeks to provide current income exempt from regular federal and California state income taxes and alternative minimum tax (AMT).	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.831261	2020-10-11 23:38:31.024353
NLST	Netlist Inc.	Netlist Inc designs, manufactures and sells high performance modular memory subsystems serving customers in diverse industries that require superior memory performance to empower critical business decisions.	{Technology,"Semiconductor Memory",Semiconductors}	{CY,IDTI,MU,TXN,WDC,SANM,AJRD,USAU,XLK}	Semiconductors	Technology	OTCQX Marketplace	\N	2020-10-11 23:12:07.847427	2020-10-11 23:38:31.025871
NMRK	Newmark Group Inc.	Newmark Group Inc is a commercial real estate advisory firm. It offers services, including leasing and corporate advisory services, investment sales, commercial mortgage brokerage, appraisal and valuation, project management, and property, among others.	{"Real Estate","Real Estate Services"}	{}	Real Estate Services	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:07.868162	2020-10-11 23:38:31.025871
NNVC	NanoViricides Inc.	Nanoviricides Inc is a development stage nano-biopharmaceutical company. Its business activities are to discover, develop and commercialize therapeutics to take care of patients suffering from life-threatening viral infections.	{Healthcare,"Drug Manufacturers - Major","Drug Manufacturers"}	{BCRX,NVAX,GILD,ACHN,GSK,BMY,MRK,CMRX,NBY,XLV}	Drug Manufacturers	Healthcare	NYSE American	\N	2020-10-11 23:12:07.886426	2020-10-11 23:38:31.02582
NPK	National Presto Industries Inc.	National Presto Industries Inc is engaged in manufacturing defense products. Its product portfolio includes 40mm ammunition, precision and electro-mechanical assemblies and medium caliber cartridge cases, amongst others.	{Industrials,"Aerospace & Defense"}	{TRS,KMB,RGR,AJRD,EPC,AOBC}	Aerospace & Defense	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:07.927738	2020-10-11 23:38:31.02582
NRP	Natural Resource Partners LP Limited Partnership	Natural Resource Partners LP is a diversified natural resource company. It is primarily engaged in the business of owning, managing and leasing a diversified portfolio of mineral properties in the United States.	{"Basic Materials",Coal}	{ARLP,HNRG}	Coal	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:07.955602	2020-10-11 23:38:31.023428
NSPR	InspireMD Inc.	InspireMD Inc is a United States based medical device company. It is primarily focused on the development and commercialization of proprietary MicroNet stent platform technology for the treatment of complex vascular and coronary disease.	{Healthcare,"Medical Devices"}	{XLV}	Medical Devices	Healthcare	NYSE American	\N	2020-10-11 23:12:07.969579	2020-10-11 23:38:31.028166
NTCT	NetScout Systems Inc.	NetScout Systems Inc offers real-time operational intelligence and performance analytics for service assurance and cyber security solutions used by service provider, enterprise and government networks.	{Technology,"Software - Application","Application Software"}	{FFIV,JKHY,DHR,IBM,VIAV,HPQ,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:07.995526	2020-10-11 23:38:31.028166
NTR	Nutrien Ltd.	Nutrien Ltd is a provider of crop nutrients, inputs and services. The company produces and sells potash, nitrogen and phosphate products for agricultural, industrial and feed customers. The company's main focus is of potash sales.	{"Basic Materials","Agricultural Inputs",Agriculture}	{}	Agriculture	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:08.018551	2020-10-11 23:38:31.023428
NTZ	Natuzzi S.p.A.	Natuzzi SPA designs, manufactures and sells contemporary and traditional leather & fabric-upholstered furniture including sofas, loveseats, armchairs, sectional furniture, motion furniture and sofa beds, living room furnishings and accessories.	{"Consumer Cyclical","Home Furnishings & Fixtures","Manufacturing - Apparel & Furniture"}	{BSET,FLXS,HOFT,LZB,SNBR}	Manufacturing - Apparel & Furniture	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:08.044882	2020-10-11 23:38:31.021426
NUMG	NuShares ESG Mid-Cap Growth	The investment seeks to track the investment results, before fees and expenses, of the TIAA ESG USA Mid-Cap Growth Index (the "index").\n Under normal market conditions, the fund invests at least 80% of the sum of its net assets and the amount of any borrowings for investment purposes in component securities of the index. The index is comprised of equity securities issued by mid-capitalization companies listed on U.S. exchanges that meet certain environmental, social, and governance ("ESG") criteria.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:08.072817	2020-10-11 23:38:31.02423
NVAX	Novavax Inc.	Novavax Inc is a clinical-stage vaccine company. It is engaged in the discovery, development and commercialization of recombinant nanoparticle vaccines and adjuvants.	{Healthcare,Biotechnology}	{BCRX,ACHN,CLDX,AZN,GSK,NVS,PFE,EBS,AGEN,SRPT,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:08.102723	2020-10-11 23:38:31.025871
NVUS	Novus Therapeutics Inc.	Novus Therapeutics Inc is a pharmaceutical company focusing on the acquisition, development, and commercialization of ear, nose, and throat products. Its technologies have the potential to be developed for multiple ENT indications.	{Healthcare,Biotechnology}	{PDLI,AGEN,ADXS,AFMD,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:08.138783	2020-10-11 23:38:31.023428
NXEO	Nexeo Solutions Inc.	Nexeo Solutions Inc distributes chemicals and plastic products across North America and Asia. The company operates its business through three segments that are; Chemicals, Plastics, and Environmental Services.	{"Basic Materials",Chemicals}	{AGFS,INSE}	Chemicals	Basic Materials	NASDAQ Capital Market	\N	2020-10-11 23:12:08.170313	2020-10-11 23:38:31.028166
NML	Neuberger Berman MLP Income Fund Inc.	Neuberger Berman MLP Income Fund Inc. is an asset management firm. Its activities involve investing primarily in master limited partnership enterprises through equity, fixed income, private equity instruments.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:07.854826	2020-10-11 23:38:31.028166
NNA	Navios Maritime Acquisition Corporation	Navios Maritime Acquisition Corporation is an owner and operator of tanker vessels focusing in the transportation of petroleum products and bulk liquid chemicals. It offers its services across Asia, Europe, and the United States.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:07.873456	2020-10-11 23:38:31.028166
NOC	Northrop Grumman Corporation	Northrop Grumman Corp offers portfolio of capabilities and technologies to deliver products, systems and solutions for applications to government and commercial customers. It provides products, systems and solutions in autonomous systems.	{Industrials,"Aerospace & Defense"}	{KTOS,LMT,BA,GD,HRS,UTX,RTN,COL,LLL}	Aerospace & Defense	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:07.895146	2020-10-11 23:38:31.031328
NOMD	Nomad Foods Limited	Nomad Foods Ltd is a frozen foods company. The company along with its subsidiaries produces, markets and distributes brands in approximately 17 countries and has a market share in Western Europe. Its brands includes Birdseye, Iglo, and Findus.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{JAX,KHC,NOMD}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:07.911839	2020-10-11 23:38:31.028166
NPV	Nuveen Virginia Quality Municipal Income Fund	The Company provides services related to investment, risk management and other financial services.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.932181	2020-10-11 23:38:31.028166
NRO	Neuberger Berman Real Estate Securities Income Fund Inc.	Neuberger Berman R/E Securities Income is a closed-end management investment company that invests mainly in securities issued by Real Estate Companies, including Real Estate Investment Trusts (REITs).	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:07.949442	2020-10-11 23:38:31.031328
NSL	Nuveen Senior Income Fund	Nuveen Senior Income Fund is a diversified closed-end management investment company. Its investment objective is to achieve a high level of current income, consistent with capital preservation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.967822	2020-10-11 23:38:31.023428
NTC	Nuveen Connecticut Quality Municipal Income Fund	To provide current income exempt from regular federal and Connecticut income tax and to enhance portfolio value relative to the municipal bond market by investing in tax-exempt municipal bonds that the Funds investment adviser believes are underrated or undervalued or that represent municipal market sectors that are undervalued. It will invest at least 80% of its assets in investment grade quality municipal bonds (BBB/Baa or better at the time of purchase by Moody, Fitch or Standard & Poor). It may invest up to 20% of its net assets in municipal bonds that are rated Ba/BB or B or that are unrated but judged to be of comparable quality by the investment adviser.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.995824	2020-10-11 23:38:31.024353
NTP	Nam Tai Property Inc.	Nam Tai Property Inc is engaged in the business of property development and management. The company is focused on redevelopment of parcels of land into commercial complexes.	{"Real Estate","Real Estate Services"}	{CLS,ARE,SANM,BXMT}	Real Estate Services	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:08.018711	2020-10-11 23:38:31.031328
NUE	Nucor Corporation	Nucor Corp and its affiliates are engaged in the manufacturing of steel and steel products. It also produces direct reduced iron for use in the steel mills. The company is involved in every phase of the steelmaking.	{"Basic Materials",Steel}	{SCHN,USAP,STLD,GGB,CRS,CMC,WOR,X,PKX,MT}	Steel	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:08.047779	2020-10-11 23:38:31.02582
NURO	NeuroMetrix Inc.	NeuroMetrix Inc is engaged in product development, manufacturing, regulatory affairs and sales and marketing. It mainly produces bioelectrical and digital medicine to address chronic health conditions including chronic pain, sleep disorders, and diabetes.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{IART,CASM,BABY,NUVA,SYK,MDT,LIVN,XLV}	Medical Instruments & Equipment	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:08.079695	2020-10-11 23:38:31.031328
NVEC	NVE Corporation	NVE Corp is engaged in the research, design, and manufacturing of spintronics for commercial and military markets.	{Technology,Semiconductors}	{CY,MXIM,SLAB,VSH,TXN,ADI,XLK}	Semiconductors	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:08.105829	2020-10-11 23:38:31.023428
NVLN	Novelion Therapeutics Inc.	Novelion Therapeutics Inc is a biopharmaceutical company. It is engaged in developing patient care facility and providing services for people suffering from rare diseases like lipodystrophy, leber congenital amaurosis, and retinitis pigmentosa.	{Healthcare,Biotechnology}	{HBP,SRDX,ALIM,SNY,INSM,OPHT,AERI,OPK,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:08.121421	2020-10-11 23:38:31.023428
NJV	Nuveen New Jersey Municipal Value Fund of Beneficial Interest	Nuveen NJ Municipal Value Fund is a closed-end investment company. Its investment objective is to provide current income exempt from both regular federal and designated state income taxes by investing primarily in a portfolio of municipal obligations.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.818136	2020-10-11 23:38:31.025871
NLS	Nautilus Inc.	Nautilus Inc is a consumer fitness products company. It is engaged in designing, developing, sourcing and marketing of cardio and strength fitness products and related accessories for consumer use.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{ESCA,JOUT,ELY}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:07.836607	2020-10-11 23:38:31.021426
NNI	Nelnet Inc.	Nelnet Inc is focused on delivering education-related products and services and loan asset management. It is engaged in student loan servicing, tuition payment processing and school information systems, and communications.	{"Financial Services","Credit Services"}	{FIS,DFS,SLM,NAVI}	Credit Services	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.88239	2020-10-11 23:38:31.025871
NOK	Nokia Corporation Sponsored American Depositary Shares	Nokia Oyj is a telecommunication company. It develops network equipment to transmit data over wireless, copper, optical fiber, and microwave mediums. Its products are used by the world's largest telecommunication service providers.	{Technology,"Communication Equipment"}	{CSCO,MSFT,AMZN,AAPL,ERIC,MSI,BB,XLK}	Communication Equipment	Technology	New York Stock Exchange	\N	2020-10-11 23:12:07.907947	2020-10-11 23:38:31.023007
NP	Neenah Inc.	Neenah Inc, formerly Neenah Paper Inc, is a producer of filtration media and other durable, saturated, and coated substrates, and also premium, specialty and sustainable papers used for premium writing, text, cover, private watermarks.	{"Basic Materials","Paper & Paper Products","Forest Products"}	{UFS,SWM,GLT,CLW}	Forest Products	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:07.926984	2020-10-11 23:38:31.031328
NRG	NRG Energy Inc.	NRG Energy Inc is engaged in the utilities sector. Its primary business involves operation of power generation facilities, trading of energy, capacity and related products, transacting in and trading of fuel & transportation services.	{Utilities,"Utilities - Independent Power Producers"}	{EIX,EE,OGE,AES,SO,EXC,NEE,DUK,ES,XLU}	Utilities - Independent Power Producers	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:07.946928	2020-10-11 23:38:31.028166
NSP	Insperity Inc.	Insperity Inc is engaged in providing human resources and business solutions designed to improve business performance. Its product offerings are provided through its Workforce Optimization and Workforce Synchronization solutions.	{Industrials,"Staffing & Outsourcing Services","Employment Services"}	{BBSI,ADP,KELYA,PAYX,KFY,WAGE,HSON,TNET}	Employment Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:07.969494	2020-10-11 23:38:31.02423
NTEC	Intec Pharma Ltd.	Intec Pharma Ltd is an Israel-based drug development company. It is a development stage biopharmaceutical company that develops formulations of drugs using its proprietary gastric retention technology, the Accordion Pill.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:07.995076	2020-10-11 23:38:31.028166
NTNX	Nutanix Inc.	Nutanix Inc provides cloud capabilities for businesses. It mainly offers web-scale engineering and consumer-grade design, virtualization and storage into a resilient, and software-defined solution to its customers.	{Technology,"Software - Infrastructure","Application Software"}	{PRGS,PXLW,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:08.01925	2020-10-11 23:38:31.023007
NUAG	NuShares Enhanced Yield US Aggregate Bond ETF	The investment seeks to track the investment results, before fees and expenses, of the ICE BofAML Enhanced Yield 1-5 Year US Broad Bond Index (the "index").\n Under normal market conditions, the fund invests at least 80% of the sum of its net assets and the amount of any borrowings for investment purposes in component securities of the index. The index is designed to broadly capture the 1-5 year U.S. investment grade fixed income market, as represented by a modified version of the ICE BofAML 1-5 Year US Broad Market Index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.045461	2020-10-11 23:38:31.031328
NUO	Nuveen Ohio Quality Municipal Income Fund	Nuveen Ohio Quality Municipal Income Fund is a closed-end investment management company. Its investment objective is to provide current income exempt from both regular federal and designated state income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.077255	2020-10-11 23:38:31.025871
NVDA	NVIDIA Corporation	NVIDIA Corp is a developer of graphics processing unit. It caters to areas like gaming, professional visualization, datacenter and automobiles.	{Technology,Semiconductors}	{INTC,MSFT,QCOM,STM,TXN,AMBA,AMD,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:08.106308	2020-10-11 23:38:31.021426
NVO	Novo Nordisk A/S	Novo Nordisk A/S is a healthcare company. It is engaged in the discovery, development, manufacturing and marketing of pharmaceutical products. The company's business segments include, diabetes and obesity care, and biopharmaceuticals.	{Healthcare,Biotechnology}	{BMY,LLY,PFE,SNY,XLV}	Biotechnology	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:08.125957	2020-10-11 23:38:31.028166
NWLI	National Western Life Group Inc.	National Western Life Group Inc is a stock life insurance company. It offers a portfolio of individual whole life, universal life and term insurance plans, and annuities, including supplementary riders.	{"Financial Services","Insurance - Life"}	{AAME,CIA,LNC,TMK,AEL,PRI,FFG}	Insurance - Life	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:08.152921	2020-10-11 23:38:31.025871
NMI	Nuveen Municipal Income Fund Inc.	Nuveen Municipal Income Fund Inc is a diversified closed end management investment company. Its investment objective is to provide current income exempt from regular federal income tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.854656	2020-10-11 23:38:31.031328
NOM	Nuveen Missouri Quality Municipal Income Fund	Nuveen MO Premium Income Muni Fund is a closed-end management investment company which pursues to provide current income exempt from regular federal and designated state income taxes by investing primarily in a portfolio of municipal obligations.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.910515	2020-10-11 23:38:31.02582
NPTN	NeoPhotonics Corporation	NeoPhotonics Corp manufactures optoelectronic products that transmits, receives and switches high speed digital optical signals for communication networks. Its products are offered under High Speed Products, and Network Products and Solutions segments.	{Technology,Semiconductors}	{FNSR,OCLR,EMKR,AAOI,VIAV,XLK}	Semiconductors	Technology	New York Stock Exchange	\N	2020-10-11 23:12:07.933133	2020-10-11 23:38:31.023428
NRZ	New Residential Investment Corp.	New Residential Investment Corp is a real estate investment trust, which focuses on investing in and managing, investments related to residential real estate in the United States.	{"Real Estate","REIT - Residential",REITs}	{NYMT,O,NTP}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:07.959092	2020-10-11 23:38:31.028166
NSSC	NAPCO Security Technologies Inc.	NAPCO Security Technologies Inc is engaged in manufacturing of security products, encompassing access control systems, door security products, intrusion and fire alarm systems and video surveillance products.	{Industrials,"Security & Protection Services","Consulting & Outsourcing"}	{MAGS,UUU,HON,UTX}	Consulting & Outsourcing	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:07.980762	2020-10-11 23:38:31.025871
NTIC	Northern Technologies International Corporation	Northern Technologies International Corp develops and markets proprietary environmentally beneficial products and services. The Company is engaged in developing, manufacturing and marketing ZERUST rust and corrosion inhibiting products and services.	{"Basic Materials","Specialty Chemicals",Chemicals}	{UFPT,WDFC,IOSP,GRA,KWR,ASH}	Chemicals	Basic Materials	NASDAQ Global Market	\N	2020-10-11 23:12:08.002306	2020-10-11 23:38:31.02423
NTRS	Northern Trust Corporation	Northern Trust Corp is a provider of asset servicing, fund administration, asset management, fiduciary and banking solutions for corporations, institutions, families and individuals.	{"Financial Services","Asset Management"}	{WAFD,BOKF,PBCT,COF,STT,WFC,NYCB,JPM,BK}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:08.027054	2020-10-11 23:38:31.024353
NULG	NuShares ESG Large-Cap Growth	The investment seeks to track the investment results, before fees and expenses, of the TIAA ESG USA Large-Cap Growth Index (the "index").\n Under normal market conditions, the fund invests at least 80% of the sum of its net assets and the amount of any borrowings for investment purposes in component securities of the index. The index is comprised of equity securities issued by large capitalization companies listed on U.S. exchanges that meet certain environmental, social, and governance ("ESG") criteria.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:08.063336	2020-10-11 23:38:31.021426
NUS	Nu Skin Enterprises Inc.	Nu Skin Enterprises Inc develops and distributes consumer products including anti-aging skin products, body care, hair care, men's care, oral care, sun protection and cosmetics under Nu Skin and Pharmanex brands.	{"Consumer Defensive","Household & Personal Products","Consumer Packaged Goods"}	{IPAR,NATR,EL,TUP,AVP,HLF,REV,USNA,MTEX}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:08.084689	2020-10-11 23:38:31.028166
NVCR	NovoCure Limited	NovoCure Ltd is a medical systems developer in the United States. Its primary product is the TTFields delivery system, which is a low-intensity therapy to combat acute tumors.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{NUVA,XLV}	Medical Instruments & Equipment	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:08.10626	2020-10-11 23:38:31.028166
NVR	NVR Inc.	NVR Inc is a United States based company engaged in the construction and sale of single-family detached homes, town homes and condominium buildings. It also operates in mortgage banking and title services business.	{"Consumer Cyclical","Residential Construction","Homebuilding & Construction"}	{CRD.B,DHI,HOV,LEN,MDC,KBH}	Homebuilding & Construction	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:08.129752	2020-10-11 23:38:31.028166
NWS	News Corporation	News Corp is a diversified media and information services company focused on creating and distributing authoritative and engaging content to consumers and businesses.	{"Consumer Cyclical","Broadcasting - TV",Entertainment}	{TRI,LEE,NYT,DIS,CBS,PSO,NEWM,TGNA,RELX}	Entertainment	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:08.156679	2020-10-11 23:38:31.023428
NXJ	Nuveen New Jersey Qualified Municipal Fund	Nuveen New Jersey Quality Municipal Income Fund is an asset management company. Its objective is to provide current income exempt from both regular federal and designated state income taxes by investing primarily in a portfolio of municipal obligations.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.17656	2020-10-11 23:38:31.021426
NMFC	New Mountain Finance Corporation	New Mountain Finance Corp is an investment management company. It invests in equity interests such as preferred stock, common stock, warrants or options received in connection with debt investments.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.853493	2020-10-11 23:38:31.023007
NMZ	Nuveen Municipal High Income Opportunity Fund $0.01 par value per share	Nuveen Municipal High Income Opp Fund is a diversified, closed-end management investment company. Its investment objectives is to provide current income exempt from regular federal income tax by investing primarily in municipal securities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.870241	2020-10-11 23:38:31.02423
NORW	Global X MSCI Norway	The investment seeks investment results that correspond generally to the price and yield performance, before fees and expenses, of the MSCI Norway IMI 25/50 Index.\n The fund invests at least 80% of its total assets in the securities of the underlying index and in American Depositary Receipts ("ADRs") and Global Depositary Receipts ("GDRs") based on the securities in the underlying index. The underlying index is designed to represent the performance of the broad Norway equity universe. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:07.913437	2020-10-11 23:38:31.025871
NR	Newpark Resources Inc.	Newpark Resources Inc is in providing products and services to the oil and gas exploration industry serving customers in North America, Europe, the Middle East and Africa, Latin America and Asia Pacific regions.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{SPN,NOV,SLB,DRQ,HAL,OIS,FTK,BHGE}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:07.943526	2020-10-11 23:38:31.02582
NSEC	National Security Group Inc.	National Security Group Inc is an insurance holding company. The company operates through two segments namely property and casualty insurance, and life insurance.	{"Financial Services","Insurance - Property & Casualty"}	{AAME,ALL,MET,AIZ,HTH,UVE,UIHC,HCI,HRTG}	Insurance - Property & Casualty	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:07.967914	2020-10-11 23:38:31.02582
NSTG	NanoString Technologies Inc.	NanoString Technologies Inc is a biotechnology company. It develops, manufactures and sells products that unlock scientifically valuable and clinically actionable genomic information from minute amounts of tissue.	{Healthcare,Biotechnology}	{LMNX,ILMN,A,BIO,TMO,PKI,PACB,FLDM,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:07.984662	2020-10-11 23:38:31.023007
NTGR	NETGEAR Inc.	Netgear Inc is a networking equipment provider. Its array of products includes WiFi routers, DSL modems, Hotspots, USB Modems, WiFi adapters & switches.	{Technology,"Communication Equipment"}	{CSCO,ADTN,AAPL,FTNT,MSI,WDC,HPQ,ARRS,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:08.000905	2020-10-11 23:38:31.025871
NTRI	NutriSystem Inc	Nutrisystem Inc is a provider of weight loss solutions and weight management products and services in the United States.	{"Consumer Cyclical","Personal Services"}	{COST,SFLY,RGS,WMT,MED,WTW}	Personal Services	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:08.023708	2020-10-11 23:38:31.025871
NUBD	NuShares ESG U.S. Aggregate Bond	The investment seeks to track the investment results, before fees and expenses, of the Bloomberg Barclays MSCI US Aggregate ESG Select Index (the "index").\n Under normal market conditions, the fund invests at least 80% of the sum of its net assets and the amount of any borrowings for investment purposes in component securities of the index. The index utilizes certain environmental, social, and governance ("ESG") criteria to select from the securities included in the Bloomberg Barclays US Aggregate Bond Index (the "base index"), which is designed to broadly capture the U.S. investment grade, taxable fixed income market.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.047616	2020-10-11 23:38:31.02423
NUM	Nuveen Michigan Quality Municipal Income Fund	Nuveen MI Quality Income Muni Fund Inc is a closed-end investment company. The fund's investment objective is to provide current income exempt from both regular federal and designated state income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.072697	2020-10-11 23:38:31.028166
NUV	Nuveen Municipal Value Fund Inc.	Nuveen Municipal Value Fund Inc is a diversified closed end management investment company. Its investment objective is to provide current income exempt from regular federal income tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.093726	2020-10-11 23:38:31.024353
NVGS	Navigator Holdings Ltd. (Marshall Islands)	Navigator Holdings Ltd is an owner and operator of a fleet of liquefied gas carriers.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{GLNG,CUK,GSL,KNOP}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:08.114109	2020-10-11 23:38:31.02582
NVTA	Invitae Corporation	Invitae Corp is a genetic information company engaged in genetic diagnostics for hereditary disorders. It provides testing for multiple genes including hereditary cancer, neurological disorders, cardiovascular disorders and other hereditary conditions.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{CLDX,RDUS,HTGM,XLV}	Medical Diagnostics & Research	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:08.132833	2020-10-11 23:38:31.023007
NWSA	News Corporation	News Corp is a diversified media and information services company focused on creating and distributing authoritative and engaging content to consumers and businesses.	{"Consumer Cyclical","Broadcasting - TV",Entertainment}	{TRI,LEE,NYT,DIS,CBS,PSO,NEWM,TGNA,RELX}	Entertainment	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:08.159726	2020-10-11 23:38:31.02582
NNY	Nuveen New York Municipal Value Fund Inc.	Nuveen New York Municipal Value Fund Inc is a diversified closed end management investment company. Its investment objective is to provide current income exempt from both regular federal and New York state income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.889681	2020-10-11 23:38:31.023007
NOV	National Oilwell Varco Inc.	National Oilwell Varco Inc is a provider in the design, manufacture and sale of equipment and components used in oil and gas drilling, completion and production operations, and the provision of oilfield services to the upstream oil and gas industry.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{SLB,DRQ,NR,OIS,FET,BHGE}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:07.914269	2020-10-11 23:38:31.023428
NPO	EnPro Industries Inc	EnPro Industries Inc is engaged in the designing, developing, manufacturing, and marketing engineered industrial products. It operates with three segments namely Sealing Products, Engineered Products and Power Systems.	{Industrials,"Diversified Industrials","Industrial Products"}	{TRS,CSL,CAT,DOV,GE,GGG,PH,ATU}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:07.9296	2020-10-11 23:38:31.024353
NRK	Nuveen New York AMT-Free Quality Municipal Income Fund	Nuveen NY AMT-Free Muni Income Fund is a closed-end investment company. The company offers products such as mutual funds, closed-end funds, exchange-traded funds, and unit trusts.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.949304	2020-10-11 23:38:31.024353
NSIT	Insight Enterprises Inc.	Insight Enterprises Inc is an Information Technology provider helping businesses of all sizes from small and medium sized firms to enterprises, governments, schools and healthcare organizations; define architect, manage Intelligent Technology Solutions.	{Technology,"Information Technology Services","Application Software"}	{MSFT,CNXN,SYMC,TECD,PCMI,SYX,IBM,SNX,CDW,HPQ,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:07.967701	2020-10-11 23:38:31.031328
NTB	Bank of N.T. Butterfield & Son Limited (The) Voting	Bank of N.T Butterfield & Son Ltd is a community bank providing international financial services. Its services include retail, private, corporate banking, treasury, custody, asset management and personal and institutional trust services.	{"Financial Services","Banks - Global",Banks}	{}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.991516	2020-10-11 23:38:31.02423
NTN	NTN Buzztime Inc.	NTN Buzztime Inc provides interactive entertainment and dining technology to bars and restaurants in North America. Its main products are Buzztime, Playmaker, Mobile Playmaker, BEOND Powered by Buzztime and Play Along.	{"Consumer Cyclical","Broadcasting - TV",Entertainment}	{SEAC,DIS,RUBI,CIDM,TIVO}	Entertainment	Consumer Cyclical	NYSE American	\N	2020-10-11 23:12:08.016962	2020-10-11 23:38:31.021426
NUGT	Direxion Daily Gold Miners Index Bull 3X Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the daily performance of the NYSE Arca Gold Miners Index.\n The fund invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, ETFs that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index is a modified market capitalization weighted index comprised of publicly traded companies that operate globally in both developed and emerging markets, and are involved primarily in mining for gold and, in mining for silver. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.054962	2020-10-11 23:38:31.031328
NUSC	NuShares ESG Small-Cap	The investment seeks to track the investment results, before fees and expenses, of the TIAA ESG USA Small-Cap Index (the "index").\n Under normal market conditions, the fund invests at least 80% of the sum of its net assets and the amount of any borrowings for investment purposes in component securities of the index. The index is comprised of equity securities issued by small-capitalization companies listed on U.S. exchanges that meet certain environmental, social, and governance ("ESG") criteria.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:08.087616	2020-10-11 23:38:31.02423
NVEE	NV5 Global Inc.	NV5 Global Inc is a provider of professional and technical engineering and consulting solutions to public and private sector clients in the infrastructure, energy, construction, real estate, and environmental markets.	{Industrials,"Engineering & Construction"}	{FC}	Engineering & Construction	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:08.10892	2020-10-11 23:38:31.024353
NVRO	Nevro Corp.	Nevro Corp develops and sells medical devices mainly in the United States. Its main product is the Senza system which is intended to relieve leg and spinal cord pain with the application of the HF10 therapy.	{Healthcare,"Medical Devices"}	{ABMD,ISRG,TMO,XLV}	Medical Devices	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:08.131158	2020-10-11 23:38:31.021426
NWL	Newell Brands Inc.	Newell Brands Inc is engaged in the household products market. Its products include stationery material such as pens and markers, food storage products and hygiene systems.	{"Consumer Defensive","Household & Personal Products","Consumer Packaged Goods"}	{HELE,TUP,ACU,CSL,CLX,ACCO,EPC}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:08.1506	2020-10-11 23:38:31.021426
NXEOU	Nexeo Solutions Inc. Unit	Nexeo Solutions Inc distributes chemicals and plastic products across North America and Asia. The company operates its business through three segments that are; Chemicals, Plastics, and Environmental Services.	{"Basic Materials",Chemicals}	{}	Chemicals	Basic Materials	NASDAQ Capital Market	\N	2020-10-11 23:12:08.171822	2020-10-11 23:38:31.025871
NUVA	NuVasive Inc.	NuVasive Inc is a medical device company. It develops minimally-disruptive surgical products and procedurally-integrated solutions for the spine. Its product portfolio is focused on applications for spine fusion surgery.	{Healthcare,"Medical Devices"}	{RTIX,SYK,ZBH,GMED,MDT,ATEC,XLV}	Medical Devices	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:08.09724	2020-10-11 23:38:31.023007
NVG	Nuveen AMT-Free Municipal Credit Income Fund	Nuveen AMT-Free Municipal Credit Income Fund is a closed-end investment company. Its investment objective is to provide current income by investing in a portfolio of municipal obligations issued by state and local government authorities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.113804	2020-10-11 23:38:31.023007
NVS	Novartis AG	Novartis AG is a part of the healthcare industry. Its main activity is to research and develop healthcare products and pharmaceuticals such as oncology medicines, generic and biosimilar medicines and eye care devices.	{Healthcare,"Drug Manufacturers - Major","Drug Manufacturers"}	{GSK,ABT,BMY,JNJ,PFE,SNY,MRK,TEVA,XLV}	Drug Manufacturers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:08.132763	2020-10-11 23:38:31.02423
NX	Quanex Building Products Corporation	Quanex Building Products Corp manufactures engineered products such as window components that include flexible insulating glass spacers, extruded vinyl profiles, window and door screens, solar panel sealants & precision-formed metal & wood products.	{"Basic Materials","Building Materials"}	{BLDR,KALU,SSD,VMI,NCS,DOOR,AA,PGTI}	Building Materials	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:08.167605	2020-10-11 23:38:31.02423
NXEOW	Nexeo Solutions (Warrant)	Nexeo Solutions, Inc. engages in the distribution of chemicals products. It operates through the following segments: Chemicals, Plastics, and Other. The Chemicals segment distributes specialty and industrial chemicals, additives and solvents to industrial users via railcars, barges, bulk tanker trucks, and as packaged goods in trucks. The Plastics segment offers a product line consisting of commodity polymer products and prime engineering resins to plastics processors engaged in blow molding, extrusion, injection molding, and rotation molding via railcars, bulk trucks, truckload boxes, and less-than-truckload quantities. The Other segment involves in customers with comprehensive on-site and off-site hazardous and non-hazardous waste collection, recovery, and arrangement for disposal services or recycling. The company was founded on March 24, 2014 and is headquartered in The Woodlands, TX.	{}	{}	Chemicals Specialty	Process Industries	https://api.robinhood.com/markets/XNAS/	\N	2020-10-11 23:12:08.178477	2020-10-11 23:38:31.024353
NXR	Nuveen Select Tax Free Income Portfolio III	Nuveen Select Tax Free Income Port 3 is a diversified closed-end management investment company. It seeks to provide current income and stable dividends, exempt from regular federal and designated state income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.189581	2020-10-11 23:38:31.028166
NXTD	NXT-ID Inc.	Nxt-ID Inc provides products, solutions, and services that have a need for biometric secure access control. Its lines of business include mobile commerce, law enforcement and biometric access control applications and law enforcement.	{Industrials,"Security & Protection Services","Consulting & Outsourcing"}	{}	Consulting & Outsourcing	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:08.195979	2020-10-11 23:38:31.024353
NXTM	NxStage Medical Inc.	NxStage Medical Inc is associated to the healthcare industry. It develops medical products for patients suffering from kidney failure.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{RMTI,FMS,BAX,BSX,CMD,DVA,XLV}	Medical Instruments & Equipment	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:08.198334	2020-10-11 23:38:31.02423
NYCB	New York Community Bancorp Inc.	New York Community Bancorp Inc has over $45 billion of total assets and operations spread across various locations across five states. It operates most of its business operations from the New York metropolitan area.	{"Financial Services","Savings & Cooperative Banks",Banks}	{WVFC,WAFD,TRST,GCBC,NWBI,BHLB,PBIP}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.207988	2020-10-11 23:38:31.023428
NYNY	Empire Resorts Inc.	Empire Resorts Inc is a casino operating company. It owns the Monticello casino and raceway in New York.	{"Consumer Cyclical","Resorts & Casinos","Travel & Leisure"}	{PENN,CNTY,CPHC,CHDN,DDE,UWN,CZR,FLL,ERI,GDEN}	Travel & Leisure	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:08.222202	2020-10-11 23:38:31.024353
NYMX	Nymox Pharmaceutical Corporation	Nymox Pharmaceutical Corp, along with its subsidiaries, is a biopharmaceutical corporation, engaged in research and development of drugs and diagnostic products for aging population. Its product portfolio consists of Nic alert and Tobac alert.	{Healthcare,Biotechnology}	{DGX,ABT,JNJ,LLY,PFE,SPHS,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:08.219735	2020-10-11 23:38:31.025871
NUW	Nuveen AMT-Free Municipal Value Fund	Nuveen AMT-Free Municipal Value Fund is a diversified, closed-end management investment company. Its investment objective is to provide current income exempt from regular federal income tax by investing in a portfolio of municipal obligations.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.099732	2020-10-11 23:38:31.028166
NVIV	InVivo Therapeutics Holdings Corp.	InVivo Therapeutics Holdings Corp is a research and clinical-stage biomaterials and biotechnology company with a focus on treatment of spinal cord injuries.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:08.120305	2020-10-11 23:38:31.031328
NWE	NorthWestern Corporation	NorthWestern Corp is a part of the utilities market sector. The primary occupation of the company involves the generation and transmission of electricity. It also provides natural gas.	{Utilities,"Utilities - Diversified","Utilities - Regulated"}	{PCG,NI,BKH,IDA,MDU,AVA,EXC,CNP,PNM,XEL,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:08.147805	2020-10-11 23:38:31.031328
NXC	Nuveen California Select Tax-Free Income Portfolio	Nuveen CA Select Tax Free Income Port is a diversified closed end management investment company. Its objective is to provide current income exempt from both regular federal and California state income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.168703	2020-10-11 23:38:31.031328
NXN	Nuveen New York Select Tax-Free Income Portfolio	Nuveen NY Select Tax-Free Income Port is a diversified closed end management investment company. Its investment objective is to provide current income and stable dividends, exempt from regular federal and designated state income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.178465	2020-10-11 23:38:31.023007
NXQ	Nuveen Select Tax Free Income Portfolio II	Nuveen Select Tax Free Income Port 2 is a diversified closed-end management investment company. It seeks to provide current income and stable dividends, exempt from regular federal and designated state income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.188935	2020-10-11 23:38:31.031328
NYF	iShares New York Muni Bond	The investment seeks to track the investment results of the S&P New York AMT-Free Municipal Bond IndexTM.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The index measures the performance of the investment-grade segment of the New York municipal bond market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.214221	2020-10-11 23:38:31.023007
NYMT	New York Mortgage Trust Inc.	New York Mortgage Trust Inc is a real estate investment trust. The Company is engaged in acquiring, investing in, financing and managing mortgage-related assets and, to a lesser extent, financial assets.	{"Real Estate","REIT - Residential",REITs}	{DX,CMO,MFA,IVR,PMT}	REITs	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:08.212776	2020-10-11 23:38:31.031328
NYT	New York Times Company (The)	New York Times Co is a publishing service provider. Its activities include creating, collecting and distributing news and information digitally and through newspapers.	{"Consumer Cyclical",Publishing}	{DJCO,LEE,AHC,PSO,NWSA,NEWM,TGNA,MNI,SSP}	Publishing	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:08.227227	2020-10-11 23:38:31.02423
NYV	Nuveen New York Municipal Value Fund 2 of Beneficial Interest	Nuveen NY Municipal Value Fund 2 is a diversified closed-end management investment company which seeks to provide current income exempt from both regular federal and New York state income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.227733	2020-10-11 23:38:31.023007
OAK	Oaktree Capital Group LLC Class A Units Representing Limited Liability Company Interests	Oaktree Capital Group LLC is an investment management company. It invests in distressed debt, corporate debt, control investing, convertible securities, real estate and listed equities.	{"Financial Services","Asset Management"}	{AMG,BLK,AB,BX,OZM,IVZ,GBL,APO,CG}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.243281	2020-10-11 23:38:31.031328
OBCI	Ocean Bio-Chem Inc.	Ocean Bio-Chem Inc is engaged in the manufacturing, marketing & distribution of appearance, performance and maintenance products for the marine, automotive, power sports, recreational vehicle and outdoor power equipment markets.	{"Consumer Defensive","Household & Personal Products","Consumer Packaged Goods"}	{CLX,CL}	Consumer Packaged Goods	Consumer Defensive	NASDAQ Capital Market	\N	2020-10-11 23:12:08.247475	2020-10-11 23:38:31.02423
OBAS	Optibase Ltd.	Optibase Ltd purchases and operates real estate properties. It currently holds properties in Switzerland and Miami Florida.	{"Real Estate","Real Estate Services"}	{GILT,SILC,EMITF,CRNT,GZT}	Real Estate Services	Real Estate	NASDAQ Global Market	\N	2020-10-11 23:12:08.244325	2020-10-11 23:38:31.023007
NWHM	The New Home Company Inc.	The New Home Co Inc is a homebuilding company. It is engaged in the design, construction and sale of consumer-driven homes in metropolitan areas within select growth markets in California and Arizona.	{"Consumer Cyclical","Residential Construction","Homebuilding & Construction"}	{PHM,NVR,DHI,HOV,LEN,MDC,KBH,WLH}	Homebuilding & Construction	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:08.149535	2020-10-11 23:38:31.028166
NXE	Nexgen Energy Ltd.	NexGen Energy Ltd is a mineral exploration company. It acquires, explores for uranium properties in Canada. Its projects includes Arrow, South Arrow, Harpoon, IsoEnergy, Rook I and others.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{NXE}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:08.169923	2020-10-11 23:38:31.028166
NXRT	NexPoint Residential Trust Inc.	NexPoint Residential Trust Inc focuses on acquiring multifamily properties in markets with attractive job growth and household formation fundamentals primarily in the Southeastern and Southwestern United States.	{"Real Estate","REIT - Residential",REITs}	{CBRE,AMT,FRPH,CHCT}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:08.192514	2020-10-11 23:38:31.02582
NXTDW	Nxt-ID (Warrant)	Nxt-ID, Inc. engages in the development of proprietary technology products and solutions. It focuses on security, healthcare, financial technology, and the Internet of Things markets. Its products and solutions include MobileBio, a suite of biometric solutions that secure consumers' mobile platforms; the Wocket, a next-generation smart wallet; and the Flye, a digital credit card developed in collaboration with WorldVentures. The company founded by Gino Miguel Pereira and David Charles Tunnell on February 8, 2012 and is headquartered Melbourne, FL.	{}	{}	Information Technology Services	Technology Services	NasdaqCM	\N	2020-10-11 23:12:08.196825	2020-10-11 23:38:31.028166
NYH	Eaton Vance New York Municipal Bond Fund II of Beneficial Interest $.01 par value	Eaton Vance New York Municipal Bond Fund II is a closed-end investment company. Its objective is to provide current income exempt from regular federal income tax, including alternative minimum tax, and, in state specific funds, tax in its specified state.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:08.211368	2020-10-11 23:38:31.021426
NYMTN	New York Mortgage Trust Inc. 8.00% Series D Fixed-to-Floating Rate Cumulative Redeemable Preferred Stock	New York Mortgage Trust Inc is a real estate investment trust. The Company is engaged in acquiring, investing in, financing and managing mortgage-related assets and, to a lesser extent, financial assets.	{"Real Estate","REIT - Residential",REITs}	{}	REITs	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:08.21551	2020-10-11 23:38:31.02582
O	Realty Income Corporation	Realty Income Corp owns, manages and leases over 5,000 properties across U.S and Puerto Rico. It operates a diversified portfolio of properties that includes commercial, manufacturing and distribution facilities.	{"Real Estate","REIT - Retail",REITs}	{MPW,PMT,COR,TWO}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:08.23264	2020-10-11 23:38:31.02582
OASI	O'Shares FTSE Asia Pacific Quality Dividend	The investment seeks to track the performance (before fees and expenses) of the FTSE Developed Asia Pacific Qual/Vol/Yield 5% Capped Factor Index.\n The underlying index is designed to reflect the performance of publicly-listed large capitalization and mid-capitalization dividend-paying issuers in the Asia-Pacific region exhibiting high quality, low volatility and high dividend yields, as determined by FTSE-Russell (the "index provider"). Under normal market conditions, the fund will invest at least 80% of its total assets in the components of the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.244304	2020-10-11 23:38:31.024353
OASM	Oasmia Pharmaceutical AB	Oasmia Pharmaceutical AB is a pharmaceutical company. It develops a new generation of drugs within human and veterinary oncology. It has two approved products: Paclical and Paccal Vet-CA1.	{Healthcare,"Drug Manufacturers - Major","Drug Manufacturers"}	{OMER,XLV}	Drug Manufacturers	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:08.243853	2020-10-11 23:38:31.025871
OBE	Obsidian Energy Ltd.	Obsidian Energy Ltd is a exploration and production company. It explores, develops and holds interests in oil and natural gas properties and related infrastructure in the Western Canada.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{CNQ,ECA,AAV,BTE,VET}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:08.247626	2020-10-11 23:38:31.028166
OBOR	KraneShares MSCI One Belt One Road Index	The investment seeks to track the MSCI Global China Infrastructure Exposure Index.\n The fund will invest at least 80% of its total assets in components of the index, depositary receipts, including ADRs, representing such components and securities underlying depositary receipts in the index. The index is designed to track the equity market performance of companies determined by the provider of the index provider to have high revenue exposure to Chinese infrastructure development in industries relevant to the Chinese Government's "One Belt, One Road" initiative. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.25846	2020-10-11 23:38:31.028166
MYO	Myomo Inc.	Myomo Inc is a medical robotics company that offers expanded mobility for those suffering from neurological disorders and upper-limb paralysis. It develops and markets the MyoPro product line of lightweight, non-invasive, powered arm braces.	{Healthcare,"Medical Devices"}	{PDCO,XLV}	Medical Devices	Healthcare	NYSE American	\N	2020-10-11 23:12:07.461501	2020-10-11 23:38:31.023007
NAN	Nuveen New York Quality Municipal Income Fund	Nuveen New York Quality Municipal Income Fund is a diversified closed end management investment company. Its investment objective is to provide current income exempt from both regular federal and New York state income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.496808	2020-10-11 23:38:31.02423
NAV	Navistar International Corporation	Navistar International Corp is a holding company whose individual units provide integrated and best in class transportation solutions. It also provides retail, wholesale and lease financing services for trucks and parts.	{Industrials,"Truck Manufacturing"}	{PCAR,SPAR,CAT,CMI,OSK,WNC,F,CNHI}	Truck Manufacturing	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:07.535783	2020-10-11 23:38:31.02582
NBR	Nabors Industries Ltd.	Nabors Industries Ltd owns and operates land-based drilling rig fleet. It provides offshore platform workover and drilling rigs in the United States and international markets.	{Energy,"Oil & Gas Drilling","Oil & Gas - Drilling"}	{PDS,PTEN,HP,UNT,PES,NE}	Oil & Gas - Drilling	Energy	New York Stock Exchange	\N	2020-10-11 23:12:07.569353	2020-10-11 23:38:31.02423
NCNA	NuCana plc	NuCana PLC is a clinical-stage biopharmaceutical company. It is focused on improving treatment outcomes for cancer patients by applying its ProTide technology.	{Healthcare,Biotechnology}	{PDLI,PIRS,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:07.597083	2020-10-11 23:38:31.031328
NCZ	AllianzGI Convertible & Income Fund II of Beneficial Interest	AllianzGI Convertible & Income Fund II is a diversified, closed-end management investment company. Its investment objective is to provide total return through a combination of capital appreciation and high current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.621429	2020-10-11 23:38:31.024353
NEN	New England Realty Associates Limited Partnership Class A Depositary Receipts Evidencing Units of Limited Partnership	New England Realty Associates LP is engaged in the business of acquiring, developing, holding for investment, operating and selling real estate.	{"Real Estate","Real Estate - General","Real Estate Services"}	{INTG,MAYS,TCI,IOR,CTO,ARL,HHC,AHH}	Real Estate Services	Real Estate	NYSE American	\N	2020-10-11 23:12:07.649892	2020-10-11 23:38:31.023007
NESRW	National Energy Services Reunited Corp. Warrant		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:07.684772	2020-10-11 23:38:31.02423
NEXA	Nexa Resources S.A.	Nexa Resources SA is a metals and mining company which is engaged in zinc content production. Its product portfolio also includes copper, lead, silver, and gold.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{RTH}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:07.707743	2020-10-11 23:38:31.02423
NFJ	AllianzGI NFJ Dividend Interest & Premium Strategy Fund	Allianzgi NFJ Dividend, Interest & Premium Strategy Fund is a Massachusetts based business trust. Its primary investment objective is to seek current income and gains, with a secondary objective of long-term capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.723227	2020-10-11 23:38:31.02582
NGG	National Grid Transco PLC PLC (NEW) American Depositary Shares	National Grid PLC transmits and distributes electricity and gas. It owns regulated transmission and electricity generation facilities in the United Kingdom and United States.	{Utilities,"Utilities - Diversified","Utilities - Regulated"}	{TRP,DTE,NI,ED,SRE,SCG,AGR,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:07.739687	2020-10-11 23:38:31.02582
NHC	National HealthCare Corporation	National Healthcare Corp is engaged in the operation of skilled nursing facilities with associated assisted living and independent living facilities. It provides sub-acute and post-acute skilled nursing care, senior living and home health care services.	{Healthcare,"Long-Term Care Facilities","Health Care Providers"}	{ENSG,CSU,DVCR,GEN,FVE,RHE,XLV}	Health Care Providers	Healthcare	NYSE American	\N	2020-10-11 23:12:07.780505	2020-10-11 23:38:31.028166
NICK	Nicholas Financial Inc.	Nicholas Financial Inc is a specialized consumer finance company engaged in acquiring and servicing automobile finance installment contracts for purchases of new and used automobiles and light trucks.	{"Financial Services","Credit Services"}	{CPSS,CACC,AXP,SC}	Credit Services	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:07.799257	2020-10-11 23:38:31.028166
NITE	Nightstar Therapeutics plc	Nightstar Therapeutics Ltd is a clinical-stage gene therapy company. It is focused on developing and commercializing a pipeline of novel and potentially curative, one-time retinal gene therapies for patients suffering from rare inherited retinal diseases.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:07.814214	2020-10-11 23:38:31.02582
NKTR	Nektar Therapeutics	Nektar Therapeutics Inc is a biopharmaceutical company engaged in the development of new molecules to treat diseases with unmet medical needs. The company is researching and developing drugs for treating cancer, auto-immune disease and chronic pain.	{Healthcare,Biotechnology}	{AMGN,RELL,AZN,NVS,BAX,JNJ,PFE,MRK,ALKS,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:07.830386	2020-10-11 23:38:31.023007
OCUL	Ocular Therapeutix Inc.	Ocular Therapeutix Inc is a biopharmaceutical company. It is engaged in developing and commercializing therapies for diseases and conditions of the eye using its proprietary hydrogel platform technology.	{Healthcare,Biotechnology}	{ANIK,ARDM,XENT,KALV,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:08.282734	2020-10-11 23:38:31.024353
OEF	iShares S&P 100	The investment seeks to track the investment results of the S&P 100®, which measures the performance of the large-capitalization sector of the U.S. equity market.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, including shares of money market funds advised by BFA or its affiliates, as well as in securities not included in the underlying index, but which the advisor believes will help the fund track the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.302547	2020-10-11 23:38:31.024353
OFS	OFS Capital Corporation	OFS Capital Corp is an externally managed, closed-end, non-diversified management investment company. Its investment objective is to provide its shareholders with both current income and capital appreciation through debt investments.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:08.325349	2020-10-11 23:38:31.028166
OHI	Omega Healthcare Investors Inc.	Omega Healthcare Investors Inc is a self-administered real estate investment trust. It is engaged in investing in income-producing healthcare facilities, including long-term care facilities located in the United States and United Kingdom.	{"Real Estate","REIT - Healthcare Facilities",REITs}	{}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:08.349062	2020-10-11 23:38:31.02423
OILK	ProShares K-1 Free Crude Oil Strategy	The investment seeks to provide total return through actively managed exposure to the West Texas Intermediate crude oil futures markets.\n The fund's strategy seeks to outperform certain index based strategies by actively managing the rolling of WTI crude oil futures contracts. The fund generally will not invest directly in WTI crude oil futures. The advisor expects to gain exposure to these investments by investing a portion of its assets in the ProShares Cayman Crude Oil Strategy Portfolio, a wholly-owned subsidiary of the fund organized under the laws of the Cayman Islands. It is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:08.375924	2020-10-11 23:38:31.028166
OMEX	Odyssey Marine Exploration Inc.	Odyssey Marine Exploration Inc is engaged in deep-ocean exploration. The Company's techniques are applied to shipwreck cargo recovery and other marine survey and exploration charter services.	{Industrials,"Business Services"}	{ICFI,BBW,XON}	Business Services	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:08.42029	2020-10-11 23:38:31.023007
ONDK	On Deck Capital Inc.	On Deck Capital Inc along with its subsidiaries is engaged in providing financing products to small businesses located throughout the United States, including term loans and lines of credit.	{"Financial Services","Credit Services"}	{NNI,RM,NAVI}	Credit Services	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.462352	2020-10-11 23:38:31.023428
OPB	Opus Bank	Opus Bank provides banking products, services and solutions to small and medium-sized companies, entrepreneurs, real estate investors, professionals and high net worth individuals.	{"Financial Services","Banks - Regional - US",Banks}	{CWBC,CATY,EWBC,PLBC,PPBI,SIVB,HAFC}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:08.482287	2020-10-11 23:38:31.028166
OPP	RiverNorth/DoubleLine Strategic Opportunity Fund Inc.	RiverNorth/DoubleLine Strategic Opportunity Fund Inc is a closed-end management investment company. The company's investment objective is to seek current income and overall total return.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.501459	2020-10-11 23:38:31.021426
ORBK	Orbotech Ltd.	Orbotech Ltd is a scientific equipment manufacturer. It mainly manufactures testing devices that cater to the electronic, solar and recognition software market segments.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{AMAT,COHU,ESIO,NANO,NVMI,CAMT,A,TER,RTEC,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:08.519045	2020-10-11 23:38:31.02423
ORIT	Oritani Financial Corp.	Oritani Financial Corp is the stock holding company for Oritani Bank. It operates as a retail and commercial bank and provides banking and financial services to individual and business customers in New Jersey.	{"Financial Services","Banks - Regional - US",Banks}	{OCFC,MGYR,NYCB,PFS,KRNY,MSBF}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:08.534481	2020-10-11 23:38:31.02582
OSBC	Old Second Bancorp Inc.	Old Second Bancorp Inc is a bank holding company. It provides consumer and commercial banking products such as demand, money market, savings, time deposit and also commercial, industrial, consumer and real estate lending services.	{"Financial Services","Banks - Regional - US",Banks}	{WTFC,FMBI,MBFI,ASB,BUSE}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:08.555169	2020-10-11 23:38:31.028166
OSN	Ossen Innovation Co. Ltd.	Ossen Innovation Co Ltd, formerly known as San Meditech Holdings Ltd designs, manufactures, sells and markets customised prestressed concrete materials. Its materials are used in construction of bridges, highways & other infrastructure projects.	{"Basic Materials",Steel}	{}	Steel	Basic Materials	NASDAQ Capital Market	\N	2020-10-11 23:12:08.582516	2020-10-11 23:38:31.024353
OFED	Oconee Federal Financial Corp.	Oconee Federal Financial Corp is engaged in attracting retail deposits from the general public and investing those deposits together with funds generated from operations in residential mortgage loans and other loans.	{"Financial Services","Savings & Cooperative Banks",Banks}	{KFFB,HTBI}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:08.316678	2020-10-11 23:38:31.023007
OGS	ONE Gas Inc.	ONE Gas Inc is engaged in the business of natural gas distribution. The Company serves residential, commercial, industrial, transportation and wholesale and public authority customers in Oklahoma, Kansas, and Texas.	{Utilities,"Utilities - Regulated Gas","Utilities - Regulated"}	{RGCO,ATO,CNP,SR,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:08.335664	2020-10-11 23:38:31.028166
OIA	Invesco Municipal Income Opportunities Trust	To provide a high level of current income which is exempt from federal income tax. It mainly invests in better-than-average quality issues within the high-yield sector of the municipal market.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.357523	2020-10-11 23:38:31.025871
OIS	Oil States International Inc.	Oil States International Inc provides products and services to oil and natural gas companies. It operates in oil and natural gas producing regions, including U.S., Canada, West Africa, North Sea, South America and Southeast and Central Asia.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{SPN,NOV,DRQ,GE,NR,BHGE}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:08.380435	2020-10-11 23:38:31.023428
OLLI	Ollie's Bargain Outlet Holdings Inc.	Ollie's Bargain Outlet Holdings Inc is a retailer of brand name merchandise. It offers customers a selection of brand name products, including housewares, food, books and stationery, bed and bath, floor coverings, toys and hardware at reduced prices.	{"Consumer Defensive","Discount Stores","Retail - Defensive"}	{DDS,JCP,M}	Retail - Defensive	Consumer Defensive	NASDAQ Global Market	\N	2020-10-11 23:12:08.400477	2020-10-11 23:38:31.021426
OMCL	Omnicell Inc.	Omnicell Inc provides automation and business analytics software solutions for patient-centric medication and supplies management across the entire healthcare continuum, from the acute care hospital setting to post-acute skilled nursing.	{Technology,"Health Information Services","Application Software"}	{CERN,CAH,IVC,MCK,ABC,PINC,STE,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:08.413692	2020-10-11 23:38:31.023428
OMF	OneMain Holdings Inc.	OneMain Holdings Inc is a consumer finance company that provides loan products to non-prime customers.	{"Financial Services","Credit Services"}	{WRLD,RM}	Credit Services	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.424789	2020-10-11 23:38:31.021426
OMI	Owens & Minor Inc.	Owens & Minor Inc is a healthcare logistics firm distributing low-tech, consumable medical supplies to acute-care hospitals. The company distributes products to healthcare service providers under various brands such as MediChoice and ArcRoyal.	{Healthcare,"Medical Distribution"}	{HSIC,PDCO,CAH,GE,JNJ,TFX,MCK,ABC,PINC,XLV}	Medical Distribution	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:08.441651	2020-10-11 23:38:31.024353
OMOM	Oppenheimer Russell 1000 Momentum Factor	The investment seeks to provide investment results that correspond generally, before fees and expenses, to the performance of the Russell 1000 Momentum Factor Index (the "underlying index").\n Under normal circumstances, the fund will invest at least 80% of its net assets, plus any borrowings for investment purposes, in securities of companies included in the underlying index. The underlying index is constructed using a rules-based methodology that selects a subset of the equity securities from the Russell 1000 Index (the "parent index"), which is comprised of the 1,000 largest-capitalization companies in the United States.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:08.445689	2020-10-11 23:38:31.02582
ONEY	SPDR Russell 1000 Yield Focus	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the Russell 1000 Yield Focused Factor Index.\n Under normal market conditions, the fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index is designed to reflect the performance of a segment of large-capitalization U.S. equity securities demonstrating a combination of core factors (high value, high quality, and low size characteristics), with a focus factor comprising high yield characteristics (the "Factor Characteristics"). It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.469839	2020-10-11 23:38:31.023007
ONTL	O'Shares FTSE Russell International Quality Dividend	The investment seeks to track the performance (before fees and expenses) of the FTSE Developed ex US Qual/Vol/Yield Factor 5% Capped Index (the "target index").\n Under normal market conditions, the fund will invest at least 80% of its total assets in the components of the target index and in depositary receipts representing such securities. The target index is designed to reflect the performance of publicly-listed large capitalization and mid-capitalization dividend-paying international issuers exhibiting high quality, low volatility and high dividend yields, as determined by FTSE-Russell (the "index provider").	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.473732	2020-10-11 23:38:31.031328
ODT	Odonate Therapeutics Inc.	Odonate Therapeutics Inc is a pharmaceutical company which is engaged in the development of therapeutics which improves and extend the lives of patients with cancer. It is developing tesetaxel an chemotherapy agent used in the treatment of cancer.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:08.294145	2020-10-11 23:38:31.028166
OFIX	Orthofix Medical Inc.	Orthofix International NV is a medical device company. It focuses on providing reconstructive and regenerative orthopedic and spine solutions to physicians.	{Healthcare,"Medical Devices"}	{XLV}	Medical Devices	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:08.326307	2020-10-11 23:38:31.025871
OI	Owens-Illinois Inc.	Owens-Illinois Inc is a glass container manufacturer. The company is also a preferred partner for various food and beverage brands. It provides glass packaging for beer, wine, spirits, food, non-alcoholic beverages, cosmetics and pharmaceuticals.	{"Consumer Cyclical","Packaging & Containers"}	{SLGN,ATR,BLL,CCK,TAP,PEP}	Packaging & Containers	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:08.356678	2020-10-11 23:38:31.02582
OKE	ONEOK Inc.	ONEOK Inc is a diversified energy company. Its business operations are divided into Natural Gas Gathering and Processing, Natural Gas Liquids and Natural Gas Pipelines.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{SRE,ATO,NFG,SWX,UGI,EQT,WES}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:08.386087	2020-10-11 23:38:31.024353
OLN	Olin Corporation	Olin Corp is manufactures and sells chlor alkali products. It is engaged in three segments Chlor Alkali Products, Epoxy and Winchester.	{"Basic Materials","Specialty Chemicals",Chemicals}	{CBT,MTX,PPG,SCL,WLK}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:08.405596	2020-10-11 23:38:31.024353
ONCE	Spark Therapeutics Inc.	Spark Therapeutics Inc is a gene therapy company. The company is engaged in developing one-time, life-altering gene therapy treatments to transform the lives of patients and re-imagine the treatment of debilitating diseases.	{Healthcare,Biotechnology}	{IMMU,ILMN,HBIO,FGEN,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:08.456086	2020-10-11 23:38:31.021426
ONTXW	Onconova Therapeutics Inc. Warrants		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:08.478363	2020-10-11 23:38:31.02582
OPGNW	OpGen (Warrant)	OpGen, Inc. operates as a precision medicine company using molecular diagnostics and informatics to help combat infectious disease. It is developing molecular information products and services for global healthcare settings, helping to guide clinicians with more rapid and actionable information about life threatening infections, improve patient outcomes, and decrease the spread of infections caused by multidrug-resistant microorganisms, or MDROs. The company was founded on January 22, 2001 and is headquartered in Gaithersburg, MD.	{}	{}	Biotechnology	Health Technology	NasdaqCM	\N	2020-10-11 23:12:08.48785	2020-10-11 23:38:31.023007
OPOF	Old Point Financial Corporation	Old Point Financial Corp provides banking products and solutions to its customers in the United States. Its primary occupation is the provision of loan, deposit and cash management services to retail and commercial customers.	{"Financial Services","Banks - Regional - US",Banks}	{AMNB,NKSH,FBSS,TOWN,BOTJ,ESXB}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:08.496248	2020-10-11 23:38:31.031328
OPTT	Ocean Power Technologies Inc.	Ocean Power Technologies Inc is engaged in developing systems that generate electricity by harnessing the renewable energy of ocean waves.	{Industrials,"Diversified Industrials","Industrial Products"}	{WWD,LMT,GE,BWEN,CPST}	Industrial Products	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:08.506782	2020-10-11 23:38:31.02423
OQAL	Oppenheimer Russell 1000 Quality Factor	The investment seeks to provide investment results that correspond generally, before fees and expenses, to the performance of the Russell 1000 Quality Factor Index (the "underlying index").\n Under normal circumstances, the fund will invest at least 80% of its net assets, plus any borrowings for investment purposes, in securities of companies included in the underlying index. The underlying index is constructed using a rules-based methodology that selects a subset of the equity securities from the Russell 1000 Index (the "parent index"), which is comprised of the 1,000 largest-capitalization companies in the United States.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:08.515137	2020-10-11 23:38:31.023007
ORCL	Oracle Corporation	Oracle Corp is a computer software company. It sells enterprise information technology solutions, including databases, middleware, applications, and hardware. The Company primarily offers software licenses, support, and maintenance services worldwide.	{Technology,"Software - Infrastructure","Application Software"}	{MSFT,INTU,CTXS,ADBE,QCOM,SYMC,SAP,RHT,CRM,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:08.531955	2020-10-11 23:38:31.028166
ORIG	Ocean Rig UDW Inc.	Ocean Rig UDW Inc is an offshore drilling contractor providing oilfield services for offshore oil and gas exploration, development and production drilling and specializing in the ultra-deepwater and harsh-environment segment.	{Energy,"Oil & Gas Drilling","Oil & Gas - Drilling"}	{DO,E,DRQ,RIG,RDC,ATH,ESV,DRYS}	Oil & Gas - Drilling	Energy	Nasdaq Global Select	\N	2020-10-11 23:12:08.534571	2020-10-11 23:38:31.031328
OSBCP	Old Second Bancorp Inc. 7.80% Cumulative Trust Preferred Securities	Old Second Capital Trust I is a statutory trust.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:08.555073	2020-10-11 23:38:31.023428
OGCP	Empire State Realty OP L.P. Series 60 Operating Partnership Units Representing Limited Partnership Interests	Empire State Realty OP LP owns, operates, acquires and repositions properties such as Empire State Building, One Grand Central Place in Manhattan and the greater New York metropolitan area. It operates in two segments, Real Estate and Observatory.	{"Real Estate","REIT - Diversified",REITs}	{}	REITs	Real Estate	NYSE Arca	\N	2020-10-11 23:12:08.330487	2020-10-11 23:38:31.021426
OHRP	Ohr Pharmaceutical Inc.	Ohr Pharmaceutical Inc is a pharmaceutical company engaged in development of novel therapeutics and delivery technologies for the treatment of ocular disease.	{Healthcare,Biotechnology}	{REGN,JNJ,PFE,OPHT,IONS,ARWR,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:08.356907	2020-10-11 23:38:31.028166
OKTA	Okta Inc.	Okta Inc is an identity and access management company. Its products include Universal Directory,Single Sign-On, Adaptive Multi-Factor Authentication, Lifecycle Management, Mobility Management, and API Access Management.	{Technology,"Software - Infrastructure","Application Software"}	{PRGS,PXLW,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:08.387204	2020-10-11 23:38:31.02582
OMED	OncoMed Pharmaceuticals Inc.	OncoMed Pharmaceuticals Inc is a clinical development-stage biopharmaceutical company focused on discovering and developing novel anti-cancer stem cell and immuno-oncology product candidates.	{Healthcare,Biotechnology}	{CELG,REGN,ECYT,GSK,VSTM,STML,XLRN,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:08.416087	2020-10-11 23:38:31.025871
ONCS	OncoSec Medical Incorporated	OncoSec Medical Inc is a biotechnology company engaged in designing, developing and commercializing gene therapies, therapeutics and proprietary medical approaches to stimulate and guide an anti-tumor immune response for the treatment of cancer.	{Healthcare,Biotechnology}	{IMMU,CASS,ZIOP,INO,EIGR,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:08.455915	2020-10-11 23:38:31.028166
ONTX	Onconova Therapeutics Inc.	Onconova Therapeutics Inc is a biopharmaceutical company focusing on discovering and developing novel small molecule product candidates primarily to treat cancer.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:08.472713	2020-10-11 23:38:31.025871
OPK	Opko Health Inc.	OPKO Health Inc is a diversified healthcare company. Its business includes diagnostics and provision of pharmaceutical products.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{REGN,ALNY,ABT,BMY,JNJ,LH,TEVA,TSRO,ARDX,NVLN,XLV}	Drug Manufacturers	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:08.495391	2020-10-11 23:38:31.024353
OPTN	OptiNose Inc.	OptiNose Inc is a specialty pharmaceutical company focused on the development and commercialization of products for patients treated by ear, nose and throat (ENT) and allergy specialists.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{OPTN,XLV}	Drug Manufacturers	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:08.50413	2020-10-11 23:38:31.023428
ORBC	ORBCOMM Inc.	ORBCOMM Inc provides machine-to-machine communication, which is designed to track, monitor, and control fixed and mobile assets. It operates in various markets such as transportation and distribution, heavy equipment, and industrial fixed assets.	{"Communication Services","Telecom Services"}	{IDSY,IRDM,VZ,T,TDS,GOGO,GSAT}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:12:08.527941	2020-10-11 23:38:31.024353
ORM	Owens Realty Mortgage Inc.	Owens Realty Mortgage Inc is a specialty finance company which focuses on the origination, investment, and management of commercial real estate loans. It provides customized, short-term loans to small and middle-market investors and developers.	{"Real Estate","REIT - Office",REITs}	{STWD,CMCT}	REITs	Real Estate	NYSE American	\N	2020-10-11 23:12:08.546489	2020-10-11 23:38:31.028166
OSPRU	Osprey Energy Acquisition Corp Unit 1 Cl A Com & 1/2 Wt Exp 05/01/2022	Osprey Energy Acquisition Corp is a newly organized blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:08.582296	2020-10-11 23:38:31.028166
OSK	Oshkosh Corporation (Holding Company)Common Stock	Oshkosh Corp designs, manufactures and markets specialty vehicles and aerial work platforms under the JLG brand. It serves customers in the private and public sectors.	{Industrials,"Truck Manufacturing"}	{SPAR,NAV,TEX,CAT,FSS,AXL,WBC,SRI,MLR}	Truck Manufacturing	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:08.576776	2020-10-11 23:38:31.031328
OTIC	Otonomy Inc.	Otonomy Inc is a biopharmaceutical company, focuses on the development and commercialization of therapeutics for diseases and disorders of the ear.	{Healthcare,Biotechnology}	{AMAG,ADXS,AGTC,AKBA,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:08.607778	2020-10-11 23:38:31.031328
OUNZ	VanEck Merk Gold Trust	The Trust’s primary objective is to provide investors with an opportunity to invest in gold through the shares and be able to take delivery of physical gold in exchange for those shares. The Trust’s secondary objective is for the shares to reflect the performance of the price of gold less the expenses of the Trust’s operations. Each share represents a fractional undivided beneficial interest in the Trust’s net assets. The Trust’s assets consist principally of gold held on the Trust’s behalf in financial institutions for safekeeping.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.611186	2020-10-11 23:38:31.025871
OXBRW	Oxbridge Re Holdings Limited Warrant		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:08.642821	2020-10-11 23:38:31.023007
OCSLL	Oaktree Specialty Lending Corporation 6.125% senior notes due 2028		{}	{}			Nasdaq Global Select	\N	2020-10-11 23:12:08.281232	2020-10-11 23:38:31.021426
OFG	OFG Bancorp	OFG Bancorp through its subsidiaries provides commercial, consumer and mortgage lending, leasing, auto loans, financial planning, insurance sales, money management and investment banking, and corporate and individual trust services.	{"Financial Services","Banks - Regional - US",Banks}	{BOKF,HWC,RNST,CFR,WFC,USB,SGB,FBP,BPOP,BXS}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.315831	2020-10-11 23:38:31.031328
OIH	VanEck Vectors Oil Services	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the MVIS® US Listed Oil Services 25 Index.\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The index includes common stocks and depositary receipts of U.S. exchange-listed companies in the oil services sector. Such companies may include small- and medium-capitalization companies and foreign companies that are listed on a U.S. exchange. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.358674	2020-10-11 23:38:31.024353
OILU	ProShares UltraPro 3x Crude Oil	The investment seeks to return a multiple (3x) of the performance of the Bloomberg WTI Crude Oil Subindex for a single day.\nThe fund seeks to meet its investment objective by investing, under normal market conditions, in futures contracts for WTI sweet, light crude oil listed on the NYMEX, ICE Futures U.S. or other U.S. exchanges and listed options on such contracts.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.375934	2020-10-11 23:38:31.02423
OLED	Universal Display Corporation	Universal Display Corp is engaged in the research, development & commercialization of organic light emitting diode or OLED, technologies & materials for use in displays for wearables, smartphones, tablets & televisions & solid-state lighting applications.	{Technology,"Semiconductor Equipment & Materials",Semiconductors}	{KOPN,SNE,PHG,AUO,LPL,MVIS,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:08.398577	2020-10-11 23:38:31.02423
OMFL	Oppenheimer Russell 1000 Dynamic Multifactor	The investment seeks to provide investment results that correspond generally, before fees and expenses, to the performance of the Russell 1000 OFI Dynamic Multifactor Index.\n Under normal circumstances, the fund will invest at least 80% of its net assets, plus any borrowings for investment purposes, in securities of companies included in the underlying index. The underlying index is constructed using a rules-based methodology by selecting equity securities from the Russell 1000 Index (the "parent index"), which measures the performance of the 1,000 largest-capitalization companies in the United States.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:08.429199	2020-10-11 23:38:31.02423
ON	ON Semiconductor Corporation	ON Semiconductor Corp manufactures and markets semiconductor components. It provides a portfolio of analog, digital and mixed-signal integrated circuits, standard products, image sensors, custom devices, and power management and others.	{Technology,Semiconductors}	{DIOD,MXIM,SMTC,STM,VSH,TXN,ADI,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:08.451333	2020-10-11 23:38:31.023007
OOMA	Ooma Inc.	Ooma Inc provides communication solutions and other connected services to small business, home, and mobile users. The company's hybrid SaaS platform, consisting of its proprietary cloud provides the connectivity and functionality that power its solutions.	{"Communication Services","Telecom Services"}	{NTGR}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:08.483264	2020-10-11 23:38:31.023428
OR	Osisko Gold Royalties Ltd	Osisko Gold Royalties Ltd is engaged in acquiring and managing precious metal and other royalties and similar interests in Canada and other parts of the world. The company's projects include Canadian Malartic 5% NSR and Eleonore 2-3.5% NSR.	{"Basic Materials",Gold,"Metals & Mining"}	{}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:08.515246	2020-10-11 23:38:31.02582
ORG	The Organics ETF	The investment seeks investment results that correspond generally to the performance, before fees and expenses, of an index which is designed to track the performance of companies globally that are positioned to profit from increasing demand for organic products, including companies which service, produce, distribute, market or sell organic food, beverages, cosmetics, supplements, or packaging.\n The fund invests at least 80% of its net assets in the stocks that comprise the Solactive Organics Index. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:08.530797	2020-10-11 23:38:31.023007
ORPN	Bioblast Pharma Ltd.	Bioblast Pharma Ltd is a clinical stage biotechnology company. It is engaged in the identification, licensing, acquisition, development and commercialization of drugs for rare and ultra-rare genetic and metabolic diseases.	{Healthcare,Biotechnology}	{GILD,KERX,CASI,AUPH,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:08.548406	2020-10-11 23:38:31.025871
OSG	Overseas Shipholding Group Inc. Class A	Overseas Shipholding Group Inc through its subsidiaries owns and operates a fleet of oceangoing vessels engaged in the transportation of crude oil and petroleum products for oil companies and traders under short- and medium-term contracts.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{NM,NMM}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:08.559809	2020-10-11 23:38:31.02582
OSIS	OSI Systems Inc.	OSI Systems Inc, together with its subsidiaries, is a designer and manufacturer of electronic systems and components for critical applications. The Company's business segments are Security, Healthcare, and Optoelectronics and Manufacturing.	{Technology,"Electronic Components","Computer Hardware"}	{CAH,A,GE,PHG,LDOS,VIAV,LLL,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:08.56921	2020-10-11 23:38:31.028166
OFLX	Omega Flex Inc.	Omega Flex Inc is a US based company engaged in the manufaturing of flexible metal hose.	{Industrials,"Diversified Industrials","Industrial Products"}	{NWPX,SNHY,PKOH,CR,PH,ATU}	Industrial Products	Industrials	NASDAQ Global Market	\N	2020-10-11 23:12:08.32349	2020-10-11 23:38:31.02582
OHGI	One Horizon Group Inc.	One Horizon Group Inc is a media and digital technology company. It has proprietary digitally secure messaging software and sells licenses primarily into the gaming, security and educational fields.	{Technology,"Software - Application","Application Software"}	{MTD,FTR,TEUM,XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:08.347003	2020-10-11 23:38:31.031328
OIIM	O2Micro International Limited	O2Micro International Ltd is engaged in designing, developing and marketing high performance integrated circuits and solutions for manufacturers of products in the consumer electronics, computer, industrial, communications, and automotive markets.	{Technology,"Electronic Components","Computer Hardware"}	{MXIM,MPWR,TXN,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:08.369551	2020-10-11 23:38:31.021426
OLP	One Liberty Properties Inc.	One Liberty Properties Inc is a self-administered and self-managed real estate investment trust. It acquires, owns and manages a geographically diversified portfolio of retail, industrial, flex, health and fitness, and other properties.	{"Real Estate","REIT - Diversified",REITs}	{BRT,PMT,BXMT}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:08.413241	2020-10-11 23:38:31.028166
OMN	OMNOVA Solutions Inc.	Omnova Solutions Inc is engaged in producing emulsion polymers, speciality chemicals, and engineered surfaces for commercial, industrial, and residential end uses.	{"Basic Materials","Specialty Chemicals",Chemicals}	{MX,FOE,MAS,PPG,RPM,POL,CE,TSE,DWDP}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:08.43995	2020-10-11 23:38:31.028166
ONEO	SPDR Russell 1000 Momentum Focus	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the Russell 1000 Momentum Focused Factor Index.\n Under normal market conditions, the fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index is designed to reflect the performance of a segment of large-capitalization U.S. equity securities demonstrating a combination of core factors (high value, high quality, and low size characteristics), with a focus factor comprising high momentum characteristics. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.466238	2020-10-11 23:38:31.024353
OPHC	OptimumBank Holdings Inc.	OptimumBank Holdings Inc is a bank holding company for OptimumBank. The company through its subsidiaries offers banking products and services to individuals and businesses.	{"Financial Services","Banks - Regional - US",Banks}	{CCBG,CSFL,BBT,STI,BKU,SBCF}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:08.493112	2020-10-11 23:38:31.02582
ORAN	Orange	Orange SA provides consumers, businesses and other telecommunications operators with a variety of services including fixed telephony and mobile telecommunications, data transmission and value-added services, mainly in Europe, Africa and the Middle East.	{"Communication Services","Telecom Services"}	{MSFT,TEF,MBT,BT,VOD}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:08.516289	2020-10-11 23:38:31.028166
ORLY	O'Reilly Automotive Inc.	O'Reilly Automotive Inc acts as a seller of aftermarket automotive parts, tools, and accessories, serving professional and DIY (do-it-yourself) customers in the US.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{MNRO,LKQ,PRTS,AZO,GPC,AAP}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:08.545132	2020-10-11 23:38:31.021426
OSS	One Stop Systems Inc.	One Stop Systems Inc designs, manufactures and markets high-end systems for high performance computing applications. Its products are used by automated equipment used for telecommunications, industrial and military applications.	{Technology,"Computer Systems","Computer Hardware"}	{XLK}	Computer Hardware	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:08.583743	2020-10-11 23:38:31.023007
OSTK	Overstock.com Inc.	Overstock.com Inc is an online retailer which provides products and services through websites. It offers a broad range of products, including furniture and home decor, apparel and accessories, books, electronics, and other items.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{AMZN,EBAY,ROST,PCMI,TJX,WMT,TGT,EVLV,GAIA}	Retail - Apparel & Specialty	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:08.583426	2020-10-11 23:38:31.02423
OUSA	O'Shares FTSE U.S. Quality Dividend	The investment seeks to track the performance (before fees and expenses) of the FTSE USA Qual/Vol/Yield Factor 5% Capped Index (the "target index").\n The target index is designed to reflect the performance of publicly-listed large capitalization and mid-capitalization dividend-paying issuers in the United States exhibiting high quality, low volatility and high dividend yields, as determined by FTSE-Russell (the "index provider"). Under normal market conditions, the fund will invest at least 80% of its total assets in the components of the target index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.615235	2020-10-11 23:38:31.024353
OCX	OncoCyte Corporation	OncoCyte Corp focuses on the discovery, development, and commercialization of novel, non-invasive, liquid biopsy diagnostics for the early detection of cancer.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	NYSE American	\N	2020-10-11 23:12:08.282427	2020-10-11 23:38:31.02423
OHAI	OHA Investment Corporation	OHA Investment Corp is a closed-end, non-diversified specialty finance company. It focuses primarily on providing creative direct lending solutions to middle market private companies across industry sectors.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:08.342151	2020-10-11 23:38:31.023428
OII	Oceaneering International Inc.	Oceaneering International Inc is an oil & gas enterprise. It provides engineered services and products to the offshore oil and gas industry, with a focus on deepwater applications.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{SPN,HAL,TTI,HLX,FET}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:08.366453	2020-10-11 23:38:31.023007
OLBK	Old Line Bancshares Inc.	Old Line Bancshares Inc is a bank holding company. The firm through its subsidiary provides general banking and financial services to small and medium-sized businesses, entrepreneurs, professionals, consumers and high net worth clients.	{"Financial Services","Banks - Regional - US",Banks}	{GLBZ,SASR,ORRF,SHBI,EGBN,FUNC,HBMD}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:08.388026	2020-10-11 23:38:31.028166
OMAB	Grupo Aeroportuario del Centro Norte S.A.B. de C.V.	Grupo Aeroportuario del Centro Norte SAB de CV through its subsidiaries, holds concessions to operate, maintain and develop 13 airports in Mexico. It provides aeronautical services related to use of its airport facilities by airlines and passengers.	{Industrials,"Airports & Air Services","Transportation & Logistics"}	{ASR,PAC}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:08.414408	2020-10-11 23:38:31.028166
OMNT	Ominto Inc.	Ominto Inc is a E-commerce and network marketing company. It operates an online shopping portal, through which its customers search for & purchase products offered by various online stores, including consumer products, travel related-products & services.	{}	{AMZN,CNXN,PCMI,ATHN,EVLV}	Retail - Apparel & Specialty	Consumer Cyclical	OTC Pink	\N	2020-10-11 23:12:08.442538	2020-10-11 23:38:31.023428
OPHT	Ophthotech Corporation	Ophthotech Corporation, a biopharmaceutical company, develops novel therapeutics to treat diseases of the back of the eye. Its principal product candidate is called as Fovista.	{Healthcare,Biotechnology}	{REGN,NKTR,ALIM,INSM,OHRP,AERI,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:08.502428	2020-10-11 23:38:31.028166
ORN	Orion Group Holdings Inc. Common	Orion Group Holdings Inc is active in the business of marine construction, design and specialty services. It provides services such as marine transportation facility construction, marine environmental structures and commercial concrete construction.	{Industrials,"Engineering & Construction"}	{STRL,GLDD,PRIM,MTZ,GVA,TPC}	Engineering & Construction	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:08.551796	2020-10-11 23:38:31.02423
OSUR	OraSure Technologies Inc.	OraSure Technologies Inc develops, manufactures, markets and sells oral fluid diagnostic products and specimen collection devices using its proprietary oral fluid technologies.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{QDEL,TRIB,VRML,COO,DGX,BIO,CEMI,RIOT,XLV}	Medical Instruments & Equipment	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:08.594963	2020-10-11 23:38:31.02582
OTEL	Otelco Inc.	Otelco Inc is a telecommunications service provider, offering an array of communication services including telephone, long distance, Internet, broadband services and television.	{"Communication Services","Telecom Services"}	{ALSK,DISH,CTL,CHTR,CBB}	Communication Services	Communication Services	NASDAQ Capital Market	\N	2020-10-11 23:12:08.594184	2020-10-11 23:38:31.023428
OVBC	Ohio Valley Banc Corp.	Ohio Valley Banc Corporation is a financial holding company. It is engaged in commercial and retail banking. It is a full-service financial institution, offers commercial and consumer banking services in Ohio as well as western West Virginia.	{"Financial Services","Banks - Regional - US",Banks}	{PEBO,UBCP,FFBC,HBAN,WSBC,PRK,FMNB,SBFG,CIVB}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:08.626222	2020-10-11 23:38:31.023007
OUT	OUTFRONT Media Inc.	Outfront Media Inc is the United States-based real estate investment trust. It provides advertising space on out-of-home advertising structures and sites in the United States and Canada.	{"Real Estate","REIT - Diversified",REITs}	{O,ALX,PGRE}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:08.621354	2020-10-11 23:38:31.02582
OXLCM	Oxford Lane Capital Corp. 6.75% Series 2024 Term Preferred Stock	Oxford Lane Capital is a closed end fund engaged in investing in collateralized loan obligation vehicles which own senior corporate debt instruments.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:08.650599	2020-10-11 23:38:31.031328
OXLCO	Oxford Lane Capital Corp. Term Preferred Shares 7.50% Series 2023	Oxford Lane Capital is a closed end fund engaged in investing in collateralized loan obligation vehicles which own senior corporate debt instruments.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:08.651791	2020-10-11 23:38:31.02423
OGEN	Oragenics Inc.	Oragenics Inc is a biopharmaceutical company which engages in the discovery, development and commercialization of technologies associated with oral health, broad spectrum antibiotics and other general health benefits.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NYSE American	\N	2020-10-11 23:12:08.334416	2020-10-11 23:38:31.023007
OILX	ETRACS S&P GSCI Crude Oil Total Return Index ETN due February 22 2046	The investment seeks to provide exposure to potential price appreciation in the S&P GSCI Crude Oil Total Return Index (the “index”).\n The index is a sub-index of the S&P GSCI commodity index (“S&P GSCI”) and measures the returns from the West Texas Intermediate crude oil futures contract traded on the New York Mercantile Exchange.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.384508	2020-10-11 23:38:31.031328
OMC	Omnicom Group Inc.	Omnicom Group Inc is an advertising, marketing and corporate communications company. It provides professional services to clients through multiple agencies.	{"Consumer Cyclical","Advertising Agencies","Advertising & Marketing Services"}	{MDCA,EBAY,IPG,ADS,CCO,LAMR}	Advertising & Marketing Services	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:08.414639	2020-10-11 23:38:31.02582
ONB	Old National Bancorp	Old National Bancorp is holding company engaged in the single business of insurance underwriting and related services. It provides financial services including commercial and retail banking, trust, brokerage, correspondent banking, and insurance.	{"Financial Services","Banks - Regional - US",Banks}	{PEBO,THFF,LKFN,GABC,FRME,SRCE,HBNC,HBAN,RF}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:08.455562	2020-10-11 23:38:31.025871
ONVO	Organovo Holdings Inc.	Organovo Holdings Inc is a development-stage company. It is engaged in developing and commercializing functional three-dimensional human tissues.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{MSON,UTHR,ISRG,BVX,MMM,DDD,VJET,TRXC,VCEL,XLV}	Medical Diagnostics & Research	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:08.476858	2020-10-11 23:38:31.021426
OPNT	Opiant Pharmaceuticals Inc.	Opiant Pharmaceuticals Inc is engaged in developing pharmacological treatments for substance use, addictive and eating disorders.	{Healthcare,Biotechnology}	{GWPH,AKTX,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:08.496205	2020-10-11 23:38:31.025871
ORA	Ormat Technologies Inc.	Ormat Technologies Inc is engaged in the geothermal and recovered energy power business. It designs, develops, builds, sells, owns, and operates clean, environmentally friendly geothermal and recovered energy-based power plants.	{Utilities,"Utilities - Independent Power Producers"}	{EIX,AES,NRG,CVA,PEGI,XLU}	Utilities - Independent Power Producers	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:08.514959	2020-10-11 23:38:31.031328
ORI	Old Republic International Corporation	Old Republic International Corporation is engaged in insurance underwriting and related services. It conducts its operations through General Insurance, Title Insurance, and the Republic Financial Indemnity and consumer credit indemnity Run-off Business.	{"Financial Services","Insurance - Diversified",Insurance}	{HALL,RDN,RLI,STC,TRV,FAF}	Insurance	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.532654	2020-10-11 23:38:31.023428
ORRF	Orrstown Financial Services Inc	Orrstown Financial Services Inc is a holding company. The Bank is engaged in commercial banking and trust business, this involves accepting demand, time and savings deposits, and granting loans.	{"Financial Services","Banks - Regional - US",Banks}	{CZNC,NWFL,FRBK,BMTC,MPB,PWOD,STBA,UVSP,ACNB}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:08.549848	2020-10-11 23:38:31.023007
OSLE	Oaktree Specialty Lending Corporation 5.875% Senior Notes due 2024		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:08.574051	2020-10-11 23:38:31.021426
OTEX	Open Text Corporation	Open Text Corp develops and sells enterprise information management (EIM) software. The company offers EIM software that allows clients to archive, aggregate, retrieve, and search unstructured information for better management.	{Technology,"Software - Application","Application Software"}	{MSFT,TISA,ADBE,ADSK,BVSN,SAP,IBM,HPQ,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:08.604617	2020-10-11 23:38:31.023007
OTIV	On Track Innovations Ltd	On Track Innovations Ltd is engaged in the design and development of secure cashless payment solutions. Its products include UNO, TRIO Reader, Saturn 6500 TRIO & Saturn 600 DUO, SCI 6100, Wave and NFC insert.	{Industrials,"Business Equipment","Industrial Products"}	{TSEM,TACT,ZBRA,USAT,SPCB,INVE}	Industrial Products	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:08.608523	2020-10-11 23:38:31.021426
OVID	Ovid Therapeutics Inc.	Ovid Therapeutics Inc is a US-based biopharmaceutical company. The firm is engaged primarily in developing medicines for patients suffering with rare neurological disorders.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:08.632363	2020-10-11 23:38:31.023428
OVLY	Oak Valley Bancorp (CA)	Oak Valley Bancorp is a bank holding company of Oak Valley Community Bank. It provides personal banking and commercial banking services.	{"Financial Services","Banks - Regional - US",Banks}	{TCBK,AMRB,CVBF,HTBK,CVCY,BSRR,BOCH,BMRC}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:08.636948	2020-10-11 23:38:31.021426
QTM	Quantum Corporation	Quantum Corp provides storage, archive and data protection solutions for capturing, sharing, and preserving digital assets over the entire data lifecycle.	{Technology,"Data Storage","Computer Hardware"}	{NTAP,STX,IBM,HPQ,QBAK,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:09.950685	2020-10-11 23:38:31.028166
OXM	Oxford Industries Inc.	Oxford Industries Inc is an apparel company, which designs, sources, markets and distributes products bearing the trademarks of its company-owned lifestyle brands as well as certain licensed and private label apparel products.	{"Consumer Cyclical","Apparel Manufacturing","Manufacturing - Apparel & Furniture"}	{GIL,COLM,SGC,VFC,DLA,PVH}	Manufacturing - Apparel & Furniture	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:08.652813	2020-10-11 23:38:31.021426
PAHC	Phibro Animal Health Corporation	Phibro Animal Health Corp is a diversified animal health and mineral nutrition company. It develops, manufactures and markets products for food animals including poultry, swine and cattle.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{ICCC,ADM,LLY,MRK,HSKA,ZTS,PETX,KIN,XLV}	Drug Manufacturers	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:08.691535	2020-10-11 23:38:31.031328
PAR	PAR Technology Corporation	PAR Technology Corp together with its subsidiaries provides management technology solutions including software, hardware, and related services.	{Technology,"Software - Application","Application Software"}	{AGYS,USAT,NCR,TYL,DBD,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:08.716625	2020-10-11 23:38:31.024353
PBCTP	People's United Financial Inc. Perpetual Preferred Series A Fixed-to-floating Rate	People's United Financial Inc is a bank holding company. It provides commercial banking, retail and business banking, and wealth management services to individual, corporate and municipal customers.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:08.756064	2020-10-11 23:38:31.023428
PBI	Pitney Bowes Inc.	Pitney Bowes Inc is a technology company that offers products and solutions to assist consumers in marketing to their customers. Its products include customer information management, location intelligence, and customer engagement products and solutions.	{Industrials,"Business Equipment","Industrial Products"}	{CAJ,DHR,DBD,ACCO,RRD,XRX}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:08.774915	2020-10-11 23:38:31.02582
PBTP	Invesco PureBeta 0-5 Yr US TIPS	The investment seeks to track the investment results (before fees and expenses) of the ICE BofAML 0-5 Year US Inflation-Linked Treasury IndexSM (the "underlying index").\n The fund generally will invest at least 80% of its total assets in the component securities that comprise the underlying index. The index is designed to measure the performance of the shorter maturity subset of the U.S. Treasury Inflation-Protected Securities ("TIPS") market, represented by TIPS with a remaining maturity of at least one month and less than five years. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:08.802094	2020-10-11 23:38:31.023007
PCAR	PACCAR Inc.	PACCAR Inc manufactures and sells light, medium and heavy-duty trucks and parts under the Kenworth, Peterbilt and DAF brands. It also designs and manufactures advanced diesel engines and provides financial and information technology services.	{Industrials,"Truck Manufacturing"}	{NAV,OSK,WNC,F}	Truck Manufacturing	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:08.816511	2020-10-11 23:38:31.025871
PCN	Pimco Corporate & Income Strategy Fund	PIMCO Corporate & Income Strategy Fund is a closed-end management investment company. Its objective is to seek high current income. The secondary investment objective of the fund is capital preservation and appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.831902	2020-10-11 23:38:31.023007
PCY	Invesco Emerging Markets Sovereign Debt	The investment seeks to track the investment results (before fees and expenses) of the DBIQ Emerging Market USD Liquid Balanced Index (the "underlying index").\n The fund generally will invest at least 80% of its total assets in U.S. dollar-denominated government bonds from emerging market countries that comprise the underlying index. The underlying index measures potential returns of a theoretical portfolio of liquid emerging market U.S. dollar-denominated government bonds.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.851744	2020-10-11 23:38:31.031328
PDLB	PDL Community Bancorp	PDL Community Bancorp is a United States based bank holding company. It offers deposit and loan products. The bank provides Personal solutions, Business solutions, and Lending Solutions.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:08.886196	2020-10-11 23:38:31.028166
PEB	Pebblebrook Hotel Trust of Beneficial Interest	Pebblebrook Hotel Trust is an internally managed hotel investment company organized to acquire and invest in hotel properties. The firm focuses on both branded and independent full-service hotels in the upper upscale segment of the lodging industry.	{"Real Estate","REIT - Hotel & Motel",REITs}	{LHO,HST,CHSP,RLJ,INN,SOHO,HT,HPT,CDOR}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:08.906075	2020-10-11 23:38:31.025871
PEIX	Pacific Ethanol Inc.	Pacific Ethanol Inc is a marketer and producer of low-carbon renewable fuels in the United States. It also sells ethanol co-products, including wet distillers grain, a nutritious animal feed, and corn oil.	{Energy,"Oil & Gas Refining & Marketing","Oil & Gas - Refining & Marketing"}	{GPRE,REX,VLO,ADM,CVX,REGI,GRBK}	Oil & Gas - Refining & Marketing	Energy	NASDAQ Capital Market	\N	2020-10-11 23:12:08.926412	2020-10-11 23:38:31.02423
PERI	Perion Network Ltd	Perion Network Ltd is a technology company that delivers digital advertising solutions for the brands, agencies and publishers. The company provides cross-screen digital advertising through desktop, mobile and social channels to media properties.	{Technology,"Internet Content & Information","Online Media"}	{MSFT,IAC,CEL,GOOGL,XLK}	Online Media	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:08.948468	2020-10-11 23:38:31.02423
PAAS	Pan American Silver Corp.	Pan American Silver Corp produces various kinds of metals. It is engaged in the production and sale of silver, gold and base metals including copper, lead and zinc.	{"Basic Materials",Silver,"Metals & Mining"}	{MAG,SVM,AXU,CDE,HL,AG,WPM}	Metals & Mining	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:12:08.668865	2020-10-11 23:38:31.031328
PAI	Western Asset Investment Grade Income Fund Inc.	Western Asset Income Fund is a closed end diversified investment company. The fund seeks a high level of current income, consistent with prudent investment risk, through investment in a diversified portfolio of debt securities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.693401	2020-10-11 23:38:31.023428
PAVM	PAVmed Inc.	PAVmed Inc is a medical device company engaged in advancing a pipeline of medical technologies from concept to commercialization using a business model focused on capital and time efficiency.	{Healthcare,"Medical Devices"}	{NUVA,XLV}	Medical Devices	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:08.73056	2020-10-11 23:38:31.02582
PBHC	Pathfinder Bancorp Inc.	Pathfinder Bancorp Inc is a stock holding company. It is engaged in the attracting deposits and investing deposits in loans secured by one-to-four family residential real estate, commercial real estate, business assets and investment securities.	{"Financial Services","Savings & Cooperative Banks",Banks}	{FFIC,DCOM,ESBK,TRST,GCBC,LSBK,CARV}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:08.773608	2020-10-11 23:38:31.031328
PBYI	Puma Biotechnology Inc	Puma Biotechnology Inc is a biopharmaceutical company. It is engaged in the acquisition, development and commercialization of products to enhance cancer care. Its drug candidates are PB272 (neratinib, oral), PB272 (neratinib, intravenous), and PB357.	{Healthcare,Biotechnology}	{ARRY,GSK,PFE,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:08.805894	2020-10-11 23:38:31.02582
PCRX	Pacira Pharmaceuticals Inc.	Pacira Pharmaceuticals Inc is a specialty and generic drug manufacturing company. The company develops, manufactures, and commercializes pharmaceutical products for use in hospitals and ambulatory surgery centers across the United States.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{DRRX,BDSI,CPIX,ABC,REPH,CARA,FLXN,HRTX,XLV}	Drug Manufacturers	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:08.841246	2020-10-11 23:38:31.025871
PDFS	PDF Solutions Inc.	PDF Solutions Inc provides electrical characterization and data analytics for process-design optimization and yield enhancement. It offers Manufacturing Process Solutions, Volume Manufacturing Solution, and Design-for-Inspection solutions.	{Technology,"Software - Application","Application Software"}	{CDNS,SNPS,IBM,TER,IMI,RTEC,XPER,QTRH,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:08.885456	2020-10-11 23:38:31.025871
PE	Parsley Energy Inc. Class A	Parsley Energy Inc is an independent oil and natural gas company. It has operations spread across the Permian Basin located in West Texas and Southeastern New Mexico engaged in the development of unconventional oil and natural gas reserves.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{AREX,PXD,CXO,LPI,FANG}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:08.906927	2020-10-11 23:38:31.031328
PEJ	Invesco Dynamic Leisure and Entertainment	The investment seeks to track the investment results (before fees and expenses) of the Dynamic Leisure & Entertainment IntellidexSM Index.\n The fund generally will invest at least 90% of its total assets in common stocks of leisure companies and entertainment companies that comprise the underlying intellidex. The underlying intellidex was composed of common stocks of 30 U.S. leisure and entertainment companies. These companies are engaged principally in the design, production or distribution of goods or services in the leisure and entertainment industries. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.926836	2020-10-11 23:38:31.025871
PES	Pioneer Energy Services Corp. Common Stk	Pioneer Energy Services Corp provides drilling services and production services to a group of independent and large oil and gas exploration and production companies throughout much of the onshore oil and gas producing regions of the USA and Colombia.	{Energy,"Oil & Gas Drilling","Oil & Gas - Drilling"}	{PDS,PTEN,HP,UNT,NBR}	Oil & Gas - Drilling	Energy	New York Stock Exchange	\N	2020-10-11 23:12:08.948664	2020-10-11 23:38:31.021426
PEZ	Invesco DWA Consumer Cyclicals Momentum ETF	The investment seeks to track the investment results (before fees and expenses) of the Dorsey Wright® Consumer Cyclicals Technical Leaders Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities that comprise the underlying index. The underlying index is composed of at least 30 securities of companies in the consumer discretionary (or cyclicals) sector that have powerful relative strength or "momentum" characteristics.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:08.976413	2020-10-11 23:38:31.02582
PFG	Principal Financial Group Inc	Principal Financial Group Inc is an investment management company. It offers a range of financial products and services, including retirement, asset management and insurance. The company receives maximum revenue in the form of premiums.	{"Financial Services","Insurance - Life"}	{AEG,AFL,CI,AIZ,CNO}	Insurance - Life	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:09.009697	2020-10-11 23:38:31.024353
PFO	Flaherty & Crumrine Preferred Income Opportunity Fund Incorporated	Flaherty & Crumrine Prfd Income Oppo is a United States based closed-end investment company. Its primary investment objective is to produce high current income consistent with the preservation of capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.046826	2020-10-11 23:38:31.028166
PAA	Plains All American Pipeline L.P.	Plains All American Pipeline LP is engaged in providing transportation, storage, processing, fractionation, and marketing services for crude oil, refined products, natural gas liquids, liquefied petroleum gas, and related products.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{EPD,EEP,MMP,NS,SEMG,EEQ,BKEP,MPLX,DKL,PSXP}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:08.66883	2020-10-11 23:38:31.02582
PAF	Invesco FTSE RAFI Asia Pacific ex-Japan	The investment seeks to track the investment results (before fees and expenses) of the FTSE RAFI Developed Asia Pacific ex-Japan Index (the "underlying index").\n The fund will invest at least 90% of its total assets in equity securities of companies that are classified as "Asia Pacific" within the country classification definition of FTSE International Limited, excluding Japanese companies, and that comprise the underlying index, as well as American depositary receipts ("ADRs") and global depositary receipts ("GDRs") that are based on the securities in the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.68718	2020-10-11 23:38:31.025871
PAM	Pampa Energia S.A.	Pampa Energia SA is an integrated energy company that generates, transmits and distributes electricity in Argentina. It also operates in the oil and gas exploration and production activity through its subsidiary.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{YPF,ENIA,EOCC,EBR.B,CPL,EDN,CIG.C,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:08.700922	2020-10-11 23:38:31.02582
PATI	Patriot Transportation Holding Inc.	Patriot Transportation Holding Inc, through its subsidiaries, is engaged in the transportation and real estate businesses. Its activities include hauling petroleum related products and dry bulk commodities and liquid chemicals.	{Industrials,Trucking,"Transportation & Logistics"}	{LSTR,JBHT,WERN,KSU}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:08.718689	2020-10-11 23:38:31.023007
PBA	Pembina Pipeline Corp. (Canada)	Pembina Pipeline Corp caters to the oil & gas industry. Its services include transportation of oil & gas through its pipelines.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{ENB,TRP,MMP,NS,BPL,GLOP}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:08.748191	2020-10-11 23:38:31.023007
PBE	Invesco Dynamic Biotech & Genome	The investment seeks to track the investment results (before fees and expenses) of the Dynamic Biotech & Genome IntellidexSM Index.\n The fund generally will invest at least 90% of its total assets in common stocks of biotechnology companies and genome companies that comprise the underlying intellidex. The underlying intellidex was composed of common stocks of 30 U.S. biotechnology and genome companies. These companies are engaged principally in the research, development, manufacture and marketing and distribution of various biotechnological products, services and processes, etc. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.763228	2020-10-11 23:38:31.028166
PBND	Invesco PureBeta US Aggregate Bond	The investment seeks to track the investment results (before fees and expenses) of the ICE BofAML US Broad Market IndexSM (the "underlying index").\n The fund generally will invest at least 80% of its total assets in the component securities that comprise the underlying index. To qualify for inclusion in the underlying index, securities must have: (i) at least one year to final maturity; (ii) at least 18 months to final maturity when issued; (iii) a fixed coupon schedule; and (iv) with the exception of U.S. Treasuries, an investment grade rating. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:08.781234	2020-10-11 23:38:31.02423
PBT	Permian Basin Royalty Trust	Permian Basin Royalty Trust is an oil and natural gas royalty trust. The Company owns overriding royalty rights in mineral properties in the United States.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{MARPS,DMLP,PHX,CRT,MTR,NRT,SBR,SJT,TPL}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:08.799657	2020-10-11 23:38:31.028166
PCM	PCM Fund Inc.	Pcm Fund Inc is a closed-end management investment company. Its investment objective is to seek current income as a primary focus and also capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.82509	2020-10-11 23:38:31.02423
PCYO	Pure Cycle Corporation	Pure Cycle Corp provides wholesale water and wastewater services which includes designing and constructing water and wastewater systems as well as operating and maintaining such systems.	{Utilities,"Utilities - Regulated Water","Utilities - Regulated"}	{YORW,ARTNA,CTWS,MSEX,CDZI,CWT,AWR,WTR,AWK,SJW,XLU}	Utilities - Regulated	Utilities	NASDAQ Capital Market	\N	2020-10-11 23:12:08.852205	2020-10-11 23:38:31.023007
PEG	Public Service Enterprise Group Incorporated	Public Service Enterprise Group Inc operates in the utilities sector. Its primary business involves the production and transmission of power.	{Utilities,"Utilities - Diversified","Utilities - Regulated"}	{FE,AEE,ED,SO,EXC,SCG,PPL,NRG,DUK,XEL,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:08.916243	2020-10-11 23:38:31.028166
PEO	Adams Natural Resources Fund Inc.	Adams Natural Resources Fund Inc is a closed-end equity investment company. The investment objectives of the fund are the preservation of capital, the attainment of reasonable income from investments, and an opportunity for capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.935893	2020-10-11 23:38:31.028166
PAC	Grupo Aeroportuario Del Pacifico S.A. B. de C.V. de C.V. (each representing 10 Series B shares)	Grupo Aeroportuario del Pacifico SAB de CV manage and develop airport facilities in Mexico. The company's segments include Guadalajara, Tijuana, Puerto Vallarta, San Jose del Cabo, Montego Bay, Hermosillo, Bajio, Other Airports and Others Companies.	{Industrials,"Airports & Air Services","Transportation & Logistics"}	{OMAB,ASR}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:08.671241	2020-10-11 23:38:31.023428
PAGP	Plains Group Holdings L.P. Class A Shares representing limited partner interests	Plains GP Holdings LP owns and operates midstream energy infrastructure and provides logistics services for crude oil, natural gas liquids, natural gas and refined products. Its business segments are Transportation, Facilities, and Supply and Logistics.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{PAA,MMP,HEP,TLP,NS,SEMG,BKEP,PSXP}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:08.688957	2020-10-11 23:38:31.02423
PATK	Patrick Industries Inc.	Patrick Industries Inc is a manufacturer of component products and distributor of building products and materials to the recreational vehicle and manufactured housing industries.	{"Consumer Cyclical","Home Furnishings & Fixtures","Manufacturing - Apparel & Furniture"}	{AMWD,MAS,THO,WGO,FBHS,LCII}	Manufacturing - Apparel & Furniture	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:08.719731	2020-10-11 23:38:31.021426
PBB	Prospect Capital Corporation 6.25% Notes due 2024		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:08.748633	2020-10-11 23:38:31.024353
PBH	Prestige Consumer Healthcare Inc.	Prestige Brands Holdings Inc is a healthcare company. Its core business involves the distribution of healthcare and household cleaning products to supermarkets and drug stores.	{Healthcare,"Medical Distribution"}	{GSK,CVS,NVS,ABT,JNJ,PFE,PG,RAD,TARO,WBA,XLV}	Medical Distribution	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:08.772731	2020-10-11 23:38:31.028166
PBR	Petroleo Brasileiro S.A.- Petrobras	Petroleo Brasileiro SA Petrobras is an oil and gas company focused on the exploration and production in Brazilian fields. Its segments include Exploration & Production; Refining, Transportation and Marketing; Distribution, Gas and Power and Biofuel.	{Energy,"Oil & Gas Integrated","Oil & Gas - Integrated"}	{YPF,TOT,XOM,BP,PTR,CVX,MRO,RDS.A,HES}	Oil & Gas - Integrated	Energy	New York Stock Exchange	\N	2020-10-11 23:12:08.789953	2020-10-11 23:38:31.023007
PCG	Pacific Gas & Electric Co.	PG&E Corp is an electric utility company that provides electricity as well as natural gas distribution, electricity generation, procurement, and transmission, and natural gas procurement, transportation, and storage.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{EIX,SRE,CMS,HE,AVA,D,SCG,POR,WEC,XEL,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:08.822422	2020-10-11 23:38:31.028166
PCSB	PCSB Financial Corporation	PCSB Financial Corp operates as a bank holding company. The Company through its subsidiary provides banking services and offers lending and depositing services.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:08.844063	2020-10-11 23:38:31.028166
PDEX	Pro-Dex Inc.	Pro-Dex Inc is engaged in designing, development, and manufacturing of powered rotary drive surgical and dental instruments used primarily in the orthopedic, spine, maxocranial facial and dental markets.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{ALGN,DHR,TMO,RVP,TDG,MDT,XRAY,XLV}	Medical Instruments & Equipment	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:08.876146	2020-10-11 23:38:31.023428
PDT	John Hancock Premium Dividend Fund	John Hancock Patriot Premium Div Fund II is a United States based closed-end management investment company. It seeks to provide a high current income consistent with modest growth of capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.90439	2020-10-11 23:38:31.023428
PENN	Penn National Gaming Inc.	Penn National Gaming Inc is a diversified, multi-jurisdictional owner and manager of gaming and racing facilities and video gaming terminal operations with a focus on slot machine entertainment.	{"Consumer Cyclical","Resorts & Casinos","Travel & Leisure"}	{MCRI,CNTY,CHDN,BYD,MGM,ERI}	Travel & Leisure	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:08.933731	2020-10-11 23:38:31.028166
PFBI	Premier Financial Bancorp Inc.	Premier Financial Bancorp Inc is a multi-bank holding company. It focuses on providing quality community banking services to individuals and small-to-medium sized businesses.	{"Financial Services","Banks - Regional - US",Banks}	{FCBC,CHCO,EGBN,FCNCA,UBSI,SMMF,SONA,STI}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:08.980548	2020-10-11 23:38:31.031328
PFL	PIMCO Income Strategy Fund Shares of Beneficial Interest	PIMCO Income Strategy Fund is a United States based closed-end management investment company. The fund's investment objective is to seek high current income, consistent with the preservation of capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.025597	2020-10-11 23:38:31.023428
PACW	PacWest Bancorp	PacWest Bancorp is a bank holding company. It provides commercial banking services, including real estate, construction, and commercial loans.	{"Financial Services","Banks - Regional - US",Banks}	{CVBF,EWBC,WABC,PMBC,UBFO,PPBI,SIVB,WFC,HAFC}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:08.687275	2020-10-11 23:38:31.023007
PBBI	PB Bancorp Inc.	PB Bancorp Inc is a federally chartered stock holding company of Putnam Bank which provides banking services to individual and small business customers located in Connecticut.	{"Financial Services","Savings & Cooperative Banks",Banks}	{PBCT,SIFI,BHLB,UBNK}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:08.752357	2020-10-11 23:38:31.028166
PBIP	Prudential Bancorp Inc.	Prudential Bancorp Inc, through Prudential Savings Bank attracts deposits from its community through its branch offices and invests those deposits, together with funds from borrowings and operations, in single-family residential loans.	{"Financial Services","Savings & Cooperative Banks",Banks}	{WVFC,ESSA,BNCL}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:08.776201	2020-10-11 23:38:31.023428
PBUS	Invesco PureBeta MSCI USA	The investment seeks to track the investment results (before fees and expenses) of the MSCI USA Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in common stocks that comprise the underlying index. The index is designed to measure the performance of the large- and mid-capitalization segments of the U.S. equity market. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:08.802363	2020-10-11 23:38:31.028166
PCF	High Income Securities Fund	Putnam High Income Securities Fund is a closed-end management investment company. The fund seeks high current income and capital appreciation by investing primarily in corporate and convertible bonds and preferred stock.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.819583	2020-10-11 23:38:31.023007
PCQ	PIMCO California Municipal Income Fund	PIMCO California Municipal Income Fund is a United States based company operating as a closed-end management investment company. It seeks to provide current income exempt from federal and California income tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.837547	2020-10-11 23:38:31.028166
PDCO	Patterson Companies Inc.	Patterson Companies Inc is a value-added specialty distributor. It operates through its two strategic business units, Patterson Dental and Patterson Animal Health, offering similar products and services to different customer bases.	{Healthcare,"Medical Distribution"}	{HSIC,CAH,OMI,MCK,ABC,LIVN,XLV}	Medical Distribution	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:08.86829	2020-10-11 23:38:31.028166
PDP	Invesco DWA Momentum ETF	The investment seeks to track the investment results (before fees and expenses) of the Dorsey Wright® Technical Leaders Index.\n The fund generally will invest at least 90% of its total assets in securities that comprise the underlying index. The underlying index is composed of approximately 100 securities from an eligible universe of approximately 1,000 securities of the largest constituents by market capitalization within the NASDAQ US Benchmark Index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:08.897837	2020-10-11 23:38:31.028166
PEGA	Pegasystems Inc.	Pegasystems Inc. develops, market, and license software and also provides implementation, consulting, training, and technical support services to help its customers maximize the business value from the use of the company's software.	{Technology,"Software - Application","Application Software"}	{MSFT,VRNT,MSTR,SAP,FICO,IBM,CRM,GWRE,ORCL,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:08.92045	2020-10-11 23:38:31.024353
PETQ	PetIQ Inc.	PetIQ Inc is a manufacturer and distributor of health and wellness products for dogs and cats. The company's product offering includes pet Rx medications, OTC flea and tick preventatives, vitamins treats, nutritional supplements, and hygiene products.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{OMER,XLV}	Drug Manufacturers	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:08.961192	2020-10-11 23:38:31.024353
PFIN	P & F Industries Inc.	P & F Industries Inc is a United States based company engaged in the manufacturing and importing of air-powered tools through its subsidiaries. The company operates in the single business segment of Tools business.	{Industrials,"Tools & Accessories","Industrial Products"}	{BGG,SNA,ATU,SWK,SCX,EML}	Industrial Products	Industrials	NASDAQ Global Market	\N	2020-10-11 23:12:09.026401	2020-10-11 23:38:31.021426
PG	Procter & Gamble Company (The)	Procter & Gamble Co is a consumer goods company. The firm markets its products internationally through various channels which include mass merchandisers, grocery stores, membership club stores, drug stores, and department stores.	{"Consumer Defensive","Household & Personal Products","Consumer Packaged Goods"}	{EL,CAW,CHD,CLX,CL,JNJ,KMB}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:09.057832	2020-10-11 23:38:31.028166
PHB	Invesco Fundamental High Yield Corporate Bond	The investment seeks to track the investment results (before fees and expenses) of the RAFI® Bonds U.S. High Yield 1-10 Index (the "underlying index").\n The fund generally will invest at least 80% of its total assets in high-yield corporate bonds that comprise the underlying index. The underlying index is comprised of U.S. dollar-denominated bonds that are SEC-registered securities or Rule 144A securities with registration rights (issued after July 31, 2013) and whose issuers are public companies listed on a major U.S. stock exchange.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.101792	2020-10-11 23:38:31.021426
PAGS	PagSeguro Digital Ltd. Class A	PagSeguro Digital Ltd provides financial technology solutions. Its business model includes multiple digital payment solutions, in-person payments, free digital accounts, and issue of prepaid cards.	{Technology,"Software - Infrastructure","Application Software"}	{XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:08.690065	2020-10-11 23:38:31.021426
PANW	Palo Alto Networks Inc.	Palo Alto Networks Inc develops and sells network and endpoint cybersecurity solutions to enterprise customers.	{Technology,"Software - Application","Application Software"}	{CSCO,INTC,CHKP,FFIV,FTNT,JNPR,IBM,FEYE,HPQ,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:08.712262	2020-10-11 23:38:31.028166
PAYX	Paychex Inc.	Paychex Inc operates in payroll outsourcing industry. The company provides integrated payroll, human resource, insurance, and benefits outsourcing solutions for small and medium-sized businesses in the United States.	{Industrials,"Business Services"}	{INTU,ADP,EFX,NSP}	Business Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:08.735815	2020-10-11 23:38:31.025871
PBFX	PBF Logistics LP representing limited partner interests	PBF Logistics LP is engaged in the receiving, handling and transferring of crude oil and the receipt, storage and delivery of crude oil, refined products and intermediates.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{PAA,NS,SEMG,DKL,PBF,PAGP}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:08.770373	2020-10-11 23:38:31.023007
PBPB	Potbelly Corporation	Potbelly Corporation is a neighborhood sandwich concept offering sandwiches, salads and other menu items. Geographically, the group has its business presence internationally and generates most of the income through the sale of food items.	{"Consumer Cyclical",Restaurants}	{NATH,PZZA,JACK,FRSH,RAVE}	Restaurants	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:08.788574	2020-10-11 23:38:31.025871
PCEF	Invesco CEF Income Composite	The investment seeks to track the investment results (before fees and expenses) of the S-Network Composite Closed-End Fund IndexSM.\n The fund generally invests at least 90% of its total assets in securities of U.S.-listed closed-end funds that comprise the underlying index. It is a "fund of funds," as it invests its assets in the common shares of funds included in the underlying index rather than in individual securities (the "underlying funds").	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.816562	2020-10-11 23:38:31.031328
PCOM	Points International Ltd.	Points International Ltd provides e-commerce and technology solutions to the loyalty program industry. It provides web-based solutions to the loyalty program industry such as retailing and wholesaling of loyalty program currencies.	{Technology,"Internet Content & Information","Online Media"}	{DSGX,AMZN,EBAY,JBLU,XOM,ADS,MA,MAXR,XLK}	Online Media	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:08.833949	2020-10-11 23:38:31.02582
PDCE	PDC Energy Inc.	PDC Energy Inc is a domestic independent exploration and production company that produces, develops, acquires and explores for crude oil, natural gas and NGLs with operations in the Wattenberg Field in Colorado, the Utica Shale in southeastern Ohio.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{CLR,OAS,WPX,CRK}	Oil & Gas - E&P	Energy	Nasdaq Global Select	\N	2020-10-11 23:12:08.868188	2020-10-11 23:38:31.02582
PDS	Precision Drilling Corporation	Precision Drilling Corp is a provider of oil and natural gas drilling and related services and products. It is engaged in providing contract drilling and completion and production services in Canada and United States.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{TDG,PTEN,HP,UNT,NBR,PES}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:08.902501	2020-10-11 23:38:31.024353
PEI	Pennsylvania Real Estate Investment Trust	Pennsylvania Real Estate Investment Trust is a self-managed and self-administered REIT in the United States. It is engaged in the ownership, management, leasing, acquisition, redevelopment, development and disposition of retail shopping malls.	{"Real Estate","REIT - Retail",REITs}	{UHT,ARR}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:08.923653	2020-10-11 23:38:31.023428
PER	SandRidge Permian Trust of Benficial Interest	SandRidge Permian Trust earns royalty interests in specified oil & natural gas properties in the Permian Basin located in Andrews County, Texas & the equivalent of 888 oil & natural gas development wells to be drilled within an area of mutual interest.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{APA}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:08.948903	2020-10-11 23:38:31.025871
PFD	Flaherty & Crumrine Preferred Income Fund Incorporated	Flaherty & Crumrine Prefd Inc Fund Inc is a United States based diversified, closed end management investment company. Its investment objective is to provide its common shareholders with high current income consistent with the preservation of capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.982782	2020-10-11 23:38:31.028166
PFLT	PennantPark Floating Rate Capital Ltd.	PennantPark Floating Rate Capital Ltd is an investment company. Its objective is to generate current income and capital appreciation by investing in Floating Rate Loans and other investments made to U.S. middle-market companies.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:09.028138	2020-10-11 23:38:31.023007
PFSI	PennyMac Financial Services Inc. - Ordinary Shares	PennyMac Financial Services Inc provides mortgage loans to customers in the United States. The company's services span its main function of loan production and loan servicing which includes the recovery of loans and other related charges.	{"Financial Services","Specialty Finance",Banks}	{}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.051081	2020-10-11 23:38:31.023428
PAK	Global X MSCI Pakistan	The investment seeks to provide investment results that correspond generally to the price and yield performance, before fees and expenses, of the MSCI All Pakistan Select 25/50 Index ("underlying index").\n The fund will invest at least 80% of its total assets in the securities of the underlying index and in American Depositary Receipts ("ADRs") and Global Depositary Receipts ("GDRs") based on the securities in the underlying index. The underlying index is designed to represent the performance of the broad Pakistan equity universe. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.696138	2020-10-11 23:38:31.023007
PAVE	Global X Funds U.S. Infrastructure Development	The investment seeks to provide investment results that correspond generally to the price and yield performance, of the Indxx U.S. Infrastructure Development Index.\n The fund invests at least 80% of its total assets in the securities of the underlying index. The underlying index is designed to measure the performance of U.S. listed companies that provide exposure to domestic infrastructure development, including companies involved in construction and engineering; production of infrastructure raw materials, composites and products; industrial transportation; and producers/distributors of heavy construction equipment. It is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:08.720402	2020-10-11 23:38:31.028166
PBCT	People's United Financial Inc.	People's United Financial Inc is a bank holding company. It provides commercial banking, retail and business banking, and wealth management services to individual, corporate and municipal customers.	{"Financial Services","Banks - Regional - US",Banks}	{TRST,BRKL,SIFI,WEBK,BHLB,UBNK,PBBI}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:08.753446	2020-10-11 23:38:31.02423
PBS	Invesco Dynamic Media	The investment seeks to track the investment results (before fees and expenses) of the Dynamic Media IntellidexSM Index.\n The fund generally will invest at least 90% of its total assets in common stocks of media companies that comprise the underlying intellidex. The underlying intellidex was composed of common stocks of 3 U.S. media companies. These are companies that are principally engaged in the development, production, sale and distribution of goods or services used in the media industry. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.793694	2020-10-11 23:38:31.024353
PCH	PotlatchDeltic Corporation	PotlatchDeltic Corp, formerly Potlatch Corp, is a real estate investment trust. It operates a real estate sales business and five wood products manufacturing facilities that produce lumber and plywood.	{"Real Estate","REIT - Diversified",REITs}	{UFPI,POPE,DRE,LPX,WY,CLW,PW,CTT,SIR}	REITs	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:08.823073	2020-10-11 23:38:31.024353
PDBC	PowerShares DB Optimum Yield Diversified Commoditi	The investment seeks long term capital appreciation.\n The fund is an actively managed exchange-traded fund ("ETF") that seeks to achieve its investment objective by investing in a combination of financial instruments that are economically linked to the world's most heavily traded commodities. Commodities are assets that have tangible properties, such as oil, agricultural produce or raw metals. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:08.866331	2020-10-11 23:38:31.025871
PEBK	Peoples Bancorp of North Carolina Inc.	Peoples Bancorp of North Carolina Inc is a bank holding company, has no operations, and conducts no business of its own other than owning the Bank. It is a state-chartered commercial bank serving the retail and corporate customers.	{"Financial Services","Banks - Regional - US",Banks}	{FBNC,FCNCA,CART}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:08.906501	2020-10-11 23:38:31.021426
PEN	Penumbra Inc.	Penumbra Inc designs, develops, manufactures and markets medical devices. The products of the company address ischemic stroke, hemorrhagic stroke and vascular conditions that can be treated through thrombectomy and embolization procedures.	{Healthcare,"Medical Devices"}	{NUVA,XLV}	Medical Devices	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:08.93053	2020-10-11 23:38:31.02582
PETX	Aratana Therapeutics Inc.	Aratana Therapeutics Inc is a pet therapeutics company. Its main activity is the licensing, developing, and commercializing of innovative biopharmaceutical products for companion animals.	{Healthcare,Biotechnology}	{ICCC,LLY,MRK,HSKA,PCRX,ZTS,KIN,PAHC,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:08.963543	2020-10-11 23:38:31.031328
PFFD	Global X U.S. Preferred	The investment seeks to provide investment results that correspond generally to the price and yield performance, before fees and expenses, of BofA Merrill Lynch Diversified Core U.S. Preferred Securities Index.\n The fund invests at least 80% of its total assets in the securities of its underlying index. It also invests at least 80% of its total assets in preferred securities that are domiciled in, principally traded in or whose revenues are primarily from the U.S. The underlying index is designed to track the broad-based performance of the U.S. preferred securities market. It is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:08.988254	2020-10-11 23:38:31.028166
PFIS	Peoples Financial Services Corp.	Peoples Financial Services Corp is a bank holding company, The company through its subsidiaries provides a full range of financial services. Its products and services include checking accounts, business accounts, savings accounts, and others.	{"Financial Services","Banks - Regional - US",Banks}	{PEBO,DNBF,EMCF,FFBC,FULT,UVSP,SMMF,ASRV,TBBK,FNB}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:09.025768	2020-10-11 23:38:31.025871
PFSW	PFSweb Inc.	PFSweb Inc is a global commerce service provider using integrated technologies, professional services, and a worldwide network of systems and logistics to deliver global commerce solutions.	{Industrials,"Business Services"}	{EBAY,IBM,VZ,SNX,T,ORCL,XRX}	Business Services	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:09.051604	2020-10-11 23:38:31.02582
PALL	Aberdeen Standard Physical Palladium Shares	The investment seeks to reflect the performance of the price of physical palladium, less the expenses of the Trust’s operations.\n The fund is designed for investors who want a cost-effective and convenient way to invest in palladium with minimal credit risk.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.701175	2020-10-11 23:38:31.028166
PAVMW	PAVmed Inc. Warrant		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:08.734628	2020-10-11 23:38:31.023428
PBJ	Invesco Dynamic Food & Beverage	The investment seeks to track the investment results (before fees and expenses) of the Dynamic Food & Beverage IntellidexSM Index.\n The fund generally will invest at least 90% of its total assets in common stocks of food and beverage companies that comprise the underlying intellidex. The underlying intellidex was composed of common stocks of 30 U.S. food and beverage companies. These companies are engaged principally in the manufacture, sale or distribution of food and beverage products, agricultural products and products related to the development of new food technologies. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.778441	2020-10-11 23:38:31.028166
PBSK	Poage Bankshares Inc.	Poage Bankshares Inc is a holding company for Town Square Bank. Its business consists of attracting retail deposits from the general public and investing those deposits in one- to four-family residential mortgage loans and commercial loans.	{"Financial Services","Savings & Cooperative Banks",Banks}	{HFBC,KFFB,HBCP}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:08.795603	2020-10-11 23:38:31.021426
PCK	Pimco California Municipal Income Fund II of Beneficial Interest	PIMCO CA Muni Income Fund II is a United States based closed-ended investment company. Its investment objective is to seek to provide current income exempt from federal and California income tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.822781	2020-10-11 23:38:31.021426
PCTI	PC-Tel Inc.	PCTEL Inc designs and delivers Performance Critical TELecom technology solutions to the wireless industry. It supplies wireless network antenna and testing solutions.	{Technology,"Communication Equipment"}	{CMTL,CAMP,UBNT,COMM,VIAV,LLL,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:08.848917	2020-10-11 23:38:31.024353
PDM	Piedmont Office Realty Trust Inc. Class A	Piedmont Office Realty Trust Inc is a real estate investment trust. It is engaged in the acquisition and ownership of commercial real estate properties throughout the United States.	{"Real Estate","REIT - Office",REITs}	{VNO,BDN,KRC,BXP,SLG,LXP,CUZ,DEI,GOV}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:08.891431	2020-10-11 23:38:31.021426
PED	Pedevco Corp.	PEDEVCO Corp is a U.S based energy company. It is engaged in the acquisition, exploration, development and production of oil and natural gas shale plays in the Denver-Julesberg Basin (D-J Basin) in Colorado.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{}	Oil & Gas - E&P	Energy	NYSE American	\N	2020-10-11 23:12:08.913273	2020-10-11 23:38:31.023007
PETS	PetMed Express Inc.	PetMed Express Inc is a nationwide pet pharmacy. The Company markets prescription and non-prescription pet medications and other health products for dogs, cats, and horses direct to the consumer.	{"Consumer Defensive","Pharmaceutical Retailers","Retail - Defensive"}	{CJJD,CVS,RAD,WBA}	Retail - Defensive	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:08.956794	2020-10-11 23:38:31.028166
PFFR	ETFIS Series Trust I	The investment seeks investment results that correspond, before fees and expenses, to the price and yield performance of the Indxx REIT Preferred Stock Index (the "underlying index").\n Under normal market conditions, the fund will invest not less than 90% of its assets in component securities of the underlying index. The underlying index is comprised of preferred securities listed on U.S. exchanges that are issued by real estate investment trusts ("REITs"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.988254	2020-10-11 23:38:31.025871
PFNX	Pfenex Inc.	Pfenex Inc is a clinical-stage biotechnology company. It is engaged in the development of biosimilar and therapeutic equivalent products to branded therapeutics and difficult-to-manufacture proteins.	{Healthcare,Biotechnology}	{CGEN,GNCA,AFMD,CBPO,XLV}	Biotechnology	Healthcare	NYSE American	\N	2020-10-11 23:12:09.039848	2020-10-11 23:38:31.024353
PGAL	Global X MSCI Portugal	The investment seeks to provide investment results that correspond generally to the price and yield performance, before fees and expenses, of the MSCI All Portugal Plus 25/50 Index.\n The fund will invest at least 80% of its total assets in the securities of the underlying index and in American Depositary Receipts ("ADRs") and Global Depositary Receipts ("GDRs") based on the securities in the underlying index. The underlying index is designed to represent the performance of the broad Portugal equity universe, while including a minimum number of constituents. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.059129	2020-10-11 23:38:31.021426
PGJ	Invesco Golden Dragon China ETF	The investment seeks to track the investment results (before fees and expenses) of the NASDAQ Golden Dragon China Index.\n The fund generally will invest at least 90% of its total assets in equity securities that comprise the underlying index. The underlying index is composed of securities of U.S. exchange-listed companies that are headquartered or incorporated in the People's Republic of China. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.075753	2020-10-11 23:38:31.024353
QTNA	Quantenna Communications Inc.	Quantenna Communications Inc is engaged in the design, development, and marketing of high-speed wireless communication solutions enabling wireless local area networking.	{Technology,Semiconductors}	{QRVO,QTNA,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:09.950231	2020-10-11 23:38:31.021426
PARR	Par Pacific Holdings Inc.	Par Pacific Holdings Inc is an oil refining and marketing company. It owns and operates refineries with related logistics and retail networks. It also transports, markets and distributes crude oil.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{PDCE,REN}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:08.712433	2020-10-11 23:38:31.02423
PB	Prosperity Bancshares Inc.	Prosperity Bancshares, Inc. operates as the financial holding company for Prosperity Bank. It provides range of financial products and services to small and medium-sized businesses and consumers.	{"Financial Services","Banks - Regional - US",Banks}	{FFIN,IBOC,SBSI,TCBI,CFR,IBTX,LTXB}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.745703	2020-10-11 23:38:31.028166
PBW	Invesco WilderHill Clean Energy	The investment seeks to track the investment results (before fees and expenses) of the WilderHill Clean Energy Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in common stocks of companies that comprise the underlying index. The underlying index was composed of the stocks of about 39 companies that are publicly traded companies in the United States that are engaged in the business of the advancement of cleaner energy and conservation.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.80223	2020-10-11 23:38:31.02423
PCMI	PCM Inc.	PCM Inc operates as a multi-vendor provider of technology products and solutions primarily in the United States and Canada.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{NSIT,AMZN,CNXN,TECD,PRTS,SYX,SNX,CDW,EVLV,GAIA}	Retail - Apparel & Specialty	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:08.825658	2020-10-11 23:38:31.023428
PCTY	Paylocity Holding Corporation	Paylocity Holding Corp is a provider of cloud-based payroll and human capital management, software solutions for medium-sized organizations. Its services are provided in a software-as-a-service delivery model utilizing its cloud-based platform.	{Technology,"Software - Application","Application Software"}	{MSFT,ADP,ULTI,PAYX,CSOD,ORCL,PAYC,WDAY,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:08.850267	2020-10-11 23:38:31.02423
PDI	PIMCO Dynamic Income Fund	PIMCO Dynamic Income Fund is a closed-end management investment company. The fund's investment objective is to seek current income and capital appreciation is a secondary objective.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.884516	2020-10-11 23:38:31.031328
PDVW	pdvWireless Inc.	pdvWireless Inc is engaged in the development and sale of wireless communications applications, including the sale and installation of equipment used to run these applications. It offers cloud-based pdvConnect mobile resource management solution.	{"Communication Services","Telecom Services"}	{USM,GOGO,TMUS}	Communication Services	Communication Services	NASDAQ Capital Market	\N	2020-10-11 23:12:08.902913	2020-10-11 23:38:31.02582
PEGI	Pattern Energy Group Inc.	Pattern Energy Group Inc is a power generation company based in the United States. It produces and sells electricity mainly in the United States and in Canada and Chile.	{Utilities,"Utilities - Independent Power Producers"}	{PCG,SRE,ORA,XLU}	Utilities - Independent Power Producers	Utilities	Nasdaq Global Select	\N	2020-10-11 23:12:08.921633	2020-10-11 23:38:31.031328
PEP	PepsiCo Inc.	PepsiCo Inc is a food, snack and beverage company. It manufactures, markets, distributes and sells convenient and enjoyable beverages, foods and snacks, serving customers and consumers in more than 200 countries and territories.	{"Consumer Defensive","Beverages - Soft Drinks","Beverages - Non-Alcoholic"}	{UL,KO,K,MDLZ,MNST,KHC}	Beverages - Non-Alcoholic	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:08.943079	2020-10-11 23:38:31.023428
PEX	ProShares Global Listed Private Equity	The investment seeks investment results, before fees and expenses, that track the performance of the LPX Direct Listed Private Equity Index (the "index").\n The index consists of up to 30 qualifying listed private equity companies. The fund invests in securities that ProShare Advisors believes, in combination, should track the performance of the index. It will invest at least 80% of its total assets in component securities. The fund will concentrate its investments in a particular industry or group of industries, country or region to approximately the same extent as the index is so concentrated. It is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:08.96962	2020-10-11 23:38:31.028166
PFH	Cabco Tr Jcp 7.625		{"Financial Services","Specialty Finance",Banks}	{}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.013894	2020-10-11 23:38:31.02582
PFS	Provident Financial Services Inc	Provident Financial Services Inc is a financial service provider operating through its subsidiary. The company provides banking services to individual and business customers. Net interest income is the largest portion of the company's net revenue.	{"Financial Services","Savings & Cooperative Banks",Banks}	{OCFC,MGYR,ORIT,KRNY,MSBF}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.049086	2020-10-11 23:38:31.025871
PGM	Barclays Bank PLC	The investment seeks to provide investors with exposure to the Dow Jones-UBS Platinum Subindex Total ReturnService Mark. \nThe Dow Jones-UBS Platinum Subindex Total ReturnService Mark (the "index") reflects the returns that are potentially available through an unleveraged investment in the futures contracts on platinum. The index currently consists of one futures contract on the commodity of platinum which is included in the Dow Jones-UBS Commodity Index Total ReturnService Mark.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.075509	2020-10-11 23:38:31.023007
PBP	Invesco S&P 500 BuyWrite	The investment seeks to track the investment results (before fees and expenses) of the CBOE S&P 500 BuyWrite IndexSM.\n The fund generally will invest at least 90% of its total assets in securities that comprise the underlying index and will write (sell) call options thereon. The underlying index is a total return benchmark index that is designed to track the performance of a hypothetical "buy-write" strategy on the S&P 500® Index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.782185	2020-10-11 23:38:31.028166
PCI	PIMCO Dynamic Credit and Mortgage Income Fund of Beneficial Interest	PIMCO Dynamic Credit & Mortgage Income Fund is a closed-end management investment company. Its investment objective is to seek current income and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.822126	2020-10-11 23:38:31.031328
PCYG	Park City Group Inc.	Park City Group Inc is a software service provider. The company and its subsidiaries develop, market and support proprietary software products. These products assist management of business operations, which helps clients to make more informed decisions.	{Technology,"Software - Application","Application Software"}	{SPSC,KR,DPZ,SPTN,XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:08.851835	2020-10-11 23:38:31.021426
PDLI	PDL BioPharma Inc.	PDL BioPharma Inc is engaged in the discovery of a new generation of targeted treatments for cancer and immunologic diseases. It focuses on intellectual property asset management, investing in new royalty bearing assets and maximizing value of its patent.	{Healthcare,Biotechnology}	{AMGN,SGMO,CGEN,NKTR,BTX,LLY,MYL,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:08.889144	2020-10-11 23:38:31.023007
PEBO	Peoples Bancorp Inc.	Peoples Bancorp Inc (Marietta OH) is a financial holding company. It is engaged in providing banking, insurance, investment and trust solutions through its financial units. It offers demand deposit accounts, savings accounts, and money market accounts.	{"Financial Services","Banks - Regional - US",Banks}	{UBCP,OVBC,UBOH,FFBC,HBAN,PRK,LCNB,FMNB,ONB}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:08.909047	2020-10-11 23:38:31.028166
PEK	VanEck Vectors ChinaAMC CSI 3000	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the CSI 300 Index.\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index and/or in investments that have economic characteristics that are substantially identical to the economic characteristics of the securities that comprise its benchmark index. The index is comprised of the largest and most liquid stocks in the Chinese A-share market.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.926771	2020-10-11 23:38:31.021426
PESI	Perma-Fix Environmental Services Inc.	Perma-Fix Environmental Services Inc is an environmental and technology know-how company. It provides treatment, storage, processing and disposal of mixed waste. The company operates in three reportable segments; Treatment, Services, and Medical.	{Industrials,"Waste Management"}	{IDSA,CWST,HCCI,ECOL,CLH,RSG,WM,VHI}	Waste Management	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:08.949735	2020-10-11 23:38:31.023007
PFBC	Preferred Bank	Preferred Bank is a commercial bank in California. Its Services are divided into Personal Banking, Business Banking and Treasury Management which includes Checking, Savings & CDs, Loans, Information Solutions, and others.	{"Financial Services","Banks - Regional - US",Banks}	{TCBK,AMRB,CVBF,HTBK,PMBC,CVCY,UBFO,PPBI,HAFC}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:08.977762	2020-10-11 23:38:31.023007
PFGC	Performance Food Group Company	Performance Food Group Co delivers food and related products in the United States that operates in three segments Performance foodservice, Vistar and PFG Customised. Its customers include restaurants, schools, companies, hospitals, theatres, among others.	{"Consumer Defensive","Food Distribution","Retail - Defensive"}	{SPTN,PFGC}	Retail - Defensive	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:09.010134	2020-10-11 23:38:31.023428
PFM	Invesco Dividend Achievers ETF	The investment seeks to track the investment results (before fees and expenses) of the NASDAQ US Broad Dividend AchieversTM Index.\n The fund generally will invest at least 90% of its total assets in common stocks of companies that comprise the underlying index. Strictly in accordance with its guidelines and mandated procedures, Nasdaq, Inc. ("Nasdaq" or the "index provider") includes common stock in the underlying index pursuant to a proprietary selection methodology that identifies a universe of "Dividend AchieversTM."	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.030084	2020-10-11 23:38:31.02582
PGC	Peapack-Gladstone Financial Corporation	Peapack Gladstone Financial Corp provides commercial banking services. It also provides consumer banking services, including checking and savings account, residential mortgages, home equity lines of credit and other second mortgage loans.	{"Financial Services","Banks - Regional - US",Banks}	{UNTY,SSFN,FCCY,LBAI,PKBK,BKJ,TRCB,VLY}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:09.063076	2020-10-11 23:38:31.031328
PGZ	Principal Real Estate Income Fund of Beneficial Interest	Principal Real Estate Income Fund is a non-diversified, closed-end management investment company. Its investment objective is to provide high current income, with capital appreciation by investing in commercial real estate related securities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.094254	2020-10-11 23:38:31.02423
PFE	Pfizer Inc.	Pfizer Inc is a biopharmaceutical company that discovers, develops, manufactures, and markets healthcare products including medicines, and vaccines as well as many of the consumer healthcare products.	{Healthcare,"Drug Manufacturers - Major","Drug Manufacturers"}	{AMGN,AZN,GSK,NVS,ABT,BMY,JNJ,LLY,MRK,MYL,XLV}	Drug Manufacturers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:08.990253	2020-10-11 23:38:31.02423
PFN	PIMCO Income Strategy Fund II	PIMCO Income Strategy Fund II is a closed-ended fixed income mutual fund. The company's primary investment objective is to seek high current income, consistent with the preservation of capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.039235	2020-10-11 23:38:31.028166
PGNX	Progenics Pharmaceuticals Inc.	Progenics Pharmaceuticals Inc is a biopharmaceutical company that develops innovative medicines and other technologies to identify, target and treat cancer. Its pipeline drugs include AZEDRA and 1095; 1404 and PyL; and imaging analysis tools.	{Healthcare,Biotechnology}	{SGEN,NKTR,IRWD,AZN,JNJ,SGYP,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:09.076236	2020-10-11 23:38:31.021426
PHD	Pioneer Floating Rate Trust Shares of Beneficial Interest	Pioneer Floating Rate Trust is a diversified fund. Its investment objective is to provide a high level of current income and seek preservation of capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.103813	2020-10-11 23:38:31.02582
PHT	Pioneer High Income Trust of Beneficial Interest	Pioneer High Income Trust is a closed-end management investment company. Its investment objective is to provide high level of current income and to seek capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.125052	2020-10-11 23:38:31.023428
PJT	PJT Partners Inc. Class A	PJT Partners Inc is a United States-based company. It delivers strategic advisory, restructuring and special situations and fund placement and secondary advisory services to corporations, financial sponsors, institutional investors and governments.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{GHL,LAZ,EVR,MC,HLI}	Brokers & Exchanges	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.186166	2020-10-11 23:38:31.021426
PKX	POSCO	POSCO is the producer of steel in Korea. The company is engaged in the manufacturing and distribution of steel rolled products and plates in the domestic and overseas markets.	{"Basic Materials",Steel}	{SID,GGB,CMC,NUE,X,TX,MT}	Steel	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:09.210718	2020-10-11 23:38:31.02423
PLSE	Pulse Biosciences Inc	Pulse Biosciences Inc is a development stage medical device company using a novel and proprietary platform technology called Nano-Pulse Electro-Signaling or NPES for biomedical applications.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{NUVA,XLV}	Medical Instruments & Equipment	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:09.237642	2020-10-11 23:38:31.028166
PMD	Psychemedics Corporation	Psychemedics Corp provides testing services for the detection of drugs of abuse through the analysis of hair samples. This information is useful to employers for both applicant and employee testing, as well as to physicians and among others.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{RDNT,DGX,ENZ,LH,NEO,XLV}	Medical Diagnostics & Research	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:09.269514	2020-10-11 23:38:31.028166
PMTS	CPI Card Group Inc.	CPI Card Group Inc provides financial payment card solutions and services, It offers credit, debit and prepaid cards including Europay, Mastercard and Visa chip, personalization, instant issuance, fulfillment and mobile payment services.	{Industrials,"Business Equipment","Industrial Products"}	{PMTS}	Industrial Products	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:09.29534	2020-10-11 23:38:31.025871
PNR	Pentair plc.	Pentair PLC is engaged in manufacturing pumps for commercial and residential applications, including sump pumps, well pumps, and fire-protection pumps. It also provides advanced connectors and fasteners and protection for sensitive electronics.	{Industrials,"Diversified Industrials","Industrial Products"}	{CR,DOV,IEX,XYL}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:09.32064	2020-10-11 23:38:31.025871
POR	Portland General Electric Co	Portland General Electric Co is a vertically integrated electric utility. It is engaged in the generation, wholesale purchase, transmission, distribution, and retail sale of electricity in the State of Oregon.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{EIX,PCG,HE,IDA,PNW,AVA,PPL,PNM,XEL,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:09.449602	2020-10-11 23:38:31.02423
PPIH	Perma-Pipe International Holdings Inc.	Perma-Pipe International Holdings Inc manufactures pre-insulated piping and leak detection systems for oil and gas gathering, district heating and cooling, and other applications.	{Industrials,"Pollution & Treatment Controls","Industrial Products"}	{NWPX,CECE,OFLX,FTEK,DCI}	Industrial Products	Industrials	NASDAQ Global Market	\N	2020-10-11 23:12:09.471308	2020-10-11 23:38:31.031328
PFIE	Profire Energy Inc.	Profire Energy Inc is an oilfield technology company. The Company provides products that enhance the efficiency, safety, and compliance of the oil and gas industry.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{IR,GHM,PPIH}	Oil & Gas - Services	Energy	NASDAQ Capital Market	\N	2020-10-11 23:12:09.01534	2020-10-11 23:38:31.028166
PFMT	Performant Financial Corporation	Performant Financial Corp provides technology-enabled recovery and related analytics services in the United States. It helps identify and recover delinquent or defaulted assets and improper payments for both government and private clients.	{Industrials,"Business Services"}	{HMSY,EXLS,ARC}	Business Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:09.031937	2020-10-11 23:38:31.031328
PFXF	VanEck Vectors Preferred Securities ex Financials	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the Wells Fargo® Hybrid and Preferred Securities ex Financials Index (the "Preferred Securities Index").\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The index is comprised of convertible or exchangeable and non-convertible preferred securities listed on U.S. exchanges. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.056942	2020-10-11 23:38:31.024353
PGP	Pimco Global Stocksplus & Income Fund StocksPlus & Income Fund of Beneficial Interest	PIMCO Global Stocksplus & Income Fund is a closed-end management investment company. Its objective of the fund is to seek total return comprised of current income, current gains, and long-term capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.080677	2020-10-11 23:38:31.02582
PHDG	Invesco S&P 500 Downside Hedged	The investment seeks to achieve positive total returns in rising or falling markets that are not directly correlated to broad equity or fixed income market returns.\n The fund uses a quantitative, rules based strategy to allocate its assets among components of the S&P 500® Dynamic VEQTOR Index in a combination of (i) equity securities contained in the S&P 500® Index and that are listed on a U.S. securities exchange, (ii) Chicago Board Options Exchange Volatility Index related instruments, such as listed VIX Index futures contracts that reflect exposure to the S&P 500® VIX Short Term Futures Index, and (iii) money market instruments, cash and cash equivalents.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.104388	2020-10-11 23:38:31.023428
PIO	Invesco Global Water ETF	The investment seeks to track the investment results (before fees and expenses) of the NASDAQ OMX Global Water IndexSM (the "underlying index").\n The fund will invest at least 90% of its total assets in the securities of companies listed on a global exchange that create products designed to conserve and purify water for homes, businesses and industries that comprise the underlying index, as well as American depositary receipts ("ADRs") and global depositary receipts ("GDRs") that are based on the securities in the underlying index. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.164081	2020-10-11 23:38:31.02582
PJH	Prudential Financial Inc. 5.75% Junior Subordinated Notes due 2052	Prudential Financial Inc offers annuities, life insurance, retirement plan services, and asset management products to individual and institutional customers in the United States, Asia, Europe and Latin America.	{"Financial Services","Insurance - Life"}	{}	Insurance - Life	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.179845	2020-10-11 23:38:31.028166
PKW	Invesco BuyBack Achievers ETF	The investment seeks to track the investment results (before fees and expenses) of the NASDAQ US BuyBack AchieversTM Index.\n The fund generally will invest at least 90% of its total assets in common stocks that comprise the underlying index. The NASDAQ includes common stocks in the underlying index pursuant to a proprietary selection methodology that identifies a universe of "BuyBack Achievers TM".	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.205353	2020-10-11 23:38:31.028166
PLNT	Planet Fitness Inc.	Planet Fitness Inc is a franchisor and operator of fitness centers in the United States. It has approximately more than 8.7 million members and more than 1,200 stores in 47 states, the District of Columbia, Puerto Rico, Canada and the Dominican Republic.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{CLUB,PLNT}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:09.232589	2020-10-11 23:38:31.021426
PLX	Protalix BioTherapeutics Inc. (DE)	Protalix BioTherapeutics Inc is a biopharmaceutical company focused on the development and commercialization of recombinant therapeutic proteins based on its proprietary ProCellEx protein expression system, or ProCellEx.	{Healthcare,Biotechnology}	{NBIX,FOLD,PFE,TEVA,XLV}	Biotechnology	Healthcare	NYSE American	\N	2020-10-11 23:12:09.250318	2020-10-11 23:38:31.023428
PML	Pimco Municipal Income Fund II of Beneficial Interest	PIMCO Municipal Income Fund II is a United States based closed-end management investment company. Its investment objective is to seek to provide current income exempt from federal income tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.279668	2020-10-11 23:38:31.024353
PNM	PNM Resources Inc. (Holding Co.)	PNM Resources Inc is an investor-owned holding company which owns regulated utilities providing electricity and electric services in New Mexico and Texas. It provides electric generation, transmission, and distribution services.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{EIX,EE,PCG,IDA,PNW,NWE,NEE,XEL,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:09.309209	2020-10-11 23:38:31.028166
PGTI	PGT Innovations Inc.	PGT Innovations Inc, formerly PGT Inc is a manufacturer and supplier of impact-resistant aluminum and vinyl-framed windows and doors and offers a broad range of fully customizable window and door products.	{"Basic Materials","Building Materials"}	{PATK,ROCK,GFF,AWI,NX,NCS,DOOR}	Building Materials	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:09.092636	2020-10-11 23:38:31.025871
PHI	PLDT Inc. Sponsored ADR	PLDT Inc provides telecommunication services. It offers wireless services and fixed line services. Its segments include Wireless, Fixed Line and Others.	{"Communication Services","Telecom Services"}	{SKM,TLK,CHL,VOD}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:09.109202	2020-10-11 23:38:31.02423
PI	Impinj Inc.	Impinj Inc provides radio frequency identification solutions. The company's software includes - ItemSense.	{Technology,"Electronic Components","Computer Hardware"}	{QCOM,SCKT,PRKR,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:09.128322	2020-10-11 23:38:31.02423
PIM	Putnam Master Intermediate Income Trust	Putnam Master Intermediate Income Trust is a closed-end fund. Its investment objective is to t provide high current income and relative stability of net asset value.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.153639	2020-10-11 23:38:31.025871
PIZ	Invesco DWA Developed Markets Momentum ETF	The investment seeks to track the investment results (before fees and expenses) of the Dorsey Wright® Developed Markets Technical Leaders Index (the "underlying index").\n The fund will invest at least 90% of its total assets in the equity securities that comprise the underlying index. The underlying index is comprised of equity securities of large capitalization companies based in countries with developed economies, excluding the United States.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.174965	2020-10-11 23:38:31.025871
PKBK	Parke Bancorp Inc.	Parke Bancorp Inc is a bank holding company. It operates as a commercial bank providing personal and financial services to individuals and small to mid-sized businesses in various states of USA. It provides commercial loan products and retail banking.	{"Financial Services","Banks - Regional - US",Banks}	{ICE,UNTY,SSFN,PGC,LBAI,BKJ,TRCB,VLY}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:09.192785	2020-10-11 23:38:31.023428
PLAB	Photronics Inc.	Photronics Inc is principally engaged in manufacturing photomasks which are used as masters to transfer circuit patterns onto semiconductor wafers and flat panel substrates.	{Technology,"Semiconductor Equipment & Materials",Semiconductors}	{MU,AXTI,IKNX,UMC,ASML,ACLS,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:09.213487	2020-10-11 23:38:31.028166
PLPC	Preformed Line Products Company	Preformed Line Products Co is a designer and manufacturer of products and systems for construction and maintenance of overhead and underground networks for the energy, telecommunication, cable operators, data communication and other industries.	{Technology,"Electronic Components","Computer Hardware"}	{RFIL,OCC,CLFD,APH,GLW,BDC,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:09.233626	2020-10-11 23:38:31.028166
PLXS	Plexus Corp.	Plexus Corp is an electronic equipment manufacturer. It delivers optimized product realization solutions through a product realization value stream services model.	{Technology,"Contract Manufacturers","Computer Hardware"}	{CLS,FLEX,SGMA,KBAL,SMTX,JBL,BHE,CTS,SPA,SANM,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:09.256625	2020-10-11 23:38:31.028166
PMM	Putnam Managed Municipal Income Trust	Putnam Managed Municipal Income Trust is a closed-end management investment company. Its investment objective is to seek high level of current income exempt from federal income tax by investing in diversified portfolio of tax-exempt municipal securities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.281752	2020-10-11 23:38:31.023428
PNC	PNC Financial Services Group Inc. (The)	PNC Financial Services Group Inc operates in the financial services domain. Its main business includes retail banking, corporate and institutional banking, asset management and residential mortgage banking.	{"Financial Services","Banks - Regional - US",Banks}	{FITB,WSBC,KEY,BBT,WFC,STI,JPM,RF}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.302614	2020-10-11 23:38:31.021426
PNRG	PrimeEnergy Corporation	PrimeEnergy Corp is an independent oil and natural gas company. The company is engaged in acquiring, developing and producing oil and natural gas. It owns properties located in Texas, Oklahoma, West Virginia, New Mexico, Colorado and Louisiana.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{ISRL,CHK,PXD,APC,OXY,NBL,QEP}	Oil & Gas - E&P	Energy	NASDAQ Capital Market	\N	2020-10-11 23:12:09.323211	2020-10-11 23:38:31.031328
PPDF	PPDAI Group Inc. American Depositary Shares each representing five Class A	PPDAI Group Inc operates as an online consumer finance marketplace in China. It offers short-term loans to its borrowers to meet their immediate credit needs.	{"Financial Services","Credit Services"}	{NICK,NAVI,OMF,PPDF}	Credit Services	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.462879	2020-10-11 23:38:31.02423
PPT	Putnam Premier Income Trust	Putnam Premier Income Trust is a closed-end fund. Its investment objective is to seek high current income consistent with the preservation of capital by allocating its investments among the U.S. government sector.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.485297	2020-10-11 23:38:31.024353
PHG	Koninklijke Philips N.V. NY Registry Shares	Royal Philips NV is a health technology company. The company provides various services in the area of personal health, diagnosis and treatment, connected care and health informatics businesses, and health tech.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{OSIS,HOLX,SNE,GE,VAR,MDT,XLV}	Medical Diagnostics & Research	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:09.107577	2020-10-11 23:38:31.031328
PID	Invesco International Dividend Achievers ETF	The investment seeks to track the investment results (before fees and expenses) of the NASDAQ International Dividend AchieversTM Index.\n The fund generally will invest at least 90% of its total assets in dividend-paying common stocks and other securities that comprise the underlying index. The underlying index is composed of Global Depositary Receipts ("GDRs") and American Depositary Receipts ("ADRs") that are listed on the London Stock Exchange ("LSE") or the London International Exchange, in addition to ADRs and non-U.S. common or ordinary stocks traded on NYSE, NASDAQ or NYSE American.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.145618	2020-10-11 23:38:31.021426
PIRS	Pieris Pharmaceuticals Inc.	Pieris Pharmaceuticals Inc is a clinical-stage biopharmaceutical company engaged in discovering and developing Anticalin class of biotherapeutics for patients with diseases in which there is high unmet medical needs.	{Healthcare,Biotechnology}	{CASI,CALA,AUPH,CSBR,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:09.1716	2020-10-11 23:38:31.021426
PKB	Invesco Dynamic Building & Construction	The investment seeks to track the investment results (before fees and expenses) of the Dynamic Building & Construction IntellidexSM Index.\n The fund generally will invest at least 90% of its total assets in common stocks of building and construction companies that comprise the underlying intellidex. The underlying intellidex was composed of common stocks of 30 U.S. building and construction companies. These companies are engaged primarily in providing construction and related engineering services for building and remodeling residential properties, commercial or industrial buildings, etc. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.193595	2020-10-11 23:38:31.031328
PLAY	Dave & Buster's Entertainment Inc.	Dave & Buster's Entertainment Inc is an owner and operator of high-volume venues that combine dining and entertainment in North America for both adults and families.	{"Consumer Cyclical",Restaurants}	{NATH,RUTH,THS,MDLZ}	Restaurants	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:09.215301	2020-10-11 23:38:31.02582
PLOW	Douglas Dynamics Inc.	Douglas Dynamics Inc manufacturer and upfitter of commercial vehicle attachments and equipment. Its products include snow plows, sand and salt spreaders, and turf-care equipment sold under the Blizzard, Fisher, Snowex, Western, and Turfex brands.	{"Consumer Cyclical","Auto Parts",Autos}	{AGCO,DE,TTC}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:09.232948	2020-10-11 23:38:31.024353
PLUS	ePlus inc.	ePlus Inc is engaged in the sales of information technology hardware, third-party software, professional engineering services, & third-party maintenance contracts & its proprietary software as well as financing of equipment, software & related services.	{Technology,"Software - Application","Application Software"}	{MSFT,NSIT,AMSWA,TECD,IBM,SNX,CDW,ORCL,HPQ,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:09.248353	2020-10-11 23:38:31.031328
PM	Philip Morris International Inc	Philip Morris International Inc is a holding company. It manufactures and sells cigarettes and other tobacco products in markets outside the United States of America.	{"Consumer Defensive",Tobacco,"Tobacco Products"}	{BTI,VGR,MO,XXII}	Tobacco Products	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:09.266281	2020-10-11 23:38:31.02423
PMOM	Principal Sustainable Momentum Index ETF	The investment seeks to provide investment results that closely correspond, before expenses, to the performance of the Nasdaq U.S. Sustainable Momentum Index (the "index").\n The fund normally invests at least 80% of its net assets, plus any borrowings for investment purposes, in equity securities that compose the index at the time of purchase. The index uses a quantitative model designed to identify equity securities, including growth and value stocks, within the Nasdaq US Large Mid Cap Index that exhibit sustainable price momentum, based on historical stock prices over multiple periods and taking multiple market environments into consideration.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.283801	2020-10-11 23:38:31.023007
POOL	Pool Corporation	Pool Corp is a wholesale distributor of swimming pool supplies, equipment and related leisure products. It also distributes irrigation and landscape products in the United States.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{BECN,TTS,BCC,HBP,SPWH,BXC}	Travel & Leisure	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:09.35185	2020-10-11 23:38:31.031328
PPG	PPG Industries Inc.	PPG Industries Inc is engaged in manufacturing and distributing coatings, specialty materials and glass products. It sells its products to a wide variety of end users, including the automotive, aerospace, construction, and industrial markets.	{"Basic Materials","Specialty Chemicals",Chemicals}	{GLW,RPM,SHW,OC}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:09.464828	2020-10-11 23:38:31.028166
PHIIK	PHI Inc.	PHI Inc provides helicopter transportation services to, from, and among offshore facilities for customers engaged in the oil and gas exploration, development, and production industry.	{Industrials,"Airports & Air Services","Transportation & Logistics"}	{CKH,SPN,BRS,ERA,TDW}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:09.108221	2020-10-11 23:38:31.025871
PIXY	ShiftPixy Inc.	ShiftPixy Inc is a provider of employment services solutions for businesses and workers in an environment in which shift or other part-time/temporary positions are performed.	{Industrials,"Staffing & Outsourcing Services","Employment Services"}	{KELYB,NSP,PIXY}	Employment Services	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:09.171488	2020-10-11 23:38:31.023428
PKD	Parker Drilling Company	Parker Drilling Co provides contract drilling and drilling-related services and rental tools. Its services include drilling services, rental services, rig fleet & locations and among others.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:09.195304	2020-10-11 23:38:31.02582
PLBC	Plumas Bancorp	Plumas Bancorp is a bank holding company for Plumas Bank. It provides banking and investment services. Its source of income is dividends from the bank.	{"Financial Services","Banks - Regional - US",Banks}	{AMRB,CWBC,CATY,HTBK,EWBC,BSRR,UBFO,SIVB,PACW,OVLY}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:09.218896	2020-10-11 23:38:31.023428
PLT	Plantronics Inc.	Plantronics Inc is a manufacturer of communication equipment. It designs and markets communications headsets, telephone headset systems, and accessories for the business and consumer markets under the Plantronic brand.	{Technology,"Communication Equipment"}	{ADTN,AAPL,LOGI,GOOGL,XLK}	Communication Equipment	Technology	New York Stock Exchange	\N	2020-10-11 23:12:09.24186	2020-10-11 23:38:31.02582
PLYA	Playa Hotels & Resorts N.V.	Playa Hotels & Resorts NV is an owner, operator and developer of all-inclusive resorts in prime beachfront locations in Mexico and the Caribbean. Its portfolio consists of approximately 13 resorts located in Mexico, the Dominican Republic and Jamaica.	{"Consumer Cyclical","Resorts & Casinos","Travel & Leisure"}	{}	Travel & Leisure	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:09.259388	2020-10-11 23:38:31.021426
PMF	PIMCO Municipal Income Fund	PIMCO Municipal Income Fund is a United States based closed-end management investment. The company's primary investment objective is to seek high current income, with capital appreciation as a secondary objective.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.28097	2020-10-11 23:38:31.02582
PNBK	Patriot National Bancorp Inc.	Patriot National Bancorp Inc is the holding company for Patriot Bank NA, which provides consumer and commercial banking services to individuals, small and medium-sized businesses, professionals and municipalities.	{"Financial Services","Banks - Regional - US",Banks}	{CAC,EBTC,UNB,BPFH,INDB,WASH,FNLC,WBS,SAL,BWFG}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:09.29891	2020-10-11 23:38:31.031328
PNW	Pinnacle West Capital Corporation	Pinnacle West Capital Corp is an investor owned electric utility holding company based in Phoenix, Arizona. It provides retail or/and wholesale electric service through its subsidiary Arizona Public Service.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{EIX,EE,IDA,AVA,PNM,NWE,XEL,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:09.325228	2020-10-11 23:38:31.023428
POWI	Power Integrations Inc.	Power Integrations Inc designs, develops and markets analog, mixed-signal integrated circuits, and other electronic components and circuitry used in power conversion. The company's product is used in converting electricity.	{Technology,Semiconductors}	{CRUS,DIOD,ON,MPWR,NXPI,STM,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:09.460449	2020-10-11 23:38:31.028166
PPSI	Pioneer Power Solutions Inc.	Pioneer Power Solutions Inc manufactures, sells and services a broad range of specialty electrical transmission, distribution, and on-site power generation equipment for applications in the utility, industrial, commercial and backup power markets.	{Technology,"Electronic Components","Computer Hardware"}	{POWL,AETI,EMR,GE,ABB,XLK}	Computer Hardware	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:09.479726	2020-10-11 23:38:31.028166
PRAN	Prana Biotechnology Ltd	Prana Biotechnology Ltd is active in the healthcare domain. The company is engaged in research collaboration with Takeda for the treatment of Parkinson's disease gastrointestinal neuropathology.	{Healthcare,Biotechnology}	{RDY,NYMX,IQV,KZIA,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:09.498576	2020-10-11 23:38:31.02582
PRI	Primerica Inc.	Primerica Inc is a life insurance company that distributes its products to middle income households in the United States and Canada. The company assists its clients in terms of life insurance, mutual funds, annuities and other financial products.	{"Financial Services","Insurance - Life"}	{ALL,CIA,LNC,TMK,MET,FFG}	Insurance - Life	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.525208	2020-10-11 23:38:31.031328
PINC	Premier Inc.	Premier Inc is a healthcare alliance. The Company together with its subsidiaries and affiliates, is a healthcare improvement company that unites hospitals, health systems, physicians and other healthcare providers.	{Technology,"Health Information Services","Application Software"}	{CRVL,MCK,INFU,TVTY,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:09.153311	2020-10-11 23:38:31.028166
PK	Park Hotels & Resorts Inc.	Park Hotels & Resorts Inc is a lodging real estate company. It portfolio consists of approximately 67 premium-branded hotels and resorts with over 35,000 rooms located in prime United States and internationally.	{"Real Estate","REIT - Hotel & Motel",REITs}	{NYMT,O,NTP}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:09.18577	2020-10-11 23:38:31.024353
PKOH	Park-Ohio Holdings Corp.	Park-Ohio Holdings Corp is a United States based company, engaged in industrial supply chain logistics and diversified manufacturing business. It operates in three business segment namely Supply Technologies, Assembly Components and Engineered Products.	{Industrials,"Industrial Distribution"}	{SNHY,LECO,OFLX,AXE,CR,DOV,PH,ATU}	Industrial Distribution	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:09.202112	2020-10-11 23:38:31.021426
PLND	VanEck Vectors Poland	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the MVIS® Poland Index.\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The index includes securities of Polish companies. A company is generally considered to be a Polish company if it is incorporated in Poland or is incorporated outside of Poland but has at least 50% of its revenues/related assets in Poland. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.224343	2020-10-11 23:38:31.031328
PLTM	GraniteShares Platinum Shares	The investment seeks to reflect, at any given time, the value of the assets owned by the Trust at that time less the Trust’s accrued expenses and liabilities as of that time.\nThe Shares constitute a simple and cost-effective means of making an investment similar to an investment in platinum. An investment in allocated physical platinum bullion requires expensive and sometimes complicated arrangements in connection with the assay, transportation and warehousing of the metal.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.24515	2020-10-11 23:38:31.025871
PMBC	Pacific Mercantile Bancorp	Pacific Mercantile Bancorp, through its subsidiary, is engaged in the commercial banking business in Southern California. It offers various banking services including accepting deposits, providing loans, and other services.	{"Financial Services","Banks - Regional - US",Banks}	{UNB,CVBF,WABC,ZION,UBFO,PACW,CMA,USB,HAFC}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:09.267556	2020-10-11 23:38:31.031328
PMR	Invesco Dynamic Retail	The investment seeks to track the investment results (before fees and expenses) of the Dynamic Retail IntellidexSM Index.\n The fund generally will invest at least 90% of its total assets in common stocks of retail companies that comprise the underlying intellidex. The underlying intellidex was composed of common stocks of 30 U.S. retailers. These companies are engaged principally in operating general merchandise stores such as department stores, discount stores, warehouse clubs and superstores; specialty stores, including apparel, electronics, accessories and footwear stores; and home improvement and home furnishings stores. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.287346	2020-10-11 23:38:31.031328
PNF	PIMCO New York Municipal Income Fund	PIMCO New York Municipal Income Fund is a United States based closed-end management investment company. Its investment objective is to seek to provide current income exempt from federal, New York State and New York City income tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.303657	2020-10-11 23:38:31.028166
POLA	Polar Power Inc.	Polar Power Inc designs, manufactures and sells direct current, or DC, power systems for applications in the telecommunications, military, electric vehicle charging, cogeneration, distributed power and uninterruptable power supply markets.	{Industrials,"Diversified Industrials","Industrial Products"}	{NOVT}	Industrial Products	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:09.352709	2020-10-11 23:38:31.024353
PPC	Pilgrim's Pride Corporation	Pilgrims Pride Corp engages in processing, marketing, and distribution fresh, frozen and value-added chicken products to retailers, grocers, distributors and food service operators. It offers chickens as a whole and different cuts.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{CALM,SAFM,HAIN,HRL,TSN,IBA}	Consumer Packaged Goods	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:09.459946	2020-10-11 23:38:31.024353
PRA	ProAssurance Corporation	Proassurance Corp is a property and casualty insurance company that provides professional liability insurance for healthcare professionals and facilities, professional liability insurance for attorneys, and liability insurance for medical technology.	{"Financial Services","Insurance - Property & Casualty"}	{HALL,AFSI,Y,WRB,L,MKL,RLI,AHL,TRV}	Insurance - Property & Casualty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.495712	2020-10-11 23:38:31.021426
PIR	Pier 1 Imports Inc.	Pier 1 Imports Inc acts as a retailer of decorative accessories, furniture, candles, homeware, gifts and seasonal products. It operates through retail stores and an e-commerce website under the brand Pier 1 Imports.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{BBBY,KIRK,ETH,HVT,WSM,RH}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:09.159405	2020-10-11 23:38:31.024353
PKI	PerkinElmer Inc.	PerkinElmer Inc is a diagnostics company that provides instruments, consumables, and services to biomedical, chemical and to general industrial markets. It operates in two segments namely Discovery and Analytical Solutions and Diagnostics.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{ILMN,HBIO,BRKR,WAT,A,BDX,BIO,TMO,FLDM,PRPO,XLV}	Medical Diagnostics & Research	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:09.198249	2020-10-11 23:38:31.025871
PLD	Prologis Inc.	Prologis Inc is a real estate investment trust that develops, acquires, and operates industrial and logistics facilities across over 18 countries. It operates in two business segments that are Real Estate Operations and Strategic Capital.	{"Real Estate","REIT - Industrial",REITs}	{FR,EGP,MNR,DLR,COR,STAG,REXR,QTS}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:09.223828	2020-10-11 23:38:31.024353
PLYM	Plymouth Industrial REIT Inc.	Plymouth Industrial REIT Inc is a full service, vertically integrated, self-administered and self-managed REIT focused on the acquisition, ownership and management of single and multi-tenant Class B industrial properties.	{"Real Estate","REIT - Industrial",REITs}	{NYMT,O,NTP}	REITs	Real Estate	NYSE American	\N	2020-10-11 23:12:09.266791	2020-10-11 23:38:31.023007
PMX	PIMCO Municipal Income Fund III of Beneficial Interest	PIMCO Municipal Income Fund III is a United States based closed-end management investment company. Its investment objective is to seek to provide current income exempt from federal income tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.294716	2020-10-11 23:38:31.023428
PNTR	Pointer Telocation Ltd.	Pointer Telocation Ltd offers command and control technologies for mobile resource management in the automotive and insurance industries and other mobile tracking markets. It has Cellocator and Mobile resource management segments.	{Technology,"Communication Equipment"}	{GILT,ITRN,MNRO,CRNT,VOXX,XLK}	Communication Equipment	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:09.32455	2020-10-11 23:38:31.028166
POST	Post Holdings Inc.	Post Holdings Inc is a consumer packaged goods company producing and selling cereals, egg products, refrigerated potato and dairy items, private-label nut butters, and protein shakes, bars, and powders.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{HAIN,CAG,GIS,K,FLO,BGS,THS,PEP}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:09.451211	2020-10-11 23:38:31.023428
PPL	PPL Corporation	PPL Corp is an energy and utility holding company operating through its subsidiaries. The company generates and markets electricity in the northeastern and western U.S. and in the delivery of electricity in Pennsylvania and the U.K.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{ETR,FE,ED,AES,AEP,SO,EXC,NEE,DUK,AGR,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:09.474537	2020-10-11 23:38:31.023007
PRCP	Perceptron Inc.	Perceptron Inc develops, produces and sells automated industrial metrology products and solutions to manufacturing organizations for dimensional gauging, dimensional inspection and 3D scanning.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{FARO,OSIS,MTSC,ROK,GEOS,XLK}	Computer Hardware	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:09.499989	2020-10-11 23:38:31.031328
PRID	InsightShares LGBT Employment Equality	The investment seeks to provide investment results that, before fees and expenses, track the price and yield performance of the UBS LGBT Employment Equality Index (the "index").\n The fund will normally invest at least 80% of its total assets in securities of the index. The index is designed to measure the performance of a portfolio of large and mid-capitalization U.S. stocks of companies with workplaces that promote and provide equality for lesbian, gay, bisexual and transgender ("LGBT") employees, as indicated by the Human Rights Campaign Foundation™ ("HRC")'s Corporate Equality Index® (the "CEI").	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.52555	2020-10-11 23:38:31.02582
PRPH	ProPhase Labs Inc.	ProPhase Labs Inc is a manufacturer and distributor of homeopathic and health products. It is also engaged in the research and development of potential over-the-counter drug, natural health products, supplements, personal care & cosmeceutical products.	{Healthcare,"Drug Manufacturers - Major","Drug Manufacturers"}	{NATR,CYAN,USNA,MTEX,RELV,XLV}	Drug Manufacturers	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:09.544378	2020-10-11 23:38:31.023428
PRU	Prudential Financial Inc.	Prudential Financial Inc offers annuities, life insurance, retirement plan services, and asset management products to individual and institutional customers in the United States, Asia, Europe and Latin America.	{"Financial Services","Insurance - Life"}	{PUK,AEG,LNC,MET,GNW}	Insurance - Life	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.575622	2020-10-11 23:38:31.021426
PIN	Invesco India	The investment seeks to track the investment results (before fees and expenses) of the Indus India Index (the "underlying index").\n The fund seeks to achieve its investment objective by investing at least 90% of its total assets in securities of Indian companies that comprise the underlying index, as well as American depositary receipts ("ADRs") and global depositary receipts ("GDRs") based on the securities in the underlying index. The underlying index is designed to represent the large-cap segment of the Indian equity markets. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.155594	2020-10-11 23:38:31.028166
PJP	Invesco Dynamic Pharmaceuticals	The investment seeks to track the investment results (before fees and expenses) of the Dynamic Pharmaceutical IntellidexSM Index.\n The fund generally will invest at least 90% of its total assets in common stocks of pharmaceutical companies that comprise the underlying intellidex. The underlying intellidex was composed of common stocks of 23 U.S. pharmaceuticals companies. These companies are engaged principally in the research, development, manufacture, sale or distribution of pharmaceuticals and drugs of all types. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.179708	2020-10-11 23:38:31.02423
PKG	Packaging Corporation of America	Packaging Corp of America is a producer of container board and corrugated products in the US. Further, it also produces multi-color boxes and displays, as well as meat boxes and wax-coated boxes for the agricultural industry.	{"Consumer Cyclical","Packaging & Containers"}	{GLT,IP,GPK,CLW,WRK}	Packaging & Containers	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:09.196863	2020-10-11 23:38:31.024353
PLG	Platinum Group Metals Ltd. (Canada)	Platinum Group Metals Ltd is a mineral exploration and development stage company. The Company is focused on the operation and development of platinum and palladium mining operations. Its projects include Maseve Mine and Waterberg.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{PLM}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:09.225878	2020-10-11 23:38:31.025871
PLXP	PLx Pharma Inc.	PLx Pharma Inc is a late-stage specialty pharmaceutical company initially focused on developing its clinically validated and patent-protected PLxGuard delivery system to provide safer and more effective aspirin products.	{Healthcare,Biotechnology}	{PRGO,AMPH,INVA,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:09.261498	2020-10-11 23:38:31.024353
PMPT	iSectors Post-MPT Growth ETF	The investment seeks to provide a high level of current income and, secondarily, capital appreciation.\n Under normal market conditions, the fund will invest not less than 80% of its net assets (plus the amount of any borrowings for investment purposes) in credit investments, and in derivatives and other instruments that have economic characteristics similar to such investments. Credit investments include corporate and sovereign bonds, including U.S. Treasury securities; loans (including, without limitation, through loan participations and assignments); and securitized instruments.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.285947	2020-10-11 23:38:31.02423
PNNT	PennantPark Investment Corporation	PennantPark Investment Corporation is an investment management company. Its investment objective is to generate current income and capital appreciation also seeking to preserve capital through debt and equity investments.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:09.312467	2020-10-11 23:38:31.023428
POPE	Pope Resources	Pope Resources LP is a US-based company that earns the majority of its revenue through growing and harvesting timber from land that it owns or co-owns with its investors. Its other activities include timber fund management and real estate investment.	{"Basic Materials","Lumber & Wood Production","Forest Products"}	{UFPI,RYN}	Forest Products	Basic Materials	NASDAQ Capital Market	\N	2020-10-11 23:12:09.422378	2020-10-11 23:38:31.028166
PPH	VanEck Vectors Pharmaceutical ETF	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the MVIS® US Listed Pharmaceutical 25 Index.\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The index includes common stocks and depositary receipts of U.S. exchange-listed companies in the pharmaceutical sector. Such companies may include medium-capitalization companies and foreign companies that are listed on a U.S. exchange. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.471009	2020-10-11 23:38:31.021426
PPX	PPL Capital Funding Inc. 2013 Series B Junior Subordinated Notes due 2073		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:09.488747	2020-10-11 23:38:31.02423
PRIM	Primoris Services Corporation	Primoris Services Corp provides construction, maintenance, replacement, water and waste-water and engineering services to public utilities, petrochemical companies, energy companies and municipalities.	{Industrials,"Engineering & Construction"}	{MTRX,STRL,GLDD,MYRG,MTZ,PWR,FLR,KBR,TPC}	Engineering & Construction	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:09.52555	2020-10-11 23:38:31.025871
PRPO	Precipio Inc.	Precipio Inc is a diagnostics company, which has built a platform designed to eradicate the problem of misdiagnosis by harnessing the expertise and technology developed within academic institutions and delivering diagnostic information to physicians.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{ILMN,HBIO,DGX,A,BIO,GE,TMO,PKI,LH,XLV}	Medical Diagnostics & Research	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:09.545255	2020-10-11 23:38:31.031328
QADA	QAD Inc.	QAD Inc provides enterprise software solutions for manufacturing companies across the automotive, life sciences, consumer products, food and beverage, high technology and industrial products industries.	{Technology,"Software - Application","Application Software"}	{MSFT,AZPN,AMSWA,ROK,GE,FLO,PRO,PTC,ORCL,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:09.856634	2020-10-11 23:38:31.023007
PIY	Preferred Plus Trust (Ser CZN) Ser CZN-1 Tr Ctf 8.375% Maturity 10/01/2046	Preferred Plus Trust is a trust company.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.17557	2020-10-11 23:38:31.023007
PKE	Park Electrochemical Corporation	Park Electrochemical Corp develops, manufactures, markets and sells digital and radio frequency microwave printed circuit materials products for the telecommunications and internet infrastructure and high-end computing markets.	{Technology,"Semiconductor Equipment & Materials",Semiconductors}	{FLEX,OLED,CCMP,TTMI,JBL,ROG,SANM,RBCN,XLK}	Semiconductors	Technology	New York Stock Exchange	\N	2020-10-11 23:12:09.197525	2020-10-11 23:38:31.023007
PLCE	Children's Place Inc. (The)	Children's Place Inc is an apparel retailer that operates stores in North America and sells through its website and wholesale. It designs, contracts to manufacture, and sells merchandise under the brand names The Children's Place, Place and Baby Place.	{"Consumer Cyclical","Apparel Stores","Retail - Apparel & Specialty"}	{URBN,DEST,AEO,GPS,KSS,CRI,ASNA}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:09.222837	2020-10-11 23:38:31.023007
PLUG	Plug Power Inc.	Plug Power Inc provides alternative energy technology. It is engaged in the design, development, commercialization and manufacture of hydrogen fuel cell systems used for the industrial off-road market and the stationary power market.	{Technology,"Electronic Components","Computer Hardware"}	{BLDP,AETI,APD,GE,LOW,PG,WMT,ESNC,CPST,FCEL,XLK}	Computer Hardware	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:09.247769	2020-10-11 23:38:31.02423
PME	Pingtan Marine Enterprise Ltd.	Pingtan Marine Enterprise Ltd through its subsidiaries is engaged in ocean fishing. It harvests a variety of fish species with its owned and licensed vessels operating within the Indian Exclusive Economic Zone and the Arafura Sea of Indonesia.	{"Consumer Defensive","Farm Products","Consumer Packaged Goods"}	{OMEX,CCCL}	Consumer Packaged Goods	Consumer Defensive	NASDAQ Capital Market	\N	2020-10-11 23:12:09.269243	2020-10-11 23:38:31.025871
PMT	PennyMac Mortgage Investment Trust of Beneficial Interest	PennyMac Mortgage Investment Trust is a real estate investment trust.The company operates through four segments namely correspondent production, credit sensitive strategies, interest rate sensitive strategies and corporate activities.	{"Real Estate","REIT - Diversified",REITs}	{NYMT,DX,CMO,MFA,IVR,MITT}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:09.289554	2020-10-11 23:38:31.021426
PNI	Pimco New York Municipal Income Fund II of Beneficial Interest	PIMCO New York Municipal Income Fund II is a United States based closed-end management investment company. Its investment objective is to seek to provide current income exempt from federal, New York State and New York City income tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.306831	2020-10-11 23:38:31.025871
PODD	Insulet Corporation	Insulet Corp is a medical device company based in Billerica, Massachusetts. The company develops, manufactures and sells OmniPod Insulin Management System, an insulin infusion system for people with insulin-dependent diabetes.	{Healthcare,"Medical Devices"}	{ICUI,DXCM,ABT,JNJ,LLY,MDT,TNDM,XLV}	Medical Devices	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:09.350261	2020-10-11 23:38:31.021426
PPA	Invesco Aerospace & Defense	The investment seeks to track the investment results (before fees and expenses) of the SPADE® Defense Index.\n The fund generally will invest at least 90% of its total assets in common stocks of that comprise the underlying index. The underlying index was composed of common stocks of 54 U.S. companies whose shares are listed on the New York Stock Exchange ("NYSE") or the NASDAQ. These companies are engaged principally in the development, manufacture, operation and support of U.S. defense, military, homeland security and space operations. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.457539	2020-10-11 23:38:31.02582
PPLT	Aberdeen Standard Physical Platinum Shares	The investment seeks to reflect the performance of the price of physical platinum, less the expenses of the Trust’s operations.\n The fund designed for investors who want a cost-effective and convenient way to invest in platinum with minimal credit risk. Advantages of investing in the Shares include Ease and Flexibility of Investment, Expenses, Minimal Credit Risk.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.478211	2020-10-11 23:38:31.023428
PRAA	PRA Group Inc.	PRA Group Inc is a United States-based company that primarily acquires, collects, and processes nonperforming loans. The company generates the majority of its sales from the United States.	{"Financial Services","Credit Services"}	{ECPG,RLGY,RMAX}	Credit Services	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:09.496954	2020-10-11 23:38:31.025871
PRFT	Perficient Inc.	Perficient Inc provides a variety of information technology services such as big data analytics, technology platform implementations, enterprise content management, portals and collaboration, management consulting, business integration and others.	{Technology,"Information Technology Services","Application Software"}	{MSFT,CTSH,VRTU,IBM,WIT,CRM,ACN,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:09.511517	2020-10-11 23:38:31.025871
PRK	Park National Corporation	Park National Corp is a financial bank holding company. It provides commercial banking services to small and medium population Ohio communities.	{"Financial Services","Banks - Regional - US",Banks}	{PEBO,OVBC,UBOH,FFBC,HBAN,WSBC,LCNB,FMNB,SBFG}	Banks	Financial Services	NYSE American	\N	2020-10-11 23:12:09.528849	2020-10-11 23:38:31.02423
PJC	Piper Jaffray Companies	Piper Jaffray Cos is an investment bank and asset management firm, serving the needs of corporations, private equity groups, public entities, non-profit entities and institutional investors in the U.S. and internationally.	{"Financial Services","Asset Management"}	{SF,EVR,OPY,COWN}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.176653	2020-10-11 23:38:31.028166
PKO	Pimco Income Opportunity Fund of Beneficial Interest	PIMCO Income Opportunity Fund is a closed-end management investment. The company's primary investment objective is to seek high current income, with capital appreciation as a secondary objective.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.200819	2020-10-11 23:38:31.031328
PLM	Polymet Mining Corporation (Canada)	PolyMet Mining Corp is engaged in the exploration and development of natural resource properties. The company's primary mineral property is the NorthMet Project, which is a polymetallic asset in northeastern Minnesota, USA.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{HBM,WRN,PLG}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:09.224108	2020-10-11 23:38:31.02423
PLW	Invesco 1-30 Laddered Treasury ETF	The investment seeks to track the investment results (before fees and expenses) of the Ryan/NASDAQ U.S. 1-30 Year Treasury Laddered Index (the "underlying index").\n The fund generally invests at least 80% of its total assets in U.S. Treasury securities that comprise the underlying index. The index seeks to maintain a continuous maturity laddered portfolio of securities, meaning that securities holdings are scheduled to mature in a proportional, annual sequential pattern. Nasdaq, Inc. (the "index provider") allows a six-month maturity deviation if securities with a desired maturity date are not available.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.249103	2020-10-11 23:38:31.023007
PMO	Putnam Municipal Opportunities Trust	Putnam Municipal Opportunities Trust is a closed end fund. It seeks to provide as high a level of current income free from federal income tax as Putnam Management believes is consistent with the preservation of capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.280924	2020-10-11 23:38:31.028166
PNQI	PowerShares NASDAQ Internet Portfolio	The investment seeks to track the investment results (before fees and expenses) of the NASDAQ Internet IndexSM.\n The fund generally will invest at least 90% of its total assets in securities that comprise the underlying index. The underlying index is designed to track the performance of the largest and most liquid U.S.-listed companies engaged in Internet-related businesses that are listed on one of the three major U.S. stock exchanges. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.317388	2020-10-11 23:38:31.02582
PPBI	Pacific Premier Bancorp Inc	Pacific Premier Bancorp Inc, through its subsidiaries, provides banking products and services. It offers cash management, electronic banking services, credit facilities and retirement accounts, among others.	{"Financial Services","Banks - Regional - US",Banks}	{CATY,CVBF,EWBC,CVCY,BSRR,BOCH,BMRC,PACW}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:09.458243	2020-10-11 23:38:31.025871
PPR	Voya Prime Rate Trust Shares of Beneficial Interest	Voya Prime Rate Trust is a United States based diversified, closed-end, management investment company. It seeks to provide investors with as high a level of current income as is consistent with the preservation of capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.479617	2020-10-11 23:38:31.02582
PRAH	PRA Health Sciences Inc.	PRA Health Sciences Inc is a United State based full-service global contract research organization. It provides a broad range of product development services for pharmaceutical and biotechnology companies around the world.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{NVAX,FLXN,XLV}	Medical Diagnostics & Research	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:09.497545	2020-10-11 23:38:31.023007
PRGO	Perrigo Company plc	Perrigo Co PLC is a healthcare company providing branded OTC products throughout Europe and the USA. It sells its products primarily in North America and Europe, as well as in other markets, including Australia, Israel and China.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{UTHR,CTLT,XLV}	Drug Manufacturers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:09.512252	2020-10-11 23:38:31.024353
PRQR	ProQR Therapeutics N.V.	ProQR Therapeutics NV is a biopharmaceutical company engaged in the discovery & development of RNA-based therapeutics for the treatment of severe genetic disorders.	{Healthcare,Biotechnology}	{ECYT,ATNM,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:09.551721	2020-10-11 23:38:31.023007
PSA	Public Storage	Public Storage is a part of the real estate sector. It is engaged in the acquisition, development, ownership and operation of self-storage facilities.	{"Real Estate","REIT - Industrial",REITs}	{PSB,EXR,LSI}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:09.574146	2020-10-11 23:38:31.024353
QDEL	Quidel Corporation	Quidel Corp is engaged in the development, manufacturing, and marketing of rapid diagnostic testing solutions. It includes applications in infectious diseases, women's health, gastrointestinal diseases, and others.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{OSUR,VIVO,BDX,DHR,TMO,PKI,BIO.B,XLV}	Medical Diagnostics & Research	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:09.883912	2020-10-11 23:38:31.02582
PPLN	Cushing 30 MLP Index ETNs due June 15 2037	The investment seeks return linked to the performance of the Cushing® 30 MLP Index (the index).\nThe payment on the notes at maturity or upon early repurchase or redemption is based on the VWAP level of the index. The notes may pay a variable quarterly coupon based on cash distributions on the components of the index. The index tracks the performance of publicly traded equity securities of 30 U.S.-based companies that are engaged in, or that hold an entity that is engaged in, the midstream activities of transportation, storage, processing or production of energy commodities.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.477829	2020-10-11 23:38:31.025871
PRB	VanEck Vectors Pre-refunded Municipal Index	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the Bloomberg Barclays Municipal Pre-Refunded-Treasury-Escrowed Index.\n The fund normally invests at least 80% of its total assets in securities that comprise the benchmark index. The index is comprised of publicly traded municipal bonds that cover the U.S. dollar denominated tax-exempt bond market. Constituent bonds include pre-refunded and/or escrowed-to-maturity municipal bonds.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.499535	2020-10-11 23:38:31.023428
PRGS	Progress Software Corporation	Progress Software Corp is involved in application development, empowering enterprises to build mission critical business applications. It offers spanning web, mobile and data for on-premise and cloud environments.	{Technology,"Software - Application","Application Software"}	{MSFT,CTXS,IBM,ATTU,ORCL,HPQ,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:09.512705	2020-10-11 23:38:31.021426
PRNT	3D Printing (The)	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the Total 3D-Printing Index ("index").\n The fund normally invests at least 80% of its total assets in securities that are included in the fund's benchmark index, depositary receipts representing securities included in the index or underlying stocks in respect of depositary receipts included in the index. The index is designed to track the price movements of stocks of companies involved in the 3D printing industry. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:09.539432	2020-10-11 23:38:31.02423
PRTA	Prothena Corporation plc	Prothena Corp PLC is a biotechnology company. It discovers and develops novel antibodies for the potential treatment of a broad range of diseases that involve protein misfolding or cell adhesion.	{Healthcare,Biotechnology}	{SHPG,ALNY,AMRN,ALKS,MNK,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:09.561618	2020-10-11 23:38:31.025871
PSCC	PowerShares S&P SmallCap Consumer Staples Ptf	The investment seeks to track the investment results (before fees and expenses) of the S&P SmallCap 600® Capped Consumer Staples Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in common stocks of small capitalization U.S. consumer staples companies that comprise the underlying index. These companies are principally engaged in the business of providing consumer goods and services that have non-cyclical characteristics, including tobacco, food and beverage, and non-discretionary retail. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.590148	2020-10-11 23:38:31.028166
PSDO	Presidio Inc.	Presidio Inc is a provider of information technology solutions to the middle market in North America. Its services include consulting, data analytics, deployment and integration, Internet of Things (IoT) solutions, cloud and hosting solutions.	{Technology,"Information Technology Services","Application Software"}	{NTGR,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:09.615653	2020-10-11 23:38:31.028166
PSMG	Invesco Growth Multi-Asset Allocation	The investment seeks to provide long-term capital appreciation.\n The fund seeks to achieve its investment objective by allocating its assets using a growth investment style that seeks to maximize the benefits of diversification, which focuses on investing a greater portion of fund assets in Equity ETFs, but also provides some exposure to Fixed Income ETFs. Specifically, the fund's target allocation is to invest approximately 65%-95% of its total assets in Equity ETFs and approximately 5%-35% of its total assets in Fixed Income ETFs. It is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:09.646321	2020-10-11 23:38:31.024353
PSTI	Pluristem Therapeutics Inc.	Pluristem Therapeutics Inc is a bio-therapeutics company. It is engaged in developing off-the-shelf allogeneic cell therapy products for treatment of ischemic & inflammatory, cardiovascular, orthopedic, pulmonary, hematological, & women's health diseases.	{Healthcare,Biotechnology}	{UTHR,CELG,IMMU,OSIR,ATHX,BTX,VCEL,CBLI,CLBS,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:09.668954	2020-10-11 23:38:31.023007
PTI	Proteostasis Therapeutics Inc.	Proteostasis Therapeutics Inc operates in the healthcare sector of United States. Its business involves the discovery and development of therapeutics to correct the imbalance of the protein biosynthesis process.	{Healthcare,Biotechnology}	{NEO,NVTA,ONCS,NTRA,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:09.700331	2020-10-11 23:38:31.028166
PTX	Pernix Therapeutics Holdings Inc.	Pernix Therapeutics Holdings Inc is a specialty pharmaceutical company engaged in identifying, developing and commercializing differentiated products that address unmet medical needs.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{VNDA,CPIX,LCI,NVS,JNJ,LLY,PFE,EBS,MRK,PRTK,XLV}	Drug Manufacturers	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:09.72977	2020-10-11 23:38:31.02423
PVI	Invesco VRDO Tax Free Weekly	The investment seeks to track the investment results (before fees and expenses) of the Bloomberg U.S. Municipal AMT-Free Weekly VRDO Index (the "underlying index").\n The fund generally will invest at least 80% of its total assets in variable rate demand obligation bonds that are exempt from federal income tax with interest rates that reset weekly, which comprise the underlying index, which is comprised of municipal securities issued in the primary market as VRDOs.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.770585	2020-10-11 23:38:31.023428
PRLB	Proto Labs Inc.	Proto Labs Inc is an online and technology-enabled manufacturer of on-demand three-dimensional printed, computer numerical control machined and injection-molded custom parts for prototyping and short-run production.	{Industrials,"Tools & Accessories","Industrial Products"}	{AIN,KAI,JBT,DDD,XONE,VJET}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:09.530835	2020-10-11 23:38:31.028166
PRSC	The Providence Service Corporation	Providence Service Corp s an US based company, provides and manages government sponsored non-emergency transportation and workforce development services.	{Healthcare,"Medical Care","Health Care Providers"}	{BIOS,BKD,ACHC,XLV}	Health Care Providers	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:09.558295	2020-10-11 23:38:31.028166
PSCF	PowerShares S&P SmallCap Financials Portfolio	The investment seeks to track the investment results (before fees and expenses) of the S&P SmallCap 600® Capped Financials & Real Estate Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in common stocks of small capitalization U.S. financial service companies that comprise the underlying index. These companies are principally engaged in the business of providing financial services and products, including banking, investment services, insurance and real estate finance services.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.598857	2020-10-11 23:38:31.028166
PSL	Invesco DWA Consumer Staples Momentum ETF	The investment seeks to track the investment results (before fees and expenses) of the Dorsey Wright® Consumer Staples Technical Leaders Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities that comprise the underlying index. The underlying index is composed of at least 30 securities of companies in the consumer staples sector that have powerful relative strength or "momentum" characteristics.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.629183	2020-10-11 23:38:31.028166
PSO	Pearson Plc	Pearson PLC is a content publisher. Its products comprise of educational courseware, assessments and services, and consumer publishing.	{"Consumer Cyclical",Publishing}	{SCHL,DIS,CBS.A,NWSA,FOXA,TGNA,RELX,SPGI}	Publishing	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:09.649319	2020-10-11 23:38:31.028166
PTC	PTC Inc.	PTC Inc is a US-based company develops and delivers technology solutions comprised of software and services that help companies design products, manage product information and improve their product development and services processes.	{Technology,"Software - Application","Application Software"}	{AZPN,ANSS,ADBE,ADSK,SNPS,GE,IBM,QADB,ORCL,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:09.678794	2020-10-11 23:38:31.024353
PTIE	Pain Therapeutics Inc.	Pain Therapeutics Inc is a biopharmaceutical company that develops novel drugs. The Company has four drug candidates in clinical programs, including Remoxy, Oxytrex, PTI-202 and a novel radio-labeled monoclonal antibody to treat metastatic melanoma.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:09.70342	2020-10-11 23:38:31.02423
PTY	Pimco Corporate & Income Opportunity Fund	PIMCO Corporate & Income Oppurtunity Fds operates as a closed end management investment company. It seeks to maximize total return through a combination of current income and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.737224	2020-10-11 23:38:31.023428
PVAC	Penn Virginia Corporation	Penn Virginia Corp is an independent oil and gas company. The company is engaged in the exploration, development, and production of crude oil, natural gas liquids, and natural gas in onshore regions of the United States.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{}	Oil & Gas - E&P	Energy	Nasdaq Global Select	\N	2020-10-11 23:12:09.760171	2020-10-11 23:38:31.028166
PWV	Invesco Dynamic Large Cap Value	The investment seeks to track the investment results (before fees and expenses) of the Dynamic Large Cap Value IntellidexSM Index.\n The fund generally will invest at least 90% of its total assets in common stocks of large capitalization companies that comprise the underlying intellidex. The underlying intellidex for the fund is composed of 50 large capitalization U.S. value stocks that, strictly in accordance with its guidelines and mandated procedures, the intellidex provider includes principally on the basis of their capital appreciation potential.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.784106	2020-10-11 23:38:31.025871
PXMG	Invesco Russell MidCap Pure Growth	The investment seeks to track the investment results (before fees and expenses) of the Russell Midcap® Pure Growth Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the component securities that comprise the underlying index. The underlying index is composed of a subset of securities from the Russell Midcap® Index, which is composed of the smallest 800 securities of the Russell 1000® Index, an index designed to measure the performance of the largest 1,000 companies in the U.S. equity market.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.804099	2020-10-11 23:38:31.028166
PYDS	Payment Data Systems Inc.	Payment Data Systems Inc is a United States based company engaged in providing integrated payment solutions. It offers range of customised services to merchants, billers, banks, service bureaus and card issuers.	{Technology,"Software - Infrastructure","Application Software"}	{EEFT,GPN,WU,XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:09.827895	2020-10-11 23:38:31.028166
QADB	QAD Inc.	QAD Inc provides enterprise software solutions for manufacturing companies across the automotive, life sciences, consumer products, food and beverage, high technology and industrial products industries.	{Technology,"Software - Application","Application Software"}	{MSFT,AZPN,AMSWA,ROK,GE,FLO,PRO,PTC,ORCL,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:09.867748	2020-10-11 23:38:31.028166
PRN	Invesco DWA Industrials Momentum ETF	The investment seeks to track the investment results (before fees and expenses) of the Dorsey Wright® Industrials Technical Leaders Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities that comprise the underlying index. The underlying index is composed of at least 30 securities of companies in the industrials sector that have powerful relative strength or "momentum" characteristics.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.538355	2020-10-11 23:38:31.024353
PRTS	U.S. Auto Parts Network Inc.	U.S. Auto Parts Network Inc is an online provider of automotive aftermarket parts and repair information. Its products are classified into three subcategories: collision parts, engine parts, and performance parts and accessories.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{AMZN,CNXN,PCMI,OSTK,LKQ,STMP,AZO,GPC,AAP,ORLY}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:09.564246	2020-10-11 23:38:31.02423
PSC	Principal U.S. Small Cap Index ETF	The investment seeks investment results that closely correspond, before expenses, to the performance of the Nasdaq U.S. Small Cap Select Leaders Index.\n The fund invests at least 80% of its net assets, plus any borrowings for investment purposes, in equity securities of companies that compose the index. The index uses a quantitative model designed to identify equity securities (including growth and value stock) of small-capitalization companies in the Nasdaq US Small Cap Index (the "parent index") that exhibit potential for high degrees of sustainable shareholder yield, pricing power and strong momentum, while adjusting for liquidity and quality.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.589717	2020-10-11 23:38:31.02423
PSEC	Prospect Capital Corporation	Prospect Capital Corporation is a closed-end investment company. It aims to generate current income and long-term capital appreciation through debt and equity investments.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:09.618176	2020-10-11 23:38:31.031328
PSMC	Invesco Conservative Multi-Asset Allocation	The investment seeks total return consistent with a lower level of risk relative to the broad stock market.\n The fund seeks to achieve its investment objective by allocating its assets using a conservative investment style that seeks to maximize the benefits of diversification, which focuses on investing a greater portion of fund assets in Fixed Income ETFs, but also provides some exposure to underlying ETFs that invest primarily in Equity ETFs. Specifically, the fund's target allocation is to invest approximately 5%-35% of its total assets in Equity ETFs and approximately 65%-95% of its total assets in Fixed Income ETFs. It is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:09.639648	2020-10-11 23:38:31.031328
PSTG	Pure Storage Inc. Class A	Pure Storage Inc provides data storage services. It provides flash optimized software, flash array hardware and cloud based management services.	{Technology,"Data Storage","Computer Hardware"}	{STX,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:09.661553	2020-10-11 23:38:31.02582
PTGX	Protagonist Therapeutics Inc.	Protagonist Therapeutics Inc is a clinical-stage biopharmaceutical company with a proprietary peptide-based technology platform focused on discovering and developing new chemical entities to address significant unmet medical needs.	{Healthcare,Biotechnology}	{PDLI,PIRS,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:09.690936	2020-10-11 23:38:31.023428
PTSI	P.A.M. Transportation Services Inc.	P.A.M. Transportation Services Inc through its subsidiaries operates as a truckload transportation and logistics company. The company is a truckload dry-van carrier, transporting general commodities throughout the continental United States and Canada.	{Industrials,Trucking,"Transportation & Logistics"}	{CVTI,HTLD,MRTN,USAK,WERN,ULH,RRTS,KNX}	Transportation & Logistics	Industrials	NASDAQ Global Market	\N	2020-10-11 23:12:09.730044	2020-10-11 23:38:31.028166
PVBC	Provident Bancorp Inc.	Provident Bancorp Inc provides a range of banking products and services to small and medium-sized commercial customers. The company, through its subsidiary, is a Massachusetts-chartered stock savings bank.	{"Financial Services","Banks - Regional - US",Banks}	{MPB,MOFG}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:09.759578	2020-10-11 23:38:31.028166
PWR	Quanta Services Inc.	Quanta Services Inc is a provider of specialty contracting services. The company offers infrastructure solutions to the electric power and oil and gas industries in North America and in select international markets.	{Industrials,"Engineering & Construction"}	{MYRG,PRIM,MTZ,EME,DY,FLR,IESC}	Engineering & Construction	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:09.781047	2020-10-11 23:38:31.02582
PXJ	Invesco Dynamic Oil & Gas Services	The investment seeks to track the investment results (before fees and expenses) of the Dynamic Oil Services IntellidexSM Index.\n The fund generally will invest at least 90% of its total assets in common stocks of companies that assist in the production, processing and distribution of oil and gas that comprise the underlying intellidex. The underlying intellidex was composed of common stocks of 30 U.S. companies that assist in the production, processing and distribution of oil and gas. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.795727	2020-10-11 23:38:31.021426
PXS	Pyxis Tankers Inc.	Pyxis Tankers Inc is an international maritime transportation company. It owns a fleet which comprises of six double hull product tankers which include Pyxis epsilon, Pyxis theta, Pyxis malou, Pyxis delta, Northsea alpha and Northsea beta.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{NM,NMM}	Transportation & Logistics	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:09.812919	2020-10-11 23:38:31.023007
PRME	First Trust Heitman Global Prime Real Estate	The investment seeks to provide long-term total return.\n Under normal market conditions, the fund will seek to achieve its investment objective by investing at least 80% of its net assets (including investment borrowings) in U.S. and non-U.S. exchange-traded real estate securities, which includes real estate investment trusts ("REITs"), real estate operating companies ("REOCs") and common stocks or depositary receipts of companies primarily engaged in the real estate industry (collectively, "Real Estate Securities"). It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.535457	2020-10-11 23:38:31.028166
PRTK	Paratek Pharmaceuticals Inc.	Paratek Pharmaceuticals Inc is a clinical-stage biopharmaceutical company. It is focused on the development and commercialization of therapeutics based upon tetracycline chemistry.	{Healthcare,Biotechnology}	{NBIX,AKRX,BDSI,VNDA,PFE,MRK,JAZZ,TTNP,PTX,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:09.56642	2020-10-11 23:38:31.02582
PSF	Cohen & Steers Select Preferred and Income Fund Inc.	Cohen & Steers Select Preferred and Income Fund is an investment management company. It focusses on investing for the purpose of securing a higher income rate for its investors by placing funds primarily in the banking sector.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.625656	2020-10-11 23:38:31.025871
PSMT	PriceSmart Inc.	Pricesmart Inc through its subsidiaries is engaged in the ownership and operation of membership shopping warehouse clubs under the trade name PriceSmart in Central America and the Caribbean.	{"Consumer Defensive","Discount Stores","Retail - Defensive"}	{COST,DLTR,WMT,TGT,BIG,DG,FIVE}	Retail - Defensive	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:09.649889	2020-10-11 23:38:31.021426
PTEN	Patterson-UTI Energy Inc.	Patterson-UTI Energy Inc is a North America-based land rig drilling contractors. It operates in three segments namely, Contract Drilling Services, Pressure Pumping Services, and Oil and Natural Gas Exploration and Production.	{Energy,"Oil & Gas Drilling","Oil & Gas - Drilling"}	{PDS,DO,HP,UNT,NBR,RIG,PES}	Oil & Gas - Drilling	Energy	Nasdaq Global Select	\N	2020-10-11 23:12:09.683454	2020-10-11 23:38:31.02423
PTNQ	Pacer Trendpilot 100	The investment seeks to track the total return performance, before fees and expenses, of the Pacer NASDAQ-100 Trendpilot Index.\n The fund invests at least 80% of its total assets (exclusive of collateral held from securities lending) in the component securities of the index. The index implements a systematic trend-following strategy that directs exposure (i) 100% to the NASDAQ-100, (ii) 50% to the NASDAQ-100 and 50% to 3‑Month U.S. Treasury bills, or (iii) 100% to 3-Month U.S. Treasury bills, depending on the relative performance of the NASDAQ-100 TR and its 200-business day historical simple moving average. It is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:09.723302	2020-10-11 23:38:31.02582
PUMP	ProPetro Holding Corp.	ProPetro Holding Corp is an oilfield services company providing hydraulic fracturing and other complementary services to upstream oil and gas companies.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{PUMP}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:09.753302	2020-10-11 23:38:31.031328
PWOD	Penns Woods Bancorp Inc.	Penns Woods Bancorp Inc is a bank holding company. It is engaged in a full-service commercial banking business, making available to the community a range of financial services including installment loans, credit cards, among others.	{"Financial Services","Banks - Regional - US",Banks}	{CZNC,DNBF,ORRF,FRBK,BMTC,FULT,STBA,UVSP,FCF,ACNB}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:09.776465	2020-10-11 23:38:31.028166
PXLV	Invesco Russell Top 200 Pure Value	The investment seeks to track the investment results (before fees and expenses) of the Russell Top 200® Pure Value Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the component securities that comprise the underlying index. The underlying index is composed of a subset of securities from the Russell Top 200® Index, which is composed of the largest 200 securities of the Russell 3000® Index, an index designed to measure the performance of the largest 3,000 companies in the U.S. equity market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.801145	2020-10-11 23:38:31.02423
PYT	PPlus Tr GSC-2 Tr Ctf Fltg Rate		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:09.833653	2020-10-11 23:38:31.023428
QAT	iShares MSCI Qatar Capped ETF	The investment seeks to track the investment results of the MSCI All Qatar Capped Index.\n The fund will invest at least 90% of its assets in the component securities of the index and in investments that have economic characteristics that are substantially identical to the component securities of the index. The index, which is designed to measure the equity market in Qatar, is a free float-adjusted market capitalization-weighted index with a capping methodology applied to issuer weights. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.867758	2020-10-11 23:38:31.024353
QDEF	FlexShares Quality Dividend Defensive Index Fund	The investment seeks investment results that correspond generally to the price and yield performance, before fees and expenses, of the Northern Trust Quality Dividend Defensive IndexSM.\n The fund will invest at least 80% of its total assets (exclusive of collateral held from securities lending) in the securities of the underlying index. The underlying index is designed to provide exposure to a high-quality, income-oriented portfolio of U.S. equity securities, with an emphasis on long-term capital growth and a targeted overall volatility that is lower than that of the Northern Trust 1250 IndexSM.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.883789	2020-10-11 23:38:31.023428
PRTO	Proteon Therapeutics Inc.	Proteon Therapeutics Inc is an late-stage biopharmaceutical company engaged in the development of novel pharmaceuticals to address the needs of patients with renal and vascular disease.	{Healthcare,Biotechnology}	{AGEN,SRPT,AFMD,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:09.56433	2020-10-11 23:38:31.028166
PSCH	PowerShares S&P SmallCap Health Care Portfolio	The investment seeks to track the investment results (before fees and expenses) of the S&P SmallCap 600® Capped Health Care Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in common stocks of small capitalization U.S. healthcare companies that comprise the underlying index. These companies are principally engaged in the business of providing healthcare-related products, facilities and services, including biotechnology, pharmaceuticals, medical technology and supplies.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.598896	2020-10-11 23:38:31.023007
PSLV	Sprott Physical Silver Trust ETV	Sprott Physical Silver Trust is a closed-end mutual fund trust, created to invest and hold substantially all of its assets in physical silver bullion.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE Arca	\N	2020-10-11 23:12:09.63225	2020-10-11 23:38:31.02582
PSXP	Phillips 66 Partners LP representing limited partner interest in the Partnership	Phillips 66 Partners LP is engaged in transporting crude, natural gas liquids, and refined products. It mainly owns, operates, develops and acquires fee-based crude oil, refined petroleum product and natural gas liquids pipelines and terminals.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{PAA,MMP,HEP,TLP,NS,BKEP,MPLX,PAGP}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:09.673015	2020-10-11 23:38:31.021426
PTLA	Portola Pharmaceuticals Inc.	Portola Pharmaceuticals Inc is engaged in the healthcare sector. It develops & commercializes novel therapeutics to combat thrombosis, hematologic disorders & inflammation.	{Healthcare,Biotechnology}	{BIIB,BMY,JNJ,PFE,SNY,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:09.703454	2020-10-11 23:38:31.025871
PUW	Invesco WilderHill Progressive Energy	The investment seeks to track the investment results (before fees and expenses) of the WilderHill Progressive Energy Index.\n The fund generally will invest at least 90% of its total assets in common stocks that comprise the underlying index. The underlying index was composed of common stocks of 42 companies that are publicly traded in the United States and are engaged in a business or businesses that the index provider believes may substantially benefit from a societal shift toward the transitional energy technologies significant for reducing carbon dioxide, less-polluting use of fossil fuels, and improving energy efficiency.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.755299	2020-10-11 23:38:31.02423
PWC	Invesco Dynamic Market	The investment seeks to track the investment results (before fees and expenses) of the Dynamic Market IntellidexSM Index.\n The fund generally will invest at least 90% of its total assets in common stocks that comprise the underlying intellidex. The underlying intellidex was composed of 100 U.S. stocks that the intellidex provider, strictly in accordance with its guidelines and mandated procedures, included pursuant to a proprietary selection methodology. Stocks are selected from the top of each sector and size category in a manner designed to produce an index with sector and size dispersion similar to the overall broad market.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.776116	2020-10-11 23:38:31.028166
PXF	Invesco FTSE RAFI Developed Markets ex-U.S.	The investment seeks to track the investment results (before fees and expenses) of the FTSE RAFI Developed ex U.S. 1000 Index (the "underlying index").\n The fund will invest at least 90% of its total assets in the securities of companies originating in countries that are classified as "developed" within the country classification definition of FTSE International Limited, excluding the United States, and that comprise the underlying index, as well as American depositary receipts ("ADRs") and global depositary receipts ("GDRs") that are based on the securities in the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.794852	2020-10-11 23:38:31.028166
PXUS	Principal International Multi-Factor Index ETF	The investment seeks to provide investment results that closely correspond, before expenses, to the performance of the Nasdaq Global Millennial Opportunity Index (the "index").\n Under normal circumstances, the fund invests at least 80% of its net assets, plus any borrowings for investment purposes, in equity securities of companies that compose the index at the time of purchase. The index uses a quantitative model designed to identify equity securities of companies in the Nasdaq Global Index (the "parent index") that are impacted by the spending and lifestyle activities of the Millennial generation, which refers to people born from 1980 to the mid-2000s.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.81421	2020-10-11 23:38:31.02582
PZC	PIMCO California Municipal Income Fund III of Beneficial Interest	PIMCO CA Muni Income Fund III operates as a closed-end management investment company. The investment objective of the fund is to seek to provide current income exempt from federal and California income tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.839332	2020-10-11 23:38:31.031328
QAI	IQ Hedge MultiIQ Hedge Multi-Strategy Tracker	The investment seeks investment results that correspond generally to the price and yield performance of its underlying index, the IQ Hedge Multi-Strategy Index.\n The fund is a "fund of funds" which means it invests, under normal circumstances, at least 80% of its net assets, plus the amount of any borrowings for investment purposes, in the investments included in its underlying index, which includes underlying funds. The underlying index consists of a number of components ("underlying index Components") selected in accordance with IndexIQ's rules-based methodology of such underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.867919	2020-10-11 23:38:31.031328
PRTY	Party City Holdco Inc.	Party City Holdco Inc is a party goods retailer in North America. It designs, manufactures, contracts for manufacture and distributes party goods, including paper and plastic tableware, metallic and latex balloons, accessories, novelties and among others.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{TIF,SPWH,BOOT,RH}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:09.572773	2020-10-11 23:38:31.023007
PSET	Principal Price Setters Index ETF	The investment seeks to provide investment results that closely correspond, before expenses, to the performance of the Nasdaq U.S. Price Setters Index.\n Under normal circumstances, the fund invests at least 80% of its net assets, plus any borrowings for investment purposes, in equity securities of companies that compose the index at the time of purchase. The index uses a quantitative model designed to identify equity securities of mid- to large-capitalization companies in the Nasdaq US Large Mid Cap Index (the "parent index") that exhibit high degrees of pricing power.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.618274	2020-10-11 23:38:31.028166
PSMM	Invesco Moderately Conservative Multi-Asset Allocation	The investment seeks to provide current income and some capital appreciation.\n The fund seeks to achieve its investment objective by allocating its assets using a moderately conservative investment style that seeks to maximize the benefits of diversification, which focuses on investing a greater portion of fund assets in Fixed Income ETFs, but also provides some exposure to Equity ETFs. Specifically, the fund's target allocation is to invest approximately 25%-55% of its total assets in Equity ETFs and approximately 45%-75% of its total assets in Fixed Income ETFs. It is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:09.64606	2020-10-11 23:38:31.025871
PSX	Phillips 66	Phillips 66 is an oil and gas company that purchases for resale and markets refined petroleum products. Its joint venture operates facilities in the United States and the Middle East and primarily produces olefins and polyolefins.	{Energy,"Oil & Gas Refining & Marketing","Oil & Gas - Refining & Marketing"}	{CLMT,VLO,XOM,CVX,MPC,HFC}	Oil & Gas - Refining & Marketing	Energy	New York Stock Exchange	\N	2020-10-11 23:12:09.675799	2020-10-11 23:38:31.025871
PTMC	Pacer Trendpilot US Mid Cap	The investment seeks to track the total return performance, before fees and expenses, of the Pacer Trendpilot US Mid Cap Index.\n The fund invests at least 80% of its total assets (exclusive of collateral held from securities lending) in the component securities of the index. The index implements a systematic trend-following strategy that directs exposure (i) 100% to the &P MidCap 400, (ii) 50% to the S&P MidCap 400 and 50% to 3-Month U.S. Treasury bills, or (iii) 100% to 3-Month U.S. Treasury bills, depending on the relative performance of the S&P MidCap 400 TR and its 200-business day historical simple moving average.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:09.706276	2020-10-11 23:38:31.024353
PUK	Prudential Public Limited Company	Prudential PLC is a financial services group that offers life insurance, long-term savings and protection products, retirement income solutions and asset management services throughout Asia, United States and United Kingdom.	{"Financial Services","Insurance - Life"}	{MFC,AEG,MET,LFC}	Insurance - Life	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.73813	2020-10-11 23:38:31.024353
PW	Power REIT (MD)	Power REIT holds, develops, acquires and manages real estate assets related to transportation and energy infrastructure in the United States.	{"Real Estate","REIT - Diversified",REITs}	{PCH,EPR,LAND,CTT,SIR}	REITs	Real Estate	NYSE American	\N	2020-10-11 23:12:09.77038	2020-10-11 23:38:31.023007
PXD	Pioneer Natural Resources Company	Pioneer Natural Resources Co is an independent oil and gas exploration and production company with operations in the United States. Its focus includes the Permian Basin and Eagle Ford Shale regions of Texas.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{DVN,APC,APA,EGN,EOG,MRO,NBL,CXO,QEP,LPI}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:09.788237	2020-10-11 23:38:31.028166
PXQ	Invesco Dynamic Networking	The investment seeks to track the investment results (before fees and expenses) of the Dynamic Networking IntellidexSM Index.\n The fund generally will invest at least 90% of its total assets in common stocks of networking companies that comprise the underlying intellidex. The underlying intellidex was composed of common stocks of 30 U.S. networking companies. These are companies that are principally engaged in the development, manufacture, sale or distribution of products, services or technologies that support the flow of electronic information, including voice, data, images and commercial transactions. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.809211	2020-10-11 23:38:31.024353
PYS	Merrill Lynch Depositor Inc PPlus Tr Ser RRD-1 Tr Ctf Cl A		{"Financial Services","Capital Markets","Brokers & Exchanges"}	{}	Brokers & Exchanges	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.828626	2020-10-11 23:38:31.025871
PZZA	Papa John's International Inc.	Papa John's International Inc operates and franchises pizza delivery and carryout restaurants activities. The company also operates dine-in and delivery restaurants under the trademark "Papa John's".	{"Consumer Cyclical",Restaurants}	{NATH,JACK,TAST,YUM,DPZ,WEN,PBPB,FRSH,RAVE}	Restaurants	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:09.850742	2020-10-11 23:38:31.025871
QGEN	Qiagen N.V.	Qiagen NV provides sample and assay technologies to extract, purify, amplify, and interpret DNA (Deoxyribonucleic acid), RNA (Ribonucleic acid), and proteins. It also offers kits, assays, related automated systems and bioinformatics solutions.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{HOLX,GHDX,GNMK,BDX,GE,TMO,TECH,XLV}	Medical Diagnostics & Research	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:09.904832	2020-10-11 23:38:31.028166
PSB	PS Business Parks Inc.	PS Business Parks Inc is a fully-integrated, self-advised and self-managed real estate investment trust. The company owns, operates, acquires and develops commercial properties, including multi-tenant flex, office and industrial space.	{"Real Estate","REIT - Diversified",REITs}	{PSA,WRE,VER}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:09.584795	2020-10-11 23:38:31.023428
PSCT	PowerShares S&P SmallCap Information Tech Ptf	The investment seeks to track the investment results (before fees and expenses) of the S&P SmallCap 600® Capped Information Technology Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in common stocks of small capitalization U.S. information technology companies that comprise the underlying index. These companies are principally engaged in the business of providing information technology-related products and services, including computer hardware and software, Internet, electronics and semiconductors, and communication technologies.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.604934	2020-10-11 23:38:31.023428
PSK	SPDR Wells Fargo Preferred Stock	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of an index.\n In seeking to track the performance of Wells Fargo Hybrid and Preferred Securities Aggregate Index (the "index"), the fund employs a sampling strategy. It generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index is a modified market capitalization weighted index designed to measure the performance of non-convertible preferred stock and securities that are functionally equivalent to preferred stock. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.6277	2020-10-11 23:38:31.023428
PST	ProShares UltraShort Lehman 7-10 Year Treasury	The investment seeks daily investment results that correspond to two times the inverse (-2x) of the daily performance of the ICE U.S. Treasury 7-10 Year Bond Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index includes publicly-issued U.S. Treasury securities that have a remaining maturity of greater than seven years and less than or equal to ten years and have $300 million or more of outstanding face value, excluding amounts held by the Federal Reserve. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.655369	2020-10-11 23:38:31.031328
PTEU	Pacer TrendpilotTM European Index	The investment seeks to track the total return performance of the Pacer Trendpilot European Index.\n Normally the fund will invest at least 80% of its total assets in the component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index uses an objective, rules-based methodology to implement a systematic trend-following strategy that directs exposure (i) 100% to the FTSE Eurozone Index, (ii) 50% to the FTSE Eurozone Index and 50% to 3-Month U.S. Treasury bills, or (iii) 100% to 3-Month U.S. Treasury bills. It is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:09.681971	2020-10-11 23:38:31.028166
PTN	Palatin Technologies Inc.	Palatin Technologies Inc is a biopharmaceutical company engaged in developing targeted, receptor-specific peptide therapeutics for the treatment of diseases with unmet medical need and commercial potential.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{NBIX,MNOV,AZN,JNJ,LGND,ANIP,GLYC,DRNA,ARDX,TLGT,XLV}	Medical Diagnostics & Research	Healthcare	NYSE American	\N	2020-10-11 23:12:09.719029	2020-10-11 23:38:31.028166
PULM	Pulmatrix Inc.	Pulmatrix Inc is is a clinical stage biotechnology company. The Company is engaged in discovery and development of a novel class of inhaled therapeutic products intended to prevent and treat respiratory diseases and infections.	{Healthcare,Biotechnology}	{REGN,NKTR,GLYC,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:09.747106	2020-10-11 23:38:31.028166
PVH	PVH Corp.	PVH Corp is apparel company, which designs and markets branded dress shirts, neckwear, sportswear, jeanswear, intimate apparel, swim products, handbags, footwear and other related products. Its brands include Tommy Hilfiger and Calvin Klein.	{"Consumer Cyclical","Apparel Manufacturing","Manufacturing - Apparel & Furniture"}	{GIII,OXM,VFC,HBI,DLA,RL}	Manufacturing - Apparel & Furniture	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:09.763132	2020-10-11 23:38:31.024353
PXE	Invesco Dynamic Energy Exploration &Production	The investment seeks to track the investment results (before fees and expenses) of the Dynamic Energy Exploration & Production IntellidexSM Index.\n The fund invests at least 90% of its total assets in common stocks of companies engaged in energy exploration and production that comprise the underlying intellidex. The intellidex was composed of common stocks of 30 U.S. companies involved in the exploration and production of natural resources used to produce energy. These companies are engaged principally in exploration, extraction and production of crude oil and natural gas from land-based or offshore wells. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.788237	2020-10-11 23:38:31.031328
PXLW	Pixelworks Inc.	Pixelworks Inc manufactures and sells semiconductor products primarily in Japan. The company's technology provides key essentials for the development of display features in electronic devices such as smartphones and tablets.	{Technology,Semiconductors}	{QCOM,NVDA,STM,TXN,XLK}	Semiconductors	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:09.803811	2020-10-11 23:38:31.025871
PY	Principal Shareholder Yield Index ETF	The investment seeks to provide investment results that closely correspond, before expenses, to the performance of the Nasdaq U.S. Shareholder Yield Index.\n Under normal circumstances, the fund invests at least 80% of its net assets, plus any borrowings for investment purposes, in equity securities of companies that compose the index at the time of purchase. The index uses a quantitative model designed to identify equity securities (including value stock) of mid- to large-capitalization companies in the Nasdaq US Large Mid Cap Index (the "parent index") that exhibit high degrees of sustainable, shareholder yield.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.825375	2020-10-11 23:38:31.021426
PSCM	PowerShares S&P SmallCap Materials Portfolio	The investment seeks to track the investment results that (before fees and expenses) of the S&P SmallCap 600® Capped Materials Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in common stocks of small capitalization U.S. basic materials companies that comprise the underlying index. These companies are principally engaged in the business of producing raw materials, including paper or wood products, chemicals, construction materials, and mining and metals. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.604389	2020-10-11 23:38:31.021426
PSI	Invesco Dynamic Semiconductors	The investment seeks to track the investment results (before fees and expenses) of the Dynamic Semiconductor IntellidexSM Index.\n The fund generally will invest at least 90% of its total assets in common stocks of semiconductor companies that comprise the underlying intellidex. The underlying intellidex was composed of common stocks of 30 U.S. semiconductor companies. These are companies that are principally engaged in the manufacture of semiconductors. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.626076	2020-10-11 23:38:31.021426
PSP	Invesco Global Listed Private Equity	The investment seeks to track the investment results (before fees and expenses) of the Red Rocks Global Listed Private Equity Index.\n The fund generally will invest at least 90% of its total assets in securities (including American depositary receipts ("ADRs") and global depositary receipts ("GDRs")) that comprise the underlying index. The underlying index is composed of securities, ADRs and GDRs of 40 to 75 private equity companies, including business development companies ("BDCs"), master limited partnerships ("MLPs") and other vehicles.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.650313	2020-10-11 23:38:31.023428
PTH	Invesco DWA Healthcare Momentum ETF	The investment seeks to track the investment results (before fees and expenses) of the Dorsey Wright® Healthcare Technical Leaders Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities that comprise the underlying index. The underlying index is composed of at least 30 securities of companies in the healthcare sector that have powerful relative strength or "momentum" characteristics.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.693147	2020-10-11 23:38:31.021426
PUI	Invesco DWA Utilities Momentum ETF	The investment seeks to track the investment results (before fees and expenses) of the Dorsey Wright® Utilities Technical Leaders Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities that comprise the underlying index. The underlying index is composed of at least 30 securities of companies in the utilities sector that have powerful relative strength or "momentum" characteristics.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.736924	2020-10-11 23:38:31.023007
PVAL	Principal Contrarian Value Index ETF	The investment seeks to provide investment results that closely correspond, before expenses, to the performance of the Nasdaq U.S. Contrarian Value Index (the "index").\n Under normal circumstances, the fund invests at least 80% of its net assets, plus any borrowings for investment purposes, in equity securities of companies that compose the index at the time of purchase. The index uses a quantitative model designed to identify equity securities in the Nasdaq US Large Mid Cap Index (the "parent index") that appear to be undervalued by the market relative to their fundamental value.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.759589	2020-10-11 23:38:31.025871
PWS	Pacer WealthShield	The investment seeks to track the total return performance, before fees and expenses, of the Pacer WealthShield Index.\n Under normal circumstances, at least 80% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in (i) the component securities of the index or (ii) ETFs that seek to track the performance of some or all of the component securities of the index in the same approximate weight as such component securities. The index utilizes a systematic risk management strategy that directs the index's exposure to U.S. equity securities, U.S. Treasury securities, or a mix of each. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:09.780995	2020-10-11 23:38:31.02423
PXLG	Invesco Russell Top 200 Pure Growth	The investment seeks to track the investment results (before fees and expenses) of the Russell Top 200® Pure Growth Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the component securities that comprise the underlying index. The underlying index is composed of a subset of securities from the Russell Top 200® Index, which is composed of the largest 200 securities of the Russell 3000® Index, an index designed to measure the performance of the largest 3,000 companies in the U.S. equity market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.797075	2020-10-11 23:38:31.031328
PXSV	Invesco Russell 2000 Pure Value	The investment seeks to track the investment results (before fees and expenses) of the Russell 2000® Pure Value Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the component securities that comprise the underlying index. The underlying index is composed of a subset of securities from the Russell 2000® Index, which is composed of the smallest 2,000 securities of the Russell 3000® Index, an index designed to measure the performance of the largest 3,000 companies in the U.S. equity market.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.812405	2020-10-11 23:38:31.031328
PZG	Paramount Gold Nevada Corp.	Paramount Gold Nevada Corp is an exploration stage mining company with projects in Nevada and northern Mexico. The Company acquires and develops known precious metals deposits in large-scale geological environments in North America.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:09.842283	2020-10-11 23:38:31.024353
PSJ	Invesco Dynamic Software	The investment seeks to track the investment results (before fees and expenses) of the Dynamic Software IntellidexSM Index.\n The fund generally will invest at least 90% of its total assets in common stocks of software companies that comprise the underlying intellidex. The underlying intellidex was composed of common stocks of 30 U.S. software companies. These are companies that are principally engaged in the research, design, production or distribution of products or processes that relate to software applications and systems and information-based services. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.624542	2020-10-11 23:38:31.02423
PSR	Invesco Active U.S. Real Estate Fund	The investment seeks high total return through growth of capital and current income.\n The fund invests normally at least 80% of its assets in securities of companies that are principally engaged in the U.S. real estate industry and included within the FTSE NAREIT All Equity REITs Index. It also may invest in real estate operating companies ("REOCs"), as well as securities of other companies principally engaged in the U.S. real estate industry. REOCs are similar to REITs, except that REOCs reinvest their earnings into the business, rather than distributing them to unitholders like REITs. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.65475	2020-10-11 23:38:31.028166
PTCT	PTC Therapeutics Inc.	PTC Therapeutics Inc is a biopharmaceutical company. It is focused on the discovery and development of orally administered, proprietary small-molecule drugs that target post-transcriptional control processes.	{Healthcare,Biotechnology}	{VRTX,NVS,SRPT,FATE,XNCR,RTRX,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:09.678612	2020-10-11 23:38:31.028166
PUB	People's Utah Bancorp	People's Utah Bancorp is a bank holding company. Its subsidiaries offer a full range of short-term to long-term commercial, personal and mortgage loans. It also offers mortgage loans secured by personal residences.	{"Financial Services","Banks - Regional - US",Banks}	{MPB,MOFG}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:09.737196	2020-10-11 23:38:31.028166
PVG	Pretium Resources Inc. (Canada)	Pretium Resources Inc is in the business of acquiring, owning, evaluating and developing gold, silver, copper mineral interests and owns the Brucejack and Snowfield Projects located in Northwest British Columbia, Canada.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{EGO,NGD,BTG}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:09.763786	2020-10-11 23:38:31.021426
PWZ	Invesco California AMT-Free Municipal Bond Portfolio	The investment seeks to track the investment results (before fees and expenses) of the ICE BofAML California Long-Term Core Plus Municipal Securities Index.\n The fund generally will invest at least 80% of its total assets in municipal securities that comprise the underlying index and that also are exempt from the federal AMT. The underlying index is composed of U.S. dollar-denominated, investment grade, tax-exempt debt publicly issued by California or any U.S. territory or their political subdivisions, in the U.S. domestic market with a term of at least 15 years remaining to final maturity. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.78656	2020-10-11 23:38:31.021426
PXMV	Invesco Russell Midcap Pure Value	The investment seeks to track the investment results (before fees and expenses) of the Russell Midcap® Pure Value Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the component securities that comprise the underlying index. The underlying index is composed of a subset of securities from the Russell Midcap® Index, which is composed of the smallest 800 securities of the Russell 1000® Index, an index designed to measure the performance of the largest 1,000 companies in the U.S. equity market.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.805637	2020-10-11 23:38:31.02582
PYPL	PayPal Holdings Inc.	PayPal Holdings Inc is a technology platform company that enables digital and mobile payments on behalf of consumers and merchants. The company's payments platform includes PayPal, PayPal Credit, Venmo, Xoom and Braintree products.	{"Financial Services","Credit Services"}	{}	Credit Services	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:09.828726	2020-10-11 23:38:31.024353
QABA	First Trust NASDAQ ABA Community Bank Index Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the NASDAQ OMX® ABA Community Bank IndexSM.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is a market-capitalization-weighted index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.852448	2020-10-11 23:38:31.02582
QCRH	QCR Holdings Inc.	QCR Holdings Inc is engaged in the financial service sector. As a banking firm, subsidiaries it provides commercial and consumer banking, & trust & asset management services.	{"Financial Services","Banks - Regional - US",Banks}	{UMBF,ATLO,WTBA,GFED,CBSH,GSBC,EFSC,HWBK,MOFG}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:09.880004	2020-10-11 23:38:31.021426
QDF	FlexShares Quality Dividend Index Fund	The investment seeks investment results that correspond generally to the price and yield performance, before fees and expenses, of the Northern Trust Quality Dividend IndexSM.\n Under normal circumstances, the fund will invest at least 80% of its total assets (exclusive of collateral held from securities lending) in the securities of the underlying index. The underlying index is designed to provide exposure to a high-quality, income-oriented portfolio of U.S. equity securities, with an emphasis on long-term capital growth and a targeted overall volatility similar to that of the Northern Trust 1250 IndexSM.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.887704	2020-10-11 23:38:31.031328
PSQ	ProShares Short QQQ	The investment seeks daily investment results before fees and expenses that correspond to the inverse (-1x) of the daily performance of the NASDAQ-100 Index®.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index includes 100 of the largest domestic and international non-financial companies listed on the Nasdaq Stock Market based on market capitalization. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.655906	2020-10-11 23:38:31.02423
PTF	Invesco DWA Technology Momentum ETF	The investment seeks to track the investment results (before fees and expenses) of the Dorsey Wright® Technology Technical Leaders Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities that comprise the underlying index. The underlying index is composed of at least 30 securities of companies in the technology sector that have powerful relative strength or "momentum" characteristics.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.690307	2020-10-11 23:38:31.031328
PTR	PetroChina Company Limited	PetroChina Co Ltd is engaged in the exploration, development, production and sale of crude oil and natural gas, refining of crude oil and petroleum products, transmission of natural gas, crude oil and refined products and sale of natural gas.	{Energy,"Oil & Gas Integrated","Oil & Gas - Integrated"}	{XOM,MUR,BP,SNP,CEO,CVX,COP,MPC}	Oil & Gas - Integrated	Energy	New York Stock Exchange	\N	2020-10-11 23:12:09.723209	2020-10-11 23:38:31.021426
PUTW	WisdomTree CBOE S&P 500 PutWrite Strategy Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the CBOE S&P 500 PutWrite Index (the "index").\n Under normal circumstances, at least 80% of the fund's total assets will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index tracks the value of a cash-secured put option sales strategy, which consists of selling (or "writing") S&P 500 Index put options ("SPX Puts") and investing the sale proceeds in one- and three-month Treasury bills. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.754556	2020-10-11 23:38:31.021426
PWB	Invesco Dynamic Large Cap Growth	The investment seeks to track the investment results (before fees and expenses) of the Dynamic Large Cap Growth IntellidexSM Index.\n The fund generally will invest at least 90% of its total assets in common stocks of large capitalization companies that comprise the underlying intellidex. The intellidex provider. considers a company to be a large capitalization company if it falls within the underlying intellidex model. The underlying intellidex for the fund is composed of 50 large capitalization U.S. growth stocks that the intellidex provider includes principally on the basis of their capital appreciation potential.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.770512	2020-10-11 23:38:31.031328
PXI	Invesco DWA Energy Momentum ETF	The investment seeks to track the investment results (before fees and expenses) of the Dorsey Wright® Energy Technical Leaders Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities that comprise the underlying index. The underlying index is composed of at least 30 securities of companies in the energy sector that have powerful relative strength or "momentum" characteristics.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.794747	2020-10-11 23:38:31.023007
PXR	Invesco Emerging Markets Infrastructure	The investment seeks to track the investment results (before fees and expenses) of the S-Network Emerging Infrastructure Builders IndexSM.\n The fund will invest at least 90% of its total assets in the equity securities of companies involved in the following sectors related to infrastructure construction and development in emerging market countries: 1) construction and engineering; 2) construction machinery; 3) construction materials; 4) diversified metals and mining; 5) heavy electrical equipment; 6) industrial machinery; and 7) steel, and that comprise the underlying index, as well as ADRs and GDRs that are based on the securities in the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.810746	2020-10-11 23:38:31.023428
PZD	Invesco Cleantech	The investment seeks to track the investment results (before fees and expenses) of The Cleantech IndexTM.\n The fund generally will invest at least 90% of its total assets in securities that comprise the underlying index, which is designed to track the performance of publicly traded clean technology (or "cleantech") companies. Cleantech considers a company to be a cleantech company when it derives at least 50% of its revenues or operating profits from cleantech businesses.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.841429	2020-10-11 23:38:31.028166
QCLN	First Trust NASDAQ Clean Edge Green Energy Idx Fd	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the NASDAQ® Clean Edge® Green Energy IndexSM.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is a modified market capitalization-weighted index designed to track the performance of companies whose securities are listed on NASDAQ, NYSE MKT or NYSE that are primarily manufacturers, developers, distributors or installers of clean-energy technologies. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.870155	2020-10-11 23:38:31.02423
QTNT	Quotient Limited	Quotient Ltd is a commercial-stage diagnostics company engaged in reducing healthcare costs and improving patient care with focus in blood grouping and donor disease screening.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{BAX,BTX,XLV}	Medical Diagnostics & Research	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:09.959792	2020-10-11 23:38:31.02582
PXSG	Invesco Russell 2000 Pure Growth	The investment seeks to track the investment results (before fees and expenses) of the Russell 2000® Pure Growth Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the component securities that comprise the underlying index. The underlying index is composed of a subset of securities from the Russell 2000® Index, which is composed of the smallest 2,000 securities of the Russell 3000® Index, an index designed to measure the performance of the largest 3,000 companies in the U.S. equity market.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.812405	2020-10-11 23:38:31.028166
PYZ	Invesco DWA Basic Materials Momentum ETF	The investment seeks to track the investment results (before fees and expenses) of the Dorsey Wright® Basic Materials Technical Leaders Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities that comprise the underlying index. The underlying index is composed of at least 30 securities of companies in the basic materials sector that have powerful relative strength or "momentum" characteristics.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.830732	2020-10-11 23:38:31.023007
PZN	Pzena Investment Management Inc Class A	Pzena Investment Management Inc is a global investment management firm that employs a value investment approach. It provides investment management and advisory services to funds in both U.S. and non-U.S. capital markets.	{"Financial Services","Asset Management"}	{DHIL,AMG,LM,WHG,CNS,AB,MN,SAMG}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.847214	2020-10-11 23:38:31.021426
QBAK	Qualstar Corporation	Qualstar Corp is a U.S based provider of data storage systems marketed under the Qualstar brand and of power solutions marketed under the N2Power brand.	{Technology,"Electronic Components","Computer Hardware"}	{VICR,IBM,CUI,QTM,USAU,XLK}	Computer Hardware	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:09.87144	2020-10-11 23:38:31.028166
QDYN	FlexShares Quality Dynamic Index Fund	The investment seeks investment results that correspond generally to the price and yield performance, before fees and expenses, of the Northern Trust Quality Dividend Dynamic IndexSM.\n The fund will invest at least 80% of its total assets (exclusive of collateral held from securities lending) in the securities of the index. The underlying index is designed to provide exposure to a high-quality, income-oriented portfolio of U.S. equity securities, with an emphasis on long-term capital growth and a targeted overall volatility that is greater than that of the Northern Trust 1250 IndexSM. It may also invest up to 20% of its assets in cash and cash equivalents.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.893352	2020-10-11 23:38:31.028166
QGTA	IQ Leaders GTAA Tracker	The investment seeks investment results that correspond generally to the price and yield performance of its underlying index, the IQ Leaders GTAA Index.\n The fund is a "fund of funds" which means it invests, under normal circumstances, at least 80% of its net assets, plus the amount of any borrowings for investment purposes, in the investments included in its underlying index, which includes underlying funds. The underlying index seeks to track the performance of the 10 leading global allocation mutual funds based on fund performance and fund asset size, re-set annually in accordance with IndexIQ's rules-based methodology ("Selected Global Allocation Funds").	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.903423	2020-10-11 23:38:31.031328
QLS	IQ Hedge Long Short Tracker	The investment seeks investment results that correspond (before fees and expenses) generally to the price and yield performance of its underlying index, the IQ Hedge Long/Short Index (the "underlying index").\n The fund is a "fund of funds" which means it invests, under normal circumstances, at least 80% of its net assets, plus the amount of any borrowings for investment purposes, in the investments included in its underlying index, which includes underlying funds. The underlying index consists of a number of components ("underlying index components") selected in accordance with IndexIQ's rules-based methodology of such underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.924381	2020-10-11 23:38:31.028166
QLC	FlexShares US Quality Large Cap Index Fund	The investment seeks investment results that correspond generally to the price and yield performance, before fees and expenses, of the Northern Trust Quality Large Cap IndexSM.\n The fund generally will invest under normal circumstances at least 80% of its total assets (exclusive of collateral held from securities lending) in the securities of its underlying index. The underlying index is designed to measure the performance of companies that exhibit certain quality, valuation and momentum characteristics within a universe of publicly-traded U.S. large capitalization equity securities.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.911308	2020-10-11 23:38:31.025871
PZT	Invesco New York AMT-Free Municipal Bond	The investment seeks to track the investment results (before fees and expenses) of the ICE BofAML New York Long-Term Core Plus Municipal Securities Index (the "underlying index").\n The fund generally will invest at least 80% of its total assets in municipal securities that comprise the underlying index and that also are exempt from the federal alternative minimum tax. The index is composed of U.S. dollar-denominated, investment grade, tax-exempt debt publicly issued by New York or any U.S. territory or their political subdivisions, in the U.S. domestic market with a term of at least 15 years remaining to final maturity. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.850253	2020-10-11 23:38:31.023428
QCOM	QUALCOMM Incorporated	Qualcomm Inc develops and licenses wireless technology. It also engages in designing chips for mobile phones. The company is also the world's largest wireless chip vendor, supplying many premier handset makers with leading-edge processors.	{Technology,Semiconductors}	{CSCO,MXIM,NVDA,ERIC,NOK,SNE,MSI,TXN,GOOGL,BB,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:09.874769	2020-10-11 23:38:31.025871
QHC	Quorum Health Corporation	Quorum Health Corp along with its subsidiaries owns and operates hospitals. Its hospital provides services including general acute care, emergency room, general & specialty surgery, medicine, obstetrics, diagnostic, psychiatric & rehabilitation services.	{Healthcare,"Medical Care","Health Care Providers"}	{PINC,QHC,XLV}	Health Care Providers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:09.911144	2020-10-11 23:38:31.021426
QID	ProShares UltraShort QQQ	The investment seeks daily investment results, before fees and expenses, that correspond to two times the inverse (-2x) of the daily performance of the NASDAQ-100 Index®.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index includes 100 of the largest domestic and international non-financial companies listed on the Nasdaq Stock Market based on market capitalization. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.907884	2020-10-11 23:38:31.02423
QLD	ProShares Ultra QQQ	The investment seeks daily investment results, before fees and expenses, that correspond to two times (2x) the daily performance of the NASDAQ-100 Index®.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index includes 100 of the largest domestic and international non-financial companies listed on the Nasdaq Stock Market based on market capitalization. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.92103	2020-10-11 23:38:31.02582
QLTA	iShares Aaa A Rated Corporate Bond	The investment seeks to track the investment results of the Bloomberg Barclays U.S. Corporate Aaa - A Capped Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The underlying index is a subset of the Bloomberg Barclays U.S. Corporate Index, which measures the performance of the Aaa - A rated range of the fixed-rate, U.S. dollar-denominated taxable, corporate bond market.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.924482	2020-10-11 23:38:31.023007
QQXT	First Trust NASDAQ-100 Ex-Tech Sector Index Fd	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the NASDAQ-100 Ex-Tech Sector IndexSM.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is an equal-weighted index based on the securities of the NASDAQ-100 Index® that are not classified as "technology" according to the Industry Classification Benchmark ("ICB") classification system and, as a result, is a subset of the NASDAQ-100 Index®.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.944515	2020-10-11 23:38:31.021426
QSR	Restaurant Brands International Inc.	Restaurant Brands International Inc is a quick service restaurant company. It owns and operates restaurants brands include Burger King, Tim Hortons, and Popeyes Louisiana Kitchen.	{"Consumer Cyclical",Restaurants}	{JACK,SONC,MCD,WEN}	Restaurants	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:09.945588	2020-10-11 23:38:31.031328
QD	Qudian Inc. American Depositary Shares each representing one Class A	Qudian Inc is engaged in the operation of online platforms to provide small cash credit products in China. It uses big data-enabled technologies, such as artificial intelligence and machine learning, to transform the consumer finance experience in China.	{"Financial Services","Credit Services"}	{NICK,NAVI,OMF,PPDF}	Credit Services	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.880257	2020-10-11 23:38:31.023007
QIWI	QIWI plc	QIWI PLC along with its subsidiaries is engaged in operating electronic online payment systems in Russia and sells electronic payment kiosks and maintains banking activity supporting processing of payments.	{"Financial Services","Credit Services"}	{EEFT,EBAY,MBT,WU,V,VEON}	Credit Services	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:09.907674	2020-10-11 23:38:31.023428
QLYS	Qualys Inc.	Qualys Inc provides cloud security and compliance solutions to organizations identifying security risks to their IT infrastructures, to protect their IT systems from cyber-attacks and achieve compliance with internal policies and external regulations.	{Technology,"Software - Application","Application Software"}	{INTC,MSFT,SYMC,IBM,PFPT,FEYE,HPQ,IMPV,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:09.924592	2020-10-11 23:38:31.021426
QMN	IQ Hedge Market Neutral Tracker	The investment seeks investment results that correspond (before fees and expenses) generally to the price and yield performance of its underlying index, the IQ Hedge Market Neutral Index.\n The fund is a "fund of funds" which means it invests, under normal circumstances, at least 80% of its net assets, plus the amount of any borrowings for investment purposes, in the investments included in its underlying index, which includes underlying funds. The underlying index consists of a number of components ("underlying index components") selected in accordance with IndexIQ's rules-based methodology of such underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.925927	2020-10-11 23:38:31.024353
QQQX	Nuveen NASDAQ 100 Dynamic Overwrite Fund	The Fund intends to pursue its investment objective principally through a two-part strategy, namely The NASDAQ Investment Portfolio and The Option Strategies. Firstly, the Fund will invest, under normal circumstances, substantially all of its net assets in a portfolio of investments (NASDAQ Investment Portfolio) designed to closely track the performance, before fees and expenses, of the NASDAQ 100ª Index. The NASDAQ Investment Portfolio may include stocks that are members of the Index, stocks which are not members of the Index, and other investments that have economic characteristics similar to the securities that constitute the Index. These other investments may include futures, forwards, swaps, option contracts based on the Index and options on futures and swaps, as well as investment funds or baskets of securities with a composition similar to all or a portion of the Index. Secondly, in attempting to generate premium income and reduce the volatility of the Funds returns, with the intent of improving the Funds risk-adjusted returns, the Fund will write (sell) call options on the Index (Written Options) that are fully collateralized by the NASDAQ Investment Portfolio. Under normal circumstances, the notional value of the Written Options is not expected to exceed 50% of the Funds net assets.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:09.944246	2020-10-11 23:38:31.02582
QRVO	Qorvo Inc.	Qorvo Inc is a semiconductor product manufacturer. It sells its products to companies engaged in the telecommunication, infrastructure and defense industries.	{Technology,Semiconductors}	{MXIM,SWKS,MTSI,ADI,PRKR,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:09.946274	2020-10-11 23:38:31.024353
QSY	WisdomTree U.S. Quality Shareholder Yield Fund	The investment seeks income and capital appreciation.\n The fund, an exchange traded fund, is actively managed using a model-based approach. It seeks to achieve its investment objective by investing primarily in U.S. equity securities that provide a high "total shareholder yield" and exhibit favorable quality characteristics that demonstrate a company's profitability, such as strong ROE and/or ROA. The fund invests primarily in equity securities of companies domiciled in the U.S. or listed on a U.S. exchange. The advisor generally expects to invest in large- and mid-capitalization companies and may invest in any sector. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.945532	2020-10-11 23:38:31.025871
QTRX	Quanterix Corporation	Quanterix Corp is a life science company. The company digitizing biomarker analysis with the goal of advancing the science of precision health.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:09.964276	2020-10-11 23:38:31.023007
OLEM	iPath Pure Beta Crude Oil ETN	The investment seeks to provide investors with exposure to the Barclays WTI Crude Oil Pure Beta Total Return Index. \nThe Barclays WTI Crude Oil Pure Beta Total Return Index (the "index") reflects the returns that are potentially available through an unleveraged investment in the futures contracts in the Crude Oil markets. The index may roll into one of a number of futures contracts with varying expiration dates, as selected using the Barclays Pure Beta Series 2 Methodology.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.398739	2020-10-11 23:38:31.031328
OMER	Omeros Corporation	Omeros Corp is a biopharmaceutical company. It focuses on research and development of small-molecule and protein therapeutics to treat inflammation, coagulopathies, and disorders of the central nervous system and sells Omidria for cataract surgeries.	{Healthcare,Biotechnology}	{AKRX,DRRX,BTX,JNJ,PFE,GENE,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:08.420229	2020-10-11 23:38:31.031328
OMP	Oasis Midstream Partners LP Representing Limited Partner Interests	Oasis Midstream Partners LP owns, develops, operates and acquires a diversified portfolio of midstream assets in North America that are integral to the oil and natural gas operations of Oasis Petroleum.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{OKE,SMLP,OMP}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:08.450343	2020-10-11 23:38:31.031328
OPGN	OpGen Inc.	OpGen Inc is an precision medicine company using molecular diagnostics and bioinformatics to help combat infectious disease.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{XLV}	Medical Diagnostics & Research	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:08.48531	2020-10-11 23:38:31.02423
OPY	Oppenheimer Holdings Inc. Class A (DE)	Oppenheimer Holdings Inc. is a middle-market investment bank and service broker-dealer. It is involved in various activities such as retail securities brokerage, institutional sales and trading, investment banking, trust services and many others.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{SF,LTS,PJC,COWN}	Brokers & Exchanges	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.506884	2020-10-11 23:38:31.028166
ORC	Orchid Island Capital Inc.	Orchid Island Capital Inc invests in residential mortgage-backed securities, or RMBS across the United States. The company is externally managed and advised by Bimini Advisors, LLC, a wholly-owned subsidiary of Bimini Capital Management, Inc.	{"Real Estate","REIT - Diversified",REITs}	{NYMT,IVR,WMC,MITT,EARN,TWO}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:08.529713	2020-10-11 23:38:31.025871
ORMP	Oramed Pharmaceuticals Inc.	Oramed Pharmaceuticals Inc is a pharmaceutical company. The Company is engaged in the research and development of pharmaceutical solutions, including an orally ingestible insulin capsule or tablet.	{Healthcare,Biotechnology}	{CGEN,PSTI,NVO,SNY,PLX,RARE,LXRX,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:08.548796	2020-10-11 23:38:31.024353
OSIZ	Oppenheimer Russell 1000 Size Factor	The investment seeks to provide investment results that correspond generally, before fees and expenses, to the performance of the Russell 1000 Size Factor Index (the "underlying index").\n Under normal circumstances, the fund will invest at least 80% of its net assets, plus any borrowings for investment purposes, in securities of companies included in the underlying index. The underlying index is constructed using a rules-based methodology that selects a subset of the equity securities from the Russell 1000 Index (the "parent index"), which is comprised of the 1,000 largest-capitalization companies in the United States.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:08.573916	2020-10-11 23:38:31.025871
OTTW	Ottawa Bancorp Inc.	Ottawa Bancorp Inc is a bank holding company. The Company through its holding provides financial services to individual and corporate customers in the United States.	{"Financial Services","Banks - Regional - US",Banks}	{SMBC,HFBC,BYFC,CVLY,FCAP,BCBP,IROQ,MSBF}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:08.60889	2020-10-11 23:38:31.028166
OVOL	Oppenheimer Russell 1000 Low Volatility Factor	The investment seeks to provide investment results that correspond generally, before fees and expenses, to the performance of the Russell 1000 Volatility Factor Index (the "underlying index").\n Under normal circumstances, the fund will invest at least 80% of its net assets, plus any borrowings for investment purposes, in securities of companies included in the underlying index. The underlying index is constructed using a rules-based methodology that selects a subset of the equity securities from the Russell 1000 Index (the "parent index"), which is comprised of the 1,000 largest-capitalization companies in the United States.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:08.637226	2020-10-11 23:38:31.025871
OYLD	Oppenheimer Russell 1000 Yield Factor	The investment seeks to provide investment results that correspond generally, before fees and expenses, to the performance of the Russell 1000 Yield Factor Index (the "underlying index").\n Under normal circumstances, the fund will invest at least 80% of its net assets, plus any borrowings for investment purposes, in securities of companies included in the underlying index. The underlying index is constructed using a rules-based methodology that selects a subset of the equity securities from the Russell 1000 Index (the "parent index").	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:08.655316	2020-10-11 23:38:31.025871
PAG	Penske Automotive Group Inc.	Penske Automotive Group Inc is a transportation services company. It operates automotive and commercial vehicle dealership in the United States and Western Europe & distributes commercial vehicles, diesel engines and related parts.	{"Consumer Cyclical","Auto & Truck Dealerships",Autos}	{CPRT,CRMT,RUSHA,LAD,AN,GPI,SAH,ABG,KMX,CAR}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:08.686205	2020-10-11 23:38:31.028166
QVAL	Alpha Architect U.S. Quantitative Value	The investment seeks to track the total return performance, before fees and expenses, of the Alpha Architect Value Momentum Trend Index.\n Under normal circumstances, at least 80% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in the component securities of the index and other instruments of the index. The index will be composed primarily of the other ETFs advised by the Adviser. Currently, there are four Alpha Architect ETFs, which invest in either domestic or international equity securities, and employ either a "momentum" or a "value" investment strategy. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:09.984719	2020-10-11 23:38:31.024353
RA	Brookfield Real Assets Income Fund Inc.	Brookfield Real Assets Income Fund Inc. is a diversified, closed-end management investment company. Its investment objective is to seek high total return, primarily through high current income and secondarily, through growth of capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:10.022783	2020-10-11 23:38:31.02582
RAND	Rand Capital Corporation	Rand Capital Corp is an internally managed investment company that lends to and invests in small and medium-sized companies in connection with other investors. It invests in financial instruments such as equity instruments, debt instruments, among others.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:10.041212	2020-10-11 23:38:31.024353
RBC	Regal Beloit Corporation	Regal Beloit Corp is a manufacturer of electric motors , electrical motion controls,power generation and power transmission products.	{Industrials,"Diversified Industrials","Industrial Products"}	{AMOT,AIMC,CMI,EMR,GE,AOS,TKR,UQM,SVT}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:10.064968	2020-10-11 23:38:31.023007
RCD	Invesco S&P 500 Equal Weight Consumer Discretionary	The investment seeks to track the investment results (before fees and expenses) of the S&P 500® Equal Weight Consumer Discretionary Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities that comprise the underlying index. The underlying index is an equal-weighted version of the S&P 500® Consumer Discretionary Index. Strictly in accordance with its guidelines and mandated procedures, the index provider compiles, maintains and calculates the underlying index, which is comprised of common stocks of companies in the consumer discretionary sector of the S&P 500® Index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.08169	2020-10-11 23:38:31.023007
RCMT	RCM Technologies Inc.	RCM Technologies Inc is a provider of business and technology solutions designed to enhance and maximize the operational performance of its customers through the adaptation and deployment of advanced engineering and information technology services.	{Industrials,"Engineering & Construction"}	{MSFT,ADP,CCRN,IBM,AMN}	Engineering & Construction	Industrials	NASDAQ Global Market	\N	2020-10-11 23:12:10.103393	2020-10-11 23:38:31.021426
RDIB	Reading International Inc	Reading International Inc is a diversified company, engaged in the development, ownership and operation of multiplex cinemas and retail and commercial real estate in the United States, Australia and New Zealand.	{"Consumer Cyclical","Media - Diversified",Entertainment}	{MCS}	Entertainment	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:10.130607	2020-10-11 23:38:31.028166
RE	Everest Re Group Ltd.	Everest Re Group Ltd is engaged in the insurance business. Its offering comprises of property and casualty reinsurance & insurance products and services.	{"Financial Services","Insurance - Reinsurance","Insurance - Specialty"}	{ACGL,MHLD,RNR,AHL}	Insurance - Specialty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:10.152081	2020-10-11 23:38:31.02423
REGI	Renewable Energy Group Inc.	Renewable Energy Group Inc (REG) is a producer of biofuels. It is engaged in providing cleaner, lower carbon intensity products, and services. REG generates most of its revenue from the United States of America.	{Energy,"Oil & Gas Refining & Marketing","Oil & Gas - Refining & Marketing"}	{REX,FF,PEIX,GRBK}	Oil & Gas - Refining & Marketing	Energy	Nasdaq Global Select	\N	2020-10-11 23:12:10.175296	2020-10-11 23:38:31.02423
REN	Resolute Energy Corporation Comon Stock	Resolute Energy Corp is an independent oil and gas company engaged in the exploitation, development, exploration for and acquisition of oil and gas properties located in Utah and Texas.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{APC,NOG,USEG,HK,BCEI,ESTE,WLL}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:10.208427	2020-10-11 23:38:31.02423
REV	Revlon Inc.	Revlon Inc is a cosmetics company. It manufactures and sells cosmetics, hair color, beauty tools, fragrances, skincare, deodorants and personal care products.	{"Consumer Defensive","Household & Personal Products","Consumer Packaged Goods"}	{IPAR,EL,NUS,AVP,CAW,PG,COTY,EPC}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:10.23251	2020-10-11 23:38:31.024353
RFCI	RiverFront Dynamic Core Income	The investment seeks total return, with an emphasis on income as the source of that total return.\n The fund invests in a global portfolio of fixed income securities of various maturities, ratings and currency denominations. It may purchase fixed income securities issued by U.S. or foreign corporations or financial institutions, including debt securities of all types and maturities, convertible securities and preferred stocks. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.255226	2020-10-11 23:38:31.025871
RFTA	RAIT Financial Trust 7.125% Senior Notes due 2019		{}	{}			OTCQB - U.S. Registered	\N	2020-10-11 23:12:10.277361	2020-10-11 23:38:31.021426
R	Ryder System Inc.	Ryder System Inc is engaged in supply chain activities. Its services include transportation, supply chain and fleet management solutions.	{Industrials,"Rental & Leasing Services","Consulting & Outsourcing"}	{CVTI,UHAL,HUBG,JBHT,GATX,KNX}	Consulting & Outsourcing	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:10.016406	2020-10-11 23:38:31.02423
RAVE	Rave Restaurant Group Inc.	Rave Restaurant Group Inc operates and franchises pizza buffet, delivery/carry-out & express restaurants domestically and internationally under the trademark Pizza Inn and operates domestic fast casual pizza restaurants.	{"Consumer Cyclical",Restaurants}	{NATH,PZZA,YUM,DPZ,PBPB,FRSH}	Restaurants	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:10.04529	2020-10-11 23:38:31.031328
RBNC	Reliant Bancorp Inc.	Reliant Bancorp Inc provides a full range of traditional banking services throughout the Middle Tennessee Region and the Nashville-Davidson-Murfreesboro-Franklin Metropolitan Statistical Area.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:10.06716	2020-10-11 23:38:31.021426
RCKT	Rocket Pharmaceuticals Inc.	Rocket Pharmaceuticals Inc, formerly Inotek Pharmaceuticals Corp, is a clinical-stage biopharmaceutical company focused on the discovery, development and commercialization of therapies for glaucoma and other diseases of the eye.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:10.088571	2020-10-11 23:38:31.02582
RDIV	Oppenheimer S&P Ultra Dividend Revenue	The investment seeks to provide investment results that correspond generally, before fees and expenses, to the performance of the S&P 900 Dividend Revenue-Weighted Index (the "underlying index").\n The fund will normally invest at least 80% of its net assets, plus any borrowings for investment purposes, in the securities of companies included in the underlying index. The underlying index is constructed using a rules-based methodology that starts with the S&P 900® Index (the "parent index"), subject to a maximum 5% per company weighting. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.130921	2020-10-11 23:38:31.031328
RECN	Resources Connection Inc.	Resources Connection Inc is a multinational consulting firm. It provides consulting and business initiative support services to its client in the areas of accounting, finance, corporate governance, risk and compliance management.	{Industrials,"Business Services"}	{ADP,HURN,CRAI,ICFI,PRGX,NCI,FCN,MAN,CBZ,HSON}	Business Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:10.157622	2020-10-11 23:38:31.028166
REK	ProShares Short Real Estate	The investment seeks daily investment results, before fees and expenses, that correspond to the inverse (-1x) of the daily performance of the Dow Jones U.S. Real EstateSM Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index measures the performance of certain companies in the real estate sector of the U.S. equity market. Component companies include, among others, real estate holding and development and real estate services companies and real estate investment trusts ("REITs"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.190258	2020-10-11 23:38:31.028166
RES	RPC Inc.	RPC Inc provides oilfield services and equipment to oil and gas companies engaged in the exploration, production and development of oil and gas properties throughout the United States.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{PTEN,SPN,NOV,SLB,HAL,TTI,PES,BHGE}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:10.222621	2020-10-11 23:38:31.031328
REX	REX American Resources Corporation	REX American Resources Corp is a holding company that conducts substantially all of its business operations through its subsidiaries. It is currently invested in ethanol production entities.	{"Basic Materials",Chemicals}	{GPRE,ADM,REGI,PEIX,GRBK}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:10.243141	2020-10-11 23:38:31.031328
RFUN	RiverFront Dynamic Unconstrained Income	The investment seeks total return, with an emphasis on income as the source of that total return.\n The fund invests in a global portfolio of fixed income securities of various maturities, ratings and currency denominations. It may purchase fixed income securities issued by U.S. or foreign corporations or financial institutions, including debt securities of all types and maturities, convertible securities and preferred stocks. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.283507	2020-10-11 23:38:31.025871
RGT	Royce Global Value Trust Inc.	Royce Global Value Trust Inc. is a close ended equity mutual fund launched and managed by Royce & Associates, LLC. The company invests in value oriented stocks of small cap and microcap companies.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:10.305328	2020-10-11 23:38:31.02423
RIBTW	RiceBran Technologies Warrant	RiceBran Technologies is an ingredient company serving food, animal nutrition and specialty markets focused on value-added processing and marketing of healthy, natural and nutrient dense products derived from raw rice bran.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{}	Consumer Packaged Goods	Consumer Defensive	NASDAQ Capital Market	\N	2020-10-11 23:12:10.334871	2020-10-11 23:38:31.028166
RACE	Ferrari N.V.	Ferrari NV is a designer and manufacturer of luxury performance sports cars under the brand name of Ferrari. Its sports cars portfolio includes F12berlinetta, FF, Ferrari 488 GTB, 488 Spider, 458 Speciale, Ferrari California, among others.	{"Consumer Cyclical","Auto Manufacturers",Autos}	{RACE}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:10.022303	2020-10-11 23:38:31.028166
RARX	Ra Pharmaceuticals Inc.	Ra Pharmaceuticals Inc is a clinical-stage biopharmaceutical company. It discovers and develops therapeutics for the treatment of diseases caused by excessive or uncontrolled activation of the complement system.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:10.044463	2020-10-11 23:38:31.025871
RBCN	Rubicon Technology Inc.	Rubicon Technology Inc is an electronic materials provider that develops, manufactures and sells monocrystalline sapphire and other crystalline products for LEDs, RFICs, blue laser diodes, optoelectronics and other optical applications.	{Technology,"Semiconductor Equipment & Materials",Semiconductors}	{OLED,CCMP,IPGP,FSLR,PKE,ROG,NOVT,XLK}	Semiconductors	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:10.067339	2020-10-11 23:38:31.031328
RCII	Rent-A-Center Inc.	Rent-A-Center Inc is prevalent in the rental business in North America. It offers durable products, such as consumer electronics, appliances, computers and smartphones.	{Industrials,"Rental & Leasing Services","Consulting & Outsourcing"}	{CONN,BBY,LZB,GME,AAN}	Consulting & Outsourcing	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:10.086952	2020-10-11 23:38:31.025871
RDHL	Redhill Biopharma Ltd.	Redhill Biopharma Ltd is a specialty biopharmaceutical company primarily focused on the development and commercialization of late clinical-stage, proprietary drugs for the treatment of gastrointestinal diseases.	{Healthcare,Biotechnology}	{GSK,SNY,MRK,TARO,TEVA,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:10.125801	2020-10-11 23:38:31.02582
REEM	Oppenheimer Revenue Weighted ETF Trust	The investment seeks to provide investment results that correspond generally, before fees and expenses, to the performance of the OFI Revenue Weighted Emerging Markets Index (the "underlying index").\n The fund will normally invest at least 80% of its net assets, plus any borrowings for investment purposes, in securities of companies included in the underlying index. The underlying index is constructed using a rules-based methodology that re-weights the constituent securities of the MSCI Emerging Markets Index (the "parent index") according to the revenue earned by the companies in the parent index, subject to a maximum 5% per company weighting.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:10.159787	2020-10-11 23:38:31.031328
REGL	ProShares S&P MidCap 400 Dividend Aristocrats	The investment seeks investment results, before fees and expenses, that track the performance of the S&P MidCap 400® Dividend Aristocrats Index (the "index").\n Under normal circumstances, the fund will invest at least 80% of its total assets in component securities (i.e., securities of the index and comparable securities that have economic characteristics that are substantially identical to the economic characteristics of the securities of the index). The index contains a minimum of 40 stocks which are equally weighted. No single sector is allowed to comprise more than 30% of the index weight.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:10.175503	2020-10-11 23:38:31.024353
REMX	VanEck Vectors Rare Earth Strategic Metals	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the MVIS® Global Rare Earth/Strategic Metals Index.\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The index includes companies primarily engaged in a variety of activities that are related to the producing, refining and recycling of rare earth and strategic metals and minerals. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.196407	2020-10-11 23:38:31.028166
RESN	Resonant Inc.	Resonant Inc is a late stage development company. It is developing software, intellectual property and services platform for mobile device industry.	{Technology,Semiconductors}	{SWKS,NVEC,CEVA,ADI,QRVO,MOSY,XLK}	Semiconductors	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:10.224429	2020-10-11 23:38:31.023428
REXR	Rexford Industrial Realty Inc.	Rexford Industrial Realty Inc is a United States based self-administered and self-managed full-service real estate investment trust focused on owning, operating and acquiring industrial properties in Southern California infill markets.	{"Real Estate","REIT - Industrial",REITs}	{FR,EGP,MNR,DLR,COR,STAG,PLD,QTS}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:10.247596	2020-10-11 23:38:31.028166
RFG	Invesco S&P Midcap 400 Pure Growth	The investment seeks to track the investment results (before fees and expenses) of the S&P MidCap 400® Pure Growth Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities that comprise the underlying index. The underlying index is comprised only those S&P MidCap 400® companies with strong growth characteristics as measured using three factors: three-year sales per share growth, three-year ratio of earnings per share change to price per share, and momentum (12-month percentage price change). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.270459	2020-10-11 23:38:31.028166
QXMI	QuantX Risk Managed Multi-Asset Income	The investment seeks to replicate investment results that correspond to the performance of the QuantX Risk Managed Real Return Index.\n The fund generally will invest at least 80% of its total assets in the component securities of the index. The index methodology selects other investment companies (ETFs and closed-end funds), ETNs, limited or master limited partnerships, and REITs that invest in foreign and domestic real assets pursuant to a proprietary selection methodology that is designed to increase exposure to the best performing markets, while decreasing exposure to the worst performing markets.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:09.993953	2020-10-11 23:38:31.028166
RAD	Rite Aid Corporation	Rite Aid Corp operates retail drugstore chains in the United States. The business activity of the group is functioned through two segments namely Retail Pharmacy & Pharmacy Services. It generates a majority of its revenue from the Retail Pharmacy segment.	{"Consumer Defensive","Pharmaceutical Retailers","Retail - Defensive"}	{PETS,SFLY,CJJD,CVS,VSI,GNC,WBA}	Retail - Defensive	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:10.026782	2020-10-11 23:38:31.028166
RADA	RADA Electronic Industries Ltd.	Rada Electronics Industries Ltd develops, manufactures & sells defense electronics including, avionics solutions, airborne data/video recording & management systems, inertial navigation systems & land radars for defense forces & border protection systems.	{Industrials,"Aerospace & Defense"}	{TATT,ESLT,ISSC,LMT,GE,NOC,TIK}	Aerospace & Defense	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:10.026891	2020-10-11 23:38:31.031328
RAVI	FlexShares Ready Access Variable Income Fund	The investment seeks maximum current income consistent with the preservation of capital and liquidity.\n The fund seeks to achieve its investment objective by investing at least 80% of its total assets in a non-diversified portfolio of fixed-income instruments, including bonds, debt securities and other similar instruments issued by U.S. and non-U.S. public and private sector entities. The dollar-weighted average portfolio maturity of the fund is normally not expected to exceed two years. It may invest up to 20% of its total assets in fixed-income securities and instruments of issuers in emerging markets. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.046166	2020-10-11 23:38:31.028166
RBA	Ritchie Bros. Auctioneers Incorporated	Ritchie Bros Auctioneers Inc is an industrial auctioneer and used equipment distributor. It sells a broad range of used and unused equipment, including trucks, construction and heavy machinery and other assets.	{Industrials,"Business Services"}	{AMZN,DXPE,EBAY,NFLX,MSM,AIT,CLWT}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:10.053368	2020-10-11 23:38:31.02423
RBIN	Nationwide Risk-Based International Equity	The investment seeks to track the total return performance, before fees and expenses, of the Rothschild & Co Risk-Based International Index.\n At least 80% of its total assets will be invested in the component securities of the index. The index is a rules-based, equal risk-weighted index that is designed to provide exposure to large capitalization companies in developed markets outside the U.S. and Canada with lower volatility, reduced maximum drawdown, and an improved Sharpe ratio as compared to traditional, market capitalization weighted approaches.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.067782	2020-10-11 23:38:31.028166
RBS	Royal Bank of Scotland Group Plc New (The) ADS	The Royal Bank of Scotland Group PLC is an international banking and financial services company. It provides banking products and services to personal, commercial and large corporate and institutional customers.	{"Financial Services","Banks - Global",Banks}	{BMO,TD,BCS,HSBC,CIB}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:10.07283	2020-10-11 23:38:31.028166
RCL	Royal Caribbean Cruises Ltd.	Royal Caribbean Cruises Ltd is a global cruise company. It owns and operates three global cruise brands: Royal Caribbean International, Celebrity Cruises and Azamara Club Cruises.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{DIS,CUK,CCL,WYND,MAR,IHG,NCLH}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:10.095832	2020-10-11 23:38:31.028166
RCKY	Rocky Brands Inc.	Rocky Brands Inc is a designer, manufacturer and marketer of footwear and apparel marketed under a portfolio of brand names including Rocky, Georgia Boot, Durango, Lehigh, Creative Recreation and the licensed brand Michelin.	{"Consumer Cyclical","Footwear & Accessories","Manufacturing - Apparel & Furniture"}	{SHOO,COLM,SKX,WWW,DECK,CAL}	Manufacturing - Apparel & Furniture	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:10.093943	2020-10-11 23:38:31.031328
RDC	Rowan Companies plc Class A	Rowan Companies PLC is an offshore drilling contractor firm. The company engages in drilling oil and gas properties on contractual basis and mainly serves the petroleum industry.	{Energy,"Oil & Gas Drilling","Oil & Gas - Drilling"}	{DO,RIG,ESV,SDLP}	Oil & Gas - Drilling	Energy	New York Stock Exchange	\N	2020-10-11 23:12:10.116575	2020-10-11 23:38:31.023428
RDCM	Radcom Ltd.	Radcom Ltd provides service assurance and customer experience management solutions for leading telecom operators and communication service providers. It specializes in solutions for next generation mobile and fixed networks and mobile broadband.	{"Communication Services","Telecom Services"}	{EXFO,NATI,NTCT,A,DHR,IBM,VIAV}	Communication Services	Communication Services	NASDAQ Capital Market	\N	2020-10-11 23:12:10.121547	2020-10-11 23:38:31.025871
QYLD	Recon Capital NASDAQ-100 Covered Call ETF	The investment seeks to provide investment results that closely correspond, before fees and expenses, generally to the price and yield performance of the CBOE NASDAQ-100® BuyWrite V2 Index.\n The fund will invest at least 80% of its total assets in common stocks of the companies included in the NASDAQ-100 Index ("80% Policy"). The CBOE NASDAQ-100® BuyWrite Index is a benchmark index that measures the performance of a theoretical portfolio that holds a portfolio of the stocks included in the NASDAQ-100® Index, and "writes" (or sells) a succession of one-month at-the-money NASDAQ-100 Index covered call options. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:10.015811	2020-10-11 23:38:31.023428
RARE	Ultragenyx Pharmaceutical Inc.	Ultragenyx Pharmaceutical Inc is biopharmaceutical company. It is engaged identification, acquisition, development and commercialization of novel products for treatment of rare & ultra-rare diseases, with a focus on debilitating genetic diseases.	{Healthcare,Biotechnology}	{ALXN,BMRN,FOLD,ESPR,LXRX,SPEX,ALBO,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:10.041883	2020-10-11 23:38:31.028166
RBBN	Ribbon Communications Inc.	Ribbon Communications Inc is a real-time communications company. It is a provider of network solutions to telecommunications, wireless and cable service providers and enterprises across industry verticals.	{"Communication Services","Telecom Services"}	{CSCO,EXTR,AUDC,ERIC,INFN,JNPR,UTSI}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:12:10.064302	2020-10-11 23:38:31.02582
RCG	RENN Fund Inc	RENN Fund Inc is a registered non-diversified, closed-end fund focuses on investing in emerging publicly-traded growth companies.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:10.085243	2020-10-11 23:38:31.023428
RCS	PIMCO Strategic Income Fund Inc.	PIMCO Strategic Global Government Fund operates as a closed-end management investment company. Its primary investment objective is to achieve high current income and secondary objective is capital gains from disposition of investments.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:10.108165	2020-10-11 23:38:31.023007
RDN	Radian Group Inc.	Radian Group Inc provides mortgage insurance and products and services to the real estate and mortgage finance industries. It operates with two business segments namely Mortgage Insurance and Services.	{"Financial Services","Insurance - Specialty"}	{MTG,ORI,AGO,AIG,ESNT,NMIH}	Insurance - Specialty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:10.133357	2020-10-11 23:38:31.021426
REDU	RISE Education Cayman Ltd	RISE Education Cayman Ltd through its subsidiaries provides junior English Language Training services in China under RISE brand. It offers educational programs, services & products, consisting of educational courses, and other complementary products.	{"Consumer Defensive","Education & Training Services",Education}	{LRN,HMHC,GHC}	Education	Consumer Defensive	NASDAQ Global Market	\N	2020-10-11 23:12:10.15448	2020-10-11 23:38:31.02582
REED	Reeds Inc.	Reed's Inc sells Ginger Beer brand. Its Virgil brand is independent, all-natural craft soda brand. The company's portfolio has sold in over 20,000 retail doors across the natural, specialty, grocery, drug, club and mass channels nationwide.	{"Consumer Defensive","Beverages - Soft Drinks","Beverages - Non-Alcoholic"}	{COT,COKE,FIZZ,KO,MNST,PEP}	Beverages - Non-Alcoholic	Consumer Defensive	NYSE American	\N	2020-10-11 23:12:10.159729	2020-10-11 23:38:31.021426
REI	Ring Energy Inc.	Ring Energy Inc is a Midland-based exploration and production company that is engaged in oil and natural gas acquisition, exploration, development and production activities. Its exploration and production interests are in Texas.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{AREX,EGN,MXC,CXO,LPI}	Oil & Gas - E&P	Energy	NYSE American	\N	2020-10-11 23:12:10.184716	2020-10-11 23:38:31.028166
REML	Credit Suisse AG X-Links Monthly Pay 2xLeveraged Mortgage REIT Exchange Traded Notes (ETNs) due July 11 2036	The investment seeks to provide a monthly compounded 2x leveraged long exposure to the price return version of the FTSE NAREIT All Mortgage Capped Index (the “index”).\n The index measures the composite performance of tax-qualified U.S. mortgage real estate investment trusts (“Mortgage REITs”) with more than 50% of total assets invested in mortgage loans or mortgage-backed securities secured by interests in real property that are listed on the New York Stock Exchange, the NYSE Arca or the NASDAQ National Market List (the “index constituents”).	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.19793	2020-10-11 23:38:31.023007
REPH	Recro Pharma Inc.	Recro Pharma Inc is a specialty pharmaceutical company. It develops non-opioid products for treatment of serious acute pain. The Company operates through two business divisions: Acute Care division and contract development and manufacturing division.	{Healthcare,Biotechnology}	{DRRX,BDSI,JNJ,PCRX,ACRX,INSY,CARA,FLXN,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:10.224149	2020-10-11 23:38:31.028166
QXRR	QuantX Risk Managed Real Return	The investment seeks to replicate investment results that correspond to the performance of the QuantX Risk Managed Real Return Index.\n The fund generally will invest at least 80% of its total assets in the component securities of the index. The index methodology selects other investment companies (ETFs and closed-end funds), ETNs, limited or master limited partnerships, and REITs that invest in foreign and domestic real assets pursuant to a proprietary selection methodology that is designed to increase exposure to the best performing markets, while decreasing exposure to the worst performing markets.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:10.010669	2020-10-11 23:38:31.023007
RALS	ProShares RAFI Long Short	The investment seeks investment results, before fees and expenses, that track the performance of the FTSE RAFITM US 1000 Long/Short Total Return Index (the "index").\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should track the performance of the index. The index allocates an aggregate equal dollar amount to both long and short equity positions each time that the index rebalances. To be "long" means to hold or have long exposure to an asset with the expectation that its value will increase over time. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.035929	2020-10-11 23:38:31.023007
RBB	RBB Bancorp	RBB Bancorp is a bank holding company of the Royal Business Bank. The company is based in Los Angeles, California.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:10.061239	2020-10-11 23:38:31.024353
RBUS	Nationwide Risk-Based U.S. Equity	The investment seeks to track the total return performance, before fees and expenses, of the Rothschild & Co Risk-Based US Index (the "index").\n The advisor attempts to invest all, or substantially all, of its assets in the component securities that make up the index. Normally, at least 80% of the fund's total assets will be invested in the component securities of the index. The index is a rules-based, equal risk-weighted index that is designed to provide exposure to U.S.-listed large capitalization companies with lower volatility, reduced maximum drawdown, and an improved Sharpe ratio as compared to traditional, market capitalization weighted approaches.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.079282	2020-10-11 23:38:31.024353
RCM	R1 RCM Inc.	R1 RCM Inc provides management services to hospitals, physicians and other healthcare providers across US. The company offers services to efficiently manage cycle operations such as patient registration, insurance, and medical treatment documentation.	{Technology,"Health Information Services","Application Software"}	{XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:10.097054	2020-10-11 23:38:31.028166
RDFN	Redfin Corporation	Redfin Corp is a technology-powered residential real estate brokerage company. It represents people buying and selling homes in over 80 markets throughout the United States.	{"Real Estate","Real Estate Services"}	{NEN,MMI}	Real Estate Services	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:10.126935	2020-10-11 23:38:31.028166
RDWR	Radware Ltd.	Radware Ltd is a provider of cyber security and application delivery solutions designed to secure the user digital experience from applications in virtual, cloud and software defined data centers.	{Technology,"Communication Equipment"}	{CSCO,CTXS,SILC,FFIV,ALLT,JNPR,ATEN,HPQ,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:10.147981	2020-10-11 23:38:31.023428
REFA	Oppenheimer International Revenue	The investment seeks to provide investment results that correspond generally, before fees and expenses, to the performance of the OFI Revenue Weighted International Index (the "underlying index").\n The fund will normally invest at least 80% of its net assets, plus any borrowings for investment purposes, in securities of companies included in the underlying index. The underlying index is constructed using a rules-based methodology that re-weights the constituent securities of the MSCI EAFE Index (the "parent index") according to the revenue earned by the companies in the parent index, subject to a maximum 5% per company weighting.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:10.168715	2020-10-11 23:38:31.028166
RELL	Richardson Electronics Ltd.	Richardson Electronics Ltd is a provider of engineered solutions, power grid and microwave tubes and related consumables and customized display solutions.	{Technology,"Electronics Distribution","Computer Hardware"}	{TESS,TAIT,MKSI,AVX,ARW,GE,MMM,SANM,AVT,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:10.19043	2020-10-11 23:38:31.02582
REVG	REV Group Inc.	REV Group Inc is a designer, manufacturer and distributor of specialty vehicles and related aftermarket parts and services. It serves a diversified customer base in the United States through three segments, Fire & Emergency, Commercial and Recreation.	{"Consumer Cyclical","Recreational Vehicles",Autos}	{RACE}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:10.237637	2020-10-11 23:38:31.021426
SNPS	Synopsys Inc.	Synopsys Inc is a software development company. It supplies electronic design automation software used by engineers to design and test integrated circuits. It provides software and hardware used to develop the electronic systems that incorporate chips.	{Technology,"Software - Application","Application Software"}	{INTC,NATI,ANSS,ADSK,CDNS,PDFS,CEVA,IMI,XPER,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:11.506947	2020-10-11 23:38:31.021426
QXTR	QuantX Risk Managed Multi-Asset Total Return	The investment seeks to replicate investment results the QuantX Risk Managed Multi-Asset Total Return Index.\n The fund generally will invest at least 80% of its total assets in the component securities of the index. The index methodology selects other investment companies (ETFs and closed-end funds), ETNs, limited or master limited partnerships, and REITs that invest in foreign and domestic equity securities, real estate, commodities, corporate bonds, high-yield bonds pursuant to a proprietary selection methodology that is designed to increase exposure to the best performing markets, while decreasing exposure to the worst performing markets.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:10.01422	2020-10-11 23:38:31.025871
RAIL	Freightcar America Inc.	FreightCar America Inc is a manufacturer of aluminum-bodied railcars in North America. The Company specializes in production of coal cars.	{Industrials,Railroads,"Transportation & Logistics"}	{ARII,GBX,TRN,WAB}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:10.030726	2020-10-11 23:38:31.021426
RAVN	Raven Industries Inc.	Raven Industries Inc is a diversified technology company providing products to customers within the industrial, agricultural, energy, construction, and defense markets.	{Industrials,"Diversified Industrials","Industrial Products"}	{OTTR,TRMB,CSL,TXT,TG,UTX,FFHL}	Industrial Products	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:10.051144	2020-10-11 23:38:31.023428
RBCAA	Republic Bancorp Inc. Class A Common Stock	Republic Bancorp Inc is a financial holding company. The company along with its subsidiaries provide property and casualty insurance coverage and also is in directing, planning and coordinating the business activities.	{"Financial Services","Banks - Regional - US",Banks}	{UNTY,CWBC}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:10.067296	2020-10-11 23:38:31.025871
RCI	Rogers Communication Inc.	Rogers Communications Inc is the largest wireless carrier in Canada. The company provides cable network, Internet access, TV, and phone services. Through its subsidiaries it owns the Sportsnet TV network.	{"Communication Services","Telecom Services"}	{SJR,BCE,NFLX,TU,CCA}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:10.086698	2020-10-11 23:38:31.02423
RCON	Recon Technology Ltd.	Recon Technology Ltd provides oilfield specialized equipment, automation systems, tools, chemicals and field services to petroleum companies mainly in the People's Republic of China.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{NOV,SLB,BHGE}	Oil & Gas - Services	Energy	NASDAQ Capital Market	\N	2020-10-11 23:12:10.108113	2020-10-11 23:38:31.024353
RDI	Reading International Inc	Reading International Inc is a diversified company, engaged in the development, ownership and operation of multiplex cinemas and retail and commercial real estate in the United States, Australia and New Zealand.	{"Consumer Cyclical","Media - Diversified",Entertainment}	{MCS,CNK,AMC}	Entertainment	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:10.129293	2020-10-11 23:38:31.02423
RDY	Dr. Reddy's Laboratories Ltd	Dr Reddy's Laboratories Ltd is associated with the healthcare sector. Its core business includes the manufacturing of pharmaceutical products.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{GSK,NVS,PFE,SNY,TEVA,XLV}	Drug Manufacturers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:10.147585	2020-10-11 23:38:31.023007
REFR	Research Frontiers Incorporated	Research Frontiers Inc develops and licenses its patented suspended particle device light-control technology to other companies that manufacture and market either the SPD-Smart chemical emulsion, light-control film made from the chemical emulsion.	{Technology,"Electronic Components","Computer Hardware"}	{IIVI,LPTH,APD,AVY,GE,DYSL,AA,XLK}	Computer Hardware	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:10.16895	2020-10-11 23:38:31.023428
REET	iShares Trust Global REIT	The investment seeks to track the investment results of the FTSE EPRA Nareit Global REITS Index.\n The index is designed to track the performance of publicly-listed real estate investment trusts ("REITs") (or their local equivalents) in both developed and emerging markets. The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.166519	2020-10-11 23:38:31.025871
REGN	Regeneron Pharmaceuticals Inc.	Regeneron Pharmaceuticals Inc is an integrated biopharmaceutical company. It discovers, invents, develops, manufactures, and commercializes medicines for the treatment of serious medical conditions.	{Healthcare,Biotechnology}	{AMGN,BIIB,ALNY,NVS,JNJ,SNY,OPHT,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:10.183157	2020-10-11 23:38:31.031328
REM	iShares Trust Mortgage Real Estate	The investment seeks to track the investment results of the FTSE Nareit All Mortgage Capped Index composed of U.S. real estate investment trusts ("REITs") that hold U.S. residential and commercial mortgages.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The underlying index measures the performance of the residential and commercial mortgage real estate, mortgage finance and savings associations sectors of the U.S. equity market. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:10.196677	2020-10-11 23:38:31.025871
REW	ProShares UltraShort Technology	The investment seeks daily investment results, before fees and expenses, that correspond to two times the inverse (-2x) of the daily performance of the Dow Jones U.S. TechnologySM Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index seeks to measure the performance of certain companies in the technology sector of the U.S. equity market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.237872	2020-10-11 23:38:31.023428
RFV	Invesco S&P Midcap 400 Pure Value	The investment seeks to track the investment results (before fees and expenses) of the S&P MidCap 400® Pure Value Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities that comprise the underlying index. The underlying index is comprised only of those S&P MidCap 400® companies with strong value characteristics as measured using three factors: book to value price ratio, earnings to price ratio, and sales to price ratio. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.283369	2020-10-11 23:38:31.031328
RGR	Sturm Ruger & Company Inc.	Sturm Ruger & Co Inc is engaged in the design, manufacture, and sale of firearms to domestic customers. The Company also manufactures and sells investment castings made from steel alloys for internal use in the firearms segment.	{Industrials,"Aerospace & Defense"}	{NPK,AOBC}	Aerospace & Defense	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:10.303168	2020-10-11 23:38:31.031328
RICK	RCI Hospitality Holdings Inc.	RCI Hospitality Holdings Inc through its subsidiaries owns and operates establishments that offer live adult entertainment, restaurant, and bar operations.	{"Consumer Cyclical",Restaurants}	{ARKR,CAKE,BJRI,BDL}	Restaurants	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:10.334466	2020-10-11 23:38:31.021426
RIO	Rio Tinto Plc	Rio Tinto PLC is a mining and exploration company. It discovers, mines, processes and supplies metals and minerals used in everyday modern life. Its main products include Bauxite, Alumina, Copper, Gold, Molybdenum, Silver, Nickel, and Diamonds.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{TECK,BBL,FNV,FCX,BHP,VALE}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:10.364451	2020-10-11 23:38:31.024353
RL	Ralph Lauren Corporation	Ralph Lauren Corp is a fashion apparels manufacturer that designs and distribute lifestyle products including apparel, accessories, home furnishings, and other licensed product categories through department stores, specialty retailers and its own outlets.	{"Consumer Cyclical","Apparel Manufacturing","Manufacturing - Apparel & Furniture"}	{}	Manufacturing - Apparel & Furniture	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:10.39852	2020-10-11 23:38:31.021426
RMBS	Rambus Inc.	Rambus Inc is a technology solutions company. The company creates semiconductor and IP products, spanning memory and interfaces to security, smart sensors and lighting.	{Technology,"Semiconductor Memory",Semiconductors}	{MU,NVDA,NXPI,TXN,AMD,MOSY,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:10.419056	2020-10-11 23:38:31.023007
RNEM	First Trust Emerging Markets Equity Select ETF	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the Nasdaq Riskalyze Emerging Markets Index (the "index").\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in the equity securities that comprise the index. The index is designed to select low volatility emerging market securities that are included in the Nasdaq Emerging Markets Large Mid Cap Index (the "base index"). The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:10.439922	2020-10-11 23:38:31.028166
RODM	Hartford Multifactor Developed Markets (ex-US)	The investment seeks to provide investment results that, before fees and expenses, correspond to the total return performance of the Hartford Risk-Optimized Multifactor Developed Markets (ex-US) Index.\n The fund generally invests at least 80% of its assets in securities included in the index and in depositary receipts representing securities included in the index. The index is designed to address risks and opportunities within developed international economies outside the U.S. by selecting equity securities of companies domiciled within developed international equity markets exhibiting a favorable combination of factor characteristics.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.477099	2020-10-11 23:38:31.023428
ROP	Roper Technologies Inc.	Roper Technologies Inc is a diversified technology company. The firm offers industrial pumps, water meters, sensors, valves, control systems, highway tolling systems and SaaS brokerage for freight and food markets, digital imaging, and software systems.	{Industrials,"Diversified Industrials","Industrial Products"}	{AME,BMI,DHR,EMR,GE,IEX,TFX,TMO,WTS,PKI}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:10.495736	2020-10-11 23:38:31.028166
RPG	Invesco S&P 500 Pure Growth	The investment seeks to track the investment results (before fees and expenses) of the S&P 500® Pure Growth Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities that comprise the underlying index. The underlying index includes securities that exhibit the strongest growth characteristics as measured using three factors: three-year sales per share growth, three-year ratio of earnings per share change to price per share, and momentum (12-month percentage price change) as selected by the index provider strictly in accordance with its guidelines and mandated procedures. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.516333	2020-10-11 23:38:31.028166
RFFC	RiverFront Dynamic US Flex-Cap	The investment seeks to provide capital appreciation.\n The fund invests at least 65% of its net assets in a portfolio of equity securities of publicly traded U.S. companies. Equity securities include common stocks and common or preferred shares of real estate investment trusts. The fund may invest in small-, mid- and large-capitalization companies. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.26696	2020-10-11 23:38:31.02582
RGCO	RGC Resources Inc.	RGC Resources Inc is an energy services company engaged in the regulated sale and distribution of natural gas through its subsidiary.	{Utilities,"Utilities - Regulated Gas","Utilities - Regulated"}	{NI,ATO,CPK,SWX,OGS,XLU}	Utilities - Regulated	Utilities	NASDAQ Global Market	\N	2020-10-11 23:12:10.286347	2020-10-11 23:38:31.028166
RHP	Ryman Hospitality Properties Inc. (REIT)	Ryman Hospitality Properties Inc specializes in group-oriented, destination hotel assets in urban and resort markets.	{"Real Estate","REIT - Hotel & Motel",REITs}	{RLJ,INN,HASI,XHR}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:10.312934	2020-10-11 23:38:31.023428
RILYZ	B. Riley Financial Inc. 7.50% Senior Notes Due 2027		{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:10.355925	2020-10-11 23:38:31.028166
RJF	Raymond James Financial Inc.	Raymond James Financial Inc is a diversified financial services company providing private client group, capital markets, asset management, banking and other services to individuals, corporations and municipalities.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{SIEB,AMTD,ETFC,SCHW,SF,LTS,LPLA}	Brokers & Exchanges	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:10.374913	2020-10-11 23:38:31.023428
RLH	Red Lions Hotels Corporation	Red Lion Hotels Corp is a hospitality group covering upscale, midscale and economy hotels. It owns, manages and franchises hotels and owns and operates an event ticket distribution business. It operates in the United States.	{"Consumer Cyclical",Lodging,"Travel & Leisure"}	{}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:10.40717	2020-10-11 23:38:31.024353
RMD	ResMed Inc.	ResMed Inc is a part of the healthcare sector. It develops and distributes medical products for treating and diagnosing respiratory disorders.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{MASI,TMO,PHG,ELMD,INGN,MDT,AHPI,XLV}	Medical Instruments & Equipment	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:10.427514	2020-10-11 23:38:31.02582
RNET	RigNet Inc.	RigNet Inc provides managed remote communications, engineered telecommunications solutions and applications. It offers fully-managed voice and data networks to applications that include video conferencing, crew welfare, and real-time data services.	{Technology,"Communication Equipment"}	{ITRN,LORL,IRDM,SLB,HRS,OII,GSAT,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:10.443458	2020-10-11 23:38:31.02582
ROLL	RBC Bearings Incorporated	RBC Bearings Inc is a manufacturer and marketer of engineered precision plain, roller and ball bearings. It operates in four business segments namely roller bearings, plain bearings, ball bearings, and engineered products.	{Industrials,"Tools & Accessories","Industrial Products"}	{NNBR,AIMC,TKR,KAMN,RXN}	Industrial Products	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:10.489126	2020-10-11 23:38:31.021426
ROST	Ross Stores Inc.	Ross Stores Inc is the off-price apparel and home fashion chains which is engaged in selling name-brand and designer apparel, accessories, footwear, and home fashions to middle-income consumers.	{"Consumer Cyclical","Apparel Stores","Retail - Apparel & Specialty"}	{URBN,SMRT,OSTK,CTRN,GES,ANF,AEO,GPS,JWN,TJX}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:10.505567	2020-10-11 23:38:31.02423
RPV	Invesco S&P 500 Pure Value	The investment seeks to track the investment results (before fees and expenses) of the S&P 500® Pure Value Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities that comprise the underlying index. The underlying index includes securities that exhibit the strongest value characteristics as measured using three factors: book to value price ratio, earnings to price ratio, and sales to price ratio, as selected by the index provider strictly in accordance with its guidelines and mandated procedures. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.528911	2020-10-11 23:38:31.023428
RSLS	ReShape Lifesciences Inc.	Reshape Lifesciences Inc is a medical device company. The company is focused on the design and development of devices which use neuroblocking technology to treat obesity, metabolic diseases and other gastrointestinal disorders.	{Healthcare,"Medical Devices"}	{XLV}	Medical Devices	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:10.5515	2020-10-11 23:38:31.025871
RUN	Sunrun Inc.	Sunrun Inc provides clean, solar energy to homeowners. It is engaged in the design, development, installation sale, ownership, and maintenance of residential solar energy systems ("Projects") in the United States.	{Technology,Solar,Semiconductors}	{RUN,LMB,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:10.574588	2020-10-11 23:38:31.02582
RVNC	Revance Therapeutics Inc.	Revance Therapeutics Inc is a clinical-stage biopharmaceutical company. It is engaged in the development, manufacturing, and commercialization of novel botulinum toxin products for multiple aesthetic and therapeutic applications.	{Healthcare,Biotechnology}	{BIIB,MNOV,JNJ,IONS,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:10.597149	2020-10-11 23:38:31.02423
RFIL	RF Industries Ltd.	RF Industries Ltd is engaged in the design, manufacture and marketing of interconnect products and systems, including coaxial and specialty cables, fiber optic cables and connectors, and electrical and electronic specialty cables.	{Technology,"Electronic Components","Computer Hardware"}	{OCC,CLFD,APH,GLW,MEI,BDC,TEL,XLK}	Computer Hardware	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:10.275735	2020-10-11 23:38:31.023007
RGLD	Royal Gold Inc.	Royal Gold Inc manages precious metal royalties and streams, with a focus on gold. The company seeks to acquire existing stream and royalty interests or to finance projects that are in production or in the development stage.	{"Basic Materials",Gold,"Metals & Mining"}	{ABX,TECK,GG,CDE,NEM,VALE,WPM}	Metals & Mining	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:12:10.291724	2020-10-11 23:38:31.023428
RHI	Robert Half International Inc.	Robert Half International Inc is a staffing agency providing specialized staffing and risk consulting services through divisions such as Accountemps, Robert Half Finance & Accounting, OfficeTeam, Robert Half Technology, and others.	{Industrials,"Staffing & Outsourcing Services","Employment Services"}	{BBSI,HSII,KELYA,KFRC,KFY,MAN,TBI}	Employment Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:10.311618	2020-10-11 23:38:31.02423
RIGL	Rigel Pharmaceuticals Inc.	Rigel Pharmaceuticals Inc is a clinical-stage biotechnology company. It discovers and develops novel, small-molecule drugs for the treatment of inflammatory/autoimmune diseases, as well as for certain cancers and metabolic diseases.	{Healthcare,Biotechnology}	{AMGN,SGMO,INFI,AZN,CCXI,ARDM,MYL,INVA,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:10.342242	2020-10-11 23:38:31.023428
RIOT	Riot Blockchain Inc	Riot Blockchain Inc is a strategic investor and operator in the blockchain ecosystem with focus on the bitcoin and ethereum blockchains.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{MYGN,OSUR,VRML,CEMI}	Brokers & Exchanges	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:10.36729	2020-10-11 23:38:31.025871
RKDA	Arcadia Biosciences Inc.	Arcadia Biosciences Inc is an agricultural biotechnology trait company. The company develops a portfolio of yield and traits addressing multiple crops that supply the global food and feed markets.	{"Basic Materials","Agricultural Inputs",Agriculture}	{}	Agriculture	Basic Materials	NASDAQ Capital Market	\N	2020-10-11 23:12:10.385502	2020-10-11 23:38:31.028166
RMBL	RumbleOn Inc.	RumbleON Inc is an online retailer which allows consumers and dealers to buy, sell and trade pre-owned motorcycles. It provides cash offers for the purchase of vehicles and flexibility to trade, list, and auction vehicle through website and application.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{PDFS,OMCL}	Retail - Apparel & Specialty	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:10.41384	2020-10-11 23:38:31.025871
RNSC	First Trust Small Cap US Equity Select ETF	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the Nasdaq Riskalyze US Small Cap Index (the "index").\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is designed to select dividend-paying US small capitalization securities that are included in the Nasdaq US 700 Small Cap Index (the "base index"). The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:10.45721	2020-10-11 23:38:31.028166
ROG	Rogers Corporation	Rogers Corp designs, develops, manufactures and sells engineered materials and components for mission critical applications.	{Technology,"Electronic Components","Computer Hardware"}	{OLED,IIVI,KBAL,BHE,PKE,VSH,MMM,SPA,RBCN,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:10.476622	2020-10-11 23:38:31.025871
RORE	Hartford Multifactor REIT	The investment seeks to track the investment results of the Hartford Risk-Optimized Multifactor REIT Index.\n The fund generally will invest at least 80 percent of its net assets (plus the amount of borrowings for investment purposes) in securities of REITs. The index is designed to capture the income and growth potential of investing within the U.S. REIT universe. The Strategy selects equity securities of REITs exhibiting a favorable combination of factor characteristics, including quality, momentum, and value. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.495803	2020-10-11 23:38:31.024353
RPAI	Retail Properties of America Inc. Class A	Retail Properties of America Inc is a self-managed real estate investment trust. The company invests, develops, and manages retail property assets that include shopping centers in the United States.	{"Real Estate","REIT - Retail",REITs}	{}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:10.515669	2020-10-11 23:38:31.021426
RQI	Cohen & Steers Quality Income Realty Fund Inc	Cohen & Steers Quality Inc Realty Fund is a diversified, closed-end management investment company. The primary investment objective of the Fund is high current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:10.534284	2020-10-11 23:38:31.025871
SNV	Synovus Financial Corp.	Synovus Financial Corp is a financial services and bank holding company. It provides financial services including commercial and retail banking, financial management, insurance and mortgage services. It also provides life insurance premium financing.	{"Financial Services","Banks - Regional - US",Banks}	{FCBC,FCCO,FITB,UBSI,BBT,PNC,WFC,STI,FHN,RF}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:11.510553	2020-10-11 23:38:31.023007
RFP	Resolute Forest Products Inc.	Resolute Forest Products Inc is engaged in the forest products industry which includes newsprint, specialty papers, market pulp, wood products and others. The company owns or operates facilities in the United States, Canada, and South Korea.	{"Basic Materials","Paper & Paper Products","Forest Products"}	{UFS,MERC,SWM,GLT}	Forest Products	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:10.276028	2020-10-11 23:38:31.023428
RGNX	REGENXBIO Inc.	Regenxbio Inc is a biotechnology company. It is engaged in the development, commercialization and licensing of recombinant adeno-associated virus gene therapy.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:10.292673	2020-10-11 23:38:31.021426
RHT	Red Hat Inc.	Red Hat Inc is a software company operating business across the world. It is primarily engaged in providing services to its version of the open-source Linux software operating systems, middleware, storage, virtualization, and management tools.	{Technology,"Software - Application","Application Software"}	{MSFT,CTXS,SYMC,NTAP,IBM,VMW,ORCL,HPQ,GOOGL,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:10.319364	2020-10-11 23:38:31.02582
RILY	B. Riley Financial Inc.	B. Riley Financial Inc is a diversified financial services company. The reportable operating segments of the company include Capital Markets, Auction and Liquidation, Valuation and Appraisal and Principal Investments.	{"Financial Services","Asset Management"}	{INTX,ZG}	Asset Management	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:10.349255	2020-10-11 23:38:31.028166
RIV	RiverNorth Opportunities Fund Inc.	Rivernorth Tactical Oppurtunities Fund is a diversified, closed-end management investment company. The Fund's investment objective is total return consisting of capital appreciation and current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:10.371079	2020-10-11 23:38:31.021426
RLGY	Realogy Holdings Corp.	Realogy Holdings Corp is a provider of residential real estate services in the U.S. The Company is a franchisor of residential real estate brokerages in the real estate industry.	{"Real Estate","Real Estate Services"}	{JOE,JLL,RMAX,MMI}	Real Estate Services	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:10.404182	2020-10-11 23:38:31.025871
RMCF	Rocky Mountain Chocolate Factory Inc.	Rocky Mountain Chocolate Factory Inc is an international franchisor and confectionery manufacturer. It produces an extensive line of chocolate candies and other confectionery products.	{"Consumer Defensive",Confectioners,"Consumer Packaged Goods"}	{JBSS,HSY,TR}	Consumer Packaged Goods	Consumer Defensive	NASDAQ Global Market	\N	2020-10-11 23:12:10.42458	2020-10-11 23:38:31.031328
ROGS	Hartford Multifactor Global Small Cap	The investment seeks to provide investment results that, before fees and expenses, correspond to the total return performance of the Hartford Risk-Optimized Multifactor US Equity Index.\n The fund generally invests at least 80 percent of its assets in securities of the index and in depositary receipts representing securities of the index. The index seeks to improve returns through a market cycle relative to traditional cap-weighted U.S. equity market indices and active U.S. equity market strategies.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.480205	2020-10-11 23:38:31.028166
ROSEU	Rosehill Resources Inc. Unit	Rosehill Resources Inc is an independent oil and natural gas company focused on growing production and reserves in the core of the Delaware Basin, through drilling and bolt-on acquisitions.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{}	Oil & Gas - E&P	Energy	NASDAQ Capital Market	\N	2020-10-11 23:12:10.502771	2020-10-11 23:38:31.02582
RRC	Range Resources Corporation	Range Resources Corp is an independent natural gas exploration and production company. All the business activities are functioned in the areas of Pennsylvania and North Louisiana. It generates its revenue from Natural gas, NGLs and oil sales.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{ECA,CHK,PXD,COG,NBL,EQT,CXO,AR,ECR}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:10.534208	2020-10-11 23:38:31.023007
RSP	Invesco S&P 500 Equal Weight	The investment seeks to track the investment results (before fees and expenses) of the S&P 500® Equal Weight Index (the "underlying index").\n The fund generally invests in all of the securities comprising the underlying index in proportion to their weightings in the underlying index. Strictly in accordance with its guidelines and mandated procedures, the index provider compiles, maintains and calculates the underlying index, which is an equal-weighted version of the S&P 500® Index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.551554	2020-10-11 23:38:31.02582
RUBI	The Rubicon Project Inc.	The Rubicon Project Inc provides technology solutions consisting of applications for sellers, providers of websites, mobile applications and other digital media properties to automate the purchase and sale of advertising for both buyers and sellers.	{Technology,"Software - Infrastructure","Application Software"}	{SEAC,NWSA,DOX,GOOGL,CIDM,NTN,TIVO,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:10.57483	2020-10-11 23:38:31.031328
RVNU	Xtrackers Municipal Infrastructure Revenue Bond	The investment seeks investment results that correspond generally to the performance, before fees and expenses, of the Solactive Municipal Infrastructure Revenue Bond Index (the "underlying index").\n The fund will invest at least 80% of its total assets (but typically far more) in instruments that comprise the underlying index. The underlying index is comprised of tax-exempt municipal securities issued by states, cities, counties, districts, their respective agencies, and other tax-exempt issuers. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.59867	2020-10-11 23:38:31.028166
RGA	Reinsurance Group of America Incorporated	Reinsurance Group of America Inc is an insurance holding company. It provides various products including life reinsurance, living benefits reinsurance, group reinsurance, health reinsurance, financial solutions, and product development.	{"Financial Services","Insurance - Reinsurance","Insurance - Specialty"}	{MHLD,UNM,AFL,TMK}	Insurance - Specialty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:10.284908	2020-10-11 23:38:31.02423
RGS	Regis Corporation	Regis Corp owns, franchises and operates beauty salons throughout North America and the United Kingdom. The company's locations provide salon products and services to the mass market, including haircutting, styling, and hair coloring.	{"Consumer Cyclical","Personal Services"}	{NTRI,SFLY,ULTA,WTW}	Personal Services	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:10.304651	2020-10-11 23:38:31.023007
RIG	Transocean Ltd (Switzerland)	Transocean Ltd is an international provider of offshore contract drilling services for oil and gas wells. Its business activities include contracting drilling rigs, related equipment and work crews to drill oil and gas wells.	{Energy,"Oil & Gas Drilling","Oil & Gas - Drilling"}	{PTEN,DO,HP,NBR,RDC,ESV,NE}	Oil & Gas - Drilling	Energy	New York Stock Exchange	\N	2020-10-11 23:12:10.340292	2020-10-11 23:38:31.031328
RISE	Sit Rising Rate	The investment seeks to profit from rising interest rates by tracking the performance of a portfolio consisting of exchange traded futures contracts and options on futures on 2, 5 and 10-year U.S. Treasury securities weighted to achieve a targeted negative 10 year average effective portfolio duration.\n The weighting of the Treasury Instruments constituting the Benchmark Component Instruments will be based on each maturity's duration contribution. The expected range for the duration weighted percentage of the 2 year and 5 year maturity Treasury Instruments will be from 30% to 70%.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.368883	2020-10-11 23:38:31.028166
RLI	RLI Corp. (DE)	RLI Corp provides property and casualty insurance coverages in both the specialty admitted and excess and surplus markets. It distributes its property and casualty insurance through branch offices.	{"Financial Services","Insurance - Property & Casualty"}	{ACGL,AFG,WRB,CNA,MKL,ORI,TRV,WTM}	Insurance - Property & Casualty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:10.409387	2020-10-11 23:38:31.028166
RMNI	Rimini Street Inc.	Rimini Street Inc is a provider of enterprise software support products and services, and the third-party support provider for Oracle and SAP software products. Its products include Oracle e-business suite, SAP business suite, Agile PLM, among others.	{Technology,"Software - Application","Application Software"}	{OTEX,PANW,XLK}	Application Software	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:10.43139	2020-10-11 23:38:31.028166
RNMC	First Trust Mid Cap US Equity Select ETF	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the Nasdaq Riskalyze US Mid Cap Index (the "index").\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in the equity securities that comprise the index. The index is designed to select dividend-paying US mid cap securities that are included in the Nasdaq US 600 Mid Cap Index (the "base index"). The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:10.453945	2020-10-11 23:38:31.028166
RODI	Barclays Return on Disability ETN	The investment seeks a return linked to the performance of the Return on Disability US LargeCap ETN Total Return USD Index.\n The index notionally tracks the returns that may be available from investing in a Basket comprised of stocks of up to 100 companies that, according to the RoD Ranking, are considered the top ranked firms in the disability market with respect to creation of shareholder value and which meet other eligibility requirements.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.473464	2020-10-11 23:38:31.024353
ROOF	IQ US Real Estate Small Cap	The investment seeks investment results that correspond generally to the price and yield performance of its underlying index, the IQ U.S. Real Estate Small Cap Index.\n The fund invests at least 80% of its net assets, plus the amount of any borrowings for investment purposes, in the investments included in its index. The index is a rules based, modified capitalization weighted, float adjusted index intended to give investors a means of tracking the overall performance of the small capitalization sector of publicly traded companies domiciled and primarily listed on an exchange in the U.S. and that invest in real estate. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.492915	2020-10-11 23:38:31.025871
RPD	Rapid7 Inc.	Rapid7 Inc is a provider of security data and analytics solutions that enable organizations to implement an active approach to cyber security.	{Technology,"Software - Application","Application Software"}	{OTEX,PANW,XLK}	Application Software	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:10.517226	2020-10-11 23:38:31.02423
RS	Reliance Steel & Aluminum Co. (DE)	Reliance Steel & Aluminum Co is engaged in providing metal processing and inventory management services for carbon and stainless steel, aluminum, alloys, brass and copper, and titanium.	{"Basic Materials",Steel}	{ZEUS,CRS,NUE,WOR,FRD,AA}	Steel	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:10.540232	2020-10-11 23:38:31.028166
RTIX	RTI Surgical Inc.	RTI Surgical Inc is a surgical implant company. It processes donated human musculoskeletal and other tissue, including bone, cartilage, tendon, ligament, fascia lata, pericardium, sclera, dermal tissue and bovine and porcine animal tissue.	{Healthcare,"Medical Devices"}	{IART,NUVA,JNJ,SYK,ZBH,MDT,ATEC,XLV}	Medical Devices	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:10.562243	2020-10-11 23:38:31.024353
RUSHB	Rush Enterprises Inc. Class B Common Stock	Rush Enterprises Inc is a commercial vehicle dealerships operator. It sells buses and trucks, of various brands such as Peterbilt, Hino, Isuzu, Ford, IC Bus, and Blue Bird. It also offers used vehicles, aftermarket parts, and maintenance services.	{"Consumer Cyclical","Auto & Truck Dealerships",Autos}	{LAD,AN,GPI,SAH,ABG,KMX,PAG}	Autos	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:10.579656	2020-10-11 23:38:31.023428
RGI	Invesco S&P 500 Equal Weight Industrials Portfolio	The investment seeks to track the investment results (before fees and expenses) of the S&P 500® Equal Weight Information Technology Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities that comprise the underlying index. The underlying index is an equal-weighted version of the S&P 500® Information Technology Index. Strictly in accordance with its guidelines and mandated procedures, the index provider compiles, maintains and calculates the underlying index, which is comprised of common stocks of companies in the information technology sector of the S&P 500® Index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.288363	2020-10-11 23:38:31.028166
RH	RH	RH, formerly Restoration Hardware Holdings Inc is a luxury brand in the home furnishings marketplace offering furniture, lighting, textiles, outdoor and garden products. It offers collections through its retail galleries, source books and websites.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{BBBY,KIRK,HVT,PIR,WSM}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:10.307102	2020-10-11 23:38:31.028166
RIF	RMR Real Estate Income Fund (MD)	Rmr Real Estate Income Fund is a closed-end investment company. Its investment objective is to earn and pay a high level of current income to its common shareholders by investing in real estate companies, including REITs.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:10.335069	2020-10-11 23:38:31.02423
RJA	AB Svensk Ekportkredit (Swedish Export Credit Corporation) ELEMENTS Linked to the Rogers International Commodity Index - Agriculture Total Return Structured Pro	The investment seeks to replicate, net of expenses, the Rogers International Commodity Index – Agriculture Total Return index.\n The index represents the value of a basket of 20 agricultural commodity futures contracts.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.372527	2020-10-11 23:38:31.02423
RMAX	RE/MAX Holdings Inc. Class A	RE/MAX Holdings Inc operates as a franchisor of real estate brokerage services. The company recruits and retains agents and sells franchises.	{"Real Estate","Real Estate Services"}	{RLGY,GLPI,MMI,NEWT}	Real Estate Services	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:10.412154	2020-10-11 23:38:31.021426
RMT	Royce Micro-Cap Trust Inc.	Royce Micro-Cap Trust Inc is a diversified closed-end investment company. It is engaged in investing in micro-cap securities. The company distributes capital gains on a quarterly basis.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:10.434761	2020-10-11 23:38:31.028166
RNR	RenaissanceRe Holdings Ltd.	RenaissanceRe Holdings Ltd is a provider of insurance products. Its portfolio encompasses property catastrophe and specialty reinsurance risks.	{"Financial Services","Insurance - Reinsurance","Insurance - Specialty"}	{RE,AXS,AHL,BCRH}	Insurance - Specialty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:10.455258	2020-10-11 23:38:31.02582
ROIC	Retail Opportunity Investments Corp.	Retail Opportunity Investments Corp is a self-managed real estate investment trust. The company invests in, leases, and manages retail properties in the United States. Its portfolio consists primarily of community and neighbourhood shopping centres.	{"Real Estate","REIT - Retail",REITs}	{VNO,ARE,CHSP}	REITs	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:10.481783	2020-10-11 23:38:31.02582
ROSE	Rosehill Resources Inc.	Rosehill Resources Inc is an independent oil and natural gas company focused on growing production and reserves in the core of the Delaware Basin, through drilling and bolt-on acquisitions.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{}	Oil & Gas - E&P	Energy	NASDAQ Capital Market	\N	2020-10-11 23:12:10.495896	2020-10-11 23:38:31.023428
ROYT	Pacific Coast Oil Trust Units of Beneficial Interest	Pacific Coast Oil Trust is engaged in acquiring and holding net profits and royalty interests in certain oil and natural gas properties located in California for the benefit of the Trust unitholders.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{BPT,CRT,MTR,SJT}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:10.50976	2020-10-11 23:38:31.023007
RRTS	Roadrunner Transportation Systems Inc	Roadrunner Transportation Systems Inc provides non-asset based transportation and logistics services including customized and expedited LTL (less-than-truckload), truckload logistics and intermodal brokerage, and domestic and international air.	{Industrials,Trucking,"Transportation & Logistics"}	{LSTR,CVTI,ODFL,ULH,CHRW,SAIA,YRCW,ARCB,KNX}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:10.538186	2020-10-11 23:38:31.024353
RSYS	RadiSys Corporation	Radisys Corp provides wireless infrastructure solutions for the telecom, aerospace and defense markets. The Company provides hardware and software platforms for Next Generation IP-based wireless, wireline and video networks.	{Technology,"Computer Systems","Computer Hardware"}	{CSCO,DGII,CRAY,EMR,GE,HPQ,SMCI,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:10.556929	2020-10-11 23:38:31.028166
RUSHA	Rush Enterprises Inc. Class A Common Stock	Rush Enterprises Inc is a commercial vehicle dealerships operator. It sells buses and trucks, of various brands such as Peterbilt, Hino, Isuzu, Ford, IC Bus, and Blue Bird. It also offers used vehicles, aftermarket parts, and maintenance services.	{"Consumer Cyclical","Auto & Truck Dealerships",Autos}	{LAD,AN,GPI,SAH,ABG,KMX,PAG}	Autos	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:10.581207	2020-10-11 23:38:31.028166
RVSB	Riverview Bancorp Inc	Riverview Bancorp Inc is the savings & loan holding company of Riverview Community Bank. The company is engaged in the business of attracting deposits from the general public & using such funds in its market area to originate commercial business.	{"Financial Services","Savings & Cooperative Banks",Banks}	{WAFD,PROV,TSBK,FFNW,TBNK,ANCB,FSBW,BANC}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:10.599967	2020-10-11 23:38:31.023428
RHE	Regional Health Properties Inc.	Regional Health Properties Inc is a self-managed healthcare real estate investment company. It invests primarily in real estate purposed for senior living and long-term healthcare through facility lease and sub-lease transactions.	{Healthcare,"Long-Term Care Facilities","Health Care Providers"}	{ENSG,NHC,CSU,DVCR,GEN,FVE,XLV}	Health Care Providers	Healthcare	NYSE American	\N	2020-10-11 23:12:10.310133	2020-10-11 23:38:31.024353
RINF	ProShares Inflation Expectations	The investment seeks investment results, before fees and expenses, that track the performance of the FTSE 30-Year TIPS (Treasury Rate Hedged) Index (the "index").\n The fund, under normal circumstances, seeks to remain fully exposed to the index and will invest at least 80% of its total assets in securities of the index. The index tracks the performance of (i) long positions in the most recently issued 30-year Treasury Inflation-Protected Securities ("TIPS") and (ii) duration-adjusted short positions in U.S. Treasury bonds of, in aggregate, approximate equivalent duration dollars to the TIPS. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.363514	2020-10-11 23:38:31.02582
RLGT	Radiant Logistics Inc.	Radiant Logistics Inc is a transportation and logistics services company providing customers domestic and international freight forwarding services through a network of Company-owned and strategic operating partner locations.	{Industrials,"Integrated Shipping & Logistics","Transportation & Logistics"}	{HUBG,EXPD,CHRW}	Transportation & Logistics	Industrials	NYSE American	\N	2020-10-11 23:12:10.406526	2020-10-11 23:38:31.02582
RMR	The RMR Group Inc.	The RMR Group Inc is a holding company. The Company provides management services to four publicly owned real estate investment trusts, and three real estate operating companies.	{"Real Estate","Real Estate Services"}	{OBAS,RMR}	Real Estate Services	Real Estate	NASDAQ Capital Market	\N	2020-10-11 23:12:10.431608	2020-10-11 23:38:31.024353
RNG	Ringcentral Inc. Class A	RingCentral Inc provides software-as-a-service, or SaaS, solutions for business communications. Its solutions can be used in multiple devices including Smartphones, Tablets, PC's and Desk Phones which allows for communication across multiple channels.	{Technology,"Software - Application","Application Software"}	{CSCO,MSFT,AMZN,AKAM,T,JCOM,EGHT,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:10.450765	2020-10-11 23:38:31.031328
ROCK	Gibraltar Industries Inc.	Gibraltar Industries Inc is a manufacturer and distributor of building products for industrial, transportation infrastructure, residential housing, renewable energy and resource conservation markets.	{"Basic Materials","Building Materials"}	{NWPX,GIFI,APOG,PATK,BECN,SSD,VMI,AWI,NCS,PGTI}	Building Materials	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:12:10.471757	2020-10-11 23:38:31.028166
ROM	ProShares Ultra Technology	The investment seeks daily investment results that correspond to two times (2x) the daily performance of the Dow Jones U.S. TechnologySM Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index measures the performance of certain companies in the technology sector of the U.S. equity market. Component companies include, among others, those involved in computers and office equipment, software, communications technology, semiconductors, diversified technology services and Internet services. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.48977	2020-10-11 23:38:31.023007
ROUS	Hartford Multifactor U.S. Equity	The investment seeks to provide investment results that, before fees and expenses, correspond to the total return performance of the Hartford Risk-Optimized Multifactor US Equity Index.\n The fund generally invests at least 80 percent of its assets in securities of the index and in depositary receipts representing securities of the index. The index seeks to improve returns through a market cycle relative to traditional cap-weighted U.S. equity market indices and active U.S. equity market strategies.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.505951	2020-10-11 23:38:31.031328
RPT	Ramco-Gershenson Properties Trust	Ramco-Gershenson Properties Trust is a fully integrated, self-administered, publicly-traded equity real estate investment trust. Its primary business is the ownership and management of multi-anchored shopping centers in the United States.	{"Real Estate","REIT - Retail",REITs}	{}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:10.524758	2020-10-11 23:38:31.028166
RST	Rosetta Stone Inc.	Rosetta Stone Inc develops, markets, and sells language learning solutions consisting of software, online services, and audio practice tools. It customers include individuals, corporations, educational institutions, armed forces, and government agencies.	{Technology,"Software - Application","Application Software"}	{AMZN,SCHL,CBS,LRN,GRPN,ODP,SPGI,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:10.552338	2020-10-11 23:38:31.023007
RTRX	Retrophin Inc.	Retrophin Inc is a pharmaceutical company based in the United States. Its drug candidates are targeted at serious rare diseases such as gallstones and bile acid synthesis disorders.	{Healthcare,"Drug Manufacturers - Major","Drug Manufacturers"}	{LCI,NVS,PFE,LGND,SRPT,ICPT,PTCT,XLRN,XLV}	Drug Manufacturers	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:10.572413	2020-10-11 23:38:31.023007
RVLT	Revolution Lighting Technologies Inc.	Revolution Lighting Technologies Inc designs, manufactures, markets and sells, commercial grade, LED replacement lamps, LED fixtures and LED-based signage, channel-letter and contour lighting products.	{Technology,"Electronic Components","Computer Hardware"}	{LYTS,AYI,EFOI,OESX,XLK}	Computer Hardware	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:10.590458	2020-10-11 23:38:31.023007
RGLS	Regulus Therapeutics Inc.	Regulus Therapeutics Inc is a biotechnology company that focuses on the development of microRNA therapies. MicroRNA is a noncoding ribonucleic acid that regulates most genes in the genome.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:10.292836	2020-10-11 23:38:31.025871
RILYL	B. Riley Financial Inc. 7.50% Senior Notes		{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:10.357177	2020-10-11 23:38:31.023007
RJN	AB Svensk Ekportkredit (Swedish Export Credit Corporation) ELEMENTS Linked to the Rogers International Commodity Index - Energy Total Return Structured Product	The investment seeks to replicate, net of expenses, the Rogers International Commodity Index – Energy Total Return index.\n The index represents the value of a basket of 6 energy commodity futures contracts.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.385053	2020-10-11 23:38:31.023007
RLJ	RLJ Lodging Trust of Beneficial Interest $0.01 par value	RLJ Lodging Trust is a United States-based self-advised and self-administered real estate investment trust (REIT). It acquires branded, focused-service and compact full-service hotels.	{"Real Estate","REIT - Hotel & Motel",REITs}	{AHT,DRH,PEB,CHSP,INN,SOHO,HPT,CDOR}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:10.4112	2020-10-11 23:38:31.023428
RNDB	Randolph Bancorp Inc.	Randolph Bancorp Inc is a bank holding company. The company through its holding provides financial services to individuals, families and small to mid-size businesses.	{"Financial Services","Banks - Regional - US",Banks}	{OCFC,NWBI,ORIT,NYCB}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:10.434513	2020-10-11 23:38:31.025871
RNGR	Ranger Energy Services Inc. Class A	Ranger Energy Services Inc is a provider of high-spec well service rigs and associated services in the United States, with a focus on technically demanding unconventional horizontal well completion and production operations.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{PUMP}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:10.453543	2020-10-11 23:38:31.024353
ROAM	Hartford Multifactor Emerging Markets	The investment seeks to provide investment results that, before fees and expenses, correspond to the total return performance of the Hartford Risk-Optimized Multifactor Emerging Markets Index.\n The fund generally invests at least 80% of its assets in securities of the index and in depositary receipts (such as American Depositary Receipts ("ADRs"), Global Depositary Receipts ("GDRs") and European Depositary Receipts ("EDRs") representing securities of the index. The index is designed to balance risks and opportunities within equity markets of emerging economies while emphasizing constituents exhibiting a favorable combination of factor characteristics.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.470728	2020-10-11 23:38:31.021426
ROKU	Roku Inc.	Roku Inc operates TV streaming platform in the United States. Its TV streaming platform allows users to discover and access a variety of movies and TV episodes, as well as live sports, music, news and more.	{"Communication Services","Pay TV"}	{MSN,ROKU}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:12:10.487464	2020-10-11 23:38:31.028166
RPM	RPM International Inc.	RPM International Inc manufactures, markets, and sells various specialty chemical product lines, including high-quality specialty paints, protective coatings, roofing systems, sealants and adhesives.	{"Basic Materials","Specialty Chemicals",Chemicals}	{GRA,FOE,FUL,MAS,PPG,SHW,HUN,CCF}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:10.516774	2020-10-11 23:38:31.024353
RRGB	Red Robin Gourmet Burgers Inc.	Red Robin Gourmet Burgers Inc develops and operates casual-dining and fast-casual restaurants. The company operates casual dining restaurant chain in North America. It generates its revenue from gift cards, francise royalties and fees and other income.	{"Consumer Cyclical",Restaurants}	{DAVE,ARKR,DENN,TXRH,BJRI,DRI,EAT,CHUY,SAUC}	Restaurants	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:10.537457	2020-10-11 23:38:31.031328
RTEC	Rudolph Technologies Inc.	Rudolph Technologies Inc designs, develops, manufactures and supports defect inspection,packaging lithography, thin film metrology and data analysis systems and software used by microelectronics device manufacturers.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{AMAT,ASYS,KLAC,NANO,NVMI,CAMT,TER,INTT,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:10.561286	2020-10-11 23:38:31.028166
RUTH	Ruth's Hospitality Group Inc.	Ruth's Hospitality Group Inc operates and franchises approximately 150 fine-dining restaurants under Ruth's Chris Steak House brand in US. The company also has several franchised restaurants in Canada, Mexico, Central America, Asia, and the Middle East.	{"Consumer Cyclical",Restaurants}	{ARKR,RRGB,TXRH,BJRI,DRI,DIN,DFRG,BLMN}	Restaurants	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:10.591423	2020-10-11 23:38:31.028166
RWT	Redwood Trust Inc.	Redwood Trust Inc focuses on residential mortgage loans and mortgage-backed securities. It operates through two segments, Residential Mortgage Banking and Investment Portfolio.	{"Real Estate","REIT - Diversified",REITs}	{DX,IVR,HASI,CHMI}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:10.624414	2020-10-11 23:38:31.025871
RXN	Rexnord Corporation	Rexnord Corp is a multi-platform industrial company comprised of two strategic platforms: Process & Motion Control and Water Management. It offers products to improve water quality, safety, flow control and conservation.	{Industrials,"Diversified Industrials","Industrial Products"}	{NNBR,AMOT,ROLL,AIMC,TKR,KAMN}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:10.64361	2020-10-11 23:38:31.02423
RYN	Rayonier Inc. REIT	Rayonier Inc is a timberland real estate investment trust which focuses on investing in timberlands. It is engaged in activities associated with timberland management and production and sale of specialty cellulose fibers.	{"Basic Materials","Lumber & Wood Production","Forest Products"}	{EMN,IP,WY,CLW}	Forest Products	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:10.664937	2020-10-11 23:38:31.024353
RJZ	AB Svensk Ekportkredit (Swedish Export Credit Corporation) ELEMENTS Linked to the Rogers International Commodity Index - Metals Total Return Structured Product	The investment seeks to replicate, net of expenses, the Rogers International Commodity Index – Metals Total Return index.\n The index represents the value of a basket of 10 metals commodity futures contracts.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.384842	2020-10-11 23:38:31.031328
RLY	SPDR SSgA Multi Asset Real Return	The investment seeks to achieve real return consisting of capital appreciation and current income.\n Under normal circumstances, the Adviser invests at least 80% of the net assets of the fund among ETPs that provide exposure to the following primary asset classes: (i) inflation protected securities issued by the United States government, its agencies and/or instrumentalities, as well as inflation protected securities issued by foreign governments, agencies, and/or instrumentalities; (ii) domestic and international real estate securities; (iii) commodities; and (iv) publicly-traded companies in natural resources and/or commodities businesses.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.411914	2020-10-11 23:38:31.02423
RMTI	Rockwell Medical Inc.	Rockwell Medical Inc is a biopharmaceutical company targeting end-stage renal disease (ESRD) and chronic kidney disease with its products and services for the treatment of iron deficiency, secondary hyperparathyroidism, and hemodialysis.	{Healthcare,"Drug Manufacturers - Major","Drug Manufacturers"}	{ICUI,MLAB,NXTM,AMAG,FMS,ABT,BAX,CMD,XLV}	Drug Manufacturers	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:10.434364	2020-10-11 23:38:31.021426
RNP	Cohen & Steers Reit and Preferred Income Fund Inc	Cohen & Steers REIT & Preferred Income is a closed-ended investment management fund. Its investment objective is to provide high current income through investment in real estate and diversified preferred securities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:10.454805	2020-10-11 23:38:31.025871
RNWK	RealNetworks Inc.	RealNetworks Inc is a provider of digital media services and software. It creates applications and services which connect with and enjoy digital media. Its segments are Consumer media, Mobile services and Games.	{Technology,"Electronic Gaming & Multimedia","Application Software"}	{MSFT,ADBE,AAPL,DLB,VIAB,GOOGL,TIVO,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:10.471108	2020-10-11 23:38:31.023007
ROL	Rollins Inc.	Rollins Inc offers pest and termite control services to residential and commercial customers in North America with international franchises in Central America and others. The company operates in one business segment that is Pest and Termite Control.	{Industrials,"Business Services"}	{CTAS,HCSG,ABM,ECL,UNF,AMRC,SPRT}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:10.489491	2020-10-11 23:38:31.02423
RP	RealPage Inc.	RealPage Inc is a provider of demand software and software-enabled services for the rental housing and vacation rental industries. Its solutions include marketing, pricing, leasing, accounting, purchasing, and other property operation capabilities.	{Technology,"Software - Application","Application Software"}	{CSGP,FISV,IBM,CLGX,ZG,GOOG,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:10.510205	2020-10-11 23:38:31.025871
RRR	Red Rock Resorts Inc.	Red Rock Resorts Inc along with its subsidiary is a gaming, development and management company. The company develops strategically-located casino and entertainment properties. It generates a majority of its revenue from Casino.	{"Consumer Cyclical","Resorts & Casinos","Travel & Leisure"}	{RRR}	Travel & Leisure	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:10.539387	2020-10-11 23:38:31.028166
RTH	VanEck Vectors Retail	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the MVIS® US Listed Retail 25 Index.\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The index includes common stocks and depositary receipts of U.S. exchange-listed companies that derive at least 50% of their revenues from (or, in certain circumstances, have at least 50% of their assets related to) retail. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.562481	2020-10-11 23:38:31.031328
RUSL	Direxion Daily Russia Bull 3x Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the daily performance of the MVIS Russia Index.\n The fund invests at least 80% of its net assets in securities of the index, ETFs that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index is a rules-based index, intended to represent the overall performance of publically traded companies that are domiciled and primarily listed on an exchange in Russia or that are not Russian companies, but nonetheless generate at least 50% of their revenues in Russia. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.582324	2020-10-11 23:38:31.021426
RVP	Retractable Technologies Inc.	Retractable Technologies Inc designs, develops, manufactures, and markets safety syringes and other safety medical products for the healthcare profession. It offers 1mL tuberculin; allergy antigen VanishPoint syringes.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{ICUI,PODD,PDEX,BDX,ATRS,TNDM,XLV}	Medical Instruments & Equipment	Healthcare	NYSE American	\N	2020-10-11 23:12:10.599254	2020-10-11 23:38:31.028166
RWM	ProShares Short Russell2000	The investment seeks daily investment results that correspond to the inverse (-1x) of the daily performance of the Russell 2000® Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. It is a float-adjusted, market capitalization-weighted index containing approximately 2000 of the smallest companies in the Russell 3000® Index or approximately 8% of the total market capitalization of the Russell 3000® Index, which in turn represents approximately 98% of the investable U.S. equity market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.621287	2020-10-11 23:38:31.024353
RSG	Republic Services Inc.	Republic Services Inc is the provider of solid waste management services in the United States. In addition, the company operates landfill gas-to-energy projects, as well as an exploration and production waste business.	{Industrials,"Waste Management"}	{SRCL,CWST,CLH,WM,PESI,WCN}	Waste Management	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:10.548194	2020-10-11 23:38:31.021426
RTM	Invesco S&P 500 Equal Weight Materials	The investment seeks to track the investment results (before fees and expenses) of the S&P 500® Equal Weight Materials Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities that comprise the underlying index. The underlying index is an equal-weighted version of the S&P 500® Materials Index. Strictly in accordance with its guidelines and mandated procedures, the index provider compiles, maintains and calculates the underlying index, which is comprised of common stocks of companies in the materials sector of the S&P 500® Index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.568333	2020-10-11 23:38:31.021426
RUSS	Direxion Daily Russia Bear 3x Shares	The investment seeks daily investment results before fees and expenses of 300% of the inverse of the daily performance of the MVIS Russia Index.\n The fund invests in swap agreements, futures contracts, short positions or other financial instruments that, in combination, provide inverse or short leveraged exposure to the index equal to at least 80% of its net assets. The index is a rules-based index, intended to represent the overall performance of publically traded companies that are domiciled and primarily listed on an exchange in Russia or that are not Russian companies, but nonetheless generate at least 50% of their revenues in Russia. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.587269	2020-10-11 23:38:31.024353
RWK	Oppenheimer S&P MidCap 400 Revenue	The investment seeks to provide investment results that correspond generally, before fees and expenses, to the performance of the S&P MidCap 400 Revenue-Weighted Index (the "underlying index").\n The fund will normally invest at least 80% of its net assets, plus any borrowings for investment purposes, in the securities of mid capitalization companies included in the underlying index. The underlying index is constructed using a rules-based methodology that re-weights the constituent securities of the S&P MidCap 400® Index (the "parent index") according to the revenue earned by the companies in the parent index, subject to a maximum 5% per company weighting.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.620937	2020-10-11 23:38:31.024353
RYAAY	Ryanair Holdings plc American Depositary Shares each representing five Ordinary Shares	Ryanair Holdings PLC is an airline operator. The company mainly offers short-haul airline transport services within Europe. It manages transports through a single route-scheduling with a system of its own.	{Industrials,Airlines}	{CEA,ZNH,GE,MA,GOL,UAL}	Airlines	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:10.650145	2020-10-11 23:38:31.031328
RZV	Invesco S&P Smallcap 600 Pure Value	The investment seeks to track the investment results (before fees and expenses) of the S&P SmallCap 600® Pure Value Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities that comprise the underlying index. Strictly in accordance with its guidelines and mandated procedures, the index provider, compiles, maintains and calculates the underlying index, which is comprised of those S&P SmallCap 600® companies with strong value characteristics selected by the index provider. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.681881	2020-10-11 23:38:31.023428
SAFM	Sanderson Farms Inc.	Sanderson Farms Inc is engaged in dressed chicken processing as well as producing and selling a variety of fresh, frozen, and value-added chicken products to grocers, food-service operators, and distributors.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{CALM,FDP,SYY,TSN,IBA,PPC}	Consumer Packaged Goods	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:10.704666	2020-10-11 23:38:31.02582
SAIL	SailPoint Technologies Holdings Inc.	SailPoint Technologies Holdings Inc provides enterprise identity governance solutions. Its open identity platform empowers users and governs their access to applications and data across complex, hybrid IT environments.	{Technology,"Software - Infrastructure","Application Software"}	{XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:10.71955	2020-10-11 23:38:31.025871
SANW	S&W Seed Company	S&W Seed Co is engaged in breeding, growing, processing and selling of alfalfa seed. It also offers seed cleaning and processing for other seed manufacturers. Its products include Alfalfa seed varieties and Stevia.	{"Consumer Defensive","Farm Products","Consumer Packaged Goods"}	{ADM,INGR}	Consumer Packaged Goods	Consumer Defensive	NASDAQ Capital Market	\N	2020-10-11 23:12:10.740161	2020-10-11 23:38:31.02423
SB	Safe Bulkers Inc ($0.001 par value)	Safe Bulkers Inc rents its fleet of dry-bulk vessels on spot and time charters. The company through a private corporation manages and operates its vessels.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{DSX,NM,NMM,ESEA,SALT,SHIP,SBLK,DRYS}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:10.753456	2020-10-11 23:38:31.028166
SBLKZ	Star Bulk Carriers Corp. 8.30% Senior Notes due 2022		{}	{}			Nasdaq Global Select	\N	2020-10-11 23:12:10.782193	2020-10-11 23:38:31.028166
SCA	Stellus Capital Investment Corporation 5.75% Notes due 2022		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:10.818528	2020-10-11 23:38:31.024353
SCHA	Schwab U.S. Small-Cap	The investment seeks to track as closely as possible, before fees and expenses, the total return of the Dow Jones U.S. Small-Cap Total Stock Market Index.\n The fund will invest at least 90% of its net assets (net assets plus borrowings for investment purposes) in these stocks that are included in the index. The index includes the small-cap portion of the Dow Jones U.S. Total Stock Market Index actually available to investors in the marketplace. The Dow Jones U.S. Small-Cap Total Stock Market Index includes the components ranked 751-2500 by full market capitalization. The index is a float-adjusted market capitalization weighted index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.846566	2020-10-11 23:38:31.024353
RSXJ	VanEck Vectors Russia Small-Cap	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the MVIS® Russia Small-Cap Index.\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The index includes securities of Russian small- and medium-capitalization companies. It will normally invest at least 80% of its total assets in securities of small-capitalization Russian companies. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.555599	2020-10-11 23:38:31.023428
RTN	Raytheon Company	Raytheon Co offers contractual services to the US defense sector. It operates through five segments: Integrated Defense Systems, Intelligence and Information, Missile Systems, Space and Airborne Systems, and Forcepoint.	{Industrials,"Aerospace & Defense"}	{FLIR,LMT,BA,GD,HON,NOC,COL,ITT,LLL}	Aerospace & Defense	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:10.569721	2020-10-11 23:38:31.025871
RVEN	Reven Housing REIT Inc.	Reven Housing REIT Inc is engaged in the acquisition, ownership and operation of portfolios of leased single family homes in the United States. It owns and operates single-family houses in Houston, Jacksonville, Memphis and Atlanta metropolitan areas.	{"Real Estate","REIT - Residential",REITs}	{NYMT,O,NTP}	REITs	Real Estate	NASDAQ Capital Market	\N	2020-10-11 23:12:10.589475	2020-10-11 23:38:31.025871
RWX	SPDR DJ Wilshire Intl Real Estate	The investment seeks to provide investment results, before fees and expenses, correspond generally to the total return performance of the Dow Jones Global ex-U.S. Select Real Estate Securities Indexsm.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index and in depositary receipts based on securities comprising the index. The index is a float-adjusted market capitalization index designed to measure the performance of publicly traded real estate securities in countries excluding the United States. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.627188	2020-10-11 23:38:31.028166
RYE	Invesco S&P 500 Equal Weight Energy	The investment seeks to track the investment results (before fees and expenses) of the S&P 500® Equal Weight Energy Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities that comprise the underlying index. The underlying index is an equal-weighted version of the S&P 500® Energy Index. Strictly in accordance with its guidelines and mandated procedures, the index provider compiles, maintains and calculates the underlying index, which is comprised of common stocks of companies in the energy sector of the S&P 500® Index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.652905	2020-10-11 23:38:31.025871
RYTM	Rhythm Pharmaceuticals Inc.	Rhythm Pharmaceuticals Inc is a biopharmaceutical company focused on the development and commercialization of peptide therapeutics for the treatment of gastrointestinal, or GI, diseases, and genetic deficiencies that result in metabolic disorders.	{Healthcare,Biotechnology}	{PDLI,PIRS,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:10.670334	2020-10-11 23:38:31.031328
SABR	Sabre Corporation	Sabre Corp is a travel technology company. It primarily engages in the air global distribution system market. The company also has an IT solutions division that focuses on the airline and travel agent end markets.	{Technology,"Information Technology Services","Application Software"}	{BA,DAL,ACN,EXPE,GOOGL,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:10.695319	2020-10-11 23:38:31.024353
SAH	Sonic Automotive Inc.	Sonic Automotive Inc is an auto dealership group based in the US with more than 100 stores spread across the country. The company deals in the addition of new- and used-vehicle. It provides parts and collision repair, and wholesale auctions services.	{"Consumer Cyclical","Auto & Truck Dealerships",Autos}	{}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:10.709799	2020-10-11 23:38:31.024353
SAMG	Silvercrest Asset Management Group Inc.	Silvercrest Asset Management Group Inc provides financial advisory services to clients in the United States. Its area of coverage includes equities, fixed income and cash, as well as non-traditional investment strategies, such as hedge funds.	{"Financial Services","Asset Management"}	{SF,LAZ,AMP,EVR}	Asset Management	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:10.727204	2020-10-11 23:38:31.021426
SAP	SAP SE ADS	SAP SE is a global software vendor, providing solutions for enterprise resource planning, database management, business intelligence, and vertical-specific software.	{Technology,"Software - Application","Application Software"}	{MSFT,INTU,ADBE,EA,SYMC,IBM,CRM,VMW,ORCL,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:10.740306	2020-10-11 23:38:31.025871
SBAC	SBA Communications Corporation	SBA Communications Corp is an independent owner and operator of wireless communications towers in the United States, Brazil, Puerto Rico, Canada, and Central America.	{"Communication Services","Telecom Services"}	{}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:12:10.752811	2020-10-11 23:38:31.024353
SBFG	SB Financial Group Inc.	SB Financial Group Inc is a bank holding company engaged in commercial banking, item processing & wealth management services. It offers services, including checking & savings accounts, money market accounts & time certificates of deposit, among others.	{"Financial Services","Banks - Regional - US",Banks}	{PEBO,UBCP,OVBC,FFBC,HBAN,PRK,FMNB,CIVB}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:10.766434	2020-10-11 23:38:31.028166
RTTR	Ritter Pharmaceuticals Inc.	Ritter Pharmaceuticals Inc develops novel therapeutic products that modulate the gut microbiome to treat gastrointestinal diseases.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:10.574598	2020-10-11 23:38:31.02423
RVRS	Reverse Cap Weighted US Large Cap	The investment seeks to track the performance, before fees and expenses, of the Reverse Cap Weighted U.S. Large Cap Index (the "index").\n Under normal circumstances, at least 80% of the fund's total assets (exclusive of any collateral held from securities lending) will be invested in the component securities of the index. The index is a rules-based, reverse capitalization weighted index comprised of the constituents of the S&P 500® Index, which consists of approximately 500 leading U.S.-listed companies representing approximately 80% of the U.S. equity market capitalization.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:10.598584	2020-10-11 23:38:31.02582
RXI	iShares Global Consumer Discretionary	The investment seeks to track the S&P Global 1200 Consumer Discretionary IndexTM.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index. The index measures the performance of companies that the index provider deems to be part of the consumer discretionary sector of the economy and that the index provider believes are important to global markets.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.638536	2020-10-11 23:38:31.023007
RYT	Invesco S&P 500 Equal Weight Technology	The investment seeks to track the investment results (before fees and expenses) of the S&P 500® Equal Weight Information Technology Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities that comprise the underlying index. The underlying index is an equal-weighted version of the S&P 500® Information Technology Index. Strictly in accordance with its guidelines and mandated procedures, the index provider compiles, maintains and calculates the underlying index, which is comprised of common stocks of companies in the information technology sector of the S&P 500® Index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.66687	2020-10-11 23:38:31.02423
SACH	Sachem Capital Corp.	Sachem Capital Corp specializes in originating, underwriting, funding, servicing and managing a portfolio of short-term (three years or less) loans secured by first mortgage liens on real property located primarily in Connecticut.	{"Real Estate","REIT - Residential",REITs}	{NYMT,O,NTP}	REITs	Real Estate	NYSE American	\N	2020-10-11 23:12:10.696266	2020-10-11 23:38:31.028166
SAIC	SCIENCE APPLICATIONS INTERNATIONAL CORPORATION	Science Applications International Corp is a provider of technical, engineering and enterprise information technology services to the U.S. government, including the Department of Defense and federal civilian agencies.	{Technology,"Information Technology Services","Application Software"}	{CTSH,GWRE,SPLK,BNFT,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:10.716721	2020-10-11 23:38:31.031328
SAN	Banco Santander S.A. Sponsored ADR (Spain)	Banco Santander SA is a global bank engaged in retail banking. The firm also offers other services such as wholesale banking, insurance and asset management. Its business operations are spread across Latin America, Continental Europe, and the U.S.	{"Financial Services","Banks - Global",Banks}	{BCS,BAC,DB,HSBC,CIB,BBVA,BSAC,ITUB}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:10.731801	2020-10-11 23:38:31.031328
SATS	EchoStar Corporation	EchoStar Corp provides satellite operations, video delivery solutions, digital set-top boxes, & broadband satellite technologies & services for home & office, delivering network technologies, managed services, solutions for enterprises and governments.	{Technology,"Communication Equipment"}	{GILT,VSAT,LORL,DISH,HRS,GSAT,ARRS,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:10.745955	2020-10-11 23:38:31.023007
SBFGP	SB Financial Group Inc. Depositary Shares each representing a 1/100th interest in a 6.50% Noncumulative convertible perpetual preferred share Series A	SB Financial Group Inc is a bank holding company engaged in commercial banking, item processing & wealth management services. It offers services, including checking & savings accounts, money market accounts & time certificates of deposit, among others.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:10.76918	2020-10-11 23:38:31.02582
SBNA	Scorpio Tankers Inc. 6.75% Senior Notes due 2020		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:10.794957	2020-10-11 23:38:31.02582
SC	Santander Consumer USA Holdings Inc.	Santander Consumer USA Holdings Inc is a consumer finance company. Its core business is the indirect origination of retail installment contracts, through manufacturer-franchised dealers with their sale of new and used vehicles to retail consumer.	{"Financial Services","Specialty Finance",Banks}	{NICK,CPSS,CACC}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:10.818412	2020-10-11 23:38:31.025871
SNX	Synnex Corporation	Synnex Corp is a business process services company, providing a comprehensive range of distribution, logistics and integration services. The company operates in two segments: Technology Solutions and Concentrix.	{Industrials,"Business Services"}	{SCSC,NSIT,TECD,BSQR,ARW,IBM,XRX,AVT}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:11.514493	2020-10-11 23:38:31.023428
RY	Royal Bank Of Canada	Royal Bank of Canada is one of the largest Canadian banks. It provides diversified financial services, personal and commercial banking, wealth management services, insurance, corporate and investment banking, and transaction processing services worldwide.	{"Financial Services","Banks - Global",Banks}	{BMO,BNS,CM,TD,BAC}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:10.643654	2020-10-11 23:38:31.024353
RYJ	Invesco Raymond James SB-1 Equity	The investment seeks investment results that correspond generally to the performance, before the fund's fees and expenses, of an equity index called the Raymond James SB-1 Equity Index.\n The fund will invest at least 90% of its total assets in securities that comprise the index and depositary receipts representing securities that comprise the index. Index constituents include equity securities of all market capitalizations, as defined by the index provider, that trade on a U.S. securities exchange, including common stocks, ADRs, REITs, master limited partnerships and business development companies. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.66164	2020-10-11 23:38:31.023007
SAA	ProShares Ultra SmallCap600	The investment seeks daily investment results that correspond to two times (2x) the daily performance of the S&P SmallCap 600®.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. It is a float-adjusted, market capitalization-weighted index of 600 U.S. operating companies selected through a process that factors in criteria such as liquidity, price, market capitalization, financial viability and public float. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.694933	2020-10-11 23:38:31.023007
SAGE	Sage Therapeutics Inc.	Sage Therapeutics Inc is a biopharmaceutical company conducting development programs and clinical trials to discover medicines for central nervous system disorders. It has development programs for refractory status epilepticus, and Parkinson's di-sease.	{Healthcare,Biotechnology}	{ICPT,HZNP,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:10.709587	2020-10-11 23:38:31.023428
SAM	Boston Beer Company Inc. (The)	Boston Beer Co Inc is engaged in the business of selling low alcohol beverages in the United States. The company also markets Twisted Tea (a flavored malt beverage) and Angry Orchard ciders.	{"Consumer Defensive","Beverages - Brewers","Beverages - Alcoholic"}	{DEO,BREW,CCU,STZ,TAP,BUD}	Beverages - Alcoholic	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:10.725302	2020-10-11 23:38:31.023007
SANM	Sanmina Corporation	Sanmina Corp is engaged in the business of Integrated Manufacturing Solutions and Components, Products, and Services, which includes interconnect systems and mechanical systems.	{Technology,"Electronic Components","Computer Hardware"}	{CLS,FLEX,KBAL,PLXS,TTMI,SMTX,JBL,BHE,CTS,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:10.739236	2020-10-11 23:38:31.024353
SAUC	Diversified Restaurant Holdings Inc.	Diversified Restaurant Holdings Inc is a restaurant company and also acts as a franchisee for Buffalo Wild Wings (BWW). It operates more than 60 BWW restaurants located in key urban and suburban markets in Florida, Illinois, Indiana and Missouri.	{"Consumer Cyclical",Restaurants}	{DAVE,ARKR,RRGB,DRI,DIN,BDL,CHUY,BLMN}	Restaurants	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:10.753176	2020-10-11 23:38:31.02582
SBGI	Sinclair Broadcast Group Inc.	Sinclair Broadcast Group Inc is a diversified television broadcasting company that owns and operates, programs or provides sales services to television stations. The company owns a multicast network, four radio stations and a cable network.	{"Consumer Cyclical","Broadcasting - TV",Entertainment}	{NXST,CMCSA,DIS,EVC,GTN,CBS,FOXA,SSP}	Entertainment	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:10.770651	2020-10-11 23:38:31.023007
SBRA	Sabra Health Care REIT Inc.	Sabra Health Care REIT Inc operates as a self-administered and self-managed REIT. The company, through its subsidiaries, acquires, finances and owns real estate property to be leased to third party tenants in the healthcare sector.	{"Real Estate","REIT - Healthcare Facilities",REITs}	{DRE,VTR,UHT,HCP,CHSP,INN,TIPT,HTA,WELL}	REITs	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:10.806434	2020-10-11 23:38:31.028166
SCG	SCANA Corporation	SCANA Corp is a holding company engaged in the generation and sale of electricity and in the purchase, sale and transportation of natural gas. It also conducts other energy-related businesses and provides fiber optic communications in South Carolina.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{NI,AEE,SO,D,NEE,DUK,WEC,XEL,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:10.834717	2020-10-11 23:38:31.025871
SCHG	Schwab U.S. Large-Cap Growth	The investment seeks to track as closely as possible, before fees and expenses, the total return of the Dow Jones U.S. Large-Cap Growth Total Stock Market Index.\n To pursue its goal, the fund generally invests in stocks that are included in the Dow Jones U.S. Large-Cap Growth Total Stock Market Index. The index includes the large-cap growth portion of the Dow Jones U.S. Total Stock Market Index actually available to investors in the marketplace. The Dow Jones U.S. Large-Cap Growth Total Stock Market Index includes the components ranked 1-750 by full market capitalization and that are classified as "growth" based on a number of factors.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.862096	2020-10-11 23:38:31.024353
RYB	RYB Education Inc. American depositary shares each representing one Class A	RYB Education Inc is a early childhood education service provider in China. It offers kindergarten services, play-and-learn center services as well as at-home education products and services.	{"Consumer Defensive","Education & Training Services",Education}	{LRN,HMHC,GHC}	Education	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:10.653151	2020-10-11 23:38:31.021426
RZB	Reinsurance Group of America Incorporated 5.75% Fixed-To-Floating Rate Subordinated Debentures due 2056		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:10.671934	2020-10-11 23:38:31.02582
SAEX	SAExploration Holdings Inc.	SAExploration Holdings Inc is an oilfield services company offering a vertically-integrated seismic data acquisition and logistical support services in Alaska, Canada, South America, West Africa and Southeast Asia to oil and natural gas industry.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{}	Oil & Gas - Services	Energy	NASDAQ Capital Market	\N	2020-10-11 23:12:10.696633	2020-10-11 23:38:31.021426
SALT	Scorpio Bulkers Inc.	Scorpio Bulkers Inc is a shipping company. Its core activity is marine transportation of goods such as include iron ore, coal and grain and minor bulks which include bauxite, fertilizers and steel products.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{DSX,NMM,SB,SBLK,GOGL}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:10.724451	2020-10-11 23:38:31.02582
SAVE	Spirit Airlines Inc.	Spirit Airlines Inc is an airline company. The company provides air transportation for passengers. It's all all-Airbus fleet operates more than 385 daily flights to over 56 destinations in the United States, Caribbean and Latin America.	{Industrials,Airlines}	{SKYW,JBLU,HA,ALGT,ALK,LUV,DAL,AAL,UAL}	Airlines	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:10.753386	2020-10-11 23:38:31.021426
SBNYW	Signature Bank Warrants 12/12/2018	Signature Bank offers a wide variety of business and personal banking products and services. It also provides brokerage, asset management and insurance products and services.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:10.794309	2020-10-11 23:38:31.021426
SBT	Sterling Bancorp Inc.	Sterling Bancorp Inc provides banking solutions, innovative loan products and outstanding personal service to the residential and commercial markets, as well as retail banking services.	{"Financial Services","Banks - Regional - US",Banks}	{OCFC,NWBI,ORIT,NYCB}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:10.814263	2020-10-11 23:38:31.02423
SCD	LMP Capital and Income Fund Inc.	Lmp Capital & Income Fund Inc is a United States based non-diversified, closed-end management investment company. Its investment objective is total return with an emphasis on the income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:10.835417	2020-10-11 23:38:31.023007
SCHK	Schwab 1000 Index	The investment seeks to track as closely as possible, before fees and expenses, the total return of the Schwab 1000 Index®.\n It is the fund's policy that under normal circumstances it will invest at least 90% of its net assets (net assets plus borrowings for investment purposes) in stocks included in the index. The Schwab 1000 Index is a float-adjusted market capitalization weighted index that includes the 1,000 largest stocks of publicly traded companies in the United States, with size being determined by market capitalization (total market value of all shares outstanding).	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.858468	2020-10-11 23:38:31.031328
SCIN	Columbia India Small Cap	Scottish Investment Trust (The) PLC is an investment trust. Its objective is to provide investors with above-average returns over the long term through a diversified portfolio of international equities and to achieve dividend growth ahead of UK inflation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE Arca	\N	2020-10-11 23:12:10.890367	2020-10-11 23:38:31.028166
SCS	Steelcase Inc.	Steelcase Inc is an international furniture company based in the United States. The firm markets furniture for office, healthcare, and education industry under brands such as Steelcase, Coalesse, Designtex, PolyVision, and Turnstone.	{Industrials,"Business Equipment","Industrial Products"}	{MLHR,KEQU,KBAL,HNI,KNL}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:10.908196	2020-10-11 23:38:31.031328
SCYX	SCYNEXIS Inc.	SCYNEXIS Inc is a pharmaceutical company engaged in the discovery, development, and commercialization of novel anti-infectives to address unmet therapeutic needs.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{PTLA,ARCT,GLMD,KMPH,XLV}	Drug Manufacturers	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:10.927992	2020-10-11 23:38:31.02423
SDOG	ALPS Sector Dividend Dogs	The investment seeks investment results that replicate as closely as possible, before fees and expenses, the performance of the S-Network® Sector Dividend Dogs Index.\n The underlying index generally consists of 50 stocks on each annual reconstitution date, which is the third Friday of December each year. The underlying index's stocks must be constituents of the S&P 500 Index, the leading benchmark index for U.S. large capitalization stocks. The underlying index methodology selects the five stocks in each of the ten GICS sectors that make up the S&P 500 which offer the highest dividend yields as of the last business day of November.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.943291	2020-10-11 23:38:31.023007
SEA	Invesco Shipping	The investment seeks to track investment results (before fees and expenses) of the Dow Jones Global Shipping IndexSM (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities that comprise the underlying index, as well as American depositary receipts ("ADRs") and global depositary receipts ("GDRs") that represent securities in the underlying index. The underlying index is comprised of developed market-listed equity securities of companies that are classified as being in the shipping industry. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.965502	2020-10-11 23:38:31.02582
RYH	Invesco S&P 500 Equal Weight Health Care	The investment seeks to track the investment results (before fees and expenses) of the S&P 500® Equal Weight Health Care Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities that comprise the underlying index. The underlying index is an equal-weighted version of the S&P 500® Health Care Index. Strictly in accordance with its guidelines and mandated procedures, the index provider compiles, maintains and calculates the underlying index, which is comprised of common stocks of companies in the health care sector of the S&P 500® Index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.65329	2020-10-11 23:38:31.023428
SAGG	Direxion Daily Total Bond Market Bear 1X Shares	The investment seeks daily investment results, before fees and expenses, of 100% of the inverse (or opposite) of the daily performance of the Bloomberg Barclays Capital US Aggregate Bond Index.\n The fund, under normal circumstances, invests in swap agreements, futures contracts, short positions or other financial instruments that, in combination, provide inverse (opposite) or short exposure to the index equal to at least 80% of the fund's net assets (plus borrowing for investment purposes). The index measures the performance of the investment grade, U.S. Dollar denominated, fixed-rate taxable bond market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.709201	2020-10-11 23:38:31.025871
SAND	Sandstorm Gold Ltd. (Canada)	Sandstorm Gold Ltd provides financing to companies engaged in gold mining through gold stream and royalty. The company has various mines that are generating revenues located in Canada, Mexico, Brazil and United States.	{"Basic Materials",Gold,"Metals & Mining"}	{TMQ,CMCL}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:10.738736	2020-10-11 23:38:31.028166
SBI	Western Asset Intermediate Muni Fund Inc	Western Asset Intermediate Muni Fd Inc is a closed-end management investment company. The company's objective is to provide a high level of current income exempt from Federal income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:10.775455	2020-10-11 23:38:31.025871
SBSI	Southside Bancshares Inc.	Southside Bancshares Inc is a financial institution offering financial services to individuals, businesses, municipal entities, and non-profit organizations in the communities it serves.	{"Financial Services","Banks - Regional - US",Banks}	{FFIN,IBOC,APOG,BOKF,TCBI,CFR,PB,IBTX,LTXB}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:10.80974	2020-10-11 23:38:31.023007
SCC	ProShares UltraShort Consumer Services	The investment seeks daily investment results, before fees and expenses, that correspond to two times the inverse (-2x) of the daily performance of the Dow Jones U.S. Consumer ServicesSM Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index seeks to measure the stock performance of certain U.S. companies in the consumer services sector of the U.S. equity market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.829104	2020-10-11 23:38:31.031328
SCHF	Schwab International Equity	The investment seeks to track as closely as possible, before fees and expenses, the total return of the FTSE Developed ex US Index.\n The fund will invest at least 90% of its net assets in stocks, including depositary receipts representing securities of the index; such depository receipts may be in the form of American Depositary receipts, Global Depositary receipts and European Depositary receipts. The index is comprised of large and mid capitalization companies in developed countries outside the United States, as defined by the index provider. The index defines the large and mid capitalization universe as approximately the top 90% of the eligible universe.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.855505	2020-10-11 23:38:31.02423
SCHP	Schwab U.S. TIPs	The investment seeks to track as closely as possible, before fees and expenses, the total return of the Bloomberg Barclays US Treasury Inflation-Linked Bond Index (Series-L).\n The fund will invest at least 90% of its net assets in securities included in the index. The index includes all publicly-issued U.S. Treasury Inflation-Protected Securities (TIPS) that have at least one year remaining to maturity, are rated investment grade and have $500 million or more of outstanding face value. The TIPS in the index must be denominated in U.S. dollars and must be fixed-rate and non-convertible.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.872271	2020-10-11 23:38:31.021426
SCIU	Global X Scientific Beta US	The investment seeks investment results that correspond generally to the price and yield performance, before fees and expenses, of the Scientific Beta United States Multi-Beta Multi-Strategy Four-Factor Equal Risk Contribution (ERC) Index.\n The fund invests at least 80% of its total assets in the securities of the underlying index. The underlying index generally comprises approximately 500 or less U.S. listed common stocks selected based on a proprietary methodology developed by EDHEC Risk Institute Asia Ltd.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.890547	2020-10-11 23:38:31.023428
SCSC	ScanSource Inc.	ScanSource Inc is a wholesale distributor of specialty technology products. The Company provides value-added distribution services for technology manufacturers and sells to resellers in technology markets.	{Technology,"Computer Distribution","Computer Hardware"}	{NSIT,TECD,PLUS,SYX,SNX,ESND,AVT,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:10.908342	2020-10-11 23:38:31.023007
SD	SandRidge Energy Inc.	SandRidge Energy Inc is an oil and natural gas company. Its business is comprised of exploration and development of oil properties.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{PBFX,PARR}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:10.929115	2020-10-11 23:38:31.031328
RYI	Ryerson Holding Corporation	Ryerson Holding Corp is in processing and distribution of metals. It is involved in coil and sheet processing, plate processing, bar, tube and structural processing, supply chain services and fabrication services.	{Industrials,"Metal Fabrication","Industrial Products"}	{AKS,RS,ALJJ}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:10.658073	2020-10-11 23:38:31.028166
S	Sprint Corporation	Sprint Corp is a communications company offering wireless and wireline communications products and services. It offers products and services to individual consumers, businesses, government subscribers and resellers.	{"Communication Services","Telecom Services"}	{TDS,TU,FTR}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:10.685395	2020-10-11 23:38:31.025871
SAIA	Saia Inc.	Saia Inc is a transportation company. The company through its wholly-owned subsidiaries, provides less-than-truckload, non-asset truckload, expedited and logistics services across the United States.	{Industrials,Trucking,"Transportation & Logistics"}	{FWRD,ODFL,ULH,RRTS,YRCW,ARCB}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:10.716275	2020-10-11 23:38:31.023007
SBCF	Seacoast Banking Corporation of Florida	Seacoast Banking Corp provides banking services through the Seacoast National Bank. Its suite of services includes deposit accounts and retail banking, commercial lending and asset management.	{"Financial Services","Banks - Regional - US",Banks}	{CCBG,FCNCA,CSFL,BKU,BOTJ,OPHC}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:10.764296	2020-10-11 23:38:31.023428
SBNY	Signature Bank	Signature Bank offers a wide variety of business and personal banking products and services. It also provides brokerage, asset management and insurance products and services.	{"Financial Services","Banks - Regional - US",Banks}	{BDGE,AROW,NBTB,UVSP,TMP,MTB,EVBN,CHMG}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:10.793224	2020-10-11 23:38:31.024353
SBS	Companhia de saneamento Basico Do Estado De Sao Paulo - Sabesp American Depositary Shares (Each repstg 250)	Companhia De Saneamento Basico Do Estado De Sao Paulo is a Brazilian water and waste management company. It provides water and sewage services to residential, commercial, industrial and governmental customers.	{Utilities,"Utilities - Regulated Water","Utilities - Regulated"}	{CWCO,AWK,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:10.808455	2020-10-11 23:38:31.024353
SCACU	Saban Capital Acquisition Corp. Unit	Saban Capital Acquisition Corp is a blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:10.823464	2020-10-11 23:38:31.023428
SCHB	Schwab U.S. Broad Market	The investment seeks to track as closely as possible, before fees and expenses, the total return of the Dow Jones U.S. Broad Stock Market Index.\n The fund will invest at least 90% of its net assets (net assets plus borrowings for investment purposes) in these stocks. It generally invests in stocks that are included in the Dow Jones U.S. Broad Stock Market Index. The index includes the largest 2,500 publicly traded U.S. companies for which pricing information is readily available. The index is a float-adjusted market capitalization weighted index that reflects the shares of securities actually available to investors in the marketplace.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.844754	2020-10-11 23:38:31.028166
SCHO	Schwab Short-Term U.S. Treasury	The investment seeks to track as closely as possible, before fees and expenses, the total return of the Bloomberg Barclays US Treasury 1-3 Year Index.\n The fund will invest at least 90% of its net assets (including, for this purpose, any borrowings for investment purposes) in securities included in the index. The index includes all publicly-issued U.S. Treasury securities that have a remaining maturity of greater than or equal to one year and less than three years, are rated investment grade, and have $300 million or more of outstanding face value. The securities in the index must be denominated in U.S. dollars and must be fixed-rate and non-convertible.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.867481	2020-10-11 23:38:31.028166
SCHW	Charles Schwab Corporation (The)	Charles Schwab Corp through its subsidiaries provides retail brokerage and banking, retirement plan, and other corporate brokerage services. The company's operating business segments are Investor Services and Advisor Services.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{SEIC,AMTD,ETFC,MS,BAC,RJF,SF,LPLA,IBKR}	Brokers & Exchanges	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:10.88129	2020-10-11 23:38:31.023007
SCKT	Socket Mobile Inc.	Socket Mobile Inc is a producer of data capture products. The company's products are integrated into mobile applications used in mobile point of sale (mPOS), enterprise mobility, asset tracking and control systems.	{Technology,"Computer Systems","Computer Hardware"}	{TACT,ALOT,DGII,ZBRA,USAT,HON,HPQ,XLK}	Computer Hardware	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:10.89875	2020-10-11 23:38:31.024353
SCWX	SecureWorks Corp.	SecureWorks Corp is a provider of intelligence-driven information security solutions exclusively focused on protecting its clients from cyber attacks. Its solutions enable organizations to fortify their cyber defenses to prevent security breaches.	{Technology,"Software - Application","Application Software"}	{OTEX,PANW,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:10.924867	2020-10-11 23:38:31.023428
SDRL	Seadrill Limited		{Energy,"Oil & Gas Drilling","Oil & Gas - Drilling"}	{}	Oil & Gas - Drilling	Energy	New York Stock Exchange	\N	2020-10-11 23:12:10.94684	2020-10-11 23:38:31.028166
RYF	Invesco S&P 500 Equal Weight Financial	The investment seeks to track the investment results (before fees and expenses) of the S&P 500® Equal Weight Financials Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities that comprise the underlying index. The underlying index is an equal-weighted version of the S&P 500® Financials Index. Strictly in accordance with its guidelines and mandated procedures, the index provider compiles, maintains and calculates the underlying index, which is comprised of common stocks of companies in the financials sector of the S&P 500® Index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.653248	2020-10-11 23:38:31.028166
RZA	Reinsurance Group of America Incorporated 6.20% Fixed-to-Floating Rate Subordinated Debentures due 2042	Reinsurance Group of America Inc is an insurance holding company. It provides various products including life reinsurance, living benefits reinsurance, group reinsurance, health reinsurance, financial solutions, and product development.	{"Financial Services","Insurance - Reinsurance","Insurance - Specialty"}	{}	Insurance - Specialty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:10.671599	2020-10-11 23:38:31.028166
SAFT	Safety Insurance Group Inc.	Safety Insurance Group Inc is a provider of private passenger and commercial automobile insurance in Massachusetts.It also offers property and casualty insurance products, including commercial automobile, homeowners, and others.	{"Financial Services","Insurance - Property & Casualty"}	{ALL,MCY,PGR,AIG}	Insurance - Property & Casualty	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:10.709374	2020-10-11 23:38:31.02423
SBB	ProShares Short SmallCap600	The investment seeks daily investment results that correspond to the inverse (-1x) of the daily performance of the S&P SmallCap 600® Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index is a measure of small-cap company U.S. stock market performance. It is a float-adjusted, market capitalization-weighted index of 600 U.S. operating companies selected through a process that factors in criteria such as liquidity, price, market capitalization, financial viability and public float. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.753702	2020-10-11 23:38:31.02423
SBIO	ALPS Medical Breakthroughs	The investment seeks investment results that correspond (before fees and expenses) generally to the performance of its underlying index, the Poliwogg Medical Breakthroughs IndexSM.\n The fund employs a "passive management" - or indexing - investment approach designed to track the performance of the underlying index. It will normally invest at least 80% of its net assets in securities that comprise the underlying index. The underlying index is comprised of small- and mid-cap stocks of biotechnology and pharmaceutical companies that have one or more drugs in either Phase II or Phase III U.S. Food and Drug Administration clinical trials. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.779412	2020-10-11 23:38:31.02423
SBR	Sabine Royalty Trust	Sabine Royalty Trust is an express trust, which receives a distribution of royalty and mineral interests from certain producing and proved undeveloped oil and gas properties located in Florida, Louisiana, Mississippi, New Mexico, Oklahoma and Texas.	{Energy,"Oil & Gas Drilling","Oil & Gas - Drilling"}	{MARPS,DMLP,PHX,PBT,CRT,MTR,NRT,SJT}	Oil & Gas - Drilling	Energy	New York Stock Exchange	\N	2020-10-11 23:12:10.802967	2020-10-11 23:38:31.031328
SCAP	AdvisorShares Cornerstone Small Cap	The investment seeks to provide total return through long-term capital appreciation and current income.\n The fund seeks to achieve its investment objective by investing in a diversified group of U.S.-traded equity securities consisting of common and preferred stock, ADRs, and REITs. Under normal circumstances, it invests at least 80% of its net assets (plus any borrowings for investment purposes) in small cap securities. The fund's sub-advisor generally defines a small cap company as one having a market capitalization less than the market cap of the largest company in the Russell 2000 Index at the time of acquisition.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.832142	2020-10-11 23:38:31.028166
SCHL	Scholastic Corporation	Scholastic Corp is an American publishing and education media company. It focuses on books and educational material for schools, teachers, parents, and children.	{"Consumer Cyclical",Publishing}	{EDUC,DIS,JW.A,PSO,NWSA,HMHC,RELX,SPGI}	Publishing	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:10.858439	2020-10-11 23:38:31.028166
SCI	Service Corporation International	Service Corp International is a personal services company that provides funeral and cemetery services. It derives its revenue from its funeral locations, while the cemetery division also generates a significant amount of total income.	{"Consumer Cyclical","Personal Services"}	{MATW,HI,CSV,STON}	Personal Services	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:10.886824	2020-10-11 23:38:31.02423
SCON	Superconductor Technologies Inc.	Superconductor Technologies Inc develops and produces high-temperature superconductor materials. The company operates through the research, development, manufacture, and marketing of high-performance products used in cellular base stations segment.	{Technology,"Semiconductor Equipment & Materials",Semiconductors}	{XLK}	Semiconductors	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:10.906186	2020-10-11 23:38:31.028166
SCX	L.S. Starrett Company (The)	The L S Starrett Co is engaged in manufacturing industrial, professional and consumer measuring and cutting tools and related products. It operates in North America, Brazil, China and the United Kingdom.	{Industrials,"Tools & Accessories","Industrial Products"}	{PFIN,ALG,SNA,SWK}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:10.928172	2020-10-11 23:38:31.028166
SDR	SandRidge Mississippian Trust II representing Beneficial Interests	SandRidge Mississippian Trust II holds Royalty Interests in specified oil & natural gas properties located in Mississippian formation in Alfalfa, Kay, Noble & Woods counties in Oklahoma and Barber, Comanche, Harper and Sumner counties in southern Kansas.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:10.946785	2020-10-11 23:38:31.021426
SAFE	Safety Income & Growth Inc.	Safety Income and Growth Inc is a real estate company. It is formed primarily to acquire, own, manage, finance and capitalize ground net leases.	{"Real Estate","Real Estate - General","Real Estate Services"}	{NYMT,O,NTP}	Real Estate Services	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:10.701958	2020-10-11 23:38:31.028166
SALM	Salem Media Group Inc.	Salem Media Group Inc is a domestic multimedia company with integrated operations including radio broadcasting, digital media, and publishing. The Company has three operating segments, Broadcast, Digital Media, and Publishing.	{"Consumer Cyclical","Broadcasting - Radio",Entertainment}	{BBGI,ETM,EVC,EMMS,UONEK,SGA}	Entertainment	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:10.722853	2020-10-11 23:38:31.028166
SASR	Sandy Spring Bancorp Inc.	Sandy Spring Bancorp Inc through its subsidiary, provides commercial banking, mortgage banking, and trust services. It also offers investment and wealth management services through its subsidiary.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:10.743071	2020-10-11 23:38:31.028166
SBBX	SB One Bancorp	SB One Bancorp is a bank holding company. It conducts traditional commercial banking business, and offer services including personal and business checking accounts and time deposits, money market accounts and savings accounts.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:10.757424	2020-10-11 23:38:31.031328
SBH	Sally Beauty Holdings Inc. (Name to be changed from Holdings Inc.)	Sally Beauty Holdings Inc is a specialty retailer and distributor of professional beauty products with operations in North America, South America and Europe.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{ULTA,REV,GNC,JMEI}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:10.774424	2020-10-11 23:38:31.028166
SBOT	Stellar Biotechnologies Inc.	Stellar Biotechnologies Inc is a biotechnology company engaged in the aquaculture, research and development, manufacture and commercialization of Keyhole Limpet Hemocyanin (KLH) protein.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:10.795126	2020-10-11 23:38:31.028166
SBUX	Starbucks Corporation	Starbucks Corp is the roaster, marketer and retailer of specialty coffee in the world, operating globally. It sells a variety of coffee and tea products. It sells goods and services under brands including Teavana, Tazo, and Seattle's Best Coffee.	{"Consumer Cyclical",Restaurants}	{CBRL,DRI,EAT,MCD,YUM,DNKN,WEN,QSR,PEP}	Restaurants	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:10.816423	2020-10-11 23:38:31.02582
SCCO	Southern Copper Corporation	Southern Copper Corp is a metal producer having its focus on copper. Its segments include Peruvian operations, Mexican open-pit operations and Mexican underground mining operations.	{"Basic Materials",Copper,"Metals & Mining"}	{FM,TECK,BBL,FCX,RIO,BHP}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:10.832687	2020-10-11 23:38:31.02582
SCHH	Schwab U.S. REIT	The investment seeks to track as closely as possible, before fees and expenses, the total return of the Bloomberg Barclays US Aggregate Bond Index.\n The fund will invest at least 90% of its net assets in securities included in the index. The index is a broad-based benchmark measuring the performance of the U.S. investment grade, taxable bond market, including U.S. Treasuries, government-related and corporate bonds, mortgage pass-through securities, commercial mortgage-backed securities, and asset-backed securities that are publicly available for sale in the United States.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.864771	2020-10-11 23:38:31.023007
SCIF	VanEck Vectors India Small-Cap Index	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the MVIS® India Small-Cap Index.\n The fund invests substantially all of its assets in the Subsidiary, a wholly-owned subsidiary located in the Republic of Mauritius. The Subsidiary in turn will normally invest at least 80% of its total assets in securities that comprise the fund's benchmark index, and depositary receipts based on the securities in the fund's benchmark index. The index includes Indian small-capitalization companies selected on the basis of their relative market capitalizations.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.885172	2020-10-11 23:38:31.028166
SCPH	scPharmaceuticals Inc.	scPharmaceuticals Inc is a pharmaceutical company. Its pipeline includes products in various stages of clinical and non-clinical development. It focuses on proven pharmaceutical products in the cardiovascular and infectious disease therapeutic areas.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:10.905912	2020-10-11 23:38:31.028166
SCZ	iShares MSCI EAFE Small-Cap ETF	The investment seeks to track the investment results of the MSCI EAFE Small Cap Index composed of small-capitalization developed market equities, excluding the U.S. and Canada.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index represents the small-capitalization segment of the MSCI EAFE IMI Index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:10.928099	2020-10-11 23:38:31.025871
SDPI	Superior Drilling Products Inc.	Superior Drilling Products Inc is a drilling and completion tool technology company. The Company is a manufacturer of both remanufactured and new diamond drill bits and drill string components for the oil, natural gas and mining drilling industries.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{OII,FI}	Oil & Gas - Services	Energy	NYSE American	\N	2020-10-11 23:12:10.946033	2020-10-11 23:38:31.031328
SAR	Saratoga Investment Corp New	Saratoga Investment Corp is a closed end management investment company that invests in leveraged loans and mezzanine debt issued by private U.S. middle-market companies.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:10.7422	2020-10-11 23:38:31.023428
SBBP	Strongbridge Biopharma plc	Strongbridge Biopharma PLC is a biopharmaceutical company which focuses on the development, in-licensing, acquisition, and commercialization of various complementary products and product candidates that target rare diseases.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:10.753712	2020-10-11 23:38:31.025871
SBGL	D/B/A Sibanye-Stillwater Limited American Depositary Shares (Each representing four)	Sibanye-Stillwater is an independent precious metal mining group, producing a unique mix of metals that includes gold and the platinum group metals. Its projects include Burnstone, Kloof & driefontein decline, SOFS, Blitz, Altar and Marathon.	{"Basic Materials",Gold,"Metals & Mining"}	{ABX,HMY,GFI,AU,NEM,DRD}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:10.771655	2020-10-11 23:38:31.021426
SBPH	Spring Bank Pharmaceuticals Inc.	Spring Bank Pharmaceuticals Inc is a clinical-stage biopharmaceutical company. It is engaged in the discovery and development of a novel class of therapeutics using a proprietary small molecule nucleic acid hybrid, or SMNH, chemistry platform.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:10.802332	2020-10-11 23:38:31.023428
SCAC	Saban Capital Acquisition Corp.	Saban Capital Acquisition Corp is a blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:10.823558	2020-10-11 23:38:31.028166
SCHC	Schwab International Small-Cap Equity	The investment seeks to track as closely as possible, before fees and expenses, the total return of the FTSE Developed Small Cap ex US Liquid Index.\n The fund will invest at least 90% of its net assets in stocks, including depositary receipts representing securities of the index; such depositary receipts may be in the form of American Depositary receipts, Global Depositary receipts and European Depositary receipts. The index is comprised of small capitalization companies in developed countries outside the United States.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.847321	2020-10-11 23:38:31.021426
SCHR	Schwab Intermediate-Term U.S. Treasury	The investment seeks to track as closely as possible, before fees and expenses, the total return of the Bloomberg Barclays US Treasury 3-10 Year Index.\n The fund will invest at least 90% of its net assets (including, for this purpose, any borrowings for investment purposes) in securities included in the index. The index includes all publicly-issued U.S. Treasury securities that have a remaining maturity of greater than or equal to three years and less than ten years, are rated investment grade, and have $300 million or more of outstanding face value. The securities in the index must be denominated in U.S. dollars and must be fixed-rate and non-convertible.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.869769	2020-10-11 23:38:31.023428
SCJ	iShares MSCI Japan Sm Cap	The investment seeks to track the investment results of the MSCI Japan Small Cap Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is designed to measure the performance of equity securities of small-capitalization companies in Japan.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.898823	2020-10-11 23:38:31.02582
SCVL	Shoe Carnival Inc.	Shoe Carnival Inc is a family footwear retailer that offers a range of moderately priced dress, casual and athletic footwear for men, women and children. Its store has several brands; Nike and Skechers contributing the majority portion.	{"Consumer Cyclical","Apparel Stores","Retail - Apparel & Specialty"}	{GCO,FL,DSW,CAL}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:10.926137	2020-10-11 23:38:31.02582
SDOW	UltraPro Short Dow30	The investment seeks daily investment results, before fees and expenses, that correspond to three times the inverse (-3x) of the daily performance of the Dow Jones Industrial Average® Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index is a price-weighted index and includes 30 large-cap, "blue-chip" U.S. stocks, excluding utility and transportation companies. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.945466	2020-10-11 23:38:31.025871
SDYL	ETRACS Monthly Pay 2xLeveraged S&P Dividend ETN	The investment seeks to replicate, net of expenses, the monthly compounded 2x leveraged performance of the S&P High Yield Dividend Aristocrats® Index. \n The index is designed the measure the performance of the 60 highest dividend yielding S&P Composite 1500 Index constituents, which have followed a managed-dividends policy of consistently increasing dividends every year for at least 25 consecutive years.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.959646	2020-10-11 23:38:31.023428
SOHOB	Sotherly Hotels Inc. 8.0% Series B Cumulative Redeemable Perpetual Preferred Stock	Sotherly Hotels Inc is a self-managed and self-administered real estate investment trust that owns and manages a portfolio of upper up-scale, up-scale and mid-scale hotels located in main markets in the Mid-Atlantic and Southeastern United States.	{"Real Estate","REIT - Hotel & Motel",REITs}	{}	REITs	Real Estate	NASDAQ Global Market	\N	2020-10-11 23:12:11.531765	2020-10-11 23:38:31.028166
SBOW	SilverBow Resorces Inc.	SilverBow Resources Inc is a US-based energy company. It explores, develops, and produces oil and gas from the Eagle Ford Shale. It possesses understanding of regional reservoirs which enhances its operations to maximize returns on capital invested.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:10.802255	2020-10-11 23:38:31.025871
SCACW	Saban Capital Acquisition Corp. Warrants		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:10.823767	2020-10-11 23:38:31.021426
SCID	Global X Scientific Beta Europe	The investment seeks investment results that correspond generally to the price and yield performance, before fees and expenses, of the Scientific Beta Extended Developed Europe Multi-Beta Multi-Strategy Four-Factor Equal Risk Contribution (ERC) Index.\n The fund invests at least 80% of its total assets in the securities of the underlying index and in American Depositary Receipts ("ADRs") and Global Depositary Receipts based on the securities in the underlying index. The underlying index generally comprises approximately 600 or less European-listed common stocks selected based on a proprietary methodology developed by the index provider.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.884047	2020-10-11 23:38:31.031328
SCO	ProShares UltraShort Bloomberg Crude Oil	The investment seeks to provide daily investment results (before fees and expenses) that correspond to twice (200%) the inverse of the daily performance of the Bloomberg WTI Crude Oil SubindexSM. \nThe “UltraShort” funds seek results for a single day that match (before fees and expenses) two times the inverse (-2x) of the daily performance of a benchmark. It does not seek to achieve their stated objectives over a period greater than a single day. The Bloomberg WTI Crude Oil SubindexSM is designed to track crude oil futures prices.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.906186	2020-10-11 23:38:31.02423
SDLP	Seadrill Partners LLC Representing Limited Liability Company Interests	Seadrill Partners LLC is engaged in owing, operating and acquiring offshore drilling units. Its drilling rigs are under long-term contracts with oil companies such as Chevron, Total, BP and ExxonMobil.	{Energy,"Oil & Gas Drilling","Oil & Gas - Drilling"}	{DO,RIG,RDC,ESV}	Oil & Gas - Drilling	Energy	New York Stock Exchange	\N	2020-10-11 23:12:10.931522	2020-10-11 23:38:31.024353
SDT	SandRidge Mississippian Trust I of Beneficial Interest	SandRidge Mississippian Trust I is a statutory trust. It holds Royalty Interests in specified oil and natural gas properties located in the Mississippian formation in Alfalfa, Garfield, Grant and Woods counties in Oklahoma.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{ZN,CHKR,VNOM}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:10.968021	2020-10-11 23:38:31.024353
SEND	SendGrid Inc.	SendGrid Inc offers cloud-based email services. Its digital communication platform, enables businesses to engage with their customers via email reliably, effectively and at scale. It offers three services: Email API, Marketing Campaign, & Expert Services.	{Technology,"Software - Infrastructure","Application Software"}	{OTEX,PANW,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:11.00821	2020-10-11 23:38:31.02423
SFLY	Shutterfly Inc.	Shutterfly Inc is an online manufacturer and retailer of personalized products and services. Its brands include Shutterfly, Tiny Prints, Wedding Paper Divas, MyPublisher, BorrowLenses and Groovebook.	{"Consumer Cyclical","Personal Services"}	{CVS,RAD,WBA,HPQ,GOOGL}	Personal Services	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:11.061088	2020-10-11 23:38:31.031328
SGH	SMART Global Holdings Inc.	SMART Global Holdings Inc offers specialty memory solutions to the electronics industry. It manufactures memory for desktops, notebooks and servers, as well as mobile memory for smartphones.	{Technology,"Semiconductor Memory",Semiconductors}	{QRVO,QTNA,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:11.090564	2020-10-11 23:38:31.02582
SHE	SPDR Series Trust SSGA Gender Diversity Index	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the SSGA Gender Diversity Index, which tracks U.S. companies that are leaders in advancing women through gender diversity on their boards of directors and in management.\n Under normal market conditions, the fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. In addition, it may invest in equity securities that are not included in the index, cash and cash equivalents or money market instruments. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.135196	2020-10-11 23:38:31.02582
SHIP	Seanergy Maritime Holdings Corp	Seanergy Maritime Holdings Corp provides global transportation solutions in the dry bulk shipping sectors through its vessel owning subsidiaries.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{DSX,NM,NMM,SB,SBLK,EGLE}	Transportation & Logistics	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:11.151782	2020-10-11 23:38:31.023428
SHPG	Shire plc	Shire PLC is a biotechnology company focused mainly in the research, development, licensing, manufacturing, marketing, distribution and sale of specialist medicines for patients with rare diseases and other select conditions.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:11.168263	2020-10-11 23:38:31.031328
SCHV	Schwab U.S. Large-Cap Value	The investment seeks to track as closely as possible, before fees and expenses, the total return of the Dow Jones U.S. Large-Cap Value Total Stock Market Index.\n To pursue its goal, the fund generally invests in stocks that are included in the Dow Jones U.S. Large-Cap Value Total Stock Market Index. The index includes the large-cap value portion of the Dow Jones U.S. Total Stock Market Index actually available to investors in the marketplace. The Dow Jones U.S. Large-Cap Value Total Stock Market Index includes the components ranked 1-750 by full market capitalization and that are classified as "value" based on a number of factors.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.873691	2020-10-11 23:38:31.02582
SCIJ	Global X Scientific Beta Japan	The investment seeks investment results that correspond generally to the price and yield performance, before fees and expenses, of the Scientific Beta Japan Multi-Beta Multi-Strategy Four-Factor Equal Risk Contribution (ERC) Index.\n The fund invests at least 80% of its total assets in the securities of the underlying index and in American Depositary Receipts ("ADRs") and Global Depositary Receipts ("GDRs") based on the securities in the underlying index. The underlying index generally comprises approximately 500 or less Japanese-listed common stocks selected based on a proprietary methodology developed by EDHEC Risk Institute Asia Ltd.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.888574	2020-10-11 23:38:31.021426
SCM	Stellus Capital Investment Corporation	Stellus Capital Investment Corp is a closed-end, non-diversified management investment company. Its objective is to maximize the total return to its stockholders in the form of current income and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:10.904847	2020-10-11 23:38:31.021426
SDIV	Global X SuperDividend	The investment seeks investment results that correspond generally to the price and yield performance, before fees and expenses, of the Solactive Global SuperDividend® Index.\n The fund invests at least 80% of its total assets in the securities of the underlying index and in American Depositary Receipts ("ADRs") and Global Depositary Receipts ("GDRs") based on the securities in the underlying index. The underlying index tracks the performance of 100 equally-weighted companies that rank among the highest dividend yielding equity securities in the world, including emerging market countries.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.932491	2020-10-11 23:38:31.023428
SDVY	First Trust SMID Cap Rising Dividend Achievers ETF	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an index called the NASDAQ US Small Mid Cap Rising Dividend AchieversTM Index (the "index").\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is composed of the securities of 100 small and mid cap companies with a history of raising their dividends and exhibit the characteristics to continue to do so in the future. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:10.960793	2020-10-11 23:38:31.02423
SEF	ProShares Short Financials	The investment seeks daily investment results, before fees and expenses, that correspond to the inverse (-1x) of the daily performance of the Dow Jones U.S. FinancialsSM Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index seeks to measure the performance of certain companies in the financial services sector of the U.S. equity market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.990324	2020-10-11 23:38:31.02582
SES	Synthesis Energy Systems Inc.	Synthesis Energy Sys Inc with its subsidiaries develops, builds, owns and operates clean energy and chemical projects and also provides proprietary gasification technology systems and solutions to produce synthesis gas.	{"Basic Materials","Specialty Chemicals",Chemicals}	{GE,SXC,SXCP}	Chemicals	Basic Materials	NASDAQ Capital Market	\N	2020-10-11 23:12:11.018961	2020-10-11 23:38:31.024353
SFL	Ship Finance International Limited	Ship Finance International Ltd is an international ship owning and chartering company. The company is engaged in transporting crude oil and oil products, dry bulk and containerized cargos, and in offshore drilling and related activities.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{GLNG,NAT,TK,TGP,TOO,TNK,DHT,ASC,FRO,GOGL}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:11.058705	2020-10-11 23:38:31.028166
SGYP	Synergy Pharmaceuticals Inc.	Synergy Pharmaceuticals Inc is a manufacturer of pharmaceutical products in the United States. It develops drugs for the treatment of gastrointestinal disorders such as constipation and irritable bowel syndrome.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{PGNX,SHPG,IRWD,EVOK,INVA,XLV}	Drug Manufacturers	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:11.122811	2020-10-11 23:38:31.023428
SHM	SPDR Nuveen Bloomberg Barclays Short Term Municipal Bond	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the price and yield performance of the Bloomberg Barclays Managed Money Municipal Short Term Index.\n The fund invests substantially all, but at least 80%, of its total assets in the securities comprising the index or in securities that the Sub-Adviser determines have economic characteristics that are substantially identical to the economic characteristics of the securities that comprise the index. The index tracks the short term tax exempt municipal bond market and provides income that is exempt from federal income taxes. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.15579	2020-10-11 23:38:31.028166
SCTO	Global X JPMorgan US Sector Rotator Index	The investment seeks investment results that correspond generally to the price and yield performance, before fees and expenses of the JPMorgan U.S. Sector Rotator TR Series X Index.\n The fund invests at least 80% of its total assets in the securities of the underlying index. The underlying index aims to participate in the best-performing U.S. sectors of the market while limiting its exposure to the market during periods of high volatility or market declines. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.915121	2020-10-11 23:38:31.024353
SDEM	Global X MSCI SuperDividend Emerging Markets	The investment seeks investment results that correspond generally to the price and yield performance, before fees and expenses, of the MSCI Emerging Markets Top 50 Dividend Index.\n The fund invests at least 80% of its total assets in the securities of the underlying index and in American Depositary Receipts ("ADRs") and Global Depositary Receipts ("GDRs") based on the securities in the underlying index. The underlying index tracks the performance of 50 equally-weighted companies that rank among the highest dividend yielding equity securities in emerging markets, as defined by MSCI. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.931007	2020-10-11 23:38:31.028166
SDY	SPDR S&P Dividend	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the S&P High Yield Dividend Aristocrats Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index is designed to measure the performance of the highest dividend yielding S&P Composite 1500® Index constituents that have followed a managed-dividends policy of consistently increasing dividends every year for at least 20 consecutive years. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.958347	2020-10-11 23:38:31.025871
SECO	Secoo Holding Limited	Secoo Holding Ltd operates Online-Offline shopping platform in China. It operates as an online retailer in China, with website secoo.com, mobile APP and SECOO experience centers.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{SECO,TPR}	Retail - Apparel & Specialty	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:10.980571	2020-10-11 23:38:31.028166
SEII	Sharing Economy International Inc.	Sharing Economy International Inc designs, manufactures and distributes a line of proprietary high and low-temperature dyeing and finishing machinery to the textile industry.	{Industrials,"Diversified Industrials","Industrial Products"}	{}	Industrial Products	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:10.998529	2020-10-11 23:38:31.028166
SF	Stifel Financial Corporation	Stifel Financial Corp is a full-service brokerage and investment banking firm. It provides securities brokerage, investment banking, trading, investment advisory, and related financial services to individual investors, businesses, and municipalities.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{SIEB,AMTD,ETFC,RJF,LTS,OPY,LPLA}	Brokers & Exchanges	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:11.025212	2020-10-11 23:38:31.02423
SFIX	Stitch Fix Inc.	Stitch Fix Inc offers personal style service for men and women. It delivers various product types, including denim, dresses, blouses, skirts, shoes, jewelry and handbags and sells merchandise across a various range of price points.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{LB,SFIX}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:11.054431	2020-10-11 23:38:31.028166
SGB	Southwest Georgia Financial Corporation	Southwest Georgia Financial Corp operates as the holding company for Southwest Georgia Bank. It provides financial products and services such as checking accounts, consumer loans, and commercial loans.	{"Financial Services","Banks - Regional - US",Banks}	{CBAN,HWC,LION,RNST,ABCB,CFR,STBZ,UCBI,ASB}	Banks	Financial Services	NYSE American	\N	2020-10-11 23:12:11.077933	2020-10-11 23:38:31.023428
SGRP	SPAR Group Inc.	SPAR Group Inc is a supplier of merchandising and other marketing services. It also provides in-store event staffing, product sampling, audit services, furniture and other product assembly services, technology services and marketing research services.	{Industrials,"Business Services"}	{ISIG,AUTO}	Business Services	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:11.113374	2020-10-11 23:38:31.025871
SHI	SINOPEC Shangai Petrochemical Company Ltd.	Sinopec Shanghai Petrochemical Co Ltd is a petrochemical company that produces over sixty different type of products including refined oil products, intermediate petrochemicals, synthetic resins and synthetic fibers.	{Energy,"Oil & Gas Refining & Marketing","Oil & Gas - Refining & Marketing"}	{HON,BP,PTR,SNP,CEO,BAK}	Oil & Gas - Refining & Marketing	Energy	New York Stock Exchange	\N	2020-10-11 23:12:11.151941	2020-10-11 23:38:31.02423
SHOS	Sears Hometown and Outlet Stores Inc.	Sears Hometown and Outlet Stores Inc is a national retailer engaged in selling home appliances, lawn and garden equipment, tools, and hardware.	{"Consumer Cyclical","Department Stores","Retail - Apparel & Specialty"}	{TSCO,ARCI,HD,KSS,JCP,LE}	Retail - Apparel & Specialty	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:11.165115	2020-10-11 23:38:31.02423
SHYG	iShares 0-5 Year High Yield Corporate Bond	The investment seeks to track the investment results of the Markit iBoxx® USD Liquid High Yield 0-5 Index composed of U.S. dollar-denominated, high yield corporate bonds with remaining maturities of less than five years.\n The fund generally invests at least 90% of its assets in the component securities of the index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The index is designed to reflect the performance of U.S. dollar-denominated high yield corporate debt.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.181608	2020-10-11 23:38:31.028166
SCL	Stepan Company	Stepan Co, along with its subsidiaries produces specialty and intermediate chemicals, which are sold to other manufacturers and are used in end products. The company manufactures and markets surfactants, polymers and specialty products.	{"Basic Materials","Specialty Chemicals",Chemicals}	{UG,WDFC,IFF,MTX,OLN,SXT,HUN}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:10.905031	2020-10-11 23:38:31.025871
SDD	ProShares UltraShort SmallCap600	The investment seeks daily investment results before fees and expenses that correspond to two times the inverse (-2x) of the daily performance of the S&P SmallCap 600® Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index is a measure of small-cap company U.S. stock market performance. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.932338	2020-10-11 23:38:31.021426
SDS	ProShares UltraShort S&P500	The investment seeks daily investment results, before fees and expenses, that correspond to two times the inverse (-2x) of the daily performance of the S&P 500® Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index is a measure of large-cap U.S. stock market performance. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.950078	2020-10-11 23:38:31.028166
SEAC	SeaChange International Inc.	SeaChange International Inc licenses, manages, distributes video and advertising content for cable television system operators, telecommunications companies, satellite operators and media companies.	{Technology,"Software - Application","Application Software"}	{CSCO,AVID,HLIT,ERIC,DOX,RUBI,CIDM,NTN,TIVO,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:10.965436	2020-10-11 23:38:31.021426
SEE	Sealed Air Corporation	Sealed Air Corp is engaged in three reporting segments, Food care, Diversey Care and Product care. It mainly provides cleaning, sanitation, and housekeeping solutions to the food service industry.	{"Consumer Cyclical","Packaging & Containers"}	{SLGN,ATR,GEF.B,BMS,SON,BERY,WRK}	Packaging & Containers	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:10.985348	2020-10-11 23:38:31.02423
SENEB	Seneca Foods Corp. Class B Common Stock	Seneca Foods Corp is produces & distributes processed fruits & vegetables. Its products include canned, frozen & bottled produce and snack chips. Its products are sold under private label as well as national & regional brands.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{LANC,GIS,SJM,THS}	Consumer Packaged Goods	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:11.012462	2020-10-11 23:38:31.023007
SFE	Safeguard Scientifics Inc.	Safeguard Scientifics Inc is an asset management company. It provides capital as well as strategic, operational and management resources to entrepreneurs across an evolving and spectrum of industries.	{"Financial Services","Asset Management"}	{CNSL,MMS,BDX,FCN,CBZ,IRM}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:11.047816	2020-10-11 23:38:31.025871
SGEN	Seattle Genetics Inc.	Seattle Genetics Inc is a biotechnology company. The company is focused on development and commercialization of targeted therapies for the treatment of cancer. Its products are ADCETRIS (brentuximab vedotin) US and ADCETRIS (brentuximab vedotin) Canada.	{Healthcare,Biotechnology}	{PGNX,CELG,IMGN,SPPI,BIIB,CLDX,GSK,BMY,PFE,ABBV,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:11.088187	2020-10-11 23:38:31.023007
SGRY	Surgery Partners Inc.	Surgery Partners Inc is a healthcare service company. It owns and operates ambulatory surgery centers providing services including anesthesia, diagnostic testing and radiation oncology.	{Healthcare,"Medical Care","Health Care Providers"}	{NHC,SGRY,XLV}	Health Care Providers	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:11.118584	2020-10-11 23:38:31.023007
SHO	Sunstone Hotel Investors Inc.	Sunstone Hotel Investors Inc is a real estate investment trust. The company along with its subsidiaries is engaged in acquiring, owning, asset managing and renovating the full-service hotel and select focus-service hotel properties in the United States.	{"Real Estate","REIT - Hotel & Motel",REITs}	{HST,DRH,CHSP,CLDT,RLJ,INN,SOHO,HPT}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:11.158361	2020-10-11 23:38:31.025871
SIEB	Siebert Financial Corp.	Siebert Financial Corp is a holding company. It conducts retail discount brokerage business through its wholly owned subsidiary. The firm mainly provides online and traditional brokerage and related services to retail investors.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{AMTD,ETFC,RJF,SF,LTS,OPY,GCAP,LPLA}	Brokers & Exchanges	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:11.192601	2020-10-11 23:38:31.021426
SIX	Six Flags Entertainment Corporation	Six Flags Entertainment Corp is the owner and operator of regional theme and water parks in the United States, Mexico and Canada. It offers thrill rides, water attractions, themed areas, concerts and shows, restaurants, game venues and retail outlets.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{FUN}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:11.229446	2020-10-11 23:38:31.025871
SJR	Shaw Communications Inc.	Shaw Communications Inc is a communications and media company that offers consumers with broadband cable television, high-speed internet, home phone, telecommunications services, satellite direct-to-home services and engaging programming.	{"Communication Services","Telecom Services"}	{CGO,RCI,BCE,NFLX,T,TU}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:11.252018	2020-10-11 23:38:31.031328
SKT	Tanger Factory Outlet Centers Inc.	Tanger Factory Outlet Centers Inc owns and operates outlet centers in the United States and Canada. It is a self-administered and self-managed REIT. It also develops, acquires, owns and manages outlet shopping centers.	{"Real Estate","REIT - Retail",REITs}	{ALX}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:11.268668	2020-10-11 23:38:31.023428
SEB	Seaboard Corporation	Seaboard Corp is an agribusiness and transportation company. The group operates six business segments, pork, commodity trading and milling, marine, sugar, power, and turkey.	{Industrials,Conglomerates}	{HRL,TSN,BG,BRFS,ALEX}	Conglomerates	Industrials	NYSE American	\N	2020-10-11 23:12:10.980846	2020-10-11 23:38:31.028166
SEMG	Semgroup Corporation Class A	Semgroup Corp provides gathering, transportation, storage, distribution, marketing and other midstream services to independent producers, refiners of petroleum products and other market participants in the Midwest and Rocky Mountain regions.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{PAA,EEP,MMP,TLP,DCP,NGL,BKEP,PBA,PSXP,PAGP}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:11.005906	2020-10-11 23:38:31.025871
SFBS	ServisFirst Bancshares Inc.	Servisfirst Bancshares Inc is a bank holding company. It offers commercial, consumer and other loans and accepts deposits, provides electronic banking services, deliver treasury and cash management services.	{"Financial Services","Banks - Regional - US",Banks}	{AUBN,CTBI,TRMK,RNST,SYBT,FUSB,BXS}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:11.040688	2020-10-11 23:38:31.031328
SFST	Southern First Bancshares Inc.	Southern First Bancshares Inc is a bank holding company. It is engaged in the business of accepting demand deposits and savings deposits and providing commercial, consumer and mortgage loans to the general public.	{"Financial Services","Banks - Regional - US",Banks}	{BKSC,FCCO,BBT,WFC,STI}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:11.075716	2020-10-11 23:38:31.023007
SGLB	Sigma Labs Inc.	Sigma Labs Inc is a technology company that specializes in the development and commercialization of manufacturing and materials technologies.	{Technology,"Information Technology Services","Application Software"}	{ICCC,ONVO,XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:11.095838	2020-10-11 23:38:31.021426
SHEN	Shenandoah Telecommunications Co	Shenandoah Telecommunications Co is a telecommunications provider. Its services include wireless, cable and wireline based communication solutions.	{"Communication Services","Telecom Services"}	{CNSL,CTL,USM,VZ,T,TDS,SPOK}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:12:11.136153	2020-10-11 23:38:31.023007
SHOO	Steven Madden Ltd.	Steven Madden Ltd designs, markets and sells fashion-forward name brand and private label footwear for women, men and children and name brand and private label fashion handbags and accessories.	{"Consumer Cyclical","Footwear & Accessories","Manufacturing - Apparel & Furniture"}	{CROX,SKX,WWW,DECK,CAL}	Manufacturing - Apparel & Furniture	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:11.162872	2020-10-11 23:38:31.02582
SID	Companhia Siderurgica Nacional S.A.	Companhia Siderurgica Nacional is an integrated steel producer. The company operates in five segments namely, Steel, Mining, Logistics, Energy and Cement.	{"Basic Materials",Steel}	{STLD,GGB,NUE,PBR,X,PKX,TX,CZZ,MTL,MT}	Steel	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:11.183595	2020-10-11 23:38:31.024353
SIGM	Sigma Designs Inc.	Sigma Designs Inc is a system-on-chip solutions provider offering intelligent media platforms for use in the home entertainment and home control appliances.	{Technology,Semiconductors}	{DSPG,QCOM,SLAB,PXLW,STM,TXN,XLK}	Semiconductors	Technology	OTC Pink	\N	2020-10-11 23:12:11.199562	2020-10-11 23:38:31.023428
SIR	Select Income REIT	Select Income REIT is a part of the real estate sector in the United States. It primarily invests in land and properties that are primarily net leased to single tenants. The company's primary source of revenue is rental income.	{"Real Estate","REIT - Diversified",REITs}	{CUZ,PCH,PW,LAND,CTT}	REITs	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:11.222422	2020-10-11 23:38:31.023428
SJI	South Jersey Industries Inc.	South Jersey Industries Inc is an energy services holding company. The company through its subsidiaries provides regulated natural gas service to residential, commercial, and industrial customers in United States.	{Utilities,"Utilities - Regulated Gas","Utilities - Regulated"}	{NI,CPK,NFG,NJR,SWX,UGI,VVC,CNP,SR,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:11.24523	2020-10-11 23:38:31.02423
SKYS	Sky Solar Holdings Ltd.	Sky Solar Holdings Ltd develops, owns and operates solar parks and sells electricity. Its geographic operations include Greece, Canada, Bulgaria, Germany, Czech, Japan, Spain, Italy, and Uruguay.	{Utilities,"Utilities - Independent Power Producers"}	{ORA,NEP,XLU}	Utilities - Independent Power Producers	Utilities	NASDAQ Capital Market	\N	2020-10-11 23:12:11.271406	2020-10-11 23:38:31.031328
SLG	SL Green Realty Corp	SL Green Realty Corp is a self-managed real estate investment trust which is engaged in the management, acquisition, development, and leasing of property. The company generates its revenue from tenant rents, escalations, and reimbursement.	{"Real Estate","REIT - Office",REITs}	{VNO,DEI,BX,HPP}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:11.300337	2020-10-11 23:38:31.023428
SLV	iShares Silver Trust	The investment seeks to reflect generally the performance of the price of silver.\n The Trust seeks to reflect such performance before payment of the Trust’s expenses and liabilities. It is not actively managed. The Trust does not engage in any activities designed to obtain a profit from, or to ameliorate losses caused by, changes in the price of silver. The Trust seeks to reflect generally the performance of the price of silver. The Trust seeks to reflect such performance before payment of the Trust’s expenses and liabilities.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.335614	2020-10-11 23:38:31.031328
SMBK	SmartFinancial Inc.	SmartFinancial Inc is a bank holding company. The company provides commercial bank products and services including personal banking, business banking and E-banking services.	{"Financial Services","Banks - Regional - US",Banks}	{FLIC,WSBC,ASRV,LARK,HWBK}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:11.367908	2020-10-11 23:38:31.031328
SEIC	SEI Investments Company	SEI Investments Co is an asset management firm. It caters to financial advisors and investors with investment management activities. It is a provider of investment processing, investment management, and investment operations platforms.	{"Financial Services","Asset Management"}	{NTRS,MORN,STT,FII,BEN,SCHW,BK}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:10.99058	2020-10-11 23:38:31.021426
SENS	Senseonics Holdings Inc.	Senseonics Holdings Inc is a medical technology company engaged in designing, development and commercialization of glucose monitoring systems for diabetes patients.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{SENS,XLV}	Medical Diagnostics & Research	Healthcare	NYSE American	\N	2020-10-11 23:12:11.012835	2020-10-11 23:38:31.025871
SFNC	Simmons First National Corporation	Simmons First National Corp is a financial holding company. The company through its subsidiaries is engaged in providing banking services including consumer, real estate and commercial loans, checking, savings and time deposits.	{"Financial Services","Banks - Regional - US",Banks}	{BANF,UMBF,BOKF,IBKC,HOMB,CFR,USB,RF,MSL}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:11.062301	2020-10-11 23:38:31.024353
SGDJ	ALPS ETF Trust Sprott Junior Gold Miners	The investment seeks investment results that correspond (before fees and expenses) generally to the performance of its underlying index, the Sprott Zacks Junior Gold Miners Index.\n The fund will invest at least 90% of its net assets in securities that comprise the underlying index. The underlying index aims to track the performance of "junior" gold and junior silver mining companies whose stocks are traded on major U.S. or Canadian exchanges. Junior companies include early stage mining companies that are in the exploration stage only or that hold properties that might not ultimately produce gold or silver. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.082499	2020-10-11 23:38:31.031328
SGMS	Scientific Games Corp	Scientific Games Corp is a provider of gaming products, systems, and services to the lottery and pari-mutuel industries. Its business segments are; Gaming, Lottery and Interactive.	{"Consumer Cyclical",Gambling,"Travel & Leisure"}	{PENN,CHDN,DDE}	Travel & Leisure	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:11.108729	2020-10-11 23:38:31.023428
SGMO	Sangamo Therapeutics Inc.	Sangamo Therapeutics Inc is a clinical stage biotechnology company that translates ground-breaking science into genomic therapies transforming patients' lives using platform technologies in genome editing, gene therapy, gene regulation and cell therapy.	{Healthcare,Biotechnology}	{AMGN,BMRN,SHPG,GILD,BIIB,ALNY,PDLI,JNJ,MRK,DWDP,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:11.096083	2020-10-11 23:38:31.028166
SGU	Star Group L.P.	Star Group LP is a full service energy provider. It specializes in the sale of home heating products and services to residential and commercial customers. It also sells heating and air conditioning equipment.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{FGP,APU,UGI,GLP,SPH,SRLP,CEQP,DK}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:11.122281	2020-10-11 23:38:31.02423
SHBI	Shore Bancshares Inc	Shore Bancshares Inc is a financial holding company. It offers personalized banking, insurance and investment services to families and businesses in the Mid-Atlantic region.	{"Financial Services","Banks - Regional - US",Banks}	{GLBZ,SASR,FUNC,OLBK,MTB,PNC,WFC,HBMD}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:11.135262	2020-10-11 23:38:31.024353
SHG	Shinhan Financial Group Co Ltd American Depositary Shares	Shinhan Financial Group Co Ltd is a banking company whose services include financial services such as retail banking, corporate banking, wealth management, investment banking, credit card services, brokerage services, asset management and digital banking.	{"Financial Services","Banks - Regional - Asia",Banks}	{KB,MFG,C}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:11.151782	2020-10-11 23:38:31.025871
SHW	Sherwin-Williams Company (The)	Sherwin-Williams Co provides architectural paint and paint related products; and provides coatings for original-equipment manufacturers. It conducts its operations in North and South America, the Caribbean region, Europe, Asia, and Australia.	{"Basic Materials","Specialty Chemicals",Chemicals}	{FUL,HD,MAS,PPG,RPM}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:11.172511	2020-10-11 23:38:31.023428
SIG	Signet Jewelers Limited	Signet Jewelers Ltd is a retailer of diamond jewelry. It operates jewelry stores in malls and off-mall locations in the US under national brands including Kay, Kay Jewelers Outlet, Jared and Jared Vault, as well as a type of mall-based regional brands.	{"Consumer Cyclical","Luxury Goods","Retail - Apparel & Specialty"}	{TIF,DGSE,BGI}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:11.195802	2020-10-11 23:38:31.028166
SINO	Sino-Global Shipping America Ltd.	Sino-Global Shipping America Ltd is a shipping agency and logistic service provider. It provides ship owners, operators and charters with comprehensive and customized shipping agency and logistic services.	{Industrials,"Integrated Shipping & Logistics","Transportation & Logistics"}	{AIRT,ATSG,UPS}	Transportation & Logistics	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:11.220949	2020-10-11 23:38:31.024353
SOHU	Sohu.com Limited	Sohu.com Ltd is a Chinese online media, search and game service group. The company provides comprehensive online products and services on PCs and mobile devices in the People's Republic of China.	{Technology,"Internet Content & Information","Online Media"}	{XLK}	Online Media	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:11.535412	2020-10-11 23:38:31.031328
SEM	Select Medical Holdings Corporation	Select Medical Holdings Corp provides long term acute care hospital services and inpatient acute rehabilitative hospital care which requires extended recovery time with medical services.	{Healthcare,"Medical Care","Health Care Providers"}	{NHC,UHS,CYH,USPH,THC,GEN,XLV}	Health Care Providers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:10.999649	2020-10-11 23:38:31.031328
SFBC	Sound Financial Bancorp Inc.	Sound Financial Bancorp Inc through Sound Community Bank serves as a financial institution providing to its customers traditional banking and other financial services.	{"Financial Services","Banks - Regional - US",Banks}	{UNTY,SHBI}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:11.040567	2020-10-11 23:38:31.021426
SGA	Saga Communications Inc.	Saga Communications Inc is engaged acquiring, developing and operating broadcast properties. Its broadcasts shows like Classic Hits, Adult Contemporary, Classic Rock, News/Talk and Country.	{"Consumer Cyclical","Broadcasting - Radio",Entertainment}	{SALM,BBGI,ETM,EMMS,UONEK}	Entertainment	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:11.079226	2020-10-11 23:38:31.028166
SHAK	Shake Shack Inc. Class A	Shake Shack Inc is a roadside burger stand. The Company along with its subsidiaries operates and licenses restaurants in the food service industry, serving hamburgers, hot dogs, crinkle-cut fries, shakes, frozen custard, beer and wine.	{"Consumer Cyclical",Restaurants}	{YUM,DPZ,WEN}	Restaurants	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:11.133467	2020-10-11 23:38:31.031328
SHIPW	Seanergy Maritime Holdings Corp Class A Warrants		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:11.152099	2020-10-11 23:38:31.028166
SHYL	Xtrackers Short Duration High Yield Bond	The investment seeks investment results that correspond generally to the performance, before fees and expenses, of the Solactive USD High Yield Corporates Total Market 0-5 Year Index (the "underlying index").\n The fund will invest at least 80% of its total assets (but typically far more) in component securities of the underlying index. The underlying index is designed to track the performance of short-term publicly issued U.S. dollar-denominated below investment grade corporate debt. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.182315	2020-10-11 23:38:31.02423
SINA	Sina Corporation	SINA Corp is an online media company. It digital media network of SINA.com, SINA mobile and Weibo enables internet users to access professional media and user generated content in multi-media formats from personal computers and mobile devices.	{Technology,"Internet Content & Information","Online Media"}	{BIDU,FENG,YY,GOOGL,RENN,XLK}	Online Media	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:11.219623	2020-10-11 23:38:31.028166
SIJ	ProShares UltraShort Industrials	The investment seeks daily investment results, before fees and expenses, that correspond to two times the inverse (-2x) of the daily performance of the Dow Jones U.S. IndustrialsSM Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index seeks to measure the performance of certain companies in the industrial sector of the U.S. equity market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.201391	2020-10-11 23:38:31.02423
SIZE	iShares Edge MSCI USA Size Factor	The investment seeks to track the investment results of the MSCI USA Risk Weighted Index composed of U.S. large- and mid-capitalization stocks with relatively smaller average market capitalization.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The index is based on a traditional market capitalization-weighted parent index, the MSCI USA Index (the "parent index"). The parent index includes U.S. large- and mid- capitalization stocks.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.238217	2020-10-11 23:38:31.021426
SKF	ProShares UltraShort Financials	The investment seeks daily investment results, before fees and expenses, that correspond to two times the inverse (-2x) of the daily performance of the Dow Jones U.S. FinancialsSM Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index seeks to measure the performance of certain companies in the financial services sector of the U.S. equity market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.256375	2020-10-11 23:38:31.028166
SLCA	U.S. Silica Holdings Inc.	US Silica Holdings Inc is engaged in the mining industry. Its activities include exploration of minerals, especially silica used in making sand. The company is one of the leading suppliers of sand used in hydraulic fracturing of oil and gas wells.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{CRR,PXD,GLW,PPG,HCLP,DWDP}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:11.286525	2020-10-11 23:38:31.021426
SLVP	iShares MSCI Global Silver Miners Fund	The investment seeks to track the investment results of the MSCI ACWI Select Silver Miners Investable Market Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index measures the combined performance of equity securities of companies primarily engaged in the business of silver mining in both developed and emerging markets. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:11.3425	2020-10-11 23:38:31.023428
SO	Southern Company (The)	Southern Co is an electricity generating company. It develops, constructs, acquires, owns, and manages generation assets, including renewable energy projects, and sells electricity at market-based rates in the wholesale market.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{ETR,EE,FE,AEP,D,SCG,PPL,NRG,NEE,DUK,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:11.517831	2020-10-11 23:38:31.031328
SEP	Spectra Energy Partners LP representing Limited Partner Interests	Spectra Energy Partners LP is a Houston-based master limited partnership. The company owns and operates natural gas transmission, gathering and storage assets, and crude oil transportation and storage assets in United States and Canada.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{ENB,TRP,EPD,WMB,DCP,EQT,KMI}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:11.017872	2020-10-11 23:38:31.028166
SFM	Sprouts Farmers Market Inc.	Sprouts Farmers Market Inc is US based company which operates as a specialty retailer of natural and organic food. It offers products including fresh produce, bulk foods, vitamins and supplements, bakery, and body care and natural household items.	{"Consumer Defensive","Grocery Stores","Retail - Defensive"}	{IMKTA,KR,NGVC}	Retail - Defensive	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:11.060298	2020-10-11 23:38:31.021426
SGBX	SG Blocks Inc.	SG Blocks Inc is a United States based design and construction services company engaged in using code-engineered cargo shipping containers for construction. It fabricates modules for construction of buildings.	{Industrials,"Metal Fabrication","Industrial Products"}	{OFLX,SGBX}	Industrial Products	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:11.081493	2020-10-11 23:38:31.025871
SGLBW	Sigma Labs Inc. Warrant		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:11.095795	2020-10-11 23:38:31.02423
SGOL	Aberdeen Standard Physical Swiss Gold Shares	The investment seeks to reflect the performance of the price of gold bullion, less the Trust’s expenses. \n The Shares are intended to constitute a simple and cost-effective means of making an investment similar to an investment in gold. An investment in physical gold requires expensive and sometimes complicated arrangements in connection with the assay, transportation, warehousing and insurance of the metal. Although the Shares are not the exact equivalent of an investment in gold, they provide investors with an alternative that allows a level of participation in the gold market through the securities market.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.112782	2020-10-11 23:38:31.024353
SH	ProShares Short S&P500	The investment seeks daily investment results that correspond to the inverse (-1x) of the daily performance of the S&P 500® Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index is a measure of large-cap U.S. stock market performance. It is a float-adjusted, market capitalization-weighted index of 500 U.S. operating companies and real estate investment trusts selected through a process that factors in criteria such as liquidity, price, market capitalization and financial viability. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.129833	2020-10-11 23:38:31.021426
SHLO	Shiloh Industries Inc.	Shiloh Industries Inc is a supplier of lightweighting, noise and vibration solutions to the automotive, commercial vehicle and industrial markets. It operates through Automotive and Commercial Vehicles segment.	{Industrials,"Metal Fabrication","Industrial Products"}	{TSLA,WOR,F,GM,MT}	Industrial Products	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:11.154955	2020-10-11 23:38:31.021426
SHV	iShares Short Treasury Bond ETF	The investment seeks to track the investment results of the ICE U.S. Treasury Short Bond Index.\n The fund generally invests at least 90% of its assets in the bonds of the underlying index and at least 95% of its assets in U.S. government bonds. The index measures the performance of public obligations of the U.S. Treasury that have a remaining maturity of equal to or greater than one month and less than one year. It may invest up to 10% of its assets in U.S. government bonds not included in the underlying index, but which BFA believes will help the fund track the underlying index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:11.170229	2020-10-11 23:38:31.028166
SIFI	SI Financial Group Inc.	SI Financial Group Inc is a bank holding company. The Bank operates as a community-oriented financial institution offering a financial services to consumers & businesses, including insurance, trust & investment services.	{"Financial Services","Savings & Cooperative Banks",Banks}	{PBCT,WEBK,BHLB,UBNK,PBBI}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:11.189435	2020-10-11 23:38:31.023007
SILC	Silicom Ltd	Silicom Ltd designs and manufactures networking and data infrastructure solutions for various servers, server based systems and communications devices. Its product portfolio consists of server adapters, x86 open appliances, and bypass switches.	{Technology,"Communication Equipment"}	{SWIR,INTC,JCS,RDWR,CRNT,ALLT,LTRX,ANET,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:11.205062	2020-10-11 23:38:31.025871
SITE	SiteOne Landscape Supply Inc.	SiteOne Landscape Supply Inc is one of the largest suppliers of tools and equipment. The company serves various business areas which include the wholesale irrigation, outdoor lighting, nursery, landscape supplies, grass seeds, and fertilizers.	{Industrials,"Industrial Distribution"}	{SITE}	Industrial Distribution	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:11.226582	2020-10-11 23:38:31.028166
SJNK	SPDR Bloomberg Barclays Short Term High Yield Bond	The investment seeks to provide investment results that correspond generally to the price and yield performance of the Bloomberg Barclays US High Yield 350mn Cash Pay 0-5 Yr 2% Capped Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index or in securities that the Adviser determines have economic characteristics that are substantially identical to the economic characteristics of the securities that comprise the index. The index is designed to measure the performance of short-term publicly issued U.S. dollar-denominated high yield corporate bonds. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.249857	2020-10-11 23:38:31.028166
SELB	Selecta Biosciences Inc.	Selecta Biosciences Inc is a clinical-stage biopharmaceutical company using its proprietary synthetic vaccine particle technology to discover and develop targeted therapies designed to modulate the immune system to treat rare and serious diseases.	{Healthcare,Biotechnology}	{PDLI,PIRS,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:10.999421	2020-10-11 23:38:31.028166
SERV	ServiceMaster Global Holdings Inc.	ServiceMaster Global Holdings Inc is a provider of essential residential and commercial services. The company operates through three segments namely Terminix, American home shield and the Franchise services group.	{Industrials,"Business Services"}	{CNK,WAIR,AMC}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:11.019242	2020-10-11 23:38:31.02582
SFS	Smart & Final Stores Inc.	Smart & Final Stores Inc is a food retailer serving household and business customers through two store banners. Its Smart & Final stores serve both household and business customers, and its Cash & Carry stores serve primarily business customers.	{"Consumer Defensive","Grocery Stores","Retail - Defensive"}	{IMKTA,SFM}	Retail - Defensive	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:11.063895	2020-10-11 23:38:31.02423
SGDM	Sprott Gold Miners	The investment seeks results that correspond (before fees and expenses) generally to the performance of the Sprott Zacks Gold Miners Index.\n The index aims to track the performance of gold and silver mining companies whose stocks are traded on major U.S. exchanges. In addition to common stock or American Depository Receipts of gold mining companies, the index may include common stock or ADRs of silver mining companies. The fund will normally invest at least 90% of its net assets in securities that comprise the index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.083107	2020-10-11 23:38:31.024353
SGOC	SGOCO Group Ltd	SGOCO Group Ltd is principally engaged in product design, brand development and distribution in the Chinese display and computer product market as well as energy saving products and services.	{Technology,"Consumer Electronics","Computer Hardware"}	{AAPL,PHG,XLK}	Computer Hardware	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:11.109841	2020-10-11 23:38:31.028166
SGMA	SigmaTron International Inc.	Sigmatron International Inc provides electronic manufacturing services in the United States, Mexico, China, Vietnam and Taiwan. The Company's services include printed circuit board assemblies and completely assembled electronic products.	{Technology,"Contract Manufacturers","Computer Hardware"}	{KTCC,SYNA,STX,CTS,XLK}	Computer Hardware	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:11.096219	2020-10-11 23:38:31.031328
SGZA	Selective Insurance Group Inc. 5.875% Senior Notes due 2043	Selective Insurance Group Inc is a holding company that offers property and casualty insurance products and services in United States. It offers standard commercial, standard personal, and excess and surplus lines property and casualty insurance products.	{"Financial Services","Insurance - Property & Casualty"}	{}	Insurance - Property & Casualty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:11.128662	2020-10-11 23:38:31.028166
SHLX	Shell Midstream Partners L.P. representing Limited Partner Interests	Shell Midstream Partners LP owns, operates, develop and acquires pipelines and other midstream assets.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{TRP,MMP,PSXP,ENLK}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:11.155441	2020-10-11 23:38:31.023007
SIF	SIFCO Industries Inc.	SIFCO Industries Inc is engaged in the production of forgings and machined components primarily for the Aerospace and Energy markets.	{Industrials,"Metal Fabrication","Industrial Products"}	{TATT,HEI,TGI,ESL,UTX}	Industrial Products	Industrials	NYSE American	\N	2020-10-11 23:12:11.189522	2020-10-11 23:38:31.02582
SIM	Grupo Simec S.A.B. de C.V. American Depositary Shares	Grupo Simec SAB de CV is a diversified manufacturer, processor and distributor of SBQ steel and structural steel products. It also offers hot rolled bars, cold finished bars, semi-finished tube rounds and other semi-finished trade products.	{"Basic Materials",Steel}	{SCHN,STLD,GGB,CMC,NUE,X,TX,MT}	Steel	Basic Materials	NYSE American	\N	2020-10-11 23:12:11.210179	2020-10-11 23:38:31.023007
SITO	SITO Mobile Ltd.	SITO Mobile Ltd is a mobile location-based advertising platform serving businesses, advertisers and brands. The company offers wireless applications, licensing and royalties, and media placement services.	{"Communication Services","Telecom Services"}	{MCHX,RUBI,GOOGL}	Communication Services	Communication Services	NASDAQ Capital Market	\N	2020-10-11 23:12:11.227394	2020-10-11 23:38:31.031328
SJT	San Juan Basin Royalty Trust	San Juan Basin Royalty Trust is an express trust, which holds royalty interests in properties located in the San Juan Basin of northwestern New Mexico.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{MARPS,DMLP,PHX,PBT,CRT,MTR,NRT,SBR,TPL}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:11.253201	2020-10-11 23:38:31.02582
SKX	Skechers U.S.A. Inc.	Skechers USA Inc manufactures footwear for children, men and women under Skechers GO brand name offering two distinct footwear categories: a lifestyle division and performance footwear. Its products include branded apparel, bags, eyewear, toys, and more.	{"Consumer Cyclical","Footwear & Accessories","Manufacturing - Apparel & Furniture"}	{SHOO,COLM,CROX,RCKY,WWW,DECK,CAL}	Manufacturing - Apparel & Furniture	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:11.269457	2020-10-11 23:38:31.024353
SOJC	Southern Company (The) Series 2017B 5.25% Junior Subordinated Notes due December 1 2077		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:11.541627	2020-10-11 23:38:31.028166
SENEA	Seneca Foods Corp. Class A Common Stock	Seneca Foods Corp is produces & distributes processed fruits & vegetables. Its products include canned, frozen & bottled produce and snack chips. Its products are sold under private label as well as national & regional brands.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{LANC,GIS,SJM,THS}	Consumer Packaged Goods	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:11.010858	2020-10-11 23:38:31.028166
SFB	Stifel Financial Corporation 5.20% Senior Notes due 2047		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:11.032877	2020-10-11 23:38:31.023428
SHAG	WisdomTree Yield Enhanced U.S. Short-Term Aggregate Bond Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the Bloomberg Barclays U.S. Short Aggregate Enhanced Yield Index (the "index").\n Under normal circumstances, at least 80% of the fund's total assets will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is designed to broadly capture the short-term U.S. investment grade, fixed income securities market while seeking to enhance yield within desired risk parameters and constraints. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:11.131888	2020-10-11 23:38:31.028166
SHOP	Shopify Inc. Class A Subordinate	Shopify Inc provides cloud-based, multi-channel commerce platform designed for small and medium-sized businesses. The company's platform provides merchants with a single view of their business and customers across all of their sales channels.	{Technology,"Software - Application","Application Software"}	{CNXN,STMP,MELI,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:11.162225	2020-10-11 23:38:31.024353
SIEN	Sientra Inc.	Sientra Inc is a medical instruments provider in the United States. Its core product offering includes breast implants and breast tissue expanders which find use in breast reconstruction procedures.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{WST,ZBH,GKOS,XRAY,XLV}	Medical Instruments & Equipment	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:11.18696	2020-10-11 23:38:31.023428
SILJ	ETFMG Prime Junior Silver	The investment seeks investment results that, before fees and expenses, correspond generally to the price and yield performance of the Prime Junior Silver Miners & Explorers Index.\n The fund invests at least 80% of its total assets in the component securities of the index and in ADRs and GDRs based on the component securities in the index. The index tracks the performance of the equity securities (or corresponding American Depositary Receipts ("ADRs") or Global Depositary Receipts ("GDRs")) of small-capitalization companies actively engaged in silver refining, mining, or exploration ("Junior Silver Companies"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.207531	2020-10-11 23:38:31.02582
SIVR	Aberdeen Standard Physical Silver Shares	The investment seeks to replicate, net of expenses, the price of silver bullion.\n The shares are backed by physical allocated silver bullion held by the custodian. All physical silver held conforms to the London Bullion Market Association's rules for good delivery.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.2284	2020-10-11 23:38:31.02582
SJM	J.M. Smucker Company (The)	JM Smucker Co operates in the consumer foods industry. It deals with a range of products spanning food, beverage and pet products under brands such as Folgers, Crisco, Milk-Bone, and Kibbles 'n Bits.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{FARM,HAIN,JVA,CAG,GIS,BGS,THS,KHC}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:11.249442	2020-10-11 23:38:31.023428
SKY	Skyline Champion Corporation	Skyline Champion Corp, formerly Skyline Corp designs, produces and markets manufactured housing, modular housing and park models to independent dealers and manufactured housing communities located throughout the United States and Canada.	{"Consumer Cyclical","Residential Construction","Homebuilding & Construction"}	{CVCO,THO,LCII}	Homebuilding & Construction	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:11.270114	2020-10-11 23:38:31.028166
SLF	Sun Life Financial Inc.	Sun Life Financial Inc provides diversified financial services. It provides insurance, retirement, and wealth management products and services to individual and corporate customers in Canada, the United States and Asia.	{"Financial Services","Insurance - Diversified",Insurance}	{MFC,MIC,PUK,AEG,AET,VOYA}	Insurance	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:11.292194	2020-10-11 23:38:31.028166
SLNO	Soleno Therapeutics Inc.	Soleno Therapeutics Inc is focused on the development and commercialization of novel therapeutics for the treatment of rare diseases.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{NEOG,CEMI,NYMX,NTRP,XLV}	Medical Diagnostics & Research	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:11.314937	2020-10-11 23:38:31.021426
SLYG	SPDR S&P 600 Small Cap Growth ETF (based on S&P SmallCap 600 Growth Index --symbol CGK)	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the S&P SmallCap 600 Growth Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index measures the performance of the small-capitalization growth segment of the U.S. equity market. It may purchase a subset of the securities in the index in an effort to hold a portfolio of securities with generally the same risk and return characteristics of the index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.343583	2020-10-11 23:38:31.021426
SMED	Sharps Compliance Corp.	Sharps Compliance Corp is a provider of waste management services including medical, pharmaceutical and hazardous. It serve customers in multiple markets such as home health care, retail clinics and pharmaceutical manufacturers.	{Industrials,"Waste Management"}	{SRCL,CWST,WM,PESI}	Waste Management	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:11.373892	2020-10-11 23:38:31.031328
SIGI	Selective Insurance Group Inc.	Selective Insurance Group Inc is a holding company that offers property and casualty insurance products and services in United States. It offers standard commercial, standard personal, and excess and surplus lines property and casualty insurance products.	{"Financial Services","Insurance - Property & Casualty"}	{ERIE,CINF,HIG,WRB,FUL,MKL,PGR,THG,TRV}	Insurance - Property & Casualty	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:11.197699	2020-10-11 23:38:31.031328
SIVB	SVB Financial Group	SVB Financial Group provides a diverse set of banking and financial products and services including asset management, private wealth management, brokerage and other investment services.	{"Financial Services","Banks - Regional - US",Banks}	{CWBC,CATY,CVBF,HTBK,EWBC,CVCY,BMRC,TCF,OPB}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:11.230475	2020-10-11 23:38:31.028166
SLCT	Select Bancorp Inc.	Select Bancorp Inc provides banking and financial services. The company's services include checking and savings accounts, commercial, consumer, mortgage and personal loans, and other associated financial services.	{"Financial Services","Banks - Regional - US",Banks}	{PEBK,FBNC,FCNCA}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:11.292463	2020-10-11 23:38:31.025871
SLNOW	Soleno Therapeutics (Warrant)	Soleno Therapeutics, Inc. is a healthcare company, focused on the development and commercialization of novel therapeutics for the treatment of rare diseases. It markets CoSense End-Tidal Carbon Monoxide monitor, which measures ETCO and is used by hospitals to detect hemolysis in newborns. The company was founded on August 25, 1999 and is headquartered in Redwood City, CA.	{}	{}	Medical Specialties	Health Technology	NasdaqCM	\N	2020-10-11 23:12:11.315232	2020-10-11 23:38:31.02423
SLY	SPDR S&P 600 Small Cap ETF (based on S&P SmallCap 600 Index -- symbol SML)	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the S&P SmallCap 600 Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index measures the performance of the small-capitalization segment of the U.S. equity market. It may purchase a subset of the securities in the index in an effort to hold a portfolio of securities with generally the same risk and return characteristics of the index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.344051	2020-10-11 23:38:31.025871
SMDV	ProShares Russell 2000 Dividend Growers	The investment seeks investment results, before fees and expenses, that track the performance of the Russell 2000 Dividend Growth Index (the "index").\n Under normal circumstances, the fund will invest at least 80% of its total assets in component securities (i.e., securities of the index and comparable securities that have economic characteristics that are substantially identical to the economic characteristics of the securities of the index). The index contains a minimum of 40 stocks, which are equally weighted, and no single sector is allowed to comprise more than 30% of the index weight.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:11.373623	2020-10-11 23:38:31.02582
SMDD	UltraPro Short MidCap400	The investment seeks daily investment results, before fees and expenses, that correspond to three times the inverse (-3x) of the daily performance of the S&P MidCap 400® Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index is a measure of mid-size company U.S. stock market performance. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.3752	2020-10-11 23:38:31.021426
SMHI	SEACOR Marine Holdings Inc.	Seacor Marine Holdings Inc provides marine and support transportation services to offshore oil and gas exploration, development and production facilities.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{SMHI}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:11.395568	2020-10-11 23:38:31.023007
SMI	Semiconductor Manufacturing International Corporation ADR	Semiconductor Manufacturing International Corp is engaged in the computer-aided design, manufacturing, testing, packaging, and trading of integrated circuits and other semiconductor services. It also designs and manufactures semiconductor masks.	{Technology,Semiconductors}	{TSEM,AMKR,TSM,IBM,UMC,XLK}	Semiconductors	Technology	New York Stock Exchange	\N	2020-10-11 23:12:11.40263	2020-10-11 23:38:31.02423
SMMD	iShares Trust Russell 2500	The investment seeks to track the investment results of the Russell 2500TM Index.\n The fund generally will invest at least 90% of its assets in the component securities, including indirect investments through the underlying fund, of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The underlying index measures the performance of the small- to mid-capitalization sector of the U.S. equity market, as defined by FTSE Russell.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:11.414368	2020-10-11 23:38:31.028166
SMPL	The Simply Good Foods Company	The Simply Good Foods Co is a developer, marketer and seller of branded nutritional foods and snacking products. Its portfolio consists primarily of nutrition bars, ready-to-drink shakes, snacks and confectionery products.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{SANW,TYHT,SMPL}	Consumer Packaged Goods	Consumer Defensive	NASDAQ Capital Market	\N	2020-10-11 23:12:11.429238	2020-10-11 23:38:31.028166
SOFO	Sonic Foundry Inc.	Sonic Foundry Inc is engaged in the business of providing enterprise solutions and services for the Web communications market.	{Technology,"Software - Application","Application Software"}	{CSCO,CTXS,ADBE,TYPE,DLB,QUMU,GVP,XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:11.523678	2020-10-11 23:38:31.028166
SIMO	Silicon Motion Technology Corporation	Silicon Motion Technology Corp designs, develops and supplies NAND flash controllers ICs for solid state storage devices and SSD controllers. It also supplies specialized high-performance hyperscale datacenter and industrial SSD solutions.	{Technology,Semiconductors}	{INTC,MRVL,TSM,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:11.213544	2020-10-11 23:38:31.028166
SIZ	AGFiQ U.S. Market Neutral Size Fund	The investment seeks performance results that correspond to the price and yield performance, before fees and expenses, of the Dow Jones U.S. Thematic Market Neutral Size Index.\n The fund invests at least 80% of its net assets (plus borrowing for investment purposes) in common stock of the long position in the underlying index and sells short at least 80% of the short positions in the underlying index. As such, it identifies long and short securities positions of approximately equal dollar amounts. In choosing to track a market neutral index, the fund seeks to limit the effects of general market movements on the fund.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.230846	2020-10-11 23:38:31.023007
SJW	SJW Group (DE)	SJW Group along with its subsidiaries is engaged in the production, purchase, storage, purification, distribution, wholesale, and retail sale of water. Its operating business segments are Water Utility Services and Real Estate Investment.	{Utilities,"Utilities - Regulated Water","Utilities - Regulated"}	{YORW,ARTNA,CTWS,MSEX,CDZI,PCYO,CWT,AWR,WTR,AWK,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:11.25582	2020-10-11 23:38:31.023007
SKYY	First Trust ISE Cloud Computing Index Fund	The investment seeks investment results that correspond generally to the price and yield, before the fund's fees and expenses, of an equity index called the ISE Cloud ComputingTM Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks or in depositary receipts representing securities in the index. The index is designed to provide a benchmark for investors interested in tracking companies actively involved in the cloud computing industry.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:11.282186	2020-10-11 23:38:31.023007
SLQD	iShares 0-5 Year Investment Grade Corporate Bond E	The investment seeks to track the investment results of the Markit iBoxx® USD Liquid Investment Grade 0-5 Index composed of U.S. dollar-denominated, investment-grade corporate bonds with remaining maturities of less than five years.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The index is designed to reflect the performance of U.S. dollar-denominated investment-grade corporate debt.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:11.322454	2020-10-11 23:38:31.028166
SMCP	AlphaMark Small Cap Growth ETF	The investment seeks long-term growth of capital.\n Under normal circumstances, the fund will invest at least 80% of its net assets (plus any borrowings for investment purposes) in equity securities of small cap companies listed on a U.S. exchange. The adviser defines "equity securities" to include common and preferred stock, American Depositary Receipts ("ADRs"), real estate investment trusts ("REITs"), and exchange-traded funds ("ETFs"). The fund may invest up to 30% of its net assets in foreign equity securities of small cap companies traded on a U.S. exchange as ADRs, which may include companies in emerging markets.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:11.369108	2020-10-11 23:38:31.023428
SMIT	Schmitt Industries Inc.	Schmitt Industries Inc is a U.S based company engaged in designing, manufacturing and selling test and measurement products for two business segments: The Balancer segment and the Measurement segment.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{FLIR,CGNX,ELSE,HURC,AME,CAT,EMR,GE,SWK,XLK}	Computer Hardware	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:11.399425	2020-10-11 23:38:31.024353
SMMU	Short Term Municipal Bond Active Exchange-Traded Fund	The investment seeks attractive tax-exempt income, consistent with preservation of capital.\n The fund invests at least 80% of its assets in a diversified portfolio of debt securities whose interest is, in the opinion of bond counsel for the issuer at the time of issuance, exempt from federal income tax ("Municipal Bonds"). Municipal Bonds generally are issued by or on behalf of states and local governments and their agencies, authorities and other instrumentalities. It may only invest in U.S. dollar-denominated investment grade debt securities, rated Baa or higher.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.422885	2020-10-11 23:38:31.024353
SMRT	Stein Mart Inc.	Stein Mart Inc operates several hundred department stores in the United States. It is engaged in providing fashion merchandise, service and presentation of a department or specialty store at prices competitive with off-price retail chains.	{"Consumer Cyclical","Apparel Stores","Retail - Apparel & Specialty"}	{URBN,ROST,CTRN,AEO,BKE,GPS,JWN,TJX,SSI,TPHS}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:11.436602	2020-10-11 23:38:31.02582
SNDX	Syndax Pharmaceuticals Inc.	Syndax Pharmaceuticals Inc is a clinical stage biopharmaceutical company focused on developing a pipeline of combination therapies in multiple cancer indications.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:11.464228	2020-10-11 23:38:31.02423
SIRI	Sirius XM Holdings Inc.	Sirius XM Holdings Inc is a radio company. It offers commercial-free music, sports and live events, news and comedy, exclusive talk and entertainment, and a range of Latin commercial-free music and sports.	{"Consumer Cyclical","Broadcasting - Radio",Entertainment}	{P,TRCO,EMMS,UONE}	Entertainment	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:11.223761	2020-10-11 23:38:31.021426
SKIS	Peak Resorts Inc.	Peak Resorts Inc through its subsidiaries owns and operates day ski and overnight drive ski resorts. The Company operates 14 ski resorts located in the Northeast and Midwest.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{MTN,MCS,WYND,BEL}	Travel & Leisure	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:11.256917	2020-10-11 23:38:31.025871
SLAB	Silicon Laboratories Inc.	Silicon Laboratories Inc is a leading provider of silicon, software and system solutions for the Internet of Things, Internet infrastructure, industrial control, consumer and automotive markets.	{Technology,Semiconductors}	{MCHP,CY,IDTI,MXIM,SMTC,TXN,ADI,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:11.282156	2020-10-11 23:38:31.028166
SLGL	Sol-Gel Technologies Ltd.	Sol-Gel Technologies Ltd is a clinical-stage specialty pharmaceutical company. It is engaged in identifying, developing and commercializing branded and generic topical drug products for the treatment of skin diseases.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:11.29853	2020-10-11 23:38:31.02582
SLP	Simulations Plus Inc.	Simulations Plus Inc is a developer of drug discovery and development software for mechanistic modelling, simulation and machine learning based technologies. It also explores the application of machine learning technologies in other industries.	{Technology,"Health Information Services","Application Software"}	{MDSO,MDRX,XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:11.320471	2020-10-11 23:38:31.025871
SMB	VanEck Vectors AMT-Free Short Municipal Index	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the Bloomberg Barclays AMT-Free Short Continuous Municipal Index.\n The fund normally invests at least 80% of its total assets in fixed income securities that comprise the index. The index is comprised of publicly traded municipal bonds that cover the U.S. dollar denominated short-term tax-exempt bond market.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.354741	2020-10-11 23:38:31.024353
SMEZ	SPDR EURO STOXX Small Cap	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the EURO STOXX Small Index that tracks the performance of small capitalization Eurozone equity securities.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index and in depositary receipts (including ADRs or GDRs) based on securities comprising the index. The index is designed to provide a representation of small companies across the Eurozone. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.381491	2020-10-11 23:38:31.024353
SMMT	Summit Therapeutics plc	Summit Therapeutics PLC is a biopharmaceutical company focused on the discovery, development and commercialization of novel medicines for indications for which there are no existing or only inadequate therapies.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{SRPT,ACER,XLV}	Drug Manufacturers	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:11.421504	2020-10-11 23:38:31.02423
SMTS	Sierra Metals Inc.	Sierra Metals Inc is a precious and base metals producer in Latin America. The company acquires, explores, extracts, and produces mineral concentrates consisting of silver, copper, lead, zinc and gold in Mexico and Peru.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{PAAS}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:11.440668	2020-10-11 23:38:31.023428
SN	Sanchez Energy Corporation	Sanchez Energy Corp is an independent exploration and production company. It is engaged in exploration, acquisition, and development of unconventional oil and natural gas resources in the onshore U.S. Gulf Coast and others.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{CRZO,CHK,EOG,MRO,SM,HK}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:11.444208	2020-10-11 23:38:31.031328
SNDE	Sundance Energy Australia Limited	Sundance Energy Australia Ltd is an onshore oil and gas company engaged in the acquisition & development of repeatable resource plays in south Texas targeting the Eagle Ford basin.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{NOG,CLR,OAS,REN,BCEI,ESTE,WLL}	Oil & Gas - E&P	Energy	NASDAQ Global Market	\N	2020-10-11 23:12:11.459879	2020-10-11 23:38:31.024353
SOJB	Southern Company (The) Series 2016A 5.25% Junior Subordinated Notes due October 1 2076		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:11.545699	2020-10-11 23:38:31.021426
SKM	SK Telecom Co. Ltd.	SK Telecom Co Ltd provides wireless telecommunications services. Its services include wireless voice and data transmission services, sales of wireless devices, fixed-line telephone services, broadband Internet services, e-commerce services and others.	{"Communication Services","Telecom Services"}	{TLK,KT,PHI}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:11.259487	2020-10-11 23:38:31.02423
SLB	Schlumberger N.V.	Schlumberger Ltd is a supplier of products and services to the oil and gas industry. It provides technology for reservoir characterization, drilling, production, and processing to the oil and gas industry.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{CKH,NOV,HAL,OII,AROC,BHGE}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:11.28627	2020-10-11 23:38:31.031328
SLGN	Silgan Holdings Inc.	Silgan Holdings Inc is a manufacturer of rigid packaging for shelf-stable food and other consumer goods products. Its products include steel and aluminum containers, metal, composite and plastic closures, custom designed plastic containers and others.	{"Consumer Cyclical","Packaging & Containers"}	{ATR,GEF.B,BLL,SON,CCK,BERY}	Packaging & Containers	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:11.303244	2020-10-11 23:38:31.024353
SM	SM Energy Company	SM Energy Company is an energy company engaged in the acquisition, exploration, development, and production of oil, gas, and natural gas liquids in onshore North America.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{CRZO,CHK,NFX,DNR,APC,RRC,XEC,SN}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:11.353527	2020-10-11 23:38:31.028166
SMG	Scotts Miracle-Gro Company (The)	The Scotts Miracle Gro Co is operative in the agricultural industry. It manufactures and sells dry, granular slow-release lawn fertilizers, combination lawn fertilizer and control products.	{"Basic Materials","Agricultural Inputs",Agriculture}	{CENT,CF,CGA,MBII}	Agriculture	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:11.381417	2020-10-11 23:38:31.028166
SMLV	SPDR SSGA US Small Cap Low Volatility Index	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the SSGA US Small Cap Low Volatility Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index is designed to measure the performance of the stocks of U.S. small capitalization companies that exhibit low volatility. Volatility is a statistical measurement of the magnitude of movements in a stock's price over time. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.407365	2020-10-11 23:38:31.023007
SMN	ProShares UltraShort Basic Materials	The investment seeks daily investment results, before fees and expenses, that correspond to two times the inverse (-2x) of the daily performance of the Dow Jones U.S. Basic MaterialsSM Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index seeks to measure the performance of certain companies in the basic materials sector of the U.S. equity market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.424588	2020-10-11 23:38:31.023007
SNA	Snap-On Incorporated	Snap-on Inc manufactures and markets tools, equipment, diagnostics, repair information and systems solutions for professional users. The product range consists of hand and power tools, tool storage, fuel and engine systems, and hand-held diagnostics.	{Industrials,"Tools & Accessories","Industrial Products"}	{LECO,PFIN,ALG,NWL,DHR,TTC,SWK}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:11.447315	2020-10-11 23:38:31.023007
SMTC	Semtech Corporation	Semtech Corp is a provider of analog and mixed-signal semiconductor products. These products are found in enterprise computing, communications, high-end consumer, industrial, and other end markets.	{Technology,Semiconductors}	{MCHP,MXIM,SLAB,ON,MPWR,IPHI,TXN,ADI,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:11.440439	2020-10-11 23:38:31.024353
SNDR	Schneider National Inc.	Schneider National Inc. provides transportation and logistics services. The company delivers broad portfolio of premier truckload, intermodal and logistics solutions and operating for-hire trucking fleets in North America.	{Industrials,Trucking,"Transportation & Logistics"}	{ULH,YRCW}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:11.463572	2020-10-11 23:38:31.025871
SNE	Sony Corporation	Sony Corp is engaged in the development, design, manufacture, and sale of various kinds of electronic equipment, instruments and devices for consumer, professional and industrial markets as well as game consoles and software.	{Technology,"Consumer Electronics","Computer Hardware"}	{MSFT,CAJ,PHG,DLB,FOXA,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:11.464904	2020-10-11 23:38:31.02582
SNN	Smith & Nephew SNATS Inc.	Smith & Nephew PLC designs, manufactures, and markets orthopedic devices and wound-care solutions. Its products include knee replacements, hip implants, nails, fixation devices, arthroscopy tools and others.	{Healthcare,"Medical Devices"}	{CNMD,BSX,JNJ,SYK,EW,ZBH,XLV}	Medical Devices	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:11.492751	2020-10-11 23:38:31.02582
SLIM	The Obesity ETF	The investment seeks investment results that correspond generally to the performance, before fees and expenses, of an index which is designed to track the performance of companies globally that are positioned to profit from servicing the obese, including biotechnology, pharmaceutical, health care and medical device companies whose business is focused on obesity and obesity related disease and companies focused on weight loss programs, weight loss supplements, or plus sized apparel.\n The fund invests at least 80% of its net assets in the stocks that comprise the Solactive Obesity Index. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:11.307513	2020-10-11 23:38:31.023007
SLX	VanEck Vectors Steel	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the NYSE Arca Steel Index.\n The fund normally invests at least 80% of its total assets in common stocks and depositary receipts of companies involved in the steel sector. Such companies may include small- and medium-capitalization companies and foreign and emerging market issuers. It may concentrate its investments in a particular industry or group of industries to the extent that the Steel Index concentrates in an industry or group of industries. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.340711	2020-10-11 23:38:31.028166
SMBC	Southern Missouri Bancorp Inc.	Southern Missouri Bancorp Inc is a holding company for Southern Missouri Savings Bank. It is engaged in providing range of banking and financial services to individuals and corporate customers in its market areas.	{"Financial Services","Savings & Cooperative Banks",Banks}	{HMNF,GFED,LARK,CASH,CFFN}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:11.367928	2020-10-11 23:38:31.02423
SMH	VanEck Vectors Semiconductor	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the MVIS® US Listed Semiconductor 25 Index.\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The index includes common stocks and depositary receipts of U.S. exchange-listed companies in the semiconductor sector. Such companies may include medium-capitalization companies and foreign companies that are listed on a U.S. exchange. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.396556	2020-10-11 23:38:31.023428
SMM	Salient Midstream of Beneficial Interest	Salient Midstream & MLP Fund, along with its subsidiaries provides asset management and advisory services. The Fund's objective is to provide a high level of total return with an emphasis on making quarterly cash distributions to its common shareholders.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:11.412029	2020-10-11 23:38:31.025871
SMP	Standard Motor Products Inc.	Standard Motor Products Inc is a manufacturer and distributor of replacement parts for motor vehicles in the automotive aftermarket industry. The company is organized into two major operating segments: Engine Management and Temperature Control.	{"Consumer Cyclical","Auto Parts",Autos}	{DORM,BWA,CMI,MOD,TEN,DAN,APTV,VC}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:11.425323	2020-10-11 23:38:31.021426
SNCR	Synchronoss Technologies Inc.	Synchronoss Technologies Inc is a software company providing technologies and services for the mobile transformation of business. It provides personal cloud, secure-mobility, identity management and scalable messaging platforms, products and solutions.	{Technology,"Software - Application","Application Software"}	{CSGS,EVOL,CTL,VZ,T,DOX,APPS,GEC,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:11.453809	2020-10-11 23:38:31.021426
SND	Smart Sand Inc.	Smart Sand Inc is a producer of Northern White raw frac sand. The Company is engaged in the excavation, processing and sale of industrial sand, or proppant, for use in hydraulic fracturing operations for the oil and gas industry.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{SND}	Metals & Mining	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:12:11.458675	2020-10-11 23:38:31.023007
SNHNI	Senior Housing Properties Trust 5.625% Senior Notes due 2042	Senior Housing Properties Trust is a real estate investment trust. It owns and operates independent living and assisted living communities, continuing care retirement communities and nursing homes.	{"Real Estate","REIT - Healthcare Facilities",REITs}	{}	REITs	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:11.481789	2020-10-11 23:38:31.023428
SNLN	Highland/iBoxx Senior Loan ETF	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the price and yield performance of the Markit iBoxx USD Liquid Leveraged Loan Index.\n The fund will, under normal circumstances, invest at least 80% of its assets (the "80% basket") in component securities of the underlying index. The underlying index is a subset of the Markit iBoxx USD Leveraged Loan Index. "Leveraged Loans" are loans to companies that typically already have a high amount of debt and are often characterized by lower credit ratings or higher interest rates. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:11.485286	2020-10-11 23:38:31.023007
SLVO	Credit Suisse X-Links Silver Call ETN IOPV	The investment seeks a return linked to the performance of the Credit Suisse NASDAQ Silver FLOWSTM 106 Index.\n The index measures the return of a “covered call” strategy on the shares of the iShares® Silver Trust (the “SLV Shares”) by reflecting changes in the price of the SLV Shares and the notional option premiums received from the notional sale of monthly call options on the SLV Shares less notional transaction costs incurred in connection with the covered call strategy.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:11.33881	2020-10-11 23:38:31.023007
SMCI	Super Micro Computer Inc.	Super Micro Computer Inc is a high-performance server technology service provider. It offers server, storage, blade and workstations to full racks, networking devices, and server management software across US, Europe and Asia.	{Technology,"Communication Equipment"}	{XLK}	Communication Equipment	Technology	OTC Pink	\N	2020-10-11 23:12:11.368083	2020-10-11 23:38:31.023007
SMHD	ETRACS Monthly Pay 2xLeveraged US Small Cap High Dividend ETN due February 6 2045	The investment seeks to provides a monthly compounded two times leveraged long exposure to the performance of the index, reduced by the accrued fees.\nThe fund is a series of Monthly Pay 2xLeveraged Exchange Traded Access Securities (ETRACS) linked to the performance of the price return version of the Solactive US Small Cap High Dividend Index. The index is designed to measure the performance of 100 relatively small capitalization, dividend yielding index constituent Securities selected from a universe of qualifying U.S. listed stocks.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.39741	2020-10-11 23:38:31.028166
SMMF	Summit Financial Group Inc.	Summit Financial Group Inc provides insurance brokerage services to individuals and businesses covering corporate and personal property and casualty insurance products, as well as group health and life insurance products and consulting services.	{"Financial Services","Banks - Regional - US",Banks}	{PEBO,AMNB,PFBI,TOWN,FCBC,CHCO,HBAN,FNB,RF}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:11.421067	2020-10-11 23:38:31.028166
SNAP	Snap Inc. Class A	Snap Inc is a camera application developer. Its main product is Snapchat, a camera application created to help people communicate through short videos and images. It also offers Spectacles, which are camera-enabled sunglasses.	{Technology,"Internet Content & Information","Online Media"}	{XLK}	Online Media	Technology	New York Stock Exchange	\N	2020-10-11 23:12:11.453289	2020-10-11 23:38:31.02423
SNBR	Sleep Number Corporation	Sleep Number Corp offer consumers with individualized sleep solutions and services, which include a complete line of SLEEP NUMBER beds and bedding accessories.	{"Consumer Cyclical","Home Furnishings & Fixtures","Manufacturing - Apparel & Furniture"}	{FLXS,HOFT,ETH,HVT,LZB,LEG,TPX,NVFY,RH}	Manufacturing - Apparel & Furniture	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:11.451726	2020-10-11 23:38:31.028166
SNGXW	Soligenix Inc. Warrant		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:11.475956	2020-10-11 23:38:31.028166
SNGX	Soligenix Inc.	Soligenix Inc focuses on developing and commercializing products to treat rare diseases where there is an unmet medical need.	{Healthcare,Biotechnology}	{ATHX,JNJ,PFE,SNY,EBS,TEVA,CBLI,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:11.470467	2020-10-11 23:38:31.031328
SNMP	Sanchez Midstream Partners LP	Sanchez Midstream Partners LP is a United States based company, focused on the acquisition, development, ownership and operation of midstream and other energy producing assets.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{PHX,MCF,HK}	Oil & Gas - Midstream	Energy	NYSE American	\N	2020-10-11 23:12:11.485483	2020-10-11 23:38:31.025871
SNOAW	Sonoma Pharmaceuticals (Warrant)	Sonoma Pharmaceuticals, Inc. engages in identifying, developing, and commercializing therapies for chronic skin conditions. It offers solutions in dermatology, animal healthcare, and advanced tissue care product areas. Its products target skin diseases including acne, atopic dermatitis, scarring, infections, itch, pain, and harmful inflammatory responses. The company was founded by Hojabr Alimi and Linda Alimi in April 1999 and is headquartered in Petaluma, CA.	{}	{}	Pharmaceuticals Other	Health Technology	https://api.robinhood.com/markets/XNAS/	\N	2020-10-11 23:12:11.501216	2020-10-11 23:38:31.023428
SNR	New Senior Investment Group Inc.	New Senior Investment Group Inc is a real estate company which owns and operates a portfolio of senior housing properties including independent living properties, assisted living and memory care properties, and care retirement community properties.	{"Real Estate","REIT - Diversified",REITs}	{UBP,SPG,INN,CHMI}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:11.506116	2020-10-11 23:38:31.025871
SODA	SodaStream International Ltd.	SodaStream International Ltd manufactures and sells home beverage carbonation systems, which enable consumers to instantly transform tap water into carbonated soft drinks and sparkling water. The group sells its products in retail stores around the world.	{"Consumer Cyclical","Home Furnishings & Fixtures","Manufacturing - Apparel & Furniture"}	{COT,BNSO,IRBT,NC,WHR}	Manufacturing - Apparel & Furniture	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:11.520374	2020-10-11 23:38:31.02423
SLS	SELLAS Life Sciences Group Inc.	SELLAS Life Sciences Group Inc is a biopharmaceutical company. The company is engaged in the development of novel cancer immunotherapies for a broad range of cancer indications. Its pipeline products are Galinpepimut-S and NeuVax.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:11.333294	2020-10-11 23:38:31.02582
SLYV	SPDR S&P 600 Small Cap Value ETF (based on S&P SmallCap Value Index--symbol--CVK	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the S&P SmallCap 600 Value Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index measures the performance of the small-capitalization value segment of the U.S. equity market. It may purchase a subset of the securities in the index in an effort to hold a portfolio of securities with generally the same risk and return characteristics of the index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.350739	2020-10-11 23:38:31.02582
SMFG	Sumitomo Mitsui Financial Group Inc Unsponsored American Depositary Shares (Japan)	Sumitomo Mitsui Financial Group Inc and its subsidiaries offer a diverse range of financial services, including commercial banking, leasing, securities, consumer finance and other services.	{"Financial Services","Banks - Global",Banks}	{BCS,STT,IBN,SHG,MUFG,MFG,C}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:11.381851	2020-10-11 23:38:31.025871
SMLP	Summit Midstream Partners LP Representing Limited Partner Interests	Summit Midstream Partners LP is engaged in developing, owning and operating midstream energy infrastructure assets that are strategically located in the core producing areas of unconventional resource basins,shale formations in the continental USA.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{WMB,DCP,TRGP,AMID,TCP,ENLK}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:11.406149	2020-10-11 23:38:31.023428
SMTX	SMTC Corporation	SMTC Corp is a provider of end-to-end electronics manufacturing services, including product design & sustaining engineering services, printed circuit board assembly, systems integration and comprehensive testing services.	{Technology,"Contract Manufacturers","Computer Hardware"}	{CLS,FLEX,SGMA,KBAL,PLXS,JBL,BHE,SPA,XLK}	Computer Hardware	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:11.43986	2020-10-11 23:38:31.028166
SNFCA	Security National Financial Corporation Class A Common Stock	Security National Financial Corp and its subsidiaries operate in three business segments namely life insurance, cemetery and mortuary, and mortgage.	{"Financial Services","Credit Services"}	{MFC,AFL,RGA,CSV}	Credit Services	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:11.470526	2020-10-11 23:38:31.023007
SNES	SenesTech Inc.	SenesTech Inc is a biotech platform and research company. The Company is engaged in developing technology for managing animal pest populations through fertility control.	{"Basic Materials","Agricultural Inputs",Agriculture}	{PDLI,PIRS}	Agriculture	Basic Materials	NASDAQ Capital Market	\N	2020-10-11 23:12:11.466529	2020-10-11 23:38:31.028166
SNOA	Sonoma Pharmaceuticals Inc.	Sonoma Pharmaceuticals Inc is engaged in developing and commercializing differentiated therapies to patients living with chronic skin conditions. It develops solutions in to treat skin conditions, such as acne, atopic dermatitis, and scarring.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{XLV}	Drug Manufacturers	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:11.494569	2020-10-11 23:38:31.02423
SNNA	Sienna Biopharmaceuticals Inc.	Sienna Biopharmaceuticals Inc is a clinical-stage biopharmaceutical company focused on bringing innovations in biotechnology to the discovery, development and commercialization of targeted, topical products in medical dermatology and aesthetics.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:11.492251	2020-10-11 23:38:31.028166
SNSS	Sunesis Pharmaceuticals Inc.	Sunesis Pharmaceuticals Inc is a biopharmaceutical company engaged in the development and commercialization of new oncology therapeutics for the treatment of solid and hematologic cancers.	{Healthcare,Biotechnology}	{CELG,SGEN,SPPI,BIIB,STML,FBIO,CYCC,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:11.50912	2020-10-11 23:38:31.028166
SNY	Sanofi American Depositary Shares (Each repstg one-half of one)	Sanofi SA is healthcare company which is engaged in the research, development, manufacture and marketing of therapeutic solutions. It business activities include operations of specialty care, Established Prescription Products and vaccines operations.	{Healthcare,"Drug Manufacturers - Major","Drug Manufacturers"}	{GSK,NVS,BMY,JNJ,LLY,PFE,MRK,TEVA,XLV}	Drug Manufacturers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:11.518695	2020-10-11 23:38:31.02582
SOCL	Global X Social Media ETF	The investment seeks to provide investment results that correspond generally to the price and yield performance, before fees and expenses, of the Solactive Social Media Total Return Index.\n The fund will invest at least 80% of its total assets in the securities of the underlying index and in American Depositary Receipts and Global Depositary Receipts based on the securities in the underlying index. The underlying index tracks the equity performance of the largest and most liquid companies involved in the social media industry, including companies that provide social networking, file sharing, and other web-based media applications. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:11.518341	2020-10-11 23:38:31.021426
RELX	RELX PLC PLC American Depositary Shares (Each representing One)	RELX PLC is an information content provider in the United Kingdom. Its published content is used by various organisations such as scientific, technical, and medical. The company generates its revenue from Scientific, Technical and Medical segment.	{"Consumer Cyclical",Publishing}	{TRI,JW.A,PSO,NWSA,FOXA,SPGI}	Publishing	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:10.19604	2020-10-11 23:38:31.023428
RFAP	First Trust RiverFront Dynamic Asia Pacific ETF	The investment seeks capital appreciation.\n The fund invests at least 80% of its net assets (including investment borrowings) in a portfolio of equity securities of "Asian Pacific companies", through investments in common stock, depositary receipts, and common and preferred shares of REITs, and forward foreign currency exchange contracts and currency spot transactions used to hedge the fund's exposure to the currencies in which the equity securities of such Asian Pacific companies are denominated (each, an "Asian Pacific currency" and, collectively, the "Asian Pacific currencies"). It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:10.251753	2020-10-11 23:38:31.028166
RFI	Cohen & Steers Total Return Realty Fund Inc.	Cohen & Steers Total Return Realty Fund is a a diversified, closed-end management investment company. Its investment objective is to achieve a high total return through investment in real estate securities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:10.272234	2020-10-11 23:38:31.024353
RGLB	Oppenheimer Global Revenue	The investment seeks to provide investment results that correspond generally, before fees and expenses, to the performance of the OFI Revenue Weighted Global Index (the "underlying index").\n The fund will normally invest at least 80% of its net assets, plus any borrowings for investment purposes, in the securities of companies included in the underlying index. The underlying index is constructed using a rules-based methodology that re-weights the constituent securities of the MSCI ACWI Index (the "parent index") according to the revenue earned by the companies in the parent index, subject to a maximum 5% per company weighting.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:10.291667	2020-10-11 23:38:31.02582
RHS	Invesco S&P 500 Equal Weight Consumer Staples	The investment seeks to track the investment results (before fees and expenses) of the S&P 500® Equal Weight Consumer Staples Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities that comprise the underlying index. The underlying index is an equal-weighted version of the S&P 500® Consumer Staples Index. Strictly in accordance with its guidelines and mandated procedures, the index provider compiles, maintains and calculates the underlying index, which is comprised of common stocks of companies in the consumer staples sector of the S&P 500® Index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.31988	2020-10-11 23:38:31.023007
RILYG	B. Riley Financial Inc. 7.25% Senior Notes due 2027		{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:10.358152	2020-10-11 23:38:31.021426
RJI	AB Svensk Ekportkredit (Swedish Export Credit Corporation) ELEMENTS Linked to the Rogers International Commodity Index - Total Return Structured Product	The investment seeks to replicate, net of expenses, the Rogers International Commodity Index – Total Return index.\n The index represents the value of a basket of 35 commodity futures contracts.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.379971	2020-10-11 23:38:31.028166
RM	Regional Management Corp.	Regional Management Corp is a diversified specialty consumer finance company. The firm provides a range of loan products to customers with limited access to consumer credit from banks, thrifts, credit card companies, and other traditional lenders.	{"Financial Services","Credit Services"}	{WRLD,CIT,OMF}	Credit Services	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:10.410975	2020-10-11 23:38:31.028166
RNDV	First Trust US Equity Dividend Select ETF	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the Nasdaq Riskalyze US Large Cap Select Dividend Index (the "index").\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in the equity securities that comprise the index. The index is designed to select dividend-paying US securities that are included in the Nasdaq US 500 Large Cap Index (the "base index"). The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:10.436481	2020-10-11 23:38:31.023007
RNST	Renasant Corporation	Renasant Corp is a part of the financial services sector. Its business involves the provision of community banking services, which includes checking and savings accounts, business and personal loans and interim construction loans.	{"Financial Services","Banks - Regional - US",Banks}	{HWC,TRMK,CIZN,PNFP,FBMS,CFR,FHN,BXS,PFBX}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:10.463881	2020-10-11 23:38:31.031328
ROK	Rockwell Automation Inc.	Rockwell Automation Inc engages in providing industrial automation and information solutions to make its customers more productive and the world more sustainable. It operates in two segments: architecture and software and control products and solutions.	{Industrials,"Diversified Industrials","Industrial Products"}	{MSFT,FLIR,AZPN,PRCP,HOLI,AME,EMR,HON,PH,ABB}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:10.485445	2020-10-11 23:38:31.031328
ROX	Castle Brands Inc.	Castle Brands Inc develops & markets alcoholic beverages. Its product categories are rum, whiskey, liqueurs, vodka, tequila and ginger beer. It reports its operations in two geographic areas: International and United States.	{"Consumer Defensive","Beverages - Wineries & Distilleries","Beverages - Alcoholic"}	{DEO,WVVI,BF.B,STZ}	Beverages - Alcoholic	Consumer Defensive	NYSE American	\N	2020-10-11 23:12:10.509226	2020-10-11 23:38:31.028166
SONC	Sonic Corp.	Sonic Corp is a fast food restaurant company. The firm operates and franchises drive-in restaurants throughout the United States. It serves fast food such as burgers, hot dogs, burritos, and snacks from more than 3,000 locations.	{"Consumer Cyclical",Restaurants}	{JACK,DENN,TAST,MCD,GTIM,WEN}	Restaurants	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:11.556266	2020-10-11 23:38:31.02582
SOXX	iShares PHLX SOX Semiconductor Sector Index Fund	The investment seeks to track the investment results of the PHLX Semiconductor Sector Index composed of U.S. equities in the semiconductor sector.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index measures the performance of U.S.-traded securities of companies engaged in the semiconductor business. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:11.572379	2020-10-11 23:38:31.028166
SPGI	S&P Global Inc.	S&P Global Inc, together with its subsidiaries is a benchmarks and ratings, analytics, data and research provider serving the capital, commodities and commercial markets.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{TRI,MORN,FDS,JW.A,MCO,DNB,MSCI,PSO,RRD}	Brokers & Exchanges	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:11.615053	2020-10-11 23:38:31.025871
SPIB	SPDR Portfolio Intermediate Term Corporate Bond	The investment seeks to provide investment results that correspond generally to the price and yield performance of the Bloomberg Barclays U.S. Intermediate Corporate Bond Index.\n The fund invests substantially all, but at least 80%, of its total assets in the securities comprising the index or in securities that the Adviser determines have economic characteristics that are substantially identical to the economic characteristics of the securities that comprise the index. The index is designed to measure the performance of U.S. corporate bonds that have a maturity of greater than or equal to 1 year and less than 10 years. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.633774	2020-10-11 23:38:31.031328
SPLP	Steel Partners Holdings LP LTD PARTNERSHIP UNIT	Steel Partner Holdings LP operates businesses through subsidiaries in industrial products, energy, defense, supply chain and logistics, banking, food products, oilfield services, sports, training, education and entertainment and lifestyle industries.	{Industrials,Conglomerates}	{ROLL,SSD,B,MLI,SIF,TKR,CIX,CVR}	Conglomerates	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:11.652539	2020-10-11 23:38:31.02582
SPNE	SeaSpine Holdings Corporation	SeaSpine Holdings Corp is a medical technology company. It focuses on design, development, and commercialization of surgical solutions for the treatment of patients suffering from spinal disorders.	{Healthcare,"Medical Devices"}	{NUVA,XLV}	Medical Devices	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:11.669954	2020-10-11 23:38:31.023007
SPSB	SPDR Portfolio Short Term Corporate Bond	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the price and yield performance of the Bloomberg Barclays U.S. 1-3 Year Corporate Bond Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index or in securities that the Adviser determines have economic characteristics that are substantially identical to the economic characteristics of the securities that comprise the index. The index is designed to measure the performance of the short term U.S. corporate bond market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.689918	2020-10-11 23:38:31.031328
SPTM	SPDR Portfolio Total Stock Market	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the SSGA Total Stock Market Index that tracks a broad universe of exchange traded U.S. equity securities.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. It may invest in equity securities that are not included in the index (including common stock, preferred stock, depositary receipts and shares of other investment companies), cash and cash equivalents or money market instruments. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.707136	2020-10-11 23:38:31.02423
SPXC	SPX Corporation	SPX Corp is an industrial goods supplier. It offers miscellaneous products to the food & beverage, oil & gas and energy companies.	{Industrials,"Diversified Industrials","Industrial Products"}	{FLS,DOV,EMR,GE,IEX,KMT,CIR,CFX,GHM,ITT}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:11.726665	2020-10-11 23:38:31.028166
SPXT	ProShares S&P 500 Ex-Technology	The investment seeks investment results, before fees and expenses, that track the performance of the S&P 500® Ex-Information Technology Index (the "index").\n Under normal circumstances, the fund will invest at least 80% of its total assets in component securities. The index and fund seek to provide exposure to the companies of the S&P 500® Index (the "S&P 500") with the exception of those companies included in the Information Technology Sector.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.741778	2020-10-11 23:38:31.023007
SON	Sonoco Products Company	Sonoco Products Co is a manufacturer of industrial and consumer packaging products and provides packaging services. It operates in Consumer Packaging, Paper and Industrial Converted Products, Display and Packaging, and Protective Solutions segments.	{"Consumer Cyclical","Packaging & Containers"}	{SLGN,SEE,BMS,GEF,MMM,CCK,HAS,WRK}	Packaging & Containers	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:11.547712	2020-10-11 23:38:31.028166
SPA	Sparton Corporation	Sparton Corp is a provider of design, development, and manufacturing services for complex electromechanical devices and engineered products. The Company serves the medical & biotechnology, military & aerospace and industrial & commercial markets.	{Technology,"Electronic Components","Computer Hardware"}	{SGMA,ALOG,KBAL,PLXS,BRKR,BIO,PKI,SANM,ITGR,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:11.579213	2020-10-11 23:38:31.021426
SPE	Special Opportunities Fund Inc	Special Opportunities Fund Inc is a United States based closed-end diversified management investment company. Its principal investment objective is to have the total return.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:11.602254	2020-10-11 23:38:31.02582
SPHD	Invesco S&P 500 High Dividend Low Volatility	The investment seeks to track the investment results (before fees and expenses) of the S&P 500® Low Volatility High Dividend Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in securities that comprise the underlying index. S&P Dow Jones Indices LLC compiles, maintains and calculates the underlying index, which is composed of 50 securities in the S&P 500® Index that historically have provided high dividend yields with lower volatility.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.621779	2020-10-11 23:38:31.02582
SPLG	SPDR Series Trust Portfolio Large Cap	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the SSGA Large Cap Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index is designed to measure the performance of the large-capitalization segment of the U.S. equity market. The fund may purchase a subset of the securities in the index in an effort to hold a portfolio of securities with generally the same risk and return characteristics of the index. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.648991	2020-10-11 23:38:31.02423
SPN	Superior Energy Services Inc.	Superior Energy Services Inc is a part of the oil & gas sector. Its core business is to provide equipment and services to cater to the companies involved in oil & gas drilling and exploration.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{NOV,SLB,HAL,RES,TTI,HLX,SCL,BHGE}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:11.669922	2020-10-11 23:38:31.02423
SPR	Spirit Aerosystems Holdings Inc.	Spirit AeroSystems Holdings Inc is an independent non-OEM aircraft parts designer and manufacturer of commercial aerostructures. The Company design, engineer, and manufactures fuselages,wing structures, nacelles, pylons, and flight control surfaces.	{Industrials,"Aerospace & Defense"}	{LMT,TGI,CVU,AIR,BA,GD,TXT,UTX,TDG,ERJ}	Aerospace & Defense	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:11.684165	2020-10-11 23:38:31.023428
SPTL	SPDR Portfolio Long Term Treasury	The investment seeks to provide investment results that correspond generally to the price and yield performance of the Bloomberg Barclays Long U.S. Treasury Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index or in securities that the adviser determines have economic characteristics that are substantially identical to the economic characteristics of the securities that comprise the index. The index is designed to measure the performance of public obligations of the U.S. Treasury that have a remaining maturity of 10 years or more. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.705389	2020-10-11 23:38:31.02582
SPXS	Direxion Daily S&P 500 Bear 3X	The investment seeks daily investment results, before fees and expenses, of 300% of the inverse (or opposite) of the daily performance of the S&P 500® Index.\n The fund, under normal circumstances, invests in swap agreements, futures contracts, short positions or other financial instruments that, in combination, provide inverse (opposite) or short leveraged exposure to the index equal to at least 80% of the fund's net assets (plus borrowing for investment purposes). The index is a float-adjusted, market capitalization-weighted index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.738336	2020-10-11 23:38:31.028166
SPYX	SPDR S&P 500 Fossil Fuel Reserves Free	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the S&P 500 Fossil Fuel Free Index.\n Normally, the fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. In addition, it may invest in equity securities that are not included in the index, cash and cash equivalents or money market instruments. The index is designed to measure the performance of companies in the S&P 500 Index that do not own fossil fuel reserves. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.76344	2020-10-11 23:38:31.031328
SOVB	Cambria Sovereign Bond	The investment seeks income and capital appreciation from investments in securities and instruments that provide exposure to sovereign and quasi-sovereign bonds.\n Under normal market conditions, at least 80% of the value of the fund's net assets (plus borrowings for investment purposes) will be invested in sovereign and quasi-sovereign bonds. The fund may invest up to 20% of its net assets in ETPs, including ETFs and ETNs, which invest in or provide exposure to sovereign and quasi-sovereign bonds, money market instruments or other high quality debt securities, cash or cash equivalents. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:11.566931	2020-10-11 23:38:31.023007
SPB	Spectrum Brands Holdings Inc.	HRG Group Inc is a consumer products company that manufactures consumer batteries, small appliances, global pet supplies, home and garden control products, personal care products, hardware and home improvement products and global auto care.	{"Consumer Defensive","Household & Personal Products","Consumer Packaged Goods","Consumer Cyclical","Home Furnishings & Fixtures","Manufacturing - Apparel & Furniture"}	{}	Manufacturing - Apparel & Furniture	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:11.587451	2020-10-11 23:38:31.023007
SPEM	SPDR Index Shares Fund Portfolio Emerging Markets	The investment seeks investment results that, before fees and expenses, correspond generally to the total return performance of the S&P Emerging BMI Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index and in depositary receipts based on securities comprising the index. The index is a market capitalization weighted index designed to define and measure the investable universe of publicly traded companies domiciled in emerging markets. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.602401	2020-10-11 23:38:31.023007
SPKE	Spark Energy Inc.	Spark Energy Inc provides independent retail energy services. It provides residential and commercial customers across the United States with an alternative choice for their natural gas and electricity.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{DTE,SO,SCG,DUK,XLU}	Utilities - Regulated	Utilities	Nasdaq Global Select	\N	2020-10-11 23:12:11.638674	2020-10-11 23:38:31.028166
SPLV	Invesco S&P 500 Low Volatility	The investment seeks to track the investment results (before fees and expenses) of the S&P 500® Low Volatility Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in common stocks that comprise the underlying index. Volatility is a statistical measurement of the magnitude of up and down asset price fluctuations (increases or decreases in a stock's price) over time. It generally invests in all of the securities comprising the underlying index in proportion to their weightings in the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.658987	2020-10-11 23:38:31.023428
SPOK	Spok Holdings Inc.	Spok Holdings Inc is a provider of critical communication solutions for enterprises to the healthcare, government, public safety and other industries. It offers paging services and selected software solutions in the United States and abroad.	{"Communication Services","Telecom Services"}	{SHEN,USM,VZ,T,TDS,TMUS}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:12:11.675812	2020-10-11 23:38:31.02582
SPSC	SPS Commerce Inc.	SPS Commerce Inc is a provider of cloud based supply chain management solutions, providing network integrations and comprehensive retail performance analytics to thousands of customers.	{Technology,"Software - Application","Application Software"}	{FICO,IBM,PCYG,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:11.694778	2020-10-11 23:38:31.021426
SPUU	Direxion Daily S&P 500 Bull 2X Shares	The investment seeks daily investment results, before fees and expenses, of 200% of the daily performance of the S&P 500® Index.\n The fund, under normal circumstances, invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, exchange-traded funds ("ETFs") that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index is a float-adjusted, market capitalization-weighted index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.716928	2020-10-11 23:38:31.025871
SPXE	ProShares S&P 500 Ex-Energy	The investment seeks investment results, before fees and expenses, that track the performance of the S&P 500® Ex-Energy Index (the "index").\n Under normal circumstances, the fund will invest at least 80% of its total assets in component securities. The index and fund seek to provide exposure to the companies of the S&P 500® Index (the "S&P 500®") with the exception of those companies included in the Energy Sector.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.734513	2020-10-11 23:38:31.02582
SPXX	Nuveen S&P 500 Dynamic Overwrite Fund	Nuveen S&P 500 Dynamic Overwrite Fund is a diversified closed-end management investment company. Its investment objective is to seek attractive total returns with less volatility than the S&P 500 Index.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:11.754932	2020-10-11 23:38:31.024353
SPYB	SPDR S&P 500 Buyback	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the S&P 500 Buyback Index that tracks the performance of publicly traded issuers that have a high buyback ratio.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index provides exposure to the 100 constituent companies in the S&P 500® with the highest buyback ratio in the last 12 months. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.75873	2020-10-11 23:38:31.028166
SOYB	Teucrium Soybean Fund ETV	The investment seeks to have the daily changes in percentage terms of the shares’ NAV reflect the daily changes in percentage terms of a weighted average of the closing settlement prices for three futures contracts for soybeans that are traded on the Chicago Board of Trade.\n The fund seeks to achieve its investment objective by investing under normal market conditions in Benchmark Component Futures Contracts or, in certain circumstances, in other Soybean Futures Contracts traded on the CBOT or on foreign exchanges.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.576119	2020-10-11 23:38:31.02582
SPCB	SuperCom Ltd.	SuperCom Ltd is a provider of traditional and digital identity solutions. The Company provides advanced M2M solutions, cyber security devices and solutions, secure payment solutions and connectivity products and solutions.	{Industrials,"Security & Protection Services","Consulting & Outsourcing"}	{MAGS,NSSC,MMM,INVE,IDN}	Consulting & Outsourcing	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:11.594601	2020-10-11 23:38:31.025871
SPFF	Global X SuperIncome Preferred	The investment seeks investment results that correspond generally to the price and yield performance, before fees and expenses, of the S&P Enhanced Yield North American Preferred Stock Index ("underlying index").\n The fund will invest at least 80% of its total assets in the securities of the underlying index and in American Depositary Receipts ("ADRs") and Global Depositary Receipts ("GDRs") based on the securities in the underlying index. The underlying index tracks the performance of the highest-yielding preferred securities in the United States and Canada. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.606578	2020-10-11 23:38:31.028166
SPKEP	Spark Energy Inc. 8.75% Series A Fixed-to-Floating Rate Cumulative Redeemable Perpetual Preferred Stock	Spark Energy Inc provides independent retail energy services. It provides residential and commercial customers across the United States with an alternative choice for their natural gas and electricity.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{XLU}	Utilities - Regulated	Utilities	Nasdaq Global Select	\N	2020-10-11 23:12:11.644886	2020-10-11 23:38:31.023007
SPMD	SPDR Portfolio Mid Cap	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the S&P 1000 Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index measures the performance of the small- and mid-capitalization segments of the U.S. equity market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.661461	2020-10-11 23:38:31.024353
SPPI	Spectrum Pharmaceuticals Inc.	Spectrum Pharmaceuticals Inc is a biotechnology company in the United States. It develops drugs to cure serious diseases such as chemotherapy-induced neutropenia in patients with breast cancer.	{Healthcare,Biotechnology}	{CELG,SGEN,BIIB,NVS,ABT,BMY,JNJ,TEVA,PFE,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:11.675205	2020-10-11 23:38:31.025871
SPRT	Support.com Inc.	support.com Inc is a provider of cloud-based software and services for technology support. Its solutions include a SaaS-based Nexus Service Platform, mobile and desktop apps, and a scalable workforce of technology specialists.	{Technology,"Software - Infrastructure","Application Software"}	{CTAS,HCSG,CMCSA,ABM,ROL,UNF,AMRC,SP,ODP,XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:11.690325	2020-10-11 23:38:31.023007
SPTN	SpartanNash Company	SpartanNash Company is a multi-regional grocery distributor and retailer. It distributes grocery products to independent grocery retailers, national retailers, and food service distributors.	{"Consumer Defensive","Food Distribution","Retail - Defensive"}	{UNFI,WILC,VLGEA,CORE,SYY,DIT,WMK,CHEF}	Retail - Defensive	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:11.706846	2020-10-11 23:38:31.028166
SPWR	SunPower Corporation	SunPower Corp is engaged in the solar industry. The company's business includes designing, manufacturing, installation and ongoing maintenance and monitoring of solar products.	{Technology,Solar,Semiconductors}	{FSLR,CSIQ,TOT,MTH,NRG,ENPH,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:11.726549	2020-10-11 23:38:31.021426
SPXV	ProShares S&P 500 Ex-Health Care	The investment seeks investment results, before fees and expenses, that track the performance of the S&P 500® Ex-Health Care Index (the "index").\n Under normal circumstances, the fund will invest at least 80% of its total assets in component securities. The index and fund seek to provide exposure to the companies of the S&P 500® Index (the "S&P 500®") with the exception of those companies included in the Health Care Sector.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.751486	2020-10-11 23:38:31.023428
SQLV	Legg Mason Small-Cap Quality Value ETF	The investment seeks to track the investment results of an index composed of equity securities of small-capitalization companies that are traded in the United States.\n The fund seeks to track the investment results of the Royce Small-Cap Quality Value Index (the "underlying index"). The underlying index utilizes a proprietary methodology created and sponsored by the fund's subadviser. The subadviser is affiliated with both LMPFA and the fund. The fund will invest at least 80% of its net assets, plus borrowings for investment purposes, if any, in securities that compose its underlying index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:11.768659	2020-10-11 23:38:31.02423
SPAR	Spartan Motors Inc.	Spartan Motors Inc is a market engineer and manufacturer in the heavy-duty, custom vehicles marketplace. It operates in three segments Emergency Response Vehicles, Delivery and Service Vehicles, and Specialty Chassis and Vehicles.	{Industrials,"Truck Manufacturing"}	{NAV,FSS,OSK,THO,WGO,MLR,ALSN,LCII}	Truck Manufacturing	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:11.585356	2020-10-11 23:38:31.028166
SPDW	SPDR Portfolio Developed World ex-US	The investment seeks investment results that, before fees and expenses, correspond generally to the total return performance of the S&P Developed Ex-U.S.BMI Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index and in depositary receipts based on securities comprising the index. The index is a market capitalization weighted index designed to define and measure the investable universe of publicly traded companies domiciled in developed countries outside the United States. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.598638	2020-10-11 23:38:31.031328
SPH	Suburban Propane Partners L.P.	Suburban Propane Partners LP is engaged in distribution of propane, fuel oil and refined fuels, as well as the marketing of natural gas and electricity in deregulated markets.	{Utilities,"Utilities - Regulated Gas","Utilities - Regulated"}	{FGP,APU,INT,UGI,GLP,SGU,AE,SRLP,CEQP,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:11.615589	2020-10-11 23:38:31.021426
SPI	SPI Energy Co. Ltd.	SPI Energy Co Ltd provides EPC services to third party project developers, as well as develops, owns and operates solar projects that sell electricity to the grid in multiple countries.	{Technology,Solar,Semiconductors}	{QRVO,QTNA,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:11.629718	2020-10-11 23:38:31.021426
SPLK	Splunk Inc.	Splunk Inc provides software products. Its flagship product is Splunk Enterprise and at its core is a proprietary machine data engine, comprised of collection, indexing, search, reporting analysis and data management capabilities.	{Technology,"Software - Application","Application Software"}	{DWCH,MSTR,IBM,TDC,DATA,VRNS,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:11.649833	2020-10-11 23:38:31.025871
SPMV	Invesco S&P 500 Minimum Variance	The investment seeks to track the investment results (before fees and expenses) of the S&P 500® Minimum Volatility Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in common stocks that comprise the underlying index. The underlying index measures the performance of a portfolio of equity securities using a managed volatility strategy that seeks to achieve lower total risk than the S&P 500® Index, while maintaining other similar characteristics of the S&P 500® Index. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:11.66729	2020-10-11 23:38:31.031328
SPPP	Sprott Physical Platinum and Palladium Trust	Sprott Physical Platinum and Palladium Trust invests and holds all of its assets in physical platinum & palladium bullion. It seeks to provide a convenient & exchange-traded investment alternative for investors interested in holding physical platinum.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{}	Metals & Mining	Basic Materials	NYSE Arca	\N	2020-10-11 23:12:11.683184	2020-10-11 23:38:31.028166
SPSM	SPDR Portfolio Small Cap	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the SSGA Small Cap Index that tracks the performance of small capitalization exchange traded U.S. equity securities.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. It may purchase a subset of the securities in the index in an effort to hold a portfolio of securities with generally the same risk and return characteristics of the index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.702264	2020-10-11 23:38:31.028166
SPVU	Invesco S&P 500 Enhanced Value	The investment seeks to track the investment results (before fees and expenses) of the S&P 500 Enhanced Value Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in common stocks that comprise the underlying index. The underlying index is designed to track the performance of approximately 100 stocks in the S&P 500® Index that have the highest "value score," which the index provider calculates based on fundamental ratios of a company's stock. A value stock tends to trade at a lower price relative to such fundamentals and thus may be considered undervalued by investors. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.719478	2020-10-11 23:38:31.024353
SPXL	Direxion Daily S&P 500 Bull 3X Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the daily performance of the S&P 500® Index.\n The fund, under normal circumstances, invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, exchange-traded funds ("ETFs") that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index is a float-adjusted, market capitalization-weighted index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.734632	2020-10-11 23:38:31.025871
SPXU	ProShares UltraPro Short S&P500	The investment seeks daily investment results before fees and expenses that correspond to three times the inverse (-3x) of the daily performance of the S&P 500® Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index is a measure of large-cap U.S. stock market performance. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.747577	2020-10-11 23:38:31.02423
SP	SP Plus Corporation	SP Plus Corp is a provider of parking management, ground transportation and other ancillary services to commercial, institutional and municipal clients in the United States, Puerto Rico and Canada.	{Industrials,"Business Services"}	{CTAS,HURN,ABM,MMS}	Business Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:11.57881	2020-10-11 23:38:31.023428
SPDN	Direxion Daily S&P 500 Bear 1X Shares	The investment seeks daily investment results, before fees and expenses, of 100% of the inverse (or opposite) of the daily performance of the S&P 500® Index.\n The fund, under normal circumstances, invests in swap agreements, futures contracts, short positions or other financial instruments that, in combination, provide inverse (opposite) or short exposure to the index equal to at least 80% of the fund's net assets (plus borrowing for investment purposes). The index is a float-adjusted, market capitalization-weighted index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.594867	2020-10-11 23:38:31.02423
SPG	Simon Property Group Inc.	Simon Property Group Inc is real estate investment trust. It owns, develops and manages retail real estate properties including malls, outlets and community and lifestyle centers.	{"Real Estate","REIT - Retail",REITs}	{VNO,ARE,SNR}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:11.60904	2020-10-11 23:38:31.028166
SPHS	Sophiris Bio Inc.	Sophiris Bio Inc is a clinical-stage biopharmaceutical company. It is engaged in research and development of products for the treatment of urological diseases. It is developing topsalysin (PRX302) for treatment of benign prostatic hyperplasia.	{Healthcare,Biotechnology}	{KERX,LLY,PFE,MRK,MRTX,NYMX,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:11.626899	2020-10-11 23:38:31.024353
SPLB	SPDR Portfolio Long Term Corporate Bond	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the price and yield performance of the Bloomberg Barclays U.S. Long Term Corporate Bond Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index or in securities that the Adviser determines have economic characteristics that are substantially identical to the economic characteristics of the securities that comprise the index. The index is designed to measure the performance of U.S. corporate bonds that have a maturity of greater than or equal to 10 years. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.644862	2020-10-11 23:38:31.028166
SPNS	Sapiens International Corporation N.V.	Sapiens International Corp NV is a provider of software solutions for the insurance industry, with an emerging focus on the financial services sector.	{Technology,"Software - Infrastructure","Application Software"}	{XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:11.673927	2020-10-11 23:38:31.028166
SPRO	Spero Therapeutics Inc.	Spero Therapeutics Inc is a multi asset clinical stage biopharmaceutical company. It is focused on identifying, developing and commercializing novel treatments for multi-drug resistant bacterial infections.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:11.686158	2020-10-11 23:38:31.024353
SPTS	SPDR Portfolio Short Term Treasury	The investment seeks to provide investment results that correspond generally to the price and yield performance of the Bloomberg Barclays 1-3 Year U.S. Treasury Index.\n The fund invests at least 80%, of its total assets in the securities comprising the index or in securities that the Adviser determines have economic characteristics that are substantially identical to the economic characteristics of the securities that comprise the index. The index is designed to measure the performance of short term (1-3 years) public obligations of the U.S. Treasury. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.708612	2020-10-11 23:38:31.031328
SPWH	Sportsman's Warehouse Holdings Inc.	Sportsman's Warehouse Holdings Inc is an outdoor sporting goods retailer. It provides a one-stop shopping experience that equips customers with right quality, brand name hunting, shooting, fishing and camping gear to maximize enjoyment of the outdoors.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{AMZN,POOL,HIBB,DKS}	Travel & Leisure	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:11.723912	2020-10-11 23:38:31.023428
SPXN	ProShares S&P 500 Ex-Financials	The investment seeks investment results, before fees and expenses, that track the performance of the S&P 500® Ex-Financials and Real Estate Index (the "index").\n Under normal circumstances, the fund will invest at least 80% of its total assets in component securities. The index and fund seek to provide exposure to the companies of the S&P 500® Index (the "S&P 500®") with the exception of those companies included in the Financials and Real Estate Sectors.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.73767	2020-10-11 23:38:31.031328
SPYG	SPDR Series Trust Portfolio S&P 500 Growth	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the S&P 500 Growth Index that tracks the performance of large capitalization exchange traded U.S. equity securities exhibiting "growth" characteristics.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index measures the performance of the large-capitalization growth segment of the U.S. equity market. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.761274	2020-10-11 23:38:31.02582
SR	Spire Inc.	Spire Inc, along with its subsidiaries is engaged in the purchase, retail distribution and sale of natural gas. The Company is also indulged in marketing of natural gas and related activities on a non-regulated basis.	{Utilities,"Utilities - Regulated Gas","Utilities - Regulated"}	{SRE,ATO,CPK,NFG,NJR,VVC,CNP,ET,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:11.780779	2020-10-11 23:38:31.025871
SRG	Seritage Growth Properties Class A	Seritage Growth Properties is a self-administered, self-managed REIT engaged in the real estate property business through its investment in Seritage Growth Properties, L.P. It generates a majority of its revenue from tenants.	{"Real Estate","REIT - Retail",REITs}	{NYMT,O,NTP}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:11.807271	2020-10-11 23:38:31.028166
SRTS	Sensus Healthcare Inc.	Sensus Healthcare Inc is a manufacturer of superficial radiation therapy devices. The Company's products and services include SRT-100, SRT-100 Vision, and Sentinel service program.	{Healthcare,"Medical Devices"}	{SIEN,XLV}	Medical Devices	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:11.824297	2020-10-11 23:38:31.028166
SSKN	Strata Skin Sciences Inc.	Strata Skin Sciences Inc is engaged in developing and commercializing products for the treatment of dermatological disorders.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{ARAY,DHR,GE,BVX,VAR,ISR,ICAD,FCRE,XLV}	Medical Instruments & Equipment	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:11.838714	2020-10-11 23:38:31.028166
SSWA	Seaspan Corporation 7.125% Notes due 2027		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:11.85344	2020-10-11 23:38:31.023007
STAY	Extended Stay America Inc.	Extended Stay America Inc owns and operates company-branded hotels in North America. Its business operates in the extended stay lodging industry. It operates hotels under brand names Extended Stay America, Extended Stay Canada & Crossland Economy Studios.	{"Consumer Cyclical",Lodging,"Travel & Leisure"}	{CHH,H,EXPE,MAR}	Travel & Leisure	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:11.870642	2020-10-11 23:38:31.02582
STKS	The ONE Group Hospitality Inc.	The One Group Hospitality Inc is a hospitality company that develops and operates upscale, high-energy restaurants and lounges and provides turn-key food and beverage services for hospitality venues including hotels, casinos and other locations.	{"Consumer Cyclical",Restaurants}	{ARKR,RUTH,DRI,DFRG,BLMN}	Restaurants	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:11.885045	2020-10-11 23:38:31.02582
STML	Stemline Therapeutics Inc.	Stemline Therapeutics Inc is a clinical-stage biopharmaceutical company. The company is engaged in discovering, acquiring, developing and commercializing proprietary therapeutics that target both cancer stem cells, or CSCs, and tumor bulk.	{Healthcare,Biotechnology}	{CELG,JNJ,PFE,AGEN,VSTM,OMED,CYCC,SNSS,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:11.897247	2020-10-11 23:38:31.024353
STOR	STORE Capital Corporation	STORE Capital Corp is engaged in the business of acquisition, investments, and management of single tenant operational real estate. The company owns and operates a diversified portfolio of investments with over 1,500 property across the US.	{"Real Estate","REIT - Diversified",REITs}	{NYMT,O,NTP}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:11.913512	2020-10-11 23:38:31.031328
STX	Seagate Technology PLC	Seagate Technology PLC is a producer of hard disk drives (HDD) globally. It designs, develops, and manufactures a broad range of HDDs used in desktop PC, notebook PC, game consoles, digital video recorders, and enterprise storage markets.	{Technology,"Data Storage","Computer Hardware"}	{MSFT,NTAP,WDC,HPQ,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:11.930705	2020-10-11 23:38:31.023428
SUNW	Sunworks Inc.	Sunworks Inc is provide photo voltaic (PV) based power systems for the residential, commercial and agricultural markets in California and Nevada.	{Technology,Solar,Semiconductors}	{JCS,FNSR,OCLR,XLK}	Semiconductors	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:11.949301	2020-10-11 23:38:31.02423
SVVC	Firsthand Technology Value Fund Inc.	Firsthand Technology Value Fund Inc is an externally managed, closed-end, non-diversified management investment company. Its investment objective is to seek long-term growth of capital by seeking capital gains on equity and equity-related investments.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:11.967306	2020-10-11 23:38:31.024353
SWZ	Swiss Helvetia Fund Inc. (The)	Swiss Helvetia Fund Inc is a closed-end fund. Its objective is to seek long-term growth of capital through investment in equity and equity-linked securities of Swiss companies.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:11.996164	2020-10-11 23:38:31.031328
SYE	SPDR MFS Systematic Core Equity	The investment seeks capital appreciation.\n Under normal circumstances, the fund invests at least 80% of its net assets (plus the amount of borrowings for investment purposes) in equity securities. Equity securities in which the fund invests include common stocks, preferred stocks, and securities convertible into stocks. In selecting securities for the fund, the Sub-Adviser utilizes a bottom-up approach to buying and selling investments for the fund.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.014988	2020-10-11 23:38:31.024353
SYN	Synthetic Biologics Inc.	Synthetic Biologics Inc is a late-stage biotechnology company focused on developing gut microbiome therapeutics. It is also developing monoclonal antibody therapies for the treatment of pertussis.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NYSE American	\N	2020-10-11 23:12:12.034373	2020-10-11 23:38:31.02423
SYX	Systemax Inc.	Systemax Inc is direct marketer of brand name and private label products including industrial, material handling and supplies, computer equipment and related accessories, technology supplies and consumer electronics.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{NSIT,CNXN,TECD,PCMI,OSTK,STMP,SNX,EVLV,ESND,GAIA}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:12.04862	2020-10-11 23:38:31.028166
SRCE	1st Source Corporation	1st Source Corp through its subsidiary offers commercial, agricultural, and real estate loans, consumer services, trust and wealth advisory services, and specialty finance group services.	{"Financial Services","Banks - Regional - US",Banks}	{THFF,LKFN,GABC,FRME,HBNC,MFSF,TCF,ONB,BXS}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:11.791435	2020-10-11 23:38:31.031328
SRLP	Sprague Resources LP representing Limited Partner Interests	Sprague Resources LP is a wholesale and commercial distributor engaged in the purchase, storage, distribution and sale of refined products and natural gas, and provides storage and handling services for a broad range of materials.	{Energy,"Oil & Gas Refining & Marketing","Oil & Gas - Refining & Marketing"}	{CLNE,FGP,INT,UGI,CAPL,SUN}	Oil & Gas - Refining & Marketing	Energy	New York Stock Exchange	\N	2020-10-11 23:12:11.817434	2020-10-11 23:38:31.025871
SSB	South State Corporation	South State Corp is a bank holding company. It provides retail and commercial banking services, mortgage lending services, trust and investment services and consumer loans.	{"Financial Services","Banks - Regional - US",Banks}	{FFIC,GBCI,BHLB,BUSE}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:11.835165	2020-10-11 23:38:31.028166
SSTK	Shutterstock Inc.	Shutterstock Inc operates marketplace and provides commercial digital imagery including licensed photographs, vectors, illustrations and video clips; and commercial music including music tracks and sound effects.	{Technology,"Internet Content & Information","Online Media"}	{TRI,DISCA,DLB,IMAX,GOOGL,XLK}	Online Media	Technology	New York Stock Exchange	\N	2020-10-11 23:12:11.852537	2020-10-11 23:38:31.02423
STAG	Stag Industrial Inc.	Stag Industrial Inc is an industrial real estate company focused on the acquisition, ownership and management of single-tenant industrial buildings throughout the United States.	{"Real Estate","REIT - Industrial",REITs}	{FR,EGP,MNR,DLR,COR,PLD,REXR}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:11.866692	2020-10-11 23:38:31.024353
STNG	Scorpio Tankers Inc.	Scorpio Tankers Inc acts as a provider in the transportation of refined petroleum products. The company operates its vessels in the Scorpio LR2 Pool, Scorpio Aframax Pool, Scorpio Panamax Pool, Scorpio MR Pool, and Scorpio Handymax Pool.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{GASS,CPLP,TNP,TK,TNK,ASC}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:11.900158	2020-10-11 23:38:31.028166
STT	State Street Corporation	State Street Corporation is a financial holding company operating through Investment Servicing and Investment Management business. It provides services to mutual funds, corporate and public retirement plans, insurance companies, foundations, endowments.	{"Financial Services","Asset Management"}	{NTRS,BCS,BAC,JPM,BK,C}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:11.927125	2020-10-11 23:38:31.028166
SUNS	Solar Senior Capital Ltd.	Solar Senior Capital Ltd is a non-diversified management investment company. The company's investment objective is to seek to maximize current income consistent with the preservation of capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:11.947275	2020-10-11 23:38:31.023428
SVM	Silvercorp Metals Inc.	Silvercorp Metals Inc is a mineral mining company. It acquires, explores, develops, and mines precious and base metal mineral properties at its producing mines and exploration and development projects in China.	{"Basic Materials",Silver,"Metals & Mining"}	{PAAS,MAG,EXK,AXU,AG,GPL,WPM}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:11.96321	2020-10-11 23:38:31.028166
SWIR	Sierra Wireless Inc.	Sierra Wireless Inc designs and manufactures wireless communications equipment for use in automotive, transportation, energy, healthcare, industrial, and mobile computing industries.	{Technology,"Communication Equipment"}	{RCI,CSCO,DGII,NTGR,ERIC,CAMP,INSG,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:11.976304	2020-10-11 23:38:31.02582
SXE	Southcross Energy Partners L.P.	Southcross Energy Partners LP provides natural gas gathering, processing, treating, compression and transportation services, and natural gas liquid (NGL) fractionation and transportation services for its producer customers.	{Utilities,"Utilities - Regulated Gas","Utilities - Regulated"}	{DVN,PAA,EEP,MMP,EEQ,KMI,PSXP,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:12.004541	2020-10-11 23:38:31.028166
SYNA	Synaptics Incorporated	Synaptics Inc is a developer and supplier of custom-designed human interface solutions that enable people to interact easily and intuitively with mobile computing, communications, entertainment, and other electronic devices.	{Technology,"Software - Application","Application Software"}	{MRCY,AAPL,CY,MXIM,IMMR,LOGI,CRAY,HPQ,SMCI,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:12.035885	2020-10-11 23:38:31.024353
TA	TravelCenters of America LLC	TravelCenters of America LLC operates and franchise travel centers, standalone convenience stores, and standalone restaurants. It collects rents, royalties and other fees from our tenants and franchisees.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{CASY,LAD,GT,MUSA}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:12.058414	2020-10-11 23:38:31.031328
TBB	AT&T Inc. 5.350% Global Notes due 2066		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:12.099635	2020-10-11 23:38:31.031328
SRCI	SRC Energy Inc.	SRC Energy Inc, is an oil & natural gas company. It is engaged in the acquisition, development, exploitation, exploration and production of oil and natural gas properties.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{}	Oil & Gas - E&P	Energy	NYSE American	\N	2020-10-11 23:12:11.791446	2020-10-11 23:38:31.028166
SRS	ProShares UltraShort Real Estate	The investment seeks daily investment results, before fees and expenses, that correspond to two times the inverse (-2x) of the daily performance of the Dow Jones U.S. Real EstateSM Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index seeks to measure the performance of certain companies in the real estate sector of the U.S. equity market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.820844	2020-10-11 23:38:31.023428
SSI	Stage Stores Inc.	Stage Stores Inc operates specialty department stores. Its merchandise assortment is a well-edited selection of moderately priced brand name and private label apparel, accessories, cosmetics, footwear and home goods.	{"Consumer Cyclical","Apparel Stores","Retail - Apparel & Specialty"}	{ROST,SMRT,CTRN,ANF,DDS,AEO,JWN,JCP,TJX}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:11.839076	2020-10-11 23:38:31.02582
SSW	Seaspan Corporation	Seaspan Co is an independent charter owner and manager of containerships. It operates a fleet of around 85 containerships.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{DSX,DAC,GSL,CMRE,DCIX}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:11.852119	2020-10-11 23:38:31.02582
STBZ	State Bank Financial Corporation.	State Bank Financial Corp is a bank holding company. The services offered are loans to small and medium sized businesses, lender finance loans, commercial real estate loans, residential and commercial construction and development loan.	{"Financial Services","Banks - Regional - US",Banks}	{CBAN,LION,ABCB,BBT,WFC,STI,SGB}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:11.875558	2020-10-11 23:38:31.023007
STL	Sterling Bancorp	Sterling Bancorp is a provider of financial services. The company functions through the Sterling National Bank, which delivers deposit, lending, and wealth management products to commercial businesses and affluent consumers.	{"Financial Services","Banks - Regional - US",Banks}	{MBWM,CBSH,OFG}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:11.89075	2020-10-11 23:38:31.023007
STOT	SPDR DoubleLine Short Duration Total Return Tactical	The investment seeks to maximize current income with a dollar-weighted average effective duration between one and three years.\n Under normal circumstances, the fund will invest at least 80% of its net assets in a diversified portfolio of fixed income securities of any credit quality. Fixed income securities in which the fund principally invests are defined as securities issued or guaranteed by the U.S. government or its agencies, instrumentalities or sponsored corporations; TIPS; municipal bonds (the fund may invest up to 20% of its portfolio in municipal bonds); asset-backed securities; etc.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:11.914747	2020-10-11 23:38:31.02423
STWD	STARWOOD PROPERTY TRUST INC. Starwood Property Trust Inc.	Starwood Property Trust Inc finances, manages and invests in commercial mortgage loans and other commercial real estate debt investments, commercial mortgage-backed securities, and other commercial real estate-related debt investments.	{"Real Estate","REIT - Diversified",REITs}	{ABR,BXMT,CMCT}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:11.927622	2020-10-11 23:38:31.025871
SVBI	Severn Bancorp Inc	Severn Bancorp Inc is a savings & loan holding company. It offers savings, money market, time deposit, individual retirement accounts & mortgage banking services. It also provides safe deposit boxes, ATMs, internet banking & debit cards, among others.	{"Financial Services","Banks - Regional - US",Banks}	{WSFS,FFIC,DCOM,ESBK,NWBI,NYCB,HBK}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:11.962637	2020-10-11 23:38:31.025871
SWK	Stanley Black & Decker Inc.	Stanley Black & Decker Inc provides hand tools, power tools and accessories, mechanical access solutions, electronic security and monitoring systems, healthcare solutions, engineered fastening systems and products for various industrial applications.	{Industrials,"Tools & Accessories","Industrial Products"}	{LECO,PFIN,GE,ITW,KMT,SNA,TTC,MMM}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:11.983089	2020-10-11 23:38:31.025871
SXCP	SunCoke Energy Partners L.P. Representing Limited partner Interests	SunCoke Energy Partners LP produces coke used in the blast furnace production of steel. It also provides handling and mixing services to steel, coke, electric utility, coal producing, and other manufacturing based customers.	{"Basic Materials",Coal}	{SXC,SES}	Coal	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:12.001831	2020-10-11 23:38:31.025871
SYKE	Sykes Enterprises Incorporated	Sykes Enterprises Inc provides outsourced customer contact management solutions and services in the business process outsourcing arena to companies within the communications, financial services, technology/consumer and other industries.	{Technology,"Information Technology Services","Application Software"}	{FFIV,EBAY,SRT,SNX,XRX,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:12.023669	2020-10-11 23:38:31.02582
SYNL	Synalloy Corporation	Synalloy Corp manufactures dyes, metal pipes and systems, fiberglass, and steel tanks. Its products are widely used in the chemical, paper, mining, textiles and liquefied natural gas industries.	{"Basic Materials",Steel}	{NWPX,NOV,MLI,RPM,TS,HUN,FRD,NL,SPLP,ASH}	Steel	Basic Materials	NASDAQ Global Market	\N	2020-10-11 23:12:12.039151	2020-10-11 23:38:31.021426
SRCL	Stericycle Inc.	Stericycle Inc is engaged in providing regulated and compliance solutions to healthcare and commercial businesses. This includes the collection and processing of specialized waste for disposal, and a variety of training, and consulting.	{Industrials,"Waste Management"}	{SMED,RSG,WM}	Waste Management	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:11.792791	2020-10-11 23:38:31.024353
SSG	ProShares UltraShort Semiconductors	The investment seeks daily investment results, before fees and expenses, that correspond to two times the inverse (-2x) of the daily performance of the Dow Jones U.S. SemiconductorsSM Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index seeks to measure the performance of certain companies in the semiconductor sub-sector of the U.S. equity market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.836394	2020-10-11 23:38:31.023428
SSRM	SSR Mining Inc.	SSR Mining Inc is a Vancouver-based mining company focused on the operation, development, exploration and acquisition of precious metal projects. The company is engaged in exploring gold and silver properties.	{"Basic Materials",Gold,"Metals & Mining"}	{}	Metals & Mining	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:12:11.85182	2020-10-11 23:38:31.021426
STAF	Staffing 360 Solutions Inc.	Staffing 360 Solutions Inc is an international staffing company engaged in the acquisition of United States and United Kingdom based staffing companies. It pursues spectrum of staffing companies supporting accounting,IT engineering, administration.	{Industrials,"Staffing & Outsourcing Services","Employment Services"}	{}	Employment Services	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:11.865613	2020-10-11 23:38:31.02423
STFC	State Auto Financial Corporation	State Auto Financial Corp is a property and casualty insurance holding company. It is engaged in writing both personal and business and specialty insurance.	{"Financial Services","Insurance - Property & Casualty"}	{ERIE,DGICA,WRB,MKL,ORI,RLI,TRV,UVE}	Insurance - Property & Casualty	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:11.879191	2020-10-11 23:38:31.031328
STLY	HG Holdings Inc	HG Holdings Inc is a shell company.	{}	{BSET,FLXS,HOFT,ETH,LZB,NTZ}	Conglomerates	Industrials	OTCQB - U.S. Registered	\N	2020-10-11 23:12:11.895416	2020-10-11 23:38:31.031328
STON	StoneMor Partners L.P. Rep Limited Partnership Interests	Stonemor Partners LP mainly provides various products and services through the ownership, development, and operation of cemeteries and funeral homes in multiple states. It sells products and services at the time of death and prior to death (preneed).	{"Consumer Cyclical","Personal Services"}	{MATW,SCI,CSV}	Personal Services	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:11.911589	2020-10-11 23:38:31.023428
STRT	STRATTEC SECURITY CORPORATION	Strattec Security Corp designs, develops, manufactures and markets mechanical locks, electronically enhanced locks and keys. It also produces ignition lock housings; access control products, including latches, power sliding door systems,and door handles.	{"Consumer Cyclical","Auto Parts",Autos}	{MGA,MPAA,SUP,AXL,LEA,SRI,APTV,THRM}	Autos	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:11.926316	2020-10-11 23:38:31.02582
SUB	iShares Short-Term National Muni Bond	The investment seeks to track the investment results of the S&P Short Term National AMT-Free Municipal Bond IndexTM.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The index measures the performance of the short-term investment-grade segment of the U.S. municipal bond market.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.937805	2020-10-11 23:38:31.024353
SUPN	Supernus Pharmaceuticals Inc.	Supernus Pharmaceuticals Inc is a specialty pharmaceutical company. The company is engaged in developing and commercializing products for the treatment of central nervous system diseases, including neurological and psychiatric disorders.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{UTHR,SHPG,ABT,BMY,JNJ,LLY,ARCT,ZGNX,XLV}	Drug Manufacturers	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:11.948826	2020-10-11 23:38:31.028166
SWM	Schweitzer-Mauduit International Inc.	Schweitzer-Mauduit International, Inc. is a diversified producer of premium specialty papers and resin-based products. It manufactures and sells paper and reconstituted tobacco products to the tobacco industry as well as specialized paper products.	{"Basic Materials","Paper & Paper Products","Forest Products"}	{GLT,BTI,MO,NP,PM}	Forest Products	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:11.987443	2020-10-11 23:38:31.02423
SXI	Standex International Corporation	Standex International Corp is a manufacturer of products and services for industrial market segments.	{Industrials,"Diversified Industrials","Industrial Products"}	{AAON,MIDD,AME,LII,LDL,PH,JBT,MTW}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:12.006847	2020-10-11 23:38:31.023007
SRF	Cushing Energy Income Fund of Beneficial Interest	The Cushing Energy Income Fund is a non-diversified, closed-end investment company. Its investment objective is to seek a high total return with an emphasis on current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:11.807903	2020-10-11 23:38:31.023007
SRT	StarTek Inc.	StarTek Inc is a provider of business process outsourcing services. Its services include customer care, sales support, inbound sales, complex order processing, accounts receivable management, technical and product support, up-sell and cross-sell.	{Industrials,"Business Services"}	{SYKE,MMS}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:11.826241	2020-10-11 23:38:31.02582
SSNT	SilverSun Technologies Inc.	SilverSun Technologies Inc is a business application, technology and consulting company. It is engaged in providing strategies and solutions to meet information, technology and business management needs.	{Technology,"Software - Application","Application Software"}	{XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:11.847802	2020-10-11 23:38:31.028166
SSYS	Stratasys Ltd.	Stratasys Ltd provides additive manufacturing solutions for the creation of parts used in the processes of designing and manufacturing products and for the direct manufacture of end parts. It offers 3D printers and 3D production systems.	{Technology,"Computer Systems","Computer Hardware"}	{XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:11.85988	2020-10-11 23:38:31.031328
STLD	Steel Dynamics Inc.	Steel Dynamics Inc is a steel producers and metal recycler in the United States. The company produces steel sheet products, long products and steel finishing products. It also operates metals recycling and steel fabrication business.	{"Basic Materials",Steel}	{SCHN,AKS,SIM,GGB,CMC,NUE,WOR,X,PKX,MT}	Steel	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:12:11.893632	2020-10-11 23:38:31.028166
STNLU	Sentinel Energy Services Inc. Unit	Sentinel Energy Services Inc is a blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:11.910912	2020-10-11 23:38:31.023007
STRM	Streamline Health Solutions Inc.	Streamline Health Solutions Inc is a provider of transformational data-driven solutions for healthcare organizations. The company provides computer software-based solutions through its Looking Glass platform.	{Technology,"Health Information Services","Application Software"}	{CERN,MDRX,CPSI,NUAN,GE,MCK,XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:11.924563	2020-10-11 23:38:31.028166
SUMR	Summer Infant Inc.	Summer Infant Inc manufactures juvenile safety and infant care products. Its product categories include monitoring, safety, nursery, baby gear, and feeding products. The United States of America generates maximum revenue for the company.	{"Consumer Defensive","Household & Personal Products","Consumer Packaged Goods"}	{IPAR,CRWS,MAT,NWL,AVP,REV,USNA,FBHS}	Consumer Packaged Goods	Consumer Defensive	NASDAQ Capital Market	\N	2020-10-11 23:12:11.943048	2020-10-11 23:38:31.021426
SVRA	Savara Inc.	Savara Inc is a pharmaceutical company. The company develops novel therapies for the treatment of serious or life-threatening rare respiratory diseases.	{Healthcare,Biotechnology}	{ARQL,SGMO,NVS,BMY,JNJ,LLY,PFE,GLYC,CYTR,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:11.963579	2020-10-11 23:38:31.023428
SWJ	Stanley Black & Decker Inc. 5.75% Junior Subordinated Debenture due 2052	Stanley Black & Decker Inc provides hand tools, power tools and accessories, mechanical access solutions, electronic security and monitoring systems, healthcare solutions, engineered fastening systems and products for various industrial applications.	{Industrials,"Tools & Accessories","Industrial Products"}	{}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:11.981133	2020-10-11 23:38:31.023007
SYK	Stryker Corporation	Stryker Corp is engaged in designing, manufacturing and marketing of medical equipment, instruments, and implantable devices. Its products include hip and knee replacements, endoscopy systems, operating room equipment, embolic coils, and spinal devices.	{Healthcare,"Medical Devices"}	{IART,BAX,BSX,EW,SNN,ZBH,HRC,MDT,WMGI,XLV}	Medical Devices	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:12.02288	2020-10-11 23:38:31.028166
SYPR	Sypris Solutions Inc.	Sypris Solutions Inc is a provider of outsourced services and specialty products. It is a supplier of forged and machined components, serving the commercial vehicle, off highway vehicle. It also provides Information Security Solutions.	{"Consumer Cyclical","Auto Parts",Autos}	{JBL,NAV,DCO,AXL,WBC,DAN,SRI,MTOR,ALSN}	Autos	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:12.038885	2020-10-11 23:38:31.025871
TACT	TransAct Technologies Incorporated	Transact Technologies, Inc. is engaged in developing and selling software-driven technology and printing solutions for restaurant, POS automation and banking, casino and gaming, lottery, mobile and oil and gas.	{Technology,"Computer Systems","Computer Hardware"}	{ALOT,ZBRA,SYNA,USAT,BRC,DOV,XLK}	Computer Hardware	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:12.064102	2020-10-11 23:38:31.02582
TAST	Carrols Restaurant Group Inc.	Carrols Restaurant Group Inc owns and operates restaurants under the Burger King brand in Northeastern, Midwestern and Southeastern states. It offers hamburgers, chicken and other specialty sandwiches, french fries, breakfast items and other offerings.	{"Consumer Cyclical",Restaurants}	{JACK,SONC,MCD,GTIM,ARCO,WEN,ZOES,RAVE}	Restaurants	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:12.091038	2020-10-11 23:38:31.028166
SRNE	Sorrento Therapeutics Inc.	Sorrento Therapeutics Inc is a biotechnology company focused on the development of novel immunotherapies for oncology and autoimmune/inflammation diseases.	{Healthcare,Biotechnology}	{EXEL,SGYP,KPTI,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:11.818429	2020-10-11 23:38:31.028166
SRV	Cushing MLP & Infrastructure Total Return Fund	Cushing Mlp Total Return Fund is a non-diversified, closed-end management investment company with an investment objective of seeking a high after-tax total return from a combination of capital appreciation and current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:11.834474	2020-10-11 23:38:31.024353
SSO	ProShares Ultra S&P500	The investment seeks daily investment results, before fees and expenses, that correspond to two times (2x) the daily performance of the S&P 500® Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index is a measure of large-cap U.S. stock market performance. It is a float-adjusted, market capitalization-weighted index of 500 U.S. operating companies and real estate investment trusts selected through a process that factors in criteria such as liquidity, price, market capitalization and financial viability. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.847144	2020-10-11 23:38:31.025871
ST	Sensata Technologies Holding plc	Sensata Technologies Holding N.V. is an industrial technology company. It produces sensors and controls for applications in automotive systems, thermal circuit breakers in aircraft and bimetal current and temperature control devices in electric motors.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:11.864221	2020-10-11 23:38:31.025871
STK	Columbia Seligman Premium Technology Growth Fund Inc	Columbia Seligman Premium Technology Growth Fund Inc is a closed-end management investment company. Its investment objectives is to seek growth of capital and current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:11.883442	2020-10-11 23:38:31.02423
STNL	Sentinel Energy Services Inc.	Sentinel Energy Services Inc is a blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:11.904281	2020-10-11 23:38:31.021426
STRL	Sterling Construction Company Inc	Sterling Construction Co Inc is a civil construction company. The company operates through its subsidiaries in construction of buildings and reconstruction of transportation and water infrastructure projects.	{Industrials,"Engineering & Construction"}	{GLDD,PRIM,MTZ,GVA,ORN,TPC,IESC}	Engineering & Construction	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:11.922237	2020-10-11 23:38:31.024353
SUP	Superior Industries International Inc. (DE)	Superior Industries International Inc manufactures aluminum wheels. The company manufactures and supplies aluminum wheels to its multinational automotive clients primarily used in passenger automobiles and light-duty vehicles.	{"Consumer Cyclical","Auto Parts",Autos}	{STRT,CVGI,TWI,AXL,LEA,SRI,MLR,TOWR,LCII}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:11.94713	2020-10-11 23:38:31.025871
SVT	Servotronics Inc.	Servotronics Inc is a U.S based company which designs, manufactures and markets advanced technology products consisting of control components and consumer products consisting of knives and various types of cutlery and other edged products.	{Technology,"Electronic Components","Computer Hardware"}	{AMOT,LCUT,ACU,RBC,UQM,GNRC,XLK}	Computer Hardware	Technology	NYSE American	\N	2020-10-11 23:12:11.971275	2020-10-11 23:38:31.02423
SXC	SunCoke Energy Inc.	SunCoke Energy Inc is engaged in the production of metallurgical coke in the Americas. The Company also owns and operates coal mining operations in Virginia and West Virginia.	{"Basic Materials",Coal}	{CLD,SXCP,MT,SES}	Coal	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:12.00091	2020-10-11 23:38:31.02582
SYG	SPDR MFS Systematic Growth Equity	The investment seeks capital appreciation.\n Under normal circumstances, the fund invests at least 80% of its net assets (plus the amount of borrowings for investment purposes) in equity securities. Equity securities in which the fund invests include common stocks, preferred stocks, and securities convertible into stocks. In selecting securities for the fund, the Sub-Adviser utilizes a bottom-up approach to buying and selling investments for the fund.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.022696	2020-10-11 23:38:31.031328
SYNH	Syneos Health Inc.	Syneos Health Inc is a Contract Research Organization providing clinical development services for biopharmaceutical & medical device industries. It operates in North America, Europe, Middle East and Africa, Asia-Pacific and Latin America.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{XLV}	Medical Diagnostics & Research	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:12.036896	2020-10-11 23:38:31.031328
SZC	Cushing Renaissance Fund (The) of Beneficial Interest	Cushing Renaissance Fund is a closed end investment management company. Its investment objective is to seek high total return with an focus on current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:12.052747	2020-10-11 23:38:31.024353
TACO	Del Taco Restaurants Inc.	Del Taco Restaurants Inc is an operator and franchisor of restaurants featuring fresh and fast made-to-order cuisine, including both Mexican inspired and American classic dishes.	{"Consumer Cyclical",Restaurants}	{SHIP,SGLB}	Restaurants	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:12.063543	2020-10-11 23:38:31.028166
SRI	Stoneridge Inc.	Stoneridge Inc acts as a manufacturer of electrical and electronic components used in automotive vehicles. It produces instrumentation systems, vehicle management electronics, application-specific switches and actuators, security alarms and others.	{"Consumer Cyclical","Auto Parts",Autos}	{MPAA,STRT,CVGI,NAV,SUP,AXL,MLR,MTOR,APTV,VC}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:11.807428	2020-10-11 23:38:31.031328
SRTY	ProShares UltraPro Short Russell2000	The investment seeks daily investment results that correspond to three times the inverse (-3x) of the daily performance of the Russell 2000® Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index is a measure of small-cap U.S. stock market performance. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.825513	2020-10-11 23:38:31.031328
SSFN	Stewardship Financial Corp	Stewardship Financial Corp is a bank holding company. It offers personal and business checking accounts and time deposits, money market accounts, regular savings accounts, commercial, residential real estate and personal loans.	{"Financial Services","Banks - Regional - US",Banks}	{UNTY,PGC,LBAI,PKBK,BKJ,TRCB,VLY}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:11.836035	2020-10-11 23:38:31.02423
SSP	E.W. Scripps Company (The)	The E W Scripps Co is a media company engaged in television and radio broadcasting. Its operates more than 30 television and radio stations and distributes its content on broadcast, Internet, smartphones and tablets.	{"Consumer Cyclical","Broadcasting - TV",Entertainment}	{NXST,CMCSA,LEE,NYT,AHC,TGNA,MNI}	Entertainment	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:11.847802	2020-10-11 23:38:31.024353
STAA	STAAR Surgical Company	Staar Surgical along with its subsidiaries designs, develops, manufactures and sells implantable lenses and delivery systems for the eye. The Company develops, produces, and markets intraocular lenses.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{IRIX,ATRI,COO,NVS,ABT,TEAR,XLV}	Medical Instruments & Equipment	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:11.861882	2020-10-11 23:38:31.023428
STE	STERIS plc	STERIS PLC is a provider of infection prevention and other procedural products and services. The Company's business segments are Healthcare Products, Healthcare Specialty Services, Life Sciences, and Applied Sterilization Technologies.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{CNMD,OMI,CMD,ECL,IVC,JNJ,KMB,SYK,TMO,HRC,XLV}	Medical Instruments & Equipment	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:11.87895	2020-10-11 23:38:31.028166
STLRU	Stellar Acquisition III Inc. Unit	Stellar Acquisition III Inc is a blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:11.895381	2020-10-11 23:38:31.025871
STNLW	Sentinel Energy Services Inc. Warrant		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:11.911511	2020-10-11 23:38:31.02582
STRS	Stratus Properties Inc.	Stratus Properties Inc is a diversified real estate company which is engaged in the acquisition, entitlement, development, management, operation, and sale of commercial, hotel, entertainment, and residential real estate properties.	{"Real Estate","Real Estate - General","Real Estate Services"}	{IOR,TRC}	Real Estate Services	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:11.925388	2020-10-11 23:38:31.021426
SU	Suncor Energy Inc.	Suncor Energy Inc is a Canadian integrated energy company. It operations include oil sands development and upgrading, offshore oil and gas production, petroleum refining, product marketing and a renewable energy portfolio.	{Energy,"Oil & Gas Integrated","Oil & Gas - Integrated"}	{CNQ,IMO,TOT,XOM,BP,CVX,COP,RDS.A}	Oil & Gas - Integrated	Energy	New York Stock Exchange	\N	2020-10-11 23:12:11.93852	2020-10-11 23:38:31.02423
SUPV	Grupo Supervielle S.A. American Depositary Shares each Representing five Class B shares	Grupo Supervielle SA offers financial products and services and its operating segments include retail banking, corporate banking, treasury, consumer finance, insurance and asset management & other services.	{"Financial Services","Banks - Regional - Latin America",Banks}	{}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:11.955067	2020-10-11 23:38:31.024353
SWN	Southwestern Energy Company	Southwestern Energy Co is an oil and gas company that explores, develops, and produces oil and natural gas within the United States. It is focused on unconventional natural gas in two US shale plays and oil and gas formations in the United States.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{CHK,NFX,COG,OXY,MRO,NBL,EQT,HES,QEP}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:11.990299	2020-10-11 23:38:31.024353
SYBT	SPDR Barclays US Treasury Bond UCITS ETF	Stock Yards Bancorp Inc is the holding company. Its services include loan and deposit services, cash management services, securities brokerage activities, mortgage origination and wealth management and trust activities.	{"Financial Services","Banks - Regional - US",Banks}	{CTBI,CZFC,RNST,BXS,PFBX}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:12.01434	2020-10-11 23:38:31.023428
SYNC	Synacor Inc.	Synacor Inc provides managed portals, advertising solutions, email and collaboration platforms, end-to-end video solutions and cloud-based identity management.	{Technology,"Internet Content & Information","Online Media"}	{MSFT,VRSN,CTL,TCX,GOOGL,CHTR,XLK}	Online Media	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:12.03652	2020-10-11 23:38:31.023428
SZO	DB Crude Oil Short ETN due June 1 2038	The investment seeks to track the price and yield performance, before fees and expenses, of the Deutsche Bank Liquid Commodity index - Optimum Yield Oil Excess Return.\n The fund allows investors to take a short view on the performance of the index. The index is a rules-based index composed of futures contracts on light sweet crude oil (WTI) and is intended to reflect the performance of crude oil.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.054358	2020-10-11 23:38:31.023428
SRTSW	Sensus Healthcare Inc. Warrant		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:11.825167	2020-10-11 23:38:31.023007
SSL	Sasol Ltd. American Depositary Shares	Sasol Ltd is an integrated chemicals and energy company. It operates coal mines and its upstream interests in oil and gas. It products include Automotive lubricants, industrial lubricants, greases, automotive fuels, burner fuels and other.	{Energy,"Oil & Gas Integrated","Oil & Gas - Integrated"}	{TOT,XOM,BP,SHI,CVX,HUN,RDS.A,PSX,DWDP}	Oil & Gas - Integrated	Energy	New York Stock Exchange	\N	2020-10-11 23:12:11.84105	2020-10-11 23:38:31.023007
SSWN	Seaspan Corporation 6.375% Notes due 2019		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:11.858773	2020-10-11 23:38:31.028166
STC	Stewart Information Services Corporation	Stewart Information Services Corp is a title insurance and real estate service company. It is engaged in offering products and services through its direct operations, network of approved agencies and other companies.	{"Financial Services","Insurance - Property & Casualty"}	{ITIC,ORI,FAF,ARGO}	Insurance - Property & Casualty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:11.876589	2020-10-11 23:38:31.025871
STMP	Stamps.com Inc.	Stamps.com Inc provides internet-based postage solutions. Its customers use the Company's service to mail and ship a variety of mail pieces, including postcards, envelopes, flats and packages, using a range United States Postal Service mail classes.	{Technology,"Software - Application","Application Software"}	{CNXN,MELI,SHOP,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:11.897224	2020-10-11 23:38:31.028166
STPZ	PIMCO 1-5 Year U.S. TIPS Index Exchange-Traded Fund	The investment seeks to provide total return that closely corresponds, before fees and expenses, to the total return of the ICE BofAML 1-5 Year US Inflation-Linked Treasury Index.\n The fund invests at least 80% of its total assets (exclusive of collateral held from securities lending) in the component securities of the ICE BofAML 1-5 Year US Inflation-Linked Treasury Index (the "underlying index"). The underlying index is an unmanaged index comprised of Treasury Inflation-Protected Securities ("TIPS") with a maturity of at least 1 year and less than 5 years.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.918548	2020-10-11 23:38:31.025871
SUN	Sunoco LP representing limited partner interests	Sunoco LP is a wholesale and retail fuel distributor in the United States also manufactures the official fuel for NASCAR. It distributes fuel through an extensive line of distributes across the United States.	{Energy,"Oil & Gas Refining & Marketing","Oil & Gas - Refining & Marketing"}	{CLNE,FGP,VLO,XOM,GLP,SRLP,CAPL}	Oil & Gas - Refining & Marketing	Energy	New York Stock Exchange	\N	2020-10-11 23:12:11.944086	2020-10-11 23:38:31.02582
SUSC	iShares ESG USD Corporate Bond ETF	The investment seeks to track the investment results of the Bloomberg Barclays MSCI US Corporate ESG Focus Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The index has been developed by Bloomberg Barclays Capital Inc. with environmental, social and governance ("ESG") rating inputs from MSCI ESG Research LLC ("MSCI ESG Research") pursuant to an agreement between MSCI ESG Research and Bloomberg Index Services Limited, a subsidiary of Bloomberg Barclays. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:11.962521	2020-10-11 23:38:31.023007
SWKS	Skyworks Solutions Inc.	Skyworks Solutions Inc is engaged in producing semiconductors for wireless handsets that are used to enable wireless connectivity. Its products include power amplifiers, filters, and integrated front-end modules.	{Technology,Semiconductors}	{MXIM,MTSI,ADI,QRVO,PRKR,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:11.989244	2020-10-11 23:38:31.028166
SXT	Sensient Technologies Corporation	Sensient Technologies Corp operates in the chemical industry. It manufactures colors, flavors and fragrances used to develop foods and beverage systems, cosmetic and pharmaceuticals.	{"Basic Materials","Specialty Chemicals",Chemicals}	{IPHS,EMN,IFF,VHI,KRO,TORM}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:12.006649	2020-10-11 23:38:31.028166
SYLD	Cambria Shareholder Yield	The investment seeks investment results that correspond (before fees and expenses) generally to the price and yield performance of the Cambria Shareholder Yield Index (the "underlying index").\n Under normal market conditions, the fund will invest at least 80% of its total assets in the components of the underlying index. The underlying index is comprised of equity securities issued by U.S.-based issuers. The adviser considers an issuer to be U.S.-based if it is domiciled or incorporated or has substantial business activity in the United States.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.028189	2020-10-11 23:38:31.025871
T	AT&T Inc.	AT&T Inc is engaged in provision of communications and digital entertainment services in the United States and the world. It provides fixed-line services, including voice, data, and television services to consumers and small businesses.	{"Communication Services","Telecom Services"}	{SHEN,CMCSA,CTL,USM,DIS,VZ,AMX,TDS,TMUS}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:12.05452	2020-10-11 23:38:31.021426
TAIL	Cambria Tail Risk	The investment seeks to provide income and capital appreciation from investments in the U.S. market while protecting against significant downside risk.\n The fund is actively managed and seeks to achieve its investment objective by investing in cash and U.S. government bonds, and utilizing a put option strategy to manage the risk of a significant negative movement in the value of domestic equities. The adviser intends to spend approximately one percent of the fund's total assets per month to purchase put options.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:12.074571	2020-10-11 23:38:31.023428
SRPT	Sarepta Therapeutics Inc.	Sarepta Therapeutics Inc is a biotechnology company. It uses proprietary RNA-targeted technology platforms for developing pharmaceutical products to address serious diseases.	{Healthcare,Biotechnology}	{BCRX,NVAX,GILD,ALNY,NLNK,PTCT,RTRX,IONS,HEB,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:11.823524	2020-10-11 23:38:31.02423
SSTI	ShotSpotter Inc.	ShotSpotter Inc is the leader in gunfire detection and location technology providing the trusted, scalable and reliable gunfire alert and analysis.	{Technology,"Software - Application","Application Software"}	{VSTO,SSTI,XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:11.852305	2020-10-11 23:38:31.023428
STAR	iStar Inc.	iStar Inc finances, invests, and develops real estate and its related projects. The company operates manages a portfolio of senior and subordinated loans, provides lease transactions across various property types, invests in properties, and sells land.	{"Real Estate","REIT - Diversified",REITs}	{VNO,CBL,SLG,BRT,ABR,VER}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:11.867619	2020-10-11 23:38:31.021426
STI	SunTrust Banks Inc.	SunTrust Banks Inc is engaged in commercial, corporate, and retail banking operations. It provides financial services to consumer, business and corporate clients. Most SunTrust's deposits are in Southeastern U.S. states and the District of Columbia.	{"Financial Services","Banks - Regional - US",Banks}	{FCBC,UBSI,PNFP,BBT,PNC,WFC,FHN,BKU,SNV}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:11.879381	2020-10-11 23:38:31.023428
STM	STMicroelectronics N.V.	STMicroelectronics NV is a semiconductor company. It designs, develops, manufactures and markets semiconductor products used in different microelectronic applications, including automotive products, consumer products and control systems and others.	{Technology,Semiconductors}	{INTC,ASYS,MXIM,NVDA,ON,NXPI,TXN,ADI,XLK}	Semiconductors	Technology	New York Stock Exchange	\N	2020-10-11 23:12:11.89647	2020-10-11 23:38:31.023428
STRA	Strategic Education Inc.	Strayer Education Inc provides academic programs through Strayer University. It offers undergraduate and graduate degrees in the fields including business administration, accounting, information technology, and health services administration.	{"Consumer Defensive","Education & Training Services",Education}	{}	Education	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:11.920923	2020-10-11 23:38:31.023007
SUM	Summit Materials Inc. Class A	Summit Materials Inc is a top-10 aggregates supplier and top-15 cement producer in the United States. The company has three reportable segments namely West, East, and Cement segment. It generates a majority of its revenue from West Segment.	{"Basic Materials","Building Materials"}	{FSTR,TEX,CAT,MTW}	Building Materials	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:11.941937	2020-10-11 23:38:31.023007
SVA	Sinovac Biotech Ltd.	Sinovac Biotech Ltd is a biopharmaceutical company engaged in research, development, manufacturing and commercialization of vaccines that protect against human infectious diseases.	{Healthcare,Biotechnology}	{NVAX,SIGA,AZN,GSK,NVS,MRK,NLNK,NNVC,DVAX,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:11.962817	2020-10-11 23:38:31.021426
SWCH	Switch Inc. Class A	Switch Inc is a technology infrastructure company. It is engaged in the design, construction and operation of advanced data centers.	{Technology,"Information Technology Services","Application Software"}	{NTGR,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:11.97616	2020-10-11 23:38:31.028166
SWP	Stanley Black & Decker Inc. Corporate Units	Stanley Black & Decker Inc provides hand tools, power tools and accessories, mechanical access solutions, electronic security and monitoring systems, healthcare solutions, engineered fastening systems and products for various industrial applications.	{Industrials,"Tools & Accessories","Industrial Products"}	{}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:11.991559	2020-10-11 23:38:31.021426
SYF	Synchrony Financial	Synchrony Financial provides consumer financial products services to the diverse group of national and regional retailers, local merchants, manufacturers, buying groups, industry associations and healthcare service providers.	{"Financial Services","Credit Services"}	{MA,V,FLT,HTZ}	Credit Services	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:12.020299	2020-10-11 23:38:31.021426
SYV	SPDR MFS Systematic Value Equity	The investment seeks capital appreciation.\n Under normal circumstances, the fund invests at least 80% of its net assets (plus the amount of borrowings for investment purposes) in equity securities. Equity securities in which the fund invests include common stocks, preferred stocks, and securities convertible into stocks. In selecting securities for the fund, the Sub-Adviser utilizes a bottom-up approach to buying and selling investments for the fund.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.042908	2020-10-11 23:38:31.028166
TACOW	Del Taco Restaurants Inc. Warrants	Del Taco Restaurants Inc is an operator and franchisor of restaurants featuring fresh and fast made-to-order cuisine, including both Mexican inspired and American classic dishes.	{"Consumer Cyclical",Restaurants}	{}	Restaurants	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:12.063703	2020-10-11 23:38:31.024353
TANH	Tantech Holdings Ltd.	Tantech Holdings Ltd is engaged in developing and manufacturing bamboo-based charcoal products for industrial energy applications and household cooking, heating, purification, agricultural and cleaning uses.	{"Basic Materials",Coal}	{APD,FMC,POL,NL}	Coal	Basic Materials	NASDAQ Capital Market	\N	2020-10-11 23:12:12.080175	2020-10-11 23:38:31.024353
TBBK	The Bancorp Inc.	The Bancorp Inc through its subsidiary is engaged in specialty lending: securities backed lines of credit, or SBLOC, automobile fleet and other equipment leasing, Small Business Administration, or SBA.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:12.100033	2020-10-11 23:38:31.025871
SYBX	Synlogic Inc.	Synlogic Inc is a biopharmaceutical company focused on advancing the drug discovery and development platform for Synthetic Biotic medicines, which are designed using synthetic biology to genetically reprogram beneficial microbes to treat diseases.	{Healthcare,Biotechnology}	{MNTA,MLNT,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:12.015167	2020-10-11 23:38:31.02423
SYMC	Symantec Corporation	Symantec Corp provides software solutions attending to security and backup. Its products and services protect data across mobile devices, enterprise data centers and cloud-based systems.	{Technology,"Software - Application","Application Software"}	{INTC,MSFT,INTU,ADBE,SAP,IBM,VMW,ORCL,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:12.033118	2020-10-11 23:38:31.023007
SYY	Sysco Corporation	Sysco Corp is in the business of food distribution. Its primary business involves the distribution of food and related products to the foodservice or food-away-from-home industry.	{"Consumer Defensive","Food Distribution","Retail - Defensive"}	{UNFI,WILC,CORE,DIT,CHEF,SPTN}	Retail - Defensive	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:12.049796	2020-10-11 23:38:31.02423
TANNI	TravelCenters of America LLC 8.25% Senior Notes due 2028	TravelCenters of America LLC operates and franchise travel centers, standalone convenience stores, and standalone restaurants. It collects rents, royalties and other fees from our tenants and franchisees.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:12.080444	2020-10-11 23:38:31.021426
TBK	Triumph Bancorp Inc.	Triumph Bancorp Inc is a financial holding company offering traditional banking and asset management services. The company through its Banking, Factoring, Asset Management segments provides financial lending services and investment management services.	{"Financial Services","Banks - Regional - US",Banks}	{ESSA,OFED,BOTJ,ICBK}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:12.103815	2020-10-11 23:38:31.02423
TCBK	TriCo Bancshares	TriCo Bancshares is a bank holding company. The company offers banking services to retail customers and small to medium-sized businesses through its branch offices in Northern and Central California.	{"Financial Services","Banks - Regional - US",Banks}	{AMRB,CWBC,CATY,CVBF,EWBC,CVCY,BOCH,BMRC,OVLY}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:12.124212	2020-10-11 23:38:31.023007
TCRZ	THL Credit Inc. 6.75% Notes due 2022		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:12.145041	2020-10-11 23:38:31.028166
TDJ	Telephone and Data Systems Inc. 7% Senior Notes due 2060	Telephone and Data Systems Inc is a diversified telecommunications service company providing telecommunications services to wireless customers and wireline and cable connections.	{"Communication Services","Telecom Services"}	{}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:12.166394	2020-10-11 23:38:31.02423
TECH	Bio-Techne Corp	Bio-Techne Corp and is a biotechnology company which develops, manufactures and sells biotechnology reagents, instruments and clinical diagnostic products.	{Healthcare,Biotechnology}	{BDX,BIO,DHR,ENZ,GE,TMO,QGEN,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:12.181326	2020-10-11 23:38:31.024353
TELL	Tellurian Inc.	Tellurian Inc is an oil and gas exploration and production company. It is engaged in building a natural gas business that includes 26 mtpa production from Driftwood LNG, trading of LNG cargoes, and development.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{PAA,OAS,BRN,WPX,TNP,BCEI}	Oil & Gas - E&P	Energy	NASDAQ Capital Market	\N	2020-10-11 23:12:12.197393	2020-10-11 23:38:31.031328
TERP	TerraForm Power Inc.	TerraForm Power Inc owns and operates portfolio of solar and wind assets located primarily in the United States and underpinned by long-term contracts.	{Utilities,"Utilities - Independent Power Producers"}	{ETR,PCG,AY,XEL,XLU}	Utilities - Independent Power Producers	Utilities	Nasdaq Global Select	\N	2020-10-11 23:12:12.222072	2020-10-11 23:38:31.02423
TFI	SPDR Nuveen Bloomberg Barclays Municipal Bond	The investment seeks investment results that correspond generally to the price and yield performance of the Bloomberg Barclays Municipal Managed Money Index.\n The fund invests substantially all, but at least 80%, of its total assets in the securities comprising the index or in securities that the Sub-Adviser determines have economic characteristics that are substantially identical to the economic characteristics of the securities that comprise the index. The index is designed to track the U.S. long term tax-exempt bond market, including state and local general obligation bonds, revenue bonds, pre-refunded bonds, and insured bonds. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.236149	2020-10-11 23:38:31.025871
TGLS	Tecnoglass Inc.	Tecnoglass Inc designs, manufactures, distributes, and markets architectural glass and windows. The company's products portfolio includes tempered, laminated, insulating and Low-E glass.	{"Basic Materials","Building Materials"}	{PPG,DOOR,HBP,PGTI}	Building Materials	Basic Materials	NASDAQ Capital Market	\N	2020-10-11 23:12:12.253956	2020-10-11 23:38:31.028166
THFF	First Financial Corporation Indiana	First Financial Corp is a financial holding company. It offers financial services such as commercial, mortgage and consumer lending, lease financing, trust account services and depositor services through its subsidiaries.	{"Financial Services","Banks - Regional - US",Banks}	{LKFN,GABC,FRME,SRCE,HBNC,MFSF,ONB}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:12.267781	2020-10-11 23:38:31.02582
THW	Tekla World Healthcare Fund Shares of Beneficial Interest	Tekla World Healthcare Fund is a closed end fund. Its objective is to provide a high level of current income and long-term capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:12.283611	2020-10-11 23:38:31.024353
TAHO	Tahoe Resources Inc. (Canada)	Tahoe Resources Inc is a mining firm. It is engaged in the operation of mineral properties for the mining of precious metals in America. Its mines for silver, gold, lead and zinc, and earns majority from gold.	{"Basic Materials",Gold,"Metals & Mining"}	{}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:12.071238	2020-10-11 23:38:31.028166
TARO	Taro Pharmaceutical Industries Ltd.	Taro Pharmaceutical Industries Ltd is a science-based pharmaceutical company. It develops, manufacture and market Rx and OTC (over-the-counter) pharmaceutical products in the United States, Canada and Israel.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{GSK,NVS,JNJ,PFE,MRK,TEVA,MYL,TLGT,XLV}	Drug Manufacturers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:12.090048	2020-10-11 23:38:31.023007
TBI	TrueBlue Inc.	TrueBlue Inc is a provider of staffing and workforce-management solutions. It services are blue-collar staffing services including general labor, light industrial, skilled trades, aviation and transportation mechanics and technicians and drivers.	{Industrials,"Staffing & Outsourcing Services","Employment Services"}	{BBSI,KELYA,KFRC,MAN,RHI,MHH,HSON}	Employment Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:12.103219	2020-10-11 23:38:31.02582
TCBIL	Texas Capital Bancshares Inc. 6.50% Subordinated Notes due 2042		{}	{}			Nasdaq Global Select	\N	2020-10-11 23:12:12.118873	2020-10-11 23:38:31.028166
TCO	Taubman Centers Inc.	Taubman Centers Inc owns and operates regional and super-regional shopping malls in North America. The company through its subsidiaries also has operations in Asia.	{"Real Estate","REIT - Retail",REITs}	{MAC,BXP,DLR,CTT}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:12.135673	2020-10-11 23:38:31.02423
TD	Toronto Dominion Bank (The)	The Toronto-Dominion Bank and its subsidiaries provide financial products and services. It offers asset management, insurance, wealth management, investment banking, wholesale banking, personal banking and commercial banking services and others.	{"Financial Services","Banks - Global",Banks}	{BMO,BNS,CM,LB,RY,SAN,WFC,RBS}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:12.157832	2020-10-11 23:38:31.028166
TDTF	FlexShares iBoxx 5 Year Target Duration TIPS Index Fund	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the price and yield performance of the iBoxx 5-Year Target Duration TIPS Index.\n Under normal circumstances, the fund will invest at least 80% of its total assets (exclusive of collateral held from securities lending) in the securities of the underlying index. The underlying index reflects the performance of a selection of inflation protected public obligations of the U.S. Treasury, commonly known as "TIPS," with a targeted average modified adjusted duration, as defined by the underlying index, of approximately five years. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.175256	2020-10-11 23:38:31.028166
TEI	Templeton Emerging Markets Income Fund Inc.	Templeton Emerging Markets Income Fund is a closed-end fund. It seeks high, current income, with a secondary goal of capital appreciation, by investing, under normal market conditions.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:12.190419	2020-10-11 23:38:31.028166
TEUM	Pareteum Corp - Ordinary Shares	Pareteum Corp provides one stop solution for mobile, fixed and convergent telecommunications software services. Through its subsidiary it also provides layered security services for critical applications in the cloud.	{"Communication Services","Telecom Services"}	{CCOI,IBM,BT,DOX}	Communication Services	Communication Services	NASDAQ Capital Market	\N	2020-10-11 23:12:12.22336	2020-10-11 23:38:31.02582
TGA	TransGlobe Energy Corporation	TransGlobe Energy Corp is engaged in oil exploration, development and production and the acquisition of properties.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{ZN,EGY,TOT,OXY,TAT}	Oil & Gas - E&P	Energy	Nasdaq Global Select	\N	2020-10-11 23:12:12.23808	2020-10-11 23:38:31.028166
TGNA	TEGNA Inc	Tegna Inc is a media conglomerate engaged in broadcasting and digital services. The company was formerly part of Gannett, after the split has local stations like Cars.com, and CareerBuilder and operates daily newspapers in the US and UK.	{"Consumer Cyclical","Broadcasting - TV",Entertainment}	{DJCO,LEE,NYT,AHC,NWSA,FOXA,NEWM,MNI,SSP}	Entertainment	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:12.254778	2020-10-11 23:38:31.021426
THG	Hanover Insurance Group Inc	The Hanover Insurance Group Inc is a holding company. The company is engaged in providing property and casualty insurance products and services to businesses, individuals, and families through a group of independent agents and brokers.	{"Financial Services","Insurance - Property & Casualty"}	{SIGI,AFG,Y,WRB,MKL}	Insurance - Property & Casualty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:12.269536	2020-10-11 23:38:31.028166
TI	Telecom Italia S.P.A. New	Telecom Italia SpA is engaged in the communications sector. It specializes in providing fixed and mobile national and international telecommunications services.	{"Communication Services","Telecom Services"}	{TEF,BT,VIV,ORAN,VOD}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:12.284909	2020-10-11 23:38:31.031328
TIPT	Tiptree Inc.	Tiptree Inc is focused on enhancing shareholder value by generating consistent growth and profitability at its operating companies. Its subsidiaries operate in specialty insurance, asset management, senior living and specialty finance business.	{"Financial Services","Insurance - Specialty"}	{}	Insurance - Specialty	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:12.299673	2020-10-11 23:38:31.024353
TAPR	Barclays Inverse US Treasury Composite ETN	The investment seeks a return linked to the performance of the Barclays Inverse US Treasury Futures Aggregate Index™.\n The index employs a strategy that tracks the sum of the returns of periodically rebalanced short positions in equal face values of each of the 2-year, 5-year, 10-year, long-bond and ultra-long U.S. Treasury futures contracts (together, the “Treasury futures contracts”). The index is composed of an equal synthetic short position in each Treasury futures contract that is either the Treasury futures contract closest to expiration or the next Treasury futures contract scheduled to expire immediately following the front contract.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:12.088754	2020-10-11 23:38:31.02582
TBF	ProShares Short 20+ Year Treasury	The investment seeks daily investment results, before fees and expenses, that correspond to the inverse (-1x) of the daily performance of the ICE U.S. Treasury 20+ Year Bond Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index includes publicly- issued U.S. Treasury securities that have a remaining maturity greater than twenty years and have $300 million or more of outstanding face value, excluding amounts held by the Federal Reserve. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.102583	2020-10-11 23:38:31.028166
TBX	ProShares Short 7 10 Year Treasury	The investment seeks daily investment results that correspond to the inverse (-1x) of the daily performance of the ICE U.S. Treasury 7-10 Year Bond Index.\n The fund invests in derivatives that ProShare Advisors believes, in combination, should have similar daily return characteristics as the inverse (-1x) of the daily return of the index. The index includes publicly- issued U.S. Treasury securities that have a remaining maturity of greater than seven years and less than or equal to ten years and have $300 million or more of outstanding face value, excluding amounts held by the Federal Reserve. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.114431	2020-10-11 23:38:31.024353
TCF	TCF Financial Corporation	TCF Financial Corp is a bank holding company of TCF National Bank. It operates in three segments namely Consumer Banking, Wholesale Banking and Enterprise Services.	{"Financial Services","Banks - Regional - US",Banks}	{WTFC,BOKF,CBSH,FITB,HBAN,PNC,WFC,USB,ONB,ASB}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:12.125775	2020-10-11 23:38:31.025871
TCRD	THL Credit Inc.	THL Credit Inc is an externally managed, non-diversified closed-end management investment company. The company's investment approach consists of four separate and distinct phases: sourcing, selecting, structuring and supervising investments.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:12.141697	2020-10-11 23:38:31.025871
TDC	Teradata Corporation	Teradata Corp offers analytic data platforms, applications, and services for data warehousing and analytic technologies.	{Technology,"Data Storage","Computer Hardware"}	{CSCO,ADBE,DWCH,MSTR,CRAY,SAP,IBM,SPLK,ORCL,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:12.157437	2020-10-11 23:38:31.028166
TDTT	FlexShares iBoxx 3 Year Target Duration TIPS Index Fund	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the price and yield performance of the iBoxx 3-Year Target Duration TIPS Index.\n Under normal circumstances, the fund will invest at least 80% of its total assets (exclusive of collateral held from securities lending) in the securities of the underlying index. The underlying index reflects the performance of a selection of inflation protected public obligations of the U.S. Treasury, commonly known as "TIPS," with a targeted average modified adjusted duration, as defined by the underlying index, of approximately three years. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.174748	2020-10-11 23:38:31.021426
TECL	Direxion Technology Bull 3X Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the daily performance of the Technology Select Sector Index.\n The fund invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, ETFs that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index includes domestic companies from the technology sector. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.187426	2020-10-11 23:38:31.023428
TEVA	Teva Pharmaceutical Industries Limited American Depositary Shares	Teva Pharmaceutical Industries Ltd is a pharmaceutical company which develops, produces and markets generic and specialty medicines which include chemical and therapeutic medicines in a variety of dosage forms and central nervous system medicines.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{AZN,GSK,NVS,PFE,RDY,SNY,MRK,MYL,XLV}	Drug Manufacturers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:12.224902	2020-10-11 23:38:31.023428
TFX	Teleflex Incorporated	Teleflex Inc designs, develops, manufactures and supplies single-use medical devices used by hospitals and healthcare providers for common diagnostic and therapeutic procedures in critical care and surgical applications.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{ABMD,IART,MMSI,ANGO,COO,BDX,BSX,GE,JNJ,XLV}	Medical Instruments & Equipment	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:12.237455	2020-10-11 23:38:31.021426
TGS	Transportadora de Gas del Sur SA TGS	Transportadora de Gas del Sur SA is one of the largest transporter of natural gas in Latin America. It also processes and markets natural gas liquids for domestic and international sale. The company also operates a telecommunications network.	{Utilities,"Utilities - Regulated Gas","Utilities - Regulated"}	{PBR,WGP,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:12.256577	2020-10-11 23:38:31.023007
TAP	Molson Coors Brewing Company Class B	Molson Coors Brewing Co is engaged in brewing activities. It has a diverse portfolio of owned and partner brands, including core brands Carling, Coors Light, Molson Canadian and Staropramen, as well as craft and specialty beers such as Blue Moon.	{"Consumer Defensive","Beverages - Brewers","Beverages - Alcoholic"}	{BREW,SAM,CCU,STZ.B,BUD}	Beverages - Alcoholic	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:12.086246	2020-10-11 23:38:31.023428
TBPH	Theravance Biopharma Inc.	Theravance Biopharma Inc is a diversified biopharmaceutical company. It is engaged in discovery, research, development and commercialization of human therapeutics.	{Healthcare,Biotechnology}	{CORT,CPIX,COLL,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:12.109184	2020-10-11 23:38:31.023007
TCFC	The Community Financial Corporation	The Community Financial Corp is a bank holding company. It is engaged in the commercial and retail banking business, including the acceptance of deposits, and the origination of loans to individuals, associations, partnerships and corporations.	{"Financial Services","Banks - Regional - US",Banks}	{TCBK,THFF,CTBI,OLBK,MFNC,SONA,HBMD,ASB}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:12.127785	2020-10-11 23:38:31.021426
TCP	TC PipeLines LP representing Limited Partner Interests	TC Pipelines LP acquires, owns and participates in the management of energy infrastructure assets in North America. Its pipeline systems transport natural gas in the United States. It earns revenue by charging rates for transporting natural gas.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{ENB,TRP,EPD,DCP,TRGP,SMLP,ENLK}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:12.139771	2020-10-11 23:38:31.02582
TDA	Telephone and Data Systems Inc. 5.875% Senior Notes due 2061	Telephone and Data Systems Inc is a diversified telecommunications service company providing telecommunications services to wireless customers and wireline and cable connections.	{"Communication Services","Telecom Services"}	{}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:12.159629	2020-10-11 23:38:31.023007
TDS	Telephone and Data Systems Inc.	Telephone and Data Systems Inc is a diversified telecommunications service company providing telecommunications services to wireless customers and wireline and cable connections.	{"Communication Services","Telecom Services"}	{AMZN,SHEN,CTL,USM,VZ,T,SPOK,HPQ,TMUS}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:12.172859	2020-10-11 23:38:31.023007
TERM	EquityCompass Tactical Risk Manager	The investment seeks to provide long term capital appreciation with capital preservation as a secondary objective.\n The fund will seek to achieve its investment objectives by investing in equity securities of companies domiciled in the U.S. or listed on a U.S. exchange. The fund's strategy seeks to provide exposure to U.S. listed equity securities and to avoid large, prolonged market losses and reduce volatility. It may also invest in inverse ETFs which seek to provide investment results that match a negative return of the performance of an underlying index like the S&P 500 Index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.219297	2020-10-11 23:38:31.028166
TG	Tredegar Corporation	Tredegar Corporation and its subsidiaries manufactures polyethylene plastic films, polyester films and aluminum extrusions. The company's reportable business segments are PE Films, Flexible Packaging Films and Aluminum Extrusions.	{Industrials,Conglomerates}	{RAVN,CMT,MLI,BERY,FFHL}	Conglomerates	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:12.237926	2020-10-11 23:38:31.023007
TGP	Teekay LNG Partners L.P.	Teekay LNG Partners LP is a provider of marine transportation services for liquefied natural gas, liquefied petroleum gas and crude oil.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{GLNG,TNP,TK,SFL,TOO,TNK,GMLP,DHT,ASC,FRO}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:12.254174	2020-10-11 23:38:31.02423
THM	International Tower Hill Mines Ltd. (Canada)	International Tower Hill Mines Ltd is a mineral exploration company. It is engaged in the business of acquiring, exploring and evaluating mineral properties. The Company controls a 100% interest in its Livengood Gold Project in Alaska, USA.	{"Basic Materials",Gold,"Metals & Mining"}	{AEM,NG,SA,AU}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:12.272257	2020-10-11 23:38:31.021426
TK	Teekay Corporation	Teekay Corp is a provider of crude oil and gas marine transportation services. It also offers offshore oil production, storage and offloading services, mainly under long-term, fixed-rate contracts.	{Energy,"Oil & Gas Integrated","Oil & Gas - Integrated"}	{GLNG,TNP,NAT,SFL,TGP,TOO,TNK,ASC}	Oil & Gas - Integrated	Energy	New York Stock Exchange	\N	2020-10-11 23:12:12.313655	2020-10-11 23:38:31.02423
TLI	Western Asset Corporate Loan Fund Inc	Western Asset Corporate Loan Fund Inc is a United States based non-diversified, closed-end management investment company. Its investment objective is to maximize current income consistent with prudent efforts to preserve capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:12.329408	2020-10-11 23:38:31.028166
TM	Toyota Motor Corporation	Toyota Motor Corp designs, manufactures and sells sedans, minivans, compact cars, sport-utility vehicles, trucks and related parts and accessories. It also provides financing, vehicle and equipment leasing and certain other financial services.	{"Consumer Cyclical","Auto Manufacturers",Autos}	{TSLA,HMC,F}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:12.344194	2020-10-11 23:38:31.023428
TAT	TransAtlantic Petroleum Ltd (Bermuda)	TransAtlantic Petroleum Ltd is an oil and natural gas company. It is engaged in acquisition, exploration, development and production of oil and natural gas.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{ZN,EGY,TELL}	Oil & Gas - E&P	Energy	NYSE American	\N	2020-10-11 23:12:12.092648	2020-10-11 23:38:31.02423
TCBI	Texas Capital Bancshares Inc.	Texas Capital Bancshares Inc is engaged in providing lending services to businesses. Its services included credit solutions, treasury and liquidity services, mortgage finance, private wealth solution and personal banking solution.	{"Financial Services","Banks - Regional - US",Banks}	{FFIN,IBOC,SBSI,HWC,CFR,PB,IBTX,LTXB}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:12.11848	2020-10-11 23:38:31.02423
TCMD	Tactile Systems Technology Inc.	Tactile Systems Technology Inc is a medical technology company. The Company develops and provides medical devices for the treatment of chronic swelling condition at home. Its products include Flexitouch System, the Entre System, and the ACTitouch System.	{Healthcare,"Medical Devices"}	{XLV}	Medical Devices	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:12.135498	2020-10-11 23:38:31.023007
TCTL	Premise Capital Frontier Advantage Diversified Tactical	The investment seeks to track the performance, before fees and expenses, of the Premise Capital Frontier Advantage Diversified Tactical Index (the "index").\n Under normal circumstances, at least 80% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in the component securities of the index. The index consists of an investible portfolio of ETFs ("underlying ETFs") with exposure to major U.S. and non-U.S. asset classes.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:12.151113	2020-10-11 23:38:31.021426
TDIV	First Trust NASDAQ Technology Dividend Index Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the NASDAQ Technology Dividend IndexSM.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in the common stocks and/or depositary receipts included in the index. The index is owned and was developed by Nasdaq, Inc. (the "index provider"). The index is calculated and maintained by the index Provider. The index includes up to 100 technology and telecommunications companies that pay a regular or common dividend. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:12.166189	2020-10-11 23:38:31.021426
TEAM	Atlassian Corporation Plc	Atlassian Corporation PLC is a holding company engaged in designing, development, licensing and maintaining of software and the provisioning of software hosting services to help teams organize, discuss and complete their work.	{Technology,"Software - Application","Application Software"}	{PRGS,PXLW,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:12.178257	2020-10-11 23:38:31.031328
TEDU	Tarena International Inc.	Tarena International Inc through its subsidiaries provides professional education services in China. It offers courses in IT subjects, non-IT subjects and kid education programs.	{"Consumer Defensive","Education & Training Services",Education}	{FC,EDU,DL}	Education	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:12.190878	2020-10-11 23:38:31.028166
TETF	ETF Industry Exposure & Financial Services	The investment seeks to provide investment results that, before fees and expenses, track the total return performance of the Toroso ETF Industry Index.\n The fund normally invests at least 80% of its total assets in securities of the index. The index is designed to measure the performance of an investable universe of publicly-traded companies that directly or indirectly provide services or support to exchange-traded funds ("ETFs"), including but not limited to the management, servicing, trading or sale of ETFs ("ETF Activities"), as determined by Toroso Investments, LLC (the "index provider"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.223431	2020-10-11 23:38:31.031328
TFSL	TFS Financial Corporation	TFS Financial Corp is the holding company of the Third Federal Savings and Loan Association based in the United States. Its business consists of originating and servicing residential real estate mortgage loans and attracting retail savings deposits.	{"Financial Services","Banks - Regional - US",Banks}	{FDEF,UCFC,CZWI,FBC,WAYN}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:12.236858	2020-10-11 23:38:31.028166
TGI	Triumph Group Inc.	Triumph Group Inc designs, manufactures, repairs, overhauls and distributes a portfolio of aerostructures, aircraft components, accessories, subassemblies and systems. It serves original equipment manufacturers and commercial and regional airlines.	{Industrials,"Aerospace & Defense"}	{TATT,HEI,CVU,BA,ESL,HON,TXT,UTX,SPR}	Aerospace & Defense	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:12.249531	2020-10-11 23:38:31.028166
THRM	Gentherm Inc	Gentherm Inc is an automotive parts manufacturer. The business activities of the group are functioned through Automotive and Industrial segments. Its portfolio comprises of thermal temperature control devices and seat comfort cable systems.	{"Consumer Cyclical","Auto Parts",Autos}	{MGA,MPAA,STRT,SPAR,TEN,LEA,APTV,VC}	Autos	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:12.274396	2020-10-11 23:38:31.025871
TITN	Titan Machinery Inc.	Titan Machinery Inc is engaged in retail sale, service and rental of agricultural and construction machinery. It sells and repairs agricultural equipment, including machinery and attachments for large-scale farming and home and gardening purposes.	{Industrials,"Industrial Distribution"}	{TSCO,CONN,AGCO,GWW,PLOW,HDS}	Industrial Distribution	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:12.313501	2020-10-11 23:38:31.031328
TAYD	Taylor Devices Inc.	Taylor Devices Inc is engaged in the design, development, manufacture and marketing of shock absorption, rate control and energy storage devices for use in machinery, equipment and structures.	{Industrials,"Diversified Industrials","Industrial Products"}	{ASTE,NDSN,DCO,UTX,AGX}	Industrial Products	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:12.094081	2020-10-11 23:38:31.021426
TCCO	Technical Communications Corporation	Technical Communications Corp designs, develop, manufactures, distributes, and markets communications security equipment. The company's products consist of voice, data and facsimile encryptors and many more.	{Technology,"Communication Equipment"}	{CSCO,MRCY,CAMP,GD,ESP,MSI,XLK}	Communication Equipment	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:12.125451	2020-10-11 23:38:31.023428
TCS	Container Store (The)	The Container Store Group Inc a retailer of storage & organization products in USA. Its operates in two segment: The Container store and Elfa.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{AMZN,TUES,BBBY,KIRK,ETH,PIR,WMT,WSM,TGT,SNBR}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:12.14864	2020-10-11 23:38:31.031328
TDG	Transdigm Group Incorporated Inc.	TransDigm Group Inc is a designer, producer and supplier of engineered aircraft components for use on commercial and military aircraft in service.	{Industrials,"Aerospace & Defense"}	{ATRO,WWD,CVU,BA,HON,MOG.A,SPR,AIRI}	Aerospace & Defense	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:12.165284	2020-10-11 23:38:31.024353
TECD	Tech Data Corporation	Tech Data Corp is a wholesale distributor of technology products. It operates through the segment of distributing technology products, logistics management and other value-added services.	{Technology,"Computer Distribution","Computer Hardware"}	{SCSC,NSIT,CNXN,PCMI,ARW,SNX,CDW,AVT,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:12.179073	2020-10-11 23:38:31.025871
TEF	Telefonica SA	Telefonica SA is a telecommunications group which provides fixed and mobile telephony, broadband, internet, data traffic, pay TV and other digital services. It serves over 300 million clients across more than 20 countries.	{"Communication Services","Telecom Services"}	{AMX,BT,TI,VIV,ORAN,VOD}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:12.190581	2020-10-11 23:38:31.021426
TFLO	iShares Treasury Floating Rate Bond	The investment seeks to track the investment results of the Bloomberg Barclays U.S. Treasury Floating Rate Index, which is composed of U.S. Treasury floating rate bonds.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The underlying index is a market capitalization-weighted index that measures the performance of floating rate public obligations of the U.S. Treasury.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.236927	2020-10-11 23:38:31.024353
TGT	Target Corporation	Target Corp is one of the largest department store retailer in North America. It is engaged in operating general merchandise discount stores.	{"Consumer Defensive","Discount Stores","Retail - Defensive"}	{}	Retail - Defensive	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:12.256478	2020-10-11 23:38:31.031328
THGA	The Hanover Insurance Group Inc. 6.35% Subordinated Debentures due 2053	The Hanover Insurance Group Inc is a holding company. The company is engaged in providing property and casualty insurance products and services to businesses, individuals, and families through a group of independent agents and brokers.	{"Financial Services","Insurance - Property & Casualty"}	{}	Insurance - Property & Casualty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:12.270396	2020-10-11 23:38:31.028166
TIER	TIER REIT Inc.	Tier REIT Inc is a self-managed real estate investment trust. The company is engaged in providing well-managed commercial office properties located in select markets throughout the United States.	{"Real Estate","REIT - Office",REITs}	{NYMT,O,NTP}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:12.286231	2020-10-11 23:38:31.028166
TIPZ	PIMCO Broad U.S. TIPS Index Exchange-Traded Fund	The investment seeks to provide total return that closely corresponds, before fees and expenses, to the total return of the ICE BofAML US Inflation-Linked Treasury Index.\n The fund invests at least 80% of its total assets (exclusive of collateral held from securities lending) in the component securities of the ICE BofAML US Inflation-Linked Treasury Index (the "underlying index"). The underlying index is an unmanaged index comprised of Treasury Inflation-Protected Securities ("TIPS").	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.299867	2020-10-11 23:38:31.025871
TJX	TJX Companies Inc. (The)	TJX Companies Inc is an off-price apparel and home fashions retailer in the United States and other countries. Its stores offer family apparel, including footwear and accessories, home fashions, such as home basics, accent furniture, lamps, among others.	{"Consumer Cyclical","Apparel Stores","Retail - Apparel & Specialty"}	{}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:12.313508	2020-10-11 23:38:31.028166
TLGT	Teligent Inc.	Teligent Inc is a specialty generic pharmaceutical company. It markets and sells generic topical and branded generic injectable pharmaceutical products in the United States and Canada.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{NVAX,CERS,ACET,NVS,JNJ,PFE,MRK,TARO,MYL,VICL,XLV}	Drug Manufacturers	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:12.32573	2020-10-11 23:38:31.021426
YELP	Yelp Inc.	Yelp Inc is a provider of online platforms to connect buyers and sellers. It generates revenue by selling advertising space to businesses mainly in the United States.	{Technology,"Internet Content & Information","Online Media"}	{MSFT,FB,CRCM,GOOGL,XLK}	Online Media	Technology	New York Stock Exchange	\N	2020-10-11 23:12:13.705842	2020-10-11 23:38:31.02423
TBT	ProShares UltraShort Lehman 20 Year Treasury	The investment seeks daily investment results, before fees and expenses, that correspond to two times the inverse (-2x) of the daily performance of the ICE U.S. Treasury 20+ Year Bond Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index includes publicly-issued U.S. Treasury securities that have a remaining maturity greater than twenty years and have $300 million or more of outstanding face value, excluding amounts held by the Federal Reserve. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.113518	2020-10-11 23:38:31.021426
TCI	Transcontinental Realty Investors Inc.	Transcontinental Realty Investors Inc is engaged in the business of acquisition, development and ownership of income-producing residential and commercial real estate properties.	{"Real Estate","REIT - Diversified",REITs}	{INTG,MAYS,NEN,IOR,CTO,ARL,HHC,AHH,IRT}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:12.134397	2020-10-11 23:38:31.028166
TDE	Telephone and Data Systems Inc. 6.875% Senior Notes due 2059	Telephone and Data Systems Inc is a diversified telecommunications service company providing telecommunications services to wireless customers and wireline and cable connections.	{"Communication Services","Telecom Services"}	{}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:12.158361	2020-10-11 23:38:31.023428
TDW	Tidewater Inc.	Tidewater Inc is a provider of larger offshore service vessels to the global energy industry. It serves the customers who are operating in more remote, deeper and increasingly hostile environments in order to meet the world's energy demands.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{PHII,CKH,SPN,HAL,OII,HOS,BRS,ERA,NAO}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:12.178286	2020-10-11 23:38:31.028166
TEN	Tenneco Inc. Class A Voting	Tenneco Inc produces clean air and ride performance products and systems for light vehicle, commercial truck, off-highway and other vehicle applications. It serves original equipment vehicle manufacturers and repair and replacement markets or aftermarket.	{"Consumer Cyclical","Auto Parts",Autos}	{BWA,CMI,SMP,AXL,DAN,LEA,MTOR,FOXF}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:12.20042	2020-10-11 23:38:31.024353
TGEN	Tecogen Inc.	Tecogen Inc produces commercial and industrial natural gas fueled engine driven, combined heat and power products.	{"Basic Materials","Building Materials"}	{NSSC,CUO,LIQT}	Building Materials	Basic Materials	NASDAQ Capital Market	\N	2020-10-11 23:12:12.245947	2020-10-11 23:38:31.023428
THD	iShares Inc MSCI Thailand	The investment seeks to track the investment results of the MSCI Thailand IMI 25/50 Index.\n The fund generally will invest at least 90% of its assets in the component securities of the index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index consists of stocks traded primarily on the Stock Exchange of Thailand. A capping methodology is applied to issuer weights so that no single issuer of a component exceeds 25% of the index weight, and all issuers with weight above 5% do not cumulatively exceed 50% of the index weight. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.260779	2020-10-11 23:38:31.024353
THS	Treehouse Foods Inc.	Treehouse Foods Inc along with its subsidiaries is engaged in manufacturing consumer packaged food and beverage. Its products include snack nuts, retail bakery, beverages, salad dressings, cookies and crackers, cereals, pickles, Mexican & other sauces.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{LANC,HAIN,CPB,CAG,GIS,K,SJM,BGS,MKC.V,KHC}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:12.275503	2020-10-11 23:38:31.023007
TILE	Interface Inc.	Interface Inc designs, produces and sells modular carpets, also known as carpet tiles. The Company markets modular carpet under brand names such as Interface and FLOR. Its geographic markets include Americas, Europe and Asia-Pacific.	{"Consumer Cyclical","Textile Manufacturing","Homebuilding & Construction"}	{DXYN,MLHR,SCS,AIN,MHK,HNI,KNL,AWI,UFI}	Homebuilding & Construction	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:12.289145	2020-10-11 23:38:31.023428
TISA	Top Image Systems Ltd.	Top Image Systems Ltd is engaged in the development and marketing of a variety of information recognition systems and technologies and automated document capture solutions for the efficient flow of information within and between organizations.	{Technology,"Software - Application","Application Software"}	{CYRN,MITK,ATTU,XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:12.303267	2020-10-11 23:38:31.02582
TKR	Timken Company (The)	The Timken Co engineers, manufactures and markets bearings, transmissions, gearboxes, chain, and related products under Timken brand, and offers a spectrum of power system rebuild and repair services around the world.	{Industrials,"Tools & Accessories","Industrial Products"}	{}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:12.316311	2020-10-11 23:38:31.023428
TLND	Talend S.A.	Talend SA is engaged in providing open source software. Its software platform, Data Fabric, integrates data and applications in real time across modern big data and cloud environments as well as traditional systems.	{Technology,"Software - Infrastructure","Application Software"}	{OTEX,PANW,XLK}	Application Software	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:12.329444	2020-10-11 23:38:31.031328
AAL	American Airlines Group Inc.	American Airlines Group Inc operates over 6,000 flights per day to more than 300 destinations across the world from hubs in Charlotte, Chicago, Dallas/Fort Worth, Los Angeles, Miami, New York, Philadelphia, Phoenix and Washington, D.C.	{Industrials,Airlines}	{SKYW,JBLU,HA,ALGT,ALK,LUV,DAL,LTM,SAVE,UAL}	Airlines	Industrials	Nasdaq Global Select	\N	2020-10-11 23:11:59.248515	2020-10-11 23:38:31.023428
TCGP	The Carlyle Group L.P.	The Carlyle Group LP is a global alternative asset management firm. It operates business across, Corporate Private Equity, Global Credit, Real Assets, and Investment Solutions segments.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:12.131054	2020-10-11 23:38:31.024353
TCX	Tucows Inc.	Tucows Inc mainly provides internet content solutions in the United States. Its primary services span wholesale and retail domain name registration processes. Additionally, it sells mobile phones, provides broadband and consulting services.	{Technology,"Internet Content & Information","Online Media"}	{VRSN,INOD,SYNC,WIX,LEAF,XLK}	Online Media	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:12.152653	2020-10-11 23:38:31.02423
TDOC	Teladoc Health Inc.	Teladoc Inc provides telehealth platform, delivering on-demand healthcare anytime, anywhere, via mobile devices, the internet, video and phone.	{Technology,"Health Information Services","Application Software"}	{LPNT,CYH,BKD,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:12.167663	2020-10-11 23:38:31.025871
TER	Teradyne Inc.	Teradyne Inc caters to the semiconductor industry. Its products include test systems used to test semiconductors, wireless products, hard disk drives, solid state disks and circuit boards.	{Technology,"Semiconductor Equipment & Materials",Semiconductors}	{NATI,AEHR,ORBK,COHU,NANO,A,INTT,RTEC,XLK}	Semiconductors	Technology	New York Stock Exchange	\N	2020-10-11 23:12:12.214459	2020-10-11 23:38:31.023428
TGB	Taseko Mines Ltd.	Taseko Mines Ltd is engaged in the production and sale of metals, as well as related activities including exploration and mine development, within the province of British Columbia, Canada and the state of Arizona, USA.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{IAG,TECK,HBM,WRN}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:12.242133	2020-10-11 23:38:31.02582
TGTX	TG Therapeutics Inc.	TG Therapeutics Inc is a biopharmaceutical company. It is engaged in the acquisition, development & commercialization of novel treatments for B-cell malignancies and autoimmune diseases.	{Healthcare,Biotechnology}	{INFI,RDY,NAVB,HSGX,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:12.257549	2020-10-11 23:38:31.023428
THO	Thor Industries Inc.	Thor Industries Inc is engaged in manufacturing recreational vehicles in the United States and sells those vehicles in the United States and Canada. Its segments are towable recreational vehicles & motorized recreational vehicles.	{"Consumer Cyclical","Recreational Vehicles",Autos}	{PATK,SPAR,NAV,SKY,WGO,LCII}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:12.271977	2020-10-11 23:38:31.031328
TIPX	SPDR Bloomberg Barclays 1-10 Year TIPS	The investment seeks to provide investment results that correspond generally to the price and yield performance of the Bloomberg Barclays 1-10 Year U.S. Government Inflation-Linked Bond Index that tracks the 1-10 year inflation protected sector of the United States Treasury market.\n Under normal market conditions, the fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index or in securities that the Adviser determines have economic characteristics that are substantially identical to the economic characteristics of the securities that comprise the index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.302238	2020-10-11 23:38:31.023007
TLDH	FlexShares Currency Hedged Morningstar DM ex-US Factor Tilt Index Fund	The investment seeks investment results that correspond generally to the price and yield performance, before fees and expenses, of the Morningstar® Developed Markets ex-US Factor Tilt Hedged IndexSM.\n The fund will invest at least 80% of its total assets (exclusive of collateral held from securities lending) in the securities of the index (including indirect investments through the underlying fund) and in ADRs and GDRs based on the securities in the index. The underlying index reflects the performance of a selection of equity securities designed to provide broad exposure to the global developed equities markets, excluding the U.S. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.318199	2020-10-11 23:38:31.02582
TMFC	Motley Fool 100 Index ETF	The investment seeks investment results that correspond (before fees and expenses) generally to the total return performance of the Motley Fool 100 Index (the "index").\n Under normal circumstances, at least 80% of the fund's total assets (exclusive of any collateral held from securities lending) will be invested in the component securities of the index. The index was established by The Motley Fool in 2017 and is a proprietary, rules-based index designed to track the performance of the 100 largest, most liquid U.S. companies that have been recommended by The Motley Fool's analysts and newsletters. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:12.346221	2020-10-11 23:38:31.02423
TNDM	Tandem Diabetes Care Inc.	Tandem Diabetes Care Inc is a provider of medical care devices in the United States. Its products aid in the delivery of insulin and getting accurate readings through its software application.	{Healthcare,"Medical Devices"}	{ICUI,DXCM,PODD,JNJ,MDT,XLV}	Medical Devices	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:12.370794	2020-10-11 23:38:31.023428
TOT	Total S.A.	Total SA is an integrated oil and gas company. It is engaged in exploring, producing, refining oil and gas and related products.	{Energy,"Oil & Gas Integrated","Oil & Gas - Integrated"}	{SU,XOM,BP,CEO,CVX,MRO,RDS.A,HES}	Oil & Gas - Integrated	Energy	New York Stock Exchange	\N	2020-10-11 23:12:12.389186	2020-10-11 23:38:31.02582
TCON	TRACON Pharmaceuticals Inc.	TRACON Pharmaceuticals Inc is a clinical stage biopharmaceutical company. The company is engaged in the development and commercialization of novel targeted therapeutics for cancer, age-related macular degeneration and fibrotic diseases.	{Healthcare,Biotechnology}	{BCRX,AVEO,ATNM,ATRA,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:12.140603	2020-10-11 23:38:31.023428
TDI	Telephone and Data Systems Inc. Sr Nt	Telephone and Data Systems Inc is a diversified telecommunications service company providing telecommunications services to wireless customers and wireline and cable connections.	{"Communication Services","Telecom Services"}	{}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:12.166208	2020-10-11 23:38:31.031328
TECK	Teck Resources Ltd	Teck Resources Ltd is a mining company whose activities include exploration, development, processing, smelting, refining and reclamation. It has operations in Canada, United States, Chile and Peru. The company's products are coal, copper, zinc and lead.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{NG,GG,BBL,FCX,BHP,SCCO,VALE,TRQ}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:12.183114	2020-10-11 23:38:31.023007
TENX	Tenax Therapeutics Inc.	Tenax Therapeutics Inc is a specialty pharmaceutical company. The company is engaged in developing and commercializing pharmaceutical products for critical care patients and its products include Levosimendan and Oxycyte.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:12.199203	2020-10-11 23:38:31.023007
TEX	Terex Corporation	Terex Corp is a global manufacturer of a wide variety of construction and materials processing equipment such as cranes, aerial work platforms, and rock crushers.	{Industrials,"Farm & Construction Equipment","Farm & Construction Machinery"}	{CMCO,ASTE,MNTX,CAT,NC,OSK,HY,MTW}	Farm & Construction Machinery	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:12.230413	2020-10-11 23:38:31.02423
TGH	Textainer Group Holdings Limited	Textainer Group Holdings Ltd operates as a lessor of intermodal containers based on fleet size. The company engages in the purchase, ownership, management, leasing, and disposal of a fleet of intermodal containers.	{Industrials,"Rental & Leasing Services","Consulting & Outsourcing"}	{WLFC,AYR,AER,CAI,FLY,ACY,AL}	Consulting & Outsourcing	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:12.245643	2020-10-11 23:38:31.024353
THQ	Tekla Healthcare Opportunies Fund Shares of Beneficial Interest	Tekla Healthcare Opportunities Fund is a non-diversified closed-end fund. The fund's objective is to seek current income and long-term capital appreciation through investing in companies engaged in the healthcare industry.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:12.273175	2020-10-11 23:38:31.02423
TIK	Tel-Instrument Electronics Corp.	Tel Instrument Electronics Corp is a designer and manufacturer of avionics test and measurement solutions for the commercial air transport, general aviation, and government/military aerospace and defense markets.	{Industrials,"Aerospace & Defense"}	{RADA,KTOS,GD,HRS,RTN,LLL}	Aerospace & Defense	Industrials	NYSE American	\N	2020-10-11 23:12:12.2886	2020-10-11 23:38:31.02582
TISI	Team Inc.	Team Inc is a provider of specialty industrial services, including inspection and assessment, required in maintaining high temperature and high pressure piping systems and vessels that are utilized extensively in the refining, petrochemical, among others.	{Industrials,"Business Services"}	{MTRX,ENG,GLDD,ABM,MG}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:12.304062	2020-10-11 23:38:31.023428
TLTD	FlexShares Morningstar Developed Markets ex-US Factor Tilt Index Fund	The investment seeks investment results that correspond generally to the price and yield performance, before fees and expenses, of the Morningstar® Developed Markets ex-US Factor Tilt IndexSM.\n The fund will invest at least 80% of its total assets (exclusive of collateral held from securities lending) in the securities of the index and in American ADRs and GDRs based on the securities in the index. The underlying index reflects the performance of a selection of equity securities designed to provide broad exposure to the global developed equities markets, excluding the U.S., with increased exposure (or a "tilt") to small-capitalization stocks and value stocks.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.341394	2020-10-11 23:38:31.025871
TMP	Tompkins Financial Corporation	Tompkins Financial Corp is a community-based financial services organization that offers commercial and consumer banking, leasing, trust and investment management, financial planning and wealth management, and insurance.	{"Financial Services","Banks - Regional - US",Banks}	{BDGE,AROW,FLIC,NBTB,SBNY,CBU,EVBN,CHMG}	Banks	Financial Services	NYSE American	\N	2020-10-11 23:12:12.354643	2020-10-11 23:38:31.028166
TNET	TriNet Group Inc.	Trinet Group Inc provides human resources solution for small to medium-sized businesses. Its HR solution includes multi-state payroll processing and tax administration, employee benefits programs, workers' compensation insurance, and claims management.	{Industrials,"Business Services"}	{BBSI,ADP,PAYX,EFX,CBZ,NSP,WAGE}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:12.371927	2020-10-11 23:38:31.031328
TOPS	TOP Ships Inc.	TOP Ships Inc is an owner and operator of fuel efficient eco medium range, or MR, tanker vessels focusing on the transportation of crude oil, petroleum products (clean and dirty) and bulk liquid chemicals.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{}	Transportation & Logistics	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:12.387914	2020-10-11 23:38:31.023428
TPGH	TPG Pace Holdings Corp. Class A	TPG Pace Holdings Corp is a blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:12.402161	2020-10-11 23:38:31.024353
TCPC	BlackRock TCP Capital Corp.	TCP Capital Corp is an externally managed investment company. It invests in enterprises which have a valuation of $100 million to $1.5 billion. The company primarily invests in the senior debt instruments.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:12.140876	2020-10-11 23:38:31.024353
TDF	Templeton Dragon Fund Inc.	Templeton Dragon Fund Inc is a closed-end management investment company. The fund seeks long-term capital appreciation by investing at least 45% of its total assets in equity securities of China companies.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:12.159554	2020-10-11 23:38:31.02582
TDY	Teledyne Technologies Incorporated	Teledyne Technologies Inc provides enabling technologies for industrial growth markets such as deepwater oil & gas exploration & production, oceanographic research, air & water quality environmental monitoring, & among others.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{CMTL,LMT,CUB,GD,NOC,PKI,RTN,COL,LLL,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:12.177374	2020-10-11 23:38:31.02582
TEO	Telecom Argentina SA	Telecom Argentina SA is a part of the communication service industry. Its scope of business includes fixed-line public telecommunication services, international long-distance service, data transmission and internet services.	{"Communication Services","Telecom Services"}	{TEF,AMX,TI,TSU,VIV}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:12.202163	2020-10-11 23:38:31.025871
TGC	Tengasco Inc.	Tengasco Inc is a U.S based company engaged in the business of exploration for and production of oil and natural gas. Its area of oil exploration and production is in Kansas.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{}	Oil & Gas - E&P	Energy	NYSE American	\N	2020-10-11 23:12:12.246088	2020-10-11 23:38:31.031328
THR	Thermon Group Holdings Inc.	Thermon Group Holdings Inc provides engineered thermal solutions for process industries. It also provides turnkey services and design and engineering services. It products include electric heat tracing cables, tubing bundles, control products and others.	{Industrials,"Diversified Industrials","Industrial Products"}	{FLR,MMM,VPG,DWDP}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:12.273491	2020-10-11 23:38:31.023428
TIF	Tiffany & Co.	Tiffany & Co functions in the luxury goods market. As a retailer of jewelry, it sells timepieces, sterling silver goods, china, crystal and accessories.	{"Consumer Cyclical","Luxury Goods","Retail - Apparel & Specialty"}	{CTHR,DGSE,SIG,BGI}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:12.2871	2020-10-11 23:38:31.021426
TIS	Orchids Paper Products Company	Orchids Paper Products Co manufactures bulk tissue paper, known as parent rolls, and convert parent rolls into a line of finished products, including paper towels, bathroom tissue and paper napkins, for the consumer, or at-home, market.	{"Consumer Defensive","Household & Personal Products","Consumer Packaged Goods"}	{GLT,KMB,PG,NP,CLW}	Consumer Packaged Goods	Consumer Defensive	NYSE American	\N	2020-10-11 23:12:12.303238	2020-10-11 23:38:31.028166
TKC	Turkcell Iletisim Hizmetleri AS	Turkcell Iletisim Hizmetleri AS provides mobile telephone services in Turkey. It mainly provides mobile voice and data services for mobile communications networks.	{"Communication Services","Telecom Services"}	{PTNR,TEF,MBT,CEL,VOD,VEON}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:12.315852	2020-10-11 23:38:31.024353
TLK	PT Telekomunikasi Indonesia Tbk	PT Telekomunikasi Indonesia (Persero) Tbk is an integrated telecommunications service provider in Indonesia. It provides fixed-line services in Indonesia, and through its subsidiary Telkomsel operates as a wireless carrier.	{"Communication Services","Telecom Services"}	{SKM,ORAN,PHI}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:12.328634	2020-10-11 23:38:31.02423
TMST	TimkenSteel Corporation	TimkenSteel Corp is the United States based manufacturer of alloy steel and carbon and micro-alloy steel. The company's portfolio includes SBQ bars, seamless mechanical tubing, and precision steel components.	{"Basic Materials",Steel}	{USAP,STLD,AKS,NUE}	Steel	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:12.355807	2020-10-11 23:38:31.023428
TNXP	Tonix Pharmaceuticals Holding Corp.	Tonix Pharmaceuticals Holding Corp is a clinical-stage pharmaceutical company dedicated to the development of pharmaceutical products to address public health challenges. Its drug development program is focused on delivering treatment for PTSD.	{Healthcare,Biotechnology}	{ARQL,SUPN,PRGO,AGRX,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:12.373135	2020-10-11 23:38:31.025871
TOUR	Tuniu Corporation	Tuniu Corp is a online leisure travel company in China. The company offers selection of packaged tours, including organized tours and self-guided tours, as well as travel-related services for leisure travelers.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{CTRP,MMYT,EXPE}	Travel & Leisure	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:12.389722	2020-10-11 23:38:31.024353
TIVO	TiVo Corporation	TiVo Corp is engaged in the digital entertainment industry. The company provides entertainment discovery hardware, media guides, entertainment metadata, cloud services, analytics, and advertising to its clients across the United States.	{Technology,"Software - Application","Application Software"}	{CSCO,MSFT,ADBE,AAPL,SNE,NOC,T,VIAB,GOOGL,NTN,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:12.312243	2020-10-11 23:38:31.021426
TLRD	Tailored Brands Inc.	Tailored Brands Inc is a retailer of men's suits and a provider of tuxedo rental product in the United States and Canada.	{"Consumer Cyclical","Apparel Stores","Retail - Apparel & Specialty"}	{ZUMZ,OXM,EXPR,FRAN,DXLG}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:12.335908	2020-10-11 23:38:31.02582
TMHC	Taylor Morrison Home Corporation Class A	Taylor Morrison Home Corp is public homebuilders in US. The company is a land developer, with a portfolio of lifestyle and master-planned communities. It provides a diverse assortment of homes at various prices.	{"Consumer Cyclical","Residential Construction","Homebuilding & Construction"}	{PHM,MHO,NVR,DHI,HOV,LEN,MDC,TPH}	Homebuilding & Construction	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:12.353255	2020-10-11 23:38:31.021426
TNAV	Telenav Inc.	Telenav Inc is a provider of connected car and location-based platform services. Its services consist of map and navigation platform and advertising delivery platform.	{Technology,"Internet Content & Information","Online Media"}	{MSFT,KVHI,TRMB,ORBC,GRMN,NOK,USM,T,GSAT,GOOGL,XLK}	Online Media	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:12.36483	2020-10-11 23:38:31.028166
TOLZ	ProShares DJ Brookfield Global Infrastructure	The investment seeks investment results, before fees and expenses, that track the performance of the Dow Jones Brookfield Global Infrastructure Composite Index (the "index").\n The index consists of companies domiciled globally that qualify as "pure-play" infrastructure companies-companies whose primary business is the ownership and operation of infrastructure assets, activities that generally generate long-term stable cash flows. Under normal circumstances, the fund will invest at least 80% of its total assets in component securities.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.381249	2020-10-11 23:38:31.024353
TPB	Turning Point Brands Inc.	Turning Point Brands Inc is engaged in providing tobacco products in the U.S. It sells products including moist snuff, loose leaf chewing tobacco, cigarette papers, make- your-own cigar wraps & cigar smoking tobacco, cigars & tobacco vaporizer products.	{"Consumer Defensive",Tobacco,"Tobacco Products"}	{TPB}	Tobacco Products	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:12.39629	2020-10-11 23:38:31.02423
TPR	Coach Inc.	Tapestry Inc functions in the luxury goods industry. Its products include women's and men's bags, accessories, business cases, footwear, wearables and jewelry.	{"Consumer Cyclical","Luxury Goods","Retail - Apparel & Specialty"}	{TLF,EL,MOV,GES,RL,FOSL}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:12.412781	2020-10-11 23:38:31.028166
TREE	LendingTree Inc.	LendingTree Inc is an online lender exchange that connects consumers with lenders, and provides online tools to aid consumers in their financial divisions. It provides services such as mortgages, refinance loans, home equity loans, among others.	{"Financial Services","Specialty Finance",Banks}	{WLFC,TST,TGH}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:12.434232	2020-10-11 23:38:31.025871
TRNO	Terreno Realty Corporation	Terreno Realty Corp is a real estate investment trust engaged in the acquisition, ownership, and operation of industrial real estate. It also invests in several types of industrial real estate, including warehouse/distribution, flex and trans-shipment.	{"Real Estate","REIT - Industrial",REITs}	{FR,EGP,DLR,COR,STAG,CONE,REXR,QTS}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:12.451104	2020-10-11 23:38:31.025871
TRT	Trio-Tech International	Trio-Tech International is a provider of third-party semiconductor testing and burn-in services primarily through its laboratories in Southeast Asia. The company's segments include Manufacturing, Testing services, Distribution and Real Estate.	{Technology,"Semiconductor Equipment & Materials",Semiconductors}	{CVV,ESIO,RTEC,XLK}	Semiconductors	Technology	NYSE American	\N	2020-10-11 23:12:12.468518	2020-10-11 23:38:31.023007
TRX	Tanzanian Royalty Exploration Corporation	Tanzanian Royalty Exploration Corp is a mineral resource company with exploration stage properties, which engages in acquisition of interests in the exploration of natural resource properties and the development of those properties where warranted.	{"Basic Materials",Gold,"Metals & Mining"}	{ABX,NSU,AKG,AAU}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:12.482695	2020-10-11 23:38:31.02423
TSG	The Stars Group Inc.	The Stars Group Inc is a gambling company operating online, mobile, and land-based casinos. The company offers real and simulated gambling along with games such as poker through its online platform at its desktop and mobile outlets.	{"Consumer Cyclical",Gambling,"Travel & Leisure"}	{BYD,LVS}	Travel & Leisure	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:12.497477	2020-10-11 23:38:31.02423
TST	TheStreet Inc.	TheStreet Inc is a financial news and information provider. Its business-to-business and business-to-consumer content and products provide individuals, institutional investors, and advisors with actionable information from the finance and business.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{TRI,MSFT,VALU,MORN,NWSA,SPGI}	Brokers & Exchanges	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:12.517622	2020-10-11 23:38:31.028166
TKAT	Takung Art Co. Ltd.	Takung Art Co Ltd along with its wholly owned subsidiary, operates an electronic online platform located at www.takungae.com for artists, art dealers and art investors to offer and trade in valuable artwork.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{USAT,WDAY}	Retail - Apparel & Specialty	Consumer Cyclical	NYSE American	\N	2020-10-11 23:12:12.313894	2020-10-11 23:38:31.025871
TLP	TransMontaigne Partners L.P. Transmontaigne Partners L.P. representing limited partner interests	TransMontaigne Partners LP is an oil pipeline and terminal company. It engages in terminaling and transporting refined petroleum products, crude oil, chemicals, and fertilizer, in the Southeast and Gulf Coast and along the Mississippi and Ohio rivers.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{PAA,EEP,MMP,HEP,NS,SEMG,BKEP,PSXP,PAGP}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:12.333553	2020-10-11 23:38:31.024353
TMQ	Trilogy Metals Inc.	Trilogy Metals Inc is a mineral exploration company. The Company is engaged in exploration and development of its upper kobuk mineral projects located in the Ambler mining district in Northwest Alaska, United States.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{NAK,NSU,TRQ}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:12.35535	2020-10-11 23:38:31.031328
TNP	Tsakos Energy Navigation Ltd	Tsakos Energy Navigation Ltd operates crude oil and petroleum product tankers which provide marine transportation services for national, major and other independent oil companies and refiners. It also transport liquefied natural gas.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{GLNG,GASS,CPLP,TK,TGP,STNG,KNOP,ASC,DLNG,FRO}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:12.373252	2020-10-11 23:38:31.02423
TORC	resTORbio Inc.	resTORbio Inc is a clinical-stage bio-pharmaceutical company. It is engaged in the development and commercialization of novel therapeutics for the treatment of aging-related diseases.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:12.388897	2020-10-11 23:38:31.025871
TPHS	Trinity Place Holdings Inc.	Trinity Place Holdings Inc is a real estate holding, investment, and asset management company. Its core business is to own, invest in, manage, develop or redevelop real estate assets and real estate-related securities.	{"Real Estate","Real Estate - General","Real Estate Services"}	{SMRT,CTRN,BKE,SSI}	Real Estate Services	Real Estate	NYSE American	\N	2020-10-11 23:12:12.404471	2020-10-11 23:38:31.028166
TPYP	Tortoise North American Pipeline	The investment seeks to outperform its benchmark, the Bloomberg Barclays Capital Intermediate Government/Credit Index, measured over an entire market cycle, while maintaining key risks (interest rate risk, credit risk, structure risk, and liquidity risk) similar to the benchmark.\n Under normal market conditions, the fund invests at least 80% of its net assets (plus any borrowings for investment purposes) in fixed income securities. It invests primarily in investment grade fixed income securities. The average-dollar weighted maturity of the securities in which the fund is expected to invest will generally range from 3 to 8 years.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.417912	2020-10-11 23:38:31.028166
TRCB	Two River Bancorp	Two River Bancorp is the holding company for Two River Community Bank. It provides banking services to small and medium-sized businesses, professionals and individual consumers located in Central and Northern New Jersey.	{"Financial Services","Banks - Regional - US",Banks}	{UNTY,SSFN,PGC,LBAI,PKBK,BKJ}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:12.428274	2020-10-11 23:38:31.021426
TRIB	Trinity Biotech plc	Trinity Biotech PLC develops, manufactures and markets clinical diagnostic products. Its products are used for detecting autoimmune, sexually transmitted diseases, diabetes and disorders of the liver.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{QDEL,OSUR,VRML,ABT,BIO,DHR,TMO,CEMI,XLV}	Medical Diagnostics & Research	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:12.440523	2020-10-11 23:38:31.02423
TRN	Trinity Industries Inc.	Trinity Industries Inc is a diversified industrial company. The company owns businesses providing products and services to the to the energy, transportation, chemical, and construction sectors. The company generates its revenue from Rail group.	{Industrials,Railroads,"Transportation & Logistics"}	{RAIL,ARII,GBX,CAT,GATX,WAB,IEP}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:12.451564	2020-10-11 23:38:31.021426
TRP	TransCanada Corporation	TransCanada Corp is an energy infrastructure company. Its business segments include, Natural Gas Pipelines, Liquid Pipelines and Energy. The company has pipeline and power generation assets in Canada, the United States, and Mexico.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{ENB,EPD,PAA,WMB,EEP,SEP,KMI,AMID,TCP,PBA}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:12.464853	2020-10-11 23:38:31.02582
TS	Tenaris S.A. American Depositary Shares	Tenaris SA is a global manufacturer of welded steel casing and tubing for oil and gas drilling projects. The company also manufactures various diameter pipes for oil and gas transportation projects and other industrial uses.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{SYNL,OFLX,YPF,NOV,X,TX,MT}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:12.489832	2020-10-11 23:38:31.023007
TSLX	TPG Specialty Lending Inc.	TPG Specialty Lending Inc is a US-based specialty finance company focused on lending to middle-market companies. The company generates revenues primarily in the form of interest income from the investments it holds.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:12.507119	2020-10-11 23:38:31.024353
DSS	Document Security Systems Inc.	Document Security Systems Inc operates in the security and commercial printing, packaging and plastic ID markets. It develops, markets, manufactures and sells paper and plastic products designed to protect valuable information.	{Industrials,"Business Services"}	{}	Business Services	Industrials	NYSE American	\N	2020-10-11 23:12:01.834516	2020-10-11 23:38:31.02582
TMK	Torchmark Corporation	Torchmark Corp is a life and health insurance provider. The company through its Life Insurance and Health Insurance segments offers various plans and policies to several niche target groups.	{"Financial Services","Insurance - Life"}	{AAME,UNM,CIA,CNO,FFG}	Insurance - Life	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:12.354078	2020-10-11 23:38:31.02582
TNK	Teekay Tankers Ltd.	Teekay Tankers Ltd is engaged in the shipping industry. Its is a provider of marine services to the global oil and natural gas industries and an operator of medium-sized oil tankers.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{GLNG,NAT,TK,SFL,TGP,TOO,DHT,ASC,FRO,GOGL}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:12.373636	2020-10-11 23:38:31.028166
TORM	TOR Minerals International Inc	TOR Minerals International Inc produces specialty mineral products. Its products include flame retardant & smoke suppressant fillers, engineered fillers, & TiO2-color hybrid pigments, for use in plastics, coatings, paints & catalysts applications.	{}	{VHI,SXT,KRO}	Chemicals	Basic Materials	OTC Pink	\N	2020-10-11 23:12:12.388963	2020-10-11 23:38:31.028166
TPL	Texas Pacific Land Trust	Texas Pacific Land Trust is engaged in sale and leasing of land owned by it, retaining oil and gas royalties and the overall management of the land owned by it.	{"Real Estate","Real Estate - General","Real Estate Services"}	{DMLP,PHX,PBT,CRT,MTR,SBR,SJT}	Real Estate Services	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:12.404582	2020-10-11 23:38:31.023428
TRC	Tejon Ranch Co	Tejon Ranch Co is a diversified real estate development and agribusiness company committed to responsibly using its land and resources to meet the housing, employment, and lifestyle needs of Californians and create value for its shareholders.	{"Real Estate","Real Estate - General","Real Estate Services"}	{LMNR,WVVI,ALCO,JOE,FDP,HHC,FCE.A,FOR}	Real Estate Services	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:12.422483	2020-10-11 23:38:31.023007
TRI	Thomson Reuters Corp	Thomson Reuters Corp is a leader in the global information services industry. It provides information for businesses and professionals. The company allows market participants to connect, access content, and trade in a secure environment.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{MORN,FDS,DNB,MSCI,PSO,NWSA,TGNA,RELX,SPGI}	Brokers & Exchanges	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:12.438041	2020-10-11 23:38:31.028166
TRMT	Tremont Mortgage Trust	Tremont Mortgage Trust is a real estate investment trust. It focuses on originating and investing in first mortgage loans secured by middle market and transitional commercial real estate.	{"Real Estate","REIT - Diversified",REITs}	{}	REITs	Real Estate	NASDAQ Capital Market	\N	2020-10-11 23:12:12.450349	2020-10-11 23:38:31.02423
TRS	TriMas Corporation	TriMas Corp is an engineered and applied products company. It designs, manufactures, closure and dispensing systems, blind bolts, fasteners, rivets, gaskets, cylinders, natural-gas-powered engines and parts, and gas supplied to various industries.	{Industrials,"Diversified Industrials","Industrial Products"}	{CPRT,ATR,CMI,GD,GE,GEF,WOR,F,NPO,GM}	Industrial Products	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:12.465754	2020-10-11 23:38:31.02423
TRVN	Trevena Inc.	Trevena Inc is a clinical stage biopharmaceutical company. It discovers, develops and commercialize therapeutics that use a novel approach to target G protein coupled receptors.	{Healthcare,Biotechnology}	{AMGN,MDCO,MRK,ACRX,CYTK,CARA,FLXN,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:12.482573	2020-10-11 23:38:31.021426
TSLA	Tesla Inc.	Tesla Inc is a vertically integrated sustainable energy company. It designs, develops, manufactures and sells high-performance fully electric vehicles and electric vehicle powertrain components.	{"Consumer Cyclical","Auto Manufacturers",Autos}	{HMC,TM,F,GM}	Autos	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:12.498762	2020-10-11 23:38:31.028166
TSRI	TSR Inc.	TSR Inc is primarily engaged in the business of providing contract computer programming services to its customers. The company offers technical computer personnel to supplement its in-house information technology (IT) capabilities.	{Technology,"Software - Application","Application Software"}	{CTG,KFRC,MAN,RHI,MHH,JOB,XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:12.512677	2020-10-11 23:38:31.023428
TTEK	Tetra Tech Inc.	Tetra Tech Inc provides consulting and engineering services to various industries. The company specializes in providing water-related services for public and private clients. It offers services from planning and forecasting to execution and maintenance.	{Industrials,"Engineering & Construction"}	{ICFI,GLDD,MYRG,CLH,WM,JEC,HIL,ACM}	Engineering & Construction	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:12.527671	2020-10-11 23:38:31.02423
TTMI	TTM Technologies Inc.	TTM Technologies Inc is a printed circuit board manufacturer. Its operating segment includes Printed Circuit Boards and electro-mechanical solutions. The company generates a majority of its revenue from the sale of Printed Circuit Boards.	{Technology,"Contract Manufacturers","Computer Hardware"}	{FLEX,SGMA,ELTK,KBAL,PLXS,JBL,BHE,SANM,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:12.539139	2020-10-11 23:38:31.023428
TLT	iShares 20+ Year Treasury Bond ETF	The investment seeks to track the investment results of the ICE U.S. Treasury 20+ Year Bond Index (the "underlying index").\n The fund generally invests at least 90% of its assets in the bonds of the underlying index and at least 95% of its assets in U.S. government bonds. The underlying index measures the performance of public obligations of the U.S. Treasury that have a remaining maturity greater than twenty years.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:12.339925	2020-10-11 23:38:31.028166
TMUS	T-Mobile US Inc.	T-Mobile US Inc is a US-based wireless company which provides wireless communications services, including voice, messaging and data in the postpaid, prepaid and wholesale markets. It also provides wireless devices and accessories.	{"Communication Services","Telecom Services"}	{SHEN,USM,VZ,AMX,T,WIFI,TDS,SPOK}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:12:12.358393	2020-10-11 23:38:31.023007
TOCA	Tocagen Inc.	Tocagen Inc is a clinical-stage, cancer-selective gene therapy company focused on developing product candidates designed to activate a patient's immune system against cancer from within. Its pipeline product includes Toca 511 and Toca FC.	{Healthcare,Biotechnology}	{PDLI,PIRS,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:12.374068	2020-10-11 23:38:31.02582
TOWN	Towne Bank	Towne Bank offers a diversified range of financial services, through the operation of banks. Its activities include traditional banking commercial mortgage brokerage and asset management services.	{"Financial Services","Banks - Regional - US",Banks}	{AMNB,FBSS,OPOF,FCBC,ANCX,BOTJ,UBSH}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:12.394892	2020-10-11 23:38:31.021426
TPVY	TriplePoint Venture Growth BDC Corp. 5.75% Notes due 2022		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:12.41827	2020-10-11 23:38:31.028166
TRCO	Tribune Media Company Class A	Tribune Media Co is a media and entertainment company. The company owns a national general entertainment cable network, a radio station, a production studio, a portfolio of real estate assets and has investments in various media and websites.	{"Consumer Cyclical","Broadcasting - TV",Entertainment}	{DJCO,LEE,NYT,AHC,NWSA,NEWM,TGNA,MNI,SSP}	Entertainment	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:12.432635	2020-10-11 23:38:31.028166
TRIP	TripAdvisor Inc.	TripAdvisor Inc provides travel related services. Its website offers reviews on restaurants, hotels and tourist destinations. It secures revenue from providing advertisement space on its website as well as through commissions on bookings.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{CTRP,MMYT,EXPE}	Travel & Leisure	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:12.447017	2020-10-11 23:38:31.023007
TROX	Tronox Limited Class A $0.01 par	Tronox Ltd is engaged in producing and marketing of titanium bearing mineral sands and titanium dioxide pigment. The Company operates under two segments, TiO2 and Alkali.	{"Basic Materials",Chemicals}	{VHI,SXT,HUN,KRO,TORM}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:12.46209	2020-10-11 23:38:31.028166
TRUE	TrueCar Inc.	TrueCar Inc operates as an Internet-based information, technology, and communication services company. It operates its platform for affinity group marketing partners, such as USAA and consumer reports, financial institutions, and other large enterprises.	{Technology,"Internet Content & Information","Online Media"}	{EBAY,GM,CRCM,TGNA,GOOGL,IPDN,AUTO,XLK}	Online Media	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:12.478553	2020-10-11 23:38:31.023428
TSCO	Tractor Supply Company	Tractor Supply Co is an operator of rural lifestyle retail stores in the United States. It is engaged in supplying the needs of recreational farmers and ranchers as well as tradesmen and small businesses.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{JCTCF,AMZN,CONN,LKQ,POOL,TITN,HD,LOW,SHW,SHOS}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:12.493357	2020-10-11 23:38:31.023428
TSN	Tyson Foods Inc.	Tyson Foods Inc processes and distributes value-added beef, chicken pork, prepared foods and related allied products, including animal and pet food ingredients. Its product brands are Wright, Jimmy Dean, Hillshire Farm, Ball Park and Sara Lee.	{"Consumer Defensive","Farm Products","Consumer Packaged Goods"}	{SAFM,HRL,BRFS,SEB,PPC}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:12.509676	2020-10-11 23:38:31.023007
TTAI	TRIMTABS ETF TRUST	The investment seeks to generate long-term returns in excess of the total return of the S&P Developed ex-U.S. BMI® Index, with similar volatility to that index.\n To pursue its investment objective, the fund invests, under normal market circumstances, at least 80% of its net assets (plus any borrowings for investment purposes) in equity securities of companies from foreign countries, or depositary receipts representing such securities. The fund seeks to invest in stocks of any market capitalization that have fundamental characteristics, which are associated with superior long-term performance, based on the extensive historical research of the Adviser.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:12.522895	2020-10-11 23:38:31.023428
TTNP	Titan Pharmaceuticals Inc.	Titan Pharmaceuticals Inc is a specialty pharmaceutical company. The company is primarily engaged in the development of pharmaceutical products.	{Healthcare,Biotechnology}	{DRRX,BDSI,VNDA,MNOV,CLDX,NVS,JNJ,PRTK,CUR,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:12.539871	2020-10-11 23:38:31.021426
TUR	iShares MSCI Turkey ETF	The investment seeks to track the investment results of the MSCI Turkey Investable Market Index (IMI).\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The underlying index consists of stocks traded primarily on the Istanbul Stock Exchange (ISE). The underlying index may include large-, mid- or small- capitalization companies. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:12.553545	2020-10-11 23:38:31.02423
TMO	Thermo Fisher Scientific Inc	Thermo Fisher Scientific Inc is a healthcare company. It offers analytical instruments, laboratory equipment, software, services, consumables, reagents and chemicals.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{HBIO,ISRG,WAT,A,BAX,BDX,BIO,DHR,TFX,PKI,XLV}	Medical Diagnostics & Research	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:12.353942	2020-10-11 23:38:31.025871
TNC	Tennant Company	Tennant Co designs, manufactures & sells products used in the maintenance of nonresidential surfaces. Its products include floor maintenance & outdoor cleaning equipment specialty surface coatings, and aftermarket parts and consumables.	{Industrials,"Diversified Industrials","Industrial Products"}	{MIDD,SXI,INS,EVI}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:12.369562	2020-10-11 23:38:31.021426
TOO	Teekay Offshore Partners L.P. representing Limited Partner Interests	Teekay Offshore Partners LP operates fleet of shuttle tanker ships which transport oil from offshore rigs to land. It also owns several storage vessels and oil tankers.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{NAT,TK,PBR,MMP,SFL,TGP,TNK,DHT,ASC,FRO}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:12.383743	2020-10-11 23:38:31.021426
TPC	Tutor Perini Corporation	Tutor Perini Corp is a construction company. The company offers construction services and management, and design-build services to private and public customers on contractual basis. It constructs and repairs all kinds of buildings across United States.	{Industrials,"Engineering & Construction"}	{STRL,PRIM,EME,FIX,GVA,JEC,FLR,KBR,ACM}	Engineering & Construction	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:12.399261	2020-10-11 23:38:31.023007
TPVG	TriplePoint Venture Growth BDC Corp.	TriplePoint Venture Growth BDC Corp is an externally managed, closed-end, non-diversified management investment company. It originates and invests in loans that have a secured collateral position and are used by venture growth stage companies.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:12.415102	2020-10-11 23:38:31.024353
TREC	Trecora Resources	Trecora Resources manufactures specialty petrochemical products and synthetic waxes. In addition, it also provides custom processing services. The majority of the company's revenue is derived from its petrochemical offerings.	{"Basic Materials",Chemicals}	{REX,XOM,SHI,PSX}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:12.433494	2020-10-11 23:38:31.031328
TRMD	TORM plc	TORM PLC owns and operates product tankers and dry bulk carriers. Its products include naphtha, gasoline, gas oil, jet fuel, and diesel oil.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:12.447464	2020-10-11 23:38:31.024353
TRPX	Therapix Biosciences Ltd.	Therapix Biosciences Ltd focuses on creating and enhancing a portfolio of technologies and assets based on cannabinoid pharmaceuticals.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:12.464472	2020-10-11 23:38:31.024353
TRUP	Trupanion Inc.	Trupanion Inc is an insurance service provider. The company offers medical insurance for cats and dogs throughout the United States, Canada and Puerto Rico. Its segments include subscription business and other business.	{"Financial Services","Insurance - Specialty"}	{CLS,NEWR,FIT}	Insurance - Specialty	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:12.47894	2020-10-11 23:38:31.031328
TSM	Taiwan Semiconductor Manufacturing Company Ltd.	Taiwan Semiconductor Manufacturing Co Ltd engages in the manufacturing, selling, packaging, testing and computer-aided design of integrated circuits and other semiconductor devices. In addition, it also manufactures masks.	{Technology,Semiconductors}	{AMAT,INTC,TSEM,XLNX,NVDA,STM,SMI,TXN,ADI,XLK}	Semiconductors	Technology	New York Stock Exchange	\N	2020-10-11 23:12:12.508771	2020-10-11 23:38:31.031328
TTD	The Trade Desk Inc.	The Trade Desk Inc provides technology platform for ad buyers. It provides a self-service platform that enables clients to purchase and manage data-driven digital advertising campaigns using their own teams.	{Technology,"Software - Application","Application Software"}	{MRIN,XLK}	Application Software	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:12.523443	2020-10-11 23:38:31.023007
TTPH	Tetraphase Pharmaceuticals Inc.	Tetraphase Pharmaceuticals Inc is a clinical-stage biopharmaceutical company. It creates novel antibiotics for serious and life-threatening multidrug-resistant infections. Its pipeline products are ERAVACYCLINE, TP-271, and TP-6076.	{Healthcare,Biotechnology}	{PFE,MRK,AKAO,NBY,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:12.540775	2020-10-11 23:38:31.028166
TUSA	First Trust Total US Market AlphaDEX ETF	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the NASDAQ AlphaDEX® Total US Market Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The advisor, using an "indexing" investment approach, attempts to replicate, before fees and expenses, the performance of the index. The index is a modified equal-dollar weighted index, comprised of U.S. exchange-listed securities of companies with capital appreciation potential.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:12.557471	2020-10-11 23:38:31.021426
TVTY	Tivity Health Inc.	Tivity Health Inc is engaged in providing fitness and health improvement programs. The company's products include SilverSneakers fitness program for older adults, Prime Fitness and WholeHealth Living.	{Healthcare,"Medical Care","Health Care Providers"}	{CRVL,INFU,PINC,XLV}	Health Care Providers	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:12.572137	2020-10-11 23:38:31.024353
TOL	Toll Brothers Inc.	Toll Brothers Inc is a luxury homebuilder in the United States. The company caters to move-up, active-adult, and second-home buyers. It also builds luxury for-sale and for-rent properties in urban centers, primarily on the East Coast.	{"Consumer Cyclical","Residential Construction","Homebuilding & Construction"}	{PHM,NVR,DHI,MTH,MDC,KBH,LEN.B,TPH,TMHC,WLH}	Homebuilding & Construction	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:12.376305	2020-10-11 23:38:31.023007
TOWR	Tower International Inc.	Tower International Inc is a manufacturer of structural metal components and assemblies used in the automotive industry. It offers body-structure stampings, frame and other chassis structures, complex welded assemblies, and suspension components.	{"Consumer Cyclical","Auto Parts",Autos}	{MGA,SUP,AXL,SRI,MLR}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:12.397012	2020-10-11 23:38:31.028166
TPRE	Third Point Reinsurance Ltd.	Third Point Reinsurance Ltd provides property and casualty reinsurance coverage to insurance and reinsurance companies in the Americas, Europe, the Middle East, Africa, and Asia.	{"Financial Services","Insurance - Reinsurance","Insurance - Specialty"}	{ACGL,GLRE,MHLD,RNR,RE,BCRH}	Insurance - Specialty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:12.414724	2020-10-11 23:38:31.021426
TRCH	Torchlight Energy Resources Inc.	Torchlight Energy Resources Inc acquires, explores, exploits, and/or develops oil and natural gas properties. Its oil and gas projects are the Orogrande Project, Hazel Project, Winkler Project and the Hunton wells project.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{KOS}	Oil & Gas - E&P	Energy	NASDAQ Capital Market	\N	2020-10-11 23:12:12.428291	2020-10-11 23:38:31.02582
TRK	Speedway Motorsports Inc.	Speedway Motorsports Inc act as a promoter, marketer and sponsor of motorsports activities in the United States.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{ISCA,CHDN,DVD}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:12.447736	2020-10-11 23:38:31.028166
TRST	TrustCo Bank Corp NY	Trustco Bank Corp N Y is a savings and loan holding company. It is engaged in providing general banking services to individuals, partnerships, and corporations.	{"Financial Services","Banks - Regional - US",Banks}	{ESBK,GCBC,LSBK,PBCT,NYCB,CARV,BHLB,NFBK}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:12.467479	2020-10-11 23:38:31.023428
TRVG	trivago N.V.	trivago NV develops and operates an online platform for travelers. Its platform allows travelers to make informed decisions by personalizing their hotel search and providing access to a deep supply of hotel information and prices.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{RENN}	Travel & Leisure	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:12.482653	2020-10-11 23:38:31.02582
TSI	TCW Strategic Income Fund Inc.	Tcw Strategic Income Fund Inc operates as a closed end management investment company. Its investment objective is to seek a total return comprised of current income and capital appreciation by investing in wide range of securities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:12.498805	2020-10-11 23:38:31.02582
TSS	Total System Services Inc.	Total System Services Inc is a payment solutions provider. It provides services to financial and nonfinancial institutions. Its services include issuing services, acquiring services, prepaid solutions and merchant solutions.	{"Financial Services","Credit Services"}	{FISV,ACIW,GPN,FIS,MA,V,GDOT,MGI}	Credit Services	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:12.516663	2020-10-11 23:38:31.021426
TTM	Tata Motors Ltd Limited	Tata Motors Ltd is an automotive manufacturer. The company's product portfolio includes motor vehicles and trucks. Some of its brands include Tata its flagship brand, Jaguar, and Land Rover. It is based in India and sells its products worldwide.	{"Consumer Cyclical","Auto Manufacturers",Autos}	{}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:12.53483	2020-10-11 23:38:31.031328
TUES	Tuesday Morning Corp.	Tuesday Morning Corp is principally engaged in operating discount retail stores in US. It offers a broad range of merchandise, including decorative home accessories, furniture, housewares, crafts, goods related to pets, and seasonal goods.	{"Consumer Defensive","Discount Stores","Retail - Defensive"}	{BBBY,FRED,DLTR,WMT,WSM,TGT,BIG,TCS}	Retail - Defensive	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:12.54933	2020-10-11 23:38:31.028166
TVC	Tennessee Valley Authority		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:12.566057	2020-10-11 23:38:31.028166
TWTR	Twitter Inc.	Twitter Inc is a social networking platform for public self-expression and conversation in real time. Its services are live, which includes live commentary, live connections, and live conversations. It generates a majority of its revenue from advertising.	{Technology,"Internet Content & Information","Online Media"}	{MSFT,SINA,TRIP,FB,GOOGL,LEAF,RENN,MEET,XLK}	Online Media	Technology	New York Stock Exchange	\N	2020-10-11 23:12:12.586076	2020-10-11 23:38:31.025871
TYHT	Shineco Inc.	Shineco Inc through its subsidiaries, produces, distributes, and sells health and well-being focused plantbased products under the Tenethealth and Tenet Bojian brands in China.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{SANW,TYHT,SMPL,XLV}	Drug Manufacturers	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:12.603354	2020-10-11 23:38:31.02423
UAG	E-TRACS USB Bloomberg Commodity Index Exchange Traded Notes UBS Bloomberg CMCI Agriculture ETN	The investment seeks to track the price and performance yield, before fees and expenses, of the UBS Bloomberg CMCI Agriculture Total Return index.\n The fund is designed to be representative of the entire liquid forward curve of each commodity in the index. The index measures the collateralized returns from a diversified basket of agriculture futures contracts. It is comprised of the 12 agriculture futures contracts included in the CMCI with three target maturities for each individual commodity.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.621492	2020-10-11 23:38:31.021426
TOK	iShares MSCI Kokusai	The investment seeks to track the investment results of the MSCI Kokusai Index composed of developed market equities, excluding Japan.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is designed to measure equity market performance in those countries that MSCI Inc. (the "index provider" or "MSCI") has classified as having developed economies, excluding Japan ("DEEJ").	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.373573	2020-10-11 23:38:31.028166
TOTL	SPDR DoubleLine Total Return Tactical	The investment seeks to maximize total return.\n Under normal circumstances, the Sub-Adviser will invest at least 80% of the fund's net assets in a portfolio of fixed income securities of any credit quality. The fund may invest up to 25% of its net assets in corporate high yield securities (commonly known as "junk bonds"). It may invest up to 15% of its net assets in securities denominated in foreign currencies, and may invest beyond this limit in U.S. dollar-denominated securities of foreign issuers. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.388663	2020-10-11 23:38:31.031328
TPIC	TPI Composites Inc.	TPI Composites Inc is a manufacturer of composite wind blades. It serves the wind energy market. The Company's business operating segments are the United States, Asia, Mexico and Europe, the Middle East and Africa (EMEA).	{Industrials,"Diversified Industrials","Industrial Products"}	{POWL}	Industrial Products	Industrials	NASDAQ Global Market	\N	2020-10-11 23:12:12.404171	2020-10-11 23:38:31.02582
TPZ	Tortoise Power and Energy Infrastructure Fund Inc	Tortoise Power & Energy Infra Fd Inc is a closed ended fund. It invests in a portfolio of fixed income and equity securities issued by power and energy infrastructure companies.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:12.418662	2020-10-11 23:38:31.02423
TRHC	Tabula Rasa HealthCare Inc.	Tabula Rasa HealthCare Inc provides patient-specific, data-driven technology and solutions that enable healthcare organizations to optimize medication regimens to improve patient outcomes, reduce hospitalizations, lower healthcare costs and manage risk.	{Technology,"Health Information Services","Application Software"}	{PFPT,PFM,PCYG,XLK}	Application Software	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:12.435107	2020-10-11 23:38:31.023428
TRMK	Trustmark Corporation	Trustmark Corp is a bank holding company. It operates through Trustmark National Bank providing financial services including Commercial Banking, Consumer Banking, Insurance, Wealth Management and Trust Services, and New Market Tax Credits.	{"Financial Services","Banks - Regional - US",Banks}	{CIZN,PNFP,RNST,BBT,STI,FHN,RF,SP,BXS,PFBX}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:12.448281	2020-10-11 23:38:31.023428
TRQ	Turquoise Hill Resources Ltd.	Turquoise Hill Resources Ltd is a mining company engaged in the operation and further development of the Oyu Tolgoi copper-gold mine in southern Mongolia, which is the Company's principal and only material mineral resource property.	{"Basic Materials",Copper,"Metals & Mining"}	{EGO,AUY,K,GOLD,RIO,AU,EGI}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:12.464249	2020-10-11 23:38:31.021426
TRV	The Travelers Companies Inc.	The Travelers Companies Inc operates in property and casualty insurance industry. Its operations are divided into three segments: Business Insurance, Bond & Specialty Insurance and Personal Insurance.	{"Financial Services","Insurance - Property & Casualty"}	{SIGI,ACGL,ALL,AFG,Y,RLI,AHL,WTM}	Insurance - Property & Casualty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:12.479341	2020-10-11 23:38:31.024353
TSC	TriState Capital Holdings Inc.	TriState Capital Holdings Inc is a bank holding company. The company through its subsidiaries provides banking products and services serving the middle-market businesses in markets throughout the states of Pennsylvania, Ohio, New Jersey and New York.	{"Financial Services","Banks - Regional - US",Banks}	{ORRF,NWFL,BMTC,CCNE,MPB,STBA,UVSP,ACNB}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:12.493108	2020-10-11 23:38:31.021426
TSLF	THL Credit Senior Loan Fund of Beneficial Interest	THL Credit Senior Loan Fund is an alternative credit investment manager. Its investment strategies and credit expertise accommodate a spectrum of targeted return profiles.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:12.506084	2020-10-11 23:38:31.025871
TTAC	TrimTabs All Cap U.S. Free-Cash-Flow	The investment seeks to generate long-term returns in excess of the total return of the Russell 3000® Index (the "index"), with less volatility than the index.\n The fund is an actively managed exchange-traded fund ("ETF") that seeks to achieve its investment objective by primarily investing in the broad U.S. equity market, as represented by the index. It seeks to achieve this goal by investing in stocks with liquidity and fundamental characteristics that are historically associated with superior long-term performance.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:12.522017	2020-10-11 23:38:31.025871
TTS	Tile Shop Hldgs Inc.	Tile Shop Holdings Inc is a retailer of manufactured and natural stone tiles, setting and maintenance materials, and related accessories. Its products include natural stone, ceramic, porcelain, glass, and metal tiles.	{"Consumer Cyclical","Home Improvement Stores","Retail - Apparel & Specialty"}	{FRAN,COMM,SPWH,RH}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:12.541163	2020-10-11 23:38:31.02582
TV	Grupo Televisa S.A.	Grupo Televisa SAB is one of the largest media company. The company operates broadcast channels in Mexico and produces pay-television channels for its subscribers in North America, Asia, Europe, and Latin America.	{"Consumer Cyclical","Broadcasting - TV",Entertainment}	{NFLX,CMCSA,DIS,AMX,CBS}	Entertainment	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:12.568276	2020-10-11 23:38:31.031328
DTF	DTF Tax-Free Income Inc.	To seek current income exempt from regular federal income tax consistent with preservation of capital. The Fund may not have concentration of its investments in the utilities sector of the municipal market. The Fund may not invest more than 25% of its total assets in any single industry.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.848571	2020-10-11 23:38:31.02582
TREX	Trex Company Inc.	Trex Co Inc offers an alternative composite decking from the wood that was being used traditionally. The company's composite decking offering provides low-maintenance, high-performance, eco-friendly decking products for outdoor spaces.	{"Basic Materials","Building Materials"}	{UFPI,AWI,HBP}	Building Materials	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:12.434469	2020-10-11 23:38:31.024353
TRNS	Transcat Inc.	Transcat Inc is a provider of accredited calibration and laboratory instrument services and a value-added distributor of professional grade test, measurement and control instrumentation. Its business segments are Service and Distribution.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{COHU,BELFA,AMOT,A,DHR,EMR,ESP,XLK}	Computer Hardware	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:12.452517	2020-10-11 23:38:31.028166
TRU	TransUnion	TransUnion is a risk and information solutions provider. It obtains financial, credit, alternative credit, identity, bankruptcy, lien, judgment, insurance claims, automotive and other relevant information.	{Industrials,"Business Services"}	{FORR,JKHY,NCI,CDK}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:12.475488	2020-10-11 23:38:31.028166
TSE	Trinseo S.A.	Trinseo SA operates in the rubber and plastic industry. It manufactures and markets emulsion polymers and plastics, including various specialty and technologically differentiated products. The majority of revenue comes from Europe.	{"Consumer Cyclical","Rubber & Plastics",Autos}	{OMN,WLK,CE,KRA,DWDP}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:12.494106	2020-10-11 23:38:31.024353
TSQ	Townsquare Media Inc. Class A	Townsquare Media Inc is an entertainment and media company offering digital marketing solutions in the United States and Canada. It owns and operates radio stations, social media properties focusing the small and mid-cap companies.	{"Consumer Cyclical","Broadcasting - Radio",Entertainment}	{BBGI,EMMS,UONEK}	Entertainment	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:12.511668	2020-10-11 23:38:31.02582
TTEC	TTEC Holdings Inc.	TTEC Holdings Inc is engaged in designing and managing customers front-to-back office processes, offering sales and marketing solutions, developing customer engagement strategies and leveraging communication.	{Industrials,"Business Services"}	{}	Business Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:12.525669	2020-10-11 23:38:31.024353
TTOO	T2 Biosystems Inc.	T2 Biosystems Inc is an in vitro diagnostic company. It has developed the proprietary platform that enables direct detection of pathogens, biomarkers and other abnormalities across various unpurified patient sample types.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{CORI,BLCM,ALQA,XLV}	Medical Diagnostics & Research	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:12.540076	2020-10-11 23:38:31.023007
TUP	Tupperware Brands Corporation	Tupperware Brands Corp is a direct selling consumer products company. It sells food preparation, storage, and serving products as well as cookware and microfiber textiles under the Tupperware brand name.	{"Consumer Cyclical","Packaging & Containers"}	{AMWD,CTIB,LCUT,LBY,EL,NWL,NUS,AVP}	Packaging & Containers	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:12.552653	2020-10-11 23:38:31.024353
TVIX	VelocityShares Daily 2x VIX Short-Term ETN	The investment seeks to replicate, net of expenses, the returns of twice (2x) the daily performance of the S&P 500 VIX Short-Term Futures index.\n The index was designed to provide investors with exposure to one or more maturities of futures contracts on the VIX, which reflects implied volatility of the S&P 500 Index at various points along the volatility forward curve. The ETNs are linked to a multiple (2x) of the daily return of the index and do not represent an investment in the VIX.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:12.568901	2020-10-11 23:38:31.028166
TWO	Two Harbors Investment Corp	Two Harbors Investment Corp is a REIT. It focuses on investing, financing and managing residential mortgage-backed securities, residential mortgage loans, mortgage servicing rights, commercial real estate debt and other financial assets.	{"Real Estate","REIT - Residential",REITs}	{NYMT,NLY,DX,ANH,IVR,CIM}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:12.584717	2020-10-11 23:38:31.028166
TYD	Direxion Daily 10-Yr Treasury Bull 3x Shrs	The investment seeks daily investment results, before fees and expenses, of 300% of the daily performance of the ICE U.S. Treasury 7-10 Year Bond Index.\n The fund invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, exchange-traded funds ("ETFs") that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index is a market value weighted index that includes publicly issued U.S. Treasury securities that have a remaining maturity of greater than seven years and less than or equal to ten years. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.603208	2020-10-11 23:38:31.024353
UAE	iShares MSCI UAE Capped ETF	The investment seeks to track the investment results of the MSCI All UAE Capped Index composed of UAE equities.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is designed to measure the equity market in the United Arab Emirates ("UAE"). The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:12.620069	2020-10-11 23:38:31.025871
TR	Tootsie Roll Industries Inc.	Tootsie Roll Industries Inc manufactures and sells confectionery products. Its manufacturing operations are located in the United States and Canada.	{"Consumer Defensive",Confectioners,"Consumer Packaged Goods"}	{JBSS,HSY,MDLZ,RMCF}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:12.422209	2020-10-11 23:38:31.025871
TRIL	Trillium Therapeutics Inc.	Trillium Therapeutics Inc is an immuno-oncology company developing therapies for the treatment of cancer. Its SIRPaFc and CD200 monoclonal antibody targets immuno regulatory pathways that tumor cells exploit to evade the host immune system.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:12.444307	2020-10-11 23:38:31.031328
TROW	T. Rowe Price Group Inc.	T. Rowe Price Group Inc is a financial services holding company. It provides company sponsored U.S. mutual funds, other sponsored pooled investment vehicles, subadvisory services, separate account management, recordkeeping, and related services.	{"Financial Services","Asset Management"}	{GROW,VRTS,AMG,WDR,FII,EV,BEN,AMP,AB}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:12.457807	2020-10-11 23:38:31.031328
TRTX	TPG RE Finance Trust Inc.	TPG RE Finance Trust Inc originates and acquires a diversified portfolio of commercial real estate related assets, consisting of senior loans, senior participation interests in first mortgage loans, and commercial mortgage-backed securities.	{"Real Estate","REIT - Diversified",REITs}	{NYMT,O,NTP}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:12.472291	2020-10-11 23:38:31.028166
TSRO	TESARO Inc.	Tesaro Inc is an oncology-focused biopharmaceutical company. It focuses on developing treatments for cancer.	{Healthcare,Biotechnology}	{AMGN,ZIOP,BMY,PFE,MRK,CLVS,INSY,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:12.51493	2020-10-11 23:38:31.02423
TTI	Tetra Technologies Inc.	Tetra Technologies Inc is a geographically diversified oil and gas services company. It is engaged in completion of fluids and associated products and services, water management, frac flowback, production well testing, and offshore rig cooling.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:12.532513	2020-10-11 23:38:31.025871
TU	Telus Corporation	TELUS Corp is engaged in providing phone, Internet access, and television services to residential and business customers. It also offers cloud-based services to business customers through its data centers.	{"Communication Services","Telecom Services"}	{RCI,BCE,BCOM,VZ,T,ORAN,S}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:12.553893	2020-10-11 23:38:31.023428
TWIN	Twin Disc Incorporated	Twin Disc Inc designs, manufactures and sells marine and heavy duty off-highway power transmission equipment. Its products include marine transmissions, surface drives, propellers, boat management systems, power-shift transmissions and controls systems.	{Industrials,"Diversified Industrials","Industrial Products"}	{BC,CAT,CMI,MPX,ALSN,MBUU}	Industrial Products	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:12.574623	2020-10-11 23:38:31.02423
TXMD	TherapeuticsMD Inc.	TherapeuticsMD Inc is a women's health care company focused on creating and commercializing products targeted exclusively for women. It is engaged in pre-commercialization activities for advanced hormone therapy pharmaceutical product.	{Healthcare,"Drug Manufacturers - Major","Drug Manufacturers"}	{AMAG,LCI,PFE,ABBV,AGRX,PTX,XLV}	Drug Manufacturers	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:12.59157	2020-10-11 23:38:31.023007
TYL	Tyler Technologies Inc.	Tyler Technologies Inc is a provider of integrated information management solutions and services to address the information technology needs in areas of operations for cities, counties, schools and other local government entities.	{Technology,"Software - Application","Application Software"}	{TRI,ZBRA,IBM,LRN,MSI,TWOU,HPQ,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:12.604691	2020-10-11 23:38:31.021426
UAA	Under Armour Inc. Class A	Under Armour Inc is a developer, marketer and distributor of branded performance apparel, footwear and accessories for men, women and youth. It markets its products under the brand name of Under Armour, Heatgear, Coldgear, and Allseasongear.	{"Consumer Cyclical","Apparel Manufacturing","Manufacturing - Apparel & Furniture"}	{COLM,GES,SKX,NKE,PVH,SQBG}	Manufacturing - Apparel & Furniture	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:12.619395	2020-10-11 23:38:31.024353
UBA	Urstadt Biddle Properties Inc.	Urstadt Biddle Properties Inc is a real estate investment trust. The Company is engaged in the acquisition, ownership and management of commercial real estate. Its properties consist of neighborhood and community shopping centers & office buildings.	{"Real Estate","REIT - Retail",REITs}	{SPG,CHMI,CIO,SNR}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:12.631412	2020-10-11 23:38:31.028166
UBP	Urstadt Biddle Properties Inc.	Urstadt Biddle Properties Inc is a real estate investment trust. The Company is engaged in the acquisition, ownership and management of commercial real estate. Its properties consist of neighborhood and community shopping centers & office buildings.	{"Real Estate","REIT - Retail",REITs}	{SPG,INN,CHMI,DOC}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:12.641916	2020-10-11 23:38:31.028166
UCI	E-TRACS USB Bloomberg Commodity Index Exchange Traded Notes UBS Bloomberg CMCI ETN	The investment seeks to track the price and performance yield, before fees and expenses, of the UBS Bloomberg Constant Maturity Commodity index.\n The fund is designed to be a diversified benchmark for commodities as an asset class. The index is comprised of 28 futures contracts with up to five different maturities for each individual commodity.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.663876	2020-10-11 23:38:31.028166
TRGP	Targa Resources Inc.	Targa Resources Corp is engaged in gathering, compressing, treating, processing and selling natural gas. It is also engaged in storing, fractionating, treating, transporting, terminaling and selling of natural gas liquid.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{ENB,E,XOM,EPD,MUR,CVX,ENLK}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:12.434731	2020-10-11 23:38:31.02423
TRMB	Trimble Inc.	Trimble Inc provides location-based solutions used in global positioning system products. The company offers 3D laser scanning, flow and application control and monitoring systems, water management, and navigation infrastructure for various applications.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{FARO,ADSK,GRMN,DHR,HON,COL,MMM,MSI,LLL,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:12.447491	2020-10-11 23:38:31.02582
TRTN	Triton International Limited	Triton International Ltd operates as a lessor of intermodal containers. The company also lease chasis which are used for the transportation of containers. The operating segments are Equipment leasing, and Equipment trading.	{Industrials,"Rental & Leasing Services","Consulting & Outsourcing"}	{MINI,WLFC,GFN,TGH,AYR,AER,CAI,ACY,AL}	Consulting & Outsourcing	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:12.47049	2020-10-11 23:38:31.025871
TSEM	Tower Semiconductor Ltd.	Tower Semiconductor Ltd is a pure-play independent specialty foundry dedicated to the manufacture of semiconductors. It manufactures ICs on silicon wafers, using the customer's proprietary circuit designs.	{Technology,Semiconductors}	{MU,QUIK,ON,TSM,IBM,TDY,SMI,TXN,QRVO,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:12.49695	2020-10-11 23:38:31.028166
TSU	TIM Participacoes S.A. American Depositary Shares (Each representing 5)	Tim Participacoes SA is a part of the telecom industry. It operates as a mobile, fixed, long-distance and data transmission operator.	{"Communication Services","Telecom Services"}	{TEO,TEF,AMX,VIV,GOOGL}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:12.521108	2020-10-11 23:38:31.028166
TTP	Tortoise Pipeline & Energy Fund Inc.	Tortoise Pipeline & Energy Fund Inc is a non-diversified, closed-end management investment company. It invests in equity securities of North American pipeline companies that transport natural gas, natural gas liquids, crude oil and refined products.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:12.540431	2020-10-11 23:38:31.02423
TUSK	Mammoth Energy Services Inc.	Mammoth Energy Services Inc is an integrated, growth-oriented oilfield service company serving companies engaged in the exploration and development of North American onshore unconventional oil and natural gas reserves.	{Energy,"Oil & Gas Integrated","Oil & Gas - Integrated"}	{PUMP}	Oil & Gas - Integrated	Energy	Nasdaq Global Select	\N	2020-10-11 23:12:12.557811	2020-10-11 23:38:31.023007
TWI	Titan International Inc. (DE)	Titan International Inc is a manufacturer of wheels, tires, wheel and tire assemblies, and undercarriage systems and components for off-highway vehicles. It operates through three segments namely Agricultural, Earthmoving/Construction, and Consumer.	{Industrials,"Farm & Construction Equipment","Farm & Construction Machinery"}	{CVGI,AGCO,CSL,CAT,CTB,SUP,DAN,GT}	Farm & Construction Machinery	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:12.573175	2020-10-11 23:38:31.023007
TWOU	2U Inc.	2U Inc is engaged in providing cloud-based software-as-a-service platform. Its platform enables nonprofit colleges and universities to deliver education to qualified students anywhere.	{Technology,"Software - Infrastructure","Application Software"}	{AMZN,TYL,JW.A,LRN,PSO,NWSA,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:12.58693	2020-10-11 23:38:31.024353
TZOO	Travelzoo	Travelzoo is an internet media company, which informs subscribers, as well as millions of Web site users, about the travel and entertainment deals available from thousands of Companies.	{Technology,"Internet Content & Information","Online Media"}	{RCL,DAL,EXPE,TRIP,XLK}	Online Media	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:12.61543	2020-10-11 23:38:31.031328
UAUD	ETNs linked to the VelocityShares Daily 4X Long AUD vs. USD Index	The investment seeks to track the daily performance of the VelocityShares Daily 4X Long AUD vs. USD Index.\nThe index is designed to provide 4 times leveraged exposure, reset daily, to changes in the spot exchange rate between an underlying pair of currencies consisting of the U.S. dollar and a foreign currency. It provides long exposure to one currency (the “long currency”) in the underlying currency pair relative to the other currency (the “reference currency”). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.630888	2020-10-11 23:38:31.023428
UBSI	United Bankshares Inc.	United Bankshares Inc is a bank holding company. It offers commercial, retail and community banking services and products. The company operates in two segments namely Community banking and Mortgage Banking.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:12.651168	2020-10-11 23:38:31.023428
UCO	ProShares Ultra Bloomberg Crude Oil	The investment seeks to provide daily investment results (before fees and expenses) that correspond to twice the daily performance of the Bloomberg WTI Crude Oil SubindexSM.\nThe “Ultra” funds seek results for a single day that match (before fees and expenses) two times (2x) the daily performance of a benchmark. It does not seek to achieve their stated objective over a period greater than a single day. The Bloomberg WTI Crude Oil SubindexSM is designed to track crude oil futures prices.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.664846	2020-10-11 23:38:31.02582
DVYL	ETRACS Monthly Pay 2x Leveraged Dow Jones Select Dividend Index ETN	The investment seeks to replicate, net of expenses, the monthly compounded 2x leveraged performance of the Dow Jones U.S. Select Dividend IndexSM.\n The index represents the top U.S. stocks as measured by dividend yield, selected annually and subject to screening and buffering criteria.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.899384	2020-10-11 23:38:31.031328
TUZ	PIMCO 1-3 Year US Treasury Index Exchage-Traded Fund	The investment seeks total return that closely corresponds, before fees and expenses, to the total return of the ICE BofAML 1-3 Year US Treasury Index.\n The fund invests at least 80% of its total assets (exclusive of collateral held from securities lending) in the component securities of the ICE BofAML 1-3 Year US Treasury Index (the "underlying index"). The underlying index is an unmanaged index comprised of U.S. dollar denominated sovereign debt securities publicly issued by the U.S. Treasury having a maturity of at least 1 year and less than 3 years.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.557656	2020-10-11 23:38:31.025871
TWN	Taiwan Fund Inc. (The)	Taiwan Fund Inc is a closed-end management investment company. Its investment objective is to seek long-term capital appreciation through investment primarily in equity securities listed in Taiwan.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:12.582752	2020-10-11 23:38:31.023428
TYME	Tyme Technologies Inc.	Tyme Technologies Inc is a clinical-stage pharmaceutical company focused on discovering and developing targeted cancer therapeutics for a broad range of oncology indications. The company is developing proprietary combination drug product.	{Healthcare,Biotechnology}	{AGTC,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:12.60607	2020-10-11 23:38:31.025871
UBNT	Ubiquiti Networks Inc.	Ubiquiti Networks Inc provides wireless and wireline network equipment for small Internet service providers and small- and midsize-business integrators.	{Technology,"Communication Equipment"}	{CSCO,PCTI,CRNT,CAMP,MSI,AVNW,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:12.638776	2020-10-11 23:38:31.024353
UDOW	ProShares UltraPro Dow30	The investment seeks daily investment results, before fees and expenses, that correspond to three times (3x) the daily performance of the Dow Jones Industrial AverageSM.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index is a price-weighted index and includes 30 large-cap, "blue-chip" U.S. stocks, excluding utility and transportation companies. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.667937	2020-10-11 23:38:31.024353
UFCS	United Fire Group Inc	United Fire Group Inc is engaged in the business of writing property and casualty insurance and life insurance and selling annuities. Its operations are divided into two segments: property and casualty insurance and life insurance.	{"Financial Services","Insurance - Property & Casualty"}	{ERIE,CINF,ACGL,AFG,CNA,L,RLI,THG,TRV,WTM}	Insurance - Property & Casualty	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:12.685867	2020-10-11 23:38:31.023007
UGE	ProShares Ultra Consumer Goods	The investment seeks daily investment results, before fees and expenses, that correspond to two times (2x) the daily performance of the Dow Jones U.S. Consumer GoodsSM Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index seeks to measure the stock performance of certain U.S. companies in the consumer goods sector of the U.S. equity market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.702107	2020-10-11 23:38:31.02582
UITB	USAA Core Intermediate-Term Bond	The investment seeks high current income without undue risk to principal.\n Under normal circumstances, the fund invests at least 80% of its net assets (plus any borrowings for investment purposes, exclusive of collateral held from securities lending) in debt securities and in derivatives and other instruments that have economic characteristics similar to such securities. The fund may not invest more than 20% of fixed-income securities (by weight of all fixed-income securities in the portfolio) in non-agency, non-government sponsored entities (GSEs), or privately issued mortgage- or asset-backed securities.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.718025	2020-10-11 23:38:31.024353
ULVM	USAA MSCI USA Value Momentum Blend Index	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the MSCI USA Select Value Momentum Blend Index.\n Under normal circumstances, the fund seeks to achieve its investment objective by investing at least 80% of its net assets in securities in the index. The index is designed to deliver exposure to equity securities of large- and mid-capitalization U.S. issuers that have higher exposure to value and momentum factors within the MSCI USA Index (the "parent index") while also maintaining moderate index turnover and lower realized volatility than traditional capitalization weighted indexes.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.737156	2020-10-11 23:38:31.023428
UNH	UnitedHealth Group Incorporated (DE)	UnitedHealth Group Inc is a diversified health care company in the United States. It designs products, provides services and applies technologies that improve access to health and well-being services, and make health care more affordable.	{Healthcare,"Health Care Plans"}	{CI,HUM,AET,CNC,ANTM,XLV}	Health Care Plans	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:12.760354	2020-10-11 23:38:31.031328
UQM	UQM Technologies Inc	UQM Technologies Inc develops, manufactures and sells power dense, high efficiency electric motors, generators, power electronic controllers and fuel cell compressors for the commercial truck, bus, automotive, marine, military and industrial markets.	{"Consumer Cyclical","Auto Parts",Autos}	{NAV,CAT,DHR,RBC,SVT,LCII}	Autos	Consumer Cyclical	NYSE American	\N	2020-10-11 23:12:12.783431	2020-10-11 23:38:31.028166
TTWO	Take-Two Interactive Software Inc.	Take-Two Interactive Software Inc is a developer, marketer and publisher of interactive entertainment for consumers around the globe. It develops and publishes products through its two wholly-owned labels Rockstar Games and 2K.	{Technology,"Electronic Gaming & Multimedia","Application Software"}	{MSFT,EA,GLUU,ATVI,SNE,PTE,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:12.547785	2020-10-11 23:38:31.031328
TVE	Tennessee Valley Authority		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:12.567091	2020-10-11 23:38:31.023428
TWNK	Hostess Brands Inc.	Hostess Brands Inc is a packaged food company. The Company is engaged in developing, manufacturing, marketing, selling and distributing fresh baked sweet goods in the United States.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{}	Consumer Packaged Goods	Consumer Defensive	NASDAQ Capital Market	\N	2020-10-11 23:12:12.583221	2020-10-11 23:38:31.028166
TZA	Direxion Small Cap Bear 3X Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the inverse (or opposite) of the daily performance of the Russell 2000® Index.\n The fund invests in swap agreements, futures contracts, short positions or other financial instruments that, in combination, provide inverse (opposite) or short leveraged exposure to the index equal to at least 80% of the fund's net assets (plus borrowing for investment purposes). The index measures the performance of approximately 2,000 small-capitalization companies in the Russell 3000® Index, based on a combination of their market capitalization and current index membership. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.614764	2020-10-11 23:38:31.02582
UBNK	United Financial Bancorp Inc.	United Financial Bancorp Inc, through its subsidiaries, provides traditional banking products and services to business organizations and individuals offering products such as residential and commercial real estate loans, and commercial business loans.	{"Financial Services","Savings & Cooperative Banks",Banks}	{PBCT,SIFI,WEBK,BHLB}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:12.637074	2020-10-11 23:38:31.028166
UCC	ProShares Ultra Consumer Services	The investment seeks daily investment results, before fees and expenses, that correspond to two times (2x) the daily performance of the Dow Jones U.S. Consumer ServicesSM Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index seeks to measure the stock performance of certain U.S. companies in the consumer services sector of the U.S. equity market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.659425	2020-10-11 23:38:31.024353
UE	Urban Edge Properties of Beneficial Interest	Urban Edge Properties is a Maryland real estate investment trust. The company owns, manages, acquires, develops, redevelops and operates retail real estate in high barrier-to-entry markets.	{"Real Estate","REIT - Diversified",REITs}	{O,DEA}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:12.676119	2020-10-11 23:38:31.021426
UGI	UGI Corporation	UGI Corp through its subsidiaries distributes, stores, transports and markets energy products and related services. It is a retail distributor of propane and butane, provides natural gas and electric services.	{Utilities,"Utilities - Regulated Gas","Utilities - Regulated"}	{CLNE,APU,NI,OKE,SRE,NJR,SJI,VVC,SPH,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:12.702383	2020-10-11 23:38:31.024353
UL	Unilever PLC	Unilever PLC is engaged in the household products industry. The company is a supplier of fast moving consumer goods. Its areas of operations are personal care, home care, foods and refreshment.	{"Consumer Defensive","Household & Personal Products","Consumer Packaged Goods"}	{CL,GIS,PG,PEP}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:12.723929	2020-10-11 23:38:31.023007
UNF	Unifirst Corporation	UniFirst Corp together with its subsidiaries is engaged in designing, manufacturing, personalizing, renting, cleaning, delivering and selling uniforms & protective clothing, including shirts, jackets, coveralls, lab coats and specialized protective wear.	{"Consumer Cyclical","Apparel Manufacturing","Manufacturing - Apparel & Furniture"}	{CTAS,HCSG,ABM,ARMK}	Manufacturing - Apparel & Furniture	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:12.753361	2020-10-11 23:38:31.021426
UPW	ProShares Ultra Utilities	The investment seeks daily investment results, before fees and expenses, that correspond to two times (2x) the daily performance of the Dow Jones U.S. UtilitiesSM Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index measures the performance of certain companies in the utilities sector of the U.S. equity market. Component companies include, among others, electric utilities, gas utilities, multi-utilities and water utilities. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.781476	2020-10-11 23:38:31.031328
URTH	Ishares MSCI World Index Fund	The investment seeks to track the investment results of the MSCI World Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is designed to measure the performance of equity securities in the top 85% of equity market capitalization, as calculated by the index provider, in certain developed market countries.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.800632	2020-10-11 23:38:31.028166
USAU	U.S. Gold Corp.	U.S. Gold Corp is a gold exploration and development company advancing high potential projects in Nevada and Wyoming. Its properties include Keystone project, Copper King project and Gold Bar North project.	{"Basic Materials",Gold,"Metals & Mining"}	{INTC,IBM,WDC,ORCL,AMD,HPQ,QBAK,NLST}	Metals & Mining	Basic Materials	NASDAQ Capital Market	\N	2020-10-11 23:12:12.812472	2020-10-11 23:38:31.031328
TWLO	Twilio Inc. Class A	Twilio Inc provides a cloud communications platform that enables developers to build, scale and operate communications within software applications through the cloud as a pay-as- you-go service.	{Technology,"Software - Application","Application Software"}	{OTEX,PANW,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:12.577686	2020-10-11 23:38:31.021426
TXRH	Texas Roadhouse Inc.	Texas Roadhouse Inc is a full-service casual dining restaurant chain, offering an assortment of steaks, ribs, fish, seafood, chicken, pork chops, vegetable plates, hamburgers, salads, and sandwiches.	{"Consumer Cyclical",Restaurants}	{DAVE,CAKE,RRGB,DENN,DRI,EAT,BDL,CHUY,BLMN}	Restaurants	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:12.595164	2020-10-11 23:38:31.028166
UA	Under Armour Inc. Class C	Under Armour Inc is a developer, marketer and distributor of branded performance apparel, footwear and accessories for men, women and youth. It markets its products under the brand name of Under Armour, Heatgear, Coldgear, and Allseasongear.	{"Consumer Cyclical","Apparel Manufacturing","Manufacturing - Apparel & Furniture"}	{UAA}	Manufacturing - Apparel & Furniture	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:12.615468	2020-10-11 23:38:31.023007
UBCP	United Bancorp Inc.	United Bancorp Inc is a bank holding company. It offers banking and financial services, which includes accepting demand, savings and time deposits and granting commercial, real estate & consumer loans.	{"Financial Services","Banks - Regional - US",Banks}	{PEBO,OVBC,FFBC,HBAN,WSBC,PRK,FMNB,SBFG,CIVB}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:12.633137	2020-10-11 23:38:31.025871
UBSH	Union Bankshares Corporation	Union Bankshares Corp is a financial holding company and a bank holding company. The Company through its community bank subsidiary provides financial services including banking, trust, and wealth management services.	{"Financial Services","Banks - Regional - US",Banks}	{AMNB,FBSS,TOWN,UBSI,ANCX,BOTJ,VBFC}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:12.651271	2020-10-11 23:38:31.025871
UEIC	Universal Electronics Inc.	Universal Electronics Inc is engaged in developing and manufacturing a series of electronic products, including wireless control products, audio-video accessories, intelligent wireless automation components, and software.	{Technology,"Consumer Electronics","Computer Hardware"}	{KOSS,LOGI,MSN,SNE,PHG,VOXX,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:12.678248	2020-10-11 23:38:31.031328
UFPT	UFP Technologies Inc.	UFP Technologies Inc is a U.S. based company designing and manufacturing various high-performance cushion packaging and specialty foam and plastic products for industrial and consumer products.	{"Consumer Cyclical","Packaging & Containers"}	{CXDC,CMT,ASPN}	Packaging & Containers	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:12.693326	2020-10-11 23:38:31.02423
UGP	Ultrapar Participacoes S.A. (New) American Depositary Shares (Each representing one)	Ultrapar Participacoes SA through its subsidiaries is engaged in distribution of LPG (liquefied petroleum gas) and fuel and related businesses, production and marketing of chemicals, and storage services for liquid bulk.	{Energy,"Oil & Gas Refining & Marketing","Oil & Gas - Refining & Marketing"}	{APU,PBR,BAK,RDS.A,VLP,DWDP}	Oil & Gas - Refining & Marketing	Energy	New York Stock Exchange	\N	2020-10-11 23:12:12.705774	2020-10-11 23:38:31.028166
UJB	ProShares Ultra High Yield	The investment seeks daily investment results, before fees and expenses, that correspond to two times (2x) the daily performance of the Markit iBoxx $ Liquid High Yield Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index is a market-value weighted index designed to provide a balanced representation of U.S. dollar denominated high yield corporate bonds for sale within the United States by means of including the most liquid high yield corporate bonds available. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.722574	2020-10-11 23:38:31.025871
ULH	Universal Logistics Holdings Inc.	Universal Logistics Holdings Inc is a provider of customized transportation & logistics solutions in United States, Mexico, Canada & Colombia. Its transportation services include dry van, flatbed, heavy haul, refrigerated, shuttle & switching operations.	{Industrials,Trucking,"Transportation & Logistics"}	{PTSI,CVTI,HTLD,MRTN,STAA,USAK,WERN,COO,KNX}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:12.731782	2020-10-11 23:38:31.024353
UMDD	UltraPro MidCap400	The investment seeks daily investment results, before fees and expenses, that correspond to three times (3x) the daily performance of the S&P MidCap 400®.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index is a measure of mid-size company U.S. stock market performance. It is a float-adjusted, market capitalization-weighted index of 400 U.S. operating companies and real estate investment trusts selected through a process. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.744156	2020-10-11 23:38:31.025871
UNIT	Uniti Group Inc.	Uniti Group Inc is a real estate investment trust focused on the acquisition and construction of mission-critical communications infrastructure including but not limited to fiber, wireless towers and ground leases.	{"Real Estate","REIT - Industrial",REITs}	{NLY,SHO,LC}	REITs	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:12.759219	2020-10-11 23:38:31.02423
TWM	ProShares UltraShort Russell2000	The investment seeks daily investment results that correspond to two times the inverse (-2x) of the daily performance of the Russell 2000® Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index is a measure of small-cap U.S. stock market performance. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.582487	2020-10-11 23:38:31.02582
TXT	Textron Inc.	Textron Inc is engaged in the aerospace industry. Its primary products include aircraft and related defense equipment. Its segments include Textron Aviation, Bell, Textron Systems and Industrial, and Finance.	{Industrials,"Aerospace & Defense"}	{AVAV,LMT,AIR,BA,GD,SPR,ERJ}	Aerospace & Defense	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:12.596339	2020-10-11 23:38:31.02582
UAL	United Continental Holdings Inc.	United Continental Holdings Inc is a holding company. It transports people and cargo through its mainline and regional operations.	{Industrials,Airlines}	{SKYW,JBLU,HA,ALGT,ALK,LUV,DAL,AAL,SAVE}	Airlines	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:12.622572	2020-10-11 23:38:31.028166
UBFO	United Security Bancshares	United Security Bancshares operates as the holding for United Security Bank. It provides commercial banking services to the business and professional community, and individuals located in Fresno, Madera, and Kern Counties.	{"Financial Services","Banks - Regional - US",Banks}	{AMRB,CVBF,HTBK,WABC,PMBC,CVCY,BSRR,PACW,HAFC}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:12.633977	2020-10-11 23:38:31.023007
UBS	UBS Group AG Registered	UBS Group AG operates as a financial services provider and a bank in Switzerland. It provides wealth management, investment banking, asset management services and personal & corporate banking.	{"Financial Services","Banks - Global",Banks}	{CBSH,UVSP,BAP,FFWM}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:12.649504	2020-10-11 23:38:31.021426
UDBI	Legg Mason US Diversified Core ETF	The investment seeks to track the investment results of the QS DBI US Diversified Index (the "underlying index").\n The fund will invest at least 80% of its net assets, plus borrowings for investment purposes, if any, in securities that compose the underlying index. The underlying index is composed of publicly traded U.S. equity securities.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:12.665455	2020-10-11 23:38:31.023007
UFAB	Unique Fabricating Inc.	Unique Fabricating Inc is a United States-based company, which is engaged in the engineering and manufacturing of components for customers in the automotive and industrial appliance market.	{"Consumer Cyclical","Auto Parts",Autos}	{ROCK,UFAB}	Autos	Consumer Cyclical	NYSE American	\N	2020-10-11 23:12:12.683102	2020-10-11 23:38:31.028166
UGA	United States Gasoline Fund LP	The investment seeks the daily changes in percentage terms of the spot price of gasoline, for delivery to the New York harbor, as measured by the daily changes in the price of a specified short-term futures contract on gasoline called the “Benchmark Futures Contract,” less UGA’s expenses.\n The fund invests in futures contracts for gasoline, other types of gasoline, crude oil, diesel-heating oil, natural gas and other petroleum-based fuels. The Benchmark Futures Contract is the futures contract on gasoline as traded on the New York Mercantile Exchange that is the near month contract to expire, except when the near month contract is within two weeks of expiration.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.700422	2020-10-11 23:38:31.031328
UHS	Universal Health Services Inc.	Universal Health Services Inc owns and operates acute care hospitals, behavior health centers, surgical hospitals, ambulatory surgery centers, and radiation oncology centers.	{Healthcare,"Medical Care","Health Care Providers"}	{LPNT,CYH,SEM,MD,SSY,HCA,ACHC,THC,XLV}	Health Care Providers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:12.710205	2020-10-11 23:38:31.028166
ULBR	Citigroup Global Markets Holdings Inc VelocityShares Long LIBOR ETN	The investment seeks to be linked to the Janus Velocity Long LIBOR Index (the “Long LIBOR Index”). The Long LIBOR Index aims to approximate the daily performance of a hypothetical long investment in the composite forward LIBOR rate, as if the composite forward LIBOR rate itself were an asset that could be invested in. It does so by tracking the return on a hypothetical short position in Eurodollar futures contracts, where that position is recalibrated daily to result in a return over the next Index Business Day that approximates the percentage change in the composite forward LIBOR rate over that next day, subject to the long LIBOR floor.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.725284	2020-10-11 23:38:31.028166
UMBF	UMB Financial Corporation	UMB Financial Corp is in the business of providing financial solutions in the United States. Its service portfolio comprises of banking and asset management solutions.	{"Financial Services","Banks - Regional - US",Banks}	{WTBA,BOKF,CBSH,GSBC,LARK,EFSC,HWBK,WFC,USB,NBHC}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:12.737847	2020-10-11 23:38:31.028166
UNG	United States Natural Gas Fund LP	The investment seeks to reflect the daily changes in percentage terms of the price of natural gas delivered at the Henry Hub, Louisiana, as measured by the daily changes in the price of a specified short-term futures contract.\nThe fund invests primarily in futures contracts for natural gas that are traded on the NYMEX, ICE Futures Europe and ICE Futures U.S. (together, “ICE Futures”) or other U.S. and foreign exchanges. The Benchmark Futures Contract is the futures contract on natural gas as traded on the New York Mercantile Exchange that is the near month contract to expire, except when the near month contract is within two weeks of expiration.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.754934	2020-10-11 23:38:31.025871
TY	Tri Continental Corporation	Tri-Continental Corp is a United States based closed-end management investment company. Its investment objective is to produce future growth of both capital and income while providing reasonable current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:12.59707	2020-10-11 23:38:31.023428
UAMY	United States Antimony Corporation	United States Antimony Corp is a fully integrated mining, transportation, milling and smelting company. It is in the business of production and sale of antimony, silver, gold and zeolite products.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{PKOH,MLI,AMRK}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:12.626497	2020-10-11 23:38:31.02423
UBOH	United Bancshares Inc.	United Bancshares Inc through its subsidiary provides commercial and retail banking services including checking accounts, savings & money market accounts, certificates of deposit, wealth and treasury management services.	{"Financial Services","Banks - Regional - US",Banks}	{PEBO,UBCP,FFBC,HBAN,WSBC,PRK,LCNB,SBFG,CIVB}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:12.641173	2020-10-11 23:38:31.02423
UCHF	ETNs linked to the VelocityShares Daily 4X Long CHF vs. USD Index	The investment seeks to track the daily performance of the VelocityShares Daily 4X Long CHF vs. USD Index.\nThe index is designed to provide 4 times leveraged exposure, reset daily, to changes in the spot exchange rate between an underlying pair of currencies consisting of the U.S. dollar and a foreign currency. It provides long exposure to one currency (the “long currency”) in the underlying currency pair relative to the other currency (the “reference currency”). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.662305	2020-10-11 23:38:31.028166
UDR	UDR Inc.	UDR Inc is a self-administered real estate investment trust. It owns, acquires, renovates, develops and manages apartment communities.	{"Real Estate","REIT - Residential",REITs}	{MMLP,EQR,CPT,MAA,ESS,FGP,AIV,AVB,IRT,AMH}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:12.67488	2020-10-11 23:38:31.028166
UFI	Unifi Inc.	Unifi Inc is a multi-national manufacturing company that processes and sells high-volume commodity yarns. It is engaged in selling yarns made from polyester and nylon.	{"Consumer Cyclical","Textile Manufacturing","Homebuilding & Construction"}	{DXYN,SCS,CULP,MHK,TILE}	Homebuilding & Construction	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:12.688348	2020-10-11 23:38:31.024353
UHAL	Amerco	Amerco Inc provides trucks and trailers on rent, sells moving supplies, towing accessories, propane and rents fixed and portable moving and storage units to the 'do-it-yoursef'' mover and management of self-storage properties owned by others.	{Industrials,"Rental & Leasing Services","Consulting & Outsourcing"}	{LSTR,R,F,PAG,CAR,FRPH,HTZ}	Consulting & Outsourcing	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:12.708403	2020-10-11 23:38:31.02423
ULBI	Ultralife Corporation	Ultralife Corp designs, manufactures, installs & maintains power and communications systems including; rechargeable and non-rechargeable batteries, charging systems, communications and electronics systems and accessories, and custom engineered systems.	{Industrials,"Diversified Industrials","Industrial Products"}	{EPC,CBAK}	Industrial Products	Industrials	NASDAQ Global Market	\N	2020-10-11 23:12:12.724507	2020-10-11 23:38:31.031328
UNB	Union Bankshares Inc.	Union Bankshares Inc is a bank holding company for Union Bank, which offers retail and commercial banking services to its customers through its branches, ATMs, telebanking, and internet banking systems.	{"Financial Services","Banks - Regional - US",Banks}	{EBTC,BPFH,CNBKA,INDB,NBTB,WASH,PMBC,BHB,NBN}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:12.751938	2020-10-11 23:38:31.028166
UONE	Urban One Inc.	Urban One Inc is an urban oriented, multi-media company. Its business is radio broadcasting franchise that is the radio broadcasting operation that targets African-American and urban listeners. It owns and operates approximately 50 broadcast stations.	{"Consumer Cyclical","Broadcasting - Radio",Entertainment}	{SALM,BBGI,ETM,CBS,EMMS,SGA}	Entertainment	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:12.772107	2020-10-11 23:38:31.02423
URA	Global X Uranium	The investment seeks to provide investment results that correspond generally to the price and yield performance, before fees and expenses, of the Solactive Global Uranium Total Return Index.\n The fund invests at least 80% of its total assets in the securities of the underlying index and in American Depositary Receipts ("ADRs") and Global Depositary Receipts ("GDRs") based on the securities in the underlying index. The underlying index is designed to measure broad-based equity market performance of global companies involved in the uranium industry. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.789891	2020-10-11 23:38:31.023428
USA	Liberty All-Star Equity Fund	Liberty All Star Equity Fund is a closed end fund. Its objective is to provide a high level of current income and long-term capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:12.802675	2020-10-11 23:38:31.025871
USCR	U S Concrete Inc.	US Concrete Inc is engaged in producing and providing ready-mixed concrete, aggregates and concrete-related products and services to the construction industry in the markets in the United States.	{"Basic Materials","Building Materials"}	{MLM,CX,VMC,EXP,CADC}	Building Materials	Basic Materials	NASDAQ Capital Market	\N	2020-10-11 23:12:12.820044	2020-10-11 23:38:31.02582
TYPE	Monotype Imaging Holdings Inc.	Monotype Imaging Holdings Inc is a provider of type, technology and expertise for creative applications and consumer electronics, or CE, devices. The business is organized as Creative Professional and Original equipment manufacturers (OEM).	{Technology,"Software - Application","Application Software"}	{MSFT,ADBE,AAPL,ADSK,SOFO,ORCL,TGNA,HPQ,GOOGL,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:12.613352	2020-10-11 23:38:31.023428
UAN	CVR Partners LP representing Limited Partner Interests	CVR Partners LP is a manufacturer and supplier of nitrogen fertilizer products. Its principal products include Urea Ammonium Nitrate (UAN) and ammonia. The company's product sales are heavily weighted toward UAN.	{"Basic Materials","Agricultural Inputs",Agriculture}	{CF,CGA,MOS,HFC}	Agriculture	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:12.630777	2020-10-11 23:38:31.02582
UCBI	United Community Banks Inc.	United Community Banks Inc is a financial services provider. The company functions through the Union Community Bank, which provides retail and corporate banking services, including checking, savings, and time deposit accounts.	{"Financial Services","Banks - Regional - US",Banks}	{AMNB,LION,ABCB}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:12.651478	2020-10-11 23:38:31.031328
UDN	Invesco DB USD Index Bearish	The investment seeks to track the changes, whether positive or negative, in the level of the Deutsche Bank Short US Dollar Index Futures Index.\n The index reflects the changes in market value over time, whether positive or negative, of a short position in the DX Contract which expires during the months of March, June, September and December. The fund seeks to track the index by establishing short positions in DX Contracts. DX Contracts are linked to the six underlying currencies, or the index currencies, of the U.S. Dollar index (USDX®), or the USDX®. The index currencies are Euro, Japanese Yen, British Pound, Canadian Dollar, Swedish Krona and Swiss Franc.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.666196	2020-10-11 23:38:31.025871
UFPI	Universal Forest Products Inc.	Universal Forest Products Inc and its subsidiaries supply wood and wood-alternative products to three primary markets namely retail,industrial and construction.	{"Basic Materials","Lumber & Wood Production","Forest Products"}	{LPX,BCC}	Forest Products	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:12:12.689421	2020-10-11 23:38:31.021426
UGAZ	VS 3X NAT GAS	The investment seeks to replicate, net of expenses, three times the performance of the S&P GSCI Natural Gas Index ER.\n The index comprises futures contracts on a single commodity and is calculated according to the methodology of the S&P GSCI Index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.700367	2020-10-11 23:38:31.023007
UIHC	United Insurance Holdings Corp.	United Insurance Holdings Corp is a holding company primarily engaged in the residential and commercial property and casualty insurance business in the United States.	{"Financial Services","Insurance - Property & Casualty"}	{NSEC,STFC,KINS,ALL,UVE,HCI,HRTG}	Insurance - Property & Casualty	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:12.716666	2020-10-11 23:38:31.021426
ULST	SPDR SSgA Ultra Short Term Bond	The investment seeks to provide current income consistent with preservation of capital and daily liquidity through short duration high quality investments.\n The Adviser invests, under normal circumstances, at least 80% of the fund's net assets (plus the amount of borrowings for investment purposes) in a diversified portfolio of U.S. dollar-denominated investment-grade fixed income securities. The fund may also invest in exchange traded products ("ETPs"). It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.733602	2020-10-11 23:38:31.02423
UMPQ	Umpqua Holdings Corporation	Umpqua Holdings Corp is the holding company of Umpqua Bank. It provides community banking services involving banking, wealth management, mortgage, and other financial services in the United States.	{"Financial Services","Banks - Regional - US",Banks}	{COLB,HFWA,CVBF,ZION,PBCT,BMRC,WAL,FRC,BANR}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:12.748519	2020-10-11 23:38:31.02423
UNP	Union Pacific Corporation	Union Pacific Corp belongs to the rail transport industry. It provides railroad freight transportation services in North America. It transports coal, rocks, soda ash, and grains.	{Industrials,Railroads,"Transportation & Logistics"}	{CNI,CP,GWR,NSC,KSU,CSX}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:12.763519	2020-10-11 23:38:31.024353
UPS	United Parcel Service Inc.	United Parcel Service Inc is a package delivery organization and provider of supply chain management solutions. Its services include transportation, distribution, contract logistics, ground freight, ocean freight, air freight, insurance and financing.	{Industrials,"Integrated Shipping & Logistics","Transportation & Logistics"}	{AIRT,EXPD,CHRW,ATSG,FDX}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:12.779754	2020-10-11 23:38:31.025871
URGN	UroGen Pharma Ltd.	UroGen Pharma Ltd is a clinical stage biopharmaceutical company focused on developing novel therapies designed to change the standard of care for urological pathologies.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:12.795241	2020-10-11 23:38:31.024353
USAP	Universal Stainless & Alloy Products Inc.	Universal Stainless & Alloy Products Inc and its subsidiaries, manufactures and markets semi-finished and finished specialty steel products. It offers stainless steel, nickel alloys, tool steel and certain other premium alloyed steels.	{"Basic Materials",Steel}	{HAYN,AKS,SIM,CRS,CMC,NUE,WOR,ATI,FRD}	Steel	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:12:12.808763	2020-10-11 23:38:31.028166
UBR	ProShares Ultra MSCI Brazil Capped	The investment seeks daily investment results that correspond to two times (2x) the daily performance of the MSCI Brazil 25/50 Index®.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index is a free-float-adjusted, market capitalization-weighted index that is designed to measure the equity market performance of the Brazilian market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.648334	2020-10-11 23:38:31.02582
UCFC	United Community Financial Corp.	United Community Financial Corporation is a US-based financial services company which provides consumer banking, business banking, and investment services in Ohio and western Pennsylvania.	{"Financial Services","Savings & Cooperative Banks",Banks}	{FDEF,TFSL,NYCB,WAYN}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:12.66271	2020-10-11 23:38:31.02423
UEC	Uranium Energy Corp.	Uranium Energy Corp is engaged in uranium mining and related activities including exploration, pre-extraction, extraction and processing on uranium projects located in the United States and Parguay.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{URG,DNN,GMO,LEU,WWR}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:12.677336	2020-10-11 23:38:31.02582
UGBP	ETNs linked to the VelocityShares Daily 4X Long GBP vs. USD Index	The investment seeks to track the daily performance of the VelocityShares Daily 4X Long GBP vs. USD Index.\nThe index is designed to provide 4 times leveraged exposure, reset daily, to changes in the spot exchange rate between an underlying pair of currencies consisting of the U.S. dollar and a foreign currency. It provides long exposure to one currency (the “long currency”) in the underlying currency pair relative to the other currency (the “reference currency”). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.702959	2020-10-11 23:38:31.023428
UIS	Unisys Corporation	Unisys Corp provides IT (Information Technology) outsourcing solutions and technology products to clients primarily in the government, financial services, and transportation sectors. The company operates in two segments namely Services and Technology.	{Technology,"Information Technology Services","Application Software"}	{CSCO,MSFT,RHT,IBM,WYY,ORCL,LDOS,DXC,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:12.716788	2020-10-11 23:38:31.023428
ULTA	Ulta Beauty Inc.	Ulta Beauty Inc operates as a beauty retailer in the United States. It offers cosmetics, fragrance, skin, hair care products and salon services.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{RGS,CVS,SBH,LB,WBA}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:12.733543	2020-10-11 23:38:31.023007
UNAM	Unico American Corporation	Unico American Corp is an insurance holding company that underwrites property and casualty insurance; provides property, casualty, and health insurance; and provides insurance premium financing and membership association services.	{"Financial Services","Insurance - Property & Casualty"}	{CINF,AFSI,AFG,CNA,ORI,RLI}	Insurance - Property & Casualty	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:12.751072	2020-10-11 23:38:31.031328
UNTY	Unity Bancorp Inc.	Unity Bancorp Inc conducts a traditional and community-oriented commercial banking business and offers services, including personal and business checking accounts, time deposits, money market accounts and regular savings accounts.	{"Financial Services","Banks - Regional - US",Banks}	{SSFN,PGC,LBAI,PKBK,BKJ,TRCB}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:12.768296	2020-10-11 23:38:31.02582
URI	United Rentals Inc.	United Rentals Inc operates in the rental and leasing industry. It provides equipment on rent to construction and industrial companies, manufacturers, utilities, municipalities, homeowners, government entities.	{Industrials,"Rental & Leasing Services","Consulting & Outsourcing"}	{FAST,HEES,TEX,OSK,HTZ}	Consulting & Outsourcing	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:12.793577	2020-10-11 23:38:31.021426
USAT	USA Technologies Inc.	USA Technologies Inc operates in the small ticket electronic payments industry. It provides wireless, cashless, micro-transactions and networking services within the unattended Point of Sale (POS) market.	{Technology,"Software - Infrastructure","Application Software"}	{TACT,ALOT,ZBRA,NCR,CR,DBD,PAR,XLK}	Application Software	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:12.808763	2020-10-11 23:38:31.025871
USEQ	Invesco Russell 1000 Enhanced Equal Weight	The investment seeks to track the investment results (before fees and expenses) of the Russell 1000® Enhanced Value Equal Weight Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in securities that comprise the underlying index. The underlying index is a subset of the Russell 1000, which measures the performance of the large-cap segment of the U.S. equity universe. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:12.824571	2020-10-11 23:38:31.028166
USMF	WisdomTree U.S. Multifactor Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree U.S. Multifactor Index.\n Under normal circumstances, at least 80% of the fund's total assets will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is generally comprised of 200 U.S. companies with the highest composite scores based on two fundamental factors (value and quality measures) and two technical factors (momentum and correlation). The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:12.841146	2020-10-11 23:38:31.023428
USPH	U.S. Physical Therapy Inc.	US Physical Therapy Inc operates outpatient physical therapy and occupational therapy clinics across US which provide treatment for orthopedic disorders, sports-related injuries, and work-related injuries.	{Healthcare,"Medical Care","Health Care Providers"}	{HNGR,SEM,ACHC,TVTY,XLV}	Health Care Providers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:12.855919	2020-10-11 23:38:31.02582
UBT	ProShares Ultra 20+ Year Treasury	The investment seeks daily investment results that correspond to two times (2x) the daily performance of the ICE U.S. Treasury 20+ Year Bond Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index includes publicly- issued U.S. Treasury securities that have a remaining maturity greater than twenty years and have $300 million or more of outstanding face value, excluding amounts held by the Federal Reserve. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.651794	2020-10-11 23:38:31.023007
UCTT	Ultra Clean Holdings Inc.	Ultra Clean Holdings Inc, through its subsidiaries, manufactures and supplies production tools, modules and subsystems for the semiconductor capital equipment industry. Its principal markets include North America, Asia and Europe.	{Technology,"Semiconductor Equipment & Materials",Semiconductors}	{AMAT,AEIS,CVV,MKSI,COHU,KLAC,LRCX,ENTG,BRKS,ACLS,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:12.663983	2020-10-11 23:38:31.023428
UGL	ProShares Ultra Gold	The investment seeks to provide daily investment results (before fees and expenses) that correspond to twice (200%) the daily performance of gold bullion as measured by the U.S. Dollar p.m. fixing price for delivery in London.\nThe “Ultra” funds seek results for a single day that match (before fees and expenses) two times (2x) the daily performance of a benchmark. It does not seek to achieve their stated objective over a period greater than a single day.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.70511	2020-10-11 23:38:31.025871
UIVM	USAA MSCI International Value Momentum Blend Index	The investment seeks to provide investment results that correspond to the MSCI World ex USA Select Value Momentum Blend Index.\n The fund seeks to achieve its investment objective by investing at least 80% of its net assets in securities in the index, depositary receipts on securities in the index, and securities underlying depositary receipts in the index. The index is designed to deliver exposure to equity market performance in non-U.S. markets and provide higher exposure to value and momentum factors within the MSCI World ex USA Index while also maintaining moderate index turnover and lower realized volatility than traditional capitalization weighted indexes.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.721575	2020-10-11 23:38:31.02582
UMC	United Microelectronics Corporation (NEW)	United Microelectronics Corp manufactures integrated circuits wafers and related electronic products. It is also engaged in developing, manufacturing and providing solar energy and new generation light-emitting diode.	{Technology,Semiconductors}	{INTC,TSEM,TSM,IBM,SMI,XLK}	Semiconductors	Technology	New York Stock Exchange	\N	2020-10-11 23:12:12.738785	2020-10-11 23:38:31.024353
UNFI	United Natural Foods Inc.	United Natural Foods Inc acts as a wholesale distributor of natural, organic and specialty foods and nonfood products across North America. Its products include produce, perishables and frozen foods, nutritional supplements and sports nutrition.	{"Consumer Defensive","Food Distribution","Retail - Defensive"}	{WILC,HAIN,CORE,KR,SYY,DIT,CHEF,SPTN}	Retail - Defensive	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:12.754702	2020-10-11 23:38:31.028166
UNT	Unit Corporation	Unit Corp is engaged in the exploration, development, acquisition, and production of oil and natural gas properties, the land contract drilling of natural gas and oil wells, and the buying, selling, gathering, processing, and treating of natural gas.	{Energy,"Oil & Gas Drilling","Oil & Gas - Drilling"}	{PTEN,OKE,HP,NBR,QEP,PES}	Oil & Gas - Drilling	Energy	New York Stock Exchange	\N	2020-10-11 23:12:12.767725	2020-10-11 23:38:31.031328
UPRO	ProShares UltraPro S&P 500	The investment seeks daily investment results, before fees and expenses, that correspond to three times (3x) the daily performance of the S&P 500® Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index is a measure of large-cap U.S. stock market performance. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.778446	2020-10-11 23:38:31.028166
URR	Market Vectors Double Long Euro ETN	The investment seeks to replicate, net of expenses, the Double Long Euro Index. \n As the index is two-times leveraged, for every 1% strengthening of the euro relative to the U.S. dollar, the level of the index will generally increase by 2%, while for every 1% weakening of the euro relative to the U.S. dollar, the index will generally decrease by 2%.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.797297	2020-10-11 23:38:31.023007
USATP	USA Technologies Inc. Preferred Stock	USA Technologies Inc operates in the small ticket electronic payments industry. It provides wireless, cashless, micro-transactions and networking services within the unattended Point of Sale (POS) market.	{Technology,"Software - Infrastructure","Application Software"}	{XLK}	Application Software	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:12.812388	2020-10-11 23:38:31.024353
USFR	WisdomTree Floating Rate Treasury Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the Bloomberg U.S. Treasury Floating Rate Bond Index (the "index").\n The fund invests at least 80% of its total assets (exclusive of collateral held from securities lending) in the component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is designed to measure the performance of floating rate public obligations of the U.S. Treasury. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.826481	2020-10-11 23:38:31.021426
UGLD	VS 3X GOLD	The investment seeks to replicate, net of expenses, three times the S&P GSCI Gold index ER.\n The index comprises futures contracts on a single commodity. The fluctuations in the values of it are intended generally to correlate with changes in the price of gold in global markets.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:12.705916	2020-10-11 23:38:31.023007
UJPY	ETNs linked to the VelocityShares Daily 4X Long JPY vs. USD Index	The investment seeks to track the daily performance of the VelocityShares Daily 4X Long JPY vs. USD Index.\nThe index is designed to provide 4 times leveraged exposure, reset daily, to changes in the spot exchange rate between an underlying pair of currencies consisting of the U.S. dollar and a foreign currency. It provides long exposure to one currency (the “long currency”) in the underlying currency pair relative to the other currency (the “reference currency”). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.722937	2020-10-11 23:38:31.02423
ULTI	The Ultimate Software Group Inc.	The Ultimate Software Group Inc is a provider of human capital management software solutions. Its UltiPro product suite includes human resources, payroll, and benefits management functionality.	{Technology,"Software - Infrastructure","Application Software"}	{MSFT,ADP,IBM,CSOD,ORCL,PCTY,PAYC,WDAY,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:12.734324	2020-10-11 23:38:31.031328
UMH	UMH Properties Inc.	UMH Properties Inc is in the business of ownership and operation of manufactured home communities - leasing manufactured home spaces on a month-to-month basis to private manufactured home owners.	{"Real Estate","REIT - Residential",REITs}	{GOOD,SUI,CLI,UBA,MNR,ELS,GOV,SIR}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:12.746838	2020-10-11 23:38:31.021426
UNL	United States 12 Month Natural Gas Fund	The investment seeks to reflect the daily changes in percentage terms of the price of natural gas delivered at the Henry Hub, Louisiana, as measured by the daily changes in the price of a specified short-term futures contracts on natural gas.\n The Benchmark Futures Contracts are the futures contracts on natural gas as traded on the New York Mercantile Exchange (the "NYMEX”) that are the near month contract to expire and the contracts for the following 11 months, for a total of 12 consecutive months’ contracts, except when the near month contract is within two weeks of expiration.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.758901	2020-10-11 23:38:31.023007
UPL	Ultra Petroleum Corp.	Ultra Petroleum Corp is an independent oil and natural gas company which is engaged in the development, production, operation, exploration and acquisition of oil and natural gas properties. Its operations are in Wyoming, and Utah.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{}	Oil & Gas - E&P	Energy	Nasdaq Global Select	\N	2020-10-11 23:12:12.77543	2020-10-11 23:38:31.021426
URBN	Urban Outfitters Inc.	Urban Outfitters Inc is a lifestyle specialty retail company that operates under the Urban Outfitters, Anthropologie, Free People, Terrain and Bhldn brands. It also operates a Wholesale segment under the Free People brand.	{"Consumer Cyclical","Apparel Stores","Retail - Apparel & Specialty"}	{ZUMZ,GES,ANF,AEO,BKE,GPS,JWN,TJX,EXPR,TLYS}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:12.790419	2020-10-11 23:38:31.025871
USAK	USA Truck Inc.	USA Truck Inc is a transportation and logistics service provider. The company transports commodities throughout the continental United States and into and out of the portions of Canada. It also transports general commodities into and out of Mexico.	{Industrials,Trucking,"Transportation & Logistics"}	{PTSI,CVTI,HTLD,MRTN,WERN,ULH,RRTS,KNX}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:12.805092	2020-10-11 23:38:31.02423
USEG	U.S. Energy Corp.	US Energy Corp is a diversified natural resource company. Its objective is to identify and strategically invest in oil and gas exploration and development opportunities with a primary focus on oil.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{MCF,NOG,PARR,REN,BCEI,ESTE,WLL}	Oil & Gas - E&P	Energy	NASDAQ Capital Market	\N	2020-10-11 23:12:12.82449	2020-10-11 23:38:31.025871
USM	United States Cellular Corporation	United States Cellular Corp is a wireless telecommunications service provider. Its ownership interests include consolidated and investment interests in licenses covering portions of nearly 20 states & total population of around 30 million.	{"Communication Services","Telecom Services"}	{SHEN,VZ,T,TDS,SPOK,TMUS}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:12.839669	2020-10-11 23:38:31.023007
UTES	ETFIS Series Trust I Reaves Utilities	The investment seeks to provide total return through a combination of capital appreciation and income.\n The fund invests not less than 80% of its net assets (plus the amount of any borrowings for investment purposes) in equity securities of companies in the Utility Sector ("Utility Sector Companies"). The manager considers a company to be a "Utility Sector Company" if at least 50% of the company's assets or customers are committed to, or at least 50% of the company's revenues, gross income or profits derive from, the provision of products, services or equipment for the generation or distribution of electricity, gas or water. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.867099	2020-10-11 23:38:31.023007
UG	United-Guardian Inc.	United-Guardian Inc is a manufacturer and distributor of personal care products such as cosmetics mainly in the United States. Its portfolio also includes pharmaceuticals, medical lubricants, healthcare solutions.	{"Consumer Defensive","Household & Personal Products","Consumer Packaged Goods"}	{IFF,JNJ,PFE,SCL,TFX,ASH}	Consumer Packaged Goods	Consumer Defensive	NASDAQ Global Market	\N	2020-10-11 23:12:12.697498	2020-10-11 23:38:31.028166
UHT	Universal Health Realty Income Trust	Universal Health Realty Income Trust is a health-care REIT. It holds interest in various property portfolios including hospitals, medical office buildings, and child-care centers. It operates its business in the United States.	{"Real Estate","REIT - Healthcare Facilities",REITs}	{DRE,VTR,HCP,SBRA,TIPT,HTA,WELL,SNH}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:12.716606	2020-10-11 23:38:31.031328
UN	Unilever NV	Unilever NV is engaged in the household products industry. The company is a supplier of fast moving consumer goods. Its areas of operations are personal care, home care, foods and refreshment.	{"Consumer Defensive","Household & Personal Products","Consumer Packaged Goods"}	{EL,CHD,CL,PG}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:12.750193	2020-10-11 23:38:31.02582
UNM	Unum Group	Unum Group is a financial services provider. Its services include insurance and non-insurance products and services.	{"Financial Services","Insurance - Life"}	{}	Insurance - Life	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:12.760858	2020-10-11 23:38:31.023428
UPLD	Upland Software Inc.	Upland Software Inc is the provider of cloud-based enterprise work management software. It provides a family of cloud-based enterprise work management software applications for the information technology, marketing, finance and professional services.	{Technology,"Software - Application","Application Software"}	{EVOL,RP,EIGI,XLK}	Application Software	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:12.776842	2020-10-11 23:38:31.024353
URG	Ur-Energy Inc (Canada)	Ur-Energy Inc is an exploration stage mining company. It is engaged in uranium mining, recovery and processing activities, including the acquisition, exploration, development and operation of uranium mineral properties in the United States.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{DNN,GMO,UEC,UUUU,LEU,WWR}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:12.791596	2020-10-11 23:38:31.031328
USAI	American Energy Independence	The investment seeks to track the performance, before fees and expenses, of the American Energy Independence Index.\n Under normal circumstances, at least 80% of the fund's total assets (exclusive of any collateral held from securities lending) will be invested in the component securities of the index and depositary receipts representing foreign securities. The index uses a proprietary, rules-based methodology to measure the performance of a portfolio of U.S. and Canadian exchange-listed equity securities of companies that generate a majority of their cash flow from certain qualifying "midstream" energy infrastructure activities. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.806337	2020-10-11 23:38:31.023428
USDU	WisdomTree Bloomberg U.S. Dollar Bullish Fund	The investment seeks to provide total returns, before fees and expenses, that exceed the performance of the Bloomberg Dollar Total Return Index (the "index").\n The fund is an actively managed ETF that seeks to provide total returns, before fees and expenses, that exceed the performance of the index. The index is structured to potentially benefit as the U.S. dollar appreciates relative to a basket of global currencies. The fund will seek exposure to both the U.S. dollar and global currencies held by the index through investing, under normal circumstances, at least 80% of its assets in money market securities and other liquid securities. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.823601	2020-10-11 23:38:31.023428
USMC	Principal U.S. Mega-Cap Multi-Factor Index ETF	The investment seeks to provide investment results that closely correspond, before expenses, to the performance of the Nasdaq U.S. Mega Cap Select Leaders Index (the "index").\n Under normal circumstances, the fund invests at least 80% of its net assets, plus any borrowings for investment purposes, in equity securities that compose the index at the time of purchase. The index uses a quantitative model designed to identify equity securities of companies in the Nasdaq US 500 Large Cap Index (the "parent index") that have the largest market capitalizations, with higher weights given to less volatile securities.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:12.84161	2020-10-11 23:38:31.021426
USV	E-TRACS USB Bloomberg Commodity Index Exchange Traded Notes UBS Bloomberg CMCI Silver ETN	The investment seeks to track the price and performance yield, before fees and expenses, of the UBS Bloomberg CMCI Silver Total Return index. \n The fund is designed to be representative of the entire liquid forward curve of the silver contracts. The index measures the collateralized returns from a basket of silver futures contracts. It is comprised of the silver futures contracts included in the CMCI with five target maturities.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.862141	2020-10-11 23:38:31.02423
UWM	ProShares Ultra Russell2000	The investment seeks daily investment results that correspond to two times (2x) the daily performance of the Russell 2000® Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index is a float-adjusted, market capitalization-weighted index containing approximately 2000 of the smallest companies in the Russell 3000® Index or approximately 8% of the total market capitalization of the Russell 3000® Index, which in turn represents approximately 98% of the investable U.S. equity market. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.890265	2020-10-11 23:38:31.02423
UPV	ProShares Ultra FTSE Europe	The investment seeks daily investment results, before fees and expenses, that correspond to two times (2x) the daily performance of the FTSE Developed Europe All Cap Index®.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index is a free float-adjusted market cap weighted index representing the performance of large, mid and small cap companies in Developed European markets, including the UK. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.782607	2020-10-11 23:38:31.02582
USAC	USA Compression Partners LP Representing Limited Partner Interests	USA Compression Partners LP provides natural gas compression services under term contracts with customers in the oil and gas industry, using natural gas compressor packages that it designs, engineers, owns, operates and maintains.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{TRS,CAT,GE,HSC,SWN}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:12.803659	2020-10-11 23:38:31.031328
USCI	United States Commodity Index Fund ETV	The investment seeks the daily changes in percentage terms of its shares’ per share net asset value (“NAV”) to reflect the daily changes in percentage terms of the spot price of Brent crude oil. \n The Benchmark Futures Contract is the futures contract on Brent crude oil as traded on the Ice Futures Europe Exchange that is the near month contract to expire, except when the near month contract is within two weeks of expiration, in which case it will be measured by the futures contract that is the next month contract to expire.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.81924	2020-10-11 23:38:31.023007
USNA	USANA Health Sciences Inc.	Usana Health Sciences Inc manufactures health products. Its key products include science-based nutritional and personal care products.	{"Consumer Defensive","Household & Personal Products","Consumer Packaged Goods"}	{NATR,CYAN,PRPH,NUS,AVP,HLF,REV,MTEX,RELV}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:12.849158	2020-10-11 23:38:31.02423
USVM	USAA MSCI USA Small Cap Value Momentum Blend Index	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the MSCI USA Small Cap Select Value Momentum Blend Index.\n Under normal circumstances, the fund seeks to achieve its investment objective by investing at least 80% of its net assets in securities in the index. The index is designed to deliver exposure to equity securities of small-capitalization U.S. issuers that have higher exposure to value and momentum factors within the MSCI USA Small Cap Index while also maintaining moderate Index turnover and lower realized volatility than traditional capitalization weighted indexes.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.868552	2020-10-11 23:38:31.031328
UWN	Nevada Gold & Casinos Inc.	Nevada Gold & Casinos Inc is a gaming company. The Company is engaged in the financing, developing, owning and operating gaming properties & projects and its gaming facility operations are located in the USA.	{"Consumer Cyclical","Resorts & Casinos","Travel & Leisure"}	{MCRI,CNTY,CZR,FLL,ERI,GDEN,NYNY}	Travel & Leisure	Consumer Cyclical	NYSE American	\N	2020-10-11 23:12:12.890177	2020-10-11 23:38:31.024353
VALX	Validea Market Legends ETF	The investment seeks to track the performance, before fees and expenses, of the Validea Market Legends Index (the "index").\n Under normal circumstances, at least 80% of the fund's total assets (exclusive of any collateral held from securities lending) will be invested in the component securities of the index. The index uses a proprietary, rules-based methodology created by Validea Capital Management, LLC, the fund's investment adviser (the "Adviser"), to select companies based on the published stock selection strategies of historically successful investors.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:12.910938	2020-10-11 23:38:31.028166
VBR	Vanguard Small-Cap Value	The investment seeks to track the performance of a benchmark index that measures the investment return of small-capitalization value stocks.\n The fund employs an indexing investment approach designed to track the performance of the CRSP US Small Cap Value Index, a broadly diversified index of value stocks of small U.S. companies. The advisor attempts to replicate the target index by investing all, or substantially all, of its assets in the stocks that make up the index, holding each stock in approximately the same proportion as its weighting in the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.932242	2020-10-11 23:38:31.02423
VCV	Invesco California Value Municipal Income Trust	Invesco California Value Municipal Income Trust is engaged in the field of financial services. Its objective is to seek to provide a high level of current income exempt from federal and California income taxes, consistent with preservation of capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:12.946772	2020-10-11 23:38:31.02582
VEC	Vectrus Inc.	Vectrus Inc is a provider of services to the United States Government across the world. The company offers services such as Facility and Logistics service and Information Technology and Network Communications services.	{Industrials,"Business Services"}	{CRAI,HCKT,SRT,ATTO}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:12.962452	2020-10-11 23:38:31.02423
VET	Vermilion Energy Inc. Common (Canada)	Vermilion Energy Inc is oil- and gas-producing company which is engaged in full-cycle exploration and production programs which focus on the acquisition, exploration, development, and optimization of producing properties.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{CNQ,IMO,SU,XOM,ERF}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:12.980146	2020-10-11 23:38:31.031328
ECPG	Encore Capital Group Inc	Encore Capital Group Inc is a specialty finance company. The Company provides debt recovery solutions for consumers and property owners across a broad range of financial assets.	{"Financial Services","Asset Management"}	{ASFI,NICK,CACC,LOAN,AXP,ATLC,PRAA}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:02.080397	2020-10-11 23:38:31.024353
USAS	Americas Silver Corporation no par value	Americas Silver Corp is engaged in the evaluation, acquisition, exploration, development and operation of precious metals mineral properties in North America.	{"Basic Materials",Silver,"Metals & Mining"}	{PAAS}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:12.808315	2020-10-11 23:38:31.028166
USD	ProShares Ultra Semiconductors	The investment seeks daily investment results that correspond to two times (2x) the daily performance of the Dow Jones U.S. SemiconductorsSM Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index measures the performance of certain companies in the semiconductor sub-sector of the U.S. equity market. Component companies are engaged in the production of semiconductors and other integrated chips, as well as other related products such as semiconductor capital equipment and mother-boards. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.822969	2020-10-11 23:38:31.02423
USLM	United States Lime & Minerals Inc.	United States Lime & Minerals Inc engages in the business of manufacturing lime and lime products with interests in natural gas. The company operates its business in two segments; Lime and Limestone Operations, and Natural Gas.	{"Basic Materials","Building Materials"}	{MLM,XOM,VMC,EOG,EXP}	Building Materials	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:12:12.838904	2020-10-11 23:38:31.02582
USOI	Credit Suisse X-Links Crude Oil Call ETN IOPV	The fund seeks a return linked to the performance of the price return version of the Credit Suisse Nasdaq WTI Crude Oil FLOWSTM 106 Index (the “index”). The index measures the return of a “covered call” strategy on the shares of the United States Oil Fund, LP (the “USO Fund”, and such shares the “USO Shares”) by reflecting changes in the price of the USO Shares and the notional option premiums received from the notional sale of monthly call options on the USO Shares less the Notional Transaction Costs incurred in connection with the implementation of the covered call strategy.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:12.853372	2020-10-11 23:38:31.031328
UTG	Reaves Utility Income Fund of Beneficial Interest	Reaves Utility Income Fund is a closed-end management investment company. The company's investment objective is to provide a high level of after-tax income and total return consisting primarily of tax-advantaged dividend income and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:12.868923	2020-10-11 23:38:31.028166
UUU	Universal Security Instruments Inc.	Universal Security Instruments Inc is a US-based manufacturer and distributor of safety and security devices. The company markets a line of residential smoke and carbon monoxide alarms under the trade names UNIVERSAL and USI Electric.	{Industrials,"Security & Protection Services","Consulting & Outsourcing"}	{MAGS,NSSC,LRAD,DBD,UTX}	Consulting & Outsourcing	Industrials	NYSE American	\N	2020-10-11 23:12:12.884943	2020-10-11 23:38:31.023428
UYG	ProShares Ultra Financials	The investment seeks daily investment results, before fees and expenses, that correspond to two times (2x) the daily performance of the Dow Jones U.S. FinancialsSM Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index seeks to measure the performance of certain companies in the financial services sector of the U.S. equity market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.898439	2020-10-11 23:38:31.02582
VALE	VALE S.A. American Depositary Shares Each Representing one	Vale SA is a metals and mining company. It produces and supplies iron ore, iron ore pellets, nickel, manganese ore, ferroalloys, copper, coal, phosphates, potash, cobalt and others.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{TECK,BBL,SID,RIO,BHP,MT}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:12.911103	2020-10-11 23:38:31.024353
VCLT	Vanguard Long-Term Corporate Bond ETF	The investment seeks to track the performance of a market-weighted corporate bond index with a long-term dollar-weighted average maturity.\n The fund employs an indexing investment approach designed to track the performance of the Bloomberg Barclays U.S. 10+ Year Corporate Bond Index. This index includes U.S. dollar-denominated, investment-grade, fixed-rate, taxable securities issued by industrial, utility, and financial companies, with maturities greater than 10 years. Under normal circumstances, at least 80% of the fund's assets will be invested in bonds included in the index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:12.942888	2020-10-11 23:38:31.031328
VEACU	Vantage Energy Acquisition Corp. Unit	Vantage Energy Acquisition Corp is a blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:12.957478	2020-10-11 23:38:31.028166
VESH	Virtus Enhanced Short U.S. Equity	The investment seeks capital appreciation.\n The fund is an actively managed exchange-traded fund that seeks to provide inverse (opposite) exposure to investment returns of the broad U.S. large capitalization equity market by selling S&P 500 futures contracts and S&P Select Sector futures contracts and/or engaging in short sales of U.S. equity ETFs and other U.S. equity securities. Under normal market conditions, it will invest not less than 80% of its net assets (plus the amount of any borrowings for investment purposes) in investments that create or result in short (or inverse) exposure to U.S. equity securities. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.978957	2020-10-11 23:38:31.02582
USB	U.S. Bancorp	US Bancorp is a multi-state financial holding company. It offers financial services such as lending and depository service, cash management, foreign exchange and trust and investment management services.	{"Financial Services","Banks - Regional - US",Banks}	{GSBC,BAC,WFC,C,NBHC}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:12.817228	2020-10-11 23:38:31.021426
USG	USG Corporation	USG Corp produces wallboard and ceiling tile in the United States. The company manufactures and sells gypsum wallboard (drywall), wallboard finishing materials, tile backers and underlayment, and ceiling tile and grid used for construction.	{"Basic Materials","Building Materials"}	{SHW,EXP,AWI,CBPX}	Building Materials	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:12.835327	2020-10-11 23:38:31.024353
UTF	Cohen & Steers Infrastructure Fund Inc	Cohen & Steers Select Utility Fund Inc is a diversified closed-end management investment company. The primary investment objective of the Fund is total return with an emphasis on income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:12.867928	2020-10-11 23:38:31.028166
UUUU	Energy Fuels Inc (Canada)	Energy Fuels Inc together with its subsidiary is engaged in the extraction and recovery of uranium properties in the United States. The company operates in two segments, ISR Uranium and Conventional Uranium.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{URG,DNN,WWR}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:12.885639	2020-10-11 23:38:31.028166
UZA	United States Cellular Corporation 6.95% Senior Notes due 2060	United States Cellular Corp is a wireless telecommunications service provider. Its ownership interests include consolidated and investment interests in licenses covering portions of nearly 20 states & total population of around 30 million.	{"Communication Services","Telecom Services"}	{}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:12.90063	2020-10-11 23:38:31.021426
VB	Vanguard Small-Cap	The investment seeks to track the performance of a benchmark index that measures the investment return of small-capitalization stocks.\n The fund employs an indexing investment approach designed to track the performance of the CRSP US Small Cap Index, a broadly diversified index of stocks of small U.S. companies. The advisor attempts to replicate the target index by investing all, or substantially all, of its assets in the stocks that make up the index, holding each stock in approximately the same proportion as its weighting in the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.917145	2020-10-11 23:38:31.025871
VCF	Delaware Investments Colorado Municipal Income Fund Inc	Delaware Investments Colorado Municipal Income Fund, Inc is a closed-end investment company. It provides current income exempt from both regular federal income tax and Colorado state personal income tax, consistent with the preservation of capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:12.939356	2020-10-11 23:38:31.023428
VEA	Vanguard FTSE Developed Markets	The investment seeks to track the performance of the FTSE Developed All Cap ex US Index.\n The fund employs an indexing investment approach designed to track the performance of the FTSE Developed All Cap ex US Index, a market-capitalization-weighted index that is made up of approximately 3,790 common stocks of large-, mid-, and small-cap companies located in Canada and the major markets of Europe and the Pacific region. The adviser attempts to replicate the target index by investing all, or substantially all, of its assets in the stocks that make up the index, holding each stock in approximately the same proportion as its weighting in the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.954426	2020-10-11 23:38:31.024353
VEON	VEON Ltd.	VEON Ltd is a provider of communications services. It provides voice, data and other telecommunication services through an array of wireless, fixed and broadband internet services, as well as selling equipment and accessories.	{"Communication Services","Telecom Services"}	{BCE,MBT,TU,ORAN,VOD}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:12:12.97387	2020-10-11 23:38:31.028166
VFH	Vanguard Financials	The investment seeks to track the performance of a benchmark index.\n The fund employs an indexing investment approach designed to track the performance of the MSCI US Investable Market Index (IMI)/Financials 25/50, an index made up of stocks of large, mid-size, and small U.S. companies within the financials sector, as classified under the Global Industry Classification Standard (GICS). The Advisor attempts to replicate the target index by seeking to invest all, or substantially all, of its assets in the stocks that make up the index, in order to hold each stock in approximately the same proportion as its weighting in the index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.986706	2020-10-11 23:38:31.021426
VGSH	Vanguard Short-Term Government Bond ETF	The investment seeks to track the performance of a market-weighted government bond index with a short-term dollar-weighted average maturity.\n The fund employs an indexing investment approach designed to track the performance of the Bloomberg Barclays US Treasury 1-3 Year Bond Index. This index includes fixed income securities issued by the U.S. Treasury (not including inflation-protected securities), all with maturities between 1 and 3 years. At least 80% of the fund's assets will be invested in bonds included in the index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:12.999106	2020-10-11 23:38:31.02423
VIAB	Viacom Inc.	Viacom Inc is a global media company. It creates compelling television programs, motion pictures, short-form content, applications and games. Its brands include MTV, VH1, CMT, Logo, BET, Nickelodeon, Comedy central, TV Land, SPIKE, Tr3s, and VIVA.	{"Consumer Cyclical","Media - Diversified",Entertainment}	{RCI,CMCSA,DISCA,DIS,CBS,AMCX,FOXA,LGF.A}	Entertainment	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:13.021294	2020-10-11 23:38:31.02423
USOU	United States 3x Oil Fund	The investment seeks the daily changes in percentage terms of its shares’ per share NAV to reflect three times the daily change in percentage terms of the price of a specified short-term futures contract on light, sweet crude oil (the “Benchmark Oil Futures Contract”) less the fund’s expenses.\nUSCF will endeavor to have the notional value of the fund’s aggregate exposure to the Benchmark Oil Futures Contract at the close of each trading day approximately equal to 300% of the fund’s NAV. The Benchmark Oil Futures Contract is the futures contract on light, sweet crude oil as traded on the NYMEX, traded under the trading symbol “CL” (for WTI Crude Oil futures).	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.854465	2020-10-11 23:38:31.023007
UTL	UNITIL Corporation	Unitil Corp is a local distributor of electricity and natural gas throughout its service territory in the states of New Hampshire, Massachusetts and Maine. Its business segments are utility gas operations, utility electric operations and non-regulated.	{Utilities,"Utilities - Diversified","Utilities - Regulated"}	{NI,ED,PEG,SCG,NWE,ES,AGR,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:12.871134	2020-10-11 23:38:31.02423
UVXY	ProShares Trust Ultra VIX Short Term Futures	The investment seeks results (before fees and expenses) that correspond to one and one-half times (1.5x) of the performance of the S&P 500 VIX Short-Term Futures Index for a single day.\n The index seeks to offer exposure to market volatility through publicly traded futures markets and is designed to measure the return from a rolling long position in the first and second month VIX futures contracts.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.889343	2020-10-11 23:38:31.02582
VALQ	American Century STOXX U.S. Quality Value	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the iSTOXX® American Century USA Quality Value Index (the underlying index).\n Under normal market conditions, the fund invests at least 80% of its assets in the component securities of the underlying index. The underlying index is designed to select securities of large- and mid-capitalization companies that are undervalued or have sustainable income.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.911055	2020-10-11 23:38:31.025871
VBLT	Vascular Biogenics Ltd.	Vascular Biogenics Ltd is a clinical-stage bio-pharmaceutical company committed to the discovery, development and commercialization of first-in-class treatments for cancer and immune-inflammatory diseases.	{Healthcare,Biotechnology}	{OHRP,MACK,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:12.929765	2020-10-11 23:38:31.02582
VCSH	Vanguard Short-Term Corporate Bond ETF	The investment seeks to track the performance of a market-weighted corporate bond index with a short-term dollar-weighted average maturity.\n The fund employs an indexing investment approach designed to track the performance of the Bloomberg Barclays U.S. 1-5 Year Corporate Bond Index. This index includes U.S. dollar-denominated, investment-grade, fixed-rate, taxable securities issued by industrial, utility, and financial companies, with maturities between 1 and 5 years. Under normal circumstances, at least 80% of the fund's assets will be invested in bonds included in the index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:12.9477	2020-10-11 23:38:31.028166
VEDL	Vedanta Limited American Depositary Shares (Each representing four equity shares)	Vedanta Ltd along with its subsidiaries is a diversified natural resource company engaged iron ore mining, non-ferrous metals (copper and aluminium production) and commercial power generation.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{CCJ}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:12.96448	2020-10-11 23:38:31.023428
VERI	Veritone Inc.	Veritone Inc is a provider of artificial intelligence. Its platform stores cognitive engine results in a searchable, time-correlated index, creating an online, searchable library of audio & video data enabling analysis & automated business solutions.	{Technology,"Software - Infrastructure","Application Software"}	{OTEX,PANW,XLK}	Application Software	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:12.976067	2020-10-11 23:38:31.024353
VG	Vonage Holdings Corp.	Vonage Holdings Corp is a provider of telecom services. The North American firm provides unified communications, which consists of integrated voice, text, video, data, and mobile applications over Voice over Internet Protocol network.	{"Communication Services","Telecom Services"}	{BCE,CNSL,CTL,VZ,T,IDT,TU,EGHT}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:12.99239	2020-10-11 23:38:31.028166
VHC	VirnetX Holding Corp	VirnetX Holding Corp is an Internet security software and technology company with patented technology for secure communications. The company's software and technology solutions, include its Secure Domain Name Registry and GABRIEL Connection Technology.	{Technology,"Software - Infrastructure","Application Software"}	{MSFT,CTXS,SYMC,ZIXI,PFPT,QLYS,XLK}	Application Software	Technology	NYSE American	\N	2020-10-11 23:12:13.00609	2020-10-11 23:38:31.024353
VICR	Vicor Corporation	Vicor Corp manufactures and markets modular power components and complete power systems. The company operates in BBU, VI Chip, Picor and Corporate segment. It derives a majority of its revenue from the BBU and Picor segment.	{Technology,"Electronic Components","Computer Hardware"}	{CSCO,IIN,AVX,CTS,KEM,CUI,QRVO,QBAK,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:13.026244	2020-10-11 23:38:31.025871
VIVE	Viveve Medical Inc.	Viveve Medical Inc is a women's sexual health company focused on the commercialization of a non-surgical, non-ablative medical device that remodels collagen and restores vaginal tissue.	{Healthcare,"Medical Devices"}	{ISRG,NXTM,EW,MDT,XLV}	Medical Devices	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:13.057545	2020-10-11 23:38:31.031328
USRT	iShares Core U.S. REIT	The investment seeks to track the investment results of the FTSE Nareit Equity REITS Index composed of U.S. real estate equities.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The index measures the performance of U.S. listed equity real estate investment trusts ("REITs"), excluding infrastructure REITs, mortgage REITs, and timber REITs.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.856744	2020-10-11 23:38:31.023428
UTHR	United Therapeutics Corporation	United Therapeutics Corp is engaged in the healthcare sector. Its core offering includes pharmaceutical products to address unmet medical needs of patients.	{Healthcare,Biotechnology}	{GILD,CAH,GSK,BAX,LLY,PFE,TEVA,PTLA,CYTK,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:12.868791	2020-10-11 23:38:31.023428
UUP	Invesco DB USD Index Bullish Fund	The investment seeks to track the changes, whether positive or negative, in the level of the Deutsche Bank Short US Dollar Index Futures Index.\n The index reflects the changes in market value over time, whether positive or negative, of a short position in the DX Contract which expires during the months of March, June, September and December. The fund seeks to track the index by establishing short positions in DX Contracts. DX Contracts are linked to the six underlying currencies, or the index currencies, of the U.S. Dollar index (USDX®), or the USDX®. The index currencies are Euro, Japanese Yen, British Pound, Canadian Dollar, Swedish Krona and Swiss Franc.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.882676	2020-10-11 23:38:31.023007
UXI	ProShares Ultra Industrials	The investment seeks daily investment results, before fees and expenses, that correspond to two times (2x) the daily performance of the Dow Jones U.S. IndustrialsSM Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index seeks to measure the performance of certain companies in the industrial sector of the U.S. equity market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.894838	2020-10-11 23:38:31.028166
VAC	Marriot Vacations Worldwide Corporation	Marriott Vacations Worldwide Corporation is an owner of holiday resorts primarily in the United States. Its portfolio of properties includes Marriott Vacation Club, Grand Residencies and The Ritz-Carlton Destination Club.	{"Consumer Cyclical","Resorts & Casinos","Travel & Leisure"}	{PEB,RLJ,RHP,XHR}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:12.907486	2020-10-11 23:38:31.031328
VBF	Invesco Bond Fund	Invesco Bond Fund is an independent investment management firm. Its investment objective is to seek interest income while conserving capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:12.922848	2020-10-11 23:38:31.023428
VCIT	Vanguard Intermediate-Term Corporate Bond ETF	The investment seeks to track the performance of a market-weighted corporate bond index with an intermediate-term dollar-weighted average maturity.\n The fund employs an indexing investment approach designed to track the performance of the Bloomberg Barclays U.S. 5-10 Year Corporate Bond Index. This index includes U.S. dollar-denominated, investment-grade, fixed-rate, taxable securities issued by industrial, utility, and financial companies, with maturities between 5 and 10 years. Under normal circumstances, at least 80% of the fund's assets will be invested in bonds included in the index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:12.939568	2020-10-11 23:38:31.024353
VEAC	Vantage Energy Acquisition Corp.	Vantage Energy Acquisition Corp is a blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:12.955378	2020-10-11 23:38:31.023007
VFC	V.F. Corporation	VF Corp owns a large portfolio of lifestyle brands including The North Face, Timberland, Vans, Lee, Wrangler, Nautica, and others. The company offers product into outdoor and action sports, jeanswear, imagewear, and sportswear.	{"Consumer Cyclical","Apparel Manufacturing","Manufacturing - Apparel & Furniture"}	{GIL,CHKE,NKE,HBI,PVH,RL,VNCE,TPR}	Manufacturing - Apparel & Furniture	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:12.983618	2020-10-11 23:38:31.023428
VGM	Invesco Trust for Investment Grade Municipals (DE)	Invesco Trust for Investment Grade Municipals is active in the financial services sector. Its investment objective involves seeking to achieve a high level of current income exempt from federal income tax, consistent with preservation of capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:13.001798	2020-10-11 23:38:31.023007
VIG	Vanguard Div Appreciation	The investment seeks to track the performance of a benchmark index that measures the investment return of common stocks of companies that have a record of increasing dividends over time.\n The fund employs an indexing investment approach designed to track the performance of the Nasdaq US Dividend Achievers Select Index, which consists of common stocks of companies that have a record of increasing dividends over time. The adviser attempts to replicate the target index by investing all, or substantially all, of its assets in the stocks that make up the index, holding each stock in approximately the same proportion as its weighting in the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.026711	2020-10-11 23:38:31.031328
XLF	SPDR Select Sector Fund - Financial	The investment seeks investment results that, before expenses, correspond generally to the price and yield performance of publicly traded equity securities of companies in the Financial Select Sector Index.\n The fund generally invests substantially all, but at least 95%, of its total assets in the securities comprising the index. The index includes securities of companies from the following industries: diversified financial services; insurance; banks; capital markets; mortgage real estate investment trusts ("REITs"); consumer finance; and thrifts and mortgage finance. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.599156	2020-10-11 23:38:31.023428
F	Ford Motor Company	Ford Motor Co manufactures, markets, and services Ford cars, trucks, sport utility vehicles, electronic vehicles, and Lincoln luxury vehicles. It also provides financial services through Ford Motor Credit Company.	{"Consumer Cyclical","Auto Manufacturers",Autos}	{TSLA,HMC,TM,TTM,GM}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:02.655051	2020-10-11 23:38:31.02582
UTMD	Utah Medical Products Inc.	Utah Medical Products Inc is in the business of producing medical devices that are predominantly proprietary, disposable and for hospital use. The firm's product includes Electrosurgical Pens, Footswitches, Fuses, Dispersive Pads and others.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{CNMD,MMSI,COO,JNJ,TFX,XLV}	Medical Instruments & Equipment	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:12.873167	2020-10-11 23:38:31.02582
UVV	Universal Corporation	Universal Corp is a leaf tobacco supplier. It is engaged in procuring and processing flue-cured and burley leaf tobacco for manufacturers of consumer tobacco products.	{"Consumer Defensive",Tobacco,"Tobacco Products"}	{SWM,BTI,VGR,MO,PM}	Tobacco Products	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:12.888914	2020-10-11 23:38:31.025871
VALU	Value Line Inc.	Value Line Inc is a U.S based company. It produces investment periodicals based on underlying research and making available copyright data, including ranking system and other information, to third parties under written agreements.	{"Consumer Cyclical",Publishing}	{TRI,TST,MORN,FDS,LEE,MDP,NWSA,SPGI}	Publishing	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:12.911039	2020-10-11 23:38:31.02582
VBIV	VBI Vaccines Inc.	VBI Vaccines Inc is a biopharmaceutical company. It is engaged in developing novel technologies that seek to expand vaccine protection in large underserved markets.	{Healthcare,Biotechnology}	{PDLI,PIRS,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:12.923275	2020-10-11 23:38:31.021426
VCEL	Vericel Corporation	Vericel Corp is a fully integrated, commercial-stage biopharmaceutical company. The company operates in one reportable segment: the research, manufacture, and distribution of patient-specific, used in the treatment of specific diseases.	{Healthcare,Biotechnology}	{GERN,MDCO,OSIR,ATHX,GLW,JNJ,MDT,CLBS,CUR,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:12.933542	2020-10-11 23:38:31.025871
VCR	Vanguard Consumer Discretion	The investment seeks to track the performance of a benchmark index.\n The fund employs an indexing investment approach designed to track the performance of the MSCI US Investable Market Index/Consumer Discretionary 25/50, an index made up of stocks of large, mid-size, and small U.S. companies within the consumer discretionary sector, as classified under the Global Industry Classification Standard. The Advisor attempts to replicate the target index by seeking to invest all, or substantially all, of its assets in the stocks that make up the index, in order to hold each stock in approximately the same proportion as its weighting in the index. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.94536	2020-10-11 23:38:31.021426
VEACW	Vantage Energy Acquisition Corp. Warrant		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:12.960762	2020-10-11 23:38:31.031328
VER	VEREIT Inc.	VEREIT Inc is a real estate investment trust primarily involved in the ownership of property located in the U.S. and Canada through its Real Estate Investment segment.	{"Real Estate","REIT - Diversified",REITs}	{ARE,ELS,GEO}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:12.975058	2020-10-11 23:38:31.028166
VGI	Virtus Global Multi-Sector Income Fund of Beneficial Interest	Virtus Global Multi-Sector Inc is a diversified, closed-end management investment company. It investment objective is to maximize current income while preserving capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:12.991858	2020-10-11 23:38:31.02423
VHT	Vanguard Health Care	The investment seeks to track the performance of a benchmark index.\n The fund employs an indexing investment approach designed to track the performance of the MSCI US Investable Market Index (IMI)/Health Care 25/50, an index made up of stocks of large, mid-size, and small U.S. companies within the health care sector, as classified under the Global Industry Classification Standard (GICS). The Advisor attempts to replicate the target index by seeking to invest all, or substantially all, of its assets in the stocks that make up the index, in order to hold each stock in approximately the same proportion as its weighting in the index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.014176	2020-10-11 23:38:31.02582
VIIX	VelocityShares Daily Long VIX Short-Term ETN	The investment seeks to replicate, net of expenses, the return of the daily performance of the S&P 500 VIX Short-Term Futures index.\n The index was designed to provide investors with exposure to one or more maturities of futures contracts on the VIX, which reflects implied volatility of the S&P 500 Index at various points along the volatility forward curve. The calculation of the VIX is based on prices of put and call options on the S&P 500 Index. The ETNs are linked to the daily return of the index and do not represent an investment in the VIX.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:13.036162	2020-10-11 23:38:31.023428
VKQ	Invesco Municipal Trust	Invesco Municipal Trust is active in the financial services domain. The trust's investment objective is to seek to provide a high level of current income exempt from federal income tax, consistent with preservation of capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:13.064563	2020-10-11 23:38:31.021426
VMI	Valmont Industries Inc.	Valmont Industries Inc is a producer of fabricated metal products, steel, irrigation systems, and other components.	{Industrials,"Metal Fabrication","Industrial Products"}	{NWPX,GIFI,APOG,IIIN,ROCK,OFLX,SSD,NOV,AWI}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:13.085248	2020-10-11 23:38:31.028166
UTSI	UTStarcom Holdings Corp	UTStarcom Holdings Corp is a telecom infrastructure provider. The company is engaged in developing technology that will serve the rapidly growing demand for bandwidth from mobile, streaming and other applications.	{"Communication Services","Telecom Services"}	{CSCO,ADTN,NTGR,RBBN}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:12:12.873579	2020-10-11 23:38:31.024353
UVSP	Univest Corporation of Pennsylvania	Univest Corp of Pennsylvania provides commercial and consumer banking and a full range of banking and trust services.	{"Financial Services","Banks - Regional - US",Banks}	{CZNC,NWFL,BMTC,PWOD,STBA,MTB,FNB,ACNB,BXS}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:12.885628	2020-10-11 23:38:31.028166
UYM	ProShares Ultra Basic Materials	The investment seeks daily investment results, before fees and expenses, that correspond to two times (2x) the daily performance of the Dow Jones U.S. Basic MaterialsSM Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index seeks to measure the performance of certain companies in the basic materials sector of the U.S. equity market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.901132	2020-10-11 23:38:31.023428
VAR	Varian Medical Systems Inc.	Varian Medical Systems Inc is engaged in the healthcare sector. It manufactures medical devices for conditions with radiotherapy, radiosurgery, proton therapy and brachytherapy.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{HOLX,ARAY,RMD,BDX,GE,TMO,PHG,ISR,ICAD,LLL,XLV}	Medical Instruments & Equipment	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:12.915369	2020-10-11 23:38:31.023007
VBK	Vanguard Small-Cap Growth	The investment seeks to track the performance of a benchmark index that measures the investment return of small-capitalization growth stocks.\n The fund employs an indexing investment approach designed to track the performance of the CRSP US Small Cap Growth Index, a broadly diversified index of growth stocks of small U.S. companies. The advisor attempts to replicate the target index by investing all, or substantially all, of its assets in the stocks that make up the index, holding each stock in approximately the same proportion as its weighting in the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.927526	2020-10-11 23:38:31.031328
VCRA	Vocera Communications Inc.	Vocera Communications Inc is a provider of secure, integrated, intelligent communication solutions, focused on empowering mobile workers in healthcare, hospitality, energy, and other mission-critical mobile work environments.	{Technology,"Communication Equipment"}	{CSCO,PLT,HRC,CLRO,HPQ,XLK}	Communication Equipment	Technology	New York Stock Exchange	\N	2020-10-11 23:12:12.948073	2020-10-11 23:38:31.02423
VEGA	AdvisorShares STAR Global Buy-Write	The investment seeks consistent repeatable returns across all market cycles.\n The fund is an actively managed exchange-traded fund ("ETF") that is primarily a "fund of funds". It invests in ETFs and exchange-traded notes ("ETNs") that seek to track a diversified basket of global indices and investment sectors that meet certain selection criteria established by Partnervest Advisory Services, LLC (the "Sub-Advisor"). It also may invest, subject to the same selection criteria, in exchange-traded products that invest directly in commodities or currencies and that are registered only pursuant to the Securities Act of 1933 (collectively with ETFs and ETNs, "ETPs").	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.967128	2020-10-11 23:38:31.021426
VFL	Delaware Investments National Municipal Income Fund	Delaware Investments Natl Muni Income is a closed-end investment fund. The Fund seeks to provide current income exempt from regular federal income tax consistent with the preservation of capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:12.9871	2020-10-11 23:38:31.025871
VGT	Vanguard Information Tech	The investment seeks to track the performance of a benchmark index.\n The fund employs an indexing investment approach designed to track the performance of the MSCI US Investable Market Index/Information Technology 25/50, an index made up of stocks of large, mid-size, and small U.S. companies within the information technology sector, as classified under the Global Industry Classification Standard. The Advisor attempts to replicate the target index by seeking to invest all, or substantially all, of its assets in the stocks that make up the index, in order to hold each stock in approximately the same proportion as its weighting in the index. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.001744	2020-10-11 23:38:31.023428
VIGI	Vanguard International Dividend Appreciation ETF	The investment seeks to track the performance of Nasdaq International Dividend Achievers Select Index that measures the investment return of non-U.S. companies that have a history of increasing dividends.\n The index focuses on high-quality companies located in developed and emerging markets, excluding the U.S., that have both the ability and the commitment to grow their dividends over time. The manager attempts to replicate the target index by investing all, or substantially all, of its assets in the broadly diversified collection of securities that make up the index, holding each stock in approximately the same proportion as its weighting in the index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:13.030438	2020-10-11 23:38:31.021426
VIV	Telefonica Brasil S.A. ADS	Telefonica Brasil SA is one of the largest wireless telephone company operating in Brazil. The firm also offers fixed-line telephone services in Sao Paulo State and owns broadband network in the country.	{"Communication Services","Telecom Services"}	{TEO,AMX,TI,TSU}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:13.046974	2020-10-11 23:38:31.021426
UVE	UNIVERSAL INSURANCE HOLDINGS INC	Universal Insurance Holdings Inc is a vertically integrated insurance holding company, which through its various subsidiaries, covers insurance underwriting, distribution, claims processing and exposure management.	{"Financial Services","Insurance - Property & Casualty"}	{}	Insurance - Property & Casualty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:12.885776	2020-10-11 23:38:31.031328
UZC	United States Cellular Corporation 7.25% Senior Notes due 2064		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:12.902126	2020-10-11 23:38:31.02423
VBND	ETF Series Solutions Trust Vident Core U.S. Bond Strategy Fund	The investment seeks capital appreciation.\n The fund is actively managed and seeks to achieve its investment objective by investing primarily in equity securities (or depositary receipts) of companies in developed markets outside the United States based on the results of a proprietary, quantitative model (the "EquBot Model") developed by the adviser that runs on the Watson™ platform. Its investment adviser is a technology based company focused on applying artificial intelligence ("AI") based solutions to investment analyses. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.93042	2020-10-11 23:38:31.028166
VDE	Vanguard Energy	The investment seeks to track the performance of a benchmark index.\n The fund employs an indexing investment approach designed to track the performance of the MSCI US Investable Market Index (IMI)/Energy 25/50, an index made up of stocks of large, mid-size, and small U.S. companies within the energy sector, as classified under the Global Industry Classification Standard (GICS). The Advisor attempts to replicate the target index by seeking to invest all, or substantially all, of its assets in the stocks that make up the index, in order to hold each stock in approximately the same proportion as its weighting in the index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.949475	2020-10-11 23:38:31.028166
VEGI	iShares MSCI Agriculture Producers Fund	The investment seeks to track the investment results of the MSCI ACWI Select Agriculture Producers Investable Market Index (IMI).\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index measures the combined performance of equity securities of companies in both developed and emerging markets that are primarily engaged in the business of agriculture. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.966877	2020-10-11 23:38:31.025871
VGFO	Virtus WMC Global Factor Opportunities	The investment seeks capital appreciation.\n The fund is an actively managed exchange-traded fund ("ETF") that seeks to achieve its objective by attempting to outperform the global equity market over a full market cycle within a framework that attempts to manage portfolio risk. It invests primarily in equity securities of U.S. and foreign issuers, including emerging markets issuers, of any market capitalization. The principal types of equity securities in which the fund invests are common and preferred stock, ADRs and GDRs. The fund may also invest in equity interests in real estate investment trusts ("REITs") of U.S. and foreign issuers.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.99239	2020-10-11 23:38:31.024353
VHI	Valhi Inc.	Valhi Inc through its subsidiaries provides chemicals, security products, waste management systems and real estate development and management services. It operates its business across three segments.	{"Basic Materials",Chemicals}	{ECOL,BAK,HUN,SWK,CIX,NL,KRO,PESI,TORM}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:13.006596	2020-10-11 23:38:31.021426
VIOO	Vanguard S&P Small-Cap 600	The investment seeks to track the performance of a benchmark index that measures the investment return of small-capitalization stocks in the United States.\n The fund employs an indexing investment approach designed to track the performance of the S&P SmallCap 600® Index. The Advisor attempts to replicate the target index by investing all, or substantially all, of its assets in the stocks that make up the index, holding each stock in approximately the same proportion as its weighting in the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.03942	2020-10-11 23:38:31.02423
VIXY	ProShares Trust VIX Short-Term Futures	The investment seeks results (before fees and expenses) that, both for a single day and over time, match the performance of the S&P 500 VIX Short-Term Futures Index for a single day.\nThe index seeks to offer exposure to market volatility through publicly traded futures markets and is designed to measure the return from a rolling long position in the first and second month VIX futures contracts.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.062179	2020-10-11 23:38:31.02582
VMC	Vulcan Materials Company (Holding Company)	Vulcan Materials Co is a producer of construction aggregates including crushed stone, sand and gravel; and asphalt mix and ready-mixed concrete in the United States. These materials are used to build the roads, tunnels, bridges, railroads and airports.	{"Basic Materials","Building Materials"}	{}	Building Materials	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:13.083153	2020-10-11 23:38:31.028166
VNDA	Vanda Pharmaceuticals Inc.	Vanda Pharmaceuticals Inc is a biopharmaceutical company focused on the development and commercialization of products for the treatment of central nervous system disorders to address unmet needs.	{Healthcare,Biotechnology}	{NVS,BMY,JNJ,LLY,PFE,MRK,PRTK,TTNP,PTX,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:13.095687	2020-10-11 23:38:31.02423
XYL	Xylem Inc. New	Xylem Inc is an industrial technology company. It provides equipment, technology, and services to transport, treat, test, and efficiently use water in public utilities, residential and commercial buildings, and industrial and agricultural settings.	{Industrials,"Diversified Industrials","Industrial Products"}	{FELE,FLS,GRC,GGG,IEX,TMO,TTC,CFX,ITT}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:13.697088	2020-10-11 23:38:31.025871
UWT	VelocityShares 3x Long Crude Oil ETNs linked to the S&P GSCI Crude Oil Index ER	The investment seeks to replicate, net of expenses, three times of the S&P GSCI® Crude Oil Index ER.\n The index tracks a hypothetical position in the nearest-to-expiration NYMEX light sweet crude oil futures contract, which is rolled each month into the futures contract expiring in the next month. The value of the index fluctuates with changes in the price of the relevant NYMEX light sweet crude oil futures contracts.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.892631	2020-10-11 23:38:31.028166
VBFC	Village Bank and Trust Financial Corp.	Village Bank and Trust Financial Corp is a bank holding company. The Company through its wholly-owned subsidiaries offers banking and related financial services, including checking, savings and certificates of deposit.	{"Financial Services","Banks - Regional - US",Banks}	{FBSS,ANCX,SONA,BOTJ,ESXB,UBSH}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:12.922786	2020-10-11 23:38:31.024353
VC	Visteon Corporation	Visteon Corp is an automotive supplier that designs, engineers and manufactures electronics products for original equipment vehicle manufacturer. Its products include automotive systems, modules and components and audio head units.	{"Consumer Cyclical","Auto Parts",Autos}	{MGA,LEA,SRI,APTV}	Autos	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:12.934735	2020-10-11 23:38:31.028166
VDC	Vanguard Consumer Staples	The investment seeks to track the performance of a benchmark index.\n The fund employs an indexing investment approach designed to track the performance of the MSCI US Investable Market Index/Consumer Staples 25/50, an index made up of stocks of large, mid-size, and small U.S. companies within the consumer staples sector, as classified under the Global Industry Classification Standard. The Advisor attempts to replicate the target index by seeking to invest all, or substantially all, of its assets in the stocks that make up the index, in order to hold each stock in approximately the same proportion as its weighting in the index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.948817	2020-10-11 23:38:31.023428
VEEV	Veeva Systems Inc. Class A	Veeva Systems Inc is a provider of industry-specific, cloud-based software solutions for the life sciences industry.	{Technology,"Health Information Services","Application Software"}	{MSFT,CERN,HSTM,CPSI,NUAN,MODN,CSLT,DXC,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:12.96429	2020-10-11 23:38:31.023007
VEU	Vanguard FTSE All World Ex US	The investment seeks to track the performance of a benchmark index that measures the investment return of stocks of companies located in developed and emerging markets outside of the United States.\n The fund employs an indexing investment approach designed to track the performance of the FTSE All-World ex US Index, a float-adjusted, market-capitalization-weighted index designed to measure equity market performance of international markets, excluding the United States. The index included 2,539 stocks of companies located in 46 countries, including both developed and emerging markets.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.980653	2020-10-11 23:38:31.023007
VGR	Vector Group Ltd.	Vector Group Ltd, through its subsidiaries, is engaged in the manufacture and sale of cigarettes in the United States and the real estate business. Its business segment includes: Tobacco, E-Cigarettes and Real Estate.	{"Consumer Defensive",Tobacco,"Tobacco Products"}	{BTI,MO,PM,XXII}	Tobacco Products	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:12.998449	2020-10-11 23:38:31.025871
VIAV	Viavi Solutions Inc.	Viavi Solutions Inc provides network testing, monitoring, and assurance products to communications service providers, wireless operators, cable operators, network-equipment manufacturers and enterprises.	{Technology,"Communication Equipment"}	{EXFO,RDCM,FNSR,OCLR,A,DHR,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:13.022352	2020-10-11 23:38:31.023007
VIRC	Virco Manufacturing Corporation	Virco Manufacturing Corp manufactures an assortment of products, including mobile tables, mobile storage equipment, desks, computer furniture, chairs, activity tables, folding chairs and folding tables.	{Industrials,"Business Equipment","Industrial Products"}	{FLXS,MLHR,KEQU,SCS,LEG,HNI,KNL}	Industrial Products	Industrials	NASDAQ Global Market	\N	2020-10-11 23:12:13.043327	2020-10-11 23:38:31.024353
VIXM	ProShares Trust VIX Mid-Term Futures	The investment seeks results that, both over a single day and over time, match (before fees and expenses) the performance of the S&P 500® VIX Mid-Term Futures Index.\n The fund seeks to meet its investment objective under normal market conditions by taking long or short positions in VIX futures contracts. It will also hold cash or cash equivalents such as U.S. Treasury securities or other high credit quality, short-term fixed-income or similar securities for direct investment or as collateral for Financial Instruments. The fund may invest up to 100% of its assets in any of these types of cash or cash equivalent securities.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.059838	2020-10-11 23:38:31.02423
VLT	Invesco High Income Trust II	Invesco High Income Trust II provides high current income, while seeking to preserve shareholders' capital, through investment in a professionally managed diversified portfolio of high-income producing fixed income securities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:13.077356	2020-10-11 23:38:31.023007
VMO	Invesco Municipal Opportunity Trust	Invesco Municipal Opportunity Trust is an asset management firm. It functions as a trust, whose investment objective is to provide a high level of current income exempt from federal income tax, consistent with preservation of capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:13.094118	2020-10-11 23:38:31.021426
VBTX	Veritex Holdings Inc.	Veritex Holdings Inc is a bank holding company. The Company through its subsidiary provides banking services to individual and corporate customers, which include commercial and retail lending, and the acceptance of checking and savings deposits.	{"Financial Services","Banks - Regional - US",Banks}	{GLBZ,NCOM}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:12.932259	2020-10-11 23:38:31.023007
VCYT	Veracyte Inc.	Veracyte Inc is a molecular diagnostics company that uses genomic technology to resolve diagnostic ambiguity. The Company's first commercial solution, the Afirma Thyroid FNA Analysis, centers on the proprietary Afirma Gene Expression Classifier.	{Healthcare,Biotechnology}	{MYGN,EXAS,GHDX,VRML,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:12.948421	2020-10-11 23:38:31.025871
VERU	Veru Inc.	Veru Inc is a biopharmaceutical company focused on urology and oncology. It develops prescription products for benign prostatic hyperplasia hot flashes associated with cancer treatment, male infertility and novel chemotherapies for a variety of cancers.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{XLV}	Drug Manufacturers	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:12.98034	2020-10-11 23:38:31.02423
VGLT	Vanguard Long-Term Government Bond ETF	The investment seeks to track the performance of a market-weighted government bond index with a long-term dollar-weighted average maturity.\n The fund employs an indexing investment approach designed to track the performance of the Bloomberg Barclays US Long Treasury Bond Index. This index includes fixed income securities issued by the U.S. Treasury (not including inflation-protected bonds), with maturities greater than 10 years. Under normal circumstances, at least 80% of the fund's assets will be invested in bonds included in the index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:13.002265	2020-10-11 23:38:31.028166
VICI	VICI Properties Inc.	Vici Properties Inc acts as an owner, acquirer and developer of real estate assets across gaming, hospitality, entertainment and leisure destinations.	{"Real Estate","REIT - Diversified",REITs}	{}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:13.022264	2020-10-11 23:38:31.028166
VIOV	Vanguard S&P Small-Cap 600 Value	The investment seeks to track the performance of a benchmark index that measures the investment return of small-capitalization value stocks in the United States.\n The fund employs an indexing investment approach designed to track the performance of the S&P SmallCap 600® Value Index, which represents the value companies, as determined by the index sponsor, of the S&P SmallCap 600 Index. The Advisor attempts to replicate the target index by investing all, or substantially all, of its assets in the stocks that make up the index, holding each stock in approximately the same proportion as its weighting in the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.038221	2020-10-11 23:38:31.023007
VIVO	Meridian Bioscience Inc.	Meridian Bioscience Inc is an integrated life science company. It is engaged in development, manufacture, sale and distribution of diagnostic test kits for certain gastrointestinal, viral, respiratory and parasitic infectious diseases.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{HOLX,QDEL,BDX,TMO,OXFD,XLV}	Medical Diagnostics & Research	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:13.059265	2020-10-11 23:38:31.023428
VLP	Valero Energy Partners LP representing limited partner interests	Valero Energy Partners LP owns, operates, develops, and acquires crude oil and refined petroleum products pipelines, terminals, and other transportation and logistics assets.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{RGCO,APU,VLO,UGI,UGP,ANDX}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:13.074246	2020-10-11 23:38:31.025871
VNM	VanEck Vectors Vietnam	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the MVIS® Vietnam Index.\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The index includes securities of Vietnamese companies. A company is generally considered to be a Vietnamese company if it is incorporated in Vietnam or is incorporated outside of Vietnam but has at least 50% of its revenues/related assets in Vietnam. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.099825	2020-10-11 23:38:31.024353
VOC	VOC Energy Trust Units of Beneficial Interest	VOC Energy Trust is a statutory trust, The Trust is created to acquire and hold a term net profits interest for the benefit of the Trust unitholders from production from the properties.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{DMLP,ROYT}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:13.111934	2020-10-11 23:38:31.021426
VOO	Vanguard S&P 500	The investment seeks to track the performance of a benchmark index that measures the investment return of large-capitalization stocks.\n The fund employs an indexing investment approach designed to track the performance of the Standard & Poor's 500 Index, a widely recognized benchmark of U.S. stock market performance that is dominated by the stocks of large U.S. companies. The advisor attempts to replicate the target index by investing all, or substantially all, of its assets in the stocks that make up the index, holding each stock in approximately the same proportion as its weighting in the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.123478	2020-10-11 23:38:31.023007
VPG	Vishay Precision Group Inc.	Vishay Precision Group Inc designs, manufactures and markets sensors, and sensor-based measurement systems, as well as specialty resistors and strain gages based upon proprietary technology.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{AME,MTD,CTS,ITW,TER,VSH,CTRL,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:13.134318	2020-10-11 23:38:31.023428
VGZ	Vista Gold Corp	Vista Gold Corp is a gold mining company. Its principal asset is its flagship Mt Todd gold project in Northern Territory, Australia. Other projects include non-core projects in Mexico & US, and royalty interests in projects in Indonesia.	{"Basic Materials",Gold,"Metals & Mining"}	{RGLD,HL,NEM,JAG,LODE}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:13.005361	2020-10-11 23:38:31.028166
VIDI	ETF Series Solutions Trust Vident International Equity Fund	The investment seeks to track the performance, before fees and expenses, of the Vident Core International Equity Index™.\n The advisor attempts to invest all, or substantially all, of its assets in the common stocks that make up the underlying index. The underlying index is a rules-based, systematic strategy index comprised of equity securities of issuers in developed and emerging markets outside of the United States.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.026872	2020-10-11 23:38:31.024353
VIPS	Vipshop Holdings Limited American Depositary Shares each representing two	Vipshop Holdings Ltd is an online discount retailer for brands in China. It offers branded products to consumers in China through flash sales on its vipshop.com, vip.com and lefeng.com websites.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{AMZN,EBAY,GPS,EXPR,RL,LITB,JD,JMEI}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:13.042165	2020-10-11 23:38:31.028166
VKTX	Viking Therapeutics Inc.	Viking Therapeutics Inc is a clinical-stage biopharmaceutical company. The company develops therapies for metabolic and endocrine disorders. It includes portfolio of five drug candidates in clinical trials or preclinical studies.	{Healthcare,Biotechnology}	{REPH,RARE,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:13.065394	2020-10-11 23:38:31.028166
VMIN	REX VolMAXX Short VIX Futures Strategy	The investment seeks to provide investors with short exposure to the implied volatility of the broad-based, large-cap U.S. equity market.\n The fund seeks to achieve its investment objective, under normal circumstances, by obtaining investment exposure to an actively managed portfolio of short positions in futures contracts based on the Chicago Board Options Exchange, Incorporated ("CBOE") Volatility Index (the "VIX Index"). The VIX Index seeks to measure the market's current expectation of 30-day volatility of the S&P 500® Index, as reflected by the prices of near-term S&P 500® options. It is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:13.087882	2020-10-11 23:38:31.02423
VNQ	Vanguard Real Estate	The investment seeks to provide a high level of income and moderate long-term capital appreciation by tracking the performance of the MSCI US Investable Market Real Estate 25/50 Index that measures the performance of publicly traded equity REITs and other real estate-related investments.\n The advisor attempts to track the index by investing all, or substantially all, of its assets-either directly or indirectly through a wholly owned subsidiary, which is itself a registered investment company-in the stocks that make up the index, holding each stock in approximately the same proportion as its weighting in the index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.107659	2020-10-11 23:38:31.023428
VONV	Vanguard Russell 1000 Value Index Fund	The investment seeks to track the performance of a benchmark index that measures the investment return of large-capitalization value stocks in the United States.\n The fund employs an indexing investment approach designed to track the performance of the Russell 1000® Value Index. The index is designed to measure the performance of large-capitalization value stocks in the United States. The Advisor attempts to replicate the target index by investing all, or substantially all, of its assets in the stocks that make up the index, holding each stock in approximately the same proportion as its weighting in the index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:13.123849	2020-10-11 23:38:31.02423
VPV	Invesco Pennsylvania Value Municipal Income Trust (DE)	Invesco Pennsylvania Value Municipal Income Trust is in the field of financial services. Its objective is to provide shareholders with a high level of current income exempt from federal and Pennsylvania income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:13.140158	2020-10-11 23:38:31.021426
VRP	Invesco Variable Rate Preferred	The investment seeks to track the investment results (before fees and expenses) of the Wells Fargo® Hybrid and Preferred Securities Floating and Variable Rate Index.\n The fund will invest at least 90% of its total assets in the securities that comprise the underlying index. The underlying index is a market capitalization-weighted index designed to track the performance of preferred stock, as well as certain types of "hybrid securities" that are functionally equivalent to preferred stocks, that are issued by U.S.-based or foreign issuers and that pay a floating or variable rate dividend or coupon. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.152564	2020-10-11 23:38:31.028166
VSEC	VSE Corporation	VSE Corp is a diversified services and supply chain management company. It provides logistics and distribution services for legacy systems and equipment. It also provides parts, supply-chain solutions, and other services for aviation jet aircraft engines.	{Industrials,"Engineering & Construction"}	{KTOS,KEYW,LMT,GD,NOC,TDY,RTN,EGL,AJRD}	Engineering & Construction	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:13.172762	2020-10-11 23:38:31.031328
VIA	Viacom Inc.	Viacom Inc is a global media company. It creates compelling television programs, motion pictures, short-form content, applications and games. Its brands include MTV, VH1, CMT, Logo, BET, Nickelodeon, Comedy central, TV Land, SPIKE, Tr3s, and VIVA.	{"Consumer Cyclical","Media - Diversified",Entertainment}	{RCI,NFLX,CMCSA,DISCA,DIS,CBS,AMCX,FOXA}	Entertainment	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:13.020411	2020-10-11 23:38:31.028166
VIOG	Vanguard S&P Small-Cap 600 Growth	The investment seeks to track the performance of a benchmark index that measures the investment return of small-capitalization growth stocks in the United States.\n The fund employs an indexing investment approach designed to track the performance of the S&P SmallCap 600® Growth Index, which represents the growth companies, as determined by the index sponsor, of the S&P SmallCap 600 Index. The Advisor attempts to replicate the target index by investing all, or substantially all, of its assets in the stocks that make up the index, holding each stock in approximately the same proportion as its weighting in the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.036162	2020-10-11 23:38:31.02582
VKI	Invesco Advantage Municipal Income Trust II of Beneficial Interest (DE)	Invesco Advantage Municipal Income Trust II is a diversified, closed-end management investment company. It investment objective is to provide a current income exempt from federal income tax, consistent with preservation of capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:13.06258	2020-10-11 23:38:31.024353
VLY	Valley National Bancorp	Valley National Bancorp is a bank holding company. It offers commercial, retail, insurance and wealth management financial services products. It also offers automated teller machines, telephone and internet banking services.	{"Financial Services","Banks - Regional - US",Banks}	{UNTY,SSFN,UEIC,PGC,LBAI,MTB,TRCB}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:13.079106	2020-10-11 23:38:31.023428
VOD	Vodafone Group Plc	Vodafone Group PLC is one of the largest wireless phone companies in the world. Its main line of business involves the provision of wireless mobile and data services to various parts of the world.	{"Communication Services","Telecom Services"}	{TEF,BT,TI,ORAN,VEON}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:12:13.11274	2020-10-11 23:38:31.025871
VOXX	VOXX International Corporation	VOXX International Corp is a manufacturer and distributor in the Automotive, Premium Audio and Consumer Accessories industries. It manufactures, distributes and markets rear-seat entertainment devices, home theater systems, music systems, among others.	{"Consumer Cyclical","Auto Parts",Autos}	{KOSS,LOGI,MSN,SNE,BBY,PHG}	Autos	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:13.131259	2020-10-11 23:38:31.025871
VRAY	ViewRay Inc.	ViewRay Inc is a medical device company. The company is engaged in designing, manufacturing and marketing MRI guided radiation therapy system that images and treats cancer patients.	{Healthcare,"Medical Devices"}	{RMD,XLV}	Medical Devices	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:13.145808	2020-10-11 23:38:31.023007
VRTSP	Virtus Investment Partners Inc. 7.25% Series D Mandatory Convertible Preferred Stock	Virtus Investment Partners Inc provides investment management and related services to individuals and institutions. Its retail products include open-end mutual funds, closed-end funds, exchange traded funds, and variable insurance funds.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:13.167078	2020-10-11 23:38:31.023428
VSLR	Vivint Solar Inc.	Vivint Solar Inc offers distributed solar energy to residential customers based on long-term customer contracts, such as power purchase agreements or lease the solar energy systems.	{Technology,Solar,Semiconductors}	{POWL,GNRC,SPLP,ETN,XLK}	Semiconductors	Technology	New York Stock Exchange	\N	2020-10-11 23:12:13.181926	2020-10-11 23:38:31.023007
VTA	Invesco Credit Opportunities Fund of Beneficial Interest	Invesco Dynamic Credit Opportunities Fund is an investment fund. The Fund's investment objective is to seek a high level of current income, with a secondary objective of capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:13.195321	2020-10-11 23:38:31.031328
VTNR	Vertex Energy Inc	Vertex Energy Inc is an environmental services company that recycles industrial waste streams and off-specification commercial chemical products. The Company's operating business segments are Black Oil, Refining and Marketing, and Recovery.	{Industrials,"Waste Management"}	{HCCI,ECOL,CLH,PESI}	Waste Management	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:13.20971	2020-10-11 23:38:31.028166
VUZI	Vuzix Corporation	Vuzix Corporation is engaged in the design, manufacture, marketing and sale of wearable display devices that are worn like eyeglasses and feature built-in video screens that enable the user to view video and digital content.	{Technology,"Consumer Electronics","Computer Hardware"}	{MRCY,HURC,SSYS,QUMU,XLK}	Computer Hardware	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:13.224244	2020-10-11 23:38:31.023428
YEXT	Yext Inc.	Yext Inc provides a knowledge engine platform that lets businesses manage their digital knowledge in the cloud and sync it to over 100 services including Apple Maps, Bing, Cortana, Facebook, Google, Google Maps, Instagram, Siri and Yelp.	{Technology,"Software - Infrastructure","Application Software"}	{PFPT,PFM,PCYG,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:13.706426	2020-10-11 23:38:31.028166
VLGEA	Village Super Market Inc. Class A Common Stock	Village Super Market Inc operates a chain of ShopRite supermarkets in New Jersey, eastern Pennsylvania and Maryland. It is engaged in retail sale of food and nonfood products.	{"Consumer Defensive","Grocery Stores","Retail - Defensive"}	{IMKTA,KR,WMK,SPTN}	Retail - Defensive	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:13.066974	2020-10-11 23:38:31.028166
VLUE	iShares Edge MSCI USA Value Factor	The investment seeks to track the investment results of the MSCI USA Enhanced Value Index composed of U.S. large- and mid-capitalization stocks with value characteristics and relatively lower valuations.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The index is based on a traditional market capitalization-weighted parent index, the MSCI USA Index (the "parent index"). The parent index includes U.S. large- and mid- capitalization stocks.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:13.07953	2020-10-11 23:38:31.023007
VNET	21Vianet Group Inc.	21Vianet Group Inc is a carrier-neutral internet data center service provider in China. It provides hosting and related services, managed network services and cloud computing infrastructure services, improving the reliability, security and speed.	{Technology,"Information Technology Services","Application Software"}	{BIDU,CCIH,CHA,INXN,EQIX,CATM,INAP,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:13.095561	2020-10-11 23:38:31.028166
VNTR	Venator Materials PLC	Venator Materials PLC is engaged in the manufacturing and marketing of chemical products. Its products include iron oxide, chromium oxide, titanium dioxide and among others. The titanium dioxide generates maximum revenue for the company.	{"Basic Materials",Chemicals}	{KRO,TROX,VNTR,TORM}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:13.109754	2020-10-11 23:38:31.024353
VOX	Vanguard Communication Services	The investment seeks to track the performance of a benchmark index.\n The fund employs an indexing investment approach designed to track the performance of the MSCI US Investable Market Transition Index/ Communication Services 25/50, an index made up of stocks of large, mid-size, and small U.S. companies within the communication services sector, as classified under the GICS. The Advisor attempts to replicate the target index by seeking to invest all, or substantially all, of its assets in the stocks that make up the index, in order to hold each stock in approximately the same proportion as its weighting in the index. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.129085	2020-10-11 23:38:31.024353
VQT	Barclays ETN S&P VEQTOR ETN	The investment seeks to replicate, net of expenses, the S&P 500 Dynamic VEQTOR Total Return Index.\n The index seeks to provide broad equity market exposure with an implied volatility hedge by dynamically allocating its notional investments among three components: equity, volatility and cash. The equity component of the index is represented by the S&P 500 Total Return Index and the volatility component of the index is represented by the S&P 500 VIX Short-Term Futures Index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.142911	2020-10-11 23:38:31.031328
VRTV	Veritiv Corporation	Veritiv Corp is engaged in business-to-business printing, packaging and facility supplies and equipment distribution services to printers, manufacturers, retailers and commercial facilities.	{Industrials,"Business Equipment","Industrial Products"}	{GTLS,XONE,CREG}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:13.16826	2020-10-11 23:38:31.02582
VSMV	VictoryShares US Multi-Factor Minimum Volatility E	The investment seeks to provide investment results that track the performance of the Nasdaq Victory US Multi-Factor Minimum Volatility Index before fees and expenses.\n The fund seeks to achieve its investment objective by investing, under normal market conditions, at least 80% of its assets in securities included in the Nasdaq Victory US Multi-Factor Minimum Volatility Index (the "index"). The index utilizes a rules-based approach designed to generate investment returns with less volatility than the broader U.S. market.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:13.184271	2020-10-11 23:38:31.02582
VTIP	Vanguard Short-Term Inflation-Protected Securities	The investment seeks to track the performance of a benchmark index that measures the investment return of inflation-protected public obligations of the U.S. Treasury with remaining maturities of less than 5 years.\n The fund employs an indexing investment approach designed to track the performance of the Bloomberg Barclays U.S. Treasury Inflation-Protected Securities (TIPS) 0-5 Year Index. The index is a market-capitalization-weighted index that includes all inflation-protected public obligations issued by the U.S. Treasury with remaining maturities of less than 5 years.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:13.201731	2020-10-11 23:38:31.023428
VTWV	Vanguard Russell 2000 Value Index Fund	The investment seeks to track the performance of a benchmark index that measures the investment return of small-capitalization value stocks in the United States.\n The fund employs an indexing investment approach designed to track the performance of the Russell 2000® Value Index. The index is designed to measure the performance of small-capitalization value stocks in the United States. The advisor attempts to replicate the target index by investing all, or substantially all, of its assets in the stocks that make up the index, holding each stock in approximately the same proportion as its weighting in the index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:13.217644	2020-10-11 23:38:31.031328
VVV	Valvoline Inc.	Valvoline Inc manufactures and distributes automotive, commercial, industrial lubricants, and automotive chemicals.	{Energy,"Oil & Gas Refining & Marketing","Oil & Gas - Refining & Marketing"}	{INDU,NL,NGVT}	Oil & Gas - Refining & Marketing	Energy	New York Stock Exchange	\N	2020-10-11 23:12:13.238044	2020-10-11 23:38:31.025871
VXZB	iPath Series B S&P 500 VIX Mid-Term Futures ETN	The investment seeks return linked to the performance of the S&P 500® VIX Mid-Term Futures Index TR.\nThe ETN offers exposure to futures contracts of specified maturities on the VIX index and not direct exposure to the VIX index or its spot level. The index is designed to provide investors with exposure to one or more maturities of futures contracts on the CBOE Volatility Index®.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:13.252437	2020-10-11 23:38:31.031328
VLO	Valero Energy Corporation	Valero Energy Corp is a petroleum refining and marketing company. The company produces conventional gasolines, distillates, jet fuel, asphalt, petrochemicals, lubricants, and other refined products as well as a slate of premium products.	{Energy,"Oil & Gas Refining & Marketing","Oil & Gas - Refining & Marketing"}	{CLMT,CVI,MPC,HFC,PSX,PBF,DK}	Oil & Gas - Refining & Marketing	Energy	New York Stock Exchange	\N	2020-10-11 23:12:13.067463	2020-10-11 23:38:31.02423
VMBS	Vanguard Mortgage-Backed Securities ETF	The investment seeks to track the performance of a market-weighted mortgage-backed securities index.\n The fund employs an indexing investment approach designed to track the performance of the Bloomberg Barclays U.S. MBS Float Adjusted Index. This index covers U.S. agency mortgage-backed pass-through securities. To be included in the index, pool aggregates must have at least $250 million currently outstanding and a weighted average maturity of at least 1 year. All of the fund's investments will be selected through the sampling process, and under normal circumstances, at least 80% of the fund's assets will be invested in bonds included in the index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:13.080182	2020-10-11 23:38:31.024353
VMW	Vmware Inc. Class A	VMWare Inc provides virtualization and cloud infrastructure solutions. It is engaged in the development and application of virtualization technologies with x86 server-based computing, separating application software from the underlying hardware.	{Technology,"Software - Infrastructure","Application Software"}	{MSFT,CTXS,SYMC,LOGM,RHT,IBM,ORCL,HPQ,GOOGL,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:13.093282	2020-10-11 23:38:31.031328
VOOG	Vanguard S&P 500 Growth	The investment seeks to track the performance of a benchmark index that measures the investment return of large-capitalization growth stocks in the United States.\n The fund employs an indexing investment approach designed to track the performance of the S&P 500® Growth Index, which represents the growth companies, as determined by the index sponsor, of the S&P 500 Index. The index measures the performance of large-capitalization growth stocks in the United States.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.124811	2020-10-11 23:38:31.021426
VPU	Vanguard Utilities	The investment seeks to track the performance of a benchmark index.\n The fund employs an indexing investment approach designed to track the performance of the MSCI US Investable Market Index (IMI)/Utilities 25/50, an index made up of stocks of large, mid-size, and small U.S. companies within the utilities sector, as classified under the Global Industry Classification Standard (GICS). The Advisor attempts to replicate the target index by seeking to invest all, or substantially all, of its assets in the stocks that make up the index, in order to hold each stock in approximately the same proportion as its weighting in the index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.140569	2020-10-11 23:38:31.024353
VRS	Verso Corporation	Verso Corp is a North American producer of printing papers, specialty papers, and pulp. Its products are used primarily in media and marketing applications, such as catalogs, magazines, advertising brochures, and annual reports.	{"Basic Materials","Paper & Paper Products","Forest Products"}	{}	Forest Products	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:13.157548	2020-10-11 23:38:31.021426
VSH	Vishay Intertechnology Inc.	Vishay Intertechnology Inc manufactures and supplies discrete semiconductors and passive components. The semiconductors include MOSFETs, diodes, and optoelectronic components. The passive components include resistive products, capacitors, and inductors.	{Technology,Semiconductors}	{DIOD,ON,AOSL,AVX,STM,KEM,XLK}	Semiconductors	Technology	New York Stock Exchange	\N	2020-10-11 23:12:13.175575	2020-10-11 23:38:31.021426
VTEB	Vanguard Tax-Exempt Bond	The investment seeks to track the Standard & Poor's National AMT-Free Municipal Bond Index, which measures the performance of the investment-grade segment of the U.S. municipal bond market.\n This index includes municipal bonds from issuers that are primarily state or local governments or agencies whose interest is exempt from U.S. federal income taxes and the federal alternative minimum tax (AMT). All of the fund's investments will be selected through the sampling process, and at least 80% of the fund's assets will be invested in securities held in the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.198407	2020-10-11 23:38:31.025871
VTWG	Vanguard Russell 2000 Growth Index Fund	The investment seeks to track the performance of a benchmark index that measures the investment return of small-capitalization growth stocks in the United States.\n The fund employs an indexing investment approach designed to track the performance of the Russell 2000® Growth Index. The index is designed to measure the performance of small-capitalization growth stocks in the United States. The advisor attempts to replicate the target index by investing all, or substantially all, of its assets in the stocks that make up the index, holding each stock in approximately the same proportion as its weighting in the index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:13.213962	2020-10-11 23:38:31.025871
VVR	Invesco Senior Income Trust (DE)	Invesco Senior Income Trust is a closed-ended investment management fund. Its investment objective involves a high level of current income consistent with the preservation of capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:13.231262	2020-10-11 23:38:31.02582
VXX	iPath S&P 500 VIX Short Term Futures TM ETN	The investment seeks to provide investors with exposure to the S&P 500 VIX Short-Term Futures™ Index Total Return. \nThe S&P 500 VIX Short-Term Futures™ Index Total Return (the "index") is designed to provide access to equity market volatility through CBOE Volatility Index® futures. The index offers exposure to a daily rolling long position in the first and second month VIX futures contracts and reflects the implied volatility of the S&P 500® at various points along the volatility forward curve.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.243083	2020-10-11 23:38:31.023007
VLRX	Valeritas Holdings Inc.	Valeritas Holdings Inc is a commercial-stage medical technology company. It is engaged in the business of improving health and simplifying life for diabetic patients by developing and commercializing innovative technologies.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{VLRX,XLV}	Medical Instruments & Equipment	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:13.075945	2020-10-11 23:38:31.023428
VMM	Delaware Investments Minnesota Municipal Income Fund II Inc.	Delaware Investments MN Muni Inc Fund II is a closed-end investment fund. The fund seeks to provide current income exempt from both regular federal income tax and Minnesota state personal income tax, consistent with the preservation of capital.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:13.090811	2020-10-11 23:38:31.02582
VONE	Vanguard Russell 1000 Index Fund	The investment seeks to track the performance of a benchmark index that measures the investment return of large-capitalization stocks in the United States.\n The fund employs an indexing investment approach designed to track the performance of the Russell 1000® Index. The index is designed to measure the performance of large-capitalization stocks in the United States. The Advisor attempts to replicate the target index by investing all, or substantially all, of its assets in the stocks that make up the index, holding each stock in approximately the same proportion as its weighting in the index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:13.121539	2020-10-11 23:38:31.031328
VRIG	PowerShares Variable Rate Investment Grade Portfol	The investment seeks to generate current income while maintaining low portfolio duration as a primary objective and capital appreciation as a secondary objective.\n The fund invests at least 80% of its net assets (plus any borrowings for investment purposes) in a portfolio of investment-grade, variable rate or floating rate debt securities. At least 80% of its net assets (plus any borrowings for investment purposes) will be invested in Variable Rate Instruments that are, at the time of purchase, investment grade (or in affiliated ETFs that invest primarily in any or all of the foregoing securities). It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:13.148714	2020-10-11 23:38:31.023428
VRTU	Virtusa Corporation	Virtusa Corp is an information technology services company. It uses an offshore delivery model to provide various information technology services, including IT consulting, technology implementation and application outsourcing.	{Technology,"Information Technology Services","Application Software"}	{TRI,PEGA,PRFT,CTSH,IBM,DXC,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:13.170399	2020-10-11 23:38:31.028166
VT	Vanguard Total World Stock Index	The investment seeks to track the performance of a benchmark index that measures the investment return of stocks of companies located in developed and emerging markets around the world.\n The fund employs an indexing investment approach designed to track the performance of the FTSE Global All Cap Index, a float-adjusted, market-capitalization-weighted index designed to measure the market performance of large-, mid-, and small-capitalization stocks of companies located around the world. The index included 7,781 stocks of companies located in 41 countries, including both developed and emerging markets.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.193178	2020-10-11 23:38:31.024353
VTR	Ventas Inc.	Ventas Inc is a real estate investment trust. It holds a diversified portfolio of seniors housing communities, skilled nursing facilities, medical office buildings, life science buildings, and hospitals.	{"Real Estate","REIT - Healthcare Facilities",REITs}	{DRE,UHT,BKD,HCP,SBRA,DOC,TIPT,HTA,WELL}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:13.209386	2020-10-11 23:38:31.024353
VVC	Vectren Corporation	Vectren Corp is an energy holding company. It provides gas and electricity to customers in adjoining service territories that cover nearly two-thirds of Indiana and west central Ohio.	{Utilities,"Utilities - Regulated Gas","Utilities - Regulated"}	{NI,OKE,SRE,ATO,BKH,SJI,UGI,GNE,SR,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:13.225912	2020-10-11 23:38:31.02423
VXUS	Vanguard Total International Stock	The investment seeks to track the performance of a benchmark index that measures the investment return of stocks issued by companies located in developed and emerging markets, excluding the United States.\n The fund employs an indexing investment approach designed to track the performance of the FTSE Global All Cap ex US Index, a float-adjusted market-capitalization-weighted index designed to measure equity market performance of companies located in developed and emerging markets, excluding the United States. The index includes approximately 5,800 stocks of companies located in over 45 countries.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:13.241573	2020-10-11 23:38:31.023428
WAB	Westinghouse Air Brake Technologies Corporation	Westinghouse Air Brake Technologies Corp is a provider of value-added, technology-based products and services for the rail industry. It provides its products and services through two main business segments, the Freight and Transit.	{Industrials,Railroads,"Transportation & Logistics"}	{RAIL,ARII,GBX,GE,TRN}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:13.263231	2020-10-11 23:38:31.028166
WB	Weibo Corporation	Weibo Corp operates in the internet content industry in China. It offers a social media platform for people to create, distribute and discover Chinese-language content.	{Technology,"Internet Content & Information","Online Media"}	{SINA,BIDU,FENG,FB,RENN,MEET,XLK}	Online Media	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:13.281177	2020-10-11 23:38:31.028166
YGYI	Youngevity International Inc.	Youngevity International Inc offers nutritional products and commercial coffee. The company operates in two segments, Direct Selling segment and Commercial Coffee segment.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{NUS,YGYI}	Consumer Packaged Goods	Consumer Defensive	NASDAQ Capital Market	\N	2020-10-11 23:12:13.708554	2020-10-11 23:38:31.025871
VNCE	Vince Holding Corp.	Vince Holding Corp is engaged in delivering luxury essentials like women's and men's apparel, women's and men's footwear, and handbags. It focuses on developing an elevated collection of Vince apparel and accessories.	{"Consumer Cyclical","Apparel Manufacturing","Manufacturing - Apparel & Furniture"}	{GIII,VFC,EXPR,PVH,TLRD}	Manufacturing - Apparel & Furniture	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:13.095451	2020-10-11 23:38:31.023428
VNRX	VolitionRX Limited	VolitionRX Ltd is a clinical-stage life sciences company. The company is engaged in developing blood-based diagnostic tests for detecting and diagnosing cancer and other diseases.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{CGIX,AMEH,XLV}	Medical Diagnostics & Research	Healthcare	NYSE American	\N	2020-10-11 23:12:13.108818	2020-10-11 23:38:31.02423
VOYA	Voya Financial Inc.	Voya Financial Inc is an investment and insurance company focused on providing retirement solutions to individual and institutional clients in the United States.	{"Financial Services","Asset Management"}	{AFL,TMK,RGA}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:13.134193	2020-10-11 23:38:31.031328
VRNA	Verona Pharma plc	Verona Pharma PLC develops drugs for respiratory diseases. Its products mainly include anti-inflammatory agents and bronchodilators.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:13.148926	2020-10-11 23:38:31.028166
VRSN	VeriSign Inc.	VeriSign Inc is a part of the internet content industry. It provides domain name registry and internet security for websites and enterprises around the world.	{Technology,"Internet Content & Information","Online Media"}	{AKAM,SYMC,IBM,VZ,T,SYNC,TCX,XLK}	Online Media	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:13.165484	2020-10-11 23:38:31.023007
VSTM	Verastem Inc.	Verastem Inc is a biopharmaceutical company engaged in discovering and developing drugs to improve outcomes for patients with cancer. Its product candidates include duvelisib and defactinib.	{Healthcare,Biotechnology}	{CELG,ECYT,GSK,PFE,LH,STML,OMED,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:13.188291	2020-10-11 23:38:31.028166
VTL	Vital Therapies Inc.	Vital Therapies Inc is a biotherapeutic company. The Company is engaged in developing a cell-based therapy targeting treatment acute forms of liver failure. Its product includes the ELAD System an extracorporeal bio-artificial liver support system.	{Healthcare,Biotechnology}	{QDEL,OSUR,VRML,CEMI,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:13.20193	2020-10-11 23:38:31.021426
VUG	Vanguard Growth	The investment seeks to track the performance of a benchmark index that measures the investment return of large-capitalization growth stocks.\n The fund employs an indexing investment approach designed to track the performance of the CRSP US Large Cap Growth Index, a broadly diversified index predominantly made up of growth stocks of large U.S. companies. The advisor attempts to replicate the target index by investing all, or substantially all, of its assets in the stocks that make up the index, holding each stock in approximately the same proportion as its weighting in the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.221228	2020-10-11 23:38:31.021426
VZ	Verizon Communications Inc.	Verizon Communications Inc is a provider of communications, information and entertainment products and services to consumers, businesses and governmental agencies. It offers voice, data and video services and solutions to wireless and wireline networks.	{"Communication Services","Telecom Services"}	{SHEN,AAPL,USM,T,TDS,VOD,SPOK,GOOGL,TMUS}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:13.259957	2020-10-11 23:38:31.028166
WAIR	Wesco Aircraft Holdings Inc.	Wesco Aircraft Holdings Inc distributes & supplies supply chain management services to aerospace industry. It offers services including traditional distribution to the management of supplier relationships, JIT delivery & point-of-use inventory management.	{Industrials,"Industrial Distribution"}	{HEI,TGI,ESL,SIF,UTX}	Industrial Distribution	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:13.274695	2020-10-11 23:38:31.025871
WBIC	WBI BullBear Yield 2000	The investment seeks long-term capital appreciation and the potential for current income, while also seeking to protect principal during unfavorable market conditions.\n The fund will seek to invest in the dividend-paying equity securities of small-capitalization and mid-capitalization domestic and foreign companies. These securities will be selected on the basis of the Sub-Advisor's proprietary selection process ("Selection Process"). Cash and cash equivalents are some of the investment opportunities evaluated by the Selection Process. It may invest up to 50% of its net assets in the securities of issuers in emerging markets.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.293694	2020-10-11 23:38:31.028166
WBT	Welbilt Inc.	Manitowoc Foodservice Inc operates as a foodservice equipment company serving commercial and institutional sectors.	{Industrials,"Diversified Industrials","Industrial Products"}	{MIDD,SXI,WBT}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:13.312499	2020-10-11 23:38:31.028166
VO	Vanguard Mid-Cap	The investment seeks to track the performance of a benchmark index that measures the investment return of mid-capitalization stocks.\n The fund employs an indexing investment approach designed to track the performance of the CRSP US Mid Cap Index, a broadly diversified index of stocks of mid-size U.S. companies. The advisor attempts to replicate the target index by investing all, or substantially all, of its assets in the stocks that make up the index, holding each stock in approximately the same proportion as its weighting in the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.113362	2020-10-11 23:38:31.023007
VOOV	Vanguard S&P 500 Value	The investment seeks to track the performance of a benchmark index that measures the investment return of large-capitalization value stocks in the United States.\n The fund employs an indexing investment approach designed to track the performance of the S&P 500® Value Index, which represents the value companies, as determined by the index sponsor, of the S&P 500 Index. The index measures the performance of large-capitalization value stocks in the United States.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.12866	2020-10-11 23:38:31.028166
VRA	Vera Bradley Inc.	Vera Bradley Inc operates in the footwear and accessories industry. Its products include handbags and accessories, luggage and travel items, eyewear and stationery.	{"Consumer Cyclical","Footwear & Accessories","Manufacturing - Apparel & Furniture"}	{SHOO,TLF,DIS,KORS,TPR}	Manufacturing - Apparel & Furniture	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:13.144805	2020-10-11 23:38:31.02423
VRSK	Verisk Analytics Inc.	Verisk Analytics Inc is a data analytics provider serving customers in insurance, natural resources and financial services.	{Industrials,"Business Services"}	{CRD.B,FICO,CLGX}	Business Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:13.158466	2020-10-11 23:38:31.031328
VSDA	VictoryShares Dividend Accelerator ETF	The investment seeks to provide investment results that track the performance of the Nasdaq Victory Dividend Accelerator Index before fees and expenses.\n The fund seeks to achieve its investment objective by investing, under normal market conditions, at least 80% of its assets in securities included in the Nasdaq Victory Dividend Accelerator Index (the "index"). The index utilizes a rules-based approach designed to identify dividend paying stocks with a higher likelihood of future dividend growth.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:13.173922	2020-10-11 23:38:31.028166
VTHR	Vanguard Russell 3000 Index Fund	The investment seeks to track the performance of a benchmark index that measures the investment return of the broad U.S. stock market.\n The fund employs an indexing investment approach designed to track the performance of the Russell 3000® Index, which represents approximately 98% of the U.S. equity market and comprises the 3,000 largest companies in the United States. The Advisor attempts to replicate the target index by investing all, or substantially all, of its assets in the stocks that make up the index, holding each stock in approximately the same proportion as its weighting in the index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:13.198381	2020-10-11 23:38:31.023007
VTRB	Ventas Realty Limited Partnership // Capital Corporation 5.45% Senior Notes due 2043	Ventas Inc is a real estate investment trust. It holds a diversified portfolio of seniors housing communities, skilled nursing facilities, medical office buildings, life science buildings, and hospitals.	{"Real Estate","REIT - Healthcare Facilities",REITs}	{}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:13.209942	2020-10-11 23:38:31.02582
VWAGY	Volkswagen AG ADR Repstg 1/10th Sh	Volkswagen AG is a Germany-based automotive manufacturers. The company's automotive brands comprises Volkswagen passenger cars, Audi, Bentley, Bugatti, Lamborghini, Porsche, SEAT, and Skoda.	{}	{}	Autos	Consumer Cyclical	OTC Pink	\N	2020-10-11 23:12:13.242013	2020-10-11 23:38:31.024353
VZA	Verizon Communications Inc. 5.90% Notes due 2054	Verizon Communications Inc is a provider of communications, information and entertainment products and services to consumers, businesses and governmental agencies. It offers voice, data and video services and solutions to wireless and wireline networks.	{"Communication Services","Telecom Services"}	{}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:13.260843	2020-10-11 23:38:31.023428
WAL	Western Alliance Bancorporation (DE)	Western Alliance Bancorp is a bank holding company. It provides specialized banking services including deposit, lending, treasury management, and online banking products and services. The company's primary source of revenue is interest income.	{"Financial Services","Banks - Regional - US",Banks}	{CVBF,GBCI,ZION,PBCT,FIBK,WFC,NBHC,GBNK}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:13.274243	2020-10-11 23:38:31.02423
WBAL	WisdomTree Balanced Income Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree Balanced Income Index.\n Under normal circumstances, at least 80% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is designed to provide a balanced exposure to global equities and fixed income, including developed and emerging markets. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.287273	2020-10-11 23:38:31.021426
VPL	Vanguard FTSE Pacific	The investment seeks to track the performance of a benchmark index that measures the investment return of stocks issued by companies located in the major markets of the Pacific region.\n The fund employs an indexing investment approach by investing all, or substantially all, of its assets in the common stocks included in the FTSE Developed Asia Pacific All Cap Index. The FTSE Developed Asia Pacific All Cap Index is a market-capitalization-weighted index that is made up of approximately 2,256 common stocks of large-, mid-, and small-cap companies located in Japan, Australia, South Korea, Hong Kong, and Singapore.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.136601	2020-10-11 23:38:31.028166
VRNT	Verint Systems Inc.	Verint Systems Inc is a provider of actionable intelligence solutions and services for customer engagement optimization, security intelligence, and fraud, risk and compliance.	{Technology,"Software - Application","Application Software"}	{NICE,PEGA,HON,UTX,CRM,EGAN,ORCL,JCI,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:13.151352	2020-10-11 23:38:31.024353
VRTS	Virtus Investment Partners Inc.	Virtus Investment Partners Inc provides investment management and related services to individuals and institutions. Its retail products include open-end mutual funds, closed-end funds, exchange traded funds, and variable insurance funds.	{"Financial Services","Asset Management"}	{GROW,TROW,WDR,FII,EV,BEN,LM,MN,APAM}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:13.167376	2020-10-11 23:38:31.02423
VSI	Vitamin Shoppe Inc	Vitamin Shoppe Inc is a specialty retailer and contract manufacturer of nutritional products. It offers vitamins, minerals, specialty supplements, herbs, sports nutrition, under brands such as The Vitamin Shoppe, BodyTech, True Athlete, and MyTrition.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{RAD,USNA,GNC,WBA}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:13.1818	2020-10-11 23:38:31.02423
VTC	Vanguard Total Corporate ETF	The investment seeks to track the performance of a broad, market-weighted corporate bond index.\n The fund is a fund of funds and employs an indexing investment approach designed to track the performance of the Bloomberg Barclays U.S. Corporate Bond Index, which measures the investment-grade, fixed-rate, taxable corporate bond market. The index includes U.S. dollar-denominated securities that are publicly issued by industrial, utility, and financial issuers.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:13.197851	2020-10-11 23:38:31.028166
VTWO	Vanguard Russell 2000 Index Fund	The investment seeks to track the performance of a benchmark index that measures the investment return of small-capitalization stocks in the United States.\n The fund employs an indexing investment approach designed to track the performance of the Russell 2000® Index. The index is designed to measure the performance of small-capitalization stocks in the United States. The advisor attempts to replicate the target index by investing all, or substantially all, of its assets in the stocks that make up the index, holding each stock in approximately the same proportion as its weighting in the index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:13.216448	2020-10-11 23:38:31.023007
VVUS	VIVUS Inc.	VIVUS Incis a biopharmaceutical company engaged in the development and commercialization of therapeutic products for large underserved markets. Its product line includes Qsymia for sleep apnea, STENDRA for erectile dysfunction.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:13.231353	2020-10-11 23:38:31.031328
VXXB	iPath Series B S&P 500 VIX Short-Term Futures ETN	The investment seeks return linked to the performance of the S&P 500® VIX Short-Term Futures Index TR.\nThe ETN offers exposure to futures contracts of specified maturities on the VIX index and not direct exposure to the VIX index or its spot level. The index is designed to provide investors with exposure to one or more maturities of futures contracts on the CBOE Volatility Index®.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:13.245146	2020-10-11 23:38:31.02582
VYMI	Vanguard International High Dividend Yield ETF	The investment seeks to track the performance of FTSE All-World ex US High Dividend Yield Index that measures the investment return of non-U.S. companies that are characterized by high dividend yield.\n The fund invests by sampling the index, meaning that it holds a broadly diversified collection of securities that, in the aggregate, approximates the full index in terms of key characteristics. The index focuses on companies located in developed and emerging markets, excluding the United States, that are forecasted to have above-average dividend yields.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:13.260086	2020-10-11 23:38:31.024353
WALA	Western Alliance Bancorporation 6.25% Subordinated Debentures due 2056		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:13.27477	2020-10-11 23:38:31.023428
WBIB	WBI BullBear Value 2000	The investment seeks long-term capital appreciation and the potential for current income, while protecting principal during unfavorable market conditions.\n The fund will seek to invest in the equity securities of small-capitalization and mid-capitalization domestic and foreign companies. These securities will be selected on the basis of the Sub-Advisor's proprietary selection process ("Selection Process"). Cash and cash equivalents are some of the investment opportunities evaluated by the Selection Process. It may invest up to 50% of its net assets in the securities of issuers in emerging markets.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.292145	2020-10-11 23:38:31.025871
WCG	Wellcare Health Plans Inc.	WellCare Health Plans Inc operates in the healthcare industry. Its key business includes provision of health care plans and prescription drug plans to families, children and individuals.	{Healthcare,"Health Care Plans"}	{HUM,AET,CNC,MOH,GTS,XLV}	Health Care Plans	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:13.319359	2020-10-11 23:38:31.023007
VREX	Varex Imaging Corporation	Varex Imaging Corp is engaged in designing and manufacturing of X-ray tubes, flat panel detectors, imaging software and inspection accelerators.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{HOLX,VREX,XLV}	Medical Diagnostics & Research	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:13.147453	2020-10-11 23:38:31.02582
VRTX	Vertex Pharmaceuticals Incorporated	Vertex Pharmaceuticals Inc is engaged in the business of discovering, developing, manufacturing and commercializing small molecule drugs for patients with serious diseases in specialty markets. Its products include SYMDEKO, ORKAMBI, and KALYDECO.	{Healthcare,Biotechnology}	{GILD,BMY,JNJ,MRK,ABBV,ENTA,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:13.169281	2020-10-11 23:38:31.025871
VSM	Versum Materials Inc.	Versum Materials Inc is a provider of electronic materials for the integrated circuit and flat-panel display markets. It supplies specialty chemicals and materials, high-purity specialty gases, proprietary delivery systems, and related services.	{"Basic Materials",Chemicals}	{TAIT}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:13.182171	2020-10-11 23:38:31.028166
VTGN	VistaGen Therapeutics Inc.	VistaGen Therapeutics Inc is a clinical-stage biopharmaceutical company that develops and commercializes product candidates for patients with central nervous system diseases. Its product AV-101 is currently being evaluated in a Phase 2 monotherapy study.	{Healthcare,Biotechnology}	{GNCA,IOVA,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:13.197634	2020-10-11 23:38:31.02582
VTV	Vanguard Value	The investment seeks to track the performance of a benchmark index that measures the investment return of large-capitalization value stocks.\n The fund employs an indexing investment approach designed to track the performance of the CRSP US Large Cap Value Index, a broadly diversified index predominantly made up of value stocks of large U.S. companies. The advisor attempts to replicate the target index by investing all, or substantially all, of its assets in the stocks that make up the index, holding each stock in approximately the same proportion as its weighting in the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.212913	2020-10-11 23:38:31.028166
VVPR	VivoPower International PLC	Vivopower International PLC is a solar power company that operates a build, transfer, operate model to establish an installed solar power assets.	{Technology,Solar,Semiconductors}	{VSLR,XLK}	Semiconductors	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:13.230732	2020-10-11 23:38:31.023007
WAAS	AquaVenture Holdings Limited	AquaVenture Holdings Ltd is a multinational provider of desalination and wastewater treatment solutions and filtered water systems. It delivers WAAS solutions through two operating platforms: Seven Seas Water and Quench.	{Utilities,"Utilities - Regulated Water","Utilities - Regulated"}	{SJW,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:13.263123	2020-10-11 23:38:31.02423
WATT	Energous Corporation	Energous Corp is the developer of WattUp, a wire-free charging technology. WattUp is a radio frequency based charging solution that delivers intelligent, scalable power via radio bands, similar to a Wi-Fi router.	{Technology,"Electronic Components","Computer Hardware"}	{PNTR,MTSL,BB,XLK}	Computer Hardware	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:13.280925	2020-10-11 23:38:31.024353
WBIE	WBI BullBear Rising Income 1000	The investment seeks long-term capital appreciation and the potential for current income, while also seeking to protect principal during unfavorable market conditions.\n The fund will seek to invest in the equity securities of large capitalization domestic and foreign companies. These securities will be selected on the basis of the Sub-Advisor's proprietary selection process ("Selection Process"). Cash and cash equivalents are some of the investment opportunities evaluated by the Selection Process. It may invest up to 50% of its net assets in the securities of issuers in emerging markets.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.294576	2020-10-11 23:38:31.028166
WBK	Westpac Banking Corporation	Westpac Banking Corp is engaged in providing banking and financial services. It offers various services such as lending, deposit, payments service, investment portfolio management, superannuation and funds management, insurance services and others.	{"Financial Services","Banks - Global",Banks}	{CM,RY,TD,BCS,IBN,SHG,C}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:13.309848	2020-10-11 23:38:31.025871
VSAT	ViaSat Inc.	Viasat Inc is engaged in the provision of high-speed fixed and mobile broadband services. The Company also offers satellite and other wireless networks and secure networking systems, products and services.	{Technology,"Communication Equipment"}	{GILT,CMTL,TRMB,LORL,SATS,DISH,GD,HRS,LLL,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:13.172334	2020-10-11 23:38:31.024353
VSTO	Vista Outdoor Inc.	Vista Outdoor Inc is a U.S. based designer, manufacturer, and marketer of consumer products in the outdoor sports and recreation markets. The company organizes itself into two business segments: Shooting Sports and Outdoor Products.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{POOL,CENTA,BC,ELY}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:13.192649	2020-10-11 23:38:31.021426
VTN	Invesco Trust for Investment Grade New York Municipals	Invesco Trust For Investment Grade New York Municipals is a closed end management investment company. Its objective is to provide common shareholders with a high level of current income exempt from federal as well as from New York State income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:13.208003	2020-10-11 23:38:31.031328
VVI	Viad Corp	Viad Corp is a United States based marketing company. The company offers a complete range of live event services, which include official show services, audio-visual services, design services to event holders and brand marketers.	{Industrials,"Business Services"}	{TRIP}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:13.226265	2020-10-11 23:38:31.028166
VYM	Vanguard High Dividend Yield	The investment seeks to track the performance of a benchmark index that measures the investment return of common stocks of companies that are characterized by high dividend yield.\n The fund employs an indexing investment approach designed to track the performance of the FTSE High Dividend Yield Index, which consists of common stocks of companies that pay dividends that generally are higher than average. The adviser attempts to replicate the target index by investing all, or substantially all, of its assets in the stocks that make up the index, holding each stock in approximately the same proportion as its weighting in the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.259731	2020-10-11 23:38:31.025871
WAGE	WageWorks Inc.	WageWorks Inc is a provider of tax-advantaged programs in the United States. It mainly offers consumer-directed benefits such as health savings accounts, health and dependent-care flexible spending accounts and commuter benefit services.	{Industrials,"Business Services"}	{BBSI,ADP,PAYX,KFY,NSP,TNET}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:13.274344	2020-10-11 23:38:31.021426
WBID	WBI BullBear Quality 2000	The investment seeks long-term capital appreciation and the potential for current income, while protecting principal during unfavorable market conditions.\n The fund will seek to invest in the equity securities of small-capitalization and mid-capitalization domestic and foreign companies. These securities will be selected on the basis of the Sub-Advisor's proprietary selection process ("Selection Process"). Cash and cash equivalents are some of the investment opportunities evaluated by the Selection Process. It may invest up to 50% of its net assets in the securities of issuers in emerging markets.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.295188	2020-10-11 23:38:31.023428
WBIL	WBI BullBear Quality 1000	The investment seeks long-term capital appreciation and the potential for current income, while also seeking to protect principal during unfavorable market conditions.\n The fund will seek to invest in the equity securities of large capitalization domestic and foreign companies that WBI Investments, Inc., the sub-advisor ("Sub-Advisor") to the fund and an affiliate of Millington Securities Inc., the advisor ("Advisor"), believes display an attractive financial condition and prospects for ongoing financial stability, and in other tactical investment opportunities. It may invest up to 50% of its net assets in the securities of issuers in emerging markets.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.310245	2020-10-11 23:38:31.021426
WDR	Waddell & Reed Financial Inc.	Waddell & Reed Financial Inc is an asset management compant that dervies revenues from providing investment management, investment advisory, investment product underwriting and distribution, and shareholder services administration.	{"Financial Services","Asset Management"}	{GROW,TROW,VRTS,EV,BEN,LM,WETF,MN}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:13.333413	2020-10-11 23:38:31.023428
WEX	WEX Inc.	WEX Inc is a provider of corporate payment solutions. It operates through Fleet Solutions, Travel and Corporate Solutions, and Health and Employee Benefit Solutions.	{Industrials,"Business Services"}	{EEFT,GPN,FIS,MA,WU,FLT,MGI,CATM}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:13.350285	2020-10-11 23:38:31.02582
WHG	Westwood Holdings Group Inc	Westwood Holdings Group Inc, through its subsidiaries is engaged in managing investment assets & providing services for its clients. It provides investment advisory services to corporate retirement plans, public retirement plans, endowments & foundations.	{"Financial Services","Asset Management"}	{GROW,DHIL,AMG,BEN,PZN,GBL,SAMG}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:13.369365	2020-10-11 23:38:31.031328
WINA	Winmark Corporation	Winmark Corp is a franchisor of retail store concepts that buy, sell, trade and consign merchandise. Additionally, it operates a small-ticket financing business.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{ANF,GPS,WMT,DKS,BBW}	Retail - Apparel & Specialty	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:13.385305	2020-10-11 23:38:31.023007
VTVT	vTv Therapeutics Inc.	vTv Therapeutics Inc is a biopharmaceutical company engaged in drug discovery and development. Its therapies are intended to treat diseases such as Alzheimer's and type-II diabetes. Its other products include TTP399, type II diabetes drug candidate.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:13.21281	2020-10-11 23:38:31.02423
VXZ	iPath S&P 500 VIX Mid-Term Futures ETN	The investment seeks to provide investors with exposure to the S&P 500 VIX Mid-Term Futures™ Index Total Return. \nThe S&P 500 VIX Mid-Term Futures™ Index Total Return (the "index") is designed to provide access to equity market volatility through CBOE Volatility Index® futures. The index offers exposure to a daily rolling long position in the fourth, fifth, sixth and seventh month VIX futures contracts and reflects the implied volatility of the S&P 500® at various points along the volatility forward curve.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.251856	2020-10-11 23:38:31.028166
WABC	Westamerica Bancorporation	Westamerica Bancorp is a regional community bank. The bank focuses on commercial clients, such as smaller businesses across the United States.	{"Financial Services","Banks - Regional - US",Banks}	{CATY,BSRR,HAFC}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:13.269753	2020-10-11 23:38:31.031328
WBA	Walgreens Boots Alliance Inc.	Walgreens Boots Alliance Inc is a pharmacy led health and wellbeing company. It operates retail pharmacy store chain in the United States. The firm sells and markets its products under brands such as Walgreens, Duane Reade, Boots and Alliance Healthcare.	{"Consumer Defensive","Pharmaceutical Retailers","Retail - Defensive"}	{PETS,BIOS,CVS,RAD,VSI}	Retail - Defensive	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:13.286732	2020-10-11 23:38:31.023007
WBIR	WBI BullBear Global Rotation	The investment seeks long term-capital appreciation while also seeking to protect principal during unfavorable market conditions.\n The fund will seek to achieve its investment objective by using a broad and flexible investment strategy and may invest directly in different types of instruments, such as: stocks and bonds issued by public and private companies and bonds issued by governments and governmental agencies ("Direct Investments"); and shares of funds that invest directly in such instruments. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.310806	2020-10-11 23:38:31.02423
WDC	Western Digital Corporation	Western Digital Corp is the global leader in the hard disk drive market. It develops, manufactures, and provides data storage solutions that enable consumers to create, manage, experience and preserve digital content. Its products include HDDs and SSDs.	{Technology,"Data Storage","Computer Hardware"}	{NTGR,NTAP,STX,TDC,HPQ,SMCI,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:13.329777	2020-10-11 23:38:31.028166
WHF	WhiteHorse Finance Inc.	WhiteHorse Finance Inc is an externally managed, non-diversified, closed-end management investment company that has elected to be treated as a business development. It makes debt investments in privately held, small-cap companies.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:13.364774	2020-10-11 23:38:31.021426
WHLRW	Wheeler Real Estate Investment Trust Inc. Warrants		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:13.378147	2020-10-11 23:38:31.023007
WING	Wingstop Inc.	Wingstop Inc is a franchisor and operator of restaurants that specializes in cooked-to-order, hand-sauced and tossed chicken wings.	{"Consumer Cyclical",Restaurants}	{DAVE,DENN,EAT}	Restaurants	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:13.392733	2020-10-11 23:38:31.028166
WLDR	Affinity World Leaders Equity	The investment seeks to provide investment results that correspond generally, before fees and expenses, to the performance of the Thomson Reuters StarMine Affinity World Leaders Index (the "index").\n The fund normally invests at least 80% of its net assets (plus any borrowings for investment purposes) in equity securities included in the index. The index is expected to consist of approximately 150 to 250 stocks of companies located in the U.S. and in at least three countries outside of the U.S. and may invest in up to 20 countries. It will be significantly invested in the securities of international companies in addition to securities of U.S. companies.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:13.406513	2020-10-11 23:38:31.025871
WMGIZ	Wright Medical Group N.V. Contingent Value Right		{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:13.423098	2020-10-11 23:38:31.028166
WNS	WNS (Holdings) Limited Sponsored ADR (Jersey)	WNS (Holdings) Ltd is a global provider of business process management (BPM) services, offering comprehensive data, voice, analytical and business transformation services with a blended onshore, near shore and offshore delivery model.	{Industrials,"Business Services"}	{INOD,EXLS,WIT,BR,G,ACN,AXR,INFY,XRX}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:13.437197	2020-10-11 23:38:31.028166
YIN	Yintech Investment Holdings Limited	Yintech Investment Holdings Ltd is an online provider of spot commodity trading services in China. It facilitates the trading by individual customers of silver, gold and other precious metals and commodities.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{RJF}	Brokers & Exchanges	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:13.711231	2020-10-11 23:38:31.023428
VYGR	Voyager Therapeutics Inc.	Voyager Therapeutics Inc in collaboration with Genzyme, conducts research and development work to develop therapies for the treatment of central nervous system disorders. It targets diseases such as Parkinson's, Alzheimer's Huntington's.	{Healthcare,Biotechnology}	{PDLI,PIRS,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:13.254621	2020-10-11 23:38:31.021426
WAFD	Washington Federal Inc.	Washington Federal Inc is a bank holding company operating through its subsidiaries. Its activity includes accepting deposits and lending out loans of various types. The bank is also engaged in real estate investment and insurance brokerage activities.	{"Financial Services","Savings & Cooperative Banks",Banks}	{RVSB,TSBK,BYFC,FFNW,TBNK,ANCB,FSBW,BANC}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:13.273135	2020-10-11 23:38:31.02582
WBAI	500.com Limited American Depositary Shares each representing 10 Class A shares	500.com Ltd is an online sports lottery service provider which operates only in China. It generates most of the revenue from service fees which are paid by administration centers and others for purchase of lottery.	{"Consumer Cyclical",Gambling,"Travel & Leisure"}	{SGMS,AMC,EVRI}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:13.287441	2020-10-11 23:38:31.031328
WBII	WBI BullBear Global Income	The investment seeks current income with the potential for long-term capital appreciation, while also seeking to protect principal during unfavorable market conditions.\n Under normal market conditions, the fund will invest at least 80% of its net assets, plus the amount of any borrowings for investment purposes, in income-producing debt and equity securities of foreign and domestic issuers, including the securities of foreign and domestic corporate and government entities.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.306928	2020-10-11 23:38:31.025871
WFHY	WisdomTree Fundamental U.S. High Yield Corporate Bond Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree Fundamental U.S. High Yield Corporate Bond Index.\n Under normal circumstances, at least 80% of the fund's total assets will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is designed to capture the performance of selected issuers in the U.S. non-investment-grade corporate bond market that are deemed to have favorable fundamental and income characteristics. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:13.361239	2020-10-11 23:38:31.031328
WHLRD	Wheeler Real Estate Investment Trust Inc. Series D Cumulative Preferred Stock	Wheeler Real Estate Investment Trust Inc is a self-managed commercial real estate investment company.. It primarily invests in shopping centers, neighborhood centers, grocery-anchored centers, community centers and free-standing retail properties.	{"Real Estate","REIT - Retail",REITs}	{}	REITs	Real Estate	NASDAQ Capital Market	\N	2020-10-11 23:12:13.375875	2020-10-11 23:38:31.02582
WIP	SPDR FTSE International Government Inflation-Protected Bond	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the price and yield performance of the FTSE International Inflation-Linked Securities Select Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index or in securities that the Adviser determines have economic characteristics. The index is designed to measure the total return performance of inflation-linked bonds outside the United States with fixed-rate coupon payments that are linked to an inflation index. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.396407	2020-10-11 23:38:31.02423
WLFC	Willis Lease Finance Corporation	Willis Lease Finance Corp is a provider of aviation services. It is focused on providing operating leases of commercial aircraft engines and other aircraft-related equipment to air carriers, manufacturers and overhaul/repair facilities.	{Industrials,"Rental & Leasing Services","Consulting & Outsourcing"}	{GATX,GE,AYR,AER,CAI,FLY,ACY,AL}	Consulting & Outsourcing	Industrials	NASDAQ Global Market	\N	2020-10-11 23:12:13.409147	2020-10-11 23:38:31.023007
WMC	Western Asset Mortgage Capital Corporation	Western Asset Mortgage Capital Corp is a specialty financial REIT company managing diversified mortgage-market investment portfolio. The Corporation is externally managed by Western Asset Management Company and emphasizes on undervalued assets.	{"Real Estate","REIT - Diversified",REITs}	{NYMT,DX,ANH,IVR,ORC,TWO}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:13.421888	2020-10-11 23:38:31.028166
WOOD	iShares S&P Global Timber & Forestry Index Fund	The investment seeks to track the investment results of the S&P Global Timber & Forestry IndexTM.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The index is comprised of approximately 25 of the largest publicly-traded companies engaged in the ownership, management or upstream supply chain of forests and timberlands. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:13.438559	2020-10-11 23:38:31.024353
WASH	Washington Trust Bancorp Inc.	Washington Trust Bancorp Inc offers banking and financial services to individuals and businesses. It operates in two segments namely Commercial Banking and Wealth Management Services. Generating, a majority of its revenue from Commercial Banking Segment.	{"Financial Services","Banks - Regional - US",Banks}	{EBTC,UNB,BPFH,CNBKA,INDB,BRKL,WBS,NBN,PNBK}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:13.275073	2020-10-11 23:38:31.025871
WBC	Wabco Holdings Inc.	WABCO Holdings Inc and its subsidiaries engineers, develops, manufactures and sells integrated systems controlling braking, stability, suspension and transmission automation as well as air compression and processing for commercial vehicles.	{"Consumer Cyclical","Auto Parts",Autos}	{SYPR,PCAR,CVGI,SORL,NAV,CMI,SRI,MTOR}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:13.288737	2020-10-11 23:38:31.02423
WBS	Webster Financial Corporation	Webster Financial Corp is a bank and financial holding company. It delivers financial services to individuals, families and businesses throughout southern New England and eastern New York State.	{"Financial Services","Banks - Regional - US",Banks}	{CAC,BPFH,CNBKA,INDB,WASH,BRKL,FNLC,SAL,BWFG,PNBK}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:13.312933	2020-10-11 23:38:31.024353
WFT	Weatherford International plc (Ireland)	Weatherford International PLC together with its subsidiaries is a multinational oilfield service company. The company provides equipment and services used in the drilling, evaluation, completion, production and intervention of oil and natural gas wells.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{SLB,DOV,HAL,FTK,FET,FI,BHGE}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:13.362553	2020-10-11 23:38:31.025871
WILC	G. Willi-Food International  Ltd.	G. Willi-Food International Ltd develops, imports, exports, markets and distributes food products. It offers canned vegetables, packaged fruits, pickled vegetables, canned & frozen fish, pasta and edible oil, & among others.	{"Consumer Defensive","Food Distribution","Retail - Defensive"}	{UNFI,CORE,SYY,DIT,CHEF,SPTN}	Retail - Defensive	Consumer Defensive	NASDAQ Capital Market	\N	2020-10-11 23:12:13.384574	2020-10-11 23:38:31.028166
WK	Workiva Inc. Class A	Workiva Inc provides a cloud-based and mobile-enabled platform for enterprises to collaboratively collect, manage, report and analyze critical business data in real time.	{Technology,"Software - Application","Application Software"}	{SPSC,MOBL,HDP,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:13.401197	2020-10-11 23:38:31.024353
WLL	Whiting Petroleum Corporation	Whiting Petroleum Corp is a oil and gas company that acquires, exploits, develops and explores for crude oil, natural gas and natural gas liquids in the Rocky Mountain, Permian Basin, Mid-Continent, Gulf Coast and Michigan regions of the United States.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{GPOR,NFX,EOG,MRO,NOG,OAS,BCEI}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:13.415081	2020-10-11 23:38:31.031328
WMT	Walmart Inc.	Walmart Inc operates retail stores in various formats under various banners. Its operations comprise of three reportable business segments, Walmart U.S., Walmart International and Sam's Club in three categories retail, wholesale and others.	{"Consumer Defensive","Discount Stores","Retail - Defensive"}	{AMZN,COST,EBAY,DLTR,KSS,JCP,TGT,M,DG}	Retail - Defensive	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:13.429764	2020-10-11 23:38:31.02582
WPM	Wheaton Precious Metals Corp (Canada)	Wheaton Precious Metals Corp is pure precious metals streaming company. Its portfolio includes Hudbay's Constancia project and Vale's Salobo and Sudbury mines.	{"Basic Materials",Silver,"Metals & Mining"}	{PAAS,EXK,CDE,HL,AG,TAHO}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:13.451916	2020-10-11 23:38:31.02582
WRN	Western Copper and Gold Corporation	Western Copper & Gold Corp is an exploration and development company that is directly engaged in the development of the Casino mineral property located in Yukon, Canada. It holds gold, copper and molybdenum resources and reserves in its Casino Project.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{NAN}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:13.473512	2020-10-11 23:38:31.031328
WSR	Whitestone REIT	Whitestone REIT is a self-managed, fully integrated real estate investment trust that owns, manages and redevelops portfolio of neighborhood, community and lifestyle retail centers. Its property portfolio includes retail and office/flex properties.	{"Real Estate","REIT - Retail",REITs}	{BFS,O,ALX,UBA,OLP,EPR}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:13.490659	2020-10-11 23:38:31.021426
WTM	White Mountains Insurance Group Ltd.	White Mountains Insurance Group Ltd provides specialty property and casualty insurance and offers a range of insurance products in the United States. It operates its business through three segments, OneBeacon, HG Global/BAM and White Mountains Advisors.	{"Financial Services","Insurance - Property & Casualty"}	{Y,WRB,L,MKL,RLI,TRV,AIG}	Insurance - Property & Casualty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:13.50847	2020-10-11 23:38:31.028166
WBIA	WBI BullBear Rising Income 2000	The investment seeks long-term capital appreciation and the potential for current income, while also seeking to protect principal during unfavorable market conditions.\n The fund will seek to invest in the equity securities of small-capitalization and mid-capitalization domestic and foreign companies. These securities will be selected on the basis of the Sub-Advisor's proprietary selection process ("Selection Process"). Cash and cash equivalents are some of the investment opportunities evaluated by the Selection Process. It may invest up to 50% of its net assets in the securities of issuers in emerging markets.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.289318	2020-10-11 23:38:31.02582
WETF	WisdomTree Investments Inc.	WisdomTree Investments Inc is an exchange traded fund (ETF) and exchange traded product (ETP) sponsor and asset manager. It offers Equity ETFs, International Hedged Equity ETFs, Currency ETFs, and Fixed Income ETFs.	{"Financial Services","Asset Management"}	{TROW,VRTS,AB,APAM}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:13.348901	2020-10-11 23:38:31.021426
WGP	Western Gas Equity Partners LP Representing Limited Partner Interests	Western Gas Equity Partners LP is a US-based company engaged in gathering, processing, compressing, treating and transporting natural gas condensate.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{TRP,APC,EPD,WMB,EEP,KMI}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:13.36466	2020-10-11 23:38:31.028166
WIFI	Boingo Wireless Inc.	Boingo Wireless Inc is a leading global provider of wireless connectivity solutions for smartphones, tablets, laptops, wearables and other wireless-enabled consumer devices.	{"Communication Services","Telecom Services"}	{AAPL,CMCSA,VZ,T,GOOGL,TMUS}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:12:13.381954	2020-10-11 23:38:31.024353
WIT	Wipro Limited	Wipro Ltd is one of the world's largest IT services vendors. The company provides services such as business application, technology infrastructure, business process outsourcing, and consulting across over 50 countries.	{Technology,"Information Technology Services","Application Software"}	{CTSH,IBM,ACN,INFY,HPQ,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:13.397942	2020-10-11 23:38:31.021426
WLKP	Westlake Chemical Partners LP representing limited partner interests	Westlake Chemical Partners LP operates, acquires and develops facilities for the processing of natural gas liquids as well as other qualified assets.	{"Basic Materials",Chemicals}	{GPRE,CBM,WLK,AMTX}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:13.414627	2020-10-11 23:38:31.024353
WOW	WideOpenWest Inc.	WideOpenWest Inc is a cable operator and broadband service provider in the United States. It offers high-speed internet, data, voice, cloud and cable television services.	{"Communication Services","Pay TV"}	{S}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:13.442119	2020-10-11 23:38:31.023428
WRLD	World Acceptance Corporation	World Acceptance Corp is a small-loan consumer finance company. It provides short-term small loans, medium-term larger loans, related credit insurance products and ancillary products and services to individuals.	{"Financial Services","Credit Services"}	{RM,OMF}	Credit Services	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:13.469803	2020-10-11 23:38:31.02582
WSTL	Westell Technologies Inc.	Westell Technologies Inc, through its subsidiaries, designs, manufactures, and distributes telecommunications products to telephone companies and other telecommunications service providers.	{Technology,"Communication Equipment"}	{XLK}	Communication Equipment	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:13.494899	2020-10-11 23:38:31.024353
WTT	Wireless Telecom Group Inc.	Wireless Telecom Group Inc designs and manufactures radio frequency (RF) and microwave-based products for communications industries. It produces RF passive components and integrated subsystems as well as electronic test and measurement equipment.	{Technology,"Communication Equipment"}	{NATI,MRCY,A,MTSI,COMM,XLK}	Communication Equipment	Technology	NYSE American	\N	2020-10-11 23:12:13.516653	2020-10-11 23:38:31.023007
WWR	Westwater Resources Inc.	Westwater Resources Inc is an explorer and developer of mineral resources that are materials essential to clean energy production.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{URG,DNN,GMO,UEC,UUUU,LEU}	Metals & Mining	Basic Materials	NASDAQ Capital Market	\N	2020-10-11 23:12:13.536569	2020-10-11 23:38:31.02582
XBIT	XBiotech Inc.	XBiotech Inc is engaged in discovering and developing True Human (antibody technology) monoclonal antibodies for treating a variety of diseases.	{Healthcare,Biotechnology}	{KERX,RDUS,CALA,NVLN,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:13.553031	2020-10-11 23:38:31.02582
WFIG	WisdomTree Fundamental U.S. Corporate Bond Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree Fundamental U.S. Corporate Bond Index.\n Under normal circumstances, at least 80% of the fund's total assets will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is designed to capture the performance of selected issuers in the U.S. investment grade corporate bond market that are deemed to have favorable fundamental and income characteristics. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:13.362381	2020-10-11 23:38:31.023007
WHR	Whirlpool Corporation	Whirlpool Corp manufactures home appliances such as washing machine, refrigerators and freezers, and other domestic appliances under Whirlpool, KitchenAid, Maytag, Consul, Brastemp, Amana, Bauknecht, Jenn-Air, Indesit and other major brand names.	{"Consumer Cyclical","Home Furnishings & Fixtures","Manufacturing - Apparel & Furniture"}	{GE}	Manufacturing - Apparel & Furniture	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:13.379119	2020-10-11 23:38:31.025871
WIX	Wix.com Ltd.	Wix.com Ltd is a web development platform provider. Its tools allow users to perform various web-based activities such as selling goods, taking reservations, and scheduling and confirming appointments.	{Technology,"Internet Content & Information","Online Media"}	{AMZN,VRSN,SYNC,TCX,GOOGL,XLK}	Online Media	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:13.400099	2020-10-11 23:38:31.028166
WMB	Williams Companies Inc. (The)	Williams Companies Inc is a large pipeline company. It owns stakes in and operates 15,000 miles of interstate gas pipelines and 1,000 miles of natural gas liquids pipelines. The company also produces olefins for petrochemical feedstocks.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{ENB,TRP,EPD,SWN,ET,EQT,SEP}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:13.424771	2020-10-11 23:38:31.02423
WP	Worldpay Inc. Class A	Worldpay Inc is a payment processing company that provides integrated technology platform. It offers suite of payment processing services through a single provider, including e-commerce and mobile devices.	{Industrials,"Business Services"}	{}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:13.442673	2020-10-11 23:38:31.028166
WPRT	Westport Fuel Systems Inc	Westport Fuel Systems Inc is a provider of high-performance, low-emission engine and fuel system technologies utilizing gaseous fuels.	{"Consumer Cyclical","Auto Parts",Autos}	{MGA,PCAR,BWA,CAT,CMI,TEN,F,TTM,GM}	Autos	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:13.455079	2020-10-11 23:38:31.025871
WRLSU	Pensare Acquisition Corp. Unit	Pensare Acquisition Corp is a blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:13.471521	2020-10-11 23:38:31.023428
WST	West Pharmaceutical Services Inc.	West Pharmaceutical Services Inc develops, manufactures and sells packaging components and systems for injectable drug delivery, including stoppers and seals for vials, closures and other components used in syringe, and prefillable syringe components.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{ICUI,BAX,BDX,JNJ,SYK,TFX,ATRS,MDT,XLV}	Medical Instruments & Equipment	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:13.491104	2020-10-11 23:38:31.031328
WTMF	WisdomTree Managed Futures Strategy Fund	The investment seeks to provide investors with positive total returns in rising or falling markets.\n The fund normally invests at least 80% of its net assets, plus the amount of any borrowings for investment purposes, in "managed futures". It is an actively managed exchange traded fund ("ETF") that seeks to achieve positive total returns in rising or falling markets that are not directly correlated to broad market equity or fixed income returns. The fund is managed using a quantitative, rules-based strategy designed to provide returns that correspond to the performance of the WisdomTree Managed Futures Index. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.510691	2020-10-11 23:38:31.021426
WU	Western Union Company (The)	The Western Union Co is a provider of money movement and payment services. The company provides flexible and convenient options for making one-time or recurring payments. Its segments are Consumer-to-Consumer, Consumer-to-Business and Business Solutions.	{"Financial Services","Credit Services"}	{GPN,FIS,MA,WEX,MGI}	Credit Services	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:13.524749	2020-10-11 23:38:31.02423
WEYS	Weyco Group Inc.	Weyco Group Inc is an US based company, engaged in manufacture, design distribution of quality and innovative footwear for men, women and children. It offers mid-priced leather dress shoes and casual footwear composed of man-made materials or leather.	{"Consumer Cyclical","Footwear & Accessories","Manufacturing - Apparel & Furniture"}	{SHOO,CROX,SKX,WWW,DECK}	Manufacturing - Apparel & Furniture	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:13.3527	2020-10-11 23:38:31.028166
WHLM	Wilhelmina International Inc.	Wilhelmina International Inc is engaged in fashion model management and complementary business activities. The company provides traditional, full-service fashion model and talent management services.	{Industrials,"Business Services"}	{HEES,GPX,G,TRXC}	Business Services	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:13.368697	2020-10-11 23:38:31.028166
WIN	Windstream Holdings Inc.	Windstream Holdings Inc provides telecom services, including telephone and internet access to individuals and business customers. It mainly operates in Southeast United States and Southern Midwest.	{"Communication Services","Telecom Services"}	{}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:12:13.383419	2020-10-11 23:38:31.02582
WKHS	Workhorse Group Inc.	Workhorse Group Inc is a technology company. It designs, develops, manufactures, and sells high-performance, medium-duty trucks with powertrain components under the Workhorse chassis brand.	{"Consumer Cyclical","Auto Parts",Autos}	{PCAR,TSLA,NAV,GM}	Autos	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:13.401146	2020-10-11 23:38:31.02582
WM	Waste Management Inc.	Waste Management Inc acts as a provider of waste management environmental services. The services provided by the company include collection, landfill, transfer, recycling and resource recovery and other services.	{Industrials,"Waste Management"}	{SRCL,CWST,ECOL,CLH,RSG,PESI,WCN}	Waste Management	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:13.419302	2020-10-11 23:38:31.025871
WNC	Wabash National Corporation	Wabash National Corp is a diversified industrial manufacturer and producer of semi-trailers and liquid transportation systems. It designs, manufactures and markets various types of trailers, dry and refrigerated truck bodies, among others.	{Industrials,"Truck Manufacturing"}	{PCAR,SPAR,NAV,HY}	Truck Manufacturing	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:13.437288	2020-10-11 23:38:31.023007
WPX	WPX Energy Inc.	WPX Energy Inc, based in the United States, is an oil & gas producer. It also offers natural gas on a nominal basis.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{APC,QEP,PDCE,BCEI}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:13.455598	2020-10-11 23:38:31.028166
WSFS	WSFS Financial Corporation	WSFS Financial Corp is a savings and loan holding company, which provides residential and commercial real estate, commercial and consumer lending services, as well as retail deposit and cash management services.	{"Financial Services","Banks - Regional - US",Banks}	{SVBI,NWBI,PBIP,BNCL}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:13.486176	2020-10-11 23:38:31.028166
WTFCW	Wintrust Financial Corporation Warrants	Wintrust Financial Corp is engaged in providing community banking, specialty finance, and wealth management. It focuses on providing community-oriented, personal, and commercial banking and origination and purchase of residential mortgages.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:13.502595	2020-10-11 23:38:31.028166
WTS	Watts Water Technologies Inc. Class A	Watts Water Technologies Inc supplies industrial goods. Its products are used to manage the flow of fluids in residential and commercial buildings.	{Industrials,"Diversified Industrials","Industrial Products"}	{CW,DOV,CIR,MWA,GHM,RXN}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:13.516263	2020-10-11 23:38:31.02582
WVVI	Willamette Valley Vineyards Inc.	Willamette Valley Vineyards Inc produces and sells premium, super premium and ultra-premium varietals. The Company sells its wines under the labels: Willamette Valley Vineyards, Griffin Creek and Tualatin Estates.	{"Consumer Defensive","Beverages - Wineries & Distilleries","Beverages - Alcoholic"}	{BF.B,STZ,ROX,TRC}	Beverages - Alcoholic	Consumer Defensive	NASDAQ Capital Market	\N	2020-10-11 23:12:13.530633	2020-10-11 23:38:31.021426
WYY	WidePoint Corporation	WidePoint Corp is a provider of managed mobility services, telecom lifecycle management, and cybersecurity solutions.	{Technology,"Information Technology Services","Application Software"}	{EGOV,LMT,IBM,NOC,COL,UIS,MSI,HPQ,XLK}	Application Software	Technology	NYSE American	\N	2020-10-11 23:12:13.546039	2020-10-11 23:38:31.031328
XEC	Cimarex Energy Co	Cimarex Energy Co is involved in the oil and gas industry. Its primary activities include oil and gas exploration and production.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{CHK,NFX,DNR,APA,SM,CRK}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:13.556577	2020-10-11 23:38:31.031328
WF	Woori Bank American Depositary Shares (Each representing 3 shares of)	Woori Bank operates as a commercial bank. The company provides corporate banking, consumer banking, investment banking and capital markets services and other banking activities.	{"Financial Services","Banks - Regional - Asia",Banks}	{IBN,HDB,SHG,KB}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:13.356142	2020-10-11 23:38:31.023428
WHLR	Wheeler Real Estate Investment Trust Inc.	Wheeler Real Estate Investment Trust Inc is a self-managed commercial real estate investment company.. It primarily invests in shopping centers, neighborhood centers, grocery-anchored centers, community centers and free-standing retail properties.	{"Real Estate","REIT - Retail",REITs}	{CBL,NLY,SPG}	REITs	Real Estate	NASDAQ Capital Market	\N	2020-10-11 23:12:13.369952	2020-10-11 23:38:31.02423
WIL	Barclays PLC Women in Leadership ETN	The investment seeks a return linked to the performance of the Barclays Women in Leadership Total Return USD Index (the “index”).\n The index is designed to provide investors with exposure to U.S.-based companies that satisfy one or both of the gender diversity criteria of having a female chief executive officer or having at least 25% female members on the board of directors and that satisfy certain market capitalization, liquidity and other selection criteria and concentration limits.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.382117	2020-10-11 23:38:31.031328
WINS	Wins Finance Holdings Inc.	Wins Finance Holdings Inc is a diversified investment and asset management company. It is in the business of leasing, financial advisory, financial guarantees, and partnerships.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:13.396271	2020-10-11 23:38:31.025871
WLH	Lyon William Homes (Class A)	William Lyon Homes, along with its subsidiaries, designs, constructs & sells single-family detached & attached homes in California, Arizona, Nevada, Colorado, Washington and Oregon.	{"Consumer Cyclical","Residential Construction","Homebuilding & Construction"}	{MDC,TMHC,CCS}	Homebuilding & Construction	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:13.409331	2020-10-11 23:38:31.023428
WMGI	Wright Medical Group N.V.	Wright Medical Group NV is a medical device company that manufactures & sells extremities and biologics products in USA & internationally. It offers joint implants & bone fusion & fixation devices for shoulder, elbow, wrist, and hand.	{Healthcare,"Medical Devices"}	{IART,RTIX,JNJ,SYK,EW,SNN,ZBH,ATEC,XLV}	Medical Devices	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:13.421727	2020-10-11 23:38:31.023428
WNEB	Western New England Bancorp Inc.	Western New England Bancorp Inc through its subsidiaries provides banking products and services offering deposit products and makes mortgage loans for the construction, purchase, and refinancing of residential and commercial real estate.	{"Financial Services","Banks - Regional - US",Banks}	{HIFS,BRKL,PBCT,NBN,BLMT,WEBK,BHLB,UBNK}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:13.437101	2020-10-11 23:38:31.025871
WPP	WPP plc American Depositary Shares	WPP PLC is an advertising agency in the United Kingdom. Under its coverage of operations are activities like media investment management, public relations and public affairs, branding and identity and programmatic media.	{"Consumer Cyclical","Advertising Agencies","Advertising & Marketing Services"}	{IPG,OMC}	Advertising & Marketing Services	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:13.450103	2020-10-11 23:38:31.023428
WRI	Weingarten Realty Investors	Weingarten Realty Investors is a real estate investment trust. The company is engaged in providing leasing space to tenants in the shopping centers that it owns or leases. It also provides property management services.	{"Real Estate","REIT - Retail",REITs}	{ADC,KIM,CDR}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:13.463175	2020-10-11 23:38:31.021426
WSBF	Waterstone Financial Inc.	Waterstone Financial Inc is a one-bank holding company with two segments: community banking & mortgage banking. The company provides consumer and business banking products and services to customers, and also provides residential mortgage loans.	{"Financial Services","Savings & Cooperative Banks",Banks}	{ESSA,HMST,FBC}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:13.474236	2020-10-11 23:38:31.02423
WTBA	West Bancorporation	West Bancorp Inc is a holding company for West Bank. It provides commercial banking services, including checking accounts, savings accounts, money market accounts, and time certificates of deposit.	{"Financial Services","Banks - Regional - US",Banks}	{UMBF,ATLO,GFED,CBSH,GSBC,LARK,QCRH,EFSC,HWBK,MOFG}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:13.49521	2020-10-11 23:38:31.02423
WTR	Aqua America Inc.	Aqua America Inc is the holding company for regulated utilities providing water or wastewater services concentrated in Pennsylvania, Ohio, Texas, Illinois, North Carolina, New Jersey, Indiana, and Virginia.	{Utilities,"Utilities - Regulated Water","Utilities - Regulated"}	{YORW,ARTNA,CTWS,MSEX,CWT,AWR,AWK,SJW,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:13.511578	2020-10-11 23:38:31.024353
WVVIP	Willamette Valley Vineyards Inc. Series A Redeemable Preferred Stock	Willamette Valley Vineyards Inc produces and sells premium, super premium and ultra-premium varietals. The Company sells its wines under the labels: Willamette Valley Vineyards, Griffin Creek and Tualatin Estates.	{"Consumer Defensive","Beverages - Wineries & Distilleries","Beverages - Alcoholic"}	{}	Beverages - Alcoholic	Consumer Defensive	NASDAQ Capital Market	\N	2020-10-11 23:12:13.53191	2020-10-11 23:38:31.024353
WGO	Winnebago Industries Inc.	Winnebago Industries Inc is engaged in the manufacturing recreational vehicles used mainly in leisure travel and outdoor recreation activities. Its products include motorhomes, travel trailers, fifth wheel products and toy haulers etc.	{"Consumer Cyclical","Recreational Vehicles",Autos}	{PATK,SPAR,NAV,BC,PII,SKY,THO,LCII}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:13.365572	2020-10-11 23:38:31.024353
WIA	Western Asset Inflation-Linked Income Fund	Western Asset Claymore Inf-lkd Sec & Inc is a closed end fund. Its objective is to provide a high level of current income and long-term capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:13.379407	2020-10-11 23:38:31.021426
WIRE	Encore Wire Corporation	Encore Wire Corp is engaged in manufacturing electrical building wire and cable. It supplies building wire for interior electrical wiring in commercial and industrial buildings, homes, apartments, and manufactured housing.	{Technology,"Electronic Components","Computer Hardware"}	{PLPC,OCC,APH,BDC,APWC,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:13.39717	2020-10-11 23:38:31.023428
WLK	Westlake Chemical Corporation	Westlake Chemical Corp is a manufacturer and supplier of basic chemicals, vinyls, polymers and fabricated products that enhance the daily lives of people around the planet. It operates in two principal business segments, Olefins and Vinyls.	{"Basic Materials","Specialty Chemicals",Chemicals}	{EMN,ALB,XOM,HON,OLN,POL,HUN,CE,DWDP}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:13.412193	2020-10-11 23:38:31.021426
WMS	Advanced Drainage Systems Inc.	Advanced Drainage Systems Inc designs, manufactures and markets thermoplastic corrugated pipe and related water management products in North and South America and Europe. Group operates through two segments: Domestic and International.	{"Consumer Cyclical","Rubber & Plastics",Autos}	{RAVN,DOV,ROG,PRLB}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:13.429904	2020-10-11 23:38:31.021426
WPC	W.P. Carey Inc. REIT	W.P. Carey Inc operates as a self-managed diversified real estate investment trust. The company owns and manages commercial real estate, leased to companies on a long-term basis.	{"Real Estate","REIT - Diversified",REITs}	{SLG,HPP,CBRE,PGRE}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:13.445648	2020-10-11 23:38:31.031328
WRLSR	Pensare Acquisition Corp. Right		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:13.473344	2020-10-11 23:38:31.025871
WSTG	Wayside Technology Group Inc.	Wayside Technology Group Inc is an information technology channel company. It distributes software developed by others through resellers indirectly to customers.	{Technology,"Computer Distribution","Computer Hardware"}	{INTC,MSFT,NSIT,PLUS,DWCH,SNX,CDW,HPQ,TIVO,XLK}	Computer Hardware	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:13.49432	2020-10-11 23:38:31.028166
WTIU	UBS ETRACS - ProShares Daily 3x Long Crude ETN	The 3X Long Securities provide a daily long leveraged exposure to the performance of the Bloomberg WTI Crude Oil Subindex ERSM. The index is designed to measure the return from a rolling long position in WTI crude oil futures contracts that trade on major U.S. exchanges.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.507729	2020-10-11 23:38:31.031328
XBIO	Xenetic Biosciences Inc.	Xenetic Biosciences Inc is a clinical-stage biopharmaceutical company focused on the discovery, research and development of next-generation biologic drugs and novel orphan oncology therapeutics.	{Healthcare,Biotechnology}	{AAWW,BRS,ERA,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:13.551775	2020-10-11 23:38:31.028166
XFLT	XAI Octagon Floating Rate & Alternative Income Term Trust of Beneficial Interest	XAI Octagon FR & Alt Income Term Trust is a diversified, closed-end management investment company. Its investment objective is to seek attractive total return with an emphasis on income generation across multiple stages of the credit cycle.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:13.570131	2020-10-11 23:38:31.028166
XINA	SPDR MSCI China A Shares IMI	The investment seeks to track the performance of the MSCI China A International IMI Index.\n Under normal circumstances, the fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index and in depositary receipts based on securities comprising the index. The index is a free-float adjusted, market capitalization weighted index designed to capture large, mid and small cap representation across Chinese securities with A Share listings on the Shanghai or Shenzhen Stock Exchanges. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.582592	2020-10-11 23:38:31.023007
XLB	Materials Select Sector SPDR	The investment seeks to provide investment results that, before expenses, correspond generally to the price and yield performance of publicly traded equity securities of companies in the Materials Select Sector Index.\n In seeking to track the performance of the index, the fund employs a replication strategy. It generally invests substantially all, but at least 95%, of its total assets in the securities comprising the index. The index includes securities of companies from the following industries: chemicals; metals and mining; paper and forest products; containers and packaging; and construction materials. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.596925	2020-10-11 23:38:31.031328
WHLRP	Wheeler Real Estate Investment Trust Inc. Preferred Stock	Wheeler Real Estate Investment Trust Inc is a self-managed commercial real estate investment company.. It primarily invests in shopping centers, neighborhood centers, grocery-anchored centers, community centers and free-standing retail properties.	{"Real Estate","REIT - Retail",REITs}	{}	REITs	Real Estate	NASDAQ Capital Market	\N	2020-10-11 23:12:13.379291	2020-10-11 23:38:31.023428
WIW	Western Asset Inflation-Linked Opportunities & Income Fund	Western Asset Claymore Inf-lkd Opp& Inc is a diversified, closed-end management investment company. Its primary investment objective is to provide current income for its shareholders.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:13.397276	2020-10-11 23:38:31.031328
WLTW	Willis Towers Watson Public Limited Company	Willis Towers Watson PLC is a global advisory, insurance brokerage, and solutions company worldwide.	{"Financial Services","Insurance Brokers","Brokers & Exchanges"}	{BRO,AJG,MMC,AON}	Brokers & Exchanges	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:13.418602	2020-10-11 23:38:31.02582
WMW	DEUTSCHE BANK AKTIENGESELLSCHAFT ELEMENTS Linked to the Morningstar Wide Moat Focus Total Return Index due October 24 2022	The investment seeks to replicate, net of expense, the Morningstar Wide Moat Focus Total Return Index.\n The index contains companies that have sustainable competitive advantages and that trade at discounts or Morningstar's determination of their fair value. It contains approximately 20 companies.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.435264	2020-10-11 23:38:31.031328
WPG	Washington Prime Group Inc.	Washington Prime Group Inc operates in the real estate sector. As a self-managed real estate investment trust, it invests in, manages, and develops retail properties including community shopping centers and malls.	{"Real Estate","REIT - Retail",REITs}	{FR,PEI,SHO}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:13.448635	2020-10-11 23:38:31.021426
WRE	Washington Real Estate Investment Trust	Washington REIT is a real estate investment trust. The Company is engaged in the ownership and operation of income-producing real properties in the greater Washington metro region.	{"Real Estate","REIT - Office",REITs}	{AAT,VER,WELL}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:13.462578	2020-10-11 23:38:31.02423
WSC	WillScot Corporation	WillScot Corp operates in the specialty rental services market providing modular space and portable storage solutions across North America for construction, education, healthcare, government, retail, commercial, transportation, security & energy sectors.	{Industrials,"Rental & Leasing Services","Consulting & Outsourcing"}	{WSC}	Consulting & Outsourcing	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:13.475244	2020-10-11 23:38:31.02582
WTFC	Wintrust Financial Corporation	Wintrust Financial Corp is engaged in providing community banking, specialty finance, and wealth management. It focuses on providing community-oriented, personal, and commercial banking and origination and purchase of residential mortgages.	{"Financial Services","Banks - Regional - US",Banks}	{OSBC,FMBI,MBFI,TCF,BUSE}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:13.500565	2020-10-11 23:38:31.023007
WVFC	WVS Financial Corp.	WVS Financial Corp operates as the holding company of West View Savings Bank. It provides financial products and services to residential and commercial customers.	{"Financial Services","Savings & Cooperative Banks",Banks}	{ESSA,PBIP,BNCL}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:13.528604	2020-10-11 23:38:31.031328
XELA	Exela Technologies Inc.	Exela Technologies Inc provides transaction processing solutions and enterprise information management. Its products include Business process management, enterprise information management, enrollment and applications and mobile applications.	{Technology,"Software - Infrastructure","Application Software"}	{XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:13.564139	2020-10-11 23:38:31.021426
XHE	SPDR S&P Health Care Equipment	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of an index derived from the health care equipment and supplies segment of a U.S. total market composite index.\n In seeking to track the performance of the S&P Health Care Equipment Select Industry Index (the "index"), the fund employs a sampling strategy. It generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index represents the health care equipment segment of the S&P Total Market Index ("S&P TMI"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.576786	2020-10-11 23:38:31.02582
XITK	SPDR FactSet Innovative Technology	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the FactSet Innovative Technology Index.\n The index is designed to represent the performance of U.S.-listed stock and American Depository Receipts of Technology companies and Technology-related companies within the most innovative segments of the Technology sector and Electronic Media sub-sector of the Media sector, as defined by FactSet Research Systems, Inc. The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.587704	2020-10-11 23:38:31.024353
XLI	SPDR Select Sector Fund - Industrial	The investment seeks to provide investment results that, before expenses, correspond generally to the price and yield performance of publicly traded equity securities of companies in the Industrial Select Sector Index.\n Under normal market conditions, the fund generally invests substantially all, but at least 95%, of its total assets in the securities comprising the index. The index includes securities of companies from the following industries: aerospace and defense; industrial conglomerates; marine; transportation infrastructure; machinery; road and rail; air freight and logistics; commercial services and supplies; etc. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.600238	2020-10-11 23:38:31.025871
WMK	Weis Markets Inc.	Weis Markets Inc operates grocery stores. Through its chain of retail stores, it sells groceries, dairy products, frozen foods, meats, seafood, fresh produce, floral and deli products.	{"Consumer Defensive","Grocery Stores","Retail - Defensive"}	{IMKTA,VLGEA,KR,SPTN}	Retail - Defensive	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:13.426482	2020-10-11 23:38:31.024353
WOR	Worthington Industries Inc.	Worthington Industries Inc is metals manufacturing company. Its products are pressure cylinders for liquefied petroleum gas, compressed natural gas, oxygen, cylinders, helium-filled balloon kits, steel and fiberglass tanks and processing equipment.	{"Basic Materials",Steel}	{SCHN,STLD,AKS,RS,CRS,CMC,HSC,NUE,X,AA}	Steel	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:13.440438	2020-10-11 23:38:31.02423
WRD	WildHorse Resource Development Corporation	WildHorse Resource Development Corp is an independent oil and natural gas company. The company is focused on the acquisition, exploitation, development and production of oil, natural gas and Natural as liquids resources.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:13.457588	2020-10-11 23:38:31.031328
WSBC	WesBanco Inc.	Wesbanco Inc is a financial services provider. The company offers a full range of financial services, including retail banking, corporate banking, personal and corporate trust services, brokerage services, mortgage banking, and insurance.	{"Financial Services","Banks - Regional - US",Banks}	{PEBO,FFBC,HBAN,KEY,CMA,CFR,FNB,PRK,PNC,WFC}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:13.474188	2020-10-11 23:38:31.024353
WTFCM	Wintrust Financial Corporation Fixed-to-Floating Rate Non-Cumulative Perpetual Preferred Stock Series D	Wintrust Financial Corp is engaged in providing community banking, specialty finance, and wealth management. It focuses on providing community-oriented, personal, and commercial banking and origination and purchase of residential mortgages.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:13.50358	2020-10-11 23:38:31.02582
WVE	Wave Life Sciences Ltd.	WAVE Life Sciences Ltd is a preclinical therapeutics developer in Singapore. Its pipeline of drugs is intended to cure rare genetic disorders such as Huntington, duchenne muscular dystrophy (DMD) and inflammatory bowel disease.	{Healthcare,Biotechnology}	{EXC,NEE,WVE,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:13.527139	2020-10-11 23:38:31.028166
WY	Weyerhaeuser Company	Weyerhaeuser Co is a real estate investment trust and is engaged in selling timber. Its objective is to maximize the long-term value of timberlands that it owns. It derives its revenue from investments in timberland, including the sale of standing timber.	{"Basic Materials","Lumber & Wood Production","Forest Products"}	{VTR,LHO,TWO}	Forest Products	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:13.54426	2020-10-11 23:38:31.025871
XEL	Xcel Energy Inc.	Xcel Energy Inc manages utilities to serve over 3.5 million electric customers and 2.0 million natural gas customers across United States. The company through its subsidiary firms operates the electric and gas business in US and Mexico.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{DTE,PCG,LNT,CMS,IDA,AVA,PPL,PNM,DUK,WEC,XLU}	Utilities - Regulated	Utilities	Nasdaq Global Select	\N	2020-10-11 23:12:13.558545	2020-10-11 23:38:31.025871
XHS	SPDR S&P Health Care Services	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of an index derived from the health care providers and services segment of a U.S. total market composite index.\n In seeking to track the performance of the S&P Health Care Services Select Industry Index (the "index"), the fund employs a sampling strategy. It generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index represents the health care services segment of the S&P Total Market Index ("S&P TMI"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.583705	2020-10-11 23:38:31.023428
XLNX	Xilinx Inc.	Xilinx Inc is a semiconductor manufacturing company. It engages in designing programmable logic such as chips for multiple applications that include communications, data processing, industrial, and automobile markets.	{Technology,Semiconductors}	{CY,LSCC,QCOM,QUIK,ON,IBM,TXN,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:13.605437	2020-10-11 23:38:31.028166
XLY	SPDR Select Sector Fund - Consumer Discretionary	The investment seeks investment results that, before expenses, correspond to the price and yield performance of publicly traded equity securities of companies in the Consumer Discretionary Select Sector Index.\n The fund employs a replication strategy. It generally invests substantially all, but at least 95%, of its total assets in the securities comprising the index. The index includes securities of companies from the following industries: media; retail; hotels, restaurants and leisure; textiles, apparel and luxury goods; household durables; automobiles; auto components; distributors; leisure products; and diversified consumer services. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.620224	2020-10-11 23:38:31.02582
XMX	WisdomTree Global ex-Mexico Equity Fund	The investment seeks to track the price and yield performance of the WisdomTree Global ex-Mexico Equity Index.\n Under normal circumstances, at least 80% of the fund's total assets will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a float-adjusted market capitalization weighted index that is comprised of the equity securities of companies in developed countries and emerging markets throughout the world, excluding Mexico, that meet certain size and volume requirements. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.629289	2020-10-11 23:38:31.024353
WSCWW	WillScot Corporation Warrant	Silver Eagle Acquisition Corp is a blank check company.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	OTC Pink	\N	2020-10-11 23:12:13.484633	2020-10-11 23:38:31.023007
WTID	UBS ETRACS - ProShares Daily 3x Inverse Crude ETN	The 3X Inverse Securities provide a three times leveraged inverse exposure to changes in the level of the Bloomberg WTI Crude Oil Subindex ERSM. The index is designed to measure the return from a rolling long position in WTI crude oil futures contracts that trade on major U.S. exchanges.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.510671	2020-10-11 23:38:31.023428
WUBA	58.com Inc. American Depositary Shares each representing 2 Class A	58.com Inc operates online marketplace serving local merchants and consumers in China. Its online marketplace enables local merchants and consumers to connect, share information and conduct business.	{Technology,"Internet Content & Information","Online Media"}	{SCOR,YELP,TWTR,GOOGL,XLK}	Online Media	Technology	New York Stock Exchange	\N	2020-10-11 23:12:13.52554	2020-10-11 23:38:31.025871
WWW	Wolverine World Wide Inc.	Wolverine World Wide Inc is a designer, manufacturer and marketer of footwear, apparel and accessories. The company sells casual footwear and apparel through its lifestyle group, and it sells performance and athletic apparel through its performance group.	{"Consumer Cyclical","Footwear & Accessories","Manufacturing - Apparel & Furniture"}	{SHOO,CROX,RCKY,SKX,DECK,CAL}	Manufacturing - Apparel & Furniture	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:13.539614	2020-10-11 23:38:31.02423
XCEM	Columbia EM Core ex-China	The investment seeks investment results that correspond to the price and yield performance of the Beta Thematic Emerging Markets ex-China Index.\n The fund will invest at least 80% of its net assets in the companies included in the index and the advisor generally expects to be substantially invested at such times, with at least 95% of its net assets invested in these securities. The index is designed to provide broad, core emerging markets equity exposure by measuring the stock performance of up to 700 emerging markets companies, excluding companies domiciled or exchange-listed in China or domiciled in Hong Kong. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.553346	2020-10-11 23:38:31.02423
XLE	SPDR Select Sector Fund - Energy Select Sector	The investment seeks to provide investment results that, before expenses, correspond generally to the price and yield performance of publicly traded equity securities of companies in the Energy Select Sector Index.\n In seeking to track the performance of the index, the fund employs a replication strategy. It generally invests substantially all, but at least 95%, of its total assets in the securities comprising the index. The index includes securities of companies from the following industries: oil, gas and consumable fuels; and energy equipment and services. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.597826	2020-10-11 23:38:31.021426
XLV	SPDR Select Sector Fund - Health Care	The investment seeks investment results that, before expenses, correspond generally to the price and yield performance of publicly traded equity securities of companies in the Health Care Select Sector Index.\n In seeking to track the performance of the index, the fund employs a replication strategy. It generally invests substantially all, but at least 95%, of its total assets in the securities comprising the index. The index includes companies from the following industries: pharmaceuticals; health care equipment & supplies; health care providers & services; biotechnology; life sciences tools & services; and health care technology. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.613311	2020-10-11 23:38:31.023007
XOP	SPDR S&P Oil & Gas Explor & Product	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of an index derived from the oil and gas exploration and production segment of a U.S. total market composite index.\n In seeking to track the performance of the S&P Oil & Gas Exploration & Production Select Industry Index, the fund employs a sampling strategy. It generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index represents the oil and gas exploration and production segment of the S&P Total Market Index ("S&P TMI"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.640332	2020-10-11 23:38:31.028166
XONE	The ExOne Company	The ExOne Co is a global provider of 3D printing machines and 3D printed and other products, materials and services to industrial customers. The company produces a component as well as makes a tool to produce a component.	{Industrials,"Diversified Industrials","Industrial Products"}	{DDD,ITT}	Industrial Products	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:13.638523	2020-10-11 23:38:31.025871
XRX	Xerox Corporation	Xerox Corp provides printing equipment and supplies. The company is engaged in the designing, development, and sale of document management systems and solutions. It generates its revenue from the sale of equipment.	{Technology,"Information Technology Services","Application Software"}	{CAJ,IBM,PBI,ACN,HPQ,DXC,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:13.660829	2020-10-11 23:38:31.025871
WRLSW	Pensare Acquisition Corp. Warrant		{}	{}			NASDAQ Capital Market	\N	2020-10-11 23:12:13.471612	2020-10-11 23:38:31.028166
WSO	Watsco Inc.	Watsco Inc distributes air conditioning, heating and refrigeration equipment and related parts and supplies in the HVAC/R distribution industry in North America. Watsco has operations in the United States, Canada, Mexico and Puerto Rico.	{Technology,"Electronics Distribution","Computer Hardware"}	{HDSN,BECN,POOL,WCC,LII,GWW,UTX,HDS,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:13.488121	2020-10-11 23:38:31.023428
WTI	W&T Offshore Inc.	W&T Offshore Inc is engaged in the oil and gas sector. The company is engaged in exploration, development, and acquisition of oil and natural gas properties in the Gulf of Mexico.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{CPE,MCF}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:13.503837	2020-10-11 23:38:31.025871
WTTR	Select Energy Services Inc. Class A	Select Energy Services Inc is a provider of water solutions. Its solutions include Water Solutions, Well Testing, Fluid Handling, and Disposal Solutions.	{Industrials,"Pollution & Treatment Controls","Industrial Products"}	{PUMP}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:13.524195	2020-10-11 23:38:31.023428
X	United States Steel Corporation	United States Steel Corp is an integrated steel producer. Its products include flat-rolled and tubular products with production operations in North America and Europe. Its products cater to automotive, construction and energy companies.	{"Basic Materials",Steel}	{STLD,AKS,WOR,PKX}	Steel	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:13.548892	2020-10-11 23:38:31.024353
XELB	Xcel Brands Inc	Xcel Brands Inc is a media and brand management company engaged in the design, production, licensing, marketing, and direct to consumer sales of branded apparel, footwear, accessories, jewelry, home goods, and other consumer products.	{"Consumer Cyclical","Apparel Manufacturing","Manufacturing - Apparel & Furniture"}	{CHKR}	Manufacturing - Apparel & Furniture	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:13.567126	2020-10-11 23:38:31.023007
XKII	SPDR Kensho Intelligent Structures	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the Kensho Intelligent Infrastructure Index.\n Under normal market conditions, the fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index is comprised of U.S.-listed equity securities (including depositary receipts) of companies domiciled across developed and emerging markets worldwide which are included in the Intelligent Infrastructure sector as determined by a classification standard produced by the index provider. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.593159	2020-10-11 23:38:31.028166
XLRE	Real Estate Select Sector SPDR Fund (The)	The investment seeks to provide investment results that, before expenses, correspond generally to the price and yield performance of publicly traded equity securities of companies in the Real Estate Select Sector Index (the "index").\n Under normal market conditions, the fund generally invests substantially all, but at least 95%, of its total assets in the securities comprising the index. The index includes securities of companies from the following industries: real estate management and development and REITs, excluding mortgage REITs. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.61015	2020-10-11 23:38:31.031328
XME	SPDR S&P Metals & Mining	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of an index derived from the metals and mining segment of a U.S. total market composite index.\n In seeking to track the performance of the S&P Metals & Mining Select Industry Index (the "index"), the fund employs a sampling strategy. It generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index represents the metals and mining segment of the S&P Total Market Index ("S&P TMI"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.620544	2020-10-11 23:38:31.023428
XNCR	Xencor Inc.	Xencor Inc is engaged in the field of biotechnology. The United States-based firm develops drugs to combat severe diseases such as allergies and cancers.	{Healthcare,Biotechnology}	{AMGN,ALXN,SGEN,JNJ,MRK,ABBV,MGNX,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:13.634409	2020-10-11 23:38:31.031328
XOMA	XOMA Corporation	XOMA Corp is a biotechnology company. It discovers and develops human and humanized monoclonal antibody-based therapeutics.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:13.637546	2020-10-11 23:38:31.02582
XPL	Solitario Zinc Corp.	Solitario Zinc Corp is a leading zinc focused exploration company engaging in the acquisition, exploration, and development of zinc properties in safe jurisdictions in North and South America.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{RGLD,AUMN,HL,SCCO,TAHO}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:13.653262	2020-10-11 23:38:31.028166
XRT	SPDR S&P Retail	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of an index derived from the retail segment of a U.S. total market composite index.\n In seeking to track the performance of the S&P Retail Select Industry Index (the "index"), the fund employs a sampling strategy. It generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index represents the retail segment of the S&P Total Market Index ("S&P TMI"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.661086	2020-10-11 23:38:31.028166
CSD	Invesco S&P Spin-Off	The investment seeks to track the investment results (before the fund's fees and expenses) of the S&P U.S. Spin-Off Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities that comprise the underlying index. The index is designed to measure the performance of U.S. companies that have been spun off from larger corporations within the past four years. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.291907	2020-10-11 23:38:31.024353
XHB	SPDR Series Trust Homebuilders	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of an index derived from the homebuilding segment of a U.S. total market composite index.\n In seeking to track the performance of the S&P Homebuilders Select Industry Index (the "index"), the fund employs a sampling strategy. It generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index represents the homebuilders segment of the S&P Total Market Index ("S&P TMI"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.575529	2020-10-11 23:38:31.021426
XKFS	SPDR Kensho Future Security	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the Kensho Future Security Index.\n Under normal market conditions, the fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index is comprised of U.S.-listed equity securities (including depositary receipts) of companies domiciled across developed and emerging markets worldwide which are included in the Future Security sector as determined by a classification standard produced by the index provider. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.591321	2020-10-11 23:38:31.028166
XLP	SPDR Select Sector Fund - Consumer Staples	The investment seeks to provide investment results that, before expenses, correspond generally to the price and yield performance of publicly traded equity securities of companies in the Consumer Staples Select Sector Index.\n In seeking to track the performance of the index, the fund employs a replication strategy. It generally invests substantially all, but at least 95%, of its total assets in the securities comprising the index. The index includes securities of companies from the following industries: food and staples retailing; household products; food products; beverages; tobacco; and personal products. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.605081	2020-10-11 23:38:31.028166
XNET	Xunlei Limited	Xunlei Ltd is an online advertising service provider. It also provides downloading services and online game platforms for game developers and users. Its products and services include Xunlei accelerator and cloud acceleration subscription services.	{Technology,"Software - Application","Application Software"}	{SINA,NFLX,BIDU,FENG,P,YY,RENN,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:13.634335	2020-10-11 23:38:31.02423
XPH	SPDR S&P Pharmaceuticals	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of an index derived from the pharmaceuticals segment of a U.S. total market composite index.\n In seeking to track the performance of the S&P Pharmaceuticals Select Industry Index (the "index"), the fund employs a sampling strategy. It generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index represents the pharmaceuticals segment of the S&P Total Market Index ("S&P TMI"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.650415	2020-10-11 23:38:31.024353
XRLV	Invesco S&P 500 ex-Rate Sensitive Low Volatility	The investment seeks to track the investment results (before fees and expenses) of the S&P 500 Low Volatility Rate Response Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in common stocks that comprise the underlying index. The underlying index is designed to include stocks exhibiting low volatility characteristics, after removing stocks that historically have performed poorly in rising interest rate environments. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.660249	2020-10-11 23:38:31.02582
XSHD	Invesco S&P SmallCap High Dividend Low Volatility	The investment seeks to track the investment results (before fees and expenses) of the S&P SmallCap 600® Low Volatility High Dividend Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in securities that comprise the underlying index. The underlying index is composed of 60 securities in the S&P SmallCap 600® Index that historically have provided high dividend yields with lower volatility. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:13.668536	2020-10-11 23:38:31.024353
XSOE	WisdomTree Emerging Markets Ex-State Owned Enterprises Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree Emerging Markets ex-State-Owned Enterprises Index (the "index").\n Under normal circumstances, at least 80% of the fund's total assets will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a modified float-adjusted market cap weighted index that consists of common stocks in emerging markets, excluding common stocks of "state-owned enterprises." The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.675128	2020-10-11 23:38:31.023428
XENE	Xenon Pharmaceuticals Inc.	Xenon Pharmaceuticals Inc is a clinical-stage biopharmaceutical company. It is engaged in discovering and developing a pipeline of differentiated therapeutics for orphan indications. Its products portfolio includes Glybera, TV-45070.	{Healthcare,Biotechnology}	{GHDX,CMRX,ATRA,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:13.566724	2020-10-11 23:38:31.028166
XKST	SPDR Kensho Smart Mobility	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the Kensho Smart Transportation Index.\n Under normal market conditions, the fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index is comprised of U.S.-listed equity securities (including depositary receipts) of companies domiciled across developed and emerging markets worldwide which are included in the Smart Transportation sector as determined by a classification standard produced by the index provider. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.593871	2020-10-11 23:38:31.02582
XOG	Extraction Oil & Gas Inc.	Extraction Oil & Gas Inc is an energy company. The company engages in the exploration and production of oil and gas from its reserves in the Rocky Mountains in the Greater Wattenberg Field of Colorado's Denver-Julesburg Basin.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{}	Oil & Gas - E&P	Energy	Nasdaq Global Select	\N	2020-10-11 23:12:13.635656	2020-10-11 23:38:31.021426
XOXO	XO Group Inc.	XO Group Inc is a life stage media company. It serves its audience with information, products, and services during critical life stages: planning a wedding, sharing life as a couple for the first time, and planning for the birth of a first child.	{Technology,"Internet Content & Information","Online Media"}	{AMZN,IAC,JNJ,MDP,DIS,GHC,EVLV,GOOGL,XLK}	Online Media	Technology	New York Stock Exchange	\N	2020-10-11 23:12:13.640928	2020-10-11 23:38:31.031328
XPO	XPO Logistics Inc.	XPO Logistics Inc is a global transportation and logistics company. Its transportation operations, include asset-light truck brokerage, intermodal, and global forwarding, as well as asset-based truckload and less-than-truckload shipping services.	{Industrials,"Integrated Shipping & Logistics","Transportation & Logistics"}	{WERN,CHRW,ECHO,FDX,RRTS,YRCW,RLGT,ARCB,FRPH}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:13.654751	2020-10-11 23:38:31.02423
XPP	ProShares Ultra FTSE China 50	The investment seeks daily investment results, before fees and expenses, that correspond to two times (2x) the daily performance of the FTSE China 50 Index®.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index consists of 50 of the largest and most liquid Chinese stocks listed and traded on the Stock Exchange of Hong Kong. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.65563	2020-10-11 23:38:31.023428
XSHQ	Invesco S&P SmallCap Quality	The investment seeks to track the investment results (before fees and expenses) of the S&P SmallCap 600® Quality Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in securities that comprise the underlying index. S&P DJI weights each component stock of the underlying index by the total of its quality score multiplied by its market capitalization; stocks with higher scores receive relatively greater weights. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:13.668352	2020-10-11 23:38:31.028166
XSW	SPDR S&P Software & Services	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of an index derived from the computer software segment of a U.S. total market composite index.\n In seeking to track the performance of the S&P Software & Services Select Industry Index (the "index"), the fund employs a sampling strategy. It generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index represents the software and services segment of the S&P Total Market Index ("S&P TMI"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.67853	2020-10-11 23:38:31.025871
XTLB	XTL Biopharmaceuticals Ltd.	XTL Biopharmaceuticals Ltd is engaged in the acquisition & development of pharmaceutical drugs for the treatment of unmet medical needs, currently for the treatment of schizophrenia and Hepatitis C. It has also developed a home therapeutic device.	{Healthcare,Biotechnology}	{VRTX,AZN,GSK,BMY,JNJ,LLY,PFE,MRK,TEVA,ANTH,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:13.684752	2020-10-11 23:38:31.02582
XXII	22nd Century Group Inc.	22nd Century Group Inc is a plant biotechnology company. It is focused on technology that allow to increase or decrease the level of nicotine and other nicotinic alkaloids in tobacco plants and levels of cannabinoids in cannabis plants.	{Healthcare,Biotechnology}	{BTI,VGR,MO,PM,XLV}	Biotechnology	Healthcare	NYSE American	\N	2020-10-11 23:12:13.693255	2020-10-11 23:38:31.028166
XIN	Xinyuan Real Estate Co Ltd American Depositary Shares	Xinyuan Real Estate Co Ltd is a real estate developer and property manager. The Company develops residential properties in China.	{"Real Estate","Real Estate - General","Real Estate Services"}	{JOE,JLL}	Real Estate Services	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:13.58164	2020-10-11 23:38:31.02423
XLK	SPDR Select Sector Fund - Technology	The investment seeks investment results that, before expenses, correspond generally to the price and yield performance of publicly traded equity securities of companies in the Technology Select Sector Index.\n In seeking to track the performance of the index, the fund employs a replication strategy, which means that the fund typically invests in substantially all of the securities represented in the index in approximately the same proportions as the index. It generally invests substantially all, but at least 95%, of its total assets in the securities comprising the index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.605245	2020-10-11 23:38:31.024353
XMPT	VanEck Vectors CEF Municipal Income	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the S-Network Municipal Bond Closed-End Fund IndexSM (the "CEFMX Index").\n The fund normally invests at least 80% of its total assets in investments the income from which is exempt from U.S. federal income tax (other than federal alternative minimum tax). It normally invests at least 80% of its total assets in securities of issuers that comprise the fund's benchmark index. The CEFMX Index is comprised of shares of U.S.-listed closed-end funds.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.622456	2020-10-11 23:38:31.028166
XOM	Exxon Mobil Corporation	Exxon Mobil Corp is an integrated oil and gas company. It is engaged in exploration for, and production of, crude oil and natural gas. It is also engaged in manufacturing, transportation and sale of crude oil, natural gas and petroleum products.	{Energy,"Oil & Gas Integrated","Oil & Gas - Integrated"}	{E,TOT,HON,OXY,BP,CVX,COP,HES,HFC,PSX}	Oil & Gas - Integrated	Energy	New York Stock Exchange	\N	2020-10-11 23:12:13.637581	2020-10-11 23:38:31.023428
XRAY	DENTSPLY SIRONA Inc.	Dentsply Sirona Inc is a designer, developer, manufacturer and marketer of consumable dental products for the professional dental market. It also manufactures and markets other consumable medical device products.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{IART,ALGN,PDEX,CMD,DHR,ZBH,MMM,XLV}	Medical Instruments & Equipment	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:13.656055	2020-10-11 23:38:31.023007
XSLV	Invesco S&P SmallCap Low Volatility	The investment seeks to track the investment results (before fees and expenses) of the S&P SmallCap 600® Low Volatility Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in common stocks that comprise the underlying index. Strictly in accordance with its procedures and mandated guidelines, S&P DJI selects for inclusion in the underlying index the 120 securities that it has determined have the lowest volatility over the past 12 months out of the 600 small capitalization securities that are contained in the S&P SmallCap 600® Index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.672814	2020-10-11 23:38:31.02423
XTNT	Xtant Medical Holdings Inc.	Xtant Medical Holdings Inc develops, manufactures and markets regenerative medicine products and medical devices for domestic and international markets.	{Healthcare,"Medical Devices"}	{XLV}	Medical Devices	Healthcare	NYSE American	\N	2020-10-11 23:12:13.687952	2020-10-11 23:38:31.031328
XTL	SPDR S&P Telecom	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of an index derived from the telecommunications segment of a U.S. total market composite index.\n In seeking to track the performance of the S&P Telecom Select Industry Index (the "index"), the fund employs a sampling strategy. It generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index represents the telecommunications segment of the S&P Total Market Index ("S&P TMI"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.68426	2020-10-11 23:38:31.021426
XVZ	iPath S&P 500 Dynamic VIX ETN	The investment seeks to provide investors with exposure to the S&P 500® Dynamic VIX Futures™ Total Return Index.\nThe S&P 500® Dynamic VIX Futures™ Total Return Index (the "index") is designed to dynamically allocate between the S&P 500® VIX Short-Term Futures™ Index Excess Return and the S&P 500® VIX Mid-Term Futures™ Index Excess Return by monitoring the steepness of the implied volatility curve. The index seeks to react positively to overall increases in market volatility and aims to lower the roll cost of investments linked to future implied volatility.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.69183	2020-10-11 23:38:31.023007
YCS	ProShares UltraShort Yen New	The investment seeks to provide daily investment results (before fees and expenses) that correspond to twice (200%) the inverse of the daily performance of the U.S. Dollar price of the Japanese Yen.\nThe “UltraShort” funds seek results for a single day that match (before fees and expenses) two times the inverse (-2x) of the daily performance of a benchmark. It does not seek to achieve their stated objectives over a period greater than a single day.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.703576	2020-10-11 23:38:31.031328
XLRN	Acceleron Pharma Inc.	Acceleron Pharma Inc is a biotechnology company which focuses on discovery, development, and commercialization of novel therapies. It aims to discover key natural regulators of cellular growth and repair.	{Healthcare,Biotechnology}	{AMGN,SHPG,CELG,REGN,SGMO,JNJ,PFE,BLUE,OMED,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:13.611694	2020-10-11 23:38:31.021426
XON	Intrexon Corporation	Intrexon Corp is a biotechnology company that develops synthetic biology solutions. It designs, builds and regulates gene programs using its proprietary and complementary technologies.	{Healthcare,Biotechnology}	{ZIOP,LLY,TMO,OMEX,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:13.638195	2020-10-11 23:38:31.023007
XRF	China Rapid Finance Limited American Depositary Shares each representing one Class A	China Rapid Finance Ltd operates consumer lending marketplace for lenders and borrowers in the People's Republic of China with predictive selection technology , automated decisioning technology, non-credit data analytic & risk-based pricing capabilities.	{"Financial Services","Credit Services"}	{RENN}	Credit Services	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:13.65965	2020-10-11 23:38:31.031328
XT	iShares Exponential Technologies ETF	The investment seeks to track the investment results of the Morningstar® Exponential Technologies IndexSM which composed of stocks of developed and emerging market companies that create or use exponential technologies.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The underlying index is a subset of the Morningstar US Market Index and the Morningstar Global Markets ex-US Index family.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:13.680118	2020-10-11 23:38:31.028166
XTN	SPDR S&P Transportation	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of an index derived from the transportation segment of a U.S. total market composite index.\n In seeking to track the performance of the S&P Transportation Select Industry Index (the "index"), the fund employs a sampling strategy. It generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index represents the transportation segment of the S&P Total Market Index ("S&P TMI"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.687557	2020-10-11 23:38:31.024353
XUSA	QuantX Dynamic Beta US Equity	The investment seeks to replicate investment results that correspond to the performance of the QuantX Dynamic Beta US Equity Index.\n The fund generally will invest at least 80% of its total assets in the component securities of the index. The index selects domestic large, mid and small capitalization equity securities of companies listed on the Russell 1000 Index pursuant to a proprietary selection methodology that is designed to increase exposure to companies that have the highest estimated upside volatility relative to downside volatility while also managing overall portfolio risk by allocating to companies that have the lowest estimated downside volatility.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:13.688319	2020-10-11 23:38:31.02423
Y	Alleghany Corporation	Alleghany Corp together with its subsidiaries is engaged in the property and casualty reinsurance and insurance business in the United States. It operates through two segments namely reinsurance and insurance.	{"Financial Services","Insurance - Property & Casualty"}	{ACGL,GLRE,MHLD,WRB,CB,WTM}	Insurance - Property & Casualty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:13.696957	2020-10-11 23:38:31.028166
YCL	ProShares Ultra Yen	The investment seeks results for a single day that match (before fees and expenses) two times (2x) the daily performance of the U.S. dollar price of the Japanese yen.\nThe fund seeks to meet its investment objective by investing under normal market conditions in any one of, or combinations of, Financial Instruments (including swap agreements, futures contracts and forward contracts) based on the fund’s benchmark. It will also hold cash or cash equivalents such as U.S. Treasury securities or other high credit quality, short-term fixed-income or similar securities for direct investment or as collateral for Financial Instruments.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.703059	2020-10-11 23:38:31.021426
YECO	Yulong Eco-Materials Limited	Yulong Eco-Materials Ltd is a producer of fly-ash bricks and concrete in Pingdingshan. It operates in three segments Yulong Bricks, Yulong Concrete and Yulong Transport and Yulong Renewable.	{"Basic Materials","Building Materials"}	{YECO}	Building Materials	Basic Materials	NASDAQ Capital Market	\N	2020-10-11 23:12:13.704455	2020-10-11 23:38:31.02582
SPY	SPDR S&P 500	The investment seeks to provide investment results that, before expenses, correspond generally to the price and yield performance of the S&P 500® Index.\n The Trust seeks to achieve its investment objective by holding a portfolio of the common stocks that are included in the index (the “Portfolio”), with the weight of each stock in the Portfolio substantially corresponding to the weight of such stock in the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.756413	2020-10-11 23:38:31.021426
SQNS	Sequans Communications S.A. American Depositary Shares each representing one	Sequans Communications SA is a fabless designer, developer and supplier of 4G LTE & WiMAX semiconductor solutions for wireless mobile broadband applications, with a specific focus on the single-mode device market.	{Technology,Semiconductors}	{INTC,DSPG,QCOM,NVDA,MRVL,ON,XLK}	Semiconductors	Technology	New York Stock Exchange	\N	2020-10-11 23:12:11.775417	2020-10-11 23:38:31.021426
SRRA	Sierra Oncology Inc.	Sierra Oncology Inc is a clinical-stage drug development company engaged in next generation DNA Damage Response (DDR) therapeutics for the treatment of patients with cancer.	{Healthcare,Biotechnology}	{NAVB,PIRS,CSBR,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:11.822117	2020-10-11 23:38:31.021426
SSBI	Summit State Bank	Summit State Bank is a state-chartered commercial bank which provides banking services to individuals and businesses. It offers depository and lending services to meet the needs of its business and individual clientele.	{"Financial Services","Banks - Regional - US",Banks}	{AMRB,HTBK,WABC,PPBI,BOCH,BMRC,PFBC}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:11.834623	2020-10-11 23:38:31.025871
SSNC	SS&C Technologies Holdings Inc.	SS&C Technologies Holdings Inc provides software and software-enabled services including SaaS to the asset and wealth management, alternative investment management, and financial advisory markets.	{Technology,"Software - Infrastructure","Application Software"}	{MANH,FISV,LOGM,FICO,ELLI,MODN,DATA,PAYC,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:11.844782	2020-10-11 23:38:31.031328
SSY	SunLink Health Systems Inc.	SunLink Health Systems Inc provides healthcare services. It, through its subsidiaries, owns and operate nearly 80-licensed-bed, acute care hospitals, and nursing home. It also offers pharmacy products and services.	{Healthcare,"Medical Care","Health Care Providers"}	{LPNT,UHS,CYH,HCA,THC,XLV}	Health Care Providers	Healthcare	NYSE American	\N	2020-10-11 23:12:11.857949	2020-10-11 23:38:31.028166
STBA	S&T Bancorp Inc.	S&T Bancorp Inc provides banking products and services. The company's products and services include commercial banking, cash management, trust & brokerage and insurance.	{"Financial Services","Banks - Regional - US",Banks}	{CZNC,BMTC,PWOD,UVSP,TMP,MTB,FCF,FNB,ACNB}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:11.871704	2020-10-11 23:38:31.028166
STKL	SunOpta Inc.	SunOpta Inc is engaged in sourcing, processing and packaging of natural, organic and specialty food products. The company also produces, distributes and recycles industrial minerals, silica-free abrasives and speciality sands.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{ADM,BORN,POST,INGR}	Consumer Packaged Goods	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:11.884222	2020-10-11 23:38:31.021426
STN	Stantec Inc	Stantec Inc provides professional services in the area of infrastructure and facilities for clients in the public and private sectors. The company's services include planning, engineering, architecture, interior design, surveying and project management.	{Industrials,"Engineering & Construction"}	{TTEK,WLDN,JEC}	Engineering & Construction	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:11.899516	2020-10-11 23:38:31.02423
STPP	iPath US Treasury Steepener ETN	The investment seeks to provide investors with exposure to the Barclays US Treasury 2Y/10Y Yield Curve Index™. \nThe Barclays US Treasury 2Y/10Y Yield Curve Index™ (the "index") employs a strategy that seeks to capture returns that are potentially available from a "steepening" or "flattening", as applicable, of the U.S. Treasury yield curve through a notional rolling investment in U.S. Treasury note futures contracts. The level of the index is designed to increase in response to a "steepening" of the yield curve and to decrease in response to a "flattening" of the yield curve.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:11.916826	2020-10-11 23:38:31.028166
STZ	Constellation Brands Inc.	Constellation Brands Inc is engaged in the beverage industry. Its core products are alcoholic drinks such as beers, wines and other spirits.	{"Consumer Defensive","Beverages - Wineries & Distilleries","Beverages - Alcoholic"}	{DEO,WVVI,SAM,CCU,BF.B,TAP}	Beverages - Alcoholic	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:11.934764	2020-10-11 23:38:31.028166
SUSA	iShares MSCI USA ESG Select	The investment seeks to track the investment results of the MSCI USA Extended ESG Select Index composed of U.S. companies that have positive environmental, social and governance characteristics as identified by the index provider.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index is an optimized index designed to maximize exposure to favorable environmental, social and governance ("ESG") characteristics, while exhibiting risk and return characteristics similar to the MSCI USA Index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.956213	2020-10-11 23:38:31.028166
XHR	Xenia Hotels & Resorts Inc.	Xenia Hotels & Resorts Inc is a real estate investment trust. The Company invests in premium full service, lifestyle and urban upscale hotels in United States.	{"Real Estate","REIT - Hotel & Motel",REITs}	{PEB,RLJ,VAC,RHP}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:13.582323	2020-10-11 23:38:31.031328
AADR	AdvisorShares Dorsey Wright ADR	The investment seeks long-term capital appreciation above international benchmarks such as the BNY Mellon Classic ADR Index, the fund's primary benchmark, and the MSCI EAFE Index, the fund's secondary benchmark.\n The fund seeks to achieve the fund's investment objective by selecting primarily a portfolio of U.S.-traded securities of non-U.S. organizations, most often American Depositary Receipts ("ADRs"). It will invest at least 80% of its total assets in ADRs and in securities that have economic characteristics similar to ADRs.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.243591	2020-10-11 23:38:31.02582
AAXN	Axon Enterprise Inc.	Axon Enterprise Inc offers a network of devices, apps, and people that helps law enforcement become smarter and safer. Its products include the connected body cameras and evidence-management cloud, and TASER Smart Weapons.	{Industrials,"Aerospace & Defense"}	{}	Aerospace & Defense	Industrials	Nasdaq Global Select	\N	2020-10-11 23:11:59.302467	2020-10-11 23:38:31.02582
ADXS	Advaxis Inc.	Advaxis Inc focuses on discovery, development, and commercialization of proprietary Lm (Listeria monocytogenes)-LLO (Listeriolysin O) cancer immunotherapies.	{Healthcare,Biotechnology}	{HSKA,GNCA,INO,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.470319	2020-10-11 23:38:31.02582
AEO	American Eagle Outfitters Inc.	American Eagle Outfitters is a retailer of apparel and accessories that also offers personal care products. It sells its products in stores as well as online. It operates stores in the USA, Canada, Mexico, Hong Kong, China and the UK.	{"Consumer Cyclical","Apparel Stores","Retail - Apparel & Specialty"}	{URBN,ZUMZ,ANF,BKE,GPS,EXPR,TLYS}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:11:59.484631	2020-10-11 23:38:31.02582
AETI	American Electric Technologies Inc.	American Electric Technologies Inc is a provider of power delivery solutions to the global energy industry. The principal markets that company serves include power generation and distribution, oil and natural gas, marine and industrial.	{Industrials,"Diversified Industrials","Industrial Products"}	{AEIS,POWL,NOV,GE,TRN,AZZ,UQM,ABB,PLUG,PPSI}	Industrial Products	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:11:59.495343	2020-10-11 23:38:31.02582
ALTS	ProShares Morningstar Alternatives Solution	The investment seeks investment results, before fees and expenses, that track the performance of the Morningstar® Diversified Alternatives IndexSM (the "index").\n The fund is a fund of ETFs and seeks to achieve its investment objective by investing primarily in the underlying ETFs. It is designed to provide investors with a comprehensive solution to their alternatives allocation by investing in the alternative ETFs comprising its index. The index is designed to provide diversified exposure to alternative asset classes when combined with a range of traditional investments. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:11:59.651688	2020-10-11 23:38:31.02582
AMOT	Allied Motion Technologies Inc.	Allied Motion Technologies Inc designs, manufactures and sells precision and speciality motion control components and systems. The firm's primary targets are vehicle, medical, aerospace, defense, electronics and industrial, pumps and robotics sectors.	{Technology,"Electronic Components","Computer Hardware"}	{TRNS,AIMC,RBC,UQM,GNRC,SVT,RXN,XLK}	Computer Hardware	Technology	NASDAQ Global Market	\N	2020-10-11 23:11:59.693252	2020-10-11 23:38:31.02582
YORW	The York Water Company	The York Water Co is an investor-owned water utility in the United States. The primary business of the company is to impound, purify to meet or exceed safe drinking water standards and distribute water.	{Utilities,"Utilities - Regulated Water","Utilities - Regulated"}	{ARTNA,CTWS,MSEX,CDZI,CWT,AWR,WTR,AWK,SJW,XLU}	Utilities - Regulated	Utilities	Nasdaq Global Select	\N	2020-10-11 23:12:13.7286	2020-10-11 23:38:31.028166
YUMC	Yum China Holdings Inc.	Yum China Holdings Inc is a restaurant company. It holds the right to operate various restaurant chains including KFC, Pizza Hut Casual Dining, Pizza Hut Home Service, East Dawning, Little Sheep and Taco Bell.	{"Consumer Cyclical",Restaurants}	{SAUC,QSR}	Restaurants	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:13.740524	2020-10-11 23:38:31.028166
ZAGG	ZAGG Inc	Zagg Inc designs, produces, and distributes professional and premium product solutions for mobile devices. It offers screen protection, keyboards for tablet computers and mobile devices, keyboard cases, earbuds, and mobile power solutions.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{AAPL,LOGI,BBY,ACCO}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:13.757137	2020-10-11 23:38:31.023007
ZEN	Zendesk Inc.	Zendesk Inc develops and sells the Saas software platform and other communication applications mainly in the United States. The Saas platform is used by organizations to communicate with its customers and resolve their queries.	{Technology,"Software - Application","Application Software"}	{NICE,LPSN,VRNT,CRM,SREV,EGAN,NOW,ORCL,FIVN,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:13.773542	2020-10-11 23:38:31.021426
ZLAB	Zai Lab Limited	Zai Lab Ltd is a biopharmaceutical company. The company is engaged in discovering or licensing, developing and commercializing proprietary therapeutics in the fields of oncology, autoimmune and infectious diseases.	{Healthcare,Biotechnology}	{PDLI,PIRS,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:13.795162	2020-10-11 23:38:31.023428
ZOES	Zoe's Kitchen Inc.	Zoe's Kitchen Inc operates fast casual restaurant concept serving a distinct menu of fresh, wholesome, Mediterranean-inspired dishes delivered with Southern hospitality.	{"Consumer Cyclical",Restaurants}	{PZZA,JACK,TAST,MCD,YUM,DNKN,RAVE}	Restaurants	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:13.807624	2020-10-11 23:38:31.02582
ZUMZ	Zumiez Inc.	Zumiez Inc is a multi-channel specialty retailer. It is engaged in the retail sales of apparel, footwear, accessories and hardgoods for young men and women.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{URBN,ANF,AEO,BKE,EXPR,FRAN,TLYS,DXLG,TLRD}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:13.820894	2020-10-11 23:38:31.023007
AAOI	Applied Optoelectronics Inc.	Applied Optoelectronics Inc is a vertically integrated provider of fiber-optic networking products,for four networking end-markets: internet data center, cable television, or CATV, telecommunications, or telecom, fiber-to-the-home, or FTTH.	{Technology,Semiconductors}	{CSCO,FNSR,OCLR,MXL,NPTN,EMKR,VIAV,XLK}	Semiconductors	Technology	NASDAQ Global Market	\N	2020-10-11 23:11:59.252669	2020-10-11 23:38:31.023007
AAT	American Assets Trust Inc.	American Assets Trust Inc is a self-administered real estate investment trust based in the United States. The company mainly invests in, operates, and develops retail, office, residential, and mixed-use properties in California, Oregon, and Hawaii.	{"Real Estate","REIT - Retail",REITs}	{VNO,BXP,FRT,HMG,WRE,EPR,VER}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:11:59.289389	2020-10-11 23:38:31.023007
ACWF	iShares Edge MSCI Multifactor Global	The investment seeks to track the investment results of the MSCI ACWI Diversified Multiple-Factor Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The underlying index is designed to contain equity securities from the MSCI ACWI Index (the "parent index") that have high exposure to four investment style factors: value, quality, momentum and low size, while maintaining a level of risk similar to that of the parent index.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.436843	2020-10-11 23:38:31.023007
AMBA	Ambarella Inc.	Ambarella Inc is a developer of semiconductor processing solutions for video that enable high-definition (HD), video capture, analysis, sharing and display.	{Technology,"Semiconductor Equipment & Materials",Semiconductors}	{INTC,QCOM,NVDA,TXN,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:11:59.662777	2020-10-11 23:38:31.023007
YTEN	Yield10 Bioscience Inc.	Yield10 Bioscience Inc, formerly known as Metabolix Inc is an agricultural bioscience company. It is engaged in the development of disruptive technologies to produce the improvements in crop yield for food and feed crops to enhance food security.	{"Basic Materials","Specialty Chemicals",Chemicals}	{}	Chemicals	Basic Materials	NASDAQ Capital Market	\N	2020-10-11 23:12:13.734108	2020-10-11 23:38:31.024353
YXI	ProShares Short FTSE China 50	The investment seeks daily investment results, before fees and expenses, that correspond to the inverse (-1x) of the daily performance of the FTSE China 50 Index®.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index consists of 50 of the largest and most liquid Chinese stocks listed and traded on the Stock Exchange of Hong Kong. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.746824	2020-10-11 23:38:31.031328
ZBIO	ProShares UltraPro Short NASDAQ Biotechnology	The investment seeks daily investment results, before fees and expenses, that correspond to three times the inverse (-3x) of the daily performance of the NASDAQ Biotechnology Index®.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index is a modified capitalization weighted index that includes securities of the Nasdaq Stock Market listed companies that are classified as either biotechnology or pharmaceutical. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:13.76328	2020-10-11 23:38:31.028166
ZF	Virtus Total Return Fund Inc.	Virtus Total Return Fund is a non-diversified, closed-end management investment company. The Fund's investment objective is total return, consisting of both capital appreciation and current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:13.776188	2020-10-11 23:38:31.023428
ZGNX	Zogenix Inc.	Zogenix Inc is a pharmaceutical company engaged in commercializing and developing therapies that address clinical needs for people living with pain-related and CNS disorders that needs treatment alternatives to help return to normal daily functioning.	{Healthcare,"Drug Manufacturers - Major","Drug Manufacturers"}	{DRRX,NKTR,CAH,JNJ,LLY,PFE,MRK,ALKS,ATRS,MNK,XLV}	Drug Manufacturers	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:13.786137	2020-10-11 23:38:31.028166
ZIXI	Zix Corporation	Zix Corp provides email encryption and data loss prevention solutions. The solutions are provided to meet the data protection and compliance needs of organizations primarily in the healthcare, financial services, and government sectors.	{Technology,"Software - Application","Application Software"}	{MSFT,CTXS,IBM,VMW,VHC,PFPT,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:13.793818	2020-10-11 23:38:31.02423
ZN	Zion Oil & Gas Inc	Zion Oil & Gas Inc is the United States-based an oil and gas exploration and production company. Zion currently holds one active petroleum exploration license onshore Israel, the Megiddo-Jezreel License.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{TGA,EGY,TAT}	Oil & Gas - E&P	Energy	NASDAQ Global Market	\N	2020-10-11 23:12:13.803494	2020-10-11 23:38:31.028166
ZTO	ZTO Express (Cayman) Inc. American Depositary Shares each representing one Class A.	ZTO Express (Cayman) Inc is an express delivery company in China. It provides express delivery service through its nationwide network as well as other value-added logistics services.	{Industrials,"Business Services","Integrated Shipping & Logistics","Transportation & Logistics"}	{PKOH}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:13.814397	2020-10-11 23:38:31.025871
AAMC	Altisource Asset Management Corp Com	Altisource Asset Management Corp is an asset management company that provides portfolio management and corporate governance services. It provides management services to its Altisource Residential Corporation and NewSource Reinsurance Company Ltd.	{"Financial Services","Asset Management"}	{RWT,NLY,CMO,ORC,TWO}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:11:59.246494	2020-10-11 23:38:31.021426
ACGLP	Arch Capital Group Ltd. Depositary Shares Representing Interest in 5.25% Non-Cumulative Preferred Series E Shrs	Arch Capital Group Ltd provides insurance, reinsurance and mortgage. It provides property, casualty and mortgage insurance and reinsurance lines, and focus on writing specialty lines of insurance and reinsurance.	{"Financial Services","Insurance - Diversified",Insurance}	{}	Insurance	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:11:59.402515	2020-10-11 23:38:31.021426
ADC	Agree Realty Corporation	Agree Realty Corporation is engaged in real estate activities primarily in the United States. It manages retail properties for the purpose of generating shareholders income on a consistent basis.	{"Real Estate","REIT - Retail",REITs}	{BFS,CBL,ANH,APTS}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:11:59.443657	2020-10-11 23:38:31.021426
ALTY	Global X SuperDividend Alternatives ETF	The investment seeks to track, before fees and expenses, the price and yield performance of the Indxx SuperDividend® Alternatives Index ("underlying index").\n The fund invests at least 80% of its total assets in the securities of the underlying index and in American Depositary Receipts ("ADRs") and Global Depositary Receipts ("GDRs") based on the securities in the underlying index. The underlying index is comprised of securities that rank among the highest dividend-yielding securities in each eligible category of alternative income investments, at the time of index reconstitution, as defined by the index provider. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:11:59.655487	2020-10-11 23:38:31.021426
APVO	Aptevo Therapeutics Inc.	Aptevo Therapeutics Inc is a biosciences company. The company is engaged in the discovery, development, commercialization and sale of novel oncology and hematology therapeutics.	{Healthcare,Biotechnology}	{PDLI,PIRS,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.790465	2020-10-11 23:38:31.021426
ASHS	Xtrackers Harvest CSI 500 China A-Shares Small Cap	The investment seeks investment results that correspond generally to the performance, before fees and expenses, of the CSI 500 Index.\n The index is designed to reflect the price fluctuation and performance of small-cap companies in the China A-Share market and is composed of the 500 smallest and most liquid stocks in the China A-Share market. Under normal circumstances, the fund invests at least 80% of its net assets, plus the amount of any borrowings for investment purposes, in A-Shares of Chinese small-cap issuers or in derivative instruments and other securities that provide investment exposure to A-Shares of Chinese small-cap issuers.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.859513	2020-10-11 23:38:31.021426
AXDX	Accelerate Diagnostics Inc.	Accelerate Diagnostics Inc is an in vitro diagnostics company, dedicated in providing solutions that improve patient outcomes and lower healthcare costs through the rapid diagnosis of serious infections.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{BRKR,BDX,BIO,TMO,PKI,NSTG,XLV}	Medical Diagnostics & Research	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:11:59.978602	2020-10-11 23:38:31.021426
BFIT	Global X Health & Wellness Thematic ETF	The investment seeks investment results that correspond generally to the price and yield performance, before fees and expenses, of the Indxx Global Health & Wellness Thematic Index.\n The fund invests more than 80% of its total assets in the securities of the underlying index and in ADRs and GDRs based on the securities in the underlying index. The underlying index is designed to provide exposure to exchange-listed companies in developed markets that provide products and services that facilitate physical wellness through active and healthy lifestyles. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:00.142845	2020-10-11 23:38:31.021426
YRD	Yirendai Ltd. American Depositary Shares each representing two	Yirendai Ltd provides consumers in China with easy access to affordable credit and investors with attractive investment opportunities. It operates online consumer finance marketplace connecting investors and individual borrowers.	{"Financial Services","Credit Services"}	{RENN}	Credit Services	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:13.733397	2020-10-11 23:38:31.021426
ZDGE	Zedge Inc. Class B	Zedge Inc provides content distribution platforms. Its content platform enables consumers to personalize their mobile devices with free, high-quality ringtones, wallpapers, home screen app icons, widgets and notification sounds.	{Technology,"Software - Application","Application Software"}	{OTEX,PANW,XLK}	Application Software	Technology	NYSE American	\N	2020-10-11 23:12:13.769848	2020-10-11 23:38:31.02582
ZIONW	Zions Bancorporation N.A. Warrants 05/21/2020	Zions Bancorp is a bank holding company. The bank has over 400 branches across the United States and has more than $50 billion worth of assets. It provides banking services to small and midsize businesses as well as individuals.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:13.792922	2020-10-11 23:38:31.031328
ZSAN	Zosano Pharma Corporation	Zosano Pharma Corp is a clinical-stage specialty pharmaceutical company. It has developed a transdermal microneedle patch system to deliver its formulations of existing drugs through the skin for the treatment of various indications.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:13.81043	2020-10-11 23:38:31.021426
A	Agilent Technologies Inc.	Agilent Technologies Inc is engaged in life sciences, diagnostics and applied chemical markets. The company provides application focused solutions that include instruments, software, services and consumables for the entire laboratory workflow.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{NATI,ILMN,BRKR,AME,DHR,GE,TMO,PKI,TER,XLV}	Medical Diagnostics & Research	Healthcare	New York Stock Exchange	\N	2020-10-11 23:11:59.25509	2020-10-11 23:38:31.028166
ABB	ABB Ltd	ABB Ltd is a power and automation systems provider that are designed to boost industrial productivity, increase power grid reliability and enhance energy efficiency. The company works closely with utility, industry, transport and infrastructure customers.	{Industrials,"Diversified Industrials","Industrial Products"}	{EMR,GE,ITW,HON,UTX}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.30925	2020-10-11 23:38:31.028166
ACH	Aluminum Corporation of China Limited American Depositary Shares	Aluminum Corporation of China Ltd is one of the world's leading producer of aluminum. The company has operations across every phase of upstream aluminum production--bauxite mining, alumina refining and primary aluminum smelting.	{"Basic Materials",Aluminum,"Metals & Mining"}	{KALU,AA}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:11:59.404709	2020-10-11 23:38:31.028166
ACNB	ACNB Corporation	ACNB Corp provides banking, insurance, and financial services to businesses, and consumers through its wholly-owned subsidiaries, ACNB Bank and Russell Insurance Group Inc (RIG). The corporation has two reporting segments, the Bank and RIG.	{"Financial Services","Banks - Regional - US",Banks}	{CZNC,ORRF,FUNC,FULT,PWOD,STBA,UVSP,FNB}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:11:59.422487	2020-10-11 23:38:31.028166
AIRR	First Trust RBA American Industrial Renaissance ET	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an index called the Richard Bernstein Advisors American Industrial Renaissance® Index (the "index").\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in U.S. equity securities that comprise the index. The index is designed to measure the performance of small and mid cap U.S. companies in the industrial and community banking sectors.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:11:59.57482	2020-10-11 23:38:31.028166
AMP	Ameriprise Financial Inc.	Ameriprise Financial Inc is a diversified financial services company. It offers products and services designed to achieve the financial objectives of individual and institutional clients. It also has the largest branded advisor networks in the industry.	{"Financial Services","Asset Management"}	{SEIC,TROW,AMG,FII,RJF,PFG}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.695417	2020-10-11 23:38:31.028166
APD	Air Products and Chemicals Inc.	Air Products & Chemicals Inc is a supplier of hydrogen and helium. It also provides semiconductor materials, refinery hydrogen, natural gas liquefaction and coatings and adhesives.	{"Basic Materials",Chemicals}	{MEOH,VLO,DWDP}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:11:59.764154	2020-10-11 23:38:31.028166
AQMS	Aqua Metals Inc.	Aqua Metals Inc is engaged in recycling lead-acid batteries. The company has developed AquaRefining, a modular electrochemical process that produces ultra-pure lead from used lead-acid batteries.	{Industrials,"Waste Management"}	{}	Waste Management	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:11:59.794497	2020-10-11 23:38:31.028166
YUM	Yum! Brands Inc.	Yum Brands Inc is engaged in operating quick-service restaurant systems. The company's restaurant brands includes KFC, Pizza Hut and Taco Bell .	{"Consumer Cyclical",Restaurants}	{SBUX,JACK,DRI,EAT,MCD,DPZ,CMG,WEN,FRSH}	Restaurants	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:13.740379	2020-10-11 23:38:31.02423
Z	Zillow Group Inc.	Zillow Group Inc provides real estate and home-related information marketplaces on mobile and web-based platforms. It helps customers on each stage of the home lifecycle such as renting, buying, selling, financing, and home improvement.	{"Real Estate","Real Estate Services"}	{PDFS,OMCL}	Real Estate Services	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:13.757755	2020-10-11 23:38:31.024353
ZEUS	Olympic Steel Inc.	Olympic Steel Inc is a metal services center in U.S. The company is engaged in the processing and distribution of carbon, coated, aluminum and stainless steel, flat-rolled coil, sheet and plate products and tubular and pipe products to various industries.	{"Basic Materials",Steel}	{FSTR,RS,HSC,WOR,FRD,MRC}	Steel	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:12:13.77708	2020-10-11 23:38:31.02423
ZM	Zoom Video Communications, Inc.	Zoom Video Communications, Inc. develops a people-centric cloud service that transforms real-time collaboration experience. The Company offers unified meeting experience, a cloud service that provides a 3-in-1 meeting platform with HD video conferencing, mobility, and web meetings. Zoom Video Communications serves customers worldwide.	{Technology,Software,"Telecom Services"}	{WORK,MSFT,GOOGL}	Telecom Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:12:13.798208	2020-10-11 23:38:31.024353
ZSL	ProShares UltraShort Silver	The investment seeks results for a single day that match (before fees and expenses) two times the inverse (-2x) of the daily performance of silver bullion as measured by the London Silver Price.\n The fund seeks to meet its investment objective by investing under normal market conditions in any one of, or combinations of, Financial Instruments (including swap agreements, futures contracts and forward contracts) based on the fund’s benchmark. It may invest up to 100% of its assets in any of these types of cash or cash equivalent securities.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.81235	2020-10-11 23:38:31.023428
ZYME	Zymeworks Inc.	Zymeworks Inc is a clinical-stage biopharmaceutical company dedicated to the discovery, development and commercialization of next-generation multifunctional biotherapeutics, initially focused on the treatment of cancer.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:13.826715	2020-10-11 23:38:31.021426
AAN	Aaron's Inc.	Aaron's Inc is a retailer of furniture, consumer electronics, computers, appliances and household accessories. It also offers lease-purchase solutions to customers of traditional retailers and revolving credit products.	{Industrials,"Rental & Leasing Services","Consulting & Outsourcing"}	{RCII,CONN,BBY,PIR,WSM,GME}	Consulting & Outsourcing	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.25022	2020-10-11 23:38:31.031328
AAU	Almaden Minerals Ltd.	Almaden Minerals Ltd is a mineral exploration company. It is engaged in the business of exploring and developing mineral projects and its principal asset is the Ixtaca precious metals project located on its Tuligtic claim in Mexico.	{"Basic Materials",Gold,"Metals & Mining"}	{TECK,NG,SA,GG,AUY,TRX,DF}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:11:59.295472	2020-10-11 23:38:31.031328
ACET	Aceto Corporation	Aceto Corp is engaged in the marketing, sells and distribution of finished dosage form generic pharmaceuticals, nutraceutical products, pharmaceutical active ingredients and intermediates, and specialty performance chemicals.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{IPHS,NVS,CBM,PFE,KWR,MRK,TEVA,MYL,TLGT,XLV}	Drug Manufacturers	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.390926	2020-10-11 23:38:31.031328
ADMP	Adamis Pharmaceuticals Corporation	Adamis Pharmaceuticals Corp is a specialty biopharmaceutical company that develops & commercializes products in the therapeutic areas of respiratory disease & allergy. Its products include Epinephrine Injection PFS syringe, APC-1000, APC-2000, & APC-5000.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{SIGA,CORT,JNJ,PFE,TEVA,ATRS,MYL,PTX,XLV}	Drug Manufacturers	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:11:59.451557	2020-10-11 23:38:31.031328
AEB	AEGON N.V. Perp. Cap. Secs. Floating Rate (Netherlands)	Aegon NV is engaged in providing life insurance, pensions and asset management products and services in the USA, Europe, and Asia. It offers life and protection, and accidental death products as well as individual savings and retirement products.	{"Financial Services","Insurance - Diversified",Insurance}	{}	Insurance	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.474427	2020-10-11 23:38:31.031328
AEHR	Aehr Test Systems	Aehr Test Systems is engaged in the designing, manufacturing and marketing of test and burn-in products to the semiconductor manufacturing industry in several geographic areas.	{Technology,"Semiconductor Equipment & Materials",Semiconductors}	{COHU,NANO,FORM,TER,INTT,RTEC,XLK}	Semiconductors	Technology	NASDAQ Capital Market	\N	2020-10-11 23:11:59.482794	2020-10-11 23:38:31.031328
AMNB	American National Bankshares Inc.	American National Bankshares Inc is a one-bank holding company which provides community banking which making loans to and generating deposits from individuals and Trust and investment services to its customers.	{"Financial Services","Banks - Regional - US",Banks}	{NKSH,FBSS,TOWN,OPOF,UCBI,UBSH}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:11:59.692941	2020-10-11 23:38:31.031328
ARII	American Railcar Industries Inc.	American Railcar Industries Inc designs, manufactures and sells special, customized and general-purpose railcars and components for the North American railcar and industrial markets. Its segments are manufacturing, railcar leasing and railcar services.	{Industrials,Railroads,"Transportation & Logistics"}	{RAIL,GBX,TRN,WAB}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:11:59.82326	2020-10-11 23:38:31.031328
BBU	Brookfield Business Partners L.P. Limited Partnership Units	Brookfield Business Partners LP are business services and industrial company focused on owning & operating businesses. Its principal business service operations include construction services, residential real estate services and facilities management.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:00.068962	2020-10-11 23:38:31.031328
BIOS	BioScrip Inc.	BioScrip Inc provides home infusion & other home healthcare services to patients, physicians, hospitals, healthcare payors & pharmaceutical manufacturers to provide clinical management solutions & delivers cost-effective access to medications.	{Healthcare,"Medical Care","Health Care Providers"}	{BCRX,AMED,LHCG,CVS,BKD,WBA,XLV}	Health Care Providers	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:00.230106	2020-10-11 23:38:31.031328
YTRA	Yatra Online Inc.	Yatra Online Inc offer comprehensive travel-related services, which include domestic and international air ticketing, hotel bookings, homestays, holiday packages, bus ticketing, rail ticketing, activities and ancillary services.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{}	Travel & Leisure	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:13.739063	2020-10-11 23:38:31.028166
ZAYO	Zayo Group Holdings Inc.	Zayo Group Holdings Inc provides fiber-based communications infrastructure services. Its services include leased dark fiber, fiber to cellular towers and small cell sites, dedicated wavelength connections, Ethernet and other high-bandwidth offerings.	{Technology,"Communication Equipment"}	{CCOI,CTL,FTR,XLK}	Communication Equipment	Technology	New York Stock Exchange	\N	2020-10-11 23:12:13.757681	2020-10-11 23:38:31.021426
ZEAL	Zealand Pharma A/S	Zealand Pharma A/S is a biotech company. It is focused on the discovery, design, and development of peptide-based medicines.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:13.770531	2020-10-11 23:38:31.025871
ZIOP	ZIOPHARM Oncology Inc	ZIOPHARM Oncology Inc is a biotechnology company which acquires, develops, and commercializes cancer therapies that can address unmet medical needs through synthetic biology.	{Healthcare,Biotechnology}	{AMGN,BAX,BMY,TSRO,CBLI,CYTR,IQV,XON,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:13.792258	2020-10-11 23:38:31.02582
AA	Alcoa Corporation	Alcoa Corp is an integrated aluminum company. The company engages in aluminum production from bauxite mining to manufactures finished aluminum goods. It serves finished aluminum goods to a wide-variety of industrial end markets.	{"Basic Materials",Aluminum,"Metals & Mining"}	{BBL,CENX,KALU,BHP,ACH}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:11:59.243865	2020-10-11 23:38:31.024353
AB	AllianceBernstein Holding L.P. Units	AllianceBernstein Holding LP is engaged in the financial service sector. Its activities include investment management services and its sell-side business.	{"Financial Services","Asset Management"}	{AMG,WDR,EV,BEN,LM,BLK,LAZ,GBL,OAK}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.308466	2020-10-11 23:38:31.024353
ACCO	Acco Brands Corporation	ACCO Brands Corp is a manufacturer and marketer of office, school and calendar products and selected computer and electronic accessories. It sells its products to consumers and commercial end-users through resellers.	{Industrials,"Business Equipment","Industrial Products"}	{LOGI,ZAGG,NWL,ACU,PBI,MMM}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.383557	2020-10-11 23:38:31.024353
AIRT	Air T Inc.	Air T Inc provides overnight air freight service to the express delivery industry, manufactures and sells aircraft deicers and other special purpose industrial equipment, and provides ground support equipment and facilities maintenance to airlines.	{Industrials,"Integrated Shipping & Logistics","Transportation & Logistics"}	{ATSG,SINO,UPS,FDX}	Transportation & Logistics	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:11:59.574865	2020-10-11 23:38:31.024353
AKAM	Akamai Technologies Inc.	Akamai Technologies Inc is a United States based company which provides cloud services for delivering, optimizing and securing online content and business applications over the Internet.	{Technology,"Software - Application","Application Software"}	{FFIV,VRSN,QCOM,NUAN,LLNW,RHT,IBM,T,EQIX,INAP,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:11:59.592096	2020-10-11 23:38:31.024353
AOK	iShares Core Conservative Allocation	The investment seeks to track the investment results of the S&P Target Risk Conservative Index composed of a portfolio of underlying equity and fixed income funds intended to represent a conservative target risk allocation strategy.\n The fund is a fund of funds and seeks its investment objective by investing primarily in other iShares Underlying Funds that themselves seek investment results corresponding to their own respective underlying indexes. It generally will invest at least 90% of its assets in the component securities of the underlying index. The index measures the performance of the S&P Dow Jones Indices LLC proprietary allocation model.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.75118	2020-10-11 23:38:31.024353
ASET	FlexShares Real Assets Allocation Index Fund	The investment seeks investment results that correspond generally to the price and yield performance, before fees and expenses, of the Northern Trust Real Assets Allocation IndexSM.\n Under normal circumstances, the fund will invest at least 80% of its total assets (exclusive of collateral held from securities lending) in the securities of the underlying index. The underlying index measures the performance of an optimized allocation to the underlying funds that is intended to provide exposures to certain real assets and minimize overall volatility of investment in the underlying funds. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:11:59.854612	2020-10-11 23:38:31.024353
BANC	Banc of California Inc.	Banc of California Inc is a is a financial holding company. It offers banking and financial services. Its services include banking services, lending services and private banking services.	{"Financial Services","Banks - Regional - US",Banks}	{WAFD,PROV,RVSB,TSBK,BYFC,FFNW,TBNK,ANCB,FSBW}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.02576	2020-10-11 23:38:31.024353
BCD	Aberdeen Standard Bloomberg All Commodity Longer Dated Strategy K-1 Free	The investment seeks to provide a total return designed to exceed the performance of the Bloomberg Commodity Index 3 Month ForwardSM which is calculated on an excess return basis.\n The fund is an actively managed ETF that seeks to provide a total return designed to exceed the performance of the index. It will generally seek to hold similar interests to those included in the index and will seek exposure to many of the commodities included in the index under the same futures rolling schedule as the index. The fund will also hold short-term fixed-income securities. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.086425	2020-10-11 23:38:31.024353
YUMA	Yuma Energy Inc.	Yuma Energy Inc is an independent oil & gas exploration and production company engaged in acquiring, developing and exploring for conventional and unconventional oil and natural gas resources in the U.S. Gulf Coast.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{}	Oil & Gas - E&P	Energy	NYSE American	\N	2020-10-11 23:12:13.740492	2020-10-11 23:38:31.02582
ZBH	Zimmer Biomet Holdings Inc.	Zimmer Biomet Holdings Inc is a medical device company. It designs, manufactures, and markets orthopedic reconstructive implants, as well as supplies and surgical equipment for orthopedic surgery under brands like NexGen, Persona, Zimmer, and Polaris.	{Healthcare,"Medical Devices"}	{CNMD,NUVA,RTIX,JNJ,SYK,SNN,GMED,WMGI,XLV}	Medical Devices	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:13.761265	2020-10-11 23:38:31.02423
ZG	Zillow Group Inc.	Zillow Group Inc provides real estate and home-related information marketplaces on mobile and web-based platforms. It helps customers on each stage of the home lifecycle such as renting, buying, selling, financing, and home improvement.	{"Real Estate","Real Estate Services"}	{CSGP,RP,ELLI}	Real Estate Services	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:13.778425	2020-10-11 23:38:31.023007
AAME	Atlantic American Corporation	Atlantic American Corp provides property and casualty insurance and a variety of life and supplemental health products. It operates its business through its subsidiaries.	{"Financial Services","Insurance - Life"}	{UNM,AFL,CIA,TMK,CNO,FFG,NWLI}	Insurance - Life	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:11:59.251697	2020-10-11 23:38:31.028166
AAON	AAON Inc.	AAON Inc is a heating, ventilation and air conditioning equipment manufacturer. Its products include rooftop units, chillers, air-handling units, make-up air units, heat recovery units, condensing units and coils.	{"Basic Materials","Building Materials"}	{LII,LXU,AOS,SXI,UTX,IR,CUO}	Building Materials	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:11:59.282599	2020-10-11 23:38:31.028166
ABEV	Ambev S.A. American Depositary Shares (Each representing 1)	Ambev SA produces, distributes and sells beer, CSDs and other non-alcoholic and non-carbonated products in countries across the Americas. The company operates in Latin America North, Latin America South and Canada.	{"Consumer Defensive","Beverages - Brewers","Beverages - Alcoholic"}	{SAM,BF.B,BUD}	Beverages - Alcoholic	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:11:59.335482	2020-10-11 23:38:31.028166
AGD	Aberdeen Global Dynamic Dividend Fund	The Funds primary investment objective is to seek high current dividend income, more than 50% of which qualifies for the reduced federal income tax rates. The Fund also focuses on long-term growth of capital as secondary investment objective. The Fund expects to invest at least 80% of its net assets in the equity securities of domestic and foreign corporations that pay dividends. The Fund expects to invest in securities of both the U.S. and non - U.S. issuers. Investments will include primarily common stocks, although the Fund may, from time to time, also invest in real estate investment trusts, preferred stocks, exchange traded funds and securities convertible into exchangeable for common stocks, such as convertible debt.The Fund will attempt to provide high current dividend income and growth of capital using strategies focused on Dividend Capture, Value and Growth.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.518977	2020-10-11 23:38:31.028166
AINV	Apollo Investment Corporation	Apollo Investment Corp is a closed-end, non-diversified management investment company. It invests in various forms of debt investments including secured and unsecured loans, mezzanine investments, and/or equity in private middle-market companies.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:11:59.569137	2020-10-11 23:38:31.028166
AIV	Apartment Investment and Management Company	Apartment Investment & Management Co is a self-administered and self-managed real estate investment trust, or REIT, engaged in the ownership and operation of a diversified portfolio of apartment properties.	{"Real Estate","REIT - Residential",REITs}	{EQR,MAA,ESS,AVB,UDR,IRT,AMH}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:11:59.575353	2020-10-11 23:38:31.028166
ALLT	Allot Communications Ltd.	Allot Communications Ltd provides broadband solutions, focused on developing mobile, fixed and enterprise networks. Its deep packet inspection technology identifies and leverages the business intelligence in data networks and in network traffics.	{Technology,"Communication Equipment"}	{CSCO,ADTN,CALL,EXTR,FFIV,RDWR,ERIC,JNPR,CALX,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:11:59.630306	2020-10-11 23:38:31.028166
AMRH	Ameri Holdings Inc.	Ameri Holdings Inc provides SAP cloud and digital enterprise services. The company provides services in three categories namely Cloud services, Digital Services, and Enterprise Services.	{Technology,"Information Technology Services","Application Software"}	{CREE,PLAB,CEVA,AAOI,XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:11:59.704368	2020-10-11 23:38:31.028166
YY	YY Inc.	YY Inc is a social communication platform that engages users in real-time online group activities through voice, text and video. Its platform consists of YY Client, the YY.com and Duowan.com web portals and Mobile YY.	{Technology,"Internet Content & Information","Online Media"}	{SINA,BIDU,CYOU,RENN,XLK}	Online Media	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:13.749785	2020-10-11 23:38:31.025871
ZBRA	Zebra Technologies Corporation	Zebra Technologies Corp develops products for the automatic identification and data capture market. Its products find use in mobiles, computers, barcode scanners, RFID readers, specialty printers.	{Industrials,"Diversified Industrials","Industrial Products"}	{TACT,ALOT,CAJ,BRC,DHR,DOV,HPQ,XRX}	Industrial Products	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:13.766981	2020-10-11 23:38:31.031328
ZIV	VelocityShares Daily Inverse VIX Medium-Term ETN	The investment seeks to replicate, net of expenses, the inverse of the daily performance of the S&P 500 VIX Mid-Term Futures index.\n The index was designed to provide investors with exposure to one or more maturities of futures contracts on the VIX, which reflects implied volatility of the S&P 500 Index at various points along the volatility forward curve. The calculation of the VIX is based on prices of put and call options on the S&P 500 Index. The ETNs are linked to the daily inverse return of the index and do not represent an investment in the inverse of the VIX.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:13.793358	2020-10-11 23:38:31.025871
ZNH	China Southern Airlines Company Limited	China Southern Airlines Co Ltd in collaboration with its subsidiaries, functions in the airline industry. Its provides commercial airline services throughout China, Hong Kong and Macau regions, Southeast Asia and other parts of the world.	{Industrials,Airlines}	{CEA,DAL,UAL}	Airlines	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:13.803649	2020-10-11 23:38:31.028166
ZTR	Virtus Global Dividend & Income Fund Inc.	Virtus Global Dividend & Income Fund Inc is a closed-end diversified management investment company. The company's investment objective is to seek total return, consisting of capital appreciation and income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:13.818654	2020-10-11 23:38:31.028166
AABA	Altaba Inc.	Altaba Inc is an independent, non-diversified, closed-end management investment company. The Fund's objective is to seek to increase the price per share at which it trades relative to then-current values of its principal underlying assets, and others.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:11:59.247834	2020-10-11 23:38:31.02423
AAXJ	iShares MSCI All Country Asia ex Japan Index Fund	The investment seeks to track the investment results of the MSCI AC Asia ex Japan Index.\n The fund will invest at least 90% of its assets in the component securities of the index and in investments that have economic characteristics that are substantially identical to the component securities of the index. The index is a free float-adjusted market capitalization index designed to measure equity market performance of securities from the following 11 developed, emerging and frontier market countries or regions: China, Hong Kong, India, Indonesia, Malaysia, Pakistan, the Philippines, Singapore, South Korea, Taiwan and Thailand.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:11:59.301777	2020-10-11 23:38:31.02423
AEL	American Equity Investment Life Holding Company	American Equity Investment Life Holding Co is engaged in the development and sale of fixed index and fixed rate annuity products. The company through its wholly-owned life insurance subsidiaries issues fixed annuity and life insurance products.	{"Financial Services","Insurance - Life"}	{AFG,CIA,TMK,FFG,VOYA,NWLI}	Insurance - Life	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.483554	2020-10-11 23:38:31.02423
AM	Antero Midstream Partners LP representing limited partner interests	Antero Midstream Partners LP is engaged in providing midstream services to Antero under a long-term, fixed-fee contract. Its assets consist of gathering pipelines and compressor stations.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{TCP,EQM,SMLP}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:11:59.657559	2020-10-11 23:38:31.02423
ANAT	American National Insurance Company	American National Insurance Co is engaged in the business of life insurance, annuities, and property and casualty insurance. It also offers limited health insurance.	{"Financial Services","Insurance - Property & Casualty"}	{AFSI,AFG,WRB,ORI,THG,UFCS}	Insurance - Property & Casualty	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:11:59.730394	2020-10-11 23:38:31.02423
APTV	Aptiv PLC	Aptiv PLC primarily supplies components and systems that are used to build a vehicle's electrical system. The company also provides body controls, infotainment and connectivity systems for the automobile industry.	{"Consumer Cyclical","Auto Parts",Autos}	{MGA,GNTX,ALV,VC}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:11:59.785436	2020-10-11 23:38:31.02423
ARDC	Ares Dynamic Credit Allocation Fund Inc.	Ares Dynamic Credit Allocation Fund Inc is a closed-end management investment company. Its investment objective is to seek an attractive risk adjusted level of total return, through current income and through capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.813338	2020-10-11 23:38:31.02423
ATR	AptarGroup Inc.	AptarGroup, Inc. is a solution provider of packaging delivery solutions for the beauty, personal care, home care, prescription drug, consumer health care, injectables, food and beverage markets.	{"Consumer Cyclical","Packaging & Containers"}	{SLGN,SEE,BLL,BMS,GEF,OI,SON,PKG,CCK,BERY}	Packaging & Containers	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:11:59.915331	2020-10-11 23:38:31.02423
BBC	Virtus LifeSci Biotech Clinical Trials	The investment seeks investment results that correspond, before fees and expenses, to the price and yield performance of the LifeSci Biotechnology Clinical Trials Index.\n Under normal market conditions, the fund will invest not less than 80% of its assets in component securities of the index. The index seeks to track the performance of the common stock of U.S. exchange-listed biotechnology companies with a primary product offering ("lead drug") that is typically in a Phase 1, Phase 2 or Phase 3 clinical trial stage of development, but prior to receiving marketing approval. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.046613	2020-10-11 23:38:31.02423
FBNC	First Bancorp	First Bancorp is a bank holding company of First Bank. It is company is engaged in providing banking services such as acceptance of deposits and the lending funds to the customers for various purposes.	{"Financial Services","Banks - Regional - US",Banks}	{MBWM,PEBK,FCNCA}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:02.706871	2020-10-11 23:38:31.02582
ZION	Zions Bancorporation N.A.	Zions Bancorp is a bank holding company. The bank has over 400 branches across the United States and has more than $50 billion worth of assets. It provides banking services to small and midsize businesses as well as individuals.	{"Financial Services","Banks - Regional - US",Banks}	{GBCI,PMBC,FIBK,TCF,USB,WAL,NBHC,GBNK}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:13.78585	2020-10-11 23:38:31.028166
ZMLP	Direxion Zacks MLP High Income Index Shares	The investment seeks investment results, before fees and expenses, that track the price and yield performance of the Zacks MLP High Income Index.\n The fund, under normal circumstances, invests at least 80% of its net assets in the securities that comprise the index. The index is comprised of 25 securities selected from a universe of master limited partnerships ("MLPs") listed on U.S. exchanges. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.80329	2020-10-11 23:38:31.023007
ZTS	Zoetis Inc. Class A	Zoetis Inc is a developer and manufacturer of drugs providing medicines for animal health, and vaccines for livestock and companion animals. Its products are anti-infectives, vaccines, parasiticides, medicated feed additives and other pharmaceuticals.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{ICCC,LLY,PFE,SNY,MRK,HSKA,PETX,KIN,PAHC,XLV}	Drug Manufacturers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:13.820682	2020-10-11 23:38:31.024353
AAC	AAC Holdings Inc.	AAC Holdings Inc provides inpatient and outpatient substance abuse treatment services for individuals with drug and alcohol addiction. The company also performs drug testing and diagnostics laboratory services and provides physician services.	{Healthcare,"Medical Care","Health Care Providers"}	{SEM,ACHC,XLV}	Health Care Providers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:11:59.245489	2020-10-11 23:38:31.025871
AAWW	Atlas Air Worldwide Holdings	Atlas Air Worldwide Holdings Inc is engaged in providing outsourced aircraft and aviation operating services. It operates a fleet of approximately 747 freighters and offers aircraft for domestic, regional & international cargo & passenger applications.	{Industrials,"Airports & Air Services","Transportation & Logistics"}	{ATSG,BA,FDX}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:11:59.29578	2020-10-11 23:38:31.025871
ACHN	Achillion Pharmaceuticals Inc.	Achillion Pharmaceuticals Inc is a biotechnology company. It is engaged in the discovery and development of innovative small-molecule drug therapies for infectious diseases and immune system disorders.	{Healthcare,Biotechnology}	{GILD,NEOG,VRTX,GSK,BMY,JNJ,MRK,ABBV,ENTA,NNVC,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.404555	2020-10-11 23:38:31.025871
AKTX	Akari Therapeutics Plc	Akari Therapeutics PLC is a biopharmaceutical company in clinical stage. It is focused on developing inhibitors of acute and chronic inflammation which includes the complement system, the bioamine system and the eicosanoid system.	{Healthcare,Biotechnology}	{JNJ,LLY,GWPH,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:11:59.602038	2020-10-11 23:38:31.025871
AOD	Aberdeen Total Dynamic Dividend Fund	The primary investment objective is high current dividend income, with a secondary focus on long-term growth of capital. It will invest at least 80% of its net assets in equity securities of foreign and domestic companies that pay dividends. The Fund will attempt to provide high current dividend income and growth of capital using strategies focused on Dividend Capture, Value and Growth. It seeks to provide dividend income without regard to whether the dividends qualify for the reduced federal income tax rates applicable to qualified dividends under the Internal Revenue Code. It may invest up to 25% of its total assets in securities in emerging markets. It may engage in short sales of securities for investment or hedging purposes. It may also short sell individual stocks.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.750086	2020-10-11 23:38:31.025871
APC	Anadarko Petroleum Corporation	Anadarko Petroleum Corp is engaged in the exploration and production of oil and natural gas. Its asset base includes conventional and unconventional properties in the U.S. and deep-water oil and gas projects in the Gulf of Mexico and Africa.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{CHK,NFX,XOM,DVN,MUR,OXY,CVX,MRO,COP,HES}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:11:59.763043	2020-10-11 23:38:31.025871
AWI	Armstrong World Industries Inc	Armstrong World Industries Inc manufactures and sells and designs flooring products and ceiling systems for use in the construction and renovation of residential, commercial and institutional buildings.	{"Basic Materials","Building Materials"}	{PATK,USG,MAS,MHK,FBHS}	Building Materials	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:11:59.968851	2020-10-11 23:38:31.025871
AXJV	iShares Edge MSCI A Min Vol Asia ex Japan	The investment seeks to track the investment results of the MSCI AC Asia ex Japan Minimum Volatility (USD) Index.\n The fund generally will invest at least 90% of its assets in the component securities of the index and in investments that have economic characteristics that are substantially identical to the component securities of the index. The index is designed by selecting securities from the MSCI AC Asia ex Japan Index (the "parent index"), which is a capitalization-weighted index, and then follows a rules-based methodology to optimize the index and determine weights for securities in the index that seeks to minimize total risk of the parent index.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.980188	2020-10-11 23:38:31.025871
BBH	VanEck Vectors Biotech ETF	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the MVIS® US Listed Biotech 25 Index.\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The Biotech Index includes common stocks and depositary receipts of U.S. exchange-listed companies in the biotechnology sector. Such companies may include medium-capitalization companies and foreign companies that are listed on a U.S. exchange. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:00.05511	2020-10-11 23:38:31.025871
BDCL	2xLeveraged Long Exchange Traded Access Securities (E-TRACS) Linked to the Wells Fargo Business Development Company Index due May 24 2041	The investment seeks to replicate, net of expenses, twice the performance of the Wells Fargo Business Development Company Index.\n The index is a float adjusted, capitalization-weighted index that is intended to measure the performance of all Business Development Companies (“BDC”) that are listed on the New York Stock Exchange or NASDAQ and satisfy specified market capitalization and other eligibility requirements. The BDC business model is to lend to small and midsized companies at high yield equivalent rates while also at times taking equity stakes in such companies.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.109377	2020-10-11 23:38:31.025871
CHAD	Direxion Daily CSI 300 China A Shares Bear 1X Shares	The investment seeks daily investment results, before fees and expenses, of 100% of the inverse of the daily performance of the CSI 300 Index.\n The fund, under normal circumstances, invests in swap agreements, futures contracts, short positions or other financial instruments that, in combination, provide inverse (opposite) or short exposure to the index equal to at least 80% of the fund's net assets (plus borrowing for investment purposes). The index is a modified free-float market capitalization weighted index comprised of the largest and most liquid stocks in the Chinese A-share market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.836528	2020-10-11 23:38:31.025871
CHK	Chesapeake Energy Corporation	Chesapeake Energy Corp is a natural gas and oil exploration, development, acquisition and production company. It also owns an oil and natural gas marketing business. Its portfolio includes unconventional oil and natural gas assets in U.S. onshore plays.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{CRZO,NFX,FCX,SLB,DVN,APC,APA,EOG,MRO}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:00.871241	2020-10-11 23:38:31.025871
ACC	American Campus Communities Inc	American Campus Communities Inc is a real estate investment trust that owns, manages and develops student housing properties in the United States in terms of beds owned, developed, and under management.	{"Real Estate","REIT - Residential",REITs}	{CUZ,GOV}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:11:59.381795	2020-10-11 23:38:31.023428
AKCA	Akcea Therapeutics Inc.	Akcea Therapeutics Inc is a late stage biopharmaceutical company focused on developing and commercializing drugs to treat patients with serious cardiometabolic diseases caused by lipid disorders.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.592624	2020-10-11 23:38:31.023428
AL	Air Lease Corporation Class A	Air Lease Corp is a U.S based aircraft leasing company. Its main source of revenue comes through the leasing of narrow and wide-bodied jet aircrafts primarily in the European market.	{Industrials,"Rental & Leasing Services","Consulting & Outsourcing"}	{WLFC,TGH,AYR,AER,CAI,FLY,ACY}	Consulting & Outsourcing	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.606365	2020-10-11 23:38:31.023428
AMRC	Ameresco Inc. Class A	Ameresco, Inc. is an independent provider of comprehensive energy services including energy efficiency, infrastructure upgrades, energy security and resilience, asset sustainability and renewable energy solutions for businesses and organizations.	{Industrials,"Engineering & Construction"}	{ABM,HON,UTX,EXC,CVX,KBR}	Engineering & Construction	Industrials	New York Stock Exchange	\N	2020-10-11 23:11:59.703828	2020-10-11 23:38:31.023428
ARCM	Arrow Reserve Capital Management	The investment seeks to preserve capital while maximizing current income.\n The fund invests in a variety of domestic fixed income securities. It will invest in fixed income instruments with a dollar-weighted average effective maturity of 0 to 2 years issued by U.S. Dollar-denominated issuers, including mortgage- or asset-backed securities, rated Baa- or higher by Moody's, or equivalently rated by S&P or Fitch, or, if unrated, determined by the Sub-Advisor. It may also invest in interest rate futures and forwards. The Sub-Advisor attempts to maximize income by identifying securities that offer an acceptable yield for a given level of credit risk and maturity.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:11:59.808276	2020-10-11 23:38:31.023428
ASG	Liberty All-Star Growth Fund Inc.	Liberty All Star Growth Fund Inc is a United States based diversified, closed-end management investment company. Its investment objective is to seek long-term capital appreciation through investment in a diversified portfolio of equity securities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.855945	2020-10-11 23:38:31.023428
BHLB	Berkshire Hills Bancorp Inc.	Berkshire Hills Bancorp Inc is a holding company for Berkshire Bank and Berkshire Insurance Group. It offers deposit, lending, investment, wealth management, and insurance services.	{"Financial Services","Banks - Regional - US",Banks}	{HIFS,BRKL,PBCT,SIFI,NBN,WEBK,UBNK,WNEB}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.201991	2020-10-11 23:38:31.023428
BKEP	Blueknight Energy Partners L.P. L.L.C.	Blueknight Energy Partners LP provides terminalling, gathering, transportation, and storage services for companies that produce, distribute, and market crude oil and liquid asphalt cement.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{XOM,EPD,PAA,EEP,COP,MMP,HEP,NS,SEMG}	Oil & Gas - Midstream	Energy	NASDAQ Global Market	\N	2020-10-11 23:12:00.267924	2020-10-11 23:38:31.023428
BSJN	Invesco BulletShares 2023 High Yield Corporate Bond	The investment seeks investment results that correspond generally to the performance, of a high yield corporate bond index called the Nasdaq BulletShares® USD High Yield Corporate Bond 2023 Index.\n The fund will invest at least 80% of its total assets in component securities that comprise the index. The index is designed to represent the performance of a held-to-maturity portfolio of U.S. dollar-denominated high yield corporate bonds with effective maturities in 2023. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.493338	2020-10-11 23:38:31.023428
BZF	WisdomTree Brazilian Real Strategy Fund	The investment seeks to achieve total returns reflective of both money market rates in Brazil available to foreign investors and changes in value of the Brazilian real relative to the U.S. dollar.\n The fund normally invests at least 80% of its net assets, plus the amount of any borrowings for investment purposes, in investments whose combined performance is tied economically to Brazil. It generally will maintain a weighted average portfolio maturity of 90 days or less with respect to the money market securities in its portfolio. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.574209	2020-10-11 23:38:31.023428
CDK	CDK Global Inc.	CDK Global Inc provides integrated information technology and digital marketing solutions to the automotive retail industry. The Company offers their services to Auto, Truck, Motorcycle, Marine, Recreational Vehicle & Heavy Equipment dealers.	{Technology,"Software - Application","Application Software"}	{ULTI,MDSO,TRU,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:00.729944	2020-10-11 23:38:31.023428
CIDM	Cinedigm Corp	Cinedigm Corp is engaged in the media industry in the United States. It is a distributor and aggregator of the independent movie, television and other short-form content.	{"Consumer Cyclical","Media - Diversified",Entertainment}	{AMCX,VIAB,FOXA,MSGN}	Entertainment	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:00.915803	2020-10-11 23:38:31.023428
CJNK	SPDR ICE BofAML Crossover Corporate Bond	The investment seeks investment results that, before fees and expenses, correspond generally to the price and yield performance of the ICE BofAML US Diversified Crossover Corporate Index.\n The fund invests substantially all, but at least 80%, of its total assets in the securities comprising the index or in securities that the Adviser determines have economic characteristics that are substantially identical to the economic characteristics of the securities that comprise the index. The index is designed to measure the performance of U.S. dollar denominated BBB and BB corporate debt publicly issued in the U.S. domestic market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.956597	2020-10-11 23:38:31.023428
CLIR	ClearSign Combustion Corporation	ClearSign Combustion Corp through its Duplex and Electrodynamic Combustion Control technology develops high-performance combustion systems. The company offers Duplex Plug and Play burner for use in refinery heaters and steam generation engines.	{Industrials,"Pollution & Treatment Controls","Industrial Products"}	{HYGS,ELSE,SMIT,AME,EMR,GE,HON,CVA,BWXT}	Industrial Products	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:00.987367	2020-10-11 23:38:31.023428
AMED	Amedisys Inc	Amedisys Inc is a provider of home health services to the chronic, co-morbid, aging American population. The company's operations include servicing patients through its segments home health, hospice and personal care.	{Healthcare,"Medical Care","Health Care Providers"}	{LHCG,BIOS,NHC,CHE,XLV}	Health Care Providers	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:11:59.674151	2020-10-11 23:38:31.023007
AOA	iShares Core Aggressive Allocation	The investment seeks to track the investment results of the S&P Target Risk Aggressive Index composed of a portfolio of underlying equity and fixed income funds intended to represent an aggressive target risk allocation strategy.\n The fund is a fund of funds and seeks its investment objective by investing primarily in other iShares Underlying Funds that themselves seek investment results corresponding to their own respective underlying indexes. It generally will invest at least 90% of its assets in the component securities of the underlying index. The index measures the performance of the S&P Dow Jones Indices LLC proprietary allocation model.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.74992	2020-10-11 23:38:31.023007
ARCB	ArcBest Corporation	ArcBest Corp is a transportation company. The company provides freight transportation services and solutions. It operates through three segments namely Asset-Based, ArcBest and FleetNet.	{Industrials,Trucking,"Transportation & Logistics"}	{JBHT,FWRD,ODFL,ULH,SAIA,FDX,RRTS,YRCW}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:11:59.805936	2020-10-11 23:38:31.023007
ASTC	Astrotech Corporation	Astrotech Corp is a science and technology development company. The firm invents, acquires, and commercializes technological innovations sourced from internal research, universities, laboratories, and research institutions.	{Industrials,"Aerospace & Defense"}	{LMT,BA,NOC,AJRD}	Aerospace & Defense	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:11:59.882142	2020-10-11 23:38:31.023007
AXJL	WisdomTree Asia Pacific ex-Japan Total Dividend Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree Asia Pacific ex-Japan Index.\n Under normal circumstances, at least 95% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a fundamentally weighted index that is comprised of dividend-paying companies in the Asia Pacific region, excluding Japan. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.980412	2020-10-11 23:38:31.023007
BEDU	Bright Scholar Education Holdings Limited American Depositary Shares each representing one Class A	Bright Scholar Education Holdings Ltd operates international and bilingual K-12 schools in the People's republic of China (PRC). The company offers a broad range of internationally-accredited curriculum in its international schools.	{"Consumer Defensive","Education & Training Services",Education}	{BEDU}	Education	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:00.125031	2020-10-11 23:38:31.023007
BIBL	Inspire 100	The investment seeks to replicate investment results that generally correspond, before fees and expenses, to the performance of the Inspire 100 Index.\n The fund generally will invest at least 80% of its total assets in the component securities of the Inspire 100 Index. The index provider selects domestic large capitalization equity securities using the index provider's Inspire Impact Score®, a proprietary selection methodology that is designed to assign a score to a particular security based on the security's alignment with biblical values and the positive impact that company has on the world through various environmental, social and governance criterion.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.214652	2020-10-11 23:38:31.023007
BOS	DB Base Metals Short Exchange Traded Notes due June 1 2038	The investment seeks to track the price and yield performance, before fees and expenses, of the Deutsche Bank Liquid Commodity index - Optimum Yield Industrial Metals Excess Return.\n The fund allows investors to take a short view on the performance of the industrial metals sector. The index is a rules-based index composed of futures contracts on some of the most liquid and widely used base metals, aluminum, zinc and copper.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.40649	2020-10-11 23:38:31.023007
BRKR	Bruker Corporation	Bruker Corp is a developer, manufacturer and distributor of scientific instruments and analytical and dignostic solutions that enable customers to explore life and materials at microscopic, molecular and cellular levels.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{ILMN,WAT,AME,A,BIO,DHR,GE,TMO,PKI,AXDX,XLV}	Medical Diagnostics & Research	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:00.457383	2020-10-11 23:38:31.023007
BRZU	Direxion Daily Brazil Bull 3X Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the daily performance of the MSCI Brazil 25/50 Index.\n The fund invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, exchange-traded funds ("ETFs") that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index is designed to measure the performance of the large- and mid-capitalization segments of the Brazilian equity market, covering approximately 85% of the free float-adjusted market capitalization of Brazilian issuers. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.475431	2020-10-11 23:38:31.023007
BSJM	Invesco BulletShares 2022 High Yield Corporate Bond	The investment seeks investment results that correspond generally to the performance, before the fund's fees and expenses, of a high yield corporate bond index called the Nasdaq BulletShares® USD High Yield Corporate Bond 2022 Index.\n The fund will invest at least 80% of its total assets in component securities that comprise the index. The index is designed to represent the performance of a held-to-maturity portfolio of U.S. dollar-denominated high yield corporate bonds with effective maturities in the year 2022. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.494766	2020-10-11 23:38:31.023007
CAAP	Corporacion America Airports SA	Corporacion America Airports SA acquires, develops and operates airport concessions. The company is the largest private sector airport concession operator in the world based on the number of airports under management.	{Industrials,"Airports & Air Services","Transportation & Logistics"}	{}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:00.581227	2020-10-11 23:38:31.023007
CASY	Caseys General Stores Inc.	Casey's General Stores Inc operates convenience stores under the Caseys General Store name in the United States. It provides self-service gasoline, grocery items, and processed foods such as pizza, donuts, and sandwiches.	{"Consumer Defensive","Grocery Stores","Retail - Defensive"}	{IMKTA,KR,WMK,MUSA,TA}	Retail - Defensive	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:00.63621	2020-10-11 23:38:31.023007
AQXP	Aquinox Pharmaceuticals Inc.	Aquinox Pharmaceuticals Inc is a clinical-stage pharmaceutical company discovering and developing targeted therapeutics in disease areas of inflammation and immuno-oncology.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{GILD,JNJ,AEZS,NYMX,XLV}	Drug Manufacturers	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:11:59.800739	2020-10-11 23:38:31.028166
ASEA	Global X FTSE Southeast Asia	The investment seeks to provide investment results that correspond generally to the price and yield performance, before fees and expenses, of the FTSE/ASEAN 40 Index ("underlying index").\n The fund invests at least 80% of its total assets in the securities of the underlying index and in American Depositary Receipts ("ADRs") and Global Depositary Receipts ("GDRs") based on the securities in the underlying index. The underlying index tracks the equity performance of the 40 largest and most liquid companies in the five Association of Southeast Asian Nations ("ASEAN") regions: Singapore, Malaysia, Indonesia, Thailand and the Philippines. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.853536	2020-10-11 23:38:31.028166
BC	Brunswick Corporation	Brunswick Corp is engaged in designing, manufacturing and marketing of recreation products including marine engines, boats, fitness and billiards equipment. Its products include sterndrive, yachts, deck boats, marine parts and cardiovascular equipment.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{TWIN,BGG,BWL.A,MPX,MBUU}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:00.074458	2020-10-11 23:38:31.028166
BGI	Birks Group Inc.	Birks Group Inc and its subsidiaries activities include design, manufacture and retail sale of prestige jewelry, timepieces and gift ware.	{"Consumer Cyclical","Luxury Goods","Retail - Apparel & Specialty"}	{TIF,DGSE,SIG}	Retail - Apparel & Specialty	Consumer Cyclical	NYSE American	\N	2020-10-11 23:12:00.167394	2020-10-11 23:38:31.028166
BOTZ	Global X Robotics & Artificial Intelligence ETF	The investment seeks to provide investment results that correspond generally to the price and yield performance, before fees and expenses, of the Indxx Global Robotics & Artificial Intelligence Thematic Index.\n The fund invests at least 80% of its total assets in the securities of the underlying index. The underlying index is designed to provide exposure to exchange-listed companies in developed markets that are involved in the development of robotics and/or artificial intelligence as defined by Indxx, the provider of the underlying index. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:00.408844	2020-10-11 23:38:31.028166
BSJL	Invesco BulletShares 2021 High Yield Corporate Bond	The investment seeks investment results that correspond generally to the performance, before the fund's fees and expenses, of a high yield corporate bond index called the Nasdaq BulletShares® USD High Yield Corporate Bond 2021 Index.\n The fund will invest at least 80% of its total assets in component securities that comprise the index. The index is designed to represent the performance of a held-to-maturity portfolio of U.S. dollar-denominated high yield corporate bonds with effective maturities in the year 2021. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.494543	2020-10-11 23:38:31.028166
BTA	BlackRock Long-Term Municipal Advantage Trust of Beneficial Interest	Blackrock Long-term Municipal AdtgTrust is a non-diversified investment fund with an investment objective to provide current income exempt from regular U.S. federal income tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.512093	2020-10-11 23:38:31.028166
BYLD	iShares Yield Optimized Bond	The investment seeks to track the investment results of the Morningstar® U.S. Bond Market Yield-Optimized IndexSM.\n The fund invests primarily in underlying funds that themselves seek investment results corresponding to their own underlying indexes. It also invests in a combination of other ETFs (including other iShares ETFs), cash and cash equivalents, including shares of money market funds advised by BFA or its affiliates. The index is a broadly diversified fixed income index that seeks to deliver current income. It will invest at least 90% of its assets in the component securities (including indirect investments through the underlying funds) of the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.56952	2020-10-11 23:38:31.028166
CCR	CONSOL Coal Resources LP representing limited partner interests	CONSOL Coal Resources LP is engaged in the bituminous coal operations in Pennsylvania. It has ownership interest and management and control rights in CONSOL Energy's Pennsylvania mining complex including Bailey, Enlow Fork and Harvey.	{"Basic Materials",Coal}	{ARLP,FELP,NRP}	Coal	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:00.71228	2020-10-11 23:38:31.028166
CHGG	Chegg Inc.	Chegg Inc is a student-first connected learning platform empowering students to take control of their education to save time and money. It offers its products and services under Chegg services and required materials categories.	{"Consumer Defensive","Education & Training Services",Education}	{AMZN,BKS,JW.A,GOOGL}	Education	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:00.857655	2020-10-11 23:38:31.028166
CINF	Cincinnati Financial Corporation	Cincinnati Financial Corp is a property casualty insurance company. It provides coverage for commercial casualty, commercial property, commercial auto, workers' compensation, personal auto and life insurance.	{"Financial Services","Insurance - Property & Casualty"}	{ERIE,SIGI,HALL,AFG,ORI,TRV,UFCS}	Insurance - Property & Casualty	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.930578	2020-10-11 23:38:31.028166
CLNE	Clean Energy Fuels Corp.	Clean Energy Fuels Corp is a natural gas marketer and retailer operating in the United States and Canada. The company supplies compressed natural gas, liquefied natural gas, and renewable natural gas as an alternative fuel for vehicles.	{Energy,"Oil & Gas Refining & Marketing","Oil & Gas - Refining & Marketing"}	{FGP,SRE,RSG,GE,MDU,UGI,VVC,MUSA,CAPL,SUN}	Oil & Gas - Refining & Marketing	Energy	Nasdaq Global Select	\N	2020-10-11 23:12:01.00513	2020-10-11 23:38:31.028166
CMCO	Columbus McKinnon Corporation	Columbus McKinnon Corp is a designer, manufacturer and marketer of hoists, rigging tools, cranes, actuators, and other material handling products serving commercial and industrial end-user markets.	{Industrials,"Farm & Construction Equipment","Farm & Construction Machinery"}	{GENC,TEX,CAT,KAI,MTW}	Farm & Construction Machinery	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:01.043722	2020-10-11 23:38:31.028166
COMM	CommScope Holding Company Inc.	CommScope Holding Co Inc provides infrastructure solutions for communication networks. The company's portfolio of network infrastructure includes wireless and fiber optic solutions. Its segments are CommScope Connectivity and CommScope Mobility Solutions.	{Technology,"Communication Equipment"}	{ERIC,APH,GLW,EMR,BDC,TEL,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:01.162594	2020-10-11 23:38:31.028166
ARKG	ARK Genomic Revolution Multi-Sector	The investment seeks long-term growth of capital.\n The fund is an actively-managed exchange-traded fund ("ETF") that will invest under normal circumstances primarily (at least 80% of its assets) in domestic and foreign equity securities of companies across multiple sectors, including healthcare, information technology, materials, energy and consumer discretionary, that are relevant to the fund's investment theme of the genomics revolution ("Genomics Revolution Companies"). It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:11:59.82436	2020-10-11 23:38:31.028166
AUBN	Auburn National Bancorporation Inc.	Auburn National Bancorp Inc is the holding company for AuburnBank. It offers checking, savings, transaction deposit accounts and certificates of deposit, and is an active residential mortgage lender in its service area.	{"Financial Services","Banks - Regional - US",Banks}	{CTBI,CIZN,RNST,SFBS,FUSB,BXS,PFBX}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:11:59.933401	2020-10-11 23:38:31.028166
AXP	American Express Company	American Express Co is a payments, network and travel company. It provides charge and credit card products, travel services, network services, stored value products, loans, and other products and services to businesses and individuals.	{"Financial Services","Credit Services"}	{COF,BAC,JPM,MA,DFS,V,C}	Credit Services	Financial Services	New York Stock Exchange	\N	2020-10-11 23:11:59.98375	2020-10-11 23:38:31.028166
BBL	BHP Billiton plc Sponsored ADR	BHP Billiton PLC is a producer of major commodities. The company's operations involved exploration, development and production of oil and gas, mining of copper, iron ore, and metallurgical coal and thermal (energy) coal.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{TECK,FCX,RIO,XOM,CVX,RDS.A,VALE}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:00.055877	2020-10-11 23:38:31.028166
CARB	Carbonite Inc.	Carbonite Inc provides data protection solutions including cloud, hybrid, and on-premise backup and restores, disaster recovery as a service and email archiving. It offers annual and multi-year cloud backup plans for multi-year subscriptions.	{Technology,"Software - Infrastructure","Application Software"}	{MSFT,AAPL,SYMC,JCOM,EQIX,GOOGL,INAP,XLK}	Application Software	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:00.622179	2020-10-11 23:38:31.028166
CEPU	Central Puerto S.A. American Depositary Shares (each represents ten)	Central Puerto SA is a private sector power generation company in Argentina. The company is engaged in electric power generation and commercialization. The company's operating segments are Production of electric power and Natural gas distribution.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:00.782228	2020-10-11 23:38:31.028166
CEV	Eaton Vance California Municipal Income Trust Shares of Beneficial Interest	Eaton Vance CA Muni Income Trust is a non-diversified closed-end investment company. The company's investment objective is to provide current income exempt from regular federal income tax and taxes in its specified state.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE American	\N	2020-10-11 23:12:00.797106	2020-10-11 23:38:31.028166
CGA	China Green Agriculture Inc.	China Green Agriculture Inc is engaged in the research, development, production, and sale of fertilizers and distribution of agricultural products in the People's Republic of China. Its fertilizer business is the major revenue driver for the firm.	{"Basic Materials","Agricultural Inputs",Agriculture}	{SMG,CF,UAN}	Agriculture	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:00.824198	2020-10-11 23:38:31.028166
CNHX	CSOP MSCI China A International Hedged	The investment seeks to track the performance of the MSCI China A International with CNH 100% Hedged to USD Index.\n The index is designed to track the equity market performance of Chinese securities (A-Shares) listed on both the Shanghai Stock Exchange and the Shenzhen Stock Exchange, while at the same time neutralizing exposure to the fluctuations of the RMB relative to the USD. The fund will typically invest at least 80% of its total assets in the securities of the index. It may invest the remainder of its assets in investments that are not included in the index, but which the Adviser believes will help the fund track the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.103612	2020-10-11 23:38:31.028166
COMG	GraniteShares S&P GSCI Commodity Broad Strategy No K-1	The investment seeks to provide long-term capital appreciation, primarily through exposure to commodity futures markets.\n The fund is an actively managed exchange-traded fund ("ETF") that seeks to provide long-term capital appreciation, primarily through exposure to commodity futures markets. The fund's investment strategy is based in part on the S&P GSCI Index (the "GSCI Benchmark"), which is designed to be measure of commodity market performance over time. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.162061	2020-10-11 23:38:31.028166
CROC	ProShares UltraShort Australian Dollar	The investment seeks results for a single day that match (before fees and expenses) two times the inverse of the daily performance of the U.S. dollar price of the Australian dollar.\nThe fund obtains short exposures to its benchmark through futures contracts on its underlying currency. It may also invest in forward contracts if the market for a specific futures contract experiences emergencies (e.g., natural disaster, terrorist attack or an act of God) or disruptions (e.g., a trading halt or a flash crash) that would prevent or make it impractical for the fund to obtain the appropriate amount of investment exposure to the affected futures contracts directly.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.267432	2020-10-11 23:38:31.028166
CSV	Carriage Services Inc.	Carriage Services Inc is a provider of death care services and merchandise in the United States. It operates Funeral Home Operations and Cemetery Operations. It provides funeral services and sell related merchandise, such as caskets and urns.	{"Consumer Cyclical","Personal Services"}	{MATW,SCI,STON}	Personal Services	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:01.318343	2020-10-11 23:38:31.02582
CUT	Invesco MSCI Global Timber	The investment seeks to track the investment results (before fees and expenses) of the MSCI ACWI IMI Timber Select Capped Index (the underlying index).\n The fund generally will invest at least 90% of its total assets in the securities that comprise the underlying index, as well as American depositary receipts ("ADRs") and global depositary receipts ("GDRs") that represent securities in the underlying index. The underlying index is comprised of equity securities of companies that are primarily engaged in the ownership and management of forests and timberlands and the production of finished products that use timber as a raw material. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.396548	2020-10-11 23:38:31.028166
ATV	Acorn International Inc. ADS	Acorn International Inc is a marketing and branding company. It is engaged in developing, promoting and selling a diverse portfolio of proprietary-branded products, as well as well-established and promising new products from third parties.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{OSTK,EVLV,SGOC}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:11:59.927169	2020-10-11 23:38:31.02582
BIL	SPDR Bloomberg Barclays 1-3 Month T-Bill	The investment seeks to provide investment results that correspond generally to the price and yield performance of the Bloomberg Barclays 1-3 Month U.S. Treasury Bill Index.\n The fund invests substantially all, but at least 80%, of its total assets in the securities comprising the index or in securities that the Adviser determines have economic characteristics that are substantially identical to the economic characteristics of the securities that comprise the index. The index measures the performance of public obligations of the U.S. Treasury that have a remaining maturity of greater than or equal to 1 month and less than 3 months. It is non-diversified.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{}	Brokers & Exchanges	Financial Services	NYSE Arca	\N	2020-10-11 23:12:00.22386	2020-10-11 23:38:31.02582
BITA	Bitauto Holdings Limited American Depositary Shares (each representing one)	Bitauto Holdings Ltd is a provider of internet content and marketing services. The company operates in three main segments: Advertising and subscription business, Transaction service business and Digital marketing solutions business.	{Technology,"Internet Content & Information","Online Media"}	{SINA,BIDU,WUBA,ATHM,XLK}	Online Media	Technology	New York Stock Exchange	\N	2020-10-11 23:12:00.231359	2020-10-11 23:38:31.02582
BOIL	ProShares Ultra Bloomberg Natural Gas	The investment seeks results for a single day that match (before fees and expenses) two times the daily performance of the Bloomberg Natural Gas SubindexSM.\nThe fund seeks to meet their investment objectives by investing under normal market conditions in Natural Gas futures contracts. It may also invest in swaps if the market for a specific futures contract experiences emergencies (e.g., natural disaster, terrorist attack, or an act of God) or disruptions (e.g., a trading halt or a flash crash) that would prevent or make it impractical for such fund to obtain the appropriate amount of investment exposure to the affected Natural Gas futures contracts directly.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.391879	2020-10-11 23:38:31.02582
BRX	Brixmor Property Group Inc.	Brixmor Property Group Inc is a real estate investment trust. The company owns and operates a portfolio of grocery-anchored community and neighborhood shopping centers across the US. It leases its rentable areas to retailers, restaurants, and theatres.	{"Real Estate","REIT - Retail",REITs}	{PSA,WY,BX,CUBE}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:00.472395	2020-10-11 23:38:31.02582
BUZ	Buzz US Sentiment Leaders	The investment seeks investment results that correspond to the BUZZ NextGen AI US Sentiment Leaders Index.\n The fund will invest at least 90% of its net assets in securities that comprise the underlying index. It employs a "passive management" - or indexing - investment approach designed to track the performance of the underlying index. The underlying index utilizes a rules-based quantitative methodology developed by BUZZ Indexes Inc. (the "index provider"), which is designed to identify the U.S. common stocks with the most "positive insights" collected from social media networks. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.536636	2020-10-11 23:38:31.02582
BWZ	SPDR Bloomberg Barclays Short Term International Treasury Bond	The investment seeks to provide investment results that correspond generally to the Bloomberg Barclays 1-3 Year Global Treasury ex-US Capped Index.\n The fund invests substantially all, but at least 80%, of its total assets in the securities comprising the index or in securities that the Adviser determines have economic characteristics that are substantially identical to the economic characteristics of the securities that comprise the index. The index is designed to measure the performance of fixed-rate local currency sovereign debt of investment grade countries outside the United States that have remaining maturities of 1-3 years. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.554383	2020-10-11 23:38:31.02582
CATH	Global X S&P 500 Catholic Values ETF	The investment seeks investment results that correspond generally to the price and yield performance, of the S&P 500® Catholic Values Index.\n The fund invests at least 80% of its total assets in the securities of the underlying index. The underlying index is based on the S&P 500® Index, and generally comprises approximately 500 or less U.S. listed common stocks. From this starting universe, constituents are screened to exclude companies involved in activities which are perceived to be inconsistent with Catholic values as outlined in the Socially Responsible Investment Guidelines of the United States Conference of Catholic Bishops.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:00.647945	2020-10-11 23:38:31.02582
CHEKW	Check-Cap (Warrant)	Check-Cap Ltd. clinical stage medical diagnostics company, which engages in developing gastrointestinal imaging devices. It develops capsule-based system that utilizes low-dose X-rays to detect polyps, masses and colorectal cancer screening. The company was founded by Yoav Kimchy in 2005 and is headquartered in Isfiya, Israel.	{}	{}	Medical Specialties	Health Technology	NasdaqCM	\N	2020-10-11 23:12:00.852715	2020-10-11 23:38:31.02582
CHMG	Chemung Financial Corp	Chemung Financial Corp provides financial services, including demand, savings and time deposits, commercial, residential and consumer loans, and wealth management services.	{"Financial Services","Banks - Regional - US",Banks}	{AMNB,WTFC,FISI,MCBC,NBTB,PNFP,SBNY,TMP,CBU}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.877627	2020-10-11 23:38:31.02582
CLS	Celestica Inc.	Celestica Inc is a US-based electronic manufacturing service company. It delivers supply chain solutions to customers in the communications, industrial, aerospace and defense, healthcare, solar, semiconductor equipment and other sectors.	{Technology,"Contract Manufacturers","Computer Hardware"}	{FLEX,PLXS,SMTX,IEC,JBL,BHE,CTS,FN,SANM,NTP,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:01.020675	2020-10-11 23:38:31.02582
CNX	CNX Resources Corporation	CNX Resources Corp is a Pittsburgh-based energy producer. The company is engaged in natural gas exploration, development and production with operations centered in the major shale formations of the Appalachian basin.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{ARLP,CLD,NRP}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:01.126615	2020-10-11 23:38:31.02582
CORR	CorEnergy Infrastructure Trust Inc.	CorEnergy Infrastructure Trust Inc is engaged in acquiring and financing midstream and downstream real estate assets within the U.S. energy infrastructure sector.	{"Real Estate","REIT - Diversified",REITs}	{}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:01.177075	2020-10-11 23:38:31.02582
BEL	Belmond Ltd. Class A	Belmond Ltd is a luxury hotel company. It is an adventure travel operator with exposure to both mature and emerging national economies. The company also operates tourist trains along with luxury hotel properties.	{"Consumer Cyclical",Lodging,"Travel & Leisure"}	{RCL,CUK,CCL,RLH,H,FUN,MAR,IHG}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:00.12924	2020-10-11 23:38:31.02423
BKH	Black Hills Corporation	Black Hills Corp is a vertically-integrated energy company. The company operates in segments including regulated Electric Utilities segment, regulated Gas Utilities segment, Power Generation segment, and Mining Segment.	{Utilities,"Utilities - Diversified","Utilities - Regulated"}	{LNT,AEE,CMS,MDU,AVA,EXC,CNP,NWE,DUK,XEL,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:00.270765	2020-10-11 23:38:31.02423
CARS	Cars.com Inc.	Cars.com Inc is an online destination for buying and selling of new and used vehicles. The company also operates Auto.com, DealerRater.com, NewCars.com and PickupTrucks.com, websites directed towards different consumer segments.	{"Consumer Cyclical","Auto & Truck Dealerships",Autos}	{RENN}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:00.62685	2020-10-11 23:38:31.02423
CBND	SPDR Bloomberg Barclays Corporate Bond	The investment seeks investment results that, before fees and expenses, correspond generally to the price and yield performance of an index that tracks the Bloomberg Barclays US Corporate Bond Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index or in securities that the Adviser determines have economic characteristics that are substantially identical to the economic characteristics of the securities that comprise the index. The index is designed to measure the performance of the investment grade corporate bond market. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.671197	2020-10-11 23:38:31.02423
CCOI	Cogent Communications Holdings Inc.	Cogent Communications Holdings Inc is an enterprise Internet service provider offering Internet access and data transport using Internet Protocol. It mainly serves the small and medium sized businesses around the world.	{"Communication Services","Telecom Services"}	{CSCO,TEUM}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.71132	2020-10-11 23:38:31.02423
CERS	Cerus Corporation	Cerus Corp is a biomedical products company. It is engaged in developing and commercializing the INTERCEPT Blood System to enhance blood safety.	{Healthcare,Biotechnology}	{BTX,HAE,TLGT,KOOL,VICL,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:00.791898	2020-10-11 23:38:31.02423
CN	Xtrackers MSCI All China Equity	The investment seeks investment results that correspond to the performance, before fees and expenses, of the MSCI China All Shares Index.\n The fund will normally invest at least 80% of its total assets in securities of issuers that comprise either directly or indirectly the underlying index or securities with economic characteristics similar to those included in the underlying index. The underlying index is designed to capture large- and mid-capitalization representation across all China securities listed in Hong Kong, Shanghai and Shenzhen. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.081798	2020-10-11 23:38:31.02423
CTRP	Ctrip.com International Ltd.	Ctrip.com International Ltd provides travel related services. Its areas of interest include reservation, transportation ticketing, packaged-tour, corporate travel management services, as well as Internet-related advertising.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{EXPE,TRIP}	Travel & Leisure	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:01.359743	2020-10-11 23:38:31.02423
CVRR	CVR Refining LP Representing Limited Partner Interests	CVR Refining LP is an independent downstream energy limited partnership with refining & related logistics assets that operates in the mid-continent region. Its business includes a complex full coking medium-sour crude oil refinery in Coffeyville, Kansas.	{Energy,"Oil & Gas Refining & Marketing","Oil & Gas - Refining & Marketing"}	{VLO,CVI,MPC,HFC,PSX,DK}	Oil & Gas - Refining & Marketing	Energy	New York Stock Exchange	\N	2020-10-11 23:12:01.421239	2020-10-11 23:38:31.02582
CWI	SPDR MSCI ACWI ex-US	The investment seeks investment results that, before fees and expenses, correspond generally to the total return performance of the MSCI All Country World Index ex USA Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index and in depositary receipts based on securities comprising the index. The index is a free float-adjusted market capitalization index that is designed to measure the combined equity market performance of large- and mid-cap securities in developed and emerging market countries excluding the United States. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.446568	2020-10-11 23:38:31.02582
DCF	Dreyfus Alcentra Global Credit Income 2024 Target Term Fund Inc.	The Fund’s investment objectives are to seek high current income and to return at least $9.835 per Common Share (the public offering price per Common Share (as defined below) after deducting a sales load of $0.165 per Common Share but before deducting offering costs of $0.02 per Common Share (“Original NAV”)) to holders of record of Common Shares on or about December 1, 2024 (the “Termination Date”) (subject to certain extensions described herein).	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.551313	2020-10-11 23:38:31.02582
DGP	DB Gold Double Long ETN due February 15 2038	The investment seeks to replicate, net of expenses, twice the daily performance of the Deutsche Bank Liquid Commodity index - Optimum Yield Gold Excess Return.\n The index is intended to reflect changes in the market value of certain gold futures contracts and is comprised of a single unfunded gold futures contract.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.649772	2020-10-11 23:38:31.02423
DIAX	Nuveen Dow 30SM Dynamic Overwrite Fund of Beneficial Interest	Nuveen Dow 30sm Dynamic Overwrite Fund is a closed-end management investment company. Its investment objective is to seek attractive total return with less volatility than the Dow Jones Industrial Average.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.678221	2020-10-11 23:38:31.02423
DIT	AMCON Distributing Company	Amcon Distributing Co and its subsidiaries is in the wholesale distribution of consumer products including cigarettes & tobacco products, candy & confectionery, beverages, foodservice, groceries, paper products, automotive & health & beauty care products.	{"Consumer Defensive","Food Distribution","Retail - Defensive"}	{UNFI,WILC,CORE,SYY,CHEF,SPTN}	Retail - Defensive	Consumer Defensive	NYSE American	\N	2020-10-11 23:12:01.694548	2020-10-11 23:38:31.02423
BH	Biglari Holdings Inc. Class B	Biglari Holdings Inc owns subsidiaries engaged in business activities, including media, property and casualty insurance, and restaurants. It is primarily involved in franchising and operating of restaurants.	{"Consumer Cyclical",Restaurants}	{}	Restaurants	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:00.198255	2020-10-11 23:38:31.021426
BIB	ProShares Ultra Nasdaq Biotechnology	The investment seeks daily investment results that correspond to two times (2x) the daily performance of the Nasdaq Biotechnology Index®.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index is a modified capitalization weighted index that includes securities of Nasdaq listed companies that are classified as either biotechnology or pharmaceutical. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:00.21175	2020-10-11 23:38:31.021426
BKLN	Invesco Senior Loan	The investment seeks to track the investment results (before fees and expenses) of the S&P/LSTA U.S. Leveraged Loan 100 Index (the "underlying index").\n The fund generally will invest at least 80% of its net assets (plus any borrowings for investment purposes) in senior loans that comprise the underlying index. Banks and other lending institutions generally issue senior loans to corporations, partnerships or other entities ("borrowers"). These borrowers operate in a variety of industries and geographic regions, including foreign countries.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.276858	2020-10-11 23:38:31.021426
BLBD	Blue Bird Corporation	Blue Bird Corp is an American bus manufacturing company. The company operates its business in two segments; Bus and Parts segment. Its primary business it to manufacture and designs school buses.	{"Consumer Cyclical","Auto Manufacturers",Autos}	{PCAR,FSS,WNC,MLR}	Autos	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:00.296342	2020-10-11 23:38:31.021426
BNO	United States Brent Oil Fund LP ETV	The investment seeks the daily changes in percentage terms of its shares’ per share net asset value (“NAV”) to reflect the daily changes in percentage terms of the spot price of Brent crude oil. \n The Benchmark Futures Contract is the futures contract on Brent crude oil as traded on the Ice Futures Europe Exchange that is the near month contract to expire, except when the near month contract is within two weeks of expiration, in which case it will be measured by the futures contract that is the next month contract to expire.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.374726	2020-10-11 23:38:31.021426
BSX	Boston Scientific Corporation	Boston Scientific Corp is a developer, manufacturer, and marketer of medical devices. The firm markets its devices to health-care professionals and institutions globally. Foreign sales account for nearly half of the firm's total sales.	{Healthcare,"Medical Devices"}	{CNMD,MMSI,ANGO,SYK,TFX,EW,MDT,XLV}	Medical Devices	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:00.511764	2020-10-11 23:38:31.021426
BUYN	USCF SummerHaven SHPEN Index Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the SummerHaven Private Equity Natural Resources Strategy IndexSM ("SHPEN").\n Under normal market conditions, the fund generally invests substantially all, but at least 80%, of its total assets in the common stocks comprising SHPEN. It generally invests in substantially all of the common stocks comprising SHPEN and in approximately the same proportions as SHPEN. SHPEN attempts to replicate the long-term (i.e., 10 years or more) return characteristics of diversified natural resources private equity allocations.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.536195	2020-10-11 23:38:31.021426
CAPE	Barclays ETN Schiller CAPE	The investment seeks to replicate, net of expenses, the Shiller Barclays CAPETM US Core Sector Index.\n The index seeks to provide a notional long exposure to the top four relatively undervalued U.S. equity sectors that also exhibit relatively strong price momentum. It incorporates the CAPE (Cyclically Adjusted Price Earnings) ratio to assess equity market valuations of nine sectors on a monthly basis and to identify the relatively undervalued sectors represented in the S&P 500®.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.617711	2020-10-11 23:38:31.021426
CCOR	Cambria Core Equity	The investment seeks capital appreciation and capital preservation with a low correlation to the broader U.S. equity market.\n Under normal market conditions, at least 80% of the value of the fund's net assets (plus borrowings for investment purposes) will be invested in equity securities. It invests primarily in U.S. equity securities that tend to offer current dividends. The fund focuses on high-quality companies that have prospects for long-term total returns as a result of their ability to grow earnings and their willingness to increase dividends over time.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.712172	2020-10-11 23:38:31.021426
CHEF	The Chefs' Warehouse Inc.	The Chefs' Warehouse Inc is a distributor of specialty food products in the United States and Canada. It operates in one reportable segment, Food Product Distribution, which is concentrated on the East and West Coasts of the United States.	{"Consumer Defensive","Food Distribution","Retail - Defensive"}	{UNFI,WILC,CORE,SYY,DIT,SPTN}	Retail - Defensive	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:00.849681	2020-10-11 23:38:31.021426
CHT	Chunghwa Telecom Co. Ltd.	Chunghwa Telecom Co Ltd is a Taiwan based telecommunication company. It is engaged in providing fixed-line, wireless, and Internet and data services.	{"Communication Services","Telecom Services"}	{ERIC,SIFY,TLK,CHU,BT,KT}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:00.89786	2020-10-11 23:38:31.021426
CMS	CMS Energy Corporation	CMS Energy Corp is utility company operating in Michigan. It is engaged in independent power production and owns power generation facilities fueled by natural gas and renewable sources.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{MGEE,DTE,LNT,AEE,EXC,WEC,XEL,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:01.068407	2020-10-11 23:38:31.021426
CODX	Co-Diagnostics Inc.	Co-Diagnostics Inc is a molecular diagnostics company that has developed, and intends to sell molecular diagnostic technology such as lab systems and manufacture and sell reagents used for tests.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{XLV}	Medical Diagnostics & Research	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:01.134908	2020-10-11 23:38:31.021426
DKL	Delek Logistics Partners L.P. representing Limited Partner Interests	Delek Logistics Partners LP owns and operates logistics and marketing assets for crude oil, and intermediate and refined products. It operates in two segments, Pipelines and Transportation, and Wholesale Marketing and Terminalling.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{PAA,EEP,MMP,HEP,TLP,NS,PBFX,DK}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:01.712391	2020-10-11 23:38:31.024353
BLES	Inspire Global Hope	The investment seeks to replicate investment results that generally correspond to the Inspire Global Hope Large Cap Index.\n The fund will invest at least 80% of its total assets in the component securities of the index. The index provider selects foreign and domestic equity securities included in the Russell 1,000 Index, MSCI EAFE Index, and MSCI EM Large Cap Index using the index provider's Inspire Impact Score®, a proprietary selection methodology that is designed to assign a score to a particular security based on the security's alignment with biblical values and the positive impact that company has on the world through various ESG criterion.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.312803	2020-10-11 23:38:31.024353
BLUE	bluebird bio Inc.	bluebird bio Inc is a clinical-stage biotechnology company which develops gene therapies for severe genetic and rare diseases and in the field of T cell-based immunotherapy.	{Healthcare,Biotechnology}	{BMRN,CELG,SGMO,SNY,MRK,XLRN,GLYC,IONS,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:00.330443	2020-10-11 23:38:31.024353
BSTC	BioSpecifics Technologies Corp	BioSpecifics Technologies Corp is a biopharmaceutical company. It is developing an injectable collagenase clostridium histolyticum for multiple indications which is marketed as XIAFLEX in the United States or Xiapex in Europe.	{Healthcare,Biotechnology}	{ABT,LLY,ABBV,ALDR,RDUS,HZNP,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:00.507556	2020-10-11 23:38:31.024353
CBPO	China Biologic Products Holdings Inc.	China Biologic Products Holdings Inc is a biopharmaceutical company engaged in the research, development, manufacturing and sales of plasma-based pharmaceutical products to hospitals and inoculation centers directly or through distributors in China.	{Healthcare,Biotechnology}	{NVAX,GILD,SVA,GSK,BAX,CPHI,HEB,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:00.675533	2020-10-11 23:38:31.024353
CEMB	iShares J.P. Morgan EM Corporate Bond	The investment seeks to track the investment results of the J.P. Morgan CEMBI Broad Diversified Core Index composed of U.S. dollar-denominated, emerging market corporate bonds.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The index tracks the performance of the U.S. dollar-denominated emerging market corporate bond market.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:00.774792	2020-10-11 23:38:31.024353
CHEP	AGFiQ U.S. Market Neutral Value Fund	The investment seeks performance results that correspond to the price and yield performance, before fees and expenses, of the Dow Jones U.S. Thematic Market Neutral Value Index.\n The fund invests at least 80% of its net assets in common stock of the long positions in the underlying index and sells short at least 80% of the short positions in the underlying index. The index is a long/short market neutral index that is dollar-neutral. As such, it identifies long and short securities positions of approximately equal dollar amounts. In choosing to track a market neutral index, the fund seeks to limit the effects of general market movements on the fund.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.852697	2020-10-11 23:38:31.024353
CHIX	Global X China Financials	The investment seeks to provide investment results that correspond generally to the price and yield performance, before fees and expenses, of the Solactive China Financials Total Return Index.\n The fund invests at least 80% of its total assets in the securities of the underlying index and in American Depositary Receipts ("ADRs") and Global Depositary Receipts ("GDRs") based on the securities in the underlying index. The underlying index is designed to measure the equity performance of the investable universe of companies in the financials sector of the Chinese economy. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.867602	2020-10-11 23:38:31.024353
CMI	Cummins Inc.	Cummins Inc designs, manufactures, distributes, and services diesel and natural gas engines and engine-related component products, including filtration, after treatment, turbochargers, fuel systems, controls systems, air handling systems, and others.	{Industrials,"Diversified Industrials","Industrial Products"}	{WPRT,PCAR,NAV,BWA,BGG,CAT,GE,RBC,CNHI}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:01.059189	2020-10-11 23:38:31.024353
CNS	Cohen & Steers Inc	Cohen & Steers Inc is an investment manager engaged in real assets, including real estate, infrastructure, and commodities, along with preferred securities and other income solutions. North America generates maximum revenue for the company.	{"Financial Services","Asset Management"}	{VRTS,BEN,BLK,AB,IVZ,GBL,MN,OAK,CG}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.116806	2020-10-11 23:38:31.024353
CNXT	VanEck Vectors ChinaAMC SME-ChNext	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the SME-ChiNext 100 Index.\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The index is a modified, free-float adjusted index intended to track the performance of the 100 largest and most liquid stocks listed and trading on the Small and Medium Enterprise ("SME") Board and the ChiNext Board of the Shenzhen Stock Exchange. Such companies may include small- and medium-capitalization companies. The index is comprised of China A-shares.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.127409	2020-10-11 23:38:31.024353
COMT	iShares Commodities Select Strategy ETF	The investment seeks total return by providing investors with broad commodity exposure.\n The fund seeks to achieve its investment objective by investing in a combination of exchange-traded commodity futures contracts, exchange-traded options on commodity-related futures contracts and exchange-cleared commodity-related swaps (together, "Commodity-Linked Investments") and commodity-related equity securities ("Commodity-Related Equities"), thereby obtaining exposure to the commodities markets. It is an actively managed exchange-traded fund ("ETF") that does not seek to replicate the performance of a specified index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.164104	2020-10-11 23:38:31.024353
CRUS	Cirrus Logic Inc.	Cirrus Logic Inc is a provider of integrated circuits for audio and voice signal processing applications. It products include analog and mixed-signal components targeting mobile devices, and applications in the automotive, energy, and industrial markets.	{Technology,Semiconductors}	{MXIM,QCOM,ON,NXPI,STM,TXN,ADI,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:01.278839	2020-10-11 23:38:31.024353
PGRE	Paramount Group Inc.	Paramount Group Inc is a real estate investment trust which focuses on owning, operating, managing, acquiring and redeveloping office properties.Its portfolio consists of over 12 Class A office properties aggregating nearly10.4 million square feet.	{"Real Estate","REIT - Office",REITs}	{SLG,HPP,WPC,GLPI}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:09.086816	2020-10-11 23:38:31.028166
BNSO	Bonso Electronics International Inc.	Bonso Electronics International Inc mainly designs, develops, produces and sells electronic sensor-based and wireless products for private label original equipment manufacturers, original brand manufacturers and original design manufacturers.	{Technology,"Electronic Components","Computer Hardware"}	{MTD,NC,SODA,XLK}	Computer Hardware	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:00.375739	2020-10-11 23:38:31.031328
BOSS	Global X Funds Founder-Run Companies	The investment seeks to provide investment results that correspond generally to the price and yield performance, before fees and expenses, of the Solactive U.S. Founder-Run Companies Index ("underlying index").\n The fund invests at least 80% of its total assets in the securities of the underlying index. The underlying index is designed to provide exposure to U.S. companies in which a founder or co-founder of the company is serving as the Chief Executive Officer of the company (collectively, "Founder-Run Companies"), as determined by Solactive AG, the provider of the underlying index. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:00.406461	2020-10-11 23:38:31.031328
BTEC	Principal Healthcare Innovators Index ETF	The investment seeks to provide investment results that closely correspond, before expenses, to the performance of the Nasdaq Healthcare Innovators Index (the "index").\n Under normal circumstances, the fund invests at least 80% of its net assets, plus any borrowings for investment purposes, in equity securities of companies that compose the index at the time of purchase. The index uses a quantitative model designed to identify equity securities in the Nasdaq US benchmark index (the "Parent Index") (including growth and value stock) that are small and medium capitalization U.S. healthcare companies.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:00.516995	2020-10-11 23:38:31.031328
CHIM	Global X China Materials	The investment seeks to provide investment results that correspond generally to the price and yield performance, before fees and expenses, of the Solactive China Materials Total Return Index.\n The fund invests at least 80% of its total assets in the securities of the underlying index and in American Depositary Receipts ("ADRs") and Global Depositary Receipts ("GDRs") based on the securities in the underlying index. The underlying index is designed to measure the equity performance of the investable universe of companies in the materials sector of the Chinese economy. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:00.867832	2020-10-11 23:38:31.031328
CNDT	Conduent Incorporated	Conduent Inc is engaged in providing business process services with expertise in transaction-intensive processing, analytics, and automation to healthcare, transportation, and other industries. It operates its business in US, Europe, and other areas.	{Industrials,"Business Services"}	{PFPT,PFM,PCYG}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:01.10149	2020-10-11 23:38:31.031328
COMB	GraniteShares Bloomberg Commodity Broad Strategy No K-1	The investment seeks to provide long-term capital appreciation, primarily through exposure to commodity futures markets.\n The fund is an actively managed ETF that seeks to provide long-term capital appreciation, primarily through exposure to commodity futures markets. While the fund generally will seek exposure to the commodity futures markets included in the Bloomberg Commodity Index, it is not an index tracking ETF and will seek to improve its performance, in part through a cash management strategy consisting of investments in investment grade fixed income securities. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.160429	2020-10-11 23:38:31.031328
CTSO	CytoSorbents Corporation	CytoSorbents Corp is a immunotherapy company. The Company uses blood purification to modulate inflammation with the goal of preventing or treating multiple organ failure in life-threatening illnesses and cardiac surgery.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{HOLX,VIVO,BSX,XLV}	Medical Diagnostics & Research	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:01.36574	2020-10-11 23:38:31.031328
CVR	Chicago Rivet & Machine Co.	Chicago Rivet & Machine Co is in the fastener industry in North America. It manufactures rivets, cold-formed fasteners and parts and screw machine products. It operates in two segments of the fastener industry, fasteners and assembly equipment.	{Industrials,"Tools & Accessories","Industrial Products"}	{ROLL,TRS,SSD,INS,EML,SPLP}	Industrial Products	Industrials	NYSE American	\N	2020-10-11 23:12:01.421105	2020-10-11 23:38:31.031328
CWB	SPDR Bloomberg Barclays Convertible Securities	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the price and yield performance of the Bloomberg Barclays U.S. Convertible Liquid Bond Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index or in securities that the Adviser determines have economic characteristics that are substantially identical to the economic characteristics of the securities that comprise the index. The index is designed to represent the market of U.S. convertible securities, such as convertible bonds and convertible preferred stock. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.434828	2020-10-11 23:38:31.021426
DAUD	ETNs linked to the VelocityShares Daily 4X Long USD vs. AUD Index	The investment seeks to track the daily performance of the VelocityShares Daily 4X Long USD vs. AUD Index.\nThe index is designed to provide 4 times leveraged exposure, reset daily, to changes in the spot exchange rate between an underlying pair of currencies consisting of the U.S. dollar and a foreign currency. It provides long exposure to one currency (the “long currency”) in the underlying currency pair relative to the other currency (the “reference currency”). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.514729	2020-10-11 23:38:31.021426
DCUD	Dominion Energy Inc. 2016 Series A Corporate Units	Dominion Energy Inc is a producer and transporter of energy. It manages its daily operations through three operating segments namely Dominion Virginia Power of DVP, Dominion Energy and Dominion Generation.	{Utilities,"Utilities - Diversified","Utilities - Regulated"}	{XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:01.560131	2020-10-11 23:38:31.021426
DEM	WisdomTree Emerging Markets High Dividend Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree Emerging Markets High Dividend Index.\n Under normal circumstances, at least 95% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a fundamentally weighted index that is comprised of the highest dividend-yielding common stocks selected from the WisdomTree Emerging Markets Dividend Index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.594141	2020-10-11 23:38:31.021426
CCNE	CNB Financial Corporation	CNB Financial Corp is a financial holding company. The company engaged in providing banking activities and services including checking, savings, and time deposit accounts, real estate, commercial, industrial, residential and consumer loans.	{"Financial Services","Banks - Regional - US",Banks}	{CZNC,NWFL,FRBK,BMTC,MPB,PWOD,STBA,UVSP,FNB,TSC}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:00.70789	2020-10-11 23:38:31.023007
CCUR	CCUR Holdings Inc - Ordinary Shares	CCUR Holdings Inc, formerly Concurrent Computer Corp, is a provider of software, hardware and professional services for the multi-screen video market and the high-performance, real-time market.	{}	{XLK}	Computer Hardware	Technology	OTCQB - U.S. Registered	\N	2020-10-11 23:12:00.722932	2020-10-11 23:38:31.023007
CDMOP	Avid Bioservices Inc. 10.50% Series E Convertible Preferred Stock	Avid Bioservices Inc is engaged in development and cGMP manufacturing of biopharmaceutical products derived from mammalian cell culture.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:00.732609	2020-10-11 23:38:31.023007
CSIQ	Canadian Solar Inc.	Canadian Solar Inc is a solar power company. The company designs, develops and manufactures solar wafers, cells and solar power products. It operates in two segments namely MSS segment and energy segment.	{Technology,Solar,Semiconductors}	{FSLR,JKS,SPWR,HQCL,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:01.301735	2020-10-11 23:38:31.023007
CTL	CenturyLink Inc.	CenturyLink Inc operates as a telecommunications company in the United States. It provides broadband, voice and wireless services to consumers and businesses across the country.	{"Communication Services","Telecom Services"}	{CMCSA,VZ,HCOM}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:01.345027	2020-10-11 23:38:31.023007
CWH	Camping World Holdings Inc. Class A Commom Stock	Camping World Holdings Inc provides services, protection plans, products and resources for recreational vehicle enthusiasts across the United States. The firm offers its products and services through the Good Sam and Camping World brands across the US.	{"Consumer Cyclical","Recreational Vehicles",Autos}	{DOO,CWH}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:01.44676	2020-10-11 23:38:31.024353
DATA	Tableau Software Inc. Class A	Tableau Software Inc is a software company that enables customers to visualize and analyze data through a single, easy-to-use platform. It offers products including, Tableau Desktop, Tableau Server, Tableau Online, and Tableau Public.	{Technology,"Software - Application","Application Software"}	{MSFT,AMZN,DWCH,MSTR,SAP,IBM,TDC,ORCL,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:01.512698	2020-10-11 23:38:31.023007
DBKO	Xtrackers MSCI South Korea Hedged Equity	The investment seeks investment results that correspond generally to the performance, before fees and expenses, of the MSCI Korea 25/50 US Dollar Hedged Index.\n The fund, using a "passive" or indexing investment approach, seeks investment results that correspond generally to the performance, before fees and expenses, of the underlying index, which is designed to track the performance of the South Korean equity market while mitigating exposure to fluctuations between the value of the U.S. dollar and the South Korean won. It will invest at least 80% of its total assets in component securities of the underlying index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.539611	2020-10-11 23:38:31.024353
DBUK	Xtrackers MSCI United Kingdom Hedged Equity	The investment seeks investment results that correspond generally to the performance, before fees and expenses, of the MSCI United Kingdom US Dollar Hedged Index.\n The fund, using a "passive" or indexing investment approach, seeks investment results that correspond generally to the performance, before fees and expenses, of the underlying index, which is designed to track the performance of the United Kingdom equity markets, while mitigating exposure to fluctuations between the value of the U.S. dollar and British pound sterling. It will invest at least 80% of its total assets in component securities of the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.543189	2020-10-11 23:38:31.024353
DBVT	DBV Technologies S.A.	DBV Technologies SA is a biopharmaceutical company. It is engaged in research and development of patient-friendly therapy for food and pediatric allergy patients.	{Healthcare,Biotechnology}	{ALXN,ALNY,FLXN,IONS,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:01.547786	2020-10-11 23:38:31.023007
DFIN	Donnelley Financial Solutions Inc.	Donnelley Financial Solutions Inc is a global integrated communications provider. The company helps organizations communicate by creating, managing, producing, distributing and processing content for its clients globally.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{DFIN}	Brokers & Exchanges	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.621801	2020-10-11 23:38:31.024353
DGS	WisdomTree Emerging Market SmallCap Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree Emerging Markets SmallCap Dividend Index.\n Under normal circumstances, at least 95% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a fundamentally weighted index that is comprised of small cap common stocks selected from the WisdomTree Emerging Markets Dividend Index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.65342	2020-10-11 23:38:31.024353
DHXM	DHX Media Ltd.	DHX Media Ltd is a children's content and brands company, recognized globally for properties such as Peanuts, Teletubbies, Strawberry Shortcake, Caillou, Inspector Gadget and Degrassi franchise.	{"Consumer Cyclical","Broadcasting - TV",Entertainment}	{}	Entertainment	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:01.673953	2020-10-11 23:38:31.023007
DIVB	iShares U.S. Dividend and Buyback	The investment seeks to track the investment results of the Morningstar® US Dividend and Buyback IndexSM.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents, including shares of money market funds advised by BFA or its affiliates, as well as in securities not included in the underlying index. The underlying index is designed to provide exposure to U.S.-based companies that return capital to shareholders through either dividend payments or share buybacks. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:01.700824	2020-10-11 23:38:31.024353
CIF	MFS Intermediate High Income Fund	Mfs Intermediate High Income Fund is a United States based diversified closed-end management investment company. Its investment objective is to seek high current income but may also consider capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:00.921222	2020-10-11 23:38:31.025871
CSII	Cardiovascular Systems Inc.	Cardiovascular Systems Inc is a medical technology company developing solutions for vascular and coronary disease. It treats patients suffering from peripheral and coronary artery diseases, including those with arterial calcium.	{Healthcare,"Medical Devices"}	{MMSI,ELGX,ANGO,LMAT,BSX,JNJ,MDT,XLV}	Medical Devices	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:01.301089	2020-10-11 23:38:31.025871
CVLT	Commvault Systems Inc.	CommVault Systems Inc is a provider of data and information management software applications and related services. It develops, markets and sells software applications and services in North America, Europe, Australia and Asia.	{Technology,"Software - Application","Application Software"}	{MSFT,SYMC,NTAP,IBM,VMW,VNET,HPQ,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:01.415686	2020-10-11 23:38:31.025871
CWS	AdvisorShares Focused Equity	The investment seeks long-term capital appreciation.\n The fund seeks to achieve its investment objective by investing primarily in a focused group of U.S. exchange listed equity securities, including common and preferred stock and American Depositary Receipts. It will invest at least 80% of its net assets (plus any borrowings for investment purposes) in equity securities. The Advisor may use a variety of methods for security selection and will seek to focus on firms that are fundamentally sound and have shown consistency in their financial results and high earnings quality.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.446905	2020-10-11 23:38:31.025871
CZA	Invesco Zacks Mid-Cap	The investment seeks to track the investment results (before fees and expenses) of the Zacks Mid-Cap Core Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in securities that comprise the underlying index and depositary receipts representing securities that comprise the underlying index. The underlying index was comprised of 100 securities selected, according to the investment and other criteria in the underlying index methodology, from a universe of mid-capitalization securities including common stocks, MLPs, ADRs, REITs and BDCs. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.487897	2020-10-11 23:38:31.025871
DAN	Dana Incorporated	Dana Inc provides technology driveline, sealing and thermal-management products for vehicle manufacturers in the light vehicle, medium/heavy vehicle and off-highway markets.	{"Consumer Cyclical","Auto Parts",Autos}	{MGA,BWA,MOD,SMP,AXL,LEA,MTOR,VC}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:01.510227	2020-10-11 23:38:31.025871
DDBI	Legg Mason Developed EX-US Diversified Core ETF	The investment seeks to track the investment results of the QS DBI Developed ex-US Diversified Index which composed of publicly traded equity securities of developed markets outside the United States.\n The fund will invest at least 80% of its net assets, plus borrowings for investment purposes, if any, in securities that compose the underlying index. The underlying index is composed of equity securities in developed markets outside the United States that are included in the MSCI World ex-US Index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.560439	2020-10-11 23:38:31.028166
DDJP	WisdomTree Dynamic Currency Hedged Japan Equity Fund	The investment seeks to track the price and yield performance, of the WisdomTree Dynamic Currency Hedged Japan Equity Index.\n Under normal circumstances, at least 80% of the fund's total assets will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a dividend weighted index designed to provide exposure to Japanese equity securities while at the same time dynamically hedging currency exposure to fluctuations between the value of the Japanese yen and the USD. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:01.570289	2020-10-11 23:38:31.028166
DMLP	Dorchester Minerals L.P.	Dorchester Minerals LP is engaged in the acquisition, ownership and administration of producing and nonproducing natural gas and crude oil royalty, overriding royalty, net profits and leasehold interests in properties.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{MARPS,PHX,PBT,MTR,NRT,SBR,SJT}	Oil & Gas - E&P	Energy	Nasdaq Global Select	\N	2020-10-11 23:12:01.743738	2020-10-11 23:38:31.025871
DOGZ	Dogness (International) Corporation	Dogness (International) Corp designs, manufactures and distributes pet-related products. The company offers its products to major retail stores and wholesalers around the world.	{"Consumer Defensive","Household & Personal Products","Consumer Packaged Goods"}	{DOGZ}	Consumer Packaged Goods	Consumer Defensive	NASDAQ Global Market	\N	2020-10-11 23:12:01.776736	2020-10-11 23:38:31.025871
DRW	WisdomTree Global ex-US Real Estate Index	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree Global ex-U.S. Real Estate Index.\n Under normal circumstances, at least 95% of the fund's total assets will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a fundamentally weighted index that is comprised of companies from developed and emerging markets outside of the United States that are classified as being part of the "Global Real Estate" sector. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.824571	2020-10-11 23:38:31.031328
DTUS	iPath US Treasury 2-year Bear ETN	The investment seeks to provide investors with inverse exposure to the Barclays 2Y US Treasury Futures Targeted Exposure Index™. \nThe Barclays 2Y US Treasury Futures Targeted Exposure Index™ (the "index") is designed to decrease in response to an increase in the 2-year Treasury note yields and to increase in response to a decrease in 2-year Treasury note yields. The index targets a fixed level of sensitivity to changes in the yield of the current "cheapest-to-deliver" note underlying the relevant 2-year Treasury futures contract at a given point in time.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.862081	2020-10-11 23:38:31.02582
DVYE	iShares Emerging Markets Dividend Index Fund Exchange Traded Fund	The investment seeks to track the investment results of the Dow Jones Emerging Markets Select Dividend Index, which measures the performance of a group of equity securities issued by companies in emerging market countries that have provided relatively high dividend yields on a consistent basis over time.\n The fund generally will invest at least 90% of its assets in the component securities of the index and in investments that have economic characteristics that are substantially identical to the component securities of the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.894836	2020-10-11 23:38:31.025871
COO	The Cooper Companies Inc.	The Cooper Companies Inc is a medical device company operating through its subsidiaries. It manufactures products for contact lens wearers and supplies women's health clinicians with products and treatment options to improve the delivery of healthcare.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{ATRI,UTMD,NVS,BSX,JNJ,TFX,TEAR,XLV}	Medical Instruments & Equipment	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:01.166051	2020-10-11 23:38:31.023428
CYB	WisdomTree Chinese Yuan Strategy Fund	The investment seeks to achieve total returns reflective of both money market rates in China available to foreign investors and changes in value of the Chinese yuan relative to the U.S. dollar.\n The fund normally invests at least 80% of its net assets, plus the amount of any borrowings for investment purposes, in investments whose combined performance is economically tied to China. It is an actively managed exchange traded fund ("ETF"). The fund generally will maintain a weighted average portfolio maturity of 90 days or less with respect to the money market securities in its portfolio. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.464646	2020-10-11 23:38:31.023428
DEF	Invesco Defensive Equity	The investment seeks investment results that correspond generally to the performance, before the fund's fees and expenses, of an equity index called the Guggenheim Defensive Equity Index.\n The fund will invest at least 80% of its total assets in securities that comprise the underlying index. The underlying index is designed to provide exposure to equity securities of large capitalization U.S. issuers that Invesco Indexing, LLC ("Invesco indexing" or the "index provider") has selected for inclusion in the underlying index based on a rules-based screening criteria. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.58688	2020-10-11 23:38:31.023428
DIVY	Realty Shares DIVS	The investment seeks to produce long-term capital appreciation.\n The fund's principal investment strategy is designed to provide exposure to the aggregate value of ordinary dividends expected to be paid on a portfolio of large capitalization equity securities listed for trading in the U.S. ("Large Cap Securities"). The fund may use a variety of investment strategies to achieve this objective. Under normal circumstances, it generally invests in a combination of dividend swaps, dividend futures and forwards on indexes of Large Cap Securities ("Large Cap Securities Indexes"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.705657	2020-10-11 23:38:31.023007
DLN	WisdomTree U.S. LargeCap Dividend Fund	The investment seeks a high level of total return consisting of both income and capital appreciation.\n The fund normally invests at least 80% of its net assets, plus the amount of any borrowings for investment purposes, in Local Debt. The advisor intends to focus its investments on Local Debt issued by governments (national, state, and local), government agencies and instrumentalities, and government sponsored enterprises. The fund may invest up to 20% of its assets in debt instruments denominated in U.S. dollars issued by Asian governments and government sponsored enterprises. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.724121	2020-10-11 23:38:31.023007
DMRM	DeltaShares S&P 400 Managed Risk	The investment seeks to track the investment results, before fees and expenses, of the S&P 400 Managed Risk 2.0 Index.\n Under normal market conditions, the fund invests a substantial portion, but at least 80%, of its assets, exclusive of collateral held from securities lending, in securities comprising the S&P 400 Managed Risk 2.0 Index. The underlying index seeks to achieve these objectives by allocating weightings among the S&P MidCap 400 Index, the S&P U.S. Treasury Bond Current 5-Year Index and the S&P U.S. Treasury Bill 0-3 Month Index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.752896	2020-10-11 23:38:31.023428
DNI	Dividend and Income Fund	Chartwell Dividend & Income Fund Inc is a closed end management investment company. Its investment objective is to seek high current income and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.759467	2020-10-11 23:38:31.023007
DOG	ProShares Short Dow30	The investment seeks daily investment results, before fees and expenses, that correspond to the inverse (-1x) of the daily performance of the Dow Jones Industrial AverageSM.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index is a price-weighted index and includes 30 large-cap, "blue-chip" U.S. stocks, excluding utility and transportation companies. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.775116	2020-10-11 23:38:31.023007
DRYS	DryShips Inc.	DryShips Inc is a provider of drybulk commodities transportation services for the steel, electric utility, construction and agri-food industries and offshore support services to the offshore energy industry. It provides ultra-deep water drilling services.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{GASS,CPLP,TNP,DSX,NM,ESEA,SBLK,EGLE}	Transportation & Logistics	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:01.824322	2020-10-11 23:38:31.024353
DUSA	Davis Select U.S. Equity ETF	The investment seeks long-term capital growth and capital preservation.\n Under normal market conditions, the fund will invest at least 80% of its net assets plus any borrowings for investment purposes in equity securities issued by U.S. companies. The fund's portfolio generally contains between 15 and 35 companies. It may invest a portion of its assets in financial services companies. The fund may also invest in mid- and small-capitalization companies, which the manager considers to be those companies with less than $10 billion in market capitalization. It may invest up to 20% of net assets in non-U.S. companies. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.873597	2020-10-11 23:38:31.023428
DUSL	Direxion Daily Industrials Bull 3X Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the daily performance of the Industrials Select Sector Index.\n The fund, under normal circumstances, invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, exchange-traded funds ("ETFs") that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index includes domestic companies from the industrials sector which includes the following industries: aerospace and defense: industry conglomerates; and machinery. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.874687	2020-10-11 23:38:31.028166
EBR	Centrais Electricas Brasileiras S A American Depositary Shares (Each representing one)	Centrais Eletricas Brasileiras SA, along with its subsidiaries, is engaged in generation, transmission and distribution of electricity in Brazil. It operates in three segments namely generation, transmission and distribution segment.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{CIG,ELP,FE,ED,PBR,CPL,NEE,DUK,AA,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:02.049895	2020-10-11 23:38:31.028166
CRI	Carter's Inc.	Carter's Inc is an apparel manufacturing company that sells its products through branded retail stores, company websites, and department stores and other locations. Its brands include Carter's, Child of Mine, Just One You, Precious Firsts, and OshKosh.	{"Consumer Cyclical","Apparel Manufacturing","Manufacturing - Apparel & Furniture"}	{PLCE,DEST,GES,AEO,GPS,KSS,WMT,TGT,RL}	Manufacturing - Apparel & Furniture	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:01.254014	2020-10-11 23:38:31.028166
CTR	ClearBridge Energy MLP Total Return Fund Inc.	ClearBridge Energy MLP Total Return Fund operates as a closed end management investment company. Its investment objective is to provide a high level of total return consisting of cash distributions and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.352595	2020-10-11 23:38:31.028166
DBV	Invesco DB G10 Currency Harvest Fund	The investment seeks to track changes, whether positive or negative, in the level of the Deutsche Bank G10 Currency Future Harvest Index®—Excess Return.\n The index is comprised of long futures positions in the three eligible index currencies associated with the highest interest rates and short futures positions in the three eligible index currencies associated with the lowest interest rates. The index’s six component currencies from time-to-time, comprised of the three long and three short futures positions, are referred to as the index currencies and are used to calculate the value of the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.543828	2020-10-11 23:38:31.028166
DCP	DCP Midstream LP	DCP Midstream LP is a midstream master limited partnership. It is engaged in gathering, compressing, treating, processing, transporting, storing and selling natural gas, Natural Gas Liquids and condensate.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{EPD,ET,SEP,TRGP,TCP,ENLK}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:01.554783	2020-10-11 23:38:31.028166
DDLS	WisdomTree Dynamic Currency Hedged International SmallCap Equity Fund	The investment seeks to track the WisdomTree Dynamic Currency Hedged International SmallCap Equity Index.\n The fund will invest at least 80% of its total assets in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a dividend weighted index designed to provide exposure to small-capitalization equity securities in the industrialized world, excluding Canada and U.S., while at the same time dynamically hedging currency exposure to fluctuations between the value of foreign currencies and the USD. It is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:01.571746	2020-10-11 23:38:31.028166
DJD	Invesco Dow Jones Industrial Average Dividend	The investment seeks to track the investment results (before fees and expenses) of the Dow Jones Industrial Average Yield Weighted (the "underlying index").\n The fund will generally invest at least 90% of its total assets in common stocks that comprise the underlying index. The underlying index is designed to provide exposure to dividend-paying equity securities of companies included in the Dow Jones Industrial Average™, which is a price-weighted index of 30 U.S. companies that meet certain size, listing and liquidity requirements. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.708655	2020-10-11 23:38:31.02582
DMO	Western Asset Mortgage Defined Opportunity Fund Inc	Western Asset Mortg Defined Oppt Fd Inc. is a non-diversified, limited-term, closed-end management investment company. The fund's primary investment objective is to provide current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.743621	2020-10-11 23:38:31.02582
DMRS	DeltaShares S&P 600 Managed Risk	The investment seeks to track the investment results, before fees and expenses, of the S&P 600 Managed Risk 2.0 Index.\n Under normal market conditions, the fund invests a substantial portion, but at least 80%, of its assets, exclusive of collateral held from securities lending, in securities comprising the S&P 600 Managed Risk 2.0 Index. The underlying index seeks to achieve these objectives by allocating weightings among the S&P SmallCap 600 Index, the S&P U.S. Treasury Bond Current 5-Year Index and the S&P U.S. Treasury Bill 0-3 Month Index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.751368	2020-10-11 23:38:31.02582
DNL	WisdomTree Global ex-US Quality Dividend Growth Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree Global ex-U.S. Quality Dividend Growth Index.\n Under normal circumstances, at least 95% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in the component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a fundamentally weighted index that consists of dividend-paying global ex-U.S. common stocks with growth characteristics. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.761213	2020-10-11 23:38:31.028166
ZFGN	Zafgen Inc.	Zafgen Inc is a biopharmaceutical company engaged in improving the health and well-being of patients affected by obesity and complex metabolic disorders. Its product candidate include Beloranib.	{Healthcare,Biotechnology}	{GSK,BSX,JNJ,MDT,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:13.778133	2020-10-11 23:38:31.024353
ZKIN	ZK International Group Co. Ltd	ZK International Group Co Ltd is engaged in designing and producing pipes and fittings such as double-press thin-walled stainless steel tubes and fittings, carbon steel tubes and fittings and single-press tubes and fittings.	{"Basic Materials",Steel}	{}	Steel	Basic Materials	NASDAQ Capital Market	\N	2020-10-11 23:12:13.794781	2020-10-11 23:38:31.021426
ZOM	Zomedica Pharmaceuticals Corp.	Zomedica Pharmaceuticals Corp is a pharmaceutical company. It is engaged in the discovery, development and commercialization of pharmaceuticals for the companion pet.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{OMER,XLV}	Drug Manufacturers	Healthcare	NYSE American	\N	2020-10-11 23:12:13.809767	2020-10-11 23:38:31.031328
DUST	Direxion Daily Gold Miners Index Bear 3X Shares	The investment seeks daily investment results before fees and expenses of 300% of the inverse of the daily performance of the NYSE Arca Gold Miners Index.\n The fund invests in swap agreements, futures contracts, short positions or other financial instruments that, in combination, provide inverse or short leveraged exposure to the index equal to at least 80% of the fund's net assets. The index is a modified market capitalization weighted index comprised of publicly traded companies that operate globally in both developed and emerging markets, and are involved primarily in mining for gold and, to a lesser extent, in mining for silver. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.875051	2020-10-11 23:38:31.028166
DXB	Deutsche Bk Contingent Cap TR II Tr Pfd Sec	Deutsche Bank Contingent Capital Trust II is a financial company.	{"Financial Services","Banks - Regional - Asia",Banks}	{}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:01.95582	2020-10-11 23:38:31.028166
ECHO	Echo Global Logistics Inc.	Echo Global Logistics Inc is a provider of technology-enabled transportation and supply chain management services. It also offers intermodal, small parcel, domestic air, expedited and international transportation services.	{Industrials,"Integrated Shipping & Logistics","Transportation & Logistics"}	{HUBG,EXPD,FWRD,AAWW,CHRW,INWK,RRTS,XPO,RLGT}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:02.067786	2020-10-11 23:38:31.028166
EEH	Aktiebolaget Svensk Exportkredit (Swed Ex Cred Corp) Elements (SM) Linked to the SPECTRUM Large Cap U.S. Sector Momentum Index developed by BNP Paribas due Augu	The investment seeks to replicate, net of expenses, the SPECTRUM Large Cap U.S. Sector Momentum Index.\n The index tracks the value of a notional portfolio composed of the ten sub-indices of the S&P 500. It is based on the relative weights of the sub-indices within the index. The weights of the sub-indices that outperform the S&P 500 Total Return Index are increased in the index, which weights of the sub-indices that underperform are reduced.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.126296	2020-10-11 23:38:31.028166
EMBH	iShares Interest Rate Hedged Emerging Markets Bond	The investment seeks to mitigate the interest rate risk of a portfolio composed of U.S. dollar-denominated, emerging market bonds.\n The fund invests at least 80% of its net assets in U.S. dollar-denominated emerging market bonds, in one or more underlying funds that principally invest in emerging market bonds, and in U.S. Treasury securities. It may also invest in other ETFs, U.S. government securities, options and swap contracts, short-term paper, cash and cash equivalents, including shares of money market funds advised by BFA or its affiliates.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.251928	2020-10-11 23:38:31.028166
EMF	Templeton Emerging Markets Fund	Templeton Emerging Markets Fund is a closed-end management investment company. The company seeks long-term capital appreciation by investing, under normal market conditions, at least 80% of its net assets in emerging country equity securities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.266306	2020-10-11 23:38:31.028166
EQIX	Equinix Inc.	Equinix Inc is a network-neutral provider of retail data center, co-location, and interconnection services. Its client base includes network services firms, cloud and IT service providers, financial service firms, digital, and media content providers.	{"Real Estate","REIT - Diversified",REITs}	{AKAM,CTL,COR}	REITs	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:02.381748	2020-10-11 23:38:31.028166
ESGN	Columbia Sustainable International Equity Income	The investment seeks investment results that, before fees and expenses, closely correspond to the performance of the Beta Advantage® Sustainable International Equity Income 100 Index.\n The fund invests at least 80% of its assets in the component securities of the index and depositary receipts representing such securities. The index, which typically holds common stocks and depositary receipts, was constructed to provide exposure to foreign large- and mid-cap companies that are believed to offer sustainable levels of income, as well as total return opportunity.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.44006	2020-10-11 23:38:31.028166
EUM	ProShares Short MSCI Emerging Markets	The investment seeks daily investment results, before fees and expenses, that correspond to the inverse (-1x) of the daily performance of the MSCI Emerging Markets Index®.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index includes 85% of the free float-adjusted market capitalization in each industry group in emerging market countries. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.498944	2020-10-11 23:38:31.028166
EWGS	Ishares MSCI Germany Small Cap	The investment seeks to track the investment results of the MSCI Germany Small Cap Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is designed to measure the performance of equity securities of small-capitalization companies whose market capitalization, as calculated by the index provider, represents the bottom 14% of the German securities market.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:02.5622	2020-10-11 23:38:31.028166
FFTY	Innovator IBD 50	The investment seeks to track, before fees and expenses, the performance of the IBD® 50 Index (the "index").\n The fund will normally invest at least 80% of its net assets (including investment borrowings) in securities that comprise the index. The IBD® 50 Index is a weekly, rules-based, computer-generated stock index compiled and published by Investor's Business Daily® ("IBD" or the "index provider") that seeks to identify the current top 50 growth stocks.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.861397	2020-10-11 23:38:31.028166
FLAT	iPath US Treasury Flattener ETN	The investment seeks to provide investors with inverse exposure to the Barclays US Treasury 2Y/10Y Yield Curve Index™. \nThe Barclays US Treasury 2Y/10Y Yield Curve Index™ (the "index") employs a strategy that seeks to capture returns that are potentially available from a "steepening" or "flattening", as applicable, of the U.S. Treasury yield curve through a notional rolling investment in U.S. Treasury note futures contracts. The level of the index is designed to increase in response to a "steepening" of the yield curve and to decrease in response to a "flattening" of the yield curve.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.023699	2020-10-11 23:38:31.028166
FSS	Federal Signal Corporation	Federal Signal Corp designs and manufactures a suite of products and integrated solutions for municipality, industrial and others. The company operates in two segments namely Environmental Solutions Group and Safety and Security Systems Group.	{Industrials,"Pollution & Treatment Controls","Industrial Products"}	{CTAS,GNTX,SPAR,KNDI,TTM}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:03.269307	2020-10-11 23:38:31.025871
DVEM	WisdomTree Emerging Markets Dividend Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree Emerging Markets Dividend Index (the "index").\n Under normal circumstances, at least 80% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a dividend weighted index that consists of emerging market dividend-paying common stocks. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:01.88666	2020-10-11 23:38:31.02582
DWM	WisdomTree International Equity Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree International Equity Index.\n At least 95% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a fundamentally weighted index that is comprised of dividend-paying companies in the industrialized world, excluding Canada and the United States, that pay regular cash dividends. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:01.930385	2020-10-11 23:38:31.02582
DXR	Daxor Corporation	Daxor Corp is a US-based biotechnology company which provides original medical equipment for medical personnel. It manufactures the BVA-100 Blood Volume Analyzer that provides a rapid direct measurement of a patient's blood volume.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{XLV}	Medical Instruments & Equipment	Healthcare	NYSE American	\N	2020-10-11 23:12:01.99504	2020-10-11 23:38:31.02582
EDOW	First Trust Dow 30 Equal Weight	The investment seeks investment results that correspond generally to the price and yield of an equity index called the Dow Jones Industrial Average Equal Weight Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index consists of an equally weighted portfolio of the 30 securities that comprise the Dow Jones Industrial Average (TM) (the "DJIA"). The 30 securities comprising the DJIA are domestic, blue-chip companies covering all industries, with the exception of transportation and utilities. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.110514	2020-10-11 23:38:31.02582
EET	ProShares Ultra MSCI Emerging Markets	The investment seeks daily investment results, before fees and expenses, that correspond to two times (2x) the daily performance of the MSCI Emerging Markets Index®.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index includes 100 of the largest domestic and international non-financial companies listed on the Nasdaq Stock Market based on market capitalization. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.146718	2020-10-11 23:38:31.02582
EMEM	Virtus Glovista Emerging Markets	The investment seeks investment results that correspond to the Solactive Most Favored Emerging Market Index.\n Under normal market conditions, the fund will invest at least 80% of its assets in component securities of the underlying index and in depositary receipts representing such securities. The underlying index is based on a proprietary methodology developed by the fund's sub-adviser, and is designed to identify securities of companies in "large liquid emerging market countries," based on a country selection model, and provide exposure to those securities on a free-float adjusted market capitalization-weighted basis. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.26501	2020-10-11 23:38:31.02582
EMLP	First Trust North American Energy Infrastructure Fund	The investment seeks total return.\n The fund invests at least 80% of its net assets in equity securities of companies deemed by the sub-advisor to be engaged in the energy infrastructure sector. These companies principally include publicly-traded MLPs and limited liability companies taxed as partnerships, MLP affiliates, pipeline companies, utilities, and other companies that derive the majority of their revenues from operating or providing services in support of infrastructure assets such as pipelines, power transmission and petroleum and natural gas storage in the petroleum, natural gas and power generation industries. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.282049	2020-10-11 23:38:31.02582
EMTL	SPDR DoubleLine Emerging Markets Fixed Income	The investment seeks to provide high total return from current income and capital appreciation.\n Under normal circumstances, the fund will invest at least 80% of its net assets (plus the amount of borrowings for investment purposes) in emerging market fixed income securities. Fixed income securities are defined as fixed income securities issued or guaranteed by foreign corporations or foreign governments; corporate or government bonds; sovereign debt; structured securities; foreign currency transactions; certain derivatives; preferred securities; zero coupon bonds; credit-linked notes; pass through notes; bank loans; and perpetual maturity bonds.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:02.293619	2020-10-11 23:38:31.02582
EPZM	Epizyme Inc.	Epizyme Inc is a clinical stage biopharmaceutical company. The Company discovers, develops and commercializes novel epigenetic therapies for cancer patients.	{Healthcare,Biotechnology}	{CELG,SGEN,GSK,NVS,ABT,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:02.375161	2020-10-11 23:38:31.02582
EQRR	ProShares Equities for Rising Rates ETF	The investment seeks investment results, before fees and expenses, that track the performance of the Nasdaq U.S. Large Cap Equities for Rising Rates Index (the "index").\n The fund invests in securities that ProShare Advisors believes, in combination, should track the performance of the index. Under normal circumstances, the fund will invest at least 80% of its total assets in component securities. The index consists of 50 companies whose stock prices historically have tended to outperform the Universe during periods of rising interest rates.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.394332	2020-10-11 23:38:31.02582
ESP	Espey Mfg. & Electronics Corp.	Espey Manufacturing & Electronics Corp is a power electronics design & original equipment manufacturing company. It manufactures electronic equipment including filters, power transformers, magnetic components used in military and industrial applications.	{Technology,"Electronic Components","Computer Hardware"}	{MRCY,BELFB,TCCO,TIK,DYSL,XLK}	Computer Hardware	Technology	NYSE American	\N	2020-10-11 23:12:02.455934	2020-10-11 23:38:31.02582
ETB	Eaton Vance Tax-Managed Buy-Write Income Fund of Beneficial Interest	Eaton Vance Tax-Managed Buy-Write Income Fund is a United States based diversified closed-controlment investment company. The fund's investment objective is to provide current income and gains, with a secondary objective of capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.470627	2020-10-11 23:38:31.02582
DWCR	Arrow DWA Country Rotation ETF	The investment seeks investment results that generally correspond, before fees and expenses, to the price and yield performance of the Dow Jones Global Composite Yield Index (the "underlying index").\n The fund uses a "passive" or "indexing" investment approach to seek to track the price and yield performance of the underlying index. It invests at least 80% of its total assets in the component securities of the underlying index (or depositary receipts representing those securities). The underlying index seeks to identify the 150 highest yielding investable securities in the world within three "asset classes."	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.915012	2020-10-11 23:38:31.02423
DWLV	PowerShares DWA Momentum & Low Volatility Rotation	The investment seeks to track the investment results that generally correspond to the price and yield of the Dorsey Wright® Multi-Factor Global Equity Index.\n The fund generally will invest at least 90% of its total assets in securities that comprise the index. It is a "fund of funds", meaning that it invests its assets in the shares of other ETFs, rather than in securities of individual companies. The underlying index includes other ETFs that are advised by the fund's adviser or its affiliates (the "PowerShares ETFs"), as well as 1- to 6-month U.S. Treasury Bills. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.931401	2020-10-11 23:38:31.02423
EFG	iShares MSCI EAFE Growth	The investment seeks to track the investment results of the MSCI EAFE Growth Index composed of developed market equities, excluding the U.S. and Canada, that exhibit growth characteristics.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The index is a subset of the MSCI EAFE Index. Constituents of the underlying index include securities of companies located in Europe, Australasia and the Far East.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:02.157618	2020-10-11 23:38:31.02423
EGF	Blackrock Enhanced Government Fund Inc.	Blackrock Enhanced Government Fund Inc is a United States based diversified closed-end management investment company. Its investment objective is to provide shareholders with current income and gains.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.185676	2020-10-11 23:38:31.02423
ESL	Esterline Technologies Corporation	Esterline Technologies Corp is a manufacturing company serving aerospace and defense customers. It designs, manufactures and markets engineered products and systems for application within the industries.	{Industrials,"Aerospace & Defense"}	{TATT,WWD,HEI,TGI,GD,HON,UTX,LLL}	Aerospace & Defense	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:02.448136	2020-10-11 23:38:31.02423
EWK	iShares Inc MSCI Belgium	The investment seeks to track the investment results of the MSCI Belgium IMI 25/50 Index.\n The fund will at all times invest at least 80% of its assets in the securities of its underlying index and in depositary receipts representing securities in its underlying index. The index is designed to measure broad-based equity market performance in Belgium. A capping methodology is applied that limits the weight of any single component to a maximum of 25% of the underlying index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.564893	2020-10-11 23:38:31.02423
FRAK	VanEck Vectors Unconventional Oil & Gas	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the MVIS® Global Unconventional Oil & Gas Index.\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The index contains companies that generate at least 50% of their revenues from unconventional oil and gas or that have properties with the potential to generate at least 50% of their revenues from unconventional oil and gas. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.217508	2020-10-11 23:38:31.02423
FTXH	First Trust Nasdaq Pharmaceuticals ETF	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of the Nasdaq US Smart Pharmaceuticals Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is designed to select pharmaceuticals stocks from the base index based on a ranking methodology of three price factors which aims to select companies that exhibit: strong growth through high average 3-, 6-, 9- and 12-month price return; value; and low expected volatility based on historical stock price fluctuation. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.314697	2020-10-11 23:38:31.02423
FTXR	First Trust Nasdaq Transportation ETF	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of the Nasdaq US Smart Transportation Index.\n The fund invests at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is designed to select transportation stocks from the NASDAQ US Benchmark Index based on a ranking methodology of three price factors which aims to select companies that exhibit: strong growth through high average 3-, 6-, 9- and 12-month price return; value; and low expected volatility based on historical stock price fluctuation. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.323934	2020-10-11 23:38:31.02423
GDL	GDL Fund The of Beneficial Interest	Under normal circumstances, the Fund will invest at least 80% of its assets in securities or hedging arrangements relating to companies involved in corporate transactions or reorganizations, giving rise to the possibility of realizing gains upon or within relatively short periods of time after the completion of such transactions or reorganizations. This policy is not fundamental and may be changed by the Fund with notice of not less than 60 days to its shareholders. In market cycles with scarce transaction opportunities, the Fund may seek to accomplish its objective of achieving absolute returns by temporarily investing in other assets, including, but not limited to, short-term debt securities, which may make it less likely for the Fund to achieve an attractive rate of return. As a global fund, the Fund may invest without limitation in the securities of foreign and domestic issuers. The Funds investment strategy is to invest in merger arbitrage transactions and corporate reorganizations throughout the world. Under normal market conditions, the Fund expects to have at least 40% of its assets invested in at least three countries, other than the United States. To the extent that the majority of mergers, takeovers, tender offers and leveraged buyouts and corporate reorganizations are concentrated in any given geographic region, such as Europe, North America or Asia, a relatively high proportion of the Funds assets may be invested in that particular region.	{"Financial Services","Asset Management"}	{}			New York Stock Exchange	\N	2020-10-11 23:12:03.4563	2020-10-11 23:38:31.02423
DWLD	Davis Select Worldwide ETF	The investment seeks long-term growth of capital.\n The fund's investment adviser, uses the Davis Investment Discipline to invest the fund's portfolio principally in common stocks issued by both United States and foreign companies, including countries with developed or emerging markets. It will invest significantly in issuers (i) organized or located outside of the U.S.; (ii) whose primary trading market is located outside the U.S.; or (iii) doing a substantial amount of business outside the U.S., which the adviser considers to be a company that derives at least 50% of its revenue from business outside the U.S. or has at least 50% of its assets outside the U.S.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:01.93034	2020-10-11 23:38:31.025871
DXC	DXC Technology Company	DXC Technology Co is an independent, end-to-end IT services company. It provides information technology and consulting services and solutions to enterprise clients.	{Technology,"Information Technology Services","Application Software"}	{MSFT,CERN,CTSH,IBM,WIT,ACN,UIS,EPAM,INFY,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:01.9557	2020-10-11 23:38:31.025871
EMD	Western Asset Emerging Markets Debt Fund Inc	Western Asset Emg Markets Debt Fund Inc is a non-diversified closed-end management investment company. Its primary investment objective is to seek high current income and the secondary objective is to seek capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.260085	2020-10-11 23:38:31.025871
ESGW	Columbia Sustainable Global Equity Income	The investment seeks investment results that, before fees and expenses, closely correspond to the performance of the Beta Advantage® Sustainable Global Equity Income 200 Index.\n The fund invests at least 80% of its assets in the component securities of the index and depositary receipts representing such securities. The index, which typically holds common stocks and depositary receipts, was constructed to provide exposure to U.S. and foreign (developed market) large- and mid-cap companies (located in developed markets) that are believed to offer sustainable levels of income, as well as total return opportunity.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.444172	2020-10-11 23:38:31.025871
EUFL	Direxion Daily European Financials Bull 2X Shares	The investment seeks daily investment results, of 200% of the daily performance of the MSCI Europe Financials Index.\n The fund, under normal circumstances, invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, exchange-traded funds ("ETFs") that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index is a free float-adjusted, market capitalization-weighted index and represents securities of large-capitalization and mid-capitalization companies across developed market countries in Europe. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.494668	2020-10-11 23:38:31.025871
EUXL	Direxion Daily EURO STOXX 50? Bull 3X Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the daily performance of the EURO STOXX 50® Index.\n The fund invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, exchange-traded funds ("ETFs") that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index is part of the STOXX blue-chip index family and represents the performance of the 50 largest companies among the 19 supersectors in terms of free-float market capitalization in 11 Eurozone countries. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.507648	2020-10-11 23:38:31.025871
EWM	iShares MSCI Malaysia Index Fund	The investment seeks to track the investment results of the MSCI Malaysia Index.\n The fund will at all times invest at least 90% of its assets in the securities of its underlying index and in depositary receipts representing securities in its underlying index. The underlying index consists of stocks traded primarily on the Kuala Lumpur Stock Exchange. It may include large-, mid- or small-capitalization companies. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.567351	2020-10-11 23:38:31.025871
EWSC	Invesco S&P SmallCap 600 Equal Weight	The investment seeks to track the investment results (before fees and expenses) of the S&P SmallCap 600® Equal Weight Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities that comprise the underlying index. The underlying index is an equal-weighted version of the S&P SmallCap 600® Index. The index provider compiles, maintains and calculates the underlying index, which consists of U.S. common equities listed on the NYSE, and also may include equity interests in real estate investment trusts ("REITs") and business development companies ("BDCs"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.587717	2020-10-11 23:38:31.025871
EYLD	Cambria ETF Trust Emerging Shareholder Yield	The investment seeks investment results that correspond (before fees and expenses) generally to the price and yield performance of its underlying index, the Cambria Emerging Shareholder Yield Index (the "underlying index").\n Under normal market conditions, the fund will invest at least 80% of its total assets in the components of the underlying index and in depositary receipts representing components of the underlying index. The underlying index is comprised of equity securities of issuers in emerging foreign markets. The fund may invest up to 20% of its net assets in instruments not included in the underlying index.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:02.646723	2020-10-11 23:38:31.025871
FIW	First Trust Water	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the ISE Clean Edge Water Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is a modified market-capitalization weighted portfolio of 36 stocks of companies as of March 29, 2018 that derive a substantial portion of their revenues from the potable and wastewater industries which are generally industrial and utilities companies.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.003466	2020-10-11 23:38:31.025871
FNY	First Trust Mid Cap Growth AlphaDEX Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the Nasdaq AlphaDEX® Mid Cap Growth Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is designed to select growth stocks from the NASDAQ US 600 Mid Cap Index (the "base index") that may generate positive alpha, or risk-adjusted returns, relative to traditional indices through the use of the AlphaDEX® selection methodology.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.167416	2020-10-11 23:38:31.025871
EBND	SPDR Bloomberg Barclays Emerging Markets Local Bond	The investment seeks to provide investment results that correspond generally to the price and yield performance of the Bloomberg Barclays EM Local Currency Government Diversified Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index or in securities that the Sub-Adviser determines have economic characteristics that are substantially identical to the economic characteristics of the securities that comprise the index. The index is designed to measure the performance of the fixed-rate local currency sovereign debt of emerging market countries. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.049242	2020-10-11 23:38:31.023007
EMDV	ProShares MSCI Emerging Markets Dividend Growers	The investment seeks investment results, before fees and expenses, that track the performance of the MSCI Emerging Markets Dividend Masters Index (the "index").\n Under normal circumstances, the fund will invest at least 80% of its total assets in component securities. The index, constructed and maintained by MSCI, targets companies that are currently members of the MSCI Emerging Markets Index ("MSCI Emerging Markets") and have increased dividend payments each year for at least 7 years.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:02.261452	2020-10-11 23:38:31.023007
EUFN	iShares MSCI Europe Financial Sector Index Fund	The investment seeks to track the investment results of the MSCI Europe Financials Index composed of developed market European equities in the financials sector.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is a free float-adjusted market capitalization-weighted index designed to measure the combined equity market performance of the financials sector of developed market countries in Europe.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.495692	2020-10-11 23:38:31.023007
EVIX	VelocityShares 1x Long VSTOXX Futures ETN	The investment seeks to link to the VSTOXX Short-Term Futures Investable Index (the “Long Investable Index”).\n The Long Investable Index replicates the performance of a long position in a portfolio of VSTOXX futures designed to provide a long exposure to constant-maturity one-month forward, one-month implied volatilities on the underlying EURO STOXX 50® Index, taking into account the bid-ask spread in the roll procedure, which will reduce the value of the Long Investable Index, and therefore the Long Securities.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:02.526011	2020-10-11 23:38:31.023007
FENY	Fidelity MSCI Energy Index	The investment seeks to provide investment returns that correspond, before fees and expenses, generally to the performance of the MSCI USA IMI Energy Index.\n The fund invests at least 80% of assets in securities included in the fund's underlying index. The fund's underlying index is the MSCI USA IMI Energy Index, which represents the performance of the energy sector in the U.S. equity market. It may or may not hold all of the securities in the MSCI USA IMI Energy Index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.815933	2020-10-11 23:38:31.023007
FFTI	FormulaFolios Tactical Income	The investment seeks to replicate investment results that generally correspond to the Inspire Corporate Bond Impact Index.\n The fund generally will invest at least 80% of its total assets in the component securities of the index. The index provider selects domestic corporate bonds issued by companies that have market capitalizations of $5 billion or more and are included in the S&P 500 Investment Grade Corporate Bond Index using the Inspire Impact Score®, a proprietary selection methodology that is designed to assign a score to a particular security based on the security's alignment with biblical values and the positive impact that company has on the world.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:02.854736	2020-10-11 23:38:31.023007
FJP	First Trust Japan AlphaDEX Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the NASDAQ AlphaDEX® Japan Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks and/or depositary receipts that comprise the index. The index is designed to select stocks from the NASDAQ Japan Index (the "base index") that may generate positive alpha, or risk-adjusted returns, relative to traditional indices through the use of the AlphaDEX® selection methodology.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.011806	2020-10-11 23:38:31.023007
FOX	Twenty-First Century Fox Inc.	Twenty-First Century Fox Inc is a diversified media and entertainment company. It operates in various business segments namely Cable Network Programming, Television, Filmed Entertainment, and Direct Broadcast Satellite Television.	{"Consumer Cyclical","Media - Diversified",Entertainment}	{DIS,CBS,AMCX,VIAB,NWSA,GHC,TGNA,LGF.A}	Entertainment	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:03.189324	2020-10-11 23:38:31.023007
FTAI	Fortress Transportation and Infrastructure Investors LLC	Fortress Transportation and Infrastructure Investors LLC is an infrastructure and equipment leasing company. It leases aircrafts, shipping containers, support vessels and anchor handling tug supply vessels.	{Industrials,"Rental & Leasing Services","Consulting & Outsourcing"}	{}	Consulting & Outsourcing	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:03.280545	2020-10-11 23:38:31.023007
FUT	ProShares Managed Futures Strategy	The investment seeks to provide positive returns that are not directly correlated to broad equity or fixed income markets.\n Under normal market conditions, the fund invests in a portfolio of commodity futures contracts and currency and U.S. Treasury futures contracts. The Advisor attempts to capture the economic benefit derived from rising and declining trends based on the price changes of these Futures Contracts. It will also hold cash or cash equivalents such as short-term U.S. Treasury securities or other high credit quality, short-term fixed-income or similar securities for direct investment or as collateral for Futures Contracts. It is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:03.334343	2020-10-11 23:38:31.023007
FXG	First Trust Cons. Staples AlphaDEX	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the StrataQuant® Consumer Staples Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is a modified equal-dollar weighted index designed by IDI to objectively identify and select stocks from the Russell 1000® Index in the consumer staples sector that may generate positive alpha relative to traditional passive-style indices through the use of the AlphaDEX® selection methodology.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.365244	2020-10-11 23:38:31.023007
ECA	Encana Corporation	Encana Corp is an independent oil and gas producer in North America. It is engaged in developing diverse resource plays producing natural gas, oil and Natural Gas Liquids (NGL). The company is also engaged in marketing of natural gas, oil and NGLs.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{CNQ,SU,DVN,MRO,BT,ATH,VET}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:02.05783	2020-10-11 23:38:31.023428
EDOG	ALPS Emerging Sector Dividend Dogs	The investment seeks investment results that replicate as closely as possible, before fees and expenses, the performance of the S-Network® Emerging Sector Dividend Dogs Index (the "index").\n The index is a rules-based index intended to give investors a means of tracking the overall performance of the highest dividend paying stocks (i.e. "Dividend Dogs") in the S-Network® Emerging Markets Index, a universe of mainly large capitalization stocks domiciled in emerging markets (the "S-Network Emerging Markets") on a sector-by-sector basis.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.10842	2020-10-11 23:38:31.023428
ENIA	Enel Americas S.A. American Depositary Shares	Enel Americas SA, generates, transmits and distributes electric energy in Argentina, Brazil, Colombia and Peru.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{CIG,ELP,PCG,SRE,AES,PPL,CPL,DUK,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:02.306847	2020-10-11 23:38:31.023428
EQAL	Invesco Russell 1000 Equal Weight	The investment seeks to track the investment results (before fees and expenses) of the Russell 1000® Equal Weight Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in common stocks that comprise the underlying index. Strictly in accordance with its guidelines and mandated procedures, Frank Russell Company compiles and maintains the underlying index, which is composed of all of the securities in the Russell 1000® Index (the "Russell 1000") "equally weighted."	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.376203	2020-10-11 23:38:31.023428
EQR	Equity Residential of Beneficial Interest	Equity Residential is a real estate investment trust which is focused on the acquisition, development and management of rental apartment properties in urban and high-density suburban coastal gateway markets in the US.	{"Real Estate","REIT - Residential",REITs}	{MAA,AIV,AVB,UDR,IRT,AMH}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:02.390059	2020-10-11 23:38:31.023428
FANH	Fanhua Inc.	Fanhua Inc, formerly CNinsure Inc is an independent insurance intermediary company operating in China. The company's segments include insurance agency business, insurance brokerage business and claims adjustment.	{"Financial Services","Insurance Brokers","Brokers & Exchanges"}	{CRVL,CRD.B,WAGE,WLTW}	Brokers & Exchanges	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:02.670348	2020-10-11 23:38:31.023428
FBGX	UBS AG FI Enhanced Large Cap Growth ETN	The investment seeks a return linked to the Russell 1000® Growth Total Return Index (the “index”).\n The UBS AG FI Enhanced Large Cap Growth ETN due June 19, 2024 (the “Securities”) are a series of FI Enhanced ETNs. The level of the index reflects both the price performance of the index constituent Securities and the reinvestment of dividends on the index constituent securities. The Securities are two times leveraged with respect to the index and, as a result, may benefit from two times any positive, but will be exposed to two times any negative, quarterly performance of the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.69495	2020-10-11 23:38:31.023428
FDM	First Trust DJ Select MicroCap	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the Dow Jones Select MicroCap Index(SM).\n The fund will normally invest at least 90% of its net assets in common stocks that comprise the index. The index is comprised of selected U.S. micro-capitalization companies chosen from all common stocks traded on the New York Stock Exchange, the NYSE MKT and NASDAQ® with limited partnerships excluded. It will normally invest at least 80% of its net assets in common stocks of U.S. micro-capitalization companies which are publicly traded in the United States.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.766561	2020-10-11 23:38:31.023428
FIBR	iShares Edge U.S. Fixed Income Balanced Risk	The investment seeks to track the investment results of the Bloomberg Barclays U.S. Fixed Income Balanced Risk Index.\n The underlying index measures the performance of the corporate and mortgage portion of the Bloomberg Barclays U.S. Universal Index while targeting an equal allocation between interest rate and credit spread risk. The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. It is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:02.886164	2020-10-11 23:38:31.023428
FINX	Global X FinTech ETF	The investment seeks to provide investment results that correspond generally to the price and yield performance, before fees and expenses, of the Indxx Global Fintech Thematic Index.\n The fund invests at least 80% of its total assets in the securities of the underlying index. The underlying index is designed to provide exposure to exchange-listed companies in developed markets that provide financial technology products and services, including companies involved in mobile payments, peer-to-peer (P2P) and marketplace lending, financial analytics software and alternative currencies, as defined by the index provider. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.964089	2020-10-11 23:38:31.023428
FIVA	Fidelity International Value Factor	The investment seeks to provide investment returns that correspond, before fees and expenses, generally to the performance of the Fidelity ® International Value Factor Index.\n The fund normally invests at least 80% of assets in securities included in the Fidelity ® International Value Factor Index and in depository receipts representing securities included in the index. The Fidelity ® International Value Factor Index is designed to reflect the performance of stocks of large- and mid-capitalization developed international companies that have attractive valuations.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.997338	2020-10-11 23:38:31.023428
FLBR	Franklin FTSE Brazil	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the FTSE Brazil RIC Capped Index (the FTSE Brazil Capped Index).\n Under normal market conditions, the fund invests at least 80% of its assets in the component securities of the FTSE Brazil Capped Index and in depositary receipts representing such securities. The FTSE Brazil Capped Index is based on the FTSE Brazil Index and is designed to measure the performance of Brazilian large- and mid-capitalization stocks. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.030104	2020-10-11 23:38:31.023428
EDIV	SPDR S&P Emerging Markets Dividend	The investment seeks investment results that, before fees and expenses, correspond generally to the total return performance of the S&P Emerging Markets Dividend Opportunities Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index and in depositary receipts based on securities comprising the index. The index is designed to measure the performance of 100 high-yielding emerging market common stocks. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.102325	2020-10-11 23:38:31.021426
EGO	Eldorado Gold Corporation	Eldorado is a gold and base metals producer with mining, development and exploration operations in Turkey, Greece, Romania, Canada and Brazil. The Company has a highly skilled workforce, safe and responsible operations, and a portfolio of quality assets. 	{"Basic Materials",Gold,"Metals & Mining"}	{AEM,SVM,NGD,AU,HL}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:02.197052	2020-10-11 23:38:31.021426
ENLK	EnLink Midstream Partners LP Representing Limited Partnership Interests	EnLink Midstream Partners LP provides midstream energy services, including gathering, transmission, processing, fractionation, brine services and marketing, to producers of natural gas, NGLs, crude oil, and condensate.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{EPD,WMB,DCP,TRGP,KMI}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:02.314201	2020-10-11 23:38:31.021426
FBND	Fidelity Total Bond	The investment seeks a high level of current income.\n Normally, the fund invests at least 80% of assets in debt securities of all types and repurchase agreements for those securities. It uses the Bloomberg Barclays U.S. Universal Bond Index as a guide in allocating assets across the investment-grade, high yield, and emerging market asset classes. The fund invests up to 20% of assets in lower-quality debt securities. It is managed to have similar overall interest rate risk to the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.708133	2020-10-11 23:38:31.021426
FCAP	First Capital Inc.	First Capital Inc is a holding company. The Company, through its wholly-owned subsidiary, provides banking services to individuals and business customers through seventeen locations in Indiana and Kentucky.	{"Financial Services","Savings & Cooperative Banks",Banks}	{FSFG,FBC}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:02.723194	2020-10-11 23:38:31.021426
FDTS	First Trust Developed Markets ex-US Small Cap AlphaDEX Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the NASDAQ AlphaDEX® Developed Markets Ex-US Small Cap Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks and/or depositary receipts that comprise the index. The index is designed to select small cap stocks from the NASDAQ Developed Markets Ex-US Index that may generate positive alpha, or risk-adjusted returns, relative to traditional indices through the use of the AlphaDEX® selection methodology.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.795542	2020-10-11 23:38:31.021426
FIXD	First Trust TCW Opportunistic Fixed Income ETF	The investment seeks to maximize long-term total return.\n Under normal market conditions, the fund invests at least 80% of its net assets (including investment borrowings) in a portfolio of fixed income securities. Its average portfolio duration will vary from between 0 to 10 years. The fund may invest a significant portion of its assets in securitized investment products, including up to 50% in each of ABS, RMBS and CMBS. It may also utilize listed and over-the-counter ("OTC") traded derivatives instruments for duration/yield curve management and/or hedging purposes. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.010724	2020-10-11 23:38:31.021426
FPF	First Trust Intermediate Duration Preferred & Income Fund of Beneficial Interest	First Trust Inter Dur Pref& Income Fund is a non-diversified, closed-end management investment company. Its primary investment objective is to seek a high level of current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.201174	2020-10-11 23:38:31.021426
FTXO	First Trust Nasdaq Bank ETF	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the Nasdaq US Smart Banks Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the underlying index. The underlying index is designed to select bank stocks from the NASDAQ US Benchmark Index based on a ranking methodology of three price factors which aims to select companies. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.323467	2020-10-11 23:38:31.021426
FXN	First Trust Energy AlphaDEX Fund	The investment seeks results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the StrataQuant® Energy Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is a modified equal-dollar weighted index designed by IDI to objectively identify and select stocks from the Russell 1000® Index in the energy sector that may generate positive alpha relative to traditional passive-style indices through the use of the AlphaDEX® selection methodology.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.371064	2020-10-11 23:38:31.021426
GPP	Green Plains Partners LP	Green Plains Partners LP is a mid-stream oil and gas company in the United States. It acquires revenues for the storage and transportation of ethanol on behalf of its parent company, Green Plains Inc.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{}	Oil & Gas - Midstream	Energy	NASDAQ Global Market	\N	2020-10-11 23:12:03.757464	2020-10-11 23:38:31.021426
GRNB	VanEck Vectors Green Bond	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the S&P Green Bond Select Index (the "index").\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The index is comprised of bonds issued for qualified "green" purposes and seeks to measure the performance of green-labeled bonds issued globally. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.796175	2020-10-11 23:38:31.021426
GSVC	GSV Capital Corp	GSV Capital Corp is an externally managed, non-diversified closed-end management investment company. The company has elected to be treated as a business development company.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:03.854153	2020-10-11 23:38:31.021426
EDV	Vanguard Extended Duration Treasury	The investment seeks to track the performance of an index of extended-duration zero-coupon U.S. Treasury securities.\n The fund employs an indexing investment approach designed to track the performance of the Bloomberg Barclays U.S. Treasury STRIPS 20-30 Year Equal Par Bond Index. This index includes zero-coupon U.S. Treasury securities (Treasury STRIPS), which are backed by the full faith and credit of the U.S. government, with maturities ranging from 20 to 30 years. The fund invests by sampling the index. At least 80% of the fund's assets will be invested in U.S. Treasury securities held in the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.11332	2020-10-11 23:38:31.028166
EFT	Eaton Vance Floating Rate Income Trust of Beneficial Interest	Eaton Vance Floating-rate Income Trust is a closed end management investment company. Its objective is high current income and its secondary objective is capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.169361	2020-10-11 23:38:31.028166
ETY	Eaton Vance Tax-Managed Diversified Equity Income Fund of Beneficial Interest	Eaton Vance Tax-mgd Dvsf Eq Inc Fd is a United States based diversified, closed-end management investment company. Its primary investment objective is to provide current income and gains. The secondary objective of the fund is capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.488929	2020-10-11 23:38:31.028166
EXPR	Express Inc.	Express, Inc. is a specialty retailer company in apparel and related. It serves both both women's and men's merchandise. The company operates in Express brick-and-mortar retail stores and e-commerce operations.	{"Consumer Cyclical","Apparel Stores","Retail - Apparel & Specialty"}	{URBN,ZUMZ,ANF,AEO,BKE,GPS,TLRD,BEBE}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:02.626494	2020-10-11 23:38:31.028166
FCVT	First Trust SSI Strategic Convertible Securities E	The investment seeks total return.\n Under normal market conditions, the fund seeks to achieve its investment objective by investing at least 80% of its net assets (including investment borrowings) in a portfolio of U.S. and non-U.S. convertible securities. In general, convertible securities combine the investment characteristics of bonds and common stocks and typically consist of debt securities or preferred securities that may be converted or exchanged within a specified period of time into a certain amount of common stock or other equity security of the same or a different issuer. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.747086	2020-10-11 23:38:31.028166
FIX	Comfort Systems USA Inc.	Comfort Systems USA Inc provides mechanical contracting services. It offers heating, ventilation and air conditioning, plumbing, piping and controls, as well as off-site construction, electrical, monitoring and fire protection.	{Industrials,"Engineering & Construction"}	{EME,PWR,TPC,IESC}	Engineering & Construction	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:03.011203	2020-10-11 23:38:31.028166
FLC	Flaherty & Crumrine Total Return Fund Inc	Flaherty & Crumrine Total Return Fund Inc is an investment fund. The Fund's investment objective is to provide its common shareholders with high current income. The Fund's secondary investment objective is capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.033118	2020-10-11 23:38:31.028166
FLJH	Franklin FTSE Japan Hedged	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the FTSE Japan RIC Capped Hedged to USD Index (the FTSE Japan Capped Hedged Index).\n Under normal market conditions, the fund invests at least 80% of its assets in the component securities of the FTSE Japan Capped Hedged Index and in depositary receipts representing such securities, including indirectly through the Franklin FTSE Japan ETF (underlying fund). The FTSE Japan Capped Hedged Index is based on the FTSE Japan Hedged to USD Index and is designed to measure the performance of Japanese large- and mid-capitalization stocks.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.056437	2020-10-11 23:38:31.028166
FNX	First Trust Mid Cap Core AlphaDEX Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the Nasdaq AlphaDEX® Mid Cap Core Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is designed to select stocks from the NASDAQ US 600 Mid Cap Index (the "base index") that may generate positive alpha, or risk-adjusted returns, relative to traditional indices through the use of the AlphaDEX® selection methodology.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.167248	2020-10-11 23:38:31.028166
FPH	Five Point Holdings LLC Class A	Five Point Holdings LLC is an owner and developer of mixed-use, master-planned communities in coastal California. The house sales generates maximum revenue for the company.	{"Real Estate","Real Estate - General","Real Estate Services"}	{NEN,MMI}	Real Estate Services	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:03.201516	2020-10-11 23:38:31.028166
GAB	Gabelli Equity Trust Inc. (The)	To achieve long-term growth of capital primarily through investment in equity securities, with income being a secondary objective. The Fund will invest at least 80% of its assets in equity securities. The Fund may invest, from time to time, in shares of other investment companies. It may purchase or write call or put options on securities or indices. It may invest up to 10% of its net assets in securities, for which the markets are illiquid. The Fund invests in various industries, including food and beverage, financial services, energy and utilities, telecommunications, healthcare, diversified industrial, consumer products, publishing, entertainment, cable and satellite, hotels and gaming, and equipment and supplies.	{"Financial Services","Asset Management"}	{}			New York Stock Exchange	\N	2020-10-11 23:12:03.391328	2020-10-11 23:38:31.028166
GFN	General Finance Corporation	General Finance Corp manufactures, leases & sells storage & freight containers, portable container buildings, portable liquid storage tank containers & modular buildings. The Company operates in Asia-Pacific area and North America.	{Industrials,"Rental & Leasing Services","Consulting & Outsourcing"}	{MINI,MGRC,GEF,CAI,CYRX}	Consulting & Outsourcing	Industrials	NASDAQ Global Market	\N	2020-10-11 23:12:03.50725	2020-10-11 23:38:31.028166
GRFS	Grifols S.A.	Grifols SA is a pharmaceutical company. It primarily collects the liquid portion of human blood called plasma and then manufactures and sells plasma-derived products globally. It also operates diagnostics, hospital supplies, and raw materials businesses.	{Healthcare,Biotechnology}	{AMGN,BAX,JNJ,KMB,ARDM,MYL,CBPO,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:03.781813	2020-10-11 23:38:31.028166
EEMD	AAM S&P Emerging Markets High Dividend Value ETF	The investment seeks to track the total return performance, before fees and expenses, of the S&P Emerging Markets Dividend and Free Cash Flow Yield Index.\n At least 80% of the fund's total assets (exclusive of any collateral held from securities lending) will be invested in the component securities of the index and depositary receipts representing index components. The index is a rules-based, equal-weighted index that is designed to provide exposure to the constituents of the S&P Emerging Plus LargeMidCap Index that exhibit both high dividend yield and sustainable dividend distribution characteristics, while maintaining diversified sector exposure.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.129256	2020-10-11 23:38:31.024353
EGBN	Eagle Bancorp Inc.	Eagle Bancorp Inc is a part of the financial services domain. The company functions through its wholly-owned subsidiary, EagleBank in providing community banking solutions to businesses and professionals.	{"Financial Services","Banks - Regional - US",Banks}	{GLBZ,SASR,ORRF,SHBI,OLBK,MTB,HBMD}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:02.185048	2020-10-11 23:38:31.024353
EQM	EQM Midstream Partners, LP	EQT Midstream Partners LP owns, operates, acquires and develops midstream assets in the Appalachian Basin. It provides natural gas transmission, storage and gathering services. The gathering generates maximum revenue for the company.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{NI,NFG,D,EQT,WES,KMI,SXE,ENLK}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:02.389427	2020-10-11 23:38:31.024353
FAN	First Trust Global Wind Energy	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the ISE Clean Edge Global Wind Energy Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks or in depositary receipts representing securities in the index. The index provides a benchmark for investors interested in tracking public companies throughout the world that are active in the wind energy industry based on analysis of the products and services offered by those companies. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.668491	2020-10-11 23:38:31.024353
FCA	First Trust China AlphaDEX Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the NASDAQ AlphaDEX® China Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks and/or depositary receipts that comprise the index. The index is designed to select stocks from the NASDAQ China Index (the "base index") that may generate positive alpha, or risk-adjusted returns, relative to traditional indices through the use of the AlphaDEX® selection methodology.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.716858	2020-10-11 23:38:31.024353
FLL	Full House Resorts Inc.	Full House Resorts Inc, through its subsidiaries, owns, operates, develops, manages, and invests in casinos and related hospitality and entertainment facilities. The Company manages its casinos based on geographic regions within the United States.	{"Consumer Cyclical","Resorts & Casinos","Travel & Leisure"}	{PENN,CNTY,BYD,MGM,DDE,UWN,CZR,ERI,GDEN,NYNY}	Travel & Leisure	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:03.062652	2020-10-11 23:38:31.024353
FLOT	iShares Floating Rate Bond	The investment seeks to track the investment results of the Bloomberg Barclays US Floating Rate Note < 5 Years Index (the "underlying index"), which measures the performance of U.S. dollar-denominated, investment-grade floating rate notes.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index, but which BFA believes will help the fund track the underlying index.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:03.074894	2020-10-11 23:38:31.024353
FQAL	Fidelity Quality Factor	The investment seeks to provide investment returns that correspond, before fees and expenses, generally to the performance of the Fidelity U.S. Quality Factor Index℠.\n The fund normally invests at least 80% of assets in securities included in the Fidelity U.S. Quality Factor Index℠, which is designed to reflect the performance of stocks of large and mid-capitalization U.S. companies with a higher quality profile than the broader market. It may lend securities to earn income for the fund.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.214931	2020-10-11 23:38:31.024353
FTLS	First Trust Long/Short Equity	The investment seeks to provide investors with long-term total return.\n The fund pursues its investment objective by establishing long and short positions in a portfolio of U.S. exchange-listed equity securities and/or U.S. exchange-traded funds ("ETFs") that provide exposure to such securities (collectively, "Equity Securities"). Under normal circumstances, at least 80% of the fund's net assets (including investment borrowings) will be exposed to Equity Securities. The fund may invest up to 20% of its net assets (including investment borrowings) in U.S. exchange-listed equity index futures contracts.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.29972	2020-10-11 23:38:31.024353
FULT	Fulton Financial Corporation	Fulton Financial Corp is a multi-bank financial holding company. It provides banking and financial services to businesses and consumers. The company's primary source of revenue is interest income on loans.	{"Financial Services","Banks - Regional - US",Banks}	{DNBF,BMTC,PWOD,ASRV,MTB,FCF,FNB,PNC,ACNB,PFIS}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:03.327633	2020-10-11 23:38:31.024353
FV	First Trust Dorsey Wright Focus 5 ETF	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an index called the Dorsey Wright Focus Five Index (the "index").\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in the exchange-traded funds ("ETFs") that comprise the index. The index is designed to provide targeted exposure to the five First Trust sector-based ETFs that the index provider believes offer the greatest potential to outperform the other ETFs in the selection universe and that satisfy certain trading volume and liquidity requirements. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.33742	2020-10-11 23:38:31.024353
GDP	Goodrich Petroleum Corporation	Goodrich Petroleum Corp is an independent oil and natural gas company. It is engaged in the exploration, development and production of oil and natural gas in the Tuscaloosa Marine Shale region.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{}	Oil & Gas - E&P	Energy	NYSE American	\N	2020-10-11 23:12:03.461707	2020-10-11 23:38:31.024353
EMCB	WisdomTree Emerging Markets Corporate Bond Fund	The investment seeks a high level of total return consisting of both income and capital appreciation.\n Under normal circumstances, the fund will invest at least 80% of its net assets, plus the amount of any borrowings for investment purposes, in corporate debt. The manager attempts to maintain an aggregate portfolio duration of between two and ten years under normal market conditions. The fund may invest up to 20% of its net assets in derivatives, such as swaps and forward currency contracts. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.253101	2020-10-11 23:38:31.031328
EPHE	iShares MSCI Philippines	The investment seeks to track the investment results of the MSCI Philippines Investable Market Index (IMI).\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is a free float-adjusted market capitalization-weighted index designed to measure the performance of the Philippine equity markets. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.356771	2020-10-11 23:38:31.031328
ESBK	Elmira Savings Bank NY (The)	Elmira Savings Bank Elmira NY engaged in attracting deposits from public and originating mortgag e loans secured by first and second liens on one to four family residences.It also issues commercial loans and consumer loans and offers financial services.	{"Financial Services","Banks - Regional - US",Banks}	{FFIC,DCOM,TRST,GCBC,LSBK,CARV,PBHC}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:02.4253	2020-10-11 23:38:31.031328
EWX	SPDR S&P Emerging Markets Small Cap	The investment seeks investment results that, before fees and expenses, correspond generally to the total return performance of the S&P Emerging Markets Under USD2 Billion Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index and in depositary receipts based on securities comprising the index. The index is a float-adjusted market capitalization weighted index designed to represent the small capitalization segment of emerging countries included in the S&P Global BMI. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.59348	2020-10-11 23:38:31.031328
FBK	FB Financial Corporation	FB Financial Corp is a bank holding company. The Company through its wholly-owned bank subsidiary provides commercial and consumer banking services to clients in select markets in Tennessee, North Alabama and North Georgia.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:02.701703	2020-10-11 23:38:31.031328
FBT	First Trust Amex Biotech Index Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the NYSE Arca Biotechnology IndexSM.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is an equal-dollar weighted index designed to measure the performance of a cross section of companies in the biotechnology industry that are primarily involved in the use of biological processes to develop products or provide services.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.714259	2020-10-11 23:38:31.031328
FDRR	Fidelity Dividend ETF for Rising Rates	The investment seeks to provide investment returns that correspond, before fees and expenses, generally to the performance of the Fidelity Dividend Index for Rising Rates℠.\n The fund normally invests at least 80% of assets in securities included in the underlying index and in depository receipts representing securities included in the underlying index. The underlying index is designed to reflect the performance of stocks of large and mid-capitalization dividend-paying companies that are expected to continue to pay and grow their dividends and have a positive correlation of returns to increasing 10-year U.S. Treasury yields.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:02.781855	2020-10-11 23:38:31.031328
FKO	First Trust South Korea AlphaDEX Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the NASDAQ AlphaDEX® South Korea Index.\n The fund normally invests at least 90% of its net assets (including investment borrowings) in common stocks and/or depositary receipts that comprise the index. The index is designed to select stocks from the NASDAQ South Korea Index (the "base index") that may generate positive alpha, or risk-adjusted returns, relative to traditional indices through the use of the AlphaDEX® selection methodology.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.017291	2020-10-11 23:38:31.031328
FTA	First Trust Large Cap Value AlphaDEX Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the Nasdaq AlphaDEX® Large Cap Value Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is designed to select value stocks from the NASDAQ US 500 Large Cap Index (the "base index") that may generate positive alpha, or risk-adjusted returns, relative to traditional indices through the use of the AlphaDEX® selection methodology.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.275086	2020-10-11 23:38:31.031328
FUND	Sprott Focus Trust Inc.	Sprott Focus Trust Inc is a diversified closed-end investment company. It aims for a long-term growth of capital through investments made in various sectors such as materials, information technology, consumer discretionary, financial services and energy.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:03.331664	2020-10-11 23:38:31.031328
FVE	Five Star Senior Living Inc.	Five Star Senior Living Inc, formerly Five Star Quality Care Inc is engaged in operating senior living communities, including independent living or congregate care communities, assisted living communities and skilled nursing facilities.	{Healthcare,"Long-Term Care Facilities","Health Care Providers"}	{ENSG,ADUS,NHC,BKD,CSU,GEN,RHE,XLV}	Health Care Providers	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:03.344156	2020-10-11 23:38:31.031328
GIII	G-III Apparel Group LTD.	G-III Apparel Group Ltd designs, manufactures & markets apparel, including outerwear, dresses, sportswear, swimwear, women's suits & women's performance wear, as well as footwear, luggage & women handbags, small leather goods & cold weather accessories.	{"Consumer Cyclical","Apparel Manufacturing","Manufacturing - Apparel & Furniture"}	{COLM,GES,PVH,RL,EVK,SQBG}	Manufacturing - Apparel & Furniture	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:03.546949	2020-10-11 23:38:31.031328
GPRK	Geopark Ltd	GeoPark Ltd is an oil and natural gas exploration and production company with operations in Latin America. The company operates in Chile, Colombia, Brazil, Peru and Argentina.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{EC,MEOH,GTE,YPF,TOT,PBR,NBR}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:03.756695	2020-10-11 23:38:31.031328
FCAL	First Trust California Municipal High Income ETF	The investment seeks to provide current income that is exempt from regular federal income taxes and California income taxes, and its secondary objective is long-term capital appreciation.\n The fund seeks to achieve its investment objectives by investing at least 80% of its net assets (including investment borrowings) in municipal debt securities that pay interest that is exempt from regular federal income taxes and California income taxes. It will invest no more than 50% of its net assets in Municipal Securities that are, at the time of investment, not investment grade, commonly referred to as "high yield" or "junk" bonds. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:02.718544	2020-10-11 23:38:31.02582
FPEI	First Trust Institutional Preferred Securities and Income	The investment seeks total return and to provide current income.\n The fund invests at least 80% of its net assets (including investment borrowings) in institutional preferred securities and income-producing debt securities ("Income Securities"). Preferred securities are a type of equity security that have preference over common stock in the payment of distributions and the liquidation of a company's assets, but are generally junior to all forms of the company's debt, including both senior and subordinated debt. The fund's investments in preferred securities will primarily be in institutional preferred securities. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.197488	2020-10-11 23:38:31.02582
GCVRZ	Sanofi Contingent Value Right (Expiring 12/31/2020)	Sanofi SA is healthcare company which is engaged in the research, development, manufacture and marketing of therapeutic solutions. It business activities include operations of specialty care, Established Prescription Products and vaccines operations.	{Healthcare,"Drug Manufacturers - Major","Drug Manufacturers"}	{XLV}	Drug Manufacturers	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:03.447472	2020-10-11 23:38:31.02582
GEX	VanEck Vectors Global Alternative Energy	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the Ardour Global IndexSM (Extra Liquid).\n The fund normally invests at least 80% of its total assets in stocks of companies primarily engaged in the business of alternative energy. Such companies may include small- and medium-capitalization companies and foreign issuers. Alternative energy refers to the generation of power through environmentally friendly, non traditional sources. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.49981	2020-10-11 23:38:31.02582
GIGB	Goldman Sachs Access Investment Grade Corporate Bond	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the FTSE Goldman Sachs Investment Grade Corporate Bond Index (the "index").\n The fund seeks to achieve its investment objective by investing at least 80% of its assets (exclusive of collateral held from securities lending) in securities included in its underlying index. The index is a rules-based index that is designed to measure the performance of investment grade, corporate bonds denominated in U.S. dollars ("USD") that meet certain liquidity and fundamental screening criteria.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.543558	2020-10-11 23:38:31.02582
GNTX	Gentex Corporation	Gentex Corp designs and manufactures automatic-dimming rearview mirrors and electronics for the automotive industry, dimmable aircraft windows for the aviation industry, and commercial smoke alarms and signaling devices for the fire protection industry.	{"Consumer Cyclical","Auto Parts",Autos}	{MGA,CTAS,DORM,ALV,FSS,LEA,APTV,THRM,VC}	Autos	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:03.681143	2020-10-11 23:38:31.02582
GSBD	Goldman Sachs BDC Inc.	Goldman Sachs BDC Inc is a closed-end management investment company focused on lending to middle-market companies. Its objective is to generate current income, capital appreciation through direct originations of secured debt and second lien debt.	{"Financial Services","Credit Services"}	{}	Credit Services	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.817219	2020-10-11 23:38:31.02582
GYRO	Gyrodyne  LLC	Gyrodyne LLC is a real estate investment trust. It is engaged in the investment and acquisition, ownership & management of a geographically diverse portfolio of medical office, industrial properties and development of industrial & residential properties.	{"Real Estate","REIT - Diversified",REITs}	{HR,ARE,LTC,NHI,MPW,DOC,SNH}	REITs	Real Estate	NASDAQ Capital Market	\N	2020-10-11 23:12:03.937039	2020-10-11 23:38:31.02582
HAYN	Haynes International Inc.	Haynes International Inc is a producer of nickel and cobalt-based alloys in sheet, coil and plate forms. It is engaged in the developing, manufacturing, marketing and distribution of alloys.	{Industrials,"Metal Fabrication","Industrial Products"}	{USAP,AKS,CRS,CMC,NUE,WOR,ATI,X,GSM}	Industrial Products	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:03.966192	2020-10-11 23:38:31.02582
HBIO	Harvard Bioscience Inc.	Harvard Bioscience Inc is a developer, manufacturer and marketer of scientific instruments, systems and lab consumables used to advance life science for basic research, drug discovery, clinical and environmental testing.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{BRKR,A,BDX,BIO,DHR,GE,JNJ,TMO,PKI,XLV}	Medical Instruments & Equipment	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:03.975911	2020-10-11 23:38:31.02582
IBDS	iShares iBonds Dec 2027 Term Corporate	The investment seeks to meet its investment objective generally by investing in individual securities which satisfy the criteria of the Bloomberg Barclays December 2027 Maturity Corporate Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index, except during the last months of the fund's operations, and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents, including shares of money market funds. The index is composed of U.S. dollar-denominated, investment-grade corporate bonds maturing in 2027. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.512187	2020-10-11 23:38:31.02582
IDLB	PowerShares FTSE International Low Beta Equal Weig	The investment seeks to track the investment results (before fees and expenses) of the FTSE Developed ex-U.S. Low Beta Equal Weight Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in common stocks that comprise the underlying index. Strictly in accordance with its guidelines and mandated procedures, the index provider compiles, maintains and calculates the underlying index, which is designed to provide exposure to constituents of the FTSE Developed ex US Index that exhibit low beta characteristics.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:04.596009	2020-10-11 23:38:31.02582
FLM	First Trust Global Engineering and Construction	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the ISE Global Engineering and ConstructionTM Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks or in depositary receipts representing securities in the index. The index provides a benchmark for investors interested in tracking public companies throughout the world that are active in the engineering and construction industries, based on analysis of the products and services offered by those companies.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.066211	2020-10-11 23:38:31.023428
FNI	First Trust Chindia	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the ISE ChIndiaTM Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in securities that comprise the index. The index is a modified market capitalization weighted of 50 American Depositary Receipts ("ADRs"), American Depositary Shares ("ADSs") and/or stocks selected from a universe of all listed ADRs, ADSs and/or stocks of companies from China and India currently trading on U.S. exchanges. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.1577	2020-10-11 23:38:31.023428
FPX	First Trust US Equity Opportunities	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the IPOX®-100 U.S. Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is an applied, market-cap weighted price index measuring the performance of the top U.S. companies ranked quarterly by market capitalization in the IPOX® U.S. Composite Index, a sub-index of the IPOX® Global Composite Index. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.209663	2020-10-11 23:38:31.023428
FUD	E-TRACS USB Bloomberg Commodity Index Exchange Traded Notes UBS Bloomberg CMCI Food ETN	The investment seeks to track the price and performance yield, before fees and expenses, of the UBS Bloomberg CMCI Food Total Return index. \n The fund is designed to be representative of the entire liquid forward curve of each commodity in the index. The index measures the collateralized returns from a diversified basket of agriculture and livestock futures contracts. It is comprised of the 11 agriculture futures contracts and two livestock futures contracts included in the CMCI with three target maturities for each individual commodity.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.323924	2020-10-11 23:38:31.023428
GECC	Great Elm Capital Corp.	Great Elm Capital Corp is an externally managed, non-diversified closed-end management investment company. The Company invests primarily in secured and senior unsecured debt instruments that it purchases in the secondary markets.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:03.475187	2020-10-11 23:38:31.023428
GSBC	Great Southern Bancorp Inc.	Great Southern Bancorp Inc is a bank holding company & a financial holding company. Its banking operation is engaged in the business of originating residential & commercial real estate loans, construction loans, commercial business loans & consumer loans.	{"Financial Services","Banks - Regional - US",Banks}	{UMBF,ATLO,WTBA,GFED,CBSH,QCRH,EFSC,HWBK,MOFG,WFC}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:03.820155	2020-10-11 23:38:31.023428
HBANN	Huntington Bancshares Incorporated Depositary Shares each representing a 1/40th interest in a share of 5.875% Series C Non-Cumulative Perpetual Preferred Stock	Huntington Bancshares Inc is a multi-state financial holding company. It provides full-service commercial, small business, consumer, and mortgage banking services, as well as automobile financing, investment management services, and among others.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:03.971821	2020-10-11 23:38:31.023428
HBM	Hudbay Minerals Inc. (Canada)	Hudbay Minerals Inc is an integrated mining company. It is engaged in producing copper concentrate and zinc metal, along with base and precious metals.	{"Basic Materials",Copper,"Metals & Mining"}	{TGB,TECK,PLM,SCCO,WRN,TRQ,WPM}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:03.983614	2020-10-11 23:38:31.023428
HCLP	Hi-Crush Partners LP representing limited partner interests	Hi-Crush Partners LP is a producer and supplier of monocrystalline sand used as a proppant in oil and gas industries. Its earns revenue by excavation, delivery and sale of Frac Sand.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{HAL,UNP,SLCA,BHGE}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:04.008766	2020-10-11 23:38:31.023428
HEWU	iShares Currency Hedged MSCI United Kingdom	The investment seeks to track the investment results of the MSCI United Kingdom 100% Hedged to USD Index, which consists of stocks traded primarily on the London Stock Exchange with the currency risk inherent in the securities included in the underlying index hedged to the U.S. dollar on a monthly basis.\n The fund generally will invest at least 90% of its assets in the component securities (including indirect investments through the underlying fund) and other instruments of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.086032	2020-10-11 23:38:31.023428
HTZ	Hertz Global Holdings Inc	Hertz Global Holdings Inc is engaged in car rental and leasing through the Hertz, Dollar, Thrifty, and Firefly brands. The company operates in countries, including locations in North America, Europe, and Asia.	{Industrials,"Rental & Leasing Services","Consulting & Outsourcing"}	{UHAL,URI,CAR}	Consulting & Outsourcing	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:04.35716	2020-10-11 23:38:31.023428
HURC	Hurco Companies Inc.	Hurco Companies Inc is an industrial technology company that designs, manufactures and sells computerized machine tools, consisting of vertical machining centers (mills) and turning centers (lathes), to companies in the metal working industry.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:04.37367	2020-10-11 23:38:31.023428
OESX	Orion Energy Systems Inc.	Orion Energy Systems Inc researches, develops, designs, manufactures, markets, sells and implements energy management systems consisting of energy efficient commercial and industrial interior and exterior lighting systems and related services.	{Technology,"Electronic Components","Computer Hardware"}	{LYTS,AYI,EFOI,RVLT,XLK}	Computer Hardware	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:08.303951	2020-10-11 23:38:31.023428
FLMB	Franklin Templeton ETF Trust Liberty Municipal Bond	The investment seeks a high level of current income that is exempt from federal income taxes.\n Under normal market conditions, the fund invests at least 80% of its net assets in municipal securities whose interest is free from federal income taxes, including the federal alternative minimum tax. Although the fund tries to invest all of its assets in tax-free securities, it is possible that up to 20% of the fund's net assets may be in securities that pay interest that may be subject to the federal alternative minimum tax and, although not anticipated, in securities that pay interest subject to other federal or state income taxes.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.070225	2020-10-11 23:38:31.028166
FSTA	Fidelity MSCI COnsumer Staples Index	The investment seeks to provide investment returns that correspond, before fees and expenses, generally to the performance of the MSCI USA IMI Consumer Staples Index.\n The fund invests at least 80% of assets in securities included in the fund's underlying index. The fund's underlying index is the MSCI USA IMI Consumer Staples Index, which represents the performance of the consumer staples sector in the U.S. equity market. It may or may not hold all of the securities in the MSCI USA IMI Consumer Staples Index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.269147	2020-10-11 23:38:31.028166
GCBC	Greene County Bancorp Inc.	Greene County Bancorp Inc is a federally chartered holding company of The Bank of Greene County. It is engaged in the business of attracting deposits from customers within its market area and investing those funds in loans.	{"Financial Services","Savings & Cooperative Banks",Banks}	{FFIC,ESBK,TRST,LSBK,NWBI,CARV,PBHC}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:03.435488	2020-10-11 23:38:31.028166
GHY	PGIM Global Short Duration High Yield Fund Inc.	Prudential Global Short Duration High Yield Fund Inc is a diversified, closed-end management investment company. The fund's investment objective is to provide a high level of current income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:03.535933	2020-10-11 23:38:31.028166
GRC	Gorman-Rupp Company (The)	Gorman-Rupp Co designs, manufactures and sells pumps and related equipment for use in water, wastewater, construction, industrial, original equipment, agriculture, fire protection, heating, ventilating and other liquid-handling application.	{Industrials,"Diversified Industrials","Industrial Products"}	{FELE,FLS,GGG,IEX,CFX,XYL,ITT}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:03.775735	2020-10-11 23:38:31.028166
GRMN	Garmin Ltd.	Garmin Ltd designs, develops, manufactures, markets and distributes hand-held, wrist-based, portable, fixed-mount, positioning system enabled and other navigation, communication and sensor-based products.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{FLIR,KVHI,CMTL,TRMB,JOUT,HON,COL,MSI,LLL,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:03.785614	2020-10-11 23:38:31.028166
GSEW	Goldman Sachs Equal Weight U.S. Large Cap Equity	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the Solactive US Large Cap Equal Weight Index (GTR).\n The fund seeks to achieve its investment objective by investing at least 80% of its assets (exclusive of collateral held from securities lending) in securities included in its underlying index. The index consists of equity securities of large capitalization U.S. issuers. The index is an equal-weight version of the Solactive US Large Cap Index, a market capitalization-weighted index that includes equity securities of approximately 500 of the largest U.S. companies.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:03.827378	2020-10-11 23:38:31.028166
HBI	Hanesbrands Inc.	Hanesbrands Inc designs, manufactures, sources and sells apparel essentials such as t-shirts, innerwear, casualwear, activewear, socks and hosiery. It markets its products under the brand names of Hanes, Champion, and Maidenform.	{"Consumer Cyclical","Apparel Manufacturing","Manufacturing - Apparel & Furniture"}	{GIL,ICON,OXM,PVH,RL}	Manufacturing - Apparel & Furniture	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:03.976243	2020-10-11 23:38:31.028166
HSCZ	iShares Currency Hedged MSCI EAFE Small-Cap	The investment seeks to track the investment results of the MSCI EAFE Small Cap 100% Hedged to USD Index, an equity benchmark which represents the performance of international small-cap stocks with the currency risk inherent in the securities included in the underlying index hedged to the U.S. dollar on a monthly basis.\n The fund generally will invest at least 90% of its assets in the component securities (including indirect investments through the underlying fund) and other instruments of the index and in investments that have economic characteristics that are substantially identical to the component securities of the index. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.305373	2020-10-11 23:38:31.028166
HTFA	Horizon Technology Finance Corporation 6.25% Notes due 2022		{}	{}			New York Stock Exchange	\N	2020-10-11 23:12:04.335095	2020-10-11 23:38:31.028166
HTUS	Hull Tactical US	The investment seeks long-term capital appreciation.\n The fund seeks to achieve its investment objective by taking long and short positions in one or more exchange-traded funds ("ETFs") that seek to track the performance of the S&P 500 Index. In seeking to achieve its investment objective, it may engage in short sales of S&P 500-related ETFs. The fund may also invest up to 10% of its total assets in leveraged or inverse ETFs that seek to deliver multiples (long), or the inverse (short), of the performance of the S&P 500 Index, respectively.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.350157	2020-10-11 23:38:31.028166
HYXE	iShares iBoxx $ High Yield ex Oil & Gas Corporate	The investment seeks to track the investment results of the Markit iBoxx® USD Liquid High Yield ex-Oil and Gas Index composed of a broad range of U.S. dollar-denominated, high yield corporate bonds that excludes those issued by companies in the Oil and Gas sector.\n The fund generally will invest at least 90% of its assets in the component securities of the index. The index is designed to reflect the performance of U.S. dollar-denominated, high yield corporate bonds, excluding those issued by companies in the Oil and Gas sector, as determined by the index provider. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:04.44552	2020-10-11 23:38:31.028166
IDA	IDACORP Inc.	Idacorp Inc is engaged in the generation, transmission, distribution, sale and purchase of electric energy regulated by the state regulatory commissions and the FERC. The company's segment is utility operations.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{EIX,PCG,HE,PNW,AVA,PNM,NWE,POR,XEL,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:04.586085	2020-10-11 23:38:31.028166
FTSL	First Trust Senior Loan Fund ETF	The investment seeks to provide high current income; the fund's secondary investment objective is the preservation of capital.\n Under normal market conditions, the fund seeks to outperform each of the primary index and secondary index by investing at least 80% of its net assets (including investment borrowings) in first lien senior floating rate bank loans ("Senior Loans").	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.309437	2020-10-11 23:38:31.025871
FVL	First Trust Value Line 100 Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the Value Line® 100 Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is an equal-dollar weighted index that is designed to objectively identify and select 100 stocks from the universe of stocks to which Value Line assigns a #1 ranking in the Value Line Timeliness Ranking System.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.34623	2020-10-11 23:38:31.025871
GHII	Invesco S&P High Income Infrastructure	The investment seeks investment results that correspond generally to the performance, before the fund's fees and expenses, of an equity index called the S&P High Income Infrastructure Index.\n The fund will invest at least 90% of its total assets in common stocks that comprise the index and depositary receipts representing common stocks included in the index. The index is designed to measure and monitor the performance of 50 high-yielding global equity securities of companies that engage in various infrastructure-related sub-industries. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.532735	2020-10-11 23:38:31.025871
GREK	Global X MSCI Greece	The investment seeks to provide investment results that correspond generally to the price and yield performance, before fees and expenses, of the MSCI All Greece Select 25/50 Index.\n The fund invests at least 80% of its total assets in the securities of the underlying index and in American Depositary Receipts ("ADRs") and Global Depositary Receipts ("GDRs") based on the securities in the underlying index. The underlying index is designed to represent the performance of the broad Greece equity universe. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.777285	2020-10-11 23:38:31.025871
HASI	Hannon Armstrong Sustainable Infrastructure Capital Inc.	Hannon Armstrong Sustainable Infrastructure Capital Inc is a specialty finance company. It provides debt and equity financing to energy efficiency and renewable energy markets.	{"Real Estate","REIT - Diversified",REITs}	{ATAX,EFC,WD,LADR}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:03.963088	2020-10-11 23:38:31.025871
HCAP	Harvest Capital Credit Corporation	Harvest Capital Credit Corp is an externally managed, non-diversified closed-end investment company. It provides customized financing solutions to small to mid-sized companies to generate both current income and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:03.993158	2020-10-11 23:38:31.025871
HCM	Hutchison China MediTech Limited	Hutchison China Meditech Ltd is a major drug manufacturing company. The company operates two platforms--an innovation platform and a commercial platform aimed at discovering and developing therapeutics in oncology and autoimmune diseases.	{Healthcare,"Drug Manufacturers - Major","Drug Manufacturers"}	{OMER,XLV}	Drug Manufacturers	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:04.009384	2020-10-11 23:38:31.025871
HQY	HealthEquity Inc.	HealthEquity Inc is a technology-enabled services platform that empowers consumers to make healthcare saving and spending decisions. Its products & services include healthcare saving & spending platform, health savings accounts, and healthcare incentives.	{Technology,"Health Information Services","Application Software"}	{UHAL,FCFS,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:04.286415	2020-10-11 23:38:31.025871
HSBC	HSBC Holdings plc.	HSBC Holdings PLC provides commercial banking, global banking, and wealth management among other services. It operates in Europe, Asia, the Middle East and North Africa, and North America.	{"Financial Services","Banks - Global",Banks}	{TD,BCS,SAN,BAC,DB,JPM,BBD,ITUB,C}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.304595	2020-10-11 23:38:31.025871
HTA	Healthcare Trust of America Inc. Class A	Healthcare Trust of America Inc is a real estate investment trust that owns and operates medical office buildings in the United States. The company generates all of its revenue in the United States.	{"Real Estate","REIT - Healthcare Facilities",REITs}	{DRE,VTR,UHT,HCP,SBRA,TIPT,WELL}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:04.325294	2020-10-11 23:38:31.025871
HTRB	Hartford Total Return Bond	The investment seeks a competitive total return, with income as a secondary objective.\n The fund invests at least 80% of its net assets in bonds that the sub-adviser considers to be attractive from a total return perspective along with current income. It normally invests at least 70% of its portfolio in investment grade debt securities and may invest up to 20% of its net assets in securities rated below investment grade (also known as "junk bonds"). The fund may invest up to 40% of its net assets in debt securities of foreign issuers, including from emerging markets, and up to 20% of its net assets in non-dollar securities.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.347213	2020-10-11 23:38:31.025871
HYGH	iShares Interest Rate Hedged High Yield Bond	The investment seeks to mitigate the interest rate risk of a portfolio composed of U.S. dollar-denominated, high yield corporate bonds.\n The fund seeks to achieve its investment objective by investing, under normal circumstances, at least 80% of its net assets in U.S. dollar-denominated high yield bonds, in one or more underlying funds that principally invest in high yield bonds, and in U.S. Treasury securities (or cash equivalents). It may also invest in other interest rate futures contracts, including but not limited to, Eurodollar and Federal Funds futures.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.422305	2020-10-11 23:38:31.025871
ITM	VanEck Vectors AMT-Free Intermediate Municipal Index	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the Bloomberg Barclays AMT-Free Intermediate Continuous Municipal Index.\n The fund normally invests at least 80% of its total assets in fixed income securities that comprise the index. The index is comprised of publicly traded municipal bonds that cover the U.S. dollar denominated intermediate term tax-exempt bond market.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.094155	2020-10-11 23:38:31.025871
JCE	Nuveen Core Equity Alpha Fund of Beneficial Interest	Nuveen Core Equity Alpha Fund is a diversified closed end management investment company. Its investment objective is to provide an attractive level of total return, through long-term capital appreciation and through income and gains.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.292727	2020-10-11 23:38:31.025871
GHC	Graham Holdings Company	Graham Holdings Co is an education and media company that provides educational services; television broadcasting; online, print and local TV news; home health and hospice care; and manufacturing. It provides a range of education and related services.	{"Consumer Defensive","Education & Training Services",Education}	{CECO,APEI,LOPE,NAUH,BPI,ATGE}	Education	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:03.530538	2020-10-11 23:38:31.02423
GLD	SPDR Gold Trust	The investment seeks to reflect the performance of the price of gold bullion, less the expenses of the Trust’s operations.\n The Trust holds gold bars and from time to time, issues Baskets in exchange for deposits of gold and distributes gold in connection with redemptions of Baskets. The investment objective of the Trust is for the Shares to reflect the performance of the price of gold bullion, less the Trust’s expenses. The Sponsor believes that, for many investors, the Shares represent a cost-effective investment in gold.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.581413	2020-10-11 23:38:31.02423
GNMA	iShares Barclays GNMA Bond Fund	The investment seeks to track the investment results of the Bloomberg Barclays U.S. GNMA Bond Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The underlying index includes fixed-rate mortgage pass-through securities issued by GNMA that have 30- or 15-year maturities. The index measures the performance of mortgage-backed pass-through securities issued by GNMA.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:03.668668	2020-10-11 23:38:31.02423
GPRO	GoPro Inc.	GoPro Inc is a United States-based company engaged in the development and sale of cameras, mountable and wearable accessories and drones. The company has presence, including in the Americas, Europe, Middle East, Africa, and Asia-Pacific.	{Technology,"Consumer Electronics","Computer Hardware"}	{AVID,CAJ,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:03.756926	2020-10-11 23:38:31.02423
HABT	The Habit Restaurants Inc.	The Habit Restaurants Inc is a casual restaurant concept that specializes in preparing fresh, made-to-order char-grilled burgers and sandwiches featuring USDA choice tri-tip steak, grilled chicken and sushi-grade albacore tuna cooked over an open flame.	{"Consumer Cyclical",Restaurants}	{DENN,ARCO,WING}	Restaurants	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:03.941285	2020-10-11 23:38:31.02423
HBP	Huttig Building Products Inc.	Huttig Building Products Inc along with its subsidiaries is engaged in the distribution of millwork, building materials and wood products used in new residential construction and in home improvement, remodeling and repair work.	{Industrials,"Industrial Distribution"}	{JCTCF,UFPI,BECN,BLDR,POOL,TREX,BCC,DOOR,BXC}	Industrial Distribution	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:03.988695	2020-10-11 23:38:31.02423
HCKT	The Hackett Group Inc.	The Hackett Group Inc is an intellectual property-based strategic consultancy and enterprise benchmarking and practices implementation firm to global companies. It has one operating segment of Business and Technology Consulting Services.	{Industrials,"Business Services"}	{HURN,ICFI,III,MMS,XOM,FCN,IBM,HPQ}	Business Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:04.008949	2020-10-11 23:38:31.02423
HONR	InsightShares Patriotic Employers	The investment seeks to provide investment results that, before fees and expenses, track the price and yield performance of the Military Veterans Index (the "index").\n The fund will normally invest at least 80% of its total assets in securities of the index. The index is designed to measure the performance of a portfolio of large and mid-capitalization U.S. stocks of companies with policies and practices determined to be Military Friendly®, as indicated by VIQTORY's list of Military Friendly® Employers.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.257604	2020-10-11 23:38:31.02423
IBDD	iShares iBonds Mar 2023 Term Corporate	The investment seeks to track the investment results of Bloomberg Barclays 2023 Maturity Corporate Index which composed of U.S. dollar-denominated, investment-grade corporate bonds maturing after March 31, 2022 and before April 1, 2023.\n The fund generally will invest at least 90% of its assets in the component securities (including indirect investments through an underlying fund) of the underlying index, and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.485341	2020-10-11 23:38:31.02423
IBUY	Amplify Online Retail ETF	The investment seeks investment results that generally correspond (before fees and expenses) to the price and yield of the EQM Online Retail Index.\n The fund will invest at least 80% of its total assets in global equity securities that comprise the index, which will primarily include common stocks and/or depositary receipts, such as ADRs and GDRs. The index seeks to measure the performance of global equity securities of publicly traded companies with significant revenue from the online retail business. The index methodology is designed to result in a portfolio that has the potential for capital appreciation. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:04.544308	2020-10-11 23:38:31.02423
IGVT	Xtrackers Barclays International Treasury Bond Hedged	The investment seeks investment results that correspond generally to the performance of the Barclays Global Aggregate Treasury Ex USD Issuer Diversified Bond Index (USD Hedged).\n The fund will normally invest at least 80% of its net assets plus the amount of any borrowings for investment purposes in sovereign bonds and in instruments designed to hedge against its exposure to non-U.S. currencies. The index is designed to track the performance of investment grade sovereign debt publicly issued in the developed and emerging markets and denominated in the issuer's own domestic currency (excluding all securities denominated in U.S. dollars). It is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:04.692281	2020-10-11 23:38:31.02423
IJJ	iShares S&P Mid-Cap 400 Value	The investment seeks to track the investment results of the S&P MidCap 400 Value IndexTM, which measures the performance of the mid-capitalization value sector of the U.S. equity market.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index, but which the advisor believes will help the fund track the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.729457	2020-10-11 23:38:31.02423
GHYB	Goldman Sachs Access High Yield Corporate Bond	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the FTSE Goldman Sachs High Yield Corporate Bond Index (the "index").\n The fund seeks to achieve its investment objective by investing at least 80% of its assets (exclusive of collateral held from securities lending) in securities included in its underlying index. The index is a rules-based index that is designed to measure the performance of high yield corporate bonds denominated in U.S. dollars ("USD") that meet certain liquidity and fundamental screening criteria.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.536747	2020-10-11 23:38:31.023007
GLMD	Galmed Pharmaceuticals Ltd.	Galmed Pharmaceuticals Ltd is a clinical-stage biopharmaceutical company. It is engaged in the development and commercialization of oral therapy for the treatment of liver diseases and cholesterol gallstones.	{Healthcare,Biotechnology}	{EVOK,EYEG,FOMX,FLKS,XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:03.591505	2020-10-11 23:38:31.023007
GPX	GP Strategies Corporation	GP Strategies Corp provides performance improvement solutions training, e-Learning solutions, management consulting and engineering services. The company offers customized training solutions focused on performance improvement initiatives for clients.	{"Consumer Defensive","Education & Training Services",Education}	{MSFT,LNDC,EXPO,HURN,CRAI,ICFI,GE,UPS,CBZ,CBAK}	Education	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:03.759938	2020-10-11 23:38:31.023007
GRES	IQ ARB Global Resources	The investment seeks investment results that correspond generally to the price and yield performance of its underlying index, the IQ Global Resources Index.\n The fund invests at least 80% of its net assets, in the investments included in its underlying index. The underlying index seeks to employ a systematic investment process designed to identify opportunities in markets exhibiting trending or momentum characteristics across commodity asset classes, represented by companies that operate in commodity-specific market segments and whose equity securities trade in developed markets, including the U.S. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.777587	2020-10-11 23:38:31.023007
GSD	WisdomTree Global SmallCap Dividend Fund	The investment seeks to track the price and yield performance of the WisdomTree Global SmallCap Dividend Index.\n The fund will invest at least 80% of its total assets in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is a fundamentally weighted index that consists of small-capitalization companies selected from the WisdomTree Global Dividend Index, which defines the dividend-paying universe of companies in the United States, developed countries, and emerging markets throughout the world. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:03.822216	2020-10-11 23:38:31.023007
HBMD	Howard Bancorp Inc.	Howard Bancorp Inc is a bank holding company. The company, through its wholly-owned subsidiary, provides commercial banking, mortgage banking, and consumer finance through banking branches, the internet and other distribution channels.	{"Financial Services","Banks - Regional - US",Banks}	{GLBZ,SASR,SHBI,FUNC,UVSP,OLBK,MTB}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:03.984078	2020-10-11 23:38:31.023007
HCP	HCP Inc.	HCP Inc is a real estate investment trust. It invests in real estate serving the healthcare industry in the U.S. It acquires, develops, leases, manages and disposes of healthcare real estate and provides financing to healthcare providers.	{"Real Estate","REIT - Healthcare Facilities",REITs}	{VTR,UHT,SBRA,DOC,TIPT,HTA,WELL}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:04.011406	2020-10-11 23:38:31.023007
HYS	PIMCO 0-5 Year High Yield Corporat Bond Index Exchange-Traded Fund	The investment seeks to provide total return that closely corresponds, before fees and expenses, to the total return of the ICE BofAML 0-5 Year US High Yield Constrained Index.\n The fund invests under normal circumstances at least 80% of its total assets (exclusive of collateral held from securities lending) in the component securities of the ICE BofAML 0-5 Year US High Yield Constrained Index. The underlying index is an unmanaged index comprised of U.S. dollar denominated below investment grade corporate debt securities publicly issued in the U.S. domestic market with remaining maturities of less than 5 years.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.43974	2020-10-11 23:38:31.023007
IFV	First Trust Dorsey Wright International Focus 5	The investment seeks investment results that correspond generally to the price and yield (before fees and expenses) of an index called the Dorsey Wright International Focus Five Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in the ETFs that comprise the index. The ETFs in which the fund invests are advised by First Trust Advisors L.P., the fund's investment advisor. The index is designed to provide targeted exposure to the five First Trust country/region-based ETFs that the index provider believes offer the greatest potential to outperform the other ETFs in the selection universe. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:04.660952	2020-10-11 23:38:31.023007
IMTB	iShares Core 5-10 Year USD Bond	The investment seeks to track the investment results of the Bloomberg Barclays U.S. Universal 5-10 Year Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents, including shares of money market funds advised by BFA or its affiliates. The index measures the performance of U.S. dollar-denominated taxable bonds that are rated either investment-grade or high yield with remaining effective maturities between five and ten years. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.778622	2020-10-11 23:38:31.023007
IQDY	FlexShares International Quality Dividend Dynamic Index Fund	The investment seeks investment results that correspond generally to the price and yield performance, before fees and expenses, of the Northern Trust International Quality Dividend Dynamic IndexSM.\n The fund will invest at least 80% of its total assets in the securities of the index and in ADRs and GDRs based on the securities in the index. The index is designed to provide exposure to a high-quality, income-oriented portfolio of international equity securities issued by non-U.S.-based companies, with an emphasis on long-term capital growth and a targeted overall volatility that is greater than that of the Northern Trust International Large Cap IndexSM.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.98255	2020-10-11 23:38:31.023007
GHYG	iShares US & Intl High Yield Corp Bond	The investment seeks to track the investment results of the Markit iBoxx Global Developed Markets High Yield Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents, including shares of money market funds advised by BFA or its affiliates. The index is a rules-based index consisting of high yield corporate bonds denominated in U.S. dollars, euros, British pounds sterling and Canadian dollars.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:03.53955	2020-10-11 23:38:31.024353
GPIC	Gaming Partners International Corporation	Gaming Partners International Corp manufactures & sells casino currencies, playing cards, table accessories, table layouts, dice, gaming furniture, roulette wheels, & RFID readers used with casino table games such as blackjack, baccarat, craps & roulette.	{"Consumer Cyclical",Gambling,"Travel & Leisure"}	{CPHC,SGMS,DDE,ERI}	Travel & Leisure	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:03.732742	2020-10-11 23:38:31.024353
GQRE	FlexShares Global Quality Real Estate Index Fund	The investment seeks investment results that correspond generally to the price and yield performance, before fees and expenses, of the Northern Trust Global Quality Real Estate IndexSM.\n The fund normally will invest at least 80% of its total assets (exclusive of collateral held from securities lending) in the securities of the index and in ADRs and GDRs based on the securities in the index. The index is designed to measure the performance of companies that exhibit certain quality, valuation and momentum characteristics within a universe of publicly-traded equity securities of U.S. and non-U.S. REITs and real estate companies. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.763306	2020-10-11 23:38:31.024353
HBK	Hamilton Bancorp Inc.	Hamilton Bancorp Inc is a bank holding company. The Company through its bank offers deposit accounts, including certificate of deposit accounts, money market accounts, savings accounts, NOW accounts and individual retirement accounts.	{"Financial Services","Banks - Regional - US",Banks}	{DCOM,SVBI,ESSA,NWBI,ORIT,PBHC}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:03.976805	2020-10-11 23:38:31.024353
HCHC	HC2 Holdings Inc.	HC2 Holdings Inc is a diversified holding company which engages in construction, marine services, insurance, telecommunications, energy, life sciences, and other businesses.	{"Communication Services","Telecom Services"}	{ALSK,CCOI,IDT,EGHT,TEUM}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:03.998292	2020-10-11 23:38:31.024353
IBKCP	IBERIABANK Corporation Depositary Shares Representing Series B Fixed to Floating	IBERIABANK Corp provides commercial banking as well as financial services, including retail banking, private banking, and mortgages.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:04.520243	2020-10-11 23:38:31.024353
ICD	Independence Contract Drilling Inc.	Independence Contract Drilling Inc provides land-based contract drilling services for oil and natural gas producers targeting unconventional resource plays in the United States.	{Energy,"Oil & Gas Drilling","Oil & Gas - Drilling"}	{PDS,NBR}	Oil & Gas - Drilling	Energy	New York Stock Exchange	\N	2020-10-11 23:12:04.560753	2020-10-11 23:38:31.024353
ICOW	Pacer Developed Markets International Cash Cows 100	The investment seeks to track the total return performance, before fees and expenses, of the Pacer Developed Markets International Cash Cows 100 Index.\n The fund will invest at least 80% of its total assets (exclusive of collateral held from securities lending) in the component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index uses an objective, rules-based methodology to provide exposure to large and mid-capitalization non-U.S. companies in developed markets with high free cash flow yields. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:04.582869	2020-10-11 23:38:31.024353
IDV	iShares International Select Dividend	The investment seeks to track the investment results of the Dow Jones EPAC Select Dividend Index composed of relatively high dividend paying equities in non-U.S. developed markets.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The underlying index measures the performance of a select group of equity securities issued by companies that have provided relatively high dividend yields on a consistent basis over time.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:04.614368	2020-10-11 23:38:31.024353
IHC	Independence Holding Company	Independence Holding Co is in the business of life and health insurance. The company mainly provides disability and health coverages and related services to commercial and individual customers.	{"Financial Services","Insurance - Life"}	{LNC,MET,PRU,AIZ,GNW,CNO}	Insurance - Life	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.694937	2020-10-11 23:38:31.024353
IMLP	iPath S&P MLP ETN	The investment seeks to replicate, net of expenses, the S&P MLP Index.\n The index is designed to provide exposure to leading partnerships that trade on major U.S. exchanges and are classified in the GICS Energy Sector and GICS Gas Utilities Industry according to the Global Industry Classification Standard® (“GICS”). It includes both master limited partnerships (“MLPs”) and publicly traded limited liability companies which have a similar legal structure to MLPs and share the same tax benefits as MLPs (collectively, the “index constituents”). The index is calculated, maintained and published by S&P Dow Jones Indices LLC.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.760244	2020-10-11 23:38:31.024353
IRBT	iRobot Corporation	iRobot Corp designs and sells small, artificially intelligent consumer robots. The company's most successful products include its automatic Roomba vacuum and the floor-mopping Braava.	{Technology,"Consumer Electronics","Computer Hardware"}	{CSCO,HELE,KTOS,MSN,NC,WHR,SODA,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:04.989692	2020-10-11 23:38:31.024353
JACK	Jack In The Box Inc.	Jack In The Box Inc mainly operates and franchises Jack in the Box quick-service restaurants and Qdoba Mexican Grill fast-casual restaurants.	{"Consumer Cyclical",Restaurants}	{SONC,TAST,MCD,YUM,GTIM,WEN}	Restaurants	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:05.261448	2020-10-11 23:38:31.024353
GRI	Cohen & Steers Global Realty Majors	The investment seeks investment results that correspond generally to the performance, before the fund's fees and expenses, of an index called the Cohen & Steers Global Realty Majors Index.\n The fund will normally invest at least 90% of its total assets in common stocks and other equity securities (which may include ADRs, ADSs and GDRs) that comprise the underlying index. The underlying index consists of the largest and most liquid securities within the global real estate universe that the index provider believes are likely to lead the global securitization of real estate. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.78154	2020-10-11 23:38:31.031328
GRU	AB Svensk Ekportkredit (Swedish Export Credit Corporation) ELEMENTS Linked to the MLCX Grains Index - Total Return	The investment seeks to replicate, net of expenses, the MLCX Grains Total Return Index.\n The index is designed reflect the performance of a fully collateralized investment in the four exchange-traded futures contracts on four physical commodities: corn, soybeans, soy meal and wheat.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.800691	2020-10-11 23:38:31.031328
GVA	Granite Construction Incorporated	Granite Construction Inc is a diversified heavy civil contractors and construction materials producer in the United States. It is engaged in the construction and improvement of streets, roads, highways, mass transit facilities, airport and others.	{Industrials,"Engineering & Construction"}	{STRL,EME,PWR,JEC,VMC,FLR,KBR,ACM,TPC}	Engineering & Construction	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:03.911511	2020-10-11 23:38:31.031328
HBNC	Horizon Bancorp Inc.	Horizon Bancorp (IN) is a holding company providing banking services through its Horizon Bank N.A. It provides retail banking services as well as commercial loans, real estate loans, mortgage warehouse loans, consumer loans and other credit facilities.	{"Financial Services","Banks - Regional - US",Banks}	{THFF,LKFN,GABC,FRME,SRCE,ONB}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:03.983962	2020-10-11 23:38:31.031328
HCCI	Heritage-Crystal Clean Inc.	Heritage-Crystal Clean, Inc provides parts cleaning, used oil re-refining and hazardous and non-hazardous waste services to small and mid-sized customers in both the manufacturing and vehicle service sectors.	{Industrials,"Pollution & Treatment Controls","Industrial Products"}	{ECOL,CLH,WM,CVA,AWX,VTNR,PESI}	Industrial Products	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:03.993494	2020-10-11 23:38:31.031328
HK	Halcon Resources Corporation	Halcon Resources Corp is an independent oil and natural gas company. It is engaged in the acquisition, production, exploration and development of onshore liquids-rich oil and natural gas assets in the United States.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{NFX,MCF,MRO,SN}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:04.170601	2020-10-11 23:38:31.031328
HNDL	Strategy Shares Nasdaq 7HANDL Index ETF	The investment seeks investment results that correlate generally, before fees and expenses, to the price and yield performance of the NASDAQ 7 HANDL™ Index.\n The fund will invest at least 80% of its assets in securities of the NASDAQ 7 HANDL™ Index (the "index"). The index consists of securities issued by exchange-traded funds ("ETFs") and is split into two components, with a 50% allocation to fixed income and equity ETFs (the "Core Portfolio") and a 50% allocation to ETFs of 12 asset categories (the "Explore Portfolio").	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:04.22005	2020-10-11 23:38:31.031328
HOVNP	Hovnanian Enterprises Inc Depositary Share representing 1/1000th of 7.625% Series A Preferred Stock	Hovnanian Enterprises Inc is an American construction company. It designs, constructs, markets and sells single-family detached homes, attached townhomes and condominiums, urban infill and active lifestyle homes in planned residential developments.	{"Consumer Cyclical","Residential Construction","Homebuilding & Construction"}	{}	Homebuilding & Construction	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:04.260565	2020-10-11 23:38:31.031328
HRZN	Horizon Technology Finance Corporation	Horizon Technology Finance Corp is a specialty finance company that lends to and invests in development-stage companies in the technology, life science, healthcare information and services and cleantech industries, referred to as Target Industries.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:04.30379	2020-10-11 23:38:31.031328
HT	Hersha Hospitality Trust Class A of Beneficial Interest	Hersha Hospitality Trust is a self-advised Maryland real estate investment trust. It invests in institutional hotels in urban markets including New York, Washington DC, and South Florida. Its portfolio consists of approximately 46 full service properties.	{"Real Estate","REIT - Hotel & Motel",REITs}	{HST,AHT,DRH,PEB,CHSP,SOHO,HPT,CDOR}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:04.324808	2020-10-11 23:38:31.031328
IAU	ishares Gold Trust	The investment seeks to reflect generally the performance of the price of gold.\n The Trust seeks to reflect such performance before payment of the Trust’s expenses and liabilities. It is not actively managed. The Trust does not engage in any activities designed to obtain a profit from, or to ameliorate losses caused by, changes in the price of gold. The advisor intends to constitute a simple and cost-effective means of making an investment similar to an investment in gold. An investment in physical gold requires expensive and sometimes complicated arrangements in connection with the assay, transportation, warehousing and insurance of the metal.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.47726	2020-10-11 23:38:31.031328
IDOG	ALPS International Sector Dividend Dogs	The investment seeks investment results that replicate as closely as possible, before fees and expenses, the performance of the S-Network® International Sector Dividend Dogs Index.\n The fund seeks investment results that replicate as closely as possible, before fees and expenses, the performance of the underlying index. The underlying index is a rules-based index intended to give investors a means of tracking the overall performance of the highest dividend paying stocks (i.e. "Dividend Dogs") in the S-Network Developed International 1000 Index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.606139	2020-10-11 23:38:31.031328
IJS	iShares S&P SmallCap 600 Value	The investment seeks to track the investment results of the S&P SmallCap 600 Value IndexTM, which measures the performance of the small-capitalization value sector of the U.S. equity market.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index, but which the advisor believes will help the fund track the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.736773	2020-10-11 23:38:31.031328
HAWX	iShares Currency Hedged MSCI ACWI ex U.S.	The investment seeks to track the investment results of the MSCI ACWI ex USA 100% Hedged to USD Index, an equity benchmark for developed and emerging markets equity stock market performance, excluding the U.S., with the currency risk inherent in the securities included in the index hedged to the U.S. dollar on a monthly basis.\n The fund generally will invest at least 90% of its assets in the component securities (including indirect investments through the underlying fund) and other instruments of the index and in investments that have economic characteristics that are substantially identical to the component securities of the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.96534	2020-10-11 23:38:31.028166
HCA	HCA Healthcare Inc.	HCA Healthcare Inc is a health care services company. It operates general acute care hospitals, psychiatric hospitals, and rehabilitation hospitals. The company also operates freestanding surgery centers.	{Healthcare,"Medical Care","Health Care Providers"}	{LPNT,UHS,CYH,THC,XLV}	Health Care Providers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:03.989175	2020-10-11 23:38:31.028166
HCI	HCI Group Inc.	HCI Group Inc is a Florida-based company property and casualty insurance company. Its operations are divided into four divisions property and casualty insurance, reinsurance, real estate, and information technology.	{"Financial Services","Insurance - Property & Casualty"}	{ERIE,NSEC,STFC,DGICA,KINS,HTH,UVE,UIHC,HRTG}	Insurance - Property & Casualty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.004892	2020-10-11 23:38:31.028166
HECO	Strategy Shs EcoLogical	The investment seeks capital appreciation.\n The fund is an actively managed exchange-traded fund ("ETF") and, under normal conditions, will invest at least 80% of its net assets (plus borrowings for investment purposes), directly or indirectly through mutual funds and ETFs, in the equity and fixed income securities of ecologically-focused companies and/or green bonds. It may also invest up to 20% of its net assets in cash and cash equivalents including U.S. government securities. The fund may invest in domestic and foreign securities of companies of any market capitalization.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.039884	2020-10-11 23:38:31.028166
HTBI	HomeTrust Bancshares Inc.	HomeTrust Bancshares Inc is a bank holding company. It is engaged in attracting deposits from public & investing funds, along with borrowed funds, in loans secured by first & second mortgages on one-to four-family residences.	{"Financial Services","Banks - Regional - US",Banks}	{OFED}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:04.327082	2020-10-11 23:38:31.028166
HUSV	First Trust Exchange-Traded Fund III Horizon Managed Volatility Domestic	The investment seeks to provide capital appreciation.\n The fund seeks to achieve its investment objective by investing at least 80% of its net assets in common stocks of domestic companies listed and traded on U.S. national securities exchanges that the sub-advisor believes exhibit low future expected volatility. To implement this strategy, the sub-advisor employs volatility forecasting models to forecast future expected volatility. The strategy is largely quantitative and rules-based, but also includes multiple parameters over which the sub-advisor may exercise discretion in connection with its active management of the fund. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.38194	2020-10-11 23:38:31.028166
HYGS	Hydrogenics Corporation	Hydrogenics Corp along with its subsidiaries designs, develops and manufactures hydrogen generation and fuel cell products based on water electrolysis technology and proton exchange membrane technology.	{Industrials,"Diversified Industrials","Industrial Products"}	{ENB,BLDP,PLUG,CLIR,AMSC}	Industrial Products	Industrials	NASDAQ Global Market	\N	2020-10-11 23:12:04.423073	2020-10-11 23:38:31.028166
HYND	WisdomTree BofA Merrill Lynch High Yield Bond Nega	The investment seeks to track the price and yield performance, before fees and expenses, of the BofA Merrill Lynch 0-5 Year U.S. High Yield Constrained, Negative Seven Duration Index.\n The index is designed to provide long exposure to the BofA Merrill Lynch 0-5 Year US High Yield Constrained Index while seeking to manage interest rate risk through the use of short positions in U.S. Treasury securities. The fund normally invests at least 80% of its total assets in the component securities of the index and investments that have economic characteristics that are substantially identical to such component securities. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:04.434243	2020-10-11 23:38:31.028166
ICE	Intercontinental Exchange Inc.	Intercontinental Exchange Inc is an operator of regulated exchanges, clearing houses and listings venues, and a provider of data services for commodity and financial markets. It serves the risk management needs of global markets.	{"Financial Services","Financial Exchanges","Brokers & Exchanges"}	{NDAQ,CME,CBOE}	Brokers & Exchanges	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.560948	2020-10-11 23:38:31.028166
IMUC	ImmunoCellular Therapeutics Ltd.	ImmunoCellular Therapeutics Ltd is engaged in developing immune-based therapies for the treatment of cancers which can elicit an immune response against several antigens.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	OTC Pink	\N	2020-10-11 23:12:04.782386	2020-10-11 23:38:31.028166
INF	Brookfield Global Listed Infrastructure Income Fund Closed End Fund	BROOKFIELD GLOBAL LISTED INFRASTRUCTURE is a closed end management investment company. Its investment objective is to provide a high level of total return, with an emphasis on income.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.808366	2020-10-11 23:38:31.028166
INT	World Fuel Services Corporation	World Fuel Services Corp is an energy management company. The company offers fuel and related products and services to commercial airlines, second and third tier airlines, regional and low cost carriers. Its segments are aviation, marine and land.	{Energy,"Oil & Gas Refining & Marketing","Oil & Gas - Refining & Marketing"}	{GLP,SPH,AE,MPC,SRLP,CEQP}	Oil & Gas - Refining & Marketing	Energy	New York Stock Exchange	\N	2020-10-11 23:12:04.874223	2020-10-11 23:38:31.028166
ONEV	SPDR Russell 1000 Low Volatility Focus	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the Russell 1000 Low Volatility Focused Factor Index.\n Under normal market conditions, the fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index is designed to reflect the performance of a segment of large-capitalization U.S. equity securities demonstrating a combination of core factors (high value, high quality, and low size characteristics), with a focus factor comprising low volatility characteristics. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.468682	2020-10-11 23:38:31.02423
IBDC	iShares iBonds Mar 2020 Term Corporate	The investment seeks to track the investment results of the Bloomberg Barclays 2020 Maturity Corporate Index composed of U.S. dollar-denominated, investment-grade corporate bonds maturing after March 31, 2019 and before April 1, 2020.\n The fund generally will invest at least 90% of its assets in the component securities (including indirect investments through an underlying fund) of the index, except during the last months of the fund's operations. The index includes U.S. dollar-denominated, investment-grade securities publicly issued by U.S. and non-U.S. corporate issuers that have $300 million or more of outstanding face value at the time of inclusion.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.486232	2020-10-11 23:38:31.023428
IDT	IDT Corporation Class B	IDT Corp provides telecommunications services to consumers and businesses, wholesale carrier services and local, long distance and wireless services.	{"Communication Services","Telecom Services"}	{MSFT,FISV,CNSL,TEF,VZ,T,VG,ORAN,EGHT}	Communication Services	Communication Services	New York Stock Exchange	\N	2020-10-11 23:12:04.611449	2020-10-11 23:38:31.023428
IJR	iShares Core S&P Small-Cap	The investment seeks to track the investment results of the S&P SmallCap 600 (the "underlying index"), which measures the performance of the small-capitalization sector of the U.S. equity market.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index, but which the advisor believes will help the fund track the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.736233	2020-10-11 23:38:31.023428
IRWD	Ironwood Pharmaceuticals Inc.	Ironwood Pharmaceuticals Inc is a biotechnology company which is focused on advancing innovative product opportunities in areas of large unmet needs such as constipation, gastrointestinal disorders, hyperuricemia, among others.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{PGNX,NKTR,CPIX,AZN,SGYP,XLV}	Drug Manufacturers	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:05.028509	2020-10-11 23:38:31.023428
IXP	iShares Global Comm Services	The investment seeks to track the investment results of the S&P Global 1200 Communication Services Sector IndexTM.\n The fund invests at least 90% of its assets in securities of the index and in depositary receipts representing securities of the index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the index. The index measures the performance of companies that the index provider deems to be a part of the communication services sector of the economy and that the index provider believes are important to global markets. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.18369	2020-10-11 23:38:31.023428
IYW	iShares U.S. Technology	The investment seeks to track the investment results of the Dow Jones U.S. Technology Index.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index measures the performance of the technology sector of the U.S. equity market and may include large-, mid- or small-capitalization companies. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.244984	2020-10-11 23:38:31.023428
JAZZ	Jazz Pharmaceuticals plc	Jazz Pharmaceuticals PLC is a biopharmaceutical company focused on improving patients' lives by identifying, developing and commercializing differentiated products that address unmet medical needs.	{Healthcare,Biotechnology}	{NVS,ABC,TEVA,ENDP,PRTK,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:05.269058	2020-10-11 23:38:31.023428
KBWB	Invesco KBW Bank ETF	The investment seeks to track the investment results (before fees and expenses) of the KBW Nasdaq Bank Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in securities of national money centers, regional banks and thrift institutions that are listed on a U.S. national securities exchange and that comprise the underlying index. The underlying index is a modified-market capitalization-weighted index that seeks to reflect the performance of national money centers, regional banks and thrift institutions that are publicly traded in the U.S. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:05.692494	2020-10-11 23:38:31.023428
KEN	Kenon Holdings Ltd.	Kenon Holdings Ltd, through its subsidiaries, provides electricity generation using technologies such as hydroelectric, natural gas and diesel turbines and heavy fuel oil engines, in Peru, Chile, Colombia, El Salvador, Jamaica, Nicaragua, among others.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{BIP,ED,NEE,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:05.71986	2020-10-11 23:38:31.023428
KIE	SPDR S&P Insurance	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of an index that tracks the performance of publicly traded companies in the insurance industry.\n In seeking to track the performance of the S&P Insurance Select Industry Index (the "index"), the fund employs a sampling strategy. It generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index represents the insurance segment of the S&P Total Market Index ("S&P TMI"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.767634	2020-10-11 23:38:31.023428
LACQ	Leisure Acquisition Corp.	Leisure Acquisition Corp is a blank check company formed for the purpose of effecting a merger, capital stock exchange, asset acquisition, stock purchase, reorganization or similar business combination with one or more businesses.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:05.969129	2020-10-11 23:38:31.023428
LBJ	Direxion Daily Latin America 3x Bull Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the daily performance of the S&P Latin America 40 Index.\n The fund, under normal circumstances, invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, exchange-traded funds ("ETFs") that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index is a float-adjusted market capitalization weighted equity index of issuers drawn from five major Latin American markets: Brazil, Chile, Columbia, Mexico, and Perú. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.016823	2020-10-11 23:38:31.023428
IFGL	iShares FTSE EPRA/NAREIT Global Real Estate ex-U.S. Index Fund	The investment seeks to track the investment results of the FTSE EPRA Nareit Developed ex-U.S. Index composed of real estate equities in developed non-U.S. markets.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The underlying index measures the stock performance of companies engaged in the ownership and development of real estate markets in developed countries (except for the United States) as defined by FTSE EPRA/NAREIT.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:04.651953	2020-10-11 23:38:31.02582
IQDE	FlexShares International Quality Dividend Defensive Index Fund	The investment seeks investment results that correspond generally to the price and yield performance, before fees and expenses, of the Northern Trust International Quality Dividend Defensive IndexSM.\n The fund will invest at least 80% of its total assets in the securities of the index and in ADRs and GDRs based on the securities in the index. The index is designed to provide exposure to a high-quality, income-oriented portfolio of international equity securities issued by non-U.S.-based companies, with an emphasis on long-term capital growth and a targeted overall volatility that is lower than that of the Northern Trust International Large Cap IndexSM.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.965302	2020-10-11 23:38:31.02582
IR	Ingersoll-Rand plc (Ireland)	Ingersoll-Rand PLC is a diversified company. It provides products, services and solutions to enhance the quality and comfort of air in homes and buildings, transport and protect food and perishables, and increase industrial productivity and efficiency.	{Industrials,"Diversified Industrials","Industrial Products"}	{AAON,FLS,LII,DOV,PH,SPXC,CUO}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:04.99056	2020-10-11 23:38:31.02582
IYE	iShares U.S. Energy	The investment seeks to track the investment results of the Dow Jones U.S. Oil & Gas Index composed of U.S. equities in the energy sector.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index measures the performance of the oil and gas sector of the U.S. equity market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.228215	2020-10-11 23:38:31.02582
JASNW	Jason Industries (Warrant)	Jason Industries, Inc. engages in the provision of components and manufacturing solutions for industrial equipment, motorcycles, lawn and turf care, rail, and automotive. It operates through the following segments: Finishing, Components, Seating, and Acoustics. The Finishing segment comprises production of industrial brushes, buffing wheels, buffing compounds, and abrasives that are used in a broad range of industrial and infrastructure applications. The Components segment manufactures expanded and perforated metal components, slip-resistant walking surfaces, and subassemblies for smart utility meters. The Seating segment supplies seating solutions to equipment manufacturers in the motorcycle, lawn and turf care, industrial, agricultural, construction, and power sports end markets. The Acoustics segment includes manufacture of engineered non-woven fiber-based acoustical products for the automotive industry. The company was founded in May 2013 and is headquartered in Milwaukee, WI.	{}	{}	Industrial Machinery	Producer Manufacturing	NasdaqCM	\N	2020-10-11 23:12:05.265096	2020-10-11 23:38:31.02582
JKI	iShares Morningstar Mid-Cap Value ETF	The investment seeks to track the investment results of the Morningstar® Mid Value IndexSM composed of mid-capitalization U.S. equities that exhibit value characteristics.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index measures the performance of stocks issued by mid-capitalization companies that have exhibited "value" characteristics as determined by Morningstar, Inc.'s ("Morningstar" or the "index provider") proprietary index methodology.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:05.432454	2020-10-11 23:38:31.02582
JSD	Nuveen Short Duration Credit Opportunities Fund of Beneficial Interest	Under normal circumstances, the Fund will invest at least 80% of “Assets,” at time of purchase, in loans or securities in the issuing company’s capital structure that are senior to its common equity, including but not limited to debt securities, preferred securities.\n\nPlease note that this new policy is separate from and in addition to the fund’s existing policy that it will invest at least 70% of its Managed Assets in adjustable rate corporate debt instruments, including senior secured loans, second lien loans and other adjustable rate corporate debt instruments.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.611484	2020-10-11 23:38:31.02582
KEX	Kirby Corporation	Kirby Corp is a domestic tank barge operator, transporting bulk liquid products throughout the Mississippi River System, on the Gulf Intracoastal Waterway, coastwise along all three United States coasts and in Alaska and Hawaii.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{MMLP,CKH,CAT,EPD}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:05.736926	2020-10-11 23:38:31.02582
KGJI	Kingold Jewelry Inc.	Kingold Jewelry Inc is a professional designers and manufacturers of 24-karat gold jewelry and Chinese ornaments. It develops, promotes and sells products including gold necklaces, rings, earrings, bracelets, and pendants to the jewelry market in China.	{"Consumer Cyclical","Luxury Goods","Retail - Apparel & Specialty"}	{CTHR,FOSL}	Retail - Apparel & Specialty	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:05.759557	2020-10-11 23:38:31.02582
LAD	Lithia Motors Inc.	Lithia Motors Inc acts as a retailer of new and used vehicles and related services. The company also provides vehicle maintenance, warranty, paint, and repair services. It also helps its customers to arrange for financing new vehicles.	{"Consumer Cyclical","Auto & Truck Dealerships",Autos}	{RUSHA,AN,GPI,SAH,GPC,ABG,KMX,PAG}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:05.974885	2020-10-11 23:38:31.02582
LBDC	2xLeveraged Long ETRACS Wells Fargo Business Development Company Index Series B due May 24 2041	The investment seeks to provide a monthly compounded two times leveraged long exposure to the performance of the Wells Fargo® Business Development Company Index.\n The index is intended to measure the performance of all business development companies listed on the NYSE MKT or the NASDAQ that satisfy specified market capitalization and other eligibility requirements. The index refers to the business development companies included in the Wells Fargo® Business Development Company Index as the “index constituents.”	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.012825	2020-10-11 23:38:31.02582
IIN	IntriCon Corporation	IntriCon Corp, along with its subsidiaries, is engaged in designing, developing, engineering and manufacturing body-worn devices for medical bio-telemetry devices, value hearing health devices and professional audio communication devices.	{Technology,"Electronic Components","Computer Hardware"}	{KOSS,VICR,ON,VOXX,CUI,QBAK,XLK}	Computer Hardware	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:04.724903	2020-10-11 23:38:31.028166
INVH	Invitation Homes Inc.	Invitation Homes Inc is an owner and operator of single-family homes for lease in the United States. It buys and sells vacant, leased or occupied homes.	{"Real Estate","REIT - Residential",REITs}	{RDIB,INVH}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:04.899944	2020-10-11 23:38:31.028166
IQDG	WisdomTree International Quality Dividend Growth Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree International Quality Dividend Growth Index.\n Under normal circumstances, at least 80% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index consists of dividend-paying common stocks with growth characteristics of companies in the industrialized world, excluding Canada and the United States. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:04.968099	2020-10-11 23:38:31.028166
JSYN	Jensyn Acquistion Corp.	Jensyn Acquisition Corp is a blank check company. It was formed for the purpose of entering into a merger, share exchange, asset acquisition, stock purchase, reorganization or other similar business combination with one or more target business.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:05.623405	2020-10-11 23:38:31.028166
KEYW	The KEYW Holding Corporation	KEYW Holding Corp is a provider of engineering services, cybersecurity and analytic products, and integrated platforms that support the entire intelligence process, including collection, processing, analysis and impact.	{Technology,"Software - Application","Application Software"}	{VSEC,LMT,GD,NOC,RTN,SAIC,EGL,LLL,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:05.736683	2020-10-11 23:38:31.028166
KOLD	ProShares UltraShort Bloomberg Natural Gas	The investment seeks results for a single day that match (before fees and expenses) two times the inverse of the daily performance of the Bloomberg Natural Gas SubindexSM.\nThe fund seeks to meet their investment objectives by investing in Natural Gas futures contracts. It may also invest in swaps if the market for a specific futures contract experiences emergencies (e.g., natural disaster, terrorist attack, or an act of God) or disruptions (e.g., a trading halt or a flash crash) that would prevent or make it impractical for such fund to obtain the appropriate amount of investment exposure to the affected Natural Gas futures contracts directly.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.863964	2020-10-11 23:38:31.028166
KTOS	Kratos Defense & Security Solutions Inc.	Kratos Defense & Security Solutions Inc provides advanced engineering, security and services solutions to the US Defense sector. It focuses on unmanned systems, satellite communications, cyber security and warfare, and missile defense and combat systems.	{Industrials,"Security & Protection Services","Consulting & Outsourcing"}	{FLIR,VSEC,LMT,BA,GD,NOC,UTX,RTN,SAIC,LLL}	Consulting & Outsourcing	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:05.931366	2020-10-11 23:38:31.028166
LABU	Direxion Daily S&P Biotech Bull 3X Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the daily performance of the S&P Biotechnology Select Industry Index ("index").\n The fund invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, exchange-traded funds ("ETFs") that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index is designed to measure the performance of the biotechnology sub-industry based on the Global Industry Classification Standards ("GICS"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.967648	2020-10-11 23:38:31.028166
LITB	LightInTheBox Holding Co. Ltd. American Depositary Shares each representing 2	LightInTheBox Holding Co Ltd is an online retail company delivering products directly to consumers around the world. It provides apparel products including customized, special occasion, and fast fashion apparel products.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{AMZN,XOXO,VIPS,LEJU,JD,JMEI}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:06.159288	2020-10-11 23:38:31.028166
LLQD	iShares 10 Year Investment Grade Corporate Bond	The investment seeks to track the investment results of the Markit iBoxx® USD Liquid Investment Grade Long Index (the "underlying index").\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents, including shares of money market funds. The index is designed to reflect the performance of U.S. dollar-denominated investment-grade corporate debt with remaining maturities greater than ten years. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:06.186198	2020-10-11 23:38:31.028166
MDCO	The Medicines Company	The Medicines Co is a pharmaceutical company. It provides transformational solutions to address the most pressing healthcare needs facing patients, physicians and providers. The company's products are Angiomax, Ionsys, Minocin for injection and Orbactiv.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{MNTA,AZN,BMY,JNJ,LLY,PFE,TEVA,EGRX,VCEL,INVA,XLV}	Drug Manufacturers	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:06.62848	2020-10-11 23:38:31.028166
MIDZ	Direxion Mid Cap Bear 3X Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the inverse (or opposite) of the daily performance of the S&P MidCap® 400 Index.\n The fund, under normal circumstances, invests in swap agreements, futures contracts, short positions or other financial instruments that, in combination, provide inverse (opposite) or short leveraged exposure to the index equal to at least 80% of the fund's net assets (plus borrowing for investment purposes). The index measures the performance of 400 mid-sized companies in the United States. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.858489	2020-10-11 23:38:31.028166
VST	Vistra Energy Corp.	Vistra Energy Corp through its subsidiaries is in electricity market activities including power generation, wholesale energy sales and purchases, commodity risk management and retail sales of electricity to end users.	{Utilities,"Utilities - Independent Power Producers"}	{NEP,XLU}	Utilities - Independent Power Producers	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:13.187476	2020-10-11 23:38:31.025871
INDB	Independent Bank Corp.	Independent Bank Corp is a provider of financial services. The company functions through the Rockland Trust Company to render commercial banking solutions including lending activities, acceptance of demand, savings and investment management.	{"Financial Services","Banks - Regional - US",Banks}	{EBTC,UNB,BPFH,CNBKA,WASH,BRKL,WBS,BHB,NBN}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:04.801138	2020-10-11 23:38:31.02423
INKM	SPDR SSgA Income Allocation	The investment seeks to provide total return by focusing on investments in income and yield-generating assets.\n The Adviser primarily invests the assets of the fund among ETPs that provide exposure to five primary asset classes: (i) domestic and international equity securities; (ii) domestic and international investment-grade and high yield debt securities; (iii) hybrid equity/debt securities; (iv) first lien senior secured floating rate bank loans, commonly referred to as "Senior Loans"; and (v) REITs, including equity REITs and mortgage REITs.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.838451	2020-10-11 23:38:31.02423
IP	International Paper Company	International Paper Co produces and distributes printing paper, packaging, forest products, and chemical products. The company operates in three segments namely; Industrial Packaging, Printing Papers, and Global Cellulose Fibers.	{"Consumer Cyclical","Packaging & Containers"}	{UFS,PKG,CLW,FBR,WRK}	Packaging & Containers	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:04.926855	2020-10-11 23:38:31.02423
IPO	Renaissance IPO	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the Renaissance IPO Index (the "index").\n The fund seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the index. It normally invests at least 80% of its total assets in securities that comprise the index. The index is a portfolio of companies that have recently completed an initial public offering ("IPO") and are listed on a U.S. exchange. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.960473	2020-10-11 23:38:31.02423
IQLT	iShares Edge MSCI Intl Quality Factor	The investment seeks to track the investment results of the MSCI World ex USA Sector Neutral Quality Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index measures the performance of international developed large- and mid-capitalization stocks exhibiting relatively higher quality characteristics as identified through three fundamental variables: return on equity, earnings variability and debt-to-equity.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.982707	2020-10-11 23:38:31.02423
IROQ	IF Bancorp Inc.	IF Bancorp Inc is a savings and loan holding company. The Company is engaged in the business of directing, planning, and coordinating the business activities of its subsidiary, Iroquois Federal Savings and Loan Association.	{"Financial Services","Savings & Cooperative Banks",Banks}	{BFIN,FSFG,WAYN}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:05.021723	2020-10-11 23:38:31.02423
IWS	iShares Russell Mid-cap Value	The investment seeks to track the investment results of the Russell Midcap Value Index, which measures the performance of the mid-capitalization value sector of the U.S. equity market.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index, but which the advisor believes will help the fund track the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.168346	2020-10-11 23:38:31.02423
IXUS	iShares Core MSCI Total International Stock ETF	The investment seeks to track the investment results of the MSCI ACWI ex USA IMI composed of large-, mid- and small-capitalization non-U.S. equities.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is a free float-adjusted market capitalization index designed to measure the combined equity market performance of developed and emerging markets countries, excluding the United States.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:05.184909	2020-10-11 23:38:31.02423
IYM	iShares U.S. Basic Materials	The investment seeks to track the investment results of the Dow Jones U.S. Basic Materials Index composed of U.S. equities in the basic materials sector.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index measures the performance of the basic materials sector of the U.S. equity market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.239966	2020-10-11 23:38:31.02423
JAGX	Jaguar Health Inc.	Jaguar Health Inc is an animal health company focused on developing and commercializing first-in-class gastrointestinal products for companion and production animals, foals, and high value horses.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:05.261828	2020-10-11 23:38:31.02423
JPNL	Direxion Daily Japan Bull 3X	The investment seeks daily investment results, before fees and expenses, of 300% of the daily performance of the MSCI Japan Index.\n The fund invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, exchange-traded funds ("ETFs") that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index is designed to measure the performance of the large- and mid-cap segments of the Japanese equity market, covering approximately 85% of the free float-adjusted market capitalization of Japanese issuers. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.584566	2020-10-11 23:38:31.02423
K	Kellogg Company	Kellogg Co is engaged in the manufacturing and marketing of ready-to-eat cereal and convenience foods. Its products include cereal, cookies, crackers, and other packaged foods.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{HAIN,GIS,TSN,FLO,THS,POST,MDLZ,PEP}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:05.651473	2020-10-11 23:38:31.02423
OUSM	O'Shares FTSE Russell Small Cap Quality Dividend	The investment seeks to track the performance (before fees and expenses) of the FTSE USA Small Cap ex Real Estate 2Qual/Vol/Yield 3% Capped Factor Index.\n Under normal market conditions, the fund will invest at least 80% of its total assets in the components of the index. The index is constructed using a proprietary, rules-based methodology designed to select equity securities from the FTSE USA Small Cap Index that have exposure to the following three factors: 1) quality, 2) low volatility and 3) yield.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.621403	2020-10-11 23:38:31.02423
INFN	Infinera Corporation	Infinera Corp is an optical transport networking equipment provider. The company serves telecommunications service providers, Internet content providers, cable providers, research and education institutions, and government entities.	{Technology,"Communication Equipment"}	{CSCO,ADTN,EXTR,JNPR,CIEN,RBBN,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:04.817815	2020-10-11 23:38:31.031328
IPAC	iShares Core MSCI Pacific	The investment seeks to track the investment results of the MSCI Pacific IMI (the "underlying index").\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is a free float-adjusted market capitalization-weighted index which consists of securities from the following five countries or regions: Australia, Hong Kong, Japan, New Zealand and Singapore.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.92691	2020-10-11 23:38:31.031328
IPKW	PowerShares International BuyBack Achievers Portfo	The investment seeks to track the investment results (before fees and expenses) of the NASDAQ International BuyBack AchieversTM Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in common stocks that comprise the underlying index. Strictly in accordance with its guidelines and mandated procedures, the Nasdaq, Inc. includes in the underlying index common stocks of foreign companies that are classified as "International BuyBack AchieversTM" pursuant to a proprietary selection methodology. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:04.960482	2020-10-11 23:38:31.031328
IQDF	FlexShares International Quality Dividend Index Fund	The investment seeks investment results that correspond generally to the price and yield performance, before fees and expenses, of the Northern Trust International Quality Dividend IndexSM.\n The fund will invest at least 80% of its total assets in the securities of the index and in ADRs and GDRs based on the securities in the index. The index is designed to provide exposure to a high-quality, income-oriented portfolio of international equity securities issued by non-U.S.-based companies, with an emphasis on long-term capital growth and a targeted overall volatility similar to that of the Northern Trust International Large Cap IndexSM.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.966669	2020-10-11 23:38:31.031328
IRET	Investors Real Estate Trust Shares of Beneficial Interest	Investors Real Estate Trust is a real estate investment trust that owns and operates income producing real estate properties. Its multifamily leases are for one-year term and commercial properties are leased to tenants under long-term lease arrangements.	{"Real Estate","REIT - Residential",REITs}	{DRE,ESS,IRT,VER,SNH}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:05.002165	2020-10-11 23:38:31.031328
IXJ	iShares Global Healthcare	The investment seeks to track the S&P Global 1200 Health Care IndexTM.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index. The index measures the performance of companies that the index provider deems to be a part of the healthcare sector of the economy and that the index provider believes are important to global markets.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.183445	2020-10-11 23:38:31.031328
JAG	Jagged Peak Energy Inc.	Jagged Peak Energy Inc is an oil & gas entity in the United States. Its business involves the acquisition of assets to carry out oil & gas exploration activities.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:05.261578	2020-10-11 23:38:31.031328
JHMF	John Hancock Multifactor Financials	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the John Hancock Dimensional Financials Index.\n The fund normally invests at least 80% of its net assets (plus any borrowings for investment purposes) in securities that compose the fund's benchmark index. The index is designed to comprise securities in the financial sector within the U.S. Universe whose market capitalizations are larger than that of the 1001st largest U.S. company at the time of reconstitution.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.358068	2020-10-11 23:38:31.031328
JPUS	JPMorgan Diversified Return U.S. Equity	The investment seeks investment results that closely correspond, to the performance of the JP Morgan Diversified Factor Europe Currency Hedged (USD) Index.\n The fund principally invests in the JPMorgan Diversified Return Europe Equity ETF and forward foreign currency contracts. The underlying index has been developed to represent European stock performance, with the foreign currency risk of the securities included in the underlying index hedged against the U.S. dollar on a monthly basis. The underlying index is comprised of equity securities across developed Europe selected to represent a diversified set of factor characteristics: value, momentum, and quality.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.593557	2020-10-11 23:38:31.031328
KFYP	KraneShares Zacks New China	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the price and yield performance of CSI CICC Select 100 Index.\n Under normal circumstances, the fund will invest at least 80% of its total assets in components of the underlying index and depositary receipts, including ADRs, representing such components. The underlying index includes the China A-Shares of 100 Chinese companies with high historical returns on equity, dividend yields and growth in net profits that have listed China A-Shares, as determined by the index provider. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.756433	2020-10-11 23:38:31.031328
KIO	KKR Income Opportunities Fund	KKR Income Opportunities Fund is an investment management company investing in the portfolio of loans and fixed-income instruments of U.S. and non-U.S. issuers. It seeks to generate a high level of current income, along with capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.784729	2020-10-11 23:38:31.031328
KMF	Kayne Anderson Midstream Energy Fund Inc	Kayne Anderson Midstream/Energy Fund Inc is a non-diversified closed-end fund. The company's investment objective is to provide a high level of total return with an emphasis on making quarterly cash distributions to stockholders.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.808501	2020-10-11 23:38:31.031328
LACQU	Leisure Acquisition Corp. Unit	Leisure Acquisition Corp is a blank check company formed for the purpose of effecting a merger, capital stock exchange, asset acquisition, stock purchase, reorganization or similar business combination with one or more businesses.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:05.97095	2020-10-11 23:38:31.031328
IRR	Voya Natural Resources Equity Income Fund of Beneficial Interest	Under normal market conditions, the Fund will seek to achieve its investment objective by investing at least 80% of its managed assets in the equity securities of, or derivatives linked to the equity securities of, companies that are primarily engaged in owning or developing energy, other natural resources and basic materials, or supplying goods and services to such companies. The Fund will consider a company to be principally engaged in owning or developing energy, other natural resource and basic materials, or supplying goods and services to such companies if: (i) at least 50% of the companys assets, sales, or profits are committed to or derived from any of the energy, other natural resources or basic materials industries; or (ii) a third party has given the company an industry or sector classification in the energy, other natural resources or basic materials sector. Equity securities held by the Fund may include common stocks, preferred shares, convertible securities, warrants and depository receipts. The Fund may also invest in Exchange Traded Funds composed primarily of Natural Resources Companies. Additionally the Fund will employ an integrated options collar strategy to seek to partially reduce the exposure of the Fund to declines in the value of the energy and natural resources securities in its portfolio.	{"Financial Services","Asset Management"}	{}			New York Stock Exchange	\N	2020-10-11 23:12:05.023363	2020-10-11 23:38:31.028166
IVFVC	Ivy Focused Value NextShares	The investment seeks to provide capital appreciation with a secondary objective of providing current income.\nThe fund seeks to achieve its objectives by investing primarily in the common stocks of companies that IICO believes offer a high yield and that possess quality, valuation and growth potential characteristics. It seeks to invest in companies that meet all three of these criteria. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:05.129102	2020-10-11 23:38:31.028166
JBT	John Bean Technologies Corporation	John Bean Technologies Corp is a technology solutions provider of food & beverage industry with focus on proteins, liquid foods and automated system solutions. It designs, produces & services products & systems for multi-national & regional customers.	{Industrials,"Diversified Industrials","Industrial Products"}	{AIR,ITW,NOC,SXI}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:05.287737	2020-10-11 23:38:31.028166
JHMM	John Hancock Multifactor Mid Cap	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the John Hancock Dimensional Mid Cap Index.\n The fund normally invests at least 80% of its net assets (plus any borrowings for investment purposes) in securities that compose the fund's benchmark index. The index is designed to comprise a subset of securities in the U.S. Universe issued by companies whose market capitalizations are between the 200th and 951st largest U.S. company at the time of reconstitution.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.367806	2020-10-11 23:38:31.028166
JQUA	JPMorgan U.S. Quality Factor	The investment seeks investment results that closely correspond, before fees and expenses, to the performance of the JP Morgan US Quality Factor Index.\n The fund will invest at least 80% of its assets in securities included in the underlying index. "Assets" means net assets, plus the amount of borrowing for investment purposes. The underlying index is comprised of U.S. equity securities selected to represent quality factor characteristics.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.595962	2020-10-11 23:38:31.028166
KIDS	OrthoPediatrics Corp.	OrthoPediatrics Corp is a medical device company. It provides a comprehensive product offering to the pediatric orthopedic market in order to improve the lives of children with orthopedic conditions.	{Healthcare,"Medical Devices"}	{XLV}	Medical Devices	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:05.765354	2020-10-11 23:38:31.028166
LADR	Ladder Capital Corp Class A	Ladder Capital Corp is an internally-managed real estate investment trust. The company invests in loans, securities and other interests in U.S. commercial real estate, with a focus on senior secured assets.	{"Real Estate","REIT - Diversified",REITs}	{ATAX,EFC,WD,HASI}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:05.977439	2020-10-11 23:38:31.028166
LBTYB	Liberty Global plc Class B Ordinary Shares	Liberty Global PLC is a holding company engaged in making investments into international television and broadband companies. Through its subsidiaries, it offers telecommunication, broadband services and cable network.	{"Communication Services","Pay TV"}	{RCI,SJR,DISH,CHTR}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:12:06.028604	2020-10-11 23:38:31.028166
LEDS	SemiLEDS Corporation	SemiLEDs Corp develop, manufacture and sell light emitting diode chips and LED components. The company's products include LED Chips, LED Components, and Lighting Products.	{Technology,"Semiconductor Memory",Semiconductors}	{CREE,AOSL,TSM,PHG,XLK}	Semiconductors	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:06.077771	2020-10-11 23:38:31.028166
LSBK	Lake Shore Bancorp Inc.	Lake Shore Bancorp Inc operates as a savings and loan holdings company for Lake Shore Savings Bank. It attracts deposits and invests those deposits with funds in residential and real estate loans and home equity lines of credit.	{"Financial Services","Savings & Cooperative Banks",Banks}	{FFIC,ESBK,TRST,GCBC,CARV}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:06.32955	2020-10-11 23:38:31.028166
LTXB	LegacyTexas Financial Group Inc.	LegacyTexas Financial Group Inc is a financial services provider. It operates banks, which provide commercial banking services. The company's revenue is derived from loans and investment securities, and service charges and fees on deposits.	{"Financial Services","Banks - Regional - US",Banks}	{FFIN,IBOC,SBSI,TCBI,HBCP,CFR,PB,IBTX}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:06.383887	2020-10-11 23:38:31.028166
MAIN	Main Street Capital Corporation	Main Street Capital Corp is an investment firm engaged in providing customized debt and equity financing to lower middle market companies and debt capital to middle market companies.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.459903	2020-10-11 23:38:31.028166
MAV	Pioneer Municipal High Income Advantage Trust of Beneficial Interest	Pioneer Muni High Income Advantage Tr is a closed-end fund designed to pursue high current income exempt from regular federal income tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.50153	2020-10-11 23:38:31.028166
IYK	iShares U.S. Consumer Goods	The investment seeks to track the investment results of the Dow Jones U.S. Consumer Goods Index composed of U.S. equities in the consumer goods sector.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index measures the performance of the consumer goods sector of the U.S. equity market. The fund may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.238049	2020-10-11 23:38:31.023007
IZEA	IZEA Worldwide Inc.	IZEA Inc engages in creating and operating marketplaces connecting marketers with content creator that produce and distribute text, videos and photos for marketers, blogs and social media channels.	{Technology,"Internet Content & Information","Online Media"}	{XLK}	Online Media	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:05.255357	2020-10-11 23:38:31.023007
JAX	J. Alexander's Holdings Inc.	J. Alexander's Holdings Inc conducts restaurant business in the United States. It owns a collection of restaurants that offer a casual fine dining experience such as J. Alexander's, Redlands Grill and Stoney River Steakhouse.	{"Consumer Cyclical",Restaurants}	{JAX,KHC,NOMD}	Restaurants	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:05.268345	2020-10-11 23:38:31.023007
JHME	John Hancock Multifactor Energy	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the John Hancock Dimensional Energy Index (the index).\n The fund normally invests at least 80% of its net assets (plus any borrowings for investment purposes) in securities that compose the fund's benchmark index. The index is designed to comprise securities in the energy sector within the U.S. Universe whose market capitalizations are larger than that of the 1001st largest U.S. company at the time of reconstitution. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.355708	2020-10-11 23:38:31.023007
KFFB	Kentucky First Federal Bancorp	Kentucky First Federal Bancorp is a savings and loan holding company for First Federal Savings and Loan Association of Hazard, and First Federal Savings Bank of Frankfort that provide financial products and services in Kentucky and Perry County.	{"Financial Services","Savings & Cooperative Banks",Banks}	{HFBC,HBCP,PBSK}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:05.740941	2020-10-11 23:38:31.023007
KOF	Coca Cola Femsa S.A.B. de C.V.	Coca-Cola Femsa SAB de CV caters to the beverage industry. It is a franchise bottler of Coca-Cola trademark beverages.	{"Consumer Defensive","Beverages - Soft Drinks","Beverages - Non-Alcoholic"}	{COKE,FIZZ,AKO.B,FMX,KO,CCEP,PEP}	Beverages - Non-Alcoholic	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:05.863339	2020-10-11 23:38:31.023007
LDUR	PIMCO Enhanced Low Duration Active Exchange-Traded Fund	The investment seeks maximum total return, consistent with preservation of capital and prudent investment management.\n The fund seeks to achieve its investment objective by investing at least 80% of its net assets in a diversified portfolio of Fixed Income Instruments of varying maturities, which may be represented by forwards or derivatives such as options, futures contracts, or swap agreements. It invests primarily in investment grade debt securities, but may invest up to 15% of its total assets in high yield securities, as rated by Moody's, S&P or Fitch, or, if unrated, as determined by PIMCO.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.065337	2020-10-11 23:38:31.023007
LTC	LTC Properties Inc.	LTC Properties Inc is a self-administered health care real estate investment trust. It invests in senior housing and health care properties through acquisitions, development, mortgage loans and other investments.	{"Real Estate","REIT - Healthcare Facilities",REITs}	{CALM,HR,ARE,NHI,MPW,BKD,HCP,DOC,TIPT,WELL}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:06.349364	2020-10-11 23:38:31.023007
LULU	lululemon athletica inc.	Lululemon Athletica Inc is a designer and retailer of technical athletic apparel operating in North America and Australia. Its yoga-inspired apparel is marketed under the lululemon athletica and ivivva athletica brand names.	{"Consumer Cyclical","Apparel Stores","Retail - Apparel & Specialty"}	{ZUMZ,GPS,NKE,EXPR,TLRD,BEBE}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:06.389399	2020-10-11 23:38:31.023007
LW	Lamb Weston Holdings Inc.	Lamb Weston Holdings Inc is a producer, provider, and marketer of value-added frozen potato products. It is a supplier of frozen potato products internationally. Its products include puff, chips, steak cut fries, trim fries, appetizers, among others.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{LW}	Consumer Packaged Goods	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:06.402748	2020-10-11 23:38:31.023007
MCA	Blackrock MuniYield California Quality Fund Inc.	Blackrock Muniyield CA Insured FD is a diversified, closed-end management investment company. The Fund's investment objective is to provide total return through a combination of current income and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.557899	2020-10-11 23:38:31.023007
MFL	Blackrock MuniHoldings Investment Quality Fund of Beneficial Interest	Blackrock Muniholdings Investment Quality Fund is a non-diversified perpetual closed-end management investment company. Its investment objective is to provide shareholders with current income exempt from regular Federal income tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.744925	2020-10-11 23:38:31.023007
MITK	Mitek Systems Inc.	Mitek Systems Inc offers mobile capture and identity verification software solutions for enterprises. The company through its mobile and imaging technology provides products such as Mobile Verify, Mobile Docs, Mobile Deposit, and Multi Check Capture.	{Technology,"Software - Application","Application Software"}	{NATI,TISA,FISV,JKHY,SSNC,NCR,FICO,FIS,ELLI,XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:06.884513	2020-10-11 23:38:31.023007
MMTM	SPDR S&P 1500 Momentum Tilt	The investment seeks investment results that, before fees and expenses, correspond generally to the total return performance of the S&P 1500 Positive Momentum Tilt Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index applies an alternative weighting methodology to the S&P Composite 1500 Index so that stocks with relatively high momentum are overweight relative to the S&P Composite 1500 Index and stocks with relatively low momentum are underweight. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:07.037577	2020-10-11 23:38:31.023007
JBHT	J.B. Hunt Transport Services Inc.	JB Hunt Transport Services Inc, together with its wholly owned subsidiaries provides surface transportation and delivery services to a group of customers and consumers throughout the continental United States, Canada and Mexico.	{Industrials,"Integrated Shipping & Logistics","Transportation & Logistics"}	{HUBG,ODFL,WERN,NSC,R,KSU,RRTS,ARCB,KNX}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:05.272563	2020-10-11 23:38:31.024353
KMPR	Kemper Corporation	Kemper Corp through its subsidiaries, provides automobile, homeowners, life, health, and other insurance products to individuals and businesses. It conducts its operations through two segments: Property & Casualty Insurance and Life & Health Insurance.	{"Financial Services","Insurance - Diversified",Insurance}	{SLF,HIG,CNA,FFG}	Insurance	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.824297	2020-10-11 23:38:31.024353
KWEB	KraneShares Trust CSI China Internet	The investment seeks to provide investment results that, before fees and expenses, correspond to the price and yield performance of the MSCI China A Inclusion Index (the "underlying index").\n The fund will invest at least 80% of its total assets in securities of the underlying index, depositary receipts representing such securities and securities underlying depositary receipts in the underlying index. The index reflects the Chinese renminbi -denominated equity securities listed on the Shenzhen or Shanghai Stock Exchanges ("A Shares") included in the MSCI Emerging Markets Index assuming that index's methodology permitted the full inclusion of A Shares.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.948626	2020-10-11 23:38:31.024353
LBTYA	Liberty Global plc Class A Ordinary Shares	Liberty Global PLC is a holding company engaged in making investments into international television and broadband companies. Through its subsidiaries, it offers telecommunication, broadband services and cable network.	{"Communication Services","Pay TV"}	{RCI,SJR,DISH,CHTR}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:12:06.02668	2020-10-11 23:38:31.024353
LFUS	Littelfuse Inc.	Littelfuse Inc is engaged in designing, manufacturing and selling of circuit protection product for the electronics, automotive and industrial markets. It offers electromechanical and electronic switch and control devices for commercial vehicles.	{Technology,"Electronics Distribution","Computer Hardware"}	{BELFA,POWL,SMTC,ON,AVX,STM,CTS,GE,VSH,TEL,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:06.111653	2020-10-11 23:38:31.024353
LKOR	FlexShares Credit-Scored US Long Corporate Bond In	The investment seeks investment results that correspond generally to the price and yield performance, before fees and expenses, of the Northern Trust Credit-Scored US Long Corporate Bond IndexSM.\n The fund generally will invest at least 80% of its total assets in the securities of its underlying index. The underlying index reflects the performance of U.S.-dollar denominated bonds of companies to have higher credit quality, lower risk of default and the potential for higher yield, price appreciation and liquidity relative to the universe of securities comprising the Northern Trust Investment-Grade US Long Corporate Bond IndexSM. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:06.168571	2020-10-11 23:38:31.024353
LNC	Lincoln National Corporation	Lincoln National Corp offers individual and group insurance, retirement, and investment products in the United States and the United Kingdom. It distributes its products through independent and company-employed agents, and banks.	{"Financial Services","Insurance - Life"}	{PUK,AEG,UNM,MET,PRU,CNO,VOYA,NWLI}	Insurance - Life	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.230344	2020-10-11 23:38:31.024353
MBCN	Middlefield Banc Corp.	Middlefield Banc Corp is a bank holding company. The company through its subsidiaries is engaged in the general commercial banking business in Northeastern and Central Ohio.	{"Financial Services","Banks - Regional - US",Banks}	{CCBG,OVBC,CCNE,CBSH,FBNC,BSRR,BHB,FMNB,ASB,CIVB}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:06.511762	2020-10-11 23:38:31.024353
MHD	Blackrock MuniHoldings Fund Inc.	Blackrock Muniholdings Fund Inc is a diversified, closed-end management investment company. The Fund's investment objective is to provide total return through a combination of current income and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.820368	2020-10-11 23:38:31.024353
MWA	MUELLER WATER PRODUCTS	Mueller Water Products, Inc. manufactures and markets products used in transmission, distribution and measurement of water. Its products are used by municipalities and the residential and non-residential construction industries.	{Industrials,"Diversified Industrials","Industrial Products"}	{SNHY,ITRI,FLS,BMI,CW,MLI,WTS,CIR,HDS}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:07.406622	2020-10-11 23:38:31.024353
MYD	Blackrock MuniYield Fund Inc.	Blackrock Muniyield Fund Inc is a diversified, closed-end management investment company. The Fund's investment objective is to provide total return through a combination of current income and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.430058	2020-10-11 23:38:31.024353
NCB	Nuveen California Municipal Value Fund 2 of Beneficial Interest	Nuveen CA Municipal Value Fund 2 is a fund company. The primary investment objective is to provide current income exempt from regular federal and California income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.588655	2020-10-11 23:38:31.024353
NGL	NGL ENERGY PARTNERS LP representing Limited Partner Interests	NGL Energy Partners LP purchases propane, butane, and other natural gas liquids from refiners, processing plants, producers, and other parties, and sells the product to retailers, refiners, and other participants in the wholesale markets.	{Energy,"Oil & Gas Refining & Marketing","Oil & Gas - Refining & Marketing"}	{MMLP,LNG,SEMG,BKEP,WPX,ANDX}	Oil & Gas - Refining & Marketing	Energy	New York Stock Exchange	\N	2020-10-11 23:12:07.759747	2020-10-11 23:38:31.024353
NMS	Nuveen Minnesota Quality Municipal Income Fund	The Fund invests at least 80% of its Managed Assets in investment grade securities that, at the time of investment, are rated within the four highest grades (Baa or BBB or better) by at least one of the NRSROs that rate such security or\nare unrated but judged to be of comparable quality by the adviser and/or the sub-adviser. The Fund may invest up to 20% of its Managed Assets in municipal\nsecurities that, at the time of investment, are rated below investment grade or are unrated but judged to be of comparable quality by the adviser and/or the sub-adviser.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.869406	2020-10-11 23:38:31.024353
JETS	U.S. Global Jets	The investment seeks to track the performance, before fees and expenses, of the U.S. Global Jets Index.\n The fund uses a "passive management" (or indexing) approach to track the performance, before fees and expenses, of the index. The index is composed of the exchange-listed common stock (or depository receipts) of U.S. and international passenger airlines, aircraft manufacturers, airports, and terminal services companies (as determined by independent industry listings) across the globe (collectively, "Airline Companies"). The index may include small-, mid-, and large-capitalization companies. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.33073	2020-10-11 23:38:31.025871
KBE	SPDR S&P Bank	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the S&P Banks Select Industry Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index tracks the performance of publicly traded national money centers and leading regional banks. It may invest in equity securities that are not included in the index, cash and cash equivalents or money market instruments, such as repurchase agreements and money market funds. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.672356	2020-10-11 23:38:31.025871
KFS	Kingsway Financial Services Inc.	Kingsway Financial Services Inc acts as a merchant bank which is engaged in the property and casualty insurance business. It provides coverage for liability, accident benefits and physical damage.	{"Financial Services","Insurance - Property & Casualty"}	{ALL,MCY,PGR,AFH}	Insurance - Property & Casualty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:05.740719	2020-10-11 23:38:31.025871
LDP	Cohen & Steers Limited Duration Preferred and Income Fund Inc.	Cohen & Steers Ltd Duration Pref & Inc is a closed-end management investment company. Its objective is high current income. The company invests maximum in Banks.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.062279	2020-10-11 23:38:31.025871
LEGR	First Trust Indxx Innovative Transaction & Process	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an index called the Indxx Blockchain Index (the "index").\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks and depositary receipts that comprise the index. The index is designed to track the performance of companies that are either actively using, investing in, developing, or have products that are poised to benefit from blockchain technology and/or the potential for increased efficiency that it provides to various business processes. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:06.082946	2020-10-11 23:38:31.025871
LMNX	Luminex Corporation	Luminex Corp develops, manufactures and sells proprietary biological testing technologies and products with applications throughout the life sciences and diagnostics industries. Its products include MAGPIX, Luminex 100/200, and FLEXMAP 3D.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{HOLX,ILMN,ABT,BDX,BIO,TMO,PACB,FLDM,NSTG,XLV}	Medical Instruments & Equipment	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:06.21352	2020-10-11 23:38:31.025871
LVHB	Innovator Lunt Low Vol/High Beta Tactical	The investment seeks investment results that generally correspond (before fees and expenses) to the price and yield of the Lunt Capital U.S. Large Cap Equity Rotation Index.\n The fund generally will invest at least 80% of its total assets in the U.S.-listed large capitalization common stocks that comprise the index. The index utilizes Lunt Capital's proprietary relative strength methodology to rotate between one of two sub-indices that identify certain components of the S&P 500. The first sub-index is the S&P 500 Low Volatility Index; the second sub-index is the S&P 500 High Beta Index. The fund is non-diversified.	{}	{}	Application Software	Technology	Cboe Global Markets EDGX	\N	2020-10-11 23:12:06.388532	2020-10-11 23:38:31.025871
MBG	SPDR Bloomberg Barclays Mortgage Backed Bond	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the price and yield performance of the Bloomberg Barclays U.S. MBS Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index or in securities that the Adviser determines have economic characteristics that are substantially identical to the economic characteristics of the securities that comprise the index. The index is designed to measure the performance of the U.S. agency mortgage pass-through segment of the U.S. investment grade bond market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.522521	2020-10-11 23:38:31.025871
MEAR	iShares Short Maturity Municipal Bond	The investment seeks to maximize tax-free current income.\n The fund normally invests at least 80% of its net assets in municipal securities such that the interest on each bond is exempt from U.S. federal income taxes and the federal alternative minimum tax ("AMT"). It primarily invests in U.S. dollar-denominated investment-grade short-term fixed- and floating-rate municipal securities with remaining maturities of five years or less, such as municipal bonds, municipal notes and variable rate demand obligations, as well as money market instruments and registered investment companies. The fund is an actively managed exchange-traded fund ("ETF").	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:06.685697	2020-10-11 23:38:31.025871
MFC	Manulife Financial Corporation	Manulife Financial Corp together with its subsidiaries provides individual life insurance and individual and group long-term care insurance services. Its business segments are Asia Division, Canadian Division, U.S. Division, and Corporate and Other.	{"Financial Services","Insurance - Life"}	{SLF,MIC,AEG,LFC,CNO,VOYA}	Insurance - Life	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.72165	2020-10-11 23:38:31.025871
MGP	MGM Growth Properties LLC Class A representing limited liability company interests	MGM Growth Properties LLC is a REIT engaged in owning, acquiring and leasing casino resort properties, which include casino gaming, hotel, convention, dining, entertainment, retail and mixed-use facilities, and other resort amenities.	{"Real Estate","REIT - Hotel & Motel",REITs}	{NYMT,O,NTP}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:06.803485	2020-10-11 23:38:31.025871
MLAB	Mesa Laboratories Inc.	Mesa Laboratories Inc designs, manufactures and markets quality control instruments and disposable products and also manufactures and markets biological indicators and distributes chemical indicators.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{RMTI,HBIO,BRKR,ANGO,AME,CMD,GE,TMO,MMM,STE,XLK}	Computer Hardware	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:06.935683	2020-10-11 23:38:31.025871
JHML	John Hancock Multifactor Large Cap	The investment seeks to provide investment results that closely correspond, before fees and expenses, to the performance of the John Hancock Dimensional Large Cap Index.\n The fund normally invests at least 80% of its net assets (plus any borrowings for investment purposes) in securities that compose the fund's benchmark index. The index is designed to comprise a subset of securities in the U.S. Universe issued by companies whose market capitalizations are larger than that of the 801st largest U.S. company at the time of reconstitution.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.363248	2020-10-11 23:38:31.021426
JPXN	iShares JPX-Nikkei 400	The investment seeks to track the investment results of the JPX-Nikkei Index 400 composed of Japanese equities.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. The underlying index may include large-, mid- or small-capitalization companies. The currency of the component securities of the underlying index is the Japanese yen ("JPY").	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.595962	2020-10-11 23:38:31.021426
JWN	Nordstrom Inc.	Nordstrom Inc operates as a fashion specialty retailer in the United States. It sells its products through various channels, including Nordstrom full-line stores, Nordstrom Rack off-price stores, Last Chance clearance stores, and Jeffrey boutiques.	{"Consumer Cyclical","Department Stores","Retail - Apparel & Specialty"}	{ROST,SMRT,CTRN,DDS,BKE,KSS,JCP,TJX,SSI,M}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:05.638186	2020-10-11 23:38:31.021426
KBWP	Invesco KBW Property & Casualty Insurance ETF	The investment seeks to track the investment results (before fees and expenses) of the KBW Nasdaq Property & Casualty Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities of property and casualty insurance companies that comprise the underlying index. It generally invests in all of the securities comprising the underlying index in proportion to their weightings in the underlying index. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:05.698303	2020-10-11 23:38:31.021426
KEYS	Keysight Technologies Inc.	Keysight Technologies Inc is a measurement company providing core electronic design and test solutions to communications and electronics industries. It provides electronic measurement instruments and systems and related software, software design tools.	{Technology,"Scientific & Technical Instruments","Computer Hardware"}	{FARO,ITRI,MTSC,BRKR,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:05.736588	2020-10-11 23:38:31.021426
KGC	Kinross Gold Corporation	Kinross Gold Corp is gold mining company. Its mining activities include exploration and acquisition of gold-bearing properties, extraction and processing of gold-containing ore and reclamation of gold mining properties.	{"Basic Materials",Gold,"Metals & Mining"}	{HL}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:05.756527	2020-10-11 23:38:31.021426
KRNT	Kornit Digital Ltd.	Kornit Digital Ltd develops designs and markets digital printing solutions for the global printed textile industry.	{Industrials,"Diversified Industrials","Industrial Products"}	{EBF,QUAD,ADES}	Industrial Products	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:05.902265	2020-10-11 23:38:31.021426
LEG	Leggett & Platt Incorporated	Leggett & Platt Inc is a manufacturer that conceives designs and produces engineered components and products found in homes, offices, retail stores, automobiles and commercial aircraft.	{"Consumer Cyclical","Home Furnishings & Fixtures","Manufacturing - Apparel & Furniture"}	{CRWS,FLXS,MLHR,ETH,SCS,LZB,TPX,HNI,KNL,SNBR}	Manufacturing - Apparel & Furniture	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:06.083741	2020-10-11 23:38:31.021426
LGLV	SPDR SSGA US Large Cap Low Volatility Index	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the SSGA US Large Cap Low Volatility Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index is designed to measure the performance of the stocks of U.S. large capitalization companies that exhibit low volatility. Volatility is a statistical measurement of the magnitude of movements in a stock's price over time. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.121764	2020-10-11 23:38:31.021426
LTRPA	Liberty TripAdvisor Holdings Inc. Series A Common Stock	Liberty TripAdvisor Holdings Inc provides online travel site services and online retail through its subsidiaries TripAdvisor and BuySeasons.	{Technology,"Internet Content & Information","Online Media"}	{MCRI,XLK}	Online Media	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:06.362498	2020-10-11 23:38:31.021426
MFNC	Mackinac Financial Corporation	Mackinac Financial Corp is a bank holding company. The company, through its subsidiary, is engaged in the general commercial banking business, providing a full range of loan and deposit products.	{"Financial Services","Banks - Regional - US",Banks}	{MBWM,MCBC,CHFC,FITB,MBTF,IBCP}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:06.746556	2020-10-11 23:38:31.021426
MSEX	Middlesex Water Company	Middlesex Water Co is a water utility based in the U.S. state of New Jersey. It is engaged the business of collecting, treating, distributing and selling water for domestic, commercial, municipal, industrial and fire protection purposes.	{Utilities,"Utilities - Regulated Water","Utilities - Regulated"}	{YORW,ARTNA,CTWS,CDZI,CWT,AWR,WTR,AWK,SJW,XLU}	Utilities - Regulated	Utilities	Nasdaq Global Select	\N	2020-10-11 23:12:07.245909	2020-10-11 23:38:31.021426
MUSA	Murphy USA Inc.	Murphy USA Inc. is engaged in the marketing of retail motor fuel products and convenience merchandise through a chain of retail stores in the United States.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{CASY,CLNE,VLO,MPC,TA}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:07.37417	2020-10-11 23:38:31.021426
MXC	Mexco Energy Corporation	Mexco Energy Corporation is an independent oil and gas company. It is engaged in the exploration, development, and production of natural gas and crude oil properties located in the United States.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{PXD,EGN,EOG,XEC,CXO,LPI}	Oil & Gas - E&P	Energy	NYSE American	\N	2020-10-11 23:12:07.414707	2020-10-11 23:38:31.021426
NI	NiSource Inc	NiSource Inc is an energy holding company, whose subsidiaries are fully regulated natural gas and electric utility companies serving customers in seven states.	{Utilities,"Utilities - Regulated Gas","Utilities - Regulated"}	{SRE,ATO,CPK,VVC,GNE,DUK,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:07.792708	2020-10-11 23:38:31.021426
KGRN	KraneShares MSCI China Environment Index	The investment seeks to provide investment results that correspond to the price and yield performance of MSCI China IMI Environment 10/40 Index.\n Under normal circumstances, the fund will invest at least 80% of its total assets in components of the underlying index, depositary receipts. The underlying index is a modified, free float adjusted market capitalization weighted index designed to track the equity market performance of Chinese companies that derive at least a majority of their revenues from environmentally beneficial products and services, as determined by MSCI Inc. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.762097	2020-10-11 23:38:31.02423
KINS	Kingstone Companies Inc	Kingstone Companies Inc is a property and casualty insurance holding company. The company through its wholly owned subsidiary, Kingstone Insurance Company offers insurance policies to small and mid-sized businesses as well as to individuals.	{"Financial Services","Insurance Brokers","Brokers & Exchanges"}	{ERIE,STFC,DGICA,ALL,MKL,RLI,WTM}	Brokers & Exchanges	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:05.782966	2020-10-11 23:38:31.02423
KVHI	KVH Industries Inc.	KVH Industries Inc is a manufacturer of solutions that provide high-speed Internet, television and voice services through satellite to mobile users at sea and on land.	{Technology,"Communication Equipment"}	{VSAT,CMTL,FEIM,DISH,IRDM,HRS,HON,NYT,NWSA,GSAT,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:05.948768	2020-10-11 23:38:31.02423
LBY	Libbey Inc.	Libbey Inc is engaged in manufacturing and marketing of glass tableware products. Its tableware products consist of glass tableware, ceramic dinnerware, and metal flatware, hollowware and serveware and glass tableware include tumblers, stemware.	{"Consumer Cyclical","Home Furnishings & Fixtures","Manufacturing - Apparel & Furniture"}	{AMWD,APOG,LCUT,TUP}	Manufacturing - Apparel & Furniture	Consumer Cyclical	NYSE American	\N	2020-10-11 23:12:06.028905	2020-10-11 23:38:31.02423
MLSS	Milestone Scientific Inc.	Milestone Scientific Inc provides computer-controlled injection technologies and solutions for the medical and dental markets. It has developed a proprietary, computer-controlled anesthetic delivery instrument, through the use of The Wand.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{ALGN,APT,XRAY,XLV}	Medical Instruments & Equipment	Healthcare	NYSE American	\N	2020-10-11 23:12:06.984465	2020-10-11 23:38:31.02423
MOM	AGFiQ U.S. Market Neutral Momentum Fund	The investment seeks performance results that correspond to the price and yield performance, before fees and expenses, of the Dow Jones U.S. Thematic Market Neutral Momentum Index.\n The fund invests at least 80% of its net assets in common stock of the long positions in the underlying index and sell short at least 80% of the short positions in the underlying index. The index is a long/short market neutral index that is dollar-neutral. As such, it identifies long and short securities positions of approximately equal dollar amounts. In choosing to track a market neutral index, the fund seeks to limit the effects of general market movements on the fund.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:07.103668	2020-10-11 23:38:31.02423
MPAA	Motorcar Parts of America Inc.	Motorcar Parts of America Inc is a manufacturer, remanufacturer, and distributor of aftermarket automotive parts for import and domestic cars, light trucks, heavy duty, agricultural and industrial applications.	{"Consumer Cyclical","Auto Parts",Autos}	{STRT,SRI,THRM}	Autos	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:07.14112	2020-10-11 23:38:31.02423
MVIN	Natixis ETF Trust	The investment seeks long-term capital appreciation with less volatility than typically experienced by international equity markets.\n Under normal circumstances, the fund invests primarily in non-U.S. equity securities, which may include common stocks, preferred stocks, and REITs. It may invest in companies of any size and typically invests in a number of different countries throughout the world. The fund may invest in the stocks of non-U.S. issuers directly or indirectly through depositary receipts. The portfolio may also be exposed to currencies other than the U.S. dollar.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:07.38889	2020-10-11 23:38:31.02423
MXF	Mexico Fund Inc. (The)	Mexico Fund Inc is a closed-end non-diversified management investment company. Its investment objective is to seek long-term capital appreciation through investment in securities, primarily equity, listed on the Mexican Stock Exchange.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.409349	2020-10-11 23:38:31.02423
NGE	Global X MSCI Nigeria	The investment seeks to provide investment results that correspond generally to the price and yield performance, before fees and expenses, of the MSCI All Nigeria Select 25/50 Index.\n The fund will invest at least 80% of its total assets in the securities of the underlying index and in American Depositary Receipts ("ADRs") and Global Depositary Receipts ("GDRs") based on the securities in the underlying index. The underlying index is designed to represent the performance of the broad Nigeria equity universe, while including a minimum number of constituents. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:07.742067	2020-10-11 23:38:31.02423
NIHD	NII Holdings Inc.	NII Holdings Inc is a provider of telecom services. The company's services include mobile telephone voice and wireless data services,international voice and data roaming services, application-based radio connection and value-added services.	{"Communication Services","Telecom Services"}	{SPOK}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:12:07.803141	2020-10-11 23:38:31.02423
NXP	Nuveen Select Tax Free Income Portfolio	Nuveen Select Tax Free Income Portfolio is a diversified closed-end management investment company. It seeks to provide current income and stable dividends, exempt from regular federal and designated state income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.181316	2020-10-11 23:38:31.02423
OFC	Corporate Office Properties Trust	Corporate Office Properties Trust is a fully-integrated and self-managed real estate investment trust. Its portfolio comprises of office buildings in the Baltimore-Washington Corridor and Northern Virginia area.	{"Real Estate","REIT - Office",REITs}	{ARE,CTRE,CMCT}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:08.314549	2020-10-11 23:38:31.02423
PBSM	Invesco PureBeta MSCI USA Small Cap	The investment seeks to track the investment results (before fees and expenses) of the MSCI USA Small Cap Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in common stocks that comprise the underlying index. The index is designed to measure the performance of the small-capitalization segment of the U.S. equity market. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:08.797508	2020-10-11 23:38:31.031328
LDRI	PowerShares LadderRite 0-5 Year Corporate Bond Ptf	The investment seeks to track the investment results (before fees and expenses) of the NASDAQ LadderRite® 0-5 Year USD Corporate Bond Index (the "underlying index").\n The fund generally will invest at least 80% of its total assets in U.S. and foreign investment grade corporate bonds that comprise the underlying index. The underlying index measures potential returns of a theoretical portfolio of fixed income securities with a yield curve based upon remaining effective terms to maturity of five years or less.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:06.063487	2020-10-11 23:38:31.023428
MAG	MAG Silver Corporation	MAG Silver Corp is a mineral exploration and development company based in Canada. The group focuses on district-scale projects in the Mexican Silver Belt, and its main source of revenue is interest income from high interest savings accounts.	{"Basic Materials",Silver,"Metals & Mining"}	{PAAS,EXK,SVM,AXR,HL,AG,GPL,WPM}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:06.449548	2020-10-11 23:38:31.023428
MIC	Macquarie Infrastructure Corporation	Macquarie Infrastructure Corp is a infrastructure company which owns, operates, and invests in a diversified group of infrastructure businesses. Its business activities include airport refueling, automobile parking, water cooling, and gas distribution.	{Industrials,"Airports & Air Services","Transportation & Logistics"}	{}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:06.848747	2020-10-11 23:38:31.023428
MTD	Mettler-Toledo International Inc.	Mettler-Toledo International Inc supplies weighing and precision instruments to customers in the life sciences, industrial and food retail industries. Its products are laboratory scales, pipettes, pH meters, thermal analysis equipment and others.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{BNSO,HBIO,BRKR,WAT,BIO,DHR,TMO,PKI,XYL,XLV}	Medical Diagnostics & Research	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:07.290344	2020-10-11 23:38:31.023428
MTUM	iShares Edge MSCI USA Momentum Factor	The investment seeks to track the investment results of the MSCI USA Momentum Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The index consists of stocks exhibiting relatively higher momentum characteristics than the traditional market capitalization-weighted parent index, the MSCI USA Index, which includes U.S. large- and mid-capitalization stocks.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:07.340344	2020-10-11 23:38:31.023428
MUNI	PIMCO Intermediate Municipal Bond Active Exchange-Traded Fund	The investment seeks attractive tax-exempt income, consistent with preservation of capital.\n The fund invests at least 80% of its assets in a diversified portfolio of debt securities whose interest is, in the opinion of bond counsel for the issuer at the time of issuance, exempt from federal income tax ("Municipal Bonds"). It may invest 25% or more of its total assets in Municipal Bonds that finance similar projects, such as those relating to education, health care, housing, transportation, and utilities, and 25% or more of its total assets in industrial development bonds.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:07.368333	2020-10-11 23:38:31.023428
MVV	ProShares Ultra MidCap400	The investment seeks daily investment results, before fees and expenses, that correspond to two times (2x) the daily performance of the S&P MidCap 400®.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index is a float-adjusted, market capitalization-weighted index of 400 U.S. operating companies and real estate investment trusts selected through a process that assesses criteria such as liquidity, price, market capitalization and financial viability. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:07.396573	2020-10-11 23:38:31.023428
MYF	Blackrock MuniYield Investment Fund	Blackrock Muniyield Invstmt Fd is a closed-ended investment management fund. Its investment objective is to provide shareholders with as high a level of current income exempt from federal income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.442901	2020-10-11 23:38:31.023428
NEE	NextEra Energy Inc.	NextEra Energy Inc is an electric power company in North America. It generates renewable energy from wind and sun. The company also owns generation, transmission and distribution facilities and has investments in gas infrastructure assets.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{ETR,FE,SO,EXC,D,SCG,PPL,DUK,ES,XEL,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:07.644739	2020-10-11 23:38:31.023428
NUROW	NeuroMetrix (Warrant)	NeuroMetrix, Inc. is a healthcare company, which engages in the research, development, and trade of medical equipment and consumables. It offers an over-the-counter wearable therapeutic device for chronic pain, the Quell Health Cloud, DPNCheck, and pain management devices to treat chronic pain, nerve diseases, and sleep disorder. The company was founded by Shai N. Gozani in June 1996 and is headquartered in Waltham, MA.	{}	{}	Medical Specialties	Health Technology	NasdaqCM	\N	2020-10-11 23:12:08.079215	2020-10-11 23:38:31.023428
NXPI	NXP Semiconductors N.V.	NXP Semiconductors NV is a semiconductor company. It mainly provides high-performance mixed signal and standard product solutions for automotive, identification, wireless infrastructure, industrial, mobile, consumer and computing applications.	{Technology,Semiconductors}	{MXIM,MU,SMTC,XLNX,NVDA,MRVL,ON,STM,TXN,ADI,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:08.185848	2020-10-11 23:38:31.023428
OAS	Oasis Petroleum Inc.	Oasis Petroleum Inc is an independent oil and gas producer in the United States. The company operates exclusively in the Williston Basin.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{DNR,MRO,XEC,NOG,CLR,QEP,ERF,VET,WLL}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:08.243935	2020-10-11 23:38:31.023428
OCIO	ETF Series Solutions Trust	The investment seeks to outperform a traditional 60/40 mix of global equity and fixed-income investments.\n The fund is expected to typically invest approximately 40% to 70% of its total assets in underlying funds that principally invest in equity securities of any market capitalization. It is expected to typically invest approximately 20% to 50% of its total assets in underlying funds that principally invest in debt obligations. In addition to the underlying funds, the fund may invest in non-investment company exchange-traded products.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.268175	2020-10-11 23:38:31.023428
LEU	Centrus Energy Corp. Class A	Centrus Energy Corp is engaged in supplying low enriched uranium (LEU) for commercial nuclear power plants. The LEU is used for the production of nuclear fuel for reactors to produce electricity.	{"Basic Materials","Industrial Metals & Minerals","Metals & Mining"}	{URG,EE,SO,EXC,PPL,UEC,NEE,DUK,GIGM,WWR}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:06.097733	2020-10-11 23:38:31.031328
LIT	Global X Lithium & Battery Tech	The investment seeks to provide investment results that correspond generally to the price and yield performance, before fees and expenses, of the Solactive Global Lithium Index.\n The fund invests at least 80% of its total assets in the securities of the underlying index and in American Depositary Receipts ("ADRs") and Global Depositary Receipts ("GDRs") based on the securities in the underlying index. The underlying index is designed to measure broad-based equity market performance of global companies involved in the lithium industry. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.154455	2020-10-11 23:38:31.031328
LVHI	Legg Mason International Low Volatility High Dividend	The investment seeks to track the investment results of the QS International Low Volatility High Dividend Hedged Index.\n The fund will invest at least 80% of its net assets, plus borrowings for investment purposes, if any, in securities that compose its underlying index. The underlying index seeks to provide more stable income through investments in stocks of profitable companies in developed markets outside of the United States with relatively high dividend yields or anticipated dividend yields and lower price and earnings volatility, while mitigating exposure to exchange-rate fluctuations between the USD and other international currencies.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:06.394901	2020-10-11 23:38:31.031328
MAR	Marriott International	Marriott International Inc is an operator, franchisor and licensor of hotels and timeshare properties under different brand names. It also operates and develops residential properties and provides services to home/condominium owner associations.	{"Consumer Cyclical",Lodging,"Travel & Leisure"}	{MTN,CHH,WYND,H,HLT,IHG,STAY}	Travel & Leisure	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:06.47979	2020-10-11 23:38:31.031328
MC	Moelis & Company Class A	Moelis & Company is a global independent investment bank that provides innovative, strategic advice to a diverse client base. It serves client such as corporations, governments, and financial sponsors. It generates a majority of its revenue from the U.S.	{"Financial Services","Asset Management"}	{GHL,LAZ,EVR,HF}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.559595	2020-10-11 23:38:31.031328
MGC	Vanguard Mega Cap	The investment seeks to track the performance of a benchmark index that measures the investment return of large-capitalization stocks in the United States.\n The fund employs an indexing investment approach designed to track the performance of the CRSP US Mega Cap Index. The index is a float-adjusted, market-capitalization-weighted index designed to measure equity market performance of mega-capitalization stocks in the United States. The Advisor attempts to replicate the target index by investing all, or substantially all, of its assets in the stocks that make up the index, holding each stock in approximately the same proportion as its weighting in the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.777103	2020-10-11 23:38:31.031328
MLTI	Credit Suisse X-Links Multi-Asset High Income Exchange Traded Notes (ETNs) due September 28 2035	The investment seeks to provide exposure to the price return version of the NYSE Multi-Asset High Income Index (the ¿index¿).\n The ETNs are listed on NYSE Arca under the ticker symbol ¿MLTI¿. The NYSE Multi-Asset High Income Index (the ¿index¿) measures the performance of a broad, diversified basket of up to 120 publicly-traded securities that historically have paid high dividends or distributions (the ¿index constituents¿).	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.986571	2020-10-11 23:38:31.031328
MXIM	Maxim Integrated Products Inc.	Maxim Integrated Products Inc designs, develops, manufactures and markets a range of linear and mixed-signal integrated circuits, commonly referred to as analog circuits, for number of customers in diverse geographical locations.	{Technology,Semiconductors}	{MCHP,CRUS,SMTC,SLAB,ON,STM,TXN,ADI,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:07.416872	2020-10-11 23:38:31.031328
NKE	Nike Inc.	Nike Inc designs, develops and markets footwear, apparel, equipment, and accessory products. It is a seller of athletic footwear and athletic apparel. It sells its products through NIKE-owned in-line and factory retail stores and internet websites.	{"Consumer Cyclical","Footwear & Accessories","Manufacturing - Apparel & Furniture"}	{SHOO,CROX,SKX,UAA,DECK}	Manufacturing - Apparel & Furniture	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:07.819449	2020-10-11 23:38:31.031328
NMY	Nuveen Maryland Quality Municipal Income Fund	To provide current income exempt from regular federal and Maryland income tax and to enhance portfolio value relative to the municipal bond market by investing in tax-exempt municipal bonds that the Funds investment adviser believes are underrated or undervalued or that represent municipal market sectors that are undervalued. It will invest at least 80% of its assets in investment grade quality municipal bonds (BBB/Baa or better at the time of purchase by either Moody or Standard & Poor). It may invest up to 20% of its net assets in municipal bonds that are unrated but judged to be of comparable quality by the investment adviser.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.869843	2020-10-11 23:38:31.031328
OCC	Optical Cable Corporation	Optical Cable Corp manufactures tight-buffer fiber optic and copper data communication cables and data communication connectivity solutions for enterprise markets.	{Technology,"Communication Equipment"}	{PLPC,JCS,RFIL,CLFD,CAW,GLW,BDC,TEL,COMM,XLK}	Communication Equipment	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:08.265474	2020-10-11 23:38:31.031328
ODC	Oil-Dri Corporation Of America	Oil-Dri Corp develops, manufactures, and markets sorbent products made primarily from clay. Its offerings include cat litter, floor products, toxin control substances for livestock, agricultural chemical carriers, and filtration and purification aids.	{"Basic Materials","Specialty Chemicals",Chemicals}	{OBCI,CENTA,BRC,CLX}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:08.28879	2020-10-11 23:38:31.031328
OSB	Norbord Inc.	Norbord Inc is a producer of wood-based panels. It is engaged in manufacturing, sales, marketing and distribution of panelboards and related products. It has over 15 plant locations in the United States, Europe and Canada.	{"Basic Materials","Lumber & Wood Production","Forest Products"}	{UFPI,LPX}	Forest Products	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:08.552868	2020-10-11 23:38:31.031328
LVIN	Hartford Multifactor Low Volatility International Equity	The investment seeks to provide investment results that, before fees and expenses, correspond to the total return performance of the Hartford Multifactor Low Volatility International Equity Index.\n The fund generally invests at least 80 percent of its assets in the equity securities that are components of the index and in depositary receipts (such as ADRs, GDRs and EDRs) representing securities of the index. The index is designed to outperform a capitalization-weighted universe of developed and emerging markets located outside the U.S. with up to one quarter less volatility over a complete market cycle.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:06.396815	2020-10-11 23:38:31.02582
MAGA	Point Bridge GOP Stock Tracker	The investment seeks to track the performance, before fees and expenses, of the Point Bridge GOP Stock Tracker Index.\n Under normal circumstances, at least 80% of the fund's total assets will be invested in the component securities of the index. The index uses an objective, rules-based methodology to track the performance of companies whose employees and political action committees are highly supportive of Republican candidates for election to the United States Congress, the Vice Presidency, or the Presidency ("Candidates") and party-affiliated federal committees or groups that are subject to federal campaign contribution limits. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:06.462171	2020-10-11 23:38:31.02582
MDIV	Multi-Asset Diversified Income Index Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an index called the NASDAQ US Multi-Asset Diversified Income IndexSM.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in the common stocks and/or depositary receipts (20%), real estate investment trusts ("REITs") (20%), preferred securities (20%), master limited partnerships ("MLPs") (20%) and an exchange-traded fund ("ETF")(20%) that comprise the index (each, an "index segment").	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:06.642502	2020-10-11 23:38:31.02582
MEIP	MEI Pharma Inc.	MEI Pharma Inc is an oncology company focused on the clinical development of novel therapeutics for cancer. The Company's portfolio of drug candidates includes Pracinostat, an oral histone deacetylase inhibitor.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{CELG,ARRY,LGND,CYCC,SVRA,RNN,CYTR,XLV}	Drug Manufacturers	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:06.695623	2020-10-11 23:38:31.02582
MFDX	PIMCO Equitiy Series RAFI Dynamic Multi-Factor International Equity	The investment seeks to track the investment results of the RAFI Dynamic Multi-Factor Developed Ex-U.S. Index.\n The fund seeks to achieve its investment objective by investing at least 80% of its total assets (exclusive of collateral held from securities lending) in the component securities of the RAFI Dynamic Multi-Factor Developed Ex-U.S. Index. The underlying index is constructed by the index provider using a rules-based approach to construct factor portfolios within the underlying index. The underlying index consists of "factor portfolios," each of which emphasizes one of the following factors: value, low volatility, quality, momentum and size.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.72862	2020-10-11 23:38:31.02582
MOH	Molina Healthcare Inc	Molina Healthcare Inc is a healthcare company. It offers Medicaid-related solutions to meet the health care needs of low-income families and individuals. It also provides plans through health insurance exchanges.	{Healthcare,"Health Care Plans"}	{HUM,UNH,CNC,WCG,GTS,XLV}	Health Care Plans	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:07.102572	2020-10-11 23:38:31.02582
MXI	iShares Global Materials	The investment seeks to track the investment results of the S&P Global 1200 Materials IndexTM.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index. The index measures the performance of companies that the index provider deems to be part of the materials sector of the economy and that the index provider believes are important to global markets.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:07.412768	2020-10-11 23:38:31.02582
MYGN	Myriad Genetics Inc.	Myriad Genetics Inc is a molecular diagnostic company. It develops and markets novel predictive medicine, personalized medicine and prognostic medicine tests. It also provides physicians with information to solve unmet medical needs.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{BMRN,EXAS,GHDX,VRML,ABT,BIO,XLV}	Medical Diagnostics & Research	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:07.443702	2020-10-11 23:38:31.02582
MYOS	MYOS RENS Technology Inc.	Myos Rens Technology Inc focuses on the discovery, development, and commercialization of products that improve muscle health and function essential to the management of sarcopenia, cachexia and degenerative muscle diseases.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{OMER,XNCR,GTXI,XLV}	Drug Manufacturers	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:07.466392	2020-10-11 23:38:31.02582
NKG	Nuveen Georgia Quality Municipal Income Fund	Nuveen Georgia Div Advantage Muni Fund 2 is a closed-ended fixed income mutual fund. The fund invests in municipal securities that are exempt from federal and Georgia state income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.825612	2020-10-11 23:38:31.02582
ODFL	Old Dominion Freight Line Inc.	Old Dominion Freight Lines Inc is a leading, less-than-truckload (LTL) union-free company providing premium service to all its customers. The company offers regional, inter-regional and national LTL service and value-added logistics services.	{Industrials,Trucking,"Transportation & Logistics"}	{HTLD,JBHT,WERN,ULH,SAIA,FDX,RRTS,ARCB,KNX}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:08.292101	2020-10-11 23:38:31.02582
OVLU	Oppenheimer Russell 1000 Value Factor	The investment seeks to provide investment results that correspond generally, before fees and expenses, to the performance of the Russell 1000 Value Factor Index (the "underlying index").\n Under normal circumstances, the fund will invest at least 80% of its net assets, plus any borrowings for investment purposes, in securities of companies included in the underlying index. The underlying index is constructed using a rules-based methodology that selects a subset of the equity securities from the Russell 1000 Index (the "parent index"), which is comprised of the 1,000 largest-capitalization companies in the United States.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:08.634441	2020-10-11 23:38:31.031328
MELR	Melrose Bancorp Inc.	Melrose Bancorp Inc is engaged in taking deposits and investing in one- to four-family residential real estate loans and home equity loans and lines of credit. The company provides financial services to individuals, families and businesses.	{"Financial Services","Banks - Regional - US",Banks}	{FBNC,WEBK,FBC}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:06.702382	2020-10-11 23:38:31.028166
MFT	Blackrock MuniYield Investment Quality Fund of Beneficial Interest	Blackrock Muniyield Investment Quality Fund is a closed-ended investment management fund. Its investment objective is to provide shareholders with as high a level of current income exempt from federal income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:06.750489	2020-10-11 23:38:31.028166
MIDD	The Middleby Corporation	The Middleby Corp through its subsidiaries is engaged in designing, manufacturing, marketing, distributing and servicing foodservice equipment, food preparation, cooking, baking, chilling and packaging equipment and premium kitchen equipment.	{Industrials,"Diversified Industrials","Industrial Products"}	{DOV,GE,ITW,SXI,TNC,WHR,EVI,MTW}	Industrial Products	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:06.851175	2020-10-11 23:38:31.028166
MLPQ	UBS AG ETRACS 2xMonthly Leveraged Alerian MLP Infrastructure Index ETN Series B due February 12 2046	The investment seeks to provide two times leveraged long exposure to the compounded monthly performance of the Alerian MLP Infrastructure Index (the “index”).\n The index, comprising 22 energy infrastructure master limited partnerships, is a subset of the Alerian MLP Infrastructure Index. The index constituent securities earn the majority of their cash flow from the transportation, storage, and processing of energy commodities.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.976713	2020-10-11 23:38:31.028166
NEO	NeoGenomics Inc.	NeoGenomics Inc operates a network of cancer-related testing laboratories in Ft. Myers and Tampa, Florida, Irvine, California and Nashville, and Tennessee.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{PMD,RDNT,DGX,ENZ,LH,XLV}	Medical Diagnostics & Research	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:07.659335	2020-10-11 23:38:31.028166
NGHCN	National General Holdings Corp Depositary Shares each representing 1/40th of a share of 7.50% Non-Cumulative Preferred Stock Series C	National General Holdings Corp is a specialty personal lines insurance holding company. The company manages its business through two segments, Property and Casualty segment and Accident and Health segment.	{"Financial Services","Insurance - Property & Casualty"}	{}	Insurance - Property & Casualty	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:07.746628	2020-10-11 23:38:31.028166
NIM	Nuveen Select Maturities Municipal Fund	Nuveen Select Maturities Municipal Fund is a diversified closed end management investment company. Its investment objective is to provide current income exempt from regular federal income tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.813229	2020-10-11 23:38:31.028166
NOA	North American Construction Group Ltd. (no par)	North American Construction Group Ltd, fomrerly known as North American Energy Partners Inc provides mining and heavy construction services to customers in the resource development and industrial construction sectors, within Western Canada.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:07.888449	2020-10-11 23:38:31.028166
NWFL	Norwood Financial Corp.	Norwood Financial Corp is the holding company for Wayne Bank. It offers personal and business credit services and trust and investment products and real estate settlement services.	{"Financial Services","Banks - Regional - US",Banks}	{CZNC,ORRF,FRBK,BMTC,CCNE,STBA,UVSP,ACNB,TSC,CUBI}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:08.148563	2020-10-11 23:38:31.028166
NYMTP	New York Mortgage Trust Inc. 7.75% Series B Cumulative Redeemable Preferred Stock	New York Mortgage Trust Inc is a real estate investment trust. The Company is engaged in acquiring, investing in, financing and managing mortgage-related assets and, to a lesser extent, financial assets.	{"Real Estate","REIT - Residential",REITs}	{}	REITs	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:08.220215	2020-10-11 23:38:31.028166
OCSL	Oaktree Specialty Lending Corporation	Oaktree Specialty Lending Corp is a specialty finance company. It focuses on providing customized one-stop credit solutions to companies with limited access to public or syndicated capital markets.	{"Financial Services","Credit Services"}	{}	Credit Services	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:08.28152	2020-10-11 23:38:31.028166
OMFS	Oppenheimer Russell 2000 Dynamic Multifactor	The investment seeks to provide investment results that correspond generally, before fees and expenses, to the performance of the Russell 2000 OFI Dynamic Multifactor Index (the "underlying index").\n Under normal circumstances, the fund will invest at least 80% of its net assets, plus any borrowings for investment purposes, in securities of companies included in the underlying index. The underlying index is constructed using a rules-based methodology by selecting equity securities from the Russell 2000 Index (the "parent index"), which measures the performance of 2,000 small-capitalization companies in the United States.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:08.432928	2020-10-11 23:38:31.028166
ONEQ	Fidelity Nasdaq Composite Tracking Stock	The investment seeks to provide investment returns that closely correspond to the price and yield performance of the Nasdaq Composite Index ®.\n The fund normally invests at least 80% of assets in common stocks included in the index. It uses statistical sampling techniques that take into account such factors as capitalization, industry exposures, dividend yield, price/earnings (P/E) ratio, price/book (P/B) ratio, and earnings growth to create a portfolio of securities listed in the Nasdaq Composite Index that have a similar investment profile to the entire index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:08.471411	2020-10-11 23:38:31.028166
OTTR	Otter Tail Corporation	Otter Tail Corp through its utility business units provides electricity and energy services. It operates in segments namely, Electric, Manufacturing, and Plastics.	{Utilities,"Utilities - Diversified","Utilities - Regulated"}	{MGEE,LNT,AEE,MDU,NWE,XEL,XLU}	Utilities - Regulated	Utilities	Nasdaq Global Select	\N	2020-10-11 23:12:08.608348	2020-10-11 23:38:31.028166
OXFD	Oxford Immunotec Global PLC	Oxford Immunotec Global PLC is a high-growth diagnostics company. It is engaged in developing and commercializing proprietary tests for the management of immune-regulated conditions. Its platform includes T-SPOT technology platform.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{QDEL,VIVO,PMD,XLV}	Medical Diagnostics & Research	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:08.643919	2020-10-11 23:38:31.028166
MLPC	C-Tracks ETNs based on Performance of the Miller/Howard MLP Fundamental Index	The investment seeks a return that based on the Performance of the Miller/Howard MLP Fundamental Index (the “index”), which the adviser refer to as the C-Tracks, are unsecured senior debt securities issued by Citigroup Inc.\nThe index is designed to measure the performance of 25 energy master limited partnerships (“MLPs”) selected quarterly by a methodology that is based upon certain quantitative fundamental factors of publicly traded MLPs, including distribution growth, estimated capital expenditures and distribution coverage.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:06.961434	2020-10-11 23:38:31.028166
MPV	Barings Participation Investors	Barings Participation Investors is a closed-end management investment company. Its investment objective is to maximize total return by providing a high level of current income, the potential for growth of income, and capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.147437	2020-10-11 23:38:31.028166
MX	MagnaChip Semiconductor Corporation	MagnaChip Semiconductor Corp designs and manufactures analog and mixed-signal semiconductor products. Its products consist of large display solutions, mobile display solutions, sensor solutions, LED solutions, mobile solutions and power conversions.	{Technology,Semiconductors}	{CRUS,ON,HIMX,AOSL,STM,LPL,XLK}	Semiconductors	Technology	New York Stock Exchange	\N	2020-10-11 23:12:07.406124	2020-10-11 23:38:31.028166
NEAR	iShares Short Maturity Bond	The investment seeks to maximize current income.\n The fund seeks to achieve its investment objective by investing, under normal circumstances, at least 80% of its net assets in a portfolio of U.S. dollar-denominated investment-grade fixed-income securities. Under normal circumstances, the effective duration of its portfolio is expected to be one year or less, as calculated by the management team. It is an actively managed exchange-traded fund ("ETF") that does not seek to replicate the performance of a specified index. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:07.638272	2020-10-11 23:38:31.028166
NGD	New Gold Inc.	New Gold Inc is an intermediate gold mining company. The firm is engaged in the acquisition, exploration, and development of natural resource properties.	{"Basic Materials",Gold,"Metals & Mining"}	{ABX,GG,CDE,HL,AGI}	Metals & Mining	Basic Materials	NYSE American	\N	2020-10-11 23:12:07.741839	2020-10-11 23:38:31.028166
NINE	Nine Energy Service Inc.	Nine Energy Service Inc is a North American onshore completion and production services provider that targets unconventional oil and gas resource development.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:07.812957	2020-10-11 23:38:31.028166
NKSH	National Bankshares Inc.	National Bankshares Inc is a bank holding company of National Bank of Blacksburg, offering a range of financial products and services, including deposit accounts, commercial, consumer and mortgage loans, credit cards and trust services.	{"Financial Services","Banks - Regional - US",Banks}	{AMNB,FBSS,OPOF,UBSH}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:07.825882	2020-10-11 23:38:31.028166
NM	Navios Maritime Holdings Inc.	Navios Maritime Holdings Inc is a shipping company which provides transportation, trading, storage, and logistics services for bulk cargoes. It operates a diverse portfolio of capesize, panamax, ultra-handymax and handy size bulk carriers.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{PBR,DSX,NMM,ESEA,SBLK,GNK,EGLE,DRYS}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:07.85088	2020-10-11 23:38:31.028166
NTX	Nuveen Texas Quality Municipal Income Fund	Nuveen Texas Quality Income Muni Fund is a closed-ended management investment company. It seeks to provide current income exempt from both regular federal and designated state income taxes by investing primarily in a portfolio of municipal obligations.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.040048	2020-10-11 23:38:31.028166
NYMTO	New York Mortgage Trust Inc. 7.875% Series C Cumulative Redeemable Preferred Stock	New York Mortgage Trust Inc is a real estate investment trust. The Company is engaged in acquiring, investing in, financing and managing mortgage-related assets and, to a lesser extent, financial assets.	{"Real Estate","REIT - Residential",REITs}	{}	REITs	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:08.219152	2020-10-11 23:38:31.028166
OCSI	Oaktree Strategic Income Corporation	Oaktree Strategic Income Corp is a specialty finance company. It focuses on providing customized capital solutions to middle-market companies, offering a range of first-lien financing solutions to companies across a wide variety of industries.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:08.27573	2020-10-11 23:38:31.028166
OEUR	O'Shares FTSE Europe Quality Dividend	The investment seeks to track the performance (before fees and expenses) of the FTSE Developed Europe Qual/Vol/Yield 5% Capped Factor Index (the "target index").\n The target index is designed to reflect the performance of publicly-listed large capitalization and mid-capitalization dividend-paying issuers in Europe exhibiting high quality, low volatility and high dividend yields, as determined by FTSE-Russell (the "index provider"). Under normal market conditions, the fund will invest at least 80% of its total assets in the components of the target index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.314348	2020-10-11 23:38:31.028166
OXBR	Oxbridge Re Holdings Limited	Oxbridge Re Holdings Ltd is engaged in providing reinsurance business solutions. The company through its subsidiary offers reinsurance to property and casualty insurers in the Gulf Coast region of the United States.	{"Financial Services","Insurance - Reinsurance","Insurance - Specialty"}	{RNR,BCRH}	Insurance - Specialty	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:08.636938	2020-10-11 23:38:31.028166
OXY	Occidental Petroleum Corporation	Occidental Petroleum Corp is in the business of exploration and production of oil and gas. It also processes, gathers, transports, stores, purchases and markets oil, condensate, natural gas liquids, natural gas.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{XOM,DVN,APC,APA,SWN,EOG,CVX,MRO,COP,HES}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:08.654021	2020-10-11 23:38:31.023428
OZM	Och-Ziff Capital Management Group LLC Class A Shares representing Class A limited liability company interests	Och-Ziff Capital Management Group LLC is an institutional alternative asset manager. It provides asset management services through its funds which aim to generate consistent, positive, risk-adjusted returns across market cycles with low volatility.	{"Financial Services","Asset Management"}	{BX,IVZ,APO,OAK,CG,ARES}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.658003	2020-10-11 23:38:31.028166
MLQD	iShares 5-10 Year Investment Grade Corporate Bond	The investment seeks to track the investment results of the Markit iBoxx® USD Liquid Investment Grade Intermediate Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents, including shares of money market funds. The index is designed to reflect the performance of U.S. dollar-denominated investment-grade corporate debt with remaining maturities between five and ten years. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:06.98043	2020-10-11 23:38:31.025871
MTDR	Matador Resources Company	Matador Resources Co is an independent energy company, engaged in the exploration, development, production and acquisition of oil and natural gas resources in the United States, with an emphasis on oil and natural gas shale and other unconventional plays.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{CHK,DVN,EOG,MRO,CXO,EPE,CRK,BCEI}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:07.295234	2020-10-11 23:38:31.025871
NBL	Noble Energy Inc.	Noble Energy Inc is an independent energy company. It is engaged in crude oil, natural gas and natural gas liquids (NGLs) exploration and production.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{CHK,NFX,PXD,DVN,APC,APA,COG,EOG,MRO,QEP}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:07.565913	2020-10-11 23:38:31.025871
NGHC	National General Holdings Corp	National General Holdings Corp is a specialty personal lines insurance holding company. The company manages its business through two segments, Property and Casualty segment and Accident and Health segment.	{"Financial Services","Insurance - Property & Casualty"}	{ERIE,STFC,AFSI,ALL,AFG,HMN,ORI,WTM}	Insurance - Property & Casualty	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:07.743183	2020-10-11 23:38:31.025871
NID	Nuveen Intermediate Duration Municipal Term Fund of Beneficial Interest	Nuveen Intermediate Dur Muni Term Fund is a diversified closed-end management investment company. It seeks to provide a high level of current income exempt from regular federal income tax with a secondary objective of seeking additional total return.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.802282	2020-10-11 23:38:31.025871
NLR	VanEck Vectors Uranium & Nuclear Energy	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the MVIS® Global Uranium & Nuclear Energy Index.\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The index includes equity securities and depositary receipts issued by companies involved in uranium and nuclear energy and companies. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:07.835788	2020-10-11 23:38:31.025871
NOBL	ProShares S&P 500 Dividend Aristocrats	The investment seeks investment results, before fees and expenses, that track the performance of the S&P 500® Dividend Aristocrats® Index (the "index").\n The fund will invest at least 80% of its total assets in component securities (i.e., securities of the index and comparable securities that have economic characteristics that are substantially identical to the economic characteristics of the securities of the index). The index contains a minimum of 40 stocks, which are equally weighted, and no single sector is allowed to comprise more than 30% of the index weight.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:07.895723	2020-10-11 23:38:31.025871
NVMI	Nova Measuring Instruments Ltd.	Nova Measuring Instruments Ltd is an Israel-based semiconductor equipment manufacturer. The company provides metrology solutions for advanced process control used in semiconductor manufacturing.	{Technology,"Semiconductor Equipment & Materials",Semiconductors}	{AMAT,ORBK,KLAC,NANO,CAMT,ASML,RTEC,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:08.121444	2020-10-11 23:38:31.025871
NXST	Nexstar Media Group Inc.	Nexstar Media Group Inc is a television broadcasting company serving the entertainment industry in the United States. The company is primarily involved in the operations of television stations.	{"Consumer Cyclical","Broadcasting - TV",Entertainment}	{SBGI,CMCSA,DIS,EVC,GTN,CBS,FOXA,SSP}	Entertainment	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:08.195966	2020-10-11 23:38:31.025871
ODP	Office Depot Inc.	Office Depot Inc is a supplier of office products and services. It offers products including office supplies, paper, ink, and tone, cleaning chemical and equipment's, monitors and equipment's, computers and tablets, office appliances, and school supplies.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{EBAY,STMP,PIR,WSM,BIG,ESND}	Retail - Apparel & Specialty	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:08.293862	2020-10-11 23:38:31.025871
OKDCC	Eaton Vance Oaktree Diversified Credit NextShares	The investment seeks total return.\n The fund invests in securities and other instruments to establish investment exposures to credit markets around the world. It invests at least 80% of its net assets (plus any borrowings for investment purposes) in credit-related investments (the “80% Policy”). For purposes of this 80% Policy, “credit-related investments” are fixed income, variable rate, and floating-rate debt investments as well as derivatives that provide exposure to such investments.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:08.383923	2020-10-11 23:38:31.025871
PACB	Pacific Biosciences of California Inc.	Pacific Biosciences of California Inc is a biotechnology company. It is engaged in design, development, and manufacture of sequencing system to help scientists resolve genetically complex problems.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{LMNX,ILMN,GE,TMO,PKI,FLDM,NSTG,XLV}	Medical Diagnostics & Research	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:08.678848	2020-10-11 23:38:31.028166
PANL	Pangaea Logistics Solutions Ltd.	Pangaea Logistics Solutions Ltd and its subsidiaries provide seaborne drybulk transportation services. It transports drybulk cargos including grains, coal, iron, ore, pig, iron, hot briquetted iron, bauxite, alumina etc.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{}	Transportation & Logistics	Industrials	NASDAQ Capital Market	\N	2020-10-11 23:12:08.704252	2020-10-11 23:38:31.025871
PFI	Invesco DWA Financial Momentum ETF	The investment seeks to track the investment results (before fees and expenses) of the Dorsey Wright® Financials Technical Leaders Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities that comprise the underlying index. The underlying index is composed of at least 30 securities of companies in the financials sector that have powerful relative strength or "momentum" characteristics.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.00997	2020-10-11 23:38:31.023007
MOV	Movado Group Inc.	Movado Group Inc designs, sources, markets and distributes fine watches. Its brands include Coach Watches, Concord, Ebel, ESQ Movado, Scuderia Ferrari Watches, HUGO BOSS Watches, Juicy Couture Watches, Lacoste Watches, Movado, and Tommy Hilfiger Watches.	{"Consumer Cyclical","Luxury Goods","Retail - Apparel & Specialty"}	{GES,KORS,FOSL,TPR}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:07.131246	2020-10-11 23:38:31.023007
MTNB	Matinas Biopharma Holdings Inc.	Matinas BioPharma Holdings Inc is a clinical-stage bio-pharmaceutical company focused on enabling delivery of life changing medicines using its LNC platform technology.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	NYSE American	\N	2020-10-11 23:12:07.317208	2020-10-11 23:38:31.023007
MTSI	MACOM Technology Solutions Holdings Inc.	MACOM Technology Solutions Holdings Inc offers high-performance analog RF, microwave, millimeterwave & photonic semiconductor products for diverse applications ranging from optical, satellite, wired & wireless networks to military and civil radar systems.	{Technology,Semiconductors}	{SWKS,IPHI,TXN,QRVO,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:07.333244	2020-10-11 23:38:31.023007
MUC	Blackrock MuniHoldings California Quality Fund Inc.	BlackRock MuniHoldings California Quality Fund Inc is a diversified, closed-end management investment firm. Its investment objective is to provide shareholders with current income exempt from U.S. federal income taxes and California personal income taxes.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.356307	2020-10-11 23:38:31.023007
MVIS	Microvision Inc.	Microvision Inc is engaged in developing its proprietary PicoP display technology. Its technology can be used to create high-resolution miniature laser display and imaging engines.	{Technology,"Electronic Components","Computer Hardware"}	{KOPN,MU,IIN,HIMX,EMAN,TXN,QRVO,XLK}	Computer Hardware	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:07.389744	2020-10-11 23:38:31.023007
MXDU	Nationwide Maximum Diversification U.S. Core Equity	The investment seeks to track the total return performance, before fees and expenses, of the TOBAM Maximum Diversification USA Index (the "index").\n The advisor attempts to invest all, or substantially all, of its assets in the component securities that make up the index. Normally, at least 80% of the fund's total assets will be invested in the component securities of the index. The index is a rules-based index that is designed to create a more diversified equity portfolio of the common and preferred stock of large and mid-capitalization U.S. companies relative to traditional market capitalization weighted benchmarks.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:07.406448	2020-10-11 23:38:31.023007
NGS	Natural Gas Services Group Inc.	Natural Gas Services Group Inc manufactures and sells natural gas compression equipment and industrial flare systems. It primarily serves the non-conventional natural gas production businesses and oil and gas plant and production facilities across the US.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{SPN,FTK,GHM}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:07.761384	2020-10-11 23:38:31.023007
NIB	iPath Bloomberg Cocoa Subindex Total Return SM IndexETN	The investment seeks to provide investors with exposure to the Jones-UBS Cocoa Subindex Total ReturnService Mark. \nThe Dow Jones-UBS Cocoa Subindex Total ReturnService Mark (the "index") reflects the returns that are potentially available through an unleveraged investment in the futures contracts on cocoa. The index currently consists of one futures contract on the commodity of cocoa which is included in the Dow Jones-UBS Commodity Index Total ReturnService Mark.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:07.792943	2020-10-11 23:38:31.023007
NMT	Nuveen Massachusetts Quality Municipal Income Fund	To provide current income exempt from regular federal and Massachusetts income tax and to enhance portfolio value relative to the municipal bond market by investing in tax-exempt municipal bonds that the Funds investment adviser believes are underrated or undervalued or that represent municipal market sectors that are undervalued. It will invest at least 80% of its assets in investment grade quality municipal bonds (BBB/Baa or better at the time of purchase by either Moody or Standard & Poor). It may invest up to 20% of its net assets in municipal bonds that are unrated but judged to be of comparable quality by the investment adviser.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.869296	2020-10-11 23:38:31.023007
OCN	Ocwen Financial Corporation	Ocwen Financial Corp is a financial services holding company. The Company through its subsidiaries provides residential and commercial mortgage loan servicing, special servicing and asset management services.	{"Financial Services","Specialty Finance",Banks}	{EFC}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.27334	2020-10-11 23:38:31.023007
OLD	The Long-Term Care ETF	The investment seeks investment results that correspond generally to the performance, before fees and expenses, of an index which is designed to track the performance of companies globally that are positioned to profit from providing long-term care to the aging population, including companies owning or operating senior living facilities, nursing services, specialty hospitals, and senior housing, biotech companies for age-related illnesses, and companies that sell products and services to such facilities.\n The fund invests at least 80% of its net assets in the stocks that comprise the Solactive Long-Term Care Index. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:08.397573	2020-10-11 23:38:31.023007
P	Pandora Media Inc.	Pandora Media Inc is a broadcasting company which offers music streaming and automated music recommendation services. The service is available in the United States, Australia, and New Zealand.	{"Consumer Cyclical","Broadcasting - Radio",Entertainment}	{AMZN,SALM,AAPL,NFLX,ETM,GOOGL,EMMS,UONEK}	Entertainment	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:08.666135	2020-10-11 23:38:31.023007
PAYC	Paycom Software Inc.	Paycom Software Inc is a provider of cloud-based HCM software solution delivered as Software-as-a-Services. It provides functionality and data analytics that businesses need to manage the complete employment life cycle from recruitment to retirement.	{Technology,"Software - Application","Application Software"}	{MANH,ULTI,PAYX,SSNC,CSOD,ORCL,PCTY,WDAY,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:08.735137	2020-10-11 23:38:31.031328
PGLC	Pershing Gold Corporation	Pershing Gold Corp is a gold and precious metals exploration company. It pursues exploration, development and mining opportunities primarily in Nevada. The Relief Canyon properties are located about 100 miles northeast of Reno, Nevada.	{"Basic Materials",Gold,"Metals & Mining"}	{MAG,MUX}	Metals & Mining	Basic Materials	NASDAQ Global Market	\N	2020-10-11 23:12:09.076649	2020-10-11 23:38:31.028166
NMM	Navios Maritime Partners LP Representing Limited Partner Interests	Navios Maritime Partners LP owns and operates dry cargo and container vessels. It provides seaborne transportation services to a range of dry cargo commodities chartering its vessels under medium to long-term charters.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{DSX,NM,SB,ESEA,SALT,SHIP,SBLK,DRYS}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:07.859735	2020-10-11 23:38:31.021426
NURE	NuShares Enhanced Yield US Aggregate Bond ETF	The investment seeks to track the investment results of the Dow Jones U.S. Select Short-Term REIT Index.\n The fund invests at least 80% of the sum of its net assets and the amount of any borrowings for investment purposes in REITs. The index is a subset of the Dow Jones U.S. Select REIT Index, which generally includes equity REITs traded on a national securities exchange in the U.S. that derive at least 75% of their total revenue from the ownership and operation of real estate assets and that have a minimum total market capitalization of $200 million at the time of their inclusion. It is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:08.079318	2020-10-11 23:38:31.021426
NZF	Nuveen Municipal Credit Income Fund	Nuveen Municipal Credit Income Fund is a diversified, closed-end management investment company. Its investment objective is to provide current income by investing in a portfolio of municipal obligations issued by state and local government authorities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:08.232266	2020-10-11 23:38:31.021426
OEC	Orion Engineered Carbons S.A	Orion Engineered Carbons SA is a producer of carbon black. The company's segment consists of Specialty Carbon Black and Rubber Carbon Black.	{"Basic Materials","Specialty Chemicals",Chemicals}	{MEOH,BCPC}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:08.296163	2020-10-11 23:38:31.021426
PBD	Invesco Global Clean Energy	The investment seeks to track the investment results (before fees and expenses) of the WilderHill New Energy Global Innovation Index (the "underlying index").\n The fund will invest at least 90% of its total assets in the securities of companies engaged in the business of the advancement of cleaner energy and conservation that comprise the underlying index, as well as American depositary receipts and global depositary receipts that are based on the securities in the underlying index. The adviser anticipates that the majority of its investments will be in the securities that comprise the underlying index rather than in ADRs and GDRs.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.756235	2020-10-11 23:38:31.02582
PBDM	Invesco PureBeta FTSE Developed ex-North America	The investment seeks to track the investment results (before fees and expenses) of the FTSE Developed ex North America Index.\n The fund generally will invest at least 90% of its total assets in common stocks that comprise the underlying index, as well as ADRs and GDRs that are based on the securities in the underlying index. The index is designed to measure the performance of the large- and mid-capitalization segments of equity markets of countries around the world that are classified as developed markets within the country classification definition of the index provider, excluding the United States and Canada. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:08.757344	2020-10-11 23:38:31.021426
PDN	Invesco FTSE RAFI Developed Markets ex-U.S. Small-Mid	The investment seeks to track the investment results (before fees and expenses) of the FTSE RAFI Developed ex U.S. Mid-Small 1500 Index (the "underlying index").\n The fund will invest at least 90% of its total assets in securities of small-and mid-capitalization companies that are classified as "developed" within the country classification definition of FTSE International Limited ("FTSE"), excluding the United States, and that comprise the underlying index, as well as American depositary receipts ("ADRs") and global depositary receipts ("GDRs") that are based on the securities in the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:08.891431	2020-10-11 23:38:31.02423
PFF	iShares U.S. Preferred Stock	The investment seeks to track the investment results of the S&P U.S. Preferred Stock Index, which measures the performance of a select group of preferred stocks listed on the New York Stock Exchange, NYSE Arca, NYSE Amex, NASDAQ Global Select Market, NASDAQ Select Market or NASDAQ Capital Market.\n The fund generally will invest at least 90% of its assets in the component securities of the index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the index, but which the advisor believes will help the fund track the index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:08.99034	2020-10-11 23:38:31.021426
PFIG	Invesco Fundamental Investment Grade Corporate Bond	The investment seeks to track the investment results (before fees and expenses) of the RAFI® Bonds U.S. Investment Grade 1-10 Index (the "underlying index").\n The fund generally will invest at least 80% of its total assets in the component securities that comprise the underlying index. The underlying index is comprised of U.S. dollar-denominated bonds which are SEC-registered securities or Rule 144A securities with registration rights (issued after July 31, 2013) and whose issuers are public companies listed on a major U.S. stock exchange.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.021029	2020-10-11 23:38:31.02423
PFPT	Proofpoint Inc.	Proofpoint Inc is a provider of web-based protection applications in the United States. Its data protection solutions address web-based security threats posed on platforms like e-mail, instant messaging and social media.	{Technology,"Software - Application","Application Software"}	{INTC,MSFT,CYRN,SYMC,ZIXI,IBM,VMW,QLYS,FEYE,HPQ,XLK}	Application Software	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:09.050736	2020-10-11 23:38:31.021426
PGF	Invesco Financial Preferred	The investment seeks to track the investment results (before fees and expenses) of the Wells Fargo® Hybrid and Preferred Securities Financial Index.\n The fund generally will invest at least 90% of its total assets in preferred securities of financial institutions that comprise the underlying index. The underlying index is a market capitalization weighted index designed to track the performance of preferred securities and securities that the index provider believes are functionally equivalent to preferred securities, including, but not limited to, depositary preferred securities, perpetual subordinated debt and certain capital securities. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.062441	2020-10-11 23:38:31.02423
PGR	Progressive Corporation (The)	Progressive Corp is an insurance holding company which provides personal and commercial auto insurance, residential property insurance, and other specialty property-casualty insurance and related services.	{"Financial Services","Insurance - Property & Casualty"}	{SIGI,SAFT,HALL,ALL,HMN,MCY,TRV}	Insurance - Property & Casualty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.080515	2020-10-11 23:38:31.031328
NNC	Nuveen North Carolina Quality Municipal Income Fund	To provide current income exempt from regular federal and North Carolina income tax and to enhance portfolio value relative to the municipal bond market by investing in tax-exempt municipal bonds that the Funds investment adviser believes are underrated or undervalued or that represent municipal market sectors that are undervalued. It will invest at least 80% of its assets in investment grade quality municipal bonds (BBB/Baa or better at the time of purchase by either Moody or Standard & Poor). It may invest up to 20% of its net assets in municipal bonds that are unrated but judged to be of comparable quality by the investment adviser.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:07.87879	2020-10-11 23:38:31.024353
OC	Owens Corning Inc New	Owens-Corning Inc manufactures and sells glass fiber products in the US. The company mainly offers Composites, Insulation, and Roofing solutions. Through these businesses it provides glass fiber, insulation service, shingles and roofing accessories.	{"Basic Materials","Building Materials"}	{PATK,GLW,LPX,MAS,PPG,ASPN,DWDP}	Building Materials	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:08.264882	2020-10-11 23:38:31.024353
OGE	OGE Energy Corp	OGE Energy Corp, together with its subsidiaries, operates as an energy and energy services provider. It offers physical delivery and related services for electricity and natural gas in the south central United States.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{ETR,EIX,EE,PCG,AEP,D,SCG,CNP,NRG,DUK,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:08.334416	2020-10-11 23:38:31.024353
OXLC	Oxford Lane Capital Corp.	The Fund seeks to maximize its portfolios total return. The Fund seeks to achieves it investment objective by investing primarily in senior, secured loans made to companeis whose debt is unrated or rated below investment grade (Senior Loans), with an emphasis on current income. The Fund intends to initially implement its investment objective by purchasing in the secondary market the equity and junior debt tranches of collateralized loan obligation (CLO) vehicles. Structurally, CLO vehicles are entities that were formed to originate and manage a portfolio of loans. The loans within the CLO vehicle are limited to loans which meet established credit criteria and are subject to concentration limitations in order to limit a CLO vehicles exposure to a single credit. A CLO vehicle is formed by raising various classes or tranches of debt (with the most senior tranches being rated AAA to the most junior tranches typically being rated BB) and equity. The CLO vehicles which the company intends to focus on will be collateralized primarily by Senior Loans, and will generally have very little or no exposure to real estate, mortgage loans or to pools of consumer-based debt, such as credit card receivables or auto loans. It may also invest, on an opportunistic basis, in other corporate credits of a variety of types. It also expects that each of its investments will range in size from USD1 million to USD8 million, although the investment size may vary consistent with the size of its overall portfolio.	{"Financial Services","Asset Management"}	{}			Nasdaq Global Select	\N	2020-10-11 23:12:08.645315	2020-10-11 23:38:31.024353
PAGG	PowerShares Global Agriculture Portfolio	The investment seeks to track the investment results (before fees and expenses) of the NASDAQ OMX Global Agriculture IndexSM (the "index").\n The fund will invest at least 90% of its total assets in the securities of companies engaged in agriculture and farming-related activities that comprise the index, as well as ADRs and GDRs that are based on the securities in the index. The index provider calculates and maintains the index, which is designed to measure the performance of the largest, most liquid, globally traded securities of companies engaged in agriculture and farming-related activities. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:08.686376	2020-10-11 23:38:31.024353
PBF	PBF Energy Inc. Class A	PBF Energy Inc is engaged in the oil and gas sector. It is a petroleum refiner and supplier of unbranded transportation fuels.	{Energy,"Oil & Gas Refining & Marketing","Oil & Gas - Refining & Marketing"}	{CLMT,VLO,MPC,HFC,PSX,DK}	Oil & Gas - Refining & Marketing	Energy	New York Stock Exchange	\N	2020-10-11 23:12:08.76909	2020-10-11 23:38:31.024353
PBEE	Invesco PureBeta FTSE Emerging Markets	The investment seeks to track the investment results (before fees and expenses) of the FTSE Emerging Index.\n The fund generally will invest at least 90% of its total assets common stocks that comprise the underlying index, as well as American depositary receipts ("ADRs") and global depositary receipts ("GDRs") that are based on the securities in the underlying index. The index is designed to measure the performance of the large- and mid-capitalization segments of equity markets of countries around the world that are classified as emerging markets within the country classification definition of the index provider. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:08.769288	2020-10-11 23:38:31.025871
PETZ	TDH Holdings Inc.	TDH Holdings Inc is engaged in the business of manufacturing and selling of pet food. Its products include dog food made of meat, pet food, pet biscuits, dental dog food, dog chews, pet food made of fish, vegetarian pet food, canned pet food and cat food.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{}	Consumer Packaged Goods	Consumer Defensive	NASDAQ Capital Market	\N	2020-10-11 23:12:08.964411	2020-10-11 23:38:31.025871
PEY	Invesco High Yield Equity Dividend Achievers ETF	The investment seeks to track the investment results (before fees and expenses) of the NASDAQ US Dividend AchieversTM 50 Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in common stocks of companies that comprise the underlying index. Strictly in accordance with its guidelines and mandated procedures, Nasdaq, Inc. includes common stocks in the underlying index that have a consistent record of dividend increases, principally on the basis of dividend yield and consistent growth in dividends.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:08.975655	2020-10-11 23:38:31.023428
PGHY	Invesco Global Short Term High Yield Bond	The investment seeks investment results that correspond to the DB Global Short Maturity High Yield Bond Index.\n The fund will invest at least 80% of its total assets in U.S. and foreign short-term, non-investment grade bonds that comprise the index. The index provider selects such bonds issued by corporations, as well as sovereign, sub-sovereign or quasi-government entities, from a universe of eligible securities that: are denominated in USD; are rated below investment grade; have not been marked as defaulted by any rating agency; have three years or less to maturity; have a minimum amount outstanding of $250 million; and have a fixed coupon.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.070351	2020-10-11 23:38:31.025871
HAUD	iShares Currency Hedged MSCI Australia	The investment seeks to track the investment results of the MSCI Australia 100% Hedged to USD Index, which consists of stocks traded primarily on the Australian Stock Exchange with the currency risk inherent in the securities included in the underlying index hedged to the U.S. dollar on a monthly basis.\n The fund generally will invest at least 90% of its assets in the component securities (including indirect investments through the underlying fund) and other instruments of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:03.962764	2020-10-11 23:38:31.021426
HIX	Western Asset High Income Fund II Inc.	Western Asset High Income Fund II Inc is a diversified closed-end management investment company. Its objective is to maximize current income by investing at least 80% of its net assets and borrowings for investment purposes in high-yield debt securities.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:04.162319	2020-10-11 23:38:31.021426
HP	Helmerich & Payne Inc.	Helmerich & Payne Inc is engaged in contract drilling of oil and gas wells for oil and gas exploration and production companies. Its business comprises of three reportable segments, U.S. Land, Offshore and International Land.	{Energy,"Oil & Gas Drilling","Oil & Gas - Drilling"}	{PDS,PTEN,DO,UNT,NBR,PES}	Oil & Gas - Drilling	Energy	New York Stock Exchange	\N	2020-10-11 23:12:04.269544	2020-10-11 23:38:31.021426
IBDP	iShares iBonds Dec 2024 Term Corporate	The investment seeks to track the investment results of the Bloomberg Barclays December 2024 Maturity Corporate Index which composed of U.S. dollar-denominated, investment-grade corporate bonds maturing in 2024.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index. The underlying index is composed of U.S. dollar-denominated, taxable, investment-grade corporate bonds scheduled to mature after December 31, 2023 and before December 16, 2024.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.504607	2020-10-11 23:38:31.021426
IBP	Installed Building Products Inc.	Installed Building Products Inc is engaged in the installation of insulation, garage doors, rain gutters, shower doors, closet shelving and mirrors, and other products in the continental United States.	{"Basic Materials","Building Materials"}	{APOG,OC,CBPX,BMCH}	Building Materials	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:04.540838	2020-10-11 23:38:31.021426
IMDZ	Immune Design Corp.	Immune Design Corp is a clinical-stage immunotherapy company with next-generation in vivo approaches designed to enable the body's immune system to fight disease.	{Healthcare,Biotechnology}	{XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:04.753822	2020-10-11 23:38:31.021426
INFR	Legg Mason Global Infrastructure ETF	The investment seeks to track the investment results of the RARE Global Infrastructure Index (the "underlying index").\n The fund will invest at least 80% of its net assets, plus borrowings for investment purposes, if any, in securities that compose its underlying index. The underlying index is composed of equity securities in developed and developing markets that are included in the MSCI ACWI All Cap Index. Companies in the MSCI ACWI All Cap Index are screened to include only companies within the Global Industry Classification Standard (GICS) sub-industries and are classified into infrastructure sectors.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:04.82648	2020-10-11 23:38:31.021426
IPAY	ETFMG Prime Mobile Payments	The investment seeks to provide investment results that correspond generally to the Prime Mobile Payments Index.\n The fund invests at least 80% of its total assets in the component securities of the index and in ADRs and GDRs based on the component securities in the index. The index tracks the performance of the exchange-listed equity securities of companies across the globe that (i) engage in providing payment processing services or applications, (ii) provide payment solutions, (iii) build or provide payment industry architecture, infrastructure or software, or (iv) provide services as a credit card network. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:04.929344	2020-10-11 23:38:31.021426
IPHS	Innophos Holdings Inc.	Innophos Holdings Inc is producer of specialty grade phosphate products for the Food, Pharmaceutical and Industrial market segments. Its products act as flavor enhancers in beverages, leavening agents in baked goods and cleaning agents in toothpaste.	{"Basic Materials","Specialty Chemicals",Chemicals}	{}	Chemicals	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:12:04.947776	2020-10-11 23:38:31.021426
IQV	IQVIA Holdings Inc.	IQVIA Holdings Inc is an information and technology-enabled healthcare service provider to the pharmaceutical services, biotechnology, and healthcare industries. The Commercial Solutions and Research and Development Solutions generate maximum revenue.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{XLV}	Medical Diagnostics & Research	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:04.988441	2020-10-11 23:38:31.021426
IVW	iShares S&P 500 Growth	The investment seeks to track the investment results of the S&P 500 Growth IndexTM, which measures the performance of the large-capitalization growth sector of the U.S. equity market.\n The fund generally invests at least 90% of its assets in securities of the underlying index and in depositary receipts representing securities of the underlying index. It may invest the remainder of its assets in certain futures, options and swap contracts, cash and cash equivalents, as well as in securities not included in the underlying index, but which BFA believes will help the fund track the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.150067	2020-10-11 23:38:31.021426
JDST	Direxion Daily Junior Gold Miners Index Bear 3X Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the inverse (or opposite) of the daily performance of the MVIS Global Junior Gold Miners Index.\n The fund invests in swap agreements, futures contracts, short positions or other financial instruments that, in combination, provide inverse (opposite) or short leveraged exposure to the index equal to at least 80% of the fund's net assets (plus borrowing for investment purposes). The index tracks the performance of foreign and domestic micro-, small- and mid-capitalization companies. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.314505	2020-10-11 23:38:31.021426
JHDG	WisdomTree Japan Hedged Quality Dividend Growth Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree Japan Hedged Quality Dividend Growth Index.\n The fund invests at least 80% of the fund's total assets in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is designed to provide exposure to dividend-paying common stocks of Japanese companies with growth characteristics while at the same time neutralizing exposure to fluctuations of the value of the Japanese yen relative to the U.S. dollar. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:05.340537	2020-10-11 23:38:31.021426
PGX	Invesco Preferred	The investment seeks to track the investment results (before fees and expenses) of the ICE BofAML Core Plus Fixed Rate Preferred Securities Index (the "underlying index").\n The fund generally will invest at least 80% of its total assets in fixed rate U.S. dollar-denominated preferred securities that comprise the underlying index. The underlying index is a market capitalization-weighted index designed to reflect the total return performance of the fixed rate U.S. dollar-denominated preferred securities market. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.094663	2020-10-11 23:38:31.024353
PHO	Invesco Water Resources ETF	The investment seeks to track the investment results (before fees and expenses) of the NASDAQ OMX US Water IndexSM (the "underlying index").\n The fund generally will invest at least 90% of its total assets in securities of companies in the water industry that comprise the underlying index. The underlying index seeks to track the performance of U.S. exchange-listed companies that create products designed to conserve and purify water for homes, businesses and industries. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.125824	2020-10-11 23:38:31.024353
PIH	1347 Property Insurance Holdings Inc.	1347 Property Insurance Holdings Inc is a property and casualty insurance holding company. The company through its wholly-owned subsidiary provides property and casualty insurance to individuals in Louisiana.	{"Financial Services","Insurance - Property & Casualty"}	{ERIE,STFC,TPRE}	Insurance - Property & Casualty	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:09.15244	2020-10-11 23:38:31.024353
PSCI	PowerShares S&P SmallCap Industrials Portfolio	The investment seeks to track the investment results (before fees and expenses) of the S&P SmallCap 600® Capped Industrials Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in common stocks of small capitalization U.S. industrial companies that comprise the underlying index. These companies are principally engaged in the business of providing industrial products and services, including engineering, heavy machinery, construction, electrical equipment, aerospace and defense and general manufacturing.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.599124	2020-10-11 23:38:31.024353
QEMM	SPDR MSCI Emerging Markets StrategicFactors	The investment seeks to track the performance of the MSCI Emerging Markets (EM) Factor Mix A-Series Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index and in depositary receipts based on securities comprising the index. The index captures large- and mid-cap representation across 23 emerging markets countries and aims to represent the performance of value, low volatility, and quality factor strategies. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.899487	2020-10-11 23:38:31.024353
QTS	QTS Realty Trust Inc. Class A	QTS Realty Trust Inc is a real estate investment trust in the United States. Its properties mainly include data centers which are mainly located in the United States and in other locations such as Canada, Europe, and Asia.	{"Real Estate","REIT - Industrial",REITs}	{DRE,VTR,AHT,DLR,DRH,PEB,TIPT}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:09.968425	2020-10-11 23:38:31.024353
RDUS	Radius Health Inc.	Radius Health Inc is a science-driven biopharmaceutical company engaged in developing new therapeutics for patients with osteoporosis and other serious endocrine-mediated diseases.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{AMGN,BSTC,NVS,LLY,CRL,MRK,ALDR,HZNP,XLV}	Drug Manufacturers	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:10.14193	2020-10-11 23:38:31.024353
RENN	Renren Inc. American Depositary Shares each representing fifteen Class A	Renren Inc is engaged in the operation of social networking internet platform, as well as provision of online advertising services and internet value-added services ("IVAS") as well as the operation of financial services platforms.	{Technology,"Internet Content & Information","Online Media"}	{SINA,JOBS,BIDU,FB,YY,WB,GOOGL,XLK}	Online Media	Technology	New York Stock Exchange	\N	2020-10-11 23:12:10.217568	2020-10-11 23:38:31.024353
RGEN	Repligen Corporation	Repligen Corp is a bioprocessing company developing, producing and commercializing biologic drugs. The company offers its products to life science companies, global biopharmaceutical companies, and contract manufacturers worldwide.	{Healthcare,Biotechnology}	{NVAX,NBIX,BMRN,SGMO,BMY,GE,PFE,TMO,CRL,CLBS,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:10.287914	2020-10-11 23:38:31.024353
RIGS	RiverFront Strategic Income Fund	The investment seeks total return, with an emphasis on income as the source of that total return.\n The fund seeks to achieve its investment objective by investing in a global portfolio of fixed income securities of various maturities, ratings and currency denominations. The adviser intends to utilize various investment strategies in a broad array of fixed income sectors. The fund may purchase fixed income securities issued by U.S. or foreign corporations or financial institutions, including debt securities of all types and maturities, convertible securities and preferred stocks. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.341993	2020-10-11 23:38:31.024353
SCHX	Schwab U.S. Large-Cap	The investment seeks to track as closely as possible, before fees and expenses, the total return of the Dow Jones U.S. Large-Cap Total Stock Market Index.\n To pursue its goal, the fund generally invests in stocks that are included in the Dow Jones U.S. Large-Cap Total Stock Market Index. The index includes the large-cap portion of the Dow Jones U.S. Total Stock Market Index actually available to investors in the marketplace. The Dow Jones U.S. Large-Cap Total Stock Market Index includes the components ranked 1-750 by full market capitalization. The index is a float-adjusted market capitalization weighted index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.88342	2020-10-11 23:38:31.024353
SECT	Northern Lights Fund Trust IV Main Sector Rotation	The investment seeks to outperform the S&P 500 in rising markets while limiting losses during periods of decline.\n The fund utilizes a "fund of funds" structure to invest in sector based equity ETFs. It seeks to achieve its objective through dynamic sector rotation. The Adviser focuses its research primarily on sector selection by carefully reviewing the sector, industry, and sub-industries in the fund's portfolio. The Adviser chooses sectors it believes are undervalued and poised to respond favorably to financial market catalysts. The fund will sell a security when it achieves its target price and is, in the opinion of the Adviser, no longer undervalued.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:10.982788	2020-10-11 23:38:31.024353
PHII	PHI Inc.	PHI Inc provides helicopter transportation services to, from, and among offshore facilities for customers engaged in the oil and gas exploration, development, and production industry.	{Industrials,"Airports & Air Services","Transportation & Logistics"}	{CKH,SPN,BRS,ERA,TDW}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:09.109359	2020-10-11 23:38:31.028166
PICO	PICO Holdings Inc.	PICO Holdings Inc primarily invests in the water storage business in the United States. Its supplementary occupations include real estate activities and other investments.	{"Real Estate","Real Estate - General","Real Estate Services"}	{CDZI,CWT,AWR,AWK,SJW,IEP}	Real Estate Services	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:09.136362	2020-10-11 23:38:31.028166
PILL	Direxion Daily Pharmaceutical & Medical Bull 3X Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the daily performance of the Dynamic Pharmaceutical Intellidex Index (the "index").\n The fund invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, exchange-traded funds ("ETFs") that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index consists of common shares of companies that are principally engaged in research, development, manufacture, sale or distribution of pharmaceuticals and drugs of all types. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.152279	2020-10-11 23:38:31.02423
PNFP	Pinnacle Financial Partners Inc.	Pinnacle Financial Partners Inc is a financial service provider. The company operates through its wholly owned subsidiary, Pinnacle Bank. It offers banking, investment, trust, mortgage and insurance products, and services.	{"Financial Services","Banks - Regional - US",Banks}	{TRMK,RNST,CFR,WFC,STI,FHN,RF,BXS}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:09.305414	2020-10-11 23:38:31.02423
PQG	PQ Group Holdings Inc.	PQ Holdings Inc manufactures integrated manufacturer of specialty inorganic performance chemicals, specialty catalysts, and specialty glass materials. Its segments include performance chemicals, catalysts, and specialty glass materials.	{"Basic Materials","Specialty Chemicals",Chemicals}	{INDU,NL,NGVT}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:09.495355	2020-10-11 23:38:31.028166
PRFZ	PowerShares FTSE RAFI US 1500 Small-Mid Portfolio	The investment seeks to track the investment results (before fees and expenses) of the FTSE RAFI™ US Mid Small 1500 Index.\n The fund generally will invest at least 90% of its total assets in common stocks that comprise the underlying index. The underlying index is composed of 1,500 common stocks that the index provider strictly in accordance with its guidelines and mandated procedures, include to track the performance of small- and medium-sized U.S. companies based on the following four fundamental measures of firm size: book value, cash flow, sales and dividends.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.512315	2020-10-11 23:38:31.02423
PRF	Invesco FTSE RAFI US 1000	The investment seeks to track the investment results (before fees and expenses) of the FTSE RAFI™ US 1000 Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in common stocks that comprise the underlying index. The underlying index is composed of 1,000 common stocks that FTSE International Limited and Research Affiliates LLC strictly in accordance with their guidelines and mandated procedures, include to track the performance of the largest U.S. companies based on the following four fundamental measures: book value, cash flow, sales and dividends.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.511444	2020-10-11 23:38:31.028166
PSCU	PowerShares S&P SmallCap Utilities Portfolio	The investment seeks to track the investment results (before fees and expenses) of the S&P SmallCap 600® Capped Utilities & Communication Services Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in common stocks of small capitalization U.S. utility companies that comprise the underlying index. These companies are principally engaged in providing either energy, water, electric or natural gas utilities. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.6092	2020-10-11 23:38:31.02423
PYN	PIMCO New York Municipal Income Fund III of Beneficial Interest	PIMCO New York Municipal Income Fund III is a United States based closed-end management investment company. Its investment objective is to seek to provide current income exempt from federal, New York State and New York City income tax.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.82859	2020-10-11 23:38:31.02423
PZA	Invesco National AMT-Free Municipal Bond ETFo	The investment seeks to track the investment results (before fees and expenses) of the ICE BofAML National Long-Term Core Plus Municipal Securities Index (the "underlying index").\n The fund generally will invest at least 80% of its total assets in municipal securities that comprise the underlying index and that also are exempt from the federal alternative minimum tax ("AMT"). The underlying index is composed of U.S. dollar-denominated, investment grade, tax-exempt debt publicly issued by U.S. states and territories or their political subdivisions, in the U.S. domestic market with a term of at least 15 years remaining to final maturity.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.833963	2020-10-11 23:38:31.028166
PZI	Invesco Zacks Micro Cap	The investment seeks to track the investment results (before fees and expenses) of the Zacks Micro Cap IndexTM.\n The fund generally will invest at least 90% of its total assets in common stocks of micro capitalization companies that comprise the underlying index. The underlying index was composed of 400 U.S. stocks that, strictly in accordance with its guidelines and mandated procedures, Zacks includes from a universe of 7,000 domestic companies.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.844183	2020-10-11 23:38:31.02423
QEFA	SPDR MSCI EAFE StrategicFactors	The investment seeks to track the performance of the MSCI EAFE Factor Mix A-Series Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index and in depositary receipts based on securities comprising the index. The index captures large- and mid-cap representation across 21 developed market Europe, Australasia, and Far East countries and aims to represent the performance of value, low volatility, and quality factor strategies. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.897892	2020-10-11 23:38:31.02423
QNST	QuinStreet Inc.	QuinStreet Inc is an online marketing and media company that delivers qualified clicks and inquiries at low cost with great scalability. The company's operations and revenue are in North America, but has emerging businesses in Brazil and India.	{Technology,"Internet Content & Information","Online Media"}	{MSFT,MDCA,TTGT,INUV,GOOGL,GAIA,XLK}	Online Media	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:09.926531	2020-10-11 23:38:31.02423
PHK	Pimco High Income Fund	PIMCO High Income Fund is a United States based closed-end management investment company. The company's primary investment objective is to seek high current income. Its secondary objective is to seek capital appreciation.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.114933	2020-10-11 23:38:31.028166
PICK	iShares MSCI Global Select Metals & Mining Producers Fund	The investment seeks to track the investment results of the MSCI ACWI Select Metals & Mining Producers ex Gold and Silver Investable Market Index.\n The fund generally will invest at least 90% of its assets in the component securities of the index and in investments that have economic characteristics that are substantially identical to the component securities of the index. The index composed of global equities of companies primarily engaged in mining, extraction or production of diversified metals, excluding gold and silver. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:09.133574	2020-10-11 23:38:31.028166
PREF	Principal Spectrum Preferred Securities Active	The investment seeks to provide current income.\n Under normal circumstances, the fund invests at least 80% of its net assets, plus any borrowings for investment purposes, in preferred securities at the time of purchase. Examples of preferred securities include preferred stock, certain depositary receipts, and various types of junior subordinated debt (such debt generally includes the contractual ability to defer payment of interest without accelerating an immediate default event). It concentrates its investments (invests more than 25% of its net assets) in securities in the financial services (i.e., banking, insurance and commercial finance) industry.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:09.500646	2020-10-11 23:38:31.028166
QED	IQ Hedge Event-Driven Tracker	The investment seeks investment results that correspond (before fees and expenses) generally to the price and yield performance of its underlying index, the IQ Hedge Event-Driven Index (the "underlying index").\n The fund is a "fund of funds" which means it invests, under normal circumstances, at least 80% of its net assets, plus the amount of any borrowings for investment purposes, in the investments included in its underlying index, which includes underlying funds. The underlying index consists of a number of components ("underlying index components") selected in accordance with IndexIQ's rules-based methodology of such underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.89411	2020-10-11 23:38:31.028166
QQQ	PowerShares QQQ Trust Ser 1	The investment seeks investment results that generally correspond to the price and yield performance of the index.\n To maintain the correspondence between the composition and weights of the securities in the trust (the "securities") and the stocks in the NASDAQ-100 Index®, the adviser adjusts the securities from time to time to conform to periodic changes in the identity and/or relative weights of index securities. The composition and weighting of the securities portion of a portfolio deposit are also adjusted to conform to changes in the index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.929681	2020-10-11 23:38:31.028166
QMOM	Alpha Architect U.S. Quantitative Momentum	The investment seeks to track the total return performance, before fees and expenses, of the Alpha Architect Quantitative Momentum Index.\n The fund will normally invest at least 80% of its total assets in the component securities of the index. The fund may also invest up to 20% of its assets in cash and cash equivalents, other investment companies, as well as securities and other instruments not included in the index but which the Adviser believes will help the fund track the index. The index uses a 5-step, quantitative, rules-based methodology to identify a portfolio of approximately 40-50 U.S. equity securities with positive momentum. It is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:09.926402	2020-10-11 23:38:31.028166
QUIK	QuickLogic Corporation	QuickLogic Corp is engaged in developing and marketing semiconductor and software algorithm solutions for mobile and portable electronics. Its solutions fall into three categories: Sensor Processing, Display and Visual Enhancement, and Smart connectivity.	{Technology,Semiconductors}	{INTC,LSCC,SMTC,XLNX,IDCC,TXN,XLK}	Semiconductors	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:09.971562	2020-10-11 23:38:31.028166
QUAD	Quad Graphics Inc Class A	Quad/Graphics Inc is a US based marketing services provider. It helps brand owners market their products, services and content by using its strong print foundation in combination with other media channels.	{Industrials,"Business Services"}	{INWK,RRD}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:09.968477	2020-10-11 23:38:31.028166
QWLD	SPDR MSCI World StrategicFactors	The investment seeks to track the performance of the MSCI World Factor Mix A-Series Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index and in depositary receipts based on securities comprising the index. The index captures large-and mid-cap representation across 23 developed countries and aims to represent the performance of value, low volatility, and quality factor strategies. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.987204	2020-10-11 23:38:31.028166
RDNT	RadNet Inc.	RadNet Inc is a provider of freestanding, fixed-site outpatient diagnostic imaging services in the United States. Its services include magnetic resonance imaging, computed tomography, positron emission tomography, nuclear medicine, and among others.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{PMD,DRAD,DGX,ENZ,LH,CVR,INS,XLV}	Medical Diagnostics & Research	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:10.137799	2020-10-11 23:38:31.028166
RETA	Reata Pharmaceuticals Inc.	Reata Pharmaceuticals Inc is a clinical stage biopharmaceutical company engaged in identifying, developing, and commercializing product candidates to address rare and life-threatening diseases by targeting molecular pathways.	{Healthcare,Biotechnology}	{OMER,XLV}	Biotechnology	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:10.227298	2020-10-11 23:38:31.028166
RFEM	First Trust RiverFront Dynamic Emerging Markets ETF	The investment seeks to provide capital appreciation.\n The fund invests at least 80% of its net assets (including investment borrowings) in a portfolio of equity securities of emerging market companies, including through investments in common stock, depositary receipts, and common and preferred shares of real estate investment trusts, and forward foreign currency exchange contracts and currency spot transactions used to hedge the fund's exposure to the currencies in which the equity securities of such emerging market companies are denominated. The fund may invest in small, mid and large capitalization companies. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:10.260572	2020-10-11 23:38:31.028166
PHM	PulteGroup Inc.	PulteGroup Inc is engaged in the homebuilding business. Its business includes the acquisition and development of land for residential purposes within the U.S. It is also engaged in mortgage banking operations.	{"Consumer Cyclical","Residential Construction","Homebuilding & Construction"}	{DHI,MDC,TOL,KBH,LEN.B}	Homebuilding & Construction	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:09.124242	2020-10-11 23:38:31.021426
POWL	Powell Industries Inc.	Powell Industries Inc develops industrial goods. Its custom engineered equipment and systems find use in control of electrical energy and other critical processes.	{Industrials,"Diversified Industrials","Industrial Products"}	{LFUS,AETI,GE,AZZ,ABB,AMSC}	Industrial Products	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:09.457747	2020-10-11 23:38:31.021426
PRKR	ParkerVision Inc.	Parkervision Inc is in the business of innovating fundamental wireless technologies. It designs, develops & markets its proprietary radio frequency technologies & products for use in semiconductor circuits for mobile & other wireless applications.	{Technology,Semiconductors}	{INTC,QCOM,NVDA,SWKS,MTSI,QRVO,XLK}	Semiconductors	Technology	OTCQB - U.S. Registered	\N	2020-10-11 23:12:09.531332	2020-10-11 23:38:31.021426
PROV	Provident Financial Holdings Inc.	Provident Financial Holdings Inc is engaged in providing community banking & mortgage banking services. It accepts deposits, offers banking services, and provides multi-family, commercial real estate, commercial business, consumer & other mortgage loans.	{"Financial Services","Savings & Cooperative Banks",Banks}	{WAFD,RVSB,TSBK,BYFC,FFNW,TBNK,ANCB,FSBW,BANC}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:09.544441	2020-10-11 23:38:31.021426
QEP	QEP Resources Inc.	QEP Resources Inc is a crude oil and natural gas exploration and production company focused in two regions of the United States: the Northern Region (North Dakota, Wyoming and Utah) and the Southern Region (Texas and Louisiana).	{Energy,"Oil & Gas Integrated","Oil & Gas - Integrated"}	{NFX,PXD,DNR,COG,SWN,MRO,NBL,EQT,SM,WPX}	Oil & Gas - Integrated	Energy	New York Stock Exchange	\N	2020-10-11 23:12:09.90314	2020-10-11 23:38:31.021426
QTEC	First Trust NASDAQ-100 Technology Sector Index Fd	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the NASDAQ-100 Technology Sector IndexSM.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The index is an equal-weighted index based on the securities of the NASDAQ-100 Index® that are classified as "technology" according to the Industry Classification Benchmark classification system.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.948609	2020-10-11 23:38:31.02423
QUMU	Qumu Corporation	Qumu Corp is an enterprise video content management software company. It is engaged in providing tools businesses needs to create, manage, secure, deliver and measure their videos.	{Technology,"Software - Application","Application Software"}	{CSCO,TACT,ALOT,IMMR,SOFO,BCOV,WDC,XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:09.970667	2020-10-11 23:38:31.021426
QURE	uniQure N.V.	uniQure NV is engaged in the field of gene therapy platform across three therapeutic focus areas: Liver/Metabolic Disease, Central Nervous System (CNS) Disease and Cardiovascular Disease.	{Healthcare,Biotechnology}	{AMGN,BMRN,SGMO,SNY,BLUE,IONS,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:09.980361	2020-10-11 23:38:31.02423
QXGG	QuantX Risk Managed Growth ETF	The investment seeks to replicate investment results that correspond to the performance of the QuantX Risk Managed Growth Index.\n The fund generally will invest at least 80% of its total assets in the component securities of the index. The index methodology selects ETFs that invest in foreign and domestic equity securities of any market capitalization pursuant to a proprietary selection methodology that is designed to increase exposure to the best performing markets, while decreasing exposure to the worst performing markets while also managing overall portfolio risk by allocating to ETFs that invest in cash and cash equivalents or fixed income instruments.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:09.988835	2020-10-11 23:38:31.021426
RELV	Reliv' International Inc.	Reliv' International Inc is a developer, manufacturer and marketer of a proprietary line of nutritional supplements addressing basic nutrition, specific wellness needs, weight management and sports nutrition.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{NATR,PRPH,NUS,USNA,MTEX,NHTC}	Consumer Packaged Goods	Consumer Defensive	NASDAQ Capital Market	\N	2020-10-11 23:12:10.196763	2020-10-11 23:38:31.021426
RETO	ReTo Eco-Solutions Inc.	ReTo Eco-Solutions Inc is engaged in manufacturing and distribution of eco-friendly construction materials and fly-ash, as well as equipment used to produce these eco-friendly construction materials.	{"Basic Materials","Building Materials"}	{}	Building Materials	Basic Materials	NASDAQ Capital Market	\N	2020-10-11 23:12:10.229276	2020-10-11 23:38:31.02423
RFDA	RiverFront Dynamic US Dividend Advantage	The investment seeks to provide capital appreciation and dividend income.\n The fund invests at least 65% of its net assets in a portfolio of equity securities of publicly traded U.S. companies with the potential for dividend income. Equity securities include common stocks and common or preferred shares of real estate investment trusts. The fund may invest in small-, mid- and large-capitalization companies. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.25652	2020-10-11 23:38:31.02423
RFDI	First Trust RiverFront Dynamic Developed International ETF	The investment seeks capital appreciation.\n The fund will seek to achieve its investment objective by investing at least 80% of its net assets in a portfolio of equity securities of developed market companies, including through investments in common stock, depositary receipts, and common and preferred shares of real estate investment trusts ("REITs"), and forward foreign currency exchange contracts and currency spot transactions used to hedge the fund's exposure to the currencies in which the equity securities of such developed market companies are denominated. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:10.261387	2020-10-11 23:38:31.021426
RNLC	First Trust Large Cap US Equity Select ETF	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the Nasdaq Riskalyze US Large Cap Index (the "index").\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in the equity securities that comprise the index. The index is designed to select dividend-paying US large cap securities that are included in the Nasdaq US 500 Large Cap Index (the "base index"). The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:10.455162	2020-10-11 23:38:31.02423
PHX	Panhandle Oil and Gas Inc	Panhandle Oil and Gas Inc is engaged in the acquisition, management and development of non-operated oil and natural gas properties, including wells located on the its mineral and leasehold acreage.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{ELGX,CHK,NFX,DVN,APA,PBT,SWN,MTR,SBR,HK}	Oil & Gas - E&P	Energy	New York Stock Exchange	\N	2020-10-11 23:12:09.1263	2020-10-11 23:38:31.031328
PII	Polaris Industries Inc.	Polaris Industries Inc designs and manufactures powersports vehicles including Off-Road Vehicles, All-Terrain Vehicles, side-by-side vehicles for recreational and utility use, Snowmobiles, Motorcycles, Commercial, Government and Defense vehicles.	{"Consumer Cyclical","Recreational Vehicles",Autos}	{KNDI,BC,HOG,TXT,WGO}	Autos	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:09.152218	2020-10-11 23:38:31.031328
PSAU	PowerShares Global Gold and Precious Metals Portf	The investment seeks to track the investment results (before fees and expenses) of the NASDAQ OMX Global Gold and Precious Metals IndexSM (the "underlying index").\n The fund will invest at least 90% of its total assets in the securities of companies involved in the gold, silver and other precious metals mining industries that comprise the underlying index, as well as American depositary receipts and global depositary receipts that are based on the securities in the underlying index. The adviser anticipates that the majority of its investments will be in the securities that comprise the underlying index rather than in ADRs and GDRs. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.576658	2020-10-11 23:38:31.031328
PTNR	Partner Communications Company Ltd.	Partner Communications Co Ltd is an Israeli telecommunications company. The company provides cellular and fixed-line telecommunication services, including infrastructure, ILD and ISP services under its Partner brand.	{"Communication Services","Telecom Services"}	{SKM,IGLD,AAPL,BCOM,TKC,CEL,ORAN}	Communication Services	Communication Services	Nasdaq Global Select	\N	2020-10-11 23:12:09.723541	2020-10-11 23:38:31.031328
QQEW	First Trust NASDAQ-100 Equal Weighted Index Fund	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the NASDAQ-100 Equal Weighted IndexSM.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in common stocks that comprise the index. The NASDAQ-100 Equal Weighted Index is the equal-weighted version of the NASDAQ-100 Index® which includes 100 of the largest non-financial companies listed on Nasdaq based on market capitalization.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.928518	2020-10-11 23:38:31.031328
QUAL	iShares Edge MSCI USA Quality Factor	The investment seeks to track the investment results of the MSCI USA Sector Neutral Quality Index composed of U.S. large- and mid-capitalization stocks with quality characteristics as identified through certain fundamental metrics.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The index is based on a traditional market capitalization-weighted parent index, the MSCI USA Index.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:09.969239	2020-10-11 23:38:31.031328
QVM	Arrow QVM Equity Factor	The investment seeks to replicate investment results that generally correspond, before fees and expenses, to the price and yield performance of AI Quality Value Momentum Index.\n The fund generally will invest at least 80% of its total assets in common stocks that comprise the AI Quality Value Momentum Index. Arrow Insights ("index provider") selects common stocks pursuant to a proprietary selection methodology that is designed to identify Tri-Factor™ 50, the leaders with the strongest fundamental performance characteristics.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.98735	2020-10-11 23:38:31.031328
RFEU	First Trust RiverFront Dynamic Europe ETF	The investment seeks capital appreciation.\n The fund seeks to achieve its investment objective by investing at least 80% of its net assets (including investment borrowings) in a portfolio of equity securities of European companies, including through investments in common stock, depositary receipts, and common and preferred shares of real estate investment trusts ("REITs"), and forward foreign currency exchange contracts and currency spot transactions used to hedge the fund's exposure to the currencies in which the equity securities of such European companies are denominated. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:10.26578	2020-10-11 23:38:31.031328
RING	iShares MSCI Global Gold Miners ETF	The investment seeks to track the investment results of the MSCI ACWI Select Gold Miners Investable Market Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index measures the combined performance of equity securities of companies primarily engaged in the business of gold mining in both developed and emerging markets. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:10.363342	2020-10-11 23:38:31.031328
ROBO	Exchange Traded Concepts Trust ROBO Global Robotics and Automation Index	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the price and yield performance of the ROBO Global® Robotics and Automation Index.\n The fund will normally invest at least 80% of its total assets in securities of the index or in depositary receipts representing securities of the index. The index is designed to measure the performance of robotics-related and/or automation-related companies. The fund may invest up to 20% of its assets in investments that are not included in the index, but which the Adviser and Sub-Adviser believe will help it track the index. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.471632	2020-10-11 23:38:31.02423
RRD	R.R. Donnelley & Sons Company	R.R.Donnelley & Sons Co provides printing and media solutions. It also helps customer in developing and executing multichannel communication strategies. The company's products are commercial and digital print, statement printing, labels and others.	{Industrials,"Business Services"}	{TRI,LABL,INWK,CMPR,DLX,QUAD,TGNA}	Business Services	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:10.535306	2020-10-11 23:38:31.02423
RSX	VanEck Vectors Russia	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the MVIS® Russia Index.\n The fund normally invests at least 80% of its total assets in securities that comprise the fund's benchmark index. The index includes securities, which may include depositary receipts, of Russian companies. A company is generally considered to be a Russian company if it is incorporated in Russia or is incorporated outside of Russia but has at least 50% of its revenues/related assets in Russia. Such companies may include medium-capitalization companies. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.557096	2020-10-11 23:38:31.02423
PICB	Invesco International Corporate Bond	The investment seeks to track the investment results (before fees and expenses) of the S&P International Corporate Bond Index® (the "underlying index").\n The fund generally will invest at least 80% of its total assets in investment grade corporate bonds that comprise the underlying index. The underlying index measures the performance of investment grade corporate bonds issued in the following currencies of Group of Ten countries, excluding the U.S. Dollar (USD): Australian Dollar (AUD), British Pound (GBP), Canadian Dollar (CAD), Euro (EUR), Japanese Yen (JPY), New Zealand Dollar (NZD), Norwegian Krone (NOK), Swedish Krona (SEK) and Swiss Franc (SFR).	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.13046	2020-10-11 23:38:31.025871
PRMW	Primo Water Corporation	Primo Water Corp is a provider of multi-gallon purified bottled water, self-service refill water and water dispensers sold through retailers in the United States and Canada.	{"Consumer Defensive","Beverages - Soft Drinks","Beverages - Non-Alcoholic"}	{MIDD,KO,PEP}	Beverages - Non-Alcoholic	Consumer Defensive	NASDAQ Global Market	\N	2020-10-11 23:12:09.537936	2020-10-11 23:38:31.025871
PSCD	PowerShares S&P SmallCap Consumer Discretionary Pf	The investment seeks to track the investment results (before fees and expenses) of the S&P SmallCap 600® Capped Consumer Discretionary Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in common stocks of small capitalization U.S. consumer discretionary companies that comprise the underlying index. These companies are principally engaged in the businesses of providing consumer goods and services that are cyclical in nature, including, but not limited to, household durables, leisure products and services, apparel and luxury goods, computers and electronics, automobiles, etc.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.60049	2020-10-11 23:38:31.025871
QQQE	Direxion NASDAQ-100 Equal Weighted Index Shares	The investment seeks investment results before fees and expenses that track the NASDAQ-100® Equal Weighted Index.\n The fund, under normal circumstances, invests at least 80% of its assets in the securities that comprise the index. The index is the equal weighted version of the NASDAQ-100 Index® which includes 100 of the largest domestic and international non-financial companies listed on the NASDAQ® Stock Market based on market capitalization. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.937704	2020-10-11 23:38:31.025871
QTWO	Q2 Holdings Inc.	Q2 Holdings Inc is a provider of secure, cloud-based virtual banking solutions. It enables regional and community financial institutions to deliver a range of virtual banking services.	{Technology,"Software - Infrastructure","Application Software"}	{EPAY,FISV,JKHY,ACIW,NCR,FIS,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:09.968561	2020-10-11 23:38:31.025871
RDVY	First Trust NASDAQ Rising Dividend Achievers ETF	The investment seeks investment results that correspond generally to the price and yield (before the fees and expenses) of the NASDAQ US Rising Dividend Achievers Index.\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in securities that comprise the index. The index is composed of the securities of 50 companies with a history of raising their dividends and exhibit the characteristics to continue to do so in the future ("index securities"). The index is designed to provide access to a diversified portfolio of small, mid and large capitalization income producing securities.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:10.141497	2020-10-11 23:38:31.025871
RETL	Direxion Daily Retail Bull 3X Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the daily performance of the S&P Retail Select Industry Index.\n The fund invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, ETFs that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index is a modified equal-weighted index that is designed to measure performance of the stocks comprising the S&P Total Market Index that are classified in the Global Industry Classification Standard (GICS) retail sub-industry. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.229432	2020-10-11 23:38:31.025871
RIBT	RiceBran Technologies	RiceBran Technologies is an ingredient company serving food, animal nutrition and specialty markets focused on value-added processing and marketing of healthy, natural and nutrient dense products derived from raw rice bran.	{"Consumer Defensive","Packaged Foods","Consumer Packaged Goods"}	{GIS,BGS,SEB,KHC}	Consumer Packaged Goods	Consumer Defensive	NASDAQ Capital Market	\N	2020-10-11 23:12:10.319704	2020-10-11 23:38:31.025871
RWJ	Oppenheimer S&P SmallCap 600 Revenue	The investment seeks to provide investment results that correspond generally, before fees and expenses, to the performance of the S&P SmallCap 600 Revenue-Weighted Index (the "underlying index").\n The fund will normally invest at least 80% of its net assets, plus any borrowings for investment purposes, in the securities of small capitalization companies included in the underlying index. The underlying index is constructed using a rules-based methodology that re-weights the constituent securities of the S&P SmallCap 600® Index (the "parent index") according to the revenue earned by the companies in the parent index, subject to a maximum 5% per company weighting.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.609457	2020-10-11 23:38:31.025871
RWGE	Regalwood Global Energy Ltd. Class A	Regalwood Global Energy Ltd is a newly organized blank check company. It acts as an investment vehicle to Carlyle International Energy Partners L.P.	{Industrials,Conglomerates}	{}	Conglomerates	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:10.607778	2020-10-11 23:38:31.031328
RWO	SPDR DJ Wilshire Global Real Estate	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the Dow Jones Global Select Real Estate Securities Indexsm based upon the global real estate market.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index and in depositary receipts based on securities comprising the index. The index is a float-adjusted market capitalization index designed to measure the performance of publicly traded global real estate securities. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.620661	2020-10-11 23:38:31.031328
SA	Seabridge Gold Inc. (Canada)	Seabridge Gold Inc is a development stage company. It is involved in the evaluation, acquisition, exploration and development of gold properties sited in North America.	{"Basic Materials",Gold,"Metals & Mining"}	{ABX,AEM,NG,RGLD,GG,THM,AAU}	Metals & Mining	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:10.685905	2020-10-11 23:38:31.031328
PHYS	Sprott Physical Gold Trust ETV	Sprott Physical Gold Trust is a closed-end mutual fund trust. The Trust provides a secure, convenient, and exchange-traded investment, without the inconvenience associated with a direct investment in physical gold bullion.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NYSE Arca	\N	2020-10-11 23:12:09.128484	2020-10-11 23:38:31.02582
PRO	PROS Holdings Inc.	Pros Holdings Inc is a provider of cloud-based revenue and profit realization software solutions to business-to-business and business-to-consumer companies. Its solutions assist customers in growing revenue, modernizing business processes, and others.	{Technology,"Software - Application","Application Software"}	{DWCH,MSTR,SAP,HON,TDC,SPLK,ORCL,VRNS,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:09.542203	2020-10-11 23:38:31.02582
PSCE	PowerShares S&P SmallCap Energy Portfolio	The investment seeks to track the investment results (before fees and expenses) of the S&P SmallCap 600® Capped Energy Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in common stocks of small capitalization U.S. energy companies that comprise the underlying index. These companies are principally engaged in the business of producing, distributing or servicing energy related products, including oil and gas exploration and production, refining, oil services and pipelines. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.598296	2020-10-11 23:38:31.02582
QUOT	Quotient Technology Inc.	Quotient Technology Inc provides digital promotions and media solutions. The company offers digital platform that enables consumer packaged goods (CPG) brands and retailers to engage shoppers through personalized and targeted promotions and media.	{"Consumer Cyclical","Marketing Services","Advertising & Marketing Services"}	{MSFT,LEE,FB,NWSA,NEWM,TGNA,GOOGL}	Advertising & Marketing Services	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:09.978588	2020-10-11 23:38:31.02582
RESI	Front Yard Residential Corporation	Front Yard Residential Corp formerly, Altisource Residential Corp is a real estate investment trust, which focuses on acquiring, owning and managing single-family rental properties throughout the United States.	{"Real Estate","REIT - Residential",REITs}	{}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:10.223598	2020-10-11 23:38:31.02582
REZ	iShares Trust Residential Real Estate	The investment seeks to track the investment results of the FTSE Nareit All Residential Capped Index composed of U.S. residential, healthcare and self-storage real estate equities.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The underlying index measures the performance of the residential apartments, manufactured homes, healthcare and self-storage real estate sectors of the U.S. equity market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.248955	2020-10-11 23:38:31.02582
RPUT	WisdomTree CBOE Russell 2000 PutWrite Strategy Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the Cboe Russell 2000 PutWrite Index (the "index").\n Normally at least 80% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index tracks the value of a cash-secured put option sales strategy, which consists of selling Russell 2000 Index put options and investing the sale proceeds in one-month Treasury bills. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:10.529112	2020-10-11 23:38:31.02582
RVT	Royce Value Trust Inc.	Royce Value Trust Inc is a diversified closed-end investment company. It invests in diversified sectors including consumer discretionary, consumer staples, healthcare, financial, IT, telecommunications, and materials.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:10.603569	2020-10-11 23:38:31.021426
RWR	SPDR DJ Wilshire REIT	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the Dow Jones U.S. Select REIT Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index is designed to provide a measure of real estate securities that serve as proxies for direct real estate investing, in part by excluding securities whose value is not always closely tied to the value of the underlying real estate. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.623695	2020-10-11 23:38:31.02582
RXD	ProShares UltraShort Health Care	The investment seeks daily investment results, before fees and expenses, that correspond to two times the inverse (-2x) of the daily performance of the Dow Jones U.S. Health CareSM Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index seeks to measure the performance of certain companies in the healthcare sector of the U.S. equity market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.628848	2020-10-11 23:38:31.021426
RYAM	Rayonier Advanced Materials Inc.	Rayonier Advanced Materials Inc manufactures cellulose from wood. Its products are mainly used in a variety of applications, including cigarette filters, liquid crystal displays, paints, pharmaceuticals, and food.	{"Basic Materials",Chemicals}	{BCPC,SCL}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:10.649839	2020-10-11 23:38:31.02582
RYU	Invesco S&P 500 Equal Weight Utilities	The investment seeks to track the investment results (before fees and expenses) of the S&P 500® Equal Weight Telecommunications Services & Utilities Index.\n The fund generally will invest at least 90% of its total assets in the securities that comprise the underlying index. The underlying index is an equal-weighted version of the S&P 500® Utilities Index. Strictly in accordance with its guidelines and mandated procedures, the index provider compiles, maintains and calculates the underlying index, which is comprised of common stocks of companies in the utilities sector and telecommunication services of the S&P 500® Index. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.671418	2020-10-11 23:38:31.021426
SCHD	Schwab US Dividend Equity	The investment seeks to track as closely as possible, before fees and expenses, the total return of the Dow Jones U.S. Dividend 100™ Index.\n The fund invests at least 90% of its net assets (net assets plus borrowings for investment purposes) in stocks that are included in the index. The index is designed to measure the performance of high dividend yielding stocks issued by U.S. companies that have a record of consistently paying dividends, selected for fundamental strength relative to their peers, based on financial ratios.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.850885	2020-10-11 23:38:31.02582
PIE	Invesco DWA Emerging Markets Momentum ETF	The investment seeks to track the investment results (before fees and expenses) of the Dorsey Wright® Emerging Markets Technical Leaders Index (the "underlying index").\n The fund will invest at least 90% of its total assets in the equity securities that comprise the underlying index. The underlying index is comprised of equity securities of large capitalization companies based in emerging market countries.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:09.148416	2020-10-11 23:38:31.023428
PRH	Prudential Financial Inc. 5.70% Junior Subordinated Notes due 2053	Prudential Financial Inc offers annuities, life insurance, retirement plan services, and asset management products to individual and institutional customers in the United States, Asia, Europe and Latin America.	{"Financial Services","Insurance - Life"}	{}	Insurance - Life	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:09.518837	2020-10-11 23:38:31.023428
PXH	Invesco FTSE RAFI Emerging Markets	The investment seeks to track the investment results (before fees and expenses) of the FTSE RAFI Emerging Markets Index (the "underlying index").\n The fund will invest at least 90% of its total assets in securities of companies domiciled in countries that are classified as emerging markets within the country classification definition of FTSE International Limited that comprise the underlying index, as well as ADRs and GDRs that are based on the securities in the underlying index. The underlying index is designed to track the performance of securities of companies domiciled in emerging market countries with the highest ranking cumulative score.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.793833	2020-10-11 23:38:31.023428
QTRH	Quarterhill Inc.	Quarterhill Inc is a diversified investment holding company focused on acquiring technology companies in the Industrial Internet of Things segment. It targets companies which capture, analyze and interpret data.	{Technology,"Communication Equipment"}	{SWIR,INTC,QCOM,HPQ,GOOGL,XPER,BB,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:09.962529	2020-10-11 23:38:31.023428
RNDM	First Trust Developed International Equity Select ETF	The investment seeks investment results that correspond generally to the price and yield (before the fund's fees and expenses) of an equity index called the Nasdaq Riskalyze Developed Index (the "index").\n The fund will normally invest at least 90% of its net assets (including investment borrowings) in the equity securities that comprise the index. The index is designed to select low volatility developed markets (excluding the United States) securities that are included in the Nasdaq Developed Markets Ex-US Large Mid Cap Index (the "base index"). The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:10.434684	2020-10-11 23:38:31.023428
RWLK	ReWalk Robotics Ltd	ReWalk Robotics Ltd is an Israel based medical device company. It is engaged in designing, developing and commercializing the Rewalk systems which are exoskeletons allowing wheelchair-bound individuals to stand and walk once again.	{Healthcare,"Medical Devices"}	{HOLX,BABY,CMD,VAR,XLV}	Medical Devices	Healthcare	NASDAQ Capital Market	\N	2020-10-11 23:12:10.620813	2020-10-11 23:38:31.02423
RWW	Oppenheimer S&P Financials Revenue	The investment seeks to provide investment results that correspond generally, before fees and expenses, to the performance of the S&P Financials Sector Revenue-Weighted Index (the "underlying index").\n The fund will normally invest at least 80% of its net assets, plus any borrowings for investment purposes, in financials companies included in the underlying index. The underlying index is constructed by using a rules-based methodology that re-weights the constituent securities of the S&P 500® Financials Index (the "parent index") according to the revenue earned by the companies in the parent index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.626295	2020-10-11 23:38:31.023428
SAL	Salisbury Bancorp Inc.	Salisbury Bancorp Inc is a holding company. The Company, together with its subsidiary, provides commercial banking, consumer financing, retail banking & trust and wealth advisory services through bank offices and ATMs.	{"Financial Services","Banks - Regional - US",Banks}	{EBTC,BPFH,CNBKA,INDB,WASH,BRKL,FNLC,WBS,BWFG,PNBK}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:10.720603	2020-10-11 23:38:31.023428
SBM	Short Basic Materials	The investment seeks daily investment results that correspond to the inverse (-1x) of the daily performance of the Dow Jones U.S. Basic MaterialsSM Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index seeks to measure the performance of certain companies in the basic materials sector of the U.S. equity market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.785443	2020-10-11 23:38:31.023428
SCHE	Schwab Emerging Markets Equity	The investment seeks to track as closely as possible, before fees and expenses, the total return of the FTSE Emerging Index.\n The fund will invest at least 90% of its net assets (net assets plus borrowings for investment purposes) in these stocks, including depositary receipts representing securities of the index; such depositary receipts may be in the form of ADRs, GDRs and EDRs. It generally invests in stocks that are included in the FTSE Emerging Index. The index is comprised of large and mid capitalization companies in emerging market countries, as defined by the index provider.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.854675	2020-10-11 23:38:31.023428
SCHN	Schnitzer Steel Industries Inc.	Schnitzer Steel Industries Inc is a ferrous and non-ferrous scrap metal recycler and manufacturer of finished steel products in the US. Its metal recycling business collects and recycles auto bodies, rail cars, appliances, machinery etc.	{"Basic Materials",Steel}	{STLD,IDSA,AKS,SIM,GGB,CMC,NUE,WOR,X}	Steel	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:12:10.868518	2020-10-11 23:38:31.02423
SELF	Global Self Storage Inc.	Global Self Storage Inc is a self-administered and self-managed real estate investment trust. The company is engaged ownership, operation, acquisition, development and redevelopment of self-storage facilities in the United States.	{"Real Estate","REIT - Industrial",REITs}	{MTRX,SELF}	REITs	Real Estate	NASDAQ Capital Market	\N	2020-10-11 23:12:10.999473	2020-10-11 23:38:31.023428
SLDB	Solid Biosciences Inc.	Solid Biosciences Inc is a life science company engaged in developing therapies for Duchenne muscular dystrophy. Its product candidate, SGT-001, is a gene transfer under development to restore functional dystrophin protein expression in patients' muscles.	{Healthcare,"Drug Manufacturers - Specialty & Generic","Drug Manufacturers"}	{XLV}	Drug Manufacturers	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:11.292112	2020-10-11 23:38:31.02423
RXL	ProShares Ultra Health Care	The investment seeks daily investment results, before fees and expenses, that correspond to two times (2x) the daily performance of the Dow Jones U.S. Health CareSM Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index measures the performance of certain companies in the healthcare sector of the U.S. equity market. Component companies include, among others, health care providers, biotechnology companies, medical supplies, advanced medical devices and pharmaceuticals. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.642586	2020-10-11 23:38:31.028166
RZG	Invesco S&P Smallcap 600 Pure Growth	The investment seeks to track the investment results (before fees and expenses) of the S&P SmallCap 600® Pure Growth Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities that comprise the underlying index. Strictly in accordance with its guidelines and mandated procedures, the index provider, compiles, maintains and calculates the underlying index, which is comprised of those S&P SmallCap 600® companies with strong growth characteristics selected by the index provider. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.676739	2020-10-11 23:38:31.028166
SIFY	Sify Technologies Limited	Sify Technologies Ltd is an ICT Solutions and Services company. It operates in five segments including Telecom services, Data Center services, Cloud and Managed services, Technology Integration services and Application integration services.	{"Communication Services","Telecom Services"}	{IIJI,IBM,KT,CHT,ORCL}	Communication Services	Communication Services	NASDAQ Capital Market	\N	2020-10-11 23:12:11.195765	2020-10-11 23:38:31.028166
SNHY	Sun Hydraulics Corporation	Sun Hydraulics Corp develops and manufactures solutions for hydraulics and electronic markets. It produces screw-in hydraulic cartridge valves and electro-hydraulics, manifolds and package solutions.	{Industrials,"Diversified Industrials","Industrial Products"}	{PKOH,OFLX,CR,PH,ATU,MWA}	Industrial Products	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:11.48363	2020-10-11 23:38:31.028166
SPYD	SPDR Series Trust Portfolio S&P 500 High Dividend	The investment seeks to track the performance of the S&P 500 High Dividend Index.\n Under normal market conditions, the fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index is designed to measure the performance of 80 high dividend-yielding companies within the S&P 500® Index. The S&P 500 Index focuses on the large capitalization U.S. equity market, including common stock and real estate investment trusts ("REITs"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.761942	2020-10-11 23:38:31.028166
TBNK	Territorial Bancorp Inc.	Territorial Bancorp Inc is a bank holding company. The company through its subsidiary, provides financial services to individuals, families and businesses through its banking offices located throughout the State of Hawaii.	{"Financial Services","Savings & Cooperative Banks",Banks}	{WAFD,PROV,RVSB,TSBK,BYFC,FFNW,ANCB,FSBW,BANC}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:12.108586	2020-10-11 23:38:31.028166
TESS	TESSCO Technologies Incorporated	Tessco Technologies Inc is a United States based value-added technology distributor, manufacturer, and solutions provider. The company supplies wireless communications products for network infrastructure, site support, fixed and mobile broadband networks.	{Technology,"Communication Equipment"}	{AEY,RELL,TECD,SYX,ARW,SNX,AVT,XLK}	Communication Equipment	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:12.222172	2020-10-11 23:38:31.028166
THST	Truett-Hurst Inc.	Truett-Hurst Inc produces and sells premium, super-premium, and ultra-premium wines made generally from grapes purchased from California-based growers.	{"Consumer Defensive","Beverages - Wineries & Distilleries","Beverages - Alcoholic"}	{WVVI,BF.B,STZ}	Beverages - Alcoholic	Consumer Defensive	NASDAQ Capital Market	\N	2020-10-11 23:12:12.279506	2020-10-11 23:38:31.028166
TIP	iShares TIPS Bond	The investment seeks to track the investment results of Bloomberg Barclays U.S. Treasury Inflation Protected Securities (TIPS) Index (Series-L) which composed of inflation-protected U.S. Treasury bonds.\n The fund generally invests at least 90% of its assets in the bonds of the underlying index and at least 95% of its assets in U.S. government bonds. It may invest up to 10% of its assets in U.S. government bonds not included in the underlying index, but which BFA believes will help the fund track the underlying index. It also may invest up to 5% of its assets in repurchase agreements collateralized by U.S. government obligations and in cash and cash equivalents.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.297861	2020-10-11 23:38:31.028166
TLF	Tandy Leather Factory Inc.	Tandy Leather Factory Inc is a retailer and wholesale distributor of leather and related products, including leather, leatherworking tools, buckles and adornments for belts, leather dyes and finishes, saddle and tack hardware, and do-it-yourself kits.	{"Consumer Cyclical","Footwear & Accessories","Manufacturing - Apparel & Furniture"}	{FORD,ICON,TIF,VRA,TPR}	Manufacturing - Apparel & Furniture	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:12.31956	2020-10-11 23:38:31.028166
TRXC	TransEnterix Inc.	TransEnterix Inc is a medical device company engaged in the development and commercialization of a robotic assisted surgical system called the SurgiBot System. The SurgiBot System is a single-port, robotically enhanced laparoscopic surgical platform.	{Healthcare,"Medical Instruments & Supplies","Medical Instruments & Equipment"}	{CNMD,ISRG,CMD,JNJ,SYK,TFX,XLV}	Medical Instruments & Equipment	Healthcare	NYSE American	\N	2020-10-11 23:12:12.490555	2020-10-11 23:38:31.028166
TYO	Direxion Daily 10-Yr Treasury Bear 3x Shrs	The investment seeks daily investment results before fees and expenses of 300% of the inverse (or opposite) of the daily performance of the ICE U.S. Treasury 7-10 Year Bond Index.\n The fund invests in swap agreements, futures contracts, short positions or other financial instruments that, in combination, provide inverse or short leveraged exposure to the index equal to at least 80% of the fund's net assets (plus borrowing for investment purposes). The index is a market value weighted index that includes publicly issued U.S. Treasury securities that have a remaining maturity of greater than seven years and less than or equal to ten years. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.610176	2020-10-11 23:38:31.028166
VWO	Vanguard FTSE Emerging Markets	The investment seeks to track the performance of a benchmark index that measures the investment return of stocks issued by companies located in emerging market countries.\n The fund employs an indexing investment approach designed to track the performance of the FTSE Emerging Markets All Cap China A Inclusion Index, a market-capitalization-weighted index that is made up of approximately 4,032 common stocks of large-, mid-, and small-cap companies located in emerging markets around the world.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.240619	2020-10-11 23:38:31.021426
SBLK	Star Bulk Carriers Corp.	Star Bulk Carriers Corp is a shipping company that owns and operates dry-bulk vessels. Through its vessels the firm transports iron ore, coal, and grain around the world.	{Industrials,"Shipping & Ports","Transportation & Logistics"}	{DSX,NM,NMM,ESEA,SHIP,DRYS,DCIX}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:10.782436	2020-10-11 23:38:31.031328
SEAS	SeaWorld Entertainment Inc.	SeaWorld Entertainment Inc owns theme parks in the United States. Most of its revenue is obtained from ticket sales to its leisure facilities under brands such as SeaWorld, Sea Rescue and Busch Gardens.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{CMCSA,DIS,SIX,FUN}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:10.968196	2020-10-11 23:38:31.031328
SLMBP	SLM Corporation Floating Rate Non-Cumulative Preferred Stock Series B	SLM Corp is a saving, planning and paying for college company providing private education loans to students or their families. It operates a consumer savings network that provides financial rewards on everyday purchases to help families save for college.	{"Financial Services","Credit Services"}	{}	Credit Services	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:11.312945	2020-10-11 23:38:31.031328
SMLF	iShares Edge MSCI Multifactor USA Small-Cap	The investment seeks to track the investment results of the MSCI USA Small Cap Diversified Multiple-Factor Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The underlying index is designed to select equity securities from the MSCI USA Small Cap Index (the "parent index") that have high exposure to four investment style factors: value, quality, momentum and low size, while maintaining a level of risk similar to that of the parent index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.404217	2020-10-11 23:38:31.031328
SMMV	iShares Trust	The investment seeks to track the investment results of the MSCI USA Small Cap Minimum Volatility (USD) Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index. The index has been developed by MSCI Inc. (the "index provider" or "MSCI") to measure the performance of small-capitalization U.S. equities that in the aggregate have lower volatility characteristics relative to the small-capitalization U.S. equity market. The manager uses a "passive" or indexing approach to try to achieve the fund's investment objective. It is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:11.423225	2020-10-11 23:38:31.031328
SNP	China Petroleum & Chemical Corporation	China Petroleum & Chemical Corp is engaged in the oil & gas industry. Its activities involve exploration and development of crude oil and natural gas.	{Energy,"Oil & Gas Integrated","Oil & Gas - Integrated"}	{XOM,BP,PTR,CEO,COP,RDS.A,DK}	Oil & Gas - Integrated	Energy	New York Stock Exchange	\N	2020-10-11 23:12:11.501327	2020-10-11 23:38:31.031328
SOXS	Direxion Daily Semiconductor Bear 3x Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the inverse (or opposite) of the daily performance of the PHLX Semiconductor Sector Index.\n The fund, under normal circumstances, invests in swap agreements, futures contracts, short positions or other financial instruments that, in combination, provide inverse (opposite) or short leveraged exposure to the index equal to at least 80% of the fund's net assets (plus borrowing for investment purposes). The index measures the performance of domestic companies engaged in the design, distribution, manufacture and sale of semiconductors. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.57121	2020-10-11 23:38:31.031328
SUI	Sun Communities Inc.	Sun Communities Inc is a self-administered and self-managed real estate investment trust. The Company through its subsidiaries owns, operates, and develops manufactured housing and recreational vehicle communities throughout the United States.	{"Real Estate","REIT - Residential",REITs}	{AIV,ELS,UMH,IRT}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:11.942284	2020-10-11 23:38:31.031328
SUSB	iShares ESG 1-5 Year USD Corporate Bond ETF	The investment seeks to track the investment results of the Bloomberg Barclays MSCI US Corporate 1-5 Year ESG Focus Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The index has been developed by Bloomberg Barclays Capital Inc. with environmental, social and governance ("ESG") rating inputs from MSCI ESG Research LLC pursuant to an agreement between MSCI ESG Research and Bloomberg Index Services Limited, a subsidiary of Bloomberg Barclays. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:11.961411	2020-10-11 23:38:31.031328
SVXY	ProShares Short VIX Short Term Futures	The investment seeks results (before fees and expenses) that correspond to one-half the inverse (-0.5x) of the performance of the S&P 500 VIX Short-Term Futures Index for a single day.\n The index seeks to offer exposure to market volatility through publicly traded futures markets and is designed to measure the return from a rolling long position in the first and second month VIX futures contracts.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.971135	2020-10-11 23:38:31.031328
TAO	Invesco China Real Estate	The investment seeks to track the investment results (before fees and expenses) of the AlphaShares China Real Estate Index (the index).\n The fund generally will invest at least 90% of its total assets in the securities (including ADRs and GDRs) that comprise the index. The index is comprised of REITs and equity securities of publicly-traded companies that derive a majority of their revenues from real estate development, management and/or ownership of property in China or the Special Administrative Regions of China (Hong Kong and Macau), ADRs, and GDRs. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.08115	2020-10-11 23:38:31.031328
TCBIW	Texas Capital Bancshares Inc. Warrants 01/16/2019	Texas Capital Bancshares Inc is engaged in providing lending services to businesses. Its services included credit solutions, treasury and liquidity services, mortgage finance, private wealth solution and personal banking solution.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:12.11916	2020-10-11 23:38:31.031328
TPOR	Direxion Daily Transportation Bull 3X Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the daily performance of the Dow Jones Transportation AverageTM.\n The fund invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, ETFs that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index measures the performance of large, well-known companies within the transportation industry. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.404514	2020-10-11 23:38:31.031328
SCHM	Schwab U.S. Mid Cap	The investment seeks to track as closely as possible, before fees and expenses, the total return of the Dow Jones U.S. Mid-Cap Total Stock Market Index.\n The fund will invest at least 90% of its net assets (net assets plus borrowings for investment purposes) in securities that are included in the index. The index includes the mid-cap portion of the Dow Jones U.S. Total Stock Market Index actually available to investors in the marketplace. The Dow Jones U.S. Mid-Cap Total Stock Market Index includes the components ranked 501-1000 by full market capitalization. The index is a float-adjusted market capitalization weighted index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.867438	2020-10-11 23:38:31.025871
SCHZ	Schwab US Aggregate Bond	The investment seeks to track as closely as possible, before fees and expenses, the total return of the Bloomberg Barclays US Aggregate Bond Index.\n The fund will invest at least 90% of its net assets in securities included in the index. The index is a broad-based benchmark measuring the performance of the U.S. investment grade, taxable bond market, including U.S. Treasuries, government-related and corporate bonds, mortgage pass-through securities, commercial mortgage-backed securities, and asset-backed securities that are publicly available for sale in the United States.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.883654	2020-10-11 23:38:31.025871
SEDG	SolarEdge Technologies Inc.	SolarEdge Technologies Inc along with its subsidiaries designs, develops, and sells intelligent inverter solutions maximizing power generation at the individual PV module level while lowering the cost of energy produced by the solar PV system.	{Technology,Solar,Semiconductors}	{DIOD,TRMB,TER,ADI,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:10.984516	2020-10-11 23:38:31.025871
SHYD	VanEck Vectors Short High-Yield Municipal Index	The investment seeks to replicate as closely as possible, before fees and expenses, the price and yield performance of the Bloomberg Barclays Municipal High Yield Short Duration Index.\n The fund normally invests at least 80% of its total assets in securities that comprise the benchmark index. The index is composed of publicly traded municipal bonds that cover the U.S. dollar denominated high yield short-term tax-exempt bond market.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.18114	2020-10-11 23:38:31.025871
SMSI	Smith Micro Software Inc.	Smith Micro Software Inc develops and markets wireless solutions for wireless service providers, mobile device and chipset manufacturers, and enterprise businesses. Its operating segments are Wireless and Graphics.	{Technology,"Software - Application","Application Software"}	{MSFT,ADBE,AAPL,ADSK,EVOL,GSB,DOX,GOOGL,GEC,XLK}	Application Software	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:11.438626	2020-10-11 23:38:31.025871
SOGO	Sogou Inc. American Depositary Shares each representing one Class A	Sogou Inc is engaged in providing search and search-related services in China. Its services enables advertisers' promotional links to be displayed on the group's search result pages and other properties and third parties Internet properties.	{Technology,"Internet Content & Information","Online Media"}	{RENN,XLK}	Online Media	Technology	New York Stock Exchange	\N	2020-10-11 23:12:11.529656	2020-10-11 23:38:31.025871
SORL	SORL Auto Parts Inc.	SORL Auto Parts Inc develops, manufactures and distributes automotive air brake systems and other related components to automotive original equipment manufacturers, and related aftermarket.	{"Consumer Cyclical","Auto Parts",Autos}	{CAAS,AXL}	Autos	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:11.563233	2020-10-11 23:38:31.025871
SPYV	SPDR Series Trust Portfolio S&P 500 Value	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the S&P 500 Value Index that tracks the performance of large capitalization exchange traded U.S. equity securities exhibiting "value" characteristics.\n The fund employs a sampling strategy in seeking to track the performance of the S&P 500 Value Index. It generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index measures the performance of the large-capitalization value segment of the U.S. equity market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.761711	2020-10-11 23:38:31.025871
TAC	TransAlta Corporation	TransAlta Corp is engaged in the production and sale of electric energy in Alberta, Canada. It also has an energy trading and marketing business as well as it owns transmission lines and coal mines.	{Utilities,"Utilities - Independent Power Producers"}	{ENB,EOCC,AES,CPL,CIG.C,XLU}	Utilities - Independent Power Producers	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:12.062804	2020-10-11 23:38:31.025871
TAL	TAL Education Group American Depositary Shares	TAL Education Group is engaged in providing after-school tutoring programs for primary and secondary school students in China. It delivers its tutoring services through small classes, personalized premium services, and online course offerings.	{"Consumer Defensive","Education & Training Services",Education}	{ATAI,EDU,DL}	Education	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:12.07613	2020-10-11 23:38:31.025871
THC	Tenet Healthcare Corporation	Tenet Healthcare Corp is a healthcare service provider in the US. The company operates acute-care hospitals that includes ambulatory surgery centers, diagnostic imaging centers, and urgent care centers across United States.	{Healthcare,"Medical Care","Health Care Providers"}	{LPNT,FMS,UHS,CYH,DVA,SEM,MD,SSY,HCA,XLV}	Health Care Providers	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:12.261319	2020-10-11 23:38:31.025871
TLH	iShares 10-20 Year Treasury Bond	The investment seeks to track the investment results of the ICE U.S. Treasury 10-20 Year Bond Index.\n The fund generally invests at least 90% of its assets in the bonds of its underlying index and at least 95% of its assets in U.S. government bonds. It seeks to track the investment results of the underlying index which measures the performance of public obligations of the U.S. Treasury that have a remaining maturity of greater than ten years and less than or equal to twenty years.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.326473	2020-10-11 23:38:31.025871
TPH	TRI Pointe Group Inc.	TRI Pointe Group Inc is engaged in the design, construction and sale of single-family attached and detached homes. The company's operations are organized in two principal businesses, including homebuilding and financial services.	{"Consumer Cyclical","Residential Construction","Homebuilding & Construction"}	{MHO,NVR,DHI,LEN,KBH,TMHC,WLH,NWHM,LGIH}	Homebuilding & Construction	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:12.40374	2020-10-11 23:38:31.025871
SDP	ProShares UltraShort Utilities	The investment seeks daily investment results, before fees and expenses, that correspond to two times the inverse (-2x) of the daily performance of the Dow Jones U.S. UtilitiesSM Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index seeks to measure the performance of certain companies in the utilities sector of the U.S. equity market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.950031	2020-10-11 23:38:31.02582
SFIG	WisdomTree Fundamental U.S. Short-Term Corporate Bond Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree Fundamental U.S. Short-term Corporate Bond Index.\n Under normal circumstances, at least 80% of the fund's total assets will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is designed to capture the performance of selected issuers in the short-term U.S. investment grade corporate bond market that are deemed to have favorable fundamental and income characteristics. It is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:11.049556	2020-10-11 23:38:31.02582
SFUN	Fang Holdings Limited American Depositary Shares (Each representing Four Class A HK$1.00 par value)	Fang Holdings Ltd is a provider of online real estate business services. Through its website, the company sells new and second homes and provides financial services to real estate customers.	{Technology,"Internet Content & Information","Online Media"}	{SINA,WUBA,LEJU,GOOGL,XLK}	Online Media	Technology	New York Stock Exchange	\N	2020-10-11 23:12:11.075817	2020-10-11 23:38:31.02582
SKYW	SkyWest Inc.	SkyWest Inc offers scheduled passenger service to destinations in the United States, Canada, Mexico and the Caribbean. It helps to connect major airlines and large hubs to smaller outlying cities.	{Industrials,Airlines}	{JBLU,HA,ALGT,ALK,LUV,DAL,AAL,SAVE,UAL}	Airlines	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:11.272117	2020-10-11 23:38:31.02582
SMLL	Direxion Daily Small Cap Bull 2X Shares	The investment seeks daily investment results, before fees and expenses, of 200% of the daily performance of the Russell 2000® Index.\n The fund invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, exchange-traded funds ("ETFs") that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index measures the performance of approximately 2,000 small-capitalization companies in the Russell 3000® Index, based on a combination of their market capitalization and current index membership. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.403663	2020-10-11 23:38:31.02582
SOHO	Sotherly Hotels Inc.	Sotherly Hotels Inc is a self-managed and self-administered real estate investment trust that owns and manages a portfolio of upper up-scale, up-scale and mid-scale hotels located in main markets in the Mid-Atlantic and Southeastern United States.	{"Real Estate","REIT - Hotel & Motel",REITs}	{LHO,HST,AHT,PEB,CHSP,HT,HPT,CDOR}	REITs	Real Estate	NASDAQ Global Market	\N	2020-10-11 23:12:11.531521	2020-10-11 23:38:31.02582
SQQQ	ProShares UltraPro Short QQQ	The investment seeks daily investment results, before fees and expenses, that correspond to three times the inverse (-3x) of the daily performance of the NASDAQ-100 Index®.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index includes 100 of the largest domestic and international non-financial companies listed on the Nasdaq Stock Market based on market capitalization. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:11.77818	2020-10-11 23:38:31.02582
SRET	Global X SuperDividend REIT ETF	The investment seeks investment results that correspond generally to the price and yield performance, before fees and expenses, of the Solactive Global SuperDividend® REIT Index.\n The fund invests at least 80% of its total assets in the securities of the underlying index and in American Depositary Receipts ("ADRs") and Global Depositary Receipts ("GDRs") based on the securities in the underlying index. The underlying index tracks the performance of REITs that rank among the highest yielding REITs globally, as determined by Solactive AG, the provider of the underlying index. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:11.804512	2020-10-11 23:38:31.02582
SYRS	Syros Pharmaceuticals Inc.	Syros Pharmaceuticals Inc is a biopharmaceutical company engaged in the understanding of the region of the genome controlling the activation and repression of genes. The companies lead candidates are SY-1425 and SY-1365.	{Healthcare,Biotechnology}	{PDLI,PIRS,XLV}	Biotechnology	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:12.040371	2020-10-11 23:38:31.02582
TCBIP	Texas Capital Bancshares Inc. Non Cumulative Preferred Perpetual Stock Series A	Texas Capital Bancshares Inc is engaged in providing lending services to businesses. Its services included credit solutions, treasury and liquidity services, mortgage finance, private wealth solution and personal banking solution.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:12.119802	2020-10-11 23:38:31.02582
TEL	TE Connectivity Ltd. New Switzerland Registered Shares	TE Connectivity Ltd is an electronic component producer. The company manufactures and designs products such as antennas, cable assemblies, fiber optics, sensors, relays and switches that are used in various application.	{Technology,"Electronic Components","Computer Hardware"}	{LFUS,OCC,AVX,APH,GLW,MMM,COMM,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:12.190644	2020-10-11 23:38:31.02582
TPX	Tempur Sealy International Inc.	Tempur Sealy International Inc is engaged in the home furnishing market. It manufactures bedding products including mattresses, foundations and adjustable bases.	{"Consumer Cyclical","Home Furnishings & Fixtures","Manufacturing - Apparel & Furniture"}	{CULP,LZB,LEG,SNBR}	Manufacturing - Apparel & Furniture	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:12.418036	2020-10-11 23:38:31.02582
TTC	Toro Company (The)	The Toro Co manufactures tools and equipment for outdoor maintenance. The company develops products such as reel and rotary riding products, trim cutting and walking mowers, greens rollers, turf sprayer equipment, and underground irrigation systems.	{Industrials,"Tools & Accessories","Industrial Products"}	{ARTW,ALG,AGCO,BGG,DE,LNN,PLOW}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:12.524474	2020-10-11 23:38:31.02582
SFHY	WisdomTree Fundamental U.S. Short-Term High Yield Corporate Bond Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the WisdomTree Fundamental U.S. Short-term High Yield Corporate Bond Index.\n Under normal circumstances, at least 80% of the fund's total assets will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index is designed to capture the performance of selected issuers in the short-term U.S. non-investment-grade corporate bond market that are deemed to have favorable fundamental and income characteristics. It is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:11.046733	2020-10-11 23:38:31.023007
SHY	iShares 1-3 Year Treasury Bond ETF	The investment seeks to track the investment results of the ICE U.S. Treasury 1-3 Year Bond Index (the "underlying index").\n The fund generally invests at least 90% of its assets in the bonds of the underlying index and at least 95% of its assets in U.S. government bonds. The underlying index measures the performance of public obligations of the U.S. Treasury that have a remaining maturity of greater than one year and less than or equal to three years.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:11.176102	2020-10-11 23:38:31.023007
SOHOO	Sotherly Hotels Inc. 7.875% Series C Cumulative Redeemable Perpetual Preferred Stock	Sotherly Hotels Inc is a self-managed and self-administered real estate investment trust that owns and manages a portfolio of upper up-scale, up-scale and mid-scale hotels located in main markets in the Mid-Atlantic and Southeastern United States.	{"Real Estate","REIT - Hotel & Motel",REITs}	{}	REITs	Real Estate	NASDAQ Global Market	\N	2020-10-11 23:12:11.532572	2020-10-11 23:38:31.023007
SPVM	Invesco S&P 500 Value With Momentum	The investment seeks to track the investment results (before fees and expenses) of the S&P 500® High Momentum Value Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in securities that comprise the underlying index. The index provider selects constituents for the underlying index using a two-step screening process: first, it identifies the 200 stocks from among the approximately 500 stocks in the S&P 500® Index with the highest value scores; second, from among those 200 stocks, it identifies the 100 stocks with the highest positive momentum scores. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:11.718236	2020-10-11 23:38:31.023007
SQ	Square Inc. Class A	Square Inc is a software company offering solutions ranging from payments and point-of-sale services to financial and marketing services. It offers a free software app with its hardware to turn mobile devices into powerful POS solutions in minutes.	{Technology,"Software - Application","Application Software"}	{OTEX,PANW,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:11.767352	2020-10-11 23:38:31.023007
SRAX	Social Reality Inc.	Social Reality Inc is an advertising technology company providing the tools to automate digital marketers and content owners' campaigns across digital channels for brands in the healthcare, CPG, automotive, wellness and lifestyle verticals.	{"Consumer Cyclical","Advertising Agencies","Advertising & Marketing Services"}	{ISIG,SGRP,NCMI}	Advertising & Marketing Services	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:11.780417	2020-10-11 23:38:31.023007
SZK	ProShares UltraShort Consumer Goods	The investment seeks daily investment results, before fees and expenses, that correspond to two times the inverse (-2x) of the daily performance of the Dow Jones U.S. Consumer GoodsSM Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index seeks to measure the stock performance of certain U.S. companies in the consumer goods sector of the U.S. equity market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.053908	2020-10-11 23:38:31.023007
TAIT	Taitron Components Incorporated	Taitron Components Inc is the supplier of electronic components. The company focuses on providing original equipment manufacturers, contract electronic manufacturers and originally designed and manufactured services for various projects.	{Technology,"Electronics Distribution","Computer Hardware"}	{TESS,RELL,AVX,SYX,UUU,ARW,VOXX,AVT,XLK}	Computer Hardware	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:12.07613	2020-10-11 23:38:31.023007
TLEH	FlexShares Currency Hedged Morningstar EM Factor Tilt Index Fund	The investment seeks investment results that correspond generally to the price and yield performance, before fees and expenses, of the Morningstar® Emerging Markets Factor Tilt Hedged IndexSM.\n Under normal circumstances, the fund will invest at least 80% of its total assets (exclusive of collateral held from securities lending) in the securities of the underlying index and in ADRs and GDRs based on the securities in the underlying index. The underlying index reflects the performance of a selection of equity securities designed to provide broad exposure to the global emerging equities markets. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.319237	2020-10-11 23:38:31.023007
TLTE	FlexShares Morningstar Emerging Markets Factor Tilt Index Fund	The investment seeks investment results that correspond generally to the price and yield performance, before fees and expenses, of the Morningstar® Developed Markets ex-US Factor Tilt IndexSM.\n The fund will invest at least 80% of its total assets (exclusive of collateral held from securities lending) in the securities of the index and in American ADRs and GDRs based on the securities in the index. The underlying index reflects the performance of a selection of equity securities designed to provide broad exposure to the global developed equities markets, excluding the U.S., with increased exposure (or a "tilt") to small-capitalization stocks and value stocks.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.341814	2020-10-11 23:38:31.023007
TYG	Tortoise Energy Infrastructure Corporation	Under normal circumstances, the Company will invest at least 90% of total assets in securities of energy infrastructure companies, and will invest at least 70% of total assets in equity securities of MLPs. The Company may also invest up to 25% of total assets in debt securities of energy infrastructure companies, including securities rated below investment grade and up to 30% of total assets in restricted securities for which no public trading market exists.	{"Financial Services","Asset Management"}	{}			New York Stock Exchange	\N	2020-10-11 23:12:12.603257	2020-10-11 23:38:31.023007
UONEK	Urban One Inc.	Urban One Inc is an urban oriented, multi-media company. Its business is radio broadcasting franchise that is the radio broadcasting operation that targets African-American and urban listeners. It owns and operates approximately 50 broadcast stations.	{"Consumer Cyclical","Broadcasting - Radio",Entertainment}	{SALM,BBGI,ETM,CBS,EMMS,SGA}	Entertainment	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:12.77565	2020-10-11 23:38:31.023007
SGC	Superior Group of Companies Inc.	Superior Group Of Companies Inc is mainly engaged in designing, manufacturing and distribution of uniforms to major domestic retailers, foodservice chains, transportation and other service industries. It also provides remote staffing solutions.	{"Consumer Cyclical","Apparel Manufacturing","Manufacturing - Apparel & Furniture"}	{CHKE,LAKE,ARTX,OXM,ENS,DLA,PVH}	Manufacturing - Apparel & Furniture	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:11.109732	2020-10-11 23:38:31.028166
SLM	SLM Corporation	SLM Corp is a saving, planning and paying for college company providing private education loans to students or their families. It operates a consumer savings network that provides financial rewards on everyday purchases to help families save for college.	{"Financial Services","Credit Services"}	{WRLD,NNI,DFS,CIT,RM,NAVI}	Credit Services	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:11.304725	2020-10-11 23:38:31.028166
SLRC	Solar Capital Ltd.	Solar Capital Ltd is an investment company which invests mainly in senior secured loans and subordinated debt of private middle market companies to generate current income that is distributed to shareholders quarterly across economic cycles.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:11.33299	2020-10-11 23:38:31.028166
SRC	Spirit Realty Capital Inc.	Spirit Realty Capital Inc is a self-administered & self-managed REIT. It invests in single-tenant, operationally essential real estate leased on a long-term, triple-net basis to tenants operating within retail, office & industrial property types.	{"Real Estate","REIT - Diversified",REITs}	{}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:11.783674	2020-10-11 23:38:31.028166
TAN	Invesco Solar	The investment seeks investment results that correspond generally to the performance, before the fund's fees and expenses, of an equity index called the MAC Global Solar Energy Index.\n The fund will invest at least 90% of its total assets in common stock, ADRs and GDRs that comprise the index and depositary receipts representing common stocks included in the index. The index is comprised of equity securities, including ADRs and GDRs, traded in developed markets. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.077334	2020-10-11 23:38:31.028166
TMF	Direxion Daily 20-Yr Treasury Bull 3x Shrs	The investment seeks daily investment results, before fees and expenses, of 300% of the daily performance of the ICE U.S. Treasury 20+ Year Bond Index.\n The fund, under normal circumstances, invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, exchange-traded funds ("ETFs") that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index is a market value weighted index that includes publicly issued U.S. Treasury securities that have a remaining maturity of greater than 20 years. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.344428	2020-10-11 23:38:31.028166
TTGT	TechTarget Inc.	TechTarget Inc operates websites that publishes content to help reach IT professionals and executives. It offers free content and primarily collects information about its registered members for IT vendors to generate qualified sales leads.	{Technology,"Internet Content & Information","Online Media"}	{CSGP,CBS,SSTK,RELX,XLK}	Online Media	Technology	NASDAQ Global Market	\N	2020-10-11 23:12:12.531303	2020-10-11 23:38:31.028166
TYNS	Direxion Daily 7 10 Year Treasury Bear 1X Shares	The investment seeks daily investment results, before fees and expenses, of 100% of the inverse (or opposite) of the daily performance of the ICE U.S. Treasury 7-10 Year Bond Index.\n The fund invests in swap agreements, futures contracts, short positions or other financial instruments that, in combination, provide inverse or short exposure to the index equal to at least 80% of the fund's net assets (plus borrowing for investment purposes). The index is a market value weighted index that includes publicly issued U.S. Treasury securities that have a remaining maturity of greater than seven years and less than or equal to ten years. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.609784	2020-10-11 23:38:31.028166
UCIB	ETRACS UBS Bloomberg Constant Maturity Commodity Index (CMCI) Total Return ETN Series B due April 5 2038	The investment seeks to provide exposure to potential price appreciation in the UBS Bloomberg Constant Maturity Commodity Index Total Return.\n The ETN Series B is senior unsecured debt securities issued by UBS. The index is designed to be a diversified benchmark for commodities as an asset class. It is comprised of futures contracts on 27 components, representing 24 commodities, with up to five different maturities for each individual commodity. The overall return on the index is generated by two components: uncollateralized returns on the futures contracts comprising the index and a daily fixed-income return.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.663931	2020-10-11 23:38:31.021426
UEUR	ETNs linked to the VelocityShares Daily 4X Long EUR vs. USD Index	The investment seeks to track the daily performance of the VelocityShares Daily 4X Long EUR vs. USD Index.\nThe index is designed to provide 4 times leveraged exposure, reset daily, to changes in the spot exchange rate between an underlying pair of currencies consisting of the U.S. dollar and a foreign currency. It provides long exposure to one currency (the “long currency”) in the underlying currency pair relative to the other currency (the “reference currency”). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.679706	2020-10-11 23:38:31.028166
UNVR	Univar Inc.	Univar Inc is a distributor of commodity and specialty chemicals. The company's chemicals portfolio includes acids and bases, surfactants, glycols, inorganic compounds, alcohols, as well as general and other specialty chemicals.	{"Basic Materials",Chemicals}	{HWKN,SXT,CHE}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:12.769668	2020-10-11 23:38:31.028166
USDP	USD Partners LP representing limited partner interest	USD Partners LP is engaged in energy-related logistics assets which operates with two reportable segment Terminalling services and Fleet services. The group operates its business primarily through United States and Canada.	{Industrials,Railroads,"Transportation & Logistics"}	{GWR,NSC,CSX}	Transportation & Logistics	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:12.82268	2020-10-11 23:38:31.028166
USMV	iShares Edge MSCI Min Vol USA	The investment seeks the investment results of the MSCI USA Minimum Volatility (USD) Index.\n The fund will invest at least 90% of its assets in the component securities of the index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The index measures the performance of equity securities in the top 85% by market capitalization of equity securities listed on stock exchanges in the U.S. that, in the aggregate, have lower volatility relative to the broader U.S. equity market.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:12.843731	2020-10-11 23:38:31.028166
SHSP	SharpSpring Inc.	SharpSpring Inc provides cloud-based marketing technology. The company's marketing automation platform uses features such as web tracking, lead scoring, and automated workflow to deliver the right message to the right customer at the right time.	{"Consumer Cyclical","Marketing Services","Advertising & Marketing Services"}	{CSGS,ZIXI,XRX,MRIN}	Advertising & Marketing Services	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:11.168644	2020-10-11 23:38:31.021426
SIL	Global X Silver Miners	The investment seeks to provide investment results that correspond generally to the price and yield performance, before fees and expenses, of the Solactive Global Silver Miners Total Return Index.\n The fund invests at least 80% of its total assets in the securities of the underlying index and in American Depositary Receipts ("ADRs") and Global Depositary Receipts ("GDRs") based on the securities in the underlying index. The underlying index is designed to measure broad-based equity market performance of global companies involved in the silver mining industry. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.203225	2020-10-11 23:38:31.021426
SKOR	FlexShares Credit-Scored US Corporate Bond Index	The investment seeks the price and yield performance, before fees and expenses, of the Northern Trust Credit-Scored US Corporate Bond IndexSM.\n The fund generally will invest under normal circumstances at least 80% of its total assets (exclusive of collateral held from securities lending) in the securities of its underlying index. The underlying index is designed to outperform the parent index on a risk-adjusted basis, as measured by a combination of yield return and price appreciation. Securities included in the underlying index are component securities of the Parent Index. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:11.267499	2020-10-11 23:38:31.021426
SMIN	Ishares MSCI India Small Cap	The investment seeks to track the investment results of the MSCI India Small Cap Index.\n The fund generally will collectively invest at least 90% of its assets in the component securities of the underlying index and in investments that have economic characteristics that are substantially identical to the component securities of the underlying index. The index is designed to measure the performance of equity securities of small-capitalization companies whose market capitalization, as calculated by the index provider, represents the bottom 14% of companies in the Indian securities market.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:11.402886	2020-10-11 23:38:31.021426
SNH	Senior Housing Properties Trust	Senior Housing Properties Trust is a real estate investment trust. It owns and operates independent living and assisted living communities, continuing care retirement communities and nursing homes.	{"Real Estate","REIT - Healthcare Facilities",REITs}	{HR,ARE,VTR,UHT,HCP,SBRA,GYRO,WELL}	REITs	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:11.480452	2020-10-11 23:38:31.021426
SOR	Source Capital Inc.	Source Capital Inc is a diversified, closed-end management investment company. Its investment portfolio includes a wide range of securities with the primary emphasis on common stock.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:11.555926	2020-10-11 23:38:31.021426
SPDV	AAM S&P 500 High Dividend Value ETF	The investment seeks to track the total return performance, before fees and expenses, of the S&P 500 Dividend and Free Cash Flow Yield Index (the "index").\n Under normal circumstances, at least 80% of the fund's total assets (exclusive of any collateral held from securities lending) will be invested in the component securities of the index. The index is a rules-based, equal-weighted index that is designed to provide exposure to the constituents of the S&P 500® Index that exhibit both high dividend yield and sustainable dividend distribution characteristics, while maintaining diversified sector exposure.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.598715	2020-10-11 23:38:31.021426
SPMO	Invesco S&P 500 Momentum	The investment seeks to track the investment results (before fees and expenses) of the S&P 500 Momentum Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in common stocks that comprise the underlying index. The underlying index is designed to track the performance of approximately 100 stocks in the S&P 500® Index that have the highest "momentum score." In general, momentum is the tendency of an investment to exhibit persistence in its relative performance; a "momentum style" of investing emphasizes investing in securities that have had better recent performance compared to other securities. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.661379	2020-10-11 23:38:31.021426
SREV	ServiceSource International Inc.	ServiceSource International Inc is a provider of customer and revenue lifecycle solutions that power enterprise revenue relationships, partnering with business to business technology and technology-enabled companies.	{Technology,"Software - Infrastructure","Application Software"}	{LPSN,SAP,CRM,EGAN,ORCL,FIVN,ZEN,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:11.804366	2020-10-11 23:38:31.021426
SSD	Simpson Manufacturing Company Inc.	Simpson Manufacturing Co Inc through its subsidiary designs, engineers and manufactures wood construction products, including connectors, truss plates, fastening systems, fasteners and pre-fabricated shearwalls and concrete construction products.	{Industrials,"Tools & Accessories","Industrial Products"}	{UFPI,APOG,TREX,HD,NUE,VMI,NX,NCS,CVR,SPLP}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:11.836186	2020-10-11 23:38:31.021426
TXN	Texas Instruments Incorporated	Texas Instruments Inc designs and makes semiconductors that it sells to electronics designers and manufacturers. It also manufactures digital signal processors used in wireless communications, and microcontrollers used in various electronics applications.	{Technology,Semiconductors}	{INTC,MXIM,QCOM,NVDA,ON,STM,ADI,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:12.595173	2020-10-11 23:38:31.021426
UBG	E-TRACS USB Bloomberg Commodity Index Exchange Traded Notes UBS Bloomberg CMCI Gold ETN	The investment seeks to track the price and performance yield, before fees and expenses, of the UBS Bloomberg CMCI Gold Total Return index.\n The fund is designed to be representative of the entire liquid forward curve of the gold contracts. The index measures the collateralized returns from a basket of gold futures contracts. It is comprised of the gold futures contracts included in the CMCI with five target maturities.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.635698	2020-10-11 23:38:31.021426
W	Wayfair Inc. Class A	Wayfair Inc operates as an online destination for the home good products in the United States. It offers a range of furniture, décor, decorative accent, housewares, seasonal décor, and other home good products.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{CONN,PIR,PRTY,BOOT}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:13.260349	2020-10-11 23:38:31.023007
SJB	ProShares Short High Yield	The investment seeks daily investment results that correspond to the inverse (-1x) of the daily performance of the Markit iBoxx $ Liquid High Yield Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index is a market-value weighted index designed to provide a balanced representation of U.S. dollar denominated high yield corporate bonds for sale within the US by means of including the most liquid high yield corporate bonds available as determined by a set of transparent and objective index rules. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.240685	2020-10-11 23:38:31.024353
SNSR	Global X Internet of Things ETF	The investment seeks to provide investment results that correspond generally to the price and yield performance, before fees and expenses, of the Indxx Global Internet of Things Thematic Index.\n The fund invests at least 80% of its total assets in the securities of the underlying index. The underlying index is designed to provide exposure to exchange-listed companies in developed markets that facilitate the Internet of Things industry, including companies involved in wearable technology, home automation, connected automotive technology, sensors, networking infrastructure/software, smart metering and energy control devices. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:11.509412	2020-10-11 23:38:31.024353
SOI	Solaris Oilfield Infrastructure Inc. Class A	Solaris Oilfield Infrastructure Inc along with its subsidiaries manufactures and provides patented proppant management systems that unload, store and deliver proppant at oil and natural gas well sites.	{Energy,"Oil & Gas Equipment & Services","Oil & Gas - Services"}	{NOV,OIS}	Oil & Gas - Services	Energy	New York Stock Exchange	\N	2020-10-11 23:12:11.535905	2020-10-11 23:38:31.024353
SONA	Southern National Bancorp of Virginia Inc.	Southern National Bancorp of Virginia Inc is the bank holding company for Sonabank. It offers savings accounts, money market accounts, commercial real estate loans, construction to permanent loans, development and builder loans, among others.	{"Financial Services","Banks - Regional - US",Banks}	{FBSS,BOTJ,ESXB,UBSH,VBFC}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:11.555878	2020-10-11 23:38:31.024353
SPAB	SPDR Portfolio Aggregate Bond	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the price and yield performance of the Bloomberg Barclays U.S. Aggregate Bond Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index or in securities that the Adviser determines have economic characteristics that are substantially identical to the economic characteristics of the securities that comprise the index. The index is designed to measure the performance of the U.S. dollar denominated investment grade bond market. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.584923	2020-10-11 23:38:31.024353
SQM	Sociedad Quimica y Minera S.A.	Sociedad Quimica Y Minera De Chile SA is an integrated producer of potassium nitrate and iodine chemicals. It also produces plant nutrients, potassium chloride, potassium sulfate and certain industrial chemicals including industrial nitrates.	{"Basic Materials",Chemicals}	{FMC,CMP,CF,IPI,MOS}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:11.773193	2020-10-11 23:38:31.024353
SRLN	SPDR Blackstone GSO Senior Loan	The investment seeks to provide current income consistent with the preservation of capital.\n Under normal circumstances, the fund invests substantially all of its assets in the Blackstone / GSO Senior Loan Portfolio, a separate series of the SSGA Master Trust with an identical investment objective as the fund. As a result, the fund invests indirectly through the Portfolio. In pursuing its investment objective, the Portfolio seeks to outperform the Markit iBoxx USD Liquid Leveraged Loan Index and the S&P/LSTA U.S. Leveraged Loan 100 Index by normally investing at least 80% of its net assets (plus any borrowings for investment purposes) in Senior Loans.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.815522	2020-10-11 23:38:31.024353
STIP	iShares 0-5 Year TIPS Bond	The investment seeks to track the investment results of the Bloomberg Barclays U.S. Treasury Inflation-Protected Securities (TIPS) 0-5 Years Index (Series-L).\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The index measures the performance of the inflation-protected public obligations of the U.S. Treasury, commonly known as "TIPS," that have a remaining maturity of less than five years.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.879632	2020-10-11 23:38:31.024353
TATT	TAT Technologies Ltd.	TAT Technologies Ltd is a provider of services and products to the commercial and military aerospace and ground defence industries. The company designs, develops, manufactures and sells heat transfer equipment and solutions.	{Industrials,"Aerospace & Defense"}	{HEI,TGI,ESL,HON,SIF,UTX}	Aerospace & Defense	Industrials	NASDAQ Global Market	\N	2020-10-11 23:12:12.092855	2020-10-11 23:38:31.024353
TLYS	Tilly's Inc.	Tilly's Inc is US based company works as a specialty retailer of casual apparel, footwear and accessories for young men, young women, boys and girls. It offers an unparalleled selection of relevant brands, styles, colors, sizes and price points.	{"Consumer Cyclical","Apparel Stores","Retail - Apparel & Specialty"}	{URBN,ZUMZ,ANF,AEO,BKE,GPS}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:12.343561	2020-10-11 23:38:31.024353
TMV	Direxion Daily 20-Year Treasury Bear 3X	The investment seeks daily investment results before fees and expenses of 300% of the inverse (or opposite) of the daily performance of the ICE U.S. Treasury 20+ Year Bond Index.\n The fund, under normal circumstances, invests in swap agreements, futures contracts, short positions or other financial instruments that, in combination, provide inverse (opposite) or short leveraged exposure to the index equal to at least 80% of the fund's net assets (plus borrowing for investment purposes). The index is a market value weighted index that includes publicly issued U.S. Treasury securities that have a remaining maturity of greater than 20 years. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.359424	2020-10-11 23:38:31.024353
USL	United States 12 Month Oil	The investment seeks to reflect the daily changes in percentage terms of the spot price of light, sweet crude oil delivered to Cushing, Oklahoma, as measured by the daily changes in the average of the prices of specified short-term futures contracts on light, sweet crude oil called the “Benchmark Oil Futures Contracts.\n The fund invests investing primarily in futures contracts for light, sweet crude oil, other types of crude oil, diesel-heating oil, gasoline, natural gas, and other petroleum-based fuels. The Benchmark Oil Futures Contracts are the futures contracts on light, sweet crude oil as traded on the New York Mercantile Exchange.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.841343	2020-10-11 23:38:31.02423
SOIL	Global X Fertilizers/Potash	The investment seeks to provide investment results that correspond generally to the price and yield performance, before fees and expenses, of the Solactive Global Fertilizers/Potash Total Return Index.\n The fund invests at least 80% of its total assets in the securities of the underlying index and in American Depositary Receipts ("ADRs") and Global Depositary Receipts ("GDRs") based on the securities in the underlying index. The underlying index is designed to track the performance of the largest listed companies globally that are active in some aspect of the fertilizer/potash industry. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.536716	2020-10-11 23:38:31.023428
SPHQ	Invesco S&P 500 Quality	The investment seeks to track the investment results (before fees and expenses) of the S&P 500® Quality Index.\n The fund generally will invest at least 90% of its total assets in securities that comprise the underlying index. In selecting constituent securities for the underlying index, the index provider first calculates the quality score of each security in the S&P 500® Index, then selects the 100 stocks with the highest quality score for inclusion in the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.625507	2020-10-11 23:38:31.023428
SQBG	Sequential Brands Group Inc.	Sequential Brands Group Inc is an apparel licensing and brand management company that owns, promotes, markets and licenses a portfolio of consumer brands in the fashion, active and lifestyle categories.	{"Consumer Cyclical","Apparel Manufacturing","Manufacturing - Apparel & Furniture"}	{CHKE,COLM,GIII,UAA}	Manufacturing - Apparel & Furniture	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:11.76814	2020-10-11 23:38:31.023428
SRDX	Surmodics Inc.	Surmodics Inc is a provider of surface modification and in vitro diagnostic technologies to the healthcare industry.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{NKTR,A,ABT,ENZ,JNJ,TMO,PKI,ALKS,HRTX,XLV}	Medical Diagnostics & Research	Healthcare	Nasdaq Global Select	\N	2020-10-11 23:12:11.799455	2020-10-11 23:38:31.023428
SWX	Southwest Gas Holdings Inc.	Southwest Gas Holdings Inc is a U.S. based utility company. It is primarily involved in the purchasing, distributing, and transporting of natural gas in the American Southwest. Its operations are divided into natural gas and construction services.	{Utilities,"Utilities - Regulated Gas","Utilities - Regulated"}	{OKE,SRE,ATO,NFG,CNP,SR,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:11.993889	2020-10-11 23:38:31.023428
TBLU	Tortoise Global Water ESG Fund	The investment seeks investment results that correspond generally to the price and distribution rate (total return) performance of the Tortoise Global Water ESG Net Total Return IndexSM.\n The fund will normally invest at least 80% of its net assets, plus the amount of any borrowings for investment purposes, in the types of securities suggested by its name (i.e., Water Companies). The index is a proprietary rules-based, modified market capitalization weighted, float adjusted index designed to track the overall performance of equity securities of global Water Companies listed on a developed country exchanges. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:12.106731	2020-10-11 23:38:31.023428
TLRA	Telaria Inc.	Telaria Inc provides software for video advertising effectiveness. Its buyer and seller platforms enable seamless transactions in a premium video marketplace by offering control and transparency to its clients.	{Technology,"Software - Application","Application Software"}	{MSFT,CMCSA,IPG,OMC,DIS,FOXA,GOOGL,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:12.334814	2020-10-11 23:38:31.023428
TQQQ	ProShares UltraPro QQQ	The investment seeks daily investment results, before fees and expenses, that correspond to three times (3x) the daily performance of the NASDAQ-100 Index®.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index includes 100 of the largest domestic and international non-financial companies listed on the Nasdaq Stock Market based on market capitalization. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:12.422241	2020-10-11 23:38:31.023428
USHY	iShares Broad USD High Yield Corporate Bond	The investment seeks to track the investment results of the ICE BofAML US High Yield Constrained Index.\n The fund generally will invest at least 90% of its assets in the component securities of the underlying index and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents, including shares of money market funds advised by BFA or its affiliates, as well as in securities not included in the underlying index. The underlying index is designed to provide a broad representation of the U.S. dollar-denominated high yield corporate bond market. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:12.834889	2020-10-11 23:38:31.023428
USO	United States Oil Fund	The investment seeks the daily changes in percentage terms of its shares’ per share net asset value (“NAV”) to reflect the daily changes in percentage terms of the spot price of light, sweet crude oil delivered to Cushing, Oklahoma, as measured by the daily changes in the price of a specified short-term futures contract on light, sweet crude oil called the “Benchmark Oil Futures Contract,” less USO’s expenses.\n USO seeks to achieve its investment objective by investing primarily in futures contracts for light, sweet crude oil, other types of crude oil, diesel-heating oil, gasoline, natural gas, and other petroleum-based fuels.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.851282	2020-10-11 23:38:31.028166
VAW	Vanguard Materials	The investment seeks to track the performance of a benchmark index.\n The fund employs an indexing investment approach designed to track the performance of the MSCI US Investable Market Index (IMI)/Materials 25/50, an index made up of stocks of large, mid-size, and small U.S. companies within the materials sector, as classified under the Global Industry Classification Standard (GICS). The Advisor attempts to replicate the target index by seeking to invest all, or substantially all, of its assets in the stocks that make up the index, in order to hold each stock in approximately the same proportion as its weighting in the index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.917217	2020-10-11 23:38:31.02423
VIRT	Virtu Financial Inc.	Virtu Financial Inc is a leading technology-enabled market maker and liquidity provider to the global financial markets. It maintains a technology platform which provides quotations to buyers and sellers of various securities.	{"Financial Services","Capital Markets","Brokers & Exchanges"}	{AMTD,BGCP,RJF,SCHW}	Brokers & Exchanges	Financial Services	Nasdaq Global Select	\N	2020-10-11 23:12:13.043447	2020-10-11 23:38:31.028166
SOL	Renesola Ltd. ADR	ReneSola Ltd is engaged in the business including the sale of project SPVs or development and sale of solar power projects as a developer and the sale of electricity generated by the solar power projects operated as an IPP.	{Technology,Solar,Semiconductors}	{CSIQ,JKS,DQ,HQCL,XLK}	Semiconductors	Technology	New York Stock Exchange	\N	2020-10-11 23:12:11.548127	2020-10-11 23:38:31.02423
SOXL	Direxion Daily Semiconductor Bull 3x Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the daily performance of the PHLX Semiconductor Sector Index.\n The fund, under normal circumstances, invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, exchange-traded funds ("ETFs") that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index measures the performance of domestic companies engaged in the design, distribution, manufacture and sale of semiconductors. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.567018	2020-10-11 23:38:31.02423
SPHB	Invesco S&P 500 High Beta	The investment seeks to track the investment results (before fees and expenses) of the S&P 500® High Beta Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in common stocks that comprise the underlying index. S&P Dow Jones Indices LLC compiles, maintains and calculates the underlying index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:11.621589	2020-10-11 23:38:31.02423
SRE	Sempra Energy	Sempra Energy is engaged in distributing natural gas and electricity in Southern California. Its non-California affiliates operates merchant gas and renewable power plants, LNG facilities and gas pipes and storage, as well as gas and electric utilities.	{Utilities,"Utilities - Diversified","Utilities - Regulated"}	{PCG,NI,MDU,SWX,UGI,D,VVC,NEE,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:11.801001	2020-10-11 23:38:31.02423
TAGS	Teucrium Agricultural Fund ETV	The investment seeks to have the daily changes in percentage terms of the shares’ NAV reflect the daily changes in percentage terms of a weighted average of the NAVs per share of four other commodity pools that are series of the trust and are sponsored by the sponsor: the Teucrium Corn Fund, the Teucrium Wheat Fund, the Teucrium Soybean Fund and the Teucrium Sugar Fund.\n The fund seeks to achieve its investment objective by investing under normal market conditions in the publicly-traded shares of each underlying fund.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.070841	2020-10-11 23:38:31.02423
TECS	Direxion Technology Bear 3X Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the inverse (or opposite) of the daily performance of the Technology Select Sector Index.\n The fund, under normal circumstances, invests in swap agreements, futures contracts, short positions or other financial instruments that, in combination, provide inverse (opposite) or short leveraged exposure to the index equal to at least 80% of the fund's net assets (plus borrowing for investment purposes). The index is provided by S&P Dow Jones Indices (the "index provider") and includes domestic companies from the technology sector. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.189487	2020-10-11 23:38:31.02423
TILT	FlexShares Mornigstar US Market Factors Tilt Index Fund	The investment seeks investment results that correspond generally to the price and yield performance, before fees and expenses, of the Morningstar® US Market Factor Tilt IndexSM.\n The fund will invest at least 80% of its total assets (exclusive of collateral held from securities lending) in the securities of the underlying index. The underlying index reflects the performance of a selection of U.S. equity securities that is designed to provide broad exposure to the overall U.S. equities market, with increased exposure (or a "tilt") to small-capitalization stocks and value stocks. It may also invest up to 20% of its assets in cash and cash equivalents.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.2963	2020-10-11 23:38:31.02423
TNA	Direxion Small Cap Bull 3X Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the daily performance of the Russell 2000® Index.\n The fund invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, exchange-traded funds ("ETFs") that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index measures the performance of approximately 2,000 small-capitalization companies in the Russell 3000® Index, based on a combination of their market capitalization and current index membership. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.358933	2020-10-11 23:38:31.02423
TVPT	Travelport Worldwide Limited	Travelport Worldwide Ltd is a global distribution system (GDS) operator. It provides distribution, technology, payment, mobile and other solutions for the travel and tourism industry.	{"Consumer Cyclical",Leisure,"Travel & Leisure"}	{MMYT,VVI,EXPE,CDK}	Travel & Leisure	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:12.568305	2020-10-11 23:38:31.02423
TX	Ternium S.A. American Depositary Shares (each representing ten shares USD1.00 par value)	Ternium SA is a producer of steel products. It is engaged in manufacturing and processing of flat and long steel products for construction, home appliances, capital goods, container, food, energy and automotive industries.	{"Basic Materials",Steel}	{SID,SIM,GGB,ATI,PKX,TS,MT}	Steel	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:12.591462	2020-10-11 23:38:31.02423
UEPS	Net 1 UEPS Technologies Inc.	Net 1 UEPS Technologies Inc provides payment solutions and transaction processing services across multiple industries. The Company also provides financial services.	{Industrials,"Business Services"}	{EEFT,GPN,FIS,MA,V,FLT,WEX}	Business Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:12.67992	2020-10-11 23:38:31.02423
URE	ProShares Ultra Real Estate	The investment seeks daily investment results, before fees and expenses, that correspond to two times (2x) the daily performance of the Dow Jones U.S. Real EstateSM Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index seeks to measure the performance of certain companies in the real estate sector of the U.S. equity market. Component companies include, among others, real estate holding and development and real estate services companies and real estate investment trusts ("REITs"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.791073	2020-10-11 23:38:31.02423
WAT	Waters Corporation	Waters Corp is an analytical instrument manufacturer. It designs, manufactures, sells and services liquid chromatography, ultra performance liquid chromatography & mass spectrometry instrument systems & support products.	{Healthcare,"Diagnostics & Research","Medical Diagnostics & Research"}	{MTSC,HBIO,BRKR,MTD,A,BIO,DHR,GE,TMO,PKI,XLV}	Medical Diagnostics & Research	Healthcare	New York Stock Exchange	\N	2020-10-11 23:12:13.276832	2020-10-11 23:38:31.028166
TSBK	Timberland Bancorp Inc.	Timberland Bancorp Inc is a bank holding company. The company through its holding as a community-oriented bank that offers a variety of savings products to its retail customers while concentrating its lending activities on real estate mortgage loans.	{"Financial Services","Savings & Cooperative Banks",Banks}	{PROV,RVSB,BYFC,FFNW,TBNK,ANCB,FSBW,BANC}	Banks	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:12.491435	2020-10-11 23:38:31.025871
TTT	ProShares UltraPro Short 20 Year Treasury	The investment seeks daily investment results, before fees and expenses, that correspond to three times the inverse (-3x) of the daily performance of the ICE U.S. Treasury 20+ Year Bond Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index includes publicly- issued U.S. Treasury securities that have a remaining maturity greater than twenty years and have $300 million or more of outstanding face value, excluding amounts held by the Federal Reserve. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.541134	2020-10-11 23:38:31.025871
UEVM	USAA MSCI Emerging Markets Value Momentum Blend Index	The investment seeks to provide investment results that correspond to the MSCI Emerging Markets Select Value Momentum Blend Index.\n The fund seeks to achieve by investing at least 80% of its net assets in securities in the index, depositary receipts on securities in the index, and securities underlying depositary receipts in the index. The index is designed to deliver exposure to equity market performance in the global emerging markets and provide higher exposure to value and momentum factors within the MSCI Emerging Markets Index while also maintaining moderate index turnover and lower realized volatility than traditional capitalization weighted indexes.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.68186	2020-10-11 23:38:31.025871
UFS	Domtar Corporation (NEW)	Domtar Corp designs, manufactures, markets, and distributes various fiber-based products. It offers communication papers, specialty and packaging papers, and adult incontinence products.	{"Basic Materials","Paper & Paper Products","Forest Products"}	{MERC,GLT,IP,FBR,RFP}	Forest Products	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:12.693174	2020-10-11 23:38:31.025871
USLB	PowerShares Russell 1000 Low Beta Equal Weight Por	The investment seeks to track the investment results (before fees and expenses) of the Russell 1000® Low Beta Equal Weight Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in common stocks that comprise the underlying index. Strictly in accordance with its guidelines and mandated procedures, the index provider compiles, maintains and calculates the underlying index, which is designed to provide exposure to constituents of the Russell 1000® Index (the "Russell 1000") that exhibit low beta characteristics.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:12.839209	2020-10-11 23:38:31.025871
USOD	United States 3x Short Oil Fund	The investment seeks the daily changes in percentage terms of its shares’ per share NAV to reflect three times the inverse of the daily change in percentage terms of the price of a specified short-term futures contract on light, sweet crude oil (the “Benchmark Oil Futures Contract”) less the fund’s expenses.\n USCF will endeavor to have the notional value of the fund’s aggregate short exposure to the Benchmark Oil Futures Contract at the close of each trading day approximately equal to 300% of the fund’s NAV. The Benchmark Oil Futures Contract is the futures contract on light, sweet crude oil as traded on the NYMEX, traded under the trading symbol “CL”.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.853372	2020-10-11 23:38:31.025871
UST	ProShares Ultra 7-10 Year Treasury	The investment seeks daily investment results that correspond to two times (2x) the daily performance of the ICE U.S. Treasury 7-10 Year Bond Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index includes publicly- issued U.S. Treasury securities that have a remaining maturity of greater than seven years and less than or equal to ten years and have $300 million or more of outstanding face value, excluding amounts held by the Federal Reserve. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.857511	2020-10-11 23:38:31.024353
UTI	Universal Technical Institute Inc	Universal Technical Institute Inc is an educational institution. It provides undergraduate degree, as well as certificate programs for technicians in the automotive, diesel, collision repair, motorcycle and marine fields.	{"Consumer Defensive","Education & Training Services",Education}	{CECO,LINC,FC,ATGE}	Education	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:12.871182	2020-10-11 23:38:31.025871
VAMO	Cambria ETF Trust Value and Momentum	The investment seeks income and capital appreciation from investments in the U.S. equity market.\n The fund will seek to achieve its investment objective by investing, under normal market conditions, at least 80% of the value of the fund's net assets in U.S. exchange-listed equity securities. It may hedge up to 100% of the value of the fund's long portfolio. The fund may invest in securities of companies in any industry, but will limit the maximum allocation to any particular sector to 25%. It may invest up to 20% of the value of its net assets in U.S. dollar and non-U.S. dollar denominated money market instruments or other high quality debt securities, or ETFs.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:12.913562	2020-10-11 23:38:31.028166
VIS	Vanguard Industrials	The investment seeks to track the performance of a benchmark index.\n The fund employs an indexing investment approach designed to track the performance of the MSCI US Investable Market Index (IMI)/Industrials 25/50, an index made up of stocks of large, mid-size, and small U.S. companies within the industrials sector, as classified under the Global Industry Classification Standard (GICS). The Advisor attempts to replicate the target index by seeking to invest all, or substantially all, of its assets in the stocks that make up the index, in order to hold each stock in approximately the same proportion as its weighting in the index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.04464	2020-10-11 23:38:31.025871
VMOT	Alpha Architect Value Momentum Trend	The investment seeks to track the total return performance, before fees and expenses, of the Alpha Architect Value Momentum Trend Index.\n Under normal circumstances, at least 80% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in the component securities of the index and other instruments of the index. The index will be composed primarily of the other ETFs advised by the Adviser. Currently, there are four Alpha Architect ETFs, which invest in either domestic or international equity securities, and employ either a "momentum" or a "value" investment strategy. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:13.092947	2020-10-11 23:38:31.025871
TURN	180 Degree Capital Corp.	180 Degree Capital Corp is a non-diversified management investment company operating as a business development company. It invests in companies commercializing and integrating products enabled by disruptive technologies in the life sciences.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	NASDAQ Global Market	\N	2020-10-11 23:12:12.55643	2020-10-11 23:38:31.02582
ULE	ProShares Ultra Euro	The investment seeks results for a single day that match (before fees and expenses) two times (2x) the daily performance of the U.S. dollar price of the euro.\nThe fund seeks to meet its investment objective by investing under normal market conditions in any one of, or combinations of, Financial Instruments (including swap agreements, futures contracts and forward contracts) based on the fund’s benchmark. It will also hold cash or cash equivalents such as U.S. Treasury securities or other high credit quality, short-term fixed-income or similar securities for direct investment or as collateral for Financial Instruments.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.731257	2020-10-11 23:38:31.02582
URTY	ProShares UltraPro Russell2000	The investment seeks daily investment results that correspond to three times (3x) the daily performance of the Russell 2000® Index.\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index is a measure of small-cap U.S. stock market performance. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.801601	2020-10-11 23:38:31.02582
USTB	USAA Core Short-Term Bond	The investment seeks high current income consistent with preservation of principal.\n Under normal circumstances, the fund invests at least 80% of its net assets in debt securities and in derivatives and other instruments that have economic characteristics similar to such securities. It primarily invests in securities that have a dollar-weighted average portfolio maturity of three years or less. The fund may not invest more than 20% of fixed-income securities in non-agency, non-government sponsored entities (GSEs), or privately issued mortgage- or asset-backed securities.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.857558	2020-10-11 23:38:31.021426
UTSL	Direxion Daily Utilities Bull 3X Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the daily performance of the Utilities Select Sector Index.\n The fund invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, ETFs that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index includes domestic companies from the utilities sector which includes the following industries: electric utilities; multi-utilities; water utilities; independent power producers and energy trades; and gas utilities. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.879958	2020-10-11 23:38:31.021426
VECO	Veeco Instruments Inc.	Veeco Instruments Inc is a semiconductor manufacturer. It develops thin film equipment, which is used in making electronic devices including LED, power electronics, wireless devises and hard disk drives.	{Technology,"Semiconductor Equipment & Materials",Semiconductors}	{AMAT,CREE,IVAC,CVV,LRCX,AXTI,CAJ,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:12.963381	2020-10-11 23:38:31.02582
VGIT	Vanguard Intermediate-Term Government Bond ETF	The investment seeks to track the performance of a market-weighted government bond index with an intermediate-term dollar-weighted average maturity.\n The fund employs an indexing investment approach designed to track the performance of the Bloomberg Barclays US Treasury 3-10 Year Bond Index. This index includes fixed income securities issued by the U.S. Treasury (not including inflation-protected bonds), with maturities between 3 and 10 years. At least 80% of the fund's assets will be invested in bonds included in the index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:12.992778	2020-10-11 23:38:31.02582
VLU	SPDR S&P 1500 Value Tilt	The investment seeks investment results that, before fees and expenses, correspond generally to the total return performance of the S&P 1500 Low Valuation Tilt Index.\n The fund employs a sampling strategy in seeking to track the index. It generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index applies an alternative weighting methodology to the S&P Composite 1500 Index so that stocks with relatively low valuations (i.e., relatively "cheap") are overweight relative to the S&P Composite 1500 Index and stocks with relatively high valuations are underweight. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.07688	2020-10-11 23:38:31.021426
VNO	Vornado Realty Trust	Vornado Realty Trust is a fully-integrated real estate investment trust. The company owns and has an ownership interest in Class A office and retail properties in New York and Washington, DC. The company receives revenue in the form of rent.	{"Real Estate","REIT - Office",REITs}	{SLG,LXP,ALX}	REITs	Real Estate	New York Stock Exchange	\N	2020-10-11 23:12:13.104405	2020-10-11 23:38:31.021426
VNOM	Viper Energy Partners LP	Viper Energy Partners LP is engaged in the acquisition and exploitation of oil & natural gas properties in North America. Its properties include mineral interests in Midland County, Texas in the Permian Basin.	{Energy,"Oil & Gas E&P","Oil & Gas - E&P"}	{CNQ,CRZO,SDT}	Oil & Gas - E&P	Energy	Nasdaq Global Select	\N	2020-10-11 23:12:13.109094	2020-10-11 23:38:31.028166
VOE	Vanguard Mid-Cap Value	The investment seeks to track the performance of the CRSP US Mid Cap Value Index that measures the investment return of mid-capitalization value stocks.\n The fund employs an indexing investment approach designed to track the performance of the CRSP US Mid Cap Value Index, a broadly diversified index of value stocks of mid-size U.S. companies. The advisor attempts to replicate the target index by investing all, or substantially all, of its assets in the stocks that make up the index, holding each stock in approximately the same proportion as its weighting in the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.112631	2020-10-11 23:38:31.023428
VONG	Vanguard Russell 1000 Growth Index Fund	The investment seeks to track the performance of a benchmark index that measures the investment return of large-capitalization growth stocks in the United States.\n The fund employs an indexing investment approach designed to track the performance of the Russell 1000® Growth Index. The index is designed to measure the performance of large-capitalization growth stocks in the United States. The Advisor attempts to replicate the target index by investing all, or substantially all, of its assets in the stocks that make up the index, holding each stock in approximately the same proportion as its weighting in the index.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:13.121926	2020-10-11 23:38:31.02582
TWMC	Trans World Entertainment Corp.	Trans World Entertainment Corp is a specialty retailer of music, video and video game products in the United States. It operates a chain of retail entertainment stores and e-commerce sites.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{AMZN,BKS,BBY}	Retail - Apparel & Specialty	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:12.582391	2020-10-11 23:38:31.031328
TYBS	Direxion Daily 20 Year Treasury Bear 1X Shares	The investment seeks daily investment results, before fees and expenses, of 100% of the inverse (or opposite) of the daily performance of the ICE U.S. Treasury 20+ Year Bond Index.\n The fund, under normal circumstances, invests in swap agreements, futures contracts, short positions or other financial instruments that, in combination, provide inverse (opposite) or short exposure to the index equal to at least 80% of the fund's net assets (plus borrowing for investment purposes). The index is a market value weighted index that includes publicly issued U.S. Treasury securities that have a remaining maturity of greater than 20 years. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.602274	2020-10-11 23:38:31.031328
UBIO	Proshares UltraPro Nasdaq Biotechnology	The investment seeks daily investment results, before fees and expenses, that correspond to three times (3x) the daily performance of the NASDAQ Biotechnology Index® (the ""index"").\n The fund invests in financial instruments that ProShare Advisors believes, in combination, should produce daily returns consistent with the fund's investment objective. The index is a modified capitalization weighted index that includes securities of the Nasdaq Stock Market listed companies that are classified as either biotechnology or pharmaceutical. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:12.636295	2020-10-11 23:38:31.031328
USFD	US Foods Holding Corp.	US Foods Holding Corp through its subsidiaries is engaged in marketing and distribution of fresh, frozen and dry food and non-food products to food service customers throughout the United States.	{"Consumer Defensive","Food Distribution","Retail - Defensive"}	{LNDC,USFD}	Retail - Defensive	Consumer Defensive	New York Stock Exchange	\N	2020-10-11 23:12:12.825667	2020-10-11 23:38:31.031328
USLV	VS 3X SILVER	The investment seeks to replicate, net of expenses, three times the S&P GSCI Silver index ER.\n The index comprises futures contracts on a single commodity. The fluctuations in the values of it are intended generally to correlate with changes in the price of silver in global markets.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:12.839861	2020-10-11 23:38:31.031328
V	Visa Inc.	Visa Inc is a payments technology company working to enable consumers, businesses, banks and governments to use fast, secure and reliable digital currency. The Company operates processing networks - VisaNet.	{"Financial Services","Credit Services"}	{EBAY,UEPS,FIS,MA,WEX}	Credit Services	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:12.903458	2020-10-11 23:38:31.023007
VGK	Vanguard FTSEEuropean	The investment seeks to track the performance of a benchmark index that measures the investment return of stocks issued by companies located in the major markets of Europe.\n The fund employs an indexing investment approach by investing all, or substantially all, of its assets in the common stocks included in the FTSE Developed Europe All Cap Index. The FTSE Developed Europe All Cap Index is a market-capitalization-weighted index that is made up of approximately 1,258 common stocks of large-, mid-, and small-cap companies located in 16 European countries-mostly companies in the United Kingdom, France, and Germany.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:12.994141	2020-10-11 23:38:31.031328
VJET	voxeljet AG American Depositary Shares each representing one-fifth of an/	voxeljet AG provides high-speed, large-format 3D printers and on-demand parts services to industrial and commercial customers. It also offers customized printed products such as sand molds and plastic models through its service centers.	{Technology,"Computer Systems","Computer Hardware"}	{DDD,PRLB,XONE,XLK}	Computer Hardware	Technology	New York Stock Exchange	\N	2020-10-11 23:12:13.062572	2020-10-11 23:38:31.023007
VLRS	Controladora Vuela Compania de Aviacion S.A.B. de C.V. American Depositary Shares each representing ten (10) Ordinary Participation Certificates	Controladora Vuela Compania de Aviacion SAB de CV is an ultra-low-cost air carrier company. The company provides low-cost air transportation services in Mexico and the United States.	{Industrials,Airlines}	{ALK,CPA,DAL,LTM,AVH,AAL,GOL,UAL}	Airlines	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:13.075896	2020-10-11 23:38:31.031328
VNLA	Janus Henderson Short Duration Income	The investment seeks investment results that correspond generally to the performance, before fees and expenses, of an index which is designed to track the performance of companies globally that are positioned to profit from increasing demand for organic products, including companies which service, produce, distribute, market or sell organic food, beverages, cosmetics, supplements, or packaging.\n The fund invests at least 80% of its net assets in the stocks that comprise the Solactive Organics Index. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.099122	2020-10-11 23:38:31.023007
VNQI	Vanguard Global ex-U.S. Real Estate ETF	The investment seeks to track the performance of a benchmark index that measures the investment return of the overall U.S. stock market. The CRSP US Total Market Index, represents approximately 100%of the investable U.S. stock market and includes large-, mid-, small-, and micro-cap stocks regularly traded on the New York Stock Exchange and Nasdaq.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:13.10828	2020-10-11 23:38:31.028166
VOT	Vanguard Mid-Cap Growth	The investment seeks to track the performance of the CRSP US Mid Cap Growth Index that measures the investment return of mid-capitalization growth stocks.\n The fund employs an indexing investment approach designed to track the performance of the CRSP US Mid Cap Growth Index, a broadly diversified index of growth stocks of mid-size U.S. companies. The advisor attempts to replicate the target index by investing all, or substantially all, of its assets in the stocks that make up the index, holding each stock in approximately the same proportion as its weighting in the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.129124	2020-10-11 23:38:31.028166
VSS	Vanguard FTSE All-Wld ex-US SmCp Idx	The investment seeks to track the performance of a benchmark index that measures the investment return of stocks of international small-cap companies.\n The fund employs an indexing investment approach designed to track the performance of the FTSE Global Small Cap ex US Index, a float-adjusted, market-capitalization-weighted index designed to measure equity market performance of international small-capitalization stocks. The index included 3,363 stocks of companies located in 46 countries, including both developed and emerging markets.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.185147	2020-10-11 23:38:31.023428
PH	Parker-Hannifin Corporation	Parker Hannifin Corp is a diversified manufacturer of motion and control technologies and systems. It operates in two reportable business segments namely Diversified Industrial and Aerospace Systems.	{Industrials,"Diversified Industrials","Industrial Products"}	{SNHY,ROK,CR,DHR,DOV,IEX,CIR,ATU,IR,ITT}	Industrial Products	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:09.097475	2020-10-11 23:38:31.023007
POL	PolyOne Corporation	PolyOne Corp is a chemical manufacturer that provides specialized polymer materials, services and solutions with operations in specialty polymer formulations, color and additive systems, plastic sheet and packaging solutions and polymer distribution.	{"Basic Materials","Specialty Chemicals",Chemicals}	{EMN,OMN,FMC,WLK,HUN,CE,KRA,DWDP}	Chemicals	Basic Materials	New York Stock Exchange	\N	2020-10-11 23:12:09.35022	2020-10-11 23:38:31.023007
PRGX	PRGX Global Inc.	PRGX Global Inc offers recovery audit and spend analytics services. It provides accounts payable recovery audit services to businesses and government agencies.	{Industrials,"Business Services"}	{ADP,PAYX,RECN,HRB,CBZ}	Business Services	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:09.515567	2020-10-11 23:38:31.023007
PSMB	Invesco Balanced Multi-Asset Allocation	The investment seeks to provide current income and capital appreciation.\n The fund seeks to achieve its investment objective by allocating its assets using a balanced investment style that seeks to maximize the benefits of diversification, which focuses on investing portion of fund assets both in underlying ETFs that invest in fixed-income securities ("Fixed Income ETFs") as well as in underlying ETFs that invest primarily in equity securities ("Equity ETFs"). Specifically, the fund's target allocation is to invest approximately 45%-75% of its total assets in Equity ETFs and approximately 25%-55% of its total assets in Fixed Income ETFs. It is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:09.630827	2020-10-11 23:38:31.023007
PTLC	Pacer Fund Trust Trendpilot US Large Cap	The investment seeks to track the total return performance, before fees and expenses, of the Pacer Trendpilot US Large Cap Index.\n The fund invests at least 80% of its total assets (exclusive of collateral held from securities lending) in the component securities of the index. The index uses an objective, rules-based methodology to implement a systematic trend-following strategy that directs exposure (i) 100% to the S&P 500, (ii) 50% to the S&P 500 and 50% to 3-Month U.S. Treasury bills, or (iii) 100% to 3-Month U.S. Treasury bills, depending on the relative performance of the S&P 500 TR and its 200-business day historical simple moving average.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:09.704757	2020-10-11 23:38:31.023007
QUS	SPDR MSCI USA StrategicFactors	The investment seeks to track the performance of the MSCI USA Factor Mix A-Series Capped Index.\n The fund invests at least 80%, of its total assets in the securities comprising the index. The index is designed to measure the equity market performance of large-and mid-cap companies across the U.S. equity market. It aims to represent the performance of a combination of three factors: value, quality, and low volatility. It is an equal weighted combination of the following three MSCI Factor Indices in a single composite index: the MSCI USA Value Weighted Index, the MSCI USA Quality Index, and the MSCI USA Minimum Volatility Index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:09.983007	2020-10-11 23:38:31.023007
REG	Regency Centers Corporation - Ordinary Shares	Regency Centers Corp is a real estate investment trust and is a grocery-anchored shopping center company. It leases space in its shopping centers to grocery stores, retail anchors, restaurants, side-shop retailers, and service providers.	{"Real Estate","REIT - Retail",REITs}	{CUZ,PEI,PEB}	REITs	Real Estate	Nasdaq Global Select	\N	2020-10-11 23:12:10.170708	2020-10-11 23:38:31.023007
RF	Regions Financial Corporation	Regions Financial Corp is a regional bank headquartered in Alabama, with branches primarily in the southeastern and midwestern United States. It provides banking and bank-related services to individual and corporate customers.	{"Financial Services","Banks - Regional - US",Banks}	{TRMK,PNFP,BBT,BAC,PNC,WFC,FHN,BKU,ONB,BXS}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:10.24975	2020-10-11 23:38:31.023007
RWL	Oppenheimer S&P 500 Revenue	The investment seeks to provide investment results that correspond generally, before fees and expenses, to the performance of the S&P 500 Revenue-Weighted Index (the "underlying index").\n The fund normally will invest at least 80% of its net assets, plus any borrowings for investment purposes, in the securities of large capitalization companies included in the underlying index. The underlying index is constructed by using a rules-based methodology that re-weights the constituent securities of the S&P 500® Index (the "parent index") according to the revenue earned by the companies in the parent index, subject to a maximum 5% per company weighting.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.618852	2020-10-11 23:38:31.023007
SCIX	Global X Scientific Beta Asia ex-Japan	The investment seeks results that correspond generally to the price and yield performance, before fees and expenses, of the Scientific Beta Developed Asia-Pacific ex-Japan Multi-Beta Multi-Strategy Four-Factor Equal Risk Contribution (ERC) Index.\n The fund invests at least 80% of its total assets in the securities of the underlying index and in American Depositary Receipts ("ADRs") and Global Depositary Receipts ("GDRs") based on the securities in the underlying index. The underlying index generally comprises approximately 150 or less Asian-listed common stocks selected based on a proprietary methodology developed by the index provider.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:10.896192	2020-10-11 23:38:31.023007
SE	Sea Limited American Depositary Shares each representing one Class A	Sea Ltd along with its subsidiaries is engaged in the digital entertainment, e-commerce and digital financial service businesses in Greater Southeast Asia.	{Technology,"Electronic Gaming & Multimedia","Application Software"}	{RENN,XLK}	Application Software	Technology	New York Stock Exchange	\N	2020-10-11 23:12:10.965618	2020-10-11 23:38:31.023007
SEED	Origin Agritech Limited	Origin Agritech Ltd along with its subsidiaries is an agricultural biotechnology company serving China. It is engaged in crop seed breeding, genetic improvement and the production, processing, distribution of seeds and related technical services.	{"Consumer Defensive","Farm Products","Consumer Packaged Goods"}	{}	Consumer Packaged Goods	Consumer Defensive	Nasdaq Global Select	\N	2020-10-11 23:12:10.990725	2020-10-11 23:38:31.023007
VRNS	Varonis Systems Inc.	Varonis Systems Inc provides software solutions that protect data from insider threats and cyberattacks. It allows organisations to analyze, secure, manage, and migrate unstructured data.	{Technology,"Software - Infrastructure","Application Software"}	{DWCH,MSTR,IBM,PRO,TDC,SPLK,DATA,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:13.15139	2020-10-11 23:38:31.025871
VTI	Vanguard Total Stock Market	The investment seeks to track the performance of a benchmark index that measures the investment return of the overall stock market.\n The fund employs an indexing investment approach designed to track the performance of the CRSP US Total Market Index, which represents approximately 100% of the investable U.S. stock market and includes large-, mid-, small-, and micro-cap stocks regularly traded on the New York Stock Exchange and Nasdaq. It invests by sampling the index, meaning that it holds a broadly diversified collection of securities that, in the aggregate, approximates the full index in terms of key characteristics.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.19847	2020-10-11 23:38:31.02423
VXF	Vanguard Extended Market	The investment seeks to track a benchmark index that measures the investment return of small- and mid-capitalization stocks.\n The fund employs an indexing investment approach designed to track the performance of S&P Completion Index, a broadly diversified index of stocks of small and mid-size U.S. companies. It invests by sampling the index, meaning that it holds a broadly diversified collection of securities that, in the aggregate, approximates the full index in terms of key characteristics. These characteristics include industry weightings and market capitalization, as well as certain financial measures, such as price/earnings ratio and dividend yield.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.241524	2020-10-11 23:38:31.02423
WCN	Waste Connections Inc.	Waste Connections Inc is a solid waste services company in North America. The company provides waste collection, transfer, disposal and recycling services in mostly exclusive and secondary markets in the United States, and Canada.	{Industrials,"Waste Management"}	{CWST,RSG,WM,VTNR}	Waste Management	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:13.32543	2020-10-11 23:38:31.02423
WERN	Werner Enterprises Inc.	Werner Enterprises Inc is a tuckload carrier company in the United States engaged in transporting truckload shipments of general commodities in both interstate and intrastate commerce. It also provides logistics services through Werner Logistics segment.	{Industrials,Trucking,"Transportation & Logistics"}	{PTSI,CVTI,HTLD,JBHT,MRTN,USAK,ULH,KNX}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:13.346545	2020-10-11 23:38:31.02423
WFC	Wells Fargo & Company	Wells Fargo & Co operates as one of the largest banks in the United States. Its business operations are split into three segments: Community banking; Wholesale banking; and Wealth and Investment Management.	{"Financial Services","Banks - Global",Banks}	{GSBC,KEY,BAC,PNC,TCF,JPM,USB,FHN,RF,C}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:13.361561	2020-10-11 23:38:31.02423
XLU	SPDR Select Sector Fund - Utilities	The investment seeks to provide investment results that, before expenses, correspond generally to the price and yield performance of publicly traded equity securities of companies in the Utilities Select Sector Index.\n In seeking to track the performance of the index, the fund employs a replication strategy. It generally invests substantially all, but at least 95%, of its total assets in the securities comprising the index. The index includes securities of companies from the following industries: electric utilities; water utilities; multi-utilities; independent power and renewable electricity producers; and gas utilities. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.61241	2020-10-11 23:38:31.02423
YLDE	ClearBridge Dividend Strategy ESG ETF	The investment seeks dividend income, growth of dividend income and long-term capital appreciation.\n Under normal circumstances, the fund will invest at least 80% of its net assets, plus borrowings for investment purposes, if any, in dividend-paying stocks or other instruments with similar economic characteristics that offer the potential for income growth and capital appreciation over time and that meet its financial and environmental, social and governance ("ESG") criteria. The fund may also invest in companies that the subadviser believes are making substantial progress toward becoming a leader in ESG policies.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:13.716722	2020-10-11 23:38:31.02423
ZROZ	PIMCO 25 Year Zero Coupon U.S. Treasury Index Exchange-Traded Fund	The investment seeks to provide total return that closely corresponds, before fees and expenses, to the total return of the ICE BofAML Long US Treasury Principal STRIPS Index.\n The fund invests at least 80% of its total assets (exclusive of collateral held from securities lending) in the component securities of the ICE BofAML Long US Treasury Principal STRIPS Index (the "underlying index"). The underlying index is an unmanaged index comprised of long maturity Separate Trading of Registered Interest and Principal of Securities ("STRIPS") representing the final principal payment of U.S. Treasury bonds.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.810283	2020-10-11 23:38:31.02423
VUSE	ETF Series Solutions Trust Vident Core US Equity	The investment seeks to track the performance, before fees and expenses, of the Vident Core U.S. Stock IndexTM (the "index").\n Under normal circumstances, at least 80% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in the component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The underlying index is a rules-based, systematic strategy index comprised of equity securities of issuers domiciled and traded in the United States.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.221441	2020-10-11 23:38:31.028166
VWOB	Vanguard Emerging Markets Government Bond ETF	The investment seeks to track the performance of a benchmark index that measures the investment return of U.S. dollar-denominated bonds issued by governments and government-related issuers in emerging market countries.\n The fund employs an indexing investment approach designed to track the performance of the Bloomberg Barclays USD Emerging Markets Government RIC Capped Index. All of the fund's investments will be selected through the sampling process, and under normal circumstances at least 80% of the fund's assets will be invested in bonds included in the index. It is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:13.241322	2020-10-11 23:38:31.028166
WCC	WESCO International Inc.	WESCO International Inc is a distributor of electrical, industrial and communications maintenance, repair and operating and original equipment manufacturer products, construction materials, and advanced supply chain management and logistics services.	{Industrials,"Industrial Distribution"}	{FAST,HWCC,MSM,AXE,AIT,GPC,GWW,HDS,AVT}	Industrial Distribution	Industrials	New York Stock Exchange	\N	2020-10-11 23:12:13.31529	2020-10-11 23:38:31.028166
WEA	Western Asset Bond Fund Share of Beneficial Interest	Western Asset Premier Bond Fund is a diversified, closed-end management investment company. Its investment objective is to provide current income and capital appreciation by investing in a diversified portfolio of investment grade bonds.	{"Financial Services","Asset Management"}	{}	Asset Management	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:13.340611	2020-10-11 23:38:31.028166
WLDN	Willdan Group Inc.	Willdan Group Inc is a provider of professional, technical and consulting services to public agencies at all levels of government, public and private utilities and commercial and industrial firms.	{Industrials,"Engineering & Construction"}	{}	Engineering & Construction	Industrials	NASDAQ Global Market	\N	2020-10-11 23:12:13.405542	2020-10-11 23:38:31.028166
WYDE	ProShares CDS Short North American HY Credit	The investment seeks to provide short exposure to the credit of North American high yield debt issuers.\n The fund seeks to achieve its investment objective primarily through investing in a portfolio of credit derivatives whose underlying reference entities are North American high yield (i.e., below investment grade or "junk bond") debt issuers. Under normal circumstances, it will invest at least 80% of its net assets in centrally cleared, index-based credit default swaps ("CDS") that, in combination, should have economic characteristics that correlate to the inverse of the North American high yield credit market. The fund is non-diversified.	{}	{}			Cboe Global Markets EDGX	\N	2020-10-11 23:12:13.544835	2020-10-11 23:38:31.028166
XNTK	SPDR NYSE Technology	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the NYSE Technology Index that tracks the performance of publicly traded technology companies.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. It may invest in equity securities that are not included in the index, cash and cash equivalents or money market instruments, such as repurchase agreements and money market funds. The index is composed of 35 leading U.S.-listed technology-related companies. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.635361	2020-10-11 23:38:31.028166
ZBK	Zions Bancorporation N.A. 6.95% Fixed-to-Floating Rate Subordinated Notes due September 15 2028	Zions Bancorp is a bank holding company. The bank has over 400 branches across the United States and has more than $50 billion worth of assets. It provides banking services to small and midsize businesses as well as individuals.	{"Financial Services","Banks - Regional - US",Banks}	{}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:13.76328	2020-10-11 23:38:31.028166
VV	Vanguard Large-Cap	The investment seeks to track the performance of the CRSP US Large Cap Index that measures the investment return of large-capitalization stocks.\n The fund employs an indexing investment approach designed to track the performance of the CRSP US Large Cap Index, a broadly diversified index of large U.S. companies representing approximately the top 85% of the U.S. market capitalization. The advisor attempts to replicate the target index by investing all, or substantially all, of its assets in the stocks that make up the index, holding each stock in approximately the same proportion as its weighting in the index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.225445	2020-10-11 23:38:31.024353
WBIH	WBI BullBear Global High Income	The investment seeks high current income with the potential for long-term capital appreciation, while also seeking to protect principal during unfavorable market conditions.\n Under normal market conditions, the fund will invest at least 80% of its net assets, plus the amount of any borrowings for investment purposes, in income-producing debt and equity securities of foreign and domestic issuers, including the securities of foreign and domestic corporate and government entities.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.303724	2020-10-11 23:38:31.024353
WDFC	WD-40 Company	WD-40 Co develops and sells products that solve problems in workshops, factories and homes. The firm sells only WD-40 product. It has product groups maintenance products and homecare and cleaning products.	{"Basic Materials","Specialty Chemicals",Chemicals}	{NTIC,CLMT,CLX,KWR,SCL}	Chemicals	Basic Materials	Nasdaq Global Select	\N	2020-10-11 23:12:13.331538	2020-10-11 23:38:31.024353
WEC	WEC Energy Group Inc.	WEC Energy Group Inc operates electric and gas utility businesses in various states across the US. The firm is primarily engaged in the generation of electricity, gas distribution, and electric transmission.	{Utilities,"Utilities - Regulated Electric","Utilities - Regulated"}	{DTE,PCG,LNT,AEE,CMS,AVA,SCG,DUK,XEL,XLU}	Utilities - Regulated	Utilities	New York Stock Exchange	\N	2020-10-11 23:12:13.34658	2020-10-11 23:38:31.024353
WPS	iShares International Developed Property	The investment seeks to track the investment results of the S&P Developed ex-U.S. Property IndexTM.\n The fund generally invests at least 90% of its assets in the component securities of the index and in investments that have economic characteristics that are substantially identical to the component securities and may invest up to 10% of its assets in certain futures, options and swap contracts, cash and cash equivalents. The index is a free float-adjusted, market capitalization-weighted index that defines and measures the investable universe of publicly-traded real estate companies domiciled in developed countries outside of the U.S.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.455566	2020-10-11 23:38:31.024353
XES	SPDR Series Trust S&P Oil & Gas Equipment & Services	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of an index derived from the oil and gas equipment and services segment of a U.S. total market composite index.\n In seeking to track the performance of the S&P Oil & Gas Equipment & Services Select Industry Index, the fund employs a sampling strategy. It generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index represents the oil and gas equipment and services segment of the S&P Total Market Index ("S&P TMI"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.569189	2020-10-11 23:38:31.024353
YANG	Direxion Daily FTSE China Bear 3x Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the inverse (or opposite) of the daily performance of the FTSE China 50 Index.\n The fund, under normal circumstances, invests in swap agreements, futures contracts, short positions or other financial instruments that, in combination, provide inverse (opposite) or short leveraged exposure to the index equal to at least 80% of the fund's net assets (plus borrowing for investment purposes). The index consists of the 50 largest and most liquid public Chinese companies currently trading on the Hong Kong Stock Exchange ("SEHK"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.69862	2020-10-11 23:38:31.024353
YMLI	VanEck Vectors High Income Infrastructure MLP	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the price and yield performance of the Solactive High Income Infrastructure MLP Index.\n The fund will normally invest at least 80% of its total assets in securities that comprise the fund's benchmark index. The index is a rules-based index designed to provide investors a means of tracking the performance of selected infrastructure master limited partnerships ("MLPs"); with an emphasis on current yield. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.718308	2020-10-11 23:38:31.024353
WBIF	WBI BullBear Value 1000	The investment seeks long-term capital appreciation and the potential for current income, while also seeking to protect principal during unfavorable market conditions.\n The fund will seek to invest in the equity securities of large capitalization domestic and foreign companies. These securities will be selected on the basis of the Sub-Advisor's proprietary selection process ("Selection Process"). Cash and cash equivalents are considered some of the investment opportunities evaluated by Selection Process. It may invest up to 50% of its net assets in the securities of issuers in emerging markets.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.297115	2020-10-11 23:38:31.023007
WEAT	Teucrium Wheat Fund ETV	The investment seeks to have the daily changes in percentage terms of the shares’ NAV reflect the daily changes in percentage terms of a weighted average of the closing settlement prices for three futures contracts for wheat that are traded on the Chicago Board of Trade.\n The fund seeks to achieve its investment objective primarily by investing in Wheat Interests such that daily changes in the fund’s NAV are expected to closely track the changes in the benchmark.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.344949	2020-10-11 23:38:31.023007
WRB	W.R. Berkley Corporation	WR Berkley Corp is an insurance holding company with operations in United States. The company operates in two segments of the property casualty insurance business: Insurance, and Reinsurance.	{"Financial Services","Insurance - Property & Casualty"}	{SIGI,ACGL,AFG,Y,WTM}	Insurance - Property & Casualty	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:13.45721	2020-10-11 23:38:31.023007
WWD	Woodward Inc.	Woodward Inc is an independent designer, manufacturer and service provider of energy control and optimization solutions for the aerospace, industrial and energy markets.	{Industrials,"Aerospace & Defense"}	{ESL,GE,HON,MOG.A,PH,UTX,TDG,CPST,ITT}	Aerospace & Defense	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:13.53313	2020-10-11 23:38:31.023007
XAR	SPDR S&P Aerospace & Defense	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the S&P Aerospace & Defense Select Industry Index.\n In seeking to track the performance of the S&P Aerospace & Defense Select Industry Index (the "index"), the fund employs a sampling strategy. It generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index represents the aerospace and defense segment of the S&P Total Market Index ("S&P TMI"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.549328	2020-10-11 23:38:31.023007
XLG	Invesco S&P 500 Top 50	The investment seeks to track the investment results (before fees and expenses) of the S&P 500® Top 50 Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in the securities that comprise the underlying index. The index provider compiles, maintains and calculates the underlying index, which includes 50 of the largest capitalization members of the S&P 500® Index. The underlying index's components are weighted by float-adjusted market capitalization. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.599114	2020-10-11 23:38:31.023007
XSPA	XpresSpa Group Inc.	XpresSpa Group Inc is a health and wellness holding company. The company is engaged in building a preeminent pure-play health and wellness services company around its core asset XpresSpa.	{"Consumer Cyclical","Personal Services"}	{}	Personal Services	Consumer Cyclical	NASDAQ Capital Market	\N	2020-10-11 23:12:13.676202	2020-10-11 23:38:31.023007
YINN	Direxion Daily FTSE China Bull 3x Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the daily performance of the FTSE China 50 Index.\n The fund, under normal circumstances, invests at least 80% of its net assets (plus borrowing for investment purposes) in securities of the index, exchange-traded funds ("ETFs") that track the index and other financial instruments that provide daily leveraged exposure to the index or ETFs that track the index. The index consists of the 50 largest and most liquid public Chinese companies currently trading on the Hong Kong Stock Exchange ("SEHK"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.712434	2020-10-11 23:38:31.023007
YPF	YPF Sociedad Anonima	YPF SA is an integrated oil & gas company engaged in exploration, development & production of crude oil, natural gas & LPG and also in refining, marketing & distribution of oil, among others.	{Energy,"Oil & Gas Integrated","Oil & Gas - Integrated"}	{TOT,XOM,BP,PBR,CVX,MRO,RDS.A}	Oil & Gas - Integrated	Energy	New York Stock Exchange	\N	2020-10-11 23:12:13.730123	2020-10-11 23:38:31.023007
WBIY	WBI Power Factor High Dividend	The investment seeks long term-capital appreciation while also seeking to protect principal during unfavorable market conditions.\n The fund will seek to achieve its investment objective by using a broad and flexible investment strategy and may invest directly in different types of instruments, such as: stocks and bonds issued by public and private companies and bonds issued by governments and governmental agencies ("Direct Investments"); and shares of funds that invest directly in such instruments. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.310564	2020-10-11 23:38:31.023428
WD	Walker & Dunlop Inc	Walker & Dunlop Inc is a provider of commercial real estate finance, with a primary focus on multifamily lending. The Company originates, sells, and service a range of multifamily and other commercial real estate finance products.	{"Financial Services","Specialty Finance",Banks}	{ATAX,EFC,HASI,LADR}	Banks	Financial Services	New York Stock Exchange	\N	2020-10-11 23:12:13.326247	2020-10-11 23:38:31.023428
WWE	World Wrestling Entertainment Inc. Class A	World Wrestling Entertainment Inc is an integrated media and entertainment company. It is engaged in the production and distribution of content through various channels including over-the-top WWE Network, live events and feature films.	{"Consumer Cyclical","Media - Diversified",Entertainment}	{TTWO,CMCSA,DIS,CBS.A,MANU,FOXA,MSGN,LGF.A}	Entertainment	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:13.535831	2020-10-11 23:38:31.023428
XBI	SPDR Series Trust S&P Biotech	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the S&P Biotechnology Select Industry Index derived from the biotechnology segment of a U.S. total market composite index.\n In seeking to track the performance of the S&P Biotechnology Select Industry Index (the "index"), the fund employs a sampling strategy. It generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index represents the biotechnology segment of the S&P Total Market Index ("S&P TMI"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.550455	2020-10-11 23:38:31.023428
XENT	Intersect ENT Inc.	Intersect ENT Inc is a commercial stage drug-device company engaged in improving the life for patients with ear, nose and throat conditions.	{Healthcare,"Medical Devices"}	{ANIK,ARDM,OCUL,KALV,XLV}	Medical Devices	Healthcare	NASDAQ Global Market	\N	2020-10-11 23:12:13.567317	2020-10-11 23:38:31.023428
XWEB	SPDR S&P Internet	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of the S&P Internet Select Industry Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. In addition, it may invest in equity securities that are not included in the index, cash and cash equivalents or money market instruments, such as repurchase agreements and money market funds (including money market funds advised by the Adviser). The index represents the internet segment of the S&P Total Market Index ("S&P TMI"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.692161	2020-10-11 23:38:31.023428
YRCW	YRC Worldwide Inc.	YRC Worldwide Inc offers transportation services. It has a comprehensive less-than-truckload networks in North America with local, regional, national and international capabilities.	{Industrials,Trucking,"Transportation & Logistics"}	{ODFL,ULH,SAIA,SINO,RRTS,ARCB}	Transportation & Logistics	Industrials	Nasdaq Global Select	\N	2020-10-11 23:12:13.731725	2020-10-11 23:38:31.023428
YYY	YieldShares High Income	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the price and yield performance of the ISE High IncomeTM Index (the "index").\n The fund will normally invest at least 80% of its total assets in securities of the index. Because the index is comprised of securities issued by other investment companies, the fund operates in a manner that is commonly referred to as a "fund of funds," meaning that it invests its assets in shares of funds included in the index. The index seeks to measure the performance of the top 30 U.S. exchange-listed closed-end funds.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.753164	2020-10-11 23:38:31.023428
WCFB	WCF Bancorp Inc.	WCF Bancorp Inc is engaged in accepting deposits from the general public and investing those deposits, together with funds generated from operations and borrowings, in real estate loans secured by one-to-four family residences.	{"Financial Services","Banks - Regional - US",Banks}	{OCFC,NWBI,ORIT,NYCB}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:13.31655	2020-10-11 23:38:31.02582
WEBK	Wellesley Bancorp Inc.	Wellesley Bancorp Inc provides financial services to individuals, non-profit organizations, small businesses and other entities within Eastern Massachusetts.	{"Financial Services","Banks - Regional - US",Banks}	{OLBK,PACW,BWFG}	Banks	Financial Services	NASDAQ Capital Market	\N	2020-10-11 23:12:13.344524	2020-10-11 23:38:31.02582
YMLP	VanEck Vectors High Income MLP	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the price and yield performance of the Solactive High Income MLP Index.\n The fund will normally invest at least 80% of its total assets in securities that comprise the fund's benchmark index. The index is a rules-based index designed to provide investors a means of tracking the performance of selected master limited partnerships ("MLPs") that are publicly traded on a U.S. securities exchange. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.722689	2020-10-11 23:38:31.02582
WCHN	WisdomTree ICBCCS S&P China 500 Fund	The investment seeks to track the price and yield performance, before fees and expenses, of the S&P China 500 Index.\n Under normal circumstances, at least 80% of the fund's total assets (exclusive of collateral held from securities lending) will be invested in component securities of the index and investments that have economic characteristics that are substantially identical to the economic characteristics of such component securities. The index selects the largest 500 eligible companies from the broader S&P Total China BMI Index. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.323277	2020-10-11 23:38:31.021426
WDIV	SPDR S&P Global Dividend	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return of the S&P Global Dividend Aristocrats Index.\n The fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index and in depositary receipts based on securities comprising the index. The index is designed to measure the performance of high dividend-yield companies included in the S&P Global BMI that have followed a managed-dividends policy of increasing or stable dividends for at least ten consecutive years. The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.333343	2020-10-11 23:38:31.021426
WYNN	Wynn Resorts Limited	Wynn Resorts Ltd is a casino resorts operator. The company integrates hotel accommodations and a range of amenities, including fine dining outlets, premium retail offerings, distinctive entertainment theaters and large meeting complexes.	{"Consumer Cyclical","Resorts & Casinos","Travel & Leisure"}	{MCRI,PENN,CNTY,MLCO,BYD,MGM,LVS}	Travel & Leisure	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:13.546183	2020-10-11 23:38:31.021426
XPER	Xperi Corporation	Xperi Corp is a provider of semiconductor packaging and interconnect solutions and intellectual property products to original equipment manufacturers. Its technologies include proprietary mobile and computational imaging.	{Technology,"Semiconductor Equipment & Materials",Semiconductors}	{INTC,CDNS,MU,SNPS,PDFS,SNE,IMI,TXN,AMD,XLK}	Semiconductors	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:13.649096	2020-10-11 23:38:31.021426
XSD	SPDR S&P Semiconductor	The investment seeks to provide investment results that, before fees and expenses, correspond generally to the total return performance of an index derived from the semiconductor segment of a U.S. total market composite index.\n In seeking to track the performance of the S&P Semiconductor Select Industry Index (the "index"), the fund employs a sampling strategy. It generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. The index represents the semiconductors segment of the S&P Total Market Index ("S&P TMI"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.669195	2020-10-11 23:38:31.021426
YLD	Principal Exchange-Traded Funds EDGE Active Income	The investment seeks to provide current income.\n The fund is an actively managed exchange-traded fund ("ETF") that seeks to achieve its investment objective by investing, under normal circumstances, its assets in investment grade and non-investment grade fixed income securities (commonly known as "junk bonds") and in equity securities. The fund's Sub-Advisors, actively and tactically allocates the fund's assets among fixed income securities and equity securities in an effort to take advantage of changing economic conditions that the Advisor believes favors one asset class over another.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.71662	2020-10-11 23:38:31.021426
WDAY	Workday Inc.	Workday Inc is a SaaS provider offering enterprise resource planning, or ERP, software for medium and large enterprises. The company's product portfolio is built around human capital and financial management software.	{Technology,"Software - Application","Application Software"}	{ADP,ULTI,PAYX,SAP,EFX,CRM,PCTY,PAYC,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:13.328442	2020-10-11 23:38:31.025871
WEN	Wendy's Company (The)	The Wendy's Co operates, develops and franchises quick-service restaurants. It offers hamburger, sandwiches and filet of chicken breast sandwiches, chicken nuggets, chili, french fries, baked potatoes, salads, soft drinks, Frosty desserts and kids meals.	{"Consumer Cyclical",Restaurants}	{JACK,TAST,MCD,YUM,DPZ,GTIM,ARCO,DNKN,RAVE}	Restaurants	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:13.346715	2020-10-11 23:38:31.025871
WSM	Williams-Sonoma Inc. (DE)	Williams-Sonoma Inc is a multi-channel specialty retailer of home furnishings in the United States and Canada. Its segments include e-commerce and retail.	{"Consumer Cyclical","Specialty Retail","Retail - Apparel & Specialty"}	{BBBY,KIRK,ETH,PIR,RH}	Retail - Apparel & Specialty	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:13.487854	2020-10-11 23:38:31.025871
XGTI	xG Technology Inc	xG Technology Inc develops portfolio of intellectual property designed to enhance wireless communications. Its intellectual property is embedded in proprietary software algorithms offering cognitive interference mitigation and spectrum access solutions.	{Technology,"Communication Equipment"}	{MITL,RBBN,XLK}	Communication Equipment	Technology	NASDAQ Capital Market	\N	2020-10-11 23:12:13.57471	2020-10-11 23:38:31.025871
XMLV	Invesco S&P MidCap Low Volatility	The investment seeks to track the investment results (before fees and expenses) of the S&P MidCap 400® Low Volatility Index (the "underlying index").\n The fund generally will invest at least 90% of its total assets in common stocks that comprise the underlying index. Strictly in accordance with its procedures and mandated guidelines, the index provider selects for inclusion in the underlying index the 80 securities that it has determined have the lowest volatility over the past 12 months out of the 400 medium capitalization securities that are contained in the S&P MidCap 400® Index.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.621162	2020-10-11 23:38:31.025871
YOGA	YogaWorks Inc.	YogaWorks Inc is a healthy lifestyle brand focused on enriching and transforming lives through yoga.	{"Consumer Cyclical","Personal Services"}	{}	Personal Services	Consumer Cyclical	NASDAQ Global Market	\N	2020-10-11 23:12:13.72341	2020-10-11 23:38:31.025871
ZNGA	Zynga Inc.	Zynga Inc is engaged in developing, marketing, and operating social games. Its games include FarmVille, Zynga Poker, and Words With Friends that are played on various platforms such as mobile, desktop, and social networking sites.	{Technology,"Electronic Gaming & Multimedia","Application Software"}	{AAPL,EA,GLUU,ATVI,CYOU,FB,GRVY,GOOGL,PTE,XLK}	Application Software	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:13.803837	2020-10-11 23:38:31.025871
WDRW	Direxion Daily Regional Banks Bear 3X Shares	The investment seeks daily investment results, before fees and expenses, of 300% of the inverse of the daily performance of the S&P Regional Banks Select Industry Index.\n The fund invests in swap agreements, futures contracts, short positions or other financial instruments that, in combination, provide inverse or short leveraged exposure to the index equal to at least 80% of the fund's net assets (plus borrowing for investment purposes). The index is a modified equal-weighted index that is designed to measure performance of the stocks comprising the S&P Total Market Index that are classified in the GICS regional banks sub-industry. It is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.333199	2020-10-11 23:38:31.028166
WES	Western Gas Partners LP Limited Partner Interests	Western Gas Partners, LP is engaged in gathering, processing, compressing, treating and transporting natural gas, condensate, NGLs and crude oil for its subsidiaries as well as third-party producers and customers.	{Energy,"Oil & Gas Midstream","Oil & Gas - Midstream"}	{OKE,APC,ATO,EPD,EEP,ET,DCP}	Oil & Gas - Midstream	Energy	New York Stock Exchange	\N	2020-10-11 23:12:13.34725	2020-10-11 23:38:31.028166
WRK	Westrock Company	WestRock Co provides paper and packaging solutions in consumer and corrugated markets. The Company offers corrugated container, paperboard, partitions and protective packaging, containerboard and beverage multipacks.	{"Consumer Cyclical","Packaging & Containers"}	{IP,PKG,GPK,CLW}	Packaging & Containers	Consumer Cyclical	New York Stock Exchange	\N	2020-10-11 23:12:13.466751	2020-10-11 23:38:31.028166
WTW	Weight Watchers International Inc	Weight Watchers International Inc is a commercial provider of weight management service. Its services and products include digital weight management products provided through websites, mobile sites and apps, products sold at meetings and retail channels.	{"Consumer Cyclical","Personal Services"}	{NTRI,RGS,GIS,SCI,MED,DIN,KHC}	Personal Services	Consumer Cyclical	Nasdaq Global Select	\N	2020-10-11 23:12:13.523254	2020-10-11 23:38:31.028166
XTH	SPDR S&P Technology Hardware	The investment seeks to track the performance of the S&P Technology Hardware Select Industry Index (the "index").\n Under normal market conditions, the fund generally invests substantially all, but at least 80%, of its total assets in the securities comprising the index. In addition, it may invest in equity securities that are not included in the index, cash and cash equivalents or money market instruments, such as repurchase agreements and money market funds (including money market funds advised by the Adviser). The index represents the technology hardware segment of the S&P Total Market Index ("S&P TMI"). The fund is non-diversified.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.683419	2020-10-11 23:38:31.028166
YLCO	Global X Yieldco Index ETF	The investment seeks to track, before fees and expenses, the price and yield performance of the Indxx Global YieldCo Index.\n The underlying index comprises publicly traded companies that are formed to own operating assets that produce defined cash flows ("YieldCos"), as well as companies that have publicly announced plans to spin-off a YieldCo in an initial public offering, as determined by the index provider. The fund invests at least 80% of its total assets in the securities of the underlying index and in American Depositary Receipts ("ADRs") and Global Depositary Receipts ("GDRs") based on the securities in the underlying index. The fund is non-diversified.	{}	{}			NASDAQ Global Market	\N	2020-10-11 23:12:13.713976	2020-10-11 23:38:31.028166
WBIG	WBI BullBear Yield 1000	The investment seeks long-term capital appreciation and the potential for current income, while also seeking to protect principal during unfavorable market conditions.\n The fund will seek to invest in the dividend-paying equity securities of large capitalization domestic and foreign companies that WBI Investments, Inc., the sub-advisor ("Sub-Advisor") to the fund and an affiliate of Millington Securities Inc., the advisor ("Advisor"), believes display attractive dividend payment prospects, and in other tactical investment opportunities. It may invest up to 50% of its net assets in the securities of issuers in emerging markets.	{}	{}			NYSE Arca	\N	2020-10-11 23:12:13.3001	2020-10-11 23:38:31.031328
YNDX	Yandex N.V.	Yandex NV is an internet company. It offers access to its search engine through personal computers, mobile phones, tablets, and navigation and other digital devices.	{Technology,"Internet Content & Information","Online Media"}	{MSFT,BIDU,FB,BCOR,GOOGL,XLK}	Online Media	Technology	Nasdaq Global Select	\N	2020-10-11 23:12:13.722507	2020-10-11 23:38:31.031328
\.


--
-- TOC entry 3023 (class 0 OID 16413)
-- Dependencies: 204
-- Data for Name: trades; Type: TABLE DATA; Schema: public; Owner: liu
--

COPY public.trades (trade_id, algo_run_id, is_win, symbol, qty, buy_price, buy_indicators, buy_time, sell_price, sell_indicators, sell_time, client_sell_time, client_buy_time) FROM stdin;
\.


--
-- TOC entry 3028 (class 0 OID 16477)
-- Dependencies: 209
-- Data for Name: trending_tickers; Type: TABLE DATA; Schema: public; Owner: liu
--

COPY public.trending_tickers (trending_id, batch_id, symbol, create_tstamp) FROM stdin;
\.


--
-- TOC entry 3041 (class 0 OID 0)
-- Dependencies: 201
-- Name: algo_run_algo_run_id_seq; Type: SEQUENCE SET; Schema: public; Owner: liu
--

SELECT pg_catalog.setval('public.algo_run_algo_run_id_seq', 12, true);


--
-- TOC entry 3042 (class 0 OID 0)
-- Dependencies: 205
-- Name: new_trades_trade_id_seq; Type: SEQUENCE SET; Schema: public; Owner: liu
--

SELECT pg_catalog.setval('public.new_trades_trade_id_seq', 206, true);


--
-- TOC entry 3043 (class 0 OID 0)
-- Dependencies: 210
-- Name: stock_ohlc_symbol_id_seq; Type: SEQUENCE SET; Schema: public; Owner: liu
--

SELECT pg_catalog.setval('public.stock_ohlc_symbol_id_seq', 91015, true);


--
-- TOC entry 3044 (class 0 OID 0)
-- Dependencies: 203
-- Name: trades_trade_id_seq; Type: SEQUENCE SET; Schema: public; Owner: liu
--

SELECT pg_catalog.setval('public.trades_trade_id_seq', 1, false);


--
-- TOC entry 3045 (class 0 OID 0)
-- Dependencies: 200
-- Name: transaction_id_seq; Type: SEQUENCE SET; Schema: public; Owner: liu
--

SELECT pg_catalog.setval('public.transaction_id_seq', 1, false);


--
-- TOC entry 3046 (class 0 OID 0)
-- Dependencies: 208
-- Name: trending_tickers_trending_id_seq; Type: SEQUENCE SET; Schema: public; Owner: liu
--

SELECT pg_catalog.setval('public.trending_tickers_trending_id_seq', 1, false);


--
-- TOC entry 2861 (class 2606 OID 16410)
-- Name: algo_run algo_run_pkey; Type: CONSTRAINT; Schema: public; Owner: liu
--

ALTER TABLE ONLY public.algo_run
    ADD CONSTRAINT algo_run_pkey PRIMARY KEY (algo_run_id);


--
-- TOC entry 2869 (class 2606 OID 16444)
-- Name: new_trades new_trades_pkey; Type: CONSTRAINT; Schema: public; Owner: liu
--

ALTER TABLE ONLY public.new_trades
    ADD CONSTRAINT new_trades_pkey PRIMARY KEY (trade_id);


--
-- TOC entry 2881 (class 2606 OID 16500)
-- Name: stock_ohlc stock_ohlc_pkey; Type: CONSTRAINT; Schema: public; Owner: liu
--

ALTER TABLE ONLY public.stock_ohlc
    ADD CONSTRAINT stock_ohlc_pkey PRIMARY KEY (symbol_id);


--
-- TOC entry 2885 (class 2606 OID 16502)
-- Name: stock_ohlc stock_ohlc_symbol_symbol_date_key; Type: CONSTRAINT; Schema: public; Owner: liu
--

ALTER TABLE ONLY public.stock_ohlc
    ADD CONSTRAINT stock_ohlc_symbol_symbol_date_key UNIQUE (symbol, symbol_date);


--
-- TOC entry 2873 (class 2606 OID 16463)
-- Name: ticker_data ticker_data_pkey; Type: CONSTRAINT; Schema: public; Owner: liu
--

ALTER TABLE ONLY public.ticker_data
    ADD CONSTRAINT ticker_data_pkey PRIMARY KEY (symbol);


--
-- TOC entry 2865 (class 2606 OID 16423)
-- Name: trades trades_pkey; Type: CONSTRAINT; Schema: public; Owner: liu
--

ALTER TABLE ONLY public.trades
    ADD CONSTRAINT trades_pkey PRIMARY KEY (trade_id);


--
-- TOC entry 2879 (class 2606 OID 16486)
-- Name: trending_tickers trending_tickers_pkey; Type: CONSTRAINT; Schema: public; Owner: liu
--

ALTER TABLE ONLY public.trending_tickers
    ADD CONSTRAINT trending_tickers_pkey PRIMARY KEY (trending_id);


--
-- TOC entry 2859 (class 1259 OID 16469)
-- Name: algo_run_batch_id_idx; Type: INDEX; Schema: public; Owner: liu
--

CREATE INDEX algo_run_batch_id_idx ON public.algo_run USING btree (batch_id);


--
-- TOC entry 2867 (class 1259 OID 16451)
-- Name: new_trades_algo_run_id_idx; Type: INDEX; Schema: public; Owner: liu
--

CREATE INDEX new_trades_algo_run_id_idx ON public.new_trades USING btree (algo_run_id);


--
-- TOC entry 2870 (class 1259 OID 16450)
-- Name: new_trades_symbol_idx; Type: INDEX; Schema: public; Owner: liu
--

CREATE INDEX new_trades_symbol_idx ON public.new_trades USING btree (symbol);


--
-- TOC entry 2882 (class 1259 OID 16504)
-- Name: stock_ohlc_symbol_date_idx; Type: INDEX; Schema: public; Owner: liu
--

CREATE INDEX stock_ohlc_symbol_date_idx ON public.stock_ohlc USING btree (symbol_date);


--
-- TOC entry 2883 (class 1259 OID 16503)
-- Name: stock_ohlc_symbol_idx; Type: INDEX; Schema: public; Owner: liu
--

CREATE INDEX stock_ohlc_symbol_idx ON public.stock_ohlc USING btree (symbol);


--
-- TOC entry 2871 (class 1259 OID 16465)
-- Name: ticker_data_industry_idx; Type: INDEX; Schema: public; Owner: liu
--

CREATE INDEX ticker_data_industry_idx ON public.ticker_data USING btree (industry);


--
-- TOC entry 2874 (class 1259 OID 16464)
-- Name: ticker_data_sector_idx; Type: INDEX; Schema: public; Owner: liu
--

CREATE INDEX ticker_data_sector_idx ON public.ticker_data USING btree (sector);


--
-- TOC entry 2875 (class 1259 OID 16467)
-- Name: ticker_data_similar_tickers_idx; Type: INDEX; Schema: public; Owner: liu
--

CREATE INDEX ticker_data_similar_tickers_idx ON public.ticker_data USING btree (similar_tickers);


--
-- TOC entry 2876 (class 1259 OID 16466)
-- Name: ticker_data_tags_idx; Type: INDEX; Schema: public; Owner: liu
--

CREATE INDEX ticker_data_tags_idx ON public.ticker_data USING btree (tags);


--
-- TOC entry 2862 (class 1259 OID 16430)
-- Name: trades_algo_run_id_idx; Type: INDEX; Schema: public; Owner: liu
--

CREATE INDEX trades_algo_run_id_idx ON public.trades USING btree (algo_run_id);


--
-- TOC entry 2863 (class 1259 OID 16431)
-- Name: trades_is_win_idx; Type: INDEX; Schema: public; Owner: liu
--

CREATE INDEX trades_is_win_idx ON public.trades USING btree (is_win);


--
-- TOC entry 2866 (class 1259 OID 16429)
-- Name: trades_symbol_idx; Type: INDEX; Schema: public; Owner: liu
--

CREATE INDEX trades_symbol_idx ON public.trades USING btree (symbol);


--
-- TOC entry 2877 (class 1259 OID 16487)
-- Name: trending_tickers_batch_id_idx; Type: INDEX; Schema: public; Owner: liu
--

CREATE INDEX trending_tickers_batch_id_idx ON public.trending_tickers USING btree (batch_id);


--
-- TOC entry 2886 (class 2606 OID 16470)
-- Name: algo_run algo_run_ref_algo_run_fkey; Type: FK CONSTRAINT; Schema: public; Owner: liu
--

ALTER TABLE ONLY public.algo_run
    ADD CONSTRAINT algo_run_ref_algo_run_fkey FOREIGN KEY (ref_algo_run) REFERENCES public.algo_run(algo_run_id);


--
-- TOC entry 2888 (class 2606 OID 16445)
-- Name: new_trades new_trades_algo_run_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: liu
--

ALTER TABLE ONLY public.new_trades
    ADD CONSTRAINT new_trades_algo_run_id_fkey FOREIGN KEY (algo_run_id) REFERENCES public.algo_run(algo_run_id);


--
-- TOC entry 2887 (class 2606 OID 16424)
-- Name: trades trades_algo_run_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: liu
--

ALTER TABLE ONLY public.trades
    ADD CONSTRAINT trades_algo_run_id_fkey FOREIGN KEY (algo_run_id) REFERENCES public.algo_run(algo_run_id);


-- Completed on 2020-10-12 20:01:21 UTC

--
-- PostgreSQL database dump complete
--

